!function(t){function e(e){for(var n,i,o=e[0],a=e[1],s=0,l=[];s<o.length;s++)i=o[s],r[i]&&l.push(r[i][0]),r[i]=0;for(n in a)Object.prototype.hasOwnProperty.call(a,n)&&(t[n]=a[n]);for(A&&A(e);l.length;)l.shift()()}var n={},r={0:0};function i(e){if(n[e])return n[e].exports;var r=n[e]={i:e,l:!1,exports:{}};return t[e].call(r.exports,r,r.exports,i),r.l=!0,r.exports}i.e=function(t){var e=[],n=r[t];if(0!==n)if(n)e.push(n[2]);else{var o=new Promise(function(e,i){n=r[t]=[e,i]});e.push(n[2]=o);var a,s=document.createElement("script");s.charset="utf-8",s.timeout=120,i.nc&&s.setAttribute("nonce",i.nc),s.src=function(t){return i.p+""+({}[t]||t)+".bundle.js"}(t);var A=new Error;a=function(e){s.onerror=s.onload=null,clearTimeout(l);var n=r[t];if(0!==n){if(n){var i=e&&("load"===e.type?"missing":e.type),o=e&&e.target&&e.target.src;A.message="Loading chunk "+t+" failed.\n("+i+": "+o+")",A.name="ChunkLoadError",A.type=i,A.request=o,n[1](A)}r[t]=void 0}};var l=setTimeout(function(){a({type:"timeout",target:s})},12e4);s.onerror=s.onload=a,document.head.appendChild(s)}return Promise.all(e)},i.m=t,i.c=n,i.d=function(t,e,n){i.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:n})},i.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},i.t=function(t,e){if(1&e&&(t=i(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var n=Object.create(null);if(i.r(n),Object.defineProperty(n,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var r in t)i.d(n,r,function(e){return t[e]}.bind(null,r));return n},i.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return i.d(e,"a",e),e},i.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},i.p="",i.oe=function(t){throw console.error(t),t};var o=window.webpackJsonp=window.webpackJsonp||[],a=o.push.bind(o);o.push=e,o=o.slice();for(var s=0;s<o.length;s++)e(o[s]);var A=a;i(i.s=742)}([function(t,e,n){"use strict";t.exports=n(1062)},function(t,e,n){t.exports=n(1147)()},function(t,e,n){"use strict";n.r(e),n.d(e,"__extends",function(){return i}),n.d(e,"__assign",function(){return o}),n.d(e,"__rest",function(){return a}),n.d(e,"__decorate",function(){return s}),n.d(e,"__param",function(){return A}),n.d(e,"__metadata",function(){return l}),n.d(e,"__awaiter",function(){return c}),n.d(e,"__generator",function(){return u}),n.d(e,"__exportStar",function(){return g}),n.d(e,"__values",function(){return h}),n.d(e,"__read",function(){return d}),n.d(e,"__spread",function(){return f}),n.d(e,"__spreadArrays",function(){return p}),n.d(e,"__await",function(){return I}),n.d(e,"__asyncGenerator",function(){return C}),n.d(e,"__asyncDelegator",function(){return m}),n.d(e,"__asyncValues",function(){return y}),n.d(e,"__makeTemplateObject",function(){return E}),n.d(e,"__importStar",function(){return v}),n.d(e,"__importDefault",function(){return B});
/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
var r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)};function i(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)}var o=function(){return(o=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)};function a(t,e){var n={};for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&e.indexOf(r)<0&&(n[r]=t[r]);if(null!=t&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(t);i<r.length;i++)e.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(t,r[i])&&(n[r[i]]=t[r[i]])}return n}function s(t,e,n,r){var i,o=arguments.length,a=o<3?e:null===r?r=Object.getOwnPropertyDescriptor(e,n):r;if("object"==typeof Reflect&&"function"==typeof Reflect.decorate)a=Reflect.decorate(t,e,n,r);else for(var s=t.length-1;s>=0;s--)(i=t[s])&&(a=(o<3?i(a):o>3?i(e,n,a):i(e,n))||a);return o>3&&a&&Object.defineProperty(e,n,a),a}function A(t,e){return function(n,r){e(n,r,t)}}function l(t,e){if("object"==typeof Reflect&&"function"==typeof Reflect.metadata)return Reflect.metadata(t,e)}function c(t,e,n,r){return new(n||(n=Promise))(function(i,o){function a(t){try{A(r.next(t))}catch(t){o(t)}}function s(t){try{A(r.throw(t))}catch(t){o(t)}}function A(t){t.done?i(t.value):new n(function(e){e(t.value)}).then(a,s)}A((r=r.apply(t,e||[])).next())})}function u(t,e){var n,r,i,o,a={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return o={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(o[Symbol.iterator]=function(){return this}),o;function s(o){return function(s){return function(o){if(n)throw new TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(i=2&o[0]?r.return:o[0]?r.throw||((i=r.return)&&i.call(r),0):r.next)&&!(i=i.call(r,o[1])).done)return i;switch(r=0,i&&(o=[2&o[0],i.value]),o[0]){case 0:case 1:i=o;break;case 4:return a.label++,{value:o[1],done:!1};case 5:a.label++,r=o[1],o=[0];continue;case 7:o=a.ops.pop(),a.trys.pop();continue;default:if(!(i=(i=a.trys).length>0&&i[i.length-1])&&(6===o[0]||2===o[0])){a=0;continue}if(3===o[0]&&(!i||o[1]>i[0]&&o[1]<i[3])){a.label=o[1];break}if(6===o[0]&&a.label<i[1]){a.label=i[1],i=o;break}if(i&&a.label<i[2]){a.label=i[2],a.ops.push(o);break}i[2]&&a.ops.pop(),a.trys.pop();continue}o=e.call(t,a)}catch(t){o=[6,t],r=0}finally{n=i=0}if(5&o[0])throw o[1];return{value:o[0]?o[1]:void 0,done:!0}}([o,s])}}}function g(t,e){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}function h(t){var e="function"==typeof Symbol&&t[Symbol.iterator],n=0;return e?e.call(t):{next:function(){return t&&n>=t.length&&(t=void 0),{value:t&&t[n++],done:!t}}}}function d(t,e){var n="function"==typeof Symbol&&t[Symbol.iterator];if(!n)return t;var r,i,o=n.call(t),a=[];try{for(;(void 0===e||e-- >0)&&!(r=o.next()).done;)a.push(r.value)}catch(t){i={error:t}}finally{try{r&&!r.done&&(n=o.return)&&n.call(o)}finally{if(i)throw i.error}}return a}function f(){for(var t=[],e=0;e<arguments.length;e++)t=t.concat(d(arguments[e]));return t}function p(){for(var t=0,e=0,n=arguments.length;e<n;e++)t+=arguments[e].length;var r=Array(t),i=0;for(e=0;e<n;e++)for(var o=arguments[e],a=0,s=o.length;a<s;a++,i++)r[i]=o[a];return r}function I(t){return this instanceof I?(this.v=t,this):new I(t)}function C(t,e,n){if(!Symbol.asyncIterator)throw new TypeError("Symbol.asyncIterator is not defined.");var r,i=n.apply(t,e||[]),o=[];return r={},a("next"),a("throw"),a("return"),r[Symbol.asyncIterator]=function(){return this},r;function a(t){i[t]&&(r[t]=function(e){return new Promise(function(n,r){o.push([t,e,n,r])>1||s(t,e)})})}function s(t,e){try{(n=i[t](e)).value instanceof I?Promise.resolve(n.value.v).then(A,l):c(o[0][2],n)}catch(t){c(o[0][3],t)}var n}function A(t){s("next",t)}function l(t){s("throw",t)}function c(t,e){t(e),o.shift(),o.length&&s(o[0][0],o[0][1])}}function m(t){var e,n;return e={},r("next"),r("throw",function(t){throw t}),r("return"),e[Symbol.iterator]=function(){return this},e;function r(r,i){e[r]=t[r]?function(e){return(n=!n)?{value:I(t[r](e)),done:"return"===r}:i?i(e):e}:i}}function y(t){if(!Symbol.asyncIterator)throw new TypeError("Symbol.asyncIterator is not defined.");var e,n=t[Symbol.asyncIterator];return n?n.call(t):(t=h(t),e={},r("next"),r("throw"),r("return"),e[Symbol.asyncIterator]=function(){return this},e);function r(n){e[n]=t[n]&&function(e){return new Promise(function(r,i){(function(t,e,n,r){Promise.resolve(r).then(function(e){t({value:e,done:n})},e)})(r,i,(e=t[n](e)).done,e.value)})}}}function E(t,e){return Object.defineProperty?Object.defineProperty(t,"raw",{value:e}):t.raw=e,t}function v(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)Object.hasOwnProperty.call(t,n)&&(e[n]=t[n]);return e.default=t,e}function B(t){return t&&t.__esModule?t:{default:t}}},function(t,e,n){"use strict";n.r(e),function(t){n.d(e,"ACTIVE",function(){return A}),n.d(e,"ALIGN_LEFT",function(){return l}),n.d(e,"ALIGN_RIGHT",function(){return c}),n.d(e,"DARK",function(){return u}),n.d(e,"DISABLED",function(){return g}),n.d(e,"FILL",function(){return h}),n.d(e,"FIXED",function(){return d}),n.d(e,"FIXED_TOP",function(){return f}),n.d(e,"INLINE",function(){return p}),n.d(e,"INTERACTIVE",function(){return I}),n.d(e,"LARGE",function(){return C}),n.d(e,"LOADING",function(){return m}),n.d(e,"MINIMAL",function(){return y}),n.d(e,"MULTILINE",function(){return E}),n.d(e,"ROUND",function(){return v}),n.d(e,"SMALL",function(){return B}),n.d(e,"VERTICAL",function(){return b}),n.d(e,"POSITION_TOP",function(){return Q}),n.d(e,"POSITION_BOTTOM",function(){return w}),n.d(e,"POSITION_LEFT",function(){return x}),n.d(e,"POSITION_RIGHT",function(){return _}),n.d(e,"ELEVATION_0",function(){return D}),n.d(e,"ELEVATION_1",function(){return M}),n.d(e,"ELEVATION_2",function(){return k}),n.d(e,"ELEVATION_3",function(){return S}),n.d(e,"ELEVATION_4",function(){return L}),n.d(e,"INTENT_PRIMARY",function(){return R}),n.d(e,"INTENT_SUCCESS",function(){return N}),n.d(e,"INTENT_WARNING",function(){return F}),n.d(e,"INTENT_DANGER",function(){return T}),n.d(e,"FOCUS_DISABLED",function(){return U}),n.d(e,"UI_TEXT",function(){return O}),n.d(e,"RUNNING_TEXT",function(){return G}),n.d(e,"MONOSPACE_TEXT",function(){return H}),n.d(e,"TEXT_LARGE",function(){return j}),n.d(e,"TEXT_SMALL",function(){return Y}),n.d(e,"TEXT_MUTED",function(){return P}),n.d(e,"TEXT_DISABLED",function(){return K}),n.d(e,"TEXT_OVERFLOW_ELLIPSIS",function(){return z}),n.d(e,"BLOCKQUOTE",function(){return J}),n.d(e,"CODE",function(){return q}),n.d(e,"CODE_BLOCK",function(){return V}),n.d(e,"HEADING",function(){return W}),n.d(e,"LIST",function(){return X}),n.d(e,"LIST_UNSTYLED",function(){return Z}),n.d(e,"RTL",function(){return $}),n.d(e,"ALERT",function(){return tt}),n.d(e,"ALERT_BODY",function(){return et}),n.d(e,"ALERT_CONTENTS",function(){return nt}),n.d(e,"ALERT_FOOTER",function(){return rt}),n.d(e,"BREADCRUMB",function(){return it}),n.d(e,"BREADCRUMB_CURRENT",function(){return ot}),n.d(e,"BREADCRUMBS",function(){return at}),n.d(e,"BREADCRUMBS_COLLAPSED",function(){return st}),n.d(e,"BUTTON",function(){return At}),n.d(e,"BUTTON_GROUP",function(){return lt}),n.d(e,"BUTTON_SPINNER",function(){return ct}),n.d(e,"BUTTON_TEXT",function(){return ut}),n.d(e,"CALLOUT",function(){return gt}),n.d(e,"CALLOUT_ICON",function(){return ht}),n.d(e,"CARD",function(){return dt}),n.d(e,"COLLAPSE",function(){return ft}),n.d(e,"COLLAPSE_BODY",function(){return pt}),n.d(e,"COLLAPSIBLE_LIST",function(){return It}),n.d(e,"CONTEXT_MENU",function(){return Ct}),n.d(e,"CONTEXT_MENU_POPOVER_TARGET",function(){return mt}),n.d(e,"CONTROL_GROUP",function(){return yt}),n.d(e,"DIALOG",function(){return Et}),n.d(e,"DIALOG_CONTAINER",function(){return vt}),n.d(e,"DIALOG_BODY",function(){return Bt}),n.d(e,"DIALOG_CLOSE_BUTTON",function(){return bt}),n.d(e,"DIALOG_FOOTER",function(){return Qt}),n.d(e,"DIALOG_FOOTER_ACTIONS",function(){return wt}),n.d(e,"DIALOG_HEADER",function(){return xt}),n.d(e,"DIVIDER",function(){return _t}),n.d(e,"DRAWER",function(){return Dt}),n.d(e,"DRAWER_BODY",function(){return Mt}),n.d(e,"DRAWER_FOOTER",function(){return kt}),n.d(e,"DRAWER_HEADER",function(){return St}),n.d(e,"EDITABLE_TEXT",function(){return Lt}),n.d(e,"EDITABLE_TEXT_CONTENT",function(){return Rt}),n.d(e,"EDITABLE_TEXT_EDITING",function(){return Nt}),n.d(e,"EDITABLE_TEXT_INPUT",function(){return Ft}),n.d(e,"EDITABLE_TEXT_PLACEHOLDER",function(){return Tt}),n.d(e,"FLEX_EXPANDER",function(){return Ut}),n.d(e,"HTML_SELECT",function(){return Ot}),n.d(e,"SELECT",function(){return Gt}),n.d(e,"HTML_TABLE",function(){return Ht}),n.d(e,"HTML_TABLE_BORDERED",function(){return jt}),n.d(e,"HTML_TABLE_CONDENSED",function(){return Yt}),n.d(e,"HTML_TABLE_STRIPED",function(){return Pt}),n.d(e,"INPUT",function(){return Kt}),n.d(e,"INPUT_GHOST",function(){return zt}),n.d(e,"INPUT_GROUP",function(){return Jt}),n.d(e,"INPUT_ACTION",function(){return qt}),n.d(e,"CONTROL",function(){return Vt}),n.d(e,"CONTROL_INDICATOR",function(){return Wt}),n.d(e,"CONTROL_INDICATOR_CHILD",function(){return Xt}),n.d(e,"CHECKBOX",function(){return Zt}),n.d(e,"RADIO",function(){return $t}),n.d(e,"SWITCH",function(){return te}),n.d(e,"SWITCH_INNER_TEXT",function(){return ee}),n.d(e,"FILE_INPUT",function(){return ne}),n.d(e,"FILE_INPUT_HAS_SELECTION",function(){return re}),n.d(e,"FILE_UPLOAD_INPUT",function(){return ie}),n.d(e,"KEY",function(){return oe}),n.d(e,"KEY_COMBO",function(){return ae}),n.d(e,"MODIFIER_KEY",function(){return se}),n.d(e,"HOTKEY",function(){return Ae}),n.d(e,"HOTKEY_LABEL",function(){return le}),n.d(e,"HOTKEY_COLUMN",function(){return ce}),n.d(e,"HOTKEY_DIALOG",function(){return ue}),n.d(e,"LABEL",function(){return ge}),n.d(e,"FORM_GROUP",function(){return he}),n.d(e,"FORM_CONTENT",function(){return de}),n.d(e,"FORM_HELPER_TEXT",function(){return fe}),n.d(e,"MENU",function(){return pe}),n.d(e,"MENU_ITEM",function(){return Ie}),n.d(e,"MENU_ITEM_LABEL",function(){return Ce}),n.d(e,"MENU_SUBMENU",function(){return me}),n.d(e,"MENU_DIVIDER",function(){return ye}),n.d(e,"MENU_HEADER",function(){return Ee}),n.d(e,"NAVBAR",function(){return ve}),n.d(e,"NAVBAR_GROUP",function(){return Be}),n.d(e,"NAVBAR_HEADING",function(){return be}),n.d(e,"NAVBAR_DIVIDER",function(){return Qe}),n.d(e,"NON_IDEAL_STATE",function(){return we}),n.d(e,"NON_IDEAL_STATE_VISUAL",function(){return xe}),n.d(e,"NUMERIC_INPUT",function(){return _e}),n.d(e,"OVERFLOW_LIST",function(){return De}),n.d(e,"OVERFLOW_LIST_SPACER",function(){return Me}),n.d(e,"OVERLAY",function(){return ke}),n.d(e,"OVERLAY_BACKDROP",function(){return Se}),n.d(e,"OVERLAY_CONTAINER",function(){return Le}),n.d(e,"OVERLAY_CONTENT",function(){return Re}),n.d(e,"OVERLAY_INLINE",function(){return Ne}),n.d(e,"OVERLAY_OPEN",function(){return Fe}),n.d(e,"OVERLAY_SCROLL_CONTAINER",function(){return Te}),n.d(e,"PANEL_STACK",function(){return Ue}),n.d(e,"PANEL_STACK_HEADER",function(){return Oe}),n.d(e,"PANEL_STACK_HEADER_BACK",function(){return Ge}),n.d(e,"PANEL_STACK_VIEW",function(){return He}),n.d(e,"POPOVER",function(){return je}),n.d(e,"POPOVER_ARROW",function(){return Ye}),n.d(e,"POPOVER_BACKDROP",function(){return Pe}),n.d(e,"POPOVER_CONTENT",function(){return Ke}),n.d(e,"POPOVER_CONTENT_SIZING",function(){return ze}),n.d(e,"POPOVER_DISMISS",function(){return Je}),n.d(e,"POPOVER_DISMISS_OVERRIDE",function(){return qe}),n.d(e,"POPOVER_OPEN",function(){return Ve}),n.d(e,"POPOVER_TARGET",function(){return We}),n.d(e,"POPOVER_WRAPPER",function(){return Xe}),n.d(e,"TRANSITION_CONTAINER",function(){return Ze}),n.d(e,"PROGRESS_BAR",function(){return $e}),n.d(e,"PROGRESS_METER",function(){return tn}),n.d(e,"PROGRESS_NO_STRIPES",function(){return en}),n.d(e,"PROGRESS_NO_ANIMATION",function(){return nn}),n.d(e,"PORTAL",function(){return rn}),n.d(e,"SKELETON",function(){return on}),n.d(e,"SLIDER",function(){return an}),n.d(e,"SLIDER_AXIS",function(){return sn}),n.d(e,"SLIDER_HANDLE",function(){return An}),n.d(e,"SLIDER_LABEL",function(){return ln}),n.d(e,"SLIDER_TRACK",function(){return cn}),n.d(e,"SLIDER_PROGRESS",function(){return un}),n.d(e,"START",function(){return gn}),n.d(e,"END",function(){return hn}),n.d(e,"SPINNER",function(){return dn}),n.d(e,"SPINNER_ANIMATION",function(){return fn}),n.d(e,"SPINNER_HEAD",function(){return pn}),n.d(e,"SPINNER_NO_SPIN",function(){return In}),n.d(e,"SPINNER_TRACK",function(){return Cn}),n.d(e,"TAB",function(){return mn}),n.d(e,"TAB_INDICATOR",function(){return yn}),n.d(e,"TAB_INDICATOR_WRAPPER",function(){return En}),n.d(e,"TAB_LIST",function(){return vn}),n.d(e,"TAB_PANEL",function(){return Bn}),n.d(e,"TABS",function(){return bn}),n.d(e,"TAG",function(){return Qn}),n.d(e,"TAG_REMOVE",function(){return wn}),n.d(e,"TAG_INPUT",function(){return xn}),n.d(e,"TAG_INPUT_ICON",function(){return _n}),n.d(e,"TAG_INPUT_VALUES",function(){return Dn}),n.d(e,"TOAST",function(){return Mn}),n.d(e,"TOAST_CONTAINER",function(){return kn}),n.d(e,"TOAST_MESSAGE",function(){return Sn}),n.d(e,"TOOLTIP",function(){return Ln}),n.d(e,"TOOLTIP_INDICATOR",function(){return Rn}),n.d(e,"TREE",function(){return Nn}),n.d(e,"TREE_NODE",function(){return Fn}),n.d(e,"TREE_NODE_CARET",function(){return Tn}),n.d(e,"TREE_NODE_CARET_CLOSED",function(){return Un}),n.d(e,"TREE_NODE_CARET_NONE",function(){return On}),n.d(e,"TREE_NODE_CARET_OPEN",function(){return Gn}),n.d(e,"TREE_NODE_CONTENT",function(){return Hn}),n.d(e,"TREE_NODE_EXPANDED",function(){return jn}),n.d(e,"TREE_NODE_ICON",function(){return Yn}),n.d(e,"TREE_NODE_LABEL",function(){return Pn}),n.d(e,"TREE_NODE_LIST",function(){return Kn}),n.d(e,"TREE_NODE_SECONDARY_LABEL",function(){return zn}),n.d(e,"TREE_NODE_SELECTED",function(){return Jn}),n.d(e,"TREE_ROOT",function(){return qn}),n.d(e,"ICON",function(){return Vn}),n.d(e,"ICON_STANDARD",function(){return Wn}),n.d(e,"ICON_LARGE",function(){return Xn}),n.d(e,"getClassNamespace",function(){return Zn}),n.d(e,"alignmentClass",function(){return $n}),n.d(e,"elevationClass",function(){return tr}),n.d(e,"iconClass",function(){return er}),n.d(e,"intentClass",function(){return nr}),n.d(e,"positionClass",function(){return rr});var r=n(128),i=n(94),o=n(74),a=n(34),s=t.env.BLUEPRINT_NAMESPACE||"bp3",A=s+"-active",l=s+"-align-left",c=s+"-align-right",u=s+"-dark",g=s+"-disabled",h=s+"-fill",d=s+"-fixed",f=s+"-fixed-top",p=s+"-inline",I=s+"-interactive",C=s+"-large",m=s+"-loading",y=s+"-minimal",E=s+"-multiline",v=s+"-round",B=s+"-small",b=s+"-vertical",Q=rr(a.a.TOP),w=rr(a.a.BOTTOM),x=rr(a.a.LEFT),_=rr(a.a.RIGHT),D=tr(i.a.ZERO),M=tr(i.a.ONE),k=tr(i.a.TWO),S=tr(i.a.THREE),L=tr(i.a.FOUR),R=nr(o.a.PRIMARY),N=nr(o.a.SUCCESS),F=nr(o.a.WARNING),T=nr(o.a.DANGER),U=s+"-focus-disabled",O=s+"-ui-text",G=s+"-running-text",H=s+"-monospace-text",j=s+"-text-large",Y=s+"-text-small",P=s+"-text-muted",K=s+"-text-disabled",z=s+"-text-overflow-ellipsis",J=s+"-blockquote",q=s+"-code",V=s+"-code-block",W=s+"-heading",X=s+"-list",Z=s+"-list-unstyled",$=s+"-rtl",tt=s+"-alert",et=tt+"-body",nt=tt+"-contents",rt=tt+"-footer",it=s+"-breadcrumb",ot=it+"-current",at=it+"s",st=it+"s-collapsed",At=s+"-button",lt=At+"-group",ct=At+"-spinner",ut=At+"-text",gt=s+"-callout",ht=gt+"-icon",dt=s+"-card",ft=s+"-collapse",pt=ft+"-body",It=s+"-collapse-list",Ct=s+"-context-menu",mt=Ct+"-popover-target",yt=s+"-control-group",Et=s+"-dialog",vt=Et+"-container",Bt=Et+"-body",bt=Et+"-close-button",Qt=Et+"-footer",wt=Et+"-footer-actions",xt=Et+"-header",_t=s+"-divider",Dt=s+"-drawer",Mt=Dt+"-body",kt=Dt+"-footer",St=Dt+"-header",Lt=s+"-editable-text",Rt=Lt+"-content",Nt=Lt+"-editing",Ft=Lt+"-input",Tt=Lt+"-placeholder",Ut=s+"-flex-expander",Ot=s+"-html-select",Gt=s+"-select",Ht=s+"-html-table",jt=Ht+"-bordered",Yt=Ht+"-condensed",Pt=Ht+"-striped",Kt=s+"-input",zt=Kt+"-ghost",Jt=Kt+"-group",qt=Kt+"-action",Vt=s+"-control",Wt=Vt+"-indicator",Xt=Wt+"-child",Zt=s+"-checkbox",$t=s+"-radio",te=s+"-switch",ee=te+"-inner-text",ne=s+"-file-input",re=s+"-file-input-has-selection",ie=s+"-file-upload-input",oe=s+"-key",ae=oe+"-combo",se=s+"-modifier-key",Ae=s+"-hotkey",le=Ae+"-label",ce=Ae+"-column",ue=Ae+"-dialog",ge=s+"-label",he=s+"-form-group",de=s+"-form-content",fe=s+"-form-helper-text",pe=s+"-menu",Ie=pe+"-item",Ce=Ie+"-label",me=s+"-submenu",ye=pe+"-divider",Ee=pe+"-header",ve=s+"-navbar",Be=ve+"-group",be=ve+"-heading",Qe=ve+"-divider",we=s+"-non-ideal-state",xe=we+"-visual",_e=s+"-numeric-input",De=s+"-overflow-list",Me=De+"-spacer",ke=s+"-overlay",Se=ke+"-backdrop",Le=ke+"-container",Re=ke+"-content",Ne=ke+"-inline",Fe=ke+"-open",Te=ke+"-scroll-container",Ue=s+"-panel-stack",Oe=Ue+"-header",Ge=Ue+"-header-back",He=Ue+"-view",je=s+"-popover",Ye=je+"-arrow",Pe=je+"-backdrop",Ke=je+"-content",ze=Ke+"-sizing",Je=je+"-dismiss",qe=Je+"-override",Ve=je+"-open",We=je+"-target",Xe=je+"-wrapper",Ze=s+"-transition-container",$e=s+"-progress-bar",tn=s+"-progress-meter",en=s+"-no-stripes",nn=s+"-no-animation",rn=s+"-portal",on=s+"-skeleton",an=s+"-slider",sn=an+"-axis",An=an+"-handle",ln=an+"-label",cn=an+"-track",un=an+"-progress",gn=s+"-start",hn=s+"-end",dn=s+"-spinner",fn=dn+"-animation",pn=dn+"-head",In=s+"-no-spin",Cn=dn+"-track",mn=s+"-tab",yn=mn+"-indicator",En=yn+"-wrapper",vn=mn+"-list",Bn=mn+"-panel",bn=mn+"s",Qn=s+"-tag",wn=Qn+"-remove",xn=s+"-tag-input",_n=xn+"-icon",Dn=xn+"-values",Mn=s+"-toast",kn=Mn+"-container",Sn=Mn+"-message",Ln=s+"-tooltip",Rn=Ln+"-indicator",Nn=s+"-tree",Fn=s+"-tree-node",Tn=Fn+"-caret",Un=Tn+"-closed",On=Tn+"-none",Gn=Tn+"-open",Hn=Fn+"-content",jn=Fn+"-expanded",Yn=Fn+"-icon",Pn=Fn+"-label",Kn=Fn+"-list",zn=Fn+"-secondary-label",Jn=Fn+"-selected",qn=s+"-tree-root",Vn=s+"-icon",Wn=Vn+"-standard",Xn=Vn+"-large";function Zn(){return s}function $n(t){switch(t){case r.a.LEFT:return l;case r.a.RIGHT:return c;default:return}}function tr(t){if(null!=t)return s+"-elevation-"+t}function er(t){if(null!=t)return 0===t.indexOf(s+"-icon-")?t:s+"-icon-"+t}function nr(t){if(null!=t&&t!==o.a.NONE)return s+"-intent-"+t.toLowerCase()}function rr(t){if(null!=t)return s+"-position-"+t}}.call(this,n(58))},function(t,e,n){var r;
/*!
  Copyright (c) 2017 Jed Watson.
  Licensed under the MIT License (MIT), see
  http://jedwatson.github.io/classnames
*/
/*!
  Copyright (c) 2017 Jed Watson.
  Licensed under the MIT License (MIT), see
  http://jedwatson.github.io/classnames
*/
!function(){"use strict";var n={}.hasOwnProperty;function i(){for(var t=[],e=0;e<arguments.length;e++){var r=arguments[e];if(r){var o=typeof r;if("string"===o||"number"===o)t.push(r);else if(Array.isArray(r)&&r.length){var a=i.apply(null,r);a&&t.push(a)}else if("object"===o)for(var s in r)n.call(r,s)&&r[s]&&t.push(s)}}return t.join(" ")}t.exports?(i.default=i,t.exports=i):void 0===(r=function(){return i}.apply(e,[]))||(t.exports=r)}()},function(t,e,n){"use strict";function r(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}Object.defineProperty(e,"__esModule",{value:!0}),r(n(746)),r(n(747)),r(n(748)),r(n(749)),r(n(750)),r(n(751)),r(n(82)),r(n(752)),r(n(753)),r(n(754)),r(n(755)),r(n(756)),r(n(757)),r(n(758)),r(n(759)),r(n(760)),r(n(761))},function(t,e,n){"use strict";function r(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}Object.defineProperty(e,"__esModule",{value:!0}),r(n(174)),r(n(288)),r(n(1033)),r(n(1034)),r(n(1035)),r(n(572)),r(n(1036)),r(n(1037)),r(n(1038)),r(n(107)),r(n(571)),r(n(1039)),r(n(215)),r(n(175)),r(n(1040)),r(n(1041)),r(n(573)),r(n(1042)),r(n(574)),r(n(575)),r(n(291)),r(n(1043)),r(n(289)),r(n(59))},function(t,e,n){(function(t){t.exports=function(){"use strict";var e,r;function i(){return e.apply(null,arguments)}function o(t){return t instanceof Array||"[object Array]"===Object.prototype.toString.call(t)}function a(t){return null!=t&&"[object Object]"===Object.prototype.toString.call(t)}function s(t){return void 0===t}function A(t){return"number"==typeof t||"[object Number]"===Object.prototype.toString.call(t)}function l(t){return t instanceof Date||"[object Date]"===Object.prototype.toString.call(t)}function c(t,e){var n,r=[];for(n=0;n<t.length;++n)r.push(e(t[n],n));return r}function u(t,e){return Object.prototype.hasOwnProperty.call(t,e)}function g(t,e){for(var n in e)u(e,n)&&(t[n]=e[n]);return u(e,"toString")&&(t.toString=e.toString),u(e,"valueOf")&&(t.valueOf=e.valueOf),t}function h(t,e,n,r){return Me(t,e,n,r,!0).utc()}function d(t){return null==t._pf&&(t._pf={empty:!1,unusedTokens:[],unusedInput:[],overflow:-2,charsLeftOver:0,nullInput:!1,invalidMonth:null,invalidFormat:!1,userInvalidated:!1,iso:!1,parsedDateParts:[],meridiem:null,rfc2822:!1,weekdayMismatch:!1}),t._pf}function f(t){if(null==t._isValid){var e=d(t),n=r.call(e.parsedDateParts,function(t){return null!=t}),i=!isNaN(t._d.getTime())&&e.overflow<0&&!e.empty&&!e.invalidMonth&&!e.invalidWeekday&&!e.weekdayMismatch&&!e.nullInput&&!e.invalidFormat&&!e.userInvalidated&&(!e.meridiem||e.meridiem&&n);if(t._strict&&(i=i&&0===e.charsLeftOver&&0===e.unusedTokens.length&&void 0===e.bigHour),null!=Object.isFrozen&&Object.isFrozen(t))return i;t._isValid=i}return t._isValid}function p(t){var e=h(NaN);return null!=t?g(d(e),t):d(e).userInvalidated=!0,e}r=Array.prototype.some?Array.prototype.some:function(t){for(var e=Object(this),n=e.length>>>0,r=0;r<n;r++)if(r in e&&t.call(this,e[r],r,e))return!0;return!1};var I=i.momentProperties=[];function C(t,e){var n,r,i;if(s(e._isAMomentObject)||(t._isAMomentObject=e._isAMomentObject),s(e._i)||(t._i=e._i),s(e._f)||(t._f=e._f),s(e._l)||(t._l=e._l),s(e._strict)||(t._strict=e._strict),s(e._tzm)||(t._tzm=e._tzm),s(e._isUTC)||(t._isUTC=e._isUTC),s(e._offset)||(t._offset=e._offset),s(e._pf)||(t._pf=d(e)),s(e._locale)||(t._locale=e._locale),I.length>0)for(n=0;n<I.length;n++)r=I[n],s(i=e[r])||(t[r]=i);return t}var m=!1;function y(t){C(this,t),this._d=new Date(null!=t._d?t._d.getTime():NaN),this.isValid()||(this._d=new Date(NaN)),!1===m&&(m=!0,i.updateOffset(this),m=!1)}function E(t){return t instanceof y||null!=t&&null!=t._isAMomentObject}function v(t){return t<0?Math.ceil(t)||0:Math.floor(t)}function B(t){var e=+t,n=0;return 0!==e&&isFinite(e)&&(n=v(e)),n}function b(t,e,n){var r,i=Math.min(t.length,e.length),o=Math.abs(t.length-e.length),a=0;for(r=0;r<i;r++)(n&&t[r]!==e[r]||!n&&B(t[r])!==B(e[r]))&&a++;return a+o}function Q(t){!1===i.suppressDeprecationWarnings&&"undefined"!=typeof console&&console.warn&&console.warn("Deprecation warning: "+t)}function w(t,e){var n=!0;return g(function(){if(null!=i.deprecationHandler&&i.deprecationHandler(null,t),n){for(var r,o=[],a=0;a<arguments.length;a++){if(r="","object"==typeof arguments[a]){for(var s in r+="\n["+a+"] ",arguments[0])r+=s+": "+arguments[0][s]+", ";r=r.slice(0,-2)}else r=arguments[a];o.push(r)}Q(t+"\nArguments: "+Array.prototype.slice.call(o).join("")+"\n"+(new Error).stack),n=!1}return e.apply(this,arguments)},e)}var x,_={};function D(t,e){null!=i.deprecationHandler&&i.deprecationHandler(t,e),_[t]||(Q(e),_[t]=!0)}function M(t){return t instanceof Function||"[object Function]"===Object.prototype.toString.call(t)}function k(t,e){var n,r=g({},t);for(n in e)u(e,n)&&(a(t[n])&&a(e[n])?(r[n]={},g(r[n],t[n]),g(r[n],e[n])):null!=e[n]?r[n]=e[n]:delete r[n]);for(n in t)u(t,n)&&!u(e,n)&&a(t[n])&&(r[n]=g({},r[n]));return r}function S(t){null!=t&&this.set(t)}i.suppressDeprecationWarnings=!1,i.deprecationHandler=null,x=Object.keys?Object.keys:function(t){var e,n=[];for(e in t)u(t,e)&&n.push(e);return n};var L={};function R(t,e){var n=t.toLowerCase();L[n]=L[n+"s"]=L[e]=t}function N(t){return"string"==typeof t?L[t]||L[t.toLowerCase()]:void 0}function F(t){var e,n,r={};for(n in t)u(t,n)&&(e=N(n))&&(r[e]=t[n]);return r}var T={};function U(t,e){T[t]=e}function O(t,e,n){var r=""+Math.abs(t),i=e-r.length,o=t>=0;return(o?n?"+":"":"-")+Math.pow(10,Math.max(0,i)).toString().substr(1)+r}var G=/(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|YYYYYY|YYYYY|YYYY|YY|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,H=/(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,j={},Y={};function P(t,e,n,r){var i=r;"string"==typeof r&&(i=function(){return this[r]()}),t&&(Y[t]=i),e&&(Y[e[0]]=function(){return O(i.apply(this,arguments),e[1],e[2])}),n&&(Y[n]=function(){return this.localeData().ordinal(i.apply(this,arguments),t)})}function K(t,e){return t.isValid()?(e=z(e,t.localeData()),j[e]=j[e]||function(t){var e,n,r,i=t.match(G);for(e=0,n=i.length;e<n;e++)Y[i[e]]?i[e]=Y[i[e]]:i[e]=(r=i[e]).match(/\[[\s\S]/)?r.replace(/^\[|\]$/g,""):r.replace(/\\/g,"");return function(e){var r,o="";for(r=0;r<n;r++)o+=M(i[r])?i[r].call(e,t):i[r];return o}}(e),j[e](t)):t.localeData().invalidDate()}function z(t,e){var n=5;function r(t){return e.longDateFormat(t)||t}for(H.lastIndex=0;n>=0&&H.test(t);)t=t.replace(H,r),H.lastIndex=0,n-=1;return t}var J=/\d/,q=/\d\d/,V=/\d{3}/,W=/\d{4}/,X=/[+-]?\d{6}/,Z=/\d\d?/,$=/\d\d\d\d?/,tt=/\d\d\d\d\d\d?/,et=/\d{1,3}/,nt=/\d{1,4}/,rt=/[+-]?\d{1,6}/,it=/\d+/,ot=/[+-]?\d+/,at=/Z|[+-]\d\d:?\d\d/gi,st=/Z|[+-]\d\d(?::?\d\d)?/gi,At=/[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i,lt={};function ct(t,e,n){lt[t]=M(e)?e:function(t,r){return t&&n?n:e}}function ut(t,e){return u(lt,t)?lt[t](e._strict,e._locale):new RegExp(gt(t.replace("\\","").replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g,function(t,e,n,r,i){return e||n||r||i})))}function gt(t){return t.replace(/[-\/\\^$*+?.()|[\]{}]/g,"\\$&")}var ht={};function dt(t,e){var n,r=e;for("string"==typeof t&&(t=[t]),A(e)&&(r=function(t,n){n[e]=B(t)}),n=0;n<t.length;n++)ht[t[n]]=r}function ft(t,e){dt(t,function(t,n,r,i){r._w=r._w||{},e(t,r._w,r,i)})}function pt(t,e,n){null!=e&&u(ht,t)&&ht[t](e,n._a,n,t)}var It=0,Ct=1,mt=2,yt=3,Et=4,vt=5,Bt=6,bt=7,Qt=8;function wt(t){return xt(t)?366:365}function xt(t){return t%4==0&&t%100!=0||t%400==0}P("Y",0,0,function(){var t=this.year();return t<=9999?""+t:"+"+t}),P(0,["YY",2],0,function(){return this.year()%100}),P(0,["YYYY",4],0,"year"),P(0,["YYYYY",5],0,"year"),P(0,["YYYYYY",6,!0],0,"year"),R("year","y"),U("year",1),ct("Y",ot),ct("YY",Z,q),ct("YYYY",nt,W),ct("YYYYY",rt,X),ct("YYYYYY",rt,X),dt(["YYYYY","YYYYYY"],It),dt("YYYY",function(t,e){e[It]=2===t.length?i.parseTwoDigitYear(t):B(t)}),dt("YY",function(t,e){e[It]=i.parseTwoDigitYear(t)}),dt("Y",function(t,e){e[It]=parseInt(t,10)}),i.parseTwoDigitYear=function(t){return B(t)+(B(t)>68?1900:2e3)};var _t,Dt=Mt("FullYear",!0);function Mt(t,e){return function(n){return null!=n?(St(this,t,n),i.updateOffset(this,e),this):kt(this,t)}}function kt(t,e){return t.isValid()?t._d["get"+(t._isUTC?"UTC":"")+e]():NaN}function St(t,e,n){t.isValid()&&!isNaN(n)&&("FullYear"===e&&xt(t.year())&&1===t.month()&&29===t.date()?t._d["set"+(t._isUTC?"UTC":"")+e](n,t.month(),Lt(n,t.month())):t._d["set"+(t._isUTC?"UTC":"")+e](n))}function Lt(t,e){if(isNaN(t)||isNaN(e))return NaN;var n,r=(e%(n=12)+n)%n;return t+=(e-r)/12,1===r?xt(t)?29:28:31-r%7%2}_t=Array.prototype.indexOf?Array.prototype.indexOf:function(t){var e;for(e=0;e<this.length;++e)if(this[e]===t)return e;return-1},P("M",["MM",2],"Mo",function(){return this.month()+1}),P("MMM",0,0,function(t){return this.localeData().monthsShort(this,t)}),P("MMMM",0,0,function(t){return this.localeData().months(this,t)}),R("month","M"),U("month",8),ct("M",Z),ct("MM",Z,q),ct("MMM",function(t,e){return e.monthsShortRegex(t)}),ct("MMMM",function(t,e){return e.monthsRegex(t)}),dt(["M","MM"],function(t,e){e[Ct]=B(t)-1}),dt(["MMM","MMMM"],function(t,e,n,r){var i=n._locale.monthsParse(t,r,n._strict);null!=i?e[Ct]=i:d(n).invalidMonth=t});var Rt=/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,Nt="January_February_March_April_May_June_July_August_September_October_November_December".split("_"),Ft="Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_");function Tt(t,e){var n;if(!t.isValid())return t;if("string"==typeof e)if(/^\d+$/.test(e))e=B(e);else if(!A(e=t.localeData().monthsParse(e)))return t;return n=Math.min(t.date(),Lt(t.year(),e)),t._d["set"+(t._isUTC?"UTC":"")+"Month"](e,n),t}function Ut(t){return null!=t?(Tt(this,t),i.updateOffset(this,!0),this):kt(this,"Month")}var Ot=At,Gt=At;function Ht(){function t(t,e){return e.length-t.length}var e,n,r=[],i=[],o=[];for(e=0;e<12;e++)n=h([2e3,e]),r.push(this.monthsShort(n,"")),i.push(this.months(n,"")),o.push(this.months(n,"")),o.push(this.monthsShort(n,""));for(r.sort(t),i.sort(t),o.sort(t),e=0;e<12;e++)r[e]=gt(r[e]),i[e]=gt(i[e]);for(e=0;e<24;e++)o[e]=gt(o[e]);this._monthsRegex=new RegExp("^("+o.join("|")+")","i"),this._monthsShortRegex=this._monthsRegex,this._monthsStrictRegex=new RegExp("^("+i.join("|")+")","i"),this._monthsShortStrictRegex=new RegExp("^("+r.join("|")+")","i")}function jt(t){var e;if(t<100&&t>=0){var n=Array.prototype.slice.call(arguments);n[0]=t+400,e=new Date(Date.UTC.apply(null,n)),isFinite(e.getUTCFullYear())&&e.setUTCFullYear(t)}else e=new Date(Date.UTC.apply(null,arguments));return e}function Yt(t,e,n){var r=7+e-n,i=(7+jt(t,0,r).getUTCDay()-e)%7;return-i+r-1}function Pt(t,e,n,r,i){var o,a,s=(7+n-r)%7,A=Yt(t,r,i),l=1+7*(e-1)+s+A;return l<=0?a=wt(o=t-1)+l:l>wt(t)?(o=t+1,a=l-wt(t)):(o=t,a=l),{year:o,dayOfYear:a}}function Kt(t,e,n){var r,i,o=Yt(t.year(),e,n),a=Math.floor((t.dayOfYear()-o-1)/7)+1;return a<1?(i=t.year()-1,r=a+zt(i,e,n)):a>zt(t.year(),e,n)?(r=a-zt(t.year(),e,n),i=t.year()+1):(i=t.year(),r=a),{week:r,year:i}}function zt(t,e,n){var r=Yt(t,e,n),i=Yt(t+1,e,n);return(wt(t)-r+i)/7}function Jt(t,e){return t.slice(e,7).concat(t.slice(0,e))}P("w",["ww",2],"wo","week"),P("W",["WW",2],"Wo","isoWeek"),R("week","w"),R("isoWeek","W"),U("week",5),U("isoWeek",5),ct("w",Z),ct("ww",Z,q),ct("W",Z),ct("WW",Z,q),ft(["w","ww","W","WW"],function(t,e,n,r){e[r.substr(0,1)]=B(t)}),P("d",0,"do","day"),P("dd",0,0,function(t){return this.localeData().weekdaysMin(this,t)}),P("ddd",0,0,function(t){return this.localeData().weekdaysShort(this,t)}),P("dddd",0,0,function(t){return this.localeData().weekdays(this,t)}),P("e",0,0,"weekday"),P("E",0,0,"isoWeekday"),R("day","d"),R("weekday","e"),R("isoWeekday","E"),U("day",11),U("weekday",11),U("isoWeekday",11),ct("d",Z),ct("e",Z),ct("E",Z),ct("dd",function(t,e){return e.weekdaysMinRegex(t)}),ct("ddd",function(t,e){return e.weekdaysShortRegex(t)}),ct("dddd",function(t,e){return e.weekdaysRegex(t)}),ft(["dd","ddd","dddd"],function(t,e,n,r){var i=n._locale.weekdaysParse(t,r,n._strict);null!=i?e.d=i:d(n).invalidWeekday=t}),ft(["d","e","E"],function(t,e,n,r){e[r]=B(t)});var qt="Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),Vt="Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),Wt="Su_Mo_Tu_We_Th_Fr_Sa".split("_"),Xt=At,Zt=At,$t=At;function te(){function t(t,e){return e.length-t.length}var e,n,r,i,o,a=[],s=[],A=[],l=[];for(e=0;e<7;e++)n=h([2e3,1]).day(e),r=this.weekdaysMin(n,""),i=this.weekdaysShort(n,""),o=this.weekdays(n,""),a.push(r),s.push(i),A.push(o),l.push(r),l.push(i),l.push(o);for(a.sort(t),s.sort(t),A.sort(t),l.sort(t),e=0;e<7;e++)s[e]=gt(s[e]),A[e]=gt(A[e]),l[e]=gt(l[e]);this._weekdaysRegex=new RegExp("^("+l.join("|")+")","i"),this._weekdaysShortRegex=this._weekdaysRegex,this._weekdaysMinRegex=this._weekdaysRegex,this._weekdaysStrictRegex=new RegExp("^("+A.join("|")+")","i"),this._weekdaysShortStrictRegex=new RegExp("^("+s.join("|")+")","i"),this._weekdaysMinStrictRegex=new RegExp("^("+a.join("|")+")","i")}function ee(){return this.hours()%12||12}function ne(t,e){P(t,0,0,function(){return this.localeData().meridiem(this.hours(),this.minutes(),e)})}function re(t,e){return e._meridiemParse}P("H",["HH",2],0,"hour"),P("h",["hh",2],0,ee),P("k",["kk",2],0,function(){return this.hours()||24}),P("hmm",0,0,function(){return""+ee.apply(this)+O(this.minutes(),2)}),P("hmmss",0,0,function(){return""+ee.apply(this)+O(this.minutes(),2)+O(this.seconds(),2)}),P("Hmm",0,0,function(){return""+this.hours()+O(this.minutes(),2)}),P("Hmmss",0,0,function(){return""+this.hours()+O(this.minutes(),2)+O(this.seconds(),2)}),ne("a",!0),ne("A",!1),R("hour","h"),U("hour",13),ct("a",re),ct("A",re),ct("H",Z),ct("h",Z),ct("k",Z),ct("HH",Z,q),ct("hh",Z,q),ct("kk",Z,q),ct("hmm",$),ct("hmmss",tt),ct("Hmm",$),ct("Hmmss",tt),dt(["H","HH"],yt),dt(["k","kk"],function(t,e,n){var r=B(t);e[yt]=24===r?0:r}),dt(["a","A"],function(t,e,n){n._isPm=n._locale.isPM(t),n._meridiem=t}),dt(["h","hh"],function(t,e,n){e[yt]=B(t),d(n).bigHour=!0}),dt("hmm",function(t,e,n){var r=t.length-2;e[yt]=B(t.substr(0,r)),e[Et]=B(t.substr(r)),d(n).bigHour=!0}),dt("hmmss",function(t,e,n){var r=t.length-4,i=t.length-2;e[yt]=B(t.substr(0,r)),e[Et]=B(t.substr(r,2)),e[vt]=B(t.substr(i)),d(n).bigHour=!0}),dt("Hmm",function(t,e,n){var r=t.length-2;e[yt]=B(t.substr(0,r)),e[Et]=B(t.substr(r))}),dt("Hmmss",function(t,e,n){var r=t.length-4,i=t.length-2;e[yt]=B(t.substr(0,r)),e[Et]=B(t.substr(r,2)),e[vt]=B(t.substr(i))});var ie,oe=Mt("Hours",!0),ae={calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},longDateFormat:{LTS:"h:mm:ss A",LT:"h:mm A",L:"MM/DD/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},invalidDate:"Invalid date",ordinal:"%d",dayOfMonthOrdinalParse:/\d{1,2}/,relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},months:Nt,monthsShort:Ft,week:{dow:0,doy:6},weekdays:qt,weekdaysMin:Wt,weekdaysShort:Vt,meridiemParse:/[ap]\.?m?\.?/i},se={},Ae={};function le(t){return t?t.toLowerCase().replace("_","-"):t}function ce(e){var r=null;if(!se[e]&&void 0!==t&&t&&t.exports)try{r=ie._abbr,n(904)("./"+e),ue(r)}catch(t){}return se[e]}function ue(t,e){var n;return t&&((n=s(e)?he(t):ge(t,e))?ie=n:"undefined"!=typeof console&&console.warn&&console.warn("Locale "+t+" not found. Did you forget to load it?")),ie._abbr}function ge(t,e){if(null!==e){var n,r=ae;if(e.abbr=t,null!=se[t])D("defineLocaleOverride","use moment.updateLocale(localeName, config) to change an existing locale. moment.defineLocale(localeName, config) should only be used for creating a new locale See http://momentjs.com/guides/#/warnings/define-locale/ for more info."),r=se[t]._config;else if(null!=e.parentLocale)if(null!=se[e.parentLocale])r=se[e.parentLocale]._config;else{if(null==(n=ce(e.parentLocale)))return Ae[e.parentLocale]||(Ae[e.parentLocale]=[]),Ae[e.parentLocale].push({name:t,config:e}),null;r=n._config}return se[t]=new S(k(r,e)),Ae[t]&&Ae[t].forEach(function(t){ge(t.name,t.config)}),ue(t),se[t]}return delete se[t],null}function he(t){var e;if(t&&t._locale&&t._locale._abbr&&(t=t._locale._abbr),!t)return ie;if(!o(t)){if(e=ce(t))return e;t=[t]}return function(t){for(var e,n,r,i,o=0;o<t.length;){for(i=le(t[o]).split("-"),e=i.length,n=(n=le(t[o+1]))?n.split("-"):null;e>0;){if(r=ce(i.slice(0,e).join("-")))return r;if(n&&n.length>=e&&b(i,n,!0)>=e-1)break;e--}o++}return ie}(t)}function de(t){var e,n=t._a;return n&&-2===d(t).overflow&&(e=n[Ct]<0||n[Ct]>11?Ct:n[mt]<1||n[mt]>Lt(n[It],n[Ct])?mt:n[yt]<0||n[yt]>24||24===n[yt]&&(0!==n[Et]||0!==n[vt]||0!==n[Bt])?yt:n[Et]<0||n[Et]>59?Et:n[vt]<0||n[vt]>59?vt:n[Bt]<0||n[Bt]>999?Bt:-1,d(t)._overflowDayOfYear&&(e<It||e>mt)&&(e=mt),d(t)._overflowWeeks&&-1===e&&(e=bt),d(t)._overflowWeekday&&-1===e&&(e=Qt),d(t).overflow=e),t}function fe(t,e,n){return null!=t?t:null!=e?e:n}function pe(t){var e,n,r,o,a,s=[];if(!t._d){for(r=function(t){var e=new Date(i.now());return t._useUTC?[e.getUTCFullYear(),e.getUTCMonth(),e.getUTCDate()]:[e.getFullYear(),e.getMonth(),e.getDate()]}(t),t._w&&null==t._a[mt]&&null==t._a[Ct]&&function(t){var e,n,r,i,o,a,s,A;if(null!=(e=t._w).GG||null!=e.W||null!=e.E)o=1,a=4,n=fe(e.GG,t._a[It],Kt(ke(),1,4).year),r=fe(e.W,1),((i=fe(e.E,1))<1||i>7)&&(A=!0);else{o=t._locale._week.dow,a=t._locale._week.doy;var l=Kt(ke(),o,a);n=fe(e.gg,t._a[It],l.year),r=fe(e.w,l.week),null!=e.d?((i=e.d)<0||i>6)&&(A=!0):null!=e.e?(i=e.e+o,(e.e<0||e.e>6)&&(A=!0)):i=o}r<1||r>zt(n,o,a)?d(t)._overflowWeeks=!0:null!=A?d(t)._overflowWeekday=!0:(s=Pt(n,r,i,o,a),t._a[It]=s.year,t._dayOfYear=s.dayOfYear)}(t),null!=t._dayOfYear&&(a=fe(t._a[It],r[It]),(t._dayOfYear>wt(a)||0===t._dayOfYear)&&(d(t)._overflowDayOfYear=!0),n=jt(a,0,t._dayOfYear),t._a[Ct]=n.getUTCMonth(),t._a[mt]=n.getUTCDate()),e=0;e<3&&null==t._a[e];++e)t._a[e]=s[e]=r[e];for(;e<7;e++)t._a[e]=s[e]=null==t._a[e]?2===e?1:0:t._a[e];24===t._a[yt]&&0===t._a[Et]&&0===t._a[vt]&&0===t._a[Bt]&&(t._nextDay=!0,t._a[yt]=0),t._d=(t._useUTC?jt:function(t,e,n,r,i,o,a){var s;return t<100&&t>=0?(s=new Date(t+400,e,n,r,i,o,a),isFinite(s.getFullYear())&&s.setFullYear(t)):s=new Date(t,e,n,r,i,o,a),s}).apply(null,s),o=t._useUTC?t._d.getUTCDay():t._d.getDay(),null!=t._tzm&&t._d.setUTCMinutes(t._d.getUTCMinutes()-t._tzm),t._nextDay&&(t._a[yt]=24),t._w&&void 0!==t._w.d&&t._w.d!==o&&(d(t).weekdayMismatch=!0)}}var Ie=/^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([\+\-]\d\d(?::?\d\d)?|\s*Z)?)?$/,Ce=/^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([\+\-]\d\d(?::?\d\d)?|\s*Z)?)?$/,me=/Z|[+-]\d\d(?::?\d\d)?/,ye=[["YYYYYY-MM-DD",/[+-]\d{6}-\d\d-\d\d/],["YYYY-MM-DD",/\d{4}-\d\d-\d\d/],["GGGG-[W]WW-E",/\d{4}-W\d\d-\d/],["GGGG-[W]WW",/\d{4}-W\d\d/,!1],["YYYY-DDD",/\d{4}-\d{3}/],["YYYY-MM",/\d{4}-\d\d/,!1],["YYYYYYMMDD",/[+-]\d{10}/],["YYYYMMDD",/\d{8}/],["GGGG[W]WWE",/\d{4}W\d{3}/],["GGGG[W]WW",/\d{4}W\d{2}/,!1],["YYYYDDD",/\d{7}/]],Ee=[["HH:mm:ss.SSSS",/\d\d:\d\d:\d\d\.\d+/],["HH:mm:ss,SSSS",/\d\d:\d\d:\d\d,\d+/],["HH:mm:ss",/\d\d:\d\d:\d\d/],["HH:mm",/\d\d:\d\d/],["HHmmss.SSSS",/\d\d\d\d\d\d\.\d+/],["HHmmss,SSSS",/\d\d\d\d\d\d,\d+/],["HHmmss",/\d\d\d\d\d\d/],["HHmm",/\d\d\d\d/],["HH",/\d\d/]],ve=/^\/?Date\((\-?\d+)/i;function Be(t){var e,n,r,i,o,a,s=t._i,A=Ie.exec(s)||Ce.exec(s);if(A){for(d(t).iso=!0,e=0,n=ye.length;e<n;e++)if(ye[e][1].exec(A[1])){i=ye[e][0],r=!1!==ye[e][2];break}if(null==i)return void(t._isValid=!1);if(A[3]){for(e=0,n=Ee.length;e<n;e++)if(Ee[e][1].exec(A[3])){o=(A[2]||" ")+Ee[e][0];break}if(null==o)return void(t._isValid=!1)}if(!r&&null!=o)return void(t._isValid=!1);if(A[4]){if(!me.exec(A[4]))return void(t._isValid=!1);a="Z"}t._f=i+(o||"")+(a||""),_e(t)}else t._isValid=!1}var be=/^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/;function Qe(t){var e=parseInt(t,10);return e<=49?2e3+e:e<=999?1900+e:e}var we={UT:0,GMT:0,EDT:-240,EST:-300,CDT:-300,CST:-360,MDT:-360,MST:-420,PDT:-420,PST:-480};function xe(t){var e=be.exec(t._i.replace(/\([^)]*\)|[\n\t]/g," ").replace(/(\s\s+)/g," ").replace(/^\s\s*/,"").replace(/\s\s*$/,""));if(e){var n=function(t,e,n,r,i,o){var a=[Qe(t),Ft.indexOf(e),parseInt(n,10),parseInt(r,10),parseInt(i,10)];return o&&a.push(parseInt(o,10)),a}(e[4],e[3],e[2],e[5],e[6],e[7]);if(!function(t,e,n){if(t){var r=Vt.indexOf(t),i=new Date(e[0],e[1],e[2]).getDay();if(r!==i)return d(n).weekdayMismatch=!0,n._isValid=!1,!1}return!0}(e[1],n,t))return;t._a=n,t._tzm=function(t,e,n){if(t)return we[t];if(e)return 0;var r=parseInt(n,10),i=r%100,o=(r-i)/100;return 60*o+i}(e[8],e[9],e[10]),t._d=jt.apply(null,t._a),t._d.setUTCMinutes(t._d.getUTCMinutes()-t._tzm),d(t).rfc2822=!0}else t._isValid=!1}function _e(t){if(t._f!==i.ISO_8601)if(t._f!==i.RFC_2822){t._a=[],d(t).empty=!0;var e,n,r,o,a,s=""+t._i,A=s.length,l=0;for(r=z(t._f,t._locale).match(G)||[],e=0;e<r.length;e++)o=r[e],(n=(s.match(ut(o,t))||[])[0])&&((a=s.substr(0,s.indexOf(n))).length>0&&d(t).unusedInput.push(a),s=s.slice(s.indexOf(n)+n.length),l+=n.length),Y[o]?(n?d(t).empty=!1:d(t).unusedTokens.push(o),pt(o,n,t)):t._strict&&!n&&d(t).unusedTokens.push(o);d(t).charsLeftOver=A-l,s.length>0&&d(t).unusedInput.push(s),t._a[yt]<=12&&!0===d(t).bigHour&&t._a[yt]>0&&(d(t).bigHour=void 0),d(t).parsedDateParts=t._a.slice(0),d(t).meridiem=t._meridiem,t._a[yt]=function(t,e,n){var r;return null==n?e:null!=t.meridiemHour?t.meridiemHour(e,n):null!=t.isPM?((r=t.isPM(n))&&e<12&&(e+=12),r||12!==e||(e=0),e):e}(t._locale,t._a[yt],t._meridiem),pe(t),de(t)}else xe(t);else Be(t)}function De(t){var e=t._i,n=t._f;return t._locale=t._locale||he(t._l),null===e||void 0===n&&""===e?p({nullInput:!0}):("string"==typeof e&&(t._i=e=t._locale.preparse(e)),E(e)?new y(de(e)):(l(e)?t._d=e:o(n)?function(t){var e,n,r,i,o;if(0===t._f.length)return d(t).invalidFormat=!0,void(t._d=new Date(NaN));for(i=0;i<t._f.length;i++)o=0,e=C({},t),null!=t._useUTC&&(e._useUTC=t._useUTC),e._f=t._f[i],_e(e),f(e)&&(o+=d(e).charsLeftOver,o+=10*d(e).unusedTokens.length,d(e).score=o,(null==r||o<r)&&(r=o,n=e));g(t,n||e)}(t):n?_e(t):function(t){var e=t._i;s(e)?t._d=new Date(i.now()):l(e)?t._d=new Date(e.valueOf()):"string"==typeof e?function(t){var e=ve.exec(t._i);null===e?(Be(t),!1===t._isValid&&(delete t._isValid,xe(t),!1===t._isValid&&(delete t._isValid,i.createFromInputFallback(t)))):t._d=new Date(+e[1])}(t):o(e)?(t._a=c(e.slice(0),function(t){return parseInt(t,10)}),pe(t)):a(e)?function(t){if(!t._d){var e=F(t._i);t._a=c([e.year,e.month,e.day||e.date,e.hour,e.minute,e.second,e.millisecond],function(t){return t&&parseInt(t,10)}),pe(t)}}(t):A(e)?t._d=new Date(e):i.createFromInputFallback(t)}(t),f(t)||(t._d=null),t))}function Me(t,e,n,r,i){var s,A={};return!0!==n&&!1!==n||(r=n,n=void 0),(a(t)&&function(t){if(Object.getOwnPropertyNames)return 0===Object.getOwnPropertyNames(t).length;var e;for(e in t)if(t.hasOwnProperty(e))return!1;return!0}(t)||o(t)&&0===t.length)&&(t=void 0),A._isAMomentObject=!0,A._useUTC=A._isUTC=i,A._l=n,A._i=t,A._f=e,A._strict=r,(s=new y(de(De(A))))._nextDay&&(s.add(1,"d"),s._nextDay=void 0),s}function ke(t,e,n,r){return Me(t,e,n,r,!1)}i.createFromInputFallback=w("value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are discouraged and will be removed in an upcoming major release. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.",function(t){t._d=new Date(t._i+(t._useUTC?" UTC":""))}),i.ISO_8601=function(){},i.RFC_2822=function(){};var Se=w("moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var t=ke.apply(null,arguments);return this.isValid()&&t.isValid()?t<this?this:t:p()}),Le=w("moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var t=ke.apply(null,arguments);return this.isValid()&&t.isValid()?t>this?this:t:p()});function Re(t,e){var n,r;if(1===e.length&&o(e[0])&&(e=e[0]),!e.length)return ke();for(n=e[0],r=1;r<e.length;++r)e[r].isValid()&&!e[r][t](n)||(n=e[r]);return n}var Ne=["year","quarter","month","week","day","hour","minute","second","millisecond"];function Fe(t){var e=F(t),n=e.year||0,r=e.quarter||0,i=e.month||0,o=e.week||e.isoWeek||0,a=e.day||0,s=e.hour||0,A=e.minute||0,l=e.second||0,c=e.millisecond||0;this._isValid=function(t){for(var e in t)if(-1===_t.call(Ne,e)||null!=t[e]&&isNaN(t[e]))return!1;for(var n=!1,r=0;r<Ne.length;++r)if(t[Ne[r]]){if(n)return!1;parseFloat(t[Ne[r]])!==B(t[Ne[r]])&&(n=!0)}return!0}(e),this._milliseconds=+c+1e3*l+6e4*A+1e3*s*60*60,this._days=+a+7*o,this._months=+i+3*r+12*n,this._data={},this._locale=he(),this._bubble()}function Te(t){return t instanceof Fe}function Ue(t){return t<0?-1*Math.round(-1*t):Math.round(t)}function Oe(t,e){P(t,0,0,function(){var t=this.utcOffset(),n="+";return t<0&&(t=-t,n="-"),n+O(~~(t/60),2)+e+O(~~t%60,2)})}Oe("Z",":"),Oe("ZZ",""),ct("Z",st),ct("ZZ",st),dt(["Z","ZZ"],function(t,e,n){n._useUTC=!0,n._tzm=He(st,t)});var Ge=/([\+\-]|\d\d)/gi;function He(t,e){var n=(e||"").match(t);if(null===n)return null;var r=n[n.length-1]||[],i=(r+"").match(Ge)||["-",0,0],o=60*i[1]+B(i[2]);return 0===o?0:"+"===i[0]?o:-o}function je(t,e){var n,r;return e._isUTC?(n=e.clone(),r=(E(t)||l(t)?t.valueOf():ke(t).valueOf())-n.valueOf(),n._d.setTime(n._d.valueOf()+r),i.updateOffset(n,!1),n):ke(t).local()}function Ye(t){return 15*-Math.round(t._d.getTimezoneOffset()/15)}function Pe(){return!!this.isValid()&&this._isUTC&&0===this._offset}i.updateOffset=function(){};var Ke=/^(\-|\+)?(?:(\d*)[. ])?(\d+)\:(\d+)(?:\:(\d+)(\.\d*)?)?$/,ze=/^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;function Je(t,e){var n,r,i,o,a,s,l=t,c=null;return Te(t)?l={ms:t._milliseconds,d:t._days,M:t._months}:A(t)?(l={},e?l[e]=t:l.milliseconds=t):(c=Ke.exec(t))?(n="-"===c[1]?-1:1,l={y:0,d:B(c[mt])*n,h:B(c[yt])*n,m:B(c[Et])*n,s:B(c[vt])*n,ms:B(Ue(1e3*c[Bt]))*n}):(c=ze.exec(t))?(n="-"===c[1]?-1:1,l={y:qe(c[2],n),M:qe(c[3],n),w:qe(c[4],n),d:qe(c[5],n),h:qe(c[6],n),m:qe(c[7],n),s:qe(c[8],n)}):null==l?l={}:"object"==typeof l&&("from"in l||"to"in l)&&(o=ke(l.from),a=ke(l.to),i=o.isValid()&&a.isValid()?(a=je(a,o),o.isBefore(a)?s=Ve(o,a):((s=Ve(a,o)).milliseconds=-s.milliseconds,s.months=-s.months),s):{milliseconds:0,months:0},(l={}).ms=i.milliseconds,l.M=i.months),r=new Fe(l),Te(t)&&u(t,"_locale")&&(r._locale=t._locale),r}function qe(t,e){var n=t&&parseFloat(t.replace(",","."));return(isNaN(n)?0:n)*e}function Ve(t,e){var n={};return n.months=e.month()-t.month()+12*(e.year()-t.year()),t.clone().add(n.months,"M").isAfter(e)&&--n.months,n.milliseconds=+e-+t.clone().add(n.months,"M"),n}function We(t,e){return function(n,r){var i;return null===r||isNaN(+r)||(D(e,"moment()."+e+"(period, number) is deprecated. Please use moment()."+e+"(number, period). See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info."),i=n,n=r,r=i),Xe(this,Je(n="string"==typeof n?+n:n,r),t),this}}function Xe(t,e,n,r){var o=e._milliseconds,a=Ue(e._days),s=Ue(e._months);t.isValid()&&(r=null==r||r,s&&Tt(t,kt(t,"Month")+s*n),a&&St(t,"Date",kt(t,"Date")+a*n),o&&t._d.setTime(t._d.valueOf()+o*n),r&&i.updateOffset(t,a||s))}Je.fn=Fe.prototype,Je.invalid=function(){return Je(NaN)};var Ze=We(1,"add"),$e=We(-1,"subtract");function tn(t,e){var n,r,i=12*(e.year()-t.year())+(e.month()-t.month()),o=t.clone().add(i,"months");return e-o<0?(n=t.clone().add(i-1,"months"),r=(e-o)/(o-n)):(n=t.clone().add(i+1,"months"),r=(e-o)/(n-o)),-(i+r)||0}function en(t){var e;return void 0===t?this._locale._abbr:(null!=(e=he(t))&&(this._locale=e),this)}i.defaultFormat="YYYY-MM-DDTHH:mm:ssZ",i.defaultFormatUtc="YYYY-MM-DDTHH:mm:ss[Z]";var nn=w("moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.",function(t){return void 0===t?this.localeData():this.locale(t)});function rn(){return this._locale}var on=1e3,an=60*on,sn=60*an,An=3506328*sn;function ln(t,e){return(t%e+e)%e}function cn(t,e,n){return t<100&&t>=0?new Date(t+400,e,n)-An:new Date(t,e,n).valueOf()}function un(t,e,n){return t<100&&t>=0?Date.UTC(t+400,e,n)-An:Date.UTC(t,e,n)}function gn(t,e){P(0,[t,t.length],0,e)}function hn(t,e,n,r,i){var o;return null==t?Kt(this,r,i).year:(o=zt(t,r,i),e>o&&(e=o),function(t,e,n,r,i){var o=Pt(t,e,n,r,i),a=jt(o.year,0,o.dayOfYear);return this.year(a.getUTCFullYear()),this.month(a.getUTCMonth()),this.date(a.getUTCDate()),this}.call(this,t,e,n,r,i))}P(0,["gg",2],0,function(){return this.weekYear()%100}),P(0,["GG",2],0,function(){return this.isoWeekYear()%100}),gn("gggg","weekYear"),gn("ggggg","weekYear"),gn("GGGG","isoWeekYear"),gn("GGGGG","isoWeekYear"),R("weekYear","gg"),R("isoWeekYear","GG"),U("weekYear",1),U("isoWeekYear",1),ct("G",ot),ct("g",ot),ct("GG",Z,q),ct("gg",Z,q),ct("GGGG",nt,W),ct("gggg",nt,W),ct("GGGGG",rt,X),ct("ggggg",rt,X),ft(["gggg","ggggg","GGGG","GGGGG"],function(t,e,n,r){e[r.substr(0,2)]=B(t)}),ft(["gg","GG"],function(t,e,n,r){e[r]=i.parseTwoDigitYear(t)}),P("Q",0,"Qo","quarter"),R("quarter","Q"),U("quarter",7),ct("Q",J),dt("Q",function(t,e){e[Ct]=3*(B(t)-1)}),P("D",["DD",2],"Do","date"),R("date","D"),U("date",9),ct("D",Z),ct("DD",Z,q),ct("Do",function(t,e){return t?e._dayOfMonthOrdinalParse||e._ordinalParse:e._dayOfMonthOrdinalParseLenient}),dt(["D","DD"],mt),dt("Do",function(t,e){e[mt]=B(t.match(Z)[0])});var dn=Mt("Date",!0);P("DDD",["DDDD",3],"DDDo","dayOfYear"),R("dayOfYear","DDD"),U("dayOfYear",4),ct("DDD",et),ct("DDDD",V),dt(["DDD","DDDD"],function(t,e,n){n._dayOfYear=B(t)}),P("m",["mm",2],0,"minute"),R("minute","m"),U("minute",14),ct("m",Z),ct("mm",Z,q),dt(["m","mm"],Et);var fn=Mt("Minutes",!1);P("s",["ss",2],0,"second"),R("second","s"),U("second",15),ct("s",Z),ct("ss",Z,q),dt(["s","ss"],vt);var pn,In=Mt("Seconds",!1);for(P("S",0,0,function(){return~~(this.millisecond()/100)}),P(0,["SS",2],0,function(){return~~(this.millisecond()/10)}),P(0,["SSS",3],0,"millisecond"),P(0,["SSSS",4],0,function(){return 10*this.millisecond()}),P(0,["SSSSS",5],0,function(){return 100*this.millisecond()}),P(0,["SSSSSS",6],0,function(){return 1e3*this.millisecond()}),P(0,["SSSSSSS",7],0,function(){return 1e4*this.millisecond()}),P(0,["SSSSSSSS",8],0,function(){return 1e5*this.millisecond()}),P(0,["SSSSSSSSS",9],0,function(){return 1e6*this.millisecond()}),R("millisecond","ms"),U("millisecond",16),ct("S",et,J),ct("SS",et,q),ct("SSS",et,V),pn="SSSS";pn.length<=9;pn+="S")ct(pn,it);function Cn(t,e){e[Bt]=B(1e3*("0."+t))}for(pn="S";pn.length<=9;pn+="S")dt(pn,Cn);var mn=Mt("Milliseconds",!1);P("z",0,0,"zoneAbbr"),P("zz",0,0,"zoneName");var yn=y.prototype;function En(t){return t}yn.add=Ze,yn.calendar=function(t,e){var n=t||ke(),r=je(n,this).startOf("day"),o=i.calendarFormat(this,r)||"sameElse",a=e&&(M(e[o])?e[o].call(this,n):e[o]);return this.format(a||this.localeData().calendar(o,this,ke(n)))},yn.clone=function(){return new y(this)},yn.diff=function(t,e,n){var r,i,o;if(!this.isValid())return NaN;if(!(r=je(t,this)).isValid())return NaN;switch(i=6e4*(r.utcOffset()-this.utcOffset()),e=N(e)){case"year":o=tn(this,r)/12;break;case"month":o=tn(this,r);break;case"quarter":o=tn(this,r)/3;break;case"second":o=(this-r)/1e3;break;case"minute":o=(this-r)/6e4;break;case"hour":o=(this-r)/36e5;break;case"day":o=(this-r-i)/864e5;break;case"week":o=(this-r-i)/6048e5;break;default:o=this-r}return n?o:v(o)},yn.endOf=function(t){var e;if(void 0===(t=N(t))||"millisecond"===t||!this.isValid())return this;var n=this._isUTC?un:cn;switch(t){case"year":e=n(this.year()+1,0,1)-1;break;case"quarter":e=n(this.year(),this.month()-this.month()%3+3,1)-1;break;case"month":e=n(this.year(),this.month()+1,1)-1;break;case"week":e=n(this.year(),this.month(),this.date()-this.weekday()+7)-1;break;case"isoWeek":e=n(this.year(),this.month(),this.date()-(this.isoWeekday()-1)+7)-1;break;case"day":case"date":e=n(this.year(),this.month(),this.date()+1)-1;break;case"hour":e=this._d.valueOf(),e+=sn-ln(e+(this._isUTC?0:this.utcOffset()*an),sn)-1;break;case"minute":e=this._d.valueOf(),e+=an-ln(e,an)-1;break;case"second":e=this._d.valueOf(),e+=on-ln(e,on)-1}return this._d.setTime(e),i.updateOffset(this,!0),this},yn.format=function(t){t||(t=this.isUtc()?i.defaultFormatUtc:i.defaultFormat);var e=K(this,t);return this.localeData().postformat(e)},yn.from=function(t,e){return this.isValid()&&(E(t)&&t.isValid()||ke(t).isValid())?Je({to:this,from:t}).locale(this.locale()).humanize(!e):this.localeData().invalidDate()},yn.fromNow=function(t){return this.from(ke(),t)},yn.to=function(t,e){return this.isValid()&&(E(t)&&t.isValid()||ke(t).isValid())?Je({from:this,to:t}).locale(this.locale()).humanize(!e):this.localeData().invalidDate()},yn.toNow=function(t){return this.to(ke(),t)},yn.get=function(t){return M(this[t=N(t)])?this[t]():this},yn.invalidAt=function(){return d(this).overflow},yn.isAfter=function(t,e){var n=E(t)?t:ke(t);return!(!this.isValid()||!n.isValid())&&("millisecond"===(e=N(e)||"millisecond")?this.valueOf()>n.valueOf():n.valueOf()<this.clone().startOf(e).valueOf())},yn.isBefore=function(t,e){var n=E(t)?t:ke(t);return!(!this.isValid()||!n.isValid())&&("millisecond"===(e=N(e)||"millisecond")?this.valueOf()<n.valueOf():this.clone().endOf(e).valueOf()<n.valueOf())},yn.isBetween=function(t,e,n,r){var i=E(t)?t:ke(t),o=E(e)?e:ke(e);return!!(this.isValid()&&i.isValid()&&o.isValid())&&(("("===(r=r||"()")[0]?this.isAfter(i,n):!this.isBefore(i,n))&&(")"===r[1]?this.isBefore(o,n):!this.isAfter(o,n)))},yn.isSame=function(t,e){var n,r=E(t)?t:ke(t);return!(!this.isValid()||!r.isValid())&&("millisecond"===(e=N(e)||"millisecond")?this.valueOf()===r.valueOf():(n=r.valueOf(),this.clone().startOf(e).valueOf()<=n&&n<=this.clone().endOf(e).valueOf()))},yn.isSameOrAfter=function(t,e){return this.isSame(t,e)||this.isAfter(t,e)},yn.isSameOrBefore=function(t,e){return this.isSame(t,e)||this.isBefore(t,e)},yn.isValid=function(){return f(this)},yn.lang=nn,yn.locale=en,yn.localeData=rn,yn.max=Le,yn.min=Se,yn.parsingFlags=function(){return g({},d(this))},yn.set=function(t,e){if("object"==typeof t)for(var n=function(t){var e=[];for(var n in t)e.push({unit:n,priority:T[n]});return e.sort(function(t,e){return t.priority-e.priority}),e}(t=F(t)),r=0;r<n.length;r++)this[n[r].unit](t[n[r].unit]);else if(M(this[t=N(t)]))return this[t](e);return this},yn.startOf=function(t){var e;if(void 0===(t=N(t))||"millisecond"===t||!this.isValid())return this;var n=this._isUTC?un:cn;switch(t){case"year":e=n(this.year(),0,1);break;case"quarter":e=n(this.year(),this.month()-this.month()%3,1);break;case"month":e=n(this.year(),this.month(),1);break;case"week":e=n(this.year(),this.month(),this.date()-this.weekday());break;case"isoWeek":e=n(this.year(),this.month(),this.date()-(this.isoWeekday()-1));break;case"day":case"date":e=n(this.year(),this.month(),this.date());break;case"hour":e=this._d.valueOf(),e-=ln(e+(this._isUTC?0:this.utcOffset()*an),sn);break;case"minute":e=this._d.valueOf(),e-=ln(e,an);break;case"second":e=this._d.valueOf(),e-=ln(e,on)}return this._d.setTime(e),i.updateOffset(this,!0),this},yn.subtract=$e,yn.toArray=function(){var t=this;return[t.year(),t.month(),t.date(),t.hour(),t.minute(),t.second(),t.millisecond()]},yn.toObject=function(){var t=this;return{years:t.year(),months:t.month(),date:t.date(),hours:t.hours(),minutes:t.minutes(),seconds:t.seconds(),milliseconds:t.milliseconds()}},yn.toDate=function(){return new Date(this.valueOf())},yn.toISOString=function(t){if(!this.isValid())return null;var e=!0!==t,n=e?this.clone().utc():this;return n.year()<0||n.year()>9999?K(n,e?"YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYYYY-MM-DD[T]HH:mm:ss.SSSZ"):M(Date.prototype.toISOString)?e?this.toDate().toISOString():new Date(this.valueOf()+60*this.utcOffset()*1e3).toISOString().replace("Z",K(n,"Z")):K(n,e?"YYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYY-MM-DD[T]HH:mm:ss.SSSZ")},yn.inspect=function(){if(!this.isValid())return"moment.invalid(/* "+this._i+" */)";var t="moment",e="";this.isLocal()||(t=0===this.utcOffset()?"moment.utc":"moment.parseZone",e="Z");var n="["+t+'("]',r=0<=this.year()&&this.year()<=9999?"YYYY":"YYYYYY",i=e+'[")]';return this.format(n+r+"-MM-DD[T]HH:mm:ss.SSS"+i)},yn.toJSON=function(){return this.isValid()?this.toISOString():null},yn.toString=function(){return this.clone().locale("en").format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ")},yn.unix=function(){return Math.floor(this.valueOf()/1e3)},yn.valueOf=function(){return this._d.valueOf()-6e4*(this._offset||0)},yn.creationData=function(){return{input:this._i,format:this._f,locale:this._locale,isUTC:this._isUTC,strict:this._strict}},yn.year=Dt,yn.isLeapYear=function(){return xt(this.year())},yn.weekYear=function(t){return hn.call(this,t,this.week(),this.weekday(),this.localeData()._week.dow,this.localeData()._week.doy)},yn.isoWeekYear=function(t){return hn.call(this,t,this.isoWeek(),this.isoWeekday(),1,4)},yn.quarter=yn.quarters=function(t){return null==t?Math.ceil((this.month()+1)/3):this.month(3*(t-1)+this.month()%3)},yn.month=Ut,yn.daysInMonth=function(){return Lt(this.year(),this.month())},yn.week=yn.weeks=function(t){var e=this.localeData().week(this);return null==t?e:this.add(7*(t-e),"d")},yn.isoWeek=yn.isoWeeks=function(t){var e=Kt(this,1,4).week;return null==t?e:this.add(7*(t-e),"d")},yn.weeksInYear=function(){var t=this.localeData()._week;return zt(this.year(),t.dow,t.doy)},yn.isoWeeksInYear=function(){return zt(this.year(),1,4)},yn.date=dn,yn.day=yn.days=function(t){if(!this.isValid())return null!=t?this:NaN;var e=this._isUTC?this._d.getUTCDay():this._d.getDay();return null!=t?(t=function(t,e){return"string"!=typeof t?t:isNaN(t)?"number"==typeof(t=e.weekdaysParse(t))?t:null:parseInt(t,10)}(t,this.localeData()),this.add(t-e,"d")):e},yn.weekday=function(t){if(!this.isValid())return null!=t?this:NaN;var e=(this.day()+7-this.localeData()._week.dow)%7;return null==t?e:this.add(t-e,"d")},yn.isoWeekday=function(t){if(!this.isValid())return null!=t?this:NaN;if(null!=t){var e=function(t,e){return"string"==typeof t?e.weekdaysParse(t)%7||7:isNaN(t)?null:t}(t,this.localeData());return this.day(this.day()%7?e:e-7)}return this.day()||7},yn.dayOfYear=function(t){var e=Math.round((this.clone().startOf("day")-this.clone().startOf("year"))/864e5)+1;return null==t?e:this.add(t-e,"d")},yn.hour=yn.hours=oe,yn.minute=yn.minutes=fn,yn.second=yn.seconds=In,yn.millisecond=yn.milliseconds=mn,yn.utcOffset=function(t,e,n){var r,o=this._offset||0;if(!this.isValid())return null!=t?this:NaN;if(null!=t){if("string"==typeof t){if(null===(t=He(st,t)))return this}else Math.abs(t)<16&&!n&&(t*=60);return!this._isUTC&&e&&(r=Ye(this)),this._offset=t,this._isUTC=!0,null!=r&&this.add(r,"m"),o!==t&&(!e||this._changeInProgress?Xe(this,Je(t-o,"m"),1,!1):this._changeInProgress||(this._changeInProgress=!0,i.updateOffset(this,!0),this._changeInProgress=null)),this}return this._isUTC?o:Ye(this)},yn.utc=function(t){return this.utcOffset(0,t)},yn.local=function(t){return this._isUTC&&(this.utcOffset(0,t),this._isUTC=!1,t&&this.subtract(Ye(this),"m")),this},yn.parseZone=function(){if(null!=this._tzm)this.utcOffset(this._tzm,!1,!0);else if("string"==typeof this._i){var t=He(at,this._i);null!=t?this.utcOffset(t):this.utcOffset(0,!0)}return this},yn.hasAlignedHourOffset=function(t){return!!this.isValid()&&(t=t?ke(t).utcOffset():0,(this.utcOffset()-t)%60==0)},yn.isDST=function(){return this.utcOffset()>this.clone().month(0).utcOffset()||this.utcOffset()>this.clone().month(5).utcOffset()},yn.isLocal=function(){return!!this.isValid()&&!this._isUTC},yn.isUtcOffset=function(){return!!this.isValid()&&this._isUTC},yn.isUtc=Pe,yn.isUTC=Pe,yn.zoneAbbr=function(){return this._isUTC?"UTC":""},yn.zoneName=function(){return this._isUTC?"Coordinated Universal Time":""},yn.dates=w("dates accessor is deprecated. Use date instead.",dn),yn.months=w("months accessor is deprecated. Use month instead",Ut),yn.years=w("years accessor is deprecated. Use year instead",Dt),yn.zone=w("moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/",function(t,e){return null!=t?("string"!=typeof t&&(t=-t),this.utcOffset(t,e),this):-this.utcOffset()}),yn.isDSTShifted=w("isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information",function(){if(!s(this._isDSTShifted))return this._isDSTShifted;var t={};if(C(t,this),(t=De(t))._a){var e=t._isUTC?h(t._a):ke(t._a);this._isDSTShifted=this.isValid()&&b(t._a,e.toArray())>0}else this._isDSTShifted=!1;return this._isDSTShifted});var vn=S.prototype;function Bn(t,e,n,r){var i=he(),o=h().set(r,e);return i[n](o,t)}function bn(t,e,n){if(A(t)&&(e=t,t=void 0),t=t||"",null!=e)return Bn(t,e,n,"month");var r,i=[];for(r=0;r<12;r++)i[r]=Bn(t,r,n,"month");return i}function Qn(t,e,n,r){"boolean"==typeof t?(A(e)&&(n=e,e=void 0),e=e||""):(n=e=t,t=!1,A(e)&&(n=e,e=void 0),e=e||"");var i,o=he(),a=t?o._week.dow:0;if(null!=n)return Bn(e,(n+a)%7,r,"day");var s=[];for(i=0;i<7;i++)s[i]=Bn(e,(i+a)%7,r,"day");return s}vn.calendar=function(t,e,n){var r=this._calendar[t]||this._calendar.sameElse;return M(r)?r.call(e,n):r},vn.longDateFormat=function(t){var e=this._longDateFormat[t],n=this._longDateFormat[t.toUpperCase()];return e||!n?e:(this._longDateFormat[t]=n.replace(/MMMM|MM|DD|dddd/g,function(t){return t.slice(1)}),this._longDateFormat[t])},vn.invalidDate=function(){return this._invalidDate},vn.ordinal=function(t){return this._ordinal.replace("%d",t)},vn.preparse=En,vn.postformat=En,vn.relativeTime=function(t,e,n,r){var i=this._relativeTime[n];return M(i)?i(t,e,n,r):i.replace(/%d/i,t)},vn.pastFuture=function(t,e){var n=this._relativeTime[t>0?"future":"past"];return M(n)?n(e):n.replace(/%s/i,e)},vn.set=function(t){var e,n;for(n in t)M(e=t[n])?this[n]=e:this["_"+n]=e;this._config=t,this._dayOfMonthOrdinalParseLenient=new RegExp((this._dayOfMonthOrdinalParse.source||this._ordinalParse.source)+"|"+/\d{1,2}/.source)},vn.months=function(t,e){return t?o(this._months)?this._months[t.month()]:this._months[(this._months.isFormat||Rt).test(e)?"format":"standalone"][t.month()]:o(this._months)?this._months:this._months.standalone},vn.monthsShort=function(t,e){return t?o(this._monthsShort)?this._monthsShort[t.month()]:this._monthsShort[Rt.test(e)?"format":"standalone"][t.month()]:o(this._monthsShort)?this._monthsShort:this._monthsShort.standalone},vn.monthsParse=function(t,e,n){var r,i,o;if(this._monthsParseExact)return function(t,e,n){var r,i,o,a=t.toLocaleLowerCase();if(!this._monthsParse)for(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[],r=0;r<12;++r)o=h([2e3,r]),this._shortMonthsParse[r]=this.monthsShort(o,"").toLocaleLowerCase(),this._longMonthsParse[r]=this.months(o,"").toLocaleLowerCase();return n?"MMM"===e?-1!==(i=_t.call(this._shortMonthsParse,a))?i:null:-1!==(i=_t.call(this._longMonthsParse,a))?i:null:"MMM"===e?-1!==(i=_t.call(this._shortMonthsParse,a))?i:-1!==(i=_t.call(this._longMonthsParse,a))?i:null:-1!==(i=_t.call(this._longMonthsParse,a))?i:-1!==(i=_t.call(this._shortMonthsParse,a))?i:null}.call(this,t,e,n);for(this._monthsParse||(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[]),r=0;r<12;r++){if(i=h([2e3,r]),n&&!this._longMonthsParse[r]&&(this._longMonthsParse[r]=new RegExp("^"+this.months(i,"").replace(".","")+"$","i"),this._shortMonthsParse[r]=new RegExp("^"+this.monthsShort(i,"").replace(".","")+"$","i")),n||this._monthsParse[r]||(o="^"+this.months(i,"")+"|^"+this.monthsShort(i,""),this._monthsParse[r]=new RegExp(o.replace(".",""),"i")),n&&"MMMM"===e&&this._longMonthsParse[r].test(t))return r;if(n&&"MMM"===e&&this._shortMonthsParse[r].test(t))return r;if(!n&&this._monthsParse[r].test(t))return r}},vn.monthsRegex=function(t){return this._monthsParseExact?(u(this,"_monthsRegex")||Ht.call(this),t?this._monthsStrictRegex:this._monthsRegex):(u(this,"_monthsRegex")||(this._monthsRegex=Gt),this._monthsStrictRegex&&t?this._monthsStrictRegex:this._monthsRegex)},vn.monthsShortRegex=function(t){return this._monthsParseExact?(u(this,"_monthsRegex")||Ht.call(this),t?this._monthsShortStrictRegex:this._monthsShortRegex):(u(this,"_monthsShortRegex")||(this._monthsShortRegex=Ot),this._monthsShortStrictRegex&&t?this._monthsShortStrictRegex:this._monthsShortRegex)},vn.week=function(t){return Kt(t,this._week.dow,this._week.doy).week},vn.firstDayOfYear=function(){return this._week.doy},vn.firstDayOfWeek=function(){return this._week.dow},vn.weekdays=function(t,e){var n=o(this._weekdays)?this._weekdays:this._weekdays[t&&!0!==t&&this._weekdays.isFormat.test(e)?"format":"standalone"];return!0===t?Jt(n,this._week.dow):t?n[t.day()]:n},vn.weekdaysMin=function(t){return!0===t?Jt(this._weekdaysMin,this._week.dow):t?this._weekdaysMin[t.day()]:this._weekdaysMin},vn.weekdaysShort=function(t){return!0===t?Jt(this._weekdaysShort,this._week.dow):t?this._weekdaysShort[t.day()]:this._weekdaysShort},vn.weekdaysParse=function(t,e,n){var r,i,o;if(this._weekdaysParseExact)return function(t,e,n){var r,i,o,a=t.toLocaleLowerCase();if(!this._weekdaysParse)for(this._weekdaysParse=[],this._shortWeekdaysParse=[],this._minWeekdaysParse=[],r=0;r<7;++r)o=h([2e3,1]).day(r),this._minWeekdaysParse[r]=this.weekdaysMin(o,"").toLocaleLowerCase(),this._shortWeekdaysParse[r]=this.weekdaysShort(o,"").toLocaleLowerCase(),this._weekdaysParse[r]=this.weekdays(o,"").toLocaleLowerCase();return n?"dddd"===e?-1!==(i=_t.call(this._weekdaysParse,a))?i:null:"ddd"===e?-1!==(i=_t.call(this._shortWeekdaysParse,a))?i:null:-1!==(i=_t.call(this._minWeekdaysParse,a))?i:null:"dddd"===e?-1!==(i=_t.call(this._weekdaysParse,a))?i:-1!==(i=_t.call(this._shortWeekdaysParse,a))?i:-1!==(i=_t.call(this._minWeekdaysParse,a))?i:null:"ddd"===e?-1!==(i=_t.call(this._shortWeekdaysParse,a))?i:-1!==(i=_t.call(this._weekdaysParse,a))?i:-1!==(i=_t.call(this._minWeekdaysParse,a))?i:null:-1!==(i=_t.call(this._minWeekdaysParse,a))?i:-1!==(i=_t.call(this._weekdaysParse,a))?i:-1!==(i=_t.call(this._shortWeekdaysParse,a))?i:null}.call(this,t,e,n);for(this._weekdaysParse||(this._weekdaysParse=[],this._minWeekdaysParse=[],this._shortWeekdaysParse=[],this._fullWeekdaysParse=[]),r=0;r<7;r++){if(i=h([2e3,1]).day(r),n&&!this._fullWeekdaysParse[r]&&(this._fullWeekdaysParse[r]=new RegExp("^"+this.weekdays(i,"").replace(".","\\.?")+"$","i"),this._shortWeekdaysParse[r]=new RegExp("^"+this.weekdaysShort(i,"").replace(".","\\.?")+"$","i"),this._minWeekdaysParse[r]=new RegExp("^"+this.weekdaysMin(i,"").replace(".","\\.?")+"$","i")),this._weekdaysParse[r]||(o="^"+this.weekdays(i,"")+"|^"+this.weekdaysShort(i,"")+"|^"+this.weekdaysMin(i,""),this._weekdaysParse[r]=new RegExp(o.replace(".",""),"i")),n&&"dddd"===e&&this._fullWeekdaysParse[r].test(t))return r;if(n&&"ddd"===e&&this._shortWeekdaysParse[r].test(t))return r;if(n&&"dd"===e&&this._minWeekdaysParse[r].test(t))return r;if(!n&&this._weekdaysParse[r].test(t))return r}},vn.weekdaysRegex=function(t){return this._weekdaysParseExact?(u(this,"_weekdaysRegex")||te.call(this),t?this._weekdaysStrictRegex:this._weekdaysRegex):(u(this,"_weekdaysRegex")||(this._weekdaysRegex=Xt),this._weekdaysStrictRegex&&t?this._weekdaysStrictRegex:this._weekdaysRegex)},vn.weekdaysShortRegex=function(t){return this._weekdaysParseExact?(u(this,"_weekdaysRegex")||te.call(this),t?this._weekdaysShortStrictRegex:this._weekdaysShortRegex):(u(this,"_weekdaysShortRegex")||(this._weekdaysShortRegex=Zt),this._weekdaysShortStrictRegex&&t?this._weekdaysShortStrictRegex:this._weekdaysShortRegex)},vn.weekdaysMinRegex=function(t){return this._weekdaysParseExact?(u(this,"_weekdaysRegex")||te.call(this),t?this._weekdaysMinStrictRegex:this._weekdaysMinRegex):(u(this,"_weekdaysMinRegex")||(this._weekdaysMinRegex=$t),this._weekdaysMinStrictRegex&&t?this._weekdaysMinStrictRegex:this._weekdaysMinRegex)},vn.isPM=function(t){return"p"===(t+"").toLowerCase().charAt(0)},vn.meridiem=function(t,e,n){return t>11?n?"pm":"PM":n?"am":"AM"},ue("en",{dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(t){var e=t%10,n=1===B(t%100/10)?"th":1===e?"st":2===e?"nd":3===e?"rd":"th";return t+n}}),i.lang=w("moment.lang is deprecated. Use moment.locale instead.",ue),i.langData=w("moment.langData is deprecated. Use moment.localeData instead.",he);var wn=Math.abs;function xn(t,e,n,r){var i=Je(e,n);return t._milliseconds+=r*i._milliseconds,t._days+=r*i._days,t._months+=r*i._months,t._bubble()}function _n(t){return t<0?Math.floor(t):Math.ceil(t)}function Dn(t){return 4800*t/146097}function Mn(t){return 146097*t/4800}function kn(t){return function(){return this.as(t)}}var Sn=kn("ms"),Ln=kn("s"),Rn=kn("m"),Nn=kn("h"),Fn=kn("d"),Tn=kn("w"),Un=kn("M"),On=kn("Q"),Gn=kn("y");function Hn(t){return function(){return this.isValid()?this._data[t]:NaN}}var jn=Hn("milliseconds"),Yn=Hn("seconds"),Pn=Hn("minutes"),Kn=Hn("hours"),zn=Hn("days"),Jn=Hn("months"),qn=Hn("years"),Vn=Math.round,Wn={ss:44,s:45,m:45,h:22,d:26,M:11},Xn=Math.abs;function Zn(t){return(t>0)-(t<0)||+t}function $n(){if(!this.isValid())return this.localeData().invalidDate();var t,e,n=Xn(this._milliseconds)/1e3,r=Xn(this._days),i=Xn(this._months);t=v(n/60),e=v(t/60),n%=60,t%=60;var o=v(i/12),a=i%=12,s=r,A=e,l=t,c=n?n.toFixed(3).replace(/\.?0+$/,""):"",u=this.asSeconds();if(!u)return"P0D";var g=u<0?"-":"",h=Zn(this._months)!==Zn(u)?"-":"",d=Zn(this._days)!==Zn(u)?"-":"",f=Zn(this._milliseconds)!==Zn(u)?"-":"";return g+"P"+(o?h+o+"Y":"")+(a?h+a+"M":"")+(s?d+s+"D":"")+(A||l||c?"T":"")+(A?f+A+"H":"")+(l?f+l+"M":"")+(c?f+c+"S":"")}var tr=Fe.prototype;return tr.isValid=function(){return this._isValid},tr.abs=function(){var t=this._data;return this._milliseconds=wn(this._milliseconds),this._days=wn(this._days),this._months=wn(this._months),t.milliseconds=wn(t.milliseconds),t.seconds=wn(t.seconds),t.minutes=wn(t.minutes),t.hours=wn(t.hours),t.months=wn(t.months),t.years=wn(t.years),this},tr.add=function(t,e){return xn(this,t,e,1)},tr.subtract=function(t,e){return xn(this,t,e,-1)},tr.as=function(t){if(!this.isValid())return NaN;var e,n,r=this._milliseconds;if("month"===(t=N(t))||"quarter"===t||"year"===t)switch(e=this._days+r/864e5,n=this._months+Dn(e),t){case"month":return n;case"quarter":return n/3;case"year":return n/12}else switch(e=this._days+Math.round(Mn(this._months)),t){case"week":return e/7+r/6048e5;case"day":return e+r/864e5;case"hour":return 24*e+r/36e5;case"minute":return 1440*e+r/6e4;case"second":return 86400*e+r/1e3;case"millisecond":return Math.floor(864e5*e)+r;default:throw new Error("Unknown unit "+t)}},tr.asMilliseconds=Sn,tr.asSeconds=Ln,tr.asMinutes=Rn,tr.asHours=Nn,tr.asDays=Fn,tr.asWeeks=Tn,tr.asMonths=Un,tr.asQuarters=On,tr.asYears=Gn,tr.valueOf=function(){return this.isValid()?this._milliseconds+864e5*this._days+this._months%12*2592e6+31536e6*B(this._months/12):NaN},tr._bubble=function(){var t,e,n,r,i,o=this._milliseconds,a=this._days,s=this._months,A=this._data;return o>=0&&a>=0&&s>=0||o<=0&&a<=0&&s<=0||(o+=864e5*_n(Mn(s)+a),a=0,s=0),A.milliseconds=o%1e3,t=v(o/1e3),A.seconds=t%60,e=v(t/60),A.minutes=e%60,n=v(e/60),A.hours=n%24,a+=v(n/24),i=v(Dn(a)),s+=i,a-=_n(Mn(i)),r=v(s/12),s%=12,A.days=a,A.months=s,A.years=r,this},tr.clone=function(){return Je(this)},tr.get=function(t){return t=N(t),this.isValid()?this[t+"s"]():NaN},tr.milliseconds=jn,tr.seconds=Yn,tr.minutes=Pn,tr.hours=Kn,tr.days=zn,tr.weeks=function(){return v(this.days()/7)},tr.months=Jn,tr.years=qn,tr.humanize=function(t){if(!this.isValid())return this.localeData().invalidDate();var e=this.localeData(),n=function(t,e,n){var r=Je(t).abs(),i=Vn(r.as("s")),o=Vn(r.as("m")),a=Vn(r.as("h")),s=Vn(r.as("d")),A=Vn(r.as("M")),l=Vn(r.as("y")),c=i<=Wn.ss&&["s",i]||i<Wn.s&&["ss",i]||o<=1&&["m"]||o<Wn.m&&["mm",o]||a<=1&&["h"]||a<Wn.h&&["hh",a]||s<=1&&["d"]||s<Wn.d&&["dd",s]||A<=1&&["M"]||A<Wn.M&&["MM",A]||l<=1&&["y"]||["yy",l];return c[2]=e,c[3]=+t>0,c[4]=n,function(t,e,n,r,i){return i.relativeTime(e||1,!!n,t,r)}.apply(null,c)}(this,!t,e);return t&&(n=e.pastFuture(+this,n)),e.postformat(n)},tr.toISOString=$n,tr.toString=$n,tr.toJSON=$n,tr.locale=en,tr.localeData=rn,tr.toIsoString=w("toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)",$n),tr.lang=nn,P("X",0,0,"unix"),P("x",0,0,"valueOf"),ct("x",ot),ct("X",/[+-]?\d+(\.\d{1,3})?/),dt("X",function(t,e,n){n._d=new Date(1e3*parseFloat(t,10))}),dt("x",function(t,e,n){n._d=new Date(B(t))}),i.version="2.24.0",e=ke,i.fn=yn,i.min=function(){return Re("isBefore",[].slice.call(arguments,0))},i.max=function(){return Re("isAfter",[].slice.call(arguments,0))},i.now=function(){return Date.now?Date.now():+new Date},i.utc=h,i.unix=function(t){return ke(1e3*t)},i.months=function(t,e){return bn(t,e,"months")},i.isDate=l,i.locale=ue,i.invalid=p,i.duration=Je,i.isMoment=E,i.weekdays=function(t,e,n){return Qn(t,e,n,"weekdays")},i.parseZone=function(){return ke.apply(null,arguments).parseZone()},i.localeData=he,i.isDuration=Te,i.monthsShort=function(t,e){return bn(t,e,"monthsShort")},i.weekdaysMin=function(t,e,n){return Qn(t,e,n,"weekdaysMin")},i.defineLocale=ge,i.updateLocale=function(t,e){if(null!=e){var n,r,i=ae;null!=(r=ce(t))&&(i=r._config),e=k(i,e),(n=new S(e)).parentLocale=se[t],se[t]=n,ue(t)}else null!=se[t]&&(null!=se[t].parentLocale?se[t]=se[t].parentLocale:null!=se[t]&&delete se[t]);return se[t]},i.locales=function(){return x(se)},i.weekdaysShort=function(t,e,n){return Qn(t,e,n,"weekdaysShort")},i.normalizeUnits=N,i.relativeTimeRounding=function(t){return void 0===t?Vn:"function"==typeof t&&(Vn=t,!0)},i.relativeTimeThreshold=function(t,e){return void 0!==Wn[t]&&(void 0===e?Wn[t]:(Wn[t]=e,"s"===t&&(Wn.ss=e-1),!0))},i.calendarFormat=function(t,e){var n=t.diff(e,"days",!0);return n<-6?"sameElse":n<-1?"lastWeek":n<0?"lastDay":n<1?"sameDay":n<2?"nextDay":n<7?"nextWeek":"sameElse"},i.prototype=yn,i.HTML5_FMT={DATETIME_LOCAL:"YYYY-MM-DDTHH:mm",DATETIME_LOCAL_SECONDS:"YYYY-MM-DDTHH:mm:ss",DATETIME_LOCAL_MS:"YYYY-MM-DDTHH:mm:ss.SSS",DATE:"YYYY-MM-DD",TIME:"HH:mm",TIME_SECONDS:"HH:mm:ss",TIME_MS:"HH:mm:ss.SSS",WEEK:"GGGG-[W]WW",MONTH:"YYYY-MM"},i}()}).call(this,n(98)(t))},function(t,e,n){"use strict";n.r(e),function(t){n.d(e,"isNodeEnv",function(){return s}),n.d(e,"isFunction",function(){return A}),n.d(e,"isReactNodeEmpty",function(){return l}),n.d(e,"ensureElement",function(){return c}),n.d(e,"isReactElement",function(){return u}),n.d(e,"getDisplayName",function(){return g}),n.d(e,"isElementOfType",function(){return h}),n.d(e,"safeInvoke",function(){return d}),n.d(e,"safeInvokeOrValue",function(){return f}),n.d(e,"elementIsOrContains",function(){return p}),n.d(e,"arrayLengthCompare",function(){return I}),n.d(e,"approxEqual",function(){return C}),n.d(e,"clamp",function(){return m}),n.d(e,"countDecimalPlaces",function(){return y}),n.d(e,"throttleEvent",function(){return E}),n.d(e,"throttleReactEventCallback",function(){return v}),n.d(e,"throttle",function(){return B});var r=n(0),i=n(23),o=n(620);n.d(e,"arraysEqual",function(){return o.a}),n.d(e,"shallowCompareKeys",function(){return o.e}),n.d(e,"deepCompareKeys",function(){return o.b}),n.d(e,"getShallowUnequalKeyValues",function(){return o.d}),n.d(e,"getDeepUnequalKeyValues",function(){return o.c});var a=n(621);function s(e){return void 0!==t&&t.env&&"production"===e}function A(t){return"function"==typeof t}function l(t,e){return void 0===e&&(e=!1),null==t||""===t||!1===t||!e&&Array.isArray(t)&&(0===t.length||t.every(function(t){return l(t,!0)}))}function c(t,e){return void 0===e&&(e="span"),null==t||"boolean"==typeof t?void 0:"string"==typeof t?t.trim().length>0?r.createElement(e,{},t):void 0:"number"==typeof t||"symbol"==typeof t.type||Array.isArray(t)?r.createElement(e,{},t):u(t)?t:void 0}function u(t){return"object"==typeof t&&void 0!==t.type&&void 0!==t.props}function g(t){return t.displayName||t.name||"Unknown"}function h(t,e){return null!=t&&null!=t.type&&null!=t.type.displayName&&t.type.displayName===e.displayName}function d(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];if(A(t))return t.apply(void 0,e)}function f(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return A(t)?t.apply(void 0,e):t}function p(t,e){return t===e||t.contains(e)}function I(t,e){return void 0===t&&(t=[]),void 0===e&&(e=[]),t.length-e.length}function C(t,e,n){return void 0===n&&(n=1e-5),Math.abs(t-e)<=n}function m(t,e,n){if(null==t)return t;if(n<e)throw new Error(i.d);return Math.min(Math.max(t,e),n)}function y(t){if(!isFinite(t))return 0;for(var e=1,n=0;Math.round(t*e)/e!==t;)e*=10,n++;return n}function E(t,e,n){var r=b(function(e){t.dispatchEvent(new CustomEvent(n,e))});return t.addEventListener(e,r),r}function v(t,e){return void 0===e&&(e={}),b(t,function(t){e.preventDefault&&t.preventDefault()},function(t){return t.persist()})}function B(t){return b(t)}function b(t,e,n){var r=!1;return function(){for(var i=[],o=0;o<arguments.length;o++)i[o]=arguments[o];A(e)&&e.apply(void 0,i),r||(r=!0,A(n)&&n.apply(void 0,i),requestAnimationFrame(function(){t.apply(void 0,i),r=!1}))}}n.d(e,"safeInvokeMember",function(){return a.a})}.call(this,n(58))},function(t,e,n){"use strict";function r(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}Object.defineProperty(e,"__esModule",{value:!0}),r(n(765)),r(n(766)),r(n(767)),r(n(395)),r(n(850)),r(n(851))},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0});var r,i=n(5),o=function(){function t(t){this.sender=t}return t.prototype.connect=function(t,e){return r.connect(this,t,e)},t.prototype.disconnect=function(t,e){return r.disconnect(this,t,e)},t.prototype.emit=function(t){r.emit(this,t)},t}();e.Signal=o,function(t){t.disconnectBetween=function(t,e){r.disconnectBetween(t,e)},t.disconnectSender=function(t){r.disconnectSender(t)},t.disconnectReceiver=function(t){r.disconnectReceiver(t)},t.disconnectAll=function(t){r.disconnectAll(t)},t.clearData=function(t){r.disconnectAll(t)},t.getExceptionHandler=function(){return r.exceptionHandler},t.setExceptionHandler=function(t){var e=r.exceptionHandler;return r.exceptionHandler=t,e}}(o=e.Signal||(e.Signal={})),e.Signal=o,function(e){e.exceptionHandler=function(t){console.error(t)},e.connect=function(t,e,i){i=i||void 0;var o=n.get(t.sender);if(o||(o=[],n.set(t.sender,o)),s(o,t,e,i))return!1;var a=i||e,A=r.get(a);A||(A=[],r.set(a,A));var l={signal:t,slot:e,thisArg:i};return o.push(l),A.push(l),!0},e.disconnect=function(t,e,i){i=i||void 0;var o=n.get(t.sender);if(!o||0===o.length)return!1;var a=s(o,t,e,i);if(!a)return!1;var A=i||e,c=r.get(A);return a.signal=null,l(o),l(c),!0},e.disconnectBetween=function(t,e){var o=n.get(t);if(o&&0!==o.length){var a=r.get(e);a&&0!==a.length&&(i.each(a,function(e){e.signal&&e.signal.sender===t&&(e.signal=null)}),l(o),l(a))}},e.disconnectSender=function(t){var e=n.get(t);e&&0!==e.length&&(i.each(e,function(t){if(t.signal){var e=t.thisArg||t.slot;t.signal=null,l(r.get(e))}}),l(e))},e.disconnectReceiver=function(t){var e=r.get(t);e&&0!==e.length&&(i.each(e,function(t){if(t.signal){var e=t.signal.sender;t.signal=null,l(n.get(e))}}),l(e))},e.disconnectAll=function(t){var e=n.get(t);e&&e.length>0&&(i.each(e,function(t){t.signal=null}),l(e));var o=r.get(t);o&&o.length>0&&(i.each(o,function(t){t.signal=null}),l(o))},e.emit=function(t,e){var r=n.get(t.sender);if(r&&0!==r.length)for(var i=0,o=r.length;i<o;++i){var a=r[i];a.signal===t&&A(a,e)}};var n=new WeakMap,r=new WeakMap,o=new Set,a="function"==typeof requestAnimationFrame?requestAnimationFrame:t;function s(t,e,n,r){return i.find(t,function(t){return t.signal===e&&t.slot===n&&t.thisArg===r})}function A(t,n){var r=t.signal,i=t.slot,o=t.thisArg;try{i.call(o,r.sender,n)}catch(t){e.exceptionHandler(t)}}function l(t){0===o.size&&a(c),o.add(t)}function c(){o.forEach(u),o.clear()}function u(t){i.ArrayExt.removeAllWhere(t,g)}function g(t){return null===t.signal}}(r||(r={}))}).call(this,n(118).setImmediate)},function(t,e,n){"use strict";n.d(e,"a",function(){return c});var r=n(2),i=n(86),o=n(161),a=n(32),s=n(130),A=n(63),l=n(104),c=function(t){function e(n,r,i){var a=t.call(this)||this;switch(a.syncErrorValue=null,a.syncErrorThrown=!1,a.syncErrorThrowable=!1,a.isStopped=!1,arguments.length){case 0:a.destination=o.a;break;case 1:if(!n){a.destination=o.a;break}if("object"==typeof n){n instanceof e?(a.syncErrorThrowable=n.syncErrorThrowable,a.destination=n,n.add(a)):(a.syncErrorThrowable=!0,a.destination=new u(a,n));break}default:a.syncErrorThrowable=!0,a.destination=new u(a,n,r,i)}return a}return r.__extends(e,t),e.prototype[s.a]=function(){return this},e.create=function(t,n,r){var i=new e(t,n,r);return i.syncErrorThrowable=!1,i},e.prototype.next=function(t){this.isStopped||this._next(t)},e.prototype.error=function(t){this.isStopped||(this.isStopped=!0,this._error(t))},e.prototype.complete=function(){this.isStopped||(this.isStopped=!0,this._complete())},e.prototype.unsubscribe=function(){this.closed||(this.isStopped=!0,t.prototype.unsubscribe.call(this))},e.prototype._next=function(t){this.destination.next(t)},e.prototype._error=function(t){this.destination.error(t),this.unsubscribe()},e.prototype._complete=function(){this.destination.complete(),this.unsubscribe()},e.prototype._unsubscribeAndRecycle=function(){var t=this._parentOrParents;return this._parentOrParents=null,this.unsubscribe(),this.closed=!1,this.isStopped=!1,this._parentOrParents=t,this},e}(a.a),u=function(t){function e(e,n,r,a){var s,A=t.call(this)||this;A._parentSubscriber=e;var l=A;return Object(i.a)(n)?s=n:n&&(s=n.next,r=n.error,a=n.complete,n!==o.a&&(l=Object.create(n),Object(i.a)(l.unsubscribe)&&A.add(l.unsubscribe.bind(l)),l.unsubscribe=A.unsubscribe.bind(A))),A._context=l,A._next=s,A._error=r,A._complete=a,A}return r.__extends(e,t),e.prototype.next=function(t){if(!this.isStopped&&this._next){var e=this._parentSubscriber;A.a.useDeprecatedSynchronousErrorHandling&&e.syncErrorThrowable?this.__tryOrSetError(e,this._next,t)&&this.unsubscribe():this.__tryOrUnsub(this._next,t)}},e.prototype.error=function(t){if(!this.isStopped){var e=this._parentSubscriber,n=A.a.useDeprecatedSynchronousErrorHandling;if(this._error)n&&e.syncErrorThrowable?(this.__tryOrSetError(e,this._error,t),this.unsubscribe()):(this.__tryOrUnsub(this._error,t),this.unsubscribe());else if(e.syncErrorThrowable)n?(e.syncErrorValue=t,e.syncErrorThrown=!0):Object(l.a)(t),this.unsubscribe();else{if(this.unsubscribe(),n)throw t;Object(l.a)(t)}}},e.prototype.complete=function(){var t=this;if(!this.isStopped){var e=this._parentSubscriber;if(this._complete){var n=function(){return t._complete.call(t._context)};A.a.useDeprecatedSynchronousErrorHandling&&e.syncErrorThrowable?(this.__tryOrSetError(e,n),this.unsubscribe()):(this.__tryOrUnsub(n),this.unsubscribe())}else this.unsubscribe()}},e.prototype.__tryOrUnsub=function(t,e){try{t.call(this._context,e)}catch(t){if(this.unsubscribe(),A.a.useDeprecatedSynchronousErrorHandling)throw t;Object(l.a)(t)}},e.prototype.__tryOrSetError=function(t,e,n){if(!A.a.useDeprecatedSynchronousErrorHandling)throw new Error("bad call");try{e.call(this._context,n)}catch(e){return A.a.useDeprecatedSynchronousErrorHandling?(t.syncErrorValue=e,t.syncErrorThrown=!0,!0):(Object(l.a)(e),!0)}return!1},e.prototype._unsubscribe=function(){var t=this._parentSubscriber;this._context=null,this._parentSubscriber=null,t.unsubscribe()},e}(c)},function(t,e){t.exports=function(t){var e=[];return e.toString=function(){return this.map(function(e){var n=function(t,e){var n=t[1]||"",r=t[3];if(!r)return n;if(e&&"function"==typeof btoa){var i=(a=r,"/*# sourceMappingURL=data:application/json;charset=utf-8;base64,"+btoa(unescape(encodeURIComponent(JSON.stringify(a))))+" */"),o=r.sources.map(function(t){return"/*# sourceURL="+r.sourceRoot+t+" */"});return[n].concat(o).concat([i]).join("\n")}var a;return[n].join("\n")}(e,t);return e[2]?"@media "+e[2]+"{"+n+"}":n}).join("")},e.i=function(t,n){"string"==typeof t&&(t=[[null,t,""]]);for(var r={},i=0;i<this.length;i++){var o=this[i][0];"number"==typeof o&&(r[o]=!0)}for(i=0;i<t.length;i++){var a=t[i];"number"==typeof a[0]&&r[a[0]]||(n&&!a[2]?a[2]=n:n&&(a[2]="("+a[2]+") and ("+n+")"),e.push(a))}},e}},function(t,e,n){"use strict";e.__esModule=!0;var r,i=n(1187),o=(r=i)&&r.__esModule?r:{default:r};e.default=o.default||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}},function(t,e,n){"use strict";e.__esModule=!0;var r,i=n(254),o=(r=i)&&r.__esModule?r:{default:r};e.default=function(t,e){if(!t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return!e||"object"!==(void 0===e?"undefined":(0,o.default)(e))&&"function"!=typeof e?t:e}},function(t,e,n){t.exports=function(){"use strict";var t=navigator.userAgent,e=navigator.platform,n=/gecko\/\d/i.test(t),r=/MSIE \d/.test(t),i=/Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec(t),o=/Edge\/(\d+)/.exec(t),a=r||i||o,s=a&&(r?document.documentMode||6:+(o||i)[1]),A=!o&&/WebKit\//.test(t),l=A&&/Qt\/\d+\.\d+/.test(t),c=!o&&/Chrome\//.test(t),u=/Opera\//.test(t),g=/Apple Computer/.test(navigator.vendor),h=/Mac OS X 1\d\D([8-9]|\d\d)\D/.test(t),d=/PhantomJS/.test(t),f=!o&&/AppleWebKit/.test(t)&&/Mobile\/\w+/.test(t),p=/Android/.test(t),I=f||p||/webOS|BlackBerry|Opera Mini|Opera Mobi|IEMobile/i.test(t),C=f||/Mac/.test(e),m=/\bCrOS\b/.test(t),y=/win/i.test(e),E=u&&t.match(/Version\/(\d*\.\d*)/);E&&(E=Number(E[1])),E&&E>=15&&(u=!1,A=!0);var v=C&&(l||u&&(null==E||E<12.11)),B=n||a&&s>=9;function b(t){return new RegExp("(^|\\s)"+t+"(?:$|\\s)\\s*")}var Q,w=function(t,e){var n=t.className,r=b(e).exec(n);if(r){var i=n.slice(r.index+r[0].length);t.className=n.slice(0,r.index)+(i?r[1]+i:"")}};function x(t){for(var e=t.childNodes.length;e>0;--e)t.removeChild(t.firstChild);return t}function _(t,e){return x(t).appendChild(e)}function D(t,e,n,r){var i=document.createElement(t);if(n&&(i.className=n),r&&(i.style.cssText=r),"string"==typeof e)i.appendChild(document.createTextNode(e));else if(e)for(var o=0;o<e.length;++o)i.appendChild(e[o]);return i}function M(t,e,n,r){var i=D(t,e,n,r);return i.setAttribute("role","presentation"),i}function k(t,e){if(3==e.nodeType&&(e=e.parentNode),t.contains)return t.contains(e);do{if(11==e.nodeType&&(e=e.host),e==t)return!0}while(e=e.parentNode)}function S(){var t;try{t=document.activeElement}catch(e){t=document.body||null}for(;t&&t.shadowRoot&&t.shadowRoot.activeElement;)t=t.shadowRoot.activeElement;return t}function L(t,e){var n=t.className;b(e).test(n)||(t.className+=(n?" ":"")+e)}function R(t,e){for(var n=t.split(" "),r=0;r<n.length;r++)n[r]&&!b(n[r]).test(e)&&(e+=" "+n[r]);return e}Q=document.createRange?function(t,e,n,r){var i=document.createRange();return i.setEnd(r||t,n),i.setStart(t,e),i}:function(t,e,n){var r=document.body.createTextRange();try{r.moveToElementText(t.parentNode)}catch(t){return r}return r.collapse(!0),r.moveEnd("character",n),r.moveStart("character",e),r};var N=function(t){t.select()};function F(t){var e=Array.prototype.slice.call(arguments,1);return function(){return t.apply(null,e)}}function T(t,e,n){for(var r in e||(e={}),t)!t.hasOwnProperty(r)||!1===n&&e.hasOwnProperty(r)||(e[r]=t[r]);return e}function U(t,e,n,r,i){null==e&&-1==(e=t.search(/[^\s\u00a0]/))&&(e=t.length);for(var o=r||0,a=i||0;;){var s=t.indexOf("\t",o);if(s<0||s>=e)return a+(e-o);a+=s-o,a+=n-a%n,o=s+1}}f?N=function(t){t.selectionStart=0,t.selectionEnd=t.value.length}:a&&(N=function(t){try{t.select()}catch(t){}});var O=function(){this.id=null};function G(t,e){for(var n=0;n<t.length;++n)if(t[n]==e)return n;return-1}O.prototype.set=function(t,e){clearTimeout(this.id),this.id=setTimeout(e,t)};var H=30,j={toString:function(){return"CodeMirror.Pass"}},Y={scroll:!1},P={origin:"*mouse"},K={origin:"+move"};function z(t,e,n){for(var r=0,i=0;;){var o=t.indexOf("\t",r);-1==o&&(o=t.length);var a=o-r;if(o==t.length||i+a>=e)return r+Math.min(a,e-i);if(i+=o-r,r=o+1,(i+=n-i%n)>=e)return r}}var J=[""];function q(t){for(;J.length<=t;)J.push(V(J)+" ");return J[t]}function V(t){return t[t.length-1]}function W(t,e){for(var n=[],r=0;r<t.length;r++)n[r]=e(t[r],r);return n}function X(){}function Z(t,e){var n;return Object.create?n=Object.create(t):(X.prototype=t,n=new X),e&&T(e,n),n}var $=/[\u00df\u0587\u0590-\u05f4\u0600-\u06ff\u3040-\u309f\u30a0-\u30ff\u3400-\u4db5\u4e00-\u9fcc\uac00-\ud7af]/;function tt(t){return/\w/.test(t)||t>""&&(t.toUpperCase()!=t.toLowerCase()||$.test(t))}function et(t,e){return e?!!(e.source.indexOf("\\w")>-1&&tt(t))||e.test(t):tt(t)}function nt(t){for(var e in t)if(t.hasOwnProperty(e)&&t[e])return!1;return!0}var rt=/[\u0300-\u036f\u0483-\u0489\u0591-\u05bd\u05bf\u05c1\u05c2\u05c4\u05c5\u05c7\u0610-\u061a\u064b-\u065e\u0670\u06d6-\u06dc\u06de-\u06e4\u06e7\u06e8\u06ea-\u06ed\u0711\u0730-\u074a\u07a6-\u07b0\u07eb-\u07f3\u0816-\u0819\u081b-\u0823\u0825-\u0827\u0829-\u082d\u0900-\u0902\u093c\u0941-\u0948\u094d\u0951-\u0955\u0962\u0963\u0981\u09bc\u09be\u09c1-\u09c4\u09cd\u09d7\u09e2\u09e3\u0a01\u0a02\u0a3c\u0a41\u0a42\u0a47\u0a48\u0a4b-\u0a4d\u0a51\u0a70\u0a71\u0a75\u0a81\u0a82\u0abc\u0ac1-\u0ac5\u0ac7\u0ac8\u0acd\u0ae2\u0ae3\u0b01\u0b3c\u0b3e\u0b3f\u0b41-\u0b44\u0b4d\u0b56\u0b57\u0b62\u0b63\u0b82\u0bbe\u0bc0\u0bcd\u0bd7\u0c3e-\u0c40\u0c46-\u0c48\u0c4a-\u0c4d\u0c55\u0c56\u0c62\u0c63\u0cbc\u0cbf\u0cc2\u0cc6\u0ccc\u0ccd\u0cd5\u0cd6\u0ce2\u0ce3\u0d3e\u0d41-\u0d44\u0d4d\u0d57\u0d62\u0d63\u0dca\u0dcf\u0dd2-\u0dd4\u0dd6\u0ddf\u0e31\u0e34-\u0e3a\u0e47-\u0e4e\u0eb1\u0eb4-\u0eb9\u0ebb\u0ebc\u0ec8-\u0ecd\u0f18\u0f19\u0f35\u0f37\u0f39\u0f71-\u0f7e\u0f80-\u0f84\u0f86\u0f87\u0f90-\u0f97\u0f99-\u0fbc\u0fc6\u102d-\u1030\u1032-\u1037\u1039\u103a\u103d\u103e\u1058\u1059\u105e-\u1060\u1071-\u1074\u1082\u1085\u1086\u108d\u109d\u135f\u1712-\u1714\u1732-\u1734\u1752\u1753\u1772\u1773\u17b7-\u17bd\u17c6\u17c9-\u17d3\u17dd\u180b-\u180d\u18a9\u1920-\u1922\u1927\u1928\u1932\u1939-\u193b\u1a17\u1a18\u1a56\u1a58-\u1a5e\u1a60\u1a62\u1a65-\u1a6c\u1a73-\u1a7c\u1a7f\u1b00-\u1b03\u1b34\u1b36-\u1b3a\u1b3c\u1b42\u1b6b-\u1b73\u1b80\u1b81\u1ba2-\u1ba5\u1ba8\u1ba9\u1c2c-\u1c33\u1c36\u1c37\u1cd0-\u1cd2\u1cd4-\u1ce0\u1ce2-\u1ce8\u1ced\u1dc0-\u1de6\u1dfd-\u1dff\u200c\u200d\u20d0-\u20f0\u2cef-\u2cf1\u2de0-\u2dff\u302a-\u302f\u3099\u309a\ua66f-\ua672\ua67c\ua67d\ua6f0\ua6f1\ua802\ua806\ua80b\ua825\ua826\ua8c4\ua8e0-\ua8f1\ua926-\ua92d\ua947-\ua951\ua980-\ua982\ua9b3\ua9b6-\ua9b9\ua9bc\uaa29-\uaa2e\uaa31\uaa32\uaa35\uaa36\uaa43\uaa4c\uaab0\uaab2-\uaab4\uaab7\uaab8\uaabe\uaabf\uaac1\uabe5\uabe8\uabed\udc00-\udfff\ufb1e\ufe00-\ufe0f\ufe20-\ufe26\uff9e\uff9f]/;function it(t){return t.charCodeAt(0)>=768&&rt.test(t)}function ot(t,e,n){for(;(n<0?e>0:e<t.length)&&it(t.charAt(e));)e+=n;return e}function at(t,e,n){for(var r=e>n?-1:1;;){if(e==n)return e;var i=(e+n)/2,o=r<0?Math.ceil(i):Math.floor(i);if(o==e)return t(o)?e:n;t(o)?n=o:e=o+r}}var st=null;function At(t,e,n){var r;st=null;for(var i=0;i<t.length;++i){var o=t[i];if(o.from<e&&o.to>e)return i;o.to==e&&(o.from!=o.to&&"before"==n?r=i:st=i),o.from==e&&(o.from!=o.to&&"before"!=n?r=i:st=i)}return null!=r?r:st}var lt=function(){var t="bbbbbbbbbtstwsbbbbbbbbbbbbbbssstwNN%%%NNNNNN,N,N1111111111NNNNNNNLLLLLLLLLLLLLLLLLLLLLLLLLLNNNNNNLLLLLLLLLLLLLLLLLLLLLLLLLLNNNNbbbbbbsbbbbbbbbbbbbbbbbbbbbbbbbbb,N%%%%NNNNLNNNNN%%11NLNNN1LNNNNNLLLLLLLLLLLLLLLLLLLLLLLNLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLN",e="nnnnnnNNr%%r,rNNmmmmmmmmmmmrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrmmmmmmmmmmmmmmmmmmmmmnnnnnnnnnn%nnrrrmrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrmmmmmmmnNmmmmmmrrmmNmmmmrr1111111111";var n=/[\u0590-\u05f4\u0600-\u06ff\u0700-\u08ac]/,r=/[stwN]/,i=/[LRr]/,o=/[Lb1n]/,a=/[1n]/;function s(t,e,n){this.level=t,this.from=e,this.to=n}return function(A,l){var c,u="ltr"==l?"L":"R";if(0==A.length||"ltr"==l&&!n.test(A))return!1;for(var g=A.length,h=[],d=0;d<g;++d)h.push((c=A.charCodeAt(d))<=247?t.charAt(c):1424<=c&&c<=1524?"R":1536<=c&&c<=1785?e.charAt(c-1536):1774<=c&&c<=2220?"r":8192<=c&&c<=8203?"w":8204==c?"b":"L");for(var f=0,p=u;f<g;++f){var I=h[f];"m"==I?h[f]=p:p=I}for(var C=0,m=u;C<g;++C){var y=h[C];"1"==y&&"r"==m?h[C]="n":i.test(y)&&(m=y,"r"==y&&(h[C]="R"))}for(var E=1,v=h[0];E<g-1;++E){var B=h[E];"+"==B&&"1"==v&&"1"==h[E+1]?h[E]="1":","!=B||v!=h[E+1]||"1"!=v&&"n"!=v||(h[E]=v),v=B}for(var b=0;b<g;++b){var Q=h[b];if(","==Q)h[b]="N";else if("%"==Q){var w=void 0;for(w=b+1;w<g&&"%"==h[w];++w);for(var x=b&&"!"==h[b-1]||w<g&&"1"==h[w]?"1":"N",_=b;_<w;++_)h[_]=x;b=w-1}}for(var D=0,M=u;D<g;++D){var k=h[D];"L"==M&&"1"==k?h[D]="L":i.test(k)&&(M=k)}for(var S=0;S<g;++S)if(r.test(h[S])){var L=void 0;for(L=S+1;L<g&&r.test(h[L]);++L);for(var R="L"==(S?h[S-1]:u),N="L"==(L<g?h[L]:u),F=R==N?R?"L":"R":u,T=S;T<L;++T)h[T]=F;S=L-1}for(var U,O=[],G=0;G<g;)if(o.test(h[G])){var H=G;for(++G;G<g&&o.test(h[G]);++G);O.push(new s(0,H,G))}else{var j=G,Y=O.length;for(++G;G<g&&"L"!=h[G];++G);for(var P=j;P<G;)if(a.test(h[P])){j<P&&O.splice(Y,0,new s(1,j,P));var K=P;for(++P;P<G&&a.test(h[P]);++P);O.splice(Y,0,new s(2,K,P)),j=P}else++P;j<G&&O.splice(Y,0,new s(1,j,G))}return"ltr"==l&&(1==O[0].level&&(U=A.match(/^\s+/))&&(O[0].from=U[0].length,O.unshift(new s(0,0,U[0].length))),1==V(O).level&&(U=A.match(/\s+$/))&&(V(O).to-=U[0].length,O.push(new s(0,g-U[0].length,g)))),"rtl"==l?O.reverse():O}}();function ct(t,e){var n=t.order;return null==n&&(n=t.order=lt(t.text,e)),n}var ut=[],gt=function(t,e,n){if(t.addEventListener)t.addEventListener(e,n,!1);else if(t.attachEvent)t.attachEvent("on"+e,n);else{var r=t._handlers||(t._handlers={});r[e]=(r[e]||ut).concat(n)}};function ht(t,e){return t._handlers&&t._handlers[e]||ut}function dt(t,e,n){if(t.removeEventListener)t.removeEventListener(e,n,!1);else if(t.detachEvent)t.detachEvent("on"+e,n);else{var r=t._handlers,i=r&&r[e];if(i){var o=G(i,n);o>-1&&(r[e]=i.slice(0,o).concat(i.slice(o+1)))}}}function ft(t,e){var n=ht(t,e);if(n.length)for(var r=Array.prototype.slice.call(arguments,2),i=0;i<n.length;++i)n[i].apply(null,r)}function pt(t,e,n){return"string"==typeof e&&(e={type:e,preventDefault:function(){this.defaultPrevented=!0}}),ft(t,n||e.type,t,e),vt(e)||e.codemirrorIgnore}function It(t){var e=t._handlers&&t._handlers.cursorActivity;if(e)for(var n=t.curOp.cursorActivityHandlers||(t.curOp.cursorActivityHandlers=[]),r=0;r<e.length;++r)-1==G(n,e[r])&&n.push(e[r])}function Ct(t,e){return ht(t,e).length>0}function mt(t){t.prototype.on=function(t,e){gt(this,t,e)},t.prototype.off=function(t,e){dt(this,t,e)}}function yt(t){t.preventDefault?t.preventDefault():t.returnValue=!1}function Et(t){t.stopPropagation?t.stopPropagation():t.cancelBubble=!0}function vt(t){return null!=t.defaultPrevented?t.defaultPrevented:0==t.returnValue}function Bt(t){yt(t),Et(t)}function bt(t){return t.target||t.srcElement}function Qt(t){var e=t.which;return null==e&&(1&t.button?e=1:2&t.button?e=3:4&t.button&&(e=2)),C&&t.ctrlKey&&1==e&&(e=3),e}var wt,xt,_t=function(){if(a&&s<9)return!1;var t=D("div");return"draggable"in t||"dragDrop"in t}();function Dt(t){if(null==wt){var e=D("span","​");_(t,D("span",[e,document.createTextNode("x")])),0!=t.firstChild.offsetHeight&&(wt=e.offsetWidth<=1&&e.offsetHeight>2&&!(a&&s<8))}var n=wt?D("span","​"):D("span"," ",null,"display: inline-block; width: 1px; margin-right: -1px");return n.setAttribute("cm-text",""),n}function Mt(t){if(null!=xt)return xt;var e=_(t,document.createTextNode("AخA")),n=Q(e,0,1).getBoundingClientRect(),r=Q(e,1,2).getBoundingClientRect();return x(t),!(!n||n.left==n.right)&&(xt=r.right-n.right<3)}var kt,St=3!="\n\nb".split(/\n/).length?function(t){for(var e=0,n=[],r=t.length;e<=r;){var i=t.indexOf("\n",e);-1==i&&(i=t.length);var o=t.slice(e,"\r"==t.charAt(i-1)?i-1:i),a=o.indexOf("\r");-1!=a?(n.push(o.slice(0,a)),e+=a+1):(n.push(o),e=i+1)}return n}:function(t){return t.split(/\r\n?|\n/)},Lt=window.getSelection?function(t){try{return t.selectionStart!=t.selectionEnd}catch(t){return!1}}:function(t){var e;try{e=t.ownerDocument.selection.createRange()}catch(t){}return!(!e||e.parentElement()!=t)&&0!=e.compareEndPoints("StartToEnd",e)},Rt="oncopy"in(kt=D("div"))||(kt.setAttribute("oncopy","return;"),"function"==typeof kt.oncopy),Nt=null,Ft={},Tt={};function Ut(t,e){arguments.length>2&&(e.dependencies=Array.prototype.slice.call(arguments,2)),Ft[t]=e}function Ot(t){if("string"==typeof t&&Tt.hasOwnProperty(t))t=Tt[t];else if(t&&"string"==typeof t.name&&Tt.hasOwnProperty(t.name)){var e=Tt[t.name];"string"==typeof e&&(e={name:e}),(t=Z(e,t)).name=e.name}else{if("string"==typeof t&&/^[\w\-]+\/[\w\-]+\+xml$/.test(t))return Ot("application/xml");if("string"==typeof t&&/^[\w\-]+\/[\w\-]+\+json$/.test(t))return Ot("application/json")}return"string"==typeof t?{name:t}:t||{name:"null"}}function Gt(t,e){e=Ot(e);var n=Ft[e.name];if(!n)return Gt(t,"text/plain");var r=n(t,e);if(Ht.hasOwnProperty(e.name)){var i=Ht[e.name];for(var o in i)i.hasOwnProperty(o)&&(r.hasOwnProperty(o)&&(r["_"+o]=r[o]),r[o]=i[o])}if(r.name=e.name,e.helperType&&(r.helperType=e.helperType),e.modeProps)for(var a in e.modeProps)r[a]=e.modeProps[a];return r}var Ht={};function jt(t,e){var n=Ht.hasOwnProperty(t)?Ht[t]:Ht[t]={};T(e,n)}function Yt(t,e){if(!0===e)return e;if(t.copyState)return t.copyState(e);var n={};for(var r in e){var i=e[r];i instanceof Array&&(i=i.concat([])),n[r]=i}return n}function Pt(t,e){for(var n;t.innerMode&&(n=t.innerMode(e))&&n.mode!=t;)e=n.state,t=n.mode;return n||{mode:t,state:e}}function Kt(t,e,n){return!t.startState||t.startState(e,n)}var zt=function(t,e,n){this.pos=this.start=0,this.string=t,this.tabSize=e||8,this.lastColumnPos=this.lastColumnValue=0,this.lineStart=0,this.lineOracle=n};function Jt(t,e){if((e-=t.first)<0||e>=t.size)throw new Error("There is no line "+(e+t.first)+" in the document.");for(var n=t;!n.lines;)for(var r=0;;++r){var i=n.children[r],o=i.chunkSize();if(e<o){n=i;break}e-=o}return n.lines[e]}function qt(t,e,n){var r=[],i=e.line;return t.iter(e.line,n.line+1,function(t){var o=t.text;i==n.line&&(o=o.slice(0,n.ch)),i==e.line&&(o=o.slice(e.ch)),r.push(o),++i}),r}function Vt(t,e,n){var r=[];return t.iter(e,n,function(t){r.push(t.text)}),r}function Wt(t,e){var n=e-t.height;if(n)for(var r=t;r;r=r.parent)r.height+=n}function Xt(t){if(null==t.parent)return null;for(var e=t.parent,n=G(e.lines,t),r=e.parent;r;e=r,r=r.parent)for(var i=0;r.children[i]!=e;++i)n+=r.children[i].chunkSize();return n+e.first}function Zt(t,e){var n=t.first;t:do{for(var r=0;r<t.children.length;++r){var i=t.children[r],o=i.height;if(e<o){t=i;continue t}e-=o,n+=i.chunkSize()}return n}while(!t.lines);for(var a=0;a<t.lines.length;++a){var s=t.lines[a],A=s.height;if(e<A)break;e-=A}return n+a}function $t(t,e){return e>=t.first&&e<t.first+t.size}function te(t,e){return String(t.lineNumberFormatter(e+t.firstLineNumber))}function ee(t,e,n){if(void 0===n&&(n=null),!(this instanceof ee))return new ee(t,e,n);this.line=t,this.ch=e,this.sticky=n}function ne(t,e){return t.line-e.line||t.ch-e.ch}function re(t,e){return t.sticky==e.sticky&&0==ne(t,e)}function ie(t){return ee(t.line,t.ch)}function oe(t,e){return ne(t,e)<0?e:t}function ae(t,e){return ne(t,e)<0?t:e}function se(t,e){return Math.max(t.first,Math.min(e,t.first+t.size-1))}function Ae(t,e){if(e.line<t.first)return ee(t.first,0);var n=t.first+t.size-1;return e.line>n?ee(n,Jt(t,n).text.length):function(t,e){var n=t.ch;return null==n||n>e?ee(t.line,e):n<0?ee(t.line,0):t}(e,Jt(t,e.line).text.length)}function le(t,e){for(var n=[],r=0;r<e.length;r++)n[r]=Ae(t,e[r]);return n}zt.prototype.eol=function(){return this.pos>=this.string.length},zt.prototype.sol=function(){return this.pos==this.lineStart},zt.prototype.peek=function(){return this.string.charAt(this.pos)||void 0},zt.prototype.next=function(){if(this.pos<this.string.length)return this.string.charAt(this.pos++)},zt.prototype.eat=function(t){var e=this.string.charAt(this.pos);if("string"==typeof t?e==t:e&&(t.test?t.test(e):t(e)))return++this.pos,e},zt.prototype.eatWhile=function(t){for(var e=this.pos;this.eat(t););return this.pos>e},zt.prototype.eatSpace=function(){for(var t=this.pos;/[\s\u00a0]/.test(this.string.charAt(this.pos));)++this.pos;return this.pos>t},zt.prototype.skipToEnd=function(){this.pos=this.string.length},zt.prototype.skipTo=function(t){var e=this.string.indexOf(t,this.pos);if(e>-1)return this.pos=e,!0},zt.prototype.backUp=function(t){this.pos-=t},zt.prototype.column=function(){return this.lastColumnPos<this.start&&(this.lastColumnValue=U(this.string,this.start,this.tabSize,this.lastColumnPos,this.lastColumnValue),this.lastColumnPos=this.start),this.lastColumnValue-(this.lineStart?U(this.string,this.lineStart,this.tabSize):0)},zt.prototype.indentation=function(){return U(this.string,null,this.tabSize)-(this.lineStart?U(this.string,this.lineStart,this.tabSize):0)},zt.prototype.match=function(t,e,n){if("string"!=typeof t){var r=this.string.slice(this.pos).match(t);return r&&r.index>0?null:(r&&!1!==e&&(this.pos+=r[0].length),r)}var i=function(t){return n?t.toLowerCase():t},o=this.string.substr(this.pos,t.length);if(i(o)==i(t))return!1!==e&&(this.pos+=t.length),!0},zt.prototype.current=function(){return this.string.slice(this.start,this.pos)},zt.prototype.hideFirstChars=function(t,e){this.lineStart+=t;try{return e()}finally{this.lineStart-=t}},zt.prototype.lookAhead=function(t){var e=this.lineOracle;return e&&e.lookAhead(t)},zt.prototype.baseToken=function(){var t=this.lineOracle;return t&&t.baseToken(this.pos)};var ce=function(t,e){this.state=t,this.lookAhead=e},ue=function(t,e,n,r){this.state=e,this.doc=t,this.line=n,this.maxLookAhead=r||0,this.baseTokens=null,this.baseTokenPos=1};function ge(t,e,n,r){var i=[t.state.modeGen],o={};Ee(t,e.text,t.doc.mode,n,function(t,e){return i.push(t,e)},o,r);for(var a=n.state,s=function(r){n.baseTokens=i;var s=t.state.overlays[r],A=1,l=0;n.state=!0,Ee(t,e.text,s.mode,n,function(t,e){for(var n=A;l<t;){var r=i[A];r>t&&i.splice(A,1,t,i[A+1],r),A+=2,l=Math.min(t,r)}if(e)if(s.opaque)i.splice(n,A-n,t,"overlay "+e),A=n+2;else for(;n<A;n+=2){var o=i[n+1];i[n+1]=(o?o+" ":"")+"overlay "+e}},o),n.state=a,n.baseTokens=null,n.baseTokenPos=1},A=0;A<t.state.overlays.length;++A)s(A);return{styles:i,classes:o.bgClass||o.textClass?o:null}}function he(t,e,n){if(!e.styles||e.styles[0]!=t.state.modeGen){var r=de(t,Xt(e)),i=e.text.length>t.options.maxHighlightLength&&Yt(t.doc.mode,r.state),o=ge(t,e,r);i&&(r.state=i),e.stateAfter=r.save(!i),e.styles=o.styles,o.classes?e.styleClasses=o.classes:e.styleClasses&&(e.styleClasses=null),n===t.doc.highlightFrontier&&(t.doc.modeFrontier=Math.max(t.doc.modeFrontier,++t.doc.highlightFrontier))}return e.styles}function de(t,e,n){var r=t.doc,i=t.display;if(!r.mode.startState)return new ue(r,!0,e);var o=function(t,e,n){for(var r,i,o=t.doc,a=n?-1:e-(t.doc.mode.innerMode?1e3:100),s=e;s>a;--s){if(s<=o.first)return o.first;var A=Jt(o,s-1),l=A.stateAfter;if(l&&(!n||s+(l instanceof ce?l.lookAhead:0)<=o.modeFrontier))return s;var c=U(A.text,null,t.options.tabSize);(null==i||r>c)&&(i=s-1,r=c)}return i}(t,e,n),a=o>r.first&&Jt(r,o-1).stateAfter,s=a?ue.fromSaved(r,a,o):new ue(r,Kt(r.mode),o);return r.iter(o,e,function(n){fe(t,n.text,s);var r=s.line;n.stateAfter=r==e-1||r%5==0||r>=i.viewFrom&&r<i.viewTo?s.save():null,s.nextLine()}),n&&(r.modeFrontier=s.line),s}function fe(t,e,n,r){var i=t.doc.mode,o=new zt(e,t.options.tabSize,n);for(o.start=o.pos=r||0,""==e&&pe(i,n.state);!o.eol();)Ie(i,o,n.state),o.start=o.pos}function pe(t,e){if(t.blankLine)return t.blankLine(e);if(t.innerMode){var n=Pt(t,e);return n.mode.blankLine?n.mode.blankLine(n.state):void 0}}function Ie(t,e,n,r){for(var i=0;i<10;i++){r&&(r[0]=Pt(t,n).mode);var o=t.token(e,n);if(e.pos>e.start)return o}throw new Error("Mode "+t.name+" failed to advance stream.")}ue.prototype.lookAhead=function(t){var e=this.doc.getLine(this.line+t);return null!=e&&t>this.maxLookAhead&&(this.maxLookAhead=t),e},ue.prototype.baseToken=function(t){if(!this.baseTokens)return null;for(;this.baseTokens[this.baseTokenPos]<=t;)this.baseTokenPos+=2;var e=this.baseTokens[this.baseTokenPos+1];return{type:e&&e.replace(/( |^)overlay .*/,""),size:this.baseTokens[this.baseTokenPos]-t}},ue.prototype.nextLine=function(){this.line++,this.maxLookAhead>0&&this.maxLookAhead--},ue.fromSaved=function(t,e,n){return e instanceof ce?new ue(t,Yt(t.mode,e.state),n,e.lookAhead):new ue(t,Yt(t.mode,e),n)},ue.prototype.save=function(t){var e=!1!==t?Yt(this.doc.mode,this.state):this.state;return this.maxLookAhead>0?new ce(e,this.maxLookAhead):e};var Ce=function(t,e,n){this.start=t.start,this.end=t.pos,this.string=t.current(),this.type=e||null,this.state=n};function me(t,e,n,r){var i,o=t.doc,a=o.mode;e=Ae(o,e);var s,A=Jt(o,e.line),l=de(t,e.line,n),c=new zt(A.text,t.options.tabSize,l);for(r&&(s=[]);(r||c.pos<e.ch)&&!c.eol();)c.start=c.pos,i=Ie(a,c,l.state),r&&s.push(new Ce(c,i,Yt(o.mode,l.state)));return r?s:new Ce(c,i,l.state)}function ye(t,e){if(t)for(;;){var n=t.match(/(?:^|\s+)line-(background-)?(\S+)/);if(!n)break;t=t.slice(0,n.index)+t.slice(n.index+n[0].length);var r=n[1]?"bgClass":"textClass";null==e[r]?e[r]=n[2]:new RegExp("(?:^|s)"+n[2]+"(?:$|s)").test(e[r])||(e[r]+=" "+n[2])}return t}function Ee(t,e,n,r,i,o,a){var s=n.flattenSpans;null==s&&(s=t.options.flattenSpans);var A,l=0,c=null,u=new zt(e,t.options.tabSize,r),g=t.options.addModeClass&&[null];for(""==e&&ye(pe(n,r.state),o);!u.eol();){if(u.pos>t.options.maxHighlightLength?(s=!1,a&&fe(t,e,r,u.pos),u.pos=e.length,A=null):A=ye(Ie(n,u,r.state,g),o),g){var h=g[0].name;h&&(A="m-"+(A?h+" "+A:h))}if(!s||c!=A){for(;l<u.start;)l=Math.min(u.start,l+5e3),i(l,c);c=A}u.start=u.pos}for(;l<u.pos;){var d=Math.min(u.pos,l+5e3);i(d,c),l=d}}var ve=!1,Be=!1;function be(t,e,n){this.marker=t,this.from=e,this.to=n}function Qe(t,e){if(t)for(var n=0;n<t.length;++n){var r=t[n];if(r.marker==e)return r}}function we(t,e){for(var n,r=0;r<t.length;++r)t[r]!=e&&(n||(n=[])).push(t[r]);return n}function xe(t,e){if(e.full)return null;var n=$t(t,e.from.line)&&Jt(t,e.from.line).markedSpans,r=$t(t,e.to.line)&&Jt(t,e.to.line).markedSpans;if(!n&&!r)return null;var i=e.from.ch,o=e.to.ch,a=0==ne(e.from,e.to),s=function(t,e,n){var r;if(t)for(var i=0;i<t.length;++i){var o=t[i],a=o.marker,s=null==o.from||(a.inclusiveLeft?o.from<=e:o.from<e);if(s||o.from==e&&"bookmark"==a.type&&(!n||!o.marker.insertLeft)){var A=null==o.to||(a.inclusiveRight?o.to>=e:o.to>e);(r||(r=[])).push(new be(a,o.from,A?null:o.to))}}return r}(n,i,a),A=function(t,e,n){var r;if(t)for(var i=0;i<t.length;++i){var o=t[i],a=o.marker,s=null==o.to||(a.inclusiveRight?o.to>=e:o.to>e);if(s||o.from==e&&"bookmark"==a.type&&(!n||o.marker.insertLeft)){var A=null==o.from||(a.inclusiveLeft?o.from<=e:o.from<e);(r||(r=[])).push(new be(a,A?null:o.from-e,null==o.to?null:o.to-e))}}return r}(r,o,a),l=1==e.text.length,c=V(e.text).length+(l?i:0);if(s)for(var u=0;u<s.length;++u){var g=s[u];if(null==g.to){var h=Qe(A,g.marker);h?l&&(g.to=null==h.to?null:h.to+c):g.to=i}}if(A)for(var d=0;d<A.length;++d){var f=A[d];if(null!=f.to&&(f.to+=c),null==f.from){var p=Qe(s,f.marker);p||(f.from=c,l&&(s||(s=[])).push(f))}else f.from+=c,l&&(s||(s=[])).push(f)}s&&(s=_e(s)),A&&A!=s&&(A=_e(A));var I=[s];if(!l){var C,m=e.text.length-2;if(m>0&&s)for(var y=0;y<s.length;++y)null==s[y].to&&(C||(C=[])).push(new be(s[y].marker,null,null));for(var E=0;E<m;++E)I.push(C);I.push(A)}return I}function _e(t){for(var e=0;e<t.length;++e){var n=t[e];null!=n.from&&n.from==n.to&&!1!==n.marker.clearWhenEmpty&&t.splice(e--,1)}return t.length?t:null}function De(t){var e=t.markedSpans;if(e){for(var n=0;n<e.length;++n)e[n].marker.detachLine(t);t.markedSpans=null}}function Me(t,e){if(e){for(var n=0;n<e.length;++n)e[n].marker.attachLine(t);t.markedSpans=e}}function ke(t){return t.inclusiveLeft?-1:0}function Se(t){return t.inclusiveRight?1:0}function Le(t,e){var n=t.lines.length-e.lines.length;if(0!=n)return n;var r=t.find(),i=e.find(),o=ne(r.from,i.from)||ke(t)-ke(e);if(o)return-o;var a=ne(r.to,i.to)||Se(t)-Se(e);return a||e.id-t.id}function Re(t,e){var n,r=Be&&t.markedSpans;if(r)for(var i=void 0,o=0;o<r.length;++o)(i=r[o]).marker.collapsed&&null==(e?i.from:i.to)&&(!n||Le(n,i.marker)<0)&&(n=i.marker);return n}function Ne(t){return Re(t,!0)}function Fe(t){return Re(t,!1)}function Te(t,e){var n,r=Be&&t.markedSpans;if(r)for(var i=0;i<r.length;++i){var o=r[i];o.marker.collapsed&&(null==o.from||o.from<e)&&(null==o.to||o.to>e)&&(!n||Le(n,o.marker)<0)&&(n=o.marker)}return n}function Ue(t,e,n,r,i){var o=Jt(t,e),a=Be&&o.markedSpans;if(a)for(var s=0;s<a.length;++s){var A=a[s];if(A.marker.collapsed){var l=A.marker.find(0),c=ne(l.from,n)||ke(A.marker)-ke(i),u=ne(l.to,r)||Se(A.marker)-Se(i);if(!(c>=0&&u<=0||c<=0&&u>=0)&&(c<=0&&(A.marker.inclusiveRight&&i.inclusiveLeft?ne(l.to,n)>=0:ne(l.to,n)>0)||c>=0&&(A.marker.inclusiveRight&&i.inclusiveLeft?ne(l.from,r)<=0:ne(l.from,r)<0)))return!0}}}function Oe(t){for(var e;e=Ne(t);)t=e.find(-1,!0).line;return t}function Ge(t,e){var n=Jt(t,e),r=Oe(n);return n==r?e:Xt(r)}function He(t,e){if(e>t.lastLine())return e;var n,r=Jt(t,e);if(!je(t,r))return e;for(;n=Fe(r);)r=n.find(1,!0).line;return Xt(r)+1}function je(t,e){var n=Be&&e.markedSpans;if(n)for(var r=void 0,i=0;i<n.length;++i)if((r=n[i]).marker.collapsed){if(null==r.from)return!0;if(!r.marker.widgetNode&&0==r.from&&r.marker.inclusiveLeft&&Ye(t,e,r))return!0}}function Ye(t,e,n){if(null==n.to){var r=n.marker.find(1,!0);return Ye(t,r.line,Qe(r.line.markedSpans,n.marker))}if(n.marker.inclusiveRight&&n.to==e.text.length)return!0;for(var i=void 0,o=0;o<e.markedSpans.length;++o)if((i=e.markedSpans[o]).marker.collapsed&&!i.marker.widgetNode&&i.from==n.to&&(null==i.to||i.to!=n.from)&&(i.marker.inclusiveLeft||n.marker.inclusiveRight)&&Ye(t,e,i))return!0}function Pe(t){t=Oe(t);for(var e=0,n=t.parent,r=0;r<n.lines.length;++r){var i=n.lines[r];if(i==t)break;e+=i.height}for(var o=n.parent;o;o=(n=o).parent)for(var a=0;a<o.children.length;++a){var s=o.children[a];if(s==n)break;e+=s.height}return e}function Ke(t){if(0==t.height)return 0;for(var e,n=t.text.length,r=t;e=Ne(r);){var i=e.find(0,!0);r=i.from.line,n+=i.from.ch-i.to.ch}for(r=t;e=Fe(r);){var o=e.find(0,!0);n-=r.text.length-o.from.ch,r=o.to.line,n+=r.text.length-o.to.ch}return n}function ze(t){var e=t.display,n=t.doc;e.maxLine=Jt(n,n.first),e.maxLineLength=Ke(e.maxLine),e.maxLineChanged=!0,n.iter(function(t){var n=Ke(t);n>e.maxLineLength&&(e.maxLineLength=n,e.maxLine=t)})}var Je=function(t,e,n){this.text=t,Me(this,e),this.height=n?n(this):1};function qe(t){t.parent=null,De(t)}Je.prototype.lineNo=function(){return Xt(this)},mt(Je);var Ve={},We={};function Xe(t,e){if(!t||/^\s*$/.test(t))return null;var n=e.addModeClass?We:Ve;return n[t]||(n[t]=t.replace(/\S+/g,"cm-$&"))}function Ze(t,e){var n=M("span",null,null,A?"padding-right: .1px":null),r={pre:M("pre",[n],"CodeMirror-line"),content:n,col:0,pos:0,cm:t,trailingSpace:!1,splitSpaces:t.getOption("lineWrapping")};e.measure={};for(var i=0;i<=(e.rest?e.rest.length:0);i++){var o=i?e.rest[i-1]:e.line,a=void 0;r.pos=0,r.addToken=tn,Mt(t.display.measure)&&(a=ct(o,t.doc.direction))&&(r.addToken=en(r.addToken,a)),r.map=[];var s=e!=t.display.externalMeasured&&Xt(o);rn(o,r,he(t,o,s)),o.styleClasses&&(o.styleClasses.bgClass&&(r.bgClass=R(o.styleClasses.bgClass,r.bgClass||"")),o.styleClasses.textClass&&(r.textClass=R(o.styleClasses.textClass,r.textClass||""))),0==r.map.length&&r.map.push(0,0,r.content.appendChild(Dt(t.display.measure))),0==i?(e.measure.map=r.map,e.measure.cache={}):((e.measure.maps||(e.measure.maps=[])).push(r.map),(e.measure.caches||(e.measure.caches=[])).push({}))}if(A){var l=r.content.lastChild;(/\bcm-tab\b/.test(l.className)||l.querySelector&&l.querySelector(".cm-tab"))&&(r.content.className="cm-tab-wrap-hack")}return ft(t,"renderLine",t,e.line,r.pre),r.pre.className&&(r.textClass=R(r.pre.className,r.textClass||"")),r}function $e(t){var e=D("span","•","cm-invalidchar");return e.title="\\u"+t.charCodeAt(0).toString(16),e.setAttribute("aria-label",e.title),e}function tn(t,e,n,r,i,o,A){if(e){var l,c=t.splitSpaces?function(t,e){if(t.length>1&&!/  /.test(t))return t;for(var n=e,r="",i=0;i<t.length;i++){var o=t.charAt(i);" "!=o||!n||i!=t.length-1&&32!=t.charCodeAt(i+1)||(o=" "),r+=o,n=" "==o}return r}(e,t.trailingSpace):e,u=t.cm.state.specialChars,g=!1;if(u.test(e)){l=document.createDocumentFragment();for(var h=0;;){u.lastIndex=h;var d=u.exec(e),f=d?d.index-h:e.length-h;if(f){var p=document.createTextNode(c.slice(h,h+f));a&&s<9?l.appendChild(D("span",[p])):l.appendChild(p),t.map.push(t.pos,t.pos+f,p),t.col+=f,t.pos+=f}if(!d)break;h+=f+1;var I=void 0;if("\t"==d[0]){var C=t.cm.options.tabSize,m=C-t.col%C;(I=l.appendChild(D("span",q(m),"cm-tab"))).setAttribute("role","presentation"),I.setAttribute("cm-text","\t"),t.col+=m}else"\r"==d[0]||"\n"==d[0]?((I=l.appendChild(D("span","\r"==d[0]?"␍":"␤","cm-invalidchar"))).setAttribute("cm-text",d[0]),t.col+=1):((I=t.cm.options.specialCharPlaceholder(d[0])).setAttribute("cm-text",d[0]),a&&s<9?l.appendChild(D("span",[I])):l.appendChild(I),t.col+=1);t.map.push(t.pos,t.pos+1,I),t.pos++}}else t.col+=e.length,l=document.createTextNode(c),t.map.push(t.pos,t.pos+e.length,l),a&&s<9&&(g=!0),t.pos+=e.length;if(t.trailingSpace=32==c.charCodeAt(e.length-1),n||r||i||g||o){var y=n||"";r&&(y+=r),i&&(y+=i);var E=D("span",[l],y,o);if(A)for(var v in A)A.hasOwnProperty(v)&&"style"!=v&&"class"!=v&&E.setAttribute(v,A[v]);return t.content.appendChild(E)}t.content.appendChild(l)}}function en(t,e){return function(n,r,i,o,a,s,A){i=i?i+" cm-force-border":"cm-force-border";for(var l=n.pos,c=l+r.length;;){for(var u=void 0,g=0;g<e.length&&!((u=e[g]).to>l&&u.from<=l);g++);if(u.to>=c)return t(n,r,i,o,a,s,A);t(n,r.slice(0,u.to-l),i,o,null,s,A),o=null,r=r.slice(u.to-l),l=u.to}}}function nn(t,e,n,r){var i=!r&&n.widgetNode;i&&t.map.push(t.pos,t.pos+e,i),!r&&t.cm.display.input.needsContentAttribute&&(i||(i=t.content.appendChild(document.createElement("span"))),i.setAttribute("cm-marker",n.id)),i&&(t.cm.display.input.setUneditable(i),t.content.appendChild(i)),t.pos+=e,t.trailingSpace=!1}function rn(t,e,n){var r=t.markedSpans,i=t.text,o=0;if(r)for(var a,s,A,l,c,u,g,h=i.length,d=0,f=1,p="",I=0;;){if(I==d){A=l=c=s="",g=null,u=null,I=1/0;for(var C=[],m=void 0,y=0;y<r.length;++y){var E=r[y],v=E.marker;if("bookmark"==v.type&&E.from==d&&v.widgetNode)C.push(v);else if(E.from<=d&&(null==E.to||E.to>d||v.collapsed&&E.to==d&&E.from==d)){if(null!=E.to&&E.to!=d&&I>E.to&&(I=E.to,l=""),v.className&&(A+=" "+v.className),v.css&&(s=(s?s+";":"")+v.css),v.startStyle&&E.from==d&&(c+=" "+v.startStyle),v.endStyle&&E.to==I&&(m||(m=[])).push(v.endStyle,E.to),v.title&&((g||(g={})).title=v.title),v.attributes)for(var B in v.attributes)(g||(g={}))[B]=v.attributes[B];v.collapsed&&(!u||Le(u.marker,v)<0)&&(u=E)}else E.from>d&&I>E.from&&(I=E.from)}if(m)for(var b=0;b<m.length;b+=2)m[b+1]==I&&(l+=" "+m[b]);if(!u||u.from==d)for(var Q=0;Q<C.length;++Q)nn(e,0,C[Q]);if(u&&(u.from||0)==d){if(nn(e,(null==u.to?h+1:u.to)-d,u.marker,null==u.from),null==u.to)return;u.to==d&&(u=!1)}}if(d>=h)break;for(var w=Math.min(h,I);;){if(p){var x=d+p.length;if(!u){var _=x>w?p.slice(0,w-d):p;e.addToken(e,_,a?a+A:A,c,d+_.length==I?l:"",s,g)}if(x>=w){p=p.slice(w-d),d=w;break}d=x,c=""}p=i.slice(o,o=n[f++]),a=Xe(n[f++],e.cm.options)}}else for(var D=1;D<n.length;D+=2)e.addToken(e,i.slice(o,o=n[D]),Xe(n[D+1],e.cm.options))}function on(t,e,n){this.line=e,this.rest=function(t){for(var e,n;e=Fe(t);)t=e.find(1,!0).line,(n||(n=[])).push(t);return n}(e),this.size=this.rest?Xt(V(this.rest))-n+1:1,this.node=this.text=null,this.hidden=je(t,e)}function an(t,e,n){for(var r,i=[],o=e;o<n;o=r){var a=new on(t.doc,Jt(t.doc,o),o);r=o+a.size,i.push(a)}return i}var sn=null,An=null;function ln(t,e){var n=ht(t,e);if(n.length){var r,i=Array.prototype.slice.call(arguments,2);sn?r=sn.delayedCallbacks:An?r=An:(r=An=[],setTimeout(cn,0));for(var o=function(t){r.push(function(){return n[t].apply(null,i)})},a=0;a<n.length;++a)o(a)}}function cn(){var t=An;An=null;for(var e=0;e<t.length;++e)t[e]()}function un(t,e,n,r){for(var i=0;i<e.changes.length;i++){var o=e.changes[i];"text"==o?dn(t,e):"gutter"==o?pn(t,e,n,r):"class"==o?fn(t,e):"widget"==o&&In(t,e,r)}e.changes=null}function gn(t){return t.node==t.text&&(t.node=D("div",null,null,"position: relative"),t.text.parentNode&&t.text.parentNode.replaceChild(t.node,t.text),t.node.appendChild(t.text),a&&s<8&&(t.node.style.zIndex=2)),t.node}function hn(t,e){var n=t.display.externalMeasured;return n&&n.line==e.line?(t.display.externalMeasured=null,e.measure=n.measure,n.built):Ze(t,e)}function dn(t,e){var n=e.text.className,r=hn(t,e);e.text==e.node&&(e.node=r.pre),e.text.parentNode.replaceChild(r.pre,e.text),e.text=r.pre,r.bgClass!=e.bgClass||r.textClass!=e.textClass?(e.bgClass=r.bgClass,e.textClass=r.textClass,fn(t,e)):n&&(e.text.className=n)}function fn(t,e){!function(t,e){var n=e.bgClass?e.bgClass+" "+(e.line.bgClass||""):e.line.bgClass;if(n&&(n+=" CodeMirror-linebackground"),e.background)n?e.background.className=n:(e.background.parentNode.removeChild(e.background),e.background=null);else if(n){var r=gn(e);e.background=r.insertBefore(D("div",null,n),r.firstChild),t.display.input.setUneditable(e.background)}}(t,e),e.line.wrapClass?gn(e).className=e.line.wrapClass:e.node!=e.text&&(e.node.className="");var n=e.textClass?e.textClass+" "+(e.line.textClass||""):e.line.textClass;e.text.className=n||""}function pn(t,e,n,r){if(e.gutter&&(e.node.removeChild(e.gutter),e.gutter=null),e.gutterBackground&&(e.node.removeChild(e.gutterBackground),e.gutterBackground=null),e.line.gutterClass){var i=gn(e);e.gutterBackground=D("div",null,"CodeMirror-gutter-background "+e.line.gutterClass,"left: "+(t.options.fixedGutter?r.fixedPos:-r.gutterTotalWidth)+"px; width: "+r.gutterTotalWidth+"px"),t.display.input.setUneditable(e.gutterBackground),i.insertBefore(e.gutterBackground,e.text)}var o=e.line.gutterMarkers;if(t.options.lineNumbers||o){var a=gn(e),s=e.gutter=D("div",null,"CodeMirror-gutter-wrapper","left: "+(t.options.fixedGutter?r.fixedPos:-r.gutterTotalWidth)+"px");if(t.display.input.setUneditable(s),a.insertBefore(s,e.text),e.line.gutterClass&&(s.className+=" "+e.line.gutterClass),!t.options.lineNumbers||o&&o["CodeMirror-linenumbers"]||(e.lineNumber=s.appendChild(D("div",te(t.options,n),"CodeMirror-linenumber CodeMirror-gutter-elt","left: "+r.gutterLeft["CodeMirror-linenumbers"]+"px; width: "+t.display.lineNumInnerWidth+"px"))),o)for(var A=0;A<t.display.gutterSpecs.length;++A){var l=t.display.gutterSpecs[A].className,c=o.hasOwnProperty(l)&&o[l];c&&s.appendChild(D("div",[c],"CodeMirror-gutter-elt","left: "+r.gutterLeft[l]+"px; width: "+r.gutterWidth[l]+"px"))}}}function In(t,e,n){e.alignable&&(e.alignable=null);for(var r=e.node.firstChild,i=void 0;r;r=i)i=r.nextSibling,"CodeMirror-linewidget"==r.className&&e.node.removeChild(r);mn(t,e,n)}function Cn(t,e,n,r){var i=hn(t,e);return e.text=e.node=i.pre,i.bgClass&&(e.bgClass=i.bgClass),i.textClass&&(e.textClass=i.textClass),fn(t,e),pn(t,e,n,r),mn(t,e,r),e.node}function mn(t,e,n){if(yn(t,e.line,e,n,!0),e.rest)for(var r=0;r<e.rest.length;r++)yn(t,e.rest[r],e,n,!1)}function yn(t,e,n,r,i){if(e.widgets)for(var o=gn(n),a=0,s=e.widgets;a<s.length;++a){var A=s[a],l=D("div",[A.node],"CodeMirror-linewidget");A.handleMouseEvents||l.setAttribute("cm-ignore-events","true"),En(A,l,n,r),t.display.input.setUneditable(l),i&&A.above?o.insertBefore(l,n.gutter||n.text):o.appendChild(l),ln(A,"redraw")}}function En(t,e,n,r){if(t.noHScroll){(n.alignable||(n.alignable=[])).push(e);var i=r.wrapperWidth;e.style.left=r.fixedPos+"px",t.coverGutter||(i-=r.gutterTotalWidth,e.style.paddingLeft=r.gutterTotalWidth+"px"),e.style.width=i+"px"}t.coverGutter&&(e.style.zIndex=5,e.style.position="relative",t.noHScroll||(e.style.marginLeft=-r.gutterTotalWidth+"px"))}function vn(t){if(null!=t.height)return t.height;var e=t.doc.cm;if(!e)return 0;if(!k(document.body,t.node)){var n="position: relative;";t.coverGutter&&(n+="margin-left: -"+e.display.gutters.offsetWidth+"px;"),t.noHScroll&&(n+="width: "+e.display.wrapper.clientWidth+"px;"),_(e.display.measure,D("div",[t.node],null,n))}return t.height=t.node.parentNode.offsetHeight}function Bn(t,e){for(var n=bt(e);n!=t.wrapper;n=n.parentNode)if(!n||1==n.nodeType&&"true"==n.getAttribute("cm-ignore-events")||n.parentNode==t.sizer&&n!=t.mover)return!0}function bn(t){return t.lineSpace.offsetTop}function Qn(t){return t.mover.offsetHeight-t.lineSpace.offsetHeight}function wn(t){if(t.cachedPaddingH)return t.cachedPaddingH;var e=_(t.measure,D("pre","x")),n=window.getComputedStyle?window.getComputedStyle(e):e.currentStyle,r={left:parseInt(n.paddingLeft),right:parseInt(n.paddingRight)};return isNaN(r.left)||isNaN(r.right)||(t.cachedPaddingH=r),r}function xn(t){return H-t.display.nativeBarWidth}function _n(t){return t.display.scroller.clientWidth-xn(t)-t.display.barWidth}function Dn(t){return t.display.scroller.clientHeight-xn(t)-t.display.barHeight}function Mn(t,e,n){if(t.line==e)return{map:t.measure.map,cache:t.measure.cache};for(var r=0;r<t.rest.length;r++)if(t.rest[r]==e)return{map:t.measure.maps[r],cache:t.measure.caches[r]};for(var i=0;i<t.rest.length;i++)if(Xt(t.rest[i])>n)return{map:t.measure.maps[i],cache:t.measure.caches[i],before:!0}}function kn(t,e,n,r){return Rn(t,Ln(t,e),n,r)}function Sn(t,e){if(e>=t.display.viewFrom&&e<t.display.viewTo)return t.display.view[lr(t,e)];var n=t.display.externalMeasured;return n&&e>=n.lineN&&e<n.lineN+n.size?n:void 0}function Ln(t,e){var n=Xt(e),r=Sn(t,n);r&&!r.text?r=null:r&&r.changes&&(un(t,r,n,ir(t)),t.curOp.forceUpdate=!0),r||(r=function(t,e){var n=Xt(e=Oe(e)),r=t.display.externalMeasured=new on(t.doc,e,n);r.lineN=n;var i=r.built=Ze(t,r);return r.text=i.pre,_(t.display.lineMeasure,i.pre),r}(t,e));var i=Mn(r,e,n);return{line:e,view:r,rect:null,map:i.map,cache:i.cache,before:i.before,hasHeights:!1}}function Rn(t,e,n,r,i){e.before&&(n=-1);var o,A=n+(r||"");return e.cache.hasOwnProperty(A)?o=e.cache[A]:(e.rect||(e.rect=e.view.text.getBoundingClientRect()),e.hasHeights||(function(t,e,n){var r=t.options.lineWrapping,i=r&&_n(t);if(!e.measure.heights||r&&e.measure.width!=i){var o=e.measure.heights=[];if(r){e.measure.width=i;for(var a=e.text.firstChild.getClientRects(),s=0;s<a.length-1;s++){var A=a[s],l=a[s+1];Math.abs(A.bottom-l.bottom)>2&&o.push((A.bottom+l.top)/2-n.top)}}o.push(n.bottom-n.top)}}(t,e.view,e.rect),e.hasHeights=!0),(o=function(t,e,n,r){var i,o=Tn(e.map,n,r),A=o.node,l=o.start,c=o.end,u=o.collapse;if(3==A.nodeType){for(var g=0;g<4;g++){for(;l&&it(e.line.text.charAt(o.coverStart+l));)--l;for(;o.coverStart+c<o.coverEnd&&it(e.line.text.charAt(o.coverStart+c));)++c;if((i=a&&s<9&&0==l&&c==o.coverEnd-o.coverStart?A.parentNode.getBoundingClientRect():Un(Q(A,l,c).getClientRects(),r)).left||i.right||0==l)break;c=l,l-=1,u="right"}a&&s<11&&(i=function(t,e){if(!window.screen||null==screen.logicalXDPI||screen.logicalXDPI==screen.deviceXDPI||!function(t){if(null!=Nt)return Nt;var e=_(t,D("span","x")),n=e.getBoundingClientRect(),r=Q(e,0,1).getBoundingClientRect();return Nt=Math.abs(n.left-r.left)>1}(t))return e;var n=screen.logicalXDPI/screen.deviceXDPI,r=screen.logicalYDPI/screen.deviceYDPI;return{left:e.left*n,right:e.right*n,top:e.top*r,bottom:e.bottom*r}}(t.display.measure,i))}else{var h;l>0&&(u=r="right"),i=t.options.lineWrapping&&(h=A.getClientRects()).length>1?h["right"==r?h.length-1:0]:A.getBoundingClientRect()}if(a&&s<9&&!l&&(!i||!i.left&&!i.right)){var d=A.parentNode.getClientRects()[0];i=d?{left:d.left,right:d.left+rr(t.display),top:d.top,bottom:d.bottom}:Fn}for(var f=i.top-e.rect.top,p=i.bottom-e.rect.top,I=(f+p)/2,C=e.view.measure.heights,m=0;m<C.length-1&&!(I<C[m]);m++);var y=m?C[m-1]:0,E=C[m],v={left:("right"==u?i.right:i.left)-e.rect.left,right:("left"==u?i.left:i.right)-e.rect.left,top:y,bottom:E};return i.left||i.right||(v.bogus=!0),t.options.singleCursorHeightPerLine||(v.rtop=f,v.rbottom=p),v}(t,e,n,r)).bogus||(e.cache[A]=o)),{left:o.left,right:o.right,top:i?o.rtop:o.top,bottom:i?o.rbottom:o.bottom}}var Nn,Fn={left:0,right:0,top:0,bottom:0};function Tn(t,e,n){for(var r,i,o,a,s,A,l=0;l<t.length;l+=3)if(s=t[l],A=t[l+1],e<s?(i=0,o=1,a="left"):e<A?o=1+(i=e-s):(l==t.length-3||e==A&&t[l+3]>e)&&(i=(o=A-s)-1,e>=A&&(a="right")),null!=i){if(r=t[l+2],s==A&&n==(r.insertLeft?"left":"right")&&(a=n),"left"==n&&0==i)for(;l&&t[l-2]==t[l-3]&&t[l-1].insertLeft;)r=t[2+(l-=3)],a="left";if("right"==n&&i==A-s)for(;l<t.length-3&&t[l+3]==t[l+4]&&!t[l+5].insertLeft;)r=t[(l+=3)+2],a="right";break}return{node:r,start:i,end:o,collapse:a,coverStart:s,coverEnd:A}}function Un(t,e){var n=Fn;if("left"==e)for(var r=0;r<t.length&&(n=t[r]).left==n.right;r++);else for(var i=t.length-1;i>=0&&(n=t[i]).left==n.right;i--);return n}function On(t){if(t.measure&&(t.measure.cache={},t.measure.heights=null,t.rest))for(var e=0;e<t.rest.length;e++)t.measure.caches[e]={}}function Gn(t){t.display.externalMeasure=null,x(t.display.lineMeasure);for(var e=0;e<t.display.view.length;e++)On(t.display.view[e])}function Hn(t){Gn(t),t.display.cachedCharWidth=t.display.cachedTextHeight=t.display.cachedPaddingH=null,t.options.lineWrapping||(t.display.maxLineChanged=!0),t.display.lineNumChars=null}function jn(){return c&&p?-(document.body.getBoundingClientRect().left-parseInt(getComputedStyle(document.body).marginLeft)):window.pageXOffset||(document.documentElement||document.body).scrollLeft}function Yn(){return c&&p?-(document.body.getBoundingClientRect().top-parseInt(getComputedStyle(document.body).marginTop)):window.pageYOffset||(document.documentElement||document.body).scrollTop}function Pn(t){var e=0;if(t.widgets)for(var n=0;n<t.widgets.length;++n)t.widgets[n].above&&(e+=vn(t.widgets[n]));return e}function Kn(t,e,n,r,i){if(!i){var o=Pn(e);n.top+=o,n.bottom+=o}if("line"==r)return n;r||(r="local");var a=Pe(e);if("local"==r?a+=bn(t.display):a-=t.display.viewOffset,"page"==r||"window"==r){var s=t.display.lineSpace.getBoundingClientRect();a+=s.top+("window"==r?0:Yn());var A=s.left+("window"==r?0:jn());n.left+=A,n.right+=A}return n.top+=a,n.bottom+=a,n}function zn(t,e,n){if("div"==n)return e;var r=e.left,i=e.top;if("page"==n)r-=jn(),i-=Yn();else if("local"==n||!n){var o=t.display.sizer.getBoundingClientRect();r+=o.left,i+=o.top}var a=t.display.lineSpace.getBoundingClientRect();return{left:r-a.left,top:i-a.top}}function Jn(t,e,n,r,i){return r||(r=Jt(t.doc,e.line)),Kn(t,r,kn(t,r,e.ch,i),n)}function qn(t,e,n,r,i,o){function a(e,a){var s=Rn(t,i,e,a?"right":"left",o);return a?s.left=s.right:s.right=s.left,Kn(t,r,s,n)}r=r||Jt(t.doc,e.line),i||(i=Ln(t,r));var s=ct(r,t.doc.direction),A=e.ch,l=e.sticky;if(A>=r.text.length?(A=r.text.length,l="before"):A<=0&&(A=0,l="after"),!s)return a("before"==l?A-1:A,"before"==l);function c(t,e,n){var r=s[e],i=1==r.level;return a(n?t-1:t,i!=n)}var u=At(s,A,l),g=st,h=c(A,u,"before"==l);return null!=g&&(h.other=c(A,g,"before"!=l)),h}function Vn(t,e){var n=0;e=Ae(t.doc,e),t.options.lineWrapping||(n=rr(t.display)*e.ch);var r=Jt(t.doc,e.line),i=Pe(r)+bn(t.display);return{left:n,right:n,top:i,bottom:i+r.height}}function Wn(t,e,n,r,i){var o=ee(t,e,n);return o.xRel=i,r&&(o.outside=!0),o}function Xn(t,e,n){var r=t.doc;if((n+=t.display.viewOffset)<0)return Wn(r.first,0,null,!0,-1);var i=Zt(r,n),o=r.first+r.size-1;if(i>o)return Wn(r.first+r.size-1,Jt(r,o).text.length,null,!0,1);e<0&&(e=0);for(var a=Jt(r,i);;){var s=er(t,a,i,e,n),A=Te(a,s.ch+(s.xRel>0?1:0));if(!A)return s;var l=A.find(1);if(l.line==i)return l;a=Jt(r,i=l.line)}}function Zn(t,e,n,r){r-=Pn(e);var i=e.text.length,o=at(function(e){return Rn(t,n,e-1).bottom<=r},i,0);return i=at(function(e){return Rn(t,n,e).top>r},o,i),{begin:o,end:i}}function $n(t,e,n,r){n||(n=Ln(t,e));var i=Kn(t,e,Rn(t,n,r),"line").top;return Zn(t,e,n,i)}function tr(t,e,n,r){return!(t.bottom<=n)&&(t.top>n||(r?t.left:t.right)>e)}function er(t,e,n,r,i){i-=Pe(e);var o=Ln(t,e),a=Pn(e),s=0,A=e.text.length,l=!0,c=ct(e,t.doc.direction);if(c){var u=(t.options.lineWrapping?function(t,e,n,r,i,o,a){var s=Zn(t,e,r,a),A=s.begin,l=s.end;/\s/.test(e.text.charAt(l-1))&&l--;for(var c=null,u=null,g=0;g<i.length;g++){var h=i[g];if(!(h.from>=l||h.to<=A)){var d=1!=h.level,f=Rn(t,r,d?Math.min(l,h.to)-1:Math.max(A,h.from)).right,p=f<o?o-f+1e9:f-o;(!c||u>p)&&(c=h,u=p)}}return c||(c=i[i.length-1]),c.from<A&&(c={from:A,to:c.to,level:c.level}),c.to>l&&(c={from:c.from,to:l,level:c.level}),c}:function(t,e,n,r,i,o,a){var s=at(function(s){var A=i[s],l=1!=A.level;return tr(qn(t,ee(n,l?A.to:A.from,l?"before":"after"),"line",e,r),o,a,!0)},0,i.length-1),A=i[s];if(s>0){var l=1!=A.level,c=qn(t,ee(n,l?A.from:A.to,l?"after":"before"),"line",e,r);tr(c,o,a,!0)&&c.top>a&&(A=i[s-1])}return A})(t,e,n,o,c,r,i);l=1!=u.level,s=l?u.from:u.to-1,A=l?u.to:u.from-1}var g,h,d=null,f=null,p=at(function(e){var n=Rn(t,o,e);return n.top+=a,n.bottom+=a,!!tr(n,r,i,!1)&&(n.top<=i&&n.left<=r&&(d=e,f=n),!0)},s,A),I=!1;if(f){var C=r-f.left<f.right-r,m=C==l;p=d+(m?0:1),h=m?"after":"before",g=C?f.left:f.right}else{l||p!=A&&p!=s||p++,h=0==p?"after":p==e.text.length?"before":Rn(t,o,p-(l?1:0)).bottom+a<=i==l?"after":"before";var y=qn(t,ee(n,p,h),"line",e,o);g=y.left,I=i<y.top||i>=y.bottom}return p=ot(e.text,p,1),Wn(n,p,h,I,r-g)}function nr(t){if(null!=t.cachedTextHeight)return t.cachedTextHeight;if(null==Nn){Nn=D("pre");for(var e=0;e<49;++e)Nn.appendChild(document.createTextNode("x")),Nn.appendChild(D("br"));Nn.appendChild(document.createTextNode("x"))}_(t.measure,Nn);var n=Nn.offsetHeight/50;return n>3&&(t.cachedTextHeight=n),x(t.measure),n||1}function rr(t){if(null!=t.cachedCharWidth)return t.cachedCharWidth;var e=D("span","xxxxxxxxxx"),n=D("pre",[e]);_(t.measure,n);var r=e.getBoundingClientRect(),i=(r.right-r.left)/10;return i>2&&(t.cachedCharWidth=i),i||10}function ir(t){for(var e=t.display,n={},r={},i=e.gutters.clientLeft,o=e.gutters.firstChild,a=0;o;o=o.nextSibling,++a){var s=t.display.gutterSpecs[a].className;n[s]=o.offsetLeft+o.clientLeft+i,r[s]=o.clientWidth}return{fixedPos:or(e),gutterTotalWidth:e.gutters.offsetWidth,gutterLeft:n,gutterWidth:r,wrapperWidth:e.wrapper.clientWidth}}function or(t){return t.scroller.getBoundingClientRect().left-t.sizer.getBoundingClientRect().left}function ar(t){var e=nr(t.display),n=t.options.lineWrapping,r=n&&Math.max(5,t.display.scroller.clientWidth/rr(t.display)-3);return function(i){if(je(t.doc,i))return 0;var o=0;if(i.widgets)for(var a=0;a<i.widgets.length;a++)i.widgets[a].height&&(o+=i.widgets[a].height);return n?o+(Math.ceil(i.text.length/r)||1)*e:o+e}}function sr(t){var e=t.doc,n=ar(t);e.iter(function(t){var e=n(t);e!=t.height&&Wt(t,e)})}function Ar(t,e,n,r){var i=t.display;if(!n&&"true"==bt(e).getAttribute("cm-not-content"))return null;var o,a,s=i.lineSpace.getBoundingClientRect();try{o=e.clientX-s.left,a=e.clientY-s.top}catch(e){return null}var A,l=Xn(t,o,a);if(r&&1==l.xRel&&(A=Jt(t.doc,l.line).text).length==l.ch){var c=U(A,A.length,t.options.tabSize)-A.length;l=ee(l.line,Math.max(0,Math.round((o-wn(t.display).left)/rr(t.display))-c))}return l}function lr(t,e){if(e>=t.display.viewTo)return null;if((e-=t.display.viewFrom)<0)return null;for(var n=t.display.view,r=0;r<n.length;r++)if((e-=n[r].size)<0)return r}function cr(t,e,n,r){null==e&&(e=t.doc.first),null==n&&(n=t.doc.first+t.doc.size),r||(r=0);var i=t.display;if(r&&n<i.viewTo&&(null==i.updateLineNumbers||i.updateLineNumbers>e)&&(i.updateLineNumbers=e),t.curOp.viewChanged=!0,e>=i.viewTo)Be&&Ge(t.doc,e)<i.viewTo&&gr(t);else if(n<=i.viewFrom)Be&&He(t.doc,n+r)>i.viewFrom?gr(t):(i.viewFrom+=r,i.viewTo+=r);else if(e<=i.viewFrom&&n>=i.viewTo)gr(t);else if(e<=i.viewFrom){var o=hr(t,n,n+r,1);o?(i.view=i.view.slice(o.index),i.viewFrom=o.lineN,i.viewTo+=r):gr(t)}else if(n>=i.viewTo){var a=hr(t,e,e,-1);a?(i.view=i.view.slice(0,a.index),i.viewTo=a.lineN):gr(t)}else{var s=hr(t,e,e,-1),A=hr(t,n,n+r,1);s&&A?(i.view=i.view.slice(0,s.index).concat(an(t,s.lineN,A.lineN)).concat(i.view.slice(A.index)),i.viewTo+=r):gr(t)}var l=i.externalMeasured;l&&(n<l.lineN?l.lineN+=r:e<l.lineN+l.size&&(i.externalMeasured=null))}function ur(t,e,n){t.curOp.viewChanged=!0;var r=t.display,i=t.display.externalMeasured;if(i&&e>=i.lineN&&e<i.lineN+i.size&&(r.externalMeasured=null),!(e<r.viewFrom||e>=r.viewTo)){var o=r.view[lr(t,e)];if(null!=o.node){var a=o.changes||(o.changes=[]);-1==G(a,n)&&a.push(n)}}}function gr(t){t.display.viewFrom=t.display.viewTo=t.doc.first,t.display.view=[],t.display.viewOffset=0}function hr(t,e,n,r){var i,o=lr(t,e),a=t.display.view;if(!Be||n==t.doc.first+t.doc.size)return{index:o,lineN:n};for(var s=t.display.viewFrom,A=0;A<o;A++)s+=a[A].size;if(s!=e){if(r>0){if(o==a.length-1)return null;i=s+a[o].size-e,o++}else i=s-e;e+=i,n+=i}for(;Ge(t.doc,n)!=n;){if(o==(r<0?0:a.length-1))return null;n+=r*a[o-(r<0?1:0)].size,o+=r}return{index:o,lineN:n}}function dr(t){for(var e=t.display.view,n=0,r=0;r<e.length;r++){var i=e[r];i.hidden||i.node&&!i.changes||++n}return n}function fr(t){t.display.input.showSelection(t.display.input.prepareSelection())}function pr(t,e){void 0===e&&(e=!0);for(var n=t.doc,r={},i=r.cursors=document.createDocumentFragment(),o=r.selection=document.createDocumentFragment(),a=0;a<n.sel.ranges.length;a++)if(e||a!=n.sel.primIndex){var s=n.sel.ranges[a];if(!(s.from().line>=t.display.viewTo||s.to().line<t.display.viewFrom)){var A=s.empty();(A||t.options.showCursorWhenSelecting)&&Ir(t,s.head,i),A||mr(t,s,o)}}return r}function Ir(t,e,n){var r=qn(t,e,"div",null,null,!t.options.singleCursorHeightPerLine),i=n.appendChild(D("div"," ","CodeMirror-cursor"));if(i.style.left=r.left+"px",i.style.top=r.top+"px",i.style.height=Math.max(0,r.bottom-r.top)*t.options.cursorHeight+"px",r.other){var o=n.appendChild(D("div"," ","CodeMirror-cursor CodeMirror-secondarycursor"));o.style.display="",o.style.left=r.other.left+"px",o.style.top=r.other.top+"px",o.style.height=.85*(r.other.bottom-r.other.top)+"px"}}function Cr(t,e){return t.top-e.top||t.left-e.left}function mr(t,e,n){var r=t.display,i=t.doc,o=document.createDocumentFragment(),a=wn(t.display),s=a.left,A=Math.max(r.sizerWidth,_n(t)-r.sizer.offsetLeft)-a.right,l="ltr"==i.direction;function c(t,e,n,r){e<0&&(e=0),e=Math.round(e),r=Math.round(r),o.appendChild(D("div",null,"CodeMirror-selected","position: absolute; left: "+t+"px;\n                             top: "+e+"px; width: "+(null==n?A-t:n)+"px;\n                             height: "+(r-e)+"px"))}function u(e,n,r){var o,a,u=Jt(i,e),g=u.text.length;function h(n,r){return Jn(t,ee(e,n),"div",u,r)}function d(e,n,r){var i=$n(t,u,null,e),o="ltr"==n==("after"==r)?"left":"right",a="after"==r?i.begin:i.end-(/\s/.test(u.text.charAt(i.end-1))?2:1);return h(a,o)[o]}var f=ct(u,i.direction);return function(t,e,n,r){if(!t)return r(e,n,"ltr",0);for(var i=!1,o=0;o<t.length;++o){var a=t[o];(a.from<n&&a.to>e||e==n&&a.to==e)&&(r(Math.max(a.from,e),Math.min(a.to,n),1==a.level?"rtl":"ltr",o),i=!0)}i||r(e,n,"ltr")}(f,n||0,null==r?g:r,function(t,e,i,u){var p="ltr"==i,I=h(t,p?"left":"right"),C=h(e-1,p?"right":"left"),m=null==n&&0==t,y=null==r&&e==g,E=0==u,v=!f||u==f.length-1;if(C.top-I.top<=3){var B=(l?m:y)&&E,b=(l?y:m)&&v,Q=B?s:(p?I:C).left,w=b?A:(p?C:I).right;c(Q,I.top,w-Q,I.bottom)}else{var x,_,D,M;p?(x=l&&m&&E?s:I.left,_=l?A:d(t,i,"before"),D=l?s:d(e,i,"after"),M=l&&y&&v?A:C.right):(x=l?d(t,i,"before"):s,_=!l&&m&&E?A:I.right,D=!l&&y&&v?s:C.left,M=l?d(e,i,"after"):A),c(x,I.top,_-x,I.bottom),I.bottom<C.top&&c(s,I.bottom,null,C.top),c(D,C.top,M-D,C.bottom)}(!o||Cr(I,o)<0)&&(o=I),Cr(C,o)<0&&(o=C),(!a||Cr(I,a)<0)&&(a=I),Cr(C,a)<0&&(a=C)}),{start:o,end:a}}var g=e.from(),h=e.to();if(g.line==h.line)u(g.line,g.ch,h.ch);else{var d=Jt(i,g.line),f=Jt(i,h.line),p=Oe(d)==Oe(f),I=u(g.line,g.ch,p?d.text.length+1:null).end,C=u(h.line,p?0:null,h.ch).start;p&&(I.top<C.top-2?(c(I.right,I.top,null,I.bottom),c(s,C.top,C.left,C.bottom)):c(I.right,I.top,C.left-I.right,I.bottom)),I.bottom<C.top&&c(s,I.bottom,null,C.top)}n.appendChild(o)}function yr(t){if(t.state.focused){var e=t.display;clearInterval(e.blinker);var n=!0;e.cursorDiv.style.visibility="",t.options.cursorBlinkRate>0?e.blinker=setInterval(function(){return e.cursorDiv.style.visibility=(n=!n)?"":"hidden"},t.options.cursorBlinkRate):t.options.cursorBlinkRate<0&&(e.cursorDiv.style.visibility="hidden")}}function Er(t){t.state.focused||(t.display.input.focus(),Br(t))}function vr(t){t.state.delayingBlurEvent=!0,setTimeout(function(){t.state.delayingBlurEvent&&(t.state.delayingBlurEvent=!1,br(t))},100)}function Br(t,e){t.state.delayingBlurEvent&&(t.state.delayingBlurEvent=!1),"nocursor"!=t.options.readOnly&&(t.state.focused||(ft(t,"focus",t,e),t.state.focused=!0,L(t.display.wrapper,"CodeMirror-focused"),t.curOp||t.display.selForContextMenu==t.doc.sel||(t.display.input.reset(),A&&setTimeout(function(){return t.display.input.reset(!0)},20)),t.display.input.receivedFocus()),yr(t))}function br(t,e){t.state.delayingBlurEvent||(t.state.focused&&(ft(t,"blur",t,e),t.state.focused=!1,w(t.display.wrapper,"CodeMirror-focused")),clearInterval(t.display.blinker),setTimeout(function(){t.state.focused||(t.display.shift=!1)},150))}function Qr(t){for(var e=t.display,n=e.lineDiv.offsetTop,r=0;r<e.view.length;r++){var i=e.view[r],o=t.options.lineWrapping,A=void 0,l=0;if(!i.hidden){if(a&&s<8){var c=i.node.offsetTop+i.node.offsetHeight;A=c-n,n=c}else{var u=i.node.getBoundingClientRect();A=u.bottom-u.top,!o&&i.text.firstChild&&(l=i.text.firstChild.getBoundingClientRect().right-u.left-1)}var g=i.line.height-A;if((g>.005||g<-.005)&&(Wt(i.line,A),wr(i.line),i.rest))for(var h=0;h<i.rest.length;h++)wr(i.rest[h]);if(l>t.display.sizerWidth){var d=Math.ceil(l/rr(t.display));d>t.display.maxLineLength&&(t.display.maxLineLength=d,t.display.maxLine=i.line,t.display.maxLineChanged=!0)}}}}function wr(t){if(t.widgets)for(var e=0;e<t.widgets.length;++e){var n=t.widgets[e],r=n.node.parentNode;r&&(n.height=r.offsetHeight)}}function xr(t,e,n){var r=n&&null!=n.top?Math.max(0,n.top):t.scroller.scrollTop;r=Math.floor(r-bn(t));var i=n&&null!=n.bottom?n.bottom:r+t.wrapper.clientHeight,o=Zt(e,r),a=Zt(e,i);if(n&&n.ensure){var s=n.ensure.from.line,A=n.ensure.to.line;s<o?(o=s,a=Zt(e,Pe(Jt(e,s))+t.wrapper.clientHeight)):Math.min(A,e.lastLine())>=a&&(o=Zt(e,Pe(Jt(e,A))-t.wrapper.clientHeight),a=A)}return{from:o,to:Math.max(a,o+1)}}function _r(t,e){var n=t.display,r=nr(t.display);e.top<0&&(e.top=0);var i=t.curOp&&null!=t.curOp.scrollTop?t.curOp.scrollTop:n.scroller.scrollTop,o=Dn(t),a={};e.bottom-e.top>o&&(e.bottom=e.top+o);var s=t.doc.height+Qn(n),A=e.top<r,l=e.bottom>s-r;if(e.top<i)a.scrollTop=A?0:e.top;else if(e.bottom>i+o){var c=Math.min(e.top,(l?s:e.bottom)-o);c!=i&&(a.scrollTop=c)}var u=t.curOp&&null!=t.curOp.scrollLeft?t.curOp.scrollLeft:n.scroller.scrollLeft,g=_n(t)-(t.options.fixedGutter?n.gutters.offsetWidth:0),h=e.right-e.left>g;return h&&(e.right=e.left+g),e.left<10?a.scrollLeft=0:e.left<u?a.scrollLeft=Math.max(0,e.left-(h?0:10)):e.right>g+u-3&&(a.scrollLeft=e.right+(h?0:10)-g),a}function Dr(t,e){null!=e&&(Sr(t),t.curOp.scrollTop=(null==t.curOp.scrollTop?t.doc.scrollTop:t.curOp.scrollTop)+e)}function Mr(t){Sr(t);var e=t.getCursor();t.curOp.scrollToPos={from:e,to:e,margin:t.options.cursorScrollMargin}}function kr(t,e,n){null==e&&null==n||Sr(t),null!=e&&(t.curOp.scrollLeft=e),null!=n&&(t.curOp.scrollTop=n)}function Sr(t){var e=t.curOp.scrollToPos;if(e){t.curOp.scrollToPos=null;var n=Vn(t,e.from),r=Vn(t,e.to);Lr(t,n,r,e.margin)}}function Lr(t,e,n,r){var i=_r(t,{left:Math.min(e.left,n.left),top:Math.min(e.top,n.top)-r,right:Math.max(e.right,n.right),bottom:Math.max(e.bottom,n.bottom)+r});kr(t,i.scrollLeft,i.scrollTop)}function Rr(t,e){Math.abs(t.doc.scrollTop-e)<2||(n||ai(t,{top:e}),Nr(t,e,!0),n&&ai(t),ei(t,100))}function Nr(t,e,n){e=Math.min(t.display.scroller.scrollHeight-t.display.scroller.clientHeight,e),(t.display.scroller.scrollTop!=e||n)&&(t.doc.scrollTop=e,t.display.scrollbars.setScrollTop(e),t.display.scroller.scrollTop!=e&&(t.display.scroller.scrollTop=e))}function Fr(t,e,n,r){e=Math.min(e,t.display.scroller.scrollWidth-t.display.scroller.clientWidth),(n?e==t.doc.scrollLeft:Math.abs(t.doc.scrollLeft-e)<2)&&!r||(t.doc.scrollLeft=e,li(t),t.display.scroller.scrollLeft!=e&&(t.display.scroller.scrollLeft=e),t.display.scrollbars.setScrollLeft(e))}function Tr(t){var e=t.display,n=e.gutters.offsetWidth,r=Math.round(t.doc.height+Qn(t.display));return{clientHeight:e.scroller.clientHeight,viewHeight:e.wrapper.clientHeight,scrollWidth:e.scroller.scrollWidth,clientWidth:e.scroller.clientWidth,viewWidth:e.wrapper.clientWidth,barLeft:t.options.fixedGutter?n:0,docHeight:r,scrollHeight:r+xn(t)+e.barHeight,nativeBarWidth:e.nativeBarWidth,gutterWidth:n}}var Ur=function(t,e,n){this.cm=n;var r=this.vert=D("div",[D("div",null,null,"min-width: 1px")],"CodeMirror-vscrollbar"),i=this.horiz=D("div",[D("div",null,null,"height: 100%; min-height: 1px")],"CodeMirror-hscrollbar");r.tabIndex=i.tabIndex=-1,t(r),t(i),gt(r,"scroll",function(){r.clientHeight&&e(r.scrollTop,"vertical")}),gt(i,"scroll",function(){i.clientWidth&&e(i.scrollLeft,"horizontal")}),this.checkedZeroWidth=!1,a&&s<8&&(this.horiz.style.minHeight=this.vert.style.minWidth="18px")};Ur.prototype.update=function(t){var e=t.scrollWidth>t.clientWidth+1,n=t.scrollHeight>t.clientHeight+1,r=t.nativeBarWidth;if(n){this.vert.style.display="block",this.vert.style.bottom=e?r+"px":"0";var i=t.viewHeight-(e?r:0);this.vert.firstChild.style.height=Math.max(0,t.scrollHeight-t.clientHeight+i)+"px"}else this.vert.style.display="",this.vert.firstChild.style.height="0";if(e){this.horiz.style.display="block",this.horiz.style.right=n?r+"px":"0",this.horiz.style.left=t.barLeft+"px";var o=t.viewWidth-t.barLeft-(n?r:0);this.horiz.firstChild.style.width=Math.max(0,t.scrollWidth-t.clientWidth+o)+"px"}else this.horiz.style.display="",this.horiz.firstChild.style.width="0";return!this.checkedZeroWidth&&t.clientHeight>0&&(0==r&&this.zeroWidthHack(),this.checkedZeroWidth=!0),{right:n?r:0,bottom:e?r:0}},Ur.prototype.setScrollLeft=function(t){this.horiz.scrollLeft!=t&&(this.horiz.scrollLeft=t),this.disableHoriz&&this.enableZeroWidthBar(this.horiz,this.disableHoriz,"horiz")},Ur.prototype.setScrollTop=function(t){this.vert.scrollTop!=t&&(this.vert.scrollTop=t),this.disableVert&&this.enableZeroWidthBar(this.vert,this.disableVert,"vert")},Ur.prototype.zeroWidthHack=function(){var t=C&&!h?"12px":"18px";this.horiz.style.height=this.vert.style.width=t,this.horiz.style.pointerEvents=this.vert.style.pointerEvents="none",this.disableHoriz=new O,this.disableVert=new O},Ur.prototype.enableZeroWidthBar=function(t,e,n){t.style.pointerEvents="auto",e.set(1e3,function r(){var i=t.getBoundingClientRect(),o="vert"==n?document.elementFromPoint(i.right-1,(i.top+i.bottom)/2):document.elementFromPoint((i.right+i.left)/2,i.bottom-1);o!=t?t.style.pointerEvents="none":e.set(1e3,r)})},Ur.prototype.clear=function(){var t=this.horiz.parentNode;t.removeChild(this.horiz),t.removeChild(this.vert)};var Or=function(){};function Gr(t,e){e||(e=Tr(t));var n=t.display.barWidth,r=t.display.barHeight;Hr(t,e);for(var i=0;i<4&&n!=t.display.barWidth||r!=t.display.barHeight;i++)n!=t.display.barWidth&&t.options.lineWrapping&&Qr(t),Hr(t,Tr(t)),n=t.display.barWidth,r=t.display.barHeight}function Hr(t,e){var n=t.display,r=n.scrollbars.update(e);n.sizer.style.paddingRight=(n.barWidth=r.right)+"px",n.sizer.style.paddingBottom=(n.barHeight=r.bottom)+"px",n.heightForcer.style.borderBottom=r.bottom+"px solid transparent",r.right&&r.bottom?(n.scrollbarFiller.style.display="block",n.scrollbarFiller.style.height=r.bottom+"px",n.scrollbarFiller.style.width=r.right+"px"):n.scrollbarFiller.style.display="",r.bottom&&t.options.coverGutterNextToScrollbar&&t.options.fixedGutter?(n.gutterFiller.style.display="block",n.gutterFiller.style.height=r.bottom+"px",n.gutterFiller.style.width=e.gutterWidth+"px"):n.gutterFiller.style.display=""}Or.prototype.update=function(){return{bottom:0,right:0}},Or.prototype.setScrollLeft=function(){},Or.prototype.setScrollTop=function(){},Or.prototype.clear=function(){};var jr={native:Ur,null:Or};function Yr(t){t.display.scrollbars&&(t.display.scrollbars.clear(),t.display.scrollbars.addClass&&w(t.display.wrapper,t.display.scrollbars.addClass)),t.display.scrollbars=new jr[t.options.scrollbarStyle](function(e){t.display.wrapper.insertBefore(e,t.display.scrollbarFiller),gt(e,"mousedown",function(){t.state.focused&&setTimeout(function(){return t.display.input.focus()},0)}),e.setAttribute("cm-not-content","true")},function(e,n){"horizontal"==n?Fr(t,e):Rr(t,e)},t),t.display.scrollbars.addClass&&L(t.display.wrapper,t.display.scrollbars.addClass)}var Pr=0;function Kr(t){var e;t.curOp={cm:t,viewChanged:!1,startHeight:t.doc.height,forceUpdate:!1,updateInput:0,typing:!1,changeObjs:null,cursorActivityHandlers:null,cursorActivityCalled:0,selectionChanged:!1,updateMaxLine:!1,scrollLeft:null,scrollTop:null,scrollToPos:null,focus:!1,id:++Pr},e=t.curOp,sn?sn.ops.push(e):e.ownsGroup=sn={ops:[e],delayedCallbacks:[]}}function zr(t){var e=t.curOp;e&&function(t,e){var n=t.ownsGroup;if(n)try{!function(t){var e=t.delayedCallbacks,n=0;do{for(;n<e.length;n++)e[n].call(null);for(var r=0;r<t.ops.length;r++){var i=t.ops[r];if(i.cursorActivityHandlers)for(;i.cursorActivityCalled<i.cursorActivityHandlers.length;)i.cursorActivityHandlers[i.cursorActivityCalled++].call(null,i.cm)}}while(n<e.length)}(n)}finally{sn=null,e(n)}}(e,function(t){for(var e=0;e<t.ops.length;e++)t.ops[e].cm.curOp=null;!function(t){for(var e=t.ops,n=0;n<e.length;n++)Jr(e[n]);for(var r=0;r<e.length;r++)(i=e[r]).updatedDisplay=i.mustUpdate&&ii(i.cm,i.update);for(var i,o=0;o<e.length;o++)qr(e[o]);for(var a=0;a<e.length;a++)Vr(e[a]);for(var s=0;s<e.length;s++)Wr(e[s])}(t)})}function Jr(t){var e=t.cm,n=e.display;!function(t){var e=t.display;!e.scrollbarsClipped&&e.scroller.offsetWidth&&(e.nativeBarWidth=e.scroller.offsetWidth-e.scroller.clientWidth,e.heightForcer.style.height=xn(t)+"px",e.sizer.style.marginBottom=-e.nativeBarWidth+"px",e.sizer.style.borderRightWidth=xn(t)+"px",e.scrollbarsClipped=!0)}(e),t.updateMaxLine&&ze(e),t.mustUpdate=t.viewChanged||t.forceUpdate||null!=t.scrollTop||t.scrollToPos&&(t.scrollToPos.from.line<n.viewFrom||t.scrollToPos.to.line>=n.viewTo)||n.maxLineChanged&&e.options.lineWrapping,t.update=t.mustUpdate&&new ri(e,t.mustUpdate&&{top:t.scrollTop,ensure:t.scrollToPos},t.forceUpdate)}function qr(t){var e=t.cm,n=e.display;t.updatedDisplay&&Qr(e),t.barMeasure=Tr(e),n.maxLineChanged&&!e.options.lineWrapping&&(t.adjustWidthTo=kn(e,n.maxLine,n.maxLine.text.length).left+3,e.display.sizerWidth=t.adjustWidthTo,t.barMeasure.scrollWidth=Math.max(n.scroller.clientWidth,n.sizer.offsetLeft+t.adjustWidthTo+xn(e)+e.display.barWidth),t.maxScrollLeft=Math.max(0,n.sizer.offsetLeft+t.adjustWidthTo-_n(e))),(t.updatedDisplay||t.selectionChanged)&&(t.preparedSelection=n.input.prepareSelection())}function Vr(t){var e=t.cm;null!=t.adjustWidthTo&&(e.display.sizer.style.minWidth=t.adjustWidthTo+"px",t.maxScrollLeft<e.doc.scrollLeft&&Fr(e,Math.min(e.display.scroller.scrollLeft,t.maxScrollLeft),!0),e.display.maxLineChanged=!1);var n=t.focus&&t.focus==S();t.preparedSelection&&e.display.input.showSelection(t.preparedSelection,n),(t.updatedDisplay||t.startHeight!=e.doc.height)&&Gr(e,t.barMeasure),t.updatedDisplay&&Ai(e,t.barMeasure),t.selectionChanged&&yr(e),e.state.focused&&t.updateInput&&e.display.input.reset(t.typing),n&&Er(t.cm)}function Wr(t){var e=t.cm,n=e.display,r=e.doc;if(t.updatedDisplay&&oi(e,t.update),null==n.wheelStartX||null==t.scrollTop&&null==t.scrollLeft&&!t.scrollToPos||(n.wheelStartX=n.wheelStartY=null),null!=t.scrollTop&&Nr(e,t.scrollTop,t.forceScroll),null!=t.scrollLeft&&Fr(e,t.scrollLeft,!0,!0),t.scrollToPos){var i=function(t,e,n,r){var i;null==r&&(r=0),t.options.lineWrapping||e!=n||(e=e.ch?ee(e.line,"before"==e.sticky?e.ch-1:e.ch,"after"):e,n="before"==e.sticky?ee(e.line,e.ch+1,"before"):e);for(var o=0;o<5;o++){var a=!1,s=qn(t,e),A=n&&n!=e?qn(t,n):s;i={left:Math.min(s.left,A.left),top:Math.min(s.top,A.top)-r,right:Math.max(s.left,A.left),bottom:Math.max(s.bottom,A.bottom)+r};var l=_r(t,i),c=t.doc.scrollTop,u=t.doc.scrollLeft;if(null!=l.scrollTop&&(Rr(t,l.scrollTop),Math.abs(t.doc.scrollTop-c)>1&&(a=!0)),null!=l.scrollLeft&&(Fr(t,l.scrollLeft),Math.abs(t.doc.scrollLeft-u)>1&&(a=!0)),!a)break}return i}(e,Ae(r,t.scrollToPos.from),Ae(r,t.scrollToPos.to),t.scrollToPos.margin);!function(t,e){if(!pt(t,"scrollCursorIntoView")){var n=t.display,r=n.sizer.getBoundingClientRect(),i=null;if(e.top+r.top<0?i=!0:e.bottom+r.top>(window.innerHeight||document.documentElement.clientHeight)&&(i=!1),null!=i&&!d){var o=D("div","​",null,"position: absolute;\n                         top: "+(e.top-n.viewOffset-bn(t.display))+"px;\n                         height: "+(e.bottom-e.top+xn(t)+n.barHeight)+"px;\n                         left: "+e.left+"px; width: "+Math.max(2,e.right-e.left)+"px;");t.display.lineSpace.appendChild(o),o.scrollIntoView(i),t.display.lineSpace.removeChild(o)}}}(e,i)}var o=t.maybeHiddenMarkers,a=t.maybeUnhiddenMarkers;if(o)for(var s=0;s<o.length;++s)o[s].lines.length||ft(o[s],"hide");if(a)for(var A=0;A<a.length;++A)a[A].lines.length&&ft(a[A],"unhide");n.wrapper.offsetHeight&&(r.scrollTop=e.display.scroller.scrollTop),t.changeObjs&&ft(e,"changes",e,t.changeObjs),t.update&&t.update.finish()}function Xr(t,e){if(t.curOp)return e();Kr(t);try{return e()}finally{zr(t)}}function Zr(t,e){return function(){if(t.curOp)return e.apply(t,arguments);Kr(t);try{return e.apply(t,arguments)}finally{zr(t)}}}function $r(t){return function(){if(this.curOp)return t.apply(this,arguments);Kr(this);try{return t.apply(this,arguments)}finally{zr(this)}}}function ti(t){return function(){var e=this.cm;if(!e||e.curOp)return t.apply(this,arguments);Kr(e);try{return t.apply(this,arguments)}finally{zr(e)}}}function ei(t,e){t.doc.highlightFrontier<t.display.viewTo&&t.state.highlight.set(e,F(ni,t))}function ni(t){var e=t.doc;if(!(e.highlightFrontier>=t.display.viewTo)){var n=+new Date+t.options.workTime,r=de(t,e.highlightFrontier),i=[];e.iter(r.line,Math.min(e.first+e.size,t.display.viewTo+500),function(o){if(r.line>=t.display.viewFrom){var a=o.styles,s=o.text.length>t.options.maxHighlightLength?Yt(e.mode,r.state):null,A=ge(t,o,r,!0);s&&(r.state=s),o.styles=A.styles;var l=o.styleClasses,c=A.classes;c?o.styleClasses=c:l&&(o.styleClasses=null);for(var u=!a||a.length!=o.styles.length||l!=c&&(!l||!c||l.bgClass!=c.bgClass||l.textClass!=c.textClass),g=0;!u&&g<a.length;++g)u=a[g]!=o.styles[g];u&&i.push(r.line),o.stateAfter=r.save(),r.nextLine()}else o.text.length<=t.options.maxHighlightLength&&fe(t,o.text,r),o.stateAfter=r.line%5==0?r.save():null,r.nextLine();if(+new Date>n)return ei(t,t.options.workDelay),!0}),e.highlightFrontier=r.line,e.modeFrontier=Math.max(e.modeFrontier,r.line),i.length&&Xr(t,function(){for(var e=0;e<i.length;e++)ur(t,i[e],"text")})}}var ri=function(t,e,n){var r=t.display;this.viewport=e,this.visible=xr(r,t.doc,e),this.editorIsHidden=!r.wrapper.offsetWidth,this.wrapperHeight=r.wrapper.clientHeight,this.wrapperWidth=r.wrapper.clientWidth,this.oldDisplayWidth=_n(t),this.force=n,this.dims=ir(t),this.events=[]};function ii(t,e){var n=t.display,r=t.doc;if(e.editorIsHidden)return gr(t),!1;if(!e.force&&e.visible.from>=n.viewFrom&&e.visible.to<=n.viewTo&&(null==n.updateLineNumbers||n.updateLineNumbers>=n.viewTo)&&n.renderedView==n.view&&0==dr(t))return!1;ci(t)&&(gr(t),e.dims=ir(t));var i=r.first+r.size,o=Math.max(e.visible.from-t.options.viewportMargin,r.first),a=Math.min(i,e.visible.to+t.options.viewportMargin);n.viewFrom<o&&o-n.viewFrom<20&&(o=Math.max(r.first,n.viewFrom)),n.viewTo>a&&n.viewTo-a<20&&(a=Math.min(i,n.viewTo)),Be&&(o=Ge(t.doc,o),a=He(t.doc,a));var s=o!=n.viewFrom||a!=n.viewTo||n.lastWrapHeight!=e.wrapperHeight||n.lastWrapWidth!=e.wrapperWidth;!function(t,e,n){var r=t.display;0==r.view.length||e>=r.viewTo||n<=r.viewFrom?(r.view=an(t,e,n),r.viewFrom=e):(r.viewFrom>e?r.view=an(t,e,r.viewFrom).concat(r.view):r.viewFrom<e&&(r.view=r.view.slice(lr(t,e))),r.viewFrom=e,r.viewTo<n?r.view=r.view.concat(an(t,r.viewTo,n)):r.viewTo>n&&(r.view=r.view.slice(0,lr(t,n)))),r.viewTo=n}(t,o,a),n.viewOffset=Pe(Jt(t.doc,n.viewFrom)),t.display.mover.style.top=n.viewOffset+"px";var l=dr(t);if(!s&&0==l&&!e.force&&n.renderedView==n.view&&(null==n.updateLineNumbers||n.updateLineNumbers>=n.viewTo))return!1;var c=function(t){if(t.hasFocus())return null;var e=S();if(!e||!k(t.display.lineDiv,e))return null;var n={activeElt:e};if(window.getSelection){var r=window.getSelection();r.anchorNode&&r.extend&&k(t.display.lineDiv,r.anchorNode)&&(n.anchorNode=r.anchorNode,n.anchorOffset=r.anchorOffset,n.focusNode=r.focusNode,n.focusOffset=r.focusOffset)}return n}(t);return l>4&&(n.lineDiv.style.display="none"),function(t,e,n){var r=t.display,i=t.options.lineNumbers,o=r.lineDiv,a=o.firstChild;function s(e){var n=e.nextSibling;return A&&C&&t.display.currentWheelTarget==e?e.style.display="none":e.parentNode.removeChild(e),n}for(var l=r.view,c=r.viewFrom,u=0;u<l.length;u++){var g=l[u];if(g.hidden);else if(g.node&&g.node.parentNode==o){for(;a!=g.node;)a=s(a);var h=i&&null!=e&&e<=c&&g.lineNumber;g.changes&&(G(g.changes,"gutter")>-1&&(h=!1),un(t,g,c,n)),h&&(x(g.lineNumber),g.lineNumber.appendChild(document.createTextNode(te(t.options,c)))),a=g.node.nextSibling}else{var d=Cn(t,g,c,n);o.insertBefore(d,a)}c+=g.size}for(;a;)a=s(a)}(t,n.updateLineNumbers,e.dims),l>4&&(n.lineDiv.style.display=""),n.renderedView=n.view,function(t){if(t&&t.activeElt&&t.activeElt!=S()&&(t.activeElt.focus(),t.anchorNode&&k(document.body,t.anchorNode)&&k(document.body,t.focusNode))){var e=window.getSelection(),n=document.createRange();n.setEnd(t.anchorNode,t.anchorOffset),n.collapse(!1),e.removeAllRanges(),e.addRange(n),e.extend(t.focusNode,t.focusOffset)}}(c),x(n.cursorDiv),x(n.selectionDiv),n.gutters.style.height=n.sizer.style.minHeight=0,s&&(n.lastWrapHeight=e.wrapperHeight,n.lastWrapWidth=e.wrapperWidth,ei(t,400)),n.updateLineNumbers=null,!0}function oi(t,e){for(var n=e.viewport,r=!0;(r&&t.options.lineWrapping&&e.oldDisplayWidth!=_n(t)||(n&&null!=n.top&&(n={top:Math.min(t.doc.height+Qn(t.display)-Dn(t),n.top)}),e.visible=xr(t.display,t.doc,n),!(e.visible.from>=t.display.viewFrom&&e.visible.to<=t.display.viewTo)))&&ii(t,e);r=!1){Qr(t);var i=Tr(t);fr(t),Gr(t,i),Ai(t,i),e.force=!1}e.signal(t,"update",t),t.display.viewFrom==t.display.reportedViewFrom&&t.display.viewTo==t.display.reportedViewTo||(e.signal(t,"viewportChange",t,t.display.viewFrom,t.display.viewTo),t.display.reportedViewFrom=t.display.viewFrom,t.display.reportedViewTo=t.display.viewTo)}function ai(t,e){var n=new ri(t,e);if(ii(t,n)){Qr(t),oi(t,n);var r=Tr(t);fr(t),Gr(t,r),Ai(t,r),n.finish()}}function si(t){var e=t.gutters.offsetWidth;t.sizer.style.marginLeft=e+"px"}function Ai(t,e){t.display.sizer.style.minHeight=e.docHeight+"px",t.display.heightForcer.style.top=e.docHeight+"px",t.display.gutters.style.height=e.docHeight+t.display.barHeight+xn(t)+"px"}function li(t){var e=t.display,n=e.view;if(e.alignWidgets||e.gutters.firstChild&&t.options.fixedGutter){for(var r=or(e)-e.scroller.scrollLeft+t.doc.scrollLeft,i=e.gutters.offsetWidth,o=r+"px",a=0;a<n.length;a++)if(!n[a].hidden){t.options.fixedGutter&&(n[a].gutter&&(n[a].gutter.style.left=o),n[a].gutterBackground&&(n[a].gutterBackground.style.left=o));var s=n[a].alignable;if(s)for(var A=0;A<s.length;A++)s[A].style.left=o}t.options.fixedGutter&&(e.gutters.style.left=r+i+"px")}}function ci(t){if(!t.options.lineNumbers)return!1;var e=t.doc,n=te(t.options,e.first+e.size-1),r=t.display;if(n.length!=r.lineNumChars){var i=r.measure.appendChild(D("div",[D("div",n)],"CodeMirror-linenumber CodeMirror-gutter-elt")),o=i.firstChild.offsetWidth,a=i.offsetWidth-o;return r.lineGutter.style.width="",r.lineNumInnerWidth=Math.max(o,r.lineGutter.offsetWidth-a)+1,r.lineNumWidth=r.lineNumInnerWidth+a,r.lineNumChars=r.lineNumInnerWidth?n.length:-1,r.lineGutter.style.width=r.lineNumWidth+"px",si(t.display),!0}return!1}function ui(t,e){for(var n=[],r=!1,i=0;i<t.length;i++){var o=t[i],a=null;if("string"!=typeof o&&(a=o.style,o=o.className),"CodeMirror-linenumbers"==o){if(!e)continue;r=!0}n.push({className:o,style:a})}return e&&!r&&n.push({className:"CodeMirror-linenumbers",style:null}),n}function gi(t){var e=t.gutters,n=t.gutterSpecs;x(e),t.lineGutter=null;for(var r=0;r<n.length;++r){var i=n[r],o=i.className,a=i.style,s=e.appendChild(D("div",null,"CodeMirror-gutter "+o));a&&(s.style.cssText=a),"CodeMirror-linenumbers"==o&&(t.lineGutter=s,s.style.width=(t.lineNumWidth||1)+"px")}e.style.display=n.length?"":"none",si(t)}function hi(t){gi(t.display),cr(t),li(t)}function di(t,e,r,i){var o=this;this.input=r,o.scrollbarFiller=D("div",null,"CodeMirror-scrollbar-filler"),o.scrollbarFiller.setAttribute("cm-not-content","true"),o.gutterFiller=D("div",null,"CodeMirror-gutter-filler"),o.gutterFiller.setAttribute("cm-not-content","true"),o.lineDiv=M("div",null,"CodeMirror-code"),o.selectionDiv=D("div",null,null,"position: relative; z-index: 1"),o.cursorDiv=D("div",null,"CodeMirror-cursors"),o.measure=D("div",null,"CodeMirror-measure"),o.lineMeasure=D("div",null,"CodeMirror-measure"),o.lineSpace=M("div",[o.measure,o.lineMeasure,o.selectionDiv,o.cursorDiv,o.lineDiv],null,"position: relative; outline: none");var l=M("div",[o.lineSpace],"CodeMirror-lines");o.mover=D("div",[l],null,"position: relative"),o.sizer=D("div",[o.mover],"CodeMirror-sizer"),o.sizerWidth=null,o.heightForcer=D("div",null,null,"position: absolute; height: "+H+"px; width: 1px;"),o.gutters=D("div",null,"CodeMirror-gutters"),o.lineGutter=null,o.scroller=D("div",[o.sizer,o.heightForcer,o.gutters],"CodeMirror-scroll"),o.scroller.setAttribute("tabIndex","-1"),o.wrapper=D("div",[o.scrollbarFiller,o.gutterFiller,o.scroller],"CodeMirror"),a&&s<8&&(o.gutters.style.zIndex=-1,o.scroller.style.paddingRight=0),A||n&&I||(o.scroller.draggable=!0),t&&(t.appendChild?t.appendChild(o.wrapper):t(o.wrapper)),o.viewFrom=o.viewTo=e.first,o.reportedViewFrom=o.reportedViewTo=e.first,o.view=[],o.renderedView=null,o.externalMeasured=null,o.viewOffset=0,o.lastWrapHeight=o.lastWrapWidth=0,o.updateLineNumbers=null,o.nativeBarWidth=o.barHeight=o.barWidth=0,o.scrollbarsClipped=!1,o.lineNumWidth=o.lineNumInnerWidth=o.lineNumChars=null,o.alignWidgets=!1,o.cachedCharWidth=o.cachedTextHeight=o.cachedPaddingH=null,o.maxLine=null,o.maxLineLength=0,o.maxLineChanged=!1,o.wheelDX=o.wheelDY=o.wheelStartX=o.wheelStartY=null,o.shift=!1,o.selForContextMenu=null,o.activeTouch=null,o.gutterSpecs=ui(i.gutters,i.lineNumbers),gi(o),r.init(o)}ri.prototype.signal=function(t,e){Ct(t,e)&&this.events.push(arguments)},ri.prototype.finish=function(){for(var t=0;t<this.events.length;t++)ft.apply(null,this.events[t])};var fi=0,pi=null;function Ii(t){var e=t.wheelDeltaX,n=t.wheelDeltaY;return null==e&&t.detail&&t.axis==t.HORIZONTAL_AXIS&&(e=t.detail),null==n&&t.detail&&t.axis==t.VERTICAL_AXIS?n=t.detail:null==n&&(n=t.wheelDelta),{x:e,y:n}}function Ci(t){var e=Ii(t);return e.x*=pi,e.y*=pi,e}function mi(t,e){var r=Ii(e),i=r.x,o=r.y,a=t.display,s=a.scroller,l=s.scrollWidth>s.clientWidth,c=s.scrollHeight>s.clientHeight;if(i&&l||o&&c){if(o&&C&&A)t:for(var g=e.target,h=a.view;g!=s;g=g.parentNode)for(var d=0;d<h.length;d++)if(h[d].node==g){t.display.currentWheelTarget=g;break t}if(i&&!n&&!u&&null!=pi)return o&&c&&Rr(t,Math.max(0,s.scrollTop+o*pi)),Fr(t,Math.max(0,s.scrollLeft+i*pi)),(!o||o&&c)&&yt(e),void(a.wheelStartX=null);if(o&&null!=pi){var f=o*pi,p=t.doc.scrollTop,I=p+a.wrapper.clientHeight;f<0?p=Math.max(0,p+f-50):I=Math.min(t.doc.height,I+f+50),ai(t,{top:p,bottom:I})}fi<20&&(null==a.wheelStartX?(a.wheelStartX=s.scrollLeft,a.wheelStartY=s.scrollTop,a.wheelDX=i,a.wheelDY=o,setTimeout(function(){if(null!=a.wheelStartX){var t=s.scrollLeft-a.wheelStartX,e=s.scrollTop-a.wheelStartY,n=e&&a.wheelDY&&e/a.wheelDY||t&&a.wheelDX&&t/a.wheelDX;a.wheelStartX=a.wheelStartY=null,n&&(pi=(pi*fi+n)/(fi+1),++fi)}},200)):(a.wheelDX+=i,a.wheelDY+=o))}}a?pi=-.53:n?pi=15:c?pi=-.7:g&&(pi=-1/3);var yi=function(t,e){this.ranges=t,this.primIndex=e};yi.prototype.primary=function(){return this.ranges[this.primIndex]},yi.prototype.equals=function(t){if(t==this)return!0;if(t.primIndex!=this.primIndex||t.ranges.length!=this.ranges.length)return!1;for(var e=0;e<this.ranges.length;e++){var n=this.ranges[e],r=t.ranges[e];if(!re(n.anchor,r.anchor)||!re(n.head,r.head))return!1}return!0},yi.prototype.deepCopy=function(){for(var t=[],e=0;e<this.ranges.length;e++)t[e]=new Ei(ie(this.ranges[e].anchor),ie(this.ranges[e].head));return new yi(t,this.primIndex)},yi.prototype.somethingSelected=function(){for(var t=0;t<this.ranges.length;t++)if(!this.ranges[t].empty())return!0;return!1},yi.prototype.contains=function(t,e){e||(e=t);for(var n=0;n<this.ranges.length;n++){var r=this.ranges[n];if(ne(e,r.from())>=0&&ne(t,r.to())<=0)return n}return-1};var Ei=function(t,e){this.anchor=t,this.head=e};function vi(t,e,n){var r=t&&t.options.selectionsMayTouch,i=e[n];e.sort(function(t,e){return ne(t.from(),e.from())}),n=G(e,i);for(var o=1;o<e.length;o++){var a=e[o],s=e[o-1],A=ne(s.to(),a.from());if(r&&!a.empty()?A>0:A>=0){var l=ae(s.from(),a.from()),c=oe(s.to(),a.to()),u=s.empty()?a.from()==a.head:s.from()==s.head;o<=n&&--n,e.splice(--o,2,new Ei(u?c:l,u?l:c))}}return new yi(e,n)}function Bi(t,e){return new yi([new Ei(t,e||t)],0)}function bi(t){return t.text?ee(t.from.line+t.text.length-1,V(t.text).length+(1==t.text.length?t.from.ch:0)):t.to}function Qi(t,e){if(ne(t,e.from)<0)return t;if(ne(t,e.to)<=0)return bi(e);var n=t.line+e.text.length-(e.to.line-e.from.line)-1,r=t.ch;return t.line==e.to.line&&(r+=bi(e).ch-e.to.ch),ee(n,r)}function wi(t,e){for(var n=[],r=0;r<t.sel.ranges.length;r++){var i=t.sel.ranges[r];n.push(new Ei(Qi(i.anchor,e),Qi(i.head,e)))}return vi(t.cm,n,t.sel.primIndex)}function xi(t,e,n){return t.line==e.line?ee(n.line,t.ch-e.ch+n.ch):ee(n.line+(t.line-e.line),t.ch)}function _i(t){t.doc.mode=Gt(t.options,t.doc.modeOption),Di(t)}function Di(t){t.doc.iter(function(t){t.stateAfter&&(t.stateAfter=null),t.styles&&(t.styles=null)}),t.doc.modeFrontier=t.doc.highlightFrontier=t.doc.first,ei(t,100),t.state.modeGen++,t.curOp&&cr(t)}function Mi(t,e){return 0==e.from.ch&&0==e.to.ch&&""==V(e.text)&&(!t.cm||t.cm.options.wholeLineUpdateBefore)}function ki(t,e,n,r){function i(t){return n?n[t]:null}function o(t,n,i){!function(t,e,n,r){t.text=e,t.stateAfter&&(t.stateAfter=null),t.styles&&(t.styles=null),null!=t.order&&(t.order=null),De(t),Me(t,n);var i=r?r(t):1;i!=t.height&&Wt(t,i)}(t,n,i,r),ln(t,"change",t,e)}function a(t,e){for(var n=[],o=t;o<e;++o)n.push(new Je(l[o],i(o),r));return n}var s=e.from,A=e.to,l=e.text,c=Jt(t,s.line),u=Jt(t,A.line),g=V(l),h=i(l.length-1),d=A.line-s.line;if(e.full)t.insert(0,a(0,l.length)),t.remove(l.length,t.size-l.length);else if(Mi(t,e)){var f=a(0,l.length-1);o(u,u.text,h),d&&t.remove(s.line,d),f.length&&t.insert(s.line,f)}else if(c==u)if(1==l.length)o(c,c.text.slice(0,s.ch)+g+c.text.slice(A.ch),h);else{var p=a(1,l.length-1);p.push(new Je(g+c.text.slice(A.ch),h,r)),o(c,c.text.slice(0,s.ch)+l[0],i(0)),t.insert(s.line+1,p)}else if(1==l.length)o(c,c.text.slice(0,s.ch)+l[0]+u.text.slice(A.ch),i(0)),t.remove(s.line+1,d);else{o(c,c.text.slice(0,s.ch)+l[0],i(0)),o(u,g+u.text.slice(A.ch),h);var I=a(1,l.length-1);d>1&&t.remove(s.line+1,d-1),t.insert(s.line+1,I)}ln(t,"change",t,e)}function Si(t,e,n){!function t(r,i,o){if(r.linked)for(var a=0;a<r.linked.length;++a){var s=r.linked[a];if(s.doc!=i){var A=o&&s.sharedHist;n&&!A||(e(s.doc,A),t(s.doc,r,A))}}}(t,null,!0)}function Li(t,e){if(e.cm)throw new Error("This document is already in use.");t.doc=e,e.cm=t,sr(t),_i(t),Ri(t),t.options.lineWrapping||ze(t),t.options.mode=e.modeOption,cr(t)}function Ri(t){("rtl"==t.doc.direction?L:w)(t.display.lineDiv,"CodeMirror-rtl")}function Ni(t){this.done=[],this.undone=[],this.undoDepth=1/0,this.lastModTime=this.lastSelTime=0,this.lastOp=this.lastSelOp=null,this.lastOrigin=this.lastSelOrigin=null,this.generation=this.maxGeneration=t||1}function Fi(t,e){var n={from:ie(e.from),to:bi(e),text:qt(t,e.from,e.to)};return Hi(t,n,e.from.line,e.to.line+1),Si(t,function(t){return Hi(t,n,e.from.line,e.to.line+1)},!0),n}function Ti(t){for(;t.length;){var e=V(t);if(!e.ranges)break;t.pop()}}function Ui(t,e,n,r){var i=t.history;i.undone.length=0;var o,a,s=+new Date;if((i.lastOp==r||i.lastOrigin==e.origin&&e.origin&&("+"==e.origin.charAt(0)&&i.lastModTime>s-(t.cm?t.cm.options.historyEventDelay:500)||"*"==e.origin.charAt(0)))&&(o=function(t,e){return e?(Ti(t.done),V(t.done)):t.done.length&&!V(t.done).ranges?V(t.done):t.done.length>1&&!t.done[t.done.length-2].ranges?(t.done.pop(),V(t.done)):void 0}(i,i.lastOp==r)))a=V(o.changes),0==ne(e.from,e.to)&&0==ne(e.from,a.to)?a.to=bi(e):o.changes.push(Fi(t,e));else{var A=V(i.done);for(A&&A.ranges||Gi(t.sel,i.done),o={changes:[Fi(t,e)],generation:i.generation},i.done.push(o);i.done.length>i.undoDepth;)i.done.shift(),i.done[0].ranges||i.done.shift()}i.done.push(n),i.generation=++i.maxGeneration,i.lastModTime=i.lastSelTime=s,i.lastOp=i.lastSelOp=r,i.lastOrigin=i.lastSelOrigin=e.origin,a||ft(t,"historyAdded")}function Oi(t,e,n,r){var i=t.history,o=r&&r.origin;n==i.lastSelOp||o&&i.lastSelOrigin==o&&(i.lastModTime==i.lastSelTime&&i.lastOrigin==o||function(t,e,n,r){var i=e.charAt(0);return"*"==i||"+"==i&&n.ranges.length==r.ranges.length&&n.somethingSelected()==r.somethingSelected()&&new Date-t.history.lastSelTime<=(t.cm?t.cm.options.historyEventDelay:500)}(t,o,V(i.done),e))?i.done[i.done.length-1]=e:Gi(e,i.done),i.lastSelTime=+new Date,i.lastSelOrigin=o,i.lastSelOp=n,r&&!1!==r.clearRedo&&Ti(i.undone)}function Gi(t,e){var n=V(e);n&&n.ranges&&n.equals(t)||e.push(t)}function Hi(t,e,n,r){var i=e["spans_"+t.id],o=0;t.iter(Math.max(t.first,n),Math.min(t.first+t.size,r),function(n){n.markedSpans&&((i||(i=e["spans_"+t.id]={}))[o]=n.markedSpans),++o})}function ji(t){if(!t)return null;for(var e,n=0;n<t.length;++n)t[n].marker.explicitlyCleared?e||(e=t.slice(0,n)):e&&e.push(t[n]);return e?e.length?e:null:t}function Yi(t,e){var n=function(t,e){var n=e["spans_"+t.id];if(!n)return null;for(var r=[],i=0;i<e.text.length;++i)r.push(ji(n[i]));return r}(t,e),r=xe(t,e);if(!n)return r;if(!r)return n;for(var i=0;i<n.length;++i){var o=n[i],a=r[i];if(o&&a)t:for(var s=0;s<a.length;++s){for(var A=a[s],l=0;l<o.length;++l)if(o[l].marker==A.marker)continue t;o.push(A)}else a&&(n[i]=a)}return n}function Pi(t,e,n){for(var r=[],i=0;i<t.length;++i){var o=t[i];if(o.ranges)r.push(n?yi.prototype.deepCopy.call(o):o);else{var a=o.changes,s=[];r.push({changes:s});for(var A=0;A<a.length;++A){var l=a[A],c=void 0;if(s.push({from:l.from,to:l.to,text:l.text}),e)for(var u in l)(c=u.match(/^spans_(\d+)$/))&&G(e,Number(c[1]))>-1&&(V(s)[u]=l[u],delete l[u])}}}return r}function Ki(t,e,n,r){if(r){var i=t.anchor;if(n){var o=ne(e,i)<0;o!=ne(n,i)<0?(i=e,e=n):o!=ne(e,n)<0&&(e=n)}return new Ei(i,e)}return new Ei(n||e,e)}function zi(t,e,n,r,i){null==i&&(i=t.cm&&(t.cm.display.shift||t.extend)),Xi(t,new yi([Ki(t.sel.primary(),e,n,i)],0),r)}function Ji(t,e,n){for(var r=[],i=t.cm&&(t.cm.display.shift||t.extend),o=0;o<t.sel.ranges.length;o++)r[o]=Ki(t.sel.ranges[o],e[o],null,i);var a=vi(t.cm,r,t.sel.primIndex);Xi(t,a,n)}function qi(t,e,n,r){var i=t.sel.ranges.slice(0);i[e]=n,Xi(t,vi(t.cm,i,t.sel.primIndex),r)}function Vi(t,e,n,r){Xi(t,Bi(e,n),r)}function Wi(t,e,n){var r=t.history.done,i=V(r);i&&i.ranges?(r[r.length-1]=e,Zi(t,e,n)):Xi(t,e,n)}function Xi(t,e,n){Zi(t,e,n),Oi(t,t.sel,t.cm?t.cm.curOp.id:NaN,n)}function Zi(t,e,n){(Ct(t,"beforeSelectionChange")||t.cm&&Ct(t.cm,"beforeSelectionChange"))&&(e=function(t,e,n){var r={ranges:e.ranges,update:function(e){this.ranges=[];for(var n=0;n<e.length;n++)this.ranges[n]=new Ei(Ae(t,e[n].anchor),Ae(t,e[n].head))},origin:n&&n.origin};return ft(t,"beforeSelectionChange",t,r),t.cm&&ft(t.cm,"beforeSelectionChange",t.cm,r),r.ranges!=e.ranges?vi(t.cm,r.ranges,r.ranges.length-1):e}(t,e,n));var r=n&&n.bias||(ne(e.primary().head,t.sel.primary().head)<0?-1:1);$i(t,eo(t,e,r,!0)),n&&!1===n.scroll||!t.cm||Mr(t.cm)}function $i(t,e){e.equals(t.sel)||(t.sel=e,t.cm&&(t.cm.curOp.updateInput=1,t.cm.curOp.selectionChanged=!0,It(t.cm)),ln(t,"cursorActivity",t))}function to(t){$i(t,eo(t,t.sel,null,!1))}function eo(t,e,n,r){for(var i,o=0;o<e.ranges.length;o++){var a=e.ranges[o],s=e.ranges.length==t.sel.ranges.length&&t.sel.ranges[o],A=ro(t,a.anchor,s&&s.anchor,n,r),l=ro(t,a.head,s&&s.head,n,r);(i||A!=a.anchor||l!=a.head)&&(i||(i=e.ranges.slice(0,o)),i[o]=new Ei(A,l))}return i?vi(t.cm,i,e.primIndex):e}function no(t,e,n,r,i){var o=Jt(t,e.line);if(o.markedSpans)for(var a=0;a<o.markedSpans.length;++a){var s=o.markedSpans[a],A=s.marker;if((null==s.from||(A.inclusiveLeft?s.from<=e.ch:s.from<e.ch))&&(null==s.to||(A.inclusiveRight?s.to>=e.ch:s.to>e.ch))){if(i&&(ft(A,"beforeCursorEnter"),A.explicitlyCleared)){if(o.markedSpans){--a;continue}break}if(!A.atomic)continue;if(n){var l=A.find(r<0?1:-1),c=void 0;if((r<0?A.inclusiveRight:A.inclusiveLeft)&&(l=io(t,l,-r,l&&l.line==e.line?o:null)),l&&l.line==e.line&&(c=ne(l,n))&&(r<0?c<0:c>0))return no(t,l,e,r,i)}var u=A.find(r<0?-1:1);return(r<0?A.inclusiveLeft:A.inclusiveRight)&&(u=io(t,u,r,u.line==e.line?o:null)),u?no(t,u,e,r,i):null}}return e}function ro(t,e,n,r,i){var o=r||1,a=no(t,e,n,o,i)||!i&&no(t,e,n,o,!0)||no(t,e,n,-o,i)||!i&&no(t,e,n,-o,!0);return a||(t.cantEdit=!0,ee(t.first,0))}function io(t,e,n,r){return n<0&&0==e.ch?e.line>t.first?Ae(t,ee(e.line-1)):null:n>0&&e.ch==(r||Jt(t,e.line)).text.length?e.line<t.first+t.size-1?ee(e.line+1,0):null:new ee(e.line,e.ch+n)}function oo(t){t.setSelection(ee(t.firstLine(),0),ee(t.lastLine()),Y)}function ao(t,e,n){var r={canceled:!1,from:e.from,to:e.to,text:e.text,origin:e.origin,cancel:function(){return r.canceled=!0}};return n&&(r.update=function(e,n,i,o){e&&(r.from=Ae(t,e)),n&&(r.to=Ae(t,n)),i&&(r.text=i),void 0!==o&&(r.origin=o)}),ft(t,"beforeChange",t,r),t.cm&&ft(t.cm,"beforeChange",t.cm,r),r.canceled?(t.cm&&(t.cm.curOp.updateInput=2),null):{from:r.from,to:r.to,text:r.text,origin:r.origin}}function so(t,e,n){if(t.cm){if(!t.cm.curOp)return Zr(t.cm,so)(t,e,n);if(t.cm.state.suppressEdits)return}if(!(Ct(t,"beforeChange")||t.cm&&Ct(t.cm,"beforeChange"))||(e=ao(t,e,!0))){var r=ve&&!n&&function(t,e,n){var r=null;if(t.iter(e.line,n.line+1,function(t){if(t.markedSpans)for(var e=0;e<t.markedSpans.length;++e){var n=t.markedSpans[e].marker;!n.readOnly||r&&-1!=G(r,n)||(r||(r=[])).push(n)}}),!r)return null;for(var i=[{from:e,to:n}],o=0;o<r.length;++o)for(var a=r[o],s=a.find(0),A=0;A<i.length;++A){var l=i[A];if(!(ne(l.to,s.from)<0||ne(l.from,s.to)>0)){var c=[A,1],u=ne(l.from,s.from),g=ne(l.to,s.to);(u<0||!a.inclusiveLeft&&!u)&&c.push({from:l.from,to:s.from}),(g>0||!a.inclusiveRight&&!g)&&c.push({from:s.to,to:l.to}),i.splice.apply(i,c),A+=c.length-3}}return i}(t,e.from,e.to);if(r)for(var i=r.length-1;i>=0;--i)Ao(t,{from:r[i].from,to:r[i].to,text:i?[""]:e.text,origin:e.origin});else Ao(t,e)}}function Ao(t,e){if(1!=e.text.length||""!=e.text[0]||0!=ne(e.from,e.to)){var n=wi(t,e);Ui(t,e,n,t.cm?t.cm.curOp.id:NaN),uo(t,e,n,xe(t,e));var r=[];Si(t,function(t,n){n||-1!=G(r,t.history)||(po(t.history,e),r.push(t.history)),uo(t,e,null,xe(t,e))})}}function lo(t,e,n){var r=t.cm&&t.cm.state.suppressEdits;if(!r||n){for(var i,o=t.history,a=t.sel,s="undo"==e?o.done:o.undone,A="undo"==e?o.undone:o.done,l=0;l<s.length&&(i=s[l],n?!i.ranges||i.equals(t.sel):i.ranges);l++);if(l!=s.length){for(o.lastOrigin=o.lastSelOrigin=null;;){if(!(i=s.pop()).ranges){if(r)return void s.push(i);break}if(Gi(i,A),n&&!i.equals(t.sel))return void Xi(t,i,{clearRedo:!1});a=i}var c=[];Gi(a,A),A.push({changes:c,generation:o.generation}),o.generation=i.generation||++o.maxGeneration;for(var u=Ct(t,"beforeChange")||t.cm&&Ct(t.cm,"beforeChange"),g=function(n){var r=i.changes[n];if(r.origin=e,u&&!ao(t,r,!1))return s.length=0,{};c.push(Fi(t,r));var o=n?wi(t,r):V(s);uo(t,r,o,Yi(t,r)),!n&&t.cm&&t.cm.scrollIntoView({from:r.from,to:bi(r)});var a=[];Si(t,function(t,e){e||-1!=G(a,t.history)||(po(t.history,r),a.push(t.history)),uo(t,r,null,Yi(t,r))})},h=i.changes.length-1;h>=0;--h){var d=g(h);if(d)return d.v}}}}function co(t,e){if(0!=e&&(t.first+=e,t.sel=new yi(W(t.sel.ranges,function(t){return new Ei(ee(t.anchor.line+e,t.anchor.ch),ee(t.head.line+e,t.head.ch))}),t.sel.primIndex),t.cm)){cr(t.cm,t.first,t.first-e,e);for(var n=t.cm.display,r=n.viewFrom;r<n.viewTo;r++)ur(t.cm,r,"gutter")}}function uo(t,e,n,r){if(t.cm&&!t.cm.curOp)return Zr(t.cm,uo)(t,e,n,r);if(e.to.line<t.first)co(t,e.text.length-1-(e.to.line-e.from.line));else if(!(e.from.line>t.lastLine())){if(e.from.line<t.first){var i=e.text.length-1-(t.first-e.from.line);co(t,i),e={from:ee(t.first,0),to:ee(e.to.line+i,e.to.ch),text:[V(e.text)],origin:e.origin}}var o=t.lastLine();e.to.line>o&&(e={from:e.from,to:ee(o,Jt(t,o).text.length),text:[e.text[0]],origin:e.origin}),e.removed=qt(t,e.from,e.to),n||(n=wi(t,e)),t.cm?function(t,e,n){var r=t.doc,i=t.display,o=e.from,a=e.to,s=!1,A=o.line;t.options.lineWrapping||(A=Xt(Oe(Jt(r,o.line))),r.iter(A,a.line+1,function(t){if(t==i.maxLine)return s=!0,!0})),r.sel.contains(e.from,e.to)>-1&&It(t),ki(r,e,n,ar(t)),t.options.lineWrapping||(r.iter(A,o.line+e.text.length,function(t){var e=Ke(t);e>i.maxLineLength&&(i.maxLine=t,i.maxLineLength=e,i.maxLineChanged=!0,s=!1)}),s&&(t.curOp.updateMaxLine=!0)),function(t,e){if(t.modeFrontier=Math.min(t.modeFrontier,e),!(t.highlightFrontier<e-10)){for(var n=t.first,r=e-1;r>n;r--){var i=Jt(t,r).stateAfter;if(i&&(!(i instanceof ce)||r+i.lookAhead<e)){n=r+1;break}}t.highlightFrontier=Math.min(t.highlightFrontier,n)}}(r,o.line),ei(t,400);var l=e.text.length-(a.line-o.line)-1;e.full?cr(t):o.line!=a.line||1!=e.text.length||Mi(t.doc,e)?cr(t,o.line,a.line+1,l):ur(t,o.line,"text");var c=Ct(t,"changes"),u=Ct(t,"change");if(u||c){var g={from:o,to:a,text:e.text,removed:e.removed,origin:e.origin};u&&ln(t,"change",t,g),c&&(t.curOp.changeObjs||(t.curOp.changeObjs=[])).push(g)}t.display.selForContextMenu=null}(t.cm,e,r):ki(t,e,r),Zi(t,n,Y)}}function go(t,e,n,r,i){var o;r||(r=n),ne(r,n)<0&&(n=(o=[r,n])[0],r=o[1]),"string"==typeof e&&(e=t.splitLines(e)),so(t,{from:n,to:r,text:e,origin:i})}function ho(t,e,n,r){n<t.line?t.line+=r:e<t.line&&(t.line=e,t.ch=0)}function fo(t,e,n,r){for(var i=0;i<t.length;++i){var o=t[i],a=!0;if(o.ranges){o.copied||((o=t[i]=o.deepCopy()).copied=!0);for(var s=0;s<o.ranges.length;s++)ho(o.ranges[s].anchor,e,n,r),ho(o.ranges[s].head,e,n,r)}else{for(var A=0;A<o.changes.length;++A){var l=o.changes[A];if(n<l.from.line)l.from=ee(l.from.line+r,l.from.ch),l.to=ee(l.to.line+r,l.to.ch);else if(e<=l.to.line){a=!1;break}}a||(t.splice(0,i+1),i=0)}}}function po(t,e){var n=e.from.line,r=e.to.line,i=e.text.length-(r-n)-1;fo(t.done,n,r,i),fo(t.undone,n,r,i)}function Io(t,e,n,r){var i=e,o=e;return"number"==typeof e?o=Jt(t,se(t,e)):i=Xt(e),null==i?null:(r(o,i)&&t.cm&&ur(t.cm,i,n),o)}function Co(t){this.lines=t,this.parent=null;for(var e=0,n=0;n<t.length;++n)t[n].parent=this,e+=t[n].height;this.height=e}function mo(t){this.children=t;for(var e=0,n=0,r=0;r<t.length;++r){var i=t[r];e+=i.chunkSize(),n+=i.height,i.parent=this}this.size=e,this.height=n,this.parent=null}Ei.prototype.from=function(){return ae(this.anchor,this.head)},Ei.prototype.to=function(){return oe(this.anchor,this.head)},Ei.prototype.empty=function(){return this.head.line==this.anchor.line&&this.head.ch==this.anchor.ch},Co.prototype={chunkSize:function(){return this.lines.length},removeInner:function(t,e){for(var n=t,r=t+e;n<r;++n){var i=this.lines[n];this.height-=i.height,qe(i),ln(i,"delete")}this.lines.splice(t,e)},collapse:function(t){t.push.apply(t,this.lines)},insertInner:function(t,e,n){this.height+=n,this.lines=this.lines.slice(0,t).concat(e).concat(this.lines.slice(t));for(var r=0;r<e.length;++r)e[r].parent=this},iterN:function(t,e,n){for(var r=t+e;t<r;++t)if(n(this.lines[t]))return!0}},mo.prototype={chunkSize:function(){return this.size},removeInner:function(t,e){this.size-=e;for(var n=0;n<this.children.length;++n){var r=this.children[n],i=r.chunkSize();if(t<i){var o=Math.min(e,i-t),a=r.height;if(r.removeInner(t,o),this.height-=a-r.height,i==o&&(this.children.splice(n--,1),r.parent=null),0==(e-=o))break;t=0}else t-=i}if(this.size-e<25&&(this.children.length>1||!(this.children[0]instanceof Co))){var s=[];this.collapse(s),this.children=[new Co(s)],this.children[0].parent=this}},collapse:function(t){for(var e=0;e<this.children.length;++e)this.children[e].collapse(t)},insertInner:function(t,e,n){this.size+=e.length,this.height+=n;for(var r=0;r<this.children.length;++r){var i=this.children[r],o=i.chunkSize();if(t<=o){if(i.insertInner(t,e,n),i.lines&&i.lines.length>50){for(var a=i.lines.length%25+25,s=a;s<i.lines.length;){var A=new Co(i.lines.slice(s,s+=25));i.height-=A.height,this.children.splice(++r,0,A),A.parent=this}i.lines=i.lines.slice(0,a),this.maybeSpill()}break}t-=o}},maybeSpill:function(){if(!(this.children.length<=10)){var t=this;do{var e=t.children.splice(t.children.length-5,5),n=new mo(e);if(t.parent){t.size-=n.size,t.height-=n.height;var r=G(t.parent.children,t);t.parent.children.splice(r+1,0,n)}else{var i=new mo(t.children);i.parent=t,t.children=[i,n],t=i}n.parent=t.parent}while(t.children.length>10);t.parent.maybeSpill()}},iterN:function(t,e,n){for(var r=0;r<this.children.length;++r){var i=this.children[r],o=i.chunkSize();if(t<o){var a=Math.min(e,o-t);if(i.iterN(t,a,n))return!0;if(0==(e-=a))break;t=0}else t-=o}}};var yo=function(t,e,n){if(n)for(var r in n)n.hasOwnProperty(r)&&(this[r]=n[r]);this.doc=t,this.node=e};function Eo(t,e,n){Pe(e)<(t.curOp&&t.curOp.scrollTop||t.doc.scrollTop)&&Dr(t,n)}yo.prototype.clear=function(){var t=this.doc.cm,e=this.line.widgets,n=this.line,r=Xt(n);if(null!=r&&e){for(var i=0;i<e.length;++i)e[i]==this&&e.splice(i--,1);e.length||(n.widgets=null);var o=vn(this);Wt(n,Math.max(0,n.height-o)),t&&(Xr(t,function(){Eo(t,n,-o),ur(t,r,"widget")}),ln(t,"lineWidgetCleared",t,this,r))}},yo.prototype.changed=function(){var t=this,e=this.height,n=this.doc.cm,r=this.line;this.height=null;var i=vn(this)-e;i&&(je(this.doc,r)||Wt(r,r.height+i),n&&Xr(n,function(){n.curOp.forceUpdate=!0,Eo(n,r,i),ln(n,"lineWidgetChanged",n,t,Xt(r))}))},mt(yo);var vo=0,Bo=function(t,e){this.lines=[],this.type=e,this.doc=t,this.id=++vo};function bo(t,e,n,r,i){if(r&&r.shared)return function(t,e,n,r,i){(r=T(r)).shared=!1;var o=[bo(t,e,n,r,i)],a=o[0],s=r.widgetNode;return Si(t,function(t){s&&(r.widgetNode=s.cloneNode(!0)),o.push(bo(t,Ae(t,e),Ae(t,n),r,i));for(var A=0;A<t.linked.length;++A)if(t.linked[A].isParent)return;a=V(o)}),new Qo(o,a)}(t,e,n,r,i);if(t.cm&&!t.cm.curOp)return Zr(t.cm,bo)(t,e,n,r,i);var o=new Bo(t,i),a=ne(e,n);if(r&&T(r,o,!1),a>0||0==a&&!1!==o.clearWhenEmpty)return o;if(o.replacedWith&&(o.collapsed=!0,o.widgetNode=M("span",[o.replacedWith],"CodeMirror-widget"),r.handleMouseEvents||o.widgetNode.setAttribute("cm-ignore-events","true"),r.insertLeft&&(o.widgetNode.insertLeft=!0)),o.collapsed){if(Ue(t,e.line,e,n,o)||e.line!=n.line&&Ue(t,n.line,e,n,o))throw new Error("Inserting collapsed marker partially overlapping an existing one");Be=!0}o.addToHistory&&Ui(t,{from:e,to:n,origin:"markText"},t.sel,NaN);var s,A=e.line,l=t.cm;if(t.iter(A,n.line+1,function(t){l&&o.collapsed&&!l.options.lineWrapping&&Oe(t)==l.display.maxLine&&(s=!0),o.collapsed&&A!=e.line&&Wt(t,0),function(t,e){t.markedSpans=t.markedSpans?t.markedSpans.concat([e]):[e],e.marker.attachLine(t)}(t,new be(o,A==e.line?e.ch:null,A==n.line?n.ch:null)),++A}),o.collapsed&&t.iter(e.line,n.line+1,function(e){je(t,e)&&Wt(e,0)}),o.clearOnEnter&&gt(o,"beforeCursorEnter",function(){return o.clear()}),o.readOnly&&(ve=!0,(t.history.done.length||t.history.undone.length)&&t.clearHistory()),o.collapsed&&(o.id=++vo,o.atomic=!0),l){if(s&&(l.curOp.updateMaxLine=!0),o.collapsed)cr(l,e.line,n.line+1);else if(o.className||o.startStyle||o.endStyle||o.css||o.attributes||o.title)for(var c=e.line;c<=n.line;c++)ur(l,c,"text");o.atomic&&to(l.doc),ln(l,"markerAdded",l,o)}return o}Bo.prototype.clear=function(){if(!this.explicitlyCleared){var t=this.doc.cm,e=t&&!t.curOp;if(e&&Kr(t),Ct(this,"clear")){var n=this.find();n&&ln(this,"clear",n.from,n.to)}for(var r=null,i=null,o=0;o<this.lines.length;++o){var a=this.lines[o],s=Qe(a.markedSpans,this);t&&!this.collapsed?ur(t,Xt(a),"text"):t&&(null!=s.to&&(i=Xt(a)),null!=s.from&&(r=Xt(a))),a.markedSpans=we(a.markedSpans,s),null==s.from&&this.collapsed&&!je(this.doc,a)&&t&&Wt(a,nr(t.display))}if(t&&this.collapsed&&!t.options.lineWrapping)for(var A=0;A<this.lines.length;++A){var l=Oe(this.lines[A]),c=Ke(l);c>t.display.maxLineLength&&(t.display.maxLine=l,t.display.maxLineLength=c,t.display.maxLineChanged=!0)}null!=r&&t&&this.collapsed&&cr(t,r,i+1),this.lines.length=0,this.explicitlyCleared=!0,this.atomic&&this.doc.cantEdit&&(this.doc.cantEdit=!1,t&&to(t.doc)),t&&ln(t,"markerCleared",t,this,r,i),e&&zr(t),this.parent&&this.parent.clear()}},Bo.prototype.find=function(t,e){var n,r;null==t&&"bookmark"==this.type&&(t=1);for(var i=0;i<this.lines.length;++i){var o=this.lines[i],a=Qe(o.markedSpans,this);if(null!=a.from&&(n=ee(e?o:Xt(o),a.from),-1==t))return n;if(null!=a.to&&(r=ee(e?o:Xt(o),a.to),1==t))return r}return n&&{from:n,to:r}},Bo.prototype.changed=function(){var t=this,e=this.find(-1,!0),n=this,r=this.doc.cm;e&&r&&Xr(r,function(){var i=e.line,o=Xt(e.line),a=Sn(r,o);if(a&&(On(a),r.curOp.selectionChanged=r.curOp.forceUpdate=!0),r.curOp.updateMaxLine=!0,!je(n.doc,i)&&null!=n.height){var s=n.height;n.height=null;var A=vn(n)-s;A&&Wt(i,i.height+A)}ln(r,"markerChanged",r,t)})},Bo.prototype.attachLine=function(t){if(!this.lines.length&&this.doc.cm){var e=this.doc.cm.curOp;e.maybeHiddenMarkers&&-1!=G(e.maybeHiddenMarkers,this)||(e.maybeUnhiddenMarkers||(e.maybeUnhiddenMarkers=[])).push(this)}this.lines.push(t)},Bo.prototype.detachLine=function(t){if(this.lines.splice(G(this.lines,t),1),!this.lines.length&&this.doc.cm){var e=this.doc.cm.curOp;(e.maybeHiddenMarkers||(e.maybeHiddenMarkers=[])).push(this)}},mt(Bo);var Qo=function(t,e){this.markers=t,this.primary=e;for(var n=0;n<t.length;++n)t[n].parent=this};function wo(t){return t.findMarks(ee(t.first,0),t.clipPos(ee(t.lastLine())),function(t){return t.parent})}function xo(t){for(var e=function(e){var n=t[e],r=[n.primary.doc];Si(n.primary.doc,function(t){return r.push(t)});for(var i=0;i<n.markers.length;i++){var o=n.markers[i];-1==G(r,o.doc)&&(o.parent=null,n.markers.splice(i--,1))}},n=0;n<t.length;n++)e(n)}Qo.prototype.clear=function(){if(!this.explicitlyCleared){this.explicitlyCleared=!0;for(var t=0;t<this.markers.length;++t)this.markers[t].clear();ln(this,"clear")}},Qo.prototype.find=function(t,e){return this.primary.find(t,e)},mt(Qo);var _o=0,Do=function(t,e,n,r,i){if(!(this instanceof Do))return new Do(t,e,n,r,i);null==n&&(n=0),mo.call(this,[new Co([new Je("",null)])]),this.first=n,this.scrollTop=this.scrollLeft=0,this.cantEdit=!1,this.cleanGeneration=1,this.modeFrontier=this.highlightFrontier=n;var o=ee(n,0);this.sel=Bi(o),this.history=new Ni(null),this.id=++_o,this.modeOption=e,this.lineSep=r,this.direction="rtl"==i?"rtl":"ltr",this.extend=!1,"string"==typeof t&&(t=this.splitLines(t)),ki(this,{from:o,to:o,text:t}),Xi(this,Bi(o),Y)};Do.prototype=Z(mo.prototype,{constructor:Do,iter:function(t,e,n){n?this.iterN(t-this.first,e-t,n):this.iterN(this.first,this.first+this.size,t)},insert:function(t,e){for(var n=0,r=0;r<e.length;++r)n+=e[r].height;this.insertInner(t-this.first,e,n)},remove:function(t,e){this.removeInner(t-this.first,e)},getValue:function(t){var e=Vt(this,this.first,this.first+this.size);return!1===t?e:e.join(t||this.lineSeparator())},setValue:ti(function(t){var e=ee(this.first,0),n=this.first+this.size-1;so(this,{from:e,to:ee(n,Jt(this,n).text.length),text:this.splitLines(t),origin:"setValue",full:!0},!0),this.cm&&kr(this.cm,0,0),Xi(this,Bi(e),Y)}),replaceRange:function(t,e,n,r){e=Ae(this,e),n=n?Ae(this,n):e,go(this,t,e,n,r)},getRange:function(t,e,n){var r=qt(this,Ae(this,t),Ae(this,e));return!1===n?r:r.join(n||this.lineSeparator())},getLine:function(t){var e=this.getLineHandle(t);return e&&e.text},getLineHandle:function(t){if($t(this,t))return Jt(this,t)},getLineNumber:function(t){return Xt(t)},getLineHandleVisualStart:function(t){return"number"==typeof t&&(t=Jt(this,t)),Oe(t)},lineCount:function(){return this.size},firstLine:function(){return this.first},lastLine:function(){return this.first+this.size-1},clipPos:function(t){return Ae(this,t)},getCursor:function(t){var e=this.sel.primary();return null==t||"head"==t?e.head:"anchor"==t?e.anchor:"end"==t||"to"==t||!1===t?e.to():e.from()},listSelections:function(){return this.sel.ranges},somethingSelected:function(){return this.sel.somethingSelected()},setCursor:ti(function(t,e,n){Vi(this,Ae(this,"number"==typeof t?ee(t,e||0):t),null,n)}),setSelection:ti(function(t,e,n){Vi(this,Ae(this,t),Ae(this,e||t),n)}),extendSelection:ti(function(t,e,n){zi(this,Ae(this,t),e&&Ae(this,e),n)}),extendSelections:ti(function(t,e){Ji(this,le(this,t),e)}),extendSelectionsBy:ti(function(t,e){var n=W(this.sel.ranges,t);Ji(this,le(this,n),e)}),setSelections:ti(function(t,e,n){if(t.length){for(var r=[],i=0;i<t.length;i++)r[i]=new Ei(Ae(this,t[i].anchor),Ae(this,t[i].head));null==e&&(e=Math.min(t.length-1,this.sel.primIndex)),Xi(this,vi(this.cm,r,e),n)}}),addSelection:ti(function(t,e,n){var r=this.sel.ranges.slice(0);r.push(new Ei(Ae(this,t),Ae(this,e||t))),Xi(this,vi(this.cm,r,r.length-1),n)}),getSelection:function(t){for(var e,n=this.sel.ranges,r=0;r<n.length;r++){var i=qt(this,n[r].from(),n[r].to());e=e?e.concat(i):i}return!1===t?e:e.join(t||this.lineSeparator())},getSelections:function(t){for(var e=[],n=this.sel.ranges,r=0;r<n.length;r++){var i=qt(this,n[r].from(),n[r].to());!1!==t&&(i=i.join(t||this.lineSeparator())),e[r]=i}return e},replaceSelection:function(t,e,n){for(var r=[],i=0;i<this.sel.ranges.length;i++)r[i]=t;this.replaceSelections(r,e,n||"+input")},replaceSelections:ti(function(t,e,n){for(var r=[],i=this.sel,o=0;o<i.ranges.length;o++){var a=i.ranges[o];r[o]={from:a.from(),to:a.to(),text:this.splitLines(t[o]),origin:n}}for(var s=e&&"end"!=e&&function(t,e,n){for(var r=[],i=ee(t.first,0),o=i,a=0;a<e.length;a++){var s=e[a],A=xi(s.from,i,o),l=xi(bi(s),i,o);if(i=s.to,o=l,"around"==n){var c=t.sel.ranges[a],u=ne(c.head,c.anchor)<0;r[a]=new Ei(u?l:A,u?A:l)}else r[a]=new Ei(A,A)}return new yi(r,t.sel.primIndex)}(this,r,e),A=r.length-1;A>=0;A--)so(this,r[A]);s?Wi(this,s):this.cm&&Mr(this.cm)}),undo:ti(function(){lo(this,"undo")}),redo:ti(function(){lo(this,"redo")}),undoSelection:ti(function(){lo(this,"undo",!0)}),redoSelection:ti(function(){lo(this,"redo",!0)}),setExtending:function(t){this.extend=t},getExtending:function(){return this.extend},historySize:function(){for(var t=this.history,e=0,n=0,r=0;r<t.done.length;r++)t.done[r].ranges||++e;for(var i=0;i<t.undone.length;i++)t.undone[i].ranges||++n;return{undo:e,redo:n}},clearHistory:function(){this.history=new Ni(this.history.maxGeneration)},markClean:function(){this.cleanGeneration=this.changeGeneration(!0)},changeGeneration:function(t){return t&&(this.history.lastOp=this.history.lastSelOp=this.history.lastOrigin=null),this.history.generation},isClean:function(t){return this.history.generation==(t||this.cleanGeneration)},getHistory:function(){return{done:Pi(this.history.done),undone:Pi(this.history.undone)}},setHistory:function(t){var e=this.history=new Ni(this.history.maxGeneration);e.done=Pi(t.done.slice(0),null,!0),e.undone=Pi(t.undone.slice(0),null,!0)},setGutterMarker:ti(function(t,e,n){return Io(this,t,"gutter",function(t){var r=t.gutterMarkers||(t.gutterMarkers={});return r[e]=n,!n&&nt(r)&&(t.gutterMarkers=null),!0})}),clearGutter:ti(function(t){var e=this;this.iter(function(n){n.gutterMarkers&&n.gutterMarkers[t]&&Io(e,n,"gutter",function(){return n.gutterMarkers[t]=null,nt(n.gutterMarkers)&&(n.gutterMarkers=null),!0})})}),lineInfo:function(t){var e;if("number"==typeof t){if(!$t(this,t))return null;if(e=t,!(t=Jt(this,t)))return null}else if(null==(e=Xt(t)))return null;return{line:e,handle:t,text:t.text,gutterMarkers:t.gutterMarkers,textClass:t.textClass,bgClass:t.bgClass,wrapClass:t.wrapClass,widgets:t.widgets}},addLineClass:ti(function(t,e,n){return Io(this,t,"gutter"==e?"gutter":"class",function(t){var r="text"==e?"textClass":"background"==e?"bgClass":"gutter"==e?"gutterClass":"wrapClass";if(t[r]){if(b(n).test(t[r]))return!1;t[r]+=" "+n}else t[r]=n;return!0})}),removeLineClass:ti(function(t,e,n){return Io(this,t,"gutter"==e?"gutter":"class",function(t){var r="text"==e?"textClass":"background"==e?"bgClass":"gutter"==e?"gutterClass":"wrapClass",i=t[r];if(!i)return!1;if(null==n)t[r]=null;else{var o=i.match(b(n));if(!o)return!1;var a=o.index+o[0].length;t[r]=i.slice(0,o.index)+(o.index&&a!=i.length?" ":"")+i.slice(a)||null}return!0})}),addLineWidget:ti(function(t,e,n){return function(t,e,n,r){var i=new yo(t,n,r),o=t.cm;return o&&i.noHScroll&&(o.display.alignWidgets=!0),Io(t,e,"widget",function(e){var n=e.widgets||(e.widgets=[]);if(null==i.insertAt?n.push(i):n.splice(Math.min(n.length-1,Math.max(0,i.insertAt)),0,i),i.line=e,o&&!je(t,e)){var r=Pe(e)<t.scrollTop;Wt(e,e.height+vn(i)),r&&Dr(o,i.height),o.curOp.forceUpdate=!0}return!0}),o&&ln(o,"lineWidgetAdded",o,i,"number"==typeof e?e:Xt(e)),i}(this,t,e,n)}),removeLineWidget:function(t){t.clear()},markText:function(t,e,n){return bo(this,Ae(this,t),Ae(this,e),n,n&&n.type||"range")},setBookmark:function(t,e){var n={replacedWith:e&&(null==e.nodeType?e.widget:e),insertLeft:e&&e.insertLeft,clearWhenEmpty:!1,shared:e&&e.shared,handleMouseEvents:e&&e.handleMouseEvents};return bo(this,t=Ae(this,t),t,n,"bookmark")},findMarksAt:function(t){t=Ae(this,t);var e=[],n=Jt(this,t.line).markedSpans;if(n)for(var r=0;r<n.length;++r){var i=n[r];(null==i.from||i.from<=t.ch)&&(null==i.to||i.to>=t.ch)&&e.push(i.marker.parent||i.marker)}return e},findMarks:function(t,e,n){t=Ae(this,t),e=Ae(this,e);var r=[],i=t.line;return this.iter(t.line,e.line+1,function(o){var a=o.markedSpans;if(a)for(var s=0;s<a.length;s++){var A=a[s];null!=A.to&&i==t.line&&t.ch>=A.to||null==A.from&&i!=t.line||null!=A.from&&i==e.line&&A.from>=e.ch||n&&!n(A.marker)||r.push(A.marker.parent||A.marker)}++i}),r},getAllMarks:function(){var t=[];return this.iter(function(e){var n=e.markedSpans;if(n)for(var r=0;r<n.length;++r)null!=n[r].from&&t.push(n[r].marker)}),t},posFromIndex:function(t){var e,n=this.first,r=this.lineSeparator().length;return this.iter(function(i){var o=i.text.length+r;if(o>t)return e=t,!0;t-=o,++n}),Ae(this,ee(n,e))},indexFromPos:function(t){var e=(t=Ae(this,t)).ch;if(t.line<this.first||t.ch<0)return 0;var n=this.lineSeparator().length;return this.iter(this.first,t.line,function(t){e+=t.text.length+n}),e},copy:function(t){var e=new Do(Vt(this,this.first,this.first+this.size),this.modeOption,this.first,this.lineSep,this.direction);return e.scrollTop=this.scrollTop,e.scrollLeft=this.scrollLeft,e.sel=this.sel,e.extend=!1,t&&(e.history.undoDepth=this.history.undoDepth,e.setHistory(this.getHistory())),e},linkedDoc:function(t){t||(t={});var e=this.first,n=this.first+this.size;null!=t.from&&t.from>e&&(e=t.from),null!=t.to&&t.to<n&&(n=t.to);var r=new Do(Vt(this,e,n),t.mode||this.modeOption,e,this.lineSep,this.direction);return t.sharedHist&&(r.history=this.history),(this.linked||(this.linked=[])).push({doc:r,sharedHist:t.sharedHist}),r.linked=[{doc:this,isParent:!0,sharedHist:t.sharedHist}],function(t,e){for(var n=0;n<e.length;n++){var r=e[n],i=r.find(),o=t.clipPos(i.from),a=t.clipPos(i.to);if(ne(o,a)){var s=bo(t,o,a,r.primary,r.primary.type);r.markers.push(s),s.parent=r}}}(r,wo(this)),r},unlinkDoc:function(t){if(t instanceof Qa&&(t=t.doc),this.linked)for(var e=0;e<this.linked.length;++e){var n=this.linked[e];if(n.doc==t){this.linked.splice(e,1),t.unlinkDoc(this),xo(wo(this));break}}if(t.history==this.history){var r=[t.id];Si(t,function(t){return r.push(t.id)},!0),t.history=new Ni(null),t.history.done=Pi(this.history.done,r),t.history.undone=Pi(this.history.undone,r)}},iterLinkedDocs:function(t){Si(this,t)},getMode:function(){return this.mode},getEditor:function(){return this.cm},splitLines:function(t){return this.lineSep?t.split(this.lineSep):St(t)},lineSeparator:function(){return this.lineSep||"\n"},setDirection:ti(function(t){var e;"rtl"!=t&&(t="ltr"),t!=this.direction&&(this.direction=t,this.iter(function(t){return t.order=null}),this.cm&&Xr(e=this.cm,function(){Ri(e),cr(e)}))})}),Do.prototype.eachLine=Do.prototype.iter;var Mo=0;function ko(t){var e=this;if(So(e),!pt(e,t)&&!Bn(e.display,t)){yt(t),a&&(Mo=+new Date);var n=Ar(e,t,!0),r=t.dataTransfer.files;if(n&&!e.isReadOnly())if(r&&r.length&&window.FileReader&&window.File)for(var i=r.length,o=Array(i),s=0,A=function(t,r){if(!e.options.allowDropFileTypes||-1!=G(e.options.allowDropFileTypes,t.type)){var a=new FileReader;a.onload=Zr(e,function(){var t=a.result;if(/[\x00-\x08\x0e-\x1f]{2}/.test(t)&&(t=""),o[r]=t,++s==i){var A={from:n=Ae(e.doc,n),to:n,text:e.doc.splitLines(o.join(e.doc.lineSeparator())),origin:"paste"};so(e.doc,A),Wi(e.doc,Bi(n,bi(A)))}}),a.readAsText(t)}},l=0;l<i;++l)A(r[l],l);else{if(e.state.draggingText&&e.doc.sel.contains(n)>-1)return e.state.draggingText(t),void setTimeout(function(){return e.display.input.focus()},20);try{var c=t.dataTransfer.getData("Text");if(c){var u;if(e.state.draggingText&&!e.state.draggingText.copy&&(u=e.listSelections()),Zi(e.doc,Bi(n,n)),u)for(var g=0;g<u.length;++g)go(e.doc,"",u[g].anchor,u[g].head,"drag");e.replaceSelection(c,"around","paste"),e.display.input.focus()}}catch(t){}}}}function So(t){t.display.dragCursor&&(t.display.lineSpace.removeChild(t.display.dragCursor),t.display.dragCursor=null)}function Lo(t){if(document.getElementsByClassName){for(var e=document.getElementsByClassName("CodeMirror"),n=[],r=0;r<e.length;r++){var i=e[r].CodeMirror;i&&n.push(i)}n.length&&n[0].operation(function(){for(var e=0;e<n.length;e++)t(n[e])})}}var Ro=!1;function No(){var t;Ro||(gt(window,"resize",function(){null==t&&(t=setTimeout(function(){t=null,Lo(Fo)},100))}),gt(window,"blur",function(){return Lo(br)}),Ro=!0)}function Fo(t){var e=t.display;e.cachedCharWidth=e.cachedTextHeight=e.cachedPaddingH=null,e.scrollbarsClipped=!1,t.setSize()}for(var To={3:"Pause",8:"Backspace",9:"Tab",13:"Enter",16:"Shift",17:"Ctrl",18:"Alt",19:"Pause",20:"CapsLock",27:"Esc",32:"Space",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"Left",38:"Up",39:"Right",40:"Down",44:"PrintScrn",45:"Insert",46:"Delete",59:";",61:"=",91:"Mod",92:"Mod",93:"Mod",106:"*",107:"=",109:"-",110:".",111:"/",145:"ScrollLock",173:"-",186:";",187:"=",188:",",189:"-",190:".",191:"/",192:"`",219:"[",220:"\\",221:"]",222:"'",63232:"Up",63233:"Down",63234:"Left",63235:"Right",63272:"Delete",63273:"Home",63275:"End",63276:"PageUp",63277:"PageDown",63302:"Insert"},Uo=0;Uo<10;Uo++)To[Uo+48]=To[Uo+96]=String(Uo);for(var Oo=65;Oo<=90;Oo++)To[Oo]=String.fromCharCode(Oo);for(var Go=1;Go<=12;Go++)To[Go+111]=To[Go+63235]="F"+Go;var Ho={};function jo(t){var e,n,r,i,o=t.split(/-(?!$)/);t=o[o.length-1];for(var a=0;a<o.length-1;a++){var s=o[a];if(/^(cmd|meta|m)$/i.test(s))i=!0;else if(/^a(lt)?$/i.test(s))e=!0;else if(/^(c|ctrl|control)$/i.test(s))n=!0;else{if(!/^s(hift)?$/i.test(s))throw new Error("Unrecognized modifier name: "+s);r=!0}}return e&&(t="Alt-"+t),n&&(t="Ctrl-"+t),i&&(t="Cmd-"+t),r&&(t="Shift-"+t),t}function Yo(t){var e={};for(var n in t)if(t.hasOwnProperty(n)){var r=t[n];if(/^(name|fallthrough|(de|at)tach)$/.test(n))continue;if("..."==r){delete t[n];continue}for(var i=W(n.split(" "),jo),o=0;o<i.length;o++){var a=void 0,s=void 0;o==i.length-1?(s=i.join(" "),a=r):(s=i.slice(0,o+1).join(" "),a="...");var A=e[s];if(A){if(A!=a)throw new Error("Inconsistent bindings for "+s)}else e[s]=a}delete t[n]}for(var l in e)t[l]=e[l];return t}function Po(t,e,n,r){var i=(e=qo(e)).call?e.call(t,r):e[t];if(!1===i)return"nothing";if("..."===i)return"multi";if(null!=i&&n(i))return"handled";if(e.fallthrough){if("[object Array]"!=Object.prototype.toString.call(e.fallthrough))return Po(t,e.fallthrough,n,r);for(var o=0;o<e.fallthrough.length;o++){var a=Po(t,e.fallthrough[o],n,r);if(a)return a}}}function Ko(t){var e="string"==typeof t?t:To[t.keyCode];return"Ctrl"==e||"Alt"==e||"Shift"==e||"Mod"==e}function zo(t,e,n){var r=t;return e.altKey&&"Alt"!=r&&(t="Alt-"+t),(v?e.metaKey:e.ctrlKey)&&"Ctrl"!=r&&(t="Ctrl-"+t),(v?e.ctrlKey:e.metaKey)&&"Cmd"!=r&&(t="Cmd-"+t),!n&&e.shiftKey&&"Shift"!=r&&(t="Shift-"+t),t}function Jo(t,e){if(u&&34==t.keyCode&&t.char)return!1;var n=To[t.keyCode];return null!=n&&!t.altGraphKey&&(3==t.keyCode&&t.code&&(n=t.code),zo(n,t,e))}function qo(t){return"string"==typeof t?Ho[t]:t}function Vo(t,e){for(var n=t.doc.sel.ranges,r=[],i=0;i<n.length;i++){for(var o=e(n[i]);r.length&&ne(o.from,V(r).to)<=0;){var a=r.pop();if(ne(a.from,o.from)<0){o.from=a.from;break}}r.push(o)}Xr(t,function(){for(var e=r.length-1;e>=0;e--)go(t.doc,"",r[e].from,r[e].to,"+delete");Mr(t)})}function Wo(t,e,n){var r=ot(t.text,e+n,n);return r<0||r>t.text.length?null:r}function Xo(t,e,n){var r=Wo(t,e.ch,n);return null==r?null:new ee(e.line,r,n<0?"after":"before")}function Zo(t,e,n,r,i){if(t){var o=ct(n,e.doc.direction);if(o){var a,s=i<0?V(o):o[0],A=i<0==(1==s.level),l=A?"after":"before";if(s.level>0||"rtl"==e.doc.direction){var c=Ln(e,n);a=i<0?n.text.length-1:0;var u=Rn(e,c,a).top;a=at(function(t){return Rn(e,c,t).top==u},i<0==(1==s.level)?s.from:s.to-1,a),"before"==l&&(a=Wo(n,a,1))}else a=i<0?s.to:s.from;return new ee(r,a,l)}}return new ee(r,i<0?n.text.length:0,i<0?"before":"after")}Ho.basic={Left:"goCharLeft",Right:"goCharRight",Up:"goLineUp",Down:"goLineDown",End:"goLineEnd",Home:"goLineStartSmart",PageUp:"goPageUp",PageDown:"goPageDown",Delete:"delCharAfter",Backspace:"delCharBefore","Shift-Backspace":"delCharBefore",Tab:"defaultTab","Shift-Tab":"indentAuto",Enter:"newlineAndIndent",Insert:"toggleOverwrite",Esc:"singleSelection"},Ho.pcDefault={"Ctrl-A":"selectAll","Ctrl-D":"deleteLine","Ctrl-Z":"undo","Shift-Ctrl-Z":"redo","Ctrl-Y":"redo","Ctrl-Home":"goDocStart","Ctrl-End":"goDocEnd","Ctrl-Up":"goLineUp","Ctrl-Down":"goLineDown","Ctrl-Left":"goGroupLeft","Ctrl-Right":"goGroupRight","Alt-Left":"goLineStart","Alt-Right":"goLineEnd","Ctrl-Backspace":"delGroupBefore","Ctrl-Delete":"delGroupAfter","Ctrl-S":"save","Ctrl-F":"find","Ctrl-G":"findNext","Shift-Ctrl-G":"findPrev","Shift-Ctrl-F":"replace","Shift-Ctrl-R":"replaceAll","Ctrl-[":"indentLess","Ctrl-]":"indentMore","Ctrl-U":"undoSelection","Shift-Ctrl-U":"redoSelection","Alt-U":"redoSelection",fallthrough:"basic"},Ho.emacsy={"Ctrl-F":"goCharRight","Ctrl-B":"goCharLeft","Ctrl-P":"goLineUp","Ctrl-N":"goLineDown","Alt-F":"goWordRight","Alt-B":"goWordLeft","Ctrl-A":"goLineStart","Ctrl-E":"goLineEnd","Ctrl-V":"goPageDown","Shift-Ctrl-V":"goPageUp","Ctrl-D":"delCharAfter","Ctrl-H":"delCharBefore","Alt-D":"delWordAfter","Alt-Backspace":"delWordBefore","Ctrl-K":"killLine","Ctrl-T":"transposeChars","Ctrl-O":"openLine"},Ho.macDefault={"Cmd-A":"selectAll","Cmd-D":"deleteLine","Cmd-Z":"undo","Shift-Cmd-Z":"redo","Cmd-Y":"redo","Cmd-Home":"goDocStart","Cmd-Up":"goDocStart","Cmd-End":"goDocEnd","Cmd-Down":"goDocEnd","Alt-Left":"goGroupLeft","Alt-Right":"goGroupRight","Cmd-Left":"goLineLeft","Cmd-Right":"goLineRight","Alt-Backspace":"delGroupBefore","Ctrl-Alt-Backspace":"delGroupAfter","Alt-Delete":"delGroupAfter","Cmd-S":"save","Cmd-F":"find","Cmd-G":"findNext","Shift-Cmd-G":"findPrev","Cmd-Alt-F":"replace","Shift-Cmd-Alt-F":"replaceAll","Cmd-[":"indentLess","Cmd-]":"indentMore","Cmd-Backspace":"delWrappedLineLeft","Cmd-Delete":"delWrappedLineRight","Cmd-U":"undoSelection","Shift-Cmd-U":"redoSelection","Ctrl-Up":"goDocStart","Ctrl-Down":"goDocEnd",fallthrough:["basic","emacsy"]},Ho.default=C?Ho.macDefault:Ho.pcDefault;var $o={selectAll:oo,singleSelection:function(t){return t.setSelection(t.getCursor("anchor"),t.getCursor("head"),Y)},killLine:function(t){return Vo(t,function(e){if(e.empty()){var n=Jt(t.doc,e.head.line).text.length;return e.head.ch==n&&e.head.line<t.lastLine()?{from:e.head,to:ee(e.head.line+1,0)}:{from:e.head,to:ee(e.head.line,n)}}return{from:e.from(),to:e.to()}})},deleteLine:function(t){return Vo(t,function(e){return{from:ee(e.from().line,0),to:Ae(t.doc,ee(e.to().line+1,0))}})},delLineLeft:function(t){return Vo(t,function(t){return{from:ee(t.from().line,0),to:t.from()}})},delWrappedLineLeft:function(t){return Vo(t,function(e){var n=t.charCoords(e.head,"div").top+5,r=t.coordsChar({left:0,top:n},"div");return{from:r,to:e.from()}})},delWrappedLineRight:function(t){return Vo(t,function(e){var n=t.charCoords(e.head,"div").top+5,r=t.coordsChar({left:t.display.lineDiv.offsetWidth+100,top:n},"div");return{from:e.from(),to:r}})},undo:function(t){return t.undo()},redo:function(t){return t.redo()},undoSelection:function(t){return t.undoSelection()},redoSelection:function(t){return t.redoSelection()},goDocStart:function(t){return t.extendSelection(ee(t.firstLine(),0))},goDocEnd:function(t){return t.extendSelection(ee(t.lastLine()))},goLineStart:function(t){return t.extendSelectionsBy(function(e){return ta(t,e.head.line)},{origin:"+move",bias:1})},goLineStartSmart:function(t){return t.extendSelectionsBy(function(e){return ea(t,e.head)},{origin:"+move",bias:1})},goLineEnd:function(t){return t.extendSelectionsBy(function(e){return function(t,e){var n=Jt(t.doc,e),r=function(t){for(var e;e=Fe(t);)t=e.find(1,!0).line;return t}(n);return r!=n&&(e=Xt(r)),Zo(!0,t,n,e,-1)}(t,e.head.line)},{origin:"+move",bias:-1})},goLineRight:function(t){return t.extendSelectionsBy(function(e){var n=t.cursorCoords(e.head,"div").top+5;return t.coordsChar({left:t.display.lineDiv.offsetWidth+100,top:n},"div")},K)},goLineLeft:function(t){return t.extendSelectionsBy(function(e){var n=t.cursorCoords(e.head,"div").top+5;return t.coordsChar({left:0,top:n},"div")},K)},goLineLeftSmart:function(t){return t.extendSelectionsBy(function(e){var n=t.cursorCoords(e.head,"div").top+5,r=t.coordsChar({left:0,top:n},"div");return r.ch<t.getLine(r.line).search(/\S/)?ea(t,e.head):r},K)},goLineUp:function(t){return t.moveV(-1,"line")},goLineDown:function(t){return t.moveV(1,"line")},goPageUp:function(t){return t.moveV(-1,"page")},goPageDown:function(t){return t.moveV(1,"page")},goCharLeft:function(t){return t.moveH(-1,"char")},goCharRight:function(t){return t.moveH(1,"char")},goColumnLeft:function(t){return t.moveH(-1,"column")},goColumnRight:function(t){return t.moveH(1,"column")},goWordLeft:function(t){return t.moveH(-1,"word")},goGroupRight:function(t){return t.moveH(1,"group")},goGroupLeft:function(t){return t.moveH(-1,"group")},goWordRight:function(t){return t.moveH(1,"word")},delCharBefore:function(t){return t.deleteH(-1,"char")},delCharAfter:function(t){return t.deleteH(1,"char")},delWordBefore:function(t){return t.deleteH(-1,"word")},delWordAfter:function(t){return t.deleteH(1,"word")},delGroupBefore:function(t){return t.deleteH(-1,"group")},delGroupAfter:function(t){return t.deleteH(1,"group")},indentAuto:function(t){return t.indentSelection("smart")},indentMore:function(t){return t.indentSelection("add")},indentLess:function(t){return t.indentSelection("subtract")},insertTab:function(t){return t.replaceSelection("\t")},insertSoftTab:function(t){for(var e=[],n=t.listSelections(),r=t.options.tabSize,i=0;i<n.length;i++){var o=n[i].from(),a=U(t.getLine(o.line),o.ch,r);e.push(q(r-a%r))}t.replaceSelections(e)},defaultTab:function(t){t.somethingSelected()?t.indentSelection("add"):t.execCommand("insertTab")},transposeChars:function(t){return Xr(t,function(){for(var e=t.listSelections(),n=[],r=0;r<e.length;r++)if(e[r].empty()){var i=e[r].head,o=Jt(t.doc,i.line).text;if(o)if(i.ch==o.length&&(i=new ee(i.line,i.ch-1)),i.ch>0)i=new ee(i.line,i.ch+1),t.replaceRange(o.charAt(i.ch-1)+o.charAt(i.ch-2),ee(i.line,i.ch-2),i,"+transpose");else if(i.line>t.doc.first){var a=Jt(t.doc,i.line-1).text;a&&(i=new ee(i.line,1),t.replaceRange(o.charAt(0)+t.doc.lineSeparator()+a.charAt(a.length-1),ee(i.line-1,a.length-1),i,"+transpose"))}n.push(new Ei(i,i))}t.setSelections(n)})},newlineAndIndent:function(t){return Xr(t,function(){for(var e=t.listSelections(),n=e.length-1;n>=0;n--)t.replaceRange(t.doc.lineSeparator(),e[n].anchor,e[n].head,"+input");e=t.listSelections();for(var r=0;r<e.length;r++)t.indentLine(e[r].from().line,null,!0);Mr(t)})},openLine:function(t){return t.replaceSelection("\n","start")},toggleOverwrite:function(t){return t.toggleOverwrite()}};function ta(t,e){var n=Jt(t.doc,e),r=Oe(n);return r!=n&&(e=Xt(r)),Zo(!0,t,r,e,1)}function ea(t,e){var n=ta(t,e.line),r=Jt(t.doc,n.line),i=ct(r,t.doc.direction);if(!i||0==i[0].level){var o=Math.max(0,r.text.search(/\S/)),a=e.line==n.line&&e.ch<=o&&e.ch;return ee(n.line,a?0:o,n.sticky)}return n}function na(t,e,n){if("string"==typeof e&&!(e=$o[e]))return!1;t.display.input.ensurePolled();var r=t.display.shift,i=!1;try{t.isReadOnly()&&(t.state.suppressEdits=!0),n&&(t.display.shift=!1),i=e(t)!=j}finally{t.display.shift=r,t.state.suppressEdits=!1}return i}var ra=new O;function ia(t,e,n,r){var i=t.state.keySeq;if(i){if(Ko(e))return"handled";if(/\'$/.test(e)?t.state.keySeq=null:ra.set(50,function(){t.state.keySeq==i&&(t.state.keySeq=null,t.display.input.reset())}),oa(t,i+" "+e,n,r))return!0}return oa(t,e,n,r)}function oa(t,e,n,r){var i=function(t,e,n){for(var r=0;r<t.state.keyMaps.length;r++){var i=Po(e,t.state.keyMaps[r],n,t);if(i)return i}return t.options.extraKeys&&Po(e,t.options.extraKeys,n,t)||Po(e,t.options.keyMap,n,t)}(t,e,r);return"multi"==i&&(t.state.keySeq=e),"handled"==i&&ln(t,"keyHandled",t,e,n),"handled"!=i&&"multi"!=i||(yt(n),yr(t)),!!i}function aa(t,e){var n=Jo(e,!0);return!!n&&(e.shiftKey&&!t.state.keySeq?ia(t,"Shift-"+n,e,function(e){return na(t,e,!0)})||ia(t,n,e,function(e){if("string"==typeof e?/^go[A-Z]/.test(e):e.motion)return na(t,e)}):ia(t,n,e,function(e){return na(t,e)}))}var sa=null;function Aa(t){var e=this;if(e.curOp.focus=S(),!pt(e,t)){a&&s<11&&27==t.keyCode&&(t.returnValue=!1);var n=t.keyCode;e.display.shift=16==n||t.shiftKey;var r=aa(e,t);u&&(sa=r?n:null,!r&&88==n&&!Rt&&(C?t.metaKey:t.ctrlKey)&&e.replaceSelection("",null,"cut")),18!=n||/\bCodeMirror-crosshair\b/.test(e.display.lineDiv.className)||function(t){var e=t.display.lineDiv;function n(t){18!=t.keyCode&&t.altKey||(w(e,"CodeMirror-crosshair"),dt(document,"keyup",n),dt(document,"mouseover",n))}L(e,"CodeMirror-crosshair"),gt(document,"keyup",n),gt(document,"mouseover",n)}(e)}}function la(t){16==t.keyCode&&(this.doc.sel.shift=!1),pt(this,t)}function ca(t){var e=this;if(!(Bn(e.display,t)||pt(e,t)||t.ctrlKey&&!t.altKey||C&&t.metaKey)){var n=t.keyCode,r=t.charCode;if(u&&n==sa)return sa=null,void yt(t);if(!u||t.which&&!(t.which<10)||!aa(e,t)){var i=String.fromCharCode(null==r?n:r);"\b"!=i&&(function(t,e,n){return ia(t,"'"+n+"'",e,function(e){return na(t,e,!0)})}(e,t,i)||e.display.input.onKeyPress(t))}}}var ua,ga,ha=function(t,e,n){this.time=t,this.pos=e,this.button=n};function da(t){var e=this,n=e.display;if(!(pt(e,t)||n.activeTouch&&n.input.supportsTouch()))if(n.input.ensurePolled(),n.shift=t.shiftKey,Bn(n,t))A||(n.scroller.draggable=!1,setTimeout(function(){return n.scroller.draggable=!0},100));else if(!Ia(e,t)){var r=Ar(e,t),i=Qt(t),o=r?function(t,e){var n=+new Date;return ga&&ga.compare(n,t,e)?(ua=ga=null,"triple"):ua&&ua.compare(n,t,e)?(ga=new ha(n,t,e),ua=null,"double"):(ua=new ha(n,t,e),ga=null,"single")}(r,i):"single";window.focus(),1==i&&e.state.selectingText&&e.state.selectingText(t),r&&function(t,e,n,r,i){var o="Click";return"double"==r?o="Double"+o:"triple"==r&&(o="Triple"+o),ia(t,zo(o=(1==e?"Left":2==e?"Middle":"Right")+o,i),i,function(e){if("string"==typeof e&&(e=$o[e]),!e)return!1;var r=!1;try{t.isReadOnly()&&(t.state.suppressEdits=!0),r=e(t,n)!=j}finally{t.state.suppressEdits=!1}return r})}(e,i,r,o,t)||(1==i?r?function(t,e,n,r){a?setTimeout(F(Er,t),0):t.curOp.focus=S();var i,o=function(t,e,n){var r=t.getOption("configureMouse"),i=r?r(t,e,n):{};if(null==i.unit){var o=m?n.shiftKey&&n.metaKey:n.altKey;i.unit=o?"rectangle":"single"==e?"char":"double"==e?"word":"line"}return(null==i.extend||t.doc.extend)&&(i.extend=t.doc.extend||n.shiftKey),null==i.addNew&&(i.addNew=C?n.metaKey:n.ctrlKey),null==i.moveOnDrag&&(i.moveOnDrag=!(C?n.altKey:n.ctrlKey)),i}(t,n,r),l=t.doc.sel;t.options.dragDrop&&_t&&!t.isReadOnly()&&"single"==n&&(i=l.contains(e))>-1&&(ne((i=l.ranges[i]).from(),e)<0||e.xRel>0)&&(ne(i.to(),e)>0||e.xRel<0)?function(t,e,n,r){var i=t.display,o=!1,l=Zr(t,function(e){A&&(i.scroller.draggable=!1),t.state.draggingText=!1,dt(i.wrapper.ownerDocument,"mouseup",l),dt(i.wrapper.ownerDocument,"mousemove",c),dt(i.scroller,"dragstart",u),dt(i.scroller,"drop",l),o||(yt(e),r.addNew||zi(t.doc,n,null,null,r.extend),A||a&&9==s?setTimeout(function(){i.wrapper.ownerDocument.body.focus(),i.input.focus()},20):i.input.focus())}),c=function(t){o=o||Math.abs(e.clientX-t.clientX)+Math.abs(e.clientY-t.clientY)>=10},u=function(){return o=!0};A&&(i.scroller.draggable=!0),t.state.draggingText=l,l.copy=!r.moveOnDrag,i.scroller.dragDrop&&i.scroller.dragDrop(),gt(i.wrapper.ownerDocument,"mouseup",l),gt(i.wrapper.ownerDocument,"mousemove",c),gt(i.scroller,"dragstart",u),gt(i.scroller,"drop",l),vr(t),setTimeout(function(){return i.input.focus()},20)}(t,r,e,o):function(t,e,n,r){var i=t.display,o=t.doc;yt(e);var a,s,A=o.sel,l=A.ranges;if(r.addNew&&!r.extend?(s=o.sel.contains(n),a=s>-1?l[s]:new Ei(n,n)):(a=o.sel.primary(),s=o.sel.primIndex),"rectangle"==r.unit)r.addNew||(a=new Ei(n,n)),n=Ar(t,e,!0,!0),s=-1;else{var c=fa(t,n,r.unit);a=r.extend?Ki(a,c.anchor,c.head,r.extend):c}r.addNew?-1==s?(s=l.length,Xi(o,vi(t,l.concat([a]),s),{scroll:!1,origin:"*mouse"})):l.length>1&&l[s].empty()&&"char"==r.unit&&!r.extend?(Xi(o,vi(t,l.slice(0,s).concat(l.slice(s+1)),0),{scroll:!1,origin:"*mouse"}),A=o.sel):qi(o,s,a,P):(s=0,Xi(o,new yi([a],0),P),A=o.sel);var u=n;function g(e){if(0!=ne(u,e))if(u=e,"rectangle"==r.unit){for(var i=[],l=t.options.tabSize,c=U(Jt(o,n.line).text,n.ch,l),g=U(Jt(o,e.line).text,e.ch,l),h=Math.min(c,g),d=Math.max(c,g),f=Math.min(n.line,e.line),p=Math.min(t.lastLine(),Math.max(n.line,e.line));f<=p;f++){var I=Jt(o,f).text,C=z(I,h,l);h==d?i.push(new Ei(ee(f,C),ee(f,C))):I.length>C&&i.push(new Ei(ee(f,C),ee(f,z(I,d,l))))}i.length||i.push(new Ei(n,n)),Xi(o,vi(t,A.ranges.slice(0,s).concat(i),s),{origin:"*mouse",scroll:!1}),t.scrollIntoView(e)}else{var m,y=a,E=fa(t,e,r.unit),v=y.anchor;ne(E.anchor,v)>0?(m=E.head,v=ae(y.from(),E.anchor)):(m=E.anchor,v=oe(y.to(),E.head));var B=A.ranges.slice(0);B[s]=function(t,e){var n=e.anchor,r=e.head,i=Jt(t.doc,n.line);if(0==ne(n,r)&&n.sticky==r.sticky)return e;var o=ct(i);if(!o)return e;var a=At(o,n.ch,n.sticky),s=o[a];if(s.from!=n.ch&&s.to!=n.ch)return e;var A,l=a+(s.from==n.ch==(1!=s.level)?0:1);if(0==l||l==o.length)return e;if(r.line!=n.line)A=(r.line-n.line)*("ltr"==t.doc.direction?1:-1)>0;else{var c=At(o,r.ch,r.sticky),u=c-a||(r.ch-n.ch)*(1==s.level?-1:1);A=c==l-1||c==l?u<0:u>0}var g=o[l+(A?-1:0)],h=A==(1==g.level),d=h?g.from:g.to,f=h?"after":"before";return n.ch==d&&n.sticky==f?e:new Ei(new ee(n.line,d,f),r)}(t,new Ei(Ae(o,v),m)),Xi(o,vi(t,B,s),P)}}var h=i.wrapper.getBoundingClientRect(),d=0;function f(e){t.state.selectingText=!1,d=1/0,e&&(yt(e),i.input.focus()),dt(i.wrapper.ownerDocument,"mousemove",p),dt(i.wrapper.ownerDocument,"mouseup",I),o.history.lastSelOrigin=null}var p=Zr(t,function(e){0!==e.buttons&&Qt(e)?function e(n){var a=++d,s=Ar(t,n,!0,"rectangle"==r.unit);if(s)if(0!=ne(s,u)){t.curOp.focus=S(),g(s);var A=xr(i,o);(s.line>=A.to||s.line<A.from)&&setTimeout(Zr(t,function(){d==a&&e(n)}),150)}else{var l=n.clientY<h.top?-20:n.clientY>h.bottom?20:0;l&&setTimeout(Zr(t,function(){d==a&&(i.scroller.scrollTop+=l,e(n))}),50)}}(e):f(e)}),I=Zr(t,f);t.state.selectingText=I,gt(i.wrapper.ownerDocument,"mousemove",p),gt(i.wrapper.ownerDocument,"mouseup",I)}(t,r,e,o)}(e,r,o,t):bt(t)==n.scroller&&yt(t):2==i?(r&&zi(e.doc,r),setTimeout(function(){return n.input.focus()},20)):3==i&&(B?e.display.input.onContextMenu(t):vr(e)))}}function fa(t,e,n){if("char"==n)return new Ei(e,e);if("word"==n)return t.findWordAt(e);if("line"==n)return new Ei(ee(e.line,0),Ae(t.doc,ee(e.line+1,0)));var r=n(t,e);return new Ei(r.from,r.to)}function pa(t,e,n,r){var i,o;if(e.touches)i=e.touches[0].clientX,o=e.touches[0].clientY;else try{i=e.clientX,o=e.clientY}catch(e){return!1}if(i>=Math.floor(t.display.gutters.getBoundingClientRect().right))return!1;r&&yt(e);var a=t.display,s=a.lineDiv.getBoundingClientRect();if(o>s.bottom||!Ct(t,n))return vt(e);o-=s.top-a.viewOffset;for(var A=0;A<t.display.gutterSpecs.length;++A){var l=a.gutters.childNodes[A];if(l&&l.getBoundingClientRect().right>=i){var c=Zt(t.doc,o),u=t.display.gutterSpecs[A];return ft(t,n,t,c,u.className,e),vt(e)}}}function Ia(t,e){return pa(t,e,"gutterClick",!0)}function Ca(t,e){Bn(t.display,e)||function(t,e){return!!Ct(t,"gutterContextMenu")&&pa(t,e,"gutterContextMenu",!1)}(t,e)||pt(t,e,"contextmenu")||B||t.display.input.onContextMenu(e)}function ma(t){t.display.wrapper.className=t.display.wrapper.className.replace(/\s*cm-s-\S+/g,"")+t.options.theme.replace(/(^|\s)\s*/g," cm-s-"),Hn(t)}ha.prototype.compare=function(t,e,n){return this.time+400>t&&0==ne(e,this.pos)&&n==this.button};var ya={toString:function(){return"CodeMirror.Init"}},Ea={},va={};function Ba(t,e,n){var r=n&&n!=ya;if(!e!=!r){var i=t.display.dragFunctions,o=e?gt:dt;o(t.display.scroller,"dragstart",i.start),o(t.display.scroller,"dragenter",i.enter),o(t.display.scroller,"dragover",i.over),o(t.display.scroller,"dragleave",i.leave),o(t.display.scroller,"drop",i.drop)}}function ba(t){t.options.lineWrapping?(L(t.display.wrapper,"CodeMirror-wrap"),t.display.sizer.style.minWidth="",t.display.sizerWidth=null):(w(t.display.wrapper,"CodeMirror-wrap"),ze(t)),sr(t),cr(t),Hn(t),setTimeout(function(){return Gr(t)},100)}function Qa(t,e){var n=this;if(!(this instanceof Qa))return new Qa(t,e);this.options=e=e?T(e):{},T(Ea,e,!1);var r=e.value;"string"==typeof r?r=new Do(r,e.mode,null,e.lineSeparator,e.direction):e.mode&&(r.modeOption=e.mode),this.doc=r;var i=new Qa.inputStyles[e.inputStyle](this),o=this.display=new di(t,r,i,e);for(var l in o.wrapper.CodeMirror=this,ma(this),e.lineWrapping&&(this.display.wrapper.className+=" CodeMirror-wrap"),Yr(this),this.state={keyMaps:[],overlays:[],modeGen:0,overwrite:!1,delayingBlurEvent:!1,focused:!1,suppressEdits:!1,pasteIncoming:-1,cutIncoming:-1,selectingText:!1,draggingText:!1,highlight:new O,keySeq:null,specialChars:null},e.autofocus&&!I&&o.input.focus(),a&&s<11&&setTimeout(function(){return n.display.input.reset(!0)},20),function(t){var e=t.display;gt(e.scroller,"mousedown",Zr(t,da)),gt(e.scroller,"dblclick",a&&s<11?Zr(t,function(e){if(!pt(t,e)){var n=Ar(t,e);if(n&&!Ia(t,e)&&!Bn(t.display,e)){yt(e);var r=t.findWordAt(n);zi(t.doc,r.anchor,r.head)}}}):function(e){return pt(t,e)||yt(e)}),gt(e.scroller,"contextmenu",function(e){return Ca(t,e)});var n,r={end:0};function i(){e.activeTouch&&(n=setTimeout(function(){return e.activeTouch=null},1e3),(r=e.activeTouch).end=+new Date)}function o(t,e){if(null==e.left)return!0;var n=e.left-t.left,r=e.top-t.top;return n*n+r*r>400}gt(e.scroller,"touchstart",function(i){if(!pt(t,i)&&!function(t){if(1!=t.touches.length)return!1;var e=t.touches[0];return e.radiusX<=1&&e.radiusY<=1}(i)&&!Ia(t,i)){e.input.ensurePolled(),clearTimeout(n);var o=+new Date;e.activeTouch={start:o,moved:!1,prev:o-r.end<=300?r:null},1==i.touches.length&&(e.activeTouch.left=i.touches[0].pageX,e.activeTouch.top=i.touches[0].pageY)}}),gt(e.scroller,"touchmove",function(){e.activeTouch&&(e.activeTouch.moved=!0)}),gt(e.scroller,"touchend",function(n){var r=e.activeTouch;if(r&&!Bn(e,n)&&null!=r.left&&!r.moved&&new Date-r.start<300){var a,s=t.coordsChar(e.activeTouch,"page");a=!r.prev||o(r,r.prev)?new Ei(s,s):!r.prev.prev||o(r,r.prev.prev)?t.findWordAt(s):new Ei(ee(s.line,0),Ae(t.doc,ee(s.line+1,0))),t.setSelection(a.anchor,a.head),t.focus(),yt(n)}i()}),gt(e.scroller,"touchcancel",i),gt(e.scroller,"scroll",function(){e.scroller.clientHeight&&(Rr(t,e.scroller.scrollTop),Fr(t,e.scroller.scrollLeft,!0),ft(t,"scroll",t))}),gt(e.scroller,"mousewheel",function(e){return mi(t,e)}),gt(e.scroller,"DOMMouseScroll",function(e){return mi(t,e)}),gt(e.wrapper,"scroll",function(){return e.wrapper.scrollTop=e.wrapper.scrollLeft=0}),e.dragFunctions={enter:function(e){pt(t,e)||Bt(e)},over:function(e){pt(t,e)||(function(t,e){var n=Ar(t,e);if(n){var r=document.createDocumentFragment();Ir(t,n,r),t.display.dragCursor||(t.display.dragCursor=D("div",null,"CodeMirror-cursors CodeMirror-dragcursors"),t.display.lineSpace.insertBefore(t.display.dragCursor,t.display.cursorDiv)),_(t.display.dragCursor,r)}}(t,e),Bt(e))},start:function(e){return function(t,e){if(a&&(!t.state.draggingText||+new Date-Mo<100))Bt(e);else if(!pt(t,e)&&!Bn(t.display,e)&&(e.dataTransfer.setData("Text",t.getSelection()),e.dataTransfer.effectAllowed="copyMove",e.dataTransfer.setDragImage&&!g)){var n=D("img",null,null,"position: fixed; left: 0; top: 0;");n.src="data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==",u&&(n.width=n.height=1,t.display.wrapper.appendChild(n),n._top=n.offsetTop),e.dataTransfer.setDragImage(n,0,0),u&&n.parentNode.removeChild(n)}}(t,e)},drop:Zr(t,ko),leave:function(e){pt(t,e)||So(t)}};var A=e.input.getField();gt(A,"keyup",function(e){return la.call(t,e)}),gt(A,"keydown",Zr(t,Aa)),gt(A,"keypress",Zr(t,ca)),gt(A,"focus",function(e){return Br(t,e)}),gt(A,"blur",function(e){return br(t,e)})}(this),No(),Kr(this),this.curOp.forceUpdate=!0,Li(this,r),e.autofocus&&!I||this.hasFocus()?setTimeout(F(Br,this),20):br(this),va)va.hasOwnProperty(l)&&va[l](n,e[l],ya);ci(this),e.finishInit&&e.finishInit(this);for(var c=0;c<wa.length;++c)wa[c](n);zr(this),A&&e.lineWrapping&&"optimizelegibility"==getComputedStyle(o.lineDiv).textRendering&&(o.lineDiv.style.textRendering="auto")}Qa.defaults=Ea,Qa.optionHandlers=va;var wa=[];function xa(t,e,n,r){var i,o=t.doc;null==n&&(n="add"),"smart"==n&&(o.mode.indent?i=de(t,e).state:n="prev");var a=t.options.tabSize,s=Jt(o,e),A=U(s.text,null,a);s.stateAfter&&(s.stateAfter=null);var l,c=s.text.match(/^\s*/)[0];if(r||/\S/.test(s.text)){if("smart"==n&&((l=o.mode.indent(i,s.text.slice(c.length),s.text))==j||l>150)){if(!r)return;n="prev"}}else l=0,n="not";"prev"==n?l=e>o.first?U(Jt(o,e-1).text,null,a):0:"add"==n?l=A+t.options.indentUnit:"subtract"==n?l=A-t.options.indentUnit:"number"==typeof n&&(l=A+n),l=Math.max(0,l);var u="",g=0;if(t.options.indentWithTabs)for(var h=Math.floor(l/a);h;--h)g+=a,u+="\t";if(g<l&&(u+=q(l-g)),u!=c)return go(o,u,ee(e,0),ee(e,c.length),"+input"),s.stateAfter=null,!0;for(var d=0;d<o.sel.ranges.length;d++){var f=o.sel.ranges[d];if(f.head.line==e&&f.head.ch<c.length){var p=ee(e,c.length);qi(o,d,new Ei(p,p));break}}}Qa.defineInitHook=function(t){return wa.push(t)};var _a=null;function Da(t){_a=t}function Ma(t,e,n,r,i){var o=t.doc;t.display.shift=!1,r||(r=o.sel);var a=+new Date-200,s="paste"==i||t.state.pasteIncoming>a,A=St(e),l=null;if(s&&r.ranges.length>1)if(_a&&_a.text.join("\n")==e){if(r.ranges.length%_a.text.length==0){l=[];for(var c=0;c<_a.text.length;c++)l.push(o.splitLines(_a.text[c]))}}else A.length==r.ranges.length&&t.options.pasteLinesPerSelection&&(l=W(A,function(t){return[t]}));for(var u=t.curOp.updateInput,g=r.ranges.length-1;g>=0;g--){var h=r.ranges[g],d=h.from(),f=h.to();h.empty()&&(n&&n>0?d=ee(d.line,d.ch-n):t.state.overwrite&&!s?f=ee(f.line,Math.min(Jt(o,f.line).text.length,f.ch+V(A).length)):s&&_a&&_a.lineWise&&_a.text.join("\n")==e&&(d=f=ee(d.line,0)));var p={from:d,to:f,text:l?l[g%l.length]:A,origin:i||(s?"paste":t.state.cutIncoming>a?"cut":"+input")};so(t.doc,p),ln(t,"inputRead",t,p)}e&&!s&&Sa(t,e),Mr(t),t.curOp.updateInput<2&&(t.curOp.updateInput=u),t.curOp.typing=!0,t.state.pasteIncoming=t.state.cutIncoming=-1}function ka(t,e){var n=t.clipboardData&&t.clipboardData.getData("Text");if(n)return t.preventDefault(),e.isReadOnly()||e.options.disableInput||Xr(e,function(){return Ma(e,n,0,null,"paste")}),!0}function Sa(t,e){if(t.options.electricChars&&t.options.smartIndent)for(var n=t.doc.sel,r=n.ranges.length-1;r>=0;r--){var i=n.ranges[r];if(!(i.head.ch>100||r&&n.ranges[r-1].head.line==i.head.line)){var o=t.getModeAt(i.head),a=!1;if(o.electricChars){for(var s=0;s<o.electricChars.length;s++)if(e.indexOf(o.electricChars.charAt(s))>-1){a=xa(t,i.head.line,"smart");break}}else o.electricInput&&o.electricInput.test(Jt(t.doc,i.head.line).text.slice(0,i.head.ch))&&(a=xa(t,i.head.line,"smart"));a&&ln(t,"electricInput",t,i.head.line)}}}function La(t){for(var e=[],n=[],r=0;r<t.doc.sel.ranges.length;r++){var i=t.doc.sel.ranges[r].head.line,o={anchor:ee(i,0),head:ee(i+1,0)};n.push(o),e.push(t.getRange(o.anchor,o.head))}return{text:e,ranges:n}}function Ra(t,e,n,r){t.setAttribute("autocorrect",n?"":"off"),t.setAttribute("autocapitalize",r?"":"off"),t.setAttribute("spellcheck",!!e)}function Na(){var t=D("textarea",null,null,"position: absolute; bottom: -1em; padding: 0; width: 1px; height: 1em; outline: none"),e=D("div",[t],null,"overflow: hidden; position: relative; width: 3px; height: 0px;");return A?t.style.width="1000px":t.setAttribute("wrap","off"),f&&(t.style.border="1px solid black"),Ra(t),e}function Fa(t,e,n,r,i){var o=e,a=n,s=Jt(t,e.line);function A(r){var o,a;if(null==(o=i?function(t,e,n,r){var i=ct(e,t.doc.direction);if(!i)return Xo(e,n,r);n.ch>=e.text.length?(n.ch=e.text.length,n.sticky="before"):n.ch<=0&&(n.ch=0,n.sticky="after");var o=At(i,n.ch,n.sticky),a=i[o];if("ltr"==t.doc.direction&&a.level%2==0&&(r>0?a.to>n.ch:a.from<n.ch))return Xo(e,n,r);var s,A=function(t,n){return Wo(e,t instanceof ee?t.ch:t,n)},l=function(n){return t.options.lineWrapping?(s=s||Ln(t,e),$n(t,e,s,n)):{begin:0,end:e.text.length}},c=l("before"==n.sticky?A(n,-1):n.ch);if("rtl"==t.doc.direction||1==a.level){var u=1==a.level==r<0,g=A(n,u?1:-1);if(null!=g&&(u?g<=a.to&&g<=c.end:g>=a.from&&g>=c.begin)){var h=u?"before":"after";return new ee(n.line,g,h)}}var d=function(t,e,r){for(var o=function(t,e){return e?new ee(n.line,A(t,1),"before"):new ee(n.line,t,"after")};t>=0&&t<i.length;t+=e){var a=i[t],s=e>0==(1!=a.level),l=s?r.begin:A(r.end,-1);if(a.from<=l&&l<a.to)return o(l,s);if(l=s?a.from:A(a.to,-1),r.begin<=l&&l<r.end)return o(l,s)}},f=d(o+r,r,c);if(f)return f;var p=r>0?c.end:A(c.begin,-1);return null==p||r>0&&p==e.text.length||!(f=d(r>0?0:i.length-1,r,l(p)))?null:f}(t.cm,s,e,n):Xo(s,e,n))){if(r||((a=e.line+n)<t.first||a>=t.first+t.size||(e=new ee(a,e.ch,e.sticky),!(s=Jt(t,a)))))return!1;e=Zo(i,t.cm,s,e.line,n)}else e=o;return!0}if("char"==r)A();else if("column"==r)A(!0);else if("word"==r||"group"==r)for(var l=null,c="group"==r,u=t.cm&&t.cm.getHelper(e,"wordChars"),g=!0;!(n<0)||A(!g);g=!1){var h=s.text.charAt(e.ch)||"\n",d=et(h,u)?"w":c&&"\n"==h?"n":!c||/\s/.test(h)?null:"p";if(!c||g||d||(d="s"),l&&l!=d){n<0&&(n=1,A(),e.sticky="after");break}if(d&&(l=d),n>0&&!A(!g))break}var f=ro(t,e,o,a,!0);return re(o,f)&&(f.hitSide=!0),f}function Ta(t,e,n,r){var i,o,a=t.doc,s=e.left;if("page"==r){var A=Math.min(t.display.wrapper.clientHeight,window.innerHeight||document.documentElement.clientHeight),l=Math.max(A-.5*nr(t.display),3);i=(n>0?e.bottom:e.top)+n*l}else"line"==r&&(i=n>0?e.bottom+3:e.top-3);for(;(o=Xn(t,s,i)).outside;){if(n<0?i<=0:i>=a.height){o.hitSide=!0;break}i+=5*n}return o}var Ua=function(t){this.cm=t,this.lastAnchorNode=this.lastAnchorOffset=this.lastFocusNode=this.lastFocusOffset=null,this.polling=new O,this.composing=null,this.gracePeriod=!1,this.readDOMTimeout=null};function Oa(t,e){var n=Sn(t,e.line);if(!n||n.hidden)return null;var r=Jt(t.doc,e.line),i=Mn(n,r,e.line),o=ct(r,t.doc.direction),a="left";if(o){var s=At(o,e.ch);a=s%2?"right":"left"}var A=Tn(i.map,e.ch,a);return A.offset="right"==A.collapse?A.end:A.start,A}function Ga(t,e){return e&&(t.bad=!0),t}function Ha(t,e,n){var r;if(e==t.display.lineDiv){if(!(r=t.display.lineDiv.childNodes[n]))return Ga(t.clipPos(ee(t.display.viewTo-1)),!0);e=null,n=0}else for(r=e;;r=r.parentNode){if(!r||r==t.display.lineDiv)return null;if(r.parentNode&&r.parentNode==t.display.lineDiv)break}for(var i=0;i<t.display.view.length;i++){var o=t.display.view[i];if(o.node==r)return ja(o,e,n)}}function ja(t,e,n){var r=t.text.firstChild,i=!1;if(!e||!k(r,e))return Ga(ee(Xt(t.line),0),!0);if(e==r&&(i=!0,e=r.childNodes[n],n=0,!e)){var o=t.rest?V(t.rest):t.line;return Ga(ee(Xt(o),o.text.length),i)}var a=3==e.nodeType?e:null,s=e;for(a||1!=e.childNodes.length||3!=e.firstChild.nodeType||(a=e.firstChild,n&&(n=a.nodeValue.length));s.parentNode!=r;)s=s.parentNode;var A=t.measure,l=A.maps;function c(e,n,r){for(var i=-1;i<(l?l.length:0);i++)for(var o=i<0?A.map:l[i],a=0;a<o.length;a+=3){var s=o[a+2];if(s==e||s==n){var c=Xt(i<0?t.line:t.rest[i]),u=o[a]+r;return(r<0||s!=e)&&(u=o[a+(r?1:0)]),ee(c,u)}}}var u=c(a,s,n);if(u)return Ga(u,i);for(var g=s.nextSibling,h=a?a.nodeValue.length-n:0;g;g=g.nextSibling){if(u=c(g,g.firstChild,0))return Ga(ee(u.line,u.ch-h),i);h+=g.textContent.length}for(var d=s.previousSibling,f=n;d;d=d.previousSibling){if(u=c(d,d.firstChild,-1))return Ga(ee(u.line,u.ch+f),i);f+=d.textContent.length}}Ua.prototype.init=function(t){var e=this,n=this,r=n.cm,i=n.div=t.lineDiv;function o(t){if(!pt(r,t)){if(r.somethingSelected())Da({lineWise:!1,text:r.getSelections()}),"cut"==t.type&&r.replaceSelection("",null,"cut");else{if(!r.options.lineWiseCopyCut)return;var e=La(r);Da({lineWise:!0,text:e.text}),"cut"==t.type&&r.operation(function(){r.setSelections(e.ranges,0,Y),r.replaceSelection("",null,"cut")})}if(t.clipboardData){t.clipboardData.clearData();var o=_a.text.join("\n");if(t.clipboardData.setData("Text",o),t.clipboardData.getData("Text")==o)return void t.preventDefault()}var a=Na(),s=a.firstChild;r.display.lineSpace.insertBefore(a,r.display.lineSpace.firstChild),s.value=_a.text.join("\n");var A=document.activeElement;N(s),setTimeout(function(){r.display.lineSpace.removeChild(a),A.focus(),A==i&&n.showPrimarySelection()},50)}}Ra(i,r.options.spellcheck,r.options.autocorrect,r.options.autocapitalize),gt(i,"paste",function(t){pt(r,t)||ka(t,r)||s<=11&&setTimeout(Zr(r,function(){return e.updateFromDOM()}),20)}),gt(i,"compositionstart",function(t){e.composing={data:t.data,done:!1}}),gt(i,"compositionupdate",function(t){e.composing||(e.composing={data:t.data,done:!1})}),gt(i,"compositionend",function(t){e.composing&&(t.data!=e.composing.data&&e.readFromDOMSoon(),e.composing.done=!0)}),gt(i,"touchstart",function(){return n.forceCompositionEnd()}),gt(i,"input",function(){e.composing||e.readFromDOMSoon()}),gt(i,"copy",o),gt(i,"cut",o)},Ua.prototype.prepareSelection=function(){var t=pr(this.cm,!1);return t.focus=this.cm.state.focused,t},Ua.prototype.showSelection=function(t,e){t&&this.cm.display.view.length&&((t.focus||e)&&this.showPrimarySelection(),this.showMultipleSelections(t))},Ua.prototype.getSelection=function(){return this.cm.display.wrapper.ownerDocument.getSelection()},Ua.prototype.showPrimarySelection=function(){var t=this.getSelection(),e=this.cm,r=e.doc.sel.primary(),i=r.from(),o=r.to();if(e.display.viewTo==e.display.viewFrom||i.line>=e.display.viewTo||o.line<e.display.viewFrom)t.removeAllRanges();else{var a=Ha(e,t.anchorNode,t.anchorOffset),s=Ha(e,t.focusNode,t.focusOffset);if(!a||a.bad||!s||s.bad||0!=ne(ae(a,s),i)||0!=ne(oe(a,s),o)){var A=e.display.view,l=i.line>=e.display.viewFrom&&Oa(e,i)||{node:A[0].measure.map[2],offset:0},c=o.line<e.display.viewTo&&Oa(e,o);if(!c){var u=A[A.length-1].measure,g=u.maps?u.maps[u.maps.length-1]:u.map;c={node:g[g.length-1],offset:g[g.length-2]-g[g.length-3]}}if(l&&c){var h,d=t.rangeCount&&t.getRangeAt(0);try{h=Q(l.node,l.offset,c.offset,c.node)}catch(t){}h&&(!n&&e.state.focused?(t.collapse(l.node,l.offset),h.collapsed||(t.removeAllRanges(),t.addRange(h))):(t.removeAllRanges(),t.addRange(h)),d&&null==t.anchorNode?t.addRange(d):n&&this.startGracePeriod()),this.rememberSelection()}else t.removeAllRanges()}}},Ua.prototype.startGracePeriod=function(){var t=this;clearTimeout(this.gracePeriod),this.gracePeriod=setTimeout(function(){t.gracePeriod=!1,t.selectionChanged()&&t.cm.operation(function(){return t.cm.curOp.selectionChanged=!0})},20)},Ua.prototype.showMultipleSelections=function(t){_(this.cm.display.cursorDiv,t.cursors),_(this.cm.display.selectionDiv,t.selection)},Ua.prototype.rememberSelection=function(){var t=this.getSelection();this.lastAnchorNode=t.anchorNode,this.lastAnchorOffset=t.anchorOffset,this.lastFocusNode=t.focusNode,this.lastFocusOffset=t.focusOffset},Ua.prototype.selectionInEditor=function(){var t=this.getSelection();if(!t.rangeCount)return!1;var e=t.getRangeAt(0).commonAncestorContainer;return k(this.div,e)},Ua.prototype.focus=function(){"nocursor"!=this.cm.options.readOnly&&(this.selectionInEditor()||this.showSelection(this.prepareSelection(),!0),this.div.focus())},Ua.prototype.blur=function(){this.div.blur()},Ua.prototype.getField=function(){return this.div},Ua.prototype.supportsTouch=function(){return!0},Ua.prototype.receivedFocus=function(){var t=this;this.selectionInEditor()?this.pollSelection():Xr(this.cm,function(){return t.cm.curOp.selectionChanged=!0}),this.polling.set(this.cm.options.pollInterval,function e(){t.cm.state.focused&&(t.pollSelection(),t.polling.set(t.cm.options.pollInterval,e))})},Ua.prototype.selectionChanged=function(){var t=this.getSelection();return t.anchorNode!=this.lastAnchorNode||t.anchorOffset!=this.lastAnchorOffset||t.focusNode!=this.lastFocusNode||t.focusOffset!=this.lastFocusOffset},Ua.prototype.pollSelection=function(){if(null==this.readDOMTimeout&&!this.gracePeriod&&this.selectionChanged()){var t=this.getSelection(),e=this.cm;if(p&&c&&this.cm.display.gutterSpecs.length&&function(t){for(var e=t;e;e=e.parentNode)if(/CodeMirror-gutter-wrapper/.test(e.className))return!0;return!1}(t.anchorNode))return this.cm.triggerOnKeyDown({type:"keydown",keyCode:8,preventDefault:Math.abs}),this.blur(),void this.focus();if(!this.composing){this.rememberSelection();var n=Ha(e,t.anchorNode,t.anchorOffset),r=Ha(e,t.focusNode,t.focusOffset);n&&r&&Xr(e,function(){Xi(e.doc,Bi(n,r),Y),(n.bad||r.bad)&&(e.curOp.selectionChanged=!0)})}}},Ua.prototype.pollContent=function(){null!=this.readDOMTimeout&&(clearTimeout(this.readDOMTimeout),this.readDOMTimeout=null);var t,e,n,r=this.cm,i=r.display,o=r.doc.sel.primary(),a=o.from(),s=o.to();if(0==a.ch&&a.line>r.firstLine()&&(a=ee(a.line-1,Jt(r.doc,a.line-1).length)),s.ch==Jt(r.doc,s.line).text.length&&s.line<r.lastLine()&&(s=ee(s.line+1,0)),a.line<i.viewFrom||s.line>i.viewTo-1)return!1;a.line==i.viewFrom||0==(t=lr(r,a.line))?(e=Xt(i.view[0].line),n=i.view[0].node):(e=Xt(i.view[t].line),n=i.view[t-1].node.nextSibling);var A,l,c=lr(r,s.line);if(c==i.view.length-1?(A=i.viewTo-1,l=i.lineDiv.lastChild):(A=Xt(i.view[c+1].line)-1,l=i.view[c+1].node.previousSibling),!n)return!1;for(var u=r.doc.splitLines(function(t,e,n,r,i){var o="",a=!1,s=t.doc.lineSeparator(),A=!1;function l(){a&&(o+=s,A&&(o+=s),a=A=!1)}function c(t){t&&(l(),o+=t)}function u(e){if(1==e.nodeType){var n=e.getAttribute("cm-text");if(n)return void c(n);var o,g=e.getAttribute("cm-marker");if(g){var h=t.findMarks(ee(r,0),ee(i+1,0),(p=+g,function(t){return t.id==p}));return void(h.length&&(o=h[0].find(0))&&c(qt(t.doc,o.from,o.to).join(s)))}if("false"==e.getAttribute("contenteditable"))return;var d=/^(pre|div|p|li|table|br)$/i.test(e.nodeName);if(!/^br$/i.test(e.nodeName)&&0==e.textContent.length)return;d&&l();for(var f=0;f<e.childNodes.length;f++)u(e.childNodes[f]);/^(pre|p)$/i.test(e.nodeName)&&(A=!0),d&&(a=!0)}else 3==e.nodeType&&c(e.nodeValue.replace(/\u200b/g,"").replace(/\u00a0/g," "));var p}for(;u(e),e!=n;)e=e.nextSibling,A=!1;return o}(r,n,l,e,A)),g=qt(r.doc,ee(e,0),ee(A,Jt(r.doc,A).text.length));u.length>1&&g.length>1;)if(V(u)==V(g))u.pop(),g.pop(),A--;else{if(u[0]!=g[0])break;u.shift(),g.shift(),e++}for(var h=0,d=0,f=u[0],p=g[0],I=Math.min(f.length,p.length);h<I&&f.charCodeAt(h)==p.charCodeAt(h);)++h;for(var C=V(u),m=V(g),y=Math.min(C.length-(1==u.length?h:0),m.length-(1==g.length?h:0));d<y&&C.charCodeAt(C.length-d-1)==m.charCodeAt(m.length-d-1);)++d;if(1==u.length&&1==g.length&&e==a.line)for(;h&&h>a.ch&&C.charCodeAt(C.length-d-1)==m.charCodeAt(m.length-d-1);)h--,d++;u[u.length-1]=C.slice(0,C.length-d).replace(/^\u200b+/,""),u[0]=u[0].slice(h).replace(/\u200b+$/,"");var E=ee(e,h),v=ee(A,g.length?V(g).length-d:0);return u.length>1||u[0]||ne(E,v)?(go(r.doc,u,E,v,"+input"),!0):void 0},Ua.prototype.ensurePolled=function(){this.forceCompositionEnd()},Ua.prototype.reset=function(){this.forceCompositionEnd()},Ua.prototype.forceCompositionEnd=function(){this.composing&&(clearTimeout(this.readDOMTimeout),this.composing=null,this.updateFromDOM(),this.div.blur(),this.div.focus())},Ua.prototype.readFromDOMSoon=function(){var t=this;null==this.readDOMTimeout&&(this.readDOMTimeout=setTimeout(function(){if(t.readDOMTimeout=null,t.composing){if(!t.composing.done)return;t.composing=null}t.updateFromDOM()},80))},Ua.prototype.updateFromDOM=function(){var t=this;!this.cm.isReadOnly()&&this.pollContent()||Xr(this.cm,function(){return cr(t.cm)})},Ua.prototype.setUneditable=function(t){t.contentEditable="false"},Ua.prototype.onKeyPress=function(t){0==t.charCode||this.composing||(t.preventDefault(),this.cm.isReadOnly()||Zr(this.cm,Ma)(this.cm,String.fromCharCode(null==t.charCode?t.keyCode:t.charCode),0))},Ua.prototype.readOnlyChanged=function(t){this.div.contentEditable=String("nocursor"!=t)},Ua.prototype.onContextMenu=function(){},Ua.prototype.resetPosition=function(){},Ua.prototype.needsContentAttribute=!0;var Ya=function(t){this.cm=t,this.prevInput="",this.pollingFast=!1,this.polling=new O,this.hasSelection=!1,this.composing=null};Ya.prototype.init=function(t){var e=this,n=this,r=this.cm;this.createField(t);var i=this.textarea;function o(t){if(!pt(r,t)){if(r.somethingSelected())Da({lineWise:!1,text:r.getSelections()});else{if(!r.options.lineWiseCopyCut)return;var e=La(r);Da({lineWise:!0,text:e.text}),"cut"==t.type?r.setSelections(e.ranges,null,Y):(n.prevInput="",i.value=e.text.join("\n"),N(i))}"cut"==t.type&&(r.state.cutIncoming=+new Date)}}t.wrapper.insertBefore(this.wrapper,t.wrapper.firstChild),f&&(i.style.width="0px"),gt(i,"input",function(){a&&s>=9&&e.hasSelection&&(e.hasSelection=null),n.poll()}),gt(i,"paste",function(t){pt(r,t)||ka(t,r)||(r.state.pasteIncoming=+new Date,n.fastPoll())}),gt(i,"cut",o),gt(i,"copy",o),gt(t.scroller,"paste",function(e){if(!Bn(t,e)&&!pt(r,e)){if(!i.dispatchEvent)return r.state.pasteIncoming=+new Date,void n.focus();var o=new Event("paste");o.clipboardData=e.clipboardData,i.dispatchEvent(o)}}),gt(t.lineSpace,"selectstart",function(e){Bn(t,e)||yt(e)}),gt(i,"compositionstart",function(){var t=r.getCursor("from");n.composing&&n.composing.range.clear(),n.composing={start:t,range:r.markText(t,r.getCursor("to"),{className:"CodeMirror-composing"})}}),gt(i,"compositionend",function(){n.composing&&(n.poll(),n.composing.range.clear(),n.composing=null)})},Ya.prototype.createField=function(t){this.wrapper=Na(),this.textarea=this.wrapper.firstChild},Ya.prototype.prepareSelection=function(){var t=this.cm,e=t.display,n=t.doc,r=pr(t);if(t.options.moveInputWithCursor){var i=qn(t,n.sel.primary().head,"div"),o=e.wrapper.getBoundingClientRect(),a=e.lineDiv.getBoundingClientRect();r.teTop=Math.max(0,Math.min(e.wrapper.clientHeight-10,i.top+a.top-o.top)),r.teLeft=Math.max(0,Math.min(e.wrapper.clientWidth-10,i.left+a.left-o.left))}return r},Ya.prototype.showSelection=function(t){var e=this.cm,n=e.display;_(n.cursorDiv,t.cursors),_(n.selectionDiv,t.selection),null!=t.teTop&&(this.wrapper.style.top=t.teTop+"px",this.wrapper.style.left=t.teLeft+"px")},Ya.prototype.reset=function(t){if(!this.contextMenuPending&&!this.composing){var e=this.cm;if(e.somethingSelected()){this.prevInput="";var n=e.getSelection();this.textarea.value=n,e.state.focused&&N(this.textarea),a&&s>=9&&(this.hasSelection=n)}else t||(this.prevInput=this.textarea.value="",a&&s>=9&&(this.hasSelection=null))}},Ya.prototype.getField=function(){return this.textarea},Ya.prototype.supportsTouch=function(){return!1},Ya.prototype.focus=function(){if("nocursor"!=this.cm.options.readOnly&&(!I||S()!=this.textarea))try{this.textarea.focus()}catch(t){}},Ya.prototype.blur=function(){this.textarea.blur()},Ya.prototype.resetPosition=function(){this.wrapper.style.top=this.wrapper.style.left=0},Ya.prototype.receivedFocus=function(){this.slowPoll()},Ya.prototype.slowPoll=function(){var t=this;this.pollingFast||this.polling.set(this.cm.options.pollInterval,function(){t.poll(),t.cm.state.focused&&t.slowPoll()})},Ya.prototype.fastPoll=function(){var t=!1,e=this;e.pollingFast=!0,e.polling.set(20,function n(){var r=e.poll();r||t?(e.pollingFast=!1,e.slowPoll()):(t=!0,e.polling.set(60,n))})},Ya.prototype.poll=function(){var t=this,e=this.cm,n=this.textarea,r=this.prevInput;if(this.contextMenuPending||!e.state.focused||Lt(n)&&!r&&!this.composing||e.isReadOnly()||e.options.disableInput||e.state.keySeq)return!1;var i=n.value;if(i==r&&!e.somethingSelected())return!1;if(a&&s>=9&&this.hasSelection===i||C&&/[\uf700-\uf7ff]/.test(i))return e.display.input.reset(),!1;if(e.doc.sel==e.display.selForContextMenu){var o=i.charCodeAt(0);if(8203!=o||r||(r="​"),8666==o)return this.reset(),this.cm.execCommand("undo")}for(var A=0,l=Math.min(r.length,i.length);A<l&&r.charCodeAt(A)==i.charCodeAt(A);)++A;return Xr(e,function(){Ma(e,i.slice(A),r.length-A,null,t.composing?"*compose":null),i.length>1e3||i.indexOf("\n")>-1?n.value=t.prevInput="":t.prevInput=i,t.composing&&(t.composing.range.clear(),t.composing.range=e.markText(t.composing.start,e.getCursor("to"),{className:"CodeMirror-composing"}))}),!0},Ya.prototype.ensurePolled=function(){this.pollingFast&&this.poll()&&(this.pollingFast=!1)},Ya.prototype.onKeyPress=function(){a&&s>=9&&(this.hasSelection=null),this.fastPoll()},Ya.prototype.onContextMenu=function(t){var e=this,n=e.cm,r=n.display,i=e.textarea;e.contextMenuPending&&e.contextMenuPending();var o=Ar(n,t),l=r.scroller.scrollTop;if(o&&!u){var c=n.options.resetSelectionOnContextMenu;c&&-1==n.doc.sel.contains(o)&&Zr(n,Xi)(n.doc,Bi(o),Y);var g,h=i.style.cssText,d=e.wrapper.style.cssText,f=e.wrapper.offsetParent.getBoundingClientRect();if(e.wrapper.style.cssText="position: static",i.style.cssText="position: absolute; width: 30px; height: 30px;\n      top: "+(t.clientY-f.top-5)+"px; left: "+(t.clientX-f.left-5)+"px;\n      z-index: 1000; background: "+(a?"rgba(255, 255, 255, .05)":"transparent")+";\n      outline: none; border-width: 0; outline: none; overflow: hidden; opacity: .05; filter: alpha(opacity=5);",A&&(g=window.scrollY),r.input.focus(),A&&window.scrollTo(null,g),r.input.reset(),n.somethingSelected()||(i.value=e.prevInput=" "),e.contextMenuPending=C,r.selForContextMenu=n.doc.sel,clearTimeout(r.detectingSelectAll),a&&s>=9&&I(),B){Bt(t);var p=function(){dt(window,"mouseup",p),setTimeout(C,20)};gt(window,"mouseup",p)}else setTimeout(C,50)}function I(){if(null!=i.selectionStart){var t=n.somethingSelected(),o="​"+(t?i.value:"");i.value="⇚",i.value=o,e.prevInput=t?"":"​",i.selectionStart=1,i.selectionEnd=o.length,r.selForContextMenu=n.doc.sel}}function C(){if(e.contextMenuPending==C&&(e.contextMenuPending=!1,e.wrapper.style.cssText=d,i.style.cssText=h,a&&s<9&&r.scrollbars.setScrollTop(r.scroller.scrollTop=l),null!=i.selectionStart)){(!a||a&&s<9)&&I();var t=0,o=function(){r.selForContextMenu==n.doc.sel&&0==i.selectionStart&&i.selectionEnd>0&&"​"==e.prevInput?Zr(n,oo)(n):t++<10?r.detectingSelectAll=setTimeout(o,500):(r.selForContextMenu=null,r.input.reset())};r.detectingSelectAll=setTimeout(o,200)}}},Ya.prototype.readOnlyChanged=function(t){t||this.reset(),this.textarea.disabled="nocursor"==t},Ya.prototype.setUneditable=function(){},Ya.prototype.needsContentAttribute=!1,function(t){var e=t.optionHandlers;function n(n,r,i,o){t.defaults[n]=r,i&&(e[n]=o?function(t,e,n){n!=ya&&i(t,e,n)}:i)}t.defineOption=n,t.Init=ya,n("value","",function(t,e){return t.setValue(e)},!0),n("mode",null,function(t,e){t.doc.modeOption=e,_i(t)},!0),n("indentUnit",2,_i,!0),n("indentWithTabs",!1),n("smartIndent",!0),n("tabSize",4,function(t){Di(t),Hn(t),cr(t)},!0),n("lineSeparator",null,function(t,e){if(t.doc.lineSep=e,e){var n=[],r=t.doc.first;t.doc.iter(function(t){for(var i=0;;){var o=t.text.indexOf(e,i);if(-1==o)break;i=o+e.length,n.push(ee(r,o))}r++});for(var i=n.length-1;i>=0;i--)go(t.doc,e,n[i],ee(n[i].line,n[i].ch+e.length))}}),n("specialChars",/[\u0000-\u001f\u007f-\u009f\u00ad\u061c\u200b-\u200f\u2028\u2029\ufeff]/g,function(t,e,n){t.state.specialChars=new RegExp(e.source+(e.test("\t")?"":"|\t"),"g"),n!=ya&&t.refresh()}),n("specialCharPlaceholder",$e,function(t){return t.refresh()},!0),n("electricChars",!0),n("inputStyle",I?"contenteditable":"textarea",function(){throw new Error("inputStyle can not (yet) be changed in a running editor")},!0),n("spellcheck",!1,function(t,e){return t.getInputField().spellcheck=e},!0),n("autocorrect",!1,function(t,e){return t.getInputField().autocorrect=e},!0),n("autocapitalize",!1,function(t,e){return t.getInputField().autocapitalize=e},!0),n("rtlMoveVisually",!y),n("wholeLineUpdateBefore",!0),n("theme","default",function(t){ma(t),hi(t)},!0),n("keyMap","default",function(t,e,n){var r=qo(e),i=n!=ya&&qo(n);i&&i.detach&&i.detach(t,r),r.attach&&r.attach(t,i||null)}),n("extraKeys",null),n("configureMouse",null),n("lineWrapping",!1,ba,!0),n("gutters",[],function(t,e){t.display.gutterSpecs=ui(e,t.options.lineNumbers),hi(t)},!0),n("fixedGutter",!0,function(t,e){t.display.gutters.style.left=e?or(t.display)+"px":"0",t.refresh()},!0),n("coverGutterNextToScrollbar",!1,function(t){return Gr(t)},!0),n("scrollbarStyle","native",function(t){Yr(t),Gr(t),t.display.scrollbars.setScrollTop(t.doc.scrollTop),t.display.scrollbars.setScrollLeft(t.doc.scrollLeft)},!0),n("lineNumbers",!1,function(t,e){t.display.gutterSpecs=ui(t.options.gutters,e),hi(t)},!0),n("firstLineNumber",1,hi,!0),n("lineNumberFormatter",function(t){return t},hi,!0),n("showCursorWhenSelecting",!1,fr,!0),n("resetSelectionOnContextMenu",!0),n("lineWiseCopyCut",!0),n("pasteLinesPerSelection",!0),n("selectionsMayTouch",!1),n("readOnly",!1,function(t,e){"nocursor"==e&&(br(t),t.display.input.blur()),t.display.input.readOnlyChanged(e)}),n("disableInput",!1,function(t,e){e||t.display.input.reset()},!0),n("dragDrop",!0,Ba),n("allowDropFileTypes",null),n("cursorBlinkRate",530),n("cursorScrollMargin",0),n("cursorHeight",1,fr,!0),n("singleCursorHeightPerLine",!0,fr,!0),n("workTime",100),n("workDelay",100),n("flattenSpans",!0,Di,!0),n("addModeClass",!1,Di,!0),n("pollInterval",100),n("undoDepth",200,function(t,e){return t.doc.history.undoDepth=e}),n("historyEventDelay",1250),n("viewportMargin",10,function(t){return t.refresh()},!0),n("maxHighlightLength",1e4,Di,!0),n("moveInputWithCursor",!0,function(t,e){e||t.display.input.resetPosition()}),n("tabindex",null,function(t,e){return t.display.input.getField().tabIndex=e||""}),n("autofocus",null),n("direction","ltr",function(t,e){return t.doc.setDirection(e)},!0),n("phrases",null)}(Qa),function(t){var e=t.optionHandlers,n=t.helpers={};t.prototype={constructor:t,focus:function(){window.focus(),this.display.input.focus()},setOption:function(t,n){var r=this.options,i=r[t];r[t]==n&&"mode"!=t||(r[t]=n,e.hasOwnProperty(t)&&Zr(this,e[t])(this,n,i),ft(this,"optionChange",this,t))},getOption:function(t){return this.options[t]},getDoc:function(){return this.doc},addKeyMap:function(t,e){this.state.keyMaps[e?"push":"unshift"](qo(t))},removeKeyMap:function(t){for(var e=this.state.keyMaps,n=0;n<e.length;++n)if(e[n]==t||e[n].name==t)return e.splice(n,1),!0},addOverlay:$r(function(e,n){var r=e.token?e:t.getMode(this.options,e);if(r.startState)throw new Error("Overlays may not be stateful.");!function(t,e,n){for(var r=0,i=n(e);r<t.length&&n(t[r])<=i;)r++;t.splice(r,0,e)}(this.state.overlays,{mode:r,modeSpec:e,opaque:n&&n.opaque,priority:n&&n.priority||0},function(t){return t.priority}),this.state.modeGen++,cr(this)}),removeOverlay:$r(function(t){for(var e=this.state.overlays,n=0;n<e.length;++n){var r=e[n].modeSpec;if(r==t||"string"==typeof t&&r.name==t)return e.splice(n,1),this.state.modeGen++,void cr(this)}}),indentLine:$r(function(t,e,n){"string"!=typeof e&&"number"!=typeof e&&(e=null==e?this.options.smartIndent?"smart":"prev":e?"add":"subtract"),$t(this.doc,t)&&xa(this,t,e,n)}),indentSelection:$r(function(t){for(var e=this.doc.sel.ranges,n=-1,r=0;r<e.length;r++){var i=e[r];if(i.empty())i.head.line>n&&(xa(this,i.head.line,t,!0),n=i.head.line,r==this.doc.sel.primIndex&&Mr(this));else{var o=i.from(),a=i.to(),s=Math.max(n,o.line);n=Math.min(this.lastLine(),a.line-(a.ch?0:1))+1;for(var A=s;A<n;++A)xa(this,A,t);var l=this.doc.sel.ranges;0==o.ch&&e.length==l.length&&l[r].from().ch>0&&qi(this.doc,r,new Ei(o,l[r].to()),Y)}}}),getTokenAt:function(t,e){return me(this,t,e)},getLineTokens:function(t,e){return me(this,ee(t),e,!0)},getTokenTypeAt:function(t){t=Ae(this.doc,t);var e,n=he(this,Jt(this.doc,t.line)),r=0,i=(n.length-1)/2,o=t.ch;if(0==o)e=n[2];else for(;;){var a=r+i>>1;if((a?n[2*a-1]:0)>=o)i=a;else{if(!(n[2*a+1]<o)){e=n[2*a+2];break}r=a+1}}var s=e?e.indexOf("overlay "):-1;return s<0?e:0==s?null:e.slice(0,s-1)},getModeAt:function(e){var n=this.doc.mode;return n.innerMode?t.innerMode(n,this.getTokenAt(e).state).mode:n},getHelper:function(t,e){return this.getHelpers(t,e)[0]},getHelpers:function(t,e){var r=[];if(!n.hasOwnProperty(e))return r;var i=n[e],o=this.getModeAt(t);if("string"==typeof o[e])i[o[e]]&&r.push(i[o[e]]);else if(o[e])for(var a=0;a<o[e].length;a++){var s=i[o[e][a]];s&&r.push(s)}else o.helperType&&i[o.helperType]?r.push(i[o.helperType]):i[o.name]&&r.push(i[o.name]);for(var A=0;A<i._global.length;A++){var l=i._global[A];l.pred(o,this)&&-1==G(r,l.val)&&r.push(l.val)}return r},getStateAfter:function(t,e){var n=this.doc;return de(this,(t=se(n,null==t?n.first+n.size-1:t))+1,e).state},cursorCoords:function(t,e){var n=this.doc.sel.primary();return qn(this,null==t?n.head:"object"==typeof t?Ae(this.doc,t):t?n.from():n.to(),e||"page")},charCoords:function(t,e){return Jn(this,Ae(this.doc,t),e||"page")},coordsChar:function(t,e){return Xn(this,(t=zn(this,t,e||"page")).left,t.top)},lineAtHeight:function(t,e){return t=zn(this,{top:t,left:0},e||"page").top,Zt(this.doc,t+this.display.viewOffset)},heightAtLine:function(t,e,n){var r,i=!1;if("number"==typeof t){var o=this.doc.first+this.doc.size-1;t<this.doc.first?t=this.doc.first:t>o&&(t=o,i=!0),r=Jt(this.doc,t)}else r=t;return Kn(this,r,{top:0,left:0},e||"page",n||i).top+(i?this.doc.height-Pe(r):0)},defaultTextHeight:function(){return nr(this.display)},defaultCharWidth:function(){return rr(this.display)},getViewport:function(){return{from:this.display.viewFrom,to:this.display.viewTo}},addWidget:function(t,e,n,r,i){var o,a,s,A=this.display,l=(t=qn(this,Ae(this.doc,t))).bottom,c=t.left;if(e.style.position="absolute",e.setAttribute("cm-ignore-events","true"),this.display.input.setUneditable(e),A.sizer.appendChild(e),"over"==r)l=t.top;else if("above"==r||"near"==r){var u=Math.max(A.wrapper.clientHeight,this.doc.height),g=Math.max(A.sizer.clientWidth,A.lineSpace.clientWidth);("above"==r||t.bottom+e.offsetHeight>u)&&t.top>e.offsetHeight?l=t.top-e.offsetHeight:t.bottom+e.offsetHeight<=u&&(l=t.bottom),c+e.offsetWidth>g&&(c=g-e.offsetWidth)}e.style.top=l+"px",e.style.left=e.style.right="","right"==i?(c=A.sizer.clientWidth-e.offsetWidth,e.style.right="0px"):("left"==i?c=0:"middle"==i&&(c=(A.sizer.clientWidth-e.offsetWidth)/2),e.style.left=c+"px"),n&&(o=this,a={left:c,top:l,right:c+e.offsetWidth,bottom:l+e.offsetHeight},null!=(s=_r(o,a)).scrollTop&&Rr(o,s.scrollTop),null!=s.scrollLeft&&Fr(o,s.scrollLeft))},triggerOnKeyDown:$r(Aa),triggerOnKeyPress:$r(ca),triggerOnKeyUp:la,triggerOnMouseDown:$r(da),execCommand:function(t){if($o.hasOwnProperty(t))return $o[t].call(null,this)},triggerElectric:$r(function(t){Sa(this,t)}),findPosH:function(t,e,n,r){var i=1;e<0&&(i=-1,e=-e);for(var o=Ae(this.doc,t),a=0;a<e&&!(o=Fa(this.doc,o,i,n,r)).hitSide;++a);return o},moveH:$r(function(t,e){var n=this;this.extendSelectionsBy(function(r){return n.display.shift||n.doc.extend||r.empty()?Fa(n.doc,r.head,t,e,n.options.rtlMoveVisually):t<0?r.from():r.to()},K)}),deleteH:$r(function(t,e){var n=this.doc.sel,r=this.doc;n.somethingSelected()?r.replaceSelection("",null,"+delete"):Vo(this,function(n){var i=Fa(r,n.head,t,e,!1);return t<0?{from:i,to:n.head}:{from:n.head,to:i}})}),findPosV:function(t,e,n,r){var i=1,o=r;e<0&&(i=-1,e=-e);for(var a=Ae(this.doc,t),s=0;s<e;++s){var A=qn(this,a,"div");if(null==o?o=A.left:A.left=o,(a=Ta(this,A,i,n)).hitSide)break}return a},moveV:$r(function(t,e){var n=this,r=this.doc,i=[],o=!this.display.shift&&!r.extend&&r.sel.somethingSelected();if(r.extendSelectionsBy(function(a){if(o)return t<0?a.from():a.to();var s=qn(n,a.head,"div");null!=a.goalColumn&&(s.left=a.goalColumn),i.push(s.left);var A=Ta(n,s,t,e);return"page"==e&&a==r.sel.primary()&&Dr(n,Jn(n,A,"div").top-s.top),A},K),i.length)for(var a=0;a<r.sel.ranges.length;a++)r.sel.ranges[a].goalColumn=i[a]}),findWordAt:function(t){var e=this.doc,n=Jt(e,t.line).text,r=t.ch,i=t.ch;if(n){var o=this.getHelper(t,"wordChars");"before"!=t.sticky&&i!=n.length||!r?++i:--r;for(var a=n.charAt(r),s=et(a,o)?function(t){return et(t,o)}:/\s/.test(a)?function(t){return/\s/.test(t)}:function(t){return!/\s/.test(t)&&!et(t)};r>0&&s(n.charAt(r-1));)--r;for(;i<n.length&&s(n.charAt(i));)++i}return new Ei(ee(t.line,r),ee(t.line,i))},toggleOverwrite:function(t){null!=t&&t==this.state.overwrite||((this.state.overwrite=!this.state.overwrite)?L(this.display.cursorDiv,"CodeMirror-overwrite"):w(this.display.cursorDiv,"CodeMirror-overwrite"),ft(this,"overwriteToggle",this,this.state.overwrite))},hasFocus:function(){return this.display.input.getField()==S()},isReadOnly:function(){return!(!this.options.readOnly&&!this.doc.cantEdit)},scrollTo:$r(function(t,e){kr(this,t,e)}),getScrollInfo:function(){var t=this.display.scroller;return{left:t.scrollLeft,top:t.scrollTop,height:t.scrollHeight-xn(this)-this.display.barHeight,width:t.scrollWidth-xn(this)-this.display.barWidth,clientHeight:Dn(this),clientWidth:_n(this)}},scrollIntoView:$r(function(t,e){null==t?(t={from:this.doc.sel.primary().head,to:null},null==e&&(e=this.options.cursorScrollMargin)):"number"==typeof t?t={from:ee(t,0),to:null}:null==t.from&&(t={from:t,to:null}),t.to||(t.to=t.from),t.margin=e||0,null!=t.from.line?function(t,e){Sr(t),t.curOp.scrollToPos=e}(this,t):Lr(this,t.from,t.to,t.margin)}),setSize:$r(function(t,e){var n=this,r=function(t){return"number"==typeof t||/^\d+$/.test(String(t))?t+"px":t};null!=t&&(this.display.wrapper.style.width=r(t)),null!=e&&(this.display.wrapper.style.height=r(e)),this.options.lineWrapping&&Gn(this);var i=this.display.viewFrom;this.doc.iter(i,this.display.viewTo,function(t){if(t.widgets)for(var e=0;e<t.widgets.length;e++)if(t.widgets[e].noHScroll){ur(n,i,"widget");break}++i}),this.curOp.forceUpdate=!0,ft(this,"refresh",this)}),operation:function(t){return Xr(this,t)},startOperation:function(){return Kr(this)},endOperation:function(){return zr(this)},refresh:$r(function(){var t=this.display.cachedTextHeight;cr(this),this.curOp.forceUpdate=!0,Hn(this),kr(this,this.doc.scrollLeft,this.doc.scrollTop),si(this.display),(null==t||Math.abs(t-nr(this.display))>.5)&&sr(this),ft(this,"refresh",this)}),swapDoc:$r(function(t){var e=this.doc;return e.cm=null,this.state.selectingText&&this.state.selectingText(),Li(this,t),Hn(this),this.display.input.reset(),kr(this,t.scrollLeft,t.scrollTop),this.curOp.forceScroll=!0,ln(this,"swapDoc",this,e),e}),phrase:function(t){var e=this.options.phrases;return e&&Object.prototype.hasOwnProperty.call(e,t)?e[t]:t},getInputField:function(){return this.display.input.getField()},getWrapperElement:function(){return this.display.wrapper},getScrollerElement:function(){return this.display.scroller},getGutterElement:function(){return this.display.gutters}},mt(t),t.registerHelper=function(e,r,i){n.hasOwnProperty(e)||(n[e]=t[e]={_global:[]}),n[e][r]=i},t.registerGlobalHelper=function(e,r,i,o){t.registerHelper(e,r,o),n[e]._global.push({pred:i,val:o})}}(Qa);var Pa="iter insert remove copy getEditor constructor".split(" ");for(var Ka in Do.prototype)Do.prototype.hasOwnProperty(Ka)&&G(Pa,Ka)<0&&(Qa.prototype[Ka]=function(t){return function(){return t.apply(this.doc,arguments)}}(Do.prototype[Ka]));return mt(Do),Qa.inputStyles={textarea:Ya,contenteditable:Ua},Qa.defineMode=function(t){Qa.defaults.mode||"null"==t||(Qa.defaults.mode=t),Ut.apply(this,arguments)},Qa.defineMIME=function(t,e){Tt[t]=e},Qa.defineMode("null",function(){return{token:function(t){return t.skipToEnd()}}}),Qa.defineMIME("text/plain","null"),Qa.defineExtension=function(t,e){Qa.prototype[t]=e},Qa.defineDocExtension=function(t,e){Do.prototype[t]=e},Qa.fromTextArea=function(t,e){if((e=e?T(e):{}).value=t.value,!e.tabindex&&t.tabIndex&&(e.tabindex=t.tabIndex),!e.placeholder&&t.placeholder&&(e.placeholder=t.placeholder),null==e.autofocus){var n=S();e.autofocus=n==t||null!=t.getAttribute("autofocus")&&n==document.body}function r(){t.value=s.getValue()}var i;if(t.form&&(gt(t.form,"submit",r),!e.leaveSubmitMethodAlone)){var o=t.form;i=o.submit;try{var a=o.submit=function(){r(),o.submit=i,o.submit(),o.submit=a}}catch(t){}}e.finishInit=function(e){e.save=r,e.getTextArea=function(){return t},e.toTextArea=function(){e.toTextArea=isNaN,r(),t.parentNode.removeChild(e.getWrapperElement()),t.style.display="",t.form&&(dt(t.form,"submit",r),"function"==typeof t.form.submit&&(t.form.submit=i))}},t.style.display="none";var s=Qa(function(e){return t.parentNode.insertBefore(e,t.nextSibling)},e);return s},function(t){t.off=dt,t.on=gt,t.wheelEventPixels=Ci,t.Doc=Do,t.splitLines=St,t.countColumn=U,t.findColumn=z,t.isWordChar=tt,t.Pass=j,t.signal=ft,t.Line=Je,t.changeEnd=bi,t.scrollbarModel=jr,t.Pos=ee,t.cmpPos=ne,t.modes=Ft,t.mimeModes=Tt,t.resolveMode=Ot,t.getMode=Gt,t.modeExtensions=Ht,t.extendMode=jt,t.copyState=Yt,t.startState=Kt,t.innerMode=Pt,t.commands=$o,t.keyMap=Ho,t.keyName=Jo,t.isModifierKey=Ko,t.lookupKey=Po,t.normalizeKeyMap=Yo,t.StringStream=zt,t.SharedTextMarker=Qo,t.TextMarker=Bo,t.LineWidget=yo,t.e_preventDefault=yt,t.e_stopPropagation=Et,t.e_stop=Bt,t.addClass=L,t.contains=k,t.rmClass=w,t.keyNames=To}(Qa),Qa.version="5.47.0",Qa}()},function(t,e,n){"use strict";!function t(){if("undefined"!=typeof __REACT_DEVTOOLS_GLOBAL_HOOK__&&"function"==typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE)try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(t)}catch(t){console.error(t)}}(),t.exports=n(1063)},function(t,e,n){"use strict";function r(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}Object.defineProperty(e,"__esModule",{value:!0}),r(n(744)),r(n(762)),r(n(763)),r(n(764)),r(n(852)),r(n(857)),r(n(431)),r(n(862)),r(n(863)),r(n(864)),r(n(901)),r(n(902)),r(n(903)),r(n(905)),r(n(430))},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}},function(t,e,n){"use strict";e.__esModule=!0;var r=a(n(1214)),i=a(n(1218)),o=a(n(254));function a(t){return t&&t.__esModule?t:{default:t}}e.default=function(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function, not "+(void 0===e?"undefined":(0,o.default)(e)));t.prototype=(0,i.default)(e&&e.prototype,{constructor:{value:t,enumerable:!1,writable:!0,configurable:!0}}),e&&(r.default?(0,r.default)(t,e):t.__proto__=e)}},function(t,e,n){"use strict";n.r(e);var r,i=n(191),o=n(9),a=n(727),s=n.n(a),A=n(728),l=n.n(A),c=(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)}),u=function(){return(u=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)};function g(t,e){return t.filter(function(t){return-1===e.indexOf(t)})}function h(t,e){return s()(t,e)}var d=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t};function f(){return o.UUID.uuid4()}var p=function(t){function e(n,r){var i=t.call(this,n)||this;return console.warn("WrappedError has been deprecated!"),i.error_stack=r instanceof e?r.error_stack:[r],i.error_stack.push(i),i}return c(e,t),e}(Error);function I(t){var e=Object.keys(t),n=[];return e.forEach(function(e){n.push(t[e])}),Promise.all(n).then(function(t){for(var n={},r=0;r<e.length;r++)n[e[r]]=t[r];return n})}function C(t,e){return function(n){throw e&&console.error(new Error(t)),n}}function m(t,e,n){for(var r=0;r<e.length;r++){for(var i=e[r],o=t,a=0;a<i.length-1;a++)o=o[i[a]];o[i[i.length-1]]=n[r]}}function y(t){var e=[],n=[];return{state:function t(r,i){if(r.toJSON&&(r=r.toJSON()),Array.isArray(r))for(var o=!1,a=0;a<r.length;a++)(A=r[a])&&(A instanceof ArrayBuffer||ArrayBuffer.isView(A)?(o||(r=r.slice(),o=!0),e.push(ArrayBuffer.isView(A)?A.buffer:A),n.push(i.concat([a])),r[a]=null):(c=t(A,i.concat([a])))!==A&&(o||(r=r.slice(),o=!0),r[a]=c));else if(l()(r))for(var s in r){var A,c;o=!1,r.hasOwnProperty(s)&&(A=r[s])&&(A instanceof ArrayBuffer||ArrayBuffer.isView(A)?(o||(r=u({},r),o=!0),e.push(ArrayBuffer.isView(A)?A.buffer:A),n.push(i.concat([s])),delete r[s]):(c=t(A,i.concat([s])))!==A&&(o||(r=u({},r),o=!0),r[s]=c))}return r}(t,[]),buffers:e,buffer_paths:n}}var E=["00","01","02","03","04","05","06","07","08","09","0A","0B","0C","0D","0E","0F","10","11","12","13","14","15","16","17","18","19","1A","1B","1C","1D","1E","1F","20","21","22","23","24","25","26","27","28","29","2A","2B","2C","2D","2E","2F","30","31","32","33","34","35","36","37","38","39","3A","3B","3C","3D","3E","3F","40","41","42","43","44","45","46","47","48","49","4A","4B","4C","4D","4E","4F","50","51","52","53","54","55","56","57","58","59","5A","5B","5C","5D","5E","5F","60","61","62","63","64","65","66","67","68","69","6A","6B","6C","6D","6E","6F","70","71","72","73","74","75","76","77","78","79","7A","7B","7C","7D","7E","7F","80","81","82","83","84","85","86","87","88","89","8A","8B","8C","8D","8E","8F","90","91","92","93","94","95","96","97","98","99","9A","9B","9C","9D","9E","9F","A0","A1","A2","A3","A4","A5","A6","A7","A8","A9","AA","AB","AC","AD","AE","AF","B0","B1","B2","B3","B4","B5","B6","B7","B8","B9","BA","BB","BC","BD","BE","BF","C0","C1","C2","C3","C4","C5","C6","C7","C8","C9","CA","CB","CC","CD","CE","CF","D0","D1","D2","D3","D4","D5","D6","D7","D8","D9","DA","DB","DC","DD","DE","DF","E0","E1","E2","E3","E4","E5","E6","E7","E8","E9","EA","EB","EC","ED","EE","EF","F0","F1","F2","F3","F4","F5","F6","F7","F8","F9","FA","FB","FC","FD","FE","FF"];function v(t){for(var e=new Uint8Array(t),n=[],r=0;r<e.length;r++)n.push(E[e[r]]);return n.join("")}function B(t){for(var e=new Uint8Array(t.length/2),n=0;n<t.length;n+=2)e[n/2]=parseInt(t.slice(n,n+2),16);return e.buffer}function b(t){return Object(i.fromByteArray)(new Uint8Array(t))}function Q(t){return Object(i.toByteArray)(t).buffer}var w=function(){return(w=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},x=function(t,e,n,r){return new(n||(n=Promise))(function(i,o){function a(t){try{A(r.next(t))}catch(t){o(t)}}function s(t){try{A(r.throw(t))}catch(t){o(t)}}function A(t){t.done?i(t.value):new n(function(e){e(t.value)}).then(a,s)}A((r=r.apply(t,e||[])).next())})},_=function(t,e){var n,r,i,o,a={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return o={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(o[Symbol.iterator]=function(){return this}),o;function s(o){return function(s){return function(o){if(n)throw new TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(i=2&o[0]?r.return:o[0]?r.throw||((i=r.return)&&i.call(r),0):r.next)&&!(i=i.call(r,o[1])).done)return i;switch(r=0,i&&(o=[2&o[0],i.value]),o[0]){case 0:case 1:i=o;break;case 4:return a.label++,{value:o[1],done:!1};case 5:a.label++,r=o[1],o=[0];continue;case 7:o=a.ops.pop(),a.trys.pop();continue;default:if(!(i=(i=a.trys).length>0&&i[i.length-1])&&(6===o[0]||2===o[0])){a=0;continue}if(3===o[0]&&(!i||o[1]>i[0]&&o[1]<i[3])){a.label=o[1];break}if(6===o[0]&&a.label<i[1]){a.label=i[1],i=o;break}if(i&&a.label<i[2]){a.label=i[2],a.ops.push(o);break}i[2]&&a.ops.pop(),a.trys.pop();continue}o=e.call(t,a)}catch(t){o=[6,t],r=0}finally{n=i=0}if(5&o[0])throw o[1];return{value:o[0]?o[1]:void 0,done:!0}}([o,s])}}},D="2.0.0".split(".",1)[0],M=function(){function t(){this.comm_target_name="jupyter.widget",this._models=Object.create(null)}return t.prototype.display_model=function(t,e,n){var r=this;return void 0===n&&(n={}),this.create_view(e,n).then(function(e){return r.display_view(t,e,n)}).catch(C("Could not create view",!0))},t.prototype.setViewOptions=function(t){return void 0===t&&(t={}),t},t.prototype.create_view=function(t,e){var n=this;void 0===e&&(e={});var r=t.state_change=t.state_change.then(function(){return n.loadClass(t.get("_view_name"),t.get("_view_module"),t.get("_view_module_version")).then(function(r){var i=new r({model:t,options:n.setViewOptions(e)});return i.listenTo(t,"destroy",i.remove),Promise.resolve(i.render()).then(function(){return i})}).catch(C("Could not create a view for model id "+t.model_id,!0))}),i=f();return t.views[i]=r,r.then(function(t){t.once("remove",function(){delete t.model.views[i]},n)}),t.state_change},t.prototype.callbacks=function(t){return{}},t.prototype.get_model=function(t){return this._models[t]},t.prototype.handle_comm_open=function(t,e){var n=(e.metadata||{}).version||"";if(n.split(".",1)[0]!==D){var r="Wrong widget protocol version: received protocol version '"+n+"', but was expecting major version '"+D+"'";return console.error(r),Promise.reject(r)}var i=e.content.data,o=i.buffer_paths||[],a=(e.buffers||[]).map(function(t){return t instanceof DataView?t:new DataView(t instanceof ArrayBuffer?t:t.buffer)});return m(i.state,o,a),this.new_model({model_name:i.state._model_name,model_module:i.state._model_module,model_module_version:i.state._model_module_version,comm:t},i.state).catch(C("Could not create a model.",!0))},t.prototype.new_widget=function(t,e){var n,r=this;if(void 0===e&&(e={}),void 0===t.view_name||void 0===t.view_module||void 0===t.view_module_version)return Promise.reject("new_widget(...) must be given view information in the options.");n=t.comm?Promise.resolve(t.comm):this._create_comm(this.comm_target_name,t.model_id,{state:{_model_module:t.model_module,_model_module_version:t.model_module_version,_model_name:t.model_name,_view_module:t.view_module,_view_module_version:t.view_module_version,_view_name:t.view_name}},{version:"2.0.0"});var i=w({},t);return n.then(function(t){return i.comm=t,r.new_model(i,e).then(function(t){return t.sync("create",t),t})},function(){return i.model_id||(i.model_id=f()),r.new_model(i,e)})},t.prototype.register_model=function(t,e){var n=this;this._models[t]=e,e.then(function(e){e.once("comm:close",function(){delete n._models[t]})})},t.prototype.new_model=function(t,e){return void 0===e&&(e={}),x(this,void 0,void 0,function(){var n,r;return _(this,function(i){switch(i.label){case 0:if(t.model_id)n=t.model_id;else{if(!t.comm)throw new Error("Neither comm nor model_id provided in options object. At least one must exist.");n=t.model_id=t.comm.comm_id}return r=this._make_model(t,e),this.register_model(n,r),[4,r];case 1:return[2,i.sent()]}})})},t.prototype._make_model=function(t,e){return void 0===e&&(e={}),x(this,void 0,void 0,function(){var n,r,i,o,a,s,A;return _(this,function(l){switch(l.label){case 0:n=t.model_id,r=this.loadClass(t.model_name,t.model_module,t.model_module_version),l.label=1;case 1:return l.trys.push([1,3,,4]),[4,r];case 2:return i=l.sent(),[3,4];case 3:throw o=l.sent(),console.error("Could not instantiate widget"),o;case 4:if(!i)throw new Error("Cannot find model module "+t.model_module+"@"+t.model_module_version+", "+t.model_name);return[4,i._deserialize_state(e,this)];case 5:return a=l.sent(),s={widget_manager:this,model_id:n,comm:t.comm},(A=new i(a,s)).name=t.model_name,A.module=t.model_module,[2,A]}})})},t.prototype.clear_state=function(){var t=this;return I(this._models).then(function(e){Object.keys(e).forEach(function(t){return e[t].close()}),t._models=Object.create(null)})},t.prototype.get_state=function(t){var e=this;void 0===t&&(t={});var n=Object.keys(this._models).map(function(t){return e._models[t]});return Promise.all(n).then(function(e){return k(e,t)})},t.prototype.set_state=function(t){var e=this;if(!(t.version_major&&t.version_major<=2))throw"Unsupported widget state format";var n=t.state;return this._get_comm_info().then(function(t){return Promise.all(Object.keys(n).map(function(r){var i={base64:Q,hex:B},o=n[r],a=o.state;if(o.buffers){var s=o.buffers.map(function(t){return t.path}),A=o.buffers.map(function(t){return new DataView(i[t.encoding](t.data))});m(o.state,s,A)}if(e._models[r])return e._models[r].then(function(t){return t.constructor._deserialize_state(a||{},e).then(function(e){return t.set_state(e),t})});var l={model_id:r,model_name:o.model_name,model_module:o.model_module,model_module_version:o.model_module_version};return t.hasOwnProperty(r)?e._create_comm(e.comm_target_name,r).then(function(t){return l.comm=t,e.new_model(l)}):e.new_model(l,a)}))})},t.prototype.disconnect=function(){var t=this;Object.keys(this._models).forEach(function(e){t._models[e].then(function(t){t.comm_live=!1})})},t.prototype.resolveUrl=function(t){return Promise.resolve(t)},t.prototype.filterExistingModelState=function(t){var e=this,n=t.state;return n=Object.keys(n).filter(function(t){return!e._models[t]}).reduce(function(t,e){return t[e]=n[e],t},{}),w({},t,{state:n})},t}();function k(t,e){void 0===e&&(e={});var n={};return t.forEach(function(t){var r=t.model_id,i=y(t.serialize(t.get_state(e.drop_defaults))),o=i.buffers.map(function(t,e){return{data:b(t),path:i.buffer_paths[e],encoding:"base64"}});n[r]={model_name:t.name,model_module:t.module,model_module_version:t.get("_model_module_version"),state:i.state},o.length>0&&(n[r].buffers=o)}),{version_major:2,version_minor:0,state:n}}var S=function(){return(S=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)};var L=n(264),R=n(44),N=n.n(R),F=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),T=Element.prototype,U=T.matches||T.webkitMatchesSelector||T.mozMatchesSelector||T.msMatchesSelector||T.oMatchesSelector||function(t){for(var e=(this.document||this.ownerDocument).querySelectorAll(t),n=e.length;--n>=0&&e.item(n)!==this;);return n>-1},O=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return F(e,t),e.prototype._removeElement=function(){this.undelegateEvents(),this.el.parentNode&&this.el.parentNode.removeChild(this.el)},e.prototype._setElement=function(t){this.el=t},e.prototype._setAttributes=function(t){for(var e in t)e in this.el?this.el[e]=t[e]:this.el.setAttribute(e,t[e])},e.prototype.delegate=function(t,e,n){"string"!=typeof e&&(n=e,e=null),void 0===this._domEvents&&(this._domEvents=[]);var r=this.el,i=e?function(t){for(var i=t.target||t.srcElement;i&&i!==r;i=i.parentNode)if(U.call(i,e))return t.delegateTarget=i,n.handleEvent?n.handleEvent(t):n(t)}:n;return this.el.addEventListener(t,i,!1),this._domEvents.push({eventName:t,handler:i,listener:n,selector:e}),i},e.prototype.undelegate=function(t,e,n){if("function"==typeof e&&(n=e,e=null),this.el&&this._domEvents)for(var r=this._domEvents.slice(),i=r.length;i--;){var o=r[i];!(o.eventName!==t||n&&o.listener!==n||e&&o.selector!==e)&&(this.el.removeEventListener(o.eventName,o.handler,!1),this._domEvents.splice(i,1))}return this},e.prototype.undelegateEvents=function(){if(this.el&&this._domEvents){for(var t=this._domEvents.length,e=0;e<t;e++){var n=this._domEvents[e];this.el.removeEventListener(n.eventName,n.handler,!1)}this._domEvents.length=0}return this},e}(L.View),G=n(6),H=n(25),j=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Y=function(){return(Y=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)};function P(t,e){if(Array.isArray(t)){var n=[];return t.forEach(function(t,r){n.push(P(t,e))}),Promise.all(n)}if(t instanceof Object){var r={};return Object.keys(t).forEach(function(n){r[n]=P(t[n],e)}),I(r)}return"string"==typeof t&&"IPY_MODEL_"===t.slice(0,10)?e.get_model(t.slice(10,t.length)):Promise.resolve(t)}var K,z=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return j(e,t),e.prototype.defaults=function(){return{_model_module:"@jupyter-widgets/base",_model_name:"WidgetModel",_model_module_version:"1.2.0",_view_module:"@jupyter-widgets/base",_view_name:null,_view_module_version:"1.2.0",_view_count:null}},e.prototype.isNew=function(){return!1},e.prototype.initialize=function(e,n){t.prototype.initialize.call(this,e,n),this.widget_manager=n.widget_manager,this.model_id=n.model_id;var r=n.comm;this.views=Object.create(null),this.state_change=Promise.resolve(),this._closed=!1,this._state_lock=null,this._msg_buffer=null,this._msg_buffer_callbacks=null,this._pending_msgs=0,this._buffered_state_diff={},r?(this.comm=r,r.on_close(this._handle_comm_closed.bind(this)),r.on_msg(this._handle_comm_msg.bind(this)),this.comm_live=!0):this.comm_live=!1},Object.defineProperty(e.prototype,"comm_live",{get:function(){return this._comm_live},set:function(t){this._comm_live=t,this.trigger("comm_live_update")},enumerable:!0,configurable:!0}),e.prototype.send=function(t,e,n){if(void 0!==this.comm){var r={method:"custom",content:t};this.comm.send(r,e,{},n)}},e.prototype.close=function(t){var e=this;if(void 0===t&&(t=!1),!this._closed){this._closed=!0,this.comm&&!t&&this.comm.close(),this.stopListening(),this.trigger("destroy",this),this.comm&&delete this.comm;var n=Object.keys(this.views).map(function(t){return e.views[t].then(function(t){return t.remove()})});return delete this.views,Promise.all(n).then(function(){})}},e.prototype._handle_comm_closed=function(t){this.trigger("comm:close"),this.close(!0)},e.prototype._handle_comm_msg=function(t){var e=this,n=t.content.data;switch(n.method){case"update":return this.state_change=this.state_change.then(function(){var r=n.state;return m(r,n.buffer_paths||[],(t.buffers||[]).map(function(t){return t instanceof DataView?t:new DataView(t instanceof ArrayBuffer?t:t.buffer)})),e.constructor._deserialize_state(r,e.widget_manager)}).then(function(t){e.set_state(t)}).catch(C("Could not process update msg for model id: "+this.model_id,!0)),this.state_change;case"custom":return this.trigger("msg:custom",n.content,t.buffers),Promise.resolve()}},e.prototype.set_state=function(t){this._state_lock=t;try{this.set(t)}catch(t){console.error("Error setting state: "+t.message)}finally{this._state_lock=null}},e.prototype.get_state=function(t){var e=this.attributes;if(t){var n=this.defaults,r="function"==typeof n?n.call(this):n,i={};return Object.keys(e).forEach(function(t){h(e[t],r[t])||(i[t]=e[t])}),i}return Y({},e)},e.prototype._handle_status=function(t){void 0!==this.comm&&"idle"===t.content.execution_state&&(this._pending_msgs--,null!==this._msg_buffer&&this._pending_msgs<1&&(this.send_sync_message(this._msg_buffer,this._msg_buffer_callbacks),this._msg_buffer=null,this._msg_buffer_callbacks=null))},e.prototype.callbacks=function(t){return this.widget_manager.callbacks(t)},e.prototype.set=function(t,e,n){var r=function(t,e,n){if(null==t)return this;var r;if("object"==typeof t?(r=t,n=e):(r={})[t]=e,n||(n={}),!this._validate(r,n))return!1;var i=n.unset,o=n.silent,a=[],s=this._changing;this._changing=!0,s||(this._previousAttributes=S({},this.attributes),this.changed={});var A=this.attributes,l=this.changed,c=this._previousAttributes;for(var u in r)e=r[u],h(A[u],e)||a.push(u),h(c[u],e)?delete l[u]:l[u]=e,i?delete A[u]:A[u]=e;if(this.id=this.get(this.idAttribute),!o){a.length&&(this._pending=n);for(var g=0;g<a.length;g++)this.trigger("change:"+a[g],this,A[a[g]],n)}if(s)return this;if(!o)for(;this._pending;)n=this._pending,this._pending=!1,this.trigger("change",this,n);return this._pending=!1,this._changing=!1,this}.call(this,t,e,n);if(void 0!==this._buffered_state_diff){var i=this.changedAttributes()||{};if(this._state_lock)for(var o=0,a=Object.keys(this._state_lock);o<a.length;o++){var s=a[o];i[s]===this._state_lock[s]&&delete i[s]}this._buffered_state_diff=d(this._buffered_state_diff,i)}return r},e.prototype.sync=function(t,e,n){if(void 0===n&&(n={}),void 0===this.comm)throw"Syncing error: no comm channel defined";var r="patch"===t?n.attrs:e.get_state(n.drop_defaults);if(this._state_lock)for(var i=0,o=Object.keys(this._state_lock);i<o.length;i++){var a=o[i];r[a]===this._state_lock[a]&&delete r[a]}var s=this.serialize(r);if(Object.keys(s).length>0){var A=n.callbacks||this.callbacks();if(this._pending_msgs>=1){switch(t){case"patch":this._msg_buffer=d(this._msg_buffer||{},s);break;case"update":case"create":this._msg_buffer=s;break;default:throw"unrecognized syncing method"}this._msg_buffer_callbacks=A}else this.send_sync_message(r,A)}},e.prototype.serialize=function(t){for(var e=this.constructor.serializers||{},n=0,r=Object.keys(t);n<r.length;n++){var i=r[n];try{e[i]&&e[i].serialize?t[i]=e[i].serialize(t[i],this):t[i]=JSON.parse(JSON.stringify(t[i])),t[i]&&t[i].toJSON&&(t[i]=t[i].toJSON())}catch(t){throw console.error("Error serializing widget state attribute: ",i),t}}return t},e.prototype.send_sync_message=function(t,e){var n=this;void 0===e&&(e={});try{e.iopub=e.iopub||{};var r=e.iopub.status;e.iopub.status=function(t){n._handle_status(t),r&&r(t)};var i=y(t);this.comm.send({method:"update",state:i.state,buffer_paths:i.buffer_paths},e,{},i.buffers),this._pending_msgs++}catch(t){console.error("Could not send widget sync message",t)}},e.prototype.save_changes=function(t){if(this.comm_live){var e={patch:!0};t&&(e.callbacks=t),this.save(this._buffered_state_diff,e),this._buffered_state_diff={}}},e.prototype.on_some_change=function(t,e,n){var r=this;this.on("change",function(){t.some(r.hasChanged,r)&&e.apply(n,arguments)},this)},e.prototype.toJSON=function(t){return"IPY_MODEL_"+this.model_id},e._deserialize_state=function(t,e){var n,r=this.serializers;if(r)for(var i in n={},t)r[i]&&r[i].deserialize?n[i]=r[i].deserialize(t[i],e):n[i]=t[i];else n=t;return I(n)},e}(L.Model),J=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return j(e,t),e.prototype.defaults=function(){return d(t.prototype.defaults.call(this),{_dom_classes:[]})},e.serializers=Y({},z.serializers,{layout:{deserialize:P},style:{deserialize:P}}),e}(z),q=function(t){function e(e){return t.call(this,e)||this}return j(e,t),e.prototype.initialize=function(t){var e=this;this.listenTo(this.model,"change",function(){var t=Object.keys(e.model.changedAttributes()||{});"_view_count"===t[0]&&1===t.length||e.update()}),this.options=t.options,this.once("remove",function(){"number"==typeof e.model.get("_view_count")&&(e.model.set("_view_count",e.model.get("_view_count")-1),e.model.save_changes())}),this.once("displayed",function(){"number"==typeof e.model.get("_view_count")&&(e.model.set("_view_count",e.model.get("_view_count")+1),e.model.save_changes())}),this.displayed=new Promise(function(t,n){e.once("displayed",t)})},e.prototype.update=function(t){},e.prototype.render=function(){},e.prototype.create_child_view=function(t,e){return void 0===e&&(e={}),e=Y({parent:this},e),this.model.widget_manager.create_view(t,e).catch(C("Could not create child view",!0))},e.prototype.callbacks=function(){return this.model.callbacks(this)},e.prototype.send=function(t,e){this.model.send(t,this.callbacks(),e)},e.prototype.touch=function(){this.model.save_changes(this.callbacks())},e.prototype.remove=function(){return t.prototype.remove.call(this),this.trigger("remove"),this},e}(O),V=function(t){function e(e){var n=this,r=e.view;return delete e.view,(n=t.call(this,e)||this)._view=r,n}return j(e,t),e.prototype.dispose=function(){this.isDisposed||(t.prototype.dispose.call(this),this._view&&this._view.remove(),this._view=null)},e.prototype.processMessage=function(e){t.prototype.processMessage.call(this,e),this._view.processPhosphorMessage(e)},e}(G.Widget),W=function(t){function e(e){var n=this,r=e.view;return delete e.view,(n=t.call(this,e)||this)._view=r,n}return j(e,t),e.prototype.processMessage=function(e){t.prototype.processMessage.call(this,e),this._view.processPhosphorMessage(e)},e.prototype.dispose=function(){this.isDisposed||(t.prototype.dispose.call(this),this._view&&this._view.remove(),this._view=null)},e}(G.Panel),X=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return j(e,t),e.prototype.initialize=function(e){var n=this;t.prototype.initialize.call(this,e),this.listenTo(this.model,"change:_dom_classes",function(t,e){var r=t.previous("_dom_classes");n.update_classes(r,e)}),this.layoutPromise=Promise.resolve(),this.listenTo(this.model,"change:layout",function(t,e){n.setLayout(e,t.previous("layout"))}),this.stylePromise=Promise.resolve(),this.listenTo(this.model,"change:style",function(t,e){n.setStyle(e,t.previous("style"))}),this.displayed.then(function(){n.update_classes([],n.model.get("_dom_classes")),n.setLayout(n.model.get("layout")),n.setStyle(n.model.get("style"))}),this._comm_live_update(),this.listenTo(this.model,"comm_live_update",function(){n._comm_live_update()})},e.prototype.setLayout=function(t,e){var n=this;t&&(this.layoutPromise=this.layoutPromise.then(function(e){return e&&(e.unlayout(),n.stopListening(e.model),e.remove()),n.create_child_view(t).then(function(t){return n.displayed.then(function(){return t.trigger("displayed"),n.listenTo(t.model,"change",function(){H.MessageLoop.postMessage(n.pWidget,G.Widget.ResizeMessage.UnknownSize)}),H.MessageLoop.postMessage(n.pWidget,G.Widget.ResizeMessage.UnknownSize),t})}).catch(C("Could not add LayoutView to DOMWidgetView",!0))}))},e.prototype.setStyle=function(t,e){var n=this;t&&(this.stylePromise=this.stylePromise.then(function(e){return e&&(e.unstyle(),n.stopListening(e.model),e.remove()),n.create_child_view(t).then(function(t){return n.displayed.then(function(){return t.trigger("displayed"),t})}).catch(C("Could not add styleView to DOMWidgetView",!0))}))},e.prototype.update_classes=function(t,e,n){void 0===n&&(n=this.el),g(t,e).map(function(t){n.classList?n.classList.remove(t):n.setAttribute("class",n.getAttribute("class").replace(t,""))}),g(e,t).map(function(t){n.classList?n.classList.add(t):n.setAttribute("class",n.getAttribute("class").concat(" ",t))})},e.prototype.update_mapped_classes=function(t,e,n){var r=this.model.previous(e),i=t[r]?t[r]:[],o=t[r=this.model.get(e)]?t[r]:[];this.update_classes(i,o,n||this.el)},e.prototype.set_mapped_classes=function(t,e,n){var r=this.model.get(e),i=t[r]?t[r]:[];this.update_classes([],i,n||this.el)},e.prototype._setElement=function(t){this.pWidget&&this.pWidget.dispose(),this.$el=t instanceof N.a?t:N()(t),this.el=this.$el[0],this.pWidget=new V({node:t,view:this})},e.prototype.remove=function(){return this.pWidget&&this.pWidget.dispose(),t.prototype.remove.call(this)},e.prototype.processPhosphorMessage=function(t){switch(t.type){case"after-attach":this.trigger("displayed")}},e.prototype._comm_live_update=function(){this.model.comm_live?this.pWidget.removeClass("jupyter-widgets-disconnected"):this.pWidget.addClass("jupyter-widgets-disconnected")},e}(q),Z=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),$={align_content:null,align_items:null,align_self:null,border:null,bottom:null,display:null,flex:null,flex_flow:null,height:null,justify_content:null,justify_items:null,left:null,margin:null,max_height:null,max_width:null,min_height:null,min_width:null,overflow:null,overflow_x:null,overflow_y:null,order:null,padding:null,right:null,top:null,visibility:null,width:null,object_fit:null,object_position:null,grid_auto_columns:null,grid_auto_flow:null,grid_auto_rows:null,grid_gap:null,grid_template_rows:null,grid_template_columns:null,grid_template_areas:null,grid_row:null,grid_column:null,grid_area:null},tt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Z(e,t),e.prototype.defaults=function(){return d(t.prototype.defaults.call(this),{_model_name:"LayoutModel",_view_name:"LayoutView"},$)},e}(z),et=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Z(e,t),e.prototype.initialize=function(e){this._traitNames=[],t.prototype.initialize.call(this,e);for(var n=0,r=Object.keys($);n<r.length;n++){var i=r[n];this.registerTrait(i)}},e.prototype.registerTrait=function(t){var e=this;if(this._traitNames.push(t),"overflow_x"===t||"overflow_y"===t)return this.listenTo(this.model,"change:"+t,function(n,r){e.handleOverflowChange(t,r)}),void this.handleOverflowChange(t,this.model.get(t));this.listenTo(this.model,"change:"+t,function(n,r){e.handleChange(t,r)}),this.handleChange(t,this.model.get(t))},e.prototype.css_name=function(t){return t.replace(/_/g,"-")},e.prototype.handleChange=function(t,e){var n=this.options.parent;n?null===e?n.el.style.removeProperty(this.css_name(t)):n.el.style[this.css_name(t)]=e:console.warn("Style not applied because a parent view does not exist")},e.prototype.handleOverflowChange=function(t,e){var n=this.options.parent;n?null===e?null===this.model.get("overflow")&&n.el.style.removeProperty(this.css_name(t)):n.el.style[this.css_name(t)]=e:console.warn("Style not applied because a parent view does not exist")},e.prototype.unlayout=function(){var t=this,e=this.options.parent;this._traitNames.forEach(function(n){e?e.el.style.removeProperty(t.css_name(n)):console.warn("Style not removed because a parent view does not exist")},this)},e}(q),nt=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),rt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return nt(e,t),e.prototype.defaults=function(){var e=this.constructor;return d(t.prototype.defaults.call(this),{_model_name:"StyleModel",_view_name:"StyleView"},Object.keys(e.styleProperties).reduce(function(t,n){return t[n]=e.styleProperties[n].default,t},{}))},e.styleProperties={},e}(z),it=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return nt(e,t),e.prototype.initialize=function(e){this._traitNames=[],t.prototype.initialize.call(this,e);for(var n=this.model.constructor,r=0,i=Object.keys(n.styleProperties);r<i.length;r++){var o=i[r];this.registerTrait(o)}this.style()},e.prototype.registerTrait=function(t){var e=this;this._traitNames.push(t),this.listenTo(this.model,"change:"+t,function(n,r){e.handleChange(t,r)})},e.prototype.handleChange=function(t,e){var n=this.options.parent;if(n){var r=this.model.constructor.styleProperties,i=r[t].attribute,o=r[t].selector,a=o?n.el.querySelectorAll(o):[n.el];if(null===e)for(var s=0;s!==a.length;++s)a[s].style.removeProperty(i);else for(s=0;s!==a.length;++s)a[s].style[i]=e}else console.warn("Style not applied because a parent view does not exist")},e.prototype.style=function(){for(var t=0,e=this._traitNames;t<e.length;t++){var n=e[t];this.handleChange(n,this.model.get(n))}},e.prototype.unstyle=function(){var t=this.options.parent,e=this.model.constructor.styleProperties;this._traitNames.forEach(function(n){if(t)for(var r=e[n].attribute,i=e[n].selector,o=i?t.el.querySelectorAll(i):[t.el],a=0;a!==o.length;++a)o[a].style.removeProperty(r);else console.warn("Style not removed because a parent view does not exist")},this)},e}(q),ot=function(t,e,n,r){return new(n||(n=Promise))(function(i,o){function a(t){try{A(r.next(t))}catch(t){o(t)}}function s(t){try{A(r.throw(t))}catch(t){o(t)}}function A(t){t.done?i(t.value):new n(function(e){e(t.value)}).then(a,s)}A((r=r.apply(t,e||[])).next())})},at=function(t,e){var n,r,i,o,a={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return o={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(o[Symbol.iterator]=function(){return this}),o;function s(o){return function(s){return function(o){if(n)throw new TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(i=2&o[0]?r.return:o[0]?r.throw||((i=r.return)&&i.call(r),0):r.next)&&!(i=i.call(r,o[1])).done)return i;switch(r=0,i&&(o=[2&o[0],i.value]),o[0]){case 0:case 1:i=o;break;case 4:return a.label++,{value:o[1],done:!1};case 5:a.label++,r=o[1],o=[0];continue;case 7:o=a.ops.pop(),a.trys.pop();continue;default:if(!(i=(i=a.trys).length>0&&i[i.length-1])&&(6===o[0]||2===o[0])){a=0;continue}if(3===o[0]&&(!i||o[1]>i[0]&&o[1]<i[3])){a.label=o[1];break}if(6===o[0]&&a.label<i[1]){a.label=i[1],i=o;break}if(i&&a.label<i[2]){a.label=i[2],a.ops.push(o);break}i[2]&&a.ops.pop(),a.trys.pop();continue}o=e.call(t,a)}catch(t){o=[6,t],r=0}finally{n=i=0}if(5&o[0])throw o[1];return{value:o[0]?o[1]:void 0,done:!0}}([o,s])}}};!function(t){!function(t){var e=function(){function t(t){this.targets=Object.create(null),this.comms=Object.create(null),this.kernel=null,this.jsServicesKernel=null,this.init_kernel(t)}return t.prototype.init_kernel=function(t){this.kernel=t,this.jsServicesKernel=t},t.prototype.new_comm=function(t,e,r,i,o,a){return ot(this,void 0,void 0,function(){var s,A;return at(this,function(l){switch(l.label){case 0:return[4,Promise.resolve(this.jsServicesKernel.connectToComm(t,o))];case 1:return s=l.sent(),A=new n(s),this.register_comm(A),A.open(e,r,i,a),[2,A]}})})},t.prototype.register_target=function(t,e){var r=this,i=this.jsServicesKernel.registerCommTarget(t,function(t,i){var o=new n(t);r.register_comm(o);try{return e(o,i)}catch(t){o.close(),console.error(t),console.error(new Error("Exception opening new comm"))}});this.targets[t]=i},t.prototype.unregister_target=function(t,e){this.targets[t].dispose(),delete this.targets[t]},t.prototype.register_comm=function(t){return this.comms[t.comm_id]=Promise.resolve(t),t.kernel=this.kernel,t.comm_id},t}();t.CommManager=e;var n=function(){function t(t){this.jsServicesComm=null,this.kernel=null,this.jsServicesComm=t}return Object.defineProperty(t.prototype,"comm_id",{get:function(){return this.jsServicesComm.commId},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"target_name",{get:function(){return this.jsServicesComm.targetName},enumerable:!0,configurable:!0}),t.prototype.open=function(t,e,n,r){var i=this.jsServicesComm.open(t,n,r);return this._hookupCallbacks(i,e),i.msg.header.msg_id},t.prototype.send=function(t,e,n,r){var i=this.jsServicesComm.send(t,n,r);return this._hookupCallbacks(i,e),i.msg.header.msg_id},t.prototype.close=function(t,e,n,r){var i=this.jsServicesComm.close(t,n,r);return this._hookupCallbacks(i,e),i.msg.header.msg_id},t.prototype.on_msg=function(t){this.jsServicesComm.onMsg=t.bind(this)},t.prototype.on_close=function(t){this.jsServicesComm.onClose=t.bind(this)},t.prototype._hookupCallbacks=function(t,e){e&&(t.onReply=function(t){e.shell&&e.shell.reply&&e.shell.reply(t)},t.onStdin=function(t){e.input&&e.input(t)},t.onIOPub=function(t){if(e.iopub)if(e.iopub.status&&"status"===t.header.msg_type)e.iopub.status(t);else if(e.iopub.clear_output&&"clear_output"===t.header.msg_type)e.iopub.clear_output(t);else if(e.iopub.output)switch(t.header.msg_type){case"display_data":case"execute_result":e.iopub.output(t)}})},t}();t.Comm=n}(t.services||(t.services={}))}(K||(K={}));var st=function(){function t(t,e,n){this.initialize(t,e,n)}return t.prototype.initialize=function(t,e,n){this._handler_context=n||this,this._models=[],this.views=[],this._create_view=t,this._remove_view=e||function(t){t.remove()}},t.prototype.update=function(t,e,n,r){var i=n||this._remove_view,o=e||this._create_view;r=r||this._handler_context;for(var a=0;a<t.length&&!(a>=this._models.length||t[a]!==this._models[a]);a++);for(var s=a,A=this.views.splice(s,this.views.length-s),l=0;l<A.length;l++)A[l].then(function(t){i.call(r,t)});for(;a<t.length;a++)this.views.push(Promise.resolve(o.call(r,t[a],a)));return this._models=t.slice(),Promise.all(this.views)},t.prototype.remove=function(){var t=this;return Promise.all(this.views).then(function(e){e.forEach(function(e){return t._remove_view.call(t._handler_context,e)}),t.views=[],t._models=[]})},t.prototype.dispose=function(){this.views=null,this._models=null},t}(),At=new o.Token("jupyter.extensions.jupyterWidgetRegistry");n.d(e,"ManagerBase",function(){return M}),n.d(e,"serialize_state",function(){return k}),n.d(e,"unpack_models",function(){return P}),n.d(e,"WidgetModel",function(){return z}),n.d(e,"DOMWidgetModel",function(){return J}),n.d(e,"WidgetView",function(){return q}),n.d(e,"JupyterPhosphorWidget",function(){return V}),n.d(e,"JupyterPhosphorPanelWidget",function(){return W}),n.d(e,"DOMWidgetView",function(){return X}),n.d(e,"LayoutModel",function(){return tt}),n.d(e,"LayoutView",function(){return et}),n.d(e,"StyleModel",function(){return rt}),n.d(e,"StyleView",function(){return it}),n.d(e,"shims",function(){return K}),n.d(e,"ViewList",function(){return st}),n.d(e,"JUPYTER_WIDGETS_VERSION",function(){return"1.2.0"}),n.d(e,"PROTOCOL_VERSION",function(){return"2.0.0"}),n.d(e,"difference",function(){return g}),n.d(e,"isEqual",function(){return h}),n.d(e,"assign",function(){return d}),n.d(e,"uuid",function(){return f}),n.d(e,"WrappedError",function(){return p}),n.d(e,"resolvePromisesDict",function(){return I}),n.d(e,"reject",function(){return C}),n.d(e,"put_buffers",function(){return m}),n.d(e,"remove_buffers",function(){return y}),n.d(e,"bufferToHex",function(){return v}),n.d(e,"hexToBuffer",function(){return B}),n.d(e,"bufferToBase64",function(){return b}),n.d(e,"base64ToBuffer",function(){return Q}),n.d(e,"IJupyterWidgetRegistry",function(){return At})},function(t,e,n){"use strict";var r=n(158),i=n(11),o=n(130),a=n(161);var s=n(79),A=n(129),l=n(63);n.d(e,"a",function(){return c});var c=function(){function t(t){this._isScalar=!1,t&&(this._subscribe=t)}return t.prototype.lift=function(e){var n=new t;return n.source=this,n.operator=e,n},t.prototype.subscribe=function(t,e,n){var r=this.operator,s=function(t,e,n){if(t){if(t instanceof i.a)return t;if(t[o.a])return t[o.a]()}return t||e||n?new i.a(t,e,n):new i.a(a.a)}(t,e,n);if(r?s.add(r.call(s,this.source)):s.add(this.source||l.a.useDeprecatedSynchronousErrorHandling&&!s.syncErrorThrowable?this._subscribe(s):this._trySubscribe(s)),l.a.useDeprecatedSynchronousErrorHandling&&s.syncErrorThrowable&&(s.syncErrorThrowable=!1,s.syncErrorThrown))throw s.syncErrorValue;return s},t.prototype._trySubscribe=function(t){try{return this._subscribe(t)}catch(e){l.a.useDeprecatedSynchronousErrorHandling&&(t.syncErrorThrown=!0,t.syncErrorValue=e),Object(r.a)(t)?t.error(e):console.warn(e)}},t.prototype.forEach=function(t,e){var n=this;return new(e=u(e))(function(e,r){var i;i=n.subscribe(function(e){try{t(e)}catch(t){r(t),i&&i.unsubscribe()}},r,e)})},t.prototype._subscribe=function(t){var e=this.source;return e&&e.subscribe(t)},t.prototype[s.a]=function(){return this},t.prototype.pipe=function(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return 0===t.length?this:Object(A.b)(t)(this)},t.prototype.toPromise=function(t){var e=this;return new(t=u(t))(function(t,n){var r;e.subscribe(function(t){return r=t},function(t){return n(t)},function(){return t(r)})})},t.create=function(e){return new t(e)},t}();function u(t){if(t||(t=l.a.Promise||Promise),!t)throw new Error("no Promise impl found");return t}},function(t,e,n){(function(t,n){var r;!function(){var i="object"==typeof self&&self.self===self&&self||"object"==typeof t&&t.global===t&&t||this||{},o=i._,a=Array.prototype,s=Object.prototype,A="undefined"!=typeof Symbol?Symbol.prototype:null,l=a.push,c=a.slice,u=s.toString,g=s.hasOwnProperty,h=Array.isArray,d=Object.keys,f=Object.create,p=function(){},I=function(t){return t instanceof I?t:this instanceof I?void(this._wrapped=t):new I(t)};e.nodeType?i._=I:(!n.nodeType&&n.exports&&(e=n.exports=I),e._=I),I.VERSION="1.9.1";var C,m=function(t,e,n){if(void 0===e)return t;switch(null==n?3:n){case 1:return function(n){return t.call(e,n)};case 3:return function(n,r,i){return t.call(e,n,r,i)};case 4:return function(n,r,i,o){return t.call(e,n,r,i,o)}}return function(){return t.apply(e,arguments)}},y=function(t,e,n){return I.iteratee!==C?I.iteratee(t,e):null==t?I.identity:I.isFunction(t)?m(t,e,n):I.isObject(t)&&!I.isArray(t)?I.matcher(t):I.property(t)};I.iteratee=C=function(t,e){return y(t,e,1/0)};var E=function(t,e){return e=null==e?t.length-1:+e,function(){for(var n=Math.max(arguments.length-e,0),r=Array(n),i=0;i<n;i++)r[i]=arguments[i+e];switch(e){case 0:return t.call(this,r);case 1:return t.call(this,arguments[0],r);case 2:return t.call(this,arguments[0],arguments[1],r)}var o=Array(e+1);for(i=0;i<e;i++)o[i]=arguments[i];return o[e]=r,t.apply(this,o)}},v=function(t){if(!I.isObject(t))return{};if(f)return f(t);p.prototype=t;var e=new p;return p.prototype=null,e},B=function(t){return function(e){return null==e?void 0:e[t]}},b=function(t,e){return null!=t&&g.call(t,e)},Q=function(t,e){for(var n=e.length,r=0;r<n;r++){if(null==t)return;t=t[e[r]]}return n?t:void 0},w=Math.pow(2,53)-1,x=B("length"),_=function(t){var e=x(t);return"number"==typeof e&&e>=0&&e<=w};I.each=I.forEach=function(t,e,n){var r,i;if(e=m(e,n),_(t))for(r=0,i=t.length;r<i;r++)e(t[r],r,t);else{var o=I.keys(t);for(r=0,i=o.length;r<i;r++)e(t[o[r]],o[r],t)}return t},I.map=I.collect=function(t,e,n){e=y(e,n);for(var r=!_(t)&&I.keys(t),i=(r||t).length,o=Array(i),a=0;a<i;a++){var s=r?r[a]:a;o[a]=e(t[s],s,t)}return o};var D=function(t){var e=function(e,n,r,i){var o=!_(e)&&I.keys(e),a=(o||e).length,s=t>0?0:a-1;for(i||(r=e[o?o[s]:s],s+=t);s>=0&&s<a;s+=t){var A=o?o[s]:s;r=n(r,e[A],A,e)}return r};return function(t,n,r,i){var o=arguments.length>=3;return e(t,m(n,i,4),r,o)}};I.reduce=I.foldl=I.inject=D(1),I.reduceRight=I.foldr=D(-1),I.find=I.detect=function(t,e,n){var r=(_(t)?I.findIndex:I.findKey)(t,e,n);if(void 0!==r&&-1!==r)return t[r]},I.filter=I.select=function(t,e,n){var r=[];return e=y(e,n),I.each(t,function(t,n,i){e(t,n,i)&&r.push(t)}),r},I.reject=function(t,e,n){return I.filter(t,I.negate(y(e)),n)},I.every=I.all=function(t,e,n){e=y(e,n);for(var r=!_(t)&&I.keys(t),i=(r||t).length,o=0;o<i;o++){var a=r?r[o]:o;if(!e(t[a],a,t))return!1}return!0},I.some=I.any=function(t,e,n){e=y(e,n);for(var r=!_(t)&&I.keys(t),i=(r||t).length,o=0;o<i;o++){var a=r?r[o]:o;if(e(t[a],a,t))return!0}return!1},I.contains=I.includes=I.include=function(t,e,n,r){return _(t)||(t=I.values(t)),("number"!=typeof n||r)&&(n=0),I.indexOf(t,e,n)>=0},I.invoke=E(function(t,e,n){var r,i;return I.isFunction(e)?i=e:I.isArray(e)&&(r=e.slice(0,-1),e=e[e.length-1]),I.map(t,function(t){var o=i;if(!o){if(r&&r.length&&(t=Q(t,r)),null==t)return;o=t[e]}return null==o?o:o.apply(t,n)})}),I.pluck=function(t,e){return I.map(t,I.property(e))},I.where=function(t,e){return I.filter(t,I.matcher(e))},I.findWhere=function(t,e){return I.find(t,I.matcher(e))},I.max=function(t,e,n){var r,i,o=-1/0,a=-1/0;if(null==e||"number"==typeof e&&"object"!=typeof t[0]&&null!=t)for(var s=0,A=(t=_(t)?t:I.values(t)).length;s<A;s++)null!=(r=t[s])&&r>o&&(o=r);else e=y(e,n),I.each(t,function(t,n,r){((i=e(t,n,r))>a||i===-1/0&&o===-1/0)&&(o=t,a=i)});return o},I.min=function(t,e,n){var r,i,o=1/0,a=1/0;if(null==e||"number"==typeof e&&"object"!=typeof t[0]&&null!=t)for(var s=0,A=(t=_(t)?t:I.values(t)).length;s<A;s++)null!=(r=t[s])&&r<o&&(o=r);else e=y(e,n),I.each(t,function(t,n,r){((i=e(t,n,r))<a||i===1/0&&o===1/0)&&(o=t,a=i)});return o},I.shuffle=function(t){return I.sample(t,1/0)},I.sample=function(t,e,n){if(null==e||n)return _(t)||(t=I.values(t)),t[I.random(t.length-1)];var r=_(t)?I.clone(t):I.values(t),i=x(r);e=Math.max(Math.min(e,i),0);for(var o=i-1,a=0;a<e;a++){var s=I.random(a,o),A=r[a];r[a]=r[s],r[s]=A}return r.slice(0,e)},I.sortBy=function(t,e,n){var r=0;return e=y(e,n),I.pluck(I.map(t,function(t,n,i){return{value:t,index:r++,criteria:e(t,n,i)}}).sort(function(t,e){var n=t.criteria,r=e.criteria;if(n!==r){if(n>r||void 0===n)return 1;if(n<r||void 0===r)return-1}return t.index-e.index}),"value")};var M=function(t,e){return function(n,r,i){var o=e?[[],[]]:{};return r=y(r,i),I.each(n,function(e,i){var a=r(e,i,n);t(o,e,a)}),o}};I.groupBy=M(function(t,e,n){b(t,n)?t[n].push(e):t[n]=[e]}),I.indexBy=M(function(t,e,n){t[n]=e}),I.countBy=M(function(t,e,n){b(t,n)?t[n]++:t[n]=1});var k=/[^\ud800-\udfff]|[\ud800-\udbff][\udc00-\udfff]|[\ud800-\udfff]/g;I.toArray=function(t){return t?I.isArray(t)?c.call(t):I.isString(t)?t.match(k):_(t)?I.map(t,I.identity):I.values(t):[]},I.size=function(t){return null==t?0:_(t)?t.length:I.keys(t).length},I.partition=M(function(t,e,n){t[n?0:1].push(e)},!0),I.first=I.head=I.take=function(t,e,n){return null==t||t.length<1?null==e?void 0:[]:null==e||n?t[0]:I.initial(t,t.length-e)},I.initial=function(t,e,n){return c.call(t,0,Math.max(0,t.length-(null==e||n?1:e)))},I.last=function(t,e,n){return null==t||t.length<1?null==e?void 0:[]:null==e||n?t[t.length-1]:I.rest(t,Math.max(0,t.length-e))},I.rest=I.tail=I.drop=function(t,e,n){return c.call(t,null==e||n?1:e)},I.compact=function(t){return I.filter(t,Boolean)};var S=function(t,e,n,r){for(var i=(r=r||[]).length,o=0,a=x(t);o<a;o++){var s=t[o];if(_(s)&&(I.isArray(s)||I.isArguments(s)))if(e)for(var A=0,l=s.length;A<l;)r[i++]=s[A++];else S(s,e,n,r),i=r.length;else n||(r[i++]=s)}return r};I.flatten=function(t,e){return S(t,e,!1)},I.without=E(function(t,e){return I.difference(t,e)}),I.uniq=I.unique=function(t,e,n,r){I.isBoolean(e)||(r=n,n=e,e=!1),null!=n&&(n=y(n,r));for(var i=[],o=[],a=0,s=x(t);a<s;a++){var A=t[a],l=n?n(A,a,t):A;e&&!n?(a&&o===l||i.push(A),o=l):n?I.contains(o,l)||(o.push(l),i.push(A)):I.contains(i,A)||i.push(A)}return i},I.union=E(function(t){return I.uniq(S(t,!0,!0))}),I.intersection=function(t){for(var e=[],n=arguments.length,r=0,i=x(t);r<i;r++){var o=t[r];if(!I.contains(e,o)){var a;for(a=1;a<n&&I.contains(arguments[a],o);a++);a===n&&e.push(o)}}return e},I.difference=E(function(t,e){return e=S(e,!0,!0),I.filter(t,function(t){return!I.contains(e,t)})}),I.unzip=function(t){for(var e=t&&I.max(t,x).length||0,n=Array(e),r=0;r<e;r++)n[r]=I.pluck(t,r);return n},I.zip=E(I.unzip),I.object=function(t,e){for(var n={},r=0,i=x(t);r<i;r++)e?n[t[r]]=e[r]:n[t[r][0]]=t[r][1];return n};var L=function(t){return function(e,n,r){n=y(n,r);for(var i=x(e),o=t>0?0:i-1;o>=0&&o<i;o+=t)if(n(e[o],o,e))return o;return-1}};I.findIndex=L(1),I.findLastIndex=L(-1),I.sortedIndex=function(t,e,n,r){for(var i=(n=y(n,r,1))(e),o=0,a=x(t);o<a;){var s=Math.floor((o+a)/2);n(t[s])<i?o=s+1:a=s}return o};var R=function(t,e,n){return function(r,i,o){var a=0,s=x(r);if("number"==typeof o)t>0?a=o>=0?o:Math.max(o+s,a):s=o>=0?Math.min(o+1,s):o+s+1;else if(n&&o&&s)return r[o=n(r,i)]===i?o:-1;if(i!=i)return(o=e(c.call(r,a,s),I.isNaN))>=0?o+a:-1;for(o=t>0?a:s-1;o>=0&&o<s;o+=t)if(r[o]===i)return o;return-1}};I.indexOf=R(1,I.findIndex,I.sortedIndex),I.lastIndexOf=R(-1,I.findLastIndex),I.range=function(t,e,n){null==e&&(e=t||0,t=0),n||(n=e<t?-1:1);for(var r=Math.max(Math.ceil((e-t)/n),0),i=Array(r),o=0;o<r;o++,t+=n)i[o]=t;return i},I.chunk=function(t,e){if(null==e||e<1)return[];for(var n=[],r=0,i=t.length;r<i;)n.push(c.call(t,r,r+=e));return n};var N=function(t,e,n,r,i){if(!(r instanceof e))return t.apply(n,i);var o=v(t.prototype),a=t.apply(o,i);return I.isObject(a)?a:o};I.bind=E(function(t,e,n){if(!I.isFunction(t))throw new TypeError("Bind must be called on a function");var r=E(function(i){return N(t,r,e,this,n.concat(i))});return r}),I.partial=E(function(t,e){var n=I.partial.placeholder,r=function(){for(var i=0,o=e.length,a=Array(o),s=0;s<o;s++)a[s]=e[s]===n?arguments[i++]:e[s];for(;i<arguments.length;)a.push(arguments[i++]);return N(t,r,this,this,a)};return r}),I.partial.placeholder=I,I.bindAll=E(function(t,e){var n=(e=S(e,!1,!1)).length;if(n<1)throw new Error("bindAll must be passed function names");for(;n--;){var r=e[n];t[r]=I.bind(t[r],t)}}),I.memoize=function(t,e){var n=function(r){var i=n.cache,o=""+(e?e.apply(this,arguments):r);return b(i,o)||(i[o]=t.apply(this,arguments)),i[o]};return n.cache={},n},I.delay=E(function(t,e,n){return setTimeout(function(){return t.apply(null,n)},e)}),I.defer=I.partial(I.delay,I,1),I.throttle=function(t,e,n){var r,i,o,a,s=0;n||(n={});var A=function(){s=!1===n.leading?0:I.now(),r=null,a=t.apply(i,o),r||(i=o=null)},l=function(){var l=I.now();s||!1!==n.leading||(s=l);var c=e-(l-s);return i=this,o=arguments,c<=0||c>e?(r&&(clearTimeout(r),r=null),s=l,a=t.apply(i,o),r||(i=o=null)):r||!1===n.trailing||(r=setTimeout(A,c)),a};return l.cancel=function(){clearTimeout(r),s=0,r=i=o=null},l},I.debounce=function(t,e,n){var r,i,o=function(e,n){r=null,n&&(i=t.apply(e,n))},a=E(function(a){if(r&&clearTimeout(r),n){var s=!r;r=setTimeout(o,e),s&&(i=t.apply(this,a))}else r=I.delay(o,e,this,a);return i});return a.cancel=function(){clearTimeout(r),r=null},a},I.wrap=function(t,e){return I.partial(e,t)},I.negate=function(t){return function(){return!t.apply(this,arguments)}},I.compose=function(){var t=arguments,e=t.length-1;return function(){for(var n=e,r=t[e].apply(this,arguments);n--;)r=t[n].call(this,r);return r}},I.after=function(t,e){return function(){if(--t<1)return e.apply(this,arguments)}},I.before=function(t,e){var n;return function(){return--t>0&&(n=e.apply(this,arguments)),t<=1&&(e=null),n}},I.once=I.partial(I.before,2),I.restArguments=E;var F=!{toString:null}.propertyIsEnumerable("toString"),T=["valueOf","isPrototypeOf","toString","propertyIsEnumerable","hasOwnProperty","toLocaleString"],U=function(t,e){var n=T.length,r=t.constructor,i=I.isFunction(r)&&r.prototype||s,o="constructor";for(b(t,o)&&!I.contains(e,o)&&e.push(o);n--;)(o=T[n])in t&&t[o]!==i[o]&&!I.contains(e,o)&&e.push(o)};I.keys=function(t){if(!I.isObject(t))return[];if(d)return d(t);var e=[];for(var n in t)b(t,n)&&e.push(n);return F&&U(t,e),e},I.allKeys=function(t){if(!I.isObject(t))return[];var e=[];for(var n in t)e.push(n);return F&&U(t,e),e},I.values=function(t){for(var e=I.keys(t),n=e.length,r=Array(n),i=0;i<n;i++)r[i]=t[e[i]];return r},I.mapObject=function(t,e,n){e=y(e,n);for(var r=I.keys(t),i=r.length,o={},a=0;a<i;a++){var s=r[a];o[s]=e(t[s],s,t)}return o},I.pairs=function(t){for(var e=I.keys(t),n=e.length,r=Array(n),i=0;i<n;i++)r[i]=[e[i],t[e[i]]];return r},I.invert=function(t){for(var e={},n=I.keys(t),r=0,i=n.length;r<i;r++)e[t[n[r]]]=n[r];return e},I.functions=I.methods=function(t){var e=[];for(var n in t)I.isFunction(t[n])&&e.push(n);return e.sort()};var O=function(t,e){return function(n){var r=arguments.length;if(e&&(n=Object(n)),r<2||null==n)return n;for(var i=1;i<r;i++)for(var o=arguments[i],a=t(o),s=a.length,A=0;A<s;A++){var l=a[A];e&&void 0!==n[l]||(n[l]=o[l])}return n}};I.extend=O(I.allKeys),I.extendOwn=I.assign=O(I.keys),I.findKey=function(t,e,n){e=y(e,n);for(var r,i=I.keys(t),o=0,a=i.length;o<a;o++)if(e(t[r=i[o]],r,t))return r};var G,H,j=function(t,e,n){return e in n};I.pick=E(function(t,e){var n={},r=e[0];if(null==t)return n;I.isFunction(r)?(e.length>1&&(r=m(r,e[1])),e=I.allKeys(t)):(r=j,e=S(e,!1,!1),t=Object(t));for(var i=0,o=e.length;i<o;i++){var a=e[i],s=t[a];r(s,a,t)&&(n[a]=s)}return n}),I.omit=E(function(t,e){var n,r=e[0];return I.isFunction(r)?(r=I.negate(r),e.length>1&&(n=e[1])):(e=I.map(S(e,!1,!1),String),r=function(t,n){return!I.contains(e,n)}),I.pick(t,r,n)}),I.defaults=O(I.allKeys,!0),I.create=function(t,e){var n=v(t);return e&&I.extendOwn(n,e),n},I.clone=function(t){return I.isObject(t)?I.isArray(t)?t.slice():I.extend({},t):t},I.tap=function(t,e){return e(t),t},I.isMatch=function(t,e){var n=I.keys(e),r=n.length;if(null==t)return!r;for(var i=Object(t),o=0;o<r;o++){var a=n[o];if(e[a]!==i[a]||!(a in i))return!1}return!0},G=function(t,e,n,r){if(t===e)return 0!==t||1/t==1/e;if(null==t||null==e)return!1;if(t!=t)return e!=e;var i=typeof t;return("function"===i||"object"===i||"object"==typeof e)&&H(t,e,n,r)},H=function(t,e,n,r){t instanceof I&&(t=t._wrapped),e instanceof I&&(e=e._wrapped);var i=u.call(t);if(i!==u.call(e))return!1;switch(i){case"[object RegExp]":case"[object String]":return""+t==""+e;case"[object Number]":return+t!=+t?+e!=+e:0==+t?1/+t==1/e:+t==+e;case"[object Date]":case"[object Boolean]":return+t==+e;case"[object Symbol]":return A.valueOf.call(t)===A.valueOf.call(e)}var o="[object Array]"===i;if(!o){if("object"!=typeof t||"object"!=typeof e)return!1;var a=t.constructor,s=e.constructor;if(a!==s&&!(I.isFunction(a)&&a instanceof a&&I.isFunction(s)&&s instanceof s)&&"constructor"in t&&"constructor"in e)return!1}r=r||[];for(var l=(n=n||[]).length;l--;)if(n[l]===t)return r[l]===e;if(n.push(t),r.push(e),o){if((l=t.length)!==e.length)return!1;for(;l--;)if(!G(t[l],e[l],n,r))return!1}else{var c,g=I.keys(t);if(l=g.length,I.keys(e).length!==l)return!1;for(;l--;)if(c=g[l],!b(e,c)||!G(t[c],e[c],n,r))return!1}return n.pop(),r.pop(),!0},I.isEqual=function(t,e){return G(t,e)},I.isEmpty=function(t){return null==t||(_(t)&&(I.isArray(t)||I.isString(t)||I.isArguments(t))?0===t.length:0===I.keys(t).length)},I.isElement=function(t){return!(!t||1!==t.nodeType)},I.isArray=h||function(t){return"[object Array]"===u.call(t)},I.isObject=function(t){var e=typeof t;return"function"===e||"object"===e&&!!t},I.each(["Arguments","Function","String","Number","Date","RegExp","Error","Symbol","Map","WeakMap","Set","WeakSet"],function(t){I["is"+t]=function(e){return u.call(e)==="[object "+t+"]"}}),I.isArguments(arguments)||(I.isArguments=function(t){return b(t,"callee")});var Y=i.document&&i.document.childNodes;"object"!=typeof Int8Array&&"function"!=typeof Y&&(I.isFunction=function(t){return"function"==typeof t||!1}),I.isFinite=function(t){return!I.isSymbol(t)&&isFinite(t)&&!isNaN(parseFloat(t))},I.isNaN=function(t){return I.isNumber(t)&&isNaN(t)},I.isBoolean=function(t){return!0===t||!1===t||"[object Boolean]"===u.call(t)},I.isNull=function(t){return null===t},I.isUndefined=function(t){return void 0===t},I.has=function(t,e){if(!I.isArray(e))return b(t,e);for(var n=e.length,r=0;r<n;r++){var i=e[r];if(null==t||!g.call(t,i))return!1;t=t[i]}return!!n},I.noConflict=function(){return i._=o,this},I.identity=function(t){return t},I.constant=function(t){return function(){return t}},I.noop=function(){},I.property=function(t){return I.isArray(t)?function(e){return Q(e,t)}:B(t)},I.propertyOf=function(t){return null==t?function(){}:function(e){return I.isArray(e)?Q(t,e):t[e]}},I.matcher=I.matches=function(t){return t=I.extendOwn({},t),function(e){return I.isMatch(e,t)}},I.times=function(t,e,n){var r=Array(Math.max(0,t));e=m(e,n,1);for(var i=0;i<t;i++)r[i]=e(i);return r},I.random=function(t,e){return null==e&&(e=t,t=0),t+Math.floor(Math.random()*(e-t+1))},I.now=Date.now||function(){return(new Date).getTime()};var P={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#x27;","`":"&#x60;"},K=I.invert(P),z=function(t){var e=function(e){return t[e]},n="(?:"+I.keys(t).join("|")+")",r=RegExp(n),i=RegExp(n,"g");return function(t){return t=null==t?"":""+t,r.test(t)?t.replace(i,e):t}};I.escape=z(P),I.unescape=z(K),I.result=function(t,e,n){I.isArray(e)||(e=[e]);var r=e.length;if(!r)return I.isFunction(n)?n.call(t):n;for(var i=0;i<r;i++){var o=null==t?void 0:t[e[i]];void 0===o&&(o=n,i=r),t=I.isFunction(o)?o.call(t):o}return t};var J=0;I.uniqueId=function(t){var e=++J+"";return t?t+e:e},I.templateSettings={evaluate:/<%([\s\S]+?)%>/g,interpolate:/<%=([\s\S]+?)%>/g,escape:/<%-([\s\S]+?)%>/g};var q=/(.)^/,V={"'":"'","\\":"\\","\r":"r","\n":"n","\u2028":"u2028","\u2029":"u2029"},W=/\\|'|\r|\n|\u2028|\u2029/g,X=function(t){return"\\"+V[t]};I.template=function(t,e,n){!e&&n&&(e=n),e=I.defaults({},e,I.templateSettings);var r,i=RegExp([(e.escape||q).source,(e.interpolate||q).source,(e.evaluate||q).source].join("|")+"|$","g"),o=0,a="__p+='";t.replace(i,function(e,n,r,i,s){return a+=t.slice(o,s).replace(W,X),o=s+e.length,n?a+="'+\n((__t=("+n+"))==null?'':_.escape(__t))+\n'":r?a+="'+\n((__t=("+r+"))==null?'':__t)+\n'":i&&(a+="';\n"+i+"\n__p+='"),e}),a+="';\n",e.variable||(a="with(obj||{}){\n"+a+"}\n"),a="var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};\n"+a+"return __p;\n";try{r=new Function(e.variable||"obj","_",a)}catch(t){throw t.source=a,t}var s=function(t){return r.call(this,t,I)},A=e.variable||"obj";return s.source="function("+A+"){\n"+a+"}",s},I.chain=function(t){var e=I(t);return e._chain=!0,e};var Z=function(t,e){return t._chain?I(e).chain():e};I.mixin=function(t){return I.each(I.functions(t),function(e){var n=I[e]=t[e];I.prototype[e]=function(){var t=[this._wrapped];return l.apply(t,arguments),Z(this,n.apply(I,t))}}),I},I.mixin(I),I.each(["pop","push","reverse","shift","sort","splice","unshift"],function(t){var e=a[t];I.prototype[t]=function(){var n=this._wrapped;return e.apply(n,arguments),"shift"!==t&&"splice"!==t||0!==n.length||delete n[0],Z(this,n)}}),I.each(["concat","join","slice"],function(t){var e=a[t];I.prototype[t]=function(){return Z(this,e.apply(this._wrapped,arguments))}}),I.prototype.value=function(){return this._wrapped},I.prototype.valueOf=I.prototype.toJSON=I.prototype.value,I.prototype.toString=function(){return String(this._wrapped)},void 0===(r=function(){return I}.apply(e,[]))||(n.exports=r)}()}).call(this,n(37),n(98)(t))},function(t,e,n){"use strict";n.d(e,"d",function(){return i}),n.d(e,"c",function(){return o}),n.d(e,"a",function(){return a}),n.d(e,"b",function(){return s}),n.d(e,"e",function(){return A}),n.d(e,"g",function(){return l}),n.d(e,"f",function(){return c}),n.d(e,"l",function(){return u}),n.d(e,"n",function(){return g}),n.d(e,"m",function(){return h}),n.d(e,"t",function(){return d}),n.d(e,"r",function(){return f}),n.d(e,"p",function(){return p}),n.d(e,"s",function(){return I}),n.d(e,"q",function(){return C}),n.d(e,"u",function(){return m}),n.d(e,"v",function(){return y}),n.d(e,"x",function(){return E}),n.d(e,"y",function(){return v}),n.d(e,"w",function(){return B}),n.d(e,"A",function(){return b}),n.d(e,"z",function(){return Q}),n.d(e,"F",function(){return w}),n.d(e,"B",function(){return x}),n.d(e,"C",function(){return _}),n.d(e,"D",function(){return D}),n.d(e,"E",function(){return M}),n.d(e,"G",function(){return k}),n.d(e,"H",function(){return S}),n.d(e,"I",function(){return L}),n.d(e,"L",function(){return R}),n.d(e,"K",function(){return N}),n.d(e,"J",function(){return F}),n.d(e,"o",function(){return T}),n.d(e,"M",function(){return U}),n.d(e,"N",function(){return O}),n.d(e,"O",function(){return G}),n.d(e,"i",function(){return H}),n.d(e,"h",function(){return j}),n.d(e,"k",function(){return Y}),n.d(e,"j",function(){return P});var r="[Blueprint]",i=r+" clamp: max cannot be less than min",o=r+" <Alert> cancelButtonText and onCancel should be set together.",a=r+" <Alert> canEscapeKeyCancel enabled without onCancel or onClose handler.",s=r+" <Alert> canOutsideClickCancel enbaled without onCancel or onClose handler.",A=r+" <CollapsibleList> children must be <MenuItem>s",l=r+" @ContextMenuTarget-decorated class should implement renderContextMenu.",c=r+' "@ContextMenuTarget-decorated components must return a single JSX.Element or an empty render.',u=r+" <Hotkeys> only accepts <Hotkey> children.",g=r+" @HotkeysTarget-decorated class should implement renderHotkeys.",h=r+' "@HotkeysTarget-decorated components must return a single JSX.Element or an empty render.',d=r+" <NumericInput> requires min to be no greater than max if both are defined.",f=r+" <NumericInput> requires minorStepSize to be no greater than stepSize.",p=r+" <NumericInput> requires stepSize to be no greater than majorStepSize.",I=r+" <NumericInput> requires minorStepSize to be strictly greater than zero.",C=r+" <NumericInput> requires majorStepSize to be strictly greater than zero.",m=r+" <NumericInput> requires stepSize to be strictly greater than zero.",y=r+" <NumericInput> requires stepSize to be defined.",E=r+" <PanelStack> requires exactly one of initialPanel and stack prop",v=r+" <PanelStack> requires at least one panel in the stack",B=r+" <OverflowList> does not support changing observeParents after mounting.",b=r+" <Popover> requires target prop or at least one child element.",Q=r+" <Popover hasBackdrop={true}> requires interactionKind={PopoverInteractionKind.CLICK}.",w=r+" <Popover> supports one or two children; additional children are ignored. First child is the target, second child is the content. You may instead supply these two as props.",x=r+" <Popover> with two children ignores content prop; use either prop or children.",_=r+" <Popover> with children ignores target prop; use either prop or children.",D=r+" Disabling <Popover> with empty/whitespace content...",M=r+" <Popover usePortal={false}> ignores hasBackdrop",k=r+" <Popover> onInteraction is ignored when uncontrolled.",S=r+" <Portal> context blueprintPortalClassName must be string",L=r+" <RadioGroup> children and options prop are mutually exclusive, with options taking priority.",R=r+" <Slider> stepSize must be greater than zero.",N=r+" <Slider> labelStepSize must be greater than zero.",F=r+" <RangeSlider> value prop must be an array of two non-null numbers.",T=r+" <MultiSlider> children must be <SliderHandle>s or <SliderTrackStop>s",U=r+" <Spinner> Classes.SMALL/LARGE are ignored if size prop is set.",O=r+" Toaster.create() is not supported inside React lifecycle methods in React 16. See usage example on the docs site.",G=r+" Toaster.create() ignores inline prop as it always creates a new element.",H=r+" <Dialog> iconName is ignored if title is omitted.",j=r+" <Dialog> isCloseButtonShown prop is ignored if title is omitted.",Y=r+" <Drawer> vertical is ignored if position is defined",P=r+" <Drawer> all angle positions are casted into pure position (TOP, BOTTOM, LEFT or RIGHT)"},function(t,e,n){"use strict";n.d(e,"a",function(){return a});var r=n(62),i=n(132),o=n(21);function a(t,e,n,a,s){if(void 0===s&&(s=new r.a(t,n,a)),!s.closed)return e instanceof o.a?e.subscribe(s):Object(i.a)(e)(s)}},function(t,e,n){"use strict";(function(t,r){var i,o=this&&this.__extends||(i=function(t,e){return(i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}i(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var a=n(5),s=n(1030),A=function(){function t(t){this.type=t}return Object.defineProperty(t.prototype,"isConflatable",{get:function(){return!1},enumerable:!0,configurable:!0}),t.prototype.conflate=function(t){return!1},t}();e.Message=A;var l=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return o(e,t),Object.defineProperty(e.prototype,"isConflatable",{get:function(){return!0},enumerable:!0,configurable:!0}),e.prototype.conflate=function(t){return!0},e}(A);e.ConflatableMessage=l,function(e){function n(t,e){var n=o.get(t);n&&0!==n.length?a.every(a.retro(n),function(n){return!n||function(t,e,n){var r=!0;try{r="function"==typeof t?t(e,n):t.messageHook(e,n)}catch(t){l(t)}return r}(n,t,e)})&&d(t,e):d(t,e)}e.sendMessage=n,e.postMessage=function(t,e){e.isConflatable&&a.some(i,function(n){return n.handler===t&&!!n.msg&&n.msg.type===e.type&&!!n.msg.isConflatable&&n.msg.conflate(e)})||f(t,e)},e.installMessageHook=function(t,e){var n=o.get(t);n&&-1!==n.indexOf(e)||(n?n.push(e):o.set(t,[e]))},e.removeMessageHook=function(t,e){var n=o.get(t);if(n){var r=n.indexOf(e);-1!==r&&(n[r]=null,I(n))}},e.clearData=function(t){var e=o.get(t);e&&e.length>0&&(a.ArrayExt.fill(e,null),I(e)),a.each(i,function(e){e.handler===t&&(e.handler=null,e.msg=null)})},e.flush=function(){u||0===c||(h(c),u=!0,p(),u=!1)},e.getExceptionHandler=function(){return l},e.setExceptionHandler=function(t){var e=l;return l=t,e};var i=new s.LinkedList,o=new WeakMap,A=new Set,l=function(t){console.error(t)},c=0,u=!1,g="function"==typeof requestAnimationFrame?requestAnimationFrame:t,h="function"==typeof cancelAnimationFrame?cancelAnimationFrame:r;function d(t,e){try{t.processMessage(e)}catch(t){l(t)}}function f(t,e){i.addLast({handler:t,msg:e}),0===c&&(c=g(p))}function p(){if(c=0,!i.isEmpty){var t={handler:null,msg:null};for(i.addLast(t);;){var e=i.removeFirst();if(e===t)return;e.handler&&e.msg&&n(e.handler,e.msg)}}}function I(t){0===A.size&&g(C),A.add(t)}function C(){A.forEach(m),A.clear()}function m(t){a.ArrayExt.removeAllWhere(t,y)}function y(t){return null===t}}(e.MessageLoop||(e.MessageLoop={}))}).call(this,n(118).setImmediate,n(118).clearImmediate)},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=n(2),i=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.__extends(e,t),e.prototype.notifyNext=function(t,e,n,r,i){this.destination.next(e)},e.prototype.notifyError=function(t,e){this.destination.error(t)},e.prototype.notifyComplete=function(t){this.destination.complete()},e}(n(11).a)},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(5),i=function(t){this.type="text",this.content=t};e.VirtualText=i;var o,a=function(t,e,n){this.type="element",this.tag=t,this.attrs=e,this.children=n};function s(t){for(var e={},n=[],r=1,o=arguments.length;r<o;++r){var s=arguments[r];"string"==typeof s?n.push(new i(s)):s instanceof i?n.push(s):s instanceof a?n.push(s):s instanceof Array?A(n,s):1===r&&s&&"object"==typeof s&&(e=s)}return new a(t,e,n);function A(t,e){for(var n=0,r=e;n<r.length;n++){var o=r[n];"string"==typeof o?t.push(new i(o)):o instanceof i?t.push(o):o instanceof a&&t.push(o)}}}e.VirtualElement=a,e.h=s,function(t){t.a=t.bind(void 0,"a"),t.abbr=t.bind(void 0,"abbr"),t.address=t.bind(void 0,"address"),t.area=t.bind(void 0,"area"),t.article=t.bind(void 0,"article"),t.aside=t.bind(void 0,"aside"),t.audio=t.bind(void 0,"audio"),t.b=t.bind(void 0,"b"),t.bdi=t.bind(void 0,"bdi"),t.bdo=t.bind(void 0,"bdo"),t.blockquote=t.bind(void 0,"blockquote"),t.br=t.bind(void 0,"br"),t.button=t.bind(void 0,"button"),t.canvas=t.bind(void 0,"canvas"),t.caption=t.bind(void 0,"caption"),t.cite=t.bind(void 0,"cite"),t.code=t.bind(void 0,"code"),t.col=t.bind(void 0,"col"),t.colgroup=t.bind(void 0,"colgroup"),t.data=t.bind(void 0,"data"),t.datalist=t.bind(void 0,"datalist"),t.dd=t.bind(void 0,"dd"),t.del=t.bind(void 0,"del"),t.dfn=t.bind(void 0,"dfn"),t.div=t.bind(void 0,"div"),t.dl=t.bind(void 0,"dl"),t.dt=t.bind(void 0,"dt"),t.em=t.bind(void 0,"em"),t.embed=t.bind(void 0,"embed"),t.fieldset=t.bind(void 0,"fieldset"),t.figcaption=t.bind(void 0,"figcaption"),t.figure=t.bind(void 0,"figure"),t.footer=t.bind(void 0,"footer"),t.form=t.bind(void 0,"form"),t.h1=t.bind(void 0,"h1"),t.h2=t.bind(void 0,"h2"),t.h3=t.bind(void 0,"h3"),t.h4=t.bind(void 0,"h4"),t.h5=t.bind(void 0,"h5"),t.h6=t.bind(void 0,"h6"),t.header=t.bind(void 0,"header"),t.hr=t.bind(void 0,"hr"),t.i=t.bind(void 0,"i"),t.iframe=t.bind(void 0,"iframe"),t.img=t.bind(void 0,"img"),t.input=t.bind(void 0,"input"),t.ins=t.bind(void 0,"ins"),t.kbd=t.bind(void 0,"kbd"),t.label=t.bind(void 0,"label"),t.legend=t.bind(void 0,"legend"),t.li=t.bind(void 0,"li"),t.main=t.bind(void 0,"main"),t.map=t.bind(void 0,"map"),t.mark=t.bind(void 0,"mark"),t.meter=t.bind(void 0,"meter"),t.nav=t.bind(void 0,"nav"),t.noscript=t.bind(void 0,"noscript"),t.object=t.bind(void 0,"object"),t.ol=t.bind(void 0,"ol"),t.optgroup=t.bind(void 0,"optgroup"),t.option=t.bind(void 0,"option"),t.output=t.bind(void 0,"output"),t.p=t.bind(void 0,"p"),t.param=t.bind(void 0,"param"),t.pre=t.bind(void 0,"pre"),t.progress=t.bind(void 0,"progress"),t.q=t.bind(void 0,"q"),t.rp=t.bind(void 0,"rp"),t.rt=t.bind(void 0,"rt"),t.ruby=t.bind(void 0,"ruby"),t.s=t.bind(void 0,"s"),t.samp=t.bind(void 0,"samp"),t.section=t.bind(void 0,"section"),t.select=t.bind(void 0,"select"),t.small=t.bind(void 0,"small"),t.source=t.bind(void 0,"source"),t.span=t.bind(void 0,"span"),t.strong=t.bind(void 0,"strong"),t.sub=t.bind(void 0,"sub"),t.summary=t.bind(void 0,"summary"),t.sup=t.bind(void 0,"sup"),t.table=t.bind(void 0,"table"),t.tbody=t.bind(void 0,"tbody"),t.td=t.bind(void 0,"td"),t.textarea=t.bind(void 0,"textarea"),t.tfoot=t.bind(void 0,"tfoot"),t.th=t.bind(void 0,"th"),t.thead=t.bind(void 0,"thead"),t.time=t.bind(void 0,"time"),t.title=t.bind(void 0,"title"),t.tr=t.bind(void 0,"tr"),t.track=t.bind(void 0,"track"),t.u=t.bind(void 0,"u"),t.ul=t.bind(void 0,"ul"),t.var_=t.bind(void 0,"var"),t.video=t.bind(void 0,"video"),t.wbr=t.bind(void 0,"wbr")}(s=e.h||(e.h={})),function(t){t.realize=function(t){return o.createDOMNode(t)},t.render=function(t,e){var n=o.hostMap.get(e)||[],r=o.asContentArray(t);o.hostMap.set(e,r),o.updateContent(e,n,r)}}(e.VirtualDOM||(e.VirtualDOM={})),function(t){function e(t){if("text"===t.type)return document.createTextNode(t.content);var r=document.createElement(t.tag);!function(t,e){for(var r in e)r in n||("on"===r.substr(0,2)?t[r]=e[r]:t.setAttribute(r,e[r]));void 0!==e.className&&t.setAttribute("class",e.className);void 0!==e.htmlFor&&t.setAttribute("for",e.htmlFor);e.dataset&&function(t,e){for(var n in e)t.setAttribute("data-"+n,e[n])}(t,e.dataset);e.style&&function(t,e){var n,r=t.style;for(n in e)r[n]=e[n]}(t,e.style)}(r,t.attrs);for(var i=0,o=t.children.length;i<o;++i)r.appendChild(e(t.children[i]));return r}t.hostMap=new WeakMap,t.asContentArray=function(t){return t?t instanceof Array?t:[t]:[]},t.createDOMNode=e,t.updateContent=function t(n,o,a){if(o!==a){for(var s=function(t,e){for(var n=t.firstChild,r=Object.create(null),i=0,o=e;i<o.length;i++){var a=o[i];"element"===a.type&&a.attrs.key&&(r[a.attrs.key]={vNode:a,element:n}),n=n.nextSibling}return r}(n,o),A=o.slice(),l=n.firstChild,c=a.length,u=0;u<c;++u)if(u>=A.length)n.appendChild(e(a[u]));else{var g=A[u],h=a[u];if(g!==h)if("text"!==g.type||"text"!==h.type)if("text"!==g.type&&"text"!==h.type){var d=h.attrs.key;if(d&&d in s){var f=s[d];f.vNode!==g&&(r.ArrayExt.move(A,A.indexOf(f.vNode,u+1),u),n.insertBefore(f.element,l),g=f.vNode,l=f.element)}if(g!==h){var p=g.attrs.key;p&&p!==d?(r.ArrayExt.insert(A,u,h),n.insertBefore(e(h),l)):g.tag===h.tag?(i(l,g.attrs,h.attrs),t(l,g.children,h.children),l=l.nextSibling):(r.ArrayExt.insert(A,u,h),n.insertBefore(e(h),l))}else l=l.nextSibling}else r.ArrayExt.insert(A,u,h),n.insertBefore(e(h),l);else l.textContent=h.content,l=l.nextSibling;else l=l.nextSibling}for(u=A.length-c;u>0;--u)n.removeChild(n.lastChild)}};var n={key:!0,className:!0,htmlFor:!0,dataset:!0,style:!0};function i(t,e,r){if(e!==r){var i;for(i in e)i in n||i in r||("on"===i.substr(0,2)?t[i]=null:t.removeAttribute(i));for(i in r)i in n||e[i]===r[i]||("on"===i.substr(0,2)?t[i]=r[i]:t.setAttribute(i,r[i]));e.className!==r.className&&(void 0!==r.className?t.setAttribute("class",r.className):t.removeAttribute("class")),e.htmlFor!==r.htmlFor&&(void 0!==r.htmlFor?t.setAttribute("for",r.htmlFor):t.removeAttribute("for")),e.dataset!==r.dataset&&function(t,e,n){for(var r in e)r in n||t.removeAttribute("data-"+r);for(var i in n)e[i]!==n[i]&&t.setAttribute("data-"+i,n[i])}(t,e.dataset||{},r.dataset||{}),e.style!==r.style&&function(t,e,n){var r,i=t.style;for(r in e)r in n||(i[r]="");for(r in n)e[r]!==n[r]&&(i[r]=n[r])}(t,e.style||{},r.style||{})}}}(o||(o={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(630);e.AsyncTools=r.AsyncTools;var i=n(1166);e.FrameTools=i.FrameTools;var o=n(1167);e.IterTools=o.IterTools;var a=n(1168);e.MathTools=a.MathTools;var s=n(1169);e.StartTiming=s.StartTiming,e.StartTimingAsync=s.StartTimingAsync,e.StartTimingSync=s.StartTimingSync,e.StartTimingWithContext=s.StartTimingWithContext;var A=n(1170);e.CancelToken=A.CancelToken,e.CancelError=A.CancelError;var l=n(1171);e.Color=l.Color;var c=n(311);e.Converters=c.Converters,e.PartialConverter=c.PartialConverter,e.Converter=c.Converter;var u=n(1174);e.serialize=u.serialize,e.deserialize=u.deserialize;var g=n(220);e.Types=g.Types,e.Type=g.Type;var h=n(1175);e.Cache=h.Cache},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(1066);e.TypeStyle=r.TypeStyle;var i=n(1068);e.types=i;var o=n(581);e.extend=o.extend,e.classes=o.classes,e.media=o.media;var a=new r.TypeStyle({autoGenerateTag:!0});e.setStylesTarget=a.setStylesTarget,e.cssRaw=a.cssRaw,e.cssRule=a.cssRule,e.forceRenderStyles=a.forceRenderStyles,e.fontFace=a.fontFace,e.getStyles=a.getStyles,e.keyframes=a.keyframes,e.reinit=a.reinit,e.style=a.style,e.stylesheet=a.stylesheet,e.createTypeStyle=function(t){var e=new r.TypeStyle({autoGenerateTag:!1});return t&&e.setStylesTarget(t),e}},function(t,e){"function"==typeof Object.create?t.exports=function(t,e){e&&(t.super_=e,t.prototype=Object.create(e.prototype,{constructor:{value:t,enumerable:!1,writable:!0,configurable:!0}}))}:t.exports=function(t,e){if(e){t.super_=e;var n=function(){};n.prototype=e.prototype,t.prototype=new n,t.prototype.constructor=t}}},function(t,e,n){"use strict";var r={MAC_ENTER:3,BACKSPACE:8,TAB:9,NUM_CENTER:12,ENTER:13,SHIFT:16,CTRL:17,ALT:18,PAUSE:19,CAPS_LOCK:20,ESC:27,SPACE:32,PAGE_UP:33,PAGE_DOWN:34,END:35,HOME:36,LEFT:37,UP:38,RIGHT:39,DOWN:40,PRINT_SCREEN:44,INSERT:45,DELETE:46,ZERO:48,ONE:49,TWO:50,THREE:51,FOUR:52,FIVE:53,SIX:54,SEVEN:55,EIGHT:56,NINE:57,QUESTION_MARK:63,A:65,B:66,C:67,D:68,E:69,F:70,G:71,H:72,I:73,J:74,K:75,L:76,M:77,N:78,O:79,P:80,Q:81,R:82,S:83,T:84,U:85,V:86,W:87,X:88,Y:89,Z:90,META:91,WIN_KEY_RIGHT:92,CONTEXT_MENU:93,NUM_ZERO:96,NUM_ONE:97,NUM_TWO:98,NUM_THREE:99,NUM_FOUR:100,NUM_FIVE:101,NUM_SIX:102,NUM_SEVEN:103,NUM_EIGHT:104,NUM_NINE:105,NUM_MULTIPLY:106,NUM_PLUS:107,NUM_MINUS:109,NUM_PERIOD:110,NUM_DIVISION:111,F1:112,F2:113,F3:114,F4:115,F5:116,F6:117,F7:118,F8:119,F9:120,F10:121,F11:122,F12:123,NUMLOCK:144,SEMICOLON:186,DASH:189,EQUALS:187,COMMA:188,PERIOD:190,SLASH:191,APOSTROPHE:192,SINGLE_QUOTE:222,OPEN_SQUARE_BRACKET:219,BACKSLASH:220,CLOSE_SQUARE_BRACKET:221,WIN_KEY:224,MAC_FF_META:224,WIN_IME:229,isTextModifyingKeyEvent:function(t){var e=t.keyCode;if(t.altKey&&!t.ctrlKey||t.metaKey||e>=r.F1&&e<=r.F12)return!1;switch(e){case r.ALT:case r.CAPS_LOCK:case r.CONTEXT_MENU:case r.CTRL:case r.DOWN:case r.END:case r.ESC:case r.HOME:case r.INSERT:case r.LEFT:case r.MAC_FF_META:case r.META:case r.NUMLOCK:case r.NUM_CENTER:case r.PAGE_DOWN:case r.PAGE_UP:case r.PAUSE:case r.PRINT_SCREEN:case r.RIGHT:case r.SHIFT:case r.UP:case r.WIN_KEY:case r.WIN_KEY_RIGHT:return!1;default:return!0}},isCharacterKey:function(t){if(t>=r.ZERO&&t<=r.NINE)return!0;if(t>=r.NUM_ZERO&&t<=r.NUM_MULTIPLY)return!0;if(t>=r.A&&t<=r.Z)return!0;if(-1!==window.navigation.userAgent.indexOf("WebKit")&&0===t)return!0;switch(t){case r.SPACE:case r.QUESTION_MARK:case r.NUM_PLUS:case r.NUM_MINUS:case r.NUM_PERIOD:case r.NUM_DIVISION:case r.SEMICOLON:case r.DASH:case r.EQUALS:case r.COMMA:case r.PERIOD:case r.SLASH:case r.APOSTROPHE:case r.SINGLE_QUOTE:case r.OPEN_SQUARE_BRACKET:case r.BACKSLASH:case r.CLOSE_SQUARE_BRACKET:return!0;default:return!1}}};e.a=r},function(t,e,n){"use strict";n.d(e,"a",function(){return s});var r=n(38),i=n(157),o=n(86),a=n(131),s=function(){function t(t){this.closed=!1,this._parentOrParents=null,this._subscriptions=null,t&&(this._unsubscribe=t)}var e;return t.prototype.unsubscribe=function(){var e;if(!this.closed){var n=this._parentOrParents,s=this._unsubscribe,l=this._subscriptions;if(this.closed=!0,this._parentOrParents=null,this._subscriptions=null,n instanceof t)n.remove(this);else if(null!==n)for(var c=0;c<n.length;++c){n[c].remove(this)}if(Object(o.a)(s))try{s.call(this)}catch(t){e=t instanceof a.a?A(t.errors):[t]}if(Object(r.a)(l)){c=-1;for(var u=l.length;++c<u;){var g=l[c];if(Object(i.a)(g))try{g.unsubscribe()}catch(t){e=e||[],t instanceof a.a?e=e.concat(A(t.errors)):e.push(t)}}}if(e)throw new a.a(e)}},t.prototype.add=function(e){var n=e;if(!e)return t.EMPTY;switch(typeof e){case"function":n=new t(e);case"object":if(n===this||n.closed||"function"!=typeof n.unsubscribe)return n;if(this.closed)return n.unsubscribe(),n;if(!(n instanceof t)){var r=n;(n=new t)._subscriptions=[r]}break;default:throw new Error("unrecognized teardown "+e+" added to Subscription.")}var i=n._parentOrParents;if(null===i)n._parentOrParents=this;else if(i instanceof t){if(i===this)return n;n._parentOrParents=[i,this]}else{if(-1!==i.indexOf(this))return n;i.push(this)}var o=this._subscriptions;return null===o?this._subscriptions=[n]:o.push(n),n},t.prototype.remove=function(t){var e=this._subscriptions;if(e){var n=e.indexOf(t);-1!==n&&e.splice(n,1)}},t.EMPTY=((e=new t).closed=!0,e),t}();function A(t){return t.reduce(function(t,e){return t.concat(e instanceof a.a?e.errors:e)},[])}},function(t,e){t.exports=function(t){if(void 0===t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return t}},function(t,e,n){"use strict";n.d(e,"a",function(){return r}),n.d(e,"c",function(){return i}),n.d(e,"d",function(){return o}),n.d(e,"b",function(){return a});var r={BOTTOM:"bottom",BOTTOM_LEFT:"bottom-left",BOTTOM_RIGHT:"bottom-right",LEFT:"left",LEFT_BOTTOM:"left-bottom",LEFT_TOP:"left-top",RIGHT:"right",RIGHT_BOTTOM:"right-bottom",RIGHT_TOP:"right-top",TOP:"top",TOP_LEFT:"top-left",TOP_RIGHT:"top-right"};function i(t){return t===r.TOP||t===r.TOP_LEFT||t===r.TOP_RIGHT||t===r.BOTTOM||t===r.BOTTOM_LEFT||t===r.BOTTOM_RIGHT}function o(t){return t===r.LEFT||t===r.LEFT_TOP||t===r.LEFT_BOTTOM||t===r.RIGHT||t===r.RIGHT_TOP||t===r.RIGHT_BOTTOM}function a(t){return t===r.TOP||t===r.TOP_LEFT||t===r.TOP_RIGHT?r.TOP:t===r.BOTTOM||t===r.BOTTOM_LEFT||t===r.BOTTOM_RIGHT?r.BOTTOM:t===r.LEFT||t===r.LEFT_TOP||t===r.LEFT_BOTTOM?r.LEFT:r.RIGHT}},function(t,e,n){var r=n(42),i=r.Buffer;function o(t,e){for(var n in t)e[n]=t[n]}function a(t,e,n){return i(t,e,n)}i.from&&i.alloc&&i.allocUnsafe&&i.allocUnsafeSlow?t.exports=r:(o(r,e),e.Buffer=a),a.prototype=Object.create(i.prototype),o(i,a),a.from=function(t,e,n){if("number"==typeof t)throw new TypeError("Argument must not be a number");return i(t,e,n)},a.alloc=function(t,e,n){if("number"!=typeof t)throw new TypeError("Argument must be a number");var r=i(t);return void 0!==e?"string"==typeof n?r.fill(e,n):r.fill(e):r.fill(0),r},a.allocUnsafe=function(t){if("number"!=typeof t)throw new TypeError("Argument must be a number");return i(t)},a.allocUnsafeSlow=function(t){if("number"!=typeof t)throw new TypeError("Argument must be a number");return r.SlowBuffer(t)}},function(t,e,n){"use strict";n.d(e,"b",function(){return c}),n.d(e,"a",function(){return u});var r=n(2),i=n(21),o=n(11),a=n(32),s=n(81),A=n(258),l=n(130),c=function(t){function e(e){var n=t.call(this,e)||this;return n.destination=e,n}return r.__extends(e,t),e}(o.a),u=function(t){function e(){var e=t.call(this)||this;return e.observers=[],e.closed=!1,e.isStopped=!1,e.hasError=!1,e.thrownError=null,e}return r.__extends(e,t),e.prototype[l.a]=function(){return new c(this)},e.prototype.lift=function(t){var e=new g(this,this);return e.operator=t,e},e.prototype.next=function(t){if(this.closed)throw new s.a;if(!this.isStopped)for(var e=this.observers,n=e.length,r=e.slice(),i=0;i<n;i++)r[i].next(t)},e.prototype.error=function(t){if(this.closed)throw new s.a;this.hasError=!0,this.thrownError=t,this.isStopped=!0;for(var e=this.observers,n=e.length,r=e.slice(),i=0;i<n;i++)r[i].error(t);this.observers.length=0},e.prototype.complete=function(){if(this.closed)throw new s.a;this.isStopped=!0;for(var t=this.observers,e=t.length,n=t.slice(),r=0;r<e;r++)n[r].complete();this.observers.length=0},e.prototype.unsubscribe=function(){this.isStopped=!0,this.closed=!0,this.observers=null},e.prototype._trySubscribe=function(e){if(this.closed)throw new s.a;return t.prototype._trySubscribe.call(this,e)},e.prototype._subscribe=function(t){if(this.closed)throw new s.a;return this.hasError?(t.error(this.thrownError),a.a.EMPTY):this.isStopped?(t.complete(),a.a.EMPTY):(this.observers.push(t),new A.a(this,t))},e.prototype.asObservable=function(){var t=new i.a;return t.source=this,t},e.create=function(t,e){return new g(t,e)},e}(i.a),g=function(t){function e(e,n){var r=t.call(this)||this;return r.destination=e,r.source=n,r}return r.__extends(e,t),e.prototype.next=function(t){var e=this.destination;e&&e.next&&e.next(t)},e.prototype.error=function(t){var e=this.destination;e&&e.error&&this.destination.error(t)},e.prototype.complete=function(){var t=this.destination;t&&t.complete&&this.destination.complete()},e.prototype._subscribe=function(t){return this.source?this.source.subscribe(t):a.a.EMPTY},e}(u)},function(t,e){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(t){"object"==typeof window&&(n=window)}t.exports=n},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r=Array.isArray||function(t){return t&&"number"==typeof t.length}},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=n(93),i=new(n(91).a)(r.a)},function(t,e,n){"use strict";n.d(e,"b",function(){return u}),n.d(e,"c",function(){return g}),n.d(e,"a",function(){return h});var r,i=n(5),o=n(9),a=n(25),s=n(6),A=n(115),l=n(76),c=n(243);function u(t={}){return new h(t).launch()}function g(t,e,n=[h.okButton({label:"Dismiss"})]){console.warn("Showing error:",e);let i="string"==typeof e?e:e.message,o=t+"----"+i,a=r.errorMessagePromiseCache.get(o);if(a)return a;{let e=u({title:t,body:i,buttons:n}).then(()=>{r.errorMessagePromiseCache.delete(o)},t=>{throw r.errorMessagePromiseCache.delete(o),t});return r.errorMessagePromiseCache.set(o,e),e}}class h extends s.Widget{constructor(t={}){super(),this._focusNodeSelector="",this.addClass("jp-Dialog");let e=r.handleOptions(t),n=e.renderer;this._host=e.host,this._defaultButton=e.defaultButton,this._buttons=e.buttons,this._buttonNodes=Object(i.toArray)(Object(i.map)(this._buttons,t=>n.createButtonNode(t)));let o=this.layout=new s.PanelLayout,a=new s.Panel;a.addClass("jp-Dialog-content"),o.addWidget(a),this._body=e.body;let A=n.createHeader(e.title),l=n.createBody(e.body),c=n.createFooter(this._buttonNodes);a.addWidget(A),a.addWidget(l),a.addWidget(c),this._primary=this._buttonNodes[this._defaultButton],this._focusNodeSelector=t.focusNodeSelector,h.tracker.add(this)}dispose(){const t=this._promise;t&&(this._promise=null,t.reject(void 0),i.ArrayExt.removeFirstOf(r.launchQueue,t.promise)),super.dispose()}launch(){if(this._promise)return this._promise.promise;const t=this._promise=new o.PromiseDelegate;let e=Promise.all(r.launchQueue);return r.launchQueue.push(this._promise.promise),e.then(()=>(s.Widget.attach(this,this._host),t.promise))}resolve(t){this._promise&&(void 0===t&&(t=this._defaultButton),this._resolve(this._buttons[t]))}reject(){this._promise&&this._resolve(h.cancelButton())}handleEvent(t){switch(t.type){case"keydown":this._evtKeydown(t);break;case"click":this._evtClick(t);break;case"focus":this._evtFocus(t);break;case"contextmenu":t.preventDefault(),t.stopPropagation()}}onAfterAttach(t){let e=this.node;if(e.addEventListener("keydown",this,!0),e.addEventListener("contextmenu",this,!0),e.addEventListener("click",this,!0),document.addEventListener("focus",this,!0),this._first=r.findFirstFocusable(this.node),this._original=document.activeElement,this._focusNodeSelector){let t=this.node.querySelector(".jp-Dialog-body").querySelector(this._focusNodeSelector);t&&(this._primary=t)}this._primary.focus()}onAfterDetach(t){let e=this.node;e.removeEventListener("keydown",this,!0),e.removeEventListener("contextmenu",this,!0),e.removeEventListener("click",this,!0),document.removeEventListener("focus",this,!0),this._original.focus()}onCloseRequest(t){this._promise&&this.reject(),super.onCloseRequest(t)}_evtClick(t){if(!this.node.getElementsByClassName("jp-Dialog-content")[0].contains(t.target))return t.stopPropagation(),void t.preventDefault();for(let e of this._buttonNodes)if(e.contains(t.target)){let t=this._buttonNodes.indexOf(e);this.resolve(t)}}_evtKeydown(t){switch(t.keyCode){case 27:t.stopPropagation(),t.preventDefault(),this.reject();break;case 9:let e=this._buttonNodes[this._buttons.length-1];document.activeElement!==e||t.shiftKey||(t.stopPropagation(),t.preventDefault(),this._first.focus());break;case 13:t.stopPropagation(),t.preventDefault(),this.resolve()}}_evtFocus(t){let e=t.target;this.node.contains(e)||(t.stopPropagation(),this._buttonNodes[this._defaultButton].focus())}_resolve(t){const e=this._promise;if(!e)return void this.dispose();this._promise=null,i.ArrayExt.removeFirstOf(r.launchQueue,e.promise);let n=this._body,o=null;t.accept&&n instanceof s.Widget&&"function"==typeof n.getValue&&(o=n.getValue()),this.dispose(),e.resolve({button:t,value:o})}}!function(t){function e(t){t.accept=!1!==t.accept;let e=t.accept?"OK":"Cancel";return{label:t.label||e,iconClass:t.iconClass||"",iconLabel:t.iconLabel||"",caption:t.caption||"",className:t.className||"",accept:t.accept,displayType:t.displayType||"default"}}t.createButton=e,t.cancelButton=function(t={}){return t.accept=!1,e(t)},t.okButton=function(t={}){return t.accept=!0,e(t)},t.warnButton=function(t={}){return t.displayType="warn",e(t)},t.flush=function(){t.tracker.forEach(t=>{t.dispose()})};class n{createHeader(t){let e;return"string"==typeof t?(e=new s.Widget({node:document.createElement("span")})).node.textContent=t:e=l.a.create(t),e.addClass("jp-Dialog-header"),A.a.styleNode(e.node),e}createBody(t){let e;return"string"==typeof t?(e=new s.Widget({node:document.createElement("span")})).node.textContent=t:t instanceof s.Widget?e=t:(e=l.a.create(t),a.MessageLoop.sendMessage(e,s.Widget.Msg.UpdateRequest)),e.addClass("jp-Dialog-body"),A.a.styleNode(e.node),e}createFooter(t){let e=new s.Widget;return e.addClass("jp-Dialog-footer"),Object(i.each)(t,t=>{e.node.appendChild(t)}),A.a.styleNode(e.node),e}createButtonNode(t){const e=document.createElement("button");return e.className=this.createItemClass(t),e.appendChild(this.renderIcon(t)),e.appendChild(this.renderLabel(t)),e}createItemClass(t){let e="jp-Dialog-button";t.accept?e+=" jp-mod-accept":e+=" jp-mod-reject","warn"===t.displayType&&(e+=" jp-mod-warn");let n=t.className;return n&&(e+=` ${n}`),e}renderIcon(t){const e=document.createElement("div");return e.className=this.createIconClass(t),e.appendChild(document.createTextNode(t.iconLabel)),e}createIconClass(t){let e="jp-Dialog-buttonIcon",n=t.iconClass;return n?`${e} ${n}`:e}renderLabel(t){const e=document.createElement("div");return e.className="jp-Dialog-buttonLabel",e.title=t.caption,e.appendChild(document.createTextNode(t.label)),e}}t.Renderer=n,t.defaultRenderer=new n,t.tracker=new c.a({namespace:"@jupyterlab/apputils:Dialog"})}(h||(h={})),function(t){t.launchQueue=[],t.errorMessagePromiseCache=new Map,t.handleOptions=function(t={}){let e=t.buttons||[h.cancelButton(),h.okButton()],n=e.length-1;return void 0!==t.defaultButton&&(n=t.defaultButton),{title:t.title||"",body:t.body||"",host:t.host||document.body,buttons:e,defaultButton:n,renderer:t.renderer||h.defaultRenderer,focusNodeSelector:t.focusNodeSelector||""}},t.findFirstFocusable=function(t){let e=["input","select","a[href]","textarea","button","[tabindex]"].join(",");return t.querySelectorAll(e)[0]}}(r||(r={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r=n(2),i=n(11);function o(t,e){return function(n){if("function"!=typeof t)throw new TypeError("argument is not a function. Are you looking for `mapTo()`?");return n.lift(new a(t,e))}}var a=function(){function t(t,e){this.project=t,this.thisArg=e}return t.prototype.call=function(t,e){return e.subscribe(new s(t,this.project,this.thisArg))},t}(),s=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.project=n,i.count=0,i.thisArg=r||i,i}return r.__extends(e,t),e.prototype._next=function(t){var e;try{e=this.project.call(this.thisArg,t,this.count++)}catch(t){return void this.destination.error(t)}this.destination.next(e)},e}(i.a)},function(t,e,n){"use strict";(function(t){
/*!
 * The buffer module from node.js, for the browser.
 *
 * @author   Feross Aboukhadijeh <feross@feross.org> <http://feross.org>
 * @license  MIT
 */
var r=n(191),i=n(769),o=n(770);function a(){return A.TYPED_ARRAY_SUPPORT?2147483647:1073741823}function s(t,e){if(a()<e)throw new RangeError("Invalid typed array length");return A.TYPED_ARRAY_SUPPORT?(t=new Uint8Array(e)).__proto__=A.prototype:(null===t&&(t=new A(e)),t.length=e),t}function A(t,e,n){if(!(A.TYPED_ARRAY_SUPPORT||this instanceof A))return new A(t,e,n);if("number"==typeof t){if("string"==typeof e)throw new Error("If encoding is specified then the first argument must be a string");return u(this,t)}return l(this,t,e,n)}function l(t,e,n,r){if("number"==typeof e)throw new TypeError('"value" argument must not be a number');return"undefined"!=typeof ArrayBuffer&&e instanceof ArrayBuffer?function(t,e,n,r){if(e.byteLength,n<0||e.byteLength<n)throw new RangeError("'offset' is out of bounds");if(e.byteLength<n+(r||0))throw new RangeError("'length' is out of bounds");e=void 0===n&&void 0===r?new Uint8Array(e):void 0===r?new Uint8Array(e,n):new Uint8Array(e,n,r);A.TYPED_ARRAY_SUPPORT?(t=e).__proto__=A.prototype:t=g(t,e);return t}(t,e,n,r):"string"==typeof e?function(t,e,n){"string"==typeof n&&""!==n||(n="utf8");if(!A.isEncoding(n))throw new TypeError('"encoding" must be a valid string encoding');var r=0|d(e,n),i=(t=s(t,r)).write(e,n);i!==r&&(t=t.slice(0,i));return t}(t,e,n):function(t,e){if(A.isBuffer(e)){var n=0|h(e.length);return 0===(t=s(t,n)).length?t:(e.copy(t,0,0,n),t)}if(e){if("undefined"!=typeof ArrayBuffer&&e.buffer instanceof ArrayBuffer||"length"in e)return"number"!=typeof e.length||(r=e.length)!=r?s(t,0):g(t,e);if("Buffer"===e.type&&o(e.data))return g(t,e.data)}var r;throw new TypeError("First argument must be a string, Buffer, ArrayBuffer, Array, or array-like object.")}(t,e)}function c(t){if("number"!=typeof t)throw new TypeError('"size" argument must be a number');if(t<0)throw new RangeError('"size" argument must not be negative')}function u(t,e){if(c(e),t=s(t,e<0?0:0|h(e)),!A.TYPED_ARRAY_SUPPORT)for(var n=0;n<e;++n)t[n]=0;return t}function g(t,e){var n=e.length<0?0:0|h(e.length);t=s(t,n);for(var r=0;r<n;r+=1)t[r]=255&e[r];return t}function h(t){if(t>=a())throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x"+a().toString(16)+" bytes");return 0|t}function d(t,e){if(A.isBuffer(t))return t.length;if("undefined"!=typeof ArrayBuffer&&"function"==typeof ArrayBuffer.isView&&(ArrayBuffer.isView(t)||t instanceof ArrayBuffer))return t.byteLength;"string"!=typeof t&&(t=""+t);var n=t.length;if(0===n)return 0;for(var r=!1;;)switch(e){case"ascii":case"latin1":case"binary":return n;case"utf8":case"utf-8":case void 0:return H(t).length;case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return 2*n;case"hex":return n>>>1;case"base64":return j(t).length;default:if(r)return H(t).length;e=(""+e).toLowerCase(),r=!0}}function f(t,e,n){var r=!1;if((void 0===e||e<0)&&(e=0),e>this.length)return"";if((void 0===n||n>this.length)&&(n=this.length),n<=0)return"";if((n>>>=0)<=(e>>>=0))return"";for(t||(t="utf8");;)switch(t){case"hex":return M(this,e,n);case"utf8":case"utf-8":return w(this,e,n);case"ascii":return _(this,e,n);case"latin1":case"binary":return D(this,e,n);case"base64":return Q(this,e,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return k(this,e,n);default:if(r)throw new TypeError("Unknown encoding: "+t);t=(t+"").toLowerCase(),r=!0}}function p(t,e,n){var r=t[e];t[e]=t[n],t[n]=r}function I(t,e,n,r,i){if(0===t.length)return-1;if("string"==typeof n?(r=n,n=0):n>2147483647?n=2147483647:n<-2147483648&&(n=-2147483648),n=+n,isNaN(n)&&(n=i?0:t.length-1),n<0&&(n=t.length+n),n>=t.length){if(i)return-1;n=t.length-1}else if(n<0){if(!i)return-1;n=0}if("string"==typeof e&&(e=A.from(e,r)),A.isBuffer(e))return 0===e.length?-1:C(t,e,n,r,i);if("number"==typeof e)return e&=255,A.TYPED_ARRAY_SUPPORT&&"function"==typeof Uint8Array.prototype.indexOf?i?Uint8Array.prototype.indexOf.call(t,e,n):Uint8Array.prototype.lastIndexOf.call(t,e,n):C(t,[e],n,r,i);throw new TypeError("val must be string, number or Buffer")}function C(t,e,n,r,i){var o,a=1,s=t.length,A=e.length;if(void 0!==r&&("ucs2"===(r=String(r).toLowerCase())||"ucs-2"===r||"utf16le"===r||"utf-16le"===r)){if(t.length<2||e.length<2)return-1;a=2,s/=2,A/=2,n/=2}function l(t,e){return 1===a?t[e]:t.readUInt16BE(e*a)}if(i){var c=-1;for(o=n;o<s;o++)if(l(t,o)===l(e,-1===c?0:o-c)){if(-1===c&&(c=o),o-c+1===A)return c*a}else-1!==c&&(o-=o-c),c=-1}else for(n+A>s&&(n=s-A),o=n;o>=0;o--){for(var u=!0,g=0;g<A;g++)if(l(t,o+g)!==l(e,g)){u=!1;break}if(u)return o}return-1}function m(t,e,n,r){n=Number(n)||0;var i=t.length-n;r?(r=Number(r))>i&&(r=i):r=i;var o=e.length;if(o%2!=0)throw new TypeError("Invalid hex string");r>o/2&&(r=o/2);for(var a=0;a<r;++a){var s=parseInt(e.substr(2*a,2),16);if(isNaN(s))return a;t[n+a]=s}return a}function y(t,e,n,r){return Y(H(e,t.length-n),t,n,r)}function E(t,e,n,r){return Y(function(t){for(var e=[],n=0;n<t.length;++n)e.push(255&t.charCodeAt(n));return e}(e),t,n,r)}function v(t,e,n,r){return E(t,e,n,r)}function B(t,e,n,r){return Y(j(e),t,n,r)}function b(t,e,n,r){return Y(function(t,e){for(var n,r,i,o=[],a=0;a<t.length&&!((e-=2)<0);++a)n=t.charCodeAt(a),r=n>>8,i=n%256,o.push(i),o.push(r);return o}(e,t.length-n),t,n,r)}function Q(t,e,n){return 0===e&&n===t.length?r.fromByteArray(t):r.fromByteArray(t.slice(e,n))}function w(t,e,n){n=Math.min(t.length,n);for(var r=[],i=e;i<n;){var o,a,s,A,l=t[i],c=null,u=l>239?4:l>223?3:l>191?2:1;if(i+u<=n)switch(u){case 1:l<128&&(c=l);break;case 2:128==(192&(o=t[i+1]))&&(A=(31&l)<<6|63&o)>127&&(c=A);break;case 3:o=t[i+1],a=t[i+2],128==(192&o)&&128==(192&a)&&(A=(15&l)<<12|(63&o)<<6|63&a)>2047&&(A<55296||A>57343)&&(c=A);break;case 4:o=t[i+1],a=t[i+2],s=t[i+3],128==(192&o)&&128==(192&a)&&128==(192&s)&&(A=(15&l)<<18|(63&o)<<12|(63&a)<<6|63&s)>65535&&A<1114112&&(c=A)}null===c?(c=65533,u=1):c>65535&&(c-=65536,r.push(c>>>10&1023|55296),c=56320|1023&c),r.push(c),i+=u}return function(t){var e=t.length;if(e<=x)return String.fromCharCode.apply(String,t);var n="",r=0;for(;r<e;)n+=String.fromCharCode.apply(String,t.slice(r,r+=x));return n}(r)}e.Buffer=A,e.SlowBuffer=function(t){+t!=t&&(t=0);return A.alloc(+t)},e.INSPECT_MAX_BYTES=50,A.TYPED_ARRAY_SUPPORT=void 0!==t.TYPED_ARRAY_SUPPORT?t.TYPED_ARRAY_SUPPORT:function(){try{var t=new Uint8Array(1);return t.__proto__={__proto__:Uint8Array.prototype,foo:function(){return 42}},42===t.foo()&&"function"==typeof t.subarray&&0===t.subarray(1,1).byteLength}catch(t){return!1}}(),e.kMaxLength=a(),A.poolSize=8192,A._augment=function(t){return t.__proto__=A.prototype,t},A.from=function(t,e,n){return l(null,t,e,n)},A.TYPED_ARRAY_SUPPORT&&(A.prototype.__proto__=Uint8Array.prototype,A.__proto__=Uint8Array,"undefined"!=typeof Symbol&&Symbol.species&&A[Symbol.species]===A&&Object.defineProperty(A,Symbol.species,{value:null,configurable:!0})),A.alloc=function(t,e,n){return function(t,e,n,r){return c(e),e<=0?s(t,e):void 0!==n?"string"==typeof r?s(t,e).fill(n,r):s(t,e).fill(n):s(t,e)}(null,t,e,n)},A.allocUnsafe=function(t){return u(null,t)},A.allocUnsafeSlow=function(t){return u(null,t)},A.isBuffer=function(t){return!(null==t||!t._isBuffer)},A.compare=function(t,e){if(!A.isBuffer(t)||!A.isBuffer(e))throw new TypeError("Arguments must be Buffers");if(t===e)return 0;for(var n=t.length,r=e.length,i=0,o=Math.min(n,r);i<o;++i)if(t[i]!==e[i]){n=t[i],r=e[i];break}return n<r?-1:r<n?1:0},A.isEncoding=function(t){switch(String(t).toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"latin1":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return!0;default:return!1}},A.concat=function(t,e){if(!o(t))throw new TypeError('"list" argument must be an Array of Buffers');if(0===t.length)return A.alloc(0);var n;if(void 0===e)for(e=0,n=0;n<t.length;++n)e+=t[n].length;var r=A.allocUnsafe(e),i=0;for(n=0;n<t.length;++n){var a=t[n];if(!A.isBuffer(a))throw new TypeError('"list" argument must be an Array of Buffers');a.copy(r,i),i+=a.length}return r},A.byteLength=d,A.prototype._isBuffer=!0,A.prototype.swap16=function(){var t=this.length;if(t%2!=0)throw new RangeError("Buffer size must be a multiple of 16-bits");for(var e=0;e<t;e+=2)p(this,e,e+1);return this},A.prototype.swap32=function(){var t=this.length;if(t%4!=0)throw new RangeError("Buffer size must be a multiple of 32-bits");for(var e=0;e<t;e+=4)p(this,e,e+3),p(this,e+1,e+2);return this},A.prototype.swap64=function(){var t=this.length;if(t%8!=0)throw new RangeError("Buffer size must be a multiple of 64-bits");for(var e=0;e<t;e+=8)p(this,e,e+7),p(this,e+1,e+6),p(this,e+2,e+5),p(this,e+3,e+4);return this},A.prototype.toString=function(){var t=0|this.length;return 0===t?"":0===arguments.length?w(this,0,t):f.apply(this,arguments)},A.prototype.equals=function(t){if(!A.isBuffer(t))throw new TypeError("Argument must be a Buffer");return this===t||0===A.compare(this,t)},A.prototype.inspect=function(){var t="",n=e.INSPECT_MAX_BYTES;return this.length>0&&(t=this.toString("hex",0,n).match(/.{2}/g).join(" "),this.length>n&&(t+=" ... ")),"<Buffer "+t+">"},A.prototype.compare=function(t,e,n,r,i){if(!A.isBuffer(t))throw new TypeError("Argument must be a Buffer");if(void 0===e&&(e=0),void 0===n&&(n=t?t.length:0),void 0===r&&(r=0),void 0===i&&(i=this.length),e<0||n>t.length||r<0||i>this.length)throw new RangeError("out of range index");if(r>=i&&e>=n)return 0;if(r>=i)return-1;if(e>=n)return 1;if(this===t)return 0;for(var o=(i>>>=0)-(r>>>=0),a=(n>>>=0)-(e>>>=0),s=Math.min(o,a),l=this.slice(r,i),c=t.slice(e,n),u=0;u<s;++u)if(l[u]!==c[u]){o=l[u],a=c[u];break}return o<a?-1:a<o?1:0},A.prototype.includes=function(t,e,n){return-1!==this.indexOf(t,e,n)},A.prototype.indexOf=function(t,e,n){return I(this,t,e,n,!0)},A.prototype.lastIndexOf=function(t,e,n){return I(this,t,e,n,!1)},A.prototype.write=function(t,e,n,r){if(void 0===e)r="utf8",n=this.length,e=0;else if(void 0===n&&"string"==typeof e)r=e,n=this.length,e=0;else{if(!isFinite(e))throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");e|=0,isFinite(n)?(n|=0,void 0===r&&(r="utf8")):(r=n,n=void 0)}var i=this.length-e;if((void 0===n||n>i)&&(n=i),t.length>0&&(n<0||e<0)||e>this.length)throw new RangeError("Attempt to write outside buffer bounds");r||(r="utf8");for(var o=!1;;)switch(r){case"hex":return m(this,t,e,n);case"utf8":case"utf-8":return y(this,t,e,n);case"ascii":return E(this,t,e,n);case"latin1":case"binary":return v(this,t,e,n);case"base64":return B(this,t,e,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return b(this,t,e,n);default:if(o)throw new TypeError("Unknown encoding: "+r);r=(""+r).toLowerCase(),o=!0}},A.prototype.toJSON=function(){return{type:"Buffer",data:Array.prototype.slice.call(this._arr||this,0)}};var x=4096;function _(t,e,n){var r="";n=Math.min(t.length,n);for(var i=e;i<n;++i)r+=String.fromCharCode(127&t[i]);return r}function D(t,e,n){var r="";n=Math.min(t.length,n);for(var i=e;i<n;++i)r+=String.fromCharCode(t[i]);return r}function M(t,e,n){var r=t.length;(!e||e<0)&&(e=0),(!n||n<0||n>r)&&(n=r);for(var i="",o=e;o<n;++o)i+=G(t[o]);return i}function k(t,e,n){for(var r=t.slice(e,n),i="",o=0;o<r.length;o+=2)i+=String.fromCharCode(r[o]+256*r[o+1]);return i}function S(t,e,n){if(t%1!=0||t<0)throw new RangeError("offset is not uint");if(t+e>n)throw new RangeError("Trying to access beyond buffer length")}function L(t,e,n,r,i,o){if(!A.isBuffer(t))throw new TypeError('"buffer" argument must be a Buffer instance');if(e>i||e<o)throw new RangeError('"value" argument is out of bounds');if(n+r>t.length)throw new RangeError("Index out of range")}function R(t,e,n,r){e<0&&(e=65535+e+1);for(var i=0,o=Math.min(t.length-n,2);i<o;++i)t[n+i]=(e&255<<8*(r?i:1-i))>>>8*(r?i:1-i)}function N(t,e,n,r){e<0&&(e=4294967295+e+1);for(var i=0,o=Math.min(t.length-n,4);i<o;++i)t[n+i]=e>>>8*(r?i:3-i)&255}function F(t,e,n,r,i,o){if(n+r>t.length)throw new RangeError("Index out of range");if(n<0)throw new RangeError("Index out of range")}function T(t,e,n,r,o){return o||F(t,0,n,4),i.write(t,e,n,r,23,4),n+4}function U(t,e,n,r,o){return o||F(t,0,n,8),i.write(t,e,n,r,52,8),n+8}A.prototype.slice=function(t,e){var n,r=this.length;if((t=~~t)<0?(t+=r)<0&&(t=0):t>r&&(t=r),(e=void 0===e?r:~~e)<0?(e+=r)<0&&(e=0):e>r&&(e=r),e<t&&(e=t),A.TYPED_ARRAY_SUPPORT)(n=this.subarray(t,e)).__proto__=A.prototype;else{var i=e-t;n=new A(i,void 0);for(var o=0;o<i;++o)n[o]=this[o+t]}return n},A.prototype.readUIntLE=function(t,e,n){t|=0,e|=0,n||S(t,e,this.length);for(var r=this[t],i=1,o=0;++o<e&&(i*=256);)r+=this[t+o]*i;return r},A.prototype.readUIntBE=function(t,e,n){t|=0,e|=0,n||S(t,e,this.length);for(var r=this[t+--e],i=1;e>0&&(i*=256);)r+=this[t+--e]*i;return r},A.prototype.readUInt8=function(t,e){return e||S(t,1,this.length),this[t]},A.prototype.readUInt16LE=function(t,e){return e||S(t,2,this.length),this[t]|this[t+1]<<8},A.prototype.readUInt16BE=function(t,e){return e||S(t,2,this.length),this[t]<<8|this[t+1]},A.prototype.readUInt32LE=function(t,e){return e||S(t,4,this.length),(this[t]|this[t+1]<<8|this[t+2]<<16)+16777216*this[t+3]},A.prototype.readUInt32BE=function(t,e){return e||S(t,4,this.length),16777216*this[t]+(this[t+1]<<16|this[t+2]<<8|this[t+3])},A.prototype.readIntLE=function(t,e,n){t|=0,e|=0,n||S(t,e,this.length);for(var r=this[t],i=1,o=0;++o<e&&(i*=256);)r+=this[t+o]*i;return r>=(i*=128)&&(r-=Math.pow(2,8*e)),r},A.prototype.readIntBE=function(t,e,n){t|=0,e|=0,n||S(t,e,this.length);for(var r=e,i=1,o=this[t+--r];r>0&&(i*=256);)o+=this[t+--r]*i;return o>=(i*=128)&&(o-=Math.pow(2,8*e)),o},A.prototype.readInt8=function(t,e){return e||S(t,1,this.length),128&this[t]?-1*(255-this[t]+1):this[t]},A.prototype.readInt16LE=function(t,e){e||S(t,2,this.length);var n=this[t]|this[t+1]<<8;return 32768&n?4294901760|n:n},A.prototype.readInt16BE=function(t,e){e||S(t,2,this.length);var n=this[t+1]|this[t]<<8;return 32768&n?4294901760|n:n},A.prototype.readInt32LE=function(t,e){return e||S(t,4,this.length),this[t]|this[t+1]<<8|this[t+2]<<16|this[t+3]<<24},A.prototype.readInt32BE=function(t,e){return e||S(t,4,this.length),this[t]<<24|this[t+1]<<16|this[t+2]<<8|this[t+3]},A.prototype.readFloatLE=function(t,e){return e||S(t,4,this.length),i.read(this,t,!0,23,4)},A.prototype.readFloatBE=function(t,e){return e||S(t,4,this.length),i.read(this,t,!1,23,4)},A.prototype.readDoubleLE=function(t,e){return e||S(t,8,this.length),i.read(this,t,!0,52,8)},A.prototype.readDoubleBE=function(t,e){return e||S(t,8,this.length),i.read(this,t,!1,52,8)},A.prototype.writeUIntLE=function(t,e,n,r){(t=+t,e|=0,n|=0,r)||L(this,t,e,n,Math.pow(2,8*n)-1,0);var i=1,o=0;for(this[e]=255&t;++o<n&&(i*=256);)this[e+o]=t/i&255;return e+n},A.prototype.writeUIntBE=function(t,e,n,r){(t=+t,e|=0,n|=0,r)||L(this,t,e,n,Math.pow(2,8*n)-1,0);var i=n-1,o=1;for(this[e+i]=255&t;--i>=0&&(o*=256);)this[e+i]=t/o&255;return e+n},A.prototype.writeUInt8=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,1,255,0),A.TYPED_ARRAY_SUPPORT||(t=Math.floor(t)),this[e]=255&t,e+1},A.prototype.writeUInt16LE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,2,65535,0),A.TYPED_ARRAY_SUPPORT?(this[e]=255&t,this[e+1]=t>>>8):R(this,t,e,!0),e+2},A.prototype.writeUInt16BE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,2,65535,0),A.TYPED_ARRAY_SUPPORT?(this[e]=t>>>8,this[e+1]=255&t):R(this,t,e,!1),e+2},A.prototype.writeUInt32LE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,4,4294967295,0),A.TYPED_ARRAY_SUPPORT?(this[e+3]=t>>>24,this[e+2]=t>>>16,this[e+1]=t>>>8,this[e]=255&t):N(this,t,e,!0),e+4},A.prototype.writeUInt32BE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,4,4294967295,0),A.TYPED_ARRAY_SUPPORT?(this[e]=t>>>24,this[e+1]=t>>>16,this[e+2]=t>>>8,this[e+3]=255&t):N(this,t,e,!1),e+4},A.prototype.writeIntLE=function(t,e,n,r){if(t=+t,e|=0,!r){var i=Math.pow(2,8*n-1);L(this,t,e,n,i-1,-i)}var o=0,a=1,s=0;for(this[e]=255&t;++o<n&&(a*=256);)t<0&&0===s&&0!==this[e+o-1]&&(s=1),this[e+o]=(t/a>>0)-s&255;return e+n},A.prototype.writeIntBE=function(t,e,n,r){if(t=+t,e|=0,!r){var i=Math.pow(2,8*n-1);L(this,t,e,n,i-1,-i)}var o=n-1,a=1,s=0;for(this[e+o]=255&t;--o>=0&&(a*=256);)t<0&&0===s&&0!==this[e+o+1]&&(s=1),this[e+o]=(t/a>>0)-s&255;return e+n},A.prototype.writeInt8=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,1,127,-128),A.TYPED_ARRAY_SUPPORT||(t=Math.floor(t)),t<0&&(t=255+t+1),this[e]=255&t,e+1},A.prototype.writeInt16LE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,2,32767,-32768),A.TYPED_ARRAY_SUPPORT?(this[e]=255&t,this[e+1]=t>>>8):R(this,t,e,!0),e+2},A.prototype.writeInt16BE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,2,32767,-32768),A.TYPED_ARRAY_SUPPORT?(this[e]=t>>>8,this[e+1]=255&t):R(this,t,e,!1),e+2},A.prototype.writeInt32LE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,4,2147483647,-2147483648),A.TYPED_ARRAY_SUPPORT?(this[e]=255&t,this[e+1]=t>>>8,this[e+2]=t>>>16,this[e+3]=t>>>24):N(this,t,e,!0),e+4},A.prototype.writeInt32BE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,4,2147483647,-2147483648),t<0&&(t=4294967295+t+1),A.TYPED_ARRAY_SUPPORT?(this[e]=t>>>24,this[e+1]=t>>>16,this[e+2]=t>>>8,this[e+3]=255&t):N(this,t,e,!1),e+4},A.prototype.writeFloatLE=function(t,e,n){return T(this,t,e,!0,n)},A.prototype.writeFloatBE=function(t,e,n){return T(this,t,e,!1,n)},A.prototype.writeDoubleLE=function(t,e,n){return U(this,t,e,!0,n)},A.prototype.writeDoubleBE=function(t,e,n){return U(this,t,e,!1,n)},A.prototype.copy=function(t,e,n,r){if(n||(n=0),r||0===r||(r=this.length),e>=t.length&&(e=t.length),e||(e=0),r>0&&r<n&&(r=n),r===n)return 0;if(0===t.length||0===this.length)return 0;if(e<0)throw new RangeError("targetStart out of bounds");if(n<0||n>=this.length)throw new RangeError("sourceStart out of bounds");if(r<0)throw new RangeError("sourceEnd out of bounds");r>this.length&&(r=this.length),t.length-e<r-n&&(r=t.length-e+n);var i,o=r-n;if(this===t&&n<e&&e<r)for(i=o-1;i>=0;--i)t[i+e]=this[i+n];else if(o<1e3||!A.TYPED_ARRAY_SUPPORT)for(i=0;i<o;++i)t[i+e]=this[i+n];else Uint8Array.prototype.set.call(t,this.subarray(n,n+o),e);return o},A.prototype.fill=function(t,e,n,r){if("string"==typeof t){if("string"==typeof e?(r=e,e=0,n=this.length):"string"==typeof n&&(r=n,n=this.length),1===t.length){var i=t.charCodeAt(0);i<256&&(t=i)}if(void 0!==r&&"string"!=typeof r)throw new TypeError("encoding must be a string");if("string"==typeof r&&!A.isEncoding(r))throw new TypeError("Unknown encoding: "+r)}else"number"==typeof t&&(t&=255);if(e<0||this.length<e||this.length<n)throw new RangeError("Out of range index");if(n<=e)return this;var o;if(e>>>=0,n=void 0===n?this.length:n>>>0,t||(t=0),"number"==typeof t)for(o=e;o<n;++o)this[o]=t;else{var a=A.isBuffer(t)?t:H(new A(t,r).toString()),s=a.length;for(o=0;o<n-e;++o)this[o+e]=a[o%s]}return this};var O=/[^+\/0-9A-Za-z-_]/g;function G(t){return t<16?"0"+t.toString(16):t.toString(16)}function H(t,e){var n;e=e||1/0;for(var r=t.length,i=null,o=[],a=0;a<r;++a){if((n=t.charCodeAt(a))>55295&&n<57344){if(!i){if(n>56319){(e-=3)>-1&&o.push(239,191,189);continue}if(a+1===r){(e-=3)>-1&&o.push(239,191,189);continue}i=n;continue}if(n<56320){(e-=3)>-1&&o.push(239,191,189),i=n;continue}n=65536+(i-55296<<10|n-56320)}else i&&(e-=3)>-1&&o.push(239,191,189);if(i=null,n<128){if((e-=1)<0)break;o.push(n)}else if(n<2048){if((e-=2)<0)break;o.push(n>>6|192,63&n|128)}else if(n<65536){if((e-=3)<0)break;o.push(n>>12|224,n>>6&63|128,63&n|128)}else{if(!(n<1114112))throw new Error("Invalid code point");if((e-=4)<0)break;o.push(n>>18|240,n>>12&63|128,n>>6&63|128,63&n|128)}}return o}function j(t){return r.toByteArray(function(t){if((t=function(t){return t.trim?t.trim():t.replace(/^\s+|\s+$/g,"")}(t).replace(O,"")).length<2)return"";for(;t.length%4!=0;)t+="=";return t}(t))}function Y(t,e,n,r){for(var i=0;i<r&&!(i+n>=e.length||i>=t.length);++i)e[i+n]=t[i];return i}}).call(this,n(37))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r,i=function(){function t(t){this._pid=r.nextPID(),this.name=t.name,this._create=t.create,this._coerce=t.coerce||null,this._compare=t.compare||null,this._changed=t.changed||null}return t.prototype.get=function(t){var e=r.ensureMap(t);return this._pid in e?e[this._pid]:e[this._pid]=this._createValue(t)},t.prototype.set=function(t,e){var n,i=r.ensureMap(t);n=this._pid in i?i[this._pid]:i[this._pid]=this._createValue(t);var o=this._coerceValue(t,e);this._maybeNotify(t,n,i[this._pid]=o)},t.prototype.coerce=function(t){var e,n=r.ensureMap(t);e=this._pid in n?n[this._pid]:n[this._pid]=this._createValue(t);var i=this._coerceValue(t,e);this._maybeNotify(t,e,n[this._pid]=i)},t.prototype._createValue=function(t){return(0,this._create)(t)},t.prototype._coerceValue=function(t,e){var n=this._coerce;return n?n(t,e):e},t.prototype._compareValue=function(t,e){var n=this._compare;return n?n(t,e):t===e},t.prototype._maybeNotify=function(t,e,n){var r=this._changed;r&&!this._compareValue(e,n)&&r(t,e,n)},t}();e.AttachedProperty=i,function(t){t.clearData=function(t){r.ownerData.delete(t)}}(i=e.AttachedProperty||(e.AttachedProperty={})),e.AttachedProperty=i,function(t){var e;t.ownerData=new WeakMap,t.nextPID=(e=0,function(){return"pid-"+(""+Math.random()).slice(2)+"-"+e++}),t.ensureMap=function(e){var n=t.ownerData.get(e);return n||(n=Object.create(null),t.ownerData.set(e,n),n)}}(r||(r={}))},function(t,e,n){var r;
/*!
 * jQuery JavaScript Library v3.4.1
 * https://jquery.com/
 *
 * Includes Sizzle.js
 * https://sizzlejs.com/
 *
 * Copyright JS Foundation and other contributors
 * Released under the MIT license
 * https://jquery.org/license
 *
 * Date: 2019-05-01T21:04Z
 */
/*!
 * jQuery JavaScript Library v3.4.1
 * https://jquery.com/
 *
 * Includes Sizzle.js
 * https://sizzlejs.com/
 *
 * Copyright JS Foundation and other contributors
 * Released under the MIT license
 * https://jquery.org/license
 *
 * Date: 2019-05-01T21:04Z
 */
!function(e,n){"use strict";"object"==typeof t.exports?t.exports=e.document?n(e,!0):function(t){if(!t.document)throw new Error("jQuery requires a window with a document");return n(t)}:n(e)}("undefined"!=typeof window?window:this,function(n,i){"use strict";var o=[],a=n.document,s=Object.getPrototypeOf,A=o.slice,l=o.concat,c=o.push,u=o.indexOf,g={},h=g.toString,d=g.hasOwnProperty,f=d.toString,p=f.call(Object),I={},C=function(t){return"function"==typeof t&&"number"!=typeof t.nodeType},m=function(t){return null!=t&&t===t.window},y={type:!0,src:!0,nonce:!0,noModule:!0};function E(t,e,n){var r,i,o=(n=n||a).createElement("script");if(o.text=t,e)for(r in y)(i=e[r]||e.getAttribute&&e.getAttribute(r))&&o.setAttribute(r,i);n.head.appendChild(o).parentNode.removeChild(o)}function v(t){return null==t?t+"":"object"==typeof t||"function"==typeof t?g[h.call(t)]||"object":typeof t}var B=function(t,e){return new B.fn.init(t,e)},b=/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g;function Q(t){var e=!!t&&"length"in t&&t.length,n=v(t);return!C(t)&&!m(t)&&("array"===n||0===e||"number"==typeof e&&e>0&&e-1 in t)}B.fn=B.prototype={jquery:"3.4.1",constructor:B,length:0,toArray:function(){return A.call(this)},get:function(t){return null==t?A.call(this):t<0?this[t+this.length]:this[t]},pushStack:function(t){var e=B.merge(this.constructor(),t);return e.prevObject=this,e},each:function(t){return B.each(this,t)},map:function(t){return this.pushStack(B.map(this,function(e,n){return t.call(e,n,e)}))},slice:function(){return this.pushStack(A.apply(this,arguments))},first:function(){return this.eq(0)},last:function(){return this.eq(-1)},eq:function(t){var e=this.length,n=+t+(t<0?e:0);return this.pushStack(n>=0&&n<e?[this[n]]:[])},end:function(){return this.prevObject||this.constructor()},push:c,sort:o.sort,splice:o.splice},B.extend=B.fn.extend=function(){var t,e,n,r,i,o,a=arguments[0]||{},s=1,A=arguments.length,l=!1;for("boolean"==typeof a&&(l=a,a=arguments[s]||{},s++),"object"==typeof a||C(a)||(a={}),s===A&&(a=this,s--);s<A;s++)if(null!=(t=arguments[s]))for(e in t)r=t[e],"__proto__"!==e&&a!==r&&(l&&r&&(B.isPlainObject(r)||(i=Array.isArray(r)))?(n=a[e],o=i&&!Array.isArray(n)?[]:i||B.isPlainObject(n)?n:{},i=!1,a[e]=B.extend(l,o,r)):void 0!==r&&(a[e]=r));return a},B.extend({expando:"jQuery"+("3.4.1"+Math.random()).replace(/\D/g,""),isReady:!0,error:function(t){throw new Error(t)},noop:function(){},isPlainObject:function(t){var e,n;return!(!t||"[object Object]"!==h.call(t))&&(!(e=s(t))||"function"==typeof(n=d.call(e,"constructor")&&e.constructor)&&f.call(n)===p)},isEmptyObject:function(t){var e;for(e in t)return!1;return!0},globalEval:function(t,e){E(t,{nonce:e&&e.nonce})},each:function(t,e){var n,r=0;if(Q(t))for(n=t.length;r<n&&!1!==e.call(t[r],r,t[r]);r++);else for(r in t)if(!1===e.call(t[r],r,t[r]))break;return t},trim:function(t){return null==t?"":(t+"").replace(b,"")},makeArray:function(t,e){var n=e||[];return null!=t&&(Q(Object(t))?B.merge(n,"string"==typeof t?[t]:t):c.call(n,t)),n},inArray:function(t,e,n){return null==e?-1:u.call(e,t,n)},merge:function(t,e){for(var n=+e.length,r=0,i=t.length;r<n;r++)t[i++]=e[r];return t.length=i,t},grep:function(t,e,n){for(var r=[],i=0,o=t.length,a=!n;i<o;i++)!e(t[i],i)!==a&&r.push(t[i]);return r},map:function(t,e,n){var r,i,o=0,a=[];if(Q(t))for(r=t.length;o<r;o++)null!=(i=e(t[o],o,n))&&a.push(i);else for(o in t)null!=(i=e(t[o],o,n))&&a.push(i);return l.apply([],a)},guid:1,support:I}),"function"==typeof Symbol&&(B.fn[Symbol.iterator]=o[Symbol.iterator]),B.each("Boolean Number String Function Array Date RegExp Object Error Symbol".split(" "),function(t,e){g["[object "+e+"]"]=e.toLowerCase()});var w=
/*!
 * Sizzle CSS Selector Engine v2.3.4
 * https://sizzlejs.com/
 *
 * Copyright JS Foundation and other contributors
 * Released under the MIT license
 * https://js.foundation/
 *
 * Date: 2019-04-08
 */
function(t){var e,n,r,i,o,a,s,A,l,c,u,g,h,d,f,p,I,C,m,y="sizzle"+1*new Date,E=t.document,v=0,B=0,b=At(),Q=At(),w=At(),x=At(),_=function(t,e){return t===e&&(u=!0),0},D={}.hasOwnProperty,M=[],k=M.pop,S=M.push,L=M.push,R=M.slice,N=function(t,e){for(var n=0,r=t.length;n<r;n++)if(t[n]===e)return n;return-1},F="checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",T="[\\x20\\t\\r\\n\\f]",U="(?:\\\\.|[\\w-]|[^\0-\\xa0])+",O="\\["+T+"*("+U+")(?:"+T+"*([*^$|!~]?=)"+T+"*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|("+U+"))|)"+T+"*\\]",G=":("+U+")(?:\\((('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|((?:\\\\.|[^\\\\()[\\]]|"+O+")*)|.*)\\)|)",H=new RegExp(T+"+","g"),j=new RegExp("^"+T+"+|((?:^|[^\\\\])(?:\\\\.)*)"+T+"+$","g"),Y=new RegExp("^"+T+"*,"+T+"*"),P=new RegExp("^"+T+"*([>+~]|"+T+")"+T+"*"),K=new RegExp(T+"|>"),z=new RegExp(G),J=new RegExp("^"+U+"$"),q={ID:new RegExp("^#("+U+")"),CLASS:new RegExp("^\\.("+U+")"),TAG:new RegExp("^("+U+"|[*])"),ATTR:new RegExp("^"+O),PSEUDO:new RegExp("^"+G),CHILD:new RegExp("^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\("+T+"*(even|odd|(([+-]|)(\\d*)n|)"+T+"*(?:([+-]|)"+T+"*(\\d+)|))"+T+"*\\)|)","i"),bool:new RegExp("^(?:"+F+")$","i"),needsContext:new RegExp("^"+T+"*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\("+T+"*((?:-\\d)?\\d*)"+T+"*\\)|)(?=[^-]|$)","i")},V=/HTML$/i,W=/^(?:input|select|textarea|button)$/i,X=/^h\d$/i,Z=/^[^{]+\{\s*\[native \w/,$=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,tt=/[+~]/,et=new RegExp("\\\\([\\da-f]{1,6}"+T+"?|("+T+")|.)","ig"),nt=function(t,e,n){var r="0x"+e-65536;return r!=r||n?e:r<0?String.fromCharCode(r+65536):String.fromCharCode(r>>10|55296,1023&r|56320)},rt=/([\0-\x1f\x7f]|^-?\d)|^-$|[^\0-\x1f\x7f-\uFFFF\w-]/g,it=function(t,e){return e?"\0"===t?"�":t.slice(0,-1)+"\\"+t.charCodeAt(t.length-1).toString(16)+" ":"\\"+t},ot=function(){g()},at=yt(function(t){return!0===t.disabled&&"fieldset"===t.nodeName.toLowerCase()},{dir:"parentNode",next:"legend"});try{L.apply(M=R.call(E.childNodes),E.childNodes),M[E.childNodes.length].nodeType}catch(t){L={apply:M.length?function(t,e){S.apply(t,R.call(e))}:function(t,e){for(var n=t.length,r=0;t[n++]=e[r++];);t.length=n-1}}}function st(t,e,r,i){var o,s,l,c,u,d,I,C=e&&e.ownerDocument,v=e?e.nodeType:9;if(r=r||[],"string"!=typeof t||!t||1!==v&&9!==v&&11!==v)return r;if(!i&&((e?e.ownerDocument||e:E)!==h&&g(e),e=e||h,f)){if(11!==v&&(u=$.exec(t)))if(o=u[1]){if(9===v){if(!(l=e.getElementById(o)))return r;if(l.id===o)return r.push(l),r}else if(C&&(l=C.getElementById(o))&&m(e,l)&&l.id===o)return r.push(l),r}else{if(u[2])return L.apply(r,e.getElementsByTagName(t)),r;if((o=u[3])&&n.getElementsByClassName&&e.getElementsByClassName)return L.apply(r,e.getElementsByClassName(o)),r}if(n.qsa&&!x[t+" "]&&(!p||!p.test(t))&&(1!==v||"object"!==e.nodeName.toLowerCase())){if(I=t,C=e,1===v&&K.test(t)){for((c=e.getAttribute("id"))?c=c.replace(rt,it):e.setAttribute("id",c=y),s=(d=a(t)).length;s--;)d[s]="#"+c+" "+mt(d[s]);I=d.join(","),C=tt.test(t)&&It(e.parentNode)||e}try{return L.apply(r,C.querySelectorAll(I)),r}catch(e){x(t,!0)}finally{c===y&&e.removeAttribute("id")}}}return A(t.replace(j,"$1"),e,r,i)}function At(){var t=[];return function e(n,i){return t.push(n+" ")>r.cacheLength&&delete e[t.shift()],e[n+" "]=i}}function lt(t){return t[y]=!0,t}function ct(t){var e=h.createElement("fieldset");try{return!!t(e)}catch(t){return!1}finally{e.parentNode&&e.parentNode.removeChild(e),e=null}}function ut(t,e){for(var n=t.split("|"),i=n.length;i--;)r.attrHandle[n[i]]=e}function gt(t,e){var n=e&&t,r=n&&1===t.nodeType&&1===e.nodeType&&t.sourceIndex-e.sourceIndex;if(r)return r;if(n)for(;n=n.nextSibling;)if(n===e)return-1;return t?1:-1}function ht(t){return function(e){return"input"===e.nodeName.toLowerCase()&&e.type===t}}function dt(t){return function(e){var n=e.nodeName.toLowerCase();return("input"===n||"button"===n)&&e.type===t}}function ft(t){return function(e){return"form"in e?e.parentNode&&!1===e.disabled?"label"in e?"label"in e.parentNode?e.parentNode.disabled===t:e.disabled===t:e.isDisabled===t||e.isDisabled!==!t&&at(e)===t:e.disabled===t:"label"in e&&e.disabled===t}}function pt(t){return lt(function(e){return e=+e,lt(function(n,r){for(var i,o=t([],n.length,e),a=o.length;a--;)n[i=o[a]]&&(n[i]=!(r[i]=n[i]))})})}function It(t){return t&&void 0!==t.getElementsByTagName&&t}for(e in n=st.support={},o=st.isXML=function(t){var e=t.namespaceURI,n=(t.ownerDocument||t).documentElement;return!V.test(e||n&&n.nodeName||"HTML")},g=st.setDocument=function(t){var e,i,a=t?t.ownerDocument||t:E;return a!==h&&9===a.nodeType&&a.documentElement?(d=(h=a).documentElement,f=!o(h),E!==h&&(i=h.defaultView)&&i.top!==i&&(i.addEventListener?i.addEventListener("unload",ot,!1):i.attachEvent&&i.attachEvent("onunload",ot)),n.attributes=ct(function(t){return t.className="i",!t.getAttribute("className")}),n.getElementsByTagName=ct(function(t){return t.appendChild(h.createComment("")),!t.getElementsByTagName("*").length}),n.getElementsByClassName=Z.test(h.getElementsByClassName),n.getById=ct(function(t){return d.appendChild(t).id=y,!h.getElementsByName||!h.getElementsByName(y).length}),n.getById?(r.filter.ID=function(t){var e=t.replace(et,nt);return function(t){return t.getAttribute("id")===e}},r.find.ID=function(t,e){if(void 0!==e.getElementById&&f){var n=e.getElementById(t);return n?[n]:[]}}):(r.filter.ID=function(t){var e=t.replace(et,nt);return function(t){var n=void 0!==t.getAttributeNode&&t.getAttributeNode("id");return n&&n.value===e}},r.find.ID=function(t,e){if(void 0!==e.getElementById&&f){var n,r,i,o=e.getElementById(t);if(o){if((n=o.getAttributeNode("id"))&&n.value===t)return[o];for(i=e.getElementsByName(t),r=0;o=i[r++];)if((n=o.getAttributeNode("id"))&&n.value===t)return[o]}return[]}}),r.find.TAG=n.getElementsByTagName?function(t,e){return void 0!==e.getElementsByTagName?e.getElementsByTagName(t):n.qsa?e.querySelectorAll(t):void 0}:function(t,e){var n,r=[],i=0,o=e.getElementsByTagName(t);if("*"===t){for(;n=o[i++];)1===n.nodeType&&r.push(n);return r}return o},r.find.CLASS=n.getElementsByClassName&&function(t,e){if(void 0!==e.getElementsByClassName&&f)return e.getElementsByClassName(t)},I=[],p=[],(n.qsa=Z.test(h.querySelectorAll))&&(ct(function(t){d.appendChild(t).innerHTML="<a id='"+y+"'></a><select id='"+y+"-\r\\' msallowcapture=''><option selected=''></option></select>",t.querySelectorAll("[msallowcapture^='']").length&&p.push("[*^$]="+T+"*(?:''|\"\")"),t.querySelectorAll("[selected]").length||p.push("\\["+T+"*(?:value|"+F+")"),t.querySelectorAll("[id~="+y+"-]").length||p.push("~="),t.querySelectorAll(":checked").length||p.push(":checked"),t.querySelectorAll("a#"+y+"+*").length||p.push(".#.+[+~]")}),ct(function(t){t.innerHTML="<a href='' disabled='disabled'></a><select disabled='disabled'><option/></select>";var e=h.createElement("input");e.setAttribute("type","hidden"),t.appendChild(e).setAttribute("name","D"),t.querySelectorAll("[name=d]").length&&p.push("name"+T+"*[*^$|!~]?="),2!==t.querySelectorAll(":enabled").length&&p.push(":enabled",":disabled"),d.appendChild(t).disabled=!0,2!==t.querySelectorAll(":disabled").length&&p.push(":enabled",":disabled"),t.querySelectorAll("*,:x"),p.push(",.*:")})),(n.matchesSelector=Z.test(C=d.matches||d.webkitMatchesSelector||d.mozMatchesSelector||d.oMatchesSelector||d.msMatchesSelector))&&ct(function(t){n.disconnectedMatch=C.call(t,"*"),C.call(t,"[s!='']:x"),I.push("!=",G)}),p=p.length&&new RegExp(p.join("|")),I=I.length&&new RegExp(I.join("|")),e=Z.test(d.compareDocumentPosition),m=e||Z.test(d.contains)?function(t,e){var n=9===t.nodeType?t.documentElement:t,r=e&&e.parentNode;return t===r||!(!r||1!==r.nodeType||!(n.contains?n.contains(r):t.compareDocumentPosition&&16&t.compareDocumentPosition(r)))}:function(t,e){if(e)for(;e=e.parentNode;)if(e===t)return!0;return!1},_=e?function(t,e){if(t===e)return u=!0,0;var r=!t.compareDocumentPosition-!e.compareDocumentPosition;return r||(1&(r=(t.ownerDocument||t)===(e.ownerDocument||e)?t.compareDocumentPosition(e):1)||!n.sortDetached&&e.compareDocumentPosition(t)===r?t===h||t.ownerDocument===E&&m(E,t)?-1:e===h||e.ownerDocument===E&&m(E,e)?1:c?N(c,t)-N(c,e):0:4&r?-1:1)}:function(t,e){if(t===e)return u=!0,0;var n,r=0,i=t.parentNode,o=e.parentNode,a=[t],s=[e];if(!i||!o)return t===h?-1:e===h?1:i?-1:o?1:c?N(c,t)-N(c,e):0;if(i===o)return gt(t,e);for(n=t;n=n.parentNode;)a.unshift(n);for(n=e;n=n.parentNode;)s.unshift(n);for(;a[r]===s[r];)r++;return r?gt(a[r],s[r]):a[r]===E?-1:s[r]===E?1:0},h):h},st.matches=function(t,e){return st(t,null,null,e)},st.matchesSelector=function(t,e){if((t.ownerDocument||t)!==h&&g(t),n.matchesSelector&&f&&!x[e+" "]&&(!I||!I.test(e))&&(!p||!p.test(e)))try{var r=C.call(t,e);if(r||n.disconnectedMatch||t.document&&11!==t.document.nodeType)return r}catch(t){x(e,!0)}return st(e,h,null,[t]).length>0},st.contains=function(t,e){return(t.ownerDocument||t)!==h&&g(t),m(t,e)},st.attr=function(t,e){(t.ownerDocument||t)!==h&&g(t);var i=r.attrHandle[e.toLowerCase()],o=i&&D.call(r.attrHandle,e.toLowerCase())?i(t,e,!f):void 0;return void 0!==o?o:n.attributes||!f?t.getAttribute(e):(o=t.getAttributeNode(e))&&o.specified?o.value:null},st.escape=function(t){return(t+"").replace(rt,it)},st.error=function(t){throw new Error("Syntax error, unrecognized expression: "+t)},st.uniqueSort=function(t){var e,r=[],i=0,o=0;if(u=!n.detectDuplicates,c=!n.sortStable&&t.slice(0),t.sort(_),u){for(;e=t[o++];)e===t[o]&&(i=r.push(o));for(;i--;)t.splice(r[i],1)}return c=null,t},i=st.getText=function(t){var e,n="",r=0,o=t.nodeType;if(o){if(1===o||9===o||11===o){if("string"==typeof t.textContent)return t.textContent;for(t=t.firstChild;t;t=t.nextSibling)n+=i(t)}else if(3===o||4===o)return t.nodeValue}else for(;e=t[r++];)n+=i(e);return n},(r=st.selectors={cacheLength:50,createPseudo:lt,match:q,attrHandle:{},find:{},relative:{">":{dir:"parentNode",first:!0}," ":{dir:"parentNode"},"+":{dir:"previousSibling",first:!0},"~":{dir:"previousSibling"}},preFilter:{ATTR:function(t){return t[1]=t[1].replace(et,nt),t[3]=(t[3]||t[4]||t[5]||"").replace(et,nt),"~="===t[2]&&(t[3]=" "+t[3]+" "),t.slice(0,4)},CHILD:function(t){return t[1]=t[1].toLowerCase(),"nth"===t[1].slice(0,3)?(t[3]||st.error(t[0]),t[4]=+(t[4]?t[5]+(t[6]||1):2*("even"===t[3]||"odd"===t[3])),t[5]=+(t[7]+t[8]||"odd"===t[3])):t[3]&&st.error(t[0]),t},PSEUDO:function(t){var e,n=!t[6]&&t[2];return q.CHILD.test(t[0])?null:(t[3]?t[2]=t[4]||t[5]||"":n&&z.test(n)&&(e=a(n,!0))&&(e=n.indexOf(")",n.length-e)-n.length)&&(t[0]=t[0].slice(0,e),t[2]=n.slice(0,e)),t.slice(0,3))}},filter:{TAG:function(t){var e=t.replace(et,nt).toLowerCase();return"*"===t?function(){return!0}:function(t){return t.nodeName&&t.nodeName.toLowerCase()===e}},CLASS:function(t){var e=b[t+" "];return e||(e=new RegExp("(^|"+T+")"+t+"("+T+"|$)"))&&b(t,function(t){return e.test("string"==typeof t.className&&t.className||void 0!==t.getAttribute&&t.getAttribute("class")||"")})},ATTR:function(t,e,n){return function(r){var i=st.attr(r,t);return null==i?"!="===e:!e||(i+="","="===e?i===n:"!="===e?i!==n:"^="===e?n&&0===i.indexOf(n):"*="===e?n&&i.indexOf(n)>-1:"$="===e?n&&i.slice(-n.length)===n:"~="===e?(" "+i.replace(H," ")+" ").indexOf(n)>-1:"|="===e&&(i===n||i.slice(0,n.length+1)===n+"-"))}},CHILD:function(t,e,n,r,i){var o="nth"!==t.slice(0,3),a="last"!==t.slice(-4),s="of-type"===e;return 1===r&&0===i?function(t){return!!t.parentNode}:function(e,n,A){var l,c,u,g,h,d,f=o!==a?"nextSibling":"previousSibling",p=e.parentNode,I=s&&e.nodeName.toLowerCase(),C=!A&&!s,m=!1;if(p){if(o){for(;f;){for(g=e;g=g[f];)if(s?g.nodeName.toLowerCase()===I:1===g.nodeType)return!1;d=f="only"===t&&!d&&"nextSibling"}return!0}if(d=[a?p.firstChild:p.lastChild],a&&C){for(m=(h=(l=(c=(u=(g=p)[y]||(g[y]={}))[g.uniqueID]||(u[g.uniqueID]={}))[t]||[])[0]===v&&l[1])&&l[2],g=h&&p.childNodes[h];g=++h&&g&&g[f]||(m=h=0)||d.pop();)if(1===g.nodeType&&++m&&g===e){c[t]=[v,h,m];break}}else if(C&&(m=h=(l=(c=(u=(g=e)[y]||(g[y]={}))[g.uniqueID]||(u[g.uniqueID]={}))[t]||[])[0]===v&&l[1]),!1===m)for(;(g=++h&&g&&g[f]||(m=h=0)||d.pop())&&((s?g.nodeName.toLowerCase()!==I:1!==g.nodeType)||!++m||(C&&((c=(u=g[y]||(g[y]={}))[g.uniqueID]||(u[g.uniqueID]={}))[t]=[v,m]),g!==e)););return(m-=i)===r||m%r==0&&m/r>=0}}},PSEUDO:function(t,e){var n,i=r.pseudos[t]||r.setFilters[t.toLowerCase()]||st.error("unsupported pseudo: "+t);return i[y]?i(e):i.length>1?(n=[t,t,"",e],r.setFilters.hasOwnProperty(t.toLowerCase())?lt(function(t,n){for(var r,o=i(t,e),a=o.length;a--;)t[r=N(t,o[a])]=!(n[r]=o[a])}):function(t){return i(t,0,n)}):i}},pseudos:{not:lt(function(t){var e=[],n=[],r=s(t.replace(j,"$1"));return r[y]?lt(function(t,e,n,i){for(var o,a=r(t,null,i,[]),s=t.length;s--;)(o=a[s])&&(t[s]=!(e[s]=o))}):function(t,i,o){return e[0]=t,r(e,null,o,n),e[0]=null,!n.pop()}}),has:lt(function(t){return function(e){return st(t,e).length>0}}),contains:lt(function(t){return t=t.replace(et,nt),function(e){return(e.textContent||i(e)).indexOf(t)>-1}}),lang:lt(function(t){return J.test(t||"")||st.error("unsupported lang: "+t),t=t.replace(et,nt).toLowerCase(),function(e){var n;do{if(n=f?e.lang:e.getAttribute("xml:lang")||e.getAttribute("lang"))return(n=n.toLowerCase())===t||0===n.indexOf(t+"-")}while((e=e.parentNode)&&1===e.nodeType);return!1}}),target:function(e){var n=t.location&&t.location.hash;return n&&n.slice(1)===e.id},root:function(t){return t===d},focus:function(t){return t===h.activeElement&&(!h.hasFocus||h.hasFocus())&&!!(t.type||t.href||~t.tabIndex)},enabled:ft(!1),disabled:ft(!0),checked:function(t){var e=t.nodeName.toLowerCase();return"input"===e&&!!t.checked||"option"===e&&!!t.selected},selected:function(t){return t.parentNode&&t.parentNode.selectedIndex,!0===t.selected},empty:function(t){for(t=t.firstChild;t;t=t.nextSibling)if(t.nodeType<6)return!1;return!0},parent:function(t){return!r.pseudos.empty(t)},header:function(t){return X.test(t.nodeName)},input:function(t){return W.test(t.nodeName)},button:function(t){var e=t.nodeName.toLowerCase();return"input"===e&&"button"===t.type||"button"===e},text:function(t){var e;return"input"===t.nodeName.toLowerCase()&&"text"===t.type&&(null==(e=t.getAttribute("type"))||"text"===e.toLowerCase())},first:pt(function(){return[0]}),last:pt(function(t,e){return[e-1]}),eq:pt(function(t,e,n){return[n<0?n+e:n]}),even:pt(function(t,e){for(var n=0;n<e;n+=2)t.push(n);return t}),odd:pt(function(t,e){for(var n=1;n<e;n+=2)t.push(n);return t}),lt:pt(function(t,e,n){for(var r=n<0?n+e:n>e?e:n;--r>=0;)t.push(r);return t}),gt:pt(function(t,e,n){for(var r=n<0?n+e:n;++r<e;)t.push(r);return t})}}).pseudos.nth=r.pseudos.eq,{radio:!0,checkbox:!0,file:!0,password:!0,image:!0})r.pseudos[e]=ht(e);for(e in{submit:!0,reset:!0})r.pseudos[e]=dt(e);function Ct(){}function mt(t){for(var e=0,n=t.length,r="";e<n;e++)r+=t[e].value;return r}function yt(t,e,n){var r=e.dir,i=e.next,o=i||r,a=n&&"parentNode"===o,s=B++;return e.first?function(e,n,i){for(;e=e[r];)if(1===e.nodeType||a)return t(e,n,i);return!1}:function(e,n,A){var l,c,u,g=[v,s];if(A){for(;e=e[r];)if((1===e.nodeType||a)&&t(e,n,A))return!0}else for(;e=e[r];)if(1===e.nodeType||a)if(c=(u=e[y]||(e[y]={}))[e.uniqueID]||(u[e.uniqueID]={}),i&&i===e.nodeName.toLowerCase())e=e[r]||e;else{if((l=c[o])&&l[0]===v&&l[1]===s)return g[2]=l[2];if(c[o]=g,g[2]=t(e,n,A))return!0}return!1}}function Et(t){return t.length>1?function(e,n,r){for(var i=t.length;i--;)if(!t[i](e,n,r))return!1;return!0}:t[0]}function vt(t,e,n,r,i){for(var o,a=[],s=0,A=t.length,l=null!=e;s<A;s++)(o=t[s])&&(n&&!n(o,r,i)||(a.push(o),l&&e.push(s)));return a}function Bt(t,e,n,r,i,o){return r&&!r[y]&&(r=Bt(r)),i&&!i[y]&&(i=Bt(i,o)),lt(function(o,a,s,A){var l,c,u,g=[],h=[],d=a.length,f=o||function(t,e,n){for(var r=0,i=e.length;r<i;r++)st(t,e[r],n);return n}(e||"*",s.nodeType?[s]:s,[]),p=!t||!o&&e?f:vt(f,g,t,s,A),I=n?i||(o?t:d||r)?[]:a:p;if(n&&n(p,I,s,A),r)for(l=vt(I,h),r(l,[],s,A),c=l.length;c--;)(u=l[c])&&(I[h[c]]=!(p[h[c]]=u));if(o){if(i||t){if(i){for(l=[],c=I.length;c--;)(u=I[c])&&l.push(p[c]=u);i(null,I=[],l,A)}for(c=I.length;c--;)(u=I[c])&&(l=i?N(o,u):g[c])>-1&&(o[l]=!(a[l]=u))}}else I=vt(I===a?I.splice(d,I.length):I),i?i(null,a,I,A):L.apply(a,I)})}function bt(t){for(var e,n,i,o=t.length,a=r.relative[t[0].type],s=a||r.relative[" "],A=a?1:0,c=yt(function(t){return t===e},s,!0),u=yt(function(t){return N(e,t)>-1},s,!0),g=[function(t,n,r){var i=!a&&(r||n!==l)||((e=n).nodeType?c(t,n,r):u(t,n,r));return e=null,i}];A<o;A++)if(n=r.relative[t[A].type])g=[yt(Et(g),n)];else{if((n=r.filter[t[A].type].apply(null,t[A].matches))[y]){for(i=++A;i<o&&!r.relative[t[i].type];i++);return Bt(A>1&&Et(g),A>1&&mt(t.slice(0,A-1).concat({value:" "===t[A-2].type?"*":""})).replace(j,"$1"),n,A<i&&bt(t.slice(A,i)),i<o&&bt(t=t.slice(i)),i<o&&mt(t))}g.push(n)}return Et(g)}return Ct.prototype=r.filters=r.pseudos,r.setFilters=new Ct,a=st.tokenize=function(t,e){var n,i,o,a,s,A,l,c=Q[t+" "];if(c)return e?0:c.slice(0);for(s=t,A=[],l=r.preFilter;s;){for(a in n&&!(i=Y.exec(s))||(i&&(s=s.slice(i[0].length)||s),A.push(o=[])),n=!1,(i=P.exec(s))&&(n=i.shift(),o.push({value:n,type:i[0].replace(j," ")}),s=s.slice(n.length)),r.filter)!(i=q[a].exec(s))||l[a]&&!(i=l[a](i))||(n=i.shift(),o.push({value:n,type:a,matches:i}),s=s.slice(n.length));if(!n)break}return e?s.length:s?st.error(t):Q(t,A).slice(0)},s=st.compile=function(t,e){var n,i=[],o=[],s=w[t+" "];if(!s){for(e||(e=a(t)),n=e.length;n--;)(s=bt(e[n]))[y]?i.push(s):o.push(s);(s=w(t,function(t,e){var n=e.length>0,i=t.length>0,o=function(o,a,s,A,c){var u,d,p,I=0,C="0",m=o&&[],y=[],E=l,B=o||i&&r.find.TAG("*",c),b=v+=null==E?1:Math.random()||.1,Q=B.length;for(c&&(l=a===h||a||c);C!==Q&&null!=(u=B[C]);C++){if(i&&u){for(d=0,a||u.ownerDocument===h||(g(u),s=!f);p=t[d++];)if(p(u,a||h,s)){A.push(u);break}c&&(v=b)}n&&((u=!p&&u)&&I--,o&&m.push(u))}if(I+=C,n&&C!==I){for(d=0;p=e[d++];)p(m,y,a,s);if(o){if(I>0)for(;C--;)m[C]||y[C]||(y[C]=k.call(A));y=vt(y)}L.apply(A,y),c&&!o&&y.length>0&&I+e.length>1&&st.uniqueSort(A)}return c&&(v=b,l=E),m};return n?lt(o):o}(o,i))).selector=t}return s},A=st.select=function(t,e,n,i){var o,A,l,c,u,g="function"==typeof t&&t,h=!i&&a(t=g.selector||t);if(n=n||[],1===h.length){if((A=h[0]=h[0].slice(0)).length>2&&"ID"===(l=A[0]).type&&9===e.nodeType&&f&&r.relative[A[1].type]){if(!(e=(r.find.ID(l.matches[0].replace(et,nt),e)||[])[0]))return n;g&&(e=e.parentNode),t=t.slice(A.shift().value.length)}for(o=q.needsContext.test(t)?0:A.length;o--&&(l=A[o],!r.relative[c=l.type]);)if((u=r.find[c])&&(i=u(l.matches[0].replace(et,nt),tt.test(A[0].type)&&It(e.parentNode)||e))){if(A.splice(o,1),!(t=i.length&&mt(A)))return L.apply(n,i),n;break}}return(g||s(t,h))(i,e,!f,n,!e||tt.test(t)&&It(e.parentNode)||e),n},n.sortStable=y.split("").sort(_).join("")===y,n.detectDuplicates=!!u,g(),n.sortDetached=ct(function(t){return 1&t.compareDocumentPosition(h.createElement("fieldset"))}),ct(function(t){return t.innerHTML="<a href='#'></a>","#"===t.firstChild.getAttribute("href")})||ut("type|href|height|width",function(t,e,n){if(!n)return t.getAttribute(e,"type"===e.toLowerCase()?1:2)}),n.attributes&&ct(function(t){return t.innerHTML="<input/>",t.firstChild.setAttribute("value",""),""===t.firstChild.getAttribute("value")})||ut("value",function(t,e,n){if(!n&&"input"===t.nodeName.toLowerCase())return t.defaultValue}),ct(function(t){return null==t.getAttribute("disabled")})||ut(F,function(t,e,n){var r;if(!n)return!0===t[e]?e.toLowerCase():(r=t.getAttributeNode(e))&&r.specified?r.value:null}),st}(n);B.find=w,B.expr=w.selectors,B.expr[":"]=B.expr.pseudos,B.uniqueSort=B.unique=w.uniqueSort,B.text=w.getText,B.isXMLDoc=w.isXML,B.contains=w.contains,B.escapeSelector=w.escape;var x=function(t,e,n){for(var r=[],i=void 0!==n;(t=t[e])&&9!==t.nodeType;)if(1===t.nodeType){if(i&&B(t).is(n))break;r.push(t)}return r},_=function(t,e){for(var n=[];t;t=t.nextSibling)1===t.nodeType&&t!==e&&n.push(t);return n},D=B.expr.match.needsContext;function M(t,e){return t.nodeName&&t.nodeName.toLowerCase()===e.toLowerCase()}var k=/^<([a-z][^\/\0>:\x20\t\r\n\f]*)[\x20\t\r\n\f]*\/?>(?:<\/\1>|)$/i;function S(t,e,n){return C(e)?B.grep(t,function(t,r){return!!e.call(t,r,t)!==n}):e.nodeType?B.grep(t,function(t){return t===e!==n}):"string"!=typeof e?B.grep(t,function(t){return u.call(e,t)>-1!==n}):B.filter(e,t,n)}B.filter=function(t,e,n){var r=e[0];return n&&(t=":not("+t+")"),1===e.length&&1===r.nodeType?B.find.matchesSelector(r,t)?[r]:[]:B.find.matches(t,B.grep(e,function(t){return 1===t.nodeType}))},B.fn.extend({find:function(t){var e,n,r=this.length,i=this;if("string"!=typeof t)return this.pushStack(B(t).filter(function(){for(e=0;e<r;e++)if(B.contains(i[e],this))return!0}));for(n=this.pushStack([]),e=0;e<r;e++)B.find(t,i[e],n);return r>1?B.uniqueSort(n):n},filter:function(t){return this.pushStack(S(this,t||[],!1))},not:function(t){return this.pushStack(S(this,t||[],!0))},is:function(t){return!!S(this,"string"==typeof t&&D.test(t)?B(t):t||[],!1).length}});var L,R=/^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/;(B.fn.init=function(t,e,n){var r,i;if(!t)return this;if(n=n||L,"string"==typeof t){if(!(r="<"===t[0]&&">"===t[t.length-1]&&t.length>=3?[null,t,null]:R.exec(t))||!r[1]&&e)return!e||e.jquery?(e||n).find(t):this.constructor(e).find(t);if(r[1]){if(e=e instanceof B?e[0]:e,B.merge(this,B.parseHTML(r[1],e&&e.nodeType?e.ownerDocument||e:a,!0)),k.test(r[1])&&B.isPlainObject(e))for(r in e)C(this[r])?this[r](e[r]):this.attr(r,e[r]);return this}return(i=a.getElementById(r[2]))&&(this[0]=i,this.length=1),this}return t.nodeType?(this[0]=t,this.length=1,this):C(t)?void 0!==n.ready?n.ready(t):t(B):B.makeArray(t,this)}).prototype=B.fn,L=B(a);var N=/^(?:parents|prev(?:Until|All))/,F={children:!0,contents:!0,next:!0,prev:!0};function T(t,e){for(;(t=t[e])&&1!==t.nodeType;);return t}B.fn.extend({has:function(t){var e=B(t,this),n=e.length;return this.filter(function(){for(var t=0;t<n;t++)if(B.contains(this,e[t]))return!0})},closest:function(t,e){var n,r=0,i=this.length,o=[],a="string"!=typeof t&&B(t);if(!D.test(t))for(;r<i;r++)for(n=this[r];n&&n!==e;n=n.parentNode)if(n.nodeType<11&&(a?a.index(n)>-1:1===n.nodeType&&B.find.matchesSelector(n,t))){o.push(n);break}return this.pushStack(o.length>1?B.uniqueSort(o):o)},index:function(t){return t?"string"==typeof t?u.call(B(t),this[0]):u.call(this,t.jquery?t[0]:t):this[0]&&this[0].parentNode?this.first().prevAll().length:-1},add:function(t,e){return this.pushStack(B.uniqueSort(B.merge(this.get(),B(t,e))))},addBack:function(t){return this.add(null==t?this.prevObject:this.prevObject.filter(t))}}),B.each({parent:function(t){var e=t.parentNode;return e&&11!==e.nodeType?e:null},parents:function(t){return x(t,"parentNode")},parentsUntil:function(t,e,n){return x(t,"parentNode",n)},next:function(t){return T(t,"nextSibling")},prev:function(t){return T(t,"previousSibling")},nextAll:function(t){return x(t,"nextSibling")},prevAll:function(t){return x(t,"previousSibling")},nextUntil:function(t,e,n){return x(t,"nextSibling",n)},prevUntil:function(t,e,n){return x(t,"previousSibling",n)},siblings:function(t){return _((t.parentNode||{}).firstChild,t)},children:function(t){return _(t.firstChild)},contents:function(t){return void 0!==t.contentDocument?t.contentDocument:(M(t,"template")&&(t=t.content||t),B.merge([],t.childNodes))}},function(t,e){B.fn[t]=function(n,r){var i=B.map(this,e,n);return"Until"!==t.slice(-5)&&(r=n),r&&"string"==typeof r&&(i=B.filter(r,i)),this.length>1&&(F[t]||B.uniqueSort(i),N.test(t)&&i.reverse()),this.pushStack(i)}});var U=/[^\x20\t\r\n\f]+/g;function O(t){return t}function G(t){throw t}function H(t,e,n,r){var i;try{t&&C(i=t.promise)?i.call(t).done(e).fail(n):t&&C(i=t.then)?i.call(t,e,n):e.apply(void 0,[t].slice(r))}catch(t){n.apply(void 0,[t])}}B.Callbacks=function(t){t="string"==typeof t?function(t){var e={};return B.each(t.match(U)||[],function(t,n){e[n]=!0}),e}(t):B.extend({},t);var e,n,r,i,o=[],a=[],s=-1,A=function(){for(i=i||t.once,r=e=!0;a.length;s=-1)for(n=a.shift();++s<o.length;)!1===o[s].apply(n[0],n[1])&&t.stopOnFalse&&(s=o.length,n=!1);t.memory||(n=!1),e=!1,i&&(o=n?[]:"")},l={add:function(){return o&&(n&&!e&&(s=o.length-1,a.push(n)),function e(n){B.each(n,function(n,r){C(r)?t.unique&&l.has(r)||o.push(r):r&&r.length&&"string"!==v(r)&&e(r)})}(arguments),n&&!e&&A()),this},remove:function(){return B.each(arguments,function(t,e){for(var n;(n=B.inArray(e,o,n))>-1;)o.splice(n,1),n<=s&&s--}),this},has:function(t){return t?B.inArray(t,o)>-1:o.length>0},empty:function(){return o&&(o=[]),this},disable:function(){return i=a=[],o=n="",this},disabled:function(){return!o},lock:function(){return i=a=[],n||e||(o=n=""),this},locked:function(){return!!i},fireWith:function(t,n){return i||(n=[t,(n=n||[]).slice?n.slice():n],a.push(n),e||A()),this},fire:function(){return l.fireWith(this,arguments),this},fired:function(){return!!r}};return l},B.extend({Deferred:function(t){var e=[["notify","progress",B.Callbacks("memory"),B.Callbacks("memory"),2],["resolve","done",B.Callbacks("once memory"),B.Callbacks("once memory"),0,"resolved"],["reject","fail",B.Callbacks("once memory"),B.Callbacks("once memory"),1,"rejected"]],r="pending",i={state:function(){return r},always:function(){return o.done(arguments).fail(arguments),this},catch:function(t){return i.then(null,t)},pipe:function(){var t=arguments;return B.Deferred(function(n){B.each(e,function(e,r){var i=C(t[r[4]])&&t[r[4]];o[r[1]](function(){var t=i&&i.apply(this,arguments);t&&C(t.promise)?t.promise().progress(n.notify).done(n.resolve).fail(n.reject):n[r[0]+"With"](this,i?[t]:arguments)})}),t=null}).promise()},then:function(t,r,i){var o=0;function a(t,e,r,i){return function(){var s=this,A=arguments,l=function(){var n,l;if(!(t<o)){if((n=r.apply(s,A))===e.promise())throw new TypeError("Thenable self-resolution");l=n&&("object"==typeof n||"function"==typeof n)&&n.then,C(l)?i?l.call(n,a(o,e,O,i),a(o,e,G,i)):(o++,l.call(n,a(o,e,O,i),a(o,e,G,i),a(o,e,O,e.notifyWith))):(r!==O&&(s=void 0,A=[n]),(i||e.resolveWith)(s,A))}},c=i?l:function(){try{l()}catch(n){B.Deferred.exceptionHook&&B.Deferred.exceptionHook(n,c.stackTrace),t+1>=o&&(r!==G&&(s=void 0,A=[n]),e.rejectWith(s,A))}};t?c():(B.Deferred.getStackHook&&(c.stackTrace=B.Deferred.getStackHook()),n.setTimeout(c))}}return B.Deferred(function(n){e[0][3].add(a(0,n,C(i)?i:O,n.notifyWith)),e[1][3].add(a(0,n,C(t)?t:O)),e[2][3].add(a(0,n,C(r)?r:G))}).promise()},promise:function(t){return null!=t?B.extend(t,i):i}},o={};return B.each(e,function(t,n){var a=n[2],s=n[5];i[n[1]]=a.add,s&&a.add(function(){r=s},e[3-t][2].disable,e[3-t][3].disable,e[0][2].lock,e[0][3].lock),a.add(n[3].fire),o[n[0]]=function(){return o[n[0]+"With"](this===o?void 0:this,arguments),this},o[n[0]+"With"]=a.fireWith}),i.promise(o),t&&t.call(o,o),o},when:function(t){var e=arguments.length,n=e,r=Array(n),i=A.call(arguments),o=B.Deferred(),a=function(t){return function(n){r[t]=this,i[t]=arguments.length>1?A.call(arguments):n,--e||o.resolveWith(r,i)}};if(e<=1&&(H(t,o.done(a(n)).resolve,o.reject,!e),"pending"===o.state()||C(i[n]&&i[n].then)))return o.then();for(;n--;)H(i[n],a(n),o.reject);return o.promise()}});var j=/^(Eval|Internal|Range|Reference|Syntax|Type|URI)Error$/;B.Deferred.exceptionHook=function(t,e){n.console&&n.console.warn&&t&&j.test(t.name)&&n.console.warn("jQuery.Deferred exception: "+t.message,t.stack,e)},B.readyException=function(t){n.setTimeout(function(){throw t})};var Y=B.Deferred();function P(){a.removeEventListener("DOMContentLoaded",P),n.removeEventListener("load",P),B.ready()}B.fn.ready=function(t){return Y.then(t).catch(function(t){B.readyException(t)}),this},B.extend({isReady:!1,readyWait:1,ready:function(t){(!0===t?--B.readyWait:B.isReady)||(B.isReady=!0,!0!==t&&--B.readyWait>0||Y.resolveWith(a,[B]))}}),B.ready.then=Y.then,"complete"===a.readyState||"loading"!==a.readyState&&!a.documentElement.doScroll?n.setTimeout(B.ready):(a.addEventListener("DOMContentLoaded",P),n.addEventListener("load",P));var K=function(t,e,n,r,i,o,a){var s=0,A=t.length,l=null==n;if("object"===v(n))for(s in i=!0,n)K(t,e,s,n[s],!0,o,a);else if(void 0!==r&&(i=!0,C(r)||(a=!0),l&&(a?(e.call(t,r),e=null):(l=e,e=function(t,e,n){return l.call(B(t),n)})),e))for(;s<A;s++)e(t[s],n,a?r:r.call(t[s],s,e(t[s],n)));return i?t:l?e.call(t):A?e(t[0],n):o},z=/^-ms-/,J=/-([a-z])/g;function q(t,e){return e.toUpperCase()}function V(t){return t.replace(z,"ms-").replace(J,q)}var W=function(t){return 1===t.nodeType||9===t.nodeType||!+t.nodeType};function X(){this.expando=B.expando+X.uid++}X.uid=1,X.prototype={cache:function(t){var e=t[this.expando];return e||(e={},W(t)&&(t.nodeType?t[this.expando]=e:Object.defineProperty(t,this.expando,{value:e,configurable:!0}))),e},set:function(t,e,n){var r,i=this.cache(t);if("string"==typeof e)i[V(e)]=n;else for(r in e)i[V(r)]=e[r];return i},get:function(t,e){return void 0===e?this.cache(t):t[this.expando]&&t[this.expando][V(e)]},access:function(t,e,n){return void 0===e||e&&"string"==typeof e&&void 0===n?this.get(t,e):(this.set(t,e,n),void 0!==n?n:e)},remove:function(t,e){var n,r=t[this.expando];if(void 0!==r){if(void 0!==e){n=(e=Array.isArray(e)?e.map(V):(e=V(e))in r?[e]:e.match(U)||[]).length;for(;n--;)delete r[e[n]]}(void 0===e||B.isEmptyObject(r))&&(t.nodeType?t[this.expando]=void 0:delete t[this.expando])}},hasData:function(t){var e=t[this.expando];return void 0!==e&&!B.isEmptyObject(e)}};var Z=new X,$=new X,tt=/^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,et=/[A-Z]/g;function nt(t,e,n){var r;if(void 0===n&&1===t.nodeType)if(r="data-"+e.replace(et,"-$&").toLowerCase(),"string"==typeof(n=t.getAttribute(r))){try{n=function(t){return"true"===t||"false"!==t&&("null"===t?null:t===+t+""?+t:tt.test(t)?JSON.parse(t):t)}(n)}catch(t){}$.set(t,e,n)}else n=void 0;return n}B.extend({hasData:function(t){return $.hasData(t)||Z.hasData(t)},data:function(t,e,n){return $.access(t,e,n)},removeData:function(t,e){$.remove(t,e)},_data:function(t,e,n){return Z.access(t,e,n)},_removeData:function(t,e){Z.remove(t,e)}}),B.fn.extend({data:function(t,e){var n,r,i,o=this[0],a=o&&o.attributes;if(void 0===t){if(this.length&&(i=$.get(o),1===o.nodeType&&!Z.get(o,"hasDataAttrs"))){for(n=a.length;n--;)a[n]&&0===(r=a[n].name).indexOf("data-")&&(r=V(r.slice(5)),nt(o,r,i[r]));Z.set(o,"hasDataAttrs",!0)}return i}return"object"==typeof t?this.each(function(){$.set(this,t)}):K(this,function(e){var n;if(o&&void 0===e)return void 0!==(n=$.get(o,t))?n:void 0!==(n=nt(o,t))?n:void 0;this.each(function(){$.set(this,t,e)})},null,e,arguments.length>1,null,!0)},removeData:function(t){return this.each(function(){$.remove(this,t)})}}),B.extend({queue:function(t,e,n){var r;if(t)return e=(e||"fx")+"queue",r=Z.get(t,e),n&&(!r||Array.isArray(n)?r=Z.access(t,e,B.makeArray(n)):r.push(n)),r||[]},dequeue:function(t,e){e=e||"fx";var n=B.queue(t,e),r=n.length,i=n.shift(),o=B._queueHooks(t,e);"inprogress"===i&&(i=n.shift(),r--),i&&("fx"===e&&n.unshift("inprogress"),delete o.stop,i.call(t,function(){B.dequeue(t,e)},o)),!r&&o&&o.empty.fire()},_queueHooks:function(t,e){var n=e+"queueHooks";return Z.get(t,n)||Z.access(t,n,{empty:B.Callbacks("once memory").add(function(){Z.remove(t,[e+"queue",n])})})}}),B.fn.extend({queue:function(t,e){var n=2;return"string"!=typeof t&&(e=t,t="fx",n--),arguments.length<n?B.queue(this[0],t):void 0===e?this:this.each(function(){var n=B.queue(this,t,e);B._queueHooks(this,t),"fx"===t&&"inprogress"!==n[0]&&B.dequeue(this,t)})},dequeue:function(t){return this.each(function(){B.dequeue(this,t)})},clearQueue:function(t){return this.queue(t||"fx",[])},promise:function(t,e){var n,r=1,i=B.Deferred(),o=this,a=this.length,s=function(){--r||i.resolveWith(o,[o])};for("string"!=typeof t&&(e=t,t=void 0),t=t||"fx";a--;)(n=Z.get(o[a],t+"queueHooks"))&&n.empty&&(r++,n.empty.add(s));return s(),i.promise(e)}});var rt=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,it=new RegExp("^(?:([+-])=|)("+rt+")([a-z%]*)$","i"),ot=["Top","Right","Bottom","Left"],at=a.documentElement,st=function(t){return B.contains(t.ownerDocument,t)},At={composed:!0};at.getRootNode&&(st=function(t){return B.contains(t.ownerDocument,t)||t.getRootNode(At)===t.ownerDocument});var lt=function(t,e){return"none"===(t=e||t).style.display||""===t.style.display&&st(t)&&"none"===B.css(t,"display")},ct=function(t,e,n,r){var i,o,a={};for(o in e)a[o]=t.style[o],t.style[o]=e[o];for(o in i=n.apply(t,r||[]),e)t.style[o]=a[o];return i};function ut(t,e,n,r){var i,o,a=20,s=r?function(){return r.cur()}:function(){return B.css(t,e,"")},A=s(),l=n&&n[3]||(B.cssNumber[e]?"":"px"),c=t.nodeType&&(B.cssNumber[e]||"px"!==l&&+A)&&it.exec(B.css(t,e));if(c&&c[3]!==l){for(A/=2,l=l||c[3],c=+A||1;a--;)B.style(t,e,c+l),(1-o)*(1-(o=s()/A||.5))<=0&&(a=0),c/=o;c*=2,B.style(t,e,c+l),n=n||[]}return n&&(c=+c||+A||0,i=n[1]?c+(n[1]+1)*n[2]:+n[2],r&&(r.unit=l,r.start=c,r.end=i)),i}var gt={};function ht(t){var e,n=t.ownerDocument,r=t.nodeName,i=gt[r];return i||(e=n.body.appendChild(n.createElement(r)),i=B.css(e,"display"),e.parentNode.removeChild(e),"none"===i&&(i="block"),gt[r]=i,i)}function dt(t,e){for(var n,r,i=[],o=0,a=t.length;o<a;o++)(r=t[o]).style&&(n=r.style.display,e?("none"===n&&(i[o]=Z.get(r,"display")||null,i[o]||(r.style.display="")),""===r.style.display&&lt(r)&&(i[o]=ht(r))):"none"!==n&&(i[o]="none",Z.set(r,"display",n)));for(o=0;o<a;o++)null!=i[o]&&(t[o].style.display=i[o]);return t}B.fn.extend({show:function(){return dt(this,!0)},hide:function(){return dt(this)},toggle:function(t){return"boolean"==typeof t?t?this.show():this.hide():this.each(function(){lt(this)?B(this).show():B(this).hide()})}});var ft=/^(?:checkbox|radio)$/i,pt=/<([a-z][^\/\0>\x20\t\r\n\f]*)/i,It=/^$|^module$|\/(?:java|ecma)script/i,Ct={option:[1,"<select multiple='multiple'>","</select>"],thead:[1,"<table>","</table>"],col:[2,"<table><colgroup>","</colgroup></table>"],tr:[2,"<table><tbody>","</tbody></table>"],td:[3,"<table><tbody><tr>","</tr></tbody></table>"],_default:[0,"",""]};function mt(t,e){var n;return n=void 0!==t.getElementsByTagName?t.getElementsByTagName(e||"*"):void 0!==t.querySelectorAll?t.querySelectorAll(e||"*"):[],void 0===e||e&&M(t,e)?B.merge([t],n):n}function yt(t,e){for(var n=0,r=t.length;n<r;n++)Z.set(t[n],"globalEval",!e||Z.get(e[n],"globalEval"))}Ct.optgroup=Ct.option,Ct.tbody=Ct.tfoot=Ct.colgroup=Ct.caption=Ct.thead,Ct.th=Ct.td;var Et,vt,Bt=/<|&#?\w+;/;function bt(t,e,n,r,i){for(var o,a,s,A,l,c,u=e.createDocumentFragment(),g=[],h=0,d=t.length;h<d;h++)if((o=t[h])||0===o)if("object"===v(o))B.merge(g,o.nodeType?[o]:o);else if(Bt.test(o)){for(a=a||u.appendChild(e.createElement("div")),s=(pt.exec(o)||["",""])[1].toLowerCase(),A=Ct[s]||Ct._default,a.innerHTML=A[1]+B.htmlPrefilter(o)+A[2],c=A[0];c--;)a=a.lastChild;B.merge(g,a.childNodes),(a=u.firstChild).textContent=""}else g.push(e.createTextNode(o));for(u.textContent="",h=0;o=g[h++];)if(r&&B.inArray(o,r)>-1)i&&i.push(o);else if(l=st(o),a=mt(u.appendChild(o),"script"),l&&yt(a),n)for(c=0;o=a[c++];)It.test(o.type||"")&&n.push(o);return u}Et=a.createDocumentFragment().appendChild(a.createElement("div")),(vt=a.createElement("input")).setAttribute("type","radio"),vt.setAttribute("checked","checked"),vt.setAttribute("name","t"),Et.appendChild(vt),I.checkClone=Et.cloneNode(!0).cloneNode(!0).lastChild.checked,Et.innerHTML="<textarea>x</textarea>",I.noCloneChecked=!!Et.cloneNode(!0).lastChild.defaultValue;var Qt=/^key/,wt=/^(?:mouse|pointer|contextmenu|drag|drop)|click/,xt=/^([^.]*)(?:\.(.+)|)/;function _t(){return!0}function Dt(){return!1}function Mt(t,e){return t===function(){try{return a.activeElement}catch(t){}}()==("focus"===e)}function kt(t,e,n,r,i,o){var a,s;if("object"==typeof e){for(s in"string"!=typeof n&&(r=r||n,n=void 0),e)kt(t,s,n,r,e[s],o);return t}if(null==r&&null==i?(i=n,r=n=void 0):null==i&&("string"==typeof n?(i=r,r=void 0):(i=r,r=n,n=void 0)),!1===i)i=Dt;else if(!i)return t;return 1===o&&(a=i,(i=function(t){return B().off(t),a.apply(this,arguments)}).guid=a.guid||(a.guid=B.guid++)),t.each(function(){B.event.add(this,e,i,r,n)})}function St(t,e,n){n?(Z.set(t,e,!1),B.event.add(t,e,{namespace:!1,handler:function(t){var r,i,o=Z.get(this,e);if(1&t.isTrigger&&this[e]){if(o.length)(B.event.special[e]||{}).delegateType&&t.stopPropagation();else if(o=A.call(arguments),Z.set(this,e,o),r=n(this,e),this[e](),o!==(i=Z.get(this,e))||r?Z.set(this,e,!1):i={},o!==i)return t.stopImmediatePropagation(),t.preventDefault(),i.value}else o.length&&(Z.set(this,e,{value:B.event.trigger(B.extend(o[0],B.Event.prototype),o.slice(1),this)}),t.stopImmediatePropagation())}})):void 0===Z.get(t,e)&&B.event.add(t,e,_t)}B.event={global:{},add:function(t,e,n,r,i){var o,a,s,A,l,c,u,g,h,d,f,p=Z.get(t);if(p)for(n.handler&&(n=(o=n).handler,i=o.selector),i&&B.find.matchesSelector(at,i),n.guid||(n.guid=B.guid++),(A=p.events)||(A=p.events={}),(a=p.handle)||(a=p.handle=function(e){return void 0!==B&&B.event.triggered!==e.type?B.event.dispatch.apply(t,arguments):void 0}),l=(e=(e||"").match(U)||[""]).length;l--;)h=f=(s=xt.exec(e[l])||[])[1],d=(s[2]||"").split(".").sort(),h&&(u=B.event.special[h]||{},h=(i?u.delegateType:u.bindType)||h,u=B.event.special[h]||{},c=B.extend({type:h,origType:f,data:r,handler:n,guid:n.guid,selector:i,needsContext:i&&B.expr.match.needsContext.test(i),namespace:d.join(".")},o),(g=A[h])||((g=A[h]=[]).delegateCount=0,u.setup&&!1!==u.setup.call(t,r,d,a)||t.addEventListener&&t.addEventListener(h,a)),u.add&&(u.add.call(t,c),c.handler.guid||(c.handler.guid=n.guid)),i?g.splice(g.delegateCount++,0,c):g.push(c),B.event.global[h]=!0)},remove:function(t,e,n,r,i){var o,a,s,A,l,c,u,g,h,d,f,p=Z.hasData(t)&&Z.get(t);if(p&&(A=p.events)){for(l=(e=(e||"").match(U)||[""]).length;l--;)if(h=f=(s=xt.exec(e[l])||[])[1],d=(s[2]||"").split(".").sort(),h){for(u=B.event.special[h]||{},g=A[h=(r?u.delegateType:u.bindType)||h]||[],s=s[2]&&new RegExp("(^|\\.)"+d.join("\\.(?:.*\\.|)")+"(\\.|$)"),a=o=g.length;o--;)c=g[o],!i&&f!==c.origType||n&&n.guid!==c.guid||s&&!s.test(c.namespace)||r&&r!==c.selector&&("**"!==r||!c.selector)||(g.splice(o,1),c.selector&&g.delegateCount--,u.remove&&u.remove.call(t,c));a&&!g.length&&(u.teardown&&!1!==u.teardown.call(t,d,p.handle)||B.removeEvent(t,h,p.handle),delete A[h])}else for(h in A)B.event.remove(t,h+e[l],n,r,!0);B.isEmptyObject(A)&&Z.remove(t,"handle events")}},dispatch:function(t){var e,n,r,i,o,a,s=B.event.fix(t),A=new Array(arguments.length),l=(Z.get(this,"events")||{})[s.type]||[],c=B.event.special[s.type]||{};for(A[0]=s,e=1;e<arguments.length;e++)A[e]=arguments[e];if(s.delegateTarget=this,!c.preDispatch||!1!==c.preDispatch.call(this,s)){for(a=B.event.handlers.call(this,s,l),e=0;(i=a[e++])&&!s.isPropagationStopped();)for(s.currentTarget=i.elem,n=0;(o=i.handlers[n++])&&!s.isImmediatePropagationStopped();)s.rnamespace&&!1!==o.namespace&&!s.rnamespace.test(o.namespace)||(s.handleObj=o,s.data=o.data,void 0!==(r=((B.event.special[o.origType]||{}).handle||o.handler).apply(i.elem,A))&&!1===(s.result=r)&&(s.preventDefault(),s.stopPropagation()));return c.postDispatch&&c.postDispatch.call(this,s),s.result}},handlers:function(t,e){var n,r,i,o,a,s=[],A=e.delegateCount,l=t.target;if(A&&l.nodeType&&!("click"===t.type&&t.button>=1))for(;l!==this;l=l.parentNode||this)if(1===l.nodeType&&("click"!==t.type||!0!==l.disabled)){for(o=[],a={},n=0;n<A;n++)void 0===a[i=(r=e[n]).selector+" "]&&(a[i]=r.needsContext?B(i,this).index(l)>-1:B.find(i,this,null,[l]).length),a[i]&&o.push(r);o.length&&s.push({elem:l,handlers:o})}return l=this,A<e.length&&s.push({elem:l,handlers:e.slice(A)}),s},addProp:function(t,e){Object.defineProperty(B.Event.prototype,t,{enumerable:!0,configurable:!0,get:C(e)?function(){if(this.originalEvent)return e(this.originalEvent)}:function(){if(this.originalEvent)return this.originalEvent[t]},set:function(e){Object.defineProperty(this,t,{enumerable:!0,configurable:!0,writable:!0,value:e})}})},fix:function(t){return t[B.expando]?t:new B.Event(t)},special:{load:{noBubble:!0},click:{setup:function(t){var e=this||t;return ft.test(e.type)&&e.click&&M(e,"input")&&St(e,"click",_t),!1},trigger:function(t){var e=this||t;return ft.test(e.type)&&e.click&&M(e,"input")&&St(e,"click"),!0},_default:function(t){var e=t.target;return ft.test(e.type)&&e.click&&M(e,"input")&&Z.get(e,"click")||M(e,"a")}},beforeunload:{postDispatch:function(t){void 0!==t.result&&t.originalEvent&&(t.originalEvent.returnValue=t.result)}}}},B.removeEvent=function(t,e,n){t.removeEventListener&&t.removeEventListener(e,n)},B.Event=function(t,e){if(!(this instanceof B.Event))return new B.Event(t,e);t&&t.type?(this.originalEvent=t,this.type=t.type,this.isDefaultPrevented=t.defaultPrevented||void 0===t.defaultPrevented&&!1===t.returnValue?_t:Dt,this.target=t.target&&3===t.target.nodeType?t.target.parentNode:t.target,this.currentTarget=t.currentTarget,this.relatedTarget=t.relatedTarget):this.type=t,e&&B.extend(this,e),this.timeStamp=t&&t.timeStamp||Date.now(),this[B.expando]=!0},B.Event.prototype={constructor:B.Event,isDefaultPrevented:Dt,isPropagationStopped:Dt,isImmediatePropagationStopped:Dt,isSimulated:!1,preventDefault:function(){var t=this.originalEvent;this.isDefaultPrevented=_t,t&&!this.isSimulated&&t.preventDefault()},stopPropagation:function(){var t=this.originalEvent;this.isPropagationStopped=_t,t&&!this.isSimulated&&t.stopPropagation()},stopImmediatePropagation:function(){var t=this.originalEvent;this.isImmediatePropagationStopped=_t,t&&!this.isSimulated&&t.stopImmediatePropagation(),this.stopPropagation()}},B.each({altKey:!0,bubbles:!0,cancelable:!0,changedTouches:!0,ctrlKey:!0,detail:!0,eventPhase:!0,metaKey:!0,pageX:!0,pageY:!0,shiftKey:!0,view:!0,char:!0,code:!0,charCode:!0,key:!0,keyCode:!0,button:!0,buttons:!0,clientX:!0,clientY:!0,offsetX:!0,offsetY:!0,pointerId:!0,pointerType:!0,screenX:!0,screenY:!0,targetTouches:!0,toElement:!0,touches:!0,which:function(t){var e=t.button;return null==t.which&&Qt.test(t.type)?null!=t.charCode?t.charCode:t.keyCode:!t.which&&void 0!==e&&wt.test(t.type)?1&e?1:2&e?3:4&e?2:0:t.which}},B.event.addProp),B.each({focus:"focusin",blur:"focusout"},function(t,e){B.event.special[t]={setup:function(){return St(this,t,Mt),!1},trigger:function(){return St(this,t),!0},delegateType:e}}),B.each({mouseenter:"mouseover",mouseleave:"mouseout",pointerenter:"pointerover",pointerleave:"pointerout"},function(t,e){B.event.special[t]={delegateType:e,bindType:e,handle:function(t){var n,r=this,i=t.relatedTarget,o=t.handleObj;return i&&(i===r||B.contains(r,i))||(t.type=o.origType,n=o.handler.apply(this,arguments),t.type=e),n}}}),B.fn.extend({on:function(t,e,n,r){return kt(this,t,e,n,r)},one:function(t,e,n,r){return kt(this,t,e,n,r,1)},off:function(t,e,n){var r,i;if(t&&t.preventDefault&&t.handleObj)return r=t.handleObj,B(t.delegateTarget).off(r.namespace?r.origType+"."+r.namespace:r.origType,r.selector,r.handler),this;if("object"==typeof t){for(i in t)this.off(i,e,t[i]);return this}return!1!==e&&"function"!=typeof e||(n=e,e=void 0),!1===n&&(n=Dt),this.each(function(){B.event.remove(this,t,n,e)})}});var Lt=/<(?!area|br|col|embed|hr|img|input|link|meta|param)(([a-z][^\/\0>\x20\t\r\n\f]*)[^>]*)\/>/gi,Rt=/<script|<style|<link/i,Nt=/checked\s*(?:[^=]|=\s*.checked.)/i,Ft=/^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g;function Tt(t,e){return M(t,"table")&&M(11!==e.nodeType?e:e.firstChild,"tr")&&B(t).children("tbody")[0]||t}function Ut(t){return t.type=(null!==t.getAttribute("type"))+"/"+t.type,t}function Ot(t){return"true/"===(t.type||"").slice(0,5)?t.type=t.type.slice(5):t.removeAttribute("type"),t}function Gt(t,e){var n,r,i,o,a,s,A,l;if(1===e.nodeType){if(Z.hasData(t)&&(o=Z.access(t),a=Z.set(e,o),l=o.events))for(i in delete a.handle,a.events={},l)for(n=0,r=l[i].length;n<r;n++)B.event.add(e,i,l[i][n]);$.hasData(t)&&(s=$.access(t),A=B.extend({},s),$.set(e,A))}}function Ht(t,e){var n=e.nodeName.toLowerCase();"input"===n&&ft.test(t.type)?e.checked=t.checked:"input"!==n&&"textarea"!==n||(e.defaultValue=t.defaultValue)}function jt(t,e,n,r){e=l.apply([],e);var i,o,a,s,A,c,u=0,g=t.length,h=g-1,d=e[0],f=C(d);if(f||g>1&&"string"==typeof d&&!I.checkClone&&Nt.test(d))return t.each(function(i){var o=t.eq(i);f&&(e[0]=d.call(this,i,o.html())),jt(o,e,n,r)});if(g&&(o=(i=bt(e,t[0].ownerDocument,!1,t,r)).firstChild,1===i.childNodes.length&&(i=o),o||r)){for(s=(a=B.map(mt(i,"script"),Ut)).length;u<g;u++)A=i,u!==h&&(A=B.clone(A,!0,!0),s&&B.merge(a,mt(A,"script"))),n.call(t[u],A,u);if(s)for(c=a[a.length-1].ownerDocument,B.map(a,Ot),u=0;u<s;u++)A=a[u],It.test(A.type||"")&&!Z.access(A,"globalEval")&&B.contains(c,A)&&(A.src&&"module"!==(A.type||"").toLowerCase()?B._evalUrl&&!A.noModule&&B._evalUrl(A.src,{nonce:A.nonce||A.getAttribute("nonce")}):E(A.textContent.replace(Ft,""),A,c))}return t}function Yt(t,e,n){for(var r,i=e?B.filter(e,t):t,o=0;null!=(r=i[o]);o++)n||1!==r.nodeType||B.cleanData(mt(r)),r.parentNode&&(n&&st(r)&&yt(mt(r,"script")),r.parentNode.removeChild(r));return t}B.extend({htmlPrefilter:function(t){return t.replace(Lt,"<$1></$2>")},clone:function(t,e,n){var r,i,o,a,s=t.cloneNode(!0),A=st(t);if(!(I.noCloneChecked||1!==t.nodeType&&11!==t.nodeType||B.isXMLDoc(t)))for(a=mt(s),r=0,i=(o=mt(t)).length;r<i;r++)Ht(o[r],a[r]);if(e)if(n)for(o=o||mt(t),a=a||mt(s),r=0,i=o.length;r<i;r++)Gt(o[r],a[r]);else Gt(t,s);return(a=mt(s,"script")).length>0&&yt(a,!A&&mt(t,"script")),s},cleanData:function(t){for(var e,n,r,i=B.event.special,o=0;void 0!==(n=t[o]);o++)if(W(n)){if(e=n[Z.expando]){if(e.events)for(r in e.events)i[r]?B.event.remove(n,r):B.removeEvent(n,r,e.handle);n[Z.expando]=void 0}n[$.expando]&&(n[$.expando]=void 0)}}}),B.fn.extend({detach:function(t){return Yt(this,t,!0)},remove:function(t){return Yt(this,t)},text:function(t){return K(this,function(t){return void 0===t?B.text(this):this.empty().each(function(){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||(this.textContent=t)})},null,t,arguments.length)},append:function(){return jt(this,arguments,function(t){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||Tt(this,t).appendChild(t)})},prepend:function(){return jt(this,arguments,function(t){if(1===this.nodeType||11===this.nodeType||9===this.nodeType){var e=Tt(this,t);e.insertBefore(t,e.firstChild)}})},before:function(){return jt(this,arguments,function(t){this.parentNode&&this.parentNode.insertBefore(t,this)})},after:function(){return jt(this,arguments,function(t){this.parentNode&&this.parentNode.insertBefore(t,this.nextSibling)})},empty:function(){for(var t,e=0;null!=(t=this[e]);e++)1===t.nodeType&&(B.cleanData(mt(t,!1)),t.textContent="");return this},clone:function(t,e){return t=null!=t&&t,e=null==e?t:e,this.map(function(){return B.clone(this,t,e)})},html:function(t){return K(this,function(t){var e=this[0]||{},n=0,r=this.length;if(void 0===t&&1===e.nodeType)return e.innerHTML;if("string"==typeof t&&!Rt.test(t)&&!Ct[(pt.exec(t)||["",""])[1].toLowerCase()]){t=B.htmlPrefilter(t);try{for(;n<r;n++)1===(e=this[n]||{}).nodeType&&(B.cleanData(mt(e,!1)),e.innerHTML=t);e=0}catch(t){}}e&&this.empty().append(t)},null,t,arguments.length)},replaceWith:function(){var t=[];return jt(this,arguments,function(e){var n=this.parentNode;B.inArray(this,t)<0&&(B.cleanData(mt(this)),n&&n.replaceChild(e,this))},t)}}),B.each({appendTo:"append",prependTo:"prepend",insertBefore:"before",insertAfter:"after",replaceAll:"replaceWith"},function(t,e){B.fn[t]=function(t){for(var n,r=[],i=B(t),o=i.length-1,a=0;a<=o;a++)n=a===o?this:this.clone(!0),B(i[a])[e](n),c.apply(r,n.get());return this.pushStack(r)}});var Pt=new RegExp("^("+rt+")(?!px)[a-z%]+$","i"),Kt=function(t){var e=t.ownerDocument.defaultView;return e&&e.opener||(e=n),e.getComputedStyle(t)},zt=new RegExp(ot.join("|"),"i");function Jt(t,e,n){var r,i,o,a,s=t.style;return(n=n||Kt(t))&&(""!==(a=n.getPropertyValue(e)||n[e])||st(t)||(a=B.style(t,e)),!I.pixelBoxStyles()&&Pt.test(a)&&zt.test(e)&&(r=s.width,i=s.minWidth,o=s.maxWidth,s.minWidth=s.maxWidth=s.width=a,a=n.width,s.width=r,s.minWidth=i,s.maxWidth=o)),void 0!==a?a+"":a}function qt(t,e){return{get:function(){if(!t())return(this.get=e).apply(this,arguments);delete this.get}}}!function(){function t(){if(c){l.style.cssText="position:absolute;left:-11111px;width:60px;margin-top:1px;padding:0;border:0",c.style.cssText="position:relative;display:block;box-sizing:border-box;overflow:scroll;margin:auto;border:1px;padding:1px;width:60%;top:1%",at.appendChild(l).appendChild(c);var t=n.getComputedStyle(c);r="1%"!==t.top,A=12===e(t.marginLeft),c.style.right="60%",s=36===e(t.right),i=36===e(t.width),c.style.position="absolute",o=12===e(c.offsetWidth/3),at.removeChild(l),c=null}}function e(t){return Math.round(parseFloat(t))}var r,i,o,s,A,l=a.createElement("div"),c=a.createElement("div");c.style&&(c.style.backgroundClip="content-box",c.cloneNode(!0).style.backgroundClip="",I.clearCloneStyle="content-box"===c.style.backgroundClip,B.extend(I,{boxSizingReliable:function(){return t(),i},pixelBoxStyles:function(){return t(),s},pixelPosition:function(){return t(),r},reliableMarginLeft:function(){return t(),A},scrollboxSize:function(){return t(),o}}))}();var Vt=["Webkit","Moz","ms"],Wt=a.createElement("div").style,Xt={};function Zt(t){var e=B.cssProps[t]||Xt[t];return e||(t in Wt?t:Xt[t]=function(t){for(var e=t[0].toUpperCase()+t.slice(1),n=Vt.length;n--;)if((t=Vt[n]+e)in Wt)return t}(t)||t)}var $t=/^(none|table(?!-c[ea]).+)/,te=/^--/,ee={position:"absolute",visibility:"hidden",display:"block"},ne={letterSpacing:"0",fontWeight:"400"};function re(t,e,n){var r=it.exec(e);return r?Math.max(0,r[2]-(n||0))+(r[3]||"px"):e}function ie(t,e,n,r,i,o){var a="width"===e?1:0,s=0,A=0;if(n===(r?"border":"content"))return 0;for(;a<4;a+=2)"margin"===n&&(A+=B.css(t,n+ot[a],!0,i)),r?("content"===n&&(A-=B.css(t,"padding"+ot[a],!0,i)),"margin"!==n&&(A-=B.css(t,"border"+ot[a]+"Width",!0,i))):(A+=B.css(t,"padding"+ot[a],!0,i),"padding"!==n?A+=B.css(t,"border"+ot[a]+"Width",!0,i):s+=B.css(t,"border"+ot[a]+"Width",!0,i));return!r&&o>=0&&(A+=Math.max(0,Math.ceil(t["offset"+e[0].toUpperCase()+e.slice(1)]-o-A-s-.5))||0),A}function oe(t,e,n){var r=Kt(t),i=(!I.boxSizingReliable()||n)&&"border-box"===B.css(t,"boxSizing",!1,r),o=i,a=Jt(t,e,r),s="offset"+e[0].toUpperCase()+e.slice(1);if(Pt.test(a)){if(!n)return a;a="auto"}return(!I.boxSizingReliable()&&i||"auto"===a||!parseFloat(a)&&"inline"===B.css(t,"display",!1,r))&&t.getClientRects().length&&(i="border-box"===B.css(t,"boxSizing",!1,r),(o=s in t)&&(a=t[s])),(a=parseFloat(a)||0)+ie(t,e,n||(i?"border":"content"),o,r,a)+"px"}function ae(t,e,n,r,i){return new ae.prototype.init(t,e,n,r,i)}B.extend({cssHooks:{opacity:{get:function(t,e){if(e){var n=Jt(t,"opacity");return""===n?"1":n}}}},cssNumber:{animationIterationCount:!0,columnCount:!0,fillOpacity:!0,flexGrow:!0,flexShrink:!0,fontWeight:!0,gridArea:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnStart:!0,gridRow:!0,gridRowEnd:!0,gridRowStart:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,widows:!0,zIndex:!0,zoom:!0},cssProps:{},style:function(t,e,n,r){if(t&&3!==t.nodeType&&8!==t.nodeType&&t.style){var i,o,a,s=V(e),A=te.test(e),l=t.style;if(A||(e=Zt(s)),a=B.cssHooks[e]||B.cssHooks[s],void 0===n)return a&&"get"in a&&void 0!==(i=a.get(t,!1,r))?i:l[e];"string"===(o=typeof n)&&(i=it.exec(n))&&i[1]&&(n=ut(t,e,i),o="number"),null!=n&&n==n&&("number"!==o||A||(n+=i&&i[3]||(B.cssNumber[s]?"":"px")),I.clearCloneStyle||""!==n||0!==e.indexOf("background")||(l[e]="inherit"),a&&"set"in a&&void 0===(n=a.set(t,n,r))||(A?l.setProperty(e,n):l[e]=n))}},css:function(t,e,n,r){var i,o,a,s=V(e);return te.test(e)||(e=Zt(s)),(a=B.cssHooks[e]||B.cssHooks[s])&&"get"in a&&(i=a.get(t,!0,n)),void 0===i&&(i=Jt(t,e,r)),"normal"===i&&e in ne&&(i=ne[e]),""===n||n?(o=parseFloat(i),!0===n||isFinite(o)?o||0:i):i}}),B.each(["height","width"],function(t,e){B.cssHooks[e]={get:function(t,n,r){if(n)return!$t.test(B.css(t,"display"))||t.getClientRects().length&&t.getBoundingClientRect().width?oe(t,e,r):ct(t,ee,function(){return oe(t,e,r)})},set:function(t,n,r){var i,o=Kt(t),a=!I.scrollboxSize()&&"absolute"===o.position,s=(a||r)&&"border-box"===B.css(t,"boxSizing",!1,o),A=r?ie(t,e,r,s,o):0;return s&&a&&(A-=Math.ceil(t["offset"+e[0].toUpperCase()+e.slice(1)]-parseFloat(o[e])-ie(t,e,"border",!1,o)-.5)),A&&(i=it.exec(n))&&"px"!==(i[3]||"px")&&(t.style[e]=n,n=B.css(t,e)),re(0,n,A)}}}),B.cssHooks.marginLeft=qt(I.reliableMarginLeft,function(t,e){if(e)return(parseFloat(Jt(t,"marginLeft"))||t.getBoundingClientRect().left-ct(t,{marginLeft:0},function(){return t.getBoundingClientRect().left}))+"px"}),B.each({margin:"",padding:"",border:"Width"},function(t,e){B.cssHooks[t+e]={expand:function(n){for(var r=0,i={},o="string"==typeof n?n.split(" "):[n];r<4;r++)i[t+ot[r]+e]=o[r]||o[r-2]||o[0];return i}},"margin"!==t&&(B.cssHooks[t+e].set=re)}),B.fn.extend({css:function(t,e){return K(this,function(t,e,n){var r,i,o={},a=0;if(Array.isArray(e)){for(r=Kt(t),i=e.length;a<i;a++)o[e[a]]=B.css(t,e[a],!1,r);return o}return void 0!==n?B.style(t,e,n):B.css(t,e)},t,e,arguments.length>1)}}),B.Tween=ae,ae.prototype={constructor:ae,init:function(t,e,n,r,i,o){this.elem=t,this.prop=n,this.easing=i||B.easing._default,this.options=e,this.start=this.now=this.cur(),this.end=r,this.unit=o||(B.cssNumber[n]?"":"px")},cur:function(){var t=ae.propHooks[this.prop];return t&&t.get?t.get(this):ae.propHooks._default.get(this)},run:function(t){var e,n=ae.propHooks[this.prop];return this.options.duration?this.pos=e=B.easing[this.easing](t,this.options.duration*t,0,1,this.options.duration):this.pos=e=t,this.now=(this.end-this.start)*e+this.start,this.options.step&&this.options.step.call(this.elem,this.now,this),n&&n.set?n.set(this):ae.propHooks._default.set(this),this}},ae.prototype.init.prototype=ae.prototype,ae.propHooks={_default:{get:function(t){var e;return 1!==t.elem.nodeType||null!=t.elem[t.prop]&&null==t.elem.style[t.prop]?t.elem[t.prop]:(e=B.css(t.elem,t.prop,""))&&"auto"!==e?e:0},set:function(t){B.fx.step[t.prop]?B.fx.step[t.prop](t):1!==t.elem.nodeType||!B.cssHooks[t.prop]&&null==t.elem.style[Zt(t.prop)]?t.elem[t.prop]=t.now:B.style(t.elem,t.prop,t.now+t.unit)}}},ae.propHooks.scrollTop=ae.propHooks.scrollLeft={set:function(t){t.elem.nodeType&&t.elem.parentNode&&(t.elem[t.prop]=t.now)}},B.easing={linear:function(t){return t},swing:function(t){return.5-Math.cos(t*Math.PI)/2},_default:"swing"},B.fx=ae.prototype.init,B.fx.step={};var se,Ae,le=/^(?:toggle|show|hide)$/,ce=/queueHooks$/;function ue(){Ae&&(!1===a.hidden&&n.requestAnimationFrame?n.requestAnimationFrame(ue):n.setTimeout(ue,B.fx.interval),B.fx.tick())}function ge(){return n.setTimeout(function(){se=void 0}),se=Date.now()}function he(t,e){var n,r=0,i={height:t};for(e=e?1:0;r<4;r+=2-e)i["margin"+(n=ot[r])]=i["padding"+n]=t;return e&&(i.opacity=i.width=t),i}function de(t,e,n){for(var r,i=(fe.tweeners[e]||[]).concat(fe.tweeners["*"]),o=0,a=i.length;o<a;o++)if(r=i[o].call(n,e,t))return r}function fe(t,e,n){var r,i,o=0,a=fe.prefilters.length,s=B.Deferred().always(function(){delete A.elem}),A=function(){if(i)return!1;for(var e=se||ge(),n=Math.max(0,l.startTime+l.duration-e),r=1-(n/l.duration||0),o=0,a=l.tweens.length;o<a;o++)l.tweens[o].run(r);return s.notifyWith(t,[l,r,n]),r<1&&a?n:(a||s.notifyWith(t,[l,1,0]),s.resolveWith(t,[l]),!1)},l=s.promise({elem:t,props:B.extend({},e),opts:B.extend(!0,{specialEasing:{},easing:B.easing._default},n),originalProperties:e,originalOptions:n,startTime:se||ge(),duration:n.duration,tweens:[],createTween:function(e,n){var r=B.Tween(t,l.opts,e,n,l.opts.specialEasing[e]||l.opts.easing);return l.tweens.push(r),r},stop:function(e){var n=0,r=e?l.tweens.length:0;if(i)return this;for(i=!0;n<r;n++)l.tweens[n].run(1);return e?(s.notifyWith(t,[l,1,0]),s.resolveWith(t,[l,e])):s.rejectWith(t,[l,e]),this}}),c=l.props;for(!function(t,e){var n,r,i,o,a;for(n in t)if(i=e[r=V(n)],o=t[n],Array.isArray(o)&&(i=o[1],o=t[n]=o[0]),n!==r&&(t[r]=o,delete t[n]),(a=B.cssHooks[r])&&"expand"in a)for(n in o=a.expand(o),delete t[r],o)n in t||(t[n]=o[n],e[n]=i);else e[r]=i}(c,l.opts.specialEasing);o<a;o++)if(r=fe.prefilters[o].call(l,t,c,l.opts))return C(r.stop)&&(B._queueHooks(l.elem,l.opts.queue).stop=r.stop.bind(r)),r;return B.map(c,de,l),C(l.opts.start)&&l.opts.start.call(t,l),l.progress(l.opts.progress).done(l.opts.done,l.opts.complete).fail(l.opts.fail).always(l.opts.always),B.fx.timer(B.extend(A,{elem:t,anim:l,queue:l.opts.queue})),l}B.Animation=B.extend(fe,{tweeners:{"*":[function(t,e){var n=this.createTween(t,e);return ut(n.elem,t,it.exec(e),n),n}]},tweener:function(t,e){C(t)?(e=t,t=["*"]):t=t.match(U);for(var n,r=0,i=t.length;r<i;r++)n=t[r],fe.tweeners[n]=fe.tweeners[n]||[],fe.tweeners[n].unshift(e)},prefilters:[function(t,e,n){var r,i,o,a,s,A,l,c,u="width"in e||"height"in e,g=this,h={},d=t.style,f=t.nodeType&&lt(t),p=Z.get(t,"fxshow");for(r in n.queue||(null==(a=B._queueHooks(t,"fx")).unqueued&&(a.unqueued=0,s=a.empty.fire,a.empty.fire=function(){a.unqueued||s()}),a.unqueued++,g.always(function(){g.always(function(){a.unqueued--,B.queue(t,"fx").length||a.empty.fire()})})),e)if(i=e[r],le.test(i)){if(delete e[r],o=o||"toggle"===i,i===(f?"hide":"show")){if("show"!==i||!p||void 0===p[r])continue;f=!0}h[r]=p&&p[r]||B.style(t,r)}if((A=!B.isEmptyObject(e))||!B.isEmptyObject(h))for(r in u&&1===t.nodeType&&(n.overflow=[d.overflow,d.overflowX,d.overflowY],null==(l=p&&p.display)&&(l=Z.get(t,"display")),"none"===(c=B.css(t,"display"))&&(l?c=l:(dt([t],!0),l=t.style.display||l,c=B.css(t,"display"),dt([t]))),("inline"===c||"inline-block"===c&&null!=l)&&"none"===B.css(t,"float")&&(A||(g.done(function(){d.display=l}),null==l&&(c=d.display,l="none"===c?"":c)),d.display="inline-block")),n.overflow&&(d.overflow="hidden",g.always(function(){d.overflow=n.overflow[0],d.overflowX=n.overflow[1],d.overflowY=n.overflow[2]})),A=!1,h)A||(p?"hidden"in p&&(f=p.hidden):p=Z.access(t,"fxshow",{display:l}),o&&(p.hidden=!f),f&&dt([t],!0),g.done(function(){for(r in f||dt([t]),Z.remove(t,"fxshow"),h)B.style(t,r,h[r])})),A=de(f?p[r]:0,r,g),r in p||(p[r]=A.start,f&&(A.end=A.start,A.start=0))}],prefilter:function(t,e){e?fe.prefilters.unshift(t):fe.prefilters.push(t)}}),B.speed=function(t,e,n){var r=t&&"object"==typeof t?B.extend({},t):{complete:n||!n&&e||C(t)&&t,duration:t,easing:n&&e||e&&!C(e)&&e};return B.fx.off?r.duration=0:"number"!=typeof r.duration&&(r.duration in B.fx.speeds?r.duration=B.fx.speeds[r.duration]:r.duration=B.fx.speeds._default),null!=r.queue&&!0!==r.queue||(r.queue="fx"),r.old=r.complete,r.complete=function(){C(r.old)&&r.old.call(this),r.queue&&B.dequeue(this,r.queue)},r},B.fn.extend({fadeTo:function(t,e,n,r){return this.filter(lt).css("opacity",0).show().end().animate({opacity:e},t,n,r)},animate:function(t,e,n,r){var i=B.isEmptyObject(t),o=B.speed(e,n,r),a=function(){var e=fe(this,B.extend({},t),o);(i||Z.get(this,"finish"))&&e.stop(!0)};return a.finish=a,i||!1===o.queue?this.each(a):this.queue(o.queue,a)},stop:function(t,e,n){var r=function(t){var e=t.stop;delete t.stop,e(n)};return"string"!=typeof t&&(n=e,e=t,t=void 0),e&&!1!==t&&this.queue(t||"fx",[]),this.each(function(){var e=!0,i=null!=t&&t+"queueHooks",o=B.timers,a=Z.get(this);if(i)a[i]&&a[i].stop&&r(a[i]);else for(i in a)a[i]&&a[i].stop&&ce.test(i)&&r(a[i]);for(i=o.length;i--;)o[i].elem!==this||null!=t&&o[i].queue!==t||(o[i].anim.stop(n),e=!1,o.splice(i,1));!e&&n||B.dequeue(this,t)})},finish:function(t){return!1!==t&&(t=t||"fx"),this.each(function(){var e,n=Z.get(this),r=n[t+"queue"],i=n[t+"queueHooks"],o=B.timers,a=r?r.length:0;for(n.finish=!0,B.queue(this,t,[]),i&&i.stop&&i.stop.call(this,!0),e=o.length;e--;)o[e].elem===this&&o[e].queue===t&&(o[e].anim.stop(!0),o.splice(e,1));for(e=0;e<a;e++)r[e]&&r[e].finish&&r[e].finish.call(this);delete n.finish})}}),B.each(["toggle","show","hide"],function(t,e){var n=B.fn[e];B.fn[e]=function(t,r,i){return null==t||"boolean"==typeof t?n.apply(this,arguments):this.animate(he(e,!0),t,r,i)}}),B.each({slideDown:he("show"),slideUp:he("hide"),slideToggle:he("toggle"),fadeIn:{opacity:"show"},fadeOut:{opacity:"hide"},fadeToggle:{opacity:"toggle"}},function(t,e){B.fn[t]=function(t,n,r){return this.animate(e,t,n,r)}}),B.timers=[],B.fx.tick=function(){var t,e=0,n=B.timers;for(se=Date.now();e<n.length;e++)(t=n[e])()||n[e]!==t||n.splice(e--,1);n.length||B.fx.stop(),se=void 0},B.fx.timer=function(t){B.timers.push(t),B.fx.start()},B.fx.interval=13,B.fx.start=function(){Ae||(Ae=!0,ue())},B.fx.stop=function(){Ae=null},B.fx.speeds={slow:600,fast:200,_default:400},B.fn.delay=function(t,e){return t=B.fx&&B.fx.speeds[t]||t,e=e||"fx",this.queue(e,function(e,r){var i=n.setTimeout(e,t);r.stop=function(){n.clearTimeout(i)}})},function(){var t=a.createElement("input"),e=a.createElement("select").appendChild(a.createElement("option"));t.type="checkbox",I.checkOn=""!==t.value,I.optSelected=e.selected,(t=a.createElement("input")).value="t",t.type="radio",I.radioValue="t"===t.value}();var pe,Ie=B.expr.attrHandle;B.fn.extend({attr:function(t,e){return K(this,B.attr,t,e,arguments.length>1)},removeAttr:function(t){return this.each(function(){B.removeAttr(this,t)})}}),B.extend({attr:function(t,e,n){var r,i,o=t.nodeType;if(3!==o&&8!==o&&2!==o)return void 0===t.getAttribute?B.prop(t,e,n):(1===o&&B.isXMLDoc(t)||(i=B.attrHooks[e.toLowerCase()]||(B.expr.match.bool.test(e)?pe:void 0)),void 0!==n?null===n?void B.removeAttr(t,e):i&&"set"in i&&void 0!==(r=i.set(t,n,e))?r:(t.setAttribute(e,n+""),n):i&&"get"in i&&null!==(r=i.get(t,e))?r:null==(r=B.find.attr(t,e))?void 0:r)},attrHooks:{type:{set:function(t,e){if(!I.radioValue&&"radio"===e&&M(t,"input")){var n=t.value;return t.setAttribute("type",e),n&&(t.value=n),e}}}},removeAttr:function(t,e){var n,r=0,i=e&&e.match(U);if(i&&1===t.nodeType)for(;n=i[r++];)t.removeAttribute(n)}}),pe={set:function(t,e,n){return!1===e?B.removeAttr(t,n):t.setAttribute(n,n),n}},B.each(B.expr.match.bool.source.match(/\w+/g),function(t,e){var n=Ie[e]||B.find.attr;Ie[e]=function(t,e,r){var i,o,a=e.toLowerCase();return r||(o=Ie[a],Ie[a]=i,i=null!=n(t,e,r)?a:null,Ie[a]=o),i}});var Ce=/^(?:input|select|textarea|button)$/i,me=/^(?:a|area)$/i;function ye(t){return(t.match(U)||[]).join(" ")}function Ee(t){return t.getAttribute&&t.getAttribute("class")||""}function ve(t){return Array.isArray(t)?t:"string"==typeof t&&t.match(U)||[]}B.fn.extend({prop:function(t,e){return K(this,B.prop,t,e,arguments.length>1)},removeProp:function(t){return this.each(function(){delete this[B.propFix[t]||t]})}}),B.extend({prop:function(t,e,n){var r,i,o=t.nodeType;if(3!==o&&8!==o&&2!==o)return 1===o&&B.isXMLDoc(t)||(e=B.propFix[e]||e,i=B.propHooks[e]),void 0!==n?i&&"set"in i&&void 0!==(r=i.set(t,n,e))?r:t[e]=n:i&&"get"in i&&null!==(r=i.get(t,e))?r:t[e]},propHooks:{tabIndex:{get:function(t){var e=B.find.attr(t,"tabindex");return e?parseInt(e,10):Ce.test(t.nodeName)||me.test(t.nodeName)&&t.href?0:-1}}},propFix:{for:"htmlFor",class:"className"}}),I.optSelected||(B.propHooks.selected={get:function(t){var e=t.parentNode;return e&&e.parentNode&&e.parentNode.selectedIndex,null},set:function(t){var e=t.parentNode;e&&(e.selectedIndex,e.parentNode&&e.parentNode.selectedIndex)}}),B.each(["tabIndex","readOnly","maxLength","cellSpacing","cellPadding","rowSpan","colSpan","useMap","frameBorder","contentEditable"],function(){B.propFix[this.toLowerCase()]=this}),B.fn.extend({addClass:function(t){var e,n,r,i,o,a,s,A=0;if(C(t))return this.each(function(e){B(this).addClass(t.call(this,e,Ee(this)))});if((e=ve(t)).length)for(;n=this[A++];)if(i=Ee(n),r=1===n.nodeType&&" "+ye(i)+" "){for(a=0;o=e[a++];)r.indexOf(" "+o+" ")<0&&(r+=o+" ");i!==(s=ye(r))&&n.setAttribute("class",s)}return this},removeClass:function(t){var e,n,r,i,o,a,s,A=0;if(C(t))return this.each(function(e){B(this).removeClass(t.call(this,e,Ee(this)))});if(!arguments.length)return this.attr("class","");if((e=ve(t)).length)for(;n=this[A++];)if(i=Ee(n),r=1===n.nodeType&&" "+ye(i)+" "){for(a=0;o=e[a++];)for(;r.indexOf(" "+o+" ")>-1;)r=r.replace(" "+o+" "," ");i!==(s=ye(r))&&n.setAttribute("class",s)}return this},toggleClass:function(t,e){var n=typeof t,r="string"===n||Array.isArray(t);return"boolean"==typeof e&&r?e?this.addClass(t):this.removeClass(t):C(t)?this.each(function(n){B(this).toggleClass(t.call(this,n,Ee(this),e),e)}):this.each(function(){var e,i,o,a;if(r)for(i=0,o=B(this),a=ve(t);e=a[i++];)o.hasClass(e)?o.removeClass(e):o.addClass(e);else void 0!==t&&"boolean"!==n||((e=Ee(this))&&Z.set(this,"__className__",e),this.setAttribute&&this.setAttribute("class",e||!1===t?"":Z.get(this,"__className__")||""))})},hasClass:function(t){var e,n,r=0;for(e=" "+t+" ";n=this[r++];)if(1===n.nodeType&&(" "+ye(Ee(n))+" ").indexOf(e)>-1)return!0;return!1}});var Be=/\r/g;B.fn.extend({val:function(t){var e,n,r,i=this[0];return arguments.length?(r=C(t),this.each(function(n){var i;1===this.nodeType&&(null==(i=r?t.call(this,n,B(this).val()):t)?i="":"number"==typeof i?i+="":Array.isArray(i)&&(i=B.map(i,function(t){return null==t?"":t+""})),(e=B.valHooks[this.type]||B.valHooks[this.nodeName.toLowerCase()])&&"set"in e&&void 0!==e.set(this,i,"value")||(this.value=i))})):i?(e=B.valHooks[i.type]||B.valHooks[i.nodeName.toLowerCase()])&&"get"in e&&void 0!==(n=e.get(i,"value"))?n:"string"==typeof(n=i.value)?n.replace(Be,""):null==n?"":n:void 0}}),B.extend({valHooks:{option:{get:function(t){var e=B.find.attr(t,"value");return null!=e?e:ye(B.text(t))}},select:{get:function(t){var e,n,r,i=t.options,o=t.selectedIndex,a="select-one"===t.type,s=a?null:[],A=a?o+1:i.length;for(r=o<0?A:a?o:0;r<A;r++)if(((n=i[r]).selected||r===o)&&!n.disabled&&(!n.parentNode.disabled||!M(n.parentNode,"optgroup"))){if(e=B(n).val(),a)return e;s.push(e)}return s},set:function(t,e){for(var n,r,i=t.options,o=B.makeArray(e),a=i.length;a--;)((r=i[a]).selected=B.inArray(B.valHooks.option.get(r),o)>-1)&&(n=!0);return n||(t.selectedIndex=-1),o}}}}),B.each(["radio","checkbox"],function(){B.valHooks[this]={set:function(t,e){if(Array.isArray(e))return t.checked=B.inArray(B(t).val(),e)>-1}},I.checkOn||(B.valHooks[this].get=function(t){return null===t.getAttribute("value")?"on":t.value})}),I.focusin="onfocusin"in n;var be=/^(?:focusinfocus|focusoutblur)$/,Qe=function(t){t.stopPropagation()};B.extend(B.event,{trigger:function(t,e,r,i){var o,s,A,l,c,u,g,h,f=[r||a],p=d.call(t,"type")?t.type:t,I=d.call(t,"namespace")?t.namespace.split("."):[];if(s=h=A=r=r||a,3!==r.nodeType&&8!==r.nodeType&&!be.test(p+B.event.triggered)&&(p.indexOf(".")>-1&&(I=p.split("."),p=I.shift(),I.sort()),c=p.indexOf(":")<0&&"on"+p,(t=t[B.expando]?t:new B.Event(p,"object"==typeof t&&t)).isTrigger=i?2:3,t.namespace=I.join("."),t.rnamespace=t.namespace?new RegExp("(^|\\.)"+I.join("\\.(?:.*\\.|)")+"(\\.|$)"):null,t.result=void 0,t.target||(t.target=r),e=null==e?[t]:B.makeArray(e,[t]),g=B.event.special[p]||{},i||!g.trigger||!1!==g.trigger.apply(r,e))){if(!i&&!g.noBubble&&!m(r)){for(l=g.delegateType||p,be.test(l+p)||(s=s.parentNode);s;s=s.parentNode)f.push(s),A=s;A===(r.ownerDocument||a)&&f.push(A.defaultView||A.parentWindow||n)}for(o=0;(s=f[o++])&&!t.isPropagationStopped();)h=s,t.type=o>1?l:g.bindType||p,(u=(Z.get(s,"events")||{})[t.type]&&Z.get(s,"handle"))&&u.apply(s,e),(u=c&&s[c])&&u.apply&&W(s)&&(t.result=u.apply(s,e),!1===t.result&&t.preventDefault());return t.type=p,i||t.isDefaultPrevented()||g._default&&!1!==g._default.apply(f.pop(),e)||!W(r)||c&&C(r[p])&&!m(r)&&((A=r[c])&&(r[c]=null),B.event.triggered=p,t.isPropagationStopped()&&h.addEventListener(p,Qe),r[p](),t.isPropagationStopped()&&h.removeEventListener(p,Qe),B.event.triggered=void 0,A&&(r[c]=A)),t.result}},simulate:function(t,e,n){var r=B.extend(new B.Event,n,{type:t,isSimulated:!0});B.event.trigger(r,null,e)}}),B.fn.extend({trigger:function(t,e){return this.each(function(){B.event.trigger(t,e,this)})},triggerHandler:function(t,e){var n=this[0];if(n)return B.event.trigger(t,e,n,!0)}}),I.focusin||B.each({focus:"focusin",blur:"focusout"},function(t,e){var n=function(t){B.event.simulate(e,t.target,B.event.fix(t))};B.event.special[e]={setup:function(){var r=this.ownerDocument||this,i=Z.access(r,e);i||r.addEventListener(t,n,!0),Z.access(r,e,(i||0)+1)},teardown:function(){var r=this.ownerDocument||this,i=Z.access(r,e)-1;i?Z.access(r,e,i):(r.removeEventListener(t,n,!0),Z.remove(r,e))}}});var we=n.location,xe=Date.now(),_e=/\?/;B.parseXML=function(t){var e;if(!t||"string"!=typeof t)return null;try{e=(new n.DOMParser).parseFromString(t,"text/xml")}catch(t){e=void 0}return e&&!e.getElementsByTagName("parsererror").length||B.error("Invalid XML: "+t),e};var De=/\[\]$/,Me=/\r?\n/g,ke=/^(?:submit|button|image|reset|file)$/i,Se=/^(?:input|select|textarea|keygen)/i;function Le(t,e,n,r){var i;if(Array.isArray(e))B.each(e,function(e,i){n||De.test(t)?r(t,i):Le(t+"["+("object"==typeof i&&null!=i?e:"")+"]",i,n,r)});else if(n||"object"!==v(e))r(t,e);else for(i in e)Le(t+"["+i+"]",e[i],n,r)}B.param=function(t,e){var n,r=[],i=function(t,e){var n=C(e)?e():e;r[r.length]=encodeURIComponent(t)+"="+encodeURIComponent(null==n?"":n)};if(null==t)return"";if(Array.isArray(t)||t.jquery&&!B.isPlainObject(t))B.each(t,function(){i(this.name,this.value)});else for(n in t)Le(n,t[n],e,i);return r.join("&")},B.fn.extend({serialize:function(){return B.param(this.serializeArray())},serializeArray:function(){return this.map(function(){var t=B.prop(this,"elements");return t?B.makeArray(t):this}).filter(function(){var t=this.type;return this.name&&!B(this).is(":disabled")&&Se.test(this.nodeName)&&!ke.test(t)&&(this.checked||!ft.test(t))}).map(function(t,e){var n=B(this).val();return null==n?null:Array.isArray(n)?B.map(n,function(t){return{name:e.name,value:t.replace(Me,"\r\n")}}):{name:e.name,value:n.replace(Me,"\r\n")}}).get()}});var Re=/%20/g,Ne=/#.*$/,Fe=/([?&])_=[^&]*/,Te=/^(.*?):[ \t]*([^\r\n]*)$/gm,Ue=/^(?:GET|HEAD)$/,Oe=/^\/\//,Ge={},He={},je="*/".concat("*"),Ye=a.createElement("a");function Pe(t){return function(e,n){"string"!=typeof e&&(n=e,e="*");var r,i=0,o=e.toLowerCase().match(U)||[];if(C(n))for(;r=o[i++];)"+"===r[0]?(r=r.slice(1)||"*",(t[r]=t[r]||[]).unshift(n)):(t[r]=t[r]||[]).push(n)}}function Ke(t,e,n,r){var i={},o=t===He;function a(s){var A;return i[s]=!0,B.each(t[s]||[],function(t,s){var l=s(e,n,r);return"string"!=typeof l||o||i[l]?o?!(A=l):void 0:(e.dataTypes.unshift(l),a(l),!1)}),A}return a(e.dataTypes[0])||!i["*"]&&a("*")}function ze(t,e){var n,r,i=B.ajaxSettings.flatOptions||{};for(n in e)void 0!==e[n]&&((i[n]?t:r||(r={}))[n]=e[n]);return r&&B.extend(!0,t,r),t}Ye.href=we.href,B.extend({active:0,lastModified:{},etag:{},ajaxSettings:{url:we.href,type:"GET",isLocal:/^(?:about|app|app-storage|.+-extension|file|res|widget):$/.test(we.protocol),global:!0,processData:!0,async:!0,contentType:"application/x-www-form-urlencoded; charset=UTF-8",accepts:{"*":je,text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript"},contents:{xml:/\bxml\b/,html:/\bhtml/,json:/\bjson\b/},responseFields:{xml:"responseXML",text:"responseText",json:"responseJSON"},converters:{"* text":String,"text html":!0,"text json":JSON.parse,"text xml":B.parseXML},flatOptions:{url:!0,context:!0}},ajaxSetup:function(t,e){return e?ze(ze(t,B.ajaxSettings),e):ze(B.ajaxSettings,t)},ajaxPrefilter:Pe(Ge),ajaxTransport:Pe(He),ajax:function(t,e){"object"==typeof t&&(e=t,t=void 0),e=e||{};var r,i,o,s,A,l,c,u,g,h,d=B.ajaxSetup({},e),f=d.context||d,p=d.context&&(f.nodeType||f.jquery)?B(f):B.event,I=B.Deferred(),C=B.Callbacks("once memory"),m=d.statusCode||{},y={},E={},v="canceled",b={readyState:0,getResponseHeader:function(t){var e;if(c){if(!s)for(s={};e=Te.exec(o);)s[e[1].toLowerCase()+" "]=(s[e[1].toLowerCase()+" "]||[]).concat(e[2]);e=s[t.toLowerCase()+" "]}return null==e?null:e.join(", ")},getAllResponseHeaders:function(){return c?o:null},setRequestHeader:function(t,e){return null==c&&(t=E[t.toLowerCase()]=E[t.toLowerCase()]||t,y[t]=e),this},overrideMimeType:function(t){return null==c&&(d.mimeType=t),this},statusCode:function(t){var e;if(t)if(c)b.always(t[b.status]);else for(e in t)m[e]=[m[e],t[e]];return this},abort:function(t){var e=t||v;return r&&r.abort(e),Q(0,e),this}};if(I.promise(b),d.url=((t||d.url||we.href)+"").replace(Oe,we.protocol+"//"),d.type=e.method||e.type||d.method||d.type,d.dataTypes=(d.dataType||"*").toLowerCase().match(U)||[""],null==d.crossDomain){l=a.createElement("a");try{l.href=d.url,l.href=l.href,d.crossDomain=Ye.protocol+"//"+Ye.host!=l.protocol+"//"+l.host}catch(t){d.crossDomain=!0}}if(d.data&&d.processData&&"string"!=typeof d.data&&(d.data=B.param(d.data,d.traditional)),Ke(Ge,d,e,b),c)return b;for(g in(u=B.event&&d.global)&&0==B.active++&&B.event.trigger("ajaxStart"),d.type=d.type.toUpperCase(),d.hasContent=!Ue.test(d.type),i=d.url.replace(Ne,""),d.hasContent?d.data&&d.processData&&0===(d.contentType||"").indexOf("application/x-www-form-urlencoded")&&(d.data=d.data.replace(Re,"+")):(h=d.url.slice(i.length),d.data&&(d.processData||"string"==typeof d.data)&&(i+=(_e.test(i)?"&":"?")+d.data,delete d.data),!1===d.cache&&(i=i.replace(Fe,"$1"),h=(_e.test(i)?"&":"?")+"_="+xe+++h),d.url=i+h),d.ifModified&&(B.lastModified[i]&&b.setRequestHeader("If-Modified-Since",B.lastModified[i]),B.etag[i]&&b.setRequestHeader("If-None-Match",B.etag[i])),(d.data&&d.hasContent&&!1!==d.contentType||e.contentType)&&b.setRequestHeader("Content-Type",d.contentType),b.setRequestHeader("Accept",d.dataTypes[0]&&d.accepts[d.dataTypes[0]]?d.accepts[d.dataTypes[0]]+("*"!==d.dataTypes[0]?", "+je+"; q=0.01":""):d.accepts["*"]),d.headers)b.setRequestHeader(g,d.headers[g]);if(d.beforeSend&&(!1===d.beforeSend.call(f,b,d)||c))return b.abort();if(v="abort",C.add(d.complete),b.done(d.success),b.fail(d.error),r=Ke(He,d,e,b)){if(b.readyState=1,u&&p.trigger("ajaxSend",[b,d]),c)return b;d.async&&d.timeout>0&&(A=n.setTimeout(function(){b.abort("timeout")},d.timeout));try{c=!1,r.send(y,Q)}catch(t){if(c)throw t;Q(-1,t)}}else Q(-1,"No Transport");function Q(t,e,a,s){var l,g,h,y,E,v=e;c||(c=!0,A&&n.clearTimeout(A),r=void 0,o=s||"",b.readyState=t>0?4:0,l=t>=200&&t<300||304===t,a&&(y=function(t,e,n){for(var r,i,o,a,s=t.contents,A=t.dataTypes;"*"===A[0];)A.shift(),void 0===r&&(r=t.mimeType||e.getResponseHeader("Content-Type"));if(r)for(i in s)if(s[i]&&s[i].test(r)){A.unshift(i);break}if(A[0]in n)o=A[0];else{for(i in n){if(!A[0]||t.converters[i+" "+A[0]]){o=i;break}a||(a=i)}o=o||a}if(o)return o!==A[0]&&A.unshift(o),n[o]}(d,b,a)),y=function(t,e,n,r){var i,o,a,s,A,l={},c=t.dataTypes.slice();if(c[1])for(a in t.converters)l[a.toLowerCase()]=t.converters[a];for(o=c.shift();o;)if(t.responseFields[o]&&(n[t.responseFields[o]]=e),!A&&r&&t.dataFilter&&(e=t.dataFilter(e,t.dataType)),A=o,o=c.shift())if("*"===o)o=A;else if("*"!==A&&A!==o){if(!(a=l[A+" "+o]||l["* "+o]))for(i in l)if((s=i.split(" "))[1]===o&&(a=l[A+" "+s[0]]||l["* "+s[0]])){!0===a?a=l[i]:!0!==l[i]&&(o=s[0],c.unshift(s[1]));break}if(!0!==a)if(a&&t.throws)e=a(e);else try{e=a(e)}catch(t){return{state:"parsererror",error:a?t:"No conversion from "+A+" to "+o}}}return{state:"success",data:e}}(d,y,b,l),l?(d.ifModified&&((E=b.getResponseHeader("Last-Modified"))&&(B.lastModified[i]=E),(E=b.getResponseHeader("etag"))&&(B.etag[i]=E)),204===t||"HEAD"===d.type?v="nocontent":304===t?v="notmodified":(v=y.state,g=y.data,l=!(h=y.error))):(h=v,!t&&v||(v="error",t<0&&(t=0))),b.status=t,b.statusText=(e||v)+"",l?I.resolveWith(f,[g,v,b]):I.rejectWith(f,[b,v,h]),b.statusCode(m),m=void 0,u&&p.trigger(l?"ajaxSuccess":"ajaxError",[b,d,l?g:h]),C.fireWith(f,[b,v]),u&&(p.trigger("ajaxComplete",[b,d]),--B.active||B.event.trigger("ajaxStop")))}return b},getJSON:function(t,e,n){return B.get(t,e,n,"json")},getScript:function(t,e){return B.get(t,void 0,e,"script")}}),B.each(["get","post"],function(t,e){B[e]=function(t,n,r,i){return C(n)&&(i=i||r,r=n,n=void 0),B.ajax(B.extend({url:t,type:e,dataType:i,data:n,success:r},B.isPlainObject(t)&&t))}}),B._evalUrl=function(t,e){return B.ajax({url:t,type:"GET",dataType:"script",cache:!0,async:!1,global:!1,converters:{"text script":function(){}},dataFilter:function(t){B.globalEval(t,e)}})},B.fn.extend({wrapAll:function(t){var e;return this[0]&&(C(t)&&(t=t.call(this[0])),e=B(t,this[0].ownerDocument).eq(0).clone(!0),this[0].parentNode&&e.insertBefore(this[0]),e.map(function(){for(var t=this;t.firstElementChild;)t=t.firstElementChild;return t}).append(this)),this},wrapInner:function(t){return C(t)?this.each(function(e){B(this).wrapInner(t.call(this,e))}):this.each(function(){var e=B(this),n=e.contents();n.length?n.wrapAll(t):e.append(t)})},wrap:function(t){var e=C(t);return this.each(function(n){B(this).wrapAll(e?t.call(this,n):t)})},unwrap:function(t){return this.parent(t).not("body").each(function(){B(this).replaceWith(this.childNodes)}),this}}),B.expr.pseudos.hidden=function(t){return!B.expr.pseudos.visible(t)},B.expr.pseudos.visible=function(t){return!!(t.offsetWidth||t.offsetHeight||t.getClientRects().length)},B.ajaxSettings.xhr=function(){try{return new n.XMLHttpRequest}catch(t){}};var Je={0:200,1223:204},qe=B.ajaxSettings.xhr();I.cors=!!qe&&"withCredentials"in qe,I.ajax=qe=!!qe,B.ajaxTransport(function(t){var e,r;if(I.cors||qe&&!t.crossDomain)return{send:function(i,o){var a,s=t.xhr();if(s.open(t.type,t.url,t.async,t.username,t.password),t.xhrFields)for(a in t.xhrFields)s[a]=t.xhrFields[a];for(a in t.mimeType&&s.overrideMimeType&&s.overrideMimeType(t.mimeType),t.crossDomain||i["X-Requested-With"]||(i["X-Requested-With"]="XMLHttpRequest"),i)s.setRequestHeader(a,i[a]);e=function(t){return function(){e&&(e=r=s.onload=s.onerror=s.onabort=s.ontimeout=s.onreadystatechange=null,"abort"===t?s.abort():"error"===t?"number"!=typeof s.status?o(0,"error"):o(s.status,s.statusText):o(Je[s.status]||s.status,s.statusText,"text"!==(s.responseType||"text")||"string"!=typeof s.responseText?{binary:s.response}:{text:s.responseText},s.getAllResponseHeaders()))}},s.onload=e(),r=s.onerror=s.ontimeout=e("error"),void 0!==s.onabort?s.onabort=r:s.onreadystatechange=function(){4===s.readyState&&n.setTimeout(function(){e&&r()})},e=e("abort");try{s.send(t.hasContent&&t.data||null)}catch(t){if(e)throw t}},abort:function(){e&&e()}}}),B.ajaxPrefilter(function(t){t.crossDomain&&(t.contents.script=!1)}),B.ajaxSetup({accepts:{script:"text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"},contents:{script:/\b(?:java|ecma)script\b/},converters:{"text script":function(t){return B.globalEval(t),t}}}),B.ajaxPrefilter("script",function(t){void 0===t.cache&&(t.cache=!1),t.crossDomain&&(t.type="GET")}),B.ajaxTransport("script",function(t){var e,n;if(t.crossDomain||t.scriptAttrs)return{send:function(r,i){e=B("<script>").attr(t.scriptAttrs||{}).prop({charset:t.scriptCharset,src:t.url}).on("load error",n=function(t){e.remove(),n=null,t&&i("error"===t.type?404:200,t.type)}),a.head.appendChild(e[0])},abort:function(){n&&n()}}});var Ve,We=[],Xe=/(=)\?(?=&|$)|\?\?/;B.ajaxSetup({jsonp:"callback",jsonpCallback:function(){var t=We.pop()||B.expando+"_"+xe++;return this[t]=!0,t}}),B.ajaxPrefilter("json jsonp",function(t,e,r){var i,o,a,s=!1!==t.jsonp&&(Xe.test(t.url)?"url":"string"==typeof t.data&&0===(t.contentType||"").indexOf("application/x-www-form-urlencoded")&&Xe.test(t.data)&&"data");if(s||"jsonp"===t.dataTypes[0])return i=t.jsonpCallback=C(t.jsonpCallback)?t.jsonpCallback():t.jsonpCallback,s?t[s]=t[s].replace(Xe,"$1"+i):!1!==t.jsonp&&(t.url+=(_e.test(t.url)?"&":"?")+t.jsonp+"="+i),t.converters["script json"]=function(){return a||B.error(i+" was not called"),a[0]},t.dataTypes[0]="json",o=n[i],n[i]=function(){a=arguments},r.always(function(){void 0===o?B(n).removeProp(i):n[i]=o,t[i]&&(t.jsonpCallback=e.jsonpCallback,We.push(i)),a&&C(o)&&o(a[0]),a=o=void 0}),"script"}),I.createHTMLDocument=((Ve=a.implementation.createHTMLDocument("").body).innerHTML="<form></form><form></form>",2===Ve.childNodes.length),B.parseHTML=function(t,e,n){return"string"!=typeof t?[]:("boolean"==typeof e&&(n=e,e=!1),e||(I.createHTMLDocument?((r=(e=a.implementation.createHTMLDocument("")).createElement("base")).href=a.location.href,e.head.appendChild(r)):e=a),o=!n&&[],(i=k.exec(t))?[e.createElement(i[1])]:(i=bt([t],e,o),o&&o.length&&B(o).remove(),B.merge([],i.childNodes)));var r,i,o},B.fn.load=function(t,e,n){var r,i,o,a=this,s=t.indexOf(" ");return s>-1&&(r=ye(t.slice(s)),t=t.slice(0,s)),C(e)?(n=e,e=void 0):e&&"object"==typeof e&&(i="POST"),a.length>0&&B.ajax({url:t,type:i||"GET",dataType:"html",data:e}).done(function(t){o=arguments,a.html(r?B("<div>").append(B.parseHTML(t)).find(r):t)}).always(n&&function(t,e){a.each(function(){n.apply(this,o||[t.responseText,e,t])})}),this},B.each(["ajaxStart","ajaxStop","ajaxComplete","ajaxError","ajaxSuccess","ajaxSend"],function(t,e){B.fn[e]=function(t){return this.on(e,t)}}),B.expr.pseudos.animated=function(t){return B.grep(B.timers,function(e){return t===e.elem}).length},B.offset={setOffset:function(t,e,n){var r,i,o,a,s,A,l=B.css(t,"position"),c=B(t),u={};"static"===l&&(t.style.position="relative"),s=c.offset(),o=B.css(t,"top"),A=B.css(t,"left"),("absolute"===l||"fixed"===l)&&(o+A).indexOf("auto")>-1?(a=(r=c.position()).top,i=r.left):(a=parseFloat(o)||0,i=parseFloat(A)||0),C(e)&&(e=e.call(t,n,B.extend({},s))),null!=e.top&&(u.top=e.top-s.top+a),null!=e.left&&(u.left=e.left-s.left+i),"using"in e?e.using.call(t,u):c.css(u)}},B.fn.extend({offset:function(t){if(arguments.length)return void 0===t?this:this.each(function(e){B.offset.setOffset(this,t,e)});var e,n,r=this[0];return r?r.getClientRects().length?(e=r.getBoundingClientRect(),n=r.ownerDocument.defaultView,{top:e.top+n.pageYOffset,left:e.left+n.pageXOffset}):{top:0,left:0}:void 0},position:function(){if(this[0]){var t,e,n,r=this[0],i={top:0,left:0};if("fixed"===B.css(r,"position"))e=r.getBoundingClientRect();else{for(e=this.offset(),n=r.ownerDocument,t=r.offsetParent||n.documentElement;t&&(t===n.body||t===n.documentElement)&&"static"===B.css(t,"position");)t=t.parentNode;t&&t!==r&&1===t.nodeType&&((i=B(t).offset()).top+=B.css(t,"borderTopWidth",!0),i.left+=B.css(t,"borderLeftWidth",!0))}return{top:e.top-i.top-B.css(r,"marginTop",!0),left:e.left-i.left-B.css(r,"marginLeft",!0)}}},offsetParent:function(){return this.map(function(){for(var t=this.offsetParent;t&&"static"===B.css(t,"position");)t=t.offsetParent;return t||at})}}),B.each({scrollLeft:"pageXOffset",scrollTop:"pageYOffset"},function(t,e){var n="pageYOffset"===e;B.fn[t]=function(r){return K(this,function(t,r,i){var o;if(m(t)?o=t:9===t.nodeType&&(o=t.defaultView),void 0===i)return o?o[e]:t[r];o?o.scrollTo(n?o.pageXOffset:i,n?i:o.pageYOffset):t[r]=i},t,r,arguments.length)}}),B.each(["top","left"],function(t,e){B.cssHooks[e]=qt(I.pixelPosition,function(t,n){if(n)return n=Jt(t,e),Pt.test(n)?B(t).position()[e]+"px":n})}),B.each({Height:"height",Width:"width"},function(t,e){B.each({padding:"inner"+t,content:e,"":"outer"+t},function(n,r){B.fn[r]=function(i,o){var a=arguments.length&&(n||"boolean"!=typeof i),s=n||(!0===i||!0===o?"margin":"border");return K(this,function(e,n,i){var o;return m(e)?0===r.indexOf("outer")?e["inner"+t]:e.document.documentElement["client"+t]:9===e.nodeType?(o=e.documentElement,Math.max(e.body["scroll"+t],o["scroll"+t],e.body["offset"+t],o["offset"+t],o["client"+t])):void 0===i?B.css(e,n,s):B.style(e,n,i,s)},e,a?i:void 0,a)}})}),B.each("blur focus focusin focusout resize scroll click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup contextmenu".split(" "),function(t,e){B.fn[e]=function(t,n){return arguments.length>0?this.on(e,null,t,n):this.trigger(e)}}),B.fn.extend({hover:function(t,e){return this.mouseenter(t).mouseleave(e||t)}}),B.fn.extend({bind:function(t,e,n){return this.on(t,null,e,n)},unbind:function(t,e){return this.off(t,null,e)},delegate:function(t,e,n,r){return this.on(e,t,n,r)},undelegate:function(t,e,n){return 1===arguments.length?this.off(t,"**"):this.off(e,t||"**",n)}}),B.proxy=function(t,e){var n,r,i;if("string"==typeof e&&(n=t[e],e=t,t=n),C(t))return r=A.call(arguments,2),(i=function(){return t.apply(e||this,r.concat(A.call(arguments)))}).guid=t.guid=t.guid||B.guid++,i},B.holdReady=function(t){t?B.readyWait++:B.ready(!0)},B.isArray=Array.isArray,B.parseJSON=JSON.parse,B.nodeName=M,B.isFunction=C,B.isWindow=m,B.camelCase=V,B.type=v,B.now=Date.now,B.isNumeric=function(t){var e=B.type(t);return("number"===e||"string"===e)&&!isNaN(t-parseFloat(t))},void 0===(r=function(){return B}.apply(e,[]))||(t.exports=r);var Ze=n.jQuery,$e=n.$;return B.noConflict=function(t){return n.$===B&&(n.$=$e),t&&n.jQuery===B&&(n.jQuery=Ze),B},i||(n.jQuery=n.$=B),B})},function(t,e,n){"use strict";function r(t){return t&&"function"==typeof t.schedule}n.d(e,"a",function(){return r})},function(t,e,n){"use strict";function r(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}Object.defineProperty(e,"__esModule",{value:!0}),r(n(1026)),r(n(1027)),r(n(1028)),r(n(1029))},function(t,e,n){"use strict";e.__esModule=!0;var r,i=n(649),o=(r=i)&&r.__esModule?r:{default:r};e.default=function(t,e,n){return e in t?(0,o.default)(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(221);e.Part=r.Part;var i=n(1275);e.OptionsBag=i.OptionsBag;var o=n(676);e.PartFactory=o.PartFactory;var a=n(1276);e.factoryExtPlugin=a.factoryExtPlugin,e.IPartFactory=a.IPartFactory;var s=n(1277);e.PartSerializer=s.PartSerializer;var A=n(1278);e.JavascriptEvalPart=A.JavascriptEvalPart;var l=n(1279);e.ReactPart=l.ReactPart;var c=n(1280);e.PartServices=c.PartServices;var u=n(337);e.ErrorPart=u.ErrorPart;var g=n(1281);e.PartUtils=g.PartUtils},function(t,e,n){"use strict";n.r(e);var r,i,o=n(17),a=n(9),s=n(5),A=n(10),l=n(6),c=n(0),u=n(40);class g{constructor(t){this._path="",this._name="",this._type="",this._prevKernelName="",this._isDisposed=!1,this._session=null,this._ready=new a.PromiseDelegate,this._initializing=!1,this._isReady=!1,this._terminated=new A.Signal(this),this._kernelChanged=new A.Signal(this),this._statusChanged=new A.Signal(this),this._iopubMessage=new A.Signal(this),this._unhandledMessage=new A.Signal(this),this._propertyChanged=new A.Signal(this),this._dialog=null,this._busyDisposable=null,this.manager=t.manager,this._path=t.path||a.UUID.uuid4(),this._type=t.type||"",this._name=t.name||"",this._setBusy=t.setBusy,this._kernelPreference=t.kernelPreference||{}}get terminated(){return this._terminated}get kernelChanged(){return this._kernelChanged}get statusChanged(){return this._statusChanged}get iopubMessage(){return this._iopubMessage}get unhandledMessage(){return this._unhandledMessage}get propertyChanged(){return this._propertyChanged}get kernel(){return this._session?this._session.kernel:null}get path(){return this._path}get name(){return this._name}get type(){return this._type}get kernelPreference(){return this._kernelPreference}set kernelPreference(t){this._kernelPreference=t}get status(){return this.isReady?this._session?this._session.status:"dead":"starting"}get isReady(){return this._isReady}get ready(){return this._ready.promise}get kernelDisplayName(){let t=this.kernel;if(!t)return"No Kernel!";let e=this.manager.specs;if(!e)return"Unknown!";let n=e.kernelspecs[t.name];return n?n.display_name:t.name}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this._isDisposed=!0,this._session&&(this.kernelPreference.shutdownOnClose&&this._session.shutdown().catch(t=>{console.error(`Kernel not shut down ${t}`)}),this._session.dispose(),this._session=null),this._dialog&&this._dialog.dispose(),this._busyDisposable&&(this._busyDisposable.dispose(),this._busyDisposable=null),A.Signal.clearData(this))}changeKernel(t){return this.initialize().then(()=>this.isDisposed?Promise.reject("Disposed"):this._changeKernel(t))}selectKernel(){return this.initialize().then(()=>this.isDisposed?Promise.reject("Disposed"):this._selectKernel(!0))}shutdown(){const t=this._session;return this.isDisposed||!t?Promise.resolve():(this._session=null,t.shutdown())}restart(){return this.initialize().then(()=>{if(this.isDisposed)return Promise.reject("session already disposed");let t=this.kernel;return t?g.restartKernel(t):this._prevKernelName?this.changeKernel({name:this._prevKernelName}).then(()=>!0):Promise.reject("No kernel to restart")})}setPath(t){return this.isDisposed||this._path===t?Promise.resolve():(this._path=t,this._session?this._session.setPath(t):(this._propertyChanged.emit("path"),Promise.resolve()))}setName(t){return this.isDisposed||this._name===t?Promise.resolve():(this._name=t,this._session?this._session.setName(t):(this._propertyChanged.emit("name"),Promise.resolve()))}setType(t){return this.isDisposed||this._type===t?Promise.resolve():(this._type=t,this._session?this._session.setType(name):(this._propertyChanged.emit("type"),Promise.resolve()))}async initialize(){if(this._initializing||this._isReady)return this._ready.promise;this._initializing=!0;let t=this.manager;await t.ready;let e=Object(s.find)(t.running(),t=>t.path===this._path);if(e)try{let n=t.connectTo(e);this._handleNewSession(n)}catch(t){return this._handleSessionError(t),Promise.reject(t)}await this._startIfNecessary(),this._isReady=!0,this._ready.resolve(void 0)}_startIfNecessary(){let t=this.kernelPreference;if(this.isDisposed||this.kernel||!1===t.shouldStart||!1===t.canStart)return Promise.resolve();if(t.id)return this._changeKernel({id:t.id}).then(()=>void 0).catch(()=>this._selectKernel(!1));let e=g.getDefaultKernel({specs:this.manager.specs,sessions:this.manager.running(),preference:t});return e?this._changeKernel({name:e}).then(()=>void 0).catch(()=>this._selectKernel(!1)):this._selectKernel(!1)}_changeKernel(t){if(this.isDisposed)return Promise.reject("Disposed");let e=this._session;return e&&"dead"!==e.status?e.changeKernel(t).catch(t=>(this._handleSessionError(t),Promise.reject(t))):this._startSession(t)}_selectKernel(t){if(this.isDisposed)return Promise.resolve();const e=t?[u.a.cancelButton(),u.a.okButton({label:"Select"})]:[u.a.okButton({label:"Select"})];return(this._dialog=new u.a({title:"Select Kernel",body:new i.KernelSelector(this),buttons:e})).launch().then(t=>{if(this.isDisposed||!t.button.accept)return;let e=t.value;return null===e&&this._session?this.shutdown().then(()=>{this._kernelChanged.emit({oldValue:null,newValue:null})}):e?this._changeKernel(e).then(()=>void 0):void 0}).then(()=>{this._dialog=null})}_startSession(t){return this.isDisposed?Promise.reject("Session is disposed."):this.manager.startNew({path:this._path,type:this._type,name:this._name,kernelName:t?t.name:void 0,kernelId:t?t.id:void 0}).then(t=>this._handleNewSession(t)).catch(t=>(this._handleSessionError(t),Promise.reject(t)))}_handleNewSession(t){if(this.isDisposed)throw Error("Disposed");return this._session&&this._session.dispose(),this._session=t,t.path!==this._path&&(this._path=t.path,this._propertyChanged.emit("path")),t.name!==this._name&&(this._name=t.name,this._propertyChanged.emit("name")),t.type!==this._type&&(this._type=t.type,this._propertyChanged.emit("type")),t.terminated.connect(this._onTerminated,this),t.propertyChanged.connect(this._onPropertyChanged,this),t.kernelChanged.connect(this._onKernelChanged,this),t.statusChanged.connect(this._onStatusChanged,this),t.iopubMessage.connect(this._onIopubMessage,this),t.unhandledMessage.connect(this._onUnhandledMessage,this),this._prevKernelName=t.kernel.name,this._kernelChanged.emit({oldValue:null,newValue:t.kernel}),t.kernel}_handleSessionError(t){return t.response.text().then(e=>{let n=t.message;try{n=JSON.parse(e).traceback}catch(t){}return(this._dialog=new u.a({title:"Error Starting Kernel",body:c.createElement("pre",null,n),buttons:[u.a.okButton()]})).launch()}).then(()=>{this._dialog=null})}_onTerminated(){let t=this.kernel;this._session&&this._session.dispose(),this._session=null,this._terminated.emit(void 0),t&&this._kernelChanged.emit({oldValue:null,newValue:null})}_onPropertyChanged(t,e){switch(e){case"path":this._path=t.path;break;case"name":this._name=t.name;break;default:this._type=t.type}this._propertyChanged.emit(e)}_onKernelChanged(t,e){this._kernelChanged.emit(e)}_onStatusChanged(){this._setBusy&&("busy"===this.status?this._busyDisposable||(this._busyDisposable=this._setBusy()):this._busyDisposable&&(this._busyDisposable.dispose(),this._busyDisposable=null)),this._statusChanged.emit(this.status)}_onIopubMessage(t,e){this._iopubMessage.emit(e)}_onUnhandledMessage(t,e){this._unhandledMessage.emit(e)}}(r=g||(g={})).restartKernel=async function(t){let e=u.a.warnButton({label:"Restart"});const n=await Object(u.b)({title:"Restart Kernel?",body:"Do you want to restart the current kernel? All variables will be lost.",buttons:[u.a.cancelButton(),e]});return!t.isDisposed&&!!n.button.accept&&(await t.restart(),!0)},r.getDefaultKernel=function(t){return i.getDefaultKernel(t)},r.populateKernelSelect=function(t,e){return i.populateKernelSelect(t,e)},function(t){function e(t){let e=document.createElement("div"),n=document.createElement("label");n.textContent=`Select kernel for: "${t.name}"`,e.appendChild(n);let r=function(t){return{specs:t.manager.specs,sessions:t.manager.running(),preference:t.kernelPreference}}(t),i=document.createElement("select");return g.populateKernelSelect(i,r),e.appendChild(i),e}function n(t,e){let n=document.createElement("option");return n.text=e,n.value=JSON.stringify({name:t}),n}function r(){let t=document.createElement("optgroup");t.label="Use No Kernel";let e=document.createElement("option");return e.text="No Kernel",e.value="null",t.appendChild(e),t}function i(t,e){let n=document.createElement("option"),r=t.name||o.PathExt.basename(t.path);return n.text=r,n.value=JSON.stringify({id:t.kernel.id}),n.title=`Path: ${t.path}\n`+`Name: ${r}\n`+`Kernel Name: ${e}\n`+`Kernel Id: ${t.kernel.id}`,n}t.KernelSelector=class extends l.Widget{constructor(t){super({node:e(t)})}getValue(){let t=this.node.querySelector("select");return JSON.parse(t.value)}},t.getDefaultKernel=function(t){let{specs:e,preference:n}=t,{name:r,language:i,shouldStart:o,canStart:a,autoStartDefault:s}=n;if(!e||!1===o||!1===a)return null;let A=s?e.default:null;if(!r&&!i)return A;for(let t in e.kernelspecs)if(t===r)return r;if(!i)return A;let l=[];for(let t in e.kernelspecs)i===e.kernelspecs[t].language&&l.push(t);if(1===l.length){let t=l[0];return console.log("No exact match found for "+t+", using kernel "+t+" that matches language="+i),t}return A},t.populateKernelSelect=function(t,e){for(;t.firstChild;)t.removeChild(t.firstChild);let{preference:o,sessions:a,specs:A}=e,{name:l,id:c,language:u,canStart:g,shouldStart:h}=o;if(!A||!1===g)return t.appendChild(r()),t.value="null",void(t.disabled=!0);t.disabled=!1;let d=Object.create(null),f=Object.create(null);for(let t in A.kernelspecs){let e=A.kernelspecs[t];d[t]=e.display_name,f[t]=e.language}let p=[];if(l&&l in A.kernelspecs&&p.push(l),u)for(let t in A.kernelspecs)l!==t&&f[t]===u&&p.push(t);p.length||p.push(A.default);let I=document.createElement("optgroup");I.label="Start Preferred Kernel",p.sort((t,e)=>d[t].localeCompare(d[e]));for(let t of p)I.appendChild(n(t,d[t]));I.firstChild&&t.appendChild(I),t.appendChild(r());let C=document.createElement("optgroup");C.label="Start Other Kernel";let m=[];for(let t in A.kernelspecs)-1===p.indexOf(t)&&m.push(t);m.sort((t,e)=>d[t].localeCompare(d[e]));for(let t of m)C.appendChild(n(t,d[t]));if(m.length&&t.appendChild(C),!1===h?t.value="null":t.selectedIndex=0,!a)return;let y=[],E=[];Object(s.each)(a,t=>{u&&f[t.kernel.name]===u&&t.kernel.id!==c?y.push(t):t.kernel.id!==c&&E.push(t)});let v=document.createElement("optgroup");v.label="Use Kernel from Preferred Session",t.appendChild(v),y.length&&(y.sort((t,e)=>t.path.localeCompare(e.path)),Object(s.each)(y,t=>{let e=d[t.kernel.name];v.appendChild(i(t,e))}));let B=document.createElement("optgroup");B.label="Use Kernel from Other Session",t.appendChild(B),E.length&&(E.sort((t,e)=>t.path.localeCompare(e.path)),Object(s.each)(E,t=>{let e=d[t.kernel.name]||t.kernel.name;B.appendChild(i(t,e))}))}}(i||(i={}));var h=n(612),d=n(613);const f="commandlinker-command",p="commandlinker-args";class I{constructor(t){this._isDisposed=!1,this._commands=t.commands,document.body.addEventListener("click",this)}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,document.body.removeEventListener("click",this))}connectNode(t,e,n){return t.setAttribute(`data-${f}`,e),void 0!==n&&t.setAttribute(`data-${p}`,JSON.stringify(n)),t}disconnectNode(t){return t.removeAttribute(`data-${f}`),t.removeAttribute(`data-${p}`),t}handleEvent(t){switch(t.type){case"click":this._evtClick(t);break;default:return}}populateVNodeDataset(t,e){let n;return n=void 0!==e?{[p]:JSON.stringify(e),[f]:t}:{[f]:t}}_evtClick(t){let e=t.target;for(;e&&e.parentElement;){if(e.hasAttribute(`data-${f}`)){t.preventDefault();let n=e.getAttribute(`data-${f}`);if(!n)return;let r=e.getAttribute(`data-${p}`),i=a.JSONExt.emptyObject;return r&&(i=JSON.parse(r)),void this._commands.execute(n,i)}e=e.parentElement}}}const C=new a.Token("@jupyterlab/apputils:ICommandPalette");var m,y=n(194),E=n(582);class v extends l.Widget{constructor(t={}){super({node:m.createNode()}),this._sandbox=[],this.addClass("jp-IFrame"),this.sandbox=t.sandbox||[],this.referrerPolicy=t.referrerPolicy||"no-referrer"}get referrerPolicy(){return this._referrerPolicy}set referrerPolicy(t){if(this._referrerPolicy===t)return;this._referrerPolicy=t,this.node.querySelector("iframe").setAttribute("referrerpolicy",t)}get sandbox(){return this._sandbox.slice()}set sandbox(t){this._sandbox=t.slice();const e=this.node.querySelector("iframe"),n=t.length?t.join(" "):"";e.setAttribute("sandbox",n)}get url(){return this.node.querySelector("iframe").getAttribute("src")||""}set url(t){this.node.querySelector("iframe").setAttribute("src",t)}}(m||(m={})).createNode=function(){let t=document.createElement("div"),e=document.createElement("iframe");return e.setAttribute("sandbox",""),e.style.height="100%",e.style.width="100%",t.appendChild(e),t};var B=n(115);const b="jp-Input-Dialog";var Q,w;(w=Q||(Q={})).getBoolean=function(t){return Object(u.b)(Object.assign({},t,{body:new _(t),buttons:[u.a.cancelButton({label:t.cancelLabel}),u.a.okButton({label:t.okLabel})],focusNodeSelector:"input"}))},w.getNumber=function(t){return Object(u.b)(Object.assign({},t,{body:new D(t),buttons:[u.a.cancelButton({label:t.cancelLabel}),u.a.okButton({label:t.okLabel})],focusNodeSelector:"input"}))},w.getItem=function(t){return Object(u.b)(Object.assign({},t,{body:new k(t),buttons:[u.a.cancelButton({label:t.cancelLabel}),u.a.okButton({label:t.okLabel})],focusNodeSelector:t.editable?"input":"select"}))},w.getText=function(t){return Object(u.b)(Object.assign({},t,{body:new M(t),buttons:[u.a.cancelButton({label:t.cancelLabel}),u.a.okButton({label:t.okLabel})],focusNodeSelector:"input"}))};class x extends l.Widget{constructor(t){if(super(),this.addClass(b),void 0!==t){let e=document.createElement("label");e.textContent=t,this.node.appendChild(e)}}}class _ extends x{constructor(t){super(t.label),this._input=document.createElement("input"),this._input.classList.add("jp-mod-styled"),this._input.type="checkbox",this._input.checked=!!t.value,this.node.appendChild(this._input)}getValue(){return this._input.checked}}class D extends x{constructor(t){super(t.label),this._input=document.createElement("input",{}),this._input.classList.add("jp-mod-styled"),this._input.type="number",this._input.value=t.value?t.value.toString():"0",this.node.appendChild(this._input)}getValue(){return this._input.value?Number(this._input.value):Number.NaN}}class M extends x{constructor(t){super(t.label),this._input=document.createElement("input",{}),this._input.classList.add("jp-mod-styled"),this._input.type="text",this._input.value=t.text?t.text:"",t.placeholder&&(this._input.placeholder=t.placeholder),this.node.appendChild(this._input)}getValue(){return this._input.value}}class k extends x{constructor(t){super(t.label),this._editable=t.editable||!1;let e,n=t.current||0;if("number"==typeof n&&(e=Math.max(0,Math.min(n,t.items.length-1)),n=""),this._list=document.createElement("select"),t.items.forEach((t,r)=>{let i=document.createElement("option");r===e&&(i.selected=!0,n=t),i.value=t,i.textContent=t,this._list.appendChild(i)}),t.editable){let e=document.createElement("datalist");e.id="input-dialog-items",e.appendChild(this._list),this._input=document.createElement("input",{}),this._input.classList.add("jp-mod-styled"),this._input.type="list",this._input.value=n,this._input.setAttribute("list",e.id),t.placeholder&&(this._input.placeholder=t.placeholder),this.node.appendChild(this._input),this.node.appendChild(e)}else this.node.appendChild(B.a.wrapSelect(this._list))}getValue(){return this._editable?this._input.value:this._list.value}}var S=n(614),L=n(195),R=n(592),N=n(244);const F=new a.Token("@jupyterlab/apputils:ISplashScreen");var T=n(50);const U=75,O=20;class G{constructor(t){this._current=null,this._links=[],this._overrides={},this._overrideProps={},this._outstanding=null,this._pending=0,this._requests={},this._themes={},this._themeChanged=new A.Signal(this);const{host:e,key:n,splash:r,url:i}=t,o=t.settings;this._base=i,this._host=e,this._splash=r||null,o.load(n).then(t=>{this._settings=t,this._settings.changed.connect(this._loadSettings,this),this._loadSettings(),this._initOverrideProps()})}get theme(){return this._current}get themes(){return Object.keys(this._themes)}get themeChanged(){return this._themeChanged}getCSS(t){return getComputedStyle(document.documentElement).getPropertyValue(`--jp-${t}`)}loadCSS(t){const e=this._base,n=o.URLExt.isLocal(t)?o.URLExt.join(e,t):t,r=this._links;return new Promise((t,e)=>{const i=document.createElement("link");i.setAttribute("rel","stylesheet"),i.setAttribute("type","text/css"),i.setAttribute("href",n),i.addEventListener("load",()=>{t(void 0)}),i.addEventListener("error",()=>{e(`Stylesheet failed to load: ${n}`)}),document.body.appendChild(i),r.push(i)})}loadCSSOverrides(){const t=this._settings.user.overrides||{};Object.keys(Object.assign({},this._overrides,t)).forEach(e=>{const n=t[e];n&&this.validateCSS(e,n)?document.documentElement.style.setProperty(`--jp-${e}`,n):document.documentElement.style.removeProperty(`--jp-${e}`)}),this._overrides=t}validateCSS(t,e){const n=this._overrideProps[t];return n?!!CSS.supports(n,e)||(console.warn("CSS validation failed: invalid value.\n"+`key: '${t}', val: '${e}', prop: '${n}'`),!1):(console.warn("CSS validation failed: could not find property corresponding to key.\n"+`key: '${t}', val: '${e}'`),!1)}register(t){const{name:e}=t,n=this._themes;if(n[e])throw new Error(`Theme already registered for ${e}`);return n[e]=t,new T.DisposableDelegate(()=>{delete n[e]})}setCSSOverride(t,e){return this._overrides[t]=e,this._settings.set("overrides",this._overrides)}setTheme(t){return this._settings.set("theme",t)}isLight(t){return this._themes[t].isLight}incrFontSize(t){return this._incrFontSize(t,!0)}decrFontSize(t){return this._incrFontSize(t,!1)}themeScrollbars(t){return!!this._settings.composite["theme-scrollbars"]&&!!this._themes[t].themeScrollbars}isToggledThemeScrollbars(){return!!this._settings.composite["theme-scrollbars"]}toggleThemeScrollbars(){return this._settings.set("theme-scrollbars",!this._settings.composite["theme-scrollbars"])}_incrFontSize(t,e=!0){const n=(this.getCSS(t)||"13px").split(/([a-zA-Z]+)/),r=(e?1:-1)*("em"===n[1]?.1:1);return this.setCSSOverride(t,`${Number(n[0])+r}${n[1]}`)}_initOverrideProps(){const t=this._settings.schema.definitions;if(t&&t.cssOverrides){const e=t.cssOverrides.properties;Object.keys(e).forEach(t=>{this._overrideProps[t]=e[t].description})}}_loadSettings(){const t=this._outstanding,e=this._pending,n=this._requests;e&&(window.clearTimeout(e),this._pending=0);const r=this._settings,i=this._themes,o=r.composite.theme;if(t)return t.then(()=>{this._loadSettings()}).catch(()=>{this._loadSettings()}),void(this._outstanding=null);if(n[o]=n[o]?n[o]+1:1,i[o])return this._outstanding=this._loadTheme(o),void delete n[o];if(n[o]>O){const t=r.default("theme");return delete n[o],i[t]?(console.warn(`Could not load theme ${o}, using default ${t}.`),void(this._outstanding=this._loadTheme(t))):void this._onError(`Neither theme ${o} nor default ${t} loaded.`)}this._pending=window.setTimeout(()=>{this._loadSettings()},U)}_loadTheme(t){const e=this._current,n=this._links,r=this._themes,i=this._splash?this._splash.show(r[t].isLight):new T.DisposableDelegate(()=>void 0);n.forEach(t=>{t.parentElement&&t.parentElement.removeChild(t)}),n.length=0;const o=e?r[e].unload():Promise.resolve();return Promise.all([o,r[t].load()]).then(()=>{this._current=t,this._themeChanged.emit({name:"theme",oldValue:e,newValue:t}),this._host.hide(),requestAnimationFrame(()=>{this._host.show(),H.fitAll(this._host),i.dispose()})}).catch(t=>{this._onError(t),i.dispose()})}_onError(t){Object(u.b)({title:"Error Loading Theme",body:String(t),buttons:[u.a.okButton({label:"OK"})]})}}var H;(H||(H={})).fitAll=function t(e){Object(s.each)(e.children(),t),e.fit()};const j=new a.Token("@jupyterlab/apputils:IThemeManager");var Y=n(245),P=n(76),K=n(243);const z=new a.Token("@jupyterlab/apputils:IWindowResolver");class J{constructor(){this._name=null}get name(){return this._name}resolve(t){return q.resolve(t).then(t=>{this._name=t})}}var q;!function(t){const e="@jupyterlab/coreutils:StateDB",n=`${e}:beacon`,r=Math.floor(200+300*Math.random()),i=`${e}:window`;let o=null,s=new a.PromiseDelegate,A={},l=null,c=!1;function u(t){if(null===t)return;const{localStorage:e}=window;e.setItem(i,`${t}-${(new Date).getTime()}`)}function g(){c=!0,s.reject(`Window name candidate "${o}" already exists`)}t.resolve=function(t){if(c)return s.promise;if((o=t)in A)return g(),s.promise;const{localStorage:e,setTimeout:i}=window;return i(()=>{if(!c){if(o in A)return g();c=!0,s.resolve(l=o),u(l)}},r),e.setItem(n,`${Math.random()}-${(new Date).getTime()}`),s.promise},window.addEventListener("storage",t=>{const{key:e,newValue:r}=t;if(null===r)return;if(e===n&&null!==o)return void u(c?l:o);if(c||e!==i)return;const a=r.replace(/\-\d+$/,"");A[a]=null,o in A&&g()})}(q||(q={})),n.d(e,"ClientSession",function(){return g}),n.d(e,"Clipboard",function(){return h.a}),n.d(e,"Collapse",function(){return d.a}),n.d(e,"CommandLinker",function(){return I}),n.d(e,"ICommandPalette",function(){return C}),n.d(e,"showDialog",function(){return u.b}),n.d(e,"showErrorMessage",function(){return u.c}),n.d(e,"Dialog",function(){return u.a}),n.d(e,"DOMUtils",function(){return y.a}),n.d(e,"HoverBox",function(){return E.a}),n.d(e,"IFrame",function(){return v}),n.d(e,"InputDialog",function(){return Q}),n.d(e,"MainAreaWidget",function(){return S.a}),n.d(e,"Printing",function(){return L.a}),n.d(e,"defaultSanitizer",function(){return R.a}),n.d(e,"Spinner",function(){return N.a}),n.d(e,"ISplashScreen",function(){return F}),n.d(e,"Styling",function(){return B.a}),n.d(e,"ThemeManager",function(){return G}),n.d(e,"IThemeManager",function(){return j}),n.d(e,"Toolbar",function(){return Y.c}),n.d(e,"ToolbarButtonComponent",function(){return Y.e}),n.d(e,"addToolbarButtonClass",function(){return Y.g}),n.d(e,"ToolbarButton",function(){return Y.d}),n.d(e,"CommandToolbarButtonComponent",function(){return Y.b}),n.d(e,"addCommandToolbarButtonClass",function(){return Y.f}),n.d(e,"CommandToolbarButton",function(){return Y.a}),n.d(e,"ReactWidget",function(){return P.a}),n.d(e,"VDomRenderer",function(){return P.d}),n.d(e,"UseSignal",function(){return P.b}),n.d(e,"VDomModel",function(){return P.c}),n.d(e,"WidgetTracker",function(){return K.a}),n.d(e,"IWindowResolver",function(){return z}),n.d(e,"WindowResolver",function(){return J})},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o=n(5),a=n(10),s=function(){function t(t){this._fn=t}return Object.defineProperty(t.prototype,"isDisposed",{get:function(){return!this._fn},enumerable:!0,configurable:!0}),t.prototype.dispose=function(){if(this._fn){var t=this._fn;this._fn=null,t()}},t}();e.DisposableDelegate=s;var A=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._disposed=new a.Signal(e),e}return i(e,t),Object.defineProperty(e.prototype,"disposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this.isDisposed||(t.prototype.dispose.call(this),this._disposed.emit(void 0),a.Signal.clearData(this))},e}(s);e.ObservableDisposableDelegate=A;var l=function(){function t(){this._isDisposed=!1,this._items=new Set}return Object.defineProperty(t.prototype,"isDisposed",{get:function(){return this._isDisposed},enumerable:!0,configurable:!0}),t.prototype.dispose=function(){this._isDisposed||(this._isDisposed=!0,this._items.forEach(function(t){t.dispose()}),this._items.clear())},t.prototype.contains=function(t){return this._items.has(t)},t.prototype.add=function(t){this._items.add(t)},t.prototype.remove=function(t){this._items.delete(t)},t.prototype.clear=function(){this._items.clear()},t}();e.DisposableSet=l,function(t){t.from=function(e){var n=new t;return o.each(e,function(t){n.add(t)}),n}}(l=e.DisposableSet||(e.DisposableSet={})),e.DisposableSet=l;var c=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._disposed=new a.Signal(e),e}return i(e,t),Object.defineProperty(e.prototype,"disposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this.isDisposed||(t.prototype.dispose.call(this),this._disposed.emit(void 0),a.Signal.clearData(this))},e}(l);e.ObservableDisposableSet=c,function(t){t.from=function(e){var n=new t;return o.each(e,function(t){n.add(t)}),n}}(c=e.ObservableDisposableSet||(e.ObservableDisposableSet={})),e.ObservableDisposableSet=c},function(t,e,n){"use strict";n.d(e,"a",function(){return i}),n.d(e,"b",function(){return o});var r=n(21),i=new r.a(function(t){return t.complete()});function o(t){return t?function(t){return new r.a(function(e){return t.schedule(function(){return e.complete()})})}(t):i}},function(t,e){t.exports=function(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t}},function(t,e,n){(function(t){!function(t,e){"use strict";function r(t,e){if(!t)throw new Error(e||"Assertion failed")}function i(t,e){t.super_=e;var n=function(){};n.prototype=e.prototype,t.prototype=new n,t.prototype.constructor=t}function o(t,e,n){if(o.isBN(t))return t;this.negative=0,this.words=null,this.length=0,this.red=null,null!==t&&("le"!==e&&"be"!==e||(n=e,e=10),this._init(t||0,e||10,n||"be"))}var a;"object"==typeof t?t.exports=o:e.BN=o,o.BN=o,o.wordSize=26;try{a=n(807).Buffer}catch(t){}function s(t,e,n){for(var r=0,i=Math.min(t.length,n),o=e;o<i;o++){var a=t.charCodeAt(o)-48;r<<=4,r|=a>=49&&a<=54?a-49+10:a>=17&&a<=22?a-17+10:15&a}return r}function A(t,e,n,r){for(var i=0,o=Math.min(t.length,n),a=e;a<o;a++){var s=t.charCodeAt(a)-48;i*=r,i+=s>=49?s-49+10:s>=17?s-17+10:s}return i}o.isBN=function(t){return t instanceof o||null!==t&&"object"==typeof t&&t.constructor.wordSize===o.wordSize&&Array.isArray(t.words)},o.max=function(t,e){return t.cmp(e)>0?t:e},o.min=function(t,e){return t.cmp(e)<0?t:e},o.prototype._init=function(t,e,n){if("number"==typeof t)return this._initNumber(t,e,n);if("object"==typeof t)return this._initArray(t,e,n);"hex"===e&&(e=16),r(e===(0|e)&&e>=2&&e<=36);var i=0;"-"===(t=t.toString().replace(/\s+/g,""))[0]&&i++,16===e?this._parseHex(t,i):this._parseBase(t,e,i),"-"===t[0]&&(this.negative=1),this.strip(),"le"===n&&this._initArray(this.toArray(),e,n)},o.prototype._initNumber=function(t,e,n){t<0&&(this.negative=1,t=-t),t<67108864?(this.words=[67108863&t],this.length=1):t<4503599627370496?(this.words=[67108863&t,t/67108864&67108863],this.length=2):(r(t<9007199254740992),this.words=[67108863&t,t/67108864&67108863,1],this.length=3),"le"===n&&this._initArray(this.toArray(),e,n)},o.prototype._initArray=function(t,e,n){if(r("number"==typeof t.length),t.length<=0)return this.words=[0],this.length=1,this;this.length=Math.ceil(t.length/3),this.words=new Array(this.length);for(var i=0;i<this.length;i++)this.words[i]=0;var o,a,s=0;if("be"===n)for(i=t.length-1,o=0;i>=0;i-=3)a=t[i]|t[i-1]<<8|t[i-2]<<16,this.words[o]|=a<<s&67108863,this.words[o+1]=a>>>26-s&67108863,(s+=24)>=26&&(s-=26,o++);else if("le"===n)for(i=0,o=0;i<t.length;i+=3)a=t[i]|t[i+1]<<8|t[i+2]<<16,this.words[o]|=a<<s&67108863,this.words[o+1]=a>>>26-s&67108863,(s+=24)>=26&&(s-=26,o++);return this.strip()},o.prototype._parseHex=function(t,e){this.length=Math.ceil((t.length-e)/6),this.words=new Array(this.length);for(var n=0;n<this.length;n++)this.words[n]=0;var r,i,o=0;for(n=t.length-6,r=0;n>=e;n-=6)i=s(t,n,n+6),this.words[r]|=i<<o&67108863,this.words[r+1]|=i>>>26-o&4194303,(o+=24)>=26&&(o-=26,r++);n+6!==e&&(i=s(t,e,n+6),this.words[r]|=i<<o&67108863,this.words[r+1]|=i>>>26-o&4194303),this.strip()},o.prototype._parseBase=function(t,e,n){this.words=[0],this.length=1;for(var r=0,i=1;i<=67108863;i*=e)r++;r--,i=i/e|0;for(var o=t.length-n,a=o%r,s=Math.min(o,o-a)+n,l=0,c=n;c<s;c+=r)l=A(t,c,c+r,e),this.imuln(i),this.words[0]+l<67108864?this.words[0]+=l:this._iaddn(l);if(0!==a){var u=1;for(l=A(t,c,t.length,e),c=0;c<a;c++)u*=e;this.imuln(u),this.words[0]+l<67108864?this.words[0]+=l:this._iaddn(l)}},o.prototype.copy=function(t){t.words=new Array(this.length);for(var e=0;e<this.length;e++)t.words[e]=this.words[e];t.length=this.length,t.negative=this.negative,t.red=this.red},o.prototype.clone=function(){var t=new o(null);return this.copy(t),t},o.prototype._expand=function(t){for(;this.length<t;)this.words[this.length++]=0;return this},o.prototype.strip=function(){for(;this.length>1&&0===this.words[this.length-1];)this.length--;return this._normSign()},o.prototype._normSign=function(){return 1===this.length&&0===this.words[0]&&(this.negative=0),this},o.prototype.inspect=function(){return(this.red?"<BN-R: ":"<BN: ")+this.toString(16)+">"};var l=["","0","00","000","0000","00000","000000","0000000","00000000","000000000","0000000000","00000000000","000000000000","0000000000000","00000000000000","000000000000000","0000000000000000","00000000000000000","000000000000000000","0000000000000000000","00000000000000000000","000000000000000000000","0000000000000000000000","00000000000000000000000","000000000000000000000000","0000000000000000000000000"],c=[0,0,25,16,12,11,10,9,8,8,7,7,7,7,6,6,6,6,6,6,6,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5],u=[0,0,33554432,43046721,16777216,48828125,60466176,40353607,16777216,43046721,1e7,19487171,35831808,62748517,7529536,11390625,16777216,24137569,34012224,47045881,64e6,4084101,5153632,6436343,7962624,9765625,11881376,14348907,17210368,20511149,243e5,28629151,33554432,39135393,45435424,52521875,60466176];function g(t,e,n){n.negative=e.negative^t.negative;var r=t.length+e.length|0;n.length=r,r=r-1|0;var i=0|t.words[0],o=0|e.words[0],a=i*o,s=67108863&a,A=a/67108864|0;n.words[0]=s;for(var l=1;l<r;l++){for(var c=A>>>26,u=67108863&A,g=Math.min(l,e.length-1),h=Math.max(0,l-t.length+1);h<=g;h++){var d=l-h|0;c+=(a=(i=0|t.words[d])*(o=0|e.words[h])+u)/67108864|0,u=67108863&a}n.words[l]=0|u,A=0|c}return 0!==A?n.words[l]=0|A:n.length--,n.strip()}o.prototype.toString=function(t,e){var n;if(e=0|e||1,16===(t=t||10)||"hex"===t){n="";for(var i=0,o=0,a=0;a<this.length;a++){var s=this.words[a],A=(16777215&(s<<i|o)).toString(16);n=0!==(o=s>>>24-i&16777215)||a!==this.length-1?l[6-A.length]+A+n:A+n,(i+=2)>=26&&(i-=26,a--)}for(0!==o&&(n=o.toString(16)+n);n.length%e!=0;)n="0"+n;return 0!==this.negative&&(n="-"+n),n}if(t===(0|t)&&t>=2&&t<=36){var g=c[t],h=u[t];n="";var d=this.clone();for(d.negative=0;!d.isZero();){var f=d.modn(h).toString(t);n=(d=d.idivn(h)).isZero()?f+n:l[g-f.length]+f+n}for(this.isZero()&&(n="0"+n);n.length%e!=0;)n="0"+n;return 0!==this.negative&&(n="-"+n),n}r(!1,"Base should be between 2 and 36")},o.prototype.toNumber=function(){var t=this.words[0];return 2===this.length?t+=67108864*this.words[1]:3===this.length&&1===this.words[2]?t+=4503599627370496+67108864*this.words[1]:this.length>2&&r(!1,"Number can only safely store up to 53 bits"),0!==this.negative?-t:t},o.prototype.toJSON=function(){return this.toString(16)},o.prototype.toBuffer=function(t,e){return r(void 0!==a),this.toArrayLike(a,t,e)},o.prototype.toArray=function(t,e){return this.toArrayLike(Array,t,e)},o.prototype.toArrayLike=function(t,e,n){var i=this.byteLength(),o=n||Math.max(1,i);r(i<=o,"byte array longer than desired length"),r(o>0,"Requested array length <= 0"),this.strip();var a,s,A="le"===e,l=new t(o),c=this.clone();if(A){for(s=0;!c.isZero();s++)a=c.andln(255),c.iushrn(8),l[s]=a;for(;s<o;s++)l[s]=0}else{for(s=0;s<o-i;s++)l[s]=0;for(s=0;!c.isZero();s++)a=c.andln(255),c.iushrn(8),l[o-s-1]=a}return l},Math.clz32?o.prototype._countBits=function(t){return 32-Math.clz32(t)}:o.prototype._countBits=function(t){var e=t,n=0;return e>=4096&&(n+=13,e>>>=13),e>=64&&(n+=7,e>>>=7),e>=8&&(n+=4,e>>>=4),e>=2&&(n+=2,e>>>=2),n+e},o.prototype._zeroBits=function(t){if(0===t)return 26;var e=t,n=0;return 0==(8191&e)&&(n+=13,e>>>=13),0==(127&e)&&(n+=7,e>>>=7),0==(15&e)&&(n+=4,e>>>=4),0==(3&e)&&(n+=2,e>>>=2),0==(1&e)&&n++,n},o.prototype.bitLength=function(){var t=this.words[this.length-1],e=this._countBits(t);return 26*(this.length-1)+e},o.prototype.zeroBits=function(){if(this.isZero())return 0;for(var t=0,e=0;e<this.length;e++){var n=this._zeroBits(this.words[e]);if(t+=n,26!==n)break}return t},o.prototype.byteLength=function(){return Math.ceil(this.bitLength()/8)},o.prototype.toTwos=function(t){return 0!==this.negative?this.abs().inotn(t).iaddn(1):this.clone()},o.prototype.fromTwos=function(t){return this.testn(t-1)?this.notn(t).iaddn(1).ineg():this.clone()},o.prototype.isNeg=function(){return 0!==this.negative},o.prototype.neg=function(){return this.clone().ineg()},o.prototype.ineg=function(){return this.isZero()||(this.negative^=1),this},o.prototype.iuor=function(t){for(;this.length<t.length;)this.words[this.length++]=0;for(var e=0;e<t.length;e++)this.words[e]=this.words[e]|t.words[e];return this.strip()},o.prototype.ior=function(t){return r(0==(this.negative|t.negative)),this.iuor(t)},o.prototype.or=function(t){return this.length>t.length?this.clone().ior(t):t.clone().ior(this)},o.prototype.uor=function(t){return this.length>t.length?this.clone().iuor(t):t.clone().iuor(this)},o.prototype.iuand=function(t){var e;e=this.length>t.length?t:this;for(var n=0;n<e.length;n++)this.words[n]=this.words[n]&t.words[n];return this.length=e.length,this.strip()},o.prototype.iand=function(t){return r(0==(this.negative|t.negative)),this.iuand(t)},o.prototype.and=function(t){return this.length>t.length?this.clone().iand(t):t.clone().iand(this)},o.prototype.uand=function(t){return this.length>t.length?this.clone().iuand(t):t.clone().iuand(this)},o.prototype.iuxor=function(t){var e,n;this.length>t.length?(e=this,n=t):(e=t,n=this);for(var r=0;r<n.length;r++)this.words[r]=e.words[r]^n.words[r];if(this!==e)for(;r<e.length;r++)this.words[r]=e.words[r];return this.length=e.length,this.strip()},o.prototype.ixor=function(t){return r(0==(this.negative|t.negative)),this.iuxor(t)},o.prototype.xor=function(t){return this.length>t.length?this.clone().ixor(t):t.clone().ixor(this)},o.prototype.uxor=function(t){return this.length>t.length?this.clone().iuxor(t):t.clone().iuxor(this)},o.prototype.inotn=function(t){r("number"==typeof t&&t>=0);var e=0|Math.ceil(t/26),n=t%26;this._expand(e),n>0&&e--;for(var i=0;i<e;i++)this.words[i]=67108863&~this.words[i];return n>0&&(this.words[i]=~this.words[i]&67108863>>26-n),this.strip()},o.prototype.notn=function(t){return this.clone().inotn(t)},o.prototype.setn=function(t,e){r("number"==typeof t&&t>=0);var n=t/26|0,i=t%26;return this._expand(n+1),this.words[n]=e?this.words[n]|1<<i:this.words[n]&~(1<<i),this.strip()},o.prototype.iadd=function(t){var e,n,r;if(0!==this.negative&&0===t.negative)return this.negative=0,e=this.isub(t),this.negative^=1,this._normSign();if(0===this.negative&&0!==t.negative)return t.negative=0,e=this.isub(t),t.negative=1,e._normSign();this.length>t.length?(n=this,r=t):(n=t,r=this);for(var i=0,o=0;o<r.length;o++)e=(0|n.words[o])+(0|r.words[o])+i,this.words[o]=67108863&e,i=e>>>26;for(;0!==i&&o<n.length;o++)e=(0|n.words[o])+i,this.words[o]=67108863&e,i=e>>>26;if(this.length=n.length,0!==i)this.words[this.length]=i,this.length++;else if(n!==this)for(;o<n.length;o++)this.words[o]=n.words[o];return this},o.prototype.add=function(t){var e;return 0!==t.negative&&0===this.negative?(t.negative=0,e=this.sub(t),t.negative^=1,e):0===t.negative&&0!==this.negative?(this.negative=0,e=t.sub(this),this.negative=1,e):this.length>t.length?this.clone().iadd(t):t.clone().iadd(this)},o.prototype.isub=function(t){if(0!==t.negative){t.negative=0;var e=this.iadd(t);return t.negative=1,e._normSign()}if(0!==this.negative)return this.negative=0,this.iadd(t),this.negative=1,this._normSign();var n,r,i=this.cmp(t);if(0===i)return this.negative=0,this.length=1,this.words[0]=0,this;i>0?(n=this,r=t):(n=t,r=this);for(var o=0,a=0;a<r.length;a++)o=(e=(0|n.words[a])-(0|r.words[a])+o)>>26,this.words[a]=67108863&e;for(;0!==o&&a<n.length;a++)o=(e=(0|n.words[a])+o)>>26,this.words[a]=67108863&e;if(0===o&&a<n.length&&n!==this)for(;a<n.length;a++)this.words[a]=n.words[a];return this.length=Math.max(this.length,a),n!==this&&(this.negative=1),this.strip()},o.prototype.sub=function(t){return this.clone().isub(t)};var h=function(t,e,n){var r,i,o,a=t.words,s=e.words,A=n.words,l=0,c=0|a[0],u=8191&c,g=c>>>13,h=0|a[1],d=8191&h,f=h>>>13,p=0|a[2],I=8191&p,C=p>>>13,m=0|a[3],y=8191&m,E=m>>>13,v=0|a[4],B=8191&v,b=v>>>13,Q=0|a[5],w=8191&Q,x=Q>>>13,_=0|a[6],D=8191&_,M=_>>>13,k=0|a[7],S=8191&k,L=k>>>13,R=0|a[8],N=8191&R,F=R>>>13,T=0|a[9],U=8191&T,O=T>>>13,G=0|s[0],H=8191&G,j=G>>>13,Y=0|s[1],P=8191&Y,K=Y>>>13,z=0|s[2],J=8191&z,q=z>>>13,V=0|s[3],W=8191&V,X=V>>>13,Z=0|s[4],$=8191&Z,tt=Z>>>13,et=0|s[5],nt=8191&et,rt=et>>>13,it=0|s[6],ot=8191&it,at=it>>>13,st=0|s[7],At=8191&st,lt=st>>>13,ct=0|s[8],ut=8191&ct,gt=ct>>>13,ht=0|s[9],dt=8191&ht,ft=ht>>>13;n.negative=t.negative^e.negative,n.length=19;var pt=(l+(r=Math.imul(u,H))|0)+((8191&(i=(i=Math.imul(u,j))+Math.imul(g,H)|0))<<13)|0;l=((o=Math.imul(g,j))+(i>>>13)|0)+(pt>>>26)|0,pt&=67108863,r=Math.imul(d,H),i=(i=Math.imul(d,j))+Math.imul(f,H)|0,o=Math.imul(f,j);var It=(l+(r=r+Math.imul(u,P)|0)|0)+((8191&(i=(i=i+Math.imul(u,K)|0)+Math.imul(g,P)|0))<<13)|0;l=((o=o+Math.imul(g,K)|0)+(i>>>13)|0)+(It>>>26)|0,It&=67108863,r=Math.imul(I,H),i=(i=Math.imul(I,j))+Math.imul(C,H)|0,o=Math.imul(C,j),r=r+Math.imul(d,P)|0,i=(i=i+Math.imul(d,K)|0)+Math.imul(f,P)|0,o=o+Math.imul(f,K)|0;var Ct=(l+(r=r+Math.imul(u,J)|0)|0)+((8191&(i=(i=i+Math.imul(u,q)|0)+Math.imul(g,J)|0))<<13)|0;l=((o=o+Math.imul(g,q)|0)+(i>>>13)|0)+(Ct>>>26)|0,Ct&=67108863,r=Math.imul(y,H),i=(i=Math.imul(y,j))+Math.imul(E,H)|0,o=Math.imul(E,j),r=r+Math.imul(I,P)|0,i=(i=i+Math.imul(I,K)|0)+Math.imul(C,P)|0,o=o+Math.imul(C,K)|0,r=r+Math.imul(d,J)|0,i=(i=i+Math.imul(d,q)|0)+Math.imul(f,J)|0,o=o+Math.imul(f,q)|0;var mt=(l+(r=r+Math.imul(u,W)|0)|0)+((8191&(i=(i=i+Math.imul(u,X)|0)+Math.imul(g,W)|0))<<13)|0;l=((o=o+Math.imul(g,X)|0)+(i>>>13)|0)+(mt>>>26)|0,mt&=67108863,r=Math.imul(B,H),i=(i=Math.imul(B,j))+Math.imul(b,H)|0,o=Math.imul(b,j),r=r+Math.imul(y,P)|0,i=(i=i+Math.imul(y,K)|0)+Math.imul(E,P)|0,o=o+Math.imul(E,K)|0,r=r+Math.imul(I,J)|0,i=(i=i+Math.imul(I,q)|0)+Math.imul(C,J)|0,o=o+Math.imul(C,q)|0,r=r+Math.imul(d,W)|0,i=(i=i+Math.imul(d,X)|0)+Math.imul(f,W)|0,o=o+Math.imul(f,X)|0;var yt=(l+(r=r+Math.imul(u,$)|0)|0)+((8191&(i=(i=i+Math.imul(u,tt)|0)+Math.imul(g,$)|0))<<13)|0;l=((o=o+Math.imul(g,tt)|0)+(i>>>13)|0)+(yt>>>26)|0,yt&=67108863,r=Math.imul(w,H),i=(i=Math.imul(w,j))+Math.imul(x,H)|0,o=Math.imul(x,j),r=r+Math.imul(B,P)|0,i=(i=i+Math.imul(B,K)|0)+Math.imul(b,P)|0,o=o+Math.imul(b,K)|0,r=r+Math.imul(y,J)|0,i=(i=i+Math.imul(y,q)|0)+Math.imul(E,J)|0,o=o+Math.imul(E,q)|0,r=r+Math.imul(I,W)|0,i=(i=i+Math.imul(I,X)|0)+Math.imul(C,W)|0,o=o+Math.imul(C,X)|0,r=r+Math.imul(d,$)|0,i=(i=i+Math.imul(d,tt)|0)+Math.imul(f,$)|0,o=o+Math.imul(f,tt)|0;var Et=(l+(r=r+Math.imul(u,nt)|0)|0)+((8191&(i=(i=i+Math.imul(u,rt)|0)+Math.imul(g,nt)|0))<<13)|0;l=((o=o+Math.imul(g,rt)|0)+(i>>>13)|0)+(Et>>>26)|0,Et&=67108863,r=Math.imul(D,H),i=(i=Math.imul(D,j))+Math.imul(M,H)|0,o=Math.imul(M,j),r=r+Math.imul(w,P)|0,i=(i=i+Math.imul(w,K)|0)+Math.imul(x,P)|0,o=o+Math.imul(x,K)|0,r=r+Math.imul(B,J)|0,i=(i=i+Math.imul(B,q)|0)+Math.imul(b,J)|0,o=o+Math.imul(b,q)|0,r=r+Math.imul(y,W)|0,i=(i=i+Math.imul(y,X)|0)+Math.imul(E,W)|0,o=o+Math.imul(E,X)|0,r=r+Math.imul(I,$)|0,i=(i=i+Math.imul(I,tt)|0)+Math.imul(C,$)|0,o=o+Math.imul(C,tt)|0,r=r+Math.imul(d,nt)|0,i=(i=i+Math.imul(d,rt)|0)+Math.imul(f,nt)|0,o=o+Math.imul(f,rt)|0;var vt=(l+(r=r+Math.imul(u,ot)|0)|0)+((8191&(i=(i=i+Math.imul(u,at)|0)+Math.imul(g,ot)|0))<<13)|0;l=((o=o+Math.imul(g,at)|0)+(i>>>13)|0)+(vt>>>26)|0,vt&=67108863,r=Math.imul(S,H),i=(i=Math.imul(S,j))+Math.imul(L,H)|0,o=Math.imul(L,j),r=r+Math.imul(D,P)|0,i=(i=i+Math.imul(D,K)|0)+Math.imul(M,P)|0,o=o+Math.imul(M,K)|0,r=r+Math.imul(w,J)|0,i=(i=i+Math.imul(w,q)|0)+Math.imul(x,J)|0,o=o+Math.imul(x,q)|0,r=r+Math.imul(B,W)|0,i=(i=i+Math.imul(B,X)|0)+Math.imul(b,W)|0,o=o+Math.imul(b,X)|0,r=r+Math.imul(y,$)|0,i=(i=i+Math.imul(y,tt)|0)+Math.imul(E,$)|0,o=o+Math.imul(E,tt)|0,r=r+Math.imul(I,nt)|0,i=(i=i+Math.imul(I,rt)|0)+Math.imul(C,nt)|0,o=o+Math.imul(C,rt)|0,r=r+Math.imul(d,ot)|0,i=(i=i+Math.imul(d,at)|0)+Math.imul(f,ot)|0,o=o+Math.imul(f,at)|0;var Bt=(l+(r=r+Math.imul(u,At)|0)|0)+((8191&(i=(i=i+Math.imul(u,lt)|0)+Math.imul(g,At)|0))<<13)|0;l=((o=o+Math.imul(g,lt)|0)+(i>>>13)|0)+(Bt>>>26)|0,Bt&=67108863,r=Math.imul(N,H),i=(i=Math.imul(N,j))+Math.imul(F,H)|0,o=Math.imul(F,j),r=r+Math.imul(S,P)|0,i=(i=i+Math.imul(S,K)|0)+Math.imul(L,P)|0,o=o+Math.imul(L,K)|0,r=r+Math.imul(D,J)|0,i=(i=i+Math.imul(D,q)|0)+Math.imul(M,J)|0,o=o+Math.imul(M,q)|0,r=r+Math.imul(w,W)|0,i=(i=i+Math.imul(w,X)|0)+Math.imul(x,W)|0,o=o+Math.imul(x,X)|0,r=r+Math.imul(B,$)|0,i=(i=i+Math.imul(B,tt)|0)+Math.imul(b,$)|0,o=o+Math.imul(b,tt)|0,r=r+Math.imul(y,nt)|0,i=(i=i+Math.imul(y,rt)|0)+Math.imul(E,nt)|0,o=o+Math.imul(E,rt)|0,r=r+Math.imul(I,ot)|0,i=(i=i+Math.imul(I,at)|0)+Math.imul(C,ot)|0,o=o+Math.imul(C,at)|0,r=r+Math.imul(d,At)|0,i=(i=i+Math.imul(d,lt)|0)+Math.imul(f,At)|0,o=o+Math.imul(f,lt)|0;var bt=(l+(r=r+Math.imul(u,ut)|0)|0)+((8191&(i=(i=i+Math.imul(u,gt)|0)+Math.imul(g,ut)|0))<<13)|0;l=((o=o+Math.imul(g,gt)|0)+(i>>>13)|0)+(bt>>>26)|0,bt&=67108863,r=Math.imul(U,H),i=(i=Math.imul(U,j))+Math.imul(O,H)|0,o=Math.imul(O,j),r=r+Math.imul(N,P)|0,i=(i=i+Math.imul(N,K)|0)+Math.imul(F,P)|0,o=o+Math.imul(F,K)|0,r=r+Math.imul(S,J)|0,i=(i=i+Math.imul(S,q)|0)+Math.imul(L,J)|0,o=o+Math.imul(L,q)|0,r=r+Math.imul(D,W)|0,i=(i=i+Math.imul(D,X)|0)+Math.imul(M,W)|0,o=o+Math.imul(M,X)|0,r=r+Math.imul(w,$)|0,i=(i=i+Math.imul(w,tt)|0)+Math.imul(x,$)|0,o=o+Math.imul(x,tt)|0,r=r+Math.imul(B,nt)|0,i=(i=i+Math.imul(B,rt)|0)+Math.imul(b,nt)|0,o=o+Math.imul(b,rt)|0,r=r+Math.imul(y,ot)|0,i=(i=i+Math.imul(y,at)|0)+Math.imul(E,ot)|0,o=o+Math.imul(E,at)|0,r=r+Math.imul(I,At)|0,i=(i=i+Math.imul(I,lt)|0)+Math.imul(C,At)|0,o=o+Math.imul(C,lt)|0,r=r+Math.imul(d,ut)|0,i=(i=i+Math.imul(d,gt)|0)+Math.imul(f,ut)|0,o=o+Math.imul(f,gt)|0;var Qt=(l+(r=r+Math.imul(u,dt)|0)|0)+((8191&(i=(i=i+Math.imul(u,ft)|0)+Math.imul(g,dt)|0))<<13)|0;l=((o=o+Math.imul(g,ft)|0)+(i>>>13)|0)+(Qt>>>26)|0,Qt&=67108863,r=Math.imul(U,P),i=(i=Math.imul(U,K))+Math.imul(O,P)|0,o=Math.imul(O,K),r=r+Math.imul(N,J)|0,i=(i=i+Math.imul(N,q)|0)+Math.imul(F,J)|0,o=o+Math.imul(F,q)|0,r=r+Math.imul(S,W)|0,i=(i=i+Math.imul(S,X)|0)+Math.imul(L,W)|0,o=o+Math.imul(L,X)|0,r=r+Math.imul(D,$)|0,i=(i=i+Math.imul(D,tt)|0)+Math.imul(M,$)|0,o=o+Math.imul(M,tt)|0,r=r+Math.imul(w,nt)|0,i=(i=i+Math.imul(w,rt)|0)+Math.imul(x,nt)|0,o=o+Math.imul(x,rt)|0,r=r+Math.imul(B,ot)|0,i=(i=i+Math.imul(B,at)|0)+Math.imul(b,ot)|0,o=o+Math.imul(b,at)|0,r=r+Math.imul(y,At)|0,i=(i=i+Math.imul(y,lt)|0)+Math.imul(E,At)|0,o=o+Math.imul(E,lt)|0,r=r+Math.imul(I,ut)|0,i=(i=i+Math.imul(I,gt)|0)+Math.imul(C,ut)|0,o=o+Math.imul(C,gt)|0;var wt=(l+(r=r+Math.imul(d,dt)|0)|0)+((8191&(i=(i=i+Math.imul(d,ft)|0)+Math.imul(f,dt)|0))<<13)|0;l=((o=o+Math.imul(f,ft)|0)+(i>>>13)|0)+(wt>>>26)|0,wt&=67108863,r=Math.imul(U,J),i=(i=Math.imul(U,q))+Math.imul(O,J)|0,o=Math.imul(O,q),r=r+Math.imul(N,W)|0,i=(i=i+Math.imul(N,X)|0)+Math.imul(F,W)|0,o=o+Math.imul(F,X)|0,r=r+Math.imul(S,$)|0,i=(i=i+Math.imul(S,tt)|0)+Math.imul(L,$)|0,o=o+Math.imul(L,tt)|0,r=r+Math.imul(D,nt)|0,i=(i=i+Math.imul(D,rt)|0)+Math.imul(M,nt)|0,o=o+Math.imul(M,rt)|0,r=r+Math.imul(w,ot)|0,i=(i=i+Math.imul(w,at)|0)+Math.imul(x,ot)|0,o=o+Math.imul(x,at)|0,r=r+Math.imul(B,At)|0,i=(i=i+Math.imul(B,lt)|0)+Math.imul(b,At)|0,o=o+Math.imul(b,lt)|0,r=r+Math.imul(y,ut)|0,i=(i=i+Math.imul(y,gt)|0)+Math.imul(E,ut)|0,o=o+Math.imul(E,gt)|0;var xt=(l+(r=r+Math.imul(I,dt)|0)|0)+((8191&(i=(i=i+Math.imul(I,ft)|0)+Math.imul(C,dt)|0))<<13)|0;l=((o=o+Math.imul(C,ft)|0)+(i>>>13)|0)+(xt>>>26)|0,xt&=67108863,r=Math.imul(U,W),i=(i=Math.imul(U,X))+Math.imul(O,W)|0,o=Math.imul(O,X),r=r+Math.imul(N,$)|0,i=(i=i+Math.imul(N,tt)|0)+Math.imul(F,$)|0,o=o+Math.imul(F,tt)|0,r=r+Math.imul(S,nt)|0,i=(i=i+Math.imul(S,rt)|0)+Math.imul(L,nt)|0,o=o+Math.imul(L,rt)|0,r=r+Math.imul(D,ot)|0,i=(i=i+Math.imul(D,at)|0)+Math.imul(M,ot)|0,o=o+Math.imul(M,at)|0,r=r+Math.imul(w,At)|0,i=(i=i+Math.imul(w,lt)|0)+Math.imul(x,At)|0,o=o+Math.imul(x,lt)|0,r=r+Math.imul(B,ut)|0,i=(i=i+Math.imul(B,gt)|0)+Math.imul(b,ut)|0,o=o+Math.imul(b,gt)|0;var _t=(l+(r=r+Math.imul(y,dt)|0)|0)+((8191&(i=(i=i+Math.imul(y,ft)|0)+Math.imul(E,dt)|0))<<13)|0;l=((o=o+Math.imul(E,ft)|0)+(i>>>13)|0)+(_t>>>26)|0,_t&=67108863,r=Math.imul(U,$),i=(i=Math.imul(U,tt))+Math.imul(O,$)|0,o=Math.imul(O,tt),r=r+Math.imul(N,nt)|0,i=(i=i+Math.imul(N,rt)|0)+Math.imul(F,nt)|0,o=o+Math.imul(F,rt)|0,r=r+Math.imul(S,ot)|0,i=(i=i+Math.imul(S,at)|0)+Math.imul(L,ot)|0,o=o+Math.imul(L,at)|0,r=r+Math.imul(D,At)|0,i=(i=i+Math.imul(D,lt)|0)+Math.imul(M,At)|0,o=o+Math.imul(M,lt)|0,r=r+Math.imul(w,ut)|0,i=(i=i+Math.imul(w,gt)|0)+Math.imul(x,ut)|0,o=o+Math.imul(x,gt)|0;var Dt=(l+(r=r+Math.imul(B,dt)|0)|0)+((8191&(i=(i=i+Math.imul(B,ft)|0)+Math.imul(b,dt)|0))<<13)|0;l=((o=o+Math.imul(b,ft)|0)+(i>>>13)|0)+(Dt>>>26)|0,Dt&=67108863,r=Math.imul(U,nt),i=(i=Math.imul(U,rt))+Math.imul(O,nt)|0,o=Math.imul(O,rt),r=r+Math.imul(N,ot)|0,i=(i=i+Math.imul(N,at)|0)+Math.imul(F,ot)|0,o=o+Math.imul(F,at)|0,r=r+Math.imul(S,At)|0,i=(i=i+Math.imul(S,lt)|0)+Math.imul(L,At)|0,o=o+Math.imul(L,lt)|0,r=r+Math.imul(D,ut)|0,i=(i=i+Math.imul(D,gt)|0)+Math.imul(M,ut)|0,o=o+Math.imul(M,gt)|0;var Mt=(l+(r=r+Math.imul(w,dt)|0)|0)+((8191&(i=(i=i+Math.imul(w,ft)|0)+Math.imul(x,dt)|0))<<13)|0;l=((o=o+Math.imul(x,ft)|0)+(i>>>13)|0)+(Mt>>>26)|0,Mt&=67108863,r=Math.imul(U,ot),i=(i=Math.imul(U,at))+Math.imul(O,ot)|0,o=Math.imul(O,at),r=r+Math.imul(N,At)|0,i=(i=i+Math.imul(N,lt)|0)+Math.imul(F,At)|0,o=o+Math.imul(F,lt)|0,r=r+Math.imul(S,ut)|0,i=(i=i+Math.imul(S,gt)|0)+Math.imul(L,ut)|0,o=o+Math.imul(L,gt)|0;var kt=(l+(r=r+Math.imul(D,dt)|0)|0)+((8191&(i=(i=i+Math.imul(D,ft)|0)+Math.imul(M,dt)|0))<<13)|0;l=((o=o+Math.imul(M,ft)|0)+(i>>>13)|0)+(kt>>>26)|0,kt&=67108863,r=Math.imul(U,At),i=(i=Math.imul(U,lt))+Math.imul(O,At)|0,o=Math.imul(O,lt),r=r+Math.imul(N,ut)|0,i=(i=i+Math.imul(N,gt)|0)+Math.imul(F,ut)|0,o=o+Math.imul(F,gt)|0;var St=(l+(r=r+Math.imul(S,dt)|0)|0)+((8191&(i=(i=i+Math.imul(S,ft)|0)+Math.imul(L,dt)|0))<<13)|0;l=((o=o+Math.imul(L,ft)|0)+(i>>>13)|0)+(St>>>26)|0,St&=67108863,r=Math.imul(U,ut),i=(i=Math.imul(U,gt))+Math.imul(O,ut)|0,o=Math.imul(O,gt);var Lt=(l+(r=r+Math.imul(N,dt)|0)|0)+((8191&(i=(i=i+Math.imul(N,ft)|0)+Math.imul(F,dt)|0))<<13)|0;l=((o=o+Math.imul(F,ft)|0)+(i>>>13)|0)+(Lt>>>26)|0,Lt&=67108863;var Rt=(l+(r=Math.imul(U,dt))|0)+((8191&(i=(i=Math.imul(U,ft))+Math.imul(O,dt)|0))<<13)|0;return l=((o=Math.imul(O,ft))+(i>>>13)|0)+(Rt>>>26)|0,Rt&=67108863,A[0]=pt,A[1]=It,A[2]=Ct,A[3]=mt,A[4]=yt,A[5]=Et,A[6]=vt,A[7]=Bt,A[8]=bt,A[9]=Qt,A[10]=wt,A[11]=xt,A[12]=_t,A[13]=Dt,A[14]=Mt,A[15]=kt,A[16]=St,A[17]=Lt,A[18]=Rt,0!==l&&(A[19]=l,n.length++),n};function d(t,e,n){return(new f).mulp(t,e,n)}function f(t,e){this.x=t,this.y=e}Math.imul||(h=g),o.prototype.mulTo=function(t,e){var n=this.length+t.length;return 10===this.length&&10===t.length?h(this,t,e):n<63?g(this,t,e):n<1024?function(t,e,n){n.negative=e.negative^t.negative,n.length=t.length+e.length;for(var r=0,i=0,o=0;o<n.length-1;o++){var a=i;i=0;for(var s=67108863&r,A=Math.min(o,e.length-1),l=Math.max(0,o-t.length+1);l<=A;l++){var c=o-l,u=(0|t.words[c])*(0|e.words[l]),g=67108863&u;s=67108863&(g=g+s|0),i+=(a=(a=a+(u/67108864|0)|0)+(g>>>26)|0)>>>26,a&=67108863}n.words[o]=s,r=a,a=i}return 0!==r?n.words[o]=r:n.length--,n.strip()}(this,t,e):d(this,t,e)},f.prototype.makeRBT=function(t){for(var e=new Array(t),n=o.prototype._countBits(t)-1,r=0;r<t;r++)e[r]=this.revBin(r,n,t);return e},f.prototype.revBin=function(t,e,n){if(0===t||t===n-1)return t;for(var r=0,i=0;i<e;i++)r|=(1&t)<<e-i-1,t>>=1;return r},f.prototype.permute=function(t,e,n,r,i,o){for(var a=0;a<o;a++)r[a]=e[t[a]],i[a]=n[t[a]]},f.prototype.transform=function(t,e,n,r,i,o){this.permute(o,t,e,n,r,i);for(var a=1;a<i;a<<=1)for(var s=a<<1,A=Math.cos(2*Math.PI/s),l=Math.sin(2*Math.PI/s),c=0;c<i;c+=s)for(var u=A,g=l,h=0;h<a;h++){var d=n[c+h],f=r[c+h],p=n[c+h+a],I=r[c+h+a],C=u*p-g*I;I=u*I+g*p,p=C,n[c+h]=d+p,r[c+h]=f+I,n[c+h+a]=d-p,r[c+h+a]=f-I,h!==s&&(C=A*u-l*g,g=A*g+l*u,u=C)}},f.prototype.guessLen13b=function(t,e){var n=1|Math.max(e,t),r=1&n,i=0;for(n=n/2|0;n;n>>>=1)i++;return 1<<i+1+r},f.prototype.conjugate=function(t,e,n){if(!(n<=1))for(var r=0;r<n/2;r++){var i=t[r];t[r]=t[n-r-1],t[n-r-1]=i,i=e[r],e[r]=-e[n-r-1],e[n-r-1]=-i}},f.prototype.normalize13b=function(t,e){for(var n=0,r=0;r<e/2;r++){var i=8192*Math.round(t[2*r+1]/e)+Math.round(t[2*r]/e)+n;t[r]=67108863&i,n=i<67108864?0:i/67108864|0}return t},f.prototype.convert13b=function(t,e,n,i){for(var o=0,a=0;a<e;a++)o+=0|t[a],n[2*a]=8191&o,o>>>=13,n[2*a+1]=8191&o,o>>>=13;for(a=2*e;a<i;++a)n[a]=0;r(0===o),r(0==(-8192&o))},f.prototype.stub=function(t){for(var e=new Array(t),n=0;n<t;n++)e[n]=0;return e},f.prototype.mulp=function(t,e,n){var r=2*this.guessLen13b(t.length,e.length),i=this.makeRBT(r),o=this.stub(r),a=new Array(r),s=new Array(r),A=new Array(r),l=new Array(r),c=new Array(r),u=new Array(r),g=n.words;g.length=r,this.convert13b(t.words,t.length,a,r),this.convert13b(e.words,e.length,l,r),this.transform(a,o,s,A,r,i),this.transform(l,o,c,u,r,i);for(var h=0;h<r;h++){var d=s[h]*c[h]-A[h]*u[h];A[h]=s[h]*u[h]+A[h]*c[h],s[h]=d}return this.conjugate(s,A,r),this.transform(s,A,g,o,r,i),this.conjugate(g,o,r),this.normalize13b(g,r),n.negative=t.negative^e.negative,n.length=t.length+e.length,n.strip()},o.prototype.mul=function(t){var e=new o(null);return e.words=new Array(this.length+t.length),this.mulTo(t,e)},o.prototype.mulf=function(t){var e=new o(null);return e.words=new Array(this.length+t.length),d(this,t,e)},o.prototype.imul=function(t){return this.clone().mulTo(t,this)},o.prototype.imuln=function(t){r("number"==typeof t),r(t<67108864);for(var e=0,n=0;n<this.length;n++){var i=(0|this.words[n])*t,o=(67108863&i)+(67108863&e);e>>=26,e+=i/67108864|0,e+=o>>>26,this.words[n]=67108863&o}return 0!==e&&(this.words[n]=e,this.length++),this},o.prototype.muln=function(t){return this.clone().imuln(t)},o.prototype.sqr=function(){return this.mul(this)},o.prototype.isqr=function(){return this.imul(this.clone())},o.prototype.pow=function(t){var e=function(t){for(var e=new Array(t.bitLength()),n=0;n<e.length;n++){var r=n/26|0,i=n%26;e[n]=(t.words[r]&1<<i)>>>i}return e}(t);if(0===e.length)return new o(1);for(var n=this,r=0;r<e.length&&0===e[r];r++,n=n.sqr());if(++r<e.length)for(var i=n.sqr();r<e.length;r++,i=i.sqr())0!==e[r]&&(n=n.mul(i));return n},o.prototype.iushln=function(t){r("number"==typeof t&&t>=0);var e,n=t%26,i=(t-n)/26,o=67108863>>>26-n<<26-n;if(0!==n){var a=0;for(e=0;e<this.length;e++){var s=this.words[e]&o,A=(0|this.words[e])-s<<n;this.words[e]=A|a,a=s>>>26-n}a&&(this.words[e]=a,this.length++)}if(0!==i){for(e=this.length-1;e>=0;e--)this.words[e+i]=this.words[e];for(e=0;e<i;e++)this.words[e]=0;this.length+=i}return this.strip()},o.prototype.ishln=function(t){return r(0===this.negative),this.iushln(t)},o.prototype.iushrn=function(t,e,n){var i;r("number"==typeof t&&t>=0),i=e?(e-e%26)/26:0;var o=t%26,a=Math.min((t-o)/26,this.length),s=67108863^67108863>>>o<<o,A=n;if(i-=a,i=Math.max(0,i),A){for(var l=0;l<a;l++)A.words[l]=this.words[l];A.length=a}if(0===a);else if(this.length>a)for(this.length-=a,l=0;l<this.length;l++)this.words[l]=this.words[l+a];else this.words[0]=0,this.length=1;var c=0;for(l=this.length-1;l>=0&&(0!==c||l>=i);l--){var u=0|this.words[l];this.words[l]=c<<26-o|u>>>o,c=u&s}return A&&0!==c&&(A.words[A.length++]=c),0===this.length&&(this.words[0]=0,this.length=1),this.strip()},o.prototype.ishrn=function(t,e,n){return r(0===this.negative),this.iushrn(t,e,n)},o.prototype.shln=function(t){return this.clone().ishln(t)},o.prototype.ushln=function(t){return this.clone().iushln(t)},o.prototype.shrn=function(t){return this.clone().ishrn(t)},o.prototype.ushrn=function(t){return this.clone().iushrn(t)},o.prototype.testn=function(t){r("number"==typeof t&&t>=0);var e=t%26,n=(t-e)/26,i=1<<e;return!(this.length<=n)&&!!(this.words[n]&i)},o.prototype.imaskn=function(t){r("number"==typeof t&&t>=0);var e=t%26,n=(t-e)/26;if(r(0===this.negative,"imaskn works only with positive numbers"),this.length<=n)return this;if(0!==e&&n++,this.length=Math.min(n,this.length),0!==e){var i=67108863^67108863>>>e<<e;this.words[this.length-1]&=i}return this.strip()},o.prototype.maskn=function(t){return this.clone().imaskn(t)},o.prototype.iaddn=function(t){return r("number"==typeof t),r(t<67108864),t<0?this.isubn(-t):0!==this.negative?1===this.length&&(0|this.words[0])<t?(this.words[0]=t-(0|this.words[0]),this.negative=0,this):(this.negative=0,this.isubn(t),this.negative=1,this):this._iaddn(t)},o.prototype._iaddn=function(t){this.words[0]+=t;for(var e=0;e<this.length&&this.words[e]>=67108864;e++)this.words[e]-=67108864,e===this.length-1?this.words[e+1]=1:this.words[e+1]++;return this.length=Math.max(this.length,e+1),this},o.prototype.isubn=function(t){if(r("number"==typeof t),r(t<67108864),t<0)return this.iaddn(-t);if(0!==this.negative)return this.negative=0,this.iaddn(t),this.negative=1,this;if(this.words[0]-=t,1===this.length&&this.words[0]<0)this.words[0]=-this.words[0],this.negative=1;else for(var e=0;e<this.length&&this.words[e]<0;e++)this.words[e]+=67108864,this.words[e+1]-=1;return this.strip()},o.prototype.addn=function(t){return this.clone().iaddn(t)},o.prototype.subn=function(t){return this.clone().isubn(t)},o.prototype.iabs=function(){return this.negative=0,this},o.prototype.abs=function(){return this.clone().iabs()},o.prototype._ishlnsubmul=function(t,e,n){var i,o,a=t.length+n;this._expand(a);var s=0;for(i=0;i<t.length;i++){o=(0|this.words[i+n])+s;var A=(0|t.words[i])*e;s=((o-=67108863&A)>>26)-(A/67108864|0),this.words[i+n]=67108863&o}for(;i<this.length-n;i++)s=(o=(0|this.words[i+n])+s)>>26,this.words[i+n]=67108863&o;if(0===s)return this.strip();for(r(-1===s),s=0,i=0;i<this.length;i++)s=(o=-(0|this.words[i])+s)>>26,this.words[i]=67108863&o;return this.negative=1,this.strip()},o.prototype._wordDiv=function(t,e){var n=(this.length,t.length),r=this.clone(),i=t,a=0|i.words[i.length-1];0!==(n=26-this._countBits(a))&&(i=i.ushln(n),r.iushln(n),a=0|i.words[i.length-1]);var s,A=r.length-i.length;if("mod"!==e){(s=new o(null)).length=A+1,s.words=new Array(s.length);for(var l=0;l<s.length;l++)s.words[l]=0}var c=r.clone()._ishlnsubmul(i,1,A);0===c.negative&&(r=c,s&&(s.words[A]=1));for(var u=A-1;u>=0;u--){var g=67108864*(0|r.words[i.length+u])+(0|r.words[i.length+u-1]);for(g=Math.min(g/a|0,67108863),r._ishlnsubmul(i,g,u);0!==r.negative;)g--,r.negative=0,r._ishlnsubmul(i,1,u),r.isZero()||(r.negative^=1);s&&(s.words[u]=g)}return s&&s.strip(),r.strip(),"div"!==e&&0!==n&&r.iushrn(n),{div:s||null,mod:r}},o.prototype.divmod=function(t,e,n){return r(!t.isZero()),this.isZero()?{div:new o(0),mod:new o(0)}:0!==this.negative&&0===t.negative?(s=this.neg().divmod(t,e),"mod"!==e&&(i=s.div.neg()),"div"!==e&&(a=s.mod.neg(),n&&0!==a.negative&&a.iadd(t)),{div:i,mod:a}):0===this.negative&&0!==t.negative?(s=this.divmod(t.neg(),e),"mod"!==e&&(i=s.div.neg()),{div:i,mod:s.mod}):0!=(this.negative&t.negative)?(s=this.neg().divmod(t.neg(),e),"div"!==e&&(a=s.mod.neg(),n&&0!==a.negative&&a.isub(t)),{div:s.div,mod:a}):t.length>this.length||this.cmp(t)<0?{div:new o(0),mod:this}:1===t.length?"div"===e?{div:this.divn(t.words[0]),mod:null}:"mod"===e?{div:null,mod:new o(this.modn(t.words[0]))}:{div:this.divn(t.words[0]),mod:new o(this.modn(t.words[0]))}:this._wordDiv(t,e);var i,a,s},o.prototype.div=function(t){return this.divmod(t,"div",!1).div},o.prototype.mod=function(t){return this.divmod(t,"mod",!1).mod},o.prototype.umod=function(t){return this.divmod(t,"mod",!0).mod},o.prototype.divRound=function(t){var e=this.divmod(t);if(e.mod.isZero())return e.div;var n=0!==e.div.negative?e.mod.isub(t):e.mod,r=t.ushrn(1),i=t.andln(1),o=n.cmp(r);return o<0||1===i&&0===o?e.div:0!==e.div.negative?e.div.isubn(1):e.div.iaddn(1)},o.prototype.modn=function(t){r(t<=67108863);for(var e=(1<<26)%t,n=0,i=this.length-1;i>=0;i--)n=(e*n+(0|this.words[i]))%t;return n},o.prototype.idivn=function(t){r(t<=67108863);for(var e=0,n=this.length-1;n>=0;n--){var i=(0|this.words[n])+67108864*e;this.words[n]=i/t|0,e=i%t}return this.strip()},o.prototype.divn=function(t){return this.clone().idivn(t)},o.prototype.egcd=function(t){r(0===t.negative),r(!t.isZero());var e=this,n=t.clone();e=0!==e.negative?e.umod(t):e.clone();for(var i=new o(1),a=new o(0),s=new o(0),A=new o(1),l=0;e.isEven()&&n.isEven();)e.iushrn(1),n.iushrn(1),++l;for(var c=n.clone(),u=e.clone();!e.isZero();){for(var g=0,h=1;0==(e.words[0]&h)&&g<26;++g,h<<=1);if(g>0)for(e.iushrn(g);g-- >0;)(i.isOdd()||a.isOdd())&&(i.iadd(c),a.isub(u)),i.iushrn(1),a.iushrn(1);for(var d=0,f=1;0==(n.words[0]&f)&&d<26;++d,f<<=1);if(d>0)for(n.iushrn(d);d-- >0;)(s.isOdd()||A.isOdd())&&(s.iadd(c),A.isub(u)),s.iushrn(1),A.iushrn(1);e.cmp(n)>=0?(e.isub(n),i.isub(s),a.isub(A)):(n.isub(e),s.isub(i),A.isub(a))}return{a:s,b:A,gcd:n.iushln(l)}},o.prototype._invmp=function(t){r(0===t.negative),r(!t.isZero());var e=this,n=t.clone();e=0!==e.negative?e.umod(t):e.clone();for(var i,a=new o(1),s=new o(0),A=n.clone();e.cmpn(1)>0&&n.cmpn(1)>0;){for(var l=0,c=1;0==(e.words[0]&c)&&l<26;++l,c<<=1);if(l>0)for(e.iushrn(l);l-- >0;)a.isOdd()&&a.iadd(A),a.iushrn(1);for(var u=0,g=1;0==(n.words[0]&g)&&u<26;++u,g<<=1);if(u>0)for(n.iushrn(u);u-- >0;)s.isOdd()&&s.iadd(A),s.iushrn(1);e.cmp(n)>=0?(e.isub(n),a.isub(s)):(n.isub(e),s.isub(a))}return(i=0===e.cmpn(1)?a:s).cmpn(0)<0&&i.iadd(t),i},o.prototype.gcd=function(t){if(this.isZero())return t.abs();if(t.isZero())return this.abs();var e=this.clone(),n=t.clone();e.negative=0,n.negative=0;for(var r=0;e.isEven()&&n.isEven();r++)e.iushrn(1),n.iushrn(1);for(;;){for(;e.isEven();)e.iushrn(1);for(;n.isEven();)n.iushrn(1);var i=e.cmp(n);if(i<0){var o=e;e=n,n=o}else if(0===i||0===n.cmpn(1))break;e.isub(n)}return n.iushln(r)},o.prototype.invm=function(t){return this.egcd(t).a.umod(t)},o.prototype.isEven=function(){return 0==(1&this.words[0])},o.prototype.isOdd=function(){return 1==(1&this.words[0])},o.prototype.andln=function(t){return this.words[0]&t},o.prototype.bincn=function(t){r("number"==typeof t);var e=t%26,n=(t-e)/26,i=1<<e;if(this.length<=n)return this._expand(n+1),this.words[n]|=i,this;for(var o=i,a=n;0!==o&&a<this.length;a++){var s=0|this.words[a];o=(s+=o)>>>26,s&=67108863,this.words[a]=s}return 0!==o&&(this.words[a]=o,this.length++),this},o.prototype.isZero=function(){return 1===this.length&&0===this.words[0]},o.prototype.cmpn=function(t){var e,n=t<0;if(0!==this.negative&&!n)return-1;if(0===this.negative&&n)return 1;if(this.strip(),this.length>1)e=1;else{n&&(t=-t),r(t<=67108863,"Number is too big");var i=0|this.words[0];e=i===t?0:i<t?-1:1}return 0!==this.negative?0|-e:e},o.prototype.cmp=function(t){if(0!==this.negative&&0===t.negative)return-1;if(0===this.negative&&0!==t.negative)return 1;var e=this.ucmp(t);return 0!==this.negative?0|-e:e},o.prototype.ucmp=function(t){if(this.length>t.length)return 1;if(this.length<t.length)return-1;for(var e=0,n=this.length-1;n>=0;n--){var r=0|this.words[n],i=0|t.words[n];if(r!==i){r<i?e=-1:r>i&&(e=1);break}}return e},o.prototype.gtn=function(t){return 1===this.cmpn(t)},o.prototype.gt=function(t){return 1===this.cmp(t)},o.prototype.gten=function(t){return this.cmpn(t)>=0},o.prototype.gte=function(t){return this.cmp(t)>=0},o.prototype.ltn=function(t){return-1===this.cmpn(t)},o.prototype.lt=function(t){return-1===this.cmp(t)},o.prototype.lten=function(t){return this.cmpn(t)<=0},o.prototype.lte=function(t){return this.cmp(t)<=0},o.prototype.eqn=function(t){return 0===this.cmpn(t)},o.prototype.eq=function(t){return 0===this.cmp(t)},o.red=function(t){return new v(t)},o.prototype.toRed=function(t){return r(!this.red,"Already a number in reduction context"),r(0===this.negative,"red works only with positives"),t.convertTo(this)._forceRed(t)},o.prototype.fromRed=function(){return r(this.red,"fromRed works only with numbers in reduction context"),this.red.convertFrom(this)},o.prototype._forceRed=function(t){return this.red=t,this},o.prototype.forceRed=function(t){return r(!this.red,"Already a number in reduction context"),this._forceRed(t)},o.prototype.redAdd=function(t){return r(this.red,"redAdd works only with red numbers"),this.red.add(this,t)},o.prototype.redIAdd=function(t){return r(this.red,"redIAdd works only with red numbers"),this.red.iadd(this,t)},o.prototype.redSub=function(t){return r(this.red,"redSub works only with red numbers"),this.red.sub(this,t)},o.prototype.redISub=function(t){return r(this.red,"redISub works only with red numbers"),this.red.isub(this,t)},o.prototype.redShl=function(t){return r(this.red,"redShl works only with red numbers"),this.red.shl(this,t)},o.prototype.redMul=function(t){return r(this.red,"redMul works only with red numbers"),this.red._verify2(this,t),this.red.mul(this,t)},o.prototype.redIMul=function(t){return r(this.red,"redMul works only with red numbers"),this.red._verify2(this,t),this.red.imul(this,t)},o.prototype.redSqr=function(){return r(this.red,"redSqr works only with red numbers"),this.red._verify1(this),this.red.sqr(this)},o.prototype.redISqr=function(){return r(this.red,"redISqr works only with red numbers"),this.red._verify1(this),this.red.isqr(this)},o.prototype.redSqrt=function(){return r(this.red,"redSqrt works only with red numbers"),this.red._verify1(this),this.red.sqrt(this)},o.prototype.redInvm=function(){return r(this.red,"redInvm works only with red numbers"),this.red._verify1(this),this.red.invm(this)},o.prototype.redNeg=function(){return r(this.red,"redNeg works only with red numbers"),this.red._verify1(this),this.red.neg(this)},o.prototype.redPow=function(t){return r(this.red&&!t.red,"redPow(normalNum)"),this.red._verify1(this),this.red.pow(this,t)};var p={k256:null,p224:null,p192:null,p25519:null};function I(t,e){this.name=t,this.p=new o(e,16),this.n=this.p.bitLength(),this.k=new o(1).iushln(this.n).isub(this.p),this.tmp=this._tmp()}function C(){I.call(this,"k256","ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff fffffffe fffffc2f")}function m(){I.call(this,"p224","ffffffff ffffffff ffffffff ffffffff 00000000 00000000 00000001")}function y(){I.call(this,"p192","ffffffff ffffffff ffffffff fffffffe ffffffff ffffffff")}function E(){I.call(this,"25519","7fffffffffffffff ffffffffffffffff ffffffffffffffff ffffffffffffffed")}function v(t){if("string"==typeof t){var e=o._prime(t);this.m=e.p,this.prime=e}else r(t.gtn(1),"modulus must be greater than 1"),this.m=t,this.prime=null}function B(t){v.call(this,t),this.shift=this.m.bitLength(),this.shift%26!=0&&(this.shift+=26-this.shift%26),this.r=new o(1).iushln(this.shift),this.r2=this.imod(this.r.sqr()),this.rinv=this.r._invmp(this.m),this.minv=this.rinv.mul(this.r).isubn(1).div(this.m),this.minv=this.minv.umod(this.r),this.minv=this.r.sub(this.minv)}I.prototype._tmp=function(){var t=new o(null);return t.words=new Array(Math.ceil(this.n/13)),t},I.prototype.ireduce=function(t){var e,n=t;do{this.split(n,this.tmp),e=(n=(n=this.imulK(n)).iadd(this.tmp)).bitLength()}while(e>this.n);var r=e<this.n?-1:n.ucmp(this.p);return 0===r?(n.words[0]=0,n.length=1):r>0?n.isub(this.p):n.strip(),n},I.prototype.split=function(t,e){t.iushrn(this.n,0,e)},I.prototype.imulK=function(t){return t.imul(this.k)},i(C,I),C.prototype.split=function(t,e){for(var n=Math.min(t.length,9),r=0;r<n;r++)e.words[r]=t.words[r];if(e.length=n,t.length<=9)return t.words[0]=0,void(t.length=1);var i=t.words[9];for(e.words[e.length++]=4194303&i,r=10;r<t.length;r++){var o=0|t.words[r];t.words[r-10]=(4194303&o)<<4|i>>>22,i=o}i>>>=22,t.words[r-10]=i,0===i&&t.length>10?t.length-=10:t.length-=9},C.prototype.imulK=function(t){t.words[t.length]=0,t.words[t.length+1]=0,t.length+=2;for(var e=0,n=0;n<t.length;n++){var r=0|t.words[n];e+=977*r,t.words[n]=67108863&e,e=64*r+(e/67108864|0)}return 0===t.words[t.length-1]&&(t.length--,0===t.words[t.length-1]&&t.length--),t},i(m,I),i(y,I),i(E,I),E.prototype.imulK=function(t){for(var e=0,n=0;n<t.length;n++){var r=19*(0|t.words[n])+e,i=67108863&r;r>>>=26,t.words[n]=i,e=r}return 0!==e&&(t.words[t.length++]=e),t},o._prime=function(t){if(p[t])return p[t];var e;if("k256"===t)e=new C;else if("p224"===t)e=new m;else if("p192"===t)e=new y;else{if("p25519"!==t)throw new Error("Unknown prime "+t);e=new E}return p[t]=e,e},v.prototype._verify1=function(t){r(0===t.negative,"red works only with positives"),r(t.red,"red works only with red numbers")},v.prototype._verify2=function(t,e){r(0==(t.negative|e.negative),"red works only with positives"),r(t.red&&t.red===e.red,"red works only with red numbers")},v.prototype.imod=function(t){return this.prime?this.prime.ireduce(t)._forceRed(this):t.umod(this.m)._forceRed(this)},v.prototype.neg=function(t){return t.isZero()?t.clone():this.m.sub(t)._forceRed(this)},v.prototype.add=function(t,e){this._verify2(t,e);var n=t.add(e);return n.cmp(this.m)>=0&&n.isub(this.m),n._forceRed(this)},v.prototype.iadd=function(t,e){this._verify2(t,e);var n=t.iadd(e);return n.cmp(this.m)>=0&&n.isub(this.m),n},v.prototype.sub=function(t,e){this._verify2(t,e);var n=t.sub(e);return n.cmpn(0)<0&&n.iadd(this.m),n._forceRed(this)},v.prototype.isub=function(t,e){this._verify2(t,e);var n=t.isub(e);return n.cmpn(0)<0&&n.iadd(this.m),n},v.prototype.shl=function(t,e){return this._verify1(t),this.imod(t.ushln(e))},v.prototype.imul=function(t,e){return this._verify2(t,e),this.imod(t.imul(e))},v.prototype.mul=function(t,e){return this._verify2(t,e),this.imod(t.mul(e))},v.prototype.isqr=function(t){return this.imul(t,t.clone())},v.prototype.sqr=function(t){return this.mul(t,t)},v.prototype.sqrt=function(t){if(t.isZero())return t.clone();var e=this.m.andln(3);if(r(e%2==1),3===e){var n=this.m.add(new o(1)).iushrn(2);return this.pow(t,n)}for(var i=this.m.subn(1),a=0;!i.isZero()&&0===i.andln(1);)a++,i.iushrn(1);r(!i.isZero());var s=new o(1).toRed(this),A=s.redNeg(),l=this.m.subn(1).iushrn(1),c=this.m.bitLength();for(c=new o(2*c*c).toRed(this);0!==this.pow(c,l).cmp(A);)c.redIAdd(A);for(var u=this.pow(c,i),g=this.pow(t,i.addn(1).iushrn(1)),h=this.pow(t,i),d=a;0!==h.cmp(s);){for(var f=h,p=0;0!==f.cmp(s);p++)f=f.redSqr();r(p<d);var I=this.pow(u,new o(1).iushln(d-p-1));g=g.redMul(I),u=I.redSqr(),h=h.redMul(u),d=p}return g},v.prototype.invm=function(t){var e=t._invmp(this.m);return 0!==e.negative?(e.negative=0,this.imod(e).redNeg()):this.imod(e)},v.prototype.pow=function(t,e){if(e.isZero())return new o(1).toRed(this);if(0===e.cmpn(1))return t.clone();var n=new Array(16);n[0]=new o(1).toRed(this),n[1]=t;for(var r=2;r<n.length;r++)n[r]=this.mul(n[r-1],t);var i=n[0],a=0,s=0,A=e.bitLength()%26;for(0===A&&(A=26),r=e.length-1;r>=0;r--){for(var l=e.words[r],c=A-1;c>=0;c--){var u=l>>c&1;i!==n[0]&&(i=this.sqr(i)),0!==u||0!==a?(a<<=1,a|=u,(4===++s||0===r&&0===c)&&(i=this.mul(i,n[a]),s=0,a=0)):s=0}A=26}return i},v.prototype.convertTo=function(t){var e=t.umod(this.m);return e===t?e.clone():e},v.prototype.convertFrom=function(t){var e=t.clone();return e.red=null,e},o.mont=function(t){return new B(t)},i(B,v),B.prototype.convertTo=function(t){return this.imod(t.ushln(this.shift))},B.prototype.convertFrom=function(t){var e=this.imod(t.mul(this.rinv));return e.red=null,e},B.prototype.imul=function(t,e){if(t.isZero()||e.isZero())return t.words[0]=0,t.length=1,t;var n=t.imul(e),r=n.maskn(this.shift).mul(this.minv).imaskn(this.shift).mul(this.m),i=n.isub(r).iushrn(this.shift),o=i;return i.cmp(this.m)>=0?o=i.isub(this.m):i.cmpn(0)<0&&(o=i.iadd(this.m)),o._forceRed(this)},B.prototype.mul=function(t,e){if(t.isZero()||e.isZero())return new o(0)._forceRed(this);var n=t.mul(e),r=n.maskn(this.shift).mul(this.minv).imaskn(this.shift).mul(this.m),i=n.isub(r).iushrn(this.shift),a=i;return i.cmp(this.m)>=0?a=i.isub(this.m):i.cmpn(0)<0&&(a=i.iadd(this.m)),a._forceRed(this)},B.prototype.invm=function(t){return this.imod(t._invmp(this.m).mul(this.r2))._forceRed(this)}}(t,this)}).call(this,n(98)(t))},function(t,e){t.exports=function(t){return t&&t.__esModule?t:{default:t}}},function(t,e,n){"use strict";n.d(e,"a",function(){return a});var r=n(21),i=n(132),o=n(255);function a(t,e){return e?Object(o.a)(t,e):t instanceof r.a?t:new r.a(Object(i.a)(t))}},function(t,e,n){"use strict";e.__esModule=!0;var r,i=n(649),o=(r=i)&&r.__esModule?r:{default:r};e.default=function(){function t(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),(0,o.default)(t,r.key,r)}}return function(e,n,r){return n&&t(e.prototype,n),r&&t(e,r),e}}()},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t,e){var n={};for(var r in t)e.indexOf(r)>=0||Object.prototype.hasOwnProperty.call(t,r)&&(n[r]=t[r]);return n}},function(t,e){var n,r,i=t.exports={};function o(){throw new Error("setTimeout has not been defined")}function a(){throw new Error("clearTimeout has not been defined")}function s(t){if(n===setTimeout)return setTimeout(t,0);if((n===o||!n)&&setTimeout)return n=setTimeout,setTimeout(t,0);try{return n(t,0)}catch(e){try{return n.call(null,t,0)}catch(e){return n.call(this,t,0)}}}!function(){try{n="function"==typeof setTimeout?setTimeout:o}catch(t){n=o}try{r="function"==typeof clearTimeout?clearTimeout:a}catch(t){r=a}}();var A,l=[],c=!1,u=-1;function g(){c&&A&&(c=!1,A.length?l=A.concat(l):u=-1,l.length&&h())}function h(){if(!c){var t=s(g);c=!0;for(var e=l.length;e;){for(A=l,l=[];++u<e;)A&&A[u].run();u=-1,e=l.length}A=null,c=!1,function(t){if(r===clearTimeout)return clearTimeout(t);if((r===a||!r)&&clearTimeout)return r=clearTimeout,clearTimeout(t);try{r(t)}catch(e){try{return r.call(null,t)}catch(e){return r.call(this,t)}}}(t)}}function d(t,e){this.fun=t,this.array=e}function f(){}i.nextTick=function(t){var e=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)e[n-1]=arguments[n];l.push(new d(t,e)),1!==l.length||c||s(h)},d.prototype.run=function(){this.fun.apply(null,this.array)},i.title="browser",i.browser=!0,i.env={},i.argv=[],i.version="",i.versions={},i.on=f,i.addListener=f,i.once=f,i.off=f,i.removeListener=f,i.removeAllListeners=f,i.emit=f,i.prependListener=f,i.prependOnceListener=f,i.listeners=function(t){return[]},i.binding=function(t){throw new Error("process.binding is not supported")},i.cwd=function(){return"/"},i.chdir=function(t){throw new Error("process.chdir is not supported")},i.umask=function(){return 0}},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(5),s=n(25),A=n(43),l=n(10),c=n(289),u=function(){function t(t){void 0===t&&(t={}),this._flags=0,this._layout=null,this._parent=null,this._disposed=new l.Signal(this),this.node=o.createNode(t),this.addClass("p-Widget")}return t.prototype.dispose=function(){this.isDisposed||(this.setFlag(t.Flag.IsDisposed),this._disposed.emit(void 0),this.parent?this.parent=null:this.isAttached&&t.detach(this),this._layout&&(this._layout.dispose(),this._layout=null),l.Signal.clearData(this),s.MessageLoop.clearData(this),A.AttachedProperty.clearData(this))},Object.defineProperty(t.prototype,"disposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isDisposed",{get:function(){return this.testFlag(t.Flag.IsDisposed)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isAttached",{get:function(){return this.testFlag(t.Flag.IsAttached)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isHidden",{get:function(){return this.testFlag(t.Flag.IsHidden)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isVisible",{get:function(){return this.testFlag(t.Flag.IsVisible)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"title",{get:function(){return o.titleProperty.get(this)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"id",{get:function(){return this.node.id},set:function(t){this.node.id=t},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dataset",{get:function(){return this.node.dataset},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"parent",{get:function(){return this._parent},set:function(e){if(this._parent!==e){if(e&&this.contains(e))throw new Error("Invalid parent widget.");if(this._parent&&!this._parent.isDisposed){var n=new t.ChildMessage("child-removed",this);s.MessageLoop.sendMessage(this._parent,n)}if(this._parent=e,this._parent&&!this._parent.isDisposed){n=new t.ChildMessage("child-added",this);s.MessageLoop.sendMessage(this._parent,n)}this.isDisposed||s.MessageLoop.sendMessage(this,t.Msg.ParentChanged)}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"layout",{get:function(){return this._layout},set:function(e){if(this._layout!==e){if(this.testFlag(t.Flag.DisallowLayout))throw new Error("Cannot set widget layout.");if(this._layout)throw new Error("Cannot change widget layout.");if(e.parent)throw new Error("Cannot change layout parent.");this._layout=e,e.parent=this}},enumerable:!0,configurable:!0}),t.prototype.children=function(){return this._layout?this._layout.iter():a.empty()},t.prototype.contains=function(t){for(var e=t;e;e=e._parent)if(e===this)return!0;return!1},t.prototype.hasClass=function(t){return this.node.classList.contains(t)},t.prototype.addClass=function(t){this.node.classList.add(t)},t.prototype.removeClass=function(t){this.node.classList.remove(t)},t.prototype.toggleClass=function(t,e){return!0===e?(this.node.classList.add(t),!0):!1===e?(this.node.classList.remove(t),!1):this.node.classList.toggle(t)},t.prototype.update=function(){s.MessageLoop.postMessage(this,t.Msg.UpdateRequest)},t.prototype.fit=function(){s.MessageLoop.postMessage(this,t.Msg.FitRequest)},t.prototype.activate=function(){s.MessageLoop.postMessage(this,t.Msg.ActivateRequest)},t.prototype.close=function(){s.MessageLoop.sendMessage(this,t.Msg.CloseRequest)},t.prototype.show=function(){if(this.testFlag(t.Flag.IsHidden)&&(!this.isAttached||this.parent&&!this.parent.isVisible||s.MessageLoop.sendMessage(this,t.Msg.BeforeShow),this.clearFlag(t.Flag.IsHidden),this.removeClass("p-mod-hidden"),!this.isAttached||this.parent&&!this.parent.isVisible||s.MessageLoop.sendMessage(this,t.Msg.AfterShow),this.parent)){var e=new t.ChildMessage("child-shown",this);s.MessageLoop.sendMessage(this.parent,e)}},t.prototype.hide=function(){if(!this.testFlag(t.Flag.IsHidden)&&(!this.isAttached||this.parent&&!this.parent.isVisible||s.MessageLoop.sendMessage(this,t.Msg.BeforeHide),this.setFlag(t.Flag.IsHidden),this.addClass("p-mod-hidden"),!this.isAttached||this.parent&&!this.parent.isVisible||s.MessageLoop.sendMessage(this,t.Msg.AfterHide),this.parent)){var e=new t.ChildMessage("child-hidden",this);s.MessageLoop.sendMessage(this.parent,e)}},t.prototype.setHidden=function(t){t?this.hide():this.show()},t.prototype.testFlag=function(t){return 0!=(this._flags&t)},t.prototype.setFlag=function(t){this._flags|=t},t.prototype.clearFlag=function(t){this._flags&=~t},t.prototype.processMessage=function(e){switch(e.type){case"resize":this.notifyLayout(e),this.onResize(e);break;case"update-request":this.notifyLayout(e),this.onUpdateRequest(e);break;case"fit-request":this.notifyLayout(e),this.onFitRequest(e);break;case"before-show":this.notifyLayout(e),this.onBeforeShow(e);break;case"after-show":this.setFlag(t.Flag.IsVisible),this.notifyLayout(e),this.onAfterShow(e);break;case"before-hide":this.notifyLayout(e),this.onBeforeHide(e);break;case"after-hide":this.clearFlag(t.Flag.IsVisible),this.notifyLayout(e),this.onAfterHide(e);break;case"before-attach":this.notifyLayout(e),this.onBeforeAttach(e);break;case"after-attach":this.isHidden||this.parent&&!this.parent.isVisible||this.setFlag(t.Flag.IsVisible),this.setFlag(t.Flag.IsAttached),this.notifyLayout(e),this.onAfterAttach(e);break;case"before-detach":this.notifyLayout(e),this.onBeforeDetach(e);break;case"after-detach":this.clearFlag(t.Flag.IsVisible),this.clearFlag(t.Flag.IsAttached),this.notifyLayout(e),this.onAfterDetach(e);break;case"activate-request":this.notifyLayout(e),this.onActivateRequest(e);break;case"close-request":this.notifyLayout(e),this.onCloseRequest(e);break;case"child-added":this.notifyLayout(e),this.onChildAdded(e);break;case"child-removed":this.notifyLayout(e),this.onChildRemoved(e);break;default:this.notifyLayout(e)}},t.prototype.notifyLayout=function(t){this._layout&&this._layout.processParentMessage(t)},t.prototype.onCloseRequest=function(e){this.parent?this.parent=null:this.isAttached&&t.detach(this)},t.prototype.onResize=function(t){},t.prototype.onUpdateRequest=function(t){},t.prototype.onFitRequest=function(t){},t.prototype.onActivateRequest=function(t){},t.prototype.onBeforeShow=function(t){},t.prototype.onAfterShow=function(t){},t.prototype.onBeforeHide=function(t){},t.prototype.onAfterHide=function(t){},t.prototype.onBeforeAttach=function(t){},t.prototype.onAfterAttach=function(t){},t.prototype.onBeforeDetach=function(t){},t.prototype.onAfterDetach=function(t){},t.prototype.onChildAdded=function(t){},t.prototype.onChildRemoved=function(t){},t}();e.Widget=u,function(t){!function(t){t[t.IsDisposed=1]="IsDisposed",t[t.IsAttached=2]="IsAttached",t[t.IsHidden=4]="IsHidden",t[t.IsVisible=8]="IsVisible",t[t.DisallowLayout=16]="DisallowLayout"}(t.Flag||(t.Flag={})),function(t){t.BeforeShow=new s.Message("before-show"),t.AfterShow=new s.Message("after-show"),t.BeforeHide=new s.Message("before-hide"),t.AfterHide=new s.Message("after-hide"),t.BeforeAttach=new s.Message("before-attach"),t.AfterAttach=new s.Message("after-attach"),t.BeforeDetach=new s.Message("before-detach"),t.AfterDetach=new s.Message("after-detach"),t.ParentChanged=new s.Message("parent-changed"),t.UpdateRequest=new s.ConflatableMessage("update-request"),t.FitRequest=new s.ConflatableMessage("fit-request"),t.ActivateRequest=new s.ConflatableMessage("activate-request"),t.CloseRequest=new s.ConflatableMessage("close-request")}(t.Msg||(t.Msg={}));var e=function(t){function e(e,n){var r=t.call(this,e)||this;return r.child=n,r}return i(e,t),e}(s.Message);t.ChildMessage=e;var n=function(t){function e(e,n){var r=t.call(this,"resize")||this;return r.width=e,r.height=n,r}return i(e,t),e}(s.Message);t.ResizeMessage=n,function(t){t.UnknownSize=new t(-1,-1)}(n=t.ResizeMessage||(t.ResizeMessage={})),t.attach=function(e,n,r){if(void 0===r&&(r=null),e.parent)throw new Error("Cannot attach a child widget.");if(e.isAttached||document.body.contains(e.node))throw new Error("Widget is already attached.");if(!document.body.contains(n))throw new Error("Host is not attached.");s.MessageLoop.sendMessage(e,t.Msg.BeforeAttach),n.insertBefore(e.node,r),s.MessageLoop.sendMessage(e,t.Msg.AfterAttach)},t.detach=function(e){if(e.parent)throw new Error("Cannot detach a child widget.");if(!e.isAttached||!document.body.contains(e.node))throw new Error("Widget is not attached.");s.MessageLoop.sendMessage(e,t.Msg.BeforeDetach),e.node.parentNode.removeChild(e.node),s.MessageLoop.sendMessage(e,t.Msg.AfterDetach)}}(u=e.Widget||(e.Widget={})),e.Widget=u,function(t){t.titleProperty=new A.AttachedProperty({name:"title",create:function(t){return new c.Title({owner:t})}}),t.createNode=function(t){return t.node||document.createElement("div")}}(o||(o={}))},function(t,e,n){"use strict";function r(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}Object.defineProperty(e,"__esModule",{value:!0}),r(n(1069)),r(n(583)),r(n(295)),r(n(1076)),r(n(120)),r(n(587)),r(n(589)),r(n(590)),r(n(591)),r(n(586))},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r="1.5.0"},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=n(2),i=function(t){function e(e,n,r){var i=t.call(this)||this;return i.parent=e,i.outerValue=n,i.outerIndex=r,i.index=0,i}return r.__extends(e,t),e.prototype._next=function(t){this.parent.notifyNext(this.outerValue,t,this.outerIndex,this.index++,this)},e.prototype._error=function(t){this.parent.notifyError(t,this),this.unsubscribe()},e.prototype._complete=function(){this.parent.notifyComplete(this),this.unsubscribe()},e}(n(11).a)},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=!1,i={Promise:void 0,set useDeprecatedSynchronousErrorHandling(t){t&&(new Error).stack;r=t},get useDeprecatedSynchronousErrorHandling(){return r}}},function(t,e,n){"use strict";n.r(e);var r=n(21),i=n(246),o=n(219),a=n(79),s=n(36),A=n(247),l=n(149),c=n(92),u=n(160),g=n(39),h=n(256),d=n(2),f=n(93),p=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.scheduler=e,r.work=n,r}return d.__extends(e,t),e.prototype.requestAsyncId=function(e,n,r){return void 0===r&&(r=0),null!==r&&r>0?t.prototype.requestAsyncId.call(this,e,n,r):(e.actions.push(this),e.scheduled||(e.scheduled=requestAnimationFrame(function(){return e.flush(null)})))},e.prototype.recycleAsyncId=function(e,n,r){if(void 0===r&&(r=0),null!==r&&r>0||null===r&&this.delay>0)return t.prototype.recycleAsyncId.call(this,e,n,r);0===e.actions.length&&(cancelAnimationFrame(n),e.scheduled=void 0)},e}(f.a),I=n(91),C=new(function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return d.__extends(e,t),e.prototype.flush=function(t){this.active=!0,this.scheduled=void 0;var e,n=this.actions,r=-1,i=n.length;t=t||n.shift();do{if(e=t.execute(t.state,t.delay))break}while(++r<i&&(t=n.shift()));if(this.active=!1,e){for(;++r<i&&(t=n.shift());)t.unsubscribe();throw e}},e}(I.a))(p),m=function(t){function e(e,n){void 0===e&&(e=y),void 0===n&&(n=Number.POSITIVE_INFINITY);var r=t.call(this,e,function(){return r.frame})||this;return r.maxFrames=n,r.frame=0,r.index=-1,r}return d.__extends(e,t),e.prototype.flush=function(){for(var t,e,n=this.actions,r=this.maxFrames;(e=n[0])&&e.delay<=r&&(n.shift(),this.frame=e.delay,!(t=e.execute(e.state,e.delay))););if(t){for(;e=n.shift();)e.unsubscribe();throw t}},e.frameTimeFactor=10,e}(I.a),y=function(t){function e(e,n,r){void 0===r&&(r=e.index+=1);var i=t.call(this,e,n)||this;return i.scheduler=e,i.work=n,i.index=r,i.active=!0,i.index=e.index=r,i}return d.__extends(e,t),e.prototype.schedule=function(n,r){if(void 0===r&&(r=0),!this.id)return t.prototype.schedule.call(this,n,r);this.active=!1;var i=new e(this.scheduler,this.work);return this.add(i),i.schedule(n,r)},e.prototype.requestAsyncId=function(t,n,r){void 0===r&&(r=0),this.delay=t.frame+r;var i=t.actions;return i.push(this),i.sort(e.sortActions),!0},e.prototype.recycleAsyncId=function(t,e,n){void 0===n&&(n=0)},e.prototype._execute=function(e,n){if(!0===this.active)return t.prototype._execute.call(this,e,n)},e.sortActions=function(t,e){return t.delay===e.delay?t.index===e.index?0:t.index>e.index?1:-1:t.delay>e.delay?1:-1},e}(f.a),E=n(199),v=n(32),B=n(11),b=n(69),Q=n(129),w=n(65),x=n(73);function _(t){return!!t&&(t instanceof r.a||"function"==typeof t.lift&&"function"==typeof t.subscribe)}var D=n(85),M=n(89),k=n(81),S=n(131),L=n(249),R=n(41),N=n(158),F=n(38),T=n(45);function U(t,e,n){if(e){if(!Object(T.a)(e))return function(){for(var r=[],i=0;i<arguments.length;i++)r[i]=arguments[i];return U(t,n).apply(void 0,r).pipe(Object(R.a)(function(t){return Object(F.a)(t)?e.apply(void 0,t):e(t)}))};n=e}return function(){for(var e=[],i=0;i<arguments.length;i++)e[i]=arguments[i];var o,a=this,s={context:a,subject:o,callbackFunc:t,scheduler:n};return new r.a(function(r){if(n){var i={args:e,subscriber:r,params:s};return n.schedule(O,0,i)}if(!o){o=new c.a;try{t.apply(a,e.concat([function(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];o.next(t.length<=1?t[0]:t),o.complete()}]))}catch(t){Object(N.a)(o)?o.error(t):console.warn(t)}}return o.subscribe(r)})}}function O(t){var e=this,n=t.args,r=t.subscriber,i=t.params,o=i.callbackFunc,a=i.context,s=i.scheduler,A=i.subject;if(!A){A=i.subject=new c.a;try{o.apply(a,n.concat([function(){for(var t=[],n=0;n<arguments.length;n++)t[n]=arguments[n];var r=t.length<=1?t[0]:t;e.add(s.schedule(G,0,{value:r,subject:A}))}]))}catch(t){A.error(t)}}this.add(A.subscribe(r))}function G(t){var e=t.value,n=t.subject;n.next(e),n.complete()}function H(t,e,n){if(e){if(!Object(T.a)(e))return function(){for(var r=[],i=0;i<arguments.length;i++)r[i]=arguments[i];return H(t,n).apply(void 0,r).pipe(Object(R.a)(function(t){return Object(F.a)(t)?e.apply(void 0,t):e(t)}))};n=e}return function(){for(var e=[],i=0;i<arguments.length;i++)e[i]=arguments[i];var o={subject:void 0,args:e,callbackFunc:t,scheduler:n,context:this};return new r.a(function(r){var i=o.context,a=o.subject;if(n)return n.schedule(j,0,{params:o,subscriber:r,context:i});if(!a){a=o.subject=new c.a;try{t.apply(i,e.concat([function(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];var n=t.shift();n?a.error(n):(a.next(t.length<=1?t[0]:t),a.complete())}]))}catch(t){Object(N.a)(a)?a.error(t):console.warn(t)}}return a.subscribe(r)})}}function j(t){var e=this,n=t.params,r=t.subscriber,i=t.context,o=n.callbackFunc,a=n.args,s=n.scheduler,A=n.subject;if(!A){A=n.subject=new c.a;try{o.apply(i,a.concat([function(){for(var t=[],n=0;n<arguments.length;n++)t[n]=arguments[n];var r=t.shift();if(r)e.add(s.schedule(P,0,{err:r,subject:A}));else{var i=t.length<=1?t[0]:t;e.add(s.schedule(Y,0,{value:i,subject:A}))}}]))}catch(t){this.add(s.schedule(P,0,{err:t,subject:A}))}}this.add(A.subscribe(r))}function Y(t){var e=t.value,n=t.subject;n.next(e),n.complete()}function P(t){var e=t.err;t.subject.error(e)}var K=n(151),z=n(101),J=n(153),q=n(51),V=n(157),W=n(55);function X(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];if(1===t.length){var n=t[0];if(Object(F.a)(n))return Z(n,null);if(Object(V.a)(n)&&Object.getPrototypeOf(n)===Object.prototype){var r=Object.keys(n);return Z(r.map(function(t){return n[t]}),r)}}if("function"==typeof t[t.length-1]){var i=t.pop();return Z(t=1===t.length&&Object(F.a)(t[0])?t[0]:t,null).pipe(Object(R.a)(function(t){return i.apply(void 0,t)}))}return Z(t,null)}function Z(t,e){return new r.a(function(n){var r=t.length;if(0!==r)for(var i=new Array(r),o=0,a=0,s=function(s){var A=Object(W.a)(t[s]),l=!1;n.add(A.subscribe({next:function(t){l||(l=!0,a++),i[s]=t},error:function(t){return n.error(t)},complete:function(){++o!==r&&l||(a===r&&n.next(e?e.reduce(function(t,e,n){return t[e]=i[n],t},{}):i),n.complete())}}))},A=0;A<r;A++)s(A);else n.complete()})}var $=n(86);Object.prototype.toString;function tt(t,e,n,i){return Object($.a)(n)&&(i=n,n=void 0),i?tt(t,e,n).pipe(Object(R.a)(function(t){return Object(F.a)(t)?i.apply(void 0,t):i(t)})):new r.a(function(r){!function t(e,n,r,i,o){var a;if(function(t){return t&&"function"==typeof t.addEventListener&&"function"==typeof t.removeEventListener}(e)){var s=e;e.addEventListener(n,r,o),a=function(){return s.removeEventListener(n,r,o)}}else if(function(t){return t&&"function"==typeof t.on&&"function"==typeof t.off}(e)){var A=e;e.on(n,r),a=function(){return A.off(n,r)}}else if(function(t){return t&&"function"==typeof t.addListener&&"function"==typeof t.removeListener}(e)){var l=e;e.addListener(n,r),a=function(){return l.removeListener(n,r)}}else{if(!e||!e.length)throw new TypeError("Invalid event target");for(var c=0,u=e.length;c<u;c++)t(e[c],n,r,i,o)}i.add(a)}(t,e,function(t){arguments.length>1?r.next(Array.prototype.slice.call(arguments)):r.next(t)},r,n)})}function et(t,e,n){return n?et(t,e).pipe(Object(R.a)(function(t){return Object(F.a)(t)?n.apply(void 0,t):n(t)})):new r.a(function(n){var r,i=function(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return n.next(1===t.length?t[0]:t)};try{r=t(i)}catch(t){return void n.error(t)}if(Object($.a)(e))return function(){return e(i,r)}})}function nt(t,e,n,i,o){var a,s;if(1==arguments.length){var A=t;s=A.initialState,e=A.condition,n=A.iterate,a=A.resultSelector||x.a,o=A.scheduler}else void 0===i||Object(T.a)(i)?(s=t,a=x.a,o=i):(s=t,a=i);return new r.a(function(t){var r=s;if(o)return o.schedule(rt,0,{subscriber:t,iterate:n,condition:e,resultSelector:a,state:r});for(;;){if(e){var i=void 0;try{i=e(r)}catch(e){return void t.error(e)}if(!i){t.complete();break}}var A=void 0;try{A=a(r)}catch(e){return void t.error(e)}if(t.next(A),t.closed)break;try{r=n(r)}catch(e){return void t.error(e)}}})}function rt(t){var e=t.subscriber,n=t.condition;if(!e.closed){if(t.needIterate)try{t.state=t.iterate(t.state)}catch(t){return void e.error(t)}else t.needIterate=!0;if(n){var r=void 0;try{r=n(t.state)}catch(t){return void e.error(t)}if(!r)return void e.complete();if(e.closed)return}var i;try{i=t.resultSelector(t.state)}catch(t){return void e.error(t)}if(!e.closed&&(e.next(i),!e.closed))return this.schedule(t)}}function it(t,e,n){return void 0===e&&(e=q.a),void 0===n&&(n=q.a),Object(J.a)(function(){return t()?e:n})}var ot=n(95);function at(t,e){return void 0===t&&(t=0),void 0===e&&(e=g.a),(!Object(ot.a)(t)||t<0)&&(t=0),e&&"function"==typeof e.schedule||(e=g.a),new r.a(function(n){return n.add(e.schedule(st,t,{subscriber:n,counter:0,period:t})),n})}function st(t){var e=t.subscriber,n=t.counter,r=t.period;e.next(n),this.schedule({subscriber:e,counter:n+1,period:r},r)}var At=n(251),lt=new r.a(w.a);function ct(){return lt}var ut=n(114);function gt(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];if(0===t.length)return q.a;var n=t[0],i=t.slice(1);return 1===t.length&&Object(F.a)(n)?gt.apply(void 0,n):new r.a(function(t){var e=function(){return t.add(gt.apply(void 0,i).subscribe(t))};return Object(W.a)(n).subscribe({next:function(e){t.next(e)},error:e,complete:e})})}function ht(t,e){return e?new r.a(function(n){var r=Object.keys(t),i=new v.a;return i.add(e.schedule(dt,0,{keys:r,index:0,subscriber:n,subscription:i,obj:t})),i}):new r.a(function(e){for(var n=Object.keys(t),r=0;r<n.length&&!e.closed;r++){var i=n[r];t.hasOwnProperty(i)&&e.next([i,t[i]])}e.complete()})}function dt(t){var e=t.keys,n=t.index,r=t.subscriber,i=t.subscription,o=t.obj;if(!r.closed)if(n<e.length){var a=e[n];r.next([a,o[a]]),i.add(this.schedule({keys:e,index:n+1,subscriber:r,subscription:i,obj:o}))}else r.complete()}var ft=n(262),pt=n(132),It=n(68);function Ct(t,e,n){return[Object(It.a)(e,n)(new r.a(Object(pt.a)(t))),Object(It.a)(Object(ft.a)(e,n))(new r.a(Object(pt.a)(t)))]}var mt=n(252);function yt(t,e,n){return void 0===t&&(t=0),new r.a(function(r){void 0===e&&(e=t,t=0);var i=0,o=t;if(n)return n.schedule(Et,0,{index:i,count:e,start:t,subscriber:r});for(;;){if(i++>=e){r.complete();break}if(r.next(o++),r.closed)break}})}function Et(t){var e=t.start,n=t.index,r=t.count,i=t.subscriber;n>=r?i.complete():(i.next(e),i.closed||(t.index=n+1,t.start=e+1,this.schedule(t)))}var vt=n(150),Bt=n(253);function bt(t,e){return new r.a(function(n){var r,i;try{r=t()}catch(t){return void n.error(t)}try{i=e(r)}catch(t){return void n.error(t)}var o=(i?Object(W.a)(i):q.a).subscribe(n);return function(){o.unsubscribe(),r&&r.unsubscribe()}})}var Qt=n(154),wt=n(255),xt=n(63);n.d(e,"Observable",function(){return r.a}),n.d(e,"ConnectableObservable",function(){return i.a}),n.d(e,"GroupedObservable",function(){return o.a}),n.d(e,"observable",function(){return a.a}),n.d(e,"Subject",function(){return s.a}),n.d(e,"BehaviorSubject",function(){return A.a}),n.d(e,"ReplaySubject",function(){return l.a}),n.d(e,"AsyncSubject",function(){return c.a}),n.d(e,"asapScheduler",function(){return u.a}),n.d(e,"asyncScheduler",function(){return g.a}),n.d(e,"queueScheduler",function(){return h.a}),n.d(e,"animationFrameScheduler",function(){return C}),n.d(e,"VirtualTimeScheduler",function(){return m}),n.d(e,"VirtualAction",function(){return y}),n.d(e,"Scheduler",function(){return E.a}),n.d(e,"Subscription",function(){return v.a}),n.d(e,"Subscriber",function(){return B.a}),n.d(e,"Notification",function(){return b.a}),n.d(e,"NotificationKind",function(){return b.b}),n.d(e,"pipe",function(){return Q.a}),n.d(e,"noop",function(){return w.a}),n.d(e,"identity",function(){return x.a}),n.d(e,"isObservable",function(){return _}),n.d(e,"ArgumentOutOfRangeError",function(){return D.a}),n.d(e,"EmptyError",function(){return M.a}),n.d(e,"ObjectUnsubscribedError",function(){return k.a}),n.d(e,"UnsubscriptionError",function(){return S.a}),n.d(e,"TimeoutError",function(){return L.a}),n.d(e,"bindCallback",function(){return U}),n.d(e,"bindNodeCallback",function(){return H}),n.d(e,"combineLatest",function(){return K.b}),n.d(e,"concat",function(){return z.a}),n.d(e,"defer",function(){return J.a}),n.d(e,"empty",function(){return q.b}),n.d(e,"forkJoin",function(){return X}),n.d(e,"from",function(){return W.a}),n.d(e,"fromEvent",function(){return tt}),n.d(e,"fromEventPattern",function(){return et}),n.d(e,"generate",function(){return nt}),n.d(e,"iif",function(){return it}),n.d(e,"interval",function(){return at}),n.d(e,"merge",function(){return At.a}),n.d(e,"never",function(){return ct}),n.d(e,"of",function(){return ut.a}),n.d(e,"onErrorResumeNext",function(){return gt}),n.d(e,"pairs",function(){return ht}),n.d(e,"partition",function(){return Ct}),n.d(e,"race",function(){return mt.a}),n.d(e,"range",function(){return yt}),n.d(e,"throwError",function(){return vt.a}),n.d(e,"timer",function(){return Bt.a}),n.d(e,"using",function(){return bt}),n.d(e,"zip",function(){return Qt.b}),n.d(e,"scheduled",function(){return wt.a}),n.d(e,"EMPTY",function(){return q.a}),n.d(e,"NEVER",function(){return lt}),n.d(e,"config",function(){return xt.a})},function(t,e,n){"use strict";function r(){}n.d(e,"a",function(){return r})},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(141);e.ReactWrapperWidget=r.ReactWrapperWidget;var i=n(312);e.HoverManager=i.HoverManager;var o=n(636);e.ITypeEditorFactory=o.ITypeEditorFactory,e.typeEditorFactoryPlugin=o.typeEditorFactoryPlugin;var a=n(1184);e.TypeEditor=a.TypeEditor;var s=n(314);e.TypeEditorHost=s.TypeEditorHost;var A=n(1185);e.defaultTypeEditors=A.defaultTypeEditors;var l=n(1264);e.InputWidgets=l.InputWidgets;var c=n(633);e.Interactions=c.Interactions,function(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}(n(222))},function(t,e,n){"use strict";function r(){var t=this.constructor.getDerivedStateFromProps(this.props,this.state);null!=t&&this.setState(t)}function i(t){this.setState(function(e){var n=this.constructor.getDerivedStateFromProps(t,e);return null!=n?n:null}.bind(this))}function o(t,e){try{var n=this.props,r=this.state;this.props=t,this.state=e,this.__reactInternalSnapshotFlag=!0,this.__reactInternalSnapshot=this.getSnapshotBeforeUpdate(n,r)}finally{this.props=n,this.state=r}}function a(t){var e=t.prototype;if(!e||!e.isReactComponent)throw new Error("Can only polyfill class components");if("function"!=typeof t.getDerivedStateFromProps&&"function"!=typeof e.getSnapshotBeforeUpdate)return t;var n=null,a=null,s=null;if("function"==typeof e.componentWillMount?n="componentWillMount":"function"==typeof e.UNSAFE_componentWillMount&&(n="UNSAFE_componentWillMount"),"function"==typeof e.componentWillReceiveProps?a="componentWillReceiveProps":"function"==typeof e.UNSAFE_componentWillReceiveProps&&(a="UNSAFE_componentWillReceiveProps"),"function"==typeof e.componentWillUpdate?s="componentWillUpdate":"function"==typeof e.UNSAFE_componentWillUpdate&&(s="UNSAFE_componentWillUpdate"),null!==n||null!==a||null!==s){var A=t.displayName||t.name,l="function"==typeof t.getDerivedStateFromProps?"getDerivedStateFromProps()":"getSnapshotBeforeUpdate()";throw Error("Unsafe legacy lifecycles will not be called for components using new component APIs.\n\n"+A+" uses "+l+" but also contains the following legacy lifecycles:"+(null!==n?"\n  "+n:"")+(null!==a?"\n  "+a:"")+(null!==s?"\n  "+s:"")+"\n\nThe above lifecycles should be removed. Learn more about this warning here:\nhttps://fb.me/react-async-component-lifecycle-hooks")}if("function"==typeof t.getDerivedStateFromProps&&(e.componentWillMount=r,e.componentWillReceiveProps=i),"function"==typeof e.getSnapshotBeforeUpdate){if("function"!=typeof e.componentDidUpdate)throw new Error("Cannot polyfill getSnapshotBeforeUpdate() for components that do not define componentDidUpdate() on the prototype");e.componentWillUpdate=o;var c=e.componentDidUpdate;e.componentDidUpdate=function(t,e,n){var r=this.__reactInternalSnapshotFlag?this.__reactInternalSnapshot:n;c.call(this,t,e,r)}}return t}n.r(e),n.d(e,"polyfill",function(){return a}),r.__suppressDeprecationWarning=!0,i.__suppressDeprecationWarning=!0,o.__suppressDeprecationWarning=!0},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r=n(2),i=n(11);function o(t,e){return function(n){return n.lift(new a(t,e))}}var a=function(){function t(t,e){this.predicate=t,this.thisArg=e}return t.prototype.call=function(t,e){return e.subscribe(new s(t,this.predicate,this.thisArg))},t}(),s=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.predicate=n,i.thisArg=r,i.count=0,i}return r.__extends(e,t),e.prototype._next=function(t){var e;try{e=this.predicate.call(this.thisArg,t,this.count++)}catch(t){return void this.destination.error(t)}e&&this.destination.next(t)},e}(i.a)},function(t,e,n){"use strict";n.d(e,"b",function(){return r}),n.d(e,"a",function(){return s});var r,i=n(51),o=n(114),a=n(150);r||(r={});var s=function(){function t(t,e,n){this.kind=t,this.value=e,this.error=n,this.hasValue="N"===t}return t.prototype.observe=function(t){switch(this.kind){case"N":return t.next&&t.next(this.value);case"E":return t.error&&t.error(this.error);case"C":return t.complete&&t.complete()}},t.prototype.do=function(t,e,n){switch(this.kind){case"N":return t&&t(this.value);case"E":return e&&e(this.error);case"C":return n&&n()}},t.prototype.accept=function(t,e,n){return t&&"function"==typeof t.next?this.observe(t):this.do(t,e,n)},t.prototype.toObservable=function(){switch(this.kind){case"N":return Object(o.a)(this.value);case"E":return Object(a.a)(this.error);case"C":return Object(i.b)()}throw new Error("unexpected notification kind value")},t.createNext=function(e){return void 0!==e?new t("N",e):t.undefinedValueNotification},t.createError=function(e){return new t("E",void 0,e)},t.createComplete=function(){return t.completeNotification},t.completeNotification=new t("C"),t.undefinedValueNotification=new t("N",void 0),t}()},function(t,e,n){"use strict";n.d(e,"a",function(){return s});var r=n(726),i=n.n(r),o=n(16),a=n.n(o);function s(t,e,n,r){var o=a.a.unstable_batchedUpdates?function(t){a.a.unstable_batchedUpdates(n,t)}:n;return i()(t,e,o,r)}},function(t,e){var n=t.exports={version:"2.6.9"};"number"==typeof __e&&(__e=n)},function(t,e,n){"use strict";n.r(e);var r=n(15),i=n.n(r),o=n(9),a=n(5),s=n(50),A=n(10),l=n(40),c=n(17),u=n(200),g=n(576);n(292),n(1044),n(734);i.a.defineMode("ipython",(t,e)=>{let n={};for(let t in e)e.hasOwnProperty(t)&&(n[t]=e[t]);return n.name="python",n.singleOperators=new RegExp("^[\\+\\-\\*/%&|@\\^~<>!\\?]"),n.identifiers=new RegExp("^[_A-Za-z¡-][_A-Za-z0-9¡-]*"),i.a.getMode(t,n)},"python"),i.a.defineMIME("text/x-ipython","ipython"),i.a.modeInfo.push({ext:[],mime:"text/x-ipython",mode:"ipython",name:"ipython"});n(735),n(736),n(391);i.a.defineMode("ipythongfm",(t,e)=>{let n=i.a.getMode(t,{name:"gfm",tokenTypeOverrides:{list3:"string-2"}}),r=i.a.getMode(t,{name:"stex",inMathMode:!0});return i.a.multiplexingMode(n,{open:"$$",close:"$$",mode:r,delimStyle:"delimit"},{open:"$",close:"$",mode:r,delimStyle:"delimit"},{open:"\\(",close:"\\)",mode:r,delimStyle:"delimit"},{open:"\\[",close:"\\]",mode:r,delimStyle:"delimit"})},"gfm"),i.a.defineMIME("text/x-ipythongfm","ipythongfm"),i.a.modeInfo.push({ext:[],mime:"text/x-ipythongfm",mode:"ipythongfm",name:"ipythongfm"});var h,d;n(390),n(730),n(737),n(738),n(392),n(733),n(739),n(740);!function(t){let e=[{loader:async t=>i.a.modes.hasOwnProperty(t.mode),rank:0},{loader:function(t){return new Promise((e,r)=>{Promise.all([n.e(1),n.e(2)]).then(function(){var r=[n(1566)(`./${t.mode}/${t.mode}.js`)];(function(){e(!0)}).apply(null,r)}).catch(n.oe)})},rank:99}];function r(t){let e="string"==typeof t?t:t.mode||t.name,n="string"!=typeof t?t.mime:e,r="string"!=typeof t?t.ext:[];return i.a.findModeByName(e||"")||i.a.findModeByMIME(n||"")||o(r)||i.a.findModeByMIME(g.a.defaultMimeType)||i.a.findModeByMIME("text/plain")}function o(t){if("string"==typeof t)return i.a.findModeByExtension(name);for(let e=0;e<t.length;e++){let n=i.a.findModeByExtension(t[e]);if(n)return n}}t.getModeInfo=function(){return i.a.modeInfo},t.run=function(t,e,n){i.a.runMode(t,e,n)},t.ensure=async function(t){let n=r(t);for(let t of e)if(await t.loader(n))return n;return null},t.addSpecLoader=function(t,n){let r={loader:t,rank:n},i=a.ArrayExt.upperBound(e,r,d.itemCmp);a.ArrayExt.insert(e,i,r)},t.findBest=r,t.findByMIME=function(t){return i.a.findModeByMIME(t)},t.findByName=function(t){return i.a.findModeByName(t)},t.findByFileName=function(t){let e=c.PathExt.basename(t);return i.a.findModeByFileName(e)},t.findByExtension=o}(h||(h={})),(d||(d={})).itemCmp=function(t,e){return t.rank-e.rank};n(1045),n(1046),n(577),n(1047),n(578),n(1048),n(1049),n(1050),n(1051),n(1052),n(1053),n(1054),n(293),n(1055),n(1056),n(1057),n(1058),n(1059),n(1060),n(1061);var f=function(t,e){var n={};for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&e.indexOf(r)<0&&(n[r]=t[r]);if(null!=t&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(t);i<r.length;i++)e.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(t,r[i])&&(n[r[i]]=t[r[i]])}return n};const p="jp-CodeMirrorEditor",I="jp-mod-readOnly",C="jp-CollaboratorCursor",m="jp-CollaboratorCursor-hover",y=38,E=40,v=1e3;class B{constructor(t){this.edgeRequested=new A.Signal(this),this.selectionMarkers={},this._keydownHandlers=new Array,this._changeGuard=!1,this._uuid="",this._needsRefresh=!1,this._isDisposed=!1,this._lastChange=null;let e=this.host=t.host;e.classList.add(p),e.classList.add("jp-Editor"),e.addEventListener("focus",this,!0),e.addEventListener("blur",this,!0),e.addEventListener("scroll",this,!0),this._uuid=t.uuid||o.UUID.uuid4();let n=t.selectionStyle||{};this._selectionStyle=Object.assign({},u.a.defaultSelectionStyle,n);let r=this._model=t.model,s=t.config||{},l=this._config=Object.assign({},B.defaultConfig,s),g=this._editor=Q.createEditor(e,l);g.getDoc().setValue(r.value.text),this.clearHistory(),this._onMimeTypeChanged(),this._onCursorActivity(),this._poll=new c.Poll({factory:async()=>{this._checkSync()},frequency:{interval:3e3,backoff:!1},standby:()=>!this._lastChange||"when-hidden"}),r.value.changed.connect(this._onValueChanged,this),r.mimeTypeChanged.connect(this._onMimeTypeChanged,this),r.selections.changed.connect(this._onSelectionsChanged,this),i.a.on(g,"keydown",(t,e)=>{-1===a.ArrayExt.findFirstIndex(this._keydownHandlers,t=>!0===t(this,e)&&(e.preventDefault(),!0))&&this.onKeydown(e)}),i.a.on(g,"cursorActivity",()=>this._onCursorActivity()),i.a.on(g.getDoc(),"beforeChange",(t,e)=>{this._beforeDocChanged(t,e)}),i.a.on(g.getDoc(),"change",(t,e)=>{"setValue"===e.origin&&this.hasFocus()&&this.refresh(),this._lastChange=e}),g.getWrapperElement().addEventListener("paste",()=>{this.hasFocus()&&this.refresh()})}get uuid(){return this._uuid}set uuid(t){this._uuid=t}get selectionStyle(){return this._selectionStyle}set selectionStyle(t){this._selectionStyle=t}get editor(){return this._editor}get doc(){return this._editor.getDoc()}get lineCount(){return this.doc.lineCount()}get model(){return this._model}get lineHeight(){return this._editor.defaultTextHeight()}get charWidth(){return this._editor.defaultCharWidth()}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,this.host.removeEventListener("focus",this,!0),this.host.removeEventListener("blur",this,!0),this.host.removeEventListener("scroll",this,!0),this._keydownHandlers.length=0,this._poll.dispose(),A.Signal.clearData(this))}getOption(t){return this._config[t]}setOption(t,e){this._config[t]!==e&&(this._config[t]=e,Q.setOption(this.editor,t,e,this._config))}getLine(t){return this.doc.getLine(t)}getOffsetAt(t){return this.doc.indexFromPos({ch:t.column,line:t.line})}getPositionAt(t){const{ch:e,line:n}=this.doc.posFromIndex(t);return{line:n,column:e}}undo(){this.doc.undo()}redo(){this.doc.redo()}clearHistory(){this.doc.clearHistory()}focus(){this._editor.focus()}hasFocus(){return this._editor.getWrapperElement().contains(document.activeElement)}blur(){this._editor.getInputField().blur()}refresh(){this._editor.refresh(),this._needsRefresh=!1}resizeToFit(){this.hasFocus()?this.refresh():this._needsRefresh=!0,this._clearHover()}addOverlay(t,e){this._editor.addOverlay(t,e)}removeOverlay(t){this._editor.removeOverlay(t)}getSearchCursor(t,e,n){return this._editor.getDoc().getSearchCursor(t,e,n)}getCursor(t){return this._editor.getDoc().getCursor(t)}get state(){return this._editor.state}operation(t){return this._editor.operation(t)}firstLine(){return this._editor.getDoc().firstLine()}lastLine(){return this._editor.getDoc().lastLine()}scrollIntoView(t,e){this._editor.scrollIntoView(t,e)}cursorCoords(t,e){return this._editor.cursorCoords(t,e)}getRange(t,e,n){return this._editor.getDoc().getRange(t,e,n)}addKeydownHandler(t){return this._keydownHandlers.push(t),new s.DisposableDelegate(()=>{a.ArrayExt.removeAllWhere(this._keydownHandlers,e=>e===t)})}setSize(t){t?this._editor.setSize(t.width,t.height):this._editor.setSize(null,null),this._needsRefresh=!1}revealPosition(t){const e=this._toCodeMirrorPosition(t);this._editor.scrollIntoView(e)}revealSelection(t){const e={from:this._toCodeMirrorPosition(t.start),to:this._toCodeMirrorPosition(t.end)};this._editor.scrollIntoView(e)}getCoordinateForPosition(t){const e=this._toCodeMirrorPosition(t);return this.editor.charCoords(e,"page")}getPositionForCoordinate(t){return this._toPosition(this.editor.coordsChar(t))||null}getCursorPosition(){const t=this.doc.getCursor();return this._toPosition(t)}setCursorPosition(t,e){const n=this._toCodeMirrorPosition(t);this.doc.setCursor(n,void 0,e),this.editor.hasFocus()||this.model.selections.set(this.uuid,this.getSelections())}getSelection(){return this.getSelections()[0]}setSelection(t){this.setSelections([t])}getSelections(){const t=this.doc.listSelections();if(t.length>0)return t.map(t=>this._toSelection(t));const e=this.doc.getCursor();return[this._toSelection({anchor:e,head:e})]}setSelections(t){const e=this._toCodeMirrorSelections(t);this.doc.setSelections(e,0)}getTokens(){let t=[];for(let e=0;e<this.lineCount;++e){const n=this.editor.getLineTokens(e).map(t=>({offset:this.getOffsetAt({column:t.start,line:e}),value:t.string,type:t.type||""}));t=t.concat(n)}return t}getTokenForPosition(t){const e=this._toCodeMirrorPosition(t),n=this.editor.getTokenAt(e);return{offset:this.getOffsetAt({column:n.start,line:e.line}),value:n.string,type:n.type}}newIndentedLine(){this.execCommand("newlineAndIndent")}execCommand(t){this._editor.execCommand(t)}onKeydown(t){let e=this.getCursorPosition(),{line:n,column:r}=e;if(0===n&&0===r&&t.keyCode===y)return t.shiftKey||this.edgeRequested.emit("top"),!1;if(0===n&&t.keyCode===y)return t.shiftKey||this.edgeRequested.emit("topLine"),!1;let i=this.lineCount-1,o=this.getLine(i).length;return n===i&&r===o&&t.keyCode===E&&(t.shiftKey||this.edgeRequested.emit("bottom"),!1)}_toCodeMirrorSelections(t){if(t.length>0)return t.map(t=>this._toCodeMirrorSelection(t));const e={line:0,ch:0};return[{anchor:e,head:e}]}_onMimeTypeChanged(){const t=this._model.mimeType;let e=this._editor;h.ensure(t).then(t=>{e.setOption("mode",t.mime)});let n=e.getOption("extraKeys")||{};"text/plain"!==t&&"text/x-ipythongfm"!==t?n.Backspace="delSpaceToPrevTabStop":delete n.Backspace,e.setOption("extraKeys",n)}_onSelectionsChanged(t,e){const n=e.key;n!==this.uuid&&(this._cleanSelections(n),"remove"!==e.type&&e.newValue&&this._markSelections(n,e.newValue))}_cleanSelections(t){const e=this.selectionMarkers[t];e&&e.forEach(t=>{t.clear()}),delete this.selectionMarkers[t]}_markSelections(t,e){const n=[];let r;t===this._hoverId&&this._clearHover(),this._model.modelDB.collaborators&&(r=this._model.modelDB.collaborators.get(t)),e.forEach(t=>{if(o.JSONExt.deepEqual(t.start,t.end)){if(r){let e=this._getCaret(r);n.push(this.doc.setBookmark(this._toCodeMirrorPosition(t.end),{widget:e}))}}else{let e,i=t.start.line<t.end.line||t.start.line===t.end.line&&t.start.column<=t.end.column,o=this._toCodeMirrorPosition(i?t.start:t.end),a=this._toCodeMirrorPosition(i?t.end:t.start);e=r?this._toTextMarkerOptions(Object.assign({},t.style,{color:r.color})):this._toTextMarkerOptions(t.style),n.push(this.doc.markText(o,a,e))}}),this.selectionMarkers[t]=n}_onCursorActivity(){if(this._editor.hasFocus()){const t=this.getSelections();this.model.selections.set(this.uuid,t)}}_toSelection(t){return{uuid:this.uuid,start:this._toPosition(t.anchor),end:this._toPosition(t.head),style:this.selectionStyle}}_toTextMarkerOptions(t){let e=`background-color: rgba( ${parseInt(t.color.slice(1,3),16)}, ${parseInt(t.color.slice(3,5),16)}, ${parseInt(t.color.slice(5,7),16)}, 0.15)`;return{className:t.className,title:t.displayName,css:e}}_toCodeMirrorSelection(t){return{anchor:this._toCodeMirrorPosition(t.start),head:this._toCodeMirrorPosition(t.end)}}_toPosition(t){return{line:t.line,column:t.ch}}_toCodeMirrorPosition(t){return{line:t.line,ch:t.column}}_onValueChanged(t,e){if(this._changeGuard)return;this._changeGuard=!0;let n=this.doc;switch(e.type){case"insert":let t=n.posFromIndex(e.start);n.replaceRange(e.value,t,t,"+input");break;case"remove":let r=n.posFromIndex(e.start),i=n.posFromIndex(e.end);n.replaceRange("",r,i,"+input");break;case"set":n.setValue(e.value)}this._changeGuard=!1}_beforeDocChanged(t,e){if(this._changeGuard)return;this._changeGuard=!0;let n=this._model.value,r=t.indexFromPos(e.from),i=t.indexFromPos(e.to),o=e.text.join("\n");i!==r&&n.remove(r,i),o&&n.insert(r,o),this._changeGuard=!1}handleEvent(t){switch(t.type){case"focus":this._evtFocus(t);break;case"blur":this._evtBlur(t);break;case"scroll":this._evtScroll()}}_evtFocus(t){this._needsRefresh&&this.refresh(),this.host.classList.add("jp-mod-focused"),this._onCursorActivity()}_evtBlur(t){this.host.classList.remove("jp-mod-focused")}_evtScroll(){this._clearHover()}_clearHover(){this._caretHover&&(window.clearTimeout(this._hoverTimeout),document.body.removeChild(this._caretHover),this._caretHover=null)}_getCaret(t){let e=t?t.displayName:"Anonymous",n=t?t.color:this._selectionStyle.color,r=document.createElement("span");return r.className=C,r.style.borderBottomColor=n,r.onmouseenter=()=>{this._clearHover(),this._hoverId=t.sessionId;let i=r.getBoundingClientRect(),o=document.createElement("div");o.className=m,o.style.left=String(i.left)+"px",o.style.top=String(i.bottom)+"px",o.textContent=e,o.style.backgroundColor=n,o.onmouseenter=()=>{window.clearTimeout(this._hoverTimeout)},o.onmouseleave=()=>{this._hoverTimeout=window.setTimeout(()=>{this._clearHover()},v)},this._caretHover=o,document.body.appendChild(o)},r.onmouseleave=()=>{this._hoverTimeout=window.setTimeout(()=>{this._clearHover()},v)},r}_checkSync(){let t=this._lastChange;if(!t)return;this._lastChange=null;let e=this._editor,n=e.getDoc();n.getValue()!==this._model.value.text&&(Object(l.b)({title:"Code Editor out of Sync",body:"Please open your browser JavaScript console for bug report instructions"}),console.log("Please paste the following to https://github.com/jupyterlab/jupyterlab/issues/2951"),console.log(JSON.stringify({model:this._model.value.text,view:n.getValue(),selections:this.getSelections(),cursor:this.getCursorPosition(),lineSep:e.getOption("lineSeparator"),mode:e.getOption("mode"),change:t})))}}var b,Q;(b=B||(B={})).defaultConfig=Object.assign({},u.a.defaultConfig,{mode:"null",theme:"jupyter",smartIndent:!0,electricChars:!0,keyMap:"default",extraKeys:null,gutters:[],fixedGutter:!0,showCursorWhenSelecting:!1,coverGutterNextToScrollbar:!1,dragDrop:!0,lineSeparator:null,scrollbarStyle:"native",lineWiseCopyCut:!0,scrollPastEnd:!1,styleActiveLine:!1,styleSelectedText:!0,selectionPointer:!1,rulers:[],foldGutter:!1}),b.addCommand=function(t,e){i.a.commands[t]=e},function(t){function e(t,e){return t.line===e.line&&t.ch===e.ch}function n(t){let e={"CodeMirror-linenumbers":"lineNumbers","CodeMirror-foldgutter":"codeFolding"};return Object.keys(e).filter(n=>t[e[n]])}t.createEditor=function(t,e){let{autoClosingBrackets:n,fontFamily:r,fontSize:o,insertSpaces:a,lineHeight:s,lineWrap:A,wordWrapColumn:l,tabSize:c,readOnly:u}=e,g=f(e,["autoClosingBrackets","fontFamily","fontSize","insertSpaces","lineHeight","lineWrap","wordWrapColumn","tabSize","readOnly"]),h=Object.assign({autoCloseBrackets:!!n&&{},indentUnit:c,indentWithTabs:!a,lineWrapping:"off"!==A,readOnly:u},g);return i()(e=>{r&&(e.style.fontFamily=r),o&&(e.style.fontSize=o+"px"),s&&(e.style.lineHeight=s.toString()),u&&e.classList.add(I),"wordWrapColumn"===A&&(e.querySelector(".CodeMirror-lines").style.width=`${l}ch`),"bounded"===A&&(e.querySelector(".CodeMirror-lines").style.maxWidth=`${l}ch`),t.appendChild(e)},h)},t.indentMoreOrinsertTab=function(t){let n=t.getDoc(),r=n.getCursor("from");if(!e(r,n.getCursor("to")))return void i.a.commands.indentMore(t);let o=n.getLine(r.line).slice(0,r.ch);/^\s*$/.test(o)?i.a.commands.indentMore(t):t.getOption("indentWithTabs")?i.a.commands.insertTab(t):i.a.commands.insertSoftTab(t)},t.delSpaceToPrevTabStop=function(t){let n=t.getDoc(),r=n.getCursor("from");if(!e(r,n.getCursor("to"))){let t=n.listSelections();for(let e=t.length-1;e>=0;e--){let r=t[e].head,o=t[e].anchor;n.replaceRange("",i.a.Pos(r.line,r.ch),i.a.Pos(o.line,o.ch))}return}let o=n.getCursor(),a=t.getOption("indentUnit"),s=o.ch-(Math.ceil(o.ch/a)-1)*a;r={ch:o.ch-s,line:o.line},null!==n.getRange(r,o).match(/^\ +$/)?n.replaceRange("",r,o):i.a.commands.delCharBefore(t)},t.posEq=e,t.setOption=function(t,e,r,i){let o=t.getWrapperElement();switch(e){case"lineWrap":const a="off"!==r,s=o.querySelector(".CodeMirror-lines"),A="bounded"===r?`${i.wordWrapColumn}ch`:null,l="wordWrapColumn"===r?`${i.wordWrapColumn}ch`:null;s.style.maxWidth=A,s.style.width=l,t.setOption("lineWrapping",a);break;case"wordWrapColumn":const{lineWrap:c}=i;if("wordWrapColumn"===c||"bounded"===c){const t="wordWrapColumn"===c?"width":"maxWidth";o.querySelector(".CodeMirror-lines").style[t]=`${r}ch`}break;case"tabSize":t.setOption("indentUnit",r);break;case"insertSpaces":t.setOption("indentWithTabs",!r);break;case"autoClosingBrackets":t.setOption("autoCloseBrackets",r);break;case"rulers":let u=r;t.setOption("rulers",u.map(t=>({column:t,className:"jp-CodeMirror-ruler"})));break;case"readOnly":o.classList.toggle(I,r),t.setOption(e,r);break;case"fontFamily":o.style.fontFamily=r;break;case"fontSize":o.style.fontSize=r?r+"px":null;break;case"lineHeight":o.style.lineHeight=r?r.toString():null;break;case"gutters":t.setOption(e,n(i));break;case"lineNumbers":t.setOption(e,r),t.setOption("gutters",n(i));break;case"codeFolding":t.setOption("foldGutter",r),t.setOption("gutters",n(i));break;default:t.setOption(e,r)}}}(Q||(Q={})),B.addCommand("delSpaceToPrevTabStop",Q.delSpaceToPrevTabStop),B.addCommand("indentMoreOrinsertTab",Q.indentMoreOrinsertTab);class w{constructor(t={}){this.newInlineEditor=t=>(t.host.dataset.type="inline",new B(Object.assign({},t,{config:Object.assign({},this.inlineCodeMirrorConfig,t.config||{})}))),this.newDocumentEditor=t=>(t.host.dataset.type="document",new B(Object.assign({},t,{config:Object.assign({},this.documentCodeMirrorConfig,t.config||{})}))),this.inlineCodeMirrorConfig=Object.assign({},B.defaultConfig,{extraKeys:{"Cmd-Right":"goLineRight",End:"goLineRight","Cmd-Left":"goLineLeft",Tab:"indentMoreOrinsertTab","Shift-Tab":"indentLess","Cmd-/":"toggleComment","Ctrl-/":"toggleComment"}},t),this.documentCodeMirrorConfig=Object.assign({},B.defaultConfig,{extraKeys:{Tab:"indentMoreOrinsertTab","Shift-Tab":"indentLess","Cmd-/":"toggleComment","Ctrl-/":"toggleComment","Shift-Enter":()=>{}},lineNumbers:!0,scrollPastEnd:!0},t)}}class x{getMimeTypeByLanguage(t){let e=t.file_extension||"";return h.findBest(t.codemirror_mode||{mimetype:t.mimetype,name:t.name,ext:[e.split(".").slice(-1)[0]]}).mime}getMimeTypeByFilePath(t){const e=c.PathExt.extname(t);if(".ipy"===e)return"text/x-python";if(".md"===e)return"text/x-ipythongfm";return(h.findByFileName(t)||h.findBest("")).mime}}var _=n(0),D=n.n(_),M=n(76),k=n(164),S=n(6);function L(t){return D.a.createElement(k.b,{source:t.mode,onClick:t.handleClick})}class R extends M.d{constructor(t){super(),this._handleClick=()=>{const t=new S.Menu({commands:this._commands});this._popup&&this._popup.dispose(),h.getModeInfo().sort((t,e)=>{let n=t.name||"",r=e.name||"";return n.localeCompare(r)}).forEach(e=>{if(0===e.mode.indexOf("brainf"))return;let n={insertSpaces:!0,name:e.name};t.addItem({command:"codemirror:change-mode",args:n})}),this._popup=Object(k.d)({body:t,anchor:this,align:"left"})},this._popup=null,this.model=new R.Model,this._commands=t.commands,this.addClass(k.c),this.title.caption="Change text editor syntax highlighting"}render(){return this.model?D.a.createElement(L,{mode:this.model.mode,handleClick:this._handleClick}):null}}(R||(R={})).Model=class extends M.c{constructor(){super(...arguments),this._onMIMETypeChange=(t,e)=>{const n=this._mode,r=h.findByMIME(e.newValue);this._mode=r.name||r.mode,this._triggerChange(n,this._mode)},this._mode="",this._editor=null}get mode(){return this._mode}get editor(){return this._editor}set editor(t){const e=this._editor;null!==e&&e.model.mimeTypeChanged.disconnect(this._onMIMETypeChange);const n=this._mode;if(this._editor=t,null===this._editor)this._mode="";else{const t=h.findByMIME(this._editor.model.mimeType);this._mode=t.name||t.mode,this._editor.model.mimeTypeChanged.connect(this._onMIMETypeChange)}this._triggerChange(n,this._mode)}_triggerChange(t,e){t!==e&&this.stateChanged.emit(void 0)}},n.d(e,"editorServices",function(){return N}),n.d(e,"Mode",function(){return h}),n.d(e,"CodeMirrorEditor",function(){return B}),n.d(e,"CodeMirrorEditorFactory",function(){return w}),n.d(e,"CodeMirrorMimeTypeService",function(){return x}),n.d(e,"EditorSyntaxStatus",function(){return R});const N={factoryService:new w,mimeTypeService:new x}},function(t,e,n){"use strict";function r(t){return t}n.d(e,"a",function(){return r})},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r={NONE:"none",PRIMARY:"primary",SUCCESS:"success",WARNING:"warning",DANGER:"danger"}},function(t,e,n){"use strict";function r(){return"function"==typeof Symbol&&Symbol.iterator?Symbol.iterator:"@@iterator"}n.d(e,"a",function(){return i});var i=r()},function(t,e,n){"use strict";n.d(e,"a",function(){return A}),n.d(e,"d",function(){return l}),n.d(e,"b",function(){return c}),n.d(e,"c",function(){return u});var r=n(25),i=n(10),o=n(6),a=n(0),s=n(16);class A extends o.Widget{static create(t){return new class extends A{render(){return t}}}onUpdateRequest(t){this.renderPromise=this.renderDOM()}onAfterAttach(t){r.MessageLoop.sendMessage(this,o.Widget.Msg.UpdateRequest)}onBeforeDetach(t){s.unmountComponentAtNode(this.node)}renderDOM(){return new Promise(t=>{let e=this.render();Array.isArray(e),s.render(e,this.node,t)})}}class l extends A{constructor(){super(...arguments),this._modelChanged=new i.Signal(this)}get modelChanged(){return this._modelChanged}set model(t){this._model!==t&&(this._model&&this._model.stateChanged.disconnect(this.update,this),this._model=t,t&&t.stateChanged.connect(this.update,this),this.update(),this._modelChanged.emit(void 0))}get model(){return this._model}dispose(){this._model=null,super.dispose()}}class c extends a.Component{constructor(t){super(t),this.slot=(t,e)=>{this.props.shouldUpdate&&!this.props.shouldUpdate(t,e)||this.setState({value:[t,e]})},this.state={value:[this.props.initialSender,this.props.initialArgs]}}componentDidMount(){this.props.signal.connect(this.slot)}componentWillUnmount(){this.props.signal.disconnect(this.slot)}render(){return this.props.children(...this.state.value)}}class u{constructor(){this.stateChanged=new i.Signal(this),this._isDisposed=!1}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,i.Signal.clearData(this))}}},function(t,e){function n(t,e){if(!t)throw new Error(e||"Assertion failed")}t.exports=n,n.equal=function(t,e,n){if(t!=e)throw new Error(n||"Assertion failed: "+t+" != "+e)}},function(t,e,n){"use strict";var r=e,i=n(53),o=n(77),a=n(417);r.assert=o,r.toArray=a.toArray,r.zero2=a.zero2,r.toHex=a.toHex,r.encode=a.encode,r.getNAF=function(t,e){for(var n=[],r=1<<e+1,i=t.clone();i.cmpn(1)>=0;){var o;if(i.isOdd()){var a=i.andln(r-1);o=a>(r>>1)-1?(r>>1)-a:a,i.isubn(o)}else o=0;n.push(o);for(var s=0!==i.cmpn(0)&&0===i.andln(r-1)?e+1:1,A=1;A<s;A++)n.push(0);i.iushrn(s)}return n},r.getJSF=function(t,e){var n=[[],[]];t=t.clone(),e=e.clone();for(var r=0,i=0;t.cmpn(-r)>0||e.cmpn(-i)>0;){var o,a,s,A=t.andln(3)+r&3,l=e.andln(3)+i&3;3===A&&(A=-1),3===l&&(l=-1),o=0==(1&A)?0:3!=(s=t.andln(7)+r&7)&&5!==s||2!==l?A:-A,n[0].push(o),a=0==(1&l)?0:3!=(s=e.andln(7)+i&7)&&5!==s||2!==A?l:-l,n[1].push(a),2*r===o+1&&(r=1-r),2*i===a+1&&(i=1-i),t.iushrn(1),e.iushrn(1)}return n},r.cachedProperty=function(t,e,n){var r="_"+e;t.prototype[e]=function(){return void 0!==this[r]?this[r]:this[r]=n.call(this)}},r.parseBytes=function(t){return"string"==typeof t?r.toArray(t,"hex"):t},r.intFromLE=function(t){return new i(t,"hex","le")}},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r="function"==typeof Symbol&&Symbol.observable||"@@observable"},function(t,e,n){"use strict";var r=function(){};t.exports=r},function(t,e,n){"use strict";function r(){return Error.call(this),this.message="object unsubscribed",this.name="ObjectUnsubscribedError",this}n.d(e,"a",function(){return i}),r.prototype=Object.create(Error.prototype);var i=r},function(t,e,n){"use strict";function r(t){return"function"==typeof t.iter?t.iter():new i(t)}Object.defineProperty(e,"__esModule",{value:!0}),e.iter=r,e.iterKeys=function(t){return new o(t)},e.iterValues=function(t){return new a(t)},e.iterItems=function(t){return new s(t)},e.iterFn=function(t){return new A(t)},e.each=function(t,e){for(var n,i=0,o=r(t);void 0!==(n=o.next());)if(!1===e(n,i++))return},e.every=function(t,e){for(var n,i=0,o=r(t);void 0!==(n=o.next());)if(!e(n,i++))return!1;return!0},e.some=function(t,e){for(var n,i=0,o=r(t);void 0!==(n=o.next());)if(e(n,i++))return!0;return!1},e.toArray=function(t){for(var e,n=0,i=[],o=r(t);void 0!==(e=o.next());)i[n++]=e;return i},e.toObject=function(t){for(var e,n=r(t),i={};void 0!==(e=n.next());)i[e[0]]=e[1];return i};var i=function(){function t(t){this._index=0,this._source=t}return t.prototype.iter=function(){return this},t.prototype.clone=function(){var e=new t(this._source);return e._index=this._index,e},t.prototype.next=function(){if(!(this._index>=this._source.length))return this._source[this._index++]},t}();e.ArrayIterator=i;var o=function(){function t(t,e){void 0===e&&(e=Object.keys(t)),this._index=0,this._source=t,this._keys=e}return t.prototype.iter=function(){return this},t.prototype.clone=function(){var e=new t(this._source,this._keys);return e._index=this._index,e},t.prototype.next=function(){if(!(this._index>=this._keys.length)){var t=this._keys[this._index++];return t in this._source?t:this.next()}},t}();e.KeyIterator=o;var a=function(){function t(t,e){void 0===e&&(e=Object.keys(t)),this._index=0,this._source=t,this._keys=e}return t.prototype.iter=function(){return this},t.prototype.clone=function(){var e=new t(this._source,this._keys);return e._index=this._index,e},t.prototype.next=function(){if(!(this._index>=this._keys.length)){var t=this._keys[this._index++];return t in this._source?this._source[t]:this.next()}},t}();e.ValueIterator=a;var s=function(){function t(t,e){void 0===e&&(e=Object.keys(t)),this._index=0,this._source=t,this._keys=e}return t.prototype.iter=function(){return this},t.prototype.clone=function(){var e=new t(this._source,this._keys);return e._index=this._index,e},t.prototype.next=function(){if(!(this._index>=this._keys.length)){var t=this._keys[this._index++];return t in this._source?[t,this._source[t]]:this.next()}},t}();e.ItemIterator=s;var A=function(){function t(t){this._fn=t}return t.prototype.iter=function(){return this},t.prototype.clone=function(){throw new Error("An `FnIterator` cannot be cloned.")},t.prototype.next=function(){return this._fn.call(void 0)},t}();e.FnIterator=A},function(t,e,n){"use strict";n.r(e);var r=n(200),i=n(627),o=n(384),a=n(576);const s=new(n(9).Token)("@jupyterlab/codeeditor:IEditorServices");n.d(e,"CodeEditor",function(){return r.a}),n.d(e,"JSONEditor",function(){return i.a}),n.d(e,"CodeEditorWrapper",function(){return o.a}),n.d(e,"IEditorMimeTypeService",function(){return a.a}),n.d(e,"IEditorServices",function(){return s})},function(t,e,n){"use strict";n.d(e,"a",function(){return l});var r=n(2),i=n(24),o=n(26),a=n(62),s=n(41),A=n(55);function l(t,e,n){return void 0===n&&(n=Number.POSITIVE_INFINITY),"function"==typeof e?function(r){return r.pipe(l(function(n,r){return Object(A.a)(t(n,r)).pipe(Object(s.a)(function(t,i){return e(n,t,r,i)}))},n))}:("number"==typeof e&&(n=e),function(e){return e.lift(new c(t,n))})}var c=function(){function t(t,e){void 0===e&&(e=Number.POSITIVE_INFINITY),this.project=t,this.concurrent=e}return t.prototype.call=function(t,e){return e.subscribe(new u(t,this.project,this.concurrent))},t}(),u=function(t){function e(e,n,r){void 0===r&&(r=Number.POSITIVE_INFINITY);var i=t.call(this,e)||this;return i.project=n,i.concurrent=r,i.hasCompleted=!1,i.buffer=[],i.active=0,i.index=0,i}return r.__extends(e,t),e.prototype._next=function(t){this.active<this.concurrent?this._tryNext(t):this.buffer.push(t)},e.prototype._tryNext=function(t){var e,n=this.index++;try{e=this.project(t,n)}catch(t){return void this.destination.error(t)}this.active++,this._innerSub(e,t,n)},e.prototype._innerSub=function(t,e,n){var r=new a.a(this,void 0,void 0);this.destination.add(r),Object(i.a)(this,t,e,n,r)},e.prototype._complete=function(){this.hasCompleted=!0,0===this.active&&0===this.buffer.length&&this.destination.complete(),this.unsubscribe()},e.prototype.notifyNext=function(t,e,n,r,i){this.destination.next(e)},e.prototype.notifyComplete=function(t){var e=this.buffer;this.remove(t),this.active--,e.length>0?this._next(e.shift()):0===this.active&&this.hasCompleted&&this.destination.complete()},e}(o.a)},function(t,e,n){"use strict";function r(){return Error.call(this),this.message="argument out of range",this.name="ArgumentOutOfRangeError",this}n.d(e,"a",function(){return i}),r.prototype=Object.create(Error.prototype);var i=r},function(t,e,n){"use strict";function r(t){return"function"==typeof t}n.d(e,"a",function(){return r})},function(t,e,n){"use strict";var r=n(77),i=n(30);function o(t,e){return 55296==(64512&t.charCodeAt(e))&&(!(e<0||e+1>=t.length)&&56320==(64512&t.charCodeAt(e+1)))}function a(t){return(t>>>24|t>>>8&65280|t<<8&16711680|(255&t)<<24)>>>0}function s(t){return 1===t.length?"0"+t:t}function A(t){return 7===t.length?"0"+t:6===t.length?"00"+t:5===t.length?"000"+t:4===t.length?"0000"+t:3===t.length?"00000"+t:2===t.length?"000000"+t:1===t.length?"0000000"+t:t}e.inherits=i,e.toArray=function(t,e){if(Array.isArray(t))return t.slice();if(!t)return[];var n=[];if("string"==typeof t)if(e){if("hex"===e)for((t=t.replace(/[^a-z0-9]+/gi,"")).length%2!=0&&(t="0"+t),i=0;i<t.length;i+=2)n.push(parseInt(t[i]+t[i+1],16))}else for(var r=0,i=0;i<t.length;i++){var a=t.charCodeAt(i);a<128?n[r++]=a:a<2048?(n[r++]=a>>6|192,n[r++]=63&a|128):o(t,i)?(a=65536+((1023&a)<<10)+(1023&t.charCodeAt(++i)),n[r++]=a>>18|240,n[r++]=a>>12&63|128,n[r++]=a>>6&63|128,n[r++]=63&a|128):(n[r++]=a>>12|224,n[r++]=a>>6&63|128,n[r++]=63&a|128)}else for(i=0;i<t.length;i++)n[i]=0|t[i];return n},e.toHex=function(t){for(var e="",n=0;n<t.length;n++)e+=s(t[n].toString(16));return e},e.htonl=a,e.toHex32=function(t,e){for(var n="",r=0;r<t.length;r++){var i=t[r];"little"===e&&(i=a(i)),n+=A(i.toString(16))}return n},e.zero2=s,e.zero8=A,e.join32=function(t,e,n,i){var o=n-e;r(o%4==0);for(var a=new Array(o/4),s=0,A=e;s<a.length;s++,A+=4){var l;l="big"===i?t[A]<<24|t[A+1]<<16|t[A+2]<<8|t[A+3]:t[A+3]<<24|t[A+2]<<16|t[A+1]<<8|t[A],a[s]=l>>>0}return a},e.split32=function(t,e){for(var n=new Array(4*t.length),r=0,i=0;r<t.length;r++,i+=4){var o=t[r];"big"===e?(n[i]=o>>>24,n[i+1]=o>>>16&255,n[i+2]=o>>>8&255,n[i+3]=255&o):(n[i+3]=o>>>24,n[i+2]=o>>>16&255,n[i+1]=o>>>8&255,n[i]=255&o)}return n},e.rotr32=function(t,e){return t>>>e|t<<32-e},e.rotl32=function(t,e){return t<<e|t>>>32-e},e.sum32=function(t,e){return t+e>>>0},e.sum32_3=function(t,e,n){return t+e+n>>>0},e.sum32_4=function(t,e,n,r){return t+e+n+r>>>0},e.sum32_5=function(t,e,n,r,i){return t+e+n+r+i>>>0},e.sum64=function(t,e,n,r){var i=t[e],o=r+t[e+1]>>>0,a=(o<r?1:0)+n+i;t[e]=a>>>0,t[e+1]=o},e.sum64_hi=function(t,e,n,r){return(e+r>>>0<e?1:0)+t+n>>>0},e.sum64_lo=function(t,e,n,r){return e+r>>>0},e.sum64_4_hi=function(t,e,n,r,i,o,a,s){var A=0,l=e;return A+=(l=l+r>>>0)<e?1:0,A+=(l=l+o>>>0)<o?1:0,t+n+i+a+(A+=(l=l+s>>>0)<s?1:0)>>>0},e.sum64_4_lo=function(t,e,n,r,i,o,a,s){return e+r+o+s>>>0},e.sum64_5_hi=function(t,e,n,r,i,o,a,s,A,l){var c=0,u=e;return c+=(u=u+r>>>0)<e?1:0,c+=(u=u+o>>>0)<o?1:0,c+=(u=u+s>>>0)<s?1:0,t+n+i+a+A+(c+=(u=u+l>>>0)<l?1:0)>>>0},e.sum64_5_lo=function(t,e,n,r,i,o,a,s,A,l){return e+r+o+s+l>>>0},e.rotr64_hi=function(t,e,n){return(e<<32-n|t>>>n)>>>0},e.rotr64_lo=function(t,e,n){return(t<<32-n|e>>>n)>>>0},e.shr64_hi=function(t,e,n){return t>>>n},e.shr64_lo=function(t,e,n){return(t<<32-n|e>>>n)>>>0}},function(t,e,n){var r=n(321)("wks"),i=n(225),o=n(110).Symbol,a="function"==typeof o;(t.exports=function(t){return r[t]||(r[t]=a&&o[t]||(a?o:i)("Symbol."+t))}).store=r},function(t,e,n){"use strict";function r(){return Error.call(this),this.message="no elements in sequence",this.name="EmptyError",this}n.d(e,"a",function(){return i}),r.prototype=Object.create(Error.prototype);var i=r},function(t,e,n){"use strict";n.d(e,"a",function(){return a});var r=n(21),i=n(259),o=n(159);function a(t,e){return e?Object(o.a)(t,e):new r.a(Object(i.a)(t))}},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r=n(2),i=n(199),o=function(t){function e(n,r){void 0===r&&(r=i.a.now);var o=t.call(this,n,function(){return e.delegate&&e.delegate!==o?e.delegate.now():r()})||this;return o.actions=[],o.active=!1,o.scheduled=void 0,o}return r.__extends(e,t),e.prototype.schedule=function(n,r,i){return void 0===r&&(r=0),e.delegate&&e.delegate!==this?e.delegate.schedule(n,r,i):t.prototype.schedule.call(this,n,r,i)},e.prototype.flush=function(t){var e=this.actions;if(this.active)e.push(t);else{var n;this.active=!0;do{if(n=t.execute(t.state,t.delay))break}while(t=e.shift());if(this.active=!1,n){for(;t=e.shift();)t.unsubscribe();throw n}}},e}(i.a)},function(t,e,n){"use strict";n.d(e,"a",function(){return a});var r=n(2),i=n(36),o=n(32),a=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.value=null,e.hasNext=!1,e.hasCompleted=!1,e}return r.__extends(e,t),e.prototype._subscribe=function(e){return this.hasError?(e.error(this.thrownError),o.a.EMPTY):this.hasCompleted&&this.hasNext?(e.next(this.value),e.complete(),o.a.EMPTY):t.prototype._subscribe.call(this,e)},e.prototype.next=function(t){this.hasCompleted||(this.value=t,this.hasNext=!0)},e.prototype.error=function(e){this.hasCompleted||t.prototype.error.call(this,e)},e.prototype.complete=function(){this.hasCompleted=!0,this.hasNext&&t.prototype.next.call(this,this.value),t.prototype.complete.call(this)},e}(i.a)},function(t,e,n){"use strict";var r=n(2),i=function(t){function e(e,n){return t.call(this)||this}return r.__extends(e,t),e.prototype.schedule=function(t,e){return void 0===e&&(e=0),this},e}(n(32).a);n.d(e,"a",function(){return o});var o=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.scheduler=e,r.work=n,r.pending=!1,r}return r.__extends(e,t),e.prototype.schedule=function(t,e){if(void 0===e&&(e=0),this.closed)return this;this.state=t;var n=this.id,r=this.scheduler;return null!=n&&(this.id=this.recycleAsyncId(r,n,e)),this.pending=!0,this.delay=e,this.id=this.id||this.requestAsyncId(r,this.id,e),this},e.prototype.requestAsyncId=function(t,e,n){return void 0===n&&(n=0),setInterval(t.flush.bind(t,this),n)},e.prototype.recycleAsyncId=function(t,e,n){if(void 0===n&&(n=0),null!==n&&this.delay===n&&!1===this.pending)return e;clearInterval(e)},e.prototype.execute=function(t,e){if(this.closed)return new Error("executing a cancelled action");this.pending=!1;var n=this._execute(t,e);if(n)return n;!1===this.pending&&null!=this.id&&(this.id=this.recycleAsyncId(this.scheduler,this.id,null))},e.prototype._execute=function(t,e){var n=!1,r=void 0;try{this.work(t)}catch(t){n=!0,r=!!t&&t||new Error(t)}if(n)return this.unsubscribe(),r},e.prototype._unsubscribe=function(){var t=this.id,e=this.scheduler,n=e.actions,r=n.indexOf(this);this.work=null,this.state=null,this.pending=!1,this.scheduler=null,-1!==r&&n.splice(r,1),null!=t&&(this.id=this.recycleAsyncId(e,t,null)),this.delay=null},e}(i)},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r={ZERO:0,ONE:1,TWO:2,THREE:3,FOUR:4}},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=n(38);function i(t){return!Object(r.a)(t)&&t-parseFloat(t)+1>=0}},function(t,e,n){"use strict";n.d(e,"a",function(){return o}),n.d(e,"c",function(){return a}),n.d(e,"b",function(){return s}),n.d(e,"f",function(){return A}),n.d(e,"d",function(){return l}),n.d(e,"g",function(){return c}),n.d(e,"h",function(){return u}),n.d(e,"i",function(){return g}),n.d(e,"e",function(){return h});var r=n(6),i=n(97);class o extends r.Widget{constructor(t){super(),this.mimeType=t.mimeType,this.sanitizer=t.sanitizer,this.resolver=t.resolver,this.linkHandler=t.linkHandler,this.latexTypesetter=t.latexTypesetter,this.node.dataset.mimeType=this.mimeType}async renderModel(t){for(;this.node.firstChild;)this.node.removeChild(this.node.firstChild);this.toggleClass("jp-mod-trusted",t.trusted),await this.render(t);const{fragment:e}=t.metadata;e&&this.setFragment(e)}setFragment(t){}}class a extends o{constructor(t){super(t),this.addClass("jp-RenderedHTMLCommon")}setFragment(t){let e;try{e=this.node.querySelector(t)}catch(t){console.warn("Unable to set URI fragment identifier.",t)}e&&e.scrollIntoView()}}class s extends a{constructor(t){super(t),this.addClass("jp-RenderedHTML")}render(t){return i.a({host:this.node,source:String(t.data[this.mimeType]),trusted:t.trusted,resolver:this.resolver,sanitizer:this.sanitizer,linkHandler:this.linkHandler,shouldTypeset:this.isAttached,latexTypesetter:this.latexTypesetter})}onAfterAttach(t){this.latexTypesetter&&this.latexTypesetter.typeset(this.node)}}class A extends o{constructor(t){super(t),this.addClass("jp-RenderedLatex")}render(t){return i.c({host:this.node,source:String(t.data[this.mimeType]),shouldTypeset:this.isAttached,latexTypesetter:this.latexTypesetter})}onAfterAttach(t){this.latexTypesetter&&this.latexTypesetter.typeset(this.node)}}class l extends o{constructor(t){super(t),this.addClass("jp-RenderedImage")}render(t){let e=t.metadata[this.mimeType];return i.b({host:this.node,mimeType:this.mimeType,source:String(t.data[this.mimeType]),width:e&&e.width,height:e&&e.height,needsBackground:t.metadata.needs_background,unconfined:e&&e.unconfined})}}class c extends a{constructor(t){super(t),this.addClass("jp-RenderedMarkdown")}render(t){return i.d({host:this.node,source:String(t.data[this.mimeType]),trusted:t.trusted,resolver:this.resolver,sanitizer:this.sanitizer,linkHandler:this.linkHandler,shouldTypeset:this.isAttached,latexTypesetter:this.latexTypesetter})}onAfterAttach(t){this.latexTypesetter&&this.latexTypesetter.typeset(this.node)}}class u extends o{constructor(t){super(t),this.addClass("jp-RenderedSVG")}render(t){let e=t.metadata[this.mimeType];return i.e({host:this.node,source:String(t.data[this.mimeType]),trusted:t.trusted,unconfined:e&&e.unconfined})}onAfterAttach(t){this.latexTypesetter&&this.latexTypesetter.typeset(this.node)}}class g extends o{constructor(t){super(t),this.addClass("jp-RenderedText")}render(t){return i.f({host:this.node,sanitizer:this.sanitizer,source:String(t.data[this.mimeType])})}}class h extends o{constructor(t){super(t),this.addClass("jp-RenderedJavaScript")}render(t){return i.f({host:this.node,sanitizer:this.sanitizer,source:"JavaScript output is disabled in JupyterLab"})}}},function(t,e,n){"use strict";n.d(e,"a",function(){return h}),n.d(e,"b",function(){return d}),n.d(e,"c",function(){return f}),n.d(e,"d",function(){return p}),n.d(e,"e",function(){return I}),n.d(e,"f",function(){return C});var r,i=n(386),o=n.n(i),a=n(72),s=n(17),A=n(5),l=n(719),c=n.n(l),u=n(192),g=function(t,e){var n={};for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&e.indexOf(r)<0&&(n[r]=t[r]);if(null!=t&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(t);i<r.length;i++)e.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(t,r[i])&&(n[r[i]]=t[r[i]])}return n};function h(t){let e,{host:n,source:i,trusted:o,sanitizer:a,resolver:s,linkHandler:A,shouldTypeset:l,latexTypesetter:c}=t,u=i;if(!i)return n.textContent="",Promise.resolve(void 0);if(o||(u=`${i}`,i=a.sanitize(i)),n.innerHTML=i,n.getElementsByTagName("script").length>0)if(o)r.evalInnerHTMLScriptTags(n);else{const t=document.createElement("div"),e=document.createElement("pre");e.textContent="This HTML output contains inline scripts. Are you sure that you want to run arbitrary Javascript within your JupyterLab session?";const i=document.createElement("button");i.textContent="Run",i.onclick=t=>{n.innerHTML=u,r.evalInnerHTMLScriptTags(n),n.removeChild(n.firstChild)},t.appendChild(e),t.appendChild(i),n.insertBefore(t,n.firstChild)}return r.handleDefaults(n,s),(e=s?r.handleUrls(n,s,A):Promise.resolve(void 0)).then(()=>{l&&c&&c.typeset(n)})}function d(t){let{host:e,mimeType:n,source:r,width:i,height:o,needsBackground:a,unconfined:s}=t;e.textContent="";let A=document.createElement("img");return A.src=`data:${n};base64,${r}`,"number"==typeof o&&(A.height=o),"number"==typeof i&&(A.width=i),"light"===a?A.classList.add("jp-needs-light-background"):"dark"===a&&A.classList.add("jp-needs-dark-background"),!0===s&&A.classList.add("jp-mod-unconfined"),e.appendChild(A),Promise.resolve(void 0)}function f(t){let{host:e,source:n,shouldTypeset:r,latexTypesetter:i}=t;return e.textContent=n,r&&i&&i.typeset(e),Promise.resolve(void 0)}async function p(t){let{host:e,source:n}=t,i=g(t,["host","source"]);if(!n)return void(e.textContent="");let o=Object(u.a)(n),a=await r.renderMarked(o.text);a=Object(u.b)(a,o.math),await h(Object.assign({host:e,source:a},i)),r.headerAnchors(e)}function I(t){let{host:e,source:n,trusted:r,unconfined:i}=t;if(!n)return e.textContent="",Promise.resolve(void 0);if(!r)return e.textContent="Cannot display an untrusted SVG. Maybe you need to run the cell?",Promise.resolve(void 0);n.search("<svg[^>]+xmlns=[^>]+svg")<0&&(n=n.replace("<svg",'<svg xmlns="http://www.w3.org/2000/svg"'));const o=new Image;return o.src=`data:image/svg+xml,${encodeURIComponent(n)}`,e.appendChild(o),!0===i&&e.classList.add("jp-mod-unconfined"),Promise.resolve()}function C(t){const{host:e,sanitizer:n,source:i}=t,o=n.sanitize(r.ansiSpan(i),{allowedTags:["span"]}),a=document.createElement("pre");return a.innerHTML=o,e.appendChild(a),Promise.resolve(void 0)}!function(t){function e(t,e,n){let r=t.getAttribute(e)||"";const i=n.isLocal?n.isLocal(r):s.URLExt.isLocal(r);return r&&i?(t.setAttribute(e,""),n.resolveUrl(r).then(t=>n.getDownloadUrl(t)).then(n=>{"data:"!==s.URLExt.parse(n).protocol&&(n+=(/\?/.test(n)?"&":"?")+(new Date).getTime()),t.setAttribute(e,n)}).catch(n=>{t.setAttribute(e,"")})):Promise.resolve(void 0)}function n(t,e,n){let r=t.getAttribute("href")||"";const i=e.isLocal?e.isLocal(r):s.URLExt.isLocal(r);if(!r||!i)return Promise.resolve(void 0);let o=t.hash;if(o){if(o===r)return t.target="_self",Promise.resolve(void 0);r=r.replace(o,"")}return e.resolveUrl(r).then(r=>{const i=decodeURI(r);return n&&n.handleLink(t,i,o),e.getDownloadUrl(r)}).then(e=>{t.href=e+o}).catch(e=>{t.href=""})}t.evalInnerHTMLScriptTags=function(t){let e=Object(A.toArray)(t.getElementsByTagName("script"));for(let t of e){if(!t.parentNode)continue;let e=document.createElement("script"),n=t.attributes;for(let t=0,r=n.length;t<r;++t){let{name:r,value:i}=n[t];e.setAttribute(r,i)}e.textContent=t.textContent,t.parentNode.replaceChild(e,t)}},t.renderMarked=function(t){return r||(r=!0,o.a.setOptions({gfm:!0,sanitize:!1,tables:!0,langPrefix:`cm-s-${a.CodeMirrorEditor.defaultConfig.theme} language-`,highlight:(t,e,n)=>{let r=(t,e)=>(n&&n(t,e),e);return e?(a.Mode.ensure(e).then(n=>{let i=document.createElement("div");if(!n)return console.log(`No CodeMirror mode: ${e}`),r(null,t);try{return a.Mode.run(t,n.mime,i),r(null,i.innerHTML)}catch(n){return console.log(`Failed to highlight ${e} code`,n),r(n,t)}}).catch(n=>(console.log(`No CodeMirror mode: ${e}`),console.log(`Require CodeMirror mode error: ${n}`),r(null,t))),t):r(null,t)}})),new Promise((e,n)=>{o()(t,(t,r)=>{t?n(t):e(r)})})},t.handleDefaults=function(t,e){let n=t.getElementsByTagName("a");for(let t=0;t<n.length;t++){const r=n[t];if(!(r instanceof HTMLAnchorElement))continue;let i=r.href;(e&&e.isLocal?e.isLocal(i):s.URLExt.isLocal(i))?r.target="_self":(r.target="_blank",r.rel="noopener")}let r=t.getElementsByTagName("img");for(let t=0;t<r.length;t++)r[t].alt||(r[t].alt="Image")},t.handleUrls=function(t,r,i){let o=[],a=t.querySelectorAll("*[src]");for(let t=0;t<a.length;t++)o.push(e(a[t],"src",r));let s=t.getElementsByTagName("a");for(let t=0;t<s.length;t++)o.push(n(s[t],r,i));let A=t.getElementsByTagName("link");for(let t=0;t<A.length;t++)o.push(e(A[t],"href",r));return Promise.all(o).then(()=>void 0)},t.headerAnchors=function(t){let e=["h1","h2","h3","h4","h5","h6"];for(let n of e){let e=t.getElementsByTagName(n);for(let t=0;t<e.length;t++){let n=e[t];n.id=n.textContent.replace(/ /g,"-");let r=document.createElement("a");r.target="_self",r.textContent="¶",r.href="#"+n.id,r.classList.add("jp-InternalAnchorLink"),n.appendChild(r)}}};let r=!1;let i=["ansi-black","ansi-red","ansi-green","ansi-yellow","ansi-blue","ansi-magenta","ansi-cyan","ansi-white","ansi-black-intense","ansi-red-intense","ansi-green-intense","ansi-yellow-intense","ansi-blue-intense","ansi-magenta-intense","ansi-cyan-intense","ansi-white-intense"];function l(t,e,n,r,o,a,s){if(t){let A=[],l=[];r&&"number"==typeof e&&0<=e&&e<8&&(e+=8),a&&([e,n]=[n,e]),"number"==typeof e?A.push(i[e]+"-fg"):e.length?l.push(`color: rgb(${e})`):a&&A.push("ansi-default-inverse-fg"),"number"==typeof n?A.push(i[n]+"-bg"):n.length?l.push(`background-color: rgb(${n})`):a&&A.push("ansi-default-inverse-bg"),r&&A.push("ansi-bold"),o&&A.push("ansi-underline"),A.length||l.length?(s.push("<span"),A.length&&s.push(` class="${A.join(" ")}"`),l.length&&s.push(` style="${l.join("; ")}"`),s.push(">"),s.push(t),s.push("</span>")):s.push(t)}}function u(t){let e,n,r,i=t.shift();if(2===i&&t.length>=3){if([e=t.shift(),n=t.shift(),r=t.shift()].some(t=>t<0||255<t))throw new RangeError("Invalid range for RGB colors")}else{if(!(5===i&&t.length>=1))throw new RangeError("Invalid extended color specification");{let i=t.shift();if(i<0)throw new RangeError("Color index must be >= 0");if(i<16)return i;if(i<232)e=(e=Math.floor((i-16)/36))>0?55+40*e:0,n=(n=Math.floor((i-16)%36/6))>0?55+40*n:0,r=(r=(i-16)%6)>0?55+40*r:0;else{if(!(i<256))throw new RangeError("Color index must be < 256");e=n=r=10*(i-232)+8}}}return[e,n,r]}t.ansiSpan=function(t){let e,n=/\x1b\[(.*?)([@-~])/g,r=[],i=[],o=!1,a=!1,s=!1,A=[],g=[],h=0;for(t=c()(t),t+="[m";e=n.exec(t);){if("m"===e[2]){let t=e[1].split(";");for(let e=0;e<t.length;e++){let n=t[e];if(""===n)g.push(0);else{if(-1===n.search(/^\d+$/)){g.length=0;break}g.push(parseInt(n,10))}}}for(l(t.substring(h,e.index),r,i,o,a,s,A),h=n.lastIndex;g.length;){let t=g.shift();switch(t){case 0:r=i=[],o=!1,a=!1,s=!1;break;case 1:case 5:o=!0;break;case 4:a=!0;break;case 7:s=!0;break;case 21:case 22:o=!1;break;case 24:a=!1;break;case 27:s=!1;break;case 30:case 31:case 32:case 33:case 34:case 35:case 36:case 37:r=t-30;break;case 38:try{r=u(g)}catch(t){g.length=0}break;case 39:r=[];break;case 40:case 41:case 42:case 43:case 44:case 45:case 46:case 47:i=t-40;break;case 48:try{i=u(g)}catch(t){g.length=0}break;case 49:i=[];break;case 90:case 91:case 92:case 93:case 94:case 95:case 96:case 97:r=t-90+8;break;case 100:case 101:case 102:case 103:case 104:case 105:case 106:case 107:i=t-100+8}}}return A.join("")}}(r||(r={}))},function(t,e){t.exports=function(t){return t.webpackPolyfill||(t.deprecate=function(){},t.paths=[],t.children||(t.children=[]),Object.defineProperty(t,"loaded",{enumerable:!0,get:function(){return t.l}}),Object.defineProperty(t,"id",{enumerable:!0,get:function(){return t.i}}),t.webpackPolyfill=1),t}},function(t,e,n){var r=n(694),i="object"==typeof self&&self&&self.Object===Object&&self,o=r||i||Function("return this")();t.exports=o},function(t,e,n){"use strict";var r=n(1419);t.exports=function(t){var e=typeof t;if("string"===e){var n=t;if(0===(t=+t)&&r(n))return!1}else if("number"!==e)return!1;return t-t<1}},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r=n(114),i=n(250);function o(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return Object(i.a)()(r.a.apply(void 0,t))}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.create=e.connect=e.Provider=void 0;var r=a(n(1265)),i=a(n(1266)),o=a(n(1268));function a(t){return t&&t.__esModule?t:{default:t}}e.Provider=r.default,e.connect=i.default,e.create=o.default},function(t,e){function n(){return t.exports=n=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t},n.apply(this,arguments)}t.exports=n},function(t,e,n){"use strict";function r(t){setTimeout(function(){throw t},0)}n.d(e,"a",function(){return r})},function(t,e,n){var r=n(35).Buffer,i=n(268).Transform,o=n(210).StringDecoder;function a(t){i.call(this),this.hashMode="string"==typeof t,this.hashMode?this[t]=this._finalOrDigest:this.final=this._finalOrDigest,this._final&&(this.__final=this._final,this._final=null),this._decoder=null,this._encoding=null}n(30)(a,i),a.prototype.update=function(t,e,n){"string"==typeof t&&(t=r.from(t,e));var i=this._update(t);return this.hashMode?this:(n&&(i=this._toString(i,n)),i)},a.prototype.setAutoPadding=function(){},a.prototype.getAuthTag=function(){throw new Error("trying to get auth tag in unsupported state")},a.prototype.setAuthTag=function(){throw new Error("trying to set auth tag in unsupported state")},a.prototype.setAAD=function(){throw new Error("trying to set aad in unsupported state")},a.prototype._transform=function(t,e,n){var r;try{this.hashMode?this._update(t):this.push(this._update(t))}catch(t){r=t}finally{n(r)}},a.prototype._flush=function(t){var e;try{this.push(this.__final())}catch(t){e=t}t(e)},a.prototype._finalOrDigest=function(t){var e=this.__final()||r.alloc(0);return t&&(e=this._toString(e,t,!0)),e},a.prototype._toString=function(t,e,n){if(this._decoder||(this._decoder=new o(e),this._encoding=e),this._encoding!==e)throw new Error("can't switch encodings");var r=this._decoder.write(t);return n&&(r+=this._decoder.end()),r},t.exports=a},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(939),""),e.i(n(940),""),e.i(n(941),""),e.i(n(942),""),e.i(n(943),""),e.i(n(944),""),e.i(n(945),""),e.i(n(946),""),e.i(n(947),""),e.push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n",""])},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(5),i=n(46),o=n(25),a=n(43),s=n(10),A=n(59),l=function(){function t(t){void 0===t&&(t={}),this._disposed=!1,this._parent=null,this._fitPolicy=t.fitPolicy||"set-min-size"}return t.prototype.dispose=function(){this._parent=null,this._disposed=!0,s.Signal.clearData(this),a.AttachedProperty.clearData(this)},Object.defineProperty(t.prototype,"isDisposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"parent",{get:function(){return this._parent},set:function(t){if(this._parent!==t){if(this._parent)throw new Error("Cannot change parent widget.");if(t.layout!==this)throw new Error("Invalid parent widget.");this._parent=t,this.init()}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fitPolicy",{get:function(){return this._fitPolicy},set:function(t){if(this._fitPolicy!==t&&(this._fitPolicy=t,this._parent)){var e=this._parent.node.style;e.minWidth="",e.minHeight="",e.maxWidth="",e.maxHeight="",this._parent.fit()}},enumerable:!0,configurable:!0}),t.prototype.processParentMessage=function(t){switch(t.type){case"resize":this.onResize(t);break;case"update-request":this.onUpdateRequest(t);break;case"fit-request":this.onFitRequest(t);break;case"before-show":this.onBeforeShow(t);break;case"after-show":this.onAfterShow(t);break;case"before-hide":this.onBeforeHide(t);break;case"after-hide":this.onAfterHide(t);break;case"before-attach":this.onBeforeAttach(t);break;case"after-attach":this.onAfterAttach(t);break;case"before-detach":this.onBeforeDetach(t);break;case"after-detach":this.onAfterDetach(t);break;case"child-removed":this.onChildRemoved(t);break;case"child-shown":this.onChildShown(t);break;case"child-hidden":this.onChildHidden(t)}},t.prototype.init=function(){var t=this;r.each(this,function(e){e.parent=t.parent})},t.prototype.onResize=function(t){r.each(this,function(t){o.MessageLoop.sendMessage(t,A.Widget.ResizeMessage.UnknownSize)})},t.prototype.onUpdateRequest=function(t){r.each(this,function(t){o.MessageLoop.sendMessage(t,A.Widget.ResizeMessage.UnknownSize)})},t.prototype.onBeforeAttach=function(t){r.each(this,function(e){o.MessageLoop.sendMessage(e,t)})},t.prototype.onAfterAttach=function(t){r.each(this,function(e){o.MessageLoop.sendMessage(e,t)})},t.prototype.onBeforeDetach=function(t){r.each(this,function(e){o.MessageLoop.sendMessage(e,t)})},t.prototype.onAfterDetach=function(t){r.each(this,function(e){o.MessageLoop.sendMessage(e,t)})},t.prototype.onBeforeShow=function(t){r.each(this,function(e){e.isHidden||o.MessageLoop.sendMessage(e,t)})},t.prototype.onAfterShow=function(t){r.each(this,function(e){e.isHidden||o.MessageLoop.sendMessage(e,t)})},t.prototype.onBeforeHide=function(t){r.each(this,function(e){e.isHidden||o.MessageLoop.sendMessage(e,t)})},t.prototype.onAfterHide=function(t){r.each(this,function(e){e.isHidden||o.MessageLoop.sendMessage(e,t)})},t.prototype.onChildRemoved=function(t){this.removeWidget(t.child)},t.prototype.onFitRequest=function(t){},t.prototype.onChildShown=function(t){},t.prototype.onChildHidden=function(t){},t}();e.Layout=l,function(t){t.getHorizontalAlignment=function(t){return c.horizontalAlignmentProperty.get(t)},t.setHorizontalAlignment=function(t,e){c.horizontalAlignmentProperty.set(t,e)},t.getVerticalAlignment=function(t){return c.verticalAlignmentProperty.get(t)},t.setVerticalAlignment=function(t,e){c.verticalAlignmentProperty.set(t,e)}}(l=e.Layout||(e.Layout={})),e.Layout=l;var c,u=function(){function t(t){this._top=NaN,this._left=NaN,this._width=NaN,this._height=NaN,this._minWidth=0,this._minHeight=0,this._maxWidth=1/0,this._maxHeight=1/0,this._disposed=!1,this.widget=t,this.widget.node.style.position="absolute"}return t.prototype.dispose=function(){if(!this._disposed){this._disposed=!0;var t=this.widget.node.style;t.position="",t.top="",t.left="",t.width="",t.height=""}},Object.defineProperty(t.prototype,"minWidth",{get:function(){return this._minWidth},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"minHeight",{get:function(){return this._minHeight},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"maxWidth",{get:function(){return this._maxWidth},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"maxHeight",{get:function(){return this._maxHeight},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isDisposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isHidden",{get:function(){return this.widget.isHidden},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isVisible",{get:function(){return this.widget.isVisible},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isAttached",{get:function(){return this.widget.isAttached},enumerable:!0,configurable:!0}),t.prototype.fit=function(){var t=i.ElementExt.sizeLimits(this.widget.node);this._minWidth=t.minWidth,this._minHeight=t.minHeight,this._maxWidth=t.maxWidth,this._maxHeight=t.maxHeight},t.prototype.update=function(t,e,n,r){var i=Math.max(this._minWidth,Math.min(n,this._maxWidth)),a=Math.max(this._minHeight,Math.min(r,this._maxHeight));if(i<n)switch(l.getHorizontalAlignment(this.widget)){case"left":break;case"center":t+=(n-i)/2;break;case"right":t+=n-i;break;default:throw"unreachable"}if(a<r)switch(l.getVerticalAlignment(this.widget)){case"top":break;case"center":e+=(r-a)/2;break;case"bottom":e+=r-a;break;default:throw"unreachable"}var s=!1,c=this.widget.node.style;if(this._top!==e&&(this._top=e,c.top=e+"px"),this._left!==t&&(this._left=t,c.left=t+"px"),this._width!==i&&(s=!0,this._width=i,c.width=i+"px"),this._height!==a&&(s=!0,this._height=a,c.height=a+"px"),s){var u=new A.Widget.ResizeMessage(i,a);o.MessageLoop.sendMessage(this.widget,u)}},t}();e.LayoutItem=u,function(t){function e(t){t.parent&&t.parent.layout&&t.parent.update()}t.horizontalAlignmentProperty=new a.AttachedProperty({name:"horizontalAlignment",create:function(){return"center"},changed:e}),t.verticalAlignmentProperty=new a.AttachedProperty({name:"verticalAlignment",create:function(){return"top"},changed:e})}(c||(c={}))},function(t,e,n){(function(t,r){var i;
/**
 * @license
 * Lodash <https://lodash.com/>
 * Copyright JS Foundation and other contributors <https://js.foundation/>
 * Released under MIT license <https://lodash.com/license>
 * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
 * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
 */(function(){var o,a=200,s="Unsupported core-js use. Try https://npms.io/search?q=ponyfill.",A="Expected a function",l="__lodash_hash_undefined__",c=500,u="__lodash_placeholder__",g=1,h=2,d=4,f=1,p=2,I=1,C=2,m=4,y=8,E=16,v=32,B=64,b=128,Q=256,w=512,x=30,_="...",D=800,M=16,k=1,S=2,L=1/0,R=9007199254740991,N=17976931348623157e292,F=NaN,T=4294967295,U=T-1,O=T>>>1,G=[["ary",b],["bind",I],["bindKey",C],["curry",y],["curryRight",E],["flip",w],["partial",v],["partialRight",B],["rearg",Q]],H="[object Arguments]",j="[object Array]",Y="[object AsyncFunction]",P="[object Boolean]",K="[object Date]",z="[object DOMException]",J="[object Error]",q="[object Function]",V="[object GeneratorFunction]",W="[object Map]",X="[object Number]",Z="[object Null]",$="[object Object]",tt="[object Proxy]",et="[object RegExp]",nt="[object Set]",rt="[object String]",it="[object Symbol]",ot="[object Undefined]",at="[object WeakMap]",st="[object WeakSet]",At="[object ArrayBuffer]",lt="[object DataView]",ct="[object Float32Array]",ut="[object Float64Array]",gt="[object Int8Array]",ht="[object Int16Array]",dt="[object Int32Array]",ft="[object Uint8Array]",pt="[object Uint8ClampedArray]",It="[object Uint16Array]",Ct="[object Uint32Array]",mt=/\b__p \+= '';/g,yt=/\b(__p \+=) '' \+/g,Et=/(__e\(.*?\)|\b__t\)) \+\n'';/g,vt=/&(?:amp|lt|gt|quot|#39);/g,Bt=/[&<>"']/g,bt=RegExp(vt.source),Qt=RegExp(Bt.source),wt=/<%-([\s\S]+?)%>/g,xt=/<%([\s\S]+?)%>/g,_t=/<%=([\s\S]+?)%>/g,Dt=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,Mt=/^\w*$/,kt=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,St=/[\\^$.*+?()[\]{}|]/g,Lt=RegExp(St.source),Rt=/^\s+|\s+$/g,Nt=/^\s+/,Ft=/\s+$/,Tt=/\{(?:\n\/\* \[wrapped with .+\] \*\/)?\n?/,Ut=/\{\n\/\* \[wrapped with (.+)\] \*/,Ot=/,? & /,Gt=/[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g,Ht=/\\(\\)?/g,jt=/\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g,Yt=/\w*$/,Pt=/^[-+]0x[0-9a-f]+$/i,Kt=/^0b[01]+$/i,zt=/^\[object .+?Constructor\]$/,Jt=/^0o[0-7]+$/i,qt=/^(?:0|[1-9]\d*)$/,Vt=/[\xc0-\xd6\xd8-\xf6\xf8-\xff\u0100-\u017f]/g,Wt=/($^)/,Xt=/['\n\r\u2028\u2029\\]/g,Zt="\\u0300-\\u036f\\ufe20-\\ufe2f\\u20d0-\\u20ff",$t="\\xac\\xb1\\xd7\\xf7\\x00-\\x2f\\x3a-\\x40\\x5b-\\x60\\x7b-\\xbf\\u2000-\\u206f \\t\\x0b\\f\\xa0\\ufeff\\n\\r\\u2028\\u2029\\u1680\\u180e\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200a\\u202f\\u205f\\u3000",te="[\\ud800-\\udfff]",ee="["+$t+"]",ne="["+Zt+"]",re="\\d+",ie="[\\u2700-\\u27bf]",oe="[a-z\\xdf-\\xf6\\xf8-\\xff]",ae="[^\\ud800-\\udfff"+$t+re+"\\u2700-\\u27bfa-z\\xdf-\\xf6\\xf8-\\xffA-Z\\xc0-\\xd6\\xd8-\\xde]",se="\\ud83c[\\udffb-\\udfff]",Ae="[^\\ud800-\\udfff]",le="(?:\\ud83c[\\udde6-\\uddff]){2}",ce="[\\ud800-\\udbff][\\udc00-\\udfff]",ue="[A-Z\\xc0-\\xd6\\xd8-\\xde]",ge="(?:"+oe+"|"+ae+")",he="(?:"+ue+"|"+ae+")",de="(?:"+ne+"|"+se+")"+"?",fe="[\\ufe0e\\ufe0f]?"+de+("(?:\\u200d(?:"+[Ae,le,ce].join("|")+")[\\ufe0e\\ufe0f]?"+de+")*"),pe="(?:"+[ie,le,ce].join("|")+")"+fe,Ie="(?:"+[Ae+ne+"?",ne,le,ce,te].join("|")+")",Ce=RegExp("['’]","g"),me=RegExp(ne,"g"),ye=RegExp(se+"(?="+se+")|"+Ie+fe,"g"),Ee=RegExp([ue+"?"+oe+"+(?:['’](?:d|ll|m|re|s|t|ve))?(?="+[ee,ue,"$"].join("|")+")",he+"+(?:['’](?:D|LL|M|RE|S|T|VE))?(?="+[ee,ue+ge,"$"].join("|")+")",ue+"?"+ge+"+(?:['’](?:d|ll|m|re|s|t|ve))?",ue+"+(?:['’](?:D|LL|M|RE|S|T|VE))?","\\d*(?:1ST|2ND|3RD|(?![123])\\dTH)(?=\\b|[a-z_])","\\d*(?:1st|2nd|3rd|(?![123])\\dth)(?=\\b|[A-Z_])",re,pe].join("|"),"g"),ve=RegExp("[\\u200d\\ud800-\\udfff"+Zt+"\\ufe0e\\ufe0f]"),Be=/[a-z][A-Z]|[A-Z]{2}[a-z]|[0-9][a-zA-Z]|[a-zA-Z][0-9]|[^a-zA-Z0-9 ]/,be=["Array","Buffer","DataView","Date","Error","Float32Array","Float64Array","Function","Int8Array","Int16Array","Int32Array","Map","Math","Object","Promise","RegExp","Set","String","Symbol","TypeError","Uint8Array","Uint8ClampedArray","Uint16Array","Uint32Array","WeakMap","_","clearTimeout","isFinite","parseInt","setTimeout"],Qe=-1,we={};we[ct]=we[ut]=we[gt]=we[ht]=we[dt]=we[ft]=we[pt]=we[It]=we[Ct]=!0,we[H]=we[j]=we[At]=we[P]=we[lt]=we[K]=we[J]=we[q]=we[W]=we[X]=we[$]=we[et]=we[nt]=we[rt]=we[at]=!1;var xe={};xe[H]=xe[j]=xe[At]=xe[lt]=xe[P]=xe[K]=xe[ct]=xe[ut]=xe[gt]=xe[ht]=xe[dt]=xe[W]=xe[X]=xe[$]=xe[et]=xe[nt]=xe[rt]=xe[it]=xe[ft]=xe[pt]=xe[It]=xe[Ct]=!0,xe[J]=xe[q]=xe[at]=!1;var _e={"\\":"\\","'":"'","\n":"n","\r":"r","\u2028":"u2028","\u2029":"u2029"},De=parseFloat,Me=parseInt,ke="object"==typeof t&&t&&t.Object===Object&&t,Se="object"==typeof self&&self&&self.Object===Object&&self,Le=ke||Se||Function("return this")(),Re=e&&!e.nodeType&&e,Ne=Re&&"object"==typeof r&&r&&!r.nodeType&&r,Fe=Ne&&Ne.exports===Re,Te=Fe&&ke.process,Ue=function(){try{var t=Ne&&Ne.require&&Ne.require("util").types;return t||Te&&Te.binding&&Te.binding("util")}catch(t){}}(),Oe=Ue&&Ue.isArrayBuffer,Ge=Ue&&Ue.isDate,He=Ue&&Ue.isMap,je=Ue&&Ue.isRegExp,Ye=Ue&&Ue.isSet,Pe=Ue&&Ue.isTypedArray;function Ke(t,e,n){switch(n.length){case 0:return t.call(e);case 1:return t.call(e,n[0]);case 2:return t.call(e,n[0],n[1]);case 3:return t.call(e,n[0],n[1],n[2])}return t.apply(e,n)}function ze(t,e,n,r){for(var i=-1,o=null==t?0:t.length;++i<o;){var a=t[i];e(r,a,n(a),t)}return r}function Je(t,e){for(var n=-1,r=null==t?0:t.length;++n<r&&!1!==e(t[n],n,t););return t}function qe(t,e){for(var n=null==t?0:t.length;n--&&!1!==e(t[n],n,t););return t}function Ve(t,e){for(var n=-1,r=null==t?0:t.length;++n<r;)if(!e(t[n],n,t))return!1;return!0}function We(t,e){for(var n=-1,r=null==t?0:t.length,i=0,o=[];++n<r;){var a=t[n];e(a,n,t)&&(o[i++]=a)}return o}function Xe(t,e){return!!(null==t?0:t.length)&&An(t,e,0)>-1}function Ze(t,e,n){for(var r=-1,i=null==t?0:t.length;++r<i;)if(n(e,t[r]))return!0;return!1}function $e(t,e){for(var n=-1,r=null==t?0:t.length,i=Array(r);++n<r;)i[n]=e(t[n],n,t);return i}function tn(t,e){for(var n=-1,r=e.length,i=t.length;++n<r;)t[i+n]=e[n];return t}function en(t,e,n,r){var i=-1,o=null==t?0:t.length;for(r&&o&&(n=t[++i]);++i<o;)n=e(n,t[i],i,t);return n}function nn(t,e,n,r){var i=null==t?0:t.length;for(r&&i&&(n=t[--i]);i--;)n=e(n,t[i],i,t);return n}function rn(t,e){for(var n=-1,r=null==t?0:t.length;++n<r;)if(e(t[n],n,t))return!0;return!1}var on=gn("length");function an(t,e,n){var r;return n(t,function(t,n,i){if(e(t,n,i))return r=n,!1}),r}function sn(t,e,n,r){for(var i=t.length,o=n+(r?1:-1);r?o--:++o<i;)if(e(t[o],o,t))return o;return-1}function An(t,e,n){return e==e?function(t,e,n){var r=n-1,i=t.length;for(;++r<i;)if(t[r]===e)return r;return-1}(t,e,n):sn(t,cn,n)}function ln(t,e,n,r){for(var i=n-1,o=t.length;++i<o;)if(r(t[i],e))return i;return-1}function cn(t){return t!=t}function un(t,e){var n=null==t?0:t.length;return n?fn(t,e)/n:F}function gn(t){return function(e){return null==e?o:e[t]}}function hn(t){return function(e){return null==t?o:t[e]}}function dn(t,e,n,r,i){return i(t,function(t,i,o){n=r?(r=!1,t):e(n,t,i,o)}),n}function fn(t,e){for(var n,r=-1,i=t.length;++r<i;){var a=e(t[r]);a!==o&&(n=n===o?a:n+a)}return n}function pn(t,e){for(var n=-1,r=Array(t);++n<t;)r[n]=e(n);return r}function In(t){return function(e){return t(e)}}function Cn(t,e){return $e(e,function(e){return t[e]})}function mn(t,e){return t.has(e)}function yn(t,e){for(var n=-1,r=t.length;++n<r&&An(e,t[n],0)>-1;);return n}function En(t,e){for(var n=t.length;n--&&An(e,t[n],0)>-1;);return n}var vn=hn({"À":"A","Á":"A","Â":"A","Ã":"A","Ä":"A","Å":"A","à":"a","á":"a","â":"a","ã":"a","ä":"a","å":"a","Ç":"C","ç":"c","Ð":"D","ð":"d","È":"E","É":"E","Ê":"E","Ë":"E","è":"e","é":"e","ê":"e","ë":"e","Ì":"I","Í":"I","Î":"I","Ï":"I","ì":"i","í":"i","î":"i","ï":"i","Ñ":"N","ñ":"n","Ò":"O","Ó":"O","Ô":"O","Õ":"O","Ö":"O","Ø":"O","ò":"o","ó":"o","ô":"o","õ":"o","ö":"o","ø":"o","Ù":"U","Ú":"U","Û":"U","Ü":"U","ù":"u","ú":"u","û":"u","ü":"u","Ý":"Y","ý":"y","ÿ":"y","Æ":"Ae","æ":"ae","Þ":"Th","þ":"th","ß":"ss","Ā":"A","Ă":"A","Ą":"A","ā":"a","ă":"a","ą":"a","Ć":"C","Ĉ":"C","Ċ":"C","Č":"C","ć":"c","ĉ":"c","ċ":"c","č":"c","Ď":"D","Đ":"D","ď":"d","đ":"d","Ē":"E","Ĕ":"E","Ė":"E","Ę":"E","Ě":"E","ē":"e","ĕ":"e","ė":"e","ę":"e","ě":"e","Ĝ":"G","Ğ":"G","Ġ":"G","Ģ":"G","ĝ":"g","ğ":"g","ġ":"g","ģ":"g","Ĥ":"H","Ħ":"H","ĥ":"h","ħ":"h","Ĩ":"I","Ī":"I","Ĭ":"I","Į":"I","İ":"I","ĩ":"i","ī":"i","ĭ":"i","į":"i","ı":"i","Ĵ":"J","ĵ":"j","Ķ":"K","ķ":"k","ĸ":"k","Ĺ":"L","Ļ":"L","Ľ":"L","Ŀ":"L","Ł":"L","ĺ":"l","ļ":"l","ľ":"l","ŀ":"l","ł":"l","Ń":"N","Ņ":"N","Ň":"N","Ŋ":"N","ń":"n","ņ":"n","ň":"n","ŋ":"n","Ō":"O","Ŏ":"O","Ő":"O","ō":"o","ŏ":"o","ő":"o","Ŕ":"R","Ŗ":"R","Ř":"R","ŕ":"r","ŗ":"r","ř":"r","Ś":"S","Ŝ":"S","Ş":"S","Š":"S","ś":"s","ŝ":"s","ş":"s","š":"s","Ţ":"T","Ť":"T","Ŧ":"T","ţ":"t","ť":"t","ŧ":"t","Ũ":"U","Ū":"U","Ŭ":"U","Ů":"U","Ű":"U","Ų":"U","ũ":"u","ū":"u","ŭ":"u","ů":"u","ű":"u","ų":"u","Ŵ":"W","ŵ":"w","Ŷ":"Y","ŷ":"y","Ÿ":"Y","Ź":"Z","Ż":"Z","Ž":"Z","ź":"z","ż":"z","ž":"z","Ĳ":"IJ","ĳ":"ij","Œ":"Oe","œ":"oe","ŉ":"'n","ſ":"s"}),Bn=hn({"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"});function bn(t){return"\\"+_e[t]}function Qn(t){return ve.test(t)}function wn(t){var e=-1,n=Array(t.size);return t.forEach(function(t,r){n[++e]=[r,t]}),n}function xn(t,e){return function(n){return t(e(n))}}function _n(t,e){for(var n=-1,r=t.length,i=0,o=[];++n<r;){var a=t[n];a!==e&&a!==u||(t[n]=u,o[i++]=n)}return o}function Dn(t){var e=-1,n=Array(t.size);return t.forEach(function(t){n[++e]=t}),n}function Mn(t){var e=-1,n=Array(t.size);return t.forEach(function(t){n[++e]=[t,t]}),n}function kn(t){return Qn(t)?function(t){var e=ye.lastIndex=0;for(;ye.test(t);)++e;return e}(t):on(t)}function Sn(t){return Qn(t)?function(t){return t.match(ye)||[]}(t):function(t){return t.split("")}(t)}var Ln=hn({"&amp;":"&","&lt;":"<","&gt;":">","&quot;":'"',"&#39;":"'"});var Rn=function t(e){var n,r=(e=null==e?Le:Rn.defaults(Le.Object(),e,Rn.pick(Le,be))).Array,i=e.Date,Zt=e.Error,$t=e.Function,te=e.Math,ee=e.Object,ne=e.RegExp,re=e.String,ie=e.TypeError,oe=r.prototype,ae=$t.prototype,se=ee.prototype,Ae=e["__core-js_shared__"],le=ae.toString,ce=se.hasOwnProperty,ue=0,ge=(n=/[^.]+$/.exec(Ae&&Ae.keys&&Ae.keys.IE_PROTO||""))?"Symbol(src)_1."+n:"",he=se.toString,de=le.call(ee),fe=Le._,pe=ne("^"+le.call(ce).replace(St,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),Ie=Fe?e.Buffer:o,ye=e.Symbol,ve=e.Uint8Array,_e=Ie?Ie.allocUnsafe:o,ke=xn(ee.getPrototypeOf,ee),Se=ee.create,Re=se.propertyIsEnumerable,Ne=oe.splice,Te=ye?ye.isConcatSpreadable:o,Ue=ye?ye.iterator:o,on=ye?ye.toStringTag:o,hn=function(){try{var t=Oo(ee,"defineProperty");return t({},"",{}),t}catch(t){}}(),Nn=e.clearTimeout!==Le.clearTimeout&&e.clearTimeout,Fn=i&&i.now!==Le.Date.now&&i.now,Tn=e.setTimeout!==Le.setTimeout&&e.setTimeout,Un=te.ceil,On=te.floor,Gn=ee.getOwnPropertySymbols,Hn=Ie?Ie.isBuffer:o,jn=e.isFinite,Yn=oe.join,Pn=xn(ee.keys,ee),Kn=te.max,zn=te.min,Jn=i.now,qn=e.parseInt,Vn=te.random,Wn=oe.reverse,Xn=Oo(e,"DataView"),Zn=Oo(e,"Map"),$n=Oo(e,"Promise"),tr=Oo(e,"Set"),er=Oo(e,"WeakMap"),nr=Oo(ee,"create"),rr=er&&new er,ir={},or=ua(Xn),ar=ua(Zn),sr=ua($n),Ar=ua(tr),lr=ua(er),cr=ye?ye.prototype:o,ur=cr?cr.valueOf:o,gr=cr?cr.toString:o;function hr(t){if(_s(t)&&!Is(t)&&!(t instanceof Ir)){if(t instanceof pr)return t;if(ce.call(t,"__wrapped__"))return ga(t)}return new pr(t)}var dr=function(){function t(){}return function(e){if(!xs(e))return{};if(Se)return Se(e);t.prototype=e;var n=new t;return t.prototype=o,n}}();function fr(){}function pr(t,e){this.__wrapped__=t,this.__actions__=[],this.__chain__=!!e,this.__index__=0,this.__values__=o}function Ir(t){this.__wrapped__=t,this.__actions__=[],this.__dir__=1,this.__filtered__=!1,this.__iteratees__=[],this.__takeCount__=T,this.__views__=[]}function Cr(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function mr(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function yr(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function Er(t){var e=-1,n=null==t?0:t.length;for(this.__data__=new yr;++e<n;)this.add(t[e])}function vr(t){var e=this.__data__=new mr(t);this.size=e.size}function Br(t,e){var n=Is(t),r=!n&&ps(t),i=!n&&!r&&Es(t),o=!n&&!r&&!i&&Fs(t),a=n||r||i||o,s=a?pn(t.length,re):[],A=s.length;for(var l in t)!e&&!ce.call(t,l)||a&&("length"==l||i&&("offset"==l||"parent"==l)||o&&("buffer"==l||"byteLength"==l||"byteOffset"==l)||zo(l,A))||s.push(l);return s}function br(t){var e=t.length;return e?t[Ei(0,e-1)]:o}function Qr(t,e){return Aa(no(t),Rr(e,0,t.length))}function wr(t){return Aa(no(t))}function xr(t,e,n){(n===o||hs(t[e],n))&&(n!==o||e in t)||Sr(t,e,n)}function _r(t,e,n){var r=t[e];ce.call(t,e)&&hs(r,n)&&(n!==o||e in t)||Sr(t,e,n)}function Dr(t,e){for(var n=t.length;n--;)if(hs(t[n][0],e))return n;return-1}function Mr(t,e,n,r){return Or(t,function(t,i,o){e(r,t,n(t),o)}),r}function kr(t,e){return t&&ro(e,iA(e),t)}function Sr(t,e,n){"__proto__"==e&&hn?hn(t,e,{configurable:!0,enumerable:!0,value:n,writable:!0}):t[e]=n}function Lr(t,e){for(var n=-1,i=e.length,a=r(i),s=null==t;++n<i;)a[n]=s?o:$s(t,e[n]);return a}function Rr(t,e,n){return t==t&&(n!==o&&(t=t<=n?t:n),e!==o&&(t=t>=e?t:e)),t}function Nr(t,e,n,r,i,a){var s,A=e&g,l=e&h,c=e&d;if(n&&(s=i?n(t,r,i,a):n(t)),s!==o)return s;if(!xs(t))return t;var u=Is(t);if(u){if(s=function(t){var e=t.length,n=new t.constructor(e);return e&&"string"==typeof t[0]&&ce.call(t,"index")&&(n.index=t.index,n.input=t.input),n}(t),!A)return no(t,s)}else{var f=jo(t),p=f==q||f==V;if(Es(t))return Wi(t,A);if(f==$||f==H||p&&!i){if(s=l||p?{}:Po(t),!A)return l?function(t,e){return ro(t,Ho(t),e)}(t,function(t,e){return t&&ro(e,oA(e),t)}(s,t)):function(t,e){return ro(t,Go(t),e)}(t,kr(s,t))}else{if(!xe[f])return i?t:{};s=function(t,e,n){var r,i=t.constructor;switch(e){case At:return Xi(t);case P:case K:return new i(+t);case lt:return function(t,e){var n=e?Xi(t.buffer):t.buffer;return new t.constructor(n,t.byteOffset,t.byteLength)}(t,n);case ct:case ut:case gt:case ht:case dt:case ft:case pt:case It:case Ct:return Zi(t,n);case W:return new i;case X:case rt:return new i(t);case et:return function(t){var e=new t.constructor(t.source,Yt.exec(t));return e.lastIndex=t.lastIndex,e}(t);case nt:return new i;case it:return r=t,ur?ee(ur.call(r)):{}}}(t,f,A)}}a||(a=new vr);var I=a.get(t);if(I)return I;if(a.set(t,s),Ls(t))return t.forEach(function(r){s.add(Nr(r,e,n,r,t,a))}),s;if(Ds(t))return t.forEach(function(r,i){s.set(i,Nr(r,e,n,i,t,a))}),s;var C=u?o:(c?l?So:ko:l?oA:iA)(t);return Je(C||t,function(r,i){C&&(r=t[i=r]),_r(s,i,Nr(r,e,n,i,t,a))}),s}function Fr(t,e,n){var r=n.length;if(null==t)return!r;for(t=ee(t);r--;){var i=n[r],a=e[i],s=t[i];if(s===o&&!(i in t)||!a(s))return!1}return!0}function Tr(t,e,n){if("function"!=typeof t)throw new ie(A);return ia(function(){t.apply(o,n)},e)}function Ur(t,e,n,r){var i=-1,o=Xe,s=!0,A=t.length,l=[],c=e.length;if(!A)return l;n&&(e=$e(e,In(n))),r?(o=Ze,s=!1):e.length>=a&&(o=mn,s=!1,e=new Er(e));t:for(;++i<A;){var u=t[i],g=null==n?u:n(u);if(u=r||0!==u?u:0,s&&g==g){for(var h=c;h--;)if(e[h]===g)continue t;l.push(u)}else o(e,g,r)||l.push(u)}return l}hr.templateSettings={escape:wt,evaluate:xt,interpolate:_t,variable:"",imports:{_:hr}},hr.prototype=fr.prototype,hr.prototype.constructor=hr,pr.prototype=dr(fr.prototype),pr.prototype.constructor=pr,Ir.prototype=dr(fr.prototype),Ir.prototype.constructor=Ir,Cr.prototype.clear=function(){this.__data__=nr?nr(null):{},this.size=0},Cr.prototype.delete=function(t){var e=this.has(t)&&delete this.__data__[t];return this.size-=e?1:0,e},Cr.prototype.get=function(t){var e=this.__data__;if(nr){var n=e[t];return n===l?o:n}return ce.call(e,t)?e[t]:o},Cr.prototype.has=function(t){var e=this.__data__;return nr?e[t]!==o:ce.call(e,t)},Cr.prototype.set=function(t,e){var n=this.__data__;return this.size+=this.has(t)?0:1,n[t]=nr&&e===o?l:e,this},mr.prototype.clear=function(){this.__data__=[],this.size=0},mr.prototype.delete=function(t){var e=this.__data__,n=Dr(e,t);return!(n<0||(n==e.length-1?e.pop():Ne.call(e,n,1),--this.size,0))},mr.prototype.get=function(t){var e=this.__data__,n=Dr(e,t);return n<0?o:e[n][1]},mr.prototype.has=function(t){return Dr(this.__data__,t)>-1},mr.prototype.set=function(t,e){var n=this.__data__,r=Dr(n,t);return r<0?(++this.size,n.push([t,e])):n[r][1]=e,this},yr.prototype.clear=function(){this.size=0,this.__data__={hash:new Cr,map:new(Zn||mr),string:new Cr}},yr.prototype.delete=function(t){var e=To(this,t).delete(t);return this.size-=e?1:0,e},yr.prototype.get=function(t){return To(this,t).get(t)},yr.prototype.has=function(t){return To(this,t).has(t)},yr.prototype.set=function(t,e){var n=To(this,t),r=n.size;return n.set(t,e),this.size+=n.size==r?0:1,this},Er.prototype.add=Er.prototype.push=function(t){return this.__data__.set(t,l),this},Er.prototype.has=function(t){return this.__data__.has(t)},vr.prototype.clear=function(){this.__data__=new mr,this.size=0},vr.prototype.delete=function(t){var e=this.__data__,n=e.delete(t);return this.size=e.size,n},vr.prototype.get=function(t){return this.__data__.get(t)},vr.prototype.has=function(t){return this.__data__.has(t)},vr.prototype.set=function(t,e){var n=this.__data__;if(n instanceof mr){var r=n.__data__;if(!Zn||r.length<a-1)return r.push([t,e]),this.size=++n.size,this;n=this.__data__=new yr(r)}return n.set(t,e),this.size=n.size,this};var Or=ao(Jr),Gr=ao(qr,!0);function Hr(t,e){var n=!0;return Or(t,function(t,r,i){return n=!!e(t,r,i)}),n}function jr(t,e,n){for(var r=-1,i=t.length;++r<i;){var a=t[r],s=e(a);if(null!=s&&(A===o?s==s&&!Ns(s):n(s,A)))var A=s,l=a}return l}function Yr(t,e){var n=[];return Or(t,function(t,r,i){e(t,r,i)&&n.push(t)}),n}function Pr(t,e,n,r,i){var o=-1,a=t.length;for(n||(n=Ko),i||(i=[]);++o<a;){var s=t[o];e>0&&n(s)?e>1?Pr(s,e-1,n,r,i):tn(i,s):r||(i[i.length]=s)}return i}var Kr=so(),zr=so(!0);function Jr(t,e){return t&&Kr(t,e,iA)}function qr(t,e){return t&&zr(t,e,iA)}function Vr(t,e){return We(e,function(e){return bs(t[e])})}function Wr(t,e){for(var n=0,r=(e=zi(e,t)).length;null!=t&&n<r;)t=t[ca(e[n++])];return n&&n==r?t:o}function Xr(t,e,n){var r=e(t);return Is(t)?r:tn(r,n(t))}function Zr(t){return null==t?t===o?ot:Z:on&&on in ee(t)?function(t){var e=ce.call(t,on),n=t[on];try{t[on]=o;var r=!0}catch(t){}var i=he.call(t);return r&&(e?t[on]=n:delete t[on]),i}(t):function(t){return he.call(t)}(t)}function $r(t,e){return t>e}function ti(t,e){return null!=t&&ce.call(t,e)}function ei(t,e){return null!=t&&e in ee(t)}function ni(t,e,n){for(var i=n?Ze:Xe,a=t[0].length,s=t.length,A=s,l=r(s),c=1/0,u=[];A--;){var g=t[A];A&&e&&(g=$e(g,In(e))),c=zn(g.length,c),l[A]=!n&&(e||a>=120&&g.length>=120)?new Er(A&&g):o}g=t[0];var h=-1,d=l[0];t:for(;++h<a&&u.length<c;){var f=g[h],p=e?e(f):f;if(f=n||0!==f?f:0,!(d?mn(d,p):i(u,p,n))){for(A=s;--A;){var I=l[A];if(!(I?mn(I,p):i(t[A],p,n)))continue t}d&&d.push(p),u.push(f)}}return u}function ri(t,e,n){var r=null==(t=ea(t,e=zi(e,t)))?t:t[ca(Ba(e))];return null==r?o:Ke(r,t,n)}function ii(t){return _s(t)&&Zr(t)==H}function oi(t,e,n,r,i){return t===e||(null==t||null==e||!_s(t)&&!_s(e)?t!=t&&e!=e:function(t,e,n,r,i,a){var s=Is(t),A=Is(e),l=s?j:jo(t),c=A?j:jo(e),u=(l=l==H?$:l)==$,g=(c=c==H?$:c)==$,h=l==c;if(h&&Es(t)){if(!Es(e))return!1;s=!0,u=!1}if(h&&!u)return a||(a=new vr),s||Fs(t)?Do(t,e,n,r,i,a):function(t,e,n,r,i,o,a){switch(n){case lt:if(t.byteLength!=e.byteLength||t.byteOffset!=e.byteOffset)return!1;t=t.buffer,e=e.buffer;case At:return!(t.byteLength!=e.byteLength||!o(new ve(t),new ve(e)));case P:case K:case X:return hs(+t,+e);case J:return t.name==e.name&&t.message==e.message;case et:case rt:return t==e+"";case W:var s=wn;case nt:var A=r&f;if(s||(s=Dn),t.size!=e.size&&!A)return!1;var l=a.get(t);if(l)return l==e;r|=p,a.set(t,e);var c=Do(s(t),s(e),r,i,o,a);return a.delete(t),c;case it:if(ur)return ur.call(t)==ur.call(e)}return!1}(t,e,l,n,r,i,a);if(!(n&f)){var d=u&&ce.call(t,"__wrapped__"),I=g&&ce.call(e,"__wrapped__");if(d||I){var C=d?t.value():t,m=I?e.value():e;return a||(a=new vr),i(C,m,n,r,a)}}return!!h&&(a||(a=new vr),function(t,e,n,r,i,a){var s=n&f,A=ko(t),l=A.length,c=ko(e).length;if(l!=c&&!s)return!1;for(var u=l;u--;){var g=A[u];if(!(s?g in e:ce.call(e,g)))return!1}var h=a.get(t);if(h&&a.get(e))return h==e;var d=!0;a.set(t,e),a.set(e,t);for(var p=s;++u<l;){g=A[u];var I=t[g],C=e[g];if(r)var m=s?r(C,I,g,e,t,a):r(I,C,g,t,e,a);if(!(m===o?I===C||i(I,C,n,r,a):m)){d=!1;break}p||(p="constructor"==g)}if(d&&!p){var y=t.constructor,E=e.constructor;y!=E&&"constructor"in t&&"constructor"in e&&!("function"==typeof y&&y instanceof y&&"function"==typeof E&&E instanceof E)&&(d=!1)}return a.delete(t),a.delete(e),d}(t,e,n,r,i,a))}(t,e,n,r,oi,i))}function ai(t,e,n,r){var i=n.length,a=i,s=!r;if(null==t)return!a;for(t=ee(t);i--;){var A=n[i];if(s&&A[2]?A[1]!==t[A[0]]:!(A[0]in t))return!1}for(;++i<a;){var l=(A=n[i])[0],c=t[l],u=A[1];if(s&&A[2]){if(c===o&&!(l in t))return!1}else{var g=new vr;if(r)var h=r(c,u,l,t,e,g);if(!(h===o?oi(u,c,f|p,r,g):h))return!1}}return!0}function si(t){return!(!xs(t)||(e=t,ge&&ge in e))&&(bs(t)?pe:zt).test(ua(t));var e}function Ai(t){return"function"==typeof t?t:null==t?MA:"object"==typeof t?Is(t)?di(t[0],t[1]):hi(t):OA(t)}function li(t){if(!Xo(t))return Pn(t);var e=[];for(var n in ee(t))ce.call(t,n)&&"constructor"!=n&&e.push(n);return e}function ci(t){if(!xs(t))return function(t){var e=[];if(null!=t)for(var n in ee(t))e.push(n);return e}(t);var e=Xo(t),n=[];for(var r in t)("constructor"!=r||!e&&ce.call(t,r))&&n.push(r);return n}function ui(t,e){return t<e}function gi(t,e){var n=-1,i=ms(t)?r(t.length):[];return Or(t,function(t,r,o){i[++n]=e(t,r,o)}),i}function hi(t){var e=Uo(t);return 1==e.length&&e[0][2]?$o(e[0][0],e[0][1]):function(n){return n===t||ai(n,t,e)}}function di(t,e){return qo(t)&&Zo(e)?$o(ca(t),e):function(n){var r=$s(n,t);return r===o&&r===e?tA(n,t):oi(e,r,f|p)}}function fi(t,e,n,r,i){t!==e&&Kr(e,function(a,s){if(xs(a))i||(i=new vr),function(t,e,n,r,i,a,s){var A=na(t,n),l=na(e,n),c=s.get(l);if(c)xr(t,n,c);else{var u=a?a(A,l,n+"",t,e,s):o,g=u===o;if(g){var h=Is(l),d=!h&&Es(l),f=!h&&!d&&Fs(l);u=l,h||d||f?Is(A)?u=A:ys(A)?u=no(A):d?(g=!1,u=Wi(l,!0)):f?(g=!1,u=Zi(l,!0)):u=[]:ks(l)||ps(l)?(u=A,ps(A)?u=Ps(A):xs(A)&&!bs(A)||(u=Po(l))):g=!1}g&&(s.set(l,u),i(u,l,r,a,s),s.delete(l)),xr(t,n,u)}}(t,e,s,n,fi,r,i);else{var A=r?r(na(t,s),a,s+"",t,e,i):o;A===o&&(A=a),xr(t,s,A)}},oA)}function pi(t,e){var n=t.length;if(n)return zo(e+=e<0?n:0,n)?t[e]:o}function Ii(t,e,n){var r=-1;return e=$e(e.length?e:[MA],In(Fo())),function(t,e){var n=t.length;for(t.sort(e);n--;)t[n]=t[n].value;return t}(gi(t,function(t,n,i){return{criteria:$e(e,function(e){return e(t)}),index:++r,value:t}}),function(t,e){return function(t,e,n){for(var r=-1,i=t.criteria,o=e.criteria,a=i.length,s=n.length;++r<a;){var A=$i(i[r],o[r]);if(A){if(r>=s)return A;var l=n[r];return A*("desc"==l?-1:1)}}return t.index-e.index}(t,e,n)})}function Ci(t,e,n){for(var r=-1,i=e.length,o={};++r<i;){var a=e[r],s=Wr(t,a);n(s,a)&&wi(o,zi(a,t),s)}return o}function mi(t,e,n,r){var i=r?ln:An,o=-1,a=e.length,s=t;for(t===e&&(e=no(e)),n&&(s=$e(t,In(n)));++o<a;)for(var A=0,l=e[o],c=n?n(l):l;(A=i(s,c,A,r))>-1;)s!==t&&Ne.call(s,A,1),Ne.call(t,A,1);return t}function yi(t,e){for(var n=t?e.length:0,r=n-1;n--;){var i=e[n];if(n==r||i!==o){var o=i;zo(i)?Ne.call(t,i,1):Ui(t,i)}}return t}function Ei(t,e){return t+On(Vn()*(e-t+1))}function vi(t,e){var n="";if(!t||e<1||e>R)return n;do{e%2&&(n+=t),(e=On(e/2))&&(t+=t)}while(e);return n}function Bi(t,e){return oa(ta(t,e,MA),t+"")}function bi(t){return br(hA(t))}function Qi(t,e){var n=hA(t);return Aa(n,Rr(e,0,n.length))}function wi(t,e,n,r){if(!xs(t))return t;for(var i=-1,a=(e=zi(e,t)).length,s=a-1,A=t;null!=A&&++i<a;){var l=ca(e[i]),c=n;if(i!=s){var u=A[l];(c=r?r(u,l,A):o)===o&&(c=xs(u)?u:zo(e[i+1])?[]:{})}_r(A,l,c),A=A[l]}return t}var xi=rr?function(t,e){return rr.set(t,e),t}:MA,_i=hn?function(t,e){return hn(t,"toString",{configurable:!0,enumerable:!1,value:xA(e),writable:!0})}:MA;function Di(t){return Aa(hA(t))}function Mi(t,e,n){var i=-1,o=t.length;e<0&&(e=-e>o?0:o+e),(n=n>o?o:n)<0&&(n+=o),o=e>n?0:n-e>>>0,e>>>=0;for(var a=r(o);++i<o;)a[i]=t[i+e];return a}function ki(t,e){var n;return Or(t,function(t,r,i){return!(n=e(t,r,i))}),!!n}function Si(t,e,n){var r=0,i=null==t?r:t.length;if("number"==typeof e&&e==e&&i<=O){for(;r<i;){var o=r+i>>>1,a=t[o];null!==a&&!Ns(a)&&(n?a<=e:a<e)?r=o+1:i=o}return i}return Li(t,e,MA,n)}function Li(t,e,n,r){e=n(e);for(var i=0,a=null==t?0:t.length,s=e!=e,A=null===e,l=Ns(e),c=e===o;i<a;){var u=On((i+a)/2),g=n(t[u]),h=g!==o,d=null===g,f=g==g,p=Ns(g);if(s)var I=r||f;else I=c?f&&(r||h):A?f&&h&&(r||!d):l?f&&h&&!d&&(r||!p):!d&&!p&&(r?g<=e:g<e);I?i=u+1:a=u}return zn(a,U)}function Ri(t,e){for(var n=-1,r=t.length,i=0,o=[];++n<r;){var a=t[n],s=e?e(a):a;if(!n||!hs(s,A)){var A=s;o[i++]=0===a?0:a}}return o}function Ni(t){return"number"==typeof t?t:Ns(t)?F:+t}function Fi(t){if("string"==typeof t)return t;if(Is(t))return $e(t,Fi)+"";if(Ns(t))return gr?gr.call(t):"";var e=t+"";return"0"==e&&1/t==-L?"-0":e}function Ti(t,e,n){var r=-1,i=Xe,o=t.length,s=!0,A=[],l=A;if(n)s=!1,i=Ze;else if(o>=a){var c=e?null:Bo(t);if(c)return Dn(c);s=!1,i=mn,l=new Er}else l=e?[]:A;t:for(;++r<o;){var u=t[r],g=e?e(u):u;if(u=n||0!==u?u:0,s&&g==g){for(var h=l.length;h--;)if(l[h]===g)continue t;e&&l.push(g),A.push(u)}else i(l,g,n)||(l!==A&&l.push(g),A.push(u))}return A}function Ui(t,e){return null==(t=ea(t,e=zi(e,t)))||delete t[ca(Ba(e))]}function Oi(t,e,n,r){return wi(t,e,n(Wr(t,e)),r)}function Gi(t,e,n,r){for(var i=t.length,o=r?i:-1;(r?o--:++o<i)&&e(t[o],o,t););return n?Mi(t,r?0:o,r?o+1:i):Mi(t,r?o+1:0,r?i:o)}function Hi(t,e){var n=t;return n instanceof Ir&&(n=n.value()),en(e,function(t,e){return e.func.apply(e.thisArg,tn([t],e.args))},n)}function ji(t,e,n){var i=t.length;if(i<2)return i?Ti(t[0]):[];for(var o=-1,a=r(i);++o<i;)for(var s=t[o],A=-1;++A<i;)A!=o&&(a[o]=Ur(a[o]||s,t[A],e,n));return Ti(Pr(a,1),e,n)}function Yi(t,e,n){for(var r=-1,i=t.length,a=e.length,s={};++r<i;){var A=r<a?e[r]:o;n(s,t[r],A)}return s}function Pi(t){return ys(t)?t:[]}function Ki(t){return"function"==typeof t?t:MA}function zi(t,e){return Is(t)?t:qo(t,e)?[t]:la(Ks(t))}var Ji=Bi;function qi(t,e,n){var r=t.length;return n=n===o?r:n,!e&&n>=r?t:Mi(t,e,n)}var Vi=Nn||function(t){return Le.clearTimeout(t)};function Wi(t,e){if(e)return t.slice();var n=t.length,r=_e?_e(n):new t.constructor(n);return t.copy(r),r}function Xi(t){var e=new t.constructor(t.byteLength);return new ve(e).set(new ve(t)),e}function Zi(t,e){var n=e?Xi(t.buffer):t.buffer;return new t.constructor(n,t.byteOffset,t.length)}function $i(t,e){if(t!==e){var n=t!==o,r=null===t,i=t==t,a=Ns(t),s=e!==o,A=null===e,l=e==e,c=Ns(e);if(!A&&!c&&!a&&t>e||a&&s&&l&&!A&&!c||r&&s&&l||!n&&l||!i)return 1;if(!r&&!a&&!c&&t<e||c&&n&&i&&!r&&!a||A&&n&&i||!s&&i||!l)return-1}return 0}function to(t,e,n,i){for(var o=-1,a=t.length,s=n.length,A=-1,l=e.length,c=Kn(a-s,0),u=r(l+c),g=!i;++A<l;)u[A]=e[A];for(;++o<s;)(g||o<a)&&(u[n[o]]=t[o]);for(;c--;)u[A++]=t[o++];return u}function eo(t,e,n,i){for(var o=-1,a=t.length,s=-1,A=n.length,l=-1,c=e.length,u=Kn(a-A,0),g=r(u+c),h=!i;++o<u;)g[o]=t[o];for(var d=o;++l<c;)g[d+l]=e[l];for(;++s<A;)(h||o<a)&&(g[d+n[s]]=t[o++]);return g}function no(t,e){var n=-1,i=t.length;for(e||(e=r(i));++n<i;)e[n]=t[n];return e}function ro(t,e,n,r){var i=!n;n||(n={});for(var a=-1,s=e.length;++a<s;){var A=e[a],l=r?r(n[A],t[A],A,n,t):o;l===o&&(l=t[A]),i?Sr(n,A,l):_r(n,A,l)}return n}function io(t,e){return function(n,r){var i=Is(n)?ze:Mr,o=e?e():{};return i(n,t,Fo(r,2),o)}}function oo(t){return Bi(function(e,n){var r=-1,i=n.length,a=i>1?n[i-1]:o,s=i>2?n[2]:o;for(a=t.length>3&&"function"==typeof a?(i--,a):o,s&&Jo(n[0],n[1],s)&&(a=i<3?o:a,i=1),e=ee(e);++r<i;){var A=n[r];A&&t(e,A,r,a)}return e})}function ao(t,e){return function(n,r){if(null==n)return n;if(!ms(n))return t(n,r);for(var i=n.length,o=e?i:-1,a=ee(n);(e?o--:++o<i)&&!1!==r(a[o],o,a););return n}}function so(t){return function(e,n,r){for(var i=-1,o=ee(e),a=r(e),s=a.length;s--;){var A=a[t?s:++i];if(!1===n(o[A],A,o))break}return e}}function Ao(t){return function(e){var n=Qn(e=Ks(e))?Sn(e):o,r=n?n[0]:e.charAt(0),i=n?qi(n,1).join(""):e.slice(1);return r[t]()+i}}function lo(t){return function(e){return en(bA(pA(e).replace(Ce,"")),t,"")}}function co(t){return function(){var e=arguments;switch(e.length){case 0:return new t;case 1:return new t(e[0]);case 2:return new t(e[0],e[1]);case 3:return new t(e[0],e[1],e[2]);case 4:return new t(e[0],e[1],e[2],e[3]);case 5:return new t(e[0],e[1],e[2],e[3],e[4]);case 6:return new t(e[0],e[1],e[2],e[3],e[4],e[5]);case 7:return new t(e[0],e[1],e[2],e[3],e[4],e[5],e[6])}var n=dr(t.prototype),r=t.apply(n,e);return xs(r)?r:n}}function uo(t){return function(e,n,r){var i=ee(e);if(!ms(e)){var a=Fo(n,3);e=iA(e),n=function(t){return a(i[t],t,i)}}var s=t(e,n,r);return s>-1?i[a?e[s]:s]:o}}function go(t){return Mo(function(e){var n=e.length,r=n,i=pr.prototype.thru;for(t&&e.reverse();r--;){var a=e[r];if("function"!=typeof a)throw new ie(A);if(i&&!s&&"wrapper"==Ro(a))var s=new pr([],!0)}for(r=s?r:n;++r<n;){var l=Ro(a=e[r]),c="wrapper"==l?Lo(a):o;s=c&&Vo(c[0])&&c[1]==(b|y|v|Q)&&!c[4].length&&1==c[9]?s[Ro(c[0])].apply(s,c[3]):1==a.length&&Vo(a)?s[l]():s.thru(a)}return function(){var t=arguments,r=t[0];if(s&&1==t.length&&Is(r))return s.plant(r).value();for(var i=0,o=n?e[i].apply(this,t):r;++i<n;)o=e[i].call(this,o);return o}})}function ho(t,e,n,i,a,s,A,l,c,u){var g=e&b,h=e&I,d=e&C,f=e&(y|E),p=e&w,m=d?o:co(t);return function I(){for(var C=arguments.length,y=r(C),E=C;E--;)y[E]=arguments[E];if(f)var v=No(I),B=function(t,e){for(var n=t.length,r=0;n--;)t[n]===e&&++r;return r}(y,v);if(i&&(y=to(y,i,a,f)),s&&(y=eo(y,s,A,f)),C-=B,f&&C<u){var b=_n(y,v);return Eo(t,e,ho,I.placeholder,n,y,b,l,c,u-C)}var Q=h?n:this,w=d?Q[t]:t;return C=y.length,l?y=function(t,e){for(var n=t.length,r=zn(e.length,n),i=no(t);r--;){var a=e[r];t[r]=zo(a,n)?i[a]:o}return t}(y,l):p&&C>1&&y.reverse(),g&&c<C&&(y.length=c),this&&this!==Le&&this instanceof I&&(w=m||co(w)),w.apply(Q,y)}}function fo(t,e){return function(n,r){return function(t,e,n,r){return Jr(t,function(t,i,o){e(r,n(t),i,o)}),r}(n,t,e(r),{})}}function po(t,e){return function(n,r){var i;if(n===o&&r===o)return e;if(n!==o&&(i=n),r!==o){if(i===o)return r;"string"==typeof n||"string"==typeof r?(n=Fi(n),r=Fi(r)):(n=Ni(n),r=Ni(r)),i=t(n,r)}return i}}function Io(t){return Mo(function(e){return e=$e(e,In(Fo())),Bi(function(n){var r=this;return t(e,function(t){return Ke(t,r,n)})})})}function Co(t,e){var n=(e=e===o?" ":Fi(e)).length;if(n<2)return n?vi(e,t):e;var r=vi(e,Un(t/kn(e)));return Qn(e)?qi(Sn(r),0,t).join(""):r.slice(0,t)}function mo(t){return function(e,n,i){return i&&"number"!=typeof i&&Jo(e,n,i)&&(n=i=o),e=Gs(e),n===o?(n=e,e=0):n=Gs(n),function(t,e,n,i){for(var o=-1,a=Kn(Un((e-t)/(n||1)),0),s=r(a);a--;)s[i?a:++o]=t,t+=n;return s}(e,n,i=i===o?e<n?1:-1:Gs(i),t)}}function yo(t){return function(e,n){return"string"==typeof e&&"string"==typeof n||(e=Ys(e),n=Ys(n)),t(e,n)}}function Eo(t,e,n,r,i,a,s,A,l,c){var u=e&y;e|=u?v:B,(e&=~(u?B:v))&m||(e&=~(I|C));var g=[t,e,i,u?a:o,u?s:o,u?o:a,u?o:s,A,l,c],h=n.apply(o,g);return Vo(t)&&ra(h,g),h.placeholder=r,aa(h,t,e)}function vo(t){var e=te[t];return function(t,n){if(t=Ys(t),n=null==n?0:zn(Hs(n),292)){var r=(Ks(t)+"e").split("e");return+((r=(Ks(e(r[0]+"e"+(+r[1]+n)))+"e").split("e"))[0]+"e"+(+r[1]-n))}return e(t)}}var Bo=tr&&1/Dn(new tr([,-0]))[1]==L?function(t){return new tr(t)}:NA;function bo(t){return function(e){var n=jo(e);return n==W?wn(e):n==nt?Mn(e):function(t,e){return $e(e,function(e){return[e,t[e]]})}(e,t(e))}}function Qo(t,e,n,i,a,s,l,c){var g=e&C;if(!g&&"function"!=typeof t)throw new ie(A);var h=i?i.length:0;if(h||(e&=~(v|B),i=a=o),l=l===o?l:Kn(Hs(l),0),c=c===o?c:Hs(c),h-=a?a.length:0,e&B){var d=i,f=a;i=a=o}var p=g?o:Lo(t),w=[t,e,n,i,a,d,f,s,l,c];if(p&&function(t,e){var n=t[1],r=e[1],i=n|r,o=i<(I|C|b),a=r==b&&n==y||r==b&&n==Q&&t[7].length<=e[8]||r==(b|Q)&&e[7].length<=e[8]&&n==y;if(!o&&!a)return t;r&I&&(t[2]=e[2],i|=n&I?0:m);var s=e[3];if(s){var A=t[3];t[3]=A?to(A,s,e[4]):s,t[4]=A?_n(t[3],u):e[4]}(s=e[5])&&(A=t[5],t[5]=A?eo(A,s,e[6]):s,t[6]=A?_n(t[5],u):e[6]),(s=e[7])&&(t[7]=s),r&b&&(t[8]=null==t[8]?e[8]:zn(t[8],e[8])),null==t[9]&&(t[9]=e[9]),t[0]=e[0],t[1]=i}(w,p),t=w[0],e=w[1],n=w[2],i=w[3],a=w[4],!(c=w[9]=w[9]===o?g?0:t.length:Kn(w[9]-h,0))&&e&(y|E)&&(e&=~(y|E)),e&&e!=I)x=e==y||e==E?function(t,e,n){var i=co(t);return function a(){for(var s=arguments.length,A=r(s),l=s,c=No(a);l--;)A[l]=arguments[l];var u=s<3&&A[0]!==c&&A[s-1]!==c?[]:_n(A,c);return(s-=u.length)<n?Eo(t,e,ho,a.placeholder,o,A,u,o,o,n-s):Ke(this&&this!==Le&&this instanceof a?i:t,this,A)}}(t,e,c):e!=v&&e!=(I|v)||a.length?ho.apply(o,w):function(t,e,n,i){var o=e&I,a=co(t);return function e(){for(var s=-1,A=arguments.length,l=-1,c=i.length,u=r(c+A),g=this&&this!==Le&&this instanceof e?a:t;++l<c;)u[l]=i[l];for(;A--;)u[l++]=arguments[++s];return Ke(g,o?n:this,u)}}(t,e,n,i);else var x=function(t,e,n){var r=e&I,i=co(t);return function e(){return(this&&this!==Le&&this instanceof e?i:t).apply(r?n:this,arguments)}}(t,e,n);return aa((p?xi:ra)(x,w),t,e)}function wo(t,e,n,r){return t===o||hs(t,se[n])&&!ce.call(r,n)?e:t}function xo(t,e,n,r,i,a){return xs(t)&&xs(e)&&(a.set(e,t),fi(t,e,o,xo,a),a.delete(e)),t}function _o(t){return ks(t)?o:t}function Do(t,e,n,r,i,a){var s=n&f,A=t.length,l=e.length;if(A!=l&&!(s&&l>A))return!1;var c=a.get(t);if(c&&a.get(e))return c==e;var u=-1,g=!0,h=n&p?new Er:o;for(a.set(t,e),a.set(e,t);++u<A;){var d=t[u],I=e[u];if(r)var C=s?r(I,d,u,e,t,a):r(d,I,u,t,e,a);if(C!==o){if(C)continue;g=!1;break}if(h){if(!rn(e,function(t,e){if(!mn(h,e)&&(d===t||i(d,t,n,r,a)))return h.push(e)})){g=!1;break}}else if(d!==I&&!i(d,I,n,r,a)){g=!1;break}}return a.delete(t),a.delete(e),g}function Mo(t){return oa(ta(t,o,Ca),t+"")}function ko(t){return Xr(t,iA,Go)}function So(t){return Xr(t,oA,Ho)}var Lo=rr?function(t){return rr.get(t)}:NA;function Ro(t){for(var e=t.name+"",n=ir[e],r=ce.call(ir,e)?n.length:0;r--;){var i=n[r],o=i.func;if(null==o||o==t)return i.name}return e}function No(t){return(ce.call(hr,"placeholder")?hr:t).placeholder}function Fo(){var t=hr.iteratee||kA;return t=t===kA?Ai:t,arguments.length?t(arguments[0],arguments[1]):t}function To(t,e){var n,r,i=t.__data__;return("string"==(r=typeof(n=e))||"number"==r||"symbol"==r||"boolean"==r?"__proto__"!==n:null===n)?i["string"==typeof e?"string":"hash"]:i.map}function Uo(t){for(var e=iA(t),n=e.length;n--;){var r=e[n],i=t[r];e[n]=[r,i,Zo(i)]}return e}function Oo(t,e){var n=function(t,e){return null==t?o:t[e]}(t,e);return si(n)?n:o}var Go=Gn?function(t){return null==t?[]:(t=ee(t),We(Gn(t),function(e){return Re.call(t,e)}))}:jA,Ho=Gn?function(t){for(var e=[];t;)tn(e,Go(t)),t=ke(t);return e}:jA,jo=Zr;function Yo(t,e,n){for(var r=-1,i=(e=zi(e,t)).length,o=!1;++r<i;){var a=ca(e[r]);if(!(o=null!=t&&n(t,a)))break;t=t[a]}return o||++r!=i?o:!!(i=null==t?0:t.length)&&ws(i)&&zo(a,i)&&(Is(t)||ps(t))}function Po(t){return"function"!=typeof t.constructor||Xo(t)?{}:dr(ke(t))}function Ko(t){return Is(t)||ps(t)||!!(Te&&t&&t[Te])}function zo(t,e){var n=typeof t;return!!(e=null==e?R:e)&&("number"==n||"symbol"!=n&&qt.test(t))&&t>-1&&t%1==0&&t<e}function Jo(t,e,n){if(!xs(n))return!1;var r=typeof e;return!!("number"==r?ms(n)&&zo(e,n.length):"string"==r&&e in n)&&hs(n[e],t)}function qo(t,e){if(Is(t))return!1;var n=typeof t;return!("number"!=n&&"symbol"!=n&&"boolean"!=n&&null!=t&&!Ns(t))||Mt.test(t)||!Dt.test(t)||null!=e&&t in ee(e)}function Vo(t){var e=Ro(t),n=hr[e];if("function"!=typeof n||!(e in Ir.prototype))return!1;if(t===n)return!0;var r=Lo(n);return!!r&&t===r[0]}(Xn&&jo(new Xn(new ArrayBuffer(1)))!=lt||Zn&&jo(new Zn)!=W||$n&&"[object Promise]"!=jo($n.resolve())||tr&&jo(new tr)!=nt||er&&jo(new er)!=at)&&(jo=function(t){var e=Zr(t),n=e==$?t.constructor:o,r=n?ua(n):"";if(r)switch(r){case or:return lt;case ar:return W;case sr:return"[object Promise]";case Ar:return nt;case lr:return at}return e});var Wo=Ae?bs:YA;function Xo(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||se)}function Zo(t){return t==t&&!xs(t)}function $o(t,e){return function(n){return null!=n&&n[t]===e&&(e!==o||t in ee(n))}}function ta(t,e,n){return e=Kn(e===o?t.length-1:e,0),function(){for(var i=arguments,o=-1,a=Kn(i.length-e,0),s=r(a);++o<a;)s[o]=i[e+o];o=-1;for(var A=r(e+1);++o<e;)A[o]=i[o];return A[e]=n(s),Ke(t,this,A)}}function ea(t,e){return e.length<2?t:Wr(t,Mi(e,0,-1))}function na(t,e){if("__proto__"!=e)return t[e]}var ra=sa(xi),ia=Tn||function(t,e){return Le.setTimeout(t,e)},oa=sa(_i);function aa(t,e,n){var r=e+"";return oa(t,function(t,e){var n=e.length;if(!n)return t;var r=n-1;return e[r]=(n>1?"& ":"")+e[r],e=e.join(n>2?", ":" "),t.replace(Tt,"{\n/* [wrapped with "+e+"] */\n")}(r,function(t,e){return Je(G,function(n){var r="_."+n[0];e&n[1]&&!Xe(t,r)&&t.push(r)}),t.sort()}(function(t){var e=t.match(Ut);return e?e[1].split(Ot):[]}(r),n)))}function sa(t){var e=0,n=0;return function(){var r=Jn(),i=M-(r-n);if(n=r,i>0){if(++e>=D)return arguments[0]}else e=0;return t.apply(o,arguments)}}function Aa(t,e){var n=-1,r=t.length,i=r-1;for(e=e===o?r:e;++n<e;){var a=Ei(n,i),s=t[a];t[a]=t[n],t[n]=s}return t.length=e,t}var la=function(t){var e=ss(t,function(t){return n.size===c&&n.clear(),t}),n=e.cache;return e}(function(t){var e=[];return 46===t.charCodeAt(0)&&e.push(""),t.replace(kt,function(t,n,r,i){e.push(r?i.replace(Ht,"$1"):n||t)}),e});function ca(t){if("string"==typeof t||Ns(t))return t;var e=t+"";return"0"==e&&1/t==-L?"-0":e}function ua(t){if(null!=t){try{return le.call(t)}catch(t){}try{return t+""}catch(t){}}return""}function ga(t){if(t instanceof Ir)return t.clone();var e=new pr(t.__wrapped__,t.__chain__);return e.__actions__=no(t.__actions__),e.__index__=t.__index__,e.__values__=t.__values__,e}var ha=Bi(function(t,e){return ys(t)?Ur(t,Pr(e,1,ys,!0)):[]}),da=Bi(function(t,e){var n=Ba(e);return ys(n)&&(n=o),ys(t)?Ur(t,Pr(e,1,ys,!0),Fo(n,2)):[]}),fa=Bi(function(t,e){var n=Ba(e);return ys(n)&&(n=o),ys(t)?Ur(t,Pr(e,1,ys,!0),o,n):[]});function pa(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=null==n?0:Hs(n);return i<0&&(i=Kn(r+i,0)),sn(t,Fo(e,3),i)}function Ia(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=r-1;return n!==o&&(i=Hs(n),i=n<0?Kn(r+i,0):zn(i,r-1)),sn(t,Fo(e,3),i,!0)}function Ca(t){return null!=t&&t.length?Pr(t,1):[]}function ma(t){return t&&t.length?t[0]:o}var ya=Bi(function(t){var e=$e(t,Pi);return e.length&&e[0]===t[0]?ni(e):[]}),Ea=Bi(function(t){var e=Ba(t),n=$e(t,Pi);return e===Ba(n)?e=o:n.pop(),n.length&&n[0]===t[0]?ni(n,Fo(e,2)):[]}),va=Bi(function(t){var e=Ba(t),n=$e(t,Pi);return(e="function"==typeof e?e:o)&&n.pop(),n.length&&n[0]===t[0]?ni(n,o,e):[]});function Ba(t){var e=null==t?0:t.length;return e?t[e-1]:o}var ba=Bi(Qa);function Qa(t,e){return t&&t.length&&e&&e.length?mi(t,e):t}var wa=Mo(function(t,e){var n=null==t?0:t.length,r=Lr(t,e);return yi(t,$e(e,function(t){return zo(t,n)?+t:t}).sort($i)),r});function xa(t){return null==t?t:Wn.call(t)}var _a=Bi(function(t){return Ti(Pr(t,1,ys,!0))}),Da=Bi(function(t){var e=Ba(t);return ys(e)&&(e=o),Ti(Pr(t,1,ys,!0),Fo(e,2))}),Ma=Bi(function(t){var e=Ba(t);return e="function"==typeof e?e:o,Ti(Pr(t,1,ys,!0),o,e)});function ka(t){if(!t||!t.length)return[];var e=0;return t=We(t,function(t){if(ys(t))return e=Kn(t.length,e),!0}),pn(e,function(e){return $e(t,gn(e))})}function Sa(t,e){if(!t||!t.length)return[];var n=ka(t);return null==e?n:$e(n,function(t){return Ke(e,o,t)})}var La=Bi(function(t,e){return ys(t)?Ur(t,e):[]}),Ra=Bi(function(t){return ji(We(t,ys))}),Na=Bi(function(t){var e=Ba(t);return ys(e)&&(e=o),ji(We(t,ys),Fo(e,2))}),Fa=Bi(function(t){var e=Ba(t);return e="function"==typeof e?e:o,ji(We(t,ys),o,e)}),Ta=Bi(ka);var Ua=Bi(function(t){var e=t.length,n=e>1?t[e-1]:o;return n="function"==typeof n?(t.pop(),n):o,Sa(t,n)});function Oa(t){var e=hr(t);return e.__chain__=!0,e}function Ga(t,e){return e(t)}var Ha=Mo(function(t){var e=t.length,n=e?t[0]:0,r=this.__wrapped__,i=function(e){return Lr(e,t)};return!(e>1||this.__actions__.length)&&r instanceof Ir&&zo(n)?((r=r.slice(n,+n+(e?1:0))).__actions__.push({func:Ga,args:[i],thisArg:o}),new pr(r,this.__chain__).thru(function(t){return e&&!t.length&&t.push(o),t})):this.thru(i)});var ja=io(function(t,e,n){ce.call(t,n)?++t[n]:Sr(t,n,1)});var Ya=uo(pa),Pa=uo(Ia);function Ka(t,e){return(Is(t)?Je:Or)(t,Fo(e,3))}function za(t,e){return(Is(t)?qe:Gr)(t,Fo(e,3))}var Ja=io(function(t,e,n){ce.call(t,n)?t[n].push(e):Sr(t,n,[e])});var qa=Bi(function(t,e,n){var i=-1,o="function"==typeof e,a=ms(t)?r(t.length):[];return Or(t,function(t){a[++i]=o?Ke(e,t,n):ri(t,e,n)}),a}),Va=io(function(t,e,n){Sr(t,n,e)});function Wa(t,e){return(Is(t)?$e:gi)(t,Fo(e,3))}var Xa=io(function(t,e,n){t[n?0:1].push(e)},function(){return[[],[]]});var Za=Bi(function(t,e){if(null==t)return[];var n=e.length;return n>1&&Jo(t,e[0],e[1])?e=[]:n>2&&Jo(e[0],e[1],e[2])&&(e=[e[0]]),Ii(t,Pr(e,1),[])}),$a=Fn||function(){return Le.Date.now()};function ts(t,e,n){return e=n?o:e,e=t&&null==e?t.length:e,Qo(t,b,o,o,o,o,e)}function es(t,e){var n;if("function"!=typeof e)throw new ie(A);return t=Hs(t),function(){return--t>0&&(n=e.apply(this,arguments)),t<=1&&(e=o),n}}var ns=Bi(function(t,e,n){var r=I;if(n.length){var i=_n(n,No(ns));r|=v}return Qo(t,r,e,n,i)}),rs=Bi(function(t,e,n){var r=I|C;if(n.length){var i=_n(n,No(rs));r|=v}return Qo(e,r,t,n,i)});function is(t,e,n){var r,i,a,s,l,c,u=0,g=!1,h=!1,d=!0;if("function"!=typeof t)throw new ie(A);function f(e){var n=r,a=i;return r=i=o,u=e,s=t.apply(a,n)}function p(t){var n=t-c;return c===o||n>=e||n<0||h&&t-u>=a}function I(){var t=$a();if(p(t))return C(t);l=ia(I,function(t){var n=e-(t-c);return h?zn(n,a-(t-u)):n}(t))}function C(t){return l=o,d&&r?f(t):(r=i=o,s)}function m(){var t=$a(),n=p(t);if(r=arguments,i=this,c=t,n){if(l===o)return function(t){return u=t,l=ia(I,e),g?f(t):s}(c);if(h)return l=ia(I,e),f(c)}return l===o&&(l=ia(I,e)),s}return e=Ys(e)||0,xs(n)&&(g=!!n.leading,a=(h="maxWait"in n)?Kn(Ys(n.maxWait)||0,e):a,d="trailing"in n?!!n.trailing:d),m.cancel=function(){l!==o&&Vi(l),u=0,r=c=i=l=o},m.flush=function(){return l===o?s:C($a())},m}var os=Bi(function(t,e){return Tr(t,1,e)}),as=Bi(function(t,e,n){return Tr(t,Ys(e)||0,n)});function ss(t,e){if("function"!=typeof t||null!=e&&"function"!=typeof e)throw new ie(A);var n=function(){var r=arguments,i=e?e.apply(this,r):r[0],o=n.cache;if(o.has(i))return o.get(i);var a=t.apply(this,r);return n.cache=o.set(i,a)||o,a};return n.cache=new(ss.Cache||yr),n}function As(t){if("function"!=typeof t)throw new ie(A);return function(){var e=arguments;switch(e.length){case 0:return!t.call(this);case 1:return!t.call(this,e[0]);case 2:return!t.call(this,e[0],e[1]);case 3:return!t.call(this,e[0],e[1],e[2])}return!t.apply(this,e)}}ss.Cache=yr;var ls=Ji(function(t,e){var n=(e=1==e.length&&Is(e[0])?$e(e[0],In(Fo())):$e(Pr(e,1),In(Fo()))).length;return Bi(function(r){for(var i=-1,o=zn(r.length,n);++i<o;)r[i]=e[i].call(this,r[i]);return Ke(t,this,r)})}),cs=Bi(function(t,e){var n=_n(e,No(cs));return Qo(t,v,o,e,n)}),us=Bi(function(t,e){var n=_n(e,No(us));return Qo(t,B,o,e,n)}),gs=Mo(function(t,e){return Qo(t,Q,o,o,o,e)});function hs(t,e){return t===e||t!=t&&e!=e}var ds=yo($r),fs=yo(function(t,e){return t>=e}),ps=ii(function(){return arguments}())?ii:function(t){return _s(t)&&ce.call(t,"callee")&&!Re.call(t,"callee")},Is=r.isArray,Cs=Oe?In(Oe):function(t){return _s(t)&&Zr(t)==At};function ms(t){return null!=t&&ws(t.length)&&!bs(t)}function ys(t){return _s(t)&&ms(t)}var Es=Hn||YA,vs=Ge?In(Ge):function(t){return _s(t)&&Zr(t)==K};function Bs(t){if(!_s(t))return!1;var e=Zr(t);return e==J||e==z||"string"==typeof t.message&&"string"==typeof t.name&&!ks(t)}function bs(t){if(!xs(t))return!1;var e=Zr(t);return e==q||e==V||e==Y||e==tt}function Qs(t){return"number"==typeof t&&t==Hs(t)}function ws(t){return"number"==typeof t&&t>-1&&t%1==0&&t<=R}function xs(t){var e=typeof t;return null!=t&&("object"==e||"function"==e)}function _s(t){return null!=t&&"object"==typeof t}var Ds=He?In(He):function(t){return _s(t)&&jo(t)==W};function Ms(t){return"number"==typeof t||_s(t)&&Zr(t)==X}function ks(t){if(!_s(t)||Zr(t)!=$)return!1;var e=ke(t);if(null===e)return!0;var n=ce.call(e,"constructor")&&e.constructor;return"function"==typeof n&&n instanceof n&&le.call(n)==de}var Ss=je?In(je):function(t){return _s(t)&&Zr(t)==et};var Ls=Ye?In(Ye):function(t){return _s(t)&&jo(t)==nt};function Rs(t){return"string"==typeof t||!Is(t)&&_s(t)&&Zr(t)==rt}function Ns(t){return"symbol"==typeof t||_s(t)&&Zr(t)==it}var Fs=Pe?In(Pe):function(t){return _s(t)&&ws(t.length)&&!!we[Zr(t)]};var Ts=yo(ui),Us=yo(function(t,e){return t<=e});function Os(t){if(!t)return[];if(ms(t))return Rs(t)?Sn(t):no(t);if(Ue&&t[Ue])return function(t){for(var e,n=[];!(e=t.next()).done;)n.push(e.value);return n}(t[Ue]());var e=jo(t);return(e==W?wn:e==nt?Dn:hA)(t)}function Gs(t){return t?(t=Ys(t))===L||t===-L?(t<0?-1:1)*N:t==t?t:0:0===t?t:0}function Hs(t){var e=Gs(t),n=e%1;return e==e?n?e-n:e:0}function js(t){return t?Rr(Hs(t),0,T):0}function Ys(t){if("number"==typeof t)return t;if(Ns(t))return F;if(xs(t)){var e="function"==typeof t.valueOf?t.valueOf():t;t=xs(e)?e+"":e}if("string"!=typeof t)return 0===t?t:+t;t=t.replace(Rt,"");var n=Kt.test(t);return n||Jt.test(t)?Me(t.slice(2),n?2:8):Pt.test(t)?F:+t}function Ps(t){return ro(t,oA(t))}function Ks(t){return null==t?"":Fi(t)}var zs=oo(function(t,e){if(Xo(e)||ms(e))ro(e,iA(e),t);else for(var n in e)ce.call(e,n)&&_r(t,n,e[n])}),Js=oo(function(t,e){ro(e,oA(e),t)}),qs=oo(function(t,e,n,r){ro(e,oA(e),t,r)}),Vs=oo(function(t,e,n,r){ro(e,iA(e),t,r)}),Ws=Mo(Lr);var Xs=Bi(function(t,e){t=ee(t);var n=-1,r=e.length,i=r>2?e[2]:o;for(i&&Jo(e[0],e[1],i)&&(r=1);++n<r;)for(var a=e[n],s=oA(a),A=-1,l=s.length;++A<l;){var c=s[A],u=t[c];(u===o||hs(u,se[c])&&!ce.call(t,c))&&(t[c]=a[c])}return t}),Zs=Bi(function(t){return t.push(o,xo),Ke(sA,o,t)});function $s(t,e,n){var r=null==t?o:Wr(t,e);return r===o?n:r}function tA(t,e){return null!=t&&Yo(t,e,ei)}var eA=fo(function(t,e,n){null!=e&&"function"!=typeof e.toString&&(e=he.call(e)),t[e]=n},xA(MA)),nA=fo(function(t,e,n){null!=e&&"function"!=typeof e.toString&&(e=he.call(e)),ce.call(t,e)?t[e].push(n):t[e]=[n]},Fo),rA=Bi(ri);function iA(t){return ms(t)?Br(t):li(t)}function oA(t){return ms(t)?Br(t,!0):ci(t)}var aA=oo(function(t,e,n){fi(t,e,n)}),sA=oo(function(t,e,n,r){fi(t,e,n,r)}),AA=Mo(function(t,e){var n={};if(null==t)return n;var r=!1;e=$e(e,function(e){return e=zi(e,t),r||(r=e.length>1),e}),ro(t,So(t),n),r&&(n=Nr(n,g|h|d,_o));for(var i=e.length;i--;)Ui(n,e[i]);return n});var lA=Mo(function(t,e){return null==t?{}:function(t,e){return Ci(t,e,function(e,n){return tA(t,n)})}(t,e)});function cA(t,e){if(null==t)return{};var n=$e(So(t),function(t){return[t]});return e=Fo(e),Ci(t,n,function(t,n){return e(t,n[0])})}var uA=bo(iA),gA=bo(oA);function hA(t){return null==t?[]:Cn(t,iA(t))}var dA=lo(function(t,e,n){return e=e.toLowerCase(),t+(n?fA(e):e)});function fA(t){return BA(Ks(t).toLowerCase())}function pA(t){return(t=Ks(t))&&t.replace(Vt,vn).replace(me,"")}var IA=lo(function(t,e,n){return t+(n?"-":"")+e.toLowerCase()}),CA=lo(function(t,e,n){return t+(n?" ":"")+e.toLowerCase()}),mA=Ao("toLowerCase");var yA=lo(function(t,e,n){return t+(n?"_":"")+e.toLowerCase()});var EA=lo(function(t,e,n){return t+(n?" ":"")+BA(e)});var vA=lo(function(t,e,n){return t+(n?" ":"")+e.toUpperCase()}),BA=Ao("toUpperCase");function bA(t,e,n){return t=Ks(t),(e=n?o:e)===o?function(t){return Be.test(t)}(t)?function(t){return t.match(Ee)||[]}(t):function(t){return t.match(Gt)||[]}(t):t.match(e)||[]}var QA=Bi(function(t,e){try{return Ke(t,o,e)}catch(t){return Bs(t)?t:new Zt(t)}}),wA=Mo(function(t,e){return Je(e,function(e){e=ca(e),Sr(t,e,ns(t[e],t))}),t});function xA(t){return function(){return t}}var _A=go(),DA=go(!0);function MA(t){return t}function kA(t){return Ai("function"==typeof t?t:Nr(t,g))}var SA=Bi(function(t,e){return function(n){return ri(n,t,e)}}),LA=Bi(function(t,e){return function(n){return ri(t,n,e)}});function RA(t,e,n){var r=iA(e),i=Vr(e,r);null!=n||xs(e)&&(i.length||!r.length)||(n=e,e=t,t=this,i=Vr(e,iA(e)));var o=!(xs(n)&&"chain"in n&&!n.chain),a=bs(t);return Je(i,function(n){var r=e[n];t[n]=r,a&&(t.prototype[n]=function(){var e=this.__chain__;if(o||e){var n=t(this.__wrapped__),i=n.__actions__=no(this.__actions__);return i.push({func:r,args:arguments,thisArg:t}),n.__chain__=e,n}return r.apply(t,tn([this.value()],arguments))})}),t}function NA(){}var FA=Io($e),TA=Io(Ve),UA=Io(rn);function OA(t){return qo(t)?gn(ca(t)):function(t){return function(e){return Wr(e,t)}}(t)}var GA=mo(),HA=mo(!0);function jA(){return[]}function YA(){return!1}var PA=po(function(t,e){return t+e},0),KA=vo("ceil"),zA=po(function(t,e){return t/e},1),JA=vo("floor");var qA,VA=po(function(t,e){return t*e},1),WA=vo("round"),XA=po(function(t,e){return t-e},0);return hr.after=function(t,e){if("function"!=typeof e)throw new ie(A);return t=Hs(t),function(){if(--t<1)return e.apply(this,arguments)}},hr.ary=ts,hr.assign=zs,hr.assignIn=Js,hr.assignInWith=qs,hr.assignWith=Vs,hr.at=Ws,hr.before=es,hr.bind=ns,hr.bindAll=wA,hr.bindKey=rs,hr.castArray=function(){if(!arguments.length)return[];var t=arguments[0];return Is(t)?t:[t]},hr.chain=Oa,hr.chunk=function(t,e,n){e=(n?Jo(t,e,n):e===o)?1:Kn(Hs(e),0);var i=null==t?0:t.length;if(!i||e<1)return[];for(var a=0,s=0,A=r(Un(i/e));a<i;)A[s++]=Mi(t,a,a+=e);return A},hr.compact=function(t){for(var e=-1,n=null==t?0:t.length,r=0,i=[];++e<n;){var o=t[e];o&&(i[r++]=o)}return i},hr.concat=function(){var t=arguments.length;if(!t)return[];for(var e=r(t-1),n=arguments[0],i=t;i--;)e[i-1]=arguments[i];return tn(Is(n)?no(n):[n],Pr(e,1))},hr.cond=function(t){var e=null==t?0:t.length,n=Fo();return t=e?$e(t,function(t){if("function"!=typeof t[1])throw new ie(A);return[n(t[0]),t[1]]}):[],Bi(function(n){for(var r=-1;++r<e;){var i=t[r];if(Ke(i[0],this,n))return Ke(i[1],this,n)}})},hr.conforms=function(t){return function(t){var e=iA(t);return function(n){return Fr(n,t,e)}}(Nr(t,g))},hr.constant=xA,hr.countBy=ja,hr.create=function(t,e){var n=dr(t);return null==e?n:kr(n,e)},hr.curry=function t(e,n,r){var i=Qo(e,y,o,o,o,o,o,n=r?o:n);return i.placeholder=t.placeholder,i},hr.curryRight=function t(e,n,r){var i=Qo(e,E,o,o,o,o,o,n=r?o:n);return i.placeholder=t.placeholder,i},hr.debounce=is,hr.defaults=Xs,hr.defaultsDeep=Zs,hr.defer=os,hr.delay=as,hr.difference=ha,hr.differenceBy=da,hr.differenceWith=fa,hr.drop=function(t,e,n){var r=null==t?0:t.length;return r?Mi(t,(e=n||e===o?1:Hs(e))<0?0:e,r):[]},hr.dropRight=function(t,e,n){var r=null==t?0:t.length;return r?Mi(t,0,(e=r-(e=n||e===o?1:Hs(e)))<0?0:e):[]},hr.dropRightWhile=function(t,e){return t&&t.length?Gi(t,Fo(e,3),!0,!0):[]},hr.dropWhile=function(t,e){return t&&t.length?Gi(t,Fo(e,3),!0):[]},hr.fill=function(t,e,n,r){var i=null==t?0:t.length;return i?(n&&"number"!=typeof n&&Jo(t,e,n)&&(n=0,r=i),function(t,e,n,r){var i=t.length;for((n=Hs(n))<0&&(n=-n>i?0:i+n),(r=r===o||r>i?i:Hs(r))<0&&(r+=i),r=n>r?0:js(r);n<r;)t[n++]=e;return t}(t,e,n,r)):[]},hr.filter=function(t,e){return(Is(t)?We:Yr)(t,Fo(e,3))},hr.flatMap=function(t,e){return Pr(Wa(t,e),1)},hr.flatMapDeep=function(t,e){return Pr(Wa(t,e),L)},hr.flatMapDepth=function(t,e,n){return n=n===o?1:Hs(n),Pr(Wa(t,e),n)},hr.flatten=Ca,hr.flattenDeep=function(t){return null!=t&&t.length?Pr(t,L):[]},hr.flattenDepth=function(t,e){return null!=t&&t.length?Pr(t,e=e===o?1:Hs(e)):[]},hr.flip=function(t){return Qo(t,w)},hr.flow=_A,hr.flowRight=DA,hr.fromPairs=function(t){for(var e=-1,n=null==t?0:t.length,r={};++e<n;){var i=t[e];r[i[0]]=i[1]}return r},hr.functions=function(t){return null==t?[]:Vr(t,iA(t))},hr.functionsIn=function(t){return null==t?[]:Vr(t,oA(t))},hr.groupBy=Ja,hr.initial=function(t){return null!=t&&t.length?Mi(t,0,-1):[]},hr.intersection=ya,hr.intersectionBy=Ea,hr.intersectionWith=va,hr.invert=eA,hr.invertBy=nA,hr.invokeMap=qa,hr.iteratee=kA,hr.keyBy=Va,hr.keys=iA,hr.keysIn=oA,hr.map=Wa,hr.mapKeys=function(t,e){var n={};return e=Fo(e,3),Jr(t,function(t,r,i){Sr(n,e(t,r,i),t)}),n},hr.mapValues=function(t,e){var n={};return e=Fo(e,3),Jr(t,function(t,r,i){Sr(n,r,e(t,r,i))}),n},hr.matches=function(t){return hi(Nr(t,g))},hr.matchesProperty=function(t,e){return di(t,Nr(e,g))},hr.memoize=ss,hr.merge=aA,hr.mergeWith=sA,hr.method=SA,hr.methodOf=LA,hr.mixin=RA,hr.negate=As,hr.nthArg=function(t){return t=Hs(t),Bi(function(e){return pi(e,t)})},hr.omit=AA,hr.omitBy=function(t,e){return cA(t,As(Fo(e)))},hr.once=function(t){return es(2,t)},hr.orderBy=function(t,e,n,r){return null==t?[]:(Is(e)||(e=null==e?[]:[e]),Is(n=r?o:n)||(n=null==n?[]:[n]),Ii(t,e,n))},hr.over=FA,hr.overArgs=ls,hr.overEvery=TA,hr.overSome=UA,hr.partial=cs,hr.partialRight=us,hr.partition=Xa,hr.pick=lA,hr.pickBy=cA,hr.property=OA,hr.propertyOf=function(t){return function(e){return null==t?o:Wr(t,e)}},hr.pull=ba,hr.pullAll=Qa,hr.pullAllBy=function(t,e,n){return t&&t.length&&e&&e.length?mi(t,e,Fo(n,2)):t},hr.pullAllWith=function(t,e,n){return t&&t.length&&e&&e.length?mi(t,e,o,n):t},hr.pullAt=wa,hr.range=GA,hr.rangeRight=HA,hr.rearg=gs,hr.reject=function(t,e){return(Is(t)?We:Yr)(t,As(Fo(e,3)))},hr.remove=function(t,e){var n=[];if(!t||!t.length)return n;var r=-1,i=[],o=t.length;for(e=Fo(e,3);++r<o;){var a=t[r];e(a,r,t)&&(n.push(a),i.push(r))}return yi(t,i),n},hr.rest=function(t,e){if("function"!=typeof t)throw new ie(A);return Bi(t,e=e===o?e:Hs(e))},hr.reverse=xa,hr.sampleSize=function(t,e,n){return e=(n?Jo(t,e,n):e===o)?1:Hs(e),(Is(t)?Qr:Qi)(t,e)},hr.set=function(t,e,n){return null==t?t:wi(t,e,n)},hr.setWith=function(t,e,n,r){return r="function"==typeof r?r:o,null==t?t:wi(t,e,n,r)},hr.shuffle=function(t){return(Is(t)?wr:Di)(t)},hr.slice=function(t,e,n){var r=null==t?0:t.length;return r?(n&&"number"!=typeof n&&Jo(t,e,n)?(e=0,n=r):(e=null==e?0:Hs(e),n=n===o?r:Hs(n)),Mi(t,e,n)):[]},hr.sortBy=Za,hr.sortedUniq=function(t){return t&&t.length?Ri(t):[]},hr.sortedUniqBy=function(t,e){return t&&t.length?Ri(t,Fo(e,2)):[]},hr.split=function(t,e,n){return n&&"number"!=typeof n&&Jo(t,e,n)&&(e=n=o),(n=n===o?T:n>>>0)?(t=Ks(t))&&("string"==typeof e||null!=e&&!Ss(e))&&!(e=Fi(e))&&Qn(t)?qi(Sn(t),0,n):t.split(e,n):[]},hr.spread=function(t,e){if("function"!=typeof t)throw new ie(A);return e=null==e?0:Kn(Hs(e),0),Bi(function(n){var r=n[e],i=qi(n,0,e);return r&&tn(i,r),Ke(t,this,i)})},hr.tail=function(t){var e=null==t?0:t.length;return e?Mi(t,1,e):[]},hr.take=function(t,e,n){return t&&t.length?Mi(t,0,(e=n||e===o?1:Hs(e))<0?0:e):[]},hr.takeRight=function(t,e,n){var r=null==t?0:t.length;return r?Mi(t,(e=r-(e=n||e===o?1:Hs(e)))<0?0:e,r):[]},hr.takeRightWhile=function(t,e){return t&&t.length?Gi(t,Fo(e,3),!1,!0):[]},hr.takeWhile=function(t,e){return t&&t.length?Gi(t,Fo(e,3)):[]},hr.tap=function(t,e){return e(t),t},hr.throttle=function(t,e,n){var r=!0,i=!0;if("function"!=typeof t)throw new ie(A);return xs(n)&&(r="leading"in n?!!n.leading:r,i="trailing"in n?!!n.trailing:i),is(t,e,{leading:r,maxWait:e,trailing:i})},hr.thru=Ga,hr.toArray=Os,hr.toPairs=uA,hr.toPairsIn=gA,hr.toPath=function(t){return Is(t)?$e(t,ca):Ns(t)?[t]:no(la(Ks(t)))},hr.toPlainObject=Ps,hr.transform=function(t,e,n){var r=Is(t),i=r||Es(t)||Fs(t);if(e=Fo(e,4),null==n){var o=t&&t.constructor;n=i?r?new o:[]:xs(t)&&bs(o)?dr(ke(t)):{}}return(i?Je:Jr)(t,function(t,r,i){return e(n,t,r,i)}),n},hr.unary=function(t){return ts(t,1)},hr.union=_a,hr.unionBy=Da,hr.unionWith=Ma,hr.uniq=function(t){return t&&t.length?Ti(t):[]},hr.uniqBy=function(t,e){return t&&t.length?Ti(t,Fo(e,2)):[]},hr.uniqWith=function(t,e){return e="function"==typeof e?e:o,t&&t.length?Ti(t,o,e):[]},hr.unset=function(t,e){return null==t||Ui(t,e)},hr.unzip=ka,hr.unzipWith=Sa,hr.update=function(t,e,n){return null==t?t:Oi(t,e,Ki(n))},hr.updateWith=function(t,e,n,r){return r="function"==typeof r?r:o,null==t?t:Oi(t,e,Ki(n),r)},hr.values=hA,hr.valuesIn=function(t){return null==t?[]:Cn(t,oA(t))},hr.without=La,hr.words=bA,hr.wrap=function(t,e){return cs(Ki(e),t)},hr.xor=Ra,hr.xorBy=Na,hr.xorWith=Fa,hr.zip=Ta,hr.zipObject=function(t,e){return Yi(t||[],e||[],_r)},hr.zipObjectDeep=function(t,e){return Yi(t||[],e||[],wi)},hr.zipWith=Ua,hr.entries=uA,hr.entriesIn=gA,hr.extend=Js,hr.extendWith=qs,RA(hr,hr),hr.add=PA,hr.attempt=QA,hr.camelCase=dA,hr.capitalize=fA,hr.ceil=KA,hr.clamp=function(t,e,n){return n===o&&(n=e,e=o),n!==o&&(n=(n=Ys(n))==n?n:0),e!==o&&(e=(e=Ys(e))==e?e:0),Rr(Ys(t),e,n)},hr.clone=function(t){return Nr(t,d)},hr.cloneDeep=function(t){return Nr(t,g|d)},hr.cloneDeepWith=function(t,e){return Nr(t,g|d,e="function"==typeof e?e:o)},hr.cloneWith=function(t,e){return Nr(t,d,e="function"==typeof e?e:o)},hr.conformsTo=function(t,e){return null==e||Fr(t,e,iA(e))},hr.deburr=pA,hr.defaultTo=function(t,e){return null==t||t!=t?e:t},hr.divide=zA,hr.endsWith=function(t,e,n){t=Ks(t),e=Fi(e);var r=t.length,i=n=n===o?r:Rr(Hs(n),0,r);return(n-=e.length)>=0&&t.slice(n,i)==e},hr.eq=hs,hr.escape=function(t){return(t=Ks(t))&&Qt.test(t)?t.replace(Bt,Bn):t},hr.escapeRegExp=function(t){return(t=Ks(t))&&Lt.test(t)?t.replace(St,"\\$&"):t},hr.every=function(t,e,n){var r=Is(t)?Ve:Hr;return n&&Jo(t,e,n)&&(e=o),r(t,Fo(e,3))},hr.find=Ya,hr.findIndex=pa,hr.findKey=function(t,e){return an(t,Fo(e,3),Jr)},hr.findLast=Pa,hr.findLastIndex=Ia,hr.findLastKey=function(t,e){return an(t,Fo(e,3),qr)},hr.floor=JA,hr.forEach=Ka,hr.forEachRight=za,hr.forIn=function(t,e){return null==t?t:Kr(t,Fo(e,3),oA)},hr.forInRight=function(t,e){return null==t?t:zr(t,Fo(e,3),oA)},hr.forOwn=function(t,e){return t&&Jr(t,Fo(e,3))},hr.forOwnRight=function(t,e){return t&&qr(t,Fo(e,3))},hr.get=$s,hr.gt=ds,hr.gte=fs,hr.has=function(t,e){return null!=t&&Yo(t,e,ti)},hr.hasIn=tA,hr.head=ma,hr.identity=MA,hr.includes=function(t,e,n,r){t=ms(t)?t:hA(t),n=n&&!r?Hs(n):0;var i=t.length;return n<0&&(n=Kn(i+n,0)),Rs(t)?n<=i&&t.indexOf(e,n)>-1:!!i&&An(t,e,n)>-1},hr.indexOf=function(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=null==n?0:Hs(n);return i<0&&(i=Kn(r+i,0)),An(t,e,i)},hr.inRange=function(t,e,n){return e=Gs(e),n===o?(n=e,e=0):n=Gs(n),function(t,e,n){return t>=zn(e,n)&&t<Kn(e,n)}(t=Ys(t),e,n)},hr.invoke=rA,hr.isArguments=ps,hr.isArray=Is,hr.isArrayBuffer=Cs,hr.isArrayLike=ms,hr.isArrayLikeObject=ys,hr.isBoolean=function(t){return!0===t||!1===t||_s(t)&&Zr(t)==P},hr.isBuffer=Es,hr.isDate=vs,hr.isElement=function(t){return _s(t)&&1===t.nodeType&&!ks(t)},hr.isEmpty=function(t){if(null==t)return!0;if(ms(t)&&(Is(t)||"string"==typeof t||"function"==typeof t.splice||Es(t)||Fs(t)||ps(t)))return!t.length;var e=jo(t);if(e==W||e==nt)return!t.size;if(Xo(t))return!li(t).length;for(var n in t)if(ce.call(t,n))return!1;return!0},hr.isEqual=function(t,e){return oi(t,e)},hr.isEqualWith=function(t,e,n){var r=(n="function"==typeof n?n:o)?n(t,e):o;return r===o?oi(t,e,o,n):!!r},hr.isError=Bs,hr.isFinite=function(t){return"number"==typeof t&&jn(t)},hr.isFunction=bs,hr.isInteger=Qs,hr.isLength=ws,hr.isMap=Ds,hr.isMatch=function(t,e){return t===e||ai(t,e,Uo(e))},hr.isMatchWith=function(t,e,n){return n="function"==typeof n?n:o,ai(t,e,Uo(e),n)},hr.isNaN=function(t){return Ms(t)&&t!=+t},hr.isNative=function(t){if(Wo(t))throw new Zt(s);return si(t)},hr.isNil=function(t){return null==t},hr.isNull=function(t){return null===t},hr.isNumber=Ms,hr.isObject=xs,hr.isObjectLike=_s,hr.isPlainObject=ks,hr.isRegExp=Ss,hr.isSafeInteger=function(t){return Qs(t)&&t>=-R&&t<=R},hr.isSet=Ls,hr.isString=Rs,hr.isSymbol=Ns,hr.isTypedArray=Fs,hr.isUndefined=function(t){return t===o},hr.isWeakMap=function(t){return _s(t)&&jo(t)==at},hr.isWeakSet=function(t){return _s(t)&&Zr(t)==st},hr.join=function(t,e){return null==t?"":Yn.call(t,e)},hr.kebabCase=IA,hr.last=Ba,hr.lastIndexOf=function(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=r;return n!==o&&(i=(i=Hs(n))<0?Kn(r+i,0):zn(i,r-1)),e==e?function(t,e,n){for(var r=n+1;r--;)if(t[r]===e)return r;return r}(t,e,i):sn(t,cn,i,!0)},hr.lowerCase=CA,hr.lowerFirst=mA,hr.lt=Ts,hr.lte=Us,hr.max=function(t){return t&&t.length?jr(t,MA,$r):o},hr.maxBy=function(t,e){return t&&t.length?jr(t,Fo(e,2),$r):o},hr.mean=function(t){return un(t,MA)},hr.meanBy=function(t,e){return un(t,Fo(e,2))},hr.min=function(t){return t&&t.length?jr(t,MA,ui):o},hr.minBy=function(t,e){return t&&t.length?jr(t,Fo(e,2),ui):o},hr.stubArray=jA,hr.stubFalse=YA,hr.stubObject=function(){return{}},hr.stubString=function(){return""},hr.stubTrue=function(){return!0},hr.multiply=VA,hr.nth=function(t,e){return t&&t.length?pi(t,Hs(e)):o},hr.noConflict=function(){return Le._===this&&(Le._=fe),this},hr.noop=NA,hr.now=$a,hr.pad=function(t,e,n){t=Ks(t);var r=(e=Hs(e))?kn(t):0;if(!e||r>=e)return t;var i=(e-r)/2;return Co(On(i),n)+t+Co(Un(i),n)},hr.padEnd=function(t,e,n){t=Ks(t);var r=(e=Hs(e))?kn(t):0;return e&&r<e?t+Co(e-r,n):t},hr.padStart=function(t,e,n){t=Ks(t);var r=(e=Hs(e))?kn(t):0;return e&&r<e?Co(e-r,n)+t:t},hr.parseInt=function(t,e,n){return n||null==e?e=0:e&&(e=+e),qn(Ks(t).replace(Nt,""),e||0)},hr.random=function(t,e,n){if(n&&"boolean"!=typeof n&&Jo(t,e,n)&&(e=n=o),n===o&&("boolean"==typeof e?(n=e,e=o):"boolean"==typeof t&&(n=t,t=o)),t===o&&e===o?(t=0,e=1):(t=Gs(t),e===o?(e=t,t=0):e=Gs(e)),t>e){var r=t;t=e,e=r}if(n||t%1||e%1){var i=Vn();return zn(t+i*(e-t+De("1e-"+((i+"").length-1))),e)}return Ei(t,e)},hr.reduce=function(t,e,n){var r=Is(t)?en:dn,i=arguments.length<3;return r(t,Fo(e,4),n,i,Or)},hr.reduceRight=function(t,e,n){var r=Is(t)?nn:dn,i=arguments.length<3;return r(t,Fo(e,4),n,i,Gr)},hr.repeat=function(t,e,n){return e=(n?Jo(t,e,n):e===o)?1:Hs(e),vi(Ks(t),e)},hr.replace=function(){var t=arguments,e=Ks(t[0]);return t.length<3?e:e.replace(t[1],t[2])},hr.result=function(t,e,n){var r=-1,i=(e=zi(e,t)).length;for(i||(i=1,t=o);++r<i;){var a=null==t?o:t[ca(e[r])];a===o&&(r=i,a=n),t=bs(a)?a.call(t):a}return t},hr.round=WA,hr.runInContext=t,hr.sample=function(t){return(Is(t)?br:bi)(t)},hr.size=function(t){if(null==t)return 0;if(ms(t))return Rs(t)?kn(t):t.length;var e=jo(t);return e==W||e==nt?t.size:li(t).length},hr.snakeCase=yA,hr.some=function(t,e,n){var r=Is(t)?rn:ki;return n&&Jo(t,e,n)&&(e=o),r(t,Fo(e,3))},hr.sortedIndex=function(t,e){return Si(t,e)},hr.sortedIndexBy=function(t,e,n){return Li(t,e,Fo(n,2))},hr.sortedIndexOf=function(t,e){var n=null==t?0:t.length;if(n){var r=Si(t,e);if(r<n&&hs(t[r],e))return r}return-1},hr.sortedLastIndex=function(t,e){return Si(t,e,!0)},hr.sortedLastIndexBy=function(t,e,n){return Li(t,e,Fo(n,2),!0)},hr.sortedLastIndexOf=function(t,e){if(null!=t&&t.length){var n=Si(t,e,!0)-1;if(hs(t[n],e))return n}return-1},hr.startCase=EA,hr.startsWith=function(t,e,n){return t=Ks(t),n=null==n?0:Rr(Hs(n),0,t.length),e=Fi(e),t.slice(n,n+e.length)==e},hr.subtract=XA,hr.sum=function(t){return t&&t.length?fn(t,MA):0},hr.sumBy=function(t,e){return t&&t.length?fn(t,Fo(e,2)):0},hr.template=function(t,e,n){var r=hr.templateSettings;n&&Jo(t,e,n)&&(e=o),t=Ks(t),e=qs({},e,r,wo);var i,a,s=qs({},e.imports,r.imports,wo),A=iA(s),l=Cn(s,A),c=0,u=e.interpolate||Wt,g="__p += '",h=ne((e.escape||Wt).source+"|"+u.source+"|"+(u===_t?jt:Wt).source+"|"+(e.evaluate||Wt).source+"|$","g"),d="//# sourceURL="+("sourceURL"in e?e.sourceURL:"lodash.templateSources["+ ++Qe+"]")+"\n";t.replace(h,function(e,n,r,o,s,A){return r||(r=o),g+=t.slice(c,A).replace(Xt,bn),n&&(i=!0,g+="' +\n__e("+n+") +\n'"),s&&(a=!0,g+="';\n"+s+";\n__p += '"),r&&(g+="' +\n((__t = ("+r+")) == null ? '' : __t) +\n'"),c=A+e.length,e}),g+="';\n";var f=e.variable;f||(g="with (obj) {\n"+g+"\n}\n"),g=(a?g.replace(mt,""):g).replace(yt,"$1").replace(Et,"$1;"),g="function("+(f||"obj")+") {\n"+(f?"":"obj || (obj = {});\n")+"var __t, __p = ''"+(i?", __e = _.escape":"")+(a?", __j = Array.prototype.join;\nfunction print() { __p += __j.call(arguments, '') }\n":";\n")+g+"return __p\n}";var p=QA(function(){return $t(A,d+"return "+g).apply(o,l)});if(p.source=g,Bs(p))throw p;return p},hr.times=function(t,e){if((t=Hs(t))<1||t>R)return[];var n=T,r=zn(t,T);e=Fo(e),t-=T;for(var i=pn(r,e);++n<t;)e(n);return i},hr.toFinite=Gs,hr.toInteger=Hs,hr.toLength=js,hr.toLower=function(t){return Ks(t).toLowerCase()},hr.toNumber=Ys,hr.toSafeInteger=function(t){return t?Rr(Hs(t),-R,R):0===t?t:0},hr.toString=Ks,hr.toUpper=function(t){return Ks(t).toUpperCase()},hr.trim=function(t,e,n){if((t=Ks(t))&&(n||e===o))return t.replace(Rt,"");if(!t||!(e=Fi(e)))return t;var r=Sn(t),i=Sn(e);return qi(r,yn(r,i),En(r,i)+1).join("")},hr.trimEnd=function(t,e,n){if((t=Ks(t))&&(n||e===o))return t.replace(Ft,"");if(!t||!(e=Fi(e)))return t;var r=Sn(t);return qi(r,0,En(r,Sn(e))+1).join("")},hr.trimStart=function(t,e,n){if((t=Ks(t))&&(n||e===o))return t.replace(Nt,"");if(!t||!(e=Fi(e)))return t;var r=Sn(t);return qi(r,yn(r,Sn(e))).join("")},hr.truncate=function(t,e){var n=x,r=_;if(xs(e)){var i="separator"in e?e.separator:i;n="length"in e?Hs(e.length):n,r="omission"in e?Fi(e.omission):r}var a=(t=Ks(t)).length;if(Qn(t)){var s=Sn(t);a=s.length}if(n>=a)return t;var A=n-kn(r);if(A<1)return r;var l=s?qi(s,0,A).join(""):t.slice(0,A);if(i===o)return l+r;if(s&&(A+=l.length-A),Ss(i)){if(t.slice(A).search(i)){var c,u=l;for(i.global||(i=ne(i.source,Ks(Yt.exec(i))+"g")),i.lastIndex=0;c=i.exec(u);)var g=c.index;l=l.slice(0,g===o?A:g)}}else if(t.indexOf(Fi(i),A)!=A){var h=l.lastIndexOf(i);h>-1&&(l=l.slice(0,h))}return l+r},hr.unescape=function(t){return(t=Ks(t))&&bt.test(t)?t.replace(vt,Ln):t},hr.uniqueId=function(t){var e=++ue;return Ks(t)+e},hr.upperCase=vA,hr.upperFirst=BA,hr.each=Ka,hr.eachRight=za,hr.first=ma,RA(hr,(qA={},Jr(hr,function(t,e){ce.call(hr.prototype,e)||(qA[e]=t)}),qA),{chain:!1}),hr.VERSION="4.17.11",Je(["bind","bindKey","curry","curryRight","partial","partialRight"],function(t){hr[t].placeholder=hr}),Je(["drop","take"],function(t,e){Ir.prototype[t]=function(n){n=n===o?1:Kn(Hs(n),0);var r=this.__filtered__&&!e?new Ir(this):this.clone();return r.__filtered__?r.__takeCount__=zn(n,r.__takeCount__):r.__views__.push({size:zn(n,T),type:t+(r.__dir__<0?"Right":"")}),r},Ir.prototype[t+"Right"]=function(e){return this.reverse()[t](e).reverse()}}),Je(["filter","map","takeWhile"],function(t,e){var n=e+1,r=n==k||3==n;Ir.prototype[t]=function(t){var e=this.clone();return e.__iteratees__.push({iteratee:Fo(t,3),type:n}),e.__filtered__=e.__filtered__||r,e}}),Je(["head","last"],function(t,e){var n="take"+(e?"Right":"");Ir.prototype[t]=function(){return this[n](1).value()[0]}}),Je(["initial","tail"],function(t,e){var n="drop"+(e?"":"Right");Ir.prototype[t]=function(){return this.__filtered__?new Ir(this):this[n](1)}}),Ir.prototype.compact=function(){return this.filter(MA)},Ir.prototype.find=function(t){return this.filter(t).head()},Ir.prototype.findLast=function(t){return this.reverse().find(t)},Ir.prototype.invokeMap=Bi(function(t,e){return"function"==typeof t?new Ir(this):this.map(function(n){return ri(n,t,e)})}),Ir.prototype.reject=function(t){return this.filter(As(Fo(t)))},Ir.prototype.slice=function(t,e){t=Hs(t);var n=this;return n.__filtered__&&(t>0||e<0)?new Ir(n):(t<0?n=n.takeRight(-t):t&&(n=n.drop(t)),e!==o&&(n=(e=Hs(e))<0?n.dropRight(-e):n.take(e-t)),n)},Ir.prototype.takeRightWhile=function(t){return this.reverse().takeWhile(t).reverse()},Ir.prototype.toArray=function(){return this.take(T)},Jr(Ir.prototype,function(t,e){var n=/^(?:filter|find|map|reject)|While$/.test(e),r=/^(?:head|last)$/.test(e),i=hr[r?"take"+("last"==e?"Right":""):e],a=r||/^find/.test(e);i&&(hr.prototype[e]=function(){var e=this.__wrapped__,s=r?[1]:arguments,A=e instanceof Ir,l=s[0],c=A||Is(e),u=function(t){var e=i.apply(hr,tn([t],s));return r&&g?e[0]:e};c&&n&&"function"==typeof l&&1!=l.length&&(A=c=!1);var g=this.__chain__,h=!!this.__actions__.length,d=a&&!g,f=A&&!h;if(!a&&c){e=f?e:new Ir(this);var p=t.apply(e,s);return p.__actions__.push({func:Ga,args:[u],thisArg:o}),new pr(p,g)}return d&&f?t.apply(this,s):(p=this.thru(u),d?r?p.value()[0]:p.value():p)})}),Je(["pop","push","shift","sort","splice","unshift"],function(t){var e=oe[t],n=/^(?:push|sort|unshift)$/.test(t)?"tap":"thru",r=/^(?:pop|shift)$/.test(t);hr.prototype[t]=function(){var t=arguments;if(r&&!this.__chain__){var i=this.value();return e.apply(Is(i)?i:[],t)}return this[n](function(n){return e.apply(Is(n)?n:[],t)})}}),Jr(Ir.prototype,function(t,e){var n=hr[e];if(n){var r=n.name+"";(ir[r]||(ir[r]=[])).push({name:e,func:n})}}),ir[ho(o,C).name]=[{name:"wrapper",func:o}],Ir.prototype.clone=function(){var t=new Ir(this.__wrapped__);return t.__actions__=no(this.__actions__),t.__dir__=this.__dir__,t.__filtered__=this.__filtered__,t.__iteratees__=no(this.__iteratees__),t.__takeCount__=this.__takeCount__,t.__views__=no(this.__views__),t},Ir.prototype.reverse=function(){if(this.__filtered__){var t=new Ir(this);t.__dir__=-1,t.__filtered__=!0}else(t=this.clone()).__dir__*=-1;return t},Ir.prototype.value=function(){var t=this.__wrapped__.value(),e=this.__dir__,n=Is(t),r=e<0,i=n?t.length:0,o=function(t,e,n){for(var r=-1,i=n.length;++r<i;){var o=n[r],a=o.size;switch(o.type){case"drop":t+=a;break;case"dropRight":e-=a;break;case"take":e=zn(e,t+a);break;case"takeRight":t=Kn(t,e-a)}}return{start:t,end:e}}(0,i,this.__views__),a=o.start,s=o.end,A=s-a,l=r?s:a-1,c=this.__iteratees__,u=c.length,g=0,h=zn(A,this.__takeCount__);if(!n||!r&&i==A&&h==A)return Hi(t,this.__actions__);var d=[];t:for(;A--&&g<h;){for(var f=-1,p=t[l+=e];++f<u;){var I=c[f],C=I.iteratee,m=I.type,y=C(p);if(m==S)p=y;else if(!y){if(m==k)continue t;break t}}d[g++]=p}return d},hr.prototype.at=Ha,hr.prototype.chain=function(){return Oa(this)},hr.prototype.commit=function(){return new pr(this.value(),this.__chain__)},hr.prototype.next=function(){this.__values__===o&&(this.__values__=Os(this.value()));var t=this.__index__>=this.__values__.length;return{done:t,value:t?o:this.__values__[this.__index__++]}},hr.prototype.plant=function(t){for(var e,n=this;n instanceof fr;){var r=ga(n);r.__index__=0,r.__values__=o,e?i.__wrapped__=r:e=r;var i=r;n=n.__wrapped__}return i.__wrapped__=t,e},hr.prototype.reverse=function(){var t=this.__wrapped__;if(t instanceof Ir){var e=t;return this.__actions__.length&&(e=new Ir(this)),(e=e.reverse()).__actions__.push({func:Ga,args:[xa],thisArg:o}),new pr(e,this.__chain__)}return this.thru(xa)},hr.prototype.toJSON=hr.prototype.valueOf=hr.prototype.value=function(){return Hi(this.__wrapped__,this.__actions__)},hr.prototype.first=hr.prototype.head,Ue&&(hr.prototype[Ue]=function(){return this}),hr}();Le._=Rn,(i=function(){return Rn}.call(e,n,e,r))===o||(r.exports=i)}).call(this)}).call(this,n(37),n(98)(t))},function(t,e,n){var r=n(110),i=n(71),o=n(315),a=n(142),s=n(121),A=function(t,e,n){var l,c,u,g=t&A.F,h=t&A.G,d=t&A.S,f=t&A.P,p=t&A.B,I=t&A.W,C=h?i:i[e]||(i[e]={}),m=C.prototype,y=h?r:d?r[e]:(r[e]||{}).prototype;for(l in h&&(n=e),n)(c=!g&&y&&void 0!==y[l])&&s(C,l)||(u=c?y[l]:n[l],C[l]=h&&"function"!=typeof y[l]?n[l]:p&&c?o(u,r):I&&y[l]==u?function(t){var e=function(e,n,r){if(this instanceof t){switch(arguments.length){case 0:return new t;case 1:return new t(e);case 2:return new t(e,n)}return new t(e,n,r)}return t.apply(this,arguments)};return e.prototype=t.prototype,e}(u):f&&"function"==typeof u?o(Function.call,u):u,f&&((C.virtual||(C.virtual={}))[l]=u,t&A.R&&m&&!m[l]&&a(m,l,u)))};A.F=1,A.G=2,A.S=4,A.P=8,A.B=16,A.W=32,A.U=64,A.R=128,t.exports=A},function(t,e){var n=t.exports="undefined"!=typeof window&&window.Math==Math?window:"undefined"!=typeof self&&self.Math==Math?self:Function("return this")();"number"==typeof __g&&(__g=n)},function(t,e,n){var r=n(143),i=n(638),o=n(316),a=Object.defineProperty;e.f=n(112)?Object.defineProperty:function(t,e,n){if(r(t),e=o(e,!0),r(n),i)try{return a(t,e,n)}catch(t){}if("get"in n||"set"in n)throw TypeError("Accessors not supported!");return"value"in n&&(t[e]=n.value),t}},function(t,e,n){t.exports=!n(145)(function(){return 7!=Object.defineProperty({},"a",{get:function(){return 7}}).a})},function(t,e,n){"use strict";n.r(e);var r=n(2),i=n(26),o=n(24);function a(t){return function(e){return e.lift(new s(t))}}var s=function(){function t(t){this.durationSelector=t}return t.prototype.call=function(t,e){return e.subscribe(new A(t,this.durationSelector))},t}(),A=function(t){function e(e,n){var r=t.call(this,e)||this;return r.durationSelector=n,r.hasValue=!1,r}return r.__extends(e,t),e.prototype._next=function(t){if(this.value=t,this.hasValue=!0,!this.throttled){var e=void 0;try{e=(0,this.durationSelector)(t)}catch(t){return this.destination.error(t)}var n=Object(o.a)(this,e);!n||n.closed?this.clearThrottle():this.add(this.throttled=n)}},e.prototype.clearThrottle=function(){var t=this.value,e=this.hasValue,n=this.throttled;n&&(this.remove(n),this.throttled=null,n.unsubscribe()),e&&(this.value=null,this.hasValue=!1,this.destination.next(t))},e.prototype.notifyNext=function(t,e,n,r){this.clearThrottle()},e.prototype.notifyComplete=function(){this.clearThrottle()},e}(i.a),l=n(39),c=n(253);function u(t,e){return void 0===e&&(e=l.a),a(function(){return Object(c.a)(t,e)})}function g(t){return function(e){return e.lift(new h(t))}}var h=function(){function t(t){this.closingNotifier=t}return t.prototype.call=function(t,e){return e.subscribe(new d(t,this.closingNotifier))},t}(),d=function(t){function e(e,n){var r=t.call(this,e)||this;return r.buffer=[],r.add(Object(o.a)(r,n)),r}return r.__extends(e,t),e.prototype._next=function(t){this.buffer.push(t)},e.prototype.notifyNext=function(t,e,n,r,i){var o=this.buffer;this.buffer=[],this.destination.next(o)},e}(i.a),f=n(11);function p(t,e){return void 0===e&&(e=null),function(n){return n.lift(new I(t,e))}}var I=function(){function t(t,e){this.bufferSize=t,this.startBufferEvery=e,this.subscriberClass=e&&t!==e?m:C}return t.prototype.call=function(t,e){return e.subscribe(new this.subscriberClass(t,this.bufferSize,this.startBufferEvery))},t}(),C=function(t){function e(e,n){var r=t.call(this,e)||this;return r.bufferSize=n,r.buffer=[],r}return r.__extends(e,t),e.prototype._next=function(t){var e=this.buffer;e.push(t),e.length==this.bufferSize&&(this.destination.next(e),this.buffer=[])},e.prototype._complete=function(){var e=this.buffer;e.length>0&&this.destination.next(e),t.prototype._complete.call(this)},e}(f.a),m=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.bufferSize=n,i.startBufferEvery=r,i.buffers=[],i.count=0,i}return r.__extends(e,t),e.prototype._next=function(t){var e=this.bufferSize,n=this.startBufferEvery,r=this.buffers,i=this.count;this.count++,i%n==0&&r.push([]);for(var o=r.length;o--;){var a=r[o];a.push(t),a.length===e&&(r.splice(o,1),this.destination.next(a))}},e.prototype._complete=function(){for(var e=this.buffers,n=this.destination;e.length>0;){var r=e.shift();r.length>0&&n.next(r)}t.prototype._complete.call(this)},e}(f.a),y=n(45);function E(t){var e=arguments.length,n=l.a;Object(y.a)(arguments[arguments.length-1])&&(n=arguments[arguments.length-1],e--);var r=null;e>=2&&(r=arguments[1]);var i=Number.POSITIVE_INFINITY;return e>=3&&(i=arguments[2]),function(e){return e.lift(new v(t,r,i,n))}}var v=function(){function t(t,e,n,r){this.bufferTimeSpan=t,this.bufferCreationInterval=e,this.maxBufferSize=n,this.scheduler=r}return t.prototype.call=function(t,e){return e.subscribe(new b(t,this.bufferTimeSpan,this.bufferCreationInterval,this.maxBufferSize,this.scheduler))},t}(),B=function(){return function(){this.buffer=[]}}(),b=function(t){function e(e,n,r,i,o){var a=t.call(this,e)||this;a.bufferTimeSpan=n,a.bufferCreationInterval=r,a.maxBufferSize=i,a.scheduler=o,a.contexts=[];var s=a.openContext();if(a.timespanOnly=null==r||r<0,a.timespanOnly){var A={subscriber:a,context:s,bufferTimeSpan:n};a.add(s.closeAction=o.schedule(Q,n,A))}else{var l={subscriber:a,context:s},c={bufferTimeSpan:n,bufferCreationInterval:r,subscriber:a,scheduler:o};a.add(s.closeAction=o.schedule(x,n,l)),a.add(o.schedule(w,r,c))}return a}return r.__extends(e,t),e.prototype._next=function(t){for(var e,n=this.contexts,r=n.length,i=0;i<r;i++){var o=n[i],a=o.buffer;a.push(t),a.length==this.maxBufferSize&&(e=o)}e&&this.onBufferFull(e)},e.prototype._error=function(e){this.contexts.length=0,t.prototype._error.call(this,e)},e.prototype._complete=function(){for(var e=this.contexts,n=this.destination;e.length>0;){var r=e.shift();n.next(r.buffer)}t.prototype._complete.call(this)},e.prototype._unsubscribe=function(){this.contexts=null},e.prototype.onBufferFull=function(t){this.closeContext(t);var e=t.closeAction;if(e.unsubscribe(),this.remove(e),!this.closed&&this.timespanOnly){t=this.openContext();var n=this.bufferTimeSpan,r={subscriber:this,context:t,bufferTimeSpan:n};this.add(t.closeAction=this.scheduler.schedule(Q,n,r))}},e.prototype.openContext=function(){var t=new B;return this.contexts.push(t),t},e.prototype.closeContext=function(t){this.destination.next(t.buffer);var e=this.contexts;(e?e.indexOf(t):-1)>=0&&e.splice(e.indexOf(t),1)},e}(f.a);function Q(t){var e=t.subscriber,n=t.context;n&&e.closeContext(n),e.closed||(t.context=e.openContext(),t.context.closeAction=this.schedule(t,t.bufferTimeSpan))}function w(t){var e=t.bufferCreationInterval,n=t.bufferTimeSpan,r=t.subscriber,i=t.scheduler,o=r.openContext();r.closed||(r.add(o.closeAction=i.schedule(x,n,{subscriber:r,context:o})),this.schedule(t,e))}function x(t){var e=t.subscriber,n=t.context;e.closeContext(n)}var _=n(32);function D(t,e){return function(n){return n.lift(new M(t,e))}}var M=function(){function t(t,e){this.openings=t,this.closingSelector=e}return t.prototype.call=function(t,e){return e.subscribe(new k(t,this.openings,this.closingSelector))},t}(),k=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.openings=n,i.closingSelector=r,i.contexts=[],i.add(Object(o.a)(i,n)),i}return r.__extends(e,t),e.prototype._next=function(t){for(var e=this.contexts,n=e.length,r=0;r<n;r++)e[r].buffer.push(t)},e.prototype._error=function(e){for(var n=this.contexts;n.length>0;){var r=n.shift();r.subscription.unsubscribe(),r.buffer=null,r.subscription=null}this.contexts=null,t.prototype._error.call(this,e)},e.prototype._complete=function(){for(var e=this.contexts;e.length>0;){var n=e.shift();this.destination.next(n.buffer),n.subscription.unsubscribe(),n.buffer=null,n.subscription=null}this.contexts=null,t.prototype._complete.call(this)},e.prototype.notifyNext=function(t,e,n,r,i){t?this.closeBuffer(t):this.openBuffer(e)},e.prototype.notifyComplete=function(t){this.closeBuffer(t.context)},e.prototype.openBuffer=function(t){try{var e=this.closingSelector.call(this,t);e&&this.trySubscribe(e)}catch(t){this._error(t)}},e.prototype.closeBuffer=function(t){var e=this.contexts;if(e&&t){var n=t.buffer,r=t.subscription;this.destination.next(n),e.splice(e.indexOf(t),1),this.remove(r),r.unsubscribe()}},e.prototype.trySubscribe=function(t){var e=this.contexts,n=new _.a,r={buffer:[],subscription:n};e.push(r);var i=Object(o.a)(this,t,r);!i||i.closed?this.closeBuffer(r):(i.context=r,this.add(i),n.add(i))},e}(i.a);function S(t){return function(e){return e.lift(new L(t))}}var L=function(){function t(t){this.closingSelector=t}return t.prototype.call=function(t,e){return e.subscribe(new R(t,this.closingSelector))},t}(),R=function(t){function e(e,n){var r=t.call(this,e)||this;return r.closingSelector=n,r.subscribing=!1,r.openBuffer(),r}return r.__extends(e,t),e.prototype._next=function(t){this.buffer.push(t)},e.prototype._complete=function(){var e=this.buffer;e&&this.destination.next(e),t.prototype._complete.call(this)},e.prototype._unsubscribe=function(){this.buffer=null,this.subscribing=!1},e.prototype.notifyNext=function(t,e,n,r,i){this.openBuffer()},e.prototype.notifyComplete=function(){this.subscribing?this.complete():this.openBuffer()},e.prototype.openBuffer=function(){var t=this.closingSubscription;t&&(this.remove(t),t.unsubscribe());var e,n=this.buffer;this.buffer&&this.destination.next(n),this.buffer=[];try{e=(0,this.closingSelector)()}catch(t){return this.error(t)}t=new _.a,this.closingSubscription=t,this.add(t),this.subscribing=!0,t.add(Object(o.a)(this,e)),this.subscribing=!1},e}(i.a),N=n(62);function F(t){return function(e){var n=new T(t),r=e.lift(n);return n.caught=r}}var T=function(){function t(t){this.selector=t}return t.prototype.call=function(t,e){return e.subscribe(new U(t,this.selector,this.caught))},t}(),U=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.selector=n,i.caught=r,i}return r.__extends(e,t),e.prototype.error=function(e){if(!this.isStopped){var n=void 0;try{n=this.selector(e,this.caught)}catch(e){return void t.prototype.error.call(this,e)}this._unsubscribeAndRecycle();var r=new N.a(this,void 0,void 0);this.add(r),Object(o.a)(this,n,void 0,void 0,r)}},e}(i.a),O=n(151);function G(t){return function(e){return e.lift(new O.a(t))}}var H=n(38),j=n(55);function Y(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];var n=null;return"function"==typeof t[t.length-1]&&(n=t.pop()),1===t.length&&Object(H.a)(t[0])&&(t=t[0].slice()),function(e){return e.lift.call(Object(j.a)([e].concat(t)),new O.a(n))}}var P=n(101);function K(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return function(e){return e.lift.call(P.a.apply(void 0,[e].concat(t)))}}var z=n(250),J=n(84);function q(t,e){return Object(J.a)(t,e,1)}function V(t,e){return q(function(){return t},e)}function W(t){return function(e){return e.lift(new X(t,e))}}var X=function(){function t(t,e){this.predicate=t,this.source=e}return t.prototype.call=function(t,e){return e.subscribe(new Z(t,this.predicate,this.source))},t}(),Z=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.predicate=n,i.source=r,i.count=0,i.index=0,i}return r.__extends(e,t),e.prototype._next=function(t){this.predicate?this._tryPredicate(t):this.count++},e.prototype._tryPredicate=function(t){var e;try{e=this.predicate(t,this.index++,this.source)}catch(t){return void this.destination.error(t)}e&&this.count++},e.prototype._complete=function(){this.destination.next(this.count),this.destination.complete()},e}(f.a);function $(t){return function(e){return e.lift(new tt(t))}}var tt=function(){function t(t){this.durationSelector=t}return t.prototype.call=function(t,e){return e.subscribe(new et(t,this.durationSelector))},t}(),et=function(t){function e(e,n){var r=t.call(this,e)||this;return r.durationSelector=n,r.hasValue=!1,r.durationSubscription=null,r}return r.__extends(e,t),e.prototype._next=function(t){try{var e=this.durationSelector.call(this,t);e&&this._tryNext(t,e)}catch(t){this.destination.error(t)}},e.prototype._complete=function(){this.emitValue(),this.destination.complete()},e.prototype._tryNext=function(t,e){var n=this.durationSubscription;this.value=t,this.hasValue=!0,n&&(n.unsubscribe(),this.remove(n)),(n=Object(o.a)(this,e))&&!n.closed&&this.add(this.durationSubscription=n)},e.prototype.notifyNext=function(t,e,n,r,i){this.emitValue()},e.prototype.notifyComplete=function(){this.emitValue()},e.prototype.emitValue=function(){if(this.hasValue){var e=this.value,n=this.durationSubscription;n&&(this.durationSubscription=null,n.unsubscribe(),this.remove(n)),this.value=null,this.hasValue=!1,t.prototype._next.call(this,e)}},e}(i.a);function nt(t,e){return void 0===e&&(e=l.a),function(n){return n.lift(new rt(t,e))}}var rt=function(){function t(t,e){this.dueTime=t,this.scheduler=e}return t.prototype.call=function(t,e){return e.subscribe(new it(t,this.dueTime,this.scheduler))},t}(),it=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.dueTime=n,i.scheduler=r,i.debouncedSubscription=null,i.lastValue=null,i.hasValue=!1,i}return r.__extends(e,t),e.prototype._next=function(t){this.clearDebounce(),this.lastValue=t,this.hasValue=!0,this.add(this.debouncedSubscription=this.scheduler.schedule(ot,this.dueTime,this))},e.prototype._complete=function(){this.debouncedNext(),this.destination.complete()},e.prototype.debouncedNext=function(){if(this.clearDebounce(),this.hasValue){var t=this.lastValue;this.lastValue=null,this.hasValue=!1,this.destination.next(t)}},e.prototype.clearDebounce=function(){var t=this.debouncedSubscription;null!==t&&(this.remove(t),t.unsubscribe(),this.debouncedSubscription=null)},e}(f.a);function ot(t){t.debouncedNext()}function at(t){return void 0===t&&(t=null),function(e){return e.lift(new st(t))}}var st=function(){function t(t){this.defaultValue=t}return t.prototype.call=function(t,e){return e.subscribe(new At(t,this.defaultValue))},t}(),At=function(t){function e(e,n){var r=t.call(this,e)||this;return r.defaultValue=n,r.isEmpty=!0,r}return r.__extends(e,t),e.prototype._next=function(t){this.isEmpty=!1,this.destination.next(t)},e.prototype._complete=function(){this.isEmpty&&this.destination.next(this.defaultValue),this.destination.complete()},e}(f.a);function lt(t){return t instanceof Date&&!isNaN(+t)}var ct=n(69);function ut(t,e){void 0===e&&(e=l.a);var n=lt(t)?+t-e.now():Math.abs(t);return function(t){return t.lift(new gt(n,e))}}var gt=function(){function t(t,e){this.delay=t,this.scheduler=e}return t.prototype.call=function(t,e){return e.subscribe(new ht(t,this.delay,this.scheduler))},t}(),ht=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.delay=n,i.scheduler=r,i.queue=[],i.active=!1,i.errored=!1,i}return r.__extends(e,t),e.dispatch=function(t){for(var e=t.source,n=e.queue,r=t.scheduler,i=t.destination;n.length>0&&n[0].time-r.now()<=0;)n.shift().notification.observe(i);if(n.length>0){var o=Math.max(0,n[0].time-r.now());this.schedule(t,o)}else this.unsubscribe(),e.active=!1},e.prototype._schedule=function(t){this.active=!0,this.destination.add(t.schedule(e.dispatch,this.delay,{source:this,destination:this.destination,scheduler:t}))},e.prototype.scheduleNotification=function(t){if(!0!==this.errored){var e=this.scheduler,n=new dt(e.now()+this.delay,t);this.queue.push(n),!1===this.active&&this._schedule(e)}},e.prototype._next=function(t){this.scheduleNotification(ct.a.createNext(t))},e.prototype._error=function(t){this.errored=!0,this.queue=[],this.destination.error(t),this.unsubscribe()},e.prototype._complete=function(){this.scheduleNotification(ct.a.createComplete()),this.unsubscribe()},e}(f.a),dt=function(){return function(t,e){this.time=t,this.notification=e}}(),ft=n(21);function pt(t,e){return e?function(n){return new mt(n,e).lift(new It(t))}:function(e){return e.lift(new It(t))}}var It=function(){function t(t){this.delayDurationSelector=t}return t.prototype.call=function(t,e){return e.subscribe(new Ct(t,this.delayDurationSelector))},t}(),Ct=function(t){function e(e,n){var r=t.call(this,e)||this;return r.delayDurationSelector=n,r.completed=!1,r.delayNotifierSubscriptions=[],r.index=0,r}return r.__extends(e,t),e.prototype.notifyNext=function(t,e,n,r,i){this.destination.next(t),this.removeSubscription(i),this.tryComplete()},e.prototype.notifyError=function(t,e){this._error(t)},e.prototype.notifyComplete=function(t){var e=this.removeSubscription(t);e&&this.destination.next(e),this.tryComplete()},e.prototype._next=function(t){var e=this.index++;try{var n=this.delayDurationSelector(t,e);n&&this.tryDelay(n,t)}catch(t){this.destination.error(t)}},e.prototype._complete=function(){this.completed=!0,this.tryComplete(),this.unsubscribe()},e.prototype.removeSubscription=function(t){t.unsubscribe();var e=this.delayNotifierSubscriptions.indexOf(t);return-1!==e&&this.delayNotifierSubscriptions.splice(e,1),t.outerValue},e.prototype.tryDelay=function(t,e){var n=Object(o.a)(this,t,e);n&&!n.closed&&(this.destination.add(n),this.delayNotifierSubscriptions.push(n))},e.prototype.tryComplete=function(){this.completed&&0===this.delayNotifierSubscriptions.length&&this.destination.complete()},e}(i.a),mt=function(t){function e(e,n){var r=t.call(this)||this;return r.source=e,r.subscriptionDelay=n,r}return r.__extends(e,t),e.prototype._subscribe=function(t){this.subscriptionDelay.subscribe(new yt(t,this.source))},e}(ft.a),yt=function(t){function e(e,n){var r=t.call(this)||this;return r.parent=e,r.source=n,r.sourceSubscribed=!1,r}return r.__extends(e,t),e.prototype._next=function(t){this.subscribeToSource()},e.prototype._error=function(t){this.unsubscribe(),this.parent.error(t)},e.prototype._complete=function(){this.unsubscribe(),this.subscribeToSource()},e.prototype.subscribeToSource=function(){this.sourceSubscribed||(this.sourceSubscribed=!0,this.unsubscribe(),this.source.subscribe(this.parent))},e}(f.a);function Et(){return function(t){return t.lift(new vt)}}var vt=function(){function t(){}return t.prototype.call=function(t,e){return e.subscribe(new Bt(t))},t}(),Bt=function(t){function e(e){return t.call(this,e)||this}return r.__extends(e,t),e.prototype._next=function(t){t.observe(this.destination)},e}(f.a);function bt(t,e){return function(n){return n.lift(new Qt(t,e))}}var Qt=function(){function t(t,e){this.keySelector=t,this.flushes=e}return t.prototype.call=function(t,e){return e.subscribe(new wt(t,this.keySelector,this.flushes))},t}(),wt=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.keySelector=n,i.values=new Set,r&&i.add(Object(o.a)(i,r)),i}return r.__extends(e,t),e.prototype.notifyNext=function(t,e,n,r,i){this.values.clear()},e.prototype.notifyError=function(t,e){this._error(t)},e.prototype._next=function(t){this.keySelector?this._useKeySelector(t):this._finalizeNext(t,t)},e.prototype._useKeySelector=function(t){var e,n=this.destination;try{e=this.keySelector(t)}catch(t){return void n.error(t)}this._finalizeNext(e,t)},e.prototype._finalizeNext=function(t,e){var n=this.values;n.has(t)||(n.add(t),this.destination.next(e))},e}(i.a);function xt(t,e){return function(n){return n.lift(new _t(t,e))}}var _t=function(){function t(t,e){this.compare=t,this.keySelector=e}return t.prototype.call=function(t,e){return e.subscribe(new Dt(t,this.compare,this.keySelector))},t}(),Dt=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.keySelector=r,i.hasKey=!1,"function"==typeof n&&(i.compare=n),i}return r.__extends(e,t),e.prototype.compare=function(t,e){return t===e},e.prototype._next=function(t){var e;try{var n=this.keySelector;e=n?n(t):t}catch(t){return this.destination.error(t)}var r=!1;if(this.hasKey)try{r=(0,this.compare)(this.key,e)}catch(t){return this.destination.error(t)}else this.hasKey=!0;r||(this.key=e,this.destination.next(t))},e}(f.a);function Mt(t,e){return xt(function(n,r){return e?e(n[t],r[t]):n[t]===r[t]})}var kt=n(85),St=n(68),Lt=n(89);function Rt(t){return void 0===t&&(t=Tt),function(e){return e.lift(new Nt(t))}}var Nt=function(){function t(t){this.errorFactory=t}return t.prototype.call=function(t,e){return e.subscribe(new Ft(t,this.errorFactory))},t}(),Ft=function(t){function e(e,n){var r=t.call(this,e)||this;return r.errorFactory=n,r.hasValue=!1,r}return r.__extends(e,t),e.prototype._next=function(t){this.hasValue=!0,this.destination.next(t)},e.prototype._complete=function(){if(this.hasValue)return this.destination.complete();var t=void 0;try{t=this.errorFactory()}catch(e){t=e}this.destination.error(t)},e}(f.a);function Tt(){return new Lt.a}var Ut=n(51);function Ot(t){return function(e){return 0===t?Object(Ut.b)():e.lift(new Gt(t))}}var Gt=function(){function t(t){if(this.total=t,this.total<0)throw new kt.a}return t.prototype.call=function(t,e){return e.subscribe(new Ht(t,this.total))},t}(),Ht=function(t){function e(e,n){var r=t.call(this,e)||this;return r.total=n,r.count=0,r}return r.__extends(e,t),e.prototype._next=function(t){var e=this.total,n=++this.count;n<=e&&(this.destination.next(t),n===e&&(this.destination.complete(),this.unsubscribe()))},e}(f.a);function jt(t,e){if(t<0)throw new kt.a;var n=arguments.length>=2;return function(r){return r.pipe(Object(St.a)(function(e,n){return n===t}),Ot(1),n?at(e):Rt(function(){return new kt.a}))}}var Yt=n(114);function Pt(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return function(e){return Object(P.a)(e,Yt.a.apply(void 0,t))}}function Kt(t,e){return function(n){return n.lift(new zt(t,e,n))}}var zt=function(){function t(t,e,n){this.predicate=t,this.thisArg=e,this.source=n}return t.prototype.call=function(t,e){return e.subscribe(new Jt(t,this.predicate,this.thisArg,this.source))},t}(),Jt=function(t){function e(e,n,r,i){var o=t.call(this,e)||this;return o.predicate=n,o.thisArg=r,o.source=i,o.index=0,o.thisArg=r||o,o}return r.__extends(e,t),e.prototype.notifyComplete=function(t){this.destination.next(t),this.destination.complete()},e.prototype._next=function(t){var e=!1;try{e=this.predicate.call(this.thisArg,t,this.index++,this.source)}catch(t){return void this.destination.error(t)}e||this.notifyComplete(!1)},e.prototype._complete=function(){this.notifyComplete(!0)},e}(f.a);function qt(){return function(t){return t.lift(new Vt)}}var Vt=function(){function t(){}return t.prototype.call=function(t,e){return e.subscribe(new Wt(t))},t}(),Wt=function(t){function e(e){var n=t.call(this,e)||this;return n.hasCompleted=!1,n.hasSubscription=!1,n}return r.__extends(e,t),e.prototype._next=function(t){this.hasSubscription||(this.hasSubscription=!0,this.add(Object(o.a)(this,t)))},e.prototype._complete=function(){this.hasCompleted=!0,this.hasSubscription||this.destination.complete()},e.prototype.notifyComplete=function(t){this.remove(t),this.hasSubscription=!1,this.hasCompleted&&this.destination.complete()},e}(i.a),Xt=n(41);function Zt(t,e){return e?function(n){return n.pipe(Zt(function(n,r){return Object(j.a)(t(n,r)).pipe(Object(Xt.a)(function(t,i){return e(n,t,r,i)}))}))}:function(e){return e.lift(new $t(t))}}var $t=function(){function t(t){this.project=t}return t.prototype.call=function(t,e){return e.subscribe(new te(t,this.project))},t}(),te=function(t){function e(e,n){var r=t.call(this,e)||this;return r.project=n,r.hasSubscription=!1,r.hasCompleted=!1,r.index=0,r}return r.__extends(e,t),e.prototype._next=function(t){this.hasSubscription||this.tryNext(t)},e.prototype.tryNext=function(t){var e,n=this.index++;try{e=this.project(t,n)}catch(t){return void this.destination.error(t)}this.hasSubscription=!0,this._innerSub(e,t,n)},e.prototype._innerSub=function(t,e,n){var r=new N.a(this,void 0,void 0);this.destination.add(r),Object(o.a)(this,t,e,n,r)},e.prototype._complete=function(){this.hasCompleted=!0,this.hasSubscription||this.destination.complete(),this.unsubscribe()},e.prototype.notifyNext=function(t,e,n,r,i){this.destination.next(e)},e.prototype.notifyError=function(t){this.destination.error(t)},e.prototype.notifyComplete=function(t){this.destination.remove(t),this.hasSubscription=!1,this.hasCompleted&&this.destination.complete()},e}(i.a);function ee(t,e,n){return void 0===e&&(e=Number.POSITIVE_INFINITY),void 0===n&&(n=void 0),e=(e||0)<1?Number.POSITIVE_INFINITY:e,function(r){return r.lift(new ne(t,e,n))}}var ne=function(){function t(t,e,n){this.project=t,this.concurrent=e,this.scheduler=n}return t.prototype.call=function(t,e){return e.subscribe(new re(t,this.project,this.concurrent,this.scheduler))},t}(),re=function(t){function e(e,n,r,i){var o=t.call(this,e)||this;return o.project=n,o.concurrent=r,o.scheduler=i,o.index=0,o.active=0,o.hasCompleted=!1,r<Number.POSITIVE_INFINITY&&(o.buffer=[]),o}return r.__extends(e,t),e.dispatch=function(t){var e=t.subscriber,n=t.result,r=t.value,i=t.index;e.subscribeToProjection(n,r,i)},e.prototype._next=function(t){var n=this.destination;if(n.closed)this._complete();else{var r=this.index++;if(this.active<this.concurrent){n.next(t);try{var i=(0,this.project)(t,r);if(this.scheduler){var o={subscriber:this,result:i,value:t,index:r};this.destination.add(this.scheduler.schedule(e.dispatch,0,o))}else this.subscribeToProjection(i,t,r)}catch(t){n.error(t)}}else this.buffer.push(t)}},e.prototype.subscribeToProjection=function(t,e,n){this.active++,this.destination.add(Object(o.a)(this,t,e,n))},e.prototype._complete=function(){this.hasCompleted=!0,this.hasCompleted&&0===this.active&&this.destination.complete(),this.unsubscribe()},e.prototype.notifyNext=function(t,e,n,r,i){this._next(e)},e.prototype.notifyComplete=function(t){var e=this.buffer;this.destination.remove(t),this.active--,e&&e.length>0&&this._next(e.shift()),this.hasCompleted&&0===this.active&&this.destination.complete()},e}(i.a);function ie(t){return function(e){return e.lift(new oe(t))}}var oe=function(){function t(t){this.callback=t}return t.prototype.call=function(t,e){return e.subscribe(new ae(t,this.callback))},t}(),ae=function(t){function e(e,n){var r=t.call(this,e)||this;return r.add(new _.a(n)),r}return r.__extends(e,t),e}(f.a);function se(t,e){if("function"!=typeof t)throw new TypeError("predicate is not a function");return function(n){return n.lift(new Ae(t,n,!1,e))}}var Ae=function(){function t(t,e,n,r){this.predicate=t,this.source=e,this.yieldIndex=n,this.thisArg=r}return t.prototype.call=function(t,e){return e.subscribe(new le(t,this.predicate,this.source,this.yieldIndex,this.thisArg))},t}(),le=function(t){function e(e,n,r,i,o){var a=t.call(this,e)||this;return a.predicate=n,a.source=r,a.yieldIndex=i,a.thisArg=o,a.index=0,a}return r.__extends(e,t),e.prototype.notifyComplete=function(t){var e=this.destination;e.next(t),e.complete(),this.unsubscribe()},e.prototype._next=function(t){var e=this.predicate,n=this.thisArg,r=this.index++;try{e.call(n||this,t,r,this.source)&&this.notifyComplete(this.yieldIndex?r:t)}catch(t){this.destination.error(t)}},e.prototype._complete=function(){this.notifyComplete(this.yieldIndex?-1:void 0)},e}(f.a);function ce(t,e){return function(n){return n.lift(new Ae(t,n,!0,e))}}var ue=n(73);function ge(t,e){var n=arguments.length>=2;return function(r){return r.pipe(t?Object(St.a)(function(e,n){return t(e,n,r)}):ue.a,Ot(1),n?at(e):Rt(function(){return new Lt.a}))}}var he=n(219);function de(){return function(t){return t.lift(new fe)}}var fe=function(){function t(){}return t.prototype.call=function(t,e){return e.subscribe(new pe(t))},t}(),pe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.__extends(e,t),e.prototype._next=function(t){},e}(f.a);function Ie(){return function(t){return t.lift(new Ce)}}var Ce=function(){function t(){}return t.prototype.call=function(t,e){return e.subscribe(new me(t))},t}(),me=function(t){function e(e){return t.call(this,e)||this}return r.__extends(e,t),e.prototype.notifyComplete=function(t){var e=this.destination;e.next(t),e.complete()},e.prototype._next=function(t){this.notifyComplete(!1)},e.prototype._complete=function(){this.notifyComplete(!0)},e}(f.a);function ye(t){return function(e){return 0===t?Object(Ut.b)():e.lift(new Ee(t))}}var Ee=function(){function t(t){if(this.total=t,this.total<0)throw new kt.a}return t.prototype.call=function(t,e){return e.subscribe(new ve(t,this.total))},t}(),ve=function(t){function e(e,n){var r=t.call(this,e)||this;return r.total=n,r.ring=new Array,r.count=0,r}return r.__extends(e,t),e.prototype._next=function(t){var e=this.ring,n=this.total,r=this.count++;e.length<n?e.push(t):e[r%n]=t},e.prototype._complete=function(){var t=this.destination,e=this.count;if(e>0)for(var n=this.count>=this.total?this.total:this.count,r=this.ring,i=0;i<n;i++){var o=e++%n;t.next(r[o])}t.complete()},e}(f.a);function Be(t,e){var n=arguments.length>=2;return function(r){return r.pipe(t?Object(St.a)(function(e,n){return t(e,n,r)}):ue.a,ye(1),n?at(e):Rt(function(){return new Lt.a}))}}function be(t){return function(e){return e.lift(new Qe(t))}}var Qe=function(){function t(t){this.value=t}return t.prototype.call=function(t,e){return e.subscribe(new we(t,this.value))},t}(),we=function(t){function e(e,n){var r=t.call(this,e)||this;return r.value=n,r}return r.__extends(e,t),e.prototype._next=function(t){this.destination.next(this.value)},e}(f.a);function xe(){return function(t){return t.lift(new _e)}}var _e=function(){function t(){}return t.prototype.call=function(t,e){return e.subscribe(new De(t))},t}(),De=function(t){function e(e){return t.call(this,e)||this}return r.__extends(e,t),e.prototype._next=function(t){this.destination.next(ct.a.createNext(t))},e.prototype._error=function(t){var e=this.destination;e.next(ct.a.createError(t)),e.complete()},e.prototype._complete=function(){var t=this.destination;t.next(ct.a.createComplete()),t.complete()},e}(f.a);function Me(t,e){var n=!1;return arguments.length>=2&&(n=!0),function(r){return r.lift(new ke(t,e,n))}}var ke=function(){function t(t,e,n){void 0===n&&(n=!1),this.accumulator=t,this.seed=e,this.hasSeed=n}return t.prototype.call=function(t,e){return e.subscribe(new Se(t,this.accumulator,this.seed,this.hasSeed))},t}(),Se=function(t){function e(e,n,r,i){var o=t.call(this,e)||this;return o.accumulator=n,o._seed=r,o.hasSeed=i,o.index=0,o}return r.__extends(e,t),Object.defineProperty(e.prototype,"seed",{get:function(){return this._seed},set:function(t){this.hasSeed=!0,this._seed=t},enumerable:!0,configurable:!0}),e.prototype._next=function(t){if(this.hasSeed)return this._tryNext(t);this.seed=t,this.destination.next(t)},e.prototype._tryNext=function(t){var e,n=this.index++;try{e=this.accumulator(this.seed,t,n)}catch(t){this.destination.error(t)}this.seed=e,this.destination.next(e)},e}(f.a),Le=n(129);function Re(t,e){return arguments.length>=2?function(n){return Object(Le.a)(Me(t,e),ye(1),at(e))(n)}:function(e){return Object(Le.a)(Me(function(e,n,r){return t(e,n,r+1)}),ye(1))(e)}}function Ne(t){return Re("function"==typeof t?function(e,n){return t(e,n)>0?e:n}:function(t,e){return t>e?t:e})}var Fe=n(251);function Te(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return function(e){return e.lift.call(Fe.a.apply(void 0,[e].concat(t)))}}var Ue=n(152);function Oe(t,e,n){return void 0===n&&(n=Number.POSITIVE_INFINITY),"function"==typeof e?Object(J.a)(function(){return t},e,n):("number"==typeof e&&(n=e),Object(J.a)(function(){return t},n))}function Ge(t,e,n){return void 0===n&&(n=Number.POSITIVE_INFINITY),function(r){return r.lift(new He(t,e,n))}}var He=function(){function t(t,e,n){this.accumulator=t,this.seed=e,this.concurrent=n}return t.prototype.call=function(t,e){return e.subscribe(new je(t,this.accumulator,this.seed,this.concurrent))},t}(),je=function(t){function e(e,n,r,i){var o=t.call(this,e)||this;return o.accumulator=n,o.acc=r,o.concurrent=i,o.hasValue=!1,o.hasCompleted=!1,o.buffer=[],o.active=0,o.index=0,o}return r.__extends(e,t),e.prototype._next=function(t){if(this.active<this.concurrent){var e=this.index++,n=this.destination,r=void 0;try{r=(0,this.accumulator)(this.acc,t,e)}catch(t){return n.error(t)}this.active++,this._innerSub(r,t,e)}else this.buffer.push(t)},e.prototype._innerSub=function(t,e,n){var r=new N.a(this,void 0,void 0);this.destination.add(r),Object(o.a)(this,t,e,n,r)},e.prototype._complete=function(){this.hasCompleted=!0,0===this.active&&0===this.buffer.length&&(!1===this.hasValue&&this.destination.next(this.acc),this.destination.complete()),this.unsubscribe()},e.prototype.notifyNext=function(t,e,n,r,i){var o=this.destination;this.acc=e,this.hasValue=!0,o.next(e)},e.prototype.notifyComplete=function(t){var e=this.buffer;this.destination.remove(t),this.active--,e.length>0?this._next(e.shift()):0===this.active&&this.hasCompleted&&(!1===this.hasValue&&this.destination.next(this.acc),this.destination.complete())},e}(i.a);function Ye(t){return Re("function"==typeof t?function(e,n){return t(e,n)<0?e:n}:function(t,e){return t<e?t:e})}var Pe=n(246);function Ke(t,e){return function(n){var r;if(r="function"==typeof t?t:function(){return t},"function"==typeof e)return n.lift(new ze(r,e));var i=Object.create(n,Pe.b);return i.source=n,i.subjectFactory=r,i}}var ze=function(){function t(t,e){this.subjectFactory=t,this.selector=e}return t.prototype.call=function(t,e){var n=this.selector,r=this.subjectFactory(),i=n(r).subscribe(t);return i.add(e.subscribe(r)),i},t}(),Je=n(248);function qe(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return 1===t.length&&Object(H.a)(t[0])&&(t=t[0]),function(e){return e.lift(new Ve(t))}}var Ve=function(){function t(t){this.nextSources=t}return t.prototype.call=function(t,e){return e.subscribe(new We(t,this.nextSources))},t}(),We=function(t){function e(e,n){var r=t.call(this,e)||this;return r.destination=e,r.nextSources=n,r}return r.__extends(e,t),e.prototype.notifyError=function(t,e){this.subscribeToNextSource()},e.prototype.notifyComplete=function(t){this.subscribeToNextSource()},e.prototype._error=function(t){this.subscribeToNextSource(),this.unsubscribe()},e.prototype._complete=function(){this.subscribeToNextSource(),this.unsubscribe()},e.prototype.subscribeToNextSource=function(){var t=this.nextSources.shift();if(t){var e=new N.a(this,void 0,void 0);this.destination.add(e),Object(o.a)(this,t,void 0,void 0,e)}else this.destination.complete()},e}(i.a);function Xe(){return function(t){return t.lift(new Ze)}}var Ze=function(){function t(){}return t.prototype.call=function(t,e){return e.subscribe(new $e(t))},t}(),$e=function(t){function e(e){var n=t.call(this,e)||this;return n.hasPrev=!1,n}return r.__extends(e,t),e.prototype._next=function(t){var e;this.hasPrev?e=[this.prev,t]:this.hasPrev=!0,this.prev=t,e&&this.destination.next(e)},e}(f.a),tn=n(262);function en(t,e){return function(n){return[Object(St.a)(t,e)(n),Object(St.a)(Object(tn.a)(t,e))(n)]}}function nn(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];var n=t.length;if(0===n)throw new Error("list of properties cannot be empty.");return function(e){return Object(Xt.a)(function(t,e){return function(n){for(var r=n,i=0;i<e;i++){var o=r[t[i]];if(void 0===o)return;r=o}return r}}(t,n))(e)}}var rn=n(36);function on(t){return t?Ke(function(){return new rn.a},t):Ke(new rn.a)}var an=n(247);function sn(t){return function(e){return Ke(new an.a(t))(e)}}var An=n(92);function ln(){return function(t){return Ke(new An.a)(t)}}var cn=n(149);function un(t,e,n,r){n&&"function"!=typeof n&&(r=n);var i="function"==typeof n?n:void 0,o=new cn.a(t,e,r);return function(t){return Ke(function(){return o},i)(t)}}var gn=n(252);function hn(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return function(e){return 1===t.length&&Object(H.a)(t[0])&&(t=t[0]),e.lift.call(gn.a.apply(void 0,[e].concat(t)))}}function dn(t){return void 0===t&&(t=-1),function(e){return 0===t?Object(Ut.b)():t<0?e.lift(new fn(-1,e)):e.lift(new fn(t-1,e))}}var fn=function(){function t(t,e){this.count=t,this.source=e}return t.prototype.call=function(t,e){return e.subscribe(new pn(t,this.count,this.source))},t}(),pn=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.count=n,i.source=r,i}return r.__extends(e,t),e.prototype.complete=function(){if(!this.isStopped){var e=this.source,n=this.count;if(0===n)return t.prototype.complete.call(this);n>-1&&(this.count=n-1),e.subscribe(this._unsubscribeAndRecycle())}},e}(f.a);function In(t){return function(e){return e.lift(new Cn(t))}}var Cn=function(){function t(t){this.notifier=t}return t.prototype.call=function(t,e){return e.subscribe(new mn(t,this.notifier,e))},t}(),mn=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.notifier=n,i.source=r,i.sourceIsBeingSubscribedTo=!0,i}return r.__extends(e,t),e.prototype.notifyNext=function(t,e,n,r,i){this.sourceIsBeingSubscribedTo=!0,this.source.subscribe(this)},e.prototype.notifyComplete=function(e){if(!1===this.sourceIsBeingSubscribedTo)return t.prototype.complete.call(this)},e.prototype.complete=function(){if(this.sourceIsBeingSubscribedTo=!1,!this.isStopped){if(this.retries||this.subscribeToRetries(),!this.retriesSubscription||this.retriesSubscription.closed)return t.prototype.complete.call(this);this._unsubscribeAndRecycle(),this.notifications.next()}},e.prototype._unsubscribe=function(){var t=this.notifications,e=this.retriesSubscription;t&&(t.unsubscribe(),this.notifications=null),e&&(e.unsubscribe(),this.retriesSubscription=null),this.retries=null},e.prototype._unsubscribeAndRecycle=function(){var e=this._unsubscribe;return this._unsubscribe=null,t.prototype._unsubscribeAndRecycle.call(this),this._unsubscribe=e,this},e.prototype.subscribeToRetries=function(){var e;this.notifications=new rn.a;try{e=(0,this.notifier)(this.notifications)}catch(e){return t.prototype.complete.call(this)}this.retries=e,this.retriesSubscription=Object(o.a)(this,e)},e}(i.a);function yn(t){return void 0===t&&(t=-1),function(e){return e.lift(new En(t,e))}}var En=function(){function t(t,e){this.count=t,this.source=e}return t.prototype.call=function(t,e){return e.subscribe(new vn(t,this.count,this.source))},t}(),vn=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.count=n,i.source=r,i}return r.__extends(e,t),e.prototype.error=function(e){if(!this.isStopped){var n=this.source,r=this.count;if(0===r)return t.prototype.error.call(this,e);r>-1&&(this.count=r-1),n.subscribe(this._unsubscribeAndRecycle())}},e}(f.a);function Bn(t){return function(e){return e.lift(new bn(t,e))}}var bn=function(){function t(t,e){this.notifier=t,this.source=e}return t.prototype.call=function(t,e){return e.subscribe(new Qn(t,this.notifier,this.source))},t}(),Qn=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.notifier=n,i.source=r,i}return r.__extends(e,t),e.prototype.error=function(e){if(!this.isStopped){var n=this.errors,r=this.retries,i=this.retriesSubscription;if(r)this.errors=null,this.retriesSubscription=null;else{n=new rn.a;try{r=(0,this.notifier)(n)}catch(e){return t.prototype.error.call(this,e)}i=Object(o.a)(this,r)}this._unsubscribeAndRecycle(),this.errors=n,this.retries=r,this.retriesSubscription=i,n.next(e)}},e.prototype._unsubscribe=function(){var t=this.errors,e=this.retriesSubscription;t&&(t.unsubscribe(),this.errors=null),e&&(e.unsubscribe(),this.retriesSubscription=null),this.retries=null},e.prototype.notifyNext=function(t,e,n,r,i){var o=this._unsubscribe;this._unsubscribe=null,this._unsubscribeAndRecycle(),this._unsubscribe=o,this.source.subscribe(this)},e}(i.a),wn=n(148);function xn(t){return function(e){return e.lift(new _n(t))}}var _n=function(){function t(t){this.notifier=t}return t.prototype.call=function(t,e){var n=new Dn(t),r=e.subscribe(n);return r.add(Object(o.a)(n,this.notifier)),r},t}(),Dn=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.hasValue=!1,e}return r.__extends(e,t),e.prototype._next=function(t){this.value=t,this.hasValue=!0},e.prototype.notifyNext=function(t,e,n,r,i){this.emitValue()},e.prototype.notifyComplete=function(){this.emitValue()},e.prototype.emitValue=function(){this.hasValue&&(this.hasValue=!1,this.destination.next(this.value))},e}(i.a);function Mn(t,e){return void 0===e&&(e=l.a),function(n){return n.lift(new kn(t,e))}}var kn=function(){function t(t,e){this.period=t,this.scheduler=e}return t.prototype.call=function(t,e){return e.subscribe(new Sn(t,this.period,this.scheduler))},t}(),Sn=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.period=n,i.scheduler=r,i.hasValue=!1,i.add(r.schedule(Ln,n,{subscriber:i,period:n})),i}return r.__extends(e,t),e.prototype._next=function(t){this.lastValue=t,this.hasValue=!0},e.prototype.notifyNext=function(){this.hasValue&&(this.hasValue=!1,this.destination.next(this.lastValue))},e}(f.a);function Ln(t){var e=t.subscriber,n=t.period;e.notifyNext(),this.schedule(t,n)}function Rn(t,e){return function(n){return n.lift(new Nn(t,e))}}var Nn=function(){function t(t,e){this.compareTo=t,this.comparator=e}return t.prototype.call=function(t,e){return e.subscribe(new Fn(t,this.compareTo,this.comparator))},t}(),Fn=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.compareTo=n,i.comparator=r,i._a=[],i._b=[],i._oneComplete=!1,i.destination.add(n.subscribe(new Tn(e,i))),i}return r.__extends(e,t),e.prototype._next=function(t){this._oneComplete&&0===this._b.length?this.emit(!1):(this._a.push(t),this.checkValues())},e.prototype._complete=function(){this._oneComplete?this.emit(0===this._a.length&&0===this._b.length):this._oneComplete=!0,this.unsubscribe()},e.prototype.checkValues=function(){for(var t=this._a,e=this._b,n=this.comparator;t.length>0&&e.length>0;){var r=t.shift(),i=e.shift(),o=!1;try{o=n?n(r,i):r===i}catch(t){this.destination.error(t)}o||this.emit(!1)}},e.prototype.emit=function(t){var e=this.destination;e.next(t),e.complete()},e.prototype.nextB=function(t){this._oneComplete&&0===this._a.length?this.emit(!1):(this._b.push(t),this.checkValues())},e.prototype.completeB=function(){this._oneComplete?this.emit(0===this._a.length&&0===this._b.length):this._oneComplete=!0},e}(f.a),Tn=function(t){function e(e,n){var r=t.call(this,e)||this;return r.parent=n,r}return r.__extends(e,t),e.prototype._next=function(t){this.parent.nextB(t)},e.prototype._error=function(t){this.parent.error(t),this.unsubscribe()},e.prototype._complete=function(){this.parent.completeB(),this.unsubscribe()},e}(f.a);function Un(){return new rn.a}function On(){return function(t){return Object(wn.a)()(Ke(Un)(t))}}function Gn(t,e,n){var r;return r=t&&"object"==typeof t?t:{bufferSize:t,windowTime:e,refCount:!1,scheduler:n},function(t){return t.lift(function(t){var e,n,r=t.bufferSize,i=void 0===r?Number.POSITIVE_INFINITY:r,o=t.windowTime,a=void 0===o?Number.POSITIVE_INFINITY:o,s=t.refCount,A=t.scheduler,l=0,c=!1,u=!1;return function(t){l++,e&&!c||(c=!1,e=new cn.a(i,a,A),n=t.subscribe({next:function(t){e.next(t)},error:function(t){c=!0,e.error(t)},complete:function(){u=!0,e.complete()}}));var r=e.subscribe(this);this.add(function(){l--,r.unsubscribe(),n&&!u&&s&&0===l&&(n.unsubscribe(),n=void 0,e=void 0)})}}(r))}}function Hn(t){return function(e){return e.lift(new jn(t,e))}}var jn=function(){function t(t,e){this.predicate=t,this.source=e}return t.prototype.call=function(t,e){return e.subscribe(new Yn(t,this.predicate,this.source))},t}(),Yn=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.predicate=n,i.source=r,i.seenValue=!1,i.index=0,i}return r.__extends(e,t),e.prototype.applySingleValue=function(t){this.seenValue?this.destination.error("Sequence contains more than one element"):(this.seenValue=!0,this.singleValue=t)},e.prototype._next=function(t){var e=this.index++;this.predicate?this.tryNext(t,e):this.applySingleValue(t)},e.prototype.tryNext=function(t,e){try{this.predicate(t,e,this.source)&&this.applySingleValue(t)}catch(t){this.destination.error(t)}},e.prototype._complete=function(){var t=this.destination;this.index>0?(t.next(this.seenValue?this.singleValue:void 0),t.complete()):t.error(new Lt.a)},e}(f.a);function Pn(t){return function(e){return e.lift(new Kn(t))}}var Kn=function(){function t(t){this.total=t}return t.prototype.call=function(t,e){return e.subscribe(new zn(t,this.total))},t}(),zn=function(t){function e(e,n){var r=t.call(this,e)||this;return r.total=n,r.count=0,r}return r.__extends(e,t),e.prototype._next=function(t){++this.count>this.total&&this.destination.next(t)},e}(f.a);function Jn(t){return function(e){return e.lift(new qn(t))}}var qn=function(){function t(t){if(this._skipCount=t,this._skipCount<0)throw new kt.a}return t.prototype.call=function(t,e){return 0===this._skipCount?e.subscribe(new f.a(t)):e.subscribe(new Vn(t,this._skipCount))},t}(),Vn=function(t){function e(e,n){var r=t.call(this,e)||this;return r._skipCount=n,r._count=0,r._ring=new Array(n),r}return r.__extends(e,t),e.prototype._next=function(t){var e=this._skipCount,n=this._count++;if(n<e)this._ring[n]=t;else{var r=n%e,i=this._ring,o=i[r];i[r]=t,this.destination.next(o)}},e}(f.a);function Wn(t){return function(e){return e.lift(new Xn(t))}}var Xn=function(){function t(t){this.notifier=t}return t.prototype.call=function(t,e){return e.subscribe(new Zn(t,this.notifier))},t}(),Zn=function(t){function e(e,n){var r=t.call(this,e)||this;r.hasValue=!1;var i=new N.a(r,void 0,void 0);return r.add(i),r.innerSubscription=i,Object(o.a)(r,n,void 0,void 0,i),r}return r.__extends(e,t),e.prototype._next=function(e){this.hasValue&&t.prototype._next.call(this,e)},e.prototype.notifyNext=function(t,e,n,r,i){this.hasValue=!0,this.innerSubscription&&this.innerSubscription.unsubscribe()},e.prototype.notifyComplete=function(){},e}(i.a);function $n(t){return function(e){return e.lift(new tr(t))}}var tr=function(){function t(t){this.predicate=t}return t.prototype.call=function(t,e){return e.subscribe(new er(t,this.predicate))},t}(),er=function(t){function e(e,n){var r=t.call(this,e)||this;return r.predicate=n,r.skipping=!0,r.index=0,r}return r.__extends(e,t),e.prototype._next=function(t){var e=this.destination;this.skipping&&this.tryCallPredicate(t),this.skipping||e.next(t)},e.prototype.tryCallPredicate=function(t){try{var e=this.predicate(t,this.index++);this.skipping=Boolean(e)}catch(t){this.destination.error(t)}},e}(f.a);function nr(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];var n=t[t.length-1];return Object(y.a)(n)?(t.pop(),function(e){return Object(P.a)(t,e,n)}):function(e){return Object(P.a)(t,e)}}var rr=n(160),ir=n(95),or=function(t){function e(e,n,r){void 0===n&&(n=0),void 0===r&&(r=rr.a);var i=t.call(this)||this;return i.source=e,i.delayTime=n,i.scheduler=r,(!Object(ir.a)(n)||n<0)&&(i.delayTime=0),r&&"function"==typeof r.schedule||(i.scheduler=rr.a),i}return r.__extends(e,t),e.create=function(t,n,r){return void 0===n&&(n=0),void 0===r&&(r=rr.a),new e(t,n,r)},e.dispatch=function(t){var e=t.source,n=t.subscriber;return this.add(e.subscribe(n))},e.prototype._subscribe=function(t){var n=this.delayTime,r=this.source;return this.scheduler.schedule(e.dispatch,n,{source:r,subscriber:t})},e}(ft.a);function ar(t,e){return void 0===e&&(e=0),function(n){return n.lift(new sr(t,e))}}var sr=function(){function t(t,e){this.scheduler=t,this.delay=e}return t.prototype.call=function(t,e){return new or(e,this.delay,this.scheduler).subscribe(t)},t}();function Ar(t,e){return"function"==typeof e?function(n){return n.pipe(Ar(function(n,r){return Object(j.a)(t(n,r)).pipe(Object(Xt.a)(function(t,i){return e(n,t,r,i)}))}))}:function(e){return e.lift(new lr(t))}}var lr=function(){function t(t){this.project=t}return t.prototype.call=function(t,e){return e.subscribe(new cr(t,this.project))},t}(),cr=function(t){function e(e,n){var r=t.call(this,e)||this;return r.project=n,r.index=0,r}return r.__extends(e,t),e.prototype._next=function(t){var e,n=this.index++;try{e=this.project(t,n)}catch(t){return void this.destination.error(t)}this._innerSub(e,t,n)},e.prototype._innerSub=function(t,e,n){var r=this.innerSubscription;r&&r.unsubscribe();var i=new N.a(this,void 0,void 0);this.destination.add(i),this.innerSubscription=Object(o.a)(this,t,e,n,i)},e.prototype._complete=function(){var e=this.innerSubscription;e&&!e.closed||t.prototype._complete.call(this),this.unsubscribe()},e.prototype._unsubscribe=function(){this.innerSubscription=null},e.prototype.notifyComplete=function(e){this.destination.remove(e),this.innerSubscription=null,this.isStopped&&t.prototype._complete.call(this)},e.prototype.notifyNext=function(t,e,n,r,i){this.destination.next(e)},e}(i.a);function ur(){return Ar(ue.a)}function gr(t,e){return e?Ar(function(){return t},e):Ar(function(){return t})}function hr(t){return function(e){return e.lift(new dr(t))}}var dr=function(){function t(t){this.notifier=t}return t.prototype.call=function(t,e){var n=new fr(t),r=Object(o.a)(n,this.notifier);return r&&!n.seenValue?(n.add(r),e.subscribe(n)):n},t}(),fr=function(t){function e(e){var n=t.call(this,e)||this;return n.seenValue=!1,n}return r.__extends(e,t),e.prototype.notifyNext=function(t,e,n,r,i){this.seenValue=!0,this.complete()},e.prototype.notifyComplete=function(){},e}(i.a);function pr(t,e){return void 0===e&&(e=!1),function(n){return n.lift(new Ir(t,e))}}var Ir=function(){function t(t,e){this.predicate=t,this.inclusive=e}return t.prototype.call=function(t,e){return e.subscribe(new Cr(t,this.predicate,this.inclusive))},t}(),Cr=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.predicate=n,i.inclusive=r,i.index=0,i}return r.__extends(e,t),e.prototype._next=function(t){var e,n=this.destination;try{e=this.predicate(t,this.index++)}catch(t){return void n.error(t)}this.nextOrComplete(t,e)},e.prototype.nextOrComplete=function(t,e){var n=this.destination;Boolean(e)?n.next(t):(this.inclusive&&n.next(t),n.complete())},e}(f.a),mr=n(65),yr=n(86);function Er(t,e,n){return function(r){return r.lift(new vr(t,e,n))}}var vr=function(){function t(t,e,n){this.nextOrObserver=t,this.error=e,this.complete=n}return t.prototype.call=function(t,e){return e.subscribe(new Br(t,this.nextOrObserver,this.error,this.complete))},t}(),Br=function(t){function e(e,n,r,i){var o=t.call(this,e)||this;return o._tapNext=mr.a,o._tapError=mr.a,o._tapComplete=mr.a,o._tapError=r||mr.a,o._tapComplete=i||mr.a,Object(yr.a)(n)?(o._context=o,o._tapNext=n):n&&(o._context=n,o._tapNext=n.next||mr.a,o._tapError=n.error||mr.a,o._tapComplete=n.complete||mr.a),o}return r.__extends(e,t),e.prototype._next=function(t){try{this._tapNext.call(this._context,t)}catch(t){return void this.destination.error(t)}this.destination.next(t)},e.prototype._error=function(t){try{this._tapError.call(this._context,t)}catch(t){return void this.destination.error(t)}this.destination.error(t)},e.prototype._complete=function(){try{this._tapComplete.call(this._context)}catch(t){return void this.destination.error(t)}return this.destination.complete()},e}(f.a),br={leading:!0,trailing:!1};function Qr(t,e){return void 0===e&&(e=br),function(n){return n.lift(new wr(t,e.leading,e.trailing))}}var wr=function(){function t(t,e,n){this.durationSelector=t,this.leading=e,this.trailing=n}return t.prototype.call=function(t,e){return e.subscribe(new xr(t,this.durationSelector,this.leading,this.trailing))},t}(),xr=function(t){function e(e,n,r,i){var o=t.call(this,e)||this;return o.destination=e,o.durationSelector=n,o._leading=r,o._trailing=i,o._hasValue=!1,o}return r.__extends(e,t),e.prototype._next=function(t){this._hasValue=!0,this._sendValue=t,this._throttled||(this._leading?this.send():this.throttle(t))},e.prototype.send=function(){var t=this._hasValue,e=this._sendValue;t&&(this.destination.next(e),this.throttle(e)),this._hasValue=!1,this._sendValue=null},e.prototype.throttle=function(t){var e=this.tryDurationSelector(t);e&&this.add(this._throttled=Object(o.a)(this,e))},e.prototype.tryDurationSelector=function(t){try{return this.durationSelector(t)}catch(t){return this.destination.error(t),null}},e.prototype.throttlingDone=function(){var t=this._throttled,e=this._trailing;t&&t.unsubscribe(),this._throttled=null,e&&this.send()},e.prototype.notifyNext=function(t,e,n,r,i){this.throttlingDone()},e.prototype.notifyComplete=function(){this.throttlingDone()},e}(i.a);function _r(t,e,n){return void 0===e&&(e=l.a),void 0===n&&(n=br),function(r){return r.lift(new Dr(t,e,n.leading,n.trailing))}}var Dr=function(){function t(t,e,n,r){this.duration=t,this.scheduler=e,this.leading=n,this.trailing=r}return t.prototype.call=function(t,e){return e.subscribe(new Mr(t,this.duration,this.scheduler,this.leading,this.trailing))},t}(),Mr=function(t){function e(e,n,r,i,o){var a=t.call(this,e)||this;return a.duration=n,a.scheduler=r,a.leading=i,a.trailing=o,a._hasTrailingValue=!1,a._trailingValue=null,a}return r.__extends(e,t),e.prototype._next=function(t){this.throttled?this.trailing&&(this._trailingValue=t,this._hasTrailingValue=!0):(this.add(this.throttled=this.scheduler.schedule(kr,this.duration,{subscriber:this})),this.leading?this.destination.next(t):this.trailing&&(this._trailingValue=t,this._hasTrailingValue=!0))},e.prototype._complete=function(){this._hasTrailingValue?(this.destination.next(this._trailingValue),this.destination.complete()):this.destination.complete()},e.prototype.clearThrottle=function(){var t=this.throttled;t&&(this.trailing&&this._hasTrailingValue&&(this.destination.next(this._trailingValue),this._trailingValue=null,this._hasTrailingValue=!1),t.unsubscribe(),this.remove(t),this.throttled=null)},e}(f.a);function kr(t){t.subscriber.clearThrottle()}var Sr=n(153);function Lr(t){return void 0===t&&(t=l.a),function(e){return Object(Sr.a)(function(){return e.pipe(Me(function(e,n){var r=e.current;return{value:n,current:t.now(),last:r}},{current:t.now(),value:void 0,last:void 0}),Object(Xt.a)(function(t){var e=t.current,n=t.last,r=t.value;return new Rr(r,e-n)}))})}}var Rr=function(){return function(t,e){this.value=t,this.interval=e}}(),Nr=n(249);function Fr(t,e,n){return void 0===n&&(n=l.a),function(r){var i=lt(t),o=i?+t-n.now():Math.abs(t);return r.lift(new Tr(o,i,e,n))}}var Tr=function(){function t(t,e,n,r){this.waitFor=t,this.absoluteTimeout=e,this.withObservable=n,this.scheduler=r}return t.prototype.call=function(t,e){return e.subscribe(new Ur(t,this.absoluteTimeout,this.waitFor,this.withObservable,this.scheduler))},t}(),Ur=function(t){function e(e,n,r,i,o){var a=t.call(this,e)||this;return a.absoluteTimeout=n,a.waitFor=r,a.withObservable=i,a.scheduler=o,a.action=null,a.scheduleTimeout(),a}return r.__extends(e,t),e.dispatchTimeout=function(t){var e=t.withObservable;t._unsubscribeAndRecycle(),t.add(Object(o.a)(t,e))},e.prototype.scheduleTimeout=function(){var t=this.action;t?this.action=t.schedule(this,this.waitFor):this.add(this.action=this.scheduler.schedule(e.dispatchTimeout,this.waitFor,this))},e.prototype._next=function(e){this.absoluteTimeout||this.scheduleTimeout(),t.prototype._next.call(this,e)},e.prototype._unsubscribe=function(){this.action=null,this.scheduler=null,this.withObservable=null},e}(i.a),Or=n(150);function Gr(t,e){return void 0===e&&(e=l.a),Fr(t,Object(Or.a)(new Nr.a),e)}function Hr(t){return void 0===t&&(t=l.a),Object(Xt.a)(function(e){return new jr(e,t.now())})}var jr=function(){return function(t,e){this.value=t,this.timestamp=e}}();function Yr(t,e,n){return 0===n?[e]:(t.push(e),t)}function Pr(){return Re(Yr,[])}function Kr(t){return function(e){return e.lift(new zr(t))}}var zr=function(){function t(t){this.windowBoundaries=t}return t.prototype.call=function(t,e){var n=new Jr(t),r=e.subscribe(n);return r.closed||n.add(Object(o.a)(n,this.windowBoundaries)),r},t}(),Jr=function(t){function e(e){var n=t.call(this,e)||this;return n.window=new rn.a,e.next(n.window),n}return r.__extends(e,t),e.prototype.notifyNext=function(t,e,n,r,i){this.openWindow()},e.prototype.notifyError=function(t,e){this._error(t)},e.prototype.notifyComplete=function(t){this._complete()},e.prototype._next=function(t){this.window.next(t)},e.prototype._error=function(t){this.window.error(t),this.destination.error(t)},e.prototype._complete=function(){this.window.complete(),this.destination.complete()},e.prototype._unsubscribe=function(){this.window=null},e.prototype.openWindow=function(){var t=this.window;t&&t.complete();var e=this.destination,n=this.window=new rn.a;e.next(n)},e}(i.a);function qr(t,e){return void 0===e&&(e=0),function(n){return n.lift(new Vr(t,e))}}var Vr=function(){function t(t,e){this.windowSize=t,this.startWindowEvery=e}return t.prototype.call=function(t,e){return e.subscribe(new Wr(t,this.windowSize,this.startWindowEvery))},t}(),Wr=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.destination=e,i.windowSize=n,i.startWindowEvery=r,i.windows=[new rn.a],i.count=0,e.next(i.windows[0]),i}return r.__extends(e,t),e.prototype._next=function(t){for(var e=this.startWindowEvery>0?this.startWindowEvery:this.windowSize,n=this.destination,r=this.windowSize,i=this.windows,o=i.length,a=0;a<o&&!this.closed;a++)i[a].next(t);var s=this.count-r+1;if(s>=0&&s%e==0&&!this.closed&&i.shift().complete(),++this.count%e==0&&!this.closed){var A=new rn.a;i.push(A),n.next(A)}},e.prototype._error=function(t){var e=this.windows;if(e)for(;e.length>0&&!this.closed;)e.shift().error(t);this.destination.error(t)},e.prototype._complete=function(){var t=this.windows;if(t)for(;t.length>0&&!this.closed;)t.shift().complete();this.destination.complete()},e.prototype._unsubscribe=function(){this.count=0,this.windows=null},e}(f.a);function Xr(t){var e=l.a,n=null,r=Number.POSITIVE_INFINITY;return Object(y.a)(arguments[3])&&(e=arguments[3]),Object(y.a)(arguments[2])?e=arguments[2]:Object(ir.a)(arguments[2])&&(r=arguments[2]),Object(y.a)(arguments[1])?e=arguments[1]:Object(ir.a)(arguments[1])&&(n=arguments[1]),function(i){return i.lift(new Zr(t,n,r,e))}}var Zr=function(){function t(t,e,n,r){this.windowTimeSpan=t,this.windowCreationInterval=e,this.maxWindowSize=n,this.scheduler=r}return t.prototype.call=function(t,e){return e.subscribe(new ti(t,this.windowTimeSpan,this.windowCreationInterval,this.maxWindowSize,this.scheduler))},t}(),$r=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._numberOfNextedValues=0,e}return r.__extends(e,t),e.prototype.next=function(e){this._numberOfNextedValues++,t.prototype.next.call(this,e)},Object.defineProperty(e.prototype,"numberOfNextedValues",{get:function(){return this._numberOfNextedValues},enumerable:!0,configurable:!0}),e}(rn.a),ti=function(t){function e(e,n,r,i,o){var a=t.call(this,e)||this;a.destination=e,a.windowTimeSpan=n,a.windowCreationInterval=r,a.maxWindowSize=i,a.scheduler=o,a.windows=[];var s=a.openWindow();if(null!==r&&r>=0){var A={subscriber:a,window:s,context:null},l={windowTimeSpan:n,windowCreationInterval:r,subscriber:a,scheduler:o};a.add(o.schedule(ri,n,A)),a.add(o.schedule(ni,r,l))}else{var c={subscriber:a,window:s,windowTimeSpan:n};a.add(o.schedule(ei,n,c))}return a}return r.__extends(e,t),e.prototype._next=function(t){for(var e=this.windows,n=e.length,r=0;r<n;r++){var i=e[r];i.closed||(i.next(t),i.numberOfNextedValues>=this.maxWindowSize&&this.closeWindow(i))}},e.prototype._error=function(t){for(var e=this.windows;e.length>0;)e.shift().error(t);this.destination.error(t)},e.prototype._complete=function(){for(var t=this.windows;t.length>0;){var e=t.shift();e.closed||e.complete()}this.destination.complete()},e.prototype.openWindow=function(){var t=new $r;return this.windows.push(t),this.destination.next(t),t},e.prototype.closeWindow=function(t){t.complete();var e=this.windows;e.splice(e.indexOf(t),1)},e}(f.a);function ei(t){var e=t.subscriber,n=t.windowTimeSpan,r=t.window;r&&e.closeWindow(r),t.window=e.openWindow(),this.schedule(t,n)}function ni(t){var e=t.windowTimeSpan,n=t.subscriber,r=t.scheduler,i=t.windowCreationInterval,o=n.openWindow(),a={action:this,subscription:null},s={subscriber:n,window:o,context:a};a.subscription=r.schedule(ri,e,s),this.add(a.subscription),this.schedule(t,i)}function ri(t){var e=t.subscriber,n=t.window,r=t.context;r&&r.action&&r.subscription&&r.action.remove(r.subscription),e.closeWindow(n)}function ii(t,e){return function(n){return n.lift(new oi(t,e))}}var oi=function(){function t(t,e){this.openings=t,this.closingSelector=e}return t.prototype.call=function(t,e){return e.subscribe(new ai(t,this.openings,this.closingSelector))},t}(),ai=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.openings=n,i.closingSelector=r,i.contexts=[],i.add(i.openSubscription=Object(o.a)(i,n,n)),i}return r.__extends(e,t),e.prototype._next=function(t){var e=this.contexts;if(e)for(var n=e.length,r=0;r<n;r++)e[r].window.next(t)},e.prototype._error=function(e){var n=this.contexts;if(this.contexts=null,n)for(var r=n.length,i=-1;++i<r;){var o=n[i];o.window.error(e),o.subscription.unsubscribe()}t.prototype._error.call(this,e)},e.prototype._complete=function(){var e=this.contexts;if(this.contexts=null,e)for(var n=e.length,r=-1;++r<n;){var i=e[r];i.window.complete(),i.subscription.unsubscribe()}t.prototype._complete.call(this)},e.prototype._unsubscribe=function(){var t=this.contexts;if(this.contexts=null,t)for(var e=t.length,n=-1;++n<e;){var r=t[n];r.window.unsubscribe(),r.subscription.unsubscribe()}},e.prototype.notifyNext=function(t,e,n,r,i){if(t===this.openings){var a=void 0;try{a=(0,this.closingSelector)(e)}catch(t){return this.error(t)}var s=new rn.a,A=new _.a,l={window:s,subscription:A};this.contexts.push(l);var c=Object(o.a)(this,a,l);c.closed?this.closeWindow(this.contexts.length-1):(c.context=l,A.add(c)),this.destination.next(s)}else this.closeWindow(this.contexts.indexOf(t))},e.prototype.notifyError=function(t){this.error(t)},e.prototype.notifyComplete=function(t){t!==this.openSubscription&&this.closeWindow(this.contexts.indexOf(t.context))},e.prototype.closeWindow=function(t){if(-1!==t){var e=this.contexts,n=e[t],r=n.window,i=n.subscription;e.splice(t,1),r.complete(),i.unsubscribe()}},e}(i.a);function si(t){return function(e){return e.lift(new Ai(t))}}var Ai=function(){function t(t){this.closingSelector=t}return t.prototype.call=function(t,e){return e.subscribe(new li(t,this.closingSelector))},t}(),li=function(t){function e(e,n){var r=t.call(this,e)||this;return r.destination=e,r.closingSelector=n,r.openWindow(),r}return r.__extends(e,t),e.prototype.notifyNext=function(t,e,n,r,i){this.openWindow(i)},e.prototype.notifyError=function(t,e){this._error(t)},e.prototype.notifyComplete=function(t){this.openWindow(t)},e.prototype._next=function(t){this.window.next(t)},e.prototype._error=function(t){this.window.error(t),this.destination.error(t),this.unsubscribeClosingNotification()},e.prototype._complete=function(){this.window.complete(),this.destination.complete(),this.unsubscribeClosingNotification()},e.prototype.unsubscribeClosingNotification=function(){this.closingNotification&&this.closingNotification.unsubscribe()},e.prototype.openWindow=function(t){void 0===t&&(t=null),t&&(this.remove(t),t.unsubscribe());var e=this.window;e&&e.complete();var n,r=this.window=new rn.a;this.destination.next(r);try{n=(0,this.closingSelector)()}catch(t){return this.destination.error(t),void this.window.error(t)}this.add(this.closingNotification=Object(o.a)(this,n))},e}(i.a);function ci(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return function(e){var n;"function"==typeof t[t.length-1]&&(n=t.pop());var r=t;return e.lift(new ui(r,n))}}var ui=function(){function t(t,e){this.observables=t,this.project=e}return t.prototype.call=function(t,e){return e.subscribe(new gi(t,this.observables,this.project))},t}(),gi=function(t){function e(e,n,r){var i=t.call(this,e)||this;i.observables=n,i.project=r,i.toRespond=[];var a=n.length;i.values=new Array(a);for(var s=0;s<a;s++)i.toRespond.push(s);for(s=0;s<a;s++){var A=n[s];i.add(Object(o.a)(i,A,A,s))}return i}return r.__extends(e,t),e.prototype.notifyNext=function(t,e,n,r,i){this.values[n]=e;var o=this.toRespond;if(o.length>0){var a=o.indexOf(n);-1!==a&&o.splice(a,1)}},e.prototype.notifyComplete=function(){},e.prototype._next=function(t){if(0===this.toRespond.length){var e=[t].concat(this.values);this.project?this._tryProject(e):this.destination.next(e)}},e.prototype._tryProject=function(t){var e;try{e=this.project.apply(this,t)}catch(t){return void this.destination.error(t)}this.destination.next(e)},e}(i.a),hi=n(154);function di(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return function(e){return e.lift.call(hi.b.apply(void 0,[e].concat(t)))}}function fi(t){return function(e){return e.lift(new hi.a(t))}}n.d(e,"audit",function(){return a}),n.d(e,"auditTime",function(){return u}),n.d(e,"buffer",function(){return g}),n.d(e,"bufferCount",function(){return p}),n.d(e,"bufferTime",function(){return E}),n.d(e,"bufferToggle",function(){return D}),n.d(e,"bufferWhen",function(){return S}),n.d(e,"catchError",function(){return F}),n.d(e,"combineAll",function(){return G}),n.d(e,"combineLatest",function(){return Y}),n.d(e,"concat",function(){return K}),n.d(e,"concatAll",function(){return z.a}),n.d(e,"concatMap",function(){return q}),n.d(e,"concatMapTo",function(){return V}),n.d(e,"count",function(){return W}),n.d(e,"debounce",function(){return $}),n.d(e,"debounceTime",function(){return nt}),n.d(e,"defaultIfEmpty",function(){return at}),n.d(e,"delay",function(){return ut}),n.d(e,"delayWhen",function(){return pt}),n.d(e,"dematerialize",function(){return Et}),n.d(e,"distinct",function(){return bt}),n.d(e,"distinctUntilChanged",function(){return xt}),n.d(e,"distinctUntilKeyChanged",function(){return Mt}),n.d(e,"elementAt",function(){return jt}),n.d(e,"endWith",function(){return Pt}),n.d(e,"every",function(){return Kt}),n.d(e,"exhaust",function(){return qt}),n.d(e,"exhaustMap",function(){return Zt}),n.d(e,"expand",function(){return ee}),n.d(e,"filter",function(){return St.a}),n.d(e,"finalize",function(){return ie}),n.d(e,"find",function(){return se}),n.d(e,"findIndex",function(){return ce}),n.d(e,"first",function(){return ge}),n.d(e,"groupBy",function(){return he.b}),n.d(e,"ignoreElements",function(){return de}),n.d(e,"isEmpty",function(){return Ie}),n.d(e,"last",function(){return Be}),n.d(e,"map",function(){return Xt.a}),n.d(e,"mapTo",function(){return be}),n.d(e,"materialize",function(){return xe}),n.d(e,"max",function(){return Ne}),n.d(e,"merge",function(){return Te}),n.d(e,"mergeAll",function(){return Ue.a}),n.d(e,"mergeMap",function(){return J.a}),n.d(e,"flatMap",function(){return J.a}),n.d(e,"mergeMapTo",function(){return Oe}),n.d(e,"mergeScan",function(){return Ge}),n.d(e,"min",function(){return Ye}),n.d(e,"multicast",function(){return Ke}),n.d(e,"observeOn",function(){return Je.b}),n.d(e,"onErrorResumeNext",function(){return qe}),n.d(e,"pairwise",function(){return Xe}),n.d(e,"partition",function(){return en}),n.d(e,"pluck",function(){return nn}),n.d(e,"publish",function(){return on}),n.d(e,"publishBehavior",function(){return sn}),n.d(e,"publishLast",function(){return ln}),n.d(e,"publishReplay",function(){return un}),n.d(e,"race",function(){return hn}),n.d(e,"reduce",function(){return Re}),n.d(e,"repeat",function(){return dn}),n.d(e,"repeatWhen",function(){return In}),n.d(e,"retry",function(){return yn}),n.d(e,"retryWhen",function(){return Bn}),n.d(e,"refCount",function(){return wn.a}),n.d(e,"sample",function(){return xn}),n.d(e,"sampleTime",function(){return Mn}),n.d(e,"scan",function(){return Me}),n.d(e,"sequenceEqual",function(){return Rn}),n.d(e,"share",function(){return On}),n.d(e,"shareReplay",function(){return Gn}),n.d(e,"single",function(){return Hn}),n.d(e,"skip",function(){return Pn}),n.d(e,"skipLast",function(){return Jn}),n.d(e,"skipUntil",function(){return Wn}),n.d(e,"skipWhile",function(){return $n}),n.d(e,"startWith",function(){return nr}),n.d(e,"subscribeOn",function(){return ar}),n.d(e,"switchAll",function(){return ur}),n.d(e,"switchMap",function(){return Ar}),n.d(e,"switchMapTo",function(){return gr}),n.d(e,"take",function(){return Ot}),n.d(e,"takeLast",function(){return ye}),n.d(e,"takeUntil",function(){return hr}),n.d(e,"takeWhile",function(){return pr}),n.d(e,"tap",function(){return Er}),n.d(e,"throttle",function(){return Qr}),n.d(e,"throttleTime",function(){return _r}),n.d(e,"throwIfEmpty",function(){return Rt}),n.d(e,"timeInterval",function(){return Lr}),n.d(e,"timeout",function(){return Gr}),n.d(e,"timeoutWith",function(){return Fr}),n.d(e,"timestamp",function(){return Hr}),n.d(e,"toArray",function(){return Pr}),n.d(e,"window",function(){return Kr}),n.d(e,"windowCount",function(){return qr}),n.d(e,"windowTime",function(){return Xr}),n.d(e,"windowToggle",function(){return ii}),n.d(e,"windowWhen",function(){return si}),n.d(e,"withLatestFrom",function(){return ci}),n.d(e,"zip",function(){return di}),n.d(e,"zipAll",function(){return fi})},function(t,e,n){"use strict";n.d(e,"a",function(){return a});var r=n(45),i=n(90),o=n(159);function a(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];var n=t[t.length-1];return Object(r.a)(n)?(t.pop(),Object(o.a)(t,n)):Object(i.a)(t)}},function(t,e,n){"use strict";var r,i;n.d(e,"a",function(){return r}),function(t){function e(t,e,r=""){t.localName===e&&t.classList.add("jp-mod-styled"),"select"===t.localName&&n(t);let i=t.getElementsByTagName(e);for(let t=0;t<i.length;t++){let o=i[t];o.classList.add("jp-mod-styled"),r&&o.classList.add(r),"select"===e&&n(o)}}function n(t){let e=document.createElement("div");return e.classList.add("jp-select-wrapper"),t.addEventListener("focus",i.onFocus),t.addEventListener("blur",i.onFocus),t.classList.add("jp-mod-styled"),t.parentElement&&t.parentElement.replaceChild(e,t),e.appendChild(t),e}t.styleNode=function(t,n=""){e(t,"select",n),e(t,"textarea",n),e(t,"input",n),e(t,"button",n)},t.styleNodeByTag=e,t.wrapSelect=n}(r||(r={})),function(t){t.onFocus=function(t){let e=t.target.parentElement;e&&("focus"===t.type?e.classList.add("jp-mod-focused"):e.classList.remove("jp-mod-focused"))}}(i||(i={}))},function(t,e,n){"use strict";n.r(e);var r=n(13),i=n.n(r),o=n(18),a=n.n(o),s=n(14),A=n.n(s),l=n(19),c=n.n(l),u=n(0),g=n.n(u),h=n(1),d=n.n(h),f=n(16),p=n.n(f);function I(t,e){for(var n=e;n;){if(n===t)return!0;n=n.parentNode}return!1}var C=n(70),m=n(56),y=n.n(m),E=function(t){function e(){var t,n,r,i;a()(this,e);for(var o=arguments.length,s=Array(o),l=0;l<o;l++)s[l]=arguments[l];return n=r=A()(this,(t=e.__proto__||Object.getPrototypeOf(e)).call.apply(t,[this].concat(s))),r.removeContainer=function(){r.container&&(p.a.unmountComponentAtNode(r.container),r.container.parentNode.removeChild(r.container),r.container=null)},r.renderComponent=function(t,e){var n=r.props,i=n.visible,o=n.getComponent,a=n.forceRender,s=n.getContainer,A=n.parent;(i||A._component||a)&&(r.container||(r.container=s()),p.a.unstable_renderSubtreeIntoContainer(A,o(t),r.container,function(){e&&e.call(this)}))},i=n,A()(r,i)}return c()(e,t),y()(e,[{key:"componentDidMount",value:function(){this.props.autoMount&&this.renderComponent()}},{key:"componentDidUpdate",value:function(){this.props.autoMount&&this.renderComponent()}},{key:"componentWillUnmount",value:function(){this.props.autoDestroy&&this.removeContainer()}},{key:"render",value:function(){return this.props.children({renderComponent:this.renderComponent,removeContainer:this.removeContainer})}}]),e}(g.a.Component);E.propTypes={autoMount:d.a.bool,autoDestroy:d.a.bool,visible:d.a.bool,forceRender:d.a.bool,parent:d.a.any,getComponent:d.a.func.isRequired,getContainer:d.a.func.isRequired,children:d.a.func.isRequired},E.defaultProps={autoMount:!0,autoDestroy:!0,forceRender:!1};var v=E,B=function(t){function e(){return a()(this,e),A()(this,(e.__proto__||Object.getPrototypeOf(e)).apply(this,arguments))}return c()(e,t),y()(e,[{key:"componentDidMount",value:function(){this.createContainer()}},{key:"componentDidUpdate",value:function(t){var e=this.props.didUpdate;e&&e(t)}},{key:"componentWillUnmount",value:function(){this.removeContainer()}},{key:"createContainer",value:function(){this._container=this.props.getContainer(),this.forceUpdate()}},{key:"removeContainer",value:function(){this._container&&this._container.parentNode.removeChild(this._container)}},{key:"render",value:function(){return this._container?p.a.createPortal(this.props.children,this._container):null}}]),e}(g.a.Component);B.propTypes={getContainer:d.a.func.isRequired,children:d.a.node.isRequired,didUpdate:d.a.func};var b=B,Q=n(4),w=n.n(Q);function x(t,e,n){return n?t[0]===e[0]:t[0]===e[0]&&t[1]===e[1]}function _(t,e){this[t]=e}var D=void 0,M={Webkit:"-webkit-",Moz:"-moz-",ms:"-ms-",O:"-o-"};function k(){if(void 0!==D)return D;D="";var t=document.createElement("p").style;for(var e in M)e+"Transform"in t&&(D=e);return D}function S(){return k()?k()+"TransitionProperty":"transitionProperty"}function L(){return k()?k()+"Transform":"transform"}function R(t,e){var n=S();n&&(t.style[n]=e,"transitionProperty"!==n&&(t.style.transitionProperty=e))}function N(t,e){var n=L();n&&(t.style[n]=e,"transform"!==n&&(t.style.transform=e))}var F=/matrix\((.*)\)/,T=/matrix3d\((.*)\)/;var U="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},O=void 0;function G(t){var e=t.style.display;t.style.display="none",t.offsetHeight,t.style.display=e}function H(t,e,n){var r=n;if("object"!==(void 0===e?"undefined":U(e)))return void 0!==r?("number"==typeof r&&(r+="px"),void(t.style[e]=r)):O(t,e);for(var i in e)e.hasOwnProperty(i)&&H(t,i,e[i])}function j(t,e){var n=t["page"+(e?"Y":"X")+"Offset"],r="scroll"+(e?"Top":"Left");if("number"!=typeof n){var i=t.document;"number"!=typeof(n=i.documentElement[r])&&(n=i.body[r])}return n}function Y(t){return j(t)}function P(t){return j(t,!0)}function K(t){var e=function(t){var e,n=void 0,r=void 0,i=t.ownerDocument,o=i.body,a=i&&i.documentElement;return n=(e=t.getBoundingClientRect()).left,r=e.top,{left:n-=a.clientLeft||o.clientLeft||0,top:r-=a.clientTop||o.clientTop||0}}(t),n=t.ownerDocument,r=n.defaultView||n.parentWindow;return e.left+=Y(r),e.top+=P(r),e}function z(t){return null!=t&&t==t.window}function J(t){return z(t)?t.document:9===t.nodeType?t:t.ownerDocument}var q=new RegExp("^("+/[\-+]?(?:\d*\.|)\d+(?:[eE][\-+]?\d+|)/.source+")(?!px)[a-z%]+$","i"),V=/^(top|right|bottom|left)$/,W="currentStyle",X="runtimeStyle",Z="left",$="px";function tt(t,e){return"left"===t?e.useCssRight?"right":t:e.useCssBottom?"bottom":t}function et(t){return"left"===t?"right":"right"===t?"left":"top"===t?"bottom":"bottom"===t?"top":void 0}function nt(t,e,n){"static"===H(t,"position")&&(t.style.position="relative");var r=-999,i=-999,o=tt("left",n),a=tt("top",n),s=et(o),A=et(a);"left"!==o&&(r=999),"top"!==a&&(i=999);var l,c="",u=K(t);("left"in e||"top"in e)&&(c=(l=t).style.transitionProperty||l.style[S()]||"",R(t,"none")),"left"in e&&(t.style[s]="",t.style[o]=r+"px"),"top"in e&&(t.style[A]="",t.style[a]=i+"px"),G(t);var g=K(t),h={};for(var d in e)if(e.hasOwnProperty(d)){var f=tt(d,n),p="left"===d?r:i,I=u[d]-g[d];h[f]=f===d?p+I:p-I}H(t,h),G(t),("left"in e||"top"in e)&&R(t,c);var C={};for(var m in e)if(e.hasOwnProperty(m)){var y=tt(m,n),E=e[m]-u[m];C[y]=m===y?h[y]+E:h[y]-E}H(t,C)}function rt(t,e){var n=K(t),r=function(t){var e=window.getComputedStyle(t,null),n=e.getPropertyValue("transform")||e.getPropertyValue(L());if(n&&"none"!==n){var r=n.replace(/[^0-9\-.,]/g,"").split(",");return{x:parseFloat(r[12]||r[4],0),y:parseFloat(r[13]||r[5],0)}}return{x:0,y:0}}(t),i={x:r.x,y:r.y};"left"in e&&(i.x=r.x+e.left-n.left),"top"in e&&(i.y=r.y+e.top-n.top),function(t,e){var n=window.getComputedStyle(t,null),r=n.getPropertyValue("transform")||n.getPropertyValue(L());if(r&&"none"!==r){var i=void 0,o=r.match(F);o?((i=(o=o[1]).split(",").map(function(t){return parseFloat(t,10)}))[4]=e.x,i[5]=e.y,N(t,"matrix("+i.join(",")+")")):((i=r.match(T)[1].split(",").map(function(t){return parseFloat(t,10)}))[12]=e.x,i[13]=e.y,N(t,"matrix3d("+i.join(",")+")"))}else N(t,"translateX("+e.x+"px) translateY("+e.y+"px) translateZ(0)")}(t,i)}function it(t,e){for(var n=0;n<t.length;n++)e(t[n])}function ot(t){return"border-box"===O(t,"boxSizing")}"undefined"!=typeof window&&(O=window.getComputedStyle?function(t,e,n){var r=n,i="",o=J(t);return(r=r||o.defaultView.getComputedStyle(t,null))&&(i=r.getPropertyValue(e)||r[e]),i}:function(t,e){var n=t[W]&&t[W][e];if(q.test(n)&&!V.test(e)){var r=t.style,i=r[Z],o=t[X][Z];t[X][Z]=t[W][Z],r[Z]="fontSize"===e?"1em":n||0,n=r.pixelLeft+$,r[Z]=i,t[X][Z]=o}return""===n?"auto":n});var at=["margin","border","padding"],st=-1,At=2,lt=1;function ct(t,e,n){var r=0,i=void 0,o=void 0,a=void 0;for(o=0;o<e.length;o++)if(i=e[o])for(a=0;a<n.length;a++){var s=void 0;s="border"===i?""+i+n[a]+"Width":i+n[a],r+=parseFloat(O(t,s))||0}return r}var ut={getParent:function(t){var e=t;do{e=11===e.nodeType&&e.host?e.host:e.parentNode}while(e&&1!==e.nodeType&&9!==e.nodeType);return e}};function gt(t,e,n){var r=n;if(z(t))return"width"===e?ut.viewportWidth(t):ut.viewportHeight(t);if(9===t.nodeType)return"width"===e?ut.docWidth(t):ut.docHeight(t);var i="width"===e?["Left","Right"]:["Top","Bottom"],o="width"===e?t.getBoundingClientRect().width:t.getBoundingClientRect().height,a=(O(t),ot(t)),s=0;(null==o||o<=0)&&(o=void 0,(null==(s=O(t,e))||Number(s)<0)&&(s=t.style[e]||0),s=parseFloat(s)||0),void 0===r&&(r=a?lt:st);var A=void 0!==o||a,l=o||s;return r===st?A?l-ct(t,["border","padding"],i):s:A?r===lt?l:l+(r===At?-ct(t,["border"],i):ct(t,["margin"],i)):s+ct(t,at.slice(r),i)}it(["Width","Height"],function(t){ut["doc"+t]=function(e){var n=e.document;return Math.max(n.documentElement["scroll"+t],n.body["scroll"+t],ut["viewport"+t](n))},ut["viewport"+t]=function(e){var n="client"+t,r=e.document,i=r.body,o=r.documentElement[n];return"CSS1Compat"===r.compatMode&&o||i&&i[n]||o}});var ht={position:"absolute",visibility:"hidden",display:"block"};function dt(){for(var t=arguments.length,e=Array(t),n=0;n<t;n++)e[n]=arguments[n];var r=void 0,i=e[0];return 0!==i.offsetWidth?r=gt.apply(void 0,e):function(t,e,n){var r={},i=t.style,o=void 0;for(o in e)e.hasOwnProperty(o)&&(r[o]=i[o],i[o]=e[o]);for(o in n.call(t),e)e.hasOwnProperty(o)&&(i[o]=r[o])}(i,ht,function(){r=gt.apply(void 0,e)}),r}function ft(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n]);return t}it(["width","height"],function(t){var e=t.charAt(0).toUpperCase()+t.slice(1);ut["outer"+e]=function(e,n){return e&&dt(e,t,n?0:lt)};var n="width"===t?["Left","Right"]:["Top","Bottom"];ut[t]=function(e,r){var i=r;if(void 0===i)return e&&dt(e,t,st);if(e){O(e);return ot(e)&&(i+=ct(e,["padding","border"],n)),H(e,t,i)}}});var pt={getWindow:function(t){if(t&&t.document&&t.setTimeout)return t;var e=t.ownerDocument||t;return e.defaultView||e.parentWindow},getDocument:J,offset:function(t,e,n){if(void 0===e)return K(t);!function(t,e,n){if(n.ignoreShake){var r=K(t),i=r.left.toFixed(0),o=r.top.toFixed(0),a=e.left.toFixed(0),s=e.top.toFixed(0);if(i===a&&o===s)return}n.useCssRight||n.useCssBottom?nt(t,e,n):n.useCssTransform&&L()in document.body.style?rt(t,e):nt(t,e,n)}(t,e,n||{})},isWindow:z,each:it,css:H,clone:function(t){var e=void 0,n={};for(e in t)t.hasOwnProperty(e)&&(n[e]=t[e]);if(t.overflow)for(e in t)t.hasOwnProperty(e)&&(n.overflow[e]=t.overflow[e]);return n},mix:ft,getWindowScrollLeft:function(t){return Y(t)},getWindowScrollTop:function(t){return P(t)},merge:function(){for(var t={},e=arguments.length,n=Array(e),r=0;r<e;r++)n[r]=arguments[r];for(var i=0;i<n.length;i++)pt.mix(t,n[i]);return t},viewportWidth:0,viewportHeight:0};ft(pt,ut);var It=pt,Ct=It.getParent;var mt=function(t){if(It.isWindow(t)||9===t.nodeType)return null;var e=It.getDocument(t).body,n=void 0,r=It.css(t,"position");if("fixed"!==r&&"absolute"!==r)return"html"===t.nodeName.toLowerCase()?null:Ct(t);for(n=Ct(t);n&&n!==e;n=Ct(n))if("static"!==(r=It.css(n,"position")))return n;return null},yt=It.getParent;var Et=function(t){for(var e={left:0,right:1/0,top:0,bottom:1/0},n=mt(t),r=It.getDocument(t),i=r.defaultView||r.parentWindow,o=r.body,a=r.documentElement;n;){if(-1!==navigator.userAgent.indexOf("MSIE")&&0===n.clientWidth||n===o||n===a||"visible"===It.css(n,"overflow")){if(n===o||n===a)break}else{var s=It.offset(n);s.left+=n.clientLeft,s.top+=n.clientTop,e.top=Math.max(e.top,s.top),e.right=Math.min(e.right,s.left+n.clientWidth),e.bottom=Math.min(e.bottom,s.top+n.clientHeight),e.left=Math.max(e.left,s.left)}n=mt(n)}var A=null;It.isWindow(t)||9===t.nodeType||(A=t.style.position,"absolute"===It.css(t,"position")&&(t.style.position="fixed"));var l=It.getWindowScrollLeft(i),c=It.getWindowScrollTop(i),u=It.viewportWidth(i),g=It.viewportHeight(i),h=a.scrollWidth,d=a.scrollHeight,f=window.getComputedStyle(o);if("hidden"===f.overflowX&&(h=i.innerWidth),"hidden"===f.overflowY&&(d=i.innerHeight),t.style&&(t.style.position=A),function(t){if(It.isWindow(t)||9===t.nodeType)return!1;var e=It.getDocument(t).body,n=null;for(n=yt(t);n&&n!==e;n=yt(n))if("fixed"===It.css(n,"position"))return!0;return!1}(t))e.left=Math.max(e.left,l),e.top=Math.max(e.top,c),e.right=Math.min(e.right,l+u),e.bottom=Math.min(e.bottom,c+g);else{var p=Math.max(h,l+u);e.right=Math.min(e.right,p);var I=Math.max(d,c+g);e.bottom=Math.min(e.bottom,I)}return e.top>=0&&e.left>=0&&e.bottom>e.top&&e.right>e.left?e:null};var vt=function(t,e,n,r){var i=It.clone(t),o={width:e.width,height:e.height};return r.adjustX&&i.left<n.left&&(i.left=n.left),r.resizeWidth&&i.left>=n.left&&i.left+o.width>n.right&&(o.width-=i.left+o.width-n.right),r.adjustX&&i.left+o.width>n.right&&(i.left=Math.max(n.right-o.width,n.left)),r.adjustY&&i.top<n.top&&(i.top=n.top),r.resizeHeight&&i.top>=n.top&&i.top+o.height>n.bottom&&(o.height-=i.top+o.height-n.bottom),r.adjustY&&i.top+o.height>n.bottom&&(i.top=Math.max(n.bottom-o.height,n.top)),It.mix(i,o)};var Bt=function(t){var e=void 0,n=void 0,r=void 0;if(It.isWindow(t)||9===t.nodeType){var i=It.getWindow(t);e={left:It.getWindowScrollLeft(i),top:It.getWindowScrollTop(i)},n=It.viewportWidth(i),r=It.viewportHeight(i)}else e=It.offset(t),n=It.outerWidth(t),r=It.outerHeight(t);return e.width=n,e.height=r,e};var bt=function(t,e){var n=e.charAt(0),r=e.charAt(1),i=t.width,o=t.height,a=t.left,s=t.top;return"c"===n?s+=o/2:"b"===n&&(s+=o),"c"===r?a+=i/2:"r"===r&&(a+=i),{left:a,top:s}};var Qt=function(t,e,n,r,i){var o=bt(e,n[1]),a=bt(t,n[0]),s=[a.left-o.left,a.top-o.top];return{left:Math.round(t.left-s[0]+r[0]-i[0]),top:Math.round(t.top-s[1]+r[1]-i[1])}};function wt(t,e,n){return t.left<n.left||t.left+e.width>n.right}function xt(t,e,n){return t.top<n.top||t.top+e.height>n.bottom}function _t(t,e,n){var r=[];return It.each(t,function(t){r.push(t.replace(e,function(t){return n[t]}))}),r}function Dt(t,e){return t[e]=-t[e],t}function Mt(t,e){return(/%$/.test(t)?parseInt(t.substring(0,t.length-1),10)/100*e:parseInt(t,10))||0}function kt(t,e){t[0]=Mt(t[0],e.width),t[1]=Mt(t[1],e.height)}var St=function(t,e,n,r){var i=n.points,o=n.offset||[0,0],a=n.targetOffset||[0,0],s=n.overflow,A=n.source||t;o=[].concat(o),a=[].concat(a),s=s||{};var l={},c=0,u=Et(A),g=Bt(A);kt(o,g),kt(a,e);var h=Qt(g,e,i,o,a),d=It.merge(g,h);if(u&&(s.adjustX||s.adjustY)&&r){if(s.adjustX&&wt(h,g,u)){var f=_t(i,/[lr]/gi,{l:"r",r:"l"}),p=Dt(o,0),I=Dt(a,0);(function(t,e,n){return t.left>n.right||t.left+e.width<n.left})(Qt(g,e,f,p,I),g,u)||(c=1,i=f,o=p,a=I)}if(s.adjustY&&xt(h,g,u)){var C=_t(i,/[tb]/gi,{t:"b",b:"t"}),m=Dt(o,1),y=Dt(a,1);(function(t,e,n){return t.top>n.bottom||t.top+e.height<n.top})(Qt(g,e,C,m,y),g,u)||(c=1,i=C,o=m,a=y)}c&&(h=Qt(g,e,i,o,a),It.mix(d,h));var E=wt(h,g,u),v=xt(h,g,u);(E||v)&&(i=n.points,o=n.offset||[0,0],a=n.targetOffset||[0,0]),l.adjustX=s.adjustX&&E,l.adjustY=s.adjustY&&v,(l.adjustX||l.adjustY)&&(d=vt(h,g,u,l))}return d.width!==g.width&&It.css(A,"width",It.width(A)+d.width-g.width),d.height!==g.height&&It.css(A,"height",It.height(A)+d.height-g.height),It.offset(A,{left:d.left,top:d.top},{useCssRight:n.useCssRight,useCssBottom:n.useCssBottom,useCssTransform:n.useCssTransform,ignoreShake:n.ignoreShake}),{points:i,offset:o,targetOffset:a,overflow:l}};function Lt(t,e,n){var r=n.target||e,i=Bt(r),o=!function(t){var e=Et(t),n=Bt(t);return!e||n.left+n.width<=e.left||n.top+n.height<=e.top||n.left>=e.right||n.top>=e.bottom}(r);return St(t,i,n,o)}Lt.__getOffsetParent=mt,Lt.__getVisibleRectForElement=Et;var Rt=Lt,Nt=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t};var Ft=function(t,e,n){var r=void 0,i=void 0,o=It.getDocument(t),a=o.defaultView||o.parentWindow,s=It.getWindowScrollLeft(a),A=It.getWindowScrollTop(a),l=It.viewportWidth(a),c=It.viewportHeight(a),u={left:r="pageX"in e?e.pageX:s+e.clientX,top:i="pageY"in e?e.pageY:A+e.clientY,width:0,height:0},g=r>=0&&r<=s+l&&i>=0&&i<=A+c,h=[n.points[0],"cc"];return St(t,u,Nt({},n,{points:h}),g)};function Tt(t){return t&&"object"==typeof t&&t.window===t}function Ut(t,e){var n=Math.floor(t),r=Math.floor(e);return Math.abs(n-r)<=1}function Ot(t){return"function"==typeof t&&t?t():null}function Gt(t){return"object"==typeof t&&t?t:null}var Ht=function(t){function e(){var t,n,r,i;a()(this,e);for(var o=arguments.length,s=Array(o),l=0;l<o;l++)s[l]=arguments[l];return n=r=A()(this,(t=e.__proto__||Object.getPrototypeOf(e)).call.apply(t,[this].concat(s))),r.forceAlign=function(){var t=r.props,e=t.disabled,n=t.target,i=t.align,o=t.onAlign;if(!e&&n){var a=p.a.findDOMNode(r),s=void 0,A=Ot(n),l=Gt(n),c=document.activeElement;A?s=Rt(a,A,i):l&&(s=Ft(a,l,i)),function(t,e){t!==document.activeElement&&I(e,t)&&t.focus()}(c,a),o&&o(a,s)}},i=n,A()(r,i)}return c()(e,t),y()(e,[{key:"componentDidMount",value:function(){var t=this.props;this.forceAlign(),!t.disabled&&t.monitorWindowResize&&this.startMonitorWindowResize()}},{key:"componentDidUpdate",value:function(t){var e,n,r=!1,i=this.props;if(!i.disabled){var o=p.a.findDOMNode(this),a=o?o.getBoundingClientRect():null;if(t.disabled)r=!0;else{var s=Ot(t.target),A=Ot(i.target),l=Gt(t.target),c=Gt(i.target);Tt(s)&&Tt(A)?r=!1:(s!==A||s&&!A&&c||l&&c&&A||c&&!((e=l)===(n=c)||e&&n&&("pageX"in n&&"pageY"in n?e.pageX===n.pageX&&e.pageY===n.pageY:"clientX"in n&&"clientY"in n&&e.clientX===n.clientX&&e.clientY===n.clientY)))&&(r=!0);var u=this.sourceRect||{};r||!o||Ut(u.width,a.width)&&Ut(u.height,a.height)||(r=!0)}this.sourceRect=a}r&&this.forceAlign(),i.monitorWindowResize&&!i.disabled?this.startMonitorWindowResize():this.stopMonitorWindowResize()}},{key:"componentWillUnmount",value:function(){this.stopMonitorWindowResize()}},{key:"startMonitorWindowResize",value:function(){this.resizeHandler||(this.bufferMonitor=function(t,e){var n=void 0;function r(){n&&(clearTimeout(n),n=null)}function i(){r(),n=setTimeout(t,e)}return i.clear=r,i}(this.forceAlign,this.props.monitorBufferTime),this.resizeHandler=Object(C.a)(window,"resize",this.bufferMonitor))}},{key:"stopMonitorWindowResize",value:function(){this.resizeHandler&&(this.bufferMonitor.clear(),this.resizeHandler.remove(),this.resizeHandler=null)}},{key:"render",value:function(){var t=this,e=this.props,n=e.childrenProps,r=e.children,i=g.a.Children.only(r);if(n){var o={};return Object.keys(n).forEach(function(e){o[e]=t.props[n[e]]}),g.a.cloneElement(i,o)}return i}}]),e}(u.Component);Ht.propTypes={childrenProps:d.a.object,align:d.a.object.isRequired,target:d.a.oneOfType([d.a.func,d.a.shape({clientX:d.a.number,clientY:d.a.number,pageX:d.a.number,pageY:d.a.number})]),onAlign:d.a.func,monitorBufferTime:d.a.number,monitorWindowResize:d.a.bool,disabled:d.a.bool,children:d.a.any},Ht.defaultProps={target:function(){return window},monitorBufferTime:50,monitorWindowResize:!1,disabled:!1};var jt=Ht,Yt=n(117),Pt=n(57),Kt=n.n(Pt),zt=function(t){function e(){return a()(this,e),A()(this,t.apply(this,arguments))}return c()(e,t),e.prototype.shouldComponentUpdate=function(t){return t.hiddenClassName||t.visible},e.prototype.render=function(){var t=this.props,e=t.hiddenClassName,n=t.visible,r=Kt()(t,["hiddenClassName","visible"]);return e||g.a.Children.count(r.children)>1?(!n&&e&&(r.className+=" "+e),g.a.createElement("div",r)):g.a.Children.only(r.children)},e}(u.Component);zt.propTypes={children:d.a.any,className:d.a.string,visible:d.a.bool,hiddenClassName:d.a.string};var Jt=zt,qt=function(t){function e(){return a()(this,e),A()(this,t.apply(this,arguments))}return c()(e,t),e.prototype.render=function(){var t=this.props,e=t.className;return t.visible||(e+=" "+t.hiddenClassName),g.a.createElement("div",{className:e,onMouseEnter:t.onMouseEnter,onMouseLeave:t.onMouseLeave,onMouseDown:t.onMouseDown,onTouchStart:t.onTouchStart,style:t.style},g.a.createElement(Jt,{className:t.prefixCls+"-content",visible:t.visible},t.children))},e}(u.Component);qt.propTypes={hiddenClassName:d.a.string,className:d.a.string,prefixCls:d.a.string,onMouseEnter:d.a.func,onMouseLeave:d.a.func,onMouseDown:d.a.func,onTouchStart:d.a.func,children:d.a.any};var Vt=qt,Wt=function(t){function e(n){a()(this,e);var r=A()(this,t.call(this,n));return Xt.call(r),r.state={stretchChecked:!1,targetWidth:void 0,targetHeight:void 0},r.savePopupRef=_.bind(r,"popupInstance"),r.saveAlignRef=_.bind(r,"alignInstance"),r}return c()(e,t),e.prototype.componentDidMount=function(){this.rootNode=this.getPopupDomNode(),this.setStretchSize()},e.prototype.componentDidUpdate=function(){this.setStretchSize()},e.prototype.getPopupDomNode=function(){return p.a.findDOMNode(this.popupInstance)},e.prototype.getMaskTransitionName=function(){var t=this.props,e=t.maskTransitionName,n=t.maskAnimation;return!e&&n&&(e=t.prefixCls+"-"+n),e},e.prototype.getTransitionName=function(){var t=this.props,e=t.transitionName;return!e&&t.animation&&(e=t.prefixCls+"-"+t.animation),e},e.prototype.getClassName=function(t){return this.props.prefixCls+" "+this.props.className+" "+t},e.prototype.getPopupElement=function(){var t=this,e=this.savePopupRef,n=this.state,r=n.stretchChecked,o=n.targetHeight,a=n.targetWidth,s=this.props,A=s.align,l=s.visible,c=s.prefixCls,u=s.style,h=s.getClassNameFromAlign,d=s.destroyPopupOnHide,f=s.stretch,p=s.children,I=s.onMouseEnter,C=s.onMouseLeave,m=s.onMouseDown,y=s.onTouchStart,E=this.getClassName(this.currentAlignClassName||h(A)),v=c+"-hidden";l||(this.currentAlignClassName=null);var B={};f&&(-1!==f.indexOf("height")?B.height=o:-1!==f.indexOf("minHeight")&&(B.minHeight=o),-1!==f.indexOf("width")?B.width=a:-1!==f.indexOf("minWidth")&&(B.minWidth=a),r||(B.visibility="hidden",setTimeout(function(){t.alignInstance&&t.alignInstance.forceAlign()},0)));var b={className:E,prefixCls:c,ref:e,onMouseEnter:I,onMouseLeave:C,onMouseDown:m,onTouchStart:y,style:i()({},B,u,this.getZIndexStyle())};return d?g.a.createElement(Yt.a,{component:"",exclusive:!0,transitionAppear:!0,transitionName:this.getTransitionName()},l?g.a.createElement(jt,{target:this.getAlignTarget(),key:"popup",ref:this.saveAlignRef,monitorWindowResize:!0,align:A,onAlign:this.onAlign},g.a.createElement(Vt,i()({visible:!0},b),p)):null):g.a.createElement(Yt.a,{component:"",exclusive:!0,transitionAppear:!0,transitionName:this.getTransitionName(),showProp:"xVisible"},g.a.createElement(jt,{target:this.getAlignTarget(),key:"popup",ref:this.saveAlignRef,monitorWindowResize:!0,xVisible:l,childrenProps:{visible:"xVisible"},disabled:!l,align:A,onAlign:this.onAlign},g.a.createElement(Vt,i()({hiddenClassName:v},b),p)))},e.prototype.getZIndexStyle=function(){var t={},e=this.props;return void 0!==e.zIndex&&(t.zIndex=e.zIndex),t},e.prototype.getMaskElement=function(){var t=this.props,e=void 0;if(t.mask){var n=this.getMaskTransitionName();e=g.a.createElement(Jt,{style:this.getZIndexStyle(),key:"mask",className:t.prefixCls+"-mask",hiddenClassName:t.prefixCls+"-mask-hidden",visible:t.visible}),n&&(e=g.a.createElement(Yt.a,{key:"mask",showProp:"visible",transitionAppear:!0,component:"",transitionName:n},e))}return e},e.prototype.render=function(){return g.a.createElement("div",null,this.getMaskElement(),this.getPopupElement())},e}(u.Component);Wt.propTypes={visible:d.a.bool,style:d.a.object,getClassNameFromAlign:d.a.func,onAlign:d.a.func,getRootDomNode:d.a.func,align:d.a.any,destroyPopupOnHide:d.a.bool,className:d.a.string,prefixCls:d.a.string,onMouseEnter:d.a.func,onMouseLeave:d.a.func,onMouseDown:d.a.func,onTouchStart:d.a.func,stretch:d.a.string,children:d.a.node,point:d.a.shape({pageX:d.a.number,pageY:d.a.number})};var Xt=function(){var t=this;this.onAlign=function(e,n){var r=t.props,i=r.getClassNameFromAlign(n);t.currentAlignClassName!==i&&(t.currentAlignClassName=i,e.className=t.getClassName(i)),r.onAlign(e,n)},this.setStretchSize=function(){var e=t.props,n=e.stretch,r=e.getRootDomNode,i=e.visible,o=t.state,a=o.stretchChecked,s=o.targetHeight,A=o.targetWidth;if(n&&i){var l=r();if(l){var c=l.offsetHeight,u=l.offsetWidth;s===c&&A===u&&a||t.setState({stretchChecked:!0,targetHeight:c,targetWidth:u})}}else a&&t.setState({stretchChecked:!1})},this.getTargetElement=function(){return t.props.getRootDomNode()},this.getAlignTarget=function(){var e=t.props.point;return e||t.getTargetElement}},Zt=Wt;function $t(){}var te=["onClick","onMouseDown","onTouchStart","onMouseEnter","onMouseLeave","onFocus","onBlur","onContextMenu"],ee=!!f.createPortal,ne={rcTrigger:d.a.shape({onPopupMouseDown:d.a.func})},re=function(t){function e(n){a()(this,e);var r=A()(this,t.call(this,n));ie.call(r);var i=void 0;return i="popupVisible"in n?!!n.popupVisible:!!n.defaultPopupVisible,r.prevPopupVisible=i,r.state={popupVisible:i},r}return c()(e,t),e.prototype.getChildContext=function(){return{rcTrigger:{onPopupMouseDown:this.onPopupMouseDown}}},e.prototype.componentWillMount=function(){var t=this;te.forEach(function(e){t["fire"+e]=function(n){t.fireEvents(e,n)}})},e.prototype.componentDidMount=function(){this.componentDidUpdate({},{popupVisible:this.state.popupVisible})},e.prototype.componentWillReceiveProps=function(t){var e=t.popupVisible;void 0!==e&&this.setState({popupVisible:e})},e.prototype.componentDidUpdate=function(t,e){var n=this.props,r=this.state;if(ee||this.renderComponent(null,function(){e.popupVisible!==r.popupVisible&&n.afterPopupVisibleChange(r.popupVisible)}),this.prevPopupVisible=e.popupVisible,r.popupVisible){var i=void 0;return this.clickOutsideHandler||!this.isClickToHide()&&!this.isContextMenuToShow()||(i=n.getDocument(),this.clickOutsideHandler=Object(C.a)(i,"mousedown",this.onDocumentClick)),this.touchOutsideHandler||(i=i||n.getDocument(),this.touchOutsideHandler=Object(C.a)(i,"touchstart",this.onDocumentClick)),!this.contextMenuOutsideHandler1&&this.isContextMenuToShow()&&(i=i||n.getDocument(),this.contextMenuOutsideHandler1=Object(C.a)(i,"scroll",this.onContextMenuClose)),void(!this.contextMenuOutsideHandler2&&this.isContextMenuToShow()&&(this.contextMenuOutsideHandler2=Object(C.a)(window,"blur",this.onContextMenuClose)))}this.clearOutsideHandler()},e.prototype.componentWillUnmount=function(){this.clearDelayTimer(),this.clearOutsideHandler(),clearTimeout(this.mouseDownTimeout)},e.prototype.getPopupDomNode=function(){return this._component&&this._component.getPopupDomNode?this._component.getPopupDomNode():null},e.prototype.getPopupAlign=function(){var t=this.props,e=t.popupPlacement,n=t.popupAlign,r=t.builtinPlacements;return e&&r?function(t,e,n){var r=t[e]||{};return i()({},r,n)}(r,e,n):n},e.prototype.setPopupVisible=function(t,e){var n=this.props.alignPoint;this.clearDelayTimer(),this.state.popupVisible!==t&&("popupVisible"in this.props||this.setState({popupVisible:t}),this.props.onPopupVisibleChange(t)),n&&e&&this.setPoint(e)},e.prototype.delaySetPopupVisible=function(t,e,n){var r=this,i=1e3*e;if(this.clearDelayTimer(),i){var o=n?{pageX:n.pageX,pageY:n.pageY}:null;this.delayTimer=setTimeout(function(){r.setPopupVisible(t,o),r.clearDelayTimer()},i)}else this.setPopupVisible(t,n)},e.prototype.clearDelayTimer=function(){this.delayTimer&&(clearTimeout(this.delayTimer),this.delayTimer=null)},e.prototype.clearOutsideHandler=function(){this.clickOutsideHandler&&(this.clickOutsideHandler.remove(),this.clickOutsideHandler=null),this.contextMenuOutsideHandler1&&(this.contextMenuOutsideHandler1.remove(),this.contextMenuOutsideHandler1=null),this.contextMenuOutsideHandler2&&(this.contextMenuOutsideHandler2.remove(),this.contextMenuOutsideHandler2=null),this.touchOutsideHandler&&(this.touchOutsideHandler.remove(),this.touchOutsideHandler=null)},e.prototype.createTwoChains=function(t){var e=this.props.children.props,n=this.props;return e[t]&&n[t]?this["fire"+t]:e[t]||n[t]},e.prototype.isClickToShow=function(){var t=this.props,e=t.action,n=t.showAction;return-1!==e.indexOf("click")||-1!==n.indexOf("click")},e.prototype.isContextMenuToShow=function(){var t=this.props,e=t.action,n=t.showAction;return-1!==e.indexOf("contextMenu")||-1!==n.indexOf("contextMenu")},e.prototype.isClickToHide=function(){var t=this.props,e=t.action,n=t.hideAction;return-1!==e.indexOf("click")||-1!==n.indexOf("click")},e.prototype.isMouseEnterToShow=function(){var t=this.props,e=t.action,n=t.showAction;return-1!==e.indexOf("hover")||-1!==n.indexOf("mouseEnter")},e.prototype.isMouseLeaveToHide=function(){var t=this.props,e=t.action,n=t.hideAction;return-1!==e.indexOf("hover")||-1!==n.indexOf("mouseLeave")},e.prototype.isFocusToShow=function(){var t=this.props,e=t.action,n=t.showAction;return-1!==e.indexOf("focus")||-1!==n.indexOf("focus")},e.prototype.isBlurToHide=function(){var t=this.props,e=t.action,n=t.hideAction;return-1!==e.indexOf("focus")||-1!==n.indexOf("blur")},e.prototype.forcePopupAlign=function(){this.state.popupVisible&&this._component&&this._component.alignInstance&&this._component.alignInstance.forceAlign()},e.prototype.fireEvents=function(t,e){var n=this.props.children.props[t];n&&n(e);var r=this.props[t];r&&r(e)},e.prototype.close=function(){this.setPopupVisible(!1)},e.prototype.render=function(){var t=this,e=this.state.popupVisible,n=this.props,r=n.children,i=n.forceRender,o=n.alignPoint,a=n.className,s=g.a.Children.only(r),A={key:"trigger"};this.isContextMenuToShow()?A.onContextMenu=this.onContextMenu:A.onContextMenu=this.createTwoChains("onContextMenu"),this.isClickToHide()||this.isClickToShow()?(A.onClick=this.onClick,A.onMouseDown=this.onMouseDown,A.onTouchStart=this.onTouchStart):(A.onClick=this.createTwoChains("onClick"),A.onMouseDown=this.createTwoChains("onMouseDown"),A.onTouchStart=this.createTwoChains("onTouchStart")),this.isMouseEnterToShow()?(A.onMouseEnter=this.onMouseEnter,o&&(A.onMouseMove=this.onMouseMove)):A.onMouseEnter=this.createTwoChains("onMouseEnter"),this.isMouseLeaveToHide()?A.onMouseLeave=this.onMouseLeave:A.onMouseLeave=this.createTwoChains("onMouseLeave"),this.isFocusToShow()||this.isBlurToHide()?(A.onFocus=this.onFocus,A.onBlur=this.onBlur):(A.onFocus=this.createTwoChains("onFocus"),A.onBlur=this.createTwoChains("onBlur"));var l=w()(s&&s.props&&s.props.className,a);l&&(A.className=l);var c=g.a.cloneElement(s,A);if(!ee)return g.a.createElement(v,{parent:this,visible:e,autoMount:!1,forceRender:i,getComponent:this.getComponent,getContainer:this.getContainer},function(e){var n=e.renderComponent;return t.renderComponent=n,c});var u=void 0;return(e||this._component||i)&&(u=g.a.createElement(b,{key:"portal",getContainer:this.getContainer,didUpdate:this.handlePortalUpdate},this.getComponent())),[c,u]},e}(g.a.Component);re.propTypes={children:d.a.any,action:d.a.oneOfType([d.a.string,d.a.arrayOf(d.a.string)]),showAction:d.a.any,hideAction:d.a.any,getPopupClassNameFromAlign:d.a.any,onPopupVisibleChange:d.a.func,afterPopupVisibleChange:d.a.func,popup:d.a.oneOfType([d.a.node,d.a.func]).isRequired,popupStyle:d.a.object,prefixCls:d.a.string,popupClassName:d.a.string,className:d.a.string,popupPlacement:d.a.string,builtinPlacements:d.a.object,popupTransitionName:d.a.oneOfType([d.a.string,d.a.object]),popupAnimation:d.a.any,mouseEnterDelay:d.a.number,mouseLeaveDelay:d.a.number,zIndex:d.a.number,focusDelay:d.a.number,blurDelay:d.a.number,getPopupContainer:d.a.func,getDocument:d.a.func,forceRender:d.a.bool,destroyPopupOnHide:d.a.bool,mask:d.a.bool,maskClosable:d.a.bool,onPopupAlign:d.a.func,popupAlign:d.a.object,popupVisible:d.a.bool,defaultPopupVisible:d.a.bool,maskTransitionName:d.a.oneOfType([d.a.string,d.a.object]),maskAnimation:d.a.string,stretch:d.a.string,alignPoint:d.a.bool},re.contextTypes=ne,re.childContextTypes=ne,re.defaultProps={prefixCls:"rc-trigger-popup",getPopupClassNameFromAlign:function(){return""},getDocument:function(){return window.document},onPopupVisibleChange:$t,afterPopupVisibleChange:$t,onPopupAlign:$t,popupClassName:"",mouseEnterDelay:0,mouseLeaveDelay:.1,focusDelay:0,blurDelay:.15,popupStyle:{},destroyPopupOnHide:!1,popupAlign:{},defaultPopupVisible:!1,mask:!1,maskClosable:!0,action:[],showAction:[],hideAction:[]};var ie=function(){var t=this;this.onMouseEnter=function(e){var n=t.props.mouseEnterDelay;t.fireEvents("onMouseEnter",e),t.delaySetPopupVisible(!0,n,n?null:e)},this.onMouseMove=function(e){t.fireEvents("onMouseMove",e),t.setPoint(e)},this.onMouseLeave=function(e){t.fireEvents("onMouseLeave",e),t.delaySetPopupVisible(!1,t.props.mouseLeaveDelay)},this.onPopupMouseEnter=function(){t.clearDelayTimer()},this.onPopupMouseLeave=function(e){e.relatedTarget&&!e.relatedTarget.setTimeout&&t._component&&t._component.getPopupDomNode&&I(t._component.getPopupDomNode(),e.relatedTarget)||t.delaySetPopupVisible(!1,t.props.mouseLeaveDelay)},this.onFocus=function(e){t.fireEvents("onFocus",e),t.clearDelayTimer(),t.isFocusToShow()&&(t.focusTime=Date.now(),t.delaySetPopupVisible(!0,t.props.focusDelay))},this.onMouseDown=function(e){t.fireEvents("onMouseDown",e),t.preClickTime=Date.now()},this.onTouchStart=function(e){t.fireEvents("onTouchStart",e),t.preTouchTime=Date.now()},this.onBlur=function(e){t.fireEvents("onBlur",e),t.clearDelayTimer(),t.isBlurToHide()&&t.delaySetPopupVisible(!1,t.props.blurDelay)},this.onContextMenu=function(e){e.preventDefault(),t.fireEvents("onContextMenu",e),t.setPopupVisible(!0,e)},this.onContextMenuClose=function(){t.isContextMenuToShow()&&t.close()},this.onClick=function(e){if(t.fireEvents("onClick",e),t.focusTime){var n=void 0;if(t.preClickTime&&t.preTouchTime?n=Math.min(t.preClickTime,t.preTouchTime):t.preClickTime?n=t.preClickTime:t.preTouchTime&&(n=t.preTouchTime),Math.abs(n-t.focusTime)<20)return;t.focusTime=0}t.preClickTime=0,t.preTouchTime=0,t.isClickToShow()&&(t.isClickToHide()||t.isBlurToHide())&&e&&e.preventDefault&&e.preventDefault();var r=!t.state.popupVisible;(t.isClickToHide()&&!r||r&&t.isClickToShow())&&t.setPopupVisible(!t.state.popupVisible,e)},this.onPopupMouseDown=function(){var e=t.context.rcTrigger,n=void 0===e?{}:e;t.hasPopupMouseDown=!0,clearTimeout(t.mouseDownTimeout),t.mouseDownTimeout=setTimeout(function(){t.hasPopupMouseDown=!1},0),n.onPopupMouseDown&&n.onPopupMouseDown.apply(n,arguments)},this.onDocumentClick=function(e){if(!t.props.mask||t.props.maskClosable){var n=e.target;I(Object(f.findDOMNode)(t),n)||t.hasPopupMouseDown||t.close()}},this.getRootDomNode=function(){return Object(f.findDOMNode)(t)},this.getPopupClassNameFromAlign=function(e){var n=[],r=t.props,i=r.popupPlacement,o=r.builtinPlacements,a=r.prefixCls,s=r.alignPoint,A=r.getPopupClassNameFromAlign;return i&&o&&n.push(function(t,e,n,r){var i=n.points;for(var o in t)if(t.hasOwnProperty(o)&&x(t[o].points,i,r))return e+"-placement-"+o;return""}(o,a,e,s)),A&&n.push(A(e)),n.join(" ")},this.getComponent=function(){var e=t.props,n=e.prefixCls,r=e.destroyPopupOnHide,o=e.popupClassName,a=e.action,s=e.onPopupAlign,A=e.popupAnimation,l=e.popupTransitionName,c=e.popupStyle,u=e.mask,h=e.maskAnimation,d=e.maskTransitionName,f=e.zIndex,p=e.popup,I=e.stretch,C=e.alignPoint,m=t.state,y=m.popupVisible,E=m.point,v=t.getPopupAlign(),B={};return t.isMouseEnterToShow()&&(B.onMouseEnter=t.onPopupMouseEnter),t.isMouseLeaveToHide()&&(B.onMouseLeave=t.onPopupMouseLeave),B.onMouseDown=t.onPopupMouseDown,B.onTouchStart=t.onPopupMouseDown,g.a.createElement(Zt,i()({prefixCls:n,destroyPopupOnHide:r,visible:y,point:C&&E,className:o,action:a,align:v,onAlign:s,animation:A,getClassNameFromAlign:t.getPopupClassNameFromAlign},B,{stretch:I,getRootDomNode:t.getRootDomNode,style:c,mask:u,zIndex:f,transitionName:l,maskAnimation:h,maskTransitionName:d,ref:t.savePopup}),"function"==typeof p?p():p)},this.getContainer=function(){var e=t.props,n=document.createElement("div");return n.style.position="absolute",n.style.top="0",n.style.left="0",n.style.width="100%",(e.getPopupContainer?e.getPopupContainer(Object(f.findDOMNode)(t)):e.getDocument().body).appendChild(n),n},this.setPoint=function(e){t.props.alignPoint&&e&&t.setState({point:{pageX:e.pageX,pageY:e.pageY}})},this.handlePortalUpdate=function(){t.prevPopupVisible!==t.state.popupVisible&&t.props.afterPopupVisibleChange(t.state.popupVisible)},this.savePopup=function(e){t._component=e}};e.default=re},function(t,e,n){"use strict";var r=n(13),i=n.n(r),o=n(47),a=n.n(o),s=n(18),A=n.n(s),l=n(56),c=n.n(l),u=n(14),g=n.n(u),h=n(19),d=n.n(h),f=n(0),p=n.n(f),I=n(1),C=n.n(I);function m(t){var e=[];return p.a.Children.forEach(t,function(t){e.push(t)}),e}function y(t,e){var n=null;return t&&t.forEach(function(t){n||t&&t.key===e&&(n=t)}),n}function E(t,e,n){var r=null;return t&&t.forEach(function(t){if(t&&t.key===e&&t.props[n]){if(r)throw new Error("two child with same key for <rc-animate> children");r=t}}),r}var v=n(16),B=n.n(v),b=n(254),Q=n.n(b),w={transitionstart:{transition:"transitionstart",WebkitTransition:"webkitTransitionStart",MozTransition:"mozTransitionStart",OTransition:"oTransitionStart",msTransition:"MSTransitionStart"},animationstart:{animation:"animationstart",WebkitAnimation:"webkitAnimationStart",MozAnimation:"mozAnimationStart",OAnimation:"oAnimationStart",msAnimation:"MSAnimationStart"}},x={transitionend:{transition:"transitionend",WebkitTransition:"webkitTransitionEnd",MozTransition:"mozTransitionEnd",OTransition:"oTransitionEnd",msTransition:"MSTransitionEnd"},animationend:{animation:"animationend",WebkitAnimation:"webkitAnimationEnd",MozAnimation:"mozAnimationEnd",OAnimation:"oAnimationEnd",msAnimation:"MSAnimationEnd"}},_=[],D=[];function M(t,e,n){t.addEventListener(e,n,!1)}function k(t,e,n){t.removeEventListener(e,n,!1)}"undefined"!=typeof window&&"undefined"!=typeof document&&function(){var t=document.createElement("div").style;function e(e,n){for(var r in e)if(e.hasOwnProperty(r)){var i=e[r];for(var o in i)if(o in t){n.push(i[o]);break}}}"AnimationEvent"in window||(delete w.animationstart.animation,delete x.animationend.animation),"TransitionEvent"in window||(delete w.transitionstart.transition,delete x.transitionend.transition),e(w,_),e(x,D)}();var S={startEvents:_,addStartEventListener:function(t,e){0!==_.length?_.forEach(function(n){M(t,n,e)}):window.setTimeout(e,0)},removeStartEventListener:function(t,e){0!==_.length&&_.forEach(function(n){k(t,n,e)})},endEvents:D,addEndEventListener:function(t,e){0!==D.length?D.forEach(function(n){M(t,n,e)}):window.setTimeout(e,0)},removeEndEventListener:function(t,e){0!==D.length&&D.forEach(function(n){k(t,n,e)})}},L=n(202),R=n.n(L),N=0!==S.endEvents.length,F=["Webkit","Moz","O","ms"],T=["-webkit-","-moz-","-o-","ms-",""];function U(t,e){for(var n=window.getComputedStyle(t,null),r="",i=0;i<T.length&&!(r=n.getPropertyValue(T[i]+e));i++);return r}function O(t){if(N){var e=parseFloat(U(t,"transition-delay"))||0,n=parseFloat(U(t,"transition-duration"))||0,r=parseFloat(U(t,"animation-delay"))||0,i=parseFloat(U(t,"animation-duration"))||0,o=Math.max(n+e,i+r);t.rcEndAnimTimeout=setTimeout(function(){t.rcEndAnimTimeout=null,t.rcEndListener&&t.rcEndListener()},1e3*o+200)}}function G(t){t.rcEndAnimTimeout&&(clearTimeout(t.rcEndAnimTimeout),t.rcEndAnimTimeout=null)}var H=function(t,e,n){var r="object"===(void 0===e?"undefined":Q()(e)),i=r?e.name:e,o=r?e.active:e+"-active",a=n,s=void 0,A=void 0,l=R()(t);return n&&"[object Object]"===Object.prototype.toString.call(n)&&(a=n.end,s=n.start,A=n.active),t.rcEndListener&&t.rcEndListener(),t.rcEndListener=function(e){e&&e.target!==t||(t.rcAnimTimeout&&(clearTimeout(t.rcAnimTimeout),t.rcAnimTimeout=null),G(t),l.remove(i),l.remove(o),S.removeEndEventListener(t,t.rcEndListener),t.rcEndListener=null,a&&a())},S.addEndEventListener(t,t.rcEndListener),s&&s(),l.add(i),t.rcAnimTimeout=setTimeout(function(){t.rcAnimTimeout=null,l.add(o),A&&setTimeout(A,0),O(t)},30),{stop:function(){t.rcEndListener&&t.rcEndListener()}}};H.style=function(t,e,n){t.rcEndListener&&t.rcEndListener(),t.rcEndListener=function(e){e&&e.target!==t||(t.rcAnimTimeout&&(clearTimeout(t.rcAnimTimeout),t.rcAnimTimeout=null),G(t),S.removeEndEventListener(t,t.rcEndListener),t.rcEndListener=null,n&&n())},S.addEndEventListener(t,t.rcEndListener),t.rcAnimTimeout=setTimeout(function(){for(var n in e)e.hasOwnProperty(n)&&(t.style[n]=e[n]);t.rcAnimTimeout=null,O(t)},0)},H.setTransition=function(t,e,n){var r=e,i=n;void 0===n&&(i=r,r=""),r=r||"",F.forEach(function(e){t.style[e+"Transition"+r]=i})},H.isCssAnimationSupported=N;var j=H,Y={isAppearSupported:function(t){return t.transitionName&&t.transitionAppear||t.animation.appear},isEnterSupported:function(t){return t.transitionName&&t.transitionEnter||t.animation.enter},isLeaveSupported:function(t){return t.transitionName&&t.transitionLeave||t.animation.leave},allowAppearCallback:function(t){return t.transitionAppear||t.animation.appear},allowEnterCallback:function(t){return t.transitionEnter||t.animation.enter},allowLeaveCallback:function(t){return t.transitionLeave||t.animation.leave}},P={enter:"transitionEnter",appear:"transitionAppear",leave:"transitionLeave"},K=function(t){function e(){return A()(this,e),g()(this,(e.__proto__||Object.getPrototypeOf(e)).apply(this,arguments))}return d()(e,t),c()(e,[{key:"componentWillUnmount",value:function(){this.stop()}},{key:"componentWillEnter",value:function(t){Y.isEnterSupported(this.props)?this.transition("enter",t):t()}},{key:"componentWillAppear",value:function(t){Y.isAppearSupported(this.props)?this.transition("appear",t):t()}},{key:"componentWillLeave",value:function(t){Y.isLeaveSupported(this.props)?this.transition("leave",t):t()}},{key:"transition",value:function(t,e){var n=this,r=B.a.findDOMNode(this),i=this.props,o=i.transitionName,a="object"==typeof o;this.stop();var s=function(){n.stopper=null,e()};if((N||!i.animation[t])&&o&&i[P[t]]){var A=a?o[t]:o+"-"+t,l=A+"-active";a&&o[t+"Active"]&&(l=o[t+"Active"]),this.stopper=j(r,{name:A,active:l},s)}else this.stopper=i.animation[t](r,s)}},{key:"stop",value:function(){var t=this.stopper;t&&(this.stopper=null,t.stop())}},{key:"render",value:function(){return this.props.children}}]),e}(p.a.Component);K.propTypes={children:C.a.any,animation:C.a.any,transitionName:C.a.any};var z=K,J="rc_animate_"+Date.now();function q(t){var e=t.children;return p.a.isValidElement(e)&&!e.key?p.a.cloneElement(e,{key:J}):e}function V(){}var W=function(t){function e(t){A()(this,e);var n=g()(this,(e.__proto__||Object.getPrototypeOf(e)).call(this,t));return X.call(n),n.currentlyAnimatingKeys={},n.keysToEnter=[],n.keysToLeave=[],n.state={children:m(q(t))},n.childrenRefs={},n}return d()(e,t),c()(e,[{key:"componentDidMount",value:function(){var t=this,e=this.props.showProp,n=this.state.children;e&&(n=n.filter(function(t){return!!t.props[e]})),n.forEach(function(e){e&&t.performAppear(e.key)})}},{key:"componentWillReceiveProps",value:function(t){var e=this;this.nextProps=t;var n=m(q(t)),r=this.props;r.exclusive&&Object.keys(this.currentlyAnimatingKeys).forEach(function(t){e.stop(t)});var i,o,s,A,l=r.showProp,c=this.currentlyAnimatingKeys,u=r.exclusive?m(q(r)):this.state.children,g=[];l?(u.forEach(function(t){var e=t&&y(n,t.key),r=void 0;(r=e&&e.props[l]||!t.props[l]?e:p.a.cloneElement(e||t,a()({},l,!0)))&&g.push(r)}),n.forEach(function(t){t&&y(u,t.key)||g.push(t)})):(i=n,o=[],s={},A=[],u.forEach(function(t){t&&y(i,t.key)?A.length&&(s[t.key]=A,A=[]):A.push(t)}),i.forEach(function(t){t&&Object.prototype.hasOwnProperty.call(s,t.key)&&(o=o.concat(s[t.key])),o.push(t)}),g=o=o.concat(A)),this.setState({children:g}),n.forEach(function(t){var n=t&&t.key;if(!t||!c[n]){var r=t&&y(u,n);if(l){var i=t.props[l];if(r)!E(u,n,l)&&i&&e.keysToEnter.push(n);else i&&e.keysToEnter.push(n)}else r||e.keysToEnter.push(n)}}),u.forEach(function(t){var r=t&&t.key;if(!t||!c[r]){var i=t&&y(n,r);if(l){var o=t.props[l];if(i)!E(n,r,l)&&o&&e.keysToLeave.push(r);else o&&e.keysToLeave.push(r)}else i||e.keysToLeave.push(r)}})}},{key:"componentDidUpdate",value:function(){var t=this.keysToEnter;this.keysToEnter=[],t.forEach(this.performEnter);var e=this.keysToLeave;this.keysToLeave=[],e.forEach(this.performLeave)}},{key:"isValidChildByKey",value:function(t,e){var n=this.props.showProp;return n?E(t,e,n):y(t,e)}},{key:"stop",value:function(t){delete this.currentlyAnimatingKeys[t];var e=this.childrenRefs[t];e&&e.stop()}},{key:"render",value:function(){var t=this,e=this.props;this.nextProps=e;var n=this.state.children,r=null;n&&(r=n.map(function(n){if(null==n)return n;if(!n.key)throw new Error("must set key for <rc-animate> children");return p.a.createElement(z,{key:n.key,ref:function(e){t.childrenRefs[n.key]=e},animation:e.animation,transitionName:e.transitionName,transitionEnter:e.transitionEnter,transitionAppear:e.transitionAppear,transitionLeave:e.transitionLeave},n)}));var o=e.component;if(o){var a=e;return"string"==typeof o&&(a=i()({className:e.className,style:e.style},e.componentProps)),p.a.createElement(o,a,r)}return r[0]||null}}]),e}(p.a.Component);W.isAnimate=!0,W.propTypes={className:C.a.string,style:C.a.object,component:C.a.any,componentProps:C.a.object,animation:C.a.object,transitionName:C.a.oneOfType([C.a.string,C.a.object]),transitionEnter:C.a.bool,transitionAppear:C.a.bool,exclusive:C.a.bool,transitionLeave:C.a.bool,onEnd:C.a.func,onEnter:C.a.func,onLeave:C.a.func,onAppear:C.a.func,showProp:C.a.string,children:C.a.node},W.defaultProps={animation:{},component:"span",componentProps:{},transitionEnter:!0,transitionLeave:!0,transitionAppear:!1,onEnd:V,onEnter:V,onLeave:V,onAppear:V};var X=function(){var t=this;this.performEnter=function(e){t.childrenRefs[e]&&(t.currentlyAnimatingKeys[e]=!0,t.childrenRefs[e].componentWillEnter(t.handleDoneAdding.bind(t,e,"enter")))},this.performAppear=function(e){t.childrenRefs[e]&&(t.currentlyAnimatingKeys[e]=!0,t.childrenRefs[e].componentWillAppear(t.handleDoneAdding.bind(t,e,"appear")))},this.handleDoneAdding=function(e,n){var r=t.props;if(delete t.currentlyAnimatingKeys[e],!r.exclusive||r===t.nextProps){var i=m(q(r));t.isValidChildByKey(i,e)?"appear"===n?Y.allowAppearCallback(r)&&(r.onAppear(e),r.onEnd(e,!0)):Y.allowEnterCallback(r)&&(r.onEnter(e),r.onEnd(e,!0)):t.performLeave(e)}},this.performLeave=function(e){t.childrenRefs[e]&&(t.currentlyAnimatingKeys[e]=!0,t.childrenRefs[e].componentWillLeave(t.handleDoneLeaving.bind(t,e)))},this.handleDoneLeaving=function(e){var n=t.props;if(delete t.currentlyAnimatingKeys[e],!n.exclusive||n===t.nextProps){var r,i,o,a,s=m(q(n));if(t.isValidChildByKey(s,e))t.performEnter(e);else{var A=function(){Y.allowLeaveCallback(n)&&(n.onLeave(e),n.onEnd(e,!1))};r=t.state.children,i=s,o=n.showProp,(a=r.length===i.length)&&r.forEach(function(t,e){var n=i[e];t&&n&&(t&&!n||!t&&n?a=!1:t.key!==n.key?a=!1:o&&t.props[o]!==n.props[o]&&(a=!1))}),a?A():t.setState({children:s},A)}}}};e.a=W},function(t,e,n){(function(t){var r=void 0!==t&&t||"undefined"!=typeof self&&self||window,i=Function.prototype.apply;function o(t,e){this._id=t,this._clearFn=e}e.setTimeout=function(){return new o(i.call(setTimeout,r,arguments),clearTimeout)},e.setInterval=function(){return new o(i.call(setInterval,r,arguments),clearInterval)},e.clearTimeout=e.clearInterval=function(t){t&&t.close()},o.prototype.unref=o.prototype.ref=function(){},o.prototype.close=function(){this._clearFn.call(r,this._id)},e.enroll=function(t,e){clearTimeout(t._idleTimeoutId),t._idleTimeout=e},e.unenroll=function(t){clearTimeout(t._idleTimeoutId),t._idleTimeout=-1},e._unrefActive=e.active=function(t){clearTimeout(t._idleTimeoutId);var e=t._idleTimeout;e>=0&&(t._idleTimeoutId=setTimeout(function(){t._onTimeout&&t._onTimeout()},e))},n(745),e.setImmediate="undefined"!=typeof self&&self.setImmediate||void 0!==t&&t.setImmediate||this&&this.setImmediate,e.clearImmediate="undefined"!=typeof self&&self.clearImmediate||void 0!==t&&t.clearImmediate||this&&this.clearImmediate}).call(this,n(37))},function(t,e,n){"use strict";var r=n(209),i=Object.keys||function(t){var e=[];for(var n in t)e.push(n);return e};t.exports=u;var o=n(166);o.inherits=n(30);var a=n(397),s=n(271);o.inherits(u,a);for(var A=i(s.prototype),l=0;l<A.length;l++){var c=A[l];u.prototype[c]||(u.prototype[c]=s.prototype[c])}function u(t){if(!(this instanceof u))return new u(t);a.call(this,t),s.call(this,t),t&&!1===t.readable&&(this.readable=!1),t&&!1===t.writable&&(this.writable=!1),this.allowHalfOpen=!0,t&&!1===t.allowHalfOpen&&(this.allowHalfOpen=!1),this.once("end",g)}function g(){this.allowHalfOpen||this._writableState.ended||r.nextTick(h,this)}function h(t){t.end()}Object.defineProperty(u.prototype,"writableHighWaterMark",{enumerable:!1,get:function(){return this._writableState.highWaterMark}}),Object.defineProperty(u.prototype,"destroyed",{get:function(){return void 0!==this._readableState&&void 0!==this._writableState&&(this._readableState.destroyed&&this._writableState.destroyed)},set:function(t){void 0!==this._readableState&&void 0!==this._writableState&&(this._readableState.destroyed=t,this._writableState.destroyed=t)}}),u.prototype._destroy=function(t,e){this.push(null),this.end(),r.nextTick(e,t)}},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0});const r=n(17);let i,o,a,s;if("undefined"==typeof window){let e=n(1078);i=t.fetch||e,a=t.Request||e.Request,o=t.Headers||e.Headers,s=t.WebSocket||n(1079)}else i=fetch,a=Request,o=Headers,s=WebSocket;var A,l;!function(t){t.makeSettings=function(t){return l.makeSettings(t)},t.makeRequest=function(t,e,n){return l.handleRequest(t,e,n)};t.ResponseError=class extends Error{constructor(t,e){super(e=e||`Invalid response: ${t.status} ${t.statusText}`),this.response=t}};t.NetworkError=class extends TypeError{constructor(t){super(t.message),this.stack=t.stack}},t.defaultSettings={baseUrl:r.PageConfig.getBaseUrl(),appUrl:r.PageConfig.getOption("appUrl"),wsUrl:r.PageConfig.getWsUrl(),token:r.PageConfig.getToken(),init:{cache:"no-store",credentials:"same-origin"},fetch:i,Headers:o,Request:a,WebSocket:s}}(A=e.ServerConnection||(e.ServerConnection={})),function(t){t.makeSettings=function(t={}){const e=A.defaultSettings,n=r.URLExt.normalize(t.baseUrl)||e.baseUrl;let i=t.wsUrl;return i||n!==e.baseUrl||(i=e.wsUrl),i||0!==n.indexOf("http")||(i="ws"+n.slice(4)),i=i||e.wsUrl,Object.assign({},e,t,{wsUrl:i})},t.handleRequest=function(t,e,n){if(0!==t.indexOf(n.baseUrl))throw new Error("Can only be used for notebook server requests");"no-store"===(e.cache||n.init.cache)&&(t+=(/\?/.test(t)?"&":"?")+(new Date).getTime());const r=new n.Request(t,Object.assign({},n.init,e));let i=!1;if(n.token&&(i=!0,r.headers.append("Authorization",`token ${n.token}`)),"undefined"!=typeof document&&document&&document.cookie){const t=function(t){const e=document.cookie.match("\\b"+t+"=([^;]*)\\b");return e?e[1]:void 0}("_xsrf");void 0!==t&&(i=!0,r.headers.append("X-XSRFToken",t))}return!r.bodyUsed&&i&&r.headers.set("Content-Type","application/json"),n.fetch.call(null,r).catch(t=>{throw new A.NetworkError(t)})}}(l||(l={}))}).call(this,n(37))},function(t,e){var n={}.hasOwnProperty;t.exports=function(t,e){return n.call(t,e)}},function(t,e,n){var r=n(641),i=n(318);t.exports=function(t){return r(i(t))}},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=!("undefined"==typeof window||!window.document||!window.document.createElement);e.default=r,t.exports=e.default},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(232),i=n(6),o=n(113),a=n(25),s=n(43),A=n(28),l=n(66);class c extends r.DashboardLayoutRegion{constructor(t,e){super(t,e),this.addPartOverlay=new i.Panel,this.properties={...this.properties,caption:this.constructor.GetMetadata().name},this.setupAddPartOverlay()}static GetMetadata(){const t=super.GetMetadata(),e=new c.ParentMetadata({iconClass:t.iconClass,iconText:t.iconText});return e.copyFrom(t),e.getMetadata("showTitle").default=!1,e.addMetadata("prunable",{prettyName:"Prunable?",type:A.Types.Boolean,documentation:"Internal. Whether this region is elibible to be auto-pruned by the framework.",default:!0}),e}get widgets(){return this.content.widgets}subtree(){return A.IterTools.dfs_iter(this.widgets,t=>t.widgets)}copyAttachedProperties(t,e){for(let[n,r]of this.metadata.getAllAttachedProperties()){const r=this.metadata.getAttachedProperty(n);null!=r&&r.set(e,r.get(t))}}clearAttachedProperties(t){for(let e of this.metadata.getAllAttachedPropertyNames()){this.metadata.getAttachedProperty(e).set(t,null)}}addChild(t){this.insertChild(this.widgets.length,t)}insertChild(t,e){const n=e.parentRegion;if(e.parentRegion=this,e.OnStale.pipe(o.takeWhile(()=>!this.isDisposed&&this.widgets.indexOf(e)>-1)).subscribe(()=>{this.setStale()}),this.content.insertWidget(t,e),n){const t=this.findCommonAncestor(n);null!=t&&t.pruneSubtree()}this.setStale()}getAttachedPropertyForChild(t,e){const n=this.metadata.getAttachedProperty(t);if(null==n)throw Error("Cannot find attached property with name "+n);return n.get(e)}updateFromProperties(){this.updateOverlay(),super.updateFromProperties()}setFresh(){for(const t of this.widgets)t.setFresh();super.setFresh()}pruneSubtree(){let t;for(;(t=this.findPrunableChildren()).length>0;){this.setStale();for(const e of t)if(!e.isDisposed){if(1===e.widgets.length){const t=e.parentRegion||this.owner.root,n=e.widgets[0],r=t.widgets.indexOf(e);t.insertChild(r,e.widgets[0]),n.isFocused&&n.activate()}e.dispose()}}}sizeContentToFit(t){super.sizeContentToFit(t),this.layoutChildren()}processMessage(t){"region-child-removed"===t.type?this.onChildRegionRemoved(t.child):"region-child-added"===t.type&&this.onChildRegionAdded(t.child),super.processMessage(t)}createDragShadow(t,e,n){const r=document.createElement("span");return r.classList.add("m-DragShadow"),r.textContent=t.getLayoutProperty("caption"),r}_markSubtreeForUpdate(){for(const t of this.subtree())t._didUpdate=!1}_updateSkippedSubtrees(){for(const t of A.IterTools.filter(this.subtree(),t=>!t._didUpdate))t.updateFromProperties()}onCloseRequest(){null!=this.parentRegion&&super.onCloseRequest()}getDefault(t){return this.metadata.getAttachedProperty(""+t)?this.metadata.getAttachedMetadata(""+t).default:super.getDefault(t)}installContentTap(){const t=u.InstalledTaps.get(this);a.MessageLoop.installMessageHook(this.content,t)}onChildRegionAdded(t){}onChildRegionRemoved(t){}findCommonAncestor(t){const e=new WeakSet;let n=this;for(;n;)e.add(n),n=n.parentRegion;let r=t;for(;r;){if(e.has(r))return r;r=r.parentRegion}return null}findPrunableChildren(){const t=[];for(const e of this.subtree())e instanceof c&&e.widgets.length<=1&&e.getLayoutProperty("prunable")&&t.push(e);return t}setupAddPartOverlay(){this.addPartOverlay.addClass("m-AddPartOverlay");const t=new l.Interactions.Button;t.addClass("m-AddPartOverlay-btn"),t.label="Add New Part...",t.onClicked.subscribe(async()=>{const t=await this.owner.showAddPartDialog();null!=t&&this.owner.addPartToRegion(this.id,t)}),this.addPartOverlay.addWidget(t),this.layout.addWidget(this.addPartOverlay)}updateOverlay(){null!=this.parentRegion||this.widgets.length>0?this.addPartOverlay.hide():this.addPartOverlay.show()}}var u;e.RegionWithChildren=c,function(t){class e extends r.DashboardLayoutRegion.Metadata{constructor(){super(...arguments),this.description="",this.remarks="",this.attachedMetadataMap=new Map,this.attachedPropertyMap=new Map}copyFrom(t){for(const[e,n]of t.getAllProperties())this.addMetadata(e,n);this.description=t.description,this.remarks=t.remarks}addAttachedMetadata(t,n){const r=Object.assign({prettyName:t.name},e.defaultData,n);this.attachedMetadataMap.set(t.name,r),this.attachedPropertyMap.set(t.name,t)}getAllAttachedProperties(){return this.attachedMetadataMap.entries()}getAllAttachedPropertyNames(){return[...this.attachedMetadataMap.keys()]}getAttachedMetadata(t){return this.attachedMetadataMap.get(t)}getAttachedProperty(t){return this.attachedPropertyMap.get(t)}}t.ParentMetadata=e}(c=e.RegionWithChildren||(e.RegionWithChildren={})),function(t){class e{constructor(t){this.owner=t}messageHook(t,e){if(e instanceof i.Widget.ChildMessage&&!this.owner.isDisposed&&!e.child.isDisposed){const t=new i.Widget.ChildMessage("region-"+e.type,e.child);a.MessageLoop.sendMessage(this.owner,t)}return!0}}t.MessageTap=e,t.InstalledTaps=new s.AttachedProperty({create:t=>new e(t),name:"MessageTap Instances"})}(u||(u={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(6),i=n(233),o=n(682),a=n(124),s=n(232),A=n(25),l=n(10),c=n(64),u=n(684),g=n(1287),h=n(28),d=n(66),f=n(0),p=n(339);!function(t){t[t.StackPanelLayoutRegion=0]="StackPanelLayoutRegion",t[t.WidgetLayoutRegion=1]="WidgetLayoutRegion",t[t.TabPanelDashboardLayoutRegion=2]="TabPanelDashboardLayoutRegion",t[t.CanvasLayoutRegion=3]="CanvasLayoutRegion",t[t.GridPanelLayoutRegion=4]="GridPanelLayoutRegion"}(e.LayoutTypes||(e.LayoutTypes={}));e.LayoutManager=class extends r.Widget{constructor(t,e){super(),this.externalParts=new Map,this._maximizedRegion=null,this._focusedRegion=null,this._onFocusedChanged=new l.Signal(this),this._isDirty=!1,this._OnDirtySrc=new c.Subject,this._OnDirty=this._OnDirtySrc.asObservable(),this._forceTitlebars=!1,this.node.tabIndex=-1,this.addClass("m-LayoutManager"),this.partManager=t,this.factory=e,this.layout=new r.StackedLayout,this.root=new i.StackPanelLayoutRegion(this),this.root.setLayoutProperty("caption","Dashboard"),this.root.setFresh(),this.root.OnStale.subscribe(()=>this.setDirty()),this.dockPreview=new g.DockPreview({owner:this}),this.dockPreview.hide(),this.layout.addWidget(this.root),this.layout.addWidget(this.dockPreview)}get focusedRegion(){return this._focusedRegion}get focusedRegionChanged(){return this._onFocusedChanged}get isDirty(){return this._isDirty}get OnDirty(){return this._OnDirty}get forceTitlebars(){return this._forceTitlebars}set forceTitlebars(t){t!==this._forceTitlebars&&(this._forceTitlebars=t,this.setDirty())}setClean(){this._isDirty=!1,this.update()}getPart(t){return this.partManager.getPartById(t)||this.externalParts.get(t)}setExternalParts(t){this.externalParts=t}initLayout(t){t&&(this.root.dispose(),this.root=o.LayoutSerializer.fromJson(t,this),this.root.setFresh(),this.root.OnStale.subscribe(()=>this.setDirty()),this.root.titlebar.hide(),this.layout.insertWidget(0,this.root))}getRegion(t){if(t===this.root.uuid)return this.root;for(const e of h.IterTools.dfs_iter(this.root.widgets,t=>t instanceof a.RegionWithChildren?t.widgets:void 0))if(e.uuid===t)return e;return null}getParentRegion(t){if(t===this.root.uuid)return this.root;const e=this.getRegion(t);return null!=e&&null!=e.parentRegion?e.parentRegion:null}removeRegion(t){const e=this.getRegion(t);if(!e)throw Error("Region with "+t+" not found!");const n=e.parentRegion;e.dispose(),this.setDirty(),n&&(n.fit(),n.update()),this.root.pruneSubtree()}dispose(){this.isDisposed||(this._OnDirtySrc.complete(),this._focusedRegion=null,this.root.dispose(),this.externalParts.forEach(t=>t.dispose()),this.externalParts.clear(),super.dispose())}handleEvent(t){switch(t.type){case"focusin":case"mousedown":return this._handleFocus(t)}}moveToRootZone(t,e){return u.LayoutActions.MoveToRootZone({layoutManager:this,zone:t,target:e})}moveToRelativeZone(t,e,n){return u.LayoutActions.MoveToRelativeZone({layoutManager:this,zone:t,target:e,reference:n})}async showAddPartDialog(){const t=[],e=[];for(const t of this.factory.keys())e.push({key:t,children:[],isCollapsed:!1,isSelected:!1,selectable:!0});e.sort((t,e)=>t.key.localeCompare(e.key)),t.push({key:"Parts",children:e,isCollapsed:!1,selectable:!1,isSelected:!1});let n=null;const r=d.TreeModel.Create(t),i=d.ReactWrapperWidget.Create(f.createElement(d.TreeView,{model:r,onSelect:t=>{r.selectNode(t,!0),n=t},onCollapse:(t,e)=>{r.update(t,{isCollapsed:e})}}));i.node.style.overflowY="auto";const o=await d.HoverManager.Instance.launchDialog(i,this,300,600,"Add New Part",[{text:"Dismiss"},{text:"Ok",accept:!0}]);if(r.dispose(),o.accept)return n}async addPartToRegion(t,e){const n=this.getRegion(t);if(!(n instanceof a.RegionWithChildren))throw Error("Cannot add region to parent");const r=await this.factory.get(e),i=new p.WidgetLayoutRegion(this,r,r.id);n.addChild(i)}get bounds(){const t=this.node.getBoundingClientRect();return{width:t.width,height:t.height}}updateMaximizedRegion(){let t=null;for(const e of h.IterTools.dfs_iter(this.root.widgets,t=>t instanceof a.RegionWithChildren?t.widgets:void 0))this._maximizedRegion!==e&&e.getLayoutProperty("maximized")&&(t?e.setLayoutProperty("maximized",!1):t=e);if(this.root.getLayoutProperty("maximized")&&this.root.setLayoutProperty("maximized",!1),(this._maximizedRegion||t)&&(t||!this._maximizedRegion||!this._maximizedRegion.getLayoutProperty("maximized"))&&(null!=this._maximizedRegion&&(this._maximizedRegion.setLayoutProperty("maximized",!1),document.querySelectorAll(".no-stack").forEach(t=>t.classList.remove("no-stack")),this._maximizedRegion=null),null!=t)){t.setLayoutProperty("maximized",!0),this._maximizedRegion=t;let e=t.parent;for(;e&&e!==this.root;)e.addClass("no-stack"),e=e.parent;this.setDirty()}}onUpdateRequest(){this.updateMaximizedRegion()}onAfterAttach(){this.node.addEventListener("focusin",this),this.node.addEventListener("mousedown",this)}onBeforeDetach(){this.node.removeEventListener("focusin",this),this.node.removeEventListener("mousedown",this)}_handleFocus(t){let e=t.target;if(this.root.node.contains(e))for(;null!=e.parentElement&&e!==this.node;){if(!e.classList.contains("maven_dashboard")){e=e.parentElement;continue}const t=e.id,n=this.getRegion(t);if(null==n)return;return void this.setFocusedRegion(n)}}setFocusedRegion(t){if(t===this._focusedRegion)return;const e=this._focusedRegion;null!=e&&A.MessageLoop.sendMessage(e,s.DashboardLayoutRegion.LayoutMsg.BeforeBlur),A.MessageLoop.sendMessage(t,s.DashboardLayoutRegion.LayoutMsg.BeforeFocus),this._focusedRegion=t,this._onFocusedChanged.emit(t),A.MessageLoop.sendMessage(t,s.DashboardLayoutRegion.LayoutMsg.AfterFocus),null!=e&&A.MessageLoop.sendMessage(e,s.DashboardLayoutRegion.LayoutMsg.AfterBlur)}setDirty(){this.root.setFresh(),A.MessageLoop.sendMessage(this.root,r.Widget.Msg.FitRequest),A.MessageLoop.sendMessage(this.root,r.Widget.Msg.UpdateRequest),this._isDirty||(this._isDirty=!0,this._OnDirtySrc.next())}}},function(t,e,n){"use strict";n.r(e);var r,i=n(381),o=n(382),a=n(192),s=n(193),A=n(383),l=n(17),c=n(592);class u{constructor(t={}){if(this._id=0,this._ranks={},this._types=null,this._factories={},this.resolver=t.resolver||null,this.linkHandler=t.linkHandler||null,this.latexTypesetter=t.latexTypesetter||null,this.sanitizer=t.sanitizer||c.a,t.initialFactories)for(let e of t.initialFactories)this.addFactory(e)}get mimeTypes(){return this._types||(this._types=r.sortedTypes(this._ranks))}preferredMimeType(t,e="ensure"){if("ensure"===e||"prefer"===e)for(let e of this.mimeTypes)if(e in t&&this._factories[e].safe)return e;if("ensure"!==e)for(let e of this.mimeTypes)if(e in t)return e}createRenderer(t){if(!(t in this._factories))throw new Error(`No factory for mime type: '${t}'`);return this._factories[t].createRenderer({mimeType:t,resolver:this.resolver,sanitizer:this.sanitizer,linkHandler:this.linkHandler,latexTypesetter:this.latexTypesetter})}createModel(t={}){return new s.a(t)}clone(t={}){let e=new u({resolver:t.resolver||this.resolver||void 0,sanitizer:t.sanitizer||this.sanitizer||void 0,linkHandler:t.linkHandler||this.linkHandler||void 0,latexTypesetter:t.latexTypesetter||this.latexTypesetter});return e._factories=Object.assign({},this._factories),e._ranks=Object.assign({},this._ranks),e._id=this._id,e}getFactory(t){return this._factories[t]}addFactory(t,e){void 0===e&&void 0===(e=t.defaultRank)&&(e=100);for(let n of t.mimeTypes)this._factories[n]=t,this._ranks[n]={rank:e,id:this._id++};this._types=null}removeMimeType(t){delete this._factories[t],delete this._ranks[t],this._types=null}getRank(t){let e=this._ranks[t];return e&&e.rank}setRank(t,e){if(!this._ranks[t])return;let n=this._id++;this._ranks[t]={rank:e,id:n},this._types=null}}(u||(u={})).UrlResolver=class{constructor(t){this._session=t.session,this._contents=t.contents}resolveUrl(t){if(this.isLocal(t)){const e=encodeURI(l.PathExt.dirname(this._session.path));t=l.PathExt.resolve(e,t)}return Promise.resolve(t)}getDownloadUrl(t){return this.isLocal(t)?this._contents.getDownloadUrl(decodeURI(t)):Promise.resolve(t)}isLocal(t){const e=decodeURI(t);return l.URLExt.isLocal(t)||!!this._contents.driveName(e)}},function(t){t.sortedTypes=function(t){return Object.keys(t).sort((e,n)=>{let r=t[e],i=t[n];return r.rank!==i.rank?r.rank-i.rank:r.id-i.id})}}(r||(r={}));var g=n(97),h=n(611),d=n(96);n.d(e,"AttachmentModel",function(){return i.a}),n.d(e,"htmlRendererFactory",function(){return o.a}),n.d(e,"imageRendererFactory",function(){return o.b}),n.d(e,"latexRendererFactory",function(){return o.d}),n.d(e,"markdownRendererFactory",function(){return o.e}),n.d(e,"svgRendererFactory",function(){return o.g}),n.d(e,"textRendererFactory",function(){return o.h}),n.d(e,"javaScriptRendererFactory",function(){return o.c}),n.d(e,"standardRendererFactories",function(){return o.f}),n.d(e,"removeMath",function(){return a.a}),n.d(e,"replaceMath",function(){return a.b}),n.d(e,"MimeModel",function(){return s.a}),n.d(e,"OutputModel",function(){return A.a}),n.d(e,"RenderMimeRegistry",function(){return u}),n.d(e,"renderHTML",function(){return g.a}),n.d(e,"renderImage",function(){return g.b}),n.d(e,"renderLatex",function(){return g.c}),n.d(e,"renderMarkdown",function(){return g.d}),n.d(e,"renderSVG",function(){return g.e}),n.d(e,"renderText",function(){return g.f}),n.d(e,"IRenderMimeRegistry",function(){return h.b}),n.d(e,"ILatexTypesetter",function(){return h.a}),n.d(e,"RenderedCommon",function(){return d.a}),n.d(e,"RenderedHTMLCommon",function(){return d.c}),n.d(e,"RenderedHTML",function(){return d.b}),n.d(e,"RenderedLatex",function(){return d.f}),n.d(e,"RenderedImage",function(){return d.d}),n.d(e,"RenderedMarkdown",function(){return d.g}),n.d(e,"RenderedSVG",function(){return d.h}),n.d(e,"RenderedText",function(){return d.i}),n.d(e,"RenderedJavaScript",function(){return d.e})},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r,i=n(50),o=function(){function t(t){var e=this;this._onScrollFrame=function(){if(e._scrollTarget){var t=e._scrollTarget,n=t.element,i=t.edge,o=t.distance,a=r.SCROLL_EDGE_SIZE-o,s=Math.pow(a/r.SCROLL_EDGE_SIZE,2),A=Math.max(1,Math.round(s*r.SCROLL_EDGE_SIZE));switch(i){case"top":n.scrollTop-=A;break;case"left":n.scrollLeft-=A;break;case"right":n.scrollLeft+=A;break;case"bottom":n.scrollTop+=A}requestAnimationFrame(e._onScrollFrame)}},this._disposed=!1,this._dropAction="none",this._override=null,this._currentTarget=null,this._currentElement=null,this._promise=null,this._scrollTarget=null,this._resolve=null,this.mimeData=t.mimeData,this.dragImage=t.dragImage||null,this.proposedAction=t.proposedAction||"copy",this.supportedActions=t.supportedActions||"all",this.source=t.source||null}return t.prototype.dispose=function(){if(!this._disposed){if(this._disposed=!0,this._currentTarget){var t=r.createMouseEvent("mouseup",-1,-1);r.dispatchDragLeave(this,this._currentTarget,null,t)}this._finalize("none")}},Object.defineProperty(t.prototype,"isDisposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),t.prototype.start=function(t,e){var n=this;if(this._disposed)return Promise.resolve("none");if(this._promise)return this._promise;this._addListeners(),this._attachDragImage(t,e),this._promise=new Promise(function(t,e){n._resolve=t});var i=r.createMouseEvent("mousemove",t,e);return document.dispatchEvent(i),this._promise},t.prototype.handleEvent=function(t){switch(t.type){case"mousemove":this._evtMouseMove(t);break;case"mouseup":this._evtMouseUp(t);break;case"keydown":this._evtKeyDown(t);break;default:t.preventDefault(),t.stopPropagation()}},t.prototype._evtMouseMove=function(t){t.preventDefault(),t.stopPropagation(),this._updateCurrentTarget(t),this._updateDragScroll(t),this._moveDragImage(t.clientX,t.clientY)},t.prototype._evtMouseUp=function(t){if(t.preventDefault(),t.stopPropagation(),0===t.button)if(this._updateCurrentTarget(t),this._currentTarget){if("none"===this._dropAction)return r.dispatchDragLeave(this,this._currentTarget,null,t),void this._finalize("none");var e=r.dispatchDrop(this,this._currentTarget,t);this._finalize(e)}else this._finalize("none")},t.prototype._evtKeyDown=function(t){t.preventDefault(),t.stopPropagation(),27===t.keyCode&&this.dispose()},t.prototype._addListeners=function(){document.addEventListener("mousedown",this,!0),document.addEventListener("mousemove",this,!0),document.addEventListener("mouseup",this,!0),document.addEventListener("mouseenter",this,!0),document.addEventListener("mouseleave",this,!0),document.addEventListener("mouseover",this,!0),document.addEventListener("mouseout",this,!0),document.addEventListener("keydown",this,!0),document.addEventListener("keyup",this,!0),document.addEventListener("keypress",this,!0),document.addEventListener("contextmenu",this,!0)},t.prototype._removeListeners=function(){document.removeEventListener("mousedown",this,!0),document.removeEventListener("mousemove",this,!0),document.removeEventListener("mouseup",this,!0),document.removeEventListener("mouseenter",this,!0),document.removeEventListener("mouseleave",this,!0),document.removeEventListener("mouseover",this,!0),document.removeEventListener("mouseout",this,!0),document.removeEventListener("keydown",this,!0),document.removeEventListener("keyup",this,!0),document.removeEventListener("keypress",this,!0),document.removeEventListener("contextmenu",this,!0)},t.prototype._updateDragScroll=function(t){var e=r.findScrollTarget(t);(this._scrollTarget||e)&&(this._scrollTarget||setTimeout(this._onScrollFrame,500),this._scrollTarget=e)},t.prototype._updateCurrentTarget=function(t){var e=this._currentTarget,n=this._currentTarget,i=this._currentElement,o=document.elementFromPoint(t.clientX,t.clientY);this._currentElement=o,o!==i&&o!==n&&r.dispatchDragExit(this,n,o,t),o!==i&&o!==n&&(n=r.dispatchDragEnter(this,o,n,t)),n!==e&&(this._currentTarget=n,r.dispatchDragLeave(this,e,n,t));var a=r.dispatchDragOver(this,n,t);this._setDropAction(a)},t.prototype._attachDragImage=function(t,e){if(this.dragImage){this.dragImage.classList.add("p-mod-drag-image");var n=this.dragImage.style;n.pointerEvents="none",n.position="fixed",n.top=e+"px",n.left=t+"px",document.body.appendChild(this.dragImage)}},t.prototype._moveDragImage=function(t,e){if(this.dragImage){var n=this.dragImage.style;n.top=e+"px",n.left=t+"px"}},t.prototype._detachDragImage=function(){if(this.dragImage){var t=this.dragImage.parentNode;t&&t.removeChild(this.dragImage)}},t.prototype._setDropAction=function(e){if(e=r.validateAction(e,this.supportedActions),!this._override||this._dropAction!==e)switch(e){case"none":this._dropAction=e,this._override=t.overrideCursor("no-drop");break;case"copy":this._dropAction=e,this._override=t.overrideCursor("copy");break;case"link":this._dropAction=e,this._override=t.overrideCursor("alias");break;case"move":this._dropAction=e,this._override=t.overrideCursor("move")}},t.prototype._finalize=function(t){var e=this._resolve;this._removeListeners(),this._detachDragImage(),this._override&&(this._override.dispose(),this._override=null),this.mimeData.clear(),this._disposed=!0,this._dropAction="none",this._currentTarget=null,this._currentElement=null,this._scrollTarget=null,this._promise=null,this._resolve=null,e&&e(t)},t}();e.Drag=o,function(t){t.overrideCursor=function(t){var n=++e;return document.body.style.cursor=t,document.body.classList.add("p-mod-override-cursor"),new i.DisposableDelegate(function(){n===e&&(document.body.style.cursor="",document.body.classList.remove("p-mod-override-cursor"))})};var e=0}(o=e.Drag||(e.Drag={})),e.Drag=o,function(t){t.SCROLL_EDGE_SIZE=20,t.validateAction=function(t,r){return e[t]&n[r]?t:"none"},t.createMouseEvent=function(t,e,n){var r=document.createEvent("MouseEvent");return r.initMouseEvent(t,!0,!0,window,0,0,0,e,n,!1,!1,!1,!1,0,null),r},t.findScrollTarget=function(e){for(var n=e.clientX,r=e.clientY,i=document.elementFromPoint(n,r);i;i=i.parentElement)if(i.hasAttribute("data-p-dragscroll")){var o=0,a=0;i===document.body&&(o=window.pageXOffset,a=window.pageYOffset);var s=i.getBoundingClientRect(),A=s.top+a,l=s.left+o,c=l+s.width,u=A+s.height;if(!(n<l||n>=c||r<A||r>=u)){var g=n-l+1,h=r-A+1,d=c-n,f=u-r,p=Math.min(g,h,d,f);if(!(p>t.SCROLL_EDGE_SIZE)){var I=void 0;switch(p){case f:I="bottom";break;case h:I="top";break;case d:I="right";break;case g:I="left";break;default:throw"unreachable"}var C=i.scrollWidth-i.clientWidth,m=i.scrollHeight-i.clientHeight,y=void 0;switch(I){case"top":y=m>0&&i.scrollTop>0;break;case"left":y=C>0&&i.scrollLeft>0;break;case"right":y=C>0&&i.scrollLeft<C;break;case"bottom":y=m>0&&i.scrollTop<m;break;default:throw"unreachable"}if(y)return{element:i,edge:I,distance:p}}}}return null},t.dispatchDragEnter=function(t,e,n,i){if(!e)return null;var o=r("p-dragenter",t,i,n);return e.dispatchEvent(o)?e===document.body?n:(o=r("p-dragenter",t,i,n),document.body.dispatchEvent(o),document.body):e},t.dispatchDragExit=function(t,e,n,i){if(e){var o=r("p-dragexit",t,i,n);e.dispatchEvent(o)}},t.dispatchDragLeave=function(t,e,n,i){if(e){var o=r("p-dragleave",t,i,n);e.dispatchEvent(o)}},t.dispatchDragOver=function(t,e,n){if(!e)return"none";var i=r("p-dragover",t,n,null);return e.dispatchEvent(i)?"none":i.dropAction},t.dispatchDrop=function(t,e,n){if(!e)return"none";var i=r("p-drop",t,n,null);return e.dispatchEvent(i)?"none":i.dropAction};var e={none:0,copy:1,link:2,move:4},n={none:e.none,copy:e.copy,link:e.link,move:e.move,"copy-link":e.copy|e.link,"copy-move":e.copy|e.move,"link-move":e.link|e.move,all:e.copy|e.link|e.move};function r(t,e,n,r){var i=document.createEvent("MouseEvent");return i.initMouseEvent(t,!0,!0,window,0,n.screenX,n.screenY,n.clientX,n.clientY,n.ctrlKey,n.altKey,n.shiftKey,n.metaKey,n.button,r),i.dropAction="none",i.mimeData=e.mimeData,i.proposedAction=e.proposedAction,i.supportedActions=e.supportedActions,i.source=e.source,i}}(r||(r={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r={CENTER:"center",LEFT:"left",RIGHT:"right"}},function(t,e,n){"use strict";n.d(e,"a",function(){return i}),n.d(e,"b",function(){return o});var r=n(65);function i(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return o(t)}function o(t){return t?1===t.length?t[0]:function(e){return t.reduce(function(t,e){return e(t)},e)}:r.a}},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r="function"==typeof Symbol?Symbol("rxSubscriber"):"@@rxSubscriber_"+Math.random()},function(t,e,n){"use strict";function r(t){return Error.call(this),this.message=t?t.length+" errors occurred during unsubscription:\n"+t.map(function(t,e){return e+1+") "+t.toString()}).join("\n  "):"",this.name="UnsubscriptionError",this.errors=t,this}n.d(e,"a",function(){return i}),r.prototype=Object.create(Error.prototype);var i=r},function(t,e,n){"use strict";var r=n(259),i=n(104),o=n(75),a=n(79),s=n(260),A=n(261),l=n(157);n.d(e,"a",function(){return c});var c=function(t){if(t&&"function"==typeof t[a.a])return c=t,function(t){var e=c[a.a]();if("function"!=typeof e.subscribe)throw new TypeError("Provided object does not correctly implement Symbol.observable");return e.subscribe(t)};if(Object(s.a)(t))return Object(r.a)(t);if(Object(A.a)(t))return n=t,function(t){return n.then(function(e){t.closed||(t.next(e),t.complete())},function(e){return t.error(e)}).then(null,i.a),t};if(t&&"function"==typeof t[o.a])return e=t,function(t){for(var n=e[o.a]();;){var r=n.next();if(r.done){t.complete();break}if(t.next(r.value),t.closed)break}return"function"==typeof n.return&&t.add(function(){n.return&&n.return()}),t};var e,n,c,u=Object(l.a)(t)?"an invalid object":"'"+t+"'";throw new TypeError("You provided "+u+" where a stream was expected. You can provide an Observable, Promise, Array, or Iterable.")}},function(t,e,n){"use strict";var r=s(n(1152)),i=s(n(1156)),o=s(n(623)),a=s(n(309));function s(t){return t&&t.__esModule?t:{default:t}}t.exports={Transition:a.default,TransitionGroup:o.default,ReplaceTransition:i.default,CSSTransition:r.default}},,function(t,e,n){"use strict";(function(e,r){var i=65536,o=4294967295;var a=n(35).Buffer,s=e.crypto||e.msCrypto;s&&s.getRandomValues?t.exports=function(t,e){if(t>o)throw new RangeError("requested too many random bytes");var n=a.allocUnsafe(t);if(t>0)if(t>i)for(var A=0;A<t;A+=i)s.getRandomValues(n.slice(A,A+i));else s.getRandomValues(n);if("function"==typeof e)return r.nextTick(function(){e(null,n)});return n}:t.exports=function(){throw new Error("Secure random number generation is not supported by this browser.\nUse Chrome, Firefox or Internet Explorer 11")}}).call(this,n(37),n(58))},function(t,e,n){var r=n(35).Buffer;function i(t,e){this._block=r.alloc(t),this._finalSize=e,this._blockSize=t,this._len=0}i.prototype.update=function(t,e){"string"==typeof t&&(e=e||"utf8",t=r.from(t,e));for(var n=this._block,i=this._blockSize,o=t.length,a=this._len,s=0;s<o;){for(var A=a%i,l=Math.min(o-s,i-A),c=0;c<l;c++)n[A+c]=t[s+c];s+=l,(a+=l)%i==0&&this._update(n)}return this._len+=o,this},i.prototype.digest=function(t){var e=this._len%this._blockSize;this._block[e]=128,this._block.fill(0,e+1),e>=this._finalSize&&(this._update(this._block),this._block.fill(0));var n=8*this._len;if(n<=4294967295)this._block.writeUInt32BE(n,this._blockSize-4);else{var r=(4294967295&n)>>>0,i=(n-r)/4294967296;this._block.writeUInt32BE(i,this._blockSize-8),this._block.writeUInt32BE(r,this._blockSize-4)}this._update(this._block);var o=this._hash();return t?o.toString(t):o},i.prototype._update=function(){throw new Error("_update must be implemented by subclass")},t.exports=i},function(t,e,n){"use strict";function r(t,e,n){var r=n?" !== ":" === ",i=n?" || ":" && ",o=n?"!":"",a=n?"":"!";switch(t){case"null":return e+r+"null";case"array":return o+"Array.isArray("+e+")";case"object":return"("+o+e+i+"typeof "+e+r+'"object"'+i+a+"Array.isArray("+e+"))";case"integer":return"(typeof "+e+r+'"number"'+i+a+"("+e+" % 1)"+i+e+r+e+")";default:return"typeof "+e+r+'"'+t+'"'}}t.exports={copy:function(t,e){for(var n in e=e||{},t)e[n]=t[n];return e},checkDataType:r,checkDataTypes:function(t,e){switch(t.length){case 1:return r(t[0],e,!0);default:var n="",i=o(t);for(var a in i.array&&i.object&&(n=i.null?"(":"(!"+e+" || ",n+="typeof "+e+' !== "object")',delete i.null,delete i.array,delete i.object),i.number&&delete i.integer,i)n+=(n?" && ":"")+r(a,e,!0);return n}},coerceToTypes:function(t,e){if(Array.isArray(e)){for(var n=[],r=0;r<e.length;r++){var o=e[r];i[o]?n[n.length]=o:"array"===t&&"array"===o&&(n[n.length]=o)}if(n.length)return n}else{if(i[e])return[e];if("array"===t&&"array"===e)return["array"]}},toHash:o,getProperty:A,escapeQuotes:l,equal:n(283),ucs2length:n(868),varOccurences:function(t,e){e+="[^0-9]";var n=t.match(new RegExp(e,"g"));return n?n.length:0},varReplace:function(t,e,n){return e+="([^0-9])",n=n.replace(/\$/g,"$$$$"),t.replace(new RegExp(e,"g"),n+"$1")},cleanUpCode:function(t){return t.replace(c,"").replace(u,"").replace(g,"if (!($1))")},finalCleanUpCode:function(t,e){var n=t.match(h);n&&2==n.length&&(t=e?t.replace(f,"").replace(C,m):t.replace(d,"").replace(p,I));return(n=t.match(y))&&3===n.length?t.replace(E,""):t},schemaHasRules:function(t,e){if("boolean"==typeof t)return!t;for(var n in t)if(e[n])return!0},schemaHasRulesExcept:function(t,e,n){if("boolean"==typeof t)return!t&&"not"!=n;for(var r in t)if(r!=n&&e[r])return!0},schemaUnknownRules:function(t,e){if("boolean"==typeof t)return;for(var n in t)if(!e[n])return n},toQuotedString:v,getPathExpr:function(t,e,n,r){return Q(t,n?"'/' + "+e+(r?"":".replace(/~/g, '~0').replace(/\\//g, '~1')"):r?"'[' + "+e+" + ']'":"'[\\'' + "+e+" + '\\']'")},getPath:function(t,e,n){var r=v(n?"/"+w(e):A(e));return Q(t,r)},getData:function(t,e,n){var r,i,o,a;if(""===t)return"rootData";if("/"==t[0]){if(!B.test(t))throw new Error("Invalid JSON-pointer: "+t);i=t,o="rootData"}else{if(!(a=t.match(b)))throw new Error("Invalid JSON-pointer: "+t);if(r=+a[1],"#"==(i=a[2])){if(r>=e)throw new Error("Cannot access property/index "+r+" levels up, current level is "+e);return n[e-r]}if(r>e)throw new Error("Cannot access data "+r+" levels up, current level is "+e);if(o="data"+(e-r||""),!i)return o}for(var s=o,l=i.split("/"),c=0;c<l.length;c++){var u=l[c];u&&(o+=A(x(u)),s+=" && "+o)}return s},unescapeFragment:function(t){return x(decodeURIComponent(t))},unescapeJsonPointer:x,escapeFragment:function(t){return encodeURIComponent(w(t))},escapeJsonPointer:w};var i=o(["string","number","integer","boolean","null"]);function o(t){for(var e={},n=0;n<t.length;n++)e[t[n]]=!0;return e}var a=/^[a-z$_][a-z$_0-9]*$/i,s=/'|\\/g;function A(t){return"number"==typeof t?"["+t+"]":a.test(t)?"."+t:"['"+l(t)+"']"}function l(t){return t.replace(s,"\\$&").replace(/\n/g,"\\n").replace(/\r/g,"\\r").replace(/\f/g,"\\f").replace(/\t/g,"\\t")}var c=/else\s*{\s*}/g,u=/if\s*\([^)]+\)\s*\{\s*\}(?!\s*else)/g,g=/if\s*\(([^)]+)\)\s*\{\s*\}\s*else(?!\s*if)/g;var h=/[^v.]errors/g,d=/var errors = 0;|var vErrors = null;|validate.errors = vErrors;/g,f=/var errors = 0;|var vErrors = null;/g,p="return errors === 0;",I="validate.errors = null; return true;",C=/if \(errors === 0\) return data;\s*else throw new ValidationError\(vErrors\);/,m="return data;",y=/[^A-Za-z_$]rootData[^A-Za-z0-9_$]/g,E=/if \(rootData === undefined\) rootData = data;/;function v(t){return"'"+l(t)+"'"}var B=/^\/(?:[^~]|~0|~1)*$/,b=/^([0-9]+)(#|\/(?:[^~]|~0|~1)*)?$/;function Q(t,e){return'""'==t?e:(t+" + "+e).replace(/' \+ '/g,"")}function w(t){return t.replace(/~/g,"~0").replace(/\//g,"~1")}function x(t){return t.replace(/~1/g,"/").replace(/~0/g,"~")}},function(t,e){t.exports={Text:"text",Directive:"directive",Comment:"comment",Script:"script",Style:"style",Tag:"tag",CDATA:"cdata",Doctype:"doctype",isTag:function(t){return"tag"===t.type||"script"===t.type||"style"===t.type}}},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0});var r=n(615),i=n(616),o=n(218),a=n(617),s=t.env.BLUEPRINT_NAMESPACE||"bp3";function A(t){if(null!=t)return s+"-elevation-"+t}function l(t){if(null!=t&&t!==o.Intent.NONE)return s+"-intent-"+t.toLowerCase()}function c(t){if(null!=t)return s+"-position-"+t}e.ACTIVE=s+"-active",e.ALIGN_LEFT=s+"-align-left",e.ALIGN_RIGHT=s+"-align-right",e.DARK=s+"-dark",e.DISABLED=s+"-disabled",e.FILL=s+"-fill",e.FIXED=s+"-fixed",e.FIXED_TOP=s+"-fixed-top",e.INLINE=s+"-inline",e.INTERACTIVE=s+"-interactive",e.LARGE=s+"-large",e.LOADING=s+"-loading",e.MINIMAL=s+"-minimal",e.MULTILINE=s+"-multiline",e.ROUND=s+"-round",e.SMALL=s+"-small",e.VERTICAL=s+"-vertical",e.POSITION_TOP=c(a.Position.TOP),e.POSITION_BOTTOM=c(a.Position.BOTTOM),e.POSITION_LEFT=c(a.Position.LEFT),e.POSITION_RIGHT=c(a.Position.RIGHT),e.ELEVATION_0=A(i.Elevation.ZERO),e.ELEVATION_1=A(i.Elevation.ONE),e.ELEVATION_2=A(i.Elevation.TWO),e.ELEVATION_3=A(i.Elevation.THREE),e.ELEVATION_4=A(i.Elevation.FOUR),e.INTENT_PRIMARY=l(o.Intent.PRIMARY),e.INTENT_SUCCESS=l(o.Intent.SUCCESS),e.INTENT_WARNING=l(o.Intent.WARNING),e.INTENT_DANGER=l(o.Intent.DANGER),e.FOCUS_DISABLED=s+"-focus-disabled",e.UI_TEXT=s+"-ui-text",e.RUNNING_TEXT=s+"-running-text",e.MONOSPACE_TEXT=s+"-monospace-text",e.TEXT_LARGE=s+"-text-large",e.TEXT_SMALL=s+"-text-small",e.TEXT_MUTED=s+"-text-muted",e.TEXT_DISABLED=s+"-text-disabled",e.TEXT_OVERFLOW_ELLIPSIS=s+"-text-overflow-ellipsis",e.BLOCKQUOTE=s+"-blockquote",e.CODE=s+"-code",e.CODE_BLOCK=s+"-code-block",e.HEADING=s+"-heading",e.LIST=s+"-list",e.LIST_UNSTYLED=s+"-list-unstyled",e.RTL=s+"-rtl",e.ALERT=s+"-alert",e.ALERT_BODY=e.ALERT+"-body",e.ALERT_CONTENTS=e.ALERT+"-contents",e.ALERT_FOOTER=e.ALERT+"-footer",e.BREADCRUMB=s+"-breadcrumb",e.BREADCRUMB_CURRENT=e.BREADCRUMB+"-current",e.BREADCRUMBS=e.BREADCRUMB+"s",e.BREADCRUMBS_COLLAPSED=e.BREADCRUMB+"s-collapsed",e.BUTTON=s+"-button",e.BUTTON_GROUP=e.BUTTON+"-group",e.BUTTON_SPINNER=e.BUTTON+"-spinner",e.BUTTON_TEXT=e.BUTTON+"-text",e.CALLOUT=s+"-callout",e.CALLOUT_ICON=e.CALLOUT+"-icon",e.CARD=s+"-card",e.COLLAPSE=s+"-collapse",e.COLLAPSE_BODY=e.COLLAPSE+"-body",e.COLLAPSIBLE_LIST=s+"-collapse-list",e.CONTEXT_MENU=s+"-context-menu",e.CONTEXT_MENU_POPOVER_TARGET=e.CONTEXT_MENU+"-popover-target",e.CONTROL_GROUP=s+"-control-group",e.DIALOG=s+"-dialog",e.DIALOG_CONTAINER=e.DIALOG+"-container",e.DIALOG_BODY=e.DIALOG+"-body",e.DIALOG_CLOSE_BUTTON=e.DIALOG+"-close-button",e.DIALOG_FOOTER=e.DIALOG+"-footer",e.DIALOG_FOOTER_ACTIONS=e.DIALOG+"-footer-actions",e.DIALOG_HEADER=e.DIALOG+"-header",e.DIVIDER=s+"-divider",e.DRAWER=s+"-drawer",e.DRAWER_BODY=e.DRAWER+"-body",e.DRAWER_FOOTER=e.DRAWER+"-footer",e.DRAWER_HEADER=e.DRAWER+"-header",e.EDITABLE_TEXT=s+"-editable-text",e.EDITABLE_TEXT_CONTENT=e.EDITABLE_TEXT+"-content",e.EDITABLE_TEXT_EDITING=e.EDITABLE_TEXT+"-editing",e.EDITABLE_TEXT_INPUT=e.EDITABLE_TEXT+"-input",e.EDITABLE_TEXT_PLACEHOLDER=e.EDITABLE_TEXT+"-placeholder",e.FLEX_EXPANDER=s+"-flex-expander",e.HTML_SELECT=s+"-html-select",e.SELECT=s+"-select",e.HTML_TABLE=s+"-html-table",e.HTML_TABLE_BORDERED=e.HTML_TABLE+"-bordered",e.HTML_TABLE_CONDENSED=e.HTML_TABLE+"-condensed",e.HTML_TABLE_STRIPED=e.HTML_TABLE+"-striped",e.INPUT=s+"-input",e.INPUT_GHOST=e.INPUT+"-ghost",e.INPUT_GROUP=e.INPUT+"-group",e.INPUT_ACTION=e.INPUT+"-action",e.CONTROL=s+"-control",e.CONTROL_INDICATOR=e.CONTROL+"-indicator",e.CONTROL_INDICATOR_CHILD=e.CONTROL_INDICATOR+"-child",e.CHECKBOX=s+"-checkbox",e.RADIO=s+"-radio",e.SWITCH=s+"-switch",e.SWITCH_INNER_TEXT=e.SWITCH+"-inner-text",e.FILE_INPUT=s+"-file-input",e.FILE_INPUT_HAS_SELECTION=s+"-file-input-has-selection",e.FILE_UPLOAD_INPUT=s+"-file-upload-input",e.KEY=s+"-key",e.KEY_COMBO=e.KEY+"-combo",e.MODIFIER_KEY=s+"-modifier-key",e.HOTKEY=s+"-hotkey",e.HOTKEY_LABEL=e.HOTKEY+"-label",e.HOTKEY_COLUMN=e.HOTKEY+"-column",e.HOTKEY_DIALOG=e.HOTKEY+"-dialog",e.LABEL=s+"-label",e.FORM_GROUP=s+"-form-group",e.FORM_CONTENT=s+"-form-content",e.FORM_HELPER_TEXT=s+"-form-helper-text",e.MENU=s+"-menu",e.MENU_ITEM=e.MENU+"-item",e.MENU_ITEM_LABEL=e.MENU_ITEM+"-label",e.MENU_SUBMENU=s+"-submenu",e.MENU_DIVIDER=e.MENU+"-divider",e.MENU_HEADER=e.MENU+"-header",e.NAVBAR=s+"-navbar",e.NAVBAR_GROUP=e.NAVBAR+"-group",e.NAVBAR_HEADING=e.NAVBAR+"-heading",e.NAVBAR_DIVIDER=e.NAVBAR+"-divider",e.NON_IDEAL_STATE=s+"-non-ideal-state",e.NON_IDEAL_STATE_VISUAL=e.NON_IDEAL_STATE+"-visual",e.NUMERIC_INPUT=s+"-numeric-input",e.OVERFLOW_LIST=s+"-overflow-list",e.OVERFLOW_LIST_SPACER=e.OVERFLOW_LIST+"-spacer",e.OVERLAY=s+"-overlay",e.OVERLAY_BACKDROP=e.OVERLAY+"-backdrop",e.OVERLAY_CONTAINER=e.OVERLAY+"-container",e.OVERLAY_CONTENT=e.OVERLAY+"-content",e.OVERLAY_INLINE=e.OVERLAY+"-inline",e.OVERLAY_OPEN=e.OVERLAY+"-open",e.OVERLAY_SCROLL_CONTAINER=e.OVERLAY+"-scroll-container",e.PANEL_STACK=s+"-panel-stack",e.PANEL_STACK_HEADER=e.PANEL_STACK+"-header",e.PANEL_STACK_HEADER_BACK=e.PANEL_STACK+"-header-back",e.PANEL_STACK_VIEW=e.PANEL_STACK+"-view",e.POPOVER=s+"-popover",e.POPOVER_ARROW=e.POPOVER+"-arrow",e.POPOVER_BACKDROP=e.POPOVER+"-backdrop",e.POPOVER_CONTENT=e.POPOVER+"-content",e.POPOVER_CONTENT_SIZING=e.POPOVER_CONTENT+"-sizing",e.POPOVER_DISMISS=e.POPOVER+"-dismiss",e.POPOVER_DISMISS_OVERRIDE=e.POPOVER_DISMISS+"-override",e.POPOVER_OPEN=e.POPOVER+"-open",e.POPOVER_TARGET=e.POPOVER+"-target",e.POPOVER_WRAPPER=e.POPOVER+"-wrapper",e.TRANSITION_CONTAINER=s+"-transition-container",e.PROGRESS_BAR=s+"-progress-bar",e.PROGRESS_METER=s+"-progress-meter",e.PROGRESS_NO_STRIPES=s+"-no-stripes",e.PROGRESS_NO_ANIMATION=s+"-no-animation",e.PORTAL=s+"-portal",e.SKELETON=s+"-skeleton",e.SLIDER=s+"-slider",e.SLIDER_AXIS=e.SLIDER+"-axis",e.SLIDER_HANDLE=e.SLIDER+"-handle",e.SLIDER_LABEL=e.SLIDER+"-label",e.SLIDER_TRACK=e.SLIDER+"-track",e.SLIDER_PROGRESS=e.SLIDER+"-progress",e.START=s+"-start",e.END=s+"-end",e.SPINNER=s+"-spinner",e.SPINNER_ANIMATION=e.SPINNER+"-animation",e.SPINNER_HEAD=e.SPINNER+"-head",e.SPINNER_NO_SPIN=s+"-no-spin",e.SPINNER_TRACK=e.SPINNER+"-track",e.TAB=s+"-tab",e.TAB_INDICATOR=e.TAB+"-indicator",e.TAB_INDICATOR_WRAPPER=e.TAB_INDICATOR+"-wrapper",e.TAB_LIST=e.TAB+"-list",e.TAB_PANEL=e.TAB+"-panel",e.TABS=e.TAB+"s",e.TAG=s+"-tag",e.TAG_REMOVE=e.TAG+"-remove",e.TAG_INPUT=s+"-tag-input",e.TAG_INPUT_ICON=e.TAG_INPUT+"-icon",e.TAG_INPUT_VALUES=e.TAG_INPUT+"-values",e.TOAST=s+"-toast",e.TOAST_CONTAINER=e.TOAST+"-container",e.TOAST_MESSAGE=e.TOAST+"-message",e.TOOLTIP=s+"-tooltip",e.TOOLTIP_INDICATOR=e.TOOLTIP+"-indicator",e.TREE=s+"-tree",e.TREE_NODE=s+"-tree-node",e.TREE_NODE_CARET=e.TREE_NODE+"-caret",e.TREE_NODE_CARET_CLOSED=e.TREE_NODE_CARET+"-closed",e.TREE_NODE_CARET_NONE=e.TREE_NODE_CARET+"-none",e.TREE_NODE_CARET_OPEN=e.TREE_NODE_CARET+"-open",e.TREE_NODE_CONTENT=e.TREE_NODE+"-content",e.TREE_NODE_EXPANDED=e.TREE_NODE+"-expanded",e.TREE_NODE_ICON=e.TREE_NODE+"-icon",e.TREE_NODE_LABEL=e.TREE_NODE+"-label",e.TREE_NODE_LIST=e.TREE_NODE+"-list",e.TREE_NODE_SECONDARY_LABEL=e.TREE_NODE+"-secondary-label",e.TREE_NODE_SELECTED=e.TREE_NODE+"-selected",e.TREE_ROOT=s+"-tree-root",e.ICON=s+"-icon",e.ICON_STANDARD=e.ICON+"-standard",e.ICON_LARGE=e.ICON+"-large",e.getClassNamespace=function(){return s},e.alignmentClass=function(t){switch(t){case r.Alignment.LEFT:return e.ALIGN_LEFT;case r.Alignment.RIGHT:return e.ALIGN_RIGHT;default:return}},e.elevationClass=A,e.iconClass=function(t){if(null!=t)return 0===t.indexOf(s+"-icon-")?t:s+"-icon-"+t},e.intentClass=l,e.positionClass=c}).call(this,n(58))},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0});var r=n(2),i=r.__importStar(n(0)),o=n(619);function a(t){return"function"==typeof t}function s(t){return"object"==typeof t&&void 0!==t.type&&void 0!==t.props}function A(t,e,n){var r=!1;return function(){for(var i=[],o=0;o<arguments.length;o++)i[o]=arguments[o];a(e)&&e.apply(void 0,i),r||(r=!0,a(n)&&n.apply(void 0,i),requestAnimationFrame(function(){t.apply(void 0,i),r=!1}))}}r.__exportStar(n(1138),e),r.__exportStar(n(1139),e),e.isNodeEnv=function(e){return void 0!==t&&t.env&&"production"===e},e.isFunction=a,e.isReactNodeEmpty=function t(e,n){return void 0===n&&(n=!1),null==e||""===e||!1===e||!n&&Array.isArray(e)&&(0===e.length||e.every(function(e){return t(e,!0)}))},e.ensureElement=function(t,e){return void 0===e&&(e="span"),null==t||"boolean"==typeof t?void 0:"string"==typeof t?t.trim().length>0?i.createElement(e,{},t):void 0:"number"==typeof t||"symbol"==typeof t.type||Array.isArray(t)?i.createElement(e,{},t):s(t)?t:void 0},e.isReactElement=s,e.getDisplayName=function(t){return t.displayName||t.name||"Unknown"},e.isElementOfType=function(t,e){return null!=t&&null!=t.type&&null!=t.type.displayName&&t.type.displayName===e.displayName},e.safeInvoke=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];if(a(t))return t.apply(void 0,e)},e.safeInvokeOrValue=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return a(t)?t.apply(void 0,e):t},e.elementIsOrContains=function(t,e){return t===e||t.contains(e)},e.arrayLengthCompare=function(t,e){return void 0===t&&(t=[]),void 0===e&&(e=[]),t.length-e.length},e.approxEqual=function(t,e,n){return void 0===n&&(n=1e-5),Math.abs(t-e)<=n},e.clamp=function(t,e,n){if(null==t)return t;if(n<e)throw new Error(o.CLAMP_MIN_MAX);return Math.min(Math.max(t,e),n)},e.countDecimalPlaces=function(t){if(!isFinite(t))return 0;for(var e=1,n=0;Math.round(t*e)/e!==t;)e*=10,n++;return n},e.throttleEvent=function(t,e,n){var r=A(function(e){t.dispatchEvent(new CustomEvent(n,e))});return t.addEventListener(e,r),r},e.throttleReactEventCallback=function(t,e){return void 0===e&&(e={}),A(t,function(t){e.preventDefault&&t.preventDefault()},function(t){return t.persist()})},e.throttle=function(t){return A(t)}}).call(this,n(58))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(6),i=n(16),o=n(25);class a extends r.Widget{static Create(t){return new class extends a{render(){return t}}}constructor(){super(),this.update()}dispose(){this.isDisposed||(i.unmountComponentAtNode(this.node),super.dispose())}processMessage(t){switch(t.type){case"before-change-doc-owner":return void this.onBeforeChangeDocumentOwner(t);case"after-change-doc-owner":return void this.onAfterChangeDocumentOwner(t);default:super.processMessage(t)}}onUpdateRequest(){const t=this.render();i.render(t,this.node)}onBeforeChangeDocumentOwner(t){i.unmountComponentAtNode(this.node)}onAfterChangeDocumentOwner(t){o.MessageLoop.sendMessage(this,r.Widget.Msg.UpdateRequest)}}e.ReactWrapperWidget=a,function(t){t.BeforeChangeDocumentOwner=new o.Message("before-change-doc-owner"),t.AfterChangeDocumentOwner=new o.Message("after-change-doc-owner")}(a=e.ReactWrapperWidget||(e.ReactWrapperWidget={}))},function(t,e,n){var r=n(111),i=n(178);t.exports=n(112)?function(t,e,n){return r.f(t,e,i(1,n))}:function(t,e,n){return t[e]=n,t}},function(t,e,n){var r=n(144);t.exports=function(t){if(!r(t))throw TypeError(t+" is not an object!");return t}},function(t,e){t.exports=function(t){return"object"==typeof t?null!==t:"function"==typeof t}},function(t,e){t.exports=function(t){try{return!!t()}catch(t){return!0}}},function(t,e,n){"use strict";e.__esModule=!0,e.message=e.accessor=e.disabled=e.dateFormat=e.numberFormat=void 0;var r=s(n(1)),i=s(n(1245));e.elementType=i.default;var o=s(n(667)),a=n(1248);function s(t){return t&&t.__esModule?t:{default:t}}var A=(0,o.default)(function(){return a.number.propType.apply(a.number,arguments)});e.numberFormat=A;var l=(0,o.default)(function(){return a.date.propType.apply(a.date,arguments)});e.dateFormat=l;var c=(0,o.default)(function(){return r.default.bool.apply(r.default,arguments)});e.disabled=c,c.acceptsArray=r.default.oneOfType([c,r.default.array]);var u=r.default.oneOfType([r.default.string,r.default.func]);e.accessor=u;var g=r.default.oneOfType([r.default.node,r.default.string,r.default.func]);e.message=g},function(t,e,n){var r=n(1351),i=n(1356);t.exports=function(t,e){var n=i(t,e);return r(n)?n:void 0}},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r=n(2),i=n(11);function o(){return function(t){return t.lift(new a(t))}}var a=function(){function t(t){this.connectable=t}return t.prototype.call=function(t,e){var n=this.connectable;n._refCount++;var r=new s(t,n),i=e.subscribe(r);return r.closed||(r.connection=n.connect()),i},t}(),s=function(t){function e(e,n){var r=t.call(this,e)||this;return r.connectable=n,r}return r.__extends(e,t),e.prototype._unsubscribe=function(){var t=this.connectable;if(t){this.connectable=null;var e=t._refCount;if(e<=0)this.connection=null;else if(t._refCount=e-1,e>1)this.connection=null;else{var n=this.connection,r=t._connection;this.connection=null,!r||n&&r!==n||r.unsubscribe()}}else this.connection=null},e}(i.a)},function(t,e,n){"use strict";n.d(e,"a",function(){return c});var r=n(2),i=n(36),o=n(256),a=n(32),s=n(248),A=n(81),l=n(258),c=function(t){function e(e,n,r){void 0===e&&(e=Number.POSITIVE_INFINITY),void 0===n&&(n=Number.POSITIVE_INFINITY);var i=t.call(this)||this;return i.scheduler=r,i._events=[],i._infiniteTimeWindow=!1,i._bufferSize=e<1?1:e,i._windowTime=n<1?1:n,n===Number.POSITIVE_INFINITY?(i._infiniteTimeWindow=!0,i.next=i.nextInfiniteTimeWindow):i.next=i.nextTimeWindow,i}return r.__extends(e,t),e.prototype.nextInfiniteTimeWindow=function(e){var n=this._events;n.push(e),n.length>this._bufferSize&&n.shift(),t.prototype.next.call(this,e)},e.prototype.nextTimeWindow=function(e){this._events.push(new u(this._getNow(),e)),this._trimBufferThenGetEvents(),t.prototype.next.call(this,e)},e.prototype._subscribe=function(t){var e,n=this._infiniteTimeWindow,r=n?this._events:this._trimBufferThenGetEvents(),i=this.scheduler,o=r.length;if(this.closed)throw new A.a;if(this.isStopped||this.hasError?e=a.a.EMPTY:(this.observers.push(t),e=new l.a(this,t)),i&&t.add(t=new s.a(t,i)),n)for(var c=0;c<o&&!t.closed;c++)t.next(r[c]);else for(c=0;c<o&&!t.closed;c++)t.next(r[c].value);return this.hasError?t.error(this.thrownError):this.isStopped&&t.complete(),e},e.prototype._getNow=function(){return(this.scheduler||o.a).now()},e.prototype._trimBufferThenGetEvents=function(){for(var t=this._getNow(),e=this._bufferSize,n=this._windowTime,r=this._events,i=r.length,o=0;o<i&&!(t-r[o].time<n);)o++;return i>e&&(o=Math.max(o,i-e)),o>0&&r.splice(0,o),r},e}(i.a),u=function(){return function(t,e){this.time=t,this.value=e}}()},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=n(21);function i(t,e){return e?new r.a(function(n){return e.schedule(o,0,{error:t,subscriber:n})}):new r.a(function(e){return e.error(t)})}function o(t){var e=t.error;t.subscriber.error(e)}},function(t,e,n){"use strict";n.d(e,"b",function(){return c}),n.d(e,"a",function(){return u});var r=n(2),i=n(45),o=n(38),a=n(26),s=n(24),A=n(90),l={};function c(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];var n=null,r=null;return Object(i.a)(t[t.length-1])&&(r=t.pop()),"function"==typeof t[t.length-1]&&(n=t.pop()),1===t.length&&Object(o.a)(t[0])&&(t=t[0]),Object(A.a)(t,r).lift(new u(n))}var u=function(){function t(t){this.resultSelector=t}return t.prototype.call=function(t,e){return e.subscribe(new g(t,this.resultSelector))},t}(),g=function(t){function e(e,n){var r=t.call(this,e)||this;return r.resultSelector=n,r.active=0,r.values=[],r.observables=[],r}return r.__extends(e,t),e.prototype._next=function(t){this.values.push(l),this.observables.push(t)},e.prototype._complete=function(){var t=this.observables,e=t.length;if(0===e)this.destination.complete();else{this.active=e,this.toRespond=e;for(var n=0;n<e;n++){var r=t[n];this.add(Object(s.a)(this,r,r,n))}}},e.prototype.notifyComplete=function(t){0==(this.active-=1)&&this.destination.complete()},e.prototype.notifyNext=function(t,e,n,r,i){var o=this.values,a=o[n],s=this.toRespond?a===l?--this.toRespond:this.toRespond:0;o[n]=e,0===s&&(this.resultSelector?this._tryResultSelector(o):this.destination.next(o.slice()))},e.prototype._tryResultSelector=function(t){var e;try{e=this.resultSelector.apply(this,t)}catch(t){return void this.destination.error(t)}this.destination.next(e)},e}(a.a)},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r=n(84),i=n(73);function o(t){return void 0===t&&(t=Number.POSITIVE_INFINITY),Object(r.a)(i.a,t)}},function(t,e,n){"use strict";n.d(e,"a",function(){return a});var r=n(21),i=n(55),o=n(51);function a(t){return new r.a(function(e){var n;try{n=t()}catch(t){return void e.error(t)}return(n?Object(i.a)(n):Object(o.b)()).subscribe(e)})}},function(t,e,n){"use strict";n.d(e,"b",function(){return c}),n.d(e,"a",function(){return u});var r=n(2),i=n(90),o=n(38),a=n(11),s=n(26),A=n(24),l=n(75);function c(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];var n=t[t.length-1];return"function"==typeof n&&t.pop(),Object(i.a)(t,void 0).lift(new u(n))}var u=function(){function t(t){this.resultSelector=t}return t.prototype.call=function(t,e){return e.subscribe(new g(t,this.resultSelector))},t}(),g=function(t){function e(e,n,r){void 0===r&&(r=Object.create(null));var i=t.call(this,e)||this;return i.iterators=[],i.active=0,i.resultSelector="function"==typeof n?n:null,i.values=r,i}return r.__extends(e,t),e.prototype._next=function(t){var e=this.iterators;Object(o.a)(t)?e.push(new d(t)):"function"==typeof t[l.a]?e.push(new h(t[l.a]())):e.push(new f(this.destination,this,t))},e.prototype._complete=function(){var t=this.iterators,e=t.length;if(this.unsubscribe(),0!==e){this.active=e;for(var n=0;n<e;n++){var r=t[n];if(r.stillUnsubscribed)this.destination.add(r.subscribe(r,n));else this.active--}}else this.destination.complete()},e.prototype.notifyInactive=function(){this.active--,0===this.active&&this.destination.complete()},e.prototype.checkIterators=function(){for(var t=this.iterators,e=t.length,n=this.destination,r=0;r<e;r++){if("function"==typeof(a=t[r]).hasValue&&!a.hasValue())return}var i=!1,o=[];for(r=0;r<e;r++){var a,s=(a=t[r]).next();if(a.hasCompleted()&&(i=!0),s.done)return void n.complete();o.push(s.value)}this.resultSelector?this._tryresultSelector(o):n.next(o),i&&n.complete()},e.prototype._tryresultSelector=function(t){var e;try{e=this.resultSelector.apply(this,t)}catch(t){return void this.destination.error(t)}this.destination.next(e)},e}(a.a),h=function(){function t(t){this.iterator=t,this.nextResult=t.next()}return t.prototype.hasValue=function(){return!0},t.prototype.next=function(){var t=this.nextResult;return this.nextResult=this.iterator.next(),t},t.prototype.hasCompleted=function(){var t=this.nextResult;return t&&t.done},t}(),d=function(){function t(t){this.array=t,this.index=0,this.length=0,this.length=t.length}return t.prototype[l.a]=function(){return this},t.prototype.next=function(t){var e=this.index++,n=this.array;return e<this.length?{value:n[e],done:!1}:{value:null,done:!0}},t.prototype.hasValue=function(){return this.array.length>this.index},t.prototype.hasCompleted=function(){return this.array.length===this.index},t}(),f=function(t){function e(e,n,r){var i=t.call(this,e)||this;return i.parent=n,i.observable=r,i.stillUnsubscribed=!0,i.buffer=[],i.isComplete=!1,i}return r.__extends(e,t),e.prototype[l.a]=function(){return this},e.prototype.next=function(){var t=this.buffer;return 0===t.length&&this.isComplete?{value:null,done:!0}:{value:t.shift(),done:!1}},e.prototype.hasValue=function(){return this.buffer.length>0},e.prototype.hasCompleted=function(){return 0===this.buffer.length&&this.isComplete},e.prototype.notifyComplete=function(){this.buffer.length>0?(this.isComplete=!0,this.parent.notifyInactive()):this.destination.complete()},e.prototype.notifyNext=function(t,e,n,r,i){this.buffer.push(e),this.parent.checkIterators()},e.prototype.subscribe=function(t,e){return Object(A.a)(this,this.observable,this,e)},e}(s.a)},function(t,e){t.exports=function(t,e){t.prototype=Object.create(e.prototype),t.prototype.constructor=t,t.__proto__=e}},function(t,e,n){"use strict";n.d(e,"a",function(){return c}),n.d(e,"b",function(){return u});var r=n(6),i=n(384),o=n(72);const a="jp-InputArea",s="jp-InputArea-prompt",A="jp-InputPrompt",l="jp-InputArea-editor";class c extends r.Widget{constructor(t){super(),this._prompt=null,this._editor=null,this._rendered=null,this.addClass(a);let e=this.model=t.model,n=this.contentFactory=t.contentFactory||c.defaultContentFactory,o=this._prompt=n.createInputPrompt();o.addClass(s);let A={model:e,factory:n.editorFactory,updateOnShow:t.updateOnShow},u=this._editor=new i.a(A);u.addClass(l);let g=this.layout=new r.PanelLayout;g.addWidget(o),g.addWidget(u)}get editorWidget(){return this._editor}get editor(){return this._editor.editor}get promptNode(){return this._prompt.node}renderInput(t){let e=this.layout;this._rendered&&(this._rendered.parent=null),this._editor.hide(),this._rendered=t,e.addWidget(t)}showEditor(){this._rendered&&(this._rendered.parent=null),this._editor.show()}setPrompt(t){this._prompt.executionCount=t}dispose(){this.isDisposed||(this._prompt=null,this._editor=null,this._rendered=null,super.dispose())}}!function(t){class e{constructor(e={}){this._editor=null,this._editor=e.editorFactory||t.defaultEditorFactory}get editorFactory(){return this._editor}createInputPrompt(){return new u}}t.ContentFactory=e,t.defaultEditorFactory=(new o.CodeMirrorEditorFactory).newInlineEditor,t.defaultContentFactory=new e({})}(c||(c={}));class u extends r.Widget{constructor(){super(),this._executionCount=null,this.addClass(A)}get executionCount(){return this._executionCount}set executionCount(t){this._executionCount=t,this.node.textContent=null===t?" ":`[${t||" "}]:`}}},function(t,e,n){"use strict";function r(t){return null!==t&&"object"==typeof t}n.d(e,"a",function(){return r})},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=n(11);function i(t){for(;t;){var e=t,n=e.closed,i=e.destination,o=e.isStopped;if(n||o)return!1;t=i&&i instanceof r.a?i:null}return!0}},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r=n(21),i=n(32);function o(t,e){return new r.a(function(n){var r=new i.a,o=0;return r.add(e.schedule(function(){o!==t.length?(n.next(t[o++]),n.closed||r.add(this.schedule())):n.complete()})),r})}},function(t,e,n){"use strict";var r=n(2),i=1,o={};var a=function(t){var e=i++;return o[e]=t,Promise.resolve().then(function(){return function(t){var e=o[t];e&&e()}(e)}),e},s=function(t){delete o[t]},A=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.scheduler=e,r.work=n,r}return r.__extends(e,t),e.prototype.requestAsyncId=function(e,n,r){return void 0===r&&(r=0),null!==r&&r>0?t.prototype.requestAsyncId.call(this,e,n,r):(e.actions.push(this),e.scheduled||(e.scheduled=a(e.flush.bind(e,null))))},e.prototype.recycleAsyncId=function(e,n,r){if(void 0===r&&(r=0),null!==r&&r>0||null===r&&this.delay>0)return t.prototype.recycleAsyncId.call(this,e,n,r);0===e.actions.length&&(s(n),e.scheduled=void 0)},e}(n(93).a),l=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.__extends(e,t),e.prototype.flush=function(t){this.active=!0,this.scheduled=void 0;var e,n=this.actions,r=-1,i=n.length;t=t||n.shift();do{if(e=t.execute(t.state,t.delay))break}while(++r<i&&(t=n.shift()));if(this.active=!1,e){for(;++r<i&&(t=n.shift());)t.unsubscribe();throw e}},e}(n(91).a);n.d(e,"a",function(){return c});var c=new l(A)},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r=n(63),i=n(104),o={closed:!0,next:function(t){},error:function(t){if(r.a.useDeprecatedSynchronousErrorHandling)throw t;Object(i.a)(t)},complete:function(){}}},function(t,e,n){(function(e){for(var r=n(1273),i="undefined"==typeof window?e:window,o=["moz","webkit"],a="AnimationFrame",s=i["request"+a],A=i["cancel"+a]||i["cancelRequest"+a],l=0;!s&&l<o.length;l++)s=i[o[l]+"Request"+a],A=i[o[l]+"Cancel"+a]||i[o[l]+"CancelRequest"+a];if(!s||!A){var c=0,u=0,g=[];s=function(t){if(0===g.length){var e=r(),n=Math.max(0,1e3/60-(e-c));c=n+e,setTimeout(function(){var t=g.slice(0);g.length=0;for(var e=0;e<t.length;e++)if(!t[e].cancelled)try{t[e].callback(c)}catch(t){setTimeout(function(){throw t},0)}},Math.round(n))}return g.push({handle:++u,callback:t,cancelled:!1}),u},A=function(t){for(var e=0;e<g.length;e++)g[e].handle===t&&(g[e].cancelled=!0)}}t.exports=function(t){return s.call(i,t)},t.exports.cancel=function(){A.apply(i,arguments)},t.exports.polyfill=function(t){t||(t=i),t.requestAnimationFrame=s,t.cancelAnimationFrame=A}}).call(this,n(37))},function(t,e,n){"use strict";e.__esModule=!0;var r,i=n(1299),o=(r=i)&&r.__esModule?r:{default:r};e.default=function(t){if(Array.isArray(t)){for(var e=0,n=Array(t.length);e<t.length;e++)n[e]=t[e];return n}return(0,o.default)(t)}},function(t,e,n){"use strict";var r=n(0),i=n.n(r),o=n(29);Object(o.style)({background:"black",height:"10px",width:"100px",border:"1px solid black",borderRadius:"3px",marginLeft:"4px",overflow:"hidden"}),Object(o.style)({background:"var(--jp-brand-color2)",height:"10px"});var a="var(--jp-layout-color3)",s="var(--jp-brand-color1)",A="var(--jp-layout-color2)",l="24px",c="var(--jp-ui-font-color1)",u="white",g="2px",h="6px",d="10px";const f={fontSize:"var(--jp-ui-font-size1)",fontFamily:"var(--jp-ui-font-family)"},p=Object(o.style)(f,{lineHeight:"24px",color:c});var I=function(t,e){var n={};for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&e.indexOf(r)<0&&(n[r]=t[r]);if(null!=t&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(t);i<r.length;i++)e.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(t,r[i])&&(n[r[i]]=t[r[i]])}return n};function C(t){const{title:e,source:n,className:i}=t,a=I(t,["title","source","className"]);return r.createElement("span",Object.assign({className:Object(o.classes)(p,i),title:e},a),n)}const m={display:"flex",alignItems:"center"},y={flexDirection:"row"};var E=function(t,e){var n={};for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&e.indexOf(r)<0&&(n[r]=t[r]);if(null!=t&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(t);i<r.length;i++)e.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(t,r[i])&&(n[r[i]]=t[r[i]])}return n};const v=Object(o.style)(m,y);function B(t){const{spacing:e,children:n,className:i}=t,a=E(t,["spacing","children","className"]),s=r.Children.count(n);return r.createElement("div",Object.assign({className:Object(o.classes)(v,i)},a),r.Children.map(n,(t,n)=>0===n?r.createElement("div",{style:{marginRight:`${e}px`}},t):n===s-1?r.createElement("div",{style:{marginLeft:`${e}px`}},t):r.createElement("div",{style:{margin:`0px ${e}px`}},t)))}var b=n(582),Q=n(6);const w={paddingLeft:h,paddingRight:h},x={$nest:{"&:hover":{backgroundColor:a}}},_={backgroundColor:s,$nest:{["."+p]:{color:u}}},D=(Object(o.style)({background:A,minHeight:l,justifyContent:"space-between",paddingLeft:d,paddingRight:d},m),Object(o.style)(m),Object(o.style)(y),Object(o.style)({flexDirection:"row-reverse"}),Object(o.style)({maxHeight:l,marginLeft:g,marginRight:g,height:l},w),Object(o.style)(_)),M=Object(o.style)(x),k=Object(o.style)({boxShadow:"0px 4px 4px rgba(0, 0, 0, 0.25)"});function S(t){let e=new L(t);return e.launch(),e}class L extends Q.Widget{constructor(t){super(),this._body=t.body,this._body.addClass(k),this._anchor=t.anchor,this._align=t.align,(this.layout=new Q.PanelLayout).addWidget(t.body),this._body.node.addEventListener("resize",()=>{this.update()})}launch(){this._setGeometry(),Q.Widget.attach(this,document.body),this.update(),this._anchor.addClass(D),this._anchor.removeClass(M)}onUpdateRequest(t){this._setGeometry(),super.onUpdateRequest(t)}onAfterAttach(t){document.addEventListener("click",this,!1),this.node.addEventListener("keydown",this,!1),window.addEventListener("resize",this,!1)}onAfterDetach(t){document.removeEventListener("click",this,!1),this.node.removeEventListener("keydown",this,!1),window.removeEventListener("resize",this,!1)}onResize(){this.update()}dispose(){super.dispose(),this._anchor.removeClass(D),this._anchor.addClass(M)}handleEvent(t){switch(t.type){case"keydown":this._evtKeydown(t);break;case"click":this._evtClick(t);break;case"resize":this.onResize()}}_evtClick(t){!t.target||this._body.node.contains(t.target)||this._anchor.node.contains(t.target)||this.dispose()}_evtKeydown(t){switch(t.keyCode){case 27:t.stopPropagation(),t.preventDefault(),this.dispose()}}_setGeometry(){let t=0;const e=this._anchor.node.getBoundingClientRect(),n=this._body.node.getBoundingClientRect();"right"===this._align&&(t=-(n.width-e.width));const r=window.getComputedStyle(this._body.node);b.a.setGeometry({anchor:e,host:document.body,maxHeight:500,minHeight:20,node:this._body.node,offset:{horizontal:t},privilege:"forceAbove",style:r})}}var R=n(76),N=n(1453);Object(o.style)({boxShadow:"0px 4px 4px rgba(0, 0, 0, 0.25)"});const F=Object(o.style)({padding:"4px 12px",backgroundColor:"var(--jp-layout-color2)",boxShadow:"var(--jp-toolbar-box-shadow)",zIndex:2,fontSize:"var(--jp-ui-font-size1)"}),T=Object(o.style)({fontSize:"var(--jp-ui-font-size0)",lineHeight:"var(--jp-ui-font-size1)",marginTop:"4px",color:"var(--jp-ui-font-color0)"}),U={border:"none",borderRadius:"0px",position:"absolute",backgroundSize:"16px",backgroundRepeat:"no-repeat",backgroundPosition:"center",outline:"none",top:"0px",right:"0px"},O=Object(o.style)(U,{top:"4px",right:"8px",height:"24px",padding:"0px 12px",width:"12px"}),G=Object(o.style)(U,{backgroundColor:"var(--jp-brand-color1)",height:"100%",width:"100%",boxSizing:"border-box",padding:"4px 6px"}),H=Object(o.style)(U,{backgroundColor:"transparent",height:"100%",width:"100%",boxSizing:"border-box"}),j=Object(o.style)({overflow:"hidden",padding:"0px 8px",border:"1px solid var(--jp-border-color0)",backgroundColor:"var(--jp-input-active-background)",height:"22px"}),Y=Object(o.style)({border:"var(--jp-border-width) solid var(--md-blue-500)",boxShadow:"inset 0 0 4px var(--md-blue-300)"}),P=Object(o.style)({background:"transparent",width:"200px",height:"100%",border:"none",outline:"none",color:"var(--jp-ui-font-color0)",lineHeight:"28px"});class K extends i.a.Component{constructor(t){super(t),this._handleChange=t=>{this.setState({value:t.currentTarget.value})},this._handleSubmit=t=>{t.preventDefault();const e=parseInt(this._textInput.value,10);return!isNaN(e)&&isFinite(e)&&1<=e&&e<=this.props.maxLine&&this.props.handleSubmit(e),!1},this._handleFocus=()=>{this.setState({hasFocus:!0})},this._handleBlur=()=>{this.setState({hasFocus:!1})},this._textInput=null,this.state={value:"",hasFocus:!1}}componentDidMount(){this._textInput.focus()}render(){return i.a.createElement("div",{className:F},i.a.createElement("form",{name:"lineColumnForm",onSubmit:this._handleSubmit,noValidate:!0},i.a.createElement("div",{className:Object(o.classes)(j,"p-lineForm-wrapper",this.state.hasFocus?Y:void 0)},i.a.createElement("input",{type:"text",className:P,onChange:this._handleChange,onFocus:this._handleFocus,onBlur:this._handleBlur,value:this.state.value,ref:t=>{this._textInput=t}}),i.a.createElement("div",{className:O},i.a.createElement(N.a,{name:"line-form",className:G,center:!0}),i.a.createElement("input",{type:"submit",className:H,value:""}))),i.a.createElement("label",{className:T},"Go to line number between 1 and ",this.props.maxLine)))}}function z(t){return i.a.createElement(C,{onClick:t.handleClick,source:`Ln ${t.line}, Col ${t.column}`,title:"Go to line number…"})}class J extends R.d{constructor(){super(),this._popup=null,this.model=new J.Model,this.addClass(M)}render(){return null===this.model?null:i.a.createElement(z,{line:this.model.line,column:this.model.column,handleClick:()=>this._handleClick()})}_handleClick(){this._popup&&this._popup.dispose();const t=R.a.create(i.a.createElement(K,{handleSubmit:t=>this._handleSubmit(t),currentLine:this.model.line,maxLine:this.model.editor.lineCount}));this._popup=S({body:t,anchor:this,align:"right"})}_handleSubmit(t){this.model.editor.setCursorPosition({line:t-1,column:0}),this._popup.dispose(),this.model.editor.focus()}}(J||(J={})).Model=class extends R.c{constructor(){super(...arguments),this._onSelectionChanged=()=>{const t=this._getAllState(),e=this.editor.getCursorPosition();this._line=e.line+1,this._column=e.column+1,this._triggerChange(t,this._getAllState())},this._line=1,this._column=1,this._editor=null}get editor(){return this._editor}set editor(t){const e=this._editor;e&&e.model.selections.changed.disconnect(this._onSelectionChanged);const n=this._getAllState();if(this._editor=t,this._editor){this._editor.model.selections.changed.connect(this._onSelectionChanged);const t=this._editor.getCursorPosition();this._column=t.column+1,this._line=t.line+1}else this._column=1,this._line=1;this._triggerChange(n,this._getAllState())}get line(){return this._line}get column(){return this._column}_getAllState(){return[this._line,this._column]}_triggerChange(t,e){t[0]===e[0]&&t[1]===e[1]||this.stateChanged.emit(void 0)}};var q=n(17),V=n(9);function W(t){return i.a.createElement(C,{onClick:t.handleClick,source:`${t.kernelName} | ${q.Text.titleCase(t.status)}`,title:`Change kernel for ${t.activityName}`})}class X extends R.d{constructor(t){super(),this._handleClick=t.onClick,this.model=new X.Model,this.addClass(M)}render(){return null===this.model?null:i.a.createElement(W,{status:this.model.status,kernelName:this.model.kernelName,activityName:this.model.activityName,handleClick:this._handleClick})}}(X||(X={})).Model=class extends R.c{constructor(){super(...arguments),this._onKernelStatusChanged=(t,e)=>{this._kernelStatus=e,this.stateChanged.emit(void 0)},this._onKernelChanged=(t,e)=>{const n=this._getAllState(),{newValue:r}=e;null!==r?r.getSpec().then(t=>{this._kernelStatus=r.status,this._kernelName=t.display_name,this._triggerChange(n,this._getAllState())}).catch(t=>{throw t}):(this._kernelStatus="unknown",this._kernelName="unknown",this._triggerChange(n,this._getAllState()))},this._activityName="activity",this._kernelName="unknown",this._kernelStatus="unknown",this._session=null}get kernelName(){return this._kernelName}get status(){return this._kernelStatus}get activityName(){return this._activityName}set activityName(t){this._activityName!==t&&(this._activityName=t,this.stateChanged.emit(void 0))}get session(){return this._session}set session(t){const e=this._session;null!==e&&(e.statusChanged.disconnect(this._onKernelStatusChanged),e.kernelChanged.disconnect(this._onKernelChanged));const n=this._getAllState();this._session=t,null===this._session?(this._kernelStatus="unknown",this._kernelName="unknown"):(this._kernelStatus=this._session.status,this._kernelName=this._session.kernelDisplayName,this._session.statusChanged.connect(this._onKernelStatusChanged),this._session.kernelChanged.connect(this._onKernelChanged)),this._triggerChange(n,this._getAllState())}_getAllState(){return[this._kernelName,this._kernelStatus,this._activityName]}_triggerChange(t,e){V.JSONExt.deepEqual(t,e)&&this.stateChanged.emit(void 0)}};const Z=4;function $(t){return i.a.createElement(B,{spacing:Z,onClick:t.handleClick},i.a.createElement(B,{spacing:Z},i.a.createElement(C,{source:t.terminals}),i.a.createElement(N.a,{name:"terminal",left:"1px",top:"3px",kind:"statusBar"})),i.a.createElement(B,{spacing:Z},i.a.createElement(C,{source:t.kernels}),i.a.createElement(N.a,{name:"kernel",top:"2px",kind:"statusBar"})))}class tt extends R.d{constructor(t){super(),this._serviceManager=t.serviceManager,this._handleClick=t.onClick,this._serviceManager.sessions.runningChanged.connect(this._onKernelsRunningChanged,this),this._serviceManager.terminals.runningChanged.connect(this._onTerminalsRunningChanged,this),this.model=new tt.Model,this.addClass(M)}render(){return this.model?(this.title.caption=`${this.model.terminals} Terminals, ${this.model.kernels} Kernels`,i.a.createElement($,{kernels:this.model.kernels,terminals:this.model.terminals,handleClick:this._handleClick})):null}dispose(){super.dispose(),this._serviceManager.sessions.runningChanged.disconnect(this._onKernelsRunningChanged,this),this._serviceManager.terminals.runningChanged.disconnect(this._onTerminalsRunningChanged,this)}_onKernelsRunningChanged(t,e){this.model.kernels=e.length}_onTerminalsRunningChanged(t,e){this.model.terminals=e.length}}(tt||(tt={})).Model=class extends R.c{constructor(){super(...arguments),this._terminals=0,this._kernels=0}get kernels(){return this._kernels}set kernels(t){const e=this._kernels;this._kernels=t,e!==this._kernels&&this.stateChanged.emit(void 0)}get terminals(){return this._terminals}set terminals(t){const e=this._terminals;this._terminals=t,e!==this._terminals&&this.stateChanged.emit(void 0)}};var et,nt=n(60);class rt extends R.d{constructor(){super(),this.model=new rt.Model({refreshRate:5e3})}render(){if(!this.model)return null;let t;return t=null===this.model.memoryLimit?`Mem: ${this.model.currentMemory.toFixed(et.DECIMAL_PLACES)} ${this.model.units}`:`Mem: ${this.model.currentMemory.toFixed(et.DECIMAL_PLACES)} / ${this.model.memoryLimit.toFixed(et.DECIMAL_PLACES)} ${this.model.units}`,i.a.createElement(C,{title:"Current memory usage",source:t})}}(rt||(rt={})).Model=class extends R.c{constructor(t){super(),this._currentMemory=0,this._memoryLimit=null,this._metricsAvailable=!1,this._units="B",this._poll=new q.Poll({factory:()=>et.factory(),frequency:{interval:t.refreshRate,backoff:!0},name:"@jupyterlab/statusbar:MemoryUsage#metrics"}),this._poll.ticked.connect(t=>{const{payload:e,phase:n}=t.state;if("resolved"!==n){if("rejected"===n){const t=this._metricsAvailable;return this._metricsAvailable=!1,this._currentMemory=0,this._memoryLimit=null,this._units="B",void(t&&this.stateChanged.emit())}}else this._updateMetricsValues(e)})}get metricsAvailable(){return this._metricsAvailable}get currentMemory(){return this._currentMemory}get memoryLimit(){return this._memoryLimit}get units(){return this._units}dispose(){super.dispose(),this._poll.dispose()}_updateMetricsValues(t){const e=this._metricsAvailable,n=this._currentMemory,r=this._memoryLimit,i=this._units;if(null===t)this._metricsAvailable=!1,this._currentMemory=0,this._memoryLimit=null,this._units="B";else{const e=t.rss,n=t.limits.memory?t.limits.memory.rss:null,[r,i]=et.convertToLargestUnit(e);this._metricsAvailable=!0,this._currentMemory=r,this._units=i,this._memoryLimit=n?n/et.MEMORY_UNIT_LIMITS[i]:null}this._currentMemory===n&&this._units===i&&this._memoryLimit===r&&this._metricsAvailable===e||this.stateChanged.emit(void 0)}},function(t){t.DECIMAL_PLACES=2,t.MEMORY_UNIT_LIMITS={B:1,KB:1024,MB:1048576,GB:1073741824,TB:1099511627776,PB:0x4000000000000},t.convertToLargestUnit=function(e){return e<t.MEMORY_UNIT_LIMITS.KB?[e,"B"]:t.MEMORY_UNIT_LIMITS.KB===e||e<t.MEMORY_UNIT_LIMITS.MB?[e/t.MEMORY_UNIT_LIMITS.KB,"KB"]:t.MEMORY_UNIT_LIMITS.MB===e||e<t.MEMORY_UNIT_LIMITS.GB?[e/t.MEMORY_UNIT_LIMITS.MB,"MB"]:t.MEMORY_UNIT_LIMITS.GB===e||e<t.MEMORY_UNIT_LIMITS.TB?[e/t.MEMORY_UNIT_LIMITS.GB,"GB"]:t.MEMORY_UNIT_LIMITS.TB===e||e<t.MEMORY_UNIT_LIMITS.PB?[e/t.MEMORY_UNIT_LIMITS.TB,"TB"]:[e/t.MEMORY_UNIT_LIMITS.PB,"PB"]};const e=nt.ServerConnection.makeSettings(),n=q.URLExt.join(e.baseUrl,"metrics");t.factory=async function(){const t=nt.ServerConnection.makeRequest(n,{},e),r=await t;if(r.ok)try{return await r.json()}catch(t){throw t}return null}}(et||(et={}));var it;n(5),n(50);(it||(it={})).statusItemDefaults={align:"left",rank:0,isActive:()=>!0,activeStateChanged:void 0};new V.Token("@jupyterlab/statusbar:IStatusBar");n.d(e,"b",function(){return C}),n.d(e,"a",function(){return B}),n.d(e,"d",function(){return S}),n.d(e,"c",function(){return M})},function(t,e,n){"use strict";var r=n(30),i=n(267),o=n(272),a=n(273),s=n(105);function A(t){s.call(this,"digest"),this._hash=t}r(A,s),A.prototype._update=function(t){this._hash.update(t)},A.prototype._final=function(){return this._hash.digest()},t.exports=function(t){return"md5"===(t=t.toLowerCase())?new i:"rmd160"===t||"ripemd160"===t?new o:new A(a(t))}},function(t,e,n){(function(t){function n(t){return Object.prototype.toString.call(t)}e.isArray=function(t){return Array.isArray?Array.isArray(t):"[object Array]"===n(t)},e.isBoolean=function(t){return"boolean"==typeof t},e.isNull=function(t){return null===t},e.isNullOrUndefined=function(t){return null==t},e.isNumber=function(t){return"number"==typeof t},e.isString=function(t){return"string"==typeof t},e.isSymbol=function(t){return"symbol"==typeof t},e.isUndefined=function(t){return void 0===t},e.isRegExp=function(t){return"[object RegExp]"===n(t)},e.isObject=function(t){return"object"==typeof t&&null!==t},e.isDate=function(t){return"[object Date]"===n(t)},e.isError=function(t){return"[object Error]"===n(t)||t instanceof Error},e.isFunction=function(t){return"function"==typeof t},e.isPrimitive=function(t){return null===t||"boolean"==typeof t||"number"==typeof t||"string"==typeof t||"symbol"==typeof t||void 0===t},e.isBuffer=t.isBuffer}).call(this,n(42).Buffer)},function(t,e,n){(function(e){t.exports=function(t,n){for(var r=Math.min(t.length,n.length),i=new e(r),o=0;o<r;++o)i[o]=t[o]^n[o];return i}}).call(this,n(42).Buffer)},function(t,e,n){"use strict";var r=n(87),i=n(77);function o(){this.pending=null,this.pendingTotal=0,this.blockSize=this.constructor.blockSize,this.outSize=this.constructor.outSize,this.hmacStrength=this.constructor.hmacStrength,this.padLength=this.constructor.padLength/8,this.endian="big",this._delta8=this.blockSize/8,this._delta32=this.blockSize/32}e.BlockHash=o,o.prototype.update=function(t,e){if(t=r.toArray(t,e),this.pending?this.pending=this.pending.concat(t):this.pending=t,this.pendingTotal+=t.length,this.pending.length>=this._delta8){var n=(t=this.pending).length%this._delta8;this.pending=t.slice(t.length-n,t.length),0===this.pending.length&&(this.pending=null),t=r.join32(t,0,t.length-n,this.endian);for(var i=0;i<t.length;i+=this._delta32)this._update(t,i,i+this._delta32)}return this},o.prototype.digest=function(t){return this.update(this._pad()),i(null===this.pending),this._digest(t)},o.prototype._pad=function(){var t=this.pendingTotal,e=this._delta8,n=e-(t+this.padLength)%e,r=new Array(n+this.padLength);r[0]=128;for(var i=1;i<n;i++)r[i]=0;if(t<<=3,"big"===this.endian){for(var o=8;o<this.padLength;o++)r[i++]=0;r[i++]=0,r[i++]=0,r[i++]=0,r[i++]=0,r[i++]=t>>>24&255,r[i++]=t>>>16&255,r[i++]=t>>>8&255,r[i++]=255&t}else for(r[i++]=255&t,r[i++]=t>>>8&255,r[i++]=t>>>16&255,r[i++]=t>>>24&255,r[i++]=0,r[i++]=0,r[i++]=0,r[i++]=0,o=8;o<this.padLength;o++)r[i++]=0;return r}},function(t,e,n){var r=e;r.bignum=n(53),r.define=n(832).define,r.base=n(170),r.constants=n(423),r.decoders=n(837),r.encoders=n(839)},function(t,e,n){var r=e;r.Reporter=n(834).Reporter,r.DecoderBuffer=n(422).DecoderBuffer,r.EncoderBuffer=n(422).EncoderBuffer,r.Node=n(835)},function(t,e,n){(function(t){function n(t,e){for(var n=0,r=t.length-1;r>=0;r--){var i=t[r];"."===i?t.splice(r,1):".."===i?(t.splice(r,1),n++):n&&(t.splice(r,1),n--)}if(e)for(;n--;n)t.unshift("..");return t}function r(t,e){if(t.filter)return t.filter(e);for(var n=[],r=0;r<t.length;r++)e(t[r],r,t)&&n.push(t[r]);return n}e.resolve=function(){for(var e="",i=!1,o=arguments.length-1;o>=-1&&!i;o--){var a=o>=0?arguments[o]:t.cwd();if("string"!=typeof a)throw new TypeError("Arguments to path.resolve must be strings");a&&(e=a+"/"+e,i="/"===a.charAt(0))}return(i?"/":"")+(e=n(r(e.split("/"),function(t){return!!t}),!i).join("/"))||"."},e.normalize=function(t){var o=e.isAbsolute(t),a="/"===i(t,-1);return(t=n(r(t.split("/"),function(t){return!!t}),!o).join("/"))||o||(t="."),t&&a&&(t+="/"),(o?"/":"")+t},e.isAbsolute=function(t){return"/"===t.charAt(0)},e.join=function(){var t=Array.prototype.slice.call(arguments,0);return e.normalize(r(t,function(t,e){if("string"!=typeof t)throw new TypeError("Arguments to path.join must be strings");return t}).join("/"))},e.relative=function(t,n){function r(t){for(var e=0;e<t.length&&""===t[e];e++);for(var n=t.length-1;n>=0&&""===t[n];n--);return e>n?[]:t.slice(e,n-e+1)}t=e.resolve(t).substr(1),n=e.resolve(n).substr(1);for(var i=r(t.split("/")),o=r(n.split("/")),a=Math.min(i.length,o.length),s=a,A=0;A<a;A++)if(i[A]!==o[A]){s=A;break}var l=[];for(A=s;A<i.length;A++)l.push("..");return(l=l.concat(o.slice(s))).join("/")},e.sep="/",e.delimiter=":",e.dirname=function(t){if("string"!=typeof t&&(t+=""),0===t.length)return".";for(var e=t.charCodeAt(0),n=47===e,r=-1,i=!0,o=t.length-1;o>=1;--o)if(47===(e=t.charCodeAt(o))){if(!i){r=o;break}}else i=!1;return-1===r?n?"/":".":n&&1===r?"/":t.slice(0,r)},e.basename=function(t,e){var n=function(t){"string"!=typeof t&&(t+="");var e,n=0,r=-1,i=!0;for(e=t.length-1;e>=0;--e)if(47===t.charCodeAt(e)){if(!i){n=e+1;break}}else-1===r&&(i=!1,r=e+1);return-1===r?"":t.slice(n,r)}(t);return e&&n.substr(-1*e.length)===e&&(n=n.substr(0,n.length-e.length)),n},e.extname=function(t){"string"!=typeof t&&(t+="");for(var e=-1,n=0,r=-1,i=!0,o=0,a=t.length-1;a>=0;--a){var s=t.charCodeAt(a);if(47!==s)-1===r&&(i=!1,r=a+1),46===s?-1===e?e=a:1!==o&&(o=1):-1!==e&&(o=-1);else if(!i){n=a+1;break}}return-1===e||-1===r||0===o||1===o&&e===r-1&&e===n+1?"":t.slice(e,r)};var i="b"==="ab".substr(-1)?function(t,e,n){return t.substr(e,n)}:function(t,e,n){return e<0&&(e=t.length+e),t.substr(e,n)}}).call(this,n(58))},function(t,e){t.exports=function(t){return"string"!=typeof t?t:(/^['"].*['"]$/.test(t)&&(t=t.slice(1,-1)),/["'() \t\n]/.test(t)?'"'+t.replace(/"/g,'\\"').replace(/\n/g,"\\n")+'"':t)}},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(106),""),e.i(n(567),""),e.i(n(964),""),e.push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/* This file was auto-generated by ensurePackage() in @jupyterlab/buildutils */\n",""])},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=function(){this.sizeHint=0,this.minSize=0,this.maxSize=1/0,this.stretch=1,this.size=0,this.done=!1};e.BoxSizer=r,function(t){t.calc=function(t,e){var n=t.length;if(0===n)return e;for(var r=0,i=0,o=0,a=0,s=0,A=0;A<n;++A){var l=(I=t[A]).minSize,c=I.maxSize,u=I.sizeHint;I.done=!1,I.size=Math.max(l,Math.min(u,c)),o+=I.size,r+=l,i+=c,I.stretch>0&&(a+=I.stretch,s++)}if(e===o)return 0;if(e<=r){for(A=0;A<n;++A)(I=t[A]).size=I.minSize;return e-r}if(e>=i){for(A=0;A<n;++A)(I=t[A]).size=I.maxSize;return e-i}var g=n;if(e<o){for(var h=o-e;s>0&&h>.01;){var d=h,f=a;for(A=0;A<n;++A)if(!(I=t[A]).done&&0!==I.stretch){var p=I.stretch*d/f;I.size-p<=I.minSize?(h-=I.size-I.minSize,a-=I.stretch,I.size=I.minSize,I.done=!0,g--,s--):(h-=p,I.size-=p)}}for(;g>0&&h>.01;)for(p=h/g,A=0;A<n;++A)(I=t[A]).done||(I.size-p<=I.minSize?(h-=I.size-I.minSize,I.size=I.minSize,I.done=!0,g--):(h-=p,I.size-=p))}else{for(h=e-o;s>0&&h>.01;)for(d=h,f=a,A=0;A<n;++A)(I=t[A]).done||0===I.stretch||(p=I.stretch*d/f,I.size+p>=I.maxSize?(h-=I.maxSize-I.size,a-=I.stretch,I.size=I.maxSize,I.done=!0,g--,s--):(h-=p,I.size+=p));for(;g>0&&h>.01;)for(p=h/g,A=0;A<n;++A){var I;(I=t[A]).done||(I.size+p>=I.maxSize?(h-=I.maxSize-I.size,I.size=I.maxSize,I.done=!0,g--):(h-=p,I.size+=p))}}return 0},t.adjust=function(t,e,n){0!==t.length&&0!==n&&(n>0?function(t,e,n){for(var r=0,i=0;i<=e;++i){var o=t[i];r+=o.maxSize-o.size}for(var a=0,i=e+1,s=t.length;i<s;++i){var o=t[i];a+=o.size-o.minSize}for(var A=n=Math.min(n,r,a),i=e;i>=0&&A>0;--i){var o=t[i],l=o.maxSize-o.size;l>=A?(o.sizeHint=o.size+A,A=0):(o.sizeHint=o.size+l,A-=l)}for(var c=n,i=e+1,s=t.length;i<s&&c>0;++i){var o=t[i],l=o.size-o.minSize;l>=c?(o.sizeHint=o.size-c,c=0):(o.sizeHint=o.size-l,c-=l)}}(t,e,n):function(t,e,n){for(var r=0,i=e+1,o=t.length;i<o;++i){var a=t[i];r+=a.maxSize-a.size}for(var s=0,i=0;i<=e;++i){var a=t[i];s+=a.size-a.minSize}for(var A=n=Math.min(n,r,s),i=e+1,o=t.length;i<o&&A>0;++i){var a=t[i],l=a.maxSize-a.size;l>=A?(a.sizeHint=a.size+A,A=0):(a.sizeHint=a.size+l,A-=l)}for(var c=n,i=e;i>=0&&c>0;--i){var a=t[i],l=a.size-a.minSize;l>=c?(a.sizeHint=a.size-c,c=0):(a.sizeHint=a.size-l,c-=l)}}(t,e,-n))}}(e.BoxEngine||(e.BoxEngine={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o=n(5),a=n(25),s=n(107),A=n(59),l=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._widgets=[],e}return i(e,t),e.prototype.dispose=function(){for(;this._widgets.length>0;)this._widgets.pop().dispose();t.prototype.dispose.call(this)},Object.defineProperty(e.prototype,"widgets",{get:function(){return this._widgets},enumerable:!0,configurable:!0}),e.prototype.iter=function(){return o.iter(this._widgets)},e.prototype.addWidget=function(t){this.insertWidget(this._widgets.length,t)},e.prototype.insertWidget=function(t,e){e.parent=this.parent;var n=this._widgets.indexOf(e),r=Math.max(0,Math.min(t,this._widgets.length));if(-1===n)return o.ArrayExt.insert(this._widgets,r,e),void(this.parent&&this.attachWidget(r,e));r===this._widgets.length&&r--,n!==r&&(o.ArrayExt.move(this._widgets,n,r),this.parent&&this.moveWidget(n,r,e))},e.prototype.removeWidget=function(t){this.removeWidgetAt(this._widgets.indexOf(t))},e.prototype.removeWidgetAt=function(t){var e=o.ArrayExt.removeAt(this._widgets,t);e&&this.parent&&this.detachWidget(t,e)},e.prototype.init=function(){var e=this;t.prototype.init.call(this),o.each(this,function(t,n){e.attachWidget(n,t)})},e.prototype.attachWidget=function(t,e){var n=this.parent.node.children[t];this.parent.isAttached&&a.MessageLoop.sendMessage(e,A.Widget.Msg.BeforeAttach),this.parent.node.insertBefore(e.node,n),this.parent.isAttached&&a.MessageLoop.sendMessage(e,A.Widget.Msg.AfterAttach)},e.prototype.moveWidget=function(t,e,n){this.parent.isAttached&&a.MessageLoop.sendMessage(n,A.Widget.Msg.BeforeDetach),this.parent.node.removeChild(n.node),this.parent.isAttached&&a.MessageLoop.sendMessage(n,A.Widget.Msg.AfterDetach);var r=this.parent.node.children[e];this.parent.isAttached&&a.MessageLoop.sendMessage(n,A.Widget.Msg.BeforeAttach),this.parent.node.insertBefore(n.node,r),this.parent.isAttached&&a.MessageLoop.sendMessage(n,A.Widget.Msg.AfterAttach)},e.prototype.detachWidget=function(t,e){this.parent.isAttached&&a.MessageLoop.sendMessage(e,A.Widget.Msg.BeforeDetach),this.parent.node.removeChild(e.node),this.parent.isAttached&&a.MessageLoop.sendMessage(e,A.Widget.Msg.AfterDetach)},e}(s.Layout);e.PanelLayout=l},function(t,e){e.getArg=function(t,e,n){if(e in t)return t[e];if(3===arguments.length)return n;throw new Error('"'+e+'" is a required argument.')};var n=/^(?:([\w+\-.]+):)?\/\/(?:(\w+:\w+)@)?([\w.-]*)(?::(\d+))?(.*)$/,r=/^data:.+\,.+$/;function i(t){var e=t.match(n);return e?{scheme:e[1],auth:e[2],host:e[3],port:e[4],path:e[5]}:null}function o(t){var e="";return t.scheme&&(e+=t.scheme+":"),e+="//",t.auth&&(e+=t.auth+"@"),t.host&&(e+=t.host),t.port&&(e+=":"+t.port),t.path&&(e+=t.path),e}function a(t){var n=t,r=i(t);if(r){if(!r.path)return t;n=r.path}for(var a,s=e.isAbsolute(n),A=n.split(/\/+/),l=0,c=A.length-1;c>=0;c--)"."===(a=A[c])?A.splice(c,1):".."===a?l++:l>0&&(""===a?(A.splice(c+1,l),l=0):(A.splice(c,2),l--));return""===(n=A.join("/"))&&(n=s?"/":"."),r?(r.path=n,o(r)):n}function s(t,e){""===t&&(t="."),""===e&&(e=".");var n=i(e),s=i(t);if(s&&(t=s.path||"/"),n&&!n.scheme)return s&&(n.scheme=s.scheme),o(n);if(n||e.match(r))return e;if(s&&!s.host&&!s.path)return s.host=e,o(s);var A="/"===e.charAt(0)?e:a(t.replace(/\/+$/,"")+"/"+e);return s?(s.path=A,o(s)):A}e.urlParse=i,e.urlGenerate=o,e.normalize=a,e.join=s,e.isAbsolute=function(t){return"/"===t.charAt(0)||n.test(t)},e.relative=function(t,e){""===t&&(t="."),t=t.replace(/\/$/,"");for(var n=0;0!==e.indexOf(t+"/");){var r=t.lastIndexOf("/");if(r<0)return e;if((t=t.slice(0,r)).match(/^([^\/]+:\/)?\/*$/))return e;++n}return Array(n+1).join("../")+e.substr(t.length+1)};var A=!("__proto__"in Object.create(null));function l(t){return t}function c(t){if(!t)return!1;var e=t.length;if(e<9)return!1;if(95!==t.charCodeAt(e-1)||95!==t.charCodeAt(e-2)||111!==t.charCodeAt(e-3)||116!==t.charCodeAt(e-4)||111!==t.charCodeAt(e-5)||114!==t.charCodeAt(e-6)||112!==t.charCodeAt(e-7)||95!==t.charCodeAt(e-8)||95!==t.charCodeAt(e-9))return!1;for(var n=e-10;n>=0;n--)if(36!==t.charCodeAt(n))return!1;return!0}function u(t,e){return t===e?0:null===t?1:null===e?-1:t>e?1:-1}e.toSetString=A?l:function(t){return c(t)?"$"+t:t},e.fromSetString=A?l:function(t){return c(t)?t.slice(1):t},e.compareByOriginalPositions=function(t,e,n){var r=u(t.source,e.source);return 0!==r?r:0!=(r=t.originalLine-e.originalLine)?r:0!=(r=t.originalColumn-e.originalColumn)||n?r:0!=(r=t.generatedColumn-e.generatedColumn)?r:0!=(r=t.generatedLine-e.generatedLine)?r:u(t.name,e.name)},e.compareByGeneratedPositionsDeflated=function(t,e,n){var r=t.generatedLine-e.generatedLine;return 0!==r?r:0!=(r=t.generatedColumn-e.generatedColumn)||n?r:0!==(r=u(t.source,e.source))?r:0!=(r=t.originalLine-e.originalLine)?r:0!=(r=t.originalColumn-e.originalColumn)?r:u(t.name,e.name)},e.compareByGeneratedPositionsInflated=function(t,e){var n=t.generatedLine-e.generatedLine;return 0!==n?n:0!=(n=t.generatedColumn-e.generatedColumn)?n:0!==(n=u(t.source,e.source))?n:0!=(n=t.originalLine-e.originalLine)?n:0!=(n=t.originalColumn-e.originalColumn)?n:u(t.name,e.name)},e.parseSourceMapInput=function(t){return JSON.parse(t.replace(/^\)]}'[^\n]*\n/,""))},e.computeSourceURL=function(t,e,n){if(e=e||"",t&&("/"!==t[t.length-1]&&"/"!==e[0]&&(t+="/"),e=t+e),n){var r=i(n);if(!r)throw new Error("sourceMapURL could not be parsed");if(r.path){var A=r.path.lastIndexOf("/");A>=0&&(r.path=r.path.substring(0,A+1))}e=s(o(r),e)}return a(e)}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2);e.DISPLAYNAME_PREFIX="Blueprint3";var i=["active","alignText","containerRef","elementRef","fill","icon","inputRef","intent","inline","large","loading","leftIcon","minimal","onChildrenMount","onRemove","popoverProps","rightElement","rightIcon","round","small","text"];e.removeNonHTMLProps=function(t,e,n){return void 0===e&&(e=i),void 0===n&&(n=!1),n&&(e=e.concat(i)),e.reduce(function(t,e){return t.hasOwnProperty(e)&&delete t[e],t},r.__assign({},t))}},function(t,e){t.exports=function(t,e){return{enumerable:!(1&t),configurable:!(2&t),writable:!(4&t),value:e}}},function(t,e,n){var r=n(318);t.exports=function(t){return Object(r(t))}},function(t,e){t.exports={}},function(t,e,n){"use strict";e.__esModule=!0,e.dataIndexOf=o,e.valueMatcher=a,e.dataItem=function(t,e,n){var r=o(t,e,n);return-1!==r?t[r]:e},e.dataText=e.dataValue=void 0;var r=n(229),i=function(t,e){var n=t;return"function"==typeof e?n=e(t):null==t?n=t:"string"==typeof e&&"object"==typeof t&&e in t&&(n=t[e]),n};e.dataValue=i;function o(t,e,n){for(var r=-1,i=function(t){return a(e,t,n)};++r<t.length;){var o=t[r];if(o===e||i(o))return r}return-1}function a(t,e,n){return(0,r.isShallowEqual)(i(t,n),i(e,n))}e.dataText=function(t,e){var n=i(t,e);return null==n?"":n+""}},function(t,e,n){var r=n(344),i=n(1352),o=n(1353),a="[object Null]",s="[object Undefined]",A=r?r.toStringTag:void 0;t.exports=function(t){return null==t?void 0===t?s:a:A&&A in Object(t)?i(t):o(t)}},function(t,e){t.exports=function(t){return null!=t&&"object"==typeof t}},function(t,e,n){var r,i,o;i=[n(44)],void 0===(o="function"==typeof(r=function(t){return t.ui=t.ui||{},t.ui.version="1.12.1"})?r.apply(e,i):r)||(t.exports=o)},function(t,e,n){var r,i;!function(){var o={version:"3.5.17"},a=[].slice,s=function(t){return a.call(t)},A=this.document;function l(t){return t&&(t.ownerDocument||t.document||t).documentElement}function c(t){return t&&(t.ownerDocument&&t.ownerDocument.defaultView||t.document&&t||t.defaultView)}if(A)try{s(A.documentElement.childNodes)[0].nodeType}catch(t){s=function(t){for(var e=t.length,n=new Array(e);e--;)n[e]=t[e];return n}}if(Date.now||(Date.now=function(){return+new Date}),A)try{A.createElement("DIV").style.setProperty("opacity",0,"")}catch(t){var u=this.Element.prototype,g=u.setAttribute,h=u.setAttributeNS,d=this.CSSStyleDeclaration.prototype,f=d.setProperty;u.setAttribute=function(t,e){g.call(this,t,e+"")},u.setAttributeNS=function(t,e,n){h.call(this,t,e,n+"")},d.setProperty=function(t,e,n){f.call(this,t,e+"",n)}}function p(t,e){return t<e?-1:t>e?1:t>=e?0:NaN}function I(t){return null===t?NaN:+t}function C(t){return!isNaN(t)}function m(t){return{left:function(e,n,r,i){for(arguments.length<3&&(r=0),arguments.length<4&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)<0?r=o+1:i=o}return r},right:function(e,n,r,i){for(arguments.length<3&&(r=0),arguments.length<4&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)>0?i=o:r=o+1}return r}}}o.ascending=p,o.descending=function(t,e){return e<t?-1:e>t?1:e>=t?0:NaN},o.min=function(t,e){var n,r,i=-1,o=t.length;if(1===arguments.length){for(;++i<o;)if(null!=(r=t[i])&&r>=r){n=r;break}for(;++i<o;)null!=(r=t[i])&&n>r&&(n=r)}else{for(;++i<o;)if(null!=(r=e.call(t,t[i],i))&&r>=r){n=r;break}for(;++i<o;)null!=(r=e.call(t,t[i],i))&&n>r&&(n=r)}return n},o.max=function(t,e){var n,r,i=-1,o=t.length;if(1===arguments.length){for(;++i<o;)if(null!=(r=t[i])&&r>=r){n=r;break}for(;++i<o;)null!=(r=t[i])&&r>n&&(n=r)}else{for(;++i<o;)if(null!=(r=e.call(t,t[i],i))&&r>=r){n=r;break}for(;++i<o;)null!=(r=e.call(t,t[i],i))&&r>n&&(n=r)}return n},o.extent=function(t,e){var n,r,i,o=-1,a=t.length;if(1===arguments.length){for(;++o<a;)if(null!=(r=t[o])&&r>=r){n=i=r;break}for(;++o<a;)null!=(r=t[o])&&(n>r&&(n=r),i<r&&(i=r))}else{for(;++o<a;)if(null!=(r=e.call(t,t[o],o))&&r>=r){n=i=r;break}for(;++o<a;)null!=(r=e.call(t,t[o],o))&&(n>r&&(n=r),i<r&&(i=r))}return[n,i]},o.sum=function(t,e){var n,r=0,i=t.length,o=-1;if(1===arguments.length)for(;++o<i;)C(n=+t[o])&&(r+=n);else for(;++o<i;)C(n=+e.call(t,t[o],o))&&(r+=n);return r},o.mean=function(t,e){var n,r=0,i=t.length,o=-1,a=i;if(1===arguments.length)for(;++o<i;)C(n=I(t[o]))?r+=n:--a;else for(;++o<i;)C(n=I(e.call(t,t[o],o)))?r+=n:--a;if(a)return r/a},o.quantile=function(t,e){var n=(t.length-1)*e+1,r=Math.floor(n),i=+t[r-1],o=n-r;return o?i+o*(t[r]-i):i},o.median=function(t,e){var n,r=[],i=t.length,a=-1;if(1===arguments.length)for(;++a<i;)C(n=I(t[a]))&&r.push(n);else for(;++a<i;)C(n=I(e.call(t,t[a],a)))&&r.push(n);if(r.length)return o.quantile(r.sort(p),.5)},o.variance=function(t,e){var n,r,i=t.length,o=0,a=0,s=-1,A=0;if(1===arguments.length)for(;++s<i;)C(n=I(t[s]))&&(a+=(r=n-o)*(n-(o+=r/++A)));else for(;++s<i;)C(n=I(e.call(t,t[s],s)))&&(a+=(r=n-o)*(n-(o+=r/++A)));if(A>1)return a/(A-1)},o.deviation=function(){var t=o.variance.apply(this,arguments);return t?Math.sqrt(t):t};var y=m(p);function E(t){return t.length}o.bisectLeft=y.left,o.bisect=o.bisectRight=y.right,o.bisector=function(t){return m(1===t.length?function(e,n){return p(t(e),n)}:t)},o.shuffle=function(t,e,n){(o=arguments.length)<3&&(n=t.length,o<2&&(e=0));for(var r,i,o=n-e;o;)i=Math.random()*o--|0,r=t[o+e],t[o+e]=t[i+e],t[i+e]=r;return t},o.permute=function(t,e){for(var n=e.length,r=new Array(n);n--;)r[n]=t[e[n]];return r},o.pairs=function(t){for(var e=0,n=t.length-1,r=t[0],i=new Array(n<0?0:n);e<n;)i[e]=[r,r=t[++e]];return i},o.transpose=function(t){if(!(i=t.length))return[];for(var e=-1,n=o.min(t,E),r=new Array(n);++e<n;)for(var i,a=-1,s=r[e]=new Array(i);++a<i;)s[a]=t[a][e];return r},o.zip=function(){return o.transpose(arguments)},o.keys=function(t){var e=[];for(var n in t)e.push(n);return e},o.values=function(t){var e=[];for(var n in t)e.push(t[n]);return e},o.entries=function(t){var e=[];for(var n in t)e.push({key:n,value:t[n]});return e},o.merge=function(t){for(var e,n,r,i=t.length,o=-1,a=0;++o<i;)a+=t[o].length;for(n=new Array(a);--i>=0;)for(e=(r=t[i]).length;--e>=0;)n[--a]=r[e];return n};var v=Math.abs;function B(t){for(var e=1;t*e%1;)e*=10;return e}function b(t,e){for(var n in e)Object.defineProperty(t.prototype,n,{value:e[n],enumerable:!1})}function Q(){this._=Object.create(null)}o.range=function(t,e,n){if(arguments.length<3&&(n=1,arguments.length<2&&(e=t,t=0)),(e-t)/n==1/0)throw new Error("infinite range");var r,i=[],o=B(v(n)),a=-1;if(t*=o,e*=o,(n*=o)<0)for(;(r=t+n*++a)>e;)i.push(r/o);else for(;(r=t+n*++a)<e;)i.push(r/o);return i},o.map=function(t,e){var n=new Q;if(t instanceof Q)t.forEach(function(t,e){n.set(t,e)});else if(Array.isArray(t)){var r,i=-1,o=t.length;if(1===arguments.length)for(;++i<o;)n.set(i,t[i]);else for(;++i<o;)n.set(e.call(t,r=t[i],i),r)}else for(var a in t)n.set(a,t[a]);return n};var w="__proto__",x="\0";function _(t){return(t+="")===w||t[0]===x?x+t:t}function D(t){return(t+="")[0]===x?t.slice(1):t}function M(t){return _(t)in this._}function k(t){return(t=_(t))in this._&&delete this._[t]}function S(){var t=[];for(var e in this._)t.push(D(e));return t}function L(){var t=0;for(var e in this._)++t;return t}function R(){for(var t in this._)return!1;return!0}function N(){this._=Object.create(null)}function F(t){return t}function T(t,e,n){return function(){var r=n.apply(e,arguments);return r===e?t:r}}function U(t,e){if(e in t)return e;e=e.charAt(0).toUpperCase()+e.slice(1);for(var n=0,r=O.length;n<r;++n){var i=O[n]+e;if(i in t)return i}}b(Q,{has:M,get:function(t){return this._[_(t)]},set:function(t,e){return this._[_(t)]=e},remove:k,keys:S,values:function(){var t=[];for(var e in this._)t.push(this._[e]);return t},entries:function(){var t=[];for(var e in this._)t.push({key:D(e),value:this._[e]});return t},size:L,empty:R,forEach:function(t){for(var e in this._)t.call(this,D(e),this._[e])}}),o.nest=function(){var t,e,n={},r=[],i=[];function a(i,o,s){if(s>=r.length)return e?e.call(n,o):t?o.sort(t):o;for(var A,l,c,u,g=-1,h=o.length,d=r[s++],f=new Q;++g<h;)(u=f.get(A=d(l=o[g])))?u.push(l):f.set(A,[l]);return i?(l=i(),c=function(t,e){l.set(t,a(i,e,s))}):(l={},c=function(t,e){l[t]=a(i,e,s)}),f.forEach(c),l}return n.map=function(t,e){return a(e,t,0)},n.entries=function(t){return function t(e,n){if(n>=r.length)return e;var o=[],a=i[n++];return e.forEach(function(e,r){o.push({key:e,values:t(r,n)})}),a?o.sort(function(t,e){return a(t.key,e.key)}):o}(a(o.map,t,0),0)},n.key=function(t){return r.push(t),n},n.sortKeys=function(t){return i[r.length-1]=t,n},n.sortValues=function(e){return t=e,n},n.rollup=function(t){return e=t,n},n},o.set=function(t){var e=new N;if(t)for(var n=0,r=t.length;n<r;++n)e.add(t[n]);return e},b(N,{has:M,add:function(t){return this._[_(t+="")]=!0,t},remove:k,values:S,size:L,empty:R,forEach:function(t){for(var e in this._)t.call(this,D(e))}}),o.behavior={},o.rebind=function(t,e){for(var n,r=1,i=arguments.length;++r<i;)t[n=arguments[r]]=T(t,e,e[n]);return t};var O=["webkit","ms","moz","Moz","o","O"];function G(){}function H(){}function j(t){var e=[],n=new Q;function r(){for(var n,r=e,i=-1,o=r.length;++i<o;)(n=r[i].on)&&n.apply(this,arguments);return t}return r.on=function(r,i){var o,a=n.get(r);return arguments.length<2?a&&a.on:(a&&(a.on=null,e=e.slice(0,o=e.indexOf(a)).concat(e.slice(o+1)),n.remove(r)),i&&e.push(n.set(r,{on:i})),t)},r}function Y(){o.event.preventDefault()}function P(){for(var t,e=o.event;t=e.sourceEvent;)e=t;return e}function K(t){for(var e=new H,n=0,r=arguments.length;++n<r;)e[arguments[n]]=j(e);return e.of=function(n,r){return function(i){try{var a=i.sourceEvent=o.event;i.target=t,o.event=i,e[i.type].apply(n,r)}finally{o.event=a}}},e}o.dispatch=function(){for(var t=new H,e=-1,n=arguments.length;++e<n;)t[arguments[e]]=j(t);return t},H.prototype.on=function(t,e){var n=t.indexOf("."),r="";if(n>=0&&(r=t.slice(n+1),t=t.slice(0,n)),t)return arguments.length<2?this[t].on(r):this[t].on(r,e);if(2===arguments.length){if(null==e)for(t in this)this.hasOwnProperty(t)&&this[t].on(r,null);return this}},o.event=null,o.requote=function(t){return t.replace(z,"\\$&")};var z=/[\\\^\$\*\+\?\|\[\]\(\)\.\{\}]/g,J={}.__proto__?function(t,e){t.__proto__=e}:function(t,e){for(var n in e)t[n]=e[n]};function q(t){return J(t,Z),t}var V=function(t,e){return e.querySelector(t)},W=function(t,e){return e.querySelectorAll(t)},X=function(t,e){var n=t.matches||t[U(t,"matchesSelector")];return(X=function(t,e){return n.call(t,e)})(t,e)};"function"==typeof Sizzle&&(V=function(t,e){return Sizzle(t,e)[0]||null},W=Sizzle,X=Sizzle.matchesSelector),o.selection=function(){return o.select(A.documentElement)};var Z=o.selection.prototype=[];function $(t){return"function"==typeof t?t:function(){return V(t,this)}}function tt(t){return"function"==typeof t?t:function(){return W(t,this)}}Z.select=function(t){var e,n,r,i,o=[];t=$(t);for(var a=-1,s=this.length;++a<s;){o.push(e=[]),e.parentNode=(r=this[a]).parentNode;for(var A=-1,l=r.length;++A<l;)(i=r[A])?(e.push(n=t.call(i,i.__data__,A,a)),n&&"__data__"in i&&(n.__data__=i.__data__)):e.push(null)}return q(o)},Z.selectAll=function(t){var e,n,r=[];t=tt(t);for(var i=-1,o=this.length;++i<o;)for(var a=this[i],A=-1,l=a.length;++A<l;)(n=a[A])&&(r.push(e=s(t.call(n,n.__data__,A,i))),e.parentNode=n);return q(r)};var et="http://www.w3.org/1999/xhtml",nt={svg:"http://www.w3.org/2000/svg",xhtml:et,xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"};function rt(t,e){return t=o.ns.qualify(t),null==e?t.local?function(){this.removeAttributeNS(t.space,t.local)}:function(){this.removeAttribute(t)}:"function"==typeof e?t.local?function(){var n=e.apply(this,arguments);null==n?this.removeAttributeNS(t.space,t.local):this.setAttributeNS(t.space,t.local,n)}:function(){var n=e.apply(this,arguments);null==n?this.removeAttribute(t):this.setAttribute(t,n)}:t.local?function(){this.setAttributeNS(t.space,t.local,e)}:function(){this.setAttribute(t,e)}}function it(t){return t.trim().replace(/\s+/g," ")}function ot(t){return new RegExp("(?:^|\\s+)"+o.requote(t)+"(?:\\s+|$)","g")}function at(t){return(t+"").trim().split(/^|\s+/)}function st(t,e){var n=(t=at(t).map(At)).length;return"function"==typeof e?function(){for(var r=-1,i=e.apply(this,arguments);++r<n;)t[r](this,i)}:function(){for(var r=-1;++r<n;)t[r](this,e)}}function At(t){var e=ot(t);return function(n,r){if(i=n.classList)return r?i.add(t):i.remove(t);var i=n.getAttribute("class")||"";r?(e.lastIndex=0,e.test(i)||n.setAttribute("class",it(i+" "+t))):n.setAttribute("class",it(i.replace(e," ")))}}function lt(t,e,n){return null==e?function(){this.style.removeProperty(t)}:"function"==typeof e?function(){var r=e.apply(this,arguments);null==r?this.style.removeProperty(t):this.style.setProperty(t,r,n)}:function(){this.style.setProperty(t,e,n)}}function ct(t,e){return null==e?function(){delete this[t]}:"function"==typeof e?function(){var n=e.apply(this,arguments);null==n?delete this[t]:this[t]=n}:function(){this[t]=e}}function ut(t){return"function"==typeof t?t:(t=o.ns.qualify(t)).local?function(){return this.ownerDocument.createElementNS(t.space,t.local)}:function(){var e=this.ownerDocument,n=this.namespaceURI;return n===et&&e.documentElement.namespaceURI===et?e.createElement(t):e.createElementNS(n,t)}}function gt(){var t=this.parentNode;t&&t.removeChild(this)}function ht(t){return{__data__:t}}function dt(t){return function(){return X(this,t)}}function ft(t){return arguments.length||(t=p),function(e,n){return e&&n?t(e.__data__,n.__data__):!e-!n}}function pt(t,e){for(var n=0,r=t.length;n<r;n++)for(var i,o=t[n],a=0,s=o.length;a<s;a++)(i=o[a])&&e(i,a,n);return t}function It(t){return J(t,Ct),t}o.ns={prefix:nt,qualify:function(t){var e=t.indexOf(":"),n=t;return e>=0&&"xmlns"!==(n=t.slice(0,e))&&(t=t.slice(e+1)),nt.hasOwnProperty(n)?{space:nt[n],local:t}:t}},Z.attr=function(t,e){if(arguments.length<2){if("string"==typeof t){var n=this.node();return(t=o.ns.qualify(t)).local?n.getAttributeNS(t.space,t.local):n.getAttribute(t)}for(e in t)this.each(rt(e,t[e]));return this}return this.each(rt(t,e))},Z.classed=function(t,e){if(arguments.length<2){if("string"==typeof t){var n=this.node(),r=(t=at(t)).length,i=-1;if(e=n.classList){for(;++i<r;)if(!e.contains(t[i]))return!1}else for(e=n.getAttribute("class");++i<r;)if(!ot(t[i]).test(e))return!1;return!0}for(e in t)this.each(st(e,t[e]));return this}return this.each(st(t,e))},Z.style=function(t,e,n){var r=arguments.length;if(r<3){if("string"!=typeof t){for(n in r<2&&(e=""),t)this.each(lt(n,t[n],e));return this}if(r<2){var i=this.node();return c(i).getComputedStyle(i,null).getPropertyValue(t)}n=""}return this.each(lt(t,e,n))},Z.property=function(t,e){if(arguments.length<2){if("string"==typeof t)return this.node()[t];for(e in t)this.each(ct(e,t[e]));return this}return this.each(ct(t,e))},Z.text=function(t){return arguments.length?this.each("function"==typeof t?function(){var e=t.apply(this,arguments);this.textContent=null==e?"":e}:null==t?function(){this.textContent=""}:function(){this.textContent=t}):this.node().textContent},Z.html=function(t){return arguments.length?this.each("function"==typeof t?function(){var e=t.apply(this,arguments);this.innerHTML=null==e?"":e}:null==t?function(){this.innerHTML=""}:function(){this.innerHTML=t}):this.node().innerHTML},Z.append=function(t){return t=ut(t),this.select(function(){return this.appendChild(t.apply(this,arguments))})},Z.insert=function(t,e){return t=ut(t),e=$(e),this.select(function(){return this.insertBefore(t.apply(this,arguments),e.apply(this,arguments)||null)})},Z.remove=function(){return this.each(gt)},Z.data=function(t,e){var n,r,i=-1,o=this.length;if(!arguments.length){for(t=new Array(o=(n=this[0]).length);++i<o;)(r=n[i])&&(t[i]=r.__data__);return t}function a(t,n){var r,i,o,a=t.length,c=n.length,u=Math.min(a,c),g=new Array(c),h=new Array(c),d=new Array(a);if(e){var f,p=new Q,I=new Array(a);for(r=-1;++r<a;)(i=t[r])&&(p.has(f=e.call(i,i.__data__,r))?d[r]=i:p.set(f,i),I[r]=f);for(r=-1;++r<c;)(i=p.get(f=e.call(n,o=n[r],r)))?!0!==i&&(g[r]=i,i.__data__=o):h[r]=ht(o),p.set(f,!0);for(r=-1;++r<a;)r in I&&!0!==p.get(I[r])&&(d[r]=t[r])}else{for(r=-1;++r<u;)i=t[r],o=n[r],i?(i.__data__=o,g[r]=i):h[r]=ht(o);for(;r<c;++r)h[r]=ht(n[r]);for(;r<a;++r)d[r]=t[r]}h.update=g,h.parentNode=g.parentNode=d.parentNode=t.parentNode,s.push(h),A.push(g),l.push(d)}var s=It([]),A=q([]),l=q([]);if("function"==typeof t)for(;++i<o;)a(n=this[i],t.call(n,n.parentNode.__data__,i));else for(;++i<o;)a(n=this[i],t);return A.enter=function(){return s},A.exit=function(){return l},A},Z.datum=function(t){return arguments.length?this.property("__data__",t):this.property("__data__")},Z.filter=function(t){var e,n,r,i=[];"function"!=typeof t&&(t=dt(t));for(var o=0,a=this.length;o<a;o++){i.push(e=[]),e.parentNode=(n=this[o]).parentNode;for(var s=0,A=n.length;s<A;s++)(r=n[s])&&t.call(r,r.__data__,s,o)&&e.push(r)}return q(i)},Z.order=function(){for(var t=-1,e=this.length;++t<e;)for(var n,r=this[t],i=r.length-1,o=r[i];--i>=0;)(n=r[i])&&(o&&o!==n.nextSibling&&o.parentNode.insertBefore(n,o),o=n);return this},Z.sort=function(t){t=ft.apply(this,arguments);for(var e=-1,n=this.length;++e<n;)this[e].sort(t);return this.order()},Z.each=function(t){return pt(this,function(e,n,r){t.call(e,e.__data__,n,r)})},Z.call=function(t){var e=s(arguments);return t.apply(e[0]=this,e),this},Z.empty=function(){return!this.node()},Z.node=function(){for(var t=0,e=this.length;t<e;t++)for(var n=this[t],r=0,i=n.length;r<i;r++){var o=n[r];if(o)return o}return null},Z.size=function(){var t=0;return pt(this,function(){++t}),t};var Ct=[];function mt(t){var e,n;return function(r,i,o){var a,s=t[o].update,A=s.length;for(o!=n&&(n=o,e=0),i>=e&&(e=i+1);!(a=s[e])&&++e<A;);return a}}function yt(t,e,n){var r="__on"+t,i=t.indexOf("."),a=vt;i>0&&(t=t.slice(0,i));var A=Et.get(t);function l(){var e=this[r];e&&(this.removeEventListener(t,e,e.$),delete this[r])}return A&&(t=A,a=Bt),i?e?function(){var i=a(e,s(arguments));l.call(this),this.addEventListener(t,this[r]=i,i.$=n),i._=e}:l:e?G:function(){var e,n=new RegExp("^__on([^.]+)"+o.requote(t)+"$");for(var r in this)if(e=r.match(n)){var i=this[r];this.removeEventListener(e[1],i,i.$),delete this[r]}}}o.selection.enter=It,o.selection.enter.prototype=Ct,Ct.append=Z.append,Ct.empty=Z.empty,Ct.node=Z.node,Ct.call=Z.call,Ct.size=Z.size,Ct.select=function(t){for(var e,n,r,i,o,a=[],s=-1,A=this.length;++s<A;){r=(i=this[s]).update,a.push(e=[]),e.parentNode=i.parentNode;for(var l=-1,c=i.length;++l<c;)(o=i[l])?(e.push(r[l]=n=t.call(i.parentNode,o.__data__,l,s)),n.__data__=o.__data__):e.push(null)}return q(a)},Ct.insert=function(t,e){return arguments.length<2&&(e=mt(this)),Z.insert.call(this,t,e)},o.select=function(t){var e;return"string"==typeof t?(e=[V(t,A)]).parentNode=A.documentElement:(e=[t]).parentNode=l(t),q([e])},o.selectAll=function(t){var e;return"string"==typeof t?(e=s(W(t,A))).parentNode=A.documentElement:(e=s(t)).parentNode=null,q([e])},Z.on=function(t,e,n){var r=arguments.length;if(r<3){if("string"!=typeof t){for(n in r<2&&(e=!1),t)this.each(yt(n,t[n],e));return this}if(r<2)return(r=this.node()["__on"+t])&&r._;n=!1}return this.each(yt(t,e,n))};var Et=o.map({mouseenter:"mouseover",mouseleave:"mouseout"});function vt(t,e){return function(n){var r=o.event;o.event=n,e[0]=this.__data__;try{t.apply(this,e)}finally{o.event=r}}}function Bt(t,e){var n=vt(t,e);return function(t){var e=t.relatedTarget;e&&(e===this||8&e.compareDocumentPosition(this))||n.call(this,t)}}A&&Et.forEach(function(t){"on"+t in A&&Et.remove(t)});var bt,Qt=0;function wt(t){var e=".dragsuppress-"+ ++Qt,n="click"+e,r=o.select(c(t)).on("touchmove"+e,Y).on("dragstart"+e,Y).on("selectstart"+e,Y);if(null==bt&&(bt=!("onselectstart"in t)&&U(t.style,"userSelect")),bt){var i=l(t).style,a=i[bt];i[bt]="none"}return function(t){if(r.on(e,null),bt&&(i[bt]=a),t){var o=function(){r.on(n,null)};r.on(n,function(){Y(),o()},!0),setTimeout(o,0)}}}o.mouse=function(t){return _t(t,P())};var xt=this.navigator&&/WebKit/.test(this.navigator.userAgent)?-1:0;function _t(t,e){e.changedTouches&&(e=e.changedTouches[0]);var n=t.ownerSVGElement||t;if(n.createSVGPoint){var r=n.createSVGPoint();if(xt<0){var i=c(t);if(i.scrollX||i.scrollY){var a=(n=o.select("body").append("svg").style({position:"absolute",top:0,left:0,margin:0,padding:0,border:"none"},"important"))[0][0].getScreenCTM();xt=!(a.f||a.e),n.remove()}}return xt?(r.x=e.pageX,r.y=e.pageY):(r.x=e.clientX,r.y=e.clientY),[(r=r.matrixTransform(t.getScreenCTM().inverse())).x,r.y]}var s=t.getBoundingClientRect();return[e.clientX-s.left-t.clientLeft,e.clientY-s.top-t.clientTop]}function Dt(){return o.event.changedTouches[0].identifier}o.touch=function(t,e,n){if(arguments.length<3&&(n=e,e=P().changedTouches),e)for(var r,i=0,o=e.length;i<o;++i)if((r=e[i]).identifier===n)return _t(t,r)},o.behavior.drag=function(){var t=K(i,"drag","dragstart","dragend"),e=null,n=a(G,o.mouse,c,"mousemove","mouseup"),r=a(Dt,o.touch,F,"touchmove","touchend");function i(){this.on("mousedown.drag",n).on("touchstart.drag",r)}function a(n,r,i,a,s){return function(){var A,l=this,c=o.event.target.correspondingElement||o.event.target,u=l.parentNode,g=t.of(l,arguments),h=0,d=n(),f=".drag"+(null==d?"":"-"+d),p=o.select(i(c)).on(a+f,m).on(s+f,y),I=wt(c),C=r(u,d);function m(){var t,e,n=r(u,d);n&&(t=n[0]-C[0],e=n[1]-C[1],h|=t|e,C=n,g({type:"drag",x:n[0]+A[0],y:n[1]+A[1],dx:t,dy:e}))}function y(){r(u,d)&&(p.on(a+f,null).on(s+f,null),I(h),g({type:"dragend"}))}A=e?[(A=e.apply(l,arguments)).x-C[0],A.y-C[1]]:[0,0],g({type:"dragstart"})}}return i.origin=function(t){return arguments.length?(e=t,i):e},o.rebind(i,t,"on")},o.touches=function(t,e){return arguments.length<2&&(e=P().touches),e?s(e).map(function(e){var n=_t(t,e);return n.identifier=e.identifier,n}):[]};var Mt=1e-6,kt=Mt*Mt,St=Math.PI,Lt=2*St,Rt=Lt-Mt,Nt=St/2,Ft=St/180,Tt=180/St;function Ut(t){return t>0?1:t<0?-1:0}function Ot(t,e,n){return(e[0]-t[0])*(n[1]-t[1])-(e[1]-t[1])*(n[0]-t[0])}function Gt(t){return t>1?0:t<-1?St:Math.acos(t)}function Ht(t){return t>1?Nt:t<-1?-Nt:Math.asin(t)}function jt(t){return((t=Math.exp(t))+1/t)/2}function Yt(t){return(t=Math.sin(t/2))*t}var Pt=Math.SQRT2;o.interpolateZoom=function(t,e){var n,r,i=t[0],o=t[1],a=t[2],s=e[0],A=e[1],l=e[2],c=s-i,u=A-o,g=c*c+u*u;if(g<kt)r=Math.log(l/a)/Pt,n=function(t){return[i+t*c,o+t*u,a*Math.exp(Pt*t*r)]};else{var h=Math.sqrt(g),d=(l*l-a*a+4*g)/(2*a*2*h),f=(l*l-a*a-4*g)/(2*l*2*h),p=Math.log(Math.sqrt(d*d+1)-d),I=Math.log(Math.sqrt(f*f+1)-f);r=(I-p)/Pt,n=function(t){var e,n=t*r,s=jt(p),A=a/(2*h)*(s*(e=Pt*n+p,((e=Math.exp(2*e))-1)/(e+1))-function(t){return((t=Math.exp(t))-1/t)/2}(p));return[i+A*c,o+A*u,a*s/jt(Pt*n+p)]}}return n.duration=1e3*r,n},o.behavior.zoom=function(){var t,e,n,r,i,a,s,l,u,g={x:0,y:0,k:1},h=[960,500],d=Jt,f=250,p=0,I="mousedown.zoom",C="mousemove.zoom",m="mouseup.zoom",y="touchstart.zoom",E=K(v,"zoomstart","zoom","zoomend");function v(t){t.on(I,k).on(zt+".zoom",L).on("dblclick.zoom",R).on(y,S)}function B(t){return[(t[0]-g.x)/g.k,(t[1]-g.y)/g.k]}function b(t){g.k=Math.max(d[0],Math.min(d[1],t))}function Q(t,e){e=function(t){return[t[0]*g.k+g.x,t[1]*g.k+g.y]}(e),g.x+=t[0]-e[0],g.y+=t[1]-e[1]}function w(t,n,r,i){t.__chart__={x:g.x,y:g.y,k:g.k},b(Math.pow(2,i)),Q(e=n,r),t=o.select(t),f>0&&(t=t.transition().duration(f)),t.call(v.event)}function x(){s&&s.domain(a.range().map(function(t){return(t-g.x)/g.k}).map(a.invert)),u&&u.domain(l.range().map(function(t){return(t-g.y)/g.k}).map(l.invert))}function _(t){p++||t({type:"zoomstart"})}function D(t){x(),t({type:"zoom",scale:g.k,translate:[g.x,g.y]})}function M(t){--p||(t({type:"zoomend"}),e=null)}function k(){var t=this,e=E.of(t,arguments),n=0,r=o.select(c(t)).on(C,function(){n=1,Q(o.mouse(t),i),D(e)}).on(m,function(){r.on(C,null).on(m,null),a(n),M(e)}),i=B(o.mouse(t)),a=wt(t);ws.call(t),_(e)}function S(){var t,e=this,n=E.of(e,arguments),r={},a=0,s=".zoom-"+o.event.changedTouches[0].identifier,A="touchmove"+s,l="touchend"+s,c=[],u=o.select(e),h=wt(e);function d(){var n=o.touches(e);return t=g.k,n.forEach(function(t){t.identifier in r&&(r[t.identifier]=B(t))}),n}function f(){var t=o.event.target;o.select(t).on(A,p).on(l,C),c.push(t);for(var n=o.event.changedTouches,s=0,u=n.length;s<u;++s)r[n[s].identifier]=null;var h=d(),f=Date.now();if(1===h.length){if(f-i<500){var I=h[0];w(e,I,r[I.identifier],Math.floor(Math.log(g.k)/Math.LN2)+1),Y()}i=f}else if(h.length>1){I=h[0];var m=h[1],y=I[0]-m[0],E=I[1]-m[1];a=y*y+E*E}}function p(){var s,A,l,c,u=o.touches(e);ws.call(e);for(var g=0,h=u.length;g<h;++g,c=null)if(l=u[g],c=r[l.identifier]){if(A)break;s=l,A=c}if(c){var d=(d=l[0]-s[0])*d+(d=l[1]-s[1])*d,f=a&&Math.sqrt(d/a);s=[(s[0]+l[0])/2,(s[1]+l[1])/2],A=[(A[0]+c[0])/2,(A[1]+c[1])/2],b(f*t)}i=null,Q(s,A),D(n)}function C(){if(o.event.touches.length){for(var t=o.event.changedTouches,e=0,i=t.length;e<i;++e)delete r[t[e].identifier];for(var a in r)return void d()}o.selectAll(c).on(s,null),u.on(I,k).on(y,S),h(),M(n)}f(),_(n),u.on(I,null).on(y,f)}function L(){var i=E.of(this,arguments);r?clearTimeout(r):(ws.call(this),t=B(e=n||o.mouse(this)),_(i)),r=setTimeout(function(){r=null,M(i)},50),Y(),b(Math.pow(2,.002*Kt())*g.k),Q(e,t),D(i)}function R(){var t=o.mouse(this),e=Math.log(g.k)/Math.LN2;w(this,t,B(t),o.event.shiftKey?Math.ceil(e)-1:Math.floor(e)+1)}return zt||(zt="onwheel"in A?(Kt=function(){return-o.event.deltaY*(o.event.deltaMode?120:1)},"wheel"):"onmousewheel"in A?(Kt=function(){return o.event.wheelDelta},"mousewheel"):(Kt=function(){return-o.event.detail},"MozMousePixelScroll")),v.event=function(t){t.each(function(){var t=E.of(this,arguments),n=g;Ds?o.select(this).transition().each("start.zoom",function(){g=this.__chart__||{x:0,y:0,k:1},_(t)}).tween("zoom:zoom",function(){var r=h[0],i=h[1],a=e?e[0]:r/2,s=e?e[1]:i/2,A=o.interpolateZoom([(a-g.x)/g.k,(s-g.y)/g.k,r/g.k],[(a-n.x)/n.k,(s-n.y)/n.k,r/n.k]);return function(e){var n=A(e),i=r/n[2];this.__chart__=g={x:a-n[0]*i,y:s-n[1]*i,k:i},D(t)}}).each("interrupt.zoom",function(){M(t)}).each("end.zoom",function(){M(t)}):(this.__chart__=g,_(t),D(t),M(t))})},v.translate=function(t){return arguments.length?(g={x:+t[0],y:+t[1],k:g.k},x(),v):[g.x,g.y]},v.scale=function(t){return arguments.length?(g={x:g.x,y:g.y,k:null},b(+t),x(),v):g.k},v.scaleExtent=function(t){return arguments.length?(d=null==t?Jt:[+t[0],+t[1]],v):d},v.center=function(t){return arguments.length?(n=t&&[+t[0],+t[1]],v):n},v.size=function(t){return arguments.length?(h=t&&[+t[0],+t[1]],v):h},v.duration=function(t){return arguments.length?(f=+t,v):f},v.x=function(t){return arguments.length?(s=t,a=t.copy(),g={x:0,y:0,k:1},v):s},v.y=function(t){return arguments.length?(u=t,l=t.copy(),g={x:0,y:0,k:1},v):u},o.rebind(v,E,"on")};var Kt,zt,Jt=[0,1/0];function qt(){}function Vt(t,e,n){return this instanceof Vt?(this.h=+t,this.s=+e,void(this.l=+n)):arguments.length<2?t instanceof Vt?new Vt(t.h,t.s,t.l):Ie(""+t,Ce,Vt):new Vt(t,e,n)}o.color=qt,qt.prototype.toString=function(){return this.rgb()+""},o.hsl=Vt;var Wt=Vt.prototype=new qt;function Xt(t,e,n){var r,i;function o(t){return Math.round(255*function(t){return t>360?t-=360:t<0&&(t+=360),t<60?r+(i-r)*t/60:t<180?i:t<240?r+(i-r)*(240-t)/60:r}(t))}return t=isNaN(t)?0:(t%=360)<0?t+360:t,e=isNaN(e)?0:e<0?0:e>1?1:e,r=2*(n=n<0?0:n>1?1:n)-(i=n<=.5?n*(1+e):n+e-n*e),new ge(o(t+120),o(t),o(t-120))}function Zt(t,e,n){return this instanceof Zt?(this.h=+t,this.c=+e,void(this.l=+n)):arguments.length<2?t instanceof Zt?new Zt(t.h,t.c,t.l):Ae(t instanceof ee?t.l:(t=me((t=o.rgb(t)).r,t.g,t.b)).l,t.a,t.b):new Zt(t,e,n)}Wt.brighter=function(t){return t=Math.pow(.7,arguments.length?t:1),new Vt(this.h,this.s,this.l/t)},Wt.darker=function(t){return t=Math.pow(.7,arguments.length?t:1),new Vt(this.h,this.s,t*this.l)},Wt.rgb=function(){return Xt(this.h,this.s,this.l)},o.hcl=Zt;var $t=Zt.prototype=new qt;function te(t,e,n){return isNaN(t)&&(t=0),isNaN(e)&&(e=0),new ee(n,Math.cos(t*=Ft)*e,Math.sin(t)*e)}function ee(t,e,n){return this instanceof ee?(this.l=+t,this.a=+e,void(this.b=+n)):arguments.length<2?t instanceof ee?new ee(t.l,t.a,t.b):t instanceof Zt?te(t.h,t.c,t.l):me((t=ge(t)).r,t.g,t.b):new ee(t,e,n)}$t.brighter=function(t){return new Zt(this.h,this.c,Math.min(100,this.l+ne*(arguments.length?t:1)))},$t.darker=function(t){return new Zt(this.h,this.c,Math.max(0,this.l-ne*(arguments.length?t:1)))},$t.rgb=function(){return te(this.h,this.c,this.l).rgb()},o.lab=ee;var ne=18,re=.95047,ie=1,oe=1.08883,ae=ee.prototype=new qt;function se(t,e,n){var r=(t+16)/116,i=r+e/500,o=r-n/200;return new ge(ue(3.2404542*(i=le(i)*re)-1.5371385*(r=le(r)*ie)-.4985314*(o=le(o)*oe)),ue(-.969266*i+1.8760108*r+.041556*o),ue(.0556434*i-.2040259*r+1.0572252*o))}function Ae(t,e,n){return t>0?new Zt(Math.atan2(n,e)*Tt,Math.sqrt(e*e+n*n),t):new Zt(NaN,NaN,t)}function le(t){return t>.206893034?t*t*t:(t-4/29)/7.787037}function ce(t){return t>.008856?Math.pow(t,1/3):7.787037*t+4/29}function ue(t){return Math.round(255*(t<=.00304?12.92*t:1.055*Math.pow(t,1/2.4)-.055))}function ge(t,e,n){return this instanceof ge?(this.r=~~t,this.g=~~e,void(this.b=~~n)):arguments.length<2?t instanceof ge?new ge(t.r,t.g,t.b):Ie(""+t,ge,Xt):new ge(t,e,n)}function he(t){return new ge(t>>16,t>>8&255,255&t)}function de(t){return he(t)+""}ae.brighter=function(t){return new ee(Math.min(100,this.l+ne*(arguments.length?t:1)),this.a,this.b)},ae.darker=function(t){return new ee(Math.max(0,this.l-ne*(arguments.length?t:1)),this.a,this.b)},ae.rgb=function(){return se(this.l,this.a,this.b)},o.rgb=ge;var fe=ge.prototype=new qt;function pe(t){return t<16?"0"+Math.max(0,t).toString(16):Math.min(255,t).toString(16)}function Ie(t,e,n){var r,i,o,a=0,s=0,A=0;if(r=/([a-z]+)\((.*)\)/.exec(t=t.toLowerCase()))switch(i=r[2].split(","),r[1]){case"hsl":return n(parseFloat(i[0]),parseFloat(i[1])/100,parseFloat(i[2])/100);case"rgb":return e(Ee(i[0]),Ee(i[1]),Ee(i[2]))}return(o=ve.get(t))?e(o.r,o.g,o.b):(null==t||"#"!==t.charAt(0)||isNaN(o=parseInt(t.slice(1),16))||(4===t.length?(a=(3840&o)>>4,a|=a>>4,s=240&o,s|=s>>4,A=15&o,A|=A<<4):7===t.length&&(a=(16711680&o)>>16,s=(65280&o)>>8,A=255&o)),e(a,s,A))}function Ce(t,e,n){var r,i,o=Math.min(t/=255,e/=255,n/=255),a=Math.max(t,e,n),s=a-o,A=(a+o)/2;return s?(i=A<.5?s/(a+o):s/(2-a-o),r=t==a?(e-n)/s+(e<n?6:0):e==a?(n-t)/s+2:(t-e)/s+4,r*=60):(r=NaN,i=A>0&&A<1?0:r),new Vt(r,i,A)}function me(t,e,n){var r=ce((.4124564*(t=ye(t))+.3575761*(e=ye(e))+.1804375*(n=ye(n)))/re),i=ce((.2126729*t+.7151522*e+.072175*n)/ie);return ee(116*i-16,500*(r-i),200*(i-ce((.0193339*t+.119192*e+.9503041*n)/oe)))}function ye(t){return(t/=255)<=.04045?t/12.92:Math.pow((t+.055)/1.055,2.4)}function Ee(t){var e=parseFloat(t);return"%"===t.charAt(t.length-1)?Math.round(2.55*e):e}fe.brighter=function(t){t=Math.pow(.7,arguments.length?t:1);var e=this.r,n=this.g,r=this.b,i=30;return e||n||r?(e&&e<i&&(e=i),n&&n<i&&(n=i),r&&r<i&&(r=i),new ge(Math.min(255,e/t),Math.min(255,n/t),Math.min(255,r/t))):new ge(i,i,i)},fe.darker=function(t){return new ge((t=Math.pow(.7,arguments.length?t:1))*this.r,t*this.g,t*this.b)},fe.hsl=function(){return Ce(this.r,this.g,this.b)},fe.toString=function(){return"#"+pe(this.r)+pe(this.g)+pe(this.b)};var ve=o.map({aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074});function Be(t){return"function"==typeof t?t:function(){return t}}function be(t){return function(e,n,r){return 2===arguments.length&&"function"==typeof n&&(r=n,n=null),Qe(e,n,t,r)}}function Qe(t,e,n,r){var i={},a=o.dispatch("beforesend","progress","load","error"),A={},l=new XMLHttpRequest,c=null;function u(){var t,e=l.status;if(!e&&function(t){var e=t.responseType;return e&&"text"!==e?t.response:t.responseText}(l)||e>=200&&e<300||304===e){try{t=n.call(i,l)}catch(t){return void a.error.call(i,t)}a.load.call(i,t)}else a.error.call(i,l)}return!this.XDomainRequest||"withCredentials"in l||!/^(http(s)?:)?\/\//.test(t)||(l=new XDomainRequest),"onload"in l?l.onload=l.onerror=u:l.onreadystatechange=function(){l.readyState>3&&u()},l.onprogress=function(t){var e=o.event;o.event=t;try{a.progress.call(i,l)}finally{o.event=e}},i.header=function(t,e){return t=(t+"").toLowerCase(),arguments.length<2?A[t]:(null==e?delete A[t]:A[t]=e+"",i)},i.mimeType=function(t){return arguments.length?(e=null==t?null:t+"",i):e},i.responseType=function(t){return arguments.length?(c=t,i):c},i.response=function(t){return n=t,i},["get","post"].forEach(function(t){i[t]=function(){return i.send.apply(i,[t].concat(s(arguments)))}}),i.send=function(n,r,o){if(2===arguments.length&&"function"==typeof r&&(o=r,r=null),l.open(n,t,!0),null==e||"accept"in A||(A.accept=e+",*/*"),l.setRequestHeader)for(var s in A)l.setRequestHeader(s,A[s]);return null!=e&&l.overrideMimeType&&l.overrideMimeType(e),null!=c&&(l.responseType=c),null!=o&&i.on("error",o).on("load",function(t){o(null,t)}),a.beforesend.call(i,l),l.send(null==r?null:r),i},i.abort=function(){return l.abort(),i},o.rebind(i,a,"on"),null==r?i:i.get(function(t){return 1===t.length?function(e,n){t(null==e?n:null)}:t}(r))}ve.forEach(function(t,e){ve.set(t,he(e))}),o.functor=Be,o.xhr=be(F),o.dsv=function(t,e){var n=new RegExp('["'+t+"\n]"),r=t.charCodeAt(0);function i(t,n,r){arguments.length<3&&(r=n,n=null);var i=Qe(t,e,null==n?o:a(n),r);return i.row=function(t){return arguments.length?i.response(null==(n=t)?o:a(t)):n},i}function o(t){return i.parse(t.responseText)}function a(t){return function(e){return i.parse(e.responseText,t)}}function s(e){return e.map(A).join(t)}function A(t){return n.test(t)?'"'+t.replace(/\"/g,'""')+'"':t}return i.parse=function(t,e){var n;return i.parseRows(t,function(t,r){if(n)return n(t,r-1);var i=new Function("d","return {"+t.map(function(t,e){return JSON.stringify(t)+": d["+e+"]"}).join(",")+"}");n=e?function(t,n){return e(i(t),n)}:i})},i.parseRows=function(t,e){var n,i,o={},a={},s=[],A=t.length,l=0,c=0;function u(){if(l>=A)return a;if(i)return i=!1,o;var e=l;if(34===t.charCodeAt(e)){for(var n=e;n++<A;)if(34===t.charCodeAt(n)){if(34!==t.charCodeAt(n+1))break;++n}return l=n+2,13===(s=t.charCodeAt(n+1))?(i=!0,10===t.charCodeAt(n+2)&&++l):10===s&&(i=!0),t.slice(e+1,n).replace(/""/g,'"')}for(;l<A;){var s,c=1;if(10===(s=t.charCodeAt(l++)))i=!0;else if(13===s)i=!0,10===t.charCodeAt(l)&&(++l,++c);else if(s!==r)continue;return t.slice(e,l-c)}return t.slice(e)}for(;(n=u())!==a;){for(var g=[];n!==o&&n!==a;)g.push(n),n=u();e&&null==(g=e(g,c++))||s.push(g)}return s},i.format=function(e){if(Array.isArray(e[0]))return i.formatRows(e);var n=new N,r=[];return e.forEach(function(t){for(var e in t)n.has(e)||r.push(n.add(e))}),[r.map(A).join(t)].concat(e.map(function(e){return r.map(function(t){return A(e[t])}).join(t)})).join("\n")},i.formatRows=function(t){return t.map(s).join("\n")},i},o.csv=o.dsv(",","text/csv"),o.tsv=o.dsv("\t","text/tab-separated-values");var we,xe,_e,De,Me=this[U(this,"requestAnimationFrame")]||function(t){setTimeout(t,17)};function ke(t,e,n){var r=arguments.length;r<2&&(e=0),r<3&&(n=Date.now());var i={c:t,t:n+e,n:null};return xe?xe.n=i:we=i,xe=i,_e||(De=clearTimeout(De),_e=1,Me(Se)),i}function Se(){var t=Le(),e=Re()-t;e>24?(isFinite(e)&&(clearTimeout(De),De=setTimeout(Se,e)),_e=0):(_e=1,Me(Se))}function Le(){for(var t=Date.now(),e=we;e;)t>=e.t&&e.c(t-e.t)&&(e.c=null),e=e.n;return t}function Re(){for(var t,e=we,n=1/0;e;)e.c?(e.t<n&&(n=e.t),e=(t=e).n):e=t?t.n=e.n:we=e.n;return xe=t,n}function Ne(t,e){return e-(t?Math.ceil(Math.log(t)/Math.LN10):1)}o.timer=function(){ke.apply(this,arguments)},o.timer.flush=function(){Le(),Re()},o.round=function(t,e){return e?Math.round(t*(e=Math.pow(10,e)))/e:Math.round(t)};var Fe=["y","z","a","f","p","n","µ","m","","k","M","G","T","P","E","Z","Y"].map(function(t,e){var n=Math.pow(10,3*v(8-e));return{scale:e>8?function(t){return t/n}:function(t){return t*n},symbol:t}});function Te(t){var e=t.decimal,n=t.thousands,r=t.grouping,i=t.currency,a=r&&n?function(t,e){for(var i=t.length,o=[],a=0,s=r[0],A=0;i>0&&s>0&&(A+s+1>e&&(s=Math.max(1,e-A)),o.push(t.substring(i-=s,i+s)),!((A+=s+1)>e));)s=r[a=(a+1)%r.length];return o.reverse().join(n)}:F;return function(t){var n=Ue.exec(t),r=n[1]||" ",s=n[2]||">",A=n[3]||"-",l=n[4]||"",c=n[5],u=+n[6],g=n[7],h=n[8],d=n[9],f=1,p="",I="",C=!1,m=!0;switch(h&&(h=+h.substring(1)),(c||"0"===r&&"="===s)&&(c=r="0",s="="),d){case"n":g=!0,d="g";break;case"%":f=100,I="%",d="f";break;case"p":f=100,I="%",d="r";break;case"b":case"o":case"x":case"X":"#"===l&&(p="0"+d.toLowerCase());case"c":m=!1;case"d":C=!0,h=0;break;case"s":f=-1,d="r"}"$"===l&&(p=i[0],I=i[1]),"r"!=d||h||(d="g"),null!=h&&("g"==d?h=Math.max(1,Math.min(21,h)):"e"!=d&&"f"!=d||(h=Math.max(0,Math.min(20,h)))),d=Oe.get(d)||Ge;var y=c&&g;return function(t){var n=I;if(C&&t%1)return"";var i=t<0||0===t&&1/t<0?(t=-t,"-"):"-"===A?"":A;if(f<0){var l=o.formatPrefix(t,h);t=l.scale(t),n=l.symbol+I}else t*=f;var E,v,B=(t=d(t,h)).lastIndexOf(".");if(B<0){var b=m?t.lastIndexOf("e"):-1;b<0?(E=t,v=""):(E=t.substring(0,b),v=t.substring(b))}else E=t.substring(0,B),v=e+t.substring(B+1);!c&&g&&(E=a(E,1/0));var Q=p.length+E.length+v.length+(y?0:i.length),w=Q<u?new Array(Q=u-Q+1).join(r):"";return y&&(E=a(w+E,w.length?u-v.length:1/0)),i+=p,t=E+v,("<"===s?i+t+w:">"===s?w+i+t:"^"===s?w.substring(0,Q>>=1)+i+t+w.substring(Q):i+(y?t:w+t))+n}}}o.formatPrefix=function(t,e){var n=0;return(t=+t)&&(t<0&&(t*=-1),e&&(t=o.round(t,Ne(t,e))),n=1+Math.floor(1e-12+Math.log(t)/Math.LN10),n=Math.max(-24,Math.min(24,3*Math.floor((n-1)/3)))),Fe[8+n/3]};var Ue=/(?:([^{])?([<>=^]))?([+\- ])?([$#])?(0)?(\d+)?(,)?(\.-?\d+)?([a-z%])?/i,Oe=o.map({b:function(t){return t.toString(2)},c:function(t){return String.fromCharCode(t)},o:function(t){return t.toString(8)},x:function(t){return t.toString(16)},X:function(t){return t.toString(16).toUpperCase()},g:function(t,e){return t.toPrecision(e)},e:function(t,e){return t.toExponential(e)},f:function(t,e){return t.toFixed(e)},r:function(t,e){return(t=o.round(t,Ne(t,e))).toFixed(Math.max(0,Math.min(20,Ne(t*(1+1e-15),e))))}});function Ge(t){return t+""}var He=o.time={},je=Date;function Ye(){this._=new Date(arguments.length>1?Date.UTC.apply(this,arguments):arguments[0])}Ye.prototype={getDate:function(){return this._.getUTCDate()},getDay:function(){return this._.getUTCDay()},getFullYear:function(){return this._.getUTCFullYear()},getHours:function(){return this._.getUTCHours()},getMilliseconds:function(){return this._.getUTCMilliseconds()},getMinutes:function(){return this._.getUTCMinutes()},getMonth:function(){return this._.getUTCMonth()},getSeconds:function(){return this._.getUTCSeconds()},getTime:function(){return this._.getTime()},getTimezoneOffset:function(){return 0},valueOf:function(){return this._.valueOf()},setDate:function(){Pe.setUTCDate.apply(this._,arguments)},setDay:function(){Pe.setUTCDay.apply(this._,arguments)},setFullYear:function(){Pe.setUTCFullYear.apply(this._,arguments)},setHours:function(){Pe.setUTCHours.apply(this._,arguments)},setMilliseconds:function(){Pe.setUTCMilliseconds.apply(this._,arguments)},setMinutes:function(){Pe.setUTCMinutes.apply(this._,arguments)},setMonth:function(){Pe.setUTCMonth.apply(this._,arguments)},setSeconds:function(){Pe.setUTCSeconds.apply(this._,arguments)},setTime:function(){Pe.setTime.apply(this._,arguments)}};var Pe=Date.prototype;function Ke(t,e,n){function r(e){var n=t(e),r=o(n,1);return e-n<r-e?n:r}function i(n){return e(n=t(new je(n-1)),1),n}function o(t,n){return e(t=new je(+t),n),t}function a(t,r,o){var a=i(t),s=[];if(o>1)for(;a<r;)n(a)%o||s.push(new Date(+a)),e(a,1);else for(;a<r;)s.push(new Date(+a)),e(a,1);return s}t.floor=t,t.round=r,t.ceil=i,t.offset=o,t.range=a;var s=t.utc=ze(t);return s.floor=s,s.round=ze(r),s.ceil=ze(i),s.offset=ze(o),s.range=function(t,e,n){try{je=Ye;var r=new Ye;return r._=t,a(r,e,n)}finally{je=Date}},t}function ze(t){return function(e,n){try{je=Ye;var r=new Ye;return r._=e,t(r,n)._}finally{je=Date}}}function Je(t){var e=t.dateTime,n=t.date,r=t.time,i=t.periods,a=t.days,s=t.shortDays,A=t.months,l=t.shortMonths;function c(t){var e=t.length;function n(n){for(var r,i,o,a=[],s=-1,A=0;++s<e;)37===t.charCodeAt(s)&&(a.push(t.slice(A,s)),null!=(i=qe[r=t.charAt(++s)])&&(r=t.charAt(++s)),(o=E[r])&&(r=o(n,null==i?"e"===r?" ":"0":i)),a.push(r),A=s+1);return a.push(t.slice(A,s)),a.join("")}return n.parse=function(e){var n={y:1900,m:0,d:1,H:0,M:0,S:0,L:0,Z:null};if(u(n,t,e,0)!=e.length)return null;"p"in n&&(n.H=n.H%12+12*n.p);var r=null!=n.Z&&je!==Ye,i=new(r?Ye:je);return"j"in n?i.setFullYear(n.y,0,n.j):"W"in n||"U"in n?("w"in n||(n.w="W"in n?1:0),i.setFullYear(n.y,0,1),i.setFullYear(n.y,0,"W"in n?(n.w+6)%7+7*n.W-(i.getDay()+5)%7:n.w+7*n.U-(i.getDay()+6)%7)):i.setFullYear(n.y,n.m,n.d),i.setHours(n.H+(n.Z/100|0),n.M+n.Z%100,n.S,n.L),r?i._:i},n.toString=function(){return t},n}function u(t,e,n,r){for(var i,o,a,s=0,A=e.length,l=n.length;s<A;){if(r>=l)return-1;if(37===(i=e.charCodeAt(s++))){if(a=e.charAt(s++),!(o=v[a in qe?e.charAt(s++):a])||(r=o(t,n,r))<0)return-1}else if(i!=n.charCodeAt(r++))return-1}return r}c.utc=function(t){var e=c(t);function n(t){try{var n=new(je=Ye);return n._=t,e(n)}finally{je=Date}}return n.parse=function(t){try{je=Ye;var n=e.parse(t);return n&&n._}finally{je=Date}},n.toString=e.toString,n},c.multi=c.utc.multi=pn;var g=o.map(),h=Ze(a),d=$e(a),f=Ze(s),p=$e(s),I=Ze(A),C=$e(A),m=Ze(l),y=$e(l);i.forEach(function(t,e){g.set(t.toLowerCase(),e)});var E={a:function(t){return s[t.getDay()]},A:function(t){return a[t.getDay()]},b:function(t){return l[t.getMonth()]},B:function(t){return A[t.getMonth()]},c:c(e),d:function(t,e){return Xe(t.getDate(),e,2)},e:function(t,e){return Xe(t.getDate(),e,2)},H:function(t,e){return Xe(t.getHours(),e,2)},I:function(t,e){return Xe(t.getHours()%12||12,e,2)},j:function(t,e){return Xe(1+He.dayOfYear(t),e,3)},L:function(t,e){return Xe(t.getMilliseconds(),e,3)},m:function(t,e){return Xe(t.getMonth()+1,e,2)},M:function(t,e){return Xe(t.getMinutes(),e,2)},p:function(t){return i[+(t.getHours()>=12)]},S:function(t,e){return Xe(t.getSeconds(),e,2)},U:function(t,e){return Xe(He.sundayOfYear(t),e,2)},w:function(t){return t.getDay()},W:function(t,e){return Xe(He.mondayOfYear(t),e,2)},x:c(n),X:c(r),y:function(t,e){return Xe(t.getFullYear()%100,e,2)},Y:function(t,e){return Xe(t.getFullYear()%1e4,e,4)},Z:dn,"%":function(){return"%"}},v={a:function(t,e,n){f.lastIndex=0;var r=f.exec(e.slice(n));return r?(t.w=p.get(r[0].toLowerCase()),n+r[0].length):-1},A:function(t,e,n){h.lastIndex=0;var r=h.exec(e.slice(n));return r?(t.w=d.get(r[0].toLowerCase()),n+r[0].length):-1},b:function(t,e,n){m.lastIndex=0;var r=m.exec(e.slice(n));return r?(t.m=y.get(r[0].toLowerCase()),n+r[0].length):-1},B:function(t,e,n){I.lastIndex=0;var r=I.exec(e.slice(n));return r?(t.m=C.get(r[0].toLowerCase()),n+r[0].length):-1},c:function(t,e,n){return u(t,E.c.toString(),e,n)},d:An,e:An,H:cn,I:cn,j:ln,L:hn,m:sn,M:un,p:function(t,e,n){var r=g.get(e.slice(n,n+=2).toLowerCase());return null==r?-1:(t.p=r,n)},S:gn,U:en,w:tn,W:nn,x:function(t,e,n){return u(t,E.x.toString(),e,n)},X:function(t,e,n){return u(t,E.X.toString(),e,n)},y:on,Y:rn,Z:an,"%":fn};return c}He.year=Ke(function(t){return(t=He.day(t)).setMonth(0,1),t},function(t,e){t.setFullYear(t.getFullYear()+e)},function(t){return t.getFullYear()}),He.years=He.year.range,He.years.utc=He.year.utc.range,He.day=Ke(function(t){var e=new je(2e3,0);return e.setFullYear(t.getFullYear(),t.getMonth(),t.getDate()),e},function(t,e){t.setDate(t.getDate()+e)},function(t){return t.getDate()-1}),He.days=He.day.range,He.days.utc=He.day.utc.range,He.dayOfYear=function(t){var e=He.year(t);return Math.floor((t-e-6e4*(t.getTimezoneOffset()-e.getTimezoneOffset()))/864e5)},["sunday","monday","tuesday","wednesday","thursday","friday","saturday"].forEach(function(t,e){e=7-e;var n=He[t]=Ke(function(t){return(t=He.day(t)).setDate(t.getDate()-(t.getDay()+e)%7),t},function(t,e){t.setDate(t.getDate()+7*Math.floor(e))},function(t){var n=He.year(t).getDay();return Math.floor((He.dayOfYear(t)+(n+e)%7)/7)-(n!==e)});He[t+"s"]=n.range,He[t+"s"].utc=n.utc.range,He[t+"OfYear"]=function(t){var n=He.year(t).getDay();return Math.floor((He.dayOfYear(t)+(n+e)%7)/7)}}),He.week=He.sunday,He.weeks=He.sunday.range,He.weeks.utc=He.sunday.utc.range,He.weekOfYear=He.sundayOfYear;var qe={"-":"",_:" ",0:"0"},Ve=/^\s*\d+/,We=/^%/;function Xe(t,e,n){var r=t<0?"-":"",i=(r?-t:t)+"",o=i.length;return r+(o<n?new Array(n-o+1).join(e)+i:i)}function Ze(t){return new RegExp("^(?:"+t.map(o.requote).join("|")+")","i")}function $e(t){for(var e=new Q,n=-1,r=t.length;++n<r;)e.set(t[n].toLowerCase(),n);return e}function tn(t,e,n){Ve.lastIndex=0;var r=Ve.exec(e.slice(n,n+1));return r?(t.w=+r[0],n+r[0].length):-1}function en(t,e,n){Ve.lastIndex=0;var r=Ve.exec(e.slice(n));return r?(t.U=+r[0],n+r[0].length):-1}function nn(t,e,n){Ve.lastIndex=0;var r=Ve.exec(e.slice(n));return r?(t.W=+r[0],n+r[0].length):-1}function rn(t,e,n){Ve.lastIndex=0;var r=Ve.exec(e.slice(n,n+4));return r?(t.y=+r[0],n+r[0].length):-1}function on(t,e,n){Ve.lastIndex=0;var r,i=Ve.exec(e.slice(n,n+2));return i?(t.y=(r=+i[0])+(r>68?1900:2e3),n+i[0].length):-1}function an(t,e,n){return/^[+-]\d{4}$/.test(e=e.slice(n,n+5))?(t.Z=-e,n+5):-1}function sn(t,e,n){Ve.lastIndex=0;var r=Ve.exec(e.slice(n,n+2));return r?(t.m=r[0]-1,n+r[0].length):-1}function An(t,e,n){Ve.lastIndex=0;var r=Ve.exec(e.slice(n,n+2));return r?(t.d=+r[0],n+r[0].length):-1}function ln(t,e,n){Ve.lastIndex=0;var r=Ve.exec(e.slice(n,n+3));return r?(t.j=+r[0],n+r[0].length):-1}function cn(t,e,n){Ve.lastIndex=0;var r=Ve.exec(e.slice(n,n+2));return r?(t.H=+r[0],n+r[0].length):-1}function un(t,e,n){Ve.lastIndex=0;var r=Ve.exec(e.slice(n,n+2));return r?(t.M=+r[0],n+r[0].length):-1}function gn(t,e,n){Ve.lastIndex=0;var r=Ve.exec(e.slice(n,n+2));return r?(t.S=+r[0],n+r[0].length):-1}function hn(t,e,n){Ve.lastIndex=0;var r=Ve.exec(e.slice(n,n+3));return r?(t.L=+r[0],n+r[0].length):-1}function dn(t){var e=t.getTimezoneOffset(),n=e>0?"-":"+",r=v(e)/60|0,i=v(e)%60;return n+Xe(r,"0",2)+Xe(i,"0",2)}function fn(t,e,n){We.lastIndex=0;var r=We.exec(e.slice(n,n+1));return r?n+r[0].length:-1}function pn(t){for(var e=t.length,n=-1;++n<e;)t[n][0]=this(t[n][0]);return function(e){for(var n=0,r=t[n];!r[1](e);)r=t[++n];return r[0](e)}}o.locale=function(t){return{numberFormat:Te(t),timeFormat:Je(t)}};var In=o.locale({decimal:".",thousands:",",grouping:[3],currency:["$",""],dateTime:"%a %b %e %X %Y",date:"%m/%d/%Y",time:"%H:%M:%S",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]});function Cn(){}o.format=In.numberFormat,o.geo={},Cn.prototype={s:0,t:0,add:function(t){yn(t,this.t,mn),yn(mn.s,this.s,this),this.s?this.t+=mn.t:this.s=mn.t},reset:function(){this.s=this.t=0},valueOf:function(){return this.s}};var mn=new Cn;function yn(t,e,n){var r=n.s=t+e,i=r-t,o=r-i;n.t=t-o+(e-i)}function En(t,e){t&&Bn.hasOwnProperty(t.type)&&Bn[t.type](t,e)}o.geo.stream=function(t,e){t&&vn.hasOwnProperty(t.type)?vn[t.type](t,e):En(t,e)};var vn={Feature:function(t,e){En(t.geometry,e)},FeatureCollection:function(t,e){for(var n=t.features,r=-1,i=n.length;++r<i;)En(n[r].geometry,e)}},Bn={Sphere:function(t,e){e.sphere()},Point:function(t,e){t=t.coordinates,e.point(t[0],t[1],t[2])},MultiPoint:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)t=n[r],e.point(t[0],t[1],t[2])},LineString:function(t,e){bn(t.coordinates,e,0)},MultiLineString:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)bn(n[r],e,0)},Polygon:function(t,e){Qn(t.coordinates,e)},MultiPolygon:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)Qn(n[r],e)},GeometryCollection:function(t,e){for(var n=t.geometries,r=-1,i=n.length;++r<i;)En(n[r],e)}};function bn(t,e,n){var r,i=-1,o=t.length-n;for(e.lineStart();++i<o;)r=t[i],e.point(r[0],r[1],r[2]);e.lineEnd()}function Qn(t,e){var n=-1,r=t.length;for(e.polygonStart();++n<r;)bn(t[n],e,1);e.polygonEnd()}o.geo.area=function(t){return wn=0,o.geo.stream(t,On),wn};var wn,xn,_n,Dn,Mn,kn,Sn,Ln,Rn,Nn,Fn,Tn,Un=new Cn,On={sphere:function(){wn+=4*St},point:G,lineStart:G,lineEnd:G,polygonStart:function(){Un.reset(),On.lineStart=Gn},polygonEnd:function(){var t=2*Un;wn+=t<0?4*St+t:t,On.lineStart=On.lineEnd=On.point=G}};function Gn(){var t,e,n,r,i;function o(t,e){e=e*Ft/2+St/4;var o=(t*=Ft)-n,a=o>=0?1:-1,s=a*o,A=Math.cos(e),l=Math.sin(e),c=i*l,u=r*A+c*Math.cos(s),g=c*a*Math.sin(s);Un.add(Math.atan2(g,u)),n=t,r=A,i=l}On.point=function(a,s){On.point=o,n=(t=a)*Ft,r=Math.cos(s=(e=s)*Ft/2+St/4),i=Math.sin(s)},On.lineEnd=function(){o(t,e)}}function Hn(t){var e=t[0],n=t[1],r=Math.cos(n);return[r*Math.cos(e),r*Math.sin(e),Math.sin(n)]}function jn(t,e){return t[0]*e[0]+t[1]*e[1]+t[2]*e[2]}function Yn(t,e){return[t[1]*e[2]-t[2]*e[1],t[2]*e[0]-t[0]*e[2],t[0]*e[1]-t[1]*e[0]]}function Pn(t,e){t[0]+=e[0],t[1]+=e[1],t[2]+=e[2]}function Kn(t,e){return[t[0]*e,t[1]*e,t[2]*e]}function zn(t){var e=Math.sqrt(t[0]*t[0]+t[1]*t[1]+t[2]*t[2]);t[0]/=e,t[1]/=e,t[2]/=e}function Jn(t){return[Math.atan2(t[1],t[0]),Ht(t[2])]}function qn(t,e){return v(t[0]-e[0])<Mt&&v(t[1]-e[1])<Mt}o.geo.bounds=function(){var t,e,n,r,i,a,s,A,l,c,u,g={point:h,lineStart:f,lineEnd:p,polygonStart:function(){g.point=I,g.lineStart=C,g.lineEnd=m,l=0,On.polygonStart()},polygonEnd:function(){On.polygonEnd(),g.point=h,g.lineStart=f,g.lineEnd=p,Un<0?(t=-(n=180),e=-(r=90)):l>Mt?r=90:l<-Mt&&(e=-90),u[0]=t,u[1]=n}};function h(i,o){c.push(u=[t=i,n=i]),o<e&&(e=o),o>r&&(r=o)}function d(o,a){var s=Hn([o*Ft,a*Ft]);if(A){var l=Yn(A,s),c=Yn([l[1],-l[0],0],l);zn(c),c=Jn(c);var u=o-i,g=u>0?1:-1,d=c[0]*Tt*g,f=v(u)>180;if(f^(g*i<d&&d<g*o))(p=c[1]*Tt)>r&&(r=p);else if(f^(g*i<(d=(d+360)%360-180)&&d<g*o)){var p;(p=-c[1]*Tt)<e&&(e=p)}else a<e&&(e=a),a>r&&(r=a);f?o<i?y(t,o)>y(t,n)&&(n=o):y(o,n)>y(t,n)&&(t=o):n>=t?(o<t&&(t=o),o>n&&(n=o)):o>i?y(t,o)>y(t,n)&&(n=o):y(o,n)>y(t,n)&&(t=o)}else h(o,a);A=s,i=o}function f(){g.point=d}function p(){u[0]=t,u[1]=n,g.point=h,A=null}function I(t,e){if(A){var n=t-i;l+=v(n)>180?n+(n>0?360:-360):n}else a=t,s=e;On.point(t,e),d(t,e)}function C(){On.lineStart()}function m(){I(a,s),On.lineEnd(),v(l)>Mt&&(t=-(n=180)),u[0]=t,u[1]=n,A=null}function y(t,e){return(e-=t)<0?e+360:e}function E(t,e){return t[0]-e[0]}function B(t,e){return e[0]<=e[1]?e[0]<=t&&t<=e[1]:t<e[0]||e[1]<t}return function(i){if(r=n=-(t=e=1/0),c=[],o.geo.stream(i,g),l=c.length){c.sort(E);for(var a=1,s=[f=c[0]];a<l;++a)B((h=c[a])[0],f)||B(h[1],f)?(y(f[0],h[1])>y(f[0],f[1])&&(f[1]=h[1]),y(h[0],f[1])>y(f[0],f[1])&&(f[0]=h[0])):s.push(f=h);for(var A,l,h,d=-1/0,f=(a=0,s[l=s.length-1]);a<=l;f=h,++a)h=s[a],(A=y(f[1],h[0]))>d&&(d=A,t=h[0],n=f[1])}return c=u=null,t===1/0||e===1/0?[[NaN,NaN],[NaN,NaN]]:[[t,e],[n,r]]}}(),o.geo.centroid=function(t){xn=_n=Dn=Mn=kn=Sn=Ln=Rn=Nn=Fn=Tn=0,o.geo.stream(t,Vn);var e=Nn,n=Fn,r=Tn,i=e*e+n*n+r*r;return i<kt&&(e=Sn,n=Ln,r=Rn,_n<Mt&&(e=Dn,n=Mn,r=kn),(i=e*e+n*n+r*r)<kt)?[NaN,NaN]:[Math.atan2(n,e)*Tt,Ht(r/Math.sqrt(i))*Tt]};var Vn={sphere:G,point:Wn,lineStart:Zn,lineEnd:$n,polygonStart:function(){Vn.lineStart=tr},polygonEnd:function(){Vn.lineStart=Zn}};function Wn(t,e){t*=Ft;var n=Math.cos(e*=Ft);Xn(n*Math.cos(t),n*Math.sin(t),Math.sin(e))}function Xn(t,e,n){Dn+=(t-Dn)/++xn,Mn+=(e-Mn)/xn,kn+=(n-kn)/xn}function Zn(){var t,e,n;function r(r,i){r*=Ft;var o=Math.cos(i*=Ft),a=o*Math.cos(r),s=o*Math.sin(r),A=Math.sin(i),l=Math.atan2(Math.sqrt((l=e*A-n*s)*l+(l=n*a-t*A)*l+(l=t*s-e*a)*l),t*a+e*s+n*A);_n+=l,Sn+=l*(t+(t=a)),Ln+=l*(e+(e=s)),Rn+=l*(n+(n=A)),Xn(t,e,n)}Vn.point=function(i,o){i*=Ft;var a=Math.cos(o*=Ft);t=a*Math.cos(i),e=a*Math.sin(i),n=Math.sin(o),Vn.point=r,Xn(t,e,n)}}function $n(){Vn.point=Wn}function tr(){var t,e,n,r,i;function o(t,e){t*=Ft;var o=Math.cos(e*=Ft),a=o*Math.cos(t),s=o*Math.sin(t),A=Math.sin(e),l=r*A-i*s,c=i*a-n*A,u=n*s-r*a,g=Math.sqrt(l*l+c*c+u*u),h=n*a+r*s+i*A,d=g&&-Gt(h)/g,f=Math.atan2(g,h);Nn+=d*l,Fn+=d*c,Tn+=d*u,_n+=f,Sn+=f*(n+(n=a)),Ln+=f*(r+(r=s)),Rn+=f*(i+(i=A)),Xn(n,r,i)}Vn.point=function(a,s){t=a,e=s,Vn.point=o,a*=Ft;var A=Math.cos(s*=Ft);n=A*Math.cos(a),r=A*Math.sin(a),i=Math.sin(s),Xn(n,r,i)},Vn.lineEnd=function(){o(t,e),Vn.lineEnd=$n,Vn.point=Wn}}function er(t,e){function n(n,r){return n=t(n,r),e(n[0],n[1])}return t.invert&&e.invert&&(n.invert=function(n,r){return(n=e.invert(n,r))&&t.invert(n[0],n[1])}),n}function nr(){return!0}function rr(t,e,n,r,i){var o=[],a=[];if(t.forEach(function(t){if(!((e=t.length-1)<=0)){var e,n=t[0],r=t[e];if(qn(n,r)){i.lineStart();for(var s=0;s<e;++s)i.point((n=t[s])[0],n[1]);i.lineEnd()}else{var A=new or(n,t,null,!0),l=new or(n,null,A,!1);A.o=l,o.push(A),a.push(l),A=new or(r,t,null,!1),l=new or(r,null,A,!0),A.o=l,o.push(A),a.push(l)}}}),a.sort(e),ir(o),ir(a),o.length){for(var s=0,A=n,l=a.length;s<l;++s)a[s].e=A=!A;for(var c,u,g=o[0];;){for(var h=g,d=!0;h.v;)if((h=h.n)===g)return;c=h.z,i.lineStart();do{if(h.v=h.o.v=!0,h.e){if(d)for(s=0,l=c.length;s<l;++s)i.point((u=c[s])[0],u[1]);else r(h.x,h.n.x,1,i);h=h.n}else{if(d)for(s=(c=h.p.z).length-1;s>=0;--s)i.point((u=c[s])[0],u[1]);else r(h.x,h.p.x,-1,i);h=h.p}c=(h=h.o).z,d=!d}while(!h.v);i.lineEnd()}}}function ir(t){if(e=t.length){for(var e,n,r=0,i=t[0];++r<e;)i.n=n=t[r],n.p=i,i=n;i.n=n=t[0],n.p=i}}function or(t,e,n,r){this.x=t,this.z=e,this.o=n,this.e=r,this.v=!1,this.n=this.p=null}function ar(t,e,n,r){return function(i,a){var s,A=e(a),l=i.invert(r[0],r[1]),c={point:u,lineStart:h,lineEnd:d,polygonStart:function(){c.point=y,c.lineStart=E,c.lineEnd=v,s=[],f=[]},polygonEnd:function(){c.point=u,c.lineStart=h,c.lineEnd=d,s=o.merge(s);var t=function(t,e){var n=t[0],r=t[1],i=[Math.sin(n),-Math.cos(n),0],o=0,a=0;Un.reset();for(var s=0,A=e.length;s<A;++s){var l=e[s],c=l.length;if(c)for(var u=l[0],g=u[0],h=u[1]/2+St/4,d=Math.sin(h),f=Math.cos(h),p=1;;){p===c&&(p=0);var I=(t=l[p])[0],C=t[1]/2+St/4,m=Math.sin(C),y=Math.cos(C),E=I-g,v=E>=0?1:-1,B=v*E,b=B>St,Q=d*m;if(Un.add(Math.atan2(Q*v*Math.sin(B),f*y+Q*Math.cos(B))),o+=b?E+v*Lt:E,b^g>=n^I>=n){var w=Yn(Hn(u),Hn(t));zn(w);var x=Yn(i,w);zn(x);var _=(b^E>=0?-1:1)*Ht(x[2]);(r>_||r===_&&(w[0]||w[1]))&&(a+=b^E>=0?1:-1)}if(!p++)break;g=I,d=m,f=y,u=t}}return(o<-Mt||o<Mt&&Un<-Mt)^1&a}(l,f);s.length?(m||(a.polygonStart(),m=!0),rr(s,lr,t,n,a)):t&&(m||(a.polygonStart(),m=!0),a.lineStart(),n(null,null,1,a),a.lineEnd()),m&&(a.polygonEnd(),m=!1),s=f=null},sphere:function(){a.polygonStart(),a.lineStart(),n(null,null,1,a),a.lineEnd(),a.polygonEnd()}};function u(e,n){var r=i(e,n);t(e=r[0],n=r[1])&&a.point(e,n)}function g(t,e){var n=i(t,e);A.point(n[0],n[1])}function h(){c.point=g,A.lineStart()}function d(){c.point=u,A.lineEnd()}var f,p,I=Ar(),C=e(I),m=!1;function y(t,e){p.push([t,e]);var n=i(t,e);C.point(n[0],n[1])}function E(){C.lineStart(),p=[]}function v(){y(p[0][0],p[0][1]),C.lineEnd();var t,e=C.clean(),n=I.buffer(),r=n.length;if(p.pop(),f.push(p),p=null,r)if(1&e){var i,o=-1;if((r=(t=n[0]).length-1)>0){for(m||(a.polygonStart(),m=!0),a.lineStart();++o<r;)a.point((i=t[o])[0],i[1]);a.lineEnd()}}else r>1&&2&e&&n.push(n.pop().concat(n.shift())),s.push(n.filter(sr))}return c}}function sr(t){return t.length>1}function Ar(){var t,e=[];return{lineStart:function(){e.push(t=[])},point:function(e,n){t.push([e,n])},lineEnd:G,buffer:function(){var n=e;return e=[],t=null,n},rejoin:function(){e.length>1&&e.push(e.pop().concat(e.shift()))}}}function lr(t,e){return((t=t.x)[0]<0?t[1]-Nt-Mt:Nt-t[1])-((e=e.x)[0]<0?e[1]-Nt-Mt:Nt-e[1])}var cr=ar(nr,function(t){var e,n=NaN,r=NaN,i=NaN;return{lineStart:function(){t.lineStart(),e=1},point:function(o,a){var s=o>0?St:-St,A=v(o-n);v(A-St)<Mt?(t.point(n,r=(r+a)/2>0?Nt:-Nt),t.point(i,r),t.lineEnd(),t.lineStart(),t.point(s,r),t.point(o,r),e=0):i!==s&&A>=St&&(v(n-i)<Mt&&(n-=i*Mt),v(o-s)<Mt&&(o-=s*Mt),r=function(t,e,n,r){var i,o,a=Math.sin(t-n);return v(a)>Mt?Math.atan((Math.sin(e)*(o=Math.cos(r))*Math.sin(n)-Math.sin(r)*(i=Math.cos(e))*Math.sin(t))/(i*o*a)):(e+r)/2}(n,r,o,a),t.point(i,r),t.lineEnd(),t.lineStart(),t.point(s,r),e=0),t.point(n=o,r=a),i=s},lineEnd:function(){t.lineEnd(),n=r=NaN},clean:function(){return 2-e}}},function(t,e,n,r){var i;if(null==t)i=n*Nt,r.point(-St,i),r.point(0,i),r.point(St,i),r.point(St,0),r.point(St,-i),r.point(0,-i),r.point(-St,-i),r.point(-St,0),r.point(-St,i);else if(v(t[0]-e[0])>Mt){var o=t[0]<e[0]?St:-St;i=n*o/2,r.point(-o,i),r.point(0,i),r.point(o,i)}else r.point(e[0],e[1])},[-St,-St/2]);function ur(t){var e=Math.cos(t),n=e>0,r=v(e)>Mt;return ar(i,function(t){var e,s,A,l,c;return{lineStart:function(){l=A=!1,c=1},point:function(u,g){var h,d=[u,g],f=i(u,g),p=n?f?0:a(u,g):f?a(u+(u<0?St:-St),g):0;if(!e&&(l=A=f)&&t.lineStart(),f!==A&&(h=o(e,d),(qn(e,h)||qn(d,h))&&(d[0]+=Mt,d[1]+=Mt,f=i(d[0],d[1]))),f!==A)c=0,f?(t.lineStart(),h=o(d,e),t.point(h[0],h[1])):(h=o(e,d),t.point(h[0],h[1]),t.lineEnd()),e=h;else if(r&&e&&n^f){var I;p&s||!(I=o(d,e,!0))||(c=0,n?(t.lineStart(),t.point(I[0][0],I[0][1]),t.point(I[1][0],I[1][1]),t.lineEnd()):(t.point(I[1][0],I[1][1]),t.lineEnd(),t.lineStart(),t.point(I[0][0],I[0][1])))}!f||e&&qn(e,d)||t.point(d[0],d[1]),e=d,A=f,s=p},lineEnd:function(){A&&t.lineEnd(),e=null},clean:function(){return c|(l&&A)<<1}}},qr(t,6*Ft),n?[0,-t]:[-St,t-St]);function i(t,n){return Math.cos(t)*Math.cos(n)>e}function o(t,n,r){var i=[1,0,0],o=Yn(Hn(t),Hn(n)),a=jn(o,o),s=o[0],A=a-s*s;if(!A)return!r&&t;var l=e*a/A,c=-e*s/A,u=Yn(i,o),g=Kn(i,l);Pn(g,Kn(o,c));var h=u,d=jn(g,h),f=jn(h,h),p=d*d-f*(jn(g,g)-1);if(!(p<0)){var I=Math.sqrt(p),C=Kn(h,(-d-I)/f);if(Pn(C,g),C=Jn(C),!r)return C;var m,y=t[0],E=n[0],B=t[1],b=n[1];E<y&&(m=y,y=E,E=m);var Q=E-y,w=v(Q-St)<Mt;if(!w&&b<B&&(m=B,B=b,b=m),w||Q<Mt?w?B+b>0^C[1]<(v(C[0]-y)<Mt?B:b):B<=C[1]&&C[1]<=b:Q>St^(y<=C[0]&&C[0]<=E)){var x=Kn(h,(-d+I)/f);return Pn(x,g),[C,Jn(x)]}}}function a(e,r){var i=n?t:St-t,o=0;return e<-i?o|=1:e>i&&(o|=2),r<-i?o|=4:r>i&&(o|=8),o}}function gr(t,e,n,r){return function(i){var o,a=i.a,s=i.b,A=a.x,l=a.y,c=0,u=1,g=s.x-A,h=s.y-l;if(o=t-A,g||!(o>0)){if(o/=g,g<0){if(o<c)return;o<u&&(u=o)}else if(g>0){if(o>u)return;o>c&&(c=o)}if(o=n-A,g||!(o<0)){if(o/=g,g<0){if(o>u)return;o>c&&(c=o)}else if(g>0){if(o<c)return;o<u&&(u=o)}if(o=e-l,h||!(o>0)){if(o/=h,h<0){if(o<c)return;o<u&&(u=o)}else if(h>0){if(o>u)return;o>c&&(c=o)}if(o=r-l,h||!(o<0)){if(o/=h,h<0){if(o>u)return;o>c&&(c=o)}else if(h>0){if(o<c)return;o<u&&(u=o)}return c>0&&(i.a={x:A+c*g,y:l+c*h}),u<1&&(i.b={x:A+u*g,y:l+u*h}),i}}}}}}var hr=1e9;function dr(t,e,n,r){return function(A){var l,c,u,g,h,d,f,p,I,C,m,y=A,E=Ar(),v=gr(t,e,n,r),B={point:w,lineStart:function(){B.point=x,c&&c.push(u=[]);C=!0,I=!1,f=p=NaN},lineEnd:function(){l&&(x(g,h),d&&I&&E.rejoin(),l.push(E.buffer()));B.point=w,I&&A.lineEnd()},polygonStart:function(){A=E,l=[],c=[],m=!0},polygonEnd:function(){A=y,l=o.merge(l);var e=function(t){for(var e=0,n=c.length,r=t[1],i=0;i<n;++i)for(var o,a=1,s=c[i],A=s.length,l=s[0];a<A;++a)o=s[a],l[1]<=r?o[1]>r&&Ot(l,o,t)>0&&++e:o[1]<=r&&Ot(l,o,t)<0&&--e,l=o;return 0!==e}([t,r]),n=m&&e,i=l.length;(n||i)&&(A.polygonStart(),n&&(A.lineStart(),b(null,null,1,A),A.lineEnd()),i&&rr(l,a,e,b,A),A.polygonEnd()),l=c=u=null}};function b(o,a,A,l){var c=0,u=0;if(null==o||(c=i(o,A))!==(u=i(a,A))||s(o,a)<0^A>0)do{l.point(0===c||3===c?t:n,c>1?r:e)}while((c=(c+A+4)%4)!==u);else l.point(a[0],a[1])}function Q(i,o){return t<=i&&i<=n&&e<=o&&o<=r}function w(t,e){Q(t,e)&&A.point(t,e)}function x(t,e){var n=Q(t=Math.max(-hr,Math.min(hr,t)),e=Math.max(-hr,Math.min(hr,e)));if(c&&u.push([t,e]),C)g=t,h=e,d=n,C=!1,n&&(A.lineStart(),A.point(t,e));else if(n&&I)A.point(t,e);else{var r={a:{x:f,y:p},b:{x:t,y:e}};v(r)?(I||(A.lineStart(),A.point(r.a.x,r.a.y)),A.point(r.b.x,r.b.y),n||A.lineEnd(),m=!1):n&&(A.lineStart(),A.point(t,e),m=!1)}f=t,p=e,I=n}return B};function i(r,i){return v(r[0]-t)<Mt?i>0?0:3:v(r[0]-n)<Mt?i>0?2:1:v(r[1]-e)<Mt?i>0?1:0:i>0?3:2}function a(t,e){return s(t.x,e.x)}function s(t,e){var n=i(t,1),r=i(e,1);return n!==r?n-r:0===n?e[1]-t[1]:1===n?t[0]-e[0]:2===n?t[1]-e[1]:e[0]-t[0]}}function fr(t){var e=0,n=St/3,r=Gr(t),i=r(e,n);return i.parallels=function(t){return arguments.length?r(e=t[0]*St/180,n=t[1]*St/180):[e/St*180,n/St*180]},i}function pr(t,e){var n=Math.sin(t),r=(n+Math.sin(e))/2,i=1+n*(2*r-n),o=Math.sqrt(i)/r;function a(t,e){var n=Math.sqrt(i-2*r*Math.sin(e))/r;return[n*Math.sin(t*=r),o-n*Math.cos(t)]}return a.invert=function(t,e){var n=o-e;return[Math.atan2(t,n)/r,Ht((i-(t*t+n*n)*r*r)/(2*r))]},a}o.geo.clipExtent=function(){var t,e,n,r,i,o,a={stream:function(t){return i&&(i.valid=!1),(i=o(t)).valid=!0,i},extent:function(s){return arguments.length?(o=dr(t=+s[0][0],e=+s[0][1],n=+s[1][0],r=+s[1][1]),i&&(i.valid=!1,i=null),a):[[t,e],[n,r]]}};return a.extent([[0,0],[960,500]])},(o.geo.conicEqualArea=function(){return fr(pr)}).raw=pr,o.geo.albers=function(){return o.geo.conicEqualArea().rotate([96,0]).center([-.6,38.7]).parallels([29.5,45.5]).scale(1070)},o.geo.albersUsa=function(){var t,e,n,r,i=o.geo.albers(),a=o.geo.conicEqualArea().rotate([154,0]).center([-2,58.5]).parallels([55,65]),s=o.geo.conicEqualArea().rotate([157,0]).center([-3,19.9]).parallels([8,18]),A={point:function(e,n){t=[e,n]}};function l(i){var o=i[0],a=i[1];return t=null,e(o,a),t||(n(o,a),t)||r(o,a),t}return l.invert=function(t){var e=i.scale(),n=i.translate(),r=(t[0]-n[0])/e,o=(t[1]-n[1])/e;return(o>=.12&&o<.234&&r>=-.425&&r<-.214?a:o>=.166&&o<.234&&r>=-.214&&r<-.115?s:i).invert(t)},l.stream=function(t){var e=i.stream(t),n=a.stream(t),r=s.stream(t);return{point:function(t,i){e.point(t,i),n.point(t,i),r.point(t,i)},sphere:function(){e.sphere(),n.sphere(),r.sphere()},lineStart:function(){e.lineStart(),n.lineStart(),r.lineStart()},lineEnd:function(){e.lineEnd(),n.lineEnd(),r.lineEnd()},polygonStart:function(){e.polygonStart(),n.polygonStart(),r.polygonStart()},polygonEnd:function(){e.polygonEnd(),n.polygonEnd(),r.polygonEnd()}}},l.precision=function(t){return arguments.length?(i.precision(t),a.precision(t),s.precision(t),l):i.precision()},l.scale=function(t){return arguments.length?(i.scale(t),a.scale(.35*t),s.scale(t),l.translate(i.translate())):i.scale()},l.translate=function(t){if(!arguments.length)return i.translate();var o=i.scale(),c=+t[0],u=+t[1];return e=i.translate(t).clipExtent([[c-.455*o,u-.238*o],[c+.455*o,u+.238*o]]).stream(A).point,n=a.translate([c-.307*o,u+.201*o]).clipExtent([[c-.425*o+Mt,u+.12*o+Mt],[c-.214*o-Mt,u+.234*o-Mt]]).stream(A).point,r=s.translate([c-.205*o,u+.212*o]).clipExtent([[c-.214*o+Mt,u+.166*o+Mt],[c-.115*o-Mt,u+.234*o-Mt]]).stream(A).point,l},l.scale(1070)};var Ir,Cr,mr,yr,Er,vr,Br={point:G,lineStart:G,lineEnd:G,polygonStart:function(){Cr=0,Br.lineStart=br},polygonEnd:function(){Br.lineStart=Br.lineEnd=Br.point=G,Ir+=v(Cr/2)}};function br(){var t,e,n,r;function i(t,e){Cr+=r*t-n*e,n=t,r=e}Br.point=function(o,a){Br.point=i,t=n=o,e=r=a},Br.lineEnd=function(){i(t,e)}}var Qr={point:function(t,e){t<mr&&(mr=t);t>Er&&(Er=t);e<yr&&(yr=e);e>vr&&(vr=e)},lineStart:G,lineEnd:G,polygonStart:G,polygonEnd:G};function wr(){var t=xr(4.5),e=[],n={point:r,lineStart:function(){n.point=i},lineEnd:a,polygonStart:function(){n.lineEnd=s},polygonEnd:function(){n.lineEnd=a,n.point=r},pointRadius:function(e){return t=xr(e),n},result:function(){if(e.length){var t=e.join("");return e=[],t}}};function r(n,r){e.push("M",n,",",r,t)}function i(t,r){e.push("M",t,",",r),n.point=o}function o(t,n){e.push("L",t,",",n)}function a(){n.point=r}function s(){e.push("Z")}return n}function xr(t){return"m0,"+t+"a"+t+","+t+" 0 1,1 0,"+-2*t+"a"+t+","+t+" 0 1,1 0,"+2*t+"z"}var _r,Dr={point:Mr,lineStart:kr,lineEnd:Sr,polygonStart:function(){Dr.lineStart=Lr},polygonEnd:function(){Dr.point=Mr,Dr.lineStart=kr,Dr.lineEnd=Sr}};function Mr(t,e){Dn+=t,Mn+=e,++kn}function kr(){var t,e;function n(n,r){var i=n-t,o=r-e,a=Math.sqrt(i*i+o*o);Sn+=a*(t+n)/2,Ln+=a*(e+r)/2,Rn+=a,Mr(t=n,e=r)}Dr.point=function(r,i){Dr.point=n,Mr(t=r,e=i)}}function Sr(){Dr.point=Mr}function Lr(){var t,e,n,r;function i(t,e){var i=t-n,o=e-r,a=Math.sqrt(i*i+o*o);Sn+=a*(n+t)/2,Ln+=a*(r+e)/2,Rn+=a,Nn+=(a=r*t-n*e)*(n+t),Fn+=a*(r+e),Tn+=3*a,Mr(n=t,r=e)}Dr.point=function(o,a){Dr.point=i,Mr(t=n=o,e=r=a)},Dr.lineEnd=function(){i(t,e)}}function Rr(t){var e=4.5,n={point:r,lineStart:function(){n.point=i},lineEnd:a,polygonStart:function(){n.lineEnd=s},polygonEnd:function(){n.lineEnd=a,n.point=r},pointRadius:function(t){return e=t,n},result:G};function r(n,r){t.moveTo(n+e,r),t.arc(n,r,e,0,Lt)}function i(e,r){t.moveTo(e,r),n.point=o}function o(e,n){t.lineTo(e,n)}function a(){n.point=r}function s(){t.closePath()}return n}function Nr(t){var e=.5,n=Math.cos(30*Ft),r=16;function i(e){return(r?function(e){var n,i,a,s,A,l,c,u,g,h,d,f,p={point:I,lineStart:C,lineEnd:y,polygonStart:function(){e.polygonStart(),p.lineStart=E},polygonEnd:function(){e.polygonEnd(),p.lineStart=C}};function I(n,r){n=t(n,r),e.point(n[0],n[1])}function C(){u=NaN,p.point=m,e.lineStart()}function m(n,i){var a=Hn([n,i]),s=t(n,i);o(u,g,c,h,d,f,u=s[0],g=s[1],c=n,h=a[0],d=a[1],f=a[2],r,e),e.point(u,g)}function y(){p.point=I,e.lineEnd()}function E(){C(),p.point=v,p.lineEnd=B}function v(t,e){m(n=t,e),i=u,a=g,s=h,A=d,l=f,p.point=m}function B(){o(u,g,c,h,d,f,i,a,n,s,A,l,r,e),p.lineEnd=y,y()}return p}:function(e){return Ur(e,function(n,r){n=t(n,r),e.point(n[0],n[1])})})(e)}function o(r,i,a,s,A,l,c,u,g,h,d,f,p,I){var C=c-r,m=u-i,y=C*C+m*m;if(y>4*e&&p--){var E=s+h,B=A+d,b=l+f,Q=Math.sqrt(E*E+B*B+b*b),w=Math.asin(b/=Q),x=v(v(b)-1)<Mt||v(a-g)<Mt?(a+g)/2:Math.atan2(B,E),_=t(x,w),D=_[0],M=_[1],k=D-r,S=M-i,L=m*k-C*S;(L*L/y>e||v((C*k+m*S)/y-.5)>.3||s*h+A*d+l*f<n)&&(o(r,i,a,s,A,l,D,M,x,E/=Q,B/=Q,b,p,I),I.point(D,M),o(D,M,x,E,B,b,c,u,g,h,d,f,p,I))}}return i.precision=function(t){return arguments.length?(r=(e=t*t)>0&&16,i):Math.sqrt(e)},i}function Fr(t){var e=Nr(function(e,n){return t([e*Tt,n*Tt])});return function(t){return Hr(e(t))}}function Tr(t){this.stream=t}function Ur(t,e){return{point:e,sphere:function(){t.sphere()},lineStart:function(){t.lineStart()},lineEnd:function(){t.lineEnd()},polygonStart:function(){t.polygonStart()},polygonEnd:function(){t.polygonEnd()}}}function Or(t){return Gr(function(){return t})()}function Gr(t){var e,n,r,i,a,s,A=Nr(function(t,n){return[(t=e(t,n))[0]*l+i,a-t[1]*l]}),l=150,c=480,u=250,g=0,h=0,d=0,f=0,p=0,I=cr,C=F,m=null,y=null;function E(t){return[(t=r(t[0]*Ft,t[1]*Ft))[0]*l+i,a-t[1]*l]}function v(t){return(t=r.invert((t[0]-i)/l,(a-t[1])/l))&&[t[0]*Tt,t[1]*Tt]}function B(){r=er(n=Pr(d,f,p),e);var t=e(g,h);return i=c-t[0]*l,a=u+t[1]*l,b()}function b(){return s&&(s.valid=!1,s=null),E}return E.stream=function(t){return s&&(s.valid=!1),(s=Hr(I(n,A(C(t))))).valid=!0,s},E.clipAngle=function(t){return arguments.length?(I=null==t?(m=t,cr):ur((m=+t)*Ft),b()):m},E.clipExtent=function(t){return arguments.length?(y=t,C=t?dr(t[0][0],t[0][1],t[1][0],t[1][1]):F,b()):y},E.scale=function(t){return arguments.length?(l=+t,B()):l},E.translate=function(t){return arguments.length?(c=+t[0],u=+t[1],B()):[c,u]},E.center=function(t){return arguments.length?(g=t[0]%360*Ft,h=t[1]%360*Ft,B()):[g*Tt,h*Tt]},E.rotate=function(t){return arguments.length?(d=t[0]%360*Ft,f=t[1]%360*Ft,p=t.length>2?t[2]%360*Ft:0,B()):[d*Tt,f*Tt,p*Tt]},o.rebind(E,A,"precision"),function(){return e=t.apply(this,arguments),E.invert=e.invert&&v,B()}}function Hr(t){return Ur(t,function(e,n){t.point(e*Ft,n*Ft)})}function jr(t,e){return[t,e]}function Yr(t,e){return[t>St?t-Lt:t<-St?t+Lt:t,e]}function Pr(t,e,n){return t?e||n?er(zr(t),Jr(e,n)):zr(t):e||n?Jr(e,n):Yr}function Kr(t){return function(e,n){return[(e+=t)>St?e-Lt:e<-St?e+Lt:e,n]}}function zr(t){var e=Kr(t);return e.invert=Kr(-t),e}function Jr(t,e){var n=Math.cos(t),r=Math.sin(t),i=Math.cos(e),o=Math.sin(e);function a(t,e){var a=Math.cos(e),s=Math.cos(t)*a,A=Math.sin(t)*a,l=Math.sin(e),c=l*n+s*r;return[Math.atan2(A*i-c*o,s*n-l*r),Ht(c*i+A*o)]}return a.invert=function(t,e){var a=Math.cos(e),s=Math.cos(t)*a,A=Math.sin(t)*a,l=Math.sin(e),c=l*i-A*o;return[Math.atan2(A*i+l*o,s*n+c*r),Ht(c*n-s*r)]},a}function qr(t,e){var n=Math.cos(t),r=Math.sin(t);return function(i,o,a,s){var A=a*e;null!=i?(i=Vr(n,i),o=Vr(n,o),(a>0?i<o:i>o)&&(i+=a*Lt)):(i=t+a*Lt,o=t-.5*A);for(var l,c=i;a>0?c>o:c<o;c-=A)s.point((l=Jn([n,-r*Math.cos(c),-r*Math.sin(c)]))[0],l[1])}}function Vr(t,e){var n=Hn(e);n[0]-=t,zn(n);var r=Gt(-n[1]);return((-n[2]<0?-r:r)+2*Math.PI-Mt)%(2*Math.PI)}function Wr(t,e,n){var r=o.range(t,e-Mt,n).concat(e);return function(t){return r.map(function(e){return[t,e]})}}function Xr(t,e,n){var r=o.range(t,e-Mt,n).concat(e);return function(t){return r.map(function(e){return[e,t]})}}function Zr(t){return t.source}function $r(t){return t.target}o.geo.path=function(){var t,e,n,r,i,a=4.5;function s(t){return t&&("function"==typeof a&&r.pointRadius(+a.apply(this,arguments)),i&&i.valid||(i=n(r)),o.geo.stream(t,i)),r.result()}function A(){return i=null,s}return s.area=function(t){return Ir=0,o.geo.stream(t,n(Br)),Ir},s.centroid=function(t){return Dn=Mn=kn=Sn=Ln=Rn=Nn=Fn=Tn=0,o.geo.stream(t,n(Dr)),Tn?[Nn/Tn,Fn/Tn]:Rn?[Sn/Rn,Ln/Rn]:kn?[Dn/kn,Mn/kn]:[NaN,NaN]},s.bounds=function(t){return Er=vr=-(mr=yr=1/0),o.geo.stream(t,n(Qr)),[[mr,yr],[Er,vr]]},s.projection=function(e){return arguments.length?(n=(t=e)?e.stream||Fr(e):F,A()):t},s.context=function(t){return arguments.length?(r=null==(e=t)?new wr:new Rr(t),"function"!=typeof a&&r.pointRadius(a),A()):e},s.pointRadius=function(t){return arguments.length?(a="function"==typeof t?t:(r.pointRadius(+t),+t),s):a},s.projection(o.geo.albersUsa()).context(null)},o.geo.transform=function(t){return{stream:function(e){var n=new Tr(e);for(var r in t)n[r]=t[r];return n}}},Tr.prototype={point:function(t,e){this.stream.point(t,e)},sphere:function(){this.stream.sphere()},lineStart:function(){this.stream.lineStart()},lineEnd:function(){this.stream.lineEnd()},polygonStart:function(){this.stream.polygonStart()},polygonEnd:function(){this.stream.polygonEnd()}},o.geo.projection=Or,o.geo.projectionMutator=Gr,(o.geo.equirectangular=function(){return Or(jr)}).raw=jr.invert=jr,o.geo.rotation=function(t){function e(e){return(e=t(e[0]*Ft,e[1]*Ft))[0]*=Tt,e[1]*=Tt,e}return t=Pr(t[0]%360*Ft,t[1]*Ft,t.length>2?t[2]*Ft:0),e.invert=function(e){return(e=t.invert(e[0]*Ft,e[1]*Ft))[0]*=Tt,e[1]*=Tt,e},e},Yr.invert=jr,o.geo.circle=function(){var t,e,n=[0,0],r=6;function i(){var t="function"==typeof n?n.apply(this,arguments):n,r=Pr(-t[0]*Ft,-t[1]*Ft,0).invert,i=[];return e(null,null,1,{point:function(t,e){i.push(t=r(t,e)),t[0]*=Tt,t[1]*=Tt}}),{type:"Polygon",coordinates:[i]}}return i.origin=function(t){return arguments.length?(n=t,i):n},i.angle=function(n){return arguments.length?(e=qr((t=+n)*Ft,r*Ft),i):t},i.precision=function(n){return arguments.length?(e=qr(t*Ft,(r=+n)*Ft),i):r},i.angle(90)},o.geo.distance=function(t,e){var n,r=(e[0]-t[0])*Ft,i=t[1]*Ft,o=e[1]*Ft,a=Math.sin(r),s=Math.cos(r),A=Math.sin(i),l=Math.cos(i),c=Math.sin(o),u=Math.cos(o);return Math.atan2(Math.sqrt((n=u*a)*n+(n=l*c-A*u*s)*n),A*c+l*u*s)},o.geo.graticule=function(){var t,e,n,r,i,a,s,A,l,c,u,g,h=10,d=h,f=90,p=360,I=2.5;function C(){return{type:"MultiLineString",coordinates:m()}}function m(){return o.range(Math.ceil(r/f)*f,n,f).map(u).concat(o.range(Math.ceil(A/p)*p,s,p).map(g)).concat(o.range(Math.ceil(e/h)*h,t,h).filter(function(t){return v(t%f)>Mt}).map(l)).concat(o.range(Math.ceil(a/d)*d,i,d).filter(function(t){return v(t%p)>Mt}).map(c))}return C.lines=function(){return m().map(function(t){return{type:"LineString",coordinates:t}})},C.outline=function(){return{type:"Polygon",coordinates:[u(r).concat(g(s).slice(1),u(n).reverse().slice(1),g(A).reverse().slice(1))]}},C.extent=function(t){return arguments.length?C.majorExtent(t).minorExtent(t):C.minorExtent()},C.majorExtent=function(t){return arguments.length?(r=+t[0][0],n=+t[1][0],A=+t[0][1],s=+t[1][1],r>n&&(t=r,r=n,n=t),A>s&&(t=A,A=s,s=t),C.precision(I)):[[r,A],[n,s]]},C.minorExtent=function(n){return arguments.length?(e=+n[0][0],t=+n[1][0],a=+n[0][1],i=+n[1][1],e>t&&(n=e,e=t,t=n),a>i&&(n=a,a=i,i=n),C.precision(I)):[[e,a],[t,i]]},C.step=function(t){return arguments.length?C.majorStep(t).minorStep(t):C.minorStep()},C.majorStep=function(t){return arguments.length?(f=+t[0],p=+t[1],C):[f,p]},C.minorStep=function(t){return arguments.length?(h=+t[0],d=+t[1],C):[h,d]},C.precision=function(o){return arguments.length?(I=+o,l=Wr(a,i,90),c=Xr(e,t,I),u=Wr(A,s,90),g=Xr(r,n,I),C):I},C.majorExtent([[-180,-90+Mt],[180,90-Mt]]).minorExtent([[-180,-80-Mt],[180,80+Mt]])},o.geo.greatArc=function(){var t,e,n=Zr,r=$r;function i(){return{type:"LineString",coordinates:[t||n.apply(this,arguments),e||r.apply(this,arguments)]}}return i.distance=function(){return o.geo.distance(t||n.apply(this,arguments),e||r.apply(this,arguments))},i.source=function(e){return arguments.length?(n=e,t="function"==typeof e?null:e,i):n},i.target=function(t){return arguments.length?(r=t,e="function"==typeof t?null:t,i):r},i.precision=function(){return arguments.length?i:0},i},o.geo.interpolate=function(t,e){return function(t,e,n,r){var i=Math.cos(e),o=Math.sin(e),a=Math.cos(r),s=Math.sin(r),A=i*Math.cos(t),l=i*Math.sin(t),c=a*Math.cos(n),u=a*Math.sin(n),g=2*Math.asin(Math.sqrt(Yt(r-e)+i*a*Yt(n-t))),h=1/Math.sin(g),d=g?function(t){var e=Math.sin(t*=g)*h,n=Math.sin(g-t)*h,r=n*A+e*c,i=n*l+e*u,a=n*o+e*s;return[Math.atan2(i,r)*Tt,Math.atan2(a,Math.sqrt(r*r+i*i))*Tt]}:function(){return[t*Tt,e*Tt]};return d.distance=g,d}(t[0]*Ft,t[1]*Ft,e[0]*Ft,e[1]*Ft)},o.geo.length=function(t){return _r=0,o.geo.stream(t,ti),_r};var ti={sphere:G,point:G,lineStart:function(){var t,e,n;function r(r,i){var o=Math.sin(i*=Ft),a=Math.cos(i),s=v((r*=Ft)-t),A=Math.cos(s);_r+=Math.atan2(Math.sqrt((s=a*Math.sin(s))*s+(s=n*o-e*a*A)*s),e*o+n*a*A),t=r,e=o,n=a}ti.point=function(i,o){t=i*Ft,e=Math.sin(o*=Ft),n=Math.cos(o),ti.point=r},ti.lineEnd=function(){ti.point=ti.lineEnd=G}},lineEnd:G,polygonStart:G,polygonEnd:G};function ei(t,e){function n(e,n){var r=Math.cos(e),i=Math.cos(n),o=t(r*i);return[o*i*Math.sin(e),o*Math.sin(n)]}return n.invert=function(t,n){var r=Math.sqrt(t*t+n*n),i=e(r),o=Math.sin(i),a=Math.cos(i);return[Math.atan2(t*o,r*a),Math.asin(r&&n*o/r)]},n}var ni=ei(function(t){return Math.sqrt(2/(1+t))},function(t){return 2*Math.asin(t/2)});(o.geo.azimuthalEqualArea=function(){return Or(ni)}).raw=ni;var ri=ei(function(t){var e=Math.acos(t);return e&&e/Math.sin(e)},F);function ii(t,e){var n=Math.cos(t),r=function(t){return Math.tan(St/4+t/2)},i=t===e?Math.sin(t):Math.log(n/Math.cos(e))/Math.log(r(e)/r(t)),o=n*Math.pow(r(t),i)/i;if(!i)return si;function a(t,e){o>0?e<-Nt+Mt&&(e=-Nt+Mt):e>Nt-Mt&&(e=Nt-Mt);var n=o/Math.pow(r(e),i);return[n*Math.sin(i*t),o-n*Math.cos(i*t)]}return a.invert=function(t,e){var n=o-e,r=Ut(i)*Math.sqrt(t*t+n*n);return[Math.atan2(t,n)/i,2*Math.atan(Math.pow(o/r,1/i))-Nt]},a}function oi(t,e){var n=Math.cos(t),r=t===e?Math.sin(t):(n-Math.cos(e))/(e-t),i=n/r+t;if(v(r)<Mt)return jr;function o(t,e){var n=i-e;return[n*Math.sin(r*t),i-n*Math.cos(r*t)]}return o.invert=function(t,e){var n=i-e;return[Math.atan2(t,n)/r,i-Ut(r)*Math.sqrt(t*t+n*n)]},o}(o.geo.azimuthalEquidistant=function(){return Or(ri)}).raw=ri,(o.geo.conicConformal=function(){return fr(ii)}).raw=ii,(o.geo.conicEquidistant=function(){return fr(oi)}).raw=oi;var ai=ei(function(t){return 1/t},Math.atan);function si(t,e){return[t,Math.log(Math.tan(St/4+e/2))]}function Ai(t){var e,n=Or(t),r=n.scale,i=n.translate,o=n.clipExtent;return n.scale=function(){var t=r.apply(n,arguments);return t===n?e?n.clipExtent(null):n:t},n.translate=function(){var t=i.apply(n,arguments);return t===n?e?n.clipExtent(null):n:t},n.clipExtent=function(t){var a=o.apply(n,arguments);if(a===n){if(e=null==t){var s=St*r(),A=i();o([[A[0]-s,A[1]-s],[A[0]+s,A[1]+s]])}}else e&&(a=null);return a},n.clipExtent(null)}(o.geo.gnomonic=function(){return Or(ai)}).raw=ai,si.invert=function(t,e){return[t,2*Math.atan(Math.exp(e))-Nt]},(o.geo.mercator=function(){return Ai(si)}).raw=si;var li=ei(function(){return 1},Math.asin);(o.geo.orthographic=function(){return Or(li)}).raw=li;var ci=ei(function(t){return 1/(1+t)},function(t){return 2*Math.atan(t)});function ui(t,e){return[Math.log(Math.tan(St/4+e/2)),-t]}function gi(t){return t[0]}function hi(t){return t[1]}function di(t){for(var e=t.length,n=[0,1],r=2,i=2;i<e;i++){for(;r>1&&Ot(t[n[r-2]],t[n[r-1]],t[i])<=0;)--r;n[r++]=i}return n.slice(0,r)}function fi(t,e){return t[0]-e[0]||t[1]-e[1]}(o.geo.stereographic=function(){return Or(ci)}).raw=ci,ui.invert=function(t,e){return[-e,2*Math.atan(Math.exp(t))-Nt]},(o.geo.transverseMercator=function(){var t=Ai(ui),e=t.center,n=t.rotate;return t.center=function(t){return t?e([-t[1],t[0]]):[(t=e())[1],-t[0]]},t.rotate=function(t){return t?n([t[0],t[1],t.length>2?t[2]+90:90]):[(t=n())[0],t[1],t[2]-90]},n([0,0,90])}).raw=ui,o.geom={},o.geom.hull=function(t){var e=gi,n=hi;if(arguments.length)return r(t);function r(t){if(t.length<3)return[];var r,i=Be(e),o=Be(n),a=t.length,s=[],A=[];for(r=0;r<a;r++)s.push([+i.call(this,t[r],r),+o.call(this,t[r],r),r]);for(s.sort(fi),r=0;r<a;r++)A.push([s[r][0],-s[r][1]]);var l=di(s),c=di(A),u=c[0]===l[0],g=c[c.length-1]===l[l.length-1],h=[];for(r=l.length-1;r>=0;--r)h.push(t[s[l[r]][2]]);for(r=+u;r<c.length-g;++r)h.push(t[s[c[r]][2]]);return h}return r.x=function(t){return arguments.length?(e=t,r):e},r.y=function(t){return arguments.length?(n=t,r):n},r},o.geom.polygon=function(t){return J(t,pi),t};var pi=o.geom.polygon.prototype=[];function Ii(t,e,n){return(n[0]-e[0])*(t[1]-e[1])<(n[1]-e[1])*(t[0]-e[0])}function Ci(t,e,n,r){var i=t[0],o=n[0],a=e[0]-i,s=r[0]-o,A=t[1],l=n[1],c=e[1]-A,u=r[1]-l,g=(s*(A-l)-u*(i-o))/(u*a-s*c);return[i+g*a,A+g*c]}function mi(t){var e=t[0],n=t[t.length-1];return!(e[0]-n[0]||e[1]-n[1])}pi.area=function(){for(var t,e=-1,n=this.length,r=this[n-1],i=0;++e<n;)t=r,r=this[e],i+=t[1]*r[0]-t[0]*r[1];return.5*i},pi.centroid=function(t){var e,n,r=-1,i=this.length,o=0,a=0,s=this[i-1];for(arguments.length||(t=-1/(6*this.area()));++r<i;)e=s,s=this[r],n=e[0]*s[1]-s[0]*e[1],o+=(e[0]+s[0])*n,a+=(e[1]+s[1])*n;return[o*t,a*t]},pi.clip=function(t){for(var e,n,r,i,o,a,s=mi(t),A=-1,l=this.length-mi(this),c=this[l-1];++A<l;){for(e=t.slice(),t.length=0,i=this[A],o=e[(r=e.length-s)-1],n=-1;++n<r;)Ii(a=e[n],c,i)?(Ii(o,c,i)||t.push(Ci(o,a,c,i)),t.push(a)):Ii(o,c,i)&&t.push(Ci(o,a,c,i)),o=a;s&&t.push(t[0]),c=i}return t};var yi,Ei,vi,Bi,bi,Qi=[],wi=[];function xi(){Ki(this),this.edge=this.site=this.circle=null}function _i(t){var e=Qi.pop()||new xi;return e.site=t,e}function Di(t){Ui(t),vi.remove(t),Qi.push(t),Ki(t)}function Mi(t){var e=t.circle,n=e.x,r=e.cy,i={x:n,y:r},o=t.P,a=t.N,s=[t];Di(t);for(var A=o;A.circle&&v(n-A.circle.x)<Mt&&v(r-A.circle.cy)<Mt;)o=A.P,s.unshift(A),Di(A),A=o;s.unshift(A),Ui(A);for(var l=a;l.circle&&v(n-l.circle.x)<Mt&&v(r-l.circle.cy)<Mt;)a=l.N,s.push(l),Di(l),l=a;s.push(l),Ui(l);var c,u=s.length;for(c=1;c<u;++c)l=s[c],A=s[c-1],ji(l.edge,A.site,l.site,i);A=s[0],(l=s[u-1]).edge=Hi(A.site,l.site,null,i),Ti(A),Ti(l)}function ki(t){for(var e,n,r,i,o=t.x,a=t.y,s=vi._;s;)if((r=Si(s,a)-o)>Mt)s=s.L;else{if(!((i=o-Li(s,a))>Mt)){r>-Mt?(e=s.P,n=s):i>-Mt?(e=s,n=s.N):e=n=s;break}if(!s.R){e=s;break}s=s.R}var A=_i(t);if(vi.insert(e,A),e||n){if(e===n)return Ui(e),n=_i(e.site),vi.insert(A,n),A.edge=n.edge=Hi(e.site,A.site),Ti(e),void Ti(n);if(n){Ui(e),Ui(n);var l=e.site,c=l.x,u=l.y,g=t.x-c,h=t.y-u,d=n.site,f=d.x-c,p=d.y-u,I=2*(g*p-h*f),C=g*g+h*h,m=f*f+p*p,y={x:(p*C-h*m)/I+c,y:(g*m-f*C)/I+u};ji(n.edge,l,d,y),A.edge=Hi(l,t,null,y),n.edge=Hi(t,d,null,y),Ti(e),Ti(n)}else A.edge=Hi(e.site,A.site)}}function Si(t,e){var n=t.site,r=n.x,i=n.y,o=i-e;if(!o)return r;var a=t.P;if(!a)return-1/0;var s=(n=a.site).x,A=n.y,l=A-e;if(!l)return s;var c=s-r,u=1/o-1/l,g=c/l;return u?(-g+Math.sqrt(g*g-2*u*(c*c/(-2*l)-A+l/2+i-o/2)))/u+r:(r+s)/2}function Li(t,e){var n=t.N;if(n)return Si(n,e);var r=t.site;return r.y===e?r.x:1/0}function Ri(t){this.site=t,this.edges=[]}function Ni(t,e){return e.angle-t.angle}function Fi(){Ki(this),this.x=this.y=this.arc=this.site=this.cy=null}function Ti(t){var e=t.P,n=t.N;if(e&&n){var r=e.site,i=t.site,o=n.site;if(r!==o){var a=i.x,s=i.y,A=r.x-a,l=r.y-s,c=o.x-a,u=2*(A*(p=o.y-s)-l*c);if(!(u>=-kt)){var g=A*A+l*l,h=c*c+p*p,d=(p*g-l*h)/u,f=(A*h-c*g)/u,p=f+s,I=wi.pop()||new Fi;I.arc=t,I.site=i,I.x=d+a,I.y=p+Math.sqrt(d*d+f*f),I.cy=p,t.circle=I;for(var C=null,m=bi._;m;)if(I.y<m.y||I.y===m.y&&I.x<=m.x){if(!m.L){C=m.P;break}m=m.L}else{if(!m.R){C=m;break}m=m.R}bi.insert(C,I),C||(Bi=I)}}}}function Ui(t){var e=t.circle;e&&(e.P||(Bi=e.N),bi.remove(e),wi.push(e),Ki(e),t.circle=null)}function Oi(t,e){var n=t.b;if(n)return!0;var r,i,o=t.a,a=e[0][0],s=e[1][0],A=e[0][1],l=e[1][1],c=t.l,u=t.r,g=c.x,h=c.y,d=u.x,f=u.y,p=(g+d)/2,I=(h+f)/2;if(f===h){if(p<a||p>=s)return;if(g>d){if(o){if(o.y>=l)return}else o={x:p,y:A};n={x:p,y:l}}else{if(o){if(o.y<A)return}else o={x:p,y:l};n={x:p,y:A}}}else if(i=I-(r=(g-d)/(f-h))*p,r<-1||r>1)if(g>d){if(o){if(o.y>=l)return}else o={x:(A-i)/r,y:A};n={x:(l-i)/r,y:l}}else{if(o){if(o.y<A)return}else o={x:(l-i)/r,y:l};n={x:(A-i)/r,y:A}}else if(h<f){if(o){if(o.x>=s)return}else o={x:a,y:r*a+i};n={x:s,y:r*s+i}}else{if(o){if(o.x<a)return}else o={x:s,y:r*s+i};n={x:a,y:r*a+i}}return t.a=o,t.b=n,!0}function Gi(t,e){this.l=t,this.r=e,this.a=this.b=null}function Hi(t,e,n,r){var i=new Gi(t,e);return yi.push(i),n&&ji(i,t,e,n),r&&ji(i,e,t,r),Ei[t.i].edges.push(new Yi(i,t,e)),Ei[e.i].edges.push(new Yi(i,e,t)),i}function ji(t,e,n,r){t.a||t.b?t.l===n?t.b=r:t.a=r:(t.a=r,t.l=e,t.r=n)}function Yi(t,e,n){var r=t.a,i=t.b;this.edge=t,this.site=e,this.angle=n?Math.atan2(n.y-e.y,n.x-e.x):t.l===e?Math.atan2(i.x-r.x,r.y-i.y):Math.atan2(r.x-i.x,i.y-r.y)}function Pi(){this._=null}function Ki(t){t.U=t.C=t.L=t.R=t.P=t.N=null}function zi(t,e){var n=e,r=e.R,i=n.U;i?i.L===n?i.L=r:i.R=r:t._=r,r.U=i,n.U=r,n.R=r.L,n.R&&(n.R.U=n),r.L=n}function Ji(t,e){var n=e,r=e.L,i=n.U;i?i.L===n?i.L=r:i.R=r:t._=r,r.U=i,n.U=r,n.L=r.R,n.L&&(n.L.U=n),r.R=n}function qi(t){for(;t.L;)t=t.L;return t}function Vi(t,e){var n,r,i,o=t.sort(Wi).pop();for(yi=[],Ei=new Array(t.length),vi=new Pi,bi=new Pi;;)if(i=Bi,o&&(!i||o.y<i.y||o.y===i.y&&o.x<i.x))o.x===n&&o.y===r||(Ei[o.i]=new Ri(o),ki(o),n=o.x,r=o.y),o=t.pop();else{if(!i)break;Mi(i.arc)}e&&(function(t){for(var e,n=yi,r=gr(t[0][0],t[0][1],t[1][0],t[1][1]),i=n.length;i--;)(!Oi(e=n[i],t)||!r(e)||v(e.a.x-e.b.x)<Mt&&v(e.a.y-e.b.y)<Mt)&&(e.a=e.b=null,n.splice(i,1))}(e),function(t){for(var e,n,r,i,o,a,s,A,l,c,u=t[0][0],g=t[1][0],h=t[0][1],d=t[1][1],f=Ei,p=f.length;p--;)if((o=f[p])&&o.prepare())for(A=(s=o.edges).length,a=0;a<A;)r=(c=s[a].end()).x,i=c.y,e=(l=s[++a%A].start()).x,n=l.y,(v(r-e)>Mt||v(i-n)>Mt)&&(s.splice(a,0,new Yi((I=o.site,C=c,m=v(r-u)<Mt&&d-i>Mt?{x:u,y:v(e-u)<Mt?n:d}:v(i-d)<Mt&&g-r>Mt?{x:v(n-d)<Mt?e:g,y:d}:v(r-g)<Mt&&i-h>Mt?{x:g,y:v(e-g)<Mt?n:h}:v(i-h)<Mt&&r-u>Mt?{x:v(n-h)<Mt?e:u,y:h}:null,y=void 0,y=new Gi(I,null),y.a=C,y.b=m,yi.push(y),y),o.site,null)),++A);var I,C,m,y}(e));var a={cells:Ei,edges:yi};return vi=bi=yi=Ei=null,a}function Wi(t,e){return e.y-t.y||e.x-t.x}Ri.prototype.prepare=function(){for(var t,e=this.edges,n=e.length;n--;)(t=e[n].edge).b&&t.a||e.splice(n,1);return e.sort(Ni),e.length},Yi.prototype={start:function(){return this.edge.l===this.site?this.edge.a:this.edge.b},end:function(){return this.edge.l===this.site?this.edge.b:this.edge.a}},Pi.prototype={insert:function(t,e){var n,r,i;if(t){if(e.P=t,e.N=t.N,t.N&&(t.N.P=e),t.N=e,t.R){for(t=t.R;t.L;)t=t.L;t.L=e}else t.R=e;n=t}else this._?(t=qi(this._),e.P=null,e.N=t,t.P=t.L=e,n=t):(e.P=e.N=null,this._=e,n=null);for(e.L=e.R=null,e.U=n,e.C=!0,t=e;n&&n.C;)n===(r=n.U).L?(i=r.R)&&i.C?(n.C=i.C=!1,r.C=!0,t=r):(t===n.R&&(zi(this,n),n=(t=n).U),n.C=!1,r.C=!0,Ji(this,r)):(i=r.L)&&i.C?(n.C=i.C=!1,r.C=!0,t=r):(t===n.L&&(Ji(this,n),n=(t=n).U),n.C=!1,r.C=!0,zi(this,r)),n=t.U;this._.C=!1},remove:function(t){t.N&&(t.N.P=t.P),t.P&&(t.P.N=t.N),t.N=t.P=null;var e,n,r,i=t.U,o=t.L,a=t.R;if(n=o?a?qi(a):o:a,i?i.L===t?i.L=n:i.R=n:this._=n,o&&a?(r=n.C,n.C=t.C,n.L=o,o.U=n,n!==a?(i=n.U,n.U=t.U,t=n.R,i.L=t,n.R=a,a.U=n):(n.U=i,i=n,t=n.R)):(r=t.C,t=n),t&&(t.U=i),!r)if(t&&t.C)t.C=!1;else{do{if(t===this._)break;if(t===i.L){if((e=i.R).C&&(e.C=!1,i.C=!0,zi(this,i),e=i.R),e.L&&e.L.C||e.R&&e.R.C){e.R&&e.R.C||(e.L.C=!1,e.C=!0,Ji(this,e),e=i.R),e.C=i.C,i.C=e.R.C=!1,zi(this,i),t=this._;break}}else if((e=i.L).C&&(e.C=!1,i.C=!0,Ji(this,i),e=i.L),e.L&&e.L.C||e.R&&e.R.C){e.L&&e.L.C||(e.R.C=!1,e.C=!0,zi(this,e),e=i.L),e.C=i.C,i.C=e.L.C=!1,Ji(this,i),t=this._;break}e.C=!0,t=i,i=i.U}while(!t.C);t&&(t.C=!1)}}},o.geom.voronoi=function(t){var e=gi,n=hi,r=e,i=n,o=Xi;if(t)return a(t);function a(t){var e=new Array(t.length),n=o[0][0],r=o[0][1],i=o[1][0],a=o[1][1];return Vi(s(t),o).cells.forEach(function(o,s){var A=o.edges,l=o.site;(e[s]=A.length?A.map(function(t){var e=t.start();return[e.x,e.y]}):l.x>=n&&l.x<=i&&l.y>=r&&l.y<=a?[[n,a],[i,a],[i,r],[n,r]]:[]).point=t[s]}),e}function s(t){return t.map(function(t,e){return{x:Math.round(r(t,e)/Mt)*Mt,y:Math.round(i(t,e)/Mt)*Mt,i:e}})}return a.links=function(t){return Vi(s(t)).edges.filter(function(t){return t.l&&t.r}).map(function(e){return{source:t[e.l.i],target:t[e.r.i]}})},a.triangles=function(t){var e=[];return Vi(s(t)).cells.forEach(function(n,r){for(var i,o,a,s,A=n.site,l=n.edges.sort(Ni),c=-1,u=l.length,g=l[u-1].edge,h=g.l===A?g.r:g.l;++c<u;)g,i=h,h=(g=l[c].edge).l===A?g.r:g.l,r<i.i&&r<h.i&&(a=i,s=h,((o=A).x-s.x)*(a.y-o.y)-(o.x-a.x)*(s.y-o.y)<0)&&e.push([t[r],t[i.i],t[h.i]])}),e},a.x=function(t){return arguments.length?(r=Be(e=t),a):e},a.y=function(t){return arguments.length?(i=Be(n=t),a):n},a.clipExtent=function(t){return arguments.length?(o=null==t?Xi:t,a):o===Xi?null:o},a.size=function(t){return arguments.length?a.clipExtent(t&&[[0,0],t]):o===Xi?null:o&&o[1]},a};var Xi=[[-1e6,-1e6],[1e6,1e6]];function Zi(t){return t.x}function $i(t){return t.y}function to(t,e,n,r,i,o){if(!t(e,n,r,i,o)){var a=.5*(n+i),s=.5*(r+o),A=e.nodes;A[0]&&to(t,A[0],n,r,a,s),A[1]&&to(t,A[1],a,r,i,s),A[2]&&to(t,A[2],n,s,a,o),A[3]&&to(t,A[3],a,s,i,o)}}function eo(t,e,n,r,i,o,a){var s,A=1/0;return function t(l,c,u,g,h){if(!(c>o||u>a||g<r||h<i)){if(d=l.point){var d,f=e-l.x,p=n-l.y,I=f*f+p*p;if(I<A){var C=Math.sqrt(A=I);r=e-C,i=n-C,o=e+C,a=n+C,s=d}}for(var m=l.nodes,y=.5*(c+g),E=.5*(u+h),v=(n>=E)<<1|e>=y,B=v+4;v<B;++v)if(l=m[3&v])switch(3&v){case 0:t(l,c,u,y,E);break;case 1:t(l,y,u,g,E);break;case 2:t(l,c,E,y,h);break;case 3:t(l,y,E,g,h)}}}(t,r,i,o,a),s}function no(t,e){t=o.rgb(t),e=o.rgb(e);var n=t.r,r=t.g,i=t.b,a=e.r-n,s=e.g-r,A=e.b-i;return function(t){return"#"+pe(Math.round(n+a*t))+pe(Math.round(r+s*t))+pe(Math.round(i+A*t))}}function ro(t,e){var n,r={},i={};for(n in t)n in e?r[n]=Ao(t[n],e[n]):i[n]=t[n];for(n in e)n in t||(i[n]=e[n]);return function(t){for(n in r)i[n]=r[n](t);return i}}function io(t,e){return t=+t,e=+e,function(n){return t*(1-n)+e*n}}function oo(t,e){var n,r,i,o=ao.lastIndex=so.lastIndex=0,a=-1,s=[],A=[];for(t+="",e+="";(n=ao.exec(t))&&(r=so.exec(e));)(i=r.index)>o&&(i=e.slice(o,i),s[a]?s[a]+=i:s[++a]=i),(n=n[0])===(r=r[0])?s[a]?s[a]+=r:s[++a]=r:(s[++a]=null,A.push({i:a,x:io(n,r)})),o=so.lastIndex;return o<e.length&&(i=e.slice(o),s[a]?s[a]+=i:s[++a]=i),s.length<2?A[0]?(e=A[0].x,function(t){return e(t)+""}):function(){return e}:(e=A.length,function(t){for(var n,r=0;r<e;++r)s[(n=A[r]).i]=n.x(t);return s.join("")})}o.geom.delaunay=function(t){return o.geom.voronoi().triangles(t)},o.geom.quadtree=function(t,e,n,r,i){var o,a=gi,s=hi;if(o=arguments.length)return a=Zi,s=$i,3===o&&(i=n,r=e,n=e=0),A(t);function A(t){var A,l,c,u,g,h,d,f,p,I=Be(a),C=Be(s);if(null!=e)h=e,d=n,f=r,p=i;else if(f=p=-(h=d=1/0),l=[],c=[],g=t.length,o)for(u=0;u<g;++u)(A=t[u]).x<h&&(h=A.x),A.y<d&&(d=A.y),A.x>f&&(f=A.x),A.y>p&&(p=A.y),l.push(A.x),c.push(A.y);else for(u=0;u<g;++u){var m=+I(A=t[u],u),y=+C(A,u);m<h&&(h=m),y<d&&(d=y),m>f&&(f=m),y>p&&(p=y),l.push(m),c.push(y)}var E=f-h,B=p-d;function b(t,e,n,r,i,o,a,s){if(!isNaN(n)&&!isNaN(r))if(t.leaf){var A=t.x,l=t.y;if(null!=A)if(v(A-n)+v(l-r)<.01)Q(t,e,n,r,i,o,a,s);else{var c=t.point;t.x=t.y=t.point=null,Q(t,c,A,l,i,o,a,s),Q(t,e,n,r,i,o,a,s)}else t.x=n,t.y=r,t.point=e}else Q(t,e,n,r,i,o,a,s)}function Q(t,e,n,r,i,o,a,s){var A=.5*(i+a),l=.5*(o+s),c=n>=A,u=r>=l,g=u<<1|c;t.leaf=!1,c?i=A:a=A,u?o=l:s=l,b(t=t.nodes[g]||(t.nodes[g]={leaf:!0,nodes:[],point:null,x:null,y:null}),e,n,r,i,o,a,s)}E>B?p=d+E:f=h+B;var w={leaf:!0,nodes:[],point:null,x:null,y:null,add:function(t){b(w,t,+I(t,++u),+C(t,u),h,d,f,p)},visit:function(t){to(t,w,h,d,f,p)},find:function(t){return eo(w,t[0],t[1],h,d,f,p)}};if(u=-1,null==e){for(;++u<g;)b(w,t[u],l[u],c[u],h,d,f,p);--u}else t.forEach(w.add);return l=c=t=A=null,w}return A.x=function(t){return arguments.length?(a=t,A):a},A.y=function(t){return arguments.length?(s=t,A):s},A.extent=function(t){return arguments.length?(null==t?e=n=r=i=null:(e=+t[0][0],n=+t[0][1],r=+t[1][0],i=+t[1][1]),A):null==e?null:[[e,n],[r,i]]},A.size=function(t){return arguments.length?(null==t?e=n=r=i=null:(e=n=0,r=+t[0],i=+t[1]),A):null==e?null:[r-e,i-n]},A},o.interpolateRgb=no,o.interpolateObject=ro,o.interpolateNumber=io,o.interpolateString=oo;var ao=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,so=new RegExp(ao.source,"g");function Ao(t,e){for(var n,r=o.interpolators.length;--r>=0&&!(n=o.interpolators[r](t,e)););return n}function lo(t,e){var n,r=[],i=[],o=t.length,a=e.length,s=Math.min(t.length,e.length);for(n=0;n<s;++n)r.push(Ao(t[n],e[n]));for(;n<o;++n)i[n]=t[n];for(;n<a;++n)i[n]=e[n];return function(t){for(n=0;n<s;++n)i[n]=r[n](t);return i}}o.interpolate=Ao,o.interpolators=[function(t,e){var n=typeof e;return("string"===n?ve.has(e.toLowerCase())||/^(#|rgb\(|hsl\()/i.test(e)?no:oo:e instanceof qt?no:Array.isArray(e)?lo:"object"===n&&isNaN(e)?ro:io)(t,e)}],o.interpolateArray=lo;var co=function(){return F},uo=o.map({linear:co,poly:function(t){return function(e){return Math.pow(e,t)}},quad:function(){return Io},cubic:function(){return Co},sin:function(){return yo},exp:function(){return Eo},circle:function(){return vo},elastic:function(t,e){var n;arguments.length<2&&(e=.45);arguments.length?n=e/Lt*Math.asin(1/t):(t=1,n=e/4);return function(r){return 1+t*Math.pow(2,-10*r)*Math.sin((r-n)*Lt/e)}},back:function(t){t||(t=1.70158);return function(e){return e*e*((t+1)*e-t)}},bounce:function(){return Bo}}),go=o.map({in:F,out:fo,"in-out":po,"out-in":function(t){return po(fo(t))}});function ho(t){return function(e){return e<=0?0:e>=1?1:t(e)}}function fo(t){return function(e){return 1-t(1-e)}}function po(t){return function(e){return.5*(e<.5?t(2*e):2-t(2-2*e))}}function Io(t){return t*t}function Co(t){return t*t*t}function mo(t){if(t<=0)return 0;if(t>=1)return 1;var e=t*t,n=e*t;return 4*(t<.5?n:3*(t-e)+n-.75)}function yo(t){return 1-Math.cos(t*Nt)}function Eo(t){return Math.pow(2,10*(t-1))}function vo(t){return 1-Math.sqrt(1-t*t)}function Bo(t){return t<1/2.75?7.5625*t*t:t<2/2.75?7.5625*(t-=1.5/2.75)*t+.75:t<2.5/2.75?7.5625*(t-=2.25/2.75)*t+.9375:7.5625*(t-=2.625/2.75)*t+.984375}function bo(t,e){return e-=t,function(n){return Math.round(t+e*n)}}function Qo(t){var e=[t.a,t.b],n=[t.c,t.d],r=xo(e),i=wo(e,n),o=xo(function(t,e,n){return t[0]+=n*e[0],t[1]+=n*e[1],t}(n,e,-i))||0;e[0]*n[1]<n[0]*e[1]&&(e[0]*=-1,e[1]*=-1,r*=-1,i*=-1),this.rotate=(r?Math.atan2(e[1],e[0]):Math.atan2(-n[0],n[1]))*Tt,this.translate=[t.e,t.f],this.scale=[r,o],this.skew=o?Math.atan2(i,o)*Tt:0}function wo(t,e){return t[0]*e[0]+t[1]*e[1]}function xo(t){var e=Math.sqrt(wo(t,t));return e&&(t[0]/=e,t[1]/=e),e}o.ease=function(t){var e=t.indexOf("-"),n=e>=0?t.slice(0,e):t,r=e>=0?t.slice(e+1):"in";return n=uo.get(n)||co,ho((r=go.get(r)||F)(n.apply(null,a.call(arguments,1))))},o.interpolateHcl=function(t,e){t=o.hcl(t),e=o.hcl(e);var n=t.h,r=t.c,i=t.l,a=e.h-n,s=e.c-r,A=e.l-i;isNaN(s)&&(s=0,r=isNaN(r)?e.c:r);isNaN(a)?(a=0,n=isNaN(n)?e.h:n):a>180?a-=360:a<-180&&(a+=360);return function(t){return te(n+a*t,r+s*t,i+A*t)+""}},o.interpolateHsl=function(t,e){t=o.hsl(t),e=o.hsl(e);var n=t.h,r=t.s,i=t.l,a=e.h-n,s=e.s-r,A=e.l-i;isNaN(s)&&(s=0,r=isNaN(r)?e.s:r);isNaN(a)?(a=0,n=isNaN(n)?e.h:n):a>180?a-=360:a<-180&&(a+=360);return function(t){return Xt(n+a*t,r+s*t,i+A*t)+""}},o.interpolateLab=function(t,e){t=o.lab(t),e=o.lab(e);var n=t.l,r=t.a,i=t.b,a=e.l-n,s=e.a-r,A=e.b-i;return function(t){return se(n+a*t,r+s*t,i+A*t)+""}},o.interpolateRound=bo,o.transform=function(t){var e=A.createElementNS(o.ns.prefix.svg,"g");return(o.transform=function(t){if(null!=t){e.setAttribute("transform",t);var n=e.transform.baseVal.consolidate()}return new Qo(n?n.matrix:_o)})(t)},Qo.prototype.toString=function(){return"translate("+this.translate+")rotate("+this.rotate+")skewX("+this.skew+")scale("+this.scale+")"};var _o={a:1,b:0,c:0,d:1,e:0,f:0};function Do(t){return t.length?t.pop()+",":""}function Mo(t,e){var n=[],r=[];return t=o.transform(t),e=o.transform(e),function(t,e,n,r){if(t[0]!==e[0]||t[1]!==e[1]){var i=n.push("translate(",null,",",null,")");r.push({i:i-4,x:io(t[0],e[0])},{i:i-2,x:io(t[1],e[1])})}else(e[0]||e[1])&&n.push("translate("+e+")")}(t.translate,e.translate,n,r),function(t,e,n,r){t!==e?(t-e>180?e+=360:e-t>180&&(t+=360),r.push({i:n.push(Do(n)+"rotate(",null,")")-2,x:io(t,e)})):e&&n.push(Do(n)+"rotate("+e+")")}(t.rotate,e.rotate,n,r),function(t,e,n,r){t!==e?r.push({i:n.push(Do(n)+"skewX(",null,")")-2,x:io(t,e)}):e&&n.push(Do(n)+"skewX("+e+")")}(t.skew,e.skew,n,r),function(t,e,n,r){if(t[0]!==e[0]||t[1]!==e[1]){var i=n.push(Do(n)+"scale(",null,",",null,")");r.push({i:i-4,x:io(t[0],e[0])},{i:i-2,x:io(t[1],e[1])})}else 1===e[0]&&1===e[1]||n.push(Do(n)+"scale("+e+")")}(t.scale,e.scale,n,r),t=e=null,function(t){for(var e,i=-1,o=r.length;++i<o;)n[(e=r[i]).i]=e.x(t);return n.join("")}}function ko(t,e){return e=(e-=t=+t)||1/e,function(n){return(n-t)/e}}function So(t,e){return e=(e-=t=+t)||1/e,function(n){return Math.max(0,Math.min(1,(n-t)/e))}}function Lo(t){for(var e=t.source,n=t.target,r=function(t,e){if(t===e)return t;var n=Ro(t),r=Ro(e),i=n.pop(),o=r.pop(),a=null;for(;i===o;)a=i,i=n.pop(),o=r.pop();return a}(e,n),i=[e];e!==r;)e=e.parent,i.push(e);for(var o=i.length;n!==r;)i.splice(o,0,n),n=n.parent;return i}function Ro(t){for(var e=[],n=t.parent;null!=n;)e.push(t),t=n,n=n.parent;return e.push(t),e}function No(t){t.fixed|=2}function Fo(t){t.fixed&=-7}function To(t){t.fixed|=4,t.px=t.x,t.py=t.y}function Uo(t){t.fixed&=-5}o.interpolateTransform=Mo,o.layout={},o.layout.bundle=function(){return function(t){for(var e=[],n=-1,r=t.length;++n<r;)e.push(Lo(t[n]));return e}},o.layout.chord=function(){var t,e,n,r,i,a,s,A={},l=0;function c(){var A,c,g,h,d,f={},p=[],I=o.range(r),C=[];for(t=[],e=[],A=0,h=-1;++h<r;){for(c=0,d=-1;++d<r;)c+=n[h][d];p.push(c),C.push(o.range(r)),A+=c}for(i&&I.sort(function(t,e){return i(p[t],p[e])}),a&&C.forEach(function(t,e){t.sort(function(t,r){return a(n[e][t],n[e][r])})}),A=(Lt-l*r)/A,c=0,h=-1;++h<r;){for(g=c,d=-1;++d<r;){var m=I[h],y=C[m][d],E=n[m][y],v=c,B=c+=E*A;f[m+"-"+y]={index:m,subindex:y,startAngle:v,endAngle:B,value:E}}e[m]={index:m,startAngle:g,endAngle:c,value:p[m]},c+=l}for(h=-1;++h<r;)for(d=h-1;++d<r;){var b=f[h+"-"+d],Q=f[d+"-"+h];(b.value||Q.value)&&t.push(b.value<Q.value?{source:Q,target:b}:{source:b,target:Q})}s&&u()}function u(){t.sort(function(t,e){return s((t.source.value+t.target.value)/2,(e.source.value+e.target.value)/2)})}return A.matrix=function(i){return arguments.length?(r=(n=i)&&n.length,t=e=null,A):n},A.padding=function(n){return arguments.length?(l=n,t=e=null,A):l},A.sortGroups=function(n){return arguments.length?(i=n,t=e=null,A):i},A.sortSubgroups=function(e){return arguments.length?(a=e,t=null,A):a},A.sortChords=function(e){return arguments.length?(s=e,t&&u(),A):s},A.chords=function(){return t||c(),t},A.groups=function(){return e||c(),e},A},o.layout.force=function(){var t,e,n,r,i,a,s={},A=o.dispatch("start","tick","end"),l=[1,1],c=.9,u=Oo,g=Go,h=-30,d=Ho,f=.1,p=.64,I=[],C=[];function m(t){return function(e,n,r,i){if(e.point!==t){var o=e.cx-t.x,a=e.cy-t.y,s=i-n,A=o*o+a*a;if(s*s/p<A){if(A<d){var l=e.charge/A;t.px-=o*l,t.py-=a*l}return!0}if(e.point&&A&&A<d){l=e.pointCharge/A;t.px-=o*l,t.py-=a*l}}return!e.charge}}function y(t){t.px=o.event.x,t.py=o.event.y,s.resume()}return s.tick=function(){if((n*=.99)<.005)return t=null,A.end({type:"end",alpha:n=0}),!0;var e,s,u,g,d,p,y,E,v,B=I.length,b=C.length;for(s=0;s<b;++s)g=(u=C[s]).source,(p=(E=(d=u.target).x-g.x)*E+(v=d.y-g.y)*v)&&(E*=p=n*i[s]*((p=Math.sqrt(p))-r[s])/p,v*=p,d.x-=E*(y=g.weight+d.weight?g.weight/(g.weight+d.weight):.5),d.y-=v*y,g.x+=E*(y=1-y),g.y+=v*y);if((y=n*f)&&(E=l[0]/2,v=l[1]/2,s=-1,y))for(;++s<B;)(u=I[s]).x+=(E-u.x)*y,u.y+=(v-u.y)*y;if(h)for(!function t(e,n,r){var i=0,o=0;e.charge=0;if(!e.leaf)for(var a,s=e.nodes,A=s.length,l=-1;++l<A;)null!=(a=s[l])&&(t(a,n,r),e.charge+=a.charge,i+=a.charge*a.cx,o+=a.charge*a.cy);if(e.point){e.leaf||(e.point.x+=Math.random()-.5,e.point.y+=Math.random()-.5);var c=n*r[e.point.index];e.charge+=e.pointCharge=c,i+=c*e.point.x,o+=c*e.point.y}e.cx=i/e.charge;e.cy=o/e.charge}(e=o.geom.quadtree(I),n,a),s=-1;++s<B;)(u=I[s]).fixed||e.visit(m(u));for(s=-1;++s<B;)(u=I[s]).fixed?(u.x=u.px,u.y=u.py):(u.x-=(u.px-(u.px=u.x))*c,u.y-=(u.py-(u.py=u.y))*c);A.tick({type:"tick",alpha:n})},s.nodes=function(t){return arguments.length?(I=t,s):I},s.links=function(t){return arguments.length?(C=t,s):C},s.size=function(t){return arguments.length?(l=t,s):l},s.linkDistance=function(t){return arguments.length?(u="function"==typeof t?t:+t,s):u},s.distance=s.linkDistance,s.linkStrength=function(t){return arguments.length?(g="function"==typeof t?t:+t,s):g},s.friction=function(t){return arguments.length?(c=+t,s):c},s.charge=function(t){return arguments.length?(h="function"==typeof t?t:+t,s):h},s.chargeDistance=function(t){return arguments.length?(d=t*t,s):Math.sqrt(d)},s.gravity=function(t){return arguments.length?(f=+t,s):f},s.theta=function(t){return arguments.length?(p=t*t,s):Math.sqrt(p)},s.alpha=function(e){return arguments.length?(e=+e,n?e>0?n=e:(t.c=null,t.t=NaN,t=null,A.end({type:"end",alpha:n=0})):e>0&&(A.start({type:"start",alpha:n=e}),t=ke(s.tick)),s):n},s.start=function(){var t,e,n,o=I.length,A=C.length,c=l[0],d=l[1];for(t=0;t<o;++t)(n=I[t]).index=t,n.weight=0;for(t=0;t<A;++t)"number"==typeof(n=C[t]).source&&(n.source=I[n.source]),"number"==typeof n.target&&(n.target=I[n.target]),++n.source.weight,++n.target.weight;for(t=0;t<o;++t)n=I[t],isNaN(n.x)&&(n.x=f("x",c)),isNaN(n.y)&&(n.y=f("y",d)),isNaN(n.px)&&(n.px=n.x),isNaN(n.py)&&(n.py=n.y);if(r=[],"function"==typeof u)for(t=0;t<A;++t)r[t]=+u.call(this,C[t],t);else for(t=0;t<A;++t)r[t]=u;if(i=[],"function"==typeof g)for(t=0;t<A;++t)i[t]=+g.call(this,C[t],t);else for(t=0;t<A;++t)i[t]=g;if(a=[],"function"==typeof h)for(t=0;t<o;++t)a[t]=+h.call(this,I[t],t);else for(t=0;t<o;++t)a[t]=h;function f(n,r){if(!e){for(e=new Array(o),l=0;l<o;++l)e[l]=[];for(l=0;l<A;++l){var i=C[l];e[i.source.index].push(i.target),e[i.target.index].push(i.source)}}for(var a,s=e[t],l=-1,c=s.length;++l<c;)if(!isNaN(a=s[l][n]))return a;return Math.random()*r}return s.resume()},s.resume=function(){return s.alpha(.1)},s.stop=function(){return s.alpha(0)},s.drag=function(){if(e||(e=o.behavior.drag().origin(F).on("dragstart.force",No).on("drag.force",y).on("dragend.force",Fo)),!arguments.length)return e;this.on("mouseover.force",To).on("mouseout.force",Uo).call(e)},o.rebind(s,A,"on")};var Oo=20,Go=1,Ho=1/0;function jo(t,e){return o.rebind(t,e,"sort","children","value"),t.nodes=t,t.links=qo,t}function Yo(t,e){for(var n=[t];null!=(t=n.pop());)if(e(t),(i=t.children)&&(r=i.length))for(var r,i;--r>=0;)n.push(i[r])}function Po(t,e){for(var n=[t],r=[];null!=(t=n.pop());)if(r.push(t),(o=t.children)&&(i=o.length))for(var i,o,a=-1;++a<i;)n.push(o[a]);for(;null!=(t=r.pop());)e(t)}function Ko(t){return t.children}function zo(t){return t.value}function Jo(t,e){return e.value-t.value}function qo(t){return o.merge(t.map(function(t){return(t.children||[]).map(function(e){return{source:t,target:e}})}))}o.layout.hierarchy=function(){var t=Jo,e=Ko,n=zo;function r(i){var o,a=[i],s=[];for(i.depth=0;null!=(o=a.pop());)if(s.push(o),(l=e.call(r,o,o.depth))&&(A=l.length)){for(var A,l,c;--A>=0;)a.push(c=l[A]),c.parent=o,c.depth=o.depth+1;n&&(o.value=0),o.children=l}else n&&(o.value=+n.call(r,o,o.depth)||0),delete o.children;return Po(i,function(e){var r,i;t&&(r=e.children)&&r.sort(t),n&&(i=e.parent)&&(i.value+=e.value)}),s}return r.sort=function(e){return arguments.length?(t=e,r):t},r.children=function(t){return arguments.length?(e=t,r):e},r.value=function(t){return arguments.length?(n=t,r):n},r.revalue=function(t){return n&&(Yo(t,function(t){t.children&&(t.value=0)}),Po(t,function(t){var e;t.children||(t.value=+n.call(r,t,t.depth)||0),(e=t.parent)&&(e.value+=t.value)})),t},r},o.layout.partition=function(){var t=o.layout.hierarchy(),e=[1,1];function n(n,r){var i=t.call(this,n,r);return function t(e,n,r,i){var o=e.children;if(e.x=n,e.y=e.depth*i,e.dx=r,e.dy=i,o&&(a=o.length)){var a,s,A,l=-1;for(r=e.value?r/e.value:0;++l<a;)t(s=o[l],n,A=s.value*r,i),n+=A}}(i[0],0,e[0],e[1]/function t(e){var n=e.children,r=0;if(n&&(i=n.length))for(var i,o=-1;++o<i;)r=Math.max(r,t(n[o]));return 1+r}(i[0])),i}return n.size=function(t){return arguments.length?(e=t,n):e},jo(n,t)},o.layout.pie=function(){var t=Number,e=Vo,n=0,r=Lt,i=0;function a(s){var A,l=s.length,c=s.map(function(e,n){return+t.call(a,e,n)}),u=+("function"==typeof n?n.apply(this,arguments):n),g=("function"==typeof r?r.apply(this,arguments):r)-u,h=Math.min(Math.abs(g)/l,+("function"==typeof i?i.apply(this,arguments):i)),d=h*(g<0?-1:1),f=o.sum(c),p=f?(g-l*d)/f:0,I=o.range(l),C=[];return null!=e&&I.sort(e===Vo?function(t,e){return c[e]-c[t]}:function(t,n){return e(s[t],s[n])}),I.forEach(function(t){C[t]={data:s[t],value:A=c[t],startAngle:u,endAngle:u+=A*p+d,padAngle:h}}),C}return a.value=function(e){return arguments.length?(t=e,a):t},a.sort=function(t){return arguments.length?(e=t,a):e},a.startAngle=function(t){return arguments.length?(n=t,a):n},a.endAngle=function(t){return arguments.length?(r=t,a):r},a.padAngle=function(t){return arguments.length?(i=t,a):i},a};var Vo={};function Wo(t){return t.x}function Xo(t){return t.y}function Zo(t,e,n){t.y0=e,t.y=n}o.layout.stack=function(){var t=F,e=ea,n=na,r=Zo,i=Wo,a=Xo;function s(A,l){if(!(h=A.length))return A;var c=A.map(function(e,n){return t.call(s,e,n)}),u=c.map(function(t){return t.map(function(t,e){return[i.call(s,t,e),a.call(s,t,e)]})}),g=e.call(s,u,l);c=o.permute(c,g),u=o.permute(u,g);var h,d,f,p,I=n.call(s,u,l),C=c[0].length;for(f=0;f<C;++f)for(r.call(s,c[0][f],p=I[f],u[0][f][1]),d=1;d<h;++d)r.call(s,c[d][f],p+=u[d-1][f][1],u[d][f][1]);return A}return s.values=function(e){return arguments.length?(t=e,s):t},s.order=function(t){return arguments.length?(e="function"==typeof t?t:$o.get(t)||ea,s):e},s.offset=function(t){return arguments.length?(n="function"==typeof t?t:ta.get(t)||na,s):n},s.x=function(t){return arguments.length?(i=t,s):i},s.y=function(t){return arguments.length?(a=t,s):a},s.out=function(t){return arguments.length?(r=t,s):r},s};var $o=o.map({"inside-out":function(t){var e,n,r=t.length,i=t.map(ra),a=t.map(ia),s=o.range(r).sort(function(t,e){return i[t]-i[e]}),A=0,l=0,c=[],u=[];for(e=0;e<r;++e)n=s[e],A<l?(A+=a[n],c.push(n)):(l+=a[n],u.push(n));return u.reverse().concat(c)},reverse:function(t){return o.range(t.length).reverse()},default:ea}),ta=o.map({silhouette:function(t){var e,n,r,i=t.length,o=t[0].length,a=[],s=0,A=[];for(n=0;n<o;++n){for(e=0,r=0;e<i;e++)r+=t[e][n][1];r>s&&(s=r),a.push(r)}for(n=0;n<o;++n)A[n]=(s-a[n])/2;return A},wiggle:function(t){var e,n,r,i,o,a,s,A,l,c=t.length,u=t[0],g=u.length,h=[];for(h[0]=A=l=0,n=1;n<g;++n){for(e=0,i=0;e<c;++e)i+=t[e][n][1];for(e=0,o=0,s=u[n][0]-u[n-1][0];e<c;++e){for(r=0,a=(t[e][n][1]-t[e][n-1][1])/(2*s);r<e;++r)a+=(t[r][n][1]-t[r][n-1][1])/s;o+=a*t[e][n][1]}h[n]=A-=i?o/i*s:0,A<l&&(l=A)}for(n=0;n<g;++n)h[n]-=l;return h},expand:function(t){var e,n,r,i=t.length,o=t[0].length,a=1/i,s=[];for(n=0;n<o;++n){for(e=0,r=0;e<i;e++)r+=t[e][n][1];if(r)for(e=0;e<i;e++)t[e][n][1]/=r;else for(e=0;e<i;e++)t[e][n][1]=a}for(n=0;n<o;++n)s[n]=0;return s},zero:na});function ea(t){return o.range(t.length)}function na(t){for(var e=-1,n=t[0].length,r=[];++e<n;)r[e]=0;return r}function ra(t){for(var e,n=1,r=0,i=t[0][1],o=t.length;n<o;++n)(e=t[n][1])>i&&(r=n,i=e);return r}function ia(t){return t.reduce(oa,0)}function oa(t,e){return t+e[1]}function aa(t,e){return sa(t,Math.ceil(Math.log(e.length)/Math.LN2+1))}function sa(t,e){for(var n=-1,r=+t[0],i=(t[1]-r)/e,o=[];++n<=e;)o[n]=i*n+r;return o}function Aa(t){return[o.min(t),o.max(t)]}function la(t,e){return t.value-e.value}function ca(t,e){var n=t._pack_next;t._pack_next=e,e._pack_prev=t,e._pack_next=n,n._pack_prev=e}function ua(t,e){t._pack_next=e,e._pack_prev=t}function ga(t,e){var n=e.x-t.x,r=e.y-t.y,i=t.r+e.r;return.999*i*i>n*n+r*r}function ha(t){if((e=t.children)&&(A=e.length)){var e,n,r,i,o,a,s,A,l=1/0,c=-1/0,u=1/0,g=-1/0;if(e.forEach(da),(n=e[0]).x=-n.r,n.y=0,m(n),A>1&&((r=e[1]).x=r.r,r.y=0,m(r),A>2))for(pa(n,r,i=e[2]),m(i),ca(n,i),n._pack_prev=i,ca(i,r),r=n._pack_next,o=3;o<A;o++){pa(n,r,i=e[o]);var h=0,d=1,f=1;for(a=r._pack_next;a!==r;a=a._pack_next,d++)if(ga(a,i)){h=1;break}if(1==h)for(s=n._pack_prev;s!==a._pack_prev&&!ga(s,i);s=s._pack_prev,f++);h?(d<f||d==f&&r.r<n.r?ua(n,r=a):ua(n=s,r),o--):(ca(n,i),r=i,m(i))}var p=(l+c)/2,I=(u+g)/2,C=0;for(o=0;o<A;o++)(i=e[o]).x-=p,i.y-=I,C=Math.max(C,i.r+Math.sqrt(i.x*i.x+i.y*i.y));t.r=C,e.forEach(fa)}function m(t){l=Math.min(t.x-t.r,l),c=Math.max(t.x+t.r,c),u=Math.min(t.y-t.r,u),g=Math.max(t.y+t.r,g)}}function da(t){t._pack_next=t._pack_prev=t}function fa(t){delete t._pack_next,delete t._pack_prev}function pa(t,e,n){var r=t.r+n.r,i=e.x-t.x,o=e.y-t.y;if(r&&(i||o)){var a=e.r+n.r,s=i*i+o*o,A=.5+((r*=r)-(a*=a))/(2*s),l=Math.sqrt(Math.max(0,2*a*(r+s)-(r-=s)*r-a*a))/(2*s);n.x=t.x+A*i+l*o,n.y=t.y+A*o-l*i}else n.x=t.x+r,n.y=t.y}function Ia(t,e){return t.parent==e.parent?1:2}function Ca(t){var e=t.children;return e.length?e[0]:t.t}function ma(t){var e,n=t.children;return(e=n.length)?n[e-1]:t.t}function ya(t,e,n){var r=n/(e.i-t.i);e.c-=r,e.s+=n,t.c+=r,e.z+=n,e.m+=n}function Ea(t,e,n){return t.a.parent===e.parent?t.a:n}function va(t){return{x:t.x,y:t.y,dx:t.dx,dy:t.dy}}function Ba(t,e){var n=t.x+e[3],r=t.y+e[0],i=t.dx-e[1]-e[3],o=t.dy-e[0]-e[2];return i<0&&(n+=i/2,i=0),o<0&&(r+=o/2,o=0),{x:n,y:r,dx:i,dy:o}}function ba(t){var e=t[0],n=t[t.length-1];return e<n?[e,n]:[n,e]}function Qa(t){return t.rangeExtent?t.rangeExtent():ba(t.range())}function wa(t,e,n,r){var i=n(t[0],t[1]),o=r(e[0],e[1]);return function(t){return o(i(t))}}function xa(t,e){var n,r=0,i=t.length-1,o=t[r],a=t[i];return a<o&&(n=r,r=i,i=n,n=o,o=a,a=n),t[r]=e.floor(o),t[i]=e.ceil(a),t}function _a(t){return t?{floor:function(e){return Math.floor(e/t)*t},ceil:function(e){return Math.ceil(e/t)*t}}:Da}o.layout.histogram=function(){var t=!0,e=Number,n=Aa,r=aa;function i(i,a){for(var s,A,l=[],c=i.map(e,this),u=n.call(this,c,a),g=r.call(this,u,c,a),h=(a=-1,c.length),d=g.length-1,f=t?1:1/h;++a<d;)(s=l[a]=[]).dx=g[a+1]-(s.x=g[a]),s.y=0;if(d>0)for(a=-1;++a<h;)(A=c[a])>=u[0]&&A<=u[1]&&((s=l[o.bisect(g,A,1,d)-1]).y+=f,s.push(i[a]));return l}return i.value=function(t){return arguments.length?(e=t,i):e},i.range=function(t){return arguments.length?(n=Be(t),i):n},i.bins=function(t){return arguments.length?(r="number"==typeof t?function(e){return sa(e,t)}:Be(t),i):r},i.frequency=function(e){return arguments.length?(t=!!e,i):t},i},o.layout.pack=function(){var t,e=o.layout.hierarchy().sort(la),n=0,r=[1,1];function i(i,o){var a=e.call(this,i,o),s=a[0],A=r[0],l=r[1],c=null==t?Math.sqrt:"function"==typeof t?t:function(){return t};if(s.x=s.y=0,Po(s,function(t){t.r=+c(t.value)}),Po(s,ha),n){var u=n*(t?1:Math.max(2*s.r/A,2*s.r/l))/2;Po(s,function(t){t.r+=u}),Po(s,ha),Po(s,function(t){t.r-=u})}return function t(e,n,r,i){var o=e.children;e.x=n+=i*e.x;e.y=r+=i*e.y;e.r*=i;if(o)for(var a=-1,s=o.length;++a<s;)t(o[a],n,r,i)}(s,A/2,l/2,t?1:1/Math.max(2*s.r/A,2*s.r/l)),a}return i.size=function(t){return arguments.length?(r=t,i):r},i.radius=function(e){return arguments.length?(t=null==e||"function"==typeof e?e:+e,i):t},i.padding=function(t){return arguments.length?(n=+t,i):n},jo(i,e)},o.layout.tree=function(){var t=o.layout.hierarchy().sort(null).value(null),e=Ia,n=[1,1],r=null;function i(i,o){var l=t.call(this,i,o),c=l[0],u=function(t){var e,n={A:null,children:[t]},r=[n];for(;null!=(e=r.pop());)for(var i,o=e.children,a=0,s=o.length;a<s;++a)r.push((o[a]=i={_:o[a],parent:e,children:(i=o[a].children)&&i.slice()||[],A:null,a:null,z:0,m:0,c:0,s:0,t:null,i:a}).a=i);return n.children[0]}(c);if(Po(u,a),u.parent.m=-u.z,Yo(u,s),r)Yo(c,A);else{var g=c,h=c,d=c;Yo(c,function(t){t.x<g.x&&(g=t),t.x>h.x&&(h=t),t.depth>d.depth&&(d=t)});var f=e(g,h)/2-g.x,p=n[0]/(h.x+e(h,g)/2+f),I=n[1]/(d.depth||1);Yo(c,function(t){t.x=(t.x+f)*p,t.y=t.depth*I})}return l}function a(t){var n=t.children,r=t.parent.children,i=t.i?r[t.i-1]:null;if(n.length){!function(t){var e,n=0,r=0,i=t.children,o=i.length;for(;--o>=0;)(e=i[o]).z+=n,e.m+=n,n+=e.s+(r+=e.c)}(t);var o=(n[0].z+n[n.length-1].z)/2;i?(t.z=i.z+e(t._,i._),t.m=t.z-o):t.z=o}else i&&(t.z=i.z+e(t._,i._));t.parent.A=function(t,n,r){if(n){for(var i,o=t,a=t,s=n,A=o.parent.children[0],l=o.m,c=a.m,u=s.m,g=A.m;s=ma(s),o=Ca(o),s&&o;)A=Ca(A),(a=ma(a)).a=t,(i=s.z+u-o.z-l+e(s._,o._))>0&&(ya(Ea(s,t,r),t,i),l+=i,c+=i),u+=s.m,l+=o.m,g+=A.m,c+=a.m;s&&!ma(a)&&(a.t=s,a.m+=u-c),o&&!Ca(A)&&(A.t=o,A.m+=l-g,r=t)}return r}(t,i,t.parent.A||r[0])}function s(t){t._.x=t.z+t.parent.m,t.m+=t.parent.m}function A(t){t.x*=n[0],t.y=t.depth*n[1]}return i.separation=function(t){return arguments.length?(e=t,i):e},i.size=function(t){return arguments.length?(r=null==(n=t)?A:null,i):r?null:n},i.nodeSize=function(t){return arguments.length?(r=null==(n=t)?null:A,i):r?n:null},jo(i,t)},o.layout.cluster=function(){var t=o.layout.hierarchy().sort(null).value(null),e=Ia,n=[1,1],r=!1;function i(i,a){var s,A=t.call(this,i,a),l=A[0],c=0;Po(l,function(t){var n=t.children;n&&n.length?(t.x=function(t){return t.reduce(function(t,e){return t+e.x},0)/t.length}(n),t.y=function(t){return 1+o.max(t,function(t){return t.y})}(n)):(t.x=s?c+=e(t,s):0,t.y=0,s=t)});var u=function t(e){var n=e.children;return n&&n.length?t(n[0]):e}(l),g=function t(e){var n,r=e.children;return r&&(n=r.length)?t(r[n-1]):e}(l),h=u.x-e(u,g)/2,d=g.x+e(g,u)/2;return Po(l,r?function(t){t.x=(t.x-l.x)*n[0],t.y=(l.y-t.y)*n[1]}:function(t){t.x=(t.x-h)/(d-h)*n[0],t.y=(1-(l.y?t.y/l.y:1))*n[1]}),A}return i.separation=function(t){return arguments.length?(e=t,i):e},i.size=function(t){return arguments.length?(r=null==(n=t),i):r?null:n},i.nodeSize=function(t){return arguments.length?(r=null!=(n=t),i):r?n:null},jo(i,t)},o.layout.treemap=function(){var t,e=o.layout.hierarchy(),n=Math.round,r=[1,1],i=null,a=va,s=!1,A="squarify",l=.5*(1+Math.sqrt(5));function c(t,e){for(var n,r,i=-1,o=t.length;++i<o;)r=(n=t[i]).value*(e<0?0:e),n.area=isNaN(r)||r<=0?0:r}function u(t){var e=t.children;if(e&&e.length){var n,r,i,o=a(t),s=[],l=e.slice(),g=1/0,f="slice"===A?o.dx:"dice"===A?o.dy:"slice-dice"===A?1&t.depth?o.dy:o.dx:Math.min(o.dx,o.dy);for(c(l,o.dx*o.dy/t.value),s.area=0;(i=l.length)>0;)s.push(n=l[i-1]),s.area+=n.area,"squarify"!==A||(r=h(s,f))<=g?(l.pop(),g=r):(s.area-=s.pop().area,d(s,f,o,!1),f=Math.min(o.dx,o.dy),s.length=s.area=0,g=1/0);s.length&&(d(s,f,o,!0),s.length=s.area=0),e.forEach(u)}}function g(t){var e=t.children;if(e&&e.length){var n,r=a(t),i=e.slice(),o=[];for(c(i,r.dx*r.dy/t.value),o.area=0;n=i.pop();)o.push(n),o.area+=n.area,null!=n.z&&(d(o,n.z?r.dx:r.dy,r,!i.length),o.length=o.area=0);e.forEach(g)}}function h(t,e){for(var n,r=t.area,i=0,o=1/0,a=-1,s=t.length;++a<s;)(n=t[a].area)&&(n<o&&(o=n),n>i&&(i=n));return e*=e,(r*=r)?Math.max(e*i*l/r,r/(e*o*l)):1/0}function d(t,e,r,i){var o,a=-1,s=t.length,A=r.x,l=r.y,c=e?n(t.area/e):0;if(e==r.dx){for((i||c>r.dy)&&(c=r.dy);++a<s;)(o=t[a]).x=A,o.y=l,o.dy=c,A+=o.dx=Math.min(r.x+r.dx-A,c?n(o.area/c):0);o.z=!0,o.dx+=r.x+r.dx-A,r.y+=c,r.dy-=c}else{for((i||c>r.dx)&&(c=r.dx);++a<s;)(o=t[a]).x=A,o.y=l,o.dx=c,l+=o.dy=Math.min(r.y+r.dy-l,c?n(o.area/c):0);o.z=!1,o.dy+=r.y+r.dy-l,r.x+=c,r.dx-=c}}function f(n){var i=t||e(n),o=i[0];return o.x=o.y=0,o.value?(o.dx=r[0],o.dy=r[1]):o.dx=o.dy=0,t&&e.revalue(o),c([o],o.dx*o.dy/o.value),(t?g:u)(o),s&&(t=i),i}return f.size=function(t){return arguments.length?(r=t,f):r},f.padding=function(t){if(!arguments.length)return i;function e(e){var n=t.call(f,e,e.depth);return null==n?va(e):Ba(e,"number"==typeof n?[n,n,n,n]:n)}function n(e){return Ba(e,t)}var r;return a=null==(i=t)?va:"function"==(r=typeof t)?e:"number"===r?(t=[t,t,t,t],n):n,f},f.round=function(t){return arguments.length?(n=t?Math.round:Number,f):n!=Number},f.sticky=function(e){return arguments.length?(s=e,t=null,f):s},f.ratio=function(t){return arguments.length?(l=t,f):l},f.mode=function(t){return arguments.length?(A=t+"",f):A},jo(f,e)},o.random={normal:function(t,e){var n=arguments.length;return n<2&&(e=1),n<1&&(t=0),function(){var n,r,i;do{i=(n=2*Math.random()-1)*n+(r=2*Math.random()-1)*r}while(!i||i>1);return t+e*n*Math.sqrt(-2*Math.log(i)/i)}},logNormal:function(){var t=o.random.normal.apply(o,arguments);return function(){return Math.exp(t())}},bates:function(t){var e=o.random.irwinHall(t);return function(){return e()/t}},irwinHall:function(t){return function(){for(var e=0,n=0;n<t;n++)e+=Math.random();return e}}},o.scale={};var Da={floor:F,ceil:F};function Ma(t,e,n,r){var i=[],a=[],s=0,A=Math.min(t.length,e.length)-1;for(t[A]<t[0]&&(t=t.slice().reverse(),e=e.slice().reverse());++s<=A;)i.push(n(t[s-1],t[s])),a.push(r(e[s-1],e[s]));return function(e){var n=o.bisect(t,e,1,A)-1;return a[n](i[n](e))}}function ka(t,e){return o.rebind(t,e,"range","rangeRound","interpolate","clamp")}function Sa(t,e){return xa(t,_a(La(t,e)[2])),xa(t,_a(La(t,e)[2])),t}function La(t,e){null==e&&(e=10);var n=ba(t),r=n[1]-n[0],i=Math.pow(10,Math.floor(Math.log(r/e)/Math.LN10)),o=e/r*i;return o<=.15?i*=10:o<=.35?i*=5:o<=.75&&(i*=2),n[0]=Math.ceil(n[0]/i)*i,n[1]=Math.floor(n[1]/i)*i+.5*i,n[2]=i,n}function Ra(t,e){return o.range.apply(o,La(t,e))}function Na(t,e,n){var r=La(t,e);if(n){var i=Ue.exec(n);if(i.shift(),"s"===i[8]){var a=o.formatPrefix(Math.max(v(r[0]),v(r[1])));return i[7]||(i[7]="."+Ta(a.scale(r[2]))),i[8]="f",n=o.format(i.join("")),function(t){return n(a.scale(t))+a.symbol}}i[7]||(i[7]="."+function(t,e){var n=Ta(e[2]);return t in Fa?Math.abs(n-Ta(Math.max(v(e[0]),v(e[1]))))+ +("e"!==t):n-2*("%"===t)}(i[8],r)),n=i.join("")}else n=",."+Ta(r[2])+"f";return o.format(n)}o.scale.linear=function(){return function t(e,n,r,i){var o,a;function s(){var t=Math.min(e.length,n.length)>2?Ma:wa,s=i?So:ko;return o=t(e,n,s,r),a=t(n,e,s,Ao),A}function A(t){return o(t)}A.invert=function(t){return a(t)};A.domain=function(t){return arguments.length?(e=t.map(Number),s()):e};A.range=function(t){return arguments.length?(n=t,s()):n};A.rangeRound=function(t){return A.range(t).interpolate(bo)};A.clamp=function(t){return arguments.length?(i=t,s()):i};A.interpolate=function(t){return arguments.length?(r=t,s()):r};A.ticks=function(t){return Ra(e,t)};A.tickFormat=function(t,n){return Na(e,t,n)};A.nice=function(t){return Sa(e,t),s()};A.copy=function(){return t(e,n,r,i)};return s()}([0,1],[0,1],Ao,!1)};var Fa={s:1,g:1,p:1,r:1,e:1};function Ta(t){return-Math.floor(Math.log(t)/Math.LN10+.01)}o.scale.log=function(){return function t(e,n,r,i){function a(t){return(r?Math.log(t<0?0:t):-Math.log(t>0?0:-t))/Math.log(n)}function s(t){return r?Math.pow(n,t):-Math.pow(n,-t)}function A(t){return e(a(t))}A.invert=function(t){return s(e.invert(t))};A.domain=function(t){return arguments.length?(r=t[0]>=0,e.domain((i=t.map(Number)).map(a)),A):i};A.base=function(t){return arguments.length?(n=+t,e.domain(i.map(a)),A):n};A.nice=function(){var t=xa(i.map(a),r?Math:Oa);return e.domain(t),i=t.map(s),A};A.ticks=function(){var t=ba(i),e=[],o=t[0],A=t[1],l=Math.floor(a(o)),c=Math.ceil(a(A)),u=n%1?2:n;if(isFinite(c-l)){if(r){for(;l<c;l++)for(var g=1;g<u;g++)e.push(s(l)*g);e.push(s(l))}else for(e.push(s(l));l++<c;)for(var g=u-1;g>0;g--)e.push(s(l)*g);for(l=0;e[l]<o;l++);for(c=e.length;e[c-1]>A;c--);e=e.slice(l,c)}return e};A.tickFormat=function(t,e){if(!arguments.length)return Ua;arguments.length<2?e=Ua:"function"!=typeof e&&(e=o.format(e));var r=Math.max(1,n*t/A.ticks().length);return function(t){var i=t/s(Math.round(a(t)));return i*n<n-.5&&(i*=n),i<=r?e(t):""}};A.copy=function(){return t(e.copy(),n,r,i)};return ka(A,e)}(o.scale.linear().domain([0,1]),10,!0,[1,10])};var Ua=o.format(".0e"),Oa={floor:function(t){return-Math.ceil(-t)},ceil:function(t){return-Math.floor(-t)}};function Ga(t){return function(e){return e<0?-Math.pow(-e,t):Math.pow(e,t)}}o.scale.pow=function(){return function t(e,n,r){var i=Ga(n),o=Ga(1/n);function a(t){return e(i(t))}a.invert=function(t){return o(e.invert(t))};a.domain=function(t){return arguments.length?(e.domain((r=t.map(Number)).map(i)),a):r};a.ticks=function(t){return Ra(r,t)};a.tickFormat=function(t,e){return Na(r,t,e)};a.nice=function(t){return a.domain(Sa(r,t))};a.exponent=function(t){return arguments.length?(i=Ga(n=t),o=Ga(1/n),e.domain(r.map(i)),a):n};a.copy=function(){return t(e.copy(),n,r)};return ka(a,e)}(o.scale.linear(),1,[0,1])},o.scale.sqrt=function(){return o.scale.pow().exponent(.5)},o.scale.ordinal=function(){return function t(e,n){var r,i,a;function s(t){return i[((r.get(t)||("range"===n.t?r.set(t,e.push(t)):NaN))-1)%i.length]}function A(t,n){return o.range(e.length).map(function(e){return t+n*e})}s.domain=function(t){if(!arguments.length)return e;e=[],r=new Q;for(var i,o=-1,a=t.length;++o<a;)r.has(i=t[o])||r.set(i,e.push(i));return s[n.t].apply(s,n.a)};s.range=function(t){return arguments.length?(i=t,a=0,n={t:"range",a:arguments},s):i};s.rangePoints=function(t,r){arguments.length<2&&(r=0);var o=t[0],l=t[1],c=e.length<2?(o=(o+l)/2,0):(l-o)/(e.length-1+r);return i=A(o+c*r/2,c),a=0,n={t:"rangePoints",a:arguments},s};s.rangeRoundPoints=function(t,r){arguments.length<2&&(r=0);var o=t[0],l=t[1],c=e.length<2?(o=l=Math.round((o+l)/2),0):(l-o)/(e.length-1+r)|0;return i=A(o+Math.round(c*r/2+(l-o-(e.length-1+r)*c)/2),c),a=0,n={t:"rangeRoundPoints",a:arguments},s};s.rangeBands=function(t,r,o){arguments.length<2&&(r=0),arguments.length<3&&(o=r);var l=t[1]<t[0],c=t[l-0],u=t[1-l],g=(u-c)/(e.length-r+2*o);return i=A(c+g*o,g),l&&i.reverse(),a=g*(1-r),n={t:"rangeBands",a:arguments},s};s.rangeRoundBands=function(t,r,o){arguments.length<2&&(r=0),arguments.length<3&&(o=r);var l=t[1]<t[0],c=t[l-0],u=t[1-l],g=Math.floor((u-c)/(e.length-r+2*o));return i=A(c+Math.round((u-c-(e.length-r)*g)/2),g),l&&i.reverse(),a=Math.round(g*(1-r)),n={t:"rangeRoundBands",a:arguments},s};s.rangeBand=function(){return a};s.rangeExtent=function(){return ba(n.a[0])};s.copy=function(){return t(e,n)};return s.domain(e)}([],{t:"range",a:[[]]})},o.scale.category10=function(){return o.scale.ordinal().range(Ha)},o.scale.category20=function(){return o.scale.ordinal().range(ja)},o.scale.category20b=function(){return o.scale.ordinal().range(Ya)},o.scale.category20c=function(){return o.scale.ordinal().range(Pa)};var Ha=[2062260,16744206,2924588,14034728,9725885,9197131,14907330,8355711,12369186,1556175].map(de),ja=[2062260,11454440,16744206,16759672,2924588,10018698,14034728,16750742,9725885,12955861,9197131,12885140,14907330,16234194,8355711,13092807,12369186,14408589,1556175,10410725].map(de),Ya=[3750777,5395619,7040719,10264286,6519097,9216594,11915115,13556636,9202993,12426809,15186514,15190932,8666169,11356490,14049643,15177372,8077683,10834324,13528509,14589654].map(de),Pa=[3244733,7057110,10406625,13032431,15095053,16616764,16625259,16634018,3253076,7652470,10607003,13101504,7695281,10394312,12369372,14342891,6513507,9868950,12434877,14277081].map(de);function Ka(){return 0}o.scale.quantile=function(){return function t(e,n){var r;function i(){var t=0,i=n.length;for(r=[];++t<i;)r[t-1]=o.quantile(e,t/i);return a}function a(t){if(!isNaN(t=+t))return n[o.bisect(r,t)]}a.domain=function(t){return arguments.length?(e=t.map(I).filter(C).sort(p),i()):e};a.range=function(t){return arguments.length?(n=t,i()):n};a.quantiles=function(){return r};a.invertExtent=function(t){return(t=n.indexOf(t))<0?[NaN,NaN]:[t>0?r[t-1]:e[0],t<r.length?r[t]:e[e.length-1]]};a.copy=function(){return t(e,n)};return i()}([],[])},o.scale.quantize=function(){return function t(e,n,r){var i,o;function a(t){return r[Math.max(0,Math.min(o,Math.floor(i*(t-e))))]}function s(){return i=r.length/(n-e),o=r.length-1,a}a.domain=function(t){return arguments.length?(e=+t[0],n=+t[t.length-1],s()):[e,n]};a.range=function(t){return arguments.length?(r=t,s()):r};a.invertExtent=function(t){return[t=(t=r.indexOf(t))<0?NaN:t/i+e,t+1/i]};a.copy=function(){return t(e,n,r)};return s()}(0,1,[0,1])},o.scale.threshold=function(){return function t(e,n){function r(t){if(t<=t)return n[o.bisect(e,t)]}r.domain=function(t){return arguments.length?(e=t,r):e};r.range=function(t){return arguments.length?(n=t,r):n};r.invertExtent=function(t){return t=n.indexOf(t),[e[t-1],e[t]]};r.copy=function(){return t(e,n)};return r}([.5],[0,1])},o.scale.identity=function(){return function t(e){function n(t){return+t}n.invert=n;n.domain=n.range=function(t){return arguments.length?(e=t.map(n),n):e};n.ticks=function(t){return Ra(e,t)};n.tickFormat=function(t,n){return Na(e,t,n)};n.copy=function(){return t(e)};return n}([0,1])},o.svg={},o.svg.arc=function(){var t=Ja,e=qa,n=Ka,r=za,i=Va,o=Wa,a=Xa;function s(){var s=Math.max(0,+t.apply(this,arguments)),l=Math.max(0,+e.apply(this,arguments)),c=i.apply(this,arguments)-Nt,u=o.apply(this,arguments)-Nt,g=Math.abs(u-c),h=c>u?0:1;if(l<s&&(d=l,l=s,s=d),g>=Rt)return A(l,h)+(s?A(s,1-h):"")+"Z";var d,f,p,I,C,m,y,E,v,B,b,Q,w=0,x=0,_=[];if((I=(+a.apply(this,arguments)||0)/2)&&(p=r===za?Math.sqrt(s*s+l*l):+r.apply(this,arguments),h||(x*=-1),l&&(x=Ht(p/l*Math.sin(I))),s&&(w=Ht(p/s*Math.sin(I)))),l){C=l*Math.cos(c+x),m=l*Math.sin(c+x),y=l*Math.cos(u-x),E=l*Math.sin(u-x);var D=Math.abs(u-c-2*x)<=St?0:1;if(x&&Za(C,m,y,E)===h^D){var M=(c+u)/2;C=l*Math.cos(M),m=l*Math.sin(M),y=E=null}}else C=m=0;if(s){v=s*Math.cos(u-w),B=s*Math.sin(u-w),b=s*Math.cos(c+w),Q=s*Math.sin(c+w);var k=Math.abs(c-u+2*w)<=St?0:1;if(w&&Za(v,B,b,Q)===1-h^k){var S=(c+u)/2;v=s*Math.cos(S),B=s*Math.sin(S),b=Q=null}}else v=B=0;if(g>Mt&&(d=Math.min(Math.abs(l-s)/2,+n.apply(this,arguments)))>.001){f=s<l^h?0:1;var L=d,R=d;if(g<St){var N=null==b?[v,B]:null==y?[C,m]:Ci([C,m],[b,Q],[y,E],[v,B]),F=C-N[0],T=m-N[1],U=y-N[0],O=E-N[1],G=1/Math.sin(Math.acos((F*U+T*O)/(Math.sqrt(F*F+T*T)*Math.sqrt(U*U+O*O)))/2),H=Math.sqrt(N[0]*N[0]+N[1]*N[1]);R=Math.min(d,(s-H)/(G-1)),L=Math.min(d,(l-H)/(G+1))}if(null!=y){var j=$a(null==b?[v,B]:[b,Q],[C,m],l,L,h),Y=$a([y,E],[v,B],l,L,h);d===L?_.push("M",j[0],"A",L,",",L," 0 0,",f," ",j[1],"A",l,",",l," 0 ",1-h^Za(j[1][0],j[1][1],Y[1][0],Y[1][1]),",",h," ",Y[1],"A",L,",",L," 0 0,",f," ",Y[0]):_.push("M",j[0],"A",L,",",L," 0 1,",f," ",Y[0])}else _.push("M",C,",",m);if(null!=b){var P=$a([C,m],[b,Q],s,-R,h),K=$a([v,B],null==y?[C,m]:[y,E],s,-R,h);d===R?_.push("L",K[0],"A",R,",",R," 0 0,",f," ",K[1],"A",s,",",s," 0 ",h^Za(K[1][0],K[1][1],P[1][0],P[1][1]),",",1-h," ",P[1],"A",R,",",R," 0 0,",f," ",P[0]):_.push("L",K[0],"A",R,",",R," 0 0,",f," ",P[0])}else _.push("L",v,",",B)}else _.push("M",C,",",m),null!=y&&_.push("A",l,",",l," 0 ",D,",",h," ",y,",",E),_.push("L",v,",",B),null!=b&&_.push("A",s,",",s," 0 ",k,",",1-h," ",b,",",Q);return _.push("Z"),_.join("")}function A(t,e){return"M0,"+t+"A"+t+","+t+" 0 1,"+e+" 0,"+-t+"A"+t+","+t+" 0 1,"+e+" 0,"+t}return s.innerRadius=function(e){return arguments.length?(t=Be(e),s):t},s.outerRadius=function(t){return arguments.length?(e=Be(t),s):e},s.cornerRadius=function(t){return arguments.length?(n=Be(t),s):n},s.padRadius=function(t){return arguments.length?(r=t==za?za:Be(t),s):r},s.startAngle=function(t){return arguments.length?(i=Be(t),s):i},s.endAngle=function(t){return arguments.length?(o=Be(t),s):o},s.padAngle=function(t){return arguments.length?(a=Be(t),s):a},s.centroid=function(){var n=(+t.apply(this,arguments)+ +e.apply(this,arguments))/2,r=(+i.apply(this,arguments)+ +o.apply(this,arguments))/2-Nt;return[Math.cos(r)*n,Math.sin(r)*n]},s};var za="auto";function Ja(t){return t.innerRadius}function qa(t){return t.outerRadius}function Va(t){return t.startAngle}function Wa(t){return t.endAngle}function Xa(t){return t&&t.padAngle}function Za(t,e,n,r){return(t-n)*e-(e-r)*t>0?0:1}function $a(t,e,n,r,i){var o=t[0]-e[0],a=t[1]-e[1],s=(i?r:-r)/Math.sqrt(o*o+a*a),A=s*a,l=-s*o,c=t[0]+A,u=t[1]+l,g=e[0]+A,h=e[1]+l,d=(c+g)/2,f=(u+h)/2,p=g-c,I=h-u,C=p*p+I*I,m=n-r,y=c*h-g*u,E=(I<0?-1:1)*Math.sqrt(Math.max(0,m*m*C-y*y)),v=(y*I-p*E)/C,B=(-y*p-I*E)/C,b=(y*I+p*E)/C,Q=(-y*p+I*E)/C,w=v-d,x=B-f,_=b-d,D=Q-f;return w*w+x*x>_*_+D*D&&(v=b,B=Q),[[v-A,B-l],[v*n/m,B*n/m]]}function ts(t){var e=gi,n=hi,r=nr,i=ns,o=i.key,a=.7;function s(o){var s,A=[],l=[],c=-1,u=o.length,g=Be(e),h=Be(n);function d(){A.push("M",i(t(l),a))}for(;++c<u;)r.call(this,s=o[c],c)?l.push([+g.call(this,s,c),+h.call(this,s,c)]):l.length&&(d(),l=[]);return l.length&&d(),A.length?A.join(""):null}return s.x=function(t){return arguments.length?(e=t,s):e},s.y=function(t){return arguments.length?(n=t,s):n},s.defined=function(t){return arguments.length?(r=t,s):r},s.interpolate=function(t){return arguments.length?(o="function"==typeof t?i=t:(i=es.get(t)||ns).key,s):o},s.tension=function(t){return arguments.length?(a=t,s):a},s}o.svg.line=function(){return ts(F)};var es=o.map({linear:ns,"linear-closed":rs,step:function(t){var e=0,n=t.length,r=t[0],i=[r[0],",",r[1]];for(;++e<n;)i.push("H",(r[0]+(r=t[e])[0])/2,"V",r[1]);n>1&&i.push("H",r[0]);return i.join("")},"step-before":is,"step-after":os,basis:As,"basis-open":function(t){if(t.length<4)return ns(t);var e,n=[],r=-1,i=t.length,o=[0],a=[0];for(;++r<3;)e=t[r],o.push(e[0]),a.push(e[1]);n.push(ls(gs,o)+","+ls(gs,a)),--r;for(;++r<i;)e=t[r],o.shift(),o.push(e[0]),a.shift(),a.push(e[1]),hs(n,o,a);return n.join("")},"basis-closed":function(t){var e,n,r=-1,i=t.length,o=i+4,a=[],s=[];for(;++r<4;)n=t[r%i],a.push(n[0]),s.push(n[1]);e=[ls(gs,a),",",ls(gs,s)],--r;for(;++r<o;)n=t[r%i],a.shift(),a.push(n[0]),s.shift(),s.push(n[1]),hs(e,a,s);return e.join("")},bundle:function(t,e){var n=t.length-1;if(n)for(var r,i,o=t[0][0],a=t[0][1],s=t[n][0]-o,A=t[n][1]-a,l=-1;++l<=n;)r=t[l],i=l/n,r[0]=e*r[0]+(1-e)*(o+i*s),r[1]=e*r[1]+(1-e)*(a+i*A);return As(t)},cardinal:function(t,e){return t.length<3?ns(t):t[0]+as(t,ss(t,e))},"cardinal-open":function(t,e){return t.length<4?ns(t):t[1]+as(t.slice(1,-1),ss(t,e))},"cardinal-closed":function(t,e){return t.length<3?rs(t):t[0]+as((t.push(t[0]),t),ss([t[t.length-2]].concat(t,[t[1]]),e))},monotone:function(t){return t.length<3?ns(t):t[0]+as(t,function(t){var e,n,r,i,o=[],a=function(t){var e=0,n=t.length-1,r=[],i=t[0],o=t[1],a=r[0]=ds(i,o);for(;++e<n;)r[e]=(a+(a=ds(i=o,o=t[e+1])))/2;return r[e]=a,r}(t),s=-1,A=t.length-1;for(;++s<A;)e=ds(t[s],t[s+1]),v(e)<Mt?a[s]=a[s+1]=0:(n=a[s]/e,r=a[s+1]/e,(i=n*n+r*r)>9&&(i=3*e/Math.sqrt(i),a[s]=i*n,a[s+1]=i*r));s=-1;for(;++s<=A;)i=(t[Math.min(A,s+1)][0]-t[Math.max(0,s-1)][0])/(6*(1+a[s]*a[s])),o.push([i||0,a[s]*i||0]);return o}(t))}});function ns(t){return t.length>1?t.join("L"):t+"Z"}function rs(t){return t.join("L")+"Z"}function is(t){for(var e=0,n=t.length,r=t[0],i=[r[0],",",r[1]];++e<n;)i.push("V",(r=t[e])[1],"H",r[0]);return i.join("")}function os(t){for(var e=0,n=t.length,r=t[0],i=[r[0],",",r[1]];++e<n;)i.push("H",(r=t[e])[0],"V",r[1]);return i.join("")}function as(t,e){if(e.length<1||t.length!=e.length&&t.length!=e.length+2)return ns(t);var n=t.length!=e.length,r="",i=t[0],o=t[1],a=e[0],s=a,A=1;if(n&&(r+="Q"+(o[0]-2*a[0]/3)+","+(o[1]-2*a[1]/3)+","+o[0]+","+o[1],i=t[1],A=2),e.length>1){s=e[1],o=t[A],A++,r+="C"+(i[0]+a[0])+","+(i[1]+a[1])+","+(o[0]-s[0])+","+(o[1]-s[1])+","+o[0]+","+o[1];for(var l=2;l<e.length;l++,A++)o=t[A],s=e[l],r+="S"+(o[0]-s[0])+","+(o[1]-s[1])+","+o[0]+","+o[1]}if(n){var c=t[A];r+="Q"+(o[0]+2*s[0]/3)+","+(o[1]+2*s[1]/3)+","+c[0]+","+c[1]}return r}function ss(t,e){for(var n,r=[],i=(1-e)/2,o=t[0],a=t[1],s=1,A=t.length;++s<A;)n=o,o=a,a=t[s],r.push([i*(a[0]-n[0]),i*(a[1]-n[1])]);return r}function As(t){if(t.length<3)return ns(t);var e=1,n=t.length,r=t[0],i=r[0],o=r[1],a=[i,i,i,(r=t[1])[0]],s=[o,o,o,r[1]],A=[i,",",o,"L",ls(gs,a),",",ls(gs,s)];for(t.push(t[n-1]);++e<=n;)r=t[e],a.shift(),a.push(r[0]),s.shift(),s.push(r[1]),hs(A,a,s);return t.pop(),A.push("L",r),A.join("")}function ls(t,e){return t[0]*e[0]+t[1]*e[1]+t[2]*e[2]+t[3]*e[3]}es.forEach(function(t,e){e.key=t,e.closed=/-closed$/.test(t)});var cs=[0,2/3,1/3,0],us=[0,1/3,2/3,0],gs=[0,1/6,2/3,1/6];function hs(t,e,n){t.push("C",ls(cs,e),",",ls(cs,n),",",ls(us,e),",",ls(us,n),",",ls(gs,e),",",ls(gs,n))}function ds(t,e){return(e[1]-t[1])/(e[0]-t[0])}function fs(t){for(var e,n,r,i=-1,o=t.length;++i<o;)n=(e=t[i])[0],r=e[1]-Nt,e[0]=n*Math.cos(r),e[1]=n*Math.sin(r);return t}function ps(t){var e=gi,n=gi,r=0,i=hi,o=nr,a=ns,s=a.key,A=a,l="L",c=.7;function u(s){var u,g,h,d=[],f=[],p=[],I=-1,C=s.length,m=Be(e),y=Be(r),E=e===n?function(){return g}:Be(n),v=r===i?function(){return h}:Be(i);function B(){d.push("M",a(t(p),c),l,A(t(f.reverse()),c),"Z")}for(;++I<C;)o.call(this,u=s[I],I)?(f.push([g=+m.call(this,u,I),h=+y.call(this,u,I)]),p.push([+E.call(this,u,I),+v.call(this,u,I)])):f.length&&(B(),f=[],p=[]);return f.length&&B(),d.length?d.join(""):null}return u.x=function(t){return arguments.length?(e=n=t,u):n},u.x0=function(t){return arguments.length?(e=t,u):e},u.x1=function(t){return arguments.length?(n=t,u):n},u.y=function(t){return arguments.length?(r=i=t,u):i},u.y0=function(t){return arguments.length?(r=t,u):r},u.y1=function(t){return arguments.length?(i=t,u):i},u.defined=function(t){return arguments.length?(o=t,u):o},u.interpolate=function(t){return arguments.length?(s="function"==typeof t?a=t:(a=es.get(t)||ns).key,A=a.reverse||a,l=a.closed?"M":"L",u):s},u.tension=function(t){return arguments.length?(c=t,u):c},u}function Is(t){return t.radius}function Cs(t){return[t.x,t.y]}function ms(t){return function(){var e=t.apply(this,arguments),n=e[0],r=e[1]-Nt;return[n*Math.cos(r),n*Math.sin(r)]}}function ys(){return 64}function Es(){return"circle"}function vs(t){var e=Math.sqrt(t/St);return"M0,"+e+"A"+e+","+e+" 0 1,1 0,"+-e+"A"+e+","+e+" 0 1,1 0,"+e+"Z"}o.svg.line.radial=function(){var t=ts(fs);return t.radius=t.x,delete t.x,t.angle=t.y,delete t.y,t},is.reverse=os,os.reverse=is,o.svg.area=function(){return ps(F)},o.svg.area.radial=function(){var t=ps(fs);return t.radius=t.x,delete t.x,t.innerRadius=t.x0,delete t.x0,t.outerRadius=t.x1,delete t.x1,t.angle=t.y,delete t.y,t.startAngle=t.y0,delete t.y0,t.endAngle=t.y1,delete t.y1,t},o.svg.chord=function(){var t=Zr,e=$r,n=Is,r=Va,i=Wa;function o(n,r){var i,o,l=a(this,t,n,r),c=a(this,e,n,r);return"M"+l.p0+s(l.r,l.p1,l.a1-l.a0)+(o=c,(i=l).a0==o.a0&&i.a1==o.a1?A(l.r,l.p1,l.r,l.p0):A(l.r,l.p1,c.r,c.p0)+s(c.r,c.p1,c.a1-c.a0)+A(c.r,c.p1,l.r,l.p0))+"Z"}function a(t,e,o,a){var s=e.call(t,o,a),A=n.call(t,s,a),l=r.call(t,s,a)-Nt,c=i.call(t,s,a)-Nt;return{r:A,a0:l,a1:c,p0:[A*Math.cos(l),A*Math.sin(l)],p1:[A*Math.cos(c),A*Math.sin(c)]}}function s(t,e,n){return"A"+t+","+t+" 0 "+ +(n>St)+",1 "+e}function A(t,e,n,r){return"Q 0,0 "+r}return o.radius=function(t){return arguments.length?(n=Be(t),o):n},o.source=function(e){return arguments.length?(t=Be(e),o):t},o.target=function(t){return arguments.length?(e=Be(t),o):e},o.startAngle=function(t){return arguments.length?(r=Be(t),o):r},o.endAngle=function(t){return arguments.length?(i=Be(t),o):i},o},o.svg.diagonal=function(){var t=Zr,e=$r,n=Cs;function r(r,i){var o=t.call(this,r,i),a=e.call(this,r,i),s=(o.y+a.y)/2,A=[o,{x:o.x,y:s},{x:a.x,y:s},a];return"M"+(A=A.map(n))[0]+"C"+A[1]+" "+A[2]+" "+A[3]}return r.source=function(e){return arguments.length?(t=Be(e),r):t},r.target=function(t){return arguments.length?(e=Be(t),r):e},r.projection=function(t){return arguments.length?(n=t,r):n},r},o.svg.diagonal.radial=function(){var t=o.svg.diagonal(),e=Cs,n=t.projection;return t.projection=function(t){return arguments.length?n(ms(e=t)):e},t},o.svg.symbol=function(){var t=Es,e=ys;function n(n,r){return(Bs.get(t.call(this,n,r))||vs)(e.call(this,n,r))}return n.type=function(e){return arguments.length?(t=Be(e),n):t},n.size=function(t){return arguments.length?(e=Be(t),n):e},n};var Bs=o.map({circle:vs,cross:function(t){var e=Math.sqrt(t/5)/2;return"M"+-3*e+","+-e+"H"+-e+"V"+-3*e+"H"+e+"V"+-e+"H"+3*e+"V"+e+"H"+e+"V"+3*e+"H"+-e+"V"+e+"H"+-3*e+"Z"},diamond:function(t){var e=Math.sqrt(t/(2*Qs)),n=e*Qs;return"M0,"+-e+"L"+n+",0 0,"+e+" "+-n+",0Z"},square:function(t){var e=Math.sqrt(t)/2;return"M"+-e+","+-e+"L"+e+","+-e+" "+e+","+e+" "+-e+","+e+"Z"},"triangle-down":function(t){var e=Math.sqrt(t/bs),n=e*bs/2;return"M0,"+n+"L"+e+","+-n+" "+-e+","+-n+"Z"},"triangle-up":function(t){var e=Math.sqrt(t/bs),n=e*bs/2;return"M0,"+-n+"L"+e+","+n+" "+-e+","+n+"Z"}});o.svg.symbolTypes=Bs.keys();var bs=Math.sqrt(3),Qs=Math.tan(30*Ft);Z.transition=function(t){for(var e,n,r=Ds||++Ss,i=Ns(t),o=[],a=Ms||{time:Date.now(),ease:mo,delay:0,duration:250},s=-1,A=this.length;++s<A;){o.push(e=[]);for(var l=this[s],c=-1,u=l.length;++c<u;)(n=l[c])&&Fs(n,c,i,r,a),e.push(n)}return _s(o,i,r)},Z.interrupt=function(t){return this.each(null==t?ws:xs(Ns(t)))};var ws=xs(Ns());function xs(t){return function(){var e,n,r;(e=this[t])&&(r=e[n=e.active])&&(r.timer.c=null,r.timer.t=NaN,--e.count?delete e[n]:delete this[t],e.active+=.5,r.event&&r.event.interrupt.call(this,this.__data__,r.index))}}function _s(t,e,n){return J(t,ks),t.namespace=e,t.id=n,t}var Ds,Ms,ks=[],Ss=0;function Ls(t,e,n,r){var i=t.id,o=t.namespace;return pt(t,"function"==typeof n?function(t,a,s){t[o][i].tween.set(e,r(n.call(t,t.__data__,a,s)))}:(n=r(n),function(t){t[o][i].tween.set(e,n)}))}function Rs(t){return null==t&&(t=""),function(){this.textContent=t}}function Ns(t){return null==t?"__transition__":"__transition_"+t+"__"}function Fs(t,e,n,r,i){var o,a,s,A,l,c=t[n]||(t[n]={active:0,count:0}),u=c[r];function g(n){var i=c.active,g=c[i];for(var d in g&&(g.timer.c=null,g.timer.t=NaN,--c.count,delete c[i],g.event&&g.event.interrupt.call(t,t.__data__,g.index)),c)if(+d<r){var f=c[d];f.timer.c=null,f.timer.t=NaN,--c.count,delete c[d]}a.c=h,ke(function(){return a.c&&h(n||1)&&(a.c=null,a.t=NaN),1},0,o),c.active=r,u.event&&u.event.start.call(t,t.__data__,e),l=[],u.tween.forEach(function(n,r){(r=r.call(t,t.__data__,e))&&l.push(r)}),A=u.ease,s=u.duration}function h(i){for(var o=i/s,a=A(o),g=l.length;g>0;)l[--g].call(t,a);if(o>=1)return u.event&&u.event.end.call(t,t.__data__,e),--c.count?delete c[r]:delete t[n],1}u||(o=i.time,a=ke(function(t){var e=u.delay;if(a.t=e+o,e<=t)return g(t-e);a.c=g},0,o),u=c[r]={tween:new Q,time:o,timer:a,delay:i.delay,duration:i.duration,ease:i.ease,index:e},i=null,++c.count)}ks.call=Z.call,ks.empty=Z.empty,ks.node=Z.node,ks.size=Z.size,o.transition=function(t,e){return t&&t.transition?Ds?t.transition(e):t:o.selection().transition(t)},o.transition.prototype=ks,ks.select=function(t){var e,n,r,i=this.id,o=this.namespace,a=[];t=$(t);for(var s=-1,A=this.length;++s<A;){a.push(e=[]);for(var l=this[s],c=-1,u=l.length;++c<u;)(r=l[c])&&(n=t.call(r,r.__data__,c,s))?("__data__"in r&&(n.__data__=r.__data__),Fs(n,c,o,i,r[o][i]),e.push(n)):e.push(null)}return _s(a,o,i)},ks.selectAll=function(t){var e,n,r,i,o,a=this.id,s=this.namespace,A=[];t=tt(t);for(var l=-1,c=this.length;++l<c;)for(var u=this[l],g=-1,h=u.length;++g<h;)if(r=u[g]){o=r[s][a],n=t.call(r,r.__data__,g,l),A.push(e=[]);for(var d=-1,f=n.length;++d<f;)(i=n[d])&&Fs(i,d,s,a,o),e.push(i)}return _s(A,s,a)},ks.filter=function(t){var e,n,r=[];"function"!=typeof t&&(t=dt(t));for(var i=0,o=this.length;i<o;i++){r.push(e=[]);for(var a,s=0,A=(a=this[i]).length;s<A;s++)(n=a[s])&&t.call(n,n.__data__,s,i)&&e.push(n)}return _s(r,this.namespace,this.id)},ks.tween=function(t,e){var n=this.id,r=this.namespace;return arguments.length<2?this.node()[r][n].tween.get(t):pt(this,null==e?function(e){e[r][n].tween.remove(t)}:function(i){i[r][n].tween.set(t,e)})},ks.attr=function(t,e){if(arguments.length<2){for(e in t)this.attr(e,t[e]);return this}var n="transform"==t?Mo:Ao,r=o.ns.qualify(t);function i(){this.removeAttribute(r)}function a(){this.removeAttributeNS(r.space,r.local)}function s(t){return null==t?i:(t+="",function(){var e,i=this.getAttribute(r);return i!==t&&(e=n(i,t),function(t){this.setAttribute(r,e(t))})})}function A(t){return null==t?a:(t+="",function(){var e,i=this.getAttributeNS(r.space,r.local);return i!==t&&(e=n(i,t),function(t){this.setAttributeNS(r.space,r.local,e(t))})})}return Ls(this,"attr."+t,e,r.local?A:s)},ks.attrTween=function(t,e){var n=o.ns.qualify(t);return this.tween("attr."+t,n.local?function(t,r){var i=e.call(this,t,r,this.getAttributeNS(n.space,n.local));return i&&function(t){this.setAttributeNS(n.space,n.local,i(t))}}:function(t,r){var i=e.call(this,t,r,this.getAttribute(n));return i&&function(t){this.setAttribute(n,i(t))}})},ks.style=function(t,e,n){var r=arguments.length;if(r<3){if("string"!=typeof t){for(n in r<2&&(e=""),t)this.style(n,t[n],e);return this}n=""}function i(){this.style.removeProperty(t)}function o(e){return null==e?i:(e+="",function(){var r,i=c(this).getComputedStyle(this,null).getPropertyValue(t);return i!==e&&(r=Ao(i,e),function(e){this.style.setProperty(t,r(e),n)})})}return Ls(this,"style."+t,e,o)},ks.styleTween=function(t,e,n){function r(r,i){var o=e.call(this,r,i,c(this).getComputedStyle(this,null).getPropertyValue(t));return o&&function(e){this.style.setProperty(t,o(e),n)}}return arguments.length<3&&(n=""),this.tween("style."+t,r)},ks.text=function(t){return Ls(this,"text",t,Rs)},ks.remove=function(){var t=this.namespace;return this.each("end.transition",function(){var e;this[t].count<2&&(e=this.parentNode)&&e.removeChild(this)})},ks.ease=function(t){var e=this.id,n=this.namespace;return arguments.length<1?this.node()[n][e].ease:("function"!=typeof t&&(t=o.ease.apply(o,arguments)),pt(this,function(r){r[n][e].ease=t}))},ks.delay=function(t){var e=this.id,n=this.namespace;return arguments.length<1?this.node()[n][e].delay:pt(this,"function"==typeof t?function(r,i,o){r[n][e].delay=+t.call(r,r.__data__,i,o)}:(t=+t,function(r){r[n][e].delay=t}))},ks.duration=function(t){var e=this.id,n=this.namespace;return arguments.length<1?this.node()[n][e].duration:pt(this,"function"==typeof t?function(r,i,o){r[n][e].duration=Math.max(1,t.call(r,r.__data__,i,o))}:(t=Math.max(1,t),function(r){r[n][e].duration=t}))},ks.each=function(t,e){var n=this.id,r=this.namespace;if(arguments.length<2){var i=Ms,a=Ds;try{Ds=n,pt(this,function(e,i,o){Ms=e[r][n],t.call(e,e.__data__,i,o)})}finally{Ms=i,Ds=a}}else pt(this,function(i){var a=i[r][n];(a.event||(a.event=o.dispatch("start","end","interrupt"))).on(t,e)});return this},ks.transition=function(){for(var t,e,n,r=this.id,i=++Ss,o=this.namespace,a=[],s=0,A=this.length;s<A;s++){a.push(t=[]);for(var l,c=0,u=(l=this[s]).length;c<u;c++)(e=l[c])&&Fs(e,c,o,i,{time:(n=e[o][r]).time,ease:n.ease,delay:n.delay+n.duration,duration:n.duration}),t.push(e)}return _s(a,o,i)},o.svg.axis=function(){var t,e=o.scale.linear(),n=Ts,r=6,i=6,a=3,A=[10],l=null;function c(s){s.each(function(){var s,c=o.select(this),u=this.__chart__||e,g=this.__chart__=e.copy(),h=null==l?g.ticks?g.ticks.apply(g,A):g.domain():l,d=null==t?g.tickFormat?g.tickFormat.apply(g,A):F:t,f=c.selectAll(".tick").data(h,g),p=f.enter().insert("g",".domain").attr("class","tick").style("opacity",Mt),I=o.transition(f.exit()).style("opacity",Mt).remove(),C=o.transition(f.order()).style("opacity",1),m=Math.max(r,0)+a,y=Qa(g),E=c.selectAll(".domain").data([0]),v=(E.enter().append("path").attr("class","domain"),o.transition(E));p.append("line"),p.append("text");var B,b,Q,w,x=p.select("line"),_=C.select("line"),D=f.select("text").text(d),M=p.select("text"),k=C.select("text"),S="top"===n||"left"===n?-1:1;if("bottom"===n||"top"===n?(s=Os,B="x",Q="y",b="x2",w="y2",D.attr("dy",S<0?"0em":".71em").style("text-anchor","middle"),v.attr("d","M"+y[0]+","+S*i+"V0H"+y[1]+"V"+S*i)):(s=Gs,B="y",Q="x",b="y2",w="x2",D.attr("dy",".32em").style("text-anchor",S<0?"end":"start"),v.attr("d","M"+S*i+","+y[0]+"H0V"+y[1]+"H"+S*i)),x.attr(w,S*r),M.attr(Q,S*m),_.attr(b,0).attr(w,S*r),k.attr(B,0).attr(Q,S*m),g.rangeBand){var L=g,R=L.rangeBand()/2;u=g=function(t){return L(t)+R}}else u.rangeBand?u=g:I.call(s,g,u);p.call(s,u,g),C.call(s,g,g)})}return c.scale=function(t){return arguments.length?(e=t,c):e},c.orient=function(t){return arguments.length?(n=t in Us?t+"":Ts,c):n},c.ticks=function(){return arguments.length?(A=s(arguments),c):A},c.tickValues=function(t){return arguments.length?(l=t,c):l},c.tickFormat=function(e){return arguments.length?(t=e,c):t},c.tickSize=function(t){var e=arguments.length;return e?(r=+t,i=+arguments[e-1],c):r},c.innerTickSize=function(t){return arguments.length?(r=+t,c):r},c.outerTickSize=function(t){return arguments.length?(i=+t,c):i},c.tickPadding=function(t){return arguments.length?(a=+t,c):a},c.tickSubdivide=function(){return arguments.length&&c},c};var Ts="bottom",Us={top:1,right:1,bottom:1,left:1};function Os(t,e,n){t.attr("transform",function(t){var r=e(t);return"translate("+(isFinite(r)?r:n(t))+",0)"})}function Gs(t,e,n){t.attr("transform",function(t){var r=e(t);return"translate(0,"+(isFinite(r)?r:n(t))+")"})}o.svg.brush=function(){var t,e,n=K(g,"brushstart","brush","brushend"),r=null,i=null,a=[0,0],s=[0,0],A=!0,l=!0,u=js[0];function g(t){t.each(function(){var t=o.select(this).style("pointer-events","all").style("-webkit-tap-highlight-color","rgba(0,0,0,0)").on("mousedown.brush",p).on("touchstart.brush",p),e=t.selectAll(".background").data([0]);e.enter().append("rect").attr("class","background").style("visibility","hidden").style("cursor","crosshair"),t.selectAll(".extent").data([0]).enter().append("rect").attr("class","extent").style("cursor","move");var n=t.selectAll(".resize").data(u,F);n.exit().remove(),n.enter().append("g").attr("class",function(t){return"resize "+t}).style("cursor",function(t){return Hs[t]}).append("rect").attr("x",function(t){return/[ew]$/.test(t)?-3:null}).attr("y",function(t){return/^[ns]/.test(t)?-3:null}).attr("width",6).attr("height",6).style("visibility","hidden"),n.style("display",g.empty()?"none":null);var a,s=o.transition(t),A=o.transition(e);r&&(a=Qa(r),A.attr("x",a[0]).attr("width",a[1]-a[0]),d(s)),i&&(a=Qa(i),A.attr("y",a[0]).attr("height",a[1]-a[0]),f(s)),h(s)})}function h(t){t.selectAll(".resize").attr("transform",function(t){return"translate("+a[+/e$/.test(t)]+","+s[+/^s/.test(t)]+")"})}function d(t){t.select(".extent").attr("x",a[0]),t.selectAll(".extent,.n>rect,.s>rect").attr("width",a[1]-a[0])}function f(t){t.select(".extent").attr("y",s[0]),t.selectAll(".extent,.e>rect,.w>rect").attr("height",s[1]-s[0])}function p(){var u,p,I=this,C=o.select(o.event.target),m=n.of(I,arguments),y=o.select(I),E=C.datum(),v=!/^(n|s)$/.test(E)&&r,B=!/^(e|w)$/.test(E)&&i,b=C.classed("extent"),Q=wt(I),w=o.mouse(I),x=o.select(c(I)).on("keydown.brush",function(){32==o.event.keyCode&&(b||(u=null,w[0]-=a[1],w[1]-=s[1],b=2),Y())}).on("keyup.brush",function(){32==o.event.keyCode&&2==b&&(w[0]+=a[1],w[1]+=s[1],b=0,Y())});if(o.event.changedTouches?x.on("touchmove.brush",M).on("touchend.brush",S):x.on("mousemove.brush",M).on("mouseup.brush",S),y.interrupt().selectAll("*").interrupt(),b)w[0]=a[0]-w[0],w[1]=s[0]-w[1];else if(E){var _=+/w$/.test(E),D=+/^n/.test(E);p=[a[1-_]-w[0],s[1-D]-w[1]],w[0]=a[_],w[1]=s[D]}else o.event.altKey&&(u=w.slice());function M(){var t=o.mouse(I),e=!1;p&&(t[0]+=p[0],t[1]+=p[1]),b||(o.event.altKey?(u||(u=[(a[0]+a[1])/2,(s[0]+s[1])/2]),w[0]=a[+(t[0]<u[0])],w[1]=s[+(t[1]<u[1])]):u=null),v&&k(t,r,0)&&(d(y),e=!0),B&&k(t,i,1)&&(f(y),e=!0),e&&(h(y),m({type:"brush",mode:b?"move":"resize"}))}function k(n,r,i){var o,c,g=Qa(r),h=g[0],d=g[1],f=w[i],p=i?s:a,I=p[1]-p[0];if(b&&(h-=f,d-=I+f),o=(i?l:A)?Math.max(h,Math.min(d,n[i])):n[i],b?c=(o+=f)+I:(u&&(f=Math.max(h,Math.min(d,2*u[i]-o))),f<o?(c=o,o=f):c=f),p[0]!=o||p[1]!=c)return i?e=null:t=null,p[0]=o,p[1]=c,!0}function S(){M(),y.style("pointer-events","all").selectAll(".resize").style("display",g.empty()?"none":null),o.select("body").style("cursor",null),x.on("mousemove.brush",null).on("mouseup.brush",null).on("touchmove.brush",null).on("touchend.brush",null).on("keydown.brush",null).on("keyup.brush",null),Q(),m({type:"brushend"})}y.style("pointer-events","none").selectAll(".resize").style("display",null),o.select("body").style("cursor",C.style("cursor")),m({type:"brushstart"}),M()}return g.event=function(r){r.each(function(){var r=n.of(this,arguments),i={x:a,y:s,i:t,j:e},A=this.__chart__||i;this.__chart__=i,Ds?o.select(this).transition().each("start.brush",function(){t=A.i,e=A.j,a=A.x,s=A.y,r({type:"brushstart"})}).tween("brush:brush",function(){var n=lo(a,i.x),o=lo(s,i.y);return t=e=null,function(t){a=i.x=n(t),s=i.y=o(t),r({type:"brush",mode:"resize"})}}).each("end.brush",function(){t=i.i,e=i.j,r({type:"brush",mode:"resize"}),r({type:"brushend"})}):(r({type:"brushstart"}),r({type:"brush",mode:"resize"}),r({type:"brushend"}))})},g.x=function(t){return arguments.length?(u=js[!(r=t)<<1|!i],g):r},g.y=function(t){return arguments.length?(u=js[!r<<1|!(i=t)],g):i},g.clamp=function(t){return arguments.length?(r&&i?(A=!!t[0],l=!!t[1]):r?A=!!t:i&&(l=!!t),g):r&&i?[A,l]:r?A:i?l:null},g.extent=function(n){var o,A,l,c,u;return arguments.length?(r&&(o=n[0],A=n[1],i&&(o=o[0],A=A[0]),t=[o,A],r.invert&&(o=r(o),A=r(A)),A<o&&(u=o,o=A,A=u),o==a[0]&&A==a[1]||(a=[o,A])),i&&(l=n[0],c=n[1],r&&(l=l[1],c=c[1]),e=[l,c],i.invert&&(l=i(l),c=i(c)),c<l&&(u=l,l=c,c=u),l==s[0]&&c==s[1]||(s=[l,c])),g):(r&&(t?(o=t[0],A=t[1]):(o=a[0],A=a[1],r.invert&&(o=r.invert(o),A=r.invert(A)),A<o&&(u=o,o=A,A=u))),i&&(e?(l=e[0],c=e[1]):(l=s[0],c=s[1],i.invert&&(l=i.invert(l),c=i.invert(c)),c<l&&(u=l,l=c,c=u))),r&&i?[[o,l],[A,c]]:r?[o,A]:i&&[l,c])},g.clear=function(){return g.empty()||(a=[0,0],s=[0,0],t=e=null),g},g.empty=function(){return!!r&&a[0]==a[1]||!!i&&s[0]==s[1]},o.rebind(g,n,"on")};var Hs={n:"ns-resize",e:"ew-resize",s:"ns-resize",w:"ew-resize",nw:"nwse-resize",ne:"nesw-resize",se:"nwse-resize",sw:"nesw-resize"},js=[["n","e","s","w","nw","ne","se","sw"],["e","w"],["n","s"],[]],Ys=He.format=In.timeFormat,Ps=Ys.utc,Ks=Ps("%Y-%m-%dT%H:%M:%S.%LZ");function zs(t){return t.toISOString()}function Js(t,e,n){function r(e){return t(e)}function i(t,n){var r=(t[1]-t[0])/n,i=o.bisect(Vs,r);return i==Vs.length?[e.year,La(t.map(function(t){return t/31536e6}),n)[2]]:i?e[r/Vs[i-1]<Vs[i]/r?i-1:i]:[Zs,La(t,n)[2]]}return r.invert=function(e){return qs(t.invert(e))},r.domain=function(e){return arguments.length?(t.domain(e),r):t.domain().map(qs)},r.nice=function(t,e){var n=r.domain(),o=ba(n),a=null==t?i(o,10):"number"==typeof t&&i(o,t);function s(n){return!isNaN(n)&&!t.range(n,qs(+n+1),e).length}return a&&(t=a[0],e=a[1]),r.domain(xa(n,e>1?{floor:function(e){for(;s(e=t.floor(e));)e=qs(e-1);return e},ceil:function(e){for(;s(e=t.ceil(e));)e=qs(+e+1);return e}}:t))},r.ticks=function(t,e){var n=ba(r.domain()),o=null==t?i(n,10):"number"==typeof t?i(n,t):!t.range&&[{range:t},e];return o&&(t=o[0],e=o[1]),t.range(n[0],qs(+n[1]+1),e<1?1:e)},r.tickFormat=function(){return n},r.copy=function(){return Js(t.copy(),e,n)},ka(r,t)}function qs(t){return new Date(t)}Ys.iso=Date.prototype.toISOString&&+new Date("2000-01-01T00:00:00.000Z")?zs:Ks,zs.parse=function(t){var e=new Date(t);return isNaN(e)?null:e},zs.toString=Ks.toString,He.second=Ke(function(t){return new je(1e3*Math.floor(t/1e3))},function(t,e){t.setTime(t.getTime()+1e3*Math.floor(e))},function(t){return t.getSeconds()}),He.seconds=He.second.range,He.seconds.utc=He.second.utc.range,He.minute=Ke(function(t){return new je(6e4*Math.floor(t/6e4))},function(t,e){t.setTime(t.getTime()+6e4*Math.floor(e))},function(t){return t.getMinutes()}),He.minutes=He.minute.range,He.minutes.utc=He.minute.utc.range,He.hour=Ke(function(t){var e=t.getTimezoneOffset()/60;return new je(36e5*(Math.floor(t/36e5-e)+e))},function(t,e){t.setTime(t.getTime()+36e5*Math.floor(e))},function(t){return t.getHours()}),He.hours=He.hour.range,He.hours.utc=He.hour.utc.range,He.month=Ke(function(t){return(t=He.day(t)).setDate(1),t},function(t,e){t.setMonth(t.getMonth()+e)},function(t){return t.getMonth()}),He.months=He.month.range,He.months.utc=He.month.utc.range;var Vs=[1e3,5e3,15e3,3e4,6e4,3e5,9e5,18e5,36e5,108e5,216e5,432e5,864e5,1728e5,6048e5,2592e6,7776e6,31536e6],Ws=[[He.second,1],[He.second,5],[He.second,15],[He.second,30],[He.minute,1],[He.minute,5],[He.minute,15],[He.minute,30],[He.hour,1],[He.hour,3],[He.hour,6],[He.hour,12],[He.day,1],[He.day,2],[He.week,1],[He.month,1],[He.month,3],[He.year,1]],Xs=Ys.multi([[".%L",function(t){return t.getMilliseconds()}],[":%S",function(t){return t.getSeconds()}],["%I:%M",function(t){return t.getMinutes()}],["%I %p",function(t){return t.getHours()}],["%a %d",function(t){return t.getDay()&&1!=t.getDate()}],["%b %d",function(t){return 1!=t.getDate()}],["%B",function(t){return t.getMonth()}],["%Y",nr]]),Zs={range:function(t,e,n){return o.range(Math.ceil(t/n)*n,+e,n).map(qs)},floor:F,ceil:F};Ws.year=He.year,He.scale=function(){return Js(o.scale.linear(),Ws,Xs)};var $s=Ws.map(function(t){return[t[0].utc,t[1]]}),tA=Ps.multi([[".%L",function(t){return t.getUTCMilliseconds()}],[":%S",function(t){return t.getUTCSeconds()}],["%I:%M",function(t){return t.getUTCMinutes()}],["%I %p",function(t){return t.getUTCHours()}],["%a %d",function(t){return t.getUTCDay()&&1!=t.getUTCDate()}],["%b %d",function(t){return 1!=t.getUTCDate()}],["%B",function(t){return t.getUTCMonth()}],["%Y",nr]]);function eA(t){return JSON.parse(t.responseText)}function nA(t){var e=A.createRange();return e.selectNode(A.body),e.createContextualFragment(t.responseText)}$s.year=He.year.utc,He.scale.utc=function(){return Js(o.scale.linear(),$s,tA)},o.text=be(function(t){return t.responseText}),o.json=function(t,e){return Qe(t,"application/json",eA,e)},o.html=function(t,e){return Qe(t,"text/html",nA,e)},o.xml=be(function(t){return t.responseXML}),this.d3=o,void 0===(i="function"==typeof(r=o)?r.call(e,n,e,t):r)||(t.exports=i)}()},function(t,e,n){"use strict";var r=Array.isArray,i="undefined"!=typeof ArrayBuffer&&ArrayBuffer.isView?ArrayBuffer:{isView:function(){return!1}},o="undefined"==typeof DataView?function(){}:DataView;function a(t){return i.isView(t)&&!(t instanceof o)}function s(t){return r(t)||a(t)}function A(t,e,n){if(s(t)){if(s(t[0])){for(var r=n,i=0;i<t.length;i++)r=e(r,t[i].length);return r}return t.length}return 0}e.isTypedArray=a,e.isArrayOrTypedArray=s,e.isArray1D=function(t){return!s(t[0])},e.ensureArray=function(t,e){return r(t)||(t=[]),t.length=e,t},e.concat=function(){var t,e,n,i,o,a,s,A,l=[],c=!0,u=0;for(n=0;n<arguments.length;n++)(a=(i=arguments[n]).length)&&(e?l.push(i):(e=i,o=a),r(i)?t=!1:(c=!1,u?t!==i.constructor&&(t=!1):t=i.constructor),u+=a);if(!u)return[];if(!l.length)return e;if(c)return e.concat.apply(e,l);if(t){for((s=new t(u)).set(e),n=0;n<l.length;n++)i=l[n],s.set(i,o),o+=i.length;return s}for(s=new Array(u),A=0;A<e.length;A++)s[A]=e[A];for(n=0;n<l.length;n++){for(i=l[n],A=0;A<i.length;A++)s[o+A]=i[A];o+=A}return s},e.maxRowLength=function(t){return A(t,Math.max,0)},e.minRowLength=function(t){return A(t,Math.min,1/0)}},function(t,e,n){"use strict";t.exports={mod:function(t,e){var n=t%e;return n<0?n+e:n},modHalf:function(t,e){return Math.abs(t)>e/2?t-Math.round(t/e)*e:t}}},function(t,e,n){"use strict";var r=n(1431).dfltConfig,i=t.exports={};function o(t,e){if(t&&t.apply)try{return void t.apply(console,e)}catch(t){}for(var n=0;n<e.length;n++)try{t(e[n])}catch(t){console.log(e[n])}}i.log=function(){if(r.logging>1){for(var t=["LOG:"],e=0;e<arguments.length;e++)t.push(arguments[e]);o(console.trace||console.log,t)}},i.warn=function(){if(r.logging>0){for(var t=["WARN:"],e=0;e<arguments.length;e++)t.push(arguments[e]);o(console.trace||console.log,t)}},i.error=function(){if(r.logging>0){for(var t=["ERROR:"],e=0;e<arguments.length;e++)t.push(arguments[e]);o(console.error,t)}}},function(t,e,n){"use strict";n.r(e);var r=n(5),i=n(27);const o=5,a="jp-dragImage",s="jp-dragImage-singlePrompt",A="jp-dragImage-content",l="jp-dragImage-prompt",c="jp-dragImage-multipleBack";var u,g;(g=u||(u={})).findCell=function(t,e,n){let i=-1;for(;t&&t.parentElement;){if(n(t)){Object(r.each)(e,(e,n)=>{if(e.node===t)return i=n,!1});break}t=t.parentElement}return i},g.detectTargetArea=function(t,e){let n=null;return n=t?t.editorWidget.node.contains(e)?"input":t.promptNode.contains(e)?"prompt":"cell":"unknown"},g.shouldStartDrag=function(t,e,n,r){let i=Math.abs(n-t),a=Math.abs(r-e);return i>=o||a>=o},g.createCellDragImage=function(t,e){const n=e.length;let r;if("code"===t.model.type){let e=t.model.executionCount;r=" ",e&&(r=e.toString())}else r="";const o=t.model.value.text.split("\n")[0].slice(0,26);return n>1?""!==r?i.VirtualDOM.realize(i.h.div(i.h.div({className:a},i.h.span({className:l},"["+r+"]:"),i.h.span({className:A},o)),i.h.div({className:c},""))):i.VirtualDOM.realize(i.h.div(i.h.div({className:a},i.h.span({className:l}),i.h.span({className:A},o)),i.h.div({className:c},""))):""!==r?i.VirtualDOM.realize(i.h.div(i.h.div({className:`${a} ${s}`},i.h.span({className:l},"["+r+"]:"),i.h.span({className:A},o)))):i.VirtualDOM.realize(i.h.div(i.h.div({className:`${a} ${s}`},i.h.span({className:l}),i.h.span({className:A},o))))};var h=n(196),d=n(197),f=n(156),p=n(625),I=n(198),C=n(626);n.d(e,"CellDragUtils",function(){return u}),n.d(e,"Collapser",function(){return h.a}),n.d(e,"InputCollapser",function(){return h.b}),n.d(e,"OutputCollapser",function(){return h.c}),n.d(e,"CellHeader",function(){return d.b}),n.d(e,"CellFooter",function(){return d.a}),n.d(e,"InputArea",function(){return f.a}),n.d(e,"InputPrompt",function(){return f.b}),n.d(e,"isCodeCellModel",function(){return p.f}),n.d(e,"isMarkdownCellModel",function(){return p.g}),n.d(e,"isRawCellModel",function(){return p.h}),n.d(e,"CellModel",function(){return p.b}),n.d(e,"AttachmentsCellModel",function(){return p.a}),n.d(e,"RawCellModel",function(){return p.e}),n.d(e,"MarkdownCellModel",function(){return p.d}),n.d(e,"CodeCellModel",function(){return p.c}),n.d(e,"Placeholder",function(){return I.c}),n.d(e,"InputPlaceholder",function(){return I.a}),n.d(e,"OutputPlaceholder",function(){return I.b}),n.d(e,"Cell",function(){return C.a}),n.d(e,"CodeCell",function(){return C.b}),n.d(e,"MarkdownCell",function(){return C.c}),n.d(e,"RawCell",function(){return C.d})},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2),i=r.__importDefault(n(4)),o=r.__importStar(n(0)),a=n(716),s=n(1140),A=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.__extends(e,t),e.prototype.render=function(){var t=this.props.icon;if(null==t||"boolean"==typeof t)return null;if("string"!=typeof t)return t;var n=this.props,a=n.className,A=n.color,l=n.htmlTitle,c=n.iconSize,u=void 0===c?e.SIZE_STANDARD:c,g=n.intent,h=n.title,d=void 0===h?t:h,f=n.tagName,p=void 0===f?"span":f,I=r.__rest(n,["className","color","htmlTitle","iconSize","intent","title","tagName"]),C=u>=e.SIZE_LARGE?e.SIZE_LARGE:e.SIZE_STANDARD,m=this.renderSvgPaths(C,t),y=i.default(s.Classes.ICON,s.Classes.iconClass(t),s.Classes.intentClass(g),a),E="0 0 "+C+" "+C;return o.createElement(p,r.__assign({},I,{className:y,title:l}),o.createElement("svg",{fill:A,"data-icon":t,width:u,height:u,viewBox:E},d&&o.createElement("desc",null,d),m))},e.prototype.renderSvgPaths=function(t,n){var r=(t===e.SIZE_STANDARD?a.IconSvgPaths16:a.IconSvgPaths20)[n];return null==r?null:r.map(function(t,e){return o.createElement("path",{key:e,d:t,fillRule:"evenodd"})})},e.displayName=s.DISPLAYNAME_PREFIX+".Icon",e.SIZE_STANDARD=16,e.SIZE_LARGE=20,e}(o.PureComponent);e.Icon=A},function(t,e,n){"use strict";e.byteLength=function(t){var e=l(t),n=e[0],r=e[1];return 3*(n+r)/4-r},e.toByteArray=function(t){for(var e,n=l(t),r=n[0],a=n[1],s=new o(function(t,e,n){return 3*(e+n)/4-n}(0,r,a)),A=0,c=a>0?r-4:r,u=0;u<c;u+=4)e=i[t.charCodeAt(u)]<<18|i[t.charCodeAt(u+1)]<<12|i[t.charCodeAt(u+2)]<<6|i[t.charCodeAt(u+3)],s[A++]=e>>16&255,s[A++]=e>>8&255,s[A++]=255&e;2===a&&(e=i[t.charCodeAt(u)]<<2|i[t.charCodeAt(u+1)]>>4,s[A++]=255&e);1===a&&(e=i[t.charCodeAt(u)]<<10|i[t.charCodeAt(u+1)]<<4|i[t.charCodeAt(u+2)]>>2,s[A++]=e>>8&255,s[A++]=255&e);return s},e.fromByteArray=function(t){for(var e,n=t.length,i=n%3,o=[],a=0,s=n-i;a<s;a+=16383)o.push(c(t,a,a+16383>s?s:a+16383));1===i?(e=t[n-1],o.push(r[e>>2]+r[e<<4&63]+"==")):2===i&&(e=(t[n-2]<<8)+t[n-1],o.push(r[e>>10]+r[e>>4&63]+r[e<<2&63]+"="));return o.join("")};for(var r=[],i=[],o="undefined"!=typeof Uint8Array?Uint8Array:Array,a="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",s=0,A=a.length;s<A;++s)r[s]=a[s],i[a.charCodeAt(s)]=s;function l(t){var e=t.length;if(e%4>0)throw new Error("Invalid string. Length must be a multiple of 4");var n=t.indexOf("=");return-1===n&&(n=e),[n,n===e?0:4-n%4]}function c(t,e,n){for(var i,o,a=[],s=e;s<n;s+=3)i=(t[s]<<16&16711680)+(t[s+1]<<8&65280)+(255&t[s+2]),a.push(r[(o=i)>>18&63]+r[o>>12&63]+r[o>>6&63]+r[63&o]);return a.join("")}i["-".charCodeAt(0)]=62,i["_".charCodeAt(0)]=63},function(t,e,n){"use strict";n.d(e,"a",function(){return o}),n.d(e,"b",function(){return a});const r="$",i=/(\$\$?|\\(?:begin|end)\{[a-z]*\*?\}|\\[{}$]|[{}]|(?:\n\s*)+|@@\d+@@|\\\\(?:\(|\)|\[|\]))/i;function o(t){let e,n=[],o=null,a=null,A=null,l=0;/`/.test(t)?(t=t.replace(/~/g,"~T").replace(/(^|[^\\])(`+)([^\n]*?[^`\n])\2(?!`)/gm,t=>t.replace(/\$/g,"~D")),e=t=>t.replace(/~([TD])/g,(t,e)=>"T"===e?"~":r)):e=t=>t;let c=t.replace(/\r\n?/g,"\n").split(i);for(let t=1,i=c.length;t<i;t+=2){let i=c[t];"@"===i.charAt(0)?(c[t]="@@"+n.length+"@@",n.push(i)):null!==o?i===a?l?A=t:(c=s(o,t,e,n,c),o=null,a=null,A=null):i.match(/\n.*\n/)?(null!==A&&(c=s(o,t=A,e,n,c)),o=null,a=null,A=null,l=0):"{"===i?l++:"}"===i&&l&&l--:i===r||"$$"===i?(o=t,a=i,l=0):"\\\\("===i||"\\\\["===i?(o=t,a="("===i.slice(-1)?"\\\\)":"\\\\]",l=0):"begin"===i.substr(1,5)&&(o=t,a="\\end"+i.substr(6),l=0)}return null!==o&&null!==A&&(c=s(o,A,e,n,c),o=null,a=null,A=null),{text:e(c.join("")),math:n}}function a(t,e){return t.replace(/@@(\d+)@@/g,(t,n)=>{let r=e[n];return"\\\\("===r.substr(0,3)&&"\\\\)"===r.substr(r.length-3)?r="\\("+r.substring(3,r.length-3)+"\\)":"\\\\["===r.substr(0,3)&&"\\\\]"===r.substr(r.length-3)&&(r="\\["+r.substring(3,r.length-3)+"\\]"),r})}function s(t,e,n,r,i){let o=i.slice(t,e+1).join("").replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;");for(navigator&&"Microsoft Internet Explorer"===navigator.appName&&(o=o.replace(/(%[^\n]*)\n/g,"$1<br/>\n"));e>t;)i[e]="",e--;return i[t]="@@"+r.length+"@@",n&&(o=n(o)),r.push(o),i}},function(t,e,n){"use strict";n.d(e,"a",function(){return r});class r{constructor(t={}){this.trusted=!!t.trusted,this._data=t.data||{},this._metadata=t.metadata||{},this._callback=t.callback||i.noOp}get data(){return this._data}get metadata(){return this._metadata}setData(t){this._data=t.data||this._data,this._metadata=t.metadata||this._metadata,this._callback(t)}}var i;!function(t){t.noOp=function(){}}(i||(i={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r,i=n(5),o=n(46),a=n(9);!function(t){t.hitTestNodes=function(t,e,n){return i.ArrayExt.findFirstIndex(t,t=>o.ElementExt.hitTest(t,e,n))},t.findElement=function(t,e){return t.querySelector(`.${e}`)},t.findElements=function(t,e){return t.getElementsByClassName(e)},t.createDomID=function(){return`id-${a.UUID.uuid4()}`}}(r||(r={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r,i=n(60);!function(t){function e(e){return!("object"!=typeof e||!e)&&t.symbol in e}t.symbol=Symbol("printable"),t.isPrintable=e,t.getPrintFunction=function(n){return e(n)?n[t.symbol]():null},t.printWidget=function(t){return r(t.node)};const n=i.ServerConnection.makeSettings();async function r(t){const e="string"==typeof t,n=function(){const t=window.document.createElement("iframe");t.setAttribute("sandbox","allow-modals allow-same-origin");return t.setAttribute("style","visibility:hidden;width:0;height:0;position:absolute;z-index:-9999;bottom:0;"),t.setAttribute("width","0"),t.setAttribute("height","0"),t}(),r=window.document.body;r.appendChild(n),e?(n.srcdoc=t,await function(t){return new Promise(e=>{t.onload=()=>e()})}(n)):(n.src="about:blank",function(t,e){t.contentDocument.body.appendChild(e.cloneNode(!0)),t.contentDocument.close()}(n,t));const i=new Promise(t=>{const e=()=>{document.removeEventListener("mousemove",e,!0),document.removeEventListener("mousedown",e,!0),document.removeEventListener("keydown",e,!0),t()};document.addEventListener("mousemove",e,!0),document.addEventListener("mousedown",e,!0),document.addEventListener("keydown",e,!0)});var o;(o=n.contentWindow).document.execCommand("print",!1,null)||o.print(),await i,r.removeChild(n)}t.printURL=async function(t){return r(await(await i.ServerConnection.makeRequest(t,{},n)).text())}}(r||(r={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return l}),n.d(e,"b",function(){return c}),n.d(e,"c",function(){return u});var r=n(76),i=n(0);const o="jp-Collapser",a="jp-Collapser-child",s="jp-InputCollapser",A="jp-OutputCollapser";class l extends r.a{constructor(){super(),this.addClass(o)}get collapsed(){return!1}render(){let t=a;return i.createElement("div",{className:t,onClick:t=>this.handleClick(t)})}}class c extends l{constructor(){super(),this.addClass(s)}get collapsed(){let t=this.parent.parent;return!!t&&t.inputHidden}handleClick(t){let e=this.parent.parent;e&&(e.inputHidden=!e.inputHidden),this.update()}}class u extends l{constructor(){super(),this.addClass(A)}get collapsed(){let t=this.parent.parent;return!!t&&t.outputHidden}handleClick(t){let e=this.parent.parent;e&&(e.outputHidden=!e.outputHidden),this.update()}}},function(t,e,n){"use strict";n.d(e,"b",function(){return a}),n.d(e,"a",function(){return s});var r=n(6);const i="jp-CellHeader",o="jp-CellFooter";class a extends r.Widget{constructor(){super(),this.addClass(i)}}class s extends r.Widget{constructor(){super(),this.addClass(o)}}},function(t,e,n){"use strict";n.d(e,"c",function(){return u}),n.d(e,"a",function(){return g}),n.d(e,"b",function(){return h});var r=n(0),i=n(76);const o="jp-Placeholder",a="jp-Placeholder-prompt jp-InputPrompt",s="jp-Placeholder-prompt jp-OutputPrompt",A="jp-Placeholder-content",l="jp-InputPlaceholder",c="jp-OutputPlaceholder";class u extends i.a{constructor(t){super(),this.addClass(o),this._callback=t}handleClick(t){(0,this._callback)(t)}}class g extends u{constructor(t){super(t),this.addClass(l)}render(){return[r.createElement("div",{className:a,key:"input"}),r.createElement("div",{className:A,onClick:t=>this.handleClick(t),key:"content"},r.createElement("div",{className:"jp-MoreHorizIcon"}))]}}class h extends u{constructor(t){super(t),this.addClass(c)}render(){return[r.createElement("div",{className:s,key:"output"}),r.createElement("div",{className:A,onClick:t=>this.handleClick(t),key:"content"},r.createElement("div",{className:"jp-MoreHorizIcon"}))]}}},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r=function(){function t(e,n){void 0===n&&(n=t.now),this.SchedulerAction=e,this.now=n}return t.prototype.schedule=function(t,e,n){return void 0===e&&(e=0),new this.SchedulerAction(this,t).schedule(n,e)},t.now=function(){return Date.now()},t}()},function(t,e,n){"use strict";var r=n(10),i=n(50),o=n(9),a=n(201),s=n(204);class A{constructor(t=""){this._text="",this._isDisposed=!1,this._changed=new r.Signal(this),this._text=t}get type(){return"String"}get changed(){return this._changed}set text(t){t.length===this._text.length&&t===this._text||(this._text=t,this._changed.emit({type:"set",start:0,end:t.length,value:t}))}get text(){return this._text}insert(t,e){this._text=this._text.slice(0,t)+e+this._text.slice(t),this._changed.emit({type:"insert",start:t,end:t+e.length,value:e})}remove(t,e){let n=this._text.slice(t,e);this._text=this._text.slice(0,t)+this._text.slice(e),this._changed.emit({type:"remove",start:t,end:e,value:n})}clear(){this.text=""}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this._isDisposed=!0,r.Signal.clearData(this),this.clear())}}var l,c=n(5),u=n(387);class g extends u.a{constructor(t){super(),this._inCompound=!1,this._isUndoable=!0,this._madeCompoundChange=!1,this._index=-1,this._stack=[],this._serializer=t,this.changed.connect(this._onListChanged,this)}get canRedo(){return this._index<this._stack.length-1}get canUndo(){return this._index>=0}beginCompoundOperation(t){this._inCompound=!0,this._isUndoable=!1!==t,this._madeCompoundChange=!1}endCompoundOperation(){this._inCompound=!1,this._isUndoable=!0,this._madeCompoundChange&&this._index++}undo(){if(!this.canUndo)return;let t=this._stack[this._index];this._isUndoable=!1;for(let e of t.reverse())this._undoChange(e);this._isUndoable=!0,this._index--}redo(){if(!this.canRedo)return;this._index++;let t=this._stack[this._index];this._isUndoable=!1;for(let e of t)this._redoChange(e);this._isUndoable=!0}clearUndo(){this._index=-1,this._stack=[]}_onListChanged(t,e){if(this.isDisposed||!this._isUndoable)return;this._inCompound&&this._madeCompoundChange||(this._stack=this._stack.slice(0,this._index+1));let n=this._copyChange(e);this._stack[this._index+1]?this._stack[this._index+1].push(n):this._stack.push([n]),this._inCompound?this._madeCompoundChange=!0:this._index++}_undoChange(t){let e=0,n=this._serializer;switch(t.type){case"add":Object(c.each)(t.newValues,()=>{this.remove(t.newIndex)});break;case"set":e=t.oldIndex,Object(c.each)(t.oldValues,t=>{this.set(e++,n.fromJSON(t))});break;case"remove":e=t.oldIndex,Object(c.each)(t.oldValues,t=>{this.insert(e++,n.fromJSON(t))});break;case"move":this.move(t.newIndex,t.oldIndex);break;default:return}}_redoChange(t){let e=0,n=this._serializer;switch(t.type){case"add":e=t.newIndex,Object(c.each)(t.newValues,t=>{this.insert(e++,n.fromJSON(t))});break;case"set":e=t.newIndex,Object(c.each)(t.newValues,e=>{this.set(t.newIndex++,n.fromJSON(e))});break;case"remove":Object(c.each)(t.oldValues,()=>{this.remove(t.oldIndex)});break;case"move":this.move(t.oldIndex,t.newIndex);break;default:return}}_copyChange(t){let e=[];Object(c.each)(t.oldValues,t=>{e.push(this._serializer.toJSON(t))});let n=[];return Object(c.each)(t.newValues,t=>{n.push(this._serializer.toJSON(t))}),{type:t.type,oldIndex:t.oldIndex,newIndex:t.newIndex,oldValues:e,newValues:n}}}(g||(g={})).IdentitySerializer=class{toJSON(t){return t}fromJSON(t){return t}};class h{constructor(t=null){this._value=null,this._changed=new r.Signal(this),this._isDisposed=!1,this._value=t}get type(){return"Value"}get isDisposed(){return this._isDisposed}get changed(){return this._changed}get(){return this._value}set(t){let e=this._value;o.JSONExt.deepEqual(e,t)||(this._value=t,this._changed.emit({oldValue:e,newValue:t}))}dispose(){this._isDisposed||(this._isDisposed=!0,r.Signal.clearData(this),this._value=null)}}(h||(h={})).IChangedArgs=class{};class d{constructor(t={}){this.isPrepopulated=!1,this.isCollaborative=!1,this.connected=Promise.resolve(void 0),this._toDispose=!1,this._isDisposed=!1,this._disposables=new i.DisposableSet,this._basePath=t.basePath||"",t.baseDB?this._db=t.baseDB:(this._db=new a.a,this._toDispose=!0)}get basePath(){return this._basePath}get isDisposed(){return this._isDisposed}get(t){return this._db.get(this._resolvePath(t))}has(t){return this._db.has(this._resolvePath(t))}createString(t){let e=new A;return this._disposables.add(e),this.set(t,e),e}createList(t){let e=new g(new g.IdentitySerializer);return this._disposables.add(e),this.set(t,e),e}createMap(t){let e=new s.a;return this._disposables.add(e),this.set(t,e),e}createValue(t){let e=new h;return this._disposables.add(e),this.set(t,e),e}getValue(t){let e=this.get(t);if(!e||"Value"!==e.type)throw Error("Can only call getValue for an ObservableValue");return e.get()}setValue(t,e){let n=this.get(t);if(!n||"Value"!==n.type)throw Error("Can only call setValue on an ObservableValue");n.set(e)}view(t){let e=new d({basePath:t,baseDB:this});return this._disposables.add(e),e}set(t,e){this._db.set(this._resolvePath(t),e)}dispose(){this.isDisposed||(this._isDisposed=!0,this._toDispose&&this._db.dispose(),this._disposables.dispose())}_resolvePath(t){return this._basePath&&(t=this._basePath+"."+t),t}}n.d(e,"a",function(){return l}),function(t){t.defaultSelectionStyle={className:"",displayName:"",color:"black"};t.Model=class{constructor(t){this._isDisposed=!1,this._mimeTypeChanged=new r.Signal(this),(t=t||{}).modelDB?this.modelDB=t.modelDB:this.modelDB=new d;let e=this.modelDB.createString("value");e.text=e.text||t.value||"";let n=this.modelDB.createValue("mimeType");n.set(t.mimeType||"text/plain"),n.changed.connect(this._onMimeTypeChanged,this),this.modelDB.createMap("selections")}get mimeTypeChanged(){return this._mimeTypeChanged}get value(){return this.modelDB.get("value")}get selections(){return this.modelDB.get("selections")}get mimeType(){return this.modelDB.getValue("mimeType")}set mimeType(t){this.mimeType!==t&&this.modelDB.setValue("mimeType",t)}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this._isDisposed=!0,this.value.text="",r.Signal.clearData(this))}_onMimeTypeChanged(t,e){this._mimeTypeChanged.emit({name:"mimeType",oldValue:e.oldValue,newValue:e.newValue})}},t.defaultConfig={fontFamily:null,fontSize:null,lineHeight:null,lineNumbers:!1,lineWrap:"on",wordWrapColumn:80,readOnly:!1,tabSize:4,insertSpaces:!0,matchBrackets:!0,autoClosingBrackets:!0,rulers:[],codeFolding:!1}}(l||(l={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r,i=n(10);class o{constructor(t={}){if(this._map=new Map,this._changed=new i.Signal(this),this._isDisposed=!1,this._itemCmp=t.itemCmp||r.itemCmp,t.values)for(let e in t.values)this._map.set(e,t.values[e])}get type(){return"Map"}get changed(){return this._changed}get isDisposed(){return this._isDisposed}get size(){return this._map.size}set(t,e){let n=this._map.get(t);if(void 0===e)throw Error("Cannot set an undefined value, use remove");let r=this._itemCmp;return void 0!==n&&r(n,e)?n:(this._map.set(t,e),this._changed.emit({type:n?"change":"add",key:t,oldValue:n,newValue:e}),n)}get(t){return this._map.get(t)}has(t){return this._map.has(t)}keys(){let t=[];return this._map.forEach((e,n)=>{t.push(n)}),t}values(){let t=[];return this._map.forEach((e,n)=>{t.push(e)}),t}delete(t){let e=this._map.get(t);return this._map.delete(t)&&this._changed.emit({type:"remove",key:t,oldValue:e,newValue:void 0}),e}clear(){let t=this.keys();for(let e=0;e<t.length;e++)this.delete(t[e])}dispose(){this.isDisposed||(this._isDisposed=!0,i.Signal.clearData(this),this._map.clear())}}!function(t){t.itemCmp=function(t,e){return t===e}}(r||(r={}))},function(t,e,n){try{var r=n(650)}catch(t){r=n(650)}var i=/\s+/,o=Object.prototype.toString;function a(t){if(!t||!t.nodeType)throw new Error("A DOM element reference is required");this.el=t,this.list=t.classList}t.exports=function(t){return new a(t)},a.prototype.add=function(t){if(this.list)return this.list.add(t),this;var e=this.array();return~r(e,t)||e.push(t),this.el.className=e.join(" "),this},a.prototype.remove=function(t){if("[object RegExp]"==o.call(t))return this.removeMatching(t);if(this.list)return this.list.remove(t),this;var e=this.array(),n=r(e,t);return~n&&e.splice(n,1),this.el.className=e.join(" "),this},a.prototype.removeMatching=function(t){for(var e=this.array(),n=0;n<e.length;n++)t.test(e[n])&&this.remove(e[n]);return this},a.prototype.toggle=function(t,e){return this.list?(void 0!==e?e!==this.list.toggle(t,e)&&this.list.toggle(t):this.list.toggle(t),this):(void 0!==e?e?this.add(t):this.remove(t):this.has(t)?this.remove(t):this.add(t),this)},a.prototype.array=function(){var t=(this.el.getAttribute("class")||"").replace(/^\s+|\s+$/g,"").split(i);return""===t[0]&&t.shift(),t},a.prototype.has=a.prototype.contains=function(t){return this.list?this.list.contains(t):!!~r(this.array(),t)}},function(t,e,n){"use strict";n.r(e);var r,i=n(20),o=(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)}),a=function(){return(a=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},s=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return o(e,t),e.prototype.defaults=function(){return a({},t.prototype.defaults.call(this),{_model_name:"OutputModel",_view_name:"OutputView",_model_module:"@jupyter-widgets/output",_view_module:"@jupyter-widgets/output",_model_module_version:"1.0.0",_view_module_version:"1.0.0"})},e}(i.DOMWidgetModel),A=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return o(e,t),e}(i.DOMWidgetView);n.d(e,"OUTPUT_WIDGET_VERSION",function(){return"1.0.0"}),n.d(e,"OutputModel",function(){return s}),n.d(e,"OutputView",function(){return A})},function(t,e,n){"use strict";n.d(e,"a",function(){return a});var r=n(9),i=n(25),o=n(201);class a extends o.a{constructor(t={}){super({itemCmp:r.JSONExt.deepEqual,values:t.values})}toJSON(){const t=Object.create(null),e=this.keys();for(let n of e){const e=this.get(n);void 0!==e&&(t[n]=r.JSONExt.deepCopy(e))}return t}}!function(t){t.ChangeMessage=class extends i.Message{constructor(t,e){super(t),this.args=e}}}(a||(a={}))},function(t,e,n){"use strict";var r=n(217),i=n(1102),o=n(1103),a=n(1104),s=n(1105),A=n(1106),l=n(1107),c=n(1108),u=n(1111),g=n(1131);function h(t,e){t&&Object.keys(t).forEach(function(n){e(t[n],n)})}function d(t,e){return{}.hasOwnProperty.call(t,e)}function f(t,e){var n=[];return h(t,function(t){e(t)&&n.push(t)}),n}t.exports=I;var p=/^[^\0\t\n\f\r \/<=>]+$/;function I(t,e,n){var m="";function y(t,e){var n=this;this.tag=t,this.attribs=e||{},this.tagPosition=m.length,this.text="",this.updateParentNodeText=function(){_.length&&(_[_.length-1].text+=n.text)}}e?(e=i(I.defaults,e)).parser?e.parser=i(C,e.parser):e.parser=C:(e=I.defaults).parser=C;var E,v,B=e.nonTextTags||["script","style","textarea"];e.allowedAttributes&&(E={},v={},h(e.allowedAttributes,function(t,e){E[e]=[];var n=[];t.forEach(function(t){A(t)&&t.indexOf("*")>=0?n.push(o(t).replace(/\\\*/g,".*")):E[e].push(t)}),v[e]=new RegExp("^("+n.join("|")+")$")}));var b={};h(e.allowedClasses,function(t,e){E&&(d(E,e)||(E[e]=[]),E[e].push("class")),b[e]=t});var Q,w={};h(e.transformTags,function(t,e){var n;"function"==typeof t?n=t:"string"==typeof t&&(n=I.simpleTransform(t)),"*"===e?Q=n:w[e]=n});var x=0,_=[],D={},M={},k=!1,S=0,L=new r.Parser({onopentag:function(t,n){if(k)S++;else{var r=new y(t,n);_.push(r);var i,o=!1,A=!!r.text;d(w,t)&&(i=w[t](t,n),r.attribs=n=i.attribs,void 0!==i.text&&(r.innerText=i.text),t!==i.tagName&&(r.name=t=i.tagName,M[x]=i.tagName)),Q&&(i=Q(t,n),r.attribs=n=i.attribs,t!==i.tagName&&(r.name=t=i.tagName,M[x]=i.tagName)),e.allowedTags&&-1===e.allowedTags.indexOf(t)&&(o=!0,-1!==B.indexOf(t)&&(k=!0,S=1),D[x]=!0),x++,o||(m+="<"+t,(!E||d(E,t)||E["*"])&&h(n,function(n,i){if(p.test(i)){var o,A=!1;if(!E||d(E,t)&&-1!==E[t].indexOf(i)||E["*"]&&-1!==E["*"].indexOf(i)||d(v,t)&&v[t].test(i)||v["*"]&&v["*"].test(i))A=!0;else if(E&&E[t]){var I=!0,C=!1,y=void 0;try{for(var B,Q=E[t][Symbol.iterator]();!(I=(B=Q.next()).done);I=!0){var w=B.value;if(l(w)&&w.name&&w.name===i){A=!0;var x="";if(!0===w.multiple){var _=n.split(" "),D=!0,M=!1,k=void 0;try{for(var S,L=_[Symbol.iterator]();!(D=(S=L.next()).done);D=!0){var F=S.value;-1!==w.values.indexOf(F)&&(""===x?x=F:x+=" "+F)}}catch(t){M=!0,k=t}finally{try{!D&&L.return&&L.return()}finally{if(M)throw k}}}else w.values.indexOf(n)>=0&&(x=n);n=x}}}catch(t){C=!0,y=t}finally{try{!I&&Q.return&&Q.return()}finally{if(C)throw y}}}if(A){if(-1!==e.allowedSchemesAppliedToAttributes.indexOf(i)&&N(t,n))return void delete r.attribs[i];if("iframe"===t&&"src"===i){var T=!0;try{(o=g.parse(n,!1,!0))&&null===o.host&&null===o.protocol?T=d(e,"allowIframeRelativeUrls")?e.allowIframeRelativeUrls:!e.allowedIframeHostnames:e.allowedIframeHostnames&&(T=e.allowedIframeHostnames.find(function(t){return t===o.hostname}))}catch(t){T=!1}if(!T)return void delete r.attribs[i]}if("srcset"===i)try{if(h(o=c.parse(n),function(t){N("srcset",t.url)&&(t.evil=!0)}),!(o=f(o,function(t){return!t.evil})).length)return void delete r.attribs[i];n=c.stringify(f(o,function(t){return!t.evil})),r.attribs[i]=n}catch(t){return void delete r.attribs[i]}if("class"===i&&!(n=function(t,e){if(!e)return t;return(t=t.split(/\s+/)).filter(function(t){return-1!==e.indexOf(t)}).join(" ")}(n,b[t])).length)return void delete r.attribs[i];if("style"===i)try{if(0===(n=function(t){return t.nodes[0].nodes.reduce(function(t,e){return t.push(e.prop+":"+e.value),t},[]).join(";")}(function(t,e){if(!e)return t;var n,r=a(t),i=t.nodes[0];n=e[i.selector]&&e["*"]?s(a(e[i.selector]),e["*"],function(t,e){if(Array.isArray(t))return t.concat(e)}):e[i.selector]||e["*"];n&&(r.nodes[0].nodes=i.nodes.reduce(function(t){return function(e,n){if(t.hasOwnProperty(n.prop)){var r=t[n.prop].some(function(t){return t.test(n.value)});r&&e.push(n)}return e}}(n),[]));return r}(u.parse(t+" {"+n+"}"),e.allowedStyles))).length)return void delete r.attribs[i]}catch(t){return void delete r.attribs[i]}m+=" "+i,n.length&&(m+='="'+R(n,!0)+'"')}else delete r.attribs[i]}else delete r.attribs[i]}),-1!==e.selfClosing.indexOf(t)?m+=" />":(m+=">",!r.innerText||A||e.textFilter||(m+=r.innerText)))}},ontext:function(t){if(!k){var n,r=_[_.length-1];if(r&&(n=r.tag,t=void 0!==r.innerText?r.innerText:t),"script"===n||"style"===n)m+=t;else{var i=R(t,!1);e.textFilter?m+=e.textFilter(i):m+=i}if(_.length)_[_.length-1].text+=t}},onclosetag:function(t){if(k){if(--S)return;k=!1}var n=_.pop();if(n){if(k=!1,D[--x])return delete D[x],void n.updateParentNodeText();M[x]&&(t=M[x],delete M[x]),e.exclusiveFilter&&e.exclusiveFilter(n)?m=m.substr(0,n.tagPosition):(n.updateParentNodeText(),-1===e.selfClosing.indexOf(t)&&(m+="</"+t+">"))}}},e.parser);return L.write(t),L.end(),m;function R(t,n){return"string"!=typeof t&&(t+=""),e.parser.decodeEntities&&(t=t.replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/\>/g,"&gt;"),n&&(t=t.replace(/\"/g,"&quot;"))),t=t.replace(/&(?![a-zA-Z0-9#]{1,20};)/g,"&amp;").replace(/</g,"&lt;").replace(/\>/g,"&gt;"),n&&(t=t.replace(/\"/g,"&quot;")),t}function N(t,n){var r=(n=(n=n.replace(/[\x00-\x20]+/g,"")).replace(/<\!\-\-.*?\-\-\>/g,"")).match(/^([a-zA-Z]+)\:/);if(!r)return!!n.match(/^[\/\\]{2}/)&&!e.allowProtocolRelative;var i=r[1].toLowerCase();return d(e.allowedSchemesByTag,t)?-1===e.allowedSchemesByTag[t].indexOf(i):!e.allowedSchemes||-1===e.allowedSchemes.indexOf(i)}}var C={decodeEntities:!0};I.defaults={allowedTags:["h3","h4","h5","h6","blockquote","p","a","ul","ol","nl","li","b","i","strong","em","strike","code","hr","br","div","table","thead","caption","tbody","tr","th","td","pre","iframe"],allowedAttributes:{a:["href","name","target"],img:["src"]},selfClosing:["img","br","hr","area","base","basefont","input","link","meta"],allowedSchemes:["http","https","ftp","mailto"],allowedSchemesByTag:{},allowedSchemesAppliedToAttributes:["href","src","cite"],allowProtocolRelative:!0},I.simpleTransform=function(t,e,n){return n=void 0===n||n,e=e||{},function(r,i){var o;if(n)for(o in e)i[o]=e[o];else i=e;return{tagName:t,attribs:i}}}},function(t,e,n){"use strict";n.d(e,"a",function(){return C}),n.d(e,"c",function(){return m}),n.d(e,"b",function(){return y}),n.d(e,"d",function(){return E});var r=n(9),i=n(43),o=n(10),a=n(6),s=n(60);const A="jp-OutputArea",l="jp-OutputArea-child",c="jp-OutputArea-output",u="jp-OutputArea-prompt",g="jp-OutputPrompt",h="jp-OutputArea-executeResult",d="jp-OutputArea-stdin-item",f="jp-Stdin",p="jp-Stdin-prompt",I="jp-Stdin-input";class C extends a.Widget{constructor(t){super(),this.outputLengthChanged=new o.Signal(this),this._onIOPub=t=>{let e,n,r=this.model,i=t.header.msg_type,o=(t.content.transient||{}).display_id;switch(i){case"execute_result":case"display_data":case"stream":case"error":(e=t.content).output_type=i,r.add(e);break;case"clear_output":let a=t.content.wait;r.clear(a);break;case"update_display_data":if((e=t.content).output_type="display_data",n=this._displayIdMap.get(o))for(let t of n)r.set(t,e)}o&&"display_data"===i&&((n=this._displayIdMap.get(o)||[]).push(r.length-1),this._displayIdMap.set(o,n))},this._onExecuteReply=t=>{let e=this.model,n=t.content;if("ok"!==n.status)return;let r=n&&n.payload;if(!r||!r.length)return;let i=r.filter(t=>"page"===t.source);if(!i.length)return;let o={output_type:"display_data",data:JSON.parse(JSON.stringify(i[0])).data,metadata:{}};e.add(o)},this._minHeightTimeout=null,this._future=null,this._displayIdMap=new Map;let e=this.model=t.model;this.addClass(A),this.rendermime=t.rendermime,this.contentFactory=t.contentFactory||C.defaultContentFactory,this.layout=new a.PanelLayout;for(let t=0;t<e.length;t++){let n=e.get(t);this._insertOutput(t,n)}e.changed.connect(this.onModelChanged,this),e.stateChanged.connect(this.onStateChanged,this)}get widgets(){return this.layout.widgets}get future(){return this._future}set future(t){if(this.model.isDisposed)throw Error("Model is disposed");this._future!==t&&(this._future&&this._future.dispose(),this._future=t,this.model.clear(),this.widgets.length&&(this._clear(),this.outputLengthChanged.emit(this.model.length)),t.onIOPub=this._onIOPub,t.onReply=this._onExecuteReply,t.onStdin=e=>{s.KernelMessage.isInputRequestMsg(e)&&this.onInputRequest(e,t)})}dispose(){this._future&&this._future.dispose(),this._future=null,this._displayIdMap.clear(),super.dispose()}onModelChanged(t,e){switch(e.type){case"add":this._insertOutput(e.newIndex,e.newValues[0]),this.outputLengthChanged.emit(this.model.length);break;case"remove":this.widgets.length&&(this._clear(),this.outputLengthChanged.emit(this.model.length));break;case"set":this._setOutput(e.newIndex,e.newValues[0]),this.outputLengthChanged.emit(this.model.length)}}onStateChanged(t){for(let t=0;t<this.model.length;t++)this._setOutput(t,this.model.get(t));this.outputLengthChanged.emit(this.model.length)}_clear(){if(!this.widgets.length)return;let t=this.widgets.length;for(let e=0;e<t;e++){let t=this.widgets[0];t.parent=null,t.dispose()}this._displayIdMap.clear();let e=this.node.getBoundingClientRect();this.node.style.minHeight=`${e.height}px`,this._minHeightTimeout&&clearTimeout(this._minHeightTimeout),this._minHeightTimeout=window.setTimeout(()=>{this.isDisposed||(this.node.style.minHeight="")},50)}onInputRequest(t,e){let n=this.contentFactory,r=t.content.prompt,i=t.content.password,o=new a.Panel;o.addClass(l),o.addClass(d);let s=n.createOutputPrompt();s.addClass(u),o.addWidget(s);let A=n.createStdin({prompt:r,password:i,future:e});A.addClass(c),o.addWidget(A),this.layout.addWidget(o),A.value.then(t=>{this.model.add({output_type:"stream",name:"stdin",text:t+"\n"}),o.dispose()})}_setOutput(t,e){let n=this.layout,r=n.widgets[t],i=r.widgets?r.widgets[1]:r,o=this.rendermime.preferredMimeType(e.data,e.trusted?"any":"ensure");i.renderModel&&v.currentPreferredMimetype.get(i)===o&&C.isIsolated(o,e.metadata)===i instanceof v.IsolatedRenderer?i.renderModel(e):(n.widgets[t].dispose(),this._insertOutput(t,e))}_insertOutput(t,e){let n=this.createOutputItem(e);n?n.toggleClass(h,null!==e.executionCount):n=new a.Widget,this.layout.insertWidget(t,n)}createOutputItem(t){let e=this.createRenderedMimetype(t);if(!e)return null;let n=new a.Panel;n.addClass(l);let r=this.contentFactory.createOutputPrompt();return r.executionCount=t.executionCount,r.addClass(u),n.addWidget(r),e.addClass(c),n.addWidget(e),n}createRenderedMimetype(t){let e=this.rendermime.preferredMimeType(t.data,t.trusted?"any":"ensure");if(!e)return null;let n=this.rendermime.createRenderer(e);return!0===C.isIsolated(e,t.metadata)&&(n=new v.IsolatedRenderer(n)),v.currentPreferredMimetype.set(n,e),n.renderModel(t).catch(t=>{const e=document.createElement("pre");e.textContent=`Javascript Error: ${t.message}`,n.node.appendChild(e),n.node.className="p-Widget jp-RenderedText",n.node.setAttribute("data-mime-type","application/vnd.jupyter.stderr")}),n}}class m extends C{onInputRequest(t,e){}createOutputItem(t){let e=this.createRenderedMimetype(t);return e&&e.addClass(c),e}}!function(t){t.execute=async function(t,e,n,r){let i=!0;r&&Array.isArray(r.tags)&&-1!==r.tags.indexOf("raises-exception")&&(i=!1);let o={code:t,stop_on_error:i};if(!n.kernel)throw new Error("Session has no kernel.");let a=n.kernel.requestExecute(o,!1,r);return e.future=a,a.done},t.isIsolated=function(t,e){let n=e[t];return n&&void 0!==n.isolated?!!n.isolated:!!e.isolated};class e{createOutputPrompt(){return new y}createStdin(t){return new E(t)}}t.ContentFactory=e,t.defaultContentFactory=new e}(C||(C={}));class y extends a.Widget{constructor(){super(),this._executionCount=null,this.addClass(g)}get executionCount(){return this._executionCount}set executionCount(t){this._executionCount=t,this.node.textContent=null===t?"":`[${t}]:`}}class E extends a.Widget{constructor(t){super({node:v.createInputWidgetNode(t.prompt,t.password)}),this._future=null,this._input=null,this._promise=new r.PromiseDelegate,this.addClass(f),this._input=this.node.getElementsByTagName("input")[0],this._input.focus(),this._future=t.future,this._value=t.prompt+" "}get value(){return this._promise.promise.then(()=>this._value)}handleEvent(t){let e=this._input;"keydown"===t.type&&13===t.keyCode&&(this._future.sendInputReply({status:"ok",value:e.value}),"password"===e.type?this._value+=Array(e.value.length+1).join("·"):this._value+=e.value,this._promise.resolve(void 0))}onAfterAttach(t){this._input.addEventListener("keydown",this),this.update()}onUpdateRequest(t){this._input.focus()}onBeforeDetach(t){this._input.removeEventListener("keydown",this)}}var v;!function(t){t.createInputWidgetNode=function(t,e){let n=document.createElement("div"),r=document.createElement("pre");r.className=p,r.textContent=t;let i=document.createElement("input");return i.className=I,e&&(i.type="password"),n.appendChild(r),r.appendChild(i),n};t.IsolatedRenderer=class extends a.Widget{constructor(t){super({node:document.createElement("iframe")}),this.addClass("jp-mod-isolated"),this._wrapped=t;let e=this.node;e.frameBorder="0",e.scrolling="auto",e.addEventListener("load",()=>{e.contentDocument.open(),e.contentDocument.write(this._wrapped.node.innerHTML),e.contentDocument.close();let t=e.contentDocument.body;e.style.height=t.scrollHeight+"px"})}renderModel(t){return this._wrapped.renderModel(t).then(()=>{let t=this.node.contentWindow;t&&t.location.reload()})}},t.currentPreferredMimetype=new i.AttachedProperty({name:"preferredMimetype",create:t=>""})}(v||(v={}))},,function(t,e,n){"use strict";var r,i="object"==typeof Reflect?Reflect:null,o=i&&"function"==typeof i.apply?i.apply:function(t,e,n){return Function.prototype.apply.call(t,e,n)};r=i&&"function"==typeof i.ownKeys?i.ownKeys:Object.getOwnPropertySymbols?function(t){return Object.getOwnPropertyNames(t).concat(Object.getOwnPropertySymbols(t))}:function(t){return Object.getOwnPropertyNames(t)};var a=Number.isNaN||function(t){return t!=t};function s(){s.init.call(this)}t.exports=s,s.EventEmitter=s,s.prototype._events=void 0,s.prototype._eventsCount=0,s.prototype._maxListeners=void 0;var A=10;function l(t){return void 0===t._maxListeners?s.defaultMaxListeners:t._maxListeners}function c(t,e,n,r){var i,o,a,s;if("function"!=typeof n)throw new TypeError('The "listener" argument must be of type Function. Received type '+typeof n);if(void 0===(o=t._events)?(o=t._events=Object.create(null),t._eventsCount=0):(void 0!==o.newListener&&(t.emit("newListener",e,n.listener?n.listener:n),o=t._events),a=o[e]),void 0===a)a=o[e]=n,++t._eventsCount;else if("function"==typeof a?a=o[e]=r?[n,a]:[a,n]:r?a.unshift(n):a.push(n),(i=l(t))>0&&a.length>i&&!a.warned){a.warned=!0;var A=new Error("Possible EventEmitter memory leak detected. "+a.length+" "+String(e)+" listeners added. Use emitter.setMaxListeners() to increase limit");A.name="MaxListenersExceededWarning",A.emitter=t,A.type=e,A.count=a.length,s=A,console&&console.warn&&console.warn(s)}return t}function u(t,e,n){var r={fired:!1,wrapFn:void 0,target:t,type:e,listener:n},i=function(){for(var t=[],e=0;e<arguments.length;e++)t.push(arguments[e]);this.fired||(this.target.removeListener(this.type,this.wrapFn),this.fired=!0,o(this.listener,this.target,t))}.bind(r);return i.listener=n,r.wrapFn=i,i}function g(t,e,n){var r=t._events;if(void 0===r)return[];var i=r[e];return void 0===i?[]:"function"==typeof i?n?[i.listener||i]:[i]:n?function(t){for(var e=new Array(t.length),n=0;n<e.length;++n)e[n]=t[n].listener||t[n];return e}(i):d(i,i.length)}function h(t){var e=this._events;if(void 0!==e){var n=e[t];if("function"==typeof n)return 1;if(void 0!==n)return n.length}return 0}function d(t,e){for(var n=new Array(e),r=0;r<e;++r)n[r]=t[r];return n}Object.defineProperty(s,"defaultMaxListeners",{enumerable:!0,get:function(){return A},set:function(t){if("number"!=typeof t||t<0||a(t))throw new RangeError('The value of "defaultMaxListeners" is out of range. It must be a non-negative number. Received '+t+".");A=t}}),s.init=function(){void 0!==this._events&&this._events!==Object.getPrototypeOf(this)._events||(this._events=Object.create(null),this._eventsCount=0),this._maxListeners=this._maxListeners||void 0},s.prototype.setMaxListeners=function(t){if("number"!=typeof t||t<0||a(t))throw new RangeError('The value of "n" is out of range. It must be a non-negative number. Received '+t+".");return this._maxListeners=t,this},s.prototype.getMaxListeners=function(){return l(this)},s.prototype.emit=function(t){for(var e=[],n=1;n<arguments.length;n++)e.push(arguments[n]);var r="error"===t,i=this._events;if(void 0!==i)r=r&&void 0===i.error;else if(!r)return!1;if(r){var a;if(e.length>0&&(a=e[0]),a instanceof Error)throw a;var s=new Error("Unhandled error."+(a?" ("+a.message+")":""));throw s.context=a,s}var A=i[t];if(void 0===A)return!1;if("function"==typeof A)o(A,this,e);else{var l=A.length,c=d(A,l);for(n=0;n<l;++n)o(c[n],this,e)}return!0},s.prototype.addListener=function(t,e){return c(this,t,e,!1)},s.prototype.on=s.prototype.addListener,s.prototype.prependListener=function(t,e){return c(this,t,e,!0)},s.prototype.once=function(t,e){if("function"!=typeof e)throw new TypeError('The "listener" argument must be of type Function. Received type '+typeof e);return this.on(t,u(this,t,e)),this},s.prototype.prependOnceListener=function(t,e){if("function"!=typeof e)throw new TypeError('The "listener" argument must be of type Function. Received type '+typeof e);return this.prependListener(t,u(this,t,e)),this},s.prototype.removeListener=function(t,e){var n,r,i,o,a;if("function"!=typeof e)throw new TypeError('The "listener" argument must be of type Function. Received type '+typeof e);if(void 0===(r=this._events))return this;if(void 0===(n=r[t]))return this;if(n===e||n.listener===e)0==--this._eventsCount?this._events=Object.create(null):(delete r[t],r.removeListener&&this.emit("removeListener",t,n.listener||e));else if("function"!=typeof n){for(i=-1,o=n.length-1;o>=0;o--)if(n[o]===e||n[o].listener===e){a=n[o].listener,i=o;break}if(i<0)return this;0===i?n.shift():function(t,e){for(;e+1<t.length;e++)t[e]=t[e+1];t.pop()}(n,i),1===n.length&&(r[t]=n[0]),void 0!==r.removeListener&&this.emit("removeListener",t,a||e)}return this},s.prototype.off=s.prototype.removeListener,s.prototype.removeAllListeners=function(t){var e,n,r;if(void 0===(n=this._events))return this;if(void 0===n.removeListener)return 0===arguments.length?(this._events=Object.create(null),this._eventsCount=0):void 0!==n[t]&&(0==--this._eventsCount?this._events=Object.create(null):delete n[t]),this;if(0===arguments.length){var i,o=Object.keys(n);for(r=0;r<o.length;++r)"removeListener"!==(i=o[r])&&this.removeAllListeners(i);return this.removeAllListeners("removeListener"),this._events=Object.create(null),this._eventsCount=0,this}if("function"==typeof(e=n[t]))this.removeListener(t,e);else if(void 0!==e)for(r=e.length-1;r>=0;r--)this.removeListener(t,e[r]);return this},s.prototype.listeners=function(t){return g(this,t,!0)},s.prototype.rawListeners=function(t){return g(this,t,!1)},s.listenerCount=function(t,e){return"function"==typeof t.listenerCount?t.listenerCount(e):h.call(t,e)},s.prototype.listenerCount=h,s.prototype.eventNames=function(){return this._eventsCount>0?r(this._events):[]}},function(t,e,n){"use strict";(function(e){void 0===e||!e.version||0===e.version.indexOf("v0.")||0===e.version.indexOf("v1.")&&0!==e.version.indexOf("v1.8.")?t.exports={nextTick:function(t,n,r,i){if("function"!=typeof t)throw new TypeError('"callback" argument must be a function');var o,a,s=arguments.length;switch(s){case 0:case 1:return e.nextTick(t);case 2:return e.nextTick(function(){t.call(null,n)});case 3:return e.nextTick(function(){t.call(null,n,r)});case 4:return e.nextTick(function(){t.call(null,n,r,i)});default:for(o=new Array(s-1),a=0;a<o.length;)o[a++]=arguments[a];return e.nextTick(function(){t.apply(null,o)})}}}:t.exports=e}).call(this,n(58))},function(t,e,n){"use strict";var r=n(776).Buffer,i=r.isEncoding||function(t){switch((t=""+t)&&t.toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":case"raw":return!0;default:return!1}};function o(t){var e;switch(this.encoding=function(t){var e=function(t){if(!t)return"utf8";for(var e;;)switch(t){case"utf8":case"utf-8":return"utf8";case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return"utf16le";case"latin1":case"binary":return"latin1";case"base64":case"ascii":case"hex":return t;default:if(e)return;t=(""+t).toLowerCase(),e=!0}}(t);if("string"!=typeof e&&(r.isEncoding===i||!i(t)))throw new Error("Unknown encoding: "+t);return e||t}(t),this.encoding){case"utf16le":this.text=A,this.end=l,e=4;break;case"utf8":this.fillLast=s,e=4;break;case"base64":this.text=c,this.end=u,e=3;break;default:return this.write=g,void(this.end=h)}this.lastNeed=0,this.lastTotal=0,this.lastChar=r.allocUnsafe(e)}function a(t){return t<=127?0:t>>5==6?2:t>>4==14?3:t>>3==30?4:t>>6==2?-1:-2}function s(t){var e=this.lastTotal-this.lastNeed,n=function(t,e,n){if(128!=(192&e[0]))return t.lastNeed=0,"�";if(t.lastNeed>1&&e.length>1){if(128!=(192&e[1]))return t.lastNeed=1,"�";if(t.lastNeed>2&&e.length>2&&128!=(192&e[2]))return t.lastNeed=2,"�"}}(this,t);return void 0!==n?n:this.lastNeed<=t.length?(t.copy(this.lastChar,e,0,this.lastNeed),this.lastChar.toString(this.encoding,0,this.lastTotal)):(t.copy(this.lastChar,e,0,t.length),void(this.lastNeed-=t.length))}function A(t,e){if((t.length-e)%2==0){var n=t.toString("utf16le",e);if(n){var r=n.charCodeAt(n.length-1);if(r>=55296&&r<=56319)return this.lastNeed=2,this.lastTotal=4,this.lastChar[0]=t[t.length-2],this.lastChar[1]=t[t.length-1],n.slice(0,-1)}return n}return this.lastNeed=1,this.lastTotal=2,this.lastChar[0]=t[t.length-1],t.toString("utf16le",e,t.length-1)}function l(t){var e=t&&t.length?this.write(t):"";if(this.lastNeed){var n=this.lastTotal-this.lastNeed;return e+this.lastChar.toString("utf16le",0,n)}return e}function c(t,e){var n=(t.length-e)%3;return 0===n?t.toString("base64",e):(this.lastNeed=3-n,this.lastTotal=3,1===n?this.lastChar[0]=t[t.length-1]:(this.lastChar[0]=t[t.length-2],this.lastChar[1]=t[t.length-1]),t.toString("base64",e,t.length-n))}function u(t){var e=t&&t.length?this.write(t):"";return this.lastNeed?e+this.lastChar.toString("base64",0,3-this.lastNeed):e}function g(t){return t.toString(this.encoding)}function h(t){return t&&t.length?this.write(t):""}e.StringDecoder=o,o.prototype.write=function(t){if(0===t.length)return"";var e,n;if(this.lastNeed){if(void 0===(e=this.fillLast(t)))return"";n=this.lastNeed,this.lastNeed=0}else n=0;return n<t.length?e?e+this.text(t,n):this.text(t,n):e||""},o.prototype.end=function(t){var e=t&&t.length?this.write(t):"";return this.lastNeed?e+"�":e},o.prototype.text=function(t,e){var n=function(t,e,n){var r=e.length-1;if(r<n)return 0;var i=a(e[r]);if(i>=0)return i>0&&(t.lastNeed=i-1),i;if(--r<n||-2===i)return 0;if((i=a(e[r]))>=0)return i>0&&(t.lastNeed=i-2),i;if(--r<n||-2===i)return 0;if((i=a(e[r]))>=0)return i>0&&(2===i?i=0:t.lastNeed=i-3),i;return 0}(this,t,e);if(!this.lastNeed)return t.toString("utf8",e);this.lastTotal=n;var r=t.length-(n-this.lastNeed);return t.copy(this.lastChar,0,r),t.toString("utf8",e,r)},o.prototype.fillLast=function(t){if(this.lastNeed<=t.length)return t.copy(this.lastChar,this.lastTotal-this.lastNeed,0,this.lastNeed),this.lastChar.toString(this.encoding,0,this.lastTotal);t.copy(this.lastChar,this.lastTotal-this.lastNeed,0,t.length),this.lastNeed-=t.length}},function(t,e,n){var r=n(35).Buffer;function i(t){r.isBuffer(t)||(t=r.from(t));for(var e=t.length/4|0,n=new Array(e),i=0;i<e;i++)n[i]=t.readUInt32BE(4*i);return n}function o(t){for(;0<t.length;t++)t[0]=0}function a(t,e,n,r,i){for(var o,a,s,A,l=n[0],c=n[1],u=n[2],g=n[3],h=t[0]^e[0],d=t[1]^e[1],f=t[2]^e[2],p=t[3]^e[3],I=4,C=1;C<i;C++)o=l[h>>>24]^c[d>>>16&255]^u[f>>>8&255]^g[255&p]^e[I++],a=l[d>>>24]^c[f>>>16&255]^u[p>>>8&255]^g[255&h]^e[I++],s=l[f>>>24]^c[p>>>16&255]^u[h>>>8&255]^g[255&d]^e[I++],A=l[p>>>24]^c[h>>>16&255]^u[d>>>8&255]^g[255&f]^e[I++],h=o,d=a,f=s,p=A;return o=(r[h>>>24]<<24|r[d>>>16&255]<<16|r[f>>>8&255]<<8|r[255&p])^e[I++],a=(r[d>>>24]<<24|r[f>>>16&255]<<16|r[p>>>8&255]<<8|r[255&h])^e[I++],s=(r[f>>>24]<<24|r[p>>>16&255]<<16|r[h>>>8&255]<<8|r[255&d])^e[I++],A=(r[p>>>24]<<24|r[h>>>16&255]<<16|r[d>>>8&255]<<8|r[255&f])^e[I++],[o>>>=0,a>>>=0,s>>>=0,A>>>=0]}var s=[0,1,2,4,8,16,32,64,128,27,54],A=function(){for(var t=new Array(256),e=0;e<256;e++)t[e]=e<128?e<<1:e<<1^283;for(var n=[],r=[],i=[[],[],[],[]],o=[[],[],[],[]],a=0,s=0,A=0;A<256;++A){var l=s^s<<1^s<<2^s<<3^s<<4;l=l>>>8^255&l^99,n[a]=l,r[l]=a;var c=t[a],u=t[c],g=t[u],h=257*t[l]^16843008*l;i[0][a]=h<<24|h>>>8,i[1][a]=h<<16|h>>>16,i[2][a]=h<<8|h>>>24,i[3][a]=h,h=16843009*g^65537*u^257*c^16843008*a,o[0][l]=h<<24|h>>>8,o[1][l]=h<<16|h>>>16,o[2][l]=h<<8|h>>>24,o[3][l]=h,0===a?a=s=1:(a=c^t[t[t[g^c]]],s^=t[t[s]])}return{SBOX:n,INV_SBOX:r,SUB_MIX:i,INV_SUB_MIX:o}}();function l(t){this._key=i(t),this._reset()}l.blockSize=16,l.keySize=32,l.prototype.blockSize=l.blockSize,l.prototype.keySize=l.keySize,l.prototype._reset=function(){for(var t=this._key,e=t.length,n=e+6,r=4*(n+1),i=[],o=0;o<e;o++)i[o]=t[o];for(o=e;o<r;o++){var a=i[o-1];o%e==0?(a=a<<8|a>>>24,a=A.SBOX[a>>>24]<<24|A.SBOX[a>>>16&255]<<16|A.SBOX[a>>>8&255]<<8|A.SBOX[255&a],a^=s[o/e|0]<<24):e>6&&o%e==4&&(a=A.SBOX[a>>>24]<<24|A.SBOX[a>>>16&255]<<16|A.SBOX[a>>>8&255]<<8|A.SBOX[255&a]),i[o]=i[o-e]^a}for(var l=[],c=0;c<r;c++){var u=r-c,g=i[u-(c%4?0:4)];l[c]=c<4||u<=4?g:A.INV_SUB_MIX[0][A.SBOX[g>>>24]]^A.INV_SUB_MIX[1][A.SBOX[g>>>16&255]]^A.INV_SUB_MIX[2][A.SBOX[g>>>8&255]]^A.INV_SUB_MIX[3][A.SBOX[255&g]]}this._nRounds=n,this._keySchedule=i,this._invKeySchedule=l},l.prototype.encryptBlockRaw=function(t){return a(t=i(t),this._keySchedule,A.SUB_MIX,A.SBOX,this._nRounds)},l.prototype.encryptBlock=function(t){var e=this.encryptBlockRaw(t),n=r.allocUnsafe(16);return n.writeUInt32BE(e[0],0),n.writeUInt32BE(e[1],4),n.writeUInt32BE(e[2],8),n.writeUInt32BE(e[3],12),n},l.prototype.decryptBlock=function(t){var e=(t=i(t))[1];t[1]=t[3],t[3]=e;var n=a(t,this._invKeySchedule,A.INV_SUB_MIX,A.INV_SBOX,this._nRounds),o=r.allocUnsafe(16);return o.writeUInt32BE(n[0],0),o.writeUInt32BE(n[3],4),o.writeUInt32BE(n[2],8),o.writeUInt32BE(n[1],12),o},l.prototype.scrub=function(){o(this._keySchedule),o(this._invKeySchedule),o(this._key)},t.exports.AES=l},function(t,e,n){var r=n(35).Buffer,i=n(267);t.exports=function(t,e,n,o){if(r.isBuffer(t)||(t=r.from(t,"binary")),e&&(r.isBuffer(e)||(e=r.from(e,"binary")),8!==e.length))throw new RangeError("salt should be Buffer with 8 byte length");for(var a=n/8,s=r.alloc(a),A=r.alloc(o||0),l=r.alloc(0);a>0||o>0;){var c=new i;c.update(l),c.update(t),e&&c.update(e),l=c.digest();var u=0;if(a>0){var g=s.length-a;u=Math.min(a,l.length),l.copy(s,g,0,u),a-=u}if(u<l.length&&o>0){var h=A.length-o,d=Math.min(o,l.length-u);l.copy(A,h,u,u+d),o-=d}}return l.fill(0),{key:s,iv:A}}},function(t,e,n){"use strict";var r=n(53),i=n(78),o=i.getNAF,a=i.getJSF,s=i.assert;function A(t,e){this.type=t,this.p=new r(e.p,16),this.red=e.prime?r.red(e.prime):r.mont(this.p),this.zero=new r(0).toRed(this.red),this.one=new r(1).toRed(this.red),this.two=new r(2).toRed(this.red),this.n=e.n&&new r(e.n,16),this.g=e.g&&this.pointFromJSON(e.g,e.gRed),this._wnafT1=new Array(4),this._wnafT2=new Array(4),this._wnafT3=new Array(4),this._wnafT4=new Array(4);var n=this.n&&this.p.div(this.n);!n||n.cmpn(100)>0?this.redN=null:(this._maxwellTrick=!0,this.redN=this.n.toRed(this.red))}function l(t,e){this.curve=t,this.type=e,this.precomputed=null}t.exports=A,A.prototype.point=function(){throw new Error("Not implemented")},A.prototype.validate=function(){throw new Error("Not implemented")},A.prototype._fixedNafMul=function(t,e){s(t.precomputed);var n=t._getDoubles(),r=o(e,1),i=(1<<n.step+1)-(n.step%2==0?2:1);i/=3;for(var a=[],A=0;A<r.length;A+=n.step){var l=0;for(e=A+n.step-1;e>=A;e--)l=(l<<1)+r[e];a.push(l)}for(var c=this.jpoint(null,null,null),u=this.jpoint(null,null,null),g=i;g>0;g--){for(A=0;A<a.length;A++){(l=a[A])===g?u=u.mixedAdd(n.points[A]):l===-g&&(u=u.mixedAdd(n.points[A].neg()))}c=c.add(u)}return c.toP()},A.prototype._wnafMul=function(t,e){var n=4,r=t._getNAFPoints(n);n=r.wnd;for(var i=r.points,a=o(e,n),A=this.jpoint(null,null,null),l=a.length-1;l>=0;l--){for(e=0;l>=0&&0===a[l];l--)e++;if(l>=0&&e++,A=A.dblp(e),l<0)break;var c=a[l];s(0!==c),A="affine"===t.type?c>0?A.mixedAdd(i[c-1>>1]):A.mixedAdd(i[-c-1>>1].neg()):c>0?A.add(i[c-1>>1]):A.add(i[-c-1>>1].neg())}return"affine"===t.type?A.toP():A},A.prototype._wnafMulAdd=function(t,e,n,r,i){for(var s=this._wnafT1,A=this._wnafT2,l=this._wnafT3,c=0,u=0;u<r;u++){var g=(Q=e[u])._getNAFPoints(t);s[u]=g.wnd,A[u]=g.points}for(u=r-1;u>=1;u-=2){var h=u-1,d=u;if(1===s[h]&&1===s[d]){var f=[e[h],null,null,e[d]];0===e[h].y.cmp(e[d].y)?(f[1]=e[h].add(e[d]),f[2]=e[h].toJ().mixedAdd(e[d].neg())):0===e[h].y.cmp(e[d].y.redNeg())?(f[1]=e[h].toJ().mixedAdd(e[d]),f[2]=e[h].add(e[d].neg())):(f[1]=e[h].toJ().mixedAdd(e[d]),f[2]=e[h].toJ().mixedAdd(e[d].neg()));var p=[-3,-1,-5,-7,0,7,5,1,3],I=a(n[h],n[d]);c=Math.max(I[0].length,c),l[h]=new Array(c),l[d]=new Array(c);for(var C=0;C<c;C++){var m=0|I[0][C],y=0|I[1][C];l[h][C]=p[3*(m+1)+(y+1)],l[d][C]=0,A[h]=f}}else l[h]=o(n[h],s[h]),l[d]=o(n[d],s[d]),c=Math.max(l[h].length,c),c=Math.max(l[d].length,c)}var E=this.jpoint(null,null,null),v=this._wnafT4;for(u=c;u>=0;u--){for(var B=0;u>=0;){var b=!0;for(C=0;C<r;C++)v[C]=0|l[C][u],0!==v[C]&&(b=!1);if(!b)break;B++,u--}if(u>=0&&B++,E=E.dblp(B),u<0)break;for(C=0;C<r;C++){var Q,w=v[C];0!==w&&(w>0?Q=A[C][w-1>>1]:w<0&&(Q=A[C][-w-1>>1].neg()),E="affine"===Q.type?E.mixedAdd(Q):E.add(Q))}}for(u=0;u<r;u++)A[u]=null;return i?E:E.toP()},A.BasePoint=l,l.prototype.eq=function(){throw new Error("Not implemented")},l.prototype.validate=function(){return this.curve.validate(this)},A.prototype.decodePoint=function(t,e){t=i.toArray(t,e);var n=this.p.byteLength();if((4===t[0]||6===t[0]||7===t[0])&&t.length-1==2*n)return 6===t[0]?s(t[t.length-1]%2==0):7===t[0]&&s(t[t.length-1]%2==1),this.point(t.slice(1,1+n),t.slice(1+n,1+2*n));if((2===t[0]||3===t[0])&&t.length-1===n)return this.pointFromX(t.slice(1,1+n),3===t[0]);throw new Error("Unknown point format")},l.prototype.encodeCompressed=function(t){return this.encode(t,!0)},l.prototype._encode=function(t){var e=this.curve.p.byteLength(),n=this.getX().toArray("be",e);return t?[this.getY().isEven()?2:3].concat(n):[4].concat(n,this.getY().toArray("be",e))},l.prototype.encode=function(t,e){return i.encode(this._encode(e),t)},l.prototype.precompute=function(t){if(this.precomputed)return this;var e={doubles:null,naf:null,beta:null};return e.naf=this._getNAFPoints(8),e.doubles=this._getDoubles(4,t),e.beta=this._getBeta(),this.precomputed=e,this},l.prototype._hasDoubles=function(t){if(!this.precomputed)return!1;var e=this.precomputed.doubles;return!!e&&e.points.length>=Math.ceil((t.bitLength()+1)/e.step)},l.prototype._getDoubles=function(t,e){if(this.precomputed&&this.precomputed.doubles)return this.precomputed.doubles;for(var n=[this],r=this,i=0;i<e;i+=t){for(var o=0;o<t;o++)r=r.dbl();n.push(r)}return{step:t,points:n}},l.prototype._getNAFPoints=function(t){if(this.precomputed&&this.precomputed.naf)return this.precomputed.naf;for(var e=[this],n=(1<<t)-1,r=1===n?null:this.dbl(),i=1;i<n;i++)e[i]=e[i-1].add(r);return{wnd:t,points:e}},l.prototype._getBeta=function(){return null},l.prototype.dblp=function(t){for(var e=this,n=0;n<t;n++)e=e.dbl();return e}},function(t,e,n){var r=n(831),i=n(842),o=n(843),a=n(275),s=n(406),A=n(35).Buffer;function l(t){var e;"object"!=typeof t||A.isBuffer(t)||(e=t.passphrase,t=t.key),"string"==typeof t&&(t=A.from(t));var n,l,c=o(t,e),u=c.tag,g=c.data;switch(u){case"CERTIFICATE":l=r.certificate.decode(g,"der").tbsCertificate.subjectPublicKeyInfo;case"PUBLIC KEY":switch(l||(l=r.PublicKey.decode(g,"der")),n=l.algorithm.algorithm.join(".")){case"1.2.840.113549.1.1.1":return r.RSAPublicKey.decode(l.subjectPublicKey.data,"der");case"1.2.840.10045.2.1":return l.subjectPrivateKey=l.subjectPublicKey,{type:"ec",data:l};case"1.2.840.10040.4.1":return l.algorithm.params.pub_key=r.DSAparam.decode(l.subjectPublicKey.data,"der"),{type:"dsa",data:l.algorithm.params};default:throw new Error("unknown key id "+n)}throw new Error("unknown key type "+u);case"ENCRYPTED PRIVATE KEY":g=function(t,e){var n=t.algorithm.decrypt.kde.kdeparams.salt,r=parseInt(t.algorithm.decrypt.kde.kdeparams.iters.toString(),10),o=i[t.algorithm.decrypt.cipher.algo.join(".")],l=t.algorithm.decrypt.cipher.iv,c=t.subjectPrivateKey,u=parseInt(o.split("-")[1],10)/8,g=s.pbkdf2Sync(e,n,r,u,"sha1"),h=a.createDecipheriv(o,g,l),d=[];return d.push(h.update(c)),d.push(h.final()),A.concat(d)}(g=r.EncryptedPrivateKey.decode(g,"der"),e);case"PRIVATE KEY":switch(n=(l=r.PrivateKey.decode(g,"der")).algorithm.algorithm.join(".")){case"1.2.840.113549.1.1.1":return r.RSAPrivateKey.decode(l.subjectPrivateKey,"der");case"1.2.840.10045.2.1":return{curve:l.algorithm.curve,privateKey:r.ECPrivateKey.decode(l.subjectPrivateKey,"der").privateKey};case"1.2.840.10040.4.1":return l.algorithm.params.priv_key=r.DSAparam.decode(l.subjectPrivateKey,"der"),{type:"dsa",params:l.algorithm.params};default:throw new Error("unknown key id "+n)}throw new Error("unknown key type "+u);case"RSA PUBLIC KEY":return r.RSAPublicKey.decode(g,"der");case"RSA PRIVATE KEY":return r.RSAPrivateKey.decode(g,"der");case"DSA PRIVATE KEY":return{type:"dsa",params:r.DSAPrivateKey.decode(g,"der")};case"EC PRIVATE KEY":return{curve:(g=r.ECPrivateKey.decode(g,"der")).parameters.value,privateKey:g.privateKey};default:throw new Error("unknown key type "+u)}}t.exports=l,l.signature=r.signature},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(175),s=function(t){function e(e){void 0===e&&(e={});var n=t.call(this)||this;return n.addClass("p-Panel"),n.layout=o.createLayout(e),n}return i(e,t),Object.defineProperty(e.prototype,"widgets",{get:function(){return this.layout.widgets},enumerable:!0,configurable:!0}),e.prototype.addWidget=function(t){this.layout.addWidget(t)},e.prototype.insertWidget=function(t,e){this.layout.insertWidget(t,e)},e}(n(59).Widget);e.Panel=s,function(t){t.createLayout=function(t){return t.layout||new a.PanelLayout}}(o||(o={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9);!function(t){t.createMessage=function(t){if("comm_info_request"===t.msgType){const e=t.content;void 0===e.target_name&&(e.target_name=e.target),delete e.target}return{buffers:t.buffers||[],channel:t.channel,content:t.content,header:{date:(new Date).toISOString(),msg_id:t.msgId||r.UUID.uuid4(),msg_type:t.msgType,session:t.session,username:t.username||"",version:"5.2"},metadata:t.metadata||{},parent_header:t.parentHeader||{}}},t.isStreamMsg=function(t){return"stream"===t.header.msg_type},t.isDisplayDataMsg=function(t){return"display_data"===t.header.msg_type},t.isUpdateDisplayDataMsg=function(t){return"update_display_data"===t.header.msg_type},t.isExecuteInputMsg=function(t){return"execute_input"===t.header.msg_type},t.isExecuteResultMsg=function(t){return"execute_result"===t.header.msg_type},t.isErrorMsg=function(t){return"error"===t.header.msg_type},t.isStatusMsg=function(t){return"status"===t.header.msg_type},t.isClearOutputMsg=function(t){return"clear_output"===t.header.msg_type},t.isDebugEventMsg=function(t){return"debug_event"===t.header.msg_type},t.isCommOpenMsg=function(t){return"comm_open"===t.header.msg_type},t.isCommCloseMsg=function(t){return"comm_close"===t.header.msg_type},t.isCommMsgMsg=function(t){return"comm_msg"===t.header.msg_type},t.isInfoRequestMsg=function(t){return"kernel_info_request"===t.header.msg_type},t.isExecuteReplyMsg=function(t){return"execute_reply"===t.header.msg_type},t.isDebugRequestMsg=function(t){return"debug_request"===t.header.msg_type},t.isDebugReplyMsg=function(t){return"debug_reply"===t.header.msg_type},t.isInputRequestMsg=function(t){return"input_request"===t.header.msg_type},t.isInputReplyMsg=function(t){return"input_reply"===t.header.msg_type}}(e.KernelMessage||(e.KernelMessage={}))},function(t,e,n){var r=n(593),i=n(597);function o(e,n){return delete t.exports[e],t.exports[e]=n,n}t.exports={Parser:r,Tokenizer:n(594),ElementType:n(138),DomHandler:i,get FeedHandler(){return o("FeedHandler",n(1087))},get Stream(){return o("Stream",n(1098))},get WritableStream(){return o("WritableStream",n(600))},get ProxyHandler(){return o("ProxyHandler",n(1100))},get DomUtils(){return o("DomUtils",n(599))},get CollectingHandler(){return o("CollectingHandler",n(1101))},DefaultHandler:i,get RssHandler(){return o("RssHandler",this.FeedHandler)},parseDOM:function(t,e){var n=new i(e);return new r(n,e).end(t),n.dom},parseFeed:function(e,n){var i=new t.exports.FeedHandler(n);return new r(i,n).end(e),i.dom},createDomStream:function(t,e,n){var o=new i(t,e,n);return new r(o,e)},EVENTS:{attribute:2,cdatastart:0,cdataend:0,text:1,processinginstruction:2,comment:1,commentend:0,closetag:1,opentag:2,opentagname:1,error:1,end:0}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.Intent={NONE:"none",PRIMARY:"primary",SUCCESS:"success",WARNING:"warning",DANGER:"danger"}},function(t,e,n){"use strict";n.d(e,"b",function(){return A}),n.d(e,"a",function(){return g});var r=n(2),i=n(11),o=n(32),a=n(21),s=n(36);function A(t,e,n,r){return function(i){return i.lift(new l(t,e,n,r))}}var l=function(){function t(t,e,n,r){this.keySelector=t,this.elementSelector=e,this.durationSelector=n,this.subjectSelector=r}return t.prototype.call=function(t,e){return e.subscribe(new c(t,this.keySelector,this.elementSelector,this.durationSelector,this.subjectSelector))},t}(),c=function(t){function e(e,n,r,i,o){var a=t.call(this,e)||this;return a.keySelector=n,a.elementSelector=r,a.durationSelector=i,a.subjectSelector=o,a.groups=null,a.attemptedToUnsubscribe=!1,a.count=0,a}return r.__extends(e,t),e.prototype._next=function(t){var e;try{e=this.keySelector(t)}catch(t){return void this.error(t)}this._group(t,e)},e.prototype._group=function(t,e){var n=this.groups;n||(n=this.groups=new Map);var r,i=n.get(e);if(this.elementSelector)try{r=this.elementSelector(t)}catch(t){this.error(t)}else r=t;if(!i){i=this.subjectSelector?this.subjectSelector():new s.a,n.set(e,i);var o=new g(e,i,this);if(this.destination.next(o),this.durationSelector){var a=void 0;try{a=this.durationSelector(new g(e,i))}catch(t){return void this.error(t)}this.add(a.subscribe(new u(e,i,this)))}}i.closed||i.next(r)},e.prototype._error=function(t){var e=this.groups;e&&(e.forEach(function(e,n){e.error(t)}),e.clear()),this.destination.error(t)},e.prototype._complete=function(){var t=this.groups;t&&(t.forEach(function(t,e){t.complete()}),t.clear()),this.destination.complete()},e.prototype.removeGroup=function(t){this.groups.delete(t)},e.prototype.unsubscribe=function(){this.closed||(this.attemptedToUnsubscribe=!0,0===this.count&&t.prototype.unsubscribe.call(this))},e}(i.a),u=function(t){function e(e,n,r){var i=t.call(this,n)||this;return i.key=e,i.group=n,i.parent=r,i}return r.__extends(e,t),e.prototype._next=function(t){this.complete()},e.prototype._unsubscribe=function(){var t=this.parent,e=this.key;this.key=this.parent=null,t&&t.removeGroup(e)},e}(i.a),g=function(t){function e(e,n,r){var i=t.call(this)||this;return i.key=e,i.groupSubject=n,i.refCountSubscription=r,i}return r.__extends(e,t),e.prototype._subscribe=function(t){var e=new o.a,n=this.refCountSubscription,r=this.groupSubject;return n&&!n.closed&&e.add(new h(n)),e.add(r.subscribe(t)),e},e}(a.a),h=function(t){function e(e){var n=t.call(this)||this;return n.parent=e,e.count++,n}return r.__extends(e,t),e.prototype.unsubscribe=function(){var e=this.parent;e.closed||this.closed||(t.prototype.unsubscribe.call(this),e.count-=1,0===e.count&&e.attemptedToUnsubscribe&&e.unsubscribe())},e}(o.a)},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});class r{constructor(t,e,n=[]){this.name=t,this.baseType=e,this.genericArgs=n}get serializableName(){return this.name}static genericArgsMatch(t,e){return t.genericArgs.length==e.genericArgs.length&&t.genericArgs.every((t,n)=>t.isInstanceOf(e.genericArgs[n]))}isInstanceOf(t){if("Any"==t.name||"Any"==this.name)return!0;var e=this;do{if(e.name==t.name&&r.genericArgsMatch(e,t))return!0;e=e.baseType}while(null!=e);return!1}equals(t){return this.name==t.name}}e.Type=r;class i{static registerType(t){this.registered.push(t)}static array(t){return new r("Array",i.Object,[t])}static tuple(...t){return new r("Tuple",i.Object,t)}static findType(t){return i.registered.find(e=>e.name==t)}}i.Object=new r("Object",null),i.Any=new r("Any",i.Object),i.String=new r("String",i.Object),i.Date=new r("Date",i.Object),i.DateTime=new r("DateTime",i.Date),i.Number=new r("Number",i.Object),i.Table=new r("Table",i.Object),i.Row=new r("Row",i.Object),i.Error=new r("Error",i.Object),i.Boolean=new r("Boolean",i.Any),i.Array=i.array(i.Any),i.Color=new r("Color",i.Object),i.registered=[i.Object,i.Any,i.String,i.DateTime,i.Date,i.Number,i.Table,i.Row,i.Error,i.Boolean,i.Array,i.Color],e.Types=i},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(6),i=n(66),o=n(9),a=n(64),s=n(1274),A=n(0),l=n(25);class c extends r.Widget{constructor({services:t,uuid:e}){super(),this._isInitialized=!1,this._isIdle=!0,this._state=c.State.Uninitialized,this._refreshRequested=new a.Subject,this._cancelRequested=new a.Subject,this.uuid=e||o.UUID.uuid4(),this.id=this.uuid,this.RefreshRequested=this._refreshRequested.asObservable(),this.CancelRequested=this._cancelRequested.asObservable(),this.title.label=this.getName(),this.layout=new r.StackedLayout,this.context=t,this.addClass("m-Part"),this.overlay=new c.Overlay(this),this.overlay.hide(),this.layout.addWidget(this.overlay)}static GetMetadata(){return new c.PartMetadata}get state(){return this._state}get stateDetail(){return this._stateDetail}get isInitialized(){return this._isInitialized}get isError(){return this.state===c.State.Error}get isCanceled(){return this.state===c.State.Canceled}get isIdle(){return this._isIdle}refresh(){this._refreshRequested.next(!1)}userRefresh(){this._refreshRequested.next(!0)}cancel(){this._cancelRequested.next()}dispose(){this.isDisposed||(this._refreshRequested.complete(),this._cancelRequested.complete(),this._stateDetail=null,super.dispose())}getName(){return this.constructor._FactoryName||this.constructor.name}processMessage(t){switch(t.type){case"before-initialize":return this._isIdle=!1,this.setState(c.State.Initializing),this.onBeforeInitialize(t);case"after-initialize":return this._isIdle=!0,this._isInitialized=!0,this.setState(c.State.Idle),this.onAfterInitialize(t);case"before-render":return this._isIdle=!1,this.setState(c.State.Rendering),this.onBeforeRender(t);case"after-render":return this._isIdle=!0,this.setState(c.State.Idle),this.onAfterRender(t);case"before-calculate":return this._isIdle=!1,this.setState(c.State.Calculating,[]),this.onBeforeCalculate(t);case"after-calculate":return this._isIdle=!0,this.setState(c.State.Idle),this.onAfterCalculate(t);case"before-option-calc":const e=t;return this.state!==c.State.Calculating?(this._stateDetail=[...e.options],this.layout.addWidget(this.overlay),this.overlay.update()):(this._stateDetail.push(...e.options),this.layout.addWidget(this.overlay),this.overlay.update()),this.onOptionCalculating(e);case"after-option-calc":const n=t,r=this.stateDetail;if(this.state!==c.State.Calculating)return this.onOptionCalculated(n);const i=new Set(n.options),o=[];for(let t=0;t<r.length;t++)i.has(r[t])?i.delete(r[t]):o.push(r[t]);return this._stateDetail=o,this.layout.addWidget(this.overlay),this.overlay.update(),this.onOptionCalculated(n);case"init-error":case"render-error":case"option-error":this._isIdle=!0;const a=t;return this.setState(c.State.Error,a.errorDetail),this.onError(a);case"cancel":return this._isIdle=!0,this.setState(c.State.Canceled),this.onCancel(t);case"wait-for-user":return this._isIdle=!0,void this.setState(c.State.WaitForUser);default:super.processMessage(t)}}error(t,e){const n=new c.Lifecycle.ErrorMessage(t,e);l.MessageLoop.sendMessage(this,n)}optionCalculating(t,e){const n="string"==typeof e?[e]:e,r=new c.Lifecycle.CalculatingMessage(t,new Set(n));l.MessageLoop.sendMessage(this,r)}onBeforeInitialize(t){}onAfterInitialize(t){}onBeforeRender(t){}onAfterRender(t){}onBeforeCalculate(t){}onAfterCalculate(t){}onError(t){}onCancel(t){}onOptionCalculating(t){}onOptionCalculated(t){}setState(t,e){this._state=t,this._stateDetail=e,this.layout.addWidget(this.overlay),this.overlay.update()}}e.Part=c,function(t){let e,n;!function(t){t.BeforeInitialize=new l.Message("before-initialize"),t.AfterInitialize=new l.Message("after-initialize"),t.BeforeRender=new l.Message("before-render"),t.AfterRender=new l.Message("after-render"),t.BeforeCalculate=new l.Message("before-calculate"),t.AfterCalculate=new l.Message("after-calculate"),t.Cancel=new l.Message("cancel"),t.WaitForUser=new l.Message("wait-for-user");t.ErrorMessage=class extends l.Message{constructor(t,e){super(e),t instanceof Error||((t=new Error(t)).message="Wrapped"+t.message);let n="";switch(e){case"init-error":n="Error in Initialization";break;case"option-error":n="Error in Option Evaluation";break;case"render-error":n="Error in Rendering"}t.message=`${n}: ${t.message}`,this.errorDetail=t}static IsErrorMsg(t){return-1!==["init-error","render-error","option-error"].indexOf(t.type)}};class e extends l.Message{constructor(t,e){super(t),this.options=e}static IsCalcMsg(t){return"before-option-calc"===t.type||"after-option-calc"===t.type}get isConflatable(){return!0}conflate(t){if(!e.IsCalcMsg(t))return!1;if(t.type!==this.type)return!1;for(const e of t.options)this.options.add(e);return!0}}t.CalculatingMessage=e}(e=t.Lifecycle||(t.Lifecycle={}));t.Overlay=class extends i.ReactWrapperWidget{constructor(t){super(),this.addClass("m-PartOverlay"),this.addClass("load"),this.owner=t}shouldShowRegion(){return this.owner.state!==n.Idle&&!(this.owner.hasClass("m-hide-part-overlays")&&this.owner.state!==n.Error&&this.owner.state!==n.WaitForUser&&this.owner.state!==n.Canceled)}render(){if(!this.shouldShowRegion())return this.hide(),A.createElement("div");const t=n[this.owner.state],e=this.owner.stateDetail;return this.show(),A.createElement(s.PartOverlay,{key:"overlay",partState:t,partStateDetail:e,onRequestRefresh:()=>this.owner.userRefresh(),onRequestCancel:()=>this.owner.cancel()})}};t.PartMetadata=class{constructor(){this.options=new Map}addOption(t,e,n,r={}){this.options.set(t,{name:t,type:e,value:n,...r})}[Symbol.iterator](){return this.options.values()}getMetadataForOption(t){return this.options.get(t)}},function(t){t[t.Uninitialized=0]="Uninitialized",t[t.Initializing=1]="Initializing",t[t.Calculating=2]="Calculating",t[t.Rendering=3]="Rendering",t[t.Idle=4]="Idle",t[t.Error=5]="Error",t[t.Canceled=6]="Canceled",t[t.WaitForUser=7]="WaitForUser"}(n=t.State||(t.State={}))}(c=e.Part||(e.Part={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(1176);e.ListBox=r.ListBox;var i=n(1177);e.PropertiesEditor=i.PropertiesEditor;var o=n(1178);e.ProgressBar=o.ProgressBar;var a=n(313);e.useRender=a.useRender,e.renderOnEmit=a.renderOnEmit,e.usePrevious=a.usePrevious,e.useIntermediate=a.useIntermediate;var s=n(1179);e.UncontrolledInput=s.UncontrolledInput,function(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}(n(1180))},function(t,e,n){var r=n(640),i=n(322);t.exports=Object.keys||function(t){return r(t,i)}},function(t,e){t.exports=!0},function(t,e){var n=0,r=Math.random();t.exports=function(t){return"Symbol(".concat(void 0===t?"":t,")_",(++n+r).toString(36))}},function(t,e){e.f={}.propertyIsEnumerable},function(t,e){var n={componentWillMount:!0,componentDidMount:!0,componentWillReceiveProps:!0,getSnapshotBeforeUpdate:!0,shouldComponentUpdate:!0,componentWillUpdate:!0,componentDidUpdate:!0,componentWillUnmount:!0},r={getDerivedStateFromProps:!0};function i(t,e,n){var r=!0;return Array.isArray(e)&&(r="after"!==e[0],e=e[1]),t?function(){var i=n?null:this;r&&e.apply(i,arguments),t.apply(i,arguments),!r&&e.apply(i,arguments)}:e}t.exports=function(t,e){var o=Object.create(null);for(var a in e)r[a]&&(t.constructor[a]=i(o[a]=t.constructor[a],e[a],!0));for(var a in e)n[a]&&(t[a]=i(o[a]=t[a],e[a]));return function(e){var n=r[e]?t.constructor:t;if(e&&e in o)n[e]=o[e];else for(var e in o)n[e]=o[e]}},t.exports.mixin=function(t,e){return spyOnComponent(t.prototype,e),t}},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t){return t===t.window?t:9===t.nodeType&&(t.defaultView||t.parentWindow)},t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.isShallowEqual=function(t,e){if(t===e)return!0;if(t instanceof Date&&e instanceof Date)return+t==+e;if("object"!=typeof t&&"object"!=typeof e)return t===e;if(typeof t!=typeof e)return!1;if(null==t||null==e)return!1;var n=Object.keys(t),r=Object.keys(e);if(n.length!==r.length)return!1;for(var o=0;o<n.length;o++)if(!i(e,n[o])||t[n[o]]!==e[n[o]])return!1;return!0},e.chunk=function(t,e){var n=0,r=t?t.length:0,i=[];e=Math.max(+e||1,1);for(;n<r;)i.push(t.slice(n,n+=e));return i},e.groupBySortedKeys=function(t,e,n){var r="function"==typeof t?t:function(e){return e[t]};return n=n||[],e.reduce(function(t,e){var o=r(e);return i(t,o)?t[o].push(e):(n.push(o),t[o]=[e]),t},{})},e.has=e.makeArray=void 0;var r;(r=n(1249))&&r.__esModule;e.makeArray=function(t){return null==t?[]:[].concat(t)};var i=function(t,e){return!!t&&Object.prototype.hasOwnProperty.call(t,e)};e.has=i},function(t,e,n){"use strict";e.__esModule=!0,e.pick=function(t,e){var n=Object.keys(e.propTypes),r={};return Object.keys(t).forEach(function(e){-1!==n.indexOf(e)&&(r[e]=t[e])}),r},e.pickElementProps=function(t){for(var e=arguments.length,n=new Array(e>1?e-1:0),a=1;a<e;a++)n[a-1]=arguments[a];var s=o.apply(void 0,[t].concat(n)),A={};return Object.keys(s).forEach(function(t){(-1!==r.indexOf(t)||i.some(function(e){return!!t.match(e)}))&&(A[t]=s[t])}),A},e.omitOwn=o;var r=["style","className","role","id","autocomplete","size","tabIndex","maxLength","name"],i=[/^aria-/,/^data-/,/^on[A-Z]\w+/];function o(t){for(var e=Object.keys(t.constructor.propTypes),n=arguments.length,r=new Array(n>1?n-1:0),i=1;i<n;i++)r[i-1]=arguments[i];var o=r.reduce(function(t,e){return t.concat(Object.keys(e.propTypes))},e),a={};return Object.keys(t.props).forEach(function(e){-1===o.indexOf(e)&&(a[e]=t.props[e])}),a}},function(t,e,n){"use strict";e.__esModule=!0,e.notify=function(t,e){t&&t.apply(null,[].concat(e))},e.instanceId=function(t,e){void 0===e&&(e="");return t.__id||(t.__id=(n="rw_",""+((null==n?"":n)+ ++r))),(t.props.id||t.__id)+e;var n},e.isFirstFocusedRender=function(t){return t._firstFocus||(t.state.focused||!!t.props.open)&&(t._firstFocus=!0)};var r=0},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9),i=n(50),o=n(127),a=n(25),s=n(6),A=n(64),l=n(1286),c=n(43),u=n(28);class g extends s.Widget{constructor(t,e=r.UUID.uuid4()){super(),this._didUpdate=!1,this._properties={},this._content=new s.Widget,this._isFocused=!1,this._parentRegion=null,this._isStale=!1,this._OnStaleSrc=new A.Subject,this._OnStale=this._OnStaleSrc.asObservable(),this.overlay=new i.DisposableSet,this.rootOverlay=new i.DisposableSet,this._owner=t,this.metadata=this.constructor.GetMetadata(),this.layout=new g.DashboardPanelLayout,this.title.closable=!0,this.titlebar=new l.TitleBar.WidgetWrapper(this),this.titlebar.hide(),this.layout.addWidget(this.titlebar),s.BoxLayout.setStretch(this.titlebar,0),this.addClass("maven_dashboard"),this.id=e,this.metadata=this.constructor.GetMetadata()}static GetMetadata(){const t=new g.Metadata({iconClass:"material-icons md-18 md-dark",iconText:"insert_chart"});return t.addMetadata("backgroundColor",{prettyName:"Appearance.Background Color",type:u.Types.Color,documentation:"The background color for this region.",default:new u.Color("transparent")}),t.addMetadata("borderWidth.px",{prettyName:"Appearance.Border Width (px)",type:u.Types.Number,documentation:"The width of the border, in pixels.\nIf 0, the border will not be shown.",default:0}),t.addMetadata("borderColor",{prettyName:"Appearance.Border Color",type:u.Types.Color,documentation:"The color of the border around this region",default:new u.Color("black")}),t.addMetadata("caption",{prettyName:"Title.Caption",type:u.Types.String,documentation:"The text displayed in this region's titlebar, if the titlebar is visible",default:"New Region"}),t.addMetadata("captionBackground",{prettyName:"Title.Background Color",type:u.Types.Color,documentation:"The background color of the titlebar. If not transparent, the color will be used even when the region is focused.",default:new u.Color("transparent")}),t.addMetadata("captionColor",{prettyName:"Title.Text Color",type:u.Types.Color,documentation:"The font color used for the caption.",default:new u.Color("transparent")}),t.addMetadata("padding.px",{prettyName:"Appearance.Padding (px)",type:u.Types.Number,documentation:"Padding applied to the interior of this layout region",default:0}),t.addMetadata("showRegion",{prettyName:"General.Show Region?",type:u.Types.Boolean,documentation:"If false, this region will be hidden from view.",default:!0}),t.addMetadata("showTitle",{prettyName:"Title.Show Titlebar?",type:u.Types.Boolean,documentation:"If false, the titlebar for this region will be hidden from view.",default:!0}),t.addMetadata("maximized",{prettyName:"General.Maximized",type:u.Types.Boolean,documentation:"",default:!1}),t}dispose(){this.isDisposed||(this._OnStaleSrc.complete(),delete this._owner,delete this._parentRegion,super.dispose())}get content(){return this._content}set content(t){null!=this._content&&this._content.removeClass("m-Region-Content"),this._content=t,this.content.addClass("m-Region-Content"),this.content.node.setAttribute("tabindex","-1"),this.layout.addWidget(this.content),s.BoxLayout.setStretch(this.content,1)}get properties(){return this._properties}set properties(t){this._properties=t}get uuid(){return this.id}get parentRegion(){return this._parentRegion}set parentRegion(t){t!==this.parentRegion&&(this._parentRegion=t,h.Chroming.set(this,new i.DisposableSet))}get isFocused(){return this._isFocused}get isStale(){return this._isStale}get OnStale(){return this._OnStale}get owner(){return this._owner}sizeContentToFit(t){this.getLayoutProperty("maximized")?(t=this.owner.bounds,this.addClass("m-Region-Maximized")):this.removeClass("m-Region-Maximized"),this.calculatedSize=t,this.title.label=this.getLayoutProperty("caption"),this.title.caption=this.getLayoutProperty("caption"),this.setHidden(!this.getLayoutProperty("showRegion"));const{background:e,color:n}=this.title.dataset,r=this.getLayoutProperty("captionBackground"),i=this.getLayoutProperty("captionColor");e===r.color&&n===i.color||(this.title.dataset={...this.title.dataset,background:r?r.color:null,color:i?i.color:null}),this.parentRegion&&this.parentRegion.hasClass("m-hide-child-titlebars")?this.titlebar.hide():this.owner.forceTitlebars?this.titlebar.show():this.titlebar.setHidden(!this.getLayoutProperty("showTitle"))}setStale(){this.isStale||(this._isStale=!0,this._OnStaleSrc.next())}setFresh(){this._isStale=!1}getLayoutProperty(t){const e=this._properties[t];if(null!=e)return e;const n=this.getDefault(""+t);if(null!=n)return n;if(this.parentRegion)return this.parentRegion.getAttachedPropertyForChild(""+t,this);throw Error("Property "+t+" not found")}setLayoutProperty(t,e){this._properties[t]=e,this.setStale()}focus(){this.content.node.focus({preventScroll:!0})}updateFromProperties(){const t=this.calculatedSize||{width:0,height:0},e=this.node.style;e.width=`${Math.floor(t.width)}px`,e.height=`${Math.floor(t.height)}px`,e.borderWidth=`${this.getLayoutProperty("borderWidth.px")}px`,e.borderStyle="solid",e.borderColor=this.getLayoutProperty("borderColor").color,e.backgroundColor=this.getLayoutProperty("backgroundColor").color,e.padding=`${this.getLayoutProperty("padding.px")}px`,e.left=`${t.left||0}px`,e.top=`${t.top||0}px`,this._didUpdate=!0}startDrag(t,e){if(null==this.parentRegion)return;if(this.getLayoutProperty("maximized"))return;const n=(this.parentRegion||this.owner.root).createDragShadow(this,t,e),i=new r.MimeData;i.setData("text/vnd.maven.target",this.id),i.setData("text/vnd.maven.startPosition",[t,e]);const a=new o.Drag({mimeData:i,dragImage:n,proposedAction:"move",source:this});this.setLayoutProperty("showRegion",!1),this.owner.root.showRootOverlay(),u.FrameTools.DisableFramePointerEvents(),a.start(t,e).then(t=>{this.setLayoutProperty("showRegion",!0),this.owner.root.hideRootOverlay(),this.owner.dockPreview.stopPreview(),u.FrameTools.EnableFramePointerEvents()})}handleEvent(t){switch(t.type){case"p-dragenter":this.onDragEnter(t);break;case"p-dragover":this.onDragOver(t);break;case"p-dragleave":this.onDragLeave(t);break;case"p-drop":this.onDrop(t)}}attachChrome(t){h.Chroming.get(this).add(t),this.layout.addWidget(t)}showOverlay(){if(this.overlay.isDisposed||!(this.overlay._items.size>0))for(const t of["center","inner-left","inner-right","inner-top","inner-bottom","outer-left","outer-right","outer-top","outer-bottom"])this.createDropzoneTarget(t)}showRootOverlay(){for(const t of["far-outer-left","far-outer-right","far-outer-top","far-outer-bottom"])this.createDropzoneTarget(t)}hideRootOverlay(){this.rootOverlay.dispose(),this.rootOverlay=new i.DisposableSet}hideOverlay(){this.overlay.dispose(),this.overlay=new i.DisposableSet,this.content.removeClass("temp-disable-pointer")}processMessage(t){switch(t.type){case"before-focus":this._isFocused=!0,this.titlebar.addClass("m-mod-active"),this.onBeforeFocus(t);break;case"after-focus":this.onAfterFocus(t);break;case"before-blur":this.onBeforeBlur(t);break;case"after-blur":this._isFocused=!1,this.titlebar.removeClass("m-mod-active"),this.onAfterBlur(t);break;default:super.processMessage(t)}}onBeforeFocus(t){}onAfterFocus(t){}onBeforeBlur(t){}onAfterBlur(t){}onDragEnter(t){t.mimeData.hasData("text/vnd.maven.target")&&(t.preventDefault(),t.stopPropagation(),this.content.addClass("temp-disable-pointer"))}onDragOver(t){t.mimeData.hasData("text/vnd.maven.target")&&(this.showOverlay(),t.preventDefault(),t.stopPropagation())}onDragLeave(t){if(t.mimeData.hasData("text/vnd.maven.target")){if(null!=t.relatedTarget){if(t.relatedTarget.matches(`div[data-owner="${this.id}"].dropzone`))return void t.stopPropagation()}this.hideOverlay()}}onDrop(t){t.mimeData.hasData("text/vnd.maven.target")&&(t.preventDefault(),t.stopPropagation())}onAfterAttach(){this.node.addEventListener("p-dragenter",this),this.node.addEventListener("p-dragover",this),this.node.addEventListener("p-dragleave",this),this.node.addEventListener("p-drop",this)}onBeforeDetach(){this.node.removeEventListener("p-dragenter",this),this.node.removeEventListener("p-dragover",this),this.node.removeEventListener("p-dragleave",this),this.node.removeEventListener("p-drop",this)}onCloseRequest(){this.owner.removeRegion(this.uuid)}onActivateRequest(){this.focus()}getDefault(t){return this.metadata.getMetadata(t).default}createDropzoneTarget(t){const e=document.createElement("div"),n=t.startsWith("far");e.classList.add("dropzone"),e.dataset.zone=t,e.dataset.owner=this.id,e.addEventListener("p-dragenter",t=>{t.preventDefault(),t.stopPropagation()}),e.addEventListener("p-dragover",e=>{e.preventDefault(),e.stopPropagation(),e.dropAction="move",n?this.owner.dockPreview.startPreviewFor(t):this.owner.dockPreview.startPreviewFor(t,this.id)}),e.addEventListener("p-dragleave",e=>{e.preventDefault(),e.stopPropagation(),e.relatedTarget&&e.relatedTarget.matches(`div[data-owner="${this.id}"].dropzone`)||(this.owner.dockPreview.stopPreview(),t.startsWith("far")||this.hideOverlay())}),e.addEventListener("p-drop",e=>{this.hideOverlay(),e.preventDefault(),e.stopPropagation(),e.dropAction="move";const n=e.mimeData.getData("text/vnd.maven.target");if(t.startsWith("far"))return void this.owner.moveToRootZone(t,n);const r=this.id;this.owner.moveToRelativeZone(t,n,r)});const r=document.createElement("div");r.classList.add("dropzone__preview"),e.appendChild(r);const i=new s.Widget({node:e});this.attachChrome(i),(n?this.rootOverlay:this.overlay).add(i)}}var h;e.DashboardLayoutRegion=g,function(t){let e;!function(t){t.BeforeFocus=new a.Message("before-focus"),t.AfterFocus=new a.Message("after-focus"),t.BeforeBlur=new a.Message("before-blur"),t.AfterBlur=new a.Message("after-blur")}(e=t.LayoutMsg||(t.LayoutMsg={}));t.DashboardPanelLayout=class extends s.PanelLayout{onFitRequest(t){const e=this.parent.calculatedSize||{width:0,height:0};this.parent.sizeContentToFit(e)}onResize(t){let e,n=this.parent.calculatedSize||{};e=t===s.Widget.ResizeMessage.UnknownSize?this.parent.calculatedSize||{...n,width:0,height:0}:{...n,width:t.width,height:t.height},this.parent.calculatedSize&&e.width===this.parent.calculatedSize.width&&e.height===this.parent.calculatedSize.height||this.parent.sizeContentToFit(e),this._update()}onUpdateRequest(t){this._update()}_update(){this.parent.updateFromProperties();const t=null!=this.parent.parentRegion;t&&this.parent.parentRegion._markSubtreeForUpdate();for(const t of this.widgets)a.MessageLoop.sendMessage(t,s.Widget.ResizeMessage.UnknownSize);t&&this.parent.parentRegion._updateSkippedSubtrees()}};class n{constructor({iconClass:t,iconText:e}){this.description="",this.remarks="",this.name="",this.metadataMap=new Map,this.iconClass=t,this.iconText=e}addMetadata(t,e){const r=Object.assign({prettyName:t},n.defaultData,e);this.metadataMap.set(t,r)}getMetadata(t){return this.metadataMap.get(t)}getAllProperties(){return this.metadataMap.entries()}}n.defaultData={type:u.Types.Any,documentation:"",default:null},t.Metadata=n}(g=e.DashboardLayoutRegion||(e.DashboardLayoutRegion={})),function(t){t.Chroming=new c.AttachedProperty({create:()=>new i.DisposableSet,changed:(t,e,n)=>{e.dispose()},name:"RegionChroming"})}(h||(h={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(125),i=n(124),o=n(6),a=n(25),s=n(28),A=n(43);class l extends i.RegionWithChildren{constructor(t,e){super(t,e),this.typeName=r.LayoutTypes.StackPanelLayoutRegion,this.splitters=new Set,this.originalSizings=[],this.addClass("maven_stackpanel"),this.content=new o.Panel,this.installContentTap()}static GetMetadata(){const t=super.GetMetadata();return t.name="Stack Panel",t.iconText="view_stream",t.description="Arranges children either horizontally or vertically in a stack.",t.remarks='\nThe Stack Panel is the most essential layout region, and the root of a dashboard\nis always a Stack Panel. Stack children can be sized either using an absolute\nnumber of pixels ("Fixed Size") or a relative ratio ("Flex Size"). If you set\none property on a child, the other will be cleared.\n\nFixed sizes are useful for UI controls like sliders, that don\'t usefully expand\nto fill their container.\n\nThis container will insert draggable splitters between each child, that can be\nused to resize them.\n\n> #### Accordioning\n>\n> This container supports arranging children in a collapsable accordion layout.\n> When Accordioning is enabled, the splitters are replaced with collapsers that,\n> when clicked, will expand or collapse their associated region. These will\n> render _in place of_ the titlebars of a region.\n>\n> The accordion mode can be set to either `Single` or `Multi`. In `Single`\n> mode, expanding an accordion will collapse all others in the stack. By\n> contrast, `Multi` mode will allow multiple accordions to be open at once.\n> Both modes allow all regions to be collapsed. Accordioning can be disabled by\n> setting the Accordion mode to `None`.\n',t.addMetadata("horizontal",{prettyName:"Stack Panel.Align Horizontal?",type:s.Types.Boolean,documentation:"If true, children will be arranged left-to-right instead of top-to-bottom.",default:!1}),t.addMetadata("showSplitters",{prettyName:"Stack Panel.Show Splitters?",type:s.Types.Boolean,documentation:"If true, draggable splitters will be shown between the children of this region, allowing them to be resized.",default:!0}),t.addMetadata("splitterWidth.px",{prettyName:"Stack Panel.Splitter Width (px)",type:s.Types.Number,documentation:"How wide the draggable splitters between children should be.",default:4}),t.addMetadata("accordionMode",{prettyName:"Stack Panel.Accordion Mode",type:s.Types.String,schema:{type:"string",enum:["None","Single","Multi"]},documentation:"If enabled, how to display the children in an accordion layout. One of `None`, `Single`, or `Multi`.",default:"None"}),t.addAttachedMetadata(l.FixedSize,{type:s.Types.Number,documentation:"If specified, an exact size in pixels for this region. If Stretch is specified, this property will have no effect.",default:null}),t.addAttachedMetadata(l.FlexSize,{type:s.Types.Number,documentation:"If specified, how much this region should stretch to fill compared  to other regions. If fixed size is specified, this property will have no effect.",default:1}),t.addAttachedMetadata(l.IsExpanded,{type:s.Types.Boolean,documentation:"For Accordions, whether this child is expanded (true) or collapsed (false).",default:!0}),t}insertChild(t,e){this.removeSplitters(),super.insertChild(t,e)}dispose(){this.isDisposed||(this.removeSplitters(),super.dispose())}layoutChildren(){if(this.removeSplitters(),0===this.widgets.length)return;if(void 0===this.calculatedSize)return void console.warn("not yet setup");const t=!!this.getLayoutProperty("horizontal"),e=+this.getLayoutProperty("padding.px"),n=t?"width":"height",r=t?"height":"width",i=this.titlebar.isVisible?this.titlebar.node.getBoundingClientRect().height:0,s=this.calculatedSize[r]-2*e-(t?i:0),A="None"!==this.getLayoutProperty("accordionMode");this.toggleClass("m-hide-child-titlebars",A),this.toggleClass("m-uses-accordions",A);const c=A?25:this.getLayoutProperty("showSplitters")?+this.getLayoutProperty("splitterWidth.px"):0;let u=0,g=this.getMaxSizeConstraint(),h=0;for(const t of this.widgets)t.getLayoutProperty("showRegion")&&(u+=c,A&&!l.IsExpanded.get(t)||(null==l.FlexSize.get(t)?u+=+l.FixedSize.get(t):h+=+l.FlexSize.get(t)));A||(u-=c),u>g&&(h=1/0);const d=(g-u)/h;for(let e=0;e<this.widgets.length;e++){const i=this.widgets[e];let u;if(u=A&&!l.IsExpanded.get(i)?0:i.getLayoutProperty("showRegion")?null!=l.FixedSize.get(i)?+l.FixedSize.get(i):d*+l.FlexSize.get(i):0,i.calculatedSize={[n]:u,[r]:s},a.MessageLoop.sendMessage(i,o.Widget.Msg.FitRequest),!i.getLayoutProperty("showRegion"))continue;if(A){const e=this.makeAccordion(i,t,this.calculatedSize);i.node.insertAdjacentElement("beforebegin",e);continue}let g=!1;for(let t=e+1;t<this.widgets.length;t++){if(this.widgets[t].getLayoutProperty("showRegion")){g=!0;break}}if(!g)continue;const h=this.makeSplitter(t,c,this.calculatedSize);i.node.insertAdjacentElement("afterend",h)}}updateFromProperties(){const t=!!this.getLayoutProperty("horizontal");this.node.classList.remove(t?"maven_layout_vertical":"maven_layout_horizontal"),this.node.classList.add(t?"maven_layout_horizontal":"maven_layout_vertical"),super.updateFromProperties()}handleEvent(t){if(!t.defaultPrevented)switch(t.type){case"pointerdown":if(t.currentTarget.classList.contains("m-StackPanel-accordion")){const e=this.widgets.find(e=>e.node.previousSibling===t.currentTarget),n=l.IsExpanded.get(e);return void l.IsExpanded.set(e,!n)}return void this.startResize(t);case"pointerup":return void this.endResize();case"pointermove":return void this.moveResize(t);case"contextmenu":case"keyup":return void this.cancelResize(t);default:super.handleEvent(t)}}onAfterAttach(){this.node.querySelectorAll("div.m-StackPanel-splitter").forEach(t=>{t.addEventListener("pointerdown",this)}),super.onAfterAttach()}onBeforeDetach(){this.node.querySelectorAll("div.m-StackPanel-splitter").forEach(t=>{t.removeEventListener("pointerdown",this)}),super.onBeforeDetach()}makeAccordion(t,e,n){const r=t.getLayoutProperty("captionColor"),i=t.getLayoutProperty("captionBackground"),o=t.getLayoutProperty("caption")||"",a=l.IsExpanded.get(t),s=document.createElement("span");s.classList.add("m-FlippyTriangle"),s.dataset.collapsed=a?"false":"true";const A=document.createElement("div");A.classList.add("m-StackPanel-accordion"),A.appendChild(s),A.addEventListener("pointerdown",this),this.splitters.add(A),"transparent"!==r.color&&(A.style.color=r.color,s.style.borderLeftColor=r.color),"transparent"!==i.color&&(A.style.backgroundColor=i.color);const c=document.createElement("span");return c.style.overflow="auto",e||(c.style.display="inline-block"),c.textContent=o,A.appendChild(c),e?(A.style.width="23px",A.style.height=n.height+"px"):(A.style.width=n.width+"px",A.style.height="23px"),A}makeSplitter(t,e,n){const r=document.createElement("div");return r.classList.add("m-StackPanel-splitter"),r.addEventListener("pointerdown",this),this.splitters.add(r),t?(r.style.width=e+"px",r.style.height=n.height+"px"):(r.style.width=n.width+"px",r.style.height=e+"px"),r}getMaxSizeConstraint(){if(null==this.calculatedSize)return 0;const t=!!this.getLayoutProperty("horizontal"),e=+this.getLayoutProperty("padding.px"),n=t?"width":"height",r=this.titlebar.isVisible?this.titlebar.node.getBoundingClientRect().height:0;return this.calculatedSize[n]-2*e-(t?0:r)}removeSplitters(){for(const t of this.splitters)t.remove();this.splitters.clear()}startResize(t){t.preventDefault(),t.stopPropagation(),s.FrameTools.DisableFramePointerEvents(),this.dragIndex=-1;for(let e=0;e<this.widgets.length;e++){const n=this.widgets[e];if(n.node.nextSibling===t.target&&(this.dragIndex=e),!n.getLayoutProperty("showRegion")){this.originalSizings.push([null,0]);continue}const r=l.FixedSize.get(n);null===r?this.originalSizings.push(["flexSize",+(l.FlexSize.get(n)||1)]):this.originalSizings.push(["fixedSize.px",+r])}if(-1===this.dragIndex)return this.dragIndex=void 0,void(this.originalSizings=[]);this.getLayoutProperty("horizontal")?this.dragStart=t.screenX:this.dragStart=t.screenY;for(const t of l.evTypes)document.addEventListener(t,this)}endResize(){this.originalSizings=[],this.dragIndex=void 0,this.dragStart=void 0;for(const t of l.evTypes)document.removeEventListener(t,this);s.FrameTools.EnableFramePointerEvents()}moveResize(t){t.preventDefault(),t.stopPropagation();const e=(!!this.getLayoutProperty("horizontal")?t.screenX:t.screenY)-this.dragStart,n=this.widgets[this.dragIndex],r=this.originalSizings[this.dragIndex],i=this.widgets.findIndex((t,e)=>e>this.dragIndex&&t.getLayoutProperty("showRegion"));if(i<0)throw this.cancelResize(),Error("Splitter appeared in the wrong place!");const o=this.widgets[i],a=this.originalSizings[i];if("fixedSize.px"===r[0]&&r[0]===a[0]){const t=s.MathTools.Clamp(r[1]+a[1]-50,0,1/0),i=s.MathTools.Clamp(r[1]+e,50,t),A=s.MathTools.Clamp(a[1]-e,50,t);return l.FixedSize.set(n,i),void l.FixedSize.set(o,A)}const A=this.originalSizings.reduce((t,e)=>"fixedSize.px"===e[0]?t+e[1]:t,0)+4*(this.originalSizings.length-1),c=(this.getMaxSizeConstraint()-A)/this.originalSizings.reduce((t,e)=>"flexSize"===e[0]?t+e[1]:t,0);function u(t,e,n){return s.MathTools.Clamp(t*c+e,50,n)/c}if("flexSize"===r[0]&&r[0]===a[0]){const t=s.MathTools.Clamp(r[1]*c+a[1]*c-50,0,1/0),i=u(r[1],e,t),A=u(a[1],-e,t);return l.FlexSize.set(n,i),void l.FlexSize.set(o,A)}if("flexSize"===r[0]){const t=s.MathTools.Clamp(r[1]*c+a[1]-50,0,1/0),i=u(r[1],e,t),A=s.MathTools.Clamp(a[1]-e,50,t);l.FlexSize.set(n,i),l.FixedSize.set(o,A)}else{const t=s.MathTools.Clamp(r[1]+a[1]*c-50,0,1/0),i=s.MathTools.Clamp(r[1]+e,50,t),A=u(a[1],-e,t);l.FixedSize.set(n,i),l.FlexSize.set(o,A)}}cancelResize(t){if(t&&(t.preventDefault(),t.stopPropagation()),null==this.dragIndex)return;if(t&&"keyup"===t.type&&"Escape"!==t.key)return;const e=this.widgets[this.dragIndex],n=this.originalSizings[this.dragIndex],r=this.widgets.findIndex((t,e)=>e>this.dragIndex&&t.getLayoutProperty("showRegion")),i=this.widgets[r],o=this.originalSizings[r];"fixedSize.px"===n[0]?l.FixedSize.set(e,n[1]):l.FlexSize.set(e,n[1]),"fixedSize.px"===o[0]?l.FixedSize.set(i,o[1]):l.FlexSize.set(i,o[1]),this.endResize()}}l.evTypes=["keyup","pointerup","contextmenu","pointermove"],e.StackPanelLayoutRegion=l,function(t){t.FlexSize=new A.AttachedProperty({name:"Stretch",changed:(e,n,r)=>{null!=r&&t.FixedSize.set(e,null),e.parentRegion&&(e.parentRegion.fit(),e.parentRegion.update()),e.setStale()},create:()=>1}),t.FixedSize=new A.AttachedProperty({name:"Fixed Size (px)",changed:(e,n,r)=>{null!=r&&t.FlexSize.set(e,null),e.parentRegion&&(e.parentRegion.fit(),e.parentRegion.update()),e.setStale()},create:()=>null}),t.IsExpanded=new A.AttachedProperty({name:"Expanded",changed:(e,n,r)=>{if(n!==r){if(e.parentRegion){const n=e.parentRegion.getLayoutProperty("accordionMode");if(r&&"Single"===n)for(const n of e.parentRegion.widgets)n!==e&&t.IsExpanded.get(n)&&t.IsExpanded.set(n,!1);e.parentRegion.fit(),e.parentRegion.update()}e.setStale()}},create:()=>null})}(l=e.StackPanelLayoutRegion||(e.StackPanelLayoutRegion={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(342);e.Table=r.Table,e.TableHelper=r.TableHelper,e.Row=r.Row,e.JoinType=r.JoinType;var i=n(685);e.ArrowConverter=i.ArrowConverter,n(1289)},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(1331);e.CommManager=r.CommManager;var i=n(1332);e.KernelError=i.KernelError},function(t,e,n){var r=n(1341),i=n(1342),o=n(1343),a=n(1344),s=n(1345);function A(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}A.prototype.clear=r,A.prototype.delete=i,A.prototype.get=o,A.prototype.has=a,A.prototype.set=s,t.exports=A},function(t,e,n){var r=n(692);t.exports=function(t,e){for(var n=t.length;n--;)if(r(t[n][0],e))return n;return-1}},function(t,e,n){var r=n(147)(Object,"create");t.exports=r},function(t,e,n){var r=n(1365);t.exports=function(t,e){var n=t.__data__;return r(e)?n["string"==typeof e?"string":"hash"]:n.map}},function(t,e,n){"use strict";t.exports=function(t){return window&&window.process&&window.process.versions?"[object Object]"===Object.prototype.toString.call(t):"[object Object]"===Object.prototype.toString.call(t)&&Object.getPrototypeOf(t)===Object.prototype}},function(t,e,n){"use strict";var r=n(240),i=Array.isArray;function o(t,e,n,a){var s,A,l,c,u,g,h=t[0],d=t.length;if(2===d&&i(h)&&i(t[1])&&0===h.length){if(function(t,e){var n,r;for(n=0;n<t.length;n++){if(null!==(r=t[n])&&"object"==typeof r)return!1;void 0!==r&&(e[n]=r)}return!0}(t[1],h))return h;h.splice(0,h.length)}for(var f=1;f<d;f++)for(A in s=t[f])l=h[A],c=s[A],a&&i(c)?h[A]=c:e&&c&&(r(c)||(u=i(c)))?(u?(u=!1,g=l&&i(l)?l:[]):g=l&&r(l)?l:{},h[A]=o([g,c],e,n,a)):(void 0!==c||n)&&(h[A]=c);return h}e.extendFlat=function(){return o(arguments,!1,!1,!1)},e.extendDeep=function(){return o(arguments,!0,!1,!1)},e.extendDeepAll=function(){return o(arguments,!0,!0,!1)},e.extendDeepNoArrays=function(){return o(arguments,!0,!1,!0)}},function(t,e,n){"use strict";n.r(e);var r={};n.r(r),n.d(r,"BACKSPACE",function(){return B}),n.d(r,"TAB",function(){return b}),n.d(r,"ENTER",function(){return Q}),n.d(r,"SHIFT",function(){return w}),n.d(r,"ESCAPE",function(){return x}),n.d(r,"SPACE",function(){return _}),n.d(r,"ARROW_LEFT",function(){return D}),n.d(r,"ARROW_UP",function(){return M}),n.d(r,"ARROW_RIGHT",function(){return k}),n.d(r,"ARROW_DOWN",function(){return S}),n.d(r,"DELETE",function(){return L}),n.d(r,"isKeyboardClick",function(){return R});var i={};n.r(i),n.d(i,"show",function(){return Nt}),n.d(i,"hide",function(){return Ft}),n.d(i,"isOpen",function(){return Tt});var o=n(3),a=9,s=function(){function t(t,e){var n=this;this.container=t,this.className=e,this.isRunning=!1,this.handleKeyDown=function(t){t.which===a&&(n.reset(),n.container.addEventListener("mousedown",n.handleMouseDown))},this.handleMouseDown=function(){n.reset(),n.container.classList.add(n.className),n.container.addEventListener("keydown",n.handleKeyDown)}}return t.prototype.isActive=function(){return this.isRunning},t.prototype.start=function(){this.container.addEventListener("mousedown",this.handleMouseDown),this.isRunning=!0},t.prototype.stop=function(){this.reset(),this.isRunning=!1},t.prototype.reset=function(){this.container.classList.remove(this.className),this.container.removeEventListener("keydown",this.handleKeyDown),this.container.removeEventListener("mousedown",this.handleMouseDown)},t}(),A="undefined"!=typeof document?new s(document.documentElement,o.FOCUS_DISABLED):{isActive:function(){return!0},start:function(){return!0},stop:function(){return!0}},l={alwaysShowFocus:function(){return A.stop()},isActive:function(){return A.isActive()},onlyShowFocusOnTabs:function(){return A.start()}},c=n(2),u=n(0),g=n(8),h=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.timeoutIds=[],r.clearTimeouts=function(){if(r.timeoutIds.length>0){for(var t=0,e=r.timeoutIds;t<e.length;t++){var n=e[t];window.clearTimeout(n)}r.timeoutIds=[]}},Object(g.isNodeEnv)("production")||r.validateProps(r.props),r}return c.__extends(e,t),e.prototype.componentWillReceiveProps=function(t){Object(g.isNodeEnv)("production")||this.validateProps(t)},e.prototype.componentWillUnmount=function(){this.clearTimeouts()},e.prototype.setTimeout=function(t,e){var n=window.setTimeout(t,e);return this.timeoutIds.push(n),function(){return window.clearTimeout(n)}},e.prototype.validateProps=function(t){},e}(u.Component),d=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.timeoutIds=[],r.clearTimeouts=function(){if(r.timeoutIds.length>0){for(var t=0,e=r.timeoutIds;t<e.length;t++){var n=e[t];window.clearTimeout(n)}r.timeoutIds=[]}},Object(g.isNodeEnv)("production")||r.validateProps(r.props),r}return c.__extends(e,t),e.prototype.componentWillReceiveProps=function(t){Object(g.isNodeEnv)("production")||this.validateProps(t)},e.prototype.componentWillUnmount=function(){this.clearTimeouts()},e.prototype.setTimeout=function(t,e){var n=window.setTimeout(t,e);return this.timeoutIds.push(n),function(){return window.clearTimeout(n)}},e.prototype.validateProps=function(t){},e}(u.PureComponent),f=n(128),p={START:"start",END:"end"},I={BLACK:"#10161A",BLUE1:"#0E5A8A",BLUE2:"#106BA3",BLUE3:"#137CBD",BLUE4:"#2B95D6",BLUE5:"#48AFF0",COBALT1:"#1F4B99",COBALT2:"#2458B3",COBALT3:"#2965CC",COBALT4:"#4580E6",COBALT5:"#669EFF",DARK_GRAY1:"#182026",DARK_GRAY2:"#202B33",DARK_GRAY3:"#293742",DARK_GRAY4:"#30404D",DARK_GRAY5:"#394B59",FOREST1:"#1D7324",FOREST2:"#238C2C",FOREST3:"#29A634",FOREST4:"#43BF4D",FOREST5:"#62D96B",GOLD1:"#A67908",GOLD2:"#BF8C0A",GOLD3:"#D99E0B",GOLD4:"#F2B824",GOLD5:"#FFC940",GRAY1:"#5C7080",GRAY2:"#738694",GRAY3:"#8A9BA8",GRAY4:"#A7B6C2",GRAY5:"#BFCCD6",GREEN1:"#0A6640",GREEN2:"#0D8050",GREEN3:"#0F9960",GREEN4:"#15B371",GREEN5:"#3DCC91",INDIGO1:"#5642A6",INDIGO2:"#634DBF",INDIGO3:"#7157D9",INDIGO4:"#9179F2",INDIGO5:"#AD99FF",LIGHT_GRAY1:"#CED9E0",LIGHT_GRAY2:"#D8E1E8",LIGHT_GRAY3:"#E1E8ED",LIGHT_GRAY4:"#EBF1F5",LIGHT_GRAY5:"#F5F8FA",LIME1:"#728C23",LIME2:"#87A629",LIME3:"#9BBF30",LIME4:"#B6D94C",LIME5:"#D1F26D",ORANGE1:"#A66321",ORANGE2:"#BF7326",ORANGE3:"#D9822B",ORANGE4:"#F29D49",ORANGE5:"#FFB366",RED1:"#A82A2A",RED2:"#C23030",RED3:"#DB3737",RED4:"#F55656",RED5:"#FF7373",ROSE1:"#A82255",ROSE2:"#C22762",ROSE3:"#DB2C6F",ROSE4:"#F5498B",ROSE5:"#FF66A1",SEPIA1:"#63411E",SEPIA2:"#7D5125",SEPIA3:"#96622D",SEPIA4:"#B07B46",SEPIA5:"#C99765",TURQUOISE1:"#008075",TURQUOISE2:"#00998C",TURQUOISE3:"#00B3A4",TURQUOISE4:"#14CCBD",TURQUOISE5:"#2EE6D6",VERMILION1:"#9E2B0E",VERMILION2:"#B83211",VERMILION3:"#D13913",VERMILION4:"#EB532D",VERMILION5:"#FF6E4A",VIOLET1:"#5C255C",VIOLET2:"#752F75",VIOLET3:"#8F398F",VIOLET4:"#A854A8",VIOLET5:"#C274C2",WHITE:"#FFFFFF"},C=n(94),m=n(74),y=n(34),E=["active","alignText","containerRef","elementRef","fill","icon","inputRef","intent","inline","large","loading","leftIcon","minimal","onChildrenMount","onRemove","popoverProps","rightElement","rightIcon","round","small","text"];function v(t,e,n){return void 0===e&&(e=E),void 0===n&&(n=!1),n&&(e=e.concat(E)),e.reduce(function(t,e){return t.hasOwnProperty(e)&&delete t[e],t},c.__assign({},t))}var B=8,b=9,Q=13,w=16,x=27,_=32,D=37,M=38,k=39,S=40,L=46;function R(t){return t===Q||t===_}var N=n(4),F=n.n(N),T=n(16),U=n(724),O=n.n(U),G=n(103),H=n.n(G),j=n(155),Y=n.n(j),P=n(33),K=n.n(P),z=n(52),J=n.n(z),q=n(389),V=n(725),W=n.n(V)()({setReferenceNode:void 0,referenceNode:void 0}),X=function(t){function e(){var e;return e=t.call(this)||this,J()(K()(K()(e)),"setReferenceNode",function(t){t&&e.state.context.referenceNode!==t&&e.setState(function(e){var n=e.context;return{context:H()({},n,{referenceNode:t})}})}),e.state={context:{setReferenceNode:e.setReferenceNode,referenceNode:void 0}},e}return Y()(e,t),e.prototype.render=function(){return u.createElement(W.Provider,{value:this.state.context},this.props.children)},e}(u.Component),Z=function(t){return Array.isArray(t)?t[0]:t},$=function(t){if("function"==typeof t){for(var e=arguments.length,n=new Array(e>1?e-1:0),r=1;r<e;r++)n[r-1]=arguments[r];return t.apply(void 0,n)}},tt={position:"absolute",top:0,left:0,opacity:0,pointerEvents:"none"},et={},nt=function(t){function e(){for(var e,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return e=t.call.apply(t,[this].concat(r))||this,J()(K()(K()(e)),"state",{data:void 0,placement:void 0}),J()(K()(K()(e)),"popperInstance",void 0),J()(K()(K()(e)),"popperNode",null),J()(K()(K()(e)),"arrowNode",null),J()(K()(K()(e)),"setPopperNode",function(t){t&&e.popperNode!==t&&($(e.props.innerRef,t),e.popperNode=t,e.updatePopperInstance())}),J()(K()(K()(e)),"setArrowNode",function(t){e.arrowNode=t}),J()(K()(K()(e)),"updateStateModifier",{enabled:!0,order:900,fn:function(t){var n=t.placement;return e.setState({data:t,placement:n}),t}}),J()(K()(K()(e)),"getOptions",function(){return{placement:e.props.placement,eventsEnabled:e.props.eventsEnabled,positionFixed:e.props.positionFixed,modifiers:H()({},e.props.modifiers,{arrow:H()({},e.props.modifiers&&e.props.modifiers.arrow,{enabled:!!e.arrowNode,element:e.arrowNode}),applyStyle:{enabled:!1},updateStateModifier:e.updateStateModifier})}}),J()(K()(K()(e)),"getPopperStyle",function(){return e.popperNode&&e.state.data?H()({position:e.state.data.offsets.popper.position},e.state.data.styles):tt}),J()(K()(K()(e)),"getPopperPlacement",function(){return e.state.data?e.state.placement:void 0}),J()(K()(K()(e)),"getArrowStyle",function(){return e.arrowNode&&e.state.data?e.state.data.arrowStyles:et}),J()(K()(K()(e)),"getOutOfBoundariesState",function(){return e.state.data?e.state.data.hide:void 0}),J()(K()(K()(e)),"destroyPopperInstance",function(){e.popperInstance&&(e.popperInstance.destroy(),e.popperInstance=null)}),J()(K()(K()(e)),"updatePopperInstance",function(){e.destroyPopperInstance();var t=K()(K()(e)).popperNode,n=e.props.referenceElement;n&&t&&(e.popperInstance=new q.a(n,t,e.getOptions()))}),J()(K()(K()(e)),"scheduleUpdate",function(){e.popperInstance&&e.popperInstance.scheduleUpdate()}),e}Y()(e,t);var n=e.prototype;return n.componentDidUpdate=function(t,e){this.props.placement!==t.placement||this.props.referenceElement!==t.referenceElement||this.props.positionFixed!==t.positionFixed?this.updatePopperInstance():this.props.eventsEnabled!==t.eventsEnabled&&this.popperInstance&&(this.props.eventsEnabled?this.popperInstance.enableEventListeners():this.popperInstance.disableEventListeners()),e.placement!==this.state.placement&&this.scheduleUpdate()},n.componentWillUnmount=function(){$(this.props.innerRef,null),this.destroyPopperInstance()},n.render=function(){return Z(this.props.children)({ref:this.setPopperNode,style:this.getPopperStyle(),placement:this.getPopperPlacement(),outOfBoundaries:this.getOutOfBoundariesState(),scheduleUpdate:this.scheduleUpdate,arrowProps:{ref:this.setArrowNode,style:this.getArrowStyle()}})},e}(u.Component);J()(nt,"defaultProps",{placement:"bottom",eventsEnabled:!0,referenceElement:void 0,positionFixed:!1});q.a.placements;function rt(t){var e=t.referenceElement,n=O()(t,["referenceElement"]);return u.createElement(W.Consumer,null,function(t){var r=t.referenceNode;return u.createElement(nt,H()({referenceElement:void 0!==e?e:r},n))})}var it=n(80),ot=n.n(it),at=function(t){function e(){for(var e,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return e=t.call.apply(t,[this].concat(r))||this,J()(K()(K()(e)),"refHandler",function(t){$(e.props.innerRef,t),$(e.props.setReferenceNode,t)}),e}return Y()(e,t),e.prototype.render=function(){return ot()(Boolean(this.props.setReferenceNode),"`Reference` should not be used outside of a `Manager` component."),Z(this.props.children)({ref:this.refHandler})},e}(u.Component);function st(t){return u.createElement(W.Consumer,null,function(e){var n=e.setReferenceNode;return u.createElement(at,H()({setReferenceNode:n},t))})}var At=n(23),lt=n(621),ct=n(133),ut=!Object(g.isFunction)(T.createPortal),gt={blueprintPortalClassName:function(t,e){if(null!=t[e]&&"string"!=typeof t[e])return new Error(At.H)}},ht=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={hasMounted:!1},e}return c.__extends(e,t),e.prototype.render=function(){return ut||"undefined"==typeof document||!this.state.hasMounted?null:T.createPortal(this.props.children,this.portalElement)},e.prototype.componentDidMount=function(){this.props.container&&(this.portalElement=this.createContainerElement(),this.props.container.appendChild(this.portalElement),this.setState({hasMounted:!0},this.props.onChildrenMount),ut&&this.unstableRenderNoPortal())},e.prototype.componentDidUpdate=function(t){null!=this.portalElement&&t.className!==this.props.className&&(this.portalElement.classList.remove(t.className),dt(this.portalElement.classList,this.props.className)),ut&&this.unstableRenderNoPortal()},e.prototype.componentWillUnmount=function(){null!=this.portalElement&&(ut&&T.unmountComponentAtNode(this.portalElement),this.portalElement.remove())},e.prototype.createContainerElement=function(){var t=document.createElement("div");return t.classList.add(o.PORTAL),dt(t.classList,this.props.className),null!=this.context&&dt(t.classList,this.context.blueprintPortalClassName),t},e.prototype.unstableRenderNoPortal=function(){T.unstable_renderSubtreeIntoContainer(this,u.createElement("div",null,this.props.children),this.portalElement)},e.displayName="Blueprint3.Portal",e.contextTypes=gt,e.defaultProps={container:"undefined"!=typeof document?document.body:null},e}(u.Component);function dt(t,e){null!=e&&""!==e&&t.add.apply(t,e.split(" "))}var ft=function(t){function e(n,r){var i=t.call(this,n,r)||this;return i.refHandlers={container:function(t){return i.containerElement=Object(T.findDOMNode)(t)}},i.maybeRenderChild=function(t){if(null==t)return null;var e="object"==typeof t?u.cloneElement(t,{className:F()(t.props.className,o.OVERLAY_CONTENT),tabIndex:0}):u.createElement("span",{className:o.OVERLAY_CONTENT},t),n=i.props,r=n.onOpening,a=n.onOpened,s=n.onClosing,A=n.onClosed,l=n.transitionDuration,c=n.transitionName;return u.createElement(ct.CSSTransition,{classNames:c,onEntering:r,onEntered:a,onExiting:s,onExited:A,timeout:l},e)},i.handleBackdropMouseDown=function(t){var e=i.props,n=e.backdropProps,r=e.canOutsideClickClose,o=e.enforceFocus,a=e.onClose;r&&Object(g.safeInvoke)(a,t),o&&i.bringFocusInsideOverlay(),Object(g.safeInvoke)(n.onMouseDown,t)},i.handleDocumentClick=function(t){var n=i.props,r=n.canOutsideClickClose,o=n.isOpen,a=n.onClose,s=t.target,A=e.openStack.indexOf(i),l=e.openStack.slice(A).some(function(t){var e=t.containerElement;return e&&e.contains(s)&&!e.isSameNode(s)});o&&r&&!l&&Object(g.safeInvoke)(a,t)},i.handleDocumentFocus=function(t){i.props.enforceFocus&&null!=i.containerElement&&!i.containerElement.contains(t.target)&&(t.preventDefault(),t.stopImmediatePropagation(),i.bringFocusInsideOverlay())},i.handleKeyDown=function(t){var e=i.props,n=e.canEscapeKeyClose,r=e.onClose;t.which===x&&n&&(Object(g.safeInvoke)(r,t),t.preventDefault())},i.state={hasEverOpened:n.isOpen},i}return c.__extends(e,t),e.prototype.render=function(){if(this.props.lazy&&!this.state.hasEverOpened)return null;var t=this.props,e=t.children,n=t.className,r=t.usePortal,i=t.isOpen,a=i?u.Children.map(e,this.maybeRenderChild):[];a.unshift(this.maybeRenderBackdrop());var s,A=F()(o.OVERLAY,((s={})[o.OVERLAY_OPEN]=i,s[o.OVERLAY_INLINE]=!r,s),n),l=u.createElement(ct.TransitionGroup,{appear:!0,className:A,component:"div",onKeyDown:this.handleKeyDown,ref:this.refHandlers.container},a);return r?u.createElement(ht,{className:this.props.portalClassName,container:this.props.portalContainer},l):l},e.prototype.componentDidMount=function(){this.props.isOpen&&this.overlayWillOpen()},e.prototype.componentWillReceiveProps=function(t){this.setState({hasEverOpened:this.state.hasEverOpened||t.isOpen})},e.prototype.componentDidUpdate=function(t){t.isOpen&&!this.props.isOpen?this.overlayWillClose():!t.isOpen&&this.props.isOpen&&this.overlayWillOpen()},e.prototype.componentWillUnmount=function(){this.overlayWillClose()},e.prototype.bringFocusInsideOverlay=function(){var t=this;return requestAnimationFrame(function(){if(null!=t.containerElement&&null!=document.activeElement&&t.props.isOpen&&!t.containerElement.contains(document.activeElement)){var e=t.containerElement.querySelector("[autofocus]"),n=t.containerElement.querySelector("[tabindex]");null!=e?e.focus():null!=n&&n.focus()}})},e.prototype.maybeRenderBackdrop=function(){var t=this.props,e=t.backdropClassName,n=t.backdropProps,r=t.hasBackdrop,i=t.isOpen,a=t.transitionDuration,s=t.transitionName;return r&&i?u.createElement(ct.CSSTransition,{classNames:s,key:"__backdrop",timeout:a},u.createElement("div",c.__assign({},n,{className:F()(o.OVERLAY_BACKDROP,e,n.className),onMouseDown:this.handleBackdropMouseDown,tabIndex:this.props.canOutsideClickClose?0:null}))):null},e.prototype.overlayWillClose=function(){document.removeEventListener("focus",this.handleDocumentFocus,!0),document.removeEventListener("mousedown",this.handleDocumentClick);var t=e.openStack,n=t.indexOf(this);if(-1!==n){if(t.splice(n,1),t.length>0){var r=e.getLastOpened();r.props.enforceFocus&&document.addEventListener("focus",r.handleDocumentFocus,!0)}0===t.filter(function(t){return t.props.usePortal&&t.props.hasBackdrop}).length&&document.body.classList.remove(o.OVERLAY_OPEN)}},e.prototype.overlayWillOpen=function(){var t=e.openStack;t.length>0&&document.removeEventListener("focus",e.getLastOpened().handleDocumentFocus,!0),t.push(this),this.props.autoFocus&&this.bringFocusInsideOverlay(),this.props.enforceFocus&&document.addEventListener("focus",this.handleDocumentFocus,!0),this.props.canOutsideClickClose&&!this.props.hasBackdrop&&document.addEventListener("mousedown",this.handleDocumentClick),this.props.hasBackdrop&&this.props.usePortal&&document.body.classList.add(o.OVERLAY_OPEN)},e.displayName="Blueprint3.Overlay",e.defaultProps={autoFocus:!0,backdropProps:{},canEscapeKeyClose:!0,canOutsideClickClose:!0,enforceFocus:!0,hasBackdrop:!0,isOpen:!1,lazy:!0,transitionDuration:300,transitionName:o.OVERLAY,usePortal:!0},e.openStack=[],e.getLastOpened=function(){return e.openStack[e.openStack.length-1]},e}(u.PureComponent),pt=n(257),It=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.element=null,e.observer=new pt.a(function(t){return Object(g.safeInvoke)(e.props.onResize,t)}),e}return c.__extends(e,t),e.prototype.render=function(){return u.Children.only(this.props.children)},e.prototype.componentDidMount=function(){this.observeElement()},e.prototype.componentDidUpdate=function(t){this.observeElement(this.props.observeParents!==t.observeParents)},e.prototype.componentWillUnmount=function(){this.observer.disconnect()},e.prototype.observeElement=function(t){void 0===t&&(t=!1);var e=this.getElement();if(e instanceof Element){if((e!==this.element||t)&&(this.observer.disconnect(),this.element=e,this.observer.observe(e),this.props.observeParents))for(var n=e.parentElement;null!=n;)this.observer.observe(n),n=n.parentElement}else this.observer.disconnect()},e.prototype.getElement=function(){try{return Object(T.findDOMNode)(this)}catch(t){return null}},e.displayName="Blueprint3.ResizeSensor",e}(u.PureComponent),Ct=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.popover=null,e}return c.__extends(e,t),e.prototype.render=function(){var t=this,e=this.props,n=e.children,r=e.intent,i=e.popoverClassName,a=c.__rest(e,["children","intent","popoverClassName"]),s=F()(o.TOOLTIP,o.intentClass(r),i);return u.createElement(kt,c.__assign({interactionKind:Mt.HOVER_TARGET_ONLY},a,{autoFocus:!1,canEscapeKeyClose:!1,enforceFocus:!1,lazy:!0,popoverClassName:s,portalContainer:this.props.portalContainer,ref:function(e){return t.popover=e}}),n)},e.prototype.reposition=function(){null!=this.popover&&this.popover.reposition()},e.displayName="Blueprint3.Tooltip",e.defaultProps={hoverCloseDelay:0,hoverOpenDelay:100,transitionDuration:100},e}(u.PureComponent);function mt(t){return t.split("-")[0]}function yt(t){return-1!==["left","right"].indexOf(t)}function Et(t){switch(t){case"top":return"bottom";case"left":return"right";case"bottom":return"top";default:return"left"}}function vt(t){switch(t.split("-")[1]){case"start":return"left";case"end":return"right";default:return"center"}}function Bt(t){var e=mt(t.placement);if(null==t.arrowElement)return yt(e)?Et(e)+" "+vt(e):vt(e)+" "+Et(e);var n=t.arrowElement.clientHeight/2,r=t.offsets.arrow;return yt(e)?Et(e)+" "+(r.top+n)+"px":r.left+n+"px "+Et(e)}var bt=function(t){if(null==t.arrowElement)return t;var e=t.arrowElement.clientWidth,n=mt(t.placement),r=yt(n),i=r?"width":"height",o=r?"left":"top",a=Math.round(e/2/Math.sqrt(2));return"top"===n||"left"===n?(t.offsets.popper[o]-=a+4,t.offsets.arrow[o]=t.offsets.popper[i]-e+a):(t.offsets.popper[o]+=a+4,t.offsets.arrow[o]=-a),t};function Qt(t){if(null==t)return 0;switch(mt(t)){case"top":return-90;case"left":return 180;case"bottom":return 90;default:return 0}}var wt=function(t){var e=t.arrowProps,n=e.ref,r=e.style,i=t.placement;return u.createElement("div",{className:o.POPOVER_ARROW,ref:n,style:isNaN(+r.left)?{}:r},u.createElement("svg",{viewBox:"0 0 30 30",style:{transform:"rotate("+Qt(i)+"deg)"}},u.createElement("path",{className:o.POPOVER_ARROW+"-border",d:"M8.11 6.302c1.015-.936 1.887-2.922 1.887-4.297v26c0-1.378-.868-3.357-1.888-4.297L.925 17.09c-1.237-1.14-1.233-3.034 0-4.17L8.11 6.302z"}),u.createElement("path",{className:o.POPOVER_ARROW+"-fill",d:"M8.787 7.036c1.22-1.125 2.21-3.376 2.21-5.03V0v30-2.005c0-1.654-.983-3.9-2.21-5.03l-7.183-6.616c-.81-.746-.802-1.96 0-2.7l7.183-6.614z"})))};function xt(t){switch(t){case y.a.TOP_LEFT:return"top-start";case y.a.TOP:return"top";case y.a.TOP_RIGHT:return"top-end";case y.a.RIGHT_TOP:return"right-start";case y.a.RIGHT:return"right";case y.a.RIGHT_BOTTOM:return"right-end";case y.a.BOTTOM_RIGHT:return"bottom-end";case y.a.BOTTOM:return"bottom";case y.a.BOTTOM_LEFT:return"bottom-start";case y.a.LEFT_BOTTOM:return"left-end";case y.a.LEFT:return"left";case y.a.LEFT_TOP:return"left-start";case"auto":case"auto-start":case"auto-end":return t;default:return function(t){throw new Error("Unexpected position: "+t)}(t)}}wt.displayName="Blueprint3.PopoverArrow";var _t,Dt,Mt={CLICK:"click",CLICK_TARGET_ONLY:"click-target",HOVER:"hover",HOVER_TARGET_ONLY:"hover-target"},kt=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={hasDarkParent:!1,isOpen:e.getIsOpen(e.props),transformOrigin:""},e.isMouseInTargetOrPopover=!1,e.lostFocusOnSamePage=!0,e.refHandlers={popover:function(t){e.popoverElement=t,g.safeInvoke(e.props.popoverRef,t)},target:function(t){return e.targetElement=t}},e.reposition=function(){return g.safeInvoke(e.popperScheduleUpdate)},e.renderPopover=function(t){var n=e.props,r=n.usePortal,i=n.interactionKind,a=e.state.transformOrigin;e.popperScheduleUpdate=t.scheduleUpdate;var s={onClick:e.handlePopoverClick};(i===Mt.HOVER||!r&&i===Mt.HOVER_TARGET_ONLY)&&(s.onMouseEnter=e.handleMouseEnter,s.onMouseLeave=e.handleMouseLeave);var A,l=F()(o.POPOVER,((A={})[o.DARK]=e.props.inheritDarkTheme&&e.state.hasDarkParent,A[o.MINIMAL]=e.props.minimal,A),e.props.popoverClassName);return u.createElement("div",{className:o.TRANSITION_CONTAINER,ref:t.ref,style:t.style},u.createElement(It,{onResize:e.reposition},u.createElement("div",c.__assign({className:l,style:{transformOrigin:a}},s),e.isArrowEnabled()&&u.createElement(wt,{arrowProps:t.arrowProps,placement:t.placement}),u.createElement("div",{className:o.POPOVER_CONTENT},e.understandChildren().content))))},e.renderTarget=function(t){var n=e.props,r=n.fill,i=n.openOnTargetFocus,a=n.targetClassName,s=n.targetProps,A=void 0===s?{}:s,l=e.state.isOpen,h=e.isHoverInteractionKind(),d=e.props.targetTagName;r&&(d="div");var f=h?{onBlur:e.handleTargetBlur,onFocus:e.handleTargetFocus,onMouseEnter:e.handleMouseEnter,onMouseLeave:e.handleMouseLeave}:{onClick:e.handleTargetClick};f.className=F()(o.POPOVER_TARGET,((p={})[o.POPOVER_OPEN]=l,p),A.className,a),f.ref=t.ref;var p,I,C=g.ensureElement(e.understandChildren().target),m=C.props.tabIndex,y=null==m&&i&&h?0:m,E=u.cloneElement(C,{className:F()(C.props.className,(I={},I[o.ACTIVE]=l&&!h,I)),disabled:!(!l||!g.isElementOfType(C,Ct))||C.props.disabled,tabIndex:y});return u.createElement(It,{onResize:e.reposition},u.createElement(d,c.__assign({},A,f),E))},e.handleTargetFocus=function(t){if(e.props.openOnTargetFocus&&e.isHoverInteractionKind()){if(null==t.relatedTarget&&!e.lostFocusOnSamePage)return;e.handleMouseEnter(t)}lt.a(e.props.targetProps,"onFocus",t)},e.handleTargetBlur=function(t){e.props.openOnTargetFocus&&e.isHoverInteractionKind()&&(e.isElementInPopover(t.relatedTarget)||e.handleMouseLeave(t)),e.lostFocusOnSamePage=null!=t.relatedTarget,lt.a(e.props.targetProps,"onBlur",t)},e.handleMouseEnter=function(t){e.isMouseInTargetOrPopover=!0,e.props.usePortal||!e.isElementInPopover(t.target)||e.props.interactionKind!==Mt.HOVER_TARGET_ONLY||e.props.openOnTargetFocus?e.props.disabled||e.setOpenState(!0,t,e.props.hoverOpenDelay):e.handleMouseLeave(t),lt.a(e.props.targetProps,"onMouseEnter",t)},e.handleMouseLeave=function(t){e.isMouseInTargetOrPopover=!1,e.setTimeout(function(){e.isMouseInTargetOrPopover||e.setOpenState(!1,t,e.props.hoverCloseDelay)}),lt.a(e.props.targetProps,"onMouseLeave",t)},e.handlePopoverClick=function(t){var n=t.target,r=n.closest("."+o.POPOVER_DISMISS+", ."+o.POPOVER_DISMISS_OVERRIDE),i=null!=r&&r.classList.contains(o.POPOVER_DISMISS),a=null!=n.closest(":disabled, ."+o.DISABLED);!i||a||t.isDefaultPrevented()||(e.setOpenState(!1,t),e.props.captureDismiss&&t.preventDefault())},e.handleOverlayClose=function(t){var n=t.target;(!g.elementIsOrContains(e.targetElement,n)||t.nativeEvent instanceof KeyboardEvent)&&e.setOpenState(!1,t)},e.handleTargetClick=function(t){e.props.disabled||e.isElementInPopover(t.target)||(null==e.props.isOpen?e.setState(function(t){return{isOpen:!t.isOpen}}):e.setOpenState(!e.props.isOpen,t)),lt.a(e.props.targetProps,"onClick",t)},e.updatePopoverState=function(t){return e.setState({transformOrigin:Bt(t)}),t},e}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.className,n=t.disabled,r=t.fill,i=this.state.isOpen,a=this.props.wrapperTagName;r&&(a="div");var s=null==g.ensureElement(this.understandChildren().content);!s||n||!1===i||g.isNodeEnv("production")||console.warn(At.D);var A,l=F()(o.POPOVER_WRAPPER,e,((A={})[o.FILL]=r,A));return u.createElement(X,null,u.createElement(a,{className:l},u.createElement(st,{innerRef:this.refHandlers.target},this.renderTarget),u.createElement(ft,{autoFocus:this.props.autoFocus,backdropClassName:o.POPOVER_BACKDROP,backdropProps:this.props.backdropProps,canEscapeKeyClose:this.props.canEscapeKeyClose,canOutsideClickClose:this.props.interactionKind===Mt.CLICK,className:this.props.portalClassName,enforceFocus:this.props.enforceFocus,hasBackdrop:this.props.hasBackdrop,isOpen:i&&!s,onClose:this.handleOverlayClose,onClosed:this.props.onClosed,onClosing:this.props.onClosing,onOpened:this.props.onOpened,onOpening:this.props.onOpening,transitionDuration:this.props.transitionDuration,transitionName:o.POPOVER,usePortal:this.props.usePortal,portalContainer:this.props.portalContainer},u.createElement(rt,{innerRef:this.refHandlers.popover,placement:xt(this.props.position),modifiers:this.getPopperModifiers()},this.renderPopover))))},e.prototype.componentDidMount=function(){this.updateDarkParent()},e.prototype.componentWillReceiveProps=function(e){t.prototype.componentWillReceiveProps.call(this,e);var n=this.getIsOpen(e);null!=e.isOpen&&n!==this.state.isOpen?(this.setOpenState(n),this.setState({isOpen:n})):this.state.isOpen&&null==e.isOpen&&e.disabled&&this.setOpenState(!1)},e.prototype.componentDidUpdate=function(){this.updateDarkParent()},e.prototype.validateProps=function(t){if(null==t.isOpen&&null!=t.onInteraction&&console.warn(At.G),t.hasBackdrop&&!t.usePortal&&console.warn(At.E),t.hasBackdrop&&t.interactionKind!==Mt.CLICK)throw new Error(At.z);var e=u.Children.count(t.children),n=void 0!==t.content,r=void 0!==t.target;if(0===e&&!r)throw new Error(At.A);e>2&&console.warn(At.F),e>0&&r&&console.warn(At.C),2===e&&n&&console.warn(At.B)},e.prototype.updateDarkParent=function(){if(this.props.usePortal&&this.state.isOpen){var t=null!=this.targetElement&&null!=this.targetElement.closest("."+o.DARK);this.setState({hasDarkParent:t})}},e.prototype.understandChildren=function(){var t=this.props,e=t.children,n=t.content,r=t.target,i=u.Children.toArray(e),o=i[0],a=i[1];return{content:null==a?n:a,target:null==o?r:o}},e.prototype.getIsOpen=function(t){return!t.disabled&&(null!=t.isOpen?t.isOpen:t.defaultIsOpen)},e.prototype.getPopperModifiers=function(){var t=this.props,e=t.boundary,n=t.modifiers,r=n.flip,i=void 0===r?{}:r,o=n.preventOverflow,a=void 0===o?{}:o;return c.__assign({},n,{arrowOffset:{enabled:this.isArrowEnabled(),fn:bt,order:510},flip:c.__assign({boundariesElement:e},i),preventOverflow:c.__assign({boundariesElement:e},a),updatePopoverState:{enabled:!0,fn:this.updatePopoverState,order:900}})},e.prototype.setOpenState=function(t,e,n){var r=this;g.safeInvoke(this.cancelOpenTimeout),n>0?this.cancelOpenTimeout=this.setTimeout(function(){return r.setOpenState(t,e)},n):(null==this.props.isOpen?this.setState({isOpen:t}):g.safeInvoke(this.props.onInteraction,t,e),t||g.safeInvoke(this.props.onClose,e))},e.prototype.isArrowEnabled=function(){var t=this.props,e=t.minimal,n=t.modifiers.arrow;return!e&&(null==n||n.enabled)},e.prototype.isElementInPopover=function(t){return null!=this.popoverElement&&this.popoverElement.contains(t)},e.prototype.isHoverInteractionKind=function(){return this.props.interactionKind===Mt.HOVER||this.props.interactionKind===Mt.HOVER_TARGET_ONLY},e.displayName="Blueprint3.Popover",e.defaultProps={boundary:"scrollParent",captureDismiss:!1,defaultIsOpen:!1,disabled:!1,fill:!1,hasBackdrop:!1,hoverCloseDelay:300,hoverOpenDelay:150,inheritDarkTheme:!0,interactionKind:Mt.CLICK,minimal:!1,modifiers:{},openOnTargetFocus:!0,position:"auto",targetTagName:"span",transitionDuration:300,usePortal:!0,wrapperTagName:"span"},e}(d),St={preventOverflow:{boundariesElement:"viewport"}},Lt=100,Rt=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={isDarkTheme:!1,isOpen:!1,menu:null,offset:null},e.cancelContextMenu=function(t){return t.preventDefault()},e.handleBackdropContextMenu=function(t){t.persist(),t.preventDefault(),e.setTimeout(function(){document.elementFromPoint(t.clientX,t.clientY).dispatchEvent(new MouseEvent("contextmenu",t))},Lt)},e.handlePopoverInteraction=function(t){t||requestAnimationFrame(function(){return e.hide()})},e}return c.__extends(e,t),e.prototype.render=function(){var t,e=u.createElement("div",{onContextMenu:this.cancelContextMenu},this.state.menu),n=F()(((t={})[o.DARK]=this.state.isDarkTheme,t)),r=null==this.state.offset?"":this.state.offset.left+"x"+this.state.offset.top;return u.createElement("div",{className:o.CONTEXT_MENU_POPOVER_TARGET,style:this.state.offset},u.createElement(kt,c.__assign({},this.props,{backdropProps:{onContextMenu:this.handleBackdropContextMenu},content:e,enforceFocus:!1,key:r,hasBackdrop:!0,isOpen:this.state.isOpen,minimal:!0,modifiers:St,onInteraction:this.handlePopoverInteraction,position:y.a.RIGHT_TOP,popoverClassName:n,target:u.createElement("div",null),transitionDuration:Lt})))},e.prototype.show=function(t,e,n,r){this.setState({isOpen:!0,menu:t,offset:e,onClose:n,isDarkTheme:r})},e.prototype.hide=function(){Object(g.safeInvoke)(this.state.onClose),this.setState({isOpen:!1,onClose:void 0})},e}(d);function Nt(t,e,n,r){null==_t&&((_t=document.createElement("div")).classList.add(o.CONTEXT_MENU),document.body.appendChild(_t),Dt=T.render(u.createElement(Rt,{onClosed:Ut}),_t)),Dt.show(t,e,n,r)}function Ft(){null!=Dt&&Dt.hide()}function Tt(){return null!=Dt&&Dt.state.isOpen}function Ut(){null!=_t&&(T.unmountComponentAtNode(_t),_t.remove(),_t=null,Dt=null)}var Ot=n(307),Gt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props.icon;if(null==t||"boolean"==typeof t)return null;if("string"!=typeof t)return t;var n=this.props,r=n.className,i=n.color,a=n.htmlTitle,s=n.iconSize,A=void 0===s?e.SIZE_STANDARD:s,l=n.intent,g=n.title,h=void 0===g?t:g,d=n.tagName,f=void 0===d?"span":d,p=c.__rest(n,["className","color","htmlTitle","iconSize","intent","title","tagName"]),I=A>=e.SIZE_LARGE?e.SIZE_LARGE:e.SIZE_STANDARD,C=this.renderSvgPaths(I,t),m=F()(o.ICON,o.iconClass(t),o.intentClass(l),r),y="0 0 "+I+" "+I;return u.createElement(f,c.__assign({},p,{className:m,title:a}),u.createElement("svg",{fill:i,"data-icon":t,width:A,height:A,viewBox:y},h&&u.createElement("desc",null,h),C))},e.prototype.renderSvgPaths=function(t,n){var r=(t===e.SIZE_STANDARD?Ot.a:Ot.b)[n];return null==r?null:r.map(function(t,e){return u.createElement("path",{key:e,d:t,fillRule:"evenodd"})})},e.displayName="Blueprint3.Icon",e.SIZE_STANDARD=16,e.SIZE_LARGE=20,e}(u.PureComponent),Ht=45,jt="M 50,50 m 0,-45 a 45,45 0 1 1 0,90 a 45,45 0 1 1 0,-90",Yt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.componentDidUpdate=function(t){t.value!==this.props.value&&this.forceUpdate()},e.prototype.render=function(){var t,n=this.props,r=n.className,i=n.intent,a=n.value,s=n.tagName,A=void 0===s?"div":s,l=this.getSize(),c=F()(o.SPINNER,o.intentClass(i),((t={})[o.SPINNER_NO_SPIN]=null!=a,t),r),h=Math.min(16,4*e.SIZE_LARGE/l),d=280-280*(null==a?.25:Object(g.clamp)(a,0,1));return u.createElement(A,{className:c},u.createElement(A,{className:o.SPINNER_ANIMATION},u.createElement("svg",{width:l,height:l,strokeWidth:h.toFixed(2),viewBox:this.getViewBox(h)},u.createElement("path",{className:o.SPINNER_TRACK,d:jt}),u.createElement("path",{className:o.SPINNER_HEAD,d:jt,pathLength:280,strokeDasharray:"280 280",strokeDashoffset:d}))))},e.prototype.validateProps=function(t){var e=t.className,n=void 0===e?"":e;null!=t.size&&(n.indexOf(o.SMALL)>=0||n.indexOf(o.LARGE)>=0)&&console.warn(At.M)},e.prototype.getSize=function(){var t=this.props,n=t.className,r=void 0===n?"":n,i=t.size;return null==i?r.indexOf(o.SMALL)>=0?e.SIZE_SMALL:r.indexOf(o.LARGE)>=0?e.SIZE_LARGE:e.SIZE_STANDARD:Math.max(10,i)},e.prototype.getViewBox=function(t){var e=Ht+t/2,n=(50-e).toFixed(2),r=(2*e).toFixed(2);return n+" "+n+" "+r+" "+r},e.displayName="Blueprint3.Spinner",e.SIZE_SMALL=20,e.SIZE_STANDARD=50,e.SIZE_LARGE=100,e}(d),Pt=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={isActive:!1},e.refHandlers={button:function(t){e.buttonRef=t,Object(g.safeInvoke)(e.props.elementRef,t)}},e.currentKeyDown=null,e.handleKeyDown=function(t){R(t.which)&&(t.preventDefault(),t.which!==e.currentKeyDown&&e.setState({isActive:!0})),e.currentKeyDown=t.which,Object(g.safeInvoke)(e.props.onKeyDown,t)},e.handleKeyUp=function(t){R(t.which)&&(e.setState({isActive:!1}),e.buttonRef.click()),e.currentKeyDown=null,Object(g.safeInvoke)(e.props.onKeyUp,t)},e}return c.__extends(e,t),e.prototype.getCommonButtonProps=function(){var t,e=this.props,n=e.alignText,r=e.fill,i=e.large,a=e.loading,s=e.minimal,A=e.small,l=e.tabIndex,c=this.props.disabled||a;return{className:F()(o.BUTTON,((t={})[o.ACTIVE]=this.state.isActive||this.props.active,t[o.DISABLED]=c,t[o.FILL]=r,t[o.LARGE]=i,t[o.LOADING]=a,t[o.MINIMAL]=s,t[o.SMALL]=A,t),o.alignmentClass(n),o.intentClass(this.props.intent),this.props.className),disabled:c,onClick:c?void 0:this.props.onClick,onKeyDown:this.handleKeyDown,onKeyUp:this.handleKeyUp,ref:this.refHandlers.button,tabIndex:c?-1:l}},e.prototype.renderChildren=function(){var t=this.props,e=t.children,n=t.icon,r=t.loading,i=t.rightIcon,a=t.text;return[r&&u.createElement(Yt,{key:"loading",className:o.BUTTON_SPINNER,size:Gt.SIZE_LARGE}),u.createElement(Gt,{key:"leftIcon",icon:n}),(!Object(g.isReactNodeEmpty)(a)||!Object(g.isReactNodeEmpty)(e))&&u.createElement("span",{key:"text",className:o.BUTTON_TEXT},a,e),u.createElement(Gt,{key:"rightIcon",icon:i})]},e}(u.PureComponent),Kt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){return u.createElement("button",c.__assign({type:"button"},v(this.props),this.getCommonButtonProps()),this.renderChildren())},e.displayName="Blueprint3.Button",e}(Pt),zt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.href,n=t.tabIndex,r=void 0===n?0:n,i=this.getCommonButtonProps();return u.createElement("a",c.__assign({role:"button"},v(this.props),i,{href:i.disabled?void 0:e,tabIndex:i.disabled?-1:r}),this.renderChildren())},e.displayName="Blueprint3.AnchorButton",e}(Pt);function Jt(t,e){return function(n){var r=n.className,i=n.elementRef,o=c.__rest(n,["className","elementRef"]);return u.createElement(t,c.__assign({},o,{className:F()(e,r),ref:i}))}}var qt,Vt=Jt("h1",o.HEADING),Wt=Jt("h2",o.HEADING),Xt=Jt("h3",o.HEADING),Zt=Jt("h4",o.HEADING),$t=Jt("h5",o.HEADING),te=Jt("h6",o.HEADING),ee=Jt("blockquote",o.BLOCKQUOTE),ne=Jt("code",o.CODE),re=Jt("pre",o.CODE_BLOCK),ie=Jt("label",o.LABEL),oe=Jt("ol",o.LIST),ae=Jt("ul",o.LIST),se=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){return u.createElement(ft,c.__assign({},this.props,{className:o.OVERLAY_SCROLL_CONTAINER,hasBackdrop:!0}),u.createElement("div",{className:o.DIALOG_CONTAINER},u.createElement("div",{className:F()(o.DIALOG,this.props.className),style:this.props.style},this.maybeRenderHeader(),this.props.children)))},e.prototype.validateProps=function(t){null==t.title&&(null!=t.icon&&console.warn(At.i),null!=t.isCloseButtonShown&&console.warn(At.h))},e.prototype.maybeRenderCloseButton=function(){return!1!==this.props.isCloseButtonShown?u.createElement(Kt,{"aria-label":"Close",className:o.DIALOG_CLOSE_BUTTON,icon:u.createElement(Gt,{icon:"small-cross",iconSize:Gt.SIZE_LARGE}),minimal:!0,onClick:this.props.onClose}):void 0},e.prototype.maybeRenderHeader=function(){var t=this.props,e=t.icon,n=t.title;if(null!=n)return u.createElement("div",{className:o.DIALOG_HEADER},u.createElement(Gt,{icon:e,iconSize:Gt.SIZE_LARGE}),u.createElement(Zt,null,n),this.maybeRenderCloseButton())},e.defaultProps={canOutsideClickClose:!0,isOpen:!1},e.displayName="Blueprint3.Dialog",e}(d),Ae=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.handleCancel=function(t){return e.internalHandleCallbacks(!1,t)},e.handleConfirm=function(t){return e.internalHandleCallbacks(!0,t)},e}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.canEscapeKeyCancel,n=t.canOutsideClickCancel,r=t.children,i=t.className,a=t.icon,s=t.intent,A=t.cancelButtonText,l=t.confirmButtonText,g=(t.onClose,c.__rest(t,["canEscapeKeyCancel","canOutsideClickCancel","children","className","icon","intent","cancelButtonText","confirmButtonText","onClose"]));return u.createElement(se,c.__assign({},g,{className:F()(o.ALERT,i),canEscapeKeyClose:e,canOutsideClickClose:n,onClose:this.handleCancel,portalContainer:this.props.portalContainer}),u.createElement("div",{className:o.ALERT_BODY},u.createElement(Gt,{icon:a,iconSize:40,intent:s}),u.createElement("div",{className:o.ALERT_CONTENTS},r)),u.createElement("div",{className:o.ALERT_FOOTER},u.createElement(Kt,{intent:s,text:l,onClick:this.handleConfirm}),A&&u.createElement(Kt,{text:A,onClick:this.handleCancel})))},e.prototype.validateProps=function(t){null==t.onClose&&null==t.cancelButtonText!=(null==t.onCancel)&&console.warn(At.c);var e=null!=t.onCancel||null!=t.onClose;t.canEscapeKeyCancel&&!e&&console.warn(At.a),t.canOutsideClickCancel&&!e&&console.warn(At.b)},e.prototype.internalHandleCallbacks=function(t,e){var n=this.props,r=n.onCancel,i=n.onClose,o=n.onConfirm;Object(g.safeInvoke)(t?o:r,e),Object(g.safeInvoke)(i,t,e)},e.defaultProps={canEscapeKeyCancel:!1,canOutsideClickCancel:!1,confirmButtonText:"OK",isOpen:!1},e.displayName="Blueprint3.Alert",e}(d),le=function(t){var e,n=F()(o.BREADCRUMB,((e={})[o.BREADCRUMB_CURRENT]=t.current,e[o.DISABLED]=t.disabled,e),t.className);return null==t.href&&null==t.onClick?u.createElement("span",{className:n},t.text,t.children):u.createElement("a",{className:n,href:t.href,onClick:t.disabled?null:t.onClick,tabIndex:t.disabled?null:0,target:t.target},t.text,t.children)},ce=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.className,n=t.title;return null==n?u.createElement("li",{className:F()(o.MENU_DIVIDER,e)}):u.createElement("li",{className:F()(o.MENU_HEADER,e)},u.createElement(te,null,n))},e.displayName="Blueprint3.MenuDivider",e}(u.Component),ue=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={isContentOverflowing:!1,textContent:""},e.textRef=null,e}return c.__extends(e,t),e.prototype.componentDidMount=function(){this.update()},e.prototype.componentDidUpdate=function(){this.update()},e.prototype.render=function(){var t,e=this,n=F()(((t={})[o.TEXT_OVERFLOW_ELLIPSIS]=this.props.ellipsize,t),this.props.className),r=this.props.tagName,i=void 0===r?"div":r;return u.createElement(i,{className:n,ref:function(t){return e.textRef=t},title:this.state.isContentOverflowing?this.state.textContent:void 0},this.props.children)},e.prototype.update=function(){if(null!=this.textRef){var t={isContentOverflowing:this.props.ellipsize&&this.textRef.scrollWidth>this.textRef.clientWidth,textContent:this.textRef.textContent};this.setState(t)}},e.displayName="Blueprint3.Text",e}(u.PureComponent),ge=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t,e,n=this.props,r=n.active,i=n.className,a=n.children,s=n.disabled,A=n.icon,l=n.intent,g=n.labelElement,h=n.multiline,d=(n.popoverProps,n.shouldDismissPopover),f=n.text,p=n.textClassName,I=n.tagName,C=void 0===I?"a":I,m=c.__rest(n,["active","className","children","disabled","icon","intent","labelElement","multiline","popoverProps","shouldDismissPopover","text","textClassName","tagName"]),y=null!=a,E=o.intentClass(l),v=F()(o.MENU_ITEM,E,((t={})[o.ACTIVE]=r,t[o.INTENT_PRIMARY]=r&&null==E,t[o.DISABLED]=s,t[o.POPOVER_DISMISS]=d&&!s&&!y,t),i),B=u.createElement(C,c.__assign({},m,s?de:{},{className:v}),u.createElement(Gt,{icon:A}),u.createElement(ue,{className:F()(o.FILL,p),ellipsize:!h},f),this.maybeRenderLabel(g),y&&u.createElement(Gt,{icon:"caret-right"})),b=F()(((e={})[o.MENU_SUBMENU]=y,e));return u.createElement("li",{className:b},this.maybeRenderPopover(B,a))},e.prototype.maybeRenderLabel=function(t){var e=this.props,n=e.label,r=e.labelClassName;return null==n&&null==t?null:u.createElement("span",{className:F()(o.MENU_ITEM_LABEL,r)},n,t)},e.prototype.maybeRenderPopover=function(t,e){if(null==e)return t;var n=this.props,r=n.disabled,i=n.popoverProps;return u.createElement(kt,c.__assign({autoFocus:!1,captureDismiss:!1,disabled:r,enforceFocus:!1,hoverCloseDelay:0,interactionKind:Mt.HOVER,modifiers:he,position:y.a.RIGHT_TOP,usePortal:!1},i,{content:u.createElement(fe,null,e),minimal:!0,popoverClassName:F()(o.MENU_SUBMENU,i.popoverClassName),target:t}))},e.defaultProps={disabled:!1,multiline:!1,popoverProps:{},shouldDismissPopover:!0,text:""},e.displayName="Blueprint3.MenuItem",e}(u.PureComponent),he={flip:{boundariesElement:"viewport",padding:20},offset:{offset:-5},preventOverflow:{boundariesElement:"viewport",padding:20}},de={href:void 0,onClick:void 0,onMouseDown:void 0,onMouseEnter:void 0,onMouseLeave:void 0,tabIndex:-1},fe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.className,r=e.children,i=e.large,a=e.ulRef,s=c.__rest(e,["className","children","large","ulRef"]),A=F()(o.MENU,((t={})[o.LARGE]=i,t),n);return u.createElement("ul",c.__assign({},s,{className:A,ref:a}),r)},e.displayName="Blueprint3.Menu",e.Divider=ce,e.Item=ge,e}(u.Component),pe=n(620);!function(t){t[t.NONE=0]="NONE",t[t.GROW=1]="GROW",t[t.SHRINK=2]="SHRINK"}(qt||(qt={}));var Ie,Ce=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={direction:qt.NONE,lastOverflowCount:0,overflow:[],visible:e.props.items},e.previousWidths=new Map,e.spacer=null,e.resize=function(t){var n=t.some(function(t){var n=e.previousWidths.get(t.target)||0;return t.contentRect.width>n});e.repartition(n),t.forEach(function(t){return e.previousWidths.set(t.target,t.contentRect.width)})},e}return c.__extends(e,t),e.ofType=function(){return e},e.prototype.componentDidMount=function(){this.repartition(!1)},e.prototype.componentWillReceiveProps=function(t){var e=this.props,n=e.collapseFrom,r=e.items,i=e.minVisibleItems,o=e.observeParents,a=e.overflowRenderer,s=e.visibleItemRenderer;o!==t.observeParents&&console.warn(At.w),n===t.collapseFrom&&r===t.items&&i===t.minVisibleItems&&a===t.overflowRenderer&&s===t.visibleItemRenderer||this.setState({direction:qt.GROW,lastOverflowCount:0,overflow:[],visible:t.items})},e.prototype.shouldComponentUpdate=function(t,e){return!(this.state!==e&&Object(pe.e)(this.state,e))},e.prototype.componentDidUpdate=function(t,e){Object(pe.e)(e,this.state)||this.repartition(!1);var n=this.state,r=n.direction,i=n.overflow,o=n.lastOverflowCount;r===qt.NONE&&r!==e.direction&&i.length!==o&&Object(g.safeInvoke)(this.props.onOverflow,i)},e.prototype.render=function(){var t=this,e=this.props,n=e.className,r=e.collapseFrom,i=e.observeParents,a=e.style,s=e.tagName,A=void 0===s?"div":s,l=e.visibleItemRenderer,c=this.maybeRenderOverflow();return u.createElement(It,{onResize:this.resize,observeParents:i},u.createElement(A,{className:F()(o.OVERFLOW_LIST,n),style:a},r===p.START?c:null,this.state.visible.map(l),r===p.END?c:null,u.createElement("div",{className:o.OVERFLOW_LIST_SPACER,ref:function(e){return t.spacer=e}})))},e.prototype.maybeRenderOverflow=function(){var t=this.state.overflow;return 0===t.length?null:this.props.overflowRenderer(t)},e.prototype.repartition=function(t){var e=this;null!=this.spacer&&(t?this.setState(function(t){return{direction:qt.GROW,lastOverflowCount:t.direction===qt.NONE?t.overflow.length:t.lastOverflowCount,overflow:[],visible:e.props.items}}):this.spacer.getBoundingClientRect().width<.9?this.setState(function(t){if(t.visible.length<=e.props.minVisibleItems)return null;var n=e.props.collapseFrom===p.START,r=t.visible.slice(),i=n?r.shift():r.pop();if(void 0===i)return null;var o=n?t.overflow.concat([i]):[i].concat(t.overflow);return{direction:t.direction===qt.NONE?qt.SHRINK:t.direction,overflow:o,visible:r}}):this.setState({direction:qt.NONE}))},e.displayName="Blueprint3.OverflowList",e.defaultProps={collapseFrom:p.START,minVisibleItems:0},e}(u.Component),me=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.renderOverflow=function(t){var n=e.props.collapseFrom,r=n===p.END?y.a.BOTTOM_RIGHT:y.a.BOTTOM_LEFT,i=t;return n===p.START&&(i=t.slice().reverse()),u.createElement("li",null,u.createElement(kt,c.__assign({position:r},e.props.popoverProps),u.createElement("span",{className:o.BREADCRUMBS_COLLAPSED}),u.createElement(fe,null,i.map(e.renderOverflowBreadcrumb))))},e.renderOverflowBreadcrumb=function(t,e){var n=null!=t.href||null!=t.onClick;return u.createElement(ge,c.__assign({disabled:!n},t,{text:t.text,key:e}))},e.renderBreadcrumbWrapper=function(t,n){var r=e.props.items[e.props.items.length-1]===t;return u.createElement("li",{key:n},e.renderBreadcrumb(t,r))},e}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.className,n=t.collapseFrom,r=t.items,i=t.minVisibleItems,a=t.overflowListProps,s=void 0===a?{}:a;return u.createElement(Ce,c.__assign({collapseFrom:n,minVisibleItems:i,tagName:"ul"},s,{className:F()(o.BREADCRUMBS,s.className,e),items:r,overflowRenderer:this.renderOverflow,visibleItemRenderer:this.renderBreadcrumbWrapper}))},e.prototype.renderBreadcrumb=function(t,e){return e&&null!=this.props.currentBreadcrumbRenderer?this.props.currentBreadcrumbRenderer(t):null!=this.props.breadcrumbRenderer?this.props.breadcrumbRenderer(t):u.createElement(le,c.__assign({},t,{current:e}))},e.defaultProps={collapseFrom:p.START},e}(u.PureComponent),ye=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.alignText,r=e.className,i=e.fill,a=e.minimal,s=e.large,A=e.vertical,l=c.__rest(e,["alignText","className","fill","minimal","large","vertical"]),g=F()(o.BUTTON_GROUP,((t={})[o.FILL]=i,t[o.LARGE]=s,t[o.MINIMAL]=a,t[o.VERTICAL]=A,t),o.alignmentClass(n),r);return u.createElement("div",c.__assign({},l,{className:g}),this.props.children)},e.displayName="Blueprint3.ButtonGroup",e}(u.PureComponent),Ee=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.className,r=e.children,i=e.icon,a=e.intent,s=e.title,A=c.__rest(e,["className","children","icon","intent","title"]),l=this.getIconName(i,a),g=F()(o.CALLOUT,o.intentClass(a),((t={})[o.CALLOUT_ICON]=null!=l,t),n);return u.createElement("div",c.__assign({className:g},A),l&&u.createElement(Gt,{icon:l,iconSize:Gt.SIZE_LARGE}),s&&u.createElement(Zt,null,s),r)},e.prototype.getIconName=function(t,e){if(null!==t){if(void 0!==t)return t;switch(e){case m.a.DANGER:return"error";case m.a.PRIMARY:return"info-sign";case m.a.WARNING:return"warning-sign";case m.a.SUCCESS:return"tick";default:return}}},e.displayName="Blueprint3.Callout",e}(u.PureComponent),ve=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.className,r=e.elevation,i=e.interactive,a=c.__rest(e,["className","elevation","interactive"]),s=F()(o.CARD,((t={})[o.INTERACTIVE]=i,t),o.elevationClass(r),n);return u.createElement("div",c.__assign({className:s},a))},e.displayName="Blueprint3.Card",e.defaultProps={elevation:C.a.ZERO,interactive:!1},e}(u.PureComponent);!function(t){t[t.OPEN_START=0]="OPEN_START",t[t.OPENING=1]="OPENING",t[t.OPEN=2]="OPEN",t[t.CLOSING_START=3]="CLOSING_START",t[t.CLOSING=4]="CLOSING",t[t.CLOSED=5]="CLOSED"}(Ie||(Ie={}));var Be=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={animationState:e.props.isOpen?Ie.OPEN:Ie.CLOSED,height:"0px"},e.height=0,e.contentsRefHandler=function(t){e.contents=t,null!=t&&(e.height=e.contents.clientHeight,e.setState({animationState:e.props.isOpen?Ie.OPEN:Ie.CLOSED,height:e.height+"px"}))},e}return c.__extends(e,t),e.prototype.componentWillReceiveProps=function(t){this.props.isOpen!==t.isOpen&&(this.clearTimeouts(),this.state.animationState===Ie.CLOSED||t.isOpen?this.state.animationState!==Ie.OPEN&&t.isOpen&&this.setState({animationState:Ie.OPEN_START}):this.setState({animationState:Ie.CLOSING_START,height:this.height+"px"}))},e.prototype.render=function(){var t=this.state.animationState!==Ie.CLOSED,e=t||this.props.keepChildrenMounted,n=t&&this.state.animationState!==Ie.CLOSING,r="auto"===this.state.height,i={height:t?this.state.height:void 0,overflowY:r?"visible":void 0,transition:r?"none":void 0},a={transform:n?"translateY(0)":"translateY(-"+this.height+"px)",transition:r?"none":void 0};return u.createElement(this.props.component,{className:F()(o.COLLAPSE,this.props.className),style:i},u.createElement("div",{className:o.COLLAPSE_BODY,ref:this.contentsRefHandler,style:a,"aria-hidden":!t&&this.props.keepChildrenMounted},e?this.props.children:null))},e.prototype.componentDidMount=function(){this.forceUpdate(),this.props.isOpen?this.setState({animationState:Ie.OPEN,height:"auto"}):this.setState({animationState:Ie.CLOSED})},e.prototype.componentDidUpdate=function(){var t=this;null!=this.contents&&0!==this.contents.clientHeight&&(this.height=this.contents.clientHeight),this.state.animationState===Ie.CLOSING_START&&(this.setTimeout(function(){return t.setState({animationState:Ie.CLOSING,height:"0px"})}),this.setTimeout(function(){return t.onDelayedStateChange()},this.props.transitionDuration)),this.state.animationState===Ie.OPEN_START&&(this.setState({animationState:Ie.OPENING,height:this.height+"px"}),this.setTimeout(function(){return t.onDelayedStateChange()},this.props.transitionDuration))},e.prototype.onDelayedStateChange=function(){switch(this.state.animationState){case Ie.OPENING:this.setState({animationState:Ie.OPEN,height:"auto"});break;case Ie.CLOSING:this.setState({animationState:Ie.CLOSED})}},e.displayName="Blueprint3.Collapse",e.defaultProps={component:"div",isOpen:!1,keepChildrenMounted:!1,transitionDuration:200},e}(d),be=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t,e=this,n=this.props.collapseFrom,r=u.Children.count(this.props.children),i=this.partitionChildren(),a=i[0],s=i[1],A=a.map(function(t,i){var o=n===p.START?r-1-i:i;return u.createElement("li",{className:e.props.visibleItemClassName,key:o},e.props.visibleItemRenderer(t.props,o))});if(n===p.START&&A.reverse(),s.length>0){var l=n===p.END?y.a.BOTTOM_RIGHT:y.a.BOTTOM_LEFT;t=u.createElement("li",{className:this.props.visibleItemClassName},u.createElement(kt,c.__assign({content:u.createElement(fe,null,s),position:l},this.props.dropdownProps),this.props.dropdownTarget))}return u.createElement("ul",{className:F()(o.COLLAPSIBLE_LIST,this.props.className)},n===p.START?t:null,A,n===p.END?t:null)},e.prototype.partitionChildren=function(){if(null==this.props.children)return[[],[]];var t=u.Children.map(this.props.children,function(t,e){if(!Object(g.isElementOfType)(t,ge))throw new Error(At.e);return u.cloneElement(t,{key:"visible-"+e})});this.props.collapseFrom===p.START&&t.reverse();var e=this.props.visibleItemCount;return[t.slice(0,e),t.slice(e)]},e.displayName="Blueprint3.CollapsibleList",e.defaultProps={collapseFrom:p.START,dropdownTarget:null,visibleItemCount:3,visibleItemRenderer:null},e}(u.Component);function Qe(t){return Object(g.isFunction)(t.prototype.renderContextMenu)||console.warn(At.g),(e=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var e=this,n=t.prototype.render.call(this);if(null==n)return n;if(!u.isValidElement(n))return console.warn(At.f),n;var r=n.props.onContextMenu;return u.cloneElement(n,{onContextMenu:function(t){if(!t.defaultPrevented){if(Object(g.isFunction)(e.renderContextMenu)){var n=e.renderContextMenu(t);if(null!=n){var i=function(t){return t instanceof Element&&null!=t.closest("."+o.DARK)}(T.findDOMNode(e));t.preventDefault(),Nt(n,{left:t.clientX,top:t.clientY},e.onContextMenuClose,i)}}Object(g.safeInvoke)(r,t)}}})},e}(t)).displayName="ContextMenuTarget("+Object(g.getDisplayName)(t)+")",e;var e}var we=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.className,n=t.tagName,r=void 0===n?"div":n,i=c.__rest(t,["className","tagName"]),a=F()(o.DIVIDER,e);return u.createElement(r,c.__assign({},i,{className:a}))},e.displayName="Blueprint3.Divider",e}(u.PureComponent),xe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t,e,n=this.props,r=n.size,i=n.style,a=n.position,s=n.vertical,A=a?Object(y.b)(a):null,l=F()(o.DRAWER,((t={})[o.VERTICAL]=!A&&s,t[A?o.positionClass(A):""]=!0,t),this.props.className),g=null==r?i:c.__assign({},i,((e={})[(A?Object(y.c)(A):s)?"height":"width"]=r,e));return u.createElement(ft,c.__assign({},this.props,{className:o.OVERLAY_CONTAINER}),u.createElement("div",{className:l,style:g},this.maybeRenderHeader(),this.props.children))},e.prototype.validateProps=function(t){null==t.title&&(null!=t.icon&&console.warn(At.i),null!=t.isCloseButtonShown&&console.warn(At.h)),null!=t.position&&(t.vertical&&console.warn(At.k),t.position!==Object(y.b)(t.position)&&console.warn(At.j))},e.prototype.maybeRenderCloseButton=function(){return!1!==this.props.isCloseButtonShown?u.createElement(Kt,{"aria-label":"Close",className:o.DIALOG_CLOSE_BUTTON,icon:u.createElement(Gt,{icon:"small-cross",iconSize:Gt.SIZE_LARGE}),minimal:!0,onClick:this.props.onClose}):null},e.prototype.maybeRenderHeader=function(){var t=this.props,e=t.icon,n=t.title;return null==n?null:u.createElement("div",{className:o.DRAWER_HEADER},u.createElement(Gt,{icon:e,iconSize:Gt.SIZE_LARGE}),u.createElement(Zt,null,n),this.maybeRenderCloseButton())},e.displayName="Blueprint3.Drawer",e.defaultProps={canOutsideClickClose:!0,isOpen:!1,position:null,style:{},vertical:!1},e.SIZE_SMALL="360px",e.SIZE_STANDARD="50%",e.SIZE_LARGE="90%",e}(d),_e="undefined"!=typeof navigator?navigator.userAgent:"",De={isEdge:/Edge/.test(_e),isInternetExplorer:/Trident|rv:11/.test(_e),isWebkit:/AppleWebKit/.test(_e)},Me=function(){return De.isEdge},ke=function(){return De.isInternetExplorer},Se=function(t){function e(e,n){var r=t.call(this,e,n)||this;r.refHandlers={content:function(t){r.valueElement=t},input:function(t){if(null!=t){t.focus();var e=t.value.length;t.setSelectionRange(r.props.selectAllOnFocus?0:e,e),r.props.selectAllOnFocus||(t.scrollLeft=t.scrollWidth)}}},r.cancelEditing=function(){var t=r.state,e=t.lastValue,n=t.value;r.setState({isEditing:!1,value:e}),n!==e&&Object(g.safeInvoke)(r.props.onChange,e),Object(g.safeInvoke)(r.props.onCancel,e)},r.toggleEditing=function(){if(r.state.isEditing){var t=r.state.value;r.setState({isEditing:!1,lastValue:t}),Object(g.safeInvoke)(r.props.onConfirm,t)}else r.props.disabled||r.setState({isEditing:!0})},r.handleFocus=function(){r.props.disabled||r.setState({isEditing:!0})},r.handleTextChange=function(t){var e=t.target.value;null==r.props.value&&r.setState({value:e}),Object(g.safeInvoke)(r.props.onChange,e)},r.handleKeyEvent=function(t){var e=t.altKey,n=t.ctrlKey,i=t.metaKey,o=t.shiftKey,a=t.which;if(a!==x){var s=e||n||i||o;a===Q&&((e||o)&&t.preventDefault(),r.props.confirmOnEnterKey&&r.props.multiline?null!=t.target&&s?(!function(t,e){var n=t.selectionEnd,r=t.selectionStart,i=t.value;if(r>=0){var o=i.substring(0,r),a=i.substring(n,i.length),s=e.length;t.value=""+o+e+a,t.selectionStart=r+s,t.selectionEnd=r+s}}(t.target,"\n"),r.handleTextChange(t)):r.toggleEditing():r.props.multiline&&!s||r.toggleEditing())}else r.cancelEditing()};var i=null==e.value?e.defaultValue:e.value;return r.state={inputHeight:0,inputWidth:0,isEditing:!0===e.isEditing&&!1===e.disabled,lastValue:i,value:i},r}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.disabled,r=e.multiline,i=null==this.props.value?this.state.value:this.props.value,a=null!=i&&""!==i,s=F()(o.EDITABLE_TEXT,o.intentClass(this.props.intent),((A={})[o.DISABLED]=n,A[o.EDITABLE_TEXT_EDITING]=this.state.isEditing,A[o.EDITABLE_TEXT_PLACEHOLDER]=!a,A[o.MULTILINE]=r,A),this.props.className);t=r?{height:this.state.isEditing?null:this.state.inputHeight}:{height:this.state.inputHeight,lineHeight:null!=this.state.inputHeight?this.state.inputHeight+"px":null,minWidth:this.props.minWidth};var A,l=this.state.isEditing||n?null:0;return u.createElement("div",{className:s,onFocus:this.handleFocus,tabIndex:l},this.maybeRenderInput(i),u.createElement("span",{className:o.EDITABLE_TEXT_CONTENT,ref:this.refHandlers.content,style:t},a?i:this.props.placeholder))},e.prototype.componentDidMount=function(){this.updateInputDimensions()},e.prototype.componentDidUpdate=function(t,e){this.state.isEditing&&!e.isEditing&&Object(g.safeInvoke)(this.props.onEdit,this.state.value),this.updateInputDimensions()},e.prototype.componentWillReceiveProps=function(t){var e={};null!=t.value&&(e.value=t.value),null!=t.isEditing&&(e.isEditing=t.isEditing),(t.disabled||null==t.disabled&&this.props.disabled)&&(e.isEditing=!1),this.setState(e)},e.prototype.maybeRenderInput=function(t){var e=this.props,n=e.maxLength,r=e.multiline,i=e.type,a=e.placeholder;if(this.state.isEditing){var s={className:o.EDITABLE_TEXT_INPUT,maxLength:n,onBlur:this.toggleEditing,onChange:this.handleTextChange,onKeyDown:this.handleKeyEvent,placeholder:a,style:{height:this.state.inputHeight,lineHeight:r||null==this.state.inputHeight?null:this.state.inputHeight+"px",width:r?"100%":this.state.inputWidth},value:t};return r?u.createElement("textarea",c.__assign({ref:this.refHandlers.input},s)):u.createElement("input",c.__assign({ref:this.refHandlers.input,type:i},s))}},e.prototype.updateInputDimensions=function(){if(null!=this.valueElement){var t=this.props,e=t.maxLines,n=t.minLines,r=t.minWidth,i=t.multiline,o=this.valueElement,a=o.parentElement,s=o.textContent,A=this.valueElement,l=A.scrollHeight,c=A.scrollWidth,u=Le(this.valueElement);i&&this.state.isEditing&&/\n$/.test(s)&&(l+=u),u>0&&(l=Object(g.clamp)(l,n*u,e*u)),l=Math.max(l,(h=this.valueElement,(""===(d=getComputedStyle(h).fontSize)?0:parseInt(d.slice(0,-2),10))+1),Le(a)),Me()?c+=5:ke()&&(c+=30),this.setState({inputHeight:l,inputWidth:Math.max(c,r)}),i&&this.state.isEditing&&this.setTimeout(function(){return a.style.height=l+"px"})}var h,d},e.displayName="Blueprint3.EditableText",e.defaultProps={confirmOnEnterKey:!1,defaultValue:"",disabled:!1,maxLines:1/0,minLines:1,minWidth:80,multiline:!1,placeholder:"Click to Edit",type:"text"},e}(d);function Le(t){var e=parseInt(getComputedStyle(t).lineHeight.slice(0,-2),10);if(isNaN(e)){var n=document.createElement("span");n.innerHTML="<br>",t.appendChild(n);var r=t.offsetHeight;n.innerHTML="<br><br>";var i=t.offsetHeight;t.removeChild(n),e=i-r}return e}var Re=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.children,r=e.className,i=e.fill,a=e.vertical,s=c.__rest(e,["children","className","fill","vertical"]),A=F()(o.CONTROL_GROUP,((t={})[o.FILL]=i,t[o.VERTICAL]=a,t),r);return u.createElement("div",c.__assign({},s,{className:A}),n)},e.displayName="Blueprint3.ControlGroup",e}(u.PureComponent),Ne=function(t){var e,n=t.alignIndicator,r=t.children,i=t.className,a=t.indicatorChildren,s=t.inline,A=t.inputRef,l=t.label,g=t.labelElement,h=t.large,d=t.style,f=t.type,p=t.typeClassName,I=t.tagName,C=void 0===I?"label":I,m=c.__rest(t,["alignIndicator","children","className","indicatorChildren","inline","inputRef","label","labelElement","large","style","type","typeClassName","tagName"]),y=F()(o.CONTROL,p,((e={})[o.DISABLED]=m.disabled,e[o.INLINE]=s,e[o.LARGE]=h,e),o.alignmentClass(n),i);return u.createElement(C,{className:y,style:d},u.createElement("input",c.__assign({},m,{ref:A,type:f})),u.createElement("span",{className:o.CONTROL_INDICATOR},a),l,g,r)},Fe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.innerLabelChecked,n=t.innerLabel,r=c.__rest(t,["innerLabelChecked","innerLabel"]),i=n||e?[u.createElement("div",{key:"checked",className:o.CONTROL_INDICATOR_CHILD},u.createElement("div",{className:o.SWITCH_INNER_TEXT},e||n)),u.createElement("div",{key:"unchecked",className:o.CONTROL_INDICATOR_CHILD},u.createElement("div",{className:o.SWITCH_INNER_TEXT},n))]:null;return u.createElement(Ne,c.__assign({},r,{type:"checkbox",typeClassName:o.SWITCH,indicatorChildren:i}))},e.displayName="Blueprint3.Switch",e}(u.PureComponent),Te=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){return u.createElement(Ne,c.__assign({},this.props,{type:"radio",typeClassName:o.RADIO}))},e.displayName="Blueprint3.Radio",e}(u.PureComponent),Ue=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={indeterminate:e.props.indeterminate||e.props.defaultIndeterminate||!1},e.handleChange=function(t){var n=t.target.indeterminate;null==e.props.indeterminate&&e.setState({indeterminate:n}),Object(g.safeInvoke)(e.props.onChange,t)},e.handleInputRef=function(t){e.input=t,Object(g.safeInvoke)(e.props.inputRef,t)},e}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=(t.defaultIndeterminate,t.indeterminate,c.__rest(t,["defaultIndeterminate","indeterminate"]));return u.createElement(Ne,c.__assign({},e,{inputRef:this.handleInputRef,onChange:this.handleChange,type:"checkbox",typeClassName:o.CHECKBOX}))},e.prototype.componentWillReceiveProps=function(t){var e=t.indeterminate;null!=e&&this.setState({indeterminate:e})},e.prototype.componentDidMount=function(){this.updateIndeterminate()},e.prototype.componentDidUpdate=function(){this.updateIndeterminate()},e.prototype.updateIndeterminate=function(){null!=this.input&&(this.input.indeterminate=this.state.indeterminate)},e.displayName="Blueprint3.Checkbox",e}(u.PureComponent),Oe=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.handleInputChange=function(t){g.safeInvoke(e.props.onInputChange,t),g.safeInvoke(e.props.inputProps.onChange,t)},e}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.className,r=e.disabled,i=e.fill,a=e.hasSelection,s=e.inputProps,A=e.large,l=(e.onInputChange,e.text),g=c.__rest(e,["className","disabled","fill","hasSelection","inputProps","large","onInputChange","text"]),h=F()(o.FILE_INPUT,((t={})[o.FILE_INPUT_HAS_SELECTION]=a,t[o.DISABLED]=r,t[o.FILL]=i,t[o.LARGE]=A,t),n);return u.createElement("label",c.__assign({},g,{className:h}),u.createElement("input",c.__assign({},s,{onChange:this.handleInputChange,type:"file",disabled:r})),u.createElement("span",{className:o.FILE_UPLOAD_INPUT},l))},e.displayName="Blueprint3.FileInput",e.defaultProps={hasSelection:!1,inputProps:{},text:"Choose file..."},e}(u.PureComponent),Ge=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.children,n=t.contentClassName,r=t.helperText,i=t.label,a=t.labelFor,s=t.labelInfo,A=t.style;return u.createElement("div",{className:this.getClassName(),style:A},i&&u.createElement("label",{className:o.LABEL,htmlFor:a},i," ",u.createElement("span",{className:o.TEXT_MUTED},s)),u.createElement("div",{className:F()(o.FORM_CONTENT,n)},e,r&&u.createElement("div",{className:o.FORM_HELPER_TEXT},r)))},e.prototype.getClassName=function(){var t,e=this.props,n=e.className,r=e.disabled,i=e.inline,a=e.intent;return F()(o.FORM_GROUP,o.intentClass(a),((t={})[o.DISABLED]=r,t[o.INLINE]=i,t),n)},e.displayName="Blueprint3.FormGroup",e}(u.PureComponent),He=10,je=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={rightElementWidth:He},e.refHandlers={rightElement:function(t){return e.rightElement=t}},e}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.className,r=e.disabled,i=e.fill,a=e.intent,s=e.large,A=e.small,l=e.leftIcon,g=e.round,h=F()(o.INPUT_GROUP,o.intentClass(a),((t={})[o.DISABLED]=r,t[o.FILL]=i,t[o.LARGE]=s,t[o.SMALL]=A,t[o.ROUND]=g,t),n),d=c.__assign({},this.props.style,{paddingRight:this.state.rightElementWidth});return u.createElement("div",{className:h},u.createElement(Gt,{icon:l}),u.createElement("input",c.__assign({type:"text"},v(this.props),{className:o.INPUT,ref:this.props.inputRef,style:d})),this.maybeRenderRightElement())},e.prototype.componentDidMount=function(){this.updateInputWidth()},e.prototype.componentDidUpdate=function(){this.updateInputWidth()},e.prototype.maybeRenderRightElement=function(){var t=this.props.rightElement;if(null!=t)return u.createElement("span",{className:o.INPUT_ACTION,ref:this.refHandlers.rightElement},t)},e.prototype.updateInputWidth=function(){if(null!=this.rightElement){var t=this.rightElement.clientWidth;Math.abs(t-this.state.rightElementWidth)>2&&this.setState({rightElementWidth:t})}else this.setState({rightElementWidth:He})},e.displayName="Blueprint3.InputGroup",e}(u.PureComponent);function Ye(t){return void 0===t&&(t=""),t.toString()}var Pe,Ke=/^[Ee0-9\+\-\.]$/;function ze(t){return Ke.test(t)}!function(t){t[t.DOWN=-1]="DOWN",t[t.UP=1]="UP"}(Pe||(Pe={}));var Je=["allowNumericCharactersOnly","buttonPosition","clampValueOnBlur","className","majorStepSize","minorStepSize","onButtonClick","onValueChange","selectAllOnFocus","selectAllOnIncrement","stepSize"],qe=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={shouldSelectAfterUpdate:!1,stepMaxPrecision:e.getStepMaxPrecision(e.props),value:Ye(e.props.value)},e.didPasteEventJustOccur=!1,e.delta=0,e.inputElement=null,e.intervalId=null,e.incrementButtonHandlers=e.getButtonEventHandlers(Pe.UP),e.decrementButtonHandlers=e.getButtonEventHandlers(Pe.DOWN),e.inputRef=function(t){e.inputElement=t,g.safeInvoke(e.props.inputRef,t)},e.handleButtonClick=function(t,n){var r=e.updateDelta(n,t),i=e.incrementValue(r);e.invokeValueCallback(i,e.props.onButtonClick)},e.stopContinuousChange=function(){e.delta=0,e.clearTimeouts(),clearInterval(e.intervalId),document.removeEventListener("mouseup",e.stopContinuousChange)},e.handleContinuousChange=function(){var t=e.incrementValue(e.delta);e.invokeValueCallback(t,e.props.onButtonClick)},e.handleInputFocus=function(t){e.setState({shouldSelectAfterUpdate:e.props.selectAllOnFocus}),g.safeInvoke(e.props.onFocus,t)},e.handleInputBlur=function(t){if(e.setState({shouldSelectAfterUpdate:!1}),e.props.clampValueOnBlur){var n=t.target.value,r=e.getSanitizedValue(n);e.setState({value:r}),n!==r&&e.invokeValueCallback(r,e.props.onValueChange)}g.safeInvoke(e.props.onBlur,t)},e.handleInputKeyDown=function(t){if(!e.props.disabled&&!e.props.readOnly){var n,i=t.keyCode;if(i===r.ARROW_UP?n=Pe.UP:i===r.ARROW_DOWN&&(n=Pe.DOWN),null!=n){t.preventDefault();var o=e.updateDelta(n,t);e.incrementValue(o)}g.safeInvoke(e.props.onKeyDown,t)}},e.handleInputKeyPress=function(t){e.props.allowNumericCharactersOnly&&!function(t){return null==t.key||!!(t.ctrlKey||t.altKey||t.metaKey)||!(1===t.key.length)||ze(t.key)}(t)&&t.preventDefault(),g.safeInvoke(e.props.onKeyPress,t)},e.handleInputPaste=function(t){e.didPasteEventJustOccur=!0,g.safeInvoke(e.props.onPaste,t)},e.handleInputChange=function(t){var n=t.target.value,r=n;e.props.allowNumericCharactersOnly&&e.didPasteEventJustOccur&&(e.didPasteEventJustOccur=!1,r=n.split("").filter(ze).join(""));e.setState({shouldSelectAfterUpdate:!1,value:r}),e.invokeValueCallback(r,e.props.onValueChange)},e}return c.__extends(e,t),e.prototype.componentWillReceiveProps=function(n){t.prototype.componentWillReceiveProps.call(this,n);var r=Ye(n.value),i=n.min!==this.props.min,o=n.max!==this.props.max,a=i||o,s=r!==e.VALUE_EMPTY?this.getSanitizedValue(r,0,n.min,n.max):e.VALUE_EMPTY,A=this.getStepMaxPrecision(n);a&&s!==this.state.value?(this.setState({stepMaxPrecision:A,value:s}),this.invokeValueCallback(s,this.props.onValueChange)):this.setState({stepMaxPrecision:A,value:r})},e.prototype.render=function(){var t,e=this.props,n=e.buttonPosition,r=e.className,i=e.fill,a=e.large,s=F()(o.NUMERIC_INPUT,((t={})[o.LARGE]=a,t),r),A=this.renderButtons();return u.createElement(Re,{className:s,fill:i},n===y.a.LEFT&&A,this.renderInput(),n===y.a.RIGHT&&A)},e.prototype.componentDidUpdate=function(){this.state.shouldSelectAfterUpdate&&this.inputElement.setSelectionRange(0,this.state.value.length)},e.prototype.validateProps=function(t){var e=t.majorStepSize,n=t.max,r=t.min,i=t.minorStepSize,o=t.stepSize;if(null!=r&&null!=n&&r>n)throw new Error(At.t);if(null==o)throw new Error(At.v);if(o<=0)throw new Error(At.u);if(i&&i<=0)throw new Error(At.s);if(e&&e<=0)throw new Error(At.q);if(i&&i>o)throw new Error(At.r);if(e&&e<o)throw new Error(At.p)},e.prototype.renderButtons=function(){var t=this.props.intent,e=this.props.disabled||this.props.readOnly;return u.createElement(ye,{className:o.FIXED,key:"button-group",vertical:!0},u.createElement(Kt,c.__assign({disabled:e,icon:"chevron-up",intent:t},this.incrementButtonHandlers)),u.createElement(Kt,c.__assign({disabled:e,icon:"chevron-down",intent:t},this.decrementButtonHandlers)))},e.prototype.renderInput=function(){var t=v(this.props,Je,!0);return u.createElement(je,c.__assign({autoComplete:"off"},t,{intent:this.props.intent,inputRef:this.inputRef,large:this.props.large,leftIcon:this.props.leftIcon,onFocus:this.handleInputFocus,onBlur:this.handleInputBlur,onChange:this.handleInputChange,onKeyDown:this.handleInputKeyDown,onKeyPress:this.handleInputKeyPress,onPaste:this.handleInputPaste,rightElement:this.props.rightElement,value:this.state.value}))},e.prototype.getButtonEventHandlers=function(t){var e=this;return{onKeyDown:function(n){r.isKeyboardClick(n.keyCode)&&e.handleButtonClick(n,t)},onMouseDown:function(n){e.handleButtonClick(n,t),e.startContinuousChange()}}},e.prototype.startContinuousChange=function(){var t=this;document.addEventListener("mouseup",this.stopContinuousChange),this.setTimeout(function(){t.intervalId=window.setInterval(t.handleContinuousChange,e.CONTINUOUS_CHANGE_INTERVAL)},e.CONTINUOUS_CHANGE_DELAY)},e.prototype.invokeValueCallback=function(t,e){g.safeInvoke(e,+t,t)},e.prototype.incrementValue=function(t){var n=this.state.value||e.VALUE_ZERO,r=this.getSanitizedValue(n,t);return this.setState({shouldSelectAfterUpdate:this.props.selectAllOnIncrement,value:r}),this.invokeValueCallback(r,this.props.onValueChange),r},e.prototype.getIncrementDelta=function(t,e,n){var r=this.props,i=r.majorStepSize,o=r.minorStepSize,a=r.stepSize;return e&&null!=i?t*i:n&&null!=o?t*o:t*a},e.prototype.getSanitizedValue=function(t,n,r,i){return void 0===n&&(n=0),void 0===r&&(r=this.props.min),void 0===i&&(i=this.props.max),function(t){return null!=t&&t-parseFloat(t)+1>=0}(t)?function(t,e,n){var r=null!=e?e:-1/0,i=null!=n?n:1/0;return Object(g.clamp)(t,r,i)}(function(t,e){var n=Math.pow(10,e);return Math.round(t*n)/n}(parseFloat(t)+n,this.state.stepMaxPrecision),r,i).toString():e.VALUE_EMPTY},e.prototype.getStepMaxPrecision=function(t){return null!=t.minorStepSize?g.countDecimalPlaces(t.minorStepSize):g.countDecimalPlaces(t.stepSize)},e.prototype.updateDelta=function(t,e){return this.delta=this.getIncrementDelta(t,e.shiftKey,e.altKey),this.delta},e.displayName="Blueprint3.NumericInput",e.VALUE_EMPTY="",e.VALUE_ZERO="0",e.defaultProps={allowNumericCharactersOnly:!0,buttonPosition:y.a.RIGHT,clampValueOnBlur:!1,large:!1,majorStepSize:10,minorStepSize:.1,selectAllOnFocus:!1,selectAllOnIncrement:!1,stepSize:1,value:e.VALUE_EMPTY},e.CONTINUOUS_CHANGE_DELAY=300,e.CONTINUOUS_CHANGE_INTERVAL=100,e}(d),Ve=0;for(var We=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.autoGroupName=We.displayName+"-"+Ve++,e}return c.__extends(e,t),e.prototype.render=function(){var t=this.props.label;return u.createElement("div",{className:this.props.className},null==t?null:u.createElement("label",{className:o.LABEL},t),Array.isArray(this.props.options)?this.renderOptions():this.renderChildren())},e.prototype.validateProps=function(){null!=this.props.children&&null!=this.props.options&&console.warn(At.I)},e.prototype.renderChildren=function(){var t=this;return u.Children.map(this.props.children,function(e){return Object(g.isElementOfType)(e,Te)?u.cloneElement(e,t.getRadioProps(e.props)):e})},e.prototype.renderOptions=function(){var t=this;return this.props.options.map(function(e){return u.createElement(Te,c.__assign({},t.getRadioProps(e),{key:e.value,labelElement:e.label||e.value}))})},e.prototype.getRadioProps=function(t){var e=this.props.name,n=t.className,r=t.disabled,i=t.value;return{checked:i===this.props.selectedValue,className:n,disabled:r||this.props.disabled,inline:this.props.inline,name:null==e?this.autoGroupName:e,onChange:this.props.onChange,value:i}},e.displayName="Blueprint3.RadioGroup",e}(d),Xe=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={},e.handleChange=function(t){e.props.growVertically&&e.setState({height:t.target.scrollHeight}),null!=e.props.onChange&&e.props.onChange(t)},e}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.className,r=e.fill,i=e.inputRef,a=e.intent,s=e.large,A=e.small,l=e.growVertically,g=c.__rest(e,["className","fill","inputRef","intent","large","small","growVertically"]),h=F()(o.INPUT,o.intentClass(a),((t={})[o.FILL]=r,t[o.LARGE]=s,t[o.SMALL]=A,t),n),d=g.style,f=void 0===d?{}:d;return l&&null!=this.state.height&&(f=c.__assign({},f,{height:this.state.height+"px"})),u.createElement("textarea",c.__assign({},g,{className:h,onChange:this.handleChange,ref:i,style:f}))},e.displayName="Blueprint3.TextArea",e}(u.PureComponent),Ze=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.className,r=e.disabled,i=e.elementRef,a=e.fill,s=e.iconProps,A=e.large,l=e.minimal,g=e.options,h=void 0===g?[]:g,d=c.__rest(e,["className","disabled","elementRef","fill","iconProps","large","minimal","options"]),f=F()(o.HTML_SELECT,((t={})[o.DISABLED]=r,t[o.FILL]=a,t[o.LARGE]=A,t[o.MINIMAL]=l,t),n),p=h.map(function(t){var e="object"==typeof t?t:{value:t};return u.createElement("option",c.__assign({},e,{key:e.value,children:e.label||e.value}))});return u.createElement("div",{className:f},u.createElement("select",c.__assign({disabled:r,ref:i},d,{multiple:!1}),p,d.children),u.createElement(Gt,c.__assign({icon:"double-caret-vertical"},s)))},e}(u.PureComponent),$e=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.bordered,r=e.className,i=e.condensed,a=e.elementRef,s=e.interactive,A=e.small,l=e.striped,g=c.__rest(e,["bordered","className","condensed","elementRef","interactive","small","striped"]),h=F()(o.HTML_TABLE,((t={})[o.HTML_TABLE_BORDERED]=n,t[o.HTML_TABLE_CONDENSED]=i,t[o.HTML_TABLE_STRIPED]=l,t[o.INTERACTIVE]=s,t[o.SMALL]=A,t),r);return u.createElement("table",c.__assign({},g,{ref:a,className:h}))},e}(u.PureComponent),tn={8:"backspace",9:"tab",13:"enter",20:"capslock",27:"esc",32:"space",33:"pageup",34:"pagedown",35:"end",36:"home",37:"left",38:"up",39:"right",40:"down",45:"ins",46:"del",48:"0",49:"1",50:"2",51:"3",52:"4",53:"5",54:"6",55:"7",56:"8",57:"9",65:"a",66:"b",67:"c",68:"d",69:"e",70:"f",71:"g",72:"h",73:"i",74:"j",75:"k",76:"l",77:"m",78:"n",79:"o",80:"p",81:"q",82:"r",83:"s",84:"t",85:"u",86:"v",87:"w",88:"x",89:"y",90:"z",106:"*",107:"+",109:"-",110:".",111:"/",186:";",187:"=",188:",",189:"-",190:".",191:"/",192:"`",219:"[",220:"\\",221:"]",222:"'"},en={16:"shift",17:"ctrl",18:"alt",91:"meta",93:"meta",224:"meta"},nn={alt:1,ctrl:2,meta:4,shift:8},rn={cmd:"meta",command:"meta",escape:"esc",minus:"-",mod:un()?"meta":"ctrl",option:"alt",plus:"+",return:"enter",win:"meta"},on={"~":"`","!":"1","@":"2","#":"3",$:"4","%":"5","^":"6","&":"7","*":"8","(":"9",")":"0",_:"-","+":"=","{":"[","}":"]","|":"\\",":":";",'"':"'","<":",",">":".","?":"/"},an=1;an<=12;++an)tn[111+an]="f"+an;for(an=0;an<=9;++an)tn[96+an]="num"+an.toString();function sn(t,e){return t.modifiers===e.modifiers&&t.key===e.key}var An=function(t){for(var e=0,n=null,r=0,i=t.replace(/\s/g,"").toLowerCase().split("+");r<i.length;r++){var o=i[r];if(""===o)throw new Error('Failed to parse key combo "'+t+'".\n                Valid key combos look like "cmd + plus", "shift+p", or "!"');null!=rn[o]&&(o=rn[o]),null!=nn[o]?e+=nn[o]:null!=on[o]?(e+=nn.shift,n=on[o]):n=o.toLowerCase()}return{modifiers:e,key:n}},ln=function(t){var e=[];t.ctrlKey&&e.push("ctrl"),t.altKey&&e.push("alt"),t.shiftKey&&e.push("shift"),t.metaKey&&e.push("meta");var n=t.which;return null!=en[n]||(null!=tn[n]?e.push(tn[n]):e.push(String.fromCharCode(n).toLowerCase())),e.join(" + ")},cn=function(t){var e=null,n=t.which;null!=en[n]||(e=null!=tn[n]?tn[n]:String.fromCharCode(n).toLowerCase());var r=0;return t.altKey&&(r+=nn.alt),t.ctrlKey&&(r+=nn.ctrl),t.metaKey&&(r+=nn.meta),t.shiftKey&&(r+=nn.shift),{modifiers:r,key:e}};function un(t){var e=null!=t?t:"undefined"!=typeof navigator?navigator.platform:void 0;return null!=e&&/Mac|iPod|iPhone|iPad/.test(e)}var gn={alt:"key-option",cmd:"key-command",ctrl:"key-control",delete:"key-delete",down:"arrow-down",enter:"key-enter",left:"arrow-left",meta:"key-command",right:"arrow-right",shift:"key-shift",up:"arrow-up"},hn=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.renderKey=function(t,e){var n=gn[t],r="key-"+e;return null==n?u.createElement("kbd",{className:o.KEY,key:r},t):u.createElement("kbd",{className:F()(o.KEY,o.MODIFIER_KEY),key:r},u.createElement(Gt,{icon:n})," ",t)},e.renderMinimalKey=function(t,e){var n=gn[t];return null==n?t:u.createElement(Gt,{icon:n,key:"key-"+e})},e}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.className,n=t.combo,r=t.minimal,i=function(t,e){return t.replace(/\s/g,"").split("+").map(function(t){var n=null!=rn[t]?rn[t]:t;return"meta"===n?un(e)?"cmd":"ctrl":n})}(n).map(function(t){return 1===t.length?t.toUpperCase():t}).map(r?this.renderMinimalKey:this.renderKey);return u.createElement("span",{className:F()(o.KEY_COMBO,e)},i)},e.displayName="Blueprint3.KeyCombo",e}(u.Component),dn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.label,n=t.className,r=c.__rest(t,["label","className"]),i=F()(o.HOTKEY,n);return u.createElement("div",{className:i},u.createElement("div",{className:o.HOTKEY_LABEL},e),u.createElement(hn,c.__assign({},r)))},e.prototype.validateProps=function(t){if(!0!==t.global&&null==t.group)throw new Error("non-global <Hotkey>s must define a group")},e.displayName="Blueprint3.Hotkey",e.defaultProps={allowInInput:!1,disabled:!1,global:!1,preventDefault:!1,stopPropagation:!1},e}(d),fn=new(function(){function t(){var t=this;this.componentProps={globalHotkeysGroup:"Global hotkeys"},this.hotkeysQueue=[],this.isDialogShowing=!1,this.show=function(){t.isDialogShowing=!0,t.render()},this.hide=function(){t.isDialogShowing=!1,t.render()}}return t.prototype.render=function(){null==this.container&&(this.container=this.getContainer()),T.render(this.renderComponent(),this.container)},t.prototype.unmount=function(){null!=this.container&&(T.unmountComponentAtNode(this.container),this.container.remove(),delete this.container)},t.prototype.enqueueHotkeysForDisplay=function(t){this.hotkeysQueue.push(t),window.clearTimeout(this.showTimeoutToken),this.showTimeoutToken=window.setTimeout(this.show,10)},t.prototype.hideAfterDelay=function(){window.clearTimeout(this.hideTimeoutToken),this.hideTimeoutToken=window.setTimeout(this.hide,10)},t.prototype.isShowing=function(){return this.isDialogShowing},t.prototype.getContainer=function(){return null==this.container&&(this.container=document.createElement("div"),this.container.classList.add(o.PORTAL),document.body.appendChild(this.container)),this.container},t.prototype.renderComponent=function(){return u.createElement(se,c.__assign({},this.componentProps,{className:F()(o.HOTKEY_DIALOG,this.componentProps.className),isOpen:this.isDialogShowing,onClose:this.hide}),u.createElement("div",{className:o.DIALOG_BODY},this.renderHotkeys()))},t.prototype.renderHotkeys=function(){var t=this,e=this.emptyHotkeyQueue().map(function(e,n){var r=!0===e.global&&null==e.group?t.componentProps.globalHotkeysGroup:e.group;return u.createElement(dn,c.__assign({key:n},e,{group:r}))});return u.createElement(Bn,null,e)},t.prototype.emptyHotkeyQueue=function(){var t=this.hotkeysQueue.reduce(function(t,e){return t.concat(e)},[]);return this.hotkeysQueue.length=0,t},t}());function pn(){return fn.isShowing()}function In(t){for(var e in t)t.hasOwnProperty(e)&&(fn.componentProps[e]=t[e])}function Cn(){fn.hide()}var mn,yn="?";!function(t){t.LOCAL="local",t.GLOBAL="global"}(mn||(mn={}));var En=function(){function t(t){var e=this;this.scope=t,this.actions=[],this.handleKeyDown=function(t){var n,r=cn(t);e.isTextInput(t)||!sn(An(yn),r)?pn()||e.invokeNamedCallbackIfComboRecognized(r,"onKeyDown",t):pn()?fn.hideAfterDelay():(n=e.actions.map(function(t){return t.props}),fn.enqueueHotkeysForDisplay(n))},this.handleKeyUp=function(t){pn()||e.invokeNamedCallbackIfComboRecognized(cn(t),"onKeyUp",t)}}return t.prototype.count=function(){return this.actions.length},t.prototype.clear=function(){this.actions=[]},t.prototype.setHotkeys=function(t){var e=this,n=[];u.Children.forEach(t.children,function(t){Object(g.isElementOfType)(t,dn)&&e.isScope(t.props)&&n.push({combo:An(t.props.combo),props:t.props})}),this.actions=n},t.prototype.invokeNamedCallbackIfComboRecognized=function(t,e,n){for(var r=this.isTextInput(n),i=0,o=this.actions;i<o.length;i++){var a=o[i];!(r&&!a.props.allowInInput||a.props.disabled)&&sn(a.combo,t)&&(a.props.preventDefault&&n.preventDefault(),a.props.stopPropagation&&(n.isPropagationStopped=!0,n.stopPropagation()),Object(g.safeInvoke)(a.props[e],n))}},t.prototype.isScope=function(t){return(t.global?mn.GLOBAL:mn.LOCAL)===this.scope},t.prototype.isTextInput=function(t){var e=t.target;if(null==e||null==e.closest)return!1;var n=e.closest("input, textarea, [contenteditable=true]");if(null==n)return!1;if("input"===n.tagName.toLowerCase()){var r=n.type;if("checkbox"===r||"radio"===r)return!1}return!n.readOnly},t}();function vn(t){return Object(g.isFunction)(t.prototype.renderHotkeys)||console.warn(At.n),(e=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.componentWillMount=function(){null!=t.prototype.componentWillMount&&t.prototype.componentWillMount.call(this),this.localHotkeysEvents=new En(mn.LOCAL),this.globalHotkeysEvents=new En(mn.GLOBAL)},e.prototype.componentDidMount=function(){null!=t.prototype.componentDidMount&&t.prototype.componentDidMount.call(this),document.addEventListener("keydown",this.globalHotkeysEvents.handleKeyDown),document.addEventListener("keyup",this.globalHotkeysEvents.handleKeyUp)},e.prototype.componentWillUnmount=function(){null!=t.prototype.componentWillUnmount&&t.prototype.componentWillUnmount.call(this),document.removeEventListener("keydown",this.globalHotkeysEvents.handleKeyDown),document.removeEventListener("keyup",this.globalHotkeysEvents.handleKeyUp),this.globalHotkeysEvents.clear(),this.localHotkeysEvents.clear()},e.prototype.render=function(){var e=this,n=t.prototype.render.call(this);if(null==n)return n;if(!u.isValidElement(n))return console.warn(At.m),n;if(Object(g.isFunction)(this.renderHotkeys)){var r=this.renderHotkeys();if(this.localHotkeysEvents.setHotkeys(r.props),this.globalHotkeysEvents.setHotkeys(r.props),this.localHotkeysEvents.count()>0){var i=void 0===r.props.tabIndex?0:r.props.tabIndex,o=n.props,a=o.keyDown,s=o.keyUp;return u.cloneElement(n,{tabIndex:i,onKeyDown:function(t){e.localHotkeysEvents.handleKeyDown(t.nativeEvent),Object(g.safeInvoke)(a,t)},onKeyUp:function(t){e.localHotkeysEvents.handleKeyUp(t.nativeEvent),Object(g.safeInvoke)(s,t)}})}}return n},e}(t)).displayName="HotkeysTarget("+Object(g.getDisplayName)(t)+")",e;var e}var Bn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=u.Children.map(this.props.children,function(t){return t.props});t.sort(function(t,e){return t.global===e.global?t.group.localeCompare(e.group):t.global?-1:1});for(var e=null,n=[],r=0,i=t;r<i.length;r++){var a=i[r],s=a.group;s!==e&&(n.push(u.createElement(Zt,{key:"group-"+n.length},s)),e=s),n.push(u.createElement(dn,c.__assign({key:n.length},a)))}var A=F()(o.HOTKEY_COLUMN,this.props.className);return u.createElement("div",{className:A},n)},e.prototype.validateProps=function(t){u.Children.forEach(t.children,function(t){if(!Object(g.isElementOfType)(t,dn))throw new Error(At.l)})},e.displayName="Blueprint3.Hotkeys",e.defaultProps={tabIndex:0},e}(d),bn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.className,n=c.__rest(t,["className"]);return u.createElement("div",c.__assign({className:F()(o.NAVBAR_DIVIDER,e)},n))},e.displayName="Blueprint3.NavbarDivider",e}(u.PureComponent),Qn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.align,n=t.children,r=t.className,i=c.__rest(t,["align","children","className"]),a=F()(o.NAVBAR_GROUP,o.alignmentClass(e),r);return u.createElement("div",c.__assign({className:a},i),n)},e.displayName="Blueprint3.NavbarGroup",e.defaultProps={align:f.a.LEFT},e}(u.PureComponent),wn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.children,n=t.className,r=c.__rest(t,["children","className"]);return u.createElement("div",c.__assign({className:F()(o.NAVBAR_HEADING,n)},r),e)},e.displayName="Blueprint3.NavbarHeading",e}(u.PureComponent),xn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.children,r=e.className,i=e.fixedToTop,a=c.__rest(e,["children","className","fixedToTop"]),s=F()(o.NAVBAR,((t={})[o.FIXED_TOP]=i,t),r);return u.createElement("div",c.__assign({className:s},a),n)},e.displayName="Blueprint3.Navbar",e.Divider=bn,e.Group=Qn,e.Heading=wn,e}(u.PureComponent),_n=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.action,n=t.children,r=t.className,i=t.description,a=t.title;return u.createElement("div",{className:F()(o.NON_IDEAL_STATE,r)},this.maybeRenderVisual(),a&&u.createElement(Zt,null,a),i&&Object(g.ensureElement)(i,"div"),e,n)},e.prototype.maybeRenderVisual=function(){var t=this.props.icon;return null==t?null:u.createElement("div",{className:o.NON_IDEAL_STATE_VISUAL},u.createElement(Gt,{icon:t,iconSize:3*Gt.SIZE_LARGE}))},e.displayName="Blueprint3.NonIdealState",e}(u.PureComponent),Dn=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.handleClose=function(){return e.props.onClose(e.props.panel)},e}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.panel,n=t.onOpen;return u.createElement("div",{className:o.PANEL_STACK_VIEW},this.maybeRenderHeader(),u.createElement(e.component,c.__assign({openPanel:n,closePanel:this.handleClose},e.props)))},e.prototype.maybeRenderHeader=function(){return this.props.showHeader?u.createElement("div",{className:o.PANEL_STACK_HEADER},u.createElement("span",null,this.maybeRenderBack()),u.createElement(ue,{className:o.HEADING,ellipsize:!0},this.props.panel.title),u.createElement("span",null)):null},e.prototype.maybeRenderBack=function(){return void 0===this.props.previousPanel?null:u.createElement(Kt,{className:o.PANEL_STACK_HEADER_BACK,icon:"chevron-left",minimal:!0,onClick:this.handleClose,small:!0,text:this.props.previousPanel.title})},e}(u.PureComponent),Mn=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={direction:"push",stack:null!=e.props.stack?e.props.stack.slice().reverse():[e.props.initialPanel]},e.handlePanelClose=function(t){var n=e.state.stack;n[0]!==t||n.length<=1||(Object(g.safeInvoke)(e.props.onClose,t),null==e.props.stack&&e.setState(function(e){return{direction:"pop",stack:e.stack.filter(function(e){return e!==t})}}))},e.handlePanelOpen=function(t){Object(g.safeInvoke)(e.props.onOpen,t),null==e.props.stack&&e.setState(function(e){return{direction:"push",stack:[t].concat(e.stack)}})},e}return c.__extends(e,t),e.prototype.componentWillReceiveProps=function(t){this.props.stack!==t.stack&&null!=this.props.stack&&null!=t.stack&&this.setState({direction:this.props.stack.length-t.stack.length<0?"push":"pop",stack:t.stack.slice().reverse()})},e.prototype.render=function(){var t=F()(o.PANEL_STACK,o.PANEL_STACK+"-"+this.state.direction,this.props.className);return u.createElement(ct.TransitionGroup,{className:t,component:"div"},this.renderCurrentPanel())},e.prototype.validateProps=function(t){if(null==t.initialPanel&&null==t.stack||null!=t.initialPanel&&null!=t.stack)throw new Error(At.x);if(null!=t.stack&&0===t.stack.length)throw new Error(At.y)},e.prototype.renderCurrentPanel=function(){var t=this.props.showPanelHeader,e=void 0===t||t,n=this.state.stack;if(0===n.length)return null;var r=n[0],i=n[1];return u.createElement(ct.CSSTransition,{classNames:o.PANEL_STACK,key:n.length,timeout:400},u.createElement(Dn,{onClose:this.handlePanelClose,onOpen:this.handlePanelOpen,panel:r,previousPanel:i,showHeader:e}))},e}(d),kn=c.__assign({},y.a,{AUTO:"auto",AUTO_END:"auto-end",AUTO_START:"auto-start"}),Sn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.animate,r=void 0===n||n,i=e.className,a=e.intent,s=e.stripes,A=void 0===s||s,l=e.value,c=F()(o.PROGRESS_BAR,o.intentClass(a),((t={})[o.PROGRESS_NO_ANIMATION]=!r,t[o.PROGRESS_NO_STRIPES]=!A,t),i),h=null==l?null:100*Object(g.clamp)(l,0,1)+"%";return u.createElement("div",{className:c},u.createElement("div",{className:o.PROGRESS_METER,style:{width:h}}))},e.displayName="Blueprint3.ProgressBar",e}(u.PureComponent),Ln={FULL:"full",START:"start",END:"end"},Rn={LOCK:"lock",PUSH:"push",NONE:"none"};function Nn(t){return(100*t).toFixed(2)+"%"}function Fn(t,e,n,r){for(var i=e<n?1:-1,o=e;o!==n+i;o+=i)t[o]=r}var Tn=["max","min","stepSize","tickSize","value"],Un=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={isMoving:!1},e.refHandlers={handle:function(t){return e.handleElement=t}},e.beginHandleMovement=function(t){document.addEventListener("mousemove",e.handleHandleMovement),document.addEventListener("mouseup",e.endHandleMovement),e.setState({isMoving:!0}),e.changeValue(e.clientToValue(e.mouseEventClientOffset(t)))},e.beginHandleTouchMovement=function(t){document.addEventListener("touchmove",e.handleHandleTouchMovement),document.addEventListener("touchend",e.endHandleTouchMovement),document.addEventListener("touchcancel",e.endHandleTouchMovement),e.setState({isMoving:!0}),e.changeValue(e.clientToValue(e.touchEventClientOffset(t)))},e.endHandleMovement=function(t){e.handleMoveEndedAt(e.mouseEventClientOffset(t))},e.endHandleTouchMovement=function(t){e.handleMoveEndedAt(e.touchEventClientOffset(t))},e.handleMoveEndedAt=function(t){e.removeDocumentEventListeners(),e.setState({isMoving:!1});var n=e.props.onRelease,r=e.changeValue(e.clientToValue(t));Object(g.safeInvoke)(n,r)},e.handleHandleMovement=function(t){e.handleMovedTo(e.mouseEventClientOffset(t))},e.handleHandleTouchMovement=function(t){e.handleMovedTo(e.touchEventClientOffset(t))},e.handleMovedTo=function(t){e.state.isMoving&&!e.props.disabled&&e.changeValue(e.clientToValue(t))},e.handleKeyDown=function(t){var n=e.props,r=n.stepSize,i=n.value,o=t.which;o===S||o===D?(e.changeValue(i-r),t.preventDefault()):o!==M&&o!==k||(e.changeValue(i+r),t.preventDefault())},e.handleKeyUp=function(t){[M,S,D,k].indexOf(t.which)>=0&&Object(g.safeInvoke)(e.props.onRelease,e.props.value)},e}return c.__extends(e,t),e.prototype.componentDidMount=function(){this.forceUpdate()},e.prototype.render=function(){var t,e=this.props,n=e.className,r=e.disabled,i=e.label,a=e.min,s=e.tickSizeRatio,A=e.value,l=e.vertical,c=this.state.isMoving,g=this.getHandleMidpointAndOffset(this.handleElement,!0).handleMidpoint,h="calc("+Nn((A-a)*s)+" - "+g+"px)",d=l?{bottom:h}:{left:h};return u.createElement("span",{className:F()(o.SLIDER_HANDLE,(t={},t[o.ACTIVE]=c,t),n),onKeyDown:r?null:this.handleKeyDown,onKeyUp:r?null:this.handleKeyUp,onMouseDown:r?null:this.beginHandleMovement,onTouchStart:r?null:this.beginHandleTouchMovement,ref:this.refHandlers.handle,style:d,tabIndex:0},null==i?null:u.createElement("span",{className:o.SLIDER_LABEL},i))},e.prototype.componentWillUnmount=function(){this.removeDocumentEventListeners()},e.prototype.clientToValue=function(t){var e=this.props,n=e.stepSize,r=e.tickSize,i=e.value,o=e.vertical;if(null==this.handleElement)return i;var a=(o?window.innerHeight-t:t)-this.getHandleElementCenterPixel(this.handleElement);return isNaN(a)?i:i+Math.round(a/(r*n))*n},e.prototype.mouseEventClientOffset=function(t){return this.props.vertical?t.clientY:t.clientX},e.prototype.touchEventClientOffset=function(t){var e=t.changedTouches[0];return this.props.vertical?e.clientY:e.clientX},e.prototype.validateProps=function(t){for(var e=0,n=Tn;e<n.length;e++){var r=n[e];if("number"!=typeof t[r])throw new Error("[Blueprint] <Handle> requires number value for "+r+" prop")}},e.prototype.changeValue=function(t,e){return void 0===e&&(e=this.props.onChange),t=this.clamp(t),isNaN(t)||this.props.value===t||Object(g.safeInvoke)(e,t),t},e.prototype.clamp=function(t){return Object(g.clamp)(t,this.props.min,this.props.max)},e.prototype.getHandleElementCenterPixel=function(t){var e=this.getHandleMidpointAndOffset(t),n=e.handleMidpoint;return e.handleOffset+n},e.prototype.getHandleMidpointAndOffset=function(t,e){if(void 0===e&&(e=!1),null==t)return{handleMidpoint:0,handleOffset:0};var n=this.props.vertical,r=t.getBoundingClientRect(),i=n?e?"width":"height":e?"height":"width",o=n?window.innerHeight-(r.top+r[i]):r.left;return{handleMidpoint:r[i]/2,handleOffset:o}},e.prototype.removeDocumentEventListeners=function(){document.removeEventListener("mousemove",this.handleHandleMovement),document.removeEventListener("mouseup",this.endHandleMovement),document.removeEventListener("touchmove",this.handleHandleTouchMovement),document.removeEventListener("touchend",this.endHandleTouchMovement),document.removeEventListener("touchcancel",this.endHandleTouchMovement)},e.displayName="Blueprint3.SliderHandle",e}(d),On=function(){return null};On.displayName="Blueprint3.MultiSliderHandle";var Gn,Hn=function(t){function e(){var e,n,r,i=null!==t&&t.apply(this,arguments)||this;return i.state={labelPrecision:(e=i.props,n=e.labelPrecision,r=e.stepSize,null==n?g.countDecimalPlaces(r):n),tickSize:0,tickSizeRatio:0},i.handleElements=[],i.addHandleRef=function(t){null!=t&&i.handleElements.push(t)},i.maybeHandleTrackClick=function(t){if(i.canHandleTrackEvent(t)){var e=i.nearestHandleForValue(i.handleElements,function(e){return e.mouseEventClientOffset(t)});e&&e.beginHandleMovement(t)}},i.maybeHandleTrackTouch=function(t){if(i.canHandleTrackEvent(t)){var e=i.nearestHandleForValue(i.handleElements,function(e){return e.touchEventClientOffset(t)});e&&e.beginHandleTouchMovement(t)}},i.canHandleTrackEvent=function(t){var e=t.target;return!i.props.disabled&&null==e.closest("."+o.SLIDER_HANDLE)},i.getHandlerForIndex=function(t,e){return function(n){g.safeInvoke(e,i.getNewHandleValues(n,t))}},i.handleChange=function(t){var e=i.handleProps.map(function(t){return t.value});pe.a(t,e)||(g.safeInvoke(i.props.onChange,t),i.handleProps.forEach(function(n,r){e[r]!==t[r]&&g.safeInvoke(n.onChange,t[r])}))},i.handleRelease=function(t){g.safeInvoke(i.props.onRelease,t),i.handleProps.forEach(function(e,n){g.safeInvoke(e.onRelease,t[n])})},i}return c.__extends(e,t),e.prototype.render=function(){var t,e=this,n=F()(o.SLIDER,((t={})[o.DISABLED]=this.props.disabled,t[o.SLIDER+"-unlabeled"]=!1===this.props.labelRenderer,t[o.VERTICAL]=this.props.vertical,t),this.props.className);return u.createElement("div",{className:n,onMouseDown:this.maybeHandleTrackClick,onTouchStart:this.maybeHandleTrackTouch},u.createElement("div",{className:o.SLIDER_TRACK,ref:function(t){return e.trackElement=t}},this.renderTracks()),u.createElement("div",{className:o.SLIDER_AXIS},this.renderLabels()),this.renderHandles())},e.prototype.componentWillMount=function(){this.handleProps=jn(this.props)},e.prototype.componentDidMount=function(){this.updateTickSize()},e.prototype.componentDidUpdate=function(){this.updateTickSize()},e.prototype.componentWillReceiveProps=function(t){this.setState({labelPrecision:this.getLabelPrecision(t)});var e=jn(t);e.length!==this.handleProps.length&&(this.handleElements=[]),this.handleProps=e},e.prototype.validateProps=function(t){if(t.stepSize<=0)throw new Error(At.L);if(t.labelStepSize<=0)throw new Error(At.K);var n=!1;if(u.Children.forEach(t.children,function(t){t&&!g.isElementOfType(t,e.Handle)&&(n=!0)}),n)throw new Error(At.o)},e.prototype.formatLabel=function(t){var e=this.props.labelRenderer;return!1===e?null:g.isFunction(e)?e(t):t.toFixed(this.state.labelPrecision)},e.prototype.renderLabels=function(){if(!1===this.props.labelRenderer)return null;for(var t=this.props,e=t.labelStepSize,n=t.max,r=t.min,i=[],a=this.state.tickSizeRatio*e,s=r,A=0;s<n||g.approxEqual(s,n);s+=e,A+=a){var l=Nn(A),c=this.props.vertical?{bottom:l}:{left:l};i.push(u.createElement("div",{className:o.SLIDER_LABEL,key:s,style:c},this.formatLabel(s)))}return i},e.prototype.renderTracks=function(){var t=Yn(this.props);t.push({value:this.props.max});for(var e={value:this.props.min},n=[],r=0;r<t.length;r++){var i=t[r];n.push(this.renderTrackFill(r,e,i)),e=i}return n},e.prototype.renderTrackFill=function(t,e,n){var r=[this.getOffsetRatio(e.value),this.getOffsetRatio(n.value)].sort(),i=r[0],a=r[1],s=Nn(i),A=Nn(1-a),l=this.props.vertical?{bottom:s,top:A,left:0}:{left:s,right:A,top:0},c=F()(o.SLIDER_PROGRESS,o.intentClass(this.getTrackIntent(e,n)));return u.createElement("div",{key:"track-"+t,className:c,style:l})},e.prototype.renderHandles=function(){var t=this,e=this.props,n=e.disabled,r=e.max,i=e.min,a=e.stepSize,s=e.vertical;return this.handleProps.map(function(e,A){var l,c=e.value,g=e.type;return u.createElement(Un,{className:F()((l={},l[o.START]=g===Ln.START,l[o.END]=g===Ln.END,l)),disabled:n,key:A+"-"+t.handleProps.length,label:t.formatLabel(c),max:r,min:i,onChange:t.getHandlerForIndex(A,t.handleChange),onRelease:t.getHandlerForIndex(A,t.handleRelease),ref:t.addHandleRef,stepSize:a,tickSize:t.state.tickSize,tickSizeRatio:t.state.tickSizeRatio,value:c,vertical:s})})},e.prototype.nearestHandleForValue=function(t,e){return function(t,e){if(0!==t.length){for(var n=t[0],r=e(n),i=1;i<t.length;i++){var o=t[i],a=e(o);a<r&&(n=o,r=a)}return n}}(t,function(t){var n=e(t),r=t.clientToValue(n),i=t.props.value;return Math.abs(r-i)})},e.prototype.getNewHandleValues=function(t,e){var n=this.handleProps.map(function(t){return t.value}),r=n.slice();r[e]=t,r.sort(function(t,e){return t-e});var i=r.indexOf(t),o=this.findFirstLockedHandleIndex(e,i);return-1!==o?(Fn(n,e,o,n[o]),n):(Fn(r,e,i,t),r)},e.prototype.findFirstLockedHandleIndex=function(t,e){for(var n=t<e?1:-1,r=t+n;r!==e+n;r+=n)if(this.handleProps[r].interactionKind!==Rn.PUSH)return r;return-1},e.prototype.getLabelPrecision=function(t){var e=t.labelPrecision,n=t.stepSize;return null==e?g.countDecimalPlaces(n):e},e.prototype.getOffsetRatio=function(t){return g.clamp((t-this.props.min)*this.state.tickSizeRatio,0,1)},e.prototype.getTrackIntent=function(t,e){return this.props.showTrackFill?void 0!==t.intentAfter?t.intentAfter:void 0!==e&&void 0!==e.intentBefore?e.intentBefore:this.props.defaultTrackIntent:m.a.NONE},e.prototype.updateTickSize=function(){if(null!=this.trackElement){var t=this.props.vertical?this.trackElement.clientHeight:this.trackElement.clientWidth,e=1/(this.props.max-this.props.min),n=t*e;this.setState({tickSize:n,tickSizeRatio:e})}},e.defaultSliderProps={disabled:!1,labelStepSize:1,max:10,min:0,showTrackFill:!0,stepSize:1,vertical:!1},e.defaultProps=c.__assign({},e.defaultSliderProps,{defaultTrackIntent:m.a.NONE}),e.displayName="Blueprint3.MultiSlider",e.Handle=On,e}(d);function jn(t){return Yn(t,function(t){return t.interactionKind!==Rn.NONE})}function Yn(t,e){var n=t.children;void 0===e&&(e=function(){return!0});var r=u.Children.map(n,function(t){return g.isElementOfType(t,Hn.Handle)&&e(t.props)?t.props:null}),i=null!=r?r:[];return(i=i.filter(function(t){return null!==t})).sort(function(t,e){return t.value-e.value}),i}!function(t){t[t.START=0]="START",t[t.END=1]="END"}(Gn||(Gn={}));var Pn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.value,n=c.__rest(t,["value"]);return u.createElement(Hn,c.__assign({},n),u.createElement(Hn.Handle,{value:e[Gn.START],type:"start",intentAfter:m.a.PRIMARY}),u.createElement(Hn.Handle,{value:e[Gn.END],type:"end"}))},e.prototype.validateProps=function(t){var e=t.value;if(null==e||null==e[Gn.START]||null==e[Gn.END])throw new Error(At.J)},e.defaultProps=c.__assign({},Hn.defaultSliderProps,{value:[0,10]}),e.displayName="Blueprint3.RangeSlider",e}(d),Kn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.initialValue,n=t.value,r=t.onChange,i=t.onRelease,o=c.__rest(t,["initialValue","value","onChange","onRelease"]);return u.createElement(Hn,c.__assign({},o),u.createElement(Hn.Handle,{value:n,intentAfter:n<e?m.a.PRIMARY:void 0,intentBefore:n>=e?m.a.PRIMARY:void 0,onChange:r,onRelease:i}),u.createElement(Hn.Handle,{value:e,interactionKind:"none"}))},e.defaultProps=c.__assign({},Hn.defaultSliderProps,{initialValue:0,value:0}),e.displayName="Blueprint3.Slider",e}(d),zn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.className,n=t.panel;return u.createElement("div",{className:F()(o.TAB_PANEL,e),role:"tablist"},n)},e.defaultProps={disabled:!1,id:void 0},e.displayName="Blueprint3.Tab",e}(u.PureComponent),Jn=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.handleClick=function(t){return e.props.onClick(e.props.id,t)},e}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.disabled,n=t.id,r=t.parentId,i=t.selected;return u.createElement("div",{"aria-controls":qn(r,n),"aria-disabled":e,"aria-expanded":i,"aria-selected":i,className:F()(o.TAB,this.props.className),"data-tab-id":n,id:Vn(r,n),onClick:e?void 0:this.handleClick,role:"tab",tabIndex:e?void 0:0},this.props.title,this.props.children)},e.displayName="Blueprint3.TabTitle",e}(u.PureComponent);function qn(t,e){return o.TAB_PANEL+"_"+t+"_"+e}function Vn(t,e){return o.TAB+"-title_"+t+"_"+e}var Wn=function(){return u.createElement("div",{className:o.FLEX_EXPANDER})},Xn="."+o.TAB,Zn=function(t){function e(e){var n=t.call(this,e)||this;n.refHandlers={tablist:function(t){return n.tablistElement=t}},n.handleKeyDown=function(t){var e=document.activeElement.closest(Xn);if(null!=e){var r=n.getTabElements().filter(function(t){return"false"===t.getAttribute("aria-disabled")}),i=r.indexOf(e),o=n.getKeyCodeDirection(t);if(i>=0&&void 0!==o){t.preventDefault();var a=r.length;r[(i+o+a)%a].focus()}}},n.handleKeyPress=function(t){var e=t.target.closest(Xn);null!=e&&R(t.which)&&(t.preventDefault(),e.click())},n.handleTabClick=function(t,e){g.safeInvoke(n.props.onChange,t,n.state.selectedTabId,e),void 0===n.props.selectedTabId&&n.setState({selectedTabId:t})},n.renderTabPanel=function(t){var e=t.props,r=e.className,i=e.panel,a=e.id,s=e.panelClassName;if(void 0!==i)return u.createElement("div",{"aria-labelledby":Vn(n.props.id,a),"aria-hidden":a!==n.state.selectedTabId,className:F()(o.TAB_PANEL,r,s),id:qn(n.props.id,a),key:a,role:"tabpanel"},i)},n.renderTabTitle=function(t){if(tr(t)){var e=t.props.id;return u.createElement(Jn,c.__assign({},t.props,{parentId:n.props.id,onClick:n.handleTabClick,selected:e===n.state.selectedTabId}))}return t};var r=n.getInitialSelectedTabId();return n.state={selectedTabId:r},n}return c.__extends(e,t),e.prototype.render=function(){var t,e,n=this.state,r=n.indicatorWrapperStyle,i=n.selectedTabId,a=u.Children.map(this.props.children,this.renderTabTitle),s=this.getTabChildren().filter(this.props.renderActiveTabPanelOnly?function(t){return t.props.id===i}:function(){return!0}).map(this.renderTabPanel),A=this.props.animate?u.createElement("div",{className:o.TAB_INDICATOR_WRAPPER,style:r},u.createElement("div",{className:o.TAB_INDICATOR})):null,l=F()(o.TABS,((t={})[o.VERTICAL]=this.props.vertical,t),this.props.className),c=F()(o.TAB_LIST,((e={})[o.LARGE]=this.props.large,e));return u.createElement("div",{className:l},u.createElement("div",{className:c,onKeyDown:this.handleKeyDown,onKeyPress:this.handleKeyPress,ref:this.refHandlers.tablist,role:"tablist"},A,a),s)},e.prototype.componentDidMount=function(){this.moveSelectionIndicator()},e.prototype.componentWillReceiveProps=function(t){var e=t.selectedTabId;void 0!==e&&this.setState({selectedTabId:e})},e.prototype.componentDidUpdate=function(t,e){if(this.state.selectedTabId!==e.selectedTabId)this.moveSelectionIndicator();else if(null!=e.selectedTabId){!pe.a(this.getTabChildrenProps(t),this.getTabChildrenProps(),pe.e)&&this.moveSelectionIndicator()}},e.prototype.getInitialSelectedTabId=function(){var t=this.props,e=t.defaultSelectedTabId,n=t.selectedTabId;if(void 0!==n)return n;if(void 0!==e)return e;var r=this.getTabChildren();return 0===r.length?void 0:r[0].props.id},e.prototype.getKeyCodeDirection=function(t){return $n(t,D,M)?-1:$n(t,k,S)?1:void 0},e.prototype.getTabChildrenProps=function(t){return void 0===t&&(t=this.props),this.getTabChildren(t).map(function(t){return t.props})},e.prototype.getTabChildren=function(t){return void 0===t&&(t=this.props),u.Children.toArray(t.children).filter(tr)},e.prototype.getTabElements=function(t){return void 0===t&&(t=""),null==this.tablistElement?[]:Array.from(this.tablistElement.querySelectorAll(Xn+t))},e.prototype.moveSelectionIndicator=function(){if(null!=this.tablistElement&&this.props.animate){var t=Xn+'[data-tab-id="'+this.state.selectedTabId+'"]',e=this.tablistElement.querySelector(t),n={display:"none"};if(null!=e){var r=e.clientHeight,i=e.clientWidth,o=e.offsetLeft,a=e.offsetTop;n={height:r,transform:"translateX("+Math.floor(o)+"px) translateY("+Math.floor(a)+"px)",width:i}}this.setState({indicatorWrapperStyle:n})}},e.Expander=Wn,e.Tab=zn,e.defaultProps={animate:!0,large:!1,renderActiveTabPanelOnly:!1,vertical:!1},e.displayName="Blueprint3.Tabs",e}(d);function $n(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return e.indexOf(t.which)>=0}function tr(t){return g.isElementOfType(t,zn)}var er=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.onRemoveClick=function(t){g.safeInvoke(e.props.onRemove,t,e.props)},e}return c.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.active,r=e.children,i=e.className,a=e.fill,s=e.icon,A=e.intent,l=e.interactive,h=e.large,d=e.minimal,f=e.multiline,p=e.onRemove,I=e.rightIcon,C=e.round,m=e.tabIndex,y=void 0===m?0:m,E=c.__rest(e,["active","children","className","fill","icon","intent","interactive","large","minimal","multiline","onRemove","rightIcon","round","tabIndex"]),v=g.isFunction(p),B=F()(o.TAG,o.intentClass(A),((t={})[o.ACTIVE]=n,t[o.FILL]=a,t[o.INTERACTIVE]=l,t[o.LARGE]=h,t[o.MINIMAL]=d,t[o.ROUND]=C,t),i),b=h||B.indexOf(o.LARGE)>=0,Q=v?u.createElement("button",{type:"button",className:o.TAG_REMOVE,onClick:this.onRemoveClick},u.createElement(Gt,{icon:"small-cross",iconSize:b?Gt.SIZE_LARGE:Gt.SIZE_STANDARD})):null;return u.createElement("span",c.__assign({},E,{className:B,tabIndex:l?y:void 0}),u.createElement(Gt,{icon:s}),!Object(g.isReactNodeEmpty)(r)&&u.createElement(ue,{className:o.FILL,ellipsize:!f,tagName:"span"},r),u.createElement(Gt,{icon:I}),Q)},e.displayName="Blueprint3.Tag",e}(u.PureComponent),nr=-1,rr=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={activeIndex:nr,inputValue:e.props.inputValue||"",isInputFocused:!1},e.refHandlers={input:function(t){e.inputElement=t,g.safeInvoke(e.props.inputRef,t)}},e.addTags=function(t,n){void 0===n&&(n="default");var r=e.props,i=r.inputValue,o=r.onAdd,a=r.onChange,s=r.values,A=e.getValues(t),l=!1!==g.safeInvoke(o,A,n)&&void 0===i;g.isFunction(a)&&(l=!1!==a(s.concat(A))&&l),l&&e.setState({inputValue:""})},e.maybeRenderTag=function(t,n){if(!t)return null;var r=e.props,i=r.large,o=r.tagProps,a=g.isFunction(o)?o(t,n):o;return u.createElement(er,c.__assign({active:n===e.state.activeIndex,"data-tag-index":n,key:t+"__"+n,large:i,onRemove:e.props.disabled?null:e.handleRemoveTag},a),t)},e.handleContainerClick=function(){null!=e.inputElement&&e.inputElement.focus()},e.handleContainerBlur=function(t){var n=t.currentTarget;requestAnimationFrame(function(){n.contains(document.activeElement)||(e.props.addOnBlur&&void 0!==e.state.inputValue&&e.state.inputValue.length>0&&e.addTags(e.state.inputValue,"blur"),e.setState({activeIndex:nr,isInputFocused:!1}))})},e.handleInputFocus=function(t){e.setState({isInputFocused:!0}),g.safeInvoke(e.props.inputProps.onFocus,t)},e.handleInputChange=function(t){e.setState({activeIndex:nr,inputValue:t.currentTarget.value}),g.safeInvoke(e.props.onInputChange,t),g.safeInvoke(e.props.inputProps.onChange,t)},e.handleInputKeyDown=function(t){var n=t.currentTarget,r=n.selectionEnd,i=n.value,o=e.state.activeIndex,a=o;if(t.which===Q&&i.length>0)e.addTags(i,"default");else if(0===r&&e.props.values.length>0)if(t.which===D||t.which===k){var s=e.getNextActiveIndex(t.which===k?1:-1);s!==o&&(t.stopPropagation(),a=s,e.setState({activeIndex:s}))}else t.which===B&&e.handleBackspaceToRemove(t);e.invokeKeyPressCallback("onKeyDown",t,a)},e.handleInputKeyUp=function(t){e.invokeKeyPressCallback("onKeyUp",t,e.state.activeIndex)},e.handleInputPaste=function(t){var n=e.props.separator,r=t.clipboardData.getData("text");e.props.addOnPaste&&0!==r.length&&!1!==n&&1!==r.split(n).length&&(t.preventDefault(),e.addTags(r,"paste"))},e.handleRemoveTag=function(t){var n=+t.currentTarget.parentElement.getAttribute("data-tag-index");e.removeIndexFromValues(n)},e}return c.__extends(e,t),e.prototype.componentWillReceiveProps=function(e){t.prototype.componentWillReceiveProps.call(this,e),e.inputValue!==this.props.inputValue&&this.setState({inputValue:e.inputValue||""})},e.prototype.render=function(){var t,e=this.props,n=e.className,r=e.disabled,i=e.fill,a=e.inputProps,s=e.intent,A=e.large,l=e.leftIcon,g=e.placeholder,h=e.values,d=F()(o.INPUT,o.TAG_INPUT,((t={})[o.ACTIVE]=this.state.isInputFocused,t[o.DISABLED]=r,t[o.FILL]=i,t[o.LARGE]=A,t),o.intentClass(s),n),f=d.indexOf(o.LARGE)>nr,p=h.some(function(t){return!!t}),I=null==g||p?a.placeholder:g;return u.createElement("div",{className:d,onBlur:this.handleContainerBlur,onClick:this.handleContainerClick},u.createElement(Gt,{className:o.TAG_INPUT_ICON,icon:l,iconSize:f?Gt.SIZE_LARGE:Gt.SIZE_STANDARD}),u.createElement("div",{className:o.TAG_INPUT_VALUES},h.map(this.maybeRenderTag),this.props.children,u.createElement("input",c.__assign({value:this.state.inputValue},a,{onFocus:this.handleInputFocus,onChange:this.handleInputChange,onKeyDown:this.handleInputKeyDown,onKeyUp:this.handleInputKeyUp,onPaste:this.handleInputPaste,placeholder:I,ref:this.refHandlers.input,className:F()(o.INPUT_GHOST,a.className),disabled:r}))),this.props.rightElement)},e.prototype.getNextActiveIndex=function(t){var e=this.state.activeIndex;return e===nr?t<0?this.findNextIndex(this.props.values.length,-1):nr:this.findNextIndex(e,t)},e.prototype.findNextIndex=function(t,e){for(var n=this.props.values,r=t+e;r>0&&r<n.length&&!n[r];)r+=e;return g.clamp(r,0,n.length)},e.prototype.getValues=function(t){var e=this.props.separator;return(!1===e?[t]:t.split(e)).map(function(t){return t.trim()}).filter(function(t){return t.length>0})},e.prototype.handleBackspaceToRemove=function(t){var e=this.state.activeIndex;this.setState({activeIndex:this.getNextActiveIndex(-1)}),this.isValidIndex(e)&&(t.stopPropagation(),this.removeIndexFromValues(e))},e.prototype.removeIndexFromValues=function(t){var e=this.props,n=e.onChange,r=e.onRemove,i=e.values;g.safeInvoke(r,i[t],t),g.isFunction(n)&&n(i.filter(function(e,n){return n!==t}))},e.prototype.invokeKeyPressCallback=function(t,e,n){g.safeInvoke(this.props[t],e,n===nr?void 0:n),g.safeInvoke(this.props.inputProps[t],e)},e.prototype.isValidIndex=function(t){return t!==nr&&t<this.props.values.length},e.displayName="Blueprint3.TagInput",e.defaultProps={addOnBlur:!1,addOnPaste:!0,inputProps:{},separator:/[,\n\r]/,tagProps:{}},e}(d),ir=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.handleActionClick=function(t){Object(g.safeInvoke)(e.props.action.onClick,t),e.triggerDismiss(!1)},e.handleCloseClick=function(){return e.triggerDismiss(!1)},e.startTimeout=function(){e.clearTimeouts(),e.props.timeout>0&&e.setTimeout(function(){return e.triggerDismiss(!0)},e.props.timeout)},e}return c.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.className,n=t.icon,r=t.intent,i=t.message;return u.createElement("div",{className:F()(o.TOAST,o.intentClass(r),e),onBlur:this.startTimeout,onFocus:this.clearTimeouts,onMouseEnter:this.clearTimeouts,onMouseLeave:this.startTimeout,tabIndex:0},u.createElement(Gt,{icon:n}),u.createElement("span",{className:o.TOAST_MESSAGE},i),u.createElement(ye,{minimal:!0},this.maybeRenderActionButton(),u.createElement(Kt,{icon:"cross",onClick:this.handleCloseClick})))},e.prototype.componentDidMount=function(){this.startTimeout()},e.prototype.componentDidUpdate=function(t){t.timeout!==this.props.timeout&&(this.props.timeout>0?this.startTimeout():this.clearTimeouts())},e.prototype.componentWillUnmount=function(){this.clearTimeouts()},e.prototype.maybeRenderActionButton=function(){var t=this.props.action;return null==t?void 0:u.createElement(zt,c.__assign({},t,{intent:void 0,onClick:this.handleActionClick}))},e.prototype.triggerDismiss=function(t){this.clearTimeouts(),Object(g.safeInvoke)(this.props.onDismiss,t)},e.defaultProps={className:"",message:"",timeout:5e3},e.displayName="Blueprint3.Toast",e}(d),or=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={toasts:[]},e.toastId=0,e.getDismissHandler=function(t){return function(n){e.dismiss(t.key,n)}},e.handleClose=function(t){t.which===x&&e.clear()},e}return c.__extends(e,t),e.create=function(t,n){void 0===n&&(n=document.body),null==t||null==t.usePortal||Object(g.isNodeEnv)("production")||console.warn(At.O);var r=document.createElement("div");n.appendChild(r);var i=T.render(u.createElement(e,c.__assign({},t,{usePortal:!1})),r);if(null==i)throw new Error(At.N);return i},e.prototype.show=function(t,e){var n=this.createToastOptions(t,e);return void 0===e||this.isNewToastKey(e)?this.setState(function(t){return{toasts:[n].concat(t.toasts)}}):this.setState(function(t){return{toasts:t.toasts.map(function(t){return t.key===e?n:t})}}),n.key},e.prototype.dismiss=function(t,e){void 0===e&&(e=!1),this.setState(function(n){return{toasts:n.toasts.filter(function(n){var r=n.key===t;return r&&Object(g.safeInvoke)(n.onDismiss,e),!r})}})},e.prototype.clear=function(){this.state.toasts.map(function(t){return Object(g.safeInvoke)(t.onDismiss,!1)}),this.setState({toasts:[]})},e.prototype.getToasts=function(){return this.state.toasts},e.prototype.render=function(){var t=F()(o.TOAST_CONTAINER,this.getPositionClasses(),this.props.className);return u.createElement(ft,{autoFocus:this.props.autoFocus,canEscapeKeyClose:this.props.canEscapeKeyClear,canOutsideClickClose:!1,className:t,enforceFocus:!1,hasBackdrop:!1,isOpen:this.state.toasts.length>0||null!=this.props.children,onClose:this.handleClose,transitionDuration:350,transitionName:o.TOAST,usePortal:this.props.usePortal},this.state.toasts.map(this.renderToast,this),this.props.children)},e.prototype.isNewToastKey=function(t){return this.state.toasts.every(function(e){return e.key!==t})},e.prototype.renderToast=function(t){return u.createElement(ir,c.__assign({},t,{onDismiss:this.getDismissHandler(t)}))},e.prototype.createToastOptions=function(t,e){return void 0===e&&(e="toast-"+this.toastId++),c.__assign({},t,{key:e})},e.prototype.getPositionClasses=function(){return this.props.position.split("-").map(function(t){return o.TOAST_CONTAINER+"-"+t.toLowerCase()})},e.displayName="Blueprint3.Toaster",e.defaultProps={autoFocus:!1,canEscapeKeyClear:!0,position:y.a.TOP,usePortal:!0},e}(d),ar=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.handleCaretClick=function(t){t.stopPropagation();var n=e.props,r=n.isExpanded,i=n.onCollapse,o=n.onExpand;Object(g.safeInvoke)(r?i:o,e,t)},e.handleClick=function(t){Object(g.safeInvoke)(e.props.onClick,e,t)},e.handleContentRef=function(t){Object(g.safeInvoke)(e.props.contentRef,e,t)},e.handleContextMenu=function(t){Object(g.safeInvoke)(e.props.onContextMenu,e,t)},e.handleDoubleClick=function(t){Object(g.safeInvoke)(e.props.onDoubleClick,e,t)},e.handleMouseEnter=function(t){Object(g.safeInvoke)(e.props.onMouseEnter,e,t)},e.handleMouseLeave=function(t){Object(g.safeInvoke)(e.props.onMouseLeave,e,t)},e}return c.__extends(e,t),e.ofType=function(){return e},e.prototype.render=function(){var t,e=this.props,n=e.children,r=e.className,i=e.disabled,a=e.icon,s=e.isExpanded,A=e.isSelected,l=e.label,g=F()(o.TREE_NODE,((t={})[o.DISABLED]=i,t[o.TREE_NODE_SELECTED]=A,t[o.TREE_NODE_EXPANDED]=s,t),r),h=F()(o.TREE_NODE_CONTENT,o.TREE_NODE_CONTENT+"-"+this.props.depth),d=!0===i?{}:{onClick:this.handleClick,onContextMenu:this.handleContextMenu,onDoubleClick:this.handleDoubleClick,onMouseEnter:this.handleMouseEnter,onMouseLeave:this.handleMouseLeave};return u.createElement("li",{className:g},u.createElement("div",c.__assign({className:h,ref:this.handleContentRef},d),this.maybeRenderCaret(),u.createElement(Gt,{className:o.TREE_NODE_ICON,icon:a}),u.createElement("span",{className:o.TREE_NODE_LABEL},l),this.maybeRenderSecondaryLabel()),u.createElement(Be,{isOpen:s},n))},e.prototype.maybeRenderCaret=function(){var t=this.props,e=t.children,n=t.isExpanded,r=t.disabled,i=t.hasCaret;if(void 0===i?u.Children.count(e)>0:i){var a=F()(o.TREE_NODE_CARET,n?o.TREE_NODE_CARET_OPEN:o.TREE_NODE_CARET_CLOSED),s=!0===r?void 0:this.handleCaretClick;return u.createElement(Gt,{className:a,onClick:s,icon:"chevron-right"})}return u.createElement("span",{className:o.TREE_NODE_CARET_NONE})},e.prototype.maybeRenderSecondaryLabel=function(){return null!=this.props.secondaryLabel?u.createElement("span",{className:o.TREE_NODE_SECONDARY_LABEL},this.props.secondaryLabel):void 0},e.displayName="Blueprint3.TreeNode",e}(u.Component),sr=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.nodeRefs={},e.handleNodeCollapse=function(t,n){e.handlerHelper(e.props.onNodeCollapse,t,n)},e.handleNodeClick=function(t,n){e.handlerHelper(e.props.onNodeClick,t,n)},e.handleContentRef=function(t,n){null!=n?e.nodeRefs[t.props.id]=n:delete e.nodeRefs[t.props.id]},e.handleNodeContextMenu=function(t,n){e.handlerHelper(e.props.onNodeContextMenu,t,n)},e.handleNodeDoubleClick=function(t,n){e.handlerHelper(e.props.onNodeDoubleClick,t,n)},e.handleNodeExpand=function(t,n){e.handlerHelper(e.props.onNodeExpand,t,n)},e.handleNodeMouseEnter=function(t,n){e.handlerHelper(e.props.onNodeMouseEnter,t,n)},e.handleNodeMouseLeave=function(t,n){e.handlerHelper(e.props.onNodeMouseLeave,t,n)},e}return c.__extends(e,t),e.ofType=function(){return e},e.nodeFromPath=function(t,n){return 1===t.length?n[t[0]]:e.nodeFromPath(t.slice(1),n[t[0]].childNodes)},e.prototype.render=function(){return u.createElement("div",{className:F()(o.TREE,this.props.className)},this.renderNodes(this.props.contents,[],o.TREE_ROOT))},e.prototype.getNodeContentElement=function(t){return this.nodeRefs[t]},e.prototype.renderNodes=function(t,e,n){var r=this;if(null==t)return null;var i=t.map(function(t,n){var i=e.concat(n),o=ar.ofType();return u.createElement(o,c.__assign({},t,{key:t.id,contentRef:r.handleContentRef,depth:i.length-1,onClick:r.handleNodeClick,onContextMenu:r.handleNodeContextMenu,onCollapse:r.handleNodeCollapse,onDoubleClick:r.handleNodeDoubleClick,onExpand:r.handleNodeExpand,onMouseEnter:r.handleNodeMouseEnter,onMouseLeave:r.handleNodeMouseLeave,path:i}),r.renderNodes(t.childNodes,i))});return u.createElement("ul",{className:F()(o.TREE_NODE_LIST,n)},i)},e.prototype.handlerHelper=function(t,n,r){Object(g.isFunction)(t)&&t(e.nodeFromPath(n.props.path,this.props.contents),n.props.path,r)},e.displayName="Blueprint3.Tree",e}(u.Component);"undefined"!=typeof window&&"undefined"!=typeof document&&n(1145);var Ar=i;n.d(e,"FocusStyleManager",function(){return l}),n.d(e,"Classes",function(){return o}),n.d(e,"Keys",function(){return r}),n.d(e,"Utils",function(){return g}),n.d(e,"AbstractComponent",function(){return h}),n.d(e,"AbstractPureComponent",function(){return d}),n.d(e,"Alignment",function(){return f.a}),n.d(e,"Boundary",function(){return p}),n.d(e,"Colors",function(){return I}),n.d(e,"Elevation",function(){return C.a}),n.d(e,"Intent",function(){return m.a}),n.d(e,"Position",function(){return y.a}),n.d(e,"isPositionHorizontal",function(){return y.c}),n.d(e,"isPositionVertical",function(){return y.d}),n.d(e,"getPositionIgnoreAngles",function(){return y.b}),n.d(e,"DISPLAYNAME_PREFIX",function(){return"Blueprint3"}),n.d(e,"removeNonHTMLProps",function(){return v}),n.d(e,"ContextMenu",function(){return Ar}),n.d(e,"Alert",function(){return Ae}),n.d(e,"Breadcrumb",function(){return le}),n.d(e,"Breadcrumbs",function(){return me}),n.d(e,"Button",function(){return Kt}),n.d(e,"AnchorButton",function(){return zt}),n.d(e,"ButtonGroup",function(){return ye}),n.d(e,"Callout",function(){return Ee}),n.d(e,"Card",function(){return ve}),n.d(e,"AnimationStates",function(){return Ie}),n.d(e,"Collapse",function(){return Be}),n.d(e,"CollapsibleList",function(){return be}),n.d(e,"ContextMenuTarget",function(){return Qe}),n.d(e,"Dialog",function(){return se}),n.d(e,"Divider",function(){return we}),n.d(e,"Drawer",function(){return xe}),n.d(e,"EditableText",function(){return Se}),n.d(e,"ControlGroup",function(){return Re}),n.d(e,"Switch",function(){return Fe}),n.d(e,"Radio",function(){return Te}),n.d(e,"Checkbox",function(){return Ue}),n.d(e,"FileInput",function(){return Oe}),n.d(e,"FormGroup",function(){return Ge}),n.d(e,"InputGroup",function(){return je}),n.d(e,"NumericInput",function(){return qe}),n.d(e,"RadioGroup",function(){return We}),n.d(e,"TextArea",function(){return Xe}),n.d(e,"H1",function(){return Vt}),n.d(e,"H2",function(){return Wt}),n.d(e,"H3",function(){return Xt}),n.d(e,"H4",function(){return Zt}),n.d(e,"H5",function(){return $t}),n.d(e,"H6",function(){return te}),n.d(e,"Blockquote",function(){return ee}),n.d(e,"Code",function(){return ne}),n.d(e,"Pre",function(){return re}),n.d(e,"Label",function(){return ie}),n.d(e,"OL",function(){return oe}),n.d(e,"UL",function(){return ae}),n.d(e,"HTMLSelect",function(){return Ze}),n.d(e,"HTMLTable",function(){return $e}),n.d(e,"Hotkey",function(){return dn}),n.d(e,"KeyCombo",function(){return hn}),n.d(e,"HotkeysTarget",function(){return vn}),n.d(e,"comboMatches",function(){return sn}),n.d(e,"getKeyCombo",function(){return cn}),n.d(e,"getKeyComboString",function(){return ln}),n.d(e,"parseKeyCombo",function(){return An}),n.d(e,"hideHotkeysDialog",function(){return Cn}),n.d(e,"setHotkeysDialogProps",function(){return In}),n.d(e,"Hotkeys",function(){return Bn}),n.d(e,"Icon",function(){return Gt}),n.d(e,"Menu",function(){return fe}),n.d(e,"MenuDivider",function(){return ce}),n.d(e,"MenuItem",function(){return ge}),n.d(e,"Navbar",function(){return xn}),n.d(e,"NavbarDivider",function(){return bn}),n.d(e,"NavbarGroup",function(){return Qn}),n.d(e,"NavbarHeading",function(){return wn}),n.d(e,"NonIdealState",function(){return _n}),n.d(e,"OverflowDirection",function(){return qt}),n.d(e,"OverflowList",function(){return Ce}),n.d(e,"Overlay",function(){return ft}),n.d(e,"Text",function(){return ue}),n.d(e,"PanelStack",function(){return Mn}),n.d(e,"PopoverInteractionKind",function(){return Mt}),n.d(e,"Popover",function(){return kt}),n.d(e,"PopoverPosition",function(){return kn}),n.d(e,"Portal",function(){return ht}),n.d(e,"ProgressBar",function(){return Sn}),n.d(e,"ResizeSensor",function(){return It}),n.d(e,"HandleType",function(){return Ln}),n.d(e,"HandleInteractionKind",function(){return Rn}),n.d(e,"MultiSlider",function(){return Hn}),n.d(e,"RangeSlider",function(){return Pn}),n.d(e,"Slider",function(){return Kn}),n.d(e,"Spinner",function(){return Yt}),n.d(e,"Tab",function(){return zn}),n.d(e,"Expander",function(){return Wn}),n.d(e,"Tabs",function(){return Zn}),n.d(e,"Tag",function(){return er}),n.d(e,"TagInput",function(){return rr}),n.d(e,"Toast",function(){return ir}),n.d(e,"Toaster",function(){return or}),n.d(e,"Tooltip",function(){return Ct}),n.d(e,"Tree",function(){return sr}),n.d(e,"TreeNode",function(){return ar})},function(t,e,n){"use strict";n.d(e,"a",function(){return a});var r=n(17),i=n(10),o=n(6);class a{constructor(t){this._currentChanged=new i.Signal(this),this._isDisposed=!1,this._widgetAdded=new i.Signal(this),this._widgetUpdated=new i.Signal(this);const e=this._focusTracker=new o.FocusTracker,n=this._pool=new r.RestorablePool(t);this.namespace=t.namespace,e.currentChanged.connect((t,e)=>{e.newValue!==this.currentWidget&&(n.current=e.newValue)},this),n.added.connect((t,e)=>{this._widgetAdded.emit(e)},this),n.currentChanged.connect((t,r)=>{null===r&&e.currentWidget?n.current=e.currentWidget:(this.onCurrentChanged(r),this._currentChanged.emit(r))},this),n.updated.connect((t,e)=>{this._widgetUpdated.emit(e)},this)}get currentChanged(){return this._currentChanged}get currentWidget(){return this._pool.current||null}get restored(){return this._pool.restored}get size(){return this._pool.size}get widgetAdded(){return this._widgetAdded}get widgetUpdated(){return this._widgetUpdated}async add(t){this._focusTracker.add(t),await this._pool.add(t),this._pool.current=t}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,this._pool.dispose(),this._focusTracker.dispose(),i.Signal.clearData(this))}find(t){return this._pool.find(t)}forEach(t){return this._pool.forEach(t)}filter(t){return this._pool.filter(t)}inject(t){return this._pool.inject(t)}has(t){return this._pool.has(t)}async restore(t){return this._pool.restore(t)}async save(t){return this._pool.save(t)}onCurrentChanged(t){}}},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=n(6);class i extends r.Widget{constructor(){super(),this.addClass("jp-Spinner"),this.node.tabIndex=-1;let t=document.createElement("div");t.className="jp-SpinnerContent",this.node.appendChild(t)}onActivateRequest(t){this.node.focus()}}},function(t,e,n){"use strict";n.d(e,"c",function(){return m}),n.d(e,"e",function(){return y}),n.d(e,"g",function(){return E}),n.d(e,"d",function(){return v}),n.d(e,"b",function(){return B}),n.d(e,"f",function(){return b}),n.d(e,"a",function(){return Q});var r=n(76),i=n(1454),o=n(1453),a=n(5),s=n(25),A=n(43),l=n(6),c=n(0);const u="jp-Toolbar",g="jp-Toolbar-item",h="jp-Toolbar-kernelName",d="jp-Toolbar-spacer",f="jp-Toolbar-kernelStatus",p="jp-FilledCircleIcon",I="jp-CircleIcon";class C extends l.PanelLayout{constructor(){super(...arguments),this._dirty=!1}onFitRequest(t){super.onFitRequest(t),this.parent.isAttached&&(Object(a.some)(this.widgets,t=>!t.isHidden)?this.parent.node.style.minHeight="var(--jp-private-toolbar-height)":this.parent.node.style.minHeight=""),this._dirty=!0,this.parent.parent&&s.MessageLoop.sendMessage(this.parent.parent,l.Widget.Msg.FitRequest),this._dirty&&s.MessageLoop.sendMessage(this.parent,l.Widget.Msg.UpdateRequest)}onUpdateRequest(t){super.onUpdateRequest(t),this.parent.isVisible&&(this._dirty=!1)}onChildShown(t){super.onChildShown(t),this.parent.fit()}onChildHidden(t){super.onChildHidden(t),this.parent.fit()}onBeforeAttach(t){super.onBeforeAttach(t),this.parent.fit()}attachWidget(t,e){super.attachWidget(t,e),this.parent.fit()}detachWidget(t,e){super.detachWidget(t,e),this.parent.fit()}}class m extends l.Widget{constructor(){super(),this.addClass(u),this.layout=new C}names(){let t=this.layout;return Object(a.map)(t.widgets,t=>w.nameProperty.get(t))}addItem(t,e){let n=this.layout;return this.insertItem(n.widgets.length,t,e)}insertItem(t,e,n){return!Object(a.find)(this.names(),t=>t===e)&&(n.addClass(g),this.layout.insertWidget(t,n),w.nameProperty.set(n,e),!0)}insertAfter(t,e,n){return this._insertRelative(t,1,e,n)}insertBefore(t,e,n){return this._insertRelative(t,0,e,n)}_insertRelative(t,e,n,r){let i=Object(a.map)(this.names(),(t,e)=>({name:t,index:e})),o=Object(a.find)(i,e=>e.name===t);return!!o&&this.insertItem(o.index+e,n,r)}handleEvent(t){switch(t.type){case"click":!this.node.contains(document.activeElement)&&this.parent&&this.parent.activate()}}onAfterAttach(t){this.node.addEventListener("click",this)}onBeforeDetach(t){this.node.removeEventListener("click",this)}}function y(t){return c.createElement(i.a,{className:t.className?t.className+" jp-ToolbarButtonComponent":"jp-ToolbarButtonComponent",disabled:!1===t.enabled,onMouseDown:e=>{0===e.button&&(e.preventDefault(),t.onClick())},onKeyDown:e=>{const{key:n}=e;"Enter"!==n&&" "!==n||t.onClick()},title:t.tooltip||t.iconLabel,minimal:!0},t.iconClassName&&c.createElement(o.a,{name:`${t.iconClassName} jp-Icon jp-Icon-16`,className:"jp-ToolbarButtonComponent-icon",fallback:!0,center:!0,kind:"toolbarButton",tag:"span"}),t.label&&c.createElement("span",{className:"jp-ToolbarButtonComponent-label"},t.label))}function E(t){return t.addClass("jp-ToolbarButton"),t}!function(t){t.createInterruptButton=function(t){return new v({iconClassName:"jp-StopIcon",onClick:()=>{t.kernel&&t.kernel.interrupt()},tooltip:"Interrupt the kernel"})},t.createRestartButton=function(t){return new v({iconClassName:"jp-RefreshIcon",onClick:()=>{t.restart()},tooltip:"Restart the kernel"})},t.createSpacerItem=function(){return new w.Spacer},t.createKernelNameItem=function(t){const e=r.a.create(c.createElement(w.KernelNameComponent,{session:t}));return e.addClass("jp-KernelName"),e},t.createKernelStatusItem=function(t){return new w.KernelStatus(t)}}(m||(m={}));class v extends r.a{constructor(t={}){super(),this.props=t,E(this)}render(){return c.createElement(y,Object.assign({},this.props))}}function B(t){return c.createElement(r.b,{signal:t.commands.commandChanged,shouldUpdate:(e,n)=>n.id===t.id&&"changed"===n.type||"many-changed"===n.type},()=>c.createElement(y,Object.assign({},w.propsFromCommand(t))))}function b(t){return t.addClass("jp-CommandToolbarButton"),t}class Q extends r.a{constructor(t){super(),this.props=t,b(this)}render(){return c.createElement(B,Object.assign({},this.props))}}var w;!function(t){t.propsFromCommand=function(t){let{commands:e,id:n}=t;const r=e.iconClass(n),i=e.iconLabel(n),o=e.label(n);let a=e.className(n);return e.isToggled(n)&&(a+=" p-mod-toggled"),e.isVisible(n)||(a+=" p-mod-hidden"),{className:a,iconClassName:r,tooltip:e.caption(n)||o||i,onClick:()=>{e.execute(n)},enabled:e.isEnabled(n),label:o}},t.nameProperty=new A.AttachedProperty({name:"name",create:()=>""}),t.noOp=function(){};t.Spacer=class extends l.Widget{constructor(){super(),this.addClass(d)}},t.KernelNameComponent=function(t){return c.createElement(r.b,{signal:t.session.kernelChanged,initialSender:t.session},e=>c.createElement(y,{className:h,onClick:t.session.selectKernel.bind(t.session),tooltip:"Switch kernel",label:e.kernelDisplayName}))};t.KernelStatus=class extends l.Widget{constructor(t){super(),this.addClass(f),this._onStatusChanged(t),t.statusChanged.connect(this._onStatusChanged,this)}_onStatusChanged(t){if(this.isDisposed)return;let e=t.status;const n=this._isBusy(e);this.toggleClass(p,n),this.toggleClass(I,!n);let r="Kernel "+e[0].toUpperCase()+e.slice(1);this.node.title=r}_isBusy(t){return"busy"===t||"starting"===t||"restarting"===t}}}(w||(w={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return l}),n.d(e,"b",function(){return u});var r=n(2),i=n(36),o=n(21),a=n(11),s=n(32),A=n(148),l=function(t){function e(e,n){var r=t.call(this)||this;return r.source=e,r.subjectFactory=n,r._refCount=0,r._isComplete=!1,r}return r.__extends(e,t),e.prototype._subscribe=function(t){return this.getSubject().subscribe(t)},e.prototype.getSubject=function(){var t=this._subject;return t&&!t.isStopped||(this._subject=this.subjectFactory()),this._subject},e.prototype.connect=function(){var t=this._connection;return t||(this._isComplete=!1,(t=this._connection=new s.a).add(this.source.subscribe(new g(this.getSubject(),this))),t.closed&&(this._connection=null,t=s.a.EMPTY)),t},e.prototype.refCount=function(){return Object(A.a)()(this)},e}(o.a),c=l.prototype,u={operator:{value:null},_refCount:{value:0,writable:!0},_subject:{value:null,writable:!0},_connection:{value:null,writable:!0},_subscribe:{value:c._subscribe},_isComplete:{value:c._isComplete,writable:!0},getSubject:{value:c.getSubject},connect:{value:c.connect},refCount:{value:c.refCount}},g=function(t){function e(e,n){var r=t.call(this,e)||this;return r.connectable=n,r}return r.__extends(e,t),e.prototype._error=function(e){this._unsubscribe(),t.prototype._error.call(this,e)},e.prototype._complete=function(){this.connectable._isComplete=!0,this._unsubscribe(),t.prototype._complete.call(this)},e.prototype._unsubscribe=function(){var t=this.connectable;if(t){this.connectable=null;var e=t._connection;t._refCount=0,t._subject=null,t._connection=null,e&&e.unsubscribe()}},e}(i.b);a.a},function(t,e,n){"use strict";n.d(e,"a",function(){return a});var r=n(2),i=n(36),o=n(81),a=function(t){function e(e){var n=t.call(this)||this;return n._value=e,n}return r.__extends(e,t),Object.defineProperty(e.prototype,"value",{get:function(){return this.getValue()},enumerable:!0,configurable:!0}),e.prototype._subscribe=function(e){var n=t.prototype._subscribe.call(this,e);return n&&!n.closed&&e.next(this._value),n},e.prototype.getValue=function(){if(this.hasError)throw this.thrownError;if(this.closed)throw new o.a;return this._value},e.prototype.next=function(e){t.prototype.next.call(this,this._value=e)},e}(i.a)},function(t,e,n){"use strict";n.d(e,"b",function(){return a}),n.d(e,"a",function(){return A});var r=n(2),i=n(11),o=n(69);function a(t,e){return void 0===e&&(e=0),function(n){return n.lift(new s(t,e))}}var s=function(){function t(t,e){void 0===e&&(e=0),this.scheduler=t,this.delay=e}return t.prototype.call=function(t,e){return e.subscribe(new A(t,this.scheduler,this.delay))},t}(),A=function(t){function e(e,n,r){void 0===r&&(r=0);var i=t.call(this,e)||this;return i.scheduler=n,i.delay=r,i}return r.__extends(e,t),e.dispatch=function(t){var e=t.notification,n=t.destination;e.observe(n),this.unsubscribe()},e.prototype.scheduleMessage=function(t){this.destination.add(this.scheduler.schedule(e.dispatch,this.delay,new l(t,this.destination)))},e.prototype._next=function(t){this.scheduleMessage(o.a.createNext(t))},e.prototype._error=function(t){this.scheduleMessage(o.a.createError(t)),this.unsubscribe()},e.prototype._complete=function(){this.scheduleMessage(o.a.createComplete()),this.unsubscribe()},e}(i.a),l=function(){return function(t,e){this.notification=t,this.destination=e}}()},function(t,e,n){"use strict";function r(){return Error.call(this),this.message="Timeout has occurred",this.name="TimeoutError",this}n.d(e,"a",function(){return i}),r.prototype=Object.create(Error.prototype);var i=r},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=n(152);function i(){return Object(r.a)(1)}},function(t,e,n){"use strict";n.d(e,"a",function(){return s});var r=n(21),i=n(45),o=n(152),a=n(90);function s(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];var n=Number.POSITIVE_INFINITY,s=null,A=t[t.length-1];return Object(i.a)(A)?(s=t.pop(),t.length>1&&"number"==typeof t[t.length-1]&&(n=t.pop())):"number"==typeof A&&(n=t.pop()),null===s&&1===t.length&&t[0]instanceof r.a?t[0]:Object(o.a)(n)(Object(a.a)(t,s))}},function(t,e,n){"use strict";n.d(e,"a",function(){return A});var r=n(2),i=n(38),o=n(90),a=n(26),s=n(24);function A(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];if(1===t.length){if(!Object(i.a)(t[0]))return t[0];t=t[0]}return Object(o.a)(t,void 0).lift(new l)}var l=function(){function t(){}return t.prototype.call=function(t,e){return e.subscribe(new c(t))},t}(),c=function(t){function e(e){var n=t.call(this,e)||this;return n.hasFirst=!1,n.observables=[],n.subscriptions=[],n}return r.__extends(e,t),e.prototype._next=function(t){this.observables.push(t)},e.prototype._complete=function(){var t=this.observables,e=t.length;if(0===e)this.destination.complete();else{for(var n=0;n<e&&!this.hasFirst;n++){var r=t[n],i=Object(s.a)(this,r,r,n);this.subscriptions&&this.subscriptions.push(i),this.add(i)}this.observables=null}},e.prototype.notifyNext=function(t,e,n,r,i){if(!this.hasFirst){this.hasFirst=!0;for(var o=0;o<this.subscriptions.length;o++)if(o!==n){var a=this.subscriptions[o];a.unsubscribe(),this.remove(a)}this.subscriptions=null}this.destination.next(e)},e}(a.a)},function(t,e,n){"use strict";n.d(e,"a",function(){return s});var r=n(21),i=n(39),o=n(95),a=n(45);function s(t,e,n){void 0===t&&(t=0);var s=-1;return Object(o.a)(e)?s=Number(e)<1?1:Number(e):Object(a.a)(e)&&(n=e),Object(a.a)(n)||(n=i.a),new r.a(function(e){var r=Object(o.a)(t)?t:+t-n.now();return n.schedule(A,r,{index:0,period:s,subscriber:e})})}function A(t){var e=t.index,n=t.period,r=t.subscriber;if(r.next(e),!r.closed){if(-1===n)return r.complete();t.index=e+1,this.schedule(t,n)}}},function(t,e,n){"use strict";e.__esModule=!0;var r=a(n(1194)),i=a(n(1204)),o="function"==typeof i.default&&"symbol"==typeof r.default?function(t){return typeof t}:function(t){return t&&"function"==typeof i.default&&t.constructor===i.default&&t!==i.default.prototype?"symbol":typeof t};function a(t){return t&&t.__esModule?t:{default:t}}e.default="function"==typeof i.default&&"symbol"===o(r.default)?function(t){return void 0===t?"undefined":o(t)}:function(t){return t&&"function"==typeof i.default&&t.constructor===i.default&&t!==i.default.prototype?"symbol":void 0===t?"undefined":o(t)}},function(t,e,n){"use strict";var r=n(21),i=n(32),o=n(79);var a=n(159),s=n(75);var A=n(261),l=n(260);function c(t,e){if(null!=t){if(function(t){return t&&"function"==typeof t[o.a]}(t))return function(t,e){return new r.a(function(n){var r=new i.a;return r.add(e.schedule(function(){var i=t[o.a]();r.add(i.subscribe({next:function(t){r.add(e.schedule(function(){return n.next(t)}))},error:function(t){r.add(e.schedule(function(){return n.error(t)}))},complete:function(){r.add(e.schedule(function(){return n.complete()}))}}))})),r})}(t,e);if(Object(A.a)(t))return function(t,e){return new r.a(function(n){var r=new i.a;return r.add(e.schedule(function(){return t.then(function(t){r.add(e.schedule(function(){n.next(t),r.add(e.schedule(function(){return n.complete()}))}))},function(t){r.add(e.schedule(function(){return n.error(t)}))})})),r})}(t,e);if(Object(l.a)(t))return Object(a.a)(t,e);if(function(t){return t&&"function"==typeof t[s.a]}(t)||"string"==typeof t)return function(t,e){if(!t)throw new Error("Iterable cannot be null");return new r.a(function(n){var r,o=new i.a;return o.add(function(){r&&"function"==typeof r.return&&r.return()}),o.add(e.schedule(function(){r=t[s.a](),o.add(e.schedule(function(){if(!n.closed){var t,e;try{var i=r.next();t=i.value,e=i.done}catch(t){return void n.error(t)}e?n.complete():(n.next(t),this.schedule())}}))})),o})}(t,e)}throw new TypeError((null!==t&&typeof t||t)+" is not observable")}n.d(e,"a",function(){return c})},function(t,e,n){"use strict";var r=n(2),i=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.scheduler=e,r.work=n,r}return r.__extends(e,t),e.prototype.schedule=function(e,n){return void 0===n&&(n=0),n>0?t.prototype.schedule.call(this,e,n):(this.delay=n,this.state=e,this.scheduler.flush(this),this)},e.prototype.execute=function(e,n){return n>0||this.closed?t.prototype.execute.call(this,e,n):this._execute(e,n)},e.prototype.requestAsyncId=function(e,n,r){return void 0===r&&(r=0),null!==r&&r>0||null===r&&this.delay>0?t.prototype.requestAsyncId.call(this,e,n,r):e.flush(this)},e}(n(93).a),o=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.__extends(e,t),e}(n(91).a);n.d(e,"a",function(){return a});var a=new o(i)},function(t,e,n){"use strict";(function(t){var n=function(){if("undefined"!=typeof Map)return Map;function t(t,e){var n=-1;return t.some(function(t,r){return t[0]===e&&(n=r,!0)}),n}return function(){function e(){this.__entries__=[]}return Object.defineProperty(e.prototype,"size",{get:function(){return this.__entries__.length},enumerable:!0,configurable:!0}),e.prototype.get=function(e){var n=t(this.__entries__,e),r=this.__entries__[n];return r&&r[1]},e.prototype.set=function(e,n){var r=t(this.__entries__,e);~r?this.__entries__[r][1]=n:this.__entries__.push([e,n])},e.prototype.delete=function(e){var n=this.__entries__,r=t(n,e);~r&&n.splice(r,1)},e.prototype.has=function(e){return!!~t(this.__entries__,e)},e.prototype.clear=function(){this.__entries__.splice(0)},e.prototype.forEach=function(t,e){void 0===e&&(e=null);for(var n=0,r=this.__entries__;n<r.length;n++){var i=r[n];t.call(e,i[1],i[0])}},e}()}(),r="undefined"!=typeof window&&"undefined"!=typeof document&&window.document===document,i=void 0!==t&&t.Math===Math?t:"undefined"!=typeof self&&self.Math===Math?self:"undefined"!=typeof window&&window.Math===Math?window:Function("return this")(),o="function"==typeof requestAnimationFrame?requestAnimationFrame.bind(i):function(t){return setTimeout(function(){return t(Date.now())},1e3/60)},a=2;var s=20,A=["top","right","bottom","left","width","height","size","weight"],l="undefined"!=typeof MutationObserver,c=function(){function t(){this.connected_=!1,this.mutationEventsAdded_=!1,this.mutationsObserver_=null,this.observers_=[],this.onTransitionEnd_=this.onTransitionEnd_.bind(this),this.refresh=function(t,e){var n=!1,r=!1,i=0;function s(){n&&(n=!1,t()),r&&l()}function A(){o(s)}function l(){var t=Date.now();if(n){if(t-i<a)return;r=!0}else n=!0,r=!1,setTimeout(A,e);i=t}return l}(this.refresh.bind(this),s)}return t.prototype.addObserver=function(t){~this.observers_.indexOf(t)||this.observers_.push(t),this.connected_||this.connect_()},t.prototype.removeObserver=function(t){var e=this.observers_,n=e.indexOf(t);~n&&e.splice(n,1),!e.length&&this.connected_&&this.disconnect_()},t.prototype.refresh=function(){this.updateObservers_()&&this.refresh()},t.prototype.updateObservers_=function(){var t=this.observers_.filter(function(t){return t.gatherActive(),t.hasActive()});return t.forEach(function(t){return t.broadcastActive()}),t.length>0},t.prototype.connect_=function(){r&&!this.connected_&&(document.addEventListener("transitionend",this.onTransitionEnd_),window.addEventListener("resize",this.refresh),l?(this.mutationsObserver_=new MutationObserver(this.refresh),this.mutationsObserver_.observe(document,{attributes:!0,childList:!0,characterData:!0,subtree:!0})):(document.addEventListener("DOMSubtreeModified",this.refresh),this.mutationEventsAdded_=!0),this.connected_=!0)},t.prototype.disconnect_=function(){r&&this.connected_&&(document.removeEventListener("transitionend",this.onTransitionEnd_),window.removeEventListener("resize",this.refresh),this.mutationsObserver_&&this.mutationsObserver_.disconnect(),this.mutationEventsAdded_&&document.removeEventListener("DOMSubtreeModified",this.refresh),this.mutationsObserver_=null,this.mutationEventsAdded_=!1,this.connected_=!1)},t.prototype.onTransitionEnd_=function(t){var e=t.propertyName,n=void 0===e?"":e;A.some(function(t){return!!~n.indexOf(t)})&&this.refresh()},t.getInstance=function(){return this.instance_||(this.instance_=new t),this.instance_},t.instance_=null,t}(),u=function(t,e){for(var n=0,r=Object.keys(e);n<r.length;n++){var i=r[n];Object.defineProperty(t,i,{value:e[i],enumerable:!1,writable:!1,configurable:!0})}return t},g=function(t){return t&&t.ownerDocument&&t.ownerDocument.defaultView||i},h=m(0,0,0,0);function d(t){return parseFloat(t)||0}function f(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return e.reduce(function(e,n){return e+d(t["border-"+n+"-width"])},0)}function p(t){var e=t.clientWidth,n=t.clientHeight;if(!e&&!n)return h;var r=g(t).getComputedStyle(t),i=function(t){for(var e={},n=0,r=["top","right","bottom","left"];n<r.length;n++){var i=r[n],o=t["padding-"+i];e[i]=d(o)}return e}(r),o=i.left+i.right,a=i.top+i.bottom,s=d(r.width),A=d(r.height);if("border-box"===r.boxSizing&&(Math.round(s+o)!==e&&(s-=f(r,"left","right")+o),Math.round(A+a)!==n&&(A-=f(r,"top","bottom")+a)),!function(t){return t===g(t).document.documentElement}(t)){var l=Math.round(s+o)-e,c=Math.round(A+a)-n;1!==Math.abs(l)&&(s-=l),1!==Math.abs(c)&&(A-=c)}return m(i.left,i.top,s,A)}var I="undefined"!=typeof SVGGraphicsElement?function(t){return t instanceof g(t).SVGGraphicsElement}:function(t){return t instanceof g(t).SVGElement&&"function"==typeof t.getBBox};function C(t){return r?I(t)?function(t){var e=t.getBBox();return m(0,0,e.width,e.height)}(t):p(t):h}function m(t,e,n,r){return{x:t,y:e,width:n,height:r}}var y=function(){function t(t){this.broadcastWidth=0,this.broadcastHeight=0,this.contentRect_=m(0,0,0,0),this.target=t}return t.prototype.isActive=function(){var t=C(this.target);return this.contentRect_=t,t.width!==this.broadcastWidth||t.height!==this.broadcastHeight},t.prototype.broadcastRect=function(){var t=this.contentRect_;return this.broadcastWidth=t.width,this.broadcastHeight=t.height,t},t}(),E=function(t,e){var n,r,i,o,a,s,A,l=(r=(n=e).x,i=n.y,o=n.width,a=n.height,s="undefined"!=typeof DOMRectReadOnly?DOMRectReadOnly:Object,A=Object.create(s.prototype),u(A,{x:r,y:i,width:o,height:a,top:i,right:r+o,bottom:a+i,left:r}),A);u(this,{target:t,contentRect:l})},v=function(){function t(t,e,r){if(this.activeObservations_=[],this.observations_=new n,"function"!=typeof t)throw new TypeError("The callback provided as parameter 1 is not a function.");this.callback_=t,this.controller_=e,this.callbackCtx_=r}return t.prototype.observe=function(t){if(!arguments.length)throw new TypeError("1 argument required, but only 0 present.");if("undefined"!=typeof Element&&Element instanceof Object){if(!(t instanceof g(t).Element))throw new TypeError('parameter 1 is not of type "Element".');var e=this.observations_;e.has(t)||(e.set(t,new y(t)),this.controller_.addObserver(this),this.controller_.refresh())}},t.prototype.unobserve=function(t){if(!arguments.length)throw new TypeError("1 argument required, but only 0 present.");if("undefined"!=typeof Element&&Element instanceof Object){if(!(t instanceof g(t).Element))throw new TypeError('parameter 1 is not of type "Element".');var e=this.observations_;e.has(t)&&(e.delete(t),e.size||this.controller_.removeObserver(this))}},t.prototype.disconnect=function(){this.clearActive(),this.observations_.clear(),this.controller_.removeObserver(this)},t.prototype.gatherActive=function(){var t=this;this.clearActive(),this.observations_.forEach(function(e){e.isActive()&&t.activeObservations_.push(e)})},t.prototype.broadcastActive=function(){if(this.hasActive()){var t=this.callbackCtx_,e=this.activeObservations_.map(function(t){return new E(t.target,t.broadcastRect())});this.callback_.call(t,e,t),this.clearActive()}},t.prototype.clearActive=function(){this.activeObservations_.splice(0)},t.prototype.hasActive=function(){return this.activeObservations_.length>0},t}(),B="undefined"!=typeof WeakMap?new WeakMap:new n,b=function t(e){if(!(this instanceof t))throw new TypeError("Cannot call a class as a function.");if(!arguments.length)throw new TypeError("1 argument required, but only 0 present.");var n=c.getInstance(),r=new v(e,n,this);B.set(this,r)};["observe","unobserve","disconnect"].forEach(function(t){b.prototype[t]=function(){var e;return(e=B.get(this))[t].apply(e,arguments)}});var Q=void 0!==i.ResizeObserver?i.ResizeObserver:b;e.a=Q}).call(this,n(37))},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=n(2),i=function(t){function e(e,n){var r=t.call(this)||this;return r.subject=e,r.subscriber=n,r.closed=!1,r}return r.__extends(e,t),e.prototype.unsubscribe=function(){if(!this.closed){this.closed=!0;var t=this.subject,e=t.observers;if(this.subject=null,e&&0!==e.length&&!t.isStopped&&!t.closed){var n=e.indexOf(this.subscriber);-1!==n&&e.splice(n,1)}}},e}(n(32).a)},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r=function(t){return function(e){for(var n=0,r=t.length;n<r&&!e.closed;n++)e.next(t[n]);e.complete()}}},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r=function(t){return t&&"number"==typeof t.length&&"function"!=typeof t}},function(t,e,n){"use strict";function r(t){return!!t&&"function"!=typeof t.subscribe&&"function"==typeof t.then}n.d(e,"a",function(){return r})},function(t,e,n){"use strict";function r(t,e){function n(){return!n.pred.apply(n.thisArg,arguments)}return n.pred=t,n.thisArg=e,n}n.d(e,"a",function(){return r})},function(t,e,n){"use strict";t.exports=n(1271)},function(t,e,n){(function(r){var i,o,a;a="object"==typeof self&&self.self==self&&self||"object"==typeof r&&r.global==r&&r,i=[n(22),n(44),e],void 0===(o=function(t,e,n){a.Backbone=function(t,e,n,r){var i=t.Backbone,o=Array.prototype.slice;e.VERSION="1.2.3",e.$=r,e.noConflict=function(){return t.Backbone=i,this},e.emulateHTTP=!1,e.emulateJSON=!1;var a=function(t,e,r){n.each(e,function(e,i){n[i]&&(t.prototype[i]=function(t,e,r){switch(t){case 1:return function(){return n[e](this[r])};case 2:return function(t){return n[e](this[r],t)};case 3:return function(t,i){return n[e](this[r],s(t,this),i)};case 4:return function(t,i,o){return n[e](this[r],s(t,this),i,o)};default:return function(){var t=o.call(arguments);return t.unshift(this[r]),n[e].apply(n,t)}}}(e,i,r))})},s=function(t,e){return n.isFunction(t)?t:n.isObject(t)&&!e._isModel(t)?A(t):n.isString(t)?function(e){return e.get(t)}:t},A=function(t){var e=n.matches(t);return function(t){return e(t.attributes)}},l=e.Events={},c=/\s+/,u=function(t,e,r,i,o){var a,s=0;if(r&&"object"==typeof r){void 0!==i&&"context"in o&&void 0===o.context&&(o.context=i);for(a=n.keys(r);s<a.length;s++)e=u(t,e,a[s],r[a[s]],o)}else if(r&&c.test(r))for(a=r.split(c);s<a.length;s++)e=t(e,a[s],i,o);else e=t(e,r,i,o);return e};l.on=function(t,e,n){return g(this,t,e,n)};var g=function(t,e,n,r,i){if(t._events=u(h,t._events||{},e,n,{context:r,ctx:t,listening:i}),i){var o=t._listeners||(t._listeners={});o[i.id]=i}return t};l.listenTo=function(t,e,r){if(!t)return this;var i=t._listenId||(t._listenId=n.uniqueId("l")),o=this._listeningTo||(this._listeningTo={}),a=o[i];if(!a){var s=this._listenId||(this._listenId=n.uniqueId("l"));a=o[i]={obj:t,objId:i,id:s,listeningTo:o,count:0}}return g(t,e,r,this,a),this};var h=function(t,e,n,r){if(n){var i=t[e]||(t[e]=[]),o=r.context,a=r.ctx,s=r.listening;s&&s.count++,i.push({callback:n,context:o,ctx:o||a,listening:s})}return t};l.off=function(t,e,n){return this._events?(this._events=u(d,this._events,t,e,{context:n,listeners:this._listeners}),this):this},l.stopListening=function(t,e,r){var i=this._listeningTo;if(!i)return this;for(var o=t?[t._listenId]:n.keys(i),a=0;a<o.length;a++){var s=i[o[a]];if(!s)break;s.obj.off(e,r,this)}return n.isEmpty(i)&&(this._listeningTo=void 0),this};var d=function(t,e,r,i){if(t){var o,a=0,s=i.context,A=i.listeners;if(e||r||s){for(var l=e?[e]:n.keys(t);a<l.length;a++){e=l[a];var c=t[e];if(!c)break;for(var u=[],g=0;g<c.length;g++){var h=c[g];r&&r!==h.callback&&r!==h.callback._callback||s&&s!==h.context?u.push(h):(o=h.listening)&&0==--o.count&&(delete A[o.id],delete o.listeningTo[o.objId])}u.length?t[e]=u:delete t[e]}return n.size(t)?t:void 0}for(var d=n.keys(A);a<d.length;a++)o=A[d[a]],delete A[o.id],delete o.listeningTo[o.objId]}};l.once=function(t,e,r){var i=u(f,{},t,e,n.bind(this.off,this));return this.on(i,void 0,r)},l.listenToOnce=function(t,e,r){var i=u(f,{},e,r,n.bind(this.stopListening,this,t));return this.listenTo(t,i)};var f=function(t,e,r,i){if(r){var o=t[e]=n.once(function(){i(e,o),r.apply(this,arguments)});o._callback=r}return t};l.trigger=function(t){if(!this._events)return this;for(var e=Math.max(0,arguments.length-1),n=Array(e),r=0;r<e;r++)n[r]=arguments[r+1];return u(p,this._events,t,void 0,n),this};var p=function(t,e,n,r){if(t){var i=t[e],o=t.all;i&&o&&(o=o.slice()),i&&I(i,r),o&&I(o,[e].concat(r))}return t},I=function(t,e){var n,r=-1,i=t.length,o=e[0],a=e[1],s=e[2];switch(e.length){case 0:for(;++r<i;)(n=t[r]).callback.call(n.ctx);return;case 1:for(;++r<i;)(n=t[r]).callback.call(n.ctx,o);return;case 2:for(;++r<i;)(n=t[r]).callback.call(n.ctx,o,a);return;case 3:for(;++r<i;)(n=t[r]).callback.call(n.ctx,o,a,s);return;default:for(;++r<i;)(n=t[r]).callback.apply(n.ctx,e);return}};l.bind=l.on,l.unbind=l.off,n.extend(e,l);var C=e.Model=function(t,e){var r=t||{};e||(e={}),this.cid=n.uniqueId(this.cidPrefix),this.attributes={},e.collection&&(this.collection=e.collection),e.parse&&(r=this.parse(r,e)||{}),r=n.defaults({},r,n.result(this,"defaults")),this.set(r,e),this.changed={},this.initialize.apply(this,arguments)};n.extend(C.prototype,l,{changed:null,validationError:null,idAttribute:"id",cidPrefix:"c",initialize:function(){},toJSON:function(t){return n.clone(this.attributes)},sync:function(){return e.sync.apply(this,arguments)},get:function(t){return this.attributes[t]},escape:function(t){return n.escape(this.get(t))},has:function(t){return null!=this.get(t)},matches:function(t){return!!n.iteratee(t,this)(this.attributes)},set:function(t,e,r){if(null==t)return this;var i;if("object"==typeof t?(i=t,r=e):(i={})[t]=e,r||(r={}),!this._validate(i,r))return!1;var o=r.unset,a=r.silent,s=[],A=this._changing;this._changing=!0,A||(this._previousAttributes=n.clone(this.attributes),this.changed={});var l=this.attributes,c=this.changed,u=this._previousAttributes;for(var g in i)e=i[g],n.isEqual(l[g],e)||s.push(g),n.isEqual(u[g],e)?delete c[g]:c[g]=e,o?delete l[g]:l[g]=e;if(this.id=this.get(this.idAttribute),!a){s.length&&(this._pending=r);for(var h=0;h<s.length;h++)this.trigger("change:"+s[h],this,l[s[h]],r)}if(A)return this;if(!a)for(;this._pending;)r=this._pending,this._pending=!1,this.trigger("change",this,r);return this._pending=!1,this._changing=!1,this},unset:function(t,e){return this.set(t,void 0,n.extend({},e,{unset:!0}))},clear:function(t){var e={};for(var r in this.attributes)e[r]=void 0;return this.set(e,n.extend({},t,{unset:!0}))},hasChanged:function(t){return null==t?!n.isEmpty(this.changed):n.has(this.changed,t)},changedAttributes:function(t){if(!t)return!!this.hasChanged()&&n.clone(this.changed);var e=this._changing?this._previousAttributes:this.attributes,r={};for(var i in t){var o=t[i];n.isEqual(e[i],o)||(r[i]=o)}return!!n.size(r)&&r},previous:function(t){return null!=t&&this._previousAttributes?this._previousAttributes[t]:null},previousAttributes:function(){return n.clone(this._previousAttributes)},fetch:function(t){t=n.extend({parse:!0},t);var e=this,r=t.success;return t.success=function(n){var i=t.parse?e.parse(n,t):n;if(!e.set(i,t))return!1;r&&r.call(t.context,e,n,t),e.trigger("sync",e,n,t)},T(this,t),this.sync("read",this,t)},save:function(t,e,r){var i;null==t||"object"==typeof t?(i=t,r=e):(i={})[t]=e;var o=(r=n.extend({validate:!0,parse:!0},r)).wait;if(i&&!o){if(!this.set(i,r))return!1}else if(!this._validate(i,r))return!1;var a=this,s=r.success,A=this.attributes;r.success=function(t){a.attributes=A;var e=r.parse?a.parse(t,r):t;if(o&&(e=n.extend({},i,e)),e&&!a.set(e,r))return!1;s&&s.call(r.context,a,t,r),a.trigger("sync",a,t,r)},T(this,r),i&&o&&(this.attributes=n.extend({},A,i));var l=this.isNew()?"create":r.patch?"patch":"update";"patch"!==l||r.attrs||(r.attrs=i);var c=this.sync(l,this,r);return this.attributes=A,c},destroy:function(t){t=t?n.clone(t):{};var e=this,r=t.success,i=t.wait,o=function(){e.stopListening(),e.trigger("destroy",e,e.collection,t)};t.success=function(n){i&&o(),r&&r.call(t.context,e,n,t),e.isNew()||e.trigger("sync",e,n,t)};var a=!1;return this.isNew()?n.defer(t.success):(T(this,t),a=this.sync("delete",this,t)),i||o(),a},url:function(){var t=n.result(this,"urlRoot")||n.result(this.collection,"url")||F();if(this.isNew())return t;var e=this.get(this.idAttribute);return t.replace(/[^\/]$/,"$&/")+encodeURIComponent(e)},parse:function(t,e){return t},clone:function(){return new this.constructor(this.attributes)},isNew:function(){return!this.has(this.idAttribute)},isValid:function(t){return this._validate({},n.defaults({validate:!0},t))},_validate:function(t,e){if(!e.validate||!this.validate)return!0;t=n.extend({},this.attributes,t);var r=this.validationError=this.validate(t,e)||null;return!r||(this.trigger("invalid",this,r,n.extend(e,{validationError:r})),!1)}}),a(C,{keys:1,values:1,pairs:1,invert:1,pick:0,omit:0,chain:1,isEmpty:1},"attributes");var m=e.Collection=function(t,e){e||(e={}),e.model&&(this.model=e.model),void 0!==e.comparator&&(this.comparator=e.comparator),this._reset(),this.initialize.apply(this,arguments),t&&this.reset(t,n.extend({silent:!0},e))},y={add:!0,remove:!0,merge:!0},E={add:!0,remove:!1},v=function(t,e,n){n=Math.min(Math.max(n,0),t.length);for(var r=Array(t.length-n),i=e.length,o=0;o<r.length;o++)r[o]=t[o+n];for(o=0;o<i;o++)t[o+n]=e[o];for(o=0;o<r.length;o++)t[o+i+n]=r[o]};n.extend(m.prototype,l,{model:C,initialize:function(){},toJSON:function(t){return this.map(function(e){return e.toJSON(t)})},sync:function(){return e.sync.apply(this,arguments)},add:function(t,e){return this.set(t,n.extend({merge:!1},e,E))},remove:function(t,e){e=n.extend({},e);var r=!n.isArray(t);t=r?[t]:n.clone(t);var i=this._removeModels(t,e);return!e.silent&&i&&this.trigger("update",this,e),r?i[0]:i},set:function(t,e){if(null!=t){(e=n.defaults({},e,y)).parse&&!this._isModel(t)&&(t=this.parse(t,e));var r=!n.isArray(t);t=r?[t]:t.slice();var i=e.at;null!=i&&(i=+i),i<0&&(i+=this.length+1);for(var o,a=[],s=[],A=[],l={},c=e.add,u=e.merge,g=e.remove,h=!1,d=this.comparator&&null==i&&!1!==e.sort,f=n.isString(this.comparator)?this.comparator:null,p=0;p<t.length;p++){o=t[p];var I=this.get(o);if(I){if(u&&o!==I){var C=this._isModel(o)?o.attributes:o;e.parse&&(C=I.parse(C,e)),I.set(C,e),d&&!h&&(h=I.hasChanged(f))}l[I.cid]||(l[I.cid]=!0,a.push(I)),t[p]=I}else c&&(o=t[p]=this._prepareModel(o,e))&&(s.push(o),this._addReference(o,e),l[o.cid]=!0,a.push(o))}if(g){for(p=0;p<this.length;p++)o=this.models[p],l[o.cid]||A.push(o);A.length&&this._removeModels(A,e)}var m=!1,E=!d&&c&&g;if(a.length&&E?(m=this.length!=a.length||n.some(this.models,function(t,e){return t!==a[e]}),this.models.length=0,v(this.models,a,0),this.length=this.models.length):s.length&&(d&&(h=!0),v(this.models,s,null==i?this.length:i),this.length=this.models.length),h&&this.sort({silent:!0}),!e.silent){for(p=0;p<s.length;p++)null!=i&&(e.index=i+p),(o=s[p]).trigger("add",o,this,e);(h||m)&&this.trigger("sort",this,e),(s.length||A.length)&&this.trigger("update",this,e)}return r?t[0]:t}},reset:function(t,e){e=e?n.clone(e):{};for(var r=0;r<this.models.length;r++)this._removeReference(this.models[r],e);return e.previousModels=this.models,this._reset(),t=this.add(t,n.extend({silent:!0},e)),e.silent||this.trigger("reset",this,e),t},push:function(t,e){return this.add(t,n.extend({at:this.length},e))},pop:function(t){var e=this.at(this.length-1);return this.remove(e,t)},unshift:function(t,e){return this.add(t,n.extend({at:0},e))},shift:function(t){var e=this.at(0);return this.remove(e,t)},slice:function(){return o.apply(this.models,arguments)},get:function(t){if(null!=t){var e=this.modelId(this._isModel(t)?t.attributes:t);return this._byId[t]||this._byId[e]||this._byId[t.cid]}},at:function(t){return t<0&&(t+=this.length),this.models[t]},where:function(t,e){return this[e?"find":"filter"](t)},findWhere:function(t){return this.where(t,!0)},sort:function(t){var e=this.comparator;if(!e)throw new Error("Cannot sort a set without a comparator");t||(t={});var r=e.length;return n.isFunction(e)&&(e=n.bind(e,this)),1===r||n.isString(e)?this.models=this.sortBy(e):this.models.sort(e),t.silent||this.trigger("sort",this,t),this},pluck:function(t){return n.invoke(this.models,"get",t)},fetch:function(t){var e=(t=n.extend({parse:!0},t)).success,r=this;return t.success=function(n){var i=t.reset?"reset":"set";r[i](n,t),e&&e.call(t.context,r,n,t),r.trigger("sync",r,n,t)},T(this,t),this.sync("read",this,t)},create:function(t,e){var r=(e=e?n.clone(e):{}).wait;if(!(t=this._prepareModel(t,e)))return!1;r||this.add(t,e);var i=this,o=e.success;return e.success=function(t,e,n){r&&i.add(t,n),o&&o.call(n.context,t,e,n)},t.save(null,e),t},parse:function(t,e){return t},clone:function(){return new this.constructor(this.models,{model:this.model,comparator:this.comparator})},modelId:function(t){return t[this.model.prototype.idAttribute||"id"]},_reset:function(){this.length=0,this.models=[],this._byId={}},_prepareModel:function(t,e){if(this._isModel(t))return t.collection||(t.collection=this),t;(e=e?n.clone(e):{}).collection=this;var r=new this.model(t,e);return r.validationError?(this.trigger("invalid",this,r.validationError,e),!1):r},_removeModels:function(t,e){for(var n=[],r=0;r<t.length;r++){var i=this.get(t[r]);if(i){var o=this.indexOf(i);this.models.splice(o,1),this.length--,e.silent||(e.index=o,i.trigger("remove",i,this,e)),n.push(i),this._removeReference(i,e)}}return!!n.length&&n},_isModel:function(t){return t instanceof C},_addReference:function(t,e){this._byId[t.cid]=t;var n=this.modelId(t.attributes);null!=n&&(this._byId[n]=t),t.on("all",this._onModelEvent,this)},_removeReference:function(t,e){delete this._byId[t.cid];var n=this.modelId(t.attributes);null!=n&&delete this._byId[n],this===t.collection&&delete t.collection,t.off("all",this._onModelEvent,this)},_onModelEvent:function(t,e,n,r){if("add"!==t&&"remove"!==t||n===this){if("destroy"===t&&this.remove(e,r),"change"===t){var i=this.modelId(e.previousAttributes()),o=this.modelId(e.attributes);i!==o&&(null!=i&&delete this._byId[i],null!=o&&(this._byId[o]=e))}this.trigger.apply(this,arguments)}}}),a(m,{forEach:3,each:3,map:3,collect:3,reduce:4,foldl:4,inject:4,reduceRight:4,foldr:4,find:3,detect:3,filter:3,select:3,reject:3,every:3,all:3,some:3,any:3,include:3,includes:3,contains:3,invoke:0,max:3,min:3,toArray:1,size:1,first:3,head:3,take:3,initial:3,rest:3,tail:3,drop:3,last:3,without:0,difference:0,indexOf:3,shuffle:1,lastIndexOf:3,isEmpty:1,chain:1,sample:3,partition:3,groupBy:3,countBy:3,sortBy:3,indexBy:3},"models");var B=e.View=function(t){this.cid=n.uniqueId("view"),n.extend(this,n.pick(t,Q)),this._ensureElement(),this.initialize.apply(this,arguments)},b=/^(\S+)\s*(.*)$/,Q=["model","collection","el","id","attributes","className","tagName","events"];n.extend(B.prototype,l,{tagName:"div",$:function(t){return this.$el.find(t)},initialize:function(){},render:function(){return this},remove:function(){return this._removeElement(),this.stopListening(),this},_removeElement:function(){this.$el.remove()},setElement:function(t){return this.undelegateEvents(),this._setElement(t),this.delegateEvents(),this},_setElement:function(t){this.$el=t instanceof e.$?t:e.$(t),this.el=this.$el[0]},delegateEvents:function(t){if(t||(t=n.result(this,"events")),!t)return this;for(var e in this.undelegateEvents(),t){var r=t[e];if(n.isFunction(r)||(r=this[r]),r){var i=e.match(b);this.delegate(i[1],i[2],n.bind(r,this))}}return this},delegate:function(t,e,n){return this.$el.on(t+".delegateEvents"+this.cid,e,n),this},undelegateEvents:function(){return this.$el&&this.$el.off(".delegateEvents"+this.cid),this},undelegate:function(t,e,n){return this.$el.off(t+".delegateEvents"+this.cid,e,n),this},_createElement:function(t){return document.createElement(t)},_ensureElement:function(){if(this.el)this.setElement(n.result(this,"el"));else{var t=n.extend({},n.result(this,"attributes"));this.id&&(t.id=n.result(this,"id")),this.className&&(t.class=n.result(this,"className")),this.setElement(this._createElement(n.result(this,"tagName"))),this._setAttributes(t)}},_setAttributes:function(t){this.$el.attr(t)}}),e.sync=function(t,r,i){var o=w[t];n.defaults(i||(i={}),{emulateHTTP:e.emulateHTTP,emulateJSON:e.emulateJSON});var a={type:o,dataType:"json"};if(i.url||(a.url=n.result(r,"url")||F()),null!=i.data||!r||"create"!==t&&"update"!==t&&"patch"!==t||(a.contentType="application/json",a.data=JSON.stringify(i.attrs||r.toJSON(i))),i.emulateJSON&&(a.contentType="application/x-www-form-urlencoded",a.data=a.data?{model:a.data}:{}),i.emulateHTTP&&("PUT"===o||"DELETE"===o||"PATCH"===o)){a.type="POST",i.emulateJSON&&(a.data._method=o);var s=i.beforeSend;i.beforeSend=function(t){if(t.setRequestHeader("X-HTTP-Method-Override",o),s)return s.apply(this,arguments)}}"GET"===a.type||i.emulateJSON||(a.processData=!1);var A=i.error;i.error=function(t,e,n){i.textStatus=e,i.errorThrown=n,A&&A.call(i.context,t,e,n)};var l=i.xhr=e.ajax(n.extend(a,i));return r.trigger("request",r,l,i),l};var w={create:"POST",update:"PUT",patch:"PATCH",delete:"DELETE",read:"GET"};e.ajax=function(){return e.$.ajax.apply(e.$,arguments)};var x=e.Router=function(t){t||(t={}),t.routes&&(this.routes=t.routes),this._bindRoutes(),this.initialize.apply(this,arguments)},_=/\((.*?)\)/g,D=/(\(\?)?:\w+/g,M=/\*\w+/g,k=/[\-{}\[\]+?.,\\\^$|#\s]/g;n.extend(x.prototype,l,{initialize:function(){},route:function(t,r,i){n.isRegExp(t)||(t=this._routeToRegExp(t)),n.isFunction(r)&&(i=r,r=""),i||(i=this[r]);var o=this;return e.history.route(t,function(n){var a=o._extractParameters(t,n);!1!==o.execute(i,a,r)&&(o.trigger.apply(o,["route:"+r].concat(a)),o.trigger("route",r,a),e.history.trigger("route",o,r,a))}),this},execute:function(t,e,n){t&&t.apply(this,e)},navigate:function(t,n){return e.history.navigate(t,n),this},_bindRoutes:function(){if(this.routes){this.routes=n.result(this,"routes");for(var t,e=n.keys(this.routes);null!=(t=e.pop());)this.route(t,this.routes[t])}},_routeToRegExp:function(t){return t=t.replace(k,"\\$&").replace(_,"(?:$1)?").replace(D,function(t,e){return e?t:"([^/?]+)"}).replace(M,"([^?]*?)"),new RegExp("^"+t+"(?:\\?([\\s\\S]*))?$")},_extractParameters:function(t,e){var r=t.exec(e).slice(1);return n.map(r,function(t,e){return e===r.length-1?t||null:t?decodeURIComponent(t):null})}});var S=e.History=function(){this.handlers=[],this.checkUrl=n.bind(this.checkUrl,this),"undefined"!=typeof window&&(this.location=window.location,this.history=window.history)},L=/^[#\/]|\s+$/g,R=/^\/+|\/+$/g,N=/#.*$/;S.started=!1,n.extend(S.prototype,l,{interval:50,atRoot:function(){var t=this.location.pathname.replace(/[^\/]$/,"$&/");return t===this.root&&!this.getSearch()},matchRoot:function(){var t=this.decodeFragment(this.location.pathname),e=t.slice(0,this.root.length-1)+"/";return e===this.root},decodeFragment:function(t){return decodeURI(t.replace(/%25/g,"%2525"))},getSearch:function(){var t=this.location.href.replace(/#.*/,"").match(/\?.+/);return t?t[0]:""},getHash:function(t){var e=(t||this).location.href.match(/#(.*)$/);return e?e[1]:""},getPath:function(){var t=this.decodeFragment(this.location.pathname+this.getSearch()).slice(this.root.length-1);return"/"===t.charAt(0)?t.slice(1):t},getFragment:function(t){return null==t&&(t=this._usePushState||!this._wantsHashChange?this.getPath():this.getHash()),t.replace(L,"")},start:function(t){if(S.started)throw new Error("Backbone.history has already been started");if(S.started=!0,this.options=n.extend({root:"/"},this.options,t),this.root=this.options.root,this._wantsHashChange=!1!==this.options.hashChange,this._hasHashChange="onhashchange"in window&&(void 0===document.documentMode||document.documentMode>7),this._useHashChange=this._wantsHashChange&&this._hasHashChange,this._wantsPushState=!!this.options.pushState,this._hasPushState=!(!this.history||!this.history.pushState),this._usePushState=this._wantsPushState&&this._hasPushState,this.fragment=this.getFragment(),this.root=("/"+this.root+"/").replace(R,"/"),this._wantsHashChange&&this._wantsPushState){if(!this._hasPushState&&!this.atRoot()){var e=this.root.slice(0,-1)||"/";return this.location.replace(e+"#"+this.getPath()),!0}this._hasPushState&&this.atRoot()&&this.navigate(this.getHash(),{replace:!0})}if(!this._hasHashChange&&this._wantsHashChange&&!this._usePushState){this.iframe=document.createElement("iframe"),this.iframe.src="javascript:0",this.iframe.style.display="none",this.iframe.tabIndex=-1;var r=document.body,i=r.insertBefore(this.iframe,r.firstChild).contentWindow;i.document.open(),i.document.close(),i.location.hash="#"+this.fragment}var o=window.addEventListener||function(t,e){return attachEvent("on"+t,e)};if(this._usePushState?o("popstate",this.checkUrl,!1):this._useHashChange&&!this.iframe?o("hashchange",this.checkUrl,!1):this._wantsHashChange&&(this._checkUrlInterval=setInterval(this.checkUrl,this.interval)),!this.options.silent)return this.loadUrl()},stop:function(){var t=window.removeEventListener||function(t,e){return detachEvent("on"+t,e)};this._usePushState?t("popstate",this.checkUrl,!1):this._useHashChange&&!this.iframe&&t("hashchange",this.checkUrl,!1),this.iframe&&(document.body.removeChild(this.iframe),this.iframe=null),this._checkUrlInterval&&clearInterval(this._checkUrlInterval),S.started=!1},route:function(t,e){this.handlers.unshift({route:t,callback:e})},checkUrl:function(t){var e=this.getFragment();if(e===this.fragment&&this.iframe&&(e=this.getHash(this.iframe.contentWindow)),e===this.fragment)return!1;this.iframe&&this.navigate(e),this.loadUrl()},loadUrl:function(t){return!!this.matchRoot()&&(t=this.fragment=this.getFragment(t),n.some(this.handlers,function(e){if(e.route.test(t))return e.callback(t),!0}))},navigate:function(t,e){if(!S.started)return!1;e&&!0!==e||(e={trigger:!!e}),t=this.getFragment(t||"");var n=this.root;""!==t&&"?"!==t.charAt(0)||(n=n.slice(0,-1)||"/");var r=n+t;if(t=this.decodeFragment(t.replace(N,"")),this.fragment!==t){if(this.fragment=t,this._usePushState)this.history[e.replace?"replaceState":"pushState"]({},document.title,r);else{if(!this._wantsHashChange)return this.location.assign(r);if(this._updateHash(this.location,t,e.replace),this.iframe&&t!==this.getHash(this.iframe.contentWindow)){var i=this.iframe.contentWindow;e.replace||(i.document.open(),i.document.close()),this._updateHash(i.location,t,e.replace)}}return e.trigger?this.loadUrl(t):void 0}},_updateHash:function(t,e,n){if(n){var r=t.href.replace(/(javascript:|#).*$/,"");t.replace(r+"#"+e)}else t.hash="#"+e}}),e.history=new S,C.extend=m.extend=x.extend=B.extend=S.extend=function(t,e){var r,i=this;r=t&&n.has(t,"constructor")?t.constructor:function(){return i.apply(this,arguments)},n.extend(r,i,e);var o=function(){this.constructor=r};return o.prototype=i.prototype,r.prototype=new o,t&&n.extend(r.prototype,t),r.__super__=i.prototype,r};var F=function(){throw new Error('A "url" property or function must be specified')},T=function(t,e){var n=e.error;e.error=function(r){n&&n.call(e.context,t,r,e),t.trigger("error",t,r,e)}};return e}(a,n,t,e)}.apply(e,i))||(t.exports=o)}).call(this,n(37))},function(t,e,n){(function(n){var r;e=t.exports=V,r="object"==typeof n&&n.env&&n.env.NODE_DEBUG&&/\bsemver\b/i.test(n.env.NODE_DEBUG)?function(){var t=Array.prototype.slice.call(arguments,0);t.unshift("SEMVER"),console.log.apply(console,t)}:function(){},e.SEMVER_SPEC_VERSION="2.0.0";var i=256,o=Number.MAX_SAFE_INTEGER||9007199254740991,a=e.re=[],s=e.src=[],A=0,l=A++;s[l]="0|[1-9]\\d*";var c=A++;s[c]="[0-9]+";var u=A++;s[u]="\\d*[a-zA-Z-][a-zA-Z0-9-]*";var g=A++;s[g]="("+s[l]+")\\.("+s[l]+")\\.("+s[l]+")";var h=A++;s[h]="("+s[c]+")\\.("+s[c]+")\\.("+s[c]+")";var d=A++;s[d]="(?:"+s[l]+"|"+s[u]+")";var f=A++;s[f]="(?:"+s[c]+"|"+s[u]+")";var p=A++;s[p]="(?:-("+s[d]+"(?:\\."+s[d]+")*))";var I=A++;s[I]="(?:-?("+s[f]+"(?:\\."+s[f]+")*))";var C=A++;s[C]="[0-9A-Za-z-]+";var m=A++;s[m]="(?:\\+("+s[C]+"(?:\\."+s[C]+")*))";var y=A++,E="v?"+s[g]+s[p]+"?"+s[m]+"?";s[y]="^"+E+"$";var v="[v=\\s]*"+s[h]+s[I]+"?"+s[m]+"?",B=A++;s[B]="^"+v+"$";var b=A++;s[b]="((?:<|>)?=?)";var Q=A++;s[Q]=s[c]+"|x|X|\\*";var w=A++;s[w]=s[l]+"|x|X|\\*";var x=A++;s[x]="[v=\\s]*("+s[w]+")(?:\\.("+s[w]+")(?:\\.("+s[w]+")(?:"+s[p]+")?"+s[m]+"?)?)?";var _=A++;s[_]="[v=\\s]*("+s[Q]+")(?:\\.("+s[Q]+")(?:\\.("+s[Q]+")(?:"+s[I]+")?"+s[m]+"?)?)?";var D=A++;s[D]="^"+s[b]+"\\s*"+s[x]+"$";var M=A++;s[M]="^"+s[b]+"\\s*"+s[_]+"$";var k=A++;s[k]="(^|[^\\d])(\\d{1,16})(?:\\.(\\d{1,16}))?(?:\\.(\\d{1,16}))?(?:$|[^\\d])";var S=A++;a[S]=new RegExp(s[k],"g");var L=A++;s[L]="(?:~>?)";var R=A++;s[R]="(\\s*)"+s[L]+"\\s+",a[R]=new RegExp(s[R],"g");var N=A++;s[N]="^"+s[L]+s[x]+"$";var F=A++;s[F]="^"+s[L]+s[_]+"$";var T=A++;s[T]="(?:\\^)";var U=A++;s[U]="(\\s*)"+s[T]+"\\s+",a[U]=new RegExp(s[U],"g");var O=A++;s[O]="^"+s[T]+s[x]+"$";var G=A++;s[G]="^"+s[T]+s[_]+"$";var H=A++;s[H]="^"+s[b]+"\\s*("+v+")$|^$";var j=A++;s[j]="^"+s[b]+"\\s*("+E+")$|^$";var Y=A++;s[Y]="(\\s*)"+s[b]+"\\s*("+v+"|"+s[x]+")",a[Y]=new RegExp(s[Y],"g");var P=A++;s[P]="^\\s*("+s[x]+")\\s+-\\s+("+s[x]+")\\s*$";var K=A++;s[K]="^\\s*("+s[_]+")\\s+-\\s+("+s[_]+")\\s*$";var z=A++;s[z]="(<|>)?=?\\s*\\*";for(var J=0;J<36;J++)r(J,s[J]),a[J]||(a[J]=new RegExp(s[J]));function q(t,e){if(e&&"object"==typeof e||(e={loose:!!e,includePrerelease:!1}),t instanceof V)return t;if("string"!=typeof t)return null;if(t.length>i)return null;if(!(e.loose?a[B]:a[y]).test(t))return null;try{return new V(t,e)}catch(t){return null}}function V(t,e){if(e&&"object"==typeof e||(e={loose:!!e,includePrerelease:!1}),t instanceof V){if(t.loose===e.loose)return t;t=t.version}else if("string"!=typeof t)throw new TypeError("Invalid Version: "+t);if(t.length>i)throw new TypeError("version is longer than "+i+" characters");if(!(this instanceof V))return new V(t,e);r("SemVer",t,e),this.options=e,this.loose=!!e.loose;var n=t.trim().match(e.loose?a[B]:a[y]);if(!n)throw new TypeError("Invalid Version: "+t);if(this.raw=t,this.major=+n[1],this.minor=+n[2],this.patch=+n[3],this.major>o||this.major<0)throw new TypeError("Invalid major version");if(this.minor>o||this.minor<0)throw new TypeError("Invalid minor version");if(this.patch>o||this.patch<0)throw new TypeError("Invalid patch version");n[4]?this.prerelease=n[4].split(".").map(function(t){if(/^[0-9]+$/.test(t)){var e=+t;if(e>=0&&e<o)return e}return t}):this.prerelease=[],this.build=n[5]?n[5].split("."):[],this.format()}e.parse=q,e.valid=function(t,e){var n=q(t,e);return n?n.version:null},e.clean=function(t,e){var n=q(t.trim().replace(/^[=v]+/,""),e);return n?n.version:null},e.SemVer=V,V.prototype.format=function(){return this.version=this.major+"."+this.minor+"."+this.patch,this.prerelease.length&&(this.version+="-"+this.prerelease.join(".")),this.version},V.prototype.toString=function(){return this.version},V.prototype.compare=function(t){return r("SemVer.compare",this.version,this.options,t),t instanceof V||(t=new V(t,this.options)),this.compareMain(t)||this.comparePre(t)},V.prototype.compareMain=function(t){return t instanceof V||(t=new V(t,this.options)),X(this.major,t.major)||X(this.minor,t.minor)||X(this.patch,t.patch)},V.prototype.comparePre=function(t){if(t instanceof V||(t=new V(t,this.options)),this.prerelease.length&&!t.prerelease.length)return-1;if(!this.prerelease.length&&t.prerelease.length)return 1;if(!this.prerelease.length&&!t.prerelease.length)return 0;var e=0;do{var n=this.prerelease[e],i=t.prerelease[e];if(r("prerelease compare",e,n,i),void 0===n&&void 0===i)return 0;if(void 0===i)return 1;if(void 0===n)return-1;if(n!==i)return X(n,i)}while(++e)},V.prototype.compareBuild=function(t){t instanceof V||(t=new V(t,this.options));var e=0;do{var n=this.build[e],i=t.build[e];if(r("prerelease compare",e,n,i),void 0===n&&void 0===i)return 0;if(void 0===i)return 1;if(void 0===n)return-1;if(n!==i)return X(n,i)}while(++e)},V.prototype.inc=function(t,e){switch(t){case"premajor":this.prerelease.length=0,this.patch=0,this.minor=0,this.major++,this.inc("pre",e);break;case"preminor":this.prerelease.length=0,this.patch=0,this.minor++,this.inc("pre",e);break;case"prepatch":this.prerelease.length=0,this.inc("patch",e),this.inc("pre",e);break;case"prerelease":0===this.prerelease.length&&this.inc("patch",e),this.inc("pre",e);break;case"major":0===this.minor&&0===this.patch&&0!==this.prerelease.length||this.major++,this.minor=0,this.patch=0,this.prerelease=[];break;case"minor":0===this.patch&&0!==this.prerelease.length||this.minor++,this.patch=0,this.prerelease=[];break;case"patch":0===this.prerelease.length&&this.patch++,this.prerelease=[];break;case"pre":if(0===this.prerelease.length)this.prerelease=[0];else{for(var n=this.prerelease.length;--n>=0;)"number"==typeof this.prerelease[n]&&(this.prerelease[n]++,n=-2);-1===n&&this.prerelease.push(0)}e&&(this.prerelease[0]===e?isNaN(this.prerelease[1])&&(this.prerelease=[e,0]):this.prerelease=[e,0]);break;default:throw new Error("invalid increment argument: "+t)}return this.format(),this.raw=this.version,this},e.inc=function(t,e,n,r){"string"==typeof n&&(r=n,n=void 0);try{return new V(t,n).inc(e,r).version}catch(t){return null}},e.diff=function(t,e){if(et(t,e))return null;var n=q(t),r=q(e),i="";if(n.prerelease.length||r.prerelease.length){i="pre";var o="prerelease"}for(var a in n)if(("major"===a||"minor"===a||"patch"===a)&&n[a]!==r[a])return i+a;return o},e.compareIdentifiers=X;var W=/^[0-9]+$/;function X(t,e){var n=W.test(t),r=W.test(e);return n&&r&&(t=+t,e=+e),t===e?0:n&&!r?-1:r&&!n?1:t<e?-1:1}function Z(t,e,n){return new V(t,n).compare(new V(e,n))}function $(t,e,n){return Z(t,e,n)>0}function tt(t,e,n){return Z(t,e,n)<0}function et(t,e,n){return 0===Z(t,e,n)}function nt(t,e,n){return 0!==Z(t,e,n)}function rt(t,e,n){return Z(t,e,n)>=0}function it(t,e,n){return Z(t,e,n)<=0}function ot(t,e,n,r){switch(e){case"===":return"object"==typeof t&&(t=t.version),"object"==typeof n&&(n=n.version),t===n;case"!==":return"object"==typeof t&&(t=t.version),"object"==typeof n&&(n=n.version),t!==n;case"":case"=":case"==":return et(t,n,r);case"!=":return nt(t,n,r);case">":return $(t,n,r);case">=":return rt(t,n,r);case"<":return tt(t,n,r);case"<=":return it(t,n,r);default:throw new TypeError("Invalid operator: "+e)}}function at(t,e){if(e&&"object"==typeof e||(e={loose:!!e,includePrerelease:!1}),t instanceof at){if(t.loose===!!e.loose)return t;t=t.value}if(!(this instanceof at))return new at(t,e);r("comparator",t,e),this.options=e,this.loose=!!e.loose,this.parse(t),this.semver===st?this.value="":this.value=this.operator+this.semver.version,r("comp",this)}e.rcompareIdentifiers=function(t,e){return X(e,t)},e.major=function(t,e){return new V(t,e).major},e.minor=function(t,e){return new V(t,e).minor},e.patch=function(t,e){return new V(t,e).patch},e.compare=Z,e.compareLoose=function(t,e){return Z(t,e,!0)},e.compareBuild=function(t,e,n){var r=new V(t,n),i=new V(e,n);return r.compare(i)||r.compareBuild(i)},e.rcompare=function(t,e,n){return Z(e,t,n)},e.sort=function(t,n){return t.sort(function(t,r){return e.compareBuild(t,r,n)})},e.rsort=function(t,n){return t.sort(function(t,r){return e.compareBuild(r,t,n)})},e.gt=$,e.lt=tt,e.eq=et,e.neq=nt,e.gte=rt,e.lte=it,e.cmp=ot,e.Comparator=at;var st={};function At(t,e){if(e&&"object"==typeof e||(e={loose:!!e,includePrerelease:!1}),t instanceof At)return t.loose===!!e.loose&&t.includePrerelease===!!e.includePrerelease?t:new At(t.raw,e);if(t instanceof at)return new At(t.value,e);if(!(this instanceof At))return new At(t,e);if(this.options=e,this.loose=!!e.loose,this.includePrerelease=!!e.includePrerelease,this.raw=t,this.set=t.split(/\s*\|\|\s*/).map(function(t){return this.parseRange(t.trim())},this).filter(function(t){return t.length}),!this.set.length)throw new TypeError("Invalid SemVer Range: "+t);this.format()}function lt(t,e){for(var n=!0,r=t.slice(),i=r.pop();n&&r.length;)n=r.every(function(t){return i.intersects(t,e)}),i=r.pop();return n}function ct(t){return!t||"x"===t.toLowerCase()||"*"===t}function ut(t,e,n,r,i,o,a,s,A,l,c,u,g){return((e=ct(n)?"":ct(r)?">="+n+".0.0":ct(i)?">="+n+"."+r+".0":">="+e)+" "+(s=ct(A)?"":ct(l)?"<"+(+A+1)+".0.0":ct(c)?"<"+A+"."+(+l+1)+".0":u?"<="+A+"."+l+"."+c+"-"+u:"<="+s)).trim()}function gt(t,e,n){for(var i=0;i<t.length;i++)if(!t[i].test(e))return!1;if(e.prerelease.length&&!n.includePrerelease){for(i=0;i<t.length;i++)if(r(t[i].semver),t[i].semver!==st&&t[i].semver.prerelease.length>0){var o=t[i].semver;if(o.major===e.major&&o.minor===e.minor&&o.patch===e.patch)return!0}return!1}return!0}function ht(t,e,n){try{e=new At(e,n)}catch(t){return!1}return e.test(t)}function dt(t,e,n,r){var i,o,a,s,A;switch(t=new V(t,r),e=new At(e,r),n){case">":i=$,o=it,a=tt,s=">",A=">=";break;case"<":i=tt,o=rt,a=$,s="<",A="<=";break;default:throw new TypeError('Must provide a hilo val of "<" or ">"')}if(ht(t,e,r))return!1;for(var l=0;l<e.set.length;++l){var c=e.set[l],u=null,g=null;if(c.forEach(function(t){t.semver===st&&(t=new at(">=0.0.0")),u=u||t,g=g||t,i(t.semver,u.semver,r)?u=t:a(t.semver,g.semver,r)&&(g=t)}),u.operator===s||u.operator===A)return!1;if((!g.operator||g.operator===s)&&o(t,g.semver))return!1;if(g.operator===A&&a(t,g.semver))return!1}return!0}at.prototype.parse=function(t){var e=this.options.loose?a[H]:a[j],n=t.match(e);if(!n)throw new TypeError("Invalid comparator: "+t);this.operator=void 0!==n[1]?n[1]:"","="===this.operator&&(this.operator=""),n[2]?this.semver=new V(n[2],this.options.loose):this.semver=st},at.prototype.toString=function(){return this.value},at.prototype.test=function(t){if(r("Comparator.test",t,this.options.loose),this.semver===st||t===st)return!0;if("string"==typeof t)try{t=new V(t,this.options)}catch(t){return!1}return ot(t,this.operator,this.semver,this.options)},at.prototype.intersects=function(t,e){if(!(t instanceof at))throw new TypeError("a Comparator is required");var n;if(e&&"object"==typeof e||(e={loose:!!e,includePrerelease:!1}),""===this.operator)return""===this.value||(n=new At(t.value,e),ht(this.value,n,e));if(""===t.operator)return""===t.value||(n=new At(this.value,e),ht(t.semver,n,e));var r=!(">="!==this.operator&&">"!==this.operator||">="!==t.operator&&">"!==t.operator),i=!("<="!==this.operator&&"<"!==this.operator||"<="!==t.operator&&"<"!==t.operator),o=this.semver.version===t.semver.version,a=!(">="!==this.operator&&"<="!==this.operator||">="!==t.operator&&"<="!==t.operator),s=ot(this.semver,"<",t.semver,e)&&(">="===this.operator||">"===this.operator)&&("<="===t.operator||"<"===t.operator),A=ot(this.semver,">",t.semver,e)&&("<="===this.operator||"<"===this.operator)&&(">="===t.operator||">"===t.operator);return r||i||o&&a||s||A},e.Range=At,At.prototype.format=function(){return this.range=this.set.map(function(t){return t.join(" ").trim()}).join("||").trim(),this.range},At.prototype.toString=function(){return this.range},At.prototype.parseRange=function(t){var e=this.options.loose;t=t.trim();var n=e?a[K]:a[P];t=t.replace(n,ut),r("hyphen replace",t),t=t.replace(a[Y],"$1$2$3"),r("comparator trim",t,a[Y]),t=(t=(t=t.replace(a[R],"$1~")).replace(a[U],"$1^")).split(/\s+/).join(" ");var i=e?a[H]:a[j],o=t.split(" ").map(function(t){return function(t,e){return r("comp",t,e),t=function(t,e){return t.trim().split(/\s+/).map(function(t){return function(t,e){r("caret",t,e);var n=e.loose?a[G]:a[O];return t.replace(n,function(e,n,i,o,a){var s;return r("caret",t,e,n,i,o,a),ct(n)?s="":ct(i)?s=">="+n+".0.0 <"+(+n+1)+".0.0":ct(o)?s="0"===n?">="+n+"."+i+".0 <"+n+"."+(+i+1)+".0":">="+n+"."+i+".0 <"+(+n+1)+".0.0":a?(r("replaceCaret pr",a),s="0"===n?"0"===i?">="+n+"."+i+"."+o+"-"+a+" <"+n+"."+i+"."+(+o+1):">="+n+"."+i+"."+o+"-"+a+" <"+n+"."+(+i+1)+".0":">="+n+"."+i+"."+o+"-"+a+" <"+(+n+1)+".0.0"):(r("no pr"),s="0"===n?"0"===i?">="+n+"."+i+"."+o+" <"+n+"."+i+"."+(+o+1):">="+n+"."+i+"."+o+" <"+n+"."+(+i+1)+".0":">="+n+"."+i+"."+o+" <"+(+n+1)+".0.0"),r("caret return",s),s})}(t,e)}).join(" ")}(t,e),r("caret",t),t=function(t,e){return t.trim().split(/\s+/).map(function(t){return function(t,e){var n=e.loose?a[F]:a[N];return t.replace(n,function(e,n,i,o,a){var s;return r("tilde",t,e,n,i,o,a),ct(n)?s="":ct(i)?s=">="+n+".0.0 <"+(+n+1)+".0.0":ct(o)?s=">="+n+"."+i+".0 <"+n+"."+(+i+1)+".0":a?(r("replaceTilde pr",a),s=">="+n+"."+i+"."+o+"-"+a+" <"+n+"."+(+i+1)+".0"):s=">="+n+"."+i+"."+o+" <"+n+"."+(+i+1)+".0",r("tilde return",s),s})}(t,e)}).join(" ")}(t,e),r("tildes",t),t=function(t,e){return r("replaceXRanges",t,e),t.split(/\s+/).map(function(t){return function(t,e){t=t.trim();var n=e.loose?a[M]:a[D];return t.replace(n,function(n,i,o,a,s,A){r("xRange",t,n,i,o,a,s,A);var l=ct(o),c=l||ct(a),u=c||ct(s),g=u;return"="===i&&g&&(i=""),A=e.includePrerelease?"-0":"",l?n=">"===i||"<"===i?"<0.0.0-0":"*":i&&g?(c&&(a=0),s=0,">"===i?(i=">=",c?(o=+o+1,a=0,s=0):(a=+a+1,s=0)):"<="===i&&(i="<",c?o=+o+1:a=+a+1),n=i+o+"."+a+"."+s+A):c?n=">="+o+".0.0"+A+" <"+(+o+1)+".0.0"+A:u&&(n=">="+o+"."+a+".0"+A+" <"+o+"."+(+a+1)+".0"+A),r("xRange return",n),n})}(t,e)}).join(" ")}(t,e),r("xrange",t),t=function(t,e){return r("replaceStars",t,e),t.trim().replace(a[z],"")}(t,e),r("stars",t),t}(t,this.options)},this).join(" ").split(/\s+/);return this.options.loose&&(o=o.filter(function(t){return!!t.match(i)})),o=o.map(function(t){return new at(t,this.options)},this)},At.prototype.intersects=function(t,e){if(!(t instanceof At))throw new TypeError("a Range is required");return this.set.some(function(n){return lt(n,e)&&t.set.some(function(t){return lt(t,e)&&n.every(function(n){return t.every(function(t){return n.intersects(t,e)})})})})},e.toComparators=function(t,e){return new At(t,e).set.map(function(t){return t.map(function(t){return t.value}).join(" ").trim().split(" ")})},At.prototype.test=function(t){if(!t)return!1;if("string"==typeof t)try{t=new V(t,this.options)}catch(t){return!1}for(var e=0;e<this.set.length;e++)if(gt(this.set[e],t,this.options))return!0;return!1},e.satisfies=ht,e.maxSatisfying=function(t,e,n){var r=null,i=null;try{var o=new At(e,n)}catch(t){return null}return t.forEach(function(t){o.test(t)&&(r&&-1!==i.compare(t)||(i=new V(r=t,n)))}),r},e.minSatisfying=function(t,e,n){var r=null,i=null;try{var o=new At(e,n)}catch(t){return null}return t.forEach(function(t){o.test(t)&&(r&&1!==i.compare(t)||(i=new V(r=t,n)))}),r},e.minVersion=function(t,e){t=new At(t,e);var n=new V("0.0.0");if(t.test(n))return n;if(n=new V("0.0.0-0"),t.test(n))return n;n=null;for(var r=0;r<t.set.length;++r){var i=t.set[r];i.forEach(function(t){var e=new V(t.semver.version);switch(t.operator){case">":0===e.prerelease.length?e.patch++:e.prerelease.push(0),e.raw=e.format();case"":case">=":n&&!$(n,e)||(n=e);break;case"<":case"<=":break;default:throw new Error("Unexpected operation: "+t.operator)}})}if(n&&t.test(n))return n;return null},e.validRange=function(t,e){try{return new At(t,e).range||"*"}catch(t){return null}},e.ltr=function(t,e,n){return dt(t,e,"<",n)},e.gtr=function(t,e,n){return dt(t,e,">",n)},e.outside=dt,e.prerelease=function(t,e){var n=q(t,e);return n&&n.prerelease.length?n.prerelease:null},e.intersects=function(t,e,n){return t=new At(t,n),e=new At(e,n),t.intersects(e)},e.coerce=function(t,e){if(t instanceof V)return t;"number"==typeof t&&(t=String(t));if("string"!=typeof t)return null;var n=null;if((e=e||{}).rtl){for(var r;(r=a[S].exec(t))&&(!n||n.index+n[0].length!==t.length);)n&&r.index+r[0].length===n.index+n[0].length||(n=r),a[S].lastIndex=r.index+r[1].length+r[2].length;a[S].lastIndex=-1}else n=t.match(a[k]);if(null===n)return null;return q(n[2]+"."+(n[3]||"0")+"."+(n[4]||"0"),e)}}).call(this,n(58))},function(t,e,n){"use strict";e.__esModule=!0;var r=o(n(1293)),i=o(n(1296));function o(t){return t&&t.__esModule?t:{default:t}}e.default=function t(e,n,o){null===e&&(e=Function.prototype);var a=(0,i.default)(e,n);if(void 0===a){var s=(0,r.default)(e);return null===s?void 0:t(s,n,o)}if("value"in a)return a.value;var A=a.get;return void 0!==A?A.call(o):void 0}},function(t,e,n){"use strict";var r=n(30),i=n(396),o=n(35).Buffer,a=new Array(16);function s(){i.call(this,64),this._a=1732584193,this._b=4023233417,this._c=2562383102,this._d=271733878}function A(t,e){return t<<e|t>>>32-e}function l(t,e,n,r,i,o,a){return A(t+(e&n|~e&r)+i+o|0,a)+e|0}function c(t,e,n,r,i,o,a){return A(t+(e&r|n&~r)+i+o|0,a)+e|0}function u(t,e,n,r,i,o,a){return A(t+(e^n^r)+i+o|0,a)+e|0}function g(t,e,n,r,i,o,a){return A(t+(n^(e|~r))+i+o|0,a)+e|0}r(s,i),s.prototype._update=function(){for(var t=a,e=0;e<16;++e)t[e]=this._block.readInt32LE(4*e);var n=this._a,r=this._b,i=this._c,o=this._d;n=l(n,r,i,o,t[0],3614090360,7),o=l(o,n,r,i,t[1],3905402710,12),i=l(i,o,n,r,t[2],606105819,17),r=l(r,i,o,n,t[3],3250441966,22),n=l(n,r,i,o,t[4],4118548399,7),o=l(o,n,r,i,t[5],1200080426,12),i=l(i,o,n,r,t[6],2821735955,17),r=l(r,i,o,n,t[7],4249261313,22),n=l(n,r,i,o,t[8],1770035416,7),o=l(o,n,r,i,t[9],2336552879,12),i=l(i,o,n,r,t[10],4294925233,17),r=l(r,i,o,n,t[11],2304563134,22),n=l(n,r,i,o,t[12],1804603682,7),o=l(o,n,r,i,t[13],4254626195,12),i=l(i,o,n,r,t[14],2792965006,17),n=c(n,r=l(r,i,o,n,t[15],1236535329,22),i,o,t[1],4129170786,5),o=c(o,n,r,i,t[6],3225465664,9),i=c(i,o,n,r,t[11],643717713,14),r=c(r,i,o,n,t[0],3921069994,20),n=c(n,r,i,o,t[5],3593408605,5),o=c(o,n,r,i,t[10],38016083,9),i=c(i,o,n,r,t[15],3634488961,14),r=c(r,i,o,n,t[4],3889429448,20),n=c(n,r,i,o,t[9],568446438,5),o=c(o,n,r,i,t[14],3275163606,9),i=c(i,o,n,r,t[3],4107603335,14),r=c(r,i,o,n,t[8],1163531501,20),n=c(n,r,i,o,t[13],2850285829,5),o=c(o,n,r,i,t[2],4243563512,9),i=c(i,o,n,r,t[7],1735328473,14),n=u(n,r=c(r,i,o,n,t[12],2368359562,20),i,o,t[5],4294588738,4),o=u(o,n,r,i,t[8],2272392833,11),i=u(i,o,n,r,t[11],1839030562,16),r=u(r,i,o,n,t[14],4259657740,23),n=u(n,r,i,o,t[1],2763975236,4),o=u(o,n,r,i,t[4],1272893353,11),i=u(i,o,n,r,t[7],4139469664,16),r=u(r,i,o,n,t[10],3200236656,23),n=u(n,r,i,o,t[13],681279174,4),o=u(o,n,r,i,t[0],3936430074,11),i=u(i,o,n,r,t[3],3572445317,16),r=u(r,i,o,n,t[6],76029189,23),n=u(n,r,i,o,t[9],3654602809,4),o=u(o,n,r,i,t[12],3873151461,11),i=u(i,o,n,r,t[15],530742520,16),n=g(n,r=u(r,i,o,n,t[2],3299628645,23),i,o,t[0],4096336452,6),o=g(o,n,r,i,t[7],1126891415,10),i=g(i,o,n,r,t[14],2878612391,15),r=g(r,i,o,n,t[5],4237533241,21),n=g(n,r,i,o,t[12],1700485571,6),o=g(o,n,r,i,t[3],2399980690,10),i=g(i,o,n,r,t[10],4293915773,15),r=g(r,i,o,n,t[1],2240044497,21),n=g(n,r,i,o,t[8],1873313359,6),o=g(o,n,r,i,t[15],4264355552,10),i=g(i,o,n,r,t[6],2734768916,15),r=g(r,i,o,n,t[13],1309151649,21),n=g(n,r,i,o,t[4],4149444226,6),o=g(o,n,r,i,t[11],3174756917,10),i=g(i,o,n,r,t[2],718787259,15),r=g(r,i,o,n,t[9],3951481745,21),this._a=this._a+n|0,this._b=this._b+r|0,this._c=this._c+i|0,this._d=this._d+o|0},s.prototype._digest=function(){this._block[this._blockOffset++]=128,this._blockOffset>56&&(this._block.fill(0,this._blockOffset,64),this._update(),this._blockOffset=0),this._block.fill(0,this._blockOffset,56),this._block.writeUInt32LE(this._length[0],56),this._block.writeUInt32LE(this._length[1],60),this._update();var t=o.allocUnsafe(16);return t.writeInt32LE(this._a,0),t.writeInt32LE(this._b,4),t.writeInt32LE(this._c,8),t.writeInt32LE(this._d,12),t},t.exports=s},function(t,e,n){t.exports=i;var r=n(208).EventEmitter;function i(){r.call(this)}n(30)(i,r),i.Readable=n(269),i.Writable=n(778),i.Duplex=n(779),i.Transform=n(780),i.PassThrough=n(781),i.Stream=i,i.prototype.pipe=function(t,e){var n=this;function i(e){t.writable&&!1===t.write(e)&&n.pause&&n.pause()}function o(){n.readable&&n.resume&&n.resume()}n.on("data",i),t.on("drain",o),t._isStdio||e&&!1===e.end||(n.on("end",s),n.on("close",A));var a=!1;function s(){a||(a=!0,t.end())}function A(){a||(a=!0,"function"==typeof t.destroy&&t.destroy())}function l(t){if(c(),0===r.listenerCount(this,"error"))throw t}function c(){n.removeListener("data",i),t.removeListener("drain",o),n.removeListener("end",s),n.removeListener("close",A),n.removeListener("error",l),t.removeListener("error",l),n.removeListener("end",c),n.removeListener("close",c),t.removeListener("close",c)}return n.on("error",l),t.on("error",l),n.on("end",c),n.on("close",c),t.on("close",c),t.emit("pipe",n),t}},function(t,e,n){(e=t.exports=n(397)).Stream=e,e.Readable=e,e.Writable=n(271),e.Duplex=n(119),e.Transform=n(400),e.PassThrough=n(777)},function(t,e,n){var r=n(42),i=r.Buffer;function o(t,e){for(var n in t)e[n]=t[n]}function a(t,e,n){return i(t,e,n)}i.from&&i.alloc&&i.allocUnsafe&&i.allocUnsafeSlow?t.exports=r:(o(r,e),e.Buffer=a),o(i,a),a.from=function(t,e,n){if("number"==typeof t)throw new TypeError("Argument must not be a number");return i(t,e,n)},a.alloc=function(t,e,n){if("number"!=typeof t)throw new TypeError("Argument must be a number");var r=i(t);return void 0!==e?"string"==typeof n?r.fill(e,n):r.fill(e):r.fill(0),r},a.allocUnsafe=function(t){if("number"!=typeof t)throw new TypeError("Argument must be a number");return i(t)},a.allocUnsafeSlow=function(t){if("number"!=typeof t)throw new TypeError("Argument must be a number");return r.SlowBuffer(t)}},function(t,e,n){"use strict";(function(e,r,i){var o=n(209);function a(t){var e=this;this.next=null,this.entry=null,this.finish=function(){!function(t,e,n){var r=t.entry;t.entry=null;for(;r;){var i=r.callback;e.pendingcb--,i(n),r=r.next}e.corkedRequestsFree?e.corkedRequestsFree.next=t:e.corkedRequestsFree=t}(e,t)}}t.exports=C;var s,A=!e.browser&&["v0.10","v0.9."].indexOf(e.version.slice(0,5))>-1?r:o.nextTick;C.WritableState=I;var l=n(166);l.inherits=n(30);var c={deprecate:n(775)},u=n(398),g=n(270).Buffer,h=i.Uint8Array||function(){};var d,f=n(399);function p(){}function I(t,e){s=s||n(119),t=t||{};var r=e instanceof s;this.objectMode=!!t.objectMode,r&&(this.objectMode=this.objectMode||!!t.writableObjectMode);var i=t.highWaterMark,l=t.writableHighWaterMark,c=this.objectMode?16:16384;this.highWaterMark=i||0===i?i:r&&(l||0===l)?l:c,this.highWaterMark=Math.floor(this.highWaterMark),this.finalCalled=!1,this.needDrain=!1,this.ending=!1,this.ended=!1,this.finished=!1,this.destroyed=!1;var u=!1===t.decodeStrings;this.decodeStrings=!u,this.defaultEncoding=t.defaultEncoding||"utf8",this.length=0,this.writing=!1,this.corked=0,this.sync=!0,this.bufferProcessing=!1,this.onwrite=function(t){!function(t,e){var n=t._writableState,r=n.sync,i=n.writecb;if(function(t){t.writing=!1,t.writecb=null,t.length-=t.writelen,t.writelen=0}(n),e)!function(t,e,n,r,i){--e.pendingcb,n?(o.nextTick(i,r),o.nextTick(b,t,e),t._writableState.errorEmitted=!0,t.emit("error",r)):(i(r),t._writableState.errorEmitted=!0,t.emit("error",r),b(t,e))}(t,n,r,e,i);else{var a=v(n);a||n.corked||n.bufferProcessing||!n.bufferedRequest||E(t,n),r?A(y,t,n,a,i):y(t,n,a,i)}}(e,t)},this.writecb=null,this.writelen=0,this.bufferedRequest=null,this.lastBufferedRequest=null,this.pendingcb=0,this.prefinished=!1,this.errorEmitted=!1,this.bufferedRequestCount=0,this.corkedRequestsFree=new a(this)}function C(t){if(s=s||n(119),!(d.call(C,this)||this instanceof s))return new C(t);this._writableState=new I(t,this),this.writable=!0,t&&("function"==typeof t.write&&(this._write=t.write),"function"==typeof t.writev&&(this._writev=t.writev),"function"==typeof t.destroy&&(this._destroy=t.destroy),"function"==typeof t.final&&(this._final=t.final)),u.call(this)}function m(t,e,n,r,i,o,a){e.writelen=r,e.writecb=a,e.writing=!0,e.sync=!0,n?t._writev(i,e.onwrite):t._write(i,o,e.onwrite),e.sync=!1}function y(t,e,n,r){n||function(t,e){0===e.length&&e.needDrain&&(e.needDrain=!1,t.emit("drain"))}(t,e),e.pendingcb--,r(),b(t,e)}function E(t,e){e.bufferProcessing=!0;var n=e.bufferedRequest;if(t._writev&&n&&n.next){var r=e.bufferedRequestCount,i=new Array(r),o=e.corkedRequestsFree;o.entry=n;for(var s=0,A=!0;n;)i[s]=n,n.isBuf||(A=!1),n=n.next,s+=1;i.allBuffers=A,m(t,e,!0,e.length,i,"",o.finish),e.pendingcb++,e.lastBufferedRequest=null,o.next?(e.corkedRequestsFree=o.next,o.next=null):e.corkedRequestsFree=new a(e),e.bufferedRequestCount=0}else{for(;n;){var l=n.chunk,c=n.encoding,u=n.callback;if(m(t,e,!1,e.objectMode?1:l.length,l,c,u),n=n.next,e.bufferedRequestCount--,e.writing)break}null===n&&(e.lastBufferedRequest=null)}e.bufferedRequest=n,e.bufferProcessing=!1}function v(t){return t.ending&&0===t.length&&null===t.bufferedRequest&&!t.finished&&!t.writing}function B(t,e){t._final(function(n){e.pendingcb--,n&&t.emit("error",n),e.prefinished=!0,t.emit("prefinish"),b(t,e)})}function b(t,e){var n=v(e);return n&&(!function(t,e){e.prefinished||e.finalCalled||("function"==typeof t._final?(e.pendingcb++,e.finalCalled=!0,o.nextTick(B,t,e)):(e.prefinished=!0,t.emit("prefinish")))}(t,e),0===e.pendingcb&&(e.finished=!0,t.emit("finish"))),n}l.inherits(C,u),I.prototype.getBuffer=function(){for(var t=this.bufferedRequest,e=[];t;)e.push(t),t=t.next;return e},function(){try{Object.defineProperty(I.prototype,"buffer",{get:c.deprecate(function(){return this.getBuffer()},"_writableState.buffer is deprecated. Use _writableState.getBuffer instead.","DEP0003")})}catch(t){}}(),"function"==typeof Symbol&&Symbol.hasInstance&&"function"==typeof Function.prototype[Symbol.hasInstance]?(d=Function.prototype[Symbol.hasInstance],Object.defineProperty(C,Symbol.hasInstance,{value:function(t){return!!d.call(this,t)||this===C&&(t&&t._writableState instanceof I)}})):d=function(t){return t instanceof this},C.prototype.pipe=function(){this.emit("error",new Error("Cannot pipe, not readable"))},C.prototype.write=function(t,e,n){var r,i=this._writableState,a=!1,s=!i.objectMode&&(r=t,g.isBuffer(r)||r instanceof h);return s&&!g.isBuffer(t)&&(t=function(t){return g.from(t)}(t)),"function"==typeof e&&(n=e,e=null),s?e="buffer":e||(e=i.defaultEncoding),"function"!=typeof n&&(n=p),i.ended?function(t,e){var n=new Error("write after end");t.emit("error",n),o.nextTick(e,n)}(this,n):(s||function(t,e,n,r){var i=!0,a=!1;return null===n?a=new TypeError("May not write null values to stream"):"string"==typeof n||void 0===n||e.objectMode||(a=new TypeError("Invalid non-string/buffer chunk")),a&&(t.emit("error",a),o.nextTick(r,a),i=!1),i}(this,i,t,n))&&(i.pendingcb++,a=function(t,e,n,r,i,o){if(!n){var a=function(t,e,n){t.objectMode||!1===t.decodeStrings||"string"!=typeof e||(e=g.from(e,n));return e}(e,r,i);r!==a&&(n=!0,i="buffer",r=a)}var s=e.objectMode?1:r.length;e.length+=s;var A=e.length<e.highWaterMark;A||(e.needDrain=!0);if(e.writing||e.corked){var l=e.lastBufferedRequest;e.lastBufferedRequest={chunk:r,encoding:i,isBuf:n,callback:o,next:null},l?l.next=e.lastBufferedRequest:e.bufferedRequest=e.lastBufferedRequest,e.bufferedRequestCount+=1}else m(t,e,!1,s,r,i,o);return A}(this,i,s,t,e,n)),a},C.prototype.cork=function(){this._writableState.corked++},C.prototype.uncork=function(){var t=this._writableState;t.corked&&(t.corked--,t.writing||t.corked||t.finished||t.bufferProcessing||!t.bufferedRequest||E(this,t))},C.prototype.setDefaultEncoding=function(t){if("string"==typeof t&&(t=t.toLowerCase()),!(["hex","utf8","utf-8","ascii","binary","base64","ucs2","ucs-2","utf16le","utf-16le","raw"].indexOf((t+"").toLowerCase())>-1))throw new TypeError("Unknown encoding: "+t);return this._writableState.defaultEncoding=t,this},Object.defineProperty(C.prototype,"writableHighWaterMark",{enumerable:!1,get:function(){return this._writableState.highWaterMark}}),C.prototype._write=function(t,e,n){n(new Error("_write() is not implemented"))},C.prototype._writev=null,C.prototype.end=function(t,e,n){var r=this._writableState;"function"==typeof t?(n=t,t=null,e=null):"function"==typeof e&&(n=e,e=null),null!=t&&this.write(t,e),r.corked&&(r.corked=1,this.uncork()),r.ending||r.finished||function(t,e,n){e.ending=!0,b(t,e),n&&(e.finished?o.nextTick(n):t.once("finish",n));e.ended=!0,t.writable=!1}(this,r,n)},Object.defineProperty(C.prototype,"destroyed",{get:function(){return void 0!==this._writableState&&this._writableState.destroyed},set:function(t){this._writableState&&(this._writableState.destroyed=t)}}),C.prototype.destroy=f.destroy,C.prototype._undestroy=f.undestroy,C.prototype._destroy=function(t,e){this.end(),e(t)}}).call(this,n(58),n(118).setImmediate,n(37))},function(t,e,n){"use strict";var r=n(42).Buffer,i=n(30),o=n(396),a=new Array(16),s=[0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,7,4,13,1,10,6,15,3,12,0,9,5,2,14,11,8,3,10,14,4,9,15,8,1,2,7,0,6,13,11,5,12,1,9,11,10,0,8,12,4,13,3,7,15,14,5,6,2,4,0,5,9,7,12,2,10,14,1,3,8,11,6,15,13],A=[5,14,7,0,9,2,11,4,13,6,15,8,1,10,3,12,6,11,3,7,0,13,5,10,14,15,8,12,4,9,1,2,15,5,1,3,7,14,6,9,11,8,12,2,10,0,4,13,8,6,4,1,3,11,15,0,5,12,2,13,9,7,10,14,12,15,10,4,1,5,8,7,6,2,13,14,0,3,9,11],l=[11,14,15,12,5,8,7,9,11,13,14,15,6,7,9,8,7,6,8,13,11,9,7,15,7,12,15,9,11,7,13,12,11,13,6,7,14,9,13,15,14,8,13,6,5,12,7,5,11,12,14,15,14,15,9,8,9,14,5,6,8,6,5,12,9,15,5,11,6,8,13,12,5,12,13,14,11,8,5,6],c=[8,9,9,11,13,15,15,5,7,7,8,11,14,14,12,6,9,13,15,7,12,8,9,11,7,7,12,7,6,15,13,11,9,7,15,11,8,6,6,14,12,13,5,14,13,13,7,5,15,5,8,11,14,14,6,14,6,9,12,9,12,5,15,8,8,5,12,9,12,5,14,6,8,13,6,5,15,13,11,11],u=[0,1518500249,1859775393,2400959708,2840853838],g=[1352829926,1548603684,1836072691,2053994217,0];function h(){o.call(this,64),this._a=1732584193,this._b=4023233417,this._c=2562383102,this._d=271733878,this._e=3285377520}function d(t,e){return t<<e|t>>>32-e}function f(t,e,n,r,i,o,a,s){return d(t+(e^n^r)+o+a|0,s)+i|0}function p(t,e,n,r,i,o,a,s){return d(t+(e&n|~e&r)+o+a|0,s)+i|0}function I(t,e,n,r,i,o,a,s){return d(t+((e|~n)^r)+o+a|0,s)+i|0}function C(t,e,n,r,i,o,a,s){return d(t+(e&r|n&~r)+o+a|0,s)+i|0}function m(t,e,n,r,i,o,a,s){return d(t+(e^(n|~r))+o+a|0,s)+i|0}i(h,o),h.prototype._update=function(){for(var t=a,e=0;e<16;++e)t[e]=this._block.readInt32LE(4*e);for(var n=0|this._a,r=0|this._b,i=0|this._c,o=0|this._d,h=0|this._e,y=0|this._a,E=0|this._b,v=0|this._c,B=0|this._d,b=0|this._e,Q=0;Q<80;Q+=1){var w,x;Q<16?(w=f(n,r,i,o,h,t[s[Q]],u[0],l[Q]),x=m(y,E,v,B,b,t[A[Q]],g[0],c[Q])):Q<32?(w=p(n,r,i,o,h,t[s[Q]],u[1],l[Q]),x=C(y,E,v,B,b,t[A[Q]],g[1],c[Q])):Q<48?(w=I(n,r,i,o,h,t[s[Q]],u[2],l[Q]),x=I(y,E,v,B,b,t[A[Q]],g[2],c[Q])):Q<64?(w=C(n,r,i,o,h,t[s[Q]],u[3],l[Q]),x=p(y,E,v,B,b,t[A[Q]],g[3],c[Q])):(w=m(n,r,i,o,h,t[s[Q]],u[4],l[Q]),x=f(y,E,v,B,b,t[A[Q]],g[4],c[Q])),n=h,h=o,o=d(i,10),i=r,r=w,y=b,b=B,B=d(v,10),v=E,E=x}var _=this._b+i+B|0;this._b=this._c+o+b|0,this._c=this._d+h+y|0,this._d=this._e+n+E|0,this._e=this._a+r+v|0,this._a=_},h.prototype._digest=function(){this._block[this._blockOffset++]=128,this._blockOffset>56&&(this._block.fill(0,this._blockOffset,64),this._update(),this._blockOffset=0),this._block.fill(0,this._blockOffset,56),this._block.writeUInt32LE(this._length[0],56),this._block.writeUInt32LE(this._length[1],60),this._update();var t=r.alloc?r.alloc(20):new r(20);return t.writeInt32LE(this._a,0),t.writeInt32LE(this._b,4),t.writeInt32LE(this._c,8),t.writeInt32LE(this._d,12),t.writeInt32LE(this._e,16),t},t.exports=h},function(t,e,n){(e=t.exports=function(t){t=t.toLowerCase();var n=e[t];if(!n)throw new Error(t+" is not supported (we accept pull requests)");return new n}).sha=n(782),e.sha1=n(783),e.sha224=n(784),e.sha256=n(401),e.sha384=n(785),e.sha512=n(402)},function(t,e,n){"use strict";e.utils=n(791),e.Cipher=n(792),e.DES=n(793),e.CBC=n(794),e.EDE=n(795)},function(t,e,n){var r=n(796),i=n(804),o=n(412);e.createCipher=e.Cipher=r.createCipher,e.createCipheriv=e.Cipheriv=r.createCipheriv,e.createDecipher=e.Decipher=i.createDecipher,e.createDecipheriv=e.Decipheriv=i.createDecipheriv,e.listCiphers=e.getCiphers=function(){return Object.keys(o)}},function(t,e,n){var r={ECB:n(797),CBC:n(798),CFB:n(799),CFB8:n(800),CFB1:n(801),OFB:n(802),CTR:n(410),GCM:n(410)},i=n(412);for(var o in i)i[o].module=r[i[o].mode];t.exports=i},function(t,e,n){var r;function i(t){this.rand=t}if(t.exports=function(t){return r||(r=new i(null)),r.generate(t)},t.exports.Rand=i,i.prototype.generate=function(t){return this._rand(t)},i.prototype._rand=function(t){if(this.rand.getBytes)return this.rand.getBytes(t);for(var e=new Uint8Array(t),n=0;n<e.length;n++)e[n]=this.rand.getByte();return e},"object"==typeof self)self.crypto&&self.crypto.getRandomValues?i.prototype._rand=function(t){var e=new Uint8Array(t);return self.crypto.getRandomValues(e),e}:self.msCrypto&&self.msCrypto.getRandomValues?i.prototype._rand=function(t){var e=new Uint8Array(t);return self.msCrypto.getRandomValues(e),e}:"object"==typeof window&&(i.prototype._rand=function(){throw new Error("Not implemented yet")});else try{var o=n(808);if("function"!=typeof o.randomBytes)throw new Error("Not supported");i.prototype._rand=function(t){return o.randomBytes(t)}}catch(t){}},function(t,e,n){(function(e){var r=n(53),i=n(135);function o(t,n){var i=function(t){var e=a(t);return{blinder:e.toRed(r.mont(t.modulus)).redPow(new r(t.publicExponent)).fromRed(),unblinder:e.invm(t.modulus)}}(n),o=n.modulus.byteLength(),s=(r.mont(n.modulus),new r(t).mul(i.blinder).umod(n.modulus)),A=s.toRed(r.mont(n.prime1)),l=s.toRed(r.mont(n.prime2)),c=n.coefficient,u=n.prime1,g=n.prime2,h=A.redPow(n.exponent1),d=l.redPow(n.exponent2);h=h.fromRed(),d=d.fromRed();var f=h.isub(d).imul(c).umod(u);return f.imul(g),d.iadd(f),new e(d.imul(i.unblinder).umod(n.modulus).toArray(!1,o))}function a(t){for(var e=t.modulus.byteLength(),n=new r(i(e));n.cmp(t.modulus)>=0||!n.umod(t.prime1)||!n.umod(t.prime2);)n=new r(i(e));return n}t.exports=o,o.getr=a}).call(this,n(42).Buffer)},function(t,e,n){"use strict";var r=e;r.version=n(813).version,r.utils=n(78),r.rand=n(277),r.curve=n(418),r.curves=n(280),r.ec=n(824),r.eddsa=n(828)},function(t,e,n){"use strict";var r,i=e,o=n(281),a=n(418),s=n(78).assert;function A(t){"short"===t.type?this.curve=new a.short(t):"edwards"===t.type?this.curve=new a.edwards(t):this.curve=new a.mont(t),this.g=this.curve.g,this.n=this.curve.n,this.hash=t.hash,s(this.g.validate(),"Invalid curve"),s(this.g.mul(this.n).isInfinity(),"Invalid curve, G*N != O")}function l(t,e){Object.defineProperty(i,t,{configurable:!0,enumerable:!0,get:function(){var n=new A(e);return Object.defineProperty(i,t,{configurable:!0,enumerable:!0,value:n}),n}})}i.PresetCurve=A,l("p192",{type:"short",prime:"p192",p:"ffffffff ffffffff ffffffff fffffffe ffffffff ffffffff",a:"ffffffff ffffffff ffffffff fffffffe ffffffff fffffffc",b:"64210519 e59c80e7 0fa7e9ab 72243049 feb8deec c146b9b1",n:"ffffffff ffffffff ffffffff 99def836 146bc9b1 b4d22831",hash:o.sha256,gRed:!1,g:["188da80e b03090f6 7cbf20eb 43a18800 f4ff0afd 82ff1012","07192b95 ffc8da78 631011ed 6b24cdd5 73f977a1 1e794811"]}),l("p224",{type:"short",prime:"p224",p:"ffffffff ffffffff ffffffff ffffffff 00000000 00000000 00000001",a:"ffffffff ffffffff ffffffff fffffffe ffffffff ffffffff fffffffe",b:"b4050a85 0c04b3ab f5413256 5044b0b7 d7bfd8ba 270b3943 2355ffb4",n:"ffffffff ffffffff ffffffff ffff16a2 e0b8f03e 13dd2945 5c5c2a3d",hash:o.sha256,gRed:!1,g:["b70e0cbd 6bb4bf7f 321390b9 4a03c1d3 56c21122 343280d6 115c1d21","bd376388 b5f723fb 4c22dfe6 cd4375a0 5a074764 44d58199 85007e34"]}),l("p256",{type:"short",prime:null,p:"ffffffff 00000001 00000000 00000000 00000000 ffffffff ffffffff ffffffff",a:"ffffffff 00000001 00000000 00000000 00000000 ffffffff ffffffff fffffffc",b:"5ac635d8 aa3a93e7 b3ebbd55 769886bc 651d06b0 cc53b0f6 3bce3c3e 27d2604b",n:"ffffffff 00000000 ffffffff ffffffff bce6faad a7179e84 f3b9cac2 fc632551",hash:o.sha256,gRed:!1,g:["6b17d1f2 e12c4247 f8bce6e5 63a440f2 77037d81 2deb33a0 f4a13945 d898c296","4fe342e2 fe1a7f9b 8ee7eb4a 7c0f9e16 2bce3357 6b315ece cbb64068 37bf51f5"]}),l("p384",{type:"short",prime:null,p:"ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff fffffffe ffffffff 00000000 00000000 ffffffff",a:"ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff fffffffe ffffffff 00000000 00000000 fffffffc",b:"b3312fa7 e23ee7e4 988e056b e3f82d19 181d9c6e fe814112 0314088f 5013875a c656398d 8a2ed19d 2a85c8ed d3ec2aef",n:"ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff c7634d81 f4372ddf 581a0db2 48b0a77a ecec196a ccc52973",hash:o.sha384,gRed:!1,g:["aa87ca22 be8b0537 8eb1c71e f320ad74 6e1d3b62 8ba79b98 59f741e0 82542a38 5502f25d bf55296c 3a545e38 72760ab7","3617de4a 96262c6f 5d9e98bf 9292dc29 f8f41dbd 289a147c e9da3113 b5f0b8c0 0a60b1ce 1d7e819d 7a431d7c 90ea0e5f"]}),l("p521",{type:"short",prime:null,p:"000001ff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff",a:"000001ff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff fffffffc",b:"00000051 953eb961 8e1c9a1f 929a21a0 b68540ee a2da725b 99b315f3 b8b48991 8ef109e1 56193951 ec7e937b 1652c0bd 3bb1bf07 3573df88 3d2c34f1 ef451fd4 6b503f00",n:"000001ff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff fffffffa 51868783 bf2f966b 7fcc0148 f709a5d0 3bb5c9b8 899c47ae bb6fb71e 91386409",hash:o.sha512,gRed:!1,g:["000000c6 858e06b7 0404e9cd 9e3ecb66 2395b442 9c648139 053fb521 f828af60 6b4d3dba a14b5e77 efe75928 fe1dc127 a2ffa8de 3348b3c1 856a429b f97e7e31 c2e5bd66","00000118 39296a78 9a3bc004 5c8a5fb4 2c7d1bd9 98f54449 579b4468 17afbd17 273e662c 97ee7299 5ef42640 c550b901 3fad0761 353c7086 a272c240 88be9476 9fd16650"]}),l("curve25519",{type:"mont",prime:"p25519",p:"7fffffffffffffff ffffffffffffffff ffffffffffffffff ffffffffffffffed",a:"76d06",b:"1",n:"1000000000000000 0000000000000000 14def9dea2f79cd6 5812631a5cf5d3ed",hash:o.sha256,gRed:!1,g:["9"]}),l("ed25519",{type:"edwards",prime:"p25519",p:"7fffffffffffffff ffffffffffffffff ffffffffffffffff ffffffffffffffed",a:"-1",c:"1",d:"52036cee2b6ffe73 8cc740797779e898 00700a4d4141d8ab 75eb4dca135978a3",n:"1000000000000000 0000000000000000 14def9dea2f79cd6 5812631a5cf5d3ed",hash:o.sha256,gRed:!1,g:["216936d3cd6e53fec0a4e231fdd6dc5c692cc7609525a7b2c9562d608f25d51a","6666666666666666666666666666666666666666666666666666666666666658"]});try{r=n(823)}catch(t){r=void 0}l("secp256k1",{type:"short",prime:"k256",p:"ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff fffffffe fffffc2f",a:"0",b:"7",n:"ffffffff ffffffff ffffffff fffffffe baaedce6 af48a03b bfd25e8c d0364141",h:"1",hash:o.sha256,beta:"7ae96a2b657c07106e64479eac3434e99cf0497512f58995c1396c28719501ee",lambda:"5363ad4cc05c30e0a5261c028812645a122e22ea20816678df02967c1b23bd72",basis:[{a:"3086d221a7d46bcde86c90e49284eb15",b:"-e4437ed6010e88286f547fa90abfe4c3"},{a:"114ca50f7a8e2f3f657c1108d9d44cfd8",b:"3086d221a7d46bcde86c90e49284eb15"}],gRed:!1,g:["79be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d959f2815b16f81798","483ada7726a3c4655da4fbfc0e1108a8fd17b448a68554199c47d08ffb10d4b8",r]})},function(t,e,n){var r=e;r.utils=n(87),r.common=n(168),r.sha=n(817),r.ripemd=n(821),r.hmac=n(822),r.sha1=r.sha.sha1,r.sha256=r.sha.sha256,r.sha224=r.sha.sha224,r.sha384=r.sha.sha384,r.sha512=r.sha.sha512,r.ripemd160=r.ripemd.ripemd160},function(t,e,n){"use strict";var r=n(867),i=n(283),o=n(137),a=n(432),s=n(869);function A(t,e,n){var r=this._refs[n];if("string"==typeof r){if(!this._refs[r])return A.call(this,t,e,r);r=this._refs[r]}if((r=r||this._schemas[n])instanceof a)return h(r.schema,this._opts.inlineRefs)?r.schema:r.validate||this._compile(r);var i,o,s,c=l.call(this,e,n);return c&&(i=c.schema,e=c.root,s=c.baseId),i instanceof a?o=i.validate||t.call(this,i.schema,e,void 0,s):void 0!==i&&(o=h(i,this._opts.inlineRefs)?i:t.call(this,i,e,void 0,s)),o}function l(t,e){var n=r.parse(e),i=f(n),o=d(this._getId(t.schema));if(0===Object.keys(t.schema).length||i!==o){var s=I(i),A=this._refs[s];if("string"==typeof A)return function(t,e,n){var r=l.call(this,t,e);if(r){var i=r.schema,o=r.baseId;t=r.root;var a=this._getId(i);return a&&(o=C(o,a)),u.call(this,n,o,i,t)}}.call(this,t,A,n);if(A instanceof a)A.validate||this._compile(A),t=A;else{if(!((A=this._schemas[s])instanceof a))return;if(A.validate||this._compile(A),s==I(e))return{schema:A,root:t,baseId:o};t=A}if(!t.schema)return;o=d(this._getId(t.schema))}return u.call(this,n,o,t.schema,t)}t.exports=A,A.normalizeId=I,A.fullPath=d,A.url=C,A.ids=function(t){var e=I(this._getId(t)),n={"":e},a={"":d(e,!1)},A={},l=this;return s(t,{allKeys:!0},function(t,e,s,c,u,g,h){if(""!==e){var d=l._getId(t),f=n[c],p=a[c]+"/"+u;if(void 0!==h&&(p+="/"+("number"==typeof h?h:o.escapeFragment(h))),"string"==typeof d){d=f=I(f?r.resolve(f,d):d);var C=l._refs[d];if("string"==typeof C&&(C=l._refs[C]),C&&C.schema){if(!i(t,C.schema))throw new Error('id "'+d+'" resolves to more than one schema')}else if(d!=I(p))if("#"==d[0]){if(A[d]&&!i(t,A[d]))throw new Error('id "'+d+'" resolves to more than one schema');A[d]=t}else l._refs[d]=p}n[e]=f,a[e]=p}}),A},A.inlineRef=h,A.schema=l;var c=o.toHash(["properties","patternProperties","enum","dependencies","definitions"]);function u(t,e,n,r){if(t.fragment=t.fragment||"","/"==t.fragment.slice(0,1)){for(var i=t.fragment.split("/"),a=1;a<i.length;a++){var s=i[a];if(s){if(void 0===(n=n[s=o.unescapeFragment(s)]))break;var A;if(!c[s]&&((A=this._getId(n))&&(e=C(e,A)),n.$ref)){var u=C(e,n.$ref),g=l.call(this,r,u);g&&(n=g.schema,r=g.root,e=g.baseId)}}}return void 0!==n&&n!==r.schema?{schema:n,root:r,baseId:e}:void 0}}var g=o.toHash(["type","format","pattern","maxLength","minLength","maxProperties","minProperties","maxItems","minItems","maximum","minimum","uniqueItems","multipleOf","required","enum"]);function h(t,e){return!1!==e&&(void 0===e||!0===e?function t(e){var n;if(Array.isArray(e)){for(var r=0;r<e.length;r++)if("object"==typeof(n=e[r])&&!t(n))return!1}else for(var i in e){if("$ref"==i)return!1;if("object"==typeof(n=e[i])&&!t(n))return!1}return!0}(t):e?function t(e){var n,r=0;if(Array.isArray(e)){for(var i=0;i<e.length;i++)if("object"==typeof(n=e[i])&&(r+=t(n)),r==1/0)return 1/0}else for(var o in e){if("$ref"==o)return 1/0;if(g[o])r++;else if("object"==typeof(n=e[o])&&(r+=t(n)+1),r==1/0)return 1/0}return r}(t)<=e:void 0)}function d(t,e){return!1!==e&&(t=I(t)),f(r.parse(t))}function f(t){return r.serialize(t).split("#")[0]+"#"}var p=/#\/?$/;function I(t){return t?t.replace(p,""):""}function C(t,e){return e=I(e),r.resolve(t,e)}},function(t,e,n){"use strict";var r=Array.isArray,i=Object.keys,o=Object.prototype.hasOwnProperty;t.exports=function t(e,n){if(e===n)return!0;if(e&&n&&"object"==typeof e&&"object"==typeof n){var a,s,A,l=r(e),c=r(n);if(l&&c){if((s=e.length)!=n.length)return!1;for(a=s;0!=a--;)if(!t(e[a],n[a]))return!1;return!0}if(l!=c)return!1;var u=e instanceof Date,g=n instanceof Date;if(u!=g)return!1;if(u&&g)return e.getTime()==n.getTime();var h=e instanceof RegExp,d=n instanceof RegExp;if(h!=d)return!1;if(h&&d)return e.toString()==n.toString();var f=i(e);if((s=f.length)!==i(n).length)return!1;for(a=s;0!=a--;)if(!o.call(n,f[a]))return!1;for(a=s;0!=a--;)if(!t(e[A=f[a]],n[A]))return!1;return!0}return e!=e&&n!=n}},function(t,e,n){"use strict";var r=n(282);function i(t,e,n){this.message=n||i.message(t,e),this.missingRef=r.url(t,e),this.missingSchema=r.normalizeId(r.fullPath(this.missingRef))}function o(t){return t.prototype=Object.create(Error.prototype),t.prototype.constructor=t,t}t.exports={Validation:o(function(t){this.message="validation failed",this.errors=t,this.ajv=this.validation=!0}),MissingRef:o(i)},i.message=function(t,e){return"can't resolve reference "+e+" from id "+t}},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(106),""),e.i(n(173),""),e.i(n(286),""),e.i(n(976),""),e.i(n(977),""),e.i(n(978),""),e.i(n(979),""),e.i(n(980),""),e.i(n(981),""),e.i(n(982),""),e.i(n(983),""),e.i(n(984),""),e.i(n(985),""),e.i(n(986),""),e.i(n(987),""),e.i(n(988),""),e.i(n(989),""),e.i(n(990),""),e.i(n(991),""),e.i(n(992),""),e.i(n(993),""),e.push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/* This file was auto-generated by ensurePackage() in @jupyterlab/buildutils */\n",""])},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(568),""),e.i(n(106),""),e.i(n(975),""),e.push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/* This file was auto-generated by ensurePackage() in @jupyterlab/buildutils */\n",""])},function(t,e,n){var r,i,o={},a=(r=function(){return window&&document&&document.all&&!window.atob},function(){return void 0===i&&(i=r.apply(this,arguments)),i}),s=function(t){var e={};return function(t){if("function"==typeof t)return t();if(void 0===e[t]){var n=function(t){return document.querySelector(t)}.call(this,t);if(window.HTMLIFrameElement&&n instanceof window.HTMLIFrameElement)try{n=n.contentDocument.head}catch(t){n=null}e[t]=n}return e[t]}}(),A=null,l=0,c=[],u=n(1024);function g(t,e){for(var n=0;n<t.length;n++){var r=t[n],i=o[r.id];if(i){i.refs++;for(var a=0;a<i.parts.length;a++)i.parts[a](r.parts[a]);for(;a<r.parts.length;a++)i.parts.push(C(r.parts[a],e))}else{var s=[];for(a=0;a<r.parts.length;a++)s.push(C(r.parts[a],e));o[r.id]={id:r.id,refs:1,parts:s}}}}function h(t,e){for(var n=[],r={},i=0;i<t.length;i++){var o=t[i],a=e.base?o[0]+e.base:o[0],s={css:o[1],media:o[2],sourceMap:o[3]};r[a]?r[a].parts.push(s):n.push(r[a]={id:a,parts:[s]})}return n}function d(t,e){var n=s(t.insertInto);if(!n)throw new Error("Couldn't find a style target. This probably means that the value for the 'insertInto' parameter is invalid.");var r=c[c.length-1];if("top"===t.insertAt)r?r.nextSibling?n.insertBefore(e,r.nextSibling):n.appendChild(e):n.insertBefore(e,n.firstChild),c.push(e);else if("bottom"===t.insertAt)n.appendChild(e);else{if("object"!=typeof t.insertAt||!t.insertAt.before)throw new Error("[Style Loader]\n\n Invalid value for parameter 'insertAt' ('options.insertAt') found.\n Must be 'top', 'bottom', or Object.\n (https://github.com/webpack-contrib/style-loader#insertat)\n");var i=s(t.insertInto+" "+t.insertAt.before);n.insertBefore(e,i)}}function f(t){if(null===t.parentNode)return!1;t.parentNode.removeChild(t);var e=c.indexOf(t);e>=0&&c.splice(e,1)}function p(t){var e=document.createElement("style");return void 0===t.attrs.type&&(t.attrs.type="text/css"),I(e,t.attrs),d(t,e),e}function I(t,e){Object.keys(e).forEach(function(n){t.setAttribute(n,e[n])})}function C(t,e){var n,r,i,o;if(e.transform&&t.css){if(!(o=e.transform(t.css)))return function(){};t.css=o}if(e.singleton){var a=l++;n=A||(A=p(e)),r=E.bind(null,n,a,!1),i=E.bind(null,n,a,!0)}else t.sourceMap&&"function"==typeof URL&&"function"==typeof URL.createObjectURL&&"function"==typeof URL.revokeObjectURL&&"function"==typeof Blob&&"function"==typeof btoa?(n=function(t){var e=document.createElement("link");return void 0===t.attrs.type&&(t.attrs.type="text/css"),t.attrs.rel="stylesheet",I(e,t.attrs),d(t,e),e}(e),r=function(t,e,n){var r=n.css,i=n.sourceMap,o=void 0===e.convertToAbsoluteUrls&&i;(e.convertToAbsoluteUrls||o)&&(r=u(r));i&&(r+="\n/*# sourceMappingURL=data:application/json;base64,"+btoa(unescape(encodeURIComponent(JSON.stringify(i))))+" */");var a=new Blob([r],{type:"text/css"}),s=t.href;t.href=URL.createObjectURL(a),s&&URL.revokeObjectURL(s)}.bind(null,n,e),i=function(){f(n),n.href&&URL.revokeObjectURL(n.href)}):(n=p(e),r=function(t,e){var n=e.css,r=e.media;r&&t.setAttribute("media",r);if(t.styleSheet)t.styleSheet.cssText=n;else{for(;t.firstChild;)t.removeChild(t.firstChild);t.appendChild(document.createTextNode(n))}}.bind(null,n),i=function(){f(n)});return r(t),function(e){if(e){if(e.css===t.css&&e.media===t.media&&e.sourceMap===t.sourceMap)return;r(t=e)}else i()}}t.exports=function(t,e){if("undefined"!=typeof DEBUG&&DEBUG&&"object"!=typeof document)throw new Error("The style-loader cannot be used in a non-browser environment");(e=e||{}).attrs="object"==typeof e.attrs?e.attrs:{},e.singleton||"boolean"==typeof e.singleton||(e.singleton=a()),e.insertInto||(e.insertInto="head"),e.insertAt||(e.insertAt="bottom");var n=h(t,e);return g(n,e),function(t){for(var r=[],i=0;i<n.length;i++){var a=n[i];(s=o[a.id]).refs--,r.push(s)}t&&g(h(t,e),e);for(i=0;i<r.length;i++){var s;if(0===(s=r[i]).refs){for(var A=0;A<s.parts.length;A++)s.parts[A]();delete o[s.id]}}}};var m,y=(m=[],function(t,e){return m[t]=e,m.filter(Boolean).join("\n")});function E(t,e,n,r){var i=n?"":r.css;if(t.styleSheet)t.styleSheet.cssText=y(e,i);else{var o=document.createTextNode(i),a=t.childNodes;a[e]&&t.removeChild(a[e]),a.length?t.insertBefore(o,a[e]):t.appendChild(o)}}},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(5),s=n(46),A=n(25),l=n(43),c=n(174),u=n(107),g=n(175),h=n(59),d=function(t){function e(e){void 0===e&&(e={});var n=t.call(this)||this;return n._fixed=0,n._spacing=4,n._dirty=!1,n._sizers=[],n._items=[],n._box=null,n._alignment="start",n._direction="top-to-bottom",void 0!==e.direction&&(n._direction=e.direction),void 0!==e.alignment&&(n._alignment=e.alignment),void 0!==e.spacing&&(n._spacing=o.clampSpacing(e.spacing)),n}return i(e,t),e.prototype.dispose=function(){a.each(this._items,function(t){t.dispose()}),this._box=null,this._items.length=0,this._sizers.length=0,t.prototype.dispose.call(this)},Object.defineProperty(e.prototype,"direction",{get:function(){return this._direction},set:function(t){this._direction!==t&&(this._direction=t,this.parent&&(this.parent.dataset.direction=t,this.parent.fit()))},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"alignment",{get:function(){return this._alignment},set:function(t){this._alignment!==t&&(this._alignment=t,this.parent&&(this.parent.dataset.alignment=t,this.parent.update()))},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"spacing",{get:function(){return this._spacing},set:function(t){t=o.clampSpacing(t),this._spacing!==t&&(this._spacing=t,this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),e.prototype.init=function(){this.parent.dataset.direction=this.direction,this.parent.dataset.alignment=this.alignment,t.prototype.init.call(this)},e.prototype.attachWidget=function(t,e){a.ArrayExt.insert(this._items,t,new u.LayoutItem(e)),a.ArrayExt.insert(this._sizers,t,new c.BoxSizer),this.parent.isAttached&&A.MessageLoop.sendMessage(e,h.Widget.Msg.BeforeAttach),this.parent.node.appendChild(e.node),this.parent.isAttached&&A.MessageLoop.sendMessage(e,h.Widget.Msg.AfterAttach),this.parent.fit()},e.prototype.moveWidget=function(t,e,n){a.ArrayExt.move(this._items,t,e),a.ArrayExt.move(this._sizers,t,e),this.parent.update()},e.prototype.detachWidget=function(t,e){var n=a.ArrayExt.removeAt(this._items,t);a.ArrayExt.removeAt(this._sizers,t),this.parent.isAttached&&A.MessageLoop.sendMessage(e,h.Widget.Msg.BeforeDetach),this.parent.node.removeChild(e.node),this.parent.isAttached&&A.MessageLoop.sendMessage(e,h.Widget.Msg.AfterDetach),n.dispose(),this.parent.fit()},e.prototype.onBeforeShow=function(e){t.prototype.onBeforeShow.call(this,e),this.parent.update()},e.prototype.onBeforeAttach=function(e){t.prototype.onBeforeAttach.call(this,e),this.parent.fit()},e.prototype.onChildShown=function(t){this.parent.fit()},e.prototype.onChildHidden=function(t){this.parent.fit()},e.prototype.onResize=function(t){this.parent.isVisible&&this._update(t.width,t.height)},e.prototype.onUpdateRequest=function(t){this.parent.isVisible&&this._update(-1,-1)},e.prototype.onFitRequest=function(t){this.parent.isAttached&&this._fit()},e.prototype._fit=function(){for(var t=0,n=0,r=this._items.length;n<r;++n)t+=+!this._items[n].isHidden;this._fixed=this._spacing*Math.max(0,t-1);var i=o.isHorizontal(this._direction),a=i?this._fixed:0,l=i?0:this._fixed;for(n=0,r=this._items.length;n<r;++n){var c=this._items[n],u=this._sizers[n];c.isHidden?(u.minSize=0,u.maxSize=0):(c.fit(),u.sizeHint=e.getSizeBasis(c.widget),u.stretch=e.getStretch(c.widget),i?(u.minSize=c.minWidth,u.maxSize=c.maxWidth,a+=c.minWidth,l=Math.max(l,c.minHeight)):(u.minSize=c.minHeight,u.maxSize=c.maxHeight,l+=c.minHeight,a=Math.max(a,c.minWidth)))}var g=this._box=s.ElementExt.boxSizing(this.parent.node);a+=g.horizontalSum,l+=g.verticalSum;var d=this.parent.node.style;d.minWidth=a+"px",d.minHeight=l+"px",this._dirty=!0,this.parent.parent&&A.MessageLoop.sendMessage(this.parent.parent,h.Widget.Msg.FitRequest),this._dirty&&A.MessageLoop.sendMessage(this.parent,h.Widget.Msg.UpdateRequest)},e.prototype._update=function(t,e){this._dirty=!1;for(var n=0,r=0,i=this._items.length;r<i;++r)n+=+!this._items[r].isHidden;if(0!==n){t<0&&(t=this.parent.node.offsetWidth),e<0&&(e=this.parent.node.offsetHeight),this._box||(this._box=s.ElementExt.boxSizing(this.parent.node));var o,a=this._box.paddingTop,A=this._box.paddingLeft,l=t-this._box.horizontalSum,u=e-this._box.verticalSum;switch(this._direction){case"left-to-right":o=c.BoxEngine.calc(this._sizers,Math.max(0,l-this._fixed));break;case"top-to-bottom":o=c.BoxEngine.calc(this._sizers,Math.max(0,u-this._fixed));break;case"right-to-left":o=c.BoxEngine.calc(this._sizers,Math.max(0,l-this._fixed)),A+=l;break;case"bottom-to-top":o=c.BoxEngine.calc(this._sizers,Math.max(0,u-this._fixed)),a+=u;break;default:throw"unreachable"}var g=0,h=0;if(o>0)switch(this._alignment){case"start":break;case"center":g=0,h=o/2;break;case"end":g=0,h=o;break;case"justify":g=o/n,h=0;break;default:throw"unreachable"}for(r=0,i=this._items.length;r<i;++r){var d=this._items[r];if(!d.isHidden){var f=this._sizers[r].size;switch(this._direction){case"left-to-right":d.update(A+h,a,f+g,u),A+=f+g+this._spacing;break;case"top-to-bottom":d.update(A,a+h,l,f+g),a+=f+g+this._spacing;break;case"right-to-left":d.update(A-h-f-g,a,f+g,u),A-=f+g+this._spacing;break;case"bottom-to-top":d.update(A,a-h-f-g,l,f+g),a-=f+g+this._spacing;break;default:throw"unreachable"}}}}},e}(g.PanelLayout);e.BoxLayout=d,function(t){t.getStretch=function(t){return o.stretchProperty.get(t)},t.setStretch=function(t,e){o.stretchProperty.set(t,e)},t.getSizeBasis=function(t){return o.sizeBasisProperty.get(t)},t.setSizeBasis=function(t,e){o.sizeBasisProperty.set(t,e)}}(d=e.BoxLayout||(e.BoxLayout={})),e.BoxLayout=d,function(t){function e(t){t.parent&&t.parent.layout instanceof d&&t.parent.fit()}t.stretchProperty=new l.AttachedProperty({name:"stretch",create:function(){return 0},coerce:function(t,e){return Math.max(0,Math.floor(e))},changed:e}),t.sizeBasisProperty=new l.AttachedProperty({name:"sizeBasis",create:function(){return 0},coerce:function(t,e){return Math.max(0,Math.floor(e))},changed:e}),t.isHorizontal=function(t){return"left-to-right"===t||"right-to-left"===t},t.clampSpacing=function(t){return Math.max(0,Math.floor(t))}}(o||(o={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(10),i=function(){function t(t){this._label="",this._caption="",this._mnemonic=-1,this._iconClass="",this._iconLabel="",this._className="",this._closable=!1,this._changed=new r.Signal(this),this.owner=t.owner,void 0!==t.label&&(this._label=t.label),void 0!==t.mnemonic&&(this._mnemonic=t.mnemonic),void 0!==t.icon&&(this._iconClass=t.icon),void 0!==t.iconClass&&(this._iconClass=t.iconClass),void 0!==t.iconLabel&&(this._iconLabel=t.iconLabel),void 0!==t.caption&&(this._caption=t.caption),void 0!==t.className&&(this._className=t.className),void 0!==t.closable&&(this._closable=t.closable),this._dataset=t.dataset||{}}return Object.defineProperty(t.prototype,"changed",{get:function(){return this._changed},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"label",{get:function(){return this._label},set:function(t){this._label!==t&&(this._label=t,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"mnemonic",{get:function(){return this._mnemonic},set:function(t){this._mnemonic!==t&&(this._mnemonic=t,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"icon",{get:function(){return this.iconClass},set:function(t){this.iconClass=t},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"iconClass",{get:function(){return this._iconClass},set:function(t){this._iconClass!==t&&(this._iconClass=t,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"iconLabel",{get:function(){return this._iconLabel},set:function(t){this._iconLabel!==t&&(this._iconLabel=t,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"caption",{get:function(){return this._caption},set:function(t){this._caption!==t&&(this._caption=t,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"className",{get:function(){return this._className},set:function(t){this._className!==t&&(this._className=t,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"closable",{get:function(){return this._closable},set:function(t){this._closable!==t&&(this._closable=t,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dataset",{get:function(){return this._dataset},set:function(t){this._dataset!==t&&(this._dataset=t,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),t}();e.Title=i},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.getKeyboardLayout=function(){return r.keyboardLayout},e.setKeyboardLayout=function(t){r.keyboardLayout=t};var r,i=function(){function t(e,n){this.name=e,this._codes=n,this._keys=t.extractKeys(n)}return t.prototype.keys=function(){return Object.keys(this._keys)},t.prototype.isValidKey=function(t){return t in this._keys},t.prototype.keyForKeydownEvent=function(t){return this._codes[t.keyCode]||""},t}();e.KeycodeLayout=i,function(t){t.extractKeys=function(t){var e=Object.create(null);for(var n in t)e[t[n]]=!0;return e}}(i=e.KeycodeLayout||(e.KeycodeLayout={})),e.KeycodeLayout=i,e.EN_US=new i("en-us",{8:"Backspace",9:"Tab",13:"Enter",19:"Pause",27:"Escape",32:"Space",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",48:"0",49:"1",50:"2",51:"3",52:"4",53:"5",54:"6",55:"7",56:"8",57:"9",59:";",61:"=",65:"A",66:"B",67:"C",68:"D",69:"E",70:"F",71:"G",72:"H",73:"I",74:"J",75:"K",76:"L",77:"M",78:"N",79:"O",80:"P",81:"Q",82:"R",83:"S",84:"T",85:"U",86:"V",87:"W",88:"X",89:"Y",90:"Z",93:"ContextMenu",96:"0",97:"1",98:"2",99:"3",100:"4",101:"5",102:"6",103:"7",104:"8",105:"9",106:"*",107:"+",109:"-",110:".",111:"/",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",173:"-",186:";",187:"=",188:",",189:"-",190:".",191:"/",192:"`",219:"[",220:"\\",221:"]",222:"'"}),function(t){t.keyboardLayout=e.EN_US}(r||(r={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(5),s=n(46),A=n(127),l=n(25),c=n(10),u=n(27),g=n(289),h=n(59),d=function(t){function e(n){void 0===n&&(n={});var r=t.call(this,{node:o.createNode()})||this;return r._currentIndex=-1,r._titles=[],r._previousTitle=null,r._dragData=null,r._tabMoved=new c.Signal(r),r._currentChanged=new c.Signal(r),r._tabCloseRequested=new c.Signal(r),r._tabDetachRequested=new c.Signal(r),r._tabActivateRequested=new c.Signal(r),r.addClass("p-TabBar"),r.setFlag(h.Widget.Flag.DisallowLayout),r.tabsMovable=n.tabsMovable||!1,r.allowDeselect=n.allowDeselect||!1,r.insertBehavior=n.insertBehavior||"select-tab-if-needed",r.removeBehavior=n.removeBehavior||"select-tab-after",r.renderer=n.renderer||e.defaultRenderer,r._orientation=n.orientation||"horizontal",r.dataset.orientation=r._orientation,r}return i(e,t),e.prototype.dispose=function(){this._releaseMouse(),this._titles.length=0,this._previousTitle=null,t.prototype.dispose.call(this)},Object.defineProperty(e.prototype,"currentChanged",{get:function(){return this._currentChanged},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"tabMoved",{get:function(){return this._tabMoved},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"tabActivateRequested",{get:function(){return this._tabActivateRequested},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"tabCloseRequested",{get:function(){return this._tabCloseRequested},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"tabDetachRequested",{get:function(){return this._tabDetachRequested},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"currentTitle",{get:function(){return this._titles[this._currentIndex]||null},set:function(t){this.currentIndex=t?this._titles.indexOf(t):-1},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"currentIndex",{get:function(){return this._currentIndex},set:function(t){if((t<0||t>=this._titles.length)&&(t=-1),this._currentIndex!==t){var e=this._currentIndex,n=this._titles[e]||null,r=t,i=this._titles[r]||null;this._currentIndex=r,this._previousTitle=n,this.update(),this._currentChanged.emit({previousIndex:e,previousTitle:n,currentIndex:r,currentTitle:i})}},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"orientation",{get:function(){return this._orientation},set:function(t){this._orientation!==t&&(this._releaseMouse(),this._orientation=t,this.dataset.orientation=t)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"titles",{get:function(){return this._titles},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"contentNode",{get:function(){return this.node.getElementsByClassName("p-TabBar-content")[0]},enumerable:!0,configurable:!0}),e.prototype.addTab=function(t){return this.insertTab(this._titles.length,t)},e.prototype.insertTab=function(t,e){this._releaseMouse();var n=o.asTitle(e),r=this._titles.indexOf(n),i=Math.max(0,Math.min(t,this._titles.length));return-1===r?(a.ArrayExt.insert(this._titles,i,n),n.changed.connect(this._onTitleChanged,this),this.update(),this._adjustCurrentForInsert(i,n),n):(i===this._titles.length&&i--,r===i?n:(a.ArrayExt.move(this._titles,r,i),this.update(),this._adjustCurrentForMove(r,i),n))},e.prototype.removeTab=function(t){this.removeTabAt(this._titles.indexOf(t))},e.prototype.removeTabAt=function(t){this._releaseMouse();var e=a.ArrayExt.removeAt(this._titles,t);e&&(e.changed.disconnect(this._onTitleChanged,this),e===this._previousTitle&&(this._previousTitle=null),this.update(),this._adjustCurrentForRemove(t,e))},e.prototype.clearTabs=function(){if(0!==this._titles.length){this._releaseMouse();for(var t=0,e=this._titles;t<e.length;t++){e[t].changed.disconnect(this._onTitleChanged,this)}var n=this.currentIndex,r=this.currentTitle;this._currentIndex=-1,this._previousTitle=null,this._titles.length=0,this.update(),-1!==n&&this._currentChanged.emit({previousIndex:n,previousTitle:r,currentIndex:-1,currentTitle:null})}},e.prototype.releaseMouse=function(){this._releaseMouse()},e.prototype.handleEvent=function(t){switch(t.type){case"mousedown":this._evtMouseDown(t);break;case"mousemove":this._evtMouseMove(t);break;case"mouseup":this._evtMouseUp(t);break;case"keydown":this._evtKeyDown(t);break;case"contextmenu":t.preventDefault(),t.stopPropagation()}},e.prototype.onBeforeAttach=function(t){this.node.addEventListener("mousedown",this)},e.prototype.onAfterDetach=function(t){this.node.removeEventListener("mousedown",this),this._releaseMouse()},e.prototype.onUpdateRequest=function(t){for(var e=this._titles,n=this.renderer,r=this.currentTitle,i=new Array(e.length),o=0,a=e.length;o<a;++o){var s=e[o],A=s===r,l=A?a:a-o-1;i[o]=n.renderTab({title:s,current:A,zIndex:l})}u.VirtualDOM.render(i,this.contentNode)},e.prototype._evtKeyDown=function(t){t.preventDefault(),t.stopPropagation(),27===t.keyCode&&this._releaseMouse()},e.prototype._evtMouseDown=function(t){if((0===t.button||1===t.button)&&!this._dragData){var e=this.contentNode.children,n=a.ArrayExt.findFirstIndex(e,function(e){return s.ElementExt.hitTest(e,t.clientX,t.clientY)});if(-1!==n&&(t.preventDefault(),t.stopPropagation(),this._dragData={tab:e[n],index:n,pressX:t.clientX,pressY:t.clientY,tabPos:-1,tabSize:-1,tabPressPos:-1,targetIndex:-1,tabLayout:null,contentRect:null,override:null,dragActive:!1,dragAborted:!1,detachRequested:!1},document.addEventListener("mouseup",this,!0),1!==t.button)){var r=e[n].querySelector(this.renderer.closeIconSelector);r&&r.contains(t.target)||(this.tabsMovable&&(document.addEventListener("mousemove",this,!0),document.addEventListener("keydown",this,!0),document.addEventListener("contextmenu",this,!0)),this.allowDeselect&&this.currentIndex===n?this.currentIndex=-1:this.currentIndex=n,-1!==this.currentIndex&&this._tabActivateRequested.emit({index:this.currentIndex,title:this.currentTitle}))}}},e.prototype._evtMouseMove=function(t){var e=this._dragData;if(e){t.preventDefault(),t.stopPropagation();var n=this.contentNode.children;if(e.dragActive||o.dragExceeded(e,t)){if(!e.dragActive){var r=e.tab.getBoundingClientRect();"horizontal"===this._orientation?(e.tabPos=e.tab.offsetLeft,e.tabSize=r.width,e.tabPressPos=e.pressX-r.left):(e.tabPos=e.tab.offsetTop,e.tabSize=r.height,e.tabPressPos=e.pressY-r.top),e.tabLayout=o.snapTabLayout(n,this._orientation),e.contentRect=this.contentNode.getBoundingClientRect(),e.override=A.Drag.overrideCursor("default"),e.tab.classList.add("p-mod-dragging"),this.addClass("p-mod-dragging"),e.dragActive=!0}if(!e.detachRequested&&o.detachExceeded(e,t)){e.detachRequested=!0;var i=e.index,a=t.clientX,s=t.clientY,l=n[i],c=this._titles[i];if(this._tabDetachRequested.emit({index:i,title:c,tab:l,clientX:a,clientY:s}),e.dragAborted)return}o.layoutTabs(n,e,t,this._orientation)}}},e.prototype._evtMouseUp=function(t){var e=this;if(0===t.button||1===t.button){var n=this._dragData;if(n){if(t.preventDefault(),t.stopPropagation(),document.removeEventListener("mousemove",this,!0),document.removeEventListener("mouseup",this,!0),document.removeEventListener("keydown",this,!0),document.removeEventListener("contextmenu",this,!0),!n.dragActive){this._dragData=null;var r=this.contentNode.children,i=a.ArrayExt.findFirstIndex(r,function(e){return s.ElementExt.hitTest(e,t.clientX,t.clientY)});if(i!==n.index)return;var A=this._titles[i];if(!A.closable)return;if(1===t.button)return void this._tabCloseRequested.emit({index:i,title:A});var c=r[i].querySelector(this.renderer.closeIconSelector);return c&&c.contains(t.target)?void this._tabCloseRequested.emit({index:i,title:A}):void 0}if(0===t.button){o.finalizeTabPosition(n,this._orientation),n.tab.classList.remove("p-mod-dragging");var u=o.parseTransitionDuration(n.tab);setTimeout(function(){if(!n.dragAborted){e._dragData=null,o.resetTabPositions(e.contentNode.children,e._orientation),n.override.dispose(),e.removeClass("p-mod-dragging");var t=n.index,r=n.targetIndex;-1!==r&&t!==r&&(a.ArrayExt.move(e._titles,t,r),e._adjustCurrentForMove(t,r),e._tabMoved.emit({fromIndex:t,toIndex:r,title:e._titles[r]}),l.MessageLoop.sendMessage(e,h.Widget.Msg.UpdateRequest))}},u)}}}},e.prototype._releaseMouse=function(){var t=this._dragData;t&&(this._dragData=null,document.removeEventListener("mousemove",this,!0),document.removeEventListener("mouseup",this,!0),document.removeEventListener("keydown",this,!0),document.removeEventListener("contextmenu",this,!0),t.dragAborted=!0,t.dragActive&&(o.resetTabPositions(this.contentNode.children,this._orientation),t.override.dispose(),t.tab.classList.remove("p-mod-dragging"),this.removeClass("p-mod-dragging")))},e.prototype._adjustCurrentForInsert=function(t,e){var n=this.currentTitle,r=this._currentIndex,i=this.insertBehavior;if("select-tab"===i||"select-tab-if-needed"===i&&-1===r)return this._currentIndex=t,this._previousTitle=n,void this._currentChanged.emit({previousIndex:r,previousTitle:n,currentIndex:t,currentTitle:e});r>=t&&this._currentIndex++},e.prototype._adjustCurrentForMove=function(t,e){this._currentIndex===t?this._currentIndex=e:this._currentIndex<t&&this._currentIndex>=e?this._currentIndex++:this._currentIndex>t&&this._currentIndex<=e&&this._currentIndex--},e.prototype._adjustCurrentForRemove=function(t,e){var n=this._currentIndex,r=this.removeBehavior;if(n===t){if(0===this._titles.length)return this._currentIndex=-1,void this._currentChanged.emit({previousIndex:t,previousTitle:e,currentIndex:-1,currentTitle:null});if("select-tab-after"===r)return this._currentIndex=Math.min(t,this._titles.length-1),void this._currentChanged.emit({previousIndex:t,previousTitle:e,currentIndex:this._currentIndex,currentTitle:this.currentTitle});if("select-tab-before"===r)return this._currentIndex=Math.max(0,t-1),void this._currentChanged.emit({previousIndex:t,previousTitle:e,currentIndex:this._currentIndex,currentTitle:this.currentTitle});if("select-previous-tab"===r)return this._previousTitle?(this._currentIndex=this._titles.indexOf(this._previousTitle),this._previousTitle=null):this._currentIndex=Math.min(t,this._titles.length-1),void this._currentChanged.emit({previousIndex:t,previousTitle:e,currentIndex:this._currentIndex,currentTitle:this.currentTitle});this._currentIndex=-1,this._currentChanged.emit({previousIndex:t,previousTitle:e,currentIndex:-1,currentTitle:null})}else n>t&&this._currentIndex--},e.prototype._onTitleChanged=function(t){this.update()},e}(h.Widget);e.TabBar=d,function(t){var e=function(){function t(){this.closeIconSelector=".p-TabBar-tabCloseIcon",this._tabID=0,this._tabKeys=new WeakMap}return t.prototype.renderTab=function(t){var e=t.title.caption,n=this.createTabKey(t),r=this.createTabStyle(t),i=this.createTabClass(t),o=this.createTabDataset(t);return u.h.li({key:n,className:i,title:e,style:r,dataset:o},this.renderIcon(t),this.renderLabel(t),this.renderCloseIcon(t))},t.prototype.renderIcon=function(t){var e=this.createIconClass(t);return u.h.div({className:e},t.title.iconLabel)},t.prototype.renderLabel=function(t){return u.h.div({className:"p-TabBar-tabLabel"},t.title.label)},t.prototype.renderCloseIcon=function(t){return u.h.div({className:"p-TabBar-tabCloseIcon"})},t.prototype.createTabKey=function(t){var e=this._tabKeys.get(t.title);return void 0===e&&(e="tab-key-"+this._tabID++,this._tabKeys.set(t.title,e)),e},t.prototype.createTabStyle=function(t){return{zIndex:""+t.zIndex}},t.prototype.createTabClass=function(t){var e="p-TabBar-tab";return t.title.className&&(e+=" "+t.title.className),t.title.closable&&(e+=" p-mod-closable"),t.current&&(e+=" p-mod-current"),e},t.prototype.createTabDataset=function(t){return t.title.dataset},t.prototype.createIconClass=function(t){var e="p-TabBar-tabIcon",n=t.title.iconClass;return n?e+" "+n:e},t}();t.Renderer=e,t.defaultRenderer=new e}(d=e.TabBar||(e.TabBar={})),e.TabBar=d,function(t){t.DRAG_THRESHOLD=5,t.DETACH_THRESHOLD=20,t.createNode=function(){var t=document.createElement("div"),e=document.createElement("ul");return e.className="p-TabBar-content",t.appendChild(e),t},t.asTitle=function(t){return t instanceof g.Title?t:new g.Title(t)},t.parseTransitionDuration=function(t){var e=window.getComputedStyle(t);return 1e3*(parseFloat(e.transitionDuration)||0)},t.snapTabLayout=function(t,e){for(var n=new Array(t.length),r=0,i=t.length;r<i;++r){var o=t[r],a=window.getComputedStyle(o);n[r]="horizontal"===e?{pos:o.offsetLeft,size:o.offsetWidth,margin:parseFloat(a.marginLeft)||0}:{pos:o.offsetTop,size:o.offsetHeight,margin:parseFloat(a.marginTop)||0}}return n},t.dragExceeded=function(e,n){var r=Math.abs(n.clientX-e.pressX),i=Math.abs(n.clientY-e.pressY);return r>=t.DRAG_THRESHOLD||i>=t.DRAG_THRESHOLD},t.detachExceeded=function(e,n){var r=e.contentRect;return n.clientX<r.left-t.DETACH_THRESHOLD||n.clientX>=r.right+t.DETACH_THRESHOLD||n.clientY<r.top-t.DETACH_THRESHOLD||n.clientY>=r.bottom+t.DETACH_THRESHOLD},t.layoutTabs=function(t,e,n,r){var i,o,a,s;"horizontal"===r?(i=e.pressX,o=n.clientX-e.contentRect.left,a=n.clientX,s=e.contentRect.width):(i=e.pressY,o=n.clientY-e.contentRect.top,a=n.clientY,s=e.contentRect.height);for(var A=e.index,l=o-e.tabPressPos,c=l+e.tabSize,u=0,g=t.length;u<g;++u){var h=void 0,d=e.tabLayout[u],f=d.pos+(d.size>>1);if(u<e.index&&l<f)h=e.tabSize+e.tabLayout[u+1].margin+"px",A=Math.min(A,u);else if(u>e.index&&c>f)h=-e.tabSize-d.margin+"px",A=Math.max(A,u);else if(u===e.index){var p=a-i,I=s-(e.tabPos+e.tabSize);h=Math.max(-e.tabPos,Math.min(p,I))+"px"}else h="";"horizontal"===r?t[u].style.left=h:t[u].style.top=h}e.targetIndex=A},t.finalizeTabPosition=function(t,e){var n,r;if(n="horizontal"===e?t.contentRect.width:t.contentRect.height,t.targetIndex===t.index)r=0;else if(t.targetIndex>t.index)r=(i=t.tabLayout[t.targetIndex]).pos+i.size-t.tabSize-t.tabPos;else{var i;r=(i=t.tabLayout[t.targetIndex]).pos-t.tabPos}var o=n-(t.tabPos+t.tabSize),a=Math.max(-t.tabPos,Math.min(r,o));"horizontal"===e?t.tab.style.left=a+"px":t.tab.style.top=a+"px"},t.resetTabPositions=function(t,e){a.each(t,function(t){"horizontal"===e?t.style.left="":t.style.top=""})}}(o||(o={}))},function(t,e,n){!function(t){"use strict";t.modeInfo=[{name:"APL",mime:"text/apl",mode:"apl",ext:["dyalog","apl"]},{name:"PGP",mimes:["application/pgp","application/pgp-encrypted","application/pgp-keys","application/pgp-signature"],mode:"asciiarmor",ext:["asc","pgp","sig"]},{name:"ASN.1",mime:"text/x-ttcn-asn",mode:"asn.1",ext:["asn","asn1"]},{name:"Asterisk",mime:"text/x-asterisk",mode:"asterisk",file:/^extensions\.conf$/i},{name:"Brainfuck",mime:"text/x-brainfuck",mode:"brainfuck",ext:["b","bf"]},{name:"C",mime:"text/x-csrc",mode:"clike",ext:["c","h","ino"]},{name:"C++",mime:"text/x-c++src",mode:"clike",ext:["cpp","c++","cc","cxx","hpp","h++","hh","hxx"],alias:["cpp"]},{name:"Cobol",mime:"text/x-cobol",mode:"cobol",ext:["cob","cpy"]},{name:"C#",mime:"text/x-csharp",mode:"clike",ext:["cs"],alias:["csharp"]},{name:"Clojure",mime:"text/x-clojure",mode:"clojure",ext:["clj","cljc","cljx"]},{name:"ClojureScript",mime:"text/x-clojurescript",mode:"clojure",ext:["cljs"]},{name:"Closure Stylesheets (GSS)",mime:"text/x-gss",mode:"css",ext:["gss"]},{name:"CMake",mime:"text/x-cmake",mode:"cmake",ext:["cmake","cmake.in"],file:/^CMakeLists.txt$/},{name:"CoffeeScript",mimes:["application/vnd.coffeescript","text/coffeescript","text/x-coffeescript"],mode:"coffeescript",ext:["coffee"],alias:["coffee","coffee-script"]},{name:"Common Lisp",mime:"text/x-common-lisp",mode:"commonlisp",ext:["cl","lisp","el"],alias:["lisp"]},{name:"Cypher",mime:"application/x-cypher-query",mode:"cypher",ext:["cyp","cypher"]},{name:"Cython",mime:"text/x-cython",mode:"python",ext:["pyx","pxd","pxi"]},{name:"Crystal",mime:"text/x-crystal",mode:"crystal",ext:["cr"]},{name:"CSS",mime:"text/css",mode:"css",ext:["css"]},{name:"CQL",mime:"text/x-cassandra",mode:"sql",ext:["cql"]},{name:"D",mime:"text/x-d",mode:"d",ext:["d"]},{name:"Dart",mimes:["application/dart","text/x-dart"],mode:"dart",ext:["dart"]},{name:"diff",mime:"text/x-diff",mode:"diff",ext:["diff","patch"]},{name:"Django",mime:"text/x-django",mode:"django"},{name:"Dockerfile",mime:"text/x-dockerfile",mode:"dockerfile",file:/^Dockerfile$/},{name:"DTD",mime:"application/xml-dtd",mode:"dtd",ext:["dtd"]},{name:"Dylan",mime:"text/x-dylan",mode:"dylan",ext:["dylan","dyl","intr"]},{name:"EBNF",mime:"text/x-ebnf",mode:"ebnf"},{name:"ECL",mime:"text/x-ecl",mode:"ecl",ext:["ecl"]},{name:"edn",mime:"application/edn",mode:"clojure",ext:["edn"]},{name:"Eiffel",mime:"text/x-eiffel",mode:"eiffel",ext:["e"]},{name:"Elm",mime:"text/x-elm",mode:"elm",ext:["elm"]},{name:"Embedded Javascript",mime:"application/x-ejs",mode:"htmlembedded",ext:["ejs"]},{name:"Embedded Ruby",mime:"application/x-erb",mode:"htmlembedded",ext:["erb"]},{name:"Erlang",mime:"text/x-erlang",mode:"erlang",ext:["erl"]},{name:"Esper",mime:"text/x-esper",mode:"sql"},{name:"Factor",mime:"text/x-factor",mode:"factor",ext:["factor"]},{name:"FCL",mime:"text/x-fcl",mode:"fcl"},{name:"Forth",mime:"text/x-forth",mode:"forth",ext:["forth","fth","4th"]},{name:"Fortran",mime:"text/x-fortran",mode:"fortran",ext:["f","for","f77","f90","f95"]},{name:"F#",mime:"text/x-fsharp",mode:"mllike",ext:["fs"],alias:["fsharp"]},{name:"Gas",mime:"text/x-gas",mode:"gas",ext:["s"]},{name:"Gherkin",mime:"text/x-feature",mode:"gherkin",ext:["feature"]},{name:"GitHub Flavored Markdown",mime:"text/x-gfm",mode:"gfm",file:/^(readme|contributing|history).md$/i},{name:"Go",mime:"text/x-go",mode:"go",ext:["go"]},{name:"Groovy",mime:"text/x-groovy",mode:"groovy",ext:["groovy","gradle"],file:/^Jenkinsfile$/},{name:"HAML",mime:"text/x-haml",mode:"haml",ext:["haml"]},{name:"Haskell",mime:"text/x-haskell",mode:"haskell",ext:["hs"]},{name:"Haskell (Literate)",mime:"text/x-literate-haskell",mode:"haskell-literate",ext:["lhs"]},{name:"Haxe",mime:"text/x-haxe",mode:"haxe",ext:["hx"]},{name:"HXML",mime:"text/x-hxml",mode:"haxe",ext:["hxml"]},{name:"ASP.NET",mime:"application/x-aspx",mode:"htmlembedded",ext:["aspx"],alias:["asp","aspx"]},{name:"HTML",mime:"text/html",mode:"htmlmixed",ext:["html","htm","handlebars","hbs"],alias:["xhtml"]},{name:"HTTP",mime:"message/http",mode:"http"},{name:"IDL",mime:"text/x-idl",mode:"idl",ext:["pro"]},{name:"Pug",mime:"text/x-pug",mode:"pug",ext:["jade","pug"],alias:["jade"]},{name:"Java",mime:"text/x-java",mode:"clike",ext:["java"]},{name:"Java Server Pages",mime:"application/x-jsp",mode:"htmlembedded",ext:["jsp"],alias:["jsp"]},{name:"JavaScript",mimes:["text/javascript","text/ecmascript","application/javascript","application/x-javascript","application/ecmascript"],mode:"javascript",ext:["js"],alias:["ecmascript","js","node"]},{name:"JSON",mimes:["application/json","application/x-json"],mode:"javascript",ext:["json","map"],alias:["json5"]},{name:"JSON-LD",mime:"application/ld+json",mode:"javascript",ext:["jsonld"],alias:["jsonld"]},{name:"JSX",mime:"text/jsx",mode:"jsx",ext:["jsx"]},{name:"Jinja2",mime:"text/jinja2",mode:"jinja2",ext:["j2","jinja","jinja2"]},{name:"Julia",mime:"text/x-julia",mode:"julia",ext:["jl"]},{name:"Kotlin",mime:"text/x-kotlin",mode:"clike",ext:["kt"]},{name:"LESS",mime:"text/x-less",mode:"css",ext:["less"]},{name:"LiveScript",mime:"text/x-livescript",mode:"livescript",ext:["ls"],alias:["ls"]},{name:"Lua",mime:"text/x-lua",mode:"lua",ext:["lua"]},{name:"Markdown",mime:"text/x-markdown",mode:"markdown",ext:["markdown","md","mkd"]},{name:"mIRC",mime:"text/mirc",mode:"mirc"},{name:"MariaDB SQL",mime:"text/x-mariadb",mode:"sql"},{name:"Mathematica",mime:"text/x-mathematica",mode:"mathematica",ext:["m","nb"]},{name:"Modelica",mime:"text/x-modelica",mode:"modelica",ext:["mo"]},{name:"MUMPS",mime:"text/x-mumps",mode:"mumps",ext:["mps"]},{name:"MS SQL",mime:"text/x-mssql",mode:"sql"},{name:"mbox",mime:"application/mbox",mode:"mbox",ext:["mbox"]},{name:"MySQL",mime:"text/x-mysql",mode:"sql"},{name:"Nginx",mime:"text/x-nginx-conf",mode:"nginx",file:/nginx.*\.conf$/i},{name:"NSIS",mime:"text/x-nsis",mode:"nsis",ext:["nsh","nsi"]},{name:"NTriples",mimes:["application/n-triples","application/n-quads","text/n-triples"],mode:"ntriples",ext:["nt","nq"]},{name:"Objective-C",mime:"text/x-objectivec",mode:"clike",ext:["m","mm"],alias:["objective-c","objc"]},{name:"OCaml",mime:"text/x-ocaml",mode:"mllike",ext:["ml","mli","mll","mly"]},{name:"Octave",mime:"text/x-octave",mode:"octave",ext:["m"]},{name:"Oz",mime:"text/x-oz",mode:"oz",ext:["oz"]},{name:"Pascal",mime:"text/x-pascal",mode:"pascal",ext:["p","pas"]},{name:"PEG.js",mime:"null",mode:"pegjs",ext:["jsonld"]},{name:"Perl",mime:"text/x-perl",mode:"perl",ext:["pl","pm"]},{name:"PHP",mimes:["text/x-php","application/x-httpd-php","application/x-httpd-php-open"],mode:"php",ext:["php","php3","php4","php5","php7","phtml"]},{name:"Pig",mime:"text/x-pig",mode:"pig",ext:["pig"]},{name:"Plain Text",mime:"text/plain",mode:"null",ext:["txt","text","conf","def","list","log"]},{name:"PLSQL",mime:"text/x-plsql",mode:"sql",ext:["pls"]},{name:"PowerShell",mime:"application/x-powershell",mode:"powershell",ext:["ps1","psd1","psm1"]},{name:"Properties files",mime:"text/x-properties",mode:"properties",ext:["properties","ini","in"],alias:["ini","properties"]},{name:"ProtoBuf",mime:"text/x-protobuf",mode:"protobuf",ext:["proto"]},{name:"Python",mime:"text/x-python",mode:"python",ext:["BUILD","bzl","py","pyw"],file:/^(BUCK|BUILD)$/},{name:"Puppet",mime:"text/x-puppet",mode:"puppet",ext:["pp"]},{name:"Q",mime:"text/x-q",mode:"q",ext:["q"]},{name:"R",mime:"text/x-rsrc",mode:"r",ext:["r","R"],alias:["rscript"]},{name:"reStructuredText",mime:"text/x-rst",mode:"rst",ext:["rst"],alias:["rst"]},{name:"RPM Changes",mime:"text/x-rpm-changes",mode:"rpm"},{name:"RPM Spec",mime:"text/x-rpm-spec",mode:"rpm",ext:["spec"]},{name:"Ruby",mime:"text/x-ruby",mode:"ruby",ext:["rb"],alias:["jruby","macruby","rake","rb","rbx"]},{name:"Rust",mime:"text/x-rustsrc",mode:"rust",ext:["rs"]},{name:"SAS",mime:"text/x-sas",mode:"sas",ext:["sas"]},{name:"Sass",mime:"text/x-sass",mode:"sass",ext:["sass"]},{name:"Scala",mime:"text/x-scala",mode:"clike",ext:["scala"]},{name:"Scheme",mime:"text/x-scheme",mode:"scheme",ext:["scm","ss"]},{name:"SCSS",mime:"text/x-scss",mode:"css",ext:["scss"]},{name:"Shell",mimes:["text/x-sh","application/x-sh"],mode:"shell",ext:["sh","ksh","bash"],alias:["bash","sh","zsh"],file:/^PKGBUILD$/},{name:"Sieve",mime:"application/sieve",mode:"sieve",ext:["siv","sieve"]},{name:"Slim",mimes:["text/x-slim","application/x-slim"],mode:"slim",ext:["slim"]},{name:"Smalltalk",mime:"text/x-stsrc",mode:"smalltalk",ext:["st"]},{name:"Smarty",mime:"text/x-smarty",mode:"smarty",ext:["tpl"]},{name:"Solr",mime:"text/x-solr",mode:"solr"},{name:"SML",mime:"text/x-sml",mode:"mllike",ext:["sml","sig","fun","smackspec"]},{name:"Soy",mime:"text/x-soy",mode:"soy",ext:["soy"],alias:["closure template"]},{name:"SPARQL",mime:"application/sparql-query",mode:"sparql",ext:["rq","sparql"],alias:["sparul"]},{name:"Spreadsheet",mime:"text/x-spreadsheet",mode:"spreadsheet",alias:["excel","formula"]},{name:"SQL",mime:"text/x-sql",mode:"sql",ext:["sql"]},{name:"SQLite",mime:"text/x-sqlite",mode:"sql"},{name:"Squirrel",mime:"text/x-squirrel",mode:"clike",ext:["nut"]},{name:"Stylus",mime:"text/x-styl",mode:"stylus",ext:["styl"]},{name:"Swift",mime:"text/x-swift",mode:"swift",ext:["swift"]},{name:"sTeX",mime:"text/x-stex",mode:"stex"},{name:"LaTeX",mime:"text/x-latex",mode:"stex",ext:["text","ltx","tex"],alias:["tex"]},{name:"SystemVerilog",mime:"text/x-systemverilog",mode:"verilog",ext:["v","sv","svh"]},{name:"Tcl",mime:"text/x-tcl",mode:"tcl",ext:["tcl"]},{name:"Textile",mime:"text/x-textile",mode:"textile",ext:["textile"]},{name:"TiddlyWiki ",mime:"text/x-tiddlywiki",mode:"tiddlywiki"},{name:"Tiki wiki",mime:"text/tiki",mode:"tiki"},{name:"TOML",mime:"text/x-toml",mode:"toml",ext:["toml"]},{name:"Tornado",mime:"text/x-tornado",mode:"tornado"},{name:"troff",mime:"text/troff",mode:"troff",ext:["1","2","3","4","5","6","7","8","9"]},{name:"TTCN",mime:"text/x-ttcn",mode:"ttcn",ext:["ttcn","ttcn3","ttcnpp"]},{name:"TTCN_CFG",mime:"text/x-ttcn-cfg",mode:"ttcn-cfg",ext:["cfg"]},{name:"Turtle",mime:"text/turtle",mode:"turtle",ext:["ttl"]},{name:"TypeScript",mime:"application/typescript",mode:"javascript",ext:["ts"],alias:["ts"]},{name:"TypeScript-JSX",mime:"text/typescript-jsx",mode:"jsx",ext:["tsx"],alias:["tsx"]},{name:"Twig",mime:"text/x-twig",mode:"twig"},{name:"Web IDL",mime:"text/x-webidl",mode:"webidl",ext:["webidl"]},{name:"VB.NET",mime:"text/x-vb",mode:"vb",ext:["vb"]},{name:"VBScript",mime:"text/vbscript",mode:"vbscript",ext:["vbs"]},{name:"Velocity",mime:"text/velocity",mode:"velocity",ext:["vtl"]},{name:"Verilog",mime:"text/x-verilog",mode:"verilog",ext:["v"]},{name:"VHDL",mime:"text/x-vhdl",mode:"vhdl",ext:["vhd","vhdl"]},{name:"Vue.js Component",mimes:["script/x-vue","text/x-vue"],mode:"vue",ext:["vue"]},{name:"XML",mimes:["application/xml","text/xml"],mode:"xml",ext:["xml","xsl","xsd","svg"],alias:["rss","wsdl","xsd"]},{name:"XQuery",mime:"application/xquery",mode:"xquery",ext:["xy","xquery"]},{name:"Yacas",mime:"text/x-yacas",mode:"yacas",ext:["ys"]},{name:"YAML",mimes:["text/x-yaml","text/yaml"],mode:"yaml",ext:["yaml","yml"],alias:["yml"]},{name:"Z80",mime:"text/x-z80",mode:"z80",ext:["z80"]},{name:"mscgen",mime:"text/x-mscgen",mode:"mscgen",ext:["mscgen","mscin","msc"]},{name:"xu",mime:"text/x-xu",mode:"mscgen",ext:["xu"]},{name:"msgenny",mime:"text/x-msgenny",mode:"mscgen",ext:["msgenny"]}];for(var e=0;e<t.modeInfo.length;e++){var n=t.modeInfo[e];n.mimes&&(n.mime=n.mimes[0])}t.findModeByMIME=function(e){e=e.toLowerCase();for(var n=0;n<t.modeInfo.length;n++){var r=t.modeInfo[n];if(r.mime==e)return r;if(r.mimes)for(var i=0;i<r.mimes.length;i++)if(r.mimes[i]==e)return r}return/\+xml$/.test(e)?t.findModeByMIME("application/xml"):/\+json$/.test(e)?t.findModeByMIME("application/json"):void 0},t.findModeByExtension=function(e){for(var n=0;n<t.modeInfo.length;n++){var r=t.modeInfo[n];if(r.ext)for(var i=0;i<r.ext.length;i++)if(r.ext[i]==e)return r}},t.findModeByFileName=function(e){for(var n=0;n<t.modeInfo.length;n++){var r=t.modeInfo[n];if(r.file&&r.file.test(e))return r}var i=e.lastIndexOf("."),o=i>-1&&e.substring(i+1,e.length);if(o)return t.findModeByExtension(o)},t.findModeByName=function(e){e=e.toLowerCase();for(var n=0;n<t.modeInfo.length;n++){var r=t.modeInfo[n];if(r.name.toLowerCase()==e)return r;if(r.alias)for(var i=0;i<r.alias.length;i++)if(r.alias[i].toLowerCase()==e)return r}}}(n(15))},function(t,e,n){!function(t){"use strict";var e,n,r=t.Pos;function i(t,e){for(var n=function(t){var e=t.flags;return null!=e?e:(t.ignoreCase?"i":"")+(t.global?"g":"")+(t.multiline?"m":"")}(t),r=n,i=0;i<e.length;i++)-1==r.indexOf(e.charAt(i))&&(r+=e.charAt(i));return n==r?t:new RegExp(t.source,r)}function o(t,e,n){e=i(e,"g");for(var o=n.line,a=n.ch,s=t.lastLine();o<=s;o++,a=0){e.lastIndex=a;var A=t.getLine(o),l=e.exec(A);if(l)return{from:r(o,l.index),to:r(o,l.index+l[0].length),match:l}}}function a(t,e){for(var n,r=0;;){e.lastIndex=r;var i=e.exec(t);if(!i)return n;if((r=(n=i).index+(n[0].length||1))==t.length)return n}}function s(t,e,n,r){if(t.length==e.length)return n;for(var i=0,o=n+Math.max(0,t.length-e.length);;){if(i==o)return i;var a=i+o>>1,s=r(t.slice(0,a)).length;if(s==n)return a;s>n?o=a:i=a+1}}function A(t,A,l,c){var u;this.atOccurrence=!1,this.doc=t,l=l?t.clipPos(l):r(0,0),this.pos={from:l,to:l},"object"==typeof c?u=c.caseFold:(u=c,c=null),"string"==typeof A?(null==u&&(u=!1),this.matches=function(i,o){return(i?function(t,i,o,a){if(!i.length)return null;var A=a?e:n,l=A(i).split(/\r|\n\r?/);t:for(var c=o.line,u=o.ch,g=t.firstLine()-1+l.length;c>=g;c--,u=-1){var h=t.getLine(c);u>-1&&(h=h.slice(0,u));var d=A(h);if(1==l.length){var f=d.lastIndexOf(l[0]);if(-1==f)continue t;return{from:r(c,s(h,d,f,A)),to:r(c,s(h,d,f+l[0].length,A))}}var p=l[l.length-1];if(d.slice(0,p.length)==p){for(var I=1,o=c-l.length+1;I<l.length-1;I++)if(A(t.getLine(o+I))!=l[I])continue t;var C=t.getLine(c+1-l.length),m=A(C);if(m.slice(m.length-l[0].length)==l[0])return{from:r(c+1-l.length,s(C,m,C.length-l[0].length,A)),to:r(c,s(h,d,p.length,A))}}}}:function(t,i,o,a){if(!i.length)return null;var A=a?e:n,l=A(i).split(/\r|\n\r?/);t:for(var c=o.line,u=o.ch,g=t.lastLine()+1-l.length;c<=g;c++,u=0){var h=t.getLine(c).slice(u),d=A(h);if(1==l.length){var f=d.indexOf(l[0]);if(-1==f)continue t;var o=s(h,d,f,A)+u;return{from:r(c,s(h,d,f,A)+u),to:r(c,s(h,d,f+l[0].length,A)+u)}}var p=d.length-l[0].length;if(d.slice(p)==l[0]){for(var I=1;I<l.length-1;I++)if(A(t.getLine(c+I))!=l[I])continue t;var C=t.getLine(c+l.length-1),m=A(C),y=l[l.length-1];if(m.slice(0,y.length)==y)return{from:r(c,s(h,d,p,A)+u),to:r(c+l.length-1,s(C,m,y.length,A))}}}})(t,A,o,u)}):(A=i(A,"gm"),c&&!1===c.multiline?this.matches=function(e,n){return(e?function(t,e,n){e=i(e,"g");for(var o=n.line,s=n.ch,A=t.firstLine();o>=A;o--,s=-1){var l=t.getLine(o);s>-1&&(l=l.slice(0,s));var c=a(l,e);if(c)return{from:r(o,c.index),to:r(o,c.index+c[0].length),match:c}}}:o)(t,A,n)}:this.matches=function(e,n){return(e?function(t,e,n){e=i(e,"gm");for(var o,s=1,A=n.line,l=t.firstLine();A>=l;){for(var c=0;c<s;c++){var u=t.getLine(A--);o=null==o?u.slice(0,n.ch):u+"\n"+o}s*=2;var g=a(o,e);if(g){var h=o.slice(0,g.index).split("\n"),d=g[0].split("\n"),f=A+h.length,p=h[h.length-1].length;return{from:r(f,p),to:r(f+d.length-1,1==d.length?p+d[0].length:d[d.length-1].length),match:g}}}}:function(t,e,n){if(!function(t){return/\\s|\\n|\n|\\W|\\D|\[\^/.test(t.source)}(e))return o(t,e,n);e=i(e,"gm");for(var a,s=1,A=n.line,l=t.lastLine();A<=l;){for(var c=0;c<s&&!(A>l);c++){var u=t.getLine(A++);a=null==a?u:a+"\n"+u}s*=2,e.lastIndex=n.ch;var g=e.exec(a);if(g){var h=a.slice(0,g.index).split("\n"),d=g[0].split("\n"),f=n.line+h.length-1,p=h[h.length-1].length;return{from:r(f,p),to:r(f+d.length-1,1==d.length?p+d[0].length:d[d.length-1].length),match:g}}}})(t,A,n)})}String.prototype.normalize?(e=function(t){return t.normalize("NFD").toLowerCase()},n=function(t){return t.normalize("NFD")}):(e=function(t){return t.toLowerCase()},n=function(t){return t}),A.prototype={findNext:function(){return this.find(!1)},findPrevious:function(){return this.find(!0)},find:function(e){for(var n=this.matches(e,this.doc.clipPos(e?this.pos.from:this.pos.to));n&&0==t.cmpPos(n.from,n.to);)e?n.from.ch?n.from=r(n.from.line,n.from.ch-1):n=n.from.line==this.doc.firstLine()?null:this.matches(e,this.doc.clipPos(r(n.from.line-1))):n.to.ch<this.doc.getLine(n.to.line).length?n.to=r(n.to.line,n.to.ch+1):n=n.to.line==this.doc.lastLine()?null:this.matches(e,r(n.to.line+1,0));if(n)return this.pos=n,this.atOccurrence=!0,this.pos.match||!0;var i=r(e?this.doc.firstLine():this.doc.lastLine()+1,0);return this.pos={from:i,to:i},this.atOccurrence=!1},from:function(){if(this.atOccurrence)return this.pos.from},to:function(){if(this.atOccurrence)return this.pos.to},replace:function(e,n){if(this.atOccurrence){var i=t.splitLines(e);this.doc.replaceRange(i,this.pos.from,this.pos.to,n),this.pos.to=r(this.pos.from.line+i.length-1,i[i.length-1].length+(1==i.length?this.pos.from.ch:0))}}},t.defineExtension("getSearchCursor",function(t,e,n){return new A(this.doc,t,e,n)}),t.defineDocExtension("getSearchCursor",function(t,e,n){return new A(this,t,e,n)}),t.defineExtension("selectMatches",function(e,n){for(var r=[],i=this.getSearchCursor(e,this.getCursor("from"),n);i.findNext()&&!(t.cmpPos(i.to(),this.getCursor("to"))>0);)r.push({anchor:i.from(),head:i.to()});r.length&&this.setSelections(r,0)})}(n(15))},function(t,e,n){"use strict";
/*
object-assign
(c) Sindre Sorhus
@license MIT
*/var r=Object.getOwnPropertySymbols,i=Object.prototype.hasOwnProperty,o=Object.prototype.propertyIsEnumerable;function a(t){if(null==t)throw new TypeError("Object.assign cannot be called with null or undefined");return Object(t)}t.exports=function(){try{if(!Object.assign)return!1;var t=new String("abc");if(t[5]="de","5"===Object.getOwnPropertyNames(t)[0])return!1;for(var e={},n=0;n<10;n++)e["_"+String.fromCharCode(n)]=n;if("0123456789"!==Object.getOwnPropertyNames(e).map(function(t){return e[t]}).join(""))return!1;var r={};return"abcdefghijklmnopqrst".split("").forEach(function(t){r[t]=t}),"abcdefghijklmnopqrst"===Object.keys(Object.assign({},r)).join("")}catch(t){return!1}}()?Object.assign:function(t,e){for(var n,s,A=a(t),l=1;l<arguments.length;l++){for(var c in n=Object(arguments[l]))i.call(n,c)&&(A[c]=n[c]);if(r){s=r(n);for(var u=0;u<s.length;u++)o.call(n,s[u])&&(A[s[u]]=n[s[u]])}}return A}},function(t,e,n){"use strict";function r(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}Object.defineProperty(e,"__esModule",{value:!0}),r(n(584)),r(n(1075)),r(n(216))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(1084);!function(t){t.isAvailable=function(){return r.DefaultTerminalSession.isAvailable()},t.startNew=function(t){return r.DefaultTerminalSession.startNew(t)},t.connectTo=function(t,e){return r.DefaultTerminalSession.connectTo(t,e)},t.listRunning=function(t){return r.DefaultTerminalSession.listRunning(t)},t.shutdown=function(t,e){return r.DefaultTerminalSession.shutdown(t,e)},t.shutdownAll=function(t){return r.DefaultTerminalSession.shutdownAll(t)}}(e.TerminalSession||(e.TerminalSession={}))},function(t){t.exports=JSON.parse('{"Aacute":"Á","aacute":"á","Abreve":"Ă","abreve":"ă","ac":"∾","acd":"∿","acE":"∾̳","Acirc":"Â","acirc":"â","acute":"´","Acy":"А","acy":"а","AElig":"Æ","aelig":"æ","af":"⁡","Afr":"𝔄","afr":"𝔞","Agrave":"À","agrave":"à","alefsym":"ℵ","aleph":"ℵ","Alpha":"Α","alpha":"α","Amacr":"Ā","amacr":"ā","amalg":"⨿","amp":"&","AMP":"&","andand":"⩕","And":"⩓","and":"∧","andd":"⩜","andslope":"⩘","andv":"⩚","ang":"∠","ange":"⦤","angle":"∠","angmsdaa":"⦨","angmsdab":"⦩","angmsdac":"⦪","angmsdad":"⦫","angmsdae":"⦬","angmsdaf":"⦭","angmsdag":"⦮","angmsdah":"⦯","angmsd":"∡","angrt":"∟","angrtvb":"⊾","angrtvbd":"⦝","angsph":"∢","angst":"Å","angzarr":"⍼","Aogon":"Ą","aogon":"ą","Aopf":"𝔸","aopf":"𝕒","apacir":"⩯","ap":"≈","apE":"⩰","ape":"≊","apid":"≋","apos":"\'","ApplyFunction":"⁡","approx":"≈","approxeq":"≊","Aring":"Å","aring":"å","Ascr":"𝒜","ascr":"𝒶","Assign":"≔","ast":"*","asymp":"≈","asympeq":"≍","Atilde":"Ã","atilde":"ã","Auml":"Ä","auml":"ä","awconint":"∳","awint":"⨑","backcong":"≌","backepsilon":"϶","backprime":"‵","backsim":"∽","backsimeq":"⋍","Backslash":"∖","Barv":"⫧","barvee":"⊽","barwed":"⌅","Barwed":"⌆","barwedge":"⌅","bbrk":"⎵","bbrktbrk":"⎶","bcong":"≌","Bcy":"Б","bcy":"б","bdquo":"„","becaus":"∵","because":"∵","Because":"∵","bemptyv":"⦰","bepsi":"϶","bernou":"ℬ","Bernoullis":"ℬ","Beta":"Β","beta":"β","beth":"ℶ","between":"≬","Bfr":"𝔅","bfr":"𝔟","bigcap":"⋂","bigcirc":"◯","bigcup":"⋃","bigodot":"⨀","bigoplus":"⨁","bigotimes":"⨂","bigsqcup":"⨆","bigstar":"★","bigtriangledown":"▽","bigtriangleup":"△","biguplus":"⨄","bigvee":"⋁","bigwedge":"⋀","bkarow":"⤍","blacklozenge":"⧫","blacksquare":"▪","blacktriangle":"▴","blacktriangledown":"▾","blacktriangleleft":"◂","blacktriangleright":"▸","blank":"␣","blk12":"▒","blk14":"░","blk34":"▓","block":"█","bne":"=⃥","bnequiv":"≡⃥","bNot":"⫭","bnot":"⌐","Bopf":"𝔹","bopf":"𝕓","bot":"⊥","bottom":"⊥","bowtie":"⋈","boxbox":"⧉","boxdl":"┐","boxdL":"╕","boxDl":"╖","boxDL":"╗","boxdr":"┌","boxdR":"╒","boxDr":"╓","boxDR":"╔","boxh":"─","boxH":"═","boxhd":"┬","boxHd":"╤","boxhD":"╥","boxHD":"╦","boxhu":"┴","boxHu":"╧","boxhU":"╨","boxHU":"╩","boxminus":"⊟","boxplus":"⊞","boxtimes":"⊠","boxul":"┘","boxuL":"╛","boxUl":"╜","boxUL":"╝","boxur":"└","boxuR":"╘","boxUr":"╙","boxUR":"╚","boxv":"│","boxV":"║","boxvh":"┼","boxvH":"╪","boxVh":"╫","boxVH":"╬","boxvl":"┤","boxvL":"╡","boxVl":"╢","boxVL":"╣","boxvr":"├","boxvR":"╞","boxVr":"╟","boxVR":"╠","bprime":"‵","breve":"˘","Breve":"˘","brvbar":"¦","bscr":"𝒷","Bscr":"ℬ","bsemi":"⁏","bsim":"∽","bsime":"⋍","bsolb":"⧅","bsol":"\\\\","bsolhsub":"⟈","bull":"•","bullet":"•","bump":"≎","bumpE":"⪮","bumpe":"≏","Bumpeq":"≎","bumpeq":"≏","Cacute":"Ć","cacute":"ć","capand":"⩄","capbrcup":"⩉","capcap":"⩋","cap":"∩","Cap":"⋒","capcup":"⩇","capdot":"⩀","CapitalDifferentialD":"ⅅ","caps":"∩︀","caret":"⁁","caron":"ˇ","Cayleys":"ℭ","ccaps":"⩍","Ccaron":"Č","ccaron":"č","Ccedil":"Ç","ccedil":"ç","Ccirc":"Ĉ","ccirc":"ĉ","Cconint":"∰","ccups":"⩌","ccupssm":"⩐","Cdot":"Ċ","cdot":"ċ","cedil":"¸","Cedilla":"¸","cemptyv":"⦲","cent":"¢","centerdot":"·","CenterDot":"·","cfr":"𝔠","Cfr":"ℭ","CHcy":"Ч","chcy":"ч","check":"✓","checkmark":"✓","Chi":"Χ","chi":"χ","circ":"ˆ","circeq":"≗","circlearrowleft":"↺","circlearrowright":"↻","circledast":"⊛","circledcirc":"⊚","circleddash":"⊝","CircleDot":"⊙","circledR":"®","circledS":"Ⓢ","CircleMinus":"⊖","CirclePlus":"⊕","CircleTimes":"⊗","cir":"○","cirE":"⧃","cire":"≗","cirfnint":"⨐","cirmid":"⫯","cirscir":"⧂","ClockwiseContourIntegral":"∲","CloseCurlyDoubleQuote":"”","CloseCurlyQuote":"’","clubs":"♣","clubsuit":"♣","colon":":","Colon":"∷","Colone":"⩴","colone":"≔","coloneq":"≔","comma":",","commat":"@","comp":"∁","compfn":"∘","complement":"∁","complexes":"ℂ","cong":"≅","congdot":"⩭","Congruent":"≡","conint":"∮","Conint":"∯","ContourIntegral":"∮","copf":"𝕔","Copf":"ℂ","coprod":"∐","Coproduct":"∐","copy":"©","COPY":"©","copysr":"℗","CounterClockwiseContourIntegral":"∳","crarr":"↵","cross":"✗","Cross":"⨯","Cscr":"𝒞","cscr":"𝒸","csub":"⫏","csube":"⫑","csup":"⫐","csupe":"⫒","ctdot":"⋯","cudarrl":"⤸","cudarrr":"⤵","cuepr":"⋞","cuesc":"⋟","cularr":"↶","cularrp":"⤽","cupbrcap":"⩈","cupcap":"⩆","CupCap":"≍","cup":"∪","Cup":"⋓","cupcup":"⩊","cupdot":"⊍","cupor":"⩅","cups":"∪︀","curarr":"↷","curarrm":"⤼","curlyeqprec":"⋞","curlyeqsucc":"⋟","curlyvee":"⋎","curlywedge":"⋏","curren":"¤","curvearrowleft":"↶","curvearrowright":"↷","cuvee":"⋎","cuwed":"⋏","cwconint":"∲","cwint":"∱","cylcty":"⌭","dagger":"†","Dagger":"‡","daleth":"ℸ","darr":"↓","Darr":"↡","dArr":"⇓","dash":"‐","Dashv":"⫤","dashv":"⊣","dbkarow":"⤏","dblac":"˝","Dcaron":"Ď","dcaron":"ď","Dcy":"Д","dcy":"д","ddagger":"‡","ddarr":"⇊","DD":"ⅅ","dd":"ⅆ","DDotrahd":"⤑","ddotseq":"⩷","deg":"°","Del":"∇","Delta":"Δ","delta":"δ","demptyv":"⦱","dfisht":"⥿","Dfr":"𝔇","dfr":"𝔡","dHar":"⥥","dharl":"⇃","dharr":"⇂","DiacriticalAcute":"´","DiacriticalDot":"˙","DiacriticalDoubleAcute":"˝","DiacriticalGrave":"`","DiacriticalTilde":"˜","diam":"⋄","diamond":"⋄","Diamond":"⋄","diamondsuit":"♦","diams":"♦","die":"¨","DifferentialD":"ⅆ","digamma":"ϝ","disin":"⋲","div":"÷","divide":"÷","divideontimes":"⋇","divonx":"⋇","DJcy":"Ђ","djcy":"ђ","dlcorn":"⌞","dlcrop":"⌍","dollar":"$","Dopf":"𝔻","dopf":"𝕕","Dot":"¨","dot":"˙","DotDot":"⃜","doteq":"≐","doteqdot":"≑","DotEqual":"≐","dotminus":"∸","dotplus":"∔","dotsquare":"⊡","doublebarwedge":"⌆","DoubleContourIntegral":"∯","DoubleDot":"¨","DoubleDownArrow":"⇓","DoubleLeftArrow":"⇐","DoubleLeftRightArrow":"⇔","DoubleLeftTee":"⫤","DoubleLongLeftArrow":"⟸","DoubleLongLeftRightArrow":"⟺","DoubleLongRightArrow":"⟹","DoubleRightArrow":"⇒","DoubleRightTee":"⊨","DoubleUpArrow":"⇑","DoubleUpDownArrow":"⇕","DoubleVerticalBar":"∥","DownArrowBar":"⤓","downarrow":"↓","DownArrow":"↓","Downarrow":"⇓","DownArrowUpArrow":"⇵","DownBreve":"̑","downdownarrows":"⇊","downharpoonleft":"⇃","downharpoonright":"⇂","DownLeftRightVector":"⥐","DownLeftTeeVector":"⥞","DownLeftVectorBar":"⥖","DownLeftVector":"↽","DownRightTeeVector":"⥟","DownRightVectorBar":"⥗","DownRightVector":"⇁","DownTeeArrow":"↧","DownTee":"⊤","drbkarow":"⤐","drcorn":"⌟","drcrop":"⌌","Dscr":"𝒟","dscr":"𝒹","DScy":"Ѕ","dscy":"ѕ","dsol":"⧶","Dstrok":"Đ","dstrok":"đ","dtdot":"⋱","dtri":"▿","dtrif":"▾","duarr":"⇵","duhar":"⥯","dwangle":"⦦","DZcy":"Џ","dzcy":"џ","dzigrarr":"⟿","Eacute":"É","eacute":"é","easter":"⩮","Ecaron":"Ě","ecaron":"ě","Ecirc":"Ê","ecirc":"ê","ecir":"≖","ecolon":"≕","Ecy":"Э","ecy":"э","eDDot":"⩷","Edot":"Ė","edot":"ė","eDot":"≑","ee":"ⅇ","efDot":"≒","Efr":"𝔈","efr":"𝔢","eg":"⪚","Egrave":"È","egrave":"è","egs":"⪖","egsdot":"⪘","el":"⪙","Element":"∈","elinters":"⏧","ell":"ℓ","els":"⪕","elsdot":"⪗","Emacr":"Ē","emacr":"ē","empty":"∅","emptyset":"∅","EmptySmallSquare":"◻","emptyv":"∅","EmptyVerySmallSquare":"▫","emsp13":" ","emsp14":" ","emsp":" ","ENG":"Ŋ","eng":"ŋ","ensp":" ","Eogon":"Ę","eogon":"ę","Eopf":"𝔼","eopf":"𝕖","epar":"⋕","eparsl":"⧣","eplus":"⩱","epsi":"ε","Epsilon":"Ε","epsilon":"ε","epsiv":"ϵ","eqcirc":"≖","eqcolon":"≕","eqsim":"≂","eqslantgtr":"⪖","eqslantless":"⪕","Equal":"⩵","equals":"=","EqualTilde":"≂","equest":"≟","Equilibrium":"⇌","equiv":"≡","equivDD":"⩸","eqvparsl":"⧥","erarr":"⥱","erDot":"≓","escr":"ℯ","Escr":"ℰ","esdot":"≐","Esim":"⩳","esim":"≂","Eta":"Η","eta":"η","ETH":"Ð","eth":"ð","Euml":"Ë","euml":"ë","euro":"€","excl":"!","exist":"∃","Exists":"∃","expectation":"ℰ","exponentiale":"ⅇ","ExponentialE":"ⅇ","fallingdotseq":"≒","Fcy":"Ф","fcy":"ф","female":"♀","ffilig":"ﬃ","fflig":"ﬀ","ffllig":"ﬄ","Ffr":"𝔉","ffr":"𝔣","filig":"ﬁ","FilledSmallSquare":"◼","FilledVerySmallSquare":"▪","fjlig":"fj","flat":"♭","fllig":"ﬂ","fltns":"▱","fnof":"ƒ","Fopf":"𝔽","fopf":"𝕗","forall":"∀","ForAll":"∀","fork":"⋔","forkv":"⫙","Fouriertrf":"ℱ","fpartint":"⨍","frac12":"½","frac13":"⅓","frac14":"¼","frac15":"⅕","frac16":"⅙","frac18":"⅛","frac23":"⅔","frac25":"⅖","frac34":"¾","frac35":"⅗","frac38":"⅜","frac45":"⅘","frac56":"⅚","frac58":"⅝","frac78":"⅞","frasl":"⁄","frown":"⌢","fscr":"𝒻","Fscr":"ℱ","gacute":"ǵ","Gamma":"Γ","gamma":"γ","Gammad":"Ϝ","gammad":"ϝ","gap":"⪆","Gbreve":"Ğ","gbreve":"ğ","Gcedil":"Ģ","Gcirc":"Ĝ","gcirc":"ĝ","Gcy":"Г","gcy":"г","Gdot":"Ġ","gdot":"ġ","ge":"≥","gE":"≧","gEl":"⪌","gel":"⋛","geq":"≥","geqq":"≧","geqslant":"⩾","gescc":"⪩","ges":"⩾","gesdot":"⪀","gesdoto":"⪂","gesdotol":"⪄","gesl":"⋛︀","gesles":"⪔","Gfr":"𝔊","gfr":"𝔤","gg":"≫","Gg":"⋙","ggg":"⋙","gimel":"ℷ","GJcy":"Ѓ","gjcy":"ѓ","gla":"⪥","gl":"≷","glE":"⪒","glj":"⪤","gnap":"⪊","gnapprox":"⪊","gne":"⪈","gnE":"≩","gneq":"⪈","gneqq":"≩","gnsim":"⋧","Gopf":"𝔾","gopf":"𝕘","grave":"`","GreaterEqual":"≥","GreaterEqualLess":"⋛","GreaterFullEqual":"≧","GreaterGreater":"⪢","GreaterLess":"≷","GreaterSlantEqual":"⩾","GreaterTilde":"≳","Gscr":"𝒢","gscr":"ℊ","gsim":"≳","gsime":"⪎","gsiml":"⪐","gtcc":"⪧","gtcir":"⩺","gt":">","GT":">","Gt":"≫","gtdot":"⋗","gtlPar":"⦕","gtquest":"⩼","gtrapprox":"⪆","gtrarr":"⥸","gtrdot":"⋗","gtreqless":"⋛","gtreqqless":"⪌","gtrless":"≷","gtrsim":"≳","gvertneqq":"≩︀","gvnE":"≩︀","Hacek":"ˇ","hairsp":" ","half":"½","hamilt":"ℋ","HARDcy":"Ъ","hardcy":"ъ","harrcir":"⥈","harr":"↔","hArr":"⇔","harrw":"↭","Hat":"^","hbar":"ℏ","Hcirc":"Ĥ","hcirc":"ĥ","hearts":"♥","heartsuit":"♥","hellip":"…","hercon":"⊹","hfr":"𝔥","Hfr":"ℌ","HilbertSpace":"ℋ","hksearow":"⤥","hkswarow":"⤦","hoarr":"⇿","homtht":"∻","hookleftarrow":"↩","hookrightarrow":"↪","hopf":"𝕙","Hopf":"ℍ","horbar":"―","HorizontalLine":"─","hscr":"𝒽","Hscr":"ℋ","hslash":"ℏ","Hstrok":"Ħ","hstrok":"ħ","HumpDownHump":"≎","HumpEqual":"≏","hybull":"⁃","hyphen":"‐","Iacute":"Í","iacute":"í","ic":"⁣","Icirc":"Î","icirc":"î","Icy":"И","icy":"и","Idot":"İ","IEcy":"Е","iecy":"е","iexcl":"¡","iff":"⇔","ifr":"𝔦","Ifr":"ℑ","Igrave":"Ì","igrave":"ì","ii":"ⅈ","iiiint":"⨌","iiint":"∭","iinfin":"⧜","iiota":"℩","IJlig":"Ĳ","ijlig":"ĳ","Imacr":"Ī","imacr":"ī","image":"ℑ","ImaginaryI":"ⅈ","imagline":"ℐ","imagpart":"ℑ","imath":"ı","Im":"ℑ","imof":"⊷","imped":"Ƶ","Implies":"⇒","incare":"℅","in":"∈","infin":"∞","infintie":"⧝","inodot":"ı","intcal":"⊺","int":"∫","Int":"∬","integers":"ℤ","Integral":"∫","intercal":"⊺","Intersection":"⋂","intlarhk":"⨗","intprod":"⨼","InvisibleComma":"⁣","InvisibleTimes":"⁢","IOcy":"Ё","iocy":"ё","Iogon":"Į","iogon":"į","Iopf":"𝕀","iopf":"𝕚","Iota":"Ι","iota":"ι","iprod":"⨼","iquest":"¿","iscr":"𝒾","Iscr":"ℐ","isin":"∈","isindot":"⋵","isinE":"⋹","isins":"⋴","isinsv":"⋳","isinv":"∈","it":"⁢","Itilde":"Ĩ","itilde":"ĩ","Iukcy":"І","iukcy":"і","Iuml":"Ï","iuml":"ï","Jcirc":"Ĵ","jcirc":"ĵ","Jcy":"Й","jcy":"й","Jfr":"𝔍","jfr":"𝔧","jmath":"ȷ","Jopf":"𝕁","jopf":"𝕛","Jscr":"𝒥","jscr":"𝒿","Jsercy":"Ј","jsercy":"ј","Jukcy":"Є","jukcy":"є","Kappa":"Κ","kappa":"κ","kappav":"ϰ","Kcedil":"Ķ","kcedil":"ķ","Kcy":"К","kcy":"к","Kfr":"𝔎","kfr":"𝔨","kgreen":"ĸ","KHcy":"Х","khcy":"х","KJcy":"Ќ","kjcy":"ќ","Kopf":"𝕂","kopf":"𝕜","Kscr":"𝒦","kscr":"𝓀","lAarr":"⇚","Lacute":"Ĺ","lacute":"ĺ","laemptyv":"⦴","lagran":"ℒ","Lambda":"Λ","lambda":"λ","lang":"⟨","Lang":"⟪","langd":"⦑","langle":"⟨","lap":"⪅","Laplacetrf":"ℒ","laquo":"«","larrb":"⇤","larrbfs":"⤟","larr":"←","Larr":"↞","lArr":"⇐","larrfs":"⤝","larrhk":"↩","larrlp":"↫","larrpl":"⤹","larrsim":"⥳","larrtl":"↢","latail":"⤙","lAtail":"⤛","lat":"⪫","late":"⪭","lates":"⪭︀","lbarr":"⤌","lBarr":"⤎","lbbrk":"❲","lbrace":"{","lbrack":"[","lbrke":"⦋","lbrksld":"⦏","lbrkslu":"⦍","Lcaron":"Ľ","lcaron":"ľ","Lcedil":"Ļ","lcedil":"ļ","lceil":"⌈","lcub":"{","Lcy":"Л","lcy":"л","ldca":"⤶","ldquo":"“","ldquor":"„","ldrdhar":"⥧","ldrushar":"⥋","ldsh":"↲","le":"≤","lE":"≦","LeftAngleBracket":"⟨","LeftArrowBar":"⇤","leftarrow":"←","LeftArrow":"←","Leftarrow":"⇐","LeftArrowRightArrow":"⇆","leftarrowtail":"↢","LeftCeiling":"⌈","LeftDoubleBracket":"⟦","LeftDownTeeVector":"⥡","LeftDownVectorBar":"⥙","LeftDownVector":"⇃","LeftFloor":"⌊","leftharpoondown":"↽","leftharpoonup":"↼","leftleftarrows":"⇇","leftrightarrow":"↔","LeftRightArrow":"↔","Leftrightarrow":"⇔","leftrightarrows":"⇆","leftrightharpoons":"⇋","leftrightsquigarrow":"↭","LeftRightVector":"⥎","LeftTeeArrow":"↤","LeftTee":"⊣","LeftTeeVector":"⥚","leftthreetimes":"⋋","LeftTriangleBar":"⧏","LeftTriangle":"⊲","LeftTriangleEqual":"⊴","LeftUpDownVector":"⥑","LeftUpTeeVector":"⥠","LeftUpVectorBar":"⥘","LeftUpVector":"↿","LeftVectorBar":"⥒","LeftVector":"↼","lEg":"⪋","leg":"⋚","leq":"≤","leqq":"≦","leqslant":"⩽","lescc":"⪨","les":"⩽","lesdot":"⩿","lesdoto":"⪁","lesdotor":"⪃","lesg":"⋚︀","lesges":"⪓","lessapprox":"⪅","lessdot":"⋖","lesseqgtr":"⋚","lesseqqgtr":"⪋","LessEqualGreater":"⋚","LessFullEqual":"≦","LessGreater":"≶","lessgtr":"≶","LessLess":"⪡","lesssim":"≲","LessSlantEqual":"⩽","LessTilde":"≲","lfisht":"⥼","lfloor":"⌊","Lfr":"𝔏","lfr":"𝔩","lg":"≶","lgE":"⪑","lHar":"⥢","lhard":"↽","lharu":"↼","lharul":"⥪","lhblk":"▄","LJcy":"Љ","ljcy":"љ","llarr":"⇇","ll":"≪","Ll":"⋘","llcorner":"⌞","Lleftarrow":"⇚","llhard":"⥫","lltri":"◺","Lmidot":"Ŀ","lmidot":"ŀ","lmoustache":"⎰","lmoust":"⎰","lnap":"⪉","lnapprox":"⪉","lne":"⪇","lnE":"≨","lneq":"⪇","lneqq":"≨","lnsim":"⋦","loang":"⟬","loarr":"⇽","lobrk":"⟦","longleftarrow":"⟵","LongLeftArrow":"⟵","Longleftarrow":"⟸","longleftrightarrow":"⟷","LongLeftRightArrow":"⟷","Longleftrightarrow":"⟺","longmapsto":"⟼","longrightarrow":"⟶","LongRightArrow":"⟶","Longrightarrow":"⟹","looparrowleft":"↫","looparrowright":"↬","lopar":"⦅","Lopf":"𝕃","lopf":"𝕝","loplus":"⨭","lotimes":"⨴","lowast":"∗","lowbar":"_","LowerLeftArrow":"↙","LowerRightArrow":"↘","loz":"◊","lozenge":"◊","lozf":"⧫","lpar":"(","lparlt":"⦓","lrarr":"⇆","lrcorner":"⌟","lrhar":"⇋","lrhard":"⥭","lrm":"‎","lrtri":"⊿","lsaquo":"‹","lscr":"𝓁","Lscr":"ℒ","lsh":"↰","Lsh":"↰","lsim":"≲","lsime":"⪍","lsimg":"⪏","lsqb":"[","lsquo":"‘","lsquor":"‚","Lstrok":"Ł","lstrok":"ł","ltcc":"⪦","ltcir":"⩹","lt":"<","LT":"<","Lt":"≪","ltdot":"⋖","lthree":"⋋","ltimes":"⋉","ltlarr":"⥶","ltquest":"⩻","ltri":"◃","ltrie":"⊴","ltrif":"◂","ltrPar":"⦖","lurdshar":"⥊","luruhar":"⥦","lvertneqq":"≨︀","lvnE":"≨︀","macr":"¯","male":"♂","malt":"✠","maltese":"✠","Map":"⤅","map":"↦","mapsto":"↦","mapstodown":"↧","mapstoleft":"↤","mapstoup":"↥","marker":"▮","mcomma":"⨩","Mcy":"М","mcy":"м","mdash":"—","mDDot":"∺","measuredangle":"∡","MediumSpace":" ","Mellintrf":"ℳ","Mfr":"𝔐","mfr":"𝔪","mho":"℧","micro":"µ","midast":"*","midcir":"⫰","mid":"∣","middot":"·","minusb":"⊟","minus":"−","minusd":"∸","minusdu":"⨪","MinusPlus":"∓","mlcp":"⫛","mldr":"…","mnplus":"∓","models":"⊧","Mopf":"𝕄","mopf":"𝕞","mp":"∓","mscr":"𝓂","Mscr":"ℳ","mstpos":"∾","Mu":"Μ","mu":"μ","multimap":"⊸","mumap":"⊸","nabla":"∇","Nacute":"Ń","nacute":"ń","nang":"∠⃒","nap":"≉","napE":"⩰̸","napid":"≋̸","napos":"ŉ","napprox":"≉","natural":"♮","naturals":"ℕ","natur":"♮","nbsp":" ","nbump":"≎̸","nbumpe":"≏̸","ncap":"⩃","Ncaron":"Ň","ncaron":"ň","Ncedil":"Ņ","ncedil":"ņ","ncong":"≇","ncongdot":"⩭̸","ncup":"⩂","Ncy":"Н","ncy":"н","ndash":"–","nearhk":"⤤","nearr":"↗","neArr":"⇗","nearrow":"↗","ne":"≠","nedot":"≐̸","NegativeMediumSpace":"​","NegativeThickSpace":"​","NegativeThinSpace":"​","NegativeVeryThinSpace":"​","nequiv":"≢","nesear":"⤨","nesim":"≂̸","NestedGreaterGreater":"≫","NestedLessLess":"≪","NewLine":"\\n","nexist":"∄","nexists":"∄","Nfr":"𝔑","nfr":"𝔫","ngE":"≧̸","nge":"≱","ngeq":"≱","ngeqq":"≧̸","ngeqslant":"⩾̸","nges":"⩾̸","nGg":"⋙̸","ngsim":"≵","nGt":"≫⃒","ngt":"≯","ngtr":"≯","nGtv":"≫̸","nharr":"↮","nhArr":"⇎","nhpar":"⫲","ni":"∋","nis":"⋼","nisd":"⋺","niv":"∋","NJcy":"Њ","njcy":"њ","nlarr":"↚","nlArr":"⇍","nldr":"‥","nlE":"≦̸","nle":"≰","nleftarrow":"↚","nLeftarrow":"⇍","nleftrightarrow":"↮","nLeftrightarrow":"⇎","nleq":"≰","nleqq":"≦̸","nleqslant":"⩽̸","nles":"⩽̸","nless":"≮","nLl":"⋘̸","nlsim":"≴","nLt":"≪⃒","nlt":"≮","nltri":"⋪","nltrie":"⋬","nLtv":"≪̸","nmid":"∤","NoBreak":"⁠","NonBreakingSpace":" ","nopf":"𝕟","Nopf":"ℕ","Not":"⫬","not":"¬","NotCongruent":"≢","NotCupCap":"≭","NotDoubleVerticalBar":"∦","NotElement":"∉","NotEqual":"≠","NotEqualTilde":"≂̸","NotExists":"∄","NotGreater":"≯","NotGreaterEqual":"≱","NotGreaterFullEqual":"≧̸","NotGreaterGreater":"≫̸","NotGreaterLess":"≹","NotGreaterSlantEqual":"⩾̸","NotGreaterTilde":"≵","NotHumpDownHump":"≎̸","NotHumpEqual":"≏̸","notin":"∉","notindot":"⋵̸","notinE":"⋹̸","notinva":"∉","notinvb":"⋷","notinvc":"⋶","NotLeftTriangleBar":"⧏̸","NotLeftTriangle":"⋪","NotLeftTriangleEqual":"⋬","NotLess":"≮","NotLessEqual":"≰","NotLessGreater":"≸","NotLessLess":"≪̸","NotLessSlantEqual":"⩽̸","NotLessTilde":"≴","NotNestedGreaterGreater":"⪢̸","NotNestedLessLess":"⪡̸","notni":"∌","notniva":"∌","notnivb":"⋾","notnivc":"⋽","NotPrecedes":"⊀","NotPrecedesEqual":"⪯̸","NotPrecedesSlantEqual":"⋠","NotReverseElement":"∌","NotRightTriangleBar":"⧐̸","NotRightTriangle":"⋫","NotRightTriangleEqual":"⋭","NotSquareSubset":"⊏̸","NotSquareSubsetEqual":"⋢","NotSquareSuperset":"⊐̸","NotSquareSupersetEqual":"⋣","NotSubset":"⊂⃒","NotSubsetEqual":"⊈","NotSucceeds":"⊁","NotSucceedsEqual":"⪰̸","NotSucceedsSlantEqual":"⋡","NotSucceedsTilde":"≿̸","NotSuperset":"⊃⃒","NotSupersetEqual":"⊉","NotTilde":"≁","NotTildeEqual":"≄","NotTildeFullEqual":"≇","NotTildeTilde":"≉","NotVerticalBar":"∤","nparallel":"∦","npar":"∦","nparsl":"⫽⃥","npart":"∂̸","npolint":"⨔","npr":"⊀","nprcue":"⋠","nprec":"⊀","npreceq":"⪯̸","npre":"⪯̸","nrarrc":"⤳̸","nrarr":"↛","nrArr":"⇏","nrarrw":"↝̸","nrightarrow":"↛","nRightarrow":"⇏","nrtri":"⋫","nrtrie":"⋭","nsc":"⊁","nsccue":"⋡","nsce":"⪰̸","Nscr":"𝒩","nscr":"𝓃","nshortmid":"∤","nshortparallel":"∦","nsim":"≁","nsime":"≄","nsimeq":"≄","nsmid":"∤","nspar":"∦","nsqsube":"⋢","nsqsupe":"⋣","nsub":"⊄","nsubE":"⫅̸","nsube":"⊈","nsubset":"⊂⃒","nsubseteq":"⊈","nsubseteqq":"⫅̸","nsucc":"⊁","nsucceq":"⪰̸","nsup":"⊅","nsupE":"⫆̸","nsupe":"⊉","nsupset":"⊃⃒","nsupseteq":"⊉","nsupseteqq":"⫆̸","ntgl":"≹","Ntilde":"Ñ","ntilde":"ñ","ntlg":"≸","ntriangleleft":"⋪","ntrianglelefteq":"⋬","ntriangleright":"⋫","ntrianglerighteq":"⋭","Nu":"Ν","nu":"ν","num":"#","numero":"№","numsp":" ","nvap":"≍⃒","nvdash":"⊬","nvDash":"⊭","nVdash":"⊮","nVDash":"⊯","nvge":"≥⃒","nvgt":">⃒","nvHarr":"⤄","nvinfin":"⧞","nvlArr":"⤂","nvle":"≤⃒","nvlt":"<⃒","nvltrie":"⊴⃒","nvrArr":"⤃","nvrtrie":"⊵⃒","nvsim":"∼⃒","nwarhk":"⤣","nwarr":"↖","nwArr":"⇖","nwarrow":"↖","nwnear":"⤧","Oacute":"Ó","oacute":"ó","oast":"⊛","Ocirc":"Ô","ocirc":"ô","ocir":"⊚","Ocy":"О","ocy":"о","odash":"⊝","Odblac":"Ő","odblac":"ő","odiv":"⨸","odot":"⊙","odsold":"⦼","OElig":"Œ","oelig":"œ","ofcir":"⦿","Ofr":"𝔒","ofr":"𝔬","ogon":"˛","Ograve":"Ò","ograve":"ò","ogt":"⧁","ohbar":"⦵","ohm":"Ω","oint":"∮","olarr":"↺","olcir":"⦾","olcross":"⦻","oline":"‾","olt":"⧀","Omacr":"Ō","omacr":"ō","Omega":"Ω","omega":"ω","Omicron":"Ο","omicron":"ο","omid":"⦶","ominus":"⊖","Oopf":"𝕆","oopf":"𝕠","opar":"⦷","OpenCurlyDoubleQuote":"“","OpenCurlyQuote":"‘","operp":"⦹","oplus":"⊕","orarr":"↻","Or":"⩔","or":"∨","ord":"⩝","order":"ℴ","orderof":"ℴ","ordf":"ª","ordm":"º","origof":"⊶","oror":"⩖","orslope":"⩗","orv":"⩛","oS":"Ⓢ","Oscr":"𝒪","oscr":"ℴ","Oslash":"Ø","oslash":"ø","osol":"⊘","Otilde":"Õ","otilde":"õ","otimesas":"⨶","Otimes":"⨷","otimes":"⊗","Ouml":"Ö","ouml":"ö","ovbar":"⌽","OverBar":"‾","OverBrace":"⏞","OverBracket":"⎴","OverParenthesis":"⏜","para":"¶","parallel":"∥","par":"∥","parsim":"⫳","parsl":"⫽","part":"∂","PartialD":"∂","Pcy":"П","pcy":"п","percnt":"%","period":".","permil":"‰","perp":"⊥","pertenk":"‱","Pfr":"𝔓","pfr":"𝔭","Phi":"Φ","phi":"φ","phiv":"ϕ","phmmat":"ℳ","phone":"☎","Pi":"Π","pi":"π","pitchfork":"⋔","piv":"ϖ","planck":"ℏ","planckh":"ℎ","plankv":"ℏ","plusacir":"⨣","plusb":"⊞","pluscir":"⨢","plus":"+","plusdo":"∔","plusdu":"⨥","pluse":"⩲","PlusMinus":"±","plusmn":"±","plussim":"⨦","plustwo":"⨧","pm":"±","Poincareplane":"ℌ","pointint":"⨕","popf":"𝕡","Popf":"ℙ","pound":"£","prap":"⪷","Pr":"⪻","pr":"≺","prcue":"≼","precapprox":"⪷","prec":"≺","preccurlyeq":"≼","Precedes":"≺","PrecedesEqual":"⪯","PrecedesSlantEqual":"≼","PrecedesTilde":"≾","preceq":"⪯","precnapprox":"⪹","precneqq":"⪵","precnsim":"⋨","pre":"⪯","prE":"⪳","precsim":"≾","prime":"′","Prime":"″","primes":"ℙ","prnap":"⪹","prnE":"⪵","prnsim":"⋨","prod":"∏","Product":"∏","profalar":"⌮","profline":"⌒","profsurf":"⌓","prop":"∝","Proportional":"∝","Proportion":"∷","propto":"∝","prsim":"≾","prurel":"⊰","Pscr":"𝒫","pscr":"𝓅","Psi":"Ψ","psi":"ψ","puncsp":" ","Qfr":"𝔔","qfr":"𝔮","qint":"⨌","qopf":"𝕢","Qopf":"ℚ","qprime":"⁗","Qscr":"𝒬","qscr":"𝓆","quaternions":"ℍ","quatint":"⨖","quest":"?","questeq":"≟","quot":"\\"","QUOT":"\\"","rAarr":"⇛","race":"∽̱","Racute":"Ŕ","racute":"ŕ","radic":"√","raemptyv":"⦳","rang":"⟩","Rang":"⟫","rangd":"⦒","range":"⦥","rangle":"⟩","raquo":"»","rarrap":"⥵","rarrb":"⇥","rarrbfs":"⤠","rarrc":"⤳","rarr":"→","Rarr":"↠","rArr":"⇒","rarrfs":"⤞","rarrhk":"↪","rarrlp":"↬","rarrpl":"⥅","rarrsim":"⥴","Rarrtl":"⤖","rarrtl":"↣","rarrw":"↝","ratail":"⤚","rAtail":"⤜","ratio":"∶","rationals":"ℚ","rbarr":"⤍","rBarr":"⤏","RBarr":"⤐","rbbrk":"❳","rbrace":"}","rbrack":"]","rbrke":"⦌","rbrksld":"⦎","rbrkslu":"⦐","Rcaron":"Ř","rcaron":"ř","Rcedil":"Ŗ","rcedil":"ŗ","rceil":"⌉","rcub":"}","Rcy":"Р","rcy":"р","rdca":"⤷","rdldhar":"⥩","rdquo":"”","rdquor":"”","rdsh":"↳","real":"ℜ","realine":"ℛ","realpart":"ℜ","reals":"ℝ","Re":"ℜ","rect":"▭","reg":"®","REG":"®","ReverseElement":"∋","ReverseEquilibrium":"⇋","ReverseUpEquilibrium":"⥯","rfisht":"⥽","rfloor":"⌋","rfr":"𝔯","Rfr":"ℜ","rHar":"⥤","rhard":"⇁","rharu":"⇀","rharul":"⥬","Rho":"Ρ","rho":"ρ","rhov":"ϱ","RightAngleBracket":"⟩","RightArrowBar":"⇥","rightarrow":"→","RightArrow":"→","Rightarrow":"⇒","RightArrowLeftArrow":"⇄","rightarrowtail":"↣","RightCeiling":"⌉","RightDoubleBracket":"⟧","RightDownTeeVector":"⥝","RightDownVectorBar":"⥕","RightDownVector":"⇂","RightFloor":"⌋","rightharpoondown":"⇁","rightharpoonup":"⇀","rightleftarrows":"⇄","rightleftharpoons":"⇌","rightrightarrows":"⇉","rightsquigarrow":"↝","RightTeeArrow":"↦","RightTee":"⊢","RightTeeVector":"⥛","rightthreetimes":"⋌","RightTriangleBar":"⧐","RightTriangle":"⊳","RightTriangleEqual":"⊵","RightUpDownVector":"⥏","RightUpTeeVector":"⥜","RightUpVectorBar":"⥔","RightUpVector":"↾","RightVectorBar":"⥓","RightVector":"⇀","ring":"˚","risingdotseq":"≓","rlarr":"⇄","rlhar":"⇌","rlm":"‏","rmoustache":"⎱","rmoust":"⎱","rnmid":"⫮","roang":"⟭","roarr":"⇾","robrk":"⟧","ropar":"⦆","ropf":"𝕣","Ropf":"ℝ","roplus":"⨮","rotimes":"⨵","RoundImplies":"⥰","rpar":")","rpargt":"⦔","rppolint":"⨒","rrarr":"⇉","Rrightarrow":"⇛","rsaquo":"›","rscr":"𝓇","Rscr":"ℛ","rsh":"↱","Rsh":"↱","rsqb":"]","rsquo":"’","rsquor":"’","rthree":"⋌","rtimes":"⋊","rtri":"▹","rtrie":"⊵","rtrif":"▸","rtriltri":"⧎","RuleDelayed":"⧴","ruluhar":"⥨","rx":"℞","Sacute":"Ś","sacute":"ś","sbquo":"‚","scap":"⪸","Scaron":"Š","scaron":"š","Sc":"⪼","sc":"≻","sccue":"≽","sce":"⪰","scE":"⪴","Scedil":"Ş","scedil":"ş","Scirc":"Ŝ","scirc":"ŝ","scnap":"⪺","scnE":"⪶","scnsim":"⋩","scpolint":"⨓","scsim":"≿","Scy":"С","scy":"с","sdotb":"⊡","sdot":"⋅","sdote":"⩦","searhk":"⤥","searr":"↘","seArr":"⇘","searrow":"↘","sect":"§","semi":";","seswar":"⤩","setminus":"∖","setmn":"∖","sext":"✶","Sfr":"𝔖","sfr":"𝔰","sfrown":"⌢","sharp":"♯","SHCHcy":"Щ","shchcy":"щ","SHcy":"Ш","shcy":"ш","ShortDownArrow":"↓","ShortLeftArrow":"←","shortmid":"∣","shortparallel":"∥","ShortRightArrow":"→","ShortUpArrow":"↑","shy":"­","Sigma":"Σ","sigma":"σ","sigmaf":"ς","sigmav":"ς","sim":"∼","simdot":"⩪","sime":"≃","simeq":"≃","simg":"⪞","simgE":"⪠","siml":"⪝","simlE":"⪟","simne":"≆","simplus":"⨤","simrarr":"⥲","slarr":"←","SmallCircle":"∘","smallsetminus":"∖","smashp":"⨳","smeparsl":"⧤","smid":"∣","smile":"⌣","smt":"⪪","smte":"⪬","smtes":"⪬︀","SOFTcy":"Ь","softcy":"ь","solbar":"⌿","solb":"⧄","sol":"/","Sopf":"𝕊","sopf":"𝕤","spades":"♠","spadesuit":"♠","spar":"∥","sqcap":"⊓","sqcaps":"⊓︀","sqcup":"⊔","sqcups":"⊔︀","Sqrt":"√","sqsub":"⊏","sqsube":"⊑","sqsubset":"⊏","sqsubseteq":"⊑","sqsup":"⊐","sqsupe":"⊒","sqsupset":"⊐","sqsupseteq":"⊒","square":"□","Square":"□","SquareIntersection":"⊓","SquareSubset":"⊏","SquareSubsetEqual":"⊑","SquareSuperset":"⊐","SquareSupersetEqual":"⊒","SquareUnion":"⊔","squarf":"▪","squ":"□","squf":"▪","srarr":"→","Sscr":"𝒮","sscr":"𝓈","ssetmn":"∖","ssmile":"⌣","sstarf":"⋆","Star":"⋆","star":"☆","starf":"★","straightepsilon":"ϵ","straightphi":"ϕ","strns":"¯","sub":"⊂","Sub":"⋐","subdot":"⪽","subE":"⫅","sube":"⊆","subedot":"⫃","submult":"⫁","subnE":"⫋","subne":"⊊","subplus":"⪿","subrarr":"⥹","subset":"⊂","Subset":"⋐","subseteq":"⊆","subseteqq":"⫅","SubsetEqual":"⊆","subsetneq":"⊊","subsetneqq":"⫋","subsim":"⫇","subsub":"⫕","subsup":"⫓","succapprox":"⪸","succ":"≻","succcurlyeq":"≽","Succeeds":"≻","SucceedsEqual":"⪰","SucceedsSlantEqual":"≽","SucceedsTilde":"≿","succeq":"⪰","succnapprox":"⪺","succneqq":"⪶","succnsim":"⋩","succsim":"≿","SuchThat":"∋","sum":"∑","Sum":"∑","sung":"♪","sup1":"¹","sup2":"²","sup3":"³","sup":"⊃","Sup":"⋑","supdot":"⪾","supdsub":"⫘","supE":"⫆","supe":"⊇","supedot":"⫄","Superset":"⊃","SupersetEqual":"⊇","suphsol":"⟉","suphsub":"⫗","suplarr":"⥻","supmult":"⫂","supnE":"⫌","supne":"⊋","supplus":"⫀","supset":"⊃","Supset":"⋑","supseteq":"⊇","supseteqq":"⫆","supsetneq":"⊋","supsetneqq":"⫌","supsim":"⫈","supsub":"⫔","supsup":"⫖","swarhk":"⤦","swarr":"↙","swArr":"⇙","swarrow":"↙","swnwar":"⤪","szlig":"ß","Tab":"\\t","target":"⌖","Tau":"Τ","tau":"τ","tbrk":"⎴","Tcaron":"Ť","tcaron":"ť","Tcedil":"Ţ","tcedil":"ţ","Tcy":"Т","tcy":"т","tdot":"⃛","telrec":"⌕","Tfr":"𝔗","tfr":"𝔱","there4":"∴","therefore":"∴","Therefore":"∴","Theta":"Θ","theta":"θ","thetasym":"ϑ","thetav":"ϑ","thickapprox":"≈","thicksim":"∼","ThickSpace":"  ","ThinSpace":" ","thinsp":" ","thkap":"≈","thksim":"∼","THORN":"Þ","thorn":"þ","tilde":"˜","Tilde":"∼","TildeEqual":"≃","TildeFullEqual":"≅","TildeTilde":"≈","timesbar":"⨱","timesb":"⊠","times":"×","timesd":"⨰","tint":"∭","toea":"⤨","topbot":"⌶","topcir":"⫱","top":"⊤","Topf":"𝕋","topf":"𝕥","topfork":"⫚","tosa":"⤩","tprime":"‴","trade":"™","TRADE":"™","triangle":"▵","triangledown":"▿","triangleleft":"◃","trianglelefteq":"⊴","triangleq":"≜","triangleright":"▹","trianglerighteq":"⊵","tridot":"◬","trie":"≜","triminus":"⨺","TripleDot":"⃛","triplus":"⨹","trisb":"⧍","tritime":"⨻","trpezium":"⏢","Tscr":"𝒯","tscr":"𝓉","TScy":"Ц","tscy":"ц","TSHcy":"Ћ","tshcy":"ћ","Tstrok":"Ŧ","tstrok":"ŧ","twixt":"≬","twoheadleftarrow":"↞","twoheadrightarrow":"↠","Uacute":"Ú","uacute":"ú","uarr":"↑","Uarr":"↟","uArr":"⇑","Uarrocir":"⥉","Ubrcy":"Ў","ubrcy":"ў","Ubreve":"Ŭ","ubreve":"ŭ","Ucirc":"Û","ucirc":"û","Ucy":"У","ucy":"у","udarr":"⇅","Udblac":"Ű","udblac":"ű","udhar":"⥮","ufisht":"⥾","Ufr":"𝔘","ufr":"𝔲","Ugrave":"Ù","ugrave":"ù","uHar":"⥣","uharl":"↿","uharr":"↾","uhblk":"▀","ulcorn":"⌜","ulcorner":"⌜","ulcrop":"⌏","ultri":"◸","Umacr":"Ū","umacr":"ū","uml":"¨","UnderBar":"_","UnderBrace":"⏟","UnderBracket":"⎵","UnderParenthesis":"⏝","Union":"⋃","UnionPlus":"⊎","Uogon":"Ų","uogon":"ų","Uopf":"𝕌","uopf":"𝕦","UpArrowBar":"⤒","uparrow":"↑","UpArrow":"↑","Uparrow":"⇑","UpArrowDownArrow":"⇅","updownarrow":"↕","UpDownArrow":"↕","Updownarrow":"⇕","UpEquilibrium":"⥮","upharpoonleft":"↿","upharpoonright":"↾","uplus":"⊎","UpperLeftArrow":"↖","UpperRightArrow":"↗","upsi":"υ","Upsi":"ϒ","upsih":"ϒ","Upsilon":"Υ","upsilon":"υ","UpTeeArrow":"↥","UpTee":"⊥","upuparrows":"⇈","urcorn":"⌝","urcorner":"⌝","urcrop":"⌎","Uring":"Ů","uring":"ů","urtri":"◹","Uscr":"𝒰","uscr":"𝓊","utdot":"⋰","Utilde":"Ũ","utilde":"ũ","utri":"▵","utrif":"▴","uuarr":"⇈","Uuml":"Ü","uuml":"ü","uwangle":"⦧","vangrt":"⦜","varepsilon":"ϵ","varkappa":"ϰ","varnothing":"∅","varphi":"ϕ","varpi":"ϖ","varpropto":"∝","varr":"↕","vArr":"⇕","varrho":"ϱ","varsigma":"ς","varsubsetneq":"⊊︀","varsubsetneqq":"⫋︀","varsupsetneq":"⊋︀","varsupsetneqq":"⫌︀","vartheta":"ϑ","vartriangleleft":"⊲","vartriangleright":"⊳","vBar":"⫨","Vbar":"⫫","vBarv":"⫩","Vcy":"В","vcy":"в","vdash":"⊢","vDash":"⊨","Vdash":"⊩","VDash":"⊫","Vdashl":"⫦","veebar":"⊻","vee":"∨","Vee":"⋁","veeeq":"≚","vellip":"⋮","verbar":"|","Verbar":"‖","vert":"|","Vert":"‖","VerticalBar":"∣","VerticalLine":"|","VerticalSeparator":"❘","VerticalTilde":"≀","VeryThinSpace":" ","Vfr":"𝔙","vfr":"𝔳","vltri":"⊲","vnsub":"⊂⃒","vnsup":"⊃⃒","Vopf":"𝕍","vopf":"𝕧","vprop":"∝","vrtri":"⊳","Vscr":"𝒱","vscr":"𝓋","vsubnE":"⫋︀","vsubne":"⊊︀","vsupnE":"⫌︀","vsupne":"⊋︀","Vvdash":"⊪","vzigzag":"⦚","Wcirc":"Ŵ","wcirc":"ŵ","wedbar":"⩟","wedge":"∧","Wedge":"⋀","wedgeq":"≙","weierp":"℘","Wfr":"𝔚","wfr":"𝔴","Wopf":"𝕎","wopf":"𝕨","wp":"℘","wr":"≀","wreath":"≀","Wscr":"𝒲","wscr":"𝓌","xcap":"⋂","xcirc":"◯","xcup":"⋃","xdtri":"▽","Xfr":"𝔛","xfr":"𝔵","xharr":"⟷","xhArr":"⟺","Xi":"Ξ","xi":"ξ","xlarr":"⟵","xlArr":"⟸","xmap":"⟼","xnis":"⋻","xodot":"⨀","Xopf":"𝕏","xopf":"𝕩","xoplus":"⨁","xotime":"⨂","xrarr":"⟶","xrArr":"⟹","Xscr":"𝒳","xscr":"𝓍","xsqcup":"⨆","xuplus":"⨄","xutri":"△","xvee":"⋁","xwedge":"⋀","Yacute":"Ý","yacute":"ý","YAcy":"Я","yacy":"я","Ycirc":"Ŷ","ycirc":"ŷ","Ycy":"Ы","ycy":"ы","yen":"¥","Yfr":"𝔜","yfr":"𝔶","YIcy":"Ї","yicy":"ї","Yopf":"𝕐","yopf":"𝕪","Yscr":"𝒴","yscr":"𝓎","YUcy":"Ю","yucy":"ю","yuml":"ÿ","Yuml":"Ÿ","Zacute":"Ź","zacute":"ź","Zcaron":"Ž","zcaron":"ž","Zcy":"З","zcy":"з","Zdot":"Ż","zdot":"ż","zeetrf":"ℨ","ZeroWidthSpace":"​","Zeta":"Ζ","zeta":"ζ","zfr":"𝔷","Zfr":"ℨ","ZHcy":"Ж","zhcy":"ж","zigrarr":"⇝","zopf":"𝕫","Zopf":"ℤ","Zscr":"𝒵","zscr":"𝓏","zwj":"‍","zwnj":"‌"}')},function(t){t.exports=JSON.parse('{"amp":"&","apos":"\'","gt":">","lt":"<","quot":"\\""}')},function(t,e,n){"use strict";var r;e.__esModule=!0,e.default=void 0;var i=function(t){var e,n;function r(e){var n;return(n=t.call(this,e)||this).type="decl",n}return n=t,(e=r).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n,r}(((r=n(300))&&r.__esModule?r:{default:r}).default);e.default=i,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=a(n(601)),i=a(n(602)),o=a(n(301));function a(t){return t&&t.__esModule?t:{default:t}}var s=function(){function t(t){for(var e in void 0===t&&(t={}),this.raws={},t)this[e]=t[e]}var e=t.prototype;return e.error=function(t,e){if(void 0===e&&(e={}),this.source){var n=this.positionBy(e);return this.source.input.error(t,n.line,n.column,e)}return new r.default(t)},e.warn=function(t,e,n){var r={node:this};for(var i in n)r[i]=n[i];return t.warn(e,r)},e.remove=function(){return this.parent&&this.parent.removeChild(this),this.parent=void 0,this},e.toString=function(t){void 0===t&&(t=o.default),t.stringify&&(t=t.stringify);var e="";return t(this,function(t){e+=t}),e},e.clone=function(t){void 0===t&&(t={});var e=function t(e,n){var r=new e.constructor;for(var i in e)if(e.hasOwnProperty(i)){var o=e[i],a=typeof o;"parent"===i&&"object"===a?n&&(r[i]=n):"source"===i?r[i]=o:o instanceof Array?r[i]=o.map(function(e){return t(e,r)}):("object"===a&&null!==o&&(o=t(o)),r[i]=o)}return r}(this);for(var n in t)e[n]=t[n];return e},e.cloneBefore=function(t){void 0===t&&(t={});var e=this.clone(t);return this.parent.insertBefore(this,e),e},e.cloneAfter=function(t){void 0===t&&(t={});var e=this.clone(t);return this.parent.insertAfter(this,e),e},e.replaceWith=function(){if(this.parent){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];for(var r=0,i=e;r<i.length;r++){var o=i[r];this.parent.insertBefore(this,o)}this.remove()}return this},e.next=function(){if(this.parent){var t=this.parent.index(this);return this.parent.nodes[t+1]}},e.prev=function(){if(this.parent){var t=this.parent.index(this);return this.parent.nodes[t-1]}},e.before=function(t){return this.parent.insertBefore(this,t),this},e.after=function(t){return this.parent.insertAfter(this,t),this},e.toJSON=function(){var t={};for(var e in this)if(this.hasOwnProperty(e)&&"parent"!==e){var n=this[e];n instanceof Array?t[e]=n.map(function(t){return"object"==typeof t&&t.toJSON?t.toJSON():t}):"object"==typeof n&&n.toJSON?t[e]=n.toJSON():t[e]=n}return t},e.raw=function(t,e){return(new i.default).raw(this,t,e)},e.root=function(){for(var t=this;t.parent;)t=t.parent;return t},e.cleanRaws=function(t){delete this.raws.before,delete this.raws.after,t||delete this.raws.between},e.positionInside=function(t){for(var e=this.toString(),n=this.source.start.column,r=this.source.start.line,i=0;i<t;i++)"\n"===e[i]?(n=1,r+=1):n+=1;return{line:r,column:n}},e.positionBy=function(t){var e=this.source.start;if(t.index)e=this.positionInside(t.index);else if(t.word){var n=this.toString().indexOf(t.word);-1!==n&&(e=this.positionInside(n))}return e},t}();e.default=s,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r,i=(r=n(602))&&r.__esModule?r:{default:r};var o=function(t,e){new i.default(e).stringify(t)};e.default=o,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=o(n(1125)),i=o(n(1127));function o(t){return t&&t.__esModule?t:{default:t}}var a=function(t,e){var n=new i.default(t,e),o=new r.default(n);try{o.parse()}catch(t){throw t}return o.root};e.default=a,t.exports=e.default},function(t,e,n){"use strict";var r;e.__esModule=!0,e.default=void 0;var i=function(t){var e,n;function r(e){var n;return(n=t.call(this,e)||this).type="comment",n}return n=t,(e=r).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n,r}(((r=n(300))&&r.__esModule?r:{default:r}).default);e.default=i,t.exports=e.default},function(t,e,n){"use strict";var r;e.__esModule=!0,e.default=void 0;var i=function(t){var e,n;function r(e){var n;return(n=t.call(this,e)||this).type="atrule",n}n=t,(e=r).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n;var i=r.prototype;return i.append=function(){var e;this.nodes||(this.nodes=[]);for(var n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(e=t.prototype.append).call.apply(e,[this].concat(r))},i.prepend=function(){var e;this.nodes||(this.nodes=[]);for(var n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(e=t.prototype.prepend).call.apply(e,[this].concat(r))},r}(((r=n(305))&&r.__esModule?r:{default:r}).default);e.default=i,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=o(n(299)),i=o(n(303));function o(t){return t&&t.__esModule?t:{default:t}}function a(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}var s=function(t){var e,o;function s(){return t.apply(this,arguments)||this}o=t,(e=s).prototype=Object.create(o.prototype),e.prototype.constructor=e,e.__proto__=o;var A,l,c,u=s.prototype;return u.push=function(t){return t.parent=this,this.nodes.push(t),this},u.each=function(t){this.lastEach||(this.lastEach=0),this.indexes||(this.indexes={}),this.lastEach+=1;var e=this.lastEach;if(this.indexes[e]=0,this.nodes){for(var n,r;this.indexes[e]<this.nodes.length&&(n=this.indexes[e],!1!==(r=t(this.nodes[n],n)));)this.indexes[e]+=1;return delete this.indexes[e],r}},u.walk=function(t){return this.each(function(e,n){var r;try{r=t(e,n)}catch(t){if(t.postcssNode=e,t.stack&&e.source&&/\n\s{4}at /.test(t.stack)){var i=e.source;t.stack=t.stack.replace(/\n\s{4}at /,"$&"+i.input.from+":"+i.start.line+":"+i.start.column+"$&")}throw t}return!1!==r&&e.walk&&(r=e.walk(t)),r})},u.walkDecls=function(t,e){return e?t instanceof RegExp?this.walk(function(n,r){if("decl"===n.type&&t.test(n.prop))return e(n,r)}):this.walk(function(n,r){if("decl"===n.type&&n.prop===t)return e(n,r)}):(e=t,this.walk(function(t,n){if("decl"===t.type)return e(t,n)}))},u.walkRules=function(t,e){return e?t instanceof RegExp?this.walk(function(n,r){if("rule"===n.type&&t.test(n.selector))return e(n,r)}):this.walk(function(n,r){if("rule"===n.type&&n.selector===t)return e(n,r)}):(e=t,this.walk(function(t,n){if("rule"===t.type)return e(t,n)}))},u.walkAtRules=function(t,e){return e?t instanceof RegExp?this.walk(function(n,r){if("atrule"===n.type&&t.test(n.name))return e(n,r)}):this.walk(function(n,r){if("atrule"===n.type&&n.name===t)return e(n,r)}):(e=t,this.walk(function(t,n){if("atrule"===t.type)return e(t,n)}))},u.walkComments=function(t){return this.walk(function(e,n){if("comment"===e.type)return t(e,n)})},u.append=function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];for(var r=0,i=e;r<i.length;r++){var o=i[r],a=this.normalize(o,this.last),s=a,A=Array.isArray(s),l=0;for(s=A?s:s[Symbol.iterator]();;){var c;if(A){if(l>=s.length)break;c=s[l++]}else{if((l=s.next()).done)break;c=l.value}var u=c;this.nodes.push(u)}}return this},u.prepend=function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];var r=e=e.reverse(),i=Array.isArray(r),o=0;for(r=i?r:r[Symbol.iterator]();;){var a;if(i){if(o>=r.length)break;a=r[o++]}else{if((o=r.next()).done)break;a=o.value}var s=a,A=this.normalize(s,this.first,"prepend").reverse(),l=A,c=Array.isArray(l),u=0;for(l=c?l:l[Symbol.iterator]();;){var g;if(c){if(u>=l.length)break;g=l[u++]}else{if((u=l.next()).done)break;g=u.value}var h=g;this.nodes.unshift(h)}for(var d in this.indexes)this.indexes[d]=this.indexes[d]+A.length}return this},u.cleanRaws=function(e){if(t.prototype.cleanRaws.call(this,e),this.nodes){var n=this.nodes,r=Array.isArray(n),i=0;for(n=r?n:n[Symbol.iterator]();;){var o;if(r){if(i>=n.length)break;o=n[i++]}else{if((i=n.next()).done)break;o=i.value}o.cleanRaws(e)}}},u.insertBefore=function(t,e){var n,r=0===(t=this.index(t))&&"prepend",i=this.normalize(e,this.nodes[t],r).reverse(),o=i,a=Array.isArray(o),s=0;for(o=a?o:o[Symbol.iterator]();;){var A;if(a){if(s>=o.length)break;A=o[s++]}else{if((s=o.next()).done)break;A=s.value}var l=A;this.nodes.splice(t,0,l)}for(var c in this.indexes)t<=(n=this.indexes[c])&&(this.indexes[c]=n+i.length);return this},u.insertAfter=function(t,e){t=this.index(t);var n,r=this.normalize(e,this.nodes[t]).reverse(),i=r,o=Array.isArray(i),a=0;for(i=o?i:i[Symbol.iterator]();;){var s;if(o){if(a>=i.length)break;s=i[a++]}else{if((a=i.next()).done)break;s=a.value}var A=s;this.nodes.splice(t+1,0,A)}for(var l in this.indexes)t<(n=this.indexes[l])&&(this.indexes[l]=n+r.length);return this},u.removeChild=function(t){var e;for(var n in t=this.index(t),this.nodes[t].parent=void 0,this.nodes.splice(t,1),this.indexes)(e=this.indexes[n])>=t&&(this.indexes[n]=e-1);return this},u.removeAll=function(){var t=this.nodes,e=Array.isArray(t),n=0;for(t=e?t:t[Symbol.iterator]();;){var r;if(e){if(n>=t.length)break;r=t[n++]}else{if((n=t.next()).done)break;r=n.value}r.parent=void 0}return this.nodes=[],this},u.replaceValues=function(t,e,n){return n||(n=e,e={}),this.walkDecls(function(r){e.props&&-1===e.props.indexOf(r.prop)||e.fast&&-1===r.value.indexOf(e.fast)||(r.value=r.value.replace(t,n))}),this},u.every=function(t){return this.nodes.every(t)},u.some=function(t){return this.nodes.some(t)},u.index=function(t){return"number"==typeof t?t:this.nodes.indexOf(t)},u.normalize=function(t,e){var o=this;if("string"==typeof t)t=function t(e){return e.map(function(e){return e.nodes&&(e.nodes=t(e.nodes)),delete e.source,e})}(n(302)(t).nodes);else if(Array.isArray(t)){var a=t=t.slice(0),s=Array.isArray(a),A=0;for(a=s?a:a[Symbol.iterator]();;){var l;if(s){if(A>=a.length)break;l=a[A++]}else{if((A=a.next()).done)break;l=A.value}var c=l;c.parent&&c.parent.removeChild(c,"ignore")}}else if("root"===t.type){var u=t=t.nodes.slice(0),g=Array.isArray(u),h=0;for(u=g?u:u[Symbol.iterator]();;){var d;if(g){if(h>=u.length)break;d=u[h++]}else{if((h=u.next()).done)break;d=h.value}var f=d;f.parent&&f.parent.removeChild(f,"ignore")}}else if(t.type)t=[t];else if(t.prop){if(void 0===t.value)throw new Error("Value field is missed in node creation");"string"!=typeof t.value&&(t.value=String(t.value)),t=[new r.default(t)]}else if(t.selector){t=[new(n(306))(t)]}else if(t.name){t=[new(n(304))(t)]}else{if(!t.text)throw new Error("Unknown node type in node creation");t=[new i.default(t)]}return t.map(function(t){return t.parent&&t.parent.removeChild(t),void 0===t.raws.before&&e&&void 0!==e.raws.before&&(t.raws.before=e.raws.before.replace(/[^\s]/g,"")),t.parent=o,t})},A=s,(l=[{key:"first",get:function(){if(this.nodes)return this.nodes[0]}},{key:"last",get:function(){if(this.nodes)return this.nodes[this.nodes.length-1]}}])&&a(A.prototype,l),c&&a(A,c),s}(o(n(300)).default);e.default=s,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=o(n(305)),i=o(n(609));function o(t){return t&&t.__esModule?t:{default:t}}function a(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}var s=function(t){var e,n,r,o,s;function A(e){var n;return(n=t.call(this,e)||this).type="rule",n.nodes||(n.nodes=[]),n}return n=t,(e=A).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n,r=A,(o=[{key:"selectors",get:function(){return i.default.comma(this.selector)},set:function(t){var e=this.selector?this.selector.match(/,\s*/):null,n=e?e[0]:","+this.raw("between","beforeOpen");this.selector=t.join(n)}}])&&a(r.prototype,o),s&&a(r,s),A}(r.default);e.default=s,t.exports=e.default},function(t,e,n){"use strict";n.d(e,"a",function(){return r}),n.d(e,"b",function(){return i});var r={add:["M10.99 6.99h-2v-2c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1zm-3-7c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.68 6-6 6z"],"add-column-left":["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-5 14H2V2h8v12zm4 0h-3V2h3v12zM4 9h1v1c0 .55.45 1 1 1s1-.45 1-1V9h1c.55 0 1-.45 1-1s-.45-1-1-1H7V6c0-.55-.45-1-1-1s-1 .45-1 1v1H4c-.55 0-1 .45-1 1s.45 1 1 1z"],"add-column-right":["M8 9h1v1c0 .55.45 1 1 1s1-.45 1-1V9h1c.55 0 1-.45 1-1s-.45-1-1-1h-1V6c0-.55-.45-1-1-1s-1 .45-1 1v1H8c-.55 0-1 .45-1 1s.45 1 1 1zm7-9H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM5 14H2V2h3v12zm9 0H6V2h8v12z"],"add-row-bottom":["M6 11h1v1c0 .55.45 1 1 1s1-.45 1-1v-1h1c.55 0 1-.45 1-1s-.45-1-1-1H9V8c0-.55-.45-1-1-1s-1 .45-1 1v1H6c-.55 0-1 .45-1 1s.45 1 1 1zm9-11H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H2V6h12v8zm0-9H2V2h12v3z"],"add-row-top":["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H2v-3h12v3zm0-4H2V2h12v8zM6 7h1v1c0 .55.45 1 1 1s1-.45 1-1V7h1c.55 0 1-.45 1-1s-.45-1-1-1H9V4c0-.55-.45-1-1-1s-1 .45-1 1v1H6c-.55 0-1 .45-1 1s.45 1 1 1z"],"add-to-artifact":["M14 4.01h-1v-1c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1v-1h1c.55 0 1-.45 1-1 0-.56-.45-1-1-1zm-13 2h6c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm8 6H1c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1 0-.56-.45-1-1-1zm0-4H1c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1 0-.56-.45-1-1-1z"],"add-to-folder":["M.01 7V5H16v7c0 .55-.45 1-1 1H9.005v-2.99C8.974 8.332 7.644 7 5.996 7H.01zM15 2H7.416L5.706.29a.996.996 0 0 0-.71-.29H1C.45 0 0 .45 0 1v3h15.99V3c.01-.55-.44-1-.99-1zM5.997 9H2c-.55 0-1 .45-1 1s.45 1 1 1h1.589L.3 14.29a1.003 1.003 0 0 0 1.42 1.42l3.287-3.29v1.59c0 .55.45 1 1 1 .549 0 .999-.45.999-1v-4A1.02 1.02 0 0 0 5.996 9z"],airplane:["M16 1.5A1.498 1.498 0 0 0 13.44.44L9.91 3.97 2 1 1 3l5.93 3.95L3.88 10H1l-1 1 3 2 2 3 1-1v-2.88l3.05-3.05L13 15l2-1-2.97-7.91 3.53-3.53c.27-.27.44-.65.44-1.06z"],"align-center":["M4 4c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1H4zM1 3h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm13 10H2c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zm1-6H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm-5 5c.55 0 1-.45 1-1s-.45-1-1-1H6c-.55 0-1 .45-1 1s.45 1 1 1h4z"],"align-justify":["M15 12.98H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm-14-10h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1 0 .56.45 1 1 1zm14 4H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm0-3H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm0 6H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"align-left":["M13 13H1c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zM1 3h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm0 3h8c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm14 1H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM1 12h4c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1z"],"align-right":["M15 12.98H3c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zm-14-10h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1 0 .56.45 1 1 1zm14 1H7c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1zm0 6h-4c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1zm0-3H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"alignment-bottom":["M10 12h3c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1h-3c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1zm5 2H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM3 12h3c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1z"],"alignment-horizontal-center":["M15 7h-1V6c0-.55-.45-1-1-1h-3c-.55 0-1 .45-1 1v1H7V3c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v4H1c-.55 0-1 .45-1 1s.45 1 1 1h1v4c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V9h2v1c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V9h1c.55 0 1-.45 1-1s-.45-1-1-1z"],"alignment-left":["M9 9H5c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1zM1 0C.45 0 0 .45 0 1v14c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm13 2H5c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],"alignment-right":["M11 9H7c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1zm4-9c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm-4 2H2c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],"alignment-top":["M15 0H1C.45 0 0 .45 0 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM6 4H3c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm7 0h-3c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1z"],"alignment-vertical-center":["M13 2H9V1c0-.55-.45-1-1-1S7 .45 7 1v1H3c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h4v2H6c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1v-1h1c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1H9V7h4c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],annotation:["M15.52 2.77c.3-.29.48-.7.48-1.15C16 .73 15.27 0 14.38 0c-.45 0-.85.18-1.15.48l-1.34 1.34 2.3 2.3 1.33-1.35zM7.4 10.9l6.21-6.21-2.3-2.3L5.1 8.6l2.3 2.3zM14 14H2V2h6.34l2-2H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V5.66l-2 2V14zM3 13l3.58-1.29-2.29-2.27L3 13z"],application:["M3.5 7h7c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-7c-.28 0-.5.22-.5.5s.22.5.5.5zM15 1H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zm-1 12H2V5h12v8zM3.5 9h4c.28 0 .5-.22.5-.5S7.78 8 7.5 8h-4c-.28 0-.5.22-.5.5s.22.5.5.5zm0 2h5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-5c-.28 0-.5.22-.5.5s.22.5.5.5z"],applications:["M3.5 11h2c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-2c-.28 0-.5.22-.5.5s.22.5.5.5zm0-2h5c.28 0 .5-.22.5-.5S8.78 8 8.5 8h-5c-.28 0-.5.22-.5.5s.22.5.5.5zM11 4H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm-1 10H2V7h8v7zm5-14H5c-.55 0-1 .45-1 1v2h2V2h8v7h-1v2h2c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM3.5 13h3c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-3c-.28 0-.5.22-.5.5s.22.5.5.5z"],"arrow-bottom-left":["M14 3a1.003 1.003 0 0 0-1.71-.71L4 10.59V6c0-.55-.45-1-1-1s-1 .45-1 1v7c0 .55.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1H5.41l8.29-8.29c.19-.18.3-.43.3-.71z"],"arrow-bottom-right":["M13 5c-.55 0-1 .45-1 1v4.59l-8.29-8.3a1.003 1.003 0 0 0-1.42 1.42l8.3 8.29H6c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1z"],"arrow-down":["M13 8c-.3 0-.5.1-.7.3L9 11.6V2c0-.5-.4-1-1-1s-1 .5-1 1v9.6L3.7 8.3C3.5 8.1 3.3 8 3 8c-.5 0-1 .5-1 1 0 .3.1.5.3.7l5 5c.2.2.4.3.7.3s.5-.1.7-.3l5-5c.2-.2.3-.4.3-.7 0-.6-.4-1-1-1z"],"arrow-left":["M13.99 6.99H4.41L7.7 3.7a1.003 1.003 0 0 0-1.42-1.42l-5 5a1.014 1.014 0 0 0 0 1.42l5 5a1.003 1.003 0 0 0 1.42-1.42L4.41 8.99H14c.55 0 1-.45 1-1s-.46-1-1.01-1z"],"arrow-right":["M14.7 7.29l-5-5a.965.965 0 0 0-.71-.3 1.003 1.003 0 0 0-.71 1.71l3.29 3.29H1.99c-.55 0-1 .45-1 1s.45 1 1 1h9.59l-3.29 3.29a1.003 1.003 0 0 0 1.42 1.42l5-5c.18-.18.29-.43.29-.71s-.12-.52-.3-.7z"],"arrow-top-left":["M13.71 12.29L5.41 4H10c.55 0 1-.45 1-1s-.45-1-1-1H3c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1s1-.45 1-1V5.41l8.29 8.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71z"],"arrow-top-right":["M13 2H6c-.55 0-1 .45-1 1s.45 1 1 1h4.59L2.3 12.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L12 5.41V10c0 .55.45 1 1 1s1-.45 1-1V3c0-.55-.45-1-1-1z"],"arrow-up":["M13.7 6.3l-5-5C8.5 1.1 8.3 1 8 1s-.5.1-.7.3l-5 5c-.2.2-.3.4-.3.7 0 .6.5 1 1 1 .3 0 .5-.1.7-.3L7 4.4V14c0 .6.4 1 1 1s1-.4 1-1V4.4l3.3 3.3c.2.2.4.3.7.3.6 0 1-.4 1-1 0-.3-.1-.5-.3-.7z"],"arrows-horizontal":["M15.7 7.3l-4-4c-.2-.2-.4-.3-.7-.3-.6 0-1 .5-1 1 0 .3.1.5.3.7L12.6 7H3.4l2.3-2.3c.2-.2.3-.4.3-.7 0-.5-.4-1-1-1-.3 0-.5.1-.7.3l-4 4c-.2.2-.3.4-.3.7s.1.5.3.7l4 4c.2.2.4.3.7.3.6 0 1-.4 1-1 0-.3-.1-.5-.3-.7L3.4 9h9.2l-2.3 2.3c-.2.2-.3.4-.3.7 0 .6.4 1 1 1 .3 0 .5-.1.7-.3l4-4c.2-.2.3-.4.3-.7s-.1-.5-.3-.7z"],"arrows-vertical":["M12 10c-.3 0-.5.1-.7.3L9 12.6V3.4l2.3 2.3c.2.2.4.3.7.3.6 0 1-.4 1-1 0-.3-.1-.5-.3-.7l-4-4C8.5.1 8.3 0 8 0s-.5.1-.7.3l-4 4c-.2.2-.3.4-.3.7 0 .6.5 1 1 1 .3 0 .5-.1.7-.3L7 3.4v9.2l-2.3-2.3c-.2-.2-.4-.3-.7-.3-.5 0-1 .4-1 1 0 .3.1.5.3.7l4 4c.2.2.4.3.7.3s.5-.1.7-.3l4-4c.2-.2.3-.4.3-.7 0-.6-.4-1-1-1z"],asterisk:["M14.54 11.18l.01-.02L9.8 8l4.75-3.17-.01-.02c.27-.17.46-.46.46-.81 0-.55-.45-1-1-1-.21 0-.39.08-.54.18l-.01-.02L9 6.13V1c0-.55-.45-1-1-1S7 .45 7 1v5.13L2.55 3.17l-.01.01A.969.969 0 0 0 2 3c-.55 0-1 .45-1 1 0 .35.19.64.46.82l-.01.01L6.2 8l-4.75 3.17.01.02c-.27.17-.46.46-.46.81 0 .55.45 1 1 1 .21 0 .39-.08.54-.18l.01.02L7 9.87V15c0 .55.45 1 1 1s1-.45 1-1V9.87l4.45 2.96.01-.02c.15.11.33.19.54.19.55 0 1-.45 1-1 0-.35-.19-.64-.46-.82z"],"automatic-updates":["M8 14c-3.31 0-6-2.69-6-6 0-1.77.78-3.36 2-4.46V5c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1H1C.45 0 0 .45 0 1s.45 1 1 1h1.74A7.95 7.95 0 0 0 0 8c0 4.42 3.58 8 8 8 .55 0 1-.45 1-1s-.45-1-1-1zM8 2a5.9 5.9 0 0 1 2.95.81l1.47-1.47A7.893 7.893 0 0 0 8 0c-.55 0-1 .45-1 1s.45 1 1 1zm2.71 6.71l5-5a1.003 1.003 0 0 0-1.42-1.42L10 6.59l-1.29-1.3a1.003 1.003 0 0 0-1.42 1.42l2 2c.18.18.43.29.71.29s.53-.11.71-.29zM16 8c0-.55-.06-1.08-.16-1.6l-1.87 1.87A5.966 5.966 0 0 1 12 12.45V11c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1h-1.74A7.95 7.95 0 0 0 16 8z"],badge:["M13.36 4.59c-.15-1.13.5-2.01 1.1-2.87L13.43.53c-1.72.88-4.12.65-5.63-.53-1.51 1.18-3.91 1.41-5.63.52l-1.03 1.2c.61.86 1.25 1.74 1.1 2.87-.3 2.29-2.45 4.17-1.32 6.68.45 1.14 1.44 1.9 2.72 2.2 1.56.36 3.52.72 4.16 2.53.64-1.81 2.6-2.16 4.16-2.54 1.28-.3 2.27-1.06 2.72-2.2 1.12-2.5-1.03-4.38-1.32-6.67z"],"ban-circle":["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm3 9H5c-.55 0-1-.45-1-1s.45-1 1-1h6c.55 0 1 .45 1 1s-.45 1-1 1z"],"bank-account":["M15.36 6.46l-.62-.14c-.31-1.12-.98-2.15-1.87-2.99l.4-1.77a.438.438 0 0 0-.49-.56c-.85.09-1.6.42-2.14.98-.84-.32-1.87-.51-2.85-.51-2.49 0-4.63 1.17-5.92 2.89-.18-.04-.36-.09-.53-.09-.76 0-1.34.61-1.34 1.4 0 .56.31 1.03.76 1.26-.05.33-.09.7-.09 1.07 0 1.68.71 3.17 1.83 4.34l-.27 1.59c-.09.56.35 1.07.89 1.07h.58c.45 0 .8-.33.89-.79l.04-.37c.94.42 2 .7 3.16.7 1.11 0 2.23-.23 3.16-.7l.05.37c.09.47.45.79.89.79h.58c.53 0 .98-.51.89-1.07l-.27-1.54c.62-.61 1.07-1.35 1.38-2.15l.8-.19c.4-.09.71-.47.71-.93V7.4c.09-.47-.22-.84-.62-.94zM12 8c-.6 0-1-.7-1-1.5S11.4 5 12 5s1 .7 1 1.5S12.6 8 12 8zM6.21 4.92c-.41.2-.91.04-1.12-.36-.21-.4-.04-.88.37-1.07 1.35-.65 2.73-.65 4.08 0 .41.2.58.68.37 1.07-.21.4-.71.56-1.12.36-.87-.43-1.71-.43-2.58 0z"],barcode:["M0 14h2V2H0v12zm6 0h1V2H6v12zm2 0h1V2H8v12zm-5 0h2V2H3v12zM15 2v12h1V2h-1zm-5 12h1V2h-1v12zm2 0h2V2h-2v12z"],blank:[],"blocked-person":["M9.39 12.69c-1.2-.53-1.04-.85-1.08-1.29-.01-.07-.01-.13-.02-.2.41-.37.75-.87.97-1.44 0 0 .01-.03.01-.04.05-.13.09-.26.13-.39.27-.06.43-.36.5-.63.01-.03.03-.08.05-.12C8.18 7.8 6.94 6.04 6.94 4c0-.32.04-.62.09-.92-.17-.03-.35-.08-.51-.08-.65 0-1.37.2-1.88.59-.5.38-.87.92-1.05 1.51-.04.14-.07.27-.09.41-.09.48-.14 1.23-.14 1.74v.06c-.19.08-.36.27-.4.68-.03.31.1.59.16.7.06.28.23.59.51.64.04.14.08.27.13.39 0 .01.01.02.01.02v.01c.22.59.57 1.1.99 1.46 0 .06-.01.12-.01.17-.04.44.08.76-1.12 1.29-1.2.53-3.01 1.1-3.38 1.95C-.12 15.5.03 16 .03 16h12.96s.15-.5-.22-1.36c-.37-.85-2.18-1.42-3.38-1.95zM11.97 0C9.75 0 7.94 1.79 7.94 4s1.8 4 4.03 4S16 6.21 16 4s-1.8-4-4.03-4zM9.96 4c0-1.1.9-2 2.01-2 .37 0 .72.11 1.02.28l-2.75 2.73c-.17-.3-.28-.64-.28-1.01zm2.01 2c-.37 0-.72-.11-1.02-.28l2.75-2.73c.18.3.28.64.28 1.01.01 1.1-.9 2-2.01 2z"],bold:["M11.7 7c.2-.4.3-1 .3-1.5v-.4V5c0-.1 0-.2-.1-.3v-.1C11.4 3.1 10.1 2 8.5 2H4c-.5 0-1 .4-1 1v10c0 .5.4 1 1 1h5c2.2 0 4-1.8 4-4 0-1.2-.5-2.3-1.3-3zM6 5h2c.6 0 1 .4 1 1s-.4 1-1 1H6V5zm3 6H6V9h3c.6 0 1 .4 1 1s-.4 1-1 1z"],book:["M2 1v14c0 .55.45 1 1 1h1V0H3c-.55 0-1 .45-1 1zm11-1h-1v7l-2-2-2 2V0H5v16h8c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],bookmark:["M11.2.01h-.15C11.03.01 11.02 0 11 0H5c-.02 0-.03.01-.05.01H4.8c-.44 0-.8.37-.8.82v14.75c0 .45.25.56.57.24l2.87-2.94c.31-.32.82-.32 1.13 0l2.87 2.94c.31.32.57.21.57-.24V.83C12 .38 11.64.01 11.2.01z"],box:["M6 10h4c.55 0 1-.45 1-1s-.45-1-1-1H6c-.55 0-1 .45-1 1s.45 1 1 1zm9.93-4.37v-.02L13.94.63C13.78.26 13.42 0 13 0H3c-.42 0-.78.26-.93.63L.08 5.61l-.01.02C.03 5.74 0 5.87 0 6v9c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V6c0-.13-.03-.26-.07-.37zM9 2h3.32l1.2 3H9V2zM3.68 2H7v3H2.48l1.2-3zM14 14H2V7h12v7z"],briefcase:["M15 3.98h-3v-2c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v2H1c-.55 0-1 .45-1 1v4h3v-1h2v1h6v-1h2v1h3v-4c0-.55-.45-1-1-1zm-5 0H6v-1h4v1zm3 7h-2v-1H5v1H3v-1H0v4c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-4h-3v1z"],build:["M15.39 12.41L7.7 6l1.07-1.1c.34-.34-.12-.63.12-1.26.88-2.17 3.41-2.35 3.41-2.35s.36-.37.71-.72C9.74-.81 7.53.53 6.54 1.4L3.12 4.9l-.71.72c-.39.4-.39 1.05 0 1.45l-.7.72c-.39-.4-1.02-.4-1.41 0s-.39 1.05 0 1.45l1.41 1.45c.39.4 1.02.4 1.41 0s.39-1.05 0-1.45l.71-.72c.39.4 1.02.4 1.41 0l.8-.82 6.39 7.67c.82.82 2.14.82 2.96 0 .81-.82.81-2.15 0-2.96z"],calculator:["M13 0H3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM6 14H4v-2h2v2zm0-3H4V9h2v2zm0-3H4V6h2v2zm3 6H7v-2h2v2zm0-3H7V9h2v2zm0-3H7V6h2v2zm3 6h-2V9h2v5zm0-6h-2V6h2v2zm0-3H4V2h8v3z"],calendar:["M11 3c.6 0 1-.5 1-1V1c0-.6-.4-1-1-1s-1 .4-1 1v1c0 .5.4 1 1 1zm3-2h-1v1c0 1.1-.9 2-2 2s-2-.9-2-2V1H6v1c0 1.1-.9 2-2 2s-2-.9-2-2V1H1c-.6 0-1 .5-1 1v12c0 .6.4 1 1 1h13c.6 0 1-.4 1-1V2c0-.6-.5-1-1-1zM5 13H2v-3h3v3zm0-4H2V6h3v3zm4 4H6v-3h3v3zm0-4H6V6h3v3zm4 4h-3v-3h3v3zm0-4h-3V6h3v3zM4 3c.6 0 1-.5 1-1V1c0-.6-.4-1-1-1S3 .4 3 1v1c0 .5.4 1 1 1z"],camera:["M15 3h-2.59L10.7 1.29A.956.956 0 0 0 10 1H6c-.28 0-.53.11-.71.29L3.59 3H1c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h2.56c1.1 1.22 2.67 2 4.44 2s3.34-.78 4.44-2H15c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1zM3 6H1V5h2v1zm5 6c-2.21 0-4-1.79-4-4s1.79-4 4-4 4 1.79 4 4-1.79 4-4 4zm0-6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],"caret-down":["M12 6.5c0-.28-.22-.5-.5-.5h-7a.495.495 0 0 0-.37.83l3.5 4c.09.1.22.17.37.17s.28-.07.37-.17l3.5-4c.08-.09.13-.2.13-.33z"],"caret-left":["M9.5 4c-.13 0-.24.05-.33.13l-4 3.5c-.1.09-.17.22-.17.37s.07.28.17.37l4 3.5a.495.495 0 0 0 .83-.37v-7c0-.28-.22-.5-.5-.5z"],"caret-right":["M11 8c0-.15-.07-.28-.17-.37l-4-3.5A.495.495 0 0 0 6 4.5v7a.495.495 0 0 0 .83.37l4-3.5c.1-.09.17-.22.17-.37z"],"caret-up":["M11.87 9.17s.01 0 0 0l-3.5-4C8.28 5.07 8.15 5 8 5s-.28.07-.37.17l-3.5 4a.495.495 0 0 0 .37.83h7a.495.495 0 0 0 .37-.83z"],"cell-tower":["M8.97 6.76c-.01-.05-.04-.08-.06-.13-.02-.05-.03-.1-.05-.15.08-.14.14-.3.14-.48 0-.55-.45-1-1-1s-1 .45-1 1c0 .18.06.34.14.48-.03.05-.03.1-.05.15-.02.05-.05.08-.06.13l-2 8c-.13.54.19 1.08.73 1.21a.995.995 0 0 0 1.21-.73L7.53 13h.94l.56 2.24a1 1 0 0 0 1.94-.48l-2-8zM3.72 1.7C4.1 1.3 4.09.67 3.7.28S2.67-.09 2.28.3c-3.05 3.12-3.05 8.28 0 11.4a.996.996 0 1 0 1.43-1.39c-2.28-2.35-2.28-6.27.01-8.61zM11.6 3.2c-.44-.33-1.07-.24-1.4.2-.33.44-.24 1.07.2 1.4.43.32.53 1.96-.04 2.43-.42.35-.48.98-.13 1.41.35.42.98.48 1.41.13 1.59-1.33 1.39-4.5-.04-5.57z","M13.72.3c-.39-.4-1.02-.4-1.41-.02s-.41 1.02-.03 1.42c2.29 2.34 2.29 6.26 0 8.6-.39.39-.38 1.03.02 1.41s1.03.38 1.41-.02c3.05-3.11 3.05-8.27.01-11.39zM5.4 7.23c-.57-.47-.47-2.11-.04-2.43.44-.33.53-.96.2-1.4s-.96-.53-1.4-.2c-1.44 1.07-1.63 4.24-.04 5.57.42.35 1.05.3 1.41-.13.35-.42.29-1.06-.13-1.41z"],changes:["M8.29 7.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3a1.003 1.003 0 0 0-1.42-1.42L13 7.59V1c0-.55-.45-1-1-1s-1 .45-1 1v6.59l-1.29-1.3a1.003 1.003 0 0 0-1.42 1.42zM14.5 13h-13c-.83 0-1.5.67-1.5 1.5S.67 16 1.5 16h13c.83 0 1.5-.67 1.5-1.5s-.67-1.5-1.5-1.5zM1 5c.28 0 .53-.11.71-.29L3 3.41V10c0 .55.45 1 1 1s1-.45 1-1V3.41L6.29 4.7c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-3-3C4.53.11 4.28 0 4 0s-.53.11-.71.29l-3 3A1.003 1.003 0 0 0 1 5z"],chart:["M0 15c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V9.4L0 11v4zm6-5.5V15c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-5l-1 1-3-1.5zM13 7l-1 1v7c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V7.88c-.26.07-.58.12-1 .12-1.96 0-2-1-2-1zm2-6h-3c-.55 0-1 .45-1 1s.45 1 1 1h.59L8.8 6.78 5.45 5.11v.01C5.31 5.05 5.16 5 5 5s-.31.05-.44.11V5.1l-4 2v.01C.23 7.28 0 7.61 0 8c0 .55.45 1 1 1 .16 0 .31-.05.44-.11v.01L5 7.12 8.55 8.9v-.01c.14.06.29.11.45.11.28 0 .53-.11.71-.29L14 4.41V5c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1z"],chat:["M6 10c-1.1 0-2-.9-2-2V3H1c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1v2a1.003 1.003 0 0 0 1.71.71L5.41 13H10c.55 0 1-.45 1-1v-1.17l-.83-.83H6zm9-10H6c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h4.59l2.71 2.71c.17.18.42.29.7.29.55 0 1-.45 1-1V9c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"chevron-backward":["M7.41 8l3.29-3.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L6 6.59V4c0-.55-.45-1-1-1s-1 .45-1 1v8c0 .55.45 1 1 1s1-.45 1-1V9.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L7.41 8z"],"chevron-down":["M12 5c-.28 0-.53.11-.71.29L8 8.59l-3.29-3.3a1.003 1.003 0 0 0-1.42 1.42l4 4c.18.18.43.29.71.29s.53-.11.71-.29l4-4A1.003 1.003 0 0 0 12 5z"],"chevron-forward":["M10 3c-.55 0-1 .45-1 1v2.59l-3.29-3.3a1.003 1.003 0 0 0-1.42 1.42L7.59 8 4.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L9 9.41V12c0 .55.45 1 1 1s1-.45 1-1V4c0-.55-.45-1-1-1z"],"chevron-left":["M7.41 8l3.29-3.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-4 4C5.11 7.47 5 7.72 5 8c0 .28.11.53.29.71l4 4a1.003 1.003 0 0 0 1.42-1.42L7.41 8z"],"chevron-right":["M10.71 7.29l-4-4a1.003 1.003 0 0 0-1.42 1.42L8.59 8 5.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"chevron-up":["M12.71 9.29l-4-4C8.53 5.11 8.28 5 8 5s-.53.11-.71.29l-4 4a1.003 1.003 0 0 0 1.42 1.42L8 7.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71z"],circle:["M8 0C3.6 0 0 3.6 0 8s3.6 8 8 8 8-3.6 8-8-3.6-8-8-8zm0 14c-3.3 0-6-2.7-6-6s2.7-6 6-6 6 2.7 6 6-2.7 6-6 6z"],"circle-arrow-down":["M11 7c-.28 0-.53.11-.71.29L9 8.59V5c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-1.29-1.3a1.003 1.003 0 0 0-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0 0 11 7zM8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6z"],"circle-arrow-left":["M11 7H7.41L8.7 5.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-3 3C4.11 7.47 4 7.72 4 8c0 .28.11.53.29.71l3 3a1.003 1.003 0 0 0 1.42-1.42L7.41 9H11c.55 0 1-.45 1-1s-.45-1-1-1zM8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6z"],"circle-arrow-right":["M8.71 4.29a1.003 1.003 0 0 0-1.42 1.42L8.59 7H5c-.55 0-1 .45-1 1s.45 1 1 1h3.59L7.3 10.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3zM8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6z"],"circle-arrow-up":["M8.71 4.29C8.53 4.11 8.28 4 8 4s-.53.11-.71.29l-3 3a1.003 1.003 0 0 0 1.42 1.42L7 7.41V11c0 .55.45 1 1 1s1-.45 1-1V7.41l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-3-3zM8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6z"],citation:["M15.02 5c0-1.66-1.34-3-3-3s-3 1.34-3 3a2.996 2.996 0 0 0 3.6 2.94C12.1 9.76 11.14 11 10.02 11c-.55 0-1 .45-1 1s.45 1 1 1c2.76 0 5-3.13 5-7 0-.2-.02-.39-.04-.58.01-.14.04-.28.04-.42zm-11-3c-1.66 0-3 1.34-3 3a2.996 2.996 0 0 0 3.6 2.94C4.1 9.76 3.14 11 2.02 11c-.55 0-1 .45-1 1s.45 1 1 1c2.76 0 5-3.13 5-7 0-.2-.02-.39-.04-.58.01-.14.04-.28.04-.42 0-1.66-1.35-3-3-3z"],clean:["M12 8l-1.2 2.796-2.8 1.2 2.8 1.197L12 16l1.2-2.807L16 12l-2.8-1.204zM5 0L3.5 3.5 0 4.995 3.5 6.5 5 10l1.5-3.5L10 5 6.5 3.5z"],clipboard:["M11 2c0-.55-.45-1-1-1h.22C9.88.4 9.24 0 8.5 0S7.12.4 6.78 1H7c-.55 0-1 .45-1 1v1h5V2zm2 0h-1v2H5V2H4c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],cloud:["M12 6c-.03 0-.07 0-.1.01A5 5 0 0 0 2 7c0 .11.01.22.02.33A3.51 3.51 0 0 0 0 10.5C0 12.43 1.57 14 3.5 14H12c2.21 0 4-1.79 4-4s-1.79-4-4-4z"],"cloud-download":["M11 11c-.28 0-.53.11-.71.29L9 12.59V8c0-.55-.45-1-1-1s-1 .45-1 1v4.59L5.71 11.3A.965.965 0 0 0 5 11a1.003 1.003 0 0 0-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0 0 11 11zm1-7c-.03 0-.07 0-.1.01A5 5 0 0 0 2 5c0 .11.01.22.02.33A3.51 3.51 0 0 0 0 8.5c0 1.41.84 2.61 2.03 3.17C2.2 10.17 3.46 9 5 9c.06 0 .13.02.19.02C5.07 8.7 5 8.36 5 8c0-1.66 1.34-3 3-3s3 1.34 3 3c0 .36-.07.7-.19 1.02.06 0 .13-.02.19-.02 1.48 0 2.7 1.07 2.95 2.47A3.964 3.964 0 0 0 16 8c0-2.21-1.79-4-4-4z"],"cloud-upload":["M8.71 7.29C8.53 7.11 8.28 7 8 7s-.53.11-.71.29l-3 3a1.003 1.003 0 0 0 1.42 1.42L7 10.41V15c0 .55.45 1 1 1s1-.45 1-1v-4.59l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-3-3zM12 4c-.03 0-.07 0-.1.01A5 5 0 0 0 2 5c0 .11.01.22.02.33a3.495 3.495 0 0 0 .07 6.37c-.05-.23-.09-.46-.09-.7 0-.83.34-1.58.88-2.12l3-3a2.993 2.993 0 0 1 4.24 0l3 3c.54.54.88 1.29.88 2.12 0 .16-.02.32-.05.47C15.17 10.78 16 9.5 16 8c0-2.21-1.79-4-4-4z"],code:["M15.71 7.29l-3-3a1.003 1.003 0 0 0-1.42 1.42L13.59 8l-2.29 2.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zM5 5a1.003 1.003 0 0 0-1.71-.71l-3 3C.11 7.47 0 7.72 0 8c0 .28.11.53.29.71l3 3a1.003 1.003 0 0 0 1.42-1.42L2.41 8 4.7 5.71c.19-.18.3-.43.3-.71zm4-3c-.48 0-.87.35-.96.81l-2 10c-.01.06-.04.12-.04.19 0 .55.45 1 1 1 .48 0 .87-.35.96-.81l2-10c.01-.06.04-.12.04-.19 0-.55-.45-1-1-1z"],"code-block":["M15 3h-2V2c0-.55-.45-1-1-1h-2c-.55 0-1 .45-1 1v1H7V2c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v1H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1zm-8.29 8.29a1.003 1.003 0 0 1-1.42 1.42l-3-3C2.11 9.53 2 9.28 2 9s.11-.53.29-.71l3-3a1.003 1.003 0 0 1 1.42 1.42L4.41 9l2.3 2.29zm7-1.58l-3 3a1.003 1.003 0 0 1-1.42-1.42L11.59 9l-2.3-2.29a1.003 1.003 0 0 1 1.42-1.42l3 3c.18.18.29.43.29.71s-.11.53-.29.71z"],cog:["M15.19 6.39h-1.85c-.11-.37-.27-.71-.45-1.04l1.36-1.36c.31-.31.31-.82 0-1.13l-1.13-1.13a.803.803 0 0 0-1.13 0l-1.36 1.36c-.33-.17-.67-.33-1.04-.44V.79c0-.44-.36-.8-.8-.8h-1.6c-.44 0-.8.36-.8.8v1.86c-.39.12-.75.28-1.1.47l-1.3-1.3c-.3-.3-.79-.3-1.09 0L1.82 2.91c-.3.3-.3.79 0 1.09l1.3 1.3c-.2.34-.36.7-.48 1.09H.79c-.44 0-.8.36-.8.8v1.6c0 .44.36.8.8.8h1.85c.11.37.27.71.45 1.04l-1.36 1.36c-.31.31-.31.82 0 1.13l1.13 1.13c.31.31.82.31 1.13 0l1.36-1.36c.33.18.67.33 1.04.44v1.86c0 .44.36.8.8.8h1.6c.44 0 .8-.36.8-.8v-1.86c.39-.12.75-.28 1.1-.47l1.3 1.3c.3.3.79.3 1.09 0l1.09-1.09c.3-.3.3-.79 0-1.09l-1.3-1.3c.19-.35.36-.71.48-1.1h1.85c.44 0 .8-.36.8-.8v-1.6a.816.816 0 0 0-.81-.79zm-7.2 4.6c-1.66 0-3-1.34-3-3s1.34-3 3-3 3 1.34 3 3-1.34 3-3 3z"],"collapse-all":["M7.29 6.71c.18.18.43.29.71.29s.53-.11.71-.29l4-4a1.003 1.003 0 0 0-1.42-1.42L8 4.59l-3.29-3.3a1.003 1.003 0 0 0-1.42 1.42l4 4zm1.42 2.58C8.53 9.11 8.28 9 8 9s-.53.11-.71.29l-4 4a1.003 1.003 0 0 0 1.42 1.42L8 11.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-4-4z"],"column-layout":["M15 1H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM4 13H2V3h2v10zm3 0H5V3h2v10zm7 0H8V3h6v10z"],comment:["M14 1H1c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h2v3a1.003 1.003 0 0 0 1.71.71L8.41 12H14c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM3.5 8C2.67 8 2 7.33 2 6.5S2.67 5 3.5 5 5 5.67 5 6.5 4.33 8 3.5 8zm4 0C6.67 8 6 7.33 6 6.5S6.67 5 7.5 5 9 5.67 9 6.5 8.33 8 7.5 8zm4 0c-.83 0-1.5-.67-1.5-1.5S10.67 5 11.5 5s1.5.67 1.5 1.5S12.33 8 11.5 8z"],comparison:["M7.99-.01c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1s1-.45 1-1v-14c0-.55-.45-1-1-1zm-3 3h-4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1zm10 0h-4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1zm0 3h-4v-2h4v2zm0 3h-4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1zm0 3h-4v-2h4v2zm-10-3h-4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1z"],compass:["M12 8c0 .14-.03.27-.08.39l-3 6.99c-.15.37-.51.62-.92.62s-.77-.25-.92-.61l-3-6.99a1.006 1.006 0 0 1 0-.79l3-6.99C7.23.25 7.59 0 8 0s.77.25.92.61l3 6.99c.05.13.08.26.08.4zM8 3.54L6.09 8h3.82L8 3.54z"],compressed:["M15.93 5.63v-.02L13.94.63C13.78.26 13.42 0 13 0H3c-.42 0-.78.26-.93.63L.08 5.61l-.01.02C.03 5.74 0 5.87 0 6v9c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V6c0-.13-.03-.26-.07-.37zM9 2h3.32l1.2 3H9V2zM3.68 2H7v3H2.48l1.2-3zM14 14H2V7h5v2.59l-1.29-1.3a1.003 1.003 0 0 0-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3a1.003 1.003 0 0 0-1.42-1.42L9 9.59V7h5v7z"],confirm:["M8.7 4.29a.965.965 0 0 0-.71-.3 1.003 1.003 0 0 0-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l5-5a1.003 1.003 0 0 0-1.42-1.42l-4.29 4.3L8.7 4.29zm5.22 3.01c.03.23.07.45.07.69 0 3.31-2.69 6-6 6s-6-2.69-6-6 2.69-6 6-6c.81 0 1.59.17 2.3.46l1.5-1.5A7.998 7.998 0 0 0-.01 7.99c0 4.42 3.58 8 8 8s8-3.58 8-8c0-.83-.13-1.64-.36-2.39l-1.71 1.7z"],console:["M15 15H1c-.55 0-1-.45-1-1V2c0-.55.45-1 1-1h14c.55 0 1 .45 1 1v12c0 .55-.45 1-1 1zM14 5H2v8h12V5zM4 6c.28 0 .53.11.71.29l2 2c.18.18.29.43.29.71s-.11.53-.29.71l-2 2a1.003 1.003 0 0 1-1.42-1.42L4.59 9l-1.3-1.29A1.003 1.003 0 0 1 4 6zm5 4h3c.55 0 1 .45 1 1s-.45 1-1 1H9c-.55 0-1-.45-1-1s.45-1 1-1z"],contrast:["M15.2 6.4h-1.44c-.13-.47-.32-.92-.56-1.34L14.26 4c.31-.31.31-.82 0-1.13l-1.13-1.13a.803.803 0 0 0-1.13 0L10.94 2.8c-.42-.24-.86-.42-1.34-.56V.8c0-.44-.36-.8-.8-.8H7.2c-.44 0-.8.36-.8.8v1.44c-.5.14-.96.34-1.4.59l-1-1c-.3-.3-.79-.3-1.09 0L1.83 2.91c-.3.3-.3.79 0 1.09l1 1c-.25.44-.45.9-.59 1.4H.8c-.44 0-.8.36-.8.8v1.6c0 .44.36.8.8.8h1.44c.13.47.32.92.56 1.34L1.74 12c-.31.31-.31.82 0 1.13l1.13 1.13c.31.31.82.31 1.13 0l1.06-1.06c.42.24.86.42 1.34.56v1.44c0 .44.36.8.8.8h1.6c.44 0 .8-.36.8-.8v-1.44c.5-.14.96-.33 1.4-.59l1 1c.3.3.79.3 1.09 0l1.09-1.09c.3-.3.3-.79 0-1.09l-1-1c.25-.43.45-.9.59-1.4h1.44c.44 0 .8-.36.8-.8V7.2a.818.818 0 0 0-.81-.8zM8 12c-2.21 0-4-1.79-4-4s1.79-4 4-4v8z"],control:["M13 8H8v5h5V8zm0-5H8v4h5V3zm2-3H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H2V2h12v12zM7 3H3v10h4V3z"],"credit-card":["M14.99 2.95h-14c-.55 0-1 .45-1 1v1h16v-1c0-.55-.45-1-1-1zm-15 10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-6h-16v6zm5.5-2h5c.28 0 .5.22.5.5s-.22.5-.5.5h-5c-.28 0-.5-.22-.5-.5s.23-.5.5-.5zm-3 0h1c.28 0 .5.22.5.5s-.22.5-.5.5h-1c-.28 0-.5-.22-.5-.5s.23-.5.5-.5z"],cross:["M9.41 8l3.29-3.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L8 6.59l-3.29-3.3a1.003 1.003 0 0 0-1.42 1.42L6.59 8 3.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L8 9.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L9.41 8z"],crown:["M2 6l3 2 3-4 3 4 3-2-1 6H3L2 6zm6-5a1 1 0 1 1 0 2 1 1 0 0 1 0-2zM1 3a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm14 0a1 1 0 1 1 0 2 1 1 0 0 1 0-2zM3 13h10v2H3v-2z"],cube:["M14.194 3.54L8 7.41 1.806 3.54 7.504.283a1 1 0 0 1 .992 0l5.698 3.255zm.75.71a1 1 0 0 1 .056.33v6.84a1 1 0 0 1-.504.868L8.5 15.714V8.277l6.444-4.027zm-13.888 0L7.5 8.277v7.437l-5.996-3.426A1 1 0 0 1 1 11.42V4.58a1 1 0 0 1 .056-.33z"],"cube-add":["M14 2h1a1 1 0 0 1 0 2h-1v1a1 1 0 0 1-2 0V4h-1a1 1 0 0 1 0-2h1V1a1 1 0 0 1 2 0v1zM9.136.65a3.001 3.001 0 0 0 .992 5.222c.018.058.038.115.059.172L8 7.41 1.806 3.54 7.504.283a1 1 0 0 1 .992 0l.64.365zM15 7.235v4.184a1 1 0 0 1-.504.868L8.5 15.714V8.277l2.187-1.367A2.994 2.994 0 0 0 13 8c.768 0 1.47-.289 2-.764zM1.056 4.25L7.5 8.277v7.437l-5.996-3.426A1 1 0 0 1 1 11.42V4.58a1 1 0 0 1 .056-.33z"],"cube-remove":["M10.365 5.933L8 7.41 1.806 3.54 7.504.283a1 1 0 0 1 .992 0l.64.365a3.001 3.001 0 0 0 1.228 5.283zM15 6v5.42a1 1 0 0 1-.504.868L8.5 15.714V8.277L12.143 6H15zM1.056 4.25L7.5 8.277v7.437l-5.996-3.426A1 1 0 0 1 1 11.42V4.58a1 1 0 0 1 .056-.33zM11 2h4a1 1 0 0 1 0 2h-4a1 1 0 0 1 0-2z"],"curved-range-chart":["M15 12H3.12l1.81-1.39c1.73 1.01 5.53-.03 9.08-2.61l-1.22-1.5C10.3 8.3 7.86 9.37 6.65 9.29L14.3 3.4l-.6-.8-7.83 6.03c-.01-1.07 1.8-3.19 4.47-5.13L9.12 2C5.38 4.7 3.34 8.1 4.25 9.87L2 11.6V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],cut:["M13 2s.71-1.29 0-2L8.66 5.07l1.05 1.32L13 2zm.07 8c-.42 0-.82.09-1.18.26L3.31 0c-.69.71 0 2 0 2l3.68 5.02-2.77 3.24A2.996 2.996 0 0 0 0 13c0 1.66 1.34 3 3 3s3-1.34 3-3c0-.46-.11-.89-.29-1.27L8.1 8.54l2.33 3.19c-.18.39-.29.82-.29 1.27 0 1.66 1.31 3 2.93 3S16 14.66 16 13s-1.31-3-2.93-3zM3 14c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm10.07 0c-.54 0-.98-.45-.98-1s.44-1 .98-1 .98.45.98 1-.44 1-.98 1z"],dashboard:["M5 4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zM4 7c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm4-2c.55 0 1-.45 1-1s-.45-1-1-1-1 .45-1 1 .45 1 1 1zm-2 6c0 1.1.9 2 2 2s2-.9 2-2c0-.53-2-5-2-5s-2 4.47-2 5zM8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6zm4-9c0-.55-.45-1-1-1s-1 .45-1 1 .45 1 1 1 1-.45 1-1zm0 2c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1z"],database:["M8 4c3.31 0 6-.9 6-2s-2.69-2-6-2C4.68 0 2 .9 2 2s2.68 2 6 2zm-6-.48V8c0 1.1 2.69 2 6 2s6-.9 6-2V3.52C12.78 4.4 10.56 5 8 5s-4.78-.6-6-1.48zm0 6V14c0 1.1 2.69 2 6 2s6-.9 6-2V9.52C12.78 10.4 10.56 11 8 11s-4.78-.6-6-1.48z"],delete:["M11.99 4.99a1.003 1.003 0 0 0-1.71-.71l-2.29 2.3L5.7 4.29a.965.965 0 0 0-.71-.3 1.003 1.003 0 0 0-.71 1.71l2.29 2.29-2.29 2.29A1.003 1.003 0 0 0 5.7 11.7l2.29-2.29 2.29 2.29a1.003 1.003 0 0 0 1.42-1.42L9.41 7.99 11.7 5.7c.18-.18.29-.43.29-.71zm-4-5c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.68 6-6 6z"],delta:["M8 0L0 16h16L8 0zM7 5l5 10H2L7 5z"],"derive-column":["M6.08 6.67h-.84c.24-.92.56-1.6.96-2.03.24-.27.48-.4.71-.4.05 0 .08.01.11.04s.04.06.04.1c0 .04-.03.11-.1.21-.06.1-.1.2-.1.29 0 .13.05.24.15.33.1.09.23.14.39.14.17 0 .31-.06.42-.17A.58.58 0 0 0 8 4.73c0-.22-.09-.39-.26-.53-.17-.13-.44-.2-.81-.2-.59 0-1.12.16-1.59.48-.48.32-.93.85-1.36 1.59-.15.26-.29.42-.42.49s-.35.11-.64.1l-.19.65h.81l-1.19 4.37c-.2.72-.33 1.16-.4 1.33-.1.24-.26.45-.46.62-.08.07-.18.1-.3.1-.03 0-.06-.01-.08-.03l-.03-.04c0-.02.03-.06.09-.11.06-.06.09-.14.09-.26 0-.13-.05-.23-.14-.32a.6.6 0 0 0-.4-.13c-.21 0-.38.05-.51.16s-.21.25-.21.4c0 .16.08.3.23.42.16.12.4.18.74.18.53 0 .99-.13 1.4-.39.41-.26.76-.65 1.07-1.19.3-.54.62-1.4.94-2.59l.68-2.53h.82l.2-.63zM15 0H8c-.55 0-1 .45-1 1v2h2V2h5v12H9v-1H7v2c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM8.3 9.94c.18.52.33.89.46 1.13.13.24.28.4.44.51.17.1.37.16.62.16.24 0 .49-.08.74-.25.33-.21.66-.58 1.01-1.09l-.21-.11c-.23.31-.41.5-.52.57a.44.44 0 0 1-.26.07c-.12 0-.24-.07-.36-.21-.2-.24-.46-.91-.8-2 .3-.49.55-.81.75-.96.15-.11.3-.16.47-.16.06 0 .17.02.34.06.16.04.31.06.43.06.17 0 .31-.06.43-.17.1-.11.16-.25.16-.43 0-.19-.06-.33-.17-.44-.12-.11-.28-.16-.49-.16-.19 0-.37.04-.54.13-.17.09-.39.27-.65.56-.2.21-.48.58-.87 1.11-.15-.66-.41-1.26-.78-1.81l-2.05.33-.04.21c.15-.03.28-.04.39-.04.2 0 .37.08.5.25.21.26.5 1.03.88 2.33-.29.37-.49.61-.6.72-.18.18-.33.3-.44.36-.09.04-.19.07-.3.07-.09 0-.23-.04-.42-.13a.866.866 0 0 0-.36-.09c-.2 0-.36.06-.49.18a.59.59 0 0 0-.19.46c0 .17.06.32.18.43.12.11.28.16.48.16.2 0 .38-.04.55-.11.17-.08.39-.24.65-.49.24-.27.6-.66 1.06-1.21z"],desktop:["M15 0H1C.45 0 0 .45 0 1v10c0 .55.45 1 1 1h4.75l-.5 2H4c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1h-1.25l-.5-2H15c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 10H2V2h12v8z"],"diagram-tree":["M15 8v3h-2V9H9v2H7V9H3v2H1V8a1 1 0 0 1 1-1h5V5h2v2h5a1 1 0 0 1 1 1zM1 12h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H1a1 1 0 0 1-1-1v-2a1 1 0 0 1 1-1zm12 0h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1h-2a1 1 0 0 1-1-1v-2a1 1 0 0 1 1-1zm-6 0h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H7a1 1 0 0 1-1-1v-2a1 1 0 0 1 1-1zM7 0h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H7a1 1 0 0 1-1-1V1a1 1 0 0 1 1-1z"],"direction-left":["M16 1.99l-16 6 16 6-4-6z"],"direction-right":["M16 7.99l-16-6 4 6-4 6z"],disable:["M7.99-.01c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm-6 8c0-3.31 2.69-6 6-6 1.3 0 2.49.42 3.47 1.12l-8.35 8.35c-.7-.98-1.12-2.17-1.12-3.47zm6 6c-1.3 0-2.49-.42-3.47-1.12l8.35-8.35c.7.98 1.12 2.17 1.12 3.47 0 3.32-2.68 6-6 6z"],document:["M9 0H3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V5L9 0zm3 14H4V2h4v4h4v8z"],"document-open":["M6 12c0 .55.45 1 1 1s1-.45 1-1V8c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1s.45 1 1 1h1.59L1.3 12.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L6 10.41V12zm4-12H4c-.55 0-1 .45-1 1v4h2V2h4v4h4v8H5.24l-1.8 1.8c.16.12.35.2.56.2h10c.55 0 1-.45 1-1V5l-5-5z"],"document-share":["M10 14H2V2h4v4h1c0-.83.36-1.55.91-2.09l-.03-.03.9-.9C8.3 2.45 8 1.77 8 1L7 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V8.22c-.53.48-1.23.78-2 .78v5zm5-14h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59l-3.3 3.29a1.003 1.003 0 0 0 1.42 1.42L14 3.41V5c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1z"],dollar:["M12.83 9.51c-.1-.3-.25-.58-.45-.84s-.45-.49-.75-.7c-.3-.2-.65-.36-1.05-.48-.16-.04-.43-.11-.8-.2-.35-.09-.73-.18-1.12-.28-.39-.1-.74-.19-1.06-.27-.31-.08-.49-.12-.54-.13-.43-.12-.78-.29-1.05-.52-.27-.23-.4-.55-.4-.95 0-.29.07-.53.21-.72.14-.19.32-.34.54-.46.22-.11.46-.19.72-.24.26-.05.52-.08.77-.08.74 0 1.35.15 1.83.46.48.3.75.83.81 1.56h2.14c0-.6-.13-1.13-.38-1.58-.25-.45-.59-.84-1.02-1.15-.43-.31-.93-.54-1.49-.7-.24-.06-.49-.1-.75-.14V1c0-.55-.45-1-1-1s-1 .45-1 1v1.08c-.23.03-.46.07-.68.13-.54.13-1.02.34-1.44.61-.42.28-.76.63-1.02 1.05-.26.43-.39.93-.39 1.5 0 .3.04.59.13.88.09.29.23.56.44.82.21.26.48.49.83.7.35.21.79.38 1.31.51.85.21 1.56.38 2.14.52.58.13 1.08.28 1.52.42.25.09.48.23.69.44.21.21.32.53.32.97 0 .21-.05.42-.14.63-.09.21-.24.39-.45.55-.21.16-.47.29-.81.39-.33.1-.73.15-1.2.15-.43 0-.84-.05-1.21-.14-.37-.09-.7-.24-.99-.43-.29-.2-.51-.45-.67-.76-.16-.31-.24-.68-.24-1.12H3c.01.71.15 1.32.43 1.84.27.52.64.94 1.1 1.27.46.33.99.58 1.61.74.27.07.56.12.85.16V15c0 .55.45 1 1 1s1-.45 1-1v-1.05c.3-.03.61-.08.9-.15.58-.13 1.1-.34 1.56-.63.46-.29.83-.66 1.11-1.11.28-.45.42-1 .42-1.64 0-.31-.05-.61-.15-.91z"],dot:["M8 5a3 3 0 1 0 0 6 3 3 0 1 0 0-6z"],"double-caret-horizontal":["M13.71 7.29l-3-3A1.003 1.003 0 0 0 9 5v6a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zM6 4c-.28 0-.53.11-.71.29l-3 3C2.11 7.47 2 7.72 2 8c0 .28.11.53.29.71l3 3A1.003 1.003 0 0 0 7 11V5c0-.55-.45-1-1-1z"],"double-caret-vertical":["M5 7h6a1.003 1.003 0 0 0 .71-1.71l-3-3C8.53 2.11 8.28 2 8 2s-.53.11-.71.29l-3 3A1.003 1.003 0 0 0 5 7zm6 2H5a1.003 1.003 0 0 0-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0 0 11 9z"],"double-chevron-down":["M7.29 8.71c.18.18.43.29.71.29s.53-.11.71-.29l4-4a1.003 1.003 0 0 0-1.42-1.42L8 6.59l-3.29-3.3a1.003 1.003 0 0 0-1.42 1.42l4 4zM12 8c-.28 0-.53.11-.71.29L8 11.59l-3.29-3.3a1.003 1.003 0 0 0-1.42 1.42l4 4c.18.18.43.29.71.29s.53-.11.71-.29l4-4A1.003 1.003 0 0 0 12 8z"],"double-chevron-left":["M4.41 8L7.7 4.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-4 4C2.11 7.47 2 7.72 2 8c0 .28.11.53.29.71l4 4a1.003 1.003 0 0 0 1.42-1.42L4.41 8zm5 0l3.29-3.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-4 4C7.11 7.47 7 7.72 7 8c0 .28.11.53.29.71l4 4a1.003 1.003 0 0 0 1.42-1.42L9.41 8z"],"double-chevron-right":["M9 8c0-.28-.11-.53-.29-.71l-4-4a1.003 1.003 0 0 0-1.42 1.42L6.59 8 3.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l4-4C8.89 8.53 9 8.28 9 8zm4.71-.71l-4-4a1.003 1.003 0 0 0-1.42 1.42L11.59 8 8.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"double-chevron-up":["M4 8c.28 0 .53-.11.71-.29L8 4.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-4-4C8.53 2.11 8.28 2 8 2s-.53.11-.71.29l-4 4A1.003 1.003 0 0 0 4 8zm4.71-.71C8.53 7.11 8.28 7 8 7s-.53.11-.71.29l-4 4a1.003 1.003 0 0 0 1.42 1.42L8 9.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-4-4z"],"doughnut-chart":["M11.86 7h4.05C15.45 3.39 12.61.52 9 .07v4.07A4 4 0 0 1 11.86 7zM12 8c0 2.21-1.79 4-4 4s-4-1.79-4-4 1.79-4 4-4V0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8h-4z"],download:["M7.99-.01c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zM11.7 9.7l-3 3c-.18.18-.43.29-.71.29s-.53-.11-.71-.29l-3-3A1.003 1.003 0 0 1 5.7 8.28l1.29 1.29V3.99c0-.55.45-1 1-1s1 .45 1 1v5.59l1.29-1.29a1.003 1.003 0 0 1 1.71.71c0 .27-.11.52-.29.7z"],"drag-handle-horizontal":["M2 9c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0-4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm4 4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm8-2c.55 0 1-.45 1-1s-.45-1-1-1-1 .45-1 1 .45 1 1 1zm0 2c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm-4-4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0 4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zM6 5c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1z"],"drag-handle-vertical":["M6 9c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm4-6c.55 0 1-.45 1-1s-.45-1-1-1-1 .45-1 1 .45 1 1 1zM6 13c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0-8c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0-4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm4 8c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0-4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0 8c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1z"],draw:["M14.9 11c-.3 0-.5.1-.7.3l-3 3c-.2.2-.3.4-.3.7 0 .6.5 1 1 1 .3 0 .5-.1.7-.3l3-3c.2-.2.3-.4.3-.7 0-.5-.4-1-1-1zm-1-1v-.2l-1-5c-.1-.3-.3-.6-.6-.7l-11-4-.3.3 5.8 5.8c.2-.1.4-.2.6-.2.8 0 1.5.7 1.5 1.5S8.3 9 7.4 9s-1.5-.7-1.5-1.5c0-.2.1-.4.2-.6L.3 1.1l-.3.3 4 11c.1.3.4.6.7.6l5 1h.2c.3 0 .5-.1.7-.3l3-3c.2-.2.3-.4.3-.7z"],"drive-time":["M15.12 4.76h-1.05l-.76-2.12c-.19-.53-.76-1.08-1.27-1.24 0 0-1.32-.4-4.04-.4-2.72 0-4.04.4-4.04.4-.5.16-1.07.71-1.26 1.24l-.77 2.12H.88c-.48 0-.88.42-.88.94s.4.94.88.94h.38L1 7c-.03.69 0 1.44 0 2v5c0 .66.38 1 1 1s1-.34 1-1v-1h10v1c0 .66.38 1 1 1s1-.34 1-1V9c0-.56-.01-1.37 0-2l-.26-.37h.38c.48 0 .88-.42.88-.93 0-.52-.4-.94-.88-.94zM5 10H3V8h2v2zm8 0h-2V8h2v2zm0-4H3c-.18 0-.06-.82 0-1l.73-1.63C3.79 3.19 3.82 3 4 3h8c.18 0 .21.19.27.37L13 5c.06.18.18 1 0 1z"],duplicate:["M15 0H5c-.55 0-1 .45-1 1v2h2V2h8v7h-1v2h2c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-4 4H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm-1 10H2V6h8v8z"],edit:["M3.25 10.26l2.47 2.47 6.69-6.69-2.46-2.48-6.7 6.7zM.99 14.99l3.86-1.39-2.46-2.44-1.4 3.83zm12.25-14c-.48 0-.92.2-1.24.51l-1.44 1.44 2.47 2.47 1.44-1.44c.32-.32.51-.75.51-1.24.01-.95-.77-1.74-1.74-1.74z"],eject:["M4 9h8a1.003 1.003 0 0 0 .71-1.71l-4-4C8.53 3.11 8.28 3 8 3s-.53.11-.71.29l-4 4A1.003 1.003 0 0 0 4 9zm8 1H4c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h8c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1z"],endorsed:["M15.86 7.5l-.81-1.42V4.5c0-.36-.19-.68-.49-.87l-1.37-.8-.81-1.41c-.19-.31-.51-.49-.86-.49H9.89L8.5.14a.948.948 0 0 0-1 0l-1.39.8H4.52a1 1 0 0 0-.86.49l-.8 1.37-1.44.83c-.3.19-.49.51-.49.87v1.65l-.8 1.37c-.08.15-.13.32-.13.49s.05.34.14.49l.8 1.37v1.65c0 .36.19.68.49.87l1.42.81.8 1.37c.19.31.51.49.86.49H6.1l1.39.8c.15.09.32.14.48.14s.34-.05.49-.14l1.39-.8h1.63a1 1 0 0 0 .86-.49l.81-1.41 1.37-.8c.3-.19.49-.51.49-.87V9.93l.81-1.42a.89.89 0 0 0 .04-1.01zm-4.12-.82l-4.01 4.01c-.18.18-.43.29-.71.29s-.53-.11-.71-.29l-2-2c-.18-.19-.3-.44-.3-.71a1.003 1.003 0 0 1 1.71-.71l1.3 1.3 3.3-3.3a1.003 1.003 0 0 1 1.71.71.95.95 0 0 1-.29.7z"],envelope:["M0 3.06v9.88L4.94 8 0 3.06zM14.94 2H1.06L8 8.94 14.94 2zm-6.41 8.53c-.14.14-.32.22-.53.22s-.39-.08-.53-.22L6 9.06 1.06 14h13.88L10 9.06l-1.47 1.47zM11.06 8L16 12.94V3.06L11.06 8z"],equals:["M3 5h10a1 1 0 0 1 0 2H3a1 1 0 1 1 0-2zm0 4h10a1 1 0 0 1 0 2H3a1 1 0 0 1 0-2z"],eraser:["M8.06 13.91l7.63-7.44c.41-.4.41-1.05 0-1.45L10.86.3c-.41-.4-1.08-.4-1.49 0L.31 9.13c-.41.4-.41 1.05 0 1.45l5.58 5.44h8.12v-.01c.55 0 1-.45 1-1s-.45-1-1-1H7.96l.1-.1zm-2.17.06L1.67 9.85l4.22-4.11 4.22 4.11-4.22 4.12z"],error:["M7.99-.01c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm1 13h-2v-2h2v2zm0-3h-2v-7h2v7z"],euro:["M6.52 3.18c.51-.27 1.12-.4 1.83-.4.48 0 .91.06 1.27.18.37.12.68.29.96.51.18.14.3.33.44.51l1.53-1.53c-.12-.11-.23-.22-.36-.32a5.61 5.61 0 0 0-1.74-.83c-.66-.2-1.36-.3-2.1-.3-.99 0-1.88.18-2.66.53-.79.35-1.45.82-2 1.41-.55.58-.96 1.27-1.26 2.06H2c-.55 0-1 .45-1 1s.45 1 1 1h.04c-.01.17-.04.33-.04.5 0 .17.03.33.04.5H2c-.55 0-1 .45-1 1s.45 1 1 1h.43c0 .01 0 .02.01.02a6.2 6.2 0 0 0 1.25 2.07 5.77 5.77 0 0 0 2 1.4c.78.34 1.67.51 2.66.51.81 0 1.54-.12 2.21-.36.67-.24 1.25-.59 1.75-1.03l.03-.03-1.55-1.33c-.01.01-.02.03-.03.04-.29.3-.63.53-1.02.69-.4.17-.85.25-1.37.25-.71 0-1.32-.13-1.83-.4s-.93-.62-1.25-1.07c-.19-.24-.34-.49-.46-.76H9c.55 0 1-.45 1-1s-.45-1-1-1H4.35c-.01-.17-.03-.33-.03-.5 0-.17.02-.34.03-.5H10c.55 0 1-.45 1-1s-.45-1-1-1H4.83c.13-.27.27-.52.44-.76.32-.44.74-.8 1.25-1.06zM14 8.98v0z"],exchange:["M1.99 5.99c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.89-2-2-2zm4.15 1.86a.495.495 0 1 0 .7-.7L5.7 5.99h5.79c.28 0 .5-.22.5-.5s-.22-.5-.5-.5H5.7l1.15-1.15a.495.495 0 1 0-.7-.7l-2 2c-.1.09-.16.21-.16.35s.06.26.15.35l2 2.01zm7.85-1.86c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.89-2-2-2zM9.85 8.14a.533.533 0 0 0-.36-.15.495.495 0 0 0-.35.85l1.15 1.15h-5.8c-.28 0-.5.22-.5.5s.22.5.5.5h5.79l-1.15 1.15a.495.495 0 1 0 .7.7l2-2c.09-.09.15-.22.15-.35s-.06-.26-.15-.35l-1.98-2z"],"exclude-row":["M0 10a1.003 1.003 0 0 0 1.71.71L3 9.41l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L4.41 8 5.7 6.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L3 6.59l-1.29-1.3A1.003 1.003 0 0 0 .29 6.71L1.59 8 .29 9.29C.11 9.47 0 9.72 0 10zm1-7h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm14 10H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm-1-7H9c-1.1 0-2 .9-2 2s.9 2 2 2h5c1.1 0 2-.9 2-2s-.9-2-2-2z"],"expand-all":["M4 7c.28 0 .53-.11.71-.29L8 3.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-4-4C8.53 1.11 8.28 1 8 1s-.53.11-.71.29l-4 4A1.003 1.003 0 0 0 4 7zm8 2c-.28 0-.53.11-.71.29L8 12.59l-3.29-3.3a1.003 1.003 0 0 0-1.42 1.42l4 4c.18.18.43.29.71.29s.53-.11.71-.29l4-4A1.003 1.003 0 0 0 12 9z"],export:["M4 6c.28 0 .53-.11.71-.29L7 3.41V11c0 .55.45 1 1 1s1-.45 1-1V3.41l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-4-4C8.53.11 8.28 0 8 0s-.53.11-.71.29l-4 4A1.003 1.003 0 0 0 4 6zm11 5c-.55 0-1 .45-1 1v2H2v-2c0-.55-.45-1-1-1s-1 .45-1 1v3c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1z"],"eye-off":["M16 7.97v-.02-.01-.02-.02a.672.672 0 0 0-.17-.36c-.49-.63-1.07-1.2-1.65-1.72l-3.16 2.26a2.978 2.978 0 0 1-2.98 2.9c-.31 0-.6-.06-.88-.15L5.09 12.3c.44.19.9.36 1.37.47.97.23 1.94.24 2.92.05.88-.17 1.74-.54 2.53-.98 1.25-.7 2.39-1.67 3.38-2.75.18-.2.37-.41.53-.62.09-.1.15-.22.17-.36v-.02-.02-.01-.02-.03c.01-.02.01-.03.01-.04zm-.43-4.17c.25-.18.43-.46.43-.8 0-.55-.45-1-1-1-.22 0-.41.08-.57.2l-.01-.01-2.67 1.91c-.69-.38-1.41-.69-2.17-.87a6.8 6.8 0 0 0-2.91-.05c-.88.18-1.74.54-2.53.99-1.25.7-2.39 1.67-3.38 2.75-.18.2-.37.41-.53.62-.23.29-.23.63-.01.92.51.66 1.11 1.25 1.73 1.79.18.16.38.29.56.44l-2.09 1.5.01.01c-.25.18-.43.46-.43.8 0 .55.45 1 1 1 .22 0 .41-.08.57-.2l.01.01 14-10-.01-.01zm-10.41 5a3.03 3.03 0 0 1-.11-.8 2.99 2.99 0 0 1 2.99-2.98c.62 0 1.19.21 1.66.53L5.16 8.8z"],"eye-on":["M10.29 6.7c.18.18.43.29.71.29s.53-.11.71-.29l4-4c.17-.18.29-.43.29-.7a1.003 1.003 0 0 0-1.71-.71L11 4.58 9.71 3.29A.997.997 0 0 0 9 3c-.55 0-1 .44-1 1a1 1 0 0 0 .3.7l1.99 2zM16 7.96v-.02-.01-.02-.02a.64.64 0 0 0-.17-.36c-.3-.4-.65-.76-1-1.12l-1.7 1.7c-.55.55-1.3.88-2.13.88-.06 0-.11-.01-.17-.02C10.42 10.15 9.32 11 8.01 11A3.005 3.005 0 0 1 6.4 5.46c-.24-.43-.39-.93-.39-1.46 0-.26.04-.5.1-.74-.7.2-1.37.5-2.01.86-1.26.7-2.4 1.68-3.4 2.77-.18.21-.36.41-.53.63-.22.29-.22.64 0 .93.51.67 1.12 1.27 1.73 1.81 1.33 1.17 2.85 2.15 4.53 2.55.97.23 1.95.24 2.92.05.89-.18 1.74-.54 2.54-.99 1.25-.71 2.4-1.69 3.39-2.78.18-.2.37-.41.54-.63.09-.1.15-.23.17-.37v-.02-.02-.01-.02-.03c.01-.01.01-.02.01-.03zM8.01 9c.48 0 .87-.35.96-.81a.55.55 0 0 1-.07-.09l-.02.01L7.8 7.03c-.45.1-.79.48-.79.96 0 .56.45 1.01 1 1.01z"],"eye-open":["M8.002 7.003a1.003 1.003 0 0 0 0 2.005 1.003 1.003 0 0 0 0-2.005zm7.988.972v-.02-.01-.02-.02a.675.675 0 0 0-.17-.36c-.509-.673-1.118-1.264-1.737-1.806-1.328-1.173-2.846-2.155-4.523-2.546a6.702 6.702 0 0 0-2.925-.06c-.889.18-1.738.541-2.546.992C2.84 4.837 1.692 5.81.694 6.902c-.18.211-.36.411-.53.632a.742.742 0 0 0 0 .932c.51.672 1.119 1.264 1.738 1.805 1.328 1.173 2.846 2.156 4.523 2.547.968.23 1.947.24 2.925.04.889-.18 1.738-.542 2.546-.993 1.248-.712 2.397-1.684 3.395-2.777.18-.2.37-.411.54-.632.09-.1.149-.23.169-.36v-.02-.02-.01-.02-.03c0-.01-.01-.01-.01-.02zm-7.988 3.038a2.998 2.998 0 0 1-2.995-3.008 2.998 2.998 0 0 1 2.995-3.008 2.998 2.998 0 0 1 2.996 3.008 2.998 2.998 0 0 1-2.996 3.008z"],"fast-backward":["M14 3c-.24 0-.44.09-.62.23l-.01-.01L9 6.72V4c0-.55-.45-1-1-1-.24 0-.44.09-.62.23v-.01l-5 4 .01.01C2.16 7.41 2 7.68 2 8s.16.59.38.77v.01l5 4 .01-.01c.17.14.37.23.61.23.55 0 1-.45 1-1V9.28l4.38 3.5.01-.01c.17.14.37.23.61.23.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"fast-forward":["M15 8c0-.32-.16-.59-.38-.77l.01-.01-5-4-.01.01A.987.987 0 0 0 9 3c-.55 0-1 .45-1 1v2.72l-4.38-3.5v.01A.987.987 0 0 0 3 3c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1 .24 0 .44-.09.62-.23l.01.01L8 9.28V12c0 .55.45 1 1 1 .24 0 .44-.09.62-.23l.01.01 5-4-.01-.01c.22-.18.38-.45.38-.77z"],feed:["M1.99 11.99c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.89-2-2-2zm1-4c-.55 0-1 .45-1 1s.45 1 1 1c1.66 0 3 1.34 3 3 0 .55.45 1 1 1s1-.45 1-1c0-2.76-2.24-5-5-5zm0-4c-.55 0-1 .45-1 1s.45 1 1 1c3.87 0 7 3.13 7 7 0 .55.45 1 1 1s1-.45 1-1a9 9 0 0 0-9-9zm0-4c-.55 0-1 .45-1 1s.45 1 1 1c6.08 0 11 4.92 11 11 0 .55.45 1 1 1s1-.45 1-1c0-7.18-5.82-13-13-13z"],"feed-subscribed":["M3 2c1.06 0 2.08.16 3.06.45.13-.71.52-1.32 1.05-1.76C5.82.25 4.44 0 3 0c-.55 0-1 .45-1 1s.45 1 1 1zM2 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm8.32-6.33a.99.99 0 0 0 1.4 0l3.98-3.98c.19-.18.3-.42.3-.7 0-.55-.45-.99-1-.99-.28 0-.52.11-.7.29l-3.28 3.28-1.29-1.29a.99.99 0 0 0-.7-.29 1 1 0 0 0-1 .99c0 .27.11.52.29.7l2 1.99zm3.73.53l-.93.93-.02-.02c-.17.17-.35.33-.56.45C13.47 9.16 14 11.02 14 13c0 .55.45 1 1 1s1-.45 1-1c0-2.5-.73-4.82-1.95-6.8zM3 8c-.55 0-1 .45-1 1s.45 1 1 1c1.66 0 3 1.34 3 3 0 .55.45 1 1 1s1-.45 1-1c0-2.76-2.24-5-5-5zm5.91-.91l-.03.03-2-2 .03-.03c-.11-.11-.23-.2-.33-.33A8.9 8.9 0 0 0 3 4c-.55 0-1 .45-1 1s.45 1 1 1c3.87 0 7 3.13 7 7 0 .55.45 1 1 1s1-.45 1-1c0-1.87-.57-3.61-1.55-5.06-.61-.11-1.13-.42-1.54-.85z"],film:["M15 1h-5v2H6V1H1c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h5v-2h4v2h5c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM4 13H2v-2h2v2zm0-3H2V8h2v2zm0-3H2V5h2v2zm0-3H2V2h2v2zm6 6H6V5h4v5zm4 3h-2v-2h2v2zm0-3h-2V8h2v2zm0-3h-2V5h2v2zm0-3h-2V2h2v2z"],filter:["M13.99.99h-12a1.003 1.003 0 0 0-.71 1.71l4.71 4.71V14a1.003 1.003 0 0 0 1.71.71l2-2c.18-.18.29-.43.29-.71V7.41L14.7 2.7a1.003 1.003 0 0 0-.71-1.71z"],"filter-keep":["M15 10c-.28 0-.53.11-.71.29L12 12.59l-1.29-1.29A.965.965 0 0 0 10 11a1.003 1.003 0 0 0-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0 0 15 10zm-3-8c0-.55-.45-1-1-1H1a1.003 1.003 0 0 0-.71 1.71L4 6.41V12a1.003 1.003 0 0 0 1.71.71l2-2c.18-.18.29-.43.29-.71V6.41l3.71-3.71c.18-.17.29-.42.29-.7z"],"filter-list":["M9 8c0 .55.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1h-5c-.55 0-1 .45-1 1zm3-6c0-.55-.45-1-1-1H1a1.003 1.003 0 0 0-.71 1.71L4 6.41V12a1.003 1.003 0 0 0 1.71.71l2-2c.18-.18.29-.43.29-.71V6.41l3.71-3.71c.18-.17.29-.42.29-.7zm3 8h-5c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm0 3h-5c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1z"],"filter-open":["M15.707 10.293a1 1 0 0 1 0 1.414l-3 3c-.63.63-1.707.184-1.707-.707V8c0-.89 1.077-1.337 1.707-.707l3 3zM12 2c0 .28-.11.53-.29.7L8 6.41V10c0 .28-.11.53-.29.71l-2 2A1.003 1.003 0 0 1 4 12V6.41L.29 2.71A1.003 1.003 0 0 1 1 1h10c.55 0 1 .45 1 1z"],"filter-remove":["M12 2c0-.55-.45-1-1-1H1a1.003 1.003 0 0 0-.71 1.71L4 6.41V12a1.003 1.003 0 0 0 1.71.71l2-2c.18-.18.29-.43.29-.71V6.41l3.71-3.71c.18-.17.29-.42.29-.7zm2.41 10l1.29-1.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L13 10.59 11.71 9.3A.965.965 0 0 0 11 9a1.003 1.003 0 0 0-.71 1.71l1.3 1.29-1.29 1.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l1.29-1.3 1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L14.41 12z"],flag:["M2.99 2.99c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1s1-.45 1-1v-11c0-.55-.45-1-1-1zm0-3c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm2 3.03v7.23c2.07-2.11 5.92 1.75 9 0V3.02c-3 2.07-6.94-2.03-9 0z"],flame:["M9.217 0c0 1.368.368 2.462 1.104 3.282C12.774 5.197 14 7.385 14 9.846c0 2.735-1.472 4.786-4.415 6.154 2.165-2.4 1.84-3.385-.368-6.4-2.342 1.2-1.967 2-1.592 3.6-.786 0-1.5 0-1.875-.4 0 .547.898 2 1.464 3.2-2.943-.82-6.092-5.744-4.988-6.154.736-.273 1.594-.137 2.575.41C3.575 5.333 5.047 1.915 9.217 0z"],flash:["M4 8c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1h2c.55 0 1-.45 1-1zm4-4c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1S7 .45 7 1v2c0 .55.45 1 1 1zM3.79 5.21a1.003 1.003 0 0 0 1.42-1.42l-1.5-1.5a1.003 1.003 0 0 0-1.42 1.42l1.5 1.5zm.71 5.29c-.28 0-.53.11-.71.29l-1.5 1.5a1.003 1.003 0 0 0 1.42 1.42l1.5-1.5a1.003 1.003 0 0 0-.71-1.71zm7-5c.28 0 .53-.11.71-.29l1.5-1.5a1.003 1.003 0 0 0-1.42-1.42l-1.5 1.5a1.003 1.003 0 0 0 .71 1.71zm.71 5.29a1.003 1.003 0 0 0-1.42 1.42l1.5 1.5a1.003 1.003 0 0 0 1.42-1.42l-1.5-1.5zM15 7h-2c-.55 0-1 .45-1 1s.45 1 1 1h2c.55 0 1-.45 1-1s-.45-1-1-1zM8 5C6.34 5 5 6.34 5 8s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3zm0 4c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm0 3c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1s1-.45 1-1v-2c0-.55-.45-1-1-1z"],"floppy-disk":["M15.71 2.29l-2-2A.997.997 0 0 0 13 0h-1v6H4V0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V3c0-.28-.11-.53-.29-.71zM14 15H2V9c0-.55.45-1 1-1h10c.55 0 1 .45 1 1v6zM11 1H9v4h2V1z"],"flow-branch":["M10.643 6.595c.22.418.344.894.344 1.399 0 .439-.094.855-.263 1.231l3.265 3.462-.002-1.75a.973.973 0 0 1 .314-.68.99.99 0 0 1 1.388.048c.186.2.316.46.3.715l-.009 4.03a.959.959 0 0 1-.3.68.972.972 0 0 1-.698.266l-4.053.002a.97.97 0 0 1-.679-.314 1.031 1.031 0 0 1 .05-1.42.972.972 0 0 1 .698-.266l1.7-.001-3.305-3.35a2.998 2.998 0 0 1-4.228-1.653H.999a1 1 0 0 1 0-2h4.166a2.998 2.998 0 0 1 4.06-1.735l3.449-3.268-1.745.002a.979.979 0 0 1-.631-1.692c.199-.186.459-.316.713-.3l4.025.009c.247.008.493.1.679.3.186.2.274.451.265.7l.002 4.046a.972.972 0 0 1-.313.68 1.03 1.03 0 0 1-1.42-.05.973.973 0 0 1-.266-.7V3.295l-3.34 3.301z"],"flow-end":["M9.702 7.31c.176.176.293.41.293.684a.976.976 0 0 1-.283.695c-1.888 1.91-2.892 2.918-3.011 3.027-.179.164-.42.284-.693.284a.995.995 0 0 1-.997-.985c0-.274.112-.541.292-.72.12-.12.624-.551 1.514-1.293H.98c-.536 0-.975-.47-.975-1.008 0-.537.439-.996.975-.996h5.837c-.895-.752-1.4-1.187-1.514-1.304a1.03 1.03 0 0 1-.292-.705C5.01 4.45 5.464 4 6 4c.272 0 .52.108.695.294A535.7 535.7 0 0 0 9.702 7.31zM13 11.002c-1.657 0-3-1.347-3-3.008a3.004 3.004 0 0 1 3-3.007c1.657 0 3 1.346 3 3.007a3.004 3.004 0 0 1-3 3.008z"],"flow-linear":["M4.16 9.002H.977C.44 9.002 0 8.532 0 7.994c0-.537.44-.99.978-.99h3.18A3.01 3.01 0 0 1 6.995 5a3.01 3.01 0 0 1 2.839 2.004h2.98c-.898-.756-1.404-1.193-1.518-1.31a1.03 1.03 0 0 1-.293-.705c0-.538.454-.989.992-.989.274 0 .521.108.697.294.118.124 1.122 1.13 3.014 3.016a.96.96 0 0 1 .293.684.975.975 0 0 1-.284.695l-3.018 3.027a.974.974 0 0 1-.694.284c-.553 0-1-.447-1-.985 0-.274.117-.545.293-.72l1.518-1.293H9.833A3.01 3.01 0 0 1 6.996 11 3.01 3.01 0 0 1 4.16 9.002z"],"flow-review":["M5.175 7.004a3.003 3.003 0 0 1 2.83-2.001c1.305 0 2.416.835 2.83 2.001h1.985c-.896-.756-1.401-1.193-1.515-1.31a1.03 1.03 0 0 1-.292-.705c0-.538.453-.989.99-.989a.95.95 0 0 1 .696.294c.117.124 1.12 1.13 3.008 3.016.176.176.293.41.293.684a.976.976 0 0 1-.283.695l-3.013 3.027a.995.995 0 0 1-1.691-.702c0-.273.116-.544.292-.72l1.515-1.292h-1.98a3.003 3.003 0 0 1-2.835 2.016A3.003 3.003 0 0 1 5.17 9.002H3.18l1.515 1.292c.176.176.292.447.292.72a.995.995 0 0 1-1.69.702L.282 8.69A.976.976 0 0 1 0 7.994c0-.273.117-.508.293-.684A535.858 535.858 0 0 0 3.3 4.294.95.95 0 0 1 3.997 4c.537 0 .99.45.99.989 0 .273-.12.528-.292.705-.114.117-.62.554-1.515 1.31h1.995z"],"flow-review-branch":["M10.392 10.647A3.002 3.002 0 0 1 6.16 8.995H3.37l1.338 1.318c.172.178.287.41.282.683-.01.536-.524.995-.99.995-.465 0-.63-.187-.747-.294L.281 8.682A.956.956 0 0 1 0 7.994a.971.971 0 0 1 .294-.687l3.01-3.028a.973.973 0 0 1 .697-.27c.536.01.998.485.989 1.021a.971.971 0 0 1-.295.687L3.37 6.997h2.79a3.002 3.002 0 0 1 4.106-1.716l2.416-2.277-1.732.004a.99.99 0 0 1-.679-.329.978.978 0 0 1 .05-1.378c.199-.186.459-.315.714-.3l4.012.005c.248.009.493.1.68.3.185.2.273.45.264.699L15.99 6.05a.973.973 0 0 1-.314.679 1.03 1.03 0 0 1-1.421-.048.971.971 0 0 1-.265-.699V4.29L11.65 6.602c.219.416.343.89.343 1.394 0 .451-.1.88-.279 1.263L14 11.68l-.004-1.73a.982.982 0 0 1 .323-.68.978.978 0 0 1 1.378.049c.187.2.316.46.3.714l-.004 4.011a.983.983 0 0 1-.3.691.972.972 0 0 1-.7.265l-4.046-.001a.987.987 0 0 1-.679-.326 1.017 1.017 0 0 1 .048-1.41.972.972 0 0 1 .699-.265h1.693l-2.315-2.35z"],flows:["M13.5 6a2.5 2.5 0 0 0-2.45 2h-1.3L5.74 4l-.75.75L8.25 8h-3.3a2.5 2.5 0 1 0 0 1h3.3l-3.26 3.25.75.75 4.01-4h1.3a2.5 2.5 0 1 0 2.45-3z"],"folder-close":["M-.01 14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V7h-16v7zm15-10H7.41L5.7 2.3a.965.965 0 0 0-.71-.3h-4c-.55 0-1 .45-1 1v3h16V5c0-.55-.45-1-1-1z"],"folder-new":["M10.165 7a3.003 3.003 0 0 0 2.827 2 3.003 3.003 0 0 0 2.827-2H16v7c0 .55-.45 1-1 1H1.01c-.55 0-1-.45-1-1V7h10.155zM8.76 6H0V3c0-.55.45-1 1-1h1.998c.28 0 .53.11.71.29L5.417 4h2.578c0 .768.29 1.469.765 2zm6.23-3c.55 0 1 .45 1 1s-.45 1-1 1h-.999v1c0 .55-.45 1-1 1-.549 0-.998-.45-.998-1V5h-1c-.55 0-1-.45-1-1s.45-1 1-1h1V2c0-.55.45-1 .999-1 .55 0 1 .45 1 1v1h.999z"],"folder-open":["M2.06 6.69c.14-.4.5-.69.94-.69h11V5c0-.55-.45-1-1-1H6.41l-1.7-1.71A.997.997 0 0 0 4 2H1c-.55 0-1 .45-1 1v9.84l2.05-6.15h.01zM16 8c0-.55-.45-1-1-1H4a.99.99 0 0 0-.94.69l-2 6c-.04.09-.06.2-.06.31 0 .55.45 1 1 1h11c.44 0 .81-.29.94-.69l2-6c.04-.09.06-.2.06-.31z"],"folder-shared":["M8.76 5.98c-.47-.53-.77-1.22-.77-1.99h-.58L5.7 2.29a.965.965 0 0 0-.71-.3h-4c-.55 0-1 .45-1 1v3h8.76l.01-.01zm6.23-2.99h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59l-3.3 3.3a.99.99 0 0 0-.29.7 1.003 1.003 0 0 0 1.71.71l3.29-3.29V8c0 .55.45 1 1 1s1-.45 1-1V4c0-.56-.45-1.01-1-1.01zm-1.98 7.23l-.9.9-.01-.01c-.54.55-1.28.89-2.11.89-1.66 0-3-1.34-3-3 0-.77.3-1.47.78-2H-.01v7c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-3.18c-.31.11-.65.18-1 .18-.76-.01-1.45-.31-1.98-.78z"],"folder-shared-open":["M13.02 10.22l-.9.9-.01-.01c-.54.55-1.28.89-2.11.89-1.66 0-3-1.34-3-3 0-.77.3-1.47.78-2H4a.99.99 0 0 0-.94.69l-2 6c-.04.09-.06.2-.06.31 0 .55.45 1 1 1h11c.44 0 .81-.29.94-.69l1.11-3.32c-.01 0-.03.01-.05.01-.77 0-1.45-.3-1.98-.78zM2.06 6.69c.14-.4.5-.69.94-.69h5.76l.01-.01C8.3 5.46 8 4.77 8 4H6.41l-1.7-1.71A.997.997 0 0 0 4 2H1c-.55 0-1 .45-1 1v9.84l2.05-6.15h.01zM15 3h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59l-3.3 3.29a1.003 1.003 0 0 0 1.42 1.42L14 6.41V8c0 .55.45 1 1 1s1-.45 1-1V4c0-.55-.45-1-1-1z"],follower:["M9.37 12.69c-1.2-.53-1.04-.85-1.08-1.29-.01-.06-.01-.12-.01-.19.41-.37.75-.87.97-1.44 0 0 .01-.03.01-.04.05-.13.09-.26.12-.39.28-.06.44-.36.5-.63.06-.11.19-.39.16-.7-.04-.4-.2-.59-.38-.67v-.07c0-.52-.05-1.26-.14-1.74a2.72 2.72 0 0 0-.09-.43 3.02 3.02 0 0 0-1.04-1.51C7.87 3.2 7.15 3 6.5 3c-.64 0-1.36.2-1.87.59-.5.38-.87.92-1.05 1.51-.04.13-.07.27-.09.4-.09.49-.14 1.24-.14 1.75v.06c-.19.07-.36.26-.4.68-.03.31.1.59.16.7.06.28.23.59.51.64.04.14.08.27.13.39 0 .01.01.02.01.02v.01c.22.59.57 1.1.99 1.46 0 .06-.01.12-.01.17-.04.44.08.76-1.12 1.29-1.2.53-3.01 1.1-3.38 1.95C-.13 15.5.02 16 .02 16h12.96s.15-.5-.22-1.36c-.38-.85-2.19-1.42-3.39-1.95zm6.33-10.4l-2-2a1.003 1.003 0 0 0-1.42 1.42l.3.29H9.99c-.55 0-1 .45-1 1s.45 1 1 1h2.58l-.29.29a1.003 1.003 0 0 0 1.42 1.42l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],following:["M9.37 12.69c-1.2-.53-1.04-.85-1.08-1.29-.01-.06-.01-.12-.01-.19.41-.37.75-.87.97-1.44 0 0 .01-.03.01-.04.05-.13.09-.26.12-.39.28-.06.44-.36.5-.63.06-.11.19-.39.16-.7-.04-.4-.2-.59-.38-.67v-.07c0-.52-.05-1.26-.14-1.74a2.72 2.72 0 0 0-.09-.43 3.02 3.02 0 0 0-1.04-1.51C7.87 3.2 7.15 3 6.5 3c-.64 0-1.36.2-1.87.59-.5.38-.87.92-1.05 1.51-.04.13-.07.27-.09.4-.09.49-.14 1.24-.14 1.75v.06c-.19.07-.36.26-.4.68-.03.31.1.59.16.7.06.28.23.59.51.64.04.14.08.27.13.39 0 .01.01.02.01.02v.01c.22.59.57 1.1.99 1.46 0 .06-.01.12-.01.17-.04.44.08.76-1.12 1.29-1.2.53-3.01 1.1-3.38 1.95C-.13 15.5.02 16 .02 16h12.96s.15-.5-.22-1.36c-.38-.85-2.19-1.42-3.39-1.95zM14.99 2h-2.58l.29-.29A1.003 1.003 0 0 0 11.28.29l-2 2c-.17.18-.29.43-.29.71 0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42L12.41 4h2.58c.55 0 1-.45 1-1s-.45-1-1-1z"],font:["M13.93 14.67L8.94.67h-.01C8.79.28 8.43 0 8 0s-.79.28-.93.67h-.01l-5 14h.01c-.04.1-.07.21-.07.33 0 .55.45 1 1 1 .43 0 .79-.28.93-.67h.01L5.49 11h5.02l1.55 4.34h.01c.14.38.5.66.93.66.55 0 1-.45 1-1 0-.12-.03-.23-.07-.33zM6.2 9L8 3.97 9.8 9H6.2z"],fork:["M13.7 9.29a1.003 1.003 0 0 0-1.42 1.42l.29.29H11.4l-5-5h6.17l-.29.29a1.003 1.003 0 0 0 1.42 1.42l2-2c.18-.18.29-.43.29-.71s-.11-.53-.29-.71l-2-2a1.003 1.003 0 0 0-1.42 1.42l.29.29H.99c-.55 0-1 .45-1 1s.45 1 1 1h2.59l6.71 6.71c.18.18.43.29.71.29h1.59l-.29.29a1.003 1.003 0 0 0 1.42 1.42l2-2c.18-.18.29-.43.29-.71s-.11-.53-.29-.71l-2.02-2z"],form:["M2 11v2h2v-2H2zM1 9h4c.55 0 1 .45 1 1v4c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1v-4c0-.55.45-1 1-1zm9-6h5c.55 0 1 .45 1 1s-.45 1-1 1h-5c-.55 0-1-.45-1-1s.45-1 1-1zM6 1a1.003 1.003 0 0 1 .71 1.71l-3 4C3.53 6.89 3.28 7 3 7s-.53-.11-.71-.29l-2-2a1.003 1.003 0 0 1 1.42-1.42L3 4.59l2.29-3.3C5.47 1.11 5.72 1 6 1zm4 10h5c.55 0 1 .45 1 1s-.45 1-1 1h-5c-.55 0-1-.45-1-1s.45-1 1-1z"],"full-circle":["M8 0a8 8 0 1 0 0 16A8 8 0 1 0 8 0z"],"full-stacked-chart":["M13 12h1c.55 0 1-.45 1-1V8h-3v3c0 .55.45 1 1 1zM10 2c0-.55-.45-1-1-1H8c-.55 0-1 .45-1 1v3h3V2zm0 4H7v3h3V6zm5-4c0-.55-.45-1-1-1h-1c-.55 0-1 .45-1 1v2h3V2zm0 3h-3v2h3V5zM5 5H2v3h3V5zm-2 7h1c.55 0 1-.45 1-1V9H2v2c0 .55.45 1 1 1zm12 1H2c-.55 0-1 .45-1 1s.45 1 1 1h13c.55 0 1-.45 1-1s-.45-1-1-1zM5 2c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v2h3V2zm3 10h1c.55 0 1-.45 1-1v-1H7v1c0 .55.45 1 1 1z"],fullscreen:["M3.41 2H5c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1v4c0 .55.45 1 1 1s1-.45 1-1V3.41L5.29 6.7c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L3.41 2zM6 9c-.28 0-.53.11-.71.29L2 12.59V11c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H3.41l3.29-3.29c.19-.18.3-.43.3-.71 0-.55-.45-1-1-1zm9 1c-.55 0-1 .45-1 1v1.59L10.71 9.3A.965.965 0 0 0 10 9a1.003 1.003 0 0 0-.71 1.71l3.3 3.29H11c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1zm0-10h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59l-3.3 3.29a1.003 1.003 0 0 0 1.42 1.42L14 3.41V5c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1z"],function:["M8.12 4.74H6.98c.33-1.29.75-2.24 1.28-2.84.33-.37.64-.56.95-.56.06 0 .11.02.15.05.04.04.06.09.06.15 0 .05-.04.15-.13.29-.09.14-.13.28-.13.4 0 .18.07.33.2.46.14.13.31.19.52.19.22 0 .41-.08.56-.23.15-.16.23-.37.23-.63 0-.3-.11-.55-.34-.74C10.1 1.09 9.74 1 9.24 1c-.78 0-1.49.22-2.12.67-.64.45-1.24 1.2-1.81 2.23-.2.36-.38.59-.56.69-.18.1-.46.15-.85.15l-.26.9h1.08l-1.59 6.12c-.27 1.01-.44 1.63-.54 1.86-.14.34-.34.63-.62.87-.11.1-.24.15-.4.15a.15.15 0 0 1-.11-.04l-.04-.05c0-.03.04-.08.12-.16.08-.08.12-.2.12-.36 0-.18-.06-.33-.19-.44-.12-.12-.3-.18-.54-.18-.28 0-.51.08-.68.23-.16.14-.25.32-.25.53 0 .22.1.42.31.59.21.17.53.25.97.25.7 0 1.32-.18 1.87-.54.54-.36 1.02-.92 1.42-1.67.41-.75.82-1.96 1.25-3.63l.91-3.54h1.1l.29-.89zm5.43 1.52c.2-.15.41-.23.62-.23.08 0 .23.03.45.09s.41.09.57.09c.23 0 .42-.08.57-.23.16-.16.24-.36.24-.61 0-.26-.08-.47-.23-.62-.15-.15-.37-.23-.66-.23-.25 0-.5.06-.72.18-.23.12-.51.38-.86.78-.26.3-.64.81-1.15 1.55-.2-.91-.55-1.75-1.05-2.51l-2.72.46-.06.29c.2-.04.37-.06.51-.06.27 0 .49.11.67.34.28.36.67 1.45 1.17 3.26-.39.52-.66.85-.8 1.01-.24.26-.44.42-.59.5-.12.06-.25.09-.41.09-.11 0-.3-.06-.56-.18-.18-.08-.34-.12-.48-.12-.27 0-.48.08-.66.25-.17.17-.26.38-.26.64 0 .25.08.44.24.6.16.15.37.23.64.23.26 0 .5-.05.73-.16.23-.11.52-.34.86-.69.35-.35.82-.9 1.43-1.67.23.73.44 1.25.61 1.58s.37.57.59.71c.22.15.5.22.83.22.32 0 .65-.11.98-.34.44-.3.88-.81 1.34-1.53l-.26-.15c-.31.43-.54.7-.69.8-.1.07-.22.1-.35.1-.16 0-.32-.1-.48-.3-.27-.34-.62-1.27-1.06-2.8.4-.68.73-1.13 1-1.34z"],"gantt-chart":["M10 10c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1h-4c-.55 0-1 .45-1 1zM6 7c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H7c-.55 0-1 .45-1 1zm9 5H2V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM4 5h3c.55 0 1-.45 1-1s-.45-1-1-1H4c-.55 0-1 .45-1 1s.45 1 1 1z"],geolocation:["M-.01 6.66l7.34 2 2 7.33 6.66-16z"],geosearch:["M8.82 12.4h.66c.23 0 .36-.17.36-.4v-1.48l.19-.18c-.27.03-.55.06-.83.06-.28 0-.56-.03-.84-.07.02.04.05.08.07.13V12c0 .23.15.4.39.4zM6.4 15.1A5.51 5.51 0 0 1 .9 9.6c0-.49.06-.98.18-1.43.03 0 .05-.01.08-.01h.08v.44c0 .19.17.34.36.34.03 0 .07-.01.1-.01l.71.7c.07.07.19.07.26 0s.07-.19 0-.26l-.7-.72c0-.02.03-.03.03-.05v-.11c0-.15.08-.2.23-.33h.42c.08 0 .15-.01.22-.04h.02c.02-.02.03-.02.04-.04.01-.01.01-.01.02-.01l.02-.01.9-.9c-.13-.26-.24-.52-.34-.8h-.5v-.43c0-.01.05.05.04-.08h.31c-.03-.13-.06-.26-.08-.39h-.57c.16-.12.34-.24.51-.36-.02-.23-.04-.46-.04-.7 0-.12.01-.23.02-.34A6.385 6.385 0 0 0 0 9.6C0 13.13 2.87 16 6.4 16c3.1 0 5.67-2.22 6.26-5.15l-.78-.88c-.21 2.85-2.58 5.13-5.48 5.13zm-1.7-2.93v-.28h.12c.23 0 .39-.19.39-.42v-.54s.01-.01 0-.01L3.77 9.45h-.62c-.23 0-.38.19-.38.42v1.6c0 .23.14.42.38.42h.26v1.61c0 .23.22.41.45.41s.45-.18.45-.41v-.97H4.3c.24 0 .4-.13.4-.36zm11.07-2.34l-2.94-2.94c.11-.17.21-.34.3-.52.01-.03.03-.06.04-.09.08-.18.16-.36.22-.55v-.01c.06-.19.1-.38.14-.58.01-.05.01-.09.02-.14.03-.2.05-.4.05-.61a4.4 4.4 0 0 0-4.4-4.4C6.77 0 4.8 1.97 4.8 4.4s1.97 4.4 4.4 4.4c.21 0 .41-.02.61-.05.04 0 .09-.01.14-.02.2-.03.39-.08.58-.14h.01c.19-.06.37-.14.55-.22.03-.01.06-.03.09-.04.18-.09.35-.19.52-.3l2.94 2.94a.8.8 0 0 0 .57.23c.44 0 .8-.36.8-.8a.895.895 0 0 0-.24-.57zM9.2 7.6C7.43 7.6 6 6.17 6 4.4c0-1.77 1.43-3.2 3.2-3.2s3.2 1.43 3.2 3.2c0 1.77-1.43 3.2-3.2 3.2zm1.54 4.26v-.52c0-.09-.1-.17-.19-.17s-.19.07-.19.17v.52c0 .09.1.17.19.17s.19-.07.19-.17z"],"git-branch":["M12 1c-1.66 0-3 1.34-3 3 0 1.25.76 2.32 1.85 2.77A2.02 2.02 0 0 1 9 8H7c-.73 0-1.41.2-2 .55V5.82C6.16 5.4 7 4.3 7 3c0-1.66-1.34-3-3-3S1 1.34 1 3c0 1.3.84 2.4 2 2.82v4.37c-1.16.4-2 1.51-2 2.81 0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.04-.53-1.95-1.32-2.49.35-.31.81-.51 1.32-.51h2c1.92 0 3.52-1.35 3.91-3.15A2.996 2.996 0 0 0 12 1zM4 2c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zm0 12c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm8-9c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-commit":["M15 7h-3.14c-.45-1.72-2-3-3.86-3S4.59 5.28 4.14 7H1c-.55 0-1 .45-1 1s.45 1 1 1h3.14c.45 1.72 2 3 3.86 3s3.41-1.28 3.86-3H15c.55 0 1-.45 1-1s-.45-1-1-1zm-7 3c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2z"],"git-merge":["M12 6c-1.3 0-2.4.84-2.82 2H9c-1.62 0-3-.96-3.63-2.34C6.33 5.16 7 4.16 7 3c0-1.66-1.34-3-3-3S1 1.34 1 3c0 1.3.84 2.4 2 2.81v4.37C1.84 10.6 1 11.7 1 13c0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.3-.84-2.4-2-2.82V8.43A5.89 5.89 0 0 0 9 10h.18A2.996 2.996 0 0 0 15 9c0-1.66-1.34-3-3-3zm-8 8c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zM4 4c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm8 6c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-new-branch":["M14 2h-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1V4h1c.55 0 1-.45 1-1s-.45-1-1-1zm-3.18 4.8C10.51 7.51 9.82 8 9 8H7c-.73 0-1.41.2-2 .55V5.82C6.16 5.4 7 4.3 7 3c0-1.66-1.34-3-3-3S1 1.34 1 3c0 1.3.84 2.4 2 2.82v4.37c-1.16.4-2 1.51-2 2.81 0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.04-.53-1.95-1.32-2.49.35-.31.81-.51 1.32-.51h2c1.9 0 3.49-1.33 3.89-3.11-.29.07-.58.11-.89.11-.41 0-.8-.08-1.18-.2zM4 2c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zm0 12c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-pull":["M3 1C1.34 1 0 2.34 0 4c0 1.3.84 2.4 2 2.82v3.37C.84 10.6 0 11.7 0 13c0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.3-.84-2.4-2-2.82V6.82C5.16 6.4 6 5.3 6 4c0-1.66-1.34-3-3-3zm0 13c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm0-9c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm11 5.18V6c0-1.66-1.34-3-3-3H9.41l1.29-1.29c.19-.18.3-.43.3-.71A1.003 1.003 0 0 0 9.29.29l-3 3C6.11 3.47 6 3.72 6 4c0 .28.11.53.29.71l3 3a1.003 1.003 0 0 0 1.42-1.42L9.41 5H11c.55 0 1 .45 1 1v4.18A2.996 2.996 0 0 0 13 16c1.66 0 3-1.34 3-3 0-1.3-.84-2.4-2-2.82zM13 14c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-push":["M4 6h1V5H4v1zm9 3c0-.28-.11-.53-.29-.71l-3-3C9.53 5.11 9.28 5 9 5s-.53.11-.71.29l-3 3a1.003 1.003 0 0 0 1.42 1.42L8 8.41V15c0 .55.45 1 1 1s1-.45 1-1V8.41l1.29 1.29c.18.19.43.3.71.3.55 0 1-.45 1-1zM5 3H4v1h1V3zm10-3H1C.45 0 0 .45 0 1v13c0 .55.45 1 1 1h5v-2H2v-1h4v-1H3V2h11v9h-2v1h2v1h-2v2h3c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"git-repo":["M5 9H4v1h1V9zm10-9H1C.45 0 0 .45 0 1v13c0 .55.45 1 1 1h3v1l2-1 2 1v-1h7c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM4 13H2v-1h2v1zm10 0H8v-1h6v1zm0-2H3V2h11v9zM5 3H4v1h1V3zm0 4H4v1h1V7zm0-2H4v1h1V5z"],glass:["M2 0v4c0 2.97 2.16 5.43 5 5.91V14H5c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1H9V9.91c2.84-.48 5-2.94 5-5.91V0H2z"],globe:["M4.45 7.83c-.26 0-.41.21-.41.46v1.75c0 .26.16.46.41.46h.29v1.77c0 .25.24.45.49.45s.49-.2.49-.45V11.2h-.01c.26 0 .44-.14.44-.4v-.3h.14c.26 0 .43-.2.43-.46v-.59s.01-.01 0-.01l-1.58-1.6h-.69zM8.51 3.9h.22c.06 0 .12-.01.12-.07 0-.06-.05-.07-.12-.07h-.22c-.06 0-.12.01-.12.07.01.06.06.07.12.07zm-2.33-.05c.07-.07.07-.19 0-.26l-.5-.5a.187.187 0 0 0-.26 0c-.07.07-.07.19 0 .26l.5.5c.07.07.19.07.26 0zm3.06.89c.07 0 .14-.06.14-.12v-.31c0-.07-.07-.12-.14-.12s-.14.06-.14.12v.31c0 .07.07.12.14.12zM8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6 0-.55.1-1.07.23-1.57h.11v.47c0 .2.18.37.39.37.03 0 .08-.01.11-.02l.78.77c.08.08.2.08.28 0 .08-.08.08-.2 0-.28l-.75-.78c0-.02.04-.04.04-.06v-.12c0-.16.09-.22.25-.36h.46c.09 0 .17-.01.24-.05h.02c.02-.01.03-.02.05-.03.01-.01.01-.01.02-.01l.02-.02 1.59-1.58c.18-.18.18-.46 0-.64s-.47-.15-.65.03l-.3.34h-.57v-.48c0-.01.05.05.05-.09h.64c.12 0 .22-.09.22-.21s-.1-.21-.22-.21H4.1c.18-.15.34-.31.54-.44l.01-.01c.21-.14.45-.25.68-.37.15-.07.29-.15.44-.21.17-.07.35-.11.53-.17.18-.05.35-.12.53-.16a6.05 6.05 0 0 1 3.47.35c.05.02.1.05.16.08.25.11.48.24.71.39.25.16.49.34.71.55H10.6s0-.03-.01-.03c-.04 0-.09 0-.13.03l-.51.51a.17.17 0 0 0 0 .23c.06.06.17.06.23 0l.42-.44.01-.02h.25c0 .14-.07.09-.07.12v.07c0 .22-.15.37-.36.37h-.38c-.19 0-.38.21-.38.4v.17h-.1c-.12 0-.2.06-.2.18v.25h-.23c-.17 0-.3.11-.3.28 0 .17.13.26.3.26.07 0 .14.03.19-.11l.04.01.49-.46h.17l.39.37c.03.03.08.02.12-.01.03-.03.03-.12 0-.15l-.32-.35h.23l.09.12c.18.18.48.17.66-.01l.09-.1h.4c.02 0 .08.05.08.05v.24l-.05-.01h-.36c-.11 0-.21.1-.21.21 0 .11.09.21.21.21h.41v.15c-.14.21-.24.42-.45.42h-.94v-.01l-.44-.44a.47.47 0 0 0-.66 0l-.42.43v.01H8.6c-.26 0-.49.21-.49.46v.92c0 .26.23.45.49.45h.9c.34.14.57.35.72.69v1.68c0 .26.17.44.42.44h.72c.26 0 .4-.18.4-.44V9l.89-.86.03-.02.02-.01h.03c.07-.08.15-.19.15-.31v-.91c0-.18-.16-.32-.31-.46H13c.01.28.21.42.46.42h.42c.08.37.12.76.12 1.15 0 3.31-2.69 6-6 6zm4.54-4.27c-.1 0-.21.08-.21.18v.57c0 .1.11.18.21.18.1 0 .21-.08.21-.18v-.57c0-.1-.11-.18-.21-.18zM8.37 3.19c0-.25-.2-.42-.46-.42h-.54c-.25 0-.42.18-.42.43 0 .03-.1.04.05.08v.47c0 .15.06.27.21.27s.21-.12.21-.27v-.14h.5c.24 0 .45-.16.45-.42z"],"globe-network":["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm5.17 5h-2.44c-.21-1.11-.51-2.03-.91-2.69 1.43.46 2.61 1.43 3.35 2.69zM10 8c0 .73-.05 1.39-.12 2H6.12C6.05 9.39 6 8.73 6 8s.05-1.39.12-2h3.76c.07.61.12 1.27.12 2zM8 2c.67 0 1.36 1.1 1.73 3H6.27C6.64 3.1 7.33 2 8 2zm-1.82.31c-.4.66-.71 1.58-.91 2.69H2.83a6.025 6.025 0 0 1 3.35-2.69zM2 8c0-.7.13-1.37.35-2h2.76C5.04 6.62 5 7.28 5 8s.04 1.38.11 2H2.35C2.13 9.37 2 8.7 2 8zm.83 3h2.44c.21 1.11.51 2.03.91 2.69A6.025 6.025 0 0 1 2.83 11zM8 14c-.67 0-1.36-1.1-1.73-3h3.46c-.37 1.9-1.06 3-1.73 3zm1.82-.31c.4-.66.7-1.58.91-2.69h2.44a6.025 6.025 0 0 1-3.35 2.69zM13.65 10h-2.76c.07-.62.11-1.28.11-2s-.04-1.38-.11-2h2.76c.22.63.35 1.3.35 2s-.13 1.37-.35 2z"],graph:["M14 3c-1.06 0-1.92.83-1.99 1.88l-1.93.97A2.95 2.95 0 0 0 8 5c-.56 0-1.08.16-1.52.43L3.97 3.34C3.98 3.23 4 3.12 4 3c0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.24 0 .47-.05.68-.13l2.51 2.09C5.08 7.29 5 7.63 5 8c0 .96.46 1.81 1.16 2.35l-.56 1.69c-.91.19-1.6.99-1.6 1.96 0 1.1.9 2 2 2s2-.9 2-2c0-.51-.2-.97-.51-1.32l.56-1.69A2.99 2.99 0 0 0 11 8c0-.12-.02-.24-.04-.36l1.94-.97c.32.21.69.33 1.1.33 1.1 0 2-.9 2-2s-.9-2-2-2z"],"graph-remove":["M12.89 8.11l-.01.01-.38-.38-.38.38-.02-.02c-.54.55-1.27.9-2.1.9-1.66 0-3-1.34-3-3 0-.83.35-1.56.9-2.1l-.02-.02.38-.38-.38-.38.01-.01C7.35 2.57 7 1.83 7 1c0-.34.07-.65.17-.96A8.004 8.004 0 0 0 0 8c0 4.42 3.58 8 8 8 4.14 0 7.54-3.14 7.96-7.17-.31.1-.62.17-.96.17-.83 0-1.57-.35-2.11-.89zm1.02-4.61l1.79-1.79c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-1.79 1.8L10.71.3A.965.965 0 0 0 10 0a1.003 1.003 0 0 0-.71 1.71l1.79 1.79-1.79 1.79a1.003 1.003 0 0 0 1.42 1.42l1.79-1.79 1.79 1.79a1.003 1.003 0 0 0 1.42-1.42l-1.8-1.79z"],"greater-than":["M2.713 5.958a1 1 0 0 1 .574-1.916l10 3c.95.285.95 1.63 0 1.916l-10 3a1 1 0 0 1-.574-1.916L9.52 8 2.713 5.958z"],"greater-than-or-equal-to":["M2.713 3.958a1 1 0 0 1 .574-1.916l10 3c.95.285.95 1.63 0 1.916l-10 3a1 1 0 0 1-.574-1.916L9.52 6 2.713 3.958zM3 12h10a1 1 0 0 1 0 2H3a1 1 0 0 1 0-2z"],grid:["M15 9c.55 0 1-.45 1-1s-.45-1-1-1h-1V4h1c.55 0 1-.45 1-1s-.45-1-1-1h-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1H9V1c0-.55-.45-1-1-1S7 .45 7 1v1H4V1c0-.55-.45-1-1-1S2 .45 2 1v1H1c-.55 0-1 .45-1 1s.45 1 1 1h1v3H1c-.55 0-1 .45-1 1s.45 1 1 1h1v3H1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1v-1h3v1c0 .55.45 1 1 1s1-.45 1-1v-1h3v1c0 .55.45 1 1 1s1-.45 1-1v-1h1c.55 0 1-.45 1-1s-.45-1-1-1h-1V9h1zm-8 3H4V9h3v3zm0-5H4V4h3v3zm5 5H9V9h3v3zm0-5H9V4h3v3z"],"grid-view":["M0 1v6h7V0H1C.45 0 0 .45 0 1zm0 14c0 .55.45 1 1 1h6V9H0v6zM15 0H9v7h7V1c0-.55-.45-1-1-1zM9 16h6c.55 0 1-.45 1-1V9H9v7z"],"group-objects":["M5 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm6-3H5C2.24 3 0 5.24 0 8s2.24 5 5 5h6c2.76 0 5-2.24 5-5s-2.24-5-5-5zm0 9H5c-2.21 0-4-1.79-4-4s1.79-4 4-4h6c2.21 0 4 1.79 4 4s-1.79 4-4 4zm0-6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],"grouped-bar-chart":["M10 12c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1s-1 .45-1 1v8c0 .55.45 1 1 1zm3 0c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1s-1 .45-1 1v5c0 .55.45 1 1 1zm2 1H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm-9-1c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1s-1 .45-1 1v3c0 .55.45 1 1 1zm-3 0c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1s-1 .45-1 1v9c0 .55.45 1 1 1z"],hand:["M15 5c0-.55-.45-1-1-1-.41 0-.76.24-.91.59v.01s0 .01-.01.01L11.57 8h-.36l.78-4.84C12 3.11 12 3.05 12 3a1 1 0 1 0-1.99-.16v.01L9.18 8H9V1c0-.55-.45-1-1-1S7 .45 7 1v7h-.09l-.93-5.18A1 1 0 0 0 5 2c-.55 0-1 .45-1 1 0 .05 0 .11.01.16L5.26 11h-.04L2.83 7.44C2.65 7.18 2.35 7 2 7c-.55 0-1 .45-1 1 0 .17.04.33.12.47l3 5.69h.01v.01A5.002 5.002 0 0 0 13 11v-.59l1.93-5.05c.05-.11.07-.23.07-.36z"],"hand-down":["M14.72 7.87c-1.54-.67-2.99-2.68-3.7-3.95C10.11 1.95 9.93 0 6.14 0 4.05 0 2.71.61 1.92 2.12 1.27 3.36 1 5.21 1 7.83v.79c0 .65.6 1.18 1.35 1.18.34 0 .64-.11.88-.29.17.48.68.84 1.29.84.41 0 .78-.16 1.03-.42.23.37.67.63 1.19.63.57 0 1.05-.31 1.25-.74l.01.63v4.05c0 .83.67 1.5 1.5 1.5s1.5-.67 1.5-1.5V7.9c.58.41 1.55 1.21 2.47 1.29 1.57.14 1.82-1.07 1.25-1.32z"],"hand-left":["M12.08 4.97c-1.26-.71-3.27-2.15-3.95-3.7C7.88.7 6.67.96 6.81 2.52c.09.93.89 1.9 1.3 2.48H1.5C.67 5 0 5.67 0 6.5S.67 8 1.5 8h4.05l.63.01c-.44.2-.75.69-.75 1.25 0 .52.26.96.63 1.19-.26.25-.42.61-.42 1.03 0 .61.35 1.12.84 1.29-.18.24-.29.54-.29.88 0 .75.54 1.35 1.19 1.35h.79c2.62 0 4.47-.28 5.71-.92 1.51-.79 2.12-2.14 2.12-4.22 0-3.79-1.95-3.97-3.92-4.89z"],"hand-right":["M14.5 5H7.89c.41-.58 1.21-1.55 1.3-2.47C9.34.97 8.12.71 7.87 1.28c-.67 1.54-2.68 2.99-3.95 3.7C1.95 5.89 0 6.07 0 9.86c0 2.09.61 3.43 2.12 4.22 1.24.65 3.09.92 5.71.92h.79c.65 0 1.18-.6 1.18-1.35 0-.34-.11-.64-.29-.88.48-.17.84-.68.84-1.29 0-.41-.16-.78-.42-1.03.37-.23.63-.67.63-1.19 0-.57-.31-1.05-.74-1.25l.63-.01h4.05c.83 0 1.5-.67 1.5-1.5S15.33 5 14.5 5z"],"hand-up":["M13.65 6.19c-.34 0-.64.11-.88.29-.17-.48-.68-.84-1.29-.84-.41 0-.78.16-1.03.42-.23-.37-.67-.63-1.19-.63-.57 0-1.05.31-1.25.74L8 5.55V1.5C8 .67 7.33 0 6.5 0S5 .67 5 1.5v6.61c-.58-.41-1.55-1.21-2.48-1.3C.96 6.67.7 7.88 1.28 8.13c1.54.67 2.99 2.68 3.7 3.95C5.89 14.05 6.07 16 9.86 16c2.09 0 3.43-.61 4.22-2.12.64-1.24.92-3.09.92-5.71v-.79c0-.65-.6-1.19-1.35-1.19z"],header:["M13 1c-.55 0-1 .45-1 1v5H4V2c0-.55-.45-1-1-1s-1 .45-1 1v12c0 .55.45 1 1 1s1-.45 1-1V9h8v5c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1z"],"header-one":["M14.06 8c-.04.23-.12.44-.25.61-.13.17-.29.3-.48.41-.18.11-.39.18-.62.23-.23.04-.46.07-.71.07v1.03h1.74V16H15V8h-.94zM7 0c-.56 0-1 .45-1 1v4H2V1c0-.55-.45-1-1-1-.56 0-1 .45-1 1v10c0 .55.45 1 1 1 .56 0 1-.45 1-1V7h4v4c0 .55.45 1 1 1 .56 0 1-.45 1-1V1c0-.54-.45-1-1-1z"],"header-two":["M13.17 13.93c-.17.15-.33.29-.46.44-.13.16-.22.32-.27.49h3.55V16H11c.01-.65.16-1.22.44-1.71s.67-.91 1.17-1.27c.24-.18.49-.36.75-.54.25-.18.49-.36.71-.57.21-.2.39-.42.53-.65.14-.24.21-.51.22-.82 0-.14-.02-.29-.05-.45-.03-.16-.09-.31-.18-.45a1.13 1.13 0 0 0-.37-.35c-.16-.09-.37-.14-.63-.14-.24 0-.43.05-.59.15-.16.1-.29.24-.38.42-.1.17-.17.38-.21.62-.05.24-.07.5-.08.77h-1.19c0-.43.05-.83.16-1.2s.27-.69.49-.96c.21-.25.48-.46.79-.62.31-.15.67-.23 1.07-.23.45 0 .82.08 1.11.23.3.16.55.36.73.6.19.24.32.5.39.79.08.28.12.54.12.79 0 .31-.04.6-.13.85s-.22.49-.37.7c-.15.21-.32.41-.52.59s-.4.35-.61.51l-.63.45c-.21.14-.39.28-.57.42zM0 1c0-.55.44-1 1-1 .55 0 1 .46 1 1v10c0 .55-.44 1-1 1-.55 0-1-.46-1-1V1zm6 0c0-.55.44-1 1-1 .55 0 1 .46 1 1v10c0 .55-.44 1-1 1-.55 0-1-.46-1-1V1zM2 5h4v2H2V5z"],headset:["M14.85 6.34C14.18 2.72 11.37 0 8 0S1.82 2.72 1.15 6.34C.47 6.9 0 8.1 0 9.5 0 11.43.9 13 2 13c0 1.1.9 2 2 2h2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1H7c-.55 0-1 .45-1 1H4c-.55 0-1-.45-1-1 .55 0 1-.45 1-1V7c0-.45-.3-.81-.71-.94C3.97 3.7 5.81 2 8 2s4.03 1.7 4.71 4.06c-.41.13-.71.49-.71.94v5c0 .55.45 1 1 1h1c1.1 0 2-1.57 2-3.5 0-1.4-.47-2.6-1.15-3.16z"],heart:["M16 5.095c0-2.255-1.88-4.083-4.2-4.083-1.682 0-3.13.964-3.8 2.352a4.206 4.206 0 0 0-3.8-2.352C1.88 1.012 0 2.84 0 5.095c0 .066.007.13.01.194H.004c.001.047.01.096.014.143l.013.142c.07.8.321 1.663.824 2.573C2.073 10.354 4.232 12.018 8 15c3.767-2.982 5.926-4.647 7.144-6.854.501-.905.752-1.766.823-2.562.007-.055.012-.11.016-.164.003-.043.012-.088.013-.13h-.006c.003-.066.01-.13.01-.195z"],"heart-broken":["M7.71 8.87L6.17 6.55l.02-.01A.906.906 0 0 1 6 6c0-.07.03-.13.04-.19h-.02l.78-3.92C6.09 1.34 5.19 1 4.2 1 1.88 1 0 2.83 0 5.09c0 .07.01.13.01.19H0c0 .05.01.1.01.14 0 .05.01.1.01.14.07.8.32 1.66.82 2.57 1.07 1.94 2.88 3.47 5.86 5.84l-.68-2.74h.02C6.03 11.16 6 11.08 6 11c0-.28.11-.53.29-.71l1.42-1.42zM16 5.09C16 2.83 14.12 1 11.8 1c-1.2 0-2.27.5-3.04 1.28l-.7 3.51 1.77 2.66-.01.01c.1.15.18.33.18.54 0 .28-.11.53-.29.71l-1.6 1.6.75 3.01c3.23-2.56 5.16-4.15 6.28-6.18.5-.91.75-1.77.82-2.56.01-.05.01-.11.02-.16 0-.04.01-.09.01-.13h-.01c.01-.07.02-.14.02-.2z"],"heat-grid":["M0 10h5V7H0v3zm1-2h3v1H1V8zm14-5h-4v3h5V4c0-.55-.45-1-1-1zm0 2h-3V4h3v1zM0 4v2h5V3H1c-.55 0-1 .45-1 1zm0 9c0 .55.45 1 1 1h4v-3H0v2zm6-7h4V3H6v3zm0 8h4v-3H6v3zm1-2h2v1H7v-1zm4 2h4c.55 0 1-.45 1-1v-2h-5v3zm0-4h5V7h-5v3zm-5 0h4V7H6v3z"],heatmap:["M2 11c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm11-7c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm3 4.5A2.5 2.5 0 0 0 13.5 6c-.98 0-1.82.57-2.23 1.39-.6-.78-1.51-1.3-2.56-1.36.18-.49.29-.99.29-1.53C9 2.01 6.99 0 4.5 0S0 2.01 0 4.5 2.01 9 4.5 9c.19 0 .37-.03.56-.06-.03.19-.06.37-.06.56C5 11.43 6.57 13 8.5 13c1.63 0 2.98-1.11 3.37-2.62.44.38 1 .62 1.63.62A2.5 2.5 0 0 0 16 8.5zM14.5 13c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5z"],help:["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm1 13H7v-2h2v2zm1.93-6.52c-.14.32-.35.64-.62.97L9.25 8.83c-.12.15-.24.29-.28.42-.04.13-.09.3-.09.52V10H7.12V8.88s.05-.51.21-.71L8.4 6.73c.22-.26.35-.49.44-.68.09-.19.12-.38.12-.58 0-.3-.1-.55-.28-.75-.18-.19-.44-.28-.76-.28-.33 0-.59.1-.78.29-.19.19-.33.46-.4.81-.03.11-.1.15-.2.14l-1.7-.25c-.12-.01-.16-.08-.14-.19.12-.82.46-1.47 1.03-1.94.57-.48 1.32-.72 2.25-.72.47 0 .9.07 1.29.22s.72.34 1 .59c.28.25.49.55.65.89.15.35.22.72.22 1.12s-.07.75-.21 1.08z"],"helper-management":["M13 5h-2v2h2V5zm0 6h-2v2h2v-2zm0-3h-2v2h2V8zm2-8H1C.4 0 0 .4 0 1v14c0 .6.4 1 1 1h14c.6 0 1-.4 1-1V1c0-.6-.4-1-1-1zm-1 14H2V2h12v12zm-7-3H5v2h2v-2zm3 0H8v2h2v-2z"],highlight:["M9.12 11.07l2-2.02.71.71 4-4.04L10.17 0l-4 4.04.71.71-2 2.02 4.24 4.3zM2 12.97h4c.28 0 .53-.11.71-.3l1-1.01-3.42-3.45-3 3.03c-.18.18-.29.44-.29.72 0 .55.45 1.01 1 1.01zm13 1.01H1c-.55 0-1 .45-1 1.01S.45 16 1 16h14c.55 0 1-.45 1-1.01s-.45-1.01-1-1.01z"],history:["M8 3c-.55 0-1 .45-1 1v4c0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42L9 7.59V4c0-.55-.45-1-1-1zm0-3a7.95 7.95 0 0 0-6 2.74V1c0-.55-.45-1-1-1S0 .45 0 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H3.54C4.64 2.78 6.23 2 8 2c3.31 0 6 2.69 6 6 0 2.61-1.67 4.81-4 5.63v-.01c-.63.23-1.29.38-2 .38-3.31 0-6-2.69-6-6 0-.55-.45-1-1-1s-1 .45-1 1c0 4.42 3.58 8 8 8 .34 0 .67-.03 1-.07.02 0 .04-.01.06-.01C12.98 15.4 16 12.06 16 8c0-4.42-3.58-8-8-8z"],home:["M2 10v5c0 .55.45 1 1 1h3v-5h4v5h3c.55 0 1-.45 1-1v-5L8 4l-6 6zm13.71-2.71L14 5.59V2c0-.55-.45-1-1-1s-1 .45-1 1v1.59L8.71.29C8.53.11 8.28 0 8 0s-.53.11-.71.29l-7 7a1.003 1.003 0 0 0 1.42 1.42L8 2.41l6.29 6.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71z"],"horizontal-bar-chart":["M4 5h7c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zM1 1c-.55 0-1 .45-1 1v13c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1zm14 6H4c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h11c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-6 5H4c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h5c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1z"],"horizontal-bar-chart-asc":["M1 3h5c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm0 4h7c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm14 6H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM1 11h10c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1z"],"horizontal-bar-chart-desc":["M15 1H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM8 9H1c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1zm-2 4H1c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm5-8H1c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1z"],"horizontal-distribution":["M2 0c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm13 0c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm-5 2H7c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],"id-number":["M2 5v7h12V5H2zm0-2h12c1.1 0 2 .9 2 2v7c0 1.1-.9 2-2 2H2c-1.1 0-2-.9-2-2V5c0-1.1.9-2 2-2z","M7.9 10.48c-.14-.33-.84-.55-1.3-.75-.46-.2-.4-.33-.42-.5v-.07c.16-.14.29-.33.37-.56 0 0 0-.01.01-.02.02-.05.03-.1.05-.15.1-.01.16-.13.19-.23.03-.04.07-.15.06-.27-.02-.16-.08-.24-.15-.26v-.03c0-.2-.02-.48-.05-.67-.01-.05-.02-.1-.03-.16-.07-.23-.21-.44-.4-.58-.2-.15-.48-.23-.73-.23s-.53.08-.72.23c-.19.14-.33.35-.4.58-.02.05-.03.1-.03.16-.05.18-.06.47-.06.67v.03c-.07.03-.14.1-.15.26-.02.12.03.22.06.27.02.1.09.22.2.24.01.05.03.1.05.15v.01c.08.23.22.42.38.56v.07c-.02.17.03.29-.43.5-.46.2-1.16.42-1.3.75s-.09.52-.09.52H8c-.01 0 .05-.19-.1-.52zM10 6h2c.55 0 1 .45 1 1s-.45 1-1 1h-2c-.55 0-1-.45-1-1s.45-1 1-1zM10 9h2c.55 0 1 .45 1 1s-.45 1-1 1h-2c-.55 0-1-.45-1-1s.45-1 1-1z"],"image-rotate-left":["M13 2h-1.59l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-2 2C8.11 2.47 8 2.72 8 3c0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42l-.3-.29H13c.55 0 1 .45 1 1v3c0 .55.45 1 1 1s1-.45 1-1V5c0-1.66-1.34-3-3-3zm-5.5 9c.28 0 .5-.22.5-.5s-.22-.5-.5-.5-.5.22-.5.5.22.5.5.5zM10 7H1c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-1 6.33L7 12l-1 1-2-3-2 2.67V9h7v4.33z"],"image-rotate-right":["M5.71 5.71l2-2C7.89 3.53 8 3.28 8 3c0-.28-.11-.53-.29-.71l-2-2a1.003 1.003 0 0 0-1.42 1.42l.3.29H3C1.34 2 0 3.34 0 5v3c0 .55.45 1 1 1s1-.45 1-1V5c0-.55.45-1 1-1h1.59l-.3.29a1.003 1.003 0 0 0 1.42 1.42zM12.5 11c.28 0 .5-.22.5-.5s-.22-.5-.5-.5-.5.22-.5.5.22.5.5.5zM15 7H6c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-1 6.33L12 12l-1 1-2-3-2 2.67V9h7v4.33z"],import:["M7.29 11.71c.18.18.43.29.71.29s.53-.11.71-.29l4-4a1.003 1.003 0 0 0-1.42-1.42L9 8.59V1c0-.55-.45-1-1-1S7 .45 7 1v7.59l-2.29-2.3a1.003 1.003 0 0 0-1.42 1.42l4 4zM15 11c-.55 0-1 .45-1 1v2H2v-2c0-.55-.45-1-1-1s-1 .45-1 1v3c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1z"],inbox:["M13.91 2.6c-.16-.36-.51-.61-.92-.61h-10c-.41 0-.77.25-.92.61L-.01 7.45v5.54c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V7.45L13.91 2.6zm-1.92 5.39c-.55 0-1 .45-1 1v1h-6v-1c0-.55-.45-1-1-1H1.94l1.71-4h8.68l1.71 4h-2.05z"],"inbox-filtered":["M6.432 2c.094.14.202.273.324.394L8.42 4H3.66L1.95 8H4c.55 0 1 .45 1 1v1h6.557c.693 0 1.363-.262 1.837-.736l.103-.102.85-1.14a2.564 2.564 0 0 0 .623-1.682V5.058L16 7.46V13c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V7.46l2.08-4.85C2.23 2.25 2.59 2 3 2h3.432zm9.048-2c.31 0 .52.26.52.57 0 .16-.06.3-.17.41l-2.86 2.73v2.63c0 .16-.06.3-.17.41l-.82 1.1c-.1.1-.25.17-.41.17-.31 0-.57-.26-.57-.57V3.71L8.17.98A.566.566 0 0 1 8 .57c0-.31.26-.57.57-.57h6.91z"],"inbox-geo":["M6.341 2A5.99 5.99 0 0 0 6 4H3.66L1.95 8H4c.55 0 1 .45 1 1v1h7a5.978 5.978 0 0 0 4-1.528V13c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V7.46l2.08-4.85C2.23 2.25 2.59 2 3 2h3.341zm3.679 2.145c0-.125.075-.23.205-.225h.345l.79.8c.005 0 0 .005 0 .005v.295c0 .13-.085.23-.215.23h-.07v.15c0 .13-.09.2-.215.2v.535c0 .125-.12.225-.245.225s-.245-.1-.245-.225V5.25h-.145c-.125 0-.205-.1-.205-.23v-.875zm2.235-2.195c-.03 0-.055-.005-.06-.035 0-.03.03-.035.06-.035h.11c.035 0 .06.005.06.035 0 .03-.03.035-.06.035h-.11zm-1.165-.025a.094.094 0 0 1-.13 0l-.25-.25a.094.094 0 0 1 0-.13.094.094 0 0 1 .13 0l.25.25a.094.094 0 0 1 0 .13zm1.53.445c-.035 0-.07-.025-.07-.06v-.155c0-.03.035-.06.07-.06s.07.025.07.06v.155c0 .03-.035.06-.07.06zM12 0c2.21 0 4 1.79 4 4s-1.79 4-4 4-4-1.79-4-4 1.79-4 4-4zm0 7c1.655 0 3-1.345 3-3 0-.195-.02-.39-.06-.575h-.21c-.125 0-.225-.07-.23-.21h-.215c.075.07.155.14.155.23V3.9c0 .06-.04.115-.075.155h-.015l-.01.005-.015.01-.445.43v.815c0 .13-.07.22-.2.22h-.36c-.125 0-.21-.09-.21-.22v-.84a.627.627 0 0 0-.36-.345h-.45c-.13 0-.245-.095-.245-.225v-.46c0-.125.115-.23.245-.23l.13-.005.21-.215c.09-.09.24-.09.33 0l.22.225h.47c.105 0 .155-.105.225-.21v-.075h-.205a.106.106 0 0 1-.105-.105.11.11 0 0 1 .105-.105h.18l.025.005v-.12s-.03-.025-.04-.025h-.2l-.045.05a.235.235 0 0 1-.33.005l-.045-.06h-.115l.16.175c.015.015.015.06 0 .075-.02.015-.045.02-.06.005l-.195-.185h-.085l-.245.23-.02-.005c-.025.07-.06.055-.095.055-.085 0-.15-.045-.15-.13s.065-.14.15-.14h.115v-.125c0-.06.04-.09.1-.09h.05V2.36c0-.095.095-.2.19-.2h.19c.105 0 .18-.075.18-.185V1.94c0-.015.035.01.035-.06h-.125l-.005.01-.21.22a.085.085 0 0 1-.115 0 .085.085 0 0 1 0-.115l.255-.255c.02-.015.045-.015.065-.015.005 0 .005.015.005.015h.64a2.327 2.327 0 0 0-.355-.275 2.452 2.452 0 0 0-.355-.195c-.03-.015-.055-.03-.08-.04a3.025 3.025 0 0 0-1.735-.175c-.09.02-.175.055-.265.08-.09.03-.18.05-.265.085-.075.03-.145.07-.22.105-.115.06-.235.115-.34.185l-.005.005c-.1.065-.18.145-.27.22h.455c.06 0 .11.045.11.105s-.05.105-.11.105h-.32c0 .07-.025.04-.025.045v.24h.285l.15-.17c.09-.09.235-.105.325-.015.09.09.09.23 0 .32l-.795.79-.01.01c-.005 0-.005 0-.01.005l-.025.015h-.01a.235.235 0 0 1-.12.025h-.23c-.08.07-.125.1-.125.18v.06c0 .01-.02.02-.02.03l.375.39c.04.04.04.1 0 .14-.04.04-.1.04-.14 0l-.39-.385a.213.213 0 0 1-.055.01c-.105 0-.195-.085-.195-.185v-.235h-.055A3.1 3.1 0 0 0 9 4c0 1.655 1.345 3 3 3zm2.27-2.135c.05 0 .105.04.105.09v.285c0 .05-.055.09-.105.09-.05 0-.105-.04-.105-.09v-.285c0-.05.055-.09.105-.09zm-2.085-3.27c0 .13-.105.21-.225.21h-.25v.07c0 .075-.03.135-.105.135s-.105-.06-.105-.135V1.64c-.075-.02-.025-.025-.025-.04 0-.125.085-.215.21-.215h.27c.13 0 .23.085.23.21z"],"inbox-search":["M5.639 2a5.391 5.391 0 0 0-.144 2H3.66L1.95 8H4c.55 0 1 .45 1 1v1h6V9c0-.088.012-.174.033-.255.12-.007.238-.019.39-.038.154-.008.252-.03.442-.077a5.34 5.34 0 0 0 .24-.05h.05l.122-.04 1.266 1.271c.425.47 1.116.769 1.847.769.21 0 .414-.025.61-.071V13c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V7.46l2.08-4.85C2.23 2.25 2.59 2 3 2h2.639zM15.82 7.53c.1.12.17.27.18.44 0 .34-.27.61-.61.61a.57.57 0 0 1-.43-.18l-2.24-2.25c-.13.08-.26.16-.4.23-.02.01-.05.02-.07.03-.14.06-.27.12-.42.17h-.01c-.14.05-.29.08-.44.11-.04.01-.08.02-.11.02-.15.02-.3.04-.46.04-1.85 0-3.35-1.51-3.35-3.37S8.96.01 10.81 0c1.85 0 3.35 1.51 3.35 3.37 0 .16-.02.31-.04.47-.01.04-.01.07-.02.11-.02.15-.05.29-.1.44v.01c-.05.15-.11.28-.17.42-.01.02-.02.05-.03.07-.07.14-.14.27-.23.4l2.25 2.24zm-5.01-1.94c1.22 0 2.21-.99 2.21-2.22 0-1.23-.99-2.22-2.21-2.22S8.6 2.14 8.6 3.37c0 1.22.99 2.22 2.21 2.22z"],"inbox-update":["M8.1 2a5.023 5.023 0 0 0 0 2H3.66L1.95 8H4c.55 0 1 .45 1 1v1h6V9c0-.55.45-1 1-1h2.05c.708 0 1.352-.241 1.905-.645L16 7.46V13c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V7.46l2.08-4.85C2.23 2.25 2.59 2 3 2h5.1zM13 6a3 3 0 1 1 0-6 3 3 0 0 1 0 6z"],"info-sign":["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zM7 3h2v2H7V3zm3 10H6v-1h1V7H6V6h3v6h1v1z"],inheritance:["M5 8c0 1.66 1.34 3 3 3h4.59L11.3 9.71A.965.965 0 0 1 11 9a1.003 1.003 0 0 1 1.71-.71l3 3c.18.18.29.43.29.71 0 .28-.11.53-.29.71l-3 3a1.003 1.003 0 0 1-1.42-1.42l1.3-1.29H8c-2.76 0-5-2.24-5-5H1a1 1 0 0 1-1-1V1a1 1 0 0 1 1-1h6a1 1 0 0 1 1 1v6a1 1 0 0 1-1 1H5zM2 2v4h4V2H2z"],"inner-join":["M6.6 3.3C5.3 4.4 4.5 6.1 4.5 8s.8 3.6 2.1 4.7c-.5.2-1 .3-1.6.3-2.8 0-5-2.2-5-5s2.2-5 5-5c.6 0 1.1.1 1.6.3zm-1.96 8.68C3.92 10.83 3.5 9.46 3.5 8s.42-2.83 1.14-3.98C2.6 4.2 1 5.91 1 8s1.6 3.8 3.64 3.98zM8 4c-1.2.9-2 2.4-2 4s.8 3.1 2 4c1.2-.9 2-2.3 2-4s-.8-3.1-2-4zm3-1c2.8 0 5 2.2 5 5s-2.2 5-5 5c-.6 0-1.1-.1-1.6-.3 1.3-1.1 2.1-2.9 2.1-4.7s-.8-3.5-2.1-4.7c.5-.2 1-.3 1.6-.3zm.35 1.02c.73 1.15 1.14 2.52 1.14 3.98s-.42 2.83-1.14 3.98c2.04-.18 3.64-1.9 3.64-3.98s-1.6-3.8-3.64-3.98z"],insert:["M5 9h2v2c0 .6.4 1 1 1s1-.4 1-1V9h2c.6 0 1-.4 1-1s-.4-1-1-1H9V5c0-.6-.4-1-1-1s-1 .4-1 1v2H5c-.6 0-1 .4-1 1s.4 1 1 1zm10-9H1C.4 0 0 .4 0 1v14c0 .6.4 1 1 1h14c.6 0 1-.4 1-1V1c0-.6-.4-1-1-1zm-1 14H2V2h12v12z"],intersection:["M10 3c-.92 0-1.76.26-2.5.69C6.76 3.26 5.92 3 5 3 2.24 3 0 5.24 0 8s2.24 5 5 5c.92 0 1.76-.26 2.5-.69.74.43 1.58.69 2.5.69 2.76 0 5-2.24 5-5s-2.24-5-5-5zm-4.1 7.85c-.29.09-.59.15-.9.15-1.66 0-3-1.34-3-3s1.34-3 3-3c.31 0 .61.06.9.15C5.33 5.96 5 6.94 5 8s.33 2.04.9 2.85zM10 11c-.31 0-.61-.06-.9-.15.57-.81.9-1.79.9-2.85s-.33-2.04-.9-2.85c.29-.09.59-.15.9-.15 1.66 0 3 1.34 3 3s-1.34 3-3 3z"],"ip-address":["M5 2.66C5 4.14 8 8 8 8s3-3.86 3-5.34C10.99 1.2 9.66 0 8 0S5 1.2 5 2.66zM7 3c0-.55.45-1 1-1s1 .45 1 1-.45 1-1 1-1-.45-1-1zM10.5 10H8v5h1v-4h1v1H9v1h2v-3h-.5zM2 9h12c.55 0 1 .45 1 1v5c0 .55-.45 1-1 1H2c-.55 0-1-.45-1-1v-5c0-.55.45-1 1-1zm4 1v5h1v-5H6z"],issue:["M8 16A8 8 0 1 1 8 0a8 8 0 0 1 0 16zm0-2A6 6 0 1 0 8 2a6 6 0 0 0 0 12zm1-2H7v-2h2v2zm0-3H7V4h2v5z"],"issue-closed":["M9.296.104a2.99 2.99 0 0 0-1.003.664 2.987 2.987 0 0 0-.75 1.25 6 6 0 1 0 6.28 4.527c.043-.039.085-.079.127-.12l1.456-1.456A8 8 0 1 1 9.296.105zm2.532 5.2a.997.997 0 0 1-.707-.294L9.707 3.596a1 1 0 0 1 1.414-1.414l.707.707 1.768-1.768a1 1 0 1 1 1.414 1.415L12.536 5.01a.997.997 0 0 1-.708.293zM9 12H7v-2h2v2zm0-3H7V4h2v5z"],"issue-new":["M10.568.421c-.01.04-.018.08-.026.121-.837.156-1.53.73-1.85 1.497a6 6 0 1 0 5.27 5.273 2.51 2.51 0 0 0 1.496-1.854c.04-.008.081-.016.121-.026A8 8 0 1 1 10.568.421zM9 12H7v-2h2v2zm0-3H7V4h2v5zm1-6c0-.55.45-1 1-1h1V1c0-.55.45-1 1-1s1 .45 1 1v1h1c.55 0 1 .45 1 1s-.45 1-1 1h-1v1.005c0 .55-.45 1-1 1s-1-.45-1-1V4h-1c-.55 0-1-.45-1-1z"],italic:["M9.8 4H11c.5 0 1-.4 1-1s-.4-1-1-1H7c-.5 0-1 .4-1 1s.4 1 1 1h.8l-1.6 8H5c-.5 0-1 .4-1 1s.4 1 1 1h4c.5 0 1-.4 1-1s-.4-1-1-1h-.8l1.6-8z"],"join-table":["M15 5h-3V2c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h3v3c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm-5-1v2H6V4h4zm0 6H6V7h4v3zM2 4h3v2H2V4zm0 5V7h3v2H2zm4 4v-2h4v2H6zm8 0h-3v-2h3v2zm0-3h-3V8h3v2z"],key:["M11 0C8.24 0 6 2.24 6 5c0 1.02.31 1.96.83 2.75L.29 14.29a1.003 1.003 0 0 0 1.42 1.42L3 14.41l1.29 1.29c.18.19.43.3.71.3s.53-.11.71-.29l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71L6.41 11l1.83-1.83c.8.52 1.74.83 2.76.83 2.76 0 5-2.24 5-5s-2.24-5-5-5zm0 8c-.23 0-.45-.03-.66-.08-.01 0-.02-.01-.03-.01-.21-.05-.41-.12-.6-.21a3.014 3.014 0 0 1-1.62-2c0-.01-.01-.02-.01-.03C8.03 5.45 8 5.23 8 5c0-1.66 1.34-3 3-3s3 1.34 3 3-1.34 3-3 3z"],"key-backspace":["M15 2H6c-.28 0-.53.11-.71.29l-5 5C.11 7.47 0 7.72 0 8c0 .28.11.53.29.71l5 5c.18.18.43.29.71.29h9c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm-2.29 7.29a1.003 1.003 0 0 1-1.42 1.42L10 9.41 8.71 10.7c-.18.19-.43.3-.71.3a1.003 1.003 0 0 1-.71-1.71L8.59 8l-1.3-1.29a1.003 1.003 0 0 1 1.42-1.42L10 6.59l1.29-1.29c.18-.19.43-.3.71-.3a1.003 1.003 0 0 1 .71 1.71L11.41 8l1.3 1.29z"],"key-command":["M12 9h-1V7h1c1.66 0 3-1.34 3-3s-1.34-3-3-3-3 1.34-3 3v1H7V4c0-1.66-1.34-3-3-3S1 2.34 1 4s1.34 3 3 3h1v2H4c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3v-1h2v1c0 1.66 1.34 3 3 3s3-1.34 3-3-1.34-3-3-3zm0-6c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zM4 13c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm0-8c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm5 4H7V7h2v2zm3 4c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"key-control":["M12.71 5.29l-4-4C8.53 1.11 8.28 1 8 1s-.53.11-.71.29l-4 4a1.003 1.003 0 0 0 1.42 1.42L8 3.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71z"],"key-delete":["M15.71 7.29l-5-5A.997.997 0 0 0 10 2H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h9c.28 0 .53-.11.71-.29l5-5c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zm-7 2a1.003 1.003 0 0 1-1.42 1.42L6 9.41 4.71 10.7c-.18.19-.43.3-.71.3a1.003 1.003 0 0 1-.71-1.71L4.59 8l-1.3-1.29a1.003 1.003 0 0 1 1.42-1.42L6 6.59 7.29 5.3c.18-.19.43-.3.71-.3a1.003 1.003 0 0 1 .71 1.71L7.41 8l1.3 1.29z"],"key-enter":["M14 2c-.55 0-1 .45-1 1v3c0 1.66-1.34 3-3 3H4.41L5.7 7.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-3 3c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l3 3a1.003 1.003 0 0 0 1.42-1.42L4.41 11H10c2.76 0 5-2.24 5-5V3c0-.55-.45-1-1-1z"],"key-escape":["M2 7c.55 0 1-.45 1-1V4.41L7.29 8.7c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L4.41 3H6c.55 0 1-.45 1-1s-.45-1-1-1H2c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1zm7-5.9v2A5 5 0 1 1 3.1 9h-2c.49 3.39 3.38 6 6.9 6 3.87 0 7-3.13 7-7 0-3.52-2.61-6.41-6-6.9z"],"key-option":["M11 4h4c.55 0 1-.45 1-1s-.45-1-1-1h-4c-.55 0-1 .45-1 1s.45 1 1 1zm4 8h-3.43L5.86 2.49h-.02A.975.975 0 0 0 5 2H1c-.55 0-1 .45-1 1s.45 1 1 1h3.43l5.71 9.51.01-.01c.18.3.49.5.85.5h4c.55 0 1-.45 1-1s-.45-1-1-1z"],"key-shift":["M13.71 7.29l-5-5C8.53 2.11 8.28 2 8 2s-.53.11-.71.29l-5 5A1.003 1.003 0 0 0 3 9h2v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V9h2a1.003 1.003 0 0 0 .71-1.71z"],"key-tab":["M15 10H4.41L5.7 8.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L2 9.59V8c0-.55-.45-1-1-1s-1 .45-1 1v6c0 .55.45 1 1 1s1-.45 1-1v-1.59l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L4.41 12H15c.55 0 1-.45 1-1s-.45-1-1-1zm0-9c-.55 0-1 .45-1 1v1.59L11.71 1.3A.965.965 0 0 0 11 1a1.003 1.003 0 0 0-.71 1.71L11.59 4H1c-.55 0-1 .45-1 1s.45 1 1 1h10.59L10.3 7.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L14 6.41V8c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1z"],"known-vehicle":["M15 3a.997.997 0 0 0-.707.293L12 5.586l-1.293-1.293a1 1 0 1 0-1.414 1.414l2 2a.997.997 0 0 0 1.414 0l3-3A1 1 0 0 0 15 3zm-.879 6.121l-.007-.007c-.313.309-.69.552-1.114.702V10h-.998H12h-1v-.184c-.424-.15-.8-.395-1.112-.704l-.01.01-2-2 .012-.012A2.978 2.978 0 0 1 7.184 6H3c-.176 0-.06-.824 0-1l.73-1.63C3.79 3.192 3.823 3 4 3H7.78C8.328 2.39 9.115 2 10 2c.768 0 1.461.293 1.987.77l.844-.844c-.238-.244-.524-.442-.794-.524C12.037 1.402 10.72 1 8 1c-2.72 0-4.037.402-4.037.402-.508.155-1.078.711-1.268 1.237l-.763 2.117H.88c-.484 0-.88.423-.88.939s.396.939.88.939h.375L1 7c-.034.685 0 1.436 0 2v5c0 .657.384 1 1 1s1-.343 1-1v-1h10v1c0 .657.384 1 1 1s1-.343 1-1V9l-.003-.754-.876.875zM5.001 10H3V8h2v2z"],label:["M11 2H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V7l-5-5zm3 10H2V4h8v2H3v1h7v1h4v4zm-3-5V4l3 3h-3zm-8 3h10V9H3v1z"],layer:["M16 8c0-.37-.21-.68-.51-.85l.01-.01-7-4-.01.01C8.34 3.06 8.18 3 8 3s-.34.06-.49.15l-.01-.02-7 4 .01.01C.21 7.32 0 7.63 0 8s.21.68.51.85l-.01.01 7 4 .01-.01c.15.09.31.15.49.15s.34-.06.49-.15l.01.01 7-4-.01-.01c.3-.17.51-.48.51-.85z"],layers:["M.55 4.89l7 3.5c.14.07.29.11.45.11s.31-.04.45-.11l7-3.5a.998.998 0 0 0-.06-1.81L8.4.08a1.006 1.006 0 0 0-.79 0l-6.99 3a.992.992 0 0 0-.07 1.81zM15 10c-.16 0-.31.04-.45.11L8 13.38 1.45 10.1c-.14-.06-.29-.1-.45-.1-.55 0-1 .45-1 1 0 .39.23.73.55.89l7 3.5c.14.07.29.11.45.11s.31-.04.45-.11l7-3.5c.32-.16.55-.5.55-.89 0-.55-.45-1-1-1zm0-3.5c-.16 0-.31.04-.45.11L8 9.88 1.45 6.61A.997.997 0 0 0 1 6.5c-.55 0-1 .45-1 1 0 .39.23.73.55.89l7 3.5c.14.07.29.11.45.11s.31-.04.45-.11l7-3.5c.32-.16.55-.5.55-.89 0-.55-.45-1-1-1z"],layout:["M14 4c-1.1 0-2 .9-2 2 0 .47.17.9.44 1.24l-.68.91A1.996 1.996 0 0 0 9.07 9.5H7.93C7.71 8.64 6.93 8 6 8c-.47 0-.9.17-1.24.44l-.91-.68c.1-.23.15-.49.15-.76 0-.37-.11-.71-.28-1.01l2.27-2.27c.3.17.64.28 1.01.28 1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .37.11.71.28 1.01L3.01 5.28C2.71 5.11 2.37 5 2 5 .9 5 0 5.9 0 7s.9 2 2 2c.47 0 .9-.17 1.24-.44l.91.68c-.1.23-.15.49-.15.76 0 .37.11.71.28 1.01l-1.27 1.27C2.71 12.11 2.37 12 2 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2c0-.37-.11-.71-.28-1.01l1.27-1.27c.3.17.64.28 1.01.28.93 0 1.71-.64 1.93-1.5h1.14c.22.86 1 1.5 1.93 1.5 1.1 0 2-.9 2-2 0-.47-.17-.9-.44-1.24l.68-.91c.23.1.49.15.76.15 1.1 0 2-.9 2-2s-.9-2-2-2z"],"layout-auto":["M14 9.5c-.56 0-1.06.23-1.42.59L8.99 8l3.59-2.09A2.002 2.002 0 0 0 16 4.5c0-1.1-.9-2-2-2s-2 .9-2 2c0 .19.03.37.08.54L8.5 7.13v-3.2c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2S6 .9 6 2c0 .93.64 1.71 1.5 1.93v3.2L3.92 5.04c.05-.17.08-.35.08-.54 0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.56 0 1.06-.23 1.42-.59L7.01 8l-3.59 2.09A2.002 2.002 0 0 0 0 11.5c0 1.1.9 2 2 2s2-.9 2-2c0-.19-.03-.37-.08-.54L7.5 8.87v3.2c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93v-3.2l3.58 2.09c-.05.17-.08.35-.08.54 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2z"],"layout-balloon":["M14 11c-.2 0-.38.04-.56.09L12.42 9.4c.36-.36.58-.85.58-1.4 0-.55-.22-1.04-.58-1.4l1.01-1.69c.19.05.37.09.57.09 1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .55.22 1.04.58 1.4l-1.01 1.69C11.38 6.04 11.2 6 11 6c-.93 0-1.71.64-1.93 1.5H6.93C6.71 6.64 5.93 6 5 6c-.2 0-.38.04-.56.09L3.42 4.4C3.78 4.04 4 3.55 4 3c0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.2 0 .38-.04.56-.09L3.58 6.6C3.22 6.96 3 7.45 3 8c0 .55.22 1.04.58 1.4l-1.01 1.69C2.38 11.04 2.2 11 2 11c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2c0-.55-.22-1.04-.58-1.4l1.01-1.69c.19.05.37.09.57.09.93 0 1.71-.64 1.93-1.5h2.14c.22.86 1 1.5 1.93 1.5.2 0 .38-.04.56-.09l1.01 1.69c-.35.36-.57.85-.57 1.4 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2z"],"layout-circle":["M14.16 6.02c-.12-.36-.26-.7-.43-1.03.17-.29.27-.63.27-.99 0-1.1-.9-2-2-2-.36 0-.7.1-.99.27-.33-.17-.67-.31-1.03-.43A1.987 1.987 0 0 0 8 0C6.95 0 6.1.81 6.02 1.84c-.36.12-.7.26-1.03.43C4.7 2.1 4.36 2 4 2c-1.1 0-2 .9-2 2 0 .36.1.7.27.99-.17.33-.31.67-.43 1.03C.81 6.1 0 6.95 0 8c0 1.05.81 1.9 1.84 1.98.12.36.26.7.43 1.03-.17.29-.27.63-.27.99 0 1.1.9 2 2 2 .36 0 .7-.1.99-.27.33.17.67.32 1.03.43C6.1 15.19 6.95 16 8 16c1.05 0 1.9-.81 1.98-1.84.36-.12.7-.26 1.03-.43.29.17.63.27.99.27 1.1 0 2-.9 2-2 0-.36-.1-.7-.27-.99.17-.33.31-.67.43-1.03C15.19 9.9 16 9.05 16 8c0-1.05-.81-1.9-1.84-1.98zm-.99 3.79c-.05.16-.11.31-.17.46-.3-.17-.64-.27-1-.27-1.1 0-2 .9-2 2 0 .36.1.7.27 1-.15.07-.3.12-.46.17C9.5 12.48 8.81 12 8 12s-1.5.48-1.81 1.17c-.16-.06-.32-.11-.46-.17.17-.3.27-.64.27-1 0-1.1-.9-2-2-2-.36 0-.7.1-1 .27-.07-.15-.12-.3-.17-.46C3.52 9.5 4 8.81 4 8s-.48-1.5-1.17-1.81c.06-.16.11-.32.17-.46.3.17.64.27 1 .27 1.1 0 2-.9 2-2 0-.36-.1-.7-.27-1 .15-.07.3-.12.46-.17C6.5 3.52 7.19 4 8 4s1.5-.48 1.81-1.17c.16.06.32.11.46.17-.17.3-.27.64-.27 1 0 1.1.9 2 2 2 .36 0 .7-.1 1-.27.07.15.12.3.17.46C12.48 6.5 12 7.19 12 8s.48 1.5 1.17 1.81z"],"layout-grid":["M2 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0-6C.9 6 0 6.9 0 8s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0-6C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12 4c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm0 2c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM8 0C6.9 0 6 .9 6 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm6 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM8 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],"layout-group-by":["M2 6C.9 6 0 6.9 0 8s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0 5c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12-7c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zM2 1C.9 1 0 1.9 0 3s.9 2 2 2 2-.9 2-2-.9-2-2-2zm7 8c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0-6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm5 3c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],"layout-hierarchy":["M14.5 12.07V9.93c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2-.93 0-1.71.64-1.93 1.5H9.93c-.18-.7-.73-1.25-1.43-1.43V3.93c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2S6 .9 6 2c0 .93.64 1.71 1.5 1.93v2.14c-.7.18-1.25.73-1.43 1.43H3.93C3.71 6.64 2.93 6 2 6 .9 6 0 6.9 0 8c0 .93.64 1.71 1.5 1.93v2.14c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93V9.93c.7-.18 1.25-.73 1.43-1.43h2.14c.18.7.73 1.25 1.43 1.43v2.14c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93V9.93c.7-.18 1.25-.73 1.43-1.43h2.14c.18.7.73 1.25 1.43 1.43v2.14c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93z"],"layout-linear":["M14 6c-.93 0-1.71.64-1.93 1.5H9.93C9.71 6.64 8.93 6 8 6s-1.71.64-1.93 1.5H3.93C3.71 6.64 2.93 6 2 6 .9 6 0 6.9 0 8s.9 2 2 2c.93 0 1.71-.64 1.93-1.5h2.13C6.29 9.36 7.07 10 8 10s1.71-.64 1.93-1.5h2.13c.22.86 1 1.5 1.93 1.5 1.1 0 2-.9 2-2C16 6.9 15.1 6 14 6z"],"layout-skew-grid":["M2 6C.9 6 0 6.9 0 8s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12-2c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zM2 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM2 0C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm6 9c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm6-3c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM8 3c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm6 9c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],"layout-sorted-clusters":["M2 6C.9 6 0 6.9 0 8s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM2 0C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM8 9c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0-6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],"left-join":["M6.6 3.3C6.1 3.1 5.6 3 5 3 2.2 3 0 5.2 0 8s2.2 5 5 5c.6 0 1.1-.1 1.6-.3C5.3 11.6 4.5 9.9 4.5 8s.8-3.6 2.1-4.7zM8 4c-1.2.9-2 2.4-2 4s.8 3.1 2 4c1.2-.9 2-2.3 2-4s-.8-3.1-2-4zm3-1c2.8 0 5 2.2 5 5s-2.2 5-5 5c-.6 0-1.1-.1-1.6-.3 1.3-1.1 2.1-2.9 2.1-4.7s-.8-3.5-2.1-4.7c.5-.2 1-.3 1.6-.3zm.35 1.02c.73 1.15 1.14 2.52 1.14 3.98s-.42 2.83-1.14 3.98c2.04-.18 3.64-1.9 3.64-3.98s-1.6-3.8-3.64-3.98z"],"less-than":["M13.287 5.958a1 1 0 0 0-.574-1.916l-10 3c-.95.285-.95 1.631 0 1.916l10 3a1 1 0 0 0 .574-1.916L6.48 8l6.807-2.042z"],"less-than-or-equal-to":["M13.287 3.958a1 1 0 0 0-.575-1.916l-10 3c-.95.285-.95 1.63 0 1.916l10 3a1 1 0 0 0 .575-1.916L6.48 6l6.807-2.042zM13 12H3a1 1 0 0 0 0 2h10a1 1 0 0 0 0-2z"],lifesaver:["M9.405 11.746C8.968 11.91 8.495 12 8 12c-.494 0-.968-.09-1.405-.254l-.702 1.873C6.548 13.865 7.258 14 8 14c.742 0 1.452-.135 2.107-.38l-.702-1.874zm2.341-2.341l1.873.702C13.865 9.452 14 8.742 14 8c0-.742-.135-1.452-.38-2.107l-1.874.702c.164.437.254.91.254 1.405 0 .494-.09.968-.254 1.405zM9.405 4.254l.702-1.873A5.987 5.987 0 0 0 8 2c-.742 0-1.452.135-2.107.38l.702 1.874C7.032 4.09 7.505 4 8 4c.494 0 .968.09 1.405.254zM4.254 6.595L2.38 5.893A5.987 5.987 0 0 0 2 8c0 .742.135 1.452.38 2.107l1.874-.702A3.991 3.991 0 0 1 4 8c0-.494.09-.968.254-1.405zM8 16A8 8 0 1 1 8 0a8 8 0 0 1 0 16zm0-6a2 2 0 1 0 0-4 2 2 0 0 0 0 4z"],lightbulb:["M9.01 14h-2c-.55 0-1 .45-1 1s.45 1 1 1h2c.55 0 1-.45 1-1s-.44-1-1-1zm1-3h-4c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1s-.44-1-1-1zm-2-11C5.26 0 3.03 1.95 3.03 4.35c0 2.37 1.63 2.64 1.94 5.22 0 .24.22.44.5.44h5.09c.28 0 .5-.19.5-.44C11.37 6.99 13 6.72 13 4.35 13 1.95 10.77 0 8.01 0z"],link:["M4.99 11.99c.28 0 .53-.11.71-.29l6-6a1.003 1.003 0 0 0-1.42-1.42l-6 6a1.003 1.003 0 0 0 .71 1.71zm3.85-2.02L6.4 12.41l-1 1-.01-.01c-.36.36-.85.6-1.4.6-1.1 0-2-.9-2-2 0-.55.24-1.04.6-1.4l-.01-.01 1-1 2.44-2.44c-.33-.1-.67-.16-1.03-.16-1.1 0-2.09.46-2.81 1.19l-.02-.02-1 1 .02.02c-.73.72-1.19 1.71-1.19 2.81 0 2.21 1.79 4 4 4 1.1 0 2.09-.46 2.81-1.19l.02.02 1-1-.02-.02c.73-.72 1.19-1.71 1.19-2.81 0-.35-.06-.69-.15-1.02zm7.15-5.98c0-2.21-1.79-4-4-4-1.1 0-2.09.46-2.81 1.19l-.02-.02-1 1 .02.02c-.72.72-1.19 1.71-1.19 2.81 0 .36.06.69.15 1.02l2.44-2.44 1-1 .01.01c.36-.36.85-.6 1.4-.6 1.1 0 2 .9 2 2 0 .55-.24 1.04-.6 1.4l.01.01-1 1-2.43 2.45c.33.09.67.15 1.02.15 1.1 0 2.09-.46 2.81-1.19l.02.02 1-1-.02-.02a3.92 3.92 0 0 0 1.19-2.81z"],list:["M1 3h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm14 10H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm0-4H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm0-4H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"list-columns":["M6 1c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1s.45-1 1-1h5zm0 4c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1s.45-1 1-1h5zm0 4c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1s.45-1 1-1h5zm0 4c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1s.45-1 1-1h5zm9-12c.55 0 1 .45 1 1s-.45 1-1 1h-5c-.55 0-1-.45-1-1s.45-1 1-1h5zm0 4c.55 0 1 .45 1 1s-.45 1-1 1h-5c-.55 0-1-.45-1-1s.45-1 1-1h5zm0 4c.55 0 1 .45 1 1s-.45 1-1 1h-5c-.55 0-1-.45-1-1s.45-1 1-1h5zm0 4c.55 0 1 .45 1 1s-.45 1-1 1h-5c-.55 0-1-.45-1-1s.45-1 1-1h5z"],"list-detail-view":["M6 9H1c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm0 4H1c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm9-12h-5c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM6 5H1c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm0-4H1c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1z"],locate:["M15 7h-.09A6.98 6.98 0 0 0 9 1.1V1c0-.55-.45-1-1-1S7 .45 7 1v.09A6.98 6.98 0 0 0 1.1 7H1c-.55 0-1 .45-1 1s.45 1 1 1h.1A6.969 6.969 0 0 0 7 14.91V15c0 .55.45 1 1 1s1-.45 1-1v-.09A6.98 6.98 0 0 0 14.9 9h.1c.55 0 1-.45 1-1s-.45-1-1-1zm-6.02 5.9c-.05-.5-.46-.9-.98-.9s-.93.4-.98.9A5.017 5.017 0 0 1 3.1 8.98c.5-.05.9-.46.9-.98s-.4-.93-.9-.98A5.017 5.017 0 0 1 7.02 3.1c.05.5.46.9.98.9s.93-.4.98-.9c1.97.39 3.52 1.95 3.92 3.92-.5.05-.9.46-.9.98s.4.93.9.98a5.017 5.017 0 0 1-3.92 3.92zM8 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],lock:["M13.96 7H12V3.95C12 1.77 10.21 0 8 0S4 1.77 4 3.95V7H1.96c-.55 0-.96.35-.96.9v6.91c0 .54.41 1.19.96 1.19h12c.55 0 1.04-.65 1.04-1.19V7.9c0-.55-.49-.9-1.04-.9zM6 7V3.95c0-1.09.9-1.97 2-1.97s2 .88 2 1.97V7H6z"],"log-in":["M11 8c0-.28-.11-.53-.29-.71l-3-3a1.003 1.003 0 0 0-1.42 1.42L7.59 7H1c-.55 0-1 .45-1 1s.45 1 1 1h6.59L6.3 10.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71zm4-8H9c-.55 0-1 .45-1 1s.45 1 1 1h5v12H9c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"log-out":["M7 14H2V2h5c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1zm8.71-6.71l-3-3a1.003 1.003 0 0 0-1.42 1.42L12.59 7H6c-.55 0-1 .45-1 1s.45 1 1 1h6.59l-1.29 1.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],manual:["M15.99 1.13c-.02-.41-.33-.77-.78-.87C12.26-.36 9.84.13 8 1.7 6.16.13 3.74-.36.78.26.33.35.03.72.01 1.13H0v12c0 .08 0 .17.02.26.12.51.65.82 1.19.71 2.63-.55 4.59-.04 6.01 1.57.02.03.06.04.08.06.02.02.03.04.05.06.04.03.09.04.13.07.05.03.09.05.14.07.11.04.23.07.35.07h.04c.12 0 .24-.03.35-.07.05-.02.09-.05.14-.07.04-.02.09-.04.13-.07.02-.02.03-.04.05-.06.03-.02.06-.03.08-.06 1.42-1.6 3.39-2.12 6.01-1.57.54.11 1.07-.21 1.19-.71.04-.09.04-.18.04-.26l-.01-12zM7 12.99c-1.4-.83-3.07-1.14-5-.93V1.96c2.11-.28 3.75.2 5 1.46v9.57zm7-.92c-1.93-.21-3.6.1-5 .93V3.42c1.25-1.26 2.89-1.74 5-1.46v10.11z"],"manually-entered-data":["M1 8h3.76l2-2H1c-.55 0-1 .45-1 1s.45 1 1 1zm14.49-4.01c.31-.32.51-.76.51-1.24C16 1.78 15.22 1 14.25 1c-.48 0-.92.2-1.24.51l-1.44 1.44 2.47 2.47 1.45-1.43zM1 4h7.76l2-2H1c-.55 0-1 .45-1 1s.45 1 1 1zm0 6c-.55 0-1 .45-1 1 0 .48.35.86.8.96L2.76 10H1zm9.95-6.43l-6.69 6.69 2.47 2.47 6.69-6.69-2.47-2.47zm4.25 2.47L13.24 8H15c.55 0 1-.45 1-1 0-.48-.35-.86-.8-.96zM2 15l3.86-1.39-2.46-2.44L2 15zm13-5h-3.76l-2 2H15c.55 0 1-.45 1-1s-.45-1-1-1z"],map:["M15.55 3.17l-4.49-3A.975.975 0 0 0 9.99.15L5.53 2.82 1.56.17A1.003 1.003 0 0 0 0 1v11c0 .35.18.65.45.83l4.49 3a.975.975 0 0 0 1.07.02l4.46-2.67 3.97 2.65A1.003 1.003 0 0 0 16 15V4c0-.35-.18-.65-.45-.83zM5 13.46l-3-2v-8.6l2.94 1.96c.02.02.04.03.06.04v8.6zm5-2.32s-.01 0-.01.01L6 13.53V4.86s.01 0 .01-.01L10 2.47v8.67zm4 1.99l-2.94-1.96c-.02-.01-.04-.02-.05-.03v-8.6l3 2v8.59z"],"map-create":["M14 6.82v6.32l-2.94-1.96c-.02-.01-.04-.02-.05-.03V6.22c-.08-.07-.15-.16-.22-.24-.28-.02-.54-.08-.79-.16v5.32s-.01 0-.01.01L6 13.53V4.86s.01 0 .01-.01l2.05-1.23C8.02 3.42 8 3.21 8 3c0-.98.47-1.84 1.2-2.39l-3.67 2.2L1.56.17A1.003 1.003 0 0 0 0 1v11c0 .35.18.65.45.83l4.49 3a.975.975 0 0 0 1.07.02l4.46-2.67 3.97 2.65A1.003 1.003 0 0 0 16 15V5.82c-.25.09-.52.14-.8.16-.33.36-.73.67-1.2.84zm-9 6.64l-3-2v-8.6l2.94 1.96c.02.02.04.03.06.04v8.6zM11 4h1v1c0 .55.45 1 1 1s1-.45 1-1V4h1c.55 0 1-.45 1-1s-.45-1-1-1h-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1z"],"map-marker":["M8.46 0C5.42 0 2.95 2.39 2.95 5.33 2.95 8.28 8.46 16 8.46 16s5.51-7.72 5.51-10.67C13.96 2.39 11.5 0 8.46 0zm0 8a2.5 2.5 0 0 1 0-5 2.5 2.5 0 0 1 0 5z"],maximize:["M5.99 8.99c-.28 0-.53.11-.71.29l-3.29 3.29v-1.59c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H3.41L6.7 10.7a1.003 1.003 0 0 0-.71-1.71zm9-9h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59l-3.3 3.3a.99.99 0 0 0-.29.7 1.003 1.003 0 0 0 1.71.71l3.29-3.29V5c0 .55.45 1 1 1s1-.45 1-1V1c0-.56-.45-1.01-1-1.01z"],media:["M11.99 6.99c.55 0 1-.45 1-1s-.45-1-1-1-1 .45-1 1 .45 1 1 1zm3-5h-14c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-10c0-.55-.45-1-1-1zm-1 9l-5-3-1 2-3-4-3 5v-7h12v7z"],menu:["M1 4h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm14 8H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm0-5H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"menu-closed":["M14.99 6.99h-9c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1zm-12-2c-.28 0-.53.11-.71.29l-2 2a1.014 1.014 0 0 0 0 1.42l2 2a1.003 1.003 0 0 0 1.71-.71v-4c0-.55-.45-1-1-1zm3-1h9c.55 0 1-.45 1-1s-.45-1-1-1h-9c-.55 0-1 .45-1 1s.45 1 1 1zm9 8h-9c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1z"],"menu-open":["M9.99 11.99h-9c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1zm0-5h-9c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1zm0-5h-9c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1zm5.71 5.3l-2-2a1.003 1.003 0 0 0-1.71.71v4a1.003 1.003 0 0 0 1.71.71l2-2c.18-.18.29-.43.29-.71s-.11-.53-.29-.71z"],"merge-columns":["M5.71 5.29a1.003 1.003 0 0 0-1.42 1.42l.3.29H2V2h3v1.51c.52.06.99.29 1.34.65l.66.66V1c0-.55-.45-1-1-1H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h5c.55 0 1-.45 1-1v-3.82l-.66.66c-.35.35-.82.59-1.34.65V14H2V9h2.59l-.3.29a1.003 1.003 0 0 0 1.42 1.42l2-2C7.89 8.53 8 8.28 8 8c0-.28-.11-.53-.29-.71l-2-2zM15 0h-5c-.55 0-1 .45-1 1v3.82l.66-.66c.35-.35.82-.59 1.34-.65V2h3v5h-2.59l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-2 2C8.11 7.47 8 7.72 8 8c0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42l-.3-.29H14v5h-3v-1.51c-.52-.06-.99-.29-1.34-.65L9 11.18V15c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"merge-links":["M8 7c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0 4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0-8c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm6 3c-.93 0-1.71.64-1.93 1.5H11V3c0-1.66-1.34-3-3-3S5 1.34 5 3v4.5H3.93C3.71 6.64 2.93 6 2 6 .9 6 0 6.9 0 8s.9 2 2 2c.93 0 1.71-.64 1.93-1.5H5V13c0 1.66 1.34 3 3 3s3-1.34 3-3V8.5h1.07c.22.86 1 1.5 1.93 1.5 1.1 0 2-.9 2-2s-.9-2-2-2zm-4 7c0 1.1-.9 2-2 2s-2-.9-2-2V3c0-1.1.9-2 2-2s2 .9 2 2v10z"],minimize:["M15.99.99a1.003 1.003 0 0 0-1.71-.71l-3.29 3.29V1.99c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H12.4l3.3-3.29c.18-.18.29-.43.29-.71zm-10 8h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59L.29 14.28a1.003 1.003 0 0 0 1.42 1.42L5 12.41V14c0 .55.45 1 1 1s1-.45 1-1v-4a1.02 1.02 0 0 0-1.01-1.01z"],minus:["M13 7H3c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1z"],"mobile-phone":["M12 0H4c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h8c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM8 15c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm3-3H5V3h6v9z"],"mobile-video":["M15 4c-.28 0-.53.11-.71.29L12 6.59V4c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V9.41l2.29 2.29c.18.19.43.3.71.3.55 0 1-.45 1-1V5c0-.55-.45-1-1-1z"],moon:["M15 11.38A7.835 7.835 0 0 1 7.85 16C3.51 16 0 12.49 0 8.15 0 4.97 1.89 2.23 4.62 1c-.45.99-.7 2.08-.7 3.23a7.85 7.85 0 0 0 7.85 7.85c1.15 0 2.24-.25 3.23-.7z"],more:["M2 6.03a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM14 6.03a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM8 6.03a2 2 0 1 0 0 4 2 2 0 1 0 0-4z"],mountain:["M16 13H3l6-9h1l2 2h1l3 7zm-2.5-3.5l-1-2.5h-1l-2-2-3 4.5L9 8l1 1 1-1 2.5 1.5zM5.94 7l-4.122 6H0l5-6h.94z"],move:["M15.71 7.29l-2-2a1.003 1.003 0 0 0-1.42 1.42l.3.29H9V3.41l.29.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-2-2C8.53.11 8.28 0 8 0s-.53.11-.71.29l-2 2a1.003 1.003 0 0 0 1.42 1.42l.29-.3V7H3.41l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-2 2C.11 7.47 0 7.72 0 8c0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42L3.41 9H7v3.59l-.29-.29A.965.965 0 0 0 6 12a1.003 1.003 0 0 0-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2a1.003 1.003 0 0 0-1.42-1.42l-.29.3V9h3.59l-.29.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],mugshot:["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14h-.15c-.03-.09-.04-.16-.08-.25-.34-.79-2.01-1.31-3.12-1.8-1.11-.49-.96-.79-1-1.2-.01-.06-.01-.12-.01-.18.38-.34.69-.8.89-1.33 0 0 .01-.03.01-.04.04-.12.08-.24.11-.36.25-.05.4-.33.46-.59.06-.1.18-.36.15-.65-.04-.37-.19-.55-.35-.62v-.06c0-.48-.04-1.16-.13-1.61-.02-.12-.05-.25-.08-.37-.16-.55-.51-1.05-.96-1.39C9.26 3.19 8.6 3 8 3c-.59 0-1.26.19-1.73.55-.45.35-.8.84-.96 1.39-.04.13-.06.25-.08.38-.09.45-.13 1.13-.13 1.61v.06c-.18.06-.33.24-.37.62-.03.29.09.54.15.65.06.26.21.54.47.59.03.12.07.25.11.36 0 .01.01.02.01.02v.01c.21.54.53 1.01.92 1.35 0 .05-.01.11-.01.16-.04.41.08.7-1.03 1.2-1.11.49-2.77 1.01-3.12 1.8-.04.09-.05.16-.08.25H2V2h12v12z"],"multi-select":["M12 3.98H4c-.55 0-1 .45-1 1v1h8v5h1c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1zm3-3H7c-.55 0-1 .45-1 1v1h8v5h1c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1zm-6 6H1c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1h8c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1zm-1 5H2v-3h6v3z"],music:["M15 0c-.07 0-.13.03-.19.04V.02l-10 2v.02C4.35 2.13 4 2.52 4 3v9.12c-.31-.07-.65-.12-1-.12-1.66 0-3 .9-3 2s1.34 2 3 2 3-.9 3-2V6.32l8-1.6v5.4c-.31-.07-.65-.12-1-.12-1.66 0-3 .9-3 2s1.34 2 3 2 3-.9 3-2V1c0-.55-.45-1-1-1z"],"new-grid-item":["M6 0H1C.45 0 0 .45 0 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm5 14c0-.55-.45-1-1-1s-1 .45-1 1v1c0 .55.45 1 1 1h1c.55 0 1-.45 1-1s-.45-1-1-1zM6 9H1c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1zm9 4c-.55 0-1 .45-1 1-.55 0-1 .45-1 1s.45 1 1 1h1c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm-4-4h-1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1s1-.45 1-1c.55 0 1-.45 1-1s-.45-1-1-1zm4-9h-5c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm0 9h-1c-.55 0-1 .45-1 1s.45 1 1 1c0 .55.45 1 1 1s1-.45 1-1v-1c0-.55-.45-1-1-1z"],"new-link":["M15 3h-1V2c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1V5h1c.55 0 1-.45 1-1s-.45-1-1-1zm-3.5 6a2.5 2.5 0 0 0-2.45 2h-4.1a2.5 2.5 0 1 0 0 1h4.1a2.5 2.5 0 1 0 2.45-3z"],"new-object":["M8 4c0 .6.4 1 1 1h2v2c0 .6.4 1 1 1s1-.4 1-1V5h2c.6 0 1-.4 1-1s-.4-1-1-1h-2V1c0-.6-.4-1-1-1s-1 .4-1 1v2H9c-.6 0-1 .5-1 1zm6.5 2.5V7c0 1.4-1.1 2.5-2.5 2.5S9.5 8.4 9.5 7v-.5H9C7.6 6.5 6.5 5.4 6.5 4S7.6 1.5 9 1.5h.5V1c0-.3.1-.6.1-.8C9.1.1 8.6 0 8 0 3.6 0 0 3.6 0 8s3.6 8 8 8 8-3.6 8-8c0-.6-.1-1.3-.2-1.9-.4.3-.8.4-1.3.4z"],"new-person":["M9.12 12.69c-1.17-.53-1.01-.85-1.05-1.29-.01-.06-.01-.12-.01-.19.4-.37.73-.87.94-1.44 0 0 .01-.03.01-.04.05-.14.09-.27.12-.4.27-.06.43-.36.49-.63.06-.11.19-.39.16-.7-.04-.41-.2-.6-.38-.68v-.07c0-.51-.05-1.25-.14-1.74-.02-.13-.05-.27-.09-.4-.17-.6-.53-1.14-1.01-1.52C7.66 3.2 6.96 3 6.33 3c-.62 0-1.33.2-1.82.59-.49.38-.85.92-1.02 1.52-.04.13-.07.26-.09.4-.09.49-.13 1.23-.13 1.74v.06c-.19.08-.35.27-.39.68-.03.31.1.59.16.7.06.28.22.59.5.64.03.14.07.27.11.4 0 .01.01.02.01.02v.01c.22.59.55 1.1.96 1.46 0 .06-.01.12-.01.17-.04.44.08.76-1.09 1.29-1.17.53-2.93 1.1-3.29 1.95-.35.87-.2 1.37-.2 1.37h12.6s.15-.5-.22-1.36c-.36-.85-2.12-1.42-3.29-1.95zM14.89 2h-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1V4h1c.55 0 1-.45 1-1s-.45-1-1-1z"],"new-prescription":["M9.82 11.66l2.48-2.87c.12-.2.13-.37.04-.53-.11-.19-.3-.26-.52-.26h-1.29c-.27 0-.49.13-.63.34L8.44 9.9 6.95 8a.482.482 0 0 0-.08-.1L5.82 6.55c.57-.24 1.04-.57 1.42-1.01.49-.57.74-1.27.74-2.08 0-.51-.1-.99-.32-1.42-.21-.43-.51-.8-.89-1.11A4.1 4.1 0 0 0 5.42.24C4.91.08 4.34 0 3.72 0H.61C.26 0 0 .23 0 .56v9.89c0 .33.26.55.61.55h.8c.36 0 .61-.23.61-.56V6.99H3.3l3.73 4.74-2.71 3.48c-.12.2-.13.37-.04.53.11.19.3.26.52.26h1.27c.27 0 .51-.12.64-.34l1.69-2.15 1.66 2.14c.12.21.34.35.62.35h1.43c.2 0 .39-.08.5-.25.12-.18.09-.38-.02-.55l-2.77-3.54zM4.18 5H1.99V2.02h2.19c.62 0 1.08.13 1.38.37.29.22.44.62.44 1.08 0 .45-.15.94-.44 1.17-.31.23-.76.36-1.38.36zM15 2h-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1V4h1c.55 0 1-.45 1-1s-.45-1-1-1zM9.99 3.01c0 .02.01.04.01.06V2.95c0 .02-.01.04-.01.06z"],"new-text-box":["M5 6.5c0 .28.22.5.5.5H7v3.5c0 .28.22.5.5.5s.5-.22.5-.5V7h1.5c.28 0 .5-.22.5-.5S9.78 6 9.5 6h-4c-.28 0-.5.22-.5.5zM15 2h-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1V4h1c.55 0 1-.45 1-1s-.45-1-1-1zm-2 5c-.55 0-1 .45-1 1v5H3V4h5c.55 0 1-.45 1-1s-.45-1-1-1H2c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h11c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1z"],ninja:["M16 5s-2.52 2.11-4.96 1.99C11.03 4.89 10.39.23 5 0c0 0 2.11 2.54 1.96 4.99C4.86 5.01.23 5.65 0 11c0 0 2.56-2.12 5.02-1.95.02 2.11.67 6.72 5.98 6.95 0 0-2.09-2.54-1.94-4.99 2.11-.02 6.71-.68 6.94-6.01zM8 9.5c-.83 0-1.5-.67-1.5-1.5S7.17 6.5 8 6.5s1.5.67 1.5 1.5S8.83 9.5 8 9.5z"],"not-equal-to":["M7.58 5l.44-2.196a1 1 0 0 1 1.96.392L9.62 5H13a1 1 0 0 1 0 2H9.22l-.4 2H13a1 1 0 0 1 0 2H8.42l-.44 2.196a1 1 0 0 1-1.96-.392L6.38 11H3a1 1 0 0 1 0-2h3.78l.4-2H3a1 1 0 1 1 0-2h4.58z"],notifications:["M8 16c1.1 0 2-.9 2-2H6c0 1.1.9 2 2 2zm6-5c-.55 0-1-.45-1-1V6c0-2.43-1.73-4.45-4.02-4.9 0-.04.02-.06.02-.1 0-.55-.45-1-1-1S7 .45 7 1c0 .04.02.06.02.1A4.992 4.992 0 0 0 3 6v4c0 .55-.45 1-1 1s-1 .45-1 1 .45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1z"],"notifications-updated":["M8 16c1.1 0 2-.9 2-2H6c0 1.1.9 2 2 2zm3.399-13.667l-.413.412A2.99 2.99 0 0 0 9 1.99a3 3 0 0 0-3 2.99c0 .8.32 1.558.876 2.114l2.002 1.992A2.99 2.99 0 0 0 13 9.184V10c0 .55.45 1 1 1s1 .45 1 1-.45 1-1 1H2c-.55 0-1-.45-1-1s.45-1 1-1 1-.45 1-1V6c0-2.43 1.73-4.45 4.02-4.9 0-.04-.02-.06-.02-.1 0-.55.45-1 1-1s1 .45 1 1c0 .04-.02.06-.02.1a4.97 4.97 0 0 1 2.419 1.233zM10.29 7.67l-2-1.99a.99.99 0 0 1-.29-.7 1 1 0 0 1 1-.99c.27 0 .52.11.7.29l1.29 1.29 3.28-3.28c.18-.18.42-.29.7-.29.55 0 1 .44 1 .99 0 .28-.11.52-.3.7l-3.98 3.98a.99.99 0 0 1-1.4 0z"],"numbered-list":["M2.76 7h1.26V0h-.94c-.04.21-.12.39-.25.54-.13.15-.29.27-.48.36-.18.09-.39.16-.62.2-.23.04-.46.06-.71.06v.9h1.74V7zm-.59 7.17c.18-.12.37-.25.58-.37a10.763 10.763 0 0 0 1.24-.83c.2-.16.37-.33.52-.51.15-.19.28-.39.37-.61.09-.22.14-.47.14-.74 0-.22-.04-.45-.12-.7-.08-.26-.21-.49-.4-.69-.18-.21-.43-.39-.72-.52-.3-.14-.68-.21-1.12-.21-.41 0-.77.07-1.08.2-.32.14-.58.32-.8.56-.22.23-.38.51-.49.84-.11.32-.16.67-.16 1.05h1.19c.01-.24.03-.47.08-.67.05-.21.11-.39.21-.54.09-.15.22-.27.38-.36.16-.09.35-.13.59-.13.26 0 .47.04.63.12.16.08.29.18.38.3.09.12.15.25.18.39s.05.27.05.4c-.01.27-.08.5-.22.71-.14.21-.32.4-.53.57-.22.18-.45.34-.71.49-.26.15-.51.31-.74.47-.5.31-.89.68-1.17 1.11-.3.41-.44.91-.45 1.48h5v-1H1.43c.05-.15.14-.29.27-.43.14-.13.29-.26.47-.38zM15.01 1.99h-7c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-1c0-.55-.44-1-1-1zm0 9h-7c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-1c0-.55-.44-1-1-1z"],numerical:["M2.79 4.61c-.13.17-.29.3-.48.41-.18.11-.39.18-.62.23-.23.04-.46.07-.71.07v1.03h1.74V12h1.26V4h-.94c-.04.23-.12.44-.25.61zm4.37 5.31c.18-.14.37-.28.58-.42l.63-.45c.21-.16.41-.33.61-.51s.37-.38.52-.59c.15-.21.28-.45.37-.7.09-.25.13-.54.13-.85 0-.25-.04-.52-.12-.8-.07-.29-.2-.55-.39-.79a2.18 2.18 0 0 0-.73-.6c-.29-.15-.66-.23-1.11-.23-.41 0-.77.08-1.08.23-.31.16-.58.37-.79.64-.22.27-.38.59-.49.96-.11.37-.16.77-.16 1.2h1.19c.01-.27.03-.53.08-.77.04-.24.11-.45.21-.62.09-.18.22-.32.38-.42.16-.1.35-.15.59-.15.26 0 .47.05.63.14.15.09.28.21.37.35.09.14.15.29.18.45.03.16.05.31.05.45-.01.31-.08.58-.22.82-.14.23-.32.45-.53.65-.22.21-.46.39-.71.57-.26.18-.51.36-.75.54-.5.36-.89.78-1.17 1.27-.28.49-.43 1.06-.44 1.71h5v-1.15H6.43c.05-.17.14-.33.27-.49.13-.15.29-.29.46-.44zm8.5-1.56c-.23-.35-.54-.57-.95-.65v-.02c.34-.13.6-.34.76-.63.16-.29.24-.63.24-1.02 0-.34-.06-.64-.19-.9s-.3-.47-.51-.64c-.21-.17-.45-.3-.72-.38-.27-.09-.54-.13-.82-.13-.36 0-.68.07-.96.2-.28.13-.53.32-.72.55-.2.23-.36.51-.47.83-.11.32-.18.66-.19 1.04h1.15c-.01-.2.01-.39.06-.58.05-.19.12-.36.22-.51.1-.15.22-.27.37-.36.15-.09.32-.13.53-.13.32 0 .59.1.79.3.21.2.31.46.31.79 0 .23-.05.43-.14.59-.09.16-.21.29-.35.38-.15.09-.32.16-.51.19-.19.04-.38.05-.57.04v.93c.23-.01.45 0 .67.02.22.02.42.08.59.17.18.09.32.23.43.4.11.18.16.41.16.71 0 .44-.13.78-.39 1.02s-.58.36-.97.36c-.45 0-.79-.16-1.02-.47-.23-.31-.33-.7-.32-1.17H11c.01.4.06.77.17 1.1.11.33.26.61.47.85.21.23.46.42.77.54.31.13.67.19 1.08.19.34 0 .66-.05.96-.16.3-.11.57-.27.8-.47.23-.2.41-.45.55-.74.13-.27.2-.6.2-.97 0-.5-.11-.92-.34-1.27z"],office:["M15 5h-3V1c0-.55-.45-1-1-1H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h3v-4h4v4h7c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zM5 10H2V7h3v3zm0-5H2V2h3v3zm5 5H7V7h3v3zm0-5H7V2h3v3zm4 9h-2v-2h2v2zm0-4h-2V7h2v3z"],offline:["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zM6 14l1-5H4l6-7-1 5h3l-6 7z"],"oil-field":["M15 14h-1.35l-3.34-7.51 2.46-.95 1.45 3.21c.09.2.36.3.6.23.1-.03.18-.08.24-.15.05-.08 1.23-1.56.87-4.2-.11-.79-.52-4.62-3.26-4.62-.93 0-1.68.62-1.67 1.37 0 .14.03.28.09.42l.87 1.92L.64 8.07v.01A.98.98 0 0 0 0 9c0 .55.45 1 1 1 .13 0 .25-.03.36-.07v.01l1.04-.4L3.67 14H2c-.55 0-1 .45-1 1s.45 1 1 1h13c.55 0 1-.45 1-1s-.45-1-1-1zM4.27 8.81L7.14 7.7 5.2 12.08l-.93-3.27zM6.54 14L9 8.46 11.46 14H6.54z"],"one-column":["M11.99-.01h-3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h3c.55 0 1-.45 1-1v-14c0-.55-.45-1-1-1zm-6 5c-.28 0-.53.11-.71.29l-2 2a1.014 1.014 0 0 0 0 1.42l2 2a1.003 1.003 0 0 0 1.71-.71v-4c0-.55-.45-1-1-1z"],outdated:["M8 0c4.42 0 8 3.58 8 8 0 4.06-3.02 7.4-6.94 7.92-.02 0-.04.01-.06.01-.33.04-.66.07-1 .07-4.42 0-8-3.58-8-8 0-.55.45-1 1-1s1 .45 1 1c0 3.31 2.69 6 6 6 .71 0 1.37-.15 2-.38v.01c2.33-.82 4-3.02 4-5.63 0-3.31-2.69-6-6-6-1.78 0-3.36.78-4.46 2H5c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V1c0-.55.45-1 1-1s1 .45 1 1v1.74A7.95 7.95 0 0 1 8 0zm1 12H7v-2h2v2zm0-3H7V4h2v5z"],"page-layout":["M15 .95H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-12c0-.55-.45-1-1-1zm-9 12H2v-6h4v6zm8 0H7v-6h7v6zm0-7H2v-3h12v3z"],"panel-stats":["M10 4h3v1h-3zM10 6h3v1h-3zM10 8h3v1h-3zM10 10h3v1h-3z","M15 1H1c-.6 0-1 .4-1 1v11c0 .6.4 1 1 1h14c.6 0 1-.4 1-1V2c0-.6-.4-1-1-1zM8 12H2V3h6v9zm6 0H9V3h5v9z"],"panel-table":["M15 1H1c-.6 0-1 .4-1 1v11c0 .6.4 1 1 1h14c.6 0 1-.4 1-1V2c0-.6-.4-1-1-1zM8 9H6V7h2v2zm0-3H6V4h2v2zm-6 6V3h3v9H2zm4 0v-2h2v2H6zm8 0H9v-2h5v2zm0-3H9V7h5v2zm0-3H9V4h5v2z"],paperclip:["M14.68 2.31A4.54 4.54 0 0 0 11.46.99c-1.15 0-2.31.44-3.19 1.32L.95 9.63c-.63.63-.95 1.46-.95 2.28a3.21 3.21 0 0 0 3.23 3.22c.83 0 1.66-.31 2.3-.95l7.31-7.32c.76-.77.76-1.98.01-2.73s-1.99-.76-2.75 0l-6.07 6.08c-.24.25-.24.65.01.9s.65.25.91.01l6.07-6.08c.25-.25.67-.25.91-.01.25.25.25.67 0 .92l-7.31 7.32c-.75.75-2.04.74-2.76.01-.75-.75-.73-2.02.01-2.76L9.2 3.21c1.24-1.24 3.35-1.26 4.58-.03 1.24 1.24 1.24 3.36 0 4.6l-7.12 7.13c-.24.25-.24.64.01.88.24.24.63.24.88.01v.01l7.13-7.13A4.41 4.41 0 0 0 16 5.51c0-1.16-.44-2.32-1.32-3.2z"],paragraph:["M13 1H6C3.8 1 2 2.8 2 5s1.8 4 4 4v5c0 .6.4 1 1 1s1-.5 1-1V3h2v11c0 .6.4 1 1 1s1-.5 1-1V3h1c.5 0 1-.4 1-1s-.4-1-1-1z"],path:["M14.5 0h-13C.67 0 0 .67 0 1.5S.67 3 1.5 3H7v3H3.5C2.67 6 2 6.67 2 7.5S2.67 9 3.5 9H7v3H5.5c-.83 0-1.5.67-1.5 1.5S4.67 15 5.5 15h5c.83 0 1.5-.67 1.5-1.5s-.67-1.5-1.5-1.5H9V9h3.5c.83 0 1.5-.67 1.5-1.5S13.33 6 12.5 6H9V3h5.5c.83 0 1.5-.67 1.5-1.5S15.33 0 14.5 0z"],"path-search":["M15 14.62l-4-2.4V9.77c-.32.09-.66.15-1 .18v2.27l-4 2.4V8.71c-.38-.31-.72-.66-1-1.06v6.97l-4-2.4V8c.55 0 1-.45 1-1s-.45-1-1-1V1.38l3.15 1.89c.08-.34.18-.66.32-.97L.76.07v.01A.496.496 0 0 0 .5 0C.22 0 0 .22 0 .5v12c0 .18.1.33.25.42v.01l5 3v-.01c.07.05.16.08.25.08s.18-.03.25-.08v.01l4.74-2.85 4.74 2.85v-.01c.09.05.18.08.27.08.28 0 .5-.22.5-.5v-3.78c-.3.17-.63.28-1 .28v2.62zM2 5c0 .55.45 1 1 1s1-.45 1-1-.45-1-1-1-1 .45-1 1zm6-1c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm7.75-.92l-1.19-.72c.18.43.29.9.36 1.38l.08.04v3.39l1 1V3.5c0-.18-.1-.33-.25-.42zM10 2c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm3.3 4.89c.44-.7.7-1.51.7-2.39C14 2.01 11.99 0 9.5 0S5 2.01 5 4.5 7.01 9 9.5 9c.88 0 1.69-.26 2.39-.7l2.41 2.41c.17.18.42.29.7.29a1.003 1.003 0 0 0 .71-1.71l-2.41-2.4zM9.5 8C7.57 8 6 6.43 6 4.5S7.57 1 9.5 1 13 2.57 13 4.5 11.43 8 9.5 8z"],pause:["M6 3H4c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1zm6 0h-2c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],people:["M13.69 13.98c-.05-.24-.14-.5-.25-.76-.36-.86-1.12-1.33-2.69-2-.14-.06-.59-.25-.6-.25-.21-.09-.36-.15-.5-.22.02-.1.02-.2.03-.31 0-.04.01-.08.01-.13-.07-.06-.13-.12-.19-.19.22-.32.4-.67.54-1.05.02-.06.02-.06.03-.1.29-.23.48-.57.59-.96.16-.33.25-.73.21-1.16-.03-.4-.16-.76-.37-1.03-.02-.53-.07-1.13-.15-1.54-.01-.06-.02-.12-.03-.19.23-.06.48-.09.72-.09.49 0 1.05.16 1.44.46.38.29.67.7.8 1.17.03.1.05.21.07.31.07.37.11.94.11 1.33v.05c.14.06.27.21.29.51.02.25-.07.45-.13.54-.05.21-.16.44-.38.48-.02.1-.05.2-.09.3 0 .01-.01.03-.01.03-.17.44-.43.83-.75 1.11v.14c.03.35-.09.59.83 1 .93.41 2.32.84 2.6 1.5.29.66.17 1.04.17 1.04h-2.3zm-1.17-.38c.37.86.22 1.36.22 1.36H.06s-.14-.5.22-1.36 2.13-1.43 3.31-1.96c1.17-.54 1.05-.86 1.09-1.3 0-.05.01-.11.01-.17-.41-.35-.75-.86-.97-1.45v-.01s-.01-.01-.01-.02c-.04-.12-.09-.26-.12-.39-.28-.05-.44-.36-.5-.64-.06-.12-.19-.39-.16-.71.04-.41.21-.6.39-.68v-.06c0-.51.05-1.26.14-1.74.02-.13.05-.27.09-.4.17-.6.54-1.13 1.02-1.51.5-.39 1.21-.6 1.84-.6s1.34.21 1.84.6c.48.38.85.91 1.02 1.52.04.13.07.27.09.4.09.48.14 1.22.14 1.73v.07c.18.08.34.27.37.67.03.32-.09.59-.16.71-.06.28-.21.58-.48.63-.03.13-.07.26-.12.39 0 .01-.01.04-.01.04-.22.58-.55 1.08-.95 1.45v.18c.04.45-.12.77 1.06 1.3 1.18.53 2.95 1.09 3.31 1.95z"],percentage:["M6 6V4c0-1.1-.9-2-2-2H3c-1.1 0-2 .9-2 2v2c0 1.1.9 2 2 2h1c1.1 0 2-.9 2-2zM3.5 6c-.28 0-.5-.22-.5-.5v-1c0-.28.22-.5.5-.5s.5.22.5.5v1c0 .28-.22.5-.5.5zM13 8h-1c-1.1 0-2 .9-2 2v2c0 1.1.9 2 2 2h1c1.1 0 2-.9 2-2v-2c0-1.1-.9-2-2-2zm0 3.5c0 .28-.22.5-.5.5s-.5-.22-.5-.5v-1c0-.28.22-.5.5-.5s.5.22.5.5v1zM12 3a1.003 1.003 0 0 0-1.87-.5l-5.99 9.98c-.09.15-.14.33-.14.52a1.003 1.003 0 0 0 1.87.5l5.99-9.98c.09-.15.14-.33.14-.52z"],person:["M15.68 14.32c-.46-1.05-2.68-1.75-4.16-2.4-1.48-.65-1.28-1.05-1.33-1.59-.01-.07-.01-.15-.01-.23.51-.45.92-1.07 1.19-1.78 0 0 .01-.04.02-.05.06-.15.11-.32.15-.48.34-.07.54-.44.61-.78.08-.14.23-.48.2-.87-.05-.5-.25-.73-.47-.82v-.09c0-.63-.06-1.55-.17-2.15A3.671 3.671 0 0 0 10.32.72C9.68.25 8.79-.01 8-.01c-.79 0-1.68.25-2.31.73-.61.47-1.06 1.13-1.28 1.86-.05.17-.09.33-.11.5-.12.6-.17 1.51-.17 2.15v.08c-.24.09-.45.32-.5.83-.03.38.13.72.2.86.08.35.28.72.63.78.04.17.09.33.15.49 0 .01.01.02.01.03l.01.01c.27.72.7 1.35 1.22 1.8 0 .07-.01.14-.01.21-.05.54.1.94-1.37 1.59-1.48.65-3.7 1.35-4.16 2.4-.46 1.05-.27 1.67-.27 1.67h15.92c-.01.01.18-.61-.28-1.66z"],phone:["M15.9 12.41c-.06-.06-3.37-2-3.48-2.05a.794.794 0 0 0-.32-.08c-.15 0-.34.11-.57.32-.23.22-.94 1.19-1.15 1.4-.21.22-.38.32-.52.32-.07 0-.15-.02-.25-.06-.1-.04-1.16-.58-3.36-2.52-2.2-1.93-2.49-3.2-2.5-3.55 0-.14.11-.31.32-.52.22-.21.45-.41.7-.6.25-.19.49-.4.7-.62.22-.23.32-.42.32-.57 0-.11-.03-.21-.08-.32C5.66 3.46 3.66.15 3.59.08 3.44-.07 2.85 0 2.55.16.16 1.46-.03 3.2 0 3.89c.04.71.49 4.46 4.16 7.95C8.72 16.17 11.89 16 12.1 16c.69 0 2.82-.38 3.72-2.55.13-.32.25-.87.08-1.04z"],"pie-chart":["M7 1.08c-3.37.5-5.97 3.4-5.97 6.92 0 3.87 3.13 7 6.98 7 3.52 0 6.42-2.61 6.91-6H7V1.08z","M8 0v8h8c0-4.42-3.58-8-8-8z"],pin:["M9.41.92c-.51.51-.41 1.5.15 2.56L4.34 7.54C2.8 6.48 1.45 6.05.92 6.58l3.54 3.54-3.54 4.95 4.95-3.54 3.54 3.54c.53-.53.1-1.88-.96-3.42l4.06-5.22c1.06.56 2.04.66 2.55.15L9.41.92z"],pivot:["M4.57 7.02L.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l4.27-4.27c-.58-.35-1.07-.84-1.41-1.42zM15 8c-.55 0-1 .45-1 1v.59l-2.57-2.57c-.34.58-.83 1.07-1.41 1.41L12.59 11H12c-.55 0-1 .45-1 1s.45 1 1 1h3c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1zm-4-3c0-1.66-1.34-3-3-3S5 3.34 5 5s1.34 3 3 3 3-1.34 3-3zM8 6c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"pivot-table":["M2 4H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm0-4H1C.45 0 0 .45 0 1v1c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm11.71 4.29C13.53 4.11 13.28 4 13 4s-.53.11-.71.29l-2 2a1.003 1.003 0 0 0 1.42 1.42l.29-.3V9c0 1.66-1.34 3-3 3H7.41l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-2 2c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42l-.3-.29H9c2.76 0 5-2.24 5-5V7.41l.29.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-2-2zM15 0H5c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],play:["M12 8c0-.35-.19-.64-.46-.82l.01-.02-6-4-.01.02A.969.969 0 0 0 5 3c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1 .21 0 .39-.08.54-.18l.01.02 6-4-.01-.02c.27-.18.46-.47.46-.82z"],plus:["M13 7H9V3c0-.55-.45-1-1-1s-1 .45-1 1v4H3c-.55 0-1 .45-1 1s.45 1 1 1h4v4c0 .55.45 1 1 1s1-.45 1-1V9h4c.55 0 1-.45 1-1s-.45-1-1-1z"],"polygon-filter":["M14 5c-.24 0-.47.05-.68.13L9.97 2.34c.01-.11.03-.22.03-.34 0-1.1-.9-2-2-2S6 .9 6 2c0 .04.01.08.01.12L2.88 4.21C2.61 4.08 2.32 4 2 4 .9 4 0 4.9 0 6c0 .74.4 1.38 1 1.72v4.55c-.6.35-1 .99-1 1.73 0 1.1.9 2 2 2 .74 0 1.38-.4 1.72-1h4.55c.35.6.98 1 1.72 1 1.1 0 2-.9 2-2 0-.37-.11-.7-.28-1L14 9c1.11-.01 2-.9 2-2s-.9-2-2-2zm-4.01 7c-.73 0-1.37.41-1.71 1H3.73c-.18-.3-.43-.55-.73-.72V7.72c.6-.34 1-.98 1-1.72 0-.04-.01-.08-.01-.12l3.13-2.09c.27.13.56.21.88.21.24 0 .47-.05.68-.13l3.35 2.79c-.01.11-.03.22-.03.34 0 .37.11.7.28 1l-2.29 4z"],power:["M8 8c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1S7 .45 7 1v6c0 .55.45 1 1 1zm3-5.32v2.34c1.21.91 2 2.35 2 3.98 0 2.76-2.24 5-5 5s-5-2.24-5-5c0-1.63.79-3.06 2-3.98V2.68C2.64 3.81 1 6.21 1 9c0 3.87 3.13 7 7 7s7-3.13 7-7c0-2.79-1.64-5.19-4-6.32z"],"predictive-analysis":["M16 6.41c0-1.01-.49-1.94-1.29-2.49-.43-1.92-2.07-3.28-4-3.28-.46 0-.92.08-1.35.24C8.83.31 8.11 0 7.34 0c-.9 0-1.74.44-2.28 1.16-.12-.01-.24-.02-.36-.02-1.31 0-2.42.89-2.77 2.17C.78 3.72 0 4.84 0 6.13c0 .38.07.76.21 1.12C.07 7.6 0 7.98 0 8.36c0 1.11.58 2.11 1.51 2.63.54.56 1.27.87 2.03.87.49 0 .95-.12 1.37-.36a2.85 2.85 0 0 0 2.18 1.04c.52 0 1.03-.14 1.47-.42.49.39 1.07.65 1.69.73 1.04 1.15 1.84 2.63 1.84 2.64 0 0 .28.49.26.49.77 0 1.41-.16 1.32-1.04 0 .02-.73-2.31-.73-2.31.41-.21.75-.55.97-.98.9-.52 1.47-1.53 1.47-2.61 0-.24-.03-.48-.08-.71.45-.52.7-1.21.7-1.92zm-1.23 1.02l-.15-.16-.61-.67c-.27-.29-.54-.94-.58-1.39l-.1-1.01c-.05-.59-.94-.58-.91.11 0 .02.1 1.01.1 1.01.03.29.12.62.24.93-.06-.01-.12-.02-.18-.02 0 0-2.06-.1-2.05-.11-.58-.02-.71.97-.04 1l2.05.11c.42.02 1.04.3 1.29.58l.49.54.02.05c.08.21.12.44.12.66 0 .74-.41 1.41-1.07 1.75l-.16.08-.07.18c-.15.38-.48.66-.88.74l-.54.11.7 2.2c-.38-.61-.95-1.43-1.62-2.14l-.12-.13-.17-.01c-.41-.03-.8-.17-1.14-.38l1.36-1.18c.35-.31.83-.44.99-.39 0 0 .63.17.62.18.63.16.83-.74.23-.97l-.62-.18c-.55-.16-1.33.18-1.79.58l-1.53 1.33-.31.26c-.35.29-.75.44-1.2.44-.64 0-1.23-.33-1.58-.86V9.15c0-.4.17-.79.27-.85 0 0 .52-.34.51-.35.71-.53.18-1.23-.49-.89 0-.01-.52.35-.52.35-.26.15-.45.44-.58.77-.11-.11-.22-.2-.34-.28 0 0-1.53-1.01-1.53-1.02-.65-.45-1.2.51-.49.89 0-.01 1.51 1.02 1.51 1.02.37.24.62.78.62 1.09v.67c-.34.19-.63.29-.99.29-.54 0-1.05-.23-1.41-.63l-.05-.06-.07-.04c-.65-.34-1.05-1-1.05-1.73 0-.3.07-.6.2-.87l.12-.25L1.15 7c-.13-.27-.2-.56-.2-.87 0-.9.61-1.68 1.48-1.89l.31-.08.05-.34a1.926 1.926 0 0 1 2.38-1.58l.32.08.18-.31c.35-.6.99-.97 1.67-.97.44 0 .86.15 1.2.42l-.36.36v-.01l-.25.26c-.33.27-.74.42-.89.4 0 0-.67-.1-.67-.11-.67-.13-.87.86-.14 1.02.01 0 .67.11.67.11.02 0 .05 0 .07.01-.11.37-.15.77-.1 1.12 0 0 .17.99.15.99.11.52 1.06.36.93-.18 0-.01-.15-.99-.15-.99-.05-.37.12-.94.36-1.19l.39-.4c.05-.05.1-.09.15-.14l.74-.76c.4-.18.83-.27 1.27-.27 1.55 0 2.86 1.12 3.11 2.67l.04.25.21.12c.61.35.98 1 .98 1.7 0 .36-.1.7-.28 1.01z"],prescription:["M10.91 8.34c.14-.21.36-.34.63-.34h1.29c.22 0 .41.07.52.26.09.16.08.33-.04.53l-2.49 2.87 2.77 3.54c.12.17.14.37.02.55-.11.17-.3.25-.5.25h-1.44a.69.69 0 0 1-.61-.35L9.4 13.51l-1.69 2.15c-.13.21-.36.34-.63.34H5.8c-.22 0-.41-.07-.52-.26-.09-.16-.08-.33.04-.53l2.71-3.48L4.3 6.99H3.03v3.47c0 .33-.26.56-.62.56h-.8c-.35-.01-.61-.23-.61-.56V.56c0-.33.26-.56.62-.56h3.11c.62 0 1.19.08 1.7.24.51.16.96.39 1.34.69a3.194 3.194 0 0 1 1.21 2.53c0 .81-.25 1.5-.74 2.08-.37.44-.84.77-1.42 1.01L7.88 7.9c.04.04.07.08.08.1l1.49 1.9 1.46-1.56zM5.18 5c.62 0 1.08-.13 1.39-.37.29-.23.44-.71.44-1.16s-.15-.87-.44-1.1C6.26 2.12 5.8 2 5.18 2H2.99v3h2.19z"],presentation:["M15 1H9c0-.55-.45-1-1-1S7 .45 7 1H1c-.55 0-1 .45-1 1s.45 1 1 1v8c0 .55.45 1 1 1h3.59L3.3 14.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L7 13.41V15c0 .55.45 1 1 1s1-.45 1-1v-1.59l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L10.41 12H14c.55 0 1-.45 1-1V3c.55 0 1-.45 1-1s-.45-1-1-1zm-2 9H3V3h10v7z"],print:["M12 2.02c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v1h8v-1zm3 2H1c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h1v-3h12v3h1c.55 0 1-.45 1-1v-6c0-.56-.45-1-1-1zm-1 3h-2v-1h2v1zm-3 6H5v-3H3v4c0 .55.45 1 1 1h8c.55 0 1-.45 1-1v-4h-2v3z"],projects:["M14 3c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v1h12V3zm-2-3H4c-.55 0-1 .45-1 1h10c0-.55-.45-1-1-1zm3 5H1c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm-3 6c0 .55-.45 1-1 1H5c-.55 0-1-.45-1-1V9h1v2h6V9h1v2z"],properties:["M2 6C.9 6 0 6.9 0 8s.9 2 2 2 2-.9 2-2-.9-2-2-2zm4-3h9c.55 0 1-.45 1-1s-.45-1-1-1H6c-.55 0-1 .45-1 1s.45 1 1 1zm-4 9c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm13-5H6c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1zm0 6H6c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1zM2 0C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],property:["M3 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm-.5-6.5a2.5 2.5 0 0 0 0 5 2.5 2.5 0 0 0 0-5zM7 3h8c.55 0 1-.45 1-1s-.45-1-1-1H7c-.55 0-1 .45-1 1s.45 1 1 1zm8 10H7c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1zM3 0C1.9 0 1 .9 1 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12 6H7c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h8c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1z"],"publish-function":["M12.16 3.76c.15-.11.3-.16.47-.16.06 0 .17.02.34.06.16.04.31.06.43.06a.58.58 0 0 0 .6-.6c0-.19-.06-.33-.17-.44-.11-.11-.28-.16-.49-.16-.19 0-.37.04-.54.13-.17.09-.39.27-.65.55-.2.21-.48.58-.87 1.11a5.22 5.22 0 0 0-.78-1.79l-2.05.32-.04.21c.15-.03.28-.04.39-.04.2 0 .37.08.5.25.21.26.5 1.03.88 2.33-.29.36-.49.6-.6.71-.18.19-.33.31-.45.36-.09.04-.19.07-.3.07-.09 0-.23-.04-.42-.13a.904.904 0 0 0-.36-.09c-.2 0-.36.06-.49.18a.59.59 0 0 0-.19.46c0 .18.06.32.18.43.12.11.28.16.48.16.2 0 .38-.04.55-.12.17-.08.39-.24.65-.49s.62-.65 1.07-1.19c.18.52.33.89.46 1.13.13.24.28.4.44.51.17.1.37.16.62.16.24 0 .49-.08.74-.25.33-.21.66-.58 1.01-1.09l-.21-.11c-.23.31-.41.5-.52.57a.44.44 0 0 1-.26.07c-.12 0-.24-.07-.36-.21-.2-.24-.46-.91-.8-2 .29-.49.54-.81.74-.96zM6.37 5.83l.68-2.53h.83l.2-.64h-.84c.24-.91.56-1.59.96-2.01.24-.27.48-.4.71-.4.05 0 .08.01.11.04s.04.06.04.1c0 .04-.03.11-.1.21-.06.1-.1.2-.1.29 0 .13.05.24.15.33.1.09.23.14.39.14.17 0 .31-.06.42-.17.12-.12.18-.27.18-.46 0-.21-.08-.39-.25-.52C9.57.07 9.3 0 8.93 0c-.59 0-1.12.16-1.59.48-.48.32-.93.85-1.36 1.59-.15.26-.29.42-.42.49s-.35.11-.64.1l-.19.65h.81L4.35 7.68c-.2.72-.33 1.16-.4 1.33-.1.24-.26.45-.46.62a.48.48 0 0 1-.31.1c-.03 0-.06-.01-.08-.03l-.03-.03c0-.02.03-.06.09-.11.06-.06.09-.15.09-.26 0-.13-.05-.23-.14-.32-.1-.09-.23-.13-.41-.13-.21 0-.38.05-.51.16A.52.52 0 0 0 2 9.4c0 .16.08.3.23.42.16.12.4.18.74.18.53 0 .99-.13 1.4-.39.41-.26.76-.65 1.07-1.19.3-.53.61-1.39.93-2.59zm2.34 3.46A.997.997 0 0 0 8 9c-.28 0-.53.11-.71.29l-2 2a1.003 1.003 0 0 0 1.42 1.42l.29-.3V15c0 .55.45 1 1 1s1-.45 1-1v-2.59l.29.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-2-2z"],pulse:["M15 8h-1.46l-1.7-2.55-.02.01A.984.984 0 0 0 11 5c-.43 0-.79.27-.93.65h-.01l-1.69 4.51-1.38-8.32h-.02A.989.989 0 0 0 6 1c-.41 0-.77.25-.92.61L2.34 8H1c-.55 0-1 .45-1 1s.45 1 1 1h2c.41 0 .77-.25.92-.61l1.65-3.86 1.44 8.63h.02c.08.47.47.84.97.84.43 0 .79-.27.93-.65h.01l2.31-6.17.92 1.38.02-.01c.17.26.46.45.81.45h2c.55 0 1-.45 1-1s-.45-1-1-1z"],random:["M11.48 4h1.11l-.29.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-2-2a1.003 1.003 0 0 0-1.42 1.42l.3.29H11c-.32 0-.59.16-.77.38l-.01-.01L8.28 4.8l1.28 1.6L11.48 4zm2.23 6.29a1.003 1.003 0 0 0-1.42 1.42l.3.29h-1.11l-7.7-9.62h-.01A.996.996 0 0 0 3 2H1c-.55 0-1 .45-1 1s.45 1 1 1h1.52l7.7 9.62.01-.01c.18.23.45.39.77.39h1.59l-.29.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-2-2zM2.52 12H1c-.55 0-1 .45-1 1s.45 1 1 1h2c.32 0 .59-.16.77-.38l.01.01 1.94-2.42L4.44 9.6 2.52 12z"],record:["M8 3a5 5 0 1 0 0 10A5 5 0 1 0 8 3z"],redo:["M12 11c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm3.71-6.71l-3-3a1.003 1.003 0 0 0-1.42 1.42L12.59 4H5C2.24 4 0 6.24 0 9s2.24 5 5 5h4v-2H5c-1.66 0-3-1.34-3-3s1.34-3 3-3h7.59L11.3 7.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],refresh:["M14.99 6.99c-.55 0-1 .45-1 1 0 3.31-2.69 6-6 6-1.77 0-3.36-.78-4.46-2h1.46c.55 0 1-.45 1-1s-.45-1-1-1h-4c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1s1-.45 1-1v-1.74a7.95 7.95 0 0 0 6 2.74c4.42 0 8-3.58 8-8 0-.55-.45-1-1-1zm0-7c-.55 0-1 .45-1 1v1.74a7.95 7.95 0 0 0-6-2.74c-4.42 0-8 3.58-8 8 0 .55.45 1 1 1s1-.45 1-1c0-3.31 2.69-6 6-6 1.77 0 3.36.78 4.46 2h-1.46c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1z"],"regression-chart":["M13 6.5c0 .83.67 1.5 1.5 1.5S16 7.33 16 6.5 15.33 5 14.5 5 13 5.67 13 6.5zM8.5 5c.83 0 1.5-.67 1.5-1.5S9.33 2 8.5 2 7 2.67 7 3.5 7.67 5 8.5 5zM9 9.5c0 .83.67 1.5 1.5 1.5s1.5-.67 1.5-1.5S11.33 8 10.5 8 9 8.67 9 9.5zM4.5 8C5.33 8 6 7.33 6 6.5S5.33 5 4.5 5 3 5.67 3 6.5 3.67 8 4.5 8zM15 12H3.26l12.03-8.59-.58-.81L2 11.67V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],remove:["M10.99 6.99h-6c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1zm-3-7c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.68 6-6 6z"],"remove-column":["M14 0H4c-.55 0-1 .45-1 1v3h2V2h3v12H5v-2H3v3c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14h-3V2h3v12zm-8.71-3.29a1.003 1.003 0 0 0 1.42-1.42L4.41 8 5.7 6.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L3 6.59l-1.29-1.3A1.003 1.003 0 0 0 .29 6.71L1.59 8 .29 9.29a1.003 1.003 0 0 0 1.42 1.42L3 9.41l1.29 1.3z"],"remove-column-left":["M4 9h4c.55 0 1-.45 1-1s-.45-1-1-1H4c-.55 0-1 .45-1 1s.45 1 1 1zm11-9H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-5 14H2V2h8v12zm4 0h-3V2h3v12z"],"remove-column-right":["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM5 14H2V2h3v12zm9 0H6V2h8v12zM8 9h4c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1z"],"remove-row-bottom":["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H2V6h12v8zm0-9H2V2h12v3zm-8 6h4c.55 0 1-.45 1-1s-.45-1-1-1H6c-.55 0-1 .45-1 1s.45 1 1 1z"],"remove-row-top":["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H2v-3h12v3zm0-4H2V2h12v8zM6 7h4c.55 0 1-.45 1-1s-.45-1-1-1H6c-.55 0-1 .45-1 1s.45 1 1 1z"],repeat:["M10 5c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1.74A7.95 7.95 0 0 0 8 0C3.58 0 0 3.58 0 8c0 4.06 3.02 7.4 6.94 7.92.02 0 .04.01.06.01.33.04.66.07 1 .07 4.42 0 8-3.58 8-8 0-.55-.45-1-1-1s-1 .45-1 1c0 3.31-2.69 6-6 6-.71 0-1.37-.15-2-.38v.01C3.67 12.81 2 10.61 2 8c0-3.31 2.69-6 6-6 1.77 0 3.36.78 4.46 2H11c-.55 0-1 .45-1 1z"],reset:["M6 5c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V1c0-.55.45-1 1-1s1 .45 1 1v1.74A7.95 7.95 0 0 1 8 0c4.42 0 8 3.58 8 8 0 4.06-3.02 7.4-6.94 7.92-.02 0-.04.01-.06.01-.33.04-.66.07-1 .07-4.42 0-8-3.58-8-8 0-.55.45-1 1-1s1 .45 1 1c0 3.31 2.69 6 6 6 .71 0 1.37-.15 2-.38v.01c2.33-.82 4-3.02 4-5.63 0-3.31-2.69-6-6-6-1.77 0-3.36.78-4.46 2H5c.55 0 1 .45 1 1z"],resolve:["M6.6 3.3C6.1 3.1 5.6 3 5 3 2.2 3 0 5.2 0 8s2.2 5 5 5c.6 0 1.1-.1 1.6-.3C5.3 11.6 4.5 9.9 4.5 8s.8-3.6 2.1-4.7zM8 4c-1.2.9-2 2.4-2 4s.8 3.1 2 4c1.2-.9 2-2.3 2-4s-.8-3.1-2-4zm3-1c-.6 0-1.1.1-1.6.3 1.3 1.2 2.1 2.9 2.1 4.7s-.8 3.6-2.1 4.7c.5.2 1 .3 1.6.3 2.8 0 5-2.2 5-5s-2.2-5-5-5z"],rig:["M5.71 3c0 1.1.96 2 2.14 2C9.04 5 10 3.96 10 3c0-1.96-1.47-3-2.14-3H5c0 1.96 2.68 1.4.71 3zm2.5 3l.01.01s0-.01-.01-.01zm6.5 8.29L10 9.59V7c0-.55-.45-1-1-1H7c-.55 0-1 .45-1 1v2.58l-4.71 4.7c-.18.19-.29.44-.29.72a1.003 1.003 0 0 0 1.71.71L6 12.42V15c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-2.58l3.29 3.29a1.003 1.003 0 0 0 1.42-1.42z"],"right-join":["M6.6 3.3C5.3 4.4 4.5 6.1 4.5 8s.8 3.6 2.1 4.7c-.5.2-1 .3-1.6.3-2.8 0-5-2.2-5-5s2.2-5 5-5c.6 0 1.1.1 1.6.3zm-1.96 8.68C3.92 10.83 3.5 9.46 3.5 8s.42-2.83 1.14-3.98C2.6 4.2 1 5.91 1 8s1.6 3.8 3.64 3.98zM8 4c-1.2.9-2 2.4-2 4s.8 3.1 2 4c1.2-.9 2-2.3 2-4s-.8-3.1-2-4zm3-1c2.8 0 5 2.2 5 5s-2.2 5-5 5c-.6 0-1.1-.1-1.6-.3 1.3-1.1 2.1-2.9 2.1-4.7s-.8-3.5-2.1-4.7c.5-.2 1-.3 1.6-.3z"],ring:["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 12c-2.21 0-4-1.79-4-4s1.79-4 4-4 4 1.79 4 4-1.79 4-4 4z"],"rotate-document":["M12 2h-1.59l.29-.29c.19-.18.3-.43.3-.71A1.003 1.003 0 0 0 9.29.29l-2 2C7.11 2.47 7 2.72 7 3c0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42l-.3-.29H12c.55 0 1 .45 1 1v3c0 .55.45 1 1 1s1-.45 1-1V5c0-1.66-1.34-3-3-3zM5.71 5.29A.997.997 0 0 0 5 5H1c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V9c0-.28-.11-.53-.29-.71l-3-3zM7 14H2V7h2v2c0 .55.45 1 1 1h2v4z"],"rotate-page":["M8 6H2c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h6c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1zm-1 8H3V8h4v6zm5-12h-1.59l.29-.29c.19-.18.3-.43.3-.71A1.003 1.003 0 0 0 9.29.29l-2 2C7.11 2.47 7 2.72 7 3c0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42l-.3-.29H12c.55 0 1 .45 1 1v3c0 .55.45 1 1 1s1-.45 1-1V5c0-1.66-1.34-3-3-3z"],satellite:["M3 9c0-.6.4-1 1-1s1 .4 1 1c0 1.1.9 2 2 2 .6 0 1 .4 1 1s-.4 1-1 1c-2.2 0-4-1.8-4-4zM0 9c0-.6.4-1 1-1s1 .4 1 1c0 2.8 2.2 5 5 5 .6 0 1 .4 1 1s-.4 1-1 1c-3.9 0-7-3.1-7-7zm7 1c-.6 0-1-.4-1-1s.4-1 1-1 1 .4 1 1-.4 1-1 1zm1.3-2.8c-.4-.4-.4-1 0-1.4l4.5-4.5c.4-.4 1-.4 1.4 0l.5.5c.4.4.4 1 0 1.4l-4.5 4.5c-.4.4-1 .4-1.4 0l-.5-.5zM5.2.3c.4-.4 1-.4 1.4 0l2.1 2.1c.4.4.4 1 0 1.4l-.9.9c-.4.4-1 .4-1.4 0L4.3 2.6c-.4-.4-.4-1 0-1.4l.9-.9zm7 7c.4-.4 1-.4 1.4 0l2.1 2.1c.4.4.4 1 0 1.4l-.9.9c-.4.4-1 .4-1.4 0l-2.1-2.1c-.4-.4-.4-1 0-1.4l.9-.9z"],saved:["M6.71 9.29a1.003 1.003 0 0 0-1.42 1.42l2 2a.997.997 0 0 0 1.6-.27h.01l2-4h-.01c.06-.13.11-.28.11-.44 0-.55-.45-1-1-1-.39 0-.72.23-.89.56H9.1l-1.38 2.76-1.01-1.03zM9 0H3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V5L9 0zm3 14H4V2h4v4h4v8z"],"scatter-plot":["M15 12H2V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm-.5-7c.83 0 1.5-.67 1.5-1.5S15.33 2 14.5 2 13 2.67 13 3.5 13.67 5 14.5 5zm-3 4c.83 0 1.5-.67 1.5-1.5S12.33 6 11.5 6 10 6.67 10 7.5 10.67 9 11.5 9zm-4-2C8.33 7 9 6.33 9 5.5S8.33 4 7.5 4 6 4.67 6 5.5 6.67 7 7.5 7zm-3 4c.83 0 1.5-.67 1.5-1.5S5.33 8 4.5 8 3 8.67 3 9.5 3.67 11 4.5 11z"],search:["M15.55 13.43l-2.67-2.68a6.94 6.94 0 0 0 1.11-3.76c0-3.87-3.13-7-7-7s-7 3.13-7 7 3.13 7 7 7c1.39 0 2.68-.42 3.76-1.11l2.68 2.67a1.498 1.498 0 1 0 2.12-2.12zm-8.56-1.44c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5z"],"search-around":["M13.5 11c-.51 0-.98.15-1.38.42l-2.4-2.41c.17-.3.28-.64.28-1.01s-.11-.71-.28-1.01l2.41-2.41c.39.27.86.42 1.37.42a2.5 2.5 0 0 0 0-5A2.5 2.5 0 0 0 11 2.5c0 .51.15.98.42 1.38l-2.41 2.4C8.71 6.11 8.37 6 8 6s-.71.11-1.01.28l-2.41-2.4c.27-.4.42-.87.42-1.38a2.5 2.5 0 0 0-5 0A2.5 2.5 0 0 0 2.5 5c.51 0 .98-.15 1.38-.42l2.41 2.41C6.11 7.29 6 7.63 6 8s.11.71.28 1.01l-2.41 2.41c-.39-.27-.86-.42-1.37-.42a2.5 2.5 0 0 0 0 5A2.5 2.5 0 0 0 5 13.5c0-.51-.15-.98-.42-1.38l2.41-2.41c.3.18.64.29 1.01.29s.71-.11 1.01-.28l2.41 2.41c-.27.39-.42.86-.42 1.37a2.5 2.5 0 0 0 5 0 2.5 2.5 0 0 0-2.5-2.5zm0-10c.83 0 1.5.67 1.5 1.5S14.33 4 13.5 4 12 3.33 12 2.5 12.67 1 13.5 1zm-11 3C1.67 4 1 3.33 1 2.5S1.67 1 2.5 1 4 1.67 4 2.5 3.33 4 2.5 4zm0 11c-.83 0-1.5-.67-1.5-1.5S1.67 12 2.5 12s1.5.67 1.5 1.5S3.33 15 2.5 15zm11 0c-.83 0-1.5-.67-1.5-1.5s.67-1.5 1.5-1.5 1.5.67 1.5 1.5-.67 1.5-1.5 1.5z"],"search-template":["M15.55 13.43l-2.67-2.67c.7-1.09 1.11-2.38 1.11-3.77 0-3.87-3.13-7-7-7s-7 3.13-7 7 3.13 7 7 7c1.39 0 2.68-.41 3.77-1.11l2.67 2.67a1.498 1.498 0 1 0 2.12-2.12zm-8.56-1.44c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5zm2.5-6h-5c-.28 0-.5.22-.5.5s.22.5.5.5h5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5zm0-2h-5c-.28 0-.5.22-.5.5s.22.5.5.5h5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5zm0 4h-5c-.28 0-.5.22-.5.5s.22.5.5.5h5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5z"],"search-text":["M9 4H5c-.55 0-1 .45-1 1s.45 1 1 1h1v3c0 .55.45 1 1 1s1-.45 1-1V6h1c.55 0 1-.45 1-1s-.45-1-1-1zm6.56 9.44l-2.67-2.67C13.59 9.68 14 8.39 14 7c0-3.87-3.13-7-7-7S0 3.13 0 7s3.13 7 7 7c1.39 0 2.68-.41 3.77-1.11l2.67 2.67a1.498 1.498 0 1 0 2.12-2.12zM7 12c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5z"],"segmented-control":["M15 4H1c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm-1 6H8V6h6v4z"],select:["M16 15c0-.28-.12-.52-.31-.69l.02-.02-3.12-3.12 3.41-.84-8.05-2.86c.03-.09.05-.17.05-.27V2c0-.55-.45-1-1-1H3c0-.55-.45-1-1-1S1 .45 1 1c-.55 0-1 .45-1 1s.45 1 1 1v4c0 .55.45 1 1 1h5.2c.1 0 .18-.02.27-.05L10.33 16l.85-3.41 3.12 3.12.02-.02c.16.19.4.31.68.31.04 0 .07-.02.1-.02s.06.02.1.02c.44 0 .8-.36.8-.8 0-.04-.02-.07-.02-.1s.02-.06.02-.1zM6 6H3V3h3v3z"],selection:["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6zm0-9C6.34 5 5 6.34 5 8s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3z"],"send-to":["M15 7.5c-.8 0-1.5-.4-2-1l-1.2 1.2c-.4.5-1.1.7-1.8.7-1.4.1-2.5-1-2.5-2.4 0-.7.3-1.3.7-1.8L9.5 3c-.6-.5-1-1.2-1-2 0-.3.1-.7.2-1H8C3.6 0 0 3.6 0 8s3.6 8 8 8 8-3.6 8-8v-.7c-.3.1-.6.2-1 .2zM15 0h-4c-.6 0-1 .5-1 1s.4 1 1 1h1.6L9.3 5.3c-.2.2-.3.4-.3.7 0 .5.4 1 1 1 .3 0 .5-.1.7-.3L14 3.4V5c0 .6.4 1 1 1 .5 0 1-.4 1-1V1c0-.5-.4-1-1-1z"],"send-to-graph":["M6 9H2c-.55 0-1 .45-1 1s.45 1 1 1h1.59L.3 14.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L5 12.41V14c0 .55.45 1 1 1s1-.45 1-1v-4c0-.55-.45-1-1-1zm8 .5c-.56 0-1.06.23-1.42.59l-2.13-1.24L8.99 8l3.59-2.09A2.002 2.002 0 0 0 16 4.5c0-1.1-.9-2-2-2s-2 .9-2 2c0 .19.03.37.08.54L8.5 7.13v-3.2c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2S6 .9 6 2c0 .93.64 1.71 1.5 1.93v3.2l-.88-.52-2.7-1.57c.05-.17.08-.35.08-.54 0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.56 0 1.06-.23 1.42-.59l2.13 1.24 3.84 2.24 2.7 1.57c-.06.17-.09.35-.09.54 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2z"],"send-to-map":["M6 9H2c-.55 0-1 .45-1 1s.45 1 1 1h1.59L.3 14.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L5 12.41V14c0 .55.45 1 1 1s1-.45 1-1v-4c0-.55-.45-1-1-1zm9.55-5.83l-4.49-3A.975.975 0 0 0 9.99.15L5.53 2.82 1.56.17A1.003 1.003 0 0 0 0 1v6h2V2.87l2.94 1.96.06.03V7h1V4.86s.01 0 .01-.01L10 2.47v8.67s-.01 0-.01.01l-.99.58v2.33l1.47-.88 3.97 2.65A1.003 1.003 0 0 0 16 15V4c0-.35-.18-.65-.45-.83zM14 13.13l-2.94-1.96c-.02-.01-.04-.02-.05-.03v-8.6l3 2v8.59z"],"series-add":["M10.68 7.9c.44.54 1.07.92 1.79 1.05l-2.76 2.76c-.18.18-.43.29-.71.29s-.53-.11-.71-.3L5 8.41l-3 3V13h13c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1s1 .45 1 1v4.59l2.29-2.3C4.47 6.11 4.72 6 5 6s.53.11.71.29L9 9.59l1.68-1.69zM15 3c.55 0 1 .45 1 1s-.45 1-1 1h-1v1c0 .55-.45 1-1 1s-1-.45-1-1V5h-1c-.55 0-1-.45-1-1s.45-1 1-1h1V2c0-.55.45-1 1-1s1 .45 1 1v1h1z"],"series-configuration":["M9.94 9.64c.65.23 1.34.36 2.06.36.14 0 .29-.01.43-.01L9.7 12.71c-.18.18-.43.29-.71.29-.28 0-.53-.11-.71-.3L5 9.41l-3 3V14h12.99c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V5c0-.55.45-1 1-1s1 .45 1 1v4.59l2.29-2.3C4.47 7.11 4.72 7 5 7c.28 0 .53.11.71.29L9 10.59l.94-.95zm4.73-6.44h.92c.22 0 .4.18.4.4v.8c0 .22-.18.4-.4.4h-.93c-.06.2-.14.38-.24.55l.66.65c.15.15.15.4 0 .55l-.54.55c-.15.15-.4.15-.55 0l-.65-.65c-.17.1-.36.18-.55.24v.91c0 .22-.18.4-.4.4h-.8c-.22 0-.4-.18-.4-.4v-.93c-.18-.06-.36-.13-.52-.22l-.68.68c-.15.16-.41.16-.57 0l-.56-.56a.417.417 0 0 1 0-.57l.68-.68c-.08-.16-.16-.33-.22-.52h-.93c-.22 0-.4-.18-.4-.4v-.8c0-.22.18-.4.4-.4h.93c.06-.2.14-.38.24-.55l-.65-.64a.392.392 0 0 1 0-.55l.54-.55a.38.38 0 0 1 .54 0l.65.65c.18-.1.36-.18.55-.24V.4c0-.22.18-.4.4-.4h.8c.22 0 .4.18.4.4v.93c.18.06.35.14.52.22l.68-.68c.15-.16.41-.16.57 0l.57.57c.15.16.15.41 0 .57l-.68.68c.09.16.16.33.22.51zm-4.18.8c0 .83.67 1.5 1.5 1.5s1.5-.67 1.5-1.5-.67-1.5-1.5-1.5c-.82 0-1.5.67-1.5 1.5z"],"series-derived":["M10.66 7.92c.44.54 1.07.91 1.8 1.03L9.71 11.7c-.18.19-.43.3-.71.3s-.53-.11-.71-.3L5 8.41l-3 3V13h13c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1s1 .45 1 1v4.59l2.29-2.3C4.47 6.11 4.72 6 5 6s.53.11.71.29L9 9.59l1.66-1.67zM12.3 5.3l.3-.3H8c-.6 0-1-.4-1-1s.4-1 1-1h4.6l-.3-.3c-.2-.2-.3-.4-.3-.7 0-.6.5-1 1-1 .3 0 .5.1.7.3l2 2c.2.2.3.4.3.7s-.1.5-.3.7l-2 2c-.2.2-.4.3-.7.3-.6 0-1-.4-1-1 0-.3.1-.5.3-.7z"],"series-filtered":["M9.29 9.3c.3.62.8 1.12 1.42 1.41l-1 1c-.18.18-.43.29-.71.29s-.53-.11-.71-.3L5 8.41l-3 3V13h13c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1s1 .45 1 1v4.59l2.29-2.3C4.47 6.11 4.72 6 5 6s.53.11.71.29L9 9.59l.29-.29zM15.48 1c.31 0 .52.26.52.57 0 .16-.06.3-.17.41l-2.86 2.73v2.63c0 .16-.06.3-.17.41l-.82 1.1c-.1.1-.25.17-.41.17-.31 0-.57-.26-.57-.57V4.71L8.17 1.98A.566.566 0 0 1 8 1.57c0-.31.26-.57.57-.57h6.91z"],"series-search":["M9.6 8.94a4.937 4.937 0 0 0 1.82.01c.1-.01.22-.04.39-.08l.23-.07c.04-.01.08-.02.11-.04l.22.22-2.7 2.72c-.18.19-.43.3-.71.3s-.53-.11-.71-.3L4.98 8.41l-2.99 3V13h12.94c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V3.99c0-.55.45-1 1-1s1 .45 1 1v4.59l2.28-2.3c.17-.18.42-.29.7-.29s.53.11.7.29l3.28 3.3.64-.64zm6.22-.41c.1.12.17.27.18.44 0 .34-.27.61-.61.61a.57.57 0 0 1-.43-.18l-2.24-2.25c-.13.08-.26.16-.4.23-.02.01-.05.02-.07.03-.14.06-.27.12-.42.17h-.01c-.14.05-.29.08-.44.11-.04.01-.08.02-.11.02-.15.02-.3.04-.46.04-1.85 0-3.35-1.51-3.35-3.37S8.96 1.01 10.81 1c1.85 0 3.35 1.51 3.35 3.37 0 .16-.02.31-.04.47-.01.04-.01.07-.02.11-.02.15-.05.29-.1.44v.01c-.05.15-.11.28-.17.42-.01.02-.02.05-.03.07-.07.14-.14.27-.23.4l2.25 2.24zm-5.01-1.94c1.22 0 2.21-.99 2.21-2.22 0-1.23-.99-2.22-2.21-2.22S8.6 3.14 8.6 4.37c0 1.22.99 2.22 2.21 2.22z"],settings:["M3 1c0-.55-.45-1-1-1S1 .45 1 1v3h2V1zm0 4H1c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm12-4c0-.55-.45-1-1-1s-1 .45-1 1v2h2V1zM9 1c0-.55-.45-1-1-1S7 .45 7 1v6h2V1zM1 15c0 .55.45 1 1 1s1-.45 1-1v-5H1v5zM15 4h-2c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm-2 11c0 .55.45 1 1 1s1-.45 1-1V9h-2v6zM9 8H7c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1zm-2 7c0 .55.45 1 1 1s1-.45 1-1v-2H7v2z"],share:["M10.99 13.99h-9v-9h4.76l2-2H.99c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h11c.55 0 1-.45 1-1V7.24l-2 2v4.75zm4-14h-5c-.55 0-1 .45-1 1s.45 1 1 1h2.59L7.29 7.28a1 1 0 0 0-.3.71 1.003 1.003 0 0 0 1.71.71l5.29-5.29V6c0 .55.45 1 1 1s1-.45 1-1V1c0-.56-.45-1.01-1-1.01z"],shield:["M8 16c4.667-3.048 7-7.238 7-12.571-1.556 0-3.889-1.143-7-3.429-3.111 2.286-5.444 3.429-7 3.429C1 8.762 3.333 12.952 8 16zM8 2.121c2.005 1.388 3.715 2.304 5.186 2.735-.342 3.702-2.05 6.683-5.186 9.038V2.121z"],shop:["M3 2h10c.55 0 1-.45 1-1s-.45-1-1-1H3c-.55 0-1 .45-1 1s.45 1 1 1zm9 11H4v-3H2v5c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-5h-2v3zm4-6l-1.01-3.17C14.9 3.36 14.49 3 14 3H2c-.49 0-.9.36-.98.83L.01 7H0c0 1.1.9 2 2 2s2-.9 2-2c0 1.1.9 2 2 2s2-.9 2-2c0 1.1.9 2 2 2s2-.9 2-2c0 1.1.9 2 2 2s2-.9 2-2z"],"shopping-cart":["M14 10H7.72l-.33-1H13c.39 0 .72-.23.89-.56h.01l2-4h-.01c.06-.13.11-.28.11-.44 0-.55-.45-1-1-1H5.39l-.44-1.32h-.01C4.8 1.29 4.44 1 4 1H1c-.55 0-1 .45-1 1s.45 1 1 1h2.28l2.33 7H4c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2h6c0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2zM6.05 5h7.33l-1 2H6.72l-.67-2z"],"sim-card":["M13.71 4.29l-4-4A.997.997 0 0 0 9 0H3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V5c0-.28-.11-.53-.29-.71zM7 6h2v2H7V6zM4 6h2v2H4V6zm2 8H4v-2h2v2zm3 0H7v-2h2v2zm3 0h-2v-2h2v2zm0-3H4V9h8v2zm0-3h-2V6h2v2z"],slash:["M10 2a.99.99 0 0 0-.96.73l-2.99 9.96A1.003 1.003 0 0 0 7 14c.46 0 .85-.31.96-.73l2.99-9.96A1.003 1.003 0 0 0 10 2z"],"small-cross":["M9.41 8l2.29-2.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L8 6.59l-2.29-2.3a1.003 1.003 0 0 0-1.42 1.42L6.59 8 4.3 10.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L8 9.41l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L9.41 8z"],"small-minus":["M11 7H5c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1z"],"small-plus":["M11 7H9V5c0-.55-.45-1-1-1s-1 .45-1 1v2H5c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V9h2c.55 0 1-.45 1-1s-.45-1-1-1z"],"small-tick":["M12 5c-.28 0-.53.11-.71.29L7 9.59l-2.29-2.3a1.003 1.003 0 0 0-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l5-5A1.003 1.003 0 0 0 12 5z"],snowflake:["M13.364 9l.879.879a1 1 0 1 1-1.415 1.414l-2.12-2.121A1.003 1.003 0 0 1 10.568 9H9v1.604c.042.03.083.065.121.103l2.122 2.121a1 1 0 0 1-1.415 1.415L9 13.414V15a1 1 0 0 1-2 0v-1.636l-.879.879a1 1 0 1 1-1.414-1.415l2.121-2.12c.054-.054.111-.1.172-.139V9H5.38c-.038.06-.084.118-.137.172l-2.122 2.12A1 1 0 1 1 1.707 9.88L2.586 9H1a1 1 0 1 1 0-2h1.536l-.829-.828a1 1 0 0 1 1.414-1.415L5.243 6.88c.038.038.072.079.103.121H7V5.38a1.003 1.003 0 0 1-.172-.137L4.708 3.12A1 1 0 0 1 6.12 1.707L7 2.586V1a1 1 0 1 1 2 0v1.536l.828-.829a1 1 0 0 1 1.415 1.414L9.12 5.243A1.007 1.007 0 0 1 9 5.346V7h1.604c.03-.042.065-.083.103-.121l2.121-2.122a1 1 0 0 1 1.415 1.415L13.414 7H15a1 1 0 0 1 0 2h-1.636z"],"social-media":["M9.5 4c.4 0 .8-.1 1.1-.3C12 4.5 12.9 6 13 7.6c0 .5.5.9 1 .9.6 0 1-.4 1-1v-.2c-.2-2.4-1.5-4.4-3.5-5.5-.1-1-.9-1.8-2-1.8s-2 .9-2 2 .9 2 2 2zM4 8.5c0-.7-.4-1.3-.9-1.7.3-1.4 1.2-2.6 2.5-3.3.3-.1.6-.4.6-.9s-.4-1-1-1c-.2 0-.3 0-.5.1-1.9 1-3.2 2.8-3.6 5C.4 7.1 0 7.8 0 8.5c0 1.1.9 2 2 2s2-.9 2-2zm8.8 1.2c-1.1 0-2 .9-2 2v.3c-.8.6-1.8.9-2.8.9-1.2 0-2.3-.4-3.2-1.1-.2-.2-.4-.3-.7-.3-.6 0-1 .4-1 1 0 .3.1.6.3.8C4.6 14.4 6.2 15 8 15c1.5 0 3-.5 4.1-1.3.2.1.5.1.7.1 1.1 0 2-.9 2-2s-.9-2.1-2-2.1z"],sort:["M5 12c-.28 0-.53.11-.71.29l-.29.3V9c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29A.965.965 0 0 0 1 12a1.003 1.003 0 0 0-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2A1.003 1.003 0 0 0 5 12zm3-9h7c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zm7 2H8c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1zm0 8H8c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1zm0-4H8c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1z"],"sort-alphabetical":["M6 12c-.28 0-.53.11-.71.29l-.29.3V9c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29A.965.965 0 0 0 2 12a1.003 1.003 0 0 0-.71 1.71l2 2c.19.18.44.29.71.29.28 0 .53-.11.71-.29l2-2c.18-.18.29-.43.29-.71a.99.99 0 0 0-1-1zm7.93-.95v-1.04H9.25v1.11h2.94L9 14.96V16h5.02v-1.11h-3.27l3.18-3.84zm-1.42-4.84l.62 1.78H15L11.94.01H10.1L7 7.99h1.81l.64-1.78h3.06zm-1.52-4.24h.02l1.03 2.93H9.92l1.07-2.93z"],"sort-alphabetical-desc":["M5.99 11.99c-.28 0-.53.11-.71.29l-.29.29V8.99c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29a1.003 1.003 0 0 0-1.42 1.42l2 2c.18.18.43.29.71.29.28 0 .53-.11.71-.29l2-2c.18-.18.29-.43.29-.71 0-.56-.45-1.01-1-1.01zM12.7 10h-1.38L9 15.99h1.36l.48-1.33h2.3l.46 1.33H15L12.7 10zm-1.51 3.67l.8-2.2h.02l.77 2.2h-1.59zm3.8-7.17h-4.57l4.45-5.12V0H8.34v1.48h4.1L7.99 6.59v1.39h7V6.5z"],"sort-asc":["M8 7h3c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zm0-4h1c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zm0 8h5c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zm-3 1c-.28 0-.53.11-.71.29l-.29.3V9c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29A.965.965 0 0 0 1 12a1.003 1.003 0 0 0-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2A1.003 1.003 0 0 0 5 12zm10 1H8c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1z"],"sort-desc":["M5 12c-.28 0-.53.11-.71.29l-.29.3V9c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29A.965.965 0 0 0 1 12a1.003 1.003 0 0 0-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2A1.003 1.003 0 0 0 5 12zm4 1H8c-.55 0-1 .45-1 1s.45 1 1 1h1c.55 0 1-.45 1-1s-.45-1-1-1zm4-8H8c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm-2 4H8c-.55 0-1 .45-1 1s.45 1 1 1h3c.55 0 1-.45 1-1s-.45-1-1-1zm4-8H8c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1z"],"sort-numerical":["M6 11.99c-.28 0-.53.11-.71.29l-.29.3V8.99c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29c-.18-.18-.43-.3-.71-.3a1.003 1.003 0 0 0-.71 1.71l2 2c.18.18.43.29.71.29.28 0 .53-.11.71-.29l2-2A1.003 1.003 0 0 0 6 11.99zm7.91-.08c-.06-.36-.17-.68-.33-.96-.16-.28-.37-.51-.64-.69-.27-.17-.61-.26-1.03-.26-.28 0-.54.06-.78.17-.23.11-.43.26-.6.45-.17.19-.3.41-.39.67a2.492 2.492 0 0 0-.04 1.52 1.623 1.623 0 0 0 .89 1.03c.22.11.45.16.68.16.26 0 .5-.05.7-.15s.38-.26.53-.5l.02.02c-.01.16-.03.34-.07.54-.03.2-.09.4-.17.57-.08.18-.18.33-.31.45s-.29.19-.5.19a.63.63 0 0 1-.48-.21c-.13-.14-.21-.31-.25-.5H10.1c.03.25.1.48.19.68.1.2.22.37.38.5.16.14.33.24.54.31s.42.1.65.1c.39 0 .72-.09.99-.27.27-.18.49-.41.66-.7.17-.29.29-.61.37-.97.08-.36.12-.72.12-1.07 0-.36-.03-.72-.09-1.08zm-1.14.54c-.04.13-.09.24-.16.34a.78.78 0 0 1-.27.24c-.11.06-.24.09-.39.09a.75.75 0 0 1-.37-.09.777.777 0 0 1-.26-.25c-.07-.1-.12-.22-.15-.35-.03-.13-.05-.26-.05-.4 0-.13.02-.26.05-.39.04-.13.09-.24.16-.34.07-.1.16-.18.26-.24s.22-.09.35-.09c.14 0 .26.03.37.09.11.06.2.14.28.24a1.32 1.32 0 0 1 .23.74c0 .15-.02.28-.05.41zm-1.56-4.47H13V0h-1.42c-.05.3-.16.56-.31.76-.16.21-.35.37-.58.5-.23.13-.49.21-.78.26-.3.05-.6.07-.91.06V2.8h2.21v5.18z"],"sort-numerical-desc":["M6 11.99c-.28 0-.53.11-.71.29l-.29.3V8.99c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29a.982.982 0 0 0-.71-.3 1.003 1.003 0 0 0-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2A1.003 1.003 0 0 0 6 11.99zm7.86-9.45c-.09-.48-.26-.9-.5-1.28S12.8.58 12.4.35C12 .12 11.49 0 10.86 0c-.43 0-.82.07-1.17.22s-.65.35-.9.6-.44.55-.58.89c-.14.34-.2.71-.2 1.11 0 .31.05.61.15.91.1.3.26.57.48.8.23.24.52.43.85.58.33.14.68.21 1.03.21.4 0 .75-.07 1.05-.2.3-.13.57-.35.79-.66l.02.02c-.02.21-.05.45-.1.73-.05.27-.13.53-.25.76-.12.24-.27.44-.47.6-.19.16-.44.25-.75.25a.98.98 0 0 1-.72-.29c-.19-.18-.31-.4-.37-.66H8.15c.05.34.14.64.29.9.15.26.34.49.57.67.23.18.5.32.8.41.31.1.63.15.98.15.58 0 1.08-.12 1.48-.36.4-.24.73-.55.99-.93.26-.39.44-.82.56-1.29.12-.48.18-.96.18-1.44s-.05-.96-.14-1.44zm-1.71.72c-.05.17-.14.32-.24.46-.11.13-.24.24-.41.31-.16.08-.36.12-.58.12-.21 0-.39-.04-.55-.13-.16-.08-.29-.19-.39-.33-.12-.14-.19-.29-.24-.46-.05-.17-.08-.35-.08-.54 0-.18.03-.35.08-.52.06-.16.14-.31.25-.44.11-.13.24-.24.4-.32.16-.08.33-.12.52-.12.21 0 .4.04.56.12.16.08.3.19.41.32.11.14.2.29.26.46.06.17.09.35.09.52 0 .2-.03.38-.08.55zm-.46 7.31c-.12.15-.26.28-.44.37-.17.09-.37.16-.58.2-.22.04-.44.05-.67.05v.92h1.65v3.88h1.33V10h-1.06c-.03.23-.11.42-.23.57z"],"split-columns":["M12 10a1.003 1.003 0 0 0 1.71.71l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-2-2a1.003 1.003 0 0 0-1.42 1.42l.3.29H9V2h3v1.71c.31-.13.64-.21 1-.21s.69.08 1 .21V1c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v2.71c.31-.13.64-.21 1-.21s.69.08 1 .21V2h3v5H3.41l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-2 2C.11 7.47 0 7.72 0 8c0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42L3.41 9H7v5H4v-1.71c-.31.13-.64.21-1 .21s-.69-.08-1-.21V15c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-2.71c-.31.13-.64.21-1 .21s-.69-.08-1-.21V14H9V9h3.59l-.29.29c-.19.18-.3.43-.3.71z"],square:["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H2V2h12v12z"],"stacked-chart":["M10 2c0-.55-.45-1-1-1H8c-.55 0-1 .45-1 1v3h3V2zm3 10h1c.55 0 1-.45 1-1V8h-3v3c0 .55.45 1 1 1zm2-7c0-.55-.45-1-1-1h-1c-.55 0-1 .45-1 1v2h3V5zm-5 1H7v3h3V6zM5 7c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v1h3V7zm3 5h1c.55 0 1-.45 1-1v-1H7v1c0 .55.45 1 1 1zm7 1H2c-.55 0-1 .45-1 1s.45 1 1 1h13c.55 0 1-.45 1-1s-.45-1-1-1zM3 12h1c.55 0 1-.45 1-1V9H2v2c0 .55.45 1 1 1z"],star:["M8 0l2.5 5.3 5.5.8-4 4.1.9 5.8L8 13.3 3.1 16l.9-5.8-4-4.1 5.5-.8z"],"star-empty":["M16 6.11l-5.53-.84L8 0 5.53 5.27 0 6.11l4 4.1L3.06 16 8 13.27 12.94 16 12 10.21l4-4.1zM4.91 13.2l.59-3.62L3 7.02l3.45-.53L8 3.2l1.55 3.29 3.45.53-2.5 2.56.59 3.62L8 11.49 4.91 13.2z"],"step-backward":["M12 3c-.24 0-.44.09-.62.23l-.01-.01L7 6.72V4c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V9.28l4.38 3.5.01-.01c.17.14.37.23.61.23.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"step-chart":["M15 12H2v-2h3c.55 0 1-.45 1-1V7h2v1c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V5h1c.55 0 1-.45 1-1s-.45-1-1-1h-2c-.55 0-1 .45-1 1v3h-2V6c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v2H2V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"step-forward":["M12 3h-1c-.55 0-1 .45-1 1v2.72l-4.38-3.5v.01A.987.987 0 0 0 5 3c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1 .24 0 .44-.09.62-.23l.01.01L10 9.28V12c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],stop:["M12 3H4c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h8c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],strikethrough:["M14 7H8.65c-.38-.09-.73-.18-1.04-.26-.31-.08-.49-.13-.54-.14-.43-.11-.79-.29-1.05-.52-.27-.23-.4-.55-.4-.95 0-.29.07-.53.21-.72s.32-.34.54-.46c.22-.11.46-.19.72-.24.26-.05.52-.07.77-.07.74 0 1.36.15 1.84.46.32.2.55.5.68.9h2.22c-.06-.33-.17-.64-.32-.92-.25-.45-.59-.84-1.02-1.15-.43-.31-.93-.54-1.49-.7S8.59 2 7.95 2c-.55 0-1.1.07-1.63.2-.54.13-1.02.34-1.45.62-.42.28-.76.63-1.02 1.05-.26.42-.39.92-.39 1.5 0 .3.04.59.13.88.08.26.21.51.39.75H2c-.55 0-1 .45-1 1s.45 1 1 1h7.13c.25.07.49.14.71.22.25.09.48.23.7.44.21.21.32.53.32.97 0 .21-.05.43-.14.63-.09.21-.24.39-.45.55-.21.16-.48.29-.81.39-.33.1-.73.15-1.2.15-.44 0-.84-.05-1.21-.14-.37-.09-.7-.24-.99-.43-.29-.2-.51-.45-.67-.76-.01 0-.01-.01-.02-.02H3.14a3.68 3.68 0 0 0 1.39 2.03c.46.34 1 .58 1.62.74.61.15 1.27.23 1.97.23.61 0 1.2-.07 1.79-.2.58-.13 1.11-.34 1.56-.63.46-.29.83-.66 1.11-1.11.28-.45.42-1 .42-1.64 0-.3-.05-.6-.15-.9-.05-.19-.13-.36-.22-.52H14c.55 0 1-.45 1-1s-.45-1-1-1z"],style:["M14 14H2V2h8.76l2-2H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V6.24l-2 2V14zm1.4-14L9.7 5.7l2.1 2.1L16 3.6V0h-.6zM4 11.92c2.33.15 4.42.15 6.15-1.5.82-.83.82-2.25 0-3.08-.45-.38-.98-.6-1.5-.6-.53 0-1.05.22-1.43.6-.82.91-1.27 3.38-3.22 4.58z"],"swap-horizontal":["M0 7.02L.05 7H0v.02zm2-2.03h9.57l-1.29 1.29A1.003 1.003 0 0 0 11.7 7.7l2.99-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-2.99-3a1.07 1.07 0 0 0-.71-.28 1.003 1.003 0 0 0-.71 1.71L11.57 3H2c-.55 0-1 .45-1 1a1 1 0 0 0 1 .99zM15.96 9H16v-.02l-.04.02zM14 11.01H4.43l1.29-1.29A1.003 1.003 0 0 0 4.3 8.3l-2.99 3a.99.99 0 0 0-.29.7c0 .28.11.53.29.71l2.99 3a1.003 1.003 0 0 0 1.42-1.42L4.43 13H14c.55 0 1-.45 1-1s-.45-.99-1-.99z"],"swap-vertical":["M9 0h-.02L9 .04V0zM7 16h.02L7 15.95V16zM4.7 1.31c-.18-.18-.43-.29-.7-.29s-.53.11-.71.29l-3 2.99a1.003 1.003 0 0 0 1.42 1.42L3 4.43V14c0 .55.45 1 1 1s1-.45 1-1V4.43l1.29 1.29c.18.18.43.29.7.29A1.003 1.003 0 0 0 7.7 4.3l-3-2.99zM15 9.99c-.28 0-.53.11-.71.29L13 11.57V2c0-.55-.45-1-1-1s-1 .45-1 1v9.57l-1.29-1.29a.99.99 0 0 0-.7-.29 1.003 1.003 0 0 0-.71 1.71l3 2.99c.18.18.43.29.71.29.28 0 .53-.11.71-.29l3-2.99c.18-.18.29-.43.29-.71-.01-.55-.46-1-1.01-1z"],"symbol-circle":["M8 3.01a5 5 0 1 0 0 10 5 5 0 1 0 0-10z"],"symbol-cross":["M12 6.01h-2v-2c0-.55-.45-1-1-1H7c-.55 0-1 .45-1 1v2H4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h2v2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-2h2c.55 0 1-.45 1-1v-2c0-.56-.45-1-1-1z"],"symbol-diamond":["M12 8.01c0-.19-.07-.36-.16-.51l.01-.01-3-5-.01.01c-.17-.29-.48-.49-.84-.49s-.67.2-.84.49l-.02-.01-3 5 .02.01c-.09.15-.16.32-.16.51s.07.36.16.51h-.02l3 5 .01-.01c.18.29.49.5.85.5s.67-.2.84-.49l.01.01 3-5-.01-.01c.09-.16.16-.32.16-.51z"],"symbol-square":["M12 3.01H4c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h8c.55 0 1-.45 1-1v-8c0-.56-.45-1-1-1z"],"symbol-triangle-down":["M13 4.01c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1 0 .16.05.31.11.44H3.1l4 8h.01c.16.33.49.56.89.56s.72-.23.89-.56h.01l4-8h-.01c.06-.14.11-.28.11-.44z"],"symbol-triangle-up":["M12.89 11.56l-3.99-8h-.01c-.17-.32-.5-.55-.89-.55s-.72.23-.89.55H7.1l-4 8h.01c-.06.14-.11.29-.11.45 0 .55.45 1 1 1h8c.55 0 1-.45 1-1 0-.16-.05-.31-.11-.45z"],tag:["M1 3a2 2 0 0 1 2-2h4.584a2 2 0 0 1 1.414.586l5.413 5.412a2 2 0 0 1 0 2.829L9.827 14.41a2 2 0 0 1-2.829 0L1.586 8.998A2 2 0 0 1 1 7.584V3zm3.487-.007a1.494 1.494 0 1 0 0 2.988 1.494 1.494 0 0 0 0-2.988z"],"take-action":["M9 11a1.003 1.003 0 0 0 1.71.71l4-4a1.003 1.003 0 0 0-1.42-1.42l-4 4c-.18.18-.29.43-.29.71zM4 6c.28 0 .53-.11.71-.29l4-4A1.003 1.003 0 0 0 7.29.29l-4 4A1.003 1.003 0 0 0 4 6zm4 4l5-5-.79-.79.5-.5a1.003 1.003 0 0 0-1.42-1.42l-.5.5L10 2 5 7l.79.79-5.5 5.5a1.003 1.003 0 0 0 1.42 1.42l5.5-5.5L8 10zm7 4H7c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1z"],taxi:["M15.12 6.63h-.38L15 7c-.01.3-.01.64 0 .98V8c0 .07-.03.13-.04.19h.02L14 13.1v.9c0 .55-.45 1-1 1s-1-.45-1-1v-1H4v1c0 .55-.45 1-1 1s-1-.45-1-1v-.9l-.98-4.9h.02C1.03 8.13 1 8.07 1 8H.99c0-.33 0-.67.01-1l.26-.37H.88C.4 6.63 0 6.21 0 5.69s.4-.94.88-.94h1.05l.77-2.11c.19-.53.76-1.08 1.26-1.24 0 0 .68-.2 2.05-.32C6.01 1.05 6 1.03 6 1c0-.55.45-1 1-1h2c.55 0 1 .45 1 1 0 .03-.01.05-.02.08 1.37.12 2.05.32 2.05.32.51.15 1.08.71 1.27 1.24l.76 2.12h1.05c.49 0 .89.42.89.93 0 .52-.4.94-.88.94zM11 10h2V8h-2v2zm-8 0h2V8H3v2zm10-5l-.73-1.63C12.21 3.19 12.18 3 12 3H4c-.18 0-.21.19-.27.37L3 5c-.06.18-.18 1 0 1h10c.18 0 .06-.82 0-1z"],"text-highlight":["M9 10H2V6h7V4H1c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h8v-2zm4 3h-1V3h1c.55 0 1-.45 1-1s-.45-1-1-1h-1c-.37 0-.7.11-1 .28-.3-.17-.63-.28-1-.28H9c-.55 0-1 .45-1 1s.45 1 1 1h1v10H9c-.55 0-1 .45-1 1s.45 1 1 1h1c.37 0 .7-.11 1-.28.3.17.63.28 1 .28h1c.55 0 1-.45 1-1s-.45-1-1-1zm2-9h-2v2h1v4h-1v2h2c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1z"],th:["M15 1H1c-.6 0-1 .5-1 1v12c0 .6.4 1 1 1h14c.6 0 1-.4 1-1V2c0-.5-.4-1-1-1zM6 13H2v-2h4v2zm0-3H2V8h4v2zm0-3H2V5h4v2zm8 6H7v-2h7v2zm0-3H7V8h7v2zm0-3H7V5h7v2z"],"th-derived":["M5.6 10l-.3.3c-.2.2-.3.4-.3.7 0 .6.4 1 1 1 .3 0 .5-.1.7-.3l2-2c.2-.2.3-.4.3-.7s-.1-.5-.3-.7l-2-2C6.5 6.1 6.3 6 6 6c-.5 0-1 .4-1 1 0 .3.1.5.3.7l.3.3H1c-.6 0-1 .4-1 1s.4 1 1 1h4.6zM15 1H2c-.5 0-1 .5-1 1v5h2V5h11v2H8.8l.6.6c.1.1.2.3.3.4H14v2H9.7c-.1.1-.2.3-.3.4l-.6.6H14v2H3v-2H1v3c0 .5.5 1 1 1h13c.6 0 1-.5 1-1V2c0-.5-.4-1-1-1z"],"th-disconnect":["M12 1h3c.6 0 1 .5 1 1v12c0 .6-.4 1-1 1h-4.97l.286-2H14v-2h-3.398l.143-1H14V8h-2.97l.143-1H14V5h-2.541l.51-3.576C11.99 1.282 12 1.14 12 1zM5.97 1l-.572 4H2v2h3.112L4.97 8H2v2h2.684l-.143 1H2v2h2.255l-.225 1.576c-.02.142-.03.284-.03.424H1c-.6 0-1-.4-1-1V2c0-.5.4-1 1-1h4.97zM8.01.859a1 1 0 1 1 1.98.282l-2 14a1 1 0 1 1-1.98-.282l2-14z"],"th-filtered":["M10 10h3l1.78-2.226a1 1 0 0 0 .22-.625V4.3l1-.9V14c0 .6-.4 1-1 1H1c-.6 0-1-.4-1-1V2c0-.5.4-1 1-1h4.333L9 4.3V5H7v2h2v1H7v2h3zm-4 3v-2H2v2h4zm0-3V8H2v2h4zm0-3V5H2v2h4zm8 6v-2H7v2h7z","M15.48 0c.31 0 .52.26.52.57 0 .16-.06.3-.17.41l-2.86 2.73v2.63c0 .16-.06.3-.17.41l-.82 1.1c-.1.1-.25.17-.41.17-.31 0-.57-.26-.57-.57V3.71L8.17.98A.566.566 0 0 1 8 .57c0-.31.26-.57.57-.57h6.91z"],"th-list":["M15 1H1c-.6 0-1 .5-1 1v12c0 .6.4 1 1 1h14c.6 0 1-.4 1-1V2c0-.5-.4-1-1-1zm-1 12H2v-2h12v2zm0-3H2V8h12v2zm0-3H2V5h12v2z"],"thumbs-down":["M2 2H0v7h2c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm13.99 4.38c.08-.58-.44-1.02-1.15-1.05-.25-.01-.52-.03-.81-.05.02 0 .05-.01.07-.01.7-.1 1.34-.49 1.41-1.07.06-.58-.46-.97-1.17-1.04-.25-.02-.52-.04-.79-.06.47-.15.84-.42.87-.93.04-.58-.79-1.03-1.5-1.09-.27-.02-.51-.04-.73-.05h-.09c-.23-.02-.43-.02-.62-.03C8.35.95 5.66 1.47 4 2.51v6c2.14 1.29 4.76 3.59 4.21 5.51-.18.59.31 1.05.98.98.81-.09 1.37-.91 1.4-1.78.04-1-.15-2.01-.5-2.91-.04-.25.01-.5.37-.53.49-.03 1.11-.06 1.59-.08.26 0 .51-.01.75-.02h.01c.41-.02.8-.05 1.13-.09.7-.09 1.35-.47 1.43-1.05.08-.58-.44-.97-1.15-1.05-.05-.01-.11-.01-.16-.02.17-.01.33-.03.49-.05.72-.08 1.37-.46 1.44-1.04z"],"thumbs-up":["M15.99 9.62c-.08-.58-.73-.96-1.43-1.05-.15-.02-.32-.04-.49-.05.06-.01.11-.01.16-.02.71-.08 1.23-.47 1.15-1.05-.08-.58-.73-.96-1.43-1.05-.34-.04-.72-.07-1.13-.09h-.01c-.24-.01-.49-.02-.75-.02-.48-.02-1.11-.04-1.59-.08-.36-.03-.41-.28-.37-.53.35-.9.54-1.91.5-2.91-.04-.85-.6-1.68-1.41-1.77-.67-.07-1.16.39-.99.98C8.76 3.91 6.13 6.2 4 7.49v6c1.66 1.03 4.35 1.56 7.48 1.5.19 0 .39-.01.62-.02h.09c.22-.01.46-.03.73-.05.71-.06 1.54-.51 1.5-1.09-.03-.51-.4-.79-.87-.93.27-.02.54-.04.79-.06.71-.06 1.24-.45 1.17-1.04-.06-.58-.7-.97-1.41-1.07-.02 0-.05-.01-.07-.01.29-.02.57-.03.81-.05.71-.03 1.23-.47 1.15-1.05zM2 7H0v7h2c.55 0 1-.45 1-1V8c0-.56-.45-1-1-1z"],tick:["M14 3c-.28 0-.53.11-.71.29L6 10.59l-3.29-3.3a1.003 1.003 0 0 0-1.42 1.42l4 4c.18.18.43.29.71.29s.53-.11.71-.29l8-8A1.003 1.003 0 0 0 14 3z"],"tick-circle":["M8 16c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8zm4-11c-.28 0-.53.11-.71.29L7 9.59l-2.29-2.3a1.003 1.003 0 0 0-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l5-5A1.003 1.003 0 0 0 12 5z"],time:["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6zm1-6.41V4c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42L9 7.59z"],"timeline-area-chart":["M15 2.59L9.91 7.68 6.6 5.2l-.01.01C6.42 5.09 6.23 5 6 5c-.24 0-.44.09-.62.23v-.01L3 7.12V11h12V2.59zM15 12H2V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"timeline-bar-chart":["M8 12h1c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1H8c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1zm5 0h1c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1h-1c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1zm2 1H2c-.55 0-1 .45-1 1s.45 1 1 1h13c.55 0 1-.45 1-1s-.45-1-1-1zM3 12h1c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1z"],"timeline-events":["M8 11H7v1h1v-1zm-4 0H3v1h1v-1zm7-8c.6 0 1-.5 1-1V1c0-.5-.4-1-1-1s-1 .5-1 1v1c0 .5.5 1 1 1zM4 3c.5 0 1-.5 1-1V1c0-.5-.5-1-1-1S3 .5 3 1v1c0 .5.5 1 1 1zm10-2h-1v1c0 1.1-.9 2-2 2s-2-.9-2-2V1H6v1c0 1.1-.9 2-2 2s-2-.9-2-2V1H1c-.5 0-1 .5-1 1v12c0 .5.5 1 1 1h13c.6 0 1-.5 1-1V2c0-.5-.4-1-1-1zM5 13H2v-3h3v3zm0-4H2V6h3v3zm4 4H6v-3h3v3zm0-4H6V6h3v3zm4 4h-3v-3h3v3zm0-4h-3V6h3v3zm-1-2h-1v1h1V7z"],"timeline-line-chart":["M15 12H2V9.41l3-3L8.29 9.7c.18.19.43.3.71.3s.53-.11.71-.29l6-6a1.003 1.003 0 0 0-1.42-1.42L9 7.59l-3.29-3.3C5.53 4.11 5.28 4 5 4s-.53.11-.71.29L2 6.59V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],tint:["M7.88 1s-4.9 6.28-4.9 8.9c.01 2.82 2.34 5.1 4.99 5.1 2.65-.01 5.03-2.3 5.03-5.13C12.99 7.17 7.88 1 7.88 1z"],torch:["M5 15c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-1H5v1zm7-15H4c-.55 0-1 .45-1 1v1h10V1c0-.55-.45-1-1-1zM5 7v6h6V7l2-4H3l2 4zm2 0c0-.55.45-1 1-1s1 .45 1 1v2c0 .55-.45 1-1 1s-1-.45-1-1V7z"],train:["M13 14h-1l1 2H3l1-2H3c-1.1 0-2-.9-2-2V2C1 .9 4.13 0 8 0s7 .9 7 2v10c0 1.1-.9 2-2 2zm-2-2h2v-2h-2v2zM9 7h4V3H9v4zm-6 5h2v-2H3v2zm0-5h4V3H3v4z"],translate:["M15.89 14.56l-3.99-8h-.01c-.17-.33-.5-.56-.89-.56s-.72.23-.89.56h-.01L9 8.76 7.17 7.38l.23-.18C8.37 6.47 9 5.31 9 4V3h1c.55 0 1-.45 1-1s-.45-1-1-1H7c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1H1c-.55 0-1 .45-1 1s.45 1 1 1h6v1c0 .66-.32 1.25-.82 1.61l-.68.51-.68-.5C4.32 5.25 4 4.66 4 4H2c0 1.31.63 2.47 1.6 3.2l.23.17L1.4 9.2l.01.01C1.17 9.4 1 9.67 1 10c0 .55.45 1 1 1 .23 0 .42-.09.59-.21l.01.01 2.9-2.17 2.6 1.95-1.99 3.98h.01c-.07.13-.12.28-.12.44 0 .55.45 1 1 1 .39 0 .72-.23.89-.56h.01L8.62 14h4.76l.72 1.45h.01c.17.32.5.55.89.55.55 0 1-.45 1-1 0-.16-.05-.31-.11-.44zM9.62 12L11 9.24 12.38 12H9.62z"],trash:["M14.49 3.99h-13c-.28 0-.5.22-.5.5s.22.5.5.5h.5v10c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-10h.5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5zm-8.5 9c0 .55-.45 1-1 1s-1-.45-1-1v-6c0-.55.45-1 1-1s1 .45 1 1v6zm3 0c0 .55-.45 1-1 1s-1-.45-1-1v-6c0-.55.45-1 1-1s1 .45 1 1v6zm3 0c0 .55-.45 1-1 1s-1-.45-1-1v-6c0-.55.45-1 1-1s1 .45 1 1v6zm2-12h-4c0-.55-.45-1-1-1h-2c-.55 0-1 .45-1 1h-4c-.55 0-1 .45-1 1v1h14v-1c0-.55-.45-1-1-1z"],tree:["M9 11.857V16H7v-4.143L1 13l3.885-4.44L3 9l3.07-4.297L5 5l3-5 3 5-1.07-.297L13 9l-1.885-.44L15 13l-6-1.143z"],"trending-down":["M15 7c-.55 0-1 .45-1 1v.59l-4.29-4.3A.997.997 0 0 0 9 4c-.16 0-.31.05-.44.11V4.1L5 5.88 1.45 4.11v.01C1.31 4.05 1.16 4 1 4c-.55 0-1 .45-1 1 0 .39.23.72.56.89v.01l4 2v-.01c.13.06.28.11.44.11s.31-.05.44-.11v.01L8.8 6.22 12.59 10H12c-.55 0-1 .45-1 1s.45 1 1 1h3c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1z"],"trending-up":["M15 4h-3c-.55 0-1 .45-1 1s.45 1 1 1h.59L8.8 9.78 5.45 8.11v.01C5.31 8.05 5.16 8 5 8s-.31.05-.44.11V8.1l-4 2v.01c-.33.17-.56.5-.56.89 0 .55.45 1 1 1 .16 0 .31-.05.44-.11v.01L5 10.12l3.55 1.78v-.01c.14.06.29.11.45.11.28 0 .53-.11.71-.29L14 7.41V8c0 .55.45 1 1 1s1-.45 1-1V5c0-.55-.45-1-1-1z"],"two-columns":["M3.99-.01h-3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h3c.55 0 1-.45 1-1v-14c0-.55-.45-1-1-1zm11.71 7.3l-2-2a1.003 1.003 0 0 0-1.71.71v4a1.003 1.003 0 0 0 1.71.71l2-2c.18-.18.29-.43.29-.71s-.11-.53-.29-.71zM9.99-.01h-3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h3c.55 0 1-.45 1-1v-14c0-.55-.45-1-1-1z"],underline:["M8 14c2.8 0 5-2.2 5-5V3c0-.6-.4-1-1-1s-1 .4-1 1v6c0 1.7-1.3 3-3 3s-3-1.3-3-3V3c0-.6-.4-1-1-1s-1 .4-1 1v6c0 2.8 2.2 5 5 5zM13.5 15h-11c-.3 0-.5.2-.5.5s.2.5.5.5h11c.3 0 .5-.2.5-.5s-.2-.5-.5-.5z"],undo:["M4 11c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm7-7H3.41L4.7 2.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-3 3C.11 4.47 0 4.72 0 5c0 .28.11.53.29.71l3 3a1.003 1.003 0 0 0 1.42-1.42L3.41 6H11c1.66 0 3 1.34 3 3s-1.34 3-3 3H7v2h4c2.76 0 5-2.24 5-5s-2.24-5-5-5z"],"ungroup-objects":["M3.5 5C1.57 5 0 6.57 0 8.5S1.57 12 3.5 12 7 10.43 7 8.5 5.43 5 3.5 5zm9 0C10.57 5 9 6.57 9 8.5s1.57 3.5 3.5 3.5S16 10.43 16 8.5 14.43 5 12.5 5z"],"unknown-vehicle":["M10.507 9.75v-3.5c0-.089.023-.171.051-.25h-7.55c-.176 0-.061-.824 0-1l.729-1.63c.06-.177.095-.37.27-.37h4.5V1.01c-.166-.003-.32-.01-.5-.01-2.72 0-4.036.402-4.036.402-.508.155-1.079.711-1.268 1.237L1.94 4.756H.887c-.483 0-.88.423-.88.939s.397.939.88.939h.376L1.008 7c-.034.685 0 1.436 0 2v5c0 .657.384 1 1 1s1-.343 1-1v-1h10v1c0 .657.383 1 1 1s1-.343 1-1v-3.5h-3.75a.75.75 0 0 1-.75-.75zm-5.5.25h-2V8h2v2zm11-4.305zM15.34.826a2.807 2.807 0 0 0-.932-.598c-.386-.16-.868-.241-1.445-.241-.447 0-.851.076-1.213.228-.362.153-.67.364-.926.636s-.456.592-.598.963a3.535 3.535 0 0 0-.218 1.144V3h1.789c.003-.208.023-.405.069-.588.049-.193.124-.362.225-.506.102-.144.232-.259.39-.345.159-.087.348-.13.567-.13.325 0 .58.09.762.272.183.18.275.46.275.839.008.222-.031.407-.116.555a1.654 1.654 0 0 1-.335.408 7.4 7.4 0 0 1-.452.37c-.162.123-.316.27-.463.438a2.556 2.556 0 0 0-.384.611c-.11.239-.177.535-.2.889V6h1.645v-.1c.032-.248.111-.453.237-.618.126-.164.27-.31.433-.438.163-.128.335-.255.518-.383a2.413 2.413 0 0 0 .878-1.117c.102-.255.152-.58.152-.975A2.241 2.241 0 0 0 15.34.826zM12.007 7v2h2V7h-2z"],unlock:["M11.99-.01c-2.21 0-4 1.79-4 4v3h-7c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h12c.55 0 1-.45 1-1v-7c0-.55-.45-1-1-1h-3v-3c0-1.1.9-2 2-2s2 .9 2 2v1c0 .55.45 1 1 1s1-.45 1-1v-1c0-2.21-1.79-4-4-4z"],unpin:["M9.39 1c-.5.5-.4 1.48.15 2.53L4.38 7.54C2.85 6.5 1.52 6.07 1 6.59l3.5 3.5c-.02.02-1.4 2.8-1.4 2.8l2.8-1.4 3.5 3.5c.53-.53.1-1.86-.95-3.38l4.02-5.16c1.04.55 2.01.65 2.51.14L9.39 1z"],unresolve:["M11 3c-.55 0-1.07.09-1.57.26a6.46 6.46 0 0 1 0 9.48c.5.17 1.02.26 1.57.26 2.76 0 5-2.24 5-5s-2.24-5-5-5zM9.78 9.38l.09-.27c.08-.36.13-.73.13-1.11s-.05-.75-.13-1.11l-.09-.27a5.32 5.32 0 0 0-.29-.79l-.12-.21c-.14-.27-.31-.52-.51-.76a.7.7 0 0 0-.08-.1c-.24-.27-.49-.52-.78-.74-.43-.32-.92-.58-1.45-.75l.01-.01c-.1-.03-.2-.05-.3-.08-.12-.03-.23-.07-.36-.09A5.28 5.28 0 0 0 5 3C2.24 3 0 5.24 0 8s2.24 5 5 5c.31 0 .61-.04.9-.09.12-.02.24-.06.36-.09.1-.03.21-.04.3-.08l-.01-.01c.88-.29 1.64-.8 2.22-1.49.03-.03.06-.07.09-.1.19-.24.36-.49.51-.76.04-.07.08-.14.11-.21.13-.25.23-.52.3-.79z"],updated:["M8 0a7.95 7.95 0 0 0-6 2.74V1c0-.55-.45-1-1-1S0 .45 0 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H3.54C4.64 2.78 6.22 2 8 2c3.31 0 6 2.69 6 6 0 2.61-1.67 4.81-4 5.63-.63.22-1.29.37-2 .37-3.31 0-6-2.69-6-6 0-.55-.45-1-1-1s-1 .45-1 1c0 4.42 3.58 8 8 8 .34 0 .67-.03 1-.07.02 0 .04-.01.06-.01C12.98 15.4 16 12.06 16 8c0-4.42-3.58-8-8-8zm3 5c-.28 0-.53.11-.71.29L7 8.58 5.71 7.29a1.003 1.003 0 0 0-1.42 1.42l2 2c.18.18.43.29.71.29.28 0 .53-.11.71-.29l4-4A1.003 1.003 0 0 0 11 5z"],upload:["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm3 8c-.28 0-.53-.11-.71-.29L9 6.41V12c0 .55-.45 1-1 1s-1-.45-1-1V6.41l-1.29 1.3a1.003 1.003 0 0 1-1.42-1.42l3-3C7.47 3.11 7.72 3 8 3s.53.11.71.29l3 3A1.003 1.003 0 0 1 11 8z"],user:["M7.99-.01A7.998 7.998 0 0 0 .03 8.77c.01.09.03.18.04.28.02.15.04.31.07.47.02.11.05.22.08.34.03.13.06.26.1.38.04.12.08.25.12.37.04.11.08.21.12.32a6.583 6.583 0 0 0 .3.65c.07.14.14.27.22.4.04.07.08.13.12.2l.27.42.1.13a7.973 7.973 0 0 0 3.83 2.82c.03.01.05.02.07.03.37.12.75.22 1.14.29l.2.03c.39.06.79.1 1.2.1s.81-.04 1.2-.1l.2-.03c.39-.07.77-.16 1.14-.29.03-.01.05-.02.07-.03a8.037 8.037 0 0 0 3.83-2.82c.03-.04.06-.08.09-.13.1-.14.19-.28.28-.42.04-.07.08-.13.12-.2.08-.13.15-.27.22-.41.04-.08.08-.17.12-.26.06-.13.11-.26.17-.39.04-.1.08-.21.12-.32.04-.12.08-.24.12-.37.04-.13.07-.25.1-.38.03-.11.06-.22.08-.34.03-.16.05-.31.07-.47.01-.09.03-.18.04-.28.02-.26.04-.51.04-.78-.03-4.41-3.61-7.99-8.03-7.99zm0 14.4c-1.98 0-3.75-.9-4.92-2.31.67-.36 1.49-.66 2.14-.95 1.16-.52 1.04-.84 1.08-1.27.01-.06.01-.11.01-.17-.41-.36-.74-.86-.96-1.44v-.01c0-.01-.01-.02-.01-.02-.05-.13-.09-.26-.12-.39-.28-.05-.44-.35-.5-.63-.06-.11-.18-.38-.15-.69.04-.41.2-.59.38-.67v-.06c0-.51.05-1.24.14-1.72.02-.13.05-.26.09-.39.17-.59.53-1.12 1.01-1.49.49-.38 1.19-.59 1.82-.59.62 0 1.32.2 1.82.59.48.37.84.9 1.01 1.49.04.13.07.26.09.4.09.48.14 1.21.14 1.72v.07c.18.08.33.26.37.66.03.31-.1.58-.16.69-.06.27-.21.57-.48.62-.03.13-.07.26-.12.38 0 .01-.01.04-.01.04-.21.57-.54 1.06-.94 1.42 0 .06.01.13.01.19.04.43-.12.75 1.05 1.27.65.29 1.47.6 2.14.95a6.415 6.415 0 0 1-4.93 2.31z"],variable:["M3.94 3.15c.47-.66 1.05-1.24 1.76-1.73l.13-.4c-1.11.45-2.05 1.01-2.84 1.7-1.02.88-1.8 1.9-2.32 3.05C.22 6.76 0 7.75 0 8.75c0 1.75.66 3.5 1.99 5.25l.13-.42c-.39-.94-.59-1.82-.59-2.63 0-1.28.22-2.64.67-4.1.45-1.45 1.03-2.69 1.74-3.7zm7.51 6.41l-.27-.15c-.3.41-.52.66-.66.77-.09.06-.21.1-.33.1-.15 0-.3-.1-.45-.28-.25-.33-.59-1.22-1.01-2.69.38-.65.69-1.08.95-1.28.19-.15.39-.22.59-.22.08 0 .22.03.43.08.2.06.39.08.54.08.22 0 .4-.07.54-.22.15-.15.22-.34.22-.57 0-.25-.07-.45-.22-.59-.15-.15-.35-.22-.63-.22-.24 0-.47.06-.69.17-.21.11-.49.36-.82.74-.25.28-.61.78-1.1 1.48a6.72 6.72 0 0 0-.97-2.38l-2.59.44-.05.27c.19-.04.36-.06.49-.06.26 0 .47.11.64.33.26.34.63 1.38 1.11 3.12-.37.49-.63.81-.77.96-.23.24-.41.4-.56.47-.11.06-.24.09-.39.09-.11 0-.29-.06-.53-.18-.17-.07-.32-.11-.45-.11-.25 0-.46.08-.62.24-.16.16-.24.37-.24.61 0 .23.08.42.23.57.15.15.35.22.61.22.25 0 .48-.05.7-.15.22-.1.49-.32.82-.65.33-.33.78-.86 1.36-1.59.22.69.42 1.19.58 1.51.16.31.35.54.56.68.21.14.47.21.79.21.31 0 .62-.11.93-.33.4-.29.82-.77 1.26-1.47zm2.56-8.54l-.12.42c.39.95.59 1.82.59 2.64 0 1.09-.17 2.26-.5 3.51-.26.96-.6 1.87-1.02 2.71-.42.85-.82 1.51-1.21 1.98-.39.48-.87.92-1.44 1.32l-.14.4c1.11-.45 2.05-1.02 2.84-1.7 1.03-.89 1.81-1.91 2.33-3.05.44-.99.66-1.99.66-3 0-1.73-.66-3.48-1.99-5.23z"],"vertical-bar-chart-asc":["M6 7c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1s1-.45 1-1V8c0-.55-.45-1-1-1zM2 9c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1s1-.45 1-1v-5c0-.55-.45-1-1-1zm8-5c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1s1-.45 1-1V5c0-.55-.45-1-1-1zm4-4c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1z"],"vertical-bar-chart-desc":["M6 4c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1s1-.45 1-1V5c0-.55-.45-1-1-1zM2 0c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm8 7c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1s1-.45 1-1V8c0-.55-.45-1-1-1zm4 2c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1s1-.45 1-1v-5c0-.55-.45-1-1-1z"],"vertical-distribution":["M1 2h14c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1s.45 1 1 1zm14 11H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM3 5c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1H3z"],video:["M15 2H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zM5 11V5l6 3-6 3z"],"volume-down":["M9 2c-.28 0-.53.11-.71.29L5.59 5H3c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h2.59l2.71 2.71c.17.18.42.29.7.29.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm3.57 1.44l-1.59 1.22C11.62 5.61 12 6.76 12 8s-.38 2.39-1.02 3.34l1.59 1.22C13.47 11.27 14 9.7 14 8c0-1.7-.53-3.27-1.43-4.56z"],"volume-off":["M11 2c-.28 0-.53.11-.71.29L7.59 5H5c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h2.59l2.71 2.71c.17.18.42.29.7.29.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],"volume-up":["M7 1.86c-.28 0-.53.11-.71.29l-2.7 2.71H1c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h2.59l2.71 2.71a1.003 1.003 0 0 0 1.71-.71v-10c-.01-.55-.46-1-1.01-1zm6.74-.99l-1.58 1.22A9.985 9.985 0 0 1 14 7.86c0 2.16-.69 4.15-1.85 5.78l1.58 1.22c1.42-1.97 2.26-4.38 2.26-7 .01-2.61-.84-5.02-2.25-6.99zM8.98 4.52C9.62 5.48 10 6.63 10 7.86s-.38 2.39-1.02 3.34l1.59 1.22c.9-1.29 1.43-2.86 1.43-4.56 0-1.7-.53-3.27-1.43-4.56L8.98 4.52z"],walk:["M13 8h-2c-.16 0-.31-.05-.44-.11v.01l-1.02-.51-.37 1.86 1.38.92-.01.02c.27.17.46.46.46.81v4c0 .55-.45 1-1 1s-1-.45-1-1v-3.46l-1.27-.85-1.8 4.67h-.01A.98.98 0 0 1 5 16c-.55 0-1-.45-1-1 0-.13.03-.25.07-.36h-.01L7.39 6H5.62l-.73 1.45h-.01C4.72 7.77 4.39 8 4 8c-.55 0-1-.45-1-1 0-.16.05-.31.11-.44H3.1l1-2h.01c.17-.33.5-.56.89-.56h3.16l.29-.75C8.17 2.9 8 2.47 8 2c0-1.1.9-2 2-2s2 .9 2 2c0 1-.73 1.82-1.69 1.97l-.5 1.32 1.43.71H13c.55 0 1 .45 1 1s-.45 1-1 1z"],"warning-sign":["M15.84 13.5l.01-.01-7-12-.01.01c-.17-.3-.48-.5-.85-.5s-.67.2-.85.5l-.01-.01-7 12 .01.01c-.09.15-.15.31-.15.5 0 .55.45 1 1 1h14c.55 0 1-.45 1-1 0-.19-.06-.35-.15-.5zm-6.85-.51h-2v-2h2v2zm0-3h-2v-5h2v5z"],"waterfall-chart":["M8 7c.55 0 1-.45 1-1s-.45-1-1-1-1 .45-1 1 .45 1 1 1zm-4 4h1c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1zm7-6c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1s-1 .45-1 1v1c0 .55.45 1 1 1zm4-3h-1c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm0 10H2V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],widget:["M13 11h2V5h-2v6zM3 5H1v6h2V5zm11-1c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zM2 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12 0c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM5 3h6V1H5v2zM2 0C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm3 15h6v-2H5v2z"],"widget-button":["M1 3h14c.55 0 1 .45 1 1v8c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1zm1 2v6h12V5H2zm3 4c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm3 0c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm3 0c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"widget-footer":["M14 0H2c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h12c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H3v-3h10v3zm0-4H3V2h10v8z"],"widget-header":["M14 0H2c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h12c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H3V6h10v8zm0-9H3V2h10v3z"],wrench:["M15.83 3.7l-3.06 3.05-2.84-.7-.7-2.83L12.29.17a5.004 5.004 0 0 0-4.83 1.29 4.967 4.967 0 0 0-1.12 5.36L.58 12.58c-.36.36-.58.86-.58 1.41 0 1.1.9 2 2 2 .55 0 1.05-.22 1.41-.59l5.77-5.77c1.79.69 3.91.33 5.35-1.12 1.32-1.3 1.74-3.15 1.3-4.81z"],"zoom-in":["M7.99 5.99v-2c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1h-2zm7.56 7.44l-2.67-2.68a6.94 6.94 0 0 0 1.11-3.76c0-3.87-3.13-7-7-7s-7 3.13-7 7 3.13 7 7 7c1.39 0 2.68-.42 3.76-1.11l2.68 2.67a1.498 1.498 0 1 0 2.12-2.12zm-8.56-1.44c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5z"],"zoom-out":["M3.99 5.99c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1h-6zm11.56 7.44l-2.67-2.68a6.94 6.94 0 0 0 1.11-3.76c0-3.87-3.13-7-7-7s-7 3.13-7 7 3.13 7 7 7c1.39 0 2.68-.42 3.76-1.11l2.68 2.67a1.498 1.498 0 1 0 2.12-2.12zm-8.56-1.44c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5z"],"zoom-to-fit":["M11 10a1.003 1.003 0 0 0 1.71.71l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-2-2a1.003 1.003 0 0 0-1.42 1.42L12.59 8 11.3 9.29c-.19.18-.3.43-.3.71zM1 5c.55 0 1-.45 1-1V2h2c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1v3c0 .55.45 1 1 1zm4 1a1.003 1.003 0 0 0-1.71-.71l-2 2C1.11 7.47 1 7.72 1 8c0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42L3.41 8 4.7 6.71c.19-.18.3-.43.3-.71zm1-1c.28 0 .53-.11.71-.29L8 3.41 9.29 4.7c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-2-2C8.53 1.11 8.28 1 8 1s-.53.11-.71.29l-2 2A1.003 1.003 0 0 0 6 5zm9 6c-.55 0-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h3c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1zm0-11h-3c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zM4 14H2v-2c0-.55-.45-1-1-1s-1 .45-1 1v3c0 .55.45 1 1 1h3c.55 0 1-.45 1-1s-.45-1-1-1zm6-3c-.28 0-.53.11-.71.29L8 12.59 6.71 11.3A.965.965 0 0 0 6 11a1.003 1.003 0 0 0-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2A1.003 1.003 0 0 0 10 11z"]},i={add:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8zm5-9h-4V5c0-.55-.45-1-1-1s-1 .45-1 1v4H5c-.55 0-1 .45-1 1s.45 1 1 1h4v4c0 .55.45 1 1 1s1-.45 1-1v-4h4c.55 0 1-.45 1-1s-.45-1-1-1z"],"add-column-left":["M4 11h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1H8V7c0-.55-.45-1-1-1s-1 .45-1 1v2H4c-.55 0-1 .45-1 1s.45 1 1 1zM19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-7 18H2V2h10v16zm6 0h-5V2h5v16z"],"add-column-right":["M10 11h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1h-2V7c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1zm9-11H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM7 18H2V2h5v16zm11 0H8V2h10v16z"],"add-row-bottom":["M19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H2V8h16v10zm0-11H2V2h16v5zM7 14h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1h-2v-2c0-.55-.45-1-1-1s-1 .45-1 1v2H7c-.55 0-1 .45-1 1s.45 1 1 1z"],"add-row-top":["M7 8h2v2c0 .55.45 1 1 1s1-.45 1-1V8h2c.55 0 1-.45 1-1s-.45-1-1-1h-2V4c0-.55-.45-1-1-1s-1 .45-1 1v2H7c-.55 0-1 .45-1 1s.45 1 1 1zm12-8H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H2v-5h16v5zm0-6H2V2h16v10z"],"add-to-artifact":["M13 12H1c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zm0 4H1c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zM1 6h9c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm12 2H1c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zm6-4h-2V2c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V6h2c.55 0 1-.45 1-1s-.45-1-1-1z"],"add-to-folder":["M.01 10V6H20v10c0 .55-.45 1-1 1H9.995v-3.99C9.965 11.332 8.635 10 6.987 10H.01zM19 3c.55 0 1 .45.99 1v1H0V2c0-.55.45-1 1-1h5.997c.28 0 .53.11.71.29L9.414 3H19zM6.987 12c.55 0 .999.45 1.009 1.01v5c0 .55-.45 1-1 1s-.999-.45-.999-1v-2.59l-4.288 4.29a1.003 1.003 0 0 1-1.42-1.42L4.579 14H1.989c-.55 0-1-.45-1-1s.45-1 1-1h4.998z"],airplane:["M20 2c0-1.1-.9-2-2-2-.55 0-1.05.22-1.41.59l-4.84 4.84L2 1 1 3l7.53 5.64L4.17 13H1l-1 1 4 2 2 4 1-1v-3.17l4.36-4.36L17 19l2-1-4.43-9.74 4.84-4.84c.37-.37.59-.87.59-1.42z"],"align-center":["M5 5c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1H5zM1 3h18c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm12 12c.55 0 1-.45 1-1s-.45-1-1-1H7c-.55 0-1 .45-1 1s.45 1 1 1h6zm4 2H3c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm2-8H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],"align-justify":["M1 3h18c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm18 14H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm0-12H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm0 4H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm0 4H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],"align-left":["M1 7h10c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm0-4h18c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm14 14H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm4-8H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zM1 15h6c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1z"],"align-right":["M19 17H5c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM1 3h18c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm18 10h-6c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1zm0-4H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm0-4H9c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1z"],"alignment-bottom":["M12 16h4c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1h-4c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1zm7 2H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zM4 16h4c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1z"],"alignment-horizontal-center":["M19 9h-2V7c0-.55-.45-1-1-1h-4c-.55 0-1 .45-1 1v2H9V3c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v6H1c-.55 0-1 .45-1 1s.45 1 1 1h2v6c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-6h2v2c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1z"],"alignment-left":["M1 0C.45 0 0 .45 0 1v18c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm11 11H5c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1zm7-8H5c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"alignment-right":["M19 0c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm-4 11H8c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1zm0-8H1c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"alignment-top":["M8 4H4c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm11-4H1C.45 0 0 .45 0 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm-3 4h-4c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1z"],"alignment-vertical-center":["M17 3h-6V1c0-.55-.45-1-1-1S9 .45 9 1v2H3c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h6v2H7c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1h-2V9h6c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],annotation:["M9.41 13.41l7.65-7.65-2.83-2.83-7.65 7.65 2.83 2.83zm10-10c.37-.36.59-.86.59-1.41 0-1.1-.9-2-2-2-.55 0-1.05.22-1.41.59l-1.65 1.65 2.83 2.83 1.64-1.66zM18 18H2V2h8.93l2-2H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V7.07l-2 2V18zM4 16l4.41-1.59-2.81-2.79L4 16z"],application:["M3.5 9h9c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-9c-.28 0-.5.22-.5.5s.22.5.5.5zm0 2h5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-5c-.28 0-.5.22-.5.5s.22.5.5.5zM19 1H1c-.55 0-1 .45-1 1v16c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zm-1 16H2V6h16v11zM3.5 13h7c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-7c-.28 0-.5.22-.5.5s.22.5.5.5z"],applications:["M15 5H1c-.55 0-1 .45-1 1v13c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm-1 13H2V8h12v10zM3.5 10h7c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-7c-.28 0-.5.22-.5.5s.22.5.5.5zm0 2h3c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-3c-.28 0-.5.22-.5.5s.22.5.5.5zm0 2h5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-5c-.28 0-.5.22-.5.5s.22.5.5.5zM19 0H5c-.55 0-1 .45-1 1v3h2V3h12v10h-1v2h2c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"arrow-bottom-left":["M18 3a1.003 1.003 0 0 0-1.71-.71L4 14.59V7c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1H5.41l12.3-12.29c.18-.18.29-.43.29-.71z"],"arrow-bottom-right":["M17 6c-.55 0-1 .45-1 1v7.59L3.71 2.29a1.003 1.003 0 0 0-1.42 1.42L14.59 16H7c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1z"],"arrow-down":["M16 11c-.3 0-.5.1-.7.3L11 15.6V2c0-.5-.4-1-1-1s-1 .5-1 1v13.6l-4.3-4.3c-.2-.2-.4-.3-.7-.3-.5 0-1 .4-1 1 0 .3.1.5.3.7l6 6c.2.2.4.3.7.3s.5-.1.7-.3l6-6c.2-.2.3-.4.3-.7 0-.6-.5-1-1-1z"],"arrow-left":["M18 9H4.41L8.7 4.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-6 6c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l6 6a1.003 1.003 0 0 0 1.42-1.42L4.41 11H18c.55 0 1-.45 1-1s-.45-1-1-1z"],"arrow-right":["M18.71 9.29l-6-6a1.003 1.003 0 0 0-1.42 1.42L15.59 9H2c-.55 0-1 .45-1 1s.45 1 1 1h13.59l-4.29 4.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l6-6c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"arrow-top-left":["M17.71 16.29L5.41 4H13c.55 0 1-.45 1-1s-.45-1-1-1H3c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1s1-.45 1-1V5.41L16.29 17.7c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71z"],"arrow-top-right":["M17 2H7c-.55 0-1 .45-1 1s.45 1 1 1h7.59L2.29 16.29a1.003 1.003 0 0 0 1.42 1.42L16 5.41V13c0 .55.45 1 1 1s1-.45 1-1V3c0-.55-.45-1-1-1z"],"arrow-up":["M16.7 7.3l-6-6c-.2-.2-.4-.3-.7-.3s-.5.1-.7.3l-6 6c-.2.2-.3.4-.3.7 0 .6.5 1 1 1 .3 0 .5-.1.7-.3L9 4.4V18c0 .5.4 1 1 1s1-.5 1-1V4.4l4.3 4.3c.2.2.4.3.7.3.5 0 1-.4 1-1 0-.3-.1-.5-.3-.7z"],"arrows-horizontal":["M19.7 9.3l-5-5c-.2-.2-.4-.3-.7-.3-.6 0-1 .4-1 1 0 .3.1.5.3.7L16.6 9H3.4l3.3-3.3c.2-.2.3-.4.3-.7 0-.6-.4-1-1-1-.3 0-.5.1-.7.3l-5 5c-.2.2-.3.4-.3.7s.1.5.3.7l5 5c.2.2.4.3.7.3.6 0 1-.4 1-1 0-.3-.1-.5-.3-.7L3.4 11h13.2l-3.3 3.3c-.2.2-.3.4-.3.7 0 .6.4 1 1 1 .3 0 .5-.1.7-.3l5-5c.2-.2.3-.4.3-.7s-.1-.5-.3-.7z"],"arrows-vertical":["M15 13c-.3 0-.5.1-.7.3L11 16.6V3.4l3.3 3.3c.2.2.4.3.7.3.6 0 1-.4 1-1 0-.3-.1-.5-.3-.7l-5-5c-.2-.2-.4-.3-.7-.3s-.5.1-.7.3l-5 5c-.2.2-.3.4-.3.7 0 .6.4 1 1 1 .3 0 .5-.1.7-.3L9 3.4v13.2l-3.3-3.3c-.2-.2-.4-.3-.7-.3-.6 0-1 .4-1 1 0 .3.1.5.3.7l5 5c.2.2.4.3.7.3s.5-.1.7-.3l5-5c.2-.2.3-.4.3-.7 0-.5-.4-1-1-1z"],asterisk:["M18.52 14.17l.01-.02L11.89 10l6.64-4.15-.01-.02A.97.97 0 0 0 19 5c0-.55-.45-1-1-1-.2 0-.37.07-.52.17l-.01-.02L11 8.2V1c0-.55-.45-1-1-1S9 .45 9 1v7.2L2.53 4.15l-.01.02A.922.922 0 0 0 2 4c-.55 0-1 .45-1 1 0 .36.2.66.48.83l-.01.02L8.11 10l-6.64 4.15.01.02A.97.97 0 0 0 1 15c0 .55.45 1 1 1 .2 0 .37-.07.52-.17l.01.02L9 11.8V19c0 .55.45 1 1 1s1-.45 1-1v-7.2l6.47 4.04.01-.02c.15.11.32.18.52.18.55 0 1-.45 1-1 0-.36-.2-.66-.48-.83z"],"automatic-updates":["M10 18c-4.42 0-8-3.58-8-8 0-2.52 1.18-4.76 3-6.22V5c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1H2c-.55 0-1 .45-1 1s.45 1 1 1h2.06C1.61 3.82 0 6.71 0 10c0 5.52 4.48 10 10 10 .55 0 1-.45 1-1s-.45-1-1-1zm0-16c1.64 0 3.15.49 4.42 1.34l1.43-1.43A9.869 9.869 0 0 0 10 0c-.55 0-1 .45-1 1s.45 1 1 1zm10 8c0-1.13-.2-2.21-.54-3.22L17.84 8.4A7.962 7.962 0 0 1 15 16.22V15c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1h-2.06c2.45-1.82 4.06-4.71 4.06-8zm0-7a1.003 1.003 0 0 0-1.71-.71L12 8.59l-2.29-2.3a1.003 1.003 0 0 0-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l7-7c.18-.18.29-.43.29-.71z"],badge:["M16.94 5.73c-.19-1.41.62-2.52 1.38-3.59L17.03.65C14.89 1.76 11.88 1.48 10 0 8.12 1.48 5.11 1.76 2.97.65L1.68 2.14c.76 1.07 1.57 2.18 1.38 3.59C2.68 8.59 0 10.94 1.4 14.08c.56 1.43 1.81 2.37 3.4 2.75 1.95.46 4.4.91 5.2 3.17.8-2.26 3.25-2.71 5.2-3.17 1.6-.38 2.84-1.32 3.4-2.75 1.4-3.14-1.28-5.49-1.66-8.35z"],"ban-circle":["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm5 11H5c-.55 0-1-.45-1-1s.45-1 1-1h10c.55 0 1 .45 1 1s-.45 1-1 1z"],"bank-account":["M19.2 8.02l-.78-.18C18.03 6.4 17.2 5.08 16.08 4l.5-2.28c.11-.42-.22-.78-.61-.72-1.06.12-2 .54-2.67 1.26-1.06-.42-2.34-.66-3.56-.66-3.12 0-5.79 1.5-7.4 3.72-.23-.05-.45-.11-.67-.11C.72 5.21 0 5.98 0 7c0 .72.39 1.32.95 1.62-.06.42-.12.9-.12 1.38 0 2.16.89 4.08 2.28 5.58l-.33 2.04c-.11.72.45 1.38 1.12 1.38h.72c.56 0 1-.42 1.11-1.02l.06-.48c1.17.54 2.5.9 3.95.9 1.39 0 2.78-.3 3.95-.9l.06.48c.11.6.56 1.02 1.11 1.02h.72c.67 0 1.22-.66 1.11-1.38l-.33-1.98c.78-.78 1.34-1.74 1.73-2.76l1-.24c.5-.12.89-.6.89-1.2V9.22c.11-.6-.28-1.08-.78-1.2zM15 10c-.6 0-1-.7-1-1.5S14.4 7 15 7s1 .7 1 1.5-.4 1.5-1 1.5zM7.55 5.83a.99.99 0 0 1-1.38-.28.99.99 0 0 1 .28-1.38c2.34-1.56 4.77-1.56 7.11 0 .46.31.58.93.28 1.39-.31.46-.93.58-1.39.28-1.67-1.12-3.23-1.12-4.9-.01z"],barcode:["M6 16.98h2v-14H6v14zm3 0h1v-14H9v14zm-6 0h2v-14H3v14zm-3 0h2v-14H0v14zm16 0h2v-14h-2v14zm-4 0h1v-14h-1v14zm7-14v14h1v-14h-1zm-5 14h1v-14h-1v14z"],blank:[],"blocked-person":["M11.55 15.92c-1.48-.65-1.28-1.05-1.33-1.59-.01-.07-.01-.15-.01-.23.51-.45.92-1.07 1.19-1.78 0 0 .01-.04.02-.05.06-.15.11-.32.15-.48.34-.07.54-.44.61-.78.06-.11.14-.35.17-.62C10.33 9.42 8.92 7.38 8.92 5c0-.3.05-.58.09-.87-.33-.08-.67-.13-.99-.13-.79 0-1.68.25-2.31.73-.61.47-1.07 1.13-1.29 1.86-.05.16-.09.33-.11.5-.12.6-.17 1.51-.17 2.14v.08c-.24.09-.45.32-.49.83-.04.39.12.73.2.87.08.35.28.72.63.78.04.17.09.33.15.48 0 .01.01.02.01.03l.01.01c.27.72.7 1.35 1.22 1.8 0 .07-.01.14-.01.21-.05.54.1.94-1.38 1.59C3 16.56.77 17.26.32 18.31-.15 19.38.04 20 .04 20h15.95s.18-.62-.27-1.67c-.46-1.06-2.69-1.75-4.17-2.41zM14.97 0c-2.78 0-5.03 2.24-5.03 5s2.25 5 5.03 5S20 7.76 20 5s-2.25-5-5.03-5zm-3.03 5c0-1.66 1.35-3 3.02-3 .47 0 .9.11 1.29.3l-4.01 3.99c-.18-.4-.3-.83-.3-1.29zm3.03 3c-.47 0-.9-.11-1.29-.3l4.01-3.99c.19.39.3.82.3 1.29 0 1.66-1.36 3-3.02 3z"],bold:["M14.3 9c.4-.8.7-1.6.7-2.5C15 4 13 2 10.5 2H5c-.6 0-1 .4-1 1v13c0 .6.4 1 1 1h6.5c2.5 0 4.5-2 4.5-4.5 0-1.4-.7-2.7-1.7-3.5zM7 5h3.5c.8 0 1.5.7 1.5 1.5S11.3 8 10.5 8H7V5zm4.5 9H7v-3h4.5c.8 0 1.5.7 1.5 1.5s-.7 1.5-1.5 1.5z"],book:["M3 1v18c0 .55.45 1 1 1h2V0H4c-.55 0-1 .45-1 1zm14-1h-2v8l-2-2-2 2V0H7v20h10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],bookmark:["M6 0c-.55 0-1 .45-1 1v18c0 .55.32.68.71.29L9.3 15.7a.996.996 0 0 1 1.41 0l3.59 3.59c.38.39.7.26.7-.29v-8-4.5V1c0-.55-.45-1-1-1H6z"],box:["M19.89 6.56l-2.99-6h-.01C16.72.23 16.39 0 16 0H4c-.39 0-.72.23-.89.56H3.1l-3 6h.01C.05 6.69 0 6.84 0 7v12c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V7c0-.16-.05-.31-.11-.44zM11 2h4.38l2 4H11V2zM4.62 2H9v4H2.62l2-4zM18 18H2V8h16v10zM8 12h4c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1z"],briefcase:["M19 5h-4V2c0-.55-.45-1-1-1H6c-.55 0-1 .45-1 1v3H1c-.55 0-1 .45-1 1v5h4v-1h2v1h8v-1h2v1h4V6c0-.55-.45-1-1-1zm-6 0H7V3h6v2zm3 8h-2v-1H6v1H4v-1H0v6c0 .55.45 1 1 1h18c.55 0 1-.45 1-1v-6h-4v1z"],build:["M19.43 16.67L9.31 7.81l1.47-1.56c.41-.44-.15-.8.15-1.6 1.08-2.76 4.19-2.99 4.19-2.99s.45-.47.87-.92C11.98-1 9.26.7 8.04 1.8L3.83 6.25l-.86.92c-.48.51-.48 1.33 0 1.84l-.87.92c-.48-.51-1.26-.51-1.74 0s-.48 1.33 0 1.84l1.74 1.84c.48.51 1.26.51 1.74 0s.48-1.33 0-1.84l.87-.92c.48.51 1.26.51 1.74 0l1.41-1.49 8.81 10.07c.76.76 2 .76 2.76 0 .76-.76.76-2 0-2.76z"],calculator:["M16 0H4c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h12c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM7 18H5v-2h2v2zm0-4H5v-2h2v2zm0-4H5V8h2v2zm4 8H9v-2h2v2zm0-4H9v-2h2v2zm0-4H9V8h2v2zm4 8h-2v-6h2v6zm0-8h-2V8h2v2zm0-4H5V2h10v4z"],calendar:["M15 5c.6 0 1-.4 1-1V2c0-.5-.4-1-1-1s-1 .5-1 1v2c0 .6.4 1 1 1zM5 5c.6 0 1-.4 1-1V2c0-.5-.4-1-1-1s-1 .5-1 1v2c0 .6.4 1 1 1zm13-2h-1v1c0 1.1-.9 2-2 2s-2-.9-2-2V3H7v1c0 1.1-.9 2-2 2s-2-.9-2-2V3H2c-.5 0-1 .5-1 1v14c0 .5.5 1 1 1h16c.5 0 1-.5 1-1V4c0-.5-.5-1-1-1zM7 17H3v-4h4v4zm0-5H3V8h4v4zm5 5H8v-4h4v4zm0-5H8V8h4v4zm5 5h-4v-4h4v4zm0-5h-4V8h4v4z"],camera:["M10 8c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3zm9-4h-3.59L13.7 2.29A.956.956 0 0 0 13 2H7c-.28 0-.53.11-.71.29L4.59 4H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h4.11c1.26 1.24 2.99 2 4.89 2s3.63-.76 4.89-2H19c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zM4 8H2V6h2v2zm6 8c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5z"],"caret-down":["M16 7c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1 0 .24.1.46.24.63l-.01.01 5 6 .01-.01c.19.22.45.37.76.37s.57-.15.76-.37l.01.01 5-6-.01-.01c.14-.17.24-.39.24-.63z"],"caret-left":["M13 4c-.24 0-.46.1-.63.24l-.01-.01-6 5 .01.01c-.22.19-.37.45-.37.76s.15.57.37.76l-.01.01 6 5 .01-.01c.17.14.39.24.63.24.55 0 1-.45 1-1V5c0-.55-.45-1-1-1z"],"caret-right":["M14 10c0-.31-.15-.57-.37-.76l.01-.01-6-5-.01.01C7.46 4.1 7.24 4 7 4c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1 .24 0 .46-.1.63-.24l.01.01 6-5-.01-.01c.22-.19.37-.45.37-.76z"],"caret-up":["M15.76 12.37l.01-.01-5-6-.01.01C10.57 6.15 10.31 6 10 6s-.57.15-.76.37l-.01-.01-5 6 .01.01c-.14.17-.24.39-.24.63 0 .55.45 1 1 1h10c.55 0 1-.45 1-1 0-.24-.1-.46-.24-.63z"],"cell-tower":["M11.5 8.32c.31-.35.51-.81.51-1.32 0-1.1-.9-2-2-2s-2 .9-2 2c0 .51.2.97.51 1.32L5.06 18.69c-.17.52.11 1.09.63 1.26s1.09-.11 1.26-.63L8.39 15h3.23l1.44 4.32c.17.52.74.81 1.26.63s.81-.74.63-1.26L11.5 8.32zM10.95 13H9.06l.95-2.84.94 2.84zM5.31 10.73a.996.996 0 1 0 1.37-1.45c-1.4-1.33-1.28-3.35-.01-4.54.4-.38.43-1.01.05-1.41-.36-.41-1-.43-1.4-.06-2.09 1.95-2.28 5.3-.01 7.46z","M4.6 12.2C3 11.1 2 9 2 7c0-2.1.9-3.9 2.6-5.2.5-.3.5-1 .2-1.4-.3-.5-1-.5-1.4-.2C1.2 1.9-.1 4.2 0 7c.1 2.7 1.4 5.3 3.4 6.8.2.1.4.2.6.2.3 0 .6-.1.8-.4.4-.5.3-1.1-.2-1.4zM13.27 10.69c.38.4 1.01.42 1.41.04 2.27-2.16 2.08-5.51-.01-7.46a.996.996 0 1 0-1.36 1.46c1.28 1.19 1.39 3.21-.01 4.54-.39.39-.41 1.02-.03 1.42z","M16.6.2c-.4-.3-1.1-.3-1.4.2-.3.4-.3 1.1.2 1.4C17.1 3.1 18 4.9 18 7c0 2-1 4.1-2.6 5.2-.5.3-.6.9-.2 1.4.2.3.5.4.8.4.2 0 .4-.1.6-.2C18.7 12.3 20 9.7 20 7c.09-2.8-1.2-5.1-3.4-6.8z"],changes:["M18 16H2c-1.1 0-2 .9-2 2s.9 2 2 2h16c1.1 0 2-.9 2-2s-.9-2-2-2zM3 5c.28 0 .53-.11.71-.29L5 3.41V13c0 .55.45 1 1 1s1-.45 1-1V3.41L8.29 4.7c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-3-3C6.53.11 6.28 0 6 0s-.53.11-.71.29l-3 3A1.003 1.003 0 0 0 3 5zm7.29 5.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3a1.003 1.003 0 0 0-1.42-1.42L15 10.59V1c0-.55-.45-1-1-1s-1 .45-1 1v9.59L11.71 9.3A.965.965 0 0 0 11 9a1.003 1.003 0 0 0-.71 1.71z"],chart:["M7 11v8c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-8l-2 2-4-2zm-7 8c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-8l-6 3v5zM17 7l-3 3v9c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V8.74c-.26.15-.58.26-1 .26-1.92 0-2-2-2-2zm2-6h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59L10.8 8.78 7.45 7.11v.01C7.31 7.05 7.16 7 7 7s-.31.05-.44.11V7.1l-6 3v.01c-.33.17-.56.5-.56.89 0 .55.45 1 1 1 .16 0 .31-.05.44-.11v.01L7 9.12l3.55 1.78v-.01c.14.06.29.11.45.11.28 0 .53-.11.71-.29L18 4.41V6c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1z"],chat:["M19 0H7c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h5.59l3.71 3.71c.17.18.42.29.7.29.55 0 1-.45 1-1v-3h1c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM7 13c-1.1 0-2-.9-2-2V4H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h1v3a1.003 1.003 0 0 0 1.71.71L7.41 16H13c.55 0 1-.45 1-1v-.17L12.17 13H7z"],"chevron-backward":["M8.41 10l5.29-5.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L7 8.59V4c0-.55-.45-1-1-1s-1 .45-1 1v12c0 .55.45 1 1 1s1-.45 1-1v-4.59l5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L8.41 10z"],"chevron-down":["M16 6c-.28 0-.53.11-.71.29L10 11.59l-5.29-5.3a1.003 1.003 0 0 0-1.42 1.42l6 6c.18.18.43.29.71.29s.53-.11.71-.29l6-6A1.003 1.003 0 0 0 16 6z"],"chevron-forward":["M13 3c-.55 0-1 .45-1 1v4.59l-5.29-5.3a1.003 1.003 0 0 0-1.42 1.42l5.3 5.29-5.29 5.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l5.29-5.3V16c0 .55.45 1 1 1s1-.45 1-1V4c0-.55-.45-1-1-1z"],"chevron-left":["M8.41 10l5.29-5.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-6 6c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l6 6a1.003 1.003 0 0 0 1.42-1.42L8.41 10z"],"chevron-right":["M13.71 9.29l-6-6a1.003 1.003 0 0 0-1.42 1.42l5.3 5.29-5.29 5.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l6-6c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"chevron-up":["M16.71 12.29l-6-6C10.53 6.11 10.28 6 10 6s-.53.11-.71.29l-6 6a1.003 1.003 0 0 0 1.42 1.42L10 8.41l5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71z"],circle:["M10 0C4.5 0 0 4.5 0 10s4.5 10 10 10 10-4.5 10-10S15.5 0 10 0zm0 18c-4.4 0-8-3.6-8-8s3.6-8 8-8 8 3.6 8 8-3.6 8-8 8z"],"circle-arrow-down":["M14 10c-.28 0-.53.11-.71.29L11 12.59V5c0-.55-.45-1-1-1s-1 .45-1 1v7.59L6.71 10.3A.965.965 0 0 0 6 10a1.003 1.003 0 0 0-.71 1.71l4 4c.18.18.43.29.71.29s.53-.11.71-.29l4-4A1.003 1.003 0 0 0 14 10zM10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"],"circle-arrow-left":["M15 9H7.41L9.7 6.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-4 4c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l4 4a1.003 1.003 0 0 0 1.42-1.42L7.41 11H15c.55 0 1-.45 1-1s-.45-1-1-1zm-5-9C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"],"circle-arrow-right":["M15.71 9.29l-4-4a1.003 1.003 0 0 0-1.42 1.42L12.59 9H5c-.55 0-1 .45-1 1s.45 1 1 1h7.59l-2.29 2.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zM10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"],"circle-arrow-up":["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8zm.71-13.71C10.53 4.11 10.28 4 10 4s-.53.11-.71.29l-4 4a1.003 1.003 0 0 0 1.42 1.42L9 7.41V15c0 .55.45 1 1 1s1-.45 1-1V7.41l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-4-4z"],citation:["M4 1C1.79 1 0 2.79 0 5s1.79 4 4 4c.1 0 .2-.01.3-.02C3.82 11.32 2.53 13 1 13c-.55 0-1 .45-1 1s.45 1 1 1c3.87 0 7-4.48 7-10 0-2.21-1.79-4-4-4zM16 1c-2.21 0-4 1.79-4 4s1.79 4 4 4c.1 0 .2-.01.3-.02C15.82 11.32 14.53 13 13 13c-.55 0-1 .45-1 1s.45 1 1 1c3.87 0 7-4.48 7-10 0-2.21-1.79-4-4-4z"],clean:["M7 0L5 5 0 6.998 5 9l2 5 2-5 5-1.995L9 5zM15 10l-1.5 3.496-3.5 1.499 3.5 1.498L15 20l1.5-3.507L20 15l-3.5-1.504z"],clipboard:["M13 2c0-.55-.45-1-1-1h-.78a1.98 1.98 0 0 0-3.44 0H7c-.55 0-1 .45-1 1v2h7V2z","M16 2h-2v3H5V2H3c-.55 0-1 .45-1 1v16c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],cloud:["M15 7c-.12 0-.24.03-.36.04C13.83 4.69 11.62 3 9 3 5.69 3 3 5.69 3 9c0 .05.01.09.01.14A3.98 3.98 0 0 0 0 13c0 2.21 1.79 4 4 4h11c2.76 0 5-2.24 5-5s-2.24-5-5-5z"],"cloud-download":["M15 4c-.12 0-.24.03-.36.04C13.83 1.69 11.62 0 9 0 5.69 0 3 2.69 3 6c0 .05.01.09.01.14A3.98 3.98 0 0 0 0 10c0 2.21 1.79 4 4 4h.78c.55-.61 1.34-1 2.22-1v-2c0-1.66 1.34-3 3-3s3 1.34 3 3v2c.88 0 1.66.38 2.2.98C17.87 13.87 20 11.69 20 9c0-2.76-2.24-5-5-5zm-2 11c-.28 0-.53.11-.71.29L11 16.59V11c0-.55-.45-1-1-1s-1 .45-1 1v5.59L7.71 15.3A.965.965 0 0 0 7 15a1.003 1.003 0 0 0-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0 0 13 15z"],"cloud-upload":["M10.71 10.29c-.18-.18-.43-.29-.71-.29s-.53.11-.71.29l-3 3a1.003 1.003 0 0 0 1.42 1.42L9 13.41V19c0 .55.45 1 1 1s1-.45 1-1v-5.59l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-3-3zM15 4c-.12 0-.24.03-.36.04C13.83 1.69 11.62 0 9 0 5.69 0 3 2.69 3 6c0 .05.01.09.01.14A3.98 3.98 0 0 0 0 10c0 2.21 1.79 4 4 4 0-.83.34-1.58.88-2.12l3-3a2.993 2.993 0 0 1 4.24 0l3 3-.01.01c.52.52.85 1.23.87 2.02C18.28 13.44 20 11.42 20 9c0-2.76-2.24-5-5-5z"],code:["M6 6a1.003 1.003 0 0 0-1.71-.71l-4 4C.11 9.47 0 9.72 0 10c0 .28.11.53.29.71l4 4a1.003 1.003 0 0 0 1.42-1.42L2.41 10 5.7 6.71c.19-.18.3-.43.3-.71zm6-4c-.46 0-.83.31-.95.73l-4 14c-.02.09-.05.17-.05.27 0 .55.45 1 1 1 .46 0 .83-.31.95-.73l4-14c.02-.09.05-.17.05-.27 0-.55-.45-1-1-1zm7.71 7.29l-4-4a1.003 1.003 0 0 0-1.42 1.42l3.3 3.29-3.29 3.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"code-block":["M19 5h-2V3c0-.55-.45-1-1-1h-4c-.55 0-1 .45-1 1v2H9V3c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v2H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zM8.71 15.29a1.003 1.003 0 0 1-1.42 1.42l-4-4C3.11 12.53 3 12.28 3 12s.11-.53.29-.71l4-4a1.003 1.003 0 0 1 1.42 1.42L5.41 12l3.3 3.29zm8-2.58l-4 4a1.003 1.003 0 0 1-1.42-1.42l3.3-3.29-3.29-3.29A.965.965 0 0 1 11 8a1.003 1.003 0 0 1 1.71-.71l4 4c.18.18.29.43.29.71s-.11.53-.29.71z"],cog:["M19 8h-2.31c-.14-.46-.33-.89-.56-1.3l1.7-1.7a.996.996 0 0 0 0-1.41l-1.41-1.41a.996.996 0 0 0-1.41 0l-1.7 1.7c-.41-.22-.84-.41-1.3-.55V1c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v2.33c-.48.14-.94.34-1.37.58L5 2.28a.972.972 0 0 0-1.36 0L2.28 3.64c-.37.38-.37.99 0 1.36L3.9 6.62c-.24.44-.44.89-.59 1.38H1c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h2.31c.14.46.33.89.56 1.3L2.17 15a.996.996 0 0 0 0 1.41l1.41 1.41c.39.39 1.02.39 1.41 0l1.7-1.7c.41.22.84.41 1.3.55V19c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-2.33c.48-.14.94-.35 1.37-.59L15 17.72c.37.37.98.37 1.36 0l1.36-1.36c.37-.37.37-.98 0-1.36l-1.62-1.62c.24-.43.45-.89.6-1.38H19c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1zm-9 6c-2.21 0-4-1.79-4-4s1.79-4 4-4 4 1.79 4 4-1.79 4-4 4z"],"collapse-all":["M9.29 8.71c.18.18.43.29.71.29s.53-.11.71-.29l6-6a1.003 1.003 0 0 0-1.42-1.42L10 6.59l-5.29-5.3a1.003 1.003 0 0 0-1.42 1.42l6 6zm1.42 2.58c-.18-.18-.43-.29-.71-.29s-.53.11-.71.29l-6 6a1.003 1.003 0 0 0 1.42 1.42l5.29-5.3 5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-6-6z"],"column-layout":["M19 1H1c-.55 0-1 .45-1 1v16c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM5 17H2V3h3v14zm4 0H6V3h3v14zm9 0h-8V3h8v14z"],comment:["M19 1H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h3v4a1.003 1.003 0 0 0 1.71.71l4.7-4.71H19c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM4 10c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zm6 0c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zm6 0c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2z"],comparison:["M6 8H1c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1zm13-6h-5c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm0 3h-5V3h5v2zM6 14H1c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1zM6 2H1c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm4-2c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm9 14h-5c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1zm0 3h-5v-2h5v2zm0-9h-5c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1zm0 3h-5V9h5v2z"],compass:["M15 10c0 .14-.03.28-.09.4l-3.99 8.98-.01.02a.991.991 0 0 1-1.82 0l-.01-.02-3.99-8.98c-.06-.12-.09-.26-.09-.4s.03-.28.09-.4L9.08.62 9.09.6a.991.991 0 0 1 1.82 0l.01.02 3.99 8.98c.06.12.09.26.09.4zm-5-6.54L7.09 10h5.81L10 3.46z"],compressed:["M19.89 6.56l-2.99-6h-.01C16.72.23 16.39 0 16 0H4c-.39 0-.72.23-.89.56H3.1l-3 6h.01C.05 6.69 0 6.84 0 7v12c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V7c0-.16-.05-.31-.11-.44zM11 2h4.38l2 4H11V2zM4.62 2H9v4H2.62l2-4zM18 18H2V8h7v4.59L6.71 10.3A.965.965 0 0 0 6 10a1.003 1.003 0 0 0-.71 1.71l4 4c.18.18.43.29.71.29s.53-.11.71-.29l4-4a1.003 1.003 0 0 0-1.42-1.42L11 12.59V8h7v10z"],confirm:["M9.71 5.29a1.003 1.003 0 0 0-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l7-7a1.003 1.003 0 0 0-1.42-1.42L12 7.59l-2.29-2.3zm7.93 2.32c.23.75.36 1.56.36 2.39 0 4.42-3.58 8-8 8s-8-3.58-8-8a7.998 7.998 0 0 1 11.8-7.04l1.46-1.46C13.73.56 11.93 0 10 0 4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10c0-1.4-.29-2.73-.81-3.95l-1.55 1.56z"],console:["M19 19H1c-.55 0-1-.45-1-1V2c0-.55.45-1 1-1h18c.55 0 1 .45 1 1v16c0 .55-.45 1-1 1zM18 6H2v11h16V6zM4 8c.28 0 .53.11.71.29l2 2c.18.18.29.43.29.71s-.11.53-.29.71l-2 2a1.003 1.003 0 0 1-1.42-1.42L4.59 11l-1.3-1.29A1.003 1.003 0 0 1 4 8zm5 4h3c.55 0 1 .45 1 1s-.45 1-1 1H9c-.55 0-1-.45-1-1s.45-1 1-1z"],contrast:["M19 8h-1.26c-.19-.73-.48-1.42-.85-2.06l.94-.94a.996.996 0 0 0 0-1.41l-1.41-1.41a.996.996 0 0 0-1.41 0l-.94.94c-.65-.38-1.34-.67-2.07-.86V1c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v1.26c-.76.2-1.47.5-2.13.89L5 2.28a.972.972 0 0 0-1.36 0L2.28 3.64c-.37.38-.37.98 0 1.36l.87.87c-.39.66-.69 1.37-.89 2.13H1c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h1.26c.19.73.48 1.42.85 2.06l-.94.94a.996.996 0 0 0 0 1.41l1.41 1.41c.39.39 1.02.39 1.41 0l.94-.94c.64.38 1.33.66 2.06.85V19c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-1.26c.76-.2 1.47-.5 2.13-.89l.88.87c.37.37.98.37 1.36 0l1.36-1.36c.37-.38.37-.98 0-1.36l-.87-.87c.4-.65.7-1.37.89-2.13H19c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1zm-9 7c-2.76 0-5-2.24-5-5s2.24-5 5-5v10z"],control:["M17 10h-7v7h7v-7zm0-7h-7v6h7V3zM9 3H3v14h6V3zm10-3H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H2V2h16v16z"],"credit-card":["M19 3H1c-.55 0-1 .45-1 1v2h20V4c0-.55-.45-1-1-1zM0 16c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V8H0v8zm6.5-2h7c.28 0 .5.22.5.5s-.22.5-.5.5h-7c-.28 0-.5-.22-.5-.5s.22-.5.5-.5zm-4 0h2c.28 0 .5.22.5.5s-.22.5-.5.5h-2c-.28 0-.5-.22-.5-.5s.22-.5.5-.5z"],cross:["M11.41 10l4.29-4.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L10 8.59l-4.29-4.3a1.003 1.003 0 0 0-1.42 1.42L8.59 10 4.3 14.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l4.29-4.3 4.29 4.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L11.41 10z"],crown:["M2 8l4 2 4-5 4 5 4-2-1 7H3L2 8zm8-6a1 1 0 1 1 0 2 1 1 0 0 1 0-2zM1 5a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm18 0a1 1 0 1 1 0 2 1 1 0 0 1 0-2zM3 16h14v2H3v-2z"],cube:["M1.953 4.481l7.41-4.02c.394-.215.88-.215 1.275 0l7.409 4.02L10 9.22 1.953 4.48zm-.817.68L9.5 10.085v9.281a1.316 1.316 0 0 1-.138-.064l-7.714-4.186A1.211 1.211 0 0 1 1 14.057v-8.35c0-.193.048-.38.136-.547zm17.728 0c.088.166.136.353.136.546v8.35c0 .438-.247.842-.648 1.06l-7.714 4.186c-.045.024-.091.046-.138.064v-9.281l8.364-4.926z"],"cube-add":["M17 3h2a1 1 0 0 1 0 2h-2v2a1 1 0 0 1-2 0V5h-2a1 1 0 0 1 0-2h2V1a1 1 0 0 1 2 0v2zm-3.969 4.435L10 9.22 1.953 4.48l7.41-4.02c.394-.215.88-.215 1.275 0l1.33.721A3.001 3.001 0 0 0 13 7c0 .148.01.293.031.435zm.319.972A3 3 0 0 0 19 7v7.057c0 .438-.247.842-.648 1.06l-7.714 4.186c-.045.024-.091.046-.138.064v-9.281l2.85-1.679zM1.136 5.16L9.5 10.086v9.281a1.316 1.316 0 0 1-.138-.064l-7.714-4.186A1.211 1.211 0 0 1 1 14.057v-8.35c0-.193.048-.38.136-.547z"],"cube-remove":["M11.968 1.182A3.001 3.001 0 0 0 13 7h.77L10 9.22 1.953 4.48l7.41-4.02c.394-.215.88-.215 1.275 0l1.33.721zM19 7v7.057c0 .438-.247.842-.648 1.06l-7.714 4.186c-.045.024-.091.046-.138.064v-9.281L15.74 7H19zM1.136 5.16L9.5 10.086v9.281a1.316 1.316 0 0 1-.138-.064l-7.714-4.186A1.211 1.211 0 0 1 1 14.057v-8.35c0-.193.048-.38.136-.547zM13 3h6a1 1 0 0 1 0 2h-6a1 1 0 0 1 0-2z"],"curved-range-chart":["M19 16H3.02l2.14-1.74c2.25 1.7 7.33.46 11.83-2.99l-1.29-1.5c-3.56 2.74-7.31 4.03-8.93 3.19l10.55-8.57-.63-.78-10.59 8.6c-.64-1.64 1.46-4.91 5.09-7.7L9.9 3.01c-4.6 3.54-6.91 8.12-5.41 10.51L2 15.54V3c0-.55-.45-1-1-1s-1 .45-1 1v14a.998.998 0 0 0 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],cut:["M16 2s.72-1.28 0-2l-5.29 6.25 1.28 1.54L16 2zm.08 10c-.55 0-1.07.12-1.54.32L4.31 0c-.7.72 0 2 0 2l4.45 6.56-3.19 3.77C5.09 12.12 4.56 12 4 12c-2.21 0-4 1.79-4 4s1.79 4 4 4 4-1.79 4-4c0-.65-.17-1.26-.45-1.8l2.54-3.67 2.49 3.67c-.27.54-.44 1.15-.44 1.8 0 2.21 1.76 4 3.92 4 2.17 0 3.92-1.79 3.92-4 .02-2.21-1.74-4-3.9-4zM4 18c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zm12.08 0c-1.08 0-1.96-.9-1.96-2s.88-2 1.96-2 1.96.9 1.96 2-.88 2-1.96 2z"],dashboard:["M6 5c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zM4 9c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm6-4c.55 0 1-.45 1-1s-.45-1-1-1-1 .45-1 1 .45 1 1 1zm0-5C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8zm6-9c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm-8 5c0 1.1.9 2 2 2s2-.9 2-2c0-.33-2-8-2-8s-2 7.67-2 8zm6-9c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1z"],database:["M2.01 5.1v5.4c0 1.38 3.58 2.5 8 2.5s8-1.12 8-2.5V5.1c-1.49 1.13-4.51 1.9-8 1.9-3.48 0-6.5-.77-8-1.9zm8 .9c4.42 0 8-1.12 8-2.5s-3.58-2.5-8-2.5-8 1.12-8 2.5S5.6 6 10.01 6zm-8 6.1v5.4c0 1.38 3.58 2.5 8 2.5s8-1.12 8-2.5v-5.4c-1.49 1.13-4.51 1.9-8 1.9-3.48 0-6.5-.77-8-1.9z"],delete:["M15 6a1.003 1.003 0 0 0-1.71-.71L10 8.59l-3.29-3.3a1.003 1.003 0 0 0-1.42 1.42L8.59 10 5.3 13.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3.29-3.3 3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L11.41 10l3.29-3.29c.19-.18.3-.43.3-.71zm-5-6C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"],delta:["M10 0L0 20h20L10 0zM9 6l6 12H3L9 6z"],"derive-column":["M7.1 8.2h-.99c.28-1.11.66-1.92 1.12-2.43.28-.32.56-.48.83-.48.05 0 .1.02.13.05.03.03.05.07.05.12 0 .04-.04.13-.11.25a.64.64 0 0 0-.12.35c0 .15.06.28.18.39.12.11.27.16.45.16.2 0 .36-.07.49-.2s.2-.31.2-.54c0-.26-.1-.47-.3-.63-.19-.16-.51-.24-.95-.24-.68 0-1.3.19-1.85.58-.56.38-1.09 1.02-1.59 1.91-.17.3-.34.5-.49.59-.15.08-.4.13-.74.12l-.23.77h.95l-1.39 5.24c-.23.86-.39 1.39-.47 1.59-.12.29-.3.54-.54.75-.1.08-.21.12-.35.12-.04 0-.07-.01-.1-.03l-.03-.04c0-.02.03-.07.1-.13.07-.07.1-.17.1-.31 0-.15-.05-.28-.16-.38-.11-.1-.27-.15-.47-.15-.25 0-.44.07-.59.2-.15.12-.23.28-.23.46 0 .19.09.36.27.5.19.14.47.21.86.21.61 0 1.16-.15 1.63-.46.48-.31.89-.78 1.25-1.43.35-.64.72-1.68 1.09-3.11l.8-3.03h.96l.24-.77zM19 0h-9c-.55 0-1 .45-1 1v3h2V2h7v16h-7v-2H9v3c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-8.79 13.49c.15.28.32.49.52.61.19.12.44.19.73.19.28 0 .57-.1.86-.3.38-.25.77-.69 1.17-1.31l-.25-.14c-.27.37-.48.6-.61.69-.09.06-.19.09-.31.09-.14 0-.28-.09-.42-.26-.23-.29-.54-1.09-.93-2.4.35-.59.64-.97.87-1.15.17-.13.35-.2.55-.2.07 0 .2.03.39.08s.36.08.5.08c.2 0 .37-.07.5-.2.15-.14.22-.31.22-.52 0-.22-.07-.4-.2-.53s-.33-.2-.58-.2c-.22 0-.43.05-.63.15-.2.1-.45.32-.75.67-.23.25-.56.7-1.01 1.33a6.52 6.52 0 0 0-.91-2.15l-2.39.39-.05.25c.18-.03.33-.05.45-.05.24 0 .43.1.59.3.25.31.59 1.24 1.02 2.8-.34.44-.58.73-.7.87-.21.22-.38.36-.52.43-.1.05-.22.08-.35.08-.1 0-.26-.05-.49-.16a1.01 1.01 0 0 0-.42-.11c-.23 0-.42.07-.57.22-.15.14-.23.33-.23.55 0 .21.07.38.21.51.14.13.33.2.56.2.23 0 .44-.05.64-.14.2-.09.45-.29.75-.59s.72-.78 1.25-1.43c.21.61.39 1.06.54 1.35z"],desktop:["M19 0H1C.45 0 0 .45 0 1v13c0 .55.45 1 1 1h5.67l-.5 3H5c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1h-1.17l-.5-3H19c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 13H2V2h16v11z"],"diagram-tree":["M19 10v5h-2v-4h-6v4H9v-4H3v4H1v-5a1 1 0 0 1 1-1h7V5h2v4h7a1 1 0 0 1 1 1zM1 16h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H1a1 1 0 0 1-1-1v-2a1 1 0 0 1 1-1zm16 0h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1h-2a1 1 0 0 1-1-1v-2a1 1 0 0 1 1-1zm-8 0h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H9a1 1 0 0 1-1-1v-2a1 1 0 0 1 1-1zM9 0h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H9a1 1 0 0 1-1-1V1a1 1 0 0 1 1-1z"],"direction-left":["M20 3.02l-20 7 20 7-5-7z"],"direction-right":["M20 10.02l-20-7 5 7-5 7z"],disable:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zM2 10c0-4.42 3.58-8 8-8 1.85 0 3.55.63 4.9 1.69L3.69 14.9A7.902 7.902 0 0 1 2 10zm8 8c-1.85 0-3.55-.63-4.9-1.69L16.31 5.1A7.902 7.902 0 0 1 18 10c0 4.42-3.58 8-8 8z"],document:["M11.98 0h-8c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V6l-6-6zm4 18h-11V2h6v5h5v11z"],"document-open":["M8 15c0 .55.45 1 1 1s1-.45 1-1v-5c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1s.45 1 1 1h2.59L1.3 16.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L8 12.41V15zm5-15H5c-.55 0-1 .45-1 1v6h2V2h6v5h5v11H6v-.76L4.04 19.2c.1.45.48.8.96.8h13c.55 0 1-.45 1-1V6l-6-6z"],"document-share":["M14.09 10.09c-.31.31-.67.57-1.09.72V18H2V2h6v5h1.18c.15-.42.39-.8.7-1.11v-.01l2.45-2.45c-.42-.29-.78-.65-1.01-1.11L9 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V9.24l-.88.88-.03-.03zM19 0h-5c-.55 0-1 .45-1 1s.45 1 1 1h2.59L11.3 7.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L18 3.41V6c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1z"],dollar:["M15.57 11.19c-.27-.51-.63-.93-1.07-1.26-.44-.33-.95-.6-1.51-.79-.56-.2-1.14-.36-1.72-.5-.6-.14-1.19-.26-1.75-.38-.57-.13-1.07-.27-1.51-.44-.44-.17-.8-.38-1.07-.63s-.41-.59-.41-1c0-.33.09-.6.28-.81.19-.21.42-.36.69-.47.27-.11.57-.18.88-.22.31-.04.58-.06.8-.06.71 0 1.35.14 1.9.41.55.27.91.81 1.06 1.62h3.36c-.09-.84-.32-1.56-.69-2.16-.37-.6-.83-1.08-1.38-1.45-.56-.37-1.18-.64-1.86-.81-.19-.05-.38-.07-.57-.1V1c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v1.1c-.22.03-.43.05-.66.1-.73.13-1.39.37-1.98.71-.6.34-1.09.8-1.47 1.35-.39.56-.58 1.25-.58 2.08 0 .76.13 1.41.4 1.93.26.52.62.95 1.06 1.28.44.33.94.6 1.5.79.55.2 1.13.36 1.74.5.58.14 1.16.26 1.72.38s1.07.26 1.51.43c.44.17.8.39 1.09.66.28.27.43.63.45 1.06.02.43-.08.78-.3 1.04-.22.26-.49.47-.83.6-.34.14-.7.23-1.09.28-.39.05-.73.07-1.03.07-.87 0-1.61-.2-2.23-.59-.62-.39-.98-1.08-1.07-2.06H3c.02.9.19 1.68.52 2.34.33.66.78 1.21 1.35 1.65.57.44 1.25.77 2.03.98.35.1.71.16 1.08.21V19c0 .55.45 1 1 1h1c.55 0 1-.45 1-1v-1.13c.25-.04.5-.07.76-.13.77-.18 1.47-.46 2.1-.85.63-.39 1.14-.9 1.54-1.53.4-.63.59-1.39.59-2.29.01-.75-.13-1.37-.4-1.88z"],dot:["M10 6a4 4 0 1 0 0 8 4 4 0 1 0 0-8z"],"double-caret-horizontal":["M8 4c-.24 0-.46.1-.63.24l-.01-.01-6 5 .01.01c-.22.19-.37.45-.37.76s.15.57.37.76l-.01.01 6 5 .01-.01c.17.14.39.24.63.24.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm11 6c0-.31-.15-.57-.37-.76l.01-.01-6-5-.01.01C12.46 4.1 12.24 4 12 4c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1 .24 0 .46-.1.63-.24l.01.01 6-5-.01-.01c.22-.19.37-.45.37-.76z"],"double-caret-vertical":["M5 9h10c.55 0 1-.45 1-1 0-.24-.1-.46-.24-.63l.01-.01-5-6-.01.01C10.57 1.15 10.31 1 10 1s-.57.15-.76.37l-.01-.01-5 6 .01.01C4.1 7.54 4 7.76 4 8c0 .55.45 1 1 1zm10 2H5c-.55 0-1 .45-1 1 0 .24.1.46.24.63l-.01.01 5 6 .01-.01c.19.22.45.37.76.37s.57-.15.76-.37l.01.01 5-6-.01-.01c.14-.17.24-.39.24-.63 0-.55-.45-1-1-1z"],"double-chevron-down":["M9.29 10.71c.18.18.43.29.71.29s.53-.11.71-.29l6-6a1.003 1.003 0 0 0-1.42-1.42L10 8.59l-5.29-5.3a1.003 1.003 0 0 0-1.42 1.42l6 6zM16 9c-.28 0-.53.11-.71.29L10 14.59l-5.29-5.3a1.003 1.003 0 0 0-1.42 1.42l6 6c.18.18.43.29.71.29s.53-.11.71-.29l6-6A1.003 1.003 0 0 0 16 9z"],"double-chevron-left":["M5.41 10l5.29-5.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-6 6c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l6 6a1.003 1.003 0 0 0 1.42-1.42L5.41 10zm6 0l5.29-5.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-6 6c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l6 6a1.003 1.003 0 0 0 1.42-1.42L11.41 10z"],"double-chevron-right":["M11 10c0-.28-.11-.53-.29-.71l-6-6a1.003 1.003 0 0 0-1.42 1.42L8.59 10 3.3 15.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l6-6c.18-.18.29-.43.29-.71zm5.71-.71l-6-6a1.003 1.003 0 0 0-1.42 1.42l5.3 5.29-5.29 5.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l6-6c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"double-chevron-up":["M4 11c.28 0 .53-.11.71-.29L10 5.41l5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-6-6A.997.997 0 0 0 10 3c-.28 0-.53.11-.71.29l-6 6A1.003 1.003 0 0 0 4 11zm6.71-1.71A.997.997 0 0 0 10 9c-.28 0-.53.11-.71.29l-6 6a1.003 1.003 0 0 0 1.42 1.42l5.29-5.3 5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-6-6z"],"doughnut-chart":["M16 10c0 3.31-2.69 6-6 6s-6-2.69-6-6 2.69-6 6-6V0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10h-4zm-.09-1h4.04C19.48 4.28 15.72.52 11 .05V4.1A5.98 5.98 0 0 1 15.91 9z"],download:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm4.71 11.71l-4 4c-.18.18-.43.29-.71.29s-.53-.11-.71-.29l-4-4a1.003 1.003 0 0 1 1.42-1.42L9 12.59V5c0-.55.45-1 1-1s1 .45 1 1v7.59l2.29-2.29c.18-.19.43-.3.71-.3a1.003 1.003 0 0 1 .71 1.71z"],"drag-handle-horizontal":["M7.5 11c-.83 0-1.5.67-1.5 1.5S6.67 14 7.5 14 9 13.33 9 12.5 8.33 11 7.5 11zm-5-5C1.67 6 1 6.67 1 7.5S1.67 9 2.5 9 4 8.33 4 7.5 3.33 6 2.5 6zm0 5c-.83 0-1.5.67-1.5 1.5S1.67 14 2.5 14 4 13.33 4 12.5 3.33 11 2.5 11zm15-2c.83 0 1.5-.67 1.5-1.5S18.33 6 17.5 6 16 6.67 16 7.5 16.67 9 17.5 9zm-5 2c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5zm5 0c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5zm-10-5C6.67 6 6 6.67 6 7.5S6.67 9 7.5 9 9 8.33 9 7.5 8.33 6 7.5 6zm5 0c-.83 0-1.5.67-1.5 1.5S11.67 9 12.5 9 14 8.33 14 7.5 13.33 6 12.5 6z"],"drag-handle-vertical":["M7.5 6C6.67 6 6 6.67 6 7.5S6.67 9 7.5 9 9 8.33 9 7.5 8.33 6 7.5 6zm0 5c-.83 0-1.5.67-1.5 1.5S6.67 14 7.5 14 9 13.33 9 12.5 8.33 11 7.5 11zm0 5c-.83 0-1.5.67-1.5 1.5S6.67 19 7.5 19 9 18.33 9 17.5 8.33 16 7.5 16zm5-12c.83 0 1.5-.67 1.5-1.5S13.33 1 12.5 1 11 1.67 11 2.5 11.67 4 12.5 4zm-5-3C6.67 1 6 1.67 6 2.5S6.67 4 7.5 4 9 3.33 9 2.5 8.33 1 7.5 1zm5 10c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5zm0 5c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5zm0-10c-.83 0-1.5.67-1.5 1.5S11.67 9 12.5 9 14 8.33 14 7.5 13.33 6 12.5 6z"],draw:["M17.7 12.7c0-.1 0-.2-.1-.3l-2-7c-.1-.3-.3-.6-.6-.7L1.8 0l-.6.5L7.7 7c.3-.2.6-.3 1-.3 1.1 0 2 .9 2 2s-.9 2-2 2-2-.9-2-2c0-.4.1-.7.3-1L.5 1.2l-.5.6L4.7 15c.1.3.4.5.7.6l7 2c.1 0 .2.1.3.1.3 0 .5-.1.7-.3l4-4c.2-.2.3-.5.3-.7zm1 1c-.3 0-.5.1-.7.3l-4 4c-.2.2-.3.4-.3.7 0 .5.4 1 1 1 .3 0 .5-.1.7-.3l4-4c.2-.2.3-.4.3-.7 0-.6-.5-1-1-1z"],"drive-time":["M20.01 7.7c0-.63-.5-1.14-1.1-1.14h-1.32l-.95-2.57c-.24-.64-.95-1.31-1.59-1.5 0 0-1.65-.49-5.05-.49s-5.04.49-5.04.49c-.63.19-1.35.86-1.59 1.5l-.95 2.57H1.1C.5 6.56 0 7.07 0 7.7c0 .63.5 1.14 1.1 1.14h.47l-.34.91c-.24.64-.43 1.72-.43 2.4v5.39c0 .8.63 1.45 1.4 1.45.77 0 1.4-.65 1.4-1.45v-.83h12.8v.83c0 .8.63 1.45 1.4 1.45s1.4-.65 1.4-1.45v-5.39c0-.68-.19-1.77-.43-2.4l-.34-.91h.47c.61 0 1.11-.51 1.11-1.14zm-16.47.34l1.12-3.16c.08-.22.32-.39.54-.39h9.6c.22 0 .46.17.54.39l1.12 3.16c.08.21-.04.39-.26.39H3.8c-.22-.01-.34-.18-.26-.39zm.96 4.94c-.83 0-1.5-.67-1.5-1.5s.67-1.5 1.5-1.5 1.5.68 1.5 1.5c0 .83-.67 1.5-1.5 1.5zm11 0c-.83 0-1.5-.67-1.5-1.5s.67-1.5 1.5-1.5 1.5.67 1.5 1.5-.67 1.5-1.5 1.5z"],duplicate:["M15 4H1c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm-1 14H2V6h12v12zm5-18H5c-.55 0-1 .45-1 1v2h2V2h12v12h-1v2h2c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],edit:["M4.59 12.59l2.83 2.83 7.65-7.65-2.83-2.83-7.65 7.65zM2 18l4.41-1.59-2.81-2.79L2 18zM16 2c-.55 0-1.05.22-1.41.59l-1.65 1.65 2.83 2.83 1.65-1.65A2.006 2.006 0 0 0 16 2z"],eject:["M4 12h12c.55 0 1-.45 1-1 0-.25-.1-.47-.25-.64l.01-.01-6-7-.01.01C10.57 3.14 10.3 3 10 3s-.57.14-.75.36l-.01-.01-6 7 .01.01c-.15.17-.25.39-.25.64 0 .55.45 1 1 1zm12 1H4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h12c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1z"],endorsed:["M19.83 9.38L18.81 7.6V5.62c0-.45-.23-.85-.61-1.08l-1.71-1-1.02-1.76a1.25 1.25 0 0 0-1.08-.61h-2.03l-1.74-1c-.38-.23-.87-.23-1.25 0l-1.74 1H5.65c-.44 0-.85.23-1.08.61L3.58 3.5l-1.8 1.04c-.38.24-.62.64-.62 1.08v2.06L.17 9.4c-.11.19-.17.4-.17.61s.06.42.17.61l.99 1.72v2.06c0 .45.23.85.61 1.08l1.78 1.02.99 1.72c.23.38.63.61 1.08.61h1.99l1.74 1c.19.11.41.17.62.17.21 0 .42-.06.61-.17l1.74-1h2.03c.44 0 .85-.23 1.08-.61l1.02-1.76 1.71-1c.38-.23.61-.64.61-1.08v-1.97l1.02-1.78c.27-.38.27-.85.04-1.25zm-5.08-.71l-5.01 5.01c-.18.18-.43.29-.71.29-.28 0-.53-.11-.71-.29l-3.01-3.01a1.003 1.003 0 0 1 1.42-1.42l2.3 2.3 4.31-4.3a1.003 1.003 0 0 1 1.71.71c0 .28-.12.53-.3.71z"],envelope:["M0 4.01v11.91l6.27-6.27L0 4.01zm18.91-1.03H1.09L10 10.97l8.91-7.99zm-5.18 6.66L20 15.92V4.01l-6.27 5.63zm-3.23 2.9c-.13.12-.31.19-.5.19s-.37-.07-.5-.19l-2.11-1.89-6.33 6.33h17.88l-6.33-6.33-2.11 1.89z"],equals:["M4 7h12a1 1 0 0 1 0 2H4a1 1 0 1 1 0-2zm0 4h12a1 1 0 0 1 0 2H4a1 1 0 0 1 0-2z"],eraser:["M18.71 8.43c.39-.4.39-1.05 0-1.45l-5.53-5.72a.967.967 0 0 0-1.4 0L1.29 12.1c-.39.4-.39 1.05 0 1.45l4.25 4.39 2.13 2.05h9.27c.02 0 .03.01.05.01.55 0 1-.45 1-1s-.45-1-1-1H9.46l.05-.05h.01l.81-.84 8.38-8.68zM7.52 17.94l-4.95-5.12 4.46-4.61 4.95 5.12-4.46 4.61z"],error:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm1 16H9v-2h2v2zm0-3H9V4h2v9z"],euro:["M8.89 4.47c.56-.31 1.23-.47 2.03-.47.44 0 .85.07 1.25.22.4.14.76.35 1.07.6.17.14.33.3.47.47l2.32-2.32c-.16-.15-.3-.32-.47-.46-.62-.49-1.33-.87-2.12-1.13-.8-.25-1.64-.38-2.52-.38-1.24 0-2.35.22-3.33.66-.99.44-1.82 1.05-2.49 1.82-.68.78-1.2 1.68-1.56 2.72-.09.26-.13.54-.2.8H2c-.55 0-1 .45-1 1s.45 1 1 1h1.04c-.01.2-.04.38-.04.58 0 .15.03.28.03.42H2c-.55 0-1 .45-1 1s.45 1 1 1h1.31c.07.3.13.6.23.89.36 1.02.88 1.92 1.56 2.67.68.76 1.51 1.35 2.49 1.79.98.43 2.09.65 3.33.65.99 0 1.9-.15 2.73-.46.83-.3 1.55-.74 2.17-1.32.03-.03.05-.06.08-.09l-2.41-2.15c-.01.01-.02.02-.02.03-.61.67-1.46 1-2.54 1-.8 0-1.47-.16-2.03-.47-.56-.31-1.01-.72-1.35-1.24-.28-.38-.47-.83-.63-1.3H12c.55 0 1-.45 1-1s-.45-1-1-1H6.56c0-.14-.02-.28-.02-.42 0-.2.02-.39.03-.58H13c.55 0 1-.45 1-1s-.45-1-1-1H6.94c.15-.46.34-.9.59-1.28.35-.52.8-.94 1.36-1.25zM18 11.38v0z"],exchange:["M2.5 8a2.5 2.5 0 0 0 0 5 2.5 2.5 0 0 0 0-5zm10.35 3.15a.495.495 0 1 0-.7.7L13.3 13H5.5c-.28 0-.5.22-.5.5s.22.5.5.5h7.79l-1.15 1.15c-.08.09-.14.21-.14.35a.495.495 0 0 0 .85.35l2-2c.09-.09.15-.21.15-.35s-.06-.26-.15-.35l-2-2zM17.5 8a2.5 2.5 0 0 0 0 5 2.5 2.5 0 0 0 0-5zM7.15 9.85a.495.495 0 1 0 .7-.7L6.71 8h7.79c.28 0 .5-.22.5-.5s-.22-.5-.5-.5H6.71l1.15-1.15c.08-.09.14-.21.14-.35a.495.495 0 0 0-.85-.35l-2 2c-.09.09-.15.21-.15.35s.06.26.15.35l2 2z"],"exclude-row":["M1 3h18c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zM0 13a1.003 1.003 0 0 0 1.71.71L4 11.41l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L5.41 10 7.7 7.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L4 8.59l-2.29-2.3A1.003 1.003 0 0 0 .29 7.71L2.59 10 .3 12.29c-.19.18-.3.43-.3.71zm18-5h-7c-1.1 0-2 .9-2 2s.9 2 2 2h7c1.1 0 2-.9 2-2s-.9-2-2-2zm1 9H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],"expand-all":["M4 9c.28 0 .53-.11.71-.29L10 3.41l5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-6-6C10.53 1.11 10.28 1 10 1s-.53.11-.71.29l-6 6A1.003 1.003 0 0 0 4 9zm12 2c-.28 0-.53.11-.71.29L10 16.59 4.71 11.3A.965.965 0 0 0 4 11a1.003 1.003 0 0 0-.71 1.71l6 6c.18.18.43.29.71.29s.53-.11.71-.29l6-6A1.003 1.003 0 0 0 16 11z"],export:["M5 7c.28 0 .53-.11.71-.29L9 3.41V15c0 .55.45 1 1 1s1-.45 1-1V3.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-5-5C10.53.11 10.28 0 10 0s-.53.11-.71.29l-5 5A1.003 1.003 0 0 0 5 7zm14 7c-.55 0-1 .45-1 1v3H2v-3c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h18c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1z"],"eye-off":["M20 9.96v-.03-.01-.02-.02a.794.794 0 0 0-.21-.43c-.55-.69-1.19-1.3-1.85-1.87l-3.93 2.62a3.966 3.966 0 0 1-3.96 3.77c-.47 0-.91-.1-1.33-.24l-2.24 1.49c.52.21 1.05.39 1.6.51 1.21.27 2.43.28 3.64.05 1.11-.21 2.17-.64 3.17-1.18 1.56-.84 2.99-2 4.23-3.3.23-.24.46-.49.67-.75a.87.87 0 0 0 .21-.43v-.02-.02-.01-.03V10v-.04zm-.46-5.14c.27-.18.46-.47.46-.82 0-.55-.45-1-1-1-.21 0-.39.08-.54.18l-.01-.02L15 5.46c-.95-.53-1.95-.96-3.01-1.2a9.158 9.158 0 0 0-3.65-.04c-1.11.21-2.17.64-3.17 1.18-1.56.84-2.99 2-4.23 3.3-.23.24-.46.48-.67.75-.27.34-.27.76 0 1.1.64.79 1.39 1.5 2.16 2.15.26.21.52.41.79.61L.44 15.16l.01.02A1 1 0 0 0 0 16c0 .55.45 1 1 1 .21 0 .39-.08.54-.18l.01.02 18-12-.01-.02zm-8.67 3.4c-.25-.12-.53-.2-.83-.2-1.1 0-1.99.89-1.99 1.99 0 .03.02.06.02.09l-1.78 1.19c-.14-.4-.22-.83-.22-1.28 0-2.19 1.78-3.97 3.98-3.97 1.01 0 1.91.38 2.61 1l-1.79 1.18z"],"eye-on":["M13.3 8.71c.18.18.43.29.71.29s.53-.11.71-.29l4.99-5a1.003 1.003 0 0 0-1.42-1.42L14 6.58l-2.29-2.29a.956.956 0 0 0-.7-.29 1.003 1.003 0 0 0-.71 1.71l3 3zM20 9.96v-.03-.01-.02-.02a.823.823 0 0 0-.21-.44c-.44-.55-.94-1.05-1.46-1.52l-2.2 2.2c-.55.54-1.3.88-2.12.88-.05 0-.09-.01-.14-.01a3.978 3.978 0 0 1-3.86 3.02 4.007 4.007 0 0 1-1.66-7.65A2.97 2.97 0 0 1 8.02 5c0-.28.05-.54.12-.8-1.05.22-2.07.64-3.02 1.15-1.57.85-3 2.02-4.24 3.33-.23.25-.46.5-.67.76-.28.35-.28.77 0 1.12.64.8 1.4 1.52 2.17 2.17 1.66 1.41 3.56 2.58 5.66 3.06 1.21.27 2.43.29 3.65.05 1.11-.21 2.18-.65 3.18-1.19 1.57-.85 3-2.02 4.24-3.33.23-.24.46-.49.67-.76.11-.12.18-.27.21-.44v-.02-.02-.01-.03V10c.01-.01.01-.03.01-.04zm-9.99 2.05c1.03 0 1.87-.79 1.98-1.8l-.09-.09-.01.01-2.1-2.11c-1 .11-1.77.95-1.77 1.98-.01 1.11.89 2.01 1.99 2.01z"],"eye-open":["M10.01 7.984A2.008 2.008 0 0 0 8.012 9.99c0 1.103.9 2.006 1.998 2.006a2.008 2.008 0 0 0 1.998-2.006c0-1.103-.9-2.006-1.998-2.006zM20 9.96v-.03-.01-.02-.02a.827.827 0 0 0-.21-.442c-.64-.802-1.398-1.514-2.168-2.166-1.658-1.404-3.566-2.587-5.664-3.058a8.982 8.982 0 0 0-3.656-.05c-1.11.2-2.178.641-3.177 1.183-1.569.852-2.997 2.016-4.246 3.33-.23.25-.46.49-.67.761-.279.351-.279.773 0 1.124.64.802 1.4 1.514 2.169 2.166 1.658 1.404 3.566 2.577 5.664 3.058 1.209.271 2.438.281 3.656.05 1.11-.21 2.178-.651 3.177-1.193 1.569-.852 2.997-2.016 4.246-3.33.23-.24.46-.49.67-.751.11-.12.179-.271.209-.442v-.02-.02-.01-.03V10v-.04zM10.01 14A4.003 4.003 0 0 1 6.014 9.99a4.003 4.003 0 0 1 3.996-4.011 4.003 4.003 0 0 1 3.996 4.011 4.003 4.003 0 0 1-3.996 4.011z"],"fast-backward":["M18 3c-.23 0-.42.09-.59.21l-.01-.01L11 8V4c0-.55-.45-1-1-1-.23 0-.42.09-.59.21L9.4 3.2l-8 6 .01.01C1.17 9.4 1 9.67 1 10s.17.6.41.79l-.01.01 8 6 .01-.01c.17.12.36.21.59.21.55 0 1-.45 1-1v-4l6.4 4.8.01-.01c.17.12.36.21.59.21.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"fast-forward":["M19 10c0-.33-.17-.6-.41-.79l.01-.01-8-6-.01.01C10.42 3.09 10.23 3 10 3c-.55 0-1 .45-1 1v4L2.6 3.2l-.01.01C2.42 3.09 2.23 3 2 3c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1 .23 0 .42-.09.59-.21l.01.01L9 12v4c0 .55.45 1 1 1 .23 0 .42-.09.59-.21l.01.01 8-6-.01-.01c.24-.19.41-.46.41-.79z"],feed:["M2.5 15a2.5 2.5 0 0 0 0 5 2.5 2.5 0 0 0 0-5zm.5-5c-.55 0-1 .45-1 1s.45 1 1 1c2.76 0 5 2.24 5 5 0 .55.45 1 1 1s1-.45 1-1c0-3.87-3.13-7-7-7zM3 0c-.55 0-1 .45-1 1s.45 1 1 1c8.28 0 15 6.72 15 15 0 .55.45 1 1 1s1-.45 1-1C20 7.61 12.39 0 3 0zm0 5c-.55 0-1 .45-1 1s.45 1 1 1c5.52 0 10 4.48 10 10 0 .55.45 1 1 1s1-.45 1-1C15 10.37 9.63 5 3 5z"],"feed-subscribed":["M2.5 15a2.5 2.5 0 0 0 0 5 2.5 2.5 0 0 0 0-5zM3 2c1.76 0 3.44.31 5.01.87.03-.71.31-1.35.75-1.85C6.96.37 5.03 0 3 0c-.55 0-1 .45-1 1s.45 1 1 1zm10.32 4.67a.99.99 0 0 0 1.4 0l4.98-4.98c.19-.17.3-.42.3-.7 0-.55-.45-1-1-1a.99.99 0 0 0-.7.29l-4.27 4.27-2.28-2.28a.99.99 0 0 0-.7-.29c-.55 0-.99.45-.99 1 0 .28.11.52.29.7l2.97 2.99zM3 10c-.55 0-1 .45-1 1s.45 1 1 1c2.76 0 5 2.24 5 5 0 .55.45 1 1 1s1-.45 1-1c0-3.87-3.13-7-7-7zm13.94-2.69l-.82.82-.02-.02c-.2.2-.42.37-.67.51A14.8 14.8 0 0 1 18 17c0 .55.45 1 1 1s1-.45 1-1c0-3.61-1.14-6.94-3.06-9.69zM3 5c-.55 0-1 .45-1 1s.45 1 1 1c5.52 0 10 4.48 10 10 0 .55.45 1 1 1s1-.45 1-1C15 10.37 9.63 5 3 5z"],film:["M19 2h-5v3H6V2H1c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h5v-3h8v3h5c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zM4 17H2v-2h2v2zm0-3H2v-2h2v2zm0-3H2V9h2v2zm0-3H2V6h2v2zm0-3H2V3h2v2zm10 8H6V7h8v6zm4 4h-2v-2h2v2zm0-3h-2v-2h2v2zm0-3h-2V9h2v2zm0-3h-2V6h2v2zm0-3h-2V3h2v2z"],filter:["M18 1H2a1.003 1.003 0 0 0-.71 1.71L7 8.41V18a1.003 1.003 0 0 0 1.71.71l4-4c.18-.18.29-.43.29-.71V8.41l5.71-5.71c.18-.17.29-.42.29-.7 0-.55-.45-1-1-1z"],"filter-keep":["M15 2c0-.55-.45-1-1-1H1a1.003 1.003 0 0 0-.71 1.71L5 7.41V16a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71V7.41l4.71-4.71c.18-.17.29-.42.29-.7zm4 11c-.28 0-.53.11-.71.29L15 16.59l-1.29-1.29A.965.965 0 0 0 13 15a1.003 1.003 0 0 0-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l4-4A1.003 1.003 0 0 0 19 13z"],"filter-list":["M15 2c0-.55-.45-1-1-1H1a1.003 1.003 0 0 0-.71 1.71L5 7.41V16a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71V7.41l4.71-4.71c.18-.17.29-.42.29-.7zm-4 8c0 .55.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1h-7c-.55 0-1 .45-1 1zm8 7h-7c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1zm0-4h-7c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1z"],"filter-open":["M15 2c0 .28-.11.53-.29.7L10 7.41V13c0 .28-.11.53-.29.71l-3 3A1.003 1.003 0 0 1 5 16V7.41L.29 2.71A1.003 1.003 0 0 1 1 1h13c.55 0 1 .45 1 1zm4.707 11.293a1 1 0 0 1 0 1.414l-4 4c-.63.63-1.707.184-1.707-.707v-8c0-.89 1.077-1.337 1.707-.707l4 4z"],"filter-remove":["M15 2c0-.55-.45-1-1-1H1a1.003 1.003 0 0 0-.71 1.71L5 7.41V16a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71V7.41l4.71-4.71c.18-.17.29-.42.29-.7zm2.91 13.5l1.79-1.79c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-1.79 1.79-1.79-1.79a1.003 1.003 0 0 0-1.42 1.42l1.79 1.79-1.79 1.79a1.003 1.003 0 0 0 1.42 1.42l1.79-1.79 1.79 1.79a1.003 1.003 0 0 0 1.42-1.42l-1.8-1.79z"],flag:["M3 3c-.55 0-1 .45-1 1v15c0 .55.45 1 1 1s1-.45 1-1V4c0-.55-.45-1-1-1zm0-3c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm2 4.08v8.28c3.01-2.41 8.67 2.64 13 0V4.08C13.61 7.14 8.01 1 5 4.08z"],flame:["M11.622 0c0 1.71.49 3.077 1.472 4.103C16.364 6.496 18 9.23 18 12.308c0 3.418-1.962 5.983-5.887 7.692 2.887-3 2.453-4.23-.49-8C8.5 13.5 9 14.5 9.5 16.5c-1.048 0-2 0-2.5-.5 0 .684 1.197 2.5 1.952 4-3.924-1.026-8.123-7.18-6.651-7.692.981-.342 2.126-.171 3.434.513C4.1 6.667 6.062 2.393 11.622 0z"],flash:["M4.96 6.37a1.003 1.003 0 0 0 1.42-1.42l-2-2a1.07 1.07 0 0 0-.71-.28 1.003 1.003 0 0 0-.71 1.71l2 1.99zm9.37.3c.28 0 .53-.11.71-.29l2-2a1.003 1.003 0 0 0-1.42-1.42l-2 2a1.003 1.003 0 0 0 .71 1.71zM10 5c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1S9 .45 9 1v3c0 .55.45 1 1 1zm-5 5c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1h3c.55 0 1-.45 1-1zm14-1h-3c-.55 0-1 .45-1 1s.45 1 1 1h3c.55 0 1-.45 1-1s-.45-1-1-1zm-9-3c-2.21 0-4 1.79-4 4s1.79 4 4 4 4-1.79 4-4-1.79-4-4-4zm0 6c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zm5.04 1.63a1.003 1.003 0 0 0-1.42 1.42l2 2a1.003 1.003 0 0 0 1.42-1.42l-2-2zM10 15c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1s1-.45 1-1v-3c0-.55-.45-1-1-1zm-4.33-1.67c-.28 0-.53.11-.71.29l-2 2a1.003 1.003 0 0 0 1.42 1.42l2-2a1.003 1.003 0 0 0-.71-1.71z"],"floppy-disk":["M14 1h-3v5h3V1zm5.71 2.29l-3-3A.997.997 0 0 0 16 0h-1v7H5V0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V4c0-.28-.11-.53-.29-.71zM17 19H3v-8c0-.55.45-1 1-1h12c.55 0 1 .45 1 1v8z"],"flow-branch":["M14.425 7.953a3.98 3.98 0 0 1 .562 2.045 3.98 3.98 0 0 1-.583 2.08L18 15.671V12.98c0-.248.097-.496.29-.689.379-.379 1.047-.38 1.426 0a.94.94 0 0 1 .283.696l-.001 5.049a.957.957 0 0 1-.276.69.955.955 0 0 1-.69.273h-5.059a.971.971 0 0 1-.689-.289 1.026 1.026 0 0 1 0-1.417.972.972 0 0 1 .69-.29h2.702l-3.634-3.573a3.998 3.998 0 0 1-5.924-2.431H1a1 1 0 0 1 0-2h6.12a3.998 3.998 0 0 1 5.96-2.409L16.665 3l-2.694-.001a.972.972 0 0 1-.689-.29 1.035 1.035 0 0 1 0-1.425.94.94 0 0 1 .696-.283l5.05.001c.248 0 .497.083.69.276a.954.954 0 0 1 .272.69l.001 5.052a.971.971 0 0 1-.29.689 1.028 1.028 0 0 1-1.419 0 .972.972 0 0 1-.29-.69V4.323l-3.567 3.63z"],"flow-end":["M12 9.919a3.998 3.998 0 0 1 4-3.92c2.21 0 4 1.79 4 3.997a3.998 3.998 0 0 1-4 3.996 3.998 3.998 0 0 1-4-3.916.967.967 0 0 1-.28.612L7.685 14.71a.958.958 0 0 1-.686.285c-.536 0-.994-.461-.994-.997 0-.273.107-.528.283-.704l2.379-2.302H.98c-.537 0-.976-.46-.976-.996s.44-.992.976-.992h7.676L6.287 6.687a.957.957 0 0 1-.283-.686c0-.536.458-.996.994-.996.274 0 .51.1.686.285l4.027 4.024c.159.158.27.365.29.605z"],"flow-linear":["M5.125 10.997H.976C.439 10.997 0 10.537 0 10c0-.536.44-.993.976-.993h4.148a4.002 4.002 0 0 1 7.752 0h3.776L14.293 6.69a.962.962 0 0 1-.285-.687c0-.537.46-1.001.996-1.001a.96.96 0 0 1 .698.3l4.005 4.015c.176.176.293.41.293.683a.972.972 0 0 1-.283.693L15.702 14.7a.997.997 0 0 1-.698.297c-.537 0-.996-.453-.996-.99 0-.273.107-.517.283-.692l2.371-2.318h-3.787a4.002 4.002 0 0 1-7.75 0z"],"flow-review":["M6.13 9.004A4.005 4.005 0 0 1 10.012 6c1.87 0 3.44 1.278 3.881 3.005h2.768l-2.354-2.317a.97.97 0 0 1-.283-.691c0-.536.462-.995 1-.995.273 0 .517.107.693.283l4 4.041a.97.97 0 0 1 .284.692.956.956 0 0 1-.293.682l-3.991 3.997a.944.944 0 0 1-.694.292c-.537 0-1-.46-1-.997a.97.97 0 0 1 .284-.692l2.345-2.29h-2.765a4.005 4.005 0 0 1-3.875 2.981 4.005 4.005 0 0 1-3.874-2.981H3.349l2.376 2.308a.97.97 0 0 1 .283.691 1 1 0 0 1-.994.983.989.989 0 0 1-.713-.291L.293 10.699A.956.956 0 0 1 0 10.017a.97.97 0 0 1 .283-.692l4.03-4.037a.996.996 0 0 1 .701-.283c.537 0 .994.464.994 1a.97.97 0 0 1-.283.691L3.34 9.004h2.79z"],"flow-review-branch":["M13.04 13.424c-.6.36-1.302.568-2.052.568a4 4 0 0 1-3.868-2.999H3.342l2.372 2.31c.176.176.283.42.283.694 0 .537-.452.998-.988.998a.935.935 0 0 1-.691-.289L.292 10.683A.96.96 0 0 1 0 9.999c0-.274.107-.518.283-.694l4.035-4.04a.973.973 0 0 1 .691-.288c.536 0 .988.47.988 1.007a.975.975 0 0 1-.283.694L3.332 8.984h3.786a4 4 0 0 1 3.87-3.006c.771 0 1.492.22 2.102.599l3.565-3.57-2.538-.003a.974.974 0 0 1-.69-.29c-.38-.38-.38-1.052-.002-1.431A.94.94 0 0 1 14.122 1l4.896.005a.96.96 0 0 1 .69.277c.193.193.27.442.27.69l.005 4.9a.971.971 0 0 1-.289.69 1.023 1.023 0 0 1-1.416 0 .975.975 0 0 1-.29-.691l-.003-2.54-3.554 3.62c.351.596.553 1.291.553 2.034 0 .763-.213 1.477-.583 2.084l3.595 3.595.003-2.54c0-.249.097-.497.29-.69.38-.38 1.05-.381 1.429-.002a.94.94 0 0 1 .282.697l-.005 4.9a.927.927 0 0 1-.277.675.974.974 0 0 1-.69.291L13.974 19a.97.97 0 0 1-.69-.29 1.03 1.03 0 0 1 .002-1.42.974.974 0 0 1 .69-.29l2.696-.003-3.632-3.573z"],flows:["M17.5 7.93a2.5 2.5 0 0 0-2.45 2h-2.3l-4.01-4-.75.75 3.26 3.25h-6.3a2.5 2.5 0 1 0 0 1h6.3l-3.26 3.25.75.75 4.01-4h2.3a2.5 2.5 0 1 0 2.45-3z"],"folder-close":["M0 17c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V7H0v10zM19 4H9.41l-1.7-1.71A.997.997 0 0 0 7 2H1c-.55 0-1 .45-1 1v3h20V5c0-.55-.45-1-1-1z"],"folder-new":["M12.994 7c0 1.655 1.344 3 2.998 3a3.002 3.002 0 0 0 2.999-3H20v10c0 .55-.45 1-1 1H1.01c-.55 0-1-.45-1-1V7h12.984zM10.76 6H0V3c0-.55.45-1 1-1h3.998c.28 0 .53.11.71.29L7.415 4h2.579c0 .768.29 1.469.765 2zm8.23-3c.55 0 1 .45 1 1s-.45 1-1 1h-1.998v2c0 .55-.45 1-1 1s-1-.45-1-1V5h-1.998c-.55 0-1-.45-1-1s.45-1 1-1h1.999V1c0-.55.45-1 .999-1 .55 0 1 .45 1 1v2h1.999z"],"folder-open":["M20 9c0-.55-.45-1-1-1H5c-.43 0-.79.27-.93.65h-.01l-3 8h.01c-.04.11-.07.23-.07.35 0 .55.45 1 1 1h14c.43 0 .79-.27.93-.65h.01l3-8h-.01c.04-.11.07-.23.07-.35zM3.07 7.63C3.22 7.26 3.58 7 4 7h14V5c0-.55-.45-1-1-1H8.41l-1.7-1.71A.997.997 0 0 0 6 2H1c-.55 0-1 .45-1 1v12.31l3.07-7.68z"],"folder-shared":["M11 4H9.41l-1.7-1.71A.997.997 0 0 0 7 2H1c-.55 0-1 .45-1 1v3h11.78C11.3 5.47 11 4.77 11 4zm8-1h-5c-.55 0-1 .45-1 1s.45 1 1 1h2.59L12.3 9.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L18 6.41V9c0 .55.45 1 1 1s1-.45 1-1V4c0-.55-.45-1-1-1zm-2.46 7.7l-1.42 1.42a2.996 2.996 0 1 1-4.24-4.24l.88-.88H0v10c0 .55.45 1 1 1h18c.55 0 1-.45 1-1v-5.18c-.31.11-.65.18-1 .18-1.02 0-1.92-.52-2.46-1.3z"],"folder-shared-open":["M3.07 7.63C3.22 7.26 3.58 7 4 7h7.76l.54-.54A2.97 2.97 0 0 1 11 4H8.41l-1.7-1.71A.997.997 0 0 0 6 2H1c-.55 0-1 .45-1 1v12.31l3.07-7.68zm13.47 3.07l-1.42 1.42A2.996 2.996 0 0 1 10 10c0-.77.3-1.47.78-2H5c-.43 0-.79.27-.93.65h-.01l-3 8h.01c-.04.11-.07.23-.07.35 0 .55.45 1 1 1h14c.43 0 .79-.27.93-.65h.01l2.01-5.36c-1-.01-1.88-.52-2.41-1.29zM19 3h-5c-.55 0-1 .45-1 1s.45 1 1 1h2.59L12.3 9.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L18 6.41V9c0 .55.45 1 1 1s1-.45 1-1V4c0-.55-.45-1-1-1z"],follower:["M11.54 15.92c-1.48-.65-1.28-1.05-1.33-1.59-.01-.07-.01-.15-.01-.23.51-.45.92-1.07 1.19-1.78 0 0 .01-.04.02-.05.06-.15.11-.32.15-.48.34-.07.54-.44.61-.78.08-.14.23-.48.2-.87-.05-.5-.25-.73-.47-.82v-.09c0-.63-.06-1.55-.17-2.15-.02-.17-.06-.33-.11-.5a3.69 3.69 0 0 0-1.29-1.86C9.69 4.25 8.8 4 8.01 4c-.8 0-1.69.25-2.32.73-.61.47-1.06 1.13-1.28 1.86-.05.17-.09.33-.11.5-.12.6-.18 1.51-.18 2.14v.08c-.23.09-.44.32-.49.83-.04.39.12.73.2.87.08.35.28.72.63.78.04.17.09.33.15.48 0 .01.01.02.01.03l.01.01c.27.72.7 1.35 1.22 1.8 0 .07-.01.14-.01.21-.05.54.1.94-1.38 1.59-1.48.65-3.71 1.35-4.16 2.4C-.16 19.38.02 20 .02 20h15.95s.18-.62-.27-1.67c-.46-1.06-2.68-1.75-4.16-2.41zm8.15-12.63l-3-3a.956.956 0 0 0-.7-.29 1.003 1.003 0 0 0-.71 1.71L16.58 3H13c-.55 0-1 .45-1 1s.45 1 1 1h3.58l-1.29 1.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.3-.71z"],following:["M11.55 15.92c-1.48-.65-1.28-1.05-1.33-1.59-.01-.07-.01-.15-.01-.23.51-.45.92-1.07 1.19-1.78 0 0 .01-.04.02-.05.06-.15.11-.32.15-.48.34-.07.54-.44.61-.78.08-.14.23-.48.2-.87-.05-.5-.25-.73-.47-.82v-.09c0-.63-.06-1.55-.17-2.15-.02-.17-.06-.33-.11-.5a3.69 3.69 0 0 0-1.29-1.86C9.7 4.25 8.81 4 8.02 4c-.79 0-1.68.25-2.31.73-.61.47-1.07 1.13-1.29 1.86-.05.16-.09.33-.11.5-.12.6-.18 1.51-.18 2.14v.08c-.23.09-.44.32-.48.83-.04.39.12.73.2.87.08.35.28.72.63.78.04.17.09.33.15.48 0 .01.01.02.01.03l.01.01c.27.72.7 1.35 1.22 1.8 0 .07-.01.14-.01.21-.05.54.1.94-1.38 1.59C3 16.56.77 17.26.32 18.31-.15 19.38.04 20 .04 20h15.95s.18-.62-.27-1.67c-.46-1.06-2.69-1.75-4.17-2.41zM19 3h-3.58l1.29-1.29A1.003 1.003 0 0 0 15.29.29l-3 3c-.17.18-.28.43-.28.71 0 .28.11.53.29.71l3 3c.18.18.43.29.7.29a1.003 1.003 0 0 0 .71-1.71L15.42 5H19c.55 0 1-.45 1-1s-.45-1-1-1z"],font:["M17.93 18.64l-7-18C10.78.27 10.42 0 10 0s-.78.27-.93.64l-7 18c-.04.11-.07.23-.07.36 0 .55.45 1 1 1 .42 0 .78-.27.93-.64L6.41 13h7.19l2.47 6.36c.15.37.51.64.93.64.55 0 1-.45 1-1 0-.13-.03-.25-.07-.36zM7.18 11L10 3.76 12.82 11H7.18z"],fork:["M16.71 11.29a1.003 1.003 0 0 0-1.42 1.42l1.3 1.29h-2.17l-8-8h10.17L15.3 7.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3a1.003 1.003 0 0 0-1.42 1.42L16.59 4H1c-.55 0-1 .45-1 1s.45 1 1 1h2.59l9.71 9.71c.17.18.42.29.7.29h2.59l-1.29 1.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3z"],form:["M2 13v4h4v-4H2zm-1-2h6c.55 0 1 .45 1 1v6c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1v-6c0-.55.45-1 1-1zm11-7h7c.55 0 1 .45 1 1s-.45 1-1 1h-7c-.55 0-1-.45-1-1s.45-1 1-1zM8 1a1.003 1.003 0 0 1 .71 1.71l-5 6C3.53 8.89 3.28 9 3 9s-.53-.11-.71-.29l-2-2a1.003 1.003 0 0 1 1.42-1.42L3 6.59l4.29-5.3C7.47 1.11 7.72 1 8 1zm4 13h7c.55 0 1 .45 1 1s-.45 1-1 1h-7c-.55 0-1-.45-1-1s.45-1 1-1z"],"full-circle":["M9.96 0a10 10 0 1 0 0 20 10 10 0 1 0 0-20z"],"full-stacked-chart":["M15 16h2c.55 0 1-.45 1-1v-5h-4v5c0 .55.45 1 1 1zM12 2c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v4h4V2zm6 4h-4v3h4V6zm0-4c0-.55-.45-1-1-1h-2c-.55 0-1 .45-1 1v3h4V2zm-6 5H8v5h4V7zm-9 9h2c.55 0 1-.45 1-1v-3H2v3c0 .55.45 1 1 1zm6 0h2c.55 0 1-.45 1-1v-2H8v2c0 .55.45 1 1 1zm10 1H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zM6 2c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v3h4V2zm0 4H2v5h4V6z"],fullscreen:["M3.41 2H6c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1v5c0 .55.45 1 1 1s1-.45 1-1V3.41L7.29 8.7c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L3.41 2zM8 11c-.28 0-.53.11-.71.29L2 16.59V14c0-.55-.45-1-1-1s-1 .45-1 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1H3.41l5.29-5.29c.19-.18.3-.43.3-.71 0-.55-.45-1-1-1zM19 0h-5c-.55 0-1 .45-1 1s.45 1 1 1h2.59L11.3 7.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L18 3.41V6c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm0 13c-.55 0-1 .45-1 1v2.59l-5.29-5.29A.965.965 0 0 0 12 11a1.003 1.003 0 0 0-.71 1.71l5.3 5.29H14c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1z"],function:["M10.14 5.82H8.73c.4-1.66.94-2.87 1.6-3.64.4-.48.8-.72 1.18-.72.08 0 .14.02.19.07.05.05.07.1.07.18 0 .07-.05.19-.16.37s-.16.36-.16.52c0 .23.08.43.25.59a.9.9 0 0 0 .64.25c.28 0 .51-.1.7-.3.19-.2.28-.47.28-.81 0-.39-.14-.7-.42-.94-.28-.24-.74-.36-1.36-.36-.97 0-1.86.29-2.65.87-.79.56-1.54 1.52-2.26 2.85-.24.46-.48.75-.7.88-.22.13-.57.19-1.06.19l-.32 1.15H5.9l-1.99 7.85c-.33 1.29-.56 2.09-.67 2.39-.17.44-.43.81-.77 1.12a.74.74 0 0 1-.5.19c-.05 0-.1-.02-.14-.05l-.04-.07c0-.03.05-.1.15-.2.1-.1.15-.26.15-.47 0-.23-.08-.42-.23-.57-.16-.15-.38-.23-.67-.23-.35 0-.63.1-.85.29-.21.2-.32.43-.32.7 0 .29.13.54.39.75.25.22.65.33 1.2.33.88 0 1.66-.23 2.33-.69.68-.46 1.27-1.17 1.78-2.14.51-.96 1.03-2.52 1.56-4.66l1.14-4.54H9.8l.34-1.15zm6.8 1.95c.25-.2.51-.29.78-.29.1 0 .29.04.56.11.27.08.51.11.72.11.29 0 .52-.1.72-.3.18-.19.28-.45.28-.77 0-.33-.1-.6-.29-.8-.19-.2-.47-.29-.82-.29-.32 0-.62.08-.9.23-.28.15-.64.49-1.08 1-.33.38-.81 1.05-1.44 2a9.712 9.712 0 0 0-1.31-3.22l-3.4.59-.07.37c.25-.05.47-.08.64-.08.34 0 .62.15.84.44.35.46.84 1.85 1.46 4.19-.49.66-.82 1.09-1 1.3-.3.33-.55.54-.74.64-.15.08-.32.12-.51.12-.14 0-.38-.08-.7-.24-.22-.1-.42-.16-.59-.16-.33 0-.6.11-.82.32-.21.22-.32.49-.32.83 0 .31.1.57.3.77.2.2.47.29.8.29.32 0 .63-.07.92-.21.29-.14.64-.43 1.08-.88.43-.45 1.03-1.16 1.79-2.14.29.93.55 1.61.76 2.03.21.42.46.73.74.91.28.19.62.28 1.04.28.4 0 .81-.15 1.23-.44.55-.38 1.1-1.04 1.68-1.97l-.35-.21c-.39.55-.68.89-.87 1.03-.12.09-.27.13-.44.13-.2 0-.4-.13-.59-.38-.33-.43-.77-1.63-1.33-3.6.47-.86.89-1.44 1.23-1.71z"],"gantt-chart":["M4 7h5c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm3 2v1c0 .55.45 1 1 1h6c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1H8c-.55 0-1 .45-1 1zm12 3h-6c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h6c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm0 4H2V3c0-.55-.45-1-1-1s-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],geolocation:["M0 8.33l9.17 2.5 2.5 9.17L20 0z"],geosearch:["M8 18.88c-3.79 0-6.88-3.09-6.88-6.88 0-.61.08-1.22.23-1.79.03.01.06-.01.1-.01h.09v.55c0 .23.21.42.44.42.04 0 .09-.01.12-.02l.9.88c.09.09.23.09.32 0s.09-.23 0-.32l-.86-.9c0-.02.05-.04.05-.07v-.13c0-.18.1-.25.29-.41h.53c.1 0 .19-.01.27-.05.01-.01.02 0 .03-.01.02-.01.03-.02.05-.04.01-.01.02-.01.02-.02l.02-.02 1.13-1.13c-.16-.32-.3-.65-.42-.99h-.64v-.53c0-.01.06.06.06-.1h.38c-.04-.16-.08-.32-.1-.48h-.71c.2-.16.42-.31.64-.45C4.02 6.09 4 5.8 4 5.5c0-.14.01-.28.02-.43C1.62 6.46 0 9.04 0 12c0 4.41 3.59 8 8 8 3.87 0 7.09-2.77 7.82-6.44l-.97-1.1c-.26 3.57-3.23 6.42-6.85 6.42zm-2.12-3.67v-.35h.15c.29 0 .49-.23.49-.53v-.68c0-.01.01-.01 0-.02L4.71 11.8h-.77c-.29 0-.47.24-.47.53v2c0 .29.18.53.47.53h.33v2.02c0 .28.28.51.56.51s.56-.23.56-.51v-1.22h-.01c.29 0 .5-.16.5-.45zm13.83-2.92l-3.68-3.68c.14-.21.27-.42.38-.65.02-.04.04-.07.05-.11.11-.22.2-.45.28-.69v-.01c.07-.24.13-.48.17-.73l.03-.17c.04-.24.06-.49.06-.75C17 2.46 14.54 0 11.5 0S6 2.46 6 5.5 8.46 11 11.5 11c.26 0 .51-.02.76-.06l.17-.03c.25-.04.49-.1.73-.17h.01c.24-.08.47-.17.69-.28.04-.02.07-.04.11-.05.23-.11.44-.24.65-.38l3.68 3.68c.17.18.42.29.7.29a1.003 1.003 0 0 0 .71-1.71zM11.5 9.5c-2.21 0-4-1.79-4-4s1.79-4 4-4 4 1.79 4 4-1.79 4-4 4zm1.93 5.33v-.65c0-.11-.13-.21-.24-.21-.11 0-.24.09-.24.21v.65c0 .11.13.21.24.21.11 0 .24-.1.24-.21zm-2.41.67h.83c.29 0 .46-.21.46-.5v-1.86l.23-.22c-.34.05-.69.08-1.04.08-.36 0-.7-.03-1.05-.08.03.05.06.1.08.16V15c.01.29.2.5.49.5z"],"git-branch":["M15 2c-1.66 0-3 1.34-3 3 0 1.3.84 2.4 2 2.82V9c0 1.1-.9 2-2 2H8c-.73 0-1.41.21-2 .55V5.82C7.16 5.4 8 4.3 8 3c0-1.66-1.34-3-3-3S2 1.34 2 3c0 1.3.84 2.4 2 2.82v8.37C2.84 14.6 2 15.7 2 17c0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.25-.77-2.3-1.85-2.75C6.45 13.52 7.16 13 8 13h4c2.21 0 4-1.79 4-4V7.82C17.16 7.4 18 6.3 18 5c0-1.66-1.34-3-3-3zM5 2c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zm0 16c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zM15 6c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-commit":["M19 9h-4.1a5 5 0 0 0-9.8 0H1c-.55 0-1 .45-1 1s.45 1 1 1h4.1a5 5 0 0 0 9.8 0H19c.55 0 1-.45 1-1s-.45-1-1-1zm-9 4c-1.66 0-3-1.34-3-3s1.34-3 3-3 3 1.34 3 3-1.34 3-3 3z"],"git-merge":["M15 8c-1.3 0-2.4.84-2.82 2H11c-2.49 0-4.54-1.83-4.92-4.21A2.995 2.995 0 0 0 5 0C3.34 0 2 1.34 2 3c0 1.3.84 2.4 2 2.81v8.37C2.84 14.6 2 15.7 2 17c0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.3-.84-2.4-2-2.82V9.86C7.27 11.17 9.03 12 11 12h1.18A2.996 2.996 0 0 0 18 11c0-1.66-1.34-3-3-3zM5 18c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zM5 4c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm10 8c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-new-branch":["M17 3h-1V2c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1V5h1c.55 0 1-.45 1-1s-.45-1-1-1zm-3 4.86V9c0 1.1-.9 2-2 2H8c-.73 0-1.41.21-2 .55V5.82C7.16 5.4 8 4.3 8 3c0-1.66-1.34-3-3-3S2 1.34 2 3c0 1.3.84 2.4 2 2.82v8.37C2.84 14.6 2 15.7 2 17c0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.25-.77-2.3-1.85-2.75C6.45 13.52 7.16 13 8 13h4c2.21 0 4-1.79 4-4V7.86c-.32.08-.65.14-1 .14s-.68-.06-1-.14zM5 2c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zm0 16c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-pull":["M17 14.18V7c0-2.21-1.79-4-4-4h-2.59l1.29-1.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-3 3C7.11 3.47 7 3.72 7 4c0 .28.11.53.29.71l3 3a1.003 1.003 0 0 0 1.42-1.42L10.41 5H13c1.1 0 2 .9 2 2v7.18A2.996 2.996 0 0 0 16 20c1.66 0 3-1.34 3-3 0-1.3-.84-2.4-2-2.82zM16 18c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zM4 1C2.34 1 1 2.34 1 4c0 1.3.84 2.4 2 2.82v7.37C1.84 14.6 1 15.7 1 17c0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.3-.84-2.4-2-2.82V6.82C6.16 6.4 7 5.3 7 4c0-1.66-1.34-3-3-3zm0 17c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zM4 5c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-push":["M15 11c0-.28-.11-.53-.29-.71l-3-3C11.53 7.11 11.28 7 11 7s-.53.11-.71.29l-3 3a1.003 1.003 0 0 0 1.42 1.42l1.29-1.3V19c0 .55.45 1 1 1s1-.45 1-1v-8.59l1.29 1.29c.18.19.43.3.71.3.55 0 1-.45 1-1zm4-11H1C.45 0 0 .45 0 1v16c0 .55.45 1 1 1h7v-2H2v-2h6v-1H4V2h14v11h-4v1h4v2h-4v2h5c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM5 8h2V6H5v2zm2-5H5v2h2V3z"],"git-repo":["M7 3H5v2h2V3zm0 6H5v2h2V9zm0-3H5v2h2V6zm12-6H1C.45 0 0 .45 0 1v16c0 .55.45 1 1 1h4v2l2-1 2 1v-2h10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 16H9v-1H5v1H2v-2h16v2zm0-3H4V2h14v11z"],glass:["M17 6V0H3v6c0 3.53 2.61 6.43 6 6.92V18H6c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1h-3v-5.08c3.39-.49 6-3.39 6-6.92z"],globe:["M7.53 4.37c.1-.1.1-.26 0-.35l-.68-.68c-.1-.1-.25-.1-.35 0-.1.1-.1.26 0 .35l.68.68c.1.1.25.1.35 0zm3.17.06h.3c.09 0 .16-.01.16-.1 0-.09-.07-.1-.16-.1h-.3c-.09 0-.16.01-.16.1s.07.1.16.1zm.98 1.15c.09 0 .19-.08.19-.17v-.42c0-.09-.1-.17-.19-.17s-.19.08-.19.17v.42c0 .09.1.17.19.17zm-6.5 4.19c-.35 0-.56.28-.56.63v2.37c0 .35.21.62.56.62h.39v2.4c0 .34.33.61.67.61s.67-.27.67-.61v-1.44h-.02c.35 0 .6-.19.6-.54v-.41h.18c.35 0 .58-.28.58-.62v-.81c0-.01.01-.01 0-.02L6.1 9.77h-.92zM10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8 0-.74.11-1.46.3-2.14h.03v.65c0 .28.25.5.53.5.05 0 .1-.01.15-.02l1.05 1.05c.1.11.28.11.38 0 .1-.1.11-.27 0-.38L3.42 8.59c0-.03.05-.05.05-.08v-.16c0-.22.12-.3.34-.49h.63c.12 0 .23-.01.32-.07.01-.01.02 0 .03-.01.02-.02.04-.03.06-.04.01-.01.02-.01.03-.02l.02-.02 2.15-2.15c.24-.24.24-.63 0-.86-.23-.24-.62-.19-.86.04l-.41.46H5v-.64c0-.01.07.07.07-.12h.87c.17 0 .3-.12.3-.29 0-.17-.13-.29-.3-.29H4.88C6.27 2.7 8.05 2 10 2s3.73.7 5.12 1.86h-1.58l-.01-.04c-.06 0-.12 0-.17.04l-.71.7c-.09.09-.09.23 0 .31.09.09.23.09.32 0l.56-.6.01-.03h.34c0 .19-.1.13-.1.16v.1c0 .29-.2.5-.49.5h-.51c-.25 0-.52.28-.52.54v.23h-.12c-.16 0-.27.08-.27.24v.33h-.32c-.23 0-.41.15-.41.38 0 .22.18.35.41.35.1 0 .19.04.26-.16l.06.01.66-.59h.23l.53.5c.04.04.11.03.16-.01.04-.04.04-.16 0-.2L13 6.15h.32l.12.16c.25.25.65.23.89-.02l.12-.14H15c.02 0 .11.07.11.07v.33s-.06-.01-.07-.01h-.49c-.16 0-.28.13-.28.29 0 .16.13.29.28.29h.49c.01 0 .07-.01.07-.01v.2c-.19.28-.33.57-.62.57h-1.28s0-.01-.01-.01l-.58-.58a.622.622 0 0 0-.89 0l-.58.58s0 .01-.01.01h-.34c-.35 0-.67.28-.67.63v1.25c0 .35.32.61.67.61h1.22c.46.19.78.48.97.94v2.28c0 .35.23.6.58.6h.98c.35 0 .54-.25.54-.6v-2.2l1.21-1.17.04-.02.02-.01h.04c.1-.11.2-.26.2-.42V8.49c0-.25-.22-.44-.42-.63h.58c.02.38.29.57.63.57h.43c.13.51.18 1.03.18 1.57 0 4.42-3.58 8-8 8zm6.16-5.65c-.14 0-.29.11-.29.25v.77c0 .14.15.25.29.25.14 0 .29-.11.29-.25v-.77c0-.14-.15-.25-.29-.25zM10.5 3.48c0-.34-.28-.57-.62-.57h-.74c-.34 0-.57.25-.57.59 0 .05-.13.06.06.1v.64c0 .2.09.36.29.36.2 0 .29-.16.29-.36v-.19h.68c.33 0 .61-.23.61-.57z"],"globe-network":["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm7.39 7h-3.63c-.31-1.99-.92-3.66-1.72-4.73 2.45.65 4.41 2.42 5.35 4.73zM13 10c0 .69-.04 1.36-.11 2H7.11a18.419 18.419 0 0 1 0-4h5.77c.08.64.12 1.31.12 2zm-3-8c1.07 0 2.25 2.05 2.75 5h-5.5c.5-2.95 1.68-5 2.75-5zm-2.04.27C7.16 3.34 6.55 5.01 6.24 7H2.61c.94-2.31 2.9-4.08 5.35-4.73zM2 10c0-.69.11-1.36.28-2h3.83a18.419 18.419 0 0 0 0 4H2.28c-.17-.64-.28-1.31-.28-2zm.61 3h3.63c.31 1.99.92 3.66 1.72 4.73A7.996 7.996 0 0 1 2.61 13zM10 18c-1.07 0-2.25-2.05-2.75-5h5.5c-.5 2.95-1.68 5-2.75 5zm2.04-.27c.79-1.07 1.4-2.74 1.72-4.73h3.63a7.996 7.996 0 0 1-5.35 4.73zM13.89 12a18.419 18.419 0 0 0 0-4h3.83c.17.64.28 1.31.28 2s-.11 1.36-.28 2h-3.83z"],graph:["M17.5 4A2.5 2.5 0 0 0 15 6.5c0 .06.01.12.02.18l-1.9.84C12.38 6.6 11.27 6 10 6c-.83 0-1.59.25-2.23.68L4.91 4.14c.05-.21.09-.42.09-.64a2.5 2.5 0 0 0-5 0A2.5 2.5 0 0 0 2.5 6c.42 0 .81-.11 1.16-.3l2.79 2.48C6.17 8.73 6 9.34 6 10c0 1.41.73 2.64 1.83 3.35l-.56 1.67A2.498 2.498 0 0 0 5 17.5a2.5 2.5 0 0 0 5 0c0-.74-.32-1.39-.83-1.85l.56-1.68c.09.01.18.03.27.03 2.21 0 4-1.79 4-4 0-.22-.03-.44-.07-.65l2.02-.9c.43.34.96.55 1.55.55a2.5 2.5 0 0 0 0-5z"],"graph-remove":["M17.41 4l2.29-2.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L16 2.59 13.71.3A.965.965 0 0 0 13 0a1.003 1.003 0 0 0-.71 1.71L14.59 4 12.3 6.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L16 5.41l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L17.41 4zM19 10c-.83 0-1.55-.36-2.09-.91l-.03.03-.88-.88-.88.88a2.996 2.996 0 1 1-4.24-4.24l.88-.88-.88-.88.03-.03C10.36 2.55 10 1.83 10 1c0-.35.07-.68.18-.99-.06 0-.12-.01-.18-.01C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10c0-.06-.01-.12-.01-.18-.31.11-.64.18-.99.18z"],"greater-than":["M12.838 10l-9.154 3.051a1 1 0 0 0 .632 1.898l12-4c.912-.304.912-1.594 0-1.898l-12-4a1 1 0 0 0-.632 1.898L12.838 10z"],"greater-than-or-equal-to":["M3.684 11.051a1 1 0 0 0 .632 1.898l12-4c.912-.304.912-1.594 0-1.898l-12-4a1 1 0 0 0-.632 1.898L12.838 8l-9.154 3.051zM4 15h12a1 1 0 1 1 0 2H4a1 1 0 0 1 0-2z"],grid:["M19 11c.55 0 1-.45 1-1s-.45-1-1-1h-2V5h2c.55 0 1-.45 1-1s-.45-1-1-1h-2V1c0-.55-.45-1-1-1s-1 .45-1 1v2h-4V1c0-.55-.45-1-1-1S9 .45 9 1v2H5V1c0-.55-.45-1-1-1S3 .45 3 1v2H1c-.55 0-1 .45-1 1s.45 1 1 1h2v4H1c-.55 0-1 .45-1 1s.45 1 1 1h2v4H1c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h4v2c0 .55.45 1 1 1s1-.45 1-1v-2h4v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1h-2v-4h2zM9 15H5v-4h4v4zm0-6H5V5h4v4zm6 6h-4v-4h4v4zm0-6h-4V5h4v4z"],"grid-view":["M0 19c0 .55.45 1 1 1h8v-9H0v8zM0 1v8h9V0H1C.45 0 0 .45 0 1zm19-1h-8v9h9V1c0-.55-.45-1-1-1zm-8 20h8c.55 0 1-.45 1-1v-8h-9v9z"],"group-objects":["M6 7c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3zm8-3H6c-3.31 0-6 2.69-6 6s2.69 6 6 6h8c3.31 0 6-2.69 6-6s-2.69-6-6-6zm0 11H6c-2.76 0-5-2.24-5-5s2.24-5 5-5h8c2.76 0 5 2.24 5 5s-2.24 5-5 5zm0-8c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3z"],"grouped-bar-chart":["M12 16h1c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1h-1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1zm7 1H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm-3-1h1c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1h-1c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1zm-9 0h1c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1H7c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1zm-4 0h1c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v13c0 .55.45 1 1 1z"],hand:["M17 5c-.42 0-.79.27-.93.64L14.38 10h-.77l1.34-6.67c.03-.1.05-.21.05-.33a.998.998 0 1 0-1.98-.19h-.01L11.57 10H11V1c0-.55-.45-1-1-1S9 .45 9 1v9h-.2L6.97 2.76a.997.997 0 0 0-1.73-.41l-.03.03c-.01.02-.02.03-.03.04-.01.02-.01.03-.02.04v.01c-.01.01-.02.02-.02.03v.01c-.02.01-.02.02-.03.03 0 0 0 .01-.01.01 0 .01 0 .02-.01.03 0 0 0 .01-.01.01 0 .01-.01.02-.01.03 0 0 0 .01-.01.01 0 .01-.01.02-.01.03 0 .01 0 .01-.01.02 0 .01-.01.02-.01.03 0 .01 0 .01-.01.02 0 .01-.01.02-.01.03v.02c0 .01 0 .02-.01.03V3c0 .05 0 .09.01.14l1.45 10.25L6 12.7v.01L3.84 9.45h-.01A.98.98 0 0 0 3 9c-.55 0-1 .45-1 1 0 .2.06.39.17.55L6 18.44C7.06 19.4 8.46 20 10 20c3.31 0 6-2.69 6-6v-1.84l.01-.03v-.06l1.94-5.75A1.003 1.003 0 0 0 17 5z"],"hand-down":["M17.68 9.84C15.91 9 14.27 6.49 13.45 4.9 12.41 2.43 12.21 0 7.87 0 5.49 0 3.95.76 3.05 2.65 2.31 4.2 2 5.48 2 9.79v.99c0 .82.69 1.48 1.54 1.48.38 0 .73-.14 1-.36.19.6.78 1.05 1.47 1.05.47 0 .89-.2 1.17-.52.26.47.77.79 1.36.79.65 0 1.2-.39 1.43-.93l.03.77v5.44c0 .48.23.91.59 1.18.21.19.5.32.85.32h.06c.83 0 1.5-.67 1.5-1.5v-8.24l.01-.67c.85.98 1.92 1.76 3.24 1.89 1.79.19 2.09-1.33 1.43-1.64z"],"hand-left":["M15.1 6.54c-1.58-.81-4.09-2.46-4.94-4.23-.31-.65-1.82-.35-1.64 1.43.13 1.33.91 2.4 1.89 3.24L9.74 7H1.5C.67 7 0 7.67 0 8.5v.06c0 .36.13.64.32.85.27.36.7.59 1.18.59h5.44l.78.01c-.54.23-.93.78-.93 1.43 0 .59.32 1.1.79 1.36-.32.28-.52.7-.52 1.17 0 .69.44 1.28 1.05 1.47-.22.27-.36.62-.36 1 0 .85.66 1.54 1.48 1.54h.99c4.31 0 5.59-.31 7.14-1.05 1.89-.9 2.65-2.44 2.65-4.82-.01-4.32-2.44-4.52-4.91-5.57z"],"hand-right":["M20 8.5c0-.83-.67-1.5-1.5-1.5h-8.24l-.67-.01c.98-.85 1.76-1.92 1.89-3.24.18-1.79-1.33-2.08-1.65-1.43-.84 1.76-3.35 3.41-4.93 4.23C2.43 7.59 0 7.79 0 12.13c0 2.38.76 3.92 2.65 4.82C4.2 17.69 5.48 18 9.79 18h.99c.82 0 1.48-.69 1.48-1.54 0-.38-.14-.73-.36-1 .6-.19 1.05-.78 1.05-1.47 0-.47-.2-.89-.52-1.17.47-.26.79-.77.79-1.36 0-.65-.39-1.2-.93-1.43l.77-.03h5.44c.48 0 .91-.23 1.18-.59.19-.21.32-.49.32-.85v-.03-.03z"],"hand-up":["M16.46 7.74c-.38 0-.73.14-1 .36-.19-.6-.78-1.05-1.47-1.05-.47 0-.89.2-1.17.52-.26-.47-.77-.79-1.36-.79-.65 0-1.2.39-1.43.93L10 6.94V1.5c0-.48-.23-.91-.59-1.18C9.2.13 8.92 0 8.56 0H8.5C7.67 0 7 .67 7 1.5v8.24l-.01.67c-.84-.98-1.92-1.76-3.24-1.89-1.79-.18-2.08 1.33-1.43 1.65 1.77.84 3.41 3.35 4.23 4.94 1.05 2.47 1.25 4.9 5.58 4.9 2.38 0 3.92-.76 4.82-2.65.74-1.56 1.05-2.84 1.05-7.15v-.99c0-.81-.69-1.48-1.54-1.48z"],header:["M16 1c-.55 0-1 .45-1 1v7H5V2c0-.55-.45-1-1-1s-1 .45-1 1v16c0 .55.45 1 1 1s1-.45 1-1v-7h10v7c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1z"],"header-one":["M10 0c.55 0 1 .45 1 1v14c0 .55-.45 1-1 1s-1-.45-1-1V9H2v6c0 .55-.45 1-1 1s-1-.45-1-1V1c0-.55.45-1 1-1s1 .45 1 1v6h7V1c0-.55.45-1 1-1zm7.4 10.77c.17-.2.29-.46.34-.77H19v10h-1.5v-7.11H15v-1.24c.32 0 .63-.03.93-.08.31-.06.58-.16.83-.29.26-.12.47-.3.64-.51z"],"header-two":["M16.6 17.41c-.22.17-.4.36-.56.55-.16.19-.27.4-.33.61h4.28V20H14c.01-.81.18-1.52.53-2.13.35-.6.81-1.13 1.41-1.58.28-.23.58-.46.89-.68.31-.22.59-.46.85-.71.26-.26.48-.53.63-.83.16-.3.25-.64.26-1.02 0-.18-.02-.37-.06-.57-.04-.2-.11-.39-.22-.56s-.26-.31-.45-.43-.44-.18-.75-.18c-.28 0-.52.06-.71.19s-.34.3-.45.52c-.11.22-.2.48-.25.78-.05.3-.08.62-.09.97h-1.43c0-.54.07-1.04.2-1.5.13-.47.32-.87.58-1.2.26-.34.58-.6.95-.78.37-.19.81-.29 1.3-.29.54 0 .99.09 1.35.29.36.19.65.44.87.74.22.29.38.62.47.97.09.35.14.68.14 1 0 .4-.05.75-.16 1.07-.11.32-.26.61-.44.88-.19.27-.4.52-.63.74-.24.22-.48.43-.73.63s-.5.38-.75.56c-.26.17-.5.35-.71.53zM10 0c.55 0 1 .45 1 1v14c0 .55-.45 1-1 1s-1-.45-1-1V9H2v6c0 .55-.45 1-1 1s-1-.45-1-1V1c0-.55.45-1 1-1s1 .45 1 1v6h7V1c0-.55.45-1 1-1z"],headset:["M18.97 9H19A9 9 0 0 0 1 9h.03C.41 9.73 0 10.8 0 12c0 1.74.84 3.2 2 3.76V16c0 1.66 1.34 3 3 3h3c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1H5c-.55 0-1-.45-1-1 .55 0 1-.45 1-1V9c0-.55-.45-1-1-1h-.92C3.57 4.61 6.47 2 10 2s6.43 2.61 6.92 6H16c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h1c1.66 0 3-1.79 3-4 0-1.2-.41-2.27-1.03-3z"],heart:["M20 6.25C20 3.35 17.65 1 14.75 1c-1.02 0-1.95.31-2.75.82v-.04c-.09.06-.17.12-.26.19-.04.03-.09.06-.14.1-.68.51-1.24 1.18-1.6 1.96-.4-.86-1.04-1.57-1.8-2.1-.04-.02-.07-.05-.1-.08a7 7 0 0 0-.6-.33c-.13-.04-.23-.1-.35-.15-.05-.02-.1-.05-.15-.07v.02C6.45 1.13 5.87 1 5.25 1A5.25 5.25 0 0 0 0 6.25c0 .09.01.17.01.25H0c0 .06.01.12.02.18s.01.12.02.18C.13 7.89.44 9 1.07 10.17 2.23 12.33 4.1 14.11 7 16.53v.01c.9.75 1.89 1.55 3 2.46.71-.58 1.38-1.12 2-1.63 3.48-2.86 5.64-4.78 6.93-7.18.63-1.17.94-2.27 1.03-3.3.01-.07.01-.14.02-.21 0-.06.01-.11.02-.17h-.01c0-.09.01-.17.01-.26z"],"heart-broken":["M8.11 7.45C8.05 7.31 8 7.16 8 7c0-.07.03-.13.04-.19h-.02l.86-4.32A5.159 5.159 0 0 0 5.25 1 5.25 5.25 0 0 0 0 6.25c0 .09.01.17.01.25H0c0 .06.01.12.02.18s.01.12.02.18C.13 7.89.44 9 1.07 10.17c1.38 2.58 3.76 4.6 7.71 7.83l-.76-3.8h.02c-.01-.07-.04-.13-.04-.2 0-.21.08-.39.18-.54l-.02-.01 1.68-2.52-1.73-3.48zM20 6.25C20 3.35 17.65 1 14.75 1c-1.54 0-2.92.67-3.88 1.73l-.83 4.13 1.85 3.69h-.01c.07.14.12.29.12.45 0 .21-.08.39-.18.54l.02.01-1.77 2.66.81 4.07c4.16-3.39 6.63-5.45 8.05-8.1.63-1.17.94-2.27 1.03-3.3.01-.07.01-.14.02-.21 0-.06.01-.11.02-.17h-.01c0-.08.01-.16.01-.25z"],"heat-grid":["M14 12h6V8h-6v4zM0 12h6V8H0v4zm1-3h4v2H1V9zm-1 7c0 .55.45 1 1 1h5v-4H0v3zM19 3h-5v4h6V4c0-.55-.45-1-1-1zm0 3h-4V4h4v2zM0 4v3h6V3H1c-.55 0-1 .45-1 1zm7 3h6V3H7v4zm7 10h5c.55 0 1-.45 1-1v-3h-6v4zm-7 0h6v-4H7v4zm1-3h4v2H8v-2zm-1-2h6V8H7v4z"],heatmap:["M6 0a6 6 0 1 0 0 12A6 6 0 1 0 6 0z","M10.5 8a4.5 4.5 0 1 0 0 9 4.5 4.5 0 1 0 0-9z","M16.5 7a3.5 3.5 0 1 0 0 7 3.5 3.5 0 1 0 0-7zM18 16a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM2.5 14a2.5 2.5 0 1 0 0 5 2.5 2.5 0 1 0 0-5zM16.5 0a2.5 2.5 0 1 0 0 5 2.5 2.5 0 1 0 0-5z"],help:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zM7.41 4.62c.65-.54 1.51-.82 2.56-.82.54 0 1.03.08 1.48.25.44.17.83.39 1.14.68.32.29.56.63.74 1.02.17.39.26.82.26 1.27s-.08.87-.24 1.23c-.16.37-.4.73-.71 1.11l-1.21 1.58c-.14.17-.28.33-.32.48-.05.15-.11.35-.11.6v.97H9v-2s.06-.58.24-.81l1.21-1.64c.25-.3.41-.56.51-.77s.14-.44.14-.67c0-.35-.11-.63-.32-.85s-.5-.33-.88-.33c-.37 0-.67.11-.89.33-.22.23-.37.54-.46.94-.03.12-.11.17-.23.16l-1.95-.29c-.12-.01-.16-.08-.14-.22.13-.93.52-1.67 1.18-2.22zM9 14h2.02L11 16H9v-2z"],"helper-management":["M17 10h-3v3h3v-3zm0 4h-3v3h3v-3zm0-8h-3v3h3V6zm2-6H1C.4 0 0 .4 0 1v18c0 .5.4 1 1 1h18c.5 0 1-.5 1-1V1c0-.6-.5-1-1-1zm-1 18H2V2h16v16zm-9-4H6v3h3v-3zm4 0h-3v3h3v-3z"],highlight:["M11.22 14.09l3.03-3.03.71.71L20 6.73l-5.71-5.71-5.04 5.04.71.71-3.02 3.04 4.28 4.28zm6.8 3.91h-16c-.55 0-1 .45-1 1s.45 1 1 1h16c.55 0 1-.45 1-1s-.45-1-1-1zm-15-1h4.04c.28 0 .53-.11.71-.3l2.02-2.02-3.44-3.45-4.04 4.04c-.18.18-.3.44-.3.71.01.57.46 1.02 1.01 1.02z"],history:["M10 0C6.71 0 3.82 1.6 2 4.05V2c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H3.76C5.23 3.17 7.47 2 10 2c4.42 0 8 3.58 8 8s-3.58 8-8 8-8-3.58-8-8c0-.55-.45-1-1-1s-1 .45-1 1c0 5.52 4.48 10 10 10s10-4.48 10-10S15.52 0 10 0zm0 3c-.55 0-1 .45-1 1v6c0 .28.11.53.29.71l3 3a1.003 1.003 0 0 0 1.42-1.42L11 9.59V4c0-.55-.45-1-1-1z"],home:["M2 12v7c0 .55.45 1 1 1h5v-7h4v7h5c.55 0 1-.45 1-1v-7l-8-8-8 8zm17.71-2.71L17 6.59V3c0-.55-.45-1-1-1s-1 .45-1 1v1.59L10.71.3C10.53.11 10.28 0 10 0s-.53.11-.71.29l-9 9a1.003 1.003 0 0 0 1.42 1.42L10 2.41l8.29 8.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71z"],"horizontal-bar-chart":["M1 1c-.55 0-1 .45-1 1v16c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1zm3 5h11c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1zm8 8H4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h8c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1zm7-6H4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h15c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1z"],"horizontal-bar-chart-asc":["M1 9h11c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm0-5h9c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm18 12H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h18c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zM1 14h14c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1z"],"horizontal-bar-chart-desc":["M10 16H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm2-5H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h11c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm3-5H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1zm4-5H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1z"],"horizontal-distribution":["M12 2H8c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zM1 0C.45 0 0 .45 0 1v18c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm18 0c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1z"],"id-number":["M2 5v10h16V5H2zm0-2h16c1.1 0 2 .9 2 2v10c0 1.1-.9 2-2 2H2c-1.1 0-2-.9-2-2V5c0-1.1.9-2 2-2z","M8.88 12.38c-.17-.39-1.01-.66-1.56-.9-.56-.24-.48-.39-.5-.6v-.09c.19-.17.35-.4.45-.67 0 0 0-.02.01-.02l.06-.18c.13-.03.2-.17.23-.29.03-.05.09-.18.08-.33-.04-.18-.11-.27-.2-.3v-.03c0-.24-.02-.58-.06-.81-.01-.06-.02-.12-.04-.19-.08-.27-.25-.52-.48-.7C6.63 7.09 6.3 7 6 7s-.63.09-.87.27c-.23.17-.4.42-.48.7-.02.06-.03.13-.04.19-.04.22-.06.57-.06.81V9c-.09.03-.17.12-.19.31-.01.14.05.27.08.32.03.14.1.27.23.3.02.06.03.12.06.18v.01c.11.27.27.51.47.68v.08c-.02.2.04.35-.51.6-.56.24-1.39.51-1.56.9-.19.39-.12.62-.12.62h5.98c-.01 0 .06-.23-.11-.62zM12 7h4c.55 0 1 .45 1 1s-.45 1-1 1h-4c-.55 0-1-.45-1-1s.45-1 1-1zM12 11h4c.55 0 1 .45 1 1s-.45 1-1 1h-4c-.55 0-1-.45-1-1s.45-1 1-1z"],"image-rotate-left":["M10.5 13c.83 0 1.5-.67 1.5-1.5s-.67-1.5-1.5-1.5-1.5.67-1.5 1.5.67 1.5 1.5 1.5zM14 7H1c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-1 10l-5-3-1 2-2-4-3 4.5V9h11v8zm3-15h-1.59l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-2 2c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42l-.3-.29H16c1.1 0 2 .9 2 2v3c0 .55.45 1 1 1s1-.45 1-1V6c0-2.21-1.79-4-4-4z"],"image-rotate-right":["M5.29 4.29a1.003 1.003 0 0 0 1.42 1.42l2-2C8.89 3.53 9 3.28 9 3c0-.28-.11-.53-.29-.71l-2-2a1.003 1.003 0 0 0-1.42 1.42l.3.29H4C1.79 2 0 3.79 0 6v3c0 .55.45 1 1 1s1-.45 1-1V6c0-1.1.9-2 2-2h1.59l-.3.29zM15.5 13c.83 0 1.5-.67 1.5-1.5s-.67-1.5-1.5-1.5-1.5.67-1.5 1.5.67 1.5 1.5 1.5zM19 7H6c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-1 10l-5-3-1 2-2-4-3 4.5V9h11v8z"],import:["M9.29 15.71c.18.18.43.29.71.29s.53-.11.71-.29l5-5a1.003 1.003 0 0 0-1.42-1.42L11 12.59V1c0-.55-.45-1-1-1S9 .45 9 1v11.59l-3.29-3.3a1.003 1.003 0 0 0-1.42 1.42l5 5zM19 14c-.55 0-1 .45-1 1v3H2v-3c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h18c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1z"],inbox:["M16.92 3.56l-.01-.02c-.16-.35-.5-.6-.91-.6H4c-.41 0-.76.25-.91.6l-.01.02L0 10.49v6.46c0 .55.45 1 1 1h18c.55 0 1-.45 1-1v-6.46l-3.08-6.93zM15 10.95c-.55 0-1 .45-1 1v1H6v-1c0-.55-.45-1-1-1H1.98l2.67-6h10.7l2.67 6H15z"],"inbox-filtered":["M10.262 3l1.958 1.958v.05H4.65l-2.67 5.997H5c.55 0 1 .45 1 .999v1h8v-1c0-.55.45-1 1-1h3.02l-.635-1.426.625-.63c.354-.353.598-.8.707-1.289L20 10.545v6.456c0 .55-.45.999-1 .999H1c-.55 0-1-.45-1-1v-6.455L3.08 3.62l.01-.02c.15-.35.5-.6.91-.6h6.262zm9.088-3a.642.642 0 1 1 .46 1.1l-3.03 3.03v2.95c0 .18-.07.34-.19.46l-1.28 1.29c-.11.1-.27.17-.45.17-.35 0-.64-.29-.64-.64V4.13L11.19 1.1a.642.642 0 0 1 .45-1.1h7.71z"],"inbox-geo":["M7.427 3a7.467 7.467 0 0 0-.411 2.009H4.65l-2.67 5.996H5c.55 0 1 .45 1 .999v1h8V13c.165.01.332 0 .5 0a7.48 7.48 0 0 0 5.5-2.4V17c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1v-6.455L3.08 3.62l.01-.02c.15-.35.5-.6.91-.6h3.427zm5.715-.596a.133.133 0 0 1-.193 0l-.374-.374a.133.133 0 0 1 0-.193.133.133 0 0 1 .193 0l.373.374a.133.133 0 0 1 0 .193zm1.743.033c-.05 0-.088-.006-.088-.055 0-.05.038-.056.088-.056h.165c.05 0 .088.006.088.055 0 .05-.038.056-.088.056h-.165zm.539.632c-.05 0-.104-.044-.104-.094v-.23c0-.05.054-.094.104-.094.05 0 .104.044.104.094v.23c0 .05-.055.094-.104.094zm-3.575 2.304h.506l1.182 1.2c.006.005 0 .005 0 .01v.446c0 .187-.126.341-.319.341h-.098v.226c0 .192-.138.296-.33.296h.01v.792c0 .188-.181.336-.368.336s-.369-.149-.369-.335v-1.32h-.214c-.193 0-.308-.149-.308-.341V5.72c0-.192.115-.346.308-.346zM14.5 0C17.536 0 20 2.464 20 5.5S17.536 11 14.5 11A5.502 5.502 0 0 1 9 5.5C9 2.464 11.464 0 14.5 0zm0 9.9c2.431 0 4.4-1.969 4.4-4.4 0-.297-.027-.583-.099-.864h-.236c-.188 0-.336-.104-.347-.313h-.319c.11.104.231.209.231.346v.705c0 .088-.055.17-.11.23h-.022l-.011.006-.022.011-.666.643v1.21c0 .193-.104.33-.296.33h-.54c-.192 0-.319-.137-.319-.33V6.221a.915.915 0 0 0-.533-.518h-.671c-.192 0-.368-.143-.368-.335V4.68c0-.192.176-.346.368-.346l.193-.005.319-.32a.342.342 0 0 1 .489 0l.319.32c.005 0 .005.005.005.005h.704c.16 0 .237-.16.341-.313v-.11l-.038.005h-.27a.159.159 0 0 1-.153-.16c0-.087.066-.159.154-.159h.269l.039.006V3.42s-.05-.038-.061-.038h-.302l-.067.076a.342.342 0 0 1-.489.011l-.066-.088h-.176l.248.259c.021.022.021.088 0 .11-.028.022-.067.028-.088.006l-.292-.276h-.127l-.363.325-.033-.006c-.038.11-.087.089-.143.089-.126 0-.225-.072-.225-.193 0-.127.099-.209.225-.209h.176v-.182c0-.088.061-.131.149-.131h.066v-.127c0-.143.149-.297.286-.297h.28c.16 0 .27-.115.27-.275V2.42c0-.016.055.017.055-.088h-.187l-.005.017-.308.33a.123.123 0 0 1-.177 0c-.049-.044-.049-.121 0-.171l.391-.385c.027-.022.06-.022.094-.022l.005.022h.869A4.376 4.376 0 0 0 14.5 1.1a4.402 4.402 0 0 0-2.816 1.018h.583c.094 0 .165.066.165.159s-.072.16-.165.16h-.478c0 .104-.039.06-.039.066v.351h.429l.226-.252c.132-.127.346-.155.473-.022a.332.332 0 0 1 0 .473l-1.183 1.182-.011.011c-.005.005-.011.005-.016.011a.115.115 0 0 0-.034.022c-.005.006-.01 0-.016.006a.309.309 0 0 1-.176.038h-.347c-.12.104-.187.148-.187.27v.088c0 .016-.027.027-.027.043l.561.589c.06.06.055.154 0 .209a.143.143 0 0 1-.209 0l-.578-.578a.425.425 0 0 1-.082.011c-.154 0-.292-.12-.292-.274v-.358h-.016c-.104.374-.165.77-.165 1.177 0 2.431 1.969 4.4 4.4 4.4zm3.388-3.107c.077 0 .16.06.16.137v.424c0 .077-.083.137-.16.137s-.16-.06-.16-.137V6.93c0-.077.083-.137.16-.137zm-3.113-4.879c0 .187-.154.314-.335.314h-.374v.104c0 .11-.05.198-.16.198s-.16-.088-.16-.198V1.98c-.104-.022-.033-.028-.033-.055 0-.187.127-.325.314-.325h.407c.187 0 .341.127.341.314z"],"inbox-search":["M7.136 3a6.327 6.327 0 0 0-.098 2.009H4.65l-2.67 5.996H5c.55 0 1 .45 1 .999v1h8v-1c0-.55.45-1 1-1h1.076l1.14 1.14a2.767 2.767 0 0 0 1.974.806c.282 0 .554-.042.81-.12V17c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1v-6.455L3.08 3.62l.01-.02c.15-.35.5-.6.91-.6h3.136zm3.244 1.33c0 1.62 1.31 2.93 2.93 2.93s2.93-1.31 2.93-2.93-1.31-2.93-2.93-2.93-2.93 1.31-2.93 2.93zm6.47 2.43l2.89 2.85c.13.15.22.35.23.56 0 .43-.35.78-.78.78-.23 0-.42-.08-.56-.22l-2.87-2.87c-.17.1-.33.2-.51.29-.03.01-.06.03-.09.04-.18.07-.35.15-.55.21-.19.06-.37.11-.57.14-.05.01-.1.02-.14.02-.2.03-.39.05-.6.05A4.3 4.3 0 0 1 9 4.31C9 1.93 10.93.01 13.3 0c2.37 0 4.3 1.93 4.3 4.3 0 .21-.02.4-.05.6-.01.05-.01.09-.02.14-.04.2-.08.38-.14.58-.05.19-.13.36-.21.54-.01.03-.03.06-.04.09-.08.18-.18.34-.29.51z"],"inbox-update":["M10.083 3a6.04 6.04 0 0 0 .001 2.009H4.65l-2.67 5.996H5c.55 0 1 .45 1 .999v1h8v-1c0-.55.45-1 1-1h3.02l-.53-1.19a5.97 5.97 0 0 0 1.824-.811L20 10.545v6.456c0 .55-.45.999-1 .999H1c-.55 0-1-.45-1-1v-6.455L3.08 3.62l.01-.02c.15-.35.5-.6.91-.6h6.083zM16 8a4 4 0 1 1 0-8 4 4 0 0 1 0 8z"],"info-sign":["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zM9 4h2v2H9V4zm4 12H7v-1h2V8H8V7h3v8h2v1z"],inheritance:["M6 10c0 2.21 1.79 4 4 4h6.59l-2.29-2.29A.965.965 0 0 1 14 11a1.003 1.003 0 0 1 1.71-.71l4 4c.18.18.29.43.29.71 0 .28-.11.53-.29.71l-4 4a1.003 1.003 0 0 1-1.42-1.42l2.3-2.29H10c-3.31 0-6-2.69-6-6H1a1 1 0 0 1-1-1V1a1 1 0 0 1 1-1h8a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H6zM2 2v6h6V2H2z"],"inner-join":["M8.7 4.7C7.4 6 6.5 7.9 6.5 10s.8 4 2.2 5.3c-.8.5-1.7.7-2.7.7-3.3 0-6-2.7-6-6s2.7-6 6-6c1 0 1.9.2 2.7.7zm-3.34 9.25c-.55-1.2-.86-2.54-.86-3.95s.31-2.75.86-3.95a4.001 4.001 0 0 0 0 7.9zM14 4c3.3 0 6 2.7 6 6s-2.7 6-6 6c-1 0-1.9-.2-2.7-.7 1.3-1.3 2.2-3.2 2.2-5.3s-.8-3.9-2.2-5.3C12.1 4.2 13 4 14 4zm.6 2.05c.55 1.2.86 2.54.86 3.95s-.31 2.75-.86 3.95c1.9-.31 3.36-1.96 3.36-3.95S16.5 6.36 14.6 6.05zM10 5.5C8.8 6.7 8 8.2 8 10s.8 3.3 2 4.4c1.2-1.1 2-2.7 2-4.5s-.8-3.3-2-4.4z"],insert:["M19 0H1C.4 0 0 .4 0 1v18c0 .5.4 1 1 1h18c.5 0 1-.5 1-1V1c0-.6-.5-1-1-1zm-1 18H2V2h16v16zM5 11h4v4c0 .6.4 1 1 1s1-.4 1-1v-4h4c.6 0 1-.4 1-1s-.4-1-1-1h-4V5c0-.6-.4-1-1-1s-1 .4-1 1v4H5c-.6 0-1 .4-1 1s.4 1 1 1z"],intersection:["M13 4c-1.31 0-2.51.43-3.5 1.14A5.977 5.977 0 0 0 6 4c-3.31 0-6 2.69-6 6s2.69 6 6 6c1.31 0 2.51-.43 3.5-1.14.99.71 2.19 1.14 3.5 1.14 3.31 0 6-2.69 6-6s-2.69-6-6-6zm-4.93 9.41c-.61.37-1.31.59-2.07.59-2.21 0-4-1.79-4-4s1.79-4 4-4c.76 0 1.46.22 2.07.59C7.4 7.56 7 8.73 7 10s.4 2.44 1.07 3.41zM13 14c-.76 0-1.46-.22-2.07-.59C11.6 12.44 12 11.27 12 10s-.4-2.44-1.07-3.41C11.54 6.22 12.24 6 13 6c2.21 0 4 1.79 4 4s-1.79 4-4 4z"],"ip-address":["M6 3.66C6 5.69 10 11 10 11s4-5.31 4-7.34C13.99 1.64 12.21 0 10 0S6 1.64 6 3.66zM8 4c0-1.1.9-2 2-2s2 .9 2 2-.9 2-2 2-2-.9-2-2zM14 13.5V13h-4v1h3v2h-2v1h3v-3.5zM3 12h14c.55 0 1 .45 1 1v6c0 .55-.45 1-1 1H3c-.55 0-1-.45-1-1v-6c0-.55.45-1 1-1zm4 1v6h1v-6H7zm3 1v5h1v-5h-1z"],issue:["M10 20C4.477 20 0 15.523 0 10S4.477 0 10 0s10 4.477 10 10-4.477 10-10 10zm0-2a8 8 0 1 0 0-16 8 8 0 0 0 0 16zm1-2H9v-2h2v2zm0-3H9V4h2v9z"],"issue-closed":["M15.364 5.9a.997.997 0 0 1-.707-.293l-2.121-2.122a1 1 0 1 1 1.414-1.414l1.414 1.414L18.192.657a1 1 0 0 1 1.414 1.414l-3.535 3.536a.997.997 0 0 1-.707.292zM11.78.157a3.002 3.002 0 0 0-1.437 1.85 8 8 0 1 0 7.1 5.055l.042-.042 1.472-1.472A9.959 9.959 0 0 1 20 10c0 5.523-4.477 10-10 10S0 15.523 0 10 4.477 0 10 0c.608 0 1.202.054 1.78.158zM11 16H9v-2h2v2zm0-3H9V4h2v9z"],"issue-new":["M13.167.512a2.98 2.98 0 0 0-.131.524c-.74.115-1.39.5-1.848 1.052a8 8 0 1 0 6.724 6.724 2.997 2.997 0 0 0 1.052-1.848 2.98 2.98 0 0 0 .524-.13A9.99 9.99 0 0 1 20 10c0 5.523-4.477 10-10 10S0 15.523 0 10 4.477 0 10 0a9.99 9.99 0 0 1 3.167.512zM11 16H9v-2h2v2zm0-3H9V4h2v9zm6-10h1.5a1 1 0 0 1 0 2H17v1.5a1 1 0 0 1-2 0V5h-1.5a1 1 0 0 1 0-2H15V1.5a1 1 0 0 1 2 0V3z"],italic:["M11.7 4H14c.6 0 1-.4 1-1s-.4-1-1-1H7c-.6 0-1 .4-1 1s.4 1 1 1h2.2L7.3 15H5c-.6 0-1 .4-1 1s.4 1 1 1h7c.6 0 1-.4 1-1s-.4-1-1-1H9.8l1.9-11z"],"join-table":["M19 6h-4V2c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h4v4c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1zM6 12H2V9h4v3zm0-4H2V5h4v3zm7 9H7v-3h6v3zm0-4H7V9h6v4zm0-5H7V5h6v3zm5 9h-4v-3h4v3zm0-4h-4v-3h4v3z"],key:["M14 0c-3.31 0-6 2.69-6 6 0 1.11.32 2.14.85 3.03L.44 17.44a1.498 1.498 0 1 0 2.12 2.12l.79-.79.94.94c.18.18.43.29.71.29s.53-.11.71-.29l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-.94-.94 3.2-3.2A5.9 5.9 0 0 0 14 12c3.31 0 6-2.69 6-6s-2.69-6-6-6zm0 9c-1.66 0-3-1.34-3-3s1.34-3 3-3 3 1.34 3 3-1.34 3-3 3z"],"key-backspace":["M19 3H7c-.28 0-.53.11-.71.29l-6 6C.11 9.47 0 9.72 0 10c0 .28.11.53.29.71l6 6c.18.18.43.29.71.29h12c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1zm-2.29 9.29a1.003 1.003 0 0 1-1.42 1.42L13 11.41l-2.29 2.29c-.18.19-.43.3-.71.3a1.003 1.003 0 0 1-.71-1.71l2.3-2.29-2.3-2.29a1.003 1.003 0 0 1 1.42-1.42L13 8.59l2.29-2.29c.18-.19.43-.3.71-.3a1.003 1.003 0 0 1 .71 1.71L14.41 10l2.3 2.29z"],"key-command":["M15.5 12H14V8h1.5C17.43 8 19 6.43 19 4.5S17.43 1 15.5 1 12 2.57 12 4.5V6H8V4.5C8 2.57 6.43 1 4.5 1S1 2.57 1 4.5 2.57 8 4.5 8H6v4H4.5C2.57 12 1 13.57 1 15.5S2.57 19 4.5 19 8 17.43 8 15.5V14h4v1.5c0 1.93 1.57 3.5 3.5 3.5s3.5-1.57 3.5-3.5-1.57-3.5-3.5-3.5zm0-9c.83 0 1.5.67 1.5 1.5S16.33 6 15.5 6 14 5.33 14 4.5 14.67 3 15.5 3zm-11 14c-.83 0-1.5-.67-1.5-1.5S3.67 14 4.5 14s1.5.67 1.5 1.5S5.33 17 4.5 17zm0-11C3.67 6 3 5.33 3 4.5S3.67 3 4.5 3 6 3.67 6 4.5 5.33 6 4.5 6zm7.5 6H8V8h4v4zm3.5 5c-.83 0-1.5-.67-1.5-1.5s.67-1.5 1.5-1.5 1.5.67 1.5 1.5-.67 1.5-1.5 1.5z"],"key-control":["M16.71 7.29l-6-6C10.53 1.11 10.28 1 10 1s-.53.11-.71.29l-6 6a1.003 1.003 0 0 0 1.42 1.42L10 3.41l5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71z"],"key-delete":["M19.71 9.29l-6-6A.997.997 0 0 0 13 3H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h12c.28 0 .53-.11.71-.29l6-6c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zm-9 3a1.003 1.003 0 0 1-1.42 1.42L7 11.41 4.71 13.7c-.18.19-.43.3-.71.3a1.003 1.003 0 0 1-.71-1.71L5.59 10l-2.3-2.29a1.003 1.003 0 0 1 1.42-1.42L7 8.59 9.29 6.3c.18-.19.43-.3.71-.3a1.003 1.003 0 0 1 .71 1.71L8.41 10l2.3 2.29z"],"key-enter":["M18 2c-.55 0-1 .45-1 1v5c0 2.21-1.79 4-4 4H4.41L6.7 9.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-4 4c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l4 4a1.003 1.003 0 0 0 1.42-1.42L4.41 14H13c3.31 0 6-2.69 6-6V3c0-.55-.45-1-1-1z"],"key-escape":["M2 8c.55 0 1-.45 1-1V4.41l6.29 6.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L4.41 3H7c.55 0 1-.45 1-1s-.45-1-1-1H2c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1zm9-6.94V3.1c3.39.49 6 3.38 6 6.9 0 3.87-3.13 7-7 7-3.52 0-6.41-2.61-6.9-6H1.06c.5 4.5 4.31 8 8.94 8a9 9 0 0 0 9-9c0-4.63-3.5-8.44-8-8.94z"],"key-option":["M13 4h6c.55 0 1-.45 1-1s-.45-1-1-1h-6c-.55 0-1 .45-1 1s.45 1 1 1zm6 12h-4.42L6.87 2.5l-.02.01A.977.977 0 0 0 6 2H1c-.55 0-1 .45-1 1s.45 1 1 1h4.42l7.71 13.5.01-.01c.18.3.49.51.86.51h5c.55 0 1-.45 1-1s-.45-1-1-1z"],"key-shift":["M17.74 10.35l-6.99-8.01-.01.01C10.56 2.14 10.3 2 10 2s-.56.14-.74.35l-.01-.01-7 8 .01.01A.95.95 0 0 0 2 11c0 .55.45 1 1 1h3v5c0 .55.45 1 1 1h6c.55 0 1-.45 1-1v-5h3c.55 0 1-.45 1-1 0-.25-.1-.48-.26-.65z"],"key-tab":["M19 13H4.41l2.29-2.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L2 12.59V10c0-.55-.45-1-1-1s-1 .45-1 1v8c0 .55.45 1 1 1s1-.45 1-1v-2.59l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L4.41 15H19c.55 0 1-.45 1-1s-.45-1-1-1zm0-12c-.55 0-1 .45-1 1v2.59L14.71 1.3A.965.965 0 0 0 14 1a1.003 1.003 0 0 0-.71 1.71L15.59 5H1c-.55 0-1 .45-1 1s.45 1 1 1h14.59L13.3 9.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L18 7.41V10c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1z"],"known-vehicle":["M19 4a.997.997 0 0 0-.707.293L14 8.586l-2.293-2.293a1 1 0 0 0-1.414 1.414l3 3a.997.997 0 0 0 1.414 0l5-5A1 1 0 0 0 19 4zm-2.048 7.291c.011.072.048.134.048.209a1.5 1.5 0 0 1-1.5 1.5c-.225 0-.433-.057-.624-.145-.279.085-.57.145-.876.145a2.99 2.99 0 0 1-2.121-.879l-3-3 .007-.007A3.027 3.027 0 0 1 8.184 8H4V7l1-3h10l.19.568 1.307-1.308c-.336-.356-.758-.658-1.165-.772 0 0-1.74-.488-5.332-.488s-5.332.488-5.332.488c-.67.188-1.424.864-1.674 1.502L2.99 4H3L2 7H1a1 1 0 0 0 0 2h.333l-.28.84L1 10v7.5a1.5 1.5 0 1 0 3 0V17h12v.5a1.5 1.5 0 0 0 3 0V10l-.19-.568-1.858 1.86zM4.5 13a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"],label:["M3 12h14v-1H3v1zm11-9H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V9l-6-6zm4 12H2V5h11v3H3v1h10v1h5v5zm-4-6V5l4 4h-4z"],layer:["M19.5 9.1l-9-5c-.2-.1-.3-.1-.5-.1s-.3 0-.5.1l-9 5c-.3.2-.5.5-.5.9s.2.7.5.9l9 5c.2.1.3.1.5.1s.3 0 .5-.1l9-5c.3-.2.5-.5.5-.9s-.2-.7-.5-.9z"],layers:["M.5 6.9l9 5c.2.1.3.1.5.1s.3 0 .5-.1l9-5c.3-.2.5-.5.5-.9s-.2-.7-.5-.9l-9-5c-.2-.1-.3-.1-.5-.1s-.3 0-.5.1l-9 5c-.3.2-.5.5-.5.9s.2.7.5.9z","M19 9c-.2 0-.3 0-.5.1L10 13.9 1.5 9.1C1.3 9 1.2 9 1 9c-.6 0-1 .4-1 1 0 .4.2.7.5.9l9 5c.2.1.3.1.5.1s.3 0 .5-.1l9-5c.3-.2.5-.5.5-.9 0-.6-.4-1-1-1z","M19 13c-.2 0-.3 0-.5.1L10 17.9l-8.5-4.7c-.2-.2-.3-.2-.5-.2-.6 0-1 .4-1 1 0 .4.2.7.5.9l9 5c.2.1.3.1.5.1s.3 0 .5-.1l9-5c.3-.2.5-.5.5-.9 0-.6-.4-1-1-1z"],layout:["M18 6c-1.1 0-2 .9-2 2 0 .37.11.71.28 1.01l-2.27 2.27c-.3-.17-.64-.28-1.01-.28-.93 0-1.71.64-1.93 1.5H8.93c-.22-.86-1-1.5-1.93-1.5-.37 0-.71.11-1.01.28L3.72 9.01C3.89 8.71 4 8.37 4 8c0-.34-.09-.66-.24-.94l3.66-3.38c.31.2.68.32 1.08.32 1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .34.09.66.24.94L3.08 6.32C2.77 6.12 2.4 6 2 6 .9 6 0 6.9 0 8s.9 2 2 2c.37 0 .71-.11 1.01-.28l2.27 2.27c-.17.3-.28.64-.28 1.01s.11.71.28 1.01l-2.27 2.27C2.71 16.11 2.37 16 2 16c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2c0-.37-.11-.71-.28-1.01l2.27-2.27c.3.17.64.28 1.01.28.93 0 1.71-.64 1.93-1.5h2.14c.22.86 1 1.5 1.93 1.5 1.1 0 2-.9 2-2 0-.37-.11-.71-.28-1.01l2.27-2.27c.3.17.64.28 1.01.28 1.1 0 2-.9 2-2s-.9-2-2-2z"],"layout-auto":["M18 13c-.53 0-1.01.21-1.37.55L11.9 10.6c.06-.19.1-.39.1-.6s-.04-.41-.1-.6l4.72-2.95c.37.34.85.55 1.38.55 1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .21.04.41.1.6l-4.73 2.96c-.24-.23-.54-.4-.87-.48V3.93c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2S8 .9 8 2c0 .93.64 1.71 1.5 1.93v4.14c-.33.09-.63.26-.87.48L3.9 5.6c.06-.19.1-.39.1-.6 0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.53 0 1.01-.21 1.37-.55L8.1 9.4c-.06.19-.1.39-.1.6s.04.41.1.6l-4.72 2.95C3.01 13.21 2.53 13 2 13c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2c0-.21-.04-.41-.1-.6l4.73-2.96c.24.23.54.4.87.48v4.14C8.64 16.29 8 17.07 8 18c0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93v-4.14c.33-.09.63-.26.87-.48l4.73 2.96c-.06.18-.1.38-.1.59 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2z"],"layout-balloon":["M18 16c-.14 0-.28.02-.42.05l-1.73-3.45c.69-.45 1.14-1.22 1.14-2.1s-.46-1.65-1.14-2.1l1.73-3.45c.14.03.28.05.42.05 1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .6.27 1.13.69 1.5l-1.77 3.54c-.14-.02-.28-.04-.42-.04a2.5 2.5 0 0 0-2.45 2h-4.1A2.5 2.5 0 0 0 5.5 8c-.14 0-.28.02-.42.04L3.31 4.5C3.73 4.13 4 3.6 4 3c0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.14 0 .28-.02.42-.05L4.14 8.4C3.46 8.85 3 9.62 3 10.5s.46 1.65 1.14 2.1l-1.73 3.45A1.84 1.84 0 0 0 2 16c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2c0-.6-.27-1.13-.69-1.5l1.77-3.54c.14.02.28.04.42.04a2.5 2.5 0 0 0 2.45-2h4.1a2.5 2.5 0 0 0 2.45 2c.14 0 .28-.02.42-.04l1.77 3.54c-.42.37-.69.9-.69 1.5 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2z"],"layout-circle":["M18.3 8c-.2-.9-.6-1.7-1.1-2.5.2-.3.3-.7.3-1 0-1.1-.9-2-2-2-.4 0-.7.1-1 .3-.8-.5-1.6-.8-2.5-1.1-.1-1-1-1.7-2-1.7S8.2.8 8 1.7c-.9.3-1.7.6-2.5 1.1-.3-.2-.7-.3-1-.3-1.1 0-2 .9-2 2 0 .4.1.7.3 1-.5.8-.8 1.6-1.1 2.5C.8 8.2 0 9 0 10s.8 1.8 1.7 2c.2.9.6 1.7 1.1 2.5-.2.3-.3.7-.3 1 0 1.1.9 2 2 2 .4 0 .7-.1 1-.3.8.5 1.6.8 2.5 1.1.1 1 1 1.7 2 1.7s1.8-.8 2-1.7c.9-.2 1.7-.6 2.5-1.1.3.2.7.3 1 .3 1.1 0 2-.9 2-2 0-.4-.1-.7-.3-1 .5-.8.8-1.6 1.1-2.5 1-.1 1.7-1 1.7-2s-.8-1.8-1.7-2zm-1.8 5.8c-.3-.2-.6-.3-1-.3-1.1 0-2 .9-2 2 0 .4.1.7.3 1-.6.3-1.2.6-1.9.8-.3-.7-1-1.3-1.9-1.3-.8 0-1.6.5-1.9 1.3-.7-.2-1.3-.4-1.9-.8.2-.3.3-.6.3-1 0-1.1-.9-2-2-2-.4 0-.7.1-1 .3-.3-.6-.6-1.2-.8-1.9.8-.3 1.3-1.1 1.3-1.9s-.5-1.6-1.2-1.8c.2-.7.4-1.3.8-1.9.3.2.6.3 1 .3 1.1 0 2-.9 2-2 0-.4-.1-.7-.3-1 .6-.3 1.2-.6 1.9-.8.2.7 1 1.2 1.8 1.2s1.6-.5 1.9-1.3c.7.2 1.3.4 1.9.8-.2.3-.3.6-.3 1 0 1.1.9 2 2 2 .4 0 .7-.1 1-.3.3.6.6 1.2.8 1.9-.8.3-1.3 1.1-1.3 1.9s.5 1.6 1.2 1.8c-.1.7-.4 1.4-.7 2z"],"layout-grid":["M2 0a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM10 0a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM18 0a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM18 8a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM18 16a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM10 16a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM2 16a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM2 8a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM10 8a2 2 0 1 0 0 4 2 2 0 1 0 0-4z"],"layout-group-by":["M2 2a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM18 0a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM18 8a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM18 16a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM2 14a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM2 8a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM13 12a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM13 4a2 2 0 1 0 0 4 2 2 0 1 0 0-4z"],"layout-hierarchy":["M18.5 16.07v-4.14c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2-.93 0-1.71.64-1.93 1.5h-4.14c-.18-.7-.73-1.25-1.43-1.43V3.93c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2S8 .9 8 2c0 .93.64 1.71 1.5 1.93v4.14c-.7.18-1.25.73-1.43 1.43H3.93C3.71 8.64 2.93 8 2 8c-1.1 0-2 .9-2 2 0 .93.64 1.71 1.5 1.93v4.14c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93v-4.14c.7-.18 1.25-.73 1.43-1.43h4.14c.18.7.73 1.25 1.43 1.43v4.14c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93v-4.14c.7-.18 1.25-.73 1.43-1.43h4.14c.18.7.73 1.25 1.43 1.43v4.14c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93z"],"layout-linear":["M16.5 7a2.5 2.5 0 0 0-2.45 2h-2.1a2.5 2.5 0 0 0-4.9 0h-2.1a2.5 2.5 0 1 0 0 1h2.1a2.5 2.5 0 0 0 4.9 0h2.1a2.5 2.5 0 1 0 2.45-3z"],"layout-skew-grid":["M2 0a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM18 0a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM18 8a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM18 16a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM2 16a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM2 8a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM10 12a2 2 0 1 0 0 4 2 2 0 1 0 0-4zM10 4a2 2 0 1 0 0 4 2 2 0 1 0 0-4z"],"layout-sorted-clusters":["M2 8c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0 8c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM2 0C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm16 8c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm-8 4c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0-8c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],"left-join":["M8.7 4.7C7.4 6 6.5 7.9 6.5 10s.8 4 2.2 5.3c-.8.5-1.7.7-2.7.7-3.3 0-6-2.7-6-6s2.7-6 6-6c1 0 1.9.2 2.7.7zM14 4c3.3 0 6 2.7 6 6s-2.7 6-6 6c-1 0-1.9-.2-2.7-.7 1.3-1.3 2.2-3.2 2.2-5.3s-.8-3.9-2.2-5.3C12.1 4.2 13 4 14 4zm.6 2.05c.55 1.2.86 2.54.86 3.95s-.31 2.75-.86 3.95c1.9-.31 3.36-1.96 3.36-3.95S16.5 6.36 14.6 6.05zM10 5.5C8.8 6.7 8 8.2 8 10s.8 3.3 2 4.4c1.2-1.1 2-2.7 2-4.5s-.8-3.3-2-4.4z"],"less-than":["M7.162 10l9.154 3.052a1 1 0 0 1-.632 1.897l-12-4c-.912-.304-.912-1.594 0-1.897l12-4a1 1 0 0 1 .632 1.897L7.162 10z"],"less-than-or-equal-to":["M16.316 11.051L7.162 8l9.154-3.051a1 1 0 1 0-.632-1.898l-12 4c-.912.304-.912 1.594 0 1.898l12 4a1 1 0 1 0 .632-1.898zM16 15H4a1 1 0 1 0 0 2h12a1 1 0 1 0 0-2z"],lifesaver:["M8.143 14.644L7.028 17.43c.919.368 1.922.57 2.972.57s2.053-.202 2.972-.57l-1.115-2.786A4.986 4.986 0 0 1 10 15a4.986 4.986 0 0 1-1.857-.356zm-2.787-2.787A4.986 4.986 0 0 1 5 10c0-.656.126-1.283.356-1.857L2.57 7.028A7.978 7.978 0 0 0 2 10c0 1.05.202 2.053.57 2.972l2.786-1.115zm2.787-6.5A4.986 4.986 0 0 1 10 5c.656 0 1.283.126 1.857.356l1.115-2.786A7.978 7.978 0 0 0 10 2c-1.05 0-2.053.202-2.972.57l1.115 2.786zm6.5 2.786c.23.574.357 1.2.357 1.857 0 .656-.126 1.283-.356 1.857l2.786 1.115c.368-.919.57-1.922.57-2.972s-.202-2.053-.57-2.972l-2.786 1.115zM10 13a3 3 0 1 0 0-6 3 3 0 0 0 0 6zm0 7C4.477 20 0 15.523 0 10S4.477 0 10 0s10 4.477 10 10-4.477 10-10 10z"],lightbulb:["M6.33 13.39c0 .34.27.61.6.61h6.13c.33 0 .6-.27.6-.61C14.03 9.78 16 9.4 16 6.09 16 2.72 13.31 0 10 0S4 2.72 4 6.09c0 3.31 1.97 3.69 2.33 7.3zM13 15H7c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1zm-1 3H8c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1z"],link:["M10.85 11.98l-4.44 4.44-1 1c-.36.36-.86.58-1.41.58-1.1 0-2-.9-2-2 0-.55.22-1.05.59-1.41l5.44-5.44C7.69 9.06 7.36 9 7 9c-1.11 0-2.09.46-2.82 1.18l-.01-.01-3 3 .01.01C.46 13.91 0 14.89 0 16c0 2.21 1.79 4 4 4 1.11 0 2.09-.46 2.82-1.18l.01.01 3-3-.01-.01C10.54 15.09 11 14.11 11 13c0-.36-.06-.69-.15-1.02zM20 4c0-2.21-1.79-4-4-4-1.11 0-2.09.46-2.82 1.18l-.01-.01-3 3 .01.01C9.46 4.91 9 5.89 9 7c0 .36.06.69.15 1.02l4.44-4.44 1-1c.36-.36.86-.58 1.41-.58 1.1 0 2 .9 2 2 0 .55-.22 1.05-.59 1.41l-5.44 5.44c.34.09.67.15 1.03.15 1.11 0 2.09-.46 2.82-1.18l.01.01 3-3-.01-.01C19.54 6.09 20 5.11 20 4zM5 14a1.003 1.003 0 0 0 1.71.71l8-8a1.003 1.003 0 0 0-1.42-1.42l-2 2-2 2-2 2-2 2c-.18.18-.29.43-.29.71z"],list:["M1.03 1C.46 1 0 1.46 0 2.03v.95C0 3.54.46 4 1.03 4h17.95C19.54 4 20 3.54 20 2.97v-.94C20 1.46 19.54 1 18.97 1H1.03zM0 17.97C0 18.54.46 19 1.03 19h17.95c.56 0 1.03-.46 1.03-1.03v-.95c0-.56-.46-1.03-1.03-1.03H1.03C.46 16 0 16.46 0 17.03v.94zM0 12.97C0 13.54.46 14 1.03 14h17.95c.56 0 1.03-.46 1.03-1.03v-.95c0-.56-.46-1.03-1.03-1.03H1.03C.46 11 0 11.46 0 12.03v.94zM0 7.97C0 8.54.46 9 1.03 9h17.95C19.54 9 20 8.54 20 7.97v-.94C20 6.46 19.54 6 18.97 6H1.03C.46 6 0 6.46 0 7.03v.94z"],"list-columns":["M0 2.973v-.936C0 1.468.46 1.01 1.029 1H7.97C8.541 1 9 1.468 9 2.027v.946C9 3.542 8.53 4 7.971 4H1.03C.459 4 0 3.542 0 2.973zm0 5v-.936C0 6.468.46 6.01 1.029 6H7.97C8.541 6 9 6.468 9 7.027v.946C9 8.542 8.53 9 7.971 9H1.03C.459 9 0 8.542 0 7.973zm0 5v-.936C0 11.468.46 11.01 1.029 11H7.97C8.541 11 9 11.468 9 12.027v.946C9 13.542 8.53 14 7.971 14H1.03C.459 14 0 13.542 0 12.973zm0 5v-.936C0 16.468.46 16.01 1.029 16H7.97C8.541 16 9 16.468 9 17.027v.946C9 18.542 8.53 19 7.971 19H1.03C.459 19 0 18.542 0 17.973zm11-15v-.936c0-.569.46-1.027 1.029-1.037h6.942C19.541 1 20 1.468 20 2.027v.946C20 3.542 19.53 4 18.971 4H12.03C11.459 4 11 3.542 11 2.973zm0 5v-.936c0-.569.46-1.027 1.029-1.037h6.942C19.541 6 20 6.468 20 7.027v.946C20 8.542 19.53 9 18.971 9H12.03C11.459 9 11 8.542 11 7.973zm0 5v-.936c0-.569.46-1.027 1.029-1.037h6.942c.57 0 1.029.468 1.029 1.027v.946c0 .569-.47 1.027-1.029 1.027H12.03c-.57 0-1.029-.458-1.029-1.027zm0 5v-.936c0-.569.46-1.027 1.029-1.037h6.942c.57 0 1.029.468 1.029 1.027v.946c0 .569-.47 1.027-1.029 1.027H12.03c-.57 0-1.029-.458-1.029-1.027z"],"list-detail-view":["M8 6H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1zm0 5H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm0 5H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zM8 1H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zm11 0h-7c-.55 0-1 .45-1 1v16c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1z"],locate:["M10 8c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm9 1h-1.07c-.45-3.61-3.32-6.45-6.93-6.91V1c0-.55-.45-1-1-1S9 .45 9 1v1.09C5.39 2.55 2.52 5.39 2.07 9H1c-.55 0-1 .45-1 1s.45 1 1 1h1.07c.45 3.61 3.32 6.45 6.93 6.91V19c0 .55.45 1 1 1s1-.45 1-1v-1.09c3.61-.46 6.48-3.29 6.93-6.91H19c.55 0 1-.45 1-1s-.45-1-1-1zm-4 2h.9a5.98 5.98 0 0 1-4.9 4.91V15c0-.55-.45-1-1-1s-1 .45-1 1v.91A5.98 5.98 0 0 1 4.1 11H5c.55 0 1-.45 1-1s-.45-1-1-1h-.9A5.98 5.98 0 0 1 9 4.09V5c0 .55.45 1 1 1s1-.45 1-1v-.91A5.98 5.98 0 0 1 15.9 9H15c-.55 0-1 .45-1 1s.45 1 1 1z"],lock:["M15.93 9H14V4.99c0-2.21-1.79-4-4-4s-4 1.79-4 4V9H3.93c-.55 0-.93.44-.93.99v8c0 .55.38 1.01.93 1.01h12c.55 0 1.07-.46 1.07-1.01v-8c0-.55-.52-.99-1.07-.99zM8 9V4.99c0-1.1.9-2 2-2s2 .9 2 2V9H8z"],"log-in":["M19 0h-8c-.55 0-1 .45-1 1s.45 1 1 1h7v16h-7c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-4 10c0-.28-.11-.53-.29-.71l-5-5a1.003 1.003 0 0 0-1.42 1.42L11.59 9H1c-.55 0-1 .45-1 1s.45 1 1 1h10.59L8.3 14.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l5-5c.18-.18.29-.43.29-.71z"],"log-out":["M19.71 9.29l-5-5a1.003 1.003 0 0 0-1.42 1.42L16.59 9H6c-.55 0-1 .45-1 1s.45 1 1 1h10.59l-3.29 3.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l5-5c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zM9 18H2V2h7c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1z"],manual:["M20 1.1a.976.976 0 0 0-.83-.88C15.15-.43 12.07.34 10 2.5 7.93.34 4.85-.43.84.22.37.3.03.67 0 1.1v15.01c0 .07 0 .14.01.21.09.52.61.88 1.15.79 3.85-.62 6.4.16 8 2.46.02.02.03.04.05.07.02.02.04.04.06.07l.01.01a1.07 1.07 0 0 0 .28.19c.01 0 .01.01.02.01.03.01.07.03.1.04.01 0 .02.01.04.01.03.01.07.02.1.02.01 0 .02 0 .04.01H10c.04 0 .09 0 .13-.01.01 0 .03 0 .04-.01.03-.01.06-.01.1-.02.01 0 .03-.01.04-.01.03-.01.07-.02.1-.04.01 0 .02-.01.03-.01.07-.03.13-.07.19-.11.01 0 .01-.01.02-.01.02-.02.04-.03.06-.05.01-.01.02-.02.03-.02l.05-.05c.01-.01.02-.02.02-.03.01-.02.02-.03.04-.05 1.61-2.3 4.15-3.09 8-2.46.54.09 1.06-.26 1.15-.79-.01-.05 0-.09 0-.13V1.1zM9 16.63c-1.78-1.31-4.12-1.83-7-1.55V2c3.26-.37 5.51.39 7 2.35v12.28zm9-1.56c-2.88-.28-5.22.24-7 1.55V4.34c1.49-1.96 3.74-2.71 7-2.35v13.08z"],"manually-entered-data":["M1 12h4.34l2-2H1c-.55 0-1 .45-1 1s.45 1 1 1zm16.77-3.94l1.65-1.65c.36-.36.58-.86.58-1.41 0-1.1-.9-2-2-2-.55 0-1.05.22-1.41.59l-1.65 1.65 2.83 2.82zM1 4h12.34l2-2H1c-.55 0-1 .45-1 1s.45 1 1 1zM0 15c0 .55.45 1 1 1h.34l2-2H1c-.55 0-1 .45-1 1zm1-7h8.34l2-2H1c-.55 0-1 .45-1 1s.45 1 1 1zm18 2h-.34l-2 2H19c.55 0 1-.45 1-1s-.45-1-1-1zm0 4h-4.34l-2 2H19c.55 0 1-.45 1-1s-.45-1-1-1zM4 19l4.41-1.59-2.81-2.79L4 19zM14.23 5.94l-7.65 7.65 2.83 2.83 7.65-7.65-2.83-2.83z"],map:["M19.54 4.18l.01-.02-6-4-.01.02C13.39.08 13.21 0 13 0s-.39.08-.54.18l-.01-.02L7 3.8 1.55.17l-.01.01A.969.969 0 0 0 1 0C.45 0 0 .45 0 1v14c0 .35.19.64.46.82l-.01.02 6 4 .01-.02c.15.1.33.18.54.18s.39-.08.54-.18l.01.02L13 16.2l5.45 3.63.01-.02c.15.11.33.19.54.19.55 0 1-.45 1-1V5c0-.35-.19-.64-.46-.82zM6 17.13l-4-2.67V2.87l4 2.67v11.59zm6-2.67l-4 2.67V5.54l4-2.67v11.59zm6 2.67l-4-2.67V2.87l4 2.67v11.59z"],"map-create":["M18 9.22v7.91l-4-2.67V9.22c-.61-.55-1-1.33-1-2.22-.35 0-.69-.07-1-.18v7.65l-4 2.67V5.54l2.02-1.35c0-.06-.02-.13-.02-.19 0-1.66 1.34-3 3-3 0-.34.07-.66.17-.97C13.12.02 13.06 0 13 0c-.21 0-.39.08-.54.18l-.01-.02L7 3.8 1.55.17l-.01.01A.969.969 0 0 0 1 0C.45 0 0 .45 0 1v14c0 .35.19.64.46.82l-.01.02 6 4 .01-.02c.15.1.33.18.54.18s.39-.08.54-.18l.01.02L13 16.2l5.45 3.63.01-.02c.15.11.33.19.54.19.55 0 1-.45 1-1V6.82c-.31.11-.65.18-1 .18 0 .89-.39 1.67-1 2.22zM6 17.13l-4-2.67V2.87l4 2.67v11.59zM12 4c0 .55.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V5h2c.55 0 1-.45 1-1s-.45-1-1-1h-2V1c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1z"],"map-marker":["M9.98 0c-3.87 0-7 2.98-7 6.67 0 3.68 7 13.33 7 13.33s7-9.65 7-13.33c0-3.68-3.14-6.67-7-6.67zm0 10c-1.66 0-3-1.34-3-3s1.34-3 3-3 3 1.34 3 3-1.34 3-3 3z"],maximize:["M19 0h-5c-.55 0-1 .45-1 1s.45 1 1 1h2.59L11.3 7.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L18 3.41V6c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zM8 11c-.28 0-.53.11-.71.29L2 16.59V14c0-.55-.45-1-1-1s-1 .45-1 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1H3.41l5.29-5.29c.19-.18.3-.43.3-.71 0-.55-.45-1-1-1z"],media:["M15 9c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm4-7H1c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm-1 13l-6-5-2 2-4-5-4 8V4h16v11z"],menu:["M1 6h18c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm18 3H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm0 5H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],"menu-closed":["M8 6h11c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zM4 6c-.28 0-.53.11-.71.29l-3 3C.11 9.47 0 9.72 0 10c0 .28.11.53.29.71l3 3A1.003 1.003 0 0 0 5 13V7c0-.55-.45-1-1-1zm15 8H8c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1zm0-5H8c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1z"],"menu-open":["M12 9H1c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1zm0 5H1c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1zm0-10H1c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1zm7.71 5.29l-3-3A1.003 1.003 0 0 0 15 7v6a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"merge-columns":["M6.71 6.29a1.003 1.003 0 0 0-1.42 1.42L6.59 9H2V2h5v2.18c.42.15.8.39 1.11.7l.01-.01.88.89V1c0-.55-.45-1-1-1H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-4.76l-.88.88-.01-.01c-.31.31-.69.56-1.11.71V18H2v-7h4.59L5.3 12.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3zM19 0h-7c-.55 0-1 .45-1 1v4.76l.88-.88.01.01c.31-.31.69-.55 1.11-.7V2h5v7h-4.59l1.29-1.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-3 3c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l3 3a1.003 1.003 0 0 0 1.42-1.42L13.41 11H18v7h-5v-2.18c-.42-.15-.8-.39-1.11-.7l-.01.01-.88-.89V19c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"merge-links":["M10 13c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm8-5c-.93 0-1.71.64-1.93 1.5H14V4c0-2.21-1.79-4-4-4S6 1.79 6 4v5.5H3.93C3.71 8.64 2.93 8 2 8c-1.1 0-2 .9-2 2s.9 2 2 2c.93 0 1.71-.64 1.93-1.5H6V16c0 2.21 1.79 4 4 4s4-1.79 4-4v-5.5h2.07c.22.86 1 1.5 1.93 1.5 1.1 0 2-.9 2-2s-.9-2-2-2zm-5 8c0 1.66-1.34 3-3 3s-3-1.34-3-3V4c0-1.66 1.34-3 3-3s3 1.34 3 3v12zM10 3c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],minimize:["M8 11H3c-.55 0-1 .45-1 1s.45 1 1 1h2.59L.3 18.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L7 14.41V17c0 .55.45 1 1 1s1-.45 1-1v-5c0-.55-.45-1-1-1zM20 1a1.003 1.003 0 0 0-1.71-.71L13 5.59V3c0-.55-.45-1-1-1s-1 .45-1 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1h-2.59l5.29-5.29c.19-.18.3-.43.3-.71z"],minus:["M16 9H4c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1z"],"mobile-phone":["M15 0H5c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-5 19c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm4-3H6V3h8v13z"],"mobile-video":["M19 5c-.28 0-.53.11-.71.29L15 8.59V5c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h13c.55 0 1-.45 1-1v-3.59l3.29 3.29c.18.19.43.3.71.3.55 0 1-.45 1-1V6c0-.55-.45-1-1-1z"],moon:["M19 14.15A9.94 9.94 0 0 1 9.94 20C4.45 20 0 15.55 0 10.06 0 6.03 2.4 2.56 5.85 1a9.811 9.811 0 0 0-.88 4.09c0 5.49 4.45 9.94 9.94 9.94 1.46 0 2.84-.31 4.09-.88z"],more:["M3.5 8a2.5 2.5 0 1 0 0 5 2.5 2.5 0 1 0 0-5zM17.5 8a2.5 2.5 0 1 0 0 5 2.5 2.5 0 1 0 0-5zM10.5 8a2.5 2.5 0 1 0 0 5 2.5 2.5 0 1 0 0-5z"],mountain:["M20 16H4l7-11h1l2 2h1l5 9zm-4-5l-1.5-3h-1l-1-1-1-1L8 11.5l3-1.5 1 1 1-1 3 1zM8.055 8L2.79 16H0l7-8h1.055z"],move:["M19.71 9.29l-3-3a1.003 1.003 0 0 0-1.42 1.42L16.59 9H11V3.41l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-3-3C10.53.11 10.28 0 10 0s-.53.11-.71.29l-3 3a1.003 1.003 0 0 0 1.42 1.42L9 3.41V9H3.41L4.7 7.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-3 3C.11 9.47 0 9.72 0 10c0 .28.11.53.29.71l3 3a1.003 1.003 0 0 0 1.42-1.42L3.41 11H9v5.59L7.71 15.3A.965.965 0 0 0 7 15a1.003 1.003 0 0 0-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3a1.003 1.003 0 0 0-1.42-1.42L11 16.59V11h5.59l-1.29 1.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],mugshot:["M19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18h-.07c-.05-.2-.12-.42-.22-.67-.46-1.05-2.68-1.75-4.16-2.4-1.48-.65-1.28-1.05-1.33-1.59-.01-.07-.01-.15-.01-.23.51-.45.92-1.07 1.19-1.78 0 0 .01-.04.02-.05.06-.15.11-.32.15-.48.34-.07.54-.44.61-.78.08-.14.23-.48.2-.87-.05-.5-.25-.73-.47-.82v-.09c0-.63-.06-1.55-.17-2.15-.02-.17-.06-.33-.11-.5a3.67 3.67 0 0 0-1.29-1.86C11.7 3.25 10.81 3 10.02 3s-1.68.25-2.31.73c-.61.47-1.07 1.13-1.29 1.86-.05.16-.09.33-.11.5-.12.6-.17 1.51-.17 2.14v.08c-.24.09-.44.32-.49.83-.04.39.12.73.2.87.08.35.28.72.63.78.04.17.09.33.15.48 0 .01.01.02.01.03l.01.01c.27.72.7 1.35 1.22 1.8 0 .07-.01.14-.01.21-.05.54.1.94-1.38 1.59-1.48.65-3.7 1.35-4.16 2.4-.12.27-.18.49-.23.69H2V2h16v16z"],"multi-select":["M19 3H7c-.55 0-1 .45-1 1v1h12v6h1c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1zm-6 6H1c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h12c.55 0 1-.45 1-1v-6c0-.55-.45-1-1-1zm-1 6H2v-4h10v4zm4-9H4c-.55 0-1 .45-1 1v1h12v6h1c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1z"],music:["M19 0c-.08 0-.16.03-.24.05V.03l-12 3v.02C6.33 3.16 6 3.53 6 4v11.35c-.59-.22-1.27-.35-2-.35-2.21 0-4 1.12-4 2.5S1.79 20 4 20c1.94 0 3.55-.86 3.92-2H8V7.78l10-2.5v7.07c-.59-.22-1.27-.35-2-.35-2.21 0-4 1.12-4 2.5s1.79 2.5 4 2.5c1.94 0 3.55-.86 3.92-2H20V1c0-.55-.45-1-1-1z"],"new-grid-item":["M8 0H1C.45 0 0 .45 0 1v7c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm0 11H1c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-7c0-.55-.45-1-1-1zm6 7h-1v-1c0-.55-.45-1-1-1s-1 .45-1 1v2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1s-.45-1-1-1zm5-7h-2c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1v-2c0-.55-.45-1-1-1zm0-11h-7c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-5 11h-2c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1s1-.45 1-1v-1h1c.55 0 1-.45 1-1s-.45-1-1-1zm5 5c-.55 0-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h2c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1z"],"new-link":["M14.5 12a2.5 2.5 0 0 0-2.45 2h-7.1a2.5 2.5 0 1 0 0 1h7.1a2.5 2.5 0 1 0 2.45-3zM19 5h-2V3c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V7h2c.55 0 1-.45 1-1s-.45-1-1-1z"],"new-object":["M12 4c0 .6.4 1 1 1h2v2c0 .6.4 1 1 1 .5 0 1-.4 1-1V5h2c.5 0 1-.4 1-1s-.5-1-1-1h-2V1c0-.6-.5-1-1-1-.6 0-1 .4-1 1v2h-2c-.6 0-1 .5-1 1zm7 3c0 1.7-1.3 3-3 3s-3-1.3-3-3c-1.7 0-3-1.3-3-3s1.3-3 3-3c0-.2 0-.4.1-.5-1-.3-2-.5-3.1-.5C4.5 0 0 4.5 0 10s4.5 10 10 10 10-4.5 10-10c0-1.1-.2-2.1-.5-3H19z"],"new-person":["M11.41 15.92c-1.46-.65-1.26-1.05-1.31-1.59-.01-.07-.01-.15-.01-.23.5-.45.91-1.07 1.18-1.78 0 0 .01-.04.02-.05.06-.15.11-.32.15-.48.33-.07.53-.44.6-.78.08-.14.23-.48.2-.87-.05-.5-.24-.73-.47-.82v-.09c0-.63-.06-1.55-.17-2.15-.02-.17-.06-.33-.11-.5-.22-.73-.67-1.4-1.27-1.86C9.58 4.25 8.7 4 7.92 4c-.78 0-1.66.25-2.28.73-.61.47-1.06 1.13-1.27 1.86-.05.16-.08.33-.11.5-.12.6-.18 1.51-.18 2.14v.08c-.23.09-.43.32-.48.83-.04.39.12.73.2.87.08.35.28.72.62.78.04.17.09.33.15.48 0 .01.01.02.01.03l.01.01c.27.72.69 1.35 1.21 1.8 0 .07-.01.14-.01.21-.05.54.1.94-1.36 1.59-1.46.65-3.66 1.35-4.11 2.4C-.14 19.38.04 20 .04 20h15.75s.18-.62-.27-1.67c-.45-1.06-2.65-1.75-4.11-2.41zM18.87 3h-2V1c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V5h2c.55 0 1-.45 1-1s-.45-1-1-1z"],"new-prescription":["M11.95 10.23c.16-.18.22-.22.46-.22h1.48c.25 0 .47.08.59.33.1.2.09.41-.05.66l-2.71 3.58L14.88 19c.13.21.16.46.03.69-.12.21-.34.31-.57.31H12.7c-.31 0-.56-.17-.7-.44l-1.9-2.67-1.93 2.68c-.15.27-.42.43-.73.43H5.98c-.25 0-.47-.08-.59-.33-.1-.2-.09-.41.05-.66l3.09-4.35L4.26 9H3v4.32c0 .41-.3.69-.7.69H.7c-.41 0-.7-.28-.7-.69V.69C0 .28.3 0 .7 0h4.42c.71 0 1.36.1 1.94.3.59.2 1.11.49 1.54.87.44.38.78.84 1.02 1.39.25.54.37 1.13.37 1.77 0 1.01-.28 1.88-.84 2.6-.43.54-1.35 1.29-2 1.59l3.09 3.94 1.71-2.23zM4.71 6.04c.71 0 1.45-.16 1.81-.46.33-.28.5-.69.5-1.25s-.17-.97-.5-1.25c-.35-.3-1.1-.46-1.81-.46h-1.7v3.42h1.7zM19 3c.55 0 1 .45 1 1s-.45 1-1 1h-2v2c0 .55-.45 1-1 1s-1-.45-1-1V5h-2c-.55 0-1-.45-1-1s.45-1 1-1h2V1c0-.55.45-1 1-1s1 .45 1 1v2h2z"],"new-text-box":["M19 3h-2V1c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V5h2c.55 0 1-.45 1-1s-.45-1-1-1zM5 7.5v1c0 .28.22.5.5.5s.5-.22.5-.5V8h2v7h-.5c-.28 0-.5.22-.5.5s.22.5.5.5h2c.28 0 .5-.22.5-.5s-.22-.5-.5-.5H9V8h2v.5c0 .28.22.5.5.5s.5-.22.5-.5v-1c0-.28-.22-.5-.5-.5h-6c-.28 0-.5.22-.5.5zM16 9c-.55 0-1 .45-1 1v8H2V5h8c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1v15c0 .55.45 1 1 1h15c.55 0 1-.45 1-1v-9c0-.55-.45-1-1-1z"],ninja:["M20 6s-2.98 2.43-6.12 2.19C13.52 5.31 12.05 0 6 0c0 0 2.41 2.99 2.16 6.12C5.27 6.49 0 7.97 0 14c0 0 2.98-2.43 6.11-2.19C6.47 14.69 7.94 20 14 20c0 0-2.42-2.99-2.16-6.13C14.73 13.51 20 12.02 20 6zm-10 6c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2z"],"not-equal-to":["M9.487 7l.532-3.196a1 1 0 0 1 1.962.392L11.513 7H16a1 1 0 0 1 0 2h-4.82l-.333 2H16a1 1 0 0 1 0 2h-5.487l-.532 3.196a1 1 0 0 1-1.962-.392L8.487 13H4a1 1 0 0 1 0-2h4.82l.333-2H4a1 1 0 1 1 0-2h5.487z"],notifications:["M10 20c1.1 0 2-.9 2-2H8c0 1.1.9 2 2 2zm7-5c-.55 0-1-.45-1-1V8c0-2.61-1.67-4.81-4-5.63V2c0-1.1-.9-2-2-2S8 .9 8 2v.37C5.67 3.19 4 5.39 4 8v6c0 .55-.45 1-1 1s-1 .45-1 1 .45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"notifications-updated":["M10 20c1.1 0 2-.9 2-2H8c0 1.1.9 2 2 2zm2-17.834A2.994 2.994 0 0 0 8 4.99c0 .808.319 1.557.876 2.114l2.97 2.99a2.99 2.99 0 0 0 4.154.072V14c0 .55.45 1 1 1s1 .45 1 1-.45 1-1 1H3c-.55 0-1-.45-1-1s.45-1 1-1 1-.45 1-1V8c0-2.61 1.67-4.81 4-5.63V2c0-1.1.9-2 2-2s2 .9 2 2v.166zm1.26 6.514l-2.97-2.99a.973.973 0 0 1-.29-.7c0-.55.44-1 .99-1 .27 0 .52.11.7.29l2.28 2.28 4.27-4.27a.99.99 0 0 1 .7-.29c.55 0 1 .45 1 1 0 .28-.11.53-.3.7l-4.98 4.98a.99.99 0 0 1-1.4 0z"],"numbered-list":["M1.74 9.01h1.27V1h-.95c-.04.24-.12.45-.26.62-.13.17-.29.3-.47.41-.19.11-.4.18-.63.23-.23.04-.46.07-.71.07v1.03h1.75v5.65zm.43 7.93c.18-.14.37-.28.58-.43.21-.14.42-.29.63-.45.21-.16.41-.33.61-.5.2-.18.37-.38.52-.59.15-.21.28-.45.37-.7.09-.25.14-.54.14-.85 0-.25-.04-.52-.12-.8-.08-.28-.21-.54-.39-.78-.19-.24-.43-.44-.73-.59-.3-.17-.68-.25-1.12-.25-.41 0-.77.08-1.08.23-.32.16-.58.37-.8.64-.22.27-.38.59-.49.96-.11.37-.16.77-.16 1.21h1.19c.01-.28.03-.53.08-.77s.12-.45.21-.62c.09-.18.22-.31.38-.42.16-.1.35-.15.59-.15.26 0 .47.05.63.14.16.09.29.21.38.35.09.14.15.29.18.45.03.16.05.31.05.45-.01.31-.08.58-.22.81-.14.24-.32.45-.53.66-.22.2-.45.39-.71.57-.26.18-.51.36-.74.54-.5.36-.89.78-1.17 1.27-.3.47-.45 1.04-.46 1.69H5v-1.14H1.43c.05-.17.14-.33.27-.49.13-.15.29-.3.47-.44zM18 4.02H8c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-1c0-.56-.45-1-1-1zm0 9H8c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-1c0-.56-.45-1-1-1z"],numerical:["M2.39 5.75c-.17.21-.38.39-.63.52s-.52.23-.83.29c-.3.05-.61.08-.93.08v1.24h2.49V15h1.49V4.98H2.73c-.05.31-.17.57-.34.77zm17.2 4.71c-.27-.44-.65-.71-1.14-.82v-.02c.42-.16.72-.43.92-.79.2-.36.29-.79.29-1.27 0-.42-.08-.8-.23-1.12-.15-.33-.36-.59-.62-.8-.26-.21-.55-.37-.87-.48-.32-.11-.65-.16-.98-.16-.43 0-.82.08-1.16.25-.34.16-.63.39-.87.69-.24.29-.43.64-.57 1.04-.14.4-.22.83-.23 1.3h1.39c-.01-.25.02-.49.07-.72.06-.23.14-.44.26-.63s.27-.34.45-.45c.18-.11.39-.17.63-.17.39 0 .71.12.96.37s.37.58.37.99c0 .29-.05.54-.16.74-.11.2-.25.36-.43.47-.18.11-.38.19-.61.24-.23.05-.46.06-.68.05v1.17c.28-.01.55 0 .81.03s.5.1.71.21c.21.11.38.28.51.5.13.22.2.52.2.89 0 .55-.16.97-.47 1.27-.31.3-.7.45-1.17.45-.55 0-.95-.19-1.23-.58-.27-.39-.4-.88-.38-1.46h-1.39c.01.5.08.96.21 1.38.13.41.32.77.57 1.06.25.29.56.52.93.68.37.16.8.24 1.3.24.41 0 .79-.07 1.16-.21.37-.14.69-.33.96-.58.28-.25.5-.56.66-.92a3 3 0 0 0 .24-1.23c0-.64-.14-1.17-.41-1.61zM8.58 12.41c.21-.18.45-.36.7-.53.25-.18.5-.36.75-.56.25-.2.49-.41.73-.63.23-.22.44-.47.63-.74.18-.27.33-.56.44-.88.11-.32.16-.67.16-1.07 0-.32-.05-.65-.14-1-.09-.35-.25-.68-.47-.97-.22-.3-.51-.55-.87-.74-.36-.2-.81-.29-1.35-.29-.49 0-.93.1-1.3.29-.37.18-.69.44-.95.78-.26.33-.45.73-.58 1.2-.13.46-.2.96-.2 1.5h1.43c.01-.35.04-.67.09-.97.05-.3.14-.56.25-.78.11-.22.26-.39.45-.52s.43-.19.71-.19c.31 0 .56.06.75.18.19.12.34.26.45.43.11.17.18.36.22.56.04.2.06.39.06.57-.01.38-.1.72-.26 1.02-.15.3-.37.57-.63.83-.26.25-.54.49-.85.71-.31.22-.61.45-.89.68-.6.45-1.06.98-1.41 1.58-.35.61-.52 1.32-.53 2.13h6.01v-1.43H7.69c.06-.21.17-.42.33-.61s.34-.38.56-.55z"],office:["M19 6h-5V1c0-.55-.45-1-1-1H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h4v-6h4v6h10c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1zM6 12H2V8h4v4zm0-6H2V2h4v4zm6 6H8V8h4v4zm0-6H8V2h4v4zm6 11h-4v-3h4v3zm0-5h-4V8h4v4z"],offline:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zM7 18l2-7H5l8-9-2 7h4l-8 9z"],"oil-field":["M19 17.99h-1.36l-4.35-9.57 2.91-.86 1.66 4.1c.11.27.43.4.72.31.12-.04.22-.11.28-.2.06-.11 1.47-2.08 1.05-5.6C19.79 5.12 19.3 0 16.01 0 14.89.01 13.99.83 14 1.84c0 .19.04.38.1.56l1.34 3.31L.72 10.03v.02c-.41.12-.72.49-.72.94 0 .55.45 1 1 1 .1 0 .19-.03.28-.06v.02l2-.59 1.47 6.63H3c-.55 0-1 .45-1 1s.45 1 1 1h16c.55 0 1-.45 1-1s-.45-1-1-1zM5.2 10.8l3.95-1.16-2.83 6.22L5.2 10.8zm2.35 7.19l3.95-8.68 3.95 8.68h-7.9z"],"one-column":["M14.94 0h-4c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-8 6c-.28 0-.53.11-.71.29l-3 3c-.18.18-.29.43-.29.71s.11.53.29.71l3 3A1.003 1.003 0 0 0 7.94 13V7c0-.55-.45-1-1-1z"],outdated:["M10 0c5.52 0 10 4.48 10 10s-4.48 10-10 10S0 15.52 0 10c0-.55.45-1 1-1s1 .45 1 1c0 4.42 3.58 8 8 8s8-3.58 8-8-3.58-8-8-8C7.47 2 5.22 3.17 3.76 5H5c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V2c0-.55.45-1 1-1s1 .45 1 1v2.05C3.82 1.6 6.71 0 10 0zm1 16H9v-2h2v2zm0-3H9V4h2v9z"],"page-layout":["M19 1H1c-.55 0-1 .45-1 1v16c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM7 17H2V8h5v9zm11 0H8V8h10v9zm0-10H2V3h16v4z"],"panel-stats":["M1 1h18a1 1 0 0 1 1 1v15a1 1 0 0 1-1 1H1a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1zm1 2v13h16V3H2zm9 0h1v13h-1V3zm2 7h3.952v1H13v-1zm0 2h3.952v1H13v-1zm0 2h3.952v1H13v-1zm0-6h3.952v1H13V8zm0-2h3.952v1H13V6zm0-2h3.952v1H13V4z"],"panel-table":["M19 1H1c-.6 0-1 .4-1 1v15c0 .6.4 1 1 1h18c.6 0 1-.4 1-1V2c0-.6-.4-1-1-1zm-9 11H7V9h3v3zm0-4H7V5h3v3zm-8 8V3h4v13H2zm5 0v-3h3v3H7zm11 0h-7v-3h7v3zm0-4h-7V9h7v3zm0-4h-7V5h7v3z"],paperclip:["M18.35 2.67A5.664 5.664 0 0 0 14.33 1c-1.44 0-2.89.56-3.99 1.67l-9.16 9.27C.4 12.73 0 13.78 0 14.83s.39 2.1 1.18 2.9c.78.79 1.82 1.18 2.85 1.18 1.04 0 2.07-.39 2.87-1.2l9.14-9.27c.96-.96.96-2.5.02-3.45-.94-.95-2.49-.96-3.44 0l-7.59 7.69c-.31.32-.3.83.01 1.14.31.31.81.31 1.13.02l7.59-7.69c.31-.31.84-.31 1.13-.02.31.31.31.85 0 1.16l-9.14 9.27c-.93.95-2.54.93-3.45.02-.94-.95-.92-2.55.02-3.49l9.16-9.25c1.55-1.56 4.18-1.59 5.72-.03 1.56 1.57 1.55 4.26 0 5.82l-8.89 9.02c-.3.31-.3.81.01 1.11.3.3.79.31 1.1.01v.01l8.91-9.02A5.645 5.645 0 0 0 20 6.73c0-1.48-.55-2.94-1.65-4.06z"],paragraph:["M16.5 1H7C4.2 1 2 3.2 2 6s2.2 5 5 5v6.5c0 .8.7 1.5 1.5 1.5s1.5-.7 1.5-1.5V4h2v13.5c0 .8.7 1.5 1.5 1.5s1.5-.7 1.5-1.5V4h1.5c.8 0 1.5-.7 1.5-1.5S17.3 1 16.5 1z"],path:["M18 0H2C.9 0 0 .9 0 2s.9 2 2 2h7v4H4c-1.1 0-2 .9-2 2s.9 2 2 2h5v4H6c-1.1 0-2 .9-2 2s.9 2 2 2h8c1.1 0 2-.9 2-2s-.9-2-2-2h-3v-4h5c1.1 0 2-.9 2-2s-.9-2-2-2h-5V4h7c1.1 0 2-.9 2-2s-.9-2-2-2z"],"path-search":["M4 7c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm15 11.69l-5-2.5v-3.63c-.32.11-.66.22-1 .29v3.32l-6 2.57v-7.25c-.36-.27-.69-.57-1-.9v8.1l-5-2.5V10c.55 0 1-.45 1-1s-.45-1-1-1V1.31l3.43 1.71c.11-.31.24-.62.39-.92L.72.05A.545.545 0 0 0 .5 0C.22 0 0 .22 0 .5v16c0 .2.12.36.28.44l6 3c.07.04.14.06.22.06.07 0 .14-.01.2-.04l6.79-2.91 5.79 2.9c.07.03.14.05.22.05.28 0 .5-.22.5-.5v-4.21c-.31.13-.64.21-1 .21v3.19zM10 5c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm3-1c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm6.72-.94l-1.43-.72c.2.43.36.89.48 1.36l.23.11V5.5c-.55 0-1 .45-1 1s.45 1 1 1v1.96l1 1V3.5c0-.2-.12-.36-.28-.44zm-3.69 5.56c.14-.21.27-.42.38-.65.02-.04.04-.07.05-.11.11-.22.2-.45.28-.69v-.01c.07-.24.13-.48.17-.73l.03-.17c.04-.25.06-.5.06-.76C17 2.46 14.54 0 11.5 0S6 2.46 6 5.5 8.46 11 11.5 11c.26 0 .51-.02.76-.06l.17-.03c.25-.04.49-.1.73-.17h.01c.24-.08.47-.17.69-.28.04-.02.07-.03.11-.05.23-.11.44-.24.65-.38l.18.18 3.5 3.5c.17.18.42.29.7.29a1.003 1.003 0 0 0 .71-1.71l-3.68-3.67zm-4.53.88c-2.21 0-4-1.79-4-4s1.79-4 4-4 4 1.79 4 4-1.79 4-4 4z"],pause:["M7 3H4c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1zm9 0h-3c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],people:["M16.94 17a4.92 4.92 0 0 0-.33-1.06c-.45-.97-1.37-1.52-3.24-2.3-.17-.07-.76-.31-.77-.32-.1-.04-.2-.08-.28-.12.05-.14.04-.29.06-.45 0-.05.01-.11.01-.16-.25-.21-.47-.48-.65-.79.22-.34.41-.71.56-1.12l.04-.11c-.01.02-.01.02-.02.08l.06-.15c.36-.26.6-.67.72-1.13.18-.37.29-.82.25-1.3-.05-.5-.21-.92-.47-1.22-.02-.53-.06-1.11-.12-1.59.38-.17.83-.26 1.24-.26.59 0 1.26.19 1.73.55.46.35.8.85.97 1.4.04.13.07.25.08.38.08.45.13 1.14.13 1.61v.07c.16.07.31.24.35.62.02.29-.09.55-.15.65-.05.26-.2.53-.46.59-.03.12-.07.25-.11.36-.01.01-.01.04-.01.04-.2.53-.51 1-.89 1.34 0 .06 0 .12.01.17.04.41-.11.71 1 1.19 1.1.5 2.77 1.01 3.13 1.79.34.79.2 1.25.2 1.25h-3.04zm-5.42-3.06c1.47.66 3.7 1.35 4.18 2.39.45 1.05.27 1.67.27 1.67H.04s-.19-.62.27-1.67c.46-1.05 2.68-1.75 4.16-2.4 1.48-.65 1.33-1.05 1.38-1.59 0-.07.01-.14.01-.21-.52-.45-.95-1.08-1.22-1.8l-.01-.01c0-.01-.01-.02-.01-.03-.07-.15-.12-.32-.16-.49-.34-.06-.54-.43-.62-.78-.08-.14-.24-.48-.2-.87.05-.51.26-.74.49-.83v-.08c0-.64.05-1.55.17-2.15a3.648 3.648 0 0 1 1.4-2.36C6.32 2.25 7.21 2 8 2s1.68.25 2.31.73a3.63 3.63 0 0 1 1.4 2.36c.11.6.17 1.52.17 2.15v.09c.22.09.42.32.47.82.03.39-.12.73-.2.87-.07.34-.27.71-.61.78-.04.16-.09.33-.15.48-.01.01-.02.05-.02.05-.27.71-.68 1.33-1.19 1.78 0 .08 0 .16.01.23.05.55-.15.95 1.33 1.6z"],percentage:["M15 10c-1.66 0-3 1.34-3 3v2c0 1.66 1.34 3 3 3s3-1.34 3-3v-2c0-1.66-1.34-3-3-3zm1 5c0 .55-.45 1-1 1s-1-.45-1-1v-2c0-.55.45-1 1-1s1 .45 1 1v2zM8 7V5c0-1.66-1.34-3-3-3S2 3.34 2 5v2c0 1.66 1.34 3 3 3s3-1.34 3-3zM4 7V5c0-.55.45-1 1-1s1 .45 1 1v2c0 .55-.45 1-1 1s-1-.45-1-1zm11-4a1.003 1.003 0 0 0-1.88-.48L5.14 16.49a1.003 1.003 0 0 0 1.74.99l7.99-13.97c.08-.15.13-.32.13-.51z"],person:["M19.61 17.91c-.57-1.32-3.35-2.19-5.19-3.01-1.85-.82-1.59-1.31-1.66-1.99-.01-.09-.01-.19-.02-.29.63-.56 1.15-1.33 1.49-2.22 0 0 .02-.05.02-.06.07-.19.13-.39.19-.6.42-.09.67-.55.76-.98.1-.17.29-.6.25-1.08-.06-.62-.31-.91-.59-1.03v-.11c0-.79-.07-1.93-.22-2.68A4.55 4.55 0 0 0 12.9.92C12.11.32 11 0 10.01 0s-2.1.32-2.89.92a4.55 4.55 0 0 0-1.74 2.94c-.14.75-.22 1.89-.22 2.68v.1c-.29.11-.55.4-.61 1.04-.04.48.15.91.25 1.08.1.44.35.91.79.98.05.21.12.41.19.6 0 .01.01.03.01.04l.01.02c.34.91.87 1.69 1.52 2.25 0 .09-.01.18-.02.26-.07.68.13 1.17-1.72 1.99S.96 16.59.39 17.91C-.18 19.23.05 20 .05 20h19.9s.23-.77-.34-2.09z"],phone:["M19.91 15.51c-.08-.08-4.21-2.5-4.35-2.57a.876.876 0 0 0-.4-.1c-.19 0-.42.13-.71.4-.28.27-1.17 1.49-1.43 1.76s-.48.4-.65.4c-.08 0-.19-.02-.32-.07s-1.45-.73-4.2-3.15-3.11-4-3.13-4.44c0-.17.13-.39.4-.65.28-.25.57-.51.89-.74.32-.24.61-.5.88-.78s.4-.52.4-.71c0-.13-.03-.27-.1-.4C7.12 4.32 4.62.19 4.53.1c-.19-.18-.92-.1-1.29.1C.25 1.82 0 4 .05 4.86c.05.89.61 5.58 5.2 9.93 5.7 5.41 9.66 5.2 9.92 5.2.87 0 3.52-.48 4.65-3.19.16-.38.31-1.07.09-1.29z"],"pie-chart":["M9 .98c-4.5.5-8 4.31-8 8.94 0 4.97 4.03 9.04 9 9.04 4.63 0 8.44-3.96 8.94-7.96H9V.98z","M10-.08V10h10C20 4 15.52-.08 10-.08z"],pin:["M11.77 1.16c-.81.81-.74 2.28.02 3.76L6.1 8.71c-2.17-1.46-4.12-2-4.94-1.18l4.95 4.95-4.95 6.36 6.36-4.95 4.95 4.95c.82-.82.27-2.77-1.19-4.94l3.8-5.69c1.47.76 2.94.84 3.76.02l-7.07-7.07z"],pivot:["M5.83 9.75L.29 15.29a1.003 1.003 0 0 0 1.42 1.42l5.54-5.54c-.57-.37-1.05-.85-1.42-1.42zM19 11c-.55 0-1 .45-1 1v1.59l-3.83-3.83c-.37.56-.85 1.04-1.41 1.41L16.59 15H15c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1zm-5-4c0-2.21-1.79-4-4-4S6 4.79 6 7s1.79 4 4 4 4-1.79 4-4zm-4 2c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2z"],"pivot-table":["M3 5H1c-.55 0-1 .45-1 1v13c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm0-5H1C.45 0 0 .45 0 1v2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm13.71 5.29C16.53 5.11 16.28 5 16 5s-.53.11-.71.29l-3 3a1.003 1.003 0 0 0 1.42 1.42L15 8.41V11c0 2.21-1.79 4-4 4H8.41l1.29-1.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-3 3c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l3 3a1.003 1.003 0 0 0 1.42-1.42L8.41 17H11c3.31 0 6-2.69 6-6V8.41l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-3-3zM19 0H6c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],play:["M16 10c0-.36-.2-.67-.49-.84l.01-.01-10-6-.01.01A.991.991 0 0 0 5 3c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1 .19 0 .36-.07.51-.16l.01.01 10-6-.01-.01c.29-.17.49-.48.49-.84z"],plus:["M16 9h-5V4c0-.55-.45-1-1-1s-1 .45-1 1v5H4c-.55 0-1 .45-1 1s.45 1 1 1h5v5c0 .55.45 1 1 1s1-.45 1-1v-5h5c.55 0 1-.45 1-1s-.45-1-1-1z"],"polygon-filter":["M18 7c-.27 0-.52.05-.75.15l-6.28-4.88c.01-.09.03-.18.03-.27 0-1.1-.9-2-2-2S7 .9 7 2c0 .06.01.12.02.19l-4.19 3C2.57 5.07 2.29 5 2 5 .9 5 0 5.9 0 7c0 .74.4 1.38 1 1.72v7.55c-.6.35-1 .99-1 1.73 0 1.1.9 2 2 2 .74 0 1.38-.4 1.72-1h7.55c.35.6.98 1 1.72 1 1.1 0 2-.9 2-2 0-.37-.11-.72-.29-1.02L18.03 11A2 2 0 0 0 18 7zm-5.03 9c-.72.01-1.35.41-1.69 1H3.72c-.17-.3-.42-.55-.72-.72V8.72c.6-.34 1-.98 1-1.72 0-.06-.01-.12-.02-.19l4.19-3c.26.12.54.19.83.19.27 0 .52-.05.75-.15l6.28 4.88c-.01.09-.03.18-.03.27 0 .37.11.72.29 1.02L12.97 16z"],power:["M10 10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1S9 .45 9 1v8c0 .55.45 1 1 1zm3-7.45v2.16c2.36 1.12 4 3.5 4 6.29 0 3.87-3.13 7-7 7s-7-3.13-7-7c0-2.79 1.64-5.17 4-6.29V2.55C3.51 3.79 1 7.09 1 11a9 9 0 0 0 18 0c0-3.91-2.51-7.21-6-8.45z"],"predictive-analysis":["M20 8.01c0-1.26-.61-2.43-1.61-3.12C17.86 2.5 15.8.79 13.4.79c-.58 0-1.14.1-1.69.29A3.533 3.533 0 0 0 9.17 0C8.05 0 7 .55 6.32 1.45c-.15-.02-.3-.03-.45-.03-1.63 0-3.03 1.12-3.46 2.71C.97 4.65 0 6.05 0 7.66c0 .48.09.95.26 1.4-.17.44-.26.91-.26 1.39 0 1.38.72 2.64 1.89 3.29.67.7 1.59 1.09 2.54 1.09.61 0 1.19-.15 1.71-.45.68.82 1.68 1.3 2.73 1.3.66 0 1.28-.18 1.83-.52.61.49 1.34.81 2.11.91 1.3 1.43 2.3 3.28 2.31 3.3 0 0 .35.61.33.61.96-.01 1.77-.2 1.64-1.3.01.02-.92-2.89-.92-2.89.52-.26.94-.69 1.21-1.23 1.12-.66 1.84-1.91 1.84-3.26 0-.3-.03-.6-.1-.89.57-.64.88-1.51.88-2.4zm-1.54 1.28l-.18-.2-.77-.84c-.33-.37-.67-1.17-.73-1.73 0 0-.13-1.25-.13-1.26-.06-.74-1.17-.73-1.13.14 0 .02.13 1.26.13 1.26.04.36.15.77.3 1.17-.08-.01-.15-.02-.22-.02 0 0-2.57-.12-2.57-.13-.73-.03-.89 1.22-.05 1.25l2.57.13c.53.03 1.29.37 1.61.72l.61.67.02.06c.1.27.14.55.14.83 0 .93-.51 1.77-1.34 2.18l-.2.1-.09.23c-.19.48-.6.82-1.1.93l-.67.14.87 2.75c-.48-.76-1.19-1.79-2.02-2.67l-.15-.16-.21-.02c-.51-.04-.99-.21-1.42-.48l1.7-1.48c.44-.39 1.04-.55 1.24-.49 0 0 .78.22.78.23.78.2 1.03-.92.29-1.21l-.78-.23c-.69-.2-1.67.22-2.24.72l-1.91 1.66-.39.32c-.44.36-.93.55-1.5.55-.8 0-1.54-.41-1.97-1.07v-1.88c0-.5.21-.98.34-1.07 0 0 .65-.43.64-.43.87-.69.21-1.57-.64-1.14 0-.01-.65.43-.65.43-.31.2-.54.56-.7.97-.13-.13-.28-.25-.43-.35 0 0-1.91-1.26-1.91-1.28-.81-.56-1.5.63-.61 1.11 0-.02 1.89 1.28 1.89 1.28.46.31.77.97.77 1.36v.84c-.43.24-.78.36-1.24.36-.67 0-1.31-.29-1.77-.79l-.07-.08-.09-.05a2.425 2.425 0 0 1-1.31-2.16c0-.38.09-.74.25-1.08l.15-.31-.14-.33c-.17-.34-.25-.7-.25-1.08 0-1.13.76-2.1 1.85-2.37l.39-.09.07-.43a2.41 2.41 0 0 1 2.39-2.05c.19 0 .39.02.58.07l.4.1.22-.38A2.41 2.41 0 0 1 9.17 1.3c.55 0 1.08.19 1.5.53l-.44.45-.01-.01-.31.31c-.41.35-.92.53-1.11.5 0 0-.84-.13-.84-.14-.83-.15-1.09 1.08-.18 1.29.01 0 .84.14.84.14.03 0 .06 0 .09.01-.14.46-.18.96-.12 1.4 0 0 .21 1.24.19 1.23.13.65 1.32.44 1.16-.22 0-.01-.19-1.23-.19-1.23-.07-.48.15-1.19.45-1.5l.48-.5c.07-.06.13-.12.19-.18l.93-.95c.5-.23 1.04-.34 1.59-.34 1.93 0 3.57 1.4 3.89 3.34l.05.31.26.15a2.445 2.445 0 0 1 .87 3.4z"],prescription:["M13.95 10.23c.16-.18.22-.22.46-.22h1.48c.25 0 .47.08.59.33.1.2.09.41-.05.66l-2.71 3.58L16.88 19c.13.21.16.46.03.69-.12.21-.34.31-.57.31H14.7c-.31 0-.56-.17-.7-.44l-1.9-2.67-1.93 2.68c-.15.27-.42.43-.73.43H7.98c-.25 0-.47-.08-.59-.33-.1-.2-.09-.41.05-.66l3.09-4.35L6.26 9H5v4.32c0 .41-.3.69-.7.69H2.7c-.41 0-.7-.28-.7-.69V.69c0-.41.3-.69.7-.69h4.42c.71 0 1.36.1 1.94.3.59.2 1.11.49 1.54.87.44.38.78.84 1.02 1.39.24.54.36 1.14.36 1.78 0 1.01-.28 1.88-.84 2.6-.43.54-1.35 1.29-2 1.59l3.09 3.94 1.72-2.24zM6.71 6.04c.71 0 1.45-.16 1.81-.46.33-.28.5-.69.5-1.25s-.17-.97-.5-1.25c-.35-.3-1.1-.46-1.81-.46h-1.7v3.42h1.7z"],presentation:["M19 1h-8c0-.55-.45-1-1-1S9 .45 9 1H1c-.55 0-1 .45-1 1s.45 1 1 1h1v11c0 .55.45 1 1 1h4.59L4.3 18.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L9 16.41V19c0 .55.45 1 1 1s1-.45 1-1v-2.59l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L12.41 15H17c.55 0 1-.45 1-1V3h1c.55 0 1-.45 1-1s-.45-1-1-1zm-3 12H4V3h12v10z"],print:["M14 16H6v-4H4v5c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-5h-2v4zm2-13c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v1h12V3zm3 2H1c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h2v-3h14v3h2c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm-1 4h-2V7h2v2z"],projects:["M18 4c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v2h16V4zm-2-3c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v1h12V1zm3 6H1c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-5 7c0 .55-.45 1-1 1H7c-.55 0-1-.45-1-1v-2h1v2h6v-2h1v2z"],properties:["M2 15c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0-7c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm5-4h12c.55 0 1-.45 1-1s-.45-1-1-1H7c-.55 0-1 .45-1 1s.45 1 1 1zM2 1C.9 1 0 1.9 0 3s.9 2 2 2 2-.9 2-2-.9-2-2-2zm17 8H7c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zm0 7H7c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1z"],property:["M3 5c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm5-1h11c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zM3 15c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm16 1H8c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1zm-1-8H9c-1.1 0-2 .9-2 2s.9 2 2 2h9c1.1 0 2-.9 2-2s-.9-2-2-2zM3 7c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3z"],"publish-function":["M7.01 10.11c.35-.64.72-1.68 1.09-3.11l.8-3.03h.96l.24-.77h-.99c.28-1.11.66-1.92 1.12-2.43.28-.32.56-.48.83-.48.05 0 .1.02.13.05.03.03.05.07.05.12 0 .04-.04.13-.11.25-.08.12-.11.24-.11.35 0 .15.06.28.18.39.12.11.27.16.45.16.2 0 .36-.07.49-.2s.2-.31.2-.54c0-.26-.1-.47-.3-.63-.2-.16-.52-.24-.96-.24-.68 0-1.3.19-1.86.58-.55.38-1.08 1.02-1.58 1.91-.17.3-.34.5-.49.59-.15.08-.4.13-.74.12l-.23.77h.95L5.74 9.21c-.23.86-.39 1.39-.47 1.59-.12.29-.3.54-.54.75-.1.08-.21.12-.35.12-.04 0-.07-.01-.1-.03l-.03-.04c0-.02.03-.07.1-.13.07-.07.1-.17.1-.31 0-.15-.05-.28-.16-.38-.11-.1-.27-.15-.47-.15-.25 0-.44.07-.59.2-.15.12-.23.28-.23.46 0 .19.09.36.27.5.19.14.47.21.86.21.61 0 1.16-.15 1.63-.46.48-.31.89-.79 1.25-1.43zm3.7 1.18c-.18-.18-.43-.29-.71-.29s-.53.11-.71.29l-3 3a1.003 1.003 0 0 0 1.42 1.42L9 14.41V19c0 .55.45 1 1 1s1-.45 1-1v-4.59l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-3-3zm4.15-6.78c.17-.13.36-.2.55-.2.07 0 .2.03.39.08s.36.08.5.08c.2 0 .37-.07.5-.2.13-.14.2-.31.2-.52 0-.22-.07-.4-.2-.53s-.33-.2-.58-.2c-.22 0-.43.05-.63.15-.2.1-.45.32-.75.67-.23.25-.56.7-1.01 1.33a6.52 6.52 0 0 0-.91-2.15l-2.38.39-.05.25c.18-.03.33-.05.45-.05.24 0 .43.1.59.3.25.31.59 1.24 1.02 2.79-.34.44-.58.73-.7.87-.21.22-.38.36-.52.43-.1.05-.22.08-.35.08-.1 0-.26-.05-.49-.16a1.01 1.01 0 0 0-.42-.11c-.23 0-.42.07-.57.22-.17.14-.24.32-.24.55 0 .21.07.38.21.51.14.13.33.2.56.2.23 0 .44-.05.64-.14.2-.09.45-.29.75-.59s.72-.78 1.25-1.43c.2.62.38 1.07.53 1.35.15.28.32.49.52.61.19.12.44.19.73.19.28 0 .57-.1.86-.3.38-.25.77-.69 1.17-1.31l-.25-.14c-.27.37-.48.6-.61.69-.09.06-.19.09-.31.09-.14 0-.28-.09-.42-.26-.23-.29-.54-1.09-.93-2.4.37-.58.66-.96.9-1.14z"],pulse:["M19 10h-2.38L14.9 6.55h-.01c-.17-.32-.5-.55-.89-.55-.43 0-.79.28-.93.66h-.01l-2.75 7.57L7.98 1.82h-.02A.978.978 0 0 0 7 1c-.44 0-.8.29-.94.69h-.01L3.28 10H1c-.55 0-1 .45-1 1s.45 1 1 1h3c.44 0 .8-.29.94-.69h.01l1.78-5.34 2.29 12.21h.02c.08.46.47.82.96.82.43 0 .79-.28.93-.66h.01l3.21-8.82.96 1.92h.01c.16.33.49.56.88.56h3c.55 0 1-.45 1-1s-.45-1-1-1z"],random:["M14.47 5h2.12L15.3 6.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3a1.003 1.003 0 0 0-1.42 1.42L16.59 3H14c-.31 0-.57.15-.76.37l-.01-.01-2.93 3.52 1.3 1.56L14.47 5zm2.24 7.29a1.003 1.003 0 0 0-1.42 1.42l1.3 1.29h-2.12L4.77 3.36l-.01.01A.998.998 0 0 0 4 3H1c-.55 0-1 .45-1 1s.45 1 1 1h2.53l9.7 11.64.01-.01c.19.22.45.37.76.37h2.59l-1.29 1.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3zM3.53 15H1c-.55 0-1 .45-1 1s.45 1 1 1h3c.31 0 .57-.15.76-.37l.01.01 2.93-3.52-1.3-1.56L3.53 15z"],record:["M10 3a7 7 0 1 0 0 14 7 7 0 1 0 0-14z"],redo:["M19.71 5.29l-4-4a1.003 1.003 0 0 0-1.42 1.42L16.59 5H6c-3.31 0-6 2.69-6 6s2.69 6 6 6h5v-2H6c-2.21 0-4-1.79-4-4s1.79-4 4-4h10.59L14.3 9.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zM15 14c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],refresh:["M19 1c-.55 0-1 .45-1 1v2.06C16.18 1.61 13.29 0 10 0 4.48 0 0 4.48 0 10c0 .55.45 1 1 1s1-.45 1-1c0-4.42 3.58-8 8-8 2.52 0 4.76 1.18 6.22 3H15c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zm0 8c-.55 0-1 .45-1 1 0 4.42-3.58 8-8 8-2.52 0-4.76-1.18-6.22-3H5c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1s1-.45 1-1v-2.06C3.82 18.39 6.71 20 10 20c5.52 0 10-4.48 10-10 0-.55-.45-1-1-1z"],"regression-chart":["M19 16H3.1L19.31 3.39l-.61-.79L2 15.59V3c0-.55-.45-1-1-1s-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm-9-9c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm-5 4c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm10-2c0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2-2 .9-2 2zm-5 4c0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2-2 .9-2 2z"],remove:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8zm5-9H5c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1z"],"remove-column":["M19 0H5c-.55 0-1 .45-1 1v4h2V2h5v16H6v-3H4v4c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18h-5V2h5v16zM6.29 13.71a1.003 1.003 0 0 0 1.42-1.42L5.41 10 7.7 7.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L4 8.59l-2.29-2.3A1.003 1.003 0 0 0 .29 7.71L2.59 10 .3 12.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L4 11.41l2.29 2.3z"],"remove-column-left":["M4 11h6c.55 0 1-.45 1-1s-.45-1-1-1H4c-.55 0-1 .45-1 1s.45 1 1 1zM19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-7 18H2V2h10v16zm6 0h-5V2h5v16z"],"remove-column-right":["M19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM7 18H2V2h5v16zm11 0H8V2h10v16zm-8-7h6c.55 0 1-.45 1-1s-.45-1-1-1h-6c-.55 0-1 .45-1 1s.45 1 1 1z"],"remove-row-bottom":["M7 14h6c.55 0 1-.45 1-1s-.45-1-1-1H7c-.55 0-1 .45-1 1s.45 1 1 1zM19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H2V8h16v10zm0-11H2V2h16v5z"],"remove-row-top":["M7 8h6c.55 0 1-.45 1-1s-.45-1-1-1H7c-.55 0-1 .45-1 1s.45 1 1 1zm12-8H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H2v-5h16v5zm0-6H2V2h16v10z"],repeat:["M14 6c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1s-1 .45-1 1v2.05C16.18 1.6 13.29 0 10 0 4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10c0-.55-.45-1-1-1s-1 .45-1 1c0 4.42-3.58 8-8 8s-8-3.58-8-8 3.58-8 8-8c2.53 0 4.77 1.17 6.24 3H15c-.55 0-1 .45-1 1z"],reset:["M6 6c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V2c0-.55.45-1 1-1s1 .45 1 1v2.05C3.82 1.6 6.71 0 10 0c5.52 0 10 4.48 10 10s-4.48 10-10 10S0 15.52 0 10c0-.55.45-1 1-1s1 .45 1 1c0 4.42 3.58 8 8 8s8-3.58 8-8-3.58-8-8-8C7.47 2 5.23 3.17 3.76 5H5c.55 0 1 .45 1 1z"],resolve:["M8.7 4.7C7.9 4.2 7 4 6 4c-3.3 0-6 2.7-6 6s2.7 6 6 6c1 0 1.9-.2 2.7-.7C7.3 14 6.5 12.1 6.5 10s.9-4 2.2-5.3zM14 4c-1 0-1.9.2-2.7.7 1.4 1.4 2.2 3.2 2.2 5.3s-.9 4-2.2 5.3c.8.5 1.7.7 2.7.7 3.3 0 6-2.7 6-6s-2.7-6-6-6zm-4 1.5C8.8 6.7 8 8.2 8 10s.8 3.3 2 4.4c1.2-1.1 2-2.7 2-4.5s-.8-3.3-2-4.4z"],rig:["M7 4.2C7 5.75 8.34 7 10 7s3-1.46 3-2.8C13 1.45 10.94 0 10 0H6c0 2.74 3.76 1.96 1 4.2zm11.71 14.09L13 12.59V9.01c0-.55-.45-1-1-1H8c-.55 0-1 .45-1 1v3.58l-5.71 5.7a1.003 1.003 0 0 0 1.42 1.42L7 15.42V19c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-3.58l4.29 4.29a1.003 1.003 0 0 0 1.42-1.42zM10.21 8c.01 0 .01.01 0 0 .01.01.01 0 0 0z"],"right-join":["M8.7 4.7C7.4 6 6.5 7.9 6.5 10s.8 4 2.2 5.3c-.8.5-1.7.7-2.7.7-3.3 0-6-2.7-6-6s2.7-6 6-6c1 0 1.9.2 2.7.7zm-3.34 9.25c-.55-1.2-.86-2.54-.86-3.95s.31-2.75.86-3.95a4.001 4.001 0 0 0 0 7.9zM14 4c3.3 0 6 2.7 6 6s-2.7 6-6 6c-1 0-1.9-.2-2.7-.7 1.3-1.3 2.2-3.2 2.2-5.3s-.8-3.9-2.2-5.3C12.1 4.2 13 4 14 4zm-4 1.5C8.8 6.7 8 8.2 8 10s.8 3.3 2 4.4c1.2-1.1 2-2.7 2-4.5s-.8-3.3-2-4.4z"],ring:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 15c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5z"],"rotate-document":["M8.71 6.29A.997.997 0 0 0 8 6H3c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h9c.55 0 1-.45 1-1v-8c0-.28-.11-.53-.29-.71l-4-4zM11 18H4V8h3v3c0 .55.45 1 1 1h3v6zm3-16h-1.59l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-2 2C9.11 2.47 9 2.72 9 3c0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42l-.3-.29H14c1.1 0 2 .9 2 2v3c0 .55.45 1 1 1s1-.45 1-1V6c0-2.21-1.79-4-4-4z"],"rotate-page":["M14 2h-1.59l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-2 2C9.11 2.47 9 2.72 9 3c0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42l-.3-.29H14c1.1 0 2 .9 2 2v3c0 .55.45 1 1 1s1-.45 1-1V6c0-2.21-1.79-4-4-4zm-2 5H3c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-1 11H4V9h7v9z"],satellite:["M9 18c.6 0 1 .4 1 1s-.4 1-1 1c-5 0-9-4-9-9 0-.6.4-1 1-1s1 .4 1 1c0 3.9 3.1 7 7 7zm0-4c.6 0 1 .4 1 1s-.4 1-1 1c-2.8 0-5-2.2-5-5 0-.6.4-1 1-1s1 .4 1 1c0 1.7 1.3 3 3 3zm5.7-3.7c.4-.4 1-.4 1.4 0l3.6 3.6c.4.4.4 1 0 1.4l-1.4 1.4c-.4.4-1 .4-1.4 0l-3.6-3.6c-.4-.4-.4-1 0-1.4l1.4-1.4zM4.7.3c.4-.4 1-.4 1.4 0l3.6 3.6c.4.4.4 1 0 1.4L8.3 6.7c-.4.4-1 .4-1.4 0L3.3 3.1c-.4-.4-.4-1 0-1.4L4.7.3zm11.1 1c.4-.4 1-.4 1.4 0l1.6 1.6c.4.4.4 1 0 1.4l-6.5 6.5c-.4.4-1 .4-1.4 0L9.3 9.2c-.4-.4-.4-1 0-1.4l6.5-6.5zM9 12c-.6 0-1-.4-1-1s.4-1 1-1 1 .4 1 1-.4 1-1 1z"],saved:["M12 0H4c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V6l-6-6zm4 18H5V2h6v5h5v11zm-8.29-6.71a1.003 1.003 0 0 0-1.42 1.42l3 3c.18.18.43.29.71.29.32 0 .59-.16.77-.38l.01.01 4-5-.01-.01c.14-.18.23-.38.23-.62 0-.55-.45-1-1-1-.32 0-.59.16-.77.38l-.01-.01-3.3 4.13-2.21-2.21z"],"scatter-plot":["M9 9c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm5 2c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm4-5c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm1 10H2V3c0-.55-.45-1-1-1s-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zM5 15c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2z"],search:["M19.56 17.44l-4.94-4.94A8.004 8.004 0 0 0 16 8c0-4.42-3.58-8-8-8S0 3.58 0 8s3.58 8 8 8c1.67 0 3.21-.51 4.5-1.38l4.94 4.94a1.498 1.498 0 1 0 2.12-2.12zM8 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6z"],"search-around":["M9.9 6.9a3 3 0 1 0 0 6 3 3 0 1 0 0-6zM3 14c-1.7 0-3 1.3-3 3s1.3 3 3 3 3-1.3 3-3-1.3-3-3-3zm0 5c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zM3 0C1.3 0 0 1.3 0 3s1.3 3 3 3 3-1.3 3-3-1.3-3-3-3zm0 5c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zM17 14c-1.7 0-3 1.3-3 3s1.3 3 3 3 3-1.3 3-3-1.3-3-3-3zm0 5c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zM17 0c-1.7 0-3 1.3-3 3s1.3 3 3 3 3-1.3 3-3-1.3-3-3-3zm0 5c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zM10 10L5 5","M5.379 4.671l5.02 5.02-.707.708-5.02-5.02zM10 10l5-5","M14.621 4.671l.707.708-5.02 5.02-.707-.707z","M10 10l5 5M10.379 9.671l5.02 5.02-.707.708-5.02-5.02z","M10 10l-5 5M9.621 9.671l.707.708-5.02 5.02-.707-.707z"],"search-template":["M13 8H5c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1zm0 3H5c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1zm0-6H5c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1zm6.56 12.44l-3.23-3.23A8.939 8.939 0 0 0 18 9a9 9 0 1 0-9 9c1.94 0 3.74-.62 5.21-1.67l3.23 3.23a1.498 1.498 0 1 0 2.12-2.12zM9 16c-3.87 0-7-3.13-7-7s3.13-7 7-7 7 3.13 7 7-3.13 7-7 7z"],"search-text":["M19.56 17.44l-3.23-3.23A8.939 8.939 0 0 0 18 9a9 9 0 1 0-9 9c1.94 0 3.74-.62 5.21-1.67l3.23 3.23a1.498 1.498 0 1 0 2.12-2.12zM9 16c-3.87 0-7-3.13-7-7s3.13-7 7-7 7 3.13 7 7-3.13 7-7 7zm3.5-11h-7c-.28 0-.5.22-.5.5v2c0 .28.22.5.5.5s.5-.22.5-.5V7h2v6h-.5c-.28 0-.5.22-.5.5s.22.5.5.5h3c.28 0 .5-.22.5-.5s-.22-.5-.5-.5H10V7h2v.5c0 .28.22.5.5.5s.5-.22.5-.5v-2c0-.28-.22-.5-.5-.5z"],"segmented-control":["M19 5H1c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm-1 8h-8V7h8v6z"],select:["M19.71 18.29l-4.25-4.25L20 12.91 9.93 9.33c.04-.1.07-.21.07-.33V3c0-.55-.45-1-1-1H4V1c0-.55-.45-1-1-1S2 .45 2 1v1H1c-.55 0-1 .45-1 1s.45 1 1 1h1v5c0 .55.45 1 1 1h6c.12 0 .23-.03.34-.07L12.91 20l1.14-4.54 4.25 4.25c.17.18.42.29.7.29a1.003 1.003 0 0 0 .71-1.71zM8 8H4V4h4v4z"],selection:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z","M10 6a4 4 0 1 0 0 8 4 4 0 1 0 0-8z"],"send-to":["M19 0h-5c-.6 0-1 .4-1 1s.4 1 1 1h2.6l-4.3 4.3c-.2.2-.3.4-.3.7 0 .6.4 1 1 1 .3 0 .5-.1.7-.3L18 3.4V6c0 .5.5 1 1 1s1-.5 1-1V1c0-.6-.5-1-1-1zm0 9c-1 0-1.9-.5-2.5-1.3l-1.4 1.4c-.5.6-1.3.9-2.1.9-1.7 0-3-1.3-3-3 0-.8.3-1.6.9-2.1l1.4-1.4C11.5 2.9 11 2 11 1c0-.3.1-.6.2-.9-.4-.1-.8-.1-1.2-.1C4.5 0 0 4.5 0 10s4.5 10 10 10 10-4.5 10-10c0-.4 0-.8-.1-1.2-.3.1-.6.2-.9.2z"],"send-to-graph":["M8 11H3c-.55 0-1 .45-1 1s.45 1 1 1h2.59L.3 18.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L7 14.41V17c0 .55.45 1 1 1s1-.45 1-1v-5c0-.55-.45-1-1-1zm10 2c-.53 0-1.01.21-1.37.55L11.9 10.6c.06-.19.1-.39.1-.6 0-.21-.04-.41-.1-.6l4.72-2.95c.37.34.85.55 1.38.55 1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .21.04.41.1.6l-4.73 2.96c-.24-.23-.54-.4-.87-.48V3.93c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2S8 .9 8 2c0 .93.64 1.71 1.5 1.93v4.14c-.33.09-.63.26-.87.48L7.6 7.91 5.42 6.55 3.9 5.6c.06-.19.1-.39.1-.6 0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.53 0 1.01-.21 1.37-.55L9 9.96V10h.06L12 11.84l.4.25 1.51.94 2.19 1.37c-.06.19-.1.39-.1.6 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2zm-7-2.96l-.06-.04H11v.04z"],"send-to-map":["M8 11H3c-.55 0-1 .45-1 1s.45 1 1 1h2.59L.3 18.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71L7 14.41V17c0 .55.45 1 1 1s1-.45 1-1v-5c0-.55-.45-1-1-1zm11.54-6.82l.01-.02-6-4-.01.02C13.39.08 13.21 0 13 0s-.39.08-.54.18l-.01-.02L7 3.8 1.55.17l-.01.01A.969.969 0 0 0 1 0C.45 0 0 .45 0 1v9c0-.55.45-1 1-1h1V2.87l4 2.67V9h2V5.54l4-2.67v11.6l-1 .67v2.4l2-1.33 5.45 3.63.01-.02c.15.1.33.18.54.18.55 0 1-.45 1-1V5c0-.35-.19-.64-.46-.82zM18 17.13l-4-2.67V2.87l4 2.67v11.59z"],"series-add":["M13.29 9.29c.3.62.8 1.12 1.42 1.42l-3 3c-.18.18-.43.29-.71.29s-.53-.11-.71-.3L7 10.41l-5 5V17h17c.55 0 1 .45 1 1s-.45 1-1 1H1a.998.998 0 0 1-1-1V4c0-.55.45-1 1-1s1 .45 1 1v8.59l4.29-4.3C6.47 8.11 6.72 8 7 8s.53.11.71.29l3.29 3.3 2.29-2.3zM12 5c0-.5.4-1 1-1h2V2c0-.6.4-1 1-1 .5 0 1 .4 1 1v2h2c.5 0 1 .4 1 1s-.5 1-1 1h-2v2c0 .6-.5 1-1 1-.6 0-1-.4-1-1V6h-2c-.6 0-1-.4-1-1z"],"series-configuration":["M11.91 10.67c.52.45 1.13.8 1.8 1.03l-2.01 2.01c-.18.18-.43.29-.71.29-.28 0-.53-.11-.71-.3L7 10.41l-5 5V17h16.99c.55 0 1 .45 1 1s-.45 1-1 1H1a.998.998 0 0 1-1-1V4c0-.55.45-1 1-1s1 .45 1 1v8.59l4.29-4.3C6.47 8.11 6.72 8 7 8c.28 0 .53.11.71.29l3.29 3.3.91-.92zM18.5 4.6h1.04c.25 0 .45.2.46.44v.9c0 .25-.2.45-.45.45h-1.04c-.07.22-.16.42-.27.62l.73.73c.17.17.17.44 0 .61l-.61.61c-.17.17-.44.17-.61 0l-.73-.73c-.2.11-.4.2-.62.26v1.05c0 .25-.2.45-.45.45h-.9c-.25 0-.45-.2-.45-.45V8.51c-.21-.06-.4-.15-.58-.25l-.76.77c-.17.17-.46.17-.64 0l-.64-.64a.465.465 0 0 1 0-.64l.76-.77c-.1-.19-.19-.38-.25-.59h-1.04c-.25 0-.45-.2-.45-.45v-.9c0-.25.2-.45.45-.45h1.04c.07-.22.16-.42.27-.61l-.73-.73a.429.429 0 0 1 0-.61l.61-.61c.17-.17.44-.17.61 0l.73.73c.2-.11.4-.2.62-.26V1.45a.44.44 0 0 1 .44-.45h.9c.25 0 .45.2.45.45V2.5c.21.06.4.15.58.25l.76-.77c.17-.17.46-.17.64 0l.64.64c.17.17.17.46 0 .64l-.76.77c.1.17.19.36.25.57zm-4.69.9c0 .93.75 1.69 1.69 1.69.93 0 1.69-.75 1.69-1.69s-.75-1.69-1.69-1.69-1.69.76-1.69 1.69z"],"series-derived":["M18.82 6.58c-.03.05-.07.09-.11.13 0 0 0-.01-.01-.01l-2 2c-.2.2-.4.3-.7.3-.6 0-1-.4-1-1 0-.3.1-.5.3-.7L16.6 6H11c-.6 0-1-.4-1-1s.4-1 1-1h5.6l-1.3-1.3c-.2-.2-.3-.4-.3-.7 0-.6.4-1 1-1 .3 0 .5.1.7.3l3 3c.2.2.3.4.3.7s-.1.5-.3.7l-.88.88zm-5.53 2.71c.3.62.8 1.12 1.42 1.42l-3 3c-.18.18-.43.29-.71.29s-.53-.11-.71-.3L7 10.41l-5 5V17h17c.55 0 1 .45 1 1s-.45 1-1 1H1a.998.998 0 0 1-1-1V4c0-.55.45-1 1-1s1 .45 1 1v8.59l4.29-4.3C6.47 8.11 6.72 8 7 8s.53.11.71.29l3.29 3.3 2.29-2.3z"],"series-filtered":["M12.14 10.45c.21.67.65 1.23 1.22 1.61l-1.65 1.65c-.18.18-.43.29-.71.29s-.53-.11-.71-.3L7 10.41l-5 5V17h17c.55 0 1 .45 1 1s-.45 1-1 1H1a.998.998 0 0 1-1-1V4c0-.55.45-1 1-1s1 .45 1 1v8.59l4.29-4.3C6.47 8.11 6.72 8 7 8s.53.11.71.29l3.29 3.3 1.14-1.14zM19.35 1a.642.642 0 1 1 .46 1.1l-3.03 3.03v2.95c0 .18-.07.34-.19.46l-1.28 1.29c-.11.1-.27.17-.45.17-.35 0-.64-.29-.64-.64V5.13L11.19 2.1a.642.642 0 0 1 .45-1.1h7.71z"],"series-search":["M11.28 11.31l-.28.28-3.29-3.3C7.53 8.11 7.28 8 7 8s-.53.11-.71.29L2 12.59V4c0-.55-.45-1-1-1s-1 .45-1 1v14a.998.998 0 0 0 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1H2v-1.59l5-5 3.29 3.29c.18.19.43.3.71.3s.53-.11.71-.29l2.09-2.09c-.17.02-.34.02-.51.02-.7 0-1.38-.12-2.01-.33zm-.93-6c0-1.62 1.31-2.93 2.93-2.93s2.93 1.31 2.93 2.93-1.31 2.93-2.93 2.93-2.93-1.31-2.93-2.93zm6.47 2.43c.11-.17.21-.33.29-.51.01-.03.03-.06.04-.09.08-.18.16-.35.21-.54.06-.2.1-.38.14-.58.01-.05.01-.09.02-.14.03-.2.05-.39.05-.6 0-2.37-1.93-4.3-4.3-4.3-2.37.01-4.3 1.93-4.3 4.31s1.93 4.3 4.3 4.3c.21 0 .4-.02.6-.05.04 0 .09-.01.14-.02.2-.03.38-.08.57-.14.2-.06.37-.14.55-.21.03-.01.06-.03.09-.04.18-.09.34-.19.51-.29l2.87 2.87c.14.14.33.22.56.22.43 0 .78-.35.78-.78a.938.938 0 0 0-.23-.56l-2.89-2.85z"],settings:["M4 1c0-.55-.45-1-1-1S2 .45 2 1v5h2V1zM2 19c0 .55.45 1 1 1s1-.45 1-1v-6H2v6zm9-18c0-.55-.45-1-1-1S9 .45 9 1v8h2V1zm7 0c0-.55-.45-1-1-1s-1 .45-1 1v3h2V1zM9 19c0 .55.45 1 1 1s1-.45 1-1v-3H9v3zm9-14h-2c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm-2 14c0 .55.45 1 1 1s1-.45 1-1v-8h-2v8zM4 7H2c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm7 3H9c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1z"],share:["M15 18H2V5h8.76l2-2H1c-.55 0-1 .45-1 1v15c0 .55.45 1 1 1h15c.55 0 1-.45 1-1V7.24l-2 2V18zm4-18h-7c-.55 0-1 .45-1 1s.45 1 1 1h4.59l-7.3 7.29a1.003 1.003 0 0 0 1.42 1.42L18 3.41V8c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1z"],shield:["M10 20c6-3.81 9-9.048 9-15.714-2 0-5-1.429-9-4.286-4 2.857-7 4.286-9 4.286C1 10.952 4 16.19 10 20zm0-17.348c2.577 1.734 4.776 2.88 6.667 3.419-.44 4.627-2.636 8.353-6.667 11.297V2.652z"],shop:["M17.94 3.63c-.01-.02-.01-.03-.02-.04l-.03-.09h-.01c-.18-.3-.49-.5-.86-.5h-14c-.42 0-.77.25-.92.61L0 8.5h.02a2.5 2.5 0 0 0 5 0 2.5 2.5 0 0 0 5 0 2.5 2.5 0 0 0 5 0 2.5 2.5 0 0 0 5 0l-2.08-4.87zM3.02 2h14c.55 0 1-.45 1-1s-.45-1-1-1h-14c-.55 0-1 .45-1 1s.44 1 1 1zm13 14h-12v-4h-2v7c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-7h-2v4z"],"shopping-cart":["M18 14H8.72l-.67-2H17c.44 0 .8-.29.94-.69h.01l2-6h-.01c.03-.1.06-.2.06-.31 0-.55-.45-1-1-1H5.39l-.44-1.32h-.01C4.8 2.29 4.44 2 4 2H1c-.55 0-1 .45-1 1s.45 1 1 1h2.28l3.33 10H5c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2h9c0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2zM6.05 6h11.56l-1.33 4H7.39L6.05 6z"],"sim-card":["M16.71 5.29l-5-5A.997.997 0 0 0 11 0H4c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h12c.55 0 1-.45 1-1V6c0-.28-.11-.53-.29-.71zM9 7h2v3H9V7zM6 7h2v3H6V7zm2 11H6v-3h2v3zm3 0H9v-3h2v3zm3 0h-2v-3h2v3zm0-4H6v-3h8v3zm0-4h-2V7h2v3z"],slash:["M12 2c-.46 0-.85.32-.97.74L7.04 16.7c-.02.1-.04.2-.04.3 0 .55.45 1 1 1 .46 0 .85-.32.97-.74L12.96 3.3c.02-.1.04-.2.04-.3 0-.55-.45-1-1-1z"],"small-cross":["M11.41 10l3.29-3.29c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71L10 8.59l-3.29-3.3a1.003 1.003 0 0 0-1.42 1.42L8.59 10 5.3 13.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l3.29-3.3 3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71L11.41 10z"],"small-minus":["M14 9H6c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1z"],"small-plus":["M14 9h-3V6c0-.55-.45-1-1-1s-1 .45-1 1v3H6c-.55 0-1 .45-1 1s.45 1 1 1h3v3c0 .55.45 1 1 1s1-.45 1-1v-3h3c.55 0 1-.45 1-1s-.45-1-1-1z"],"small-tick":["M15 5c-.28 0-.53.11-.71.29L8 11.59l-2.29-2.3a1.003 1.003 0 0 0-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l7-7A1.003 1.003 0 0 0 15 5z"],snowflake:["M11 11.776v2.81l2.31 2.242a.987.987 0 0 1 0 1.415c-.399.39-1.044.39-1.442 0L11 17.414V19a.99.99 0 0 1-.996 1A.996.996 0 0 1 9 19v-1.636l-.912.879c-.398.39-1.043.39-1.441 0a.987.987 0 0 1 0-1.415L9 14.536v-2.79l-2.548 1.435-.837 3.063c-.146.534-.705.85-1.248.707a.998.998 0 0 1-.721-1.224l.309-1.132-1.4.793a1.03 1.03 0 0 1-1.393-.366.99.99 0 0 1 .373-1.366l1.445-.818-1.224-.322a.998.998 0 0 1-.72-1.225c.145-.533.704-.85 1.248-.707l3.193.84 2.462-1.395-2.532-1.434-3.123.82a1.022 1.022 0 0 1-1.249-.706.998.998 0 0 1 .721-1.225L2.91 7.18l-1.4-.793a.99.99 0 0 1-.373-1.366 1.03 1.03 0 0 1 1.392-.366l1.445.818-.328-1.2a.998.998 0 0 1 .72-1.225 1.022 1.022 0 0 1 1.25.707l.855 3.132L9 8.311V5.414L6.647 3.121a.987.987 0 0 1 0-1.414 1.033 1.033 0 0 1 1.441 0L9 2.586V1c0-.552.44-1 1.004-1A.99.99 0 0 1 11 1l-.007 1.536.875-.829a1.033 1.033 0 0 1 1.441 0 .987.987 0 0 1 0 1.414L11 5.364v2.918l2.53-1.42.855-3.131c.146-.534.705-.85 1.249-.707a.998.998 0 0 1 .72 1.224l-.327 1.2 1.4-.792a1.03 1.03 0 0 1 1.392.366.99.99 0 0 1-.373 1.366l-1.355.768 1.153.303a.998.998 0 0 1 .721 1.225c-.146.533-.705.85-1.249.707l-3.123-.821-2.576 1.459 2.506 1.42 3.193-.84a1.022 1.022 0 0 1 1.249.707.998.998 0 0 1-.72 1.225l-1.224.322 1.4.793a.99.99 0 0 1 .373 1.366 1.03 1.03 0 0 1-1.393.366l-1.356-.768.31 1.132a.998.998 0 0 1-.721 1.224 1.022 1.022 0 0 1-1.249-.707l-.837-3.063L11 11.776z"],"social-media":["M11.5 5c.8 0 1.6-.4 2-1 2 1.2 3.3 3.3 3.5 5.7 0 .5.5.9 1 .9.6 0 1-.5 1-1v-.1c-.2-3.3-2.2-6.2-5.1-7.6C13.7.8 12.7 0 11.5 0 10.1 0 9 1.1 9 2.5S10.1 5 11.5 5zm5 7c-1.4 0-2.5 1.1-2.5 2.5 0 .4.1.7.2 1.1-1.1.9-2.6 1.4-4.2 1.4-1.9 0-3.6-.8-4.9-2-.2-.2-.5-.4-.8-.4-.5 0-1 .5-1 1 0 .3.1.5.3.7C5.3 18 7.5 19 10 19c2.2 0 4.2-.8 5.8-2.1.2.1.5.1.7.1 1.4 0 2.5-1.1 2.5-2.5S17.9 12 16.5 12zM5 10.5c0-1.1-.7-2.1-1.7-2.4.5-1.9 1.9-3.5 3.6-4.4.3-.2.6-.5.6-.9 0-.5-.4-1-1-1-.2 0-.4.1-.6.2-2.4 1.2-4.2 3.6-4.7 6.4C.5 8.9 0 9.6 0 10.5 0 11.9 1.1 13 2.5 13S5 11.9 5 10.5z"],sort:["M19 16h-9c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm0-5h-9c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zM7 15c-.28 0-.53.11-.71.29L5 16.59V11c0-.55-.45-1-1-1s-1 .45-1 1v5.59L1.71 15.3A.965.965 0 0 0 1 15a1.003 1.003 0 0 0-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0 0 7 15zM19 1h-9c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zm0 5h-9c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1z"],"sort-alphabetical":["M8 15c-.28 0-.53.11-.71.29L6 16.59v-5.58c0-.55-.45-1-1-1s-1 .45-1 1v5.58L2.71 15.3c-.18-.18-.43-.3-.71-.3a1.003 1.003 0 0 0-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0 0 8 15zm8.89-.79v-1.22H11.3v1.3h3.51L11 18.78V20h5.99v-1.3h-3.91l3.81-4.49zM14.97 0h-1.95L9.01 11.01h1.89l.98-2.92h4.17l.98 2.92h1.96L14.97 0zm-2.59 6.63l1.58-4.74H14l1.57 4.74h-3.19z"],"sort-alphabetical-desc":["M8.01 15c-.28 0-.53.11-.71.29L6 16.59v-5.58c0-.55-.45-1-1-1s-1 .45-1 1v5.58L2.71 15.3c-.18-.18-.43-.3-.71-.3a1.003 1.003 0 0 0-.71 1.71l3 3a1.014 1.014 0 0 0 1.42 0l3-3c.18-.18.29-.43.29-.71.01-.55-.44-1-.99-1zm4.44-5.65l6.4-7.88V0H10.5v1.67h5.91L10 9.44v1.57h9V9.35h-6.55zm1.27 3.64L11 20h1.59l.56-1.56h2.68l.55 1.56h1.64l-2.68-7.01h-1.62zm-.16 4.3l.93-2.57h.02l.9 2.57h-1.85z"],"sort-asc":["M10 8h5c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1h-5c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm0 5h7c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1h-7c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm0-10h3c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1h-3c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm9 12h-9c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zM7 14c-.28 0-.53.11-.71.29L5 15.59V10c0-.55-.45-1-1-1s-1 .45-1 1v5.59L1.71 14.3A.965.965 0 0 0 1 14a1.003 1.003 0 0 0-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0 0 7 14z"],"sort-desc":["M13 15h-3c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h3c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm-6-1c-.28 0-.53.11-.71.29L5 15.59V10c0-.55-.45-1-1-1s-1 .45-1 1v5.59L1.71 14.3A.965.965 0 0 0 1 14a1.003 1.003 0 0 0-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0 0 7 14zM19 0h-9c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-4 10h-5c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h5c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm2-5h-7c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1z"],"sort-numerical":["M9 14.99c-.28 0-.53.11-.71.29L7 16.58v-5.59c0-.55-.45-1-1-1s-1 .45-1 1v5.59l-1.29-1.29a.965.965 0 0 0-.71-.3 1.003 1.003 0 0 0-.71 1.71l3 3c.18.18.43.29.71.29.28 0 .53-.11.71-.29l3-3c.18-.18.29-.43.29-.71a.99.99 0 0 0-1-1zm8.88.23c-.08-.42-.22-.79-.42-1.12-.2-.33-.47-.6-.8-.8-.33-.2-.76-.3-1.28-.3a2.333 2.333 0 0 0-1.72.71c-.21.22-.37.48-.49.78-.11.3-.17.62-.17.97 0 .27.04.54.13.8.08.26.22.5.4.7.19.21.43.38.71.5a2.142 2.142 0 0 0 1.72.02c.25-.12.47-.31.66-.58l.02.02c-.01.19-.04.4-.08.63-.04.24-.11.46-.21.67-.1.21-.23.38-.39.53a.92.92 0 0 1-.62.22c-.24 0-.44-.08-.6-.25-.16-.17-.27-.36-.31-.59h-1.31c.04.29.12.56.24.79.12.23.28.43.48.59.19.16.42.28.67.36.25.08.52.12.82.12.49 0 .9-.1 1.23-.31.34-.21.61-.48.82-.82.21-.34.37-.71.47-1.13.1-.42.15-.83.15-1.25 0-.43-.04-.85-.12-1.26zm-1.42.63c-.05.15-.11.28-.2.4-.09.12-.2.21-.34.27s-.3.1-.49.1c-.17 0-.33-.04-.46-.11s-.24-.17-.33-.29c-.08-.12-.15-.25-.19-.4-.04-.15-.06-.31-.06-.47 0-.15.02-.3.07-.45.05-.15.11-.28.2-.39.09-.12.2-.21.33-.28.13-.07.27-.11.44-.11.17 0 .33.04.47.11.14.07.25.17.34.28a1.387 1.387 0 0 1 .28.86c.01.17-.02.33-.06.48zM15.32 11H17V0h-1.25c-.05.34-.17.62-.34.85-.17.23-.39.42-.63.57-.25.15-.52.25-.83.31-.3.06-.62.09-.94.09v1.41h2.31V11z"],"sort-numerical-desc":["M9 15c-.28 0-.53.11-.71.29L7 16.59v-5.58c0-.55-.45-1-1-1s-1 .45-1 1v5.58L3.71 15.3c-.18-.18-.43-.3-.71-.3a1.003 1.003 0 0 0-.71 1.71l3 3c.18.18.43.29.71.29.28 0 .53-.11.71-.29l3-3A1.003 1.003 0 0 0 9 15zm6.7-1.33a1.5 1.5 0 0 1-.44.43c-.17.11-.37.19-.58.23-.22.04-.44.06-.67.05v1.07h1.66V20H17v-6.99h-1.06c-.04.26-.12.48-.24.66zm3.15-10.3c-.11-.68-.29-1.26-.55-1.76-.26-.5-.62-.89-1.08-1.18C16.75.14 16.17 0 15.46 0c-.54 0-1.03.09-1.46.27-.43.18-.79.44-1.09.76-.3.33-.52.71-.67 1.15-.16.44-.24.92-.24 1.43 0 .54.08 1.04.23 1.47.15.44.37.81.65 1.12.28.31.61.55 1 .72.39.17.82.26 1.3.26.46 0 .88-.11 1.26-.33.38-.22.68-.53.9-.94l.03.03c-.03.35-.07.74-.12 1.16-.05.42-.15.81-.29 1.18-.14.37-.35.68-.61.92-.26.25-.62.37-1.06.37-.43 0-.77-.13-1.03-.4-.25-.27-.4-.62-.44-1.05h-1.64c.02.43.11.83.29 1.18.17.35.39.66.67.91a3.027 3.027 0 0 0 2.07.8c.71 0 1.3-.17 1.79-.5.48-.33.87-.76 1.17-1.29.3-.53.51-1.12.64-1.76.13-.64.19-1.28.19-1.92.01-.77-.05-1.49-.15-2.17zM17.1 4.44c-.08.27-.19.5-.34.71-.15.21-.34.37-.57.49-.23.12-.5.18-.8.18-.3 0-.56-.06-.78-.19-.22-.13-.4-.29-.55-.49-.14-.2-.25-.44-.32-.7-.07-.27-.11-.55-.11-.84 0-.28.04-.55.11-.82.07-.26.18-.49.32-.7.14-.2.33-.36.55-.48.22-.12.48-.17.78-.17.31 0 .57.06.8.18.23.12.42.28.57.48.15.2.26.43.34.69.08.26.11.53.11.82 0 .29-.04.57-.11.84z"],"split-columns":["M15 13a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3a1.003 1.003 0 0 0-1.42 1.42L16.59 9H11V2h5v2c.77 0 1.47.3 2 .78V1c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v3.78C2.53 4.3 3.23 4 4 4V2h5v7H3.41L4.7 7.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-3 3C.11 9.47 0 9.72 0 10c0 .28.11.53.29.71l3 3a1.003 1.003 0 0 0 1.42-1.42L3.41 11H9v7H4v-2c-.77 0-1.47-.3-2-.78V19c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-3.78c-.53.48-1.23.78-2 .78v2h-5v-7h5.59l-1.29 1.29c-.19.18-.3.43-.3.71z"],square:["M19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H2V2h16v16z"],"stacked-chart":["M12 2c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v4h4V2zm3 14h2c.55 0 1-.45 1-1v-5h-4v5c0 .55.45 1 1 1zm3-10c0-.55-.45-1-1-1h-2c-.55 0-1 .45-1 1v3h4V6zm-6 1H8v5h4V7zm-9 9h2c.55 0 1-.45 1-1v-3H2v3c0 .55.45 1 1 1zm16 1H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zM6 9c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v2h4V9zm3 7h2c.55 0 1-.45 1-1v-2H8v2c0 .55.45 1 1 1z"],star:["M10 0l3.1 6.6 6.9 1-5 5.1 1.2 7.3-6.2-3.4L3.8 20 5 12.7 0 7.6l6.9-1z"],"star-empty":["M20 7.6l-6.9-1.1L10 0 6.9 6.6 0 7.6l5 5.1L3.8 20l6.2-3.4 6.2 3.4-1.2-7.2 5-5.2zM10 15l-4.5 2.4.9-5.2-3.6-3.6 5-.8L10 3.1l2.2 4.7 5 .8-3.6 3.7.9 5.2L10 15z"],"step-backward":["M15 3c-.23 0-.42.09-.59.21l-.01-.01L8 8V4c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-4l6.4 4.8.01-.01c.17.12.36.21.59.21.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"step-chart":["M19 16H2v-3h4c.55 0 1-.45 1-1V8h3v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V6h2c.55 0 1-.45 1-1s-.45-1-1-1h-3c-.55 0-1 .45-1 1v4h-3V7c0-.55-.45-1-1-1H6c-.55 0-1 .45-1 1v4H2V3c0-.55-.45-1-1-1s-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],"step-forward":["M15 3h-2c-.55 0-1 .45-1 1v4L5.6 3.2l-.01.01C5.42 3.09 5.23 3 5 3c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1 .23 0 .42-.09.59-.21l.01.01L12 12v4c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],stop:["M16 3H4c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h12c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],strikethrough:["M18 9h-4.46a4.7 4.7 0 0 0-.4-.14c-.19-.05-.51-.14-.96-.25-.45-.11-.9-.23-1.37-.35-.47-.12-.89-.23-1.27-.33s-.6-.16-.65-.17c-.53-.15-.95-.37-1.27-.66-.32-.28-.49-.68-.49-1.19 0-.36.09-.66.26-.9s.39-.43.65-.57c.26-.14.55-.24.87-.3s.63-.09.93-.09c.89 0 1.63.19 2.21.57.45.3.75.76.89 1.38h2.63c-.06-.52-.2-.98-.42-1.4-.3-.57-.71-1.05-1.23-1.43a5.33 5.33 0 0 0-1.79-.87c-.7-.2-1.42-.3-2.19-.3-.66 0-1.31.08-1.96.25s-1.22.43-1.73.77-.92.79-1.23 1.32c-.31.52-.46 1.15-.46 1.87 0 .37.05.74.15 1.1.1.36.28.7.53 1.02.18.24.41.47.69.67H2c-.55 0-1 .45-1 1s.45 1 1 1h10.14c.02.01.05.02.07.02.3.11.58.29.84.55.25.26.38.67.38 1.21 0 .27-.06.53-.17.79-.11.26-.29.49-.54.69-.25.2-.57.36-.97.49s-.88.19-1.44.19c-.52 0-1.01-.06-1.45-.17-.45-.11-.84-.29-1.19-.54s-.61-.56-.8-.95c-.05-.08-.09-.18-.12-.28H4.11c.09.43.22.82.4 1.18.33.65.77 1.18 1.32 1.59.55.41 1.2.72 1.94.92.74.2 1.53.3 2.37.3.73 0 1.44-.08 2.14-.25.7-.17 1.33-.43 1.88-.79.55-.36.99-.83 1.33-1.39.34-.56.51-1.25.51-2.05 0-.37-.06-.75-.18-1.12a3.12 3.12 0 0 0-.15-.39H18c.55 0 1-.45 1-1s-.45-1-1-1z"],style:["M18 18H2V2h12.3l2-2H1C.4 0 0 .4 0 1v18c0 .6.4 1 1 1h18c.6 0 1-.4 1-1V7.7l-2 2V18zm1.2-18l-7.6 7.6 2.8 2.8L20 4.8V0h-.8zM4 15.9c3.1.2 5.9.2 8.2-2 1.1-1.1 1.1-3 0-4.1-.6-.5-1.3-.8-2-.8s-1.4.3-1.9.8C7.2 11 6.6 14.3 4 15.9z"],"swap-horizontal":["M16.02 10c-.01 0-.01 0 0 0H16h.02zM2 6h13.58l-2.29 2.29a1 1 0 0 0-.3.71 1.003 1.003 0 0 0 1.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-4-4a1.003 1.003 0 0 0-1.42 1.42L15.58 4H2c-.55 0-1 .45-1 1s.45 1 1 1zm2 4h-.02H4zm14 4H4.42l2.29-2.29a1 1 0 0 0 .3-.71 1.003 1.003 0 0 0-1.71-.71l-4 4c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l4 4a1.003 1.003 0 0 0 1.42-1.42L4.42 16H18c.55 0 1-.45 1-1s-.45-1-1-1z"],"swap-vertical":["M9.71 5.3l-4-4A.997.997 0 0 0 5 1.01c-.28 0-.53.11-.71.29l-4 4a1.003 1.003 0 0 0 1.42 1.42L4 4.42V18c0 .55.45 1 1 1s1-.45 1-1V4.42l2.29 2.29a1 1 0 0 0 .71.3 1.003 1.003 0 0 0 .71-1.71zM10 3.98c0 .01 0 .01 0 0V4v-.02zm0 12.04c0-.01 0-.01 0 0V16v.02zm9-3.03c-.28 0-.53.11-.71.29L16 15.58V2c0-.55-.45-1-1-1s-1 .45-1 1v13.58l-2.29-2.29a1.003 1.003 0 0 0-1.42 1.42l4 4c.18.18.43.29.71.29.28 0 .53-.11.71-.29l4-4c.18-.18.29-.43.29-.71 0-.56-.45-1.01-1-1.01z"],"symbol-circle":["M10 4.01a6 6 0 1 0 0 12 6 6 0 1 0 0-12z"],"symbol-cross":["M15 8.01h-3v-3c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v3H5c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h3v3c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-3h3c.55 0 1-.45 1-1v-2c0-.56-.45-1-1-1z"],"symbol-diamond":["M15 10.01c0-.21-.08-.39-.18-.54l.02-.01-4-6-.02.01c-.18-.28-.47-.46-.82-.46s-.64.18-.82.45l-.01-.01-4 6 .02.01c-.11.16-.19.34-.19.55s.08.39.18.54l-.02.01 4 6 .02-.01c.18.27.47.46.82.46s.64-.19.82-.46l.02.01 4-6-.02-.01c.1-.16.18-.34.18-.54z"],"symbol-square":["M15 4.01H5c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-10c0-.56-.45-1-1-1z"],"symbol-triangle-down":["M16 5c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1 0 .16.05.31.11.44H4.1l5 10h.01c.17.33.5.56.89.56s.72-.23.89-.56h.01l5-10h-.01c.06-.13.11-.28.11-.44z"],"symbol-triangle-up":["M15.89 14.56l-4.99-10h-.01c-.17-.33-.5-.56-.89-.56s-.72.23-.89.56H9.1l-5 10h.01c-.06.13-.11.28-.11.44 0 .55.45 1 1 1h10c.55 0 1-.45 1-1 0-.16-.05-.31-.11-.44z"],tag:["M2 4a2 2 0 0 1 2-2h4.588a2 2 0 0 1 1.414.586l7.41 7.41a2 2 0 0 1 0 2.828l-4.588 4.588a2 2 0 0 1-2.829 0l-7.41-7.41A2 2 0 0 1 2 8.588V4zm3.489-.006a1.495 1.495 0 1 0 0 2.99 1.495 1.495 0 0 0 0-2.99z"],"take-action":["M5 7c.28 0 .53-.11.71-.29l5-5A1.003 1.003 0 0 0 9.29.29l-5 5A1.003 1.003 0 0 0 5 7zm6 6a1.003 1.003 0 0 0 1.71.71l5-5a1.003 1.003 0 0 0-1.42-1.42l-5 5c-.18.18-.29.43-.29.71zm8 5h-1c0-.55-.45-1-1-1h-7c-.55 0-1 .45-1 1H8c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1zm-9-6l6-6-1.29-1.29a1.003 1.003 0 0 0-1.42-1.42L12 2 6 8l1.29 1.29-7 7a1.003 1.003 0 0 0 1.42 1.42l7-7L10 12z"],taxi:["M19 9h-.33l.33 1v.5c0 .15-.03.3-.07.44h.01L17 17.23v.27c0 .83-.67 1.5-1.5 1.5s-1.5-.67-1.5-1.5V17H6v.5c0 .83-.67 1.5-1.5 1.5S3 18.33 3 17.5v-.27l-1.93-6.28h.01c-.05-.15-.08-.3-.08-.45V10s.02-.06.05-.16c.06-.17.16-.47.28-.84H1c-.55 0-1-.45-1-1s.45-1 1-1h1l1-3h-.01v-.01c.25-.64 1-1.31 1.67-1.5 0 0 .78-.21 2.33-.36V1c0-.55.45-1 1-1h4c.55 0 1 .45 1 1v1.13c1.55.14 2.33.36 2.33.36.67.19 1.42.86 1.67 1.5V4H17l1 3h1c.55 0 1 .45 1 1s-.45 1-1 1zM3 11.5c0 .83.67 1.5 1.5 1.5S6 12.33 6 11.5 5.33 10 4.5 10 3 10.67 3 11.5zM16 7l-1-3H5L4 7v1h12V7zm-.5 3c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5z"],"text-highlight":["M16 17c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1s1-.45 1-1-.45-1-1-1c-.77 0-1.47.3-2 .78-.53-.48-1.23-.78-2-.78-.55 0-1 .45-1 1s.45 1 1 1 1 .45 1 1v12c0 .55-.45 1-1 1s-1 .45-1 1 .45 1 1 1c.77 0 1.47-.3 2-.78.53.48 1.23.78 2 .78.55 0 1-.45 1-1s-.45-1-1-1zm-4-4H2V7h10V5H1c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h11v-2zm7-8h-3v2h2v6h-2v2h3c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1z"],th:["M19 1H1c-.6 0-1 .5-1 1v16c0 .5.4 1 1 1h18c.5 0 1-.5 1-1V2c0-.5-.5-1-1-1zM7 17H2v-3h5v3zm0-4H2v-3h5v3zm0-4H2V6h5v3zm11 8H8v-3h10v3zm0-4H8v-3h10v3zm0-4H8V6h10v3z"],"th-derived":["M5.3 13.3c-.2.2-.3.4-.3.7 0 .6.4 1 1 1 .3 0 .5-.1.7-.3l3-3c.2-.2.3-.4.3-.7s-.1-.5-.3-.7l-3-3C6.5 7.1 6.3 7 6 7c-.6 0-1 .4-1 1 0 .3.1.5.3.7L6.6 10H1c-.6 0-1 .4-1 1s.4 1 1 1h5.6l-1.3 1.3zM19 1H3c-.5 0-1 .5-1 1v6h1c0-1.7 1.3-3 3-3 .8 0 1.6.3 2.1.9l.1.1H9v.8l1 1V6h8v3h-6.8c.3.3.5.6.6 1H18v3h-6.8l-.1.1-.9.9H18v3h-8v-2.8l-1 1V17H4v-.8c-.6-.5-1-1.3-1-2.2H2v4c0 .5.5 1 1 1h16c.6 0 1-.5 1-1V2c0-.5-.5-1-1-1z"],"th-disconnect":["M14.25 1H19c.5 0 1 .5 1 1v16c0 .5-.5 1-1 1h-7.221l.278-2H18v-3h-5.527l.14-1H18v-3h-4.971l.139-1H18V6h-4.416l.637-4.587c.02-.139.03-.277.03-.413zM8.221 1l-.694 5H2v3h5.11l-.139 1H2v3h4.555l-.14 1H2v3h3.999l-.22 1.587c-.02.139-.03.277-.03.413H1c-.6 0-1-.5-1-1V2c0-.5.4-1 1-1h7.221zM10.26.862a1 1 0 0 1 1.98.276l-2.5 18a1 1 0 0 1-1.98-.276l2.5-18z"],"th-filtered":["M17.333 10l1.435-1.722a1 1 0 0 0 .232-.64V4.85l1-.9V18c0 .5-.5 1-1 1H1c-.6 0-1-.5-1-1V2c0-.5.4-1 1-1h6.722L12 4.85V6H8v3h4v1H8v3h10v-3h-.667zM7 17v-3H2v3h5zm0-4v-3H2v3h5zm0-4V6H2v3h5zm11 8v-3H8v3h10z","M19.35 0a.642.642 0 1 1 .46 1.1l-3.03 3.03v2.95c0 .18-.07.34-.19.46l-1.28 1.29c-.11.1-.27.17-.45.17-.35 0-.64-.29-.64-.64V4.13L11.19 1.1a.642.642 0 0 1 .45-1.1h7.71z"],"th-list":["M19 1H1c-.6 0-1 .5-1 1v16c0 .5.4 1 1 1h18c.5 0 1-.5 1-1V2c0-.5-.5-1-1-1zm-1 16H2v-3h16v3zm0-4H2v-3h16v3zm0-4H2V6h16v3z"],"thumbs-down":["M18.55 6.56c-.31-.01-.65-.03-1.02-.06.03 0 .06-.01.09-.01.88-.12 1.68-.63 1.76-1.37.08-.75-.58-1.25-1.46-1.33-.32-.03-.65-.05-.99-.08.59-.19 1.05-.54 1.09-1.2.05-.75-.99-1.32-1.87-1.41-.34-.03-.64-.05-.91-.07h-.11c-.28-.02-.54-.02-.77-.02-3.92-.08-7.29.6-9.36 1.93v7.72c2.67 1.66 5.95 4.61 5.26 7.08-.21.76.39 1.35 1.23 1.26 1.01-.11 1.71-1.18 1.75-2.28.05-1.29-.19-2.59-.62-3.74-.05-.32.01-.65.47-.68.61-.04 1.39-.08 1.99-.1.32 0 .64-.01.94-.03h.01c.52-.03 1-.07 1.42-.12.88-.11 1.69-.6 1.79-1.35.1-.75-.55-1.25-1.44-1.35-.07-.01-.13-.02-.2-.02.21-.02.42-.04.61-.06.88-.11 1.69-.6 1.79-1.35.09-.75-.56-1.31-1.45-1.36zM3 3H0v8h3c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"thumbs-up":["M3 9H0v8h3c.55 0 1-.45 1-1v-6c0-.55-.45-1-1-1zm16.99 3.09c-.1-.75-.91-1.24-1.79-1.35-.19-.02-.4-.05-.61-.06.07-.01.14-.01.2-.02.88-.1 1.53-.61 1.44-1.35-.1-.74-.91-1.24-1.79-1.35-.42-.05-.9-.09-1.42-.12h-.01l-.94-.03c-.6-.02-1.39-.05-1.99-.1-.45-.03-.51-.36-.47-.68.43-1.15.67-2.45.62-3.74-.04-1.11-.74-2.17-1.75-2.28-.84-.09-1.45.5-1.23 1.26.7 2.47-2.58 5.43-5.25 7.08v7.72c2.08 1.33 5.44 2.01 9.35 1.93.24 0 .49-.01.77-.02h.11c.27-.02.57-.04.91-.07.88-.08 1.92-.66 1.87-1.41-.04-.65-.5-1.01-1.09-1.2.34-.03.67-.05.99-.08.89-.08 1.55-.58 1.46-1.33-.08-.75-.88-1.25-1.76-1.37-.03 0-.06-.01-.09-.01.37-.02.71-.04 1.02-.06.91-.05 1.55-.61 1.45-1.36z"],tick:["M17 4c-.28 0-.53.11-.71.29L7 13.59 3.71 10.3A.965.965 0 0 0 3 10a1.003 1.003 0 0 0-.71 1.71l4 4c.18.18.43.29.71.29s.53-.11.71-.29l10-10A1.003 1.003 0 0 0 17 4z"],"tick-circle":["M10 20C4.48 20 0 15.52 0 10S4.48 0 10 0s10 4.48 10 10-4.48 10-10 10zm5-14c-.28 0-.53.11-.71.29L8 12.59l-2.29-2.3a1.003 1.003 0 0 0-1.42 1.42l3 3c.18.18.43.29.71.29.28 0 .53-.11.71-.29l7-7A1.003 1.003 0 0 0 15 6z"],time:["M11 9.59V4c0-.55-.45-1-1-1s-1 .45-1 1v6c0 .28.11.53.29.71l3 3a1.003 1.003 0 0 0 1.42-1.42L11 9.59zM10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"],"timeline-area-chart":["M19 16H2V3c0-.55-.45-1-1-1s-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm0-13.41l-7.07 7.07-4.3-3.44-.01.01A.987.987 0 0 0 7 6c-.24 0-.46.1-.63.24l-.01-.01L3 9.03V15h16V2.59z"],"timeline-bar-chart":["M19 17H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zM9 16h2c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v13c0 .55.45 1 1 1zm6 0h2c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1h-2c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1zM3 16h2c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1z"],"timeline-events":["M5 5c.6 0 1-.4 1-1V2c0-.5-.4-1-1-1s-1 .5-1 1v2c0 .6.4 1 1 1zm10 0c.6 0 1-.4 1-1V2c0-.5-.4-1-1-1s-1 .5-1 1v2c0 .6.4 1 1 1zm-9 9H4v2h2v-2zM17 3v1c0 1.1-.9 2-2 2s-2-.9-2-2V3H7v1c0 1.1-.9 2-2 2s-2-.9-2-2V3H2c-.5 0-1 .5-1 1v14c0 .5.5 1 1 1h16c.5 0 1-.5 1-1V4c0-.5-.5-1-1-1h-1zM7 17H3v-4h4v4zm0-5H3V8h4v4zm5 5H8v-4h4v4zm0-5H8V8h4v4zm5 5h-4v-4h4v4zm0-5h-4V8h4v4zm-6 2H9v2h2v-2zm5-5h-2v2h2V9z"],"timeline-line-chart":["M19 16H2v-1.59l5-5 3.29 3.29c.18.19.43.3.71.3s.53-.11.71-.29l7-7a1.003 1.003 0 0 0-1.42-1.42L11 10.59l-3.29-3.3C7.53 7.11 7.28 7 7 7s-.53.11-.71.29L2 11.59V3c0-.55-.45-1-1-1s-1 .45-1 1v14a.998.998 0 0 0 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],tint:["M9.86 2S3.98 9.18 3.98 12.17C3.99 15.4 6.78 18 9.96 18c3.18-.01 6.04-2.63 6.03-5.86C15.99 9.05 9.86 2 9.86 2z"],torch:["M6.97 19c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-2h-6v2zm-3-15l3 4v8h6V8l3-4h-12zm5 5c0-.55.45-1 1-1s1 .45 1 1v2c0 .55-.45 1-1 1s-1-.45-1-1V9zm6-9h-10c-.55 0-1 .45-1 1v2h12V1c0-.55-.45-1-1-1z"],train:["M16 18h-2l2 2H4l.12-.12L6 18H4c-1.1 0-2-.9-2-2V2c0-1.1 3.58-2 8-2s8 .9 8 2v14c0 1.1-.9 2-2 2zM5.5 15c.83 0 1.5-.67 1.5-1.5S6.33 12 5.5 12 4 12.67 4 13.5 4.67 15 5.5 15zM9 3H4v6h5V3zm7 0h-5v6h5V3zm-1.5 9c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5z"],translate:["M19.89 18.56l-4.99-10h-.01c-.17-.33-.5-.56-.89-.56s-.72.23-.89.56h-.01l-1.73 3.46-2.8-2.3 1.99-1.64C11.44 7.34 12 6.23 12 5V4h1c.55 0 1-.45 1-1s-.45-1-1-1H8V1c0-.55-.45-1-1-1S6 .45 6 1v1H1c-.55 0-1 .45-1 1s.45 1 1 1h9v1c0 .62-.28 1.18-.73 1.54L7 8.42 4.73 6.54C4.28 6.18 4 5.62 4 5H2c0 1.23.56 2.34 1.44 3.07l1.99 1.64-3.06 2.52.01.01c-.23.18-.38.45-.38.76 0 .55.45 1 1 1 .24 0 .45-.1.63-.24l.01.01L7 11l3.36 2.77.01-.01c.02.02.05.03.08.05.01 0 .01.01.02.02l-2.36 4.73h.01c-.07.13-.12.28-.12.44 0 .55.45 1 1 1 .39 0 .72-.23.89-.56h.01L11.12 17h5.76l1.22 2.45h.01c.17.32.5.55.89.55.55 0 1-.45 1-1 0-.16-.05-.31-.11-.44zM12.12 15L14 11.24 15.88 15h-3.76z"],trash:["M17 1h-5c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1H3c-.55 0-1 .45-1 1v1h16V2c0-.55-.45-1-1-1zm.5 3h-15c-.28 0-.5.22-.5.5s.22.5.5.5H3v14c0 .55.45 1 1 1h12c.55 0 1-.45 1-1V5h.5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5zM7 16c0 .55-.45 1-1 1s-1-.45-1-1V8c0-.55.45-1 1-1s1 .45 1 1v8zm4 0c0 .55-.45 1-1 1s-1-.45-1-1V8c0-.55.45-1 1-1s1 .45 1 1v8zm4 0c0 .55-.45 1-1 1s-1-.45-1-1V8c0-.55.45-1 1-1s1 .45 1 1v8z"],tree:["M11 15.542V20H9v-4.458L2 17l4.5-5.625L4 12l3.655-5.483L6 7l4-7 4 7-1.655-.483L16 12l-2.5-.625L18 17l-7-1.458z"],"trending-down":["M19 10c-.55 0-1 .45-1 1v1.37l-6.25-7.03-.01.01A.971.971 0 0 0 11 5c-.23 0-.42.09-.59.21l-.01-.01-3.43 2.58-5.42-3.61-.01.01A.969.969 0 0 0 1 4c-.55 0-1 .45-1 1 0 .35.19.64.46.82l-.01.01 6 4 .01-.02c.15.11.33.19.54.19.23 0 .42-.09.59-.21l.01.01 3.26-2.45L16.77 14H15c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1z"],"trending-up":["M19 4h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.77l-5.91 6.65L7.6 10.2l-.01.01C7.42 10.09 7.23 10 7 10c-.21 0-.39.08-.54.18l-.01-.02-6 4 .01.02c-.27.18-.46.47-.46.82 0 .55.45 1 1 1 .21 0 .39-.08.54-.18l.01.02 5.41-3.61 3.43 2.58.01-.01c.18.11.37.2.6.2.3 0 .56-.14.74-.34l.01.01L18 7.63V9c0 .55.45 1 1 1s1-.45 1-1V5c0-.55-.45-1-1-1z"],"two-columns":["M5 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm14.71 9.29l-3-3A1.003 1.003 0 0 0 15 7v6a1.003 1.003 0 0 0 1.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zM12 0H8c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],underline:["M10 17c3.3 0 6-2.7 6-6V3.5c0-.8-.7-1.5-1.5-1.5S13 2.7 13 3.5V11c0 1.7-1.3 3-3 3s-3-1.3-3-3V3.5C7 2.7 6.3 2 5.5 2S4 2.7 4 3.5V11c0 3.3 2.7 6 6 6zM16.5 19h-13c-.3 0-.5.2-.5.5s.2.5.5.5h13c.3 0 .5-.2.5-.5s-.2-.5-.5-.5z"],undo:["M5 14c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm9-9H3.41L5.7 2.71c.19-.18.3-.43.3-.71a1.003 1.003 0 0 0-1.71-.71l-4 4C.11 5.47 0 5.72 0 6c0 .28.11.53.29.71l4 4a1.003 1.003 0 0 0 1.42-1.42L3.41 7H14c2.21 0 4 1.79 4 4s-1.79 4-4 4H9v2h5c3.31 0 6-2.69 6-6s-2.69-6-6-6z"],"ungroup-objects":["M4.5 6C2.01 6 0 8.01 0 10.5S2.01 15 4.5 15 9 12.99 9 10.5 6.99 6 4.5 6zm11 0C13.01 6 11 8.01 11 10.5s2.01 4.5 4.5 4.5 4.5-2.01 4.5-4.5S17.99 6 15.5 6z"],"unknown-vehicle":["M13 11.988v-4H4v-1l1-3h6V2.003a35.867 35.867 0 0 0-1-.015c-3.593 0-5.332.488-5.332.488-.67.188-1.424.864-1.674 1.503l-.004.009H3l-1 3H1a1 1 0 1 0 0 2h.333l-.28.84-.053.16v7.5a1.5 1.5 0 1 0 3 0v-.5h12v.5a1.5 1.5 0 1 0 3 0v-4.5h-5a1 1 0 0 1-1-1zm-8.5 1a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zM19.83 2.782a2.392 2.392 0 0 0-.592-.853c-.276-.264-.64-.485-1.09-.663C17.695 1.09 17.132 1 16.457 1c-.523 0-.996.084-1.418.253a3.157 3.157 0 0 0-1.084.703c-.299.3-.532.656-.698 1.065-.166.41-.254.861-.264 1.353h2.096c0-.246.028-.476.085-.69.057-.214.145-.4.264-.56.119-.16.27-.287.456-.383.185-.095.406-.143.663-.143.38 0 .677.1.89.3.215.2.321.51.321.93.01.245-.035.45-.135.614-.1.164-.23.314-.392.45a8.598 8.598 0 0 1-.527.41 3.53 3.53 0 0 0-.542.485c-.171.187-.32.412-.45.676-.127.265-.206.592-.234.984v.614h1.924v-.519c.038-.273.13-.5.278-.683.147-.182.316-.343.506-.484a13.5 13.5 0 0 1 .606-.424c.214-.14.408-.312.584-.512s.323-.442.442-.724.178-.642.178-1.079c0-.264-.059-.548-.178-.854zm-4.54 6.099v2.103h2.237V8.881H15.29z"],unlock:["M14 1c-2.21 0-4 1.79-4 4v4H2c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h12c.55 0 1-.45 1-1v-8c0-.55-.45-1-1-1h-2V5c0-1.1.9-2 2-2s2 .9 2 2v2c0 .55.45 1 1 1s1-.45 1-1V5c0-2.21-1.79-4-4-4z"],unpin:["M11.77 1.16c-.81.81-.74 2.28.02 3.76L6.1 8.71c-2.17-1.46-4.12-2-4.94-1.18l4.95 4.95-2.12 3.54 3.54-2.12 4.95 4.95c.82-.82.27-2.77-1.19-4.94l3.8-5.69c1.47.76 2.94.84 3.76.02l-7.08-7.08z"],unresolve:["M11.47 12.46c.16-.36.29-.74.38-1.14 0-.02.01-.04.01-.06.09-.4.14-.82.14-1.26 0-.44-.05-.86-.14-1.27 0-.02-.01-.04-.01-.06-.09-.4-.22-.78-.38-1.14-.01-.02-.02-.03-.02-.05a5.94 5.94 0 0 0-.61-1.03c0-.01-.01-.01-.01-.02a6.308 6.308 0 0 0-2.1-1.77c-.19-.1-.39-.18-.59-.26-.03-.01-.06-.02-.1-.03-.17-.07-.34-.12-.52-.17-.05-.01-.1-.03-.15-.04a4.34 4.34 0 0 0-.52-.09c-.05-.01-.11-.02-.17-.03C6.46 4.02 6.23 4 6 4c-3.31 0-6 2.69-6 6s2.69 6 6 6c.23 0 .46-.02.68-.04l.17-.03c.17-.02.34-.06.51-.09.05-.01.1-.03.15-.04.18-.05.36-.1.53-.17l.09-.03a5.973 5.973 0 0 0 2.68-2.04c0-.01.01-.01.01-.02.24-.32.44-.66.61-1.03.02-.01.03-.03.04-.05zM14 4c-.99 0-1.91.24-2.73.66a7.51 7.51 0 0 1 0 10.68c.82.42 1.74.66 2.73.66 3.31 0 6-2.69 6-6s-2.69-6-6-6z"],updated:["M10 0C6.71 0 3.82 1.6 2 4.05V2c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H3.76C5.22 3.17 7.47 2 10 2c4.42 0 8 3.58 8 8s-3.58 8-8 8-8-3.58-8-8c0-.55-.45-1-1-1s-1 .45-1 1c0 5.52 4.48 10 10 10s10-4.48 10-10S15.52 0 10 0zm4 7c-.28 0-.53.11-.71.29L9 11.58 6.71 9.29a1.003 1.003 0 0 0-1.42 1.42l3 3c.18.18.43.29.71.29.28 0 .53-.11.71-.29l5-5A1.003 1.003 0 0 0 14 7z"],upload:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm4 10c-.28 0-.53-.11-.71-.29L11 7.41V15c0 .55-.45 1-1 1s-1-.45-1-1V7.41l-2.29 2.3a1.003 1.003 0 0 1-1.42-1.42l4-4c.18-.18.43-.29.71-.29s.53.11.71.29l4 4A1.003 1.003 0 0 1 14 10z"],user:["M10 0C4.48 0 0 4.48 0 10c0 .33.02.65.05.97.01.12.03.23.05.35.03.2.05.4.09.59.03.14.06.28.1.42l.12.48c.05.16.1.31.15.46.05.13.09.27.15.4.06.16.13.32.21.48.05.11.1.22.16.33.09.17.17.34.27.5.05.09.1.17.15.25.11.18.22.35.34.52.04.06.08.11.12.17 1.19 1.62 2.85 2.86 4.78 3.53l.09.03c.46.15.93.27 1.42.36.08.01.17.03.25.04.49.07.99.12 1.5.12s1.01-.05 1.5-.12c.08-.01.17-.02.25-.04.49-.09.96-.21 1.42-.36l.09-.03c1.93-.67 3.59-1.91 4.78-3.53.04-.05.08-.1.12-.16.12-.17.23-.35.34-.53.05-.08.1-.16.15-.25.1-.17.19-.34.27-.51.05-.11.1-.21.15-.32.07-.16.14-.32.21-.49.05-.13.1-.26.14-.39.05-.15.11-.31.15-.46.05-.16.08-.32.12-.48.03-.14.07-.28.1-.42.04-.19.06-.39.09-.59.02-.12.04-.23.05-.35.05-.32.07-.64.07-.97 0-5.52-4.48-10-10-10zm0 18a7.94 7.94 0 0 1-6.15-2.89c.84-.44 1.86-.82 2.67-1.19 1.45-.65 1.3-1.05 1.35-1.59.01-.07.01-.14.01-.21-.51-.45-.93-1.08-1.2-1.8l-.01-.01c0-.01-.01-.02-.01-.03a4.42 4.42 0 0 1-.15-.48c-.33-.07-.53-.44-.61-.79-.08-.14-.23-.48-.2-.87.05-.51.26-.74.49-.83v-.08c0-.63.06-1.55.17-2.15.02-.17.06-.33.11-.5.21-.73.66-1.4 1.26-1.86.62-.47 1.5-.72 2.28-.72.78 0 1.65.25 2.27.73.6.46 1.05 1.12 1.26 1.86.05.16.08.33.11.5.11.6.17 1.51.17 2.15v.09c.22.1.42.33.46.82.04.39-.12.73-.2.87-.07.34-.27.71-.6.78-.04.16-.09.33-.15.48 0 .01-.02.05-.02.05-.26.71-.67 1.33-1.17 1.78 0 .08.01.16.01.23.05.54-.15.94 1.31 1.59.81.36 1.84.74 2.68 1.19A7.958 7.958 0 0 1 10 18z"],variable:["M4.93 3.79a9.1 9.1 0 0 1 2.2-2.27L7.29 1c-1.38.59-2.57 1.33-3.55 2.22C2.46 4.39 1.49 5.72.83 7.23.28 8.51 0 9.81 0 11.12c0 2.28.83 4.57 2.49 6.86l.16-.55c-.49-1.23-.73-2.38-.73-3.44 0-1.67.28-3.46.84-5.36.55-1.9 1.28-3.51 2.17-4.84zm9.38 8.39l-.33-.2c-.37.54-.65.87-.82 1a.74.74 0 0 1-.42.12c-.19 0-.38-.12-.57-.37-.31-.42-.73-1.59-1.26-3.5.47-.85.86-1.41 1.19-1.67.23-.19.48-.29.74-.29.1 0 .28.04.53.11.26.07.48.11.68.11.27 0 .5-.1.68-.29.18-.19.27-.44.27-.75 0-.33-.09-.58-.27-.77-.18-.19-.44-.29-.78-.29-.3 0-.59.07-.86.22s-.61.47-1.02.97c-.31.37-.77 1.02-1.37 1.94a9.683 9.683 0 0 0-1.24-3.14l-3.24.59-.06.36c.24-.05.44-.07.61-.07.32 0 .59.14.8.43.33.45.8 1.8 1.39 4.07-.47.64-.78 1.06-.96 1.26-.28.32-.52.53-.7.62-.14.08-.3.11-.48.11-.14 0-.36-.08-.67-.23-.21-.1-.4-.15-.57-.15-.31 0-.57.11-.78.32s-.31.48-.31.8c0 .31.09.55.28.75.19.19.44.29.76.29.31 0 .6-.07.87-.2s.61-.42 1.02-.86c.41-.44.98-1.13 1.7-2.08.28.9.52 1.56.72 1.97.2.41.44.71.7.89.26.18.59.27.99.27.38 0 .77-.14 1.17-.43.54-.36 1.07-1 1.61-1.91zM17.51 1l-.15.54c.49 1.24.73 2.39.73 3.45 0 1.43-.21 2.96-.63 4.6-.33 1.26-.75 2.45-1.27 3.55-.52 1.11-1.02 1.97-1.51 2.6-.49.62-1.09 1.2-1.8 1.72l-.17.53c1.38-.59 2.57-1.34 3.55-2.23 1.29-1.17 2.26-2.5 2.91-4 .55-1.28.83-2.59.83-3.91 0-2.27-.83-4.56-2.49-6.85z"],"vertical-bar-chart-asc":["M8 7H7c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zM3 9H2c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h1c.55 0 1-.45 1-1v-9c0-.55-.45-1-1-1zm10-5h-1c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm5-4h-1c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"vertical-bar-chart-desc":["M3 0H2c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm5 4H7c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm5 3h-1c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm5 2h-1c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h1c.55 0 1-.45 1-1v-9c0-.55-.45-1-1-1z"],"vertical-distribution":["M1 2h18c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1s.45 1 1 1zm2 5c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1H3zm16 11H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],video:["M19 2H1c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zM7 14V6l6 4-6 4z"],"volume-down":["M15.92 3.93l-1.6 1.18A7.948 7.948 0 0 1 16 10c0 1.84-.63 3.54-1.68 4.89l1.6 1.18A9.878 9.878 0 0 0 18 10c0-2.29-.78-4.39-2.08-6.07zM11 3c-.28 0-.53.11-.71.29L7.59 6H3c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h4.59l2.71 2.71c.17.18.42.29.7.29.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"volume-off":["M14 3c-.28 0-.53.11-.71.29L10.59 6H6c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h4.59l2.71 2.71c.17.18.42.29.7.29.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"volume-up":["M9 3.43c-.28 0-.53.11-.71.29l-2.7 2.71H1c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h4.59l2.71 2.71a1.003 1.003 0 0 0 1.71-.71v-12c-.01-.55-.46-1-1.01-1zm8.31-1.56l-1.62 1.2C17.14 5.16 18 7.69 18 10.43s-.86 5.27-2.31 7.37l1.62 1.2C19 16.57 20 13.62 20 10.43c0-3.18-1-6.13-2.69-8.56zm-3.39 2.49l-1.6 1.18A7.948 7.948 0 0 1 14 10.43c0 1.84-.63 3.54-1.68 4.89l1.6 1.18A9.94 9.94 0 0 0 16 10.43c0-2.28-.78-4.38-2.08-6.07z"],walk:["M16 10h-2c-.23 0-.42-.09-.59-.21l-.01.01-1.69-1.27-.63 3.14 2.62 2.62c.19.18.3.43.3.71v4c0 .55-.45 1-1 1s-1-.45-1-1v-3.59L9.39 12.8l-2.45 6.55h-.01c-.14.38-.5.65-.93.65-.55 0-1-.45-1-1 0-.12.03-.24.07-.35h-.01L9.43 7h-2.9l-1.7 2.55-.01-.01c-.18.27-.47.46-.82.46-.55 0-1-.45-1-1 0-.21.08-.39.18-.54l-.01-.01 2-3 .02.01C5.36 5.19 5.65 5 6 5h4.18l.36-.96c-.33-.43-.54-.96-.54-1.54a2.5 2.5 0 0 1 5 0A2.5 2.5 0 0 1 12.5 5c-.06 0-.12-.01-.18-.02l-.44 1.18L14.33 8H16c.55 0 1 .45 1 1s-.45 1-1 1z"],"warning-sign":["M19.86 17.52l.01-.01-9-16-.01.01C10.69 1.21 10.37 1 10 1s-.69.21-.86.52l-.01-.01-9 16 .01.01c-.08.14-.14.3-.14.48 0 .55.45 1 1 1h18c.55 0 1-.45 1-1 0-.18-.06-.34-.14-.48zM11 17H9v-2h2v2zm0-3H9V6h2v8z"],"waterfall-chart":["M13 7h2c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1h-2c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm-9 8h1c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1zm4-6h2c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zm11-5h-1c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm0 12H2V3c0-.55-.45-1-1-1s-1 .45-1 1v14a.998.998 0 0 0 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],widget:["M18 4c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zM2 16c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm15-1h2V5h-2v10zM3 5H1v10h2V5zM2 0C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm3 3h10V1H5v2zm13 13c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM5 19h10v-2H5v2z"],"widget-button":["M1 4h18c.55 0 1 .45 1 1v10c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V5c0-.55.45-1 1-1zm1 2v8h16V6H2zm4 5c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm4 0c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm4 0c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"widget-footer":["M17 0H3c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H4v-4h12v4zm0-5H4V2h12v11z"],"widget-header":["M17 0H3c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H4V7h12v11zm0-12H4V2h12v4z"],wrench:["M19.8 4.44L16.13 8.1l-3.55-.71-.71-3.53L15.54.21c-2.01-.53-4.23-.03-5.8 1.53-1.86 1.85-2.23 4.6-1.14 6.83L.59 16.59C.22 16.95 0 17.45 0 18a2 2 0 0 0 2 2c.55 0 1.05-.22 1.41-.59l8.03-8.04c2.23 1.05 4.97.67 6.82-1.16 1.57-1.56 2.07-3.77 1.54-5.77z"],"zoom-in":["M19.56 17.44l-4.94-4.94A8.004 8.004 0 0 0 16 8c0-4.42-3.58-8-8-8S0 3.58 0 8s3.58 8 8 8c1.67 0 3.21-.51 4.5-1.38l4.94 4.94a1.498 1.498 0 1 0 2.12-2.12zM8 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6zm3-7H9V5c0-.55-.45-1-1-1s-1 .45-1 1v2H5c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V9h2c.55 0 1-.45 1-1s-.45-1-1-1z"],"zoom-out":["M11 7H5c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1zm8.56 10.44l-4.94-4.94A8.004 8.004 0 0 0 16 8c0-4.42-3.58-8-8-8S0 3.58 0 8s3.58 8 8 8c1.67 0 3.21-.51 4.5-1.38l4.94 4.94a1.498 1.498 0 1 0 2.12-2.12zM8 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6z"],"zoom-to-fit":["M1 7c.55 0 1-.45 1-1V2h4c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1v5c0 .55.45 1 1 1zm5 1a1.003 1.003 0 0 0-1.71-.71l-2 2c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l2 2a1.003 1.003 0 0 0 1.42-1.42L4.41 10 5.7 8.71c.19-.18.3-.43.3-.71zm2-2c.28 0 .53-.11.71-.29L10 4.41l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 0 0 .71-1.71l-2-2C10.53 2.11 10.28 2 10 2s-.53.11-.71.29l-2 2A1.003 1.003 0 0 0 8 6zM6 18H2v-4c0-.55-.45-1-1-1s-1 .45-1 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm8-6a1.003 1.003 0 0 0 1.71.71l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-2-2a1.003 1.003 0 0 0-1.42 1.42l1.3 1.29-1.29 1.29c-.19.18-.3.43-.3.71zm5-12h-5c-.55 0-1 .45-1 1s.45 1 1 1h4v4c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm-7 14c-.28 0-.53.11-.71.29L10 15.59 8.71 14.3A.965.965 0 0 0 8 14a1.003 1.003 0 0 0-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2A1.003 1.003 0 0 0 12 14zm7-1c-.55 0-1 .45-1 1v4h-4c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1z"]}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2),i=r.__importStar(n(0)),o=n(140),a=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.timeoutIds=[],r.clearTimeouts=function(){if(r.timeoutIds.length>0){for(var t=0,e=r.timeoutIds;t<e.length;t++){var n=e[t];window.clearTimeout(n)}r.timeoutIds=[]}},o.isNodeEnv("production")||r.validateProps(r.props),r}return r.__extends(e,t),e.prototype.componentWillReceiveProps=function(t){o.isNodeEnv("production")||this.validateProps(t)},e.prototype.componentWillUnmount=function(){this.clearTimeouts()},e.prototype.setTimeout=function(t,e){var n=window.setTimeout(t,e);return this.timeoutIds.push(n),function(){return window.clearTimeout(n)}},e.prototype.validateProps=function(t){},e}(i.PureComponent);e.AbstractPureComponent=a},function(t,e,n){"use strict";e.__esModule=!0,e.default=e.EXITING=e.ENTERED=e.ENTERING=e.EXITED=e.UNMOUNTED=void 0;var r=function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}return e.default=t,e}(n(1)),i=s(n(0)),o=s(n(16)),a=n(67);n(622);function s(t){return t&&t.__esModule?t:{default:t}}var A="unmounted";e.UNMOUNTED=A;var l="exited";e.EXITED=l;var c="entering";e.ENTERING=c;var u="entered";e.ENTERED=u;e.EXITING="exiting";var g=function(t){var e,n;function r(e,n){var r;r=t.call(this,e,n)||this;var i,o=n.transitionGroup,a=o&&!o.isMounting?e.enter:e.appear;return r.appearStatus=null,e.in?a?(i=l,r.appearStatus=c):i=u:i=e.unmountOnExit||e.mountOnEnter?A:l,r.state={status:i},r.nextCallback=null,r}n=t,(e=r).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n;var a=r.prototype;return a.getChildContext=function(){return{transitionGroup:null}},r.getDerivedStateFromProps=function(t,e){return t.in&&e.status===A?{status:l}:null},a.componentDidMount=function(){this.updateStatus(!0,this.appearStatus)},a.componentDidUpdate=function(t){var e=null;if(t!==this.props){var n=this.state.status;this.props.in?n!==c&&n!==u&&(e=c):n!==c&&n!==u||(e="exiting")}this.updateStatus(!1,e)},a.componentWillUnmount=function(){this.cancelNextCallback()},a.getTimeouts=function(){var t,e,n,r=this.props.timeout;return t=e=n=r,null!=r&&"number"!=typeof r&&(t=r.exit,e=r.enter,n=void 0!==r.appear?r.appear:e),{exit:t,enter:e,appear:n}},a.updateStatus=function(t,e){if(void 0===t&&(t=!1),null!==e){this.cancelNextCallback();var n=o.default.findDOMNode(this);e===c?this.performEnter(n,t):this.performExit(n)}else this.props.unmountOnExit&&this.state.status===l&&this.setState({status:A})},a.performEnter=function(t,e){var n=this,r=this.props.enter,i=this.context.transitionGroup?this.context.transitionGroup.isMounting:e,o=this.getTimeouts(),a=i?o.appear:o.enter;e||r?(this.props.onEnter(t,i),this.safeSetState({status:c},function(){n.props.onEntering(t,i),n.onTransitionEnd(t,a,function(){n.safeSetState({status:u},function(){n.props.onEntered(t,i)})})})):this.safeSetState({status:u},function(){n.props.onEntered(t)})},a.performExit=function(t){var e=this,n=this.props.exit,r=this.getTimeouts();n?(this.props.onExit(t),this.safeSetState({status:"exiting"},function(){e.props.onExiting(t),e.onTransitionEnd(t,r.exit,function(){e.safeSetState({status:l},function(){e.props.onExited(t)})})})):this.safeSetState({status:l},function(){e.props.onExited(t)})},a.cancelNextCallback=function(){null!==this.nextCallback&&(this.nextCallback.cancel(),this.nextCallback=null)},a.safeSetState=function(t,e){e=this.setNextCallback(e),this.setState(t,e)},a.setNextCallback=function(t){var e=this,n=!0;return this.nextCallback=function(r){n&&(n=!1,e.nextCallback=null,t(r))},this.nextCallback.cancel=function(){n=!1},this.nextCallback},a.onTransitionEnd=function(t,e,n){this.setNextCallback(n);var r=null==e&&!this.props.addEndListener;t&&!r?(this.props.addEndListener&&this.props.addEndListener(t,this.nextCallback),null!=e&&setTimeout(this.nextCallback,e)):setTimeout(this.nextCallback,0)},a.render=function(){var t=this.state.status;if(t===A)return null;var e=this.props,n=e.children,r=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);return i}(e,["children"]);if(delete r.in,delete r.mountOnEnter,delete r.unmountOnExit,delete r.appear,delete r.enter,delete r.exit,delete r.timeout,delete r.addEndListener,delete r.onEnter,delete r.onEntering,delete r.onEntered,delete r.onExit,delete r.onExiting,delete r.onExited,"function"==typeof n)return n(t,r);var o=i.default.Children.only(n);return i.default.cloneElement(o,r)},r}(i.default.Component);function h(){}g.contextTypes={transitionGroup:r.object},g.childContextTypes={transitionGroup:function(){}},g.propTypes={},g.defaultProps={in:!1,mountOnEnter:!1,unmountOnExit:!1,appear:!1,enter:!0,exit:!0,onEnter:h,onEntering:h,onEntered:h,onExit:h,onExiting:h,onExited:h},g.UNMOUNTED=0,g.EXITED=1,g.ENTERING=2,g.ENTERED=3,g.EXITING=4;var d=(0,a.polyfill)(g);e.default=d},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(1165);e.Dashboard=r.Dashboard;var i=n(680);e.DashboardSerializer=i.DashboardSerializer;var o=n(629);e.PartManager=o.PartManager;var a=n(1288);e.UrlParametersManager=a.UrlParametersManager},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(220);class i{canStringify(){return!1}toString(t){return""+t}tryFromString(t){throw new Error("Not Implemented")}fromString(t){let e=this.tryFromString(t);if(void 0===e)throw new Error("Invalid input string: "+t);return e}}e.Converter=i;class o extends i{convertFromSuperType(t){if(t.constructor.name===this.constructor.type.serializableName)return t}isValid(t){return this.inferInstanceOf(t)>0}canConvertFrom(t){return this.constructor.type.isInstanceOf(t)}convertFrom(t,e){if(this.canConvertFrom(e))return this.convertFromSuperType(t)}}e.PartialConverter=o;class a extends o{serialize(t){return console.warn("DATATYPE WARNING: Default no-op converter used. This may result in data loss. Check type annotations"),t}deserialize(t){return console.warn("DATATYPE WARNING: Default no-op converter used. This may result in data loss"),t}inferInstanceOf(t){return.1}}a.type=r.Types.Any,e.DefaultConverter=a;class s extends i{static isPrimitive(t){return"string"==typeof t}isValid(t){return null==t||"string"==typeof t}canStringify(){return!0}tryFromString(t){return t}canConvertFrom(t){return!0}convertFrom(t,e){return null!=t?t.toString():"null"}toString(t){return t}serialize(t){return""+t}deserialize(t){return"string"==typeof t?t:t.val}inferInstanceOf(t){return s.isPrimitive(t)||t instanceof String?1:-1}}s.type=r.Types.String,e.StringConverter=s;class A extends i{isValid(t){return!0}canConvertFrom(t){return!0}convertFrom(t,e){return t}serialize(t){return t}deserialize(t){return t}inferInstanceOf(t){return"object"==typeof t?1:0}}A.type=r.Types.Object,e.ObjectConverter=A;class l extends o{serialize(t){return t.map(t=>f.serialize(t,f.inferType(t)))}deserialize(t){return Array.from(t).map(t=>f.deserialize(t))}inferInstanceOf(t){return"undefined"!=typeof window&&window.useDevMode&&console.assert(Array.isArray(t)===t instanceof Array,"Cross-frame prototype leak detected! Data is being passed between JS contexts without proper serialization. This can lead to undefined behavior in the framework. Details: \n\tisArray:",Array.isArray(t),"\n\tinstanceOf:",t instanceof Array),t instanceof Array?1:-1}}l.type=r.Types.Array,e.ArrayConverter=l;class c extends i{isValid(t){return null==t||"Date"===t.constructor.name}canConvertFrom(t){return!1}convertFrom(t,e){}canStringify(){return!0}toString(t){if(null==t)return"";return`${(t.getMonth()+1).toString().padStart(2,"0")}-${t.getDate().toString().padStart(2,"0")}-${t.getFullYear().toString()}`}tryFromString(t){if(""===t.trim())return null;let e=(t=t.trim()).match(/^(\d{1,2})-(\d{1,2})-(\d{1,4})$/);if(null==e||4!==e.length)return;let n=Number.parseInt(e[1],10),r=Number.parseInt(e[2],10),i=Number.parseInt(e[3],10);return Number.isNaN(n)||Number.isNaN(r)||Number.isNaN(i)?void 0:new Date(t)}serialize(t){return Number(t)}deserialize(t){return new Date(t)}inferInstanceOf(t){return t instanceof Date?1:-1}}c.type=r.Types.Date,e.DateConverter=c;class u extends i{isValid(t){return null==t||"Date"===t.constructor.name}canConvertFrom(t){return!1}convertFrom(t,e){}canStringify(){return!0}toString(t){return null==t?"":t.toISOString()}tryFromString(t){if(""===t.trim())return null;let e=new Date(t);return Number.isNaN(e.getTime())?void 0:e}serialize(t){return Number(t)}deserialize(t){return new Date(t)}inferInstanceOf(t){return t instanceof Date?2:-1}}u.type=r.Types.DateTime,e.DateTimeConverter=u;class g extends i{static isPrimitive(t){return"number"==typeof t}isValid(t){return null==t||"number"==typeof t}canConvertFrom(t){return!1}convertFrom(t,e){}canStringify(){return!0}toString(t){return g.isPrimitive(t)?""+t:t.toString()}tryFromString(t){let e=+t;return"number"==typeof e?e:void 0}serialize(t){return Number.isNaN(t)?{boxed:!0,val:t.toString()}:"number"!=typeof t||Number.isFinite(t)?+t:{boxed:!0,val:t.toString()}}deserialize(t){return"number"==typeof t?t:t.boxed?Number(t.val):t.val}inferInstanceOf(t){return g.isPrimitive(t)||t instanceof Number?1:-1}}g.type=r.Types.Number,e.NumberConverter=g;class h extends i{static isPrimitive(t){return"boolean"==typeof t}isValid(t){return null==t||h.isPrimitive(t)||"Boolean"===t.constructor.name}canConvertFrom(t){return!1}convertFrom(t,e){}canStringify(){return!0}toString(t){return h.isPrimitive(t)?""+t:null==t?"":t.toString()}tryFromString(t){let e=t.toLowerCase();return"false"!==e&&Boolean(e)}serialize(t){return!!t}deserialize(t){return"boolean"==typeof t?t:t.boxed?Boolean(t.val):t.val}inferInstanceOf(t){return h.isPrimitive(t)||t instanceof Boolean?1:-1}}h.type=r.Types.Boolean,e.BooleanConverter=h;class d extends o{convertFromSuperType(t){if("Error"===t.constructor.name)return t}serialize(t){return{stack:t.stack,message:t.message,name:t.name}}deserialize(t){let e=new Function(`return new ${t.name}("${t.message}")`)();return e.stack=t.stack,e}inferInstanceOf(t){return t instanceof Error?1:-1}}d.type=r.Types.Error,e.ErrorConverter=d;class f{static convert(t,e,n){if(!f.registered.has(n))throw new Error("Converter for type not found: "+n.name);let r=f.registered.get(n);if(!r.canConvertFrom(e))throw new Error("Cannot convert from type: "+e.name+" to type "+n.name);return r.convertFrom(t,e)}static canStringify(t){let e=f.registered.get(t);return null!=e&&e.canStringify()}static toString(t,e){let n=f.registered.get(e);if(null==n)throw new Error("Converter for type not found: "+e.name);return n.toString(t)}static tryFromString(t,e){let n=f.registered.get(e);if(null==n)throw new Error("Converter for type not found: "+e.name);return n.tryFromString(t)}static registerConverter(t){const e=t.type;f.registered.has(e)?console.info("Converter for type: ",e.serializableName,"already registered"):f.registered.set(e,new t)}static isValid(t,e){if("Any"===e.name)return!0;if(!f.registered.has(e))return!1;return f.registered.get(e).isValid(t)}static inferType(t){if(null==t)return r.Types.Any;let e=[];return f.registered.forEach((n,r)=>{let i=t=>t.baseType?1+i(t.baseType):1,o=i(r);try{o*=n.inferInstanceOf(t),e.push({type:r,score:o})}catch(t){console.info("Inferrer for type",r.serializableName,"threw, skipping"),console.info(t)}}),e.reduce((t,e)=>t.score>e.score?t:e).type}static serialize(t,e){if(null==t)return null;if(!f.registered.has(e))throw Error("Converter for type "+e.serializableName+" not found");let n=f.registered.get(e).serialize(t);return{typeName:e.serializableName,value:n}}static deserialize(t){if(null==t)return;if(null==t.value)return null;let e=r.Types.findType(t.typeName);if(null==e)throw Error("Type "+t.typeName+" not available on this instance");if(!f.registered.has(e))throw Error("Converter for type "+e.serializableName+" not found on this instance!");return f.registered.get(e).deserialize(t.value)}static hasConverter(t){return f.registered.has(t)}}f.registered=new Map,e.Converters=f,[a,s,g,c,u,A,l,h,d].map(t=>f.registerConverter(t))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(6),i=n(9),o=n(25),a=n(28),s=n(141),A=n(633),l=n(113);class c extends r.Widget{constructor(){if(super(),this.hovers=new Map,null!=c.Instance)throw this.dispose(),Error("Cannot instantiate multiple HoverManagers!");c.Instance=this,this.addClass("m-HoverManager"),window.addEventListener("beforeunload",()=>{for(const t of this.hovers.values())t.dispose()})}static GetManager(){return this.Instance||new c}openDialog({hover:t,owner:e,width:n,height:r}){return this.openHover({hover:t,owner:e,x:0,y:0,width:n,height:r,mode:"dialog"})}openErrorDialog(t,e=!1){const n=e||t instanceof Error?t:new Error(t);let i;return e?(i=new r.Widget).node.innerHTML=n:((i=new r.Widget({node:document.createElement("pre")})).node.innerText=e?""+n:n.stack||n.message,i.node.style.overflow="auto",i.node.style.margin="0"),this.openDialog({hover:new c.DialogWithButtons(i,[{text:"Dismiss"}]),owner:this,height:400,width:800})}openErrorMsgDialog(t,e,n=!1){const i=new r.Widget;if(i.node.innerHTML=t,null!=e){const t=n||e instanceof Error?e:new Error(e),r=document.createElement("pre");n?r.innerHTML=t:(r.innerText=n?""+t:t.stack||t.message,r.style.overflow="auto",r.style.margin="0");const o=document.createElement("details");o.appendChild(r);const a=document.createElement("summary");a.innerText="Error Details",o.appendChild(a),i.node.appendChild(o)}const o=new c.DialogWithButtons(i,[{text:"Dismiss"}]);return o.title.label="Error",this.openDialog({hover:o,owner:this,height:400,width:800})}async launchDialog(t,e,n,r,i,o=[]){const a=new c.DialogWithButtons(t,o);a.title.label=i;const s=this.openDialog({hover:a,owner:e,width:n,height:r});return await s.onClosed,{accept:a.accept||!1,clicked:a.clicked||"",result:a.result}}async launchEditorDialog(t,e,n,r,i,o){const a=new c.EditorDialog(t,o);a.title.label=i;const s=this.openDialog({hover:a,owner:e,width:n,height:r});return await s.onClosed,{accept:a.accept||!1,clicked:a.clicked||"",result:a.result}}openHover({hover:t,owner:e,...n}){const o=i.UUID.uuid4(),a=new i.PromiseDelegate,s=new c.HoverViewModel(o,a.promise);if("relative"===n.offsetMode){let t=e.node.getBoundingClientRect();n.x=n.x+t.left,n.y=n.y+t.top}e.disposed.connect(()=>{this._handleOwnerDisposed(o)}),t.disposed.connect(()=>{this._handleOwnerDisposed(o)});const A=c.HoverWrapperFactory(t,e,n.mode||"hover",{x:n.x||0,y:n.y||0,width:n.width||300,height:n.height||300});if(A.disposed.connect(()=>{a.resolve()}),"popup"===n.mode){const t=["left="+n.x,"top="+n.y,"width="+n.width,"height="+n.height].join(",");let r=window.open("about:blank",o,t);setTimeout(()=>{this.writeToPopup(r,A,o,e,n.cloneEvents)},0)}else r.Widget.attach(A,this.node);return this.hovers.set(o,A),s}resizeHover(t,e,n){const i=this.getKey(t),a=this.hovers.get(i);return a.data.width=e,a.data.height=n,o.MessageLoop.sendMessage(a,r.Widget.Msg.UpdateRequest),a}closeHover(t,{disposeAtExit:e}={disposeAtExit:!0}){const n=this.getKey(t),r=this.hovers.get(n);return e?r.dispose():r.isAttached&&r.close(),r}dispose(){if(!this.isDisposed){for(const t of this.hovers.values())t.dispose();super.dispose()}}closeAllHovers(){for(const t of this.hovers.values())t.dispose()}writeToPopup(t,e,n,i,A){if(null==t)throw Error("Window did not open as expected, cannot continue");t.window.document.title=e.title.label||"MavenWorks";const l=t.document.createElement("link");l.rel="icon",l.href="data:image/png;base64,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",t.document.head.appendChild(l),a.FrameTools.copyStylesheets(document,t.document),o.MessageLoop.sendMessage(e,s.ReactWrapperWidget.BeforeChangeDocumentOwner);const c=t.document.adoptNode(e.node);e.node=c,o.MessageLoop.sendMessage(e,s.ReactWrapperWidget.AfterChangeDocumentOwner),o.MessageLoop.sendMessage(e,r.Widget.Msg.BeforeAttach),t.document.body.appendChild(c),t.addEventListener("beforeunload",()=>{o.MessageLoop.sendMessage(e,r.Widget.Msg.BeforeDetach),null!=t&&t.document.body.removeChild(e.node),o.MessageLoop.sendMessage(e,r.Widget.Msg.AfterDetach),this._handleOwnerDisposed(n)}),t.addEventListener("resize",()=>{e.update()});const u=A?a.FrameTools.bubbleEvents(t.document,i.node):null;e.disposed.connect(()=>{null!=u&&u.dispose(),null!=t&&t.close()}),o.MessageLoop.sendMessage(e,r.Widget.Msg.AfterAttach)}_handleOwnerDisposed(t){if(!this.hovers.has(t))return;this.hovers.get(t).dispose()}getKey(t){const e="string"==typeof t?t:t.id;if(!this.hovers.has(e))throw Error("Hover does not exist");return e}}e.HoverManager=c,function(t){t.HoverViewModel=class{constructor(t,e){this.id=t,this.onClosed=e}};class e extends r.Widget{constructor(t,e){super(),this.hover=t,this.buttons=e,this.layout=new r.BoxLayout,this.layout.direction="top-to-bottom",r.BoxLayout.setStretch(t,1),this.layout.addWidget(t),this.btnBar=new r.BoxPanel({direction:"right-to-left"}),this.setupButtons(),r.Layout.setHorizontalAlignment(this.btnBar,"right"),r.BoxLayout.setSizeBasis(this.btnBar,50),this.layout.addWidget(this.btnBar)}get accept(){return this._accept}get clicked(){return this._clicked}get result(){return this._result}onCloseRequest(){this._accept&&this.hover.getValue&&(this._result=this.hover.getValue()),this.dispose()}setupButtons(){for(const t of this.buttons.reverse()){const e=new A.Interactions.Button;e.addClass("m-Dialog-Btn"),e.dataset.type=t.warn?"warn":t.accept?"accept":"neutral",e.label=t.text.toLocaleUpperCase(),e.node.style.maxWidth="100px",this.setupButton(e,t),this.btnBar.addWidget(e)}}setupButton(t,e){t.onClicked.pipe(l.first()).toPromise().then(()=>{this._clicked=e.text,this._accept=null!=e.accept&&e.accept,this.close()})}}t.DialogWithButtons=e;t.EditorDialog=class extends e{constructor(t,e){super(t,[{text:"OK",accept:!0},{text:"Dismiss"},{text:"Apply",accept:!0}]),this.onApplyCallback=e}setupButton(t,e){"Apply"===e.text||"OK"===e.text?t.onClicked.subscribe(()=>{let t;try{t=this.hover.getValue?this.hover.getValue():void 0}catch(t){return console.error("Uncaught error in Editor Dialog calling getValue:"),void console.error(t)}this.onApplyCallback.call(void 0,t),"OK"===e.text&&this.close()}):super.setupButton(t,e)}},t.HoverWrapperFactory=function(t,e,r,o){switch(r){case"tooltip":return new i(t,e,o);case"popup":return new s(t,e,o);case"dialog":return new c(t,e,o);default:return new n(t,e,r,o)}};class n extends r.Widget{constructor(t,e,n,i){super(),this.owner=e,this.layout=new r.BoxLayout,this.widget=t,this.layout.addWidget(this.widget),this.addClass("m-Hover"),this.data=i,this.mode=n,this.dataset.mode=this.mode,this.title.label=t.title.label,this.update()}dispose(){this.isDisposed||super.dispose()}processMessage(t){switch(t.type){case"before-change-doc-owner":case"after-change-doc-owner":o.MessageLoop.sendMessage(this.widget,t);break;default:super.processMessage(t)}}onUpdateRequest(){this.updatePosition()}updatePosition(){Object.assign(this.node.style,{left:`${this.data.x}px`,top:`${this.data.y}px`,width:`${this.data.width}px`,height:`${this.data.height}px`})}}t.HoverWrapper=n;class i extends n{constructor(t,e,n){super(t,e,"tooltip",n),this.node.style.pointerEvents="none",document.body.addEventListener("mousemove",this),this.owner.node.addEventListener("mouseleave",this)}handleEvent(t){switch(t.type){case"mousemove":return this.data={...this.data,x:t.pageX+10,y:t.pageY+10},void o.MessageLoop.sendMessage(this,r.Widget.Msg.UpdateRequest);case"mouseleave":return void this.dispose();default:return void console.trace("Unhandled event",t.type)}}dispose(){this.isDisposed||(document.body.removeEventListener("mousemove",this),this.owner.node.removeEventListener("mouseleave",this),super.dispose())}}class s extends n{constructor(t,e,n){super(t,e,"popup",n),this.data.x=0,this.data.y=0}updatePosition(){Object.assign(this.node.style,{left:0,right:0,top:0,bottom:0})}}class c extends n{constructor(t,e,n){super(t,e,"dialog",n),t.node.style.padding="5px",this.node.style.zIndex=""+ ++c.Z_INDEX,this.data.x=window.innerWidth/2-n.width/2,this.data.y=window.innerHeight/2-n.height/2;const i=new u(this,t.title.label);r.BoxLayout.setStretch(i,0),r.BoxLayout.setSizeBasis(i,25),r.BoxLayout.setStretch(t,1),this.layout.insertWidget(0,i);const a=new A.Interactions.ResizerGrippy(this,document.body);a.onSizeChange.subscribe(([t,e,n,r])=>{this.data.width=n,this.data.height=r,this.update()}),o.MessageLoop.sendMessage(a,r.Widget.Msg.BeforeAttach),this.node.appendChild(a.node),o.MessageLoop.sendMessage(a,r.Widget.Msg.AfterAttach)}onActivateRequest(){this.node.style.zIndex!==""+c.Z_INDEX&&(this.node.style.zIndex=""+ ++c.Z_INDEX)}}c.Z_INDEX=1;class u extends r.Widget{constructor(t,e){super(),this.owner=t,this.label=new r.Widget({node:document.createElement("span")}),this.btn=new A.Interactions.Button,this.offset=[0,0],this.startPos=[0,0],this.addClass("m-Dialog-titlebar"),this.layout=new r.BoxLayout({direction:"left-to-right"}),this.label.node.textContent=e,this.btn.className="fa fa-times",r.BoxLayout.setStretch(this.label,1),r.BoxLayout.setStretch(this.btn,0),r.BoxLayout.setSizeBasis(this.btn,25),this.layout.addWidget(this.label),this.layout.addWidget(this.btn),this.btn.onClicked.subscribe(()=>t.dispose())}handleEvent(t){switch(t.type){case"pointerdown":this.owner.activate(),this.startDrag(t);break;case"pointermove":this.onMouseMove(t);break;case"pointerup":this.endDrag();break;case"keydown":"Escape"!==t.key||this.owner.node.contains(t.target)||this.endDrag()}}onAfterAttach(){this.node.addEventListener("pointerdown",this)}onBeforeDetach(){this.endDrag(),this.node.removeEventListener("pointerdown",this)}startDrag(t){"mouse"===t.pointerType&&0!==t.button||t.target===this.node&&(a.FrameTools.DisableFramePointerEvents(),this.offset=[t.pageX,t.pageY],this.startPos=[this.owner.data.x,this.owner.data.y],window.addEventListener("pointermove",this),window.addEventListener("pointerup",this),window.addEventListener("keydown",this))}onMouseMove(t){const e=a.MathTools.Vec2.Sub([t.pageX,t.pageY],this.offset),n=a.MathTools.Vec2.Add(this.startPos,e),{width:r,height:i}=this.owner.data,o=[window.innerWidth,window.innerHeight],[s,A]=a.MathTools.ClampRectToBounds(n,[r,i],o);this.owner.data.x=s,this.owner.data.y=A,this.owner.update()}endDrag(){window.removeEventListener("pointermove",this),window.removeEventListener("pointerup",this),window.removeEventListener("keydown",this),a.FrameTools.EnableFramePointerEvents(),this.offset=[0,0],this.startPos=[0,0]}}}(c=e.HoverManager||(e.HoverManager={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0);function i(){const[t,e]=r.useState(0);return function(){e(Math.random())}}function o(t){const e=r.useRef();return r.useEffect(()=>{e.current=t},[e,t]),e.current}e.useRender=i,e.usePrevious=o,e.useIntermediate=function(t,e){const[n,i]=r.useState(0),a=o(t),[s,A]=r.useState(t);return t!==s&&t!==a&&(A(t),n!==Number.MAX_SAFE_INTEGER?i(n+1):i(0)),[t,n,t=>{A(t),e.call(void 0,t)}]},e.renderOnEmit=function(t){const e=i();r.useEffect(()=>{const n=t.subscribe(()=>e());return()=>n.unsubscribe()},[t])}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(6);e.isWidgetCtor=function(t){return t.prototype instanceof i.Widget},function(t){t.Context=r.createContext({portalHostNode:document.body})}(e.TypeEditorHost||(e.TypeEditorHost={}))},function(t,e,n){var r=n(1190);t.exports=function(t,e,n){if(r(t),void 0===e)return t;switch(n){case 1:return function(n){return t.call(e,n)};case 2:return function(n,r){return t.call(e,n,r)};case 3:return function(n,r,i){return t.call(e,n,r,i)}}return function(){return t.apply(e,arguments)}}},function(t,e,n){var r=n(144);t.exports=function(t,e){if(!r(t))return t;var n,i;if(e&&"function"==typeof(n=t.toString)&&!r(i=n.call(t)))return i;if("function"==typeof(n=t.valueOf)&&!r(i=n.call(t)))return i;if(!e&&"function"==typeof(n=t.toString)&&!r(i=n.call(t)))return i;throw TypeError("Can't convert object to primitive value")}},function(t,e){var n={}.toString;t.exports=function(t){return n.call(t).slice(8,-1)}},function(t,e){t.exports=function(t){if(null==t)throw TypeError("Can't call method on  "+t);return t}},function(t,e){var n=Math.ceil,r=Math.floor;t.exports=function(t){return isNaN(t=+t)?0:(t>0?r:n)(t)}},function(t,e,n){var r=n(321)("keys"),i=n(225);t.exports=function(t){return r[t]||(r[t]=i(t))}},function(t,e,n){var r=n(71),i=n(110),o=i["__core-js_shared__"]||(i["__core-js_shared__"]={});(t.exports=function(t,e){return o[t]||(o[t]=void 0!==e?e:{})})("versions",[]).push({version:r.version,mode:n(224)?"pure":"global",copyright:"© 2019 Denis Pushkarev (zloirock.ru)"})},function(t,e){t.exports="constructor,hasOwnProperty,isPrototypeOf,propertyIsEnumerable,toLocaleString,toString,valueOf".split(",")},function(t,e){e.f=Object.getOwnPropertySymbols},function(t,e,n){var r=n(143),i=n(1198),o=n(322),a=n(320)("IE_PROTO"),s=function(){},A=function(){var t,e=n(639)("iframe"),r=o.length;for(e.style.display="none",n(1199).appendChild(e),e.src="javascript:",(t=e.contentWindow.document).open(),t.write("<script>document.F=Object<\/script>"),t.close(),A=t.F;r--;)delete A.prototype[o[r]];return A()};t.exports=Object.create||function(t,e){var n;return null!==t?(s.prototype=r(t),n=new s,s.prototype=null,n[a]=t):n=A(),void 0===e?n:i(n,e)}},function(t,e,n){var r=n(111).f,i=n(121),o=n(88)("toStringTag");t.exports=function(t,e,n){t&&!i(t=n?t:t.prototype,o)&&r(t,o,{configurable:!0,value:e})}},function(t,e,n){e.f=n(88)},function(t,e,n){var r=n(110),i=n(71),o=n(224),a=n(326),s=n(111).f;t.exports=function(t){var e=i.Symbol||(i.Symbol=o?{}:r.Symbol||{});"_"==t.charAt(0)||t in e||s(e,t,{value:a.f(t)})}},function(t,e,n){var r=n(226),i=n(178),o=n(122),a=n(316),s=n(121),A=n(638),l=Object.getOwnPropertyDescriptor;e.f=n(112)?l:function(t,e){if(t=o(t),e=a(e,!0),A)try{return l(t,e)}catch(t){}if(s(t,e))return i(!r.f.call(t,e),t[e])}},function(t,e,n){"use strict";e.__esModule=!0;var r=l(n(227));e.spyOnComponent=r.default;var i=l(n(1230));e.autoFocus=i.default;var o=l(n(1231));e.focusManager=o.default;var a=l(n(330));e.mountManager=a.default;var s=l(n(653));e.timeoutManager=s.default;var A=l(n(1232));function l(t){return t&&t.__esModule?t:{default:t}}e.mixin=A.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t){var e=!0;return(0,i.default)(t,{componentWillUnmount:function(){e=!1}}),function(){return e}};var r,i=(r=n(227))&&r.__esModule?r:{default:r};t.exports=e.default},function(t,e,n){"use strict";t.exports=function(t,e,n,r,i,o,a,s){if(!t){var A;if(void 0===e)A=new Error("Minified exception occurred; use the non-minified dev environment for the full error message and additional helpful warnings.");else{var l=[n,r,i,o,a,s],c=0;(A=new Error(e.replace(/%s/g,function(){return l[c++]}))).name="Invariant Violation"}throw A.framesToPop=1,A}}},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t,e){var n=(0,o.default)(t);return n?n.innerHeight:e?t.clientHeight:(0,i.default)(t).height};var i=r(n(666)),o=r(n(228));t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.defaultGetDataState=l,e.getCommonListProps=function(t,e,n){var r=n.groupBy,i=n.optionComponent,o=n.itemComponent,s=n.groupComponent,A=n.searchTerm,l=n.listProps;return a({searchTerm:A,groupBy:r,groupComponent:s,itemComponent:o,optionComponent:i},l,{data:t.data,dataState:t.state,textAccessor:e.text,valueAccessor:e.value})},e.default=function(t,e,n){var i=void 0===n?{}:n,a=i.nextProps,A=i.getDataState,g=a.disabled,h=a.valueField,d=a.textField,f=(A=A||c(a)||l)(t,a,e&&e.dataState),p=f&&f.sequentialData||t,I=u(g,h),C=function(t,e){return I(t)||e&&!r.presets.startsWith((0,o.dataText)(t,d).toLowerCase(),e.toLowerCase())},m={dataState:f,isDisabled:I,first:function(){return m.next(s)},last:function(){return m.prevEnabled(p[p.length-1])},prev:function(t,e){for(var n=Math.max(0,p.indexOf(t))-1;n>-1&&C(p[n],e);)n--;return n>=0?p[n]:I(t)?null:t},next:function(t,e){for(var n=p.indexOf(t)+1;n<p.length&&C(p[n],e);)n++;return n<p.length?p[n]:I(t)?null:t},prevEnabled:function(t){return I(t)?m.prev(t):t},nextEnabled:function(t){return I(t)?m.next(t):t}};return m};var r=n(334),i=n(229),o=n(181);function a(){return(a=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var s={},A=function(){return!1};function l(t,e,n){var r=e.groupBy;if(void 0===n&&(n={}),n.data!==t||n.groupBy!==r){if(!r)return{};var o=[],a=(0,i.groupBySortedKeys)(r,t,o);return{data:t,groupBy:r,groups:a,sortedKeys:o,sequentialData:Object.keys(a).reduce(function(t,e){return t.concat(a[e])},[])}}return n}var c=function(t){var e=t.listComponent;return e&&e.getDataState},u=function(t,e){return Array.isArray(t)?function(n){return t.some(function(t){return(0,o.dataValue)(n,e)===(0,o.dataValue)(t,e)})}:A}},function(t,e,n){"use strict";e.__esModule=!0,e.indexOf=function(t,e){var n=e.searchTerm,r=void 0===n?"":n,i=c(s(e,["searchTerm"])),o=i.filter,a=i.minLength;if(!o||!r||!r.trim()||r.length<a)return-1;for(var A=0;A<t.length;A++)if(o(t[A],r,A))return A;return-1},e.filter=function(t,e){var n=e.searchTerm,r=void 0===n?"":n,i=c(s(e,["searchTerm"])),o=i.filter,a=i.minLength;return!o||!r||!r.trim()||r.length<a?t:t.filter(function(t,e){return o(t,r,e)})},e.suggest=function(t,e){var n=e.searchTerm,r=void 0===n?"":n,i=c(s(e,["searchTerm"])),o=i.filter,a=i.minLength;if(!o||!r||!r.trim()||r.length<a)return r;for(var A=0;A<t.length;A++)if(o(t[A],r,A))return t[A];return r},e.propTypes=e.presets=void 0;var r,i=(r=n(1))&&r.__esModule?r:{default:r},o=function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}return e.default=t,e}(n(146)),a=n(181);function s(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(r=0;r<a.length;r++)n=a[r],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(i[n]=t[n])}return i}function A(){return(A=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var l={eq:function(t,e){return t===e},neq:function(t,e){return t!==e},gt:function(t,e){return t>e},gte:function(t,e){return t>=e},lt:function(t,e){return t<e},lte:function(t,e){return t<=e},contains:function(t,e){return-1!==t.indexOf(e)},startsWith:function(t,e){return 0===t.lastIndexOf(e,0)},endsWith:function(t,e){var n=t.length-e.length,r=t.indexOf(e,n);return-1!==r&&r===n}};function c(t){var e=A({},t);return e.minLengh=e.minLengh||0,e.filter=function(t){var e,n=t.filter,r=t.caseSensitive,i=void 0!==r&&r,o=t.textField;return"function"!=typeof(n=!1===(e=n)?null:!0===e?"startsWith":e||"eq")&&n?(n=l[n],function(t,e){var r=(0,a.dataText)(t,o);return i||(r=r.toLowerCase(),e=e.toLowerCase()),n(r,e)}):n}(e),e}e.presets=l;var u={textField:o.accessor,caseSensitive:i.default.bool,minLength:i.default.number,filter:i.default.oneOfType([i.default.func,i.default.bool,i.default.oneOf(Object.keys(l))])};e.propTypes=u},function(t,e,n){"use strict";e.__esModule=!0,e.getMessages=function(t){void 0===t&&(t={});var e={};return Object.keys(o).forEach(function(n){var r=t[n];null==r&&(r=o[n]),e[n]="function"==typeof r?r:function(){return r}}),e};var r,i=(r=n(0))&&r.__esModule?r:{default:r};var o={moveBack:"Navigate back",moveForward:"Navigate forward",dateButton:"Select date",timeButton:"Select time",openCombobox:"open combobox",openDropdown:"open dropdown",placeholder:"",filterPlaceholder:"",emptyList:"There are no items in this list",emptyFilter:"The filter returned no results",createOption:function(t){var e=t.searchTerm;return[" Create option",e&&" ",e&&i.default.createElement("strong",{key:"_"},'"'+e+'"')]},tagsLabel:"Selected items",removeLabel:"Remove selected item",noneSelected:"no selected items",selectedItems:function(t){return"Selected items: "+t.join(", ")},increment:"Increment value",decrement:"Decrement value"}},function(t,e,n){"use strict";e.__esModule=!0,e.widgetEditable=e.widgetEnabled=e.isInDisabledFieldset=void 0;var r,i=n(16),o=(r=n(1253))&&r.__esModule?r:{default:r};var a=function(t){var e;try{e=(0,i.findDOMNode)(t)}catch(t){}return!!e&&(0,o.default)(e,"fieldset[disabled] *")};e.isInDisabledFieldset=a;var s=l(!0);e.widgetEnabled=s;var A=l(!1);function l(t){function e(e){return function(){var n=this.props,r=n.disabled,i=n.readOnly;r=a(this)||1==r||!t&&!0===i;for(var o=arguments.length,s=new Array(o),A=0;A<o;A++)s[A]=arguments[A];if(!r)return e.apply(this,s)}}return function(t,n,r){if(r.initializer){var i=r.initializer;r.initializer=function(){return e(i.call(this)).bind(this)}}else r.value=e(r.value);return r}}e.widgetEditable=A},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(221),i=n(28);e.ErrorPart=class extends r.Part{constructor(t,e){super(t),this.originalName=e||"ErrorPart",this.node.innerHTML=`\n            <h3>Error: Unrecognized Part "${this.originalName}"</h3>\n            <p>Don't save this notebook to prevent data loss</p>\n        `,this.node.style.color="var(--jp-error-color0, red)",this.node.style.overflowY="scroll"}get typeName(){return this.originalName}get originalModel(){return this._originalModel}setModel(t){null!=t&&(this._originalModel=t,this.node.innerHTML+=`\n            <span>Options:</span>\n            <ul>\n            ${Object.keys(t.options).map(e=>{const n=t.options[e];let r,o=i.Types.Any;return n&&n.hasOwnProperty("typeName")?(r=i.Converters.deserialize(n),o=i.Types.findType(n.typeName)||i.Types.Any):null!=n&&(r=n.type+"Binding: "+n.expr,o=i.Types.String),`\n                    <li><code>${e}=${i.Converters.serialize(r,o)}</code></li>\n                `}).join("")}\n            </ul>\n        `)}async initialize(){}async render(){}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(1282);e.GlobalsService=r.GlobalsService;var i=n(1283);e.BindingsProvider=i.BindingsProvider,e.BaseBindingsEvaluator=i.BaseBindingsEvaluator,e.ErrorBindingsEvaluator=i.ErrorBindingsEvaluator,e.getCommentLine=i.getCommentLine,e.getGlobalsFromComment=i.getGlobalsFromComment;var o=n(678);e.MqlWorkerPool=o.MqlWorkerPool},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(125),i=n(232),o=n(28);e.WidgetLayoutRegion=class extends i.DashboardLayoutRegion{constructor(t,e,n,i){super(t,i),this.typeName=r.LayoutTypes.WidgetLayoutRegion,this.addClass("maven_widget"),this.content=e;const o=e.title.label;""!==o&&"New Part"!==o&&(this.properties={...this.properties,caption:o}),this.title.label=e.title.label,this.guid=n}static GetMetadata(){const t=super.GetMetadata();return t.addMetadata("showOverlays",{prettyName:"General.Show Overlays?",type:o.Types.Boolean,documentation:"If false, the Calculating overlays on Parts will be hidden. Overlays will still be shown if the Part encounters an error.",default:!0}),t.getMetadata("caption").default="New Part",t.iconClass+=" m-widget-icon",t}updateFromProperties(){super.updateFromProperties(),this.content.toggleClass("m-hide-part-overlays",!this.getLayoutProperty("showOverlays"))}getChild(){return this.content}dispose(){this.isDisposed||(this.content.dispose(),super.dispose())}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(6),i=n(125),o=n(124),a=n(27),s=n(25),A=n(28);class l extends o.RegionWithChildren{constructor(t,e){super(t,e),this.typeName=i.LayoutTypes.TabPanelDashboardLayoutRegion,this.addClass("maven_tabpanel"),this.addClass("m-hide-child-titlebars"),this.content=new r.TabPanel({tabsMovable:!0,renderer:new l.TabRenderer}),this.content.currentChanged.connect(this.handleCurrentChanged,this),this.content.tabBar.tabMoved.connect(this.handleTabMoved,this),this.content.tabBar.tabDetachRequested.connect(this.handleTabDetached,this),this.content.tabBar.addClass("p-DockPanel-tabBar"),this.content.stackedPanel.addClass("maven_tabpanel_panel"),this.layout.addWidget(this.content),this.installContentTap()}static GetMetadata(){const t=super.GetMetadata();return t.name="Tab Panel",t.iconText="tab",t.description="Arrange children in a set of tabs",t.remarks='\nTab heads are customizable using the Titlebar properties on each child. The\nactive tab is preserved as "Foreground Index," and will be saved with the rest\nof the dashboard.\n\n> #### Note\n>\n> There is an active bug with tab heads, where right clicks on them will\n> actually operate on the parent. To reliably change a property on the child,\n> right-click _inside_ the child.',t.addMetadata("ForegroundIndex",{prettyName:"Tab Index",type:A.Types.Number,documentation:"The index of the currently selected tab",default:0}),t}dispose(){this.isDisposed||(this.content.currentChanged.disconnect(this.handleCurrentChanged,this),this.content.tabBar.tabMoved.disconnect(this.handleTabMoved,this),this.content.tabBar.tabDetachRequested.disconnect(this.handleTabDetached,this),this.content.dispose(),super.dispose())}layoutChildren(){if(0===this.widgets.length)return void this.content.hide();this.content.show();let t=this.content.currentWidget;if(null==t||!t.getLayoutProperty("showRegion")){const e=this.widgets.find(t=>t.getLayoutProperty("showRegion"));if(null==e)return void this.content.hide();this.content.currentWidget=e,t=e}const{width:e,height:n}=this.content.stackedPanel.node.getBoundingClientRect();t.calculatedSize={width:e,height:n},s.MessageLoop.sendMessage(t,r.Widget.Msg.FitRequest);for(const e of this.widgets)e!==t&&(e.isHidden||e.hide());s.MessageLoop.sendMessage(this.content.tabBar,r.Widget.Msg.UpdateRequest)}updateFromProperties(){const t=this.getLayoutProperty("ForegroundIndex");this.content.currentIndex!==t&&this.widgets.length>0&&(this.content.currentIndex=t,this.content.currentWidget.activate()),s.MessageLoop.sendMessage(this.content.stackedPanel,r.Widget.Msg.FitRequest),super.updateFromProperties()}onChildRegionRemoved(t){t.isDisposed||t.show()}handleCurrentChanged(t,{currentIndex:e,currentWidget:n,previousWidget:r}){r&&!this.widgets.includes(r)&&r.show(),this.getLayoutProperty("ForegroundIndex")===e&&n.isFocused||(null!=n?(this.widgets[e].focus(),this.setLayoutProperty("ForegroundIndex",e)):this.content.widgets.length>0&&(this.content.currentIndex=0))}handleTabMoved(t,{toIndex:e}){this.setLayoutProperty("ForegroundIndex",e)}handleTabDetached(t,{clientX:e,clientY:n,title:r}){r.owner.startDrag(e,n)}}e.TabPanelDashboardLayoutRegion=l,function(t){t.TabRenderer=class extends r.TabBar.Renderer{createTabStyle({title:t,zIndex:e}){const n=t.owner.getLayoutProperty("captionColor").color,r=t.owner.getLayoutProperty("captionBackground").color;return{color:"transparent"===n?null:n,backgroundColor:"transparent"===r?null:r,zIndex:""+e}}renderLabel({title:t}){const e=t.owner.getLayoutProperty("caption")||"New Tab";return a.h.div({className:"p-TabBar-tabLabel"},e)}renderTab(t){return t.title.owner.getLayoutProperty("showRegion")?super.renderTab(t):a.h.div({className:"p-TabBar-tab",style:{display:"none"}})}}}(l=e.TabPanelDashboardLayoutRegion||(e.TabPanelDashboardLayoutRegion={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(25),i=n(6),o=n(125),a=n(124),s=n(28);var A=s.MathTools.Vec2;const l=n(43),c=n(66);class u extends a.RegionWithChildren{constructor(t,e){super(t,e),this.typeName=o.LayoutTypes.CanvasLayoutRegion,this.addClass("m-CanvasPanel"),this.content=new i.Panel,this.content.addClass("m-CanvasPanel-canvas"),this.layout.addWidget(this.content),this.installContentTap()}static GetMetadata(){const t=super.GetMetadata();return t.name="Canvas Panel",t.iconText="picture_in_picture",t.description="Arrange children freely within the container.",t.remarks="\nThe Canvas is completely free-form, and can place children anywhere inside\nitself. The canvas uses pixel coordinates and sizes, and thus may not look the\nsame on different screens.\n\nCanvas panels are a bit special in that they attach some extra chroming to each\nchild. You can resize a child by dragging a 'resize grippy' on the lower right\nhand corner, and you can move a child by dragging the titlebar.\n\nWhen dragging, the canvas will 'clamp' the boundaries of a part to itself. If\nyou don't want this behavior, you can manually edit the \"Canvas Panel.Rectangle\"\nproperty.\n\nIf you want to use the normal docking behavior, hold down <kbd>Shift</kbd> when\ndragging. This will allow you to, eg, drop a child into a tab panel where the\ncanvas is.",t.addAttachedMetadata(this.Rectangle,{type:s.Types.String,default:"0,0,100,100",documentation:"A comma-separated list of the Left-position, Top-position, Width, and Height (respectively) for this child."}),t}layoutChildren(){if(0!==this.widgets.length)for(const t of this.widgets){const e=u.Rectangle.get(t),[n,o,a,s]=e.split(",").map(t=>+t);t.calculatedSize={width:a,height:s,left:n,top:o},r.MessageLoop.sendMessage(t,i.Widget.Msg.FitRequest)}}dispose(){this.isDisposed||(this.content.dispose(),super.dispose())}createDragShadow(t,e,n){const r=document.createElement("div");r.textContent=t.getLayoutProperty("caption");const[i,o,a,s]=u.Rectangle.get(t).split(","),{top:l,left:c}=this.node.getBoundingClientRect(),g=A.Sub([e,n],A.Add([c,l],[+i,+o]));return Object.assign(r.style,{"margin-left":`-${g[0]}px`,"margin-top":`-${g[1]}px`,width:`${a}px`,height:`${s}px`,backgroundColor:"rgba(100, 100, 100, 0.7)",color:"white"}),r}onChildRegionAdded(t){const e=new c.Interactions.ResizerGrippy(t,this.node);e.onSizeChange.subscribe(([e,n,r,i])=>{u.Rectangle.set(t,`${e},${n},${r},${i}`)}),t.attachChrome(e)}onDragOver(t){if(t.shiftKey)return this.showOverlay(),void super.onDragOver(t);t.dropAction="move",t.preventDefault(),t.stopPropagation(),this.hideOverlay()}onDrop(t){if(t.shiftKey)return void super.onDrop(t);const e=t.mimeData.getData("text/vnd.maven.target"),n=this.owner.getRegion(e),r=u.Rectangle.get(n),i=this.node.getBoundingClientRect(),o=[i.width,i.height];let[a,l,c,g]=r.split(",").map(t=>+t),h=[t.clientX,t.clientY];const d=t.mimeData.getData("text/vnd.maven.startPosition");if(this.widgets.includes(n)){const t=A.Sub(h,d);[a,l]=A.Add([a,l],t)}else n.parentRegion,[a,l]=A.Sub(h,[i.left,i.top]);[a,l,c,g]=s.MathTools.ClampRectToBounds([a,l],[c,g],o),u.Rectangle.set(n,`${a},${l},${c},${g}`),this.addChild(n),t.preventDefault(),t.stopPropagation(),t.dropAction="move"}}u.Rectangle=new l.AttachedProperty({create:()=>"0,0,100,100",name:"Canvas Panel.Rectangle",changed:t=>{t.setStale(),null!=t.parentRegion&&(r.MessageLoop.sendMessage(t.parentRegion,i.Widget.Msg.FitRequest),r.MessageLoop.sendMessage(t.parentRegion,i.Widget.Msg.UpdateRequest))}}),e.CanvasLayoutRegion=u},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(108),i=n(28);class o{constructor(){this._length=0,this._rows=[],this._columnNames=[],this._columnTypes=[],this._columnLookup={}}static copyTable(t){return t.copyTable()}static CopyTable(t){return t.copyTable()}static NullTable(){const t=new o;t.setColumns(["null"],[i.Types.Any]);const e=t.createRow(null);return e.setValue("null",null),t.appendRow(e),t}static _copyRowAndChildren(t,e,n){let r=t.createRow(n.name);for(let e=0;e<t.columnNames.length;e++)r.setValue(e,n.getValue(e));null!=e?e.appendChild(n):t.appendRow(r);for(let e=0;e<n.children.length;e++)o._copyRowAndChildren(t,r,n.children[e])}get columnNames(){return this._columnNames}get columnTypes(){return this._columnTypes}get rows(){return this._rows}get length(){return this._length}get Cols(){return this._columnNames}get Rows(){return a.toObjectArray(this)}appendRow(t){this._rows.push(t),this._length+=1,t._setParent(null)}insertRow(t,e){this._rows.splice(e,0,t),this._length+=1,t._setParent(null)}createRow(t){return new s(this,t)}setColumns(t,e){this._columnTypes=null==e?new Array(t.length).fill(i.Types.Any):e.slice(0),this._columnNames=t.slice(0),this.createColumnLookup(),this.ensureRowDataLength(t.length),this.ensureRowDataType()}appendColumn(t,e=i.Types.Any){return this._columnNames.push(t),this._columnTypes.push(e),this.createColumnLookup(),this.ensureRowDataLength(this._columnNames.length),this.ensureRowDataType(),this._columnNames.length-1}dropInvalidTypes(){for(const t of i.IterTools.dfs_iter(this.rows,t=>t.children))for(let e=0;e<this.columnTypes.length;e++){const n=this.columnTypes[e];i.Converters.isValid(t.getValue(e),n)||t.setValue(e,null)}}copyTable(){let t=new o;t.setColumns(this.columnNames,this.columnTypes);for(let e=0;e<this.rows.length;e++)o._copyRowAndChildren(t,null,this.rows[e]);return t}CopyTable(){return this.copyTable()}toString(){return"[object Table]"}ensureRowDataLength(t){r.forEach(this._rows,e=>e._ensureRowDataLength(t))}ensureRowDataType(){r.forEach(this._rows,t=>t._ensureRowDataType(this.columnTypes))}createColumnLookup(){this._columnLookup=r.reduceRight(this._columnNames,(t,e,n)=>(t[e]=n,t),{})}}e.Table=o;class a{static fromObjectArray(t,e=!0){let n=new o;if(t.length<1)return n;let a=new Array(Object.keys(t[0]).length).fill(i.Types.Any),s=t[0],A=r.keys(s);e&&(a=A.map(t=>i.Converters.inferType(s[t]))),n.setColumns(A,a);let l={};return r.forEach(A,(t,e)=>{l[t]=e}),r.forEach(t,t=>{let e=n.createRow(null);r.forEach(t,(t,n)=>{let r=l[n];if(void 0===r)throw new Error("Invalid input table");e.setValue(r,t)}),n.appendRow(e)}),n}static toObjectArray(t){return r.map(t.rows,e=>r.transform(t.columnNames,(t,n,r)=>{t[n]=e.getValue(r)},{}))}static fromMatrixObject(t,e){let n=new o,a=t[0].map(t=>i.Converters.inferType(t));return n.setColumns(e,a),t.length<1?n:(r.forEach(t,t=>{let e=n.createRow(null);r.forEach(t,(t,n)=>{e.setValue(n,t)}),n.appendRow(e)}),n)}static toMatrixObject(t){return{Cols:t.columnNames,Rows:r.map(t.rows,e=>r.map(t.columnNames,(t,n)=>e.getValue(n)))}}static flattenTable(t){let e=new o;e.setColumns(t.columnNames);let n=a.depthFirstWalk(t.rows,t=>t.children);return r.forEach(n,t=>{let n=e.createRow(t.name);e.columnNames.forEach((e,r)=>n.setValue(r,t.getValue(r))),e.appendRow(n)}),e}static getRowPath(t){let e="",n=t;for(;null!=n;)e="/"+n.name+e,n=n.parent;return"/root"+e}static toMQLResultTable(t){let e=a.depthFirstWalk(t.rows,t=>t.children);return{Columns:t.columnNames.map(t=>({Name:t})),Result:{Rows:r.map(e,t=>({Depth:t.level,Name:t.name,RowData:t._data,Path:a.getRowPath(t)}))}}}static fromMQLResultTable(t,e=new o){let n=e;if(n.setColumns(t.Columns.map(t=>t.Name)),t.Result.Rows.length<1)return n;let r=[],i=t.Result.Rows[0].Depth;for(let e=0;e<t.Result.Rows.length;e++){let o=t.Result.Rows[e],a=o.Depth-i;if(a<0)throw new Error("Unsupported table depth structure.");let s=n.createRow(o.Name);for(let t=0;t<o.RowData.length;t++)s.setValue(t,o.RowData[t]);let A=r.length>0?r[r.length-1]:null;null!=A?A.appendChild(s):n.appendRow(s);let l=e+1<t.Result.Rows.length?t.Result.Rows[e+1]:null;if(null!=l&&l.Depth-i>a)r.push(s);else if(null!=l&&l.Depth-i<a)for(;r.length>0&&l.Depth-i<=r[r.length-1].level;)r.pop()}return n}static IndexTable(t,e){let n=t.columnNames.indexOf(e);return t.rows.reduce((t,e)=>{let r=e.getValue(n);return t.hasOwnProperty(r)?t[r].push(e.columns):(t[r]=[],t[r].push(e.columns)),t},{})}static Join(t,e,n,r,i){let o={Rows:a.toObjectArray(t),Cols:t.columnNames},s={Rows:a.toObjectArray(e),Cols:e.columnNames};n===A.Union&&a.Union(t,e,!1);let l=n===A.LeftOuterJoin,c=n===A.RightOuterJoin;n===A.FullOuterJoin&&(l=!0,c=!0);const u=a.IndexTable(e,i);let g=o.Cols.reduce((t,e)=>(t[e]=!0,t),{}),h=o.Cols.concat(s.Cols.map(t=>{let e=t;for(;g.hasOwnProperty(e);)e+="Copy";return g[e]=!0,e})),d=[];for(let t of o.Rows){let e=t[r];if(u.hasOwnProperty(e)){let n=u[e];for(let e in n)if("used"!==e){let r=[];for(let e in t)r.push(t[e]);for(let t in n[e])r.push(n[e][t]);d.push(r)}n.used=!0}else if(l){let e=[];for(let n in t)e.push(t[n]);h.forEach(t=>{-1===o.Cols.indexOf(t)&&e.push(null)}),d.push(e)}}if(c)for(let t in u)if(!u[t].used)for(let e in u[t]){let n=[];o.Cols.forEach(()=>{n.push(null)});for(let r in u[t][e])n.push(u[t][e][r]);d.push(n)}return a.fromMatrixObject(d,h)}static Union(t,e,n){let r={Rows:a.toObjectArray(t),Cols:t.columnNames},i={Rows:a.toObjectArray(e),Cols:e.columnNames};if(n){if(i.Cols.length===r.Cols.length&&i.Cols.reduce((t,e)=>t&&-1!==r.Cols.indexOf(e),!0)){let t=[];for(let e of r.Rows)t.push(e);for(let e of i.Rows)t.push(e);return a.fromObjectArray(t)}throw TypeError("Cannot join tables without same columns when strict is true")}{let t=[],e=r.Cols.concat(i.Cols.map(t=>-1!==r.Cols.indexOf(t)?t+"Copy":t));for(let e of r.Rows){let n=[];for(let t in e)n.push(e[t]);i.Cols.forEach(()=>{n.push(null)}),t.push(n)}for(let e of i.Rows){let n=[];r.Cols.forEach(()=>{n.push(null)});for(let t in e)n.push(e[t]);t.push(n)}return a.fromMatrixObject(t,e)}}static depthFirstWalk(t,e){let n=[];return a._depthFirstWalk(n,t,e),n}static _depthFirstWalk(t,e,n){Array.isArray(e)?r.forEach(e,e=>a._depthFirstWalk(t,e,n)):(t.push(e),r.forEach(n(e),e=>{a._depthFirstWalk(t,e,n)}))}}e.TableHelper=a;class s{constructor(t,e){this._parent=null,this._rowData=new Array(t.columnNames.length),this._children=[],this._name=""+e,this._owner=t}get parent(){return this._parent}get owner(){return this._owner}get children(){return this._children}get name(){return this._name}get columns(){return Object.assign({},...this._owner.columnNames.map((t,e)=>({[t]:this.getValue(e)})))}get level(){return null==this._parent?0:this._parent.level+1}setValue(t,e){let n=this._indexOrNameToIndex(t),r=this.owner.columnTypes[n];if(r!==i.Types.Any&&!i.Converters.isValid(e,r))throw Error("Data type mismatch: "+r.serializableName+" is not compatible with "+e);this._rowData[n]=e}setName(t){this._name=t}getValue(t){return this._rowData[this._indexOrNameToIndex(t)]}appendChild(t){this._children.push(t),this._owner._length+=1,t._setParent(this)}insertChild(t,e){this._children.splice(e,0,t),this._owner._length+=1,t._setParent(this)}_ensureRowDataLength(t){for(;t>this._rowData.length;)this._rowData.push(null);this._rowData.length=t}_ensureRowDataType(t){console.assert(this._rowData.every((e,n)=>null==e||e===i.Types.Any||i.Converters.isValid(e,t[n])),"Data type mismatch in table!"),this.children&&this.children.map(e=>e._ensureRowDataType(t))}_setParent(t){this._parent=t}cloneToTable(t){let e=t.createRow(this.name);return t.appendRow(e),e._rowData=this._rowData.map(t=>t),e}get _data(){return this._rowData}_indexOrNameToIndex(t){let e=-1;if("number"==typeof t)e=t;else if(void 0===(e=this._owner._columnLookup[t.toString()]))throw new Error("Column name not found: "+t);if(e<0||e>=this._owner.columnNames.length)throw new Error("Table.Row Index out of bounds: "+e);return e}}var A;e.Row=s,function(t){t[t.InnerJoin=0]="InnerJoin",t[t.LeftOuterJoin=1]="LeftOuterJoin",t[t.RightOuterJoin=2]="RightOuterJoin",t[t.Union=3]="Union",t[t.FullOuterJoin=4]="FullOuterJoin"}(A=e.JoinType||(e.JoinType={}))},function(t,e,n){var r=n(147)(n(99),"Map");t.exports=r},function(t,e,n){var r=n(99).Symbol;t.exports=r},function(t,e){var n=Array.isArray;t.exports=n},function(t,e,n){"use strict";t.exports={BADNUM:void 0,FP_SAFE:Number.MAX_VALUE/1e4,ONEAVGYEAR:315576e5,ONEAVGMONTH:26298e5,ONEDAY:864e5,ONEHOUR:36e5,ONEMIN:6e4,ONESEC:1e3,EPOCHJD:2440587.5,ALMOST_EQUAL:1-1e-6,LOG_CLIP:10,MINUS_SIGN:"−"}},function(t,e,n){"use strict";var r=n(100),i=n(186).isArrayOrTypedArray;function o(t,e){return function(){var n,r,a,s,A,l=t;for(s=0;s<e.length-1;s++){if(-1===(n=e[s])){for(r=!0,a=[],A=0;A<l.length;A++)a[A]=o(l[A],e.slice(s+1))(),a[A]!==a[0]&&(r=!1);return r?a[0]:a}if("number"==typeof n&&!i(l))return;if("object"!=typeof(l=l[n])||null===l)return}if("object"==typeof l&&null!==l&&null!==(a=l[e[s]]))return a}}t.exports=function(t,e){if(r(e))e=String(e);else if("string"!=typeof e||"[-1]"===e.substr(e.length-4))throw"bad property string";for(var n,i,a,s=0,l=e.split(".");s<l.length;){if(n=String(l[s]).match(/^([^\[\]]*)((\[\-?[0-9]*\])+)$/)){if(n[1])l[s]=n[1];else{if(0!==s)throw"bad property string";l.splice(0,1)}for(i=n[2].substr(1,n[2].length-2).split("]["),a=0;a<i.length;a++)s++,l.splice(s,0,Number(i[a]))}s++}return"object"!=typeof t?function(t,e,n){return{set:function(){throw"bad container"},get:function(){},astr:e,parts:n,obj:t}}(t,e,l):{set:A(t,l,e),get:o(t,l),astr:e,parts:l,obj:t}};var a=/(^|\.)args\[/;function s(t,e){return void 0===t||null===t&&!e.match(a)}function A(t,e,n){return function(r){var o,a,A=t,g="",h=[[t,g]],d=s(r,n);for(a=0;a<e.length-1;a++){if("number"==typeof(o=e[a])&&!i(A))throw"array index but container is not an array";if(-1===o){if(d=!c(A,e.slice(a+1),r,n))break;return}if(!u(A,o,e[a+1],d))break;if("object"!=typeof(A=A[o])||null===A)throw"container is not an object";g=l(g,o),h.push([A,g])}if(d){if(a===e.length-1&&(delete A[e[a]],Array.isArray(A)&&+e[a]==A.length-1))for(;A.length&&void 0===A[A.length-1];)A.pop()}else A[e[a]]=r}}function l(t,e){var n=e;return r(e)?n="["+e+"]":t&&(n="."+e),t+n}function c(t,e,n,r){var o,a=i(n),l=!0,c=n,g=r.replace("-1",0),h=!a&&s(n,g),d=e[0];for(o=0;o<t.length;o++)g=r.replace("-1",o),a&&(h=s(c=n[o%n.length],g)),h&&(l=!1),u(t,o,d,h)&&A(t[o],e,r.replace("-1",o))(c);return l}function u(t,e,n,r){if(void 0===t[e]){if(r)return!1;t[e]="number"==typeof n?[]:{}}return!0}},function(t,e,n){"use strict";t.exports=function(t){var e=t.editType,n=t.colorEditType;void 0===n&&(n=e);var r={family:{valType:"string",role:"style",noBlank:!0,strict:!0,editType:e,description:["HTML font family - the typeface that will be applied by the web browser.","The web browser will only be able to apply a font if it is available on the system","which it operates. Provide multiple font families, separated by commas, to indicate","the preference in which to apply fonts if they aren't available on the system.","The plotly service (at https://plot.ly or on-premise) generates images on a server,","where only a select number of","fonts are installed and supported.","These include *Arial*, *Balto*, *Courier New*, *Droid Sans*,, *Droid Serif*,","*Droid Sans Mono*, *Gravitas One*, *Old Standard TT*, *Open Sans*, *Overpass*,","*PT Sans Narrow*, *Raleway*, *Times New Roman*."].join(" ")},size:{valType:"number",role:"style",min:1,editType:e},color:{valType:"color",role:"style",editType:n},editType:e,description:""+(t.description||"")};return t.arrayOk&&(r.family.arrayOk=!0,r.size.arrayOk=!0,r.color.arrayOk=!0),r}},function(t,e,n){"use strict";n.r(e);var r=n(13),i=n.n(r),o=n(18),a=n.n(o),s=n(56),A=n.n(s),l=n(14),c=n.n(l),u=n(19),g=n.n(u),h=n(0),d=n.n(h),f=n(1),p=n.n(f),I=n(80),C=n.n(I),m=function(t){var e=t.className,n=t.included,r=t.vertical,o=t.offset,a=t.length,s=t.style,A=r?{bottom:o+"%",height:a+"%"}:{left:o+"%",width:a+"%"},l=i()({},s,A);return n?d.a.createElement("div",{className:e,style:l}):null},y=n(57),E=n.n(y),v=n(47),B=n.n(v),b=n(266),Q=n.n(b),w=n(70),x=n(4),_=n.n(x),D=function(t){var e=t.prefixCls,n=t.vertical,r=t.marks,o=t.dots,a=t.step,s=t.included,A=t.lowerBound,l=t.upperBound,c=t.max,u=t.min,g=t.dotStyle,h=t.activeDotStyle,f=c-u,p=function(t,e,n,r,i,o){C()(!n||r>0,"`Slider[step]` should be a positive number in order to make Slider[dots] work.");var a=Object.keys(e).map(parseFloat).sort(function(t,e){return t-e});if(n&&r)for(var s=i;s<=o;s+=r)-1===a.indexOf(s)&&a.push(s);return a}(0,r,o,a,u,c).map(function(t){var r,o=Math.abs(t-u)/f*100+"%",a=!s&&t===l||s&&t<=l&&t>=A,c=n?i()({bottom:o},g):i()({left:o},g);a&&(c=i()({},c,h));var p=_()((r={},B()(r,e+"-dot",!0),B()(r,e+"-dot-active",a),r));return d.a.createElement("span",{className:p,style:c,key:t})});return d.a.createElement("div",{className:e+"-step"},p)};D.propTypes={prefixCls:p.a.string,activeDotStyle:p.a.object,dotStyle:p.a.object,min:p.a.number,max:p.a.number,upperBound:p.a.number,lowerBound:p.a.number,included:p.a.bool,dots:p.a.bool,step:p.a.number,marks:p.a.object,vertical:p.a.bool};var M=D,k=function(t){var e=t.className,n=t.vertical,r=t.marks,o=t.included,a=t.upperBound,s=t.lowerBound,A=t.max,l=t.min,c=t.onClickLabel,u=Object.keys(r),g=A-l,h=u.map(parseFloat).sort(function(t,e){return t-e}).map(function(t){var A,u=r[t],h="object"==typeof u&&!d.a.isValidElement(u),f=h?u.label:u;if(!f&&0!==f)return null;var p=!o&&t===a||o&&t<=a&&t>=s,I=_()((A={},B()(A,e+"-text",!0),B()(A,e+"-text-active",p),A)),C=n?{marginBottom:"-50%",bottom:(t-l)/g*100+"%"}:{left:(t-l)/g*100+"%",transform:"translateX(-50%)",msTransform:"translateX(-50%)"},m=h?i()({},C,u.style):C;return d.a.createElement("span",{className:I,style:m,key:t,onMouseDown:function(e){return c(e,t)},onTouchStart:function(e){return c(e,t)}},f)});return d.a.createElement("div",{className:e},h)};k.propTypes={className:p.a.string,vertical:p.a.bool,marks:p.a.object,included:p.a.bool,upperBound:p.a.number,lowerBound:p.a.number,max:p.a.number,min:p.a.number,onClickLabel:p.a.func};var S=k,L=function(t){function e(){var t,n,r,i;a()(this,e);for(var o=arguments.length,s=Array(o),A=0;A<o;A++)s[A]=arguments[A];return n=r=c()(this,(t=e.__proto__||Object.getPrototypeOf(e)).call.apply(t,[this].concat(s))),r.state={clickFocused:!1},r.setHandleRef=function(t){r.handle=t},r.handleMouseUp=function(){document.activeElement===r.handle&&r.setClickFocus(!0)},r.handleMouseDown=function(){r.focus()},r.handleBlur=function(){r.setClickFocus(!1)},r.handleKeyDown=function(){r.setClickFocus(!1)},i=n,c()(r,i)}return g()(e,t),A()(e,[{key:"componentDidMount",value:function(){this.onMouseUpListener=Object(w.a)(document,"mouseup",this.handleMouseUp)}},{key:"componentWillUnmount",value:function(){this.onMouseUpListener&&this.onMouseUpListener.remove()}},{key:"setClickFocus",value:function(t){this.setState({clickFocused:t})}},{key:"clickFocus",value:function(){this.setClickFocus(!0),this.focus()}},{key:"focus",value:function(){this.handle.focus()}},{key:"blur",value:function(){this.handle.blur()}},{key:"render",value:function(){var t=this.props,e=t.prefixCls,n=t.vertical,r=t.offset,o=t.style,a=t.disabled,s=t.min,A=t.max,l=t.value,c=t.tabIndex,u=E()(t,["prefixCls","vertical","offset","style","disabled","min","max","value","tabIndex"]),g=_()(this.props.className,B()({},e+"-handle-click-focused",this.state.clickFocused)),h=n?{bottom:r+"%"}:{left:r+"%"},f=i()({},o,h),p=c||0;return(a||null===c)&&(p=null),d.a.createElement("div",i()({ref:this.setHandleRef,tabIndex:p},u,{className:g,style:f,onBlur:this.handleBlur,onKeyDown:this.handleKeyDown,onMouseDown:this.handleMouseDown,role:"slider","aria-valuemin":s,"aria-valuemax":A,"aria-valuenow":l,"aria-disabled":!!a}))}}]),e}(d.a.Component),R=L;L.propTypes={prefixCls:p.a.string,className:p.a.string,vertical:p.a.bool,offset:p.a.number,style:p.a.object,disabled:p.a.bool,min:p.a.number,max:p.a.number,value:p.a.number,tabIndex:p.a.number};var N=n(163),F=n.n(N),T=n(16),U=n(31);function O(t,e){try{return Object.keys(e).some(function(n){return t.target===Object(T.findDOMNode)(e[n])})}catch(t){return!1}}function G(t,e){var n=e.min,r=e.max;return t<n||t>r}function H(t){return t.touches.length>1||"touchend"===t.type.toLowerCase()&&t.touches.length>0}function j(t,e){var n=e.marks,r=e.step,i=e.min,o=e.max,a=Object.keys(n).map(parseFloat);if(null!==r){var s=Math.floor((o-i)/r),A=Math.min((t-i)/r,s),l=Math.round(A)*r+i;a.push(l)}var c=a.map(function(e){return Math.abs(t-e)});return a[c.indexOf(Math.min.apply(Math,F()(c)))]}function Y(t,e){return t?e.clientY:e.pageX}function P(t,e){return t?e.touches[0].clientY:e.touches[0].pageX}function K(t,e){var n=e.getBoundingClientRect();return t?n.top+.5*n.height:window.pageXOffset+n.left+.5*n.width}function z(t,e){var n=e.max,r=e.min;return t<=r?r:t>=n?n:t}function J(t,e){var n=e.step,r=isFinite(j(t,e))?j(t,e):0;return null===n?r:parseFloat(r.toFixed(function(t){var e=t.toString(),n=0;return e.indexOf(".")>=0&&(n=e.length-e.indexOf(".")-1),n}(n)))}function q(t){t.stopPropagation(),t.preventDefault()}function V(t,e,n){var r={increase:function(t,e){return t+e},decrease:function(t,e){return t-e}},i=r[t](Object.keys(n.marks).indexOf(JSON.stringify(e)),1),o=Object.keys(n.marks)[i];return n.step?r[t](e,n.step):Object.keys(n.marks).length&&n.marks[o]?n.marks[o]:e}function W(t){switch(t.keyCode){case U.a.UP:case U.a.RIGHT:return function(t,e){return V("increase",t,e)};case U.a.DOWN:case U.a.LEFT:return function(t,e){return V("decrease",t,e)};case U.a.END:return function(t,e){return e.max};case U.a.HOME:return function(t,e){return e.min};case U.a.PAGE_UP:return function(t,e){return t+2*e.step};case U.a.PAGE_DOWN:return function(t,e){return t-2*e.step};default:return}}function X(){}function Z(t){var e,n;return n=e=function(t){function e(t){a()(this,e);var n=c()(this,(e.__proto__||Object.getPrototypeOf(e)).call(this,t));return n.onMouseDown=function(t){if(0===t.button){var e=n.props.vertical,r=Y(e,t);if(O(t,n.handlesRefs)){var i=K(e,t.target);n.dragOffset=r-i,r=i}else n.dragOffset=0;n.removeDocumentEvents(),n.onStart(r),n.addDocumentMouseEvents()}},n.onTouchStart=function(t){if(!H(t)){var e=n.props.vertical,r=P(e,t);if(O(t,n.handlesRefs)){var i=K(e,t.target);n.dragOffset=r-i,r=i}else n.dragOffset=0;n.onStart(r),n.addDocumentTouchEvents(),q(t)}},n.onFocus=function(t){var e=n.props,r=e.onFocus,i=e.vertical;if(O(t,n.handlesRefs)){var o=K(i,t.target);n.dragOffset=0,n.onStart(o),q(t),r&&r(t)}},n.onBlur=function(t){var e=n.props.onBlur;n.onEnd(),e&&e(t)},n.onMouseUp=function(){n.handlesRefs[n.prevMovedHandleIndex]&&n.handlesRefs[n.prevMovedHandleIndex].clickFocus()},n.onMouseMove=function(t){if(n.sliderRef){var e=Y(n.props.vertical,t);n.onMove(t,e-n.dragOffset)}else n.onEnd()},n.onTouchMove=function(t){if(!H(t)&&n.sliderRef){var e=P(n.props.vertical,t);n.onMove(t,e-n.dragOffset)}else n.onEnd()},n.onKeyDown=function(t){n.sliderRef&&O(t,n.handlesRefs)&&n.onKeyboard(t)},n.onClickMarkLabel=function(t,e){t.stopPropagation(),n.onChange({value:e}),n.setState({value:e},function(){return n.onEnd(!0)})},n.saveSlider=function(t){n.sliderRef=t},n.handlesRefs={},n}return g()(e,t),A()(e,[{key:"componentDidMount",value:function(){this.document=this.sliderRef&&this.sliderRef.ownerDocument;var t=this.props,e=t.autoFocus,n=t.disabled;e&&!n&&this.focus()}},{key:"componentWillUnmount",value:function(){Q()(e.prototype.__proto__||Object.getPrototypeOf(e.prototype),"componentWillUnmount",this)&&Q()(e.prototype.__proto__||Object.getPrototypeOf(e.prototype),"componentWillUnmount",this).call(this),this.removeDocumentEvents()}},{key:"getSliderStart",value:function(){var t=this.sliderRef.getBoundingClientRect();return this.props.vertical?t.top:t.left+window.pageXOffset}},{key:"getSliderLength",value:function(){var t=this.sliderRef;if(!t)return 0;var e=t.getBoundingClientRect();return this.props.vertical?e.height:e.width}},{key:"addDocumentTouchEvents",value:function(){this.onTouchMoveListener=Object(w.a)(this.document,"touchmove",this.onTouchMove),this.onTouchUpListener=Object(w.a)(this.document,"touchend",this.onEnd)}},{key:"addDocumentMouseEvents",value:function(){this.onMouseMoveListener=Object(w.a)(this.document,"mousemove",this.onMouseMove),this.onMouseUpListener=Object(w.a)(this.document,"mouseup",this.onEnd)}},{key:"removeDocumentEvents",value:function(){this.onTouchMoveListener&&this.onTouchMoveListener.remove(),this.onTouchUpListener&&this.onTouchUpListener.remove(),this.onMouseMoveListener&&this.onMouseMoveListener.remove(),this.onMouseUpListener&&this.onMouseUpListener.remove()}},{key:"focus",value:function(){this.props.disabled||this.handlesRefs[0].focus()}},{key:"blur",value:function(){var t=this;this.props.disabled||Object.keys(this.handlesRefs).forEach(function(e){t.handlesRefs[e]&&t.handlesRefs[e].blur&&t.handlesRefs[e].blur()})}},{key:"calcValue",value:function(t){var e=this.props,n=e.vertical,r=e.min,i=e.max,o=Math.abs(Math.max(t,0)/this.getSliderLength());return n?(1-o)*(i-r)+r:o*(i-r)+r}},{key:"calcValueByPos",value:function(t){var e=t-this.getSliderStart();return this.trimAlignValue(this.calcValue(e))}},{key:"calcOffset",value:function(t){var e=this.props,n=e.min;return 100*((t-n)/(e.max-n))}},{key:"saveHandle",value:function(t,e){this.handlesRefs[t]=e}},{key:"render",value:function(){var t,n=this.props,r=n.prefixCls,o=n.className,a=n.marks,s=n.dots,A=n.step,l=n.included,c=n.disabled,u=n.vertical,g=n.min,h=n.max,f=n.children,p=n.maximumTrackStyle,I=n.style,C=n.railStyle,m=n.dotStyle,y=n.activeDotStyle,E=Q()(e.prototype.__proto__||Object.getPrototypeOf(e.prototype),"render",this).call(this),v=E.tracks,b=E.handles,w=_()(r,(t={},B()(t,r+"-with-marks",Object.keys(a).length),B()(t,r+"-disabled",c),B()(t,r+"-vertical",u),B()(t,o,o),t));return d.a.createElement("div",{ref:this.saveSlider,className:w,onTouchStart:c?X:this.onTouchStart,onMouseDown:c?X:this.onMouseDown,onMouseUp:c?X:this.onMouseUp,onKeyDown:c?X:this.onKeyDown,onFocus:c?X:this.onFocus,onBlur:c?X:this.onBlur,style:I},d.a.createElement("div",{className:r+"-rail",style:i()({},p,C)}),v,d.a.createElement(M,{prefixCls:r,vertical:u,marks:a,dots:s,step:A,included:l,lowerBound:this.getLowerBound(),upperBound:this.getUpperBound(),max:h,min:g,dotStyle:m,activeDotStyle:y}),b,d.a.createElement(S,{className:r+"-mark",onClickLabel:c?X:this.onClickMarkLabel,vertical:u,marks:a,included:l,lowerBound:this.getLowerBound(),upperBound:this.getUpperBound(),max:h,min:g}),f)}}]),e}(t),e.displayName="ComponentEnhancer("+t.displayName+")",e.propTypes=i()({},t.propTypes,{min:p.a.number,max:p.a.number,step:p.a.number,marks:p.a.object,included:p.a.bool,className:p.a.string,prefixCls:p.a.string,disabled:p.a.bool,children:p.a.any,onBeforeChange:p.a.func,onChange:p.a.func,onAfterChange:p.a.func,handle:p.a.func,dots:p.a.bool,vertical:p.a.bool,style:p.a.object,minimumTrackStyle:p.a.object,maximumTrackStyle:p.a.object,handleStyle:p.a.oneOfType([p.a.object,p.a.arrayOf(p.a.object)]),trackStyle:p.a.oneOfType([p.a.object,p.a.arrayOf(p.a.object)]),railStyle:p.a.object,dotStyle:p.a.object,activeDotStyle:p.a.object,autoFocus:p.a.bool,onFocus:p.a.func,onBlur:p.a.func}),e.defaultProps=i()({},t.defaultProps,{prefixCls:"rc-slider",className:"",min:0,max:100,step:1,marks:{},handle:function(t){var e=t.index,n=E()(t,["index"]);return delete n.dragging,null===n.value?null:d.a.createElement(R,i()({},n,{key:e}))},onBeforeChange:X,onChange:X,onAfterChange:X,included:!0,disabled:!1,dots:!1,vertical:!1,trackStyle:[{}],handleStyle:[{}],railStyle:{},dotStyle:{},activeDotStyle:{}}),n}var $=function(t){function e(t){a()(this,e);var n=c()(this,(e.__proto__||Object.getPrototypeOf(e)).call(this,t));n.onEnd=function(t){var e=n.state.dragging;n.removeDocumentEvents(),(e||t)&&n.props.onAfterChange(n.getValue()),n.setState({dragging:!1})};var r=void 0!==t.defaultValue?t.defaultValue:t.min,i=void 0!==t.value?t.value:r;return n.state={value:n.trimAlignValue(i),dragging:!1},n}return g()(e,t),A()(e,[{key:"componentWillReceiveProps",value:function(t){if("value"in t||"min"in t||"max"in t){var e=this.state.value,n=void 0!==t.value?t.value:e,r=this.trimAlignValue(n,t);r!==e&&(this.setState({value:r}),G(n,t)&&this.props.onChange(r))}}},{key:"onChange",value:function(t){var e=this.props,n=!("value"in e),r=t.value>this.props.max?i()({},t,{value:this.props.max}):t;n&&this.setState(r);var o=r.value;e.onChange(o)}},{key:"onStart",value:function(t){this.setState({dragging:!0});var e=this.props,n=this.getValue();e.onBeforeChange(n);var r=this.calcValueByPos(t);this.startValue=r,this.startPosition=t,r!==n&&(this.prevMovedHandleIndex=0,this.onChange({value:r}))}},{key:"onMove",value:function(t,e){q(t);var n=this.state.value,r=this.calcValueByPos(e);r!==n&&this.onChange({value:r})}},{key:"onKeyboard",value:function(t){var e=W(t);if(e){q(t);var n=this.state.value,r=e(n,this.props),i=this.trimAlignValue(r);if(i===n)return;this.onChange({value:i}),this.props.onAfterChange(i),this.onEnd()}}},{key:"getValue",value:function(){return this.state.value}},{key:"getLowerBound",value:function(){return this.props.min}},{key:"getUpperBound",value:function(){return this.state.value}},{key:"trimAlignValue",value:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(null===t)return null;var n=i()({},this.props,e),r=z(t,n);return J(r,n)}},{key:"render",value:function(){var t=this,e=this.props,n=e.prefixCls,r=e.vertical,o=e.included,a=e.disabled,s=e.minimumTrackStyle,A=e.trackStyle,l=e.handleStyle,c=e.tabIndex,u=e.min,g=e.max,h=e.handle,f=this.state,p=f.value,I=f.dragging,C=this.calcOffset(p),y=h({className:n+"-handle",prefixCls:n,vertical:r,offset:C,value:p,dragging:I,disabled:a,min:u,max:g,index:0,tabIndex:c,style:l[0]||l,ref:function(e){return t.saveHandle(0,e)}}),E=A[0]||A;return{tracks:d.a.createElement(m,{className:n+"-track",vertical:r,included:o,offset:0,length:C,style:i()({},s,E)}),handles:y}}}]),e}(d.a.Component);$.propTypes={defaultValue:p.a.number,value:p.a.number,disabled:p.a.bool,autoFocus:p.a.bool,tabIndex:p.a.number,min:p.a.number,max:p.a.number};var tt=Z($),et=n(385),nt=n.n(et),rt=function(t){function e(t){a()(this,e);var n=c()(this,(e.__proto__||Object.getPrototypeOf(e)).call(this,t));n.onEnd=function(t){var e=n.state.handle;n.removeDocumentEvents(),(null!==e||t)&&n.props.onAfterChange(n.getValue()),n.setState({handle:null})};var r=t.count,i=t.min,o=t.max,s=Array.apply(void 0,F()(Array(r+1))).map(function(){return i}),A="defaultValue"in t?t.defaultValue:s,l=(void 0!==t.value?t.value:A).map(function(t,e){return n.trimAlignValue(t,e)}),u=l[0]===o?0:l.length-1;return n.state={handle:null,recent:u,bounds:l},n}return g()(e,t),A()(e,[{key:"componentWillReceiveProps",value:function(t){var e=this;if(("value"in t||"min"in t||"max"in t)&&(this.props.min!==t.min||this.props.max!==t.max||!nt()(this.props.value,t.value))){var n=this.state.bounds,r=t.value||n,i=r.map(function(n,r){return e.trimAlignValue(n,r,t)});if((i.length!==n.length||!i.every(function(t,e){return t===n[e]}))&&(this.setState({bounds:i}),r.some(function(e){return G(e,t)}))){var o=r.map(function(e){return z(e,t)});this.props.onChange(o)}}}},{key:"onChange",value:function(t){var e=this.props;if(!("value"in e))this.setState(t);else{var n={};["handle","recent"].forEach(function(e){void 0!==t[e]&&(n[e]=t[e])}),Object.keys(n).length&&this.setState(n)}var r=i()({},this.state,t).bounds;e.onChange(r)}},{key:"onStart",value:function(t){var e=this.props,n=this.state,r=this.getValue();e.onBeforeChange(r);var i=this.calcValueByPos(t);this.startValue=i,this.startPosition=t;var o=this.getClosestBound(i);if(this.prevMovedHandleIndex=this.getBoundNeedMoving(i,o),this.setState({handle:this.prevMovedHandleIndex,recent:this.prevMovedHandleIndex}),i!==r[this.prevMovedHandleIndex]){var a=[].concat(F()(n.bounds));a[this.prevMovedHandleIndex]=i,this.onChange({bounds:a})}}},{key:"onMove",value:function(t,e){q(t);var n=this.state,r=this.calcValueByPos(e);r!==n.bounds[n.handle]&&this.moveTo(r)}},{key:"onKeyboard",value:function(t){var e=W(t);if(e){q(t);var n=this.state,r=this.props,i=n.bounds,o=n.handle,a=i[null===o?n.recent:o],s=e(a,r),A=this.trimAlignValue(s);if(A===a)return;this.moveTo(A,!0)}}},{key:"getValue",value:function(){return this.state.bounds}},{key:"getClosestBound",value:function(t){for(var e=this.state.bounds,n=0,r=1;r<e.length-1;++r)t>=e[r]&&(n=r);return Math.abs(e[n+1]-t)<Math.abs(e[n]-t)&&(n+=1),n}},{key:"getBoundNeedMoving",value:function(t,e){var n=this.state,r=n.bounds,i=n.recent,o=e,a=r[e+1]===r[e];return a&&r[i]===r[e]&&(o=i),a&&t!==r[e+1]&&(o=t<r[e+1]?e:e+1),o}},{key:"getLowerBound",value:function(){return this.state.bounds[0]}},{key:"getUpperBound",value:function(){var t=this.state.bounds;return t[t.length-1]}},{key:"getPoints",value:function(){var t=this.props,e=t.marks,n=t.step,r=t.min,o=t.max,a=this._getPointsCache;if(!a||a.marks!==e||a.step!==n){var s=i()({},e);if(null!==n)for(var A=r;A<=o;A+=n)s[A]=A;var l=Object.keys(s).map(parseFloat);l.sort(function(t,e){return t-e}),this._getPointsCache={marks:e,step:n,points:l}}return this._getPointsCache.points}},{key:"moveTo",value:function(t,e){var n=this,r=this.state,i=this.props,o=[].concat(F()(r.bounds)),a=null===r.handle?r.recent:r.handle;o[a]=t;var s=a;!1!==i.pushable?this.pushSurroundingHandles(o,s):i.allowCross&&(o.sort(function(t,e){return t-e}),s=o.indexOf(t)),this.onChange({recent:s,handle:s,bounds:o}),e&&(this.props.onAfterChange(o),this.setState({},function(){n.handlesRefs[s].focus()}),this.onEnd())}},{key:"pushSurroundingHandles",value:function(t,e){var n=t[e],r=this.props.pushable;r=Number(r);var i=0;if(t[e+1]-n<r&&(i=1),n-t[e-1]<r&&(i=-1),0!==i){var o=e+i,a=i*(t[o]-n);this.pushHandle(t,o,i,r-a)||(t[e]=t[o]-i*r)}}},{key:"pushHandle",value:function(t,e,n,r){for(var i=t[e],o=t[e];n*(o-i)<r;){if(!this.pushHandleOnePoint(t,e,n))return t[e]=i,!1;o=t[e]}return!0}},{key:"pushHandleOnePoint",value:function(t,e,n){var r=this.getPoints(),i=r.indexOf(t[e])+n;if(i>=r.length||i<0)return!1;var o=e+n,a=r[i],s=this.props.pushable,A=n*(t[o]-a);return!!this.pushHandle(t,o,n,s-A)&&(t[e]=a,!0)}},{key:"trimAlignValue",value:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=i()({},this.props,n),o=z(t,r),a=this.ensureValueNotConflict(e,o,r);return J(a,r)}},{key:"ensureValueNotConflict",value:function(t,e,n){var r=n.allowCross,i=n.pushable,o=this.state||{},a=o.bounds;if(t=void 0===t?o.handle:t,i=Number(i),!r&&null!=t&&void 0!==a){if(t>0&&e<=a[t-1]+i)return a[t-1]+i;if(t<a.length-1&&e>=a[t+1]-i)return a[t+1]-i}return e}},{key:"render",value:function(){var t=this,e=this.state,n=e.handle,r=e.bounds,i=this.props,o=i.prefixCls,a=i.vertical,s=i.included,A=i.disabled,l=i.min,c=i.max,u=i.handle,g=i.trackStyle,h=i.handleStyle,f=i.tabIndex,p=r.map(function(e){return t.calcOffset(e)}),I=o+"-handle",C=r.map(function(e,r){var i,s=f[r]||0;return(A||null===f[r])&&(s=null),u({className:_()((i={},B()(i,I,!0),B()(i,I+"-"+(r+1),!0),i)),prefixCls:o,vertical:a,offset:p[r],value:e,dragging:n===r,index:r,tabIndex:s,min:l,max:c,disabled:A,style:h[r],ref:function(e){return t.saveHandle(r,e)}})});return{tracks:r.slice(0,-1).map(function(t,e){var n,r=e+1,i=_()((n={},B()(n,o+"-track",!0),B()(n,o+"-track-"+r,!0),n));return d.a.createElement(m,{className:i,vertical:a,included:s,offset:p[r-1],length:p[r]-p[r-1],style:g[e],key:r})}),handles:C}}}]),e}(d.a.Component);rt.displayName="Range",rt.propTypes={autoFocus:p.a.bool,defaultValue:p.a.arrayOf(p.a.number),value:p.a.arrayOf(p.a.number),count:p.a.number,pushable:p.a.oneOfType([p.a.bool,p.a.number]),allowCross:p.a.bool,disabled:p.a.bool,tabIndex:p.a.arrayOf(p.a.number),min:p.a.number,max:p.a.number},rt.defaultProps={count:1,allowCross:!0,pushable:!1,tabIndex:[]};var it=Z(rt),ot=n(116),at={adjustX:1,adjustY:1},st=[0,0],At={left:{points:["cr","cl"],overflow:at,offset:[-4,0],targetOffset:st},right:{points:["cl","cr"],overflow:at,offset:[4,0],targetOffset:st},top:{points:["bc","tc"],overflow:at,offset:[0,-4],targetOffset:st},bottom:{points:["tc","bc"],overflow:at,offset:[0,4],targetOffset:st},topLeft:{points:["bl","tl"],overflow:at,offset:[0,-4],targetOffset:st},leftTop:{points:["tr","tl"],overflow:at,offset:[-4,0],targetOffset:st},topRight:{points:["br","tr"],overflow:at,offset:[0,-4],targetOffset:st},rightTop:{points:["tl","tr"],overflow:at,offset:[4,0],targetOffset:st},bottomRight:{points:["tr","br"],overflow:at,offset:[0,4],targetOffset:st},rightBottom:{points:["bl","br"],overflow:at,offset:[4,0],targetOffset:st},bottomLeft:{points:["tl","bl"],overflow:at,offset:[0,4],targetOffset:st},leftBottom:{points:["br","bl"],overflow:at,offset:[-4,0],targetOffset:st}},lt=function(t){function e(){return a()(this,e),c()(this,t.apply(this,arguments))}return g()(e,t),e.prototype.componentDidUpdate=function(){var t=this.props.trigger;t&&t.forcePopupAlign()},e.prototype.render=function(){var t=this.props,e=t.overlay,n=t.prefixCls,r=t.id;return d.a.createElement("div",{className:n+"-inner",id:r,role:"tooltip"},"function"==typeof e?e():e)},e}(d.a.Component);lt.propTypes={prefixCls:p.a.string,overlay:p.a.oneOfType([p.a.node,p.a.func]).isRequired,id:p.a.string,trigger:p.a.any};var ct=lt,ut=function(t){function e(){var n,r,i;a()(this,e);for(var o=arguments.length,s=Array(o),A=0;A<o;A++)s[A]=arguments[A];return n=r=c()(this,t.call.apply(t,[this].concat(s))),r.getPopupElement=function(){var t=r.props,e=t.arrowContent,n=t.overlay,i=t.prefixCls,o=t.id;return[d.a.createElement("div",{className:i+"-arrow",key:"arrow"},e),d.a.createElement(ct,{key:"content",trigger:r.trigger,prefixCls:i,id:o,overlay:n})]},r.saveTrigger=function(t){r.trigger=t},i=n,c()(r,i)}return g()(e,t),e.prototype.getPopupDomNode=function(){return this.trigger.getPopupDomNode()},e.prototype.render=function(){var t=this.props,e=t.overlayClassName,n=t.trigger,r=t.mouseEnterDelay,o=t.mouseLeaveDelay,a=t.overlayStyle,s=t.prefixCls,A=t.children,l=t.onVisibleChange,c=t.afterVisibleChange,u=t.transitionName,g=t.animation,h=t.placement,f=t.align,p=t.destroyTooltipOnHide,I=t.defaultVisible,C=t.getTooltipContainer,m=E()(t,["overlayClassName","trigger","mouseEnterDelay","mouseLeaveDelay","overlayStyle","prefixCls","children","onVisibleChange","afterVisibleChange","transitionName","animation","placement","align","destroyTooltipOnHide","defaultVisible","getTooltipContainer"]),y=i()({},m);return"visible"in this.props&&(y.popupVisible=this.props.visible),d.a.createElement(ot.default,i()({popupClassName:e,ref:this.saveTrigger,prefixCls:s,popup:this.getPopupElement,action:n,builtinPlacements:At,popupPlacement:h,popupAlign:f,getPopupContainer:C,onPopupVisibleChange:l,afterPopupVisibleChange:c,popupTransitionName:u,popupAnimation:g,defaultPopupVisible:I,destroyPopupOnHide:p,mouseLeaveDelay:o,popupStyle:a,mouseEnterDelay:r},y),A)},e}(h.Component);ut.propTypes={trigger:p.a.any,children:p.a.any,defaultVisible:p.a.bool,visible:p.a.bool,placement:p.a.string,transitionName:p.a.oneOfType([p.a.string,p.a.object]),animation:p.a.any,onVisibleChange:p.a.func,afterVisibleChange:p.a.func,overlay:p.a.oneOfType([p.a.node,p.a.func]).isRequired,overlayStyle:p.a.object,overlayClassName:p.a.string,prefixCls:p.a.string,mouseEnterDelay:p.a.number,mouseLeaveDelay:p.a.number,getTooltipContainer:p.a.func,destroyTooltipOnHide:p.a.bool,align:p.a.object,arrowContent:p.a.any,id:p.a.string},ut.defaultProps={prefixCls:"rc-tooltip",mouseEnterDelay:0,destroyTooltipOnHide:!1,mouseLeaveDelay:.1,align:{},placement:"right",trigger:["hover"],arrowContent:null};var gt=ut;function ht(t){var e,n;return n=e=function(e){function n(t){a()(this,n);var e=c()(this,(n.__proto__||Object.getPrototypeOf(n)).call(this,t));return e.handleTooltipVisibleChange=function(t,n){e.setState(function(e){return{visibles:i()({},e.visibles,B()({},t,n))}})},e.handleWithTooltip=function(t){var n=t.value,r=t.dragging,o=t.index,a=t.disabled,s=E()(t,["value","dragging","index","disabled"]),A=e.props,l=A.tipFormatter,c=A.tipProps,u=A.handleStyle,g=c.prefixCls,h=void 0===g?"rc-slider-tooltip":g,f=c.overlay,p=void 0===f?l(n):f,I=c.placement,C=void 0===I?"top":I,m=c.visible,y=void 0!==m&&m,v=E()(c,["prefixCls","overlay","placement","visible"]),B=void 0;return B=Array.isArray(u)?u[o]||u[0]:u,d.a.createElement(gt,i()({},v,{prefixCls:h,overlay:p,placement:C,visible:!a&&(e.state.visibles[o]||r)||y,key:o}),d.a.createElement(R,i()({},s,{style:i()({},B),value:n,onMouseEnter:function(){return e.handleTooltipVisibleChange(o,!0)},onMouseLeave:function(){return e.handleTooltipVisibleChange(o,!1)}})))},e.state={visibles:{}},e}return g()(n,e),A()(n,[{key:"render",value:function(){return d.a.createElement(t,i()({},this.props,{handle:this.handleWithTooltip}))}}]),n}(d.a.Component),e.propTypes={tipFormatter:p.a.func,handleStyle:p.a.oneOfType([p.a.object,p.a.arrayOf(p.a.object)]),tipProps:p.a.object},e.defaultProps={tipFormatter:function(t){return t},handleStyle:[{}],tipProps:{}},n}n.d(e,"Range",function(){return it}),n.d(e,"Handle",function(){return R}),n.d(e,"createSliderWithTooltip",function(){return ht}),tt.Range=it,tt.Handle=R,tt.createSliderWithTooltip=ht;e.default=tt},function(t,e){t.exports="data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPHN2ZyB2ZXJzaW9uPSIxLjEiIGlkPSJMYXllcl8xIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHhtbG5zOnhsaW5rPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5L3hsaW5rIiB4PSIwcHgiIHk9IjBweCIKCSB2aWV3Qm94PSIwIDAgMjIgMjIiIHN0eWxlPSJlbmFibGUtYmFja2dyb3VuZDpuZXcgMCAwIDIyIDIyOyIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+Cgk8cGF0aCBjbGFzcz0ianAtaWNvbjMganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjNjE2MTYxIiBkPSJNMTkuMyw4LjJsLTUuNS01LjVjLTAuMy0wLjMtMC43LTAuNS0xLjItMC41SDMuOUMzLjEsMi4zLDIuMywzLjEsMi4zLDR2MTQuMWMwLDAuOSwwLjcsMS42LDEuNiwxLjZoMTQuMgoJYzAuOSwwLDEuNi0wLjcsMS42LTEuNlY5LjRDMTkuOCw4LjksMTkuNiw4LjUsMTkuMyw4LjJ6IE0xMy41LDQuOWwzLjQsMy42aC0zLjRWNC45eiBNMTcuNCwxNy42bC0xMi43LDBjLTAuMSwwLTAuMiwwLTAuMi0wLjJWNC43CgljMC0wLjIsMC4xLTAuMywwLjItMC4zaDcuMnY0LjRjMCwwLDAsMC44LDAuMywxLjFjMC4zLDAuMywxLjEsMC4zLDEuMSwwLjNoNC4zbDAsNy4yQzE3LjYsMTcuNCwxNy41LDE3LjYsMTcuNCwxNy42eiIvPgo8L3N2Zz4K"},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoIGNsYXNzPSJqcC1pY29uMyBqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiM2MTYxNjEiIGQ9Ik0xMCA0SDRjLTEuMSAwLTEuOTkuOS0xLjk5IDJMMiAxOGMwIDEuMS45IDIgMiAyaDE2YzEuMSAwIDItLjkgMi0yVjhjMC0xLjEtLjktMi0yLTJoLThsLTItMnoiLz4KPC9zdmc+Cg=="},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCA1MTIgNTEyIj4KCTxnPgoJCTxwYXRoIGNsYXNzPSJqcC1pY29uMCBqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiMwMDAiIGQ9Ik0xMDguNCAwaDIzdjIyLjhoMjEuMlYwaDIzdjY5aC0yM1Y0NmgtMjF2MjNoLTIzLjJNMjA2IDIzaC0yMC4zVjBoNjMuN3YyM0gyMjl2NDZoLTIzTTI1OS41IDBoMjQuMWwxNC44IDI0LjNMMzEzLjIgMGgyNC4xdjY5aC0yM1YzNC44bC0xNi4xIDI0LjhsLTE2LjEtMjQuOHYzNC4yaC0yMi42TTM0OC43IDBoMjN2NDYuMmgzMi42VjY5aC01NS42Ii8+CgkJPHBhdGggY2xhc3M9ImpwLWljb24tc2VsZWN0YWJsZSIgZmlsbD0iI2U0NGQyNiIgZD0iTTEwNy42IDQ3MWwtMzMtMzcwLjRoMzYyLjhsLTMzIDM3MC4yTDI1NS43IDUxMiIvPgoJCTxwYXRoIGNsYXNzPSJqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiNmMTY1MjkiIGQ9Ik0yNTYgNDgwLjVWMTMxSDQwNC4zTDM3NiA0NDciLz4KCTwvZz4KCTxnPgoJCTxwYXRoIGNsYXNzPSJqcC1pY29uLXNlbGVjdGFibGUtaW52ZXJzZSIgZmlsbD0iI2ViZWJlYiIgZD0iTTE0MiAxNzYuM2gxMTR2NDUuNGgtNjQuMmw0LjIgNDYuNWg2MHY0NS4zSDE1NC40TTE1Ni40IDMzNi4zSDIwMmwzLjIgMzYuMyA1MC44IDEzLjZ2NDcuNGwtOTMuMi0yNiIvPgoJCTxwYXRoIGNsYXNzPSJqcC1pY29uLXNlbGVjdGFibGUtaW52ZXJzZSIgZmlsbD0iI2ZmZiIgZD0iTTM2OS42IDE3Ni4zSDI1NS44djQ1LjRoMTA5LjZNMzYxLjMgMjY4LjJIMjU1Ljh2NDUuNGg1NmwtNS4zIDU5LTUwLjcgMTMuNnY0Ny4ybDkzLTI1LjgiLz4KCTwvZz4+Cjwvc3ZnPgo="},function(t,e){t.exports="data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPHN2ZyB2ZXJzaW9uPSIxLjEiIGlkPSJMYXllcl8xIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHhtbG5zOnhsaW5rPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5L3hsaW5rIiB4PSIwcHgiIHk9IjBweCIKCSB2aWV3Qm94PSIwIDAgMjIgMjIiIHN0eWxlPSJlbmFibGUtYmFja2dyb3VuZDpuZXcgMCAwIDIyIDIyOyIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+Cgk8cmVjdCBjbGFzcz0ianAtaWNvbi1icmFuZDQganAtaWNvbi1zZWxlY3RhYmxlLWludmVyc2UiIGZpbGw9IiNGRkZGRkYiIHg9IjIuMiIgeT0iMi4yIiB3aWR0aD0iMTcuNSIgaGVpZ2h0PSIxNy41Ii8+Cgk8cGF0aCBjbGFzcz0ianAtaWNvbi1icmFuZDAganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjM0Y1MUI1IiBkPSJNMi4yLDIuMnYxNy41aDE3LjVsMC4xLTE3LjVDMTkuOCwyLjIsMi4yLDIuMiwyLjIsMi4yeiBNMTQuMyw0LjRjMS4yLDAsMi4yLDEsMi4yLDIuMnMtMSwyLjItMi4yLDIuMgoJCXMtMi4yLTEtMi4yLTIuMlMxMy4xLDQuNCwxNC4zLDQuNHogTTQuNCwxNy42bDMuMy04LjhsMy4zLDYuNmwyLjItMy4ybDQuNCw1LjRINC40eiIvPgo8L3N2Zz4K"},function(t,e){t.exports="data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPHN2ZyB2ZXJzaW9uPSIxLjEiIGlkPSJMYXllcl8xIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHhtbG5zOnhsaW5rPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5L3hsaW5rIiB4PSIwcHgiIHk9IjBweCIKCSB2aWV3Qm94PSIwIDAgMjIgMjIiIHN0eWxlPSJlbmFibGUtYmFja2dyb3VuZDpuZXcgMCAwIDIyIDIyOyIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+Cgk8ZyBjbGFzcz0ianAtaWNvbi13YXJuMSBqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiNGOUE4MjUiPgoJCTxwYXRoIGQ9Ik0yMC4yLDExLjhjLTEuNiwwLTEuNywwLjUtMS43LDFjMCwwLjQsMC4xLDAuOSwwLjEsMS4zYzAuMSwwLjUsMC4xLDAuOSwwLjEsMS4zYzAsMS43LTEuNCwyLjMtMy41LDIuM2gtMC45CgkJCXYtMS45aDAuNWMxLjEsMCwxLjQsMCwxLjQtMC44YzAtMC4zLDAtMC42LTAuMS0xYzAtMC40LTAuMS0wLjgtMC4xLTEuMmMwLTEuMywwLTEuOCwxLjMtMmwwLDBjLTEuMy0wLjItMS4zLTAuNy0xLjMtMgoJCQljMC0wLjQsMC4xLTAuOCwwLjEtMS4yYzAuMS0wLjQsMC4xLTAuNywwLjEtMWMwLTAuOC0wLjQtMC43LTEuNC0wLjhoLTAuNVY0LjFoMC45YzIuMiwwLDMuNSwwLjcsMy41LDIuM2MwLDAuNC0wLjEsMC45LTAuMSwxLjMKCQkJYy0wLjEsMC41LTAuMSwwLjktMC4xLDEuM2MwLDAuNSwwLjIsMSwxLjcsMUMyMC4yLDEwLDIwLjIsMTEuOCwyMC4yLDExLjh6Ii8+CgkJPHBhdGggZD0iTTEuOCwxMC4xYzEuNiwwLDEuNy0wLjUsMS43LTFjMC0wLjQtMC4xLTAuOS0wLjEtMS4zQzMuMyw3LjMsMy4zLDYuOSwzLjMsNi41YzAtMS42LDEuNC0yLjMsMy41LTIuM2gwLjkKCQkJdjEuOUg3LjJjLTEsMC0xLjQsMC0xLjQsMC44YzAsMC4zLDAsMC42LDAuMSwxYzAsMC4yLDAuMSwwLjYsMC4xLDFjMCwxLjMsMCwxLjgtMS4zLDJsMCwwQzYsMTEuMiw2LDExLjcsNiwxMwoJCQljMCwwLjQtMC4xLDAuOC0wLjEsMS4yYy0wLjEsMC4zLTAuMSwwLjctMC4xLDFDNS44LDE2LDYuMSwxNiw3LjIsMTZoMC41djEuOUg2LjhjLTIuMSwwLTMuNS0wLjYtMy41LTIuM2MwLTAuNCwwLjEtMC45LDAuMS0xLjMKCQkJYzAuMS0wLjUsMC4xLTAuOSwwLjEtMS4zYzAtMC41LTAuMi0xLTEuNy0xVjEwLjF6Ii8+CgkJPGNpcmNsZSBjeD0iMTEiIGN5PSIxMy44IiByPSIyLjEiLz4KCQk8Y2lyY2xlIGN4PSIxMSIgY3k9IjguMiIgcj0iMi4xIi8+Cgk8L2c+Cjwvc3ZnPgo="},function(t,e){t.exports="data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPHN2ZyB2ZXJzaW9uPSIxLjEiIGlkPSJMYXllcl8xIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHhtbG5zOnhsaW5rPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5L3hsaW5rIiB4PSIwcHgiIHk9IjBweCIKCSB2aWV3Qm94PSIwIDAgMjIgMjIiIHN0eWxlPSJlbmFibGUtYmFja2dyb3VuZDpuZXcgMCAwIDIyIDIyOyIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+Cgk8ZyBjbGFzcz0ianAtaWNvbi1jb250cmFzdDAganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjN0IxRkEyIj4KCQk8cG9seWdvbiBwb2ludHM9IjUsMTQuOSAxNywxNC45IDEwLjksMjAuOSAiLz4KCQk8cGF0aCBkPSJNMTQuNCw4LjFjMC0xLjMtMC4xLTIuOS0wLjEtNC41bDAsMEMxMy45LDUsMTMuNCw2LjUsMTMsNy45bC0xLjMsNC4zaC0yTDguNSw3LjlDOC4xLDYuNiw3LjgsNSw3LjUsMy42bDAsMAoJCQlDNy40LDUuMiw3LjQsNi44LDcuMyw4LjJMNywxMi40SDQuOGwwLjctMTFoMy4zTDEwLDVjMC40LDEuMiwwLjcsMi43LDEsMy45bDAsMGMwLjMtMS4yLDAuNy0yLjYsMS0zLjlsMS4yLTMuN2gzLjNsMC42LDExaC0yLjQKCQkJTDE0LjQsOC4xeiIvPgoJPC9nPgo8L3N2Zz4K"},function(t,e){t.exports="data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPHN2ZyB2ZXJzaW9uPSIxLjEiIGlkPSJMYXllcl8xIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHhtbG5zOnhsaW5rPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5L3hsaW5rIiB4PSIwcHgiIHk9IjBweCIKCSB2aWV3Qm94PSIwIDAgMjIgMjIiIHN0eWxlPSJlbmFibGUtYmFja2dyb3VuZDpuZXcgMCAwIDIyIDIyOyIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+Cgk8ZyBjbGFzcz0ianAtaWNvbi13YXJuMCBqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiNFRjZDMDAiPgoJCTxwYXRoIGNsYXNzPSJzdDEiIGQ9Ik0xOC43LDMuM3YxNS40SDMuM1YzLjNIMTguNyBNMjAuMiwxLjhIMS44djE4LjNoMTguM2wwLDBMMjAuMiwxLjhMMjAuMiwxLjh6Ii8+CgkJPHBvbHlnb24gY2xhc3M9InN0MSIgcG9pbnRzPSIxNi41LDE2LjUgMTEuMSwxMi4yIDUuNSwxNi41IDUuNSw1LjUgMTYuNSw1LjUgCSIvPgoJPC9nPgo8L3N2Zz4K"},function(t,e){t.exports="data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPHN2ZyB2ZXJzaW9uPSIxLjEiIGlkPSJMYXllcl8xIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHhtbG5zOnhsaW5rPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5L3hsaW5rIiB4PSIwcHgiIHk9IjBweCIKCSB2aWV3Qm94PSIwIDAgMjIgMjIiIHN0eWxlPSJlbmFibGUtYmFja2dyb3VuZDpuZXcgMCAwIDIyIDIyOyIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+CjxnIGNsYXNzPSJqcC1pY29uLWJyYW5kMCBqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiMwRDQ3QTEiPgoJPHBhdGggZD0iTTExLjEsNi45YzAtMC4yLDAtMS4xLDAtMS4xYy0xLjQsMC0zLjUsMC00LjIsMGMwLTAuNSwwLTEuMywwLjItMS42YzAuNC0wLjcsMC44LTEuMSwxLjctMS40CgkJYzEuNy0wLjMsMi41LTAuMywzLjktMC4xYzEsMC4xLDEuOSwwLjksMS45LDEuOWMwLDEuMSwwLDMsMCw0LjJjMCwwLjUtMC45LDEuNi0yLDEuNnMtMi41LDAtMy44LDBjLTEuNSwwLTIuNCwxLjQtMi40LDIuOAoJCWMwLDAuNCwwLDIuMiwwLDIuMmMtMC41LDAtMS4yLDAtMS43LDBDMy41LDE1LjEsMywxNCwzLDEzLjFjMC0xLjMsMC0yLjcsMC00LjFDMi45LDgsMy42LDcsNC44LDdDNi4zLDYuOSwxMS4xLDYuOSwxMS4xLDYuOXoiLz4KCTxwYXRoIGQ9Ik0xMC45LDE1LjFjMCwwLjIsMCwxLjEsMCwxLjFjMS40LDAsMy41LDAsNC4yLDBjMCwwLjUsMCwxLjMtMC4yLDEuNmMtMC40LDAuNy0wLjgsMS4xLTEuNywxLjQKCQljLTEuNywwLjMtMi41LDAuMy0zLjksMC4xYy0xLTAuMS0xLjktMC45LTEuOS0xLjljMC0xLjEsMC0zLDAtNC4yYzAtMC41LDAuOS0xLjYsMi0xLjZzMi41LDAsMy44LDBjMS41LDAsMi40LTEuNCwyLjQtMi44CgkJYzAtMC40LDAtMi4yLDAtMi4yYzAuNSwwLDEuMiwwLDEuNywwQzE4LjUsNi45LDE5LDgsMTksOC45YzAsMS4zLDAsMi43LDAsNC4xYzAsMS0wLjcsMi4xLTEuOSwyLjEKCQlDMTUuNywxNS4xLDEwLjksMTUuMSwxMC45LDE1LjF6Ii8+CjwvZz4KPC9zdmc+Cg=="},function(t,e){t.exports="data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPHN2ZyB2ZXJzaW9uPSIxLjEiIGlkPSJMYXllcl8xIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHhtbG5zOnhsaW5rPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5L3hsaW5rIiB4PSIwcHgiIHk9IjBweCIKCSB2aWV3Qm94PSIwIDAgMjIgMjIiIHN0eWxlPSJlbmFibGUtYmFja2dyb3VuZDpuZXcgMCAwIDIyIDIyOyIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+CjxnIGNsYXNzPSJqcC1pY29uLWNvbnRyYXN0MyBqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiMyMTk2RjMiPgoJPHBhdGggZD0iTTQuNCwyLjVjMS4yLTAuMSwyLjktMC4zLDQuOS0wLjNjMi41LDAsNC4xLDAuNCw1LjIsMS4zQzE1LjUsNC4yLDE2LDUuNCwxNiw3YzAsMi0xLjQsMy41LTIuOSw0LjFsMCwwCgkJYzEuMiwwLjQsMS43LDEuNiwyLjIsM2MwLjYsMS45LDEsMy45LDEuMyw0LjZoLTMuOEMxMi41LDE4LjMsMTIsMTcsMTEuNiwxNXMtMS4yLTIuNi0yLjYtMi42SDguMXY2LjRINC40VjIuNXogTTguMSw5LjRoMS40CgkJYzEuOSwwLDIuOS0wLjksMi45LTIuM3MtMS0yLjMtMi44LTIuM0M4LjksNC44LDguMyw0LjgsOCw1djQuNWgwLjFWOS40eiIvPgo8L2c+Cjwvc3ZnPgo="},function(t,e){t.exports="data:image/svg+xml;base64, "},function(t,e){t.exports="data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPHN2ZyB2ZXJzaW9uPSIxLjEiIGlkPSJMYXllcl8xIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHhtbG5zOnhsaW5rPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5L3hsaW5rIiB4PSIwcHgiIHk9IjBweCIKCSB2aWV3Qm94PSIwIDAgMjIgMjIiIHN0eWxlPSJlbmFibGUtYmFja2dyb3VuZDpuZXcgMCAwIDIyIDIyOyIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+Cgk8cGF0aCBjbGFzcz0ianAtaWNvbi1jb250cmFzdDEganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjNENBRjUwIiBkPSJNMi4yLDIuMnYxNy42aDE3LjZWMi4yQzE5LjgsMi4yLDIuMiwyLjIsMi4yLDIuMnogTTE3LjYsOS45aC01LjVWNC40aDUuNVY5Ljl6IE05LjksNC40djUuNUg0LjRWNC40CgkJQzQuNCw0LjQsOS45LDQuNCw5LjksNC40eiBNNC40LDEyLjFoNS41djUuNUg0LjRMNC40LDEyLjF6IE0xMi4xLDE3LjZ2LTUuNWg1LjV2NS41SDEyLjF6Ii8+Cjwvc3ZnPgo="},function(t,e){t.exports="data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPHN2ZyB2ZXJzaW9uPSIxLjEiIGlkPSJMYXllcl8xIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHhtbG5zOnhsaW5rPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5L3hsaW5rIiB4PSIwcHgiIHk9IjBweCIKCSB2aWV3Qm94PSIwIDAgMjIgMjIiIHN0eWxlPSJlbmFibGUtYmFja2dyb3VuZDpuZXcgMCAwIDIyIDIyOyIgeG1sOnNwYWNlPSJwcmVzZXJ2ZSI+Cgk8ZyBjbGFzcz0ianAtaWNvbi1jb250cmFzdDIganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjRDgxQjYwIj4KCQk8cGF0aCBkPSJNNy4yLDE4LjZ2LTUuNEwzLDUuNmgzLjNsMS40LDMuMWMwLjMsMC45LDAuNiwxLjYsMSwyLjVsMCwwYzAuMy0wLjgsMC42LTEuNiwxLTIuNWwxLjQtMy4xaDMuNGwtNC40LDcuNnY1LjUKCQkJTDcuMiwxOC42TDcuMiwxOC42eiIvPgoJCTxjaXJjbGUgY2xhc3M9InN0MCIgY3g9IjE3LjYiIGN5PSIxNi41IiByPSIyLjEiLz4KCQk8Y2lyY2xlIGNsYXNzPSJzdDAiIGN4PSIxNy42IiBjeT0iMTEiIHI9IjIuMSIvPgoJPC9nPgo8L3N2Zz4K"},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoCiAgICAgICAgY2xhc3M9ImpwLWljb24zIgogICAgICAgIGZpbGw9IiM2MTYxNjEiCiAgICAgICAgZD0iTTIyLjcgMTlsLTkuMS05LjFjLjktMi4zLjQtNS0xLjUtNi45LTItMi01LTIuNC03LjQtMS4zTDkgNiA2IDkgMS42IDQuN0MuNCA3LjEuOSAxMC4xIDIuOSAxMi4xYzEuOSAxLjkgNC42IDIuNCA2LjkgMS41bDkuMSA5LjFjLjQuNCAxIC40IDEuNCAwbDIuMy0yLjNjLjUtLjQuNS0xLjEuMS0xLjR6IgogICAgLz4KPC9zdmc+Cg=="},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoCiAgICAgICAgY2xhc3M9ImpwLWljb24zIgogICAgICAgIGZpbGw9IiM2MTYxNjEiCiAgICAgICAgZD0iTTIwLjUgMTFIMTlWN2MwLTEuMS0uOS0yLTItMmgtNFYzLjVDMTMgMi4xMiAxMS44OCAxIDEwLjUgMVM4IDIuMTIgOCAzLjVWNUg0Yy0xLjEgMC0xLjk5LjktMS45OSAydjMuOEgzLjVjMS40OSAwIDIuNyAxLjIxIDIuNyAyLjdzLTEuMjEgMi43LTIuNyAyLjdIMlYyMGMwIDEuMS45IDIgMiAyaDMuOHYtMS41YzAtMS40OSAxLjIxLTIuNyAyLjctMi43IDEuNDkgMCAyLjcgMS4yMSAyLjcgMi43VjIySDE3YzEuMSAwIDItLjkgMi0ydi00aDEuNWMxLjM4IDAgMi41LTEuMTIgMi41LTIuNVMyMS44OCAxMSAyMC41IDExeiIKICAgIC8+Cjwvc3ZnPgo="},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoCiAgICAgICAgY2xhc3M9ImpwLWljb24zIgogICAgICAgIGZpbGw9IiM2MTYxNjEiCiAgICAgICAgZD0iTTEyIDNjLTQuOTcgMC05IDQuMDMtOSA5czQuMDMgOSA5IDljLjgzIDAgMS41LS42NyAxLjUtMS41IDAtLjM5LS4xNS0uNzQtLjM5LTEuMDEtLjIzLS4yNi0uMzgtLjYxLS4zOC0uOTkgMC0uODMuNjctMS41IDEuNS0xLjVIMTZjMi43NiAwIDUtMi4yNCA1LTUgMC00LjQyLTQuMDMtOC05LTh6bS01LjUgOWMtLjgzIDAtMS41LS42Ny0xLjUtMS41UzUuNjcgOSA2LjUgOSA4IDkuNjcgOCAxMC41IDcuMzMgMTIgNi41IDEyem0zLTRDOC42NyA4IDggNy4zMyA4IDYuNVM4LjY3IDUgOS41IDVzMS41LjY3IDEuNSAxLjVTMTAuMzMgOCA5LjUgOHptNSAwYy0uODMgMC0xLjUtLjY3LTEuNS0xLjVTMTMuNjcgNSAxNC41IDVzMS41LjY3IDEuNSAxLjVTMTUuMzMgOCAxNC41IDh6bTMgNGMtLjgzIDAtMS41LS42Ny0xLjUtMS41UzE2LjY3IDkgMTcuNSA5czEuNS42NyAxLjUgMS41LS42NyAxLjUtMS41IDEuNXoiCiAgICAvPgo8L3N2Zz4K"},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCA1MTIgNTEyIiBjbGFzcz0iYmFyIj4KICAgIDxwYXRoCiAgICAgICAgY2xhc3M9ImpwLWljb24zIgogICAgICAgIGZpbGw9IiM2MTYxNjEiCiAgICAgICAgZD0iTTI1NiA4QzExOSA4IDggMTE5IDggMjU2czExMSAyNDggMjQ4IDI0OCAyNDgtMTExIDI0OC0yNDhTMzkzIDggMjU2IDh6bTk2IDMyOGMwIDguOC03LjIgMTYtMTYgMTZIMTc2Yy04LjggMC0xNi03LjItMTYtMTZWMTc2YzAtOC44IDcuMi0xNiAxNi0xNmgxNjBjOC44IDAgMTYgNy4yIDE2IDE2djE2MHoiCiAgICA+PC9wYXRoPj4KPC9zdmc+Cg=="},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoCiAgICAgICAgY2xhc3M9ImpwLWljb24zIgogICAgICAgIGZpbGw9IiM2MTYxNjEiCiAgICAgICAgZD0iTTIxIDNIM2MtMS4xIDAtMiAuOS0yIDJ2MTRjMCAxLjEuOSAyIDIgMmgxOGMxLjEgMCAyLS45IDItMlY1YzAtMS4xLS45LTItMi0yem0wIDE2SDNWNWgxMHY0aDh2MTB6IgogICAgLz4KPC9zdmc+Cg=="},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMTUyIiBoZWlnaHQ9IjE2NSIgdmlld0JveD0iMCAwIDE1MiAxNjUiIHZlcnNpb249IjEuMSIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDx0aXRsZT5KdXB5dGVyIEZhdmljb248L3RpdGxlPgogICAgPHBhdGggY2xhc3M9ImpwLWljb24td2FybjAiIHRyYW5zZm9ybT0idHJhbnNsYXRlKDAuMDc4OTQ3LCAxMTAuNTgyOTI3KSIgZmlsbD0iI0YzNzcyNiIgZD0iTTc1Ljk0MjI4NDIsMjkuNTgwNDU2MSBDNDMuMzAyMzk0NywyOS41ODA0NTYxIDE0Ljc5Njc4MzIsMTcuNjUzNDYzNCAwLDAgQzUuNTEwODMyMTEsMTUuODQwNjgyOSAxNS43ODE1Mzg5LDI5LjU2Njc3MzIgMjkuMzkwNDk0NywzOS4yNzg0MTcxIEM0Mi45OTk3LDQ4Ljk4OTg1MzcgNTkuMjczNyw1NC4yMDY3ODA1IDc1Ljk2MDU3ODksNTQuMjA2NzgwNSBDOTIuNjQ3NDU3OSw1NC4yMDY3ODA1IDEwOC45MjE0NTgsNDguOTg5ODUzNyAxMjIuNTMwNjYzLDM5LjI3ODQxNzEgQzEzNi4xMzk0NTMsMjkuNTY2NzczMiAxNDYuNDEwMjg0LDE1Ljg0MDY4MjkgMTUxLjkyMTE1OCwwIEMxMzcuMDg3ODY4LDE3LjY1MzQ2MzQgMTA4LjU4MjU4OSwyOS41ODA0NTYxIDc1Ljk0MjI4NDIsMjkuNTgwNDU2MSBMNzUuOTQyMjg0MiwyOS41ODA0NTYxIFoiIGlkPSJwYXRoMV9maWxsIiAvPgogICAgPHBhdGggY2xhc3M9ImpwLWljb24td2FybjAiIHRyYW5zZm9ybT0idHJhbnNsYXRlKDAuMDM3MzY4LCAwLjcwNDg3OCkiIGZpbGw9IiNGMzc3MjYiIGQ9Ik03NS45Nzg0NTc5LDI0LjYyNjQwNzMgQzEwOC42MTg3NjMsMjQuNjI2NDA3MyAxMzcuMTI0NDU4LDM2LjU1MzQ0MTUgMTUxLjkyMTE1OCw1NC4yMDY3ODA1IEMxNDYuNDEwMjg0LDM4LjM2NjIyMiAxMzYuMTM5NDUzLDI0LjY0MDEzMTcgMTIyLjUzMDY2MywxNC45Mjg0ODc4IEMxMDguOTIxNDU4LDUuMjE2ODQzOSA5Mi42NDc0NTc5LDAgNzUuOTYwNTc4OSwwIEM1OS4yNzM3LDAgNDIuOTk5Nyw1LjIxNjg0MzkgMjkuMzkwNDk0NywxNC45Mjg0ODc4IEMxNS43ODE1Mzg5LDI0LjY0MDEzMTcgNS41MTA4MzIxMSwzOC4zNjYyMjIgMCw1NC4yMDY3ODA1IEMxNC44MzMwODE2LDM2LjU4OTkyOTMgNDMuMzM4NTY4NCwyNC42MjY0MDczIDc1Ljk3ODQ1NzksMjQuNjI2NDA3MyBMNzUuOTc4NDU3OSwyNC42MjY0MDczIFoiIGlkPSJwYXRoMl9maWxsIiAvPgo8L3N2Zz4K"},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoIGNsYXNzPSJqcC1pY29uMiIgZmlsbD0iIzYxNjE2MSIgZD0iTTE1IDlIOXY2aDZWOXptLTIgNGgtMnYtMmgydjJ6bTgtMlY5aC0yVjdjMC0xLjEtLjktMi0yLTJoLTJWM2gtMnYyaC0yVjNIOXYySDdjLTEuMSAwLTIgLjktMiAydjJIM3YyaDJ2MkgzdjJoMnYyYzAgMS4xLjkgMiAyIDJoMnYyaDJ2LTJoMnYyaDJ2LTJoMmMxLjEgMCAyLS45IDItMnYtMmgydi0yaC0ydi0yaDJ6bS00IDZIN1Y3aDEwdjEweiIvPgo8L3N2Zz4K"},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoIGZpbGw9IndoaXRlIiBkPSJNNS44OCA0LjEyTDEzLjc2IDEybC03Ljg4IDcuODhMOCAyMmwxMC0xMEw4IDJ6Ii8+CiAgICA8cGF0aCBmaWxsPSJub25lIiBkPSJNMCAwaDI0djI0SDB6Ii8+Cjwvc3ZnPgo="},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB2aWV3Qm94PSIwIDAgMjUgMjUiIGZpbGw9Im5vbmUiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyI+CiAgICA8cGF0aCBjbGFzcz0ianAtaWNvbjIiIGQ9Ik0xLjg2MDk0IDExLjQ0MDlDMC44MjY0NDggOC43NzAyNyAwLjg2Mzc3OSA2LjA1NzY0IDEuMjQ5MDcgNC4xOTkzMkMyLjQ4MjA2IDMuOTMzNDcgNC4wODA2OCAzLjQwMzQ3IDUuNjAxMDIgMi44NDQ5QzcuMjM1NDkgMi4yNDQ0IDguODU2NjYgMS41ODE1IDkuOTg3NiAxLjA5NTM5QzExLjA1OTcgMS41ODM0MSAxMi42MDk0IDIuMjQ0NCAxNC4yMTggMi44NDMzOUMxNS43NTAzIDMuNDEzOTQgMTcuMzk5NSAzLjk1MjU4IDE4Ljc1MzkgNC4yMTM4NUMxOS4xMzY0IDYuMDcxNzcgMTkuMTcwOSA4Ljc3NzIyIDE4LjEzOSAxMS40NDA5QzE3LjAzMDMgMTQuMzAzMiAxNC42NjY4IDE3LjE4NDQgOS45OTk5OSAxOC45MzU0QzUuMzMzMTkgMTcuMTg0NCAyLjk2OTY4IDE0LjMwMzIgMS44NjA5NCAxMS40NDA5WiIgdHJhbnNmb3JtPSJ0cmFuc2xhdGUoMyAzKSIgc3Ryb2tlPSIjMzMzMzMzIiBzdHJva2Utd2lkdGg9IjIiLz4KICAgIDxwYXRoIGNsYXNzPSJqcC1pY29uMiIgZD0iTTcuMzY4NDIgMEwwIDcuMzY0NzkiIHRyYW5zZm9ybT0idHJhbnNsYXRlKDkuMzE1OTIgOS4zMjAzMSkiIHN0cm9rZT0iIzMzMzMzMyIgc3Ryb2tlLXdpZHRoPSIyIi8+CiAgICA8cGF0aCBjbGFzcz0ianAtaWNvbjIiIGQ9Ik03LjM2ODQyIDBMMCA3LjM2NDc5IiB0cmFuc2Zvcm09InRyYW5zbGF0ZSg5LjMxNTkyIDE2LjY4MzYpIHNjYWxlKDEgLTEpIiBzdHJva2U9IiMzMzMzMzMiIHN0cm9rZS13aWR0aD0iMiIvPgo8L3N2Zz4K"},function(t,e){t.exports="data:image/svg+xml;base64,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"},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMjQiIGhlaWdodD0iMjUiIHZpZXdCb3g9IjAgMCAyNCAyNSIgZmlsbD0ibm9uZSIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDxwYXRoIGNsYXNzPSJqcC1pY29uMiIgZD0iTTEuODYwOTQgMTEuNDQwOUMwLjgyNjQ0OCA4Ljc3MDI3IDAuODYzNzc5IDYuMDU3NjQgMS4yNDkwNyA0LjE5OTMyQzIuNDgyMDYgMy45MzM0NyA0LjA4MDY4IDMuNDAzNDcgNS42MDEwMiAyLjg0NDlDNy4yMzU0OSAyLjI0NDQgOC44NTY2NiAxLjU4MTUgOS45ODc2IDEuMDk1MzlDMTEuMDU5NyAxLjU4MzQxIDEyLjYwOTQgMi4yNDQ0IDE0LjIxOCAyLjg0MzM5QzE1Ljc1MDMgMy40MTM5NCAxNy4zOTk1IDMuOTUyNTggMTguNzUzOSA0LjIxMzg1QzE5LjEzNjQgNi4wNzE3NyAxOS4xNzA5IDguNzc3MjIgMTguMTM5IDExLjQ0MDlDMTcuMDMwMyAxNC4zMDMyIDE0LjY2NjggMTcuMTg0NCA5Ljk5OTk5IDE4LjkzNTRDNS4zMzMyIDE3LjE4NDQgMi45Njk2OCAxNC4zMDMyIDEuODYwOTQgMTEuNDQwOVoiIHRyYW5zZm9ybT0idHJhbnNsYXRlKDIgMykiIHN0cm9rZT0iIzMzMzMzMyIgc3Ryb2tlLXdpZHRoPSIyIi8+CiAgICA8cGF0aCBjbGFzcz0ianAtaWNvbjIiIGQ9Ik0yLjg2MDE1IDQuODY1MzVMMC43MjY1NDkgMi45OTk1OUwwIDMuNjMwNDVMMi44NjAxNSA2LjEzMTU3TDggMC42MzA4NzJMNy4yNzg1NyAwTDIuODYwMTUgNC44NjUzNVoiIHRyYW5zZm9ybT0idHJhbnNsYXRlKDggOS44NjcxOSkiIGZpbGw9IiMzMzMzMzMiIHN0cm9rZT0iIzMzMzMzMyIvPgo8L3N2Zz4K"},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTE5IDEzaC02djZoLTJ2LTZINXYtMmg2VjVoMnY2aDZ2MnoiLz4KICAgIDwvZz4KPC9zdmc+Cg=="},function(t,e){t.exports="data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPHN2ZyB2ZXJzaW9uPSIxLjEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgeG1sbnM6eGxpbms9Imh0dHA6Ly93d3cudzMub3JnLzE5OTkveGxpbmsiIHZpZXdCb3g9IjAgMCAxOCAxOCI+CiAgICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgICAgIDxwYXRoIGZpbGw9Im5vbmUiIGQ9Ik0wLDBoMTh2MThIMFYweiIvPgogICAgICAgIDxwYXRoIGQ9Ik0xMS45LDFIMy4yQzIuNCwxLDEuNywxLjcsMS43LDIuNXYxMC4yaDEuNVYyLjVoOC43VjF6IE0xNC4xLDMuOWgtOGMtMC44LDAtMS41LDAuNy0xLjUsMS41djEwLjJjMCwwLjgsMC43LDEuNSwxLjUsMS41aDgKICAgICAgICAgICAgYzAuOCwwLDEuNS0wLjcsMS41LTEuNVY1LjRDMTUuNSw0LjYsMTQuOSwzLjksMTQuMSwzLjl6IE0xNC4xLDE1LjVoLThWNS40aDhWMTUuNXoiLz4KICAgIDwvZz4KPC9zdmc+Cg=="},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyBoZWlnaHQ9IjE4IiB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxOCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTAgMGgyNHYyNEgweiIgZmlsbD0ibm9uZSIvPgogICAgICAgIDxjaXJjbGUgY3g9IjYiIGN5PSIxOCIgZmlsbD0ibm9uZSIgcj0iMiIvPgogICAgICAgIDxjaXJjbGUgY3g9IjEyIiBjeT0iMTIiIGZpbGw9Im5vbmUiIHI9Ii41Ii8+CiAgICAgICAgPGNpcmNsZSBjeD0iNiIgY3k9IjYiIGZpbGw9Im5vbmUiIHI9IjIiLz4KICAgICAgICA8cGF0aCBkPSJNOS42NCA3LjY0Yy4yMy0uNS4zNi0xLjA1LjM2LTEuNjQgMC0yLjIxLTEuNzktNC00LTRTMiAzLjc5IDIgNnMxLjc5IDQgNCA0Yy41OSAwIDEuMTQtLjEzIDEuNjQtLjM2TDEwIDEybC0yLjM2IDIuMzZDNy4xNCAxNC4xMyA2LjU5IDE0IDYgMTRjLTIuMjEgMC00IDEuNzktNCA0czEuNzkgNCA0IDQgNC0xLjc5IDQtNGMwLS41OS0uMTMtMS4xNC0uMzYtMS42NEwxMiAxNGw3IDdoM3YtMUw5LjY0IDcuNjR6TTYgOGMtMS4xIDAtMi0uODktMi0ycy45LTIgMi0yIDIgLjg5IDIgMi0uOSAyLTIgMnptMCAxMmMtMS4xIDAtMi0uODktMi0ycy45LTIgMi0yIDIgLjg5IDIgMi0uOSAyLTIgMnptNi03LjVjLS4yOCAwLS41LS4yMi0uNS0uNXMuMjItLjUuNS0uNS41LjIyLjUuNS0uMjIuNS0uNS41ek0xOSAzbC02IDYgMiAyIDctN1YzeiIvPgogICAgPC9nPgo8L3N2Zz4K"},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyBoZWlnaHQ9IjE4IiB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxOCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTE5IDJoLTQuMThDMTQuNC44NCAxMy4zIDAgMTIgMGMtMS4zIDAtMi40Ljg0LTIuODIgMkg1Yy0xLjEgMC0yIC45LTIgMnYxNmMwIDEuMS45IDIgMiAyaDE0YzEuMSAwIDItLjkgMi0yVjRjMC0xLjEtLjktMi0yLTJ6bS03IDBjLjU1IDAgMSAuNDUgMSAxcy0uNDUgMS0xIDEtMS0uNDUtMS0xIC40NS0xIDEtMXptNyAxOEg1VjRoMnYzaDEwVjRoMnYxNnoiLz4KICAgICAgICA8cGF0aCBkPSJNMCAwaDI0djI0SDB6IiBmaWxsPSJub25lIi8+CiAgICA8L2c+Cjwvc3ZnPgo="},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxOCIgaGVpZ2h0PSIxOCIgdmlld0JveD0iMCAwIDE4IDE4Ij4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTkgMTMuNWMtMi40OSAwLTQuNS0yLjAxLTQuNS00LjVTNi41MSA0LjUgOSA0LjVjMS4yNCAwIDIuMzYuNTIgMy4xNyAxLjMzTDEwIDhoNVYzbC0xLjc2IDEuNzZDMTIuMTUgMy42OCAxMC42NiAzIDkgMyA1LjY5IDMgMy4wMSA1LjY5IDMuMDEgOVM1LjY5IDE1IDkgMTVjMi45NyAwIDUuNDMtMi4xNiA1LjktNWgtMS41MmMtLjQ2IDItMi4yNCAzLjUtNC4zOCAzLjV6Ii8+CiAgICA8L2c+Cjwvc3ZnPgo="},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyBoZWlnaHQ9IjE4IiB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxOCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTggNXYxNGwxMS03eiIvPgogICAgICAgIDxwYXRoIGQ9Ik0wIDBoMjR2MjRIMHoiIGZpbGw9Im5vbmUiLz4KICAgIDwvZz4KPC9zdmc+Cg=="},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyBoZWlnaHQ9IjE4IiB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxOCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTAgMGgyNHYyNEgweiIgZmlsbD0ibm9uZSIvPgogICAgICAgIDxwYXRoIGQ9Ik0xNyAzSDVjLTEuMTEgMC0yIC45LTIgMnYxNGMwIDEuMS44OSAyIDIgMmgxNGMxLjEgMCAyLS45IDItMlY3bC00LTR6bS01IDE2Yy0xLjY2IDAtMy0xLjM0LTMtM3MxLjM0LTMgMy0zIDMgMS4zNCAzIDMtMS4zNCAzLTMgM3ptMy0xMEg1VjVoMTB2NHoiLz4KICAgIDwvZz4KPC9zdmc+Cg=="},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyBoZWlnaHQ9IjE4IiB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxOCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTAgMGgyNHYyNEgweiIgZmlsbD0ibm9uZSIvPgogICAgICAgIDxwYXRoIGQ9Ik02IDZoMTJ2MTJINnoiLz4KICAgIDwvZz4KPC9zdmc+Cg=="},function(t,e,n){"use strict";n.d(e,"a",function(){return s});var r,i=n(204),o=n(9),a=n(10);class s{constructor(t){this.trusted=!1,this._changed=new a.Signal(this),this._raw={};let{data:e}=r.getBundleOptions(t);this._data=new i.a({values:e}),this._rawData=e;let n=t.value;for(let t in n)switch(t){case"data":break;default:this._raw[t]=r.extract(n,t)}}get changed(){return this._changed}dispose(){this._data.dispose(),a.Signal.clearData(this)}get data(){return this._rawData}get metadata(){}setData(t){t.data&&(this._updateObservable(this._data,t.data),this._rawData=t.data),this._changed.emit(void 0)}toJSON(){let t={};for(let e in this._raw)t[e]=r.extract(this._raw,e);return t}_updateObservable(t,e){let n=t.keys(),r=Object.keys(e);for(let e of n)-1===r.indexOf(e)&&t.delete(e);for(let n of r){let r=t.get(n),i=e[n];r!==i&&t.set(n,i)}}}!function(t){t.getData=function(t){return r.getData(t)}}(s||(s={})),function(t){function e(t){return function(t){let e=Object.create(null);for(let r in t)e[r]=n(t,r);return e}(t)}function n(t,e){let n=t[e];return o.JSONExt.isPrimitive(n)?n:o.JSONExt.deepCopy(n)}t.getData=e,t.getBundleOptions=function(t){return{data:e(t.value)}},t.extract=n}(r||(r={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return i}),n.d(e,"b",function(){return o}),n.d(e,"d",function(){return a}),n.d(e,"e",function(){return s}),n.d(e,"g",function(){return A}),n.d(e,"h",function(){return l}),n.d(e,"c",function(){return c}),n.d(e,"f",function(){return u});var r=n(96);const i={safe:!0,mimeTypes:["text/html"],defaultRank:50,createRenderer:t=>new r.b(t)},o={safe:!0,mimeTypes:["image/bmp","image/png","image/jpeg","image/gif"],defaultRank:90,createRenderer:t=>new r.d(t)},a={safe:!0,mimeTypes:["text/latex"],defaultRank:70,createRenderer:t=>new r.f(t)},s={safe:!0,mimeTypes:["text/markdown"],defaultRank:60,createRenderer:t=>new r.g(t)},A={safe:!1,mimeTypes:["image/svg+xml"],defaultRank:80,createRenderer:t=>new r.h(t)},l={safe:!0,mimeTypes:["text/plain","application/vnd.jupyter.stdout","application/vnd.jupyter.stderr"],defaultRank:120,createRenderer:t=>new r.i(t)},c={safe:!1,mimeTypes:["text/javascript","application/javascript"],defaultRank:110,createRenderer:t=>new r.e(t)},u=[i,s,a,A,o,c,l]},function(t,e,n){"use strict";n.d(e,"a",function(){return A});var r,i=n(9),o=n(10),a=n(17),s=n(204);class A{constructor(t){this._changed=new o.Signal(this),this._raw={};let{data:e,metadata:n,trusted:i}=r.getBundleOptions(t);this._data=new s.a({values:e}),this._rawData=e,this._metadata=new s.a({values:n}),this._rawMetadata=n,this.trusted=i;let A=t.value;for(let t in A)switch(t){case"data":case"metadata":break;default:this._raw[t]=r.extract(A,t)}this.type=A.output_type,a.nbformat.isExecuteResult(A)?this.executionCount=A.execution_count:this.executionCount=null}get changed(){return this._changed}dispose(){this._data.dispose(),this._metadata.dispose(),o.Signal.clearData(this)}get data(){return this._rawData}get metadata(){return this._rawMetadata}setData(t){t.data&&(this._updateObservable(this._data,t.data),this._rawData=t.data),t.metadata&&(this._updateObservable(this._metadata,t.metadata),this._rawMetadata=t.metadata),this._changed.emit(void 0)}toJSON(){let t={};for(let e in this._raw)t[e]=r.extract(this._raw,e);switch(this.type){case"display_data":case"execute_result":case"update_display_data":t.data=this.data,t.metadata=this.metadata}return delete t.transient,t}_updateObservable(t,e){let n=t.keys(),r=Object.keys(e);for(let e of n)-1===r.indexOf(e)&&t.delete(e);for(let n of r){let r=t.get(n),i=e[n];r!==i&&t.set(n,i)}}}!function(t){t.getData=function(t){return r.getData(t)},t.getMetadata=function(t){return r.getMetadata(t)}}(A||(A={})),function(t){function e(t){let e={};if(a.nbformat.isExecuteResult(t)||a.nbformat.isDisplayData(t)||a.nbformat.isDisplayUpdate(t))e=t.data;else if(a.nbformat.isStream(t))"stderr"===t.name?e["application/vnd.jupyter.stderr"]=t.text:e["application/vnd.jupyter.stdout"]=t.text;else if(a.nbformat.isError(t)){let n=t.traceback.join("\n");e["application/vnd.jupyter.stderr"]=n||`${t.ename}: ${t.evalue}`}return function(t){let e=Object.create(null);for(let n in t)e[n]=r(t,n);return e}(e)}function n(t){let e=Object.create(null);if(a.nbformat.isExecuteResult(t)||a.nbformat.isDisplayData(t))for(let n in t.metadata)e[n]=r(t.metadata,n);return e}function r(t,e){let n=t[e];return i.JSONExt.isPrimitive(n)?n:JSON.parse(JSON.stringify(n))}t.getData=e,t.getMetadata=n,t.getBundleOptions=function(t){return{data:e(t.value),metadata:n(t.value),trusted:!!t.trusted}},t.extract=r}(r||(r={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return A});var r=n(6);const i="jp-mod-has-primary-selection",o="jp-mod-in-leading-whitespace",a="jp-mod-dropTarget",s=/^\s+$/;class A extends r.Widget{constructor(t){super(),(this.editor=t.factory({host:this.node,model:t.model,uuid:t.uuid,config:t.config,selectionStyle:t.selectionStyle})).model.selections.changed.connect(this._onSelectionsChanged,this),this._updateOnShow=!1!==t.updateOnShow}get model(){return this.editor.model}dispose(){this.isDisposed||(super.dispose(),this.editor.dispose())}handleEvent(t){switch(t.type){case"p-dragenter":this._evtDragEnter(t);break;case"p-dragleave":this._evtDragLeave(t);break;case"p-dragover":this._evtDragOver(t);break;case"p-drop":this._evtDrop(t)}}onActivateRequest(t){this.editor.focus()}onAfterAttach(t){super.onAfterAttach(t);let e=this.node;e.addEventListener("p-dragenter",this),e.addEventListener("p-dragleave",this),e.addEventListener("p-dragover",this),e.addEventListener("p-drop",this),this.isVisible&&this.update()}onBeforeDetach(t){let e=this.node;e.removeEventListener("p-dragenter",this),e.removeEventListener("p-dragleave",this),e.removeEventListener("p-dragover",this),e.removeEventListener("p-drop",this)}onAfterShow(t){this._updateOnShow&&this.update()}onResize(t){t.width>=0&&t.height>=0?this.editor.setSize(t):this.isVisible&&this.editor.resizeToFit()}onUpdateRequest(t){this.editor.refresh()}_onSelectionsChanged(){const{start:t,end:e}=this.editor.getSelection();t.column!==e.column||t.line!==e.line?(this.addClass(i),this.removeClass(o)):(this.removeClass(i),this.editor.getLine(e.line).slice(0,e.column).match(s)?this.addClass(o):this.removeClass(o))}_evtDragEnter(t){if(!0===this.editor.getOption("readOnly"))return;void 0!==l.findTextData(t.mimeData)&&(t.preventDefault(),t.stopPropagation(),this.addClass("jp-mod-dropTarget"))}_evtDragLeave(t){if(this.removeClass(a),!0===this.editor.getOption("readOnly"))return;void 0!==l.findTextData(t.mimeData)&&(t.preventDefault(),t.stopPropagation())}_evtDragOver(t){if(this.removeClass(a),!0===this.editor.getOption("readOnly"))return;void 0!==l.findTextData(t.mimeData)&&(t.preventDefault(),t.stopPropagation(),t.dropAction="copy",this.addClass(a))}_evtDrop(t){if(!0===this.editor.getOption("readOnly"))return;const e=l.findTextData(t.mimeData);if(void 0===e)return;if(this.removeClass(a),t.preventDefault(),t.stopPropagation(),"none"===t.proposedAction)return void(t.dropAction="none");const n={top:t.y,bottom:t.y,left:t.x,right:t.x,x:t.x,y:t.y,width:0,height:0},r=this.editor.getPositionForCoordinate(n),i=this.editor.getOffsetAt(r);this.model.value.insert(i,e)}}var l;!function(t){t.findTextData=function(t){const e=t.types().find(t=>0===t.indexOf("text"));if(void 0!==e)return t.getData(e)}}(l||(l={}))},function(t,e){t.exports=function(t,e,n,r){var i=n?n.call(r,t,e):void 0;if(void 0!==i)return!!i;if(t===e)return!0;if("object"!=typeof t||!t||"object"!=typeof e||!e)return!1;var o=Object.keys(t),a=Object.keys(e);if(o.length!==a.length)return!1;for(var s=Object.prototype.hasOwnProperty.bind(e),A=0;A<o.length;A++){var l=o[A];if(!s(l))return!1;var c=t[l],u=e[l];if(!1===(i=n?n.call(r,c,u,l):void 0)||void 0===i&&c!==u)return!1}return!0}},function(t,e,n){(function(e){!function(e){"use strict";var n={newline:/^\n+/,code:/^( {4}[^\n]+\n*)+/,fences:p,hr:/^ {0,3}((?:- *){3,}|(?:_ *){3,}|(?:\* *){3,})(?:\n+|$)/,heading:/^ *(#{1,6}) *([^\n]+?) *(?:#+ *)?(?:\n+|$)/,nptable:p,blockquote:/^( {0,3}> ?(paragraph|[^\n]*)(?:\n|$))+/,list:/^( {0,3})(bull) [\s\S]+?(?:hr|def|\n{2,}(?! )(?!\1bull )\n*|\s*$)/,html:"^ {0,3}(?:<(script|pre|style)[\\s>][\\s\\S]*?(?:</\\1>[^\\n]*\\n+|$)|comment[^\\n]*(\\n+|$)|<\\?[\\s\\S]*?\\?>\\n*|<![A-Z][\\s\\S]*?>\\n*|<!\\[CDATA\\[[\\s\\S]*?\\]\\]>\\n*|</?(tag)(?: +|\\n|/?>)[\\s\\S]*?(?:\\n{2,}|$)|<(?!script|pre|style)([a-z][\\w-]*)(?:attribute)*? */?>(?=[ \\t]*(?:\\n|$))[\\s\\S]*?(?:\\n{2,}|$)|</(?!script|pre|style)[a-z][\\w-]*\\s*>(?=[ \\t]*(?:\\n|$))[\\s\\S]*?(?:\\n{2,}|$))",def:/^ {0,3}\[(label)\]: *\n? *<?([^\s>]+)>?(?:(?: +\n? *| *\n *)(title))? *(?:\n+|$)/,table:p,lheading:/^([^\n]+)\n *(=|-){2,} *(?:\n+|$)/,paragraph:/^([^\n]+(?:\n(?!hr|heading|lheading| {0,3}>|<\/?(?:tag)(?: +|\n|\/?>)|<(?:script|pre|style|!--))[^\n]+)*)/,text:/^[^\n]+/};function r(t){this.tokens=[],this.tokens.links=Object.create(null),this.options=t||E.defaults,this.rules=n.normal,this.options.pedantic?this.rules=n.pedantic:this.options.gfm&&(this.options.tables?this.rules=n.tables:this.rules=n.gfm)}n._label=/(?!\s*\])(?:\\[\[\]]|[^\[\]])+/,n._title=/(?:"(?:\\"?|[^"\\])*"|'[^'\n]*(?:\n[^'\n]+)*\n?'|\([^()]*\))/,n.def=g(n.def).replace("label",n._label).replace("title",n._title).getRegex(),n.bullet=/(?:[*+-]|\d{1,9}\.)/,n.item=/^( *)(bull) ?[^\n]*(?:\n(?!\1bull ?)[^\n]*)*/,n.item=g(n.item,"gm").replace(/bull/g,n.bullet).getRegex(),n.list=g(n.list).replace(/bull/g,n.bullet).replace("hr","\\n+(?=\\1?(?:(?:- *){3,}|(?:_ *){3,}|(?:\\* *){3,})(?:\\n+|$))").replace("def","\\n+(?="+n.def.source+")").getRegex(),n._tag="address|article|aside|base|basefont|blockquote|body|caption|center|col|colgroup|dd|details|dialog|dir|div|dl|dt|fieldset|figcaption|figure|footer|form|frame|frameset|h[1-6]|head|header|hr|html|iframe|legend|li|link|main|menu|menuitem|meta|nav|noframes|ol|optgroup|option|p|param|section|source|summary|table|tbody|td|tfoot|th|thead|title|tr|track|ul",n._comment=/<!--(?!-?>)[\s\S]*?-->/,n.html=g(n.html,"i").replace("comment",n._comment).replace("tag",n._tag).replace("attribute",/ +[a-zA-Z:_][\w.:-]*(?: *= *"[^"\n]*"| *= *'[^'\n]*'| *= *[^\s"'=<>`]+)?/).getRegex(),n.paragraph=g(n.paragraph).replace("hr",n.hr).replace("heading",n.heading).replace("lheading",n.lheading).replace("tag",n._tag).getRegex(),n.blockquote=g(n.blockquote).replace("paragraph",n.paragraph).getRegex(),n.normal=I({},n),n.gfm=I({},n.normal,{fences:/^ {0,3}(`{3,}|~{3,})([^`\n]*)\n(?:|([\s\S]*?)\n)(?: {0,3}\1[~`]* *(?:\n+|$)|$)/,paragraph:/^/,heading:/^ *(#{1,6}) +([^\n]+?) *#* *(?:\n+|$)/}),n.gfm.paragraph=g(n.paragraph).replace("(?!","(?!"+n.gfm.fences.source.replace("\\1","\\2")+"|"+n.list.source.replace("\\1","\\3")+"|").getRegex(),n.tables=I({},n.gfm,{nptable:/^ *([^|\n ].*\|.*)\n *([-:]+ *\|[-| :]*)(?:\n((?:.*[^>\n ].*(?:\n|$))*)\n*|$)/,table:/^ *\|(.+)\n *\|?( *[-:]+[-| :]*)(?:\n((?: *[^>\n ].*(?:\n|$))*)\n*|$)/}),n.pedantic=I({},n.normal,{html:g("^ *(?:comment *(?:\\n|\\s*$)|<(tag)[\\s\\S]+?</\\1> *(?:\\n{2,}|\\s*$)|<tag(?:\"[^\"]*\"|'[^']*'|\\s[^'\"/>\\s]*)*?/?> *(?:\\n{2,}|\\s*$))").replace("comment",n._comment).replace(/tag/g,"(?!(?:a|em|strong|small|s|cite|q|dfn|abbr|data|time|code|var|samp|kbd|sub|sup|i|b|u|mark|ruby|rt|rp|bdi|bdo|span|br|wbr|ins|del|img)\\b)\\w+(?!:|[^\\w\\s@]*@)\\b").getRegex(),def:/^ *\[([^\]]+)\]: *<?([^\s>]+)>?(?: +(["(][^\n]+[")]))? *(?:\n+|$)/}),r.rules=n,r.lex=function(t,e){return new r(e).lex(t)},r.prototype.lex=function(t){return t=t.replace(/\r\n|\r/g,"\n").replace(/\t/g,"    ").replace(/\u00a0/g," ").replace(/\u2424/g,"\n"),this.token(t,!0)},r.prototype.token=function(t,e){var r,i,o,a,s,A,l,c,u,g,h,d,f,p,I,y;for(t=t.replace(/^ +$/gm,"");t;)if((o=this.rules.newline.exec(t))&&(t=t.substring(o[0].length),o[0].length>1&&this.tokens.push({type:"space"})),o=this.rules.code.exec(t))t=t.substring(o[0].length),o=o[0].replace(/^ {4}/gm,""),this.tokens.push({type:"code",text:this.options.pedantic?o:m(o,"\n")});else if(o=this.rules.fences.exec(t))t=t.substring(o[0].length),this.tokens.push({type:"code",lang:o[2]?o[2].trim():o[2],text:o[3]||""});else if(o=this.rules.heading.exec(t))t=t.substring(o[0].length),this.tokens.push({type:"heading",depth:o[1].length,text:o[2]});else if((o=this.rules.nptable.exec(t))&&(A={type:"table",header:C(o[1].replace(/^ *| *\| *$/g,"")),align:o[2].replace(/^ *|\| *$/g,"").split(/ *\| */),cells:o[3]?o[3].replace(/\n$/,"").split("\n"):[]}).header.length===A.align.length){for(t=t.substring(o[0].length),h=0;h<A.align.length;h++)/^ *-+: *$/.test(A.align[h])?A.align[h]="right":/^ *:-+: *$/.test(A.align[h])?A.align[h]="center":/^ *:-+ *$/.test(A.align[h])?A.align[h]="left":A.align[h]=null;for(h=0;h<A.cells.length;h++)A.cells[h]=C(A.cells[h],A.header.length);this.tokens.push(A)}else if(o=this.rules.hr.exec(t))t=t.substring(o[0].length),this.tokens.push({type:"hr"});else if(o=this.rules.blockquote.exec(t))t=t.substring(o[0].length),this.tokens.push({type:"blockquote_start"}),o=o[0].replace(/^ *> ?/gm,""),this.token(o,e),this.tokens.push({type:"blockquote_end"});else if(o=this.rules.list.exec(t)){for(t=t.substring(o[0].length),l={type:"list_start",ordered:p=(a=o[2]).length>1,start:p?+a:"",loose:!1},this.tokens.push(l),c=[],r=!1,f=(o=o[0].match(this.rules.item)).length,h=0;h<f;h++)g=(A=o[h]).length,~(A=A.replace(/^ *([*+-]|\d+\.) */,"")).indexOf("\n ")&&(g-=A.length,A=this.options.pedantic?A.replace(/^ {1,4}/gm,""):A.replace(new RegExp("^ {1,"+g+"}","gm"),"")),h!==f-1&&(s=n.bullet.exec(o[h+1])[0],(a.length>1?1===s.length:s.length>1||this.options.smartLists&&s!==a)&&(t=o.slice(h+1).join("\n")+t,h=f-1)),i=r||/\n\n(?!\s*$)/.test(A),h!==f-1&&(r="\n"===A.charAt(A.length-1),i||(i=r)),i&&(l.loose=!0),y=void 0,(I=/^\[[ xX]\] /.test(A))&&(y=" "!==A[1],A=A.replace(/^\[[ xX]\] +/,"")),u={type:"list_item_start",task:I,checked:y,loose:i},c.push(u),this.tokens.push(u),this.token(A,!1),this.tokens.push({type:"list_item_end"});if(l.loose)for(f=c.length,h=0;h<f;h++)c[h].loose=!0;this.tokens.push({type:"list_end"})}else if(o=this.rules.html.exec(t))t=t.substring(o[0].length),this.tokens.push({type:this.options.sanitize?"paragraph":"html",pre:!this.options.sanitizer&&("pre"===o[1]||"script"===o[1]||"style"===o[1]),text:o[0]});else if(e&&(o=this.rules.def.exec(t)))t=t.substring(o[0].length),o[3]&&(o[3]=o[3].substring(1,o[3].length-1)),d=o[1].toLowerCase().replace(/\s+/g," "),this.tokens.links[d]||(this.tokens.links[d]={href:o[2],title:o[3]});else if((o=this.rules.table.exec(t))&&(A={type:"table",header:C(o[1].replace(/^ *| *\| *$/g,"")),align:o[2].replace(/^ *|\| *$/g,"").split(/ *\| */),cells:o[3]?o[3].replace(/\n$/,"").split("\n"):[]}).header.length===A.align.length){for(t=t.substring(o[0].length),h=0;h<A.align.length;h++)/^ *-+: *$/.test(A.align[h])?A.align[h]="right":/^ *:-+: *$/.test(A.align[h])?A.align[h]="center":/^ *:-+ *$/.test(A.align[h])?A.align[h]="left":A.align[h]=null;for(h=0;h<A.cells.length;h++)A.cells[h]=C(A.cells[h].replace(/^ *\| *| *\| *$/g,""),A.header.length);this.tokens.push(A)}else if(o=this.rules.lheading.exec(t))t=t.substring(o[0].length),this.tokens.push({type:"heading",depth:"="===o[2]?1:2,text:o[1]});else if(e&&(o=this.rules.paragraph.exec(t)))t=t.substring(o[0].length),this.tokens.push({type:"paragraph",text:"\n"===o[1].charAt(o[1].length-1)?o[1].slice(0,-1):o[1]});else if(o=this.rules.text.exec(t))t=t.substring(o[0].length),this.tokens.push({type:"text",text:o[0]});else if(t)throw new Error("Infinite loop on byte: "+t.charCodeAt(0));return this.tokens};var i={escape:/^\\([!"#$%&'()*+,\-.\/:;<=>?@\[\]\\^_`{|}~])/,autolink:/^<(scheme:[^\s\x00-\x1f<>]*|email)>/,url:p,tag:"^comment|^</[a-zA-Z][\\w:-]*\\s*>|^<[a-zA-Z][\\w-]*(?:attribute)*?\\s*/?>|^<\\?[\\s\\S]*?\\?>|^<![a-zA-Z]+\\s[\\s\\S]*?>|^<!\\[CDATA\\[[\\s\\S]*?\\]\\]>",link:/^!?\[(label)\]\(href(?:\s+(title))?\s*\)/,reflink:/^!?\[(label)\]\[(?!\s*\])((?:\\[\[\]]?|[^\[\]\\])+)\]/,nolink:/^!?\[(?!\s*\])((?:\[[^\[\]]*\]|\\[\[\]]|[^\[\]])*)\](?:\[\])?/,strong:/^__([^\s_])__(?!_)|^\*\*([^\s*])\*\*(?!\*)|^__([^\s][\s\S]*?[^\s])__(?!_)|^\*\*([^\s][\s\S]*?[^\s])\*\*(?!\*)/,em:/^_([^\s_])_(?!_)|^\*([^\s*"<\[])\*(?!\*)|^_([^\s][\s\S]*?[^\s_])_(?!_|[^\spunctuation])|^_([^\s_][\s\S]*?[^\s])_(?!_|[^\spunctuation])|^\*([^\s"<\[][\s\S]*?[^\s*])\*(?!\*)|^\*([^\s*"<\[][\s\S]*?[^\s])\*(?!\*)/,code:/^(`+)([^`]|[^`][\s\S]*?[^`])\1(?!`)/,br:/^( {2,}|\\)\n(?!\s*$)/,del:p,text:/^(`+|[^`])(?:[\s\S]*?(?:(?=[\\<!\[`*]|\b_|$)|[^ ](?= {2,}\n))|(?= {2,}\n))/};function o(t,e){if(this.options=e||E.defaults,this.links=t,this.rules=i.normal,this.renderer=this.options.renderer||new a,this.renderer.options=this.options,!this.links)throw new Error("Tokens array requires a `links` property.");this.options.pedantic?this.rules=i.pedantic:this.options.gfm&&(this.options.breaks?this.rules=i.breaks:this.rules=i.gfm)}function a(t){this.options=t||E.defaults}function s(){}function A(t){this.tokens=[],this.token=null,this.options=t||E.defaults,this.options.renderer=this.options.renderer||new a,this.renderer=this.options.renderer,this.renderer.options=this.options,this.slugger=new l}function l(){this.seen={}}function c(t,e){if(e){if(c.escapeTest.test(t))return t.replace(c.escapeReplace,function(t){return c.replacements[t]})}else if(c.escapeTestNoEncode.test(t))return t.replace(c.escapeReplaceNoEncode,function(t){return c.replacements[t]});return t}function u(t){return t.replace(/&(#(?:\d+)|(?:#x[0-9A-Fa-f]+)|(?:\w+));?/gi,function(t,e){return"colon"===(e=e.toLowerCase())?":":"#"===e.charAt(0)?"x"===e.charAt(1)?String.fromCharCode(parseInt(e.substring(2),16)):String.fromCharCode(+e.substring(1)):""})}function g(t,e){return t=t.source||t,e=e||"",{replace:function(e,n){return n=(n=n.source||n).replace(/(^|[^\[])\^/g,"$1"),t=t.replace(e,n),this},getRegex:function(){return new RegExp(t,e)}}}function h(t,e,n){if(t){try{var r=decodeURIComponent(u(n)).replace(/[^\w:]/g,"").toLowerCase()}catch(t){return null}if(0===r.indexOf("javascript:")||0===r.indexOf("vbscript:")||0===r.indexOf("data:"))return null}e&&!f.test(n)&&(n=function(t,e){d[" "+t]||(/^[^:]+:\/*[^\/]*$/.test(t)?d[" "+t]=t+"/":d[" "+t]=m(t,"/",!0));return t=d[" "+t],"//"===e.slice(0,2)?t.replace(/:[\s\S]*/,":")+e:"/"===e.charAt(0)?t.replace(/(:\/*[^\/]*)[\s\S]*/,"$1")+e:t+e}(e,n));try{n=encodeURI(n).replace(/%25/g,"%")}catch(t){return null}return n}i._punctuation="!\"#$%&'()*+,\\-./:;<=>?@\\[^_{|}~",i.em=g(i.em).replace(/punctuation/g,i._punctuation).getRegex(),i._escapes=/\\([!"#$%&'()*+,\-.\/:;<=>?@\[\]\\^_`{|}~])/g,i._scheme=/[a-zA-Z][a-zA-Z0-9+.-]{1,31}/,i._email=/[a-zA-Z0-9.!#$%&'*+\/=?^_`{|}~-]+(@)[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)+(?![-_])/,i.autolink=g(i.autolink).replace("scheme",i._scheme).replace("email",i._email).getRegex(),i._attribute=/\s+[a-zA-Z:_][\w.:-]*(?:\s*=\s*"[^"]*"|\s*=\s*'[^']*'|\s*=\s*[^\s"'=<>`]+)?/,i.tag=g(i.tag).replace("comment",n._comment).replace("attribute",i._attribute).getRegex(),i._label=/(?:\[[^\[\]]*\]|\\[\[\]]?|`[^`]*`|`(?!`)|[^\[\]\\`])*?/,i._href=/\s*(<(?:\\[<>]?|[^\s<>\\])*>|[^\s\x00-\x1f]*)/,i._title=/"(?:\\"?|[^"\\])*"|'(?:\\'?|[^'\\])*'|\((?:\\\)?|[^)\\])*\)/,i.link=g(i.link).replace("label",i._label).replace("href",i._href).replace("title",i._title).getRegex(),i.reflink=g(i.reflink).replace("label",i._label).getRegex(),i.normal=I({},i),i.pedantic=I({},i.normal,{strong:/^__(?=\S)([\s\S]*?\S)__(?!_)|^\*\*(?=\S)([\s\S]*?\S)\*\*(?!\*)/,em:/^_(?=\S)([\s\S]*?\S)_(?!_)|^\*(?=\S)([\s\S]*?\S)\*(?!\*)/,link:g(/^!?\[(label)\]\((.*?)\)/).replace("label",i._label).getRegex(),reflink:g(/^!?\[(label)\]\s*\[([^\]]*)\]/).replace("label",i._label).getRegex()}),i.gfm=I({},i.normal,{escape:g(i.escape).replace("])","~|])").getRegex(),_extended_email:/[A-Za-z0-9._+-]+(@)[a-zA-Z0-9-_]+(?:\.[a-zA-Z0-9-_]*[a-zA-Z0-9])+(?![-_])/,url:/^((?:ftp|https?):\/\/|www\.)(?:[a-zA-Z0-9\-]+\.?)+[^\s<]*|^email/,_backpedal:/(?:[^?!.,:;*_~()&]+|\([^)]*\)|&(?![a-zA-Z0-9]+;$)|[?!.,:;*_~)]+(?!$))+/,del:/^~+(?=\S)([\s\S]*?\S)~+/,text:/^(`+|[^`])(?:[\s\S]*?(?:(?=[\\<!\[`*~]|\b_|https?:\/\/|ftp:\/\/|www\.|$)|[^ ](?= {2,}\n)|[^a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-](?=[a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-]+@))|(?= {2,}\n|[a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-]+@))/}),i.gfm.url=g(i.gfm.url,"i").replace("email",i.gfm._extended_email).getRegex(),i.breaks=I({},i.gfm,{br:g(i.br).replace("{2,}","*").getRegex(),text:g(i.gfm.text).replace(/\{2,\}/g,"*").getRegex()}),o.rules=i,o.output=function(t,e,n){return new o(e,n).output(t)},o.prototype.output=function(t){for(var e,n,r,i,a,s,A="";t;)if(a=this.rules.escape.exec(t))t=t.substring(a[0].length),A+=c(a[1]);else if(a=this.rules.tag.exec(t))!this.inLink&&/^<a /i.test(a[0])?this.inLink=!0:this.inLink&&/^<\/a>/i.test(a[0])&&(this.inLink=!1),!this.inRawBlock&&/^<(pre|code|kbd|script)(\s|>)/i.test(a[0])?this.inRawBlock=!0:this.inRawBlock&&/^<\/(pre|code|kbd|script)(\s|>)/i.test(a[0])&&(this.inRawBlock=!1),t=t.substring(a[0].length),A+=this.options.sanitize?this.options.sanitizer?this.options.sanitizer(a[0]):c(a[0]):a[0];else if(a=this.rules.link.exec(t)){var l=y(a[2],"()");if(l>-1){var u=a[0].length-(a[2].length-l)-(a[3]||"").length;a[2]=a[2].substring(0,l),a[0]=a[0].substring(0,u).trim(),a[3]=""}t=t.substring(a[0].length),this.inLink=!0,r=a[2],this.options.pedantic?(e=/^([^'"]*[^\s])\s+(['"])(.*)\2/.exec(r))?(r=e[1],i=e[3]):i="":i=a[3]?a[3].slice(1,-1):"",r=r.trim().replace(/^<([\s\S]*)>$/,"$1"),A+=this.outputLink(a,{href:o.escapes(r),title:o.escapes(i)}),this.inLink=!1}else if((a=this.rules.reflink.exec(t))||(a=this.rules.nolink.exec(t))){if(t=t.substring(a[0].length),e=(a[2]||a[1]).replace(/\s+/g," "),!(e=this.links[e.toLowerCase()])||!e.href){A+=a[0].charAt(0),t=a[0].substring(1)+t;continue}this.inLink=!0,A+=this.outputLink(a,e),this.inLink=!1}else if(a=this.rules.strong.exec(t))t=t.substring(a[0].length),A+=this.renderer.strong(this.output(a[4]||a[3]||a[2]||a[1]));else if(a=this.rules.em.exec(t))t=t.substring(a[0].length),A+=this.renderer.em(this.output(a[6]||a[5]||a[4]||a[3]||a[2]||a[1]));else if(a=this.rules.code.exec(t))t=t.substring(a[0].length),A+=this.renderer.codespan(c(a[2].trim(),!0));else if(a=this.rules.br.exec(t))t=t.substring(a[0].length),A+=this.renderer.br();else if(a=this.rules.del.exec(t))t=t.substring(a[0].length),A+=this.renderer.del(this.output(a[1]));else if(a=this.rules.autolink.exec(t))t=t.substring(a[0].length),r="@"===a[2]?"mailto:"+(n=c(this.mangle(a[1]))):n=c(a[1]),A+=this.renderer.link(r,null,n);else if(this.inLink||!(a=this.rules.url.exec(t))){if(a=this.rules.text.exec(t))t=t.substring(a[0].length),this.inRawBlock?A+=this.renderer.text(a[0]):A+=this.renderer.text(c(this.smartypants(a[0])));else if(t)throw new Error("Infinite loop on byte: "+t.charCodeAt(0))}else{if("@"===a[2])r="mailto:"+(n=c(a[0]));else{do{s=a[0],a[0]=this.rules._backpedal.exec(a[0])[0]}while(s!==a[0]);n=c(a[0]),r="www."===a[1]?"http://"+n:n}t=t.substring(a[0].length),A+=this.renderer.link(r,null,n)}return A},o.escapes=function(t){return t?t.replace(o.rules._escapes,"$1"):t},o.prototype.outputLink=function(t,e){var n=e.href,r=e.title?c(e.title):null;return"!"!==t[0].charAt(0)?this.renderer.link(n,r,this.output(t[1])):this.renderer.image(n,r,c(t[1]))},o.prototype.smartypants=function(t){return this.options.smartypants?t.replace(/---/g,"—").replace(/--/g,"–").replace(/(^|[-\u2014\/(\[{"\s])'/g,"$1‘").replace(/'/g,"’").replace(/(^|[-\u2014\/(\[{\u2018\s])"/g,"$1“").replace(/"/g,"”").replace(/\.{3}/g,"…"):t},o.prototype.mangle=function(t){if(!this.options.mangle)return t;for(var e,n="",r=t.length,i=0;i<r;i++)e=t.charCodeAt(i),Math.random()>.5&&(e="x"+e.toString(16)),n+="&#"+e+";";return n},a.prototype.code=function(t,e,n){var r=(e||"").match(/\S*/)[0];if(this.options.highlight){var i=this.options.highlight(t,r);null!=i&&i!==t&&(n=!0,t=i)}return r?'<pre><code class="'+this.options.langPrefix+c(r,!0)+'">'+(n?t:c(t,!0))+"</code></pre>\n":"<pre><code>"+(n?t:c(t,!0))+"</code></pre>"},a.prototype.blockquote=function(t){return"<blockquote>\n"+t+"</blockquote>\n"},a.prototype.html=function(t){return t},a.prototype.heading=function(t,e,n,r){return this.options.headerIds?"<h"+e+' id="'+this.options.headerPrefix+r.slug(n)+'">'+t+"</h"+e+">\n":"<h"+e+">"+t+"</h"+e+">\n"},a.prototype.hr=function(){return this.options.xhtml?"<hr/>\n":"<hr>\n"},a.prototype.list=function(t,e,n){var r=e?"ol":"ul";return"<"+r+(e&&1!==n?' start="'+n+'"':"")+">\n"+t+"</"+r+">\n"},a.prototype.listitem=function(t){return"<li>"+t+"</li>\n"},a.prototype.checkbox=function(t){return"<input "+(t?'checked="" ':"")+'disabled="" type="checkbox"'+(this.options.xhtml?" /":"")+"> "},a.prototype.paragraph=function(t){return"<p>"+t+"</p>\n"},a.prototype.table=function(t,e){return e&&(e="<tbody>"+e+"</tbody>"),"<table>\n<thead>\n"+t+"</thead>\n"+e+"</table>\n"},a.prototype.tablerow=function(t){return"<tr>\n"+t+"</tr>\n"},a.prototype.tablecell=function(t,e){var n=e.header?"th":"td";return(e.align?"<"+n+' align="'+e.align+'">':"<"+n+">")+t+"</"+n+">\n"},a.prototype.strong=function(t){return"<strong>"+t+"</strong>"},a.prototype.em=function(t){return"<em>"+t+"</em>"},a.prototype.codespan=function(t){return"<code>"+t+"</code>"},a.prototype.br=function(){return this.options.xhtml?"<br/>":"<br>"},a.prototype.del=function(t){return"<del>"+t+"</del>"},a.prototype.link=function(t,e,n){if(null===(t=h(this.options.sanitize,this.options.baseUrl,t)))return n;var r='<a href="'+c(t)+'"';return e&&(r+=' title="'+e+'"'),r+=">"+n+"</a>"},a.prototype.image=function(t,e,n){if(null===(t=h(this.options.sanitize,this.options.baseUrl,t)))return n;var r='<img src="'+t+'" alt="'+n+'"';return e&&(r+=' title="'+e+'"'),r+=this.options.xhtml?"/>":">"},a.prototype.text=function(t){return t},s.prototype.strong=s.prototype.em=s.prototype.codespan=s.prototype.del=s.prototype.text=function(t){return t},s.prototype.link=s.prototype.image=function(t,e,n){return""+n},s.prototype.br=function(){return""},A.parse=function(t,e){return new A(e).parse(t)},A.prototype.parse=function(t){this.inline=new o(t.links,this.options),this.inlineText=new o(t.links,I({},this.options,{renderer:new s})),this.tokens=t.reverse();for(var e="";this.next();)e+=this.tok();return e},A.prototype.next=function(){return this.token=this.tokens.pop()},A.prototype.peek=function(){return this.tokens[this.tokens.length-1]||0},A.prototype.parseText=function(){for(var t=this.token.text;"text"===this.peek().type;)t+="\n"+this.next().text;return this.inline.output(t)},A.prototype.tok=function(){switch(this.token.type){case"space":return"";case"hr":return this.renderer.hr();case"heading":return this.renderer.heading(this.inline.output(this.token.text),this.token.depth,u(this.inlineText.output(this.token.text)),this.slugger);case"code":return this.renderer.code(this.token.text,this.token.lang,this.token.escaped);case"table":var t,e,n,r,i="",o="";for(n="",t=0;t<this.token.header.length;t++)n+=this.renderer.tablecell(this.inline.output(this.token.header[t]),{header:!0,align:this.token.align[t]});for(i+=this.renderer.tablerow(n),t=0;t<this.token.cells.length;t++){for(e=this.token.cells[t],n="",r=0;r<e.length;r++)n+=this.renderer.tablecell(this.inline.output(e[r]),{header:!1,align:this.token.align[r]});o+=this.renderer.tablerow(n)}return this.renderer.table(i,o);case"blockquote_start":for(o="";"blockquote_end"!==this.next().type;)o+=this.tok();return this.renderer.blockquote(o);case"list_start":o="";for(var a=this.token.ordered,s=this.token.start;"list_end"!==this.next().type;)o+=this.tok();return this.renderer.list(o,a,s);case"list_item_start":o="";var A=this.token.loose,l=this.token.checked,c=this.token.task;for(this.token.task&&(o+=this.renderer.checkbox(l));"list_item_end"!==this.next().type;)o+=A||"text"!==this.token.type?this.tok():this.parseText();return this.renderer.listitem(o,c,l);case"html":return this.renderer.html(this.token.text);case"paragraph":return this.renderer.paragraph(this.inline.output(this.token.text));case"text":return this.renderer.paragraph(this.parseText());default:var g='Token with "'+this.token.type+'" type was not found.';if(!this.options.silent)throw new Error(g);console.log(g)}},l.prototype.slug=function(t){var e=t.toLowerCase().trim().replace(/[\u2000-\u206F\u2E00-\u2E7F\\'!"#$%&()*+,.\/:;<=>?@[\]^`{|}~]/g,"").replace(/\s/g,"-");if(this.seen.hasOwnProperty(e)){var n=e;do{this.seen[n]++,e=n+"-"+this.seen[n]}while(this.seen.hasOwnProperty(e))}return this.seen[e]=0,e},c.escapeTest=/[&<>"']/,c.escapeReplace=/[&<>"']/g,c.replacements={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"},c.escapeTestNoEncode=/[<>"']|&(?!#?\w+;)/,c.escapeReplaceNoEncode=/[<>"']|&(?!#?\w+;)/g;var d={},f=/^$|^[a-z][a-z0-9+.-]*:|^[?#]/i;function p(){}function I(t){for(var e,n,r=1;r<arguments.length;r++)for(n in e=arguments[r])Object.prototype.hasOwnProperty.call(e,n)&&(t[n]=e[n]);return t}function C(t,e){var n=t.replace(/\|/g,function(t,e,n){for(var r=!1,i=e;--i>=0&&"\\"===n[i];)r=!r;return r?"|":" |"}).split(/ \|/),r=0;if(n.length>e)n.splice(e);else for(;n.length<e;)n.push("");for(;r<n.length;r++)n[r]=n[r].trim().replace(/\\\|/g,"|");return n}function m(t,e,n){if(0===t.length)return"";for(var r=0;r<t.length;){var i=t.charAt(t.length-r-1);if(i!==e||n){if(i===e||!n)break;r++}else r++}return t.substr(0,t.length-r)}function y(t,e){if(-1===t.indexOf(e[1]))return-1;for(var n=0,r=0;r<t.length;r++)if("\\"===t[r])r++;else if(t[r]===e[0])n++;else if(t[r]===e[1]&&--n<0)return r;return-1}function E(t,e,n){if(null==t)throw new Error("marked(): input parameter is undefined or null");if("string"!=typeof t)throw new Error("marked(): input parameter is of type "+Object.prototype.toString.call(t)+", string expected");if(n||"function"==typeof e){n||(n=e,e=null);var i,o,a=(e=I({},E.defaults,e||{})).highlight,s=0;try{i=r.lex(t,e)}catch(t){return n(t)}o=i.length;var l=function(t){if(t)return e.highlight=a,n(t);var r;try{r=A.parse(i,e)}catch(e){t=e}return e.highlight=a,t?n(t):n(null,r)};if(!a||a.length<3)return l();if(delete e.highlight,!o)return l();for(;s<i.length;s++)!function(t){"code"!==t.type?--o||l():a(t.text,t.lang,function(e,n){return e?l(e):null==n||n===t.text?--o||l():(t.text=n,t.escaped=!0,void(--o||l()))})}(i[s])}else try{return e&&(e=I({},E.defaults,e)),A.parse(r.lex(t,e),e)}catch(t){if(t.message+="\nPlease report this to https://github.com/markedjs/marked.",(e||E.defaults).silent)return"<p>An error occurred:</p><pre>"+c(t.message+"",!0)+"</pre>";throw t}}p.exec=p,E.options=E.setOptions=function(t){return I(E.defaults,t),E},E.getDefaults=function(){return{baseUrl:null,breaks:!1,gfm:!0,headerIds:!0,headerPrefix:"",highlight:null,langPrefix:"language-",mangle:!0,pedantic:!1,renderer:new a,sanitize:!1,sanitizer:null,silent:!1,smartLists:!1,smartypants:!1,tables:!0,xhtml:!1}},E.defaults=E.getDefaults(),E.Parser=A,E.parser=A.parse,E.Renderer=a,E.TextRenderer=s,E.Lexer=r,E.lexer=r.lex,E.InlineLexer=o,E.inlineLexer=o.output,E.Slugger=l,E.parse=E,t.exports=E}(this||"undefined"!=typeof window&&window)}).call(this,n(37))},function(t,e,n){"use strict";n.d(e,"a",function(){return a});var r,i=n(5),o=n(10);class a{constructor(t={}){this._array=[],this._isDisposed=!1,this._changed=new o.Signal(this),void 0!==t.values&&Object(i.each)(t.values,t=>{this._array.push(t)}),this._itemCmp=t.itemCmp||r.itemCmp}get type(){return"List"}get changed(){return this._changed}get length(){return this._array.length}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this._isDisposed=!0,o.Signal.clearData(this),this.clear())}iter(){return new i.ArrayIterator(this._array)}get(t){return this._array[t]}set(t,e){let n=this._array[t];if(void 0===e)throw new Error("Cannot set an undefined item");(0,this._itemCmp)(n,e)||(this._array[t]=e,this._changed.emit({type:"set",oldIndex:t,newIndex:t,oldValues:[n],newValues:[e]}))}push(t){let e=this._array.push(t);return this._changed.emit({type:"add",oldIndex:-1,newIndex:this.length-1,oldValues:[],newValues:[t]}),e}insert(t,e){i.ArrayExt.insert(this._array,t,e),this._changed.emit({type:"add",oldIndex:-1,newIndex:t,oldValues:[],newValues:[e]})}removeValue(t){let e=this._itemCmp,n=i.ArrayExt.findFirstIndex(this._array,n=>e(n,t));return this.remove(n),n}remove(t){let e=i.ArrayExt.removeAt(this._array,t);if(void 0!==e)return this._changed.emit({type:"remove",oldIndex:t,newIndex:-1,newValues:[],oldValues:[e]}),e}clear(){let t=this._array.slice();this._array.length=0,this._changed.emit({type:"remove",oldIndex:0,newIndex:0,newValues:[],oldValues:t})}move(t,e){if(this.length<=1||t===e)return;let n=[this._array[t]];i.ArrayExt.move(this._array,t,e),this._changed.emit({type:"move",oldIndex:t,newIndex:e,oldValues:n,newValues:n})}pushAll(t){let e=this.length;return Object(i.each)(t,t=>{this._array.push(t)}),this._changed.emit({type:"add",oldIndex:-1,newIndex:e,oldValues:[],newValues:Object(i.toArray)(t)}),this.length}insertAll(t,e){let n=t;Object(i.each)(e,e=>{i.ArrayExt.insert(this._array,t++,e)}),this._changed.emit({type:"add",oldIndex:-1,newIndex:n,oldValues:[],newValues:Object(i.toArray)(e)})}removeRange(t,e){let n=this._array.slice(t,e);for(let n=t;n<e;n++)i.ArrayExt.removeAt(this._array,t);return this._changed.emit({type:"remove",oldIndex:t,newIndex:-1,oldValues:n,newValues:[]}),this.length}}!function(t){t.itemCmp=function(t,e){return t===e}}(r||(r={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2),i=r.__importDefault(n(4)),o=r.__importStar(n(0)),a=r.__importStar(n(139)),s=n(177),A=n(190),l=10,c=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={rightElementWidth:l},e.refHandlers={rightElement:function(t){return e.rightElement=t}},e}return r.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.className,l=e.disabled,c=e.fill,u=e.intent,g=e.large,h=e.small,d=e.leftIcon,f=e.round,p=i.default(a.INPUT_GROUP,a.intentClass(u),((t={})[a.DISABLED]=l,t[a.FILL]=c,t[a.LARGE]=g,t[a.SMALL]=h,t[a.ROUND]=f,t),n),I=r.__assign({},this.props.style,{paddingRight:this.state.rightElementWidth});return o.createElement("div",{className:p},o.createElement(A.Icon,{icon:d}),o.createElement("input",r.__assign({type:"text"},s.removeNonHTMLProps(this.props),{className:a.INPUT,ref:this.props.inputRef,style:I})),this.maybeRenderRightElement())},e.prototype.componentDidMount=function(){this.updateInputWidth()},e.prototype.componentDidUpdate=function(){this.updateInputWidth()},e.prototype.maybeRenderRightElement=function(){var t=this.props.rightElement;if(null!=t)return o.createElement("span",{className:a.INPUT_ACTION,ref:this.refHandlers.rightElement},t)},e.prototype.updateInputWidth=function(){if(null!=this.rightElement){var t=this.rightElement.clientWidth;Math.abs(t-this.state.rightElementWidth)>2&&this.setState({rightElementWidth:t})}else this.setState({rightElementWidth:l})},e.displayName=s.DISPLAYNAME_PREFIX+".InputGroup",e}(o.PureComponent);e.InputGroup=c},function(t,e,n){"use strict";(function(t){for(
/**!
 * @fileOverview Kickass library to create and place poppers near their reference elements.
 * @version 1.15.0
 * @license
 * Copyright (c) 2016 Federico Zivolo and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var n="undefined"!=typeof window&&"undefined"!=typeof document,r=["Edge","Trident","Firefox"],i=0,o=0;o<r.length;o+=1)if(n&&navigator.userAgent.indexOf(r[o])>=0){i=1;break}var a=n&&window.Promise?function(t){var e=!1;return function(){e||(e=!0,window.Promise.resolve().then(function(){e=!1,t()}))}}:function(t){var e=!1;return function(){e||(e=!0,setTimeout(function(){e=!1,t()},i))}};function s(t){return t&&"[object Function]"==={}.toString.call(t)}function A(t,e){if(1!==t.nodeType)return[];var n=t.ownerDocument.defaultView.getComputedStyle(t,null);return e?n[e]:n}function l(t){return"HTML"===t.nodeName?t:t.parentNode||t.host}function c(t){if(!t)return document.body;switch(t.nodeName){case"HTML":case"BODY":return t.ownerDocument.body;case"#document":return t.body}var e=A(t),n=e.overflow,r=e.overflowX,i=e.overflowY;return/(auto|scroll|overlay)/.test(n+i+r)?t:c(l(t))}var u=n&&!(!window.MSInputMethodContext||!document.documentMode),g=n&&/MSIE 10/.test(navigator.userAgent);function h(t){return 11===t?u:10===t?g:u||g}function d(t){if(!t)return document.documentElement;for(var e=h(10)?document.body:null,n=t.offsetParent||null;n===e&&t.nextElementSibling;)n=(t=t.nextElementSibling).offsetParent;var r=n&&n.nodeName;return r&&"BODY"!==r&&"HTML"!==r?-1!==["TH","TD","TABLE"].indexOf(n.nodeName)&&"static"===A(n,"position")?d(n):n:t?t.ownerDocument.documentElement:document.documentElement}function f(t){return null!==t.parentNode?f(t.parentNode):t}function p(t,e){if(!(t&&t.nodeType&&e&&e.nodeType))return document.documentElement;var n=t.compareDocumentPosition(e)&Node.DOCUMENT_POSITION_FOLLOWING,r=n?t:e,i=n?e:t,o=document.createRange();o.setStart(r,0),o.setEnd(i,0);var a,s,A=o.commonAncestorContainer;if(t!==A&&e!==A||r.contains(i))return"BODY"===(s=(a=A).nodeName)||"HTML"!==s&&d(a.firstElementChild)!==a?d(A):A;var l=f(t);return l.host?p(l.host,e):p(t,f(e).host)}function I(t){var e="top"===(arguments.length>1&&void 0!==arguments[1]?arguments[1]:"top")?"scrollTop":"scrollLeft",n=t.nodeName;if("BODY"===n||"HTML"===n){var r=t.ownerDocument.documentElement;return(t.ownerDocument.scrollingElement||r)[e]}return t[e]}function C(t,e){var n="x"===e?"Left":"Top",r="Left"===n?"Right":"Bottom";return parseFloat(t["border"+n+"Width"],10)+parseFloat(t["border"+r+"Width"],10)}function m(t,e,n,r){return Math.max(e["offset"+t],e["scroll"+t],n["client"+t],n["offset"+t],n["scroll"+t],h(10)?parseInt(n["offset"+t])+parseInt(r["margin"+("Height"===t?"Top":"Left")])+parseInt(r["margin"+("Height"===t?"Bottom":"Right")]):0)}function y(t){var e=t.body,n=t.documentElement,r=h(10)&&getComputedStyle(n);return{height:m("Height",e,n,r),width:m("Width",e,n,r)}}var E=function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")},v=function(){function t(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}return function(e,n,r){return n&&t(e.prototype,n),r&&t(e,r),e}}(),B=function(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t},b=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t};function Q(t){return b({},t,{right:t.left+t.width,bottom:t.top+t.height})}function w(t){var e={};try{if(h(10)){e=t.getBoundingClientRect();var n=I(t,"top"),r=I(t,"left");e.top+=n,e.left+=r,e.bottom+=n,e.right+=r}else e=t.getBoundingClientRect()}catch(t){}var i={left:e.left,top:e.top,width:e.right-e.left,height:e.bottom-e.top},o="HTML"===t.nodeName?y(t.ownerDocument):{},a=o.width||t.clientWidth||i.right-i.left,s=o.height||t.clientHeight||i.bottom-i.top,l=t.offsetWidth-a,c=t.offsetHeight-s;if(l||c){var u=A(t);l-=C(u,"x"),c-=C(u,"y"),i.width-=l,i.height-=c}return Q(i)}function x(t,e){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=h(10),i="HTML"===e.nodeName,o=w(t),a=w(e),s=c(t),l=A(e),u=parseFloat(l.borderTopWidth,10),g=parseFloat(l.borderLeftWidth,10);n&&i&&(a.top=Math.max(a.top,0),a.left=Math.max(a.left,0));var d=Q({top:o.top-a.top-u,left:o.left-a.left-g,width:o.width,height:o.height});if(d.marginTop=0,d.marginLeft=0,!r&&i){var f=parseFloat(l.marginTop,10),p=parseFloat(l.marginLeft,10);d.top-=u-f,d.bottom-=u-f,d.left-=g-p,d.right-=g-p,d.marginTop=f,d.marginLeft=p}return(r&&!n?e.contains(s):e===s&&"BODY"!==s.nodeName)&&(d=function(t,e){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=I(e,"top"),i=I(e,"left"),o=n?-1:1;return t.top+=r*o,t.bottom+=r*o,t.left+=i*o,t.right+=i*o,t}(d,e)),d}function _(t){if(!t||!t.parentElement||h())return document.documentElement;for(var e=t.parentElement;e&&"none"===A(e,"transform");)e=e.parentElement;return e||document.documentElement}function D(t,e,n,r){var i=arguments.length>4&&void 0!==arguments[4]&&arguments[4],o={top:0,left:0},a=i?_(t):p(t,e);if("viewport"===r)o=function(t){var e=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=t.ownerDocument.documentElement,r=x(t,n),i=Math.max(n.clientWidth,window.innerWidth||0),o=Math.max(n.clientHeight,window.innerHeight||0),a=e?0:I(n),s=e?0:I(n,"left");return Q({top:a-r.top+r.marginTop,left:s-r.left+r.marginLeft,width:i,height:o})}(a,i);else{var s=void 0;"scrollParent"===r?"BODY"===(s=c(l(e))).nodeName&&(s=t.ownerDocument.documentElement):s="window"===r?t.ownerDocument.documentElement:r;var u=x(s,a,i);if("HTML"!==s.nodeName||function t(e){var n=e.nodeName;if("BODY"===n||"HTML"===n)return!1;if("fixed"===A(e,"position"))return!0;var r=l(e);return!!r&&t(r)}(a))o=u;else{var g=y(t.ownerDocument),h=g.height,d=g.width;o.top+=u.top-u.marginTop,o.bottom=h+u.top,o.left+=u.left-u.marginLeft,o.right=d+u.left}}var f="number"==typeof(n=n||0);return o.left+=f?n:n.left||0,o.top+=f?n:n.top||0,o.right-=f?n:n.right||0,o.bottom-=f?n:n.bottom||0,o}function M(t,e,n,r,i){var o=arguments.length>5&&void 0!==arguments[5]?arguments[5]:0;if(-1===t.indexOf("auto"))return t;var a=D(n,r,o,i),s={top:{width:a.width,height:e.top-a.top},right:{width:a.right-e.right,height:a.height},bottom:{width:a.width,height:a.bottom-e.bottom},left:{width:e.left-a.left,height:a.height}},A=Object.keys(s).map(function(t){return b({key:t},s[t],{area:(e=s[t],e.width*e.height)});var e}).sort(function(t,e){return e.area-t.area}),l=A.filter(function(t){var e=t.width,r=t.height;return e>=n.clientWidth&&r>=n.clientHeight}),c=l.length>0?l[0].key:A[0].key,u=t.split("-")[1];return c+(u?"-"+u:"")}function k(t,e,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:null;return x(n,r?_(e):p(e,n),r)}function S(t){var e=t.ownerDocument.defaultView.getComputedStyle(t),n=parseFloat(e.marginTop||0)+parseFloat(e.marginBottom||0),r=parseFloat(e.marginLeft||0)+parseFloat(e.marginRight||0);return{width:t.offsetWidth+r,height:t.offsetHeight+n}}function L(t){var e={left:"right",right:"left",bottom:"top",top:"bottom"};return t.replace(/left|right|bottom|top/g,function(t){return e[t]})}function R(t,e,n){n=n.split("-")[0];var r=S(t),i={width:r.width,height:r.height},o=-1!==["right","left"].indexOf(n),a=o?"top":"left",s=o?"left":"top",A=o?"height":"width",l=o?"width":"height";return i[a]=e[a]+e[A]/2-r[A]/2,i[s]=n===s?e[s]-r[l]:e[L(s)],i}function N(t,e){return Array.prototype.find?t.find(e):t.filter(e)[0]}function F(t,e,n){return(void 0===n?t:t.slice(0,function(t,e,n){if(Array.prototype.findIndex)return t.findIndex(function(t){return t[e]===n});var r=N(t,function(t){return t[e]===n});return t.indexOf(r)}(t,"name",n))).forEach(function(t){t.function&&console.warn("`modifier.function` is deprecated, use `modifier.fn`!");var n=t.function||t.fn;t.enabled&&s(n)&&(e.offsets.popper=Q(e.offsets.popper),e.offsets.reference=Q(e.offsets.reference),e=n(e,t))}),e}function T(t,e){return t.some(function(t){var n=t.name;return t.enabled&&n===e})}function U(t){for(var e=[!1,"ms","Webkit","Moz","O"],n=t.charAt(0).toUpperCase()+t.slice(1),r=0;r<e.length;r++){var i=e[r],o=i?""+i+n:t;if(void 0!==document.body.style[o])return o}return null}function O(t){var e=t.ownerDocument;return e?e.defaultView:window}function G(t,e,n,r){n.updateBound=r,O(t).addEventListener("resize",n.updateBound,{passive:!0});var i=c(t);return function t(e,n,r,i){var o="BODY"===e.nodeName,a=o?e.ownerDocument.defaultView:e;a.addEventListener(n,r,{passive:!0}),o||t(c(a.parentNode),n,r,i),i.push(a)}(i,"scroll",n.updateBound,n.scrollParents),n.scrollElement=i,n.eventsEnabled=!0,n}function H(){var t,e;this.state.eventsEnabled&&(cancelAnimationFrame(this.scheduleUpdate),this.state=(t=this.reference,e=this.state,O(t).removeEventListener("resize",e.updateBound),e.scrollParents.forEach(function(t){t.removeEventListener("scroll",e.updateBound)}),e.updateBound=null,e.scrollParents=[],e.scrollElement=null,e.eventsEnabled=!1,e))}function j(t){return""!==t&&!isNaN(parseFloat(t))&&isFinite(t)}function Y(t,e){Object.keys(e).forEach(function(n){var r="";-1!==["width","height","top","right","bottom","left"].indexOf(n)&&j(e[n])&&(r="px"),t.style[n]=e[n]+r})}var P=n&&/Firefox/i.test(navigator.userAgent);function K(t,e,n){var r=N(t,function(t){return t.name===e}),i=!!r&&t.some(function(t){return t.name===n&&t.enabled&&t.order<r.order});if(!i){var o="`"+e+"`",a="`"+n+"`";console.warn(a+" modifier is required by "+o+" modifier in order to work, be sure to include it before "+o+"!")}return i}var z=["auto-start","auto","auto-end","top-start","top","top-end","right-start","right","right-end","bottom-end","bottom","bottom-start","left-end","left","left-start"],J=z.slice(3);function q(t){var e=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=J.indexOf(t),r=J.slice(n+1).concat(J.slice(0,n));return e?r.reverse():r}var V={FLIP:"flip",CLOCKWISE:"clockwise",COUNTERCLOCKWISE:"counterclockwise"};function W(t,e,n,r){var i=[0,0],o=-1!==["right","left"].indexOf(r),a=t.split(/(\+|\-)/).map(function(t){return t.trim()}),s=a.indexOf(N(a,function(t){return-1!==t.search(/,|\s/)}));a[s]&&-1===a[s].indexOf(",")&&console.warn("Offsets separated by white space(s) are deprecated, use a comma (,) instead.");var A=/\s*,\s*|\s+/,l=-1!==s?[a.slice(0,s).concat([a[s].split(A)[0]]),[a[s].split(A)[1]].concat(a.slice(s+1))]:[a];return(l=l.map(function(t,r){var i=(1===r?!o:o)?"height":"width",a=!1;return t.reduce(function(t,e){return""===t[t.length-1]&&-1!==["+","-"].indexOf(e)?(t[t.length-1]=e,a=!0,t):a?(t[t.length-1]+=e,a=!1,t):t.concat(e)},[]).map(function(t){return function(t,e,n,r){var i=t.match(/((?:\-|\+)?\d*\.?\d*)(.*)/),o=+i[1],a=i[2];if(!o)return t;if(0===a.indexOf("%")){var s=void 0;switch(a){case"%p":s=n;break;case"%":case"%r":default:s=r}return Q(s)[e]/100*o}if("vh"===a||"vw"===a)return("vh"===a?Math.max(document.documentElement.clientHeight,window.innerHeight||0):Math.max(document.documentElement.clientWidth,window.innerWidth||0))/100*o;return o}(t,i,e,n)})})).forEach(function(t,e){t.forEach(function(n,r){j(n)&&(i[e]+=n*("-"===t[r-1]?-1:1))})}),i}var X={placement:"bottom",positionFixed:!1,eventsEnabled:!0,removeOnDestroy:!1,onCreate:function(){},onUpdate:function(){},modifiers:{shift:{order:100,enabled:!0,fn:function(t){var e=t.placement,n=e.split("-")[0],r=e.split("-")[1];if(r){var i=t.offsets,o=i.reference,a=i.popper,s=-1!==["bottom","top"].indexOf(n),A=s?"left":"top",l=s?"width":"height",c={start:B({},A,o[A]),end:B({},A,o[A]+o[l]-a[l])};t.offsets.popper=b({},a,c[r])}return t}},offset:{order:200,enabled:!0,fn:function(t,e){var n=e.offset,r=t.placement,i=t.offsets,o=i.popper,a=i.reference,s=r.split("-")[0],A=void 0;return A=j(+n)?[+n,0]:W(n,o,a,s),"left"===s?(o.top+=A[0],o.left-=A[1]):"right"===s?(o.top+=A[0],o.left+=A[1]):"top"===s?(o.left+=A[0],o.top-=A[1]):"bottom"===s&&(o.left+=A[0],o.top+=A[1]),t.popper=o,t},offset:0},preventOverflow:{order:300,enabled:!0,fn:function(t,e){var n=e.boundariesElement||d(t.instance.popper);t.instance.reference===n&&(n=d(n));var r=U("transform"),i=t.instance.popper.style,o=i.top,a=i.left,s=i[r];i.top="",i.left="",i[r]="";var A=D(t.instance.popper,t.instance.reference,e.padding,n,t.positionFixed);i.top=o,i.left=a,i[r]=s,e.boundaries=A;var l=e.priority,c=t.offsets.popper,u={primary:function(t){var n=c[t];return c[t]<A[t]&&!e.escapeWithReference&&(n=Math.max(c[t],A[t])),B({},t,n)},secondary:function(t){var n="right"===t?"left":"top",r=c[n];return c[t]>A[t]&&!e.escapeWithReference&&(r=Math.min(c[n],A[t]-("right"===t?c.width:c.height))),B({},n,r)}};return l.forEach(function(t){var e=-1!==["left","top"].indexOf(t)?"primary":"secondary";c=b({},c,u[e](t))}),t.offsets.popper=c,t},priority:["left","right","top","bottom"],padding:5,boundariesElement:"scrollParent"},keepTogether:{order:400,enabled:!0,fn:function(t){var e=t.offsets,n=e.popper,r=e.reference,i=t.placement.split("-")[0],o=Math.floor,a=-1!==["top","bottom"].indexOf(i),s=a?"right":"bottom",A=a?"left":"top",l=a?"width":"height";return n[s]<o(r[A])&&(t.offsets.popper[A]=o(r[A])-n[l]),n[A]>o(r[s])&&(t.offsets.popper[A]=o(r[s])),t}},arrow:{order:500,enabled:!0,fn:function(t,e){var n;if(!K(t.instance.modifiers,"arrow","keepTogether"))return t;var r=e.element;if("string"==typeof r){if(!(r=t.instance.popper.querySelector(r)))return t}else if(!t.instance.popper.contains(r))return console.warn("WARNING: `arrow.element` must be child of its popper element!"),t;var i=t.placement.split("-")[0],o=t.offsets,a=o.popper,s=o.reference,l=-1!==["left","right"].indexOf(i),c=l?"height":"width",u=l?"Top":"Left",g=u.toLowerCase(),h=l?"left":"top",d=l?"bottom":"right",f=S(r)[c];s[d]-f<a[g]&&(t.offsets.popper[g]-=a[g]-(s[d]-f)),s[g]+f>a[d]&&(t.offsets.popper[g]+=s[g]+f-a[d]),t.offsets.popper=Q(t.offsets.popper);var p=s[g]+s[c]/2-f/2,I=A(t.instance.popper),C=parseFloat(I["margin"+u],10),m=parseFloat(I["border"+u+"Width"],10),y=p-t.offsets.popper[g]-C-m;return y=Math.max(Math.min(a[c]-f,y),0),t.arrowElement=r,t.offsets.arrow=(B(n={},g,Math.round(y)),B(n,h,""),n),t},element:"[x-arrow]"},flip:{order:600,enabled:!0,fn:function(t,e){if(T(t.instance.modifiers,"inner"))return t;if(t.flipped&&t.placement===t.originalPlacement)return t;var n=D(t.instance.popper,t.instance.reference,e.padding,e.boundariesElement,t.positionFixed),r=t.placement.split("-")[0],i=L(r),o=t.placement.split("-")[1]||"",a=[];switch(e.behavior){case V.FLIP:a=[r,i];break;case V.CLOCKWISE:a=q(r);break;case V.COUNTERCLOCKWISE:a=q(r,!0);break;default:a=e.behavior}return a.forEach(function(s,A){if(r!==s||a.length===A+1)return t;r=t.placement.split("-")[0],i=L(r);var l=t.offsets.popper,c=t.offsets.reference,u=Math.floor,g="left"===r&&u(l.right)>u(c.left)||"right"===r&&u(l.left)<u(c.right)||"top"===r&&u(l.bottom)>u(c.top)||"bottom"===r&&u(l.top)<u(c.bottom),h=u(l.left)<u(n.left),d=u(l.right)>u(n.right),f=u(l.top)<u(n.top),p=u(l.bottom)>u(n.bottom),I="left"===r&&h||"right"===r&&d||"top"===r&&f||"bottom"===r&&p,C=-1!==["top","bottom"].indexOf(r),m=!!e.flipVariations&&(C&&"start"===o&&h||C&&"end"===o&&d||!C&&"start"===o&&f||!C&&"end"===o&&p),y=!!e.flipVariationsByContent&&(C&&"start"===o&&d||C&&"end"===o&&h||!C&&"start"===o&&p||!C&&"end"===o&&f),E=m||y;(g||I||E)&&(t.flipped=!0,(g||I)&&(r=a[A+1]),E&&(o=function(t){return"end"===t?"start":"start"===t?"end":t}(o)),t.placement=r+(o?"-"+o:""),t.offsets.popper=b({},t.offsets.popper,R(t.instance.popper,t.offsets.reference,t.placement)),t=F(t.instance.modifiers,t,"flip"))}),t},behavior:"flip",padding:5,boundariesElement:"viewport",flipVariations:!1,flipVariationsByContent:!1},inner:{order:700,enabled:!1,fn:function(t){var e=t.placement,n=e.split("-")[0],r=t.offsets,i=r.popper,o=r.reference,a=-1!==["left","right"].indexOf(n),s=-1===["top","left"].indexOf(n);return i[a?"left":"top"]=o[n]-(s?i[a?"width":"height"]:0),t.placement=L(e),t.offsets.popper=Q(i),t}},hide:{order:800,enabled:!0,fn:function(t){if(!K(t.instance.modifiers,"hide","preventOverflow"))return t;var e=t.offsets.reference,n=N(t.instance.modifiers,function(t){return"preventOverflow"===t.name}).boundaries;if(e.bottom<n.top||e.left>n.right||e.top>n.bottom||e.right<n.left){if(!0===t.hide)return t;t.hide=!0,t.attributes["x-out-of-boundaries"]=""}else{if(!1===t.hide)return t;t.hide=!1,t.attributes["x-out-of-boundaries"]=!1}return t}},computeStyle:{order:850,enabled:!0,fn:function(t,e){var n=e.x,r=e.y,i=t.offsets.popper,o=N(t.instance.modifiers,function(t){return"applyStyle"===t.name}).gpuAcceleration;void 0!==o&&console.warn("WARNING: `gpuAcceleration` option moved to `computeStyle` modifier and will not be supported in future versions of Popper.js!");var a=void 0!==o?o:e.gpuAcceleration,s=d(t.instance.popper),A=w(s),l={position:i.position},c=function(t,e){var n=t.offsets,r=n.popper,i=n.reference,o=Math.round,a=Math.floor,s=function(t){return t},A=o(i.width),l=o(r.width),c=-1!==["left","right"].indexOf(t.placement),u=-1!==t.placement.indexOf("-"),g=e?c||u||A%2==l%2?o:a:s,h=e?o:s;return{left:g(A%2==1&&l%2==1&&!u&&e?r.left-1:r.left),top:h(r.top),bottom:h(r.bottom),right:g(r.right)}}(t,window.devicePixelRatio<2||!P),u="bottom"===n?"top":"bottom",g="right"===r?"left":"right",h=U("transform"),f=void 0,p=void 0;if(p="bottom"===u?"HTML"===s.nodeName?-s.clientHeight+c.bottom:-A.height+c.bottom:c.top,f="right"===g?"HTML"===s.nodeName?-s.clientWidth+c.right:-A.width+c.right:c.left,a&&h)l[h]="translate3d("+f+"px, "+p+"px, 0)",l[u]=0,l[g]=0,l.willChange="transform";else{var I="bottom"===u?-1:1,C="right"===g?-1:1;l[u]=p*I,l[g]=f*C,l.willChange=u+", "+g}var m={"x-placement":t.placement};return t.attributes=b({},m,t.attributes),t.styles=b({},l,t.styles),t.arrowStyles=b({},t.offsets.arrow,t.arrowStyles),t},gpuAcceleration:!0,x:"bottom",y:"right"},applyStyle:{order:900,enabled:!0,fn:function(t){var e,n;return Y(t.instance.popper,t.styles),e=t.instance.popper,n=t.attributes,Object.keys(n).forEach(function(t){!1!==n[t]?e.setAttribute(t,n[t]):e.removeAttribute(t)}),t.arrowElement&&Object.keys(t.arrowStyles).length&&Y(t.arrowElement,t.arrowStyles),t},onLoad:function(t,e,n,r,i){var o=k(i,e,t,n.positionFixed),a=M(n.placement,o,e,t,n.modifiers.flip.boundariesElement,n.modifiers.flip.padding);return e.setAttribute("x-placement",a),Y(e,{position:n.positionFixed?"fixed":"absolute"}),n},gpuAcceleration:void 0}}},Z=function(){function t(e,n){var r=this,i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};E(this,t),this.scheduleUpdate=function(){return requestAnimationFrame(r.update)},this.update=a(this.update.bind(this)),this.options=b({},t.Defaults,i),this.state={isDestroyed:!1,isCreated:!1,scrollParents:[]},this.reference=e&&e.jquery?e[0]:e,this.popper=n&&n.jquery?n[0]:n,this.options.modifiers={},Object.keys(b({},t.Defaults.modifiers,i.modifiers)).forEach(function(e){r.options.modifiers[e]=b({},t.Defaults.modifiers[e]||{},i.modifiers?i.modifiers[e]:{})}),this.modifiers=Object.keys(this.options.modifiers).map(function(t){return b({name:t},r.options.modifiers[t])}).sort(function(t,e){return t.order-e.order}),this.modifiers.forEach(function(t){t.enabled&&s(t.onLoad)&&t.onLoad(r.reference,r.popper,r.options,t,r.state)}),this.update();var o=this.options.eventsEnabled;o&&this.enableEventListeners(),this.state.eventsEnabled=o}return v(t,[{key:"update",value:function(){return function(){if(!this.state.isDestroyed){var t={instance:this,styles:{},arrowStyles:{},attributes:{},flipped:!1,offsets:{}};t.offsets.reference=k(this.state,this.popper,this.reference,this.options.positionFixed),t.placement=M(this.options.placement,t.offsets.reference,this.popper,this.reference,this.options.modifiers.flip.boundariesElement,this.options.modifiers.flip.padding),t.originalPlacement=t.placement,t.positionFixed=this.options.positionFixed,t.offsets.popper=R(this.popper,t.offsets.reference,t.placement),t.offsets.popper.position=this.options.positionFixed?"fixed":"absolute",t=F(this.modifiers,t),this.state.isCreated?this.options.onUpdate(t):(this.state.isCreated=!0,this.options.onCreate(t))}}.call(this)}},{key:"destroy",value:function(){return function(){return this.state.isDestroyed=!0,T(this.modifiers,"applyStyle")&&(this.popper.removeAttribute("x-placement"),this.popper.style.position="",this.popper.style.top="",this.popper.style.left="",this.popper.style.right="",this.popper.style.bottom="",this.popper.style.willChange="",this.popper.style[U("transform")]=""),this.disableEventListeners(),this.options.removeOnDestroy&&this.popper.parentNode.removeChild(this.popper),this}.call(this)}},{key:"enableEventListeners",value:function(){return function(){this.state.eventsEnabled||(this.state=G(this.reference,this.options,this.state,this.scheduleUpdate))}.call(this)}},{key:"disableEventListeners",value:function(){return H.call(this)}}]),t}();Z.Utils=("undefined"!=typeof window?window:t).PopperUtils,Z.placements=z,Z.Defaults=X,e.a=Z}).call(this,n(37))},function(t,e,n){!function(t){"use strict";t.defineMode("javascript",function(e,n){var r,i,o=e.indentUnit,a=n.statementIndent,s=n.jsonld,A=n.json||s,l=n.typescript,c=n.wordCharacters||/[\w$\xa1-\uffff]/,u=function(){function t(t){return{type:t,style:"keyword"}}var e=t("keyword a"),n=t("keyword b"),r=t("keyword c"),i=t("keyword d"),o=t("operator"),a={type:"atom",style:"atom"};return{if:t("if"),while:e,with:e,else:n,do:n,try:n,finally:n,return:i,break:i,continue:i,new:t("new"),delete:r,void:r,throw:r,debugger:t("debugger"),var:t("var"),const:t("var"),let:t("var"),function:t("function"),catch:t("catch"),for:t("for"),switch:t("switch"),case:t("case"),default:t("default"),in:o,typeof:o,instanceof:o,true:a,false:a,null:a,undefined:a,NaN:a,Infinity:a,this:t("this"),class:t("class"),super:t("atom"),yield:r,export:t("export"),import:t("import"),extends:r,await:r}}(),g=/[+\-*&%=<>!?|~^@]/,h=/^@(context|id|value|language|type|container|list|set|reverse|index|base|vocab|graph)"/;function d(t,e,n){return r=t,i=n,e}function f(t,e){var n,r=t.next();if('"'==r||"'"==r)return e.tokenize=(n=r,function(t,e){var r,i=!1;if(s&&"@"==t.peek()&&t.match(h))return e.tokenize=f,d("jsonld-keyword","meta");for(;null!=(r=t.next())&&(r!=n||i);)i=!i&&"\\"==r;return i||(e.tokenize=f),d("string","string")}),e.tokenize(t,e);if("."==r&&t.match(/^\d+(?:[eE][+\-]?\d+)?/))return d("number","number");if("."==r&&t.match(".."))return d("spread","meta");if(/[\[\]{}\(\),;\:\.]/.test(r))return d(r);if("="==r&&t.eat(">"))return d("=>","operator");if("0"==r&&t.match(/^(?:x[\da-f]+|o[0-7]+|b[01]+)n?/i))return d("number","number");if(/\d/.test(r))return t.match(/^\d*(?:n|(?:\.\d*)?(?:[eE][+\-]?\d+)?)?/),d("number","number");if("/"==r)return t.eat("*")?(e.tokenize=p,p(t,e)):t.eat("/")?(t.skipToEnd(),d("comment","comment")):Jt(t,e,1)?(function(t){for(var e,n=!1,r=!1;null!=(e=t.next());){if(!n){if("/"==e&&!r)return;"["==e?r=!0:r&&"]"==e&&(r=!1)}n=!n&&"\\"==e}}(t),t.match(/^\b(([gimyus])(?![gimyus]*\2))+\b/),d("regexp","string-2")):(t.eat("="),d("operator","operator",t.current()));if("`"==r)return e.tokenize=I,I(t,e);if("#"==r)return t.skipToEnd(),d("error","error");if(g.test(r))return">"==r&&e.lexical&&">"==e.lexical.type||(t.eat("=")?"!"!=r&&"="!=r||t.eat("="):/[<>*+\-]/.test(r)&&(t.eat(r),">"==r&&t.eat(r))),d("operator","operator",t.current());if(c.test(r)){t.eatWhile(c);var i=t.current();if("."!=e.lastType){if(u.propertyIsEnumerable(i)){var o=u[i];return d(o.type,o.style,i)}if("async"==i&&t.match(/^(\s|\/\*.*?\*\/)*[\[\(\w]/,!1))return d("async","keyword",i)}return d("variable","variable",i)}}function p(t,e){for(var n,r=!1;n=t.next();){if("/"==n&&r){e.tokenize=f;break}r="*"==n}return d("comment","comment")}function I(t,e){for(var n,r=!1;null!=(n=t.next());){if(!r&&("`"==n||"$"==n&&t.eat("{"))){e.tokenize=f;break}r=!r&&"\\"==n}return d("quasi","string-2",t.current())}var C="([{}])";function m(t,e){e.fatArrowAt&&(e.fatArrowAt=null);var n=t.string.indexOf("=>",t.start);if(!(n<0)){if(l){var r=/:\s*(?:\w+(?:<[^>]*>|\[\])?|\{[^}]*\})\s*$/.exec(t.string.slice(t.start,n));r&&(n=r.index)}for(var i=0,o=!1,a=n-1;a>=0;--a){var s=t.string.charAt(a),A=C.indexOf(s);if(A>=0&&A<3){if(!i){++a;break}if(0==--i){"("==s&&(o=!0);break}}else if(A>=3&&A<6)++i;else if(c.test(s))o=!0;else{if(/["'\/]/.test(s))return;if(o&&!i){++a;break}}}o&&!i&&(e.fatArrowAt=a)}}var y={atom:!0,number:!0,variable:!0,string:!0,regexp:!0,this:!0,"jsonld-keyword":!0};function E(t,e,n,r,i,o){this.indented=t,this.column=e,this.type=n,this.prev=i,this.info=o,null!=r&&(this.align=r)}function v(t,e){for(var n=t.localVars;n;n=n.next)if(n.name==e)return!0;for(var r=t.context;r;r=r.prev)for(var n=r.vars;n;n=n.next)if(n.name==e)return!0}var B={state:null,column:null,marked:null,cc:null};function b(){for(var t=arguments.length-1;t>=0;t--)B.cc.push(arguments[t])}function Q(){return b.apply(null,arguments),!0}function w(t,e){for(var n=e;n;n=n.next)if(n.name==t)return!0;return!1}function x(t){var e=B.state;if(B.marked="def",e.context)if("var"==e.lexical.info&&e.context&&e.context.block){var r=function t(e,n){if(n){if(n.block){var r=t(e,n.prev);return r?r==n.prev?n:new D(r,n.vars,!0):null}return w(e,n.vars)?n:new D(n.prev,new M(e,n.vars),!1)}return null}(t,e.context);if(null!=r)return void(e.context=r)}else if(!w(t,e.localVars))return void(e.localVars=new M(t,e.localVars));n.globalVars&&!w(t,e.globalVars)&&(e.globalVars=new M(t,e.globalVars))}function _(t){return"public"==t||"private"==t||"protected"==t||"abstract"==t||"readonly"==t}function D(t,e,n){this.prev=t,this.vars=e,this.block=n}function M(t,e){this.name=t,this.next=e}var k=new M("this",new M("arguments",null));function S(){B.state.context=new D(B.state.context,B.state.localVars,!1),B.state.localVars=k}function L(){B.state.context=new D(B.state.context,B.state.localVars,!0),B.state.localVars=null}function R(){B.state.localVars=B.state.context.vars,B.state.context=B.state.context.prev}function N(t,e){var n=function(){var n=B.state,r=n.indented;if("stat"==n.lexical.type)r=n.lexical.indented;else for(var i=n.lexical;i&&")"==i.type&&i.align;i=i.prev)r=i.indented;n.lexical=new E(r,B.stream.column(),t,null,n.lexical,e)};return n.lex=!0,n}function F(){var t=B.state;t.lexical.prev&&(")"==t.lexical.type&&(t.indented=t.lexical.indented),t.lexical=t.lexical.prev)}function T(t){return function e(n){return n==t?Q():";"==t||"}"==n||")"==n||"]"==n?b():Q(e)}}function U(t,e){return"var"==t?Q(N("vardef",e),Ct,T(";"),F):"keyword a"==t?Q(N("form"),j,U,F):"keyword b"==t?Q(N("form"),U,F):"keyword d"==t?B.stream.match(/^\s*$/,!1)?Q():Q(N("stat"),P,T(";"),F):"debugger"==t?Q(T(";")):"{"==t?Q(N("}"),L,at,F,R):";"==t?Q():"if"==t?("else"==B.state.lexical.info&&B.state.cc[B.state.cc.length-1]==F&&B.state.cc.pop()(),Q(N("form"),j,U,F,bt)):"function"==t?Q(_t):"for"==t?Q(N("form"),Qt,U,F):"class"==t||l&&"interface"==e?(B.marked="keyword",Q(N("form","class"==t?t:e),Lt,F)):"variable"==t?l&&"declare"==e?(B.marked="keyword",Q(U)):l&&("module"==e||"enum"==e||"type"==e)&&B.stream.match(/^\s*\w/,!1)?(B.marked="keyword","enum"==e?Q(Kt):"type"==e?Q(Mt,T("operator"),ct,T(";")):Q(N("form"),mt,T("{"),N("}"),at,F,F)):l&&"namespace"==e?(B.marked="keyword",Q(N("form"),G,U,F)):l&&"abstract"==e?(B.marked="keyword",Q(U)):Q(N("stat"),$):"switch"==t?Q(N("form"),j,T("{"),N("}","switch"),L,at,F,F,R):"case"==t?Q(G,T(":")):"default"==t?Q(T(":")):"catch"==t?Q(N("form"),S,O,U,F,R):"export"==t?Q(N("stat"),Tt,F):"import"==t?Q(N("stat"),Ot,F):"async"==t?Q(U):"@"==e?Q(G,U):b(N("stat"),G,T(";"),F)}function O(t){if("("==t)return Q(kt,T(")"))}function G(t,e){return Y(t,e,!1)}function H(t,e){return Y(t,e,!0)}function j(t){return"("!=t?b():Q(N(")"),G,T(")"),F)}function Y(t,e,n){if(B.state.fatArrowAt==B.stream.start){var r=n?W:V;if("("==t)return Q(S,N(")"),it(kt,")"),F,T("=>"),r,R);if("variable"==t)return b(S,mt,T("=>"),r,R)}var i=n?z:K;return y.hasOwnProperty(t)?Q(i):"function"==t?Q(_t,i):"class"==t||l&&"interface"==e?(B.marked="keyword",Q(N("form"),St,F)):"keyword c"==t||"async"==t?Q(n?H:G):"("==t?Q(N(")"),P,T(")"),F,i):"operator"==t||"spread"==t?Q(n?H:G):"["==t?Q(N("]"),Pt,F,i):"{"==t?ot(et,"}",null,i):"quasi"==t?b(J,i):"new"==t?Q(function(t){return function(e){return"."==e?Q(t?Z:X):"variable"==e&&l?Q(ft,t?z:K):b(t?H:G)}}(n)):"import"==t?Q(G):Q()}function P(t){return t.match(/[;\}\)\],]/)?b():b(G)}function K(t,e){return","==t?Q(G):z(t,e,!1)}function z(t,e,n){var r=0==n?K:z,i=0==n?G:H;return"=>"==t?Q(S,n?W:V,R):"operator"==t?/\+\+|--/.test(e)||l&&"!"==e?Q(r):l&&"<"==e&&B.stream.match(/^([^>]|<.*?>)*>\s*\(/,!1)?Q(N(">"),it(ct,">"),F,r):"?"==e?Q(G,T(":"),i):Q(i):"quasi"==t?b(J,r):";"!=t?"("==t?ot(H,")","call",r):"."==t?Q(tt,r):"["==t?Q(N("]"),P,T("]"),F,r):l&&"as"==e?(B.marked="keyword",Q(ct,r)):"regexp"==t?(B.state.lastType=B.marked="operator",B.stream.backUp(B.stream.pos-B.stream.start-1),Q(i)):void 0:void 0}function J(t,e){return"quasi"!=t?b():"${"!=e.slice(e.length-2)?Q(J):Q(G,q)}function q(t){if("}"==t)return B.marked="string-2",B.state.tokenize=I,Q(J)}function V(t){return m(B.stream,B.state),b("{"==t?U:G)}function W(t){return m(B.stream,B.state),b("{"==t?U:H)}function X(t,e){if("target"==e)return B.marked="keyword",Q(K)}function Z(t,e){if("target"==e)return B.marked="keyword",Q(z)}function $(t){return":"==t?Q(F,U):b(K,T(";"),F)}function tt(t){if("variable"==t)return B.marked="property",Q()}function et(t,e){return"async"==t?(B.marked="property",Q(et)):"variable"==t||"keyword"==B.style?(B.marked="property","get"==e||"set"==e?Q(nt):(l&&B.state.fatArrowAt==B.stream.start&&(n=B.stream.match(/^\s*:\s*/,!1))&&(B.state.fatArrowAt=B.stream.pos+n[0].length),Q(rt))):"number"==t||"string"==t?(B.marked=s?"property":B.style+" property",Q(rt)):"jsonld-keyword"==t?Q(rt):l&&_(e)?(B.marked="keyword",Q(et)):"["==t?Q(G,st,T("]"),rt):"spread"==t?Q(H,rt):"*"==e?(B.marked="keyword",Q(et)):":"==t?b(rt):void 0;var n}function nt(t){return"variable"!=t?b(rt):(B.marked="property",Q(_t))}function rt(t){return":"==t?Q(H):"("==t?b(_t):void 0}function it(t,e,n){function r(i,o){if(n?n.indexOf(i)>-1:","==i){var a=B.state.lexical;return"call"==a.info&&(a.pos=(a.pos||0)+1),Q(function(n,r){return n==e||r==e?b():b(t)},r)}return i==e||o==e?Q():n&&n.indexOf(";")>-1?b(t):Q(T(e))}return function(n,i){return n==e||i==e?Q():b(t,r)}}function ot(t,e,n){for(var r=3;r<arguments.length;r++)B.cc.push(arguments[r]);return Q(N(e,n),it(t,e),F)}function at(t){return"}"==t?Q():b(U,at)}function st(t,e){if(l){if(":"==t||"in"==e)return Q(ct);if("?"==e)return Q(st)}}function At(t){if(l&&":"==t)return B.stream.match(/^\s*\w+\s+is\b/,!1)?Q(G,lt,ct):Q(ct)}function lt(t,e){if("is"==e)return B.marked="keyword",Q()}function ct(t,e){return"keyof"==e||"typeof"==e||"infer"==e?(B.marked="keyword",Q("typeof"==e?H:ct)):"variable"==t||"void"==e?(B.marked="type",Q(dt)):"|"==e||"&"==e?Q(ct):"string"==t||"number"==t||"atom"==t?Q(dt):"["==t?Q(N("]"),it(ct,"]",","),F,dt):"{"==t?Q(N("}"),it(gt,"}",",;"),F,dt):"("==t?Q(it(ht,")"),ut,dt):"<"==t?Q(it(ct,">"),ct):void 0}function ut(t){if("=>"==t)return Q(ct)}function gt(t,e){return"variable"==t||"keyword"==B.style?(B.marked="property",Q(gt)):"?"==e||"number"==t||"string"==t?Q(gt):":"==t?Q(ct):"["==t?Q(T("variable"),st,T("]"),gt):"("==t?b(Dt,gt):void 0}function ht(t,e){return"variable"==t&&B.stream.match(/^\s*[?:]/,!1)||"?"==e?Q(ht):":"==t?Q(ct):"spread"==t?Q(ht):b(ct)}function dt(t,e){return"<"==e?Q(N(">"),it(ct,">"),F,dt):"|"==e||"."==t||"&"==e?Q(ct):"["==t?Q(ct,T("]"),dt):"extends"==e||"implements"==e?(B.marked="keyword",Q(ct)):"?"==e?Q(ct,T(":"),ct):void 0}function ft(t,e){if("<"==e)return Q(N(">"),it(ct,">"),F,dt)}function pt(){return b(ct,It)}function It(t,e){if("="==e)return Q(ct)}function Ct(t,e){return"enum"==e?(B.marked="keyword",Q(Kt)):b(mt,st,vt,Bt)}function mt(t,e){return l&&_(e)?(B.marked="keyword",Q(mt)):"variable"==t?(x(e),Q()):"spread"==t?Q(mt):"["==t?ot(Et,"]"):"{"==t?ot(yt,"}"):void 0}function yt(t,e){return"variable"!=t||B.stream.match(/^\s*:/,!1)?("variable"==t&&(B.marked="property"),"spread"==t?Q(mt):"}"==t?b():"["==t?Q(G,T("]"),T(":"),yt):Q(T(":"),mt,vt)):(x(e),Q(vt))}function Et(){return b(mt,vt)}function vt(t,e){if("="==e)return Q(H)}function Bt(t){if(","==t)return Q(Ct)}function bt(t,e){if("keyword b"==t&&"else"==e)return Q(N("form","else"),U,F)}function Qt(t,e){return"await"==e?Q(Qt):"("==t?Q(N(")"),wt,F):void 0}function wt(t){return"var"==t?Q(Ct,xt):"variable"==t?Q(xt):b(xt)}function xt(t,e){return")"==t?Q():";"==t?Q(xt):"in"==e||"of"==e?(B.marked="keyword",Q(G,xt)):b(G,xt)}function _t(t,e){return"*"==e?(B.marked="keyword",Q(_t)):"variable"==t?(x(e),Q(_t)):"("==t?Q(S,N(")"),it(kt,")"),F,At,U,R):l&&"<"==e?Q(N(">"),it(pt,">"),F,_t):void 0}function Dt(t,e){return"*"==e?(B.marked="keyword",Q(Dt)):"variable"==t?(x(e),Q(Dt)):"("==t?Q(S,N(")"),it(kt,")"),F,At,R):l&&"<"==e?Q(N(">"),it(pt,">"),F,Dt):void 0}function Mt(t,e){return"keyword"==t||"variable"==t?(B.marked="type",Q(Mt)):"<"==e?Q(N(">"),it(pt,">"),F):void 0}function kt(t,e){return"@"==e&&Q(G,kt),"spread"==t?Q(kt):l&&_(e)?(B.marked="keyword",Q(kt)):l&&"this"==t?Q(st,vt):b(mt,st,vt)}function St(t,e){return"variable"==t?Lt(t,e):Rt(t,e)}function Lt(t,e){if("variable"==t)return x(e),Q(Rt)}function Rt(t,e){return"<"==e?Q(N(">"),it(pt,">"),F,Rt):"extends"==e||"implements"==e||l&&","==t?("implements"==e&&(B.marked="keyword"),Q(l?ct:G,Rt)):"{"==t?Q(N("}"),Nt,F):void 0}function Nt(t,e){return"async"==t||"variable"==t&&("static"==e||"get"==e||"set"==e||l&&_(e))&&B.stream.match(/^\s+[\w$\xa1-\uffff]/,!1)?(B.marked="keyword",Q(Nt)):"variable"==t||"keyword"==B.style?(B.marked="property",Q(l?Ft:_t,Nt)):"number"==t||"string"==t?Q(l?Ft:_t,Nt):"["==t?Q(G,st,T("]"),l?Ft:_t,Nt):"*"==e?(B.marked="keyword",Q(Nt)):l&&"("==t?b(Dt,Nt):";"==t||","==t?Q(Nt):"}"==t?Q():"@"==e?Q(G,Nt):void 0}function Ft(t,e){if("?"==e)return Q(Ft);if(":"==t)return Q(ct,vt);if("="==e)return Q(H);var n=B.state.lexical.prev,r=n&&"interface"==n.info;return b(r?Dt:_t)}function Tt(t,e){return"*"==e?(B.marked="keyword",Q(Yt,T(";"))):"default"==e?(B.marked="keyword",Q(G,T(";"))):"{"==t?Q(it(Ut,"}"),Yt,T(";")):b(U)}function Ut(t,e){return"as"==e?(B.marked="keyword",Q(T("variable"))):"variable"==t?b(H,Ut):void 0}function Ot(t){return"string"==t?Q():"("==t?b(G):b(Gt,Ht,Yt)}function Gt(t,e){return"{"==t?ot(Gt,"}"):("variable"==t&&x(e),"*"==e&&(B.marked="keyword"),Q(jt))}function Ht(t){if(","==t)return Q(Gt,Ht)}function jt(t,e){if("as"==e)return B.marked="keyword",Q(Gt)}function Yt(t,e){if("from"==e)return B.marked="keyword",Q(G)}function Pt(t){return"]"==t?Q():b(it(H,"]"))}function Kt(){return b(N("form"),mt,T("{"),N("}"),it(zt,"}"),F,F)}function zt(){return b(mt,vt)}function Jt(t,e,n){return e.tokenize==f&&/^(?:operator|sof|keyword [bcd]|case|new|export|default|spread|[\[{}\(,;:]|=>)$/.test(e.lastType)||"quasi"==e.lastType&&/\{\s*$/.test(t.string.slice(0,t.pos-(n||0)))}return R.lex=!0,F.lex=!0,{startState:function(t){var e={tokenize:f,lastType:"sof",cc:[],lexical:new E((t||0)-o,0,"block",!1),localVars:n.localVars,context:n.localVars&&new D(null,null,!1),indented:t||0};return n.globalVars&&"object"==typeof n.globalVars&&(e.globalVars=n.globalVars),e},token:function(t,e){if(t.sol()&&(e.lexical.hasOwnProperty("align")||(e.lexical.align=!1),e.indented=t.indentation(),m(t,e)),e.tokenize!=p&&t.eatSpace())return null;var n=e.tokenize(t,e);return"comment"==r?n:(e.lastType="operator"!=r||"++"!=i&&"--"!=i?r:"incdec",function(t,e,n,r,i){var o=t.cc;for(B.state=t,B.stream=i,B.marked=null,B.cc=o,B.style=e,t.lexical.hasOwnProperty("align")||(t.lexical.align=!0);;){var a=o.length?o.pop():A?G:U;if(a(n,r)){for(;o.length&&o[o.length-1].lex;)o.pop()();return B.marked?B.marked:"variable"==n&&v(t,r)?"variable-2":e}}}(e,n,r,i,t))},indent:function(e,r){if(e.tokenize==p)return t.Pass;if(e.tokenize!=f)return 0;var i,s=r&&r.charAt(0),A=e.lexical;if(!/^\s*else\b/.test(r))for(var l=e.cc.length-1;l>=0;--l){var c=e.cc[l];if(c==F)A=A.prev;else if(c!=bt)break}for(;("stat"==A.type||"form"==A.type)&&("}"==s||(i=e.cc[e.cc.length-1])&&(i==K||i==z)&&!/^[,\.=+\-*:?[\(]/.test(r));)A=A.prev;a&&")"==A.type&&"stat"==A.prev.type&&(A=A.prev);var u=A.type,h=s==u;return"vardef"==u?A.indented+("operator"==e.lastType||","==e.lastType?A.info.length+1:0):"form"==u&&"{"==s?A.indented:"form"==u?A.indented+o:"stat"==u?A.indented+(function(t,e){return"operator"==t.lastType||","==t.lastType||g.test(e.charAt(0))||/[,.]/.test(e.charAt(0))}(e,r)?a||o:0):"switch"!=A.info||h||0==n.doubleIndentSwitch?A.align?A.column+(h?0:1):A.indented+(h?0:o):A.indented+(/^(?:case|default)\b/.test(r)?o:2*o)},electricInput:/^\s*(?:case .*?:|default:|\{|\})$/,blockCommentStart:A?null:"/*",blockCommentEnd:A?null:"*/",blockCommentContinue:A?null:" * ",lineComment:A?null:"//",fold:"brace",closeBrackets:"()[]{}''\"\"``",helperType:A?"json":"javascript",jsonldMode:s,jsonMode:A,expressionAllowed:Jt,skipExpression:function(t){var e=t.cc[t.cc.length-1];e!=G&&e!=H||t.cc.pop()}}}),t.registerHelper("wordChars","javascript",/[\w$]/),t.defineMIME("text/javascript","javascript"),t.defineMIME("text/ecmascript","javascript"),t.defineMIME("application/javascript","javascript"),t.defineMIME("application/x-javascript","javascript"),t.defineMIME("application/ecmascript","javascript"),t.defineMIME("application/json",{name:"javascript",json:!0}),t.defineMIME("application/x-json",{name:"javascript",json:!0}),t.defineMIME("application/ld+json",{name:"javascript",jsonld:!0}),t.defineMIME("text/typescript",{name:"javascript",typescript:!0}),t.defineMIME("application/typescript",{name:"javascript",typescript:!0})}(n(15))},function(t,e,n){!function(t){"use strict";t.multiplexingMode=function(e){var n=Array.prototype.slice.call(arguments,1);function r(t,e,n,r){if("string"==typeof e){var i=t.indexOf(e,n);return r&&i>-1?i+e.length:i}var o=e.exec(n?t.slice(n):t);return o?o.index+n+(r?o[0].length:0):-1}return{startState:function(){return{outer:t.startState(e),innerActive:null,inner:null}},copyState:function(n){return{outer:t.copyState(e,n.outer),innerActive:n.innerActive,inner:n.innerActive&&t.copyState(n.innerActive.mode,n.inner)}},token:function(i,o){if(o.innerActive){var a=o.innerActive,s=i.string;if(!a.close&&i.sol())return o.innerActive=o.inner=null,this.token(i,o);var A=a.close?r(s,a.close,i.pos,a.parseDelimiters):-1;if(A==i.pos&&!a.parseDelimiters)return i.match(a.close),o.innerActive=o.inner=null,a.delimStyle&&a.delimStyle+" "+a.delimStyle+"-close";A>-1&&(i.string=s.slice(0,A));var l=a.mode.token(i,o.inner);return A>-1&&(i.string=s),A==i.pos&&a.parseDelimiters&&(o.innerActive=o.inner=null),a.innerStyle&&(l=l?l+" "+a.innerStyle:a.innerStyle),l}for(var c=1/0,s=i.string,u=0;u<n.length;++u){var g=n[u],A=r(s,g.open,i.pos);if(A==i.pos){g.parseDelimiters||i.match(g.open),o.innerActive=g;var h=0;if(e.indent){var d=e.indent(o.outer,"","");d!==t.Pass&&(h=d)}return o.inner=t.startState(g.mode,h),g.delimStyle&&g.delimStyle+" "+g.delimStyle+"-open"}-1!=A&&A<c&&(c=A)}c!=1/0&&(i.string=s.slice(0,c));var f=e.token(i,o.outer);return c!=1/0&&(i.string=s),f},indent:function(n,r,i){var o=n.innerActive?n.innerActive.mode:e;return o.indent?o.indent(n.innerActive?n.inner:n.outer,r,i):t.Pass},blankLine:function(r){var i=r.innerActive?r.innerActive.mode:e;if(i.blankLine&&i.blankLine(r.innerActive?r.inner:r.outer),r.innerActive)"\n"===r.innerActive.close&&(r.innerActive=r.inner=null);else for(var o=0;o<n.length;++o){var a=n[o];"\n"===a.open&&(r.innerActive=a,r.inner=t.startState(a.mode,i.indent?i.indent(r.outer,"",""):0))}},electricChars:e.electricChars,innerMode:function(t){return t.inner?{state:t.inner,mode:t.innerActive.mode}:{state:t.outer,mode:e}}}}}(n(15))},function(t,e,n){!function(t){"use strict";t.defineMode("markdown",function(e,n){var r=t.getMode(e,"text/html"),i="null"==r.name;void 0===n.highlightFormatting&&(n.highlightFormatting=!1),void 0===n.maxBlockquoteDepth&&(n.maxBlockquoteDepth=0),void 0===n.taskLists&&(n.taskLists=!1),void 0===n.strikethrough&&(n.strikethrough=!1),void 0===n.emoji&&(n.emoji=!1),void 0===n.fencedCodeBlockHighlighting&&(n.fencedCodeBlockHighlighting=!0),void 0===n.xml&&(n.xml=!0),void 0===n.tokenTypeOverrides&&(n.tokenTypeOverrides={});var o={header:"header",code:"comment",quote:"quote",list1:"variable-2",list2:"variable-3",list3:"keyword",hr:"hr",image:"image",imageAltText:"image-alt-text",imageMarker:"image-marker",formatting:"formatting",linkInline:"link",linkEmail:"link",linkText:"link",linkHref:"string",em:"em",strong:"strong",strikethrough:"strikethrough",emoji:"builtin"};for(var a in o)o.hasOwnProperty(a)&&n.tokenTypeOverrides[a]&&(o[a]=n.tokenTypeOverrides[a]);var s=/^([*\-_])(?:\s*\1){2,}\s*$/,A=/^(?:[*\-+]|^[0-9]+([.)]))\s+/,l=/^\[(x| )\](?=\s)/i,c=n.allowAtxHeaderWithoutSpace?/^(#+)/:/^(#+)(?: |$)/,u=/^ *(?:\={1,}|-{1,})\s*$/,g=/^[^#!\[\]*_\\<>` "'(~:]+/,h=/^(~~~+|```+)[ \t]*([\w+#-]*)[^\n`]*$/,d=/^\s*\[[^\]]+?\]:.*$/,f=/[!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~\xA1\xA7\xAB\xB6\xB7\xBB\xBF\u037E\u0387\u055A-\u055F\u0589\u058A\u05BE\u05C0\u05C3\u05C6\u05F3\u05F4\u0609\u060A\u060C\u060D\u061B\u061E\u061F\u066A-\u066D\u06D4\u0700-\u070D\u07F7-\u07F9\u0830-\u083E\u085E\u0964\u0965\u0970\u0AF0\u0DF4\u0E4F\u0E5A\u0E5B\u0F04-\u0F12\u0F14\u0F3A-\u0F3D\u0F85\u0FD0-\u0FD4\u0FD9\u0FDA\u104A-\u104F\u10FB\u1360-\u1368\u1400\u166D\u166E\u169B\u169C\u16EB-\u16ED\u1735\u1736\u17D4-\u17D6\u17D8-\u17DA\u1800-\u180A\u1944\u1945\u1A1E\u1A1F\u1AA0-\u1AA6\u1AA8-\u1AAD\u1B5A-\u1B60\u1BFC-\u1BFF\u1C3B-\u1C3F\u1C7E\u1C7F\u1CC0-\u1CC7\u1CD3\u2010-\u2027\u2030-\u2043\u2045-\u2051\u2053-\u205E\u207D\u207E\u208D\u208E\u2308-\u230B\u2329\u232A\u2768-\u2775\u27C5\u27C6\u27E6-\u27EF\u2983-\u2998\u29D8-\u29DB\u29FC\u29FD\u2CF9-\u2CFC\u2CFE\u2CFF\u2D70\u2E00-\u2E2E\u2E30-\u2E42\u3001-\u3003\u3008-\u3011\u3014-\u301F\u3030\u303D\u30A0\u30FB\uA4FE\uA4FF\uA60D-\uA60F\uA673\uA67E\uA6F2-\uA6F7\uA874-\uA877\uA8CE\uA8CF\uA8F8-\uA8FA\uA8FC\uA92E\uA92F\uA95F\uA9C1-\uA9CD\uA9DE\uA9DF\uAA5C-\uAA5F\uAADE\uAADF\uAAF0\uAAF1\uABEB\uFD3E\uFD3F\uFE10-\uFE19\uFE30-\uFE52\uFE54-\uFE61\uFE63\uFE68\uFE6A\uFE6B\uFF01-\uFF03\uFF05-\uFF0A\uFF0C-\uFF0F\uFF1A\uFF1B\uFF1F\uFF20\uFF3B-\uFF3D\uFF3F\uFF5B\uFF5D\uFF5F-\uFF65]|\uD800[\uDD00-\uDD02\uDF9F\uDFD0]|\uD801\uDD6F|\uD802[\uDC57\uDD1F\uDD3F\uDE50-\uDE58\uDE7F\uDEF0-\uDEF6\uDF39-\uDF3F\uDF99-\uDF9C]|\uD804[\uDC47-\uDC4D\uDCBB\uDCBC\uDCBE-\uDCC1\uDD40-\uDD43\uDD74\uDD75\uDDC5-\uDDC9\uDDCD\uDDDB\uDDDD-\uDDDF\uDE38-\uDE3D\uDEA9]|\uD805[\uDCC6\uDDC1-\uDDD7\uDE41-\uDE43\uDF3C-\uDF3E]|\uD809[\uDC70-\uDC74]|\uD81A[\uDE6E\uDE6F\uDEF5\uDF37-\uDF3B\uDF44]|\uD82F\uDC9F|\uD836[\uDE87-\uDE8B]/;function p(t,e,n){return e.f=e.inline=n,n(t,e)}function I(t,e,n){return e.f=e.block=n,n(t,e)}function C(e){if(e.linkTitle=!1,e.linkHref=!1,e.linkText=!1,e.em=!1,e.strong=!1,e.strikethrough=!1,e.quote=0,e.indentedCode=!1,e.f==y){var n=i;if(!n){var o=t.innerMode(r,e.htmlState);n="xml"==o.mode.name&&null===o.state.tagStart&&!o.state.context&&o.state.tokenize.isInText}n&&(e.f=b,e.block=m,e.htmlState=null)}return e.trailingSpace=0,e.trailingSpaceNewLine=!1,e.prevLine=e.thisLine,e.thisLine={stream:null},null}function m(r,i){var a,g=r.column()===i.indentation,f=!(a=i.prevLine.stream)||!/\S/.test(a.string),I=i.indentedCode,C=i.prevLine.hr,m=!1!==i.list,y=(i.listStack[i.listStack.length-1]||0)+3;i.indentedCode=!1;var B=i.indentation;if(null===i.indentationDiff&&(i.indentationDiff=i.indentation,m)){for(i.em=!1,i.strong=!1,i.code=!1,i.strikethrough=!1,i.list=null;B<i.listStack[i.listStack.length-1];)i.listStack.pop(),i.listStack.length?i.indentation=i.listStack[i.listStack.length-1]:i.list=!1;!1!==i.list&&(i.indentationDiff=B-i.listStack[i.listStack.length-1])}var b=!(f||C||i.prevLine.header||m&&I||i.prevLine.fencedCodeEnd),Q=(!1===i.list||C||f)&&i.indentation<=y&&r.match(s),w=null;if(i.indentationDiff>=4&&(I||i.prevLine.fencedCodeEnd||i.prevLine.header||f))return r.skipToEnd(),i.indentedCode=!0,o.code;if(r.eatSpace())return null;if(g&&i.indentation<=y&&(w=r.match(c))&&w[1].length<=6)return i.quote=0,i.header=w[1].length,i.thisLine.header=!0,n.highlightFormatting&&(i.formatting="header"),i.f=i.inline,v(i);if(i.indentation<=y&&r.eat(">"))return i.quote=g?1:i.quote+1,n.highlightFormatting&&(i.formatting="quote"),r.eatSpace(),v(i);if(!Q&&!i.setext&&g&&i.indentation<=y&&(w=r.match(A))){var x=w[1]?"ol":"ul";return i.indentation=B+r.current().length,i.list=!0,i.quote=0,i.listStack.push(i.indentation),n.taskLists&&r.match(l,!1)&&(i.taskList=!0),i.f=i.inline,n.highlightFormatting&&(i.formatting=["list","list-"+x]),v(i)}return g&&i.indentation<=y&&(w=r.match(h,!0))?(i.quote=0,i.fencedEndRE=new RegExp(w[1]+"+ *$"),i.localMode=n.fencedCodeBlockHighlighting&&function(n){if(t.findModeByName){var r=t.findModeByName(n);r&&(n=r.mime||r.mimes[0])}var i=t.getMode(e,n);return"null"==i.name?null:i}(w[2]),i.localMode&&(i.localState=t.startState(i.localMode)),i.f=i.block=E,n.highlightFormatting&&(i.formatting="code-block"),i.code=-1,v(i)):i.setext||!(b&&m||i.quote||!1!==i.list||i.code||Q||d.test(r.string))&&(w=r.lookAhead(1))&&(w=w.match(u))?(i.setext?(i.header=i.setext,i.setext=0,r.skipToEnd(),n.highlightFormatting&&(i.formatting="header")):(i.header="="==w[0].charAt(0)?1:2,i.setext=i.header),i.thisLine.header=!0,i.f=i.inline,v(i)):Q?(r.skipToEnd(),i.hr=!0,i.thisLine.hr=!0,o.hr):"["===r.peek()?p(r,i,_):p(r,i,i.inline)}function y(e,n){var o=r.token(e,n.htmlState);if(!i){var a=t.innerMode(r,n.htmlState);("xml"==a.mode.name&&null===a.state.tagStart&&!a.state.context&&a.state.tokenize.isInText||n.md_inside&&e.current().indexOf(">")>-1)&&(n.f=b,n.block=m,n.htmlState=null)}return o}function E(t,e){var r,i=e.listStack[e.listStack.length-1]||0,a=e.indentation<i,s=i+3;return e.fencedEndRE&&e.indentation<=s&&(a||t.match(e.fencedEndRE))?(n.highlightFormatting&&(e.formatting="code-block"),a||(r=v(e)),e.localMode=e.localState=null,e.block=m,e.f=b,e.fencedEndRE=null,e.code=0,e.thisLine.fencedCodeEnd=!0,a?I(t,e,e.block):r):e.localMode?e.localMode.token(t,e.localState):(t.skipToEnd(),o.code)}function v(t){var e=[];if(t.formatting){e.push(o.formatting),"string"==typeof t.formatting&&(t.formatting=[t.formatting]);for(var r=0;r<t.formatting.length;r++)e.push(o.formatting+"-"+t.formatting[r]),"header"===t.formatting[r]&&e.push(o.formatting+"-"+t.formatting[r]+"-"+t.header),"quote"===t.formatting[r]&&(!n.maxBlockquoteDepth||n.maxBlockquoteDepth>=t.quote?e.push(o.formatting+"-"+t.formatting[r]+"-"+t.quote):e.push("error"))}if(t.taskOpen)return e.push("meta"),e.length?e.join(" "):null;if(t.taskClosed)return e.push("property"),e.length?e.join(" "):null;if(t.linkHref?e.push(o.linkHref,"url"):(t.strong&&e.push(o.strong),t.em&&e.push(o.em),t.strikethrough&&e.push(o.strikethrough),t.emoji&&e.push(o.emoji),t.linkText&&e.push(o.linkText),t.code&&e.push(o.code),t.image&&e.push(o.image),t.imageAltText&&e.push(o.imageAltText,"link"),t.imageMarker&&e.push(o.imageMarker)),t.header&&e.push(o.header,o.header+"-"+t.header),t.quote&&(e.push(o.quote),!n.maxBlockquoteDepth||n.maxBlockquoteDepth>=t.quote?e.push(o.quote+"-"+t.quote):e.push(o.quote+"-"+n.maxBlockquoteDepth)),!1!==t.list){var i=(t.listStack.length-1)%3;i?1===i?e.push(o.list2):e.push(o.list3):e.push(o.list1)}return t.trailingSpaceNewLine?e.push("trailing-space-new-line"):t.trailingSpace&&e.push("trailing-space-"+(t.trailingSpace%2?"a":"b")),e.length?e.join(" "):null}function B(t,e){if(t.match(g,!0))return v(e)}function b(e,i){var a=i.text(e,i);if(void 0!==a)return a;if(i.list)return i.list=null,v(i);if(i.taskList){var s=" "===e.match(l,!0)[1];return s?i.taskOpen=!0:i.taskClosed=!0,n.highlightFormatting&&(i.formatting="task"),i.taskList=!1,v(i)}if(i.taskOpen=!1,i.taskClosed=!1,i.header&&e.match(/^#+$/,!0))return n.highlightFormatting&&(i.formatting="header"),v(i);var A=e.next();if(i.linkTitle){i.linkTitle=!1;var c=A;"("===A&&(c=")");var u="^\\s*(?:[^"+(c=(c+"").replace(/([.?*+^\[\]\\(){}|-])/g,"\\$1"))+"\\\\]+|\\\\\\\\|\\\\.)"+c;if(e.match(new RegExp(u),!0))return o.linkHref}if("`"===A){var g=i.formatting;n.highlightFormatting&&(i.formatting="code"),e.eatWhile("`");var h=e.current().length;if(0!=i.code||i.quote&&1!=h){if(h==i.code){var d=v(i);return i.code=0,d}return i.formatting=g,v(i)}return i.code=h,v(i)}if(i.code)return v(i);if("\\"===A&&(e.next(),n.highlightFormatting)){var p=v(i),C=o.formatting+"-escape";return p?p+" "+C:C}if("!"===A&&e.match(/\[[^\]]*\] ?(?:\(|\[)/,!1))return i.imageMarker=!0,i.image=!0,n.highlightFormatting&&(i.formatting="image"),v(i);if("["===A&&i.imageMarker&&e.match(/[^\]]*\](\(.*?\)| ?\[.*?\])/,!1))return i.imageMarker=!1,i.imageAltText=!0,n.highlightFormatting&&(i.formatting="image"),v(i);if("]"===A&&i.imageAltText){n.highlightFormatting&&(i.formatting="image");var p=v(i);return i.imageAltText=!1,i.image=!1,i.inline=i.f=w,p}if("["===A&&!i.image)return i.linkText&&e.match(/^.*?\]/)?v(i):(i.linkText=!0,n.highlightFormatting&&(i.formatting="link"),v(i));if("]"===A&&i.linkText){n.highlightFormatting&&(i.formatting="link");var p=v(i);return i.linkText=!1,i.inline=i.f=e.match(/\(.*?\)| ?\[.*?\]/,!1)?w:b,p}if("<"===A&&e.match(/^(https?|ftps?):\/\/(?:[^\\>]|\\.)+>/,!1)){i.f=i.inline=Q,n.highlightFormatting&&(i.formatting="link");var p=v(i);return p?p+=" ":p="",p+o.linkInline}if("<"===A&&e.match(/^[^> \\]+@(?:[^\\>]|\\.)+>/,!1)){i.f=i.inline=Q,n.highlightFormatting&&(i.formatting="link");var p=v(i);return p?p+=" ":p="",p+o.linkEmail}if(n.xml&&"<"===A&&e.match(/^(!--|\?|!\[CDATA\[|[a-z][a-z0-9-]*(?:\s+[a-z_:.\-]+(?:\s*=\s*[^>]+)?)*\s*(?:>|$))/i,!1)){var m=e.string.indexOf(">",e.pos);if(-1!=m){var E=e.string.substring(e.start,m);/markdown\s*=\s*('|"){0,1}1('|"){0,1}/.test(E)&&(i.md_inside=!0)}return e.backUp(1),i.htmlState=t.startState(r),I(e,i,y)}if(n.xml&&"<"===A&&e.match(/^\/\w*?>/))return i.md_inside=!1,"tag";if("*"===A||"_"===A){for(var B=1,x=1==e.pos?" ":e.string.charAt(e.pos-2);B<3&&e.eat(A);)B++;var _=e.peek()||" ",D=!/\s/.test(_)&&(!f.test(_)||/\s/.test(x)||f.test(x)),M=!/\s/.test(x)&&(!f.test(x)||/\s/.test(_)||f.test(_)),k=null,S=null;if(B%2&&(i.em||!D||"*"!==A&&M&&!f.test(x)?i.em!=A||!M||"*"!==A&&D&&!f.test(_)||(k=!1):k=!0),B>1&&(i.strong||!D||"*"!==A&&M&&!f.test(x)?i.strong!=A||!M||"*"!==A&&D&&!f.test(_)||(S=!1):S=!0),null!=S||null!=k){n.highlightFormatting&&(i.formatting=null==k?"strong":null==S?"em":"strong em"),!0===k&&(i.em=A),!0===S&&(i.strong=A);var d=v(i);return!1===k&&(i.em=!1),!1===S&&(i.strong=!1),d}}else if(" "===A&&(e.eat("*")||e.eat("_"))){if(" "===e.peek())return v(i);e.backUp(1)}if(n.strikethrough)if("~"===A&&e.eatWhile(A)){if(i.strikethrough){n.highlightFormatting&&(i.formatting="strikethrough");var d=v(i);return i.strikethrough=!1,d}if(e.match(/^[^\s]/,!1))return i.strikethrough=!0,n.highlightFormatting&&(i.formatting="strikethrough"),v(i)}else if(" "===A&&e.match(/^~~/,!0)){if(" "===e.peek())return v(i);e.backUp(2)}if(n.emoji&&":"===A&&e.match(/^(?:[a-z_\d+][a-z_\d+-]*|\-[a-z_\d+][a-z_\d+-]*):/)){i.emoji=!0,n.highlightFormatting&&(i.formatting="emoji");var L=v(i);return i.emoji=!1,L}return" "===A&&(e.match(/^ +$/,!1)?i.trailingSpace++:i.trailingSpace&&(i.trailingSpaceNewLine=!0)),v(i)}function Q(t,e){var r=t.next();if(">"===r){e.f=e.inline=b,n.highlightFormatting&&(e.formatting="link");var i=v(e);return i?i+=" ":i="",i+o.linkInline}return t.match(/^[^>]+/,!0),o.linkInline}function w(t,e){if(t.eatSpace())return null;var r,i=t.next();return"("===i||"["===i?(e.f=e.inline=(r="("===i?")":"]",function(t,e){var i=t.next();if(i===r){e.f=e.inline=b,n.highlightFormatting&&(e.formatting="link-string");var o=v(e);return e.linkHref=!1,o}return t.match(x[r]),e.linkHref=!0,v(e)}),n.highlightFormatting&&(e.formatting="link-string"),e.linkHref=!0,v(e)):"error"}var x={")":/^(?:[^\\\(\)]|\\.|\((?:[^\\\(\)]|\\.)*\))*?(?=\))/,"]":/^(?:[^\\\[\]]|\\.|\[(?:[^\\\[\]]|\\.)*\])*?(?=\])/};function _(t,e){return t.match(/^([^\]\\]|\\.)*\]:/,!1)?(e.f=D,t.next(),n.highlightFormatting&&(e.formatting="link"),e.linkText=!0,v(e)):p(t,e,b)}function D(t,e){if(t.match(/^\]:/,!0)){e.f=e.inline=M,n.highlightFormatting&&(e.formatting="link");var r=v(e);return e.linkText=!1,r}return t.match(/^([^\]\\]|\\.)+/,!0),o.linkText}function M(t,e){return t.eatSpace()?null:(t.match(/^[^\s]+/,!0),void 0===t.peek()?e.linkTitle=!0:t.match(/^(?:\s+(?:"(?:[^"\\]|\\\\|\\.)+"|'(?:[^'\\]|\\\\|\\.)+'|\((?:[^)\\]|\\\\|\\.)+\)))?/,!0),e.f=e.inline=b,o.linkHref+" url")}var k={startState:function(){return{f:m,prevLine:{stream:null},thisLine:{stream:null},block:m,htmlState:null,indentation:0,inline:b,text:B,formatting:!1,linkText:!1,linkHref:!1,linkTitle:!1,code:0,em:!1,strong:!1,header:0,setext:0,hr:!1,taskList:!1,list:!1,listStack:[],quote:0,trailingSpace:0,trailingSpaceNewLine:!1,strikethrough:!1,emoji:!1,fencedEndRE:null}},copyState:function(e){return{f:e.f,prevLine:e.prevLine,thisLine:e.thisLine,block:e.block,htmlState:e.htmlState&&t.copyState(r,e.htmlState),indentation:e.indentation,localMode:e.localMode,localState:e.localMode?t.copyState(e.localMode,e.localState):null,inline:e.inline,text:e.text,formatting:!1,linkText:e.linkText,linkTitle:e.linkTitle,linkHref:e.linkHref,code:e.code,em:e.em,strong:e.strong,strikethrough:e.strikethrough,emoji:e.emoji,header:e.header,setext:e.setext,hr:e.hr,taskList:e.taskList,list:e.list,listStack:e.listStack.slice(0),quote:e.quote,indentedCode:e.indentedCode,trailingSpace:e.trailingSpace,trailingSpaceNewLine:e.trailingSpaceNewLine,md_inside:e.md_inside,fencedEndRE:e.fencedEndRE}},token:function(t,e){if(e.formatting=!1,t!=e.thisLine.stream){if(e.header=0,e.hr=!1,t.match(/^\s*$/,!0))return C(e),null;if(e.prevLine=e.thisLine,e.thisLine={stream:t},e.taskList=!1,e.trailingSpace=0,e.trailingSpaceNewLine=!1,!e.localState&&(e.f=e.block,e.f!=y)){var n=t.match(/^\s*/,!0)[0].replace(/\t/g,"    ").length;if(e.indentation=n,e.indentationDiff=null,n>0)return null}}return e.f(t,e)},innerMode:function(t){return t.block==y?{state:t.htmlState,mode:r}:t.localState?{state:t.localState,mode:t.localMode}:{state:t,mode:k}},indent:function(e,n,i){return e.block==y&&r.indent?r.indent(e.htmlState,n,i):e.localState&&e.localMode.indent?e.localMode.indent(e.localState,n,i):t.Pass},blankLine:C,getType:v,blockCommentStart:"\x3c!--",blockCommentEnd:"--\x3e",closeBrackets:"()[]{}''\"\"``",fold:"markdown"};return k},"xml"),t.defineMIME("text/markdown","markdown"),t.defineMIME("text/x-markdown","markdown")}(n(15),n(731),n(292))},,,function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){t.getRandomValues=function(){var t="undefined"!=typeof window&&(window.crypto||window.msCrypto)||n(768)||null;return t&&"function"==typeof t.getRandomValues?function(e){return t.getRandomValues(e)}:t&&"function"==typeof t.randomFillSync?function(e){return t.randomFillSync(e)}:t&&"function"==typeof t.randomBytes?function(e){for(var n=t.randomBytes(e.length),r=0,i=n.length;r<i;++r)e[r]=n[r]}:function(t){for(var e=0,n=0,r=t.length;n<r;++n)n%4==0&&(e=4294967295*Math.random()>>>0),t[n]=255&e,e>>>=8}}()}(e.Random||(e.Random={}))},function(t,e,n){"use strict";var r=n(35).Buffer,i=n(268).Transform;function o(t){i.call(this),this._block=r.allocUnsafe(t),this._blockSize=t,this._blockOffset=0,this._length=[0,0,0,0],this._finalized=!1}n(30)(o,i),o.prototype._transform=function(t,e,n){var r=null;try{this.update(t,e)}catch(t){r=t}n(r)},o.prototype._flush=function(t){var e=null;try{this.push(this.digest())}catch(t){e=t}t(e)},o.prototype.update=function(t,e){if(function(t,e){if(!r.isBuffer(t)&&"string"!=typeof t)throw new TypeError(e+" must be a string or a buffer")}(t,"Data"),this._finalized)throw new Error("Digest already called");r.isBuffer(t)||(t=r.from(t,e));for(var n=this._block,i=0;this._blockOffset+t.length-i>=this._blockSize;){for(var o=this._blockOffset;o<this._blockSize;)n[o++]=t[i++];this._update(),this._blockOffset=0}for(;i<t.length;)n[this._blockOffset++]=t[i++];for(var a=0,s=8*t.length;s>0;++a)this._length[a]+=s,(s=this._length[a]/4294967296|0)>0&&(this._length[a]-=4294967296*s);return this},o.prototype._update=function(){throw new Error("_update is not implemented")},o.prototype.digest=function(t){if(this._finalized)throw new Error("Digest already called");this._finalized=!0;var e=this._digest();void 0!==t&&(e=e.toString(t)),this._block.fill(0),this._blockOffset=0;for(var n=0;n<4;++n)this._length[n]=0;return e},o.prototype._digest=function(){throw new Error("_digest is not implemented")},t.exports=o},function(t,e,n){"use strict";(function(e,r){var i=n(209);t.exports=m;var o,a=n(771);m.ReadableState=C;n(208).EventEmitter;var s=function(t,e){return t.listeners(e).length},A=n(398),l=n(270).Buffer,c=e.Uint8Array||function(){};var u=n(166);u.inherits=n(30);var g=n(772),h=void 0;h=g&&g.debuglog?g.debuglog("stream"):function(){};var d,f=n(773),p=n(399);u.inherits(m,A);var I=["error","close","destroy","pause","resume"];function C(t,e){t=t||{};var r=e instanceof(o=o||n(119));this.objectMode=!!t.objectMode,r&&(this.objectMode=this.objectMode||!!t.readableObjectMode);var i=t.highWaterMark,a=t.readableHighWaterMark,s=this.objectMode?16:16384;this.highWaterMark=i||0===i?i:r&&(a||0===a)?a:s,this.highWaterMark=Math.floor(this.highWaterMark),this.buffer=new f,this.length=0,this.pipes=null,this.pipesCount=0,this.flowing=null,this.ended=!1,this.endEmitted=!1,this.reading=!1,this.sync=!0,this.needReadable=!1,this.emittedReadable=!1,this.readableListening=!1,this.resumeScheduled=!1,this.destroyed=!1,this.defaultEncoding=t.defaultEncoding||"utf8",this.awaitDrain=0,this.readingMore=!1,this.decoder=null,this.encoding=null,t.encoding&&(d||(d=n(210).StringDecoder),this.decoder=new d(t.encoding),this.encoding=t.encoding)}function m(t){if(o=o||n(119),!(this instanceof m))return new m(t);this._readableState=new C(t,this),this.readable=!0,t&&("function"==typeof t.read&&(this._read=t.read),"function"==typeof t.destroy&&(this._destroy=t.destroy)),A.call(this)}function y(t,e,n,r,i){var o,a=t._readableState;null===e?(a.reading=!1,function(t,e){if(e.ended)return;if(e.decoder){var n=e.decoder.end();n&&n.length&&(e.buffer.push(n),e.length+=e.objectMode?1:n.length)}e.ended=!0,b(t)}(t,a)):(i||(o=function(t,e){var n;r=e,l.isBuffer(r)||r instanceof c||"string"==typeof e||void 0===e||t.objectMode||(n=new TypeError("Invalid non-string/buffer chunk"));var r;return n}(a,e)),o?t.emit("error",o):a.objectMode||e&&e.length>0?("string"==typeof e||a.objectMode||Object.getPrototypeOf(e)===l.prototype||(e=function(t){return l.from(t)}(e)),r?a.endEmitted?t.emit("error",new Error("stream.unshift() after end event")):E(t,a,e,!0):a.ended?t.emit("error",new Error("stream.push() after EOF")):(a.reading=!1,a.decoder&&!n?(e=a.decoder.write(e),a.objectMode||0!==e.length?E(t,a,e,!1):w(t,a)):E(t,a,e,!1))):r||(a.reading=!1));return function(t){return!t.ended&&(t.needReadable||t.length<t.highWaterMark||0===t.length)}(a)}function E(t,e,n,r){e.flowing&&0===e.length&&!e.sync?(t.emit("data",n),t.read(0)):(e.length+=e.objectMode?1:n.length,r?e.buffer.unshift(n):e.buffer.push(n),e.needReadable&&b(t)),w(t,e)}Object.defineProperty(m.prototype,"destroyed",{get:function(){return void 0!==this._readableState&&this._readableState.destroyed},set:function(t){this._readableState&&(this._readableState.destroyed=t)}}),m.prototype.destroy=p.destroy,m.prototype._undestroy=p.undestroy,m.prototype._destroy=function(t,e){this.push(null),e(t)},m.prototype.push=function(t,e){var n,r=this._readableState;return r.objectMode?n=!0:"string"==typeof t&&((e=e||r.defaultEncoding)!==r.encoding&&(t=l.from(t,e),e=""),n=!0),y(this,t,e,!1,n)},m.prototype.unshift=function(t){return y(this,t,null,!0,!1)},m.prototype.isPaused=function(){return!1===this._readableState.flowing},m.prototype.setEncoding=function(t){return d||(d=n(210).StringDecoder),this._readableState.decoder=new d(t),this._readableState.encoding=t,this};var v=8388608;function B(t,e){return t<=0||0===e.length&&e.ended?0:e.objectMode?1:t!=t?e.flowing&&e.length?e.buffer.head.data.length:e.length:(t>e.highWaterMark&&(e.highWaterMark=function(t){return t>=v?t=v:(t--,t|=t>>>1,t|=t>>>2,t|=t>>>4,t|=t>>>8,t|=t>>>16,t++),t}(t)),t<=e.length?t:e.ended?e.length:(e.needReadable=!0,0))}function b(t){var e=t._readableState;e.needReadable=!1,e.emittedReadable||(h("emitReadable",e.flowing),e.emittedReadable=!0,e.sync?i.nextTick(Q,t):Q(t))}function Q(t){h("emit readable"),t.emit("readable"),M(t)}function w(t,e){e.readingMore||(e.readingMore=!0,i.nextTick(x,t,e))}function x(t,e){for(var n=e.length;!e.reading&&!e.flowing&&!e.ended&&e.length<e.highWaterMark&&(h("maybeReadMore read 0"),t.read(0),n!==e.length);)n=e.length;e.readingMore=!1}function _(t){h("readable nexttick read 0"),t.read(0)}function D(t,e){e.reading||(h("resume read 0"),t.read(0)),e.resumeScheduled=!1,e.awaitDrain=0,t.emit("resume"),M(t),e.flowing&&!e.reading&&t.read(0)}function M(t){var e=t._readableState;for(h("flow",e.flowing);e.flowing&&null!==t.read(););}function k(t,e){return 0===e.length?null:(e.objectMode?n=e.buffer.shift():!t||t>=e.length?(n=e.decoder?e.buffer.join(""):1===e.buffer.length?e.buffer.head.data:e.buffer.concat(e.length),e.buffer.clear()):n=function(t,e,n){var r;t<e.head.data.length?(r=e.head.data.slice(0,t),e.head.data=e.head.data.slice(t)):r=t===e.head.data.length?e.shift():n?function(t,e){var n=e.head,r=1,i=n.data;t-=i.length;for(;n=n.next;){var o=n.data,a=t>o.length?o.length:t;if(a===o.length?i+=o:i+=o.slice(0,t),0===(t-=a)){a===o.length?(++r,n.next?e.head=n.next:e.head=e.tail=null):(e.head=n,n.data=o.slice(a));break}++r}return e.length-=r,i}(t,e):function(t,e){var n=l.allocUnsafe(t),r=e.head,i=1;r.data.copy(n),t-=r.data.length;for(;r=r.next;){var o=r.data,a=t>o.length?o.length:t;if(o.copy(n,n.length-t,0,a),0===(t-=a)){a===o.length?(++i,r.next?e.head=r.next:e.head=e.tail=null):(e.head=r,r.data=o.slice(a));break}++i}return e.length-=i,n}(t,e);return r}(t,e.buffer,e.decoder),n);var n}function S(t){var e=t._readableState;if(e.length>0)throw new Error('"endReadable()" called on non-empty stream');e.endEmitted||(e.ended=!0,i.nextTick(L,e,t))}function L(t,e){t.endEmitted||0!==t.length||(t.endEmitted=!0,e.readable=!1,e.emit("end"))}function R(t,e){for(var n=0,r=t.length;n<r;n++)if(t[n]===e)return n;return-1}m.prototype.read=function(t){h("read",t),t=parseInt(t,10);var e=this._readableState,n=t;if(0!==t&&(e.emittedReadable=!1),0===t&&e.needReadable&&(e.length>=e.highWaterMark||e.ended))return h("read: emitReadable",e.length,e.ended),0===e.length&&e.ended?S(this):b(this),null;if(0===(t=B(t,e))&&e.ended)return 0===e.length&&S(this),null;var r,i=e.needReadable;return h("need readable",i),(0===e.length||e.length-t<e.highWaterMark)&&h("length less than watermark",i=!0),e.ended||e.reading?h("reading or ended",i=!1):i&&(h("do read"),e.reading=!0,e.sync=!0,0===e.length&&(e.needReadable=!0),this._read(e.highWaterMark),e.sync=!1,e.reading||(t=B(n,e))),null===(r=t>0?k(t,e):null)?(e.needReadable=!0,t=0):e.length-=t,0===e.length&&(e.ended||(e.needReadable=!0),n!==t&&e.ended&&S(this)),null!==r&&this.emit("data",r),r},m.prototype._read=function(t){this.emit("error",new Error("_read() is not implemented"))},m.prototype.pipe=function(t,e){var n=this,o=this._readableState;switch(o.pipesCount){case 0:o.pipes=t;break;case 1:o.pipes=[o.pipes,t];break;default:o.pipes.push(t)}o.pipesCount+=1,h("pipe count=%d opts=%j",o.pipesCount,e);var A=(!e||!1!==e.end)&&t!==r.stdout&&t!==r.stderr?c:m;function l(e,r){h("onunpipe"),e===n&&r&&!1===r.hasUnpiped&&(r.hasUnpiped=!0,h("cleanup"),t.removeListener("close",I),t.removeListener("finish",C),t.removeListener("drain",u),t.removeListener("error",p),t.removeListener("unpipe",l),n.removeListener("end",c),n.removeListener("end",m),n.removeListener("data",f),g=!0,!o.awaitDrain||t._writableState&&!t._writableState.needDrain||u())}function c(){h("onend"),t.end()}o.endEmitted?i.nextTick(A):n.once("end",A),t.on("unpipe",l);var u=function(t){return function(){var e=t._readableState;h("pipeOnDrain",e.awaitDrain),e.awaitDrain&&e.awaitDrain--,0===e.awaitDrain&&s(t,"data")&&(e.flowing=!0,M(t))}}(n);t.on("drain",u);var g=!1;var d=!1;function f(e){h("ondata"),d=!1,!1!==t.write(e)||d||((1===o.pipesCount&&o.pipes===t||o.pipesCount>1&&-1!==R(o.pipes,t))&&!g&&(h("false write response, pause",n._readableState.awaitDrain),n._readableState.awaitDrain++,d=!0),n.pause())}function p(e){h("onerror",e),m(),t.removeListener("error",p),0===s(t,"error")&&t.emit("error",e)}function I(){t.removeListener("finish",C),m()}function C(){h("onfinish"),t.removeListener("close",I),m()}function m(){h("unpipe"),n.unpipe(t)}return n.on("data",f),function(t,e,n){if("function"==typeof t.prependListener)return t.prependListener(e,n);t._events&&t._events[e]?a(t._events[e])?t._events[e].unshift(n):t._events[e]=[n,t._events[e]]:t.on(e,n)}(t,"error",p),t.once("close",I),t.once("finish",C),t.emit("pipe",n),o.flowing||(h("pipe resume"),n.resume()),t},m.prototype.unpipe=function(t){var e=this._readableState,n={hasUnpiped:!1};if(0===e.pipesCount)return this;if(1===e.pipesCount)return t&&t!==e.pipes?this:(t||(t=e.pipes),e.pipes=null,e.pipesCount=0,e.flowing=!1,t&&t.emit("unpipe",this,n),this);if(!t){var r=e.pipes,i=e.pipesCount;e.pipes=null,e.pipesCount=0,e.flowing=!1;for(var o=0;o<i;o++)r[o].emit("unpipe",this,n);return this}var a=R(e.pipes,t);return-1===a?this:(e.pipes.splice(a,1),e.pipesCount-=1,1===e.pipesCount&&(e.pipes=e.pipes[0]),t.emit("unpipe",this,n),this)},m.prototype.on=function(t,e){var n=A.prototype.on.call(this,t,e);if("data"===t)!1!==this._readableState.flowing&&this.resume();else if("readable"===t){var r=this._readableState;r.endEmitted||r.readableListening||(r.readableListening=r.needReadable=!0,r.emittedReadable=!1,r.reading?r.length&&b(this):i.nextTick(_,this))}return n},m.prototype.addListener=m.prototype.on,m.prototype.resume=function(){var t=this._readableState;return t.flowing||(h("resume"),t.flowing=!0,function(t,e){e.resumeScheduled||(e.resumeScheduled=!0,i.nextTick(D,t,e))}(this,t)),this},m.prototype.pause=function(){return h("call pause flowing=%j",this._readableState.flowing),!1!==this._readableState.flowing&&(h("pause"),this._readableState.flowing=!1,this.emit("pause")),this},m.prototype.wrap=function(t){var e=this,n=this._readableState,r=!1;for(var i in t.on("end",function(){if(h("wrapped end"),n.decoder&&!n.ended){var t=n.decoder.end();t&&t.length&&e.push(t)}e.push(null)}),t.on("data",function(i){(h("wrapped data"),n.decoder&&(i=n.decoder.write(i)),n.objectMode&&null==i)||(n.objectMode||i&&i.length)&&(e.push(i)||(r=!0,t.pause()))}),t)void 0===this[i]&&"function"==typeof t[i]&&(this[i]=function(e){return function(){return t[e].apply(t,arguments)}}(i));for(var o=0;o<I.length;o++)t.on(I[o],this.emit.bind(this,I[o]));return this._read=function(e){h("wrapped _read",e),r&&(r=!1,t.resume())},this},Object.defineProperty(m.prototype,"readableHighWaterMark",{enumerable:!1,get:function(){return this._readableState.highWaterMark}}),m._fromList=k}).call(this,n(37),n(58))},function(t,e,n){t.exports=n(208).EventEmitter},function(t,e,n){"use strict";var r=n(209);function i(t,e){t.emit("error",e)}t.exports={destroy:function(t,e){var n=this,o=this._readableState&&this._readableState.destroyed,a=this._writableState&&this._writableState.destroyed;return o||a?(e?e(t):!t||this._writableState&&this._writableState.errorEmitted||r.nextTick(i,this,t),this):(this._readableState&&(this._readableState.destroyed=!0),this._writableState&&(this._writableState.destroyed=!0),this._destroy(t||null,function(t){!e&&t?(r.nextTick(i,n,t),n._writableState&&(n._writableState.errorEmitted=!0)):e&&e(t)}),this)},undestroy:function(){this._readableState&&(this._readableState.destroyed=!1,this._readableState.reading=!1,this._readableState.ended=!1,this._readableState.endEmitted=!1),this._writableState&&(this._writableState.destroyed=!1,this._writableState.ended=!1,this._writableState.ending=!1,this._writableState.finished=!1,this._writableState.errorEmitted=!1)}}},function(t,e,n){"use strict";t.exports=a;var r=n(119),i=n(166);function o(t,e){var n=this._transformState;n.transforming=!1;var r=n.writecb;if(!r)return this.emit("error",new Error("write callback called multiple times"));n.writechunk=null,n.writecb=null,null!=e&&this.push(e),r(t);var i=this._readableState;i.reading=!1,(i.needReadable||i.length<i.highWaterMark)&&this._read(i.highWaterMark)}function a(t){if(!(this instanceof a))return new a(t);r.call(this,t),this._transformState={afterTransform:o.bind(this),needTransform:!1,transforming:!1,writecb:null,writechunk:null,writeencoding:null},this._readableState.needReadable=!0,this._readableState.sync=!1,t&&("function"==typeof t.transform&&(this._transform=t.transform),"function"==typeof t.flush&&(this._flush=t.flush)),this.on("prefinish",s)}function s(){var t=this;"function"==typeof this._flush?this._flush(function(e,n){A(t,e,n)}):A(this,null,null)}function A(t,e,n){if(e)return t.emit("error",e);if(null!=n&&t.push(n),t._writableState.length)throw new Error("Calling transform done when ws.length != 0");if(t._transformState.transforming)throw new Error("Calling transform done when still transforming");return t.push(null)}i.inherits=n(30),i.inherits(a,r),a.prototype.push=function(t,e){return this._transformState.needTransform=!1,r.prototype.push.call(this,t,e)},a.prototype._transform=function(t,e,n){throw new Error("_transform() is not implemented")},a.prototype._write=function(t,e,n){var r=this._transformState;if(r.writecb=n,r.writechunk=t,r.writeencoding=e,!r.transforming){var i=this._readableState;(r.needTransform||i.needReadable||i.length<i.highWaterMark)&&this._read(i.highWaterMark)}},a.prototype._read=function(t){var e=this._transformState;null!==e.writechunk&&e.writecb&&!e.transforming?(e.transforming=!0,this._transform(e.writechunk,e.writeencoding,e.afterTransform)):e.needTransform=!0},a.prototype._destroy=function(t,e){var n=this;r.prototype._destroy.call(this,t,function(t){e(t),n.emit("close")})}},function(t,e,n){var r=n(30),i=n(136),o=n(35).Buffer,a=[1116352408,1899447441,3049323471,3921009573,961987163,1508970993,2453635748,2870763221,3624381080,310598401,607225278,1426881987,1925078388,2162078206,2614888103,3248222580,3835390401,4022224774,264347078,604807628,770255983,1249150122,1555081692,1996064986,2554220882,2821834349,2952996808,3210313671,3336571891,3584528711,113926993,338241895,666307205,773529912,1294757372,1396182291,1695183700,1986661051,2177026350,2456956037,2730485921,2820302411,3259730800,3345764771,3516065817,3600352804,4094571909,275423344,430227734,506948616,659060556,883997877,958139571,1322822218,1537002063,1747873779,1955562222,2024104815,2227730452,2361852424,2428436474,2756734187,3204031479,3329325298],s=new Array(64);function A(){this.init(),this._w=s,i.call(this,64,56)}function l(t,e,n){return n^t&(e^n)}function c(t,e,n){return t&e|n&(t|e)}function u(t){return(t>>>2|t<<30)^(t>>>13|t<<19)^(t>>>22|t<<10)}function g(t){return(t>>>6|t<<26)^(t>>>11|t<<21)^(t>>>25|t<<7)}function h(t){return(t>>>7|t<<25)^(t>>>18|t<<14)^t>>>3}r(A,i),A.prototype.init=function(){return this._a=1779033703,this._b=3144134277,this._c=1013904242,this._d=2773480762,this._e=1359893119,this._f=2600822924,this._g=528734635,this._h=1541459225,this},A.prototype._update=function(t){for(var e,n=this._w,r=0|this._a,i=0|this._b,o=0|this._c,s=0|this._d,A=0|this._e,d=0|this._f,f=0|this._g,p=0|this._h,I=0;I<16;++I)n[I]=t.readInt32BE(4*I);for(;I<64;++I)n[I]=0|(((e=n[I-2])>>>17|e<<15)^(e>>>19|e<<13)^e>>>10)+n[I-7]+h(n[I-15])+n[I-16];for(var C=0;C<64;++C){var m=p+g(A)+l(A,d,f)+a[C]+n[C]|0,y=u(r)+c(r,i,o)|0;p=f,f=d,d=A,A=s+m|0,s=o,o=i,i=r,r=m+y|0}this._a=r+this._a|0,this._b=i+this._b|0,this._c=o+this._c|0,this._d=s+this._d|0,this._e=A+this._e|0,this._f=d+this._f|0,this._g=f+this._g|0,this._h=p+this._h|0},A.prototype._hash=function(){var t=o.allocUnsafe(32);return t.writeInt32BE(this._a,0),t.writeInt32BE(this._b,4),t.writeInt32BE(this._c,8),t.writeInt32BE(this._d,12),t.writeInt32BE(this._e,16),t.writeInt32BE(this._f,20),t.writeInt32BE(this._g,24),t.writeInt32BE(this._h,28),t},t.exports=A},function(t,e,n){var r=n(30),i=n(136),o=n(35).Buffer,a=[1116352408,3609767458,1899447441,602891725,3049323471,3964484399,3921009573,2173295548,961987163,4081628472,1508970993,3053834265,2453635748,2937671579,2870763221,3664609560,3624381080,2734883394,310598401,1164996542,607225278,1323610764,1426881987,3590304994,1925078388,4068182383,2162078206,991336113,2614888103,633803317,3248222580,3479774868,3835390401,2666613458,4022224774,944711139,264347078,2341262773,604807628,2007800933,770255983,1495990901,1249150122,1856431235,1555081692,3175218132,1996064986,2198950837,2554220882,3999719339,2821834349,766784016,2952996808,2566594879,3210313671,3203337956,3336571891,1034457026,3584528711,2466948901,113926993,3758326383,338241895,168717936,666307205,1188179964,773529912,1546045734,1294757372,1522805485,1396182291,2643833823,1695183700,2343527390,1986661051,1014477480,2177026350,1206759142,2456956037,344077627,2730485921,1290863460,2820302411,3158454273,3259730800,3505952657,3345764771,106217008,3516065817,3606008344,3600352804,1432725776,4094571909,1467031594,275423344,851169720,430227734,3100823752,506948616,1363258195,659060556,3750685593,883997877,3785050280,958139571,3318307427,1322822218,3812723403,1537002063,2003034995,1747873779,3602036899,1955562222,1575990012,2024104815,1125592928,2227730452,2716904306,2361852424,442776044,2428436474,593698344,2756734187,3733110249,3204031479,2999351573,3329325298,3815920427,3391569614,3928383900,3515267271,566280711,3940187606,3454069534,4118630271,4000239992,116418474,1914138554,174292421,2731055270,289380356,3203993006,460393269,320620315,685471733,587496836,852142971,1086792851,1017036298,365543100,1126000580,2618297676,1288033470,3409855158,1501505948,4234509866,1607167915,987167468,1816402316,1246189591],s=new Array(160);function A(){this.init(),this._w=s,i.call(this,128,112)}function l(t,e,n){return n^t&(e^n)}function c(t,e,n){return t&e|n&(t|e)}function u(t,e){return(t>>>28|e<<4)^(e>>>2|t<<30)^(e>>>7|t<<25)}function g(t,e){return(t>>>14|e<<18)^(t>>>18|e<<14)^(e>>>9|t<<23)}function h(t,e){return(t>>>1|e<<31)^(t>>>8|e<<24)^t>>>7}function d(t,e){return(t>>>1|e<<31)^(t>>>8|e<<24)^(t>>>7|e<<25)}function f(t,e){return(t>>>19|e<<13)^(e>>>29|t<<3)^t>>>6}function p(t,e){return(t>>>19|e<<13)^(e>>>29|t<<3)^(t>>>6|e<<26)}function I(t,e){return t>>>0<e>>>0?1:0}r(A,i),A.prototype.init=function(){return this._ah=1779033703,this._bh=3144134277,this._ch=1013904242,this._dh=2773480762,this._eh=1359893119,this._fh=2600822924,this._gh=528734635,this._hh=1541459225,this._al=4089235720,this._bl=2227873595,this._cl=4271175723,this._dl=1595750129,this._el=2917565137,this._fl=725511199,this._gl=4215389547,this._hl=327033209,this},A.prototype._update=function(t){for(var e=this._w,n=0|this._ah,r=0|this._bh,i=0|this._ch,o=0|this._dh,s=0|this._eh,A=0|this._fh,C=0|this._gh,m=0|this._hh,y=0|this._al,E=0|this._bl,v=0|this._cl,B=0|this._dl,b=0|this._el,Q=0|this._fl,w=0|this._gl,x=0|this._hl,_=0;_<32;_+=2)e[_]=t.readInt32BE(4*_),e[_+1]=t.readInt32BE(4*_+4);for(;_<160;_+=2){var D=e[_-30],M=e[_-30+1],k=h(D,M),S=d(M,D),L=f(D=e[_-4],M=e[_-4+1]),R=p(M,D),N=e[_-14],F=e[_-14+1],T=e[_-32],U=e[_-32+1],O=S+F|0,G=k+N+I(O,S)|0;G=(G=G+L+I(O=O+R|0,R)|0)+T+I(O=O+U|0,U)|0,e[_]=G,e[_+1]=O}for(var H=0;H<160;H+=2){G=e[H],O=e[H+1];var j=c(n,r,i),Y=c(y,E,v),P=u(n,y),K=u(y,n),z=g(s,b),J=g(b,s),q=a[H],V=a[H+1],W=l(s,A,C),X=l(b,Q,w),Z=x+J|0,$=m+z+I(Z,x)|0;$=($=($=$+W+I(Z=Z+X|0,X)|0)+q+I(Z=Z+V|0,V)|0)+G+I(Z=Z+O|0,O)|0;var tt=K+Y|0,et=P+j+I(tt,K)|0;m=C,x=w,C=A,w=Q,A=s,Q=b,s=o+$+I(b=B+Z|0,B)|0,o=i,B=v,i=r,v=E,r=n,E=y,n=$+et+I(y=Z+tt|0,Z)|0}this._al=this._al+y|0,this._bl=this._bl+E|0,this._cl=this._cl+v|0,this._dl=this._dl+B|0,this._el=this._el+b|0,this._fl=this._fl+Q|0,this._gl=this._gl+w|0,this._hl=this._hl+x|0,this._ah=this._ah+n+I(this._al,y)|0,this._bh=this._bh+r+I(this._bl,E)|0,this._ch=this._ch+i+I(this._cl,v)|0,this._dh=this._dh+o+I(this._dl,B)|0,this._eh=this._eh+s+I(this._el,b)|0,this._fh=this._fh+A+I(this._fl,Q)|0,this._gh=this._gh+C+I(this._gl,w)|0,this._hh=this._hh+m+I(this._hl,x)|0},A.prototype._hash=function(){var t=o.allocUnsafe(64);function e(e,n,r){t.writeInt32BE(e,r),t.writeInt32BE(n,r+4)}return e(this._ah,this._al,0),e(this._bh,this._bl,8),e(this._ch,this._cl,16),e(this._dh,this._dl,24),e(this._eh,this._el,32),e(this._fh,this._fl,40),e(this._gh,this._gl,48),e(this._hh,this._hl,56),t},t.exports=A},function(t,e,n){"use strict";var r=n(30),i=n(786),o=n(105),a=n(35).Buffer,s=n(404),A=n(272),l=n(273),c=a.alloc(128);function u(t,e){o.call(this,"digest"),"string"==typeof e&&(e=a.from(e));var n="sha512"===t||"sha384"===t?128:64;(this._alg=t,this._key=e,e.length>n)?e=("rmd160"===t?new A:l(t)).update(e).digest():e.length<n&&(e=a.concat([e,c],n));for(var r=this._ipad=a.allocUnsafe(n),i=this._opad=a.allocUnsafe(n),s=0;s<n;s++)r[s]=54^e[s],i[s]=92^e[s];this._hash="rmd160"===t?new A:l(t),this._hash.update(r)}r(u,o),u.prototype._update=function(t){this._hash.update(t)},u.prototype._final=function(){var t=this._hash.digest();return("rmd160"===this._alg?new A:l(this._alg)).update(this._opad).update(t).digest()},t.exports=function(t,e){return"rmd160"===(t=t.toLowerCase())||"ripemd160"===t?new u("rmd160",e):"md5"===t?new i(s,e):new u(t,e)}},function(t,e,n){var r=n(267);t.exports=function(t){return(new r).update(t).digest()}},function(t){t.exports=JSON.parse('{"sha224WithRSAEncryption":{"sign":"rsa","hash":"sha224","id":"302d300d06096086480165030402040500041c"},"RSA-SHA224":{"sign":"ecdsa/rsa","hash":"sha224","id":"302d300d06096086480165030402040500041c"},"sha256WithRSAEncryption":{"sign":"rsa","hash":"sha256","id":"3031300d060960864801650304020105000420"},"RSA-SHA256":{"sign":"ecdsa/rsa","hash":"sha256","id":"3031300d060960864801650304020105000420"},"sha384WithRSAEncryption":{"sign":"rsa","hash":"sha384","id":"3041300d060960864801650304020205000430"},"RSA-SHA384":{"sign":"ecdsa/rsa","hash":"sha384","id":"3041300d060960864801650304020205000430"},"sha512WithRSAEncryption":{"sign":"rsa","hash":"sha512","id":"3051300d060960864801650304020305000440"},"RSA-SHA512":{"sign":"ecdsa/rsa","hash":"sha512","id":"3051300d060960864801650304020305000440"},"RSA-SHA1":{"sign":"rsa","hash":"sha1","id":"3021300906052b0e03021a05000414"},"ecdsa-with-SHA1":{"sign":"ecdsa","hash":"sha1","id":""},"sha256":{"sign":"ecdsa","hash":"sha256","id":""},"sha224":{"sign":"ecdsa","hash":"sha224","id":""},"sha384":{"sign":"ecdsa","hash":"sha384","id":""},"sha512":{"sign":"ecdsa","hash":"sha512","id":""},"DSA-SHA":{"sign":"dsa","hash":"sha1","id":""},"DSA-SHA1":{"sign":"dsa","hash":"sha1","id":""},"DSA":{"sign":"dsa","hash":"sha1","id":""},"DSA-WITH-SHA224":{"sign":"dsa","hash":"sha224","id":""},"DSA-SHA224":{"sign":"dsa","hash":"sha224","id":""},"DSA-WITH-SHA256":{"sign":"dsa","hash":"sha256","id":""},"DSA-SHA256":{"sign":"dsa","hash":"sha256","id":""},"DSA-WITH-SHA384":{"sign":"dsa","hash":"sha384","id":""},"DSA-SHA384":{"sign":"dsa","hash":"sha384","id":""},"DSA-WITH-SHA512":{"sign":"dsa","hash":"sha512","id":""},"DSA-SHA512":{"sign":"dsa","hash":"sha512","id":""},"DSA-RIPEMD160":{"sign":"dsa","hash":"rmd160","id":""},"ripemd160WithRSA":{"sign":"rsa","hash":"rmd160","id":"3021300906052b2403020105000414"},"RSA-RIPEMD160":{"sign":"rsa","hash":"rmd160","id":"3021300906052b2403020105000414"},"md5WithRSAEncryption":{"sign":"rsa","hash":"md5","id":"3020300c06082a864886f70d020505000410"},"RSA-MD5":{"sign":"rsa","hash":"md5","id":"3020300c06082a864886f70d020505000410"}}')},function(t,e,n){e.pbkdf2=n(788),e.pbkdf2Sync=n(409)},function(t,e,n){(function(e){var n=Math.pow(2,30)-1;function r(t,n){if("string"!=typeof t&&!e.isBuffer(t))throw new TypeError(n+" must be a buffer or string")}t.exports=function(t,e,i,o){if(r(t,"Password"),r(e,"Salt"),"number"!=typeof i)throw new TypeError("Iterations not a number");if(i<0)throw new TypeError("Bad iterations");if("number"!=typeof o)throw new TypeError("Key length not a number");if(o<0||o>n||o!=o)throw new TypeError("Bad key length")}}).call(this,n(42).Buffer)},function(t,e,n){(function(e){var n;e.browser?n="utf-8":n=parseInt(e.version.split(".")[0].slice(1),10)>=6?"utf-8":"binary";t.exports=n}).call(this,n(58))},function(t,e,n){var r=n(404),i=n(272),o=n(273),a=n(407),s=n(408),A=n(35).Buffer,l=A.alloc(128),c={md5:16,sha1:20,sha224:28,sha256:32,sha384:48,sha512:64,rmd160:20,ripemd160:20};function u(t,e,n){var a=function(t){return"rmd160"===t||"ripemd160"===t?function(t){return(new i).update(t).digest()}:"md5"===t?r:function(e){return o(t).update(e).digest()}}(t),s="sha512"===t||"sha384"===t?128:64;e.length>s?e=a(e):e.length<s&&(e=A.concat([e,l],s));for(var u=A.allocUnsafe(s+c[t]),g=A.allocUnsafe(s+c[t]),h=0;h<s;h++)u[h]=54^e[h],g[h]=92^e[h];var d=A.allocUnsafe(s+n+4);u.copy(d,0,0,s),this.ipad1=d,this.ipad2=u,this.opad=g,this.alg=t,this.blocksize=s,this.hash=a,this.size=c[t]}u.prototype.run=function(t,e){return t.copy(e,this.blocksize),this.hash(e).copy(this.opad,this.blocksize),this.hash(this.opad)},t.exports=function(t,e,n,r,i){a(t,e,n,r),A.isBuffer(t)||(t=A.from(t,s)),A.isBuffer(e)||(e=A.from(e,s));var o=new u(i=i||"sha1",t,e.length),l=A.allocUnsafe(r),g=A.allocUnsafe(e.length+4);e.copy(g,0,0,e.length);for(var h=0,d=c[i],f=Math.ceil(r/d),p=1;p<=f;p++){g.writeUInt32BE(p,e.length);for(var I=o.run(g,o.ipad1),C=I,m=1;m<n;m++){C=o.run(C,o.ipad2);for(var y=0;y<d;y++)I[y]^=C[y]}I.copy(l,h),h+=d}return l}},function(t,e,n){var r=n(167),i=n(35).Buffer,o=n(411);function a(t){var e=t._cipher.encryptBlockRaw(t._prev);return o(t._prev),e}e.encrypt=function(t,e){var n=Math.ceil(e.length/16),o=t._cache.length;t._cache=i.concat([t._cache,i.allocUnsafe(16*n)]);for(var s=0;s<n;s++){var A=a(t),l=o+16*s;t._cache.writeUInt32BE(A[0],l+0),t._cache.writeUInt32BE(A[1],l+4),t._cache.writeUInt32BE(A[2],l+8),t._cache.writeUInt32BE(A[3],l+12)}var c=t._cache.slice(0,e.length);return t._cache=t._cache.slice(e.length),r(e,c)}},function(t,e){t.exports=function(t){for(var e,n=t.length;n--;){if(255!==(e=t.readUInt8(n))){e++,t.writeUInt8(e,n);break}t.writeUInt8(0,n)}}},function(t){t.exports=JSON.parse('{"aes-128-ecb":{"cipher":"AES","key":128,"iv":0,"mode":"ECB","type":"block"},"aes-192-ecb":{"cipher":"AES","key":192,"iv":0,"mode":"ECB","type":"block"},"aes-256-ecb":{"cipher":"AES","key":256,"iv":0,"mode":"ECB","type":"block"},"aes-128-cbc":{"cipher":"AES","key":128,"iv":16,"mode":"CBC","type":"block"},"aes-192-cbc":{"cipher":"AES","key":192,"iv":16,"mode":"CBC","type":"block"},"aes-256-cbc":{"cipher":"AES","key":256,"iv":16,"mode":"CBC","type":"block"},"aes128":{"cipher":"AES","key":128,"iv":16,"mode":"CBC","type":"block"},"aes192":{"cipher":"AES","key":192,"iv":16,"mode":"CBC","type":"block"},"aes256":{"cipher":"AES","key":256,"iv":16,"mode":"CBC","type":"block"},"aes-128-cfb":{"cipher":"AES","key":128,"iv":16,"mode":"CFB","type":"stream"},"aes-192-cfb":{"cipher":"AES","key":192,"iv":16,"mode":"CFB","type":"stream"},"aes-256-cfb":{"cipher":"AES","key":256,"iv":16,"mode":"CFB","type":"stream"},"aes-128-cfb8":{"cipher":"AES","key":128,"iv":16,"mode":"CFB8","type":"stream"},"aes-192-cfb8":{"cipher":"AES","key":192,"iv":16,"mode":"CFB8","type":"stream"},"aes-256-cfb8":{"cipher":"AES","key":256,"iv":16,"mode":"CFB8","type":"stream"},"aes-128-cfb1":{"cipher":"AES","key":128,"iv":16,"mode":"CFB1","type":"stream"},"aes-192-cfb1":{"cipher":"AES","key":192,"iv":16,"mode":"CFB1","type":"stream"},"aes-256-cfb1":{"cipher":"AES","key":256,"iv":16,"mode":"CFB1","type":"stream"},"aes-128-ofb":{"cipher":"AES","key":128,"iv":16,"mode":"OFB","type":"stream"},"aes-192-ofb":{"cipher":"AES","key":192,"iv":16,"mode":"OFB","type":"stream"},"aes-256-ofb":{"cipher":"AES","key":256,"iv":16,"mode":"OFB","type":"stream"},"aes-128-ctr":{"cipher":"AES","key":128,"iv":16,"mode":"CTR","type":"stream"},"aes-192-ctr":{"cipher":"AES","key":192,"iv":16,"mode":"CTR","type":"stream"},"aes-256-ctr":{"cipher":"AES","key":256,"iv":16,"mode":"CTR","type":"stream"},"aes-128-gcm":{"cipher":"AES","key":128,"iv":12,"mode":"GCM","type":"auth"},"aes-192-gcm":{"cipher":"AES","key":192,"iv":12,"mode":"GCM","type":"auth"},"aes-256-gcm":{"cipher":"AES","key":256,"iv":12,"mode":"GCM","type":"auth"}}')},function(t,e,n){var r=n(211),i=n(35).Buffer,o=n(105),a=n(30),s=n(803),A=n(167),l=n(411);function c(t,e,n,a){o.call(this);var A=i.alloc(4,0);this._cipher=new r.AES(e);var c=this._cipher.encryptBlock(A);this._ghash=new s(c),n=function(t,e,n){if(12===e.length)return t._finID=i.concat([e,i.from([0,0,0,1])]),i.concat([e,i.from([0,0,0,2])]);var r=new s(n),o=e.length,a=o%16;r.update(e),a&&(a=16-a,r.update(i.alloc(a,0))),r.update(i.alloc(8,0));var A=8*o,c=i.alloc(8);c.writeUIntBE(A,0,8),r.update(c),t._finID=r.state;var u=i.from(t._finID);return l(u),u}(this,n,c),this._prev=i.from(n),this._cache=i.allocUnsafe(0),this._secCache=i.allocUnsafe(0),this._decrypt=a,this._alen=0,this._len=0,this._mode=t,this._authTag=null,this._called=!1}a(c,o),c.prototype._update=function(t){if(!this._called&&this._alen){var e=16-this._alen%16;e<16&&(e=i.alloc(e,0),this._ghash.update(e))}this._called=!0;var n=this._mode.encrypt(this,t);return this._decrypt?this._ghash.update(t):this._ghash.update(n),this._len+=t.length,n},c.prototype._final=function(){if(this._decrypt&&!this._authTag)throw new Error("Unsupported state or unable to authenticate data");var t=A(this._ghash.final(8*this._alen,8*this._len),this._cipher.encryptBlock(this._finID));if(this._decrypt&&function(t,e){var n=0;t.length!==e.length&&n++;for(var r=Math.min(t.length,e.length),i=0;i<r;++i)n+=t[i]^e[i];return n}(t,this._authTag))throw new Error("Unsupported state or unable to authenticate data");this._authTag=t,this._cipher.scrub()},c.prototype.getAuthTag=function(){if(this._decrypt||!i.isBuffer(this._authTag))throw new Error("Attempting to get auth tag in unsupported state");return this._authTag},c.prototype.setAuthTag=function(t){if(!this._decrypt)throw new Error("Attempting to set auth tag in unsupported state");this._authTag=t},c.prototype.setAAD=function(t){if(this._called)throw new Error("Attempting to set AAD in unsupported state");this._ghash.update(t),this._alen+=t.length},t.exports=c},function(t,e,n){var r=n(211),i=n(35).Buffer,o=n(105);function a(t,e,n,a){o.call(this),this._cipher=new r.AES(e),this._prev=i.from(n),this._cache=i.allocUnsafe(0),this._secCache=i.allocUnsafe(0),this._decrypt=a,this._mode=t}n(30)(a,o),a.prototype._update=function(t){return this._mode.encrypt(this,t,this._decrypt)},a.prototype._final=function(){this._cipher.scrub()},t.exports=a},function(t,e,n){var r=n(135);t.exports=C,C.simpleSieve=p,C.fermatTest=I;var i=n(53),o=new i(24),a=new(n(416)),s=new i(1),A=new i(2),l=new i(5),c=(new i(16),new i(8),new i(10)),u=new i(3),g=(new i(7),new i(11)),h=new i(4),d=(new i(12),null);function f(){if(null!==d)return d;var t=[];t[0]=2;for(var e=1,n=3;n<1048576;n+=2){for(var r=Math.ceil(Math.sqrt(n)),i=0;i<e&&t[i]<=r&&n%t[i]!=0;i++);e!==i&&t[i]<=r||(t[e++]=n)}return d=t,t}function p(t){for(var e=f(),n=0;n<e.length;n++)if(0===t.modn(e[n]))return 0===t.cmpn(e[n]);return!0}function I(t){var e=i.mont(t);return 0===A.toRed(e).redPow(t.subn(1)).fromRed().cmpn(1)}function C(t,e){if(t<16)return new i(2===e||5===e?[140,123]:[140,39]);var n,d;for(e=new i(e);;){for(n=new i(r(Math.ceil(t/8)));n.bitLength()>t;)n.ishrn(1);if(n.isEven()&&n.iadd(s),n.testn(1)||n.iadd(A),e.cmp(A)){if(!e.cmp(l))for(;n.mod(c).cmp(u);)n.iadd(h)}else for(;n.mod(o).cmp(g);)n.iadd(h);if(p(d=n.shrn(1))&&p(n)&&I(d)&&I(n)&&a.test(d)&&a.test(n))return n}}},function(t,e,n){var r=n(53),i=n(277);function o(t){this.rand=t||new i.Rand}t.exports=o,o.create=function(t){return new o(t)},o.prototype._randbelow=function(t){var e=t.bitLength(),n=Math.ceil(e/8);do{var i=new r(this.rand.generate(n))}while(i.cmp(t)>=0);return i},o.prototype._randrange=function(t,e){var n=e.sub(t);return t.add(this._randbelow(n))},o.prototype.test=function(t,e,n){var i=t.bitLength(),o=r.mont(t),a=new r(1).toRed(o);e||(e=Math.max(1,i/48|0));for(var s=t.subn(1),A=0;!s.testn(A);A++);for(var l=t.shrn(A),c=s.toRed(o);e>0;e--){var u=this._randrange(new r(2),s);n&&n(u);var g=u.toRed(o).redPow(l);if(0!==g.cmp(a)&&0!==g.cmp(c)){for(var h=1;h<A;h++){if(0===(g=g.redSqr()).cmp(a))return!1;if(0===g.cmp(c))break}if(h===A)return!1}}return!0},o.prototype.getDivisor=function(t,e){var n=t.bitLength(),i=r.mont(t),o=new r(1).toRed(i);e||(e=Math.max(1,n/48|0));for(var a=t.subn(1),s=0;!a.testn(s);s++);for(var A=t.shrn(s),l=a.toRed(i);e>0;e--){var c=this._randrange(new r(2),a),u=t.gcd(c);if(0!==u.cmpn(1))return u;var g=c.toRed(i).redPow(A);if(0!==g.cmp(o)&&0!==g.cmp(l)){for(var h=1;h<s;h++){if(0===(g=g.redSqr()).cmp(o))return g.fromRed().subn(1).gcd(t);if(0===g.cmp(l))break}if(h===s)return(g=g.redSqr()).fromRed().subn(1).gcd(t)}}return!1}},function(t,e,n){"use strict";var r=e;function i(t){return 1===t.length?"0"+t:t}function o(t){for(var e="",n=0;n<t.length;n++)e+=i(t[n].toString(16));return e}r.toArray=function(t,e){if(Array.isArray(t))return t.slice();if(!t)return[];var n=[];if("string"!=typeof t){for(var r=0;r<t.length;r++)n[r]=0|t[r];return n}if("hex"===e)for((t=t.replace(/[^a-z0-9]+/gi,"")).length%2!=0&&(t="0"+t),r=0;r<t.length;r+=2)n.push(parseInt(t[r]+t[r+1],16));else for(r=0;r<t.length;r++){var i=t.charCodeAt(r),o=i>>8,a=255&i;o?n.push(o,a):n.push(a)}return n},r.zero2=i,r.toHex=o,r.encode=function(t,e){return"hex"===e?o(t):t}},function(t,e,n){"use strict";var r=e;r.base=n(213),r.short=n(814),r.mont=n(815),r.edwards=n(816)},function(t,e,n){"use strict";var r=n(87).rotr32;function i(t,e,n){return t&e^~t&n}function o(t,e,n){return t&e^t&n^e&n}function a(t,e,n){return t^e^n}e.ft_1=function(t,e,n,r){return 0===t?i(e,n,r):1===t||3===t?a(e,n,r):2===t?o(e,n,r):void 0},e.ch32=i,e.maj32=o,e.p32=a,e.s0_256=function(t){return r(t,2)^r(t,13)^r(t,22)},e.s1_256=function(t){return r(t,6)^r(t,11)^r(t,25)},e.g0_256=function(t){return r(t,7)^r(t,18)^t>>>3},e.g1_256=function(t){return r(t,17)^r(t,19)^t>>>10}},function(t,e,n){"use strict";var r=n(87),i=n(168),o=n(419),a=n(77),s=r.sum32,A=r.sum32_4,l=r.sum32_5,c=o.ch32,u=o.maj32,g=o.s0_256,h=o.s1_256,d=o.g0_256,f=o.g1_256,p=i.BlockHash,I=[1116352408,1899447441,3049323471,3921009573,961987163,1508970993,2453635748,2870763221,3624381080,310598401,607225278,1426881987,1925078388,2162078206,2614888103,3248222580,3835390401,4022224774,264347078,604807628,770255983,1249150122,1555081692,1996064986,2554220882,2821834349,2952996808,3210313671,3336571891,3584528711,113926993,338241895,666307205,773529912,1294757372,1396182291,1695183700,1986661051,2177026350,2456956037,2730485921,2820302411,3259730800,3345764771,3516065817,3600352804,4094571909,275423344,430227734,506948616,659060556,883997877,958139571,1322822218,1537002063,1747873779,1955562222,2024104815,2227730452,2361852424,2428436474,2756734187,3204031479,3329325298];function C(){if(!(this instanceof C))return new C;p.call(this),this.h=[1779033703,3144134277,1013904242,2773480762,1359893119,2600822924,528734635,1541459225],this.k=I,this.W=new Array(64)}r.inherits(C,p),t.exports=C,C.blockSize=512,C.outSize=256,C.hmacStrength=192,C.padLength=64,C.prototype._update=function(t,e){for(var n=this.W,r=0;r<16;r++)n[r]=t[e+r];for(;r<n.length;r++)n[r]=A(f(n[r-2]),n[r-7],d(n[r-15]),n[r-16]);var i=this.h[0],o=this.h[1],p=this.h[2],I=this.h[3],C=this.h[4],m=this.h[5],y=this.h[6],E=this.h[7];for(a(this.k.length===n.length),r=0;r<n.length;r++){var v=l(E,h(C),c(C,m,y),this.k[r],n[r]),B=s(g(i),u(i,o,p));E=y,y=m,m=C,C=s(I,v),I=p,p=o,o=i,i=s(v,B)}this.h[0]=s(this.h[0],i),this.h[1]=s(this.h[1],o),this.h[2]=s(this.h[2],p),this.h[3]=s(this.h[3],I),this.h[4]=s(this.h[4],C),this.h[5]=s(this.h[5],m),this.h[6]=s(this.h[6],y),this.h[7]=s(this.h[7],E)},C.prototype._digest=function(t){return"hex"===t?r.toHex32(this.h,"big"):r.split32(this.h,"big")}},function(t,e,n){"use strict";var r=n(87),i=n(168),o=n(77),a=r.rotr64_hi,s=r.rotr64_lo,A=r.shr64_hi,l=r.shr64_lo,c=r.sum64,u=r.sum64_hi,g=r.sum64_lo,h=r.sum64_4_hi,d=r.sum64_4_lo,f=r.sum64_5_hi,p=r.sum64_5_lo,I=i.BlockHash,C=[1116352408,3609767458,1899447441,602891725,3049323471,3964484399,3921009573,2173295548,961987163,4081628472,1508970993,3053834265,2453635748,2937671579,2870763221,3664609560,3624381080,2734883394,310598401,1164996542,607225278,1323610764,1426881987,3590304994,1925078388,4068182383,2162078206,991336113,2614888103,633803317,3248222580,3479774868,3835390401,2666613458,4022224774,944711139,264347078,2341262773,604807628,2007800933,770255983,1495990901,1249150122,1856431235,1555081692,3175218132,1996064986,2198950837,2554220882,3999719339,2821834349,766784016,2952996808,2566594879,3210313671,3203337956,3336571891,1034457026,3584528711,2466948901,113926993,3758326383,338241895,168717936,666307205,1188179964,773529912,1546045734,1294757372,1522805485,1396182291,2643833823,1695183700,2343527390,1986661051,1014477480,2177026350,1206759142,2456956037,344077627,2730485921,1290863460,2820302411,3158454273,3259730800,3505952657,3345764771,106217008,3516065817,3606008344,3600352804,1432725776,4094571909,1467031594,275423344,851169720,430227734,3100823752,506948616,1363258195,659060556,3750685593,883997877,3785050280,958139571,3318307427,1322822218,3812723403,1537002063,2003034995,1747873779,3602036899,1955562222,1575990012,2024104815,1125592928,2227730452,2716904306,2361852424,442776044,2428436474,593698344,2756734187,3733110249,3204031479,2999351573,3329325298,3815920427,3391569614,3928383900,3515267271,566280711,3940187606,3454069534,4118630271,4000239992,116418474,1914138554,174292421,2731055270,289380356,3203993006,460393269,320620315,685471733,587496836,852142971,1086792851,1017036298,365543100,1126000580,2618297676,1288033470,3409855158,1501505948,4234509866,1607167915,987167468,1816402316,1246189591];function m(){if(!(this instanceof m))return new m;I.call(this),this.h=[1779033703,4089235720,3144134277,2227873595,1013904242,4271175723,2773480762,1595750129,1359893119,2917565137,2600822924,725511199,528734635,4215389547,1541459225,327033209],this.k=C,this.W=new Array(160)}function y(t,e,n,r,i){var o=t&n^~t&i;return o<0&&(o+=4294967296),o}function E(t,e,n,r,i,o){var a=e&r^~e&o;return a<0&&(a+=4294967296),a}function v(t,e,n,r,i){var o=t&n^t&i^n&i;return o<0&&(o+=4294967296),o}function B(t,e,n,r,i,o){var a=e&r^e&o^r&o;return a<0&&(a+=4294967296),a}function b(t,e){var n=a(t,e,28)^a(e,t,2)^a(e,t,7);return n<0&&(n+=4294967296),n}function Q(t,e){var n=s(t,e,28)^s(e,t,2)^s(e,t,7);return n<0&&(n+=4294967296),n}function w(t,e){var n=a(t,e,14)^a(t,e,18)^a(e,t,9);return n<0&&(n+=4294967296),n}function x(t,e){var n=s(t,e,14)^s(t,e,18)^s(e,t,9);return n<0&&(n+=4294967296),n}function _(t,e){var n=a(t,e,1)^a(t,e,8)^A(t,e,7);return n<0&&(n+=4294967296),n}function D(t,e){var n=s(t,e,1)^s(t,e,8)^l(t,e,7);return n<0&&(n+=4294967296),n}function M(t,e){var n=a(t,e,19)^a(e,t,29)^A(t,e,6);return n<0&&(n+=4294967296),n}function k(t,e){var n=s(t,e,19)^s(e,t,29)^l(t,e,6);return n<0&&(n+=4294967296),n}r.inherits(m,I),t.exports=m,m.blockSize=1024,m.outSize=512,m.hmacStrength=192,m.padLength=128,m.prototype._prepareBlock=function(t,e){for(var n=this.W,r=0;r<32;r++)n[r]=t[e+r];for(;r<n.length;r+=2){var i=M(n[r-4],n[r-3]),o=k(n[r-4],n[r-3]),a=n[r-14],s=n[r-13],A=_(n[r-30],n[r-29]),l=D(n[r-30],n[r-29]),c=n[r-32],u=n[r-31];n[r]=h(i,o,a,s,A,l,c,u),n[r+1]=d(i,o,a,s,A,l,c,u)}},m.prototype._update=function(t,e){this._prepareBlock(t,e);var n=this.W,r=this.h[0],i=this.h[1],a=this.h[2],s=this.h[3],A=this.h[4],l=this.h[5],h=this.h[6],d=this.h[7],I=this.h[8],C=this.h[9],m=this.h[10],_=this.h[11],D=this.h[12],M=this.h[13],k=this.h[14],S=this.h[15];o(this.k.length===n.length);for(var L=0;L<n.length;L+=2){var R=k,N=S,F=w(I,C),T=x(I,C),U=y(I,C,m,_,D),O=E(I,C,m,_,D,M),G=this.k[L],H=this.k[L+1],j=n[L],Y=n[L+1],P=f(R,N,F,T,U,O,G,H,j,Y),K=p(R,N,F,T,U,O,G,H,j,Y);R=b(r,i),N=Q(r,i),F=v(r,i,a,s,A),T=B(r,i,a,s,A,l);var z=u(R,N,F,T),J=g(R,N,F,T);k=D,S=M,D=m,M=_,m=I,_=C,I=u(h,d,P,K),C=g(d,d,P,K),h=A,d=l,A=a,l=s,a=r,s=i,r=u(P,K,z,J),i=g(P,K,z,J)}c(this.h,0,r,i),c(this.h,2,a,s),c(this.h,4,A,l),c(this.h,6,h,d),c(this.h,8,I,C),c(this.h,10,m,_),c(this.h,12,D,M),c(this.h,14,k,S)},m.prototype._digest=function(t){return"hex"===t?r.toHex32(this.h,"big"):r.split32(this.h,"big")}},function(t,e,n){var r=n(30),i=n(170).Reporter,o=n(42).Buffer;function a(t,e){i.call(this,e),o.isBuffer(t)?(this.base=t,this.offset=0,this.length=t.length):this.error("Input not Buffer")}function s(t,e){if(Array.isArray(t))this.length=0,this.value=t.map(function(t){return t instanceof s||(t=new s(t,e)),this.length+=t.length,t},this);else if("number"==typeof t){if(!(0<=t&&t<=255))return e.error("non-byte EncoderBuffer value");this.value=t,this.length=1}else if("string"==typeof t)this.value=t,this.length=o.byteLength(t);else{if(!o.isBuffer(t))return e.error("Unsupported type: "+typeof t);this.value=t,this.length=t.length}}r(a,i),e.DecoderBuffer=a,a.prototype.save=function(){return{offset:this.offset,reporter:i.prototype.save.call(this)}},a.prototype.restore=function(t){var e=new a(this.base);return e.offset=t.offset,e.length=this.offset,this.offset=t.offset,i.prototype.restore.call(this,t.reporter),e},a.prototype.isEmpty=function(){return this.offset===this.length},a.prototype.readUInt8=function(t){return this.offset+1<=this.length?this.base.readUInt8(this.offset++,!0):this.error(t||"DecoderBuffer overrun")},a.prototype.skip=function(t,e){if(!(this.offset+t<=this.length))return this.error(e||"DecoderBuffer overrun");var n=new a(this.base);return n._reporterState=this._reporterState,n.offset=this.offset,n.length=this.offset+t,this.offset+=t,n},a.prototype.raw=function(t){return this.base.slice(t?t.offset:this.offset,this.length)},e.EncoderBuffer=s,s.prototype.join=function(t,e){return t||(t=new o(this.length)),e||(e=0),0===this.length?t:(Array.isArray(this.value)?this.value.forEach(function(n){n.join(t,e),e+=n.length}):("number"==typeof this.value?t[e]=this.value:"string"==typeof this.value?t.write(this.value,e):o.isBuffer(this.value)&&this.value.copy(t,e),e+=this.length),t)}},function(t,e,n){var r=e;r._reverse=function(t){var e={};return Object.keys(t).forEach(function(n){(0|n)==n&&(n|=0);var r=t[n];e[r]=n}),e},r.der=n(836)},function(t,e,n){var r=n(30),i=n(169),o=i.base,a=i.bignum,s=i.constants.der;function A(t){this.enc="der",this.name=t.name,this.entity=t,this.tree=new l,this.tree._init(t.body)}function l(t){o.Node.call(this,"der",t)}function c(t,e){var n=t.readUInt8(e);if(t.isError(n))return n;var r=s.tagClass[n>>6],i=0==(32&n);if(31==(31&n)){var o=n;for(n=0;128==(128&o);){if(o=t.readUInt8(e),t.isError(o))return o;n<<=7,n|=127&o}}else n&=31;return{cls:r,primitive:i,tag:n,tagStr:s.tag[n]}}function u(t,e,n){var r=t.readUInt8(n);if(t.isError(r))return r;if(!e&&128===r)return null;if(0==(128&r))return r;var i=127&r;if(i>4)return t.error("length octect is too long");r=0;for(var o=0;o<i;o++){r<<=8;var a=t.readUInt8(n);if(t.isError(a))return a;r|=a}return r}t.exports=A,A.prototype.decode=function(t,e){return t instanceof o.DecoderBuffer||(t=new o.DecoderBuffer(t,e)),this.tree._decode(t,e)},r(l,o.Node),l.prototype._peekTag=function(t,e,n){if(t.isEmpty())return!1;var r=t.save(),i=c(t,'Failed to peek tag: "'+e+'"');return t.isError(i)?i:(t.restore(r),i.tag===e||i.tagStr===e||i.tagStr+"of"===e||n)},l.prototype._decodeTag=function(t,e,n){var r=c(t,'Failed to decode tag of "'+e+'"');if(t.isError(r))return r;var i=u(t,r.primitive,'Failed to get length of "'+e+'"');if(t.isError(i))return i;if(!n&&r.tag!==e&&r.tagStr!==e&&r.tagStr+"of"!==e)return t.error('Failed to match tag: "'+e+'"');if(r.primitive||null!==i)return t.skip(i,'Failed to match body of: "'+e+'"');var o=t.save(),a=this._skipUntilEnd(t,'Failed to skip indefinite length body: "'+this.tag+'"');return t.isError(a)?a:(i=t.offset-o.offset,t.restore(o),t.skip(i,'Failed to match body of: "'+e+'"'))},l.prototype._skipUntilEnd=function(t,e){for(;;){var n=c(t,e);if(t.isError(n))return n;var r,i=u(t,n.primitive,e);if(t.isError(i))return i;if(r=n.primitive||null!==i?t.skip(i):this._skipUntilEnd(t,e),t.isError(r))return r;if("end"===n.tagStr)break}},l.prototype._decodeList=function(t,e,n,r){for(var i=[];!t.isEmpty();){var o=this._peekTag(t,"end");if(t.isError(o))return o;var a=n.decode(t,"der",r);if(t.isError(a)&&o)break;i.push(a)}return i},l.prototype._decodeStr=function(t,e){if("bitstr"===e){var n=t.readUInt8();return t.isError(n)?n:{unused:n,data:t.raw()}}if("bmpstr"===e){var r=t.raw();if(r.length%2==1)return t.error("Decoding of string type: bmpstr length mismatch");for(var i="",o=0;o<r.length/2;o++)i+=String.fromCharCode(r.readUInt16BE(2*o));return i}if("numstr"===e){var a=t.raw().toString("ascii");return this._isNumstr(a)?a:t.error("Decoding of string type: numstr unsupported characters")}if("octstr"===e)return t.raw();if("objDesc"===e)return t.raw();if("printstr"===e){var s=t.raw().toString("ascii");return this._isPrintstr(s)?s:t.error("Decoding of string type: printstr unsupported characters")}return/str$/.test(e)?t.raw().toString():t.error("Decoding of string type: "+e+" unsupported")},l.prototype._decodeObjid=function(t,e,n){for(var r,i=[],o=0;!t.isEmpty();){var a=t.readUInt8();o<<=7,o|=127&a,0==(128&a)&&(i.push(o),o=0)}128&a&&i.push(o);var s=i[0]/40|0,A=i[0]%40;if(r=n?i:[s,A].concat(i.slice(1)),e){var l=e[r.join(" ")];void 0===l&&(l=e[r.join(".")]),void 0!==l&&(r=l)}return r},l.prototype._decodeTime=function(t,e){var n=t.raw().toString();if("gentime"===e)var r=0|n.slice(0,4),i=0|n.slice(4,6),o=0|n.slice(6,8),a=0|n.slice(8,10),s=0|n.slice(10,12),A=0|n.slice(12,14);else{if("utctime"!==e)return t.error("Decoding "+e+" time is not supported yet");r=0|n.slice(0,2),i=0|n.slice(2,4),o=0|n.slice(4,6),a=0|n.slice(6,8),s=0|n.slice(8,10),A=0|n.slice(10,12);r=r<70?2e3+r:1900+r}return Date.UTC(r,i-1,o,a,s,A,0)},l.prototype._decodeNull=function(t){return null},l.prototype._decodeBool=function(t){var e=t.readUInt8();return t.isError(e)?e:0!==e},l.prototype._decodeInt=function(t,e){var n=t.raw(),r=new a(n);return e&&(r=e[r.toString(10)]||r),r},l.prototype._use=function(t,e){return"function"==typeof t&&(t=t(e)),t._getDecoder("der").tree}},function(t,e,n){var r=n(30),i=n(42).Buffer,o=n(169),a=o.base,s=o.constants.der;function A(t){this.enc="der",this.name=t.name,this.entity=t,this.tree=new l,this.tree._init(t.body)}function l(t){a.Node.call(this,"der",t)}function c(t){return t<10?"0"+t:t}t.exports=A,A.prototype.encode=function(t,e){return this.tree._encode(t,e).join()},r(l,a.Node),l.prototype._encodeComposite=function(t,e,n,r){var o,a=function(t,e,n,r){var i;"seqof"===t?t="seq":"setof"===t&&(t="set");if(s.tagByName.hasOwnProperty(t))i=s.tagByName[t];else{if("number"!=typeof t||(0|t)!==t)return r.error("Unknown tag: "+t);i=t}if(i>=31)return r.error("Multi-octet tag encoding unsupported");e||(i|=32);return i|=s.tagClassByName[n||"universal"]<<6}(t,e,n,this.reporter);if(r.length<128)return(o=new i(2))[0]=a,o[1]=r.length,this._createEncoderBuffer([o,r]);for(var A=1,l=r.length;l>=256;l>>=8)A++;(o=new i(2+A))[0]=a,o[1]=128|A;l=1+A;for(var c=r.length;c>0;l--,c>>=8)o[l]=255&c;return this._createEncoderBuffer([o,r])},l.prototype._encodeStr=function(t,e){if("bitstr"===e)return this._createEncoderBuffer([0|t.unused,t.data]);if("bmpstr"===e){for(var n=new i(2*t.length),r=0;r<t.length;r++)n.writeUInt16BE(t.charCodeAt(r),2*r);return this._createEncoderBuffer(n)}return"numstr"===e?this._isNumstr(t)?this._createEncoderBuffer(t):this.reporter.error("Encoding of string type: numstr supports only digits and space"):"printstr"===e?this._isPrintstr(t)?this._createEncoderBuffer(t):this.reporter.error("Encoding of string type: printstr supports only latin upper and lower case letters, digits, space, apostrophe, left and rigth parenthesis, plus sign, comma, hyphen, dot, slash, colon, equal sign, question mark"):/str$/.test(e)?this._createEncoderBuffer(t):"objDesc"===e?this._createEncoderBuffer(t):this.reporter.error("Encoding of string type: "+e+" unsupported")},l.prototype._encodeObjid=function(t,e,n){if("string"==typeof t){if(!e)return this.reporter.error("string objid given, but no values map found");if(!e.hasOwnProperty(t))return this.reporter.error("objid not found in values map");t=e[t].split(/[\s\.]+/g);for(var r=0;r<t.length;r++)t[r]|=0}else if(Array.isArray(t)){t=t.slice();for(r=0;r<t.length;r++)t[r]|=0}if(!Array.isArray(t))return this.reporter.error("objid() should be either array or string, got: "+JSON.stringify(t));if(!n){if(t[1]>=40)return this.reporter.error("Second objid identifier OOB");t.splice(0,2,40*t[0]+t[1])}var o=0;for(r=0;r<t.length;r++){var a=t[r];for(o++;a>=128;a>>=7)o++}var s=new i(o),A=s.length-1;for(r=t.length-1;r>=0;r--){a=t[r];for(s[A--]=127&a;(a>>=7)>0;)s[A--]=128|127&a}return this._createEncoderBuffer(s)},l.prototype._encodeTime=function(t,e){var n,r=new Date(t);return"gentime"===e?n=[c(r.getFullYear()),c(r.getUTCMonth()+1),c(r.getUTCDate()),c(r.getUTCHours()),c(r.getUTCMinutes()),c(r.getUTCSeconds()),"Z"].join(""):"utctime"===e?n=[c(r.getFullYear()%100),c(r.getUTCMonth()+1),c(r.getUTCDate()),c(r.getUTCHours()),c(r.getUTCMinutes()),c(r.getUTCSeconds()),"Z"].join(""):this.reporter.error("Encoding "+e+" time is not supported yet"),this._encodeStr(n,"octstr")},l.prototype._encodeNull=function(){return this._createEncoderBuffer("")},l.prototype._encodeInt=function(t,e){if("string"==typeof t){if(!e)return this.reporter.error("String int or enum given, but no values map");if(!e.hasOwnProperty(t))return this.reporter.error("Values map doesn't contain: "+JSON.stringify(t));t=e[t]}if("number"!=typeof t&&!i.isBuffer(t)){var n=t.toArray();!t.sign&&128&n[0]&&n.unshift(0),t=new i(n)}if(i.isBuffer(t)){var r=t.length;0===t.length&&r++;var o=new i(r);return t.copy(o),0===t.length&&(o[0]=0),this._createEncoderBuffer(o)}if(t<128)return this._createEncoderBuffer(t);if(t<256)return this._createEncoderBuffer([0,t]);r=1;for(var a=t;a>=256;a>>=8)r++;for(a=(o=new Array(r)).length-1;a>=0;a--)o[a]=255&t,t>>=8;return 128&o[0]&&o.unshift(0),this._createEncoderBuffer(new i(o))},l.prototype._encodeBool=function(t){return this._createEncoderBuffer(t?255:0)},l.prototype._use=function(t,e){return"function"==typeof t&&(t=t(e)),t._getEncoder("der").tree},l.prototype._skipDefault=function(t,e,n){var r,i=this._baseState;if(null===i.default)return!1;var o=t.join();if(void 0===i.defaultBuffer&&(i.defaultBuffer=this._encodeValue(i.default,e,n).join()),o.length!==i.defaultBuffer.length)return!1;for(r=0;r<o.length;r++)if(o[r]!==i.defaultBuffer[r])return!1;return!0}},function(t){t.exports=JSON.parse('{"1.3.132.0.10":"secp256k1","1.3.132.0.33":"p224","1.2.840.10045.3.1.1":"p192","1.2.840.10045.3.1.7":"p256","1.3.132.0.34":"p384","1.3.132.0.35":"p521"}')},function(t,e,n){var r=n(165),i=n(35).Buffer;function o(t){var e=i.allocUnsafe(4);return e.writeUInt32BE(t,0),e}t.exports=function(t,e){for(var n,a=i.alloc(0),s=0;a.length<e;)n=o(s++),a=i.concat([a,r("sha1").update(t).update(n).digest()]);return a.slice(0,e)}},function(t,e){t.exports=function(t,e){for(var n=t.length,r=-1;++r<n;)t[r]^=e[r];return t}},function(t,e,n){var r=n(53),i=n(35).Buffer;t.exports=function(t,e){return i.from(t.toRed(r.mont(e.modulus)).redPow(new r(e.publicExponent)).fromRed().toArray())}},function(t,e,n){"use strict";var r=this&&this.__importDefault||function(t){return t&&t.__esModule?t:{default:t}};Object.defineProperty(e,"__esModule",{value:!0});const i=r(n(854));!function(t){function e(t){if("undefined"!=typeof document&&document){let e=document.createElement("a");return e.href=t,e}return i.default(t)}function n(...t){const e=(t=t||[])[0]||"",n=0===e.indexOf("//"),r=e.match(/(\w+)(:)(\/\/)?/),i=r&&r[1],o=i&&r[2],a=o&&r[3],s=n?"//":[i,o,a].filter(t=>t).join("");return s+[0===e.indexOf(s)?e.replace(s,""):e].filter(t=>t).concat(t.slice(1).map(t=>t.replace(/^\//,""))).join("/").replace(/\/+/g,"/")}t.parse=e,t.normalize=function(t){return t&&e(t).toString()},t.join=n,t.encodeParts=function(t){return n(...t.split("/").map(encodeURIComponent))},t.objectToQueryString=function(t){const e=Object.keys(t).filter(t=>t.length>0);return e.length?"?"+e.map(e=>{const n=encodeURIComponent(String(t[e]));return e+(n?"="+n:"")}).join("&"):""},t.queryStringToObject=function(t){return t.replace(/^\?/,"").split("&").reduce((t,e)=>{const[n,r]=e.split("=");return n.length>0&&(t[n]=decodeURIComponent(r||"")),t},{})},t.isLocal=function(t){const{protocol:n}=e(t);return 0!==t.toLowerCase().indexOf(n)&&0!==t.indexOf("/")}}(e.URLExt||(e.URLExt={}))},function(t,e,n){"use strict";(function(t,r){Object.defineProperty(e,"__esModule",{value:!0});const i=n(9),o=n(10),a="function"==typeof requestAnimationFrame?requestAnimationFrame:t,s="function"==typeof cancelAnimationFrame?cancelAnimationFrame:r;class A{constructor(t){this._disposed=new o.Signal(this),this._tick=new i.PromiseDelegate,this._ticked=new o.Signal(this),this._timeout=-1,this._factory=t.factory,this._standby=t.standby||l.DEFAULT_STANDBY,this._state=Object.assign({},l.DEFAULT_STATE,{timestamp:(new Date).getTime()}),this.frequency=Object.assign({},l.DEFAULT_FREQUENCY,t.frequency||{}),this.name=t.name||l.DEFAULT_NAME,"auto"in t&&!t.auto||a(()=>void this.start())}get disposed(){return this._disposed}get frequency(){return this._frequency}set frequency(t){if(this.isDisposed||i.JSONExt.deepEqual(t,this.frequency||{}))return;let{backoff:e,interval:n,max:r}=t;if(n=Math.round(n),r=Math.round(r),"number"==typeof e&&e<1)throw new Error("Poll backoff growth factor must be at least 1");if((n<0||n>r)&&n!==A.NEVER)throw new Error("Poll interval must be between 0 and max");if(r>A.MAX_INTERVAL&&r!==A.NEVER)throw new Error(`Max interval must be less than ${A.MAX_INTERVAL}`);this._frequency={backoff:e,interval:n,max:r}}get isDisposed(){return"disposed"===this.state.phase}get standby(){return this._standby}set standby(t){this.isDisposed||this.standby===t||(this._standby=t)}get state(){return this._state}get tick(){return this._tick.promise}get ticked(){return this._ticked}dispose(){this.isDisposed||(this._state=Object.assign({},l.DISPOSED_STATE,{timestamp:(new Date).getTime()}),this._tick.promise.catch(t=>void 0),this._tick.reject(new Error(`Poll (${this.name}) is disposed.`)),this._disposed.emit(),o.Signal.clearData(this))}refresh(){return this.schedule({cancel:({phase:t})=>"refreshed"===t,interval:A.IMMEDIATE,phase:"refreshed"})}async schedule(t={}){if(this.isDisposed)return;if(t.cancel&&t.cancel(this.state))return;const e=this.state,n=this._tick,r=new i.PromiseDelegate,o=Object.assign({interval:this.frequency.interval,payload:null,phase:"standby",timestamp:(new Date).getTime()},t);this._state=o,this._tick=r,e.interval===A.IMMEDIATE?s(this._timeout):clearTimeout(this._timeout),this._ticked.emit(this.state),n.resolve(this),await n.promise;const l=()=>{this.isDisposed||this.tick!==r.promise||this._execute()};this._timeout=o.interval===A.IMMEDIATE?a(l):o.interval===A.NEVER?-1:setTimeout(l,o.interval)}start(){return this.schedule({cancel:({phase:t})=>"constructed"!==t&&"standby"!==t&&"stopped"!==t,interval:A.IMMEDIATE,phase:"started"})}stop(){return this.schedule({cancel:({phase:t})=>"stopped"===t,interval:A.NEVER,phase:"stopped"})}_execute(){let t="function"==typeof this.standby?this.standby():this.standby;if(t="never"!==t&&("when-hidden"!==t||!("undefined"==typeof document||!document||!document.hidden)))return void this.schedule();const e=this.tick;this._factory(this.state).then(t=>{this.isDisposed||this.tick!==e||this.schedule({payload:t,phase:"rejected"===this.state.phase?"reconnected":"resolved"})}).catch(t=>{this.isDisposed||this.tick!==e||this.schedule({interval:l.sleep(this.frequency,this.state),payload:t,phase:"rejected"})})}}var l;e.Poll=A,function(t){t.IMMEDIATE=0,t.MAX_INTERVAL=2147483647,t.NEVER=1/0}(A=e.Poll||(e.Poll={})),function(t){t.DEFAULT_BACKOFF=3,t.DEFAULT_FREQUENCY={backoff:!0,interval:1e3,max:3e4},t.DEFAULT_NAME="unknown",t.DEFAULT_STANDBY="when-hidden",t.DEFAULT_STATE={interval:A.NEVER,payload:null,phase:"constructed",timestamp:new Date(0).getTime()},t.DISPOSED_STATE={interval:A.NEVER,payload:null,phase:"disposed",timestamp:new Date(0).getTime()},t.sleep=function(e,n){const{backoff:r,interval:i,max:o}=e;if(i===A.NEVER)return i;const a=!0===r?t.DEFAULT_BACKOFF:!1===r?1:r,s=function(t,e){return t=Math.ceil(t),e=Math.floor(e),Math.floor(Math.random()*(e-t+1))+t}(i,n.interval*a);return Math.min(o,s)}}(l||(l={}))}).call(this,n(118).setImmediate,n(118).clearImmediate)},function(t,e,n){"use strict";var r=n(137);t.exports=function(t){r.copy(t,this)}},function(t,e,n){"use strict";t.exports=function(t,e){e||(e={}),"function"==typeof e&&(e={cmp:e});var n,r="boolean"==typeof e.cycles&&e.cycles,i=e.cmp&&(n=e.cmp,function(t){return function(e,r){var i={key:e,value:t[e]},o={key:r,value:t[r]};return n(i,o)}}),o=[];return function t(e){if(e&&e.toJSON&&"function"==typeof e.toJSON&&(e=e.toJSON()),void 0!==e){if("number"==typeof e)return isFinite(e)?""+e:"null";if("object"!=typeof e)return JSON.stringify(e);var n,a;if(Array.isArray(e)){for(a="[",n=0;n<e.length;n++)n&&(a+=","),a+=t(e[n])||"null";return a+"]"}if(null===e)return"null";if(-1!==o.indexOf(e)){if(r)return JSON.stringify("__cycle__");throw new TypeError("Converting circular structure to JSON")}var s=o.push(e)-1,A=Object.keys(e).sort(i&&i(e));for(a="",n=0;n<A.length;n++){var l=A[n],c=t(e[l]);c&&(a&&(a+=","),a+=JSON.stringify(l)+":"+c)}return o.splice(s,1),"{"+a+"}"}}(t)}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r="",i=!0===t.schema.$async,o=t.util.schemaHasRulesExcept(t.schema,t.RULES.all,"$ref"),a=t.self._getId(t.schema);if(t.opts.strictKeywords){var s=t.util.schemaUnknownRules(t.schema,t.RULES.keywords);if(s){var A="unknown keyword: "+s;if("log"!==t.opts.strictKeywords)throw new Error(A);t.logger.warn(A)}}if(t.isTop&&(r+=" var validate = ",i&&(t.async=!0,r+="async "),r+="function(data, dataPath, parentData, parentDataProperty, rootData) { 'use strict'; ",a&&(t.opts.sourceCode||t.opts.processCode)&&(r+=" /*# sourceURL="+a+" */ ")),"boolean"==typeof t.schema||!o&&!t.schema.$ref){var l=t.level,c=t.dataLevel,u=t.schema["false schema"],g=t.schemaPath+t.util.getProperty("false schema"),h=t.errSchemaPath+"/false schema",d=!t.opts.allErrors,f="data"+(c||""),p="valid"+l;if(!1===t.schema){t.isTop?d=!0:r+=" var "+p+" = false; ",(X=X||[]).push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'false schema' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(h)+" , params: {} ",!1!==t.opts.messages&&(r+=" , message: 'boolean schema is false' "),t.opts.verbose&&(r+=" , schema: false , parentSchema: validate.schema"+t.schemaPath+" , data: "+f+" "),r+=" } "):r+=" {} ";var I=r;r=X.pop(),!t.compositeRule&&d?t.async?r+=" throw new ValidationError(["+I+"]); ":r+=" validate.errors = ["+I+"]; return false; ":r+=" var err = "+I+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; "}else t.isTop?r+=i?" return data; ":" validate.errors = null; return true; ":r+=" var "+p+" = true; ";return t.isTop&&(r+=" }; return validate; "),r}if(t.isTop){var C=t.isTop;l=t.level=0,c=t.dataLevel=0,f="data";if(t.rootId=t.resolve.fullPath(t.self._getId(t.root.schema)),t.baseId=t.baseId||t.rootId,delete t.isTop,t.dataPathArr=[void 0],void 0!==t.schema.default&&t.opts.useDefaults&&t.opts.strictDefaults){var m="default is ignored in the schema root";if("log"!==t.opts.strictDefaults)throw new Error(m);t.logger.warn(m)}r+=" var vErrors = null; ",r+=" var errors = 0;     ",r+=" if (rootData === undefined) rootData = data; "}else{l=t.level,f="data"+((c=t.dataLevel)||"");if(a&&(t.baseId=t.resolve.url(t.baseId,a)),i&&!t.async)throw new Error("async schema in sync schema");r+=" var errs_"+l+" = errors;"}p="valid"+l,d=!t.opts.allErrors;var y="",E="",v=t.schema.type,B=Array.isArray(v);if(v&&t.opts.nullable&&!0===t.schema.nullable&&(B?-1==v.indexOf("null")&&(v=v.concat("null")):"null"!=v&&(v=[v,"null"],B=!0)),B&&1==v.length&&(v=v[0],B=!1),t.schema.$ref&&o){if("fail"==t.opts.extendRefs)throw new Error('$ref: validation keywords used in schema at path "'+t.errSchemaPath+'" (see option extendRefs)');!0!==t.opts.extendRefs&&(o=!1,t.logger.warn('$ref: keywords ignored in schema at path "'+t.errSchemaPath+'"'))}if(t.schema.$comment&&t.opts.$comment&&(r+=" "+t.RULES.all.$comment.code(t,"$comment")),v){if(t.opts.coerceTypes)var b=t.util.coerceToTypes(t.opts.coerceTypes,v);var Q=t.RULES.types[v];if(b||B||!0===Q||Q&&!Z(Q)){g=t.schemaPath+".type",h=t.errSchemaPath+"/type",g=t.schemaPath+".type",h=t.errSchemaPath+"/type";var w=B?"checkDataTypes":"checkDataType";if(r+=" if ("+t.util[w](v,f,!0)+") { ",b){var x="dataType"+l,_="coerced"+l;r+=" var "+x+" = typeof "+f+"; ","array"==t.opts.coerceTypes&&(r+=" if ("+x+" == 'object' && Array.isArray("+f+")) "+x+" = 'array'; "),r+=" var "+_+" = undefined; ";var D="",M=b;if(M)for(var k,S=-1,L=M.length-1;S<L;)k=M[S+=1],S&&(r+=" if ("+_+" === undefined) { ",D+="}"),"array"==t.opts.coerceTypes&&"array"!=k&&(r+=" if ("+x+" == 'array' && "+f+".length == 1) { "+_+" = "+f+" = "+f+"[0]; "+x+" = typeof "+f+";  } "),"string"==k?r+=" if ("+x+" == 'number' || "+x+" == 'boolean') "+_+" = '' + "+f+"; else if ("+f+" === null) "+_+" = ''; ":"number"==k||"integer"==k?(r+=" if ("+x+" == 'boolean' || "+f+" === null || ("+x+" == 'string' && "+f+" && "+f+" == +"+f+" ","integer"==k&&(r+=" && !("+f+" % 1)"),r+=")) "+_+" = +"+f+"; "):"boolean"==k?r+=" if ("+f+" === 'false' || "+f+" === 0 || "+f+" === null) "+_+" = false; else if ("+f+" === 'true' || "+f+" === 1) "+_+" = true; ":"null"==k?r+=" if ("+f+" === '' || "+f+" === 0 || "+f+" === false) "+_+" = null; ":"array"==t.opts.coerceTypes&&"array"==k&&(r+=" if ("+x+" == 'string' || "+x+" == 'number' || "+x+" == 'boolean' || "+f+" == null) "+_+" = ["+f+"]; ");r+=" "+D+" if ("+_+" === undefined) {   ",(X=X||[]).push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'type' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(h)+" , params: { type: '",r+=B?""+v.join(","):""+v,r+="' } ",!1!==t.opts.messages&&(r+=" , message: 'should be ",r+=B?""+v.join(","):""+v,r+="' "),t.opts.verbose&&(r+=" , schema: validate.schema"+g+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+f+" "),r+=" } "):r+=" {} ";I=r;r=X.pop(),!t.compositeRule&&d?t.async?r+=" throw new ValidationError(["+I+"]); ":r+=" validate.errors = ["+I+"]; return false; ":r+=" var err = "+I+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } else {  ";var R=c?"data"+(c-1||""):"parentData";r+=" "+f+" = "+_+"; ",c||(r+="if ("+R+" !== undefined)"),r+=" "+R+"["+(c?t.dataPathArr[c]:"parentDataProperty")+"] = "+_+"; } "}else{(X=X||[]).push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'type' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(h)+" , params: { type: '",r+=B?""+v.join(","):""+v,r+="' } ",!1!==t.opts.messages&&(r+=" , message: 'should be ",r+=B?""+v.join(","):""+v,r+="' "),t.opts.verbose&&(r+=" , schema: validate.schema"+g+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+f+" "),r+=" } "):r+=" {} ";I=r;r=X.pop(),!t.compositeRule&&d?t.async?r+=" throw new ValidationError(["+I+"]); ":r+=" validate.errors = ["+I+"]; return false; ":r+=" var err = "+I+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; "}r+=" } "}}if(t.schema.$ref&&!o)r+=" "+t.RULES.all.$ref.code(t,"$ref")+" ",d&&(r+=" } if (errors === ",r+=C?"0":"errs_"+l,r+=") { ",E+="}");else{var N=t.RULES;if(N)for(var F=-1,T=N.length-1;F<T;)if(Z(Q=N[F+=1])){if(Q.type&&(r+=" if ("+t.util.checkDataType(Q.type,f)+") { "),t.opts.useDefaults)if("object"==Q.type&&t.schema.properties){u=t.schema.properties;var U=Object.keys(u);if(U)for(var O,G=-1,H=U.length-1;G<H;){if(void 0!==(P=u[O=U[G+=1]]).default){var j=f+t.util.getProperty(O);if(t.compositeRule){if(t.opts.strictDefaults){m="default is ignored for: "+j;if("log"!==t.opts.strictDefaults)throw new Error(m);t.logger.warn(m)}}else r+=" if ("+j+" === undefined ","empty"==t.opts.useDefaults&&(r+=" || "+j+" === null || "+j+" === '' "),r+=" ) "+j+" = ","shared"==t.opts.useDefaults?r+=" "+t.useDefault(P.default)+" ":r+=" "+JSON.stringify(P.default)+" ",r+="; "}}}else if("array"==Q.type&&Array.isArray(t.schema.items)){var Y=t.schema.items;if(Y){S=-1;for(var P,K=Y.length-1;S<K;)if(void 0!==(P=Y[S+=1]).default){j=f+"["+S+"]";if(t.compositeRule){if(t.opts.strictDefaults){m="default is ignored for: "+j;if("log"!==t.opts.strictDefaults)throw new Error(m);t.logger.warn(m)}}else r+=" if ("+j+" === undefined ","empty"==t.opts.useDefaults&&(r+=" || "+j+" === null || "+j+" === '' "),r+=" ) "+j+" = ","shared"==t.opts.useDefaults?r+=" "+t.useDefault(P.default)+" ":r+=" "+JSON.stringify(P.default)+" ",r+="; "}}}var z=Q.rules;if(z)for(var J,q=-1,V=z.length-1;q<V;)if($(J=z[q+=1])){var W=J.code(t,J.keyword,Q.type);W&&(r+=" "+W+" ",d&&(y+="}"))}if(d&&(r+=" "+y+" ",y=""),Q.type&&(r+=" } ",v&&v===Q.type&&!b)){r+=" else { ";var X;g=t.schemaPath+".type",h=t.errSchemaPath+"/type";(X=X||[]).push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'type' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(h)+" , params: { type: '",r+=B?""+v.join(","):""+v,r+="' } ",!1!==t.opts.messages&&(r+=" , message: 'should be ",r+=B?""+v.join(","):""+v,r+="' "),t.opts.verbose&&(r+=" , schema: validate.schema"+g+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+f+" "),r+=" } "):r+=" {} ";I=r;r=X.pop(),!t.compositeRule&&d?t.async?r+=" throw new ValidationError(["+I+"]); ":r+=" validate.errors = ["+I+"]; return false; ":r+=" var err = "+I+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } "}d&&(r+=" if (errors === ",r+=C?"0":"errs_"+l,r+=") { ",E+="}")}}function Z(t){for(var e=t.rules,n=0;n<e.length;n++)if($(e[n]))return!0}function $(e){return void 0!==t.schema[e.keyword]||e.implements&&function(e){for(var n=e.implements,r=0;r<n.length;r++)if(void 0!==t.schema[n[r]])return!0}(e)}return d&&(r+=" "+E+" "),C?(i?(r+=" if (errors === 0) return data;           ",r+=" else throw new ValidationError(vErrors); "):(r+=" validate.errors = vErrors; ",r+=" return errors === 0;       "),r+=" }; return validate;"):r+=" var "+p+" = errors === errs_"+l+";",r=t.util.cleanUpCode(r),C&&(r=t.util.finalCleanUpCode(r,i)),r}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r,i=" ",o=t.level,a=t.dataLevel,s=t.schema[e],A=t.schemaPath+t.util.getProperty(e),l=t.errSchemaPath+"/"+e,c=!t.opts.allErrors,u="data"+(a||""),g=t.opts.$data&&s&&s.$data;g?(i+=" var schema"+o+" = "+t.util.getData(s.$data,a,t.dataPathArr)+"; ",r="schema"+o):r=s;var h="maximum"==e,d=h?"exclusiveMaximum":"exclusiveMinimum",f=t.schema[d],p=t.opts.$data&&f&&f.$data,I=h?"<":">",C=h?">":"<",m=void 0;if(p){var y=t.util.getData(f.$data,a,t.dataPathArr),E="exclusive"+o,v="exclType"+o,B="exclIsNumber"+o,b="' + "+(x="op"+o)+" + '";i+=" var schemaExcl"+o+" = "+y+"; ",i+=" var "+E+"; var "+v+" = typeof "+(y="schemaExcl"+o)+"; if ("+v+" != 'boolean' && "+v+" != 'undefined' && "+v+" != 'number') { ";var Q;m=d;(Q=Q||[]).push(i),i="",!1!==t.createErrors?(i+=" { keyword: '"+(m||"_exclusiveLimit")+"' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(l)+" , params: {} ",!1!==t.opts.messages&&(i+=" , message: '"+d+" should be boolean' "),t.opts.verbose&&(i+=" , schema: validate.schema"+A+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+u+" "),i+=" } "):i+=" {} ";var w=i;i=Q.pop(),!t.compositeRule&&c?t.async?i+=" throw new ValidationError(["+w+"]); ":i+=" validate.errors = ["+w+"]; return false; ":i+=" var err = "+w+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+=" } else if ( ",g&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'number') || "),i+=" "+v+" == 'number' ? ( ("+E+" = "+r+" === undefined || "+y+" "+I+"= "+r+") ? "+u+" "+C+"= "+y+" : "+u+" "+C+" "+r+" ) : ( ("+E+" = "+y+" === true) ? "+u+" "+C+"= "+r+" : "+u+" "+C+" "+r+" ) || "+u+" !== "+u+") { var op"+o+" = "+E+" ? '"+I+"' : '"+I+"='; ",void 0===s&&(m=d,l=t.errSchemaPath+"/"+d,r=y,g=p)}else{b=I;if((B="number"==typeof f)&&g){var x="'"+b+"'";i+=" if ( ",g&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'number') || "),i+=" ( "+r+" === undefined || "+f+" "+I+"= "+r+" ? "+u+" "+C+"= "+f+" : "+u+" "+C+" "+r+" ) || "+u+" !== "+u+") { "}else{B&&void 0===s?(E=!0,m=d,l=t.errSchemaPath+"/"+d,r=f,C+="="):(B&&(r=Math[h?"min":"max"](f,s)),f===(!B||r)?(E=!0,m=d,l=t.errSchemaPath+"/"+d,C+="="):(E=!1,b+="="));x="'"+b+"'";i+=" if ( ",g&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'number') || "),i+=" "+u+" "+C+" "+r+" || "+u+" !== "+u+") { "}}m=m||e,(Q=Q||[]).push(i),i="",!1!==t.createErrors?(i+=" { keyword: '"+(m||"_limit")+"' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(l)+" , params: { comparison: "+x+", limit: "+r+", exclusive: "+E+" } ",!1!==t.opts.messages&&(i+=" , message: 'should be "+b+" ",i+=g?"' + "+r:r+"'"),t.opts.verbose&&(i+=" , schema:  ",i+=g?"validate.schema"+A:""+s,i+="         , parentSchema: validate.schema"+t.schemaPath+" , data: "+u+" "),i+=" } "):i+=" {} ";w=i;return i=Q.pop(),!t.compositeRule&&c?t.async?i+=" throw new ValidationError(["+w+"]); ":i+=" validate.errors = ["+w+"]; return false; ":i+=" var err = "+w+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+=" } ",c&&(i+=" else { "),i}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r,i=" ",o=t.level,a=t.dataLevel,s=t.schema[e],A=t.schemaPath+t.util.getProperty(e),l=t.errSchemaPath+"/"+e,c=!t.opts.allErrors,u="data"+(a||""),g=t.opts.$data&&s&&s.$data;g?(i+=" var schema"+o+" = "+t.util.getData(s.$data,a,t.dataPathArr)+"; ",r="schema"+o):r=s,i+="if ( ",g&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'number') || "),i+=" "+u+".length "+("maxItems"==e?">":"<")+" "+r+") { ";var h=e,d=d||[];d.push(i),i="",!1!==t.createErrors?(i+=" { keyword: '"+(h||"_limitItems")+"' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(l)+" , params: { limit: "+r+" } ",!1!==t.opts.messages&&(i+=" , message: 'should NOT have ",i+="maxItems"==e?"more":"fewer",i+=" than ",i+=g?"' + "+r+" + '":""+s,i+=" items' "),t.opts.verbose&&(i+=" , schema:  ",i+=g?"validate.schema"+A:""+s,i+="         , parentSchema: validate.schema"+t.schemaPath+" , data: "+u+" "),i+=" } "):i+=" {} ";var f=i;return i=d.pop(),!t.compositeRule&&c?t.async?i+=" throw new ValidationError(["+f+"]); ":i+=" validate.errors = ["+f+"]; return false; ":i+=" var err = "+f+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+="} ",c&&(i+=" else { "),i}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r,i=" ",o=t.level,a=t.dataLevel,s=t.schema[e],A=t.schemaPath+t.util.getProperty(e),l=t.errSchemaPath+"/"+e,c=!t.opts.allErrors,u="data"+(a||""),g=t.opts.$data&&s&&s.$data;g?(i+=" var schema"+o+" = "+t.util.getData(s.$data,a,t.dataPathArr)+"; ",r="schema"+o):r=s;var h="maxLength"==e?">":"<";i+="if ( ",g&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'number') || "),!1===t.opts.unicode?i+=" "+u+".length ":i+=" ucs2length("+u+") ",i+=" "+h+" "+r+") { ";var d=e,f=f||[];f.push(i),i="",!1!==t.createErrors?(i+=" { keyword: '"+(d||"_limitLength")+"' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(l)+" , params: { limit: "+r+" } ",!1!==t.opts.messages&&(i+=" , message: 'should NOT be ",i+="maxLength"==e?"longer":"shorter",i+=" than ",i+=g?"' + "+r+" + '":""+s,i+=" characters' "),t.opts.verbose&&(i+=" , schema:  ",i+=g?"validate.schema"+A:""+s,i+="         , parentSchema: validate.schema"+t.schemaPath+" , data: "+u+" "),i+=" } "):i+=" {} ";var p=i;return i=f.pop(),!t.compositeRule&&c?t.async?i+=" throw new ValidationError(["+p+"]); ":i+=" validate.errors = ["+p+"]; return false; ":i+=" var err = "+p+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+="} ",c&&(i+=" else { "),i}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r,i=" ",o=t.level,a=t.dataLevel,s=t.schema[e],A=t.schemaPath+t.util.getProperty(e),l=t.errSchemaPath+"/"+e,c=!t.opts.allErrors,u="data"+(a||""),g=t.opts.$data&&s&&s.$data;g?(i+=" var schema"+o+" = "+t.util.getData(s.$data,a,t.dataPathArr)+"; ",r="schema"+o):r=s,i+="if ( ",g&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'number') || "),i+=" Object.keys("+u+").length "+("maxProperties"==e?">":"<")+" "+r+") { ";var h=e,d=d||[];d.push(i),i="",!1!==t.createErrors?(i+=" { keyword: '"+(h||"_limitProperties")+"' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(l)+" , params: { limit: "+r+" } ",!1!==t.opts.messages&&(i+=" , message: 'should NOT have ",i+="maxProperties"==e?"more":"fewer",i+=" than ",i+=g?"' + "+r+" + '":""+s,i+=" properties' "),t.opts.verbose&&(i+=" , schema:  ",i+=g?"validate.schema"+A:""+s,i+="         , parentSchema: validate.schema"+t.schemaPath+" , data: "+u+" "),i+=" } "):i+=" {} ";var f=i;return i=d.pop(),!t.compositeRule&&c?t.async?i+=" throw new ValidationError(["+f+"]); ":i+=" validate.errors = ["+f+"]; return false; ":i+=" var err = "+f+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+="} ",c&&(i+=" else { "),i}},function(t){t.exports=JSON.parse('{"$schema":"http://json-schema.org/draft-07/schema#","$id":"http://json-schema.org/draft-07/schema#","title":"Core schema meta-schema","definitions":{"schemaArray":{"type":"array","minItems":1,"items":{"$ref":"#"}},"nonNegativeInteger":{"type":"integer","minimum":0},"nonNegativeIntegerDefault0":{"allOf":[{"$ref":"#/definitions/nonNegativeInteger"},{"default":0}]},"simpleTypes":{"enum":["array","boolean","integer","null","number","object","string"]},"stringArray":{"type":"array","items":{"type":"string"},"uniqueItems":true,"default":[]}},"type":["object","boolean"],"properties":{"$id":{"type":"string","format":"uri-reference"},"$schema":{"type":"string","format":"uri"},"$ref":{"type":"string","format":"uri-reference"},"$comment":{"type":"string"},"title":{"type":"string"},"description":{"type":"string"},"default":true,"readOnly":{"type":"boolean","default":false},"examples":{"type":"array","items":true},"multipleOf":{"type":"number","exclusiveMinimum":0},"maximum":{"type":"number"},"exclusiveMaximum":{"type":"number"},"minimum":{"type":"number"},"exclusiveMinimum":{"type":"number"},"maxLength":{"$ref":"#/definitions/nonNegativeInteger"},"minLength":{"$ref":"#/definitions/nonNegativeIntegerDefault0"},"pattern":{"type":"string","format":"regex"},"additionalItems":{"$ref":"#"},"items":{"anyOf":[{"$ref":"#"},{"$ref":"#/definitions/schemaArray"}],"default":true},"maxItems":{"$ref":"#/definitions/nonNegativeInteger"},"minItems":{"$ref":"#/definitions/nonNegativeIntegerDefault0"},"uniqueItems":{"type":"boolean","default":false},"contains":{"$ref":"#"},"maxProperties":{"$ref":"#/definitions/nonNegativeInteger"},"minProperties":{"$ref":"#/definitions/nonNegativeIntegerDefault0"},"required":{"$ref":"#/definitions/stringArray"},"additionalProperties":{"$ref":"#"},"definitions":{"type":"object","additionalProperties":{"$ref":"#"},"default":{}},"properties":{"type":"object","additionalProperties":{"$ref":"#"},"default":{}},"patternProperties":{"type":"object","additionalProperties":{"$ref":"#"},"propertyNames":{"format":"regex"},"default":{}},"dependencies":{"type":"object","additionalProperties":{"anyOf":[{"$ref":"#"},{"$ref":"#/definitions/stringArray"}]}},"propertyNames":{"$ref":"#"},"const":true,"enum":{"type":"array","items":true,"minItems":1,"uniqueItems":true},"type":{"anyOf":[{"$ref":"#/definitions/simpleTypes"},{"type":"array","items":{"$ref":"#/definitions/simpleTypes"},"minItems":1,"uniqueItems":true}]},"format":{"type":"string"},"contentMediaType":{"type":"string"},"contentEncoding":{"type":"string"},"if":{"$ref":"#"},"then":{"$ref":"#"},"else":{"$ref":"#"},"allOf":{"$ref":"#/definitions/schemaArray"},"anyOf":{"$ref":"#/definitions/schemaArray"},"oneOf":{"$ref":"#/definitions/schemaArray"},"not":{"$ref":"#"}},"default":true}')},function(t,e,n){!function(t){"use strict";t.defineLocale("af",{months:"Januarie_Februarie_Maart_April_Mei_Junie_Julie_Augustus_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mrt_Apr_Mei_Jun_Jul_Aug_Sep_Okt_Nov_Des".split("_"),weekdays:"Sondag_Maandag_Dinsdag_Woensdag_Donderdag_Vrydag_Saterdag".split("_"),weekdaysShort:"Son_Maa_Din_Woe_Don_Vry_Sat".split("_"),weekdaysMin:"So_Ma_Di_Wo_Do_Vr_Sa".split("_"),meridiemParse:/vm|nm/i,isPM:function(t){return/^nm$/i.test(t)},meridiem:function(t,e,n){return t<12?n?"vm":"VM":n?"nm":"NM"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Vandag om] LT",nextDay:"[Môre om] LT",nextWeek:"dddd [om] LT",lastDay:"[Gister om] LT",lastWeek:"[Laas] dddd [om] LT",sameElse:"L"},relativeTime:{future:"oor %s",past:"%s gelede",s:"'n paar sekondes",ss:"%d sekondes",m:"'n minuut",mm:"%d minute",h:"'n uur",hh:"%d ure",d:"'n dag",dd:"%d dae",M:"'n maand",MM:"%d maande",y:"'n jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(t){return t+(1===t||8===t||t>=20?"ste":"de")},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"},n={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"},r=function(t){return 0===t?0:1===t?1:2===t?2:t%100>=3&&t%100<=10?3:t%100>=11?4:5},i={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},o=function(t){return function(e,n,o,a){var s=r(e),A=i[t][r(e)];return 2===s&&(A=A[n?0:1]),A.replace(/%d/i,e)}},a=["يناير","فبراير","مارس","أبريل","مايو","يونيو","يوليو","أغسطس","سبتمبر","أكتوبر","نوفمبر","ديسمبر"];t.defineLocale("ar",{months:a,monthsShort:a,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(t){return"م"===t},meridiem:function(t,e,n){return t<12?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:o("s"),ss:o("s"),m:o("m"),mm:o("m"),h:o("h"),hh:o("h"),d:o("d"),dd:o("d"),M:o("M"),MM:o("M"),y:o("y"),yy:o("y")},preparse:function(t){return t.replace(/[١٢٣٤٥٦٧٨٩٠]/g,function(t){return n[t]}).replace(/،/g,",")},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]}).replace(/,/g,"،")},week:{dow:6,doy:12}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ar-dz",{months:"جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),monthsShort:"جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"احد_اثنين_ثلاثاء_اربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"أح_إث_ثلا_أر_خم_جم_سب".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:0,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ar-kw",{months:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),weekdays:"الأحد_الإتنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"احد_اتنين_ثلاثاء_اربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:0,doy:12}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"1",2:"2",3:"3",4:"4",5:"5",6:"6",7:"7",8:"8",9:"9",0:"0"},n=function(t){return 0===t?0:1===t?1:2===t?2:t%100>=3&&t%100<=10?3:t%100>=11?4:5},r={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},i=function(t){return function(e,i,o,a){var s=n(e),A=r[t][n(e)];return 2===s&&(A=A[i?0:1]),A.replace(/%d/i,e)}},o=["يناير","فبراير","مارس","أبريل","مايو","يونيو","يوليو","أغسطس","سبتمبر","أكتوبر","نوفمبر","ديسمبر"];t.defineLocale("ar-ly",{months:o,monthsShort:o,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(t){return"م"===t},meridiem:function(t,e,n){return t<12?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:i("s"),ss:i("s"),m:i("m"),mm:i("m"),h:i("h"),hh:i("h"),d:i("d"),dd:i("d"),M:i("M"),MM:i("M"),y:i("y"),yy:i("y")},preparse:function(t){return t.replace(/،/g,",")},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]}).replace(/,/g,"،")},week:{dow:6,doy:12}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ar-ma",{months:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),weekdays:"الأحد_الإتنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"احد_اتنين_ثلاثاء_اربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:6,doy:12}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"},n={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"};t.defineLocale("ar-sa",{months:"يناير_فبراير_مارس_أبريل_مايو_يونيو_يوليو_أغسطس_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_مايو_يونيو_يوليو_أغسطس_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(t){return"م"===t},meridiem:function(t,e,n){return t<12?"ص":"م"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},preparse:function(t){return t.replace(/[١٢٣٤٥٦٧٨٩٠]/g,function(t){return n[t]}).replace(/،/g,",")},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]}).replace(/,/g,"،")},week:{dow:0,doy:6}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ar-tn",{months:"جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),monthsShort:"جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"-inci",5:"-inci",8:"-inci",70:"-inci",80:"-inci",2:"-nci",7:"-nci",20:"-nci",50:"-nci",3:"-üncü",4:"-üncü",100:"-üncü",6:"-ncı",9:"-uncu",10:"-uncu",30:"-uncu",60:"-ıncı",90:"-ıncı"};t.defineLocale("az",{months:"yanvar_fevral_mart_aprel_may_iyun_iyul_avqust_sentyabr_oktyabr_noyabr_dekabr".split("_"),monthsShort:"yan_fev_mar_apr_may_iyn_iyl_avq_sen_okt_noy_dek".split("_"),weekdays:"Bazar_Bazar ertəsi_Çərşənbə axşamı_Çərşənbə_Cümə axşamı_Cümə_Şənbə".split("_"),weekdaysShort:"Baz_BzE_ÇAx_Çər_CAx_Cüm_Şən".split("_"),weekdaysMin:"Bz_BE_ÇA_Çə_CA_Cü_Şə".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün saat] LT",nextDay:"[sabah saat] LT",nextWeek:"[gələn həftə] dddd [saat] LT",lastDay:"[dünən] LT",lastWeek:"[keçən həftə] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s sonra",past:"%s əvvəl",s:"birneçə saniyə",ss:"%d saniyə",m:"bir dəqiqə",mm:"%d dəqiqə",h:"bir saat",hh:"%d saat",d:"bir gün",dd:"%d gün",M:"bir ay",MM:"%d ay",y:"bir il",yy:"%d il"},meridiemParse:/gecə|səhər|gündüz|axşam/,isPM:function(t){return/^(gündüz|axşam)$/.test(t)},meridiem:function(t,e,n){return t<4?"gecə":t<12?"səhər":t<17?"gündüz":"axşam"},dayOfMonthOrdinalParse:/\d{1,2}-(ıncı|inci|nci|üncü|ncı|uncu)/,ordinal:function(t){if(0===t)return t+"-ıncı";var n=t%10,r=t%100-n,i=t>=100?100:null;return t+(e[n]||e[r]||e[i])},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n){var r,i,o={ss:e?"секунда_секунды_секунд":"секунду_секунды_секунд",mm:e?"хвіліна_хвіліны_хвілін":"хвіліну_хвіліны_хвілін",hh:e?"гадзіна_гадзіны_гадзін":"гадзіну_гадзіны_гадзін",dd:"дзень_дні_дзён",MM:"месяц_месяцы_месяцаў",yy:"год_гады_гадоў"};return"m"===n?e?"хвіліна":"хвіліну":"h"===n?e?"гадзіна":"гадзіну":t+" "+(r=+t,i=o[n].split("_"),r%10==1&&r%100!=11?i[0]:r%10>=2&&r%10<=4&&(r%100<10||r%100>=20)?i[1]:i[2])}t.defineLocale("be",{months:{format:"студзеня_лютага_сакавіка_красавіка_траўня_чэрвеня_ліпеня_жніўня_верасня_кастрычніка_лістапада_снежня".split("_"),standalone:"студзень_люты_сакавік_красавік_травень_чэрвень_ліпень_жнівень_верасень_кастрычнік_лістапад_снежань".split("_")},monthsShort:"студ_лют_сак_крас_трав_чэрв_ліп_жнів_вер_каст_ліст_снеж".split("_"),weekdays:{format:"нядзелю_панядзелак_аўторак_сераду_чацвер_пятніцу_суботу".split("_"),standalone:"нядзеля_панядзелак_аўторак_серада_чацвер_пятніца_субота".split("_"),isFormat:/\[ ?[Ууў] ?(?:мінулую|наступную)? ?\] ?dddd/},weekdaysShort:"нд_пн_ат_ср_чц_пт_сб".split("_"),weekdaysMin:"нд_пн_ат_ср_чц_пт_сб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY г.",LLL:"D MMMM YYYY г., HH:mm",LLLL:"dddd, D MMMM YYYY г., HH:mm"},calendar:{sameDay:"[Сёння ў] LT",nextDay:"[Заўтра ў] LT",lastDay:"[Учора ў] LT",nextWeek:function(){return"[У] dddd [ў] LT"},lastWeek:function(){switch(this.day()){case 0:case 3:case 5:case 6:return"[У мінулую] dddd [ў] LT";case 1:case 2:case 4:return"[У мінулы] dddd [ў] LT"}},sameElse:"L"},relativeTime:{future:"праз %s",past:"%s таму",s:"некалькі секунд",m:e,mm:e,h:e,hh:e,d:"дзень",dd:e,M:"месяц",MM:e,y:"год",yy:e},meridiemParse:/ночы|раніцы|дня|вечара/,isPM:function(t){return/^(дня|вечара)$/.test(t)},meridiem:function(t,e,n){return t<4?"ночы":t<12?"раніцы":t<17?"дня":"вечара"},dayOfMonthOrdinalParse:/\d{1,2}-(і|ы|га)/,ordinal:function(t,e){switch(e){case"M":case"d":case"DDD":case"w":case"W":return t%10!=2&&t%10!=3||t%100==12||t%100==13?t+"-ы":t+"-і";case"D":return t+"-га";default:return t}},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("bg",{months:"януари_февруари_март_април_май_юни_юли_август_септември_октомври_ноември_декември".split("_"),monthsShort:"янр_фев_мар_апр_май_юни_юли_авг_сеп_окт_ное_дек".split("_"),weekdays:"неделя_понеделник_вторник_сряда_четвъртък_петък_събота".split("_"),weekdaysShort:"нед_пон_вто_сря_чет_пет_съб".split("_"),weekdaysMin:"нд_пн_вт_ср_чт_пт_сб".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[Днес в] LT",nextDay:"[Утре в] LT",nextWeek:"dddd [в] LT",lastDay:"[Вчера в] LT",lastWeek:function(){switch(this.day()){case 0:case 3:case 6:return"[В изминалата] dddd [в] LT";case 1:case 2:case 4:case 5:return"[В изминалия] dddd [в] LT"}},sameElse:"L"},relativeTime:{future:"след %s",past:"преди %s",s:"няколко секунди",ss:"%d секунди",m:"минута",mm:"%d минути",h:"час",hh:"%d часа",d:"ден",dd:"%d дни",M:"месец",MM:"%d месеца",y:"година",yy:"%d години"},dayOfMonthOrdinalParse:/\d{1,2}-(ев|ен|ти|ви|ри|ми)/,ordinal:function(t){var e=t%10,n=t%100;return 0===t?t+"-ев":0===n?t+"-ен":n>10&&n<20?t+"-ти":1===e?t+"-ви":2===e?t+"-ри":7===e||8===e?t+"-ми":t+"-ти"},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("bm",{months:"Zanwuyekalo_Fewuruyekalo_Marisikalo_Awirilikalo_Mɛkalo_Zuwɛnkalo_Zuluyekalo_Utikalo_Sɛtanburukalo_ɔkutɔburukalo_Nowanburukalo_Desanburukalo".split("_"),monthsShort:"Zan_Few_Mar_Awi_Mɛ_Zuw_Zul_Uti_Sɛt_ɔku_Now_Des".split("_"),weekdays:"Kari_Ntɛnɛn_Tarata_Araba_Alamisa_Juma_Sibiri".split("_"),weekdaysShort:"Kar_Ntɛ_Tar_Ara_Ala_Jum_Sib".split("_"),weekdaysMin:"Ka_Nt_Ta_Ar_Al_Ju_Si".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"MMMM [tile] D [san] YYYY",LLL:"MMMM [tile] D [san] YYYY [lɛrɛ] HH:mm",LLLL:"dddd MMMM [tile] D [san] YYYY [lɛrɛ] HH:mm"},calendar:{sameDay:"[Bi lɛrɛ] LT",nextDay:"[Sini lɛrɛ] LT",nextWeek:"dddd [don lɛrɛ] LT",lastDay:"[Kunu lɛrɛ] LT",lastWeek:"dddd [tɛmɛnen lɛrɛ] LT",sameElse:"L"},relativeTime:{future:"%s kɔnɔ",past:"a bɛ %s bɔ",s:"sanga dama dama",ss:"sekondi %d",m:"miniti kelen",mm:"miniti %d",h:"lɛrɛ kelen",hh:"lɛrɛ %d",d:"tile kelen",dd:"tile %d",M:"kalo kelen",MM:"kalo %d",y:"san kelen",yy:"san %d"},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"১",2:"২",3:"৩",4:"৪",5:"৫",6:"৬",7:"৭",8:"৮",9:"৯",0:"০"},n={"১":"1","২":"2","৩":"3","৪":"4","৫":"5","৬":"6","৭":"7","৮":"8","৯":"9","০":"0"};t.defineLocale("bn",{months:"জানুয়ারী_ফেব্রুয়ারি_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্টেম্বর_অক্টোবর_নভেম্বর_ডিসেম্বর".split("_"),monthsShort:"জানু_ফেব_মার্চ_এপ্র_মে_জুন_জুল_আগ_সেপ্ট_অক্টো_নভে_ডিসে".split("_"),weekdays:"রবিবার_সোমবার_মঙ্গলবার_বুধবার_বৃহস্পতিবার_শুক্রবার_শনিবার".split("_"),weekdaysShort:"রবি_সোম_মঙ্গল_বুধ_বৃহস্পতি_শুক্র_শনি".split("_"),weekdaysMin:"রবি_সোম_মঙ্গ_বুধ_বৃহঃ_শুক্র_শনি".split("_"),longDateFormat:{LT:"A h:mm সময়",LTS:"A h:mm:ss সময়",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm সময়",LLLL:"dddd, D MMMM YYYY, A h:mm সময়"},calendar:{sameDay:"[আজ] LT",nextDay:"[আগামীকাল] LT",nextWeek:"dddd, LT",lastDay:"[গতকাল] LT",lastWeek:"[গত] dddd, LT",sameElse:"L"},relativeTime:{future:"%s পরে",past:"%s আগে",s:"কয়েক সেকেন্ড",ss:"%d সেকেন্ড",m:"এক মিনিট",mm:"%d মিনিট",h:"এক ঘন্টা",hh:"%d ঘন্টা",d:"এক দিন",dd:"%d দিন",M:"এক মাস",MM:"%d মাস",y:"এক বছর",yy:"%d বছর"},preparse:function(t){return t.replace(/[১২৩৪৫৬৭৮৯০]/g,function(t){return n[t]})},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]})},meridiemParse:/রাত|সকাল|দুপুর|বিকাল|রাত/,meridiemHour:function(t,e){return 12===t&&(t=0),"রাত"===e&&t>=4||"দুপুর"===e&&t<5||"বিকাল"===e?t+12:t},meridiem:function(t,e,n){return t<4?"রাত":t<10?"সকাল":t<17?"দুপুর":t<20?"বিকাল":"রাত"},week:{dow:0,doy:6}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"༡",2:"༢",3:"༣",4:"༤",5:"༥",6:"༦",7:"༧",8:"༨",9:"༩",0:"༠"},n={"༡":"1","༢":"2","༣":"3","༤":"4","༥":"5","༦":"6","༧":"7","༨":"8","༩":"9","༠":"0"};t.defineLocale("bo",{months:"ཟླ་བ་དང་པོ_ཟླ་བ་གཉིས་པ_ཟླ་བ་གསུམ་པ_ཟླ་བ་བཞི་པ_ཟླ་བ་ལྔ་པ_ཟླ་བ་དྲུག་པ_ཟླ་བ་བདུན་པ_ཟླ་བ་བརྒྱད་པ_ཟླ་བ་དགུ་པ_ཟླ་བ་བཅུ་པ_ཟླ་བ་བཅུ་གཅིག་པ_ཟླ་བ་བཅུ་གཉིས་པ".split("_"),monthsShort:"ཟླ་བ་དང་པོ_ཟླ་བ་གཉིས་པ_ཟླ་བ་གསུམ་པ_ཟླ་བ་བཞི་པ_ཟླ་བ་ལྔ་པ_ཟླ་བ་དྲུག་པ_ཟླ་བ་བདུན་པ_ཟླ་བ་བརྒྱད་པ_ཟླ་བ་དགུ་པ_ཟླ་བ་བཅུ་པ_ཟླ་བ་བཅུ་གཅིག་པ_ཟླ་བ་བཅུ་གཉིས་པ".split("_"),weekdays:"གཟའ་ཉི་མ་_གཟའ་ཟླ་བ་_གཟའ་མིག་དམར་_གཟའ་ལྷག་པ་_གཟའ་ཕུར་བུ_གཟའ་པ་སངས་_གཟའ་སྤེན་པ་".split("_"),weekdaysShort:"ཉི་མ་_ཟླ་བ་_མིག་དམར་_ལྷག་པ་_ཕུར་བུ_པ་སངས་_སྤེན་པ་".split("_"),weekdaysMin:"ཉི་མ་_ཟླ་བ་_མིག་དམར་_ལྷག་པ་_ཕུར་བུ_པ་སངས་_སྤེན་པ་".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[དི་རིང] LT",nextDay:"[སང་ཉིན] LT",nextWeek:"[བདུན་ཕྲག་རྗེས་མ], LT",lastDay:"[ཁ་སང] LT",lastWeek:"[བདུན་ཕྲག་མཐའ་མ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ལ་",past:"%s སྔན་ལ",s:"ལམ་སང",ss:"%d སྐར་ཆ།",m:"སྐར་མ་གཅིག",mm:"%d སྐར་མ",h:"ཆུ་ཚོད་གཅིག",hh:"%d ཆུ་ཚོད",d:"ཉིན་གཅིག",dd:"%d ཉིན་",M:"ཟླ་བ་གཅིག",MM:"%d ཟླ་བ",y:"ལོ་གཅིག",yy:"%d ལོ"},preparse:function(t){return t.replace(/[༡༢༣༤༥༦༧༨༩༠]/g,function(t){return n[t]})},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]})},meridiemParse:/མཚན་མོ|ཞོགས་ཀས|ཉིན་གུང|དགོང་དག|མཚན་མོ/,meridiemHour:function(t,e){return 12===t&&(t=0),"མཚན་མོ"===e&&t>=4||"ཉིན་གུང"===e&&t<5||"དགོང་དག"===e?t+12:t},meridiem:function(t,e,n){return t<4?"མཚན་མོ":t<10?"ཞོགས་ཀས":t<17?"ཉིན་གུང":t<20?"དགོང་དག":"མཚན་མོ"},week:{dow:0,doy:6}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n){return t+" "+function(t,e){return 2===e?function(t){var e={m:"v",b:"v",d:"z"};return void 0===e[t.charAt(0)]?t:e[t.charAt(0)]+t.substring(1)}(t):t}({mm:"munutenn",MM:"miz",dd:"devezh"}[n],t)}t.defineLocale("br",{months:"Genver_C'hwevrer_Meurzh_Ebrel_Mae_Mezheven_Gouere_Eost_Gwengolo_Here_Du_Kerzu".split("_"),monthsShort:"Gen_C'hwe_Meu_Ebr_Mae_Eve_Gou_Eos_Gwe_Her_Du_Ker".split("_"),weekdays:"Sul_Lun_Meurzh_Merc'her_Yaou_Gwener_Sadorn".split("_"),weekdaysShort:"Sul_Lun_Meu_Mer_Yao_Gwe_Sad".split("_"),weekdaysMin:"Su_Lu_Me_Mer_Ya_Gw_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h[e]mm A",LTS:"h[e]mm:ss A",L:"DD/MM/YYYY",LL:"D [a viz] MMMM YYYY",LLL:"D [a viz] MMMM YYYY h[e]mm A",LLLL:"dddd, D [a viz] MMMM YYYY h[e]mm A"},calendar:{sameDay:"[Hiziv da] LT",nextDay:"[Warc'hoazh da] LT",nextWeek:"dddd [da] LT",lastDay:"[Dec'h da] LT",lastWeek:"dddd [paset da] LT",sameElse:"L"},relativeTime:{future:"a-benn %s",past:"%s 'zo",s:"un nebeud segondennoù",ss:"%d eilenn",m:"ur vunutenn",mm:e,h:"un eur",hh:"%d eur",d:"un devezh",dd:e,M:"ur miz",MM:e,y:"ur bloaz",yy:function(t){switch(function t(e){return e>9?t(e%10):e}(t)){case 1:case 3:case 4:case 5:case 9:return t+" bloaz";default:return t+" vloaz"}}},dayOfMonthOrdinalParse:/\d{1,2}(añ|vet)/,ordinal:function(t){var e=1===t?"añ":"vet";return t+e},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n){var r=t+" ";switch(n){case"ss":return r+=1===t?"sekunda":2===t||3===t||4===t?"sekunde":"sekundi";case"m":return e?"jedna minuta":"jedne minute";case"mm":return r+=1===t?"minuta":2===t||3===t||4===t?"minute":"minuta";case"h":return e?"jedan sat":"jednog sata";case"hh":return r+=1===t?"sat":2===t||3===t||4===t?"sata":"sati";case"dd":return r+=1===t?"dan":"dana";case"MM":return r+=1===t?"mjesec":2===t||3===t||4===t?"mjeseca":"mjeseci";case"yy":return r+=1===t?"godina":2===t||3===t||4===t?"godine":"godina"}}t.defineLocale("bs",{months:"januar_februar_mart_april_maj_juni_juli_august_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj._jun._jul._aug._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[jučer u] LT",lastWeek:function(){switch(this.day()){case 0:case 3:return"[prošlu] dddd [u] LT";case 6:return"[prošle] [subote] [u] LT";case 1:case 2:case 4:case 5:return"[prošli] dddd [u] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"par sekundi",ss:e,m:e,mm:e,h:e,hh:e,d:"dan",dd:e,M:"mjesec",MM:e,y:"godinu",yy:e},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ca",{months:{standalone:"gener_febrer_març_abril_maig_juny_juliol_agost_setembre_octubre_novembre_desembre".split("_"),format:"de gener_de febrer_de març_d'abril_de maig_de juny_de juliol_d'agost_de setembre_d'octubre_de novembre_de desembre".split("_"),isFormat:/D[oD]?(\s)+MMMM/},monthsShort:"gen._febr._març_abr._maig_juny_jul._ag._set._oct._nov._des.".split("_"),monthsParseExact:!0,weekdays:"diumenge_dilluns_dimarts_dimecres_dijous_divendres_dissabte".split("_"),weekdaysShort:"dg._dl._dt._dc._dj._dv._ds.".split("_"),weekdaysMin:"dg_dl_dt_dc_dj_dv_ds".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [de] YYYY",ll:"D MMM YYYY",LLL:"D MMMM [de] YYYY [a les] H:mm",lll:"D MMM YYYY, H:mm",LLLL:"dddd D MMMM [de] YYYY [a les] H:mm",llll:"ddd D MMM YYYY, H:mm"},calendar:{sameDay:function(){return"[avui a "+(1!==this.hours()?"les":"la")+"] LT"},nextDay:function(){return"[demà a "+(1!==this.hours()?"les":"la")+"] LT"},nextWeek:function(){return"dddd [a "+(1!==this.hours()?"les":"la")+"] LT"},lastDay:function(){return"[ahir a "+(1!==this.hours()?"les":"la")+"] LT"},lastWeek:function(){return"[el] dddd [passat a "+(1!==this.hours()?"les":"la")+"] LT"},sameElse:"L"},relativeTime:{future:"d'aquí %s",past:"fa %s",s:"uns segons",ss:"%d segons",m:"un minut",mm:"%d minuts",h:"una hora",hh:"%d hores",d:"un dia",dd:"%d dies",M:"un mes",MM:"%d mesos",y:"un any",yy:"%d anys"},dayOfMonthOrdinalParse:/\d{1,2}(r|n|t|è|a)/,ordinal:function(t,e){var n=1===t?"r":2===t?"n":3===t?"r":4===t?"t":"è";return"w"!==e&&"W"!==e||(n="a"),t+n},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e="leden_únor_březen_duben_květen_červen_červenec_srpen_září_říjen_listopad_prosinec".split("_"),n="led_úno_bře_dub_kvě_čvn_čvc_srp_zář_říj_lis_pro".split("_"),r=[/^led/i,/^úno/i,/^bře/i,/^dub/i,/^kvě/i,/^(čvn|červen$|června)/i,/^(čvc|červenec|července)/i,/^srp/i,/^zář/i,/^říj/i,/^lis/i,/^pro/i],i=/^(leden|únor|březen|duben|květen|červenec|července|červen|června|srpen|září|říjen|listopad|prosinec|led|úno|bře|dub|kvě|čvn|čvc|srp|zář|říj|lis|pro)/i;function o(t){return t>1&&t<5&&1!=~~(t/10)}function a(t,e,n,r){var i=t+" ";switch(n){case"s":return e||r?"pár sekund":"pár sekundami";case"ss":return e||r?i+(o(t)?"sekundy":"sekund"):i+"sekundami";case"m":return e?"minuta":r?"minutu":"minutou";case"mm":return e||r?i+(o(t)?"minuty":"minut"):i+"minutami";case"h":return e?"hodina":r?"hodinu":"hodinou";case"hh":return e||r?i+(o(t)?"hodiny":"hodin"):i+"hodinami";case"d":return e||r?"den":"dnem";case"dd":return e||r?i+(o(t)?"dny":"dní"):i+"dny";case"M":return e||r?"měsíc":"měsícem";case"MM":return e||r?i+(o(t)?"měsíce":"měsíců"):i+"měsíci";case"y":return e||r?"rok":"rokem";case"yy":return e||r?i+(o(t)?"roky":"let"):i+"lety"}}t.defineLocale("cs",{months:e,monthsShort:n,monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(leden|ledna|února|únor|březen|března|duben|dubna|květen|května|červenec|července|červen|června|srpen|srpna|září|říjen|října|listopadu|listopad|prosinec|prosince)/i,monthsShortStrictRegex:/^(led|úno|bře|dub|kvě|čvn|čvc|srp|zář|říj|lis|pro)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"neděle_pondělí_úterý_středa_čtvrtek_pátek_sobota".split("_"),weekdaysShort:"ne_po_út_st_čt_pá_so".split("_"),weekdaysMin:"ne_po_út_st_čt_pá_so".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd D. MMMM YYYY H:mm",l:"D. M. YYYY"},calendar:{sameDay:"[dnes v] LT",nextDay:"[zítra v] LT",nextWeek:function(){switch(this.day()){case 0:return"[v neděli v] LT";case 1:case 2:return"[v] dddd [v] LT";case 3:return"[ve středu v] LT";case 4:return"[ve čtvrtek v] LT";case 5:return"[v pátek v] LT";case 6:return"[v sobotu v] LT"}},lastDay:"[včera v] LT",lastWeek:function(){switch(this.day()){case 0:return"[minulou neděli v] LT";case 1:case 2:return"[minulé] dddd [v] LT";case 3:return"[minulou středu v] LT";case 4:case 5:return"[minulý] dddd [v] LT";case 6:return"[minulou sobotu v] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"před %s",s:a,ss:a,m:a,mm:a,h:a,hh:a,d:a,dd:a,M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("cv",{months:"кӑрлач_нарӑс_пуш_ака_май_ҫӗртме_утӑ_ҫурла_авӑн_юпа_чӳк_раштав".split("_"),monthsShort:"кӑр_нар_пуш_ака_май_ҫӗр_утӑ_ҫур_авн_юпа_чӳк_раш".split("_"),weekdays:"вырсарникун_тунтикун_ытларикун_юнкун_кӗҫнерникун_эрнекун_шӑматкун".split("_"),weekdaysShort:"выр_тун_ытл_юн_кӗҫ_эрн_шӑм".split("_"),weekdaysMin:"вр_тн_ыт_юн_кҫ_эр_шм".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ]",LLL:"YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ], HH:mm",LLLL:"dddd, YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ], HH:mm"},calendar:{sameDay:"[Паян] LT [сехетре]",nextDay:"[Ыран] LT [сехетре]",lastDay:"[Ӗнер] LT [сехетре]",nextWeek:"[Ҫитес] dddd LT [сехетре]",lastWeek:"[Иртнӗ] dddd LT [сехетре]",sameElse:"L"},relativeTime:{future:function(t){var e=/сехет$/i.exec(t)?"рен":/ҫул$/i.exec(t)?"тан":"ран";return t+e},past:"%s каялла",s:"пӗр-ик ҫеккунт",ss:"%d ҫеккунт",m:"пӗр минут",mm:"%d минут",h:"пӗр сехет",hh:"%d сехет",d:"пӗр кун",dd:"%d кун",M:"пӗр уйӑх",MM:"%d уйӑх",y:"пӗр ҫул",yy:"%d ҫул"},dayOfMonthOrdinalParse:/\d{1,2}-мӗш/,ordinal:"%d-мӗш",week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("cy",{months:"Ionawr_Chwefror_Mawrth_Ebrill_Mai_Mehefin_Gorffennaf_Awst_Medi_Hydref_Tachwedd_Rhagfyr".split("_"),monthsShort:"Ion_Chwe_Maw_Ebr_Mai_Meh_Gor_Aws_Med_Hyd_Tach_Rhag".split("_"),weekdays:"Dydd Sul_Dydd Llun_Dydd Mawrth_Dydd Mercher_Dydd Iau_Dydd Gwener_Dydd Sadwrn".split("_"),weekdaysShort:"Sul_Llun_Maw_Mer_Iau_Gwe_Sad".split("_"),weekdaysMin:"Su_Ll_Ma_Me_Ia_Gw_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Heddiw am] LT",nextDay:"[Yfory am] LT",nextWeek:"dddd [am] LT",lastDay:"[Ddoe am] LT",lastWeek:"dddd [diwethaf am] LT",sameElse:"L"},relativeTime:{future:"mewn %s",past:"%s yn ôl",s:"ychydig eiliadau",ss:"%d eiliad",m:"munud",mm:"%d munud",h:"awr",hh:"%d awr",d:"diwrnod",dd:"%d diwrnod",M:"mis",MM:"%d mis",y:"blwyddyn",yy:"%d flynedd"},dayOfMonthOrdinalParse:/\d{1,2}(fed|ain|af|il|ydd|ed|eg)/,ordinal:function(t){var e=t,n="";return e>20?n=40===e||50===e||60===e||80===e||100===e?"fed":"ain":e>0&&(n=["","af","il","ydd","ydd","ed","ed","ed","fed","fed","fed","eg","fed","eg","eg","fed","eg","eg","fed","eg","fed"][e]),t+n},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("da",{months:"januar_februar_marts_april_maj_juni_juli_august_september_oktober_november_december".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec".split("_"),weekdays:"søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag".split("_"),weekdaysShort:"søn_man_tir_ons_tor_fre_lør".split("_"),weekdaysMin:"sø_ma_ti_on_to_fr_lø".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd [d.] D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[i dag kl.] LT",nextDay:"[i morgen kl.] LT",nextWeek:"på dddd [kl.] LT",lastDay:"[i går kl.] LT",lastWeek:"[i] dddd[s kl.] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s siden",s:"få sekunder",ss:"%d sekunder",m:"et minut",mm:"%d minutter",h:"en time",hh:"%d timer",d:"en dag",dd:"%d dage",M:"en måned",MM:"%d måneder",y:"et år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n,r){var i={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[t+" Tage",t+" Tagen"],M:["ein Monat","einem Monat"],MM:[t+" Monate",t+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[t+" Jahre",t+" Jahren"]};return e?i[n][0]:i[n][1]}t.defineLocale("de",{months:"Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:e,mm:"%d Minuten",h:e,hh:"%d Stunden",d:e,dd:e,M:e,MM:e,y:e,yy:e},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n,r){var i={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[t+" Tage",t+" Tagen"],M:["ein Monat","einem Monat"],MM:[t+" Monate",t+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[t+" Jahre",t+" Jahren"]};return e?i[n][0]:i[n][1]}t.defineLocale("de-at",{months:"Jänner_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jän._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:e,mm:"%d Minuten",h:e,hh:"%d Stunden",d:e,dd:e,M:e,MM:e,y:e,yy:e},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n,r){var i={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[t+" Tage",t+" Tagen"],M:["ein Monat","einem Monat"],MM:[t+" Monate",t+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[t+" Jahre",t+" Jahren"]};return e?i[n][0]:i[n][1]}t.defineLocale("de-ch",{months:"Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:e,mm:"%d Minuten",h:e,hh:"%d Stunden",d:e,dd:e,M:e,MM:e,y:e,yy:e},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e=["ޖެނުއަރީ","ފެބްރުއަރީ","މާރިޗު","އޭޕްރީލު","މޭ","ޖޫން","ޖުލައި","އޯގަސްޓު","ސެޕްޓެމްބަރު","އޮކްޓޯބަރު","ނޮވެމްބަރު","ޑިސެމްބަރު"],n=["އާދިއްތަ","ހޯމަ","އަންގާރަ","ބުދަ","ބުރާސްފަތި","ހުކުރު","ހޮނިހިރު"];t.defineLocale("dv",{months:e,monthsShort:e,weekdays:n,weekdaysShort:n,weekdaysMin:"އާދި_ހޯމަ_އަން_ބުދަ_ބުރާ_ހުކު_ހޮނި".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/M/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/މކ|މފ/,isPM:function(t){return"މފ"===t},meridiem:function(t,e,n){return t<12?"މކ":"މފ"},calendar:{sameDay:"[މިއަދު] LT",nextDay:"[މާދަމާ] LT",nextWeek:"dddd LT",lastDay:"[އިއްޔެ] LT",lastWeek:"[ފާއިތުވި] dddd LT",sameElse:"L"},relativeTime:{future:"ތެރޭގައި %s",past:"ކުރިން %s",s:"ސިކުންތުކޮޅެއް",ss:"d% ސިކުންތު",m:"މިނިޓެއް",mm:"މިނިޓު %d",h:"ގަޑިއިރެއް",hh:"ގަޑިއިރު %d",d:"ދުވަހެއް",dd:"ދުވަސް %d",M:"މަހެއް",MM:"މަސް %d",y:"އަހަރެއް",yy:"އަހަރު %d"},preparse:function(t){return t.replace(/،/g,",")},postformat:function(t){return t.replace(/,/g,"،")},week:{dow:7,doy:12}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("el",{monthsNominativeEl:"Ιανουάριος_Φεβρουάριος_Μάρτιος_Απρίλιος_Μάιος_Ιούνιος_Ιούλιος_Αύγουστος_Σεπτέμβριος_Οκτώβριος_Νοέμβριος_Δεκέμβριος".split("_"),monthsGenitiveEl:"Ιανουαρίου_Φεβρουαρίου_Μαρτίου_Απριλίου_Μαΐου_Ιουνίου_Ιουλίου_Αυγούστου_Σεπτεμβρίου_Οκτωβρίου_Νοεμβρίου_Δεκεμβρίου".split("_"),months:function(t,e){return t?"string"==typeof e&&/D/.test(e.substring(0,e.indexOf("MMMM")))?this._monthsGenitiveEl[t.month()]:this._monthsNominativeEl[t.month()]:this._monthsNominativeEl},monthsShort:"Ιαν_Φεβ_Μαρ_Απρ_Μαϊ_Ιουν_Ιουλ_Αυγ_Σεπ_Οκτ_Νοε_Δεκ".split("_"),weekdays:"Κυριακή_Δευτέρα_Τρίτη_Τετάρτη_Πέμπτη_Παρασκευή_Σάββατο".split("_"),weekdaysShort:"Κυρ_Δευ_Τρι_Τετ_Πεμ_Παρ_Σαβ".split("_"),weekdaysMin:"Κυ_Δε_Τρ_Τε_Πε_Πα_Σα".split("_"),meridiem:function(t,e,n){return t>11?n?"μμ":"ΜΜ":n?"πμ":"ΠΜ"},isPM:function(t){return"μ"===(t+"").toLowerCase()[0]},meridiemParse:/[ΠΜ]\.?Μ?\.?/i,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendarEl:{sameDay:"[Σήμερα {}] LT",nextDay:"[Αύριο {}] LT",nextWeek:"dddd [{}] LT",lastDay:"[Χθες {}] LT",lastWeek:function(){switch(this.day()){case 6:return"[το προηγούμενο] dddd [{}] LT";default:return"[την προηγούμενη] dddd [{}] LT"}},sameElse:"L"},calendar:function(t,e){var n,r=this._calendarEl[t],i=e&&e.hours();return((n=r)instanceof Function||"[object Function]"===Object.prototype.toString.call(n))&&(r=r.apply(e)),r.replace("{}",i%12==1?"στη":"στις")},relativeTime:{future:"σε %s",past:"%s πριν",s:"λίγα δευτερόλεπτα",ss:"%d δευτερόλεπτα",m:"ένα λεπτό",mm:"%d λεπτά",h:"μία ώρα",hh:"%d ώρες",d:"μία μέρα",dd:"%d μέρες",M:"ένας μήνας",MM:"%d μήνες",y:"ένας χρόνος",yy:"%d χρόνια"},dayOfMonthOrdinalParse:/\d{1,2}η/,ordinal:"%dη",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("en-SG",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(t){var e=t%10,n=1==~~(t%100/10)?"th":1===e?"st":2===e?"nd":3===e?"rd":"th";return t+n},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("en-au",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(t){var e=t%10,n=1==~~(t%100/10)?"th":1===e?"st":2===e?"nd":3===e?"rd":"th";return t+n},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("en-ca",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"YYYY-MM-DD",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(t){var e=t%10,n=1==~~(t%100/10)?"th":1===e?"st":2===e?"nd":3===e?"rd":"th";return t+n}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("en-gb",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(t){var e=t%10,n=1==~~(t%100/10)?"th":1===e?"st":2===e?"nd":3===e?"rd":"th";return t+n},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("en-ie",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(t){var e=t%10,n=1==~~(t%100/10)?"th":1===e?"st":2===e?"nd":3===e?"rd":"th";return t+n},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("en-il",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(t){var e=t%10,n=1==~~(t%100/10)?"th":1===e?"st":2===e?"nd":3===e?"rd":"th";return t+n}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("en-nz",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(t){var e=t%10,n=1==~~(t%100/10)?"th":1===e?"st":2===e?"nd":3===e?"rd":"th";return t+n},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("eo",{months:"januaro_februaro_marto_aprilo_majo_junio_julio_aŭgusto_septembro_oktobro_novembro_decembro".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aŭg_sep_okt_nov_dec".split("_"),weekdays:"dimanĉo_lundo_mardo_merkredo_ĵaŭdo_vendredo_sabato".split("_"),weekdaysShort:"dim_lun_mard_merk_ĵaŭ_ven_sab".split("_"),weekdaysMin:"di_lu_ma_me_ĵa_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D[-a de] MMMM, YYYY",LLL:"D[-a de] MMMM, YYYY HH:mm",LLLL:"dddd, [la] D[-a de] MMMM, YYYY HH:mm"},meridiemParse:/[ap]\.t\.m/i,isPM:function(t){return"p"===t.charAt(0).toLowerCase()},meridiem:function(t,e,n){return t>11?n?"p.t.m.":"P.T.M.":n?"a.t.m.":"A.T.M."},calendar:{sameDay:"[Hodiaŭ je] LT",nextDay:"[Morgaŭ je] LT",nextWeek:"dddd [je] LT",lastDay:"[Hieraŭ je] LT",lastWeek:"[pasinta] dddd [je] LT",sameElse:"L"},relativeTime:{future:"post %s",past:"antaŭ %s",s:"sekundoj",ss:"%d sekundoj",m:"minuto",mm:"%d minutoj",h:"horo",hh:"%d horoj",d:"tago",dd:"%d tagoj",M:"monato",MM:"%d monatoj",y:"jaro",yy:"%d jaroj"},dayOfMonthOrdinalParse:/\d{1,2}a/,ordinal:"%da",week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";var e="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),n="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),r=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],i=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i;t.defineLocale("es",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(t,r){return t?/-MMM-/.test(r)?n[t.month()]:e[t.month()]:e},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),n="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),r=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],i=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i;t.defineLocale("es-do",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(t,r){return t?/-MMM-/.test(r)?n[t.month()]:e[t.month()]:e},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY h:mm A",LLLL:"dddd, D [de] MMMM [de] YYYY h:mm A"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),n="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),r=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],i=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i;t.defineLocale("es-us",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(t,r){return t?/-MMM-/.test(r)?n[t.month()]:e[t.month()]:e},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"MM/DD/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY h:mm A",LLLL:"dddd, D [de] MMMM [de] YYYY h:mm A"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:0,doy:6}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n,r){var i={s:["mõne sekundi","mõni sekund","paar sekundit"],ss:[t+"sekundi",t+"sekundit"],m:["ühe minuti","üks minut"],mm:[t+" minuti",t+" minutit"],h:["ühe tunni","tund aega","üks tund"],hh:[t+" tunni",t+" tundi"],d:["ühe päeva","üks päev"],M:["kuu aja","kuu aega","üks kuu"],MM:[t+" kuu",t+" kuud"],y:["ühe aasta","aasta","üks aasta"],yy:[t+" aasta",t+" aastat"]};return e?i[n][2]?i[n][2]:i[n][1]:r?i[n][0]:i[n][1]}t.defineLocale("et",{months:"jaanuar_veebruar_märts_aprill_mai_juuni_juuli_august_september_oktoober_november_detsember".split("_"),monthsShort:"jaan_veebr_märts_apr_mai_juuni_juuli_aug_sept_okt_nov_dets".split("_"),weekdays:"pühapäev_esmaspäev_teisipäev_kolmapäev_neljapäev_reede_laupäev".split("_"),weekdaysShort:"P_E_T_K_N_R_L".split("_"),weekdaysMin:"P_E_T_K_N_R_L".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[Täna,] LT",nextDay:"[Homme,] LT",nextWeek:"[Järgmine] dddd LT",lastDay:"[Eile,] LT",lastWeek:"[Eelmine] dddd LT",sameElse:"L"},relativeTime:{future:"%s pärast",past:"%s tagasi",s:e,ss:e,m:e,mm:e,h:e,hh:e,d:e,dd:"%d päeva",M:e,MM:e,y:e,yy:e},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("eu",{months:"urtarrila_otsaila_martxoa_apirila_maiatza_ekaina_uztaila_abuztua_iraila_urria_azaroa_abendua".split("_"),monthsShort:"urt._ots._mar._api._mai._eka._uzt._abu._ira._urr._aza._abe.".split("_"),monthsParseExact:!0,weekdays:"igandea_astelehena_asteartea_asteazkena_osteguna_ostirala_larunbata".split("_"),weekdaysShort:"ig._al._ar._az._og._ol._lr.".split("_"),weekdaysMin:"ig_al_ar_az_og_ol_lr".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY[ko] MMMM[ren] D[a]",LLL:"YYYY[ko] MMMM[ren] D[a] HH:mm",LLLL:"dddd, YYYY[ko] MMMM[ren] D[a] HH:mm",l:"YYYY-M-D",ll:"YYYY[ko] MMM D[a]",lll:"YYYY[ko] MMM D[a] HH:mm",llll:"ddd, YYYY[ko] MMM D[a] HH:mm"},calendar:{sameDay:"[gaur] LT[etan]",nextDay:"[bihar] LT[etan]",nextWeek:"dddd LT[etan]",lastDay:"[atzo] LT[etan]",lastWeek:"[aurreko] dddd LT[etan]",sameElse:"L"},relativeTime:{future:"%s barru",past:"duela %s",s:"segundo batzuk",ss:"%d segundo",m:"minutu bat",mm:"%d minutu",h:"ordu bat",hh:"%d ordu",d:"egun bat",dd:"%d egun",M:"hilabete bat",MM:"%d hilabete",y:"urte bat",yy:"%d urte"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"۱",2:"۲",3:"۳",4:"۴",5:"۵",6:"۶",7:"۷",8:"۸",9:"۹",0:"۰"},n={"۱":"1","۲":"2","۳":"3","۴":"4","۵":"5","۶":"6","۷":"7","۸":"8","۹":"9","۰":"0"};t.defineLocale("fa",{months:"ژانویه_فوریه_مارس_آوریل_مه_ژوئن_ژوئیه_اوت_سپتامبر_اکتبر_نوامبر_دسامبر".split("_"),monthsShort:"ژانویه_فوریه_مارس_آوریل_مه_ژوئن_ژوئیه_اوت_سپتامبر_اکتبر_نوامبر_دسامبر".split("_"),weekdays:"یک‌شنبه_دوشنبه_سه‌شنبه_چهارشنبه_پنج‌شنبه_جمعه_شنبه".split("_"),weekdaysShort:"یک‌شنبه_دوشنبه_سه‌شنبه_چهارشنبه_پنج‌شنبه_جمعه_شنبه".split("_"),weekdaysMin:"ی_د_س_چ_پ_ج_ش".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/قبل از ظهر|بعد از ظهر/,isPM:function(t){return/بعد از ظهر/.test(t)},meridiem:function(t,e,n){return t<12?"قبل از ظهر":"بعد از ظهر"},calendar:{sameDay:"[امروز ساعت] LT",nextDay:"[فردا ساعت] LT",nextWeek:"dddd [ساعت] LT",lastDay:"[دیروز ساعت] LT",lastWeek:"dddd [پیش] [ساعت] LT",sameElse:"L"},relativeTime:{future:"در %s",past:"%s پیش",s:"چند ثانیه",ss:"ثانیه d%",m:"یک دقیقه",mm:"%d دقیقه",h:"یک ساعت",hh:"%d ساعت",d:"یک روز",dd:"%d روز",M:"یک ماه",MM:"%d ماه",y:"یک سال",yy:"%d سال"},preparse:function(t){return t.replace(/[۰-۹]/g,function(t){return n[t]}).replace(/،/g,",")},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]}).replace(/,/g,"،")},dayOfMonthOrdinalParse:/\d{1,2}م/,ordinal:"%dم",week:{dow:6,doy:12}})}(n(7))},function(t,e,n){!function(t){"use strict";var e="nolla yksi kaksi kolme neljä viisi kuusi seitsemän kahdeksan yhdeksän".split(" "),n=["nolla","yhden","kahden","kolmen","neljän","viiden","kuuden",e[7],e[8],e[9]];function r(t,r,i,o){var a="";switch(i){case"s":return o?"muutaman sekunnin":"muutama sekunti";case"ss":return o?"sekunnin":"sekuntia";case"m":return o?"minuutin":"minuutti";case"mm":a=o?"minuutin":"minuuttia";break;case"h":return o?"tunnin":"tunti";case"hh":a=o?"tunnin":"tuntia";break;case"d":return o?"päivän":"päivä";case"dd":a=o?"päivän":"päivää";break;case"M":return o?"kuukauden":"kuukausi";case"MM":a=o?"kuukauden":"kuukautta";break;case"y":return o?"vuoden":"vuosi";case"yy":a=o?"vuoden":"vuotta"}return a=function(t,r){return t<10?r?n[t]:e[t]:t}(t,o)+" "+a}t.defineLocale("fi",{months:"tammikuu_helmikuu_maaliskuu_huhtikuu_toukokuu_kesäkuu_heinäkuu_elokuu_syyskuu_lokakuu_marraskuu_joulukuu".split("_"),monthsShort:"tammi_helmi_maalis_huhti_touko_kesä_heinä_elo_syys_loka_marras_joulu".split("_"),weekdays:"sunnuntai_maanantai_tiistai_keskiviikko_torstai_perjantai_lauantai".split("_"),weekdaysShort:"su_ma_ti_ke_to_pe_la".split("_"),weekdaysMin:"su_ma_ti_ke_to_pe_la".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD.MM.YYYY",LL:"Do MMMM[ta] YYYY",LLL:"Do MMMM[ta] YYYY, [klo] HH.mm",LLLL:"dddd, Do MMMM[ta] YYYY, [klo] HH.mm",l:"D.M.YYYY",ll:"Do MMM YYYY",lll:"Do MMM YYYY, [klo] HH.mm",llll:"ddd, Do MMM YYYY, [klo] HH.mm"},calendar:{sameDay:"[tänään] [klo] LT",nextDay:"[huomenna] [klo] LT",nextWeek:"dddd [klo] LT",lastDay:"[eilen] [klo] LT",lastWeek:"[viime] dddd[na] [klo] LT",sameElse:"L"},relativeTime:{future:"%s päästä",past:"%s sitten",s:r,ss:r,m:r,mm:r,h:r,hh:r,d:r,dd:r,M:r,MM:r,y:r,yy:r},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("fo",{months:"januar_februar_mars_apríl_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan_feb_mar_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_"),weekdays:"sunnudagur_mánadagur_týsdagur_mikudagur_hósdagur_fríggjadagur_leygardagur".split("_"),weekdaysShort:"sun_mán_týs_mik_hós_frí_ley".split("_"),weekdaysMin:"su_má_tý_mi_hó_fr_le".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D. MMMM, YYYY HH:mm"},calendar:{sameDay:"[Í dag kl.] LT",nextDay:"[Í morgin kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[Í gjár kl.] LT",lastWeek:"[síðstu] dddd [kl] LT",sameElse:"L"},relativeTime:{future:"um %s",past:"%s síðani",s:"fá sekund",ss:"%d sekundir",m:"ein minuttur",mm:"%d minuttir",h:"ein tími",hh:"%d tímar",d:"ein dagur",dd:"%d dagar",M:"ein mánaður",MM:"%d mánaðir",y:"eitt ár",yy:"%d ár"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("fr",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|)/,ordinal:function(t,e){switch(e){case"D":return t+(1===t?"er":"");default:case"M":case"Q":case"DDD":case"d":return t+(1===t?"er":"e");case"w":case"W":return t+(1===t?"re":"e")}},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("fr-ca",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|e)/,ordinal:function(t,e){switch(e){default:case"M":case"Q":case"D":case"DDD":case"d":return t+(1===t?"er":"e");case"w":case"W":return t+(1===t?"re":"e")}}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("fr-ch",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|e)/,ordinal:function(t,e){switch(e){default:case"M":case"Q":case"D":case"DDD":case"d":return t+(1===t?"er":"e");case"w":case"W":return t+(1===t?"re":"e")}},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e="jan._feb._mrt._apr._mai_jun._jul._aug._sep._okt._nov._des.".split("_"),n="jan_feb_mrt_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_");t.defineLocale("fy",{months:"jannewaris_febrewaris_maart_april_maaie_juny_july_augustus_septimber_oktober_novimber_desimber".split("_"),monthsShort:function(t,r){return t?/-MMM-/.test(r)?n[t.month()]:e[t.month()]:e},monthsParseExact:!0,weekdays:"snein_moandei_tiisdei_woansdei_tongersdei_freed_sneon".split("_"),weekdaysShort:"si._mo._ti._wo._to._fr._so.".split("_"),weekdaysMin:"Si_Mo_Ti_Wo_To_Fr_So".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[hjoed om] LT",nextDay:"[moarn om] LT",nextWeek:"dddd [om] LT",lastDay:"[juster om] LT",lastWeek:"[ôfrûne] dddd [om] LT",sameElse:"L"},relativeTime:{future:"oer %s",past:"%s lyn",s:"in pear sekonden",ss:"%d sekonden",m:"ien minút",mm:"%d minuten",h:"ien oere",hh:"%d oeren",d:"ien dei",dd:"%d dagen",M:"ien moanne",MM:"%d moannen",y:"ien jier",yy:"%d jierren"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(t){return t+(1===t||8===t||t>=20?"ste":"de")},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ga",{months:["Eanáir","Feabhra","Márta","Aibreán","Bealtaine","Méitheamh","Iúil","Lúnasa","Meán Fómhair","Deaireadh Fómhair","Samhain","Nollaig"],monthsShort:["Eaná","Feab","Márt","Aibr","Beal","Méit","Iúil","Lúna","Meán","Deai","Samh","Noll"],monthsParseExact:!0,weekdays:["Dé Domhnaigh","Dé Luain","Dé Máirt","Dé Céadaoin","Déardaoin","Dé hAoine","Dé Satharn"],weekdaysShort:["Dom","Lua","Mái","Céa","Déa","hAo","Sat"],weekdaysMin:["Do","Lu","Má","Ce","Dé","hA","Sa"],longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Inniu ag] LT",nextDay:"[Amárach ag] LT",nextWeek:"dddd [ag] LT",lastDay:"[Inné aig] LT",lastWeek:"dddd [seo caite] [ag] LT",sameElse:"L"},relativeTime:{future:"i %s",past:"%s ó shin",s:"cúpla soicind",ss:"%d soicind",m:"nóiméad",mm:"%d nóiméad",h:"uair an chloig",hh:"%d uair an chloig",d:"lá",dd:"%d lá",M:"mí",MM:"%d mí",y:"bliain",yy:"%d bliain"},dayOfMonthOrdinalParse:/\d{1,2}(d|na|mh)/,ordinal:function(t){var e=1===t?"d":t%10==2?"na":"mh";return t+e},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("gd",{months:["Am Faoilleach","An Gearran","Am Màrt","An Giblean","An Cèitean","An t-Ògmhios","An t-Iuchar","An Lùnastal","An t-Sultain","An Dàmhair","An t-Samhain","An Dùbhlachd"],monthsShort:["Faoi","Gear","Màrt","Gibl","Cèit","Ògmh","Iuch","Lùn","Sult","Dàmh","Samh","Dùbh"],monthsParseExact:!0,weekdays:["Didòmhnaich","Diluain","Dimàirt","Diciadain","Diardaoin","Dihaoine","Disathairne"],weekdaysShort:["Did","Dil","Dim","Dic","Dia","Dih","Dis"],weekdaysMin:["Dò","Lu","Mà","Ci","Ar","Ha","Sa"],longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[An-diugh aig] LT",nextDay:"[A-màireach aig] LT",nextWeek:"dddd [aig] LT",lastDay:"[An-dè aig] LT",lastWeek:"dddd [seo chaidh] [aig] LT",sameElse:"L"},relativeTime:{future:"ann an %s",past:"bho chionn %s",s:"beagan diogan",ss:"%d diogan",m:"mionaid",mm:"%d mionaidean",h:"uair",hh:"%d uairean",d:"latha",dd:"%d latha",M:"mìos",MM:"%d mìosan",y:"bliadhna",yy:"%d bliadhna"},dayOfMonthOrdinalParse:/\d{1,2}(d|na|mh)/,ordinal:function(t){var e=1===t?"d":t%10==2?"na":"mh";return t+e},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("gl",{months:"xaneiro_febreiro_marzo_abril_maio_xuño_xullo_agosto_setembro_outubro_novembro_decembro".split("_"),monthsShort:"xan._feb._mar._abr._mai._xuñ._xul._ago._set._out._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"domingo_luns_martes_mércores_xoves_venres_sábado".split("_"),weekdaysShort:"dom._lun._mar._mér._xov._ven._sáb.".split("_"),weekdaysMin:"do_lu_ma_mé_xo_ve_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoxe "+(1!==this.hours()?"ás":"á")+"] LT"},nextDay:function(){return"[mañá "+(1!==this.hours()?"ás":"á")+"] LT"},nextWeek:function(){return"dddd ["+(1!==this.hours()?"ás":"a")+"] LT"},lastDay:function(){return"[onte "+(1!==this.hours()?"á":"a")+"] LT"},lastWeek:function(){return"[o] dddd [pasado "+(1!==this.hours()?"ás":"a")+"] LT"},sameElse:"L"},relativeTime:{future:function(t){return 0===t.indexOf("un")?"n"+t:"en "+t},past:"hai %s",s:"uns segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"unha hora",hh:"%d horas",d:"un día",dd:"%d días",M:"un mes",MM:"%d meses",y:"un ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n,r){var i={s:["thodde secondanim","thodde second"],ss:[t+" secondanim",t+" second"],m:["eka mintan","ek minute"],mm:[t+" mintanim",t+" mintam"],h:["eka voran","ek vor"],hh:[t+" voranim",t+" voram"],d:["eka disan","ek dis"],dd:[t+" disanim",t+" dis"],M:["eka mhoinean","ek mhoino"],MM:[t+" mhoineanim",t+" mhoine"],y:["eka vorsan","ek voros"],yy:[t+" vorsanim",t+" vorsam"]};return e?i[n][0]:i[n][1]}t.defineLocale("gom-latn",{months:"Janer_Febrer_Mars_Abril_Mai_Jun_Julai_Agost_Setembr_Otubr_Novembr_Dezembr".split("_"),monthsShort:"Jan._Feb._Mars_Abr._Mai_Jun_Jul._Ago._Set._Otu._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Aitar_Somar_Mongllar_Budvar_Brestar_Sukrar_Son'var".split("_"),weekdaysShort:"Ait._Som._Mon._Bud._Bre._Suk._Son.".split("_"),weekdaysMin:"Ai_Sm_Mo_Bu_Br_Su_Sn".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"A h:mm [vazta]",LTS:"A h:mm:ss [vazta]",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY A h:mm [vazta]",LLLL:"dddd, MMMM[achea] Do, YYYY, A h:mm [vazta]",llll:"ddd, D MMM YYYY, A h:mm [vazta]"},calendar:{sameDay:"[Aiz] LT",nextDay:"[Faleam] LT",nextWeek:"[Ieta to] dddd[,] LT",lastDay:"[Kal] LT",lastWeek:"[Fatlo] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%s",past:"%s adim",s:e,ss:e,m:e,mm:e,h:e,hh:e,d:e,dd:e,M:e,MM:e,y:e,yy:e},dayOfMonthOrdinalParse:/\d{1,2}(er)/,ordinal:function(t,e){switch(e){case"D":return t+"er";default:case"M":case"Q":case"DDD":case"d":case"w":case"W":return t}},week:{dow:1,doy:4},meridiemParse:/rati|sokalli|donparam|sanje/,meridiemHour:function(t,e){return 12===t&&(t=0),"rati"===e?t<4?t:t+12:"sokalli"===e?t:"donparam"===e?t>12?t:t+12:"sanje"===e?t+12:void 0},meridiem:function(t,e,n){return t<4?"rati":t<12?"sokalli":t<16?"donparam":t<20?"sanje":"rati"}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"૧",2:"૨",3:"૩",4:"૪",5:"૫",6:"૬",7:"૭",8:"૮",9:"૯",0:"૦"},n={"૧":"1","૨":"2","૩":"3","૪":"4","૫":"5","૬":"6","૭":"7","૮":"8","૯":"9","૦":"0"};t.defineLocale("gu",{months:"જાન્યુઆરી_ફેબ્રુઆરી_માર્ચ_એપ્રિલ_મે_જૂન_જુલાઈ_ઑગસ્ટ_સપ્ટેમ્બર_ઑક્ટ્બર_નવેમ્બર_ડિસેમ્બર".split("_"),monthsShort:"જાન્યુ._ફેબ્રુ._માર્ચ_એપ્રિ._મે_જૂન_જુલા._ઑગ._સપ્ટે._ઑક્ટ્._નવે._ડિસે.".split("_"),monthsParseExact:!0,weekdays:"રવિવાર_સોમવાર_મંગળવાર_બુધ્વાર_ગુરુવાર_શુક્રવાર_શનિવાર".split("_"),weekdaysShort:"રવિ_સોમ_મંગળ_બુધ્_ગુરુ_શુક્ર_શનિ".split("_"),weekdaysMin:"ર_સો_મં_બુ_ગુ_શુ_શ".split("_"),longDateFormat:{LT:"A h:mm વાગ્યે",LTS:"A h:mm:ss વાગ્યે",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm વાગ્યે",LLLL:"dddd, D MMMM YYYY, A h:mm વાગ્યે"},calendar:{sameDay:"[આજ] LT",nextDay:"[કાલે] LT",nextWeek:"dddd, LT",lastDay:"[ગઇકાલે] LT",lastWeek:"[પાછલા] dddd, LT",sameElse:"L"},relativeTime:{future:"%s મા",past:"%s પેહલા",s:"અમુક પળો",ss:"%d સેકંડ",m:"એક મિનિટ",mm:"%d મિનિટ",h:"એક કલાક",hh:"%d કલાક",d:"એક દિવસ",dd:"%d દિવસ",M:"એક મહિનો",MM:"%d મહિનો",y:"એક વર્ષ",yy:"%d વર્ષ"},preparse:function(t){return t.replace(/[૧૨૩૪૫૬૭૮૯૦]/g,function(t){return n[t]})},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]})},meridiemParse:/રાત|બપોર|સવાર|સાંજ/,meridiemHour:function(t,e){return 12===t&&(t=0),"રાત"===e?t<4?t:t+12:"સવાર"===e?t:"બપોર"===e?t>=10?t:t+12:"સાંજ"===e?t+12:void 0},meridiem:function(t,e,n){return t<4?"રાત":t<10?"સવાર":t<17?"બપોર":t<20?"સાંજ":"રાત"},week:{dow:0,doy:6}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("he",{months:"ינואר_פברואר_מרץ_אפריל_מאי_יוני_יולי_אוגוסט_ספטמבר_אוקטובר_נובמבר_דצמבר".split("_"),monthsShort:"ינו׳_פבר׳_מרץ_אפר׳_מאי_יוני_יולי_אוג׳_ספט׳_אוק׳_נוב׳_דצמ׳".split("_"),weekdays:"ראשון_שני_שלישי_רביעי_חמישי_שישי_שבת".split("_"),weekdaysShort:"א׳_ב׳_ג׳_ד׳_ה׳_ו׳_ש׳".split("_"),weekdaysMin:"א_ב_ג_ד_ה_ו_ש".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [ב]MMMM YYYY",LLL:"D [ב]MMMM YYYY HH:mm",LLLL:"dddd, D [ב]MMMM YYYY HH:mm",l:"D/M/YYYY",ll:"D MMM YYYY",lll:"D MMM YYYY HH:mm",llll:"ddd, D MMM YYYY HH:mm"},calendar:{sameDay:"[היום ב־]LT",nextDay:"[מחר ב־]LT",nextWeek:"dddd [בשעה] LT",lastDay:"[אתמול ב־]LT",lastWeek:"[ביום] dddd [האחרון בשעה] LT",sameElse:"L"},relativeTime:{future:"בעוד %s",past:"לפני %s",s:"מספר שניות",ss:"%d שניות",m:"דקה",mm:"%d דקות",h:"שעה",hh:function(t){return 2===t?"שעתיים":t+" שעות"},d:"יום",dd:function(t){return 2===t?"יומיים":t+" ימים"},M:"חודש",MM:function(t){return 2===t?"חודשיים":t+" חודשים"},y:"שנה",yy:function(t){return 2===t?"שנתיים":t%10==0&&10!==t?t+" שנה":t+" שנים"}},meridiemParse:/אחה"צ|לפנה"צ|אחרי הצהריים|לפני הצהריים|לפנות בוקר|בבוקר|בערב/i,isPM:function(t){return/^(אחה"צ|אחרי הצהריים|בערב)$/.test(t)},meridiem:function(t,e,n){return t<5?"לפנות בוקר":t<10?"בבוקר":t<12?n?'לפנה"צ':"לפני הצהריים":t<18?n?'אחה"צ':"אחרי הצהריים":"בערב"}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"},n={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"};t.defineLocale("hi",{months:"जनवरी_फ़रवरी_मार्च_अप्रैल_मई_जून_जुलाई_अगस्त_सितम्बर_अक्टूबर_नवम्बर_दिसम्बर".split("_"),monthsShort:"जन._फ़र._मार्च_अप्रै._मई_जून_जुल._अग._सित._अक्टू._नव._दिस.".split("_"),monthsParseExact:!0,weekdays:"रविवार_सोमवार_मंगलवार_बुधवार_गुरूवार_शुक्रवार_शनिवार".split("_"),weekdaysShort:"रवि_सोम_मंगल_बुध_गुरू_शुक्र_शनि".split("_"),weekdaysMin:"र_सो_मं_बु_गु_शु_श".split("_"),longDateFormat:{LT:"A h:mm बजे",LTS:"A h:mm:ss बजे",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm बजे",LLLL:"dddd, D MMMM YYYY, A h:mm बजे"},calendar:{sameDay:"[आज] LT",nextDay:"[कल] LT",nextWeek:"dddd, LT",lastDay:"[कल] LT",lastWeek:"[पिछले] dddd, LT",sameElse:"L"},relativeTime:{future:"%s में",past:"%s पहले",s:"कुछ ही क्षण",ss:"%d सेकंड",m:"एक मिनट",mm:"%d मिनट",h:"एक घंटा",hh:"%d घंटे",d:"एक दिन",dd:"%d दिन",M:"एक महीने",MM:"%d महीने",y:"एक वर्ष",yy:"%d वर्ष"},preparse:function(t){return t.replace(/[१२३४५६७८९०]/g,function(t){return n[t]})},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]})},meridiemParse:/रात|सुबह|दोपहर|शाम/,meridiemHour:function(t,e){return 12===t&&(t=0),"रात"===e?t<4?t:t+12:"सुबह"===e?t:"दोपहर"===e?t>=10?t:t+12:"शाम"===e?t+12:void 0},meridiem:function(t,e,n){return t<4?"रात":t<10?"सुबह":t<17?"दोपहर":t<20?"शाम":"रात"},week:{dow:0,doy:6}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n){var r=t+" ";switch(n){case"ss":return r+=1===t?"sekunda":2===t||3===t||4===t?"sekunde":"sekundi";case"m":return e?"jedna minuta":"jedne minute";case"mm":return r+=1===t?"minuta":2===t||3===t||4===t?"minute":"minuta";case"h":return e?"jedan sat":"jednog sata";case"hh":return r+=1===t?"sat":2===t||3===t||4===t?"sata":"sati";case"dd":return r+=1===t?"dan":"dana";case"MM":return r+=1===t?"mjesec":2===t||3===t||4===t?"mjeseca":"mjeseci";case"yy":return r+=1===t?"godina":2===t||3===t||4===t?"godine":"godina"}}t.defineLocale("hr",{months:{format:"siječnja_veljače_ožujka_travnja_svibnja_lipnja_srpnja_kolovoza_rujna_listopada_studenoga_prosinca".split("_"),standalone:"siječanj_veljača_ožujak_travanj_svibanj_lipanj_srpanj_kolovoz_rujan_listopad_studeni_prosinac".split("_")},monthsShort:"sij._velj._ožu._tra._svi._lip._srp._kol._ruj._lis._stu._pro.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[jučer u] LT",lastWeek:function(){switch(this.day()){case 0:case 3:return"[prošlu] dddd [u] LT";case 6:return"[prošle] [subote] [u] LT";case 1:case 2:case 4:case 5:return"[prošli] dddd [u] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"par sekundi",ss:e,m:e,mm:e,h:e,hh:e,d:"dan",dd:e,M:"mjesec",MM:e,y:"godinu",yy:e},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";var e="vasárnap hétfőn kedden szerdán csütörtökön pénteken szombaton".split(" ");function n(t,e,n,r){var i=t;switch(n){case"s":return r||e?"néhány másodperc":"néhány másodperce";case"ss":return i+(r||e)?" másodperc":" másodperce";case"m":return"egy"+(r||e?" perc":" perce");case"mm":return i+(r||e?" perc":" perce");case"h":return"egy"+(r||e?" óra":" órája");case"hh":return i+(r||e?" óra":" órája");case"d":return"egy"+(r||e?" nap":" napja");case"dd":return i+(r||e?" nap":" napja");case"M":return"egy"+(r||e?" hónap":" hónapja");case"MM":return i+(r||e?" hónap":" hónapja");case"y":return"egy"+(r||e?" év":" éve");case"yy":return i+(r||e?" év":" éve")}return""}function r(t){return(t?"":"[múlt] ")+"["+e[this.day()]+"] LT[-kor]"}t.defineLocale("hu",{months:"január_február_március_április_május_június_július_augusztus_szeptember_október_november_december".split("_"),monthsShort:"jan_feb_márc_ápr_máj_jún_júl_aug_szept_okt_nov_dec".split("_"),weekdays:"vasárnap_hétfő_kedd_szerda_csütörtök_péntek_szombat".split("_"),weekdaysShort:"vas_hét_kedd_sze_csüt_pén_szo".split("_"),weekdaysMin:"v_h_k_sze_cs_p_szo".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"YYYY.MM.DD.",LL:"YYYY. MMMM D.",LLL:"YYYY. MMMM D. H:mm",LLLL:"YYYY. MMMM D., dddd H:mm"},meridiemParse:/de|du/i,isPM:function(t){return"u"===t.charAt(1).toLowerCase()},meridiem:function(t,e,n){return t<12?!0===n?"de":"DE":!0===n?"du":"DU"},calendar:{sameDay:"[ma] LT[-kor]",nextDay:"[holnap] LT[-kor]",nextWeek:function(){return r.call(this,!0)},lastDay:"[tegnap] LT[-kor]",lastWeek:function(){return r.call(this,!1)},sameElse:"L"},relativeTime:{future:"%s múlva",past:"%s",s:n,ss:n,m:n,mm:n,h:n,hh:n,d:n,dd:n,M:n,MM:n,y:n,yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("hy-am",{months:{format:"հունվարի_փետրվարի_մարտի_ապրիլի_մայիսի_հունիսի_հուլիսի_օգոստոսի_սեպտեմբերի_հոկտեմբերի_նոյեմբերի_դեկտեմբերի".split("_"),standalone:"հունվար_փետրվար_մարտ_ապրիլ_մայիս_հունիս_հուլիս_օգոստոս_սեպտեմբեր_հոկտեմբեր_նոյեմբեր_դեկտեմբեր".split("_")},monthsShort:"հնվ_փտր_մրտ_ապր_մյս_հնս_հլս_օգս_սպտ_հկտ_նմբ_դկտ".split("_"),weekdays:"կիրակի_երկուշաբթի_երեքշաբթի_չորեքշաբթի_հինգշաբթի_ուրբաթ_շաբաթ".split("_"),weekdaysShort:"կրկ_երկ_երք_չրք_հնգ_ուրբ_շբթ".split("_"),weekdaysMin:"կրկ_երկ_երք_չրք_հնգ_ուրբ_շբթ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY թ.",LLL:"D MMMM YYYY թ., HH:mm",LLLL:"dddd, D MMMM YYYY թ., HH:mm"},calendar:{sameDay:"[այսօր] LT",nextDay:"[վաղը] LT",lastDay:"[երեկ] LT",nextWeek:function(){return"dddd [օրը ժամը] LT"},lastWeek:function(){return"[անցած] dddd [օրը ժամը] LT"},sameElse:"L"},relativeTime:{future:"%s հետո",past:"%s առաջ",s:"մի քանի վայրկյան",ss:"%d վայրկյան",m:"րոպե",mm:"%d րոպե",h:"ժամ",hh:"%d ժամ",d:"օր",dd:"%d օր",M:"ամիս",MM:"%d ամիս",y:"տարի",yy:"%d տարի"},meridiemParse:/գիշերվա|առավոտվա|ցերեկվա|երեկոյան/,isPM:function(t){return/^(ցերեկվա|երեկոյան)$/.test(t)},meridiem:function(t){return t<4?"գիշերվա":t<12?"առավոտվա":t<17?"ցերեկվա":"երեկոյան"},dayOfMonthOrdinalParse:/\d{1,2}|\d{1,2}-(ին|րդ)/,ordinal:function(t,e){switch(e){case"DDD":case"w":case"W":case"DDDo":return 1===t?t+"-ին":t+"-րդ";default:return t}},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("id",{months:"Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mei_Jun_Jul_Agt_Sep_Okt_Nov_Des".split("_"),weekdays:"Minggu_Senin_Selasa_Rabu_Kamis_Jumat_Sabtu".split("_"),weekdaysShort:"Min_Sen_Sel_Rab_Kam_Jum_Sab".split("_"),weekdaysMin:"Mg_Sn_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|siang|sore|malam/,meridiemHour:function(t,e){return 12===t&&(t=0),"pagi"===e?t:"siang"===e?t>=11?t:t+12:"sore"===e||"malam"===e?t+12:void 0},meridiem:function(t,e,n){return t<11?"pagi":t<15?"siang":t<19?"sore":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Besok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kemarin pukul] LT",lastWeek:"dddd [lalu pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lalu",s:"beberapa detik",ss:"%d detik",m:"semenit",mm:"%d menit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t){return t%100==11||t%10!=1}function n(t,n,r,i){var o=t+" ";switch(r){case"s":return n||i?"nokkrar sekúndur":"nokkrum sekúndum";case"ss":return e(t)?o+(n||i?"sekúndur":"sekúndum"):o+"sekúnda";case"m":return n?"mínúta":"mínútu";case"mm":return e(t)?o+(n||i?"mínútur":"mínútum"):n?o+"mínúta":o+"mínútu";case"hh":return e(t)?o+(n||i?"klukkustundir":"klukkustundum"):o+"klukkustund";case"d":return n?"dagur":i?"dag":"degi";case"dd":return e(t)?n?o+"dagar":o+(i?"daga":"dögum"):n?o+"dagur":o+(i?"dag":"degi");case"M":return n?"mánuður":i?"mánuð":"mánuði";case"MM":return e(t)?n?o+"mánuðir":o+(i?"mánuði":"mánuðum"):n?o+"mánuður":o+(i?"mánuð":"mánuði");case"y":return n||i?"ár":"ári";case"yy":return e(t)?o+(n||i?"ár":"árum"):o+(n||i?"ár":"ári")}}t.defineLocale("is",{months:"janúar_febrúar_mars_apríl_maí_júní_júlí_ágúst_september_október_nóvember_desember".split("_"),monthsShort:"jan_feb_mar_apr_maí_jún_júl_ágú_sep_okt_nóv_des".split("_"),weekdays:"sunnudagur_mánudagur_þriðjudagur_miðvikudagur_fimmtudagur_föstudagur_laugardagur".split("_"),weekdaysShort:"sun_mán_þri_mið_fim_fös_lau".split("_"),weekdaysMin:"Su_Má_Þr_Mi_Fi_Fö_La".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] H:mm",LLLL:"dddd, D. MMMM YYYY [kl.] H:mm"},calendar:{sameDay:"[í dag kl.] LT",nextDay:"[á morgun kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[í gær kl.] LT",lastWeek:"[síðasta] dddd [kl.] LT",sameElse:"L"},relativeTime:{future:"eftir %s",past:"fyrir %s síðan",s:n,ss:n,m:n,mm:n,h:"klukkustund",hh:n,d:n,dd:n,M:n,MM:n,y:n,yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("it",{months:"gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre".split("_"),monthsShort:"gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic".split("_"),weekdays:"domenica_lunedì_martedì_mercoledì_giovedì_venerdì_sabato".split("_"),weekdaysShort:"dom_lun_mar_mer_gio_ven_sab".split("_"),weekdaysMin:"do_lu_ma_me_gi_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Oggi alle] LT",nextDay:"[Domani alle] LT",nextWeek:"dddd [alle] LT",lastDay:"[Ieri alle] LT",lastWeek:function(){switch(this.day()){case 0:return"[la scorsa] dddd [alle] LT";default:return"[lo scorso] dddd [alle] LT"}},sameElse:"L"},relativeTime:{future:function(t){return(/^[0-9].+$/.test(t)?"tra":"in")+" "+t},past:"%s fa",s:"alcuni secondi",ss:"%d secondi",m:"un minuto",mm:"%d minuti",h:"un'ora",hh:"%d ore",d:"un giorno",dd:"%d giorni",M:"un mese",MM:"%d mesi",y:"un anno",yy:"%d anni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("it-ch",{months:"gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre".split("_"),monthsShort:"gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic".split("_"),weekdays:"domenica_lunedì_martedì_mercoledì_giovedì_venerdì_sabato".split("_"),weekdaysShort:"dom_lun_mar_mer_gio_ven_sab".split("_"),weekdaysMin:"do_lu_ma_me_gi_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Oggi alle] LT",nextDay:"[Domani alle] LT",nextWeek:"dddd [alle] LT",lastDay:"[Ieri alle] LT",lastWeek:function(){switch(this.day()){case 0:return"[la scorsa] dddd [alle] LT";default:return"[lo scorso] dddd [alle] LT"}},sameElse:"L"},relativeTime:{future:function(t){return(/^[0-9].+$/.test(t)?"tra":"in")+" "+t},past:"%s fa",s:"alcuni secondi",ss:"%d secondi",m:"un minuto",mm:"%d minuti",h:"un'ora",hh:"%d ore",d:"un giorno",dd:"%d giorni",M:"un mese",MM:"%d mesi",y:"un anno",yy:"%d anni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ja",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"日曜日_月曜日_火曜日_水曜日_木曜日_金曜日_土曜日".split("_"),weekdaysShort:"日_月_火_水_木_金_土".split("_"),weekdaysMin:"日_月_火_水_木_金_土".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日 dddd HH:mm",l:"YYYY/MM/DD",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日(ddd) HH:mm"},meridiemParse:/午前|午後/i,isPM:function(t){return"午後"===t},meridiem:function(t,e,n){return t<12?"午前":"午後"},calendar:{sameDay:"[今日] LT",nextDay:"[明日] LT",nextWeek:function(t){return t.week()<this.week()?"[来週]dddd LT":"dddd LT"},lastDay:"[昨日] LT",lastWeek:function(t){return this.week()<t.week()?"[先週]dddd LT":"dddd LT"},sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}日/,ordinal:function(t,e){switch(e){case"d":case"D":case"DDD":return t+"日";default:return t}},relativeTime:{future:"%s後",past:"%s前",s:"数秒",ss:"%d秒",m:"1分",mm:"%d分",h:"1時間",hh:"%d時間",d:"1日",dd:"%d日",M:"1ヶ月",MM:"%dヶ月",y:"1年",yy:"%d年"}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("jv",{months:"Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_Nopember_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mei_Jun_Jul_Ags_Sep_Okt_Nop_Des".split("_"),weekdays:"Minggu_Senen_Seloso_Rebu_Kemis_Jemuwah_Septu".split("_"),weekdaysShort:"Min_Sen_Sel_Reb_Kem_Jem_Sep".split("_"),weekdaysMin:"Mg_Sn_Sl_Rb_Km_Jm_Sp".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/enjing|siyang|sonten|ndalu/,meridiemHour:function(t,e){return 12===t&&(t=0),"enjing"===e?t:"siyang"===e?t>=11?t:t+12:"sonten"===e||"ndalu"===e?t+12:void 0},meridiem:function(t,e,n){return t<11?"enjing":t<15?"siyang":t<19?"sonten":"ndalu"},calendar:{sameDay:"[Dinten puniko pukul] LT",nextDay:"[Mbenjang pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kala wingi pukul] LT",lastWeek:"dddd [kepengker pukul] LT",sameElse:"L"},relativeTime:{future:"wonten ing %s",past:"%s ingkang kepengker",s:"sawetawis detik",ss:"%d detik",m:"setunggal menit",mm:"%d menit",h:"setunggal jam",hh:"%d jam",d:"sedinten",dd:"%d dinten",M:"sewulan",MM:"%d wulan",y:"setaun",yy:"%d taun"},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ka",{months:{standalone:"იანვარი_თებერვალი_მარტი_აპრილი_მაისი_ივნისი_ივლისი_აგვისტო_სექტემბერი_ოქტომბერი_ნოემბერი_დეკემბერი".split("_"),format:"იანვარს_თებერვალს_მარტს_აპრილის_მაისს_ივნისს_ივლისს_აგვისტს_სექტემბერს_ოქტომბერს_ნოემბერს_დეკემბერს".split("_")},monthsShort:"იან_თებ_მარ_აპრ_მაი_ივნ_ივლ_აგვ_სექ_ოქტ_ნოე_დეკ".split("_"),weekdays:{standalone:"კვირა_ორშაბათი_სამშაბათი_ოთხშაბათი_ხუთშაბათი_პარასკევი_შაბათი".split("_"),format:"კვირას_ორშაბათს_სამშაბათს_ოთხშაბათს_ხუთშაბათს_პარასკევს_შაბათს".split("_"),isFormat:/(წინა|შემდეგ)/},weekdaysShort:"კვი_ორშ_სამ_ოთხ_ხუთ_პარ_შაბ".split("_"),weekdaysMin:"კვ_ორ_სა_ოთ_ხუ_პა_შა".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[დღეს] LT[-ზე]",nextDay:"[ხვალ] LT[-ზე]",lastDay:"[გუშინ] LT[-ზე]",nextWeek:"[შემდეგ] dddd LT[-ზე]",lastWeek:"[წინა] dddd LT-ზე",sameElse:"L"},relativeTime:{future:function(t){return/(წამი|წუთი|საათი|წელი)/.test(t)?t.replace(/ი$/,"ში"):t+"ში"},past:function(t){return/(წამი|წუთი|საათი|დღე|თვე)/.test(t)?t.replace(/(ი|ე)$/,"ის წინ"):/წელი/.test(t)?t.replace(/წელი$/,"წლის წინ"):void 0},s:"რამდენიმე წამი",ss:"%d წამი",m:"წუთი",mm:"%d წუთი",h:"საათი",hh:"%d საათი",d:"დღე",dd:"%d დღე",M:"თვე",MM:"%d თვე",y:"წელი",yy:"%d წელი"},dayOfMonthOrdinalParse:/0|1-ლი|მე-\d{1,2}|\d{1,2}-ე/,ordinal:function(t){return 0===t?t:1===t?t+"-ლი":t<20||t<=100&&t%20==0||t%100==0?"მე-"+t:t+"-ე"},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={0:"-ші",1:"-ші",2:"-ші",3:"-ші",4:"-ші",5:"-ші",6:"-шы",7:"-ші",8:"-ші",9:"-шы",10:"-шы",20:"-шы",30:"-шы",40:"-шы",50:"-ші",60:"-шы",70:"-ші",80:"-ші",90:"-шы",100:"-ші"};t.defineLocale("kk",{months:"қаңтар_ақпан_наурыз_сәуір_мамыр_маусым_шілде_тамыз_қыркүйек_қазан_қараша_желтоқсан".split("_"),monthsShort:"қаң_ақп_нау_сәу_мам_мау_шіл_там_қыр_қаз_қар_жел".split("_"),weekdays:"жексенбі_дүйсенбі_сейсенбі_сәрсенбі_бейсенбі_жұма_сенбі".split("_"),weekdaysShort:"жек_дүй_сей_сәр_бей_жұм_сен".split("_"),weekdaysMin:"жк_дй_сй_ср_бй_жм_сн".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Бүгін сағат] LT",nextDay:"[Ертең сағат] LT",nextWeek:"dddd [сағат] LT",lastDay:"[Кеше сағат] LT",lastWeek:"[Өткен аптаның] dddd [сағат] LT",sameElse:"L"},relativeTime:{future:"%s ішінде",past:"%s бұрын",s:"бірнеше секунд",ss:"%d секунд",m:"бір минут",mm:"%d минут",h:"бір сағат",hh:"%d сағат",d:"бір күн",dd:"%d күн",M:"бір ай",MM:"%d ай",y:"бір жыл",yy:"%d жыл"},dayOfMonthOrdinalParse:/\d{1,2}-(ші|шы)/,ordinal:function(t){var n=t%10,r=t>=100?100:null;return t+(e[t]||e[n]||e[r])},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"១",2:"២",3:"៣",4:"៤",5:"៥",6:"៦",7:"៧",8:"៨",9:"៩",0:"០"},n={"១":"1","២":"2","៣":"3","៤":"4","៥":"5","៦":"6","៧":"7","៨":"8","៩":"9","០":"0"};t.defineLocale("km",{months:"មករា_កុម្ភៈ_មីនា_មេសា_ឧសភា_មិថុនា_កក្កដា_សីហា_កញ្ញា_តុលា_វិច្ឆិកា_ធ្នូ".split("_"),monthsShort:"មករា_កុម្ភៈ_មីនា_មេសា_ឧសភា_មិថុនា_កក្កដា_សីហា_កញ្ញា_តុលា_វិច្ឆិកា_ធ្នូ".split("_"),weekdays:"អាទិត្យ_ច័ន្ទ_អង្គារ_ពុធ_ព្រហស្បតិ៍_សុក្រ_សៅរ៍".split("_"),weekdaysShort:"អា_ច_អ_ព_ព្រ_សុ_ស".split("_"),weekdaysMin:"អា_ច_អ_ព_ព្រ_សុ_ស".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/ព្រឹក|ល្ងាច/,isPM:function(t){return"ល្ងាច"===t},meridiem:function(t,e,n){return t<12?"ព្រឹក":"ល្ងាច"},calendar:{sameDay:"[ថ្ងៃនេះ ម៉ោង] LT",nextDay:"[ស្អែក ម៉ោង] LT",nextWeek:"dddd [ម៉ោង] LT",lastDay:"[ម្សិលមិញ ម៉ោង] LT",lastWeek:"dddd [សប្តាហ៍មុន] [ម៉ោង] LT",sameElse:"L"},relativeTime:{future:"%sទៀត",past:"%sមុន",s:"ប៉ុន្មានវិនាទី",ss:"%d វិនាទី",m:"មួយនាទី",mm:"%d នាទី",h:"មួយម៉ោង",hh:"%d ម៉ោង",d:"មួយថ្ងៃ",dd:"%d ថ្ងៃ",M:"មួយខែ",MM:"%d ខែ",y:"មួយឆ្នាំ",yy:"%d ឆ្នាំ"},dayOfMonthOrdinalParse:/ទី\d{1,2}/,ordinal:"ទី%d",preparse:function(t){return t.replace(/[១២៣៤៥៦៧៨៩០]/g,function(t){return n[t]})},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]})},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"೧",2:"೨",3:"೩",4:"೪",5:"೫",6:"೬",7:"೭",8:"೮",9:"೯",0:"೦"},n={"೧":"1","೨":"2","೩":"3","೪":"4","೫":"5","೬":"6","೭":"7","೮":"8","೯":"9","೦":"0"};t.defineLocale("kn",{months:"ಜನವರಿ_ಫೆಬ್ರವರಿ_ಮಾರ್ಚ್_ಏಪ್ರಿಲ್_ಮೇ_ಜೂನ್_ಜುಲೈ_ಆಗಸ್ಟ್_ಸೆಪ್ಟೆಂಬರ್_ಅಕ್ಟೋಬರ್_ನವೆಂಬರ್_ಡಿಸೆಂಬರ್".split("_"),monthsShort:"ಜನ_ಫೆಬ್ರ_ಮಾರ್ಚ್_ಏಪ್ರಿಲ್_ಮೇ_ಜೂನ್_ಜುಲೈ_ಆಗಸ್ಟ್_ಸೆಪ್ಟೆಂ_ಅಕ್ಟೋ_ನವೆಂ_ಡಿಸೆಂ".split("_"),monthsParseExact:!0,weekdays:"ಭಾನುವಾರ_ಸೋಮವಾರ_ಮಂಗಳವಾರ_ಬುಧವಾರ_ಗುರುವಾರ_ಶುಕ್ರವಾರ_ಶನಿವಾರ".split("_"),weekdaysShort:"ಭಾನು_ಸೋಮ_ಮಂಗಳ_ಬುಧ_ಗುರು_ಶುಕ್ರ_ಶನಿ".split("_"),weekdaysMin:"ಭಾ_ಸೋ_ಮಂ_ಬು_ಗು_ಶು_ಶ".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[ಇಂದು] LT",nextDay:"[ನಾಳೆ] LT",nextWeek:"dddd, LT",lastDay:"[ನಿನ್ನೆ] LT",lastWeek:"[ಕೊನೆಯ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ನಂತರ",past:"%s ಹಿಂದೆ",s:"ಕೆಲವು ಕ್ಷಣಗಳು",ss:"%d ಸೆಕೆಂಡುಗಳು",m:"ಒಂದು ನಿಮಿಷ",mm:"%d ನಿಮಿಷ",h:"ಒಂದು ಗಂಟೆ",hh:"%d ಗಂಟೆ",d:"ಒಂದು ದಿನ",dd:"%d ದಿನ",M:"ಒಂದು ತಿಂಗಳು",MM:"%d ತಿಂಗಳು",y:"ಒಂದು ವರ್ಷ",yy:"%d ವರ್ಷ"},preparse:function(t){return t.replace(/[೧೨೩೪೫೬೭೮೯೦]/g,function(t){return n[t]})},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]})},meridiemParse:/ರಾತ್ರಿ|ಬೆಳಿಗ್ಗೆ|ಮಧ್ಯಾಹ್ನ|ಸಂಜೆ/,meridiemHour:function(t,e){return 12===t&&(t=0),"ರಾತ್ರಿ"===e?t<4?t:t+12:"ಬೆಳಿಗ್ಗೆ"===e?t:"ಮಧ್ಯಾಹ್ನ"===e?t>=10?t:t+12:"ಸಂಜೆ"===e?t+12:void 0},meridiem:function(t,e,n){return t<4?"ರಾತ್ರಿ":t<10?"ಬೆಳಿಗ್ಗೆ":t<17?"ಮಧ್ಯಾಹ್ನ":t<20?"ಸಂಜೆ":"ರಾತ್ರಿ"},dayOfMonthOrdinalParse:/\d{1,2}(ನೇ)/,ordinal:function(t){return t+"ನೇ"},week:{dow:0,doy:6}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ko",{months:"1월_2월_3월_4월_5월_6월_7월_8월_9월_10월_11월_12월".split("_"),monthsShort:"1월_2월_3월_4월_5월_6월_7월_8월_9월_10월_11월_12월".split("_"),weekdays:"일요일_월요일_화요일_수요일_목요일_금요일_토요일".split("_"),weekdaysShort:"일_월_화_수_목_금_토".split("_"),weekdaysMin:"일_월_화_수_목_금_토".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"YYYY.MM.DD.",LL:"YYYY년 MMMM D일",LLL:"YYYY년 MMMM D일 A h:mm",LLLL:"YYYY년 MMMM D일 dddd A h:mm",l:"YYYY.MM.DD.",ll:"YYYY년 MMMM D일",lll:"YYYY년 MMMM D일 A h:mm",llll:"YYYY년 MMMM D일 dddd A h:mm"},calendar:{sameDay:"오늘 LT",nextDay:"내일 LT",nextWeek:"dddd LT",lastDay:"어제 LT",lastWeek:"지난주 dddd LT",sameElse:"L"},relativeTime:{future:"%s 후",past:"%s 전",s:"몇 초",ss:"%d초",m:"1분",mm:"%d분",h:"한 시간",hh:"%d시간",d:"하루",dd:"%d일",M:"한 달",MM:"%d달",y:"일 년",yy:"%d년"},dayOfMonthOrdinalParse:/\d{1,2}(일|월|주)/,ordinal:function(t,e){switch(e){case"d":case"D":case"DDD":return t+"일";case"M":return t+"월";case"w":case"W":return t+"주";default:return t}},meridiemParse:/오전|오후/,isPM:function(t){return"오후"===t},meridiem:function(t,e,n){return t<12?"오전":"오후"}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"},n={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"},r=["کانونی دووەم","شوبات","ئازار","نیسان","ئایار","حوزەیران","تەمموز","ئاب","ئەیلوول","تشرینی یەكەم","تشرینی دووەم","كانونی یەکەم"];t.defineLocale("ku",{months:r,monthsShort:r,weekdays:"یه‌كشه‌ممه‌_دووشه‌ممه‌_سێشه‌ممه‌_چوارشه‌ممه‌_پێنجشه‌ممه‌_هه‌ینی_شه‌ممه‌".split("_"),weekdaysShort:"یه‌كشه‌م_دووشه‌م_سێشه‌م_چوارشه‌م_پێنجشه‌م_هه‌ینی_شه‌ممه‌".split("_"),weekdaysMin:"ی_د_س_چ_پ_ه_ش".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/ئێواره‌|به‌یانی/,isPM:function(t){return/ئێواره‌/.test(t)},meridiem:function(t,e,n){return t<12?"به‌یانی":"ئێواره‌"},calendar:{sameDay:"[ئه‌مرۆ كاتژمێر] LT",nextDay:"[به‌یانی كاتژمێر] LT",nextWeek:"dddd [كاتژمێر] LT",lastDay:"[دوێنێ كاتژمێر] LT",lastWeek:"dddd [كاتژمێر] LT",sameElse:"L"},relativeTime:{future:"له‌ %s",past:"%s",s:"چه‌ند چركه‌یه‌ك",ss:"چركه‌ %d",m:"یه‌ك خوله‌ك",mm:"%d خوله‌ك",h:"یه‌ك كاتژمێر",hh:"%d كاتژمێر",d:"یه‌ك ڕۆژ",dd:"%d ڕۆژ",M:"یه‌ك مانگ",MM:"%d مانگ",y:"یه‌ك ساڵ",yy:"%d ساڵ"},preparse:function(t){return t.replace(/[١٢٣٤٥٦٧٨٩٠]/g,function(t){return n[t]}).replace(/،/g,",")},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]}).replace(/,/g,"،")},week:{dow:6,doy:12}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={0:"-чү",1:"-чи",2:"-чи",3:"-чү",4:"-чү",5:"-чи",6:"-чы",7:"-чи",8:"-чи",9:"-чу",10:"-чу",20:"-чы",30:"-чу",40:"-чы",50:"-чү",60:"-чы",70:"-чи",80:"-чи",90:"-чу",100:"-чү"};t.defineLocale("ky",{months:"январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь".split("_"),monthsShort:"янв_фев_март_апр_май_июнь_июль_авг_сен_окт_ноя_дек".split("_"),weekdays:"Жекшемби_Дүйшөмбү_Шейшемби_Шаршемби_Бейшемби_Жума_Ишемби".split("_"),weekdaysShort:"Жек_Дүй_Шей_Шар_Бей_Жум_Ише".split("_"),weekdaysMin:"Жк_Дй_Шй_Шр_Бй_Жм_Иш".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Бүгүн саат] LT",nextDay:"[Эртең саат] LT",nextWeek:"dddd [саат] LT",lastDay:"[Кечээ саат] LT",lastWeek:"[Өткөн аптанын] dddd [күнү] [саат] LT",sameElse:"L"},relativeTime:{future:"%s ичинде",past:"%s мурун",s:"бирнече секунд",ss:"%d секунд",m:"бир мүнөт",mm:"%d мүнөт",h:"бир саат",hh:"%d саат",d:"бир күн",dd:"%d күн",M:"бир ай",MM:"%d ай",y:"бир жыл",yy:"%d жыл"},dayOfMonthOrdinalParse:/\d{1,2}-(чи|чы|чү|чу)/,ordinal:function(t){var n=t%10,r=t>=100?100:null;return t+(e[t]||e[n]||e[r])},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n,r){var i={m:["eng Minutt","enger Minutt"],h:["eng Stonn","enger Stonn"],d:["een Dag","engem Dag"],M:["ee Mount","engem Mount"],y:["ee Joer","engem Joer"]};return e?i[n][0]:i[n][1]}function n(t){if(t=parseInt(t,10),isNaN(t))return!1;if(t<0)return!0;if(t<10)return 4<=t&&t<=7;if(t<100){var e=t%10,r=t/10;return n(0===e?r:e)}if(t<1e4){for(;t>=10;)t/=10;return n(t)}return n(t/=1e3)}t.defineLocale("lb",{months:"Januar_Februar_Mäerz_Abrëll_Mee_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Febr._Mrz._Abr._Mee_Jun._Jul._Aug._Sept._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonndeg_Méindeg_Dënschdeg_Mëttwoch_Donneschdeg_Freideg_Samschdeg".split("_"),weekdaysShort:"So._Mé._Dë._Më._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mé_Dë_Më_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm [Auer]",LTS:"H:mm:ss [Auer]",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm [Auer]",LLLL:"dddd, D. MMMM YYYY H:mm [Auer]"},calendar:{sameDay:"[Haut um] LT",sameElse:"L",nextDay:"[Muer um] LT",nextWeek:"dddd [um] LT",lastDay:"[Gëschter um] LT",lastWeek:function(){switch(this.day()){case 2:case 4:return"[Leschten] dddd [um] LT";default:return"[Leschte] dddd [um] LT"}}},relativeTime:{future:function(t){return n(t.substr(0,t.indexOf(" ")))?"a "+t:"an "+t},past:function(t){return n(t.substr(0,t.indexOf(" ")))?"viru "+t:"virun "+t},s:"e puer Sekonnen",ss:"%d Sekonnen",m:e,mm:"%d Minutten",h:e,hh:"%d Stonnen",d:e,dd:"%d Deeg",M:e,MM:"%d Méint",y:e,yy:"%d Joer"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("lo",{months:"ມັງກອນ_ກຸມພາ_ມີນາ_ເມສາ_ພຶດສະພາ_ມິຖຸນາ_ກໍລະກົດ_ສິງຫາ_ກັນຍາ_ຕຸລາ_ພະຈິກ_ທັນວາ".split("_"),monthsShort:"ມັງກອນ_ກຸມພາ_ມີນາ_ເມສາ_ພຶດສະພາ_ມິຖຸນາ_ກໍລະກົດ_ສິງຫາ_ກັນຍາ_ຕຸລາ_ພະຈິກ_ທັນວາ".split("_"),weekdays:"ອາທິດ_ຈັນ_ອັງຄານ_ພຸດ_ພະຫັດ_ສຸກ_ເສົາ".split("_"),weekdaysShort:"ທິດ_ຈັນ_ອັງຄານ_ພຸດ_ພະຫັດ_ສຸກ_ເສົາ".split("_"),weekdaysMin:"ທ_ຈ_ອຄ_ພ_ພຫ_ສກ_ສ".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"ວັນdddd D MMMM YYYY HH:mm"},meridiemParse:/ຕອນເຊົ້າ|ຕອນແລງ/,isPM:function(t){return"ຕອນແລງ"===t},meridiem:function(t,e,n){return t<12?"ຕອນເຊົ້າ":"ຕອນແລງ"},calendar:{sameDay:"[ມື້ນີ້ເວລາ] LT",nextDay:"[ມື້ອື່ນເວລາ] LT",nextWeek:"[ວັນ]dddd[ໜ້າເວລາ] LT",lastDay:"[ມື້ວານນີ້ເວລາ] LT",lastWeek:"[ວັນ]dddd[ແລ້ວນີ້ເວລາ] LT",sameElse:"L"},relativeTime:{future:"ອີກ %s",past:"%sຜ່ານມາ",s:"ບໍ່ເທົ່າໃດວິນາທີ",ss:"%d ວິນາທີ",m:"1 ນາທີ",mm:"%d ນາທີ",h:"1 ຊົ່ວໂມງ",hh:"%d ຊົ່ວໂມງ",d:"1 ມື້",dd:"%d ມື້",M:"1 ເດືອນ",MM:"%d ເດືອນ",y:"1 ປີ",yy:"%d ປີ"},dayOfMonthOrdinalParse:/(ທີ່)\d{1,2}/,ordinal:function(t){return"ທີ່"+t}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={ss:"sekundė_sekundžių_sekundes",m:"minutė_minutės_minutę",mm:"minutės_minučių_minutes",h:"valanda_valandos_valandą",hh:"valandos_valandų_valandas",d:"diena_dienos_dieną",dd:"dienos_dienų_dienas",M:"mėnuo_mėnesio_mėnesį",MM:"mėnesiai_mėnesių_mėnesius",y:"metai_metų_metus",yy:"metai_metų_metus"};function n(t,e,n,r){return e?i(n)[0]:r?i(n)[1]:i(n)[2]}function r(t){return t%10==0||t>10&&t<20}function i(t){return e[t].split("_")}function o(t,e,o,a){var s=t+" ";return 1===t?s+n(0,e,o[0],a):e?s+(r(t)?i(o)[1]:i(o)[0]):a?s+i(o)[1]:s+(r(t)?i(o)[1]:i(o)[2])}t.defineLocale("lt",{months:{format:"sausio_vasario_kovo_balandžio_gegužės_birželio_liepos_rugpjūčio_rugsėjo_spalio_lapkričio_gruodžio".split("_"),standalone:"sausis_vasaris_kovas_balandis_gegužė_birželis_liepa_rugpjūtis_rugsėjis_spalis_lapkritis_gruodis".split("_"),isFormat:/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?|MMMM?(\[[^\[\]]*\]|\s)+D[oD]?/},monthsShort:"sau_vas_kov_bal_geg_bir_lie_rgp_rgs_spa_lap_grd".split("_"),weekdays:{format:"sekmadienį_pirmadienį_antradienį_trečiadienį_ketvirtadienį_penktadienį_šeštadienį".split("_"),standalone:"sekmadienis_pirmadienis_antradienis_trečiadienis_ketvirtadienis_penktadienis_šeštadienis".split("_"),isFormat:/dddd HH:mm/},weekdaysShort:"Sek_Pir_Ant_Tre_Ket_Pen_Šeš".split("_"),weekdaysMin:"S_P_A_T_K_Pn_Š".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY [m.] MMMM D [d.]",LLL:"YYYY [m.] MMMM D [d.], HH:mm [val.]",LLLL:"YYYY [m.] MMMM D [d.], dddd, HH:mm [val.]",l:"YYYY-MM-DD",ll:"YYYY [m.] MMMM D [d.]",lll:"YYYY [m.] MMMM D [d.], HH:mm [val.]",llll:"YYYY [m.] MMMM D [d.], ddd, HH:mm [val.]"},calendar:{sameDay:"[Šiandien] LT",nextDay:"[Rytoj] LT",nextWeek:"dddd LT",lastDay:"[Vakar] LT",lastWeek:"[Praėjusį] dddd LT",sameElse:"L"},relativeTime:{future:"po %s",past:"prieš %s",s:function(t,e,n,r){return e?"kelios sekundės":r?"kelių sekundžių":"kelias sekundes"},ss:o,m:n,mm:o,h:n,hh:o,d:n,dd:o,M:n,MM:o,y:n,yy:o},dayOfMonthOrdinalParse:/\d{1,2}-oji/,ordinal:function(t){return t+"-oji"},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={ss:"sekundes_sekundēm_sekunde_sekundes".split("_"),m:"minūtes_minūtēm_minūte_minūtes".split("_"),mm:"minūtes_minūtēm_minūte_minūtes".split("_"),h:"stundas_stundām_stunda_stundas".split("_"),hh:"stundas_stundām_stunda_stundas".split("_"),d:"dienas_dienām_diena_dienas".split("_"),dd:"dienas_dienām_diena_dienas".split("_"),M:"mēneša_mēnešiem_mēnesis_mēneši".split("_"),MM:"mēneša_mēnešiem_mēnesis_mēneši".split("_"),y:"gada_gadiem_gads_gadi".split("_"),yy:"gada_gadiem_gads_gadi".split("_")};function n(t,e,n){return n?e%10==1&&e%100!=11?t[2]:t[3]:e%10==1&&e%100!=11?t[0]:t[1]}function r(t,r,i){return t+" "+n(e[i],t,r)}function i(t,r,i){return n(e[i],t,r)}t.defineLocale("lv",{months:"janvāris_februāris_marts_aprīlis_maijs_jūnijs_jūlijs_augusts_septembris_oktobris_novembris_decembris".split("_"),monthsShort:"jan_feb_mar_apr_mai_jūn_jūl_aug_sep_okt_nov_dec".split("_"),weekdays:"svētdiena_pirmdiena_otrdiena_trešdiena_ceturtdiena_piektdiena_sestdiena".split("_"),weekdaysShort:"Sv_P_O_T_C_Pk_S".split("_"),weekdaysMin:"Sv_P_O_T_C_Pk_S".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY.",LL:"YYYY. [gada] D. MMMM",LLL:"YYYY. [gada] D. MMMM, HH:mm",LLLL:"YYYY. [gada] D. MMMM, dddd, HH:mm"},calendar:{sameDay:"[Šodien pulksten] LT",nextDay:"[Rīt pulksten] LT",nextWeek:"dddd [pulksten] LT",lastDay:"[Vakar pulksten] LT",lastWeek:"[Pagājušā] dddd [pulksten] LT",sameElse:"L"},relativeTime:{future:"pēc %s",past:"pirms %s",s:function(t,e){return e?"dažas sekundes":"dažām sekundēm"},ss:r,m:i,mm:r,h:i,hh:r,d:i,dd:r,M:i,MM:r,y:i,yy:r},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={words:{ss:["sekund","sekunda","sekundi"],m:["jedan minut","jednog minuta"],mm:["minut","minuta","minuta"],h:["jedan sat","jednog sata"],hh:["sat","sata","sati"],dd:["dan","dana","dana"],MM:["mjesec","mjeseca","mjeseci"],yy:["godina","godine","godina"]},correctGrammaticalCase:function(t,e){return 1===t?e[0]:t>=2&&t<=4?e[1]:e[2]},translate:function(t,n,r){var i=e.words[r];return 1===r.length?n?i[0]:i[1]:t+" "+e.correctGrammaticalCase(t,i)}};t.defineLocale("me",{months:"januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sjutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[juče u] LT",lastWeek:function(){return["[prošle] [nedjelje] [u] LT","[prošlog] [ponedjeljka] [u] LT","[prošlog] [utorka] [u] LT","[prošle] [srijede] [u] LT","[prošlog] [četvrtka] [u] LT","[prošlog] [petka] [u] LT","[prošle] [subote] [u] LT"][this.day()]},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"nekoliko sekundi",ss:e.translate,m:e.translate,mm:e.translate,h:e.translate,hh:e.translate,d:"dan",dd:e.translate,M:"mjesec",MM:e.translate,y:"godinu",yy:e.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("mi",{months:"Kohi-tāte_Hui-tanguru_Poutū-te-rangi_Paenga-whāwhā_Haratua_Pipiri_Hōngoingoi_Here-turi-kōkā_Mahuru_Whiringa-ā-nuku_Whiringa-ā-rangi_Hakihea".split("_"),monthsShort:"Kohi_Hui_Pou_Pae_Hara_Pipi_Hōngoi_Here_Mahu_Whi-nu_Whi-ra_Haki".split("_"),monthsRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsStrictRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsShortRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsShortStrictRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,2}/i,weekdays:"Rātapu_Mane_Tūrei_Wenerei_Tāite_Paraire_Hātarei".split("_"),weekdaysShort:"Ta_Ma_Tū_We_Tāi_Pa_Hā".split("_"),weekdaysMin:"Ta_Ma_Tū_We_Tāi_Pa_Hā".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [i] HH:mm",LLLL:"dddd, D MMMM YYYY [i] HH:mm"},calendar:{sameDay:"[i teie mahana, i] LT",nextDay:"[apopo i] LT",nextWeek:"dddd [i] LT",lastDay:"[inanahi i] LT",lastWeek:"dddd [whakamutunga i] LT",sameElse:"L"},relativeTime:{future:"i roto i %s",past:"%s i mua",s:"te hēkona ruarua",ss:"%d hēkona",m:"he meneti",mm:"%d meneti",h:"te haora",hh:"%d haora",d:"he ra",dd:"%d ra",M:"he marama",MM:"%d marama",y:"he tau",yy:"%d tau"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("mk",{months:"јануари_февруари_март_април_мај_јуни_јули_август_септември_октомври_ноември_декември".split("_"),monthsShort:"јан_фев_мар_апр_мај_јун_јул_авг_сеп_окт_ное_дек".split("_"),weekdays:"недела_понеделник_вторник_среда_четврток_петок_сабота".split("_"),weekdaysShort:"нед_пон_вто_сре_чет_пет_саб".split("_"),weekdaysMin:"нe_пo_вт_ср_че_пе_сa".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[Денес во] LT",nextDay:"[Утре во] LT",nextWeek:"[Во] dddd [во] LT",lastDay:"[Вчера во] LT",lastWeek:function(){switch(this.day()){case 0:case 3:case 6:return"[Изминатата] dddd [во] LT";case 1:case 2:case 4:case 5:return"[Изминатиот] dddd [во] LT"}},sameElse:"L"},relativeTime:{future:"после %s",past:"пред %s",s:"неколку секунди",ss:"%d секунди",m:"минута",mm:"%d минути",h:"час",hh:"%d часа",d:"ден",dd:"%d дена",M:"месец",MM:"%d месеци",y:"година",yy:"%d години"},dayOfMonthOrdinalParse:/\d{1,2}-(ев|ен|ти|ви|ри|ми)/,ordinal:function(t){var e=t%10,n=t%100;return 0===t?t+"-ев":0===n?t+"-ен":n>10&&n<20?t+"-ти":1===e?t+"-ви":2===e?t+"-ри":7===e||8===e?t+"-ми":t+"-ти"},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ml",{months:"ജനുവരി_ഫെബ്രുവരി_മാർച്ച്_ഏപ്രിൽ_മേയ്_ജൂൺ_ജൂലൈ_ഓഗസ്റ്റ്_സെപ്റ്റംബർ_ഒക്ടോബർ_നവംബർ_ഡിസംബർ".split("_"),monthsShort:"ജനു._ഫെബ്രു._മാർ._ഏപ്രി._മേയ്_ജൂൺ_ജൂലൈ._ഓഗ._സെപ്റ്റ._ഒക്ടോ._നവം._ഡിസം.".split("_"),monthsParseExact:!0,weekdays:"ഞായറാഴ്ച_തിങ്കളാഴ്ച_ചൊവ്വാഴ്ച_ബുധനാഴ്ച_വ്യാഴാഴ്ച_വെള്ളിയാഴ്ച_ശനിയാഴ്ച".split("_"),weekdaysShort:"ഞായർ_തിങ്കൾ_ചൊവ്വ_ബുധൻ_വ്യാഴം_വെള്ളി_ശനി".split("_"),weekdaysMin:"ഞാ_തി_ചൊ_ബു_വ്യാ_വെ_ശ".split("_"),longDateFormat:{LT:"A h:mm -നു",LTS:"A h:mm:ss -നു",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm -നു",LLLL:"dddd, D MMMM YYYY, A h:mm -നു"},calendar:{sameDay:"[ഇന്ന്] LT",nextDay:"[നാളെ] LT",nextWeek:"dddd, LT",lastDay:"[ഇന്നലെ] LT",lastWeek:"[കഴിഞ്ഞ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s കഴിഞ്ഞ്",past:"%s മുൻപ്",s:"അൽപ നിമിഷങ്ങൾ",ss:"%d സെക്കൻഡ്",m:"ഒരു മിനിറ്റ്",mm:"%d മിനിറ്റ്",h:"ഒരു മണിക്കൂർ",hh:"%d മണിക്കൂർ",d:"ഒരു ദിവസം",dd:"%d ദിവസം",M:"ഒരു മാസം",MM:"%d മാസം",y:"ഒരു വർഷം",yy:"%d വർഷം"},meridiemParse:/രാത്രി|രാവിലെ|ഉച്ച കഴിഞ്ഞ്|വൈകുന്നേരം|രാത്രി/i,meridiemHour:function(t,e){return 12===t&&(t=0),"രാത്രി"===e&&t>=4||"ഉച്ച കഴിഞ്ഞ്"===e||"വൈകുന്നേരം"===e?t+12:t},meridiem:function(t,e,n){return t<4?"രാത്രി":t<12?"രാവിലെ":t<17?"ഉച്ച കഴിഞ്ഞ്":t<20?"വൈകുന്നേരം":"രാത്രി"}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n,r){switch(n){case"s":return e?"хэдхэн секунд":"хэдхэн секундын";case"ss":return t+(e?" секунд":" секундын");case"m":case"mm":return t+(e?" минут":" минутын");case"h":case"hh":return t+(e?" цаг":" цагийн");case"d":case"dd":return t+(e?" өдөр":" өдрийн");case"M":case"MM":return t+(e?" сар":" сарын");case"y":case"yy":return t+(e?" жил":" жилийн");default:return t}}t.defineLocale("mn",{months:"Нэгдүгээр сар_Хоёрдугаар сар_Гуравдугаар сар_Дөрөвдүгээр сар_Тавдугаар сар_Зургадугаар сар_Долдугаар сар_Наймдугаар сар_Есдүгээр сар_Аравдугаар сар_Арван нэгдүгээр сар_Арван хоёрдугаар сар".split("_"),monthsShort:"1 сар_2 сар_3 сар_4 сар_5 сар_6 сар_7 сар_8 сар_9 сар_10 сар_11 сар_12 сар".split("_"),monthsParseExact:!0,weekdays:"Ням_Даваа_Мягмар_Лхагва_Пүрэв_Баасан_Бямба".split("_"),weekdaysShort:"Ням_Дав_Мяг_Лха_Пүр_Баа_Бям".split("_"),weekdaysMin:"Ня_Да_Мя_Лх_Пү_Ба_Бя".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY оны MMMMын D",LLL:"YYYY оны MMMMын D HH:mm",LLLL:"dddd, YYYY оны MMMMын D HH:mm"},meridiemParse:/ҮӨ|ҮХ/i,isPM:function(t){return"ҮХ"===t},meridiem:function(t,e,n){return t<12?"ҮӨ":"ҮХ"},calendar:{sameDay:"[Өнөөдөр] LT",nextDay:"[Маргааш] LT",nextWeek:"[Ирэх] dddd LT",lastDay:"[Өчигдөр] LT",lastWeek:"[Өнгөрсөн] dddd LT",sameElse:"L"},relativeTime:{future:"%s дараа",past:"%s өмнө",s:e,ss:e,m:e,mm:e,h:e,hh:e,d:e,dd:e,M:e,MM:e,y:e,yy:e},dayOfMonthOrdinalParse:/\d{1,2} өдөр/,ordinal:function(t,e){switch(e){case"d":case"D":case"DDD":return t+" өдөр";default:return t}}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"},n={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"};function r(t,e,n,r){var i="";if(e)switch(n){case"s":i="काही सेकंद";break;case"ss":i="%d सेकंद";break;case"m":i="एक मिनिट";break;case"mm":i="%d मिनिटे";break;case"h":i="एक तास";break;case"hh":i="%d तास";break;case"d":i="एक दिवस";break;case"dd":i="%d दिवस";break;case"M":i="एक महिना";break;case"MM":i="%d महिने";break;case"y":i="एक वर्ष";break;case"yy":i="%d वर्षे"}else switch(n){case"s":i="काही सेकंदां";break;case"ss":i="%d सेकंदां";break;case"m":i="एका मिनिटा";break;case"mm":i="%d मिनिटां";break;case"h":i="एका तासा";break;case"hh":i="%d तासां";break;case"d":i="एका दिवसा";break;case"dd":i="%d दिवसां";break;case"M":i="एका महिन्या";break;case"MM":i="%d महिन्यां";break;case"y":i="एका वर्षा";break;case"yy":i="%d वर्षां"}return i.replace(/%d/i,t)}t.defineLocale("mr",{months:"जानेवारी_फेब्रुवारी_मार्च_एप्रिल_मे_जून_जुलै_ऑगस्ट_सप्टेंबर_ऑक्टोबर_नोव्हेंबर_डिसेंबर".split("_"),monthsShort:"जाने._फेब्रु._मार्च._एप्रि._मे._जून._जुलै._ऑग._सप्टें._ऑक्टो._नोव्हें._डिसें.".split("_"),monthsParseExact:!0,weekdays:"रविवार_सोमवार_मंगळवार_बुधवार_गुरूवार_शुक्रवार_शनिवार".split("_"),weekdaysShort:"रवि_सोम_मंगळ_बुध_गुरू_शुक्र_शनि".split("_"),weekdaysMin:"र_सो_मं_बु_गु_शु_श".split("_"),longDateFormat:{LT:"A h:mm वाजता",LTS:"A h:mm:ss वाजता",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm वाजता",LLLL:"dddd, D MMMM YYYY, A h:mm वाजता"},calendar:{sameDay:"[आज] LT",nextDay:"[उद्या] LT",nextWeek:"dddd, LT",lastDay:"[काल] LT",lastWeek:"[मागील] dddd, LT",sameElse:"L"},relativeTime:{future:"%sमध्ये",past:"%sपूर्वी",s:r,ss:r,m:r,mm:r,h:r,hh:r,d:r,dd:r,M:r,MM:r,y:r,yy:r},preparse:function(t){return t.replace(/[१२३४५६७८९०]/g,function(t){return n[t]})},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]})},meridiemParse:/रात्री|सकाळी|दुपारी|सायंकाळी/,meridiemHour:function(t,e){return 12===t&&(t=0),"रात्री"===e?t<4?t:t+12:"सकाळी"===e?t:"दुपारी"===e?t>=10?t:t+12:"सायंकाळी"===e?t+12:void 0},meridiem:function(t,e,n){return t<4?"रात्री":t<10?"सकाळी":t<17?"दुपारी":t<20?"सायंकाळी":"रात्री"},week:{dow:0,doy:6}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ms",{months:"Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis".split("_"),weekdays:"Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu".split("_"),weekdaysShort:"Ahd_Isn_Sel_Rab_Kha_Jum_Sab".split("_"),weekdaysMin:"Ah_Is_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|tengahari|petang|malam/,meridiemHour:function(t,e){return 12===t&&(t=0),"pagi"===e?t:"tengahari"===e?t>=11?t:t+12:"petang"===e||"malam"===e?t+12:void 0},meridiem:function(t,e,n){return t<11?"pagi":t<15?"tengahari":t<19?"petang":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Esok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kelmarin pukul] LT",lastWeek:"dddd [lepas pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lepas",s:"beberapa saat",ss:"%d saat",m:"seminit",mm:"%d minit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ms-my",{months:"Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis".split("_"),weekdays:"Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu".split("_"),weekdaysShort:"Ahd_Isn_Sel_Rab_Kha_Jum_Sab".split("_"),weekdaysMin:"Ah_Is_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|tengahari|petang|malam/,meridiemHour:function(t,e){return 12===t&&(t=0),"pagi"===e?t:"tengahari"===e?t>=11?t:t+12:"petang"===e||"malam"===e?t+12:void 0},meridiem:function(t,e,n){return t<11?"pagi":t<15?"tengahari":t<19?"petang":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Esok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kelmarin pukul] LT",lastWeek:"dddd [lepas pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lepas",s:"beberapa saat",ss:"%d saat",m:"seminit",mm:"%d minit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("mt",{months:"Jannar_Frar_Marzu_April_Mejju_Ġunju_Lulju_Awwissu_Settembru_Ottubru_Novembru_Diċembru".split("_"),monthsShort:"Jan_Fra_Mar_Apr_Mej_Ġun_Lul_Aww_Set_Ott_Nov_Diċ".split("_"),weekdays:"Il-Ħadd_It-Tnejn_It-Tlieta_L-Erbgħa_Il-Ħamis_Il-Ġimgħa_Is-Sibt".split("_"),weekdaysShort:"Ħad_Tne_Tli_Erb_Ħam_Ġim_Sib".split("_"),weekdaysMin:"Ħa_Tn_Tl_Er_Ħa_Ġi_Si".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Illum fil-]LT",nextDay:"[Għada fil-]LT",nextWeek:"dddd [fil-]LT",lastDay:"[Il-bieraħ fil-]LT",lastWeek:"dddd [li għadda] [fil-]LT",sameElse:"L"},relativeTime:{future:"f’ %s",past:"%s ilu",s:"ftit sekondi",ss:"%d sekondi",m:"minuta",mm:"%d minuti",h:"siegħa",hh:"%d siegħat",d:"ġurnata",dd:"%d ġranet",M:"xahar",MM:"%d xhur",y:"sena",yy:"%d sni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"၁",2:"၂",3:"၃",4:"၄",5:"၅",6:"၆",7:"၇",8:"၈",9:"၉",0:"၀"},n={"၁":"1","၂":"2","၃":"3","၄":"4","၅":"5","၆":"6","၇":"7","၈":"8","၉":"9","၀":"0"};t.defineLocale("my",{months:"ဇန်နဝါရီ_ဖေဖော်ဝါရီ_မတ်_ဧပြီ_မေ_ဇွန်_ဇူလိုင်_သြဂုတ်_စက်တင်ဘာ_အောက်တိုဘာ_နိုဝင်ဘာ_ဒီဇင်ဘာ".split("_"),monthsShort:"ဇန်_ဖေ_မတ်_ပြီ_မေ_ဇွန်_လိုင်_သြ_စက်_အောက်_နို_ဒီ".split("_"),weekdays:"တနင်္ဂနွေ_တနင်္လာ_အင်္ဂါ_ဗုဒ္ဓဟူး_ကြာသပတေး_သောကြာ_စနေ".split("_"),weekdaysShort:"နွေ_လာ_ဂါ_ဟူး_ကြာ_သော_နေ".split("_"),weekdaysMin:"နွေ_လာ_ဂါ_ဟူး_ကြာ_သော_နေ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[ယနေ.] LT [မှာ]",nextDay:"[မနက်ဖြန်] LT [မှာ]",nextWeek:"dddd LT [မှာ]",lastDay:"[မနေ.က] LT [မှာ]",lastWeek:"[ပြီးခဲ့သော] dddd LT [မှာ]",sameElse:"L"},relativeTime:{future:"လာမည့် %s မှာ",past:"လွန်ခဲ့သော %s က",s:"စက္ကန်.အနည်းငယ်",ss:"%d စက္ကန့်",m:"တစ်မိနစ်",mm:"%d မိနစ်",h:"တစ်နာရီ",hh:"%d နာရီ",d:"တစ်ရက်",dd:"%d ရက်",M:"တစ်လ",MM:"%d လ",y:"တစ်နှစ်",yy:"%d နှစ်"},preparse:function(t){return t.replace(/[၁၂၃၄၅၆၇၈၉၀]/g,function(t){return n[t]})},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]})},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("nb",{months:"januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan._feb._mars_april_mai_juni_juli_aug._sep._okt._nov._des.".split("_"),monthsParseExact:!0,weekdays:"søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag".split("_"),weekdaysShort:"sø._ma._ti._on._to._fr._lø.".split("_"),weekdaysMin:"sø_ma_ti_on_to_fr_lø".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] HH:mm",LLLL:"dddd D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[i dag kl.] LT",nextDay:"[i morgen kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[i går kl.] LT",lastWeek:"[forrige] dddd [kl.] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s siden",s:"noen sekunder",ss:"%d sekunder",m:"ett minutt",mm:"%d minutter",h:"en time",hh:"%d timer",d:"en dag",dd:"%d dager",M:"en måned",MM:"%d måneder",y:"ett år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"},n={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"};t.defineLocale("ne",{months:"जनवरी_फेब्रुवरी_मार्च_अप्रिल_मई_जुन_जुलाई_अगष्ट_सेप्टेम्बर_अक्टोबर_नोभेम्बर_डिसेम्बर".split("_"),monthsShort:"जन._फेब्रु._मार्च_अप्रि._मई_जुन_जुलाई._अग._सेप्ट._अक्टो._नोभे._डिसे.".split("_"),monthsParseExact:!0,weekdays:"आइतबार_सोमबार_मङ्गलबार_बुधबार_बिहिबार_शुक्रबार_शनिबार".split("_"),weekdaysShort:"आइत._सोम._मङ्गल._बुध._बिहि._शुक्र._शनि.".split("_"),weekdaysMin:"आ._सो._मं._बु._बि._शु._श.".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"Aको h:mm बजे",LTS:"Aको h:mm:ss बजे",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, Aको h:mm बजे",LLLL:"dddd, D MMMM YYYY, Aको h:mm बजे"},preparse:function(t){return t.replace(/[१२३४५६७८९०]/g,function(t){return n[t]})},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]})},meridiemParse:/राति|बिहान|दिउँसो|साँझ/,meridiemHour:function(t,e){return 12===t&&(t=0),"राति"===e?t<4?t:t+12:"बिहान"===e?t:"दिउँसो"===e?t>=10?t:t+12:"साँझ"===e?t+12:void 0},meridiem:function(t,e,n){return t<3?"राति":t<12?"बिहान":t<16?"दिउँसो":t<20?"साँझ":"राति"},calendar:{sameDay:"[आज] LT",nextDay:"[भोलि] LT",nextWeek:"[आउँदो] dddd[,] LT",lastDay:"[हिजो] LT",lastWeek:"[गएको] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%sमा",past:"%s अगाडि",s:"केही क्षण",ss:"%d सेकेण्ड",m:"एक मिनेट",mm:"%d मिनेट",h:"एक घण्टा",hh:"%d घण्टा",d:"एक दिन",dd:"%d दिन",M:"एक महिना",MM:"%d महिना",y:"एक बर्ष",yy:"%d बर्ष"},week:{dow:0,doy:6}})}(n(7))},function(t,e,n){!function(t){"use strict";var e="jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.".split("_"),n="jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec".split("_"),r=[/^jan/i,/^feb/i,/^maart|mrt.?$/i,/^apr/i,/^mei$/i,/^jun[i.]?$/i,/^jul[i.]?$/i,/^aug/i,/^sep/i,/^okt/i,/^nov/i,/^dec/i],i=/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i;t.defineLocale("nl",{months:"januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december".split("_"),monthsShort:function(t,r){return t?/-MMM-/.test(r)?n[t.month()]:e[t.month()]:e},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,monthsShortStrictRegex:/^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag".split("_"),weekdaysShort:"zo._ma._di._wo._do._vr._za.".split("_"),weekdaysMin:"zo_ma_di_wo_do_vr_za".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[vandaag om] LT",nextDay:"[morgen om] LT",nextWeek:"dddd [om] LT",lastDay:"[gisteren om] LT",lastWeek:"[afgelopen] dddd [om] LT",sameElse:"L"},relativeTime:{future:"over %s",past:"%s geleden",s:"een paar seconden",ss:"%d seconden",m:"één minuut",mm:"%d minuten",h:"één uur",hh:"%d uur",d:"één dag",dd:"%d dagen",M:"één maand",MM:"%d maanden",y:"één jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(t){return t+(1===t||8===t||t>=20?"ste":"de")},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e="jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.".split("_"),n="jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec".split("_"),r=[/^jan/i,/^feb/i,/^maart|mrt.?$/i,/^apr/i,/^mei$/i,/^jun[i.]?$/i,/^jul[i.]?$/i,/^aug/i,/^sep/i,/^okt/i,/^nov/i,/^dec/i],i=/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i;t.defineLocale("nl-be",{months:"januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december".split("_"),monthsShort:function(t,r){return t?/-MMM-/.test(r)?n[t.month()]:e[t.month()]:e},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,monthsShortStrictRegex:/^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag".split("_"),weekdaysShort:"zo._ma._di._wo._do._vr._za.".split("_"),weekdaysMin:"zo_ma_di_wo_do_vr_za".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[vandaag om] LT",nextDay:"[morgen om] LT",nextWeek:"dddd [om] LT",lastDay:"[gisteren om] LT",lastWeek:"[afgelopen] dddd [om] LT",sameElse:"L"},relativeTime:{future:"over %s",past:"%s geleden",s:"een paar seconden",ss:"%d seconden",m:"één minuut",mm:"%d minuten",h:"één uur",hh:"%d uur",d:"één dag",dd:"%d dagen",M:"één maand",MM:"%d maanden",y:"één jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(t){return t+(1===t||8===t||t>=20?"ste":"de")},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("nn",{months:"januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan_feb_mar_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_"),weekdays:"sundag_måndag_tysdag_onsdag_torsdag_fredag_laurdag".split("_"),weekdaysShort:"sun_mån_tys_ons_tor_fre_lau".split("_"),weekdaysMin:"su_må_ty_on_to_fr_lø".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] H:mm",LLLL:"dddd D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[I dag klokka] LT",nextDay:"[I morgon klokka] LT",nextWeek:"dddd [klokka] LT",lastDay:"[I går klokka] LT",lastWeek:"[Føregåande] dddd [klokka] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s sidan",s:"nokre sekund",ss:"%d sekund",m:"eit minutt",mm:"%d minutt",h:"ein time",hh:"%d timar",d:"ein dag",dd:"%d dagar",M:"ein månad",MM:"%d månader",y:"eit år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"੧",2:"੨",3:"੩",4:"੪",5:"੫",6:"੬",7:"੭",8:"੮",9:"੯",0:"੦"},n={"੧":"1","੨":"2","੩":"3","੪":"4","੫":"5","੬":"6","੭":"7","੮":"8","੯":"9","੦":"0"};t.defineLocale("pa-in",{months:"ਜਨਵਰੀ_ਫ਼ਰਵਰੀ_ਮਾਰਚ_ਅਪ੍ਰੈਲ_ਮਈ_ਜੂਨ_ਜੁਲਾਈ_ਅਗਸਤ_ਸਤੰਬਰ_ਅਕਤੂਬਰ_ਨਵੰਬਰ_ਦਸੰਬਰ".split("_"),monthsShort:"ਜਨਵਰੀ_ਫ਼ਰਵਰੀ_ਮਾਰਚ_ਅਪ੍ਰੈਲ_ਮਈ_ਜੂਨ_ਜੁਲਾਈ_ਅਗਸਤ_ਸਤੰਬਰ_ਅਕਤੂਬਰ_ਨਵੰਬਰ_ਦਸੰਬਰ".split("_"),weekdays:"ਐਤਵਾਰ_ਸੋਮਵਾਰ_ਮੰਗਲਵਾਰ_ਬੁਧਵਾਰ_ਵੀਰਵਾਰ_ਸ਼ੁੱਕਰਵਾਰ_ਸ਼ਨੀਚਰਵਾਰ".split("_"),weekdaysShort:"ਐਤ_ਸੋਮ_ਮੰਗਲ_ਬੁਧ_ਵੀਰ_ਸ਼ੁਕਰ_ਸ਼ਨੀ".split("_"),weekdaysMin:"ਐਤ_ਸੋਮ_ਮੰਗਲ_ਬੁਧ_ਵੀਰ_ਸ਼ੁਕਰ_ਸ਼ਨੀ".split("_"),longDateFormat:{LT:"A h:mm ਵਜੇ",LTS:"A h:mm:ss ਵਜੇ",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm ਵਜੇ",LLLL:"dddd, D MMMM YYYY, A h:mm ਵਜੇ"},calendar:{sameDay:"[ਅਜ] LT",nextDay:"[ਕਲ] LT",nextWeek:"[ਅਗਲਾ] dddd, LT",lastDay:"[ਕਲ] LT",lastWeek:"[ਪਿਛਲੇ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ਵਿੱਚ",past:"%s ਪਿਛਲੇ",s:"ਕੁਝ ਸਕਿੰਟ",ss:"%d ਸਕਿੰਟ",m:"ਇਕ ਮਿੰਟ",mm:"%d ਮਿੰਟ",h:"ਇੱਕ ਘੰਟਾ",hh:"%d ਘੰਟੇ",d:"ਇੱਕ ਦਿਨ",dd:"%d ਦਿਨ",M:"ਇੱਕ ਮਹੀਨਾ",MM:"%d ਮਹੀਨੇ",y:"ਇੱਕ ਸਾਲ",yy:"%d ਸਾਲ"},preparse:function(t){return t.replace(/[੧੨੩੪੫੬੭੮੯੦]/g,function(t){return n[t]})},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]})},meridiemParse:/ਰਾਤ|ਸਵੇਰ|ਦੁਪਹਿਰ|ਸ਼ਾਮ/,meridiemHour:function(t,e){return 12===t&&(t=0),"ਰਾਤ"===e?t<4?t:t+12:"ਸਵੇਰ"===e?t:"ਦੁਪਹਿਰ"===e?t>=10?t:t+12:"ਸ਼ਾਮ"===e?t+12:void 0},meridiem:function(t,e,n){return t<4?"ਰਾਤ":t<10?"ਸਵੇਰ":t<17?"ਦੁਪਹਿਰ":t<20?"ਸ਼ਾਮ":"ਰਾਤ"},week:{dow:0,doy:6}})}(n(7))},function(t,e,n){!function(t){"use strict";var e="styczeń_luty_marzec_kwiecień_maj_czerwiec_lipiec_sierpień_wrzesień_październik_listopad_grudzień".split("_"),n="stycznia_lutego_marca_kwietnia_maja_czerwca_lipca_sierpnia_września_października_listopada_grudnia".split("_");function r(t){return t%10<5&&t%10>1&&~~(t/10)%10!=1}function i(t,e,n){var i=t+" ";switch(n){case"ss":return i+(r(t)?"sekundy":"sekund");case"m":return e?"minuta":"minutę";case"mm":return i+(r(t)?"minuty":"minut");case"h":return e?"godzina":"godzinę";case"hh":return i+(r(t)?"godziny":"godzin");case"MM":return i+(r(t)?"miesiące":"miesięcy");case"yy":return i+(r(t)?"lata":"lat")}}t.defineLocale("pl",{months:function(t,r){return t?""===r?"("+n[t.month()]+"|"+e[t.month()]+")":/D MMMM/.test(r)?n[t.month()]:e[t.month()]:e},monthsShort:"sty_lut_mar_kwi_maj_cze_lip_sie_wrz_paź_lis_gru".split("_"),weekdays:"niedziela_poniedziałek_wtorek_środa_czwartek_piątek_sobota".split("_"),weekdaysShort:"ndz_pon_wt_śr_czw_pt_sob".split("_"),weekdaysMin:"Nd_Pn_Wt_Śr_Cz_Pt_So".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Dziś o] LT",nextDay:"[Jutro o] LT",nextWeek:function(){switch(this.day()){case 0:return"[W niedzielę o] LT";case 2:return"[We wtorek o] LT";case 3:return"[W środę o] LT";case 6:return"[W sobotę o] LT";default:return"[W] dddd [o] LT"}},lastDay:"[Wczoraj o] LT",lastWeek:function(){switch(this.day()){case 0:return"[W zeszłą niedzielę o] LT";case 3:return"[W zeszłą środę o] LT";case 6:return"[W zeszłą sobotę o] LT";default:return"[W zeszły] dddd [o] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"%s temu",s:"kilka sekund",ss:i,m:i,mm:i,h:i,hh:i,d:"1 dzień",dd:"%d dni",M:"miesiąc",MM:i,y:"rok",yy:i},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("pt",{months:"Janeiro_Fevereiro_Março_Abril_Maio_Junho_Julho_Agosto_Setembro_Outubro_Novembro_Dezembro".split("_"),monthsShort:"Jan_Fev_Mar_Abr_Mai_Jun_Jul_Ago_Set_Out_Nov_Dez".split("_"),weekdays:"Domingo_Segunda-feira_Terça-feira_Quarta-feira_Quinta-feira_Sexta-feira_Sábado".split("_"),weekdaysShort:"Dom_Seg_Ter_Qua_Qui_Sex_Sáb".split("_"),weekdaysMin:"Do_2ª_3ª_4ª_5ª_6ª_Sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY HH:mm",LLLL:"dddd, D [de] MMMM [de] YYYY HH:mm"},calendar:{sameDay:"[Hoje às] LT",nextDay:"[Amanhã às] LT",nextWeek:"dddd [às] LT",lastDay:"[Ontem às] LT",lastWeek:function(){return 0===this.day()||6===this.day()?"[Último] dddd [às] LT":"[Última] dddd [às] LT"},sameElse:"L"},relativeTime:{future:"em %s",past:"há %s",s:"segundos",ss:"%d segundos",m:"um minuto",mm:"%d minutos",h:"uma hora",hh:"%d horas",d:"um dia",dd:"%d dias",M:"um mês",MM:"%d meses",y:"um ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("pt-br",{months:"Janeiro_Fevereiro_Março_Abril_Maio_Junho_Julho_Agosto_Setembro_Outubro_Novembro_Dezembro".split("_"),monthsShort:"Jan_Fev_Mar_Abr_Mai_Jun_Jul_Ago_Set_Out_Nov_Dez".split("_"),weekdays:"Domingo_Segunda-feira_Terça-feira_Quarta-feira_Quinta-feira_Sexta-feira_Sábado".split("_"),weekdaysShort:"Dom_Seg_Ter_Qua_Qui_Sex_Sáb".split("_"),weekdaysMin:"Do_2ª_3ª_4ª_5ª_6ª_Sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY [às] HH:mm",LLLL:"dddd, D [de] MMMM [de] YYYY [às] HH:mm"},calendar:{sameDay:"[Hoje às] LT",nextDay:"[Amanhã às] LT",nextWeek:"dddd [às] LT",lastDay:"[Ontem às] LT",lastWeek:function(){return 0===this.day()||6===this.day()?"[Último] dddd [às] LT":"[Última] dddd [às] LT"},sameElse:"L"},relativeTime:{future:"em %s",past:"há %s",s:"poucos segundos",ss:"%d segundos",m:"um minuto",mm:"%d minutos",h:"uma hora",hh:"%d horas",d:"um dia",dd:"%d dias",M:"um mês",MM:"%d meses",y:"um ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº"})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n){var r=" ";return(t%100>=20||t>=100&&t%100==0)&&(r=" de "),t+r+{ss:"secunde",mm:"minute",hh:"ore",dd:"zile",MM:"luni",yy:"ani"}[n]}t.defineLocale("ro",{months:"ianuarie_februarie_martie_aprilie_mai_iunie_iulie_august_septembrie_octombrie_noiembrie_decembrie".split("_"),monthsShort:"ian._febr._mart._apr._mai_iun._iul._aug._sept._oct._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"duminică_luni_marți_miercuri_joi_vineri_sâmbătă".split("_"),weekdaysShort:"Dum_Lun_Mar_Mie_Joi_Vin_Sâm".split("_"),weekdaysMin:"Du_Lu_Ma_Mi_Jo_Vi_Sâ".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[azi la] LT",nextDay:"[mâine la] LT",nextWeek:"dddd [la] LT",lastDay:"[ieri la] LT",lastWeek:"[fosta] dddd [la] LT",sameElse:"L"},relativeTime:{future:"peste %s",past:"%s în urmă",s:"câteva secunde",ss:e,m:"un minut",mm:e,h:"o oră",hh:e,d:"o zi",dd:e,M:"o lună",MM:e,y:"un an",yy:e},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n){var r,i,o={ss:e?"секунда_секунды_секунд":"секунду_секунды_секунд",mm:e?"минута_минуты_минут":"минуту_минуты_минут",hh:"час_часа_часов",dd:"день_дня_дней",MM:"месяц_месяца_месяцев",yy:"год_года_лет"};return"m"===n?e?"минута":"минуту":t+" "+(r=+t,i=o[n].split("_"),r%10==1&&r%100!=11?i[0]:r%10>=2&&r%10<=4&&(r%100<10||r%100>=20)?i[1]:i[2])}var n=[/^янв/i,/^фев/i,/^мар/i,/^апр/i,/^ма[йя]/i,/^июн/i,/^июл/i,/^авг/i,/^сен/i,/^окт/i,/^ноя/i,/^дек/i];t.defineLocale("ru",{months:{format:"января_февраля_марта_апреля_мая_июня_июля_августа_сентября_октября_ноября_декабря".split("_"),standalone:"январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь".split("_")},monthsShort:{format:"янв._февр._мар._апр._мая_июня_июля_авг._сент._окт._нояб._дек.".split("_"),standalone:"янв._февр._март_апр._май_июнь_июль_авг._сент._окт._нояб._дек.".split("_")},weekdays:{standalone:"воскресенье_понедельник_вторник_среда_четверг_пятница_суббота".split("_"),format:"воскресенье_понедельник_вторник_среду_четверг_пятницу_субботу".split("_"),isFormat:/\[ ?[Вв] ?(?:прошлую|следующую|эту)? ?\] ?dddd/},weekdaysShort:"вс_пн_вт_ср_чт_пт_сб".split("_"),weekdaysMin:"вс_пн_вт_ср_чт_пт_сб".split("_"),monthsParse:n,longMonthsParse:n,shortMonthsParse:n,monthsRegex:/^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,monthsShortRegex:/^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,monthsStrictRegex:/^(январ[яь]|феврал[яь]|марта?|апрел[яь]|ма[яй]|июн[яь]|июл[яь]|августа?|сентябр[яь]|октябр[яь]|ноябр[яь]|декабр[яь])/i,monthsShortStrictRegex:/^(янв\.|февр?\.|мар[т.]|апр\.|ма[яй]|июн[ья.]|июл[ья.]|авг\.|сент?\.|окт\.|нояб?\.|дек\.)/i,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY г.",LLL:"D MMMM YYYY г., H:mm",LLLL:"dddd, D MMMM YYYY г., H:mm"},calendar:{sameDay:"[Сегодня, в] LT",nextDay:"[Завтра, в] LT",lastDay:"[Вчера, в] LT",nextWeek:function(t){if(t.week()===this.week())return 2===this.day()?"[Во] dddd, [в] LT":"[В] dddd, [в] LT";switch(this.day()){case 0:return"[В следующее] dddd, [в] LT";case 1:case 2:case 4:return"[В следующий] dddd, [в] LT";case 3:case 5:case 6:return"[В следующую] dddd, [в] LT"}},lastWeek:function(t){if(t.week()===this.week())return 2===this.day()?"[Во] dddd, [в] LT":"[В] dddd, [в] LT";switch(this.day()){case 0:return"[В прошлое] dddd, [в] LT";case 1:case 2:case 4:return"[В прошлый] dddd, [в] LT";case 3:case 5:case 6:return"[В прошлую] dddd, [в] LT"}},sameElse:"L"},relativeTime:{future:"через %s",past:"%s назад",s:"несколько секунд",ss:e,m:e,mm:e,h:"час",hh:e,d:"день",dd:e,M:"месяц",MM:e,y:"год",yy:e},meridiemParse:/ночи|утра|дня|вечера/i,isPM:function(t){return/^(дня|вечера)$/.test(t)},meridiem:function(t,e,n){return t<4?"ночи":t<12?"утра":t<17?"дня":"вечера"},dayOfMonthOrdinalParse:/\d{1,2}-(й|го|я)/,ordinal:function(t,e){switch(e){case"M":case"d":case"DDD":return t+"-й";case"D":return t+"-го";case"w":case"W":return t+"-я";default:return t}},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e=["جنوري","فيبروري","مارچ","اپريل","مئي","جون","جولاءِ","آگسٽ","سيپٽمبر","آڪٽوبر","نومبر","ڊسمبر"],n=["آچر","سومر","اڱارو","اربع","خميس","جمع","ڇنڇر"];t.defineLocale("sd",{months:e,monthsShort:e,weekdays:n,weekdaysShort:n,weekdaysMin:n,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd، D MMMM YYYY HH:mm"},meridiemParse:/صبح|شام/,isPM:function(t){return"شام"===t},meridiem:function(t,e,n){return t<12?"صبح":"شام"},calendar:{sameDay:"[اڄ] LT",nextDay:"[سڀاڻي] LT",nextWeek:"dddd [اڳين هفتي تي] LT",lastDay:"[ڪالهه] LT",lastWeek:"[گزريل هفتي] dddd [تي] LT",sameElse:"L"},relativeTime:{future:"%s پوء",past:"%s اڳ",s:"چند سيڪنڊ",ss:"%d سيڪنڊ",m:"هڪ منٽ",mm:"%d منٽ",h:"هڪ ڪلاڪ",hh:"%d ڪلاڪ",d:"هڪ ڏينهن",dd:"%d ڏينهن",M:"هڪ مهينو",MM:"%d مهينا",y:"هڪ سال",yy:"%d سال"},preparse:function(t){return t.replace(/،/g,",")},postformat:function(t){return t.replace(/,/g,"،")},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("se",{months:"ođđajagemánnu_guovvamánnu_njukčamánnu_cuoŋománnu_miessemánnu_geassemánnu_suoidnemánnu_borgemánnu_čakčamánnu_golggotmánnu_skábmamánnu_juovlamánnu".split("_"),monthsShort:"ođđj_guov_njuk_cuo_mies_geas_suoi_borg_čakč_golg_skáb_juov".split("_"),weekdays:"sotnabeaivi_vuossárga_maŋŋebárga_gaskavahkku_duorastat_bearjadat_lávvardat".split("_"),weekdaysShort:"sotn_vuos_maŋ_gask_duor_bear_láv".split("_"),weekdaysMin:"s_v_m_g_d_b_L".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"MMMM D. [b.] YYYY",LLL:"MMMM D. [b.] YYYY [ti.] HH:mm",LLLL:"dddd, MMMM D. [b.] YYYY [ti.] HH:mm"},calendar:{sameDay:"[otne ti] LT",nextDay:"[ihttin ti] LT",nextWeek:"dddd [ti] LT",lastDay:"[ikte ti] LT",lastWeek:"[ovddit] dddd [ti] LT",sameElse:"L"},relativeTime:{future:"%s geažes",past:"maŋit %s",s:"moadde sekunddat",ss:"%d sekunddat",m:"okta minuhta",mm:"%d minuhtat",h:"okta diimmu",hh:"%d diimmut",d:"okta beaivi",dd:"%d beaivvit",M:"okta mánnu",MM:"%d mánut",y:"okta jahki",yy:"%d jagit"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("si",{months:"ජනවාරි_පෙබරවාරි_මාර්තු_අප්‍රේල්_මැයි_ජූනි_ජූලි_අගෝස්තු_සැප්තැම්බර්_ඔක්තෝබර්_නොවැම්බර්_දෙසැම්බර්".split("_"),monthsShort:"ජන_පෙබ_මාර්_අප්_මැයි_ජූනි_ජූලි_අගෝ_සැප්_ඔක්_නොවැ_දෙසැ".split("_"),weekdays:"ඉරිදා_සඳුදා_අඟහරුවාදා_බදාදා_බ්‍රහස්පතින්දා_සිකුරාදා_සෙනසුරාදා".split("_"),weekdaysShort:"ඉරි_සඳු_අඟ_බදා_බ්‍රහ_සිකු_සෙන".split("_"),weekdaysMin:"ඉ_ස_අ_බ_බ්‍ර_සි_සෙ".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"a h:mm",LTS:"a h:mm:ss",L:"YYYY/MM/DD",LL:"YYYY MMMM D",LLL:"YYYY MMMM D, a h:mm",LLLL:"YYYY MMMM D [වැනි] dddd, a h:mm:ss"},calendar:{sameDay:"[අද] LT[ට]",nextDay:"[හෙට] LT[ට]",nextWeek:"dddd LT[ට]",lastDay:"[ඊයේ] LT[ට]",lastWeek:"[පසුගිය] dddd LT[ට]",sameElse:"L"},relativeTime:{future:"%sකින්",past:"%sකට පෙර",s:"තත්පර කිහිපය",ss:"තත්පර %d",m:"මිනිත්තුව",mm:"මිනිත්තු %d",h:"පැය",hh:"පැය %d",d:"දිනය",dd:"දින %d",M:"මාසය",MM:"මාස %d",y:"වසර",yy:"වසර %d"},dayOfMonthOrdinalParse:/\d{1,2} වැනි/,ordinal:function(t){return t+" වැනි"},meridiemParse:/පෙර වරු|පස් වරු|පෙ.ව|ප.ව./,isPM:function(t){return"ප.ව."===t||"පස් වරු"===t},meridiem:function(t,e,n){return t>11?n?"ප.ව.":"පස් වරු":n?"පෙ.ව.":"පෙර වරු"}})}(n(7))},function(t,e,n){!function(t){"use strict";var e="január_február_marec_apríl_máj_jún_júl_august_september_október_november_december".split("_"),n="jan_feb_mar_apr_máj_jún_júl_aug_sep_okt_nov_dec".split("_");function r(t){return t>1&&t<5}function i(t,e,n,i){var o=t+" ";switch(n){case"s":return e||i?"pár sekúnd":"pár sekundami";case"ss":return e||i?o+(r(t)?"sekundy":"sekúnd"):o+"sekundami";case"m":return e?"minúta":i?"minútu":"minútou";case"mm":return e||i?o+(r(t)?"minúty":"minút"):o+"minútami";case"h":return e?"hodina":i?"hodinu":"hodinou";case"hh":return e||i?o+(r(t)?"hodiny":"hodín"):o+"hodinami";case"d":return e||i?"deň":"dňom";case"dd":return e||i?o+(r(t)?"dni":"dní"):o+"dňami";case"M":return e||i?"mesiac":"mesiacom";case"MM":return e||i?o+(r(t)?"mesiace":"mesiacov"):o+"mesiacmi";case"y":return e||i?"rok":"rokom";case"yy":return e||i?o+(r(t)?"roky":"rokov"):o+"rokmi"}}t.defineLocale("sk",{months:e,monthsShort:n,weekdays:"nedeľa_pondelok_utorok_streda_štvrtok_piatok_sobota".split("_"),weekdaysShort:"ne_po_ut_st_št_pi_so".split("_"),weekdaysMin:"ne_po_ut_st_št_pi_so".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd D. MMMM YYYY H:mm"},calendar:{sameDay:"[dnes o] LT",nextDay:"[zajtra o] LT",nextWeek:function(){switch(this.day()){case 0:return"[v nedeľu o] LT";case 1:case 2:return"[v] dddd [o] LT";case 3:return"[v stredu o] LT";case 4:return"[vo štvrtok o] LT";case 5:return"[v piatok o] LT";case 6:return"[v sobotu o] LT"}},lastDay:"[včera o] LT",lastWeek:function(){switch(this.day()){case 0:return"[minulú nedeľu o] LT";case 1:case 2:return"[minulý] dddd [o] LT";case 3:return"[minulú stredu o] LT";case 4:case 5:return"[minulý] dddd [o] LT";case 6:return"[minulú sobotu o] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"pred %s",s:i,ss:i,m:i,mm:i,h:i,hh:i,d:i,dd:i,M:i,MM:i,y:i,yy:i},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n,r){var i=t+" ";switch(n){case"s":return e||r?"nekaj sekund":"nekaj sekundami";case"ss":return i+=1===t?e?"sekundo":"sekundi":2===t?e||r?"sekundi":"sekundah":t<5?e||r?"sekunde":"sekundah":"sekund";case"m":return e?"ena minuta":"eno minuto";case"mm":return i+=1===t?e?"minuta":"minuto":2===t?e||r?"minuti":"minutama":t<5?e||r?"minute":"minutami":e||r?"minut":"minutami";case"h":return e?"ena ura":"eno uro";case"hh":return i+=1===t?e?"ura":"uro":2===t?e||r?"uri":"urama":t<5?e||r?"ure":"urami":e||r?"ur":"urami";case"d":return e||r?"en dan":"enim dnem";case"dd":return i+=1===t?e||r?"dan":"dnem":2===t?e||r?"dni":"dnevoma":e||r?"dni":"dnevi";case"M":return e||r?"en mesec":"enim mesecem";case"MM":return i+=1===t?e||r?"mesec":"mesecem":2===t?e||r?"meseca":"mesecema":t<5?e||r?"mesece":"meseci":e||r?"mesecev":"meseci";case"y":return e||r?"eno leto":"enim letom";case"yy":return i+=1===t?e||r?"leto":"letom":2===t?e||r?"leti":"letoma":t<5?e||r?"leta":"leti":e||r?"let":"leti"}}t.defineLocale("sl",{months:"januar_februar_marec_april_maj_junij_julij_avgust_september_oktober_november_december".split("_"),monthsShort:"jan._feb._mar._apr._maj._jun._jul._avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedelja_ponedeljek_torek_sreda_četrtek_petek_sobota".split("_"),weekdaysShort:"ned._pon._tor._sre._čet._pet._sob.".split("_"),weekdaysMin:"ne_po_to_sr_če_pe_so".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danes ob] LT",nextDay:"[jutri ob] LT",nextWeek:function(){switch(this.day()){case 0:return"[v] [nedeljo] [ob] LT";case 3:return"[v] [sredo] [ob] LT";case 6:return"[v] [soboto] [ob] LT";case 1:case 2:case 4:case 5:return"[v] dddd [ob] LT"}},lastDay:"[včeraj ob] LT",lastWeek:function(){switch(this.day()){case 0:return"[prejšnjo] [nedeljo] [ob] LT";case 3:return"[prejšnjo] [sredo] [ob] LT";case 6:return"[prejšnjo] [soboto] [ob] LT";case 1:case 2:case 4:case 5:return"[prejšnji] dddd [ob] LT"}},sameElse:"L"},relativeTime:{future:"čez %s",past:"pred %s",s:e,ss:e,m:e,mm:e,h:e,hh:e,d:e,dd:e,M:e,MM:e,y:e,yy:e},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("sq",{months:"Janar_Shkurt_Mars_Prill_Maj_Qershor_Korrik_Gusht_Shtator_Tetor_Nëntor_Dhjetor".split("_"),monthsShort:"Jan_Shk_Mar_Pri_Maj_Qer_Kor_Gus_Sht_Tet_Nën_Dhj".split("_"),weekdays:"E Diel_E Hënë_E Martë_E Mërkurë_E Enjte_E Premte_E Shtunë".split("_"),weekdaysShort:"Die_Hën_Mar_Mër_Enj_Pre_Sht".split("_"),weekdaysMin:"D_H_Ma_Më_E_P_Sh".split("_"),weekdaysParseExact:!0,meridiemParse:/PD|MD/,isPM:function(t){return"M"===t.charAt(0)},meridiem:function(t,e,n){return t<12?"PD":"MD"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Sot në] LT",nextDay:"[Nesër në] LT",nextWeek:"dddd [në] LT",lastDay:"[Dje në] LT",lastWeek:"dddd [e kaluar në] LT",sameElse:"L"},relativeTime:{future:"në %s",past:"%s më parë",s:"disa sekonda",ss:"%d sekonda",m:"një minutë",mm:"%d minuta",h:"një orë",hh:"%d orë",d:"një ditë",dd:"%d ditë",M:"një muaj",MM:"%d muaj",y:"një vit",yy:"%d vite"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={words:{ss:["sekunda","sekunde","sekundi"],m:["jedan minut","jedne minute"],mm:["minut","minute","minuta"],h:["jedan sat","jednog sata"],hh:["sat","sata","sati"],dd:["dan","dana","dana"],MM:["mesec","meseca","meseci"],yy:["godina","godine","godina"]},correctGrammaticalCase:function(t,e){return 1===t?e[0]:t>=2&&t<=4?e[1]:e[2]},translate:function(t,n,r){var i=e.words[r];return 1===r.length?n?i[0]:i[1]:t+" "+e.correctGrammaticalCase(t,i)}};t.defineLocale("sr",{months:"januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedelja_ponedeljak_utorak_sreda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sre._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedelju] [u] LT";case 3:return"[u] [sredu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[juče u] LT",lastWeek:function(){return["[prošle] [nedelje] [u] LT","[prošlog] [ponedeljka] [u] LT","[prošlog] [utorka] [u] LT","[prošle] [srede] [u] LT","[prošlog] [četvrtka] [u] LT","[prošlog] [petka] [u] LT","[prošle] [subote] [u] LT"][this.day()]},sameElse:"L"},relativeTime:{future:"za %s",past:"pre %s",s:"nekoliko sekundi",ss:e.translate,m:e.translate,mm:e.translate,h:e.translate,hh:e.translate,d:"dan",dd:e.translate,M:"mesec",MM:e.translate,y:"godinu",yy:e.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={words:{ss:["секунда","секунде","секунди"],m:["један минут","једне минуте"],mm:["минут","минуте","минута"],h:["један сат","једног сата"],hh:["сат","сата","сати"],dd:["дан","дана","дана"],MM:["месец","месеца","месеци"],yy:["година","године","година"]},correctGrammaticalCase:function(t,e){return 1===t?e[0]:t>=2&&t<=4?e[1]:e[2]},translate:function(t,n,r){var i=e.words[r];return 1===r.length?n?i[0]:i[1]:t+" "+e.correctGrammaticalCase(t,i)}};t.defineLocale("sr-cyrl",{months:"јануар_фебруар_март_април_мај_јун_јул_август_септембар_октобар_новембар_децембар".split("_"),monthsShort:"јан._феб._мар._апр._мај_јун_јул_авг._сеп._окт._нов._дец.".split("_"),monthsParseExact:!0,weekdays:"недеља_понедељак_уторак_среда_четвртак_петак_субота".split("_"),weekdaysShort:"нед._пон._уто._сре._чет._пет._суб.".split("_"),weekdaysMin:"не_по_ут_ср_че_пе_су".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[данас у] LT",nextDay:"[сутра у] LT",nextWeek:function(){switch(this.day()){case 0:return"[у] [недељу] [у] LT";case 3:return"[у] [среду] [у] LT";case 6:return"[у] [суботу] [у] LT";case 1:case 2:case 4:case 5:return"[у] dddd [у] LT"}},lastDay:"[јуче у] LT",lastWeek:function(){return["[прошле] [недеље] [у] LT","[прошлог] [понедељка] [у] LT","[прошлог] [уторка] [у] LT","[прошле] [среде] [у] LT","[прошлог] [четвртка] [у] LT","[прошлог] [петка] [у] LT","[прошле] [суботе] [у] LT"][this.day()]},sameElse:"L"},relativeTime:{future:"за %s",past:"пре %s",s:"неколико секунди",ss:e.translate,m:e.translate,mm:e.translate,h:e.translate,hh:e.translate,d:"дан",dd:e.translate,M:"месец",MM:e.translate,y:"годину",yy:e.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ss",{months:"Bhimbidvwane_Indlovana_Indlov'lenkhulu_Mabasa_Inkhwekhweti_Inhlaba_Kholwane_Ingci_Inyoni_Imphala_Lweti_Ingongoni".split("_"),monthsShort:"Bhi_Ina_Inu_Mab_Ink_Inh_Kho_Igc_Iny_Imp_Lwe_Igo".split("_"),weekdays:"Lisontfo_Umsombuluko_Lesibili_Lesitsatfu_Lesine_Lesihlanu_Umgcibelo".split("_"),weekdaysShort:"Lis_Umb_Lsb_Les_Lsi_Lsh_Umg".split("_"),weekdaysMin:"Li_Us_Lb_Lt_Ls_Lh_Ug".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Namuhla nga] LT",nextDay:"[Kusasa nga] LT",nextWeek:"dddd [nga] LT",lastDay:"[Itolo nga] LT",lastWeek:"dddd [leliphelile] [nga] LT",sameElse:"L"},relativeTime:{future:"nga %s",past:"wenteka nga %s",s:"emizuzwana lomcane",ss:"%d mzuzwana",m:"umzuzu",mm:"%d emizuzu",h:"lihora",hh:"%d emahora",d:"lilanga",dd:"%d emalanga",M:"inyanga",MM:"%d tinyanga",y:"umnyaka",yy:"%d iminyaka"},meridiemParse:/ekuseni|emini|entsambama|ebusuku/,meridiem:function(t,e,n){return t<11?"ekuseni":t<15?"emini":t<19?"entsambama":"ebusuku"},meridiemHour:function(t,e){return 12===t&&(t=0),"ekuseni"===e?t:"emini"===e?t>=11?t:t+12:"entsambama"===e||"ebusuku"===e?0===t?0:t+12:void 0},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:"%d",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("sv",{months:"januari_februari_mars_april_maj_juni_juli_augusti_september_oktober_november_december".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec".split("_"),weekdays:"söndag_måndag_tisdag_onsdag_torsdag_fredag_lördag".split("_"),weekdaysShort:"sön_mån_tis_ons_tor_fre_lör".split("_"),weekdaysMin:"sö_må_ti_on_to_fr_lö".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [kl.] HH:mm",LLLL:"dddd D MMMM YYYY [kl.] HH:mm",lll:"D MMM YYYY HH:mm",llll:"ddd D MMM YYYY HH:mm"},calendar:{sameDay:"[Idag] LT",nextDay:"[Imorgon] LT",lastDay:"[Igår] LT",nextWeek:"[På] dddd LT",lastWeek:"[I] dddd[s] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"för %s sedan",s:"några sekunder",ss:"%d sekunder",m:"en minut",mm:"%d minuter",h:"en timme",hh:"%d timmar",d:"en dag",dd:"%d dagar",M:"en månad",MM:"%d månader",y:"ett år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}(e|a)/,ordinal:function(t){var e=t%10,n=1==~~(t%100/10)?"e":1===e?"a":2===e?"a":"e";return t+n},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("sw",{months:"Januari_Februari_Machi_Aprili_Mei_Juni_Julai_Agosti_Septemba_Oktoba_Novemba_Desemba".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ago_Sep_Okt_Nov_Des".split("_"),weekdays:"Jumapili_Jumatatu_Jumanne_Jumatano_Alhamisi_Ijumaa_Jumamosi".split("_"),weekdaysShort:"Jpl_Jtat_Jnne_Jtan_Alh_Ijm_Jmos".split("_"),weekdaysMin:"J2_J3_J4_J5_Al_Ij_J1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[leo saa] LT",nextDay:"[kesho saa] LT",nextWeek:"[wiki ijayo] dddd [saat] LT",lastDay:"[jana] LT",lastWeek:"[wiki iliyopita] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s baadaye",past:"tokea %s",s:"hivi punde",ss:"sekunde %d",m:"dakika moja",mm:"dakika %d",h:"saa limoja",hh:"masaa %d",d:"siku moja",dd:"masiku %d",M:"mwezi mmoja",MM:"miezi %d",y:"mwaka mmoja",yy:"miaka %d"},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"௧",2:"௨",3:"௩",4:"௪",5:"௫",6:"௬",7:"௭",8:"௮",9:"௯",0:"௦"},n={"௧":"1","௨":"2","௩":"3","௪":"4","௫":"5","௬":"6","௭":"7","௮":"8","௯":"9","௦":"0"};t.defineLocale("ta",{months:"ஜனவரி_பிப்ரவரி_மார்ச்_ஏப்ரல்_மே_ஜூன்_ஜூலை_ஆகஸ்ட்_செப்டெம்பர்_அக்டோபர்_நவம்பர்_டிசம்பர்".split("_"),monthsShort:"ஜனவரி_பிப்ரவரி_மார்ச்_ஏப்ரல்_மே_ஜூன்_ஜூலை_ஆகஸ்ட்_செப்டெம்பர்_அக்டோபர்_நவம்பர்_டிசம்பர்".split("_"),weekdays:"ஞாயிற்றுக்கிழமை_திங்கட்கிழமை_செவ்வாய்கிழமை_புதன்கிழமை_வியாழக்கிழமை_வெள்ளிக்கிழமை_சனிக்கிழமை".split("_"),weekdaysShort:"ஞாயிறு_திங்கள்_செவ்வாய்_புதன்_வியாழன்_வெள்ளி_சனி".split("_"),weekdaysMin:"ஞா_தி_செ_பு_வி_வெ_ச".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, HH:mm",LLLL:"dddd, D MMMM YYYY, HH:mm"},calendar:{sameDay:"[இன்று] LT",nextDay:"[நாளை] LT",nextWeek:"dddd, LT",lastDay:"[நேற்று] LT",lastWeek:"[கடந்த வாரம்] dddd, LT",sameElse:"L"},relativeTime:{future:"%s இல்",past:"%s முன்",s:"ஒரு சில விநாடிகள்",ss:"%d விநாடிகள்",m:"ஒரு நிமிடம்",mm:"%d நிமிடங்கள்",h:"ஒரு மணி நேரம்",hh:"%d மணி நேரம்",d:"ஒரு நாள்",dd:"%d நாட்கள்",M:"ஒரு மாதம்",MM:"%d மாதங்கள்",y:"ஒரு வருடம்",yy:"%d ஆண்டுகள்"},dayOfMonthOrdinalParse:/\d{1,2}வது/,ordinal:function(t){return t+"வது"},preparse:function(t){return t.replace(/[௧௨௩௪௫௬௭௮௯௦]/g,function(t){return n[t]})},postformat:function(t){return t.replace(/\d/g,function(t){return e[t]})},meridiemParse:/யாமம்|வைகறை|காலை|நண்பகல்|எற்பாடு|மாலை/,meridiem:function(t,e,n){return t<2?" யாமம்":t<6?" வைகறை":t<10?" காலை":t<14?" நண்பகல்":t<18?" எற்பாடு":t<22?" மாலை":" யாமம்"},meridiemHour:function(t,e){return 12===t&&(t=0),"யாமம்"===e?t<2?t:t+12:"வைகறை"===e||"காலை"===e?t:"நண்பகல்"===e&&t>=10?t:t+12},week:{dow:0,doy:6}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("te",{months:"జనవరి_ఫిబ్రవరి_మార్చి_ఏప్రిల్_మే_జూన్_జులై_ఆగస్టు_సెప్టెంబర్_అక్టోబర్_నవంబర్_డిసెంబర్".split("_"),monthsShort:"జన._ఫిబ్ర._మార్చి_ఏప్రి._మే_జూన్_జులై_ఆగ._సెప్._అక్టో._నవ._డిసె.".split("_"),monthsParseExact:!0,weekdays:"ఆదివారం_సోమవారం_మంగళవారం_బుధవారం_గురువారం_శుక్రవారం_శనివారం".split("_"),weekdaysShort:"ఆది_సోమ_మంగళ_బుధ_గురు_శుక్ర_శని".split("_"),weekdaysMin:"ఆ_సో_మం_బు_గు_శు_శ".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[నేడు] LT",nextDay:"[రేపు] LT",nextWeek:"dddd, LT",lastDay:"[నిన్న] LT",lastWeek:"[గత] dddd, LT",sameElse:"L"},relativeTime:{future:"%s లో",past:"%s క్రితం",s:"కొన్ని క్షణాలు",ss:"%d సెకన్లు",m:"ఒక నిమిషం",mm:"%d నిమిషాలు",h:"ఒక గంట",hh:"%d గంటలు",d:"ఒక రోజు",dd:"%d రోజులు",M:"ఒక నెల",MM:"%d నెలలు",y:"ఒక సంవత్సరం",yy:"%d సంవత్సరాలు"},dayOfMonthOrdinalParse:/\d{1,2}వ/,ordinal:"%dవ",meridiemParse:/రాత్రి|ఉదయం|మధ్యాహ్నం|సాయంత్రం/,meridiemHour:function(t,e){return 12===t&&(t=0),"రాత్రి"===e?t<4?t:t+12:"ఉదయం"===e?t:"మధ్యాహ్నం"===e?t>=10?t:t+12:"సాయంత్రం"===e?t+12:void 0},meridiem:function(t,e,n){return t<4?"రాత్రి":t<10?"ఉదయం":t<17?"మధ్యాహ్నం":t<20?"సాయంత్రం":"రాత్రి"},week:{dow:0,doy:6}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("tet",{months:"Janeiru_Fevereiru_Marsu_Abril_Maiu_Juñu_Jullu_Agustu_Setembru_Outubru_Novembru_Dezembru".split("_"),monthsShort:"Jan_Fev_Mar_Abr_Mai_Jun_Jul_Ago_Set_Out_Nov_Dez".split("_"),weekdays:"Domingu_Segunda_Tersa_Kuarta_Kinta_Sesta_Sabadu".split("_"),weekdaysShort:"Dom_Seg_Ters_Kua_Kint_Sest_Sab".split("_"),weekdaysMin:"Do_Seg_Te_Ku_Ki_Ses_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Ohin iha] LT",nextDay:"[Aban iha] LT",nextWeek:"dddd [iha] LT",lastDay:"[Horiseik iha] LT",lastWeek:"dddd [semana kotuk] [iha] LT",sameElse:"L"},relativeTime:{future:"iha %s",past:"%s liuba",s:"minutu balun",ss:"minutu %d",m:"minutu ida",mm:"minutu %d",h:"oras ida",hh:"oras %d",d:"loron ida",dd:"loron %d",M:"fulan ida",MM:"fulan %d",y:"tinan ida",yy:"tinan %d"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(t){var e=t%10,n=1==~~(t%100/10)?"th":1===e?"st":2===e?"nd":3===e?"rd":"th";return t+n},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={0:"-ум",1:"-ум",2:"-юм",3:"-юм",4:"-ум",5:"-ум",6:"-ум",7:"-ум",8:"-ум",9:"-ум",10:"-ум",12:"-ум",13:"-ум",20:"-ум",30:"-юм",40:"-ум",50:"-ум",60:"-ум",70:"-ум",80:"-ум",90:"-ум",100:"-ум"};t.defineLocale("tg",{months:"январ_феврал_март_апрел_май_июн_июл_август_сентябр_октябр_ноябр_декабр".split("_"),monthsShort:"янв_фев_мар_апр_май_июн_июл_авг_сен_окт_ноя_дек".split("_"),weekdays:"якшанбе_душанбе_сешанбе_чоршанбе_панҷшанбе_ҷумъа_шанбе".split("_"),weekdaysShort:"яшб_дшб_сшб_чшб_пшб_ҷум_шнб".split("_"),weekdaysMin:"яш_дш_сш_чш_пш_ҷм_шб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Имрӯз соати] LT",nextDay:"[Пагоҳ соати] LT",lastDay:"[Дирӯз соати] LT",nextWeek:"dddd[и] [ҳафтаи оянда соати] LT",lastWeek:"dddd[и] [ҳафтаи гузашта соати] LT",sameElse:"L"},relativeTime:{future:"баъди %s",past:"%s пеш",s:"якчанд сония",m:"як дақиқа",mm:"%d дақиқа",h:"як соат",hh:"%d соат",d:"як рӯз",dd:"%d рӯз",M:"як моҳ",MM:"%d моҳ",y:"як сол",yy:"%d сол"},meridiemParse:/шаб|субҳ|рӯз|бегоҳ/,meridiemHour:function(t,e){return 12===t&&(t=0),"шаб"===e?t<4?t:t+12:"субҳ"===e?t:"рӯз"===e?t>=11?t:t+12:"бегоҳ"===e?t+12:void 0},meridiem:function(t,e,n){return t<4?"шаб":t<11?"субҳ":t<16?"рӯз":t<19?"бегоҳ":"шаб"},dayOfMonthOrdinalParse:/\d{1,2}-(ум|юм)/,ordinal:function(t){var n=t%10,r=t>=100?100:null;return t+(e[t]||e[n]||e[r])},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("th",{months:"มกราคม_กุมภาพันธ์_มีนาคม_เมษายน_พฤษภาคม_มิถุนายน_กรกฎาคม_สิงหาคม_กันยายน_ตุลาคม_พฤศจิกายน_ธันวาคม".split("_"),monthsShort:"ม.ค._ก.พ._มี.ค._เม.ย._พ.ค._มิ.ย._ก.ค._ส.ค._ก.ย._ต.ค._พ.ย._ธ.ค.".split("_"),monthsParseExact:!0,weekdays:"อาทิตย์_จันทร์_อังคาร_พุธ_พฤหัสบดี_ศุกร์_เสาร์".split("_"),weekdaysShort:"อาทิตย์_จันทร์_อังคาร_พุธ_พฤหัส_ศุกร์_เสาร์".split("_"),weekdaysMin:"อา._จ._อ._พ._พฤ._ศ._ส.".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY เวลา H:mm",LLLL:"วันddddที่ D MMMM YYYY เวลา H:mm"},meridiemParse:/ก่อนเที่ยง|หลังเที่ยง/,isPM:function(t){return"หลังเที่ยง"===t},meridiem:function(t,e,n){return t<12?"ก่อนเที่ยง":"หลังเที่ยง"},calendar:{sameDay:"[วันนี้ เวลา] LT",nextDay:"[พรุ่งนี้ เวลา] LT",nextWeek:"dddd[หน้า เวลา] LT",lastDay:"[เมื่อวานนี้ เวลา] LT",lastWeek:"[วัน]dddd[ที่แล้ว เวลา] LT",sameElse:"L"},relativeTime:{future:"อีก %s",past:"%sที่แล้ว",s:"ไม่กี่วินาที",ss:"%d วินาที",m:"1 นาที",mm:"%d นาที",h:"1 ชั่วโมง",hh:"%d ชั่วโมง",d:"1 วัน",dd:"%d วัน",M:"1 เดือน",MM:"%d เดือน",y:"1 ปี",yy:"%d ปี"}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("tl-ph",{months:"Enero_Pebrero_Marso_Abril_Mayo_Hunyo_Hulyo_Agosto_Setyembre_Oktubre_Nobyembre_Disyembre".split("_"),monthsShort:"Ene_Peb_Mar_Abr_May_Hun_Hul_Ago_Set_Okt_Nob_Dis".split("_"),weekdays:"Linggo_Lunes_Martes_Miyerkules_Huwebes_Biyernes_Sabado".split("_"),weekdaysShort:"Lin_Lun_Mar_Miy_Huw_Biy_Sab".split("_"),weekdaysMin:"Li_Lu_Ma_Mi_Hu_Bi_Sab".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"MM/D/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY HH:mm",LLLL:"dddd, MMMM DD, YYYY HH:mm"},calendar:{sameDay:"LT [ngayong araw]",nextDay:"[Bukas ng] LT",nextWeek:"LT [sa susunod na] dddd",lastDay:"LT [kahapon]",lastWeek:"LT [noong nakaraang] dddd",sameElse:"L"},relativeTime:{future:"sa loob ng %s",past:"%s ang nakalipas",s:"ilang segundo",ss:"%d segundo",m:"isang minuto",mm:"%d minuto",h:"isang oras",hh:"%d oras",d:"isang araw",dd:"%d araw",M:"isang buwan",MM:"%d buwan",y:"isang taon",yy:"%d taon"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(t){return t},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e="pagh_wa’_cha’_wej_loS_vagh_jav_Soch_chorgh_Hut".split("_");function n(t,n,r,i){var o=function(t){var n=Math.floor(t%1e3/100),r=Math.floor(t%100/10),i=t%10,o="";return n>0&&(o+=e[n]+"vatlh"),r>0&&(o+=(""!==o?" ":"")+e[r]+"maH"),i>0&&(o+=(""!==o?" ":"")+e[i]),""===o?"pagh":o}(t);switch(r){case"ss":return o+" lup";case"mm":return o+" tup";case"hh":return o+" rep";case"dd":return o+" jaj";case"MM":return o+" jar";case"yy":return o+" DIS"}}t.defineLocale("tlh",{months:"tera’ jar wa’_tera’ jar cha’_tera’ jar wej_tera’ jar loS_tera’ jar vagh_tera’ jar jav_tera’ jar Soch_tera’ jar chorgh_tera’ jar Hut_tera’ jar wa’maH_tera’ jar wa’maH wa’_tera’ jar wa’maH cha’".split("_"),monthsShort:"jar wa’_jar cha’_jar wej_jar loS_jar vagh_jar jav_jar Soch_jar chorgh_jar Hut_jar wa’maH_jar wa’maH wa’_jar wa’maH cha’".split("_"),monthsParseExact:!0,weekdays:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),weekdaysShort:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),weekdaysMin:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[DaHjaj] LT",nextDay:"[wa’leS] LT",nextWeek:"LLL",lastDay:"[wa’Hu’] LT",lastWeek:"LLL",sameElse:"L"},relativeTime:{future:function(t){var e=t;return e=-1!==t.indexOf("jaj")?e.slice(0,-3)+"leS":-1!==t.indexOf("jar")?e.slice(0,-3)+"waQ":-1!==t.indexOf("DIS")?e.slice(0,-3)+"nem":e+" pIq"},past:function(t){var e=t;return e=-1!==t.indexOf("jaj")?e.slice(0,-3)+"Hu’":-1!==t.indexOf("jar")?e.slice(0,-3)+"wen":-1!==t.indexOf("DIS")?e.slice(0,-3)+"ben":e+" ret"},s:"puS lup",ss:n,m:"wa’ tup",mm:n,h:"wa’ rep",hh:n,d:"wa’ jaj",dd:n,M:"wa’ jar",MM:n,y:"wa’ DIS",yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";var e={1:"'inci",5:"'inci",8:"'inci",70:"'inci",80:"'inci",2:"'nci",7:"'nci",20:"'nci",50:"'nci",3:"'üncü",4:"'üncü",100:"'üncü",6:"'ncı",9:"'uncu",10:"'uncu",30:"'uncu",60:"'ıncı",90:"'ıncı"};t.defineLocale("tr",{months:"Ocak_Şubat_Mart_Nisan_Mayıs_Haziran_Temmuz_Ağustos_Eylül_Ekim_Kasım_Aralık".split("_"),monthsShort:"Oca_Şub_Mar_Nis_May_Haz_Tem_Ağu_Eyl_Eki_Kas_Ara".split("_"),weekdays:"Pazar_Pazartesi_Salı_Çarşamba_Perşembe_Cuma_Cumartesi".split("_"),weekdaysShort:"Paz_Pts_Sal_Çar_Per_Cum_Cts".split("_"),weekdaysMin:"Pz_Pt_Sa_Ça_Pe_Cu_Ct".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün saat] LT",nextDay:"[yarın saat] LT",nextWeek:"[gelecek] dddd [saat] LT",lastDay:"[dün] LT",lastWeek:"[geçen] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s sonra",past:"%s önce",s:"birkaç saniye",ss:"%d saniye",m:"bir dakika",mm:"%d dakika",h:"bir saat",hh:"%d saat",d:"bir gün",dd:"%d gün",M:"bir ay",MM:"%d ay",y:"bir yıl",yy:"%d yıl"},ordinal:function(t,n){switch(n){case"d":case"D":case"Do":case"DD":return t;default:if(0===t)return t+"'ıncı";var r=t%10,i=t%100-r,o=t>=100?100:null;return t+(e[r]||e[i]||e[o])}},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n,r){var i={s:["viensas secunds","'iensas secunds"],ss:[t+" secunds",t+" secunds"],m:["'n míut","'iens míut"],mm:[t+" míuts",t+" míuts"],h:["'n þora","'iensa þora"],hh:[t+" þoras",t+" þoras"],d:["'n ziua","'iensa ziua"],dd:[t+" ziuas",t+" ziuas"],M:["'n mes","'iens mes"],MM:[t+" mesen",t+" mesen"],y:["'n ar","'iens ar"],yy:[t+" ars",t+" ars"]};return r?i[n][0]:e?i[n][0]:i[n][1]}t.defineLocale("tzl",{months:"Januar_Fevraglh_Març_Avrïu_Mai_Gün_Julia_Guscht_Setemvar_Listopäts_Noemvar_Zecemvar".split("_"),monthsShort:"Jan_Fev_Mar_Avr_Mai_Gün_Jul_Gus_Set_Lis_Noe_Zec".split("_"),weekdays:"Súladi_Lúneçi_Maitzi_Márcuri_Xhúadi_Viénerçi_Sáturi".split("_"),weekdaysShort:"Súl_Lún_Mai_Már_Xhú_Vié_Sát".split("_"),weekdaysMin:"Sú_Lú_Ma_Má_Xh_Vi_Sá".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD.MM.YYYY",LL:"D. MMMM [dallas] YYYY",LLL:"D. MMMM [dallas] YYYY HH.mm",LLLL:"dddd, [li] D. MMMM [dallas] YYYY HH.mm"},meridiemParse:/d\'o|d\'a/i,isPM:function(t){return"d'o"===t.toLowerCase()},meridiem:function(t,e,n){return t>11?n?"d'o":"D'O":n?"d'a":"D'A"},calendar:{sameDay:"[oxhi à] LT",nextDay:"[demà à] LT",nextWeek:"dddd [à] LT",lastDay:"[ieiri à] LT",lastWeek:"[sür el] dddd [lasteu à] LT",sameElse:"L"},relativeTime:{future:"osprei %s",past:"ja%s",s:e,ss:e,m:e,mm:e,h:e,hh:e,d:e,dd:e,M:e,MM:e,y:e,yy:e},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("tzm",{months:"ⵉⵏⵏⴰⵢⵔ_ⴱⵕⴰⵢⵕ_ⵎⴰⵕⵚ_ⵉⴱⵔⵉⵔ_ⵎⴰⵢⵢⵓ_ⵢⵓⵏⵢⵓ_ⵢⵓⵍⵢⵓⵣ_ⵖⵓⵛⵜ_ⵛⵓⵜⴰⵏⴱⵉⵔ_ⴽⵟⵓⴱⵕ_ⵏⵓⵡⴰⵏⴱⵉⵔ_ⴷⵓⵊⵏⴱⵉⵔ".split("_"),monthsShort:"ⵉⵏⵏⴰⵢⵔ_ⴱⵕⴰⵢⵕ_ⵎⴰⵕⵚ_ⵉⴱⵔⵉⵔ_ⵎⴰⵢⵢⵓ_ⵢⵓⵏⵢⵓ_ⵢⵓⵍⵢⵓⵣ_ⵖⵓⵛⵜ_ⵛⵓⵜⴰⵏⴱⵉⵔ_ⴽⵟⵓⴱⵕ_ⵏⵓⵡⴰⵏⴱⵉⵔ_ⴷⵓⵊⵏⴱⵉⵔ".split("_"),weekdays:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),weekdaysShort:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),weekdaysMin:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[ⴰⵙⴷⵅ ⴴ] LT",nextDay:"[ⴰⵙⴽⴰ ⴴ] LT",nextWeek:"dddd [ⴴ] LT",lastDay:"[ⴰⵚⴰⵏⵜ ⴴ] LT",lastWeek:"dddd [ⴴ] LT",sameElse:"L"},relativeTime:{future:"ⴷⴰⴷⵅ ⵙ ⵢⴰⵏ %s",past:"ⵢⴰⵏ %s",s:"ⵉⵎⵉⴽ",ss:"%d ⵉⵎⵉⴽ",m:"ⵎⵉⵏⵓⴺ",mm:"%d ⵎⵉⵏⵓⴺ",h:"ⵙⴰⵄⴰ",hh:"%d ⵜⴰⵙⵙⴰⵄⵉⵏ",d:"ⴰⵙⵙ",dd:"%d oⵙⵙⴰⵏ",M:"ⴰⵢoⵓⵔ",MM:"%d ⵉⵢⵢⵉⵔⵏ",y:"ⴰⵙⴳⴰⵙ",yy:"%d ⵉⵙⴳⴰⵙⵏ"},week:{dow:6,doy:12}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("tzm-latn",{months:"innayr_brˤayrˤ_marˤsˤ_ibrir_mayyw_ywnyw_ywlywz_ɣwšt_šwtanbir_ktˤwbrˤ_nwwanbir_dwjnbir".split("_"),monthsShort:"innayr_brˤayrˤ_marˤsˤ_ibrir_mayyw_ywnyw_ywlywz_ɣwšt_šwtanbir_ktˤwbrˤ_nwwanbir_dwjnbir".split("_"),weekdays:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),weekdaysShort:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),weekdaysMin:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[asdkh g] LT",nextDay:"[aska g] LT",nextWeek:"dddd [g] LT",lastDay:"[assant g] LT",lastWeek:"dddd [g] LT",sameElse:"L"},relativeTime:{future:"dadkh s yan %s",past:"yan %s",s:"imik",ss:"%d imik",m:"minuḍ",mm:"%d minuḍ",h:"saɛa",hh:"%d tassaɛin",d:"ass",dd:"%d ossan",M:"ayowr",MM:"%d iyyirn",y:"asgas",yy:"%d isgasn"},week:{dow:6,doy:12}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("ug-cn",{months:"يانۋار_فېۋرال_مارت_ئاپرېل_ماي_ئىيۇن_ئىيۇل_ئاۋغۇست_سېنتەبىر_ئۆكتەبىر_نويابىر_دېكابىر".split("_"),monthsShort:"يانۋار_فېۋرال_مارت_ئاپرېل_ماي_ئىيۇن_ئىيۇل_ئاۋغۇست_سېنتەبىر_ئۆكتەبىر_نويابىر_دېكابىر".split("_"),weekdays:"يەكشەنبە_دۈشەنبە_سەيشەنبە_چارشەنبە_پەيشەنبە_جۈمە_شەنبە".split("_"),weekdaysShort:"يە_دۈ_سە_چا_پە_جۈ_شە".split("_"),weekdaysMin:"يە_دۈ_سە_چا_پە_جۈ_شە".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY-يىلىM-ئاينىڭD-كۈنى",LLL:"YYYY-يىلىM-ئاينىڭD-كۈنى، HH:mm",LLLL:"dddd، YYYY-يىلىM-ئاينىڭD-كۈنى، HH:mm"},meridiemParse:/يېرىم كېچە|سەھەر|چۈشتىن بۇرۇن|چۈش|چۈشتىن كېيىن|كەچ/,meridiemHour:function(t,e){return 12===t&&(t=0),"يېرىم كېچە"===e||"سەھەر"===e||"چۈشتىن بۇرۇن"===e?t:"چۈشتىن كېيىن"===e||"كەچ"===e?t+12:t>=11?t:t+12},meridiem:function(t,e,n){var r=100*t+e;return r<600?"يېرىم كېچە":r<900?"سەھەر":r<1130?"چۈشتىن بۇرۇن":r<1230?"چۈش":r<1800?"چۈشتىن كېيىن":"كەچ"},calendar:{sameDay:"[بۈگۈن سائەت] LT",nextDay:"[ئەتە سائەت] LT",nextWeek:"[كېلەركى] dddd [سائەت] LT",lastDay:"[تۆنۈگۈن] LT",lastWeek:"[ئالدىنقى] dddd [سائەت] LT",sameElse:"L"},relativeTime:{future:"%s كېيىن",past:"%s بۇرۇن",s:"نەچچە سېكونت",ss:"%d سېكونت",m:"بىر مىنۇت",mm:"%d مىنۇت",h:"بىر سائەت",hh:"%d سائەت",d:"بىر كۈن",dd:"%d كۈن",M:"بىر ئاي",MM:"%d ئاي",y:"بىر يىل",yy:"%d يىل"},dayOfMonthOrdinalParse:/\d{1,2}(-كۈنى|-ئاي|-ھەپتە)/,ordinal:function(t,e){switch(e){case"d":case"D":case"DDD":return t+"-كۈنى";case"w":case"W":return t+"-ھەپتە";default:return t}},preparse:function(t){return t.replace(/،/g,",")},postformat:function(t){return t.replace(/,/g,"،")},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";function e(t,e,n){var r,i,o={ss:e?"секунда_секунди_секунд":"секунду_секунди_секунд",mm:e?"хвилина_хвилини_хвилин":"хвилину_хвилини_хвилин",hh:e?"година_години_годин":"годину_години_годин",dd:"день_дні_днів",MM:"місяць_місяці_місяців",yy:"рік_роки_років"};return"m"===n?e?"хвилина":"хвилину":"h"===n?e?"година":"годину":t+" "+(r=+t,i=o[n].split("_"),r%10==1&&r%100!=11?i[0]:r%10>=2&&r%10<=4&&(r%100<10||r%100>=20)?i[1]:i[2])}function n(t){return function(){return t+"о"+(11===this.hours()?"б":"")+"] LT"}}t.defineLocale("uk",{months:{format:"січня_лютого_березня_квітня_травня_червня_липня_серпня_вересня_жовтня_листопада_грудня".split("_"),standalone:"січень_лютий_березень_квітень_травень_червень_липень_серпень_вересень_жовтень_листопад_грудень".split("_")},monthsShort:"січ_лют_бер_квіт_трав_черв_лип_серп_вер_жовт_лист_груд".split("_"),weekdays:function(t,e){var n={nominative:"неділя_понеділок_вівторок_середа_четвер_п’ятниця_субота".split("_"),accusative:"неділю_понеділок_вівторок_середу_четвер_п’ятницю_суботу".split("_"),genitive:"неділі_понеділка_вівторка_середи_четверга_п’ятниці_суботи".split("_")};if(!0===t)return n.nominative.slice(1,7).concat(n.nominative.slice(0,1));if(!t)return n.nominative;var r=/(\[[ВвУу]\]) ?dddd/.test(e)?"accusative":/\[?(?:минулої|наступної)? ?\] ?dddd/.test(e)?"genitive":"nominative";return n[r][t.day()]},weekdaysShort:"нд_пн_вт_ср_чт_пт_сб".split("_"),weekdaysMin:"нд_пн_вт_ср_чт_пт_сб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY р.",LLL:"D MMMM YYYY р., HH:mm",LLLL:"dddd, D MMMM YYYY р., HH:mm"},calendar:{sameDay:n("[Сьогодні "),nextDay:n("[Завтра "),lastDay:n("[Вчора "),nextWeek:n("[У] dddd ["),lastWeek:function(){switch(this.day()){case 0:case 3:case 5:case 6:return n("[Минулої] dddd [").call(this);case 1:case 2:case 4:return n("[Минулого] dddd [").call(this)}},sameElse:"L"},relativeTime:{future:"за %s",past:"%s тому",s:"декілька секунд",ss:e,m:e,mm:e,h:"годину",hh:e,d:"день",dd:e,M:"місяць",MM:e,y:"рік",yy:e},meridiemParse:/ночі|ранку|дня|вечора/,isPM:function(t){return/^(дня|вечора)$/.test(t)},meridiem:function(t,e,n){return t<4?"ночі":t<12?"ранку":t<17?"дня":"вечора"},dayOfMonthOrdinalParse:/\d{1,2}-(й|го)/,ordinal:function(t,e){switch(e){case"M":case"d":case"DDD":case"w":case"W":return t+"-й";case"D":return t+"-го";default:return t}},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";var e=["جنوری","فروری","مارچ","اپریل","مئی","جون","جولائی","اگست","ستمبر","اکتوبر","نومبر","دسمبر"],n=["اتوار","پیر","منگل","بدھ","جمعرات","جمعہ","ہفتہ"];t.defineLocale("ur",{months:e,monthsShort:e,weekdays:n,weekdaysShort:n,weekdaysMin:n,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd، D MMMM YYYY HH:mm"},meridiemParse:/صبح|شام/,isPM:function(t){return"شام"===t},meridiem:function(t,e,n){return t<12?"صبح":"شام"},calendar:{sameDay:"[آج بوقت] LT",nextDay:"[کل بوقت] LT",nextWeek:"dddd [بوقت] LT",lastDay:"[گذشتہ روز بوقت] LT",lastWeek:"[گذشتہ] dddd [بوقت] LT",sameElse:"L"},relativeTime:{future:"%s بعد",past:"%s قبل",s:"چند سیکنڈ",ss:"%d سیکنڈ",m:"ایک منٹ",mm:"%d منٹ",h:"ایک گھنٹہ",hh:"%d گھنٹے",d:"ایک دن",dd:"%d دن",M:"ایک ماہ",MM:"%d ماہ",y:"ایک سال",yy:"%d سال"},preparse:function(t){return t.replace(/،/g,",")},postformat:function(t){return t.replace(/,/g,"،")},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("uz",{months:"январ_феврал_март_апрел_май_июн_июл_август_сентябр_октябр_ноябр_декабр".split("_"),monthsShort:"янв_фев_мар_апр_май_июн_июл_авг_сен_окт_ноя_дек".split("_"),weekdays:"Якшанба_Душанба_Сешанба_Чоршанба_Пайшанба_Жума_Шанба".split("_"),weekdaysShort:"Якш_Душ_Сеш_Чор_Пай_Жум_Шан".split("_"),weekdaysMin:"Як_Ду_Се_Чо_Па_Жу_Ша".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"D MMMM YYYY, dddd HH:mm"},calendar:{sameDay:"[Бугун соат] LT [да]",nextDay:"[Эртага] LT [да]",nextWeek:"dddd [куни соат] LT [да]",lastDay:"[Кеча соат] LT [да]",lastWeek:"[Утган] dddd [куни соат] LT [да]",sameElse:"L"},relativeTime:{future:"Якин %s ичида",past:"Бир неча %s олдин",s:"фурсат",ss:"%d фурсат",m:"бир дакика",mm:"%d дакика",h:"бир соат",hh:"%d соат",d:"бир кун",dd:"%d кун",M:"бир ой",MM:"%d ой",y:"бир йил",yy:"%d йил"},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("uz-latn",{months:"Yanvar_Fevral_Mart_Aprel_May_Iyun_Iyul_Avgust_Sentabr_Oktabr_Noyabr_Dekabr".split("_"),monthsShort:"Yan_Fev_Mar_Apr_May_Iyun_Iyul_Avg_Sen_Okt_Noy_Dek".split("_"),weekdays:"Yakshanba_Dushanba_Seshanba_Chorshanba_Payshanba_Juma_Shanba".split("_"),weekdaysShort:"Yak_Dush_Sesh_Chor_Pay_Jum_Shan".split("_"),weekdaysMin:"Ya_Du_Se_Cho_Pa_Ju_Sha".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"D MMMM YYYY, dddd HH:mm"},calendar:{sameDay:"[Bugun soat] LT [da]",nextDay:"[Ertaga] LT [da]",nextWeek:"dddd [kuni soat] LT [da]",lastDay:"[Kecha soat] LT [da]",lastWeek:"[O'tgan] dddd [kuni soat] LT [da]",sameElse:"L"},relativeTime:{future:"Yaqin %s ichida",past:"Bir necha %s oldin",s:"soniya",ss:"%d soniya",m:"bir daqiqa",mm:"%d daqiqa",h:"bir soat",hh:"%d soat",d:"bir kun",dd:"%d kun",M:"bir oy",MM:"%d oy",y:"bir yil",yy:"%d yil"},week:{dow:1,doy:7}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("vi",{months:"tháng 1_tháng 2_tháng 3_tháng 4_tháng 5_tháng 6_tháng 7_tháng 8_tháng 9_tháng 10_tháng 11_tháng 12".split("_"),monthsShort:"Th01_Th02_Th03_Th04_Th05_Th06_Th07_Th08_Th09_Th10_Th11_Th12".split("_"),monthsParseExact:!0,weekdays:"chủ nhật_thứ hai_thứ ba_thứ tư_thứ năm_thứ sáu_thứ bảy".split("_"),weekdaysShort:"CN_T2_T3_T4_T5_T6_T7".split("_"),weekdaysMin:"CN_T2_T3_T4_T5_T6_T7".split("_"),weekdaysParseExact:!0,meridiemParse:/sa|ch/i,isPM:function(t){return/^ch$/i.test(t)},meridiem:function(t,e,n){return t<12?n?"sa":"SA":n?"ch":"CH"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [năm] YYYY",LLL:"D MMMM [năm] YYYY HH:mm",LLLL:"dddd, D MMMM [năm] YYYY HH:mm",l:"DD/M/YYYY",ll:"D MMM YYYY",lll:"D MMM YYYY HH:mm",llll:"ddd, D MMM YYYY HH:mm"},calendar:{sameDay:"[Hôm nay lúc] LT",nextDay:"[Ngày mai lúc] LT",nextWeek:"dddd [tuần tới lúc] LT",lastDay:"[Hôm qua lúc] LT",lastWeek:"dddd [tuần rồi lúc] LT",sameElse:"L"},relativeTime:{future:"%s tới",past:"%s trước",s:"vài giây",ss:"%d giây",m:"một phút",mm:"%d phút",h:"một giờ",hh:"%d giờ",d:"một ngày",dd:"%d ngày",M:"một tháng",MM:"%d tháng",y:"một năm",yy:"%d năm"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(t){return t},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("x-pseudo",{months:"J~áñúá~rý_F~ébrú~árý_~Márc~h_Áp~ríl_~Máý_~Júñé~_Júl~ý_Áú~gúst~_Sép~témb~ér_Ó~ctób~ér_Ñ~óvém~bér_~Décé~mbér".split("_"),monthsShort:"J~áñ_~Féb_~Már_~Ápr_~Máý_~Júñ_~Júl_~Áúg_~Sép_~Óct_~Ñóv_~Déc".split("_"),monthsParseExact:!0,weekdays:"S~úñdá~ý_Mó~ñdáý~_Túé~sdáý~_Wéd~ñésd~áý_T~húrs~dáý_~Fríd~áý_S~átúr~dáý".split("_"),weekdaysShort:"S~úñ_~Móñ_~Túé_~Wéd_~Thú_~Frí_~Sát".split("_"),weekdaysMin:"S~ú_Mó~_Tú_~Wé_T~h_Fr~_Sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[T~ódá~ý át] LT",nextDay:"[T~ómó~rró~w át] LT",nextWeek:"dddd [át] LT",lastDay:"[Ý~ést~érdá~ý át] LT",lastWeek:"[L~ást] dddd [át] LT",sameElse:"L"},relativeTime:{future:"í~ñ %s",past:"%s á~gó",s:"á ~féw ~sécó~ñds",ss:"%d s~écóñ~ds",m:"á ~míñ~úté",mm:"%d m~íñú~tés",h:"á~ñ hó~úr",hh:"%d h~óúrs",d:"á ~dáý",dd:"%d d~áýs",M:"á ~móñ~th",MM:"%d m~óñt~hs",y:"á ~ýéár",yy:"%d ý~éárs"},dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(t){var e=t%10,n=1==~~(t%100/10)?"th":1===e?"st":2===e?"nd":3===e?"rd":"th";return t+n},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("yo",{months:"Sẹ́rẹ́_Èrèlè_Ẹrẹ̀nà_Ìgbé_Èbibi_Òkùdu_Agẹmo_Ògún_Owewe_Ọ̀wàrà_Bélú_Ọ̀pẹ̀̀".split("_"),monthsShort:"Sẹ́r_Èrl_Ẹrn_Ìgb_Èbi_Òkù_Agẹ_Ògú_Owe_Ọ̀wà_Bél_Ọ̀pẹ̀̀".split("_"),weekdays:"Àìkú_Ajé_Ìsẹ́gun_Ọjọ́rú_Ọjọ́bọ_Ẹtì_Àbámẹ́ta".split("_"),weekdaysShort:"Àìk_Ajé_Ìsẹ́_Ọjr_Ọjb_Ẹtì_Àbá".split("_"),weekdaysMin:"Àì_Aj_Ìs_Ọr_Ọb_Ẹt_Àb".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Ònì ni] LT",nextDay:"[Ọ̀la ni] LT",nextWeek:"dddd [Ọsẹ̀ tón'bọ] [ni] LT",lastDay:"[Àna ni] LT",lastWeek:"dddd [Ọsẹ̀ tólọ́] [ni] LT",sameElse:"L"},relativeTime:{future:"ní %s",past:"%s kọjá",s:"ìsẹjú aayá die",ss:"aayá %d",m:"ìsẹjú kan",mm:"ìsẹjú %d",h:"wákati kan",hh:"wákati %d",d:"ọjọ́ kan",dd:"ọjọ́ %d",M:"osù kan",MM:"osù %d",y:"ọdún kan",yy:"ọdún %d"},dayOfMonthOrdinalParse:/ọjọ́\s\d{1,2}/,ordinal:"ọjọ́ %d",week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("zh-cn",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"周日_周一_周二_周三_周四_周五_周六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日Ah点mm分",LLLL:"YYYY年M月D日ddddAh点mm分",l:"YYYY/M/D",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(t,e){return 12===t&&(t=0),"凌晨"===e||"早上"===e||"上午"===e?t:"下午"===e||"晚上"===e?t+12:t>=11?t:t+12},meridiem:function(t,e,n){var r=100*t+e;return r<600?"凌晨":r<900?"早上":r<1130?"上午":r<1230?"中午":r<1800?"下午":"晚上"},calendar:{sameDay:"[今天]LT",nextDay:"[明天]LT",nextWeek:"[下]ddddLT",lastDay:"[昨天]LT",lastWeek:"[上]ddddLT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|周)/,ordinal:function(t,e){switch(e){case"d":case"D":case"DDD":return t+"日";case"M":return t+"月";case"w":case"W":return t+"周";default:return t}},relativeTime:{future:"%s内",past:"%s前",s:"几秒",ss:"%d 秒",m:"1 分钟",mm:"%d 分钟",h:"1 小时",hh:"%d 小时",d:"1 天",dd:"%d 天",M:"1 个月",MM:"%d 个月",y:"1 年",yy:"%d 年"},week:{dow:1,doy:4}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("zh-hk",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日dddd HH:mm",l:"YYYY/M/D",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(t,e){return 12===t&&(t=0),"凌晨"===e||"早上"===e||"上午"===e?t:"中午"===e?t>=11?t:t+12:"下午"===e||"晚上"===e?t+12:void 0},meridiem:function(t,e,n){var r=100*t+e;return r<600?"凌晨":r<900?"早上":r<1130?"上午":r<1230?"中午":r<1800?"下午":"晚上"},calendar:{sameDay:"[今天]LT",nextDay:"[明天]LT",nextWeek:"[下]ddddLT",lastDay:"[昨天]LT",lastWeek:"[上]ddddLT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(t,e){switch(e){case"d":case"D":case"DDD":return t+"日";case"M":return t+"月";case"w":case"W":return t+"週";default:return t}},relativeTime:{future:"%s內",past:"%s前",s:"幾秒",ss:"%d 秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}})}(n(7))},function(t,e,n){!function(t){"use strict";t.defineLocale("zh-tw",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日dddd HH:mm",l:"YYYY/M/D",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(t,e){return 12===t&&(t=0),"凌晨"===e||"早上"===e||"上午"===e?t:"中午"===e?t>=11?t:t+12:"下午"===e||"晚上"===e?t+12:void 0},meridiem:function(t,e,n){var r=100*t+e;return r<600?"凌晨":r<900?"早上":r<1130?"上午":r<1230?"中午":r<1800?"下午":"晚上"},calendar:{sameDay:"[今天] LT",nextDay:"[明天] LT",nextWeek:"[下]dddd LT",lastDay:"[昨天] LT",lastWeek:"[上]dddd LT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(t,e){switch(e){case"d":case"D":case"DDD":return t+"日";case"M":return t+"月";case"w":case"W":return t+"週";default:return t}},relativeTime:{future:"%s內",past:"%s前",s:"幾秒",ss:"%d 秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}})}(n(7))},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(948),""),e.i(n(949),""),e.i(n(106),""),e.i(n(950),""),e.push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/* This file was auto-generated by ensurePackage() in @jupyterlab/buildutils */\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\r\n| Copyright (c) 2014-2017, PhosphorJS Contributors\r\n|\r\n| Distributed under the terms of the BSD 3-Clause License.\r\n|\r\n| The full license is in the file LICENSE, distributed with this software.\r\n|----------------------------------------------------------------------------*/\r\n\r\n\r\nbody.p-mod-override-cursor * {\r\n  cursor: inherit !important;\r\n}\r\n",""])},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(106),""),e.i(n(173),""),e.i(n(1004),""),e.push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/* This file was auto-generated by ensurePackage() in @jupyterlab/buildutils */\n",""])},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r,i=n(5),o=n(9),a=n(50),s=n(46),A=n(290),l=n(10),c=function(){function t(){this._timerID=0,this._replaying=!1,this._keystrokes=[],this._keydownEvents=[],this._keyBindings=[],this._exactKeyMatch=null,this._commands=Object.create(null),this._commandChanged=new l.Signal(this),this._commandExecuted=new l.Signal(this),this._keyBindingChanged=new l.Signal(this)}return Object.defineProperty(t.prototype,"commandChanged",{get:function(){return this._commandChanged},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"commandExecuted",{get:function(){return this._commandExecuted},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"keyBindingChanged",{get:function(){return this._keyBindingChanged},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"keyBindings",{get:function(){return this._keyBindings},enumerable:!0,configurable:!0}),t.prototype.listCommands=function(){return Object.keys(this._commands)},t.prototype.hasCommand=function(t){return t in this._commands},t.prototype.addCommand=function(t,e){var n=this;if(t in this._commands)throw new Error("Command '"+t+"' already registered.");return this._commands[t]=r.createCommand(e),this._commandChanged.emit({id:t,type:"added"}),new a.DisposableDelegate(function(){delete n._commands[t],n._commandChanged.emit({id:t,type:"removed"})})},t.prototype.notifyCommandChanged=function(t){if(void 0!==t&&!(t in this._commands))throw new Error("Command '"+t+"' is not registered.");this._commandChanged.emit({id:t,type:t?"changed":"many-changed"})},t.prototype.label=function(t,e){void 0===e&&(e=o.JSONExt.emptyObject);var n=this._commands[t];return n?n.label.call(void 0,e):""},t.prototype.mnemonic=function(t,e){void 0===e&&(e=o.JSONExt.emptyObject);var n=this._commands[t];return n?n.mnemonic.call(void 0,e):-1},t.prototype.icon=function(t,e){return void 0===e&&(e=o.JSONExt.emptyObject),this.iconClass(t,e)},t.prototype.iconClass=function(t,e){void 0===e&&(e=o.JSONExt.emptyObject);var n=this._commands[t];return n?n.iconClass.call(void 0,e):""},t.prototype.iconLabel=function(t,e){void 0===e&&(e=o.JSONExt.emptyObject);var n=this._commands[t];return n?n.iconLabel.call(void 0,e):""},t.prototype.caption=function(t,e){void 0===e&&(e=o.JSONExt.emptyObject);var n=this._commands[t];return n?n.caption.call(void 0,e):""},t.prototype.usage=function(t,e){void 0===e&&(e=o.JSONExt.emptyObject);var n=this._commands[t];return n?n.usage.call(void 0,e):""},t.prototype.className=function(t,e){void 0===e&&(e=o.JSONExt.emptyObject);var n=this._commands[t];return n?n.className.call(void 0,e):""},t.prototype.dataset=function(t,e){void 0===e&&(e=o.JSONExt.emptyObject);var n=this._commands[t];return n?n.dataset.call(void 0,e):{}},t.prototype.isEnabled=function(t,e){void 0===e&&(e=o.JSONExt.emptyObject);var n=this._commands[t];return!!n&&n.isEnabled.call(void 0,e)},t.prototype.isToggled=function(t,e){void 0===e&&(e=o.JSONExt.emptyObject);var n=this._commands[t];return!!n&&n.isToggled.call(void 0,e)},t.prototype.isVisible=function(t,e){void 0===e&&(e=o.JSONExt.emptyObject);var n=this._commands[t];return!!n&&n.isVisible.call(void 0,e)},t.prototype.execute=function(t,e){void 0===e&&(e=o.JSONExt.emptyObject);var n,r=this._commands[t];if(!r)return Promise.reject(new Error("Command '"+t+"' not registered."));try{n=r.execute.call(void 0,e)}catch(t){n=Promise.reject(t)}var i=Promise.resolve(n);return this._commandExecuted.emit({id:t,args:e,result:i}),i},t.prototype.addKeyBinding=function(t){var e=this,n=r.createKeyBinding(t);return this._keyBindings.push(n),this._keyBindingChanged.emit({binding:n,type:"added"}),new a.DisposableDelegate(function(){i.ArrayExt.removeFirstOf(e._keyBindings,n),e._keyBindingChanged.emit({binding:n,type:"removed"})})},t.prototype.processKeydownEvent=function(e){if(!this._replaying){var n=t.keystrokeForKeydownEvent(e);if(!n)return this._replayKeydownEvents(),void this._clearPendingState();this._keystrokes.push(n);var i=r.matchKeyBinding(this._keyBindings,this._keystrokes,e),o=i.exact,a=i.partial;if(!o&&!a)return this._replayKeydownEvents(),void this._clearPendingState();if(e.preventDefault(),e.stopPropagation(),o&&!a)return this._executeKeyBinding(o),void this._clearPendingState();o&&(this._exactKeyMatch=o),this._keydownEvents.push(e),this._startTimer()}},t.prototype._startTimer=function(){var t=this;this._clearTimer(),this._timerID=window.setTimeout(function(){t._onPendingTimeout()},r.CHORD_TIMEOUT)},t.prototype._clearTimer=function(){0!==this._timerID&&(clearTimeout(this._timerID),this._timerID=0)},t.prototype._replayKeydownEvents=function(){0!==this._keydownEvents.length&&(this._replaying=!0,this._keydownEvents.forEach(r.replayKeyEvent),this._replaying=!1)},t.prototype._executeKeyBinding=function(t){var e=t.command,n=t.args;if(this.hasCommand(e)&&this.isEnabled(e,n))this.execute(e,n);else{var r=this.hasCommand(e)?"enabled":"registered",i="Cannot execute key binding '"+t.keys.join(", ")+"':",o="command '"+e+"' is not "+r+".";console.warn(i+" "+o)}},t.prototype._clearPendingState=function(){this._clearTimer(),this._exactKeyMatch=null,this._keystrokes.length=0,this._keydownEvents.length=0},t.prototype._onPendingTimeout=function(){this._timerID=0,this._exactKeyMatch?this._executeKeyBinding(this._exactKeyMatch):this._replayKeydownEvents(),this._clearPendingState()},t}();e.CommandRegistry=c,function(t){function e(t){for(var e="",n=!1,r=!1,i=!1,o=!1,a=0,A=t.split(/\s+/);a<A.length;a++){var l=A[a];"Accel"===l?s.Platform.IS_MAC?r=!0:i=!0:"Alt"===l?n=!0:"Cmd"===l?r=!0:"Ctrl"===l?i=!0:"Shift"===l?o=!0:l.length>0&&(e=l)}return{cmd:r,ctrl:i,alt:n,shift:o,key:e}}t.parseKeystroke=e,t.normalizeKeystroke=function(t){var n="",r=e(t);return r.ctrl&&(n+="Ctrl "),r.alt&&(n+="Alt "),r.shift&&(n+="Shift "),r.cmd&&s.Platform.IS_MAC&&(n+="Cmd "),n+r.key},t.formatKeystroke=function(t){var n="",r=e(t);return s.Platform.IS_MAC?(r.ctrl&&(n+="⌃ "),r.alt&&(n+="⌥ "),r.shift&&(n+="⇧ "),r.cmd&&(n+="⌘ ")):(r.ctrl&&(n+="Ctrl+"),r.alt&&(n+="Alt+"),r.shift&&(n+="Shift+")),n+r.key},t.keystrokeForKeydownEvent=function(t){var e=A.getKeyboardLayout().keyForKeydownEvent(t);if(!e)return"";var n="";return t.ctrlKey&&(n+="Ctrl "),t.altKey&&(n+="Alt "),t.shiftKey&&(n+="Shift "),t.metaKey&&s.Platform.IS_MAC&&(n+="Cmd "),n+e}}(c=e.CommandRegistry||(e.CommandRegistry={})),e.CommandRegistry=c,function(t){t.CHORD_TIMEOUT=1e3,t.createCommand=function(t){return{execute:t.execute,label:A(t.label,e),mnemonic:A(t.mnemonic,n),iconClass:A(t.iconClass||t.icon,e),iconLabel:A(t.iconLabel,e),caption:A(t.caption,e),usage:A(t.usage,e),className:A(t.className,e),dataset:A(t.dataset,a),isEnabled:t.isEnabled||r,isToggled:t.isToggled||i,isVisible:t.isVisible||r}},t.createKeyBinding=function(t){return{keys:l(t),selector:u(t),command:t.command,args:t.args||o.JSONExt.emptyObject}},t.matchKeyBinding=function(t,e,n){for(var r=null,i=!1,o=1/0,a=0,A=0,l=t.length;A<l;++A){var c=t[A],u=g(c.keys,e);if(0!==u)if(2!==u){var d=h(c.selector,n);if(!(-1===d||d>o)){var f=s.Selector.calculateSpecificity(c.selector);(!r||d<o||f>=a)&&(r=c,o=d,a=f)}}else i||-1===h(c.selector,n)||(i=!0)}return{exact:r,partial:i}},t.replayKeyEvent=function(t){t.target.dispatchEvent(function(t){var e=document.createEvent("Event"),n=t.bubbles||!0,r=t.cancelable||!0;return e.initEvent(t.type||"keydown",n,r),e.key=t.key||"",e.keyCode=t.keyCode||0,e.which=t.keyCode||0,e.ctrlKey=t.ctrlKey||!1,e.altKey=t.altKey||!1,e.shiftKey=t.shiftKey||!1,e.metaKey=t.metaKey||!1,e.view=t.view||window,e}(t))};var e=function(){return""},n=function(){return-1},r=function(){return!0},i=function(){return!1},a=function(){return{}};function A(t,e){return void 0===t?e:"function"==typeof t?t:function(){return t}}function l(t){return(s.Platform.IS_WIN?t.winKeys||t.keys:s.Platform.IS_MAC?t.macKeys||t.keys:t.linuxKeys||t.keys).map(c.normalizeKeystroke)}function u(t){if(-1!==t.selector.indexOf(","))throw new Error("Selector cannot contain commas: "+t.selector);if(!s.Selector.isValid(t.selector))throw new Error("Invalid selector: "+t.selector);return t.selector}function g(t,e){if(t.length<e.length)return 0;for(var n=0,r=e.length;n<r;++n)if(t[n]!==e[n])return 0;return t.length>e.length?2:1}function h(t,e){for(var n=e.target,r=e.currentTarget,i=0;null!==n;n=n.parentElement,++i){if(s.Selector.matches(n,t))return i;if(n===r)return-1}return-1}}(r||(r={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)}),o=this&&this.__assign||function(){return(o=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)};Object.defineProperty(e,"__esModule",{value:!0});var a,s=n(5),A=n(570),l=n(9),c=n(46),u=n(290),g=n(25),h=n(10),d=n(27),f=n(59),p=function(t){function e(n){var r=t.call(this,{node:a.createNode()})||this;return r._childIndex=-1,r._activeIndex=-1,r._openTimerID=0,r._closeTimerID=0,r._items=[],r._childMenu=null,r._parentMenu=null,r._aboutToClose=new h.Signal(r),r._menuRequested=new h.Signal(r),r.addClass("p-Menu"),r.setFlag(f.Widget.Flag.DisallowLayout),r.commands=n.commands,r.renderer=n.renderer||e.defaultRenderer,r}return i(e,t),e.prototype.dispose=function(){this.close(),this._items.length=0,t.prototype.dispose.call(this)},Object.defineProperty(e.prototype,"aboutToClose",{get:function(){return this._aboutToClose},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"menuRequested",{get:function(){return this._menuRequested},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"parentMenu",{get:function(){return this._parentMenu},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"childMenu",{get:function(){return this._childMenu},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"rootMenu",{get:function(){for(var t=this;t._parentMenu;)t=t._parentMenu;return t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"leafMenu",{get:function(){for(var t=this;t._childMenu;)t=t._childMenu;return t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"contentNode",{get:function(){return this.node.getElementsByClassName("p-Menu-content")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"activeItem",{get:function(){return this._items[this._activeIndex]||null},set:function(t){this.activeIndex=t?this._items.indexOf(t):-1},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"activeIndex",{get:function(){return this._activeIndex},set:function(t){(t<0||t>=this._items.length)&&(t=-1),-1===t||a.canActivate(this._items[t])||(t=-1),this._activeIndex!==t&&(this._activeIndex=t,this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"items",{get:function(){return this._items},enumerable:!0,configurable:!0}),e.prototype.activateNextItem=function(){var t=this._items.length,e=this._activeIndex,n=e<t-1?e+1:0,r=0===n?t-1:n-1;this.activeIndex=s.ArrayExt.findFirstIndex(this._items,a.canActivate,n,r)},e.prototype.activatePreviousItem=function(){var t=this._items.length,e=this._activeIndex,n=e<=0?t-1:e-1,r=n===t-1?0:n+1;this.activeIndex=s.ArrayExt.findLastIndex(this._items,a.canActivate,n,r)},e.prototype.triggerActiveItem=function(){if(this.isAttached){var t=this.activeItem;if(t)if(this._cancelOpenTimer(),this._cancelCloseTimer(),"submenu"!==t.type){this.rootMenu.close();var e=t.command,n=t.args;this.commands.isEnabled(e,n)?this.commands.execute(e,n):console.log("Command '"+e+"' is disabled.")}else this._openChildMenu(!0)}},e.prototype.addItem=function(t){return this.insertItem(this._items.length,t)},e.prototype.insertItem=function(t,e){this.isAttached&&this.close(),this.activeIndex=-1;var n=Math.max(0,Math.min(t,this._items.length)),r=a.createItem(this,e);return s.ArrayExt.insert(this._items,n,r),this.update(),r},e.prototype.removeItem=function(t){this.removeItemAt(this._items.indexOf(t))},e.prototype.removeItemAt=function(t){this.isAttached&&this.close(),this.activeIndex=-1,s.ArrayExt.removeAt(this._items,t)&&this.update()},e.prototype.clearItems=function(){this.isAttached&&this.close(),this.activeIndex=-1,0!==this._items.length&&(this._items.length=0,this.update())},e.prototype.open=function(t,e,n){if(void 0===n&&(n={}),!this.isAttached){var r=n.forceX||!1,i=n.forceY||!1;a.openRootMenu(this,t,e,r,i),this.activate()}},e.prototype.handleEvent=function(t){switch(t.type){case"keydown":this._evtKeyDown(t);break;case"mouseup":this._evtMouseUp(t);break;case"mousemove":this._evtMouseMove(t);break;case"mouseenter":this._evtMouseEnter(t);break;case"mouseleave":this._evtMouseLeave(t);break;case"mousedown":this._evtMouseDown(t);break;case"contextmenu":t.preventDefault(),t.stopPropagation()}},e.prototype.onBeforeAttach=function(t){this.node.addEventListener("keydown",this),this.node.addEventListener("mouseup",this),this.node.addEventListener("mousemove",this),this.node.addEventListener("mouseenter",this),this.node.addEventListener("mouseleave",this),this.node.addEventListener("contextmenu",this),document.addEventListener("mousedown",this,!0)},e.prototype.onAfterDetach=function(t){this.node.removeEventListener("keydown",this),this.node.removeEventListener("mouseup",this),this.node.removeEventListener("mousemove",this),this.node.removeEventListener("mouseenter",this),this.node.removeEventListener("mouseleave",this),this.node.removeEventListener("contextmenu",this),document.removeEventListener("mousedown",this,!0)},e.prototype.onActivateRequest=function(t){this.isAttached&&this.node.focus()},e.prototype.onUpdateRequest=function(t){for(var e=this._items,n=this.renderer,r=this._activeIndex,i=a.computeCollapsed(e),o=new Array(e.length),s=0,A=e.length;s<A;++s){var l=e[s],c=s===r,u=i[s];o[s]=n.renderItem({item:l,active:c,collapsed:u})}d.VirtualDOM.render(o,this.contentNode)},e.prototype.onCloseRequest=function(e){this._cancelOpenTimer(),this._cancelCloseTimer(),this.activeIndex=-1;var n=this._childMenu;n&&(this._childIndex=-1,this._childMenu=null,n._parentMenu=null,n.close());var r=this._parentMenu;r&&(this._parentMenu=null,r._childIndex=-1,r._childMenu=null,r.activate()),this.isAttached&&this._aboutToClose.emit(void 0),t.prototype.onCloseRequest.call(this,e)},e.prototype._evtKeyDown=function(t){t.preventDefault(),t.stopPropagation();var e=t.keyCode;if(13!==e)if(27!==e)if(37!==e)if(38!==e)if(39!==e)if(40!==e){var n=u.getKeyboardLayout().keyForKeydownEvent(t);if(n){var r=this._activeIndex+1,i=a.findMnemonic(this._items,n,r);-1===i.index||i.multiple?-1!==i.index?this.activeIndex=i.index:-1!==i.auto&&(this.activeIndex=i.auto):(this.activeIndex=i.index,this.triggerActiveItem())}}else this.activateNextItem();else{var o=this.activeItem;o&&"submenu"===o.type?this.triggerActiveItem():this.rootMenu._menuRequested.emit("next")}else this.activatePreviousItem();else this._parentMenu?this.close():this._menuRequested.emit("previous");else this.close();else this.triggerActiveItem()},e.prototype._evtMouseUp=function(t){0===t.button&&(t.preventDefault(),t.stopPropagation(),this.triggerActiveItem())},e.prototype._evtMouseMove=function(t){var e=s.ArrayExt.findFirstIndex(this.contentNode.children,function(e){return c.ElementExt.hitTest(e,t.clientX,t.clientY)});if(e!==this._activeIndex){if(this.activeIndex=e,(e=this.activeIndex)===this._childIndex)return this._cancelOpenTimer(),void this._cancelCloseTimer();-1!==this._childIndex&&this._startCloseTimer(),this._cancelOpenTimer();var n=this.activeItem;n&&"submenu"===n.type&&n.submenu&&this._startOpenTimer()}},e.prototype._evtMouseEnter=function(t){for(var e=this._parentMenu;e;e=e._parentMenu)e._cancelOpenTimer(),e._cancelCloseTimer(),e.activeIndex=e._childIndex},e.prototype._evtMouseLeave=function(t){if(this._cancelOpenTimer(),this._childMenu){var e=t.clientX,n=t.clientY;c.ElementExt.hitTest(this._childMenu.node,e,n)?this._cancelCloseTimer():(this.activeIndex=-1,this._startCloseTimer())}else this.activeIndex=-1},e.prototype._evtMouseDown=function(t){this._parentMenu||(a.hitTestMenus(this,t.clientX,t.clientY)?(t.preventDefault(),t.stopPropagation()):this.close())},e.prototype._openChildMenu=function(t){void 0===t&&(t=!1);var e=this.activeItem;if(e&&"submenu"===e.type&&e.submenu){var n=e.submenu;if(n!==this._childMenu){this._closeChildMenu(),this._childMenu=n,this._childIndex=this._activeIndex,n._parentMenu=this,g.MessageLoop.sendMessage(this,f.Widget.Msg.UpdateRequest);var r=this.contentNode.children[this._activeIndex];a.openSubmenu(n,r),t&&(n.activeIndex=-1,n.activateNextItem()),n.activate()}}else this._closeChildMenu()},e.prototype._closeChildMenu=function(){this._childMenu&&this._childMenu.close()},e.prototype._startOpenTimer=function(){var t=this;0===this._openTimerID&&(this._openTimerID=window.setTimeout(function(){t._openTimerID=0,t._openChildMenu()},a.TIMER_DELAY))},e.prototype._startCloseTimer=function(){var t=this;0===this._closeTimerID&&(this._closeTimerID=window.setTimeout(function(){t._closeTimerID=0,t._closeChildMenu()},a.TIMER_DELAY))},e.prototype._cancelOpenTimer=function(){0!==this._openTimerID&&(clearTimeout(this._openTimerID),this._openTimerID=0)},e.prototype._cancelCloseTimer=function(){0!==this._closeTimerID&&(clearTimeout(this._closeTimerID),this._closeTimerID=0)},e}(f.Widget);e.Menu=p,function(t){var e=function(){function t(){}return t.prototype.renderItem=function(t){var e=this.createItemClass(t),n=this.createItemDataset(t);return d.h.li({className:e,dataset:n},this.renderIcon(t),this.renderLabel(t),this.renderShortcut(t),this.renderSubmenu(t))},t.prototype.renderIcon=function(t){var e=this.createIconClass(t);return d.h.div({className:e},t.item.iconLabel)},t.prototype.renderLabel=function(t){var e=this.formatLabel(t);return d.h.div({className:"p-Menu-itemLabel"},e)},t.prototype.renderShortcut=function(t){var e=this.formatShortcut(t);return d.h.div({className:"p-Menu-itemShortcut"},e)},t.prototype.renderSubmenu=function(t){return d.h.div({className:"p-Menu-itemSubmenuIcon"})},t.prototype.createItemClass=function(t){var e="p-Menu-item";t.item.isEnabled||(e+=" p-mod-disabled"),t.item.isToggled&&(e+=" p-mod-toggled"),t.item.isVisible||(e+=" p-mod-hidden"),t.active&&(e+=" p-mod-active"),t.collapsed&&(e+=" p-mod-collapsed");var n=t.item.className;return n&&(e+=" "+n),e},t.prototype.createItemDataset=function(t){var e=t.item,n=e.type,r=e.command,i=e.dataset;return o({},i,"command"===n?{type:n,command:r}:{type:n})},t.prototype.createIconClass=function(t){var e="p-Menu-itemIcon",n=t.item.iconClass;return n?e+" "+n:e},t.prototype.formatLabel=function(t){var e=t.item,n=e.label,r=e.mnemonic;if(r<0||r>=n.length)return n;var i=n.slice(0,r),o=n.slice(r+1),a=n[r];return[i,d.h.span({className:"p-Menu-itemMnemonic"},a),o]},t.prototype.formatShortcut=function(t){var e=t.item.keyBinding;return e?e.keys.map(A.CommandRegistry.formatKeystroke).join(", "):null},t}();t.Renderer=e,t.defaultRenderer=new e}(p=e.Menu||(e.Menu={})),e.Menu=p,function(t){function e(t){return"separator"!==t.type&&t.isEnabled&&t.isVisible}t.TIMER_DELAY=300,t.SUBMENU_OVERLAP=3,t.createNode=function(){var t=document.createElement("div"),e=document.createElement("ul");return e.className="p-Menu-content",t.appendChild(e),t.tabIndex=-1,t},t.canActivate=e,t.createItem=function(t,e){return new n(t.commands,e)},t.hitTestMenus=function(t,e,n){for(var r=t;r;r=r.childMenu)if(c.ElementExt.hitTest(r.node,e,n))return!0;return!1},t.computeCollapsed=function(t){var e=new Array(t.length);s.ArrayExt.fill(e,!1);for(var n=0,r=t.length;n<r;++n)if((a=t[n]).isVisible){if("separator"!==a.type)break;e[n]=!0}for(var i=r-1;i>=0;--i)if((a=t[i]).isVisible){if("separator"!==a.type)break;e[i]=!0}for(var o=!1;++n<i;){var a;(a=t[n]).isVisible&&("separator"!==a.type?o=!1:o?e[n]=!0:o=!0)}return e},t.openRootMenu=function(t,e,n,r,i){g.MessageLoop.sendMessage(t,f.Widget.Msg.UpdateRequest);var o=window.pageXOffset,a=window.pageYOffset,s=document.documentElement.clientWidth,A=document.documentElement.clientHeight,l=A-(i?n:0),c=t.node,u=c.style;u.top="",u.left="",u.width="",u.height="",u.visibility="hidden",u.maxHeight=l+"px",f.Widget.attach(t,document.body);var h=c.getBoundingClientRect(),d=h.width,p=h.height;!r&&e+d>o+s&&(e=o+s-d),!i&&n+p>a+A&&(n>a+A?n=a+A-p:n-=p),u.top=Math.max(0,n)+"px",u.left=Math.max(0,e)+"px",u.visibility=""},t.openSubmenu=function(e,n){g.MessageLoop.sendMessage(e,f.Widget.Msg.UpdateRequest);var r=window.pageXOffset,i=window.pageYOffset,o=document.documentElement.clientWidth,a=document.documentElement.clientHeight,s=a,A=e.node,l=A.style;l.top="",l.left="",l.width="",l.height="",l.visibility="hidden",l.maxHeight=s+"px",f.Widget.attach(e,document.body);var u=A.getBoundingClientRect(),h=u.width,d=u.height,p=c.ElementExt.boxSizing(e.node),I=n.getBoundingClientRect(),C=I.right-t.SUBMENU_OVERLAP;C+h>r+o&&(C=I.left+t.SUBMENU_OVERLAP-h);var m=I.top-p.borderTop-p.paddingTop;m+d>i+a&&(m=I.bottom+p.borderBottom+p.paddingBottom-d),l.top=Math.max(0,m)+"px",l.left=Math.max(0,C)+"px",l.visibility=""},t.findMnemonic=function(t,n,r){for(var i=-1,o=-1,a=!1,s=n.toUpperCase(),A=0,l=t.length;A<l;++A){var c=(A+r)%l,u=t[c];if(e(u)){var g=u.label;if(0!==g.length){var h=u.mnemonic;h>=0&&h<g.length?g[h].toUpperCase()===s&&(-1===i?i=c:a=!0):-1===o&&g[0].toUpperCase()===s&&(o=c)}}}return{index:i,multiple:a,auto:o}};var n=function(){function t(t,e){this._commands=t,this.type=e.type||"command",this.command=e.command||"",this.args=e.args||l.JSONExt.emptyObject,this.submenu=e.submenu||null}return Object.defineProperty(t.prototype,"label",{get:function(){return"command"===this.type?this._commands.label(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.label:""},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"mnemonic",{get:function(){return"command"===this.type?this._commands.mnemonic(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.mnemonic:-1},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"icon",{get:function(){return this.iconClass},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"iconClass",{get:function(){return"command"===this.type?this._commands.iconClass(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.iconClass:""},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"iconLabel",{get:function(){return"command"===this.type?this._commands.iconLabel(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.iconLabel:""},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"caption",{get:function(){return"command"===this.type?this._commands.caption(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.caption:""},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"className",{get:function(){return"command"===this.type?this._commands.className(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.className:""},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dataset",{get:function(){return"command"===this.type?this._commands.dataset(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.dataset:{}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isEnabled",{get:function(){return"command"===this.type?this._commands.isEnabled(this.command,this.args):"submenu"!==this.type||null!==this.submenu},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isToggled",{get:function(){return"command"===this.type&&this._commands.isToggled(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isVisible",{get:function(){return"command"===this.type?this._commands.isVisible(this.command,this.args):"submenu"!==this.type||null!==this.submenu},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"keyBinding",{get:function(){if("command"===this.type){var t=this.command,e=this.args;return s.ArrayExt.findLastValue(this._commands.keyBindings,function(n){return n.command===t&&l.JSONExt.deepEqual(n.args,e)})||null}return null},enumerable:!0,configurable:!0}),t}()}(a||(a={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(5),s=n(46),A=n(25),l=n(174),c=n(107),u=n(59),g=function(t){function e(e){var n=t.call(this)||this;return n._spacing=4,n._dirty=!1,n._root=null,n._box=null,n._items=new Map,n.renderer=e.renderer,void 0!==e.spacing&&(n._spacing=o.clampSpacing(e.spacing)),n}return i(e,t),e.prototype.dispose=function(){var e=this.iter();this._items.forEach(function(t){t.dispose()}),this._box=null,this._root=null,this._items.clear(),a.each(e,function(t){t.dispose()}),t.prototype.dispose.call(this)},Object.defineProperty(e.prototype,"spacing",{get:function(){return this._spacing},set:function(t){t=o.clampSpacing(t),this._spacing!==t&&(this._spacing=t,this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isEmpty",{get:function(){return null===this._root},enumerable:!0,configurable:!0}),e.prototype.iter=function(){return this._root?this._root.iterAllWidgets():a.empty()},e.prototype.widgets=function(){return this._root?this._root.iterUserWidgets():a.empty()},e.prototype.selectedWidgets=function(){return this._root?this._root.iterSelectedWidgets():a.empty()},e.prototype.tabBars=function(){return this._root?this._root.iterTabBars():a.empty()},e.prototype.handles=function(){return this._root?this._root.iterHandles():a.empty()},e.prototype.moveHandle=function(t,e,n){if(this._root&&!t.classList.contains("p-mod-hidden")){var r,i=this._root.findSplitNode(t);if(i)0!==(r="horizontal"===i.node.orientation?e-t.offsetLeft:n-t.offsetTop)&&(i.node.holdSizes(),l.BoxEngine.adjust(i.node.sizers,i.index,r),this.parent&&this.parent.update())}},e.prototype.saveLayout=function(){return this._root?(this._root.holdAllSizes(),{main:this._root.createConfig()}):{main:null}},e.prototype.restoreLayout=function(t){var e,n=this,r=new Set;e=t.main?o.normalizeAreaConfig(t.main,r):null;var i=this.widgets(),s=this.tabBars(),A=this.handles();this._root=null,a.each(i,function(t){r.has(t)||(t.parent=null)}),a.each(s,function(t){t.dispose()}),a.each(A,function(t){t.parentNode&&t.parentNode.removeChild(t)}),r.forEach(function(t){t.parent=n.parent}),this._root=e?o.realizeAreaConfig(e,{createTabBar:function(){return n._createTabBar()},createHandle:function(){return n._createHandle()}}):null,this.parent&&(r.forEach(function(t){n.attachWidget(t)}),this.parent.fit())},e.prototype.addWidget=function(t,e){void 0===e&&(e={});var n=e.ref||null,r=e.mode||"tab-after",i=null;if(this._root&&n&&(i=this._root.findTabNode(n)),n&&!i)throw new Error("Reference widget is not in the layout.");switch(t.parent=this.parent,r){case"tab-after":this._insertTab(t,n,i,!0);break;case"tab-before":this._insertTab(t,n,i,!1);break;case"split-top":this._insertSplit(t,n,i,"vertical",!1);break;case"split-left":this._insertSplit(t,n,i,"horizontal",!1);break;case"split-right":this._insertSplit(t,n,i,"horizontal",!0);break;case"split-bottom":this._insertSplit(t,n,i,"vertical",!0)}this.parent&&(this.attachWidget(t),this.parent.fit())},e.prototype.removeWidget=function(t){this._removeWidget(t),this.parent&&(this.detachWidget(t),this.parent.fit())},e.prototype.hitTestTabAreas=function(t,e){if(!this._root||!this.parent||!this.parent.isVisible)return null;this._box||(this._box=s.ElementExt.boxSizing(this.parent.node));var n=this.parent.node.getBoundingClientRect(),r=t-n.left-this._box.borderLeft,i=e-n.top-this._box.borderTop,o=this._root.hitTestTabNodes(r,i);if(!o)return null;var a=o.tabBar,A=o.top,l=o.left,c=o.width,u=o.height,g=this._box.borderLeft+this._box.borderRight,h=this._box.borderTop+this._box.borderBottom;return{tabBar:a,x:r,y:i,top:A,left:l,right:n.width-g-(l+c),bottom:n.height-h-(A+u),width:c,height:u}},e.prototype.init=function(){var e=this;t.prototype.init.call(this),a.each(this,function(t){e.attachWidget(t)}),a.each(this.handles(),function(t){e.parent.node.appendChild(t)}),this.parent.fit()},e.prototype.attachWidget=function(t){this.parent.node!==t.node.parentNode&&(this._items.set(t,new c.LayoutItem(t)),this.parent.isAttached&&A.MessageLoop.sendMessage(t,u.Widget.Msg.BeforeAttach),this.parent.node.appendChild(t.node),this.parent.isAttached&&A.MessageLoop.sendMessage(t,u.Widget.Msg.AfterAttach))},e.prototype.detachWidget=function(t){if(this.parent.node===t.node.parentNode){this.parent.isAttached&&A.MessageLoop.sendMessage(t,u.Widget.Msg.BeforeDetach),this.parent.node.removeChild(t.node),this.parent.isAttached&&A.MessageLoop.sendMessage(t,u.Widget.Msg.AfterDetach);var e=this._items.get(t);e&&(this._items.delete(t),e.dispose())}},e.prototype.onBeforeShow=function(e){t.prototype.onBeforeShow.call(this,e),this.parent.update()},e.prototype.onBeforeAttach=function(e){t.prototype.onBeforeAttach.call(this,e),this.parent.fit()},e.prototype.onChildShown=function(t){this.parent.fit()},e.prototype.onChildHidden=function(t){this.parent.fit()},e.prototype.onResize=function(t){this.parent.isVisible&&this._update(t.width,t.height)},e.prototype.onUpdateRequest=function(t){this.parent.isVisible&&this._update(-1,-1)},e.prototype.onFitRequest=function(t){this.parent.isAttached&&this._fit()},e.prototype._removeWidget=function(t){if(this._root){var e=this._root.findTabNode(t);if(e)if(e.tabBar.titles.length>1)e.tabBar.removeTab(t.title);else if(e.tabBar.dispose(),this._root!==e){this._root.holdAllSizes();var n=e.parent;e.parent=null;var r=a.ArrayExt.removeFirstOf(n.children,e),i=a.ArrayExt.removeAt(n.handles,r);if(a.ArrayExt.removeAt(n.sizers,r),i.parentNode&&i.parentNode.removeChild(i),n.children.length>1)n.syncHandles();else{var s=n.parent;n.parent=null;var A=n.children[0],l=n.handles[0];if(n.children.length=0,n.handles.length=0,n.sizers.length=0,l.parentNode&&l.parentNode.removeChild(l),this._root===n)return A.parent=null,void(this._root=A);var c=s,u=c.children.indexOf(n);if(A instanceof o.TabLayoutNode)return A.parent=c,void(c.children[u]=A);var g=a.ArrayExt.removeAt(c.handles,u);a.ArrayExt.removeAt(c.children,u),a.ArrayExt.removeAt(c.sizers,u),g.parentNode&&g.parentNode.removeChild(g);for(var h=0,d=A.children.length;h<d;++h){var f=A.children[h],p=A.handles[h],I=A.sizers[h];a.ArrayExt.insert(c.children,u+h,f),a.ArrayExt.insert(c.handles,u+h,p),a.ArrayExt.insert(c.sizers,u+h,I),f.parent=c}A.children.length=0,A.handles.length=0,A.sizers.length=0,A.parent=null,c.syncHandles()}}else this._root=null}},e.prototype._insertTab=function(t,e,n,r){if(t!==e){if(!this._root){var i=new o.TabLayoutNode(this._createTabBar());return i.tabBar.addTab(t.title),void(this._root=i)}var a;n||(n=this._root.findFirstTabNode()),-1===n.tabBar.titles.indexOf(t.title)&&(this._removeWidget(t),t.hide()),a=e?n.tabBar.titles.indexOf(e.title):n.tabBar.currentIndex,n.tabBar.insertTab(a+(r?1:0),t.title)}},e.prototype._insertSplit=function(t,e,n,r,i){if(t!==e||!n||1!==n.tabBar.titles.length){this._removeWidget(t);var s=new o.TabLayoutNode(this._createTabBar());if(s.tabBar.addTab(t.title),this._root){if(!n||!n.parent){var A=this._splitRoot(r),l=i?A.children.length:0;A.normalizeSizes();var c=o.createSizer(n?1:o.GOLDEN_RATIO);return a.ArrayExt.insert(A.children,l,s),a.ArrayExt.insert(A.sizers,l,c),a.ArrayExt.insert(A.handles,l,this._createHandle()),s.parent=A,A.normalizeSizes(),void A.syncHandles()}var u=n.parent;if(u.orientation===r){var g=u.children.indexOf(n);u.normalizeSizes();var h=u.sizers[g].sizeHint/=2,d=g+(i?1:0);return a.ArrayExt.insert(u.children,d,s),a.ArrayExt.insert(u.sizers,d,o.createSizer(h)),a.ArrayExt.insert(u.handles,d,this._createHandle()),s.parent=u,void u.syncHandles()}var f=a.ArrayExt.removeFirstOf(u.children,n),p=new o.SplitLayoutNode(r);p.normalized=!0,p.children.push(n),p.sizers.push(o.createSizer(.5)),p.handles.push(this._createHandle()),n.parent=p;var I=i?1:0;a.ArrayExt.insert(p.children,I,s),a.ArrayExt.insert(p.sizers,I,o.createSizer(.5)),a.ArrayExt.insert(p.handles,I,this._createHandle()),s.parent=p,p.syncHandles(),a.ArrayExt.insert(u.children,f,p),p.parent=u}else this._root=s}},e.prototype._splitRoot=function(t){var e=this._root;if(e instanceof o.SplitLayoutNode&&e.orientation===t)return e;var n=this._root=new o.SplitLayoutNode(t);return e&&(n.children.push(e),n.sizers.push(o.createSizer(0)),n.handles.push(this._createHandle()),e.parent=n),n},e.prototype._fit=function(){var t=0,e=0;if(this._root){var n=this._root.fit(this._spacing,this._items);t=n.minWidth,e=n.minHeight}var r=this._box=s.ElementExt.boxSizing(this.parent.node);t+=r.horizontalSum,e+=r.verticalSum;var i=this.parent.node.style;i.minWidth=t+"px",i.minHeight=e+"px",this._dirty=!0,this.parent.parent&&A.MessageLoop.sendMessage(this.parent.parent,u.Widget.Msg.FitRequest),this._dirty&&A.MessageLoop.sendMessage(this.parent,u.Widget.Msg.UpdateRequest)},e.prototype._update=function(t,e){if(this._dirty=!1,this._root){t<0&&(t=this.parent.node.offsetWidth),e<0&&(e=this.parent.node.offsetHeight),this._box||(this._box=s.ElementExt.boxSizing(this.parent.node));var n=this._box.paddingTop,r=this._box.paddingLeft,i=t-this._box.horizontalSum,o=e-this._box.verticalSum;this._root.update(n,r,i,o,this._spacing,this._items)}},e.prototype._createTabBar=function(){var t=this.renderer.createTabBar();return t.orientation="horizontal",this.parent&&(t.parent=this.parent,this.attachWidget(t)),t},e.prototype._createHandle=function(){var t=this.renderer.createHandle(),e=t.style;return e.position="absolute",e.top="0",e.left="0",e.width="0",e.height="0",this.parent&&this.parent.node.appendChild(t),t},e}(c.Layout);e.DockLayout=g,function(t){function e(t){var e=new l.BoxSizer;return e.sizeHint=t,e.size=t,e}function n(t,e){return"tab-area"===t.type?function(t,e){if(0===t.widgets.length)return null;var n=[];if(a.each(t.widgets,function(t){e.has(t)||(e.add(t),n.push(t))}),0===n.length)return null;var r=t.currentIndex;-1!==r&&(r<0||r>=n.length)&&(r=0);return{type:"tab-area",widgets:n,currentIndex:r}}(t,e):function(t,e){for(var r=t.orientation,i=[],o=[],a=0,s=t.children.length;a<s;++a){var A=n(t.children[a],e);A&&("tab-area"===A.type||A.orientation!==r?(i.push(A),o.push(Math.abs(t.sizes[a]||0))):(i.push.apply(i,A.children),o.push.apply(o,A.sizes)))}if(0===i.length)return null;if(1===i.length)return i[0];return{type:"split-area",orientation:r,children:i,sizes:o}}(t,e)}function r(t,n){return"tab-area"===t.type?function(t,e){var n=e.createTabBar();return a.each(t.widgets,function(t){t.hide(),n.addTab(t.title)}),n.currentIndex=t.currentIndex,new i(n)}(t,n):function(t,n){var i=new o(t.orientation);return a.each(t.children,function(o,a){var s=r(o,n),A=e(t.sizes[a]),l=n.createHandle();i.children.push(s),i.handles.push(l),i.sizers.push(A),s.parent=i}),i.syncHandles(),i.normalizeSizes(),i}(t,n)}t.GOLDEN_RATIO=.618,t.clampSpacing=function(t){return Math.max(0,Math.floor(t))},t.createSizer=e,t.normalizeAreaConfig=n,t.realizeAreaConfig=r;var i=function(){function t(t){this.parent=null,this._top=0,this._left=0,this._width=0,this._height=0;var e=new l.BoxSizer,n=new l.BoxSizer;e.stretch=0,n.stretch=1,this.tabBar=t,this.sizers=[e,n]}return Object.defineProperty(t.prototype,"top",{get:function(){return this._top},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"left",{get:function(){return this._left},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return this._width},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"height",{get:function(){return this._height},enumerable:!0,configurable:!0}),t.prototype.iterAllWidgets=function(){return a.chain(a.once(this.tabBar),this.iterUserWidgets())},t.prototype.iterUserWidgets=function(){return a.map(this.tabBar.titles,function(t){return t.owner})},t.prototype.iterSelectedWidgets=function(){var t=this.tabBar.currentTitle;return t?a.once(t.owner):a.empty()},t.prototype.iterTabBars=function(){return a.once(this.tabBar)},t.prototype.iterHandles=function(){return a.empty()},t.prototype.findTabNode=function(t){return-1!==this.tabBar.titles.indexOf(t.title)?this:null},t.prototype.findSplitNode=function(t){return null},t.prototype.findFirstTabNode=function(){return this},t.prototype.hitTestTabNodes=function(t,e){return t<this._left||t>=this._left+this._width?null:e<this._top||e>=this._top+this._height?null:this},t.prototype.createConfig=function(){return{type:"tab-area",widgets:this.tabBar.titles.map(function(t){return t.owner}),currentIndex:this.tabBar.currentIndex}},t.prototype.holdAllSizes=function(){},t.prototype.fit=function(t,e){var n=0,r=0,i=e.get(this.tabBar),o=this.tabBar.currentTitle,a=o?e.get(o.owner):void 0,s=this.sizers,A=s[0],l=s[1];return i&&i.fit(),a&&a.fit(),i&&!i.isHidden?(n=Math.max(n,i.minWidth),r+=i.minHeight,A.minSize=i.minHeight,A.maxSize=i.maxHeight):(A.minSize=0,A.maxSize=0),a&&!a.isHidden?(n=Math.max(n,a.minWidth),r+=a.minHeight,l.minSize=a.minHeight,l.maxSize=1/0):(l.minSize=0,l.maxSize=1/0),{minWidth:n,minHeight:r,maxWidth:1/0,maxHeight:1/0}},t.prototype.update=function(t,e,n,r,i,o){this._top=e,this._left=t,this._width=n,this._height=r;var a=o.get(this.tabBar),s=this.tabBar.currentTitle,A=s?o.get(s.owner):void 0;if(l.BoxEngine.calc(this.sizers,r),a&&!a.isHidden){var c=this.sizers[0].size;a.update(t,e,n,c),e+=c}if(A&&!A.isHidden){c=this.sizers[1].size;A.update(t,e,n,c)}},t}();t.TabLayoutNode=i;var o=function(){function t(t){this.parent=null,this.normalized=!1,this.children=[],this.sizers=[],this.handles=[],this.orientation=t}return t.prototype.iterAllWidgets=function(){var t=a.map(this.children,function(t){return t.iterAllWidgets()});return new a.ChainIterator(t)},t.prototype.iterUserWidgets=function(){var t=a.map(this.children,function(t){return t.iterUserWidgets()});return new a.ChainIterator(t)},t.prototype.iterSelectedWidgets=function(){var t=a.map(this.children,function(t){return t.iterSelectedWidgets()});return new a.ChainIterator(t)},t.prototype.iterTabBars=function(){var t=a.map(this.children,function(t){return t.iterTabBars()});return new a.ChainIterator(t)},t.prototype.iterHandles=function(){var t=a.map(this.children,function(t){return t.iterHandles()});return a.chain(this.handles,new a.ChainIterator(t))},t.prototype.findTabNode=function(t){for(var e=0,n=this.children.length;e<n;++e){var r=this.children[e].findTabNode(t);if(r)return r}return null},t.prototype.findSplitNode=function(t){var e=this.handles.indexOf(t);if(-1!==e)return{index:e,node:this};for(var n=0,r=this.children.length;n<r;++n){var i=this.children[n].findSplitNode(t);if(i)return i}return null},t.prototype.findFirstTabNode=function(){return 0===this.children.length?null:this.children[0].findFirstTabNode()},t.prototype.hitTestTabNodes=function(t,e){for(var n=0,r=this.children.length;n<r;++n){var i=this.children[n].hitTestTabNodes(t,e);if(i)return i}return null},t.prototype.createConfig=function(){var t=this.orientation,e=this.createNormalizedSizes();return{type:"split-area",orientation:t,children:this.children.map(function(t){return t.createConfig()}),sizes:e}},t.prototype.syncHandles=function(){var t=this;a.each(this.handles,function(e,n){e.setAttribute("data-orientation",t.orientation),n===t.handles.length-1?e.classList.add("p-mod-hidden"):e.classList.remove("p-mod-hidden")})},t.prototype.holdSizes=function(){a.each(this.sizers,function(t){t.sizeHint=t.size})},t.prototype.holdAllSizes=function(){a.each(this.children,function(t){return t.holdAllSizes()}),this.holdSizes()},t.prototype.normalizeSizes=function(){var t=this.sizers.length;if(0!==t){this.holdSizes();var e=a.reduce(this.sizers,function(t,e){return t+e.sizeHint},0);0===e?a.each(this.sizers,function(e){e.size=e.sizeHint=1/t}):a.each(this.sizers,function(t){t.size=t.sizeHint/=e}),this.normalized=!0}},t.prototype.createNormalizedSizes=function(){var t=this.sizers.length;if(0===t)return[];var e=this.sizers.map(function(t){return t.size}),n=a.reduce(e,function(t,e){return t+e},0);return 0===n?a.each(e,function(n,r){e[r]=1/t}):a.each(e,function(t,r){e[r]=t/n}),e},t.prototype.fit=function(t,e){for(var n="horizontal"===this.orientation,r=Math.max(0,this.children.length-1)*t,i=n?r:0,o=n?0:r,a=0,s=this.children.length;a<s;++a){var A=this.children[a].fit(t,e);n?(o=Math.max(o,A.minHeight),i+=A.minWidth,this.sizers[a].minSize=A.minWidth):(i=Math.max(i,A.minWidth),o+=A.minHeight,this.sizers[a].minSize=A.minHeight)}return{minWidth:i,minHeight:o,maxWidth:1/0,maxHeight:1/0}},t.prototype.update=function(t,e,n,r,i,o){var s="horizontal"===this.orientation,A=Math.max(0,this.children.length-1)*i,c=Math.max(0,(s?n:r)-A);this.normalized&&(a.each(this.sizers,function(t){t.sizeHint*=c}),this.normalized=!1),l.BoxEngine.calc(this.sizers,c);for(var u=0,g=this.children.length;u<g;++u){var h=this.children[u],d=this.sizers[u].size,f=this.handles[u].style;s?(h.update(t,e,d,r,i,o),t+=d,f.top=e+"px",f.left=t+"px",f.width=i+"px",f.height=r+"px",t+=i):(h.update(t,e,n,d,i,o),e+=d,f.top=e+"px",f.left=t+"px",f.width=n+"px",f.height=i+"px",e+=i)}},t}();t.SplitLayoutNode=o}(o||(o={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(5),s=n(46),A=n(25),l=n(43),c=n(174),u=n(107),g=n(175),h=n(59),d=function(t){function e(e){var n=t.call(this)||this;return n._fixed=0,n._spacing=4,n._dirty=!1,n._hasNormedSizes=!1,n._sizers=[],n._items=[],n._handles=[],n._box=null,n._alignment="start",n._orientation="horizontal",n.renderer=e.renderer,void 0!==e.orientation&&(n._orientation=e.orientation),void 0!==e.alignment&&(n._alignment=e.alignment),void 0!==e.spacing&&(n._spacing=o.clampSpacing(e.spacing)),n}return i(e,t),e.prototype.dispose=function(){a.each(this._items,function(t){t.dispose()}),this._box=null,this._items.length=0,this._sizers.length=0,this._handles.length=0,t.prototype.dispose.call(this)},Object.defineProperty(e.prototype,"orientation",{get:function(){return this._orientation},set:function(t){this._orientation!==t&&(this._orientation=t,this.parent&&(this.parent.dataset.orientation=t,this.parent.fit()))},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"alignment",{get:function(){return this._alignment},set:function(t){this._alignment!==t&&(this._alignment=t,this.parent&&(this.parent.dataset.alignment=t,this.parent.update()))},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"spacing",{get:function(){return this._spacing},set:function(t){t=o.clampSpacing(t),this._spacing!==t&&(this._spacing=t,this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"handles",{get:function(){return this._handles},enumerable:!0,configurable:!0}),e.prototype.relativeSizes=function(){return o.normalize(this._sizers.map(function(t){return t.size}))},e.prototype.setRelativeSizes=function(t){for(var e=this._sizers.length,n=t.slice(0,e);n.length<e;)n.push(0);for(var r=o.normalize(n),i=0;i<e;++i){var a=this._sizers[i];a.sizeHint=r[i],a.size=r[i]}this._hasNormedSizes=!0,this.parent&&this.parent.update()},e.prototype.moveHandle=function(t,e){var n,r=this._handles[t];if(r&&!r.classList.contains("p-mod-hidden")&&0!==(n="horizontal"===this._orientation?e-r.offsetLeft:e-r.offsetTop)){for(var i=0,o=this._sizers;i<o.length;i++){var a=o[i];a.size>0&&(a.sizeHint=a.size)}c.BoxEngine.adjust(this._sizers,t,n),this.parent&&this.parent.update()}},e.prototype.init=function(){this.parent.dataset.orientation=this.orientation,this.parent.dataset.alignment=this.alignment,t.prototype.init.call(this)},e.prototype.attachWidget=function(t,e){var n=new u.LayoutItem(e),r=o.createHandle(this.renderer),i=o.averageSize(this._sizers),s=o.createSizer(i);a.ArrayExt.insert(this._items,t,n),a.ArrayExt.insert(this._sizers,t,s),a.ArrayExt.insert(this._handles,t,r),this.parent.isAttached&&A.MessageLoop.sendMessage(e,h.Widget.Msg.BeforeAttach),this.parent.node.appendChild(e.node),this.parent.node.appendChild(r),this.parent.isAttached&&A.MessageLoop.sendMessage(e,h.Widget.Msg.AfterAttach),this.parent.fit()},e.prototype.moveWidget=function(t,e,n){a.ArrayExt.move(this._items,t,e),a.ArrayExt.move(this._sizers,t,e),a.ArrayExt.move(this._handles,t,e),this.parent.fit()},e.prototype.detachWidget=function(t,e){var n=a.ArrayExt.removeAt(this._items,t),r=a.ArrayExt.removeAt(this._handles,t);a.ArrayExt.removeAt(this._sizers,t),this.parent.isAttached&&A.MessageLoop.sendMessage(e,h.Widget.Msg.BeforeDetach),this.parent.node.removeChild(e.node),this.parent.node.removeChild(r),this.parent.isAttached&&A.MessageLoop.sendMessage(e,h.Widget.Msg.AfterDetach),n.dispose(),this.parent.fit()},e.prototype.onBeforeShow=function(e){t.prototype.onBeforeShow.call(this,e),this.parent.update()},e.prototype.onBeforeAttach=function(e){t.prototype.onBeforeAttach.call(this,e),this.parent.fit()},e.prototype.onChildShown=function(t){this.parent.fit()},e.prototype.onChildHidden=function(t){this.parent.fit()},e.prototype.onResize=function(t){this.parent.isVisible&&this._update(t.width,t.height)},e.prototype.onUpdateRequest=function(t){this.parent.isVisible&&this._update(-1,-1)},e.prototype.onFitRequest=function(t){this.parent.isAttached&&this._fit()},e.prototype._fit=function(){for(var t=0,n=-1,r=0,i=this._items.length;r<i;++r)this._items[r].isHidden?this._handles[r].classList.add("p-mod-hidden"):(this._handles[r].classList.remove("p-mod-hidden"),n=r,t++);-1!==n&&this._handles[n].classList.add("p-mod-hidden"),this._fixed=this._spacing*Math.max(0,t-1);var o="horizontal"===this._orientation,a=o?this._fixed:0,l=o?0:this._fixed;for(r=0,i=this._items.length;r<i;++r){var c=this._items[r],u=this._sizers[r];u.size>0&&(u.sizeHint=u.size),c.isHidden?(u.minSize=0,u.maxSize=0):(c.fit(),u.stretch=e.getStretch(c.widget),o?(u.minSize=c.minWidth,u.maxSize=c.maxWidth,a+=c.minWidth,l=Math.max(l,c.minHeight)):(u.minSize=c.minHeight,u.maxSize=c.maxHeight,l+=c.minHeight,a=Math.max(a,c.minWidth)))}var g=this._box=s.ElementExt.boxSizing(this.parent.node);a+=g.horizontalSum,l+=g.verticalSum;var d=this.parent.node.style;d.minWidth=a+"px",d.minHeight=l+"px",this._dirty=!0,this.parent.parent&&A.MessageLoop.sendMessage(this.parent.parent,h.Widget.Msg.FitRequest),this._dirty&&A.MessageLoop.sendMessage(this.parent,h.Widget.Msg.UpdateRequest)},e.prototype._update=function(t,e){this._dirty=!1;for(var n=0,r=0,i=this._items.length;r<i;++r)n+=+!this._items[r].isHidden;if(0!==n){t<0&&(t=this.parent.node.offsetWidth),e<0&&(e=this.parent.node.offsetHeight),this._box||(this._box=s.ElementExt.boxSizing(this.parent.node));var o,a=this._box.paddingTop,A=this._box.paddingLeft,l=t-this._box.horizontalSum,u=e-this._box.verticalSum,g="horizontal"===this._orientation;if(o=g?Math.max(0,l-this._fixed):Math.max(0,u-this._fixed),this._hasNormedSizes){for(var h=0,d=this._sizers;h<d.length;h++){d[h].sizeHint*=o}this._hasNormedSizes=!1}var f=c.BoxEngine.calc(this._sizers,o),p=0,I=0;if(f>0)switch(this._alignment){case"start":break;case"center":p=0,I=f/2;break;case"end":p=0,I=f;break;case"justify":p=f/n,I=0;break;default:throw"unreachable"}for(r=0,i=this._items.length;r<i;++r){var C=this._items[r];if(!C.isHidden){var m=this._sizers[r].size,y=this._handles[r].style;g?(C.update(A+I,a,m+p,u),A+=m+p,y.top=a+"px",y.left=A+I+"px",y.width=this._spacing+"px",y.height=u+"px",A+=this._spacing):(C.update(A,a+I,l,m+p),a+=m+p,y.top=a+I+"px",y.left=A+"px",y.width=l+"px",y.height=this._spacing+"px",a+=this._spacing)}}}},e}(g.PanelLayout);e.SplitLayout=d,function(t){t.getStretch=function(t){return o.stretchProperty.get(t)},t.setStretch=function(t,e){o.stretchProperty.set(t,e)}}(d=e.SplitLayout||(e.SplitLayout={})),e.SplitLayout=d,function(t){t.stretchProperty=new l.AttachedProperty({name:"stretch",create:function(){return 0},coerce:function(t,e){return Math.max(0,Math.floor(e))},changed:function(t){t.parent&&t.parent.layout instanceof d&&t.parent.fit()}}),t.createSizer=function(t){var e=new c.BoxSizer;return e.sizeHint=Math.floor(t),e},t.createHandle=function(t){var e=t.createHandle();return e.style.position="absolute",e},t.clampSpacing=function(t){return Math.max(0,Math.floor(t))},t.averageSize=function(t){return t.reduce(function(t,e){return t+e.size},0)/t.length||0},t.normalize=function(t){var e=t.length;if(0===e)return[];var n=t.reduce(function(t,e){return t+Math.abs(e)},0);return 0===n?t.map(function(t){return 1/e}):t.map(function(t){return t/n})}}(o||(o={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o=n(5),a=n(46),s=n(25),A=n(107),l=n(175),c=n(59),u=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._dirty=!1,e._items=[],e._box=null,e}return i(e,t),e.prototype.dispose=function(){o.each(this._items,function(t){t.dispose()}),this._box=null,this._items.length=0,t.prototype.dispose.call(this)},e.prototype.attachWidget=function(t,e){o.ArrayExt.insert(this._items,t,new A.LayoutItem(e)),this.parent.isAttached&&s.MessageLoop.sendMessage(e,c.Widget.Msg.BeforeAttach),this.parent.node.appendChild(e.node),this.parent.isAttached&&s.MessageLoop.sendMessage(e,c.Widget.Msg.AfterAttach),this.parent.fit()},e.prototype.moveWidget=function(t,e,n){o.ArrayExt.move(this._items,t,e),this.parent.update()},e.prototype.detachWidget=function(t,e){var n=o.ArrayExt.removeAt(this._items,t);this.parent.isAttached&&s.MessageLoop.sendMessage(e,c.Widget.Msg.BeforeDetach),this.parent.node.removeChild(e.node),this.parent.isAttached&&s.MessageLoop.sendMessage(e,c.Widget.Msg.AfterDetach),n.widget.node.style.zIndex="",n.dispose(),this.parent.fit()},e.prototype.onBeforeShow=function(e){t.prototype.onBeforeShow.call(this,e),this.parent.update()},e.prototype.onBeforeAttach=function(e){t.prototype.onBeforeAttach.call(this,e),this.parent.fit()},e.prototype.onChildShown=function(t){this.parent.fit()},e.prototype.onChildHidden=function(t){this.parent.fit()},e.prototype.onResize=function(t){this.parent.isVisible&&this._update(t.width,t.height)},e.prototype.onUpdateRequest=function(t){this.parent.isVisible&&this._update(-1,-1)},e.prototype.onFitRequest=function(t){this.parent.isAttached&&this._fit()},e.prototype._fit=function(){for(var t=0,e=0,n=0,r=this._items.length;n<r;++n){var i=this._items[n];i.isHidden||(i.fit(),t=Math.max(t,i.minWidth),e=Math.max(e,i.minHeight))}var o=this._box=a.ElementExt.boxSizing(this.parent.node);t+=o.horizontalSum,e+=o.verticalSum;var A=this.parent.node.style;A.minWidth=t+"px",A.minHeight=e+"px",this._dirty=!0,this.parent.parent&&s.MessageLoop.sendMessage(this.parent.parent,c.Widget.Msg.FitRequest),this._dirty&&s.MessageLoop.sendMessage(this.parent,c.Widget.Msg.UpdateRequest)},e.prototype._update=function(t,e){this._dirty=!1;for(var n=0,r=0,i=this._items.length;r<i;++r)n+=+!this._items[r].isHidden;if(0!==n){t<0&&(t=this.parent.node.offsetWidth),e<0&&(e=this.parent.node.offsetHeight),this._box||(this._box=a.ElementExt.boxSizing(this.parent.node));var o=this._box.paddingTop,s=this._box.paddingLeft,A=t-this._box.horizontalSum,l=e-this._box.verticalSum;for(r=0,i=this._items.length;r<i;++r){var c=this._items[r];c.isHidden||(c.widget.node.style.zIndex=""+r,c.update(s,o,A,l))}}},e}(l.PanelLayout);e.StackedLayout=u},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(10),s=n(215),A=n(574),l=function(t){function e(e){void 0===e&&(e={});var n=t.call(this,{layout:o.createLayout(e)})||this;return n._widgetRemoved=new a.Signal(n),n.addClass("p-StackedPanel"),n}return i(e,t),Object.defineProperty(e.prototype,"widgetRemoved",{get:function(){return this._widgetRemoved},enumerable:!0,configurable:!0}),e.prototype.onChildAdded=function(t){t.child.addClass("p-StackedPanel-child")},e.prototype.onChildRemoved=function(t){t.child.removeClass("p-StackedPanel-child"),this._widgetRemoved.emit(t.child)},e}(s.Panel);e.StackedPanel=l,function(t){t.createLayout=function(t){return t.layout||new A.StackedLayout}}(o||(o={}))},function(t,e,n){"use strict";var r;n.d(e,"a",function(){return r}),function(t){t.defaultMimeType="text/plain"}(r||(r={}))},function(t,e,n){!function(t){var e=/MSIE \d/.test(navigator.userAgent)&&(null==document.documentMode||document.documentMode<8),n=t.Pos,r={"(":")>",")":"(<","[":"]>","]":"[<","{":"}>","}":"{<","<":">>",">":"<<"};function i(t){return t&&t.bracketRegex||/[(){}[\]]/}function o(t,e,o){var s=t.getLineHandle(e.line),A=e.ch-1,l=o&&o.afterCursor;null==l&&(l=/(^| )cm-fat-cursor($| )/.test(t.getWrapperElement().className));var c=i(o),u=!l&&A>=0&&c.test(s.text.charAt(A))&&r[s.text.charAt(A)]||c.test(s.text.charAt(A+1))&&r[s.text.charAt(++A)];if(!u)return null;var g=">"==u.charAt(1)?1:-1;if(o&&o.strict&&g>0!=(A==e.ch))return null;var h=t.getTokenTypeAt(n(e.line,A+1)),d=a(t,n(e.line,A+(g>0?1:0)),g,h||null,o);return null==d?null:{from:n(e.line,A),to:d&&d.pos,match:d&&d.ch==u.charAt(0),forward:g>0}}function a(t,e,o,a,s){for(var A=s&&s.maxScanLineLength||1e4,l=s&&s.maxScanLines||1e3,c=[],u=i(s),g=o>0?Math.min(e.line+l,t.lastLine()+1):Math.max(t.firstLine()-1,e.line-l),h=e.line;h!=g;h+=o){var d=t.getLine(h);if(d){var f=o>0?0:d.length-1,p=o>0?d.length:-1;if(!(d.length>A))for(h==e.line&&(f=e.ch-(o<0?1:0));f!=p;f+=o){var I=d.charAt(f);if(u.test(I)&&(void 0===a||t.getTokenTypeAt(n(h,f+1))==a)){var C=r[I];if(C&&">"==C.charAt(1)==o>0)c.push(I);else{if(!c.length)return{pos:n(h,f),ch:I};c.pop()}}}}}return h-o!=(o>0?t.lastLine():t.firstLine())&&null}function s(t,r,i){for(var a=t.state.matchBrackets.maxHighlightLineLength||1e3,s=[],A=t.listSelections(),l=0;l<A.length;l++){var c=A[l].empty()&&o(t,A[l].head,i);if(c&&t.getLine(c.from.line).length<=a){var u=c.match?"CodeMirror-matchingbracket":"CodeMirror-nonmatchingbracket";s.push(t.markText(c.from,n(c.from.line,c.from.ch+1),{className:u})),c.to&&t.getLine(c.to.line).length<=a&&s.push(t.markText(c.to,n(c.to.line,c.to.ch+1),{className:u}))}}if(s.length){e&&t.state.focused&&t.focus();var g=function(){t.operation(function(){for(var t=0;t<s.length;t++)s[t].clear()})};if(!r)return g;setTimeout(g,800)}}function A(t){t.operation(function(){t.state.matchBrackets.currentlyHighlighted&&(t.state.matchBrackets.currentlyHighlighted(),t.state.matchBrackets.currentlyHighlighted=null),t.state.matchBrackets.currentlyHighlighted=s(t,!1,t.state.matchBrackets)})}t.defineOption("matchBrackets",!1,function(e,n,r){r&&r!=t.Init&&(e.off("cursorActivity",A),e.state.matchBrackets&&e.state.matchBrackets.currentlyHighlighted&&(e.state.matchBrackets.currentlyHighlighted(),e.state.matchBrackets.currentlyHighlighted=null)),n&&(e.state.matchBrackets="object"==typeof n?n:{},e.on("cursorActivity",A))}),t.defineExtension("matchBrackets",function(){s(this,!0)}),t.defineExtension("findMatchingBracket",function(t,e,n){return(n||"boolean"==typeof e)&&(n?(n.strict=e,e=n):e=e?{strict:!0}:null),o(this,t,e)}),t.defineExtension("scanForBracket",function(t,e,n,r){return a(this,t,e,n,r)})}(n(15))},function(t,e,n){!function(t){"use strict";function e(e,n,i,o){if(i&&i.call){var a=i;i=null}else var a=r(e,i,"rangeFinder");"number"==typeof n&&(n=t.Pos(n,0));var s=r(e,i,"minFoldSize");function A(t){var r=a(e,n);if(!r||r.to.line-r.from.line<s)return null;for(var i=e.findMarksAt(r.from),A=0;A<i.length;++A)if(i[A].__isFold&&"fold"!==o){if(!t)return null;r.cleared=!0,i[A].clear()}return r}var l=A(!0);if(r(e,i,"scanUp"))for(;!l&&n.line>e.firstLine();)n=t.Pos(n.line-1,0),l=A(!1);if(l&&!l.cleared&&"unfold"!==o){var c=function(t,e){var n=r(t,e,"widget");if("string"==typeof n){var i=document.createTextNode(n);(n=document.createElement("span")).appendChild(i),n.className="CodeMirror-foldmarker"}else n&&(n=n.cloneNode(!0));return n}(e,i);t.on(c,"mousedown",function(e){u.clear(),t.e_preventDefault(e)});var u=e.markText(l.from,l.to,{replacedWith:c,clearOnEnter:r(e,i,"clearOnEnter"),__isFold:!0});u.on("clear",function(n,r){t.signal(e,"unfold",e,n,r)}),t.signal(e,"fold",e,l.from,l.to)}}t.newFoldFunction=function(t,n){return function(r,i){e(r,i,{rangeFinder:t,widget:n})}},t.defineExtension("foldCode",function(t,n,r){e(this,t,n,r)}),t.defineExtension("isFolded",function(t){for(var e=this.findMarksAt(t),n=0;n<e.length;++n)if(e[n].__isFold)return!0}),t.commands.toggleFold=function(t){t.foldCode(t.getCursor())},t.commands.fold=function(t){t.foldCode(t.getCursor(),null,"fold")},t.commands.unfold=function(t){t.foldCode(t.getCursor(),null,"unfold")},t.commands.foldAll=function(e){e.operation(function(){for(var n=e.firstLine(),r=e.lastLine();n<=r;n++)e.foldCode(t.Pos(n,0),null,"fold")})},t.commands.unfoldAll=function(e){e.operation(function(){for(var n=e.firstLine(),r=e.lastLine();n<=r;n++)e.foldCode(t.Pos(n,0),null,"unfold")})},t.registerHelper("fold","combine",function(){var t=Array.prototype.slice.call(arguments,0);return function(e,n){for(var r=0;r<t.length;++r){var i=t[r](e,n);if(i)return i}}}),t.registerHelper("fold","auto",function(t,e){for(var n=t.getHelpers(e,"fold"),r=0;r<n.length;r++){var i=n[r](t,e);if(i)return i}});var n={rangeFinder:t.fold.auto,widget:"↔",minFoldSize:0,scanUp:!1,clearOnEnter:!0};function r(t,e,r){if(e&&void 0!==e[r])return e[r];var i=t.options.foldOptions;return i&&void 0!==i[r]?i[r]:n[r]}t.defineOption("foldOptions",null),t.defineExtension("foldOption",function(t,e){return r(this,t,e)})}(n(15))},function(t,e,n){!function(t){function e(e,n,r){var i,o=e.getWrapperElement();return(i=o.appendChild(document.createElement("div"))).className=r?"CodeMirror-dialog CodeMirror-dialog-bottom":"CodeMirror-dialog CodeMirror-dialog-top","string"==typeof n?i.innerHTML=n:i.appendChild(n),t.addClass(o,"dialog-opened"),i}function n(t,e){t.state.currentNotificationClose&&t.state.currentNotificationClose(),t.state.currentNotificationClose=e}t.defineExtension("openDialog",function(r,i,o){o||(o={}),n(this,null);var a=e(this,r,o.bottom),s=!1,A=this;function l(e){if("string"==typeof e)u.value=e;else{if(s)return;s=!0,t.rmClass(a.parentNode,"dialog-opened"),a.parentNode.removeChild(a),A.focus(),o.onClose&&o.onClose(a)}}var c,u=a.getElementsByTagName("input")[0];return u?(u.focus(),o.value&&(u.value=o.value,!1!==o.selectValueOnOpen&&u.select()),o.onInput&&t.on(u,"input",function(t){o.onInput(t,u.value,l)}),o.onKeyUp&&t.on(u,"keyup",function(t){o.onKeyUp(t,u.value,l)}),t.on(u,"keydown",function(e){o&&o.onKeyDown&&o.onKeyDown(e,u.value,l)||((27==e.keyCode||!1!==o.closeOnEnter&&13==e.keyCode)&&(u.blur(),t.e_stop(e),l()),13==e.keyCode&&i(u.value,e))}),!1!==o.closeOnBlur&&t.on(u,"blur",l)):(c=a.getElementsByTagName("button")[0])&&(t.on(c,"click",function(){l(),A.focus()}),!1!==o.closeOnBlur&&t.on(c,"blur",l),c.focus()),l}),t.defineExtension("openConfirm",function(r,i,o){n(this,null);var a=e(this,r,o&&o.bottom),s=a.getElementsByTagName("button"),A=!1,l=this,c=1;function u(){A||(A=!0,t.rmClass(a.parentNode,"dialog-opened"),a.parentNode.removeChild(a),l.focus())}s[0].focus();for(var g=0;g<s.length;++g){var h=s[g];!function(e){t.on(h,"click",function(n){t.e_preventDefault(n),u(),e&&e(l)})}(i[g]),t.on(h,"blur",function(){--c,setTimeout(function(){c<=0&&u()},200)}),t.on(h,"focus",function(){++c})}}),t.defineExtension("openNotification",function(r,i){n(this,l);var o,a=e(this,r,i&&i.bottom),s=!1,A=i&&void 0!==i.duration?i.duration:5e3;function l(){s||(s=!0,clearTimeout(o),t.rmClass(a.parentNode,"dialog-opened"),a.parentNode.removeChild(a))}return t.on(a,"click",function(e){t.e_preventDefault(e),l()}),A&&(o=setTimeout(l,A)),l})}(n(15))},function(t,e,n){"use strict";(function(t){var n,r=this&&this.__extends||(n=function(t,e){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function r(){this.constructor=t}n(t,e),t.prototype=null===e?Object.create(e):(r.prototype=e.prototype,new r)});Object.defineProperty(e,"__esModule",{value:!0});var i=0;e.IS_UNIQUE="__DO_NOT_DEDUPE_STYLE__";for(var o=/[A-Z]/g,a=/^ms-/,s=/&/g,A=/[ !#$%&()*+,.\/;<=>?@[\]^`{|}~"'\\]/g,l=function(t){return"-"+t.toLowerCase()},c={"animation-iteration-count":!0,"border-image-outset":!0,"border-image-slice":!0,"border-image-width":!0,"box-flex":!0,"box-flex-group":!0,"box-ordinal-group":!0,"column-count":!0,columns:!0,"counter-increment":!0,"counter-reset":!0,flex:!0,"flex-grow":!0,"flex-positive":!0,"flex-shrink":!0,"flex-negative":!0,"flex-order":!0,"font-weight":!0,"grid-area":!0,"grid-column":!0,"grid-column-end":!0,"grid-column-span":!0,"grid-column-start":!0,"grid-row":!0,"grid-row-end":!0,"grid-row-span":!0,"grid-row-start":!0,"line-clamp":!0,"line-height":!0,opacity:!0,order:!0,orphans:!0,"tab-size":!0,widows:!0,"z-index":!0,zoom:!0,"fill-opacity":!0,"flood-opacity":!0,"stop-opacity":!0,"stroke-dasharray":!0,"stroke-dashoffset":!0,"stroke-miterlimit":!0,"stroke-opacity":!0,"stroke-width":!0},u=0,g=Object.keys(c);u<g.length;u++)for(var h=g[u],d=0,f=["-webkit-","-ms-","-moz-","-o-",""];d<f.length;d++){c[f[d]+h]=!0}function p(t){return t.replace(o,l).replace(a,"-ms-")}function I(t){for(var e=5381,n=t.length;n--;)e=33*e^t.charCodeAt(n);return(e>>>0).toString(36)}function C(t,e){return"number"!=typeof e||0===e||c.hasOwnProperty(t)?t+":"+e:t+":"+e+"px"}function m(t){return t.sort(function(t,e){return t[0]>e[0]?1:-1})}function y(t){return t.map(function(t){var e=t[0],n=t[1];return Array.isArray(n)?n.map(function(t){return C(e,t)}).join(";"):C(e,n)}).join(";")}function E(t,e){return-1===t.indexOf("&")?e+" "+t:t.replace(s,e)}function v(t,n,r,i,o){var a=function(t,n){for(var r=[],i=[],o=!1,a=0,s=Object.keys(t);a<s.length;a++){var A=s[a],l=t[A];null!=l&&(A===e.IS_UNIQUE?o=!0:"object"!=typeof l||Array.isArray(l)?r.push([p(A.trim()),l]):i.push([A.trim(),l]))}return{style:y(m(r)),nested:n?i:m(i),isUnique:o}}(n,""!==t),s=a.style,A=a.nested,l=a.isUnique,c=s;if(64===t.charCodeAt(0)){var u={selector:t,styles:[],rules:[],style:o?"":s};r.push(u),s&&o&&u.styles.push({selector:o,style:s,isUnique:l});for(var g=0,h=A;g<h.length;g++){var d=h[g];c+=(B=d[0])+v(B,d[1],u.rules,u.styles,o)}}else{var f=o?E(t,o):t;s&&i.push({selector:f,style:s,isUnique:l});for(var I=0,C=A;I<C.length;I++){var B,b=C[I];c+=(B=b[0])+v(B,b[1],r,i,f)}}return c}function B(t,e,n,r,o,a){for(var s=0,A=r;s<A.length;s++){var l=A[s],c=l.selector,u=l.style,g=l.isUnique,h=a?E(c,o):c,d=g?"u\0"+(++i).toString(36):"s\0"+e+"\0"+u;(I=new _(u,d)).add(new x(h,"k\0"+e+"\0"+h)),t.add(I)}for(var f=0,p=n;f<p.length;f++){var I,C=p[f],m=(c=C.selector,u=C.style,C.rules),y=C.styles;B(I=new D(c,u,"r\0"+e+"\0"+c+"\0"+u),e,m,y,o,a),t.add(I)}}function b(t){for(var e="",n=0;n<t.length;n++)e+=t[n];return e}e.escape=function(t){return t.replace(A,"\\$&")},e.hyphenate=p,e.stringHash=I;var Q={add:function(){},change:function(){},remove:function(){}},w=function(){function t(t){void 0===t&&(t=Q),this.changes=t,this.sheet=[],this.changeId=0,this._keys=[],this._children=Object.create(null),this._counters=Object.create(null)}return t.prototype.add=function(e){var n=this._counters[e.id]||0,r=this._children[e.id]||e.clone();if(this._counters[e.id]=n+1,0===n)this._children[r.id]=r,this._keys.push(r.id),this.sheet.push(r.getStyles()),this.changeId++,this.changes.add(r,this._keys.length-1);else if(r instanceof t&&e instanceof t){var i=this._keys.indexOf(e.id),o=r.changeId;r.merge(e),r.changeId!==o&&(this.sheet.splice(i,1,r.getStyles()),this.changeId++,this.changes.change(r,i,i))}return r},t.prototype.remove=function(e){var n=this._counters[e.id];if(void 0!==n&&n>0){this._counters[e.id]=n-1;var r=this._children[e.id],i=this._keys.indexOf(r.id);if(1===n)delete this._counters[e.id],delete this._children[e.id],this._keys.splice(i,1),this.sheet.splice(i,1),this.changeId++,this.changes.remove(r,i);else if(r instanceof t&&e instanceof t){var o=r.changeId;r.unmerge(e),r.changeId!==o&&(this.sheet.splice(i,1,r.getStyles()),this.changeId++,this.changes.change(r,i,i))}}},t.prototype.values=function(){var t=this;return this._keys.map(function(e){return t._children[e]})},t.prototype.merge=function(t){for(var e=0,n=t.values();e<n.length;e++){var r=n[e];this.add(r)}return this},t.prototype.unmerge=function(t){for(var e=0,n=t.values();e<n.length;e++){var r=n[e];this.remove(r)}return this},t.prototype.clone=function(){return(new t).merge(this)},t}();e.Cache=w;var x=function(){function t(t,e){this.selector=t,this.id=e}return t.prototype.getStyles=function(){return this.selector},t.prototype.clone=function(){return new t(this.selector,this.id)},t}();e.Selector=x;var _=function(t){function e(e,n){var r=t.call(this)||this;return r.style=e,r.id=n,r}return r(e,t),e.prototype.getStyles=function(){return this.sheet.join(",")+"{"+this.style+"}"},e.prototype.clone=function(){return new e(this.style,this.id).merge(this)},e}(w);e.Style=_;var D=function(t){function e(e,n,r){var i=t.call(this)||this;return i.rule=e,i.style=n,i.id=r,i}return r(e,t),e.prototype.getStyles=function(){return this.rule+"{"+this.style+b(this.sheet)+"}"},e.prototype.clone=function(){return new e(this.rule,this.style,this.id).merge(this)},e}(w);e.Rule=D;var M=function(t){function n(e,n,r,i){var o=t.call(this,i)||this;return o.hash=e,o.debug=n,o.id=r,o}return r(n,t),n.prototype.registerStyle=function(t,n){var r=[],i=[],o=v("&",t,r,i),a="f"+this.hash(o),s=this.debug&&n?n+"_"+a:a;return B(this,o,r,i,"."+e.escape(s),!0),s},n.prototype.registerKeyframes=function(t,e){return this.registerHashRule("@keyframes",t,e)},n.prototype.registerHashRule=function(t,n,r){var i=[],o=[],a=v("",n,i,o),s="f"+this.hash(a),A=this.debug&&r?r+"_"+s:s,l=new D(t+" "+e.escape(A),"","h\0"+a+"\0"+t);return B(l,a,i,o,"",!1),this.add(l),A},n.prototype.registerRule=function(t,e){var n=[],r=[];B(this,v(t,e,n,r),n,r,"",!1)},n.prototype.registerCss=function(t){return this.registerRule("",t)},n.prototype.getStyles=function(){return b(this.sheet)},n.prototype.clone=function(){return new n(this.hash,this.debug,this.id,this.changes).merge(this)},n}(w);e.FreeStyle=M,e.create=function(e,n,r){return void 0===e&&(e=I),void 0===n&&(n=void 0!==t&&!1),void 0===r&&(r=Q),new M(e,n,"f"+(++i).toString(36),r)}}).call(this,n(58))},function(t,e,n){"use strict";function r(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];for(var n={},i=0,o=t;i<o.length;i++){var a=o[i];if(null!=a&&!1!==a)for(var s in a){var A=a[s];(A||0===A)&&("$nest"===s&&A?n[s]=n.$nest?r(n.$nest,A):A:-1!==s.indexOf("&")||0===s.indexOf("@media")?n[s]=n[s]?r(n[s],A):A:n[s]=A)}}return n}Object.defineProperty(e,"__esModule",{value:!0}),e.raf="undefined"==typeof requestAnimationFrame?function(t){return setTimeout(t)}:"undefined"==typeof window?requestAnimationFrame:requestAnimationFrame.bind(window),e.classes=function(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return t.map(function(t){return t&&"object"==typeof t?Object.keys(t).map(function(e){return!!t[e]&&e}):[t]}).reduce(function(t,e){return t.concat(e)},[]).filter(function(t){return!!t}).join(" ")},e.extend=r,e.media=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];var o=[];t.type&&o.push(t.type),t.orientation&&o.push("(orientation: "+t.orientation+")"),t.minWidth&&o.push("(min-width: "+i(t.minWidth)+")"),t.maxWidth&&o.push("(max-width: "+i(t.maxWidth)+")"),t.minHeight&&o.push("(min-height: "+i(t.minHeight)+")"),t.maxHeight&&o.push("(max-height: "+i(t.maxHeight)+")");var a,s="@media "+o.join(" and "),A={$nest:(a={},a[s]=r.apply(void 0,e),a)};return A};var i=function(t){return"string"==typeof t?t:t+"px"}},function(t,e,n){"use strict";n.d(e,"a",function(){return o});const r="jp-HoverBox",i="jp-mod-outofview";var o;!function(t){t.setGeometry=function(t){const{anchor:e,host:n,node:o,privilege:a}=t;if(o.classList.add(r),o.classList.add(i),!n.contains(document.elementFromPoint(e.left,e.top)))return;o.style.maxHeight="",o.style.marginTop="",o.classList.remove(i);const s=t.style||window.getComputedStyle(o),A=window.innerHeight,l=e.top,c=A-e.bottom,u=parseInt(s.marginTop,10)||0,g=parseInt(s.minHeight,10)||t.minHeight;let h=parseInt(s.maxHeight,10)||t.maxHeight;const d="forceAbove"!==a&&("forceBelow"===a||("above"===a?l<h&&l<c:c>=h||c>=l));if(d?h=Math.min(c-u,h):(h=Math.min(l,h),o.style.marginTop="0px"),o.style.maxHeight=`${h}px`,!(h>g&&(c>=g||l>=g)))return void o.classList.add(i);const f=t.offset&&t.offset.vertical&&t.offset.vertical.above||0,p=t.offset&&t.offset.vertical&&t.offset.vertical.below||0,I=d?A-c+p:l-o.getBoundingClientRect().height+f;o.style.top=`${Math.floor(I)}px`;const C=t.offset&&t.offset.horizontal||0;let m=e.left+C;o.style.left=`${Math.ceil(m)}px`,o.style.width="auto",o.scrollHeight>=h&&(o.style.width=`${2*o.offsetWidth-o.clientWidth}`,o.scrollTop=0);let y=o.getBoundingClientRect().right;y>window.innerWidth&&(m-=y-window.innerWidth,o.style.left=`${Math.ceil(m)}px`)}}(o||(o={}))},function(t,e,n){"use strict";var r=this&&this.__importStar||function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)Object.hasOwnProperty.call(t,n)&&(e[n]=t[n]);return e.default=t,e};Object.defineProperty(e,"__esModule",{value:!0});const i=n(17),o=n(5),a=n(10),s=n(60),A=r(n(1070)),l="api/contents",c="files";var u;!function(t){t.validateContentsModel=function(t){A.validateContentsModel(t)},t.validateCheckpointModel=function(t){A.validateCheckpointModel(t)}}(e.Contents||(e.Contents={}));e.ContentsManager=class{constructor(t={}){this._isDisposed=!1,this._additionalDrives=new Map,this._fileChanged=new a.Signal(this);let e=this.serverSettings=t.serverSettings||s.ServerConnection.makeSettings();this._defaultDrive=t.defaultDrive||new g({serverSettings:e}),this._defaultDrive.fileChanged.connect(this._onFileChanged,this)}get fileChanged(){return this._fileChanged}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,a.Signal.clearData(this))}addDrive(t){this._additionalDrives.set(t.name,t),t.fileChanged.connect(this._onFileChanged,this)}getModelDBFactory(t){let[e]=this._driveForPath(t);return e&&e.modelDBFactory||null}localPath(t){const e=t.split("/"),n=e[0].split(":");return 1!==n.length&&this._additionalDrives.has(n[0])?i.PathExt.join(n.slice(1).join(":"),...e.slice(1)):i.PathExt.removeSlash(t)}normalize(t){const e=t.split(":");return 1===e.length?i.PathExt.normalize(t):`${e[0]}:${i.PathExt.normalize(e.slice(1).join(":"))}`}driveName(t){const e=t.split("/")[0].split(":");return 1===e.length?"":this._additionalDrives.has(e[0])?e[0]:""}get(t,e){let[n,r]=this._driveForPath(t);return n.get(r,e).then(t=>{let e=[];return"directory"===t.type&&t.content?(o.each(t.content,t=>{e.push(Object.assign({},t,{path:this._toGlobalPath(n,t.path)}))}),Object.assign({},t,{path:this._toGlobalPath(n,r),content:e})):Object.assign({},t,{path:this._toGlobalPath(n,r)})})}getDownloadUrl(t){let[e,n]=this._driveForPath(t);return e.getDownloadUrl(n)}newUntitled(t={}){if(t.path){let e=this.normalize(t.path),[n,r]=this._driveForPath(e);return n.newUntitled(Object.assign({},t,{path:r})).then(t=>Object.assign({},t,{path:i.PathExt.join(e,t.name)}))}return this._defaultDrive.newUntitled(t)}delete(t){let[e,n]=this._driveForPath(t);return e.delete(n)}rename(t,e){let[n,r]=this._driveForPath(t),[i,o]=this._driveForPath(e);if(n!==i)throw Error("ContentsManager: renaming files must occur within a Drive");return n.rename(r,o).then(t=>Object.assign({},t,{path:this._toGlobalPath(n,o)}))}save(t,e={}){const n=this.normalize(t),[r,i]=this._driveForPath(t);return r.save(i,Object.assign({},e,{path:i})).then(t=>Object.assign({},t,{path:n}))}copy(t,e){let[n,r]=this._driveForPath(t),[i,o]=this._driveForPath(e);if(n===i)return n.copy(r,o).then(t=>Object.assign({},t,{path:this._toGlobalPath(n,t.path)}));throw Error("Copying files between drives is not currently implemented")}createCheckpoint(t){let[e,n]=this._driveForPath(t);return e.createCheckpoint(n)}listCheckpoints(t){let[e,n]=this._driveForPath(t);return e.listCheckpoints(n)}restoreCheckpoint(t,e){let[n,r]=this._driveForPath(t);return n.restoreCheckpoint(r,e)}deleteCheckpoint(t,e){let[n,r]=this._driveForPath(t);return n.deleteCheckpoint(r,e)}_toGlobalPath(t,e){return t===this._defaultDrive?i.PathExt.removeSlash(e):`${t.name}:${i.PathExt.removeSlash(e)}`}_driveForPath(t){const e=this.driveName(t),n=this.localPath(t);return e?[this._additionalDrives.get(e),n]:[this._defaultDrive,n]}_onFileChanged(t,e){if(t===this._defaultDrive)this._fileChanged.emit(e);else{let n=null,r=null;e.newValue&&e.newValue.path&&(n=Object.assign({},e.newValue,{path:this._toGlobalPath(t,e.newValue.path)})),e.oldValue&&e.oldValue.path&&(r=Object.assign({},e.oldValue,{path:this._toGlobalPath(t,e.oldValue.path)})),this._fileChanged.emit({type:e.type,newValue:n,oldValue:r})}}};class g{constructor(t={}){this._isDisposed=!1,this._fileChanged=new a.Signal(this),this.name=t.name||"Default",this._apiEndpoint=t.apiEndpoint||l,this.serverSettings=t.serverSettings||s.ServerConnection.makeSettings()}get fileChanged(){return this._fileChanged}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,a.Signal.clearData(this))}get(t,e){let n=this._getUrl(t);if(e){"notebook"===e.type&&delete e.format;let t=e.content?"1":"0",r=Object.assign({},e,{content:t});n+=i.URLExt.objectToQueryString(r)}let r=this.serverSettings;return s.ServerConnection.makeRequest(n,{},r).then(t=>{if(200!==t.status)throw new s.ServerConnection.ResponseError(t);return t.json()}).then(t=>(A.validateContentsModel(t),t))}getDownloadUrl(t){let e=this.serverSettings.baseUrl,n=i.URLExt.join(e,c,i.URLExt.encodeParts(t));const r=document.cookie.match("\\b_xsrf=([^;]*)\\b");if(r){const t=new URL(n);t.searchParams.append("_xsrf",r[1]),n=t.toString()}return Promise.resolve(n)}newUntitled(t={}){let e="{}";t&&(t.ext&&(t.ext=u.normalizeExtension(t.ext)),e=JSON.stringify(t));let n=this.serverSettings,r=this._getUrl(t.path||""),i={method:"POST",body:e};return s.ServerConnection.makeRequest(r,i,n).then(t=>{if(201!==t.status)throw new s.ServerConnection.ResponseError(t);return t.json()}).then(t=>(A.validateContentsModel(t),this._fileChanged.emit({type:"new",oldValue:null,newValue:t}),t))}delete(t){let e=this._getUrl(t),n=this.serverSettings;return s.ServerConnection.makeRequest(e,{method:"DELETE"},n).then(e=>{if(400===e.status)return e.json().then(t=>{throw new s.ServerConnection.ResponseError(e,t.message)});if(204!==e.status)throw new s.ServerConnection.ResponseError(e);this._fileChanged.emit({type:"delete",oldValue:{path:t},newValue:null})})}rename(t,e){let n=this.serverSettings,r=this._getUrl(t),i={method:"PATCH",body:JSON.stringify({path:e})};return s.ServerConnection.makeRequest(r,i,n).then(t=>{if(200!==t.status)throw new s.ServerConnection.ResponseError(t);return t.json()}).then(e=>(A.validateContentsModel(e),this._fileChanged.emit({type:"rename",oldValue:{path:t},newValue:e}),e))}save(t,e={}){let n=this.serverSettings,r=this._getUrl(t),i={method:"PUT",body:JSON.stringify(e)};return s.ServerConnection.makeRequest(r,i,n).then(t=>{if(200!==t.status&&201!==t.status)throw new s.ServerConnection.ResponseError(t);return t.json()}).then(t=>(A.validateContentsModel(t),this._fileChanged.emit({type:"save",oldValue:null,newValue:t}),t))}copy(t,e){let n=this.serverSettings,r=this._getUrl(e),i={method:"POST",body:JSON.stringify({copy_from:t})};return s.ServerConnection.makeRequest(r,i,n).then(t=>{if(201!==t.status)throw new s.ServerConnection.ResponseError(t);return t.json()}).then(t=>(A.validateContentsModel(t),this._fileChanged.emit({type:"new",oldValue:null,newValue:t}),t))}createCheckpoint(t){let e=this._getUrl(t,"checkpoints");return s.ServerConnection.makeRequest(e,{method:"POST"},this.serverSettings).then(t=>{if(201!==t.status)throw new s.ServerConnection.ResponseError(t);return t.json()}).then(t=>(A.validateCheckpointModel(t),t))}listCheckpoints(t){let e=this._getUrl(t,"checkpoints");return s.ServerConnection.makeRequest(e,{},this.serverSettings).then(t=>{if(200!==t.status)throw new s.ServerConnection.ResponseError(t);return t.json()}).then(t=>{if(!Array.isArray(t))throw new Error("Invalid Checkpoint list");for(let e=0;e<t.length;e++)A.validateCheckpointModel(t[e]);return t})}restoreCheckpoint(t,e){let n=this._getUrl(t,"checkpoints",e);return s.ServerConnection.makeRequest(n,{method:"POST"},this.serverSettings).then(t=>{if(204!==t.status)throw new s.ServerConnection.ResponseError(t)})}deleteCheckpoint(t,e){let n=this._getUrl(t,"checkpoints",e);return s.ServerConnection.makeRequest(n,{method:"DELETE"},this.serverSettings).then(t=>{if(204!==t.status)throw new s.ServerConnection.ResponseError(t)})}_getUrl(...t){let e=t.map(t=>i.URLExt.encodeParts(t)),n=this.serverSettings.baseUrl;return i.URLExt.join(n,this._apiEndpoint,...e)}}e.Drive=g,function(t){t.normalizeExtension=function(t){return t.length>0&&0!==t.indexOf(".")&&(t=`.${t}`),t}}(u||(u={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(1071);!function(t){t.findById=function(t,e){return r.DefaultKernel.findById(t,e)},t.getSpecs=function(t){return r.DefaultKernel.getSpecs(t)},t.listRunning=function(t){return r.DefaultKernel.listRunning(t)},t.startNew=function(t={}){return r.DefaultKernel.startNew(t)},t.connectTo=function(t,e){return r.DefaultKernel.connectTo(t,e)},t.shutdown=function(t,e){return r.DefaultKernel.shutdown(t,e)},t.shutdownAll=function(t){return r.DefaultKernel.shutdownAll(t)}}(e.Kernel||(e.Kernel={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=["username","version","session","msg_id","msg_type"],i={stream:{name:"string",text:"string"},display_data:{data:"object",metadata:"object"},execute_input:{code:"string",execution_count:"number"},execute_result:{execution_count:"number",data:"object",metadata:"object"},error:{ename:"string",evalue:"string",traceback:"object"},status:{execution_state:"string"},clear_output:{wait:"boolean"},comm_open:{comm_id:"string",target_name:"string",data:"object"},comm_msg:{comm_id:"string",data:"object"},comm_close:{comm_id:"string"},shutdown_reply:{restart:"boolean"}};function o(t,e,n){if(!t.hasOwnProperty(e))throw Error(`Missing property '${e}'`);if(void 0!==n){let r=!0,i=t[e];switch(n){case"array":r=Array.isArray(i);break;case"object":r=void 0!==i;break;default:r=typeof i===n}if(!r)throw new Error(`Property '${e}' is not of type '${n}`)}}function a(t){let e=t.spec;if(!e)throw new Error("Invalid kernel spec");return o(t,"name","string"),o(t,"resources","object"),o(e,"language","string"),o(e,"display_name","string"),o(e,"argv","array"),{name:t.name,resources:t.resources,language:e.language,display_name:e.display_name,argv:e.argv}}e.validateMessage=function(t){o(t,"metadata","object"),o(t,"content","object"),o(t,"channel","string"),function(t){for(let e=0;e<r.length;e++)o(t,r[e],"string")}(t.header),"iopub"===t.channel&&function(t){if("iopub"===t.channel){let e=i[t.header.msg_type];if(void 0===e)return;let n=Object.keys(e),r=t.content;for(let t=0;t<n.length;t++)o(r,n[t],e[n[t]])}}(t)},e.validateModel=function(t){o(t,"name","string"),o(t,"id","string")},e.validateSpecModel=a,e.validateSpecModels=function(t){if(!t.hasOwnProperty("kernelspecs"))throw new Error("No kernelspecs found");let e=Object.keys(t.kernelspecs),n=Object.create(null),r=t.default;for(let r=0;r<e.length;r++){let i=t.kernelspecs[e[r]];try{n[e[r]]=a(i)}catch(t){console.warn(`Removing errant kernel spec: ${e[r]}`)}}if(!(e=Object.keys(n)).length)throw new Error("No valid kernelspecs found");return r&&"string"==typeof r&&r in n||(r=e[0],console.warn(`Default kernel not found, using '${e[0]}'`)),{default:r,kernelspecs:n}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(17),i=n(120),o="api/nbconvert";e.NbConvertManager=class{constructor(t={}){this.serverSettings=t.serverSettings||i.ServerConnection.makeSettings()}getExportFormats(){const t=this.serverSettings.baseUrl,e=r.URLExt.join(t,o),{serverSettings:n}=this;return i.ServerConnection.makeRequest(e,{},n).then(t=>{if(200!==t.status)throw new i.ServerConnection.ResponseError(t);return t.json()}).then(t=>{let e={};return Object.keys(t).forEach(function(n){let r=t[n].output_mimetype;e[n]={output_mimetype:r}}),e})}}},function(t,e,n){"use strict";function r(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}Object.defineProperty(e,"__esModule",{value:!0}),r(n(1080)),r(n(588))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(1081);!function(t){t.listRunning=function(t){return r.DefaultSession.listRunning(t)},t.startNew=function(t){return r.DefaultSession.startNew(t)},t.findById=function(t,e){return r.DefaultSession.findById(t,e)},t.findByPath=function(t,e){return r.DefaultSession.findByPath(t,e)},t.connectTo=function(t,e){return r.DefaultSession.connectTo(t,e)},t.shutdown=function(t,e){return r.DefaultSession.shutdown(t,e)},t.shutdownAll=function(t){return r.DefaultSession.shutdownAll(t)}}(e.Session||(e.Session={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(17),i=n(120),o="api/settings";var a;e.SettingManager=class extends r.DataConnector{constructor(t={}){super(),this.serverSettings=t.serverSettings||i.ServerConnection.makeSettings()}async fetch(t){const{serverSettings:e}=this,{baseUrl:n,appUrl:r}=e,{makeRequest:o,ResponseError:s}=i.ServerConnection,A=n+r,l=a.url(A,t),c=await o(l,{},e);if(!t)throw new Error("Plugin `id` parameter is required for settings fetch.");if(200!==c.status)throw new s(c);return c.json()}async list(){const{serverSettings:t}=this,{baseUrl:e,appUrl:n}=t,{makeRequest:r,ResponseError:o}=i.ServerConnection,s=e+n,A=a.url(s,""),l=await r(A,{},t);if(200!==l.status)throw new o(l);const c=((await l.json()||{}).settings||[]).map(t=>(t.data={composite:{},user:{}},t));return{ids:c.map(t=>t.id),values:c}}async save(t,e){const{serverSettings:n}=this,{baseUrl:r,appUrl:o}=n,{makeRequest:s,ResponseError:A}=i.ServerConnection,l=r+o,c=a.url(l,t),u={body:e,method:"PUT"},g=await s(c,u,n);if(204!==g.status)throw new A(g)}},function(t){t.url=function(t,e){return r.URLExt.join(t,o,e)}}(a||(a={}))},function(t,e,n){"use strict";function r(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}Object.defineProperty(e,"__esModule",{value:!0}),r(n(1083)),r(n(296))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(17),i=n(120),o="api/workspaces";var a;e.WorkspaceManager=class extends r.DataConnector{constructor(t={}){super(),this.serverSettings=t.serverSettings||i.ServerConnection.makeSettings()}async fetch(t){const{serverSettings:e}=this,{baseUrl:n,appUrl:r}=e,{makeRequest:o,ResponseError:s}=i.ServerConnection,A=n+r,l=a.url(A,t),c=await o(l,{},e);if(200!==c.status)throw new s(c);return c.json()}async list(){const{serverSettings:t}=this,{baseUrl:e,appUrl:n}=t,{makeRequest:r,ResponseError:o}=i.ServerConnection,s=e+n,A=a.url(s,""),l=await r(A,{},t);if(200!==l.status)throw new o(l);return(await l.json()).workspaces}async remove(t){const{serverSettings:e}=this,{baseUrl:n,appUrl:r}=e,{makeRequest:o,ResponseError:s}=i.ServerConnection,A=n+r,l=a.url(A,t),c=await o(l,{method:"DELETE"},e);if(204!==c.status)throw new s(c)}async save(t,e){const{serverSettings:n}=this,{baseUrl:r,appUrl:o}=n,{makeRequest:s,ResponseError:A}=i.ServerConnection,l=r+o,c=a.url(l,t),u={body:JSON.stringify(e),method:"PUT"},g=await s(c,u,n);if(204!==g.status)throw new A(g)}},function(t){t.url=function(t,e){return r.URLExt.join(t,o,e)}}(a||(a={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return a});var r=n(205),i=n.n(r);class o{static reg(t){return new RegExp("^"+t+"$","i")}}o.N={integer:"[+-]?[0-9]+",integer_pos:"[+]?[0-9]+",integer_zero_ff:"([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])",number:"[+-]?([0-9]*[.])?[0-9]+(e-?[0-9]*)?",number_pos:"[+]?([0-9]*[.])?[0-9]+(e-?[0-9]*)?",number_zero_hundred:"[+]?(([0-9]|[1-9][0-9])([.][0-9]+)?|100)",number_zero_one:"[+]?(1([.][0]+)?|0([.][0-9]+)?)"},o.B={angle:`(${o.N.number}(deg|rad|grad|turn)|0)`,frequency:`${o.N.number}(Hz|kHz)`,ident:String.raw`-?([_a-z]|[\xA0-\xFF]|\\[0-9a-f]{1,6}(\r\n|[ \t\r\n\f])?|\\[^\r\n\f0-9a-f])([_a-z0-9-]|[\xA0-\xFF]|\\[0-9a-f]{1,6}(\r\n|[ \t\r\n\f])?|\\[^\r\n\f0-9a-f])*`,len_or_perc:`(0|${o.N.number}(px|em|rem|ex|in|cm|mm|pt|pc|%))`,length:`(${o.N.number}(px|em|rem|ex|in|cm|mm|pt|pc)|0)`,length_pos:`(${o.N.number_pos}(px|em|rem|ex|in|cm|mm|pt|pc)|0)`,percentage:`${o.N.number}%`,percentage_pos:`${o.N.number_pos}%`,percentage_zero_hundred:`${o.N.number_zero_hundred}%`,string:String.raw`(\"([^\n\r\f\\"]|\\\n|\r\n|\r|\f|\\[0-9a-f]{1,6}(\r\n|[ \t\r\n\f])?|\\[^\r\n\f0-9a-f])*\")|(\'([^\n\r\f\\']|\\\n|\r\n|\r|\f|\\[0-9a-f]{1,6}(\r\n|[ \t\r\n\f])?|\\[^\r\n\f0-9a-f])*\')`,time:`${o.N.number}(s|ms)`,url:"url\\(.*?\\)",z_index:"[+-]?[0-9]{1,7}"},o.A={absolute_size:"xx-small|x-small|small|medium|large|x-large|xx-large",attachment:"scroll|fixed|local",bg_origin:"border-box|padding-box|content-box",border_style:"none|hidden|dotted|dashed|solid|double|groove|ridge|inset|outset",box:"border-box|padding-box|content-box",display_inside:"auto|block|table|flex|grid",display_outside:"block-level|inline-level|none|table-row-group|table-header-group|table-footer-group|table-row|table-cell|table-column-group|table-column|table-caption",ending_shape:"circle|ellipse",generic_family:"serif|sans-serif|cursive|fantasy|monospace",generic_voice:"male|female|child",relative_size:"smaller|larger",repeat_style:"repeat-x|repeat-y|((?:repeat|space|round|no-repeat)(?:\\s*(?:repeat|space|round|no-repeat))?)",side_or_corner:"(left|right)?\\s*(top|bottom)?",single_animation_direction:"normal|reverse|alternate|alternate-reverse",single_animation_fill_mode:"none|forwards|backwards|both",single_animation_play_state:"running|paused"},o._COLOR={hex:"\\#(0x)?[0-9a-f]+",name:"aliceblue|antiquewhite|aqua|aquamarine|azure|beige|bisque|black|blanchedalmond|blue|blueviolet|brown|burlywood|cadetblue|chartreuse|chocolate|coral|cornflowerblue|cornsilk|crimson|cyan|darkblue|darkcyan|darkgoldenrod|darkgray|darkgreen|darkkhaki|darkmagenta|darkolivegreen|darkorange|darkorchid|darkred|darksalmon|darkseagreen|darkslateblue|darkslategray|darkturquoise|darkviolet|deeppink|deepskyblue|dimgray|dodgerblue|firebrick|floralwhite|forestgreen|fuchsia|gainsboro|ghostwhite|gold|goldenrod|gray|green|greenyellow|honeydew|hotpink|indianred|indigo|ivory|khaki|lavender|lavenderblush|lawngreen|lemonchiffon|lightblue|lightcoral|lightcyan|lightgoldenrodyellow|lightgreen|lightgrey|lightpink|lightsalmon|lightseagreen|lightskyblue|lightslategray|lightsteelblue|lightyellow|lime|limegreen|linen|magenta|maroon|mediumaquamarine|mediumblue|mediumorchid|mediumpurple|mediumseagreen|mediumslateblue|mediumspringgreen|mediumturquoise|mediumvioletred|midnightblue|mintcream|mistyrose|moccasin|navajowhite|navy|oldlace|olive|olivedrab|orange|orangered|orchid|palegoldenrod|palegreen|paleturquoise|palevioletred|papayawhip|peachpuff|peru|pink|plum|powderblue|purple|red|rosybrown|royalblue|saddlebrown|salmon|sandybrown|seagreen|seashell|sienna|silver|skyblue|slateblue|slategray|snow|springgreen|steelblue|tan|teal|thistle|tomato|turquoise|transparent|violet|wheat|white|whitesmoke|yellow|yellowgreen",rgb:String.raw`rgb\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*\)`,rgba:String.raw`rgba\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(${o.N.integer_zero_ff}|${o.N.number_zero_one}|${o.B.percentage_zero_hundred})\s*\)`},o._C={alpha:`${o.N.integer_zero_ff}|${o.N.number_zero_one}|${o.B.percentage_zero_hundred}`,alphavalue:o.N.number_zero_one,bg_position:`((${o.B.len_or_perc}|left|center|right|top|bottom)\\s*){1,4}`,bg_size:`(${o.B.length_pos}|${o.B.percentage}|auto){1,2}|cover|contain`,border_width:`thin|medium|thick|${o.B.length}`,bottom:`${o.B.length}|auto`,color:`${o._COLOR.hex}|${o._COLOR.rgb}|${o._COLOR.rgba}|${o._COLOR.name}`,family_name:`${o.B.string}|(${o.B.ident}\\s*)+`,image_decl:o.B.url,left:`${o.B.length}|auto`,loose_quotable_words:`(${o.B.ident})+`,margin_width:`${o.B.len_or_perc}|auto`,padding_width:`${o.B.length_pos}|${o.B.percentage_pos}`,page_url:o.B.url,position:`((${o.B.len_or_perc}|left|center|right|top|bottom)\\s*){1,4}`,right:`${o.B.length}|auto`,shadow:"",size:`closest-side|farthest-side|closest-corner|farthest-corner|${o.B.length}|(${o.B.len_or_perc})\\s+(${o.B.len_or_perc})`,top:`${o.B.length}|auto`},o._C1={image_list:`image\\(\\s*(${o.B.url})*\\s*(${o.B.url}|${o._C.color})\\s*\\)`,shadow:`((${o._C.color})\\s+((${o.B.length})\\s*){2,4}(s+inset)?)|((inset\\s+)?((${o.B.length})\\s*){2,4}\\s*(${o._C.color})?)`},o._C2={bg_image:`(${o.B.url}|${o._C1.image_list})|none`,image:`${o.B.url}|${o._C1.image_list}`,shape:`rect\\(\\s*(${o._C.top})\\s*,\\s*(${o._C.right})\\s*,\\s*(${o._C.bottom})\\s*,\\s*(${o._C.left})\\s*\\)`},o.C=Object.assign({},o._C,o._C1,o._C2),o.AP={border_collapse:"collapse|separate",box:"normal|none|contents",box_sizing:"content-box|padding-box|border-box",caption_side:"top|bottom",clear:"none|left|right|both",direction:"ltr|rtl",empty_cells:"show|hide",float:"left|right|none",font_stretch:"normal|wider|narrower|ultra-condensed|extra-condensed|condensed|semi-condensed|semi-expanded|expanded|extra-expanded|ultra-expanded",font_style:"normal|italic|oblique",font_variant:"normal|small-caps",font_weight:"normal|bold|bolder|lighter|100|200|300|400|500|600|700|800|900",list_style_position:"inside|outside",list_style_type:"disc|circle|square|decimal|decimal-leading-zero|lower-roman|upper-roman|lower-greek|lower-latin|upper-latin|armenian|georgian|lower-alpha|upper-alpha|none",overflow:"visible|hidden|scroll|auto",overflow_wrap:"normal|break-word",overflow_x:"visible|hidden|scroll|auto|no-display|no-content",page_break_after:"auto|always|avoid|left|right",page_break_before:"auto|always|avoid|left|right",page_break_inside:"avoid|auto",position:"static|relative|absolute",resize:"none|both|horizontal|vertical",speak:"normal|none|spell-out",speak_header:"once|always",speak_numeral:"digits|continuous",speak_punctuation:"code|none",table_layout:"auto|fixed",text_align:"left|right|center|justify",text_decoration:"none|((underline|overline|line-through|blink)\\s*)+",text_transform:"capitalize|uppercase|lowercase|none",text_wrap:"normal|unrestricted|none|suppress",unicode_bidi:"normal|embed|bidi-override",visibility:"visible|hidden|collapse",white_space:"normal|pre|nowrap|pre-wrap|pre-line",word_break:"normal|keep-all|break-all"},o._CP={background_attachment:`${o.A.attachment}(,\\s*${o.A.attachment})*`,background_color:o.C.color,background_origin:`${o.A.box}(,\\s*${o.A.box})*`,background_repeat:`${o.A.repeat_style}(,\\s*${o.A.repeat_style})*`,border:`((${o.C.border_width}|${o.A.border_style}|${o.C.color})\\s*){1,3}`,border_radius:`((${o.B.len_or_perc})\\s*){1,4}(\\/\\s*((${o.B.len_or_perc})\\s*){1,4})?`,border_spacing:`${o.B.length}\\s*(${o.B.length})?`,border_top_color:o.C.color,border_top_style:o.A.border_style,border_width:`((${o.C.border_width})\\s*){1,4}`,color:o.C.color,cursor:`(${o.B.url}(\\s*,\\s*)?)*(auto|crosshair|default|pointer|move|e-resize|ne-resize|nw-resize|n-resize|se-resize|sw-resize|s-resize|w-resize|text|wait|help|progress|all-scroll|col-resize|hand|no-drop|not-allowed|row-resize|vertical-text)`,display:`inline|block|list-item|run-in|inline-list-item|inline-block|table|inline-table|table-cell|table-caption|flex|inline-flex|grid|inline-grid|${o.A.display_inside}|${o.A.display_outside}|inherit|inline-box|inline-stack`,display_outside:o.A.display_outside,elevation:`${o.B.angle}|below|level|above|higher|lower`,font_family:`(${o.C.family_name}|${o.A.generic_family})(,\\s*(${o.C.family_name}|${o.A.generic_family}))*`,height:`${o.B.length}|${o.B.percentage}|auto`,letter_spacing:`normal|${o.B.length}`,list_style_image:`${o.C.image}|none`,margin_right:o.C.margin_width,max_height:`${o.B.length_pos}|${o.B.percentage_pos}|none|auto`,min_height:`${o.B.length_pos}|${o.B.percentage_pos}|auto`,opacity:o.C.alphavalue,outline_color:`${o.C.color}|invert`,outline_width:o.C.border_width,padding:`((${o.C.padding_width})\\s*){1,4}`,padding_top:o.C.padding_width,pitch_range:o.N.number,right:`${o.B.length}|${o.B.percentage}|auto`,stress:o.N.number,text_indent:`${o.B.length}|${o.B.percentage}`,text_shadow:`none|${o.C.shadow}(,\\s*(${o.C.shadow}))*`,volume:`${o.N.number_pos}|${o.B.percentage_pos}|silent|x-soft|soft|medium|loud|x-loud`,word_wrap:o.AP.overflow_wrap,zoom:`normal|${o.N.number_pos}|${o.B.percentage_pos}`,backface_visibility:o.AP.visibility,background_clip:`${o.A.box}(,\\s*(${o.A.box}))*`,background_position:`${o.C.bg_position}(,\\s*(${o.C.bg_position}))*`,border_bottom_color:o.C.color,border_bottom_style:o.A.border_style,border_color:`((${o.C.color})\\s*){1,4}`,border_left_color:o.C.color,border_right_color:o.C.color,border_style:`((${o.A.border_style})\\s*){1,4}`,border_top_left_radius:`(${o.B.length}|${o.B.percentage})(\\s*(${o.B.length}|${o.B.percentage}))?`,border_top_width:o.C.border_width,box_shadow:`none|${o.C.shadow}(,\\s*(${o.C.shadow}))*`,clip:`${o.C.shape}|auto`,display_inside:o.A.display_inside,font_size:`${o.A.absolute_size}|${o.A.relative_size}|${o.B.length_pos}|${o.B.percentage_pos}`,line_height:`normal|${o.N.number_pos}|${o.B.length_pos}|${o.B.percentage_pos}`,margin_left:o.C.margin_width,max_width:`${o.B.length_pos}|${o.B.percentage_pos}|none|auto`,outline_style:o.A.border_style,padding_bottom:o.C.padding_width,padding_right:o.C.padding_width,perspective:`none|${o.B.length}`,richness:o.N.number,text_overflow:`((clip|ellipsis|${o.B.string})\\s*){1,2}`,top:`${o.B.length}|${o.B.percentage}|auto`,width:`${o.B.length_pos}|${o.B.percentage_pos}|auto`,z_index:`auto|${o.B.z_index}`,background:`(((${o.C.bg_position}\\s*(\\/\\s*${o.C.bg_size})?)|(${o.A.repeat_style})|(${o.A.attachment})|(${o.A.bg_origin})|(${o.C.bg_image})|(${o.C.color}))\\s*)+`,background_size:`${o.C.bg_size}(,\\s*${o.C.bg_size})*`,border_bottom_left_radius:`(${o.B.length}|${o.B.percentage})(\\s*(${o.B.length}|${o.B.percentage}))?`,border_bottom_width:o.C.border_width,border_left_style:o.A.border_style,border_right_style:o.A.border_style,border_top:`((${o.C.border_width}|${o.A.border_style}|${o.C.color})\\s*){1,3}`,bottom:`${o.B.len_or_perc}|auto`,list_style:`((${o.AP.list_style_type}|${o.AP.list_style_position}|${o.C.image}|none})\\s*){1,3}`,margin_top:o.C.margin_width,outline:`((${o.C.color}|invert|${o.A.border_style}|${o.C.border_width})\\s*){1,3}`,overflow_y:o.AP.overflow_x,pitch:`${o.B.frequency}|x-low|low|medium|high|x-high`,vertical_align:`baseline|sub|super|top|text-top|middle|bottom|text-bottom|${o.B.len_or_perc}`,word_spacing:`normal|${o.B.length}`,background_image:`${o.C.bg_image}(,\\s*${o.C.bg_image})*`,border_bottom_right_radius:`(${o.B.length}|${o.B.percentage})(\\s*(${o.B.length}|${o.B.percentage}))?`,border_left_width:o.C.border_width,border_right_width:o.C.border_width,left:`${o.B.len_or_perc}|auto`,margin_bottom:o.C.margin_width,pause_after:`${o.B.time}|${o.B.percentage}`,speech_rate:`${o.N.number}|x-slow|slow|medium|fast|x-fast|faster|slower`,transition_duration:`${o.B.time}(,\\s*${o.B.time})*`,border_bottom:`((${o.C.border_width}|${o.A.border_style}|${o.C.color})\\s*){1,3}`,border_right:`((${o.C.border_width}|${o.A.border_style}|${o.C.color})\\s*){1,3}`,margin:`((${o.C.margin_width})\\s*){1,4}`,padding_left:o.C.padding_width,border_left:`((${o.C.border_width}|${o.A.border_style}|${o.C.color})\\s*){1,3}`,quotes:`(${o.B.string}\\s*${o.B.string})+|none`,border_top_right_radius:`(${o.B.length}|${o.B.percentage})(\\s*(${o.B.length}|${o.B.percentage}))?`,min_width:`${o.B.length_pos}|${o.B.percentage_pos}|auto`},o._CP1={font:`(((((${o.AP.font_style}|${o.AP.font_variant}|${o.AP.font_weight})\\s*){1,3})?\\s*(${o._CP.font_size})\\s*(\\/\\s*(${o._CP.line_height}))?\\s+(${o._CP.font_family}))|caption|icon|menu|message-box|small-caption|status-bar)`},o.CP=Object.assign({},o._CP,o._CP1),o.BORDER_COLLAPSE=o.reg(o.AP.border_collapse),o.BOX=o.reg(o.AP.box),o.BOX_SIZING=o.reg(o.AP.box_sizing),o.CAPTION_SIDE=o.reg(o.AP.caption_side),o.CLEAR=o.reg(o.AP.clear),o.DIRECTION=o.reg(o.AP.direction),o.EMPTY_CELLS=o.reg(o.AP.empty_cells),o.FLOAT=o.reg(o.AP.float),o.FONT_STRETCH=o.reg(o.AP.font_stretch),o.FONT_STYLE=o.reg(o.AP.font_style),o.FONT_VARIANT=o.reg(o.AP.font_variant),o.FONT_WEIGHT=o.reg(o.AP.font_weight),o.LIST_STYLE_POSITION=o.reg(o.AP.list_style_position),o.LIST_STYLE_TYPE=o.reg(o.AP.list_style_type),o.OVERFLOW=o.reg(o.AP.overflow),o.OVERFLOW_WRAP=o.reg(o.AP.overflow_wrap),o.OVERFLOW_X=o.reg(o.AP.overflow_x),o.PAGE_BREAK_AFTER=o.reg(o.AP.page_break_after),o.PAGE_BREAK_BEFORE=o.reg(o.AP.page_break_before),o.PAGE_BREAK_INSIDE=o.reg(o.AP.page_break_inside),o.POSITION=o.reg(o.AP.position),o.RESIZE=o.reg(o.AP.resize),o.SPEAK=o.reg(o.AP.speak),o.SPEAK_HEADER=o.reg(o.AP.speak_header),o.SPEAK_NUMERAL=o.reg(o.AP.speak_numeral),o.SPEAK_PUNCTUATION=o.reg(o.AP.speak_punctuation),o.TABLE_LAYOUT=o.reg(o.AP.table_layout),o.TEXT_ALIGN=o.reg(o.AP.text_align),o.TEXT_DECORATION=o.reg(o.AP.text_decoration),o.TEXT_TRANSFORM=o.reg(o.AP.text_transform),o.TEXT_WRAP=o.reg(o.AP.text_wrap),o.UNICODE_BIDI=o.reg(o.AP.unicode_bidi),o.VISIBILITY=o.reg(o.AP.visibility),o.WHITE_SPACE=o.reg(o.AP.white_space),o.WORD_BREAK=o.reg(o.AP.word_break),o.BACKGROUND_ATTACHMENT=o.reg(o.CP.background_attachment),o.BACKGROUND_COLOR=o.reg(o.CP.background_color),o.BACKGROUND_ORIGIN=o.reg(o.CP.background_origin),o.BACKGROUND_REPEAT=o.reg(o.CP.background_repeat),o.BORDER=o.reg(o.CP.border),o.BORDER_RADIUS=o.reg(o.CP.border_radius),o.BORDER_SPACING=o.reg(o.CP.border_spacing),o.BORDER_TOP_COLOR=o.reg(o.CP.border_top_color),o.BORDER_TOP_STYLE=o.reg(o.CP.border_top_style),o.BORDER_WIDTH=o.reg(o.CP.border_width),o.COLOR=o.reg(o.CP.color),o.CURSOR=o.reg(o.CP.cursor),o.DISPLAY=o.reg(o.CP.display),o.DISPLAY_OUTSIDE=o.reg(o.CP.display_outside),o.ELEVATION=o.reg(o.CP.elevation),o.FONT_FAMILY=o.reg(o.CP.font_family),o.HEIGHT=o.reg(o.CP.height),o.LETTER_SPACING=o.reg(o.CP.letter_spacing),o.LIST_STYLE_IMAGE=o.reg(o.CP.list_style_image),o.MARGIN_RIGHT=o.reg(o.CP.margin_right),o.MAX_HEIGHT=o.reg(o.CP.max_height),o.MIN_HEIGHT=o.reg(o.CP.min_height),o.OPACITY=o.reg(o.CP.opacity),o.OUTLINE_COLOR=o.reg(o.CP.outline_color),o.OUTLINE_WIDTH=o.reg(o.CP.outline_width),o.PADDING=o.reg(o.CP.padding),o.PADDING_TOP=o.reg(o.CP.padding_top),o.PITCH_RANGE=o.reg(o.CP.pitch_range),o.RIGHT=o.reg(o.CP.right),o.STRESS=o.reg(o.CP.stress),o.TEXT_INDENT=o.reg(o.CP.text_indent),o.TEXT_SHADOW=o.reg(o.CP.text_shadow),o.VOLUME=o.reg(o.CP.volume),o.WORD_WRAP=o.reg(o.CP.word_wrap),o.ZOOM=o.reg(o.CP.zoom),o.BACKFACE_VISIBILITY=o.reg(o.CP.backface_visibility),o.BACKGROUND_CLIP=o.reg(o.CP.background_clip),o.BACKGROUND_POSITION=o.reg(o.CP.background_position),o.BORDER_BOTTOM_COLOR=o.reg(o.CP.border_bottom_color),o.BORDER_BOTTOM_STYLE=o.reg(o.CP.border_bottom_style),o.BORDER_COLOR=o.reg(o.CP.border_color),o.BORDER_LEFT_COLOR=o.reg(o.CP.border_left_color),o.BORDER_RIGHT_COLOR=o.reg(o.CP.border_right_color),o.BORDER_STYLE=o.reg(o.CP.border_style),o.BORDER_TOP_LEFT_RADIUS=o.reg(o.CP.border_top_left_radius),o.BORDER_TOP_WIDTH=o.reg(o.CP.border_top_width),o.BOX_SHADOW=o.reg(o.CP.box_shadow),o.CLIP=o.reg(o.CP.clip),o.DISPLAY_INSIDE=o.reg(o.CP.display_inside),o.FONT_SIZE=o.reg(o.CP.font_size),o.LINE_HEIGHT=o.reg(o.CP.line_height),o.MARGIN_LEFT=o.reg(o.CP.margin_left),o.MAX_WIDTH=o.reg(o.CP.max_width),o.OUTLINE_STYLE=o.reg(o.CP.outline_style),o.PADDING_BOTTOM=o.reg(o.CP.padding_bottom),o.PADDING_RIGHT=o.reg(o.CP.padding_right),o.PERSPECTIVE=o.reg(o.CP.perspective),o.RICHNESS=o.reg(o.CP.richness),o.TEXT_OVERFLOW=o.reg(o.CP.text_overflow),o.TOP=o.reg(o.CP.top),o.WIDTH=o.reg(o.CP.width),o.Z_INDEX=o.reg(o.CP.z_index),o.BACKGROUND=o.reg(o.CP.background),o.BACKGROUND_SIZE=o.reg(o.CP.background_size),o.BORDER_BOTTOM_LEFT_RADIUS=o.reg(o.CP.border_bottom_left_radius),o.BORDER_BOTTOM_WIDTH=o.reg(o.CP.border_bottom_width),o.BORDER_LEFT_STYLE=o.reg(o.CP.border_left_style),o.BORDER_RIGHT_STYLE=o.reg(o.CP.border_right_style),o.BORDER_TOP=o.reg(o.CP.border_top),o.BOTTOM=o.reg(o.CP.bottom),o.LIST_STYLE=o.reg(o.CP.list_style),o.MARGIN_TOP=o.reg(o.CP.margin_top),o.OUTLINE=o.reg(o.CP.outline),o.OVERFLOW_Y=o.reg(o.CP.overflow_y),o.PITCH=o.reg(o.CP.pitch),o.VERTICAL_ALIGN=o.reg(o.CP.vertical_align),o.WORD_SPACING=o.reg(o.CP.word_spacing),o.BACKGROUND_IMAGE=o.reg(o.CP.background_image),o.BORDER_BOTTOM_RIGHT_RADIUS=o.reg(o.CP.border_bottom_right_radius),o.BORDER_LEFT_WIDTH=o.reg(o.CP.border_left_width),o.BORDER_RIGHT_WIDTH=o.reg(o.CP.border_right_width),o.LEFT=o.reg(o.CP.left),o.MARGIN_BOTTOM=o.reg(o.CP.margin_bottom),o.PAUSE_AFTER=o.reg(o.CP.pause_after),o.SPEECH_RATE=o.reg(o.CP.speech_rate),o.TRANSITION_DURATION=o.reg(o.CP.transition_duration),o.BORDER_BOTTOM=o.reg(o.CP.border_bottom),o.BORDER_RIGHT=o.reg(o.CP.border_right),o.MARGIN=o.reg(o.CP.margin),o.PADDING_LEFT=o.reg(o.CP.padding_left),o.BORDER_LEFT=o.reg(o.CP.border_left),o.FONT=o.reg(o.CP.font),o.QUOTES=o.reg(o.CP.quotes),o.BORDER_TOP_RIGHT_RADIUS=o.reg(o.CP.border_top_right_radius),o.MIN_WIDTH=o.reg(o.CP.min_width);const a=new class{constructor(){this._options={allowedTags:["a","abbr","acronym","address","area","article","aside","audio","b","bdi","bdo","big","blockquote","br","button","canvas","caption","center","cite","code","col","colgroup","colspan","command","data","datalist","dd","del","details","dfn","dir","div","dl","dt","em","fieldset","figcaption","figure","font","footer","form","h1","h2","h3","h4","h5","h6","header","hgroup","hr","i","img","input","ins","kbd","label","legend","li","map","mark","menu","meter","nav","nobr","ol","optgroup","option","output","p","pre","progress","q","rowspan","s","samp","section","select","small","source","span","strike","strong","sub","summary","sup","table","tbody","td","textarea","tfoot","th","thead","time","tr","track","tt","u","ul","var","video","wbr"],allowedAttributes:{"*":["class","dir","draggable","hidden","id","inert","itemprop","itemref","itemscope","lang","spellcheck","style","title","translate"],a:["accesskey","coords","href","hreflang","name","rel","shape","tabindex","target","type"],area:["accesskey","alt","coords","href","nohref","shape","tabindex"],audio:["autoplay","controls","loop","mediagroup","muted","preload","src"],bdo:["dir"],blockquote:["cite"],br:["clear"],button:["accesskey","disabled","name","tabindex","type","value"],canvas:["height","width"],caption:["align"],col:["align","char","charoff","span","valign","width"],colgroup:["align","char","charoff","span","valign","width"],command:["checked","command","disabled","icon","label","radiogroup","type"],data:["value"],del:["cite","datetime"],details:["open"],dir:["compact"],div:["align"],dl:["compact"],fieldset:["disabled"],font:["color","face","size"],form:["accept","action","autocomplete","enctype","method","name","novalidate"],h1:["align"],h2:["align"],h3:["align"],h4:["align"],h5:["align"],h6:["align"],hr:["align","noshade","size","width"],iframe:["align","frameborder","height","marginheight","marginwidth","width"],img:["align","alt","border","height","hspace","ismap","name","src","usemap","vspace","width"],input:["accept","accesskey","align","alt","autocomplete","checked","disabled","inputmode","ismap","list","max","maxlength","min","multiple","name","placeholder","readonly","required","size","src","step","tabindex","type","usemap","value"],ins:["cite","datetime"],label:["accesskey","for"],legend:["accesskey","align"],li:["type","value"],map:["name"],menu:["compact","label","type"],meter:["high","low","max","min","value"],ol:["compact","reversed","start","type"],optgroup:["disabled","label"],option:["disabled","label","selected","value"],output:["for","name"],p:["align"],pre:["width"],progress:["max","min","value"],q:["cite"],select:["autocomplete","disabled","multiple","name","required","size","tabindex"],source:["type"],table:["align","bgcolor","border","cellpadding","cellspacing","frame","rules","summary","width"],tbody:["align","char","charoff","valign"],td:["abbr","align","axis","bgcolor","char","charoff","colspan","headers","height","nowrap","rowspan","scope","valign","width"],textarea:["accesskey","autocomplete","cols","disabled","inputmode","name","placeholder","readonly","required","rows","tabindex","wrap"],tfoot:["align","char","charoff","valign"],th:["abbr","align","axis","bgcolor","char","charoff","colspan","headers","height","nowrap","rowspan","scope","valign","width"],thead:["align","char","charoff","valign"],tr:["align","bgcolor","char","charoff","valign"],track:["default","kind","label","srclang"],ul:["compact","type"],video:["autoplay","controls","height","loop","mediagroup","muted","poster","preload","src","width"]},allowedStyles:{"*":{"backface-visibility":[o.BACKFACE_VISIBILITY],background:[o.BACKGROUND],"background-attachment":[o.BACKGROUND_ATTACHMENT],"background-clip":[o.BACKGROUND_CLIP],"background-color":[o.BACKGROUND_COLOR],"background-image":[o.BACKGROUND_IMAGE],"background-origin":[o.BACKGROUND_ORIGIN],"background-position":[o.BACKGROUND_POSITION],"background-repeat":[o.BACKGROUND_REPEAT],"background-size":[o.BACKGROUND_SIZE],border:[o.BORDER],"border-bottom":[o.BORDER_BOTTOM],"border-bottom-color":[o.BORDER_BOTTOM_COLOR],"border-bottom-left-radius":[o.BORDER_BOTTOM_LEFT_RADIUS],"border-bottom-right-radius":[o.BORDER_BOTTOM_RIGHT_RADIUS],"border-bottom-style":[o.BORDER_BOTTOM_STYLE],"border-bottom-width":[o.BORDER_BOTTOM_WIDTH],"border-collapse":[o.BORDER_COLLAPSE],"border-color":[o.BORDER_COLOR],"border-left":[o.BORDER_LEFT],"border-left-color":[o.BORDER_LEFT_COLOR],"border-left-style":[o.BORDER_LEFT_STYLE],"border-left-width":[o.BORDER_LEFT_WIDTH],"border-radius":[o.BORDER_RADIUS],"border-right":[o.BORDER_RIGHT],"border-right-color":[o.BORDER_RIGHT_COLOR],"border-right-style":[o.BORDER_RIGHT_STYLE],"border-right-width":[o.BORDER_RIGHT_WIDTH],"border-spacing":[o.BORDER_SPACING],"border-style":[o.BORDER_STYLE],"border-top":[o.BORDER_TOP],"border-top-color":[o.BORDER_TOP_COLOR],"border-top-left-radius":[o.BORDER_TOP_LEFT_RADIUS],"border-top-right-radius":[o.BORDER_TOP_RIGHT_RADIUS],"border-top-style":[o.BORDER_TOP_STYLE],"border-top-width":[o.BORDER_TOP_WIDTH],"border-width":[o.BORDER_WIDTH],bottom:[o.BOTTOM],box:[o.BOX],"box-shadow":[o.BOX_SHADOW],"box-sizing":[o.BOX_SIZING],"caption-side":[o.CAPTION_SIDE],clear:[o.CLEAR],clip:[o.CLIP],color:[o.COLOR],cursor:[o.CURSOR],direction:[o.DIRECTION],display:[o.DISPLAY],"display-inside":[o.DISPLAY_INSIDE],"display-outside":[o.DISPLAY_OUTSIDE],elevation:[o.ELEVATION],"empty-cells":[o.EMPTY_CELLS],float:[o.FLOAT],font:[o.FONT],"font-family":[o.FONT_FAMILY],"font-size":[o.FONT_SIZE],"font-stretch":[o.FONT_STRETCH],"font-style":[o.FONT_STYLE],"font-variant":[o.FONT_VARIANT],"font-weight":[o.FONT_WEIGHT],height:[o.HEIGHT],left:[o.LEFT],"letter-spacing":[o.LETTER_SPACING],"line-height":[o.LINE_HEIGHT],"list-style":[o.LIST_STYLE],"list-style-image":[o.LIST_STYLE_IMAGE],"list-style-position":[o.LIST_STYLE_POSITION],"list-style-type":[o.LIST_STYLE_TYPE],margin:[o.MARGIN],"margin-bottom":[o.MARGIN_BOTTOM],"margin-left":[o.MARGIN_LEFT],"margin-right":[o.MARGIN_RIGHT],"margin-top":[o.MARGIN_TOP],"max-height":[o.MAX_HEIGHT],"max-width":[o.MAX_WIDTH],"min-height":[o.MIN_HEIGHT],"min-width":[o.MIN_WIDTH],opacity:[o.OPACITY],outline:[o.OUTLINE],"outline-color":[o.OUTLINE_COLOR],"outline-style":[o.OUTLINE_STYLE],"outline-width":[o.OUTLINE_WIDTH],overflow:[o.OVERFLOW],"overflow-wrap":[o.OVERFLOW_WRAP],"overflow-x":[o.OVERFLOW_X],"overflow-y":[o.OVERFLOW_Y],padding:[o.PADDING],"padding-bottom":[o.PADDING_BOTTOM],"padding-left":[o.PADDING_LEFT],"padding-right":[o.PADDING_RIGHT],"padding-top":[o.PADDING_TOP],"page-break-after":[o.PAGE_BREAK_AFTER],"page-break-before":[o.PAGE_BREAK_BEFORE],"page-break-inside":[o.PAGE_BREAK_INSIDE],"pause-after":[o.PAUSE_AFTER],perspective:[o.PERSPECTIVE],pitch:[o.PITCH],"pitch-range":[o.PITCH_RANGE],position:[o.POSITION],quotes:[o.QUOTES],resize:[o.RESIZE],richness:[o.RICHNESS],right:[o.RIGHT],speak:[o.SPEAK],"speak-header":[o.SPEAK_HEADER],"speak-numeral":[o.SPEAK_NUMERAL],"speak-punctuation":[o.SPEAK_PUNCTUATION],"speech-rate":[o.SPEECH_RATE],stress:[o.STRESS],"table-layout":[o.TABLE_LAYOUT],"text-align":[o.TEXT_ALIGN],"text-decoration":[o.TEXT_DECORATION],"text-indent":[o.TEXT_INDENT],"text-overflow":[o.TEXT_OVERFLOW],"text-shadow":[o.TEXT_SHADOW],"text-transform":[o.TEXT_TRANSFORM],"text-wrap":[o.TEXT_WRAP],top:[o.TOP],"unicode-bidi":[o.UNICODE_BIDI],"vertical-align":[o.VERTICAL_ALIGN],visibility:[o.VISIBILITY],volume:[o.VOLUME],"white-space":[o.WHITE_SPACE],width:[o.WIDTH],"word-break":[o.WORD_BREAK],"word-spacing":[o.WORD_SPACING],"word-wrap":[o.WORD_WRAP],"z-index":[o.Z_INDEX],zoom:[o.ZOOM]}},transformTags:{a:i.a.simpleTransform("a",{rel:"nofollow"}),input:i.a.simpleTransform("input",{disabled:"disabled"})},allowedSchemesByTag:{img:i.a.defaults.allowedSchemes.concat(["attachment"])},allowedSchemesAppliedToAttributes:["href","cite"]}}sanitize(t,e){return i()(t,Object.assign({},this._options,e||{}))}}},function(t,e,n){var r=n(594),i={input:!0,option:!0,optgroup:!0,select:!0,button:!0,datalist:!0,textarea:!0},o={tr:{tr:!0,th:!0,td:!0},th:{th:!0},td:{thead:!0,th:!0,td:!0},body:{head:!0,link:!0,script:!0},li:{li:!0},p:{p:!0},h1:{p:!0},h2:{p:!0},h3:{p:!0},h4:{p:!0},h5:{p:!0},h6:{p:!0},select:i,input:i,output:i,button:i,datalist:i,textarea:i,option:{option:!0},optgroup:{optgroup:!0}},a={__proto__:null,area:!0,base:!0,basefont:!0,br:!0,col:!0,command:!0,embed:!0,frame:!0,hr:!0,img:!0,input:!0,isindex:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0},s={__proto__:null,math:!0,svg:!0},A={__proto__:null,mi:!0,mo:!0,mn:!0,ms:!0,mtext:!0,"annotation-xml":!0,foreignObject:!0,desc:!0,title:!0},l=/\s|\//;function c(t,e){this._options=e||{},this._cbs=t||{},this._tagname="",this._attribname="",this._attribvalue="",this._attribs=null,this._stack=[],this._foreignContext=[],this.startIndex=0,this.endIndex=null,this._lowerCaseTagNames="lowerCaseTags"in this._options?!!this._options.lowerCaseTags:!this._options.xmlMode,this._lowerCaseAttributeNames="lowerCaseAttributeNames"in this._options?!!this._options.lowerCaseAttributeNames:!this._options.xmlMode,this._options.Tokenizer&&(r=this._options.Tokenizer),this._tokenizer=new r(this._options,this),this._cbs.onparserinit&&this._cbs.onparserinit(this)}n(30)(c,n(208).EventEmitter),c.prototype._updatePosition=function(t){null===this.endIndex?this._tokenizer._sectionStart<=t?this.startIndex=0:this.startIndex=this._tokenizer._sectionStart-t:this.startIndex=this.endIndex+1,this.endIndex=this._tokenizer.getAbsoluteIndex()},c.prototype.ontext=function(t){this._updatePosition(1),this.endIndex--,this._cbs.ontext&&this._cbs.ontext(t)},c.prototype.onopentagname=function(t){if(this._lowerCaseTagNames&&(t=t.toLowerCase()),this._tagname=t,!this._options.xmlMode&&t in o)for(var e;(e=this._stack[this._stack.length-1])in o[t];this.onclosetag(e));!this._options.xmlMode&&t in a||(this._stack.push(t),t in s?this._foreignContext.push(!0):t in A&&this._foreignContext.push(!1)),this._cbs.onopentagname&&this._cbs.onopentagname(t),this._cbs.onopentag&&(this._attribs={})},c.prototype.onopentagend=function(){this._updatePosition(1),this._attribs&&(this._cbs.onopentag&&this._cbs.onopentag(this._tagname,this._attribs),this._attribs=null),!this._options.xmlMode&&this._cbs.onclosetag&&this._tagname in a&&this._cbs.onclosetag(this._tagname),this._tagname=""},c.prototype.onclosetag=function(t){if(this._updatePosition(1),this._lowerCaseTagNames&&(t=t.toLowerCase()),(t in s||t in A)&&this._foreignContext.pop(),!this._stack.length||t in a&&!this._options.xmlMode)this._options.xmlMode||"br"!==t&&"p"!==t||(this.onopentagname(t),this._closeCurrentTag());else{var e=this._stack.lastIndexOf(t);if(-1!==e)if(this._cbs.onclosetag)for(e=this._stack.length-e;e--;)this._cbs.onclosetag(this._stack.pop());else this._stack.length=e;else"p"!==t||this._options.xmlMode||(this.onopentagname(t),this._closeCurrentTag())}},c.prototype.onselfclosingtag=function(){this._options.xmlMode||this._options.recognizeSelfClosing||this._foreignContext[this._foreignContext.length-1]?this._closeCurrentTag():this.onopentagend()},c.prototype._closeCurrentTag=function(){var t=this._tagname;this.onopentagend(),this._stack[this._stack.length-1]===t&&(this._cbs.onclosetag&&this._cbs.onclosetag(t),this._stack.pop())},c.prototype.onattribname=function(t){this._lowerCaseAttributeNames&&(t=t.toLowerCase()),this._attribname=t},c.prototype.onattribdata=function(t){this._attribvalue+=t},c.prototype.onattribend=function(){this._cbs.onattribute&&this._cbs.onattribute(this._attribname,this._attribvalue),this._attribs&&!Object.prototype.hasOwnProperty.call(this._attribs,this._attribname)&&(this._attribs[this._attribname]=this._attribvalue),this._attribname="",this._attribvalue=""},c.prototype._getInstructionName=function(t){var e=t.search(l),n=e<0?t:t.substr(0,e);return this._lowerCaseTagNames&&(n=n.toLowerCase()),n},c.prototype.ondeclaration=function(t){if(this._cbs.onprocessinginstruction){var e=this._getInstructionName(t);this._cbs.onprocessinginstruction("!"+e,"!"+t)}},c.prototype.onprocessinginstruction=function(t){if(this._cbs.onprocessinginstruction){var e=this._getInstructionName(t);this._cbs.onprocessinginstruction("?"+e,"?"+t)}},c.prototype.oncomment=function(t){this._updatePosition(4),this._cbs.oncomment&&this._cbs.oncomment(t),this._cbs.oncommentend&&this._cbs.oncommentend()},c.prototype.oncdata=function(t){this._updatePosition(1),this._options.xmlMode||this._options.recognizeCDATA?(this._cbs.oncdatastart&&this._cbs.oncdatastart(),this._cbs.ontext&&this._cbs.ontext(t),this._cbs.oncdataend&&this._cbs.oncdataend()):this.oncomment("[CDATA["+t+"]]")},c.prototype.onerror=function(t){this._cbs.onerror&&this._cbs.onerror(t)},c.prototype.onend=function(){if(this._cbs.onclosetag)for(var t=this._stack.length;t>0;this._cbs.onclosetag(this._stack[--t]));this._cbs.onend&&this._cbs.onend()},c.prototype.reset=function(){this._cbs.onreset&&this._cbs.onreset(),this._tokenizer.reset(),this._tagname="",this._attribname="",this._attribs=null,this._stack=[],this._cbs.onparserinit&&this._cbs.onparserinit(this)},c.prototype.parseComplete=function(t){this.reset(),this.end(t)},c.prototype.write=function(t){this._tokenizer.write(t)},c.prototype.end=function(t){this._tokenizer.end(t)},c.prototype.pause=function(){this._tokenizer.pause()},c.prototype.resume=function(){this._tokenizer.resume()},c.prototype.parseChunk=c.prototype.write,c.prototype.done=c.prototype.end,t.exports=c},function(t,e,n){t.exports=pt;var r=n(595),i=n(297),o=n(596),a=n(298),s=0,A=s++,l=s++,c=s++,u=s++,g=s++,h=s++,d=s++,f=s++,p=s++,I=s++,C=s++,m=s++,y=s++,E=s++,v=s++,B=s++,b=s++,Q=s++,w=s++,x=s++,_=s++,D=s++,M=s++,k=s++,S=s++,L=s++,R=s++,N=s++,F=s++,T=s++,U=s++,O=s++,G=s++,H=s++,j=s++,Y=s++,P=s++,K=s++,z=s++,J=s++,q=s++,V=s++,W=s++,X=s++,Z=s++,$=s++,tt=s++,et=s++,nt=s++,rt=s++,it=s++,ot=s++,at=s++,st=s++,At=s++,lt=0,ct=lt++,ut=lt++,gt=lt++;function ht(t){return" "===t||"\n"===t||"\t"===t||"\f"===t||"\r"===t}function dt(t,e,n){var r=t.toLowerCase();return t===r?function(t){t===r?this._state=e:(this._state=n,this._index--)}:function(i){i===r||i===t?this._state=e:(this._state=n,this._index--)}}function ft(t,e){var n=t.toLowerCase();return function(r){r===n||r===t?this._state=e:(this._state=c,this._index--)}}function pt(t,e){this._state=A,this._buffer="",this._sectionStart=0,this._index=0,this._bufferOffset=0,this._baseState=A,this._special=ct,this._cbs=e,this._running=!0,this._ended=!1,this._xmlMode=!(!t||!t.xmlMode),this._decodeEntities=!(!t||!t.decodeEntities)}pt.prototype._stateText=function(t){"<"===t?(this._index>this._sectionStart&&this._cbs.ontext(this._getSection()),this._state=l,this._sectionStart=this._index):this._decodeEntities&&this._special===ct&&"&"===t&&(this._index>this._sectionStart&&this._cbs.ontext(this._getSection()),this._baseState=A,this._state=it,this._sectionStart=this._index)},pt.prototype._stateBeforeTagName=function(t){"/"===t?this._state=g:"<"===t?(this._cbs.ontext(this._getSection()),this._sectionStart=this._index):">"===t||this._special!==ct||ht(t)?this._state=A:"!"===t?(this._state=v,this._sectionStart=this._index+1):"?"===t?(this._state=b,this._sectionStart=this._index+1):(this._state=this._xmlMode||"s"!==t&&"S"!==t?c:U,this._sectionStart=this._index)},pt.prototype._stateInTagName=function(t){("/"===t||">"===t||ht(t))&&(this._emitToken("onopentagname"),this._state=f,this._index--)},pt.prototype._stateBeforeCloseingTagName=function(t){ht(t)||(">"===t?this._state=A:this._special!==ct?"s"===t||"S"===t?this._state=O:(this._state=A,this._index--):(this._state=h,this._sectionStart=this._index))},pt.prototype._stateInCloseingTagName=function(t){(">"===t||ht(t))&&(this._emitToken("onclosetag"),this._state=d,this._index--)},pt.prototype._stateAfterCloseingTagName=function(t){">"===t&&(this._state=A,this._sectionStart=this._index+1)},pt.prototype._stateBeforeAttributeName=function(t){">"===t?(this._cbs.onopentagend(),this._state=A,this._sectionStart=this._index+1):"/"===t?this._state=u:ht(t)||(this._state=p,this._sectionStart=this._index)},pt.prototype._stateInSelfClosingTag=function(t){">"===t?(this._cbs.onselfclosingtag(),this._state=A,this._sectionStart=this._index+1):ht(t)||(this._state=f,this._index--)},pt.prototype._stateInAttributeName=function(t){("="===t||"/"===t||">"===t||ht(t))&&(this._cbs.onattribname(this._getSection()),this._sectionStart=-1,this._state=I,this._index--)},pt.prototype._stateAfterAttributeName=function(t){"="===t?this._state=C:"/"===t||">"===t?(this._cbs.onattribend(),this._state=f,this._index--):ht(t)||(this._cbs.onattribend(),this._state=p,this._sectionStart=this._index)},pt.prototype._stateBeforeAttributeValue=function(t){'"'===t?(this._state=m,this._sectionStart=this._index+1):"'"===t?(this._state=y,this._sectionStart=this._index+1):ht(t)||(this._state=E,this._sectionStart=this._index,this._index--)},pt.prototype._stateInAttributeValueDoubleQuotes=function(t){'"'===t?(this._emitToken("onattribdata"),this._cbs.onattribend(),this._state=f):this._decodeEntities&&"&"===t&&(this._emitToken("onattribdata"),this._baseState=this._state,this._state=it,this._sectionStart=this._index)},pt.prototype._stateInAttributeValueSingleQuotes=function(t){"'"===t?(this._emitToken("onattribdata"),this._cbs.onattribend(),this._state=f):this._decodeEntities&&"&"===t&&(this._emitToken("onattribdata"),this._baseState=this._state,this._state=it,this._sectionStart=this._index)},pt.prototype._stateInAttributeValueNoQuotes=function(t){ht(t)||">"===t?(this._emitToken("onattribdata"),this._cbs.onattribend(),this._state=f,this._index--):this._decodeEntities&&"&"===t&&(this._emitToken("onattribdata"),this._baseState=this._state,this._state=it,this._sectionStart=this._index)},pt.prototype._stateBeforeDeclaration=function(t){this._state="["===t?D:"-"===t?Q:B},pt.prototype._stateInDeclaration=function(t){">"===t&&(this._cbs.ondeclaration(this._getSection()),this._state=A,this._sectionStart=this._index+1)},pt.prototype._stateInProcessingInstruction=function(t){">"===t&&(this._cbs.onprocessinginstruction(this._getSection()),this._state=A,this._sectionStart=this._index+1)},pt.prototype._stateBeforeComment=function(t){"-"===t?(this._state=w,this._sectionStart=this._index+1):this._state=B},pt.prototype._stateInComment=function(t){"-"===t&&(this._state=x)},pt.prototype._stateAfterComment1=function(t){this._state="-"===t?_:w},pt.prototype._stateAfterComment2=function(t){">"===t?(this._cbs.oncomment(this._buffer.substring(this._sectionStart,this._index-2)),this._state=A,this._sectionStart=this._index+1):"-"!==t&&(this._state=w)},pt.prototype._stateBeforeCdata1=dt("C",M,B),pt.prototype._stateBeforeCdata2=dt("D",k,B),pt.prototype._stateBeforeCdata3=dt("A",S,B),pt.prototype._stateBeforeCdata4=dt("T",L,B),pt.prototype._stateBeforeCdata5=dt("A",R,B),pt.prototype._stateBeforeCdata6=function(t){"["===t?(this._state=N,this._sectionStart=this._index+1):(this._state=B,this._index--)},pt.prototype._stateInCdata=function(t){"]"===t&&(this._state=F)},pt.prototype._stateAfterCdata1=function(t){this._state="]"===t?T:N},pt.prototype._stateAfterCdata2=function(t){">"===t?(this._cbs.oncdata(this._buffer.substring(this._sectionStart,this._index-2)),this._state=A,this._sectionStart=this._index+1):"]"!==t&&(this._state=N)},pt.prototype._stateBeforeSpecial=function(t){"c"===t||"C"===t?this._state=G:"t"===t||"T"===t?this._state=W:(this._state=c,this._index--)},pt.prototype._stateBeforeSpecialEnd=function(t){this._special!==ut||"c"!==t&&"C"!==t?this._special!==gt||"t"!==t&&"T"!==t?this._state=A:this._state=tt:this._state=K},pt.prototype._stateBeforeScript1=ft("R",H),pt.prototype._stateBeforeScript2=ft("I",j),pt.prototype._stateBeforeScript3=ft("P",Y),pt.prototype._stateBeforeScript4=ft("T",P),pt.prototype._stateBeforeScript5=function(t){("/"===t||">"===t||ht(t))&&(this._special=ut),this._state=c,this._index--},pt.prototype._stateAfterScript1=dt("R",z,A),pt.prototype._stateAfterScript2=dt("I",J,A),pt.prototype._stateAfterScript3=dt("P",q,A),pt.prototype._stateAfterScript4=dt("T",V,A),pt.prototype._stateAfterScript5=function(t){">"===t||ht(t)?(this._special=ct,this._state=h,this._sectionStart=this._index-6,this._index--):this._state=A},pt.prototype._stateBeforeStyle1=ft("Y",X),pt.prototype._stateBeforeStyle2=ft("L",Z),pt.prototype._stateBeforeStyle3=ft("E",$),pt.prototype._stateBeforeStyle4=function(t){("/"===t||">"===t||ht(t))&&(this._special=gt),this._state=c,this._index--},pt.prototype._stateAfterStyle1=dt("Y",et,A),pt.prototype._stateAfterStyle2=dt("L",nt,A),pt.prototype._stateAfterStyle3=dt("E",rt,A),pt.prototype._stateAfterStyle4=function(t){">"===t||ht(t)?(this._special=ct,this._state=h,this._sectionStart=this._index-5,this._index--):this._state=A},pt.prototype._stateBeforeEntity=dt("#",ot,at),pt.prototype._stateBeforeNumericEntity=dt("X",At,st),pt.prototype._parseNamedEntityStrict=function(){if(this._sectionStart+1<this._index){var t=this._buffer.substring(this._sectionStart+1,this._index),e=this._xmlMode?a:i;e.hasOwnProperty(t)&&(this._emitPartial(e[t]),this._sectionStart=this._index+1)}},pt.prototype._parseLegacyEntity=function(){var t=this._sectionStart+1,e=this._index-t;for(e>6&&(e=6);e>=2;){var n=this._buffer.substr(t,e);if(o.hasOwnProperty(n))return this._emitPartial(o[n]),void(this._sectionStart+=e+1);e--}},pt.prototype._stateInNamedEntity=function(t){";"===t?(this._parseNamedEntityStrict(),this._sectionStart+1<this._index&&!this._xmlMode&&this._parseLegacyEntity(),this._state=this._baseState):(t<"a"||t>"z")&&(t<"A"||t>"Z")&&(t<"0"||t>"9")&&(this._xmlMode||this._sectionStart+1===this._index||(this._baseState!==A?"="!==t&&this._parseNamedEntityStrict():this._parseLegacyEntity()),this._state=this._baseState,this._index--)},pt.prototype._decodeNumericEntity=function(t,e){var n=this._sectionStart+t;if(n!==this._index){var i=this._buffer.substring(n,this._index),o=parseInt(i,e);this._emitPartial(r(o)),this._sectionStart=this._index}else this._sectionStart--;this._state=this._baseState},pt.prototype._stateInNumericEntity=function(t){";"===t?(this._decodeNumericEntity(2,10),this._sectionStart++):(t<"0"||t>"9")&&(this._xmlMode?this._state=this._baseState:this._decodeNumericEntity(2,10),this._index--)},pt.prototype._stateInHexEntity=function(t){";"===t?(this._decodeNumericEntity(3,16),this._sectionStart++):(t<"a"||t>"f")&&(t<"A"||t>"F")&&(t<"0"||t>"9")&&(this._xmlMode?this._state=this._baseState:this._decodeNumericEntity(3,16),this._index--)},pt.prototype._cleanup=function(){this._sectionStart<0?(this._buffer="",this._bufferOffset+=this._index,this._index=0):this._running&&(this._state===A?(this._sectionStart!==this._index&&this._cbs.ontext(this._buffer.substr(this._sectionStart)),this._buffer="",this._bufferOffset+=this._index,this._index=0):this._sectionStart===this._index?(this._buffer="",this._bufferOffset+=this._index,this._index=0):(this._buffer=this._buffer.substr(this._sectionStart),this._index-=this._sectionStart,this._bufferOffset+=this._sectionStart),this._sectionStart=0)},pt.prototype.write=function(t){this._ended&&this._cbs.onerror(Error(".write() after done!")),this._buffer+=t,this._parse()},pt.prototype._parse=function(){for(;this._index<this._buffer.length&&this._running;){var t=this._buffer.charAt(this._index);this._state===A?this._stateText(t):this._state===l?this._stateBeforeTagName(t):this._state===c?this._stateInTagName(t):this._state===g?this._stateBeforeCloseingTagName(t):this._state===h?this._stateInCloseingTagName(t):this._state===d?this._stateAfterCloseingTagName(t):this._state===u?this._stateInSelfClosingTag(t):this._state===f?this._stateBeforeAttributeName(t):this._state===p?this._stateInAttributeName(t):this._state===I?this._stateAfterAttributeName(t):this._state===C?this._stateBeforeAttributeValue(t):this._state===m?this._stateInAttributeValueDoubleQuotes(t):this._state===y?this._stateInAttributeValueSingleQuotes(t):this._state===E?this._stateInAttributeValueNoQuotes(t):this._state===v?this._stateBeforeDeclaration(t):this._state===B?this._stateInDeclaration(t):this._state===b?this._stateInProcessingInstruction(t):this._state===Q?this._stateBeforeComment(t):this._state===w?this._stateInComment(t):this._state===x?this._stateAfterComment1(t):this._state===_?this._stateAfterComment2(t):this._state===D?this._stateBeforeCdata1(t):this._state===M?this._stateBeforeCdata2(t):this._state===k?this._stateBeforeCdata3(t):this._state===S?this._stateBeforeCdata4(t):this._state===L?this._stateBeforeCdata5(t):this._state===R?this._stateBeforeCdata6(t):this._state===N?this._stateInCdata(t):this._state===F?this._stateAfterCdata1(t):this._state===T?this._stateAfterCdata2(t):this._state===U?this._stateBeforeSpecial(t):this._state===O?this._stateBeforeSpecialEnd(t):this._state===G?this._stateBeforeScript1(t):this._state===H?this._stateBeforeScript2(t):this._state===j?this._stateBeforeScript3(t):this._state===Y?this._stateBeforeScript4(t):this._state===P?this._stateBeforeScript5(t):this._state===K?this._stateAfterScript1(t):this._state===z?this._stateAfterScript2(t):this._state===J?this._stateAfterScript3(t):this._state===q?this._stateAfterScript4(t):this._state===V?this._stateAfterScript5(t):this._state===W?this._stateBeforeStyle1(t):this._state===X?this._stateBeforeStyle2(t):this._state===Z?this._stateBeforeStyle3(t):this._state===$?this._stateBeforeStyle4(t):this._state===tt?this._stateAfterStyle1(t):this._state===et?this._stateAfterStyle2(t):this._state===nt?this._stateAfterStyle3(t):this._state===rt?this._stateAfterStyle4(t):this._state===it?this._stateBeforeEntity(t):this._state===ot?this._stateBeforeNumericEntity(t):this._state===at?this._stateInNamedEntity(t):this._state===st?this._stateInNumericEntity(t):this._state===At?this._stateInHexEntity(t):this._cbs.onerror(Error("unknown _state"),this._state),this._index++}this._cleanup()},pt.prototype.pause=function(){this._running=!1},pt.prototype.resume=function(){this._running=!0,this._index<this._buffer.length&&this._parse(),this._ended&&this._finish()},pt.prototype.end=function(t){this._ended&&this._cbs.onerror(Error(".end() after done!")),t&&this.write(t),this._ended=!0,this._running&&this._finish()},pt.prototype._finish=function(){this._sectionStart<this._index&&this._handleTrailingData(),this._cbs.onend()},pt.prototype._handleTrailingData=function(){var t=this._buffer.substr(this._sectionStart);this._state===N||this._state===F||this._state===T?this._cbs.oncdata(t):this._state===w||this._state===x||this._state===_?this._cbs.oncomment(t):this._state!==at||this._xmlMode?this._state!==st||this._xmlMode?this._state!==At||this._xmlMode?this._state!==c&&this._state!==f&&this._state!==C&&this._state!==I&&this._state!==p&&this._state!==y&&this._state!==m&&this._state!==E&&this._state!==h&&this._cbs.ontext(t):(this._decodeNumericEntity(3,16),this._sectionStart<this._index&&(this._state=this._baseState,this._handleTrailingData())):(this._decodeNumericEntity(2,10),this._sectionStart<this._index&&(this._state=this._baseState,this._handleTrailingData())):(this._parseLegacyEntity(),this._sectionStart<this._index&&(this._state=this._baseState,this._handleTrailingData()))},pt.prototype.reset=function(){pt.call(this,{xmlMode:this._xmlMode,decodeEntities:this._decodeEntities},this._cbs)},pt.prototype.getAbsoluteIndex=function(){return this._bufferOffset+this._index},pt.prototype._getSection=function(){return this._buffer.substring(this._sectionStart,this._index)},pt.prototype._emitToken=function(t){this._cbs[t](this._getSection()),this._sectionStart=-1},pt.prototype._emitPartial=function(t){this._baseState!==A?this._cbs.onattribdata(t):this._cbs.ontext(t)}},function(t,e,n){var r=n(1085);t.exports=function(t){if(t>=55296&&t<=57343||t>1114111)return"�";t in r&&(t=r[t]);var e="";t>65535&&(t-=65536,e+=String.fromCharCode(t>>>10&1023|55296),t=56320|1023&t);return e+=String.fromCharCode(t)}},function(t){t.exports=JSON.parse('{"Aacute":"Á","aacute":"á","Acirc":"Â","acirc":"â","acute":"´","AElig":"Æ","aelig":"æ","Agrave":"À","agrave":"à","amp":"&","AMP":"&","Aring":"Å","aring":"å","Atilde":"Ã","atilde":"ã","Auml":"Ä","auml":"ä","brvbar":"¦","Ccedil":"Ç","ccedil":"ç","cedil":"¸","cent":"¢","copy":"©","COPY":"©","curren":"¤","deg":"°","divide":"÷","Eacute":"É","eacute":"é","Ecirc":"Ê","ecirc":"ê","Egrave":"È","egrave":"è","ETH":"Ð","eth":"ð","Euml":"Ë","euml":"ë","frac12":"½","frac14":"¼","frac34":"¾","gt":">","GT":">","Iacute":"Í","iacute":"í","Icirc":"Î","icirc":"î","iexcl":"¡","Igrave":"Ì","igrave":"ì","iquest":"¿","Iuml":"Ï","iuml":"ï","laquo":"«","lt":"<","LT":"<","macr":"¯","micro":"µ","middot":"·","nbsp":" ","not":"¬","Ntilde":"Ñ","ntilde":"ñ","Oacute":"Ó","oacute":"ó","Ocirc":"Ô","ocirc":"ô","Ograve":"Ò","ograve":"ò","ordf":"ª","ordm":"º","Oslash":"Ø","oslash":"ø","Otilde":"Õ","otilde":"õ","Ouml":"Ö","ouml":"ö","para":"¶","plusmn":"±","pound":"£","quot":"\\"","QUOT":"\\"","raquo":"»","reg":"®","REG":"®","sect":"§","shy":"­","sup1":"¹","sup2":"²","sup3":"³","szlig":"ß","THORN":"Þ","thorn":"þ","times":"×","Uacute":"Ú","uacute":"ú","Ucirc":"Û","ucirc":"û","Ugrave":"Ù","ugrave":"ù","uml":"¨","Uuml":"Ü","uuml":"ü","Yacute":"Ý","yacute":"ý","yen":"¥","yuml":"ÿ"}')},function(t,e,n){var r=n(138),i=/\s+/g,o=n(598),a=n(1086);function s(t,e,n){"object"==typeof t?(n=e,e=t,t=null):"function"==typeof e&&(n=e,e=A),this._callback=t,this._options=e||A,this._elementCB=n,this.dom=[],this._done=!1,this._tagStack=[],this._parser=this._parser||null}var A={normalizeWhitespace:!1,withStartIndices:!1,withEndIndices:!1};s.prototype.onparserinit=function(t){this._parser=t},s.prototype.onreset=function(){s.call(this,this._callback,this._options,this._elementCB)},s.prototype.onend=function(){this._done||(this._done=!0,this._parser=null,this._handleCallback(null))},s.prototype._handleCallback=s.prototype.onerror=function(t){if("function"==typeof this._callback)this._callback(t,this.dom);else if(t)throw t},s.prototype.onclosetag=function(){var t=this._tagStack.pop();this._options.withEndIndices&&t&&(t.endIndex=this._parser.endIndex),this._elementCB&&this._elementCB(t)},s.prototype._createDomElement=function(t){if(!this._options.withDomLvl1)return t;var e;for(var n in e="tag"===t.type?Object.create(a):Object.create(o),t)t.hasOwnProperty(n)&&(e[n]=t[n]);return e},s.prototype._addDomElement=function(t){var e=this._tagStack[this._tagStack.length-1],n=e?e.children:this.dom,r=n[n.length-1];t.next=null,this._options.withStartIndices&&(t.startIndex=this._parser.startIndex),this._options.withEndIndices&&(t.endIndex=this._parser.endIndex),r?(t.prev=r,r.next=t):t.prev=null,n.push(t),t.parent=e||null},s.prototype.onopentag=function(t,e){var n={type:"script"===t?r.Script:"style"===t?r.Style:r.Tag,name:t,attribs:e,children:[]},i=this._createDomElement(n);this._addDomElement(i),this._tagStack.push(i)},s.prototype.ontext=function(t){var e,n=this._options.normalizeWhitespace||this._options.ignoreWhitespace;if(!this._tagStack.length&&this.dom.length&&(e=this.dom[this.dom.length-1]).type===r.Text)n?e.data=(e.data+t).replace(i," "):e.data+=t;else if(this._tagStack.length&&(e=this._tagStack[this._tagStack.length-1])&&(e=e.children[e.children.length-1])&&e.type===r.Text)n?e.data=(e.data+t).replace(i," "):e.data+=t;else{n&&(t=t.replace(i," "));var o=this._createDomElement({data:t,type:r.Text});this._addDomElement(o)}},s.prototype.oncomment=function(t){var e=this._tagStack[this._tagStack.length-1];if(e&&e.type===r.Comment)e.data+=t;else{var n={data:t,type:r.Comment},i=this._createDomElement(n);this._addDomElement(i),this._tagStack.push(i)}},s.prototype.oncdatastart=function(){var t={children:[{data:"",type:r.Text}],type:r.CDATA},e=this._createDomElement(t);this._addDomElement(e),this._tagStack.push(e)},s.prototype.oncommentend=s.prototype.oncdataend=function(){this._tagStack.pop()},s.prototype.onprocessinginstruction=function(t,e){var n=this._createDomElement({name:t,data:e,type:r.Directive});this._addDomElement(n)},t.exports=s},function(t,e){var n=t.exports={get firstChild(){var t=this.children;return t&&t[0]||null},get lastChild(){var t=this.children;return t&&t[t.length-1]||null},get nodeType(){return i[this.type]||i.element}},r={tagName:"name",childNodes:"children",parentNode:"parent",previousSibling:"prev",nextSibling:"next",nodeValue:"data"},i={element:1,text:3,cdata:4,comment:8};Object.keys(r).forEach(function(t){var e=r[t];Object.defineProperty(n,t,{get:function(){return this[e]||null},set:function(t){return this[e]=t,t}})})},function(t,e,n){var r=t.exports;[n(1088),n(1093),n(1094),n(1095),n(1096),n(1097)].forEach(function(t){Object.keys(t).forEach(function(e){r[e]=t[e].bind(r)})})},function(t,e,n){t.exports=s;var r=n(593),i=n(1099).Writable,o=n(210).StringDecoder,a=n(42).Buffer;function s(t,e){var n=this._parser=new r(t,e),a=this._decoder=new o;i.call(this,{decodeStrings:!1}),this.once("finish",function(){n.end(a.end())})}n(30)(s,i),s.prototype._write=function(t,e,n){t instanceof a&&(t=this._decoder.write(t)),this._parser.write(t),n()}},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=a(n(1112)),i=a(n(1113)),o=a(n(1114));function a(t){return t&&t.__esModule?t:{default:t}}function s(t){var e="function"==typeof Map?new Map:void 0;return(s=function(t){if(null===t||(n=t,-1===Function.toString.call(n).indexOf("[native code]")))return t;var n;if("function"!=typeof t)throw new TypeError("Super expression must either be null or a function");if(void 0!==e){if(e.has(t))return e.get(t);e.set(t,r)}function r(){return A(t,arguments,c(this).constructor)}return r.prototype=Object.create(t.prototype,{constructor:{value:r,enumerable:!1,writable:!0,configurable:!0}}),l(r,t)})(t)}function A(t,e,n){return(A=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],function(){})),!0}catch(t){return!1}}()?Reflect.construct:function(t,e,n){var r=[null];r.push.apply(r,e);var i=new(Function.bind.apply(t,r));return n&&l(i,n.prototype),i}).apply(null,arguments)}function l(t,e){return(l=Object.setPrototypeOf||function(t,e){return t.__proto__=e,t})(t,e)}function c(t){return(c=Object.setPrototypeOf?Object.getPrototypeOf:function(t){return t.__proto__||Object.getPrototypeOf(t)})(t)}var u=function(t){var e,n;function a(e,n,r,i,o,s){var A;return(A=t.call(this,e)||this).name="CssSyntaxError",A.reason=e,o&&(A.file=o),i&&(A.source=i),s&&(A.plugin=s),void 0!==n&&void 0!==r&&(A.line=n,A.column=r),A.setMessage(),Error.captureStackTrace&&Error.captureStackTrace(function(t){if(void 0===t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return t}(A),a),A}n=t,(e=a).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n;var s=a.prototype;return s.setMessage=function(){this.message=this.plugin?this.plugin+": ":"",this.message+=this.file?this.file:"<css input>",void 0!==this.line&&(this.message+=":"+this.line+":"+this.column),this.message+=": "+this.reason},s.showSourceCode=function(t){var e=this;if(!this.source)return"";var n=this.source;o.default&&(void 0===t&&(t=r.default.stdout),t&&(n=(0,o.default)(n)));var a=n.split(/\r?\n/),s=Math.max(this.line-3,0),A=Math.min(this.line+2,a.length),l=String(A).length;function c(e){return t&&i.default.red?i.default.red.bold(e):e}function u(e){return t&&i.default.gray?i.default.gray(e):e}return a.slice(s,A).map(function(t,n){var r=s+1+n,i=" "+(" "+r).slice(-l)+" | ";if(r===e.line){var o=u(i.replace(/\d/g," "))+t.slice(0,e.column-1).replace(/[^\t]/g," ");return c(">")+u(i)+t+"\n "+o+c("^")}return" "+u(i)+t}).join("\n")},s.toString=function(){var t=this.showSourceCode();return t&&(t="\n\n"+t+"\n"),this.name+": "+this.message+t},a}(s(Error));e.default=u,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r={colon:": ",indent:"    ",beforeDecl:"\n",beforeRule:"\n",beforeOpen:" ",beforeClose:"\n",beforeComment:"\n",after:"\n",emptyBody:"",commentLeft:" ",commentRight:" ",semicolon:!1};var i=function(){function t(t){this.builder=t}var e=t.prototype;return e.stringify=function(t,e){this[t.type](t,e)},e.root=function(t){this.body(t),t.raws.after&&this.builder(t.raws.after)},e.comment=function(t){var e=this.raw(t,"left","commentLeft"),n=this.raw(t,"right","commentRight");this.builder("/*"+e+t.text+n+"*/",t)},e.decl=function(t,e){var n=this.raw(t,"between","colon"),r=t.prop+n+this.rawValue(t,"value");t.important&&(r+=t.raws.important||" !important"),e&&(r+=";"),this.builder(r,t)},e.rule=function(t){this.block(t,this.rawValue(t,"selector")),t.raws.ownSemicolon&&this.builder(t.raws.ownSemicolon,t,"end")},e.atrule=function(t,e){var n="@"+t.name,r=t.params?this.rawValue(t,"params"):"";if(void 0!==t.raws.afterName?n+=t.raws.afterName:r&&(n+=" "),t.nodes)this.block(t,n+r);else{var i=(t.raws.between||"")+(e?";":"");this.builder(n+r+i,t)}},e.body=function(t){for(var e=t.nodes.length-1;e>0&&"comment"===t.nodes[e].type;)e-=1;for(var n=this.raw(t,"semicolon"),r=0;r<t.nodes.length;r++){var i=t.nodes[r],o=this.raw(i,"before");o&&this.builder(o),this.stringify(i,e!==r||n)}},e.block=function(t,e){var n,r=this.raw(t,"between","beforeOpen");this.builder(e+r+"{",t,"start"),t.nodes&&t.nodes.length?(this.body(t),n=this.raw(t,"after")):n=this.raw(t,"after","emptyBody"),n&&this.builder(n),this.builder("}",t,"end")},e.raw=function(t,e,n){var i;if(n||(n=e),e&&void 0!==(i=t.raws[e]))return i;var o=t.parent;if("before"===n&&(!o||"root"===o.type&&o.first===t))return"";if(!o)return r[n];var a=t.root();if(a.rawCache||(a.rawCache={}),void 0!==a.rawCache[n])return a.rawCache[n];if("before"===n||"after"===n)return this.beforeAfter(t,n);var s,A="raw"+((s=n)[0].toUpperCase()+s.slice(1));return this[A]?i=this[A](a,t):a.walk(function(t){if(void 0!==(i=t.raws[e]))return!1}),void 0===i&&(i=r[n]),a.rawCache[n]=i,i},e.rawSemicolon=function(t){var e;return t.walk(function(t){if(t.nodes&&t.nodes.length&&"decl"===t.last.type&&void 0!==(e=t.raws.semicolon))return!1}),e},e.rawEmptyBody=function(t){var e;return t.walk(function(t){if(t.nodes&&0===t.nodes.length&&void 0!==(e=t.raws.after))return!1}),e},e.rawIndent=function(t){return t.raws.indent?t.raws.indent:(t.walk(function(n){var r=n.parent;if(r&&r!==t&&r.parent&&r.parent===t&&void 0!==n.raws.before){var i=n.raws.before.split("\n");return e=(e=i[i.length-1]).replace(/[^\s]/g,""),!1}}),e);var e},e.rawBeforeComment=function(t,e){var n;return t.walkComments(function(t){if(void 0!==t.raws.before)return-1!==(n=t.raws.before).indexOf("\n")&&(n=n.replace(/[^\n]+$/,"")),!1}),void 0===n?n=this.raw(e,null,"beforeDecl"):n&&(n=n.replace(/[^\s]/g,"")),n},e.rawBeforeDecl=function(t,e){var n;return t.walkDecls(function(t){if(void 0!==t.raws.before)return-1!==(n=t.raws.before).indexOf("\n")&&(n=n.replace(/[^\n]+$/,"")),!1}),void 0===n?n=this.raw(e,null,"beforeRule"):n&&(n=n.replace(/[^\s]/g,"")),n},e.rawBeforeRule=function(t){var e;return t.walk(function(n){if(n.nodes&&(n.parent!==t||t.first!==n)&&void 0!==n.raws.before)return-1!==(e=n.raws.before).indexOf("\n")&&(e=e.replace(/[^\n]+$/,"")),!1}),e&&(e=e.replace(/[^\s]/g,"")),e},e.rawBeforeClose=function(t){var e;return t.walk(function(t){if(t.nodes&&t.nodes.length>0&&void 0!==t.raws.after)return-1!==(e=t.raws.after).indexOf("\n")&&(e=e.replace(/[^\n]+$/,"")),!1}),e&&(e=e.replace(/[^\s]/g,"")),e},e.rawBeforeOpen=function(t){var e;return t.walk(function(t){if("decl"!==t.type&&void 0!==(e=t.raws.between))return!1}),e},e.rawColon=function(t){var e;return t.walkDecls(function(t){if(void 0!==t.raws.between)return e=t.raws.between.replace(/[^\s:]/g,""),!1}),e},e.beforeAfter=function(t,e){var n;n="decl"===t.type?this.raw(t,null,"beforeDecl"):"comment"===t.type?this.raw(t,null,"beforeComment"):"before"===e?this.raw(t,null,"beforeRule"):this.raw(t,null,"beforeClose");for(var r=t.parent,i=0;r&&"root"!==r.type;)i+=1,r=r.parent;if(-1!==n.indexOf("\n")){var o=this.raw(t,null,"indent");if(o.length)for(var a=0;a<i;a++)n+=o}return n},e.rawValue=function(t,e){var n=t[e],r=t.raws[e];return r&&r.value===n?r.raw:n},t}();e.default=i,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r,i=(r=n(604))&&r.__esModule?r:{default:r};var o=function(){function t(t){void 0===t&&(t=[]),this.version="7.0.17",this.plugins=this.normalize(t)}var e=t.prototype;return e.use=function(t){return this.plugins=this.plugins.concat(this.normalize([t])),this},e.process=function(t){function e(e){return t.apply(this,arguments)}return e.toString=function(){return t.toString()},e}(function(t,e){return void 0===e&&(e={}),0===this.plugins.length&&(e.parser,e.stringifier),new i.default(this,t,e)}),e.normalize=function(t){var e=[],n=t,r=Array.isArray(n),i=0;for(n=r?n:n[Symbol.iterator]();;){var o;if(r){if(i>=n.length)break;o=n[i++]}else{if((i=n.next()).done)break;o=i.value}var a=o;if(a.postcss&&(a=a.postcss),"object"==typeof a&&Array.isArray(a.plugins))e=e.concat(a.plugins);else if("function"==typeof a)e.push(a);else{if("object"!=typeof a||!a.parse&&!a.stringify)throw new Error(a+" is not a PostCSS plugin")}}return e},t}();e.default=o,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=s(n(1115)),i=s(n(301)),o=(s(n(1122)),s(n(1123))),a=s(n(302));function s(t){return t&&t.__esModule?t:{default:t}}function A(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}function l(t){return"object"==typeof t&&"function"==typeof t.then}var c=function(){function t(e,n,r){var i;if(this.stringified=!1,this.processed=!1,"object"==typeof n&&null!==n&&"root"===n.type)i=n;else if(n instanceof t||n instanceof o.default)i=n.root,n.map&&(void 0===r.map&&(r.map={}),r.map.inline||(r.map.inline=!1),r.map.prev=n.map);else{var s=a.default;r.syntax&&(s=r.syntax.parse),r.parser&&(s=r.parser),s.parse&&(s=s.parse);try{i=s(n,r)}catch(t){this.error=t}}this.result=new o.default(e,i,r)}var e,n,s,c=t.prototype;return c.warnings=function(){return this.sync().warnings()},c.toString=function(){return this.css},c.then=function(t,e){return this.async().then(t,e)},c.catch=function(t){return this.async().catch(t)},c.finally=function(t){return this.async().then(t,t)},c.handleError=function(t,e){try{if(this.error=t,"CssSyntaxError"!==t.name||t.plugin){if(e.postcssVersion);}else t.plugin=e.postcssPlugin,t.setMessage()}catch(t){console&&console.error&&console.error(t)}},c.asyncTick=function(t,e){var n=this;if(this.plugin>=this.processor.plugins.length)return this.processed=!0,t();try{var r=this.processor.plugins[this.plugin],i=this.run(r);this.plugin+=1,l(i)?i.then(function(){n.asyncTick(t,e)}).catch(function(t){n.handleError(t,r),n.processed=!0,e(t)}):this.asyncTick(t,e)}catch(t){this.processed=!0,e(t)}},c.async=function(){var t=this;return this.processed?new Promise(function(e,n){t.error?n(t.error):e(t.stringify())}):this.processing?this.processing:(this.processing=new Promise(function(e,n){if(t.error)return n(t.error);t.plugin=0,t.asyncTick(e,n)}).then(function(){return t.processed=!0,t.stringify()}),this.processing)},c.sync=function(){if(this.processed)return this.result;if(this.processed=!0,this.processing)throw new Error("Use process(css).then(cb) to work with async plugins");if(this.error)throw this.error;var t=this.result.processor.plugins,e=Array.isArray(t),n=0;for(t=e?t:t[Symbol.iterator]();;){var r;if(e){if(n>=t.length)break;r=t[n++]}else{if((n=t.next()).done)break;r=n.value}var i=r;if(l(this.run(i)))throw new Error("Use process(css).then(cb) to work with async plugins")}return this.result},c.run=function(t){this.result.lastPlugin=t;try{return t(this.result.root,this.result)}catch(e){throw this.handleError(e,t),e}},c.stringify=function(){if(this.stringified)return this.result;this.stringified=!0,this.sync();var t=this.result.opts,e=i.default;t.syntax&&(e=t.syntax.stringify),t.stringifier&&(e=t.stringifier),e.stringify&&(e=e.stringify);var n=new r.default(e,this.result.root,this.result.opts).generate();return this.result.css=n[0],this.result.map=n[1],this.result},e=t,(n=[{key:"processor",get:function(){return this.result.processor}},{key:"opts",get:function(){return this.result.opts}},{key:"css",get:function(){return this.stringify().css}},{key:"content",get:function(){return this.stringify().content}},{key:"map",get:function(){return this.stringify().map}},{key:"root",get:function(){return this.sync().root}},{key:"messages",get:function(){return this.sync().messages}}])&&A(e.prototype,n),s&&A(e,s),t}();e.default=c,t.exports=e.default},function(t,e,n){e.SourceMapGenerator=n(606).SourceMapGenerator,e.SourceMapConsumer=n(1118).SourceMapConsumer,e.SourceNode=n(1121).SourceNode},function(t,e,n){var r=n(607),i=n(176),o=n(608).ArraySet,a=n(1117).MappingList;function s(t){t||(t={}),this._file=i.getArg(t,"file",null),this._sourceRoot=i.getArg(t,"sourceRoot",null),this._skipValidation=i.getArg(t,"skipValidation",!1),this._sources=new o,this._names=new o,this._mappings=new a,this._sourcesContents=null}s.prototype._version=3,s.fromSourceMap=function(t){var e=t.sourceRoot,n=new s({file:t.file,sourceRoot:e});return t.eachMapping(function(t){var r={generated:{line:t.generatedLine,column:t.generatedColumn}};null!=t.source&&(r.source=t.source,null!=e&&(r.source=i.relative(e,r.source)),r.original={line:t.originalLine,column:t.originalColumn},null!=t.name&&(r.name=t.name)),n.addMapping(r)}),t.sources.forEach(function(r){var o=r;null!==e&&(o=i.relative(e,r)),n._sources.has(o)||n._sources.add(o);var a=t.sourceContentFor(r);null!=a&&n.setSourceContent(r,a)}),n},s.prototype.addMapping=function(t){var e=i.getArg(t,"generated"),n=i.getArg(t,"original",null),r=i.getArg(t,"source",null),o=i.getArg(t,"name",null);this._skipValidation||this._validateMapping(e,n,r,o),null!=r&&(r=String(r),this._sources.has(r)||this._sources.add(r)),null!=o&&(o=String(o),this._names.has(o)||this._names.add(o)),this._mappings.add({generatedLine:e.line,generatedColumn:e.column,originalLine:null!=n&&n.line,originalColumn:null!=n&&n.column,source:r,name:o})},s.prototype.setSourceContent=function(t,e){var n=t;null!=this._sourceRoot&&(n=i.relative(this._sourceRoot,n)),null!=e?(this._sourcesContents||(this._sourcesContents=Object.create(null)),this._sourcesContents[i.toSetString(n)]=e):this._sourcesContents&&(delete this._sourcesContents[i.toSetString(n)],0===Object.keys(this._sourcesContents).length&&(this._sourcesContents=null))},s.prototype.applySourceMap=function(t,e,n){var r=e;if(null==e){if(null==t.file)throw new Error('SourceMapGenerator.prototype.applySourceMap requires either an explicit source file, or the source map\'s "file" property. Both were omitted.');r=t.file}var a=this._sourceRoot;null!=a&&(r=i.relative(a,r));var s=new o,A=new o;this._mappings.unsortedForEach(function(e){if(e.source===r&&null!=e.originalLine){var o=t.originalPositionFor({line:e.originalLine,column:e.originalColumn});null!=o.source&&(e.source=o.source,null!=n&&(e.source=i.join(n,e.source)),null!=a&&(e.source=i.relative(a,e.source)),e.originalLine=o.line,e.originalColumn=o.column,null!=o.name&&(e.name=o.name))}var l=e.source;null==l||s.has(l)||s.add(l);var c=e.name;null==c||A.has(c)||A.add(c)},this),this._sources=s,this._names=A,t.sources.forEach(function(e){var r=t.sourceContentFor(e);null!=r&&(null!=n&&(e=i.join(n,e)),null!=a&&(e=i.relative(a,e)),this.setSourceContent(e,r))},this)},s.prototype._validateMapping=function(t,e,n,r){if(e&&"number"!=typeof e.line&&"number"!=typeof e.column)throw new Error("original.line and original.column are not numbers -- you probably meant to omit the original mapping entirely and only map the generated position. If so, pass null for the original mapping instead of an object with empty or null values.");if((!(t&&"line"in t&&"column"in t&&t.line>0&&t.column>=0)||e||n||r)&&!(t&&"line"in t&&"column"in t&&e&&"line"in e&&"column"in e&&t.line>0&&t.column>=0&&e.line>0&&e.column>=0&&n))throw new Error("Invalid mapping: "+JSON.stringify({generated:t,source:n,original:e,name:r}))},s.prototype._serializeMappings=function(){for(var t,e,n,o,a=0,s=1,A=0,l=0,c=0,u=0,g="",h=this._mappings.toArray(),d=0,f=h.length;d<f;d++){if(t="",(e=h[d]).generatedLine!==s)for(a=0;e.generatedLine!==s;)t+=";",s++;else if(d>0){if(!i.compareByGeneratedPositionsInflated(e,h[d-1]))continue;t+=","}t+=r.encode(e.generatedColumn-a),a=e.generatedColumn,null!=e.source&&(o=this._sources.indexOf(e.source),t+=r.encode(o-u),u=o,t+=r.encode(e.originalLine-1-l),l=e.originalLine-1,t+=r.encode(e.originalColumn-A),A=e.originalColumn,null!=e.name&&(n=this._names.indexOf(e.name),t+=r.encode(n-c),c=n)),g+=t}return g},s.prototype._generateSourcesContent=function(t,e){return t.map(function(t){if(!this._sourcesContents)return null;null!=e&&(t=i.relative(e,t));var n=i.toSetString(t);return Object.prototype.hasOwnProperty.call(this._sourcesContents,n)?this._sourcesContents[n]:null},this)},s.prototype.toJSON=function(){var t={version:this._version,sources:this._sources.toArray(),names:this._names.toArray(),mappings:this._serializeMappings()};return null!=this._file&&(t.file=this._file),null!=this._sourceRoot&&(t.sourceRoot=this._sourceRoot),this._sourcesContents&&(t.sourcesContent=this._generateSourcesContent(t.sources,t.sourceRoot)),t},s.prototype.toString=function(){return JSON.stringify(this.toJSON())},e.SourceMapGenerator=s},function(t,e,n){var r=n(1116);e.encode=function(t){var e,n="",i=function(t){return t<0?1+(-t<<1):0+(t<<1)}(t);do{e=31&i,(i>>>=5)>0&&(e|=32),n+=r.encode(e)}while(i>0);return n},e.decode=function(t,e,n){var i,o,a,s,A=t.length,l=0,c=0;do{if(e>=A)throw new Error("Expected more digits in base 64 VLQ value.");if(-1===(o=r.decode(t.charCodeAt(e++))))throw new Error("Invalid base64 digit: "+t.charAt(e-1));i=!!(32&o),l+=(o&=31)<<c,c+=5}while(i);n.value=(s=(a=l)>>1,1==(1&a)?-s:s),n.rest=e}},function(t,e,n){var r=n(176),i=Object.prototype.hasOwnProperty,o="undefined"!=typeof Map;function a(){this._array=[],this._set=o?new Map:Object.create(null)}a.fromArray=function(t,e){for(var n=new a,r=0,i=t.length;r<i;r++)n.add(t[r],e);return n},a.prototype.size=function(){return o?this._set.size:Object.getOwnPropertyNames(this._set).length},a.prototype.add=function(t,e){var n=o?t:r.toSetString(t),a=o?this.has(t):i.call(this._set,n),s=this._array.length;a&&!e||this._array.push(t),a||(o?this._set.set(t,s):this._set[n]=s)},a.prototype.has=function(t){if(o)return this._set.has(t);var e=r.toSetString(t);return i.call(this._set,e)},a.prototype.indexOf=function(t){if(o){var e=this._set.get(t);if(e>=0)return e}else{var n=r.toSetString(t);if(i.call(this._set,n))return this._set[n]}throw new Error('"'+t+'" is not in the set.')},a.prototype.at=function(t){if(t>=0&&t<this._array.length)return this._array[t];throw new Error("No element indexed by "+t)},a.prototype.toArray=function(){return this._array.slice()},e.ArraySet=a},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r={split:function(t,e,n){for(var r=[],i="",o=!1,a=0,s=!1,A=!1,l=0;l<t.length;l++){var c=t[l];s?A?A=!1:"\\"===c?A=!0:c===s&&(s=!1):'"'===c||"'"===c?s=c:"("===c?a+=1:")"===c?a>0&&(a-=1):0===a&&-1!==e.indexOf(c)&&(o=!0),o?(""!==i&&r.push(i.trim()),i="",o=!1):i+=c}return(n||""!==i)&&r.push(i.trim()),r},space:function(t){return r.split(t,[" ","\n","\t"])},comma:function(t){return r.split(t,[","],!0)}},i=r;e.default=i,t.exports=e.default},function(t,e,n){"use strict";var r;e.__esModule=!0,e.default=void 0;var i=function(t){var e,r;function i(e){var n;return(n=t.call(this,e)||this).type="root",n.nodes||(n.nodes=[]),n}r=t,(e=i).prototype=Object.create(r.prototype),e.prototype.constructor=e,e.__proto__=r;var o=i.prototype;return o.removeChild=function(e,n){var r=this.index(e);return!n&&0===r&&this.nodes.length>1&&(this.nodes[1].raws.before=this.nodes[r].raws.before),t.prototype.removeChild.call(this,e)},o.normalize=function(e,n,r){var i=t.prototype.normalize.call(this,e);if(n)if("prepend"===r)this.nodes.length>1?n.raws.before=this.nodes[1].raws.before:delete n.raws.before;else if(this.first!==n){var o=i,a=Array.isArray(o),s=0;for(o=a?o:o[Symbol.iterator]();;){var A;if(a){if(s>=o.length)break;A=o[s++]}else{if((s=o.next()).done)break;A=s.value}A.raws.before=n.raws.before}}return i},o.toResult=function(t){return void 0===t&&(t={}),new(n(604))(new(n(603)),this,t).stringify()},i}(((r=n(305))&&r.__esModule?r:{default:r}).default);e.default=i,t.exports=e.default},function(t,e,n){"use strict";n.d(e,"b",function(){return i}),n.d(e,"a",function(){return o});var r=n(9);const i=new r.Token("@jupyterlab/rendermime:IRenderMimeRegistry"),o=new r.Token("@jupyterlab/rendermime:ILatexTypesetter")},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r,i,o=n(9);!function(t){function e(t,e="copy"){let n=window.getSelection(),r=[];for(let t=0,e=n.rangeCount;t<e;++t)r[t]=n.getRangeAt(t).cloneRange();let i=document.createRange();i.selectNodeContents(t),n.removeAllRanges(),n.addRange(i),document.execCommand(e),(n=window.getSelection()).removeAllRanges();for(let t=0,e=r.length;t<e;++t)n.addRange(r[t])}t.getInstance=function(){return i.instance},t.setInstance=function(t){i.instance=t},t.copyToSystem=function(t){let n=document.body,r=e=>{(e.clipboardData||window.clipboardData).setData("text",t),e.preventDefault(),n.removeEventListener("copy",r)};n.addEventListener("copy",r),e(n)},t.generateEvent=e}(r||(r={})),function(t){t.instance=new o.MimeData}(i||(i={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return o});var r=n(10),i=n(6);class o extends i.Widget{constructor(t){super(t),this._collapseChanged=new r.Signal(this);const{widget:e,collapsed:n=!0}=t;this.addClass("jp-Collapse"),this._header=new i.Widget,this._header.addClass("jp-Collapse-header"),this._content=new i.Panel,this._content.addClass("jp-Collapse-contents");let o=new i.PanelLayout;this.layout=o,o.addWidget(this._header),o.addWidget(this._content),this.widget=e,this.collapsed=n}get widget(){return this._widget}set widget(t){let e=this._widget;e&&(e.title.changed.disconnect(this._onTitleChanged,this),e.parent=null),this._widget=t,t.title.changed.connect(this._onTitleChanged,this),this._onTitleChanged(t.title),this._content.addWidget(t)}get collapsed(){return this._collapsed}set collapsed(t){t!==this._collapsed&&(t?this._collapse():this._uncollapse())}get collapseChanged(){return this._collapseChanged}toggle(){this.collapsed=!this.collapsed}dispose(){this.isDisposed||(delete this._header,delete this._widget,delete this._content,super.dispose())}handleEvent(t){switch(t.type){case"click":this._evtClick(t)}}onAfterAttach(t){this._header.node.addEventListener("click",this)}onBeforeDetach(t){this._header.node.removeEventListener("click",this)}_collapse(){this._collapsed=!0,this._content&&this._content.hide(),this.removeClass("jp-Collapse-open"),this._collapseChanged.emit(void 0)}_uncollapse(){this._collapsed=!1,this._content&&this._content.show(),this.addClass("jp-Collapse-open"),this._collapseChanged.emit(void 0)}_evtClick(t){this.toggle()}_onTitleChanged(t){this._header.node.textContent=this._widget.title.label}}},function(t,e,n){"use strict";n.d(e,"a",function(){return l});var r=n(25),i=n(6),o=n(244),a=n(245),s=n(194),A=n(195);class l extends i.Widget{constructor(t){super(t),this._changeGuard=!1,this._spinner=new o.a,this._isRevealed=!1,this.addClass("jp-MainAreaWidget"),this.id=s.a.createDomID();const e=this._content=t.content,n=this._toolbar=t.toolbar||new a.c,r=this._spinner,A=this.layout=new i.BoxLayout({spacing:0});A.direction="top-to-bottom",i.BoxLayout.setStretch(n,0),i.BoxLayout.setStretch(e,1),A.addWidget(n),A.addWidget(e),e.id||(e.id=s.a.createDomID()),e.node.tabIndex=-1,this._updateTitle(),e.title.changed.connect(this._updateTitle,this),this.title.closable=!0,this.title.changed.connect(this._updateContentTitle,this),t.reveal?(this.node.appendChild(r.node),this._revealed=t.reveal.then(()=>{if(e.isDisposed)return void this.dispose();e.disposed.connect(()=>this.dispose());const t=document.activeElement===r.node;this.node.removeChild(r.node),r.dispose(),this._isRevealed=!0,t&&this._focusContent()}).catch(t=>{const o=new i.Widget,a=document.createElement("pre");throw a.textContent=String(t),o.node.appendChild(a),i.BoxLayout.setStretch(o,1),this.node.removeChild(r.node),r.dispose(),e.dispose(),this._content=null,n.dispose(),this._toolbar=null,A.addWidget(o),this._isRevealed=!0,o})):(r.dispose(),e.disposed.connect(()=>this.dispose()),this._isRevealed=!0,this._revealed=Promise.resolve(void 0))}[A.a.symbol](){return A.a.getPrintFunction(this._content)}get content(){return this._content}get toolbar(){return this._toolbar}get isRevealed(){return this._isRevealed}get revealed(){return this._revealed}onActivateRequest(t){this._isRevealed?this._content&&this._focusContent():this._spinner.node.focus()}onCloseRequest(t){this.dispose()}onUpdateRequest(t){r.MessageLoop.sendMessage(this._content,t)}_updateTitle(){if(this._changeGuard)return;this._changeGuard=!0;const t=this.content;this.title.label=t.title.label,this.title.mnemonic=t.title.mnemonic,this.title.iconClass=t.title.iconClass,this.title.iconLabel=t.title.iconLabel,this.title.caption=t.title.caption,this.title.className=t.title.className,this.title.dataset=t.title.dataset,this._changeGuard=!1}_updateContentTitle(){if(this._changeGuard)return;this._changeGuard=!0;const t=this.content;t.title.label=this.title.label,t.title.mnemonic=this.title.mnemonic,t.title.iconClass=this.title.iconClass,t.title.iconLabel=this.title.iconLabel,t.title.caption=this.title.caption,t.title.className=this.title.className,t.title.dataset=this.title.dataset,this._changeGuard=!1}_focusContent(){this.content.node.contains(document.activeElement)||this.content.node.focus(),this.content.activate()}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.Alignment={CENTER:"center",LEFT:"left",RIGHT:"right"}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.Elevation={ZERO:0,ONE:1,TWO:2,THREE:3,FOUR:4}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.Position={BOTTOM:"bottom",BOTTOM_LEFT:"bottom-left",BOTTOM_RIGHT:"bottom-right",LEFT:"left",LEFT_BOTTOM:"left-bottom",LEFT_TOP:"left-top",RIGHT:"right",RIGHT_BOTTOM:"right-bottom",RIGHT_TOP:"right-top",TOP:"top",TOP_LEFT:"top-left",TOP_RIGHT:"top-right"},e.isPositionHorizontal=function(t){return t===e.Position.TOP||t===e.Position.TOP_LEFT||t===e.Position.TOP_RIGHT||t===e.Position.BOTTOM||t===e.Position.BOTTOM_LEFT||t===e.Position.BOTTOM_RIGHT},e.isPositionVertical=function(t){return t===e.Position.LEFT||t===e.Position.LEFT_TOP||t===e.Position.LEFT_BOTTOM||t===e.Position.RIGHT||t===e.Position.RIGHT_TOP||t===e.Position.RIGHT_BOTTOM},e.getPositionIgnoreAngles=function(t){return t===e.Position.TOP||t===e.Position.TOP_LEFT||t===e.Position.TOP_RIGHT?e.Position.TOP:t===e.Position.BOTTOM||t===e.Position.BOTTOM_LEFT||t===e.Position.BOTTOM_RIGHT?e.Position.BOTTOM:t===e.Position.LEFT||t===e.Position.LEFT_TOP||t===e.Position.LEFT_BOTTOM?e.Position.LEFT:e.Position.RIGHT}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.BACKSPACE=8,e.TAB=9,e.ENTER=13,e.SHIFT=16,e.ESCAPE=27,e.SPACE=32,e.ARROW_LEFT=37,e.ARROW_UP=38,e.ARROW_RIGHT=39,e.ARROW_DOWN=40,e.DELETE=46,e.isKeyboardClick=function(t){return t===e.ENTER||t===e.SPACE}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r="[Blueprint]";e.CLAMP_MIN_MAX=r+" clamp: max cannot be less than min",e.ALERT_WARN_CANCEL_PROPS=r+" <Alert> cancelButtonText and onCancel should be set together.",e.ALERT_WARN_CANCEL_ESCAPE_KEY=r+" <Alert> canEscapeKeyCancel enabled without onCancel or onClose handler.",e.ALERT_WARN_CANCEL_OUTSIDE_CLICK=r+" <Alert> canOutsideClickCancel enbaled without onCancel or onClose handler.",e.COLLAPSIBLE_LIST_INVALID_CHILD=r+" <CollapsibleList> children must be <MenuItem>s",e.CONTEXTMENU_WARN_DECORATOR_NO_METHOD=r+" @ContextMenuTarget-decorated class should implement renderContextMenu.",e.CONTEXTMENU_WARN_DECORATOR_NEEDS_REACT_ELEMENT=r+' "@ContextMenuTarget-decorated components must return a single JSX.Element or an empty render.',e.HOTKEYS_HOTKEY_CHILDREN=r+" <Hotkeys> only accepts <Hotkey> children.",e.HOTKEYS_WARN_DECORATOR_NO_METHOD=r+" @HotkeysTarget-decorated class should implement renderHotkeys.",e.HOTKEYS_WARN_DECORATOR_NEEDS_REACT_ELEMENT=r+' "@HotkeysTarget-decorated components must return a single JSX.Element or an empty render.',e.NUMERIC_INPUT_MIN_MAX=r+" <NumericInput> requires min to be no greater than max if both are defined.",e.NUMERIC_INPUT_MINOR_STEP_SIZE_BOUND=r+" <NumericInput> requires minorStepSize to be no greater than stepSize.",e.NUMERIC_INPUT_MAJOR_STEP_SIZE_BOUND=r+" <NumericInput> requires stepSize to be no greater than majorStepSize.",e.NUMERIC_INPUT_MINOR_STEP_SIZE_NON_POSITIVE=r+" <NumericInput> requires minorStepSize to be strictly greater than zero.",e.NUMERIC_INPUT_MAJOR_STEP_SIZE_NON_POSITIVE=r+" <NumericInput> requires majorStepSize to be strictly greater than zero.",e.NUMERIC_INPUT_STEP_SIZE_NON_POSITIVE=r+" <NumericInput> requires stepSize to be strictly greater than zero.",e.NUMERIC_INPUT_STEP_SIZE_NULL=r+" <NumericInput> requires stepSize to be defined.",e.PANEL_STACK_INITIAL_PANEL_STACK_MUTEX=r+" <PanelStack> requires exactly one of initialPanel and stack prop",e.PANEL_STACK_REQUIRES_PANEL=r+" <PanelStack> requires at least one panel in the stack",e.OVERFLOW_LIST_OBSERVE_PARENTS_CHANGED=r+" <OverflowList> does not support changing observeParents after mounting.",e.POPOVER_REQUIRES_TARGET=r+" <Popover> requires target prop or at least one child element.",e.POPOVER_HAS_BACKDROP_INTERACTION=r+" <Popover hasBackdrop={true}> requires interactionKind={PopoverInteractionKind.CLICK}.",e.POPOVER_WARN_TOO_MANY_CHILDREN=r+" <Popover> supports one or two children; additional children are ignored. First child is the target, second child is the content. You may instead supply these two as props.",e.POPOVER_WARN_DOUBLE_CONTENT=r+" <Popover> with two children ignores content prop; use either prop or children.",e.POPOVER_WARN_DOUBLE_TARGET=r+" <Popover> with children ignores target prop; use either prop or children.",e.POPOVER_WARN_EMPTY_CONTENT=r+" Disabling <Popover> with empty/whitespace content...",e.POPOVER_WARN_HAS_BACKDROP_INLINE=r+" <Popover usePortal={false}> ignores hasBackdrop",e.POPOVER_WARN_UNCONTROLLED_ONINTERACTION=r+" <Popover> onInteraction is ignored when uncontrolled.",e.PORTAL_CONTEXT_CLASS_NAME_STRING=r+" <Portal> context blueprintPortalClassName must be string",e.RADIOGROUP_WARN_CHILDREN_OPTIONS_MUTEX=r+" <RadioGroup> children and options prop are mutually exclusive, with options taking priority.",e.SLIDER_ZERO_STEP=r+" <Slider> stepSize must be greater than zero.",e.SLIDER_ZERO_LABEL_STEP=r+" <Slider> labelStepSize must be greater than zero.",e.RANGESLIDER_NULL_VALUE=r+" <RangeSlider> value prop must be an array of two non-null numbers.",e.MULTISLIDER_INVALID_CHILD=r+" <MultiSlider> children must be <SliderHandle>s or <SliderTrackStop>s",e.SPINNER_WARN_CLASSES_SIZE=r+" <Spinner> Classes.SMALL/LARGE are ignored if size prop is set.",e.TOASTER_CREATE_NULL=r+" Toaster.create() is not supported inside React lifecycle methods in React 16. See usage example on the docs site.",e.TOASTER_WARN_INLINE=r+" Toaster.create() ignores inline prop as it always creates a new element.",e.DIALOG_WARN_NO_HEADER_ICON=r+" <Dialog> iconName is ignored if title is omitted.",e.DIALOG_WARN_NO_HEADER_CLOSE_BUTTON=r+" <Dialog> isCloseButtonShown prop is ignored if title is omitted.",e.DRAWER_VERTICAL_IS_IGNORED=r+" <Drawer> vertical is ignored if position is defined",e.DRAWER_ANGLE_POSITIONS_ARE_CASTED=r+" <Drawer> all angle positions are casted into pure position (TOP, BOTTOM, LEFT or RIGHT)"},function(t,e,n){"use strict";function r(t,e,n){return void 0===n&&(n=function(t,e){return t===e}),null==t&&null==e||null!=t&&null!=e&&t.length===e.length&&t.every(function(t,r){return n(t,e[r])})}function i(t,e,n){if(null==t&&null==e)return!0;if(null==t||null==e)return!1;if(Array.isArray(t)||Array.isArray(e))return!1;if(null!=n)return A(t,e,n);var r=Object.keys(t),i=Object.keys(e);return A(t,e,{include:r})&&A(t,e,{include:i})}function o(t,e,n){if(t===e)return!0;if(null==t&&null==e)return!0;if(null==t||null==e)return!1;if(Array.isArray(t)||Array.isArray(e))return r(t,e,o);if(c(t)||c(e))return t===e;if(null!=n)return l(t,e,n);if(t.constructor!==e.constructor)return!1;var i=Object.keys(t),a=Object.keys(e);return null!=i&&null!=a&&(0===i.length&&0===a.length||r(i,a)&&l(t,e,i))}function a(t,e,n){var r=null==t?{}:t,o=null==e?{}:e;return h(r,o,u(r,o,null==n?{exclude:[]}:n),function(t,e,n){return i(t,e,{include:[n]})})}function s(t,e,n){return void 0===t&&(t={}),void 0===e&&(e={}),h(t,e,null==n?function(t,e){var n=Object.keys(t),r=Object.keys(e),i=g(n.concat(r));return Object.keys(i)}(t,e):n,function(t,e,n){return o(t,e,[n])})}function A(t,e,n){return u(t,e,n).every(function(n){return t.hasOwnProperty(n)===e.hasOwnProperty(n)&&t[n]===e[n]})}function l(t,e,n){return n.every(function(n){return t.hasOwnProperty(n)===e.hasOwnProperty(n)&&o(t[n],e[n])})}function c(t){return"number"==typeof t||"string"==typeof t||"boolean"==typeof t}function u(t,e,n){if(function(t){return null!=t&&null!=t.include}(n))return n.include;var r=Object.keys(t),i=Object.keys(e),o=g(r.concat(i));return n.exclude.forEach(function(t){return delete o[t]}),Object.keys(o)}function g(t){return t.reduce(function(t,e){return t[e]=!0,t},{})}function h(t,e,n,r){return n.filter(function(n){return!r(t,e,n)}).map(function(n){return{key:n,valueA:t[n],valueB:e[n]}})}n.d(e,"a",function(){return r}),n.d(e,"e",function(){return i}),n.d(e,"b",function(){return o}),n.d(e,"d",function(){return a}),n.d(e,"c",function(){return s})},function(t,e,n){"use strict";n.d(e,"a",function(){return i});var r=n(8);function i(t,e){for(var n=[],i=2;i<arguments.length;i++)n[i-2]=arguments[i];if(null!=t){var o=t[e];if(Object(r.isFunction)(o))return o.apply(void 0,n)}}},function(t,e,n){"use strict";e.__esModule=!0,e.classNamesShape=e.timeoutsShape=void 0;var r;(r=n(1))&&r.__esModule;e.timeoutsShape=null;e.classNamesShape=null},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=s(n(1)),i=s(n(0)),o=n(67),a=n(1157);function s(t){return t&&t.__esModule?t:{default:t}}function A(){return(A=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}function l(t){if(void 0===t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return t}var c=Object.values||function(t){return Object.keys(t).map(function(e){return t[e]})},u=function(t){var e,n;function r(e,n){var r,i=(r=t.call(this,e,n)||this).handleExited.bind(l(l(r)));return r.state={handleExited:i,firstRender:!0},r}n=t,(e=r).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n;var o=r.prototype;return o.getChildContext=function(){return{transitionGroup:{isMounting:!this.appeared}}},o.componentDidMount=function(){this.appeared=!0,this.mounted=!0},o.componentWillUnmount=function(){this.mounted=!1},r.getDerivedStateFromProps=function(t,e){var n=e.children,r=e.handleExited;return{children:e.firstRender?(0,a.getInitialChildMapping)(t,r):(0,a.getNextChildMapping)(t,n,r),firstRender:!1}},o.handleExited=function(t,e){var n=(0,a.getChildMapping)(this.props.children);t.key in n||(t.props.onExited&&t.props.onExited(e),this.mounted&&this.setState(function(e){var n=A({},e.children);return delete n[t.key],{children:n}}))},o.render=function(){var t=this.props,e=t.component,n=t.childFactory,r=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);return i}(t,["component","childFactory"]),o=c(this.state.children).map(n);return delete r.appear,delete r.enter,delete r.exit,null===e?o:i.default.createElement(e,r,o)},r}(i.default.Component);u.childContextTypes={transitionGroup:r.default.object.isRequired},u.propTypes={},u.defaultProps={component:"div",childFactory:function(t){return t}};var g=(0,o.polyfill)(u);e.default=g,t.exports=e.default},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2),i=r.__importStar(n(1158));e.Classes=i,r.__exportStar(n(1159),e),r.__exportStar(n(1160),e),r.__exportStar(n(1161),e)},function(t,e,n){"use strict";n.d(e,"f",function(){return c}),n.d(e,"g",function(){return u}),n.d(e,"h",function(){return g}),n.d(e,"b",function(){return h}),n.d(e,"a",function(){return d}),n.d(e,"e",function(){return f}),n.d(e,"d",function(){return p}),n.d(e,"c",function(){return I});var r,i=n(9),o=n(10),a=n(729),s=n(200),A=n(1452),l=function(t,e){var n={};for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&e.indexOf(r)<0&&(n[r]=t[r]);if(null!=t&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(t);i<r.length;i++)e.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(t,r[i])&&(n[r[i]]=t[r[i]])}return n};function c(t){return"code"===t.type}function u(t){return"markdown"===t.type}function g(t){return"raw"===t.type}class h extends s.a.Model{constructor(t){super({modelDB:t.modelDB}),this.contentChanged=new o.Signal(this),this.stateChanged=new o.Signal(this),this.id=t.id||i.UUID.uuid4(),this.value.changed.connect(this.onGenericChange,this),this.modelDB.createValue("type").set(this.type);let e=this.modelDB.createMap("metadata");e.changed.connect(this.onGenericChange,this);let n=t.cell,r=this.modelDB.createValue("trusted");if(r.changed.connect(this.onTrustedChanged,this),!n)return void r.set(!1);r.set(!!n.metadata.trusted),delete n.metadata.trusted,Array.isArray(n.source)?this.value.text=n.source.join(""):this.value.text=n.source;let a=i.JSONExt.deepCopy(n.metadata);"raw"!==this.type&&delete a.format,"code"!==this.type&&(delete a.collapsed,delete a.scrolled);for(let t in a)e.set(t,a[t])}get metadata(){return this.modelDB.get("metadata")}get trusted(){return this.modelDB.getValue("trusted")}set trusted(t){this.trusted!==t&&this.modelDB.setValue("trusted",t)}toJSON(){let t=Object.create(null);for(let e of this.metadata.keys()){let n=JSON.parse(JSON.stringify(this.metadata.get(e)));t[e]=n}return this.trusted&&(t.trusted=!0),{cell_type:this.type,source:this.value.text,metadata:t}}onTrustedChanged(t,e){}onGenericChange(){this.contentChanged.emit(void 0)}}class d extends h{constructor(t){super(t),this._attachments=null;let e,n=t.contentFactory||d.defaultContentFactory,r=t.cell;!r||"raw"!==r.cell_type&&"markdown"!==r.cell_type||(e=r.attachments),this._attachments=n.createAttachmentsModel({values:e,modelDB:this.modelDB}),this._attachments.stateChanged.connect(this.onGenericChange,this)}get attachments(){return this._attachments}toJSON(){let t=super.toJSON();return this.attachments.length&&(t.attachments=this.attachments.toJSON()),t}}!function(t){class e{createAttachmentsModel(t){return new a.a(t)}}t.ContentFactory=e,t.defaultContentFactory=new e}(d||(d={}));class f extends d{get type(){return"raw"}toJSON(){return super.toJSON()}}class p extends d{constructor(t){super(t),this.mimeType="text/x-ipythongfm"}get type(){return"markdown"}toJSON(){return super.toJSON()}}class I extends h{constructor(t){super(t),this._outputs=null;let e=t.contentFactory||I.defaultContentFactory,n=this.trusted,i=t.cell,o=[],a=this.modelDB.createValue("executionCount");if(a.get()||(i&&"code"===i.cell_type?(a.set(i.execution_count||null),o=i.outputs):a.set(null)),a.changed.connect(this._onExecutionCountChanged,this),this._outputs=e.createOutputArea({trusted:n,values:o}),this._outputs.changed.connect(this.onGenericChange,this),this.metadata.changed.connect(r.collapseChanged,this),this.metadata.has("collapsed")){let t=this.metadata.get("collapsed");r.collapseChanged(this.metadata,{type:"change",key:"collapsed",oldValue:t,newValue:t})}else if(this.metadata.has("jupyter")){let t=this.metadata.get("jupyter");t.hasOwnProperty("outputs_hidden")&&r.collapseChanged(this.metadata,{type:"change",key:"jupyter",oldValue:t,newValue:t})}}get type(){return"code"}get executionCount(){return this.modelDB.getValue("executionCount")}set executionCount(t){t!==this.executionCount&&this.modelDB.setValue("executionCount",t||null)}get outputs(){return this._outputs}dispose(){this.isDisposed||(this._outputs.dispose(),this._outputs=null,super.dispose())}toJSON(){let t=super.toJSON();return t.execution_count=this.executionCount||null,t.outputs=this.outputs.toJSON(),t}onTrustedChanged(t,e){this._outputs&&(this._outputs.trusted=e.newValue),this.stateChanged.emit({name:"trusted",oldValue:e.oldValue,newValue:e.newValue})}_onExecutionCountChanged(t,e){this.contentChanged.emit(void 0),this.stateChanged.emit({name:"executionCount",oldValue:e.oldValue,newValue:e.newValue})}}!function(t){class e{createOutputArea(t){return new A.a(t)}}t.ContentFactory=e,t.defaultContentFactory=new e}(I||(I={})),function(t){t.collapseChanged=function(t,e){if("collapsed"===e.key){const n=t.get("jupyter")||{},{outputs_hidden:r}=n,i=l(n,["outputs_hidden"]);r!==e.newValue&&(void 0!==e.newValue&&(i.outputs_hidden=e.newValue),0===Object.keys(i).length?t.delete("jupyter"):t.set("jupyter",i))}else if("jupyter"===e.key){const n=e.newValue||{};n.hasOwnProperty("outputs_hidden")?t.set("collapsed",n.outputs_hidden):t.delete("collapsed")}}}(r||(r={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return k}),n.d(e,"b",function(){return S}),n.d(e,"c",function(){return L}),n.d(e,"d",function(){return R});var r=n(729),i=n(17),o=n(206),a=n(193),s=n(9),A=n(6),l=n(196),c=n(197),u=n(156),g=n(198);const h="jp-Cell",d="jp-Cell-header",f="jp-Cell-footer",p="jp-Cell-inputWrapper",I="jp-Cell-outputWrapper",C="jp-Cell-inputArea",m="jp-Cell-outputArea",y="jp-Cell-inputCollapser",E="jp-Cell-outputCollapser",v="jp-mod-readOnly",B="jp-CodeCell",b="jp-MarkdownCell",Q="jp-MarkdownOutput",w="jp-RawCell",x="jp-mod-rendered",_="jp-mod-noOutputs",D="Type Markdown and LaTeX: $ α^2 $",M=1e3;class k extends A.Widget{constructor(t){super(),this._readOnly=!1,this._model=null,this._inputHidden=!1,this._input=null,this._inputWrapper=null,this._inputPlaceholder=null,this._syncCollapse=!1,this._syncEditable=!1,this.addClass(h);let e=this._model=t.model,n=this.contentFactory=t.contentFactory||k.defaultContentFactory;this.layout=new A.PanelLayout;let r=n.createCellHeader();r.addClass(d),this.layout.addWidget(r);let i=this._inputWrapper=new A.Panel;i.addClass(p);let o=new l.b;o.addClass(y);let a=this._input=new u.a({model:e,contentFactory:n,updateOnShow:t.updateEditorOnShow});a.addClass(C),i.addWidget(o),i.addWidget(a),this.layout.addWidget(i),this._inputPlaceholder=new g.a(()=>{this.inputHidden=!this.inputHidden});let s=this.contentFactory.createCellFooter();s.addClass(f),this.layout.addWidget(s),t.editorConfig&&Object.keys(t.editorConfig).forEach(e=>{this.editor.setOption(e,t.editorConfig[e])}),e.metadata.changed.connect(this.onMetadataChanged,this)}initializeState(){return this.loadCollapseState(),this.loadEditableState(),this}get promptNode(){return this._inputHidden?this._inputPlaceholder.node.firstElementChild:this._input.promptNode}get editorWidget(){return this._input.editorWidget}get editor(){return this._input.editor}get model(){return this._model}get inputArea(){return this._input}get readOnly(){return this._readOnly}set readOnly(t){t!==this._readOnly&&(this._readOnly=t,this.syncEditable&&this.saveEditableState(),this.update())}saveEditableState(){const{metadata:t}=this.model,e=t.get("editable");this.readOnly&&!1===e||!this.readOnly&&void 0===e||(this.readOnly?this.model.metadata.set("editable",!1):this.model.metadata.delete("editable"))}loadEditableState(){this.readOnly=!1===this.model.metadata.get("editable")}get ready(){return Promise.resolve(void 0)}setPrompt(t){this._input.setPrompt(t)}get inputHidden(){return this._inputHidden}set inputHidden(t){if(this._inputHidden===t)return;let e=this._inputWrapper.layout;t?(this._input.parent=null,e.addWidget(this._inputPlaceholder)):(this._inputPlaceholder.parent=null,e.addWidget(this._input)),this._inputHidden=t,this.syncCollapse&&this.saveCollapseState(),this.handleInputHidden(t)}saveCollapseState(){const t=Object.assign({},this.model.metadata.get("jupyter"));this.inputHidden&&!0===t.source_hidden||!this.inputHidden&&void 0===t.source_hidden||(this.inputHidden?t.source_hidden=!0:delete t.source_hidden,0===Object.keys(t).length?this.model.metadata.delete("jupyter"):this.model.metadata.set("jupyter",t))}loadCollapseState(){const t=this.model.metadata.get("jupyter")||{};this.inputHidden=!!t.source_hidden}handleInputHidden(t){}get syncCollapse(){return this._syncCollapse}set syncCollapse(t){this._syncCollapse!==t&&(this._syncCollapse=t,t&&this.loadCollapseState())}get syncEditable(){return this._syncEditable}set syncEditable(t){this._syncEditable!==t&&(this._syncEditable=t,t&&this.loadEditableState())}clone(){return new(0,this.constructor)({model:this.model,contentFactory:this.contentFactory})}dispose(){this.isDisposed||(this._input=null,this._model=null,this._inputWrapper=null,this._inputPlaceholder=null,super.dispose())}onAfterAttach(t){this.update()}onActivateRequest(t){this.editor.focus()}onFitRequest(t){this.editor.refresh()}onUpdateRequest(t){this._model&&this.editor.getOption("readOnly")!==this._readOnly&&(this.editor.setOption("readOnly",this._readOnly),this.toggleClass(v,this._readOnly))}onMetadataChanged(t,e){switch(e.key){case"jupyter":this.syncCollapse&&this.loadCollapseState();break;case"editable":this.syncEditable&&this.loadEditableState()}}}!function(t){class e{constructor(t={}){this._editorFactory=null,this._editorFactory=t.editorFactory||u.a.defaultEditorFactory}get editorFactory(){return this._editorFactory}createCellHeader(){return new c.b}createCellFooter(){return new c.a}createInputPrompt(){return new u.b}createOutputPrompt(){return new o.b}createStdin(t){return new o.d(t)}}t.ContentFactory=e,t.defaultContentFactory=new e}(k||(k={}));class S extends k{constructor(t){super(t),this._rendermime=null,this._outputHidden=!1,this._outputWrapper=null,this._outputPlaceholder=null,this._output=null,this._syncScrolled=!1,this._savingMetadata=!1,this.addClass(B);let e=this._rendermime=t.rendermime,n=this.contentFactory,r=this.model,i=this._outputWrapper=new A.Panel;i.addClass(I);let a=new l.c;a.addClass(E);let s=this._output=new o.a({model:r.outputs,rendermime:e,contentFactory:n});s.addClass(m),0===r.outputs.length&&this.addClass(_),s.outputLengthChanged.connect(this._outputLengthHandler,this),i.addWidget(a),i.addWidget(s),this.layout.insertWidget(2,i),this._outputPlaceholder=new g.b(()=>{this.outputHidden=!this.outputHidden}),r.stateChanged.connect(this.onStateChanged,this)}initializeState(){return super.initializeState(),this.loadScrolledState(),this.setPrompt(`${this.model.executionCount||""}`),this}get outputArea(){return this._output}get outputHidden(){return this._outputHidden}set outputHidden(t){if(this._outputHidden===t)return;let e=this._outputWrapper.layout;t?(e.removeWidget(this._output),e.addWidget(this._outputPlaceholder),this.inputHidden&&!this._outputWrapper.isHidden&&this._outputWrapper.hide()):(this._outputWrapper.isHidden&&this._outputWrapper.show(),e.removeWidget(this._outputPlaceholder),e.addWidget(this._output)),this._outputHidden=t,this.syncCollapse&&this.saveCollapseState()}saveCollapseState(){this._savingMetadata=!0;try{super.saveCollapseState();const t=this.model.metadata,e=this.model.metadata.get("collapsed");if(this.outputHidden&&!0===e||!this.outputHidden&&void 0===e)return;this.outputHidden?t.set("collapsed",!0):t.delete("collapsed")}finally{this._savingMetadata=!1}}loadCollapseState(){super.loadCollapseState(),this.outputHidden=!!this.model.metadata.get("collapsed")}get outputsScrolled(){return this._outputsScrolled}set outputsScrolled(t){this.toggleClass("jp-mod-outputsScrolled",t),this._outputsScrolled=t,this.syncScrolled&&this.saveScrolledState()}saveScrolledState(){const{metadata:t}=this.model,e=t.get("scrolled");this.outputsScrolled&&!0===e||!this.outputsScrolled&&void 0===e||(this.outputsScrolled?t.set("scrolled",!0):t.delete("scrolled"))}loadScrolledState(){const t=this.model.metadata;"auto"===t.get("scrolled")?this.outputsScrolled=!1:this.outputsScrolled=!!t.get("scrolled")}get syncScrolled(){return this._syncScrolled}set syncScrolled(t){this._syncScrolled!==t&&(this._syncScrolled=t,t&&this.loadScrolledState())}handleInputHidden(t){!t&&this._outputWrapper.isHidden?this._outputWrapper.show():t&&!this._outputWrapper.isHidden&&this._outputHidden&&this._outputWrapper.hide()}clone(){return new(0,this.constructor)({model:this.model,contentFactory:this.contentFactory,rendermime:this._rendermime})}cloneOutputArea(){return new o.c({model:this.model.outputs,contentFactory:this.contentFactory,rendermime:this._rendermime})}dispose(){this.isDisposed||(this._output.outputLengthChanged.disconnect(this._outputLengthHandler,this),this._rendermime=null,this._output=null,this._outputWrapper=null,this._outputPlaceholder=null,super.dispose())}onStateChanged(t,e){switch(e.name){case"executionCount":this.setPrompt(`${t.executionCount||""}`)}}onMetadataChanged(t,e){if(!this._savingMetadata){switch(e.key){case"scrolled":this.syncScrolled&&this.loadScrolledState();break;case"collapsed":this.syncCollapse&&this.loadCollapseState()}super.onMetadataChanged(t,e)}}_outputLengthHandler(t,e){let n=0===e;this.toggleClass(_,n)}}!function(t){t.execute=async function(t,e,n){let r=t.model,i=r.value.text;if(!i.trim()||!e.kernel)return r.executionCount=null,void r.outputs.clear();let a,s={cellId:r.id};n=Object.assign({},r.metadata.toJSON(),n,s),r.executionCount=null,t.outputHidden=!1,t.setPrompt("*"),r.trusted=!0;try{const s=o.a.execute(i,t.outputArea,e,n);a=t.outputArea.future;const A=await s;return r.executionCount=A.content.execution_count,A}catch(e){throw"Canceled"===e.message&&t.outputArea.future===a&&t.setPrompt(""),e}}}(S||(S={}));class L extends k{constructor(t){super(t),this._monitor=null,this._renderer=null,this._rendered=!0,this._prevText="",this._ready=new s.PromiseDelegate,this.addClass(b),this._rendermime=t.rendermime.clone({resolver:new r.b({parent:t.rendermime.resolver,model:this.model.attachments})}),this._monitor=new i.ActivityMonitor({signal:this.model.contentChanged,timeout:M}),this._monitor.activityStopped.connect(()=>{this._rendered&&this.update()},this),this._updateRenderedInput().then(()=>{this._ready.resolve(void 0)}),this.renderInput(this._renderer)}get ready(){return this._ready.promise}get rendered(){return this._rendered}set rendered(t){t!==this._rendered&&(this._rendered=t,this._handleRendered(),this._rendered||this.editor.refresh())}renderInput(t){this.addClass(x),this.inputArea.renderInput(t)}showEditor(){this.removeClass(x),this.inputArea.showEditor()}onUpdateRequest(t){this._handleRendered(),super.onUpdateRequest(t)}_handleRendered(){this._rendered?(this._updateRenderedInput(),this.renderInput(this._renderer)):this.showEditor()}_updateRenderedInput(){let t=this.model,e=t&&t.value.text||D;if(e!==this._prevText){let t=new a.a({data:{"text/markdown":e}});return this._renderer||(this._renderer=this._rendermime.createRenderer("text/markdown"),this._renderer.addClass(Q)),this._prevText=e,this._renderer.renderModel(t)}return Promise.resolve(void 0)}clone(){return new(0,this.constructor)({model:this.model,contentFactory:this.contentFactory,rendermime:this._rendermime})}}class R extends k{constructor(t){super(t),this.addClass(w)}clone(){return new(0,this.constructor)({model:this.model,contentFactory:this.contentFactory})}}},function(t,e,n){"use strict";n.d(e,"a",function(){return g});var r=n(9),i=n(6),o=n(200);const a="jp-JSONEditor",s="jp-mod-error",A="jp-JSONEditor-host",l="jp-JSONEditor-header",c="jp-JSONEditor-revertButton",u="jp-JSONEditor-commitButton";class g extends i.Widget{constructor(t){super(),this._dataDirty=!1,this._inputDirty=!1,this._source=null,this._originalValue=r.JSONExt.emptyObject,this._changeGuard=!1,this.addClass(a),this.headerNode=document.createElement("div"),this.headerNode.className=l,this.revertButtonNode=document.createElement("span"),this.revertButtonNode.className=c,this.revertButtonNode.title="Revert changes to data",this.commitButtonNode=document.createElement("span"),this.commitButtonNode.className=u,this.commitButtonNode.title="Commit changes to data",this.editorHostNode=document.createElement("div"),this.editorHostNode.className=A,this.headerNode.appendChild(this.revertButtonNode),this.headerNode.appendChild(this.commitButtonNode),this.node.appendChild(this.headerNode),this.node.appendChild(this.editorHostNode);let e=new o.a.Model;e.value.text="No data!",e.mimeType="application/json",e.value.changed.connect(this._onValueChanged,this),this.model=e,this.editor=t.editorFactory({host:this.editorHostNode,model:e}),this.editor.setOption("readOnly",!0)}get source(){return this._source}set source(t){this._source!==t&&(this._source&&this._source.changed.disconnect(this._onSourceChanged,this),this._source=t,this.editor.setOption("readOnly",null===t),t&&t.changed.connect(this._onSourceChanged,this),this._setValue())}get isDirty(){return this._dataDirty||this._inputDirty}handleEvent(t){switch(t.type){case"blur":this._evtBlur(t);break;case"click":this._evtClick(t)}}onAfterAttach(t){let e=this.editorHostNode;e.addEventListener("blur",this,!0),e.addEventListener("click",this,!0),this.revertButtonNode.hidden=!0,this.commitButtonNode.hidden=!0,this.headerNode.addEventListener("click",this),this.isVisible&&this.update()}onAfterShow(t){this.update()}onUpdateRequest(t){this.editor.refresh()}onBeforeDetach(t){let e=this.editorHostNode;e.removeEventListener("blur",this,!0),e.removeEventListener("click",this,!0),this.headerNode.removeEventListener("click",this)}_onSourceChanged(t,e){this._changeGuard||(this._inputDirty||this.editor.hasFocus()?this._dataDirty=!0:this._setValue())}_onValueChanged(){let t=!0;try{let e=JSON.parse(this.editor.model.value.text);this.removeClass(s),this._inputDirty=!this._changeGuard&&!r.JSONExt.deepEqual(e,this._originalValue)}catch(e){this.addClass(s),this._inputDirty=!0,t=!1}this.revertButtonNode.hidden=!this._inputDirty,this.commitButtonNode.hidden=!t||!this._inputDirty}_evtBlur(t){!this._inputDirty&&this._dataDirty&&this._setValue()}_evtClick(t){switch(t.target){case this.revertButtonNode:this._setValue();break;case this.commitButtonNode:this.commitButtonNode.hidden||this.hasClass(s)||(this._changeGuard=!0,this._mergeContent(),this._changeGuard=!1,this._setValue());break;case this.editorHostNode:this.editor.focus()}}_mergeContent(){let t=this.editor.model,e=this._originalValue,n=JSON.parse(t.value.text),i=this.source;if(i){for(let t in n)r.JSONExt.deepEqual(n[t],e[t]||null)||i.set(t,n[t]);for(let t in e)t in n||i.delete(t)}}_setValue(){this._dataDirty=!1,this._inputDirty=!1,this.revertButtonNode.hidden=!0,this.commitButtonNode.hidden=!0,this.removeClass(s);let t=this.editor.model,e=this._source?this._source.toJSON():{};if(this._changeGuard=!0,void 0===e)t.value.text="No data!",this._originalValue=r.JSONExt.emptyObject;else{let n=JSON.stringify(e,null,4);t.value.text=n,this._originalValue=e,n.length>1&&"{"===n[0]&&this.editor.setCursorPosition({line:0,column:1})}this.editor.refresh(),this._changeGuard=!1,this.commitButtonNode.hidden=!0,this.revertButtonNode.hidden=!0}}},function(t,e,n){"use strict";n.d(e,"a",function(){return i}),n.d(e,"b",function(){return o});var r=n(9);const i=new r.Token("@jupyterlab/notebook:INotebookTools"),o=new r.Token("@jupyterlab/notebook:INotebookTracker")},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9),i=n(64),o=n(28),a=n(25),s=n(108),A=n(48),l=n(338);e.PartManager=class{constructor({globals:t,session:e,rendermime:n,factory:r,dashboardId:o,bindings:a,baseUrl:s,baseViewUrl:l}){this.OnDirtySrc$=new i.Subject,this.optionsBags=new Map,this.partMap=new Map,this.partEvaluations=new Map,this.subscriptions=new Map,this._isDisposed=!1,this._isDirty=!1,this._nErrorParts=0,this.OnDirty=this.OnDirtySrc$.asObservable(),this.globals=t,this.factory=r,this.bindings=a,this.partServices=new A.PartServices({session:e,rendermime:n,dashboardId:o,baseUrl:s,baseViewUrl:l}),this.globalChangeSubscription=this.globals.OnChange.subscribe(t=>{if(!this.subscriptions.has(t.name))return;if("delete"===t.action)return void this.subscriptions.delete(t.oldName||t.name);if("rename"===t.action){const e=this.subscriptions.get(t.oldName)||[];return this.subscriptions.delete(t.oldName),void this.subscriptions.set(t.name,e)}const e=this.subscriptions.get(t.name);for(const n of e){const e=this.optionsBags.get(n.partId),r=e.getMetadata(n.option);"Global"===r.binding.type&&e.set(n.option,t.value),e.setStale([[n.option,r]]),this.evaluateOrWaitForUser(n.partId)}})}get isDisposed(){return this._isDisposed}get isDirty(){return this._isDirty}get hasErrorParts(){return this._nErrorParts>0}[Symbol.iterator](){return this.partMap.entries()}setClean(){this._isDirty=!1}dispose(){this._isDisposed||(this.globalChangeSubscription.unsubscribe(),this._isDisposed=!0,this.OnDirtySrc$.complete(),this.partEvaluations.forEach((t,e)=>this.cancelPartEvaluations(e)),this.optionsBags.forEach(t=>t.dispose()),this.optionsBags.clear(),this.partMap.clear())}async addPart(t,e){const n=e?e.id:r.UUID.uuid4(),i=this.factory.createPart(t,{uuid:n,services:this.partServices});if(i instanceof A.ErrorPart)return i.setModel(e),this.partMap.set(i.uuid,i),this.optionsBags.set(i.uuid,new A.OptionsBag([])),this._nErrorParts++,i;const a=i.constructor.GetMetadata(),s=new A.OptionsBag(a);if(null!=e)for(const t in e.options){const n=e.options[t];this.trySetOption(t,n,s,i)}return this.partMap.set(i.uuid,i),i.RefreshRequested.subscribe(t=>{const e=this.optionsBags.get(i.uuid);if(!e)throw Error("Could not refresh part: no options bag found for ID "+i.uuid);this.cancelPartEvaluations(i.uuid),e.setAllOptionsStale(),this.evaluateOrWaitForUser(i.uuid,t)}),i.CancelRequested.subscribe(()=>{if(!this.optionsBags.get(i.uuid))throw Error("Could not cancel part: no options bag found for ID "+i.uuid);this.cancelPart(i.uuid)}),i.disposed.connect(this.onPartDisposed,this),this.optionsBags.set(i.uuid,s),this.partEvaluations.set(i.uuid,{renderLock:new o.AsyncTools.Mutex,evals:new Map}),s.OnOptionChanged.subscribe(t=>{t.isStale&&this.handleOptionChanged(i.uuid)}),o.StartTimingAsync("InitializePart",()=>this.initializePart(i.uuid)),i}removePart(t){let e;if("string"==typeof t){if(!this.partMap.has(t))throw console.error("[PartManager]","Attempted to remove unknown part"),Error("Cannot remove part: No part exists w/ ID "+t);e=this.partMap.get(t)}else e=t;return e instanceof A.ErrorPart&&this._nErrorParts--,this.cancelPartEvaluations(e.uuid),this.partEvaluations.delete(e.uuid),this.partMap.delete(e.uuid),this.optionsBags.get(e.uuid).dispose(),this.optionsBags.delete(e.uuid),this.unhookPartSubscriptions(e.uuid),e.disposed.disconnect(this.onPartDisposed,this),e.dispose(),e}getPartById(t){return this.partMap.get(t)||null}getBagById(t){return this.optionsBags.get(t)||null}setOptionForPart(t,e,n){const r=this.getBagById(t),i=this.getPartById(t);null!=r&&null!=i&&this.trySetOption(e,n,r,i)}clearParts(){for(const[t,e]of this.partMap)this.removePart(e)}trySetOption(t,e,n,r){const i=n.getMetadata(t);if(this.unhookStaleSubscriptions(r.uuid,i),!e)return n.clearBinding(t),void n.set(t,null);if(e.hasOwnProperty("typeName"))return n.clearBinding(t),void n.set(t,o.Converters.deserialize(e));if((e=e).__detect_globals){const t=this.bindings.getBindingEvaluator(e.type);e.globals=t.getGlobalsForBinding(e.expr)}if(n.setBinding(t,e.type,e.expr,e.globals),"Global"===e.type)if(this.globals.has(e.expr))n.set(t,this.globals.get(e.expr));else{const t=new Error("No global named "+e.expr+" exists!");r.error(t,"option-error")}this.setSubscriptions(r.uuid,n.getMetadata(t))}setDirty(){this._isDirty||(this._isDirty=!0,this.OnDirtySrc$.next())}async handleOptionChanged(t){this.evaluateOrWaitForUser(t)}evaluateOrWaitForUser(t,e=!1){const n=this.getPartById(t);null!=n&&(!window.WaitForUser||e?n.isCanceled&&!e||this.evaluateOptions(t):a.MessageLoop.sendMessage(n,A.Part.Lifecycle.WaitForUser))}evaluateOptions(t){const e=this.getPartById(t),n=this.optionsBags.get(t),r=this.partEvaluations.get(t);if(null!=n&&null!=e&&null!=r)if(n.isStale){0===r.evals.size&&a.MessageLoop.sendMessage(e,A.Part.Lifecycle.BeforeCalculate);for(const[e,i]of n.getStaleOptionValues()){let n=r.evals.get(e);n&&n.cancel();const a=new o.CancelToken;r.evals.set(e,a),this.evaluateOptionForPart(t,e,i,a).then(n=>this.handleOptionFinished(t,e,a,n))}0===r.evals.size&&(console.error("Stale optionbag but no evaluations were started"),this.handlePartFinished(t))}else this.renderPart(t)}handleOptionFinished(t,e,n,r){const i=this.getPartById(t),o=this.optionsBags.get(t),a=this.partEvaluations.get(t);if(null==o||null==i||null==a)return;let s=a.evals.get(e);s&&s.guid===n.guid&&(a.evals.delete(e),r.canceled||r.error&&this.cancelPartEvaluations(t),0===a.evals.size&&this.handlePartFinished(t))}handlePartFinished(t){const e=this.getPartById(t),n=this.optionsBags.get(t),r=this.partEvaluations.get(t);null!=n&&null!=e&&null!=r&&(e.isError||e.isCanceled||(n.setFresh(),a.MessageLoop.sendMessage(e,A.Part.Lifecycle.AfterCalculate)),this.renderPart(e.uuid))}cancelPartEvaluations(t){const e=this.getPartById(t),n=this.getBagById(t);if(null==e||null==n)throw Error("Invalid Part ID"+t);const r=this.partEvaluations.get(t);if(r){for(const t of r.evals.values())t.cancel();r.evals.clear()}}cancelPart(t){const e=this.getPartById(t),n=this.getBagById(t);if(null==e||null==n)throw Error("Invalid Part ID"+t);a.MessageLoop.sendMessage(e,A.Part.Lifecycle.Cancel),window.WaitForUser&&a.MessageLoop.sendMessage(e,A.Part.Lifecycle.WaitForUser),this.cancelPartEvaluations(t)}async evaluateOptionForPart(t,e,n,r){const i=this.getPartById(t),a=this.getBagById(t);let A=!1;if(null==i||null==a)throw Error("Invalid Part ID"+t);i.optionCalculating("before-option-calc",e);try{const t=a.getMetadata(e);this.isOptionChangeAlwaysStalable(n,t)&&this.setDirty();const c=t.binding;if(null==c)return{canceled:r.isCanceled};let u=this.bindings.getBindingEvaluator(c.type);try{if(u instanceof l.ErrorBindingsEvaluator)throw Error(`Invalid binding for option ${e}: ${c.expr}`);let t=!0;r.onCancel(()=>{t&&(A=!0,i.optionCalculating("after-option-calc",e),u.cancel(r.guid))});const n=await o.StartTimingAsync(`EvaluateOption-${e}`,async()=>await u.evaluate(r.guid,c.expr,c.globals));if(t=!1,i.isDisposed)return{canceled:!0};r.isCanceled||("Global"===u.name?s.isEqual(a.get(e),n)||this.globals.set(c.expr,a.get(e)):a.setBindingValue(e,n))}catch(t){return t instanceof o.CancelError?{canceled:!0}:(console.error("[PartManager]","Option",e,"failed to calculate"),console.error(i.stateDetail),a.setError([e]),i.error(t,"option-error"),{error:t,canceled:r.isCanceled})}}finally{i.isDisposed||A||i.optionCalculating("after-option-calc",e)}return{canceled:r.isCanceled}}getGlobalsForBinding(t){if(null==t.binding)return[];const e=t.binding;return this.bindings.getBindingEvaluator(e.type).getGlobalsForBinding(e.expr)}unhookPartSubscriptions(t){for(const[e,n]of this.subscriptions.entries())this.subscriptions.set(e,n.filter(e=>e.partId!==t))}unhookStaleSubscriptions(t,e){for(const n of this.getGlobalsForBinding(e)){const r=this.subscriptions.get(n);if(null==r)return;const i=r.filter(n=>n.partId!==t||n.option!==e.name);this.subscriptions.set(n,i)}}setSubscriptions(t,e){for(const n of this.getGlobalsForBinding(e)){let r;(r=this.subscriptions.has(n)?this.subscriptions.get(n):[]).push({option:e.name,partId:t}),this.subscriptions.set(n,r)}}async initializePart(t){const e=this.getPartById(t);if(null==e)throw Error("No part with id "+t+" found!");try{a.MessageLoop.sendMessage(e,A.Part.Lifecycle.BeforeInitialize),await e.initialize(),a.MessageLoop.sendMessage(e,A.Part.Lifecycle.AfterInitialize),e.isError||this.evaluateOrWaitForUser(e.uuid)}catch(t){console.log("[PartManager]","Part failed to initialize"),console.log(t),e.error(t,"init-error")}}async renderPart(t){const e=this.getPartById(t),n=this.partEvaluations.get(t);if(null!=e&&null!=n)if(e.isIdle)if(e.isInitialized){if(!e.isError&&!e.isCanceled){await n.renderLock.aquire();try{if(null==this.getPartById(t)||!e.isIdle||e.isError||e.isCanceled)return;const r=this.optionsBags.get(t);try{await o.StartTimingAsync("RenderPart",async()=>{a.MessageLoop.sendMessage(e,A.Part.Lifecycle.BeforeRender),await e.render(r),a.MessageLoop.sendMessage(e,A.Part.Lifecycle.AfterRender)})}catch(t){console.log("[PartManager]","Part failed to render"),console.log(t),e.error(t,"render-error")}}finally{n.renderLock.release()}}}else console.error("renderPart called while part is uninitialized");else console.error("renderPart called while part still running")}onPartDisposed(t){this.partMap.has(t.uuid)&&this.removePart(t.uuid)}isOptionChangeAlwaysStalable(t,e){if(null==e.binding)return!0;const n=t.binding,r=e.binding;return null==n||n.type!==r.type||n.globals!==r.globals||n.expr!==r.expr}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9);!function(t){t.wait=function(t){return new Promise(e=>setTimeout(e,t))},t.waitForFrame=function(){return new Promise(t=>requestAnimationFrame(t))},t.waitUntil=async function(t,e,n=25){const i=new Promise((t,r)=>setTimeout(()=>r(Error("Timeout: Predicate did not become true within "+n+"ms")),e));let o=new r.PromiseDelegate,a=setInterval(()=>{try{t.call(void 0)&&o.resolve(void 0)}catch(t){o.reject(t)}},n);try{await Promise.race([o.promise,i])}finally{clearInterval(a)}};t.Mutex=class{get lock(){return this._lock||Promise.resolve()}get isFree(){return void 0===this._lock}async aquire(){if(null!=this._lock)return await this._lock,this.aquire();this._lock=new Promise(t=>{this.resolve=t})}release(){null!=this.resolve&&(this.resolve(),this._lock=void 0)}}}(e.AsyncTools||(e.AsyncTools={}))},function(t,e,n){var r=n(632),i=n(1172),o={};for(var a in r)r.hasOwnProperty(a)&&(o[r[a]]=a);var s=t.exports={to:{},get:{}};function A(t,e,n){return Math.min(Math.max(e,t),n)}function l(t){var e=t.toString(16).toUpperCase();return e.length<2?"0"+e:e}s.get=function(t){var e,n;switch(t.substring(0,3).toLowerCase()){case"hsl":e=s.get.hsl(t),n="hsl";break;case"hwb":e=s.get.hwb(t),n="hwb";break;default:e=s.get.rgb(t),n="rgb"}return e?{model:n,value:e}:null},s.get.rgb=function(t){if(!t)return null;var e,n,i,o=[0,0,0,1];if(e=t.match(/^#([a-f0-9]{6})([a-f0-9]{2})?$/i)){for(i=e[2],e=e[1],n=0;n<3;n++){var a=2*n;o[n]=parseInt(e.slice(a,a+2),16)}i&&(o[3]=Math.round(parseInt(i,16)/255*100)/100)}else if(e=t.match(/^#([a-f0-9]{3,4})$/i)){for(i=(e=e[1])[3],n=0;n<3;n++)o[n]=parseInt(e[n]+e[n],16);i&&(o[3]=Math.round(parseInt(i+i,16)/255*100)/100)}else if(e=t.match(/^rgba?\(\s*([+-]?\d+)\s*,\s*([+-]?\d+)\s*,\s*([+-]?\d+)\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)$/)){for(n=0;n<3;n++)o[n]=parseInt(e[n+1],0);e[4]&&(o[3]=parseFloat(e[4]))}else{if(!(e=t.match(/^rgba?\(\s*([+-]?[\d\.]+)\%\s*,\s*([+-]?[\d\.]+)\%\s*,\s*([+-]?[\d\.]+)\%\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)$/)))return(e=t.match(/(\D+)/))?"transparent"===e[1]?[0,0,0,0]:(o=r[e[1]])?(o[3]=1,o):null:null;for(n=0;n<3;n++)o[n]=Math.round(2.55*parseFloat(e[n+1]));e[4]&&(o[3]=parseFloat(e[4]))}for(n=0;n<3;n++)o[n]=A(o[n],0,255);return o[3]=A(o[3],0,1),o},s.get.hsl=function(t){if(!t)return null;var e=t.match(/^hsla?\(\s*([+-]?(?:\d*\.)?\d+)(?:deg)?\s*,\s*([+-]?[\d\.]+)%\s*,\s*([+-]?[\d\.]+)%\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)$/);if(e){var n=parseFloat(e[4]);return[(parseFloat(e[1])+360)%360,A(parseFloat(e[2]),0,100),A(parseFloat(e[3]),0,100),A(isNaN(n)?1:n,0,1)]}return null},s.get.hwb=function(t){if(!t)return null;var e=t.match(/^hwb\(\s*([+-]?\d*[\.]?\d+)(?:deg)?\s*,\s*([+-]?[\d\.]+)%\s*,\s*([+-]?[\d\.]+)%\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)$/);if(e){var n=parseFloat(e[4]);return[(parseFloat(e[1])%360+360)%360,A(parseFloat(e[2]),0,100),A(parseFloat(e[3]),0,100),A(isNaN(n)?1:n,0,1)]}return null},s.to.hex=function(){var t=i(arguments);return"#"+l(t[0])+l(t[1])+l(t[2])+(t[3]<1?l(Math.round(255*t[3])):"")},s.to.rgb=function(){var t=i(arguments);return t.length<4||1===t[3]?"rgb("+Math.round(t[0])+", "+Math.round(t[1])+", "+Math.round(t[2])+")":"rgba("+Math.round(t[0])+", "+Math.round(t[1])+", "+Math.round(t[2])+", "+t[3]+")"},s.to.rgb.percent=function(){var t=i(arguments),e=Math.round(t[0]/255*100),n=Math.round(t[1]/255*100),r=Math.round(t[2]/255*100);return t.length<4||1===t[3]?"rgb("+e+"%, "+n+"%, "+r+"%)":"rgba("+e+"%, "+n+"%, "+r+"%, "+t[3]+")"},s.to.hsl=function(){var t=i(arguments);return t.length<4||1===t[3]?"hsl("+t[0]+", "+t[1]+"%, "+t[2]+"%)":"hsla("+t[0]+", "+t[1]+"%, "+t[2]+"%, "+t[3]+")"},s.to.hwb=function(){var t=i(arguments),e="";return t.length>=4&&1!==t[3]&&(e=", "+t[3]),"hwb("+t[0]+", "+t[1]+"%, "+t[2]+"%"+e+")"},s.to.keyword=function(t){return o[t.slice(0,3)]}},function(t,e,n){"use strict";t.exports={aliceblue:[240,248,255],antiquewhite:[250,235,215],aqua:[0,255,255],aquamarine:[127,255,212],azure:[240,255,255],beige:[245,245,220],bisque:[255,228,196],black:[0,0,0],blanchedalmond:[255,235,205],blue:[0,0,255],blueviolet:[138,43,226],brown:[165,42,42],burlywood:[222,184,135],cadetblue:[95,158,160],chartreuse:[127,255,0],chocolate:[210,105,30],coral:[255,127,80],cornflowerblue:[100,149,237],cornsilk:[255,248,220],crimson:[220,20,60],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgoldenrod:[184,134,11],darkgray:[169,169,169],darkgreen:[0,100,0],darkgrey:[169,169,169],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkseagreen:[143,188,143],darkslateblue:[72,61,139],darkslategray:[47,79,79],darkslategrey:[47,79,79],darkturquoise:[0,206,209],darkviolet:[148,0,211],deeppink:[255,20,147],deepskyblue:[0,191,255],dimgray:[105,105,105],dimgrey:[105,105,105],dodgerblue:[30,144,255],firebrick:[178,34,34],floralwhite:[255,250,240],forestgreen:[34,139,34],fuchsia:[255,0,255],gainsboro:[220,220,220],ghostwhite:[248,248,255],gold:[255,215,0],goldenrod:[218,165,32],gray:[128,128,128],green:[0,128,0],greenyellow:[173,255,47],grey:[128,128,128],honeydew:[240,255,240],hotpink:[255,105,180],indianred:[205,92,92],indigo:[75,0,130],ivory:[255,255,240],khaki:[240,230,140],lavender:[230,230,250],lavenderblush:[255,240,245],lawngreen:[124,252,0],lemonchiffon:[255,250,205],lightblue:[173,216,230],lightcoral:[240,128,128],lightcyan:[224,255,255],lightgoldenrodyellow:[250,250,210],lightgray:[211,211,211],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightsalmon:[255,160,122],lightseagreen:[32,178,170],lightskyblue:[135,206,250],lightslategray:[119,136,153],lightslategrey:[119,136,153],lightsteelblue:[176,196,222],lightyellow:[255,255,224],lime:[0,255,0],limegreen:[50,205,50],linen:[250,240,230],magenta:[255,0,255],maroon:[128,0,0],mediumaquamarine:[102,205,170],mediumblue:[0,0,205],mediumorchid:[186,85,211],mediumpurple:[147,112,219],mediumseagreen:[60,179,113],mediumslateblue:[123,104,238],mediumspringgreen:[0,250,154],mediumturquoise:[72,209,204],mediumvioletred:[199,21,133],midnightblue:[25,25,112],mintcream:[245,255,250],mistyrose:[255,228,225],moccasin:[255,228,181],navajowhite:[255,222,173],navy:[0,0,128],oldlace:[253,245,230],olive:[128,128,0],olivedrab:[107,142,35],orange:[255,165,0],orangered:[255,69,0],orchid:[218,112,214],palegoldenrod:[238,232,170],palegreen:[152,251,152],paleturquoise:[175,238,238],palevioletred:[219,112,147],papayawhip:[255,239,213],peachpuff:[255,218,185],peru:[205,133,63],pink:[255,192,203],plum:[221,160,221],powderblue:[176,224,230],purple:[128,0,128],rebeccapurple:[102,51,153],red:[255,0,0],rosybrown:[188,143,143],royalblue:[65,105,225],saddlebrown:[139,69,19],salmon:[250,128,114],sandybrown:[244,164,96],seagreen:[46,139,87],seashell:[255,245,238],sienna:[160,82,45],silver:[192,192,192],skyblue:[135,206,235],slateblue:[106,90,205],slategray:[112,128,144],slategrey:[112,128,144],snow:[255,250,250],springgreen:[0,255,127],steelblue:[70,130,180],tan:[210,180,140],teal:[0,128,128],thistle:[216,191,216],tomato:[255,99,71],turquoise:[64,224,208],violet:[238,130,238],wheat:[245,222,179],white:[255,255,255],whitesmoke:[245,245,245],yellow:[255,255,0],yellowgreen:[154,205,50]}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(141),i=n(64),o=n(6),a=n(0),s=n(28);var A=s.MathTools.Vec2;const l=n(222);!function(t){t.Button=class extends r.ReactWrapperWidget{constructor(){super(...arguments),this._label="",this._className="",this._onClicked=new i.Subject}get onClicked(){return this._onClicked}dispose(){this.isDisposed||(this._onClicked.complete(),super.dispose())}get label(){return this._label}set label(t){this._label=t,this.update()}get className(){return this._className}set className(t){this._className=t,this.update()}render(){return a.createElement("button",{className:"m-Button "+this._className,onClick:()=>this._onClicked.next()},this._label)}};t.ResizerGrippy=class extends o.Widget{constructor(t,e){super(),this._onSizeChange=new i.Subject,this.owner=t,this.boundary=e,this.addClass("m-ResizeGrippy")}get onSizeChange(){return this._onSizeChange}dispose(){this.isDisposed||(this._onSizeChange.complete(),super.dispose())}handleEvent(t){switch(t.type){case"pointerdown":this.onStartDrag(t);break;case"pointerup":case"keyup":this.onEndDrag(t);break;case"pointermove":this.onDragResize(t)}}onAfterAttach(){this.node.addEventListener("pointerdown",this)}onBeforeDetach(){this.node.removeEventListener("pointerdown",this),this.onEndDrag()}onStartDrag(t){t.preventDefault(),t.stopPropagation(),s.FrameTools.DisableFramePointerEvents();const e=this.owner.node.getBoundingClientRect(),{width:n,height:r,left:i,top:o}=this.boundary.getBoundingClientRect();this.oldSize={left:e.left-i,top:e.top-o,width:e.width,height:e.height},this.dragStartPosition=[t.clientX,t.clientY],this.bounds=[n,r],window.addEventListener("keyup",this,{capture:!0}),window.addEventListener("pointerup",this,{capture:!0}),window.addEventListener("pointermove",this,{capture:!0})}onDragResize(t){t.preventDefault(),t.stopPropagation();const e=this.dragStartPosition,n=A.Sub([t.clientX,t.clientY],e);let{left:r,top:i,width:o,height:a}=this.oldSize;const l=A.Add([o,a],n);this._onSizeChange.next(s.MathTools.ClampRectToBounds([r,i],l,this.bounds))}onEndDrag(t){if(t&&"keyup"===t.type){if("Escape"!==t.key)return;{const{left:t,top:e,width:n,height:r}=this.oldSize;this._onSizeChange.next([t,e,n,r])}}s.FrameTools.EnableFramePointerEvents(),window.removeEventListener("keyup",this,{capture:!0}),window.removeEventListener("pointerup",this,{capture:!0}),window.removeEventListener("pointermove",this,{capture:!0}),this.oldSize=void 0}};t.ListBoxWidget=class extends r.ReactWrapperWidget{constructor(){super(...arguments),this._isEditing=!1,this._selected=null,this._items=[],this._onEdit=new i.Subject,this._onSelect=new i.Subject}get isEditing(){return this._isEditing}get selected(){return this._selected}get items(){return this._items}set isEditing(t){this._isEditing=t,this.update()}set selected(t){this._selected=t,this.update()}set items(t){this._items=t,this.update()}get onEdit(){return this._onEdit}get onSelect(){return this._onSelect}dispose(){this.isDisposed||(this._onEdit.complete(),this._onSelect.complete(),super.dispose())}render(){return a.createElement(l.ListBox,{items:this._items,isEditing:this._isEditing,onEdit:(t,e)=>this._onEdit.next({key:t,newLabel:e}),onSelect:t=>{this.selected=t,this._onSelect.next(t)},selectedKey:this._selected})}}}(e.Interactions||(e.Interactions={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(313);var o;e.TreeView=function({model:t,onSelect:e,onCollapse:n,preview:a,render:s=(t=>t.key)}){return i.renderOnEmit(t.onUpdated),r.createElement("div",{className:"m-TreeView"},r.createElement(o.TreeLevel,null,t.nodes.map(i=>r.createElement(o.TreeNode,Object.assign({node:i,key:"TreeNode"+i.key},{model:t,onSelect:e,render:s,preview:a,onCollapse:n})))))},function(t){function e({isCollapsed:t,onCollapse:e}){return r.createElement("button",{className:"m-FlippyTriangle",onClick:()=>e.call(void 0,!t),"data-collapsed":""+t})}t.FlippyTriangle=e,t.TreeLevel=({children:t})=>r.createElement("ul",{className:"m-TreeView-Level"},t),t.TreeNode=function n({model:i,node:o,onCollapse:a,onSelect:s,render:A,preview:l}){const c=o.children.length>0;return r.createElement("li",{key:o.key,className:"m-TreeView-Node","data-children":o.children.length,"data-key":o.key,"data-preview":l&&l.key===o.key?l.action:null,"data-preview-validity":l&&l.key===o.key?l.validity:null},r.createElement("span",{tabIndex:o.selectable?-1:void 0,className:"m-TreeView-Node-row"+(o.isSelected?" m-selected":""),onFocus:t=>{o.selectable&&s.call(void 0,o.key)}},c?r.createElement(e,{isCollapsed:o.isCollapsed,onCollapse:t=>{a.call(void 0,o.key,t)}}):null,r.createElement("span",{className:"m-TreeView-Node-content"},A.call(void 0,o))),!o.isCollapsed&&c?r.createElement(t.TreeLevel,null,o.children.map(t=>{if(!l||l.source!==t.key)return r.createElement(n,Object.assign({key:"TreeLevel"+t.key,node:t},{model:i,onSelect:s,onCollapse:a,render:A,preview:l}))})):null)}}(o||(o={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});class r{constructor(t,e={}){this.model=t;const n=Object.assign({},r.DefaultOptions,e),{allowOrdering:i,...o}=n;this.allowOrdering=i,this.callbacks=o}async canBeginAction(t){return await this.executeCallback("canMove",[t])}acceptsChildren(t){return this.executeCallbackSync("canAcceptChildren",[t])}async canPerformAction(t,e,n){switch(n){case"add-new-child":case"insert-child-into":return!!await this.acceptsChildren(e)&&await this.executeCallback("canInsertInto",[e,t]);case"insert-sibling-above":case"insert-sibling-below":const r=e.parent;return null!=r&&await this.executeCallback("canInsertInto",[r,t]);case"reorder-child":return this.allowOrdering}}async executeAction(t,e,n,r){this.performAction(t,e,n,r),this.executeCallback("onAfterMove",[t,e,n,r])}performAction(t,e,n,r){const i=this.getReferenceIndex(t,e);switch(n){case"add-new-child":this.model.addNewNode(t,e.key,r?r.index:void 0);break;case"insert-child-into":this.model.moveNode(t,e.key,0);break;case"insert-sibling-above":this.model.moveNode(t,e.parent.key,i);break;case"insert-sibling-below":this.model.moveNode(t,e.parent.key,i+1);break;case"reorder-child":if(!this.allowOrdering)break;this.model.moveNode(t,t.parent.key,r.index)}}getReferenceIndex(t,e){const{parent:n}=e;if(!this.allowOrdering||null==n)return 0;let r=n.children.indexOf(e);const i=this.model.get(t.key);return null==i?r:(i.parent===n&&n.children.indexOf(i)<r&&(r-=1),r)}async executeCallback(t,e){return await this.executeCallbackSync(t,e)}executeCallbackSync(t,e){try{const n=this.callbacks[t].bind(void 0)(...e);return n instanceof Promise?n.catch(e=>this.handleCallbackError(t,e)):n}catch(e){return this.handleCallbackError(t,e)}}handleCallbackError(t,e){return console.warn("Callback "+t+" threw: "),console.warn(e),!1}}r.DefaultOptions={allowOrdering:!1,canMove:()=>!0,canAcceptChildren:()=>!0,canInsertInto:()=>!0,onAfterMove:()=>void 0},e.TreeController=r},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9),i=n(637),o=new r.Token("@mavenomics/ui:ITypeEditorFactory");e.ITypeEditorFactory=o,e.typeEditorFactoryPlugin={id:"@mavenomics/ui:type-editor-factory",autoStart:!0,provides:o,activate:t=>i.TypeEditorFactory.Create()}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(1183);class i{constructor(){this.registry=new Map,this.detailRegistry=new Map,this.suppressedDetailEditors=new Set}static Create(){return null!=this.Instance?this.Instance:this.Instance=new i}getEditor(t){return this.registry.has(t)?this.registry.get(t):r.FallbackEditor}isDetailEditorSuppressed(t){return this.suppressedDetailEditors.has(t)}getDetailEditor(t){return this.detailRegistry.has(t)?this.detailRegistry.get(t):r.FallbackDetailEditor}registerEditor(t,e){this.registry.has(t)&&(console.warn("Warning: Inline type editor already registered for type "+t.serializableName),console.warn("Overwriting with "+e.name)),this.registry.set(t,e)}registerDetailEditor(t,e){this.detailRegistry.has(t)&&(console.warn("Warning: Detail type editor already registered for type "+t.serializableName),console.warn("Overwriting with "+e.name)),this.detailRegistry.set(t,e)}suppressDetailEditor(t){this.suppressedDetailEditors.add(t)}}e.TypeEditorFactory=i},function(t,e,n){t.exports=!n(112)&&!n(145)(function(){return 7!=Object.defineProperty(n(639)("div"),"a",{get:function(){return 7}}).a})},function(t,e,n){var r=n(144),i=n(110).document,o=r(i)&&r(i.createElement);t.exports=function(t){return o?i.createElement(t):{}}},function(t,e,n){var r=n(121),i=n(122),o=n(1192)(!1),a=n(320)("IE_PROTO");t.exports=function(t,e){var n,s=i(t),A=0,l=[];for(n in s)n!=a&&r(s,n)&&l.push(n);for(;e.length>A;)r(s,n=e[A++])&&(~o(l,n)||l.push(n));return l}},function(t,e,n){var r=n(317);t.exports=Object("z").propertyIsEnumerable(0)?Object:function(t){return"String"==r(t)?t.split(""):Object(t)}},function(t,e,n){var r=n(319),i=Math.min;t.exports=function(t){return t>0?i(r(t),9007199254740991):0}},function(t,e,n){"use strict";var r=n(1196)(!0);n(644)(String,"String",function(t){this._t=String(t),this._i=0},function(){var t,e=this._t,n=this._i;return n>=e.length?{value:void 0,done:!0}:(t=r(e,n),this._i+=t.length,{value:t,done:!1})})},function(t,e,n){"use strict";var r=n(224),i=n(109),o=n(645),a=n(142),s=n(180),A=n(1197),l=n(325),c=n(646),u=n(88)("iterator"),g=!([].keys&&"next"in[].keys()),h=function(){return this};t.exports=function(t,e,n,d,f,p,I){A(n,e,d);var C,m,y,E=function(t){if(!g&&t in Q)return Q[t];switch(t){case"keys":case"values":return function(){return new n(this,t)}}return function(){return new n(this,t)}},v=e+" Iterator",B="values"==f,b=!1,Q=t.prototype,w=Q[u]||Q["@@iterator"]||f&&Q[f],x=w||E(f),_=f?B?E("entries"):x:void 0,D="Array"==e&&Q.entries||w;if(D&&(y=c(D.call(new t)))!==Object.prototype&&y.next&&(l(y,v,!0),r||"function"==typeof y[u]||a(y,u,h)),B&&w&&"values"!==w.name&&(b=!0,x=function(){return w.call(this)}),r&&!I||!g&&!b&&Q[u]||a(Q,u,x),s[e]=x,s[v]=h,f)if(C={values:B?x:E("values"),keys:p?x:E("keys"),entries:_},I)for(m in C)m in Q||o(Q,m,C[m]);else i(i.P+i.F*(g||b),e,C);return C}},function(t,e,n){t.exports=n(142)},function(t,e,n){var r=n(121),i=n(179),o=n(320)("IE_PROTO"),a=Object.prototype;t.exports=Object.getPrototypeOf||function(t){return t=i(t),r(t,o)?t[o]:"function"==typeof t.constructor&&t instanceof t.constructor?t.constructor.prototype:t instanceof Object?a:null}},function(t,e,n){var r=n(640),i=n(322).concat("length","prototype");e.f=Object.getOwnPropertyNames||function(t){return r(t,i)}},function(t,e,n){"use strict";e.__esModule=!0;var r=d(n(18)),i=d(n(14)),o=d(n(19)),a=d(n(0)),s=d(n(16)),A=d(n(1)),l=n(67),c=d(n(1221)),u=d(n(1222)),g=d(n(1223)),h=d(n(116));function d(t){return t&&t.__esModule?t:{default:t}}function f(){}var p=function(t){function e(n){(0,r.default)(this,e);var o=(0,i.default)(this,t.call(this,n));I.call(o);var a=void 0;a="open"in n?n.open:n.defaultOpen;var s=n.value||n.defaultValue;return o.saveCalendarRef=function(t,e){this[t]=e}.bind(o,"calendarInstance"),o.state={open:a,value:s},o}return(0,o.default)(e,t),e.prototype.componentDidUpdate=function(t,e){!e.open&&this.state.open&&(this.focusTimeout=setTimeout(this.focusCalendar,0,this))},e.prototype.componentWillUnmount=function(){clearTimeout(this.focusTimeout)},e.getDerivedStateFromProps=function(t){var e={},n=t.value,r=t.open;return"value"in t&&(e.value=n),void 0!==r&&(e.open=r),e},e.prototype.render=function(){var t=this.props,e=t.prefixCls,n=t.placement,r=t.style,i=t.getCalendarContainer,o=t.align,s=t.animation,A=t.disabled,l=t.dropdownClassName,c=t.transitionName,u=t.children,d=this.state;return a.default.createElement(h.default,{popup:this.getCalendarElement(),popupAlign:o,builtinPlacements:g.default,popupPlacement:n,action:A&&!d.open?[]:["click"],destroyPopupOnHide:!0,getPopupContainer:i,popupStyle:r,popupAnimation:s,popupTransitionName:c,popupVisible:d.open,onPopupVisibleChange:this.onVisibleChange,prefixCls:e,popupClassName:l},a.default.cloneElement(u(d,t),{onKeyDown:this.onKeyDown}))},e}(a.default.Component);p.propTypes={animation:A.default.oneOfType([A.default.func,A.default.string]),disabled:A.default.bool,transitionName:A.default.string,onChange:A.default.func,onOpenChange:A.default.func,children:A.default.func,getCalendarContainer:A.default.func,calendar:A.default.element,style:A.default.object,open:A.default.bool,defaultOpen:A.default.bool,prefixCls:A.default.string,placement:A.default.any,value:A.default.oneOfType([A.default.object,A.default.array]),defaultValue:A.default.oneOfType([A.default.object,A.default.array]),align:A.default.object,dateRender:A.default.func,onBlur:A.default.func},p.defaultProps={prefixCls:"rc-calendar-picker",style:{},align:{},placement:"bottomLeft",defaultOpen:!1,onChange:f,onOpenChange:f,onBlur:f};var I=function(){var t=this;this.onCalendarKeyDown=function(e){e.keyCode===u.default.ESC&&(e.stopPropagation(),t.close(t.focus))},this.onCalendarSelect=function(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=t.props;"value"in r||t.setState({value:e}),("keyboard"===n.source||"dateInputSelect"===n.source||!r.calendar.props.timePicker&&"dateInput"!==n.source||"todayButton"===n.source)&&t.close(t.focus),r.onChange(e)},this.onKeyDown=function(e){t.state.open||e.keyCode!==u.default.DOWN&&e.keyCode!==u.default.ENTER||(t.open(),e.preventDefault())},this.onCalendarOk=function(){t.close(t.focus)},this.onCalendarClear=function(){t.close(t.focus)},this.onCalendarBlur=function(){t.setOpen(!1)},this.onVisibleChange=function(e){t.setOpen(e)},this.getCalendarElement=function(){var e=t.props,n=t.state,r=e.calendar.props,i=n.value,o=i,s={ref:t.saveCalendarRef,defaultValue:o||r.defaultValue,selectedValue:i,onKeyDown:t.onCalendarKeyDown,onOk:(0,c.default)(r.onOk,t.onCalendarOk),onSelect:(0,c.default)(r.onSelect,t.onCalendarSelect),onClear:(0,c.default)(r.onClear,t.onCalendarClear),onBlur:(0,c.default)(r.onBlur,t.onCalendarBlur)};return a.default.cloneElement(e.calendar,s)},this.setOpen=function(e,n){var r=t.props.onOpenChange;t.state.open!==e&&("open"in t.props||t.setState({open:e},n),r(e))},this.open=function(e){t.setOpen(!0,e)},this.close=function(e){t.setOpen(!1,e)},this.focus=function(){t.state.open||s.default.findDOMNode(t).focus()},this.focusCalendar=function(){t.state.open&&t.calendarInstance&&t.calendarInstance.focus()}};(0,l.polyfill)(p),e.default=p,t.exports=e.default},function(t,e,n){t.exports={default:n(1226),__esModule:!0}},function(t,e){t.exports=function(t,e){if(t.indexOf)return t.indexOf(e);for(var n=0;n<t.length;++n)if(t[n]===e)return n;return-1}},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r,i,o,a,s,A,l,c,u,g=G(n(0)),h=n(16),d=G(n(1)),f=G(n(1229)),p=G(n(4)),I=n(329),C=n(67),m=G(n(654)),y=G(n(655)),E=G(n(656)),v=G(n(657)),B=G(n(658)),b=G(n(668)),Q=G(n(1251)),w=G(n(1252)),x=n(335),_=O(n(230)),D=O(n(334)),M=G(n(671)),k=O(n(146)),S=G(n(333)),L=G(n(672)),R=G(n(673)),N=n(336),F=n(231),T=n(181),U=n(674);function O(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}return e.default=t,e}function G(t){return t&&t.__esModule?t:{default:t}}function H(){return(H=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}function j(t,e,n,r){n&&Object.defineProperty(t,e,{enumerable:n.enumerable,configurable:n.configurable,writable:n.writable,value:n.initializer?n.initializer.call(r):void 0})}function Y(t){if(void 0===t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return t}function P(t,e,n,r,i){var o={};return Object.keys(r).forEach(function(t){o[t]=r[t]}),o.enumerable=!!o.enumerable,o.configurable=!!o.configurable,("value"in o||o.initializer)&&(o.writable=!0),o=n.slice().reverse().reduce(function(n,r){return r(t,e,n)||n},o),i&&void 0!==o.initializer&&(o.value=o.initializer?o.initializer.call(i):void 0,o.initializer=void 0),void 0===o.initializer&&(Object.defineProperty(t,e,o),o=null),o}var K={},z=(0,C.polyfill)((u=c=function(t){var e,n;function r(){for(var e,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(e=t.call.apply(t,[this].concat(r))||this).handleFocusChanged=function(t){t||e.close()},j(e,"handleSelect",o,Y(Y(e))),j(e,"handleCreate",a,Y(Y(e))),j(e,"handleClick",s,Y(Y(e))),j(e,"handleKeyDown",A,Y(Y(e))),j(e,"handleKeyPress",l,Y(Y(e))),e.handleInputChange=function(t){e.search(t.target.value,t,"input")},e.handleAutofillChange=function(t){var n=e.props.data,r=t.target.value.toLowerCase();if(""!==r){var i=n,o=Array.isArray(i),a=0;for(i=o?i:i[Symbol.iterator]();;){var s;if(o){if(a>=i.length)break;s=i[a++]}else{if((a=i.next()).done)break;s=a.value}var A=s,l=(0,T.dataValue)(A);if(String(l).toLowerCase()===r||(0,T.dataText)(A).toLowerCase()===r){e.change(A,t);break}}}else e.change(null)},e.handleAutofill=function(t){e.setState({autofilling:t})},e.attachInputRef=function(t){return e.inputRef=t},e.attachFilterRef=function(t){return e.filterRef=t},e.attachListRef=function(t){return e.listRef=t},e.focus=function(t){var n=e.props,r=n.filter,i=n.open,o=t||(r&&i?e.filterRef:e.inputRef);(o=(0,h.findDOMNode)(o))&&(0,f.default)()!==o&&o.focus()},(0,I.autoFocus)(Y(Y(e))),e.inputId=(0,F.instanceId)(Y(Y(e)),"_input"),e.listId=(0,F.instanceId)(Y(Y(e)),"_listbox"),e.activeId=(0,F.instanceId)(Y(Y(e)),"_listbox_active_option"),e.mounted=(0,I.mountManager)(Y(Y(e))),e.timeouts=(0,I.timeoutManager)(Y(Y(e))),e.handleScroll=(0,R.default)(Y(Y(e))),e.focusManager=(0,M.default)(Y(Y(e)),{didHandle:e.handleFocusChanged}),e.state={},e}n=t,(e=r).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n,r.getDerivedStateFromProps=function(t,e){var n=t.open,r=t.value,i=t.data,o=t.messages,a=t.searchTerm,s=t.filter,A=t.minLength,l=t.caseSensitive,c=e.focusedItem,u=(0,L.default)(t),g=r!==e.lastValue,h=g&&u.indexOf(i,r);n&&(i=D.filter(i,{filter:s,searchTerm:a,minLength:A,caseSensitive:l,textField:u.text}));var d=(0,S.default)(i,e.list,{nextProps:t}),f=i[h],p=~i.indexOf(c)?c:i[0];return{data:i,list:d,accessors:u,lastValue:r,messages:(0,x.getMessages)(o),selectedItem:g?d.nextEnabled(f):e.selectedItem,focusedItem:g||void 0===c?d.nextEnabled(void 0!==f?f:p):p}};var i=r.prototype;return i.change=function(t,e){var n=this.props,r=n.onChange,i=n.searchTerm,o=n.value;this.state.accessors.matches(t,o)||((0,F.notify)(r,[t,{originalEvent:e,lastValue:o,searchTerm:i}]),this.clearSearch(e),this.close())},i.renderList=function(){var t=this.props,e=t.open,n=t.filter,r=t.data,i=t.searchTerm,o=t.searchIcon,a=t.optionComponent,s=t.itemComponent,A=t.groupComponent,l=t.listProps,c=this.state,u=c.list,h=c.accessors,d=c.focusedItem,f=c.selectedItem,p=c.messages,I=c.data,C=this.props.listComponent;return g.default.createElement("div",null,n&&g.default.createElement(E.default,{className:"rw-filter-input rw-input"},g.default.createElement("input",{value:i,className:"rw-input-reset",onChange:this.handleInputChange,placeholder:p.filterPlaceholder(this.props),ref:this.attachFilterRef}),g.default.createElement(v.default,{icon:o,role:"presentation","aria-hidden":"true"})),g.default.createElement(C,H({},l,{id:this.listId,activeId:this.activeId,data:I,dataState:u.dataState,isDisabled:u.isDisabled,searchTerm:i,textAccessor:h.text,valueAccessor:h.value,itemComponent:s,groupComponent:A,optionComponent:a,selectedItem:f,focusedItem:e?d:null,onSelect:this.handleSelect,onMove:this.handleScroll,"aria-live":e&&"polite","aria-labelledby":this.inputId,"aria-hidden":!this.props.open,ref:this.attachListRef,messages:{emptyList:r.length?p.emptyFilter:p.emptyList}})),this.allowCreate()&&g.default.createElement(Q.default,{id:this.createId,searchTerm:i,onSelect:this.handleCreate,focused:!d||d===K},p.createOption(this.props)))},i.render=function(){var t=this,e=this.props,n=e.className,r=e.tabIndex,i=e.popupTransition,o=e.textField,a=e.data,s=e.busy,A=e.dropUp,l=e.placeholder,c=e.value,u=e.open,h=e.isRtl,d=e.filter,f=e.inputProps,I=e.selectIcon,C=e.busySpinner,m=e.containerClassName,b=e.valueComponent,Q=this.state,x=Q.focused,D=Q.accessors,M=Q.messages,k=Q.autofilling,S=!0===this.props.disabled,L=!0===this.props.readOnly,R=D.findOrSelf(a,c),N=(0,F.isFirstFocusedRender)(this),T=H(_.pickElementProps(this),{name:void 0,role:"combobox",id:this.inputId,tabIndex:u&&d?-1:r||0,"aria-owns":this.listId,"aria-activedescendant":u?this.activeId:null,"aria-expanded":!!u,"aria-haspopup":!0,"aria-busy":!!s,"aria-live":!u&&"polite","aria-autocomplete":"list","aria-disabled":S,"aria-readonly":L});return g.default.createElement(y.default,H({},T,{open:u,isRtl:h,dropUp:A,focused:x,disabled:S,readOnly:L,autofilling:k,onBlur:this.focusManager.handleBlur,onFocus:this.focusManager.handleFocus,onKeyDown:this.handleKeyDown,onKeyPress:this.handleKeyPress,className:(0,p.default)(n,"rw-dropdown-list"),ref:this.attachInputRef}),g.default.createElement(E.default,{onClick:this.handleClick,className:(0,p.default)(m,"rw-widget-input")},g.default.createElement(w.default,H({},f,{value:R,textField:o,name:this.props.name,autoComplete:this.props.autoComplete,onAutofill:this.handleAutofill,onAutofillChange:this.handleAutofillChange,placeholder:l,valueComponent:b})),g.default.createElement(v.default,{busy:s,icon:I,spinner:C,role:"presentational","aria-hidden":"true",disabled:S||L,label:M.openDropdown(this.props)})),N&&g.default.createElement(B.default,{open:u,dropUp:A,transition:i,onEntered:function(){return t.focus()},onEntering:function(){return t.listRef.forceUpdate()}},this.renderList(M)))},i.findOption=function(t,e){var n=this,r=((this._currentWord||"")+t).toLowerCase();t&&(this._currentWord=r,this.timeouts.set("search",function(){var t=n.state.list,i=n.props.open?"focusedItem":"selectedItem",o=t.next(n.state[i],r);o===n.state[i]&&(o=t.next(null,r)),n._currentWord="",o&&e(o)},this.props.delay))},i.clearSearch=function(t){this.search("",t,"clear")},i.search=function(t,e,n){void 0===n&&(n="input");var r=this.props,i=r.onSearch,o=r.searchTerm;t!==o&&(0,F.notify)(i,[t,{action:n,lastSearchTerm:o,originalEvent:e}])},i.open=function(){this.props.open||(0,F.notify)(this.props.onToggle,!0)},i.close=function(){this.props.open&&(0,F.notify)(this.props.onToggle,!1)},i.toggle=function(){this.props.open?this.close():this.open()},i.allowCreate=function(){var t=this.props,e=t.searchTerm,n=t.onCreate,r=t.allowCreate;return!(!n||!(!0===r||"onFilter"===r&&e)||this.hasExtactMatch())},i.hasExtactMatch=function(){var t=this.props,e=t.searchTerm,n=t.caseSensitive,r=t.filter,i=this.state,o=i.data,a=i.accessors,s=function(t){return n?t:t.toLowerCase()};return r&&o.some(function(t){return s(a.text(t))===s(e)})},r}(g.default.Component),c.propTypes=H({},D.propTypes,{value:d.default.any,onChange:d.default.func,open:d.default.bool,onToggle:d.default.func,data:d.default.array,valueField:k.accessor,textField:k.accessor,allowCreate:d.default.oneOf([!0,!1,"onFilter"]),valueComponent:k.elementType,itemComponent:k.elementType,listComponent:k.elementType,optionComponent:k.elementType,groupComponent:k.elementType,groupBy:k.accessor,onSelect:d.default.func,onCreate:d.default.func,onSearch:d.default.func,searchTerm:d.default.string,busy:d.default.bool,selectIcon:d.default.node,searchIcon:d.default.node,busySpinner:d.default.node,placeholder:d.default.string,dropUp:d.default.bool,popupTransition:k.elementType,disabled:k.disabled.acceptsArray,readOnly:k.disabled,containerClassName:d.default.string,inputProps:d.default.object,listProps:d.default.object,isRtl:d.default.bool,messages:d.default.shape({open:d.default.string,emptyList:k.message,emptyFilter:k.message,filterPlaceholder:d.default.string,createOption:k.message})}),c.defaultProps={data:[],delay:500,searchTerm:"",allowCreate:!1,searchIcon:U.search,selectIcon:U.caretDown,listComponent:b.default},o=P((i=u).prototype,"handleSelect",[N.widgetEditable],{enumerable:!0,initializer:function(){var t=this;return function(e,n){void 0!==e&&e!==K?((0,F.notify)(t.props.onSelect,[e,{originalEvent:n}]),t.change(e,n),t.close(),t.focus(t)):t.handleCreate(t.props.searchTerm)}}}),a=P(i.prototype,"handleCreate",[N.widgetEditable],{enumerable:!0,initializer:function(){var t=this;return function(e,n){void 0===e&&(e=""),(0,F.notify)(t.props.onCreate,e),t.clearSearch(n),t.close(),t.focus(t)}}}),s=P(i.prototype,"handleClick",[N.widgetEditable],{enumerable:!0,initializer:function(){var t=this;return function(e){t.focus(),t.toggle(),(0,F.notify)(t.props.onClick,e)}}}),A=P(i.prototype,"handleKeyDown",[N.widgetEditable],{enumerable:!0,initializer:function(){var t=this;return function(e){var n=e.key,r=e.altKey,i=e.ctrlKey,o=t.props,a=o.open,s=o.onKeyDown,A=o.filter,l=o.searchTerm,c=t.state,u=c.focusedItem,g=c.selectedItem,d=c.list,f=u===K,p=t.allowCreate();(0,F.notify)(s,[e]);var I=function(){t.close(),(0,h.findDOMNode)(t).focus()},C=function(n){return null!=n&&t.change(n,e)},m=function(e){return t.setState({focusedItem:e})};if(!e.defaultPrevented)if("End"===n)e.preventDefault(),a?m(d.last()):C(d.last());else if("Home"===n)e.preventDefault(),a?m(d.first()):C(d.first());else if("Escape"===n&&a)e.preventDefault(),I();else if("Enter"===n&&a&&i&&p)e.preventDefault(),t.handleCreate(l,e);else if(("Enter"===n||" "===n&&!A)&&a)e.preventDefault(),t.handleSelect(u,e);else if(" "!==n||a){if("ArrowDown"===n){if(e.preventDefault(),r)return t.open();a||C(d.next(g));var y=d.next(u);m(f||p&&u===y?K:y)}else if("ArrowUp"===n){if(e.preventDefault(),r)return I();if(!a)return C(d.prev(g));m(f?d.last():d.prev(u))}}else e.preventDefault(),t.open()}}}),l=P(i.prototype,"handleKeyPress",[N.widgetEditable],{enumerable:!0,initializer:function(){var t=this;return function(e){(0,F.notify)(t.props.onKeyPress,[e]),e.defaultPrevented||t.props.filter&&t.props.open||t.findOption(String.fromCharCode(e.which),function(n){t.mounted()&&t.props.open?t.setState({focusedItem:n}):n&&t.change(n,e)})}}}),r=i))||r,J=(0,m.default)(z,{open:"onToggle",value:"onChange",searchTerm:"onSearch"},["focus"]);e.default=J,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t){return t&&t.ownerDocument||document},t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t){var e,n=(0,i.default)(t),o=Object.create(null);return(0,r.default)(t,{componentWillUnmount:function(){for(var t in o)clearTimeout(o[t]);o=null}}),e={clear:function(t){clearTimeout(o[t])},set:function(t,r,i){n()&&(e.clear(t),o[t]=setTimeout(r,i))}}};var r=o(n(227)),i=o(n(330));function o(t){return t&&t.__esModule?t:{default:t}}t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function t(e,n,o){void 0===o&&(o=[]);var s=e.displayName||e.name||"Component";var A=i.isReactComponent(e);var l=Object.keys(n);var c=l.map(i.defaultKey);!A&&o.length&&invariant(!1);var u=function(t){var o,s;function u(){for(var e,r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];return(e=t.call.apply(t,[this].concat(i))||this).handlers=Object.create(null),l.forEach(function(t){var r=n[t];e.handlers[r]=function(n){if(e.props[r]){var i;e._notifying=!0;for(var o=arguments.length,a=new Array(o>1?o-1:0),s=1;s<o;s++)a[s-1]=arguments[s];(i=e.props)[r].apply(i,[n].concat(a)),e._notifying=!1}e._values[t]=n,e.unmounted||e.forceUpdate()}}),A&&(e.attachRef=function(t){e.inner=t}),e}s=t,(o=u).prototype=Object.create(s.prototype),o.prototype.constructor=o,o.__proto__=s;var g=u.prototype;return g.shouldComponentUpdate=function(){return!this._notifying},g.componentWillMount=function(){var t=this,e=this.props;this._values=Object.create(null),l.forEach(function(n){t._values[n]=e[i.defaultKey(n)]})},g.componentWillReceiveProps=function(t){var e=this,n=this.props;l.forEach(function(r){!i.isProp(t,r)&&i.isProp(n,r)&&(e._values[r]=t[i.defaultKey(r)])})},g.componentWillUnmount=function(){this.unmounted=!0},g.getControlledInstance=function(){return this.inner},g.render=function(){var t=this,n=a({},this.props);c.forEach(function(t){delete n[t]});var i={};return l.forEach(function(e){var n=t.props[e];i[e]=void 0!==n?n:t._values[e]}),r.default.createElement(e,a({},n,i,this.handlers,{ref:this.attachRef}))},u}(r.default.Component);u.displayName="Uncontrolled("+s+")";u.propTypes=i.uncontrolledPropTypes(n,s);o.forEach(function(t){u.prototype[t]=function(){var e;return(e=this.inner)[t].apply(e,arguments)}});u.ControlledComponent=e;u.deferControlTo=function(e,r,i){return void 0===r&&(r={}),t(e,a({},n,r),i)};return u};var r=o(n(0)),i=(o(n(331)),function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}return e.default=t,e}(n(1233)));function o(t){return t&&t.__esModule?t:{default:t}}function a(){return(a=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=a(n(0)),i=a(n(1)),o=a(n(4));function a(t){return t&&t.__esModule?t:{default:t}}function s(){return(s=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var A=function(t){var e,n;function i(){return t.apply(this,arguments)||this}return n=t,(e=i).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n,i.prototype.render=function(){var t=this.props,e=t.className,n=t.tabIndex,i=t.focused,a=t.open,A=t.dropUp,l=t.disabled,c=t.readOnly,u=t.autofilling,g=t.isRtl,h=void 0===g?this.context.isRtl:g,d=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(r=0;r<a.length;r++)n=a[r],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(i[n]=t[n])}return i}(t,["className","tabIndex","focused","open","dropUp","disabled","readOnly","autofilling","isRtl"]);return n=null!=n?n:"-1",r.default.createElement("div",s({},d,{tabIndex:n,className:(0,o.default)(e,"rw-widget",h&&"rw-rtl",l&&"rw-state-disabled",c&&"rw-state-readonly",i&&"rw-state-focus",u&&"rw-webkit-autofill",a&&"rw-open"+(A?"-up":""))}))},i}(r.default.Component);A.contextTypes={isRtl:i.default.bool},A.propTypes={tabIndex:i.default.node,focused:i.default.bool,disabled:i.default.bool,readOnly:i.default.bool,autofilling:i.default.bool,open:i.default.bool,dropUp:i.default.bool,isRtl:i.default.bool};var l=A;e.default=l,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=a(n(0)),i=a(n(1)),o=a(n(4));function a(t){return t&&t.__esModule?t:{default:t}}function s(){return(s=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var A=function(t){var e,n;function i(){return t.apply(this,arguments)||this}return n=t,(e=i).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n,i.prototype.render=function(){var t=this.props,e=t.open,n=t.dropUp,i=t.className,a=t.disabled,A=t.readOnly,l=t.focused,c=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(r=0;r<a.length;r++)n=a[r],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(i[n]=t[n])}return i}(t,["open","dropUp","className","disabled","readOnly","focused"]),u="rw-open"+(n?"-up":"");return r.default.createElement("div",s({},c,{className:(0,o.default)(i,"rw-widget-picker","rw-widget-container",e&&u,a&&"rw-state-disabled",A&&"rw-state-readonly",l&&"rw-state-focus")}))},i}(r.default.Component);A.propTypes={tabIndex:i.default.node,focused:i.default.bool,disabled:i.default.bool,readOnly:i.default.bool,open:i.default.bool,dropUp:i.default.bool,picker:i.default.bool};var l=A;e.default=l,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=s(n(0)),i=s(n(1)),o=s(n(4)),a=s(n(1234));function s(t){return t&&t.__esModule?t:{default:t}}function A(){return(A=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var l=function(t){var e,n;function i(){return t.apply(this,arguments)||this}return n=t,(e=i).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n,i.prototype.render=function(){var t=this.props,e=t.className,n=t.bordered,i=t.children,s=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(r=0;r<a.length;r++)n=a[r],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(i[n]=t[n])}return i}(t,["className","bordered","children"]);return r.default.createElement("span",{className:(0,o.default)(e,"rw-select",n&&"rw-select-bordered")},i?r.default.Children.map(i,function(t){return t&&r.default.cloneElement(t,{variant:"select"})}):r.default.createElement(a.default,A({},s,{variant:"select"})))},i}(r.default.Component);l.propTypes={bordered:i.default.bool};var c=l;e.default=c,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=A(n(4)),i=A(n(1)),o=function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}return e.default=t,e}(n(0)),a=A(n(1235)),s=n(146);function A(t){return t&&t.__esModule?t:{default:t}}function l(){return(l=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}function c(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(r=0;r<a.length;r++)n=a[r],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(i[n]=t[n])}return i}function u(t,e){t.prototype=Object.create(e.prototype),t.prototype.constructor=t,t.__proto__=e}var g=function(t){function e(){return t.apply(this,arguments)||this}u(e,t);var n=e.prototype;return n.shouldComponentUpdate=function(t){return!!t.shouldUpdate},n.render=function(){var t=this.props,e=t.className,n=t.children,i=c(t,["className","children"]);return delete i.shouldUpdate,(0,o.cloneElement)(n,l({},i,{className:(0,r.default)(e,n.props.className,"rw-popup")}))},e}(o.default.Component);g.propTypes={shouldUpdate:function(){}};var h=function(t){function e(){return t.apply(this,arguments)||this}return u(e,t),e.prototype.render=function(){var t=this.props,e=t.className,n=t.dropUp,i=t.open,a=t.transition,s=c(t,["className","dropUp","open","transition"]);return o.default.createElement(a,l({},s,{in:i,dropUp:n,className:(0,r.default)(e,"rw-popup-container")}),o.default.createElement(g,{shouldUpdate:i},o.default.Children.only(this.props.children)))},e}(o.default.Component);h.defaultProps={open:!1,transition:a.default},h.propTypes={open:i.default.bool,dropUp:i.default.bool,onEntering:i.default.func,onEntered:i.default.func,transition:s.elementType};var d=h;e.default=d,t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=void 0;var i=function(){};r(n(123)).default&&(i=document.addEventListener?function(t,e,n,r){return t.addEventListener(e,n,r||!1)}:document.attachEvent?function(t,e,n){return t.attachEvent("on"+e,function(e){(e=e||window.event).target=e.target||e.srcElement,e.currentTarget=t,n.call(t,e)})}:void 0);var o=i;e.default=o,t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=void 0;var i=function(){};r(n(123)).default&&(i=document.addEventListener?function(t,e,n,r){return t.removeEventListener(e,n,r||!1)}:document.attachEvent?function(t,e,n){return t.detachEvent("on"+e,n)}:void 0);var o=i;e.default=o,t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=void 0;var i=r(n(123)).default?function(t,e){return t.contains?t.contains(e):t.compareDocumentPosition?t===e||!!(16&t.compareDocumentPosition(e)):o(t,e)}:o;function o(t,e){if(e)do{if(e===t)return!0}while(e=e.parentNode);return!1}e.default=i,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t,e){var n,o="#"===e[0],a="."===e[0],s=o||a?e.slice(1):e;if(r.test(s))return o?(t=t.getElementById?t:document,(n=t.getElementById(s))?[n]:[]):t.getElementsByClassName&&a?i(t.getElementsByClassName(s)):i(t.getElementsByTagName(e));return i(t.querySelectorAll(e))};var r=/^[\w-]*$/,i=Function.prototype.bind.call(Function.prototype.call,[].slice);t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t,e,n){var r="",c="",u=e;if("string"==typeof e){if(void 0===n)return t.style[(0,i.default)(e)]||(0,a.default)(t).getPropertyValue((0,o.default)(e));(u={})[e]=n}Object.keys(u).forEach(function(e){var n=u[e];n||0===n?(0,l.default)(e)?c+=e+"("+n+") ":r+=(0,o.default)(e)+": "+n+";":(0,s.default)(t,(0,o.default)(e))}),c&&(r+=A.transform+": "+c+";");t.style.cssText+=";"+r};var i=r(n(664)),o=r(n(1240)),a=r(n(1242)),s=r(n(1243)),A=n(665),l=r(n(1244));t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t){return(0,i.default)(t.replace(o,"ms-"))};var i=r(n(1239)),o=/^-ms-/;t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=e.animationEnd=e.animationDelay=e.animationTiming=e.animationDuration=e.animationName=e.transitionEnd=e.transitionDuration=e.transitionDelay=e.transitionTiming=e.transitionProperty=e.transform=void 0;var i,o,a,s,A,l,c,u,g,h,d,f=r(n(123)),p="transform";if(e.transform=p,e.animationEnd=a,e.transitionEnd=o,e.transitionDelay=c,e.transitionTiming=l,e.transitionDuration=A,e.transitionProperty=s,e.animationDelay=d,e.animationTiming=h,e.animationDuration=g,e.animationName=u,f.default){var I=function(){for(var t,e,n=document.createElement("div").style,r={O:function(t){return"o"+t.toLowerCase()},Moz:function(t){return t.toLowerCase()},Webkit:function(t){return"webkit"+t},ms:function(t){return"MS"+t}},i=Object.keys(r),o="",a=0;a<i.length;a++){var s=i[a];if(s+"TransitionProperty"in n){o="-"+s.toLowerCase(),t=r[s]("TransitionEnd"),e=r[s]("AnimationEnd");break}}!t&&"transitionProperty"in n&&(t="transitionend");!e&&"animationName"in n&&(e="animationend");return n=null,{animationEnd:e,transitionEnd:t,prefix:o}}();i=I.prefix,e.transitionEnd=o=I.transitionEnd,e.animationEnd=a=I.animationEnd,e.transform=p=i+"-"+p,e.transitionProperty=s=i+"-transition-property",e.transitionDuration=A=i+"-transition-duration",e.transitionDelay=c=i+"-transition-delay",e.transitionTiming=l=i+"-transition-timing-function",e.animationName=u=i+"-animation-name",e.animationDuration=g=i+"-animation-duration",e.animationTiming=h=i+"-animation-delay",e.animationDelay=d=i+"-animation-timing-function"}var C={transform:p,end:o,property:s,timing:l,delay:c,duration:A};e.default=C},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t){var e=(0,a.default)(t),n=(0,o.default)(e),r=e&&e.documentElement,s={top:0,left:0,height:0,width:0};if(!e)return;if(!(0,i.default)(r,t))return s;void 0!==t.getBoundingClientRect&&(s=t.getBoundingClientRect());return s={top:s.top+(n.pageYOffset||r.scrollTop)-(r.clientTop||0),left:s.left+(n.pageXOffset||r.scrollLeft)-(r.clientLeft||0),width:(null==s.width?t.offsetWidth:s.width)||0,height:(null==s.height?t.offsetHeight:s.height)||0}};var i=r(n(661)),o=r(n(228)),a=r(n(652));t.exports=e.default},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.default=function(t){function e(e,n,r,i,o,a){var s=i||"<<anonymous>>",A=a||r;if(null==n[r])return e?new Error("Required "+o+" `"+A+"` was not specified in `"+s+"`."):null;for(var l=arguments.length,c=Array(l>6?l-6:0),u=6;u<l;u++)c[u-6]=arguments[u];return t.apply(void 0,[n,r,s,o,A].concat(c))}var n=e.bind(null,!1);return n.isRequired=e.bind(null,!0),n},t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=f(n(1)),i=f(n(0)),o=n(16),a=d(n(146)),s=d(n(230)),A=n(231),l=n(333),c=f(n(669)),u=f(n(670)),g=f(n(1250)),h=n(335);function d(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}return e.default=t,e}function f(t){return t&&t.__esModule?t:{default:t}}function p(){return(p=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var I={data:r.default.array,dataState:r.default.shape({sortedKeys:r.default.array,groups:r.default.object,data:r.default.array,sequentialData:r.default.array}),valueAccessor:a.accessor,textAccessor:a.accessor,onSelect:r.default.func,onMove:r.default.func,activeId:r.default.string,itemComponent:a.elementType,groupComponent:a.elementType,optionComponent:a.elementType,renderItem:r.default.func,renderGroup:r.default.func,focusedItem:r.default.any,selectedItem:r.default.any,searchTerm:r.default.string,isDisabled:r.default.func.isRequired,messages:r.default.shape({emptyList:r.default.func.isRequired})},C={onSelect:function(){},data:[],dataState:{},optionComponent:u.default},m=function(t){var e,n;function r(){for(var e,n=arguments.length,r=new Array(n),o=0;o<n;o++)r[o]=arguments[o];return(e=t.call.apply(t,[this].concat(r))||this).renderItem=function(t){var n=t.item,r=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(r=0;r<a.length;r++)n=a[r],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(i[n]=t[n])}return i}(t,["item"]),o=e.props,a=o.isDisabled,s=o.renderItem,A=o.textAccessor,l=o.valueAccessor,c=e.props.itemComponent;return s?s(p({item:n},r)):c?i.default.createElement(c,p({item:n,value:l(n),text:A(n),disabled:a(n)},r)):A(n)},e.renderGroup=function(t){var n=e.props,r=n.renderGroup,o=n.groupComponent;return r?r({group:t}):o?i.default.createElement(o,{item:t}):t},e}n=t,(e=r).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n;var a=r.prototype;return a.componentDidMount=function(){this.move()},a.componentDidUpdate=function(){this.move()},a.mapItems=function(t){var e=this.props,n=e.data,r=e.dataState,i=r.sortedKeys,o=r.groups;if(!o)return n.map(function(e,n){return t(e,n,!1)});var a=-1;return i.reduce(function(e,n){var r=o[n];return e.concat(t(n,a,!0),r.map(function(e){return t(e,++a,!1)}))},[])},a.move=function(){var t=this.props,e=t.focusedItem,n=t.onMove,r=t.data,i=t.dataState,a=(0,o.findDOMNode)(this),s=function(t,e,n,r){var i=r.groups,o=r.sortedKeys;if(!i)return n.indexOf(t);var a=-1,s=-1;return o.some(function(e){var n=i[e].indexOf(t);if(a++,-1!==n)return s=a+n+1,!0;a+=i[e].length}),s}(e,0,r,i),l=a.children[s];l&&(0,A.notify)(n,[l,a,e])},a.renderOption=function(t,e){var n=this.props,r=n.activeId,o=n.focusedItem,a=n.selectedItem,s=n.onSelect,A=n.isDisabled,l=n.searchTerm,c=n.optionComponent,u=o===t;return i.default.createElement(c,{dataItem:t,key:"item_"+e,index:e,activeId:r,focused:u,onSelect:s,disabled:A(t),selected:a===t},this.renderItem({item:t,index:e,searchTerm:l}))},a.render=function(){var t=this,e=this.props,n=e.className,r=e.messages,o=s.pickElementProps(this),a=(0,h.getMessages)(r).emptyList;return i.default.createElement(c.default,p({},o,{className:n,emptyListMessage:a(this.props)}),this.mapItems(function(e,n,r){return r?i.default.createElement(g.default,{key:"group_"+e,group:e},t.renderGroup(e)):t.renderOption(e,n)}))},r}(i.default.Component);m.getDataState=l.defaultGetDataState,m.propTypes=I,m.defaultProps=C;var y=m;e.default=y,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=s(n(0)),i=s(n(1)),o=s(n(4)),a=n(231);function s(t){return t&&t.__esModule?t:{default:t}}function A(){return(A=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var l={className:i.default.string,role:i.default.string,nodeRef:i.default.func,emptyListMessage:i.default.node},c=function(t){var e,n;function i(){return t.apply(this,arguments)||this}return n=t,(e=i).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n,i.prototype.render=function(){var t=this.props,e=t.className,n=t.role,i=t.children,s=t.emptyListMessage,l=t.nodeRef,c=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(r=0;r<a.length;r++)n=a[r],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(i[n]=t[n])}return i}(t,["className","role","children","emptyListMessage","nodeRef"]),u=(0,a.instanceId)(this);return r.default.createElement("ul",A({id:u,tabIndex:"-1",ref:l,className:(0,o.default)(e,"rw-list"),role:void 0===n?"listbox":n},c),r.default.Children.count(i)?i:r.default.createElement("li",{className:"rw-list-empty"},s))},i}(r.default.Component);c.propTypes=l;var u=c;e.default=u,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=s(n(0)),i=s(n(1)),o=s(n(4)),a=function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}return e.default=t,e}(n(230));function s(t){return t&&t.__esModule?t:{default:t}}function A(){return(A=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var l=function(t){var e,n;function i(){for(var e,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(e=t.call.apply(t,[this].concat(r))||this).handleSelect=function(t){var n=e.props,r=n.onSelect,i=n.disabled,o=n.dataItem;r&&!i&&r(o,t)},e}return n=t,(e=i).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n,i.prototype.render=function(){var t=this.props,e=t.className,n=t.children,i=t.focused,s=t.selected,l=t.disabled,c=t.activeId,u=this.props.component||"li",g=a.omitOwn(this),h={"rw-state-focus":i,"rw-state-selected":s,"rw-state-disabled":l},d=i?c:void 0;return r.default.createElement(u,A({id:d,role:"option",tabIndex:l?void 0:"-1","aria-selected":!!s,className:(0,o.default)("rw-list-option",e,h),onClick:this.handleSelect},g),n)},i}(r.default.Component);l.propTypes={activeId:i.default.string,dataItem:i.default.any,index:i.default.number,focused:i.default.bool,selected:i.default.bool,disabled:i.default.bool,onSelect:i.default.func,component:i.default.string};var c=l;e.default=c,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t,e){var n=e.didHandle;return(0,r.focusManager)(t,o({},e,{onChange:function(e){t.setState({focused:e})},isDisabled:function(){return!0===t.props.disabled||(0,i.isInDisabledFieldset)(t)},didHandle:function(t,e){var r=this.props[t?"onFocus":"onBlur"];r&&r(e),n&&!e.isWidgetDefaultPrevented&&n(t,e)}}))};var r=n(329),i=n(336);function o(){return(o=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t){var e=t.textField,n=t.valueField;return{text:function(t){return r.dataText(t,e)},value:function(t){return r.dataValue(t,n)},indexOf:function(t,e){return r.dataIndexOf(t,e,n)},matches:function(t,e){return r.valueMatcher(t,e,n)},findOrSelf:function(t,e){return r.dataItem(t,e,n)},includes:function(t,e){return-1!==r.dataIndexOf(t,e,n)}}};var r=function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}return e.default=t,e}(n(181));t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t,e){void 0===e&&(e=function(t){return t.parentNode});var n,r,a,s=(0,o.mountManager)(t);return function(t,e,o){if(!s())return;var A,l=r,c=n;r=!(!e.offsetWidth||!e.offsetHeight),n=o,A=c!==o,(r&&!l||r&&A)&&(this.props.onMove?this.props.onMove(t,e,o):(a&&a(),a=(0,i.default)(t,!1)))}.bind(t)};var r,i=(r=n(1254))&&r.__esModule?r:{default:r},o=n(329);t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=e.search=e.clock=e.calendar=e.chevronLeft=e.chevronRight=e.caretDown=e.caretUp=void 0;var r=o(n(1)),i=o(n(0));function o(t){return t&&t.__esModule?t:{default:t}}var a={icon:r.default.string.isRequired},s=function(t){var e=t.icon;return i.default.createElement("span",{"aria-hidden":"true",className:"rw-i rw-i-"+e})};s.propTypes=a;var A=i.default.createElement(s,{icon:"caret-up"});e.caretUp=A;var l=i.default.createElement(s,{icon:"caret-down"});e.caretDown=l;var c=i.default.createElement(s,{icon:"chevron-right"});e.chevronRight=c;var u=i.default.createElement(s,{icon:"chevron-left"});e.chevronLeft=u;var g=i.default.createElement(s,{icon:"calendar"});e.calendar=g;var h=i.default.createElement(s,{icon:"clock-o"});e.clock=h;var d=i.default.createElement(s,{icon:"search"});e.search=d;var f=s;e.default=f},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.storeShape=void 0;var r,i=n(1),o=(r=i)&&r.__esModule?r:{default:r};e.storeShape=o.default.shape({subscribe:o.default.func.isRequired,setState:o.default.func.isRequired,getState:o.default.func.isRequired})},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(337),i=n(28),o=n(64);e.PartFactory=class{constructor(t){this.registry=new Map,this.parent=null,this._OnUpdatedSrc$=new o.Subject,this._isDisposed=!1,null!=t?(this.parent=t,this._OnUpdated=o.merge(this.parent.OnUpdated,this._OnUpdatedSrc$)):(this.parent=null,this._OnUpdated=this._OnUpdatedSrc$.asObservable())}get OnUpdated(){return this._OnUpdated}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this._OnUpdatedSrc$.complete(),this.registry.clear(),this.parent=null)}registerPart(t,e){e._FactoryName=t,this.registry.set(t,e),this._OnUpdatedSrc$.next(t)}unregisterPart(t){this.registry.delete(t)}createPart(t,e){if(!this.has(t))return new r.ErrorPart(e,t);let n;try{n=new(this.get(t))(e)}catch(i){console.warn("Failed to initalize",t),console.log(i),n=new r.ErrorPart(e,t)}return n}getMetadataForPart(t){return this.get(t).GetMetadata()}*[Symbol.iterator](){return null!=this.parent?yield*i.IterTools.merge(this.registry.entries(),this.parent):yield*this.registry.entries()}has(t){return this.registry.has(t)||null!=this.parent&&this.parent.has(t)}get(t){return!this.registry.has(t)&&this.parent?this.parent.get(t):this.registry.get(t)}keys(){return null!=this.parent?i.IterTools.merge(this.registry.keys(),this.parent.keys()):this.registry.keys()}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});e.Disposable=class{constructor(){this._isDisposed=!1}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this._isDisposed=!0,this.disposed())}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(679),i=n(28),o=n(9),a=n(50),s=n(677);var A;!function(t){t[t.Pending=0]="Pending",t[t.Running=1]="Running",t[t.Canceling=2]="Canceling",t[t.Completed=3]="Completed"}(A||(A={}));class l extends s.Disposable{constructor(t,e,n,r){super(),this.globals=t,this.maxWorkerCount=n,this.cancelTimeout=r,this.workers=[],this.tasks=[],this.evaluator=e||null,this.fillWorkerPool(),this.cleanupTimer=setInterval(this.cleanupWorkers.bind(this),l.CLEANUP_INTERVAL_MS)}cleanupWorkers(){let t=0;for(let e=0;e<this.workers.length;e++){let n=this.workers[e],r=(new Date).getTime()-n.lastUsed.getTime();if(!n.isBusy&&r>l.CLEANUP_KILL_AFTER_MS){if(t++<l.SOFT_WORKER_COUNT)continue;let r=this.tasks.find(t=>t.owner===n);r?this.killTask(r,!1):this.killWorker(n,!1),e--}}this.fillWorkerPool()}onMessage(t,e){const{type:n,id:r}=e;if("runEvalResult"!==e.type&&"runQueryResult"!==e.type||!r){if("KernelEvalRequest"===e.type&&r){const{taskId:o,code:a}=e;let s=this.getTask(o);if(!s)return void console.warn(`Received message for non-existing task. Id: ${r} Type: ${n}`);if(null==this.evaluator)return void t.worker.postMessage({type:"KernelEvalResult",id:r,error:i.serialize(new Error("KernelEval is currently disabled in this environment"))});this.evaluator.evaluate(a,s.globalNames).then(e=>{t.worker.postMessage({type:"KernelEvalResult",id:r,result:i.serialize(e)})}).catch(e=>{null!==e.prettyTraceback&&(e=new Error(e.message+"\n\n"+e.traceback)),t.worker.postMessage({type:"KernelEvalResult",id:r,error:i.serialize(e)})})}else if("FetchRequest"===e.type&&r){const{taskId:o,url:a}=e;let s=this.getTask(o);if(!s)return void console.warn(`Received message for non-existing task. Id: ${r} Type: ${n}`);let A=new AbortController,l=A.signal;s.token.onCancel(()=>{A.abort()}),fetch(a,{signal:l}).then(t=>{if(200!==t.status)throw new Error("Server response: "+t.statusText);return t.text()}).then(e=>{t.worker.postMessage({type:"FetchResult",id:r,result:i.serialize(e)})}).catch(e=>{t.worker.postMessage({type:"FetchResult",id:r,error:i.serialize(e)})})}else if("StaticCacheCmd"===e.type&&r){const{taskId:t,data:i}=e;if(!this.getTask(t))return void console.warn(`Received message for non-existing task. Id: ${r} Type: ${n}`);const o=()=>{let e=this.getTask(t);return e&&e.owner?e.owner.worker:null},a=e=>{let n=this.getTask(t);n&&n.owner?n.owner.cleanup.push(e):e.dispose()};l.StaticCache.handleMessage(a,o,r,i)}}else{let t=this.getTask(r);if(!t)return void console.warn(`Received message for non-existing task. Id: ${r} Type: ${n}`);if(e.isCanceled)return void this.onTaskCanceled(t);e.error?this.onTaskError(t,i.deserialize(e.error)):this.OnTaskCompleted(t,i.deserialize(e.result))}}async runMql(t,e,n){return this.startNewTask(t,n,t=>{t.owner.worker.postMessage({type:"runQuery",id:t.id,queryText:e,serializedGlobals:this.getSerializedGlobals(n)})})}async runJs(t,e,n){return this.startNewTask(t,n,t=>{t.owner.worker.postMessage({type:"runEval",id:t.id,codeText:e,serializedGlobals:this.getSerializedGlobals(n)})})}tryRunningTasks(){let t=this.workers.filter(t=>!t.isBusy);if(t.length>0){let e=this.tasks.filter(t=>t.state===A.Pending);for(let n of t){let t=e.shift();if(!t)break;n.isBusy=!0,t.owner=n,t.state=A.Running;try{t.start(t)}catch(e){this.onTaskError(t,e)}}}}cancelTasks(t){let e=this.tasks.filter(e=>e.groupId===t);for(let t of e)t.state===A.Running?(t.state=A.Canceling,t.token.cancel(),t.owner.worker.postMessage({type:"cancelRequest"}),this.cancelTimeout>0&&setTimeout(()=>{t.state===A.Canceling&&this.killTask(t,!0)},this.cancelTimeout)):t.state===A.Pending&&this.onTaskCanceled(t)}killTask(t,e){this.killWorker(t.owner,e),this.onTaskCanceled(t)}killWorker(t,e){t.cleanup.forEach(t=>t.dispose()),t.cleanup=[],t.worker.terminate(),this.workers=this.workers.filter(e=>e!==t),e&&this.fillWorkerPool()}disposed(){clearInterval(this.cleanupTimer);for(let t of this.tasks)this.killTask(t,!1);for(let t of this.workers)this.killWorker(t,!1)}getSerializedGlobals(t){const e={};for(const n of t)this.globals.has(n)?e[n]=i.serialize(this.globals.get(n)):console.warn("Unknown referenced global: @"+n);return e}startNewTask(t,e,n){let r,a;this.cancelTasks(t);let s=new Promise((t,e)=>{r=t,a=e}),l=new i.CancelToken,c={groupId:t,id:o.UUID.uuid4(),promise:s,resolve:r,reject:a,state:A.Pending,globalNames:e,token:l,owner:null,start:n};return this.tasks.push(c),this.fillWorkerPool(),this.tryRunningTasks(),c.promise}onTaskError(t,e){this.removeTask(t),t.reject(e)}onTaskCanceled(t){this.removeTask(t),t.reject(new i.CancelError("Task Canceled"))}OnTaskCompleted(t,e){this.removeTask(t),t.resolve(e)}getTask(t){return this.tasks.find(e=>e.id===t)}removeTask(t){t.state=A.Completed,t.owner&&(t.owner.isBusy=!1,t.owner.lastUsed=new Date),this.tasks=this.tasks.filter(e=>e.id!==t.id),this.tryRunningTasks()}fillWorkerPool(){if(this.isDisposed)return;const t=Math.min(this.tasks.length+l.SOFT_WORKER_COUNT,this.maxWorkerCount);for(;this.workers.length<t;){let t=new r.WorkerWrapper,e={worker:t,isBusy:!1,lastUsed:new Date,cleanup:[]};t.onMessage.subscribe(t=>this.onMessage(e,t)),this.workers.push(e)}}}l.StaticCache=new class{constructor(){this.cache=new i.Cache}sendMessage(t,e,n){const r=t();r&&r.postMessage({type:"StaticCacheCmdResult",id:e,result:n})}async handleMessage(t,e,n,r){switch(r.cmd){case"LOCK":{const{key:i,isWrite:o}=r,s=await this.cache.lock(i,!!o);return t(new a.DisposableDelegate(()=>{this.cache.unlock(i,s.lockId)})),s.isFree?void console.debug("Worker died while acquiring StaticCache lock"):void this.sendMessage(e,n,{type:"LOCK",lockId:s.lockId})}case"UNLOCK":{const{key:t,lockId:i}=r,o=this.cache.getLock(t,i);return o?o.release():console.warn("[StaticCache] Attempted to release an invalid lock"),void this.sendMessage(e,n,{type:"UNLOCK"})}case"GET":{const{key:t,blocking:i,lockId:o}=r;if(!this.cache.has(t))return this.sendMessage(e,n,{type:"MISS"});const a=this.cache.get(t,!!i,o);return void this.sendMessage(e,n,{type:"HIT",value:a})}case"PUT":{const{key:t,blocking:i,lockId:o,value:a}=r;return this.cache.put(t,a,!!i,o),void this.sendMessage(e,n,{type:"HIT"})}case"DELETE":{const{key:t,blocking:i,lockId:o}=r;return"*"===t?this.cache.clear():this.cache.delete(t,!!i,o),void this.sendMessage(e,n,{type:"HIT"})}default:return void this.sendMessage(e,n,{type:"UNKNOWN_CMD"})}}},l.CLEANUP_INTERVAL_MS=5e3,l.CLEANUP_KILL_AFTER_MS=3e4,l.SOFT_WORKER_COUNT=1,e.MqlWorkerPool=l},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(1284);e.WorkerWrapper=r.WorkerWrapper},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9),i=n(28),o=n(681),a=n(48);!function(t){t.toJson=function(t){const{partManager:e,externalParts:n,layoutManager:r,globals:s}=t,A={},l={};for(const[t,n]of e)A[t]={[a.PartSerializer.MIMETYPE]:a.PartSerializer.toJson(n,e.getBagById(t)),"text/plain":"VisualEditorPart"};if(null!=n)for(const t of n.getAllPartIds()){const[e,r]=n.serializePart(t);A[t]=e,l[t]=r}const c=[];for(const t of s){const e=t.name,n=t.type.serializableName,r=i.Converters.serialize(t.value,t.type);c.push({name:e,type:n,value:r&&r.value})}return{layout:o.LayoutSerializer.toJson(r.root),parts:A,metadata:l,globals:c,localParts:t.localParts}},t.DEFAULT_DASHBOARD={layout:{children:[],properties:{flexSize:1,caption:"Dashboard"},typeName:o.LayoutTypes.StackPanelLayoutRegion,uuid:r.UUID.uuid4()},metadata:{},parts:{},visual:!0},t.MAVEN_LAYOUT_MIME_TYPE="application/vnd.maven.layout+json"}(e.DashboardSerializer||(e.DashboardSerializer={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(232);e.DashboardLayoutRegion=r.DashboardLayoutRegion;var i=n(124);e.RegionWithChildren=i.RegionWithChildren;var o=n(339);e.WidgetLayoutRegion=o.WidgetLayoutRegion;var a=n(233);e.StackPanelLayoutRegion=a.StackPanelLayoutRegion;var s=n(340);e.TabPanelDashboardLayoutRegion=s.TabPanelDashboardLayoutRegion;var A=n(341);e.CanvasLayoutRegion=A.CanvasLayoutRegion;var l=n(684);e.LayoutActions=l.LayoutActions;var c=n(125);e.LayoutManager=c.LayoutManager,e.LayoutTypes=c.LayoutTypes;var u=n(682);e.LayoutSerializer=u.LayoutSerializer;var g=n(683);e.GridLayoutRegion=g.GridLayoutRegion},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(233),i=n(125),o=n(339),a=n(9),s=n(340),A=n(341),l=n(124),c=n(683),u=n(28);!function(t){t.toJson=function(e){const n={properties:{...e.properties},typeName:e.typeName,uuid:e.uuid};if(["backgroundColor","borderColor","captionBackground","captionColor"].forEach(t=>{t in e.properties&&(n.properties[t]=e.properties[t].color)}),e instanceof l.RegionWithChildren){const t=e.constructor.GetMetadata(),r=[];for(let n=0;n<e.widgets.length;n++){const i=e.widgets[n];r[n]={};for(let e of t.getAllAttachedPropertyNames()){const o=t.getAttachedProperty(e).get(i);r[n][e]=o}}n.attachedProperties=r}switch(e.typeName){case i.LayoutTypes.StackPanelLayoutRegion:return Object.assign(n,{children:e.widgets.map(e=>t.toJson(e))});case i.LayoutTypes.WidgetLayoutRegion:return Object.assign(n,{guid:e.guid});case i.LayoutTypes.TabPanelDashboardLayoutRegion:case i.LayoutTypes.CanvasLayoutRegion:case i.LayoutTypes.GridPanelLayoutRegion:return Object.assign(n,{children:e.widgets.map(e=>t.toJson(e))});default:throw Error("Unknown layout region type "+e.typeName)}},t.fromJson=function(e,n){e.hasOwnProperty("type")&&(e.typeName={StackPanelDashboardLayoutRegion:i.LayoutTypes.StackPanelLayoutRegion,WidgetDashboardLayoutRegion:i.LayoutTypes.WidgetLayoutRegion,TabPanelDashboardLayoutRegion:i.LayoutTypes.TabPanelDashboardLayoutRegion}[e.type]),e.hasOwnProperty("uuid")||(e.uuid=a.UUID.uuid4());const l={...e.properties};switch(["backgroundColor","borderColor","captionBackground","captionColor"].forEach(t=>{t in e.properties&&(l[t]=new u.Color(""+l[t]))}),e.typeName){case i.LayoutTypes.WidgetLayoutRegion:const a=e.guid,g=n.getPart(a);if(null==g)throw Error("Child not found for WidgetLayoutRegion: "+a);const h=new o.WidgetLayoutRegion(n,g,a,e.uuid);return"caption"in l||""===g.title.label||(l.caption=g.title.label),h.properties={...l},h;case i.LayoutTypes.StackPanelLayoutRegion:const d=new r.StackPanelLayoutRegion(n,e.uuid);d.properties={...l};for(let i=0;i<e.children.length;i++){const o=e.children[i],a=t.fromJson(o,n);let s,A;d.addChild(a),null==e.attachedProperties?(A=o.properties["fixedSize.px"]||null,s=o.properties.flexSize||null):(A=e.attachedProperties[i]["Fixed Size (px)"]||null,s=e.attachedProperties[i].Stretch||null,r.StackPanelLayoutRegion.IsExpanded.set(a,e.attachedProperties[i].Expanded)),null!=A?r.StackPanelLayoutRegion.FixedSize.set(a,A):r.StackPanelLayoutRegion.FlexSize.set(a,s||1)}return d;case i.LayoutTypes.TabPanelDashboardLayoutRegion:const f=new s.TabPanelDashboardLayoutRegion(n,e.uuid);f.properties={...l};for(const r of e.children)f.addChild(t.fromJson(r,n));return f.setLayoutProperty("ForegroundIndex",l.ForegroundIndex||0),f;case i.LayoutTypes.CanvasLayoutRegion:const p=new A.CanvasLayoutRegion(n,e.uuid),I=e;p.properties={...l};for(let r=0;r<I.children.length;r++){const i=I.children[r],o=t.fromJson(i,n);let a=null;null!=i.properties["canvas.rectangle"]?a=i.properties["canvas.rectangle"]:null!=e.attachedProperties&&(a=e.attachedProperties[r]["Canvas Panel.Rectangle"]||null),null!=a&&A.CanvasLayoutRegion.Rectangle.set(o,a),p.addChild(o)}return p;case i.LayoutTypes.GridPanelLayoutRegion:const C=new c.GridLayoutRegion(n,e.uuid);C.properties={...l};const m=e;for(let r=0;r<m.children.length;r++){const i=m.children[r],o=t.fromJson(i,n);e.attachedProperties&&(c.GridLayoutRegion.Column.set(o,e.attachedProperties[r]["Grid Panel.Column"]),c.GridLayoutRegion.ColSpan.set(o,e.attachedProperties[r]["Grid Panel.Column Span"]),c.GridLayoutRegion.Row.set(o,e.attachedProperties[r]["Grid Panel.Row"]),c.GridLayoutRegion.RowSpan.set(o,e.attachedProperties[r]["Grid Panel.Row Span"])),C.addChild(o)}return C;default:throw Error("Unsupported layout type: "+e.typeName)}}}(e.LayoutSerializer||(e.LayoutSerializer={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(6),i=n(43),o=n(124),a=n(28),s=n(125),A=n(25);class l extends o.RegionWithChildren{constructor(t,e){super(t,e),this.typeName=s.LayoutTypes.GridPanelLayoutRegion,this.addClass("m-GridPanel"),this.content=new r.Panel,this.content.addClass("m-GridPanel-panel"),this.installContentTap()}static GetMetadata(){const t=super.GetMetadata();return t.name="Grid Panel",t.iconText="grid_on",t.description="Arrange children in a grid",t.remarks="\nGrids are useful for things like forms, where you might have a set of labels and\ncontrols that should all be the same width. You can technically accomplish this\nwith nested stackpanels, but it would be very painful to make changes.",t.addMetadata("nRows",{prettyName:"Grid Panel.Number of Rows",default:3,documentation:"The number of rowss to add to the grid",type:a.Types.Number}),t.addMetadata("nCols",{prettyName:"Grid Panel.Number of Columns",default:3,documentation:"The number of columns to add to the grid",type:a.Types.Number}),t.addMetadata("spacing.px",{prettyName:"Grid Panel.Spacing between children",default:4,documentation:"How much padding to put between grid items, in pixels",type:a.Types.Number}),t.addAttachedMetadata(l.Column,{default:1,documentation:"The column of the grid to put this region into",type:a.Types.Number}),t.addAttachedMetadata(l.ColSpan,{default:1,documentation:"How many columns this region should stretch across",type:a.Types.Number}),t.addAttachedMetadata(l.Row,{default:1,documentation:"The row of the grid to put this region into",type:a.Types.Number}),t.addAttachedMetadata(l.RowSpan,{default:1,documentation:"How many rows this region should stretch across",type:a.Types.Number}),t}layoutChildren(){const t=Math.max(1,this.getLayoutProperty("nCols")),e=Math.max(1,this.getLayoutProperty("nRows")),n=this.getLayoutProperty("padding.px"),i=this.getLayoutProperty("spacing.px"),{width:o,height:s}=this.calculatedSize||this.content.node.getBoundingClientRect(),c=(o-2*n)/t-i+i/t,u=(s-2*n)/e-i+i/e;for(let n=0;n<this.widgets.length;n++){const o=this.widgets[n];if(!o.getLayoutProperty("showRegion"))continue;const s=a.MathTools.Clamp(l.Column.get(o),0,t),g=a.MathTools.Clamp(l.ColSpan.get(o),1,t-s),h=a.MathTools.Clamp(l.Row.get(o),0,e),d=a.MathTools.Clamp(l.RowSpan.get(o),1,e-h);o.calculatedSize={width:g*c+(g-1)*i,height:d*u+(d-1)*i,top:h*u+h*i,left:s*c+s*i},A.MessageLoop.sendMessage(o,r.Widget.Msg.FitRequest)}}}e.GridLayoutRegion=l,function(t){function e(t){null!=t.parentRegion&&(A.MessageLoop.sendMessage(t.parentRegion,r.Widget.Msg.FitRequest),A.MessageLoop.sendMessage(t.parentRegion,r.Widget.Msg.UpdateRequest)),t.setStale()}t.Column=new i.AttachedProperty({create:()=>1,name:"Grid Panel.Column",changed:e}),t.ColSpan=new i.AttachedProperty({create:()=>1,name:"Grid Panel.Column Span",changed:e}),t.Row=new i.AttachedProperty({create:()=>1,name:"Grid Panel.Row",changed:e}),t.RowSpan=new i.AttachedProperty({create:()=>1,name:"Grid Panel.Row Span",changed:e})}(l=e.GridLayoutRegion||(e.GridLayoutRegion={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(233),i=n(25),o=n(6),a=n(340),s=n(341);!function(t){function e({layoutManager:t,regionType:e,target:n,prunable:i}){const o=t.getRegion(n);if(null==o)throw Error("Target region not found in layout");const A=o.parentRegion;if(null==A)throw Error("Target region does not have a valid parent");const l=A.widgets.indexOf(o);let c;switch(e){case"Stack Panel":c=new r.StackPanelLayoutRegion(t);break;case"Tab Panel":c=new a.TabPanelDashboardLayoutRegion(t);break;case"Canvas":c=new s.CanvasLayoutRegion(t);break;default:throw Error("Unknown region type "+e)}return A.copyAttachedProperties(o,c),c.setLayoutProperty("prunable",i),c.setFresh(),c.addChild(o),A.insertChild(l,c),o.activate(),c.id}t.SurroundWith=e,t.MoveToRootZone=function({layoutManager:t,zone:e,target:n}){const a=t.getRegion(n);if(null==a)throw Error("Drag target invalid! No region with id "+n);const s=Array.from(t.root.widgets),A=new r.StackPanelLayoutRegion(t);A.properties={...t.root.properties},A.setLayoutProperty("prunable",!0),A.setStale(),a.setLayoutProperty("flexSize",1);for(const t of s)A.addChild(t);switch(e){case"far-outer-bottom":case"far-outer-right":t.root.addChild(A),t.root.addChild(a);break;case"far-outer-top":case"far-outer-left":t.root.addChild(a),t.root.addChild(A)}const l="far-outer-left"===e||"far-outer-right"===e;t.root.setLayoutProperty("horizontal",l),t.root.pruneSubtree(),i.MessageLoop.sendMessage(t,o.Widget.Msg.UpdateRequest)},t.MoveToRelativeZone=function({layoutManager:t,zone:n,target:i,reference:o}){const s=t.getRegion(i);let A=t.getRegion(o);if(null==s)throw Error("Region to move not found in layout");if(null==A)throw Error("Reference for relative move not found in layout");switch(n){case"center":let i;(i=A instanceof a.TabPanelDashboardLayoutRegion?A:A.parentRegion instanceof a.TabPanelDashboardLayoutRegion?A.parentRegion:t.getRegion(e({layoutManager:t,target:o,regionType:"Tab Panel",prunable:!0}))).addChild(s),i.setLayoutProperty("ForegroundIndex",i.widgets.length-1);break;default:!function(t,e,n){n.endsWith("right")||n.endsWith("bottom")?t.addChild(e):t.insertChild(0,e);const r=n.endsWith("left")||n.endsWith("right");t.setLayoutProperty("horizontal",r)}(t.getRegion(function(t,n,i,o){if(null==o)throw Error("Cannot wrap a non-existent panel");if(o instanceof t){let t=!0;for(const[e,r]of n)if(o.getLayoutProperty(e)!==r){t=!1;break}if(t)return o.id}return e({layoutManager:i,target:o.id,prunable:!0,regionType:t===r.StackPanelLayoutRegion?"Stack Panel":"Tab Panel"})}(r.StackPanelLayoutRegion,[["horizontal",n.endsWith("left")||n.endsWith("right")]],t,n.startsWith("inner")?A:A.parentRegion)),s,n)}t.root.pruneSubtree()}}(e.LayoutActions||(e.LayoutActions={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(342),o=n(686);e.ArrowConverter=class{static toArrow(t){const e=this.inferSchema(t),n=this.createColumnArrays(t,e),r=this.createArrowVectors(n,e);return new o.Table(e,[r])}static fromArrow(t){const e=t.schema,n=e.fields.length,o=new Array(n),a=new Array(n),s=new Array(n),l=new Set;for(let i=0;i<n;i++){const n=e.fields[i],c=n.metadata.get("mavenType")||"Any";let u=r.Types.findType(c)||r.Types.Any;u===r.Types.Any&&(u=A(e.fields[i].type)),o[i]=u,a[i]=n.name;let g=t.getColumnAt(i);if(null===g)throw Error("Unknown column in schema: "+n.name);"true"===n.metadata.get("isSerialized")&&l.add(i),s[i]=g}const c=new i.Table;c.setColumns(a,o);for(let e=0;e<t.length;e++){const t=c.createRow(null);for(let i=0;i<n;i++){let n=s[i].get(e);null!==n&&(n=n.valueOf()),l.has(i)&&(n=r.Converters.deserialize({typeName:o[i].serializableName,value:JSON.parse(n)})),t._data[i]=n}c.appendRow(t)}return c}static inferSchema(t){const e=new Array(t.columnNames.length);for(let n=0;n<t.columnNames.length;n++){let i=t.columnTypes[n];const s=new Map;i===r.Types.Any&&(i=r.Converters.inferType(t.rows[0].getValue(n)),console.warn("Inferring type for column",t.columnNames[n],"as",i.serializableName),console.warn("This may lead to unexpected serialization errors")),s.set("mavenType",i.serializableName);let A=a.get(i);null==A&&(console.warn("Cannot translate type",i.serializableName+". This may lead to performance penalties!"),A=o.Utf8,s.set("isSerialized","true"));let l=new A;e[n]=new o.Field(t.columnNames[n],l,!0,s)}return new o.Schema(e)}static createColumnArrays(t,e){const n=t.rows.length,i=t.columnNames.length,o=new Array(i),a=new Array(i);for(let t=0;t<i;t++){const i=l.get(e.fields[t].typeId)||e.fields[t].type.ArrayType;o[t]=new i(n);const s=e.fields[t].metadata.get("mavenType")||"Any";a[t]=r.Types.findType(s)||r.Types.Any}for(let s=0;s<n;s++){const n=t.rows[s];for(let t=0;t<i;t++){let i=n._data[t];if(e.fields[t].metadata.get("isSerialized")){const e=r.Converters.serialize(i,a[t]);i=JSON.stringify(e?e.value:null)}o[t][s]=i}}return o}static createArrowVectors(t,e){const n=new Array(t.length),r=new Array(t.length);for(let i=0;i<n.length;i++)n[i]=s(e.fields[i].type,t[i]),r[i]=t[i].length;return new o.RecordBatch(e,Math.max(...r),n)}};const a=Object.freeze(new Map([[r.Types.Boolean,o.Bool],[r.Types.Date,o.Date_],[r.Types.DateTime,o.Date_],[r.Types.Number,o.Float64],[r.Types.String,o.Utf8]]));function s(t,e){switch(t.typeId){case o.Type.Bool:return o.BoolVector.from(e);case o.Type.Date:return o.DateVector.from(e);case o.Type.Float:case o.Type.Float16:case o.Type.Float32:case o.Type.Float64:return o.FloatVector.from(e);case o.Type.Utf8:default:for(let t=0;t<e.length;t++)null==e[t]&&(e[t]="");return o.Utf8Vector.from(e)}}function A(t){switch(t.typeId){case o.Type.Bool:return r.Types.Boolean;case o.Type.Date:case o.Type.DateDay:case o.Type.DateMillisecond:return r.Types.Date;case o.Type.Decimal:case o.Type.Float:case o.Type.Float16:case o.Type.Float32:case o.Type.Float64:case o.Type.Int:case o.Type.Int8:case o.Type.Int16:case o.Type.Int32:case o.Type.Int64:case o.Type.Uint8:case o.Type.Uint16:case o.Type.Uint32:case o.Type.Uint64:return r.Types.Number;case o.Type.Bool:return r.Types.Boolean;case o.Type.Utf8:return r.Types.String;default:return r.Types.Any}}const l=Object.freeze(new Map([[o.Type.Utf8,Array]]))},function(t,e,n){(function(t,n){(function(e){var r,i,o=o||{};function a(t,e){this.Oc=t,o.defineProperty(this,"description",{configurable:!0,writable:!0,value:e})}function s(){this.Pa=!1,this.ja=null,this.m=void 0,this.i=1,this.aa=this.ma=0,this.Bc=this.Z=null}function A(t){if(t.Pa)throw new TypeError("Generator is already running");t.Pa=!0}function l(t,e,n){return t.i=n,{value:e}}function c(t,e,n){var r=(e=o.w(e)).next();if(o.C.zc(r),!r.done)return t.ja=e,l(t,r.value,n);t.m=r.value,t.i=n}function u(t,e){t.ma=0,t.aa=e||0}function g(t){t.ma=0;var e=t.Z.Ac;return t.Z=null,e}function h(t){t.Bc=[t.Z],t.ma=0,t.aa=0}function d(t){var e=t.Bc.splice(0)[0];(e=t.Z=t.Z||e)?e.Fc?t.i=t.ma||t.aa:null!=e.j&&t.aa<e.j?(t.i=e.j,t.Z=null):t.i=t.aa:t.i=0}function f(t){this.G=new s,this.He=t}function p(t,e){A(t.G);var n=t.G.ja;return n?I(t,"return"in n?n.return:function(t){return{value:t,done:!0}},e,t.G.return):(t.G.return(e),C(t))}function I(t,e,n,r){try{var i=e.call(t.G.ja,n);if(o.C.zc(i),!i.done)return t.G.Pa=!1,i;var a=i.value}catch(e){return t.G.ja=null,t.G.Ta(e),C(t)}return t.G.ja=null,r.call(t.G,a),C(t)}function C(t){for(;t.G.i;)try{var e=t.He(t.G);if(e)return t.G.Pa=!1,{value:e.value,done:!1}}catch(e){t.G.m=void 0,t.G.Ta(e)}if(t.G.Pa=!1,t.G.Z){if(e=t.G.Z,t.G.Z=null,e.Fc)throw e.Ac;return{value:e.return,done:!0}}return{value:void 0,done:!0}}function m(t){this.next=function(e){return t.Qa(e)},this.throw=function(e){return t.Ta(e)},this.return=function(e){return p(t,e)},o.K(),this[Symbol.iterator]=function(){return this}}function y(t){this[Symbol.asyncIterator]=function(){return this},this[Symbol.iterator]=function(){return t},this.next=function(e){return Promise.resolve(t.next(e))},void 0!==t.throw&&(this.throw=function(e){return Promise.resolve(t.throw(e))}),void 0!==t.return&&(this.return=function(e){return Promise.resolve(t.return(e))})}function E(t,e){this.action=t,this.value=e}function v(t,e,n,r){this.method=t,this.ra=e,this.resolve=n,this.reject=r}function B(t){this.frame=t,this.next=null}function b(){this.nb=this.ba=null}function Q(t){t.ba&&(t.ba=t.ba.next,t.ba||(t.nb=null))}function w(t){this.Dc=t,this.V=null,this.S=new b,o.L(),this[Symbol.asyncIterator]=function(){return this};var e=this;this.Jd=function(t){var n=e.S.first();!0===t.done?(e.V=null,n.method="next",n.ra=t.value):(n.resolve({value:t.value,done:!1}),Q(e.S)),_(e)},this.Id=function(t){M(e,t)},this.Nb=function(t){k(e,t)}}function x(t,e,n){return new Promise(function(r,i){var o=null===t.S.ba;t.S.enqueue(new v(e,n,r,i)),o&&_(t)})}function _(t){if(null!==t.S.ba)try{if(t.V){if(!t.V)throw Error("no delegate to perform execution");var e=t.S.first();if(e.method in t.V)try{t.V[e.method](e.ra).then(t.Jd,t.Id).catch(t.Nb)}catch(e){M(t,e)}else t.V=null,_(t)}else D(t)}catch(e){k(t,e)}}function D(t){var e=t.S.first();try{var n=t.Dc[e.method](e.ra);if(n.value instanceof E)switch(n.value.action){case 0:Promise.resolve(n.value.value).then(function(r){e.resolve({value:r,done:n.done}),Q(t.S),_(t)},function(n){e.reject(n),Q(t.S),_(t)}).catch(t.Nb);break;case 1:t.V=o.kb(n.value.value),e.method="next",e.ra=void 0,_(t);break;case 2:Promise.resolve(n.value.value).then(function(n){e.method="next",e.ra=n,_(t)},function(n){e.method="throw",e.ra=n,_(t)}).catch(t.Nb);break;default:throw Error("Unrecognized AsyncGeneratorWrapper$ActionEnum")}else e.resolve(n),Q(t.S),_(t)}catch(n){e.reject(n),Q(t.S),_(t)}}function M(t,e){var n=t.S.first();t.V=null,n.method="throw",n.ra=e,_(t)}function k(t,e){null!==t.S.ba&&(t.S.first().reject(e),Q(t.S)),t.V&&"return"in t.V&&(t.V.return(void 0),t.V=null),t.Dc.return(void 0),_(t)}o.scope={},o.rc=function(t){var e=0;return function(){return e<t.length?{done:!1,value:t[e++]}:{done:!0}}},o.Fd=function(t){return{next:o.rc(t)}},o.Zb=!1,o.xf=!1,o.yf=!1,o.Af=!1,o.defineProperty=o.Zb||"function"==typeof Object.defineProperties?Object.defineProperty:function(t,e,n){t!=Array.prototype&&t!=Object.prototype&&(t[e]=n.value)},o.ye=function(e){return"undefined"!=typeof window&&window===e?e:void 0!==t&&null!=t?t:e},o.global=o.ye(this),o.Sc="jscomp_symbol_",o.l=function(){o.l=function(){},o.global.Symbol||(o.global.Symbol=o.Symbol)},a.prototype.toString=function(){return this.Oc},o.Symbol=(i=0,function t(e){if(this instanceof t)throw new TypeError("Symbol is not a constructor");return new a(o.Sc+(e||"")+"_"+i++,e)}),o.K=function(){o.l();var t=o.global.Symbol.iterator;t||(t=o.global.Symbol.iterator=o.global.Symbol("Symbol.iterator")),"function"!=typeof Array.prototype[t]&&o.defineProperty(Array.prototype,t,{configurable:!0,writable:!0,value:function(){return o.Be(o.rc(this))}}),o.K=function(){}},o.L=function(){o.l();var t=o.global.Symbol.asyncIterator;t||(t=o.global.Symbol.asyncIterator=o.global.Symbol("Symbol.asyncIterator")),o.L=function(){}},o.Be=function(t){return o.K(),(t={next:t})[o.global.Symbol.iterator]=function(){return this},t},o.w=function(t){var e="undefined"!=typeof Symbol&&Symbol.iterator&&t[Symbol.iterator];return e?e.call(t):o.Fd(t)},o.Ed=function(t){for(var e,n=[];!(e=t.next()).done;)n.push(e.value);return n},o.o=function(t){return t instanceof Array?t:o.Ed(o.w(t))},o.Ge=o.Zb||"function"==typeof Object.create?Object.create:function(t){function e(){}return e.prototype=t,new e},o.vf=function(){var t={};try{return t.__proto__={$c:!0},t.$c}catch(t){}return!1},o.setPrototypeOf="function"==typeof Object.setPrototypeOf?Object.setPrototypeOf:o.vf()?function(t,e){if(t.__proto__=e,t.__proto__!==e)throw new TypeError(t+" is not extensible");return t}:null,o.h=function(t,e){if(t.prototype=o.Ge(e.prototype),t.prototype.constructor=t,o.setPrototypeOf){var n=o.setPrototypeOf;n(t,e)}else for(n in e)if("prototype"!=n)if(Object.defineProperties){var r=Object.getOwnPropertyDescriptor(e,n);r&&Object.defineProperty(t,n,r)}else t[n]=e[n];t.mg=e.prototype},o.C={},o.C.zc=function(t){if(!(t instanceof Object))throw new TypeError("Iterator result "+t+" is not an object")},s.prototype.Qa=function(t){this.m=t},s.prototype.Ta=function(t){this.Z={Ac:t,Fc:!0},this.i=this.ma||this.aa},s.prototype.return=function(t){this.Z={return:t},this.i=this.aa},s.prototype.j=function(t){this.i=t},f.prototype.Qa=function(t){return A(this.G),this.G.ja?I(this,this.G.ja.next,t,this.G.Qa):(this.G.Qa(t),C(this))},f.prototype.Ta=function(t){return A(this.G),this.G.ja?I(this,this.G.ja.throw,t,this.G.Qa):(this.G.Ta(t),C(this))},o.C.H=function(t,e){return e=new m(new f(e)),o.setPrototypeOf&&o.setPrototypeOf(e,t.prototype),e},o.Hc=function(t,e){if(e){var n=o.global;t=t.split(".");for(var r=0;r<t.length-1;r++){var i=t[r];i in n||(n[i]={}),n=n[i]}(e=e(r=n[t=t[t.length-1]]))!=r&&null!=e&&o.defineProperty(n,t,{configurable:!0,writable:!0,value:e})}},o.Pc=!1,o.Hc("Promise",function(t){function e(t){this.Sa=0,this.Wb=void 0,this.xa=[];var e=this.Qb();try{t(e.resolve,e.reject)}catch(t){e.reject(t)}}function n(){this.la=null}function r(t){return t instanceof e?t:new e(function(e){e(t)})}if(t&&!o.Pc)return t;n.prototype.sc=function(t){if(null==this.la){this.la=[];var e=this;this.tc(function(){e.te()})}this.la.push(t)};var i=o.global.setTimeout;n.prototype.tc=function(t){i(t,0)},n.prototype.te=function(){for(;this.la&&this.la.length;){var t=this.la;this.la=[];for(var e=0;e<t.length;++e){var n=t[e];t[e]=null;try{n()}catch(t){this.Gd(t)}}}this.la=null},n.prototype.Gd=function(t){this.tc(function(){throw t})},e.prototype.Qb=function(){function t(t){return function(r){n||(n=!0,t.call(e,r))}}var e=this,n=!1;return{resolve:t(this.Ke),reject:t(this.Vb)}},e.prototype.Ke=function(t){if(t===this)this.Vb(new TypeError("A Promise cannot resolve to itself"));else if(t instanceof e)this.Me(t);else{t:switch(typeof t){case"object":var n=null!=t;break t;case"function":n=!0;break t;default:n=!1}n?this.Je(t):this.Cc(t)}},e.prototype.Je=function(t){var e=void 0;try{e=t.then}catch(t){return void this.Vb(t)}"function"==typeof e?this.Ne(e,t):this.Cc(t)},e.prototype.Vb=function(t){this.Jc(2,t)},e.prototype.Cc=function(t){this.Jc(1,t)},e.prototype.Jc=function(t,e){if(0!=this.Sa)throw Error("Cannot settle("+t+", "+e+"): Promise already settled in state"+this.Sa);this.Sa=t,this.Wb=e,this.ue()},e.prototype.ue=function(){if(null!=this.xa){for(var t=0;t<this.xa.length;++t)a.sc(this.xa[t]);this.xa=null}};var a=new n;return e.prototype.Me=function(t){var e=this.Qb();t.fb(e.resolve,e.reject)},e.prototype.Ne=function(t,e){var n=this.Qb();try{t.call(e,n.resolve,n.reject)}catch(t){n.reject(t)}},e.prototype.then=function(t,n){function r(t,e){return"function"==typeof t?function(e){try{i(t(e))}catch(t){o(t)}}:e}var i,o,a=new e(function(t,e){i=t,o=e});return this.fb(r(t,i),r(n,o)),a},e.prototype.catch=function(t){return this.then(void 0,t)},e.prototype.fb=function(t,e){function n(){switch(r.Sa){case 1:t(r.Wb);break;case 2:e(r.Wb);break;default:throw Error("Unexpected state: "+r.Sa)}}var r=this;null==this.xa?a.sc(n):this.xa.push(n)},e.resolve=r,e.reject=function(t){return new e(function(e,n){n(t)})},e.race=function(t){return new e(function(e,n){for(var i=o.w(t),a=i.next();!a.done;a=i.next())r(a.value).fb(e,n)})},e.all=function(t){var n=o.w(t),i=n.next();return i.done?r([]):new e(function(t,e){function o(e){return function(n){a[e]=n,0==--s&&t(a)}}var a=[],s=0;do{a.push(void 0),s++,r(i.value).fb(o(a.length-1),e),i=n.next()}while(!i.done)})},e}),o.Hc("Promise.prototype.finally",function(t){return t||function(t){return this.then(function(e){return Promise.resolve(t()).then(function(){return e})},function(e){return Promise.resolve(t()).then(function(){throw e})})}}),o.uc=function(t){function e(e){return t.next(e)}function n(e){return t.throw(e)}return new Promise(function(r,i){!function t(o){o.done?r(o.value):Promise.resolve(o.value).then(e,n).then(t,i)}(t.next())})},o.Ff=function(t){return o.uc(t())},o.u=function(t){return o.uc(new m(new f(t)))},o.kb=function(t){o.L();var e=t[Symbol.asyncIterator];return void 0!==e?e.call(t):new y(o.w(t))},b.prototype.first=function(){if(this.ba)return this.ba.frame;throw Error("no frames in executionQueue")},b.prototype.enqueue=function(t){t=new B(t),this.nb?this.nb.next=t:this.ba=t,this.nb=t},w.prototype.next=function(t){return x(this,"next",t)},w.prototype.return=function(t){return x(this,"return",new E(0,t))},w.prototype.throw=function(t){return x(this,"throw",t)};var S=new Int32Array(2);function L(t,e){this.low=0|t,this.high=0|e}new Float32Array(S.buffer),new Float64Array(S.buffer),new Uint16Array(new Uint8Array([1,0]).buffer),L.prototype.equals=function(t){return this.low==t.low&&this.high==t.high};var R=new L(0,0);function N(){this.a=new Z(new Uint8Array(1024)),this.N=1024,this.lb=1,this.na=null,this.Nc=0,this.jb=!1,this.Gc=0,this.ob=[],this.Mc=0,this.Na=!1}function F(t){return t.a.fa().subarray(t.a.position(),t.a.position()+t.offset())}function T(t,e,n){e>t.lb&&(t.lb=e);for(var r=1+~(t.a.B.length-t.N+n)&e-1;t.N<r+e+n;){var i=t.a.B.length,o=t,a=t.a,s=a.B.length;if(3221225472&s)throw Error("FlatBuffers: cannot grow buffer beyond 2 gigabytes.");var A=s<<1,l=new Z(new Uint8Array(A));l.setPosition(A-s),l.fa().set(a.fa(),A-s),o.a=l,t.N+=t.a.B.length-i}U(t,r)}function U(t,e){for(var n=0;n<e;n++)t.a.Ba(--t.N,0)}function O(t,e){T(t,2,0),t.Yb(e)}function G(t,e,n,r){(t.Na||n!=r)&&(T(t,1,0),t.Ba(n),t.slot(e))}function H(t,e,n,r){(t.Na||n!=r)&&(O(t,n),t.slot(e))}function j(t,e,n,r){(t.Na||n!=r)&&(T(t,4,0),t.X(n),t.slot(e))}function Y(t,e,n,r){!t.Na&&n.equals(r)||(T(t,8,0),t.ia(n),t.slot(e))}function P(t,e,n){(t.Na||0!=n)&&(z(t,n),t.slot(e))}function K(t){if(t.jb)throw Error("FlatBuffers: object serialization must not be nested.")}function z(t,e){T(t,4,0),t.X(t.offset()-e+4)}function J(t,e){K(t),null==t.na&&(t.na=[]),t.Nc=e;for(var n=0;n<e;n++)t.na[n]=0;t.jb=!0,t.Gc=t.offset()}function q(t){if(null==t.na||!t.jb)throw Error("FlatBuffers: endObject called without startObject");T(t,4,0),t.X(0);for(var e=t.offset(),n=t.Nc-1;0<=n&&0==t.na[n];n--);for(var r=n+1;0<=n;n--)O(t,0!=t.na[n]?e-t.na[n]:0);O(t,e-t.Gc),O(t,r=2*(r+2));var i=0,o=t.N;n=0;t:for(;n<t.ob.length;n++){var a=t.a.B.length-t.ob[n];if(r==tt(t.a,a)){for(var s=2;s<r;s+=2)if(tt(t.a,o+s)!=tt(t.a,a+s))continue t;i=t.ob[n];break}}return i?(t.N=t.a.B.length-e,t.a.X(t.N,i-e)):(t.ob.push(t.offset()),t.a.X(t.a.B.length-e,t.offset()-e)),t.jb=!1,e}function V(t,e,n,r){K(t),t.Mc=n,T(t,4,e*n),T(t,r,e*n)}function W(t){return t.X(t.Mc),t.offset()}function X(t,e){if(e instanceof Uint8Array)var n=e;else{n=[];for(var r=0;r<e.length;){var i=e.charCodeAt(r++);if(!(55296>i||56320<=i))i=(i<<10)+e.charCodeAt(r++)+-56613888;128>i?n.push(i):(2048>i?n.push(i>>6&31|192):(65536>i?n.push(i>>12&15|224):n.push(i>>18&7|240,i>>12&63|128),n.push(i>>6&63|128)),n.push(63&i|128))}}for(T(t,1,0),t.Ba(0),V(t,1,n.length,1),t.a.setPosition(t.N-=n.length),r=0,e=t.N,i=t.a.fa();r<n.length;r++)i[e++]=n[r];return W(t)}function Z(t){this.B=t,this.Ic=0}function $(t,e){return t.B[e]}function tt(t,e){return(t.B[e]|t.B[e+1]<<8)<<16>>16}function et(t,e){return new L(t.v(e),t.v(e+4))}function nt(t,e,n){return e-=t.v(e),n<tt(t,e)?tt(t,e+n):0}function rt(t,e,n){return e.b=n+t.v(n),e.a=t,e}function it(t,e,n){e+=t.v(e);var r=t.v(e),i="",o=0;if(e+=4,1===n)return t.B.subarray(e,e+r);for(;o<r;){if(!(192>(n=$(t,e+o++)))){var a=$(t,e+o++);if(224>n)n=(31&n)<<6|63&a;else{var s=$(t,e+o++);if(240>n)n=(15&n)<<12|(63&a)<<6|63&s;else n=(7&n)<<18|(63&a)<<12|(63&s)<<6|63&$(t,e+o++)}}65536>n?i+=String.fromCharCode(n):(n-=65536,i+=String.fromCharCode(55296+(n>>10),56320+(1023&n)))}return i}function ot(t,e){return e+t.v(e)}function at(t,e){return e+t.v(e)+4}function st(t,e){return t.v(e+t.v(e))}function At(t){if(void 0===t)return{};if(t===Object(t))return t;throw TypeError("Could not convert argument to dictionary")}function lt(t){this.sa=[].slice.call(t)}function ct(t){if(t)throw TypeError("Decoder error");return 65533}function ut(t,e){if(!(this instanceof ut))return new ut(t,e);if("utf-8"!==(t=void 0!==t?String(t).toLowerCase():"utf-8"))throw Error("Encoding not supported. Only utf-8 is supported");e=At(e),this.Ua=this.pa=!1,this.$a=null,this.hc=!!e.fatal,this.ic=!!e.ignoreBOM,Object.defineProperty(this,"encoding",{value:"utf-8"}),Object.defineProperty(this,"fatal",{value:this.hc}),Object.defineProperty(this,"ignoreBOM",{value:this.ic})}function gt(t,e){if(!(this instanceof gt))return new gt(t,e);if("utf-8"!==(t=void 0!==t?String(t).toLowerCase():"utf-8"))throw Error("Encoding not supported. Only utf-8 is supported");At(e),this.pa=!1,this.ab=null,Object.defineProperty(this,"encoding",{value:"utf-8"})}function ht(t){var e=t.fatal,n=0,r=0,i=0,o=128,a=191;this.Oa=function(t,s){if(-1===s&&0!==i)return i=0,ct(e);if(-1===s)return-1;if(0===i){if(0<=s&&127>=s)return s;if(194<=s&&223>=s)i=1,n=s-192;else if(224<=s&&239>=s)224===s&&(o=160),237===s&&(a=159),i=2,n=s-224;else{if(!(240<=s&&244>=s))return ct(e);240===s&&(o=144),244===s&&(a=143),i=3,n=s-240}return n<<=6*i,null}return o<=s&&s<=a?(o=128,a=191,n+=s-128<<6*(i-(r+=1)),r!==i?null:(t=n,n=i=r=0,t)):(n=i=r=0,o=128,a=191,t.prepend(s),ct(e))}}function dt(){this.Oa=function(t,e){if(-1===e)return-1;if(0<=e&&127>=e)return e;if(128<=e&&2047>=e)var n=1,r=192;else 2048<=e&&65535>=e?(n=2,r=224):65536<=e&&1114111>=e&&(n=3,r=240);for(t=[(e>>6*n)+r];0<n;)t.push(128|e>>6*(n-1)&63),--n;return t}}(r=N.prototype).Ba=function(t){this.a.Ba(--this.N,t)},r.Yb=function(t){this.a.Yb(this.N-=2,t)},r.X=function(t){this.a.X(this.N-=4,t)},r.ia=function(t){this.a.ia(this.N-=8,t)},r.slot=function(t){this.na[t]=this.offset()},r.offset=function(){return this.a.B.length-this.N},r.finish=function(t,e){if(e){if(T(this,this.lb,8),4!=e.length)throw Error("FlatBuffers: file identifier must be length 4");for(var n=3;0<=n;n--)this.Ba(e.charCodeAt(n))}T(this,this.lb,4),z(this,t),this.a.setPosition(this.N)},r.ha=function(t,e){return 0==t&&0==e?R:new L(t,e)},(r=Z.prototype).fa=function(){return this.B},r.position=function(){return this.Ic},r.setPosition=function(t){this.Ic=t},r.v=function(t){return this.B[t]|this.B[t+1]<<8|this.B[t+2]<<16|this.B[t+3]<<24},r.Ba=function(t,e){this.B[t]=e},r.Yb=function(t,e){this.B[t]=e,this.B[t+1]=e>>8},r.X=function(t,e){this.B[t]=e,this.B[t+1]=e>>8,this.B[t+2]=e>>16,this.B[t+3]=e>>24},r.ia=function(t,e){this.X(t,e.low),this.X(t+4,e.high)},r.ha=function(t,e){return 0==t&&0==e?R:new L(t,e)},lt.prototype={endOfStream:function(){return!this.sa.length},read:function(){return this.sa.length?this.sa.shift():-1},prepend:function(t){if(Array.isArray(t))for(;t.length;)this.sa.unshift(t.pop());else this.sa.unshift(t)},push:function(t){if(Array.isArray(t))for(;t.length;)this.sa.push(t.shift());else this.sa.push(t)}},ut.prototype={decode:function(t,e){t="object"==typeof t&&t instanceof ArrayBuffer?new Uint8Array(t):"object"==typeof t&&"buffer"in t&&t.buffer instanceof ArrayBuffer?new Uint8Array(t.buffer,t.byteOffset,t.byteLength):new Uint8Array(0),e=At(e),this.pa||(this.$a=new ht({fatal:this.hc}),this.Ua=!1),this.pa=!!e.stream,t=new lt(t),e=[];for(var n;!t.endOfStream()&&-1!==(n=this.$a.Oa(t,t.read()));)null!==n&&(Array.isArray(n)?e.push.apply(e,n):e.push(n));if(!this.pa){do{if(-1===(n=this.$a.Oa(t,t.read())))break;null!==n&&(Array.isArray(n)?e.push.apply(e,n):e.push(n))}while(!t.endOfStream());this.$a=null}for(!e.length||-1===["utf-8"].indexOf(this.encoding)||this.ic||this.Ua||(65279===e[0]?(this.Ua=!0,e.shift()):this.Ua=!0),t="",n=0;n<e.length;++n){var r=e[n];65535>=r?t+=String.fromCharCode(r):(r-=65536,t+=String.fromCharCode(55296+(r>>10),56320+(1023&r)))}return t}},gt.prototype={encode:function(t,e){t=t?String(t):"",e=At(e),this.pa||(this.ab=new dt),this.pa=!!e.stream,e=[];for(var n=String(t),r=n.length,i=0,o=[];i<r;){var a=n.charCodeAt(i);if(55296>a||57343<a)o.push(a);else if(56320<=a&&57343>=a)o.push(65533);else if(55296<=a&&56319>=a)if(i===r-1)o.push(65533);else{var s=t.charCodeAt(i+1);56320<=s&&57343>=s?(o.push(65536+((1023&a)<<10)+(1023&s)),i+=1):o.push(65533)}i+=1}for(t=new lt(o);!t.endOfStream()&&-1!==(n=this.ab.Oa(t,t.read()));)Array.isArray(n)?e.push.apply(e,n):e.push(n);if(!this.pa){for(;-1!==(n=this.ab.Oa(t,t.read()));)Array.isArray(n)?e.push.apply(e,n):e.push(n);this.ab=null}return new Uint8Array(e)}};var ft,pt="function"==typeof n?n:null,It="function"==typeof TextDecoder&&"function"==typeof TextEncoder,Ct=(ft="undefined"!=typeof TextDecoder?TextDecoder:ut,It||!pt?(ft=new ft).decode.bind(ft):function(t){return t=Wt.toUint8Array(t),pt.from(t.buffer,t.byteOffset,t.length).toString()}),mt=function(t){return It||!pt?(t=new t).encode.bind(t):function(t){return Wt.toUint8Array(pt.from(void 0===t?"":t,"utf8"))}}("undefined"!=typeof TextEncoder?TextEncoder:gt),yt=Object.freeze({done:!0,value:void 0});function Et(t){this.Ea=t}function vt(){}function Bt(t,e,n){return Bt(t.kc||(t.kc=t.toNodeStream()),e,n)}function bt(t){return t.dc||(t.dc=t.toDOMStream())}function Qt(){var t=this;this.Ka=[],this.za=[],this.yb=new Promise(function(e){return t.T=e})}o.global.Object.defineProperties(Et.prototype,{schema:{configurable:!0,enumerable:!0,get:function(){return this.Ea.schema}},vc:{configurable:!0,enumerable:!0,get:function(){return this.Ea.batches||[]}},dictionaries:{configurable:!0,enumerable:!0,get:function(){return this.Ea.dictionaries||[]}}}),vt.prototype.tee=function(){return bt(this).tee()},vt.prototype.pipeTo=function(t,e){return bt(this).pipeTo(t,e)},vt.prototype.pipeThrough=function(t,e){return bt(this).pipeThrough(t,e)},o.l(),o.L(),o.h(Qt,vt),Qt.prototype.cancel=function(t){var e=this;return o.u(function(n){return l(n,e.return(t),0)})},Qt.prototype.write=function(t){if(!this.T)throw Error(this+" is closed");0>=this.za.length?this.Ka.push(t):this.za.shift().resolve({done:!1,value:t})},Qt.prototype.abort=function(t){this.T&&(0>=this.za.length?this.Ab={error:t}:this.za.shift().reject({done:!0,value:t}))},Qt.prototype.close=function(){if(this.T){for(var t=this.za;0<t.length;)t.shift().resolve(yt);this.T(),this.T=void 0}},Qt.prototype[Symbol.asyncIterator]=function(){return this},(r=Qt.prototype).toDOMStream=function(t){return Xt(this.T||this.Ab?this:this.Ka,t)},r.toNodeStream=function(){return Zt()},r.throw=function(t){var e=this;return o.u(function(n){return 1==n.i?l(n,e.abort(t),2):n.return(yt)})},r.return=function(){var t=this;return o.u(function(e){return 1==e.i?l(e,t.close(),2):e.return(yt)})},r.read=function(t){var e=this;return o.u(function(n){return 1==n.i?l(n,e.next(t,"read"),2):n.return(n.m.value)})},r.peek=function(t){var e=this;return o.u(function(n){return 1==n.i?l(n,e.next(t,"peek"),2):n.return(n.m.value)})},r.next=function(t){for(var e=0;e<arguments.length;++e);var n=this;return 0<this.Ka.length?Promise.resolve({done:!1,value:this.Ka.shift()}):this.Ab?Promise.reject({done:!0,value:this.Ab.error}):this.T?new Promise(function(t,e){n.za.push({resolve:t,reject:e})}):Promise.resolve(yt)},o.global.Object.defineProperties(Qt.prototype,{closed:{configurable:!0,enumerable:!0,get:function(){return this.yb}}});var wt=o.w(function(){function t(){throw Error("BigInt is not available in this environment")}function e(){throw t()}return e.Df=function(){throw t()},e.Ef=function(){throw t()},"undefined"!=typeof BigInt?[BigInt,!0]:[e,!1]}()),xt=wt.next().value,_t=wt.next().value,Dt=o.w(function(){function t(){throw e()}function e(){throw Error("BigInt64Array is not available in this environment")}return t.of=function(){throw e()},t.from=function(){throw e()},o.global.Object.defineProperties(t,{BYTES_PER_ELEMENT:{configurable:!0,enumerable:!0,get:function(){return 8}}}),"undefined"!=typeof BigInt64Array?[BigInt64Array,!0]:[t,!1]}()),Mt=Dt.next().value;Dt.next();var kt=o.w(function(){function t(){throw e()}function e(){throw Error("BigUint64Array is not available in this environment")}return t.of=function(){throw e()},t.from=function(){throw e()},o.global.Object.defineProperties(t,{BYTES_PER_ELEMENT:{configurable:!0,enumerable:!0,get:function(){return 8}}}),"undefined"!=typeof BigUint64Array?[BigUint64Array,!0]:[t,!1]}()),St=kt.next().value;function Lt(t){return"function"==typeof t}function Rt(t){return null!=t&&Object(t)===t}function Nt(t){return Rt(t)&&Lt(t.then)}function Ft(t){return o.l(),o.K(),Rt(t)&&Lt(t[Symbol.iterator])}function Tt(t){return o.l(),o.L(),Rt(t)&&Lt(t[Symbol.asyncIterator])}function Ut(t){return Rt(t)&&Rt(t.schema)}function Ot(t){return Rt(t)&&Lt(t.tee)&&Lt(t.cancel)&&Lt(t.pipeTo)&&Lt(t.getReader)&&!(t instanceof vt)}function Gt(t){return Rt(t)&&Lt(t.read)&&Lt(t.pipe)&&"boolean"==typeof t.readable&&!(t instanceof vt)}kt.next();var Ht="undefined"!=typeof SharedArrayBuffer?SharedArrayBuffer:ArrayBuffer;function jt(t){for(var e,n,r,i,o,a,s=t[0]?[t[0]]:[],A=0,l=0,c=t.length;++A<c;)o=s[l],a=t[A],o&&a&&o.buffer===a.buffer?(a.byteOffset<o.byteOffset&&(o=t[A],a=s[l]),e=(n=o).byteOffset,r=n.byteLength,n=(i=a).byteOffset,i=i.byteLength,e+r<n||n+i<e?a&&(s[++l]=a):s[l]=new Uint8Array(o.buffer,e,n-e+i)):a&&(s[++l]=a);return s}function Yt(t,e,n,r){n=void 0===n?0:n,r=void 0===r?e.byteLength:r;var i=t.byteLength,o=new Uint8Array(t.buffer,t.byteOffset,i);return e=new Uint8Array(e.buffer,e.byteOffset,Math.min(r,i)),o.set(e,n),t}function Pt(t,e){return(e=Rt(e)&&"done"in e&&"value"in e?e.value:e)?("string"==typeof e&&(e=mt(e)),e instanceof t?e.constructor===t?e:new t(e.buffer,e.byteOffset,e.byteLength/t.BYTES_PER_ELEMENT):e instanceof ArrayBuffer||e instanceof Ht?new t(e):e instanceof Z?Pt(t,e.fa()):ArrayBuffer.isView(e)?0>=e.byteLength?new t(0):new t(e.buffer,e.byteOffset,e.byteLength/t.BYTES_PER_ELEMENT):t.from(e)):new t(0)}function Kt(t){return Pt(Uint16Array,t)}function zt(t){return Pt(Float64Array,t)}function Jt(t){return t.next(),t}function qt(t,e){var n,r;return o.C.H(qt,function(i){return n=function t(e){return o.C.H(t,function(t){return l(t,e,0)})},r="string"==typeof e?n(e):ArrayBuffer.isView(e)?n(e):e instanceof ArrayBuffer?n(e):e instanceof Ht?n(e):Ft(e)?e:n(e),o.l(),o.K(),c(i,Jt(function e(n){var r,i,a;return o.C.H(e,function(e){return 1==e.i&&(r=null),5!=e.i?(a=(i=n).next,l(e,Pt(t,r),5)):(r=a.call(i,e.m)).done?void(e.i=0):e.j(2)})}(r[Symbol.iterator]())),0)})}function Vt(t,e){return new w(function n(){var r,i,a,s,A,c,u;return o.C.H(n,function(n){switch(n.i){case 1:if(!Nt(e)){n.j(2);break}return r=E,i=1,a=Vt,s=t,l(n,new E(2,e),4);case 4:return l(n,new r(i,a(s,n.m)),3);case 3:return n.return(n.m);case 2:return A=function(t){return new w(function e(){var n,r;return o.C.H(e,function(e){return 1==e.i?(n=E,r=0,l(e,new E(2,t),3)):l(e,new n(r,e.m),0)})}())},c=function(t){return new w(function e(){return o.C.H(e,function(e){return o.l(),o.K(),l(e,new E(1,Jt(function t(e){var n,r,i;return o.C.H(t,function(t){return 1==t.i&&(n=null),5!=t.i?(i=(r=e).next,l(t,n&&n.value,5)):(n=i.call(r,t.m)).done?void(t.i=0):t.j(2)})}(t[Symbol.iterator]()))),0)})}())},u="string"==typeof e?A(e):ArrayBuffer.isView(e)?A(e):e instanceof ArrayBuffer?A(e):e instanceof Ht?A(e):Ft(e)?c(e):Tt(e)?e:A(e),o.l(),o.L(),l(n,new E(1,Jt(function(e){return new w(function n(){var r,i,a,s,A;return o.C.H(n,function(n){switch(n.i){case 1:r=null;case 2:return i=E,a=2,A=(s=e).next,l(n,new E(0,Pt(t,r)),6);case 6:return l(n,new i(a,A.call(s,n.m)),5);case 5:(r=n.m).done?n.i=0:n.j(2)}})}())}(u[Symbol.asyncIterator]()))),0)}})}())}var Wt={};function Xt(){throw Error('"toDOMStream" not available in this environment')}function Zt(){throw Error('"toNodeStream" not available in this environment')}function $t(t){return t.next(),t}function te(t){var e,n,r,i,a,s,A,c,u,f,p,I;return o.C.H(te,function(C){switch(C.i){case 1:return n=!1,r=[],A=0,l(C,null,2);case 2:c=C.m,a=c.ga,s=c.size,o.l(),o.K(),u=Wt.toUint8ArrayIterator(t)[Symbol.iterator](),C.ma=3,C.aa=4;case 6:if(f=isNaN(s-A)?u.next(void 0):u.next(s-A),e=f.done,i=f.value,!e&&0<i.byteLength&&(r.push(i),A+=i.byteLength),!(e||s<=A)){C.j(8);break}case 10:if("peek"===a)var m=Wt.joinUint8Arrays(r,s)[0];else m=o.w(Wt.joinUint8Arrays(r,s)),i=m.next().value,r=m.next().value,A=m.next().value,m=i;return l(C,m,13);case 13:if(p=C.m,a=p.ga,(s=p.size)<A){C.j(10);break}case 8:if(!e){C.j(6);break}case 4:h(C),!1===n&&"function"==typeof u.return&&u.return(),d(C);break;case 3:I=g(C),n=!0,"function"==typeof u.throw&&u.throw(I),C.j(4)}})}function ee(t){return new w(function e(){var n,r,i,a,s,A,c,u,f,p,I,C,m;return o.C.H(e,function(e){switch(e.i){case 1:return r=!1,i=[],c=0,l(e,new E(0,null),2);case 2:u=e.m,s=u.ga,A=u.size,o.l(),o.L(),f=Wt.toUint8ArrayAsyncIterator(t)[Symbol.asyncIterator](),e.ma=3,e.aa=4;case 6:return isNaN(A-c)?l(e,new E(2,f.next(void 0)),12):l(e,new E(2,f.next(A-c)),11);case 11:p=e.m,e.j(10);break;case 12:p=e.m;case 10:if(n=(I=p).done,a=I.value,!n&&0<a.byteLength&&(i.push(a),c+=a.byteLength),!(n||A<=c)){e.j(8);break}case 14:if("peek"===s)var y=Wt.joinUint8Arrays(i,A)[0];else y=o.w(Wt.joinUint8Arrays(i,A)),a=y.next().value,i=y.next().value,c=y.next().value,y=a;return l(e,new E(0,y),17);case 17:if(C=e.m,s=C.ga,(A=C.size)<c){e.j(14);break}case 8:if(!n){e.j(6);break}case 4:if(h(e),!1!==r||"function"!=typeof f.return){e.j(18);break}return l(e,new E(2,f.return()),19);case 19:case 18:d(e);break;case 3:if(m=g(e),r=!0,"function"!=typeof f.throw){e.j(20);break}return l(e,new E(2,f.throw(m)),21);case 21:case 20:e.j(4)}})}())}function ne(t){return new w(function e(){var n,r,i,a,s,A,c,u,f,p,I,C,m;return o.C.H(e,function(e){switch(e.i){case 1:return r=n=!1,i=[],c=0,l(e,new E(0,null),2);case 2:u=e.m,s=u.ga,A=u.size,f=new re(t),e.ma=3,e.aa=4;case 6:return isNaN(A-c)?l(e,new E(2,f.read(void 0)),12):l(e,new E(2,f.read(A-c)),11);case 11:p=e.m,e.j(10);break;case 12:p=e.m;case 10:if(n=(I=p).done,a=I.value,!n&&0<a.byteLength&&(i.push(Wt.toUint8Array(a)),c+=a.byteLength),!(n||A<=c)){e.j(8);break}case 14:if("peek"===s)var y=Wt.joinUint8Arrays(i,A)[0];else y=o.w(Wt.joinUint8Arrays(i,A)),a=y.next().value,i=y.next().value,c=y.next().value,y=a;return l(e,new E(0,y),17);case 17:if(C=e.m,s=C.ga,(A=C.size)<c){e.j(14);break}case 8:if(!n){e.j(6);break}case 4:if(h(e),!1!==r){t.locked&&f.releaseLock(),e.j(18);break}return l(e,new E(2,f.cancel()),19);case 19:case 18:d(e);break;case 3:return m=g(e),r=!0,l(e,new E(2,f.cancel(m)),21);case 21:case 20:e.j(4)}})}())}function re(t){this.source=t,this.va=this.ua=null;try{this.Lc=!!(this.ca=oe(this))}catch(t){this.Lc=!(this.ca=ie(this))}}function ie(t){return t.ua&&t.releaseLock(),t.va||(t.va=t.source.getReader(),t.va.closed.catch(function(){})),t.ca=t.va}function oe(t){return t.va&&t.releaseLock(),t.ua||(t.ua=t.source.getReader({mode:"byob"}),t.ua.closed.catch(function(){})),t.ca=t.ua}function ae(t,e){return o.u(function(n){return 1==n.i?l(n,se(oe(t),new ArrayBuffer(e),0,e),2):n.return(n.m)})}function se(t,e,n,r){var i,a,s;return o.u(function(o){switch(o.i){case 1:return n>=r?o.return({done:!1,value:new Uint8Array(e,0,r)}):l(o,t.read(new Uint8Array(e,n,r-n)),2);case 2:if(i=o.m,a=i.done,s=i.value,!((n+=s.byteLength)<r)||a){o.j(3);break}return l(o,se(t,s.buffer,n,r),4);case 4:return o.return(o.m);case 3:return o.return({done:a,value:new Uint8Array(s.buffer,0,n)})}})}function Ae(t,e){function n(t){return r([e,t])}var r;return[e,n,new Promise(function(i){return(r=i)&&t.once(e,n)})]}function le(t){return new w(function e(){var n,r,i,a,s,A,c,g,f,p,I,C,m,y;return o.C.H(e,function(e){switch(e.i){case 1:return n=[],r="error",i=!1,a=null,c=0,g=[],l(e,new E(0,null),2);case 2:if(p=e.m,s=p.ga,A=p.size,!t.isTTY){e.j(3);break}return l(e,new E(0,new Uint8Array(0)),4);case 4:return e.return(e.m);case 3:u(e,5),n[0]=Ae(t,"end"),n[1]=Ae(t,"error");case 7:return n[2]=Ae(t,"readable"),C=(I=o).w,l(e,new E(2,Promise.race(n.map(function(t){return t[2]}))),10);case 10:if(m=C.call(I,e.m),r=m.next().value,a=m.next().value,"error"===r){e.j(5);break}if((i="end"===r)||(isFinite(A-c)?(f=Wt.toUint8Array(t.read(A-c))).byteLength<A-c&&(f=Wt.toUint8Array(t.read(void 0))):f=Wt.toUint8Array(t.read(void 0)),0<f.byteLength&&(g.push(f),c+=f.byteLength)),!(i||A<=c)){e.j(9);break}case 12:return l(e,new E(0,function(){if("peek"===s)return Wt.joinUint8Arrays(g,A)[0];var t=o.w(Wt.joinUint8Arrays(g,A));return f=t.next().value,g=t.next().value,c=t.next().value,f}()),15);case 15:if(y=e.m,s=y.ga,(A=y.size)<c){e.j(12);break}case 9:if(!i){e.j(7);break}case 5:return h(e),l(e,new E(2,function(e,n){return f=g=null,new Promise(function(r,i){var a,s,A,l,c,u,g;return o.u(function(h){for(a=o.w(e),s=a.next();!s.done;s=a.next())A=s.value,l=o.w(A),c=l.next().value,u=l.next().value,t.off(c,u);try{(g=t.destroy)&&g.call(t,n),n=void 0}catch(t){n=t||n}finally{null!=n?i(n):r()}h.i=0})})}(n,"error"===r?a:null)),16);case 16:d(e)}})}())}function ce(){}Wt.memcpy=Yt,Wt.joinUint8Arrays=function(t,e){for(var n,r,i=(t=jt(t)).reduce(function(t,e){return t+e.byteLength},0),o=0,a=-1,s=Math.min(e||1/0,i),A=t.length;++a<A;){if(s<=o+(n=(e=t[a]).subarray(0,Math.min(e.length,s-o))).length){n.length<e.length?t[a]=e.subarray(n.length):n.length===e.length&&a++,r?Yt(r,n,o):r=n;break}Yt(r||(r=new Uint8Array(s)),n,o),o+=n.length}return[r||new Uint8Array(0),t.slice(a),i-(r?r.byteLength:0)]},Wt.toArrayBufferView=Pt,Wt.toInt8Array=function(t){return Pt(Int8Array,t)},Wt.toInt16Array=function(t){return Pt(Int16Array,t)},Wt.toInt32Array=function(t){return Pt(Int32Array,t)},Wt.toBigInt64Array=function(t){return Pt(Mt,t)},Wt.toUint8Array=function(t){return Pt(Uint8Array,t)},Wt.toUint16Array=Kt,Wt.toUint32Array=function(t){return Pt(Uint32Array,t)},Wt.toBigUint64Array=function(t){return Pt(St,t)},Wt.toFloat32Array=function(t){return Pt(Float32Array,t)},Wt.toFloat64Array=zt,Wt.toUint8ClampedArray=function(t){return Pt(Uint8ClampedArray,t)},Wt.toFloat16Array=function(t){var e=null;if(ArrayBuffer.isView(t))switch(t.constructor){case Float32Array:e=t;break;case Float64Array:e=t}else Ft(t)&&(e=zt(t));if(e){for(var n=-1,r=(t=new Uint16Array(e.length)).length;++n<r;)t[n]=32767*e[n]+32767;return t}return Kt(t)},Wt.toArrayBufferViewIterator=qt,Wt.toInt8ArrayIterator=function(t){return qt(Int8Array,t)},Wt.toInt16ArrayIterator=function(t){return qt(Int16Array,t)},Wt.toInt32ArrayIterator=function(t){return qt(Int32Array,t)},Wt.toUint8ArrayIterator=function(t){return qt(Uint8Array,t)},Wt.toUint16ArrayIterator=function(t){return qt(Uint16Array,t)},Wt.toUint32ArrayIterator=function(t){return qt(Uint32Array,t)},Wt.toFloat32ArrayIterator=function(t){return qt(Float32Array,t)},Wt.toFloat64ArrayIterator=function(t){return qt(Float64Array,t)},Wt.toUint8ClampedArrayIterator=function(t){return qt(Uint8ClampedArray,t)},Wt.toArrayBufferViewAsyncIterator=Vt,Wt.toInt8ArrayAsyncIterator=function(t){return Vt(Int8Array,t)},Wt.toInt16ArrayAsyncIterator=function(t){return Vt(Int16Array,t)},Wt.toInt32ArrayAsyncIterator=function(t){return Vt(Int32Array,t)},Wt.toUint8ArrayAsyncIterator=function(t){return Vt(Uint8Array,t)},Wt.toUint16ArrayAsyncIterator=function(t){return Vt(Uint16Array,t)},Wt.toUint32ArrayAsyncIterator=function(t){return Vt(Uint32Array,t)},Wt.toFloat32ArrayAsyncIterator=function(t){return Vt(Float32Array,t)},Wt.toFloat64ArrayAsyncIterator=function(t){return Vt(Float64Array,t)},Wt.toUint8ClampedArrayAsyncIterator=function(t){return Vt(Uint8ClampedArray,t)},Wt.rebaseValueOffsets=function(t,e,n){if(0!==t){n=n.slice(0,e+1);for(var r=-1;++r<=e;)n[r]+=t}return n},Wt.compareArrayLike=function(t,e){var n=0,r=t.length;if(r!==e.length)return!1;if(0<r)do{if(t[n]!==e[n])return!1}while(++n<r);return!0},re.prototype.releaseLock=function(){this.ca&&this.ca.releaseLock(),this.ca=this.ua=this.va=null},re.prototype.cancel=function(t){var e,n,r,i=this;return o.u(function(o){if(1==o.i)return n=(e=i).ca,r=e.source,n?l(o,n.cancel(t),3):o.j(2);r&&r.locked&&i.releaseLock(),o.i=0})},re.prototype.read=function(t){var e,n,r=this;return o.u(function(i){switch(i.i){case 1:return 0===t?i.return({done:null==r.ca,value:new Uint8Array(0)}):r.Lc&&"number"==typeof t?l(i,ae(r,t),4):l(i,ie(r).read(),5);case 4:e=i.m,i.j(3);break;case 5:e=i.m;case 3:return!(n=e).done&&(n.value=Wt.toUint8Array(n)),i.return(n)}})},o.global.Object.defineProperties(re.prototype,{closed:{configurable:!0,enumerable:!0,get:function(){return this.ca?this.ca.closed.catch(function(){}):Promise.resolve()}}});var ue,ge=ue||(ue={}),he=ge.c||(ge.c={}),de=he.f||(he.f={}),fe=de.g||(de.g={}),pe=fe.MetadataVersion||(fe.MetadataVersion={});pe[pe.V1=0]="V1",pe[pe.V2=1]="V2",pe[pe.V3=2]="V3",pe[pe.V4=3]="V4";var Ie=ue||(ue={}),Ce=Ie.c||(Ie.c={}),me=Ce.f||(Ce.f={}),ye=me.g||(me.g={}),Ee=ye.UnionMode||(ye.UnionMode={});Ee[Ee.Sparse=0]="Sparse",Ee[Ee.Dense=1]="Dense";var ve=ue||(ue={}),Be=ve.c||(ve.c={}),be=Be.f||(Be.f={}),Qe=be.g||(be.g={}),we=Qe.Precision||(Qe.Precision={});we[we.HALF=0]="HALF",we[we.SINGLE=1]="SINGLE",we[we.DOUBLE=2]="DOUBLE";var xe=ue||(ue={}),_e=xe.c||(xe.c={}),De=_e.f||(_e.f={}),Me=De.g||(De.g={}),ke=Me.DateUnit||(Me.DateUnit={});ke[ke.DAY=0]="DAY",ke[ke.MILLISECOND=1]="MILLISECOND";var Se=ue||(ue={}),Le=Se.c||(Se.c={}),Re=Le.f||(Le.f={}),Ne=Re.g||(Re.g={}),Fe=Ne.TimeUnit||(Ne.TimeUnit={});Fe[Fe.SECOND=0]="SECOND",Fe[Fe.MILLISECOND=1]="MILLISECOND",Fe[Fe.MICROSECOND=2]="MICROSECOND",Fe[Fe.NANOSECOND=3]="NANOSECOND";var Te=ue||(ue={}),Ue=Te.c||(Te.c={}),Oe=Ue.f||(Ue.f={}),Ge=Oe.g||(Oe.g={}),He=Ge.IntervalUnit||(Ge.IntervalUnit={});He[He.YEAR_MONTH=0]="YEAR_MONTH",He[He.DAY_TIME=1]="DAY_TIME";var je=ue||(ue={}),Ye=je.c||(je.c={}),Pe=Ye.f||(Ye.f={}),Ke=Pe.g||(Pe.g={}),ze=Ke.Type||(Ke.Type={});ze[ze.NONE=0]="NONE",ze[ze.Null=1]="Null",ze[ze.Int=2]="Int",ze[ze.FloatingPoint=3]="FloatingPoint",ze[ze.Binary=4]="Binary",ze[ze.Utf8=5]="Utf8",ze[ze.Bool=6]="Bool",ze[ze.Decimal=7]="Decimal",ze[ze.Date=8]="Date",ze[ze.Time=9]="Time",ze[ze.Timestamp=10]="Timestamp",ze[ze.Interval=11]="Interval",ze[ze.List=12]="List",ze[ze.Struct_=13]="Struct_",ze[ze.Union=14]="Union",ze[ze.FixedSizeBinary=15]="FixedSizeBinary",ze[ze.FixedSizeList=16]="FixedSizeList",ze[ze.Map=17]="Map";var Je=ue||(ue={}),qe=Je.c||(Je.c={}),Ve=qe.f||(qe.f={}),We=Ve.g||(Ve.g={}),Xe=We.VectorType||(We.VectorType={});Xe[Xe.OFFSET=0]="OFFSET",Xe[Xe.DATA=1]="DATA",Xe[Xe.VALIDITY=2]="VALIDITY",Xe[Xe.TYPE=3]="TYPE";var Ze=ue||(ue={}),$e=Ze.c||(Ze.c={}),tn=$e.f||($e.f={}),en=tn.g||(tn.g={}),nn=en.rb||(en.rb={});nn[nn.Little=0]="Little",nn[nn.Big=1]="Big",function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.Zf=function(t,n){return(n||new e).s(t.v(t.position())+t.position(),t)},e.jf=function(t){J(t,0)},e.ke=function(t){return q(t)},t.Null=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.bg=function(t,n){return(n||new e).s(t.v(t.position())+t.position(),t)},e.nf=function(t){J(t,0)},e.pe=function(t){return q(t)},t.Struct_=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.Xf=function(t,n){return(n||new e).s(t.v(t.position())+t.position(),t)},e.ef=function(t){J(t,0)},e.he=function(t){return q(t)},t.List=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.Sf=function(t,n){return(n||new e).s(t.v(t.position())+t.position(),t)},e.prototype.listSize=function(){var t=nt(this.a,this.b,4);return t?this.a.v(this.b+t):0},e.$e=function(t){J(t,1)},e.td=function(t,e){j(t,0,e,0)},e.be=function(t){return q(t)},t.FixedSizeList=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.Yf=function(t,n){return(n||new e).s(t.v(t.position())+t.position(),t)},e.prototype.keysSorted=function(){var t=nt(this.a,this.b,4);return!!t&&!!(this.a.B[this.b+t]<<24>>24)},e.ff=function(t){J(t,1)},e.rd=function(t,e){G(t,0,+e,0)},e.ie=function(t){return q(t)},t.Map=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.eg=function(t,e){return(e||new n).s(t.v(t.position())+t.position(),t)},n.prototype.mode=function(){var e=nt(this.a,this.b,4);return e?tt(this.a,this.b+e):t.c.f.g.UnionMode.Sparse},n.prototype.typeIds=function(t){var e=nt(this.a,this.b,6);return e?this.a.v(at(this.a,this.b+e)+4*t):0},n.prototype.tf=function(){var t=nt(this.a,this.b,6);return t?new Int32Array(this.a.fa().buffer,this.a.fa().byteOffset+at(this.a,this.b+t),st(this.a,this.b+t)):null},n.rf=function(t){J(t,2)},n.ud=function(e,n){H(e,0,n,t.c.f.g.UnionMode.Sparse)},n.Cd=function(t,e){P(t,1,e)},n.Sd=function(t,e){V(t,4,e.length,4);for(var n=e.length-1;0<=n;n--){var r=t,i=e[n];T(r,4,0),r.X(i)}return W(t)},n.qf=function(t,e){V(t,4,e,4)},n.re=function(t){return q(t)},e.Union=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.Uf=function(t,n){return(n||new e).s(t.v(t.position())+t.position(),t)},e.prototype.bitWidth=function(){var t=nt(this.a,this.b,4);return t?this.a.v(this.b+t):0},e.prototype.isSigned=function(){var t=nt(this.a,this.b,6);return!!t&&!!(this.a.B[this.b+t]<<24>>24)},e.cf=function(t){J(t,2)},e.eb=function(t,e){j(t,0,e,0)},e.qd=function(t,e){G(t,1,+e,0)},e.ee=function(t){return q(t)},t.Int=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.Tf=function(t,e){return(e||new n).s(t.v(t.position())+t.position(),t)},n.prototype.precision=function(){var e=nt(this.a,this.b,4);return e?tt(this.a,this.b+e):t.c.f.g.Precision.HALF},n.af=function(t){J(t,1)},n.Lb=function(e,n){H(e,0,n,t.c.f.g.Precision.HALF)},n.ce=function(t){return q(t)},e.FloatingPoint=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.fg=function(t,n){return(n||new e).s(t.v(t.position())+t.position(),t)},e.sf=function(t){J(t,0)},e.se=function(t){return q(t)},t.Utf8=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.Kf=function(t,n){return(n||new e).s(t.v(t.position())+t.position(),t)},e.Oe=function(t){J(t,0)},e.Ud=function(t){return q(t)},t.Binary=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.Rf=function(t,n){return(n||new e).s(t.v(t.position())+t.position(),t)},e.prototype.byteWidth=function(){var t=nt(this.a,this.b,4);return t?this.a.v(this.b+t):0},e.Ze=function(t){J(t,1)},e.cd=function(t,e){j(t,0,e,0)},e.ae=function(t){return q(t)},t.FixedSizeBinary=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.Lf=function(t,n){return(n||new e).s(t.v(t.position())+t.position(),t)},e.Pe=function(t){J(t,0)},e.Vd=function(t){return q(t)},t.Bool=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.Nf=function(t,n){return(n||new e).s(t.v(t.position())+t.position(),t)},e.prototype.precision=function(){var t=nt(this.a,this.b,4);return t?this.a.v(this.b+t):0},e.prototype.scale=function(){var t=nt(this.a,this.b,6);return t?this.a.v(this.b+t):0},e.Te=function(t){J(t,2)},e.Lb=function(t,e){j(t,0,e,0)},e.zd=function(t,e){j(t,1,e,0)},e.Xd=function(t){return q(t)},t.Decimal=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.Mf=function(t,e){return(e||new n).s(t.v(t.position())+t.position(),t)},n.prototype.unit=function(){var e=nt(this.a,this.b,4);return e?tt(this.a,this.b+e):t.c.f.g.DateUnit.MILLISECOND},n.Se=function(t){J(t,1)},n.qa=function(e,n){H(e,0,n,t.c.f.g.DateUnit.MILLISECOND)},n.Wd=function(t){return q(t)},e.Date=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.cg=function(t,e){return(e||new n).s(t.v(t.position())+t.position(),t)},n.prototype.unit=function(){var e=nt(this.a,this.b,4);return e?tt(this.a,this.b+e):t.c.f.g.TimeUnit.MILLISECOND},n.prototype.bitWidth=function(){var t=nt(this.a,this.b,6);return t?this.a.v(this.b+t):32},n.startTime=function(t){J(t,2)},n.qa=function(e,n){H(e,0,n,t.c.f.g.TimeUnit.MILLISECOND)},n.eb=function(t,e){j(t,1,e,32)},n.endTime=function(t){return q(t)},e.Time=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.dg=function(t,e){return(e||new n).s(t.v(t.position())+t.position(),t)},n.prototype.unit=function(){var e=nt(this.a,this.b,4);return e?tt(this.a,this.b+e):t.c.f.g.TimeUnit.SECOND},n.prototype.timezone=function(t){var e=nt(this.a,this.b,6);return e?it(this.a,this.b+e,t):null},n.pf=function(t){J(t,2)},n.qa=function(e,n){H(e,0,n,t.c.f.g.TimeUnit.SECOND)},n.Bd=function(t,e){P(t,1,e)},n.qe=function(t){return q(t)},e.Timestamp=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.Vf=function(t,e){return(e||new n).s(t.v(t.position())+t.position(),t)},n.prototype.unit=function(){var e=nt(this.a,this.b,4);return e?tt(this.a,this.b+e):t.c.f.g.IntervalUnit.YEAR_MONTH},n.df=function(t){J(t,1)},n.qa=function(e,n){H(e,0,n,t.c.f.g.IntervalUnit.YEAR_MONTH)},n.fe=function(t){return q(t)},e.Interval=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.gg=function(t,e){return(e||new n).s(t.v(t.position())+t.position(),t)},n.prototype.bitWidth=function(){var t=nt(this.a,this.b,4);return t?tt(this.a,this.b+t):0},n.prototype.type=function(){var e=nt(this.a,this.b,6);return e?tt(this.a,this.b+e):t.c.f.g.VectorType.OFFSET},n.lg=function(t){J(t,2)},n.eb=function(t,e){H(t,0,e,0)},n.pc=function(e,n){H(e,1,n,t.c.f.g.VectorType.OFFSET)},n.If=function(t){return q(t)},e.Bf=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.Wf=function(t,n){return(n||new e).s(t.v(t.position())+t.position(),t)},e.prototype.key=function(t){var e=nt(this.a,this.b,4);return e?it(this.a,this.b+e,t):null},e.prototype.value=function(t){var e=nt(this.a,this.b,6);return e?it(this.a,this.b+e,t):null},e.Kc=function(t){J(t,2)},e.oc=function(t,e){P(t,0,e)},e.qc=function(t,e){P(t,1,e)},e.yc=function(t){return q(t)},t.sb=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.Pf=function(t,e){return(e||new n).s(t.v(t.position())+t.position(),t)},n.prototype.id=function(){var t=nt(this.a,this.b,4);return t?et(this.a,this.b+t):this.a.ha(0,0)},n.prototype.Ec=function(){var e=nt(this.a,this.b,6);return e?(new t.c.f.g.Int).s(ot(this.a,this.b+e),this.a):null},n.prototype.isOrdered=function(){var t=nt(this.a,this.b,8);return!!t&&!!(this.a.B[this.b+t]<<24>>24)},n.We=function(t){J(t,3)},n.Kb=function(t,e){Y(t,0,e,t.ha(0,0))},n.nd=function(t,e){P(t,1,e)},n.pd=function(t,e){G(t,2,+e,0)},n.Zd=function(t){return q(t)},e.ac=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.Qf=function(t,e){return(e||new n).s(t.v(t.position())+t.position(),t)},n.prototype.name=function(t){var e=nt(this.a,this.b,4);return e?it(this.a,this.b+e,t):null},n.prototype.nullable=function(){var t=nt(this.a,this.b,6);return!!t&&!!(this.a.B[this.b+t]<<24>>24)},n.prototype.uf=function(){var e=nt(this.a,this.b,8);return e?this.a.B[this.b+e]:t.c.f.g.Type.NONE},n.prototype.type=function(t){var e=nt(this.a,this.b,10);return e?rt(this.a,t,this.b+e):null},n.prototype.dictionary=function(e){var n=nt(this.a,this.b,12);return n?(e||new t.c.f.g.ac).s(ot(this.a,this.b+n),this.a):null},n.prototype.children=function(e,n){var r=nt(this.a,this.b,14);return r?(n||new t.c.f.g.Field).s(ot(this.a,at(this.a,this.b+r)+4*e),this.a):null},n.prototype.Nd=function(){var t=nt(this.a,this.b,14);return t?st(this.a,this.b+t):0},n.prototype.wc=function(e){var n=nt(this.a,this.b,18);return n?(new t.c.f.g.sb).s(ot(this.a,at(this.a,this.b+n)+4*e),this.a):null},n.prototype.xc=function(){var t=nt(this.a,this.b,18);return t?st(this.a,this.b+t):0},n.Xe=function(t){J(t,8)},n.vd=function(t,e){P(t,0,e)},n.xd=function(t,e){G(t,1,+e,0)},n.Dd=function(e,n){G(e,2,n,t.c.f.g.Type.NONE)},n.pc=function(t,e){P(t,3,e)},n.hd=function(t,e){P(t,4,e)},n.dd=function(t,e){P(t,5,e)},n.Pd=function(t,e){V(t,4,e.length,4);for(var n=e.length-1;0<=n;n--)z(t,e[n]);return W(t)},n.jg=function(t,e){V(t,4,e,4)},n.Cf=function(t,e){P(t,6,e)},n.Hf=function(t,e){V(t,4,e.length,4);for(var n=e.length-1;0<=n;n--)z(t,e[n]);return W(t)},n.kg=function(t,e){V(t,4,e,4)},n.Jb=function(t,e){P(t,7,e)},n.Pb=function(t,e){V(t,4,e.length,4);for(var n=e.length-1;0<=n;n--)z(t,e[n]);return W(t)},n.Re=function(t,e){V(t,4,e,4)},n.$d=function(t){return q(t)},e.Field=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.prototype.offset=function(){return et(this.a,this.b)},e.prototype.length=function(){return et(this.a,this.b+8)},e.createBuffer=function(t,e,n){return T(t,8,16),t.ia(n),t.ia(e),t.offset()},t.$b=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(ue||(ue={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.ag=function(t,e){return(e||new n).s(t.v(t.position())+t.position(),t)},n.prototype.fields=function(e,n){var r=nt(this.a,this.b,6);return r?(n||new t.c.f.g.Field).s(ot(this.a,at(this.a,this.b+r)+4*e),this.a):null},n.prototype.ve=function(){var t=nt(this.a,this.b,6);return t?st(this.a,this.b+t):0},n.prototype.wc=function(e){var n=nt(this.a,this.b,8);return n?(new t.c.f.g.sb).s(ot(this.a,at(this.a,this.b+n)+4*e),this.a):null},n.prototype.xc=function(){var t=nt(this.a,this.b,8);return t?st(this.a,this.b+t):0},n.mf=function(t){J(t,3)},n.jd=function(e){H(e,0,wl?$A.Rc:$A.zf,t.c.f.g.rb.Rc)},n.kd=function(t,e){P(t,1,e)},n.Rd=function(t,e){V(t,4,e.length,4);for(var n=e.length-1;0<=n;n--)z(t,e[n]);return W(t)},n.Ye=function(t,e){V(t,4,e,4)},n.Jb=function(t,e){P(t,2,e)},n.Pb=function(t,e){V(t,4,e.length,4);for(var n=e.length-1;0<=n;n--)z(t,e[n]);return W(t)},n.Re=function(t,e){V(t,4,e,4)},n.oe=function(t){return q(t)},n.Jf=function(t,e){t.finish(e)},e.Schema=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(ue||(ue={}));var rn,on=ue,an=rn||(rn={}),sn=an.c||(an.c={}),An=sn.f||(sn.f={});(An.g||(An.g={})).Schema=on.c.f.g.Schema;var ln=rn||(rn={}),cn=ln.c||(ln.c={}),un=cn.f||(cn.f={}),gn=un.g||(un.g={}),hn=gn.MessageHeader||(gn.MessageHeader={});hn[hn.NONE=0]="NONE",hn[hn.Schema=1]="Schema",hn[hn.DictionaryBatch=2]="DictionaryBatch",hn[hn.RecordBatch=3]="RecordBatch",hn[hn.Tensor=4]="Tensor",function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.prototype.length=function(){return et(this.a,this.b)},e.prototype.nullCount=function(){return et(this.a,this.b+8)},e.Qd=function(t,e,n){return T(t,8,16),t.ia(n),t.ia(e),t.offset()},t.bc=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(rn||(rn={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.$f=function(t,e){return(e||new n).s(t.v(t.position())+t.position(),t)},n.prototype.length=function(){var t=nt(this.a,this.b,4);return t?et(this.a,this.b+t):this.a.ha(0,0)},n.prototype.U=function(e,n){var r=nt(this.a,this.b,6);return r?(n||new t.c.f.g.bc).s(at(this.a,this.b+r)+16*e,this.a):null},n.prototype.Fe=function(){var t=nt(this.a,this.b,6);return t?st(this.a,this.b+t):0},n.prototype.buffers=function(t,e){var n=nt(this.a,this.b,8);return n?(e||new on.c.f.g.$b).s(at(this.a,this.b+n)+16*t,this.a):null},n.prototype.Ld=function(){var t=nt(this.a,this.b,8);return t?st(this.a,this.b+t):0},n.kf=function(t){J(t,3)},n.sd=function(t,e){Y(t,0,e,t.ha(0,0))},n.wd=function(t,e){P(t,1,e)},n.hf=function(t,e){V(t,16,e,8)},n.bd=function(t,e){P(t,2,e)},n.Qe=function(t,e){V(t,16,e,8)},n.me=function(t){return q(t)},e.RecordBatch=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(rn||(rn={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.Of=function(t,e){return(e||new n).s(t.v(t.position())+t.position(),t)},n.prototype.id=function(){var t=nt(this.a,this.b,4);return t?et(this.a,this.b+t):this.a.ha(0,0)},n.prototype.data=function(e){var n=nt(this.a,this.b,6);return n?(e||new t.c.f.g.RecordBatch).s(ot(this.a,this.b+n),this.a):null},n.prototype.ib=function(){var t=nt(this.a,this.b,8);return!!t&&!!(this.a.B[this.b+t]<<24>>24)},n.Ve=function(t){J(t,3)},n.Kb=function(t,e){Y(t,0,e,t.ha(0,0))},n.ed=function(t,e){P(t,1,e)},n.od=function(t,e){G(t,2,+e,0)},n.Yd=function(t){return q(t)},e.DictionaryBatch=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(rn||(rn={})),function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.Ae=function(t){return(new n).s(t.v(t.position())+t.position(),t)},n.prototype.version=function(){var t=nt(this.a,this.b,4);return t?tt(this.a,this.b+t):on.c.f.g.MetadataVersion.V1},n.prototype.headerType=function(){var e=nt(this.a,this.b,6);return e?this.a.B[this.b+e]:t.c.f.g.MessageHeader.NONE},n.prototype.header=function(t){var e=nt(this.a,this.b,8);return e?rt(this.a,t,this.b+e):null},n.prototype.bodyLength=function(){var t=nt(this.a,this.b,10);return t?et(this.a,this.b+t):this.a.ha(0,0)},n.gf=function(t){J(t,4)},n.Mb=function(t,e){H(t,0,e,on.c.f.g.MetadataVersion.V1)},n.md=function(e,n){G(e,1,n,t.c.f.g.MessageHeader.NONE)},n.ld=function(t,e){P(t,2,e)},n.ad=function(t,e){Y(t,3,e,t.ha(0,0))},n.je=function(t){return q(t)},n.xe=function(t,e){t.finish(e)},e.Message=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(rn||(rn={}));var dn,fn=rn,pn=on.c.f.g.Type,In=on.c.f.g.DateUnit,Cn=on.c.f.g.TimeUnit,mn=on.c.f.g.Precision,yn=on.c.f.g.UnionMode,En=on.c.f.g.VectorType,vn=on.c.f.g.IntervalUnit,Bn=fn.c.f.g.MessageHeader,bn=on.c.f.g.MetadataVersion,Qn=dn||(dn={});Qn[Qn.NONE=0]="NONE",Qn[Qn.Null=1]="Null",Qn[Qn.Int=2]="Int",Qn[Qn.Float=3]="Float",Qn[Qn.Binary=4]="Binary",Qn[Qn.Utf8=5]="Utf8",Qn[Qn.Bool=6]="Bool",Qn[Qn.Decimal=7]="Decimal",Qn[Qn.Date=8]="Date",Qn[Qn.Time=9]="Time",Qn[Qn.Timestamp=10]="Timestamp",Qn[Qn.Interval=11]="Interval",Qn[Qn.List=12]="List",Qn[Qn.Struct=13]="Struct",Qn[Qn.Union=14]="Union",Qn[Qn.FixedSizeBinary=15]="FixedSizeBinary",Qn[Qn.FixedSizeList=16]="FixedSizeList",Qn[Qn.Map=17]="Map",Qn[Qn.Dictionary=-1]="Dictionary",Qn[Qn.Int8=-2]="Int8",Qn[Qn.Int16=-3]="Int16",Qn[Qn.Int32=-4]="Int32",Qn[Qn.Int64=-5]="Int64",Qn[Qn.Uint8=-6]="Uint8",Qn[Qn.Uint16=-7]="Uint16",Qn[Qn.Uint32=-8]="Uint32",Qn[Qn.Uint64=-9]="Uint64",Qn[Qn.Float16=-10]="Float16",Qn[Qn.Float32=-11]="Float32",Qn[Qn.Float64=-12]="Float64",Qn[Qn.DateDay=-13]="DateDay",Qn[Qn.DateMillisecond=-14]="DateMillisecond",Qn[Qn.TimestampSecond=-15]="TimestampSecond",Qn[Qn.TimestampMillisecond=-16]="TimestampMillisecond",Qn[Qn.TimestampMicrosecond=-17]="TimestampMicrosecond",Qn[Qn.TimestampNanosecond=-18]="TimestampNanosecond",Qn[Qn.TimeSecond=-19]="TimeSecond",Qn[Qn.TimeMillisecond=-20]="TimeMillisecond",Qn[Qn.TimeMicrosecond=-21]="TimeMicrosecond",Qn[Qn.TimeNanosecond=-22]="TimeNanosecond",Qn[Qn.DenseUnion=-23]="DenseUnion",Qn[Qn.SparseUnion=-24]="SparseUnion",Qn[Qn.IntervalDayTime=-25]="IntervalDayTime",Qn[Qn.IntervalYearMonth=-26]="IntervalYearMonth";var wn,xn=dn;!function(t){!function(e){!function(e){!function(e){function n(){this.b=0}n.prototype.s=function(t,e){return this.b=t,this.a=e,this},n.ze=function(t){return(new n).s(t.v(t.position())+t.position(),t)},n.prototype.version=function(){var t=nt(this.a,this.b,4);return t?tt(this.a,this.b+t):on.c.f.g.MetadataVersion.V1},n.prototype.schema=function(t){var e=nt(this.a,this.b,6);return e?(t||new on.c.f.g.Schema).s(ot(this.a,this.b+e),this.a):null},n.prototype.dictionaries=function(e,n){var r=nt(this.a,this.b,8);return r?(n||new t.c.f.g.qb).s(at(this.a,this.b+r)+24*e,this.a):null},n.prototype.Td=function(){var t=nt(this.a,this.b,8);return t?st(this.a,this.b+t):0},n.prototype.Ub=function(e,n){var r=nt(this.a,this.b,10);return r?(n||new t.c.f.g.qb).s(at(this.a,this.b+r)+24*e,this.a):null},n.prototype.Ie=function(){var t=nt(this.a,this.b,10);return t?st(this.a,this.b+t):0},n.bf=function(t){J(t,4)},n.Mb=function(t,e){H(t,0,e,on.c.f.g.MetadataVersion.V1)},n.Ad=function(t,e){P(t,1,e)},n.gd=function(t,e){P(t,2,e)},n.Ue=function(t,e){V(t,24,e,8)},n.yd=function(t,e){P(t,3,e)},n.lf=function(t,e){V(t,24,e,8)},n.de=function(t){return q(t)},n.we=function(t,e){t.finish(e)},e.Qc=n}(e.g||(e.g={}))}(e.f||(e.f={}))}(t.c||(t.c={}))}(wn||(wn={})),function(t){!function(t){!function(t){!function(t){function e(){this.b=0}e.prototype.s=function(t,e){return this.b=t,this.a=e,this},e.prototype.offset=function(){return et(this.a,this.b)},e.prototype.Rb=function(){return this.a.v(this.b+8)},e.prototype.bodyLength=function(){return et(this.a,this.b+16)},e.Od=function(t,e,n,r){return T(t,8,24),t.ia(r),U(t,4),t.X(n),t.ia(e),t.offset()},t.qb=e}(t.g||(t.g={}))}(t.f||(t.f={}))}(t.c||(t.c={}))}(wn||(wn={}));var _n=wn;function Dn(t,e,n,r){return 0!=(n&1<<r)}function Mn(t,e,n,r){return(n&1<<r)>>r}function kn(t){for(var e=[],n=0,r=0,i=0,a=(t=o.w(t)).next();!a.done;a=t.next())a.value&&(i|=1<<r),8==++r&&(e[n++]=i,i=r=0);return(0===n||0<r)&&(e[n++]=i),(n=new Uint8Array(e.length+7&-8)).set(e),n}function Sn(t,e,n,r,i){var a,s,A,c,u;return o.C.H(Sn,function(o){switch(o.i){case 1:a=e%8,s=e>>3,A=0,c=n;case 2:if(!(0<c)){o.j(0);break}u=t[s++];case 5:return l(o,i(r,A++,u,a),7);case 7:0<--c&&8>++a?o.j(5):(a=0,o.j(2))}})}function Ln(t,e,n){if(0>=n-e)return 0;if(8>n-e){var r=0;for(e=(t=o.w(Sn(t,e,n-e,t,Mn))).next();!e.done;e=t.next())r+=e.value;return r}r=n>>3<<3;var i=e+(0==e%8?0:8-e%8);return Ln(t,e,i)+Ln(t,r,n)+Rn(t,i>>3,r-i>>3)}function Rn(t,e,n){var r=0;e|=0;var i=new DataView(t.buffer,t.byteOffset,t.byteLength);for(t=void 0===n?t.byteLength:e+n;4<=t-e;)r+=Nn(i.getUint32(e)),e+=4;for(;2<=t-e;)r+=Nn(i.getUint16(e)),e+=2;for(;1<=t-e;)r+=Nn(i.getUint8(e)),e+=1;return r}function Nn(t){return t|=0,16843009*((t=(858993459&(t-=t>>>1&1431655765))+(t>>>2&858993459))+(t>>>4)&252645135)>>>24}var Fn={};function Tn(t,e,n,r,i,o){var a;if(this.type=t,this.offset=Math.floor(Math.max(e||0,0)),this.length=Math.floor(Math.max(n||0,0)),this.oa=Math.floor(Math.max(r||0,-1)),this.childData=(o||[]).map(function(t){return t instanceof Tn?t:t.data}),i instanceof Tn)this.stride=i.stride,this.values=i.values,this.typeIds=i.typeIds,this.nullBitmap=i.nullBitmap,this.valueOffsets=i.valueOffsets;else switch(i&&((a=i[0])&&(this.valueOffsets=a),(a=i[1])&&(this.values=a),(a=i[2])&&(this.nullBitmap=a),(a=i[3])&&(this.typeIds=a)),t.typeId){case xn.Decimal:this.stride=4;break;case xn.Timestamp:this.stride=2;break;case xn.Date:case xn.Interval:this.stride=1+t.unit;break;case xn.Int:case xn.Time:this.stride=+(32<t.bitWidth)+1;break;case xn.FixedSizeList:this.stride=t.listSize;break;case xn.FixedSizeBinary:this.stride=t.byteWidth;break;default:this.stride=1}}function Un(t,e){var n=t.length,r=t.nullCount,i=new Uint8Array((e+63&-64)>>3).fill(255,0,n>>3);i[n>>3]=(1<<n-(-8&n))-1,0<r&&i.set(Fn.truncateBitmap(t.offset,n,t.nullBitmap),0);var o=t.buffers;return o[En.VALIDITY]=i,t.clone(t.type,0,e,r+(e-n),o)}function On(t,e,n){return t.map(function(t){return t.slice(e,n)})}function Gn(){}function Hn(t,e,n){n=void 0===n||n;var r=null,i=xn.NONE;switch(e instanceof Tn?i=jn(e.type):e instanceof ce?i=jn(e.type):e instanceof nr?i=jn(e):"number"!=typeof(i=e)&&(i=xn[e]),i){case xn.Null:r=t.visitNull;break;case xn.Bool:r=t.visitBool;break;case xn.Int:r=t.visitInt;break;case xn.Int8:r=t.visitInt8||t.visitInt;break;case xn.Int16:r=t.visitInt16||t.visitInt;break;case xn.Int32:r=t.visitInt32||t.visitInt;break;case xn.Int64:r=t.visitInt64||t.visitInt;break;case xn.Uint8:r=t.visitUint8||t.visitInt;break;case xn.Uint16:r=t.visitUint16||t.visitInt;break;case xn.Uint32:r=t.visitUint32||t.visitInt;break;case xn.Uint64:r=t.visitUint64||t.visitInt;break;case xn.Float:r=t.visitFloat;break;case xn.Float16:r=t.visitFloat16||t.visitFloat;break;case xn.Float32:r=t.visitFloat32||t.visitFloat;break;case xn.Float64:r=t.visitFloat64||t.visitFloat;break;case xn.Utf8:r=t.visitUtf8;break;case xn.Binary:r=t.visitBinary;break;case xn.FixedSizeBinary:r=t.visitFixedSizeBinary;break;case xn.Date:r=t.visitDate;break;case xn.DateDay:r=t.visitDateDay||t.visitDate;break;case xn.DateMillisecond:r=t.visitDateMillisecond||t.visitDate;break;case xn.Timestamp:r=t.visitTimestamp;break;case xn.TimestampSecond:r=t.visitTimestampSecond||t.visitTimestamp;break;case xn.TimestampMillisecond:r=t.visitTimestampMillisecond||t.visitTimestamp;break;case xn.TimestampMicrosecond:r=t.visitTimestampMicrosecond||t.visitTimestamp;break;case xn.TimestampNanosecond:r=t.visitTimestampNanosecond||t.visitTimestamp;break;case xn.Time:r=t.visitTime;break;case xn.TimeSecond:r=t.visitTimeSecond||t.visitTime;break;case xn.TimeMillisecond:r=t.visitTimeMillisecond||t.visitTime;break;case xn.TimeMicrosecond:r=t.visitTimeMicrosecond||t.visitTime;break;case xn.TimeNanosecond:r=t.visitTimeNanosecond||t.visitTime;break;case xn.Decimal:r=t.visitDecimal;break;case xn.List:r=t.visitList;break;case xn.Struct:r=t.visitStruct;break;case xn.Union:r=t.visitUnion;break;case xn.DenseUnion:r=t.visitDenseUnion||t.visitUnion;break;case xn.SparseUnion:r=t.visitSparseUnion||t.visitUnion;break;case xn.Dictionary:r=t.visitDictionary;break;case xn.Interval:r=t.visitInterval;break;case xn.IntervalDayTime:r=t.visitIntervalDayTime||t.visitInterval;break;case xn.IntervalYearMonth:r=t.visitIntervalYearMonth||t.visitInterval;break;case xn.FixedSizeList:r=t.visitFixedSizeList;break;case xn.Map:r=t.visitMap}if("function"==typeof r)return r;if(!n)return function(){return null};throw Error("Unrecognized type '"+xn[i]+"'")}function jn(t){switch(t.typeId){case xn.Null:return xn.Null;case xn.Int:var e=t.isSigned;switch(t.bitWidth){case 8:return e?xn.Int8:xn.Uint8;case 16:return e?xn.Int16:xn.Uint16;case 32:return e?xn.Int32:xn.Uint32;case 64:return e?xn.Int64:xn.Uint64}return xn.Int;case xn.Float:switch(t.precision){case mn.HALF:return xn.Float16;case mn.SINGLE:return xn.Float32;case mn.DOUBLE:return xn.Float64}return xn.Float;case xn.Binary:return xn.Binary;case xn.Utf8:return xn.Utf8;case xn.Bool:return xn.Bool;case xn.Decimal:return xn.Decimal;case xn.Time:switch(t.unit){case Cn.SECOND:return xn.TimeSecond;case Cn.MILLISECOND:return xn.TimeMillisecond;case Cn.MICROSECOND:return xn.TimeMicrosecond;case Cn.NANOSECOND:return xn.TimeNanosecond}return xn.Time;case xn.Timestamp:switch(t.unit){case Cn.SECOND:return xn.TimestampSecond;case Cn.MILLISECOND:return xn.TimestampMillisecond;case Cn.MICROSECOND:return xn.TimestampMicrosecond;case Cn.NANOSECOND:return xn.TimestampNanosecond}return xn.Timestamp;case xn.Date:switch(t.unit){case In.DAY:return xn.DateDay;case In.MILLISECOND:return xn.DateMillisecond}return xn.Date;case xn.Interval:switch(t.unit){case vn.DAY_TIME:return xn.IntervalDayTime;case vn.YEAR_MONTH:return xn.IntervalYearMonth}return xn.Interval;case xn.Map:return xn.Map;case xn.List:return xn.List;case xn.Struct:return xn.Struct;case xn.Union:switch(t.mode){case yn.Dense:return xn.DenseUnion;case yn.Sparse:return xn.SparseUnion}return xn.Union;case xn.FixedSizeBinary:return xn.FixedSizeBinary;case xn.FixedSizeList:return xn.FixedSizeList;case xn.Dictionary:return xn.Dictionary}throw Error("Unrecognized type '"+xn[t.typeId]+"'")}function Yn(){}function Pn(t,e){return t===e||Array.isArray(t)&&Array.isArray(e)&&t.length===e.length&&t.every(function(t,n){return Kn(t,e[n])})}function Kn(t,e){return t===e||e instanceof t.constructor&&t.name===e.name&&t.nullable===e.nullable&&er.visit(t.type,e.type)}function zn(t,e){return e instanceof t.constructor}function Jn(t,e){return t===e||zn(t,e)}function qn(t,e){return t===e||zn(t,e)&&t.bitWidth===e.bitWidth&&t.isSigned===e.isSigned}function Vn(t,e){return t===e||zn(t,e)&&t.precision===e.precision}function Wn(t,e){return t===e||zn(t,e)&&t.unit===e.unit}function Xn(t,e){return t===e||zn(t,e)&&t.unit===e.unit&&t.timezone===e.timezone}function Zn(t,e){return t===e||zn(t,e)&&t.unit===e.unit&&t.bitWidth===e.bitWidth}function $n(t,e){return t===e||zn(t,e)&&t.mode===e.mode&&t.typeIds.every(function(t,n){return t===e.typeIds[n]})&&Pn(t.children,e.children)}function tr(t,e){return t===e||zn(t,e)&&t.unit===e.unit}Fn.getBool=Dn,Fn.getBit=Mn,Fn.setBool=function(t,e,n){return n?!!(t[e>>3]|=1<<e%8)||!0:!(t[e>>3]&=~(1<<e%8))&&!1},Fn.truncateBitmap=function(t,e,n){var r=n.byteLength+7&-8;if(0<t||n.byteLength<r){var i=new Uint8Array(r);return i.set(0==t%8?n.subarray(t>>3):kn(Sn(n,t,e,null,Dn)).subarray(0,r)),i}return n},Fn.packBools=kn,Fn.iterateBits=Sn,Fn.popcnt_bit_range=Ln,Fn.popcnt_array=Rn,Fn.popcnt_uint32=Nn,Tn.prototype.clone=function(t,e,n,r,i,o){return new Tn(t,e=void 0===e?this.offset:e,n=void 0===n?this.length:n,r=void 0===r?this.oa:r,void 0===i?this:i,o=void 0===o?this.childData:o)},Tn.prototype.slice=function(t,e){var n,r=this.stride,i=this.typeId,o=this.childData,a=+(0===this.oa)-1,s=16===i?r:1,A=this.buffers;return(n=A[En.TYPE])&&(A[En.TYPE]=n.subarray(t,t+e)),(n=A[En.OFFSET])&&(A[En.OFFSET]=n.subarray(t,t+e+1))||(n=A[En.DATA])&&(A[En.DATA]=6===i?n:n.subarray(r*t,r*(t+e))),this.clone(this.type,this.offset+t,e,a,A,!o.length||this.valueOffsets?o:On(o,s*t,s*e))},Tn.new=function(t,e,n,r,i,o){switch(i instanceof Tn?i=i.buffers:i||(i=[]),t.typeId){case xn.Null:return Tn.Null(t,e,n,r||0,i[2]);case xn.Int:return Tn.Int(t,e,n,r||0,i[2],i[1]||[]);case xn.Dictionary:return Tn.Dictionary(t,e,n,r||0,i[2],i[1]||[]);case xn.Float:return Tn.Float(t,e,n,r||0,i[2],i[1]||[]);case xn.Bool:return Tn.Bool(t,e,n,r||0,i[2],i[1]||[]);case xn.Decimal:return Tn.Decimal(t,e,n,r||0,i[2],i[1]||[]);case xn.Date:return Tn.Date(t,e,n,r||0,i[2],i[1]||[]);case xn.Time:return Tn.Time(t,e,n,r||0,i[2],i[1]||[]);case xn.Timestamp:return Tn.Timestamp(t,e,n,r||0,i[2],i[1]||[]);case xn.Interval:return Tn.Interval(t,e,n,r||0,i[2],i[1]||[]);case xn.FixedSizeBinary:return Tn.FixedSizeBinary(t,e,n,r||0,i[2],i[1]||[]);case xn.Binary:return Tn.Binary(t,e,n,r||0,i[2],i[0]||[],i[1]||[]);case xn.Utf8:return Tn.Utf8(t,e,n,r||0,i[2],i[0]||[],i[1]||[]);case xn.List:return Tn.List(t,e,n,r||0,i[2],i[0]||[],(o||[])[0]);case xn.FixedSizeList:return Tn.FixedSizeList(t,e,n,r||0,i[2],(o||[])[0]);case xn.Struct:return Tn.Struct(t,e,n,r||0,i[2],o||[]);case xn.Map:return Tn.Map(t,e,n,r||0,i[2],o||[]);case xn.Union:return Tn.Union(t,e,n,r||0,i[2],i[3]||[],i[1]||o,o)}throw Error("Unrecognized typeId "+t.typeId)},Tn.Null=function(t,e,n,r,i){return new Tn(t,e,n,r,[void 0,void 0,Wt.toUint8Array(i)])},Tn.Int=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,Wt.toArrayBufferView(t.ArrayType,o),Wt.toUint8Array(i)])},Tn.Dictionary=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,Wt.toArrayBufferView(t.indices.ArrayType,o),Wt.toUint8Array(i)])},Tn.Float=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,Wt.toArrayBufferView(t.ArrayType,o),Wt.toUint8Array(i)])},Tn.Bool=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,Wt.toArrayBufferView(t.ArrayType,o),Wt.toUint8Array(i)])},Tn.Decimal=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,Wt.toArrayBufferView(t.ArrayType,o),Wt.toUint8Array(i)])},Tn.Date=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,Wt.toArrayBufferView(t.ArrayType,o),Wt.toUint8Array(i)])},Tn.Time=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,Wt.toArrayBufferView(t.ArrayType,o),Wt.toUint8Array(i)])},Tn.Timestamp=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,Wt.toArrayBufferView(t.ArrayType,o),Wt.toUint8Array(i)])},Tn.Interval=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,Wt.toArrayBufferView(t.ArrayType,o),Wt.toUint8Array(i)])},Tn.FixedSizeBinary=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,Wt.toArrayBufferView(t.ArrayType,o),Wt.toUint8Array(i)])},Tn.Binary=function(t,e,n,r,i,o,a){return new Tn(t,e,n,r,[Wt.toInt32Array(o),Wt.toUint8Array(a),Wt.toUint8Array(i)])},Tn.Utf8=function(t,e,n,r,i,o,a){return new Tn(t,e,n,r,[Wt.toInt32Array(o),Wt.toUint8Array(a),Wt.toUint8Array(i)])},Tn.List=function(t,e,n,r,i,o,a){return new Tn(t,e,n,r,[Wt.toInt32Array(o),void 0,Wt.toUint8Array(i)],[a])},Tn.FixedSizeList=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,void 0,Wt.toUint8Array(i)],[o])},Tn.Struct=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,void 0,Wt.toUint8Array(i)],o)},Tn.Map=function(t,e,n,r,i,o){return new Tn(t,e,n,r,[void 0,void 0,Wt.toUint8Array(i)],o)},Tn.Union=function(t,e,n,r,i,o,a,s){return i=[void 0,void 0,Wt.toUint8Array(i),Wt.toArrayBufferView(t.ArrayType,o)],t.mode===yn.Sparse?new Tn(t,e,n,r,i,a):(i[En.OFFSET]=Wt.toInt32Array(a),new Tn(t,e,n,r,i,s))},o.global.Object.defineProperties(Tn.prototype,{ArrayType:{configurable:!0,enumerable:!0,get:function(){return this.type.ArrayType}},typeId:{configurable:!0,enumerable:!0,get:function(){return this.type.typeId}},buffers:{configurable:!0,enumerable:!0,get:function(){return[this.valueOffsets,this.values,this.nullBitmap,this.typeIds]}},nullCount:{configurable:!0,enumerable:!0,get:function(){var t,e=this.oa;return-1>=e&&(t=this.nullBitmap)&&(this.oa=e=this.length-Fn.popcnt_bit_range(t,this.offset,this.offset+this.length)),e}}}),Tn.prototype.childData=Object.freeze([]),(r=Gn.prototype).visitMany=function(t,e){for(var n=[],r=1;r<arguments.length;++r)n[r-1]=arguments[r];var i=this;return t.map(function(t,e){return i.visit.apply(i,[t].concat(o.o(n.map(function(t){return t[e]}))))})},r.visit=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return this.getVisitFn(e[0],!1).apply(this,e)},r.getVisitFn=function(t,e){return Hn(this,t,void 0===e||e)},r.visitNull=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitBool=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitInt=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitFloat=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitUtf8=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitBinary=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitFixedSizeBinary=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitDate=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitTimestamp=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitTime=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitDecimal=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitList=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitStruct=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitUnion=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitDictionary=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitInterval=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitFixedSizeList=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitMap=function(t,e){for(var n=1;n<arguments.length;++n);return null},r.visitInt8=null,r.visitInt16=null,r.visitInt32=null,r.visitInt64=null,r.visitUint8=null,r.visitUint16=null,r.visitUint32=null,r.visitUint64=null,r.visitFloat16=null,r.visitFloat32=null,r.visitFloat64=null,r.visitDateDay=null,r.visitDateMillisecond=null,r.visitTimestampSecond=null,r.visitTimestampMillisecond=null,r.visitTimestampMicrosecond=null,r.visitTimestampNanosecond=null,r.visitTimeSecond=null,r.visitTimeMillisecond=null,r.visitTimeMicrosecond=null,r.visitTimeNanosecond=null,r.visitDenseUnion=null,r.visitSparseUnion=null,r.visitIntervalDayTime=null,r.visitIntervalYearMonth=null,o.h(Yn,Gn),(r=Yn.prototype).visitNull=Jn,r.visitBool=Jn,r.visitInt=qn,r.visitInt8=qn,r.visitInt16=qn,r.visitInt32=qn,r.visitInt64=qn,r.visitUint8=qn,r.visitUint16=qn,r.visitUint32=qn,r.visitUint64=qn,r.visitFloat=Vn,r.visitFloat16=Vn,r.visitFloat32=Vn,r.visitFloat64=Vn,r.visitUtf8=Jn,r.visitBinary=Jn,r.visitFixedSizeBinary=function(t,e){return t===e||zn(t,e)&&t.byteWidth===e.byteWidth},r.visitDate=Wn,r.visitDateDay=Wn,r.visitDateMillisecond=Wn,r.visitTimestamp=Xn,r.visitTimestampSecond=Xn,r.visitTimestampMillisecond=Xn,r.visitTimestampMicrosecond=Xn,r.visitTimestampNanosecond=Xn,r.visitTime=Zn,r.visitTimeSecond=Zn,r.visitTimeMillisecond=Zn,r.visitTimeMicrosecond=Zn,r.visitTimeNanosecond=Zn,r.visitDecimal=Jn,r.visitList=function(t,e){return t===e||zn(t,e)&&t.children.length===e.children.length&&Pn(t.children,e.children)},r.visitStruct=function(t,e){return t===e||zn(t,e)&&t.children.length===e.children.length&&Pn(t.children,e.children)},r.visitUnion=$n,r.visitDenseUnion=$n,r.visitSparseUnion=$n,r.visitDictionary=function(t,e){return t===e||zn(t,e)&&t.id===e.id&&t.isOrdered===e.isOrdered&&er.visit(t.indices,e.indices)&&er.visit(t.dictionary,e.dictionary)},r.visitInterval=tr,r.visitIntervalDayTime=tr,r.visitIntervalYearMonth=tr,r.visitFixedSizeList=function(t,e){return t===e||zn(t,e)&&t.listSize===e.listSize&&t.children.length===e.children.length&&Pn(t.children,e.children)},r.visitMap=function(t,e){return t===e||zn(t,e)&&t.keysSorted===e.keysSorted&&t.children.length===e.children.length&&Pn(t.children,e.children)};var er=new Yn;function nr(){}nr.isNull=function(t){return t&&t.typeId===xn.Null},nr.isInt=function(t){return t&&t.typeId===xn.Int},nr.isFloat=function(t){return t&&t.typeId===xn.Float},nr.isBinary=function(t){return t&&t.typeId===xn.Binary},nr.isUtf8=function(t){return t&&t.typeId===xn.Utf8},nr.isBool=function(t){return t&&t.typeId===xn.Bool},nr.isDecimal=function(t){return t&&t.typeId===xn.Decimal},nr.isDate=function(t){return t&&t.typeId===xn.Date},nr.isTime=function(t){return t&&t.typeId===xn.Time},nr.isTimestamp=function(t){return t&&t.typeId===xn.Timestamp},nr.isInterval=function(t){return t&&t.typeId===xn.Interval},nr.isList=function(t){return t&&t.typeId===xn.List},nr.isStruct=function(t){return t&&t.typeId===xn.Struct},nr.isUnion=function(t){return t&&t.typeId===xn.Union},nr.isFixedSizeBinary=function(t){return t&&t.typeId===xn.FixedSizeBinary},nr.isFixedSizeList=function(t){return t&&t.typeId===xn.FixedSizeList},nr.isMap=function(t){return t&&t.typeId===xn.Map},nr.isDictionary=function(t){return t&&t.typeId===xn.Dictionary},nr.prototype.compareTo=function(t){return er.visit(this,t)},o.global.Object.defineProperties(nr.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.NONE}}}),o.l();var rr,ir=Symbol.toStringTag,or=nr.prototype;function ar(){}or.children=null,or.ArrayType=Array,o.l(),rr=or[Symbol.toStringTag]="DataType",nr[ir]=rr,o.h(ar,nr),ar.prototype.toString=function(){return"Null"},o.global.Object.defineProperties(ar.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Null}}}),o.l();var sr,Ar=Symbol.toStringTag,lr=ar.prototype;function cr(t,e){this.isSigned=t,this.bitWidth=e}o.l(),sr=lr[Symbol.toStringTag]="Null",ar[Ar]=sr,o.h(cr,nr),cr.prototype.toString=function(){return(this.isSigned?"I":"Ui")+"nt"+this.bitWidth},o.global.Object.defineProperties(cr.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Int}},ArrayType:{configurable:!0,enumerable:!0,get:function(){switch(this.bitWidth){case 8:return this.isSigned?Int8Array:Uint8Array;case 16:return this.isSigned?Int16Array:Uint16Array;case 32:case 64:return this.isSigned?Int32Array:Uint32Array}throw o.l(),Error("Unrecognized "+this[Symbol.toStringTag]+" type")}}}),o.l();var ur,gr=Symbol.toStringTag,hr=cr.prototype;function dr(){cr.call(this,!0,8)}function fr(){cr.call(this,!0,16)}function pr(){cr.call(this,!0,32)}function Ir(){cr.call(this,!0,64)}function Cr(){cr.call(this,!1,8)}function mr(){cr.call(this,!1,16)}function yr(){cr.call(this,!1,32)}function Er(){cr.call(this,!1,64)}function vr(t){this.precision=t}hr.isSigned=null,hr.bitWidth=null,o.l(),ur=hr[Symbol.toStringTag]="Int",cr[gr]=ur,o.h(dr,cr),o.h(fr,cr),o.h(pr,cr),o.h(Ir,cr),o.h(Cr,cr),o.h(mr,cr),o.h(yr,cr),o.h(Er,cr),Object.defineProperty(dr.prototype,"ArrayType",{value:Int8Array}),Object.defineProperty(fr.prototype,"ArrayType",{value:Int16Array}),Object.defineProperty(pr.prototype,"ArrayType",{value:Int32Array}),Object.defineProperty(Ir.prototype,"ArrayType",{value:Int32Array}),Object.defineProperty(Cr.prototype,"ArrayType",{value:Uint8Array}),Object.defineProperty(mr.prototype,"ArrayType",{value:Uint16Array}),Object.defineProperty(yr.prototype,"ArrayType",{value:Uint32Array}),Object.defineProperty(Er.prototype,"ArrayType",{value:Uint32Array}),o.h(vr,nr),vr.prototype.toString=function(){return"Float"+(this.precision<<5||16)},o.global.Object.defineProperties(vr.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Float}},ArrayType:{configurable:!0,enumerable:!0,get:function(){switch(this.precision){case mn.HALF:return Uint16Array;case mn.SINGLE:return Float32Array;case mn.DOUBLE:return Float64Array}throw o.l(),Error("Unrecognized "+this[Symbol.toStringTag]+" type")}}}),o.l();var Br,br=Symbol.toStringTag,Qr=vr.prototype;function wr(){this.precision=mn.HALF}function xr(){this.precision=mn.SINGLE}function _r(){this.precision=mn.DOUBLE}function Dr(){}Qr.precision=null,o.l(),Br=Qr[Symbol.toStringTag]="Float",vr[br]=Br,o.h(wr,vr),o.h(xr,vr),o.h(_r,vr),Object.defineProperty(wr.prototype,"ArrayType",{value:Uint16Array}),Object.defineProperty(xr.prototype,"ArrayType",{value:Float32Array}),Object.defineProperty(_r.prototype,"ArrayType",{value:Float64Array}),o.h(Dr,nr),Dr.prototype.toString=function(){return"Binary"},o.global.Object.defineProperties(Dr.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Binary}}}),o.l();var Mr,kr=Symbol.toStringTag,Sr=Dr.prototype;function Lr(){}Sr.ArrayType=Uint8Array,o.l(),Mr=Sr[Symbol.toStringTag]="Binary",Dr[kr]=Mr,o.h(Lr,nr),Lr.prototype.toString=function(){return"Utf8"},o.global.Object.defineProperties(Lr.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Utf8}}}),o.l();var Rr,Nr=Symbol.toStringTag,Fr=Lr.prototype;function Tr(){}Fr.ArrayType=Uint8Array,o.l(),Rr=Fr[Symbol.toStringTag]="Utf8",Lr[Nr]=Rr,o.h(Tr,nr),Tr.prototype.toString=function(){return"Bool"},o.global.Object.defineProperties(Tr.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Bool}}}),o.l();var Ur,Or=Symbol.toStringTag,Gr=Tr.prototype;function Hr(t,e){this.scale=t,this.precision=e}Gr.ArrayType=Uint8Array,o.l(),Ur=Gr[Symbol.toStringTag]="Bool",Tr[Or]=Ur,o.h(Hr,nr),Hr.prototype.toString=function(){return"Decimal["+this.precision+"e"+(0<this.scale?"+":"")+this.scale+"]"},o.global.Object.defineProperties(Hr.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Decimal}}}),o.l();var jr,Yr=Symbol.toStringTag,Pr=Hr.prototype;function Kr(t){this.unit=t}Pr.scale=null,Pr.precision=null,Pr.ArrayType=Uint32Array,o.l(),jr=Pr[Symbol.toStringTag]="Decimal",Hr[Yr]=jr,o.h(Kr,nr),Kr.prototype.toString=function(){return"Date"+32*(this.unit+1)+"<"+In[this.unit]+">"},o.global.Object.defineProperties(Kr.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Date}}}),o.l();var zr,Jr=Symbol.toStringTag,qr=Kr.prototype;function Vr(){this.unit=In.DAY}function Wr(){this.unit=In.MILLISECOND}function Xr(t,e){this.unit=t,this.bitWidth=e}qr.unit=null,qr.ArrayType=Int32Array,o.l(),zr=qr[Symbol.toStringTag]="Date",Kr[Jr]=zr,o.h(Vr,Kr),o.h(Wr,Kr),o.h(Xr,nr),Xr.prototype.toString=function(){return"Time"+this.bitWidth+"<"+Cn[this.unit]+">"},o.global.Object.defineProperties(Xr.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Time}}}),o.l();var Zr,$r=Symbol.toStringTag,ti=Xr.prototype;function ei(){Xr.call(this,Cn.SECOND,32)}function ni(){Xr.call(this,Cn.MILLISECOND,32)}function ri(){Xr.call(this,Cn.MICROSECOND,64)}function ii(){Xr.call(this,Cn.NANOSECOND,64)}function oi(t,e){this.unit=t,this.timezone=e}ti.unit=null,ti.bitWidth=null,ti.ArrayType=Int32Array,o.l(),Zr=ti[Symbol.toStringTag]="Time",Xr[$r]=Zr,o.h(ei,Xr),o.h(ni,Xr),o.h(ri,Xr),o.h(ii,Xr),o.h(oi,nr),oi.prototype.toString=function(){return"Timestamp<"+Cn[this.unit]+(this.timezone?", "+this.timezone:"")+">"},o.global.Object.defineProperties(oi.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Timestamp}}}),o.l();var ai,si=Symbol.toStringTag,Ai=oi.prototype;function li(t){oi.call(this,Cn.SECOND,t)}function ci(t){oi.call(this,Cn.MILLISECOND,t)}function ui(t){oi.call(this,Cn.MICROSECOND,t)}function gi(t){oi.call(this,Cn.NANOSECOND,t)}function hi(t){this.unit=t}Ai.unit=null,Ai.timezone=null,Ai.ArrayType=Int32Array,o.l(),ai=Ai[Symbol.toStringTag]="Timestamp",oi[si]=ai,o.h(li,oi),o.h(ci,oi),o.h(ui,oi),o.h(gi,oi),o.h(hi,nr),hi.prototype.toString=function(){return"Interval<"+vn[this.unit]+">"},o.global.Object.defineProperties(hi.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Interval}}}),o.l();var di,fi=Symbol.toStringTag,pi=hi.prototype;function Ii(){this.unit=vn.DAY_TIME}function Ci(){this.unit=vn.YEAR_MONTH}function mi(t){this.children=[t]}pi.unit=null,pi.ArrayType=Int32Array,o.l(),di=pi[Symbol.toStringTag]="Interval",hi[fi]=di,o.h(Ii,hi),o.h(Ci,hi),o.h(mi,nr),mi.prototype.toString=function(){return"List<"+this.valueType+">"},o.global.Object.defineProperties(mi.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.List}},valueType:{configurable:!0,enumerable:!0,get:function(){return this.children[0].type}},valueField:{configurable:!0,enumerable:!0,get:function(){return this.children[0]}},ArrayType:{configurable:!0,enumerable:!0,get:function(){return this.valueType.ArrayType}}}),o.l();var yi,Ei=Symbol.toStringTag,vi=mi.prototype;function Bi(t){this.children=this.children=t}vi.children=null,o.l(),yi=vi[Symbol.toStringTag]="List",mi[Ei]=yi,o.h(Bi,nr),Bi.prototype.toString=function(){return"Struct<["+this.children.map(function(t){return t.type}).join(", ")+"]>"},o.global.Object.defineProperties(Bi.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Struct}}}),o.l();var bi,Qi=Symbol.toStringTag,wi=Bi.prototype;function xi(t,e,n){this.mode=t,this.children=n,this.typeIds=e=Int32Array.from(e),this.typeIdToChildIndex=e.reduce(function(t,e,n){return(t[e]=n)&&t||t},Object.create(null))}wi.children=null,o.l(),bi=wi[Symbol.toStringTag]="Struct",Bi[Qi]=bi,o.h(xi,nr),xi.prototype.toString=function(){return o.l(),this[Symbol.toStringTag]+"<"+this.children.map(function(t){return""+t.type}).join(" | ")+">"},o.global.Object.defineProperties(xi.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Union}}}),o.l();var _i,Di=Symbol.toStringTag,Mi=xi.prototype;function ki(t,e){xi.call(this,yn.Dense,t,e)}function Si(t,e){xi.call(this,yn.Sparse,t,e)}function Li(t){this.byteWidth=t}Mi.mode=null,Mi.typeIds=null,Mi.children=null,Mi.typeIdToChildIndex=null,Mi.ArrayType=Int32Array,o.l(),_i=Mi[Symbol.toStringTag]="Union",xi[Di]=_i,o.h(ki,xi),o.h(Si,xi),o.h(Li,nr),Li.prototype.toString=function(){return"FixedSizeBinary["+this.byteWidth+"]"},o.global.Object.defineProperties(Li.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.FixedSizeBinary}}}),o.l();var Ri,Ni=Symbol.toStringTag,Fi=Li.prototype;function Ti(t,e){this.listSize=t,this.children=[e]}Fi.byteWidth=null,Fi.ArrayType=Uint8Array,o.l(),Ri=Fi[Symbol.toStringTag]="FixedSizeBinary",Li[Ni]=Ri,o.h(Ti,nr),Ti.prototype.toString=function(){return"FixedSizeList["+this.listSize+"]<"+this.valueType+">"},o.global.Object.defineProperties(Ti.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.FixedSizeList}},valueType:{configurable:!0,enumerable:!0,get:function(){return this.children[0].type}},valueField:{configurable:!0,enumerable:!0,get:function(){return this.children[0]}},ArrayType:{configurable:!0,enumerable:!0,get:function(){return this.valueType.ArrayType}}}),o.l();var Ui,Oi=Symbol.toStringTag,Gi=Ti.prototype;function Hi(t,e){this.children=t,this.keysSorted=void 0!==e&&e}Gi.children=null,Gi.listSize=null,o.l(),Ui=Gi[Symbol.toStringTag]="FixedSizeList",Ti[Oi]=Ui,o.h(Hi,nr),Hi.prototype.toString=function(){return"Map<{"+this.children.map(function(t){return t.name+":"+t.type}).join(", ")+"}>"},o.global.Object.defineProperties(Hi.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Map}}}),o.l();var ji,Yi=Symbol.toStringTag,Pi=Hi.prototype;Pi.children=null,Pi.keysSorted=null,o.l(),ji=Pi[Symbol.toStringTag]="Map_",Hi[Yi]=ji;var Ki=function(t){return function(){return++t}}(-1);function zi(t,e,n,r,i){this.indices=e,this.dictionary=t,this.isOrdered=r||!1,this.dictionaryVector=i,this.id=null==n?Ki():"number"==typeof n?n:n.low}o.h(zi,nr),zi.prototype.toString=function(){return"Dictionary<"+this.indices+", "+this.dictionary+">"},o.global.Object.defineProperties(zi.prototype,{typeId:{configurable:!0,enumerable:!0,get:function(){return xn.Dictionary}},children:{configurable:!0,enumerable:!0,get:function(){return this.dictionary.children}},valueType:{configurable:!0,enumerable:!0,get:function(){return this.dictionary}},ArrayType:{configurable:!0,enumerable:!0,get:function(){return this.dictionary.ArrayType}}}),o.l();var Ji,qi=Symbol.toStringTag,Vi=zi.prototype;function Wi(t){if(null===t)return"null";if(void 0===t)return"undefined";switch(typeof t){case"number":case"bigint":return""+t;case"string":return'"'+t+'"'}return o.l(),"function"==typeof t[Symbol.toPrimitive]?(o.l(),t[Symbol.toPrimitive]("string")):ArrayBuffer.isView(t)?"["+t+"]":JSON.stringify(t)}Vi.id=null,Vi.indices=null,Vi.isOrdered=null,Vi.dictionary=null,Vi.dictionaryVector=null,o.l(),Ji=Vi[Symbol.toStringTag]="Dictionary",zi[qi]=Ji,o.l();var Xi=Symbol.for("length");o.l();var Zi=Symbol.for("parent");o.l();var $i=Symbol.for("rowIndex"),to={enumerable:!0,configurable:!1,get:null},eo={writable:!1,enumerable:!1,configurable:!1,value:-1},no={writable:!1,enumerable:!1,configurable:!1,value:null};function ro(){}function io(t,e,n){var r=Object.create(ro.prototype);no.value=t,eo.value=e.length,Object.defineProperty(r,Zi,no),Object.defineProperty(r,Xi,eo),e.forEach(function(t,e){r.hasOwnProperty(t.name)||(to.enumerable=n,to.get||(to.get=oo(e)),Object.defineProperty(r,t.name,to)),r.hasOwnProperty(e)||(to.enumerable=!n,to.get||(to.get=oo(e)),Object.defineProperty(r,e,to)),to.get=null}),this.Le=r}function oo(t){return function(){var e=this[Zi].getChildAt(t);return e?e.get(this[$i]):null}}o.l(),o.K(),ro.prototype[Symbol.iterator]=function t(){var e,n,r=this;return o.C.H(t,function(t){return 1==t.i&&(e=-1,n=r[Xi]),++e<n?l(t,r[e],2):t.j(0)})},ro.prototype.get=function(t){return this[t]},ro.prototype.toJSON=function(){var t=this;return nr.isStruct(this[Zi].type)?[].concat(o.o(this)):Object.getOwnPropertyNames(this).reduce(function(e,n){return(e[n]=t[n])&&e||e},{})},ro.prototype.toString=function(){var t=this;return nr.isStruct(this[Zi].type)?[].concat(o.o(this)).map(function(t){return Wi(t)}).join(", "):Object.getOwnPropertyNames(this).reduce(function(e,n){return(e[n]=Wi(t[n]))&&e||e},{})},io.new=function(t,e,n){if(n=void 0!==n&&n,Array.isArray(e))var r=e;else n=!0,r=Object.keys(e).map(function(t){return new Mo(t,e[t])});return new io(t,r,n)},io.prototype.bind=function(t){var e=Object.create(this.Le);return e[$i]=t,e};var ao,so=_t?xt(0):0;function Ao(t){var e=typeof t;if("object"!==e||null===t)return"bigint"!==e?function(e){return e===t}:function(e){return so+e===t};if(t instanceof Date){var n=t.valueOf();return function(t){return t instanceof Date&&t.valueOf()===n}}return ArrayBuffer.isView(t)?function(e){return!!e&&Wt.compareArrayLike(t,e)}:Array.isArray(t)?lo(t):t instanceof ro?co(t):t instanceof ce?uo(t):0<(e=Object.keys(t)).length?go(t,e):function(){return!1}}function lo(t){for(var e=t.length,n=[],r=-1;++r<e;)n[r]=Ao(t[r]);return function(t){if(!t)return!1;if(t instanceof ro){if(t[Xi]!==e)return!1;for(var r=-1;++r<e;)if(!n[r](t.get(r)))return!1;return!0}if(t.length!==e)return!1;if(t instanceof ce){for(r=-1;++r<e;)if(!n[r](t.get(r)))return!1;return!0}for(r=-1;++r<e;)if(!n[r](t[r]))return!1;return!0}}function co(t){for(var e=t[Xi],n=t.constructor,r=[],i=-1;++i<e;)r[i]=Ao(t.get(i));return function(t){if(!(t instanceof n)||t[Xi]!==e)return!1;for(var i=-1;++i<e;)if(!r[i](t.get(i)))return!1;return!0}}function uo(t){for(var e=t.length,n=t.constructor,r=[],i=-1;++i<e;)r[i]=Ao(t.get(i));return function(t){if(!(t instanceof n)||t.length!==e)return!1;for(var i=-1;++i<e;)if(!r[i](t.get(i)))return!1;return!0}}function go(t,e){for(var n=e.length,r=[],i=-1;++i<n;)r[i]=Ao(t[e[i]]);return function(t){if(!t||"object"!=typeof t)return!1;for(var i=-1;++i<n;)if(!r[i](t[e[i]]))return!1;return!0}}var ho={clampIndex:function(t,e,n){var r=t.length;return e=-1<e?e:r+e%r,n?n(t,e):e},clampRange:function(t,e,n,r){var i=void 0===t.length?0:t.length;return 0>(e="number"!=typeof e?0:e)&&(e=(e%i+i)%i),0>(n="number"!=typeof n?i:n)&&(n=(n%i+i)%i),n<e&&(ao=e,e=n,n=ao),n>i&&(n=i),r?r(t,e,n):[e,n]}};function fo(t,e,n){if(e=void 0===e?[]:e,void 0===n){n=e;for(var r=new Uint32Array((n||[]).length+1),i=r[0]=0,o=r.length,a=0;++a<o;)r[a]=i+=n[a-1].length;n=r}this.oa=-1,this.$=t,this.I=e,this.xb=n,this.Fa=n[n.length-1],this.Cb=(this.$.children||[]).length}function po(t,e,n){return e.set(t,n),n+t.length}function Io(t,e,n){--n;for(var r=-1,i=t.length;++r<i;)e[++n]=t[r];return n}function Co(t,e,n){if(e=void 0===e?[]:e,e=fo.flatten.apply(fo,o.o(e)),fo.call(this,t.type,e,n),this.Da=t,1===e.length&&!(this instanceof mo))return new mo(t,e[0],this.xb)}function mo(t,e,n){return(t=Co.call(this,t,[e],n)||this).Za=e,t}ho.createElementComparator=Ao,o.l(),o.K(),o.h(fo,ce),fo.flatten=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return Bo(ce,e,[],0)},fo.concat=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return e=fo.flatten.apply(fo,o.o(e)),new fo(e[0].type,e)},fo.prototype[Symbol.iterator]=function t(){var e,n,r=this;return o.C.H(t,function(t){return 1==t.i&&(e=o.w(r.I),n=e.next()),3!=t.i?n.done?t.j(0):c(t,n.value,3):(n=e.next(),t.j(2))})},(r=fo.prototype).clone=function(t){return t=void 0===t?this.I:t,new fo(this.$,t)},r.concat=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return this.clone(fo.flatten.apply(fo,[this].concat(o.o(e))))},r.slice=function(t,e){return ho.clampRange(this,t,e,this.Gb)},r.getChildAt=function(t){if(0>t||t>=this.Cb)return null;var e,n=this.Y||(this.Y=[]);if(e=n[t])return e;if(e=(this.$.children||[])[t]){var r=this.I.map(function(e){return e.getChildAt(t)}).filter(function(t){return null!=t});if(0<r.length)return n[t]=new fo(e.type,r)}return null},r.search=function(t,e){var n=this.xb,r=n.length-1;if(0>t||t>=n[r])return null;if(1>=r)return e?e(this,0,t):[0,t];var i=0,o=0;do{if(i+1===r)return e?e(this,i,t-o):[i,t-o];var a=i+(r-i)/2|0;t>=n[a]?i=a:r=a}while(t<n[r]&&t>=(o=n[i]));return null},r.isValid=function(t){return!!this.search(t,this.isValidInternal)},r.get=function(t){return this.search(t,this.getInternal)},r.set=function(t,e){this.search(t,function(t,n,r){return t.chunks[n].set(r,e)})},r.indexOf=function(t,e){var n=this;return e&&"number"==typeof e?this.search(e,function(e,r,i){return n.indexOfInternal(e,r,i,t)}):this.indexOfInternal(this,0,Math.max(0,e||0),t)},r.toArray=function(){var t=this.chunks,e=t.length,n=this.$.ArrayType;if(0>=e)return new n(0);if(1>=e)return t[0].toArray();for(var r=0,i=Array(e),o=-1;++o<e;)r+=(i[o]=t[o].toArray()).length;for(n!==i[0].constructor&&(n=i[0].constructor),t=new n(r),n=n===Array?Io:po,r=-1,o=0;++r<e;)o=n(i[r],t,o);return t},r.getInternal=function(t,e,n){return t.I[e].get(n)},r.isValidInternal=function(t,e,n){return t.I[e].isValid(n)},r.indexOfInternal=function(t,e,n,r){--e;var i=(t=t.I).length,o=n;for(n=0;++e<i;){if(~(o=t[e].indexOf(r,o)))return n+o;o=0,n+=t[e].length}return-1},r.Gb=function(t,e,n){for(var r=[],i=t.chunks,o=t.xb,a=-1,s=i.length;++a<s;){var A=i[a],l=A.length,c=o[a];if(c>=n)break;e>=c+l||(c>=e&&c+l<=n?r.push(A):r.push(A.slice(Math.max(0,e-c),Math.min(n-c,l))))}return t.clone(r)},o.global.Object.defineProperties(fo.prototype,{type:{configurable:!0,enumerable:!0,get:function(){return this.$}},length:{configurable:!0,enumerable:!0,get:function(){return this.Fa}},chunks:{configurable:!0,enumerable:!0,get:function(){return this.I}},typeId:{configurable:!0,enumerable:!0,get:function(){return this.$.typeId}},data:{configurable:!0,enumerable:!0,get:function(){return this.I[0]?this.I[0].data:null}},ArrayType:{configurable:!0,enumerable:!0,get:function(){return this.$.ArrayType}},numChildren:{configurable:!0,enumerable:!0,get:function(){return this.Cb}},stride:{configurable:!0,enumerable:!0,get:function(){return this.I[0]?this.I[0].stride:1}},nullCount:{configurable:!0,enumerable:!0,get:function(){var t=this.oa;return 0>t&&(this.oa=t=this.I.reduce(function(t,e){return t+e.nullCount},0)),t}},indices:{configurable:!0,enumerable:!0,get:function(){if(nr.isDictionary(this.$)){if(!this.jc){var t=this.I;this.jc=1===t.length?t[0].indices:fo.concat.apply(fo,o.o(t.map(function(t){return t.indices})))}return this.jc}return null}},dictionary:{configurable:!0,enumerable:!0,get:function(){return nr.isDictionary(this.$)?this.$.dictionaryVector:null}}}),o.h(Co,fo),Co.new=function(t,e,n){for(var r=[],i=2;i<arguments.length;++i)r[i-2]=arguments[i];return r=fo.flatten(Array.isArray(e)?[].concat(o.o(e),o.o(r)):e instanceof ce?[e].concat(o.o(r)):[ce.new.apply(ce,[e].concat(o.o(r)))]),"string"==typeof t?t=new Mo(t,r[0].data.type,r.some(function(t){return 0<t.nullCount})):!t.nullable&&r.some(function(t){return 0<t.nullCount})&&(t=t.clone({nullable:!0})),new Co(t,r)},Co.prototype.clone=function(t){return t=void 0===t?this.I:t,new Co(this.Da,t)},Co.prototype.getChildAt=function(t){if(0>t||t>=this.numChildren)return null;var e,n=this.Y||(this.Y=[]);if(e=n[t])return e;if(e=(this.type.children||[])[t]){var r=this.I.map(function(e){return e.getChildAt(t)}).filter(function(t){return null!=t});if(0<r.length)return n[t]=new Co(e,r)}return null},o.global.Object.defineProperties(Co.prototype,{field:{configurable:!0,enumerable:!0,get:function(){return this.Da}},name:{configurable:!0,enumerable:!0,get:function(){return this.Da.name}},nullable:{configurable:!0,enumerable:!0,get:function(){return this.Da.nullable}},metadata:{configurable:!0,enumerable:!0,get:function(){return this.Da.metadata}}}),o.h(mo,Co),(r=mo.prototype).search=function(t,e){return e?e(this,0,t):[0,t]},r.isValid=function(t){return this.Za.isValid(t)},r.get=function(t){return this.Za.get(t)},r.set=function(t,e){this.Za.set(t,e)},r.indexOf=function(t,e){return this.Za.indexOf(t,e)};var yo=Array.isArray;function Eo(t){var e=(t=o.w(xo(t,[[],[]]))).next().value;return t.next().value.map(function(t,n){return t instanceof Co?Co.new(t.field.clone(e[n]),t):t instanceof ce?Co.new(e[n],t):Co.new(e[n],[])})}function vo(t,e,n,r){for(var i,o=-1,a=e.length;++o<a;)yo(i=e[o])?r=vo(t,i,n,r).length:i instanceof t&&(n[r++]=i);return n}function Bo(t,e,n,r){for(var i,o=-1,a=e.length;++o<a;)yo(i=e[o])?r=Bo(t,i,n,r).length:i instanceof fo?r=Bo(t,i.chunks,n,r).length:i instanceof t&&(n[r++]=i);return n}function bo(t,e,n,r){for(var i,o=-1,a=e.length;++o<a;)yo(i=e[o])?r=bo(t,i,n,r).length:i instanceof t?r=vo(ce,i.schema.fields.map(function(t,e){return i.getChildAt(e)}),n,r).length:i instanceof ce&&(n[r++]=i);return n}function Qo(t,e,n,r){for(var i,o=-1,a=e.length;++o<a;)yo(i=e[o])?r=Qo(t,i,n,r).length:i instanceof t?r=vo(Co,i.schema.fields.map(function(t,e){return Co.new(t,i.getChildAt(e))}),n,r).length:i instanceof Co&&(n[r++]=i);return n}function wo(t,e,n){var r=o.w(e);return e=r.next().value,r=r.next().value,t[0][n]=e,t[1][n]=r,t}function xo(t,e){var n;switch(n=t.length){case 0:return e;case 1:var r=e[0];if(!t[0])return e;if(yo(t[0]))return xo(t[0],e);t[0]instanceof Tn||t[0]instanceof ce||t[0]instanceof nr||(r=(t=o.w(Object.entries(t[0]).reduce(wo,e))).next().value,t=t.next().value);break;default:yo(r=t[n-1])?t=yo(t[0])?t[0]:t.slice(0,n-1):(t=yo(t[0])?t[0]:t,r=[])}for(var i=n=-1,a=-1,s=t.length,A=o.w(e),l=A.next().value,c=A.next().value;++a<s;)if((A=t[a])instanceof Co&&(c[++i]=A))l[++n]=A.field.clone(r[a],A.type,0<A.nullCount);else{var u=r[a];u=void 0===u?a:u,A instanceof nr&&(c[++i]=A)?l[++n]=Mo.new(u,A):A&&A.type&&(c[++i]=A)&&(A instanceof Tn&&(c[i]=A=ce.new(A)),l[++n]=Mo.new(u,A.type,0<A.nullCount))}return e}function _o(t,e,n,r){this.fields=(t=void 0===t?[]:t)||[],this.metadata=e||new Map,n&&r||(n=(t=Lo(t,n||new Map,r||new Map)).dictionaries,r=t.dictionaryFields),this.dictionaries=n,this.dictionaryFields=r}o.l(),_o.from=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return _o.new(e[0],e[1])},_o.new=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return new _o(xo(e,[[],[]])[0])},(r=_o.prototype).toString=function(){return"Schema<{ "+this.fields.map(function(t,e){return e+": "+t}).join(", ")+" }>"},r.compareTo=function(t){return this===t||t instanceof this.constructor&&Pn(this.fields,t.fields)},r.select=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];var r=e.reduce(function(t,e){return t[e]=!0,t},Object.create(null));return new _o(this.fields.filter(function(t){return r[t.name]}),this.metadata)},r.selectAt=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];var r=this;return new _o(e.map(function(t){return r.fields[t]}).filter(Boolean),this.metadata)},r.assign=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];n=e[0]instanceof _o?e[0]:new _o(vo(Mo,e,[],0));var r=[].concat(o.o(this.fields)),i=[].concat(o.o(this.dictionaries)),a=this.dictionaryFields;e=So(So(new Map,this.metadata),n.metadata);var s=n.fields.filter(function(t){var e=r.findIndex(function(e){return e.name===t.name});return!~e||(r[e]=t.clone({metadata:So(So(new Map,r[e].metadata),t.metadata)}))&&!1}),A=Lo(s,new Map,new Map);return n=A.dictionaryFields,A=[].concat(o.o(A.dictionaries)).filter(function(t){var e=o.w(t).next().value;return!i.every(function(t){return o.w(t).next().value===e})}),n=[].concat(o.o(n)).map(function(t){var e=(t=o.w(t)).next().value;return t=t.next().value,[e,[].concat(o.o(a.get(e)||[]),o.o(t.map(function(t){var n=s.findIndex(function(e){return t.name===e.name}),r=t.type;return r=new zi(r.dictionary,r.indices,e,r.isOrdered,r.dictionaryVector),s[n]=t.clone({type:r})})))]}),new _o([].concat(o.o(r),o.o(s)),e,new Map([].concat(o.o(i),o.o(A))),new Map([].concat(o.o(a),o.o(n))))};var Do={};function Mo(t,e,n,r){this.name=t,this.type=e,this.nullable=void 0!==n&&n,this.metadata=r||new Map}o.global.Object.defineProperties(_o.prototype,(Do[Symbol.toStringTag]={configurable:!0,enumerable:!0,get:function(){return"Schema"}},Do)),o.l(),Mo.new=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];var r=o.w(e);n=r.next().value;var i=r.next().value,a=r.next().value;return r=r.next().value,e[0]&&"object"==typeof e[0]&&(n=e[0].name,void 0===i&&(i=e[0].type),void 0===a&&(a=e[0].nullable),void 0===r&&(r=e[0].metadata)),new Mo(""+n,i,a,r)},Mo.prototype.toString=function(){return this.name+": "+this.type},Mo.prototype.compareTo=function(t){return Kn(this,t)},Mo.prototype.clone=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];var r=this,i=(n=o.w(e)).next().value,a=n.next().value,s=n.next().value,A=n.next().value;return e[0]&&"object"==typeof e[0]?function(){var t=e[0];i=void 0===t.name?r.name:t.name,a=void 0===t.type?r.type:t.type,s=void 0===t.nullable?r.nullable:t.nullable,A=void 0===t.metadata?r.metadata:t.metadata}():function(){var t=o.w(e),n=t.next().value;i=void 0===n?r.name:n,n=t.next().value,a=void 0===n?r.type:n,n=t.next().value,s=void 0===n?r.nullable:n,t=t.next().value,A=void 0===t?r.metadata:t}(),Mo.new(i,a,s,A)};var ko={};function So(t,e){return new Map([].concat(o.o(t||new Map),o.o(e||new Map)))}function Lo(t,e,n){for(var r=-1,i=t.length;++r<i;){var o=t[r],a=o.type;if(nr.isDictionary(a))if(n.get(a.id)||n.set(a.id,[]),e.has(a.id)){if(e.get(a.id)!==a.dictionary)throw Error("Cannot create Schema containing two different dictionaries with the same Id")}else e.set(a.id,a.dictionary),n.get(a.id).push(o);a.children&&Lo(a.children,e,n)}return{dictionaries:e,dictionaryFields:n}}o.global.Object.defineProperties(Mo.prototype,(ko.typeId={configurable:!0,enumerable:!0,get:function(){return this.type.typeId}},ko[Symbol.toStringTag]={configurable:!0,enumerable:!0,get:function(){return"Field"}},ko)),_o.prototype.fields=null,_o.prototype.metadata=null,_o.prototype.dictionaries=null,_o.prototype.dictionaryFields=null,Mo.prototype.type=null,Mo.prototype.name=null,Mo.prototype.nullable=null,Mo.prototype.metadata=null;var Ro=_n.c.f.g.qb,No=_n.c.f.g.Qc;function Fo(t,e,n,r){e=void 0===e?bn.V4:e,this.schema=t,this.version=e,n&&(this.lc=n),r&&(this.gc=r)}Fo.decode=function(t){return t=new Z(Wt.toUint8Array(t)),t=No.ze(t),new Uo(_o.decode(t.schema()),t)},Fo.encode=function(t){var e=new N,n=_o.encode(e,t.schema);No.lf(e,t.numRecordBatches),[].concat(o.o(t.Ub())).slice().reverse().forEach(function(t){return Oo.encode(e,t)});var r=W(e);return No.Ue(e,t.numDictionaries),[].concat(o.o(To(t))).slice().reverse().forEach(function(t){return Oo.encode(e,t)}),t=W(e),No.bf(e),No.Ad(e,n),No.Mb(e,bn.V4),No.yd(e,r),No.gd(e,t),No.we(e,No.de(e)),F(e)},Fo.prototype.Ub=function t(){var e,n,r,i=this;return o.C.H(t,function(t){return 1==t.i&&(e=void 0,n=-1,r=i.numRecordBatches),++n<r?(e=i.wa(n))?l(t,e,2):t.j(2):t.j(0)})};var To=function t(e){var n,r,i;return o.C.H(t,function(t){return 1==t.i&&(n=void 0,r=-1,i=e.numDictionaries),++r<i?(n=e.hb(r))?l(t,n,2):t.j(2):t.j(0)})};function Uo(t,e){Fo.call(this,t,e.version()),this.A=e}function Oo(t,e,n){this.Rb=t,this.offset="number"==typeof n?n:n.low,this.bodyLength="number"==typeof e?e:e.low}function Go(t,e){this.Y=e,this.numChildren=t.childData.length,this.fc(this.data=t)}Fo.prototype.wa=function(t){return 0<=t&&t<this.numRecordBatches&&this.lc[t]||null},Fo.prototype.hb=function(t){return 0<=t&&t<this.numDictionaries&&this.gc[t]||null},o.global.Object.defineProperties(Fo.prototype,{numRecordBatches:{configurable:!0,enumerable:!0,get:function(){return this.lc.length}},numDictionaries:{configurable:!0,enumerable:!0,get:function(){return this.gc.length}}}),o.h(Uo,Fo),Uo.prototype.wa=function(t){return 0<=t&&t<this.numRecordBatches&&(t=this.A.Ub(t))?Oo.decode(t):null},Uo.prototype.hb=function(t){return 0<=t&&t<this.numDictionaries&&(t=this.A.dictionaries(t))?Oo.decode(t):null},o.global.Object.defineProperties(Uo.prototype,{numRecordBatches:{configurable:!0,enumerable:!0,get:function(){return this.A.Ie()}},numDictionaries:{configurable:!0,enumerable:!0,get:function(){return this.A.Td()}}}),Oo.decode=function(t){return new Oo(t.Rb(),t.bodyLength(),t.offset())},Oo.encode=function(t,e){return Ro.Od(t,new L(e.offset,0),e.Rb,new L(e.bodyLength,0))},o.l(),o.h(Go,ce),(r=Go.prototype).clone=function(t,e){return e=void 0===e?this.Y:e,ce.new(t,e)},r.concat=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return fo.concat.apply(fo,[this].concat(o.o(e)))},r.slice=function(t,e){return ho.clampRange(this,t,e,this.Gb)},r.isValid=function(t){return!(0<this.nullCount)||(t=this.offset+t,0!=(this.nullBitmap[t>>3]&1<<t%8))},r.getChildAt=function(t){return 0>t||t>=this.numChildren?null:(this.Y||(this.Y=[]))[t]||(this.Y[t]=ce.new(this.data.childData[t]))},r.toJSON=function(){return[].concat(o.o(this))},r.Gb=function(t,e,n){return t.clone(t.data.slice(e,n-e))},r.fc=function(){};var Ho={};function jo(t){Go.apply(this,arguments)}function Yo(t){Go.apply(this,arguments)}function Po(t){return 0>t&&(t=4294967295+t+1),"0x"+t.toString(16)}o.global.Object.defineProperties(Go.prototype,(Ho.type={configurable:!0,enumerable:!0,get:function(){return this.data.type}},Ho.typeId={configurable:!0,enumerable:!0,get:function(){return this.data.typeId}},Ho.length={configurable:!0,enumerable:!0,get:function(){return this.data.length}},Ho.offset={configurable:!0,enumerable:!0,get:function(){return this.data.offset}},Ho.stride={configurable:!0,enumerable:!0,get:function(){return this.data.stride}},Ho.nullCount={configurable:!0,enumerable:!0,get:function(){return this.data.nullCount}},Ho.VectorName={configurable:!0,enumerable:!0,get:function(){return this.constructor.name}},Ho.ArrayType={configurable:!0,enumerable:!0,get:function(){return this.data.ArrayType}},Ho.values={configurable:!0,enumerable:!0,get:function(){return this.data.values}},Ho.typeIds={configurable:!0,enumerable:!0,get:function(){return this.data.typeIds}},Ho.nullBitmap={configurable:!0,enumerable:!0,get:function(){return this.data.nullBitmap}},Ho.valueOffsets={configurable:!0,enumerable:!0,get:function(){return this.data.valueOffsets}},Ho[Symbol.toStringTag]={configurable:!0,enumerable:!0,get:function(){return o.l(),this.VectorName+"<"+this.type[Symbol.toStringTag]+">"}},Ho)),o.l(),Go.prototype[Symbol.hg]=!0,o.h(jo,Go),jo.prototype.asUtf8=function(){return ce.new(this.data.clone(new Lr))},o.h(Yo,Go),Yo.from=function(t){var e=0,n=Fn.packBools(function n(){var r,i,a;return o.C.H(n,function(n){return 1==n.i&&(r=o.w(t),i=r.next()),3!=n.i?i.done?n.j(0):(a=i.value,e++,l(n,a,3)):(i=r.next(),n.j(2))})}());return ce.new(Tn.Bool(new Tr,0,e,0,null,n))};var Ko=[1,10,100,1e3,1e4,1e5,1e6,1e7,1e8];function zo(t){this.buffer=t}function Jo(t,e){var n=new Uint32Array([t.buffer[1]>>>16,65535&t.buffer[1],t.buffer[0]>>>16,65535&t.buffer[0]]);e=new Uint32Array([e.buffer[1]>>>16,65535&e.buffer[1],e.buffer[0]>>>16,65535&e.buffer[0]]);var r=n[3]*e[3];t.buffer[0]=65535&r;var i=r>>>16;i+=r=n[2]*e[3],i+=r=n[3]*e[2]>>>0,t.buffer[0]+=i<<16,t.buffer[1]=i>>>0<r?65536:0,t.buffer[1]+=i>>>16,t.buffer[1]+=n[1]*e[3]+n[2]*e[2]+n[3]*e[1],t.buffer[1]+=n[0]*e[3]+n[1]*e[2]+n[2]*e[1]+n[3]*e[0]<<16}function qo(t,e){var n=t.buffer[0]+e.buffer[0]>>>0;t.buffer[1]+=e.buffer[1],n<t.buffer[0]>>>0&&++t.buffer[1],t.buffer[0]=n}function Vo(t){zo.apply(this,arguments)}function Wo(t){zo.apply(this,arguments)}function Xo(t){this.buffer=t}(r=zo.prototype).high=function(){return this.buffer[1]},r.low=function(){return this.buffer[0]},r.lessThan=function(t){return this.buffer[1]<t.buffer[1]||this.buffer[1]===t.buffer[1]&&this.buffer[0]<t.buffer[0]},r.equals=function(t){return this.buffer[1]===t.buffer[1]&&this.buffer[0]==t.buffer[0]},r.greaterThan=function(t){return t.lessThan(this)},r.hex=function(){return Po(this.buffer[1])+" "+Po(this.buffer[0])},o.h(Vo,zo),Vo.prototype.times=function(t){return Jo(this,t),this},Vo.prototype.plus=function(t){return qo(this,t),this},Vo.from=function(t,e){return e=void 0===e?new Uint32Array(2):e,Vo.fromString("string"==typeof t?t:t.toString(),e)},Vo.fromNumber=function(t,e){return e=void 0===e?new Uint32Array(2):e,Vo.fromString(t.toString(),e)},Vo.fromString=function(t,e){e=void 0===e?new Uint32Array(2):e;var n=t.length;e=new Vo(e);for(var r=0;r<n;){var i=8<n-r?8:n-r,o=new Vo(new Uint32Array([parseInt(t.substr(r,i),10),0]));e.times(new Vo(new Uint32Array([Ko[i],0]))),e.plus(o),r+=i}return e},Vo.convertArray=function(t){for(var e=new Uint32Array(2*t.length),n=-1,r=t.length;++n<r;)Vo.from(t[n],new Uint32Array(e.buffer,e.byteOffset+8*n,2));return e},Vo.multiply=function(t,e){return new Vo(new Uint32Array(t.buffer)).times(e)},Vo.add=function(t,e){return new Vo(new Uint32Array(t.buffer)).plus(e)},o.h(Wo,zo),Wo.prototype.negate=function(){return this.buffer[0]=1+~this.buffer[0],this.buffer[1]=~this.buffer[1],0==this.buffer[0]&&++this.buffer[1],this},Wo.prototype.times=function(t){return Jo(this,t),this},Wo.prototype.plus=function(t){return qo(this,t),this},Wo.prototype.lessThan=function(t){var e=this.buffer[1]<<0,n=t.buffer[1]<<0;return e<n||e===n&&this.buffer[0]<t.buffer[0]},Wo.from=function(t,e){return e=void 0===e?new Uint32Array(2):e,Wo.fromString("string"==typeof t?t:t.toString(),e)},Wo.fromNumber=function(t,e){return e=void 0===e?new Uint32Array(2):e,Wo.fromString(t.toString(),e)},Wo.fromString=function(t,e){e=void 0===e?new Uint32Array(2):e;var n=t.startsWith("-"),r=t.length;e=new Wo(e);for(var i=n?1:0;i<r;){var o=8<r-i?8:r-i,a=new Wo(new Uint32Array([parseInt(t.substr(i,o),10),0]));e.times(new Wo(new Uint32Array([Ko[o],0]))),e.plus(a),i+=o}return n?e.negate():e},Wo.convertArray=function(t){for(var e=new Uint32Array(2*t.length),n=-1,r=t.length;++n<r;)Wo.from(t[n],new Uint32Array(e.buffer,e.byteOffset+8*n,2));return e},Wo.multiply=function(t,e){return new Wo(new Uint32Array(t.buffer)).times(e)},Wo.add=function(t,e){return new Wo(new Uint32Array(t.buffer)).plus(e)},(r=Xo.prototype).high=function(){return new Wo(new Uint32Array(this.buffer.buffer,this.buffer.byteOffset+8,2))},r.low=function(){return new Wo(new Uint32Array(this.buffer.buffer,this.buffer.byteOffset,2))},r.negate=function(){return this.buffer[0]=1+~this.buffer[0],this.buffer[1]=~this.buffer[1],this.buffer[2]=~this.buffer[2],this.buffer[3]=~this.buffer[3],0==this.buffer[0]&&++this.buffer[1],0==this.buffer[1]&&++this.buffer[2],0==this.buffer[2]&&++this.buffer[3],this},r.times=function(t){var e=new Vo(new Uint32Array([this.buffer[3],0])),n=new Vo(new Uint32Array([this.buffer[2],0])),r=new Vo(new Uint32Array([this.buffer[1],0])),i=new Vo(new Uint32Array([this.buffer[0],0])),o=new Vo(new Uint32Array([t.buffer[3],0])),a=new Vo(new Uint32Array([t.buffer[2],0])),s=new Vo(new Uint32Array([t.buffer[1],0]));t=new Vo(new Uint32Array([t.buffer[0],0]));var A=Vo.multiply(i,t);this.buffer[0]=A.low();var l=new Vo(new Uint32Array([A.high(),0]));return A=Vo.multiply(r,t),l.plus(A),A=Vo.multiply(i,s),l.plus(A),this.buffer[1]=l.low(),this.buffer[3]=l.lessThan(A)?1:0,this.buffer[2]=l.high(),new Vo(new Uint32Array(this.buffer.buffer,this.buffer.byteOffset+8,2)).plus(Vo.multiply(n,t)).plus(Vo.multiply(r,s)).plus(Vo.multiply(i,a)),this.buffer[3]+=Vo.multiply(e,t).plus(Vo.multiply(n,s)).plus(Vo.multiply(r,a)).plus(Vo.multiply(i,o)).low(),this},r.plus=function(t){var e=new Uint32Array(4);return e[3]=this.buffer[3]+t.buffer[3]>>>0,e[2]=this.buffer[2]+t.buffer[2]>>>0,e[1]=this.buffer[1]+t.buffer[1]>>>0,e[0]=this.buffer[0]+t.buffer[0]>>>0,e[0]<this.buffer[0]>>>0&&++e[1],e[1]<this.buffer[1]>>>0&&++e[2],e[2]<this.buffer[2]>>>0&&++e[3],this.buffer[3]=e[3],this.buffer[2]=e[2],this.buffer[1]=e[1],this.buffer[0]=e[0],this},r.hex=function(){return Po(this.buffer[3])+" "+Po(this.buffer[2])+" "+Po(this.buffer[1])+" "+Po(this.buffer[0])},Xo.multiply=function(t,e){return new Xo(new Uint32Array(t.buffer)).times(e)},Xo.add=function(t,e){return new Xo(new Uint32Array(t.buffer)).plus(e)},Xo.from=function(t,e){return e=void 0===e?new Uint32Array(4):e,Xo.fromString("string"==typeof t?t:t.toString(),e)},Xo.fromNumber=function(t,e){return e=void 0===e?new Uint32Array(4):e,Xo.fromString(t.toString(),e)},Xo.fromString=function(t,e){e=void 0===e?new Uint32Array(4):e;var n=t.startsWith("-"),r=t.length;e=new Xo(e);for(var i=n?1:0;i<r;){var o=8<r-i?8:r-i,a=new Xo(new Uint32Array([parseInt(t.substr(i,o),10),0,0,0]));e.times(new Xo(new Uint32Array([Ko[o],0,0,0]))),e.plus(a),i+=o}return n?e.negate():e},Xo.convertArray=function(t){for(var e=new Uint32Array(4*t.length),n=-1,r=t.length;++n<r;)Xo.from(t[n],new Uint32Array(e.buffer,e.byteOffset+16*n,4));return e};var Zo={};function $o(t){Go.apply(this,arguments)}function ta(t){$o.apply(this,arguments)}function ea(t){$o.apply(this,arguments)}function na(t){Go.apply(this,arguments)}function ra(t){Go.call(this,t),this.indices=ce.new(t.clone(this.type.indices))}function ia(t){Go.apply(this,arguments)}function oa(t){Go.apply(this,arguments)}function aa(t){Go.apply(this,arguments)}function sa(t){aa.apply(this,arguments)}function Aa(t){aa.apply(this,arguments)}function la(t){aa.apply(this,arguments)}function ca(t){Go.apply(this,arguments)}function ua(t){ca.apply(this,arguments)}function ga(t){ca.apply(this,arguments)}function ha(t){Go.apply(this,arguments)}function da(t){ha.apply(this,arguments)}function fa(t){ha.apply(this,arguments)}function pa(t){ha.apply(this,arguments)}function Ia(t){ha.apply(this,arguments)}function Ca(t){ha.apply(this,arguments)}function ma(t){ha.apply(this,arguments)}function ya(t){ha.apply(this,arguments)}function Ea(t){ha.apply(this,arguments)}function va(t){Go.apply(this,arguments)}function Ba(t){Go.apply(this,arguments)}function ba(t){Go.apply(this,arguments)}function Qa(t){Go.apply(this,arguments)}function wa(t){Go.apply(this,arguments)}function xa(t){wa.apply(this,arguments)}function _a(t){wa.apply(this,arguments)}function Da(t){wa.apply(this,arguments)}function Ma(t){wa.apply(this,arguments)}function ka(t){Go.apply(this,arguments)}function Sa(t){ka.apply(this,arguments)}function La(t){ka.apply(this,arguments)}function Ra(t){ka.apply(this,arguments)}function Na(t){ka.apply(this,arguments)}function Fa(t){Go.apply(this,arguments)}function Ta(t){Fa.apply(this,arguments)}function Ua(t){Fa.apply(this,arguments)}function Oa(t){Go.apply(this,arguments)}Zo.BaseInt64=zo,Zo.Uint64=Vo,Zo.Int64=Wo,Zo.Int128=Xo,o.h($o,Go),$o.from=function(t,e){switch(e=void 0===e?In.MILLISECOND:e){case In.DAY:return e=Int32Array.from(t.map(function(t){return t.valueOf()/864e5})),ce.new(Tn.Date(new Vr,0,t.length,0,null,e));case In.MILLISECOND:return e=Zo.Int64.convertArray(t.map(function(t){return t.valueOf()})),ce.new(Tn.Date(new Wr,0,t.length,0,null,e))}throw new TypeError('Unrecognized date unit "'+In[e]+'"')},o.h(ta,$o),o.h(ea,$o),o.h(na,Go),o.h(ra,Go),ra.from=function(t,e,n){return t=new zi(t.type,e,null,null,t),ce.new(Tn.Dictionary(t,0,n.length,0,null,n))},(r=ra.prototype).reverseLookup=function(t){return this.dictionary.indexOf(t)},r.getKey=function(t){return this.indices.get(t)},r.getValue=function(t){return this.dictionary.get(t)},r.setKey=function(t,e){return this.indices.set(t,e)},r.setValue=function(t,e){return this.dictionary.set(t,e)},o.global.Object.defineProperties(ra.prototype,{dictionary:{configurable:!0,enumerable:!0,get:function(){return this.data.type.dictionaryVector}}}),ra.prototype.indices=null,o.h(ia,Go),o.h(oa,Go),o.h(aa,Go),aa.from=function(t){var e=null;switch(this){case sa:t=Wt.toFloat16Array(t);break;case Aa:t=Wt.toFloat32Array(t);break;case la:t=Wt.toFloat64Array(t)}switch(t.constructor){case Uint16Array:e=new wr;break;case Float32Array:e=new xr;break;case Float64Array:e=new _r}if(null===e)throw new TypeError("Unrecognized FloatVector input");return t=ce.new(Tn.Float(e,0,t.length,0,null,t))},o.h(sa,aa),sa.prototype.toFloat32Array=function(){return new Float32Array(this)},sa.prototype.toFloat64Array=function(){return new Float64Array(this)},o.h(Aa,aa),o.h(la,aa),o.h(ca,Go),o.h(ua,ca),o.h(ga,ca),o.h(ha,Go),ha.from=function(t,e){var n=null;switch(this){case da:t=Wt.toInt8Array(t),e=!1;break;case fa:t=Wt.toInt16Array(t),e=!1;break;case pa:t=Wt.toInt32Array(t),e=!1;break;case Ia:t=Wt.toInt32Array(t),e=!0;break;case Ca:t=Wt.toUint8Array(t),e=!1;break;case ma:t=Wt.toUint16Array(t),e=!1;break;case ya:t=Wt.toUint32Array(t),e=!1;break;case Ea:t=Wt.toUint32Array(t),e=!0}if(!0===e)e=.5*t.length,n=t instanceof Int32Array?new Ir:new Er;else switch(e=t.length,t.constructor){case Int8Array:n=new dr;break;case Int16Array:n=new fr;break;case Int32Array:n=new pr;break;case Uint8Array:n=new Cr;break;case Uint16Array:n=new mr;break;case Uint32Array:n=new yr}if(null===n)throw new TypeError("Unrecognized IntVector input");return t=ce.new(Tn.Int(n,0,e,0,null,t))},o.h(da,ha),o.h(fa,ha),o.h(pa,ha),o.h(Ia,ha),Ia.prototype.toBigInt64Array=function(){return Wt.toBigInt64Array(this.values)},o.h(Ca,ha),o.h(ma,ha),o.h(ya,ha),o.h(Ea,ha),Ea.prototype.toBigUint64Array=function(){return Wt.toBigUint64Array(this.values)},o.h(va,Go),o.h(Ba,Go),Ba.prototype.asStruct=function(){return ce.new(this.data.clone(new Bi(this.type.children)))},o.global.Object.defineProperties(Ba.prototype,{rowProxy:{configurable:!0,enumerable:!0,get:function(){return this.Fb||(this.Fb=io.new(this,this.type.children||[],!0))}}}),o.h(ba,Go),o.h(Qa,Go),Qa.prototype.asMap=function(t){return ce.new(this.data.clone(new Hi(this.type.children,void 0!==t&&t)))},o.global.Object.defineProperties(Qa.prototype,{rowProxy:{configurable:!0,enumerable:!0,get:function(){return this.Fb||(this.Fb=io.new(this,this.type.children||[],!1))}}}),o.h(wa,Go),o.h(xa,wa),o.h(_a,wa),o.h(Da,wa),o.h(Ma,wa),o.h(ka,Go),o.h(Sa,ka),o.h(La,ka),o.h(Ra,ka),o.h(Na,ka),o.h(Fa,Go),o.global.Object.defineProperties(Fa.prototype,{typeIdToChildIndex:{configurable:!0,enumerable:!0,get:function(){return this.type.typeIdToChildIndex}}}),o.h(Ta,Fa),o.global.Object.defineProperties(Ta.prototype,{valueOffsets:{configurable:!0,enumerable:!0,get:function(){return this.data.valueOffsets}}}),o.h(Ua,Fa),o.h(Oa,Go),Oa.from=function(t){var e=t.length,n=mt(t.join(""));return t=t.reduce(function(t,e,n){return t[n+1]=t[n]+e.length,t},new Uint32Array(t.length+1)),ce.new(Tn.Utf8(new Lr,0,e,0,null,t,n))},Oa.prototype.asBinary=function(){return ce.new(this.data.clone(new Dr))},o.l();var Ga,Ha,ja={},Ya=(ja.toJSON=function(){return'"'+Ga(this)+'"'},ja.valueOf=function(){return Ja(this)},ja.toString=function(){return Ga(this)},ja[Symbol.toPrimitive]=function(t){switch(t){case"number":return Ja(this);case"default":return Ha(this)}return Ga(this)},ja),Pa=Object.assign({},Ya,{Xb:!0,pb:Mt}),Ka=Object.assign({},Ya,{Xb:!1,pb:St});function za(t,e){return e=void 0===e?t instanceof Int32Array:e,za.new(t,e)}function Ja(t){for(var e=0,n=0,r=(t=new Uint32Array(t.buffer,t.byteOffset,t.length)).length;n<r;)e+=t[n++]+t[n++]*Math.pow(n,32);return e}function qa(t){var e="",n=new Uint32Array(2);t=new Uint16Array(t.buffer,t.byteOffset,2*t.length);var r,i=new Uint32Array((t=new Uint16Array(t).reverse()).buffer),o=t.length-1;do{for(n[0]=t[r=0];r<o;)t[r++]=n[1]=n[0]/10,n[0]=(n[0]-10*n[1]<<16)+t[r];t[r]=n[1]=n[0]/10,n[0]-=10*n[1],e=""+n[0]+e}while(i[0]||i[1]||i[2]||i[3]);return e||"0"}function Va(){}function Wa(t,e){return 4294967296*t[e+1]+(t[e]>>>0)}function Xa(t,e,n){var r=e[n];return e=e[n+1],null!=r&&null!=e?t.subarray(r,e):null}function Za(t,e){return new Date(864e5*t.values[e])}function $a(t,e){return new Date(Wa(t.values,2*e))}function ts(t,e){return t.values[t.stride*e]}function es(t,e){return(t.values[t.stride*e]-32767)/32767}function ns(t,e){var n=t.stride;return za.new(t.values.subarray(n*e,n*(e+1)),t.type.isSigned)}function rs(t,e){return 1e3*Wa(t.values,2*e)}function is(t,e){return Wa(t.values,2*e)}function os(t,e){return(t=t.values)[(e*=2)+1]/1e3*4294967296+(t[e]>>>0)/1e3}function as(t,e){return(t=t.values)[(e*=2)+1]/1e6*4294967296+(t[e]>>>0)/1e6}function ss(t,e){return t.values[t.stride*e]}function As(t,e){return t.values[t.stride*e]}function ls(t,e){return za.new(t.values.subarray(2*e,2*(e+1)),!0)}function cs(t,e){return za.new(t.values.subarray(2*e,2*(e+1)),!0)}function us(t,e){return t.rowProxy.bind(e)}function gs(t,e){var n=t.getChildAt(t.type.typeIdToChildIndex[t.typeIds[e]]);return n?n.get(t.valueOffsets[e]):null}function hs(t,e){return(t=t.getChildAt(t.type.typeIdToChildIndex[t.typeIds[e]]))?t.get(e):null}function ds(t,e){return t.values.subarray(2*e,2*(e+1))}function fs(t,e){return t=t.values[e],(e=new Int32Array(2))[0]=t/12|0,e[1]=t%12|0,e}za.new=function(t,e){return!0===(e=void 0===e?t instanceof Int8Array||t instanceof Int16Array||t instanceof Int32Array:e)?za.Xb(t):za.wf(t)},za.Xb=function(t){var e=ArrayBuffer.isView(t)?t.constructor:Int32Array;return e=new e((t=Wt.toArrayBufferView(e,t)).buffer,t.byteOffset,t.length),Object.assign(e,Pa)},za.wf=function(t){var e=ArrayBuffer.isView(t)?t.constructor:Uint32Array;return e=new e((t=Wt.toArrayBufferView(e,t)).buffer,t.byteOffset,t.length),Object.assign(e,Ka)},_t?(Ha=function(t){return 2===t.length?new t.pb(t.buffer,t.byteOffset,1)[0]:qa(t)},Ga=function(t){return 2===t.length?""+new t.pb(t.buffer,t.byteOffset,1)[0]:qa(t)}):Ha=Ga=qa,o.h(Va,Gn),(r=Va.prototype).visitNull=function(){return null},r.visitBool=function(t,e){return e=t.offset+e,0!=(t.values[e>>3]&1<<e%8)},r.visitInt=function(t,e){return 64>t.type.bitWidth?ts(t,e):ns(t,e)},r.visitInt8=ts,r.visitInt16=ts,r.visitInt32=ts,r.visitInt64=ns,r.visitUint8=ts,r.visitUint16=ts,r.visitUint32=ts,r.visitUint64=ns,r.visitFloat=function(t,e){return t.type.precision!==mn.HALF?ts(t,e):es(t,e)},r.visitFloat16=es,r.visitFloat32=ts,r.visitFloat64=ts,r.visitUtf8=function(t,e){return null!==(t=Xa(t.values,t.valueOffsets,e))?Ct(t):null},r.visitBinary=function(t,e){return Xa(t.values,t.valueOffsets,e)},r.visitFixedSizeBinary=function(t,e){var n=t.stride;return t.values.subarray(n*e,n*(e+1))},r.visitDate=function(t,e){return t.type.unit===In.DAY?Za(t,e):$a(t,e)},r.visitDateDay=Za,r.visitDateMillisecond=$a,r.visitTimestamp=function(t,e){switch(t.type.unit){case Cn.SECOND:return rs(t,e);case Cn.MILLISECOND:return is(t,e);case Cn.MICROSECOND:return os(t,e);case Cn.NANOSECOND:return as(t,e)}},r.visitTimestampSecond=rs,r.visitTimestampMillisecond=is,r.visitTimestampMicrosecond=os,r.visitTimestampNanosecond=as,r.visitTime=function(t,e){switch(t.type.unit){case Cn.SECOND:return ss(t,e);case Cn.MILLISECOND:return As(t,e);case Cn.MICROSECOND:return ls(t,e);case Cn.NANOSECOND:return cs(t,e)}},r.visitTimeSecond=ss,r.visitTimeMillisecond=As,r.visitTimeMicrosecond=ls,r.visitTimeNanosecond=cs,r.visitDecimal=function(t,e){return za.new(t.values.subarray(4*e,4*(e+1)),!1)},r.visitList=function(t,e){var n=t.getChildAt(0),r=t.valueOffsets;return t=t.stride,n.slice(r[e*t],r[e*t+1])},r.visitStruct=us,r.visitUnion=function(t,e){return t.type.mode===yn.Dense?gs(t,e):hs(t,e)},r.visitDenseUnion=gs,r.visitSparseUnion=hs,r.visitDictionary=function(t,e){return t.getValue(t.getKey(e))},r.visitInterval=function(t,e){return t.type.unit===vn.DAY_TIME?ds(t,e):fs(t,e)},r.visitIntervalDayTime=ds,r.visitIntervalYearMonth=fs,r.visitFixedSizeList=function(t,e){var n=t.getChildAt(0);return t=t.stride,n.slice(e*t,(e+1)*t)},r.visitMap=us;var ps=new Va;function Is(){}function Cs(t,e,n,r){var i=e[n];e=e[n+1],null!=i&&null!=e&&t.set(r.subarray(0,e-i),i)}function ms(t,e,n){t.values[e]=n.valueOf()/864e5|0}function ys(t,e,n){t=t.values,e*=2,n=n.valueOf(),t[e]=n%4294967296|0,t[e+1]=n/4294967296|0}function Es(t,e,n){t.values[t.stride*e]=n}function vs(t,e,n){t.values[t.stride*e]=32767*n+32767}function Bs(t,e,n){var r=t.stride;t.values.set(n.subarray(0,r),r*e)}function bs(t,e,n){n/=1e3,(t=t.values)[e*=2]=n%4294967296|0,t[e+1]=n/4294967296|0}function Qs(t,e,n){(t=t.values)[e*=2]=n%4294967296|0,t[e+1]=n/4294967296|0}function ws(t,e,n){(t=t.values)[e*=2]=1e3*n%4294967296|0,t[e+1]=1e3*n/4294967296|0}function xs(t,e,n){(t=t.values)[e*=2]=1e6*n%4294967296|0,t[e+1]=1e6*n/4294967296|0}function _s(t,e,n){t.values[t.stride*e]=n}function Ds(t,e,n){t.values[t.stride*e]=n}function Ms(t,e,n){t.values.set(n.subarray(0,2),2*e)}function ks(t,e,n){t.values.set(n.subarray(0,2),2*e)}function Ss(t,e,n){t.type.children.forEach(function(r,i){r=r.name,(i=t.getChildAt(i))&&i.set(e,n[r])})}function Ls(t,e,n){var r=t.getChildAt(t.type.typeIdToChildIndex[t.typeIds[e]]);r&&r.set(t.valueOffsets[e],n)}function Rs(t,e,n){(t=t.getChildAt(t.type.typeIdToChildIndex[t.typeIds[e]]))&&t.set(e,n)}function Ns(t,e,n){t.values.set(n.subarray(0,2),2*e)}function Fs(t,e,n){t.values[e]=12*n[0]+n[1]%12}o.h(Is,Gn),(r=Is.prototype).visitBool=function(t,e,n){var r=t.values;t=t.offset+e,n?r[t>>3]|=1<<t%8:r[t>>3]&=~(1<<t%8)},r.visitInt=function(t,e,n){64>t.type.bitWidth?Es(t,e,n):Bs(t,e,n)},r.visitInt8=Es,r.visitInt16=Es,r.visitInt32=Es,r.visitInt64=Bs,r.visitUint8=Es,r.visitUint16=Es,r.visitUint32=Es,r.visitUint64=Bs,r.visitFloat=function(t,e,n){t.type.precision!==mn.HALF?Es(t,e,n):vs(t,e,n)},r.visitFloat16=vs,r.visitFloat32=Es,r.visitFloat64=Es,r.visitUtf8=function(t,e,n){Cs(t.values,t.valueOffsets,e,mt(n))},r.visitBinary=function(t,e,n){return Cs(t.values,t.valueOffsets,e,n)},r.visitFixedSizeBinary=function(t,e,n){var r=t.stride;t.values.set(n.subarray(0,r),r*e)},r.visitDate=function(t,e,n){t.type.unit===In.DAY?ms(t,e,n):ys(t,e,n)},r.visitDateDay=ms,r.visitDateMillisecond=ys,r.visitTimestamp=function(t,e,n){switch(t.type.unit){case Cn.SECOND:return bs(t,e,n);case Cn.MILLISECOND:return Qs(t,e,n);case Cn.MICROSECOND:return ws(t,e,n);case Cn.NANOSECOND:return xs(t,e,n)}},r.visitTimestampSecond=bs,r.visitTimestampMillisecond=Qs,r.visitTimestampMicrosecond=ws,r.visitTimestampNanosecond=xs,r.visitTime=function(t,e,n){switch(t.type.unit){case Cn.SECOND:return _s(t,e,n);case Cn.MILLISECOND:return Ds(t,e,n);case Cn.MICROSECOND:return Ms(t,e,n);case Cn.NANOSECOND:return ks(t,e,n)}},r.visitTimeSecond=_s,r.visitTimeMillisecond=Ds,r.visitTimeMicrosecond=Ms,r.visitTimeNanosecond=ks,r.visitDecimal=function(t,e,n){t.values.set(n.subarray(0,4),4*e)},r.visitList=function(t,e,n){var r=t.getChildAt(0),i=t.valueOffsets,o=t.stride;t=-1;var a=i[e*o];for(e=Math.min(n.length,i[e*o+1]-a);a<e;)r.set(a++,n.get(++t))},r.visitStruct=Ss,r.visitUnion=function(t,e,n){t.type.mode===yn.Dense?Ls(t,e,n):Rs(t,e,n)},r.visitDenseUnion=Ls,r.visitSparseUnion=Rs,r.visitDictionary=function(t,e,n){null!==(e=t.getKey(e))&&t.setValue(e,n)},r.visitInterval=function(t,e,n){t.type.unit===vn.DAY_TIME?Ns(t,e,n):Fs(t,e,n)},r.visitIntervalDayTime=Ns,r.visitIntervalYearMonth=Fs,r.visitFixedSizeList=function(t,e,n){var r=t.getChildAt(0),i=-1;for(e*=t=t.stride;++i<t;)r.set(e+i,n.get(i))},r.visitMap=Ss;var Ts=new Is;function Us(){}function Os(t,e,n){if(void 0===e)return-1;if(null===e){t:{var r=t.nullBitmap;if(r&&!(0>=t.nullCount))for(e=0,n=(t=o.w(Fn.iterateBits(r,t.data.offset+(n||0),t.length,r,Fn.getBool))).next();!n.done;n=t.next()){if(!n.value){t=e;break t}++e}t=-1}return t}for(e=ho.createElementComparator(e),n=(n||0)-1,r=t.length;++n<r;)if(e(t.get(n)))return n;return-1}function Gs(t,e,n){e=ho.createElementComparator(e),n=(n||0)-1;for(var r=t.length;++n<r;)if(e(t.get(n)))return n;return-1}o.h(Us,Gn),(r=Us.prototype).visitNull=function(t,e){return null===e&&0<t.length?0:-1},r.visitBool=Os,r.visitInt=Os,r.visitInt8=Os,r.visitInt16=Os,r.visitInt32=Os,r.visitInt64=Os,r.visitUint8=Os,r.visitUint16=Os,r.visitUint32=Os,r.visitUint64=Os,r.visitFloat=Os,r.visitFloat16=Os,r.visitFloat32=Os,r.visitFloat64=Os,r.visitUtf8=Os,r.visitBinary=Os,r.visitFixedSizeBinary=Os,r.visitDate=Os,r.visitDateDay=Os,r.visitDateMillisecond=Os,r.visitTimestamp=Os,r.visitTimestampSecond=Os,r.visitTimestampMillisecond=Os,r.visitTimestampMicrosecond=Os,r.visitTimestampNanosecond=Os,r.visitTime=Os,r.visitTimeSecond=Os,r.visitTimeMillisecond=Os,r.visitTimeMicrosecond=Os,r.visitTimeNanosecond=Os,r.visitDecimal=Os,r.visitList=Os,r.visitStruct=Os,r.visitUnion=Os,r.visitDenseUnion=Gs,r.visitSparseUnion=Gs,r.visitDictionary=Os,r.visitInterval=Os,r.visitIntervalDayTime=Os,r.visitIntervalYearMonth=Os,r.visitFixedSizeList=Os,r.visitMap=Os;var Hs=new Us;function js(){}function Ys(t){var e=ps.getVisitFn(t);return Fn.iterateBits(t.nullBitmap,t.offset,t.length,t,function(t,n,r,i){return 0!=(r&1<<i)?e(t,n):null})}function Ps(t){if(0<t.nullCount)return Ys(t);var e=t.type,n=t.typeId,r=t.length;return 1===t.stride&&(n===xn.Timestamp||n===xn.Int&&64!==e.bitWidth||n===xn.Time&&64!==e.bitWidth||n===xn.Float&&0<e.precision)?(o.l(),o.K(),t.values.subarray(0,r)[Symbol.iterator]()):function e(n){var i;return o.C.H(e,function(e){return 1==e.i&&(i=-1),++i<r?l(e,n(t,i),2):e.j(0)})}(ps.getVisitFn(t))}o.h(js,Gn),(r=js.prototype).visitNull=Ps,r.visitBool=Ps,r.visitInt=Ps,r.visitInt8=Ps,r.visitInt16=Ps,r.visitInt32=Ps,r.visitInt64=Ps,r.visitUint8=Ps,r.visitUint16=Ps,r.visitUint32=Ps,r.visitUint64=Ps,r.visitFloat=Ps,r.visitFloat16=Ps,r.visitFloat32=Ps,r.visitFloat64=Ps,r.visitUtf8=Ps,r.visitBinary=Ps,r.visitFixedSizeBinary=Ps,r.visitDate=Ps,r.visitDateDay=Ps,r.visitDateMillisecond=Ps,r.visitTimestamp=Ps,r.visitTimestampSecond=Ps,r.visitTimestampMillisecond=Ps,r.visitTimestampMicrosecond=Ps,r.visitTimestampNanosecond=Ps,r.visitTime=Ps,r.visitTimeSecond=Ps,r.visitTimeMillisecond=Ps,r.visitTimeMicrosecond=Ps,r.visitTimeNanosecond=Ps,r.visitDecimal=Ps,r.visitList=Ps,r.visitStruct=Ps,r.visitUnion=Ps,r.visitDenseUnion=Ps,r.visitSparseUnion=Ps,r.visitDictionary=Ps,r.visitInterval=Ps,r.visitIntervalDayTime=Ps,r.visitIntervalYearMonth=Ps,r.visitFixedSizeList=Ps,r.visitMap=Ps;var Ks=new js;function zs(){}function Js(t){var e=t.length,n=t.stride;switch(t.type.typeId){case xn.Int:case xn.Float:case xn.Decimal:case xn.Time:case xn.Timestamp:return t.values.subarray(0,e*n)}return[].concat(o.o(Ks.visit(t)))}o.h(zs,Gn),(r=zs.prototype).visitNull=Js,r.visitBool=Js,r.visitInt=Js,r.visitInt8=Js,r.visitInt16=Js,r.visitInt32=Js,r.visitInt64=Js,r.visitUint8=Js,r.visitUint16=Js,r.visitUint32=Js,r.visitUint64=Js,r.visitFloat=Js,r.visitFloat16=Js,r.visitFloat32=Js,r.visitFloat64=Js,r.visitUtf8=Js,r.visitBinary=Js,r.visitFixedSizeBinary=Js,r.visitDate=Js,r.visitDateDay=Js,r.visitDateMillisecond=Js,r.visitTimestamp=Js,r.visitTimestampSecond=Js,r.visitTimestampMillisecond=Js,r.visitTimestampMicrosecond=Js,r.visitTimestampNanosecond=Js,r.visitTime=Js,r.visitTimeSecond=Js,r.visitTimeMillisecond=Js,r.visitTimeMicrosecond=Js,r.visitTimeNanosecond=Js,r.visitDecimal=Js,r.visitList=Js,r.visitStruct=Js,r.visitUnion=Js,r.visitDenseUnion=Js,r.visitSparseUnion=Js,r.visitDictionary=Js,r.visitInterval=Js,r.visitIntervalDayTime=Js,r.visitIntervalYearMonth=Js,r.visitFixedSizeList=Js,r.visitMap=Js;var qs=new zs;function Vs(t,e){return t+e}function Ws(t){return"Cannot compute the byte width of variable-width column "+t}function Xs(){}function Zs(t,e){return(e||[]).map(function(e){return t.visit(e.type)})}o.h(Xs,Gn),(r=Xs.prototype).visitNull=function(){return 0},r.visitInt=function(t){return t.bitWidth/8},r.visitFloat=function(t){return t.ArrayType.BYTES_PER_ELEMENT},r.visitBinary=function(t){throw Error(Ws(t))},r.visitUtf8=function(t){throw Error(Ws(t))},r.visitBool=function(){return.125},r.visitDecimal=function(){return 16},r.visitDate=function(t){return 4*(t.unit+1)},r.visitTime=function(t){return t.bitWidth/8},r.visitTimestamp=function(t){return t.unit===Cn.SECOND?4:8},r.visitInterval=function(t){return 4*(t.unit+1)},r.visitList=function(t){throw Error(Ws(t))},r.visitStruct=function(t){return Zs(this,t.children).reduce(Vs,0)},r.visitUnion=function(t){return Zs(this,t.children).reduce(Vs,0)},r.visitFixedSizeBinary=function(t){return t.byteWidth},r.visitFixedSizeList=function(t){return t.listSize*Zs(this,t.children).reduce(Vs,0)},r.visitMap=function(t){return Zs(this,t.children).reduce(Vs,0)},r.visitDictionary=function(t){return this.visit(t.indices)};var $s=new Xs;function tA(){}o.h(tA,Gn),(r=tA.prototype).visitNull=function(){return ba},r.visitBool=function(){return Yo},r.visitInt=function(){return ha},r.visitInt8=function(){return da},r.visitInt16=function(){return fa},r.visitInt32=function(){return pa},r.visitInt64=function(){return Ia},r.visitUint8=function(){return Ca},r.visitUint16=function(){return ma},r.visitUint32=function(){return ya},r.visitUint64=function(){return Ea},r.visitFloat=function(){return aa},r.visitFloat16=function(){return sa},r.visitFloat32=function(){return Aa},r.visitFloat64=function(){return la},r.visitUtf8=function(){return Oa},r.visitBinary=function(){return jo},r.visitFixedSizeBinary=function(){return ia},r.visitDate=function(){return $o},r.visitDateDay=function(){return ta},r.visitDateMillisecond=function(){return ea},r.visitTimestamp=function(){return wa},r.visitTimestampSecond=function(){return xa},r.visitTimestampMillisecond=function(){return _a},r.visitTimestampMicrosecond=function(){return Da},r.visitTimestampNanosecond=function(){return Ma},r.visitTime=function(){return ka},r.visitTimeSecond=function(){return Sa},r.visitTimeMillisecond=function(){return La},r.visitTimeMicrosecond=function(){return Ra},r.visitTimeNanosecond=function(){return Na},r.visitDecimal=function(){return na},r.visitList=function(){return va},r.visitStruct=function(){return Qa},r.visitUnion=function(){return Fa},r.visitDenseUnion=function(){return Ta},r.visitSparseUnion=function(){return Ua},r.visitDictionary=function(){return ra},r.visitInterval=function(){return ca},r.visitIntervalDayTime=function(){return ua},r.visitIntervalYearMonth=function(){return ga},r.visitFixedSizeList=function(){return oa},r.visitMap=function(){return Ba};var eA=new tA;function nA(t,e){for(var n=[],r=1;r<arguments.length;++r)n[r-1]=arguments[r];return new(Function.prototype.bind.apply(eA.getVisitFn(t.type)(),[null,t].concat(o.o(n))))}function rA(t){return function(){return t(this)}}function iA(t){return function(){return t(this.type)}}function oA(t){return function(e){return t(this,e)}}function aA(t){return function(e,n){return t(this,e,n)}}function sA(t){return function(e){return this.isValid(e)?t.call(this,e):null}}function AA(t){return function(e,n){Fn.setBool(this.nullBitmap,this.offset+e,!(null==n))&&t.call(this,e,n)}}function lA(){var t=this.nullBitmap;t&&0<t.byteLength&&(this.get=sA(this.get),this.set=AA(this.set))}ce.new=nA,(r=Go.prototype).get=function(t){return ps.visit(this,t)},r.set=function(t,e){return Ts.visit(this,t,e)},r.indexOf=function(t,e){return Hs.visit(this,t,e)},r.toArray=function(){return qs.visit(this)},r.getByteWidth=function(){return $s.visit(this.type)},o.l(),o.K(),Go.prototype[Symbol.iterator]=function(){return Ks.visit(this)},Go.prototype.fc=lA,Object.keys(xn).filter(function(t){return t!==xn.NONE&&t!==xn[xn.NONE]}).map(function(t){return xn[t]}).filter(function(t){return"number"==typeof t}).forEach(function(t){switch(t){case xn.Int:t=[xn.Int8,xn.Int16,xn.Int32,xn.Int64,xn.Uint8,xn.Uint16,xn.Uint32,xn.Uint64];break;case xn.Float:t=[xn.Float16,xn.Float32,xn.Float64];break;case xn.Date:t=[xn.DateDay,xn.DateMillisecond];break;case xn.Time:t=[xn.TimeSecond,xn.TimeMillisecond,xn.TimeMicrosecond,xn.TimeNanosecond];break;case xn.Timestamp:t=[xn.TimestampSecond,xn.TimestampMillisecond,xn.TimestampMicrosecond,xn.TimestampNanosecond];break;case xn.Interval:t=[xn.IntervalDayTime,xn.IntervalYearMonth];break;case xn.Union:t=[xn.DenseUnion,xn.SparseUnion];break;default:t=[t]}t.forEach(function(t){var e=eA.visit(t);e.prototype.get=oA(ps.getVisitFn(t)),e.prototype.set=aA(Ts.getVisitFn(t)),e.prototype.indexOf=aA(Hs.getVisitFn(t)),e.prototype.toArray=rA(qs.getVisitFn(t)),e.prototype.getByteWidth=iA($s.getVisitFn(t)),o.l(),o.K(),e.prototype[Symbol.iterator]=rA(Ks.getVisitFn(t))})});var cA={get cc(){return ro},get Vector(){return ce},get BaseVector(){return Go},get BinaryVector(){return jo},get BoolVector(){return Yo},get Chunked(){return fo},get DateVector(){return $o},get DateDayVector(){return ta},get DateMillisecondVector(){return ea},get DecimalVector(){return na},get DictionaryVector(){return ra},get FixedSizeBinaryVector(){return ia},get FixedSizeListVector(){return oa},get FloatVector(){return aa},get Float16Vector(){return sa},get Float32Vector(){return Aa},get Float64Vector(){return la},get IntervalVector(){return ca},get IntervalDayTimeVector(){return ua},get IntervalYearMonthVector(){return ga},get IntVector(){return ha},get Int8Vector(){return da},get Int16Vector(){return fa},get Int32Vector(){return pa},get Int64Vector(){return Ia},get Uint8Vector(){return Ca},get Uint16Vector(){return ma},get Uint32Vector(){return ya},get Uint64Vector(){return Ea},get ListVector(){return va},get MapVector(){return Ba},get NullVector(){return ba},get StructVector(){return Qa},get TimestampVector(){return wa},get TimestampSecondVector(){return xa},get TimestampMillisecondVector(){return _a},get TimestampMicrosecondVector(){return Da},get TimestampNanosecondVector(){return Ma},get TimeVector(){return ka},get TimeSecondVector(){return Sa},get TimeMillisecondVector(){return La},get TimeMicrosecondVector(){return Ra},get TimeNanosecondVector(){return Na},get UnionVector(){return Fa},get DenseUnionVector(){return Ta},get SparseUnionVector(){return Ua},get Utf8Vector(){return Oa}},uA=new Uint8Array(0);function gA(t,e){for(var n,r=void 0===r?e.reduce(function(t,e){return Math.max(t,e.length)},0):r,i=-1,a=e.length,s=[].concat(o.o(t.fields)),A=[],l=(r+63&-64)>>3;++i<a;)(n=e[i])&&n.length===r?A[i]=n:((t=s[i]).nullable||(s[i]=s[i].clone({nullable:!0})),A[i]=n?Un(n,r):Tn.new(t.type,0,r,r,[uA,uA,new Uint8Array(l),uA]));return[new _o(s),r,A]}function hA(t){return dA(new _o(t.map(function(t){return t.field})),t)}function dA(t,e){return fA(t,e.map(function(t){return t instanceof fo?t.chunks.map(function(t){return t.data}):[t.data]}))}function fA(t,e){for(var n,r=[].concat(o.o(t.fields)),i=[],a={Sb:e.reduce(function(t,e){return Math.max(t,e.length)},0)},s=0,A=0,l=-1,c=e.length,u=[];0<a.Sb--;){for(A=Number.POSITIVE_INFINITY,l=-1;++l<c;)u[l]=n=e[l].shift(),A=Math.min(A,n?n.length:A);isFinite(A)&&(u=pA(r,A,u,e,a),0<A&&(i[s++]=[A,u.slice()]))}return[t=new _o(r,t.metadata),i.map(function(e){return new(Function.prototype.bind.apply(dc,[null,t].concat(o.o(e))))})]}function pA(t,e,n,r,i){for(var o,a,s=0,A=-1,l=r.length,c=(e+63&-64)>>3;++A<l;)(o=n[A])&&(s=o.length)>=e?s===e?n[A]=o:(n[A]=o.slice(0,e),o=o.slice(e,s-e),i.Sb=Math.max(i.Sb,r[A].unshift(o))):((a=t[A]).nullable||(t[A]=a.clone({nullable:!0})),n[A]=o?Un(o,e):Tn.new(a.type,0,e,e,[uA,uA,new Uint8Array(c),uA]));return n}var IA=on.c.f.g.Null,CA=on.c.f.g.Int,mA=on.c.f.g.FloatingPoint,yA=on.c.f.g.Binary,EA=on.c.f.g.Bool,vA=on.c.f.g.Utf8,BA=on.c.f.g.Decimal,bA=on.c.f.g.Date,QA=on.c.f.g.Time,wA=on.c.f.g.Timestamp,xA=on.c.f.g.Interval,_A=on.c.f.g.List,DA=on.c.f.g.Struct_,MA=on.c.f.g.Union,kA=on.c.f.g.ac,SA=on.c.f.g.FixedSizeBinary,LA=on.c.f.g.FixedSizeList,RA=on.c.f.g.Map;function NA(){}o.h(NA,Gn),(r=NA.prototype).visit=function(t,e){return null==t||null==e?void 0:Gn.prototype.visit.call(this,t,e)},r.visitNull=function(t,e){return IA.jf(e),IA.ke(e)},r.visitInt=function(t,e){return CA.cf(e),CA.eb(e,t.bitWidth),CA.qd(e,t.isSigned),CA.ee(e)},r.visitFloat=function(t,e){return mA.af(e),mA.Lb(e,t.precision),mA.ce(e)},r.visitBinary=function(t,e){return yA.Oe(e),yA.Ud(e)},r.visitBool=function(t,e){return EA.Pe(e),EA.Vd(e)},r.visitUtf8=function(t,e){return vA.sf(e),vA.se(e)},r.visitDecimal=function(t,e){return BA.Te(e),BA.zd(e,t.scale),BA.Lb(e,t.precision),BA.Xd(e)},r.visitDate=function(t,e){return bA.Se(e),bA.qa(e,t.unit),bA.Wd(e)},r.visitTime=function(t,e){return QA.startTime(e),QA.qa(e,t.unit),QA.eb(e,t.bitWidth),QA.endTime(e)},r.visitTimestamp=function(t,e){var n=t.timezone&&X(e,t.timezone)||void 0;return wA.pf(e),wA.qa(e,t.unit),void 0!==n&&wA.Bd(e,n),wA.qe(e)},r.visitInterval=function(t,e){return xA.df(e),xA.qa(e,t.unit),xA.fe(e)},r.visitList=function(t,e){return _A.ef(e),_A.he(e)},r.visitStruct=function(t,e){return DA.nf(e),DA.pe(e)},r.visitUnion=function(t,e){MA.qf(e,t.typeIds.length);var n=MA.Sd(e,t.typeIds);return MA.rf(e),MA.ud(e,t.mode),MA.Cd(e,n),MA.re(e)},r.visitDictionary=function(t,e){var n=this.visit(t.indices,e);return kA.We(e),kA.Kb(e,new L(t.id,0)),kA.pd(e,t.isOrdered),void 0!==n&&kA.nd(e,n),kA.Zd(e)},r.visitFixedSizeBinary=function(t,e){return SA.Ze(e),SA.cd(e,t.byteWidth),SA.ae(e)},r.visitFixedSizeList=function(t,e){return LA.$e(e),LA.td(e,t.listSize),LA.be(e)},r.visitMap=function(t,e){return RA.ff(e),RA.rd(e,t.keysSorted),RA.ie(e)};var FA=new NA;function TA(t){return new rl(t.count,GA(t.columns),HA(t.columns))}function UA(t,e,n){return(t.fields||[]).filter(Boolean).map(function(t){return Mo.fromJSON(t,e,n)})}function OA(t,e,n){return(t.children||[]).filter(Boolean).map(function(t){return Mo.fromJSON(t,e,n)})}function GA(t){return(t||[]).reduce(function(t,e){return[].concat(o.o(t),[new al(e.count,jA(e.VALIDITY))],o.o(GA(e.children)))},[])}function HA(t,e){e=void 0===e?[]:e;for(var n=-1,r=(t||[]).length;++n<r;){var i=t[n];i.VALIDITY&&e.push(new ol(e.length,i.VALIDITY.length)),i.TYPE&&e.push(new ol(e.length,i.TYPE.length)),i.OFFSET&&e.push(new ol(e.length,i.OFFSET.length)),i.DATA&&e.push(new ol(e.length,i.DATA.length)),e=HA(i.children,e)}return e}function jA(t){return(t||[]).reduce(function(t,e){return t+ +(0===e)},0)}function YA(t){return new Map(Object.entries(t||{}))}function PA(t){return new cr(t.isSigned,t.bitWidth)}function KA(t,e){var n=t.type.name;switch(n){case"NONE":return new nr;case"null":return new ar;case"binary":return new Dr;case"utf8":return new Lr;case"bool":return new Tr;case"list":return new mi((e||[])[0]);case"struct":case"struct_":return new Bi(e||[])}switch(n){case"int":return new cr((e=t.type).isSigned,e.bitWidth);case"floatingpoint":return new vr(mn[t.type.precision]);case"decimal":return new Hr((e=t.type).scale,e.precision);case"date":return new Kr(In[t.type.unit]);case"time":return e=t.type,new Xr(Cn[e.unit],e.bitWidth);case"timestamp":return e=t.type,new oi(Cn[e.unit],e.timezone);case"interval":return new hi(vn[t.type.unit]);case"union":return t=t.type,new xi(yn[t.mode],t.typeIds||[],e||[]);case"fixedsizebinary":return new Li(t.type.byteWidth);case"fixedsizelist":return new Ti(t.type.listSize,(e||[])[0]);case"map":return new Hi(e||[],t.type.keysSorted)}throw Error('Unrecognized type: "'+n+'"')}var zA=on.c.f.g.Type,JA=on.c.f.g.Field,qA=on.c.f.g.Schema,VA=on.c.f.g.$b,WA=fn.c.f.g.Message,XA=on.c.f.g.sb,ZA=fn.c.f.g.bc,$A=on.c.f.g.rb,tl=fn.c.f.g.RecordBatch,el=fn.c.f.g.DictionaryBatch;function nl(t,e,n,r){this.Zc=e,this.Wc=n,this.body=new Uint8Array(0),r&&(this.zb=function(){return r}),this.Tc="number"==typeof t?t:t.low}function rl(t,e,n){this.Bb=e,this.wb=n,this.Fa="number"==typeof t?t:t.low}function il(t,e,n){this.Vc=t,this.Yc=void 0!==n&&n,this.Xc="number"==typeof e?e:e.low}function ol(t,e){this.offset="number"==typeof t?t:t.low,this.length="number"==typeof e?e:e.low}function al(t,e){this.length="number"==typeof t?t:t.low,this.nullCount="number"==typeof e?e:e.low}function sl(t,e){return function(){switch(e){case Bn.Schema:return _o.fromJSON(t);case Bn.RecordBatch:return rl.fromJSON(t);case Bn.DictionaryBatch:return il.fromJSON(t)}throw Error("Unrecognized Message type: { name: "+Bn[e]+", type: "+e+" }")}}function Al(t,e){return function(){switch(e){case Bn.Schema:return _o.decode(t.header(new qA));case Bn.RecordBatch:return rl.decode(t.header(new tl),t.version());case Bn.DictionaryBatch:return il.decode(t.header(new el),t.version())}throw Error("Unrecognized Message type: { name: "+Bn[e]+", type: "+e+" }")}}function ll(t,e,n){for(var r,i=e=void 0===e?new Map:e,o=n=void 0===n?new Map:n,a=[],s=-1,A=-1,l=t.ve();++s<l;)(r=t.fields(s))&&(a[++A]=Mo.decode(r,i,o));return new _o(a,pl(t),e,n)}function cl(t,e){e=void 0===e?bn.V4:e;for(var n,r=t.length(),i=[],o=-1,a=-1,s=t.Fe();++o<s;)(n=t.U(o))&&(i[++a]=al.decode(n));n=[],s=a=-1;for(var A=t.Ld();++a<A;)(o=t.buffers(a))&&(e<bn.V4&&(o.b+=8*(a+1)),n[++s]=ol.decode(o));return new rl(r,i,n)}function ul(t,e){return e=void 0===e?bn.V4:e,new il(rl.decode(t.data(),e),t.id(),t.ib())}function gl(t){return new ol(t.offset(),t.length())}function hl(t){return new al(t.length(),t.nullCount())}function dl(t,e,n){for(var r,i=[],o=-1,a=-1,s=t.Nd();++o<s;)(r=t.children(o))&&(i[++a]=Mo.decode(r,e,n));return i}function fl(t,e,n){var r,i;if(e&&n&&(i=t.dictionary()))if(e.has(r=i.id().low)){var o=(o=i.Ec())?Il(o):new pr;o=new zi(e.get(r),o,r,i.isOrdered()),t=new Mo(t.name(),o,t.nullable(),pl(t)),n.get(r).push(n=t)}else o=(o=i.Ec())?Il(o):new pr,e.set(r,e=Cl(t,dl(t))),o=new zi(e,o,r,i.isOrdered()),t=new Mo(t.name(),o,t.nullable(),pl(t)),n.set(r,[n=t]);else e=Cl(t,dl(t,e,n)),n=new Mo(t.name(),e,t.nullable(),pl(t));return n||null}function pl(t){var e=new Map;if(t)for(var n=void 0,r=void 0,i=-1,o=0|t.xc();++i<o;)(n=t.wc(i))&&null!=(r=n.key())&&e.set(r,n.value());return e}function Il(t){return new cr(t.isSigned(),t.bitWidth())}function Cl(t,e){var n=t.uf();switch(n){case zA.NONE:return new nr;case zA.Null:return new ar;case zA.Binary:return new Dr;case zA.Utf8:return new Lr;case zA.Bool:return new Tr;case zA.List:return new mi((e||[])[0]);case zA.Struct_:return new Bi(e||[])}switch(n){case zA.Int:return new cr((e=t.type(new on.c.f.g.Int)).isSigned(),e.bitWidth());case zA.FloatingPoint:return new vr((e=t.type(new on.c.f.g.FloatingPoint)).precision());case zA.Decimal:return new Hr((e=t.type(new on.c.f.g.Decimal)).scale(),e.precision());case zA.Date:return new Kr((e=t.type(new on.c.f.g.Date)).unit());case zA.Time:return new Xr((e=t.type(new on.c.f.g.Time)).unit(),e.bitWidth());case zA.Timestamp:return new oi((e=t.type(new on.c.f.g.Timestamp)).unit(),e.timezone());case zA.Interval:return new hi((e=t.type(new on.c.f.g.Interval)).unit());case zA.Union:return new xi((t=t.type(new on.c.f.g.Union)).mode(),t.tf()||[],e||[]);case zA.FixedSizeBinary:return new Li((e=t.type(new on.c.f.g.FixedSizeBinary)).byteWidth());case zA.FixedSizeList:return new Ti((t=t.type(new on.c.f.g.FixedSizeList)).listSize(),(e||[])[0]);case zA.Map:return new Hi(e||[],(t=t.type(new on.c.f.g.Map)).keysSorted())}throw Error('Unrecognized type: "'+zA[n]+'" ('+n+")")}function ml(t,e){var n=e.fields.map(function(e){return Mo.encode(t,e)});return qA.Ye(t,n.length),n=qA.Rd(t,n),e=e.metadata&&0<e.metadata.size?qA.Pb(t,[].concat(o.o(e.metadata)).map(function(e){var n=o.w(e);return e=n.next().value,n=n.next().value,e=X(t,""+e),n=X(t,""+n),XA.Kc(t),XA.oc(t,e),XA.qc(t,n),XA.yc(t)})):-1,qA.mf(t),qA.kd(t,n),qA.jd(t),-1!==e&&qA.Jb(t,e),qA.oe(t)}function yl(t,e){var n=-1,r=-1,i=-1,a=e.type,s=e.typeId;nr.isDictionary(a)?(s=a.dictionary.typeId,i=FA.visit(a,t),r=FA.visit(a.dictionary,t)):r=FA.visit(a,t),a=(a.children||[]).map(function(e){return Mo.encode(t,e)}),a=JA.Pd(t,a);var A=e.metadata&&0<e.metadata.size?JA.Pb(t,[].concat(o.o(e.metadata)).map(function(e){var n=o.w(e);return e=n.next().value,n=n.next().value,e=X(t,""+e),n=X(t,""+n),XA.Kc(t),XA.oc(t,e),XA.qc(t,n),XA.yc(t)})):-1;return e.name&&(n=X(t,e.name)),JA.Xe(t),JA.pc(t,r),JA.Dd(t,s),JA.dd(t,a),JA.xd(t,!!e.nullable),-1!==n&&JA.vd(t,n),-1!==i&&JA.hd(t,i),-1!==A&&JA.Jb(t,A),JA.$d(t)}function El(t,e){var n=e.U||[],r=e.buffers||[];return tl.hf(t,n.length),n.slice().reverse().forEach(function(e){return al.encode(t,e)}),n=W(t),tl.Qe(t,r.length),r.slice().reverse().forEach(function(e){return ol.encode(t,e)}),r=W(t),tl.kf(t),tl.sd(t,new L(e.length,0)),tl.wd(t,n),tl.bd(t,r),tl.me(t)}function vl(t,e){var n=rl.encode(t,e.data);return el.Ve(t),el.Kb(t,new L(e.id,0)),el.od(t,e.ib),el.ed(t,n),el.Yd(t)}function Bl(t,e){return ZA.Qd(t,new L(e.length,0),new L(e.nullCount,0))}function bl(t,e){return VA.createBuffer(t,new L(e.offset,0),new L(e.length,0))}nl.fromJSON=function(t,e){var n=new nl(0,bn.V4,e);return n.zb=sl(t,e),n},nl.decode=function(t){t=new Z(Wt.toUint8Array(t));var e=(t=WA.Ae(t)).bodyLength(),n=t.version(),r=t.headerType();return(e=new nl(e,n,r)).zb=Al(t,r),e},nl.encode=function(t){var e=new N,n=-1;return t.isSchema()?n=_o.encode(e,t.header()):t.isRecordBatch()?n=rl.encode(e,t.header()):t.isDictionaryBatch()&&(n=il.encode(e,t.header())),WA.gf(e),WA.Mb(e,bn.V4),WA.ld(e,n),WA.md(e,t.headerType),WA.ad(e,new L(t.bodyLength,0)),WA.xe(e,WA.je(e)),F(e)},nl.from=function(t,e){if(e=void 0===e?0:e,t instanceof _o)return new nl(0,bn.V4,Bn.Schema,t);if(t instanceof rl)return new nl(e,bn.V4,Bn.RecordBatch,t);if(t instanceof il)return new nl(e,bn.V4,Bn.DictionaryBatch,t);throw Error("Unrecognized Message header: "+t)},nl.prototype.header=function(){return this.zb()},nl.prototype.isSchema=function(){return this.headerType===Bn.Schema},nl.prototype.isRecordBatch=function(){return this.headerType===Bn.RecordBatch},nl.prototype.isDictionaryBatch=function(){return this.headerType===Bn.DictionaryBatch},o.global.Object.defineProperties(nl.prototype,{type:{configurable:!0,enumerable:!0,get:function(){return this.headerType}},version:{configurable:!0,enumerable:!0,get:function(){return this.Zc}},headerType:{configurable:!0,enumerable:!0,get:function(){return this.Wc}},bodyLength:{configurable:!0,enumerable:!0,get:function(){return this.Tc}}}),o.global.Object.defineProperties(rl.prototype,{U:{configurable:!0,enumerable:!0,get:function(){return this.Bb}},length:{configurable:!0,enumerable:!0,get:function(){return this.Fa}},buffers:{configurable:!0,enumerable:!0,get:function(){return this.wb}}}),o.global.Object.defineProperties(il.prototype,{id:{configurable:!0,enumerable:!0,get:function(){return this.Xc}},data:{configurable:!0,enumerable:!0,get:function(){return this.Vc}},ib:{configurable:!0,enumerable:!0,get:function(){return this.Yc}},length:{configurable:!0,enumerable:!0,get:function(){return this.data.length}},U:{configurable:!0,enumerable:!0,get:function(){return this.data.U}},buffers:{configurable:!0,enumerable:!0,get:function(){return this.data.buffers}}}),Mo.encode=yl,Mo.decode=fl,Mo.fromJSON=function(t,e,n){var r,i;if(e&&n&&(i=t.dictionary))if(e.has(r=i.id)){var o=(o=i.indexType)?PA(o):new pr;o=new zi(e.get(r),o,r,i.isOrdered),t=new Mo(t.name,o,t.nullable,YA(t.customMetadata)),n.get(r).push(n=t)}else o=(o=i.indexType)?PA(o):new pr,e.set(r,e=KA(t,OA(t))),o=new zi(e,o,r,i.isOrdered),t=new Mo(t.name,o,t.nullable,YA(t.customMetadata)),n.set(r,[n=t]);else e=KA(t,OA(t,e,n)),n=new Mo(t.name,e,t.nullable,YA(t.customMetadata));return n||null},_o.encode=ml,_o.decode=ll,_o.fromJSON=function(t,e,n){return new _o(UA(t,e=void 0===e?new Map:e,n=void 0===n?new Map:n),YA(t.customMetadata),e,n)},rl.encode=El,rl.decode=cl,rl.fromJSON=TA,il.encode=vl,il.decode=ul,il.fromJSON=function(t){return new il(TA(t.data),t.id,t.isDelta)},al.encode=Bl,al.decode=hl,ol.encode=bl,ol.decode=gl;var Ql=new ArrayBuffer(2);new DataView(Ql).setInt16(0,256,!0);var wl=256===new Int16Array(Ql)[0];function xl(t){Qt.apply(this,arguments)}function _l(t){t&&(this.source=new Ml($t(te(t))))}function Dl(t){t instanceof Dl?this.source=t.source:t instanceof xl?this.source=new kl($t(ee(t))):Gt(t)?this.source=new kl($t(le(t))):Rt(t)&&Ot(t.body)?this.source=new kl($t(ne(t.body))):Ft(t)?this.source=new kl($t(te(t))):Nt(t)?this.source=new kl($t(ee(t))):Tt(t)?this.source=new kl($t(ee(t))):Ot(t)&&(this.source=new kl($t(ne(t))))}function Ml(t){this.source=t}function kl(t){var e=this;this.source=t,this.yb=new Promise(function(t){return e.T=t})}function Sl(t,e){_l.call(this),this.position=0,this.buffer=Wt.toUint8Array(t),this.size=void 0===e?this.buffer.byteLength:e}function Ll(t,e){Dl.call(this);var n=this;this.position=0,this.R=t,"number"==typeof e?this.size=e:this.ea=function(){var e;return o.u(function(r){if(1==r.i)return delete n.ea,e=n,l(r,t.stat(),2);e.size=r.m.size,r.i=0})}()}function Rl(t){return"Expected "+Bn[t]+" Message in stream, but was null or length 0."}function Nl(t){return"Header pointer of flatbuffer-encoded "+Bn[t]+" Message is null or length 0."}function Fl(t,e){return"Expected to read "+t+" metadata bytes, but only read "+e+"."}function Tl(t,e){return"Expected to read "+t+" bytes for message body, but only read "+e+"."}function Ul(t){this.source=t instanceof _l?t:new _l(t)}function Ol(t,e){this.source=t instanceof Dl?t:Rt(t)&&Lt(t.stat)&&"number"==typeof t.fd?new Ll(t,e):new Dl(t)}function Gl(t){Ul.call(this,new Uint8Array(0)),this.F=!1,this.Xa=[],this.da=this.ec=0,this.Ea=t instanceof Et?t:new Et(t)}o.h(xl,Qt),xl.prototype.write=function(t){if(0<(t=Wt.toUint8Array(t)).byteLength)return Qt.prototype.write.call(this,t)},xl.prototype.toString=function(t){return void 0!==t&&t?Ct(this.toUint8Array(!0)):this.toUint8Array(!1).then(Ct)},xl.prototype.toUint8Array=function(t){var e=this;return void 0!==t&&t?Wt.joinUint8Arrays(this.Ka)[0]:function(){var t,n,r,i,a;return o.u(function(s){switch(s.i){case 1:t=[],n=0,r=o.kb(e);case 2:return l(s,r.next(),5);case 5:if((i=s.m).done){s.j(4);break}a=i.value,t.push(a),n+=a.byteLength,s.j(2);break;case 4:return s.return(Wt.joinUint8Arrays(t,n)[0])}})}()},o.l(),o.K(),_l.prototype[Symbol.iterator]=function(){return this},(r=_l.prototype).next=function(t){return this.source.next(t)},r.throw=function(t){return this.source.throw(t)},r.return=function(t){return this.source.return(t)},r.peek=function(t){return this.source.peek(t)},r.read=function(t){return this.source.read(t)},o.l(),o.L(),Dl.prototype[Symbol.asyncIterator]=function(){return this},(r=Dl.prototype).next=function(t){return this.source.next(t)},r.throw=function(t){return this.source.throw(t)},r.return=function(t){return this.source.return(t)},r.cancel=function(t){return this.source.cancel(t)},r.peek=function(t){return this.source.peek(t)},r.read=function(t){return this.source.read(t)},o.global.Object.defineProperties(Dl.prototype,{closed:{configurable:!0,enumerable:!0,get:function(){return this.source.closed}}}),(r=Ml.prototype).cancel=function(t){this.return(t)},r.peek=function(t){return this.next(t,"peek").value},r.read=function(t){return this.next(t,"read").value},r.next=function(t,e){return this.source.next({ga:void 0===e?"read":e,size:t})},r.throw=function(t){return Object.create(this.source.throw&&this.source.throw(t)||yt)},r.return=function(t){return Object.create(this.source.return&&this.source.return(t)||yt)},(r=kl.prototype).cancel=function(t){var e=this;return o.u(function(n){return l(n,e.return(t),0)})},r.read=function(t){var e=this;return o.u(function(n){return 1==n.i?l(n,e.next(t,"read"),2):n.return(n.m.value)})},r.peek=function(t){var e=this;return o.u(function(n){return 1==n.i?l(n,e.next(t,"peek"),2):n.return(n.m.value)})},r.next=function(t,e){e=void 0===e?"read":e;var n=this;return o.u(function(r){return 1==r.i?l(r,n.source.next({ga:e,size:t}),2):r.return(r.m)})},r.throw=function(t){var e,n,r=this;return o.u(function(i){return 1==i.i?(e=r.source.throw)?l(i,r.source.throw(t),3):i.j(2):(2!=i.i&&(e=i.m),n=e||yt,r.T&&r.T(),r.T=void 0,i.return(Object.create(n)))})},r.return=function(t){var e,n,r=this;return o.u(function(i){return 1==i.i?(e=r.source.return)?l(i,r.source.return(t),3):i.j(2):(2!=i.i&&(e=i.m),n=e||yt,r.T&&r.T(),r.T=void 0,i.return(Object.create(n)))})},o.global.Object.defineProperties(kl.prototype,{closed:{configurable:!0,enumerable:!0,get:function(){return this.yb}}}),o.h(Sl,_l),(r=Sl.prototype).v=function(t){return t=this.ya(t,4),new DataView(t.buffer,t.byteOffset).getInt32(0,!0)},r.seek=function(t){return this.position=Math.min(t,this.size),t<this.size},r.read=function(t){var e=this.buffer,n=this.size,r=this.position;return e&&r<n?("number"!=typeof t&&(t=1/0),this.position=Math.min(n,r+Math.min(n-r,t)),e.subarray(r,this.position)):null},r.ya=function(t,e){var n=this.buffer,r=Math.min(this.size,t+e);return n?n.subarray(t,r):new Uint8Array(e)},r.close=function(){this.buffer&&(this.buffer=null)},r.throw=function(t){return this.close(),{done:!0,value:t}},r.return=function(t){return this.close(),{done:!0,value:t}},o.h(Ll,Dl),(r=Ll.prototype).v=function(t){var e,n,r,i=this;return o.u(function(o){return 1==o.i?l(o,i.ya(t,4),2):(e=o.m,n=e.buffer,r=e.byteOffset,o.return(new DataView(n,r).getInt32(0,!0)))})},r.seek=function(t){var e=this;return o.u(function(n){return 1==n.i?e.ea?l(n,e.ea,3):n.j(2):(e.position=Math.min(t,e.size),n.return(t<e.size))})},r.read=function(t){var e,n,r,i,a,s,A,c,u,g,h=this;return o.u(function(o){switch(o.i){case 1:if(!h.ea){o.j(2);break}return l(o,h.ea,3);case 3:case 2:if(n=(e=h).R,r=e.size,i=e.position,!(n&&i<r)){o.j(4);break}"number"!=typeof t&&(t=1/0),a=i,A=s=0,c=Math.min(r,a+Math.min(r-a,t)),u=new Uint8Array(Math.max(0,(h.position=c)-a));case 5:if(!((a+=A)<c&&(s+=A)<u.byteLength)){o.j(6);break}return l(o,n.read(u,s,u.byteLength-s,a),7);case 7:g=o.m,A=g.Gf,o.j(5);break;case 6:return o.return(u);case 4:return o.return(null)}})},r.ya=function(t,e){var n,r,i,a,s,A=this;return o.u(function(o){switch(o.i){case 1:if(!A.ea){o.j(2);break}return l(o,A.ea,3);case 3:case 2:if(r=(n=A).R,i=n.size,!(r&&t+e<i)){o.j(4);break}return a=Math.min(i,t+e),s=new Uint8Array(a-t),l(o,r.read(s,0,e,t),5);case 5:return o.return(o.m.buffer);case 4:return o.return(new Uint8Array(e))}})},r.close=function(){var t,e=this;return o.u(function(n){if(1==n.i)return t=e.R,e.R=null,t?l(n,t.close(),3):n.j(2);n.i=0})},r.throw=function(t){var e=this;return o.u(function(n){return 1==n.i?l(n,e.close(),2):n.return({done:!0,value:t})})},r.return=function(t){var e=this;return o.u(function(n){return 1==n.i?l(n,e.close(),2):n.return({done:!0,value:t})})},o.l(),o.K(),Ul.prototype[Symbol.iterator]=function(){return this},(r=Ul.prototype).next=function(){var t;return(t=this.readMetadataLength()).done||(t=this.readMetadata(t.value)).done?yt:t},r.throw=function(t){return this.source.throw(t)},r.return=function(t){return this.source.return(t)},r.readMessage=function(t){var e;if((e=this.next()).done)return null;if(null!=t&&e.value.headerType!==t)throw Error(Rl(t));return e.value},r.readMessageBody=function(t){if(0>=t)return new Uint8Array(0);var e=Wt.toUint8Array(this.source.read(t));if(e.byteLength<t)throw Error(Tl(t,e.byteLength));return 0==e.byteOffset%8&&e.byteOffset+e.byteLength<=e.buffer.byteLength?e:e.slice()},r.readSchema=function(t){t=void 0!==t&&t;var e=Bn.Schema,n=this.readMessage(e);if(n=n&&n.header(),t&&!n)throw Error(Nl(e));return n},r.readMetadataLength=function(){var t=this.source.read(4);return{done:0>=(t=+((t=t&&new Z(t))&&t.v(0))),value:t}},r.readMetadata=function(t){var e=this.source.read(t);if(!e)return yt;if(e.byteLength<t)throw Error(Fl(t,e.byteLength));return{done:!1,value:nl.decode(e)}},o.l(),o.L(),Ol.prototype[Symbol.asyncIterator]=function(){return this},(r=Ol.prototype).next=function(){var t,e=this;return o.u(function(n){return 1==n.i?l(n,e.readMetadataLength(),2):3!=n.i?(t=n.m).done?n.return(yt):l(n,e.readMetadata(t.value),3):(t=n.m).done?n.return(yt):n.return(t)})},r.throw=function(t){var e=this;return o.u(function(n){return 1==n.i?l(n,e.source.throw(t),2):n.return(n.m)})},r.return=function(t){var e=this;return o.u(function(n){return 1==n.i?l(n,e.source.return(t),2):n.return(n.m)})},r.readMessage=function(t){var e,n=this;return o.u(function(r){if(1==r.i)return l(r,n.next(),2);if((e=r.m).done)return r.return(null);if(null!=t&&e.value.headerType!==t)throw Error(Rl(t));return r.return(e.value)})},r.readMessageBody=function(t){var e,n,r,i=this;return o.u(function(o){if(1==o.i)return 0>=t?o.return(new Uint8Array(0)):(n=(e=Wt).toUint8Array,l(o,i.source.read(t),2));if((r=n.call(e,o.m)).byteLength<t)throw Error(Tl(t,r.byteLength));return o.return(0==r.byteOffset%8&&r.byteOffset+r.byteLength<=r.buffer.byteLength?r:r.slice())})},r.readSchema=function(t){t=void 0!==t&&t;var e,n,r,i=this;return o.u(function(o){if(1==o.i)return e=Bn.Schema,l(o,i.readMessage(e),2);if(r=(n=o.m)&&n.header(),t&&!r)throw Error(Nl(e));return o.return(r)})},r.readMetadataLength=function(){var t,e,n,r=this;return o.u(function(i){return 1==i.i?l(i,r.source.read(4),2):(e=(t=i.m)&&new Z(t),n=+(e&&e.v(0)),i.return({done:0>=n,value:n}))})},r.readMetadata=function(t){var e,n=this;return o.u(function(r){if(1==r.i)return l(r,n.source.read(t),2);if(!(e=r.m))return r.return(yt);if(e.byteLength<t)throw Error(Fl(t,e.byteLength));return r.return({done:!1,value:nl.decode(e)})})},o.h(Gl,Ul),Gl.prototype.next=function(){var t=this.Ea,e=this.ec,n=this.da,r=t.vc.length,i=t.dictionaries.length;return this.F?n<i?(t=t.dictionaries[this.da++],this.Xa=t.data.columns,{done:!1,value:nl.fromJSON(t,Bn.DictionaryBatch)}):e<r?(t=t.vc[this.ec++],this.Xa=t.columns,{done:!1,value:nl.fromJSON(t,Bn.RecordBatch)}):(this.Xa=[],yt):(this.F=!0,{value:nl.fromJSON(t.schema,Bn.Schema),done:e>=r&&n>=i})},Gl.prototype.readMessageBody=function(){return function t(e){return(e||[]).reduce(function(e,n){return[].concat(o.o(e),o.o(n.VALIDITY&&[n.VALIDITY]||[]),o.o(n.TYPE&&[n.TYPE]||[]),o.o(n.OFFSET&&[n.OFFSET]||[]),o.o(n.DATA&&[n.DATA]||[]),o.o(t(n.children)))},[])}(this.Xa)},Gl.prototype.readMessage=function(t){var e;if((e=this.next()).done)return null;if(null!=t&&e.value.headerType!==t)throw Error(Rl(t));return e.value},Gl.prototype.readSchema=function(){var t=Bn.Schema,e=this.readMessage(t),n=e&&e.header();if(!e||!n)throw Error(Nl(t));return n};for(var Hl=new Uint8Array(6),jl=0;6>jl;jl+=1)Hl[jl]="ARROW1".charCodeAt(jl);function Yl(t,e){e=void 0===e?0:e;for(var n=-1,r=Hl.length;++n<r;)if(Hl[n]!==t[e+n])return!1;return!0}var Pl=Hl.length,Kl=Pl+4,zl=2*Pl+4;function Jl(){this.Ya=0,this.Bb=[],this.wb=[],this.Uc=[]}function ql(t){var e=t.byteLength+7&-8;return this.buffers.push(t),this.Ob.push(new ol(this.Ya,e)),this.Ya+=e,this}function Vl(t){return ql.call(this,t.values.subarray(0,t.length*t.stride))}function Wl(t){var e=t.length,n=t.values,r=(t=t.valueOffsets)[0],i=Math.min(t[e]-r,n.byteLength-r);return ql.call(this,Wt.rebaseValueOffsets(-t[0],e,t)),ql.call(this,n.subarray(r,r+i)),this}function Xl(t){var e=t.length,n=t.valueOffsets;return n&&ql.call(this,Wt.rebaseValueOffsets(n[0],e,n)),this.visit(t.getChildAt(0))}function Zl(t){return this.visitMany(t.type.children.map(function(e,n){return t.getChildAt(n)}).filter(Boolean))[0]}function $l(){}function tc(){}function ec(t){var e,n;return o.C.H(ec,function(r){return 1==r.i&&(e=o.w(t),n=e.next()),3!=r.i?n.done?r.j(0):l(r,n.value.reduce(function(t,e){return""+t+("0"+(255&e).toString(16)).slice(-2)},"").toUpperCase(),3):(n=e.next(),r.j(2))})}function nc(t,e){var n,r;return o.C.H(nc,function(i){return 1==i.i&&(n=-1,r=t.length/e),++n<r?l(i,""+za.new(t.subarray((n+0)*e,(n+1)*e),!1),2):i.j(0)})}function rc(t){this.Ga=0,this.Ja=!1,this.P=new xl,this.F=null,this.Ca=[],this.ta=[],this.Va=!t||"boolean"!=typeof t.autoDestroy||t.autoDestroy}function ic(t,e){return t.Ja&&(e=Wt.toUint8Array(e))&&0<e.byteLength&&(t.P.write(e),t.Ga+=e.byteLength),t}function oc(t,e){return 0<e?ic(t,new Uint8Array(e)):t}function ac(t,e){for(var n,r,i,o=-1,a=e.length;++o<a;)(n=e[o])&&0<(r=n.byteLength)&&(ic(t,n),0<(i=(r+7&-8)-r)&&oc(t,i))}function sc(t){rc.apply(this,arguments)}function Ac(){rc.call(this),this.Va=!0}function lc(){rc.call(this),this.Va=!0}function cc(t,e){var n=e;for(e instanceof hc&&(n=e.chunks,t.reset(void 0,e.schema)),n=(e=o.w(n)).next();!n.done;n=e.next())t.write(n.value);return t.finish()}function uc(t,e){var n,r,i;return o.u(function(a){switch(a.i){case 1:n=o.kb(e);case 2:return l(a,n.next(),5);case 5:if((r=a.m).done){a.j(4);break}i=r.value,t.write(i),a.j(2);break;case 4:return a.return(t.finish())}})}function gc(t){var e=t.type,n=new $l;return{name:t.name,nullable:t.nullable,type:n.visit(e),children:(e.children||[]).map(gc),dictionary:nr.isDictionary(e)?{id:e.id,isOrdered:e.isOrdered,indexType:n.visit(e.indices)}:void 0}}function hc(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];n=null,e[0]instanceof _o&&(n=e.shift());var r=vo(dc,e,[],0);if(!n&&!(n=r[0]&&r[0].schema))throw new TypeError("Table must be initialized with a Schema or at least one RecordBatch");return r[0]||(r[0]=new dc(n,0,n.fields.map(function(t){return Tn.new(t.type,0,0)}))),(e=cA.Chunked.call(this,new Bi(n.fields),r)||this).F=n,e.I=r,e}function dc(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];if(n=e[0],e[1]instanceof Tn){var r=o.w(e);r.next();var i=r.next().value;r=r.next().value}else{i=n.fields;var a=o.w(e);a.next(),e=a.next().value,a=a.next().value,i=Tn.Struct(new Bi(i),0,e,0,null,a)}Qa.call(this,i,r),this.F=n}function fc(t,e,n){this.Kd=this.Ee=-1,this.fa=t,this.U=e,this.buffers=n}function pc(t){return t.U[++t.Ee]}function Ic(t){return t.buffers[++t.Kd]}function Cc(t,e,n){fc.call(this,new Uint8Array(0),e,n),this.mb=t}function mc(t){t=t.join("");for(var e=new Uint8Array(t.length/2),n=0;n<t.length;n+=2)e[n>>1]=parseInt(t.substr(n,2),16);return e}function yc(t){this.D=t}function Ec(t){this.D=this.D=t}function vc(t){this.D=this.D=t}function Bc(t){Ec.call(this,t),this.D=t}function bc(t){vc.call(this,t),this.D=t}function Qc(t){t=void 0===t?new Map:t,this.closed=!1,this.autoDestroy=!0,this.ka=this.da=0,this.dictionaries=t}function wc(t,e,n){return new dc(t.schema,e.length,t.bb(e,n,t.schema.fields))}function xc(t,e,n){var r=e.id,i=e.ib;e=e.data;var o=t.dictionaries,a=t.schema;if(i||!o.get(r)){var s=a.dictionaries.get(r),A=i?o.get(r).concat(ce.new(t.bb(e,n,[s])[0])):ce.new(t.bb(e,n,[s])[0]);return(a.dictionaryFields.get(r)||[]).forEach(function(t){return t.type.dictionaryVector=A}),A}return o.get(r)}function _c(t,e){Qc.call(this,e),this.J=Ut(t)?new Gl(this.R=t):new Ul(this.R=t)}function Dc(t,e){Qc.call(this,e),this.J=new Ol(this.R=t)}function Mc(t,e){_c.call(this,t instanceof Sl?t:new Sl(t),e)}function kc(t,e){for(var n=[],r=1;r<arguments.length;++r)n[r-1]=arguments[r];r="number"!=typeof n[0]?n.shift():void 0,n=n[0]instanceof Map?n.shift():void 0,Dc.call(this,t instanceof Ll?t:new Ll(t,r),n)}function Sc(t,e){_c.call(this,t,e)}function Lc(t,e){return e&&"boolean"==typeof e.autoDestroy?e.autoDestroy:t.autoDestroy}function Rc(t){var e;return o.C.H(Rc,function(n){switch(n.i){case 1:if(e=yc.from(t),u(n,2),e.open({autoDestroy:!1}).closed){n.j(2);break}case 5:return l(n,e,7);case 7:if(!e.reset().open().closed){n.j(5);break}case 2:h(n),e.cancel(),d(n)}})}function Nc(t){return new w(function e(){var n;return o.C.H(e,function(e){switch(e.i){case 1:return l(e,new E(2,yc.from(t)),2);case 2:return n=e.m,u(e,3),l(e,new E(2,n.open({autoDestroy:!1})),5);case 5:if(e.m.closed){e.j(3);break}case 7:return l(e,new E(0,n),9);case 9:return l(e,new E(2,n.reset().open()),11);case 11:if(!e.m.closed){e.j(7);break}case 3:return h(e),l(e,new E(2,n.cancel()),12);case 12:d(e)}})}())}function Fc(t){var e=t.peek(Pl+7&-8);return e&&4<=e.byteLength?Yl(e)?new Bc(new Mc(t.read())):new Ec(new _c(t)):new Ec(new _c(function t(){return o.C.H(t,function(t){t.i=0})}()))}function Tc(t){var e,n,r,i,a;return o.u(function(s){switch(s.i){case 1:return l(s,t.peek(Pl+7&-8),2);case 2:if(!((e=s.m)&&4<=e.byteLength)){n=new vc(new Dc(new w(function t(){return o.C.H(t,function(t){t.i=0})}()))),s.j(3);break}if(!Yl(e)){r=new vc(new Dc(t)),s.j(4);break}return i=Bc,a=Mc,l(s,t.read(),5);case 5:r=new i(new a(s.m));case 4:n=r;case 3:return s.return(n)}})}function Uc(t){var e,n,r,i;return o.u(function(o){switch(o.i){case 1:return l(o,t.stat(),2);case 2:if(e=o.m,n=e.size,r=new Ll(t,n),!(n>=zl)){o.j(3);break}return i=Yl,l(o,r.ya(0,Pl+7&-8),4);case 4:if(i(o.m))return o.return(new bc(new kc(r)));case 3:return o.return(new vc(new Dc(r)))}})}function Oc(t,e){function n(t,e){for(var n,r,o=t.desiredSize||null;!(r=e.next(i?o:null)).done;)if(ArrayBuffer.isView(r.value)&&(n=Wt.toUint8Array(r.value))&&(null!=o&&i&&(o=o-n.byteLength+1),r.value=n),t.enqueue(r.value),null!=o&&0>=--o)return;t.close()}var r=null,i=e&&"bytes"===e.type||!1,a=e&&e.highWaterMark||Math.pow(2,24);return new ReadableStream(Object.assign({},e,{start:function(e){o.l(),o.K(),n(e,r||(r=t[Symbol.iterator]()))},pull:function(t){r?n(t,r):t.close()},cancel:function(){r&&r.return&&r.return(),r=null}}),Object.assign({},{highWaterMark:i?a:void 0},e))}function Gc(t,e){function n(t,e){var n,r,a;return o.u(function(o){switch(o.i){case 1:r=null,a=t.desiredSize||null;case 2:return l(o,e.next(i?a:null),4);case 4:if((r=o.m).done){o.j(3);break}if(ArrayBuffer.isView(r.value)&&(n=Wt.toUint8Array(r.value))&&(null!=a&&i&&(a=a-n.byteLength+1),r.value=n),t.enqueue(r.value),null!=a&&0>=--a)return o.return();o.j(2);break;case 3:t.close(),o.i=0}})}var r=null,i=e&&"bytes"===e.type||!1,a=e&&e.highWaterMark||Math.pow(2,24);return new ReadableStream(Object.assign({},e,{start:function(e){return o.u(function(i){return o.l(),o.L(),l(i,n(e,r||(r=t[Symbol.asyncIterator]())),0)})},pull:function(t){return o.u(function(e){if(1==e.i)return r?l(e,n(t,r),3):(t.close(),e.j(2));e.i=0})},cancel:function(){return o.u(function(t){switch(t.i){case 1:if(!r){t.j(2);break}if(!r.return){t.j(3);break}return l(t,r.return(),4);case 4:case 3:case 2:r=null,t.i=0}})}}),Object.assign({},{highWaterMark:i?a:void 0},e))}function Hc(){}function jc(t){this.W=t}function Yc(t){this.name=t}function Pc(){}function Kc(t,e){this.left=t,this.right=e}function zc(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];this.children=e}function Jc(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];e=e.reduce(function(t,e){return t.concat(e instanceof Jc?e.children:e)},[]),zc.apply(this,o.o(e))}function qc(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];e=e.reduce(function(t,e){return t.concat(e instanceof qc?e.children:e)},[]),zc.apply(this,o.o(e))}function Vc(t){Kc.apply(this,arguments)}function Wc(t){Kc.apply(this,arguments)}function Xc(t){Kc.apply(this,arguments)}function Zc(t){this.Md=t}function $c(t,e){this.next=t,this.Hd=e}o.h(Jl,Gn),Jl.Ma=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return n=new Jl,e=bo(dc,e,[],0),void 0===(e=o.w(n.visitMany(e)).next().value)?n:e},Jl.prototype.visit=function(t){if(!nr.isDictionary(t.type)){var e=t.data,n=t.length,r=t.nullCount;if(2147483647<n)throw new RangeError("Cannot write arrays larger than 2^31 - 1 in length");ql.call(this,0>=r?new Uint8Array(0):Fn.truncateBitmap(e.offset,n,e.nullBitmap)).U.push(new al(n,r))}return Gn.prototype.visit.call(this,t)},Jl.prototype.visitNull=function(){return ql.call(this,new Uint8Array(0))},Jl.prototype.visitDictionary=function(t){return this.visit(t.indices)},o.global.Object.defineProperties(Jl.prototype,{U:{configurable:!0,enumerable:!0,get:function(){return this.Bb}},buffers:{configurable:!0,enumerable:!0,get:function(){return this.wb}},byteLength:{configurable:!0,enumerable:!0,get:function(){return this.Ya}},Ob:{configurable:!0,enumerable:!0,get:function(){return this.Uc}}}),(r=Jl.prototype).visitBool=function(t){var e;return t.nullCount>=t.length?ql.call(this,new Uint8Array(0)):(e=t.values)instanceof Uint8Array?ql.call(this,Fn.truncateBitmap(t.offset,t.length,e)):ql.call(this,Fn.packBools(t))},r.visitInt=Vl,r.visitFloat=Vl,r.visitUtf8=Wl,r.visitBinary=Wl,r.visitFixedSizeBinary=Vl,r.visitDate=Vl,r.visitTimestamp=Vl,r.visitTime=Vl,r.visitDecimal=Vl,r.visitList=Xl,r.visitStruct=Zl,r.visitUnion=function(t){var e=t.type,n=t.length,r=t.typeIds,i=t.valueOffsets;if(ql.call(this,r),e.mode===yn.Sparse)return Zl.call(this,t);if(e.mode===yn.Dense){if(0>=t.offset)return ql.call(this,i),Zl.call(this,t);var o=r.reduce(function(t,e){return Math.max(t,e)},r[0]),a=new Int32Array(o+1);o=new Int32Array(o+1).fill(-1);var s=new Int32Array(n);i=Wt.rebaseValueOffsets(-i[0],n,i);for(var A=void 0,l=void 0,c=-1;++c<n;)-1===(l=o[A=r[c]])&&(l=o[A]=i[A]),s[c]=i[c]-l,++a[A];for(ql.call(this,s),r=void 0,s=-1,i=e.children.length;++s<i;)(r=t.getChildAt(s))&&(A=e.typeIds[s],this.visit(r.slice(o[A],Math.min(n,a[A]))))}return this},r.visitInterval=Vl,r.visitFixedSizeList=Xl,r.visitMap=Zl,o.h($l,Gn),(r=$l.prototype).visit=function(t){return null==t?void 0:Gn.prototype.visit.call(this,t)},r.visitNull=function(t){return{name:pn[t.typeId].toLowerCase()}},r.visitInt=function(t){return{name:pn[t.typeId].toLowerCase(),bitWidth:t.bitWidth,isSigned:t.isSigned}},r.visitFloat=function(t){return{name:pn[t.typeId].toLowerCase(),precision:mn[t.precision]}},r.visitBinary=function(t){return{name:pn[t.typeId].toLowerCase()}},r.visitBool=function(t){return{name:pn[t.typeId].toLowerCase()}},r.visitUtf8=function(t){return{name:pn[t.typeId].toLowerCase()}},r.visitDecimal=function(t){return{name:pn[t.typeId].toLowerCase(),scale:t.scale,precision:t.precision}},r.visitDate=function(t){return{name:pn[t.typeId].toLowerCase(),unit:In[t.unit]}},r.visitTime=function(t){return{name:pn[t.typeId].toLowerCase(),unit:Cn[t.unit],bitWidth:t.bitWidth}},r.visitTimestamp=function(t){return{name:pn[t.typeId].toLowerCase(),unit:Cn[t.unit],timezone:t.timezone}},r.visitInterval=function(t){return{name:pn[t.typeId].toLowerCase(),unit:vn[t.unit]}},r.visitList=function(t){return{name:pn[t.typeId].toLowerCase()}},r.visitStruct=function(t){return{name:pn[t.typeId].toLowerCase()}},r.visitUnion=function(t){return{name:pn[t.typeId].toLowerCase(),mode:yn[t.mode],typeIds:[].concat(o.o(t.typeIds))}},r.visitDictionary=function(t){return this.visit(t.dictionary)},r.visitFixedSizeBinary=function(t){return{name:pn[t.typeId].toLowerCase(),byteWidth:t.byteWidth}},r.visitFixedSizeList=function(t){return{name:pn[t.typeId].toLowerCase(),listSize:t.listSize}},r.visitMap=function(t){return{name:pn[t.typeId].toLowerCase(),keysSorted:t.keysSorted}},o.h(tc,Gn),tc.Ma=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return(new tc).visitMany(Qo(dc,e,[],0))},(r=tc.prototype).visit=function(t){var e=t.data,n=t.name,r=t.length,i=e.offset,a=e.nullCount,s=e.nullBitmap;t=nr.isDictionary(t.type)?t.type.indices:t.type;var A={};return A=Object.assign([],e.buffers,(A[En.VALIDITY]=void 0,A)),Object.assign({},{name:n,count:r,VALIDITY:0>=a?Array.from({length:r},function(){return 1}):[].concat(o.o(Fn.iterateBits(s,i,r,null,Fn.getBit)))},Gn.prototype.visit.call(this,ce.new(e.clone(t,i,r,0,A))))},r.visitNull=function(){return{DATA:[]}},r.visitBool=function(t){return{DATA:[].concat(o.o(Fn.iterateBits(t.values,t.offset,t.length,null,Fn.getBool)))}},r.visitInt=function(t){return{DATA:64>t.type.bitWidth?[].concat(o.o(t.values)):[].concat(o.o(nc(t.values,2)))}},r.visitFloat=function(t){return{DATA:[].concat(o.o(t.values))}},r.visitUtf8=function(t){return{DATA:[].concat(o.o(t)),OFFSET:[].concat(o.o(t.valueOffsets))}},r.visitBinary=function(t){return{DATA:[].concat(o.o(ec(t))),OFFSET:[].concat(o.o(t.valueOffsets))}},r.visitFixedSizeBinary=function(t){return{DATA:[].concat(o.o(ec(t)))}},r.visitDate=function(t){return{DATA:t.type.unit===In.DAY?[].concat(o.o(t.values)):[].concat(o.o(nc(t.values,2)))}},r.visitTimestamp=function(t){return{DATA:[].concat(o.o(nc(t.values,2)))}},r.visitTime=function(t){return{DATA:t.type.unit<Cn.MICROSECOND?[].concat(o.o(t.values)):[].concat(o.o(nc(t.values,2)))}},r.visitDecimal=function(t){return{DATA:[].concat(o.o(nc(t.values,4)))}},r.visitList=function(t){var e=this;return{OFFSET:[].concat(o.o(t.valueOffsets)),children:t.type.children.map(function(n,r){return e.visit(new Co(n,[t.getChildAt(r)]))})}},r.visitStruct=function(t){var e=this;return{children:t.type.children.map(function(n,r){return e.visit(new Co(n,[t.getChildAt(r)]))})}},r.visitUnion=function(t){var e=this;return{TYPE:[].concat(o.o(t.typeIds)),OFFSET:t.type.mode===yn.Dense?[].concat(o.o(t.valueOffsets)):void 0,children:t.type.children.map(function(n,r){return e.visit(new Co(n,[t.getChildAt(r)]))})}},r.visitInterval=function(t){return{DATA:[].concat(o.o(t.values))}},r.visitFixedSizeList=function(t){var e=this;return{children:t.type.children.map(function(n,r){return e.visit(new Co(n,[t.getChildAt(r)]))})}},r.visitMap=function(t){var e=this;return{children:t.type.children.map(function(n,r){return e.visit(new Co(n,[t.getChildAt(r)]))})}},o.l(),o.L(),o.h(rc,vt),rc.throughNode=function(){throw Error('"throughNode" not available in this environment')},rc.throughDOM=function(){throw Error('"throughDOM" not available in this environment')},rc.prototype.toString=function(t){return this.P.toString(void 0!==t&&t)},rc.prototype.toUint8Array=function(t){return this.P.toUint8Array(void 0!==t&&t)},rc.prototype.writeAll=function(t){var e=this;return Nt(t)?t.then(function(t){return e.writeAll(t)}):Tt(t)?uc(this,t):cc(this,t)},rc.prototype[Symbol.asyncIterator]=function(){return o.l(),o.L(),this.P[Symbol.asyncIterator]()},(r=rc.prototype).toDOMStream=function(t){return this.P.toDOMStream(t)},r.toNodeStream=function(t){return this.P.toNodeStream(t)},r.close=function(){return this.reset().P.close()},r.abort=function(t){return this.reset().P.abort(t)},r.finish=function(){return this.Va?this.close():this.reset(this.P,this.F),this},r.reset=function(t,e){return e=void 0===e?null:e,(t=void 0===t?this.P:t)===this.P||t instanceof xl?this.P=t:(this.P=new xl,t&&Rt(t)&&Lt(t.abort)&&Lt(t.getWriter)&&!(t instanceof vt)?this.toDOMStream({type:"bytes"}).pipeTo(t):t&&Rt(t)&&Lt(t.end)&&Lt(t.write)&&"boolean"==typeof t.writable&&!(t instanceof vt)&&Bt(this.toNodeStream({ig:!1}),t)),this.Ja&&this.F&&this.mc(),this.Ja=!1,this.Ca=[],this.ta=[],e&&e.compareTo(this.F)||(null===e?(this.Ga=0,this.F=null):(this.Ja=!0,this.F=e,this.Ib(e))),this},r.write=function(t){var e=null;if(!this.P)throw Error("RecordBatchWriter is closed");if(null==t||t instanceof hc&&!(e=t.schema)||t instanceof dc&&!(e=t.schema))return this.finish()&&void 0;if(e&&!e.compareTo(this.F)){if(this.Ja&&this.Va)return this.close();this.reset(this.P,e)}t instanceof dc?this.nc(t):t instanceof hc?this.writeAll(t.chunks):Ft(t)&&this.writeAll(t)},r.cb=function(t){var e=nl.encode(t),n=e.byteLength,r=n+4+7&-8,i=r-n-4;return t.headerType===Bn.RecordBatch?this.ta.push(new Oo(r,t.bodyLength,this.Ga)):t.headerType===Bn.DictionaryBatch&&this.Ca.push(new Oo(r,t.bodyLength,this.Ga)),ic(this,Int32Array.of(r-4)),0<n&&ic(this,e),oc(this,i)},r.Ib=function(t){this.cb(nl.from(t)).La(t.dictionaryFields)},r.mc=function(){oc(this,4)},r.nc=function(t){var e=Jl.Ma(t),n=e.buffers;t=nl.from(new rl(t.length,e.U,e.Ob),e.byteLength),ac(this.cb(t),n)},r.Hb=function(t,e,n){n=void 0!==n&&n;var r=Jl.Ma(t),i=r.buffers;t=nl.from(new il(new rl(t.length,r.U,r.Ob),e,n),r.byteLength),ac(this.cb(t),i)},r.La=function(t){for(var e=(t=o.w(t)).next();!e.done;e=t.next()){var n=o.w(e.value);if(e=n.next().value,(n=n.next().value[0].type.dictionaryVector)instanceof fo)for(var r=-1,i=(n=n.chunks).length;++r<i;)this.Hb(n[r],e,0<r);else this.Hb(n,e,!1)}},o.global.Object.defineProperties(rc.prototype,{closed:{configurable:!0,enumerable:!0,get:function(){return this.P.closed}}}),o.h(sc,rc),sc.writeAll=function(t,e){return new sc(e).writeAll(t)},o.h(Ac,rc),Ac.writeAll=function(t){return(new Ac).writeAll(t)},Ac.prototype.Ib=function(t){oc(ic(this,Hl),2).La(t.dictionaryFields)},Ac.prototype.mc=function(){var t=Fo.encode(new Fo(this.F,bn.V4,this.ta,this.Ca));ic(ic(ic(this,t),Int32Array.of(t.byteLength)),Hl)},o.h(lc,rc),lc.writeAll=function(t){return(new lc).writeAll(t)},(r=lc.prototype).cb=function(){return this},r.Ib=function(t){ic(this,'{\n  "schema": '+JSON.stringify({fields:t.fields.map(gc)},null,2)).La(t.dictionaryFields)},r.La=function(t){ic(this,',\n  "dictionaries": [\n'),rc.prototype.La.call(this,t),ic(this,"\n  ]")},r.Hb=function(t,e,n){n=void 0!==n&&n,ic(this,0===this.Ca.length?"    ":",\n    "),n=void 0!==n&&n;var r=this.F.dictionaryFields.get(e)[0];r=tc.Ma(new Co(new Mo(r.name,r.type.dictionary,r.nullable,r.metadata),[t])),ic(this,""+(t=JSON.stringify({id:e,isDelta:n,data:{count:t.length,columns:r}},null,2))),this.Ca.push(new Oo(0,0,0))},r.nc=function(t){ic(this,0===this.ta.length?',\n  "batches": [\n    ':",\n    "),ic(this,""+JSON.stringify({count:t.length,columns:tc.Ma(t)},null,2)),this.ta.push(new Oo(0,0,0))},r.close=function(){return 0<this.ta.length&&ic(this,"\n  ]"),this.F&&ic(this,"\n}"),rc.prototype.close.call(this)},o.h(hc,cA.Chunked),hc.empty=function(){return new hc(new _o([]),[])},hc.from=function(t){if(!t)return hc.empty();var e=yc.from(t);return Nt(e)?function(){var t,n;return o.u(function(r){return 1==r.i?(t=hc,n=hc.from,l(r,e,3)):2!=r.i?l(r,n.call(t,r.m),2):r.return(r.m)})}():e.isSync()&&(e=e.open())?e.schema?new hc(e.schema,[].concat(o.o(e))):hc.empty():function(t){var e,n,r,i,a,s;return o.u(function(A){switch(A.i){case 1:return l(A,t,2);case 2:if(e=A.m,n=e.schema,r=[],!n){A.j(3);break}i=o.kb(e);case 4:return l(A,i.next(),7);case 7:if((a=A.m).done){A.j(6);break}s=a.value,r.push(s),A.j(4);break;case 6:return A.return(new hc(n,r));case 3:return A.return(hc.empty())}})}(e.open())},hc.fromAsync=function(t){return o.u(function(e){return 1==e.i?l(e,hc.from(t),2):e.return(e.m)})},hc.fromStruct=function(t){return hc.new(t.data.childData,t.type.children)},hc.new=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return new(Function.prototype.bind.apply(hc,[null].concat(o.o(hA(Eo(e))))))},(r=hc.prototype).clone=function(t){return t=void 0===t?this.I:t,new hc(this.F,t)},r.getColumn=function(t){return this.getColumnAt(this.getColumnIndex(t))},r.getColumnAt=function(t){return this.getChildAt(t)},r.getColumnIndex=function(t){return this.F.fields.findIndex(function(e){return e.name===t})},r.getChildAt=function(t){if(0>t||t>=this.numChildren)return null;var e,n=this.F.fields,r=this.Y||(this.Y=[]);return(e=r[t])?e:(e=n[t])&&0<(n=this.I.map(function(e){return e.getChildAt(t)}).filter(function(t){return null!=t})).length?r[t]=new Co(e,n):null},r.serialize=function(t,e){return(void 0===e||e?sc:Ac).writeAll(this.I).toUint8Array(!0)},r.count=function(){return this.Fa},r.select=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];var r=this.F.fields.reduce(function(t,e,n){return t.set(e.name,n)},new Map);return this.selectAt.apply(this,o.o(e.map(function(t){return r.get(t)}).filter(function(t){return-1<t})))},r.selectAt=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];var r=this.F.selectAt.apply(this.F,o.o(e));return new hc(r,this.I.map(function(t){var n=t.data.childData;return new dc(r,t.length,e.map(function(t){return n[t]}).filter(Boolean))}))},r.assign=function(t){var e=this,n=this.F.fields,r=o.w(t.schema.fields.reduce(function(t,e,r){var i=o.w(t),a=i.next().value;i=i.next().value;var s=n.findIndex(function(t){return t.name===e.name});return~s?i[s]=r:a.push(r),t},[[],[]])),i=r.next().value,a=r.next().value;return r=this.F.assign(t.schema),i=[].concat(o.o(n.map(function(n,r,i,o){return void 0===(o=void 0===o?a[r]:o)?e.getColumnAt(r):t.getColumnAt(o)})),o.o(i.map(function(e){return t.getColumnAt(e)}))).filter(Boolean),new(Function.prototype.bind.apply(hc,[null].concat(o.o(dA(r,i)))))},o.global.Object.defineProperties(hc.prototype,{schema:{configurable:!0,enumerable:!0,get:function(){return this.F}},length:{configurable:!0,enumerable:!0,get:function(){return this.Fa}},chunks:{configurable:!0,enumerable:!0,get:function(){return this.I}},numCols:{configurable:!0,enumerable:!0,get:function(){return this.Cb}}}),o.h(dc,Qa),dc.from=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return dc.new(e[0],e[1])},dc.new=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return e=(n=o.w(xo(e,[[],[]]))).next().value,n=n.next().value.filter(function(t){return t instanceof ce}),new(Function.prototype.bind.apply(dc,[null].concat(o.o(gA(new _o(e),n.map(function(t){return t.data}))))))},dc.prototype.clone=function(t,e){return e=void 0===e?this.Y:e,new dc(this.F,t,e)},dc.prototype.concat=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];var r=this.F;return e=fo.flatten.apply(fo,[this].concat(o.o(e))),new hc(r,e.map(function(t){return new dc(r,t.data)}))},dc.prototype.select=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];var r=this.F.fields.reduce(function(t,e,n){return t.set(e.name,n)},new Map);return this.selectAt.apply(this,o.o(e.map(function(t){return r.get(t)}).filter(function(t){return-1<t})))},dc.prototype.selectAt=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];var r=this;return n=this.F.selectAt.apply(this.F,o.o(e)),e=e.map(function(t){return r.data.childData[t]}).filter(Boolean),new dc(n,this.length,e)},o.global.Object.defineProperties(dc.prototype,{schema:{configurable:!0,enumerable:!0,get:function(){return this.F}},numCols:{configurable:!0,enumerable:!0,get:function(){return this.F.fields.length}}}),o.h(fc,Gn),(r=fc.prototype).visit=function(t){return Gn.prototype.visit.call(this,t instanceof Mo?t.type:t)},r.visitNull=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Null(t,0,e.length,n,this.M(t,n),this.O(t))},r.visitBool=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Bool(t,0,e.length,n,this.M(t,n),this.O(t))},r.visitInt=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Int(t,0,e.length,n,this.M(t,n),this.O(t))},r.visitFloat=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Float(t,0,e.length,n,this.M(t,n),this.O(t))},r.visitUtf8=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Utf8(t,0,e.length,n,this.M(t,n),this.Ra(t),this.O(t))},r.visitBinary=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Binary(t,0,e.length,n,this.M(t,n),this.Ra(t),this.O(t))},r.visitFixedSizeBinary=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.FixedSizeBinary(t,0,e.length,n,this.M(t,n),this.O(t))},r.visitDate=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Date(t,0,e.length,n,this.M(t,n),this.O(t))},r.visitTimestamp=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Timestamp(t,0,e.length,n,this.M(t,n),this.O(t))},r.visitTime=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Time(t,0,e.length,n,this.M(t,n),this.O(t))},r.visitDecimal=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Decimal(t,0,e.length,n,this.M(t,n),this.O(t))},r.visitList=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.List(t,0,e.length,n,this.M(t,n),this.Ra(t),this.visit(t.children[0]))},r.visitStruct=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Struct(t,0,e.length,n,this.M(t,n),this.visitMany(t.children))},r.visitUnion=function(t){return t.mode===yn.Sparse?this.visitSparseUnion(t):this.visitDenseUnion(t)},r.visitDenseUnion=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Union(t,0,e.length,n,this.M(t,n),this.Tb(t),this.Ra(t),this.visitMany(t.children))},r.visitSparseUnion=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Union(t,0,e.length,n,this.M(t,n),this.Tb(t),this.visitMany(t.children))},r.visitDictionary=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Dictionary(t,0,e.length,n,this.M(t,n),this.O(t.indices))},r.visitInterval=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Interval(t,0,e.length,n,this.M(t,n),this.O(t))},r.visitFixedSizeList=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.FixedSizeList(t,0,e.length,n,this.M(t,n),this.visit(t.children[0]))},r.visitMap=function(t,e){var n=(e=void 0===e?pc(this):e).nullCount;return Tn.Map(t,0,e.length,n,this.M(t,n),this.visitMany(t.children))},r.M=function(t,e){var n=void 0===n?Ic(this):n;return 0<e&&this.O(t,n)||new Uint8Array(0)},r.Ra=function(t){return this.O(t,void 0)},r.Tb=function(t){return this.O(t,void 0)},r.O=function(t,e){return e=(t=void 0===e?Ic(this):e).offset,this.fa.subarray(e,e+t.length)},o.h(Cc,fc),Cc.prototype.M=function(t,e){return t=Ic(this).offset,0>=e?new Uint8Array(0):Fn.packBools(this.mb[t])},Cc.prototype.Ra=function(){var t=Ic(this).offset;return Wt.toArrayBufferView(Uint8Array,Wt.toArrayBufferView(Int32Array,this.mb[t]))},Cc.prototype.Tb=function(t){var e=Ic(this).offset;return Wt.toArrayBufferView(Uint8Array,Wt.toArrayBufferView(t.ArrayType,this.mb[e]))},Cc.prototype.O=function(t,e){e=(void 0===e?Ic(this):e).offset;var n=this.mb;return nr.isTimestamp(t)||(nr.isInt(t)||nr.isTime(t))&&64===t.bitWidth||nr.isDate(t)&&t.unit===In.MILLISECOND?Wt.toArrayBufferView(Uint8Array,Zo.Int64.convertArray(n[e])):nr.isDecimal(t)?Wt.toArrayBufferView(Uint8Array,Zo.Int128.convertArray(n[e])):nr.isBinary(t)||nr.isFixedSizeBinary(t)?mc(n[e]):nr.isBool(t)?Fn.packBools(n[e]):nr.isUtf8(t)?mt(n[e].join("")):Wt.toArrayBufferView(Uint8Array,Wt.toArrayBufferView(t.ArrayType,n[e].map(function(t){return+t})))},o.l(),o.K(),o.l(),o.L(),o.h(yc,vt),(r=yc.prototype).isSync=function(){return this.D.isSync()},r.isAsync=function(){return this.D.isAsync()},r.isFile=function(){return this.D.isFile()},r.isStream=function(){return this.D.isStream()},r.next=function(){return this.D.next()},r.throw=function(t){return this.D.throw(t)},r.return=function(t){return this.D.return(t)},r.cancel=function(){return this.D.cancel()},r.reset=function(t){return this.D.reset(t),this.kc=this.dc=void 0,this},r.open=function(t){var e=this;return Nt(t=this.D.open(t))?t.then(function(){return e}):this},r.readRecordBatch=function(t){return this.D.isFile()?this.D.readRecordBatch(t):null},yc.prototype[Symbol.iterator]=function(){return o.l(),o.K(),this.D[Symbol.iterator]()},yc.prototype[Symbol.asyncIterator]=function(){return o.l(),o.L(),this.D[Symbol.asyncIterator]()},yc.prototype.toDOMStream=function(){var t=this;o.l(),o.K(),o.l(),o.L();var e={},n={};return Xt(this.isSync()?(e[Symbol.iterator]=function(){return t},e):(n[Symbol.asyncIterator]=function(){return t},n))},yc.prototype.toNodeStream=function(){var t=this;o.l(),o.K(),o.l(),o.L();var e={},n={};return Zt(this.isSync()?e[Symbol.iterator]=function(){return t}:n[Symbol.asyncIterator]=function(){return t})},yc.throughNode=function(){throw Error('"throughNode" not available in this environment')},yc.throughDOM=function(){throw Error('"throughDOM" not available in this environment')},yc.from=function(t){return t instanceof yc?t:Ut(t)?new Ec(new Sc(t)):Rt(t)&&Lt(t.stat)&&"number"==typeof t.fd?Uc(t):Nt(t)?function(){var e,n;return o.u(function(r){return 1==r.i?(e=yc,n=yc.from,l(r,t,3)):2!=r.i?l(r,n.call(e,r.m),2):r.return(r.m)})}():Rt(t)&&Ot(t.body)||Ot(t)||Gt(t)||Tt(t)?Tc(new Dl(t)):Fc(new _l(t))},yc.readAll=function(t){return t instanceof yc?t.isSync()?Rc(t):Nc(t):Ut(t)||ArrayBuffer.isView(t)||Ft(t)||Rt(t)&&"done"in t&&"value"in t?Rc(t):Nc(t)},o.global.Object.defineProperties(yc.prototype,{closed:{configurable:!0,enumerable:!0,get:function(){return this.D.closed}},schema:{configurable:!0,enumerable:!0,get:function(){return this.D.schema}},autoDestroy:{configurable:!0,enumerable:!0,get:function(){return this.D.autoDestroy}},dictionaries:{configurable:!0,enumerable:!0,get:function(){return this.D.dictionaries}},numDictionaries:{configurable:!0,enumerable:!0,get:function(){return this.D.numDictionaries}},numRecordBatches:{configurable:!0,enumerable:!0,get:function(){return this.D.numRecordBatches}},footer:{configurable:!0,enumerable:!0,get:function(){return this.D.isFile()?this.D.footer:null}}}),o.l(),o.K(),o.l(),o.L(),o.h(Ec,yc),Ec.prototype[Symbol.iterator]=function(){return o.l(),o.K(),this.D[Symbol.iterator]()},Ec.prototype[Symbol.asyncIterator]=function(){var t=this;return new w(function e(){return o.C.H(e,function(e){return o.l(),o.K(),l(e,new E(1,t[Symbol.iterator]()),0)})}())},o.l(),o.K(),o.l(),o.L(),o.h(vc,yc),vc.prototype[Symbol.iterator]=function(){throw Error("AsyncRecordBatchStreamReader is not Iterable")},vc.prototype[Symbol.asyncIterator]=function(){return o.l(),o.L(),this.D[Symbol.asyncIterator]()},o.h(Bc,Ec),o.h(bc,vc),(r=Qc.prototype).isSync=function(){return!1},r.isAsync=function(){return!1},r.isFile=function(){return!1},r.isStream=function(){return!1},r.reset=function(t){return this.ka=this.da=0,this.schema=t,this.dictionaries=new Map,this},r.bb=function(t,e,n){return new fc(e,t.U,t.buffers).visitMany(n)},o.global.Object.defineProperties(Qc.prototype,{numDictionaries:{configurable:!0,enumerable:!0,get:function(){return this.da}},numRecordBatches:{configurable:!0,enumerable:!0,get:function(){return this.ka}}}),o.l(),o.K(),o.h(_c,Qc),_c.prototype.isSync=function(){return!0},_c.prototype.isStream=function(){return!0},_c.prototype[Symbol.iterator]=function(){return this},(r=_c.prototype).cancel=function(){!this.closed&&(this.closed=!0)&&(this.reset().J.return(),this.dictionaries=this.J=null)},r.open=function(t){return this.closed||(this.autoDestroy=Lc(this,t),this.schema||(this.schema=this.J.readSchema())||this.cancel()),this},r.throw=function(t){return!this.closed&&this.autoDestroy&&(this.closed=!0)?this.reset().J.throw(t):yt},r.return=function(t){return!this.closed&&this.autoDestroy&&(this.closed=!0)?this.reset().J.return(t):yt},r.next=function(){if(this.closed)return yt;for(var t,e=this.J;t=this.Ia();)if(t.isSchema())this.reset(t.header());else{if(t.isRecordBatch()){this.ka++;var n=t.header();return{done:!1,value:wc(this,n,e=e.readMessageBody(t.bodyLength))}}t.isDictionaryBatch()&&(this.da++,n=t.header(),t=xc(this,n,t=e.readMessageBody(t.bodyLength)),this.dictionaries.set(n.id,t))}return this.return()},r.Ia=function(){return this.J.readMessage(void 0)},o.l(),o.L(),o.h(Dc,Qc),Dc.prototype.isAsync=function(){return!0},Dc.prototype.isStream=function(){return!0},Dc.prototype[Symbol.asyncIterator]=function(){return this},(r=Dc.prototype).cancel=function(){var t=this;return o.u(function(e){if(1==e.i)return t.closed||!(t.closed=!0)?e.j(0):l(e,t.reset().J.return(),3);t.J=null,t.dictionaries=null,e.i=0})},r.open=function(t){var e,n,r=this;return o.u(function(i){switch(i.i){case 1:if(r.closed){i.j(2);break}if(r.autoDestroy=Lc(r,t),e=r.schema){i.j(3);break}return n=r,l(i,r.J.readSchema(),4);case 4:e=n.schema=i.m;case 3:if(e){i.j(2);break}return l(i,r.cancel(),2);case 2:return i.return(r)}})},r.throw=function(t){var e=this;return o.u(function(n){return 1==n.i?!e.closed&&e.autoDestroy&&(e.closed=!0)?l(n,e.reset().J.throw(t),3):n.j(2):2!=n.i?n.return(n.m):n.return(yt)})},r.return=function(t){var e=this;return o.u(function(n){return 1==n.i?!e.closed&&e.autoDestroy&&(e.closed=!0)?l(n,e.reset().J.return(t),3):n.j(2):2!=n.i?n.return(n.m):n.return(yt)})},r.next=function(){var t,e,n,r,i,a,s,A,c=this;return o.u(function(o){switch(o.i){case 1:if(c.closed)return o.return(yt);e=c.J;case 2:return l(o,c.Ia(),4);case 4:if(!(t=o.m))return l(o,c.return(),13);if(t.isSchema())return l(o,c.reset(t.header()),2);if(t.isRecordBatch())return c.ka++,a=t.header(),l(o,e.readMessageBody(t.bodyLength),11);if(!t.isDictionaryBatch()){o.j(2);break}return c.da++,n=t.header(),l(o,e.readMessageBody(t.bodyLength),10);case 10:r=o.m,i=xc(c,n,r),c.dictionaries.set(n.id,i),o.j(2);break;case 11:return s=o.m,A=wc(c,a,s),o.return({done:!1,value:A});case 13:return o.return(o.m)}})},r.Ia=function(){var t=this;return o.u(function(e){return 1==e.i?l(e,t.J.readMessage(void 0),2):e.return(e.m)})},o.h(Mc,_c),(r=Mc.prototype).isSync=function(){return!0},r.isFile=function(){return!0},r.open=function(t){if(!this.closed&&!this.A){this.schema=(this.A=this.Eb()).schema;for(var e=o.w(To(this.A)),n=e.next();!n.done;n=e.next())n.value&&this.Db(this.da++)}return _c.prototype.open.call(this,t)},r.readRecordBatch=function(t){if(this.closed)return null;if(this.A||this.open(),(t=this.A&&this.A.wa(t))&&this.R.seek(t.offset)){var e=this.J.readMessage(Bn.RecordBatch);if(e&&e.isRecordBatch())return wc(this,t=e.header(),e=this.J.readMessageBody(e.bodyLength))}return null},r.Db=function(t){if((t=this.A&&this.A.hb(t))&&this.R.seek(t.offset)){var e=this.J.readMessage(Bn.DictionaryBatch);e&&e.isDictionaryBatch()&&(t=e.header(),e=xc(this,t,e=this.J.readMessageBody(e.bodyLength)),this.dictionaries.set(t.id,e))}},r.Eb=function(){var t=this.R,e=t.size-Kl,n=t.v(e);return t=t.ya(e-n,n),Fo.decode(t)},r.Ia=function(){if(this.A||this.open(),this.A&&this.ka<this.numRecordBatches){var t=this.A&&this.A.wa(this.ka);if(t&&this.R.seek(t.offset))return this.J.readMessage(void 0)}return null},o.global.Object.defineProperties(Mc.prototype,{footer:{configurable:!0,enumerable:!0,get:function(){return this.A}},numDictionaries:{configurable:!0,enumerable:!0,get:function(){return this.A?this.A.numDictionaries:0}},numRecordBatches:{configurable:!0,enumerable:!0,get:function(){return this.A?this.A.numRecordBatches:0}}}),o.h(kc,Dc),(r=kc.prototype).isFile=function(){return!0},r.isAsync=function(){return!0},r.open=function(t){var e,n,r,i,a=this;return o.u(function(s){switch(s.i){case 1:if(a.closed||a.A){s.j(2);break}return n=e=a,l(s,a.Eb(),3);case 3:e.schema=(n.A=s.m).schema,r=o.w(To(a.A)),i=r.next();case 4:if(i.done){s.j(2);break}if(!i.value){s.j(7);break}return l(s,a.Db(a.da++),8);case 8:case 7:i=r.next(),s.j(4);break;case 2:return l(s,Dc.prototype.open.call(a,t),9);case 9:return s.return(s.m)}})},r.readRecordBatch=function(t){var e,n,r,i,a,s,A=this;return o.u(function(o){switch(o.i){case 1:if(A.closed)return o.return(null);if(A.A){o.j(2);break}return l(o,A.open(),2);case 2:if(e=A.A&&A.A.wa(t),!(n=e)){o.j(4);break}return l(o,A.R.seek(e.offset),5);case 5:n=o.m;case 4:if(!n){o.j(6);break}return l(o,A.J.readMessage(Bn.RecordBatch),7);case 7:if(!(r=o.m)||!r.isRecordBatch()){o.j(6);break}return i=r.header(),l(o,A.J.readMessageBody(r.bodyLength),9);case 9:return a=o.m,s=wc(A,i,a),o.return(s);case 6:return o.return(null)}})},r.Db=function(t){var e,n,r,i,a,s,A=this;return o.u(function(o){switch(o.i){case 1:if(e=A.A&&A.A.hb(t),!(n=e)){o.j(2);break}return l(o,A.R.seek(e.offset),3);case 3:n=o.m;case 2:if(!n){o.j(0);break}return l(o,A.J.readMessage(Bn.DictionaryBatch),5);case 5:if(!(r=o.m)||!r.isDictionaryBatch()){o.j(0);break}return i=r.header(),l(o,A.J.readMessageBody(r.bodyLength),7);case 7:a=o.m,s=xc(A,i,a),A.dictionaries.set(i.id,s),o.i=0}})},r.Eb=function(){var t,e,n,r,i=this;return o.u(function(o){switch(o.i){case 1:if(!(t=i.R).ea){o.j(2);break}return l(o,t.ea,3);case 3:case 2:return e=t.size-Kl,l(o,t.v(e),4);case 4:return n=o.m,l(o,t.ya(e-n,n),5);case 5:return r=o.m,o.return(Fo.decode(r))}})},r.Ia=function(){var t,e,n=this;return o.u(function(r){switch(r.i){case 1:if(n.A){r.j(2);break}return l(r,n.open(),2);case 2:if(!(n.A&&n.ka<n.numRecordBatches)){r.j(4);break}if(t=n.A.wa(n.ka),!(e=t)){r.j(5);break}return l(r,n.R.seek(t.offset),6);case 6:e=r.m;case 5:if(!e){r.j(4);break}return l(r,n.J.readMessage(void 0),8);case 8:return r.return(r.m);case 4:return r.return(null)}})},o.global.Object.defineProperties(kc.prototype,{footer:{configurable:!0,enumerable:!0,get:function(){return this.A}},numDictionaries:{configurable:!0,enumerable:!0,get:function(){return this.A?this.A.numDictionaries:0}},numRecordBatches:{configurable:!0,enumerable:!0,get:function(){return this.A?this.A.numRecordBatches:0}}}),o.h(Sc,_c),Sc.prototype.bb=function(t,e,n){return new Cc(e,t.U,t.buffers).visitMany(n)},(r=Hc.prototype).eq=function(t){return t instanceof Hc||(t=new jc(t)),new Vc(this,t)},r.le=function(t){return t instanceof Hc||(t=new jc(t)),new Wc(this,t)},r.ge=function(t){return t instanceof Hc||(t=new jc(t)),new Xc(this,t)},r.lt=function(t){return new Zc(this.ge(t))},r.gt=function(t){return new Zc(this.le(t))},r.ne=function(t){return new Zc(this.eq(t))},o.h(jc,Hc),o.h(Yc,Hc),Yc.prototype.bind=function(t){if(!this.gb){this.gb=-1;for(var e=t.schema.fields,n=-1;++n<e.length;)if(e[n].name===this.name){this.gb=n;break}if(0>this.gb)throw Error('Failed to bind Col "'+this.name+'"')}var r=this.Aa=t.getChildAt(this.gb);return function(t){return r.get(t)}},Pc.prototype.and=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return new(Function.prototype.bind.apply(Jc,[null,this].concat(o.o(e))))},Pc.prototype.or=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return new(Function.prototype.bind.apply(qc,[null,this].concat(o.o(e))))},Pc.prototype.not=function(){return new Zc(this)},o.h(Kc,Pc),Kc.prototype.bind=function(t){return this.left instanceof jc?this.right instanceof jc?this.vb(this.left,this.right):this.ub(t,this.left,this.right):this.right instanceof jc?this.Wa(t,this.left,this.right):this.tb(t,this.left,this.right)},o.h(zc,Pc),zc.prototype.children=Object.freeze([]),o.h(Jc,zc),Jc.prototype.bind=function(t){var e=this.children.map(function(e){return e.bind(t)});return function(t,n){return e.every(function(e){return e(t,n)})}},o.h(qc,zc),qc.prototype.bind=function(t){var e=this.children.map(function(e){return e.bind(t)});return function(t,n){return e.some(function(e){return e(t,n)})}},o.h(Vc,Kc),Vc.prototype.vb=function(t,e){var n=t.W==e.W;return function(){return n}},Vc.prototype.tb=function(t,e,n){var r=e.bind(t),i=n.bind(t);return function(t,e){return r(t,e)==i(t,e)}},Vc.prototype.Wa=function(t,e,n){var r=e.bind(t);if(e.Aa instanceof ra){var i=e.Aa;if(i.dictionary!==this.Ce){var o=i.reverseLookup(n.W);this.Ce=i.dictionary,this.De=o}else o=this.De;return-1===o?function(){return!1}:function(t){return i.getKey(t)===o}}return function(t,e){return r(t,e)==n.W}},Vc.prototype.ub=function(t,e,n){return this.Wa(t,n,e)},o.h(Wc,Kc),Wc.prototype.vb=function(t,e){var n=t.W<=e.W;return function(){return n}},Wc.prototype.tb=function(t,e,n){var r=e.bind(t),i=n.bind(t);return function(t,e){return r(t,e)<=i(t,e)}},Wc.prototype.Wa=function(t,e,n){var r=e.bind(t);return function(t,e){return r(t,e)<=n.W}},Wc.prototype.ub=function(t,e,n){var r=n.bind(t);return function(t,n){return e.W<=r(t,n)}},o.h(Xc,Kc),Xc.prototype.vb=function(t,e){var n=t.W>=e.W;return function(){return n}},Xc.prototype.tb=function(t,e,n){var r=e.bind(t),i=n.bind(t);return function(t,e){return r(t,e)>=i(t,e)}},Xc.prototype.Wa=function(t,e,n){var r=e.bind(t);return function(t,e){return r(t,e)>=n.W}},Xc.prototype.ub=function(t,e,n){var r=n.bind(t);return function(t,n){return e.W>=r(t,n)}},o.h(Zc,Pc),Zc.prototype.bind=function(t){var e=this.Md.bind(t);return function(t,n){return!e(t,n)}},o.h($c,Pc),$c.prototype.bind=function(t){return this.Hd(t),this.next};var tu={};function eu(t){return hc.apply(this,arguments)||this}function nu(t,e){var n=new _o([new Mo("values",t.type),new Mo("counts",e.type)]);return hc.call(this,new dc(n,e.length,[t,e]))||this}function ru(t,e){return(t=eu.call(this,t)||this).Ha=e,t}tu.Value=Hc,tu.Literal=jc,tu.Col=Yc,tu.Predicate=Pc,tu.ComparisonPredicate=Kc,tu.CombinationPredicate=zc,tu.And=Jc,tu.Or=qc,tu.Equals=Vc,tu.LTeq=Wc,tu.GTeq=Xc,tu.Not=Zc,tu.CustomPredicate=$c,tu.lit=function(t){return new jc(t)},tu.col=function(t){return new Yc(t)},tu.and=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return new(Function.prototype.bind.apply(Jc,[null].concat(o.o(e))))},tu.or=function(t){for(var e=[],n=0;n<arguments.length;++n)e[n-0]=arguments[n];return new(Function.prototype.bind.apply(qc,[null].concat(o.o(e))))},tu.custom=function(t,e){return new $c(t,e)},hc.prototype.countBy=function(t){return new eu(this.chunks).countBy(t)},hc.prototype.scan=function(t,e){return new eu(this.chunks).scan(t,e)},hc.prototype.filter=function(t){return new eu(this.chunks).filter(t)},o.h(eu,hc),eu.prototype.filter=function(t){return new ru(this.chunks,t)},eu.prototype.scan=function(t,e){for(var n=this.chunks,r=n.length,i=-1;++i<r;){var o=n[i];e&&e(o);for(var a=-1,s=o.length;++a<s;)t(a,o)}},eu.prototype.countBy=function(t){var e=this.chunks,n=e.length;(t="string"==typeof t?new tu.Col(t):t).bind(e[n-1]);var r=t.Aa;if(!nr.isDictionary(r.type))throw Error("countBy currently only supports dictionary-encoded columns");var i=Math.ceil(Math.log(r.dictionary.length)/Math.log(256));i=new(4==i?Uint32Array:2<=i?Uint16Array:Uint8Array)(r.dictionary.length);for(var o=-1;++o<n;){var a=e[o];t.bind(a);var s=t.Aa.indices,A=-1;for(a=a.length;++A<a;){var l=s.get(A);null!==l&&i[l]++}}return new nu(r.dictionary,ha.from(i))},o.h(nu,hc),nu.prototype.toJSON=function(){for(var t=this.getColumnAt(0),e=this.getColumnAt(1),n={},r=-1;++r<this.length;)n[t.get(r)]=e.get(r);return n},o.l(),o.K(),o.h(ru,eu),ru.prototype.scan=function(t,e){for(var n=this.I,r=n.length,i=-1;++i<r;){var o=n[i];e&&e(o);for(var a=this.Ha.bind(o),s=-1,A=o.length;++s<A;)a(s,o)&&t(s,o)}},ru.prototype.count=function(){for(var t=0,e=this.I,n=e.length,r=-1;++r<n;)for(var i=e[r],o=this.Ha.bind(i),a=-1,s=i.length;++a<s;)o(a,i)&&++t;return t},ru.prototype[Symbol.iterator]=function t(){var e,n,r,i,a,s,A,c=this;return o.C.H(t,function(t){if(1==t.i&&(e=c.I,n=e.length,r=-1),5!=t.i){if(!(++r<n))return t.j(0);i=e[r],a=c.Ha.bind(i),s=-1,A=i.length}return++s<A?a(s,i)?l(t,i.get(s),5):t.j(5):t.j(2)})},ru.prototype.filter=function(t){return new ru(this.I,this.Ha.and(t))},ru.prototype.countBy=function(t){var e=this.I,n=e.length;(t="string"==typeof t?new tu.Col(t):t).bind(e[n-1]);var r=t.Aa;if(!nr.isDictionary(r.type))throw Error("countBy currently only supports dictionary-encoded columns");var i=Math.ceil(Math.log(r.dictionary.length)/Math.log(256));i=new(4==i?Uint32Array:2<=i?Uint16Array:Uint8Array)(r.dictionary.length);for(var o=-1;++o<n;){var a=e[o],s=this.Ha.bind(a);t.bind(a);for(var A=t.Aa.indices,l=-1,c=a.length;++l<c;){var u=A.get(l);null!==u&&s(l,a)&&i[u]++}}return new nu(r.dictionary,ha.from(i))};var iu=Object.assign({},Zo,Fn,Wt,ho),ou={get ArrowType(){return pn},get DateUnit(){return In},get IntervalUnit(){return vn},get MessageHeader(){return Bn},get MetadataVersion(){return bn},get Precision(){return mn},get TimeUnit(){return Cn},get Type(){return xn},get UnionMode(){return yn},get VectorType(){return En},get Data(){return Tn},get DataType(){return nr},get Null(){return ar},get Bool(){return Tr},get Int(){return cr},get Int8(){return dr},get Int16(){return fr},get Int32(){return pr},get Int64(){return Ir},get Uint8(){return Cr},get Uint16(){return mr},get Uint32(){return yr},get Uint64(){return Er},get Float(){return vr},get Float16(){return wr},get Float32(){return xr},get Float64(){return _r},get Utf8(){return Lr},get Binary(){return Dr},get FixedSizeBinary(){return Li},get Date_(){return Kr},get DateDay(){return Vr},get DateMillisecond(){return Wr},get Timestamp(){return oi},get TimestampSecond(){return li},get TimestampMillisecond(){return ci},get TimestampMicrosecond(){return ui},get TimestampNanosecond(){return gi},get Time(){return Xr},get TimeSecond(){return ei},get TimeMillisecond(){return ni},get TimeMicrosecond(){return ri},get TimeNanosecond(){return ii},get Decimal(){return Hr},get List(){return mi},get Struct(){return Bi},get Union(){return xi},get DenseUnion(){return ki},get SparseUnion(){return Si},get Dictionary(){return zi},get Interval(){return hi},get IntervalDayTime(){return Ii},get IntervalYearMonth(){return Ci},get FixedSizeList(){return Ti},get Map_(){return Hi},get Table(){return hc},get Column(){return Co},get Schema(){return _o},get Field(){return Mo},get Visitor(){return Gn},get cc(){return cA.cc},get Vector(){return cA.Vector},get BaseVector(){return cA.BaseVector},get BinaryVector(){return cA.BinaryVector},get BoolVector(){return cA.BoolVector},get Chunked(){return cA.Chunked},get DateVector(){return cA.DateVector},get DateDayVector(){return cA.DateDayVector},get DateMillisecondVector(){return cA.DateMillisecondVector},get DecimalVector(){return cA.DecimalVector},get DictionaryVector(){return cA.DictionaryVector},get FixedSizeBinaryVector(){return cA.FixedSizeBinaryVector},get FixedSizeListVector(){return cA.FixedSizeListVector},get FloatVector(){return cA.FloatVector},get Float16Vector(){return cA.Float16Vector},get Float32Vector(){return cA.Float32Vector},get Float64Vector(){return cA.Float64Vector},get IntervalVector(){return cA.IntervalVector},get IntervalDayTimeVector(){return cA.IntervalDayTimeVector},get IntervalYearMonthVector(){return cA.IntervalYearMonthVector},get IntVector(){return cA.IntVector},get Int8Vector(){return cA.Int8Vector},get Int16Vector(){return cA.Int16Vector},get Int32Vector(){return cA.Int32Vector},get Int64Vector(){return cA.Int64Vector},get Uint8Vector(){return cA.Uint8Vector},get Uint16Vector(){return cA.Uint16Vector},get Uint32Vector(){return cA.Uint32Vector},get Uint64Vector(){return cA.Uint64Vector},get ListVector(){return cA.ListVector},get MapVector(){return cA.MapVector},get NullVector(){return cA.NullVector},get StructVector(){return cA.StructVector},get TimestampVector(){return cA.TimestampVector},get TimestampSecondVector(){return cA.TimestampSecondVector},get TimestampMillisecondVector(){return cA.TimestampMillisecondVector},get TimestampMicrosecondVector(){return cA.TimestampMicrosecondVector},get TimestampNanosecondVector(){return cA.TimestampNanosecondVector},get TimeVector(){return cA.TimeVector},get TimeSecondVector(){return cA.TimeSecondVector},get TimeMillisecondVector(){return cA.TimeMillisecondVector},get TimeMicrosecondVector(){return cA.TimeMicrosecondVector},get TimeNanosecondVector(){return cA.TimeNanosecondVector},get UnionVector(){return cA.UnionVector},get DenseUnionVector(){return cA.DenseUnionVector},get SparseUnionVector(){return cA.SparseUnionVector},get Utf8Vector(){return cA.Utf8Vector},get ByteStream(){return _l},get AsyncByteStream(){return Dl},get AsyncByteQueue(){return xl},get RecordBatchReader(){return yc},get RecordBatchFileReader(){return Bc},get RecordBatchStreamReader(){return Ec},get AsyncRecordBatchFileReader(){return bc},get AsyncRecordBatchStreamReader(){return vc},get RecordBatchWriter(){return rc},get RecordBatchFileWriter(){return Ac},get RecordBatchStreamWriter(){return sc},get RecordBatchJSONWriter(){return lc},get MessageReader(){return Ul},get AsyncMessageReader(){return Ol},get JSONMessageReader(){return Gl},get Message(){return nl},get RecordBatch(){return dc},get DataFrame(){return eu},get FilteredDataFrame(){return ru},get CountByResult(){return nu},get predicate(){return tu}};Xt=function(t,e){if(Tt(t))return Gc(t,e);if(Ft(t))return Oc(t,e);throw Error("toDOMStream() must be called with an Iterable or AsyncIterable")},yc.throughDOM=function(t,e){function n(){var t;return o.u(function(n){return 1==n.i?(t=e,l(n,yc.from(i),3)):2!=n.i?l(n,n.m.open(t),2):n.return(n.m)})}function r(t,e){var n,r;return o.u(function(i){switch(i.i){case 1:n=t.desiredSize,r=null;case 2:return l(i,e.next(),4);case 4:if((r=i.m).done){i.j(3);break}if(t.enqueue(r.value),null!=n&&0>=--n)return i.return();i.j(2);break;case 3:t.close(),i.i=0}})}var i=new xl,a=null,s=new ReadableStream({cancel:function(){return o.u(function(t){return l(t,i.close(),0)})},start:function(t){var e,i,s;return o.u(function(o){return 1==o.i?(e=r,i=t,(s=a)?o.j(3):l(o,n(),4)):(3!=o.i&&(s=a=o.m),l(o,e(i,s),0))})},pull:function(t){return o.u(function(e){if(1==e.i)return a?l(e,r(t,a),3):(t.close(),e.j(2));e.i=0})}});return{writable:new WritableStream(i,Object.assign({},{highWaterMark:Math.pow(2,14)},t)),readable:s}},rc.throughDOM=function(t,e){function n(t){var e,n;return o.u(function(r){switch(r.i){case 1:e=null,n=t.desiredSize;case 2:return l(r,i.read(n||null),4);case 4:if(!(e=r.m)){r.j(3);break}if(t.enqueue(e),null!=n&&0>=(n-=e.byteLength))return r.return();r.j(2);break;case 3:t.close(),r.i=0}})}var r=new this(t),i=new Dl(r);return e=new ReadableStream({type:"bytes",cancel:function(){return o.u(function(t){return l(t,i.cancel(),0)})},pull:function(t){return o.u(function(e){return l(e,n(t),0)})},start:function(t){return o.u(function(e){return l(e,n(t),0)})}},Object.assign({},{highWaterMark:Math.pow(2,14)},e)),{writable:new WritableStream(r,t),readable:e}},Object.assign(arguments[0],{get ArrowType(){return ou.ArrowType},get DateUnit(){return ou.DateUnit},get IntervalUnit(){return ou.IntervalUnit},get MessageHeader(){return ou.MessageHeader},get MetadataVersion(){return ou.MetadataVersion},get Precision(){return ou.Precision},get TimeUnit(){return ou.TimeUnit},get Type(){return ou.Type},get UnionMode(){return ou.UnionMode},get VectorType(){return ou.VectorType},get Data(){return ou.Data},get DataType(){return ou.DataType},get Null(){return ou.Null},get Bool(){return ou.Bool},get Int(){return ou.Int},get Int8(){return ou.Int8},get Int16(){return ou.Int16},get Int32(){return ou.Int32},get Int64(){return ou.Int64},get Uint8(){return ou.Uint8},get Uint16(){return ou.Uint16},get Uint32(){return ou.Uint32},get Uint64(){return ou.Uint64},get Float(){return ou.Float},get Float16(){return ou.Float16},get Float32(){return ou.Float32},get Float64(){return ou.Float64},get Utf8(){return ou.Utf8},get Binary(){return ou.Binary},get FixedSizeBinary(){return ou.FixedSizeBinary},get Date_(){return ou.Date_},get DateDay(){return ou.DateDay},get DateMillisecond(){return ou.DateMillisecond},get Timestamp(){return ou.Timestamp},get TimestampSecond(){return ou.TimestampSecond},get TimestampMillisecond(){return ou.TimestampMillisecond},get TimestampMicrosecond(){return ou.TimestampMicrosecond},get TimestampNanosecond(){return ou.TimestampNanosecond},get Time(){return ou.Time},get TimeSecond(){return ou.TimeSecond},get TimeMillisecond(){return ou.TimeMillisecond},get TimeMicrosecond(){return ou.TimeMicrosecond},get TimeNanosecond(){return ou.TimeNanosecond},get Decimal(){return ou.Decimal},get List(){return ou.List},get Struct(){return ou.Struct},get Union(){return ou.Union},get DenseUnion(){return ou.DenseUnion},get SparseUnion(){return ou.SparseUnion},get Dictionary(){return ou.Dictionary},get Interval(){return ou.Interval},get IntervalDayTime(){return ou.IntervalDayTime},get IntervalYearMonth(){return ou.IntervalYearMonth},get FixedSizeList(){return ou.FixedSizeList},get Map_(){return ou.Map_},get Table(){return ou.Table},get Column(){return ou.Column},get Schema(){return ou.Schema},get Field(){return ou.Field},get Visitor(){return ou.Visitor},get Vector(){return ou.Vector},get BaseVector(){return ou.BaseVector},get BinaryVector(){return ou.BinaryVector},get BoolVector(){return ou.BoolVector},get Chunked(){return ou.Chunked},get DateVector(){return ou.DateVector},get DateDayVector(){return ou.DateDayVector},get DateMillisecondVector(){return ou.DateMillisecondVector},get DecimalVector(){return ou.DecimalVector},get DictionaryVector(){return ou.DictionaryVector},get FixedSizeBinaryVector(){return ou.FixedSizeBinaryVector},get FixedSizeListVector(){return ou.FixedSizeListVector},get FloatVector(){return ou.FloatVector},get Float16Vector(){return ou.Float16Vector},get Float32Vector(){return ou.Float32Vector},get Float64Vector(){return ou.Float64Vector},get IntervalVector(){return ou.IntervalVector},get IntervalDayTimeVector(){return ou.IntervalDayTimeVector},get IntervalYearMonthVector(){return ou.IntervalYearMonthVector},get IntVector(){return ou.IntVector},get Int8Vector(){return ou.Int8Vector},get Int16Vector(){return ou.Int16Vector},get Int32Vector(){return ou.Int32Vector},get Int64Vector(){return ou.Int64Vector},get Uint8Vector(){return ou.Uint8Vector},get Uint16Vector(){return ou.Uint16Vector},get Uint32Vector(){return ou.Uint32Vector},get Uint64Vector(){return ou.Uint64Vector},get ListVector(){return ou.ListVector},get MapVector(){return ou.MapVector},get NullVector(){return ou.NullVector},get StructVector(){return ou.StructVector},get TimestampVector(){return ou.TimestampVector},get TimestampSecondVector(){return ou.TimestampSecondVector},get TimestampMillisecondVector(){return ou.TimestampMillisecondVector},get TimestampMicrosecondVector(){return ou.TimestampMicrosecondVector},get TimestampNanosecondVector(){return ou.TimestampNanosecondVector},get TimeVector(){return ou.TimeVector},get TimeSecondVector(){return ou.TimeSecondVector},get TimeMillisecondVector(){return ou.TimeMillisecondVector},get TimeMicrosecondVector(){return ou.TimeMicrosecondVector},get TimeNanosecondVector(){return ou.TimeNanosecondVector},get UnionVector(){return ou.UnionVector},get DenseUnionVector(){return ou.DenseUnionVector},get SparseUnionVector(){return ou.SparseUnionVector},get Utf8Vector(){return ou.Utf8Vector},get ByteStream(){return ou.ByteStream},get AsyncByteStream(){return ou.AsyncByteStream},get AsyncByteQueue(){return ou.AsyncByteQueue},get RecordBatchReader(){return ou.RecordBatchReader},get RecordBatchFileReader(){return ou.RecordBatchFileReader},get RecordBatchStreamReader(){return ou.RecordBatchStreamReader},get AsyncRecordBatchFileReader(){return ou.AsyncRecordBatchFileReader},get AsyncRecordBatchStreamReader(){return ou.AsyncRecordBatchStreamReader},get RecordBatchWriter(){return ou.RecordBatchWriter},get RecordBatchFileWriter(){return ou.RecordBatchFileWriter},get RecordBatchStreamWriter(){return ou.RecordBatchStreamWriter},get RecordBatchJSONWriter(){return ou.RecordBatchJSONWriter},get MessageReader(){return ou.MessageReader},get AsyncMessageReader(){return ou.AsyncMessageReader},get JSONMessageReader(){return ou.JSONMessageReader},get Message(){return ou.Message},get RecordBatch(){return ou.RecordBatch},get DataFrame(){return ou.DataFrame},get FilteredDataFrame(){return ou.FilteredDataFrame},get CountByResult(){return ou.CountByResult},get predicate(){return ou.predicate},get util(){return iu}})}).bind(this)(e)}).call(this,n(37),n(42).Buffer)},function(t,e,n){var r=n(109),i=n(71),o=n(145);t.exports=function(t,e){var n=(i.Object||{})[t]||Object[t],a={};a[t]=e(n),r(r.S+r.F*o(function(){n(1)}),"Object",a)}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(689);e.DisplayHandle=r.DisplayHandle;var i=n(1330);e.DisplayHandleManager=i.DisplayHandleManager},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(126),i=n(6),o=n(113);class a extends i.Widget{constructor({name:t,registry:e,onUpdated:n,node:r,sanitizer:a}){super({node:r}),this.layout=new i.BoxLayout,this.registry=e,this.name=t,this.sanitizer=a,n.pipe(o.filter(e=>e.name===t),o.takeWhile(()=>!this.isDisposed)).subscribe(t=>this.render(t))}async render(t){for(const t of this.layout.widgets)t.dispose();const e=this.registry.preferredMimeType(t.data||{},"any");if(null==e)return Promise.reject("Cannot figure out how to render display data");const n=this.registry.createRenderer(e),o=new r.MimeModel({data:t.data||void 0,metadata:t.metadata||void 0,trusted:!0});try{await n.renderModel(o),this.layout.addWidget(n)}catch(t){console.warn("Could not render handle"),console.warn(t);const e=new i.Widget;await r.renderText({host:e.node,source:t,sanitizer:this.sanitizer}),this.layout.addWidget(e)}}}e.DisplayHandle=a,function(t){t.MIME_TYPE="application/vnd.maven.display-handle+json"}(a=e.DisplayHandle||(e.DisplayHandle={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(9),o=n(235);e.KernelExpressionEvaluator=class{constructor({session:t}){this._globals=null,this._isDisposed=!1,this.session=t,this.comm=o.CommManager.Create({session:t,commName:"expression_evaluator"})}get globals(){return this._globals}set globals(t){this._globals=t}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this.comm.dispose(),delete this.session,delete this.globals,this._isDisposed=!0)}async evaluate(t,e){const n={},a=i.UUID.uuid4();if(null==this.globals)throw Error("Could not access globals: KernelExpressionEvaluator incompletely setup");for(const t of e){if(!this.globals.has(t)){console.warn("Unknown global referenced: ",t);continue}const e=r.Converters.serialize(this.globals.get(t),this.globals.getType(t));n[t]=null==e?{typeName:"Any",value:null}:e}const s=await this.comm.sendAndAwaitResponse({msg_type:"evaluate_expr",globals:n,expr:t||"",uuid:a},t=>t.parent===a,2e4),A=r.Converters.deserialize(s.payload);if("expr_error"===s.msg_type)throw await o.KernelError.Create(A,this.session.kernelDisplayName);return A}getMetadata(){return{editorMode:""+this.session.kernel.info.language_info.mimetype}}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(48),o=n(9),a=n(113),s=n(235);e.KernelProxyPart=class extends i.Part{constructor(t){if(super(t),this.bag=null,this.model=null,this.renderer=null,this.msgId=this.context.dashboardId+" "+this.uuid,this.type="",null==this.context.session)throw Error("Kernel not found!");if(null==this.context.rendermime)throw Error("Rendermime registry not found!");this.comm=s.CommManager.Create({commName:"kernel_proxy_part",session:this.context.session})}getName(){return this.type}async setup(){const t=this.msgId;await this.comm.send({uuid:t,msg_type:"create",payload:this.type})}async initialize(){await this.setup();const t=this.msgId;this.comm.msgRecieved.pipe(a.filter(e=>"stale"===e.msg_type&&e.uuid===t)).subscribe(t=>{null!=this.bag&&this.bag.set(t.payload.name,r.Converters.deserialize(t.payload.value))});const e=await this.comm.sendAndAwaitResponse({uuid:t,msg_type:"initialize",payload:null},e=>"initialize_done"===e.msg_type&&e.uuid===t);if(e.error)throw await s.KernelError.Create(e.error,this.context.session.kernelDisplayName)}async render(t){this.bag=t;const e=this.msgId;for(;this.layout.widgets.length>1;)this.layout.removeWidgetAt(1);const n={};for(const e of t)n[e.name]=r.Converters.serialize(e.value,e.type);const i=await this.comm.sendAndAwaitResponse({uuid:e,msg_type:"render",payload:n},t=>"render_done"===t.msg_type&&t.uuid===e);if(i.error)throw await s.KernelError.Create(i.error,this.context.session.kernelDisplayName);const a=this.context.rendermime.preferredMimeType(i.payload.data,"any")||"text/plain";this.model&&this.renderer&&o.JSONExt.deepEqual(this.model.data[a],i.payload.data[a])&&o.JSONExt.deepEqual(this.model.metadata[a],i.payload.metadata[a])||(this.renderer&&this.renderer.dispose(),this.renderer=this.context.rendermime.createRenderer(a),this.model=this.context.rendermime.createModel({data:i.payload.data,metadata:i.payload.metadata,trusted:!0}),await this.renderer.renderModel(this.model),this.layout.insertWidget(0,this.renderer))}dispose(){this.isDisposed||(!this.comm.isDisposed&&this.comm.isOpen&&this.comm.send({msg_type:"dispose",uuid:this.msgId,payload:null}),this.comm.dispose(),super.dispose())}}},function(t,e){t.exports=function(t,e){return t===e||t!=t&&e!=e}},function(t,e,n){var r=n(182),i=n(695),o="[object AsyncFunction]",a="[object Function]",s="[object GeneratorFunction]",A="[object Proxy]";t.exports=function(t){if(!i(t))return!1;var e=r(t);return e==a||e==s||e==o||e==A}},function(t,e,n){(function(e){var n="object"==typeof e&&e&&e.Object===Object&&e;t.exports=n}).call(this,n(37))},function(t,e){t.exports=function(t){var e=typeof t;return null!=t&&("object"==e||"function"==e)}},function(t,e){var n=Function.prototype.toString;t.exports=function(t){if(null!=t){try{return n.call(t)}catch(t){}try{return t+""}catch(t){}}return""}},function(t,e,n){var r=n(1357),i=n(1364),o=n(1366),a=n(1367),s=n(1368);function A(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}A.prototype.clear=r,A.prototype.delete=i,A.prototype.get=o,A.prototype.has=a,A.prototype.set=s,t.exports=A},function(t,e,n){var r=n(1369),i=n(1372),o=n(1373),a=1,s=2;t.exports=function(t,e,n,A,l,c){var u=n&a,g=t.length,h=e.length;if(g!=h&&!(u&&h>g))return!1;var d=c.get(t);if(d&&c.get(e))return d==e;var f=-1,p=!0,I=n&s?new r:void 0;for(c.set(t,e),c.set(e,t);++f<g;){var C=t[f],m=e[f];if(A)var y=u?A(m,C,f,e,t,c):A(C,m,f,t,e,c);if(void 0!==y){if(y)continue;p=!1;break}if(I){if(!i(e,function(t,e){if(!o(I,e)&&(C===t||l(C,t,n,A,c)))return I.push(e)})){p=!1;break}}else if(C!==m&&!l(C,m,n,A,c)){p=!1;break}}return c.delete(t),c.delete(e),p}},function(t,e,n){(function(t){var r=n(99),i=n(1390),o=e&&!e.nodeType&&e,a=o&&"object"==typeof t&&t&&!t.nodeType&&t,s=a&&a.exports===o?r.Buffer:void 0,A=(s?s.isBuffer:void 0)||i;t.exports=A}).call(this,n(98)(t))},function(t,e,n){var r=n(1392),i=n(1393),o=n(1394),a=o&&o.isTypedArray,s=a?i(a):r;t.exports=s},function(t,e){var n=9007199254740991;t.exports=function(t){return"number"==typeof t&&t>-1&&t%1==0&&t<=n}},function(t,e){t.exports=function(t,e){return function(n){return t(e(n))}}},function(t,e,n){var r,i,o;
/*!
 * jQuery UI Widget 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */i=[n(44),n(184)],void 0===(o="function"==typeof(r=function(t){var e,n=0,r=Array.prototype.slice;return t.cleanData=(e=t.cleanData,function(n){var r,i,o;for(o=0;null!=(i=n[o]);o++)try{(r=t._data(i,"events"))&&r.remove&&t(i).triggerHandler("remove")}catch(t){}e(n)}),t.widget=function(e,n,r){var i,o,a,s={},A=e.split(".")[0],l=A+"-"+(e=e.split(".")[1]);return r||(r=n,n=t.Widget),t.isArray(r)&&(r=t.extend.apply(null,[{}].concat(r))),t.expr[":"][l.toLowerCase()]=function(e){return!!t.data(e,l)},t[A]=t[A]||{},i=t[A][e],o=t[A][e]=function(t,e){if(!this._createWidget)return new o(t,e);arguments.length&&this._createWidget(t,e)},t.extend(o,i,{version:r.version,_proto:t.extend({},r),_childConstructors:[]}),(a=new n).options=t.widget.extend({},a.options),t.each(r,function(e,r){t.isFunction(r)?s[e]=function(){function t(){return n.prototype[e].apply(this,arguments)}function i(t){return n.prototype[e].apply(this,t)}return function(){var e,n=this._super,o=this._superApply;return this._super=t,this._superApply=i,e=r.apply(this,arguments),this._super=n,this._superApply=o,e}}():s[e]=r}),o.prototype=t.widget.extend(a,{widgetEventPrefix:i&&a.widgetEventPrefix||e},s,{constructor:o,namespace:A,widgetName:e,widgetFullName:l}),i?(t.each(i._childConstructors,function(e,n){var r=n.prototype;t.widget(r.namespace+"."+r.widgetName,o,n._proto)}),delete i._childConstructors):n._childConstructors.push(o),t.widget.bridge(e,o),o},t.widget.extend=function(e){for(var n,i,o=r.call(arguments,1),a=0,s=o.length;a<s;a++)for(n in o[a])i=o[a][n],o[a].hasOwnProperty(n)&&void 0!==i&&(t.isPlainObject(i)?e[n]=t.isPlainObject(e[n])?t.widget.extend({},e[n],i):t.widget.extend({},i):e[n]=i);return e},t.widget.bridge=function(e,n){var i=n.prototype.widgetFullName||e;t.fn[e]=function(o){var a="string"==typeof o,s=r.call(arguments,1),A=this;return a?this.length||"instance"!==o?this.each(function(){var n,r=t.data(this,i);return"instance"===o?(A=r,!1):r?t.isFunction(r[o])&&"_"!==o.charAt(0)?(n=r[o].apply(r,s))!==r&&void 0!==n?(A=n&&n.jquery?A.pushStack(n.get()):n,!1):void 0:t.error("no such method '"+o+"' for "+e+" widget instance"):t.error("cannot call methods on "+e+" prior to initialization; attempted to call method '"+o+"'")}):A=void 0:(s.length&&(o=t.widget.extend.apply(null,[o].concat(s))),this.each(function(){var e=t.data(this,i);e?(e.option(o||{}),e._init&&e._init()):t.data(this,i,new n(o,this))})),A}},t.Widget=function(){},t.Widget._childConstructors=[],t.Widget.prototype={widgetName:"widget",widgetEventPrefix:"",defaultElement:"<div>",options:{classes:{},disabled:!1,create:null},_createWidget:function(e,r){r=t(r||this.defaultElement||this)[0],this.element=t(r),this.uuid=n++,this.eventNamespace="."+this.widgetName+this.uuid,this.bindings=t(),this.hoverable=t(),this.focusable=t(),this.classesElementLookup={},r!==this&&(t.data(r,this.widgetFullName,this),this._on(!0,this.element,{remove:function(t){t.target===r&&this.destroy()}}),this.document=t(r.style?r.ownerDocument:r.document||r),this.window=t(this.document[0].defaultView||this.document[0].parentWindow)),this.options=t.widget.extend({},this.options,this._getCreateOptions(),e),this._create(),this.options.disabled&&this._setOptionDisabled(this.options.disabled),this._trigger("create",null,this._getCreateEventData()),this._init()},_getCreateOptions:function(){return{}},_getCreateEventData:t.noop,_create:t.noop,_init:t.noop,destroy:function(){var e=this;this._destroy(),t.each(this.classesElementLookup,function(t,n){e._removeClass(n,t)}),this.element.off(this.eventNamespace).removeData(this.widgetFullName),this.widget().off(this.eventNamespace).removeAttr("aria-disabled"),this.bindings.off(this.eventNamespace)},_destroy:t.noop,widget:function(){return this.element},option:function(e,n){var r,i,o,a=e;if(0===arguments.length)return t.widget.extend({},this.options);if("string"==typeof e)if(a={},r=e.split("."),e=r.shift(),r.length){for(i=a[e]=t.widget.extend({},this.options[e]),o=0;o<r.length-1;o++)i[r[o]]=i[r[o]]||{},i=i[r[o]];if(e=r.pop(),1===arguments.length)return void 0===i[e]?null:i[e];i[e]=n}else{if(1===arguments.length)return void 0===this.options[e]?null:this.options[e];a[e]=n}return this._setOptions(a),this},_setOptions:function(t){var e;for(e in t)this._setOption(e,t[e]);return this},_setOption:function(t,e){return"classes"===t&&this._setOptionClasses(e),this.options[t]=e,"disabled"===t&&this._setOptionDisabled(e),this},_setOptionClasses:function(e){var n,r,i;for(n in e)i=this.classesElementLookup[n],e[n]!==this.options.classes[n]&&i&&i.length&&(r=t(i.get()),this._removeClass(i,n),r.addClass(this._classes({element:r,keys:n,classes:e,add:!0})))},_setOptionDisabled:function(t){this._toggleClass(this.widget(),this.widgetFullName+"-disabled",null,!!t),t&&(this._removeClass(this.hoverable,null,"ui-state-hover"),this._removeClass(this.focusable,null,"ui-state-focus"))},enable:function(){return this._setOptions({disabled:!1})},disable:function(){return this._setOptions({disabled:!0})},_classes:function(e){var n=[],r=this;function i(i,o){var a,s;for(s=0;s<i.length;s++)a=r.classesElementLookup[i[s]]||t(),a=e.add?t(t.unique(a.get().concat(e.element.get()))):t(a.not(e.element).get()),r.classesElementLookup[i[s]]=a,n.push(i[s]),o&&e.classes[i[s]]&&n.push(e.classes[i[s]])}return e=t.extend({element:this.element,classes:this.options.classes||{}},e),this._on(e.element,{remove:"_untrackClassesElement"}),e.keys&&i(e.keys.match(/\S+/g)||[],!0),e.extra&&i(e.extra.match(/\S+/g)||[]),n.join(" ")},_untrackClassesElement:function(e){var n=this;t.each(n.classesElementLookup,function(r,i){-1!==t.inArray(e.target,i)&&(n.classesElementLookup[r]=t(i.not(e.target).get()))})},_removeClass:function(t,e,n){return this._toggleClass(t,e,n,!1)},_addClass:function(t,e,n){return this._toggleClass(t,e,n,!0)},_toggleClass:function(t,e,n,r){r="boolean"==typeof r?r:n;var i="string"==typeof t||null===t,o={extra:i?e:n,keys:i?t:e,element:i?this.element:t,add:r};return o.element.toggleClass(this._classes(o),r),this},_on:function(e,n,r){var i,o=this;"boolean"!=typeof e&&(r=n,n=e,e=!1),r?(n=i=t(n),this.bindings=this.bindings.add(n)):(r=n,n=this.element,i=this.widget()),t.each(r,function(r,a){function s(){if(e||!0!==o.options.disabled&&!t(this).hasClass("ui-state-disabled"))return("string"==typeof a?o[a]:a).apply(o,arguments)}"string"!=typeof a&&(s.guid=a.guid=a.guid||s.guid||t.guid++);var A=r.match(/^([\w:-]*)\s*(.*)$/),l=A[1]+o.eventNamespace,c=A[2];c?i.on(l,c,s):n.on(l,s)})},_off:function(e,n){n=(n||"").split(" ").join(this.eventNamespace+" ")+this.eventNamespace,e.off(n).off(n),this.bindings=t(this.bindings.not(e).get()),this.focusable=t(this.focusable.not(e).get()),this.hoverable=t(this.hoverable.not(e).get())},_delay:function(t,e){var n=this;return setTimeout(function(){return("string"==typeof t?n[t]:t).apply(n,arguments)},e||0)},_hoverable:function(e){this.hoverable=this.hoverable.add(e),this._on(e,{mouseenter:function(e){this._addClass(t(e.currentTarget),null,"ui-state-hover")},mouseleave:function(e){this._removeClass(t(e.currentTarget),null,"ui-state-hover")}})},_focusable:function(e){this.focusable=this.focusable.add(e),this._on(e,{focusin:function(e){this._addClass(t(e.currentTarget),null,"ui-state-focus")},focusout:function(e){this._removeClass(t(e.currentTarget),null,"ui-state-focus")}})},_trigger:function(e,n,r){var i,o,a=this.options[e];if(r=r||{},(n=t.Event(n)).type=(e===this.widgetEventPrefix?e:this.widgetEventPrefix+e).toLowerCase(),n.target=this.element[0],o=n.originalEvent)for(i in o)i in n||(n[i]=o[i]);return this.element.trigger(n,r),!(t.isFunction(a)&&!1===a.apply(this.element[0],[n].concat(r))||n.isDefaultPrevented())}},t.each({show:"fadeIn",hide:"fadeOut"},function(e,n){t.Widget.prototype["_"+e]=function(r,i,o){var a;"string"==typeof i&&(i={effect:i});var s=i?!0===i||"number"==typeof i?n:i.effect||n:e;"number"==typeof(i=i||{})&&(i={duration:i}),a=!t.isEmptyObject(i),i.complete=o,i.delay&&r.delay(i.delay),a&&t.effects&&t.effects.effect[s]?r[e](i):s!==e&&r[s]?r[s](i.duration,i.easing,o):r.queue(function(n){t(this)[e](),o&&o.call(r[0]),n()})}}),t.widget})?r.apply(e,i):r)||(t.exports=o)},function(t,e,n){var r;!function(i){var o=/^\s+/,a=/\s+$/,s=0,A=i.round,l=i.min,c=i.max,u=i.random;function g(t,e){if(e=e||{},(t=t||"")instanceof g)return t;if(!(this instanceof g))return new g(t,e);var n=function(t){var e={r:0,g:0,b:0},n=1,r=null,s=null,A=null,u=!1,g=!1;"string"==typeof t&&(t=function(t){t=t.replace(o,"").replace(a,"").toLowerCase();var e,n=!1;if(M[t])t=M[t],n=!0;else if("transparent"==t)return{r:0,g:0,b:0,a:0,format:"name"};if(e=Y.rgb.exec(t))return{r:e[1],g:e[2],b:e[3]};if(e=Y.rgba.exec(t))return{r:e[1],g:e[2],b:e[3],a:e[4]};if(e=Y.hsl.exec(t))return{h:e[1],s:e[2],l:e[3]};if(e=Y.hsla.exec(t))return{h:e[1],s:e[2],l:e[3],a:e[4]};if(e=Y.hsv.exec(t))return{h:e[1],s:e[2],v:e[3]};if(e=Y.hsva.exec(t))return{h:e[1],s:e[2],v:e[3],a:e[4]};if(e=Y.hex8.exec(t))return{r:N(e[1]),g:N(e[2]),b:N(e[3]),a:O(e[4]),format:n?"name":"hex8"};if(e=Y.hex6.exec(t))return{r:N(e[1]),g:N(e[2]),b:N(e[3]),format:n?"name":"hex"};if(e=Y.hex4.exec(t))return{r:N(e[1]+""+e[1]),g:N(e[2]+""+e[2]),b:N(e[3]+""+e[3]),a:O(e[4]+""+e[4]),format:n?"name":"hex8"};if(e=Y.hex3.exec(t))return{r:N(e[1]+""+e[1]),g:N(e[2]+""+e[2]),b:N(e[3]+""+e[3]),format:n?"name":"hex"};return!1}(t));"object"==typeof t&&(P(t.r)&&P(t.g)&&P(t.b)?(h=t.r,d=t.g,f=t.b,e={r:255*L(h,255),g:255*L(d,255),b:255*L(f,255)},u=!0,g="%"===String(t.r).substr(-1)?"prgb":"rgb"):P(t.h)&&P(t.s)&&P(t.v)?(r=T(t.s),s=T(t.v),e=function(t,e,n){t=6*L(t,360),e=L(e,100),n=L(n,100);var r=i.floor(t),o=t-r,a=n*(1-e),s=n*(1-o*e),A=n*(1-(1-o)*e),l=r%6;return{r:255*[n,s,a,a,A,n][l],g:255*[A,n,n,s,a,a][l],b:255*[a,a,A,n,n,s][l]}}(t.h,r,s),u=!0,g="hsv"):P(t.h)&&P(t.s)&&P(t.l)&&(r=T(t.s),A=T(t.l),e=function(t,e,n){var r,i,o;function a(t,e,n){return n<0&&(n+=1),n>1&&(n-=1),n<1/6?t+6*(e-t)*n:n<.5?e:n<2/3?t+(e-t)*(2/3-n)*6:t}if(t=L(t,360),e=L(e,100),n=L(n,100),0===e)r=i=o=n;else{var s=n<.5?n*(1+e):n+e-n*e,A=2*n-s;r=a(A,s,t+1/3),i=a(A,s,t),o=a(A,s,t-1/3)}return{r:255*r,g:255*i,b:255*o}}(t.h,r,A),u=!0,g="hsl"),t.hasOwnProperty("a")&&(n=t.a));var h,d,f;return n=S(n),{ok:u,format:t.format||g,r:l(255,c(e.r,0)),g:l(255,c(e.g,0)),b:l(255,c(e.b,0)),a:n}}(t);this._originalInput=t,this._r=n.r,this._g=n.g,this._b=n.b,this._a=n.a,this._roundA=A(100*this._a)/100,this._format=e.format||n.format,this._gradientType=e.gradientType,this._r<1&&(this._r=A(this._r)),this._g<1&&(this._g=A(this._g)),this._b<1&&(this._b=A(this._b)),this._ok=n.ok,this._tc_id=s++}function h(t,e,n){t=L(t,255),e=L(e,255),n=L(n,255);var r,i,o=c(t,e,n),a=l(t,e,n),s=(o+a)/2;if(o==a)r=i=0;else{var A=o-a;switch(i=s>.5?A/(2-o-a):A/(o+a),o){case t:r=(e-n)/A+(e<n?6:0);break;case e:r=(n-t)/A+2;break;case n:r=(t-e)/A+4}r/=6}return{h:r,s:i,l:s}}function d(t,e,n){t=L(t,255),e=L(e,255),n=L(n,255);var r,i,o=c(t,e,n),a=l(t,e,n),s=o,A=o-a;if(i=0===o?0:A/o,o==a)r=0;else{switch(o){case t:r=(e-n)/A+(e<n?6:0);break;case e:r=(n-t)/A+2;break;case n:r=(t-e)/A+4}r/=6}return{h:r,s:i,v:s}}function f(t,e,n,r){var i=[F(A(t).toString(16)),F(A(e).toString(16)),F(A(n).toString(16))];return r&&i[0].charAt(0)==i[0].charAt(1)&&i[1].charAt(0)==i[1].charAt(1)&&i[2].charAt(0)==i[2].charAt(1)?i[0].charAt(0)+i[1].charAt(0)+i[2].charAt(0):i.join("")}function p(t,e,n,r){return[F(U(r)),F(A(t).toString(16)),F(A(e).toString(16)),F(A(n).toString(16))].join("")}function I(t,e){e=0===e?0:e||10;var n=g(t).toHsl();return n.s-=e/100,n.s=R(n.s),g(n)}function C(t,e){e=0===e?0:e||10;var n=g(t).toHsl();return n.s+=e/100,n.s=R(n.s),g(n)}function m(t){return g(t).desaturate(100)}function y(t,e){e=0===e?0:e||10;var n=g(t).toHsl();return n.l+=e/100,n.l=R(n.l),g(n)}function E(t,e){e=0===e?0:e||10;var n=g(t).toRgb();return n.r=c(0,l(255,n.r-A(-e/100*255))),n.g=c(0,l(255,n.g-A(-e/100*255))),n.b=c(0,l(255,n.b-A(-e/100*255))),g(n)}function v(t,e){e=0===e?0:e||10;var n=g(t).toHsl();return n.l-=e/100,n.l=R(n.l),g(n)}function B(t,e){var n=g(t).toHsl(),r=(n.h+e)%360;return n.h=r<0?360+r:r,g(n)}function b(t){var e=g(t).toHsl();return e.h=(e.h+180)%360,g(e)}function Q(t){var e=g(t).toHsl(),n=e.h;return[g(t),g({h:(n+120)%360,s:e.s,l:e.l}),g({h:(n+240)%360,s:e.s,l:e.l})]}function w(t){var e=g(t).toHsl(),n=e.h;return[g(t),g({h:(n+90)%360,s:e.s,l:e.l}),g({h:(n+180)%360,s:e.s,l:e.l}),g({h:(n+270)%360,s:e.s,l:e.l})]}function x(t){var e=g(t).toHsl(),n=e.h;return[g(t),g({h:(n+72)%360,s:e.s,l:e.l}),g({h:(n+216)%360,s:e.s,l:e.l})]}function _(t,e,n){e=e||6,n=n||30;var r=g(t).toHsl(),i=360/n,o=[g(t)];for(r.h=(r.h-(i*e>>1)+720)%360;--e;)r.h=(r.h+i)%360,o.push(g(r));return o}function D(t,e){e=e||6;for(var n=g(t).toHsv(),r=n.h,i=n.s,o=n.v,a=[],s=1/e;e--;)a.push(g({h:r,s:i,v:o})),o=(o+s)%1;return a}g.prototype={isDark:function(){return this.getBrightness()<128},isLight:function(){return!this.isDark()},isValid:function(){return this._ok},getOriginalInput:function(){return this._originalInput},getFormat:function(){return this._format},getAlpha:function(){return this._a},getBrightness:function(){var t=this.toRgb();return(299*t.r+587*t.g+114*t.b)/1e3},getLuminance:function(){var t,e,n,r=this.toRgb();return t=r.r/255,e=r.g/255,n=r.b/255,.2126*(t<=.03928?t/12.92:i.pow((t+.055)/1.055,2.4))+.7152*(e<=.03928?e/12.92:i.pow((e+.055)/1.055,2.4))+.0722*(n<=.03928?n/12.92:i.pow((n+.055)/1.055,2.4))},setAlpha:function(t){return this._a=S(t),this._roundA=A(100*this._a)/100,this},toHsv:function(){var t=d(this._r,this._g,this._b);return{h:360*t.h,s:t.s,v:t.v,a:this._a}},toHsvString:function(){var t=d(this._r,this._g,this._b),e=A(360*t.h),n=A(100*t.s),r=A(100*t.v);return 1==this._a?"hsv("+e+", "+n+"%, "+r+"%)":"hsva("+e+", "+n+"%, "+r+"%, "+this._roundA+")"},toHsl:function(){var t=h(this._r,this._g,this._b);return{h:360*t.h,s:t.s,l:t.l,a:this._a}},toHslString:function(){var t=h(this._r,this._g,this._b),e=A(360*t.h),n=A(100*t.s),r=A(100*t.l);return 1==this._a?"hsl("+e+", "+n+"%, "+r+"%)":"hsla("+e+", "+n+"%, "+r+"%, "+this._roundA+")"},toHex:function(t){return f(this._r,this._g,this._b,t)},toHexString:function(t){return"#"+this.toHex(t)},toHex8:function(t){return function(t,e,n,r,i){var o=[F(A(t).toString(16)),F(A(e).toString(16)),F(A(n).toString(16)),F(U(r))];if(i&&o[0].charAt(0)==o[0].charAt(1)&&o[1].charAt(0)==o[1].charAt(1)&&o[2].charAt(0)==o[2].charAt(1)&&o[3].charAt(0)==o[3].charAt(1))return o[0].charAt(0)+o[1].charAt(0)+o[2].charAt(0)+o[3].charAt(0);return o.join("")}(this._r,this._g,this._b,this._a,t)},toHex8String:function(t){return"#"+this.toHex8(t)},toRgb:function(){return{r:A(this._r),g:A(this._g),b:A(this._b),a:this._a}},toRgbString:function(){return 1==this._a?"rgb("+A(this._r)+", "+A(this._g)+", "+A(this._b)+")":"rgba("+A(this._r)+", "+A(this._g)+", "+A(this._b)+", "+this._roundA+")"},toPercentageRgb:function(){return{r:A(100*L(this._r,255))+"%",g:A(100*L(this._g,255))+"%",b:A(100*L(this._b,255))+"%",a:this._a}},toPercentageRgbString:function(){return 1==this._a?"rgb("+A(100*L(this._r,255))+"%, "+A(100*L(this._g,255))+"%, "+A(100*L(this._b,255))+"%)":"rgba("+A(100*L(this._r,255))+"%, "+A(100*L(this._g,255))+"%, "+A(100*L(this._b,255))+"%, "+this._roundA+")"},toName:function(){return 0===this._a?"transparent":!(this._a<1)&&(k[f(this._r,this._g,this._b,!0)]||!1)},toFilter:function(t){var e="#"+p(this._r,this._g,this._b,this._a),n=e,r=this._gradientType?"GradientType = 1, ":"";if(t){var i=g(t);n="#"+p(i._r,i._g,i._b,i._a)}return"progid:DXImageTransform.Microsoft.gradient("+r+"startColorstr="+e+",endColorstr="+n+")"},toString:function(t){var e=!!t;t=t||this._format;var n=!1,r=this._a<1&&this._a>=0;return e||!r||"hex"!==t&&"hex6"!==t&&"hex3"!==t&&"hex4"!==t&&"hex8"!==t&&"name"!==t?("rgb"===t&&(n=this.toRgbString()),"prgb"===t&&(n=this.toPercentageRgbString()),"hex"!==t&&"hex6"!==t||(n=this.toHexString()),"hex3"===t&&(n=this.toHexString(!0)),"hex4"===t&&(n=this.toHex8String(!0)),"hex8"===t&&(n=this.toHex8String()),"name"===t&&(n=this.toName()),"hsl"===t&&(n=this.toHslString()),"hsv"===t&&(n=this.toHsvString()),n||this.toHexString()):"name"===t&&0===this._a?this.toName():this.toRgbString()},clone:function(){return g(this.toString())},_applyModification:function(t,e){var n=t.apply(null,[this].concat([].slice.call(e)));return this._r=n._r,this._g=n._g,this._b=n._b,this.setAlpha(n._a),this},lighten:function(){return this._applyModification(y,arguments)},brighten:function(){return this._applyModification(E,arguments)},darken:function(){return this._applyModification(v,arguments)},desaturate:function(){return this._applyModification(I,arguments)},saturate:function(){return this._applyModification(C,arguments)},greyscale:function(){return this._applyModification(m,arguments)},spin:function(){return this._applyModification(B,arguments)},_applyCombination:function(t,e){return t.apply(null,[this].concat([].slice.call(e)))},analogous:function(){return this._applyCombination(_,arguments)},complement:function(){return this._applyCombination(b,arguments)},monochromatic:function(){return this._applyCombination(D,arguments)},splitcomplement:function(){return this._applyCombination(x,arguments)},triad:function(){return this._applyCombination(Q,arguments)},tetrad:function(){return this._applyCombination(w,arguments)}},g.fromRatio=function(t,e){if("object"==typeof t){var n={};for(var r in t)t.hasOwnProperty(r)&&(n[r]="a"===r?t[r]:T(t[r]));t=n}return g(t,e)},g.equals=function(t,e){return!(!t||!e)&&g(t).toRgbString()==g(e).toRgbString()},g.random=function(){return g.fromRatio({r:u(),g:u(),b:u()})},g.mix=function(t,e,n){n=0===n?0:n||50;var r=g(t).toRgb(),i=g(e).toRgb(),o=n/100;return g({r:(i.r-r.r)*o+r.r,g:(i.g-r.g)*o+r.g,b:(i.b-r.b)*o+r.b,a:(i.a-r.a)*o+r.a})},g.readability=function(t,e){var n=g(t),r=g(e);return(i.max(n.getLuminance(),r.getLuminance())+.05)/(i.min(n.getLuminance(),r.getLuminance())+.05)},g.isReadable=function(t,e,n){var r,i,o=g.readability(t,e);switch(i=!1,(r=function(t){var e,n;e=((t=t||{level:"AA",size:"small"}).level||"AA").toUpperCase(),n=(t.size||"small").toLowerCase(),"AA"!==e&&"AAA"!==e&&(e="AA");"small"!==n&&"large"!==n&&(n="small");return{level:e,size:n}}(n)).level+r.size){case"AAsmall":case"AAAlarge":i=o>=4.5;break;case"AAlarge":i=o>=3;break;case"AAAsmall":i=o>=7}return i},g.mostReadable=function(t,e,n){var r,i,o,a,s=null,A=0;i=(n=n||{}).includeFallbackColors,o=n.level,a=n.size;for(var l=0;l<e.length;l++)(r=g.readability(t,e[l]))>A&&(A=r,s=g(e[l]));return g.isReadable(t,s,{level:o,size:a})||!i?s:(n.includeFallbackColors=!1,g.mostReadable(t,["#fff","#000"],n))};var M=g.names={aliceblue:"f0f8ff",antiquewhite:"faebd7",aqua:"0ff",aquamarine:"7fffd4",azure:"f0ffff",beige:"f5f5dc",bisque:"ffe4c4",black:"000",blanchedalmond:"ffebcd",blue:"00f",blueviolet:"8a2be2",brown:"a52a2a",burlywood:"deb887",burntsienna:"ea7e5d",cadetblue:"5f9ea0",chartreuse:"7fff00",chocolate:"d2691e",coral:"ff7f50",cornflowerblue:"6495ed",cornsilk:"fff8dc",crimson:"dc143c",cyan:"0ff",darkblue:"00008b",darkcyan:"008b8b",darkgoldenrod:"b8860b",darkgray:"a9a9a9",darkgreen:"006400",darkgrey:"a9a9a9",darkkhaki:"bdb76b",darkmagenta:"8b008b",darkolivegreen:"556b2f",darkorange:"ff8c00",darkorchid:"9932cc",darkred:"8b0000",darksalmon:"e9967a",darkseagreen:"8fbc8f",darkslateblue:"483d8b",darkslategray:"2f4f4f",darkslategrey:"2f4f4f",darkturquoise:"00ced1",darkviolet:"9400d3",deeppink:"ff1493",deepskyblue:"00bfff",dimgray:"696969",dimgrey:"696969",dodgerblue:"1e90ff",firebrick:"b22222",floralwhite:"fffaf0",forestgreen:"228b22",fuchsia:"f0f",gainsboro:"dcdcdc",ghostwhite:"f8f8ff",gold:"ffd700",goldenrod:"daa520",gray:"808080",green:"008000",greenyellow:"adff2f",grey:"808080",honeydew:"f0fff0",hotpink:"ff69b4",indianred:"cd5c5c",indigo:"4b0082",ivory:"fffff0",khaki:"f0e68c",lavender:"e6e6fa",lavenderblush:"fff0f5",lawngreen:"7cfc00",lemonchiffon:"fffacd",lightblue:"add8e6",lightcoral:"f08080",lightcyan:"e0ffff",lightgoldenrodyellow:"fafad2",lightgray:"d3d3d3",lightgreen:"90ee90",lightgrey:"d3d3d3",lightpink:"ffb6c1",lightsalmon:"ffa07a",lightseagreen:"20b2aa",lightskyblue:"87cefa",lightslategray:"789",lightslategrey:"789",lightsteelblue:"b0c4de",lightyellow:"ffffe0",lime:"0f0",limegreen:"32cd32",linen:"faf0e6",magenta:"f0f",maroon:"800000",mediumaquamarine:"66cdaa",mediumblue:"0000cd",mediumorchid:"ba55d3",mediumpurple:"9370db",mediumseagreen:"3cb371",mediumslateblue:"7b68ee",mediumspringgreen:"00fa9a",mediumturquoise:"48d1cc",mediumvioletred:"c71585",midnightblue:"191970",mintcream:"f5fffa",mistyrose:"ffe4e1",moccasin:"ffe4b5",navajowhite:"ffdead",navy:"000080",oldlace:"fdf5e6",olive:"808000",olivedrab:"6b8e23",orange:"ffa500",orangered:"ff4500",orchid:"da70d6",palegoldenrod:"eee8aa",palegreen:"98fb98",paleturquoise:"afeeee",palevioletred:"db7093",papayawhip:"ffefd5",peachpuff:"ffdab9",peru:"cd853f",pink:"ffc0cb",plum:"dda0dd",powderblue:"b0e0e6",purple:"800080",rebeccapurple:"663399",red:"f00",rosybrown:"bc8f8f",royalblue:"4169e1",saddlebrown:"8b4513",salmon:"fa8072",sandybrown:"f4a460",seagreen:"2e8b57",seashell:"fff5ee",sienna:"a0522d",silver:"c0c0c0",skyblue:"87ceeb",slateblue:"6a5acd",slategray:"708090",slategrey:"708090",snow:"fffafa",springgreen:"00ff7f",steelblue:"4682b4",tan:"d2b48c",teal:"008080",thistle:"d8bfd8",tomato:"ff6347",turquoise:"40e0d0",violet:"ee82ee",wheat:"f5deb3",white:"fff",whitesmoke:"f5f5f5",yellow:"ff0",yellowgreen:"9acd32"},k=g.hexNames=function(t){var e={};for(var n in t)t.hasOwnProperty(n)&&(e[t[n]]=n);return e}(M);function S(t){return t=parseFloat(t),(isNaN(t)||t<0||t>1)&&(t=1),t}function L(t,e){(function(t){return"string"==typeof t&&-1!=t.indexOf(".")&&1===parseFloat(t)})(t)&&(t="100%");var n=function(t){return"string"==typeof t&&-1!=t.indexOf("%")}(t);return t=l(e,c(0,parseFloat(t))),n&&(t=parseInt(t*e,10)/100),i.abs(t-e)<1e-6?1:t%e/parseFloat(e)}function R(t){return l(1,c(0,t))}function N(t){return parseInt(t,16)}function F(t){return 1==t.length?"0"+t:""+t}function T(t){return t<=1&&(t=100*t+"%"),t}function U(t){return i.round(255*parseFloat(t)).toString(16)}function O(t){return N(t)/255}var G,H,j,Y=(H="[\\s|\\(]+("+(G="(?:[-\\+]?\\d*\\.\\d+%?)|(?:[-\\+]?\\d+%?)")+")[,|\\s]+("+G+")[,|\\s]+("+G+")\\s*\\)?",j="[\\s|\\(]+("+G+")[,|\\s]+("+G+")[,|\\s]+("+G+")[,|\\s]+("+G+")\\s*\\)?",{CSS_UNIT:new RegExp(G),rgb:new RegExp("rgb"+H),rgba:new RegExp("rgba"+j),hsl:new RegExp("hsl"+H),hsla:new RegExp("hsla"+j),hsv:new RegExp("hsv"+H),hsva:new RegExp("hsva"+j),hex3:/^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex6:/^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/,hex4:/^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex8:/^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/});function P(t){return!!Y.CSS_UNIT.exec(t)}t.exports?t.exports=g:void 0===(r=function(){return g}.call(e,n,e,t))||(t.exports=r)}(Math)},function(t,e,n){"use strict";var r=n(1425);t.exports={type:{valType:"enumerated",role:"info",values:[],dflt:"scatter",editType:"calc+clearAxisTypes",_noTemplating:!0},visible:{valType:"enumerated",values:[!0,!1,"legendonly"],role:"info",dflt:!0,editType:"calc",description:["Determines whether or not this trace is visible.","If *legendonly*, the trace is not drawn,","but can appear as a legend item","(provided that the legend itself is visible)."].join(" ")},showlegend:{valType:"boolean",role:"info",dflt:!0,editType:"style",description:["Determines whether or not an item corresponding to this","trace is shown in the legend."].join(" ")},legendgroup:{valType:"string",role:"info",dflt:"",editType:"style",description:["Sets the legend group for this trace.","Traces part of the same legend group hide/show at the same time","when toggling legend items."].join(" ")},opacity:{valType:"number",role:"style",min:0,max:1,dflt:1,editType:"style",description:"Sets the opacity of the trace."},name:{valType:"string",role:"info",editType:"style",description:["Sets the trace name.","The trace name appear as the legend item and on hover."].join(" ")},uid:{valType:"string",role:"info",editType:"plot",anim:!0,description:["Assign an id to this trace,","Use this to provide object constancy between traces during animations","and transitions."].join(" ")},ids:{valType:"data_array",editType:"calc",anim:!0,description:["Assigns id labels to each datum.","These ids for object constancy of data points during animation.","Should be an array of strings, not numbers or any other type."].join(" ")},customdata:{valType:"data_array",editType:"calc",description:["Assigns extra data each datum.","This may be useful when listening to hover, click and selection events.","Note that, *scatter* traces also appends customdata items in the markers","DOM elements"].join(" ")},meta:{valType:"any",arrayOk:!0,role:"info",editType:"plot",description:["Assigns extra meta information associated with this trace","that can be used in various text attributes.","Attributes such as trace `name`, graph, axis and colorbar `title.text`, annotation `text`","`rangeselector`, `updatemenues` and `sliders` `label` text","all support `meta`.","To access the trace `meta` values in an attribute in the same trace, simply use","`%{meta[i]}` where `i` is the index or key of the `meta`","item in question.","To access trace `meta` in layout attributes, use","`%{data[n[.meta[i]}` where `i` is the index or key of the `meta`","and `n` is the trace index."].join(" ")},selectedpoints:{valType:"any",role:"info",editType:"calc",description:["Array containing integer indices of selected points.","Has an effect only for traces that support selections.","Note that an empty array means an empty selection where the `unselected`","are turned on for all points, whereas, any other non-array values means no","selection all where the `selected` and `unselected` styles have no effect."].join(" ")},hoverinfo:{valType:"flaglist",role:"info",flags:["x","y","z","text","name"],extras:["all","none","skip"],arrayOk:!0,dflt:"all",editType:"none",description:["Determines which trace information appear on hover.","If `none` or `skip` are set, no information is displayed upon hovering.","But, if `none` is set, click and hover events are still fired."].join(" ")},hoverlabel:r.hoverlabel,stream:{token:{valType:"string",noBlank:!0,strict:!0,role:"info",editType:"calc",description:["The stream id number links a data trace on a plot with a stream.","See https://plot.ly/settings for more details."].join(" ")},maxpoints:{valType:"number",min:0,max:1e4,dflt:500,role:"info",editType:"calc",description:["Sets the maximum number of points to keep on the plots from an","incoming stream.","If `maxpoints` is set to *50*, only the newest 50 points will","be displayed on the plot."].join(" ")},editType:"calc"},transforms:{_isLinkedToArray:"transform",editType:"calc",description:["An array of operations that manipulate the trace data,","for example filtering or sorting the data arrays."].join(" ")},uirevision:{valType:"any",role:"info",editType:"none",description:["Controls persistence of some user-driven changes to the trace:","`constraintrange` in `parcoords` traces, as well as some","`editable: true` modifications such as `name` and `colorbar.title`.","Defaults to `layout.uirevision`.","Note that other user-driven trace attribute changes are controlled","by `layout` attributes:","`trace.visible` is controlled by `layout.legend.uirevision`,","`selectedpoints` is controlled by `layout.selectionrevision`,","and `colorbar.(x|y)` (accessible with `config: {editable: true}`)","is controlled by `layout.editrevision`.","Trace changes are tracked by `uid`, which only falls back on trace","index if no `uid` is provided. So if your app can add/remove traces","before the end of the `data` array, such that the same trace has a","different index, you can still preserve user-driven changes if you","give each trace a `uid` that stays with it as it moves."].join(" ")}}},function(t,e,n){"use strict";e.counter=function(t,e,n,r){var i=(e||"")+(n?"":"$"),o=!1===r?"":"^";return"xy"===t?new RegExp(o+"x([2-9]|[1-9][0-9]+)?y([2-9]|[1-9][0-9]+)?"+i):new RegExp(o+t+"([2-9]|[1-9][0-9]+)?"+i)}},function(t,e,n){"use strict";var r=n(188),i=n(708),o=n(709),a=n(240),s=n(710).addStyleRule,A=n(241),l=n(705),c=n(1432),u=A.extendFlat,g=A.extendDeepAll;function h(t){var n=t.name,i=t.categories,o=t.meta;if(e.modules[n])r.log("Type "+n+" already registered");else{e.subplotsRegistry[t.basePlotModule.name]||function(t){var n=t.name;if(e.subplotsRegistry[n])return void r.log("Plot type "+n+" already registered.");for(var i in I(t),e.subplotsRegistry[n]=t,e.componentsRegistry)y(i,t.name)}(t.basePlotModule);for(var a={},A=0;A<i.length;A++)a[i[A]]=!0,e.allCategories[i[A]]=!0;for(var l in e.modules[n]={_module:t,categories:a},o&&Object.keys(o).length&&(e.modules[n].meta=o),e.allTypes.push(n),e.componentsRegistry)C(l,n);t.layoutAttributes&&u(e.traceLayoutAttributes,t.layoutAttributes);var c=t.basePlotModule,g=c.name;if("mapbox"===g){var h=c.constants.styleRules;for(var d in h)s(".js-plotly-plot .plotly .mapboxgl-"+d,h[d])}"geo"!==g&&"mapbox"!==g||void 0===typeof window||void 0!==window.PlotlyGeoAssets||(window.PlotlyGeoAssets={topojson:{}})}}function d(t){if("string"!=typeof t.name)throw new Error("Component module *name* must be a string.");var n=t.name;for(var r in e.componentsRegistry[n]=t,t.layoutAttributes&&(t.layoutAttributes._isLinkedToArray&&o(e.layoutArrayContainers,n),I(t)),e.modules)C(n,r);for(var i in e.subplotsRegistry)y(n,i);for(var a in e.transformsRegistry)m(n,a);t.schema&&t.schema.layout&&g(c,t.schema.layout)}function f(t){if("string"!=typeof t.name)throw new Error("Transform module *name* must be a string.");var n="Transform module "+t.name,i="function"==typeof t.transform,o="function"==typeof t.calcTransform;if(!i&&!o)throw new Error(n+" is missing a *transform* or *calcTransform* method.");for(var s in i&&o&&r.log([n+" has both a *transform* and *calcTransform* methods.","Please note that all *transform* methods are executed","before all *calcTransform* methods."].join(" ")),a(t.attributes)||r.log(n+" registered without an *attributes* object."),"function"!=typeof t.supplyDefaults&&r.log(n+" registered without a *supplyDefaults* method."),e.transformsRegistry[t.name]=t,e.componentsRegistry)m(s,t.name)}function p(t){var n=t.name,r=n.split("-")[0],i=t.dictionary,o=t.format,a=i&&Object.keys(i).length,s=o&&Object.keys(o).length,A=e.localeRegistry,l=A[n];if(l||(A[n]=l={}),r!==n){var c=A[r];c||(A[r]=c={}),a&&c.dictionary===l.dictionary&&(c.dictionary=i),s&&c.format===l.format&&(c.format=o)}a&&(l.dictionary=i),s&&(l.format=o)}function I(t){if(t.layoutAttributes){var n=t.layoutAttributes._arrayAttrRegexps;if(n)for(var r=0;r<n.length;r++)o(e.layoutArrayRegexes,n[r])}}function C(t,n){var r=e.componentsRegistry[t].schema;if(r&&r.traces){var i=r.traces[n];i&&g(e.modules[n]._module.attributes,i)}}function m(t,n){var r=e.componentsRegistry[t].schema;if(r&&r.transforms){var i=r.transforms[n];i&&g(e.transformsRegistry[n].attributes,i)}}function y(t,n){var r=e.componentsRegistry[t].schema;if(r&&r.subplots){var i=e.subplotsRegistry[n],o=i.layoutAttributes,a="subplot"===i.attr?i.name:i.attr;Array.isArray(a)&&(a=a[0]);var s=r.subplots[a];o&&s&&g(o,s)}}function E(t){return"object"==typeof t&&(t=t.type),t}e.modules={},e.allCategories={},e.allTypes=[],e.subplotsRegistry={},e.transformsRegistry={},e.componentsRegistry={},e.layoutArrayContainers=[],e.layoutArrayRegexes=[],e.traceLayoutAttributes={},e.localeRegistry={},e.apiMethodRegistry={},e.collectableSubplotTypes=null,e.register=function(t){if(e.collectableSubplotTypes=null,!t)throw new Error("No argument passed to Plotly.register.");t&&!Array.isArray(t)&&(t=[t]);for(var n=0;n<t.length;n++){var r=t[n];if(!r)throw new Error("Invalid module was attempted to be registered!");switch(r.moduleType){case"trace":h(r);break;case"transform":f(r);break;case"component":d(r);break;case"locale":p(r);break;case"apiMethod":var i=r.name;e.apiMethodRegistry[i]=r.fn;break;default:throw new Error("Invalid module was attempted to be registered!")}}},e.getModule=function(t){var n=e.modules[E(t)];return!!n&&n._module},e.traceIs=function(t,n){if("various"===(t=E(t)))return!1;var i=e.modules[t];return i||(t&&"area"!==t&&r.log("Unrecognized trace type "+t+"."),i=e.modules[l.type.dflt]),!!i.categories[n]},e.getTransformIndices=function(t,e){for(var n=[],r=t.transforms||[],i=0;i<r.length;i++)r[i].type===e&&n.push(i);return n},e.hasTransform=function(t,e){for(var n=t.transforms||[],r=0;r<n.length;r++)if(n[r].type===e)return!0;return!1},e.getComponentMethod=function(t,n){var r=e.componentsRegistry[t];return r&&r[n]||i},e.call=function(){var t=arguments[0],n=[].slice.call(arguments,1);return e.apiMethodRegistry[t].apply(null,n)}},function(t,e,n){"use strict";t.exports=function(){}},function(t,e,n){"use strict";t.exports=function(t,e){if(e instanceof RegExp){for(var n=e.toString(),r=0;r<t.length;r++)if(t[r]instanceof RegExp&&t[r].toString()===n)return t;t.push(e)}else!e&&0!==e||-1!==t.indexOf(e)||t.push(e);return t}},function(t,e,n){"use strict";var r=n(185),i=n(188);function o(t){var e=t&&t.parentNode;e&&e.removeChild(t)}function a(t,e,n){var r="plotly.js-style-"+t,o=document.getElementById(r);o||((o=document.createElement("style")).setAttribute("id",r),o.appendChild(document.createTextNode("")),document.head.appendChild(o));var a=o.sheet;a.insertRule?a.insertRule(e+"{"+n+"}",0):a.addRule?a.addRule(e,n,0):i.warn("addStyleRule failed")}t.exports={getGraphDiv:function(t){var e;if("string"==typeof t){if(null===(e=document.getElementById(t)))throw new Error("No DOM element with id '"+t+"' exists on the page.");return e}if(null==t)throw new Error("DOM element provided is null or undefined");return t},isPlotDiv:function(t){var e=r.select(t);return e.node()instanceof HTMLElement&&e.size()&&e.classed("js-plotly-plot")},removeElement:o,addStyleRule:function(t,e){a("global",t,e)},addRelatedStyleRule:a,deleteRelatedStyleRule:function(t){var e="plotly.js-style-"+t,n=document.getElementById(e);n&&o(n)}}},function(t,e,n){"use strict";t.exports=function(t){return t}},function(t){t.exports=JSON.parse('{"name":"plotlywidget","version":"1.0.0","description":"The plotly JupyterLab extension","author":"The plotly.py team","license":"MIT","main":"src/index.js","repository":{"type":"git","url":"https://github.com/plotly/plotly.py"},"keywords":["jupyter","widgets","ipython","ipywidgets","plotly"],"files":["src/**/*.js","dist/*.js","style/*.*"],"scripts":{"build":"npm run build:src","build:src":"rimraf dist && tsc","clean":"rimraf dist/ && rimraf  ../../python/plotly/plotlywidget/static\'","prepublish":"webpack","test":"echo \\"Error: no test specified\\" && exit 1"},"devDependencies":{"webpack":"^3.10.0","rimraf":"^2.6.1","ify-loader":"^1.1.0","typescript":"~3.1.1"},"dependencies":{"plotly.js":"^1.48.3","@jupyter-widgets/base":"^2.0.0","lodash":"^4.17.4"},"jupyterlab":{"extension":"src/jupyterlab-plugin.js"}}')},function(t,e,n){"use strict";n.r(e);var r=n(13),i=n.n(r),o=n(18),a=n.n(o),s=n(14),A=n.n(s),l=n(19),c=n.n(l),u=n(0),g=n.n(u),h=n(16),d=n.n(h),f=n(1),p=n.n(f),I=n(31),C=n(67),m=6,y=7,E=n(7),v=n.n(E),B=function(t){function e(){return a()(this,e),A()(this,t.apply(this,arguments))}return c()(e,t),e.prototype.render=function(){for(var t=this.props,e=t.value.localeData(),n=t.prefixCls,r=[],i=[],o=e.firstDayOfWeek(),a=void 0,s=v()(),A=0;A<y;A++){var l=(o+A)%y;s.day(l),r[A]=e.weekdaysMin(s),i[A]=e.weekdaysShort(s)}t.showWeekNumber&&(a=g.a.createElement("th",{role:"columnheader",className:n+"-column-header "+n+"-week-number-header"},g.a.createElement("span",{className:n+"-column-header-inner"},"x")));var c=i.map(function(t,e){return g.a.createElement("th",{key:e,role:"columnheader",title:t,className:n+"-column-header"},g.a.createElement("span",{className:n+"-column-header-inner"},r[e]))});return g.a.createElement("thead",null,g.a.createElement("tr",{role:"row"},a,c))},e}(g.a.Component),b=n(4),Q=n.n(b),w={disabledHours:function(){return[]},disabledMinutes:function(){return[]},disabledSeconds:function(){return[]}};function x(t){var e=v()();return e.locale(t.locale()).utcOffset(t.utcOffset()),e}function _(t){return t.format("LL")}function D(t){return _(x(t))}function M(t,e){var n=e?e(t):{};return n=i()({},w,n)}function k(t,e,n){return(!e||!e(t))&&!(n&&!function(t,e){return function(t,e){var n=!1;if(t){var r=t.hour(),i=t.minute(),o=t.second();n=-1!==e.disabledHours().indexOf(r)||-1!==e.disabledMinutes(r).indexOf(i)||-1!==e.disabledSeconds(r,i).indexOf(o)}return!n}(t,M(t,e))}(t,n))}function S(t,e){return t?(Array.isArray(e)&&(e=e[0]),t.format(e)):""}function L(t,e){return t&&e&&t.isSame(e,"day")}function R(t,e){return t.year()<e.year()?1:t.year()===e.year()&&t.month()<e.month()}function N(t,e){return t.year()>e.year()?1:t.year()===e.year()&&t.month()>e.month()}var F=function(t){function e(){return a()(this,e),A()(this,t.apply(this,arguments))}return c()(e,t),e.prototype.render=function(){var t=this.props,e=t.contentRender,n=t.prefixCls,r=t.selectedValue,i=t.value,o=t.showWeekNumber,a=t.dateRender,s=t.disabledDate,A=t.hoverValue,l=void 0,c=void 0,u=void 0,h=[],d=x(i),f=n+"-cell",p=n+"-week-number-cell",I=n+"-date",C=n+"-today",E=n+"-selected-day",v=n+"-selected-date",B=n+"-selected-start-date",b=n+"-selected-end-date",w=n+"-in-range-cell",D=n+"-last-month-cell",M=n+"-next-month-btn-day",k=n+"-disabled-cell",S=n+"-disabled-cell-first-of-row",F=n+"-disabled-cell-last-of-row",T=n+"-last-day-of-month",U=i.clone();U.date(1);var O=(U.day()+7-i.localeData().firstDayOfWeek())%7,G=U.clone();G.add(0-O,"days");var H=0;for(l=0;l<m;l++)for(c=0;c<y;c++)u=G,H&&(u=u.clone()).add(H,"days"),h.push(u),H++;var j,Y=[];for(H=0,l=0;l<m;l++){var P,K=void 0,z=void 0,J=!1,q=[];for(o&&(z=g.a.createElement("td",{key:h[H].week(),role:"gridcell",className:p},h[H].week())),c=0;c<y;c++){var V=null,W=null;u=h[H],c<y-1&&(V=h[H+1]),c>0&&(W=h[H-1]);var X=f,Z=!1,$=!1;L(u,d)&&(X+=" "+C,K=!0);var tt=R(u,i),et=N(u,i);if(r&&Array.isArray(r)){var nt=A.length?A:r;if(!tt&&!et){var rt=nt[0],it=nt[1];rt&&L(u,rt)&&($=!0,J=!0,X+=" "+B),(rt||it)&&(L(u,it)?($=!0,J=!0,X+=" "+b):null==rt&&u.isBefore(it,"day")?X+=" "+w:null==it&&u.isAfter(rt,"day")?X+=" "+w:u.isAfter(rt,"day")&&u.isBefore(it,"day")&&(X+=" "+w))}}else L(u,i)&&($=!0,J=!0);L(u,r)&&(X+=" "+v),tt&&(X+=" "+D),et&&(X+=" "+M),u.clone().endOf("month").date()===u.date()&&(X+=" "+T),s&&s(u,i)&&(Z=!0,W&&s(W,i)||(X+=" "+S),V&&s(V,i)||(X+=" "+F)),$&&(X+=" "+E),Z&&(X+=" "+k);var ot=void 0;if(a)ot=a(u,i);else{var at=e?e(u,i):u.date();ot=g.a.createElement("div",{key:(j=u,"rc-calendar-"+j.year()+"-"+j.month()+"-"+j.date()),className:I,"aria-selected":$,"aria-disabled":Z},at)}q.push(g.a.createElement("td",{key:H,onClick:Z?void 0:t.onSelect.bind(null,u),onMouseEnter:Z?void 0:t.onDayHover&&t.onDayHover.bind(null,u)||void 0,role:"gridcell",title:_(u),className:X},ot)),H++}Y.push(g.a.createElement("tr",{key:l,role:"row",className:Q()((P={},P[n+"-current-week"]=K,P[n+"-active-week"]=J,P))},z,q))}return g.a.createElement("tbody",{className:n+"-tbody"},Y)},e}(g.a.Component);F.propTypes={contentRender:p.a.func,dateRender:p.a.func,disabledDate:p.a.func,prefixCls:p.a.string,selectedValue:p.a.oneOfType([p.a.object,p.a.arrayOf(p.a.object)]),value:p.a.object,hoverValue:p.a.any,showWeekNumber:p.a.bool},F.defaultProps={hoverValue:[]};var T=F,U=function(t){function e(){return a()(this,e),A()(this,t.apply(this,arguments))}return c()(e,t),e.prototype.render=function(){var t=this.props,e=t.prefixCls;return g.a.createElement("table",{className:e+"-table",cellSpacing:"0",role:"grid"},g.a.createElement(B,t),g.a.createElement(T,t))},e}(g.a.Component);function O(t){return t}function G(t){return g.a.Children.map(t,O)}function H(t){var e=this.state.value.clone();e.month(t),this.setAndSelectValue(e)}var j=function(t){function e(n){a()(this,e);var r=A()(this,t.call(this,n));return r.state={value:n.value},r}return c()(e,t),e.prototype.componentWillReceiveProps=function(t){"value"in t&&this.setState({value:t.value})},e.prototype.setAndSelectValue=function(t){this.setState({value:t}),this.props.onSelect(t)},e.prototype.months=function(){for(var t,e,n=this.state.value.clone(),r=[],i=0,o=0;o<4;o++){r[o]=[];for(var a=0;a<3;a++){n.month(i);var s=(e=void 0,e=(t=n).locale(),t.localeData()["zh-cn"===e?"months":"monthsShort"](t));r[o][a]={value:i,content:s,title:s},i++}}return r},e.prototype.render=function(){var t=this,e=this.props,n=this.state.value,r=x(n),i=this.months(),o=n.month(),a=e.prefixCls,s=e.locale,A=e.contentRender,l=e.cellRender,c=i.map(function(i,c){var u=i.map(function(i){var c,u=!1;if(e.disabledDate){var h=n.clone();h.month(i.value),u=e.disabledDate(h)}var d=((c={})[a+"-cell"]=1,c[a+"-cell-disabled"]=u,c[a+"-selected-cell"]=i.value===o,c[a+"-current-cell"]=r.year()===n.year()&&i.value===r.month(),c),f=void 0;if(l){var p=n.clone();p.month(i.value),f=l(p,s)}else{var I=void 0;if(A){var C=n.clone();C.month(i.value),I=A(C,s)}else I=i.content;f=g.a.createElement("a",{className:a+"-month"},I)}return g.a.createElement("td",{role:"gridcell",key:i.value,onClick:u?null:H.bind(t,i.value),title:i.title,className:Q()(d)},f)});return g.a.createElement("tr",{key:c,role:"row"},u)});return g.a.createElement("table",{className:a+"-table",cellSpacing:"0",role:"grid"},g.a.createElement("tbody",{className:a+"-tbody"},c))},e}(u.Component);j.defaultProps={onSelect:function(){}},j.propTypes={onSelect:p.a.func,cellRender:p.a.func,prefixCls:p.a.string,value:p.a.object};var Y=j;function P(t){this.props.changeYear(t)}function K(){}var z=function(t){function e(n){a()(this,e);var r=A()(this,t.call(this,n));return r.setAndSelectValue=function(t){r.setValue(t),r.props.onSelect(t)},r.setValue=function(t){"value"in r.props&&r.setState({value:t})},r.nextYear=P.bind(r,1),r.previousYear=P.bind(r,-1),r.prefixCls=n.rootPrefixCls+"-month-panel",r.state={value:n.value||n.defaultValue},r}return c()(e,t),e.getDerivedStateFromProps=function(t){var e={};return"value"in t&&(e={value:t.value}),e},e.prototype.render=function(){var t=this.props,e=this.state.value,n=t.locale,r=t.cellRender,i=t.contentRender,o=t.renderFooter,a=e.year(),s=this.prefixCls,A=o&&o("month");return g.a.createElement("div",{className:s,style:t.style},g.a.createElement("div",null,g.a.createElement("div",{className:s+"-header"},g.a.createElement("a",{className:s+"-prev-year-btn",role:"button",onClick:this.previousYear,title:n.previousYear}),g.a.createElement("a",{className:s+"-year-select",role:"button",onClick:t.onYearPanelShow,title:n.yearSelect},g.a.createElement("span",{className:s+"-year-select-content"},a),g.a.createElement("span",{className:s+"-year-select-arrow"},"x")),g.a.createElement("a",{className:s+"-next-year-btn",role:"button",onClick:this.nextYear,title:n.nextYear})),g.a.createElement("div",{className:s+"-body"},g.a.createElement(Y,{disabledDate:t.disabledDate,onSelect:this.setAndSelectValue,locale:n,value:e,cellRender:r,contentRender:i,prefixCls:s})),A&&g.a.createElement("div",{className:s+"-footer"},A)))},e}(g.a.Component);z.propTypes={onChange:p.a.func,disabledDate:p.a.func,onSelect:p.a.func,renderFooter:p.a.func,rootPrefixCls:p.a.string,value:p.a.object,defaultValue:p.a.object},z.defaultProps={onChange:K,onSelect:K},Object(C.polyfill)(z);var J=z;function q(t){var e=this.state.value.clone();e.add(t,"year"),this.setState({value:e})}var V=function(t){function e(n){a()(this,e);var r=A()(this,t.call(this,n));return r.prefixCls=n.rootPrefixCls+"-year-panel",r.state={value:n.value||n.defaultValue},r.nextDecade=q.bind(r,10),r.previousDecade=q.bind(r,-10),r}return c()(e,t),e.prototype.years=function(){for(var t=this.state.value.year(),e=10*parseInt(t/10,10)-1,n=[],r=0,i=0;i<4;i++){n[i]=[];for(var o=0;o<3;o++){var a=e+r,s=String(a);n[i][o]={content:s,year:a,title:s},r++}}return n},e.prototype.render=function(){var t=this,e=this.props,n=this.state.value,r=e.locale,i=e.renderFooter,o=this.years(),a=n.year(),s=10*parseInt(a/10,10),A=s+9,l=this.prefixCls,c=o.map(function(e,n){var r=e.map(function(e){var n,r=((n={})[l+"-cell"]=1,n[l+"-selected-cell"]=e.year===a,n[l+"-last-decade-cell"]=e.year<s,n[l+"-next-decade-cell"]=e.year>A,n),i=void 0;return i=e.year<s?t.previousDecade:e.year>A?t.nextDecade:function(t){var e=this.state.value.clone();e.year(t),e.month(this.state.value.month()),this.setState({value:e}),this.props.onSelect(e)}.bind(t,e.year),g.a.createElement("td",{role:"gridcell",title:e.title,key:e.content,onClick:i,className:Q()(r)},g.a.createElement("a",{className:l+"-year"},e.content))});return g.a.createElement("tr",{key:n,role:"row"},r)}),u=i&&i("year");return g.a.createElement("div",{className:this.prefixCls},g.a.createElement("div",null,g.a.createElement("div",{className:l+"-header"},g.a.createElement("a",{className:l+"-prev-decade-btn",role:"button",onClick:this.previousDecade,title:r.previousDecade}),g.a.createElement("a",{className:l+"-decade-select",role:"button",onClick:e.onDecadePanelShow,title:r.decadeSelect},g.a.createElement("span",{className:l+"-decade-select-content"},s,"-",A),g.a.createElement("span",{className:l+"-decade-select-arrow"},"x")),g.a.createElement("a",{className:l+"-next-decade-btn",role:"button",onClick:this.nextDecade,title:r.nextDecade})),g.a.createElement("div",{className:l+"-body"},g.a.createElement("table",{className:l+"-table",cellSpacing:"0",role:"grid"},g.a.createElement("tbody",{className:l+"-tbody"},c))),u&&g.a.createElement("div",{className:l+"-footer"},u)))},e}(g.a.Component),W=V;V.propTypes={rootPrefixCls:p.a.string,value:p.a.object,defaultValue:p.a.object,renderFooter:p.a.func},V.defaultProps={onSelect:function(){}};function X(t){var e=this.state.value.clone();e.add(t,"years"),this.setState({value:e})}var Z=function(t){function e(n){a()(this,e);var r=A()(this,t.call(this,n));return r.state={value:n.value||n.defaultValue},r.prefixCls=n.rootPrefixCls+"-decade-panel",r.nextCentury=X.bind(r,100),r.previousCentury=X.bind(r,-100),r}return c()(e,t),e.prototype.render=function(){for(var t=this,e=this.state.value,n=this.props,r=n.locale,i=n.renderFooter,o=e.year(),a=100*parseInt(o/100,10),s=a-10,A=a+99,l=[],c=0,u=this.prefixCls,h=0;h<4;h++){l[h]=[];for(var d=0;d<3;d++){var f=s+10*c,p=s+10*c+9;l[h][d]={startDecade:f,endDecade:p},c++}}var I=i&&i("decade"),C=l.map(function(e,n){var r=e.map(function(e){var n,r=e.startDecade,i=e.endDecade,s=r<a,l=i>A,c=((n={})[u+"-cell"]=1,n[u+"-selected-cell"]=r<=o&&o<=i,n[u+"-last-century-cell"]=s,n[u+"-next-century-cell"]=l,n),h=r+"-"+i,d=void 0;return d=s?t.previousCentury:l?t.nextCentury:function(t,e){var n=this.state.value.clone();n.year(t),n.month(this.state.value.month()),this.props.onSelect(n),e.preventDefault()}.bind(t,r),g.a.createElement("td",{key:r,onClick:d,role:"gridcell",className:Q()(c)},g.a.createElement("a",{className:u+"-decade"},h))});return g.a.createElement("tr",{key:n,role:"row"},r)});return g.a.createElement("div",{className:this.prefixCls},g.a.createElement("div",{className:u+"-header"},g.a.createElement("a",{className:u+"-prev-century-btn",role:"button",onClick:this.previousCentury,title:r.previousCentury}),g.a.createElement("div",{className:u+"-century"},a,"-",A),g.a.createElement("a",{className:u+"-next-century-btn",role:"button",onClick:this.nextCentury,title:r.nextCentury})),g.a.createElement("div",{className:u+"-body"},g.a.createElement("table",{className:u+"-table",cellSpacing:"0",role:"grid"},g.a.createElement("tbody",{className:u+"-tbody"},C))),I&&g.a.createElement("div",{className:u+"-footer"},I))},e}(g.a.Component),$=Z;function tt(t){var e=this.props.value.clone();e.add(t,"months"),this.props.onValueChange(e)}function et(t){var e=this.props.value.clone();e.add(t,"years"),this.props.onValueChange(e)}function nt(t,e){return t?e:null}Z.propTypes={locale:p.a.object,value:p.a.object,defaultValue:p.a.object,rootPrefixCls:p.a.string,renderFooter:p.a.func},Z.defaultProps={onSelect:function(){}};var rt=function(t){function e(n){a()(this,e);var r=A()(this,t.call(this,n));return it.call(r),r.nextMonth=tt.bind(r,1),r.previousMonth=tt.bind(r,-1),r.nextYear=et.bind(r,1),r.previousYear=et.bind(r,-1),r.state={yearPanelReferer:null},r}return c()(e,t),e.prototype.render=function(){var t=this,e=this.props,n=e.prefixCls,r=e.locale,i=e.mode,o=e.value,a=e.showTimePicker,s=e.enableNext,A=e.enablePrev,l=e.disabledMonth,c=e.renderFooter,u=null;return"month"===i&&(u=g.a.createElement(J,{locale:r,value:o,rootPrefixCls:n,onSelect:this.onMonthSelect,onYearPanelShow:function(){return t.showYearPanel("month")},disabledDate:l,cellRender:e.monthCellRender,contentRender:e.monthCellContentRender,renderFooter:c,changeYear:this.changeYear})),"year"===i&&(u=g.a.createElement(W,{locale:r,defaultValue:o,rootPrefixCls:n,onSelect:this.onYearSelect,onDecadePanelShow:this.showDecadePanel,renderFooter:c})),"decade"===i&&(u=g.a.createElement($,{locale:r,defaultValue:o,rootPrefixCls:n,onSelect:this.onDecadeSelect,renderFooter:c})),g.a.createElement("div",{className:n+"-header"},g.a.createElement("div",{style:{position:"relative"}},nt(A&&!a,g.a.createElement("a",{className:n+"-prev-year-btn",role:"button",onClick:this.previousYear,title:r.previousYear})),nt(A&&!a,g.a.createElement("a",{className:n+"-prev-month-btn",role:"button",onClick:this.previousMonth,title:r.previousMonth})),this.monthYearElement(a),nt(s&&!a,g.a.createElement("a",{className:n+"-next-month-btn",onClick:this.nextMonth,title:r.nextMonth})),nt(s&&!a,g.a.createElement("a",{className:n+"-next-year-btn",onClick:this.nextYear,title:r.nextYear}))),u)},e}(g.a.Component);rt.propTypes={prefixCls:p.a.string,value:p.a.object,onValueChange:p.a.func,showTimePicker:p.a.bool,onPanelChange:p.a.func,locale:p.a.object,enablePrev:p.a.any,enableNext:p.a.any,disabledMonth:p.a.func,renderFooter:p.a.func,onMonthSelect:p.a.func},rt.defaultProps={enableNext:1,enablePrev:1,onPanelChange:function(){},onValueChange:function(){}};var it=function(){var t=this;this.onMonthSelect=function(e){t.props.onPanelChange(e,"date"),t.props.onMonthSelect?t.props.onMonthSelect(e):t.props.onValueChange(e)},this.onYearSelect=function(e){var n=t.state.yearPanelReferer;t.setState({yearPanelReferer:null}),t.props.onPanelChange(e,n),t.props.onValueChange(e)},this.onDecadeSelect=function(e){t.props.onPanelChange(e,"year"),t.props.onValueChange(e)},this.changeYear=function(e){e>0?t.nextYear():t.previousYear()},this.monthYearElement=function(e){var n=t.props,r=n.prefixCls,i=n.locale,o=n.value,a=o.localeData(),s=i.monthBeforeYear,A=r+"-"+(s?"my-select":"ym-select"),l=e?" "+r+"-time-status":"",c=g.a.createElement("a",{className:r+"-year-select"+l,role:"button",onClick:e?null:function(){return t.showYearPanel("date")},title:e?null:i.yearSelect},o.format(i.yearFormat)),u=g.a.createElement("a",{className:r+"-month-select"+l,role:"button",onClick:e?null:t.showMonthPanel,title:e?null:i.monthSelect},i.monthFormat?o.format(i.monthFormat):a.monthsShort(o)),h=void 0;e&&(h=g.a.createElement("a",{className:r+"-day-select"+l,role:"button"},o.format(i.dayFormat)));var d=[];return d=s?[u,h,c]:[c,u,h],g.a.createElement("span",{className:A},G(d))},this.showMonthPanel=function(){t.props.onPanelChange(null,"month")},this.showYearPanel=function(e){t.setState({yearPanelReferer:e}),t.props.onPanelChange(null,"year")},this.showDecadePanel=function(){t.props.onPanelChange(null,"decade")}},ot=rt;function at(t){var e=t.prefixCls,n=t.locale,r=t.value,i=t.timePicker,o=t.disabled,a=t.disabledDate,s=t.onToday,A=t.text,l=(!A&&i?n.now:A)||n.today,c=a&&!k(x(r),a)||o,u=c?e+"-today-btn-disabled":"";return g.a.createElement("a",{className:e+"-today-btn "+u,role:"button",onClick:c?null:s,title:D(r)},l)}function st(t){var e=t.prefixCls,n=t.locale,r=t.okDisabled,i=t.onOk,o=e+"-ok-btn";return r&&(o+=" "+e+"-ok-btn-disabled"),g.a.createElement("a",{className:o,role:"button",onClick:r?null:i},n.ok)}function At(t){var e,n=t.prefixCls,r=t.locale,i=t.showTimePicker,o=t.onOpenTimePicker,a=t.onCloseTimePicker,s=t.timePickerDisabled,A=Q()(((e={})[n+"-time-picker-btn"]=!0,e[n+"-time-picker-btn-disabled"]=s,e)),l=null;return s||(l=i?a:o),g.a.createElement("a",{className:A,role:"button",onClick:l},i?r.dateSelect:r.timeSelect)}var lt=function(t){function e(){return a()(this,e),A()(this,t.apply(this,arguments))}return c()(e,t),e.prototype.onSelect=function(t){this.props.onSelect(t)},e.prototype.getRootDOMNode=function(){return d.a.findDOMNode(this)},e.prototype.render=function(){var t=this.props,e=t.value,n=t.prefixCls,r=t.showOk,o=t.timePicker,a=t.renderFooter,s=t.mode,A=null,l=a&&a(s);if(t.showToday||o||l){var c,u=void 0;t.showToday&&(u=g.a.createElement(at,i()({},t,{value:e})));var h=void 0;(!0===r||!1!==r&&t.timePicker)&&(h=g.a.createElement(st,t));var d=void 0;t.timePicker&&(d=g.a.createElement(At,t));var f=void 0;(u||d||h||l)&&(f=g.a.createElement("span",{className:n+"-footer-btn"},l,G([u,d,h])));var p=Q()(n+"-footer",((c={})[n+"-footer-show-ok"]=h,c));A=g.a.createElement("div",{className:p},f)}return A},e}(g.a.Component);lt.propTypes={prefixCls:p.a.string,showDateInput:p.a.bool,disabledTime:p.a.any,timePicker:p.a.element,selectedValue:p.a.any,showOk:p.a.bool,onSelect:p.a.func,value:p.a.object,renderFooter:p.a.func,defaultValue:p.a.object,mode:p.a.string};var ct=lt;function ut(t){return t?x(t):v()()}var gt={value:p.a.object,defaultValue:p.a.object,onKeyDown:p.a.func},ht={onKeyDown:function(){}};function dt(){}var ft={className:p.a.string,locale:p.a.object,style:p.a.object,visible:p.a.bool,onSelect:p.a.func,prefixCls:p.a.string,onChange:p.a.func,onOk:p.a.func},pt={locale:{today:"Today",now:"Now",backToToday:"Back to today",ok:"Ok",clear:"Clear",month:"Month",year:"Year",timeSelect:"select time",dateSelect:"select date",weekSelect:"Choose a week",monthSelect:"Choose a month",yearSelect:"Choose a year",decadeSelect:"Choose a decade",yearFormat:"YYYY",dateFormat:"M/D/YYYY",dayFormat:"D",dateTimeFormat:"M/D/YYYY HH:mm:ss",monthBeforeYear:!0,previousMonth:"Previous month (PageUp)",nextMonth:"Next month (PageDown)",previousYear:"Last year (Control + left)",nextYear:"Next year (Control + right)",previousDecade:"Last decade",nextDecade:"Next decade",previousCentury:"Last century",nextCentury:"Next century"},style:{},visible:!0,prefixCls:"rc-calendar",className:"",onSelect:dt,onChange:dt,onClear:dt,renderFooter:function(){return null},renderSidebar:function(){return null}},It=void 0,Ct=void 0,mt=void 0,yt=function(t){function e(n){a()(this,e);var r=A()(this,t.call(this,n));Et.call(r);var i=n.selectedValue;return r.state={str:S(i,r.props.format),invalid:!1,hasFocus:!1},r}return c()(e,t),e.prototype.componentDidUpdate=function(){!mt||!this.state.hasFocus||this.state.invalid||0===It&&0===Ct||mt.setSelectionRange(It,Ct)},e.getDerivedStateFromProps=function(t,e){var n={};mt&&(It=mt.selectionStart,Ct=mt.selectionEnd);var r=t.selectedValue;return e.hasFocus||(n={str:S(r,t.format),invalid:!1}),n},e.getInstance=function(){return mt},e.prototype.render=function(){var t=this.props,e=this.state,n=e.invalid,r=e.str,i=t.locale,o=t.prefixCls,a=t.placeholder,s=t.clearIcon,A=t.inputMode,l=n?o+"-input-invalid":"";return g.a.createElement("div",{className:o+"-input-wrap"},g.a.createElement("div",{className:o+"-date-input-wrap"},g.a.createElement("input",{ref:this.saveDateInput,className:o+"-input "+l,value:r,disabled:t.disabled,placeholder:a,onChange:this.onInputChange,onKeyDown:this.onKeyDown,onFocus:this.onFocus,onBlur:this.onBlur,inputMode:A})),t.showClear?g.a.createElement("a",{role:"button",title:i.clear,onClick:this.onClear},s||g.a.createElement("span",{className:o+"-clear-btn"})):null)},e}(g.a.Component);yt.propTypes={prefixCls:p.a.string,timePicker:p.a.object,value:p.a.object,disabledTime:p.a.any,format:p.a.oneOfType([p.a.string,p.a.arrayOf(p.a.string)]),locale:p.a.object,disabledDate:p.a.func,onChange:p.a.func,onClear:p.a.func,placeholder:p.a.string,onSelect:p.a.func,selectedValue:p.a.object,clearIcon:p.a.node,inputMode:p.a.string};var Et=function(){var t=this;this.onClear=function(){t.setState({str:""}),t.props.onClear(null)},this.onInputChange=function(e){var n=e.target.value,r=t.props,i=r.disabledDate,o=r.format,a=r.onChange,s=r.selectedValue;if(!n)return a(null),void t.setState({invalid:!1,str:n});var A=v()(n,o,!0);if(A.isValid()){var l=t.props.value.clone();l.year(A.year()).month(A.month()).date(A.date()).hour(A.hour()).minute(A.minute()).second(A.second()),!l||i&&i(l)?t.setState({invalid:!0,str:n}):(s!==l||s&&l&&!s.isSame(l))&&(t.setState({invalid:!1,str:n}),a(l))}else t.setState({invalid:!0,str:n})},this.onFocus=function(){t.setState({hasFocus:!0})},this.onBlur=function(){t.setState(function(t,e){return{hasFocus:!1,str:S(e.value,e.format)}})},this.onKeyDown=function(e){var n=e.keyCode,r=t.props,i=r.onSelect,o=r.value,a=r.disabledDate;n===I.a.ENTER&&i&&((!a||!a(o))&&i(o.clone()),e.preventDefault())},this.getRootDOMNode=function(){return d.a.findDOMNode(t)},this.focus=function(){mt&&mt.focus()},this.saveDateInput=function(t){mt=t}};Object(C.polyfill)(yt);var vt=yt;function Bt(){}var bt=function(t){return!(!v.a.isMoment(t)||!t.isValid())&&t},Qt=function(t){function e(n){a()(this,e);var r=A()(this,t.call(this,n));return wt.call(r),r.state={mode:r.props.mode||"date",value:bt(n.value)||bt(n.defaultValue)||v()(),selectedValue:n.selectedValue||n.defaultSelectedValue},r}return c()(e,t),e.prototype.componentDidMount=function(){this.props.showDateInput&&this.saveFocusElement(vt.getInstance())},e.getDerivedStateFromProps=function(t,e){var n=t.value,r=t.selectedValue,i={};return"mode"in t&&e.mode!==t.mode&&(i={mode:t.mode}),"value"in t&&(i.value=bt(n)||bt(t.defaultValue)||ut(e.value)),"selectedValue"in t&&(i.selectedValue=r),i},e.prototype.render=function(){var t=this.props,e=this.state,n=t.locale,r=t.prefixCls,o=t.disabledDate,a=t.dateInputPlaceholder,s=t.timePicker,A=t.disabledTime,l=t.clearIcon,c=t.renderFooter,u=t.inputMode,h=e.value,d=e.selectedValue,f=e.mode,p="time"===f,I=p&&A&&s?M(d,A):null,C=null;if(s&&p){var m=i()({showHour:!0,showSecond:!0,showMinute:!0},s.props,I,{onChange:this.onDateInputChange,value:d,disabledTime:A});void 0!==s.props.defaultValue&&(m.defaultOpenValue=s.props.defaultValue),C=g.a.cloneElement(s,m)}var y=t.showDateInput?g.a.createElement(vt,{format:this.getFormat(),key:"date-input",value:h,locale:n,placeholder:a,showClear:!0,disabledTime:A,disabledDate:o,onClear:this.onClear,prefixCls:r,selectedValue:d,onChange:this.onDateInputChange,onSelect:this.onDateInputSelect,clearIcon:l,inputMode:u}):null,E=[];return t.renderSidebar&&E.push(t.renderSidebar()),E.push(g.a.createElement("div",{className:r+"-panel",key:"panel"},y,g.a.createElement("div",{tabIndex:this.props.focusablePanel?0:void 0,className:r+"-date-panel"},g.a.createElement(ot,{locale:n,mode:f,value:h,onValueChange:this.setValue,onPanelChange:this.onPanelChange,renderFooter:c,showTimePicker:p,prefixCls:r}),s&&p?g.a.createElement("div",{className:r+"-time-picker"},g.a.createElement("div",{className:r+"-time-picker-panel"},C)):null,g.a.createElement("div",{className:r+"-body"},g.a.createElement(U,{locale:n,value:h,selectedValue:d,prefixCls:r,dateRender:t.dateRender,onSelect:this.onDateTableSelect,disabledDate:o,showWeekNumber:t.showWeekNumber})),g.a.createElement(ct,{showOk:t.showOk,mode:f,renderFooter:t.renderFooter,locale:n,prefixCls:r,showToday:t.showToday,disabledTime:A,showTimePicker:p,showDateInput:t.showDateInput,timePicker:s,selectedValue:d,value:h,disabledDate:o,okDisabled:!(!1===t.showOk||d&&this.isAllowedDate(d)),onOk:this.onOk,onSelect:this.onSelect,onToday:this.onToday,onOpenTimePicker:this.openTimePicker,onCloseTimePicker:this.closeTimePicker})))),this.renderRoot({children:E,className:t.showWeekNumber?r+"-week-number":""})},e}(g.a.Component);Qt.propTypes=i()({},gt,ft,{prefixCls:p.a.string,className:p.a.string,style:p.a.object,defaultValue:p.a.object,value:p.a.object,selectedValue:p.a.object,defaultSelectedValue:p.a.object,mode:p.a.oneOf(["time","date","month","year","decade"]),locale:p.a.object,showDateInput:p.a.bool,showWeekNumber:p.a.bool,showToday:p.a.bool,showOk:p.a.bool,onSelect:p.a.func,onOk:p.a.func,onKeyDown:p.a.func,timePicker:p.a.element,dateInputPlaceholder:p.a.any,onClear:p.a.func,onChange:p.a.func,onPanelChange:p.a.func,disabledDate:p.a.func,disabledTime:p.a.any,dateRender:p.a.func,renderFooter:p.a.func,renderSidebar:p.a.func,clearIcon:p.a.node,focusablePanel:p.a.bool,inputMode:p.a.string,onBlur:p.a.func}),Qt.defaultProps=i()({},ht,pt,{showToday:!0,showDateInput:!0,timePicker:null,onOk:Bt,onPanelChange:Bt,focusablePanel:!0});var wt=function(){var t=this;this.onPanelChange=function(e,n){var r=t.props,i=t.state;"mode"in r||t.setState({mode:n}),r.onPanelChange(e||i.value,n)},this.onKeyDown=function(e){if("input"!==e.target.nodeName.toLowerCase()){var n=e.keyCode,r=e.ctrlKey||e.metaKey,i=t.props.disabledDate,o=t.state.value;switch(n){case I.a.DOWN:return t.goTime(1,"weeks"),e.preventDefault(),1;case I.a.UP:return t.goTime(-1,"weeks"),e.preventDefault(),1;case I.a.LEFT:return r?t.goTime(-1,"years"):t.goTime(-1,"days"),e.preventDefault(),1;case I.a.RIGHT:return r?t.goTime(1,"years"):t.goTime(1,"days"),e.preventDefault(),1;case I.a.HOME:return t.setValue(t.state.value.clone().startOf("month")),e.preventDefault(),1;case I.a.END:return t.setValue(function(t){return t.clone().endOf("month")}(t.state.value)),e.preventDefault(),1;case I.a.PAGE_DOWN:return t.goTime(1,"month"),e.preventDefault(),1;case I.a.PAGE_UP:return t.goTime(-1,"month"),e.preventDefault(),1;case I.a.ENTER:return i&&i(o)||t.onSelect(o,{source:"keyboard"}),e.preventDefault(),1;default:return t.props.onKeyDown(e),1}}},this.onClear=function(){t.onSelect(null),t.props.onClear()},this.onOk=function(){var e=t.state.selectedValue;t.isAllowedDate(e)&&t.props.onOk(e)},this.onDateInputChange=function(e){t.onSelect(e,{source:"dateInput"})},this.onDateInputSelect=function(e){t.onSelect(e,{source:"dateInputSelect"})},this.onDateTableSelect=function(e){var n,r,i=t.props.timePicker;if(!t.state.selectedValue&&i){var o=i.props.defaultValue;o&&(n=o,r=e,v.a.isMoment(n)&&v.a.isMoment(r)&&(r.hour(n.hour()),r.minute(n.minute()),r.second(n.second()),r.millisecond(n.millisecond())))}t.onSelect(e)},this.onToday=function(){var e=x(t.state.value);t.onSelect(e,{source:"todayButton"})},this.onBlur=function(e){setTimeout(function(){var n=vt.getInstance(),r=t.rootInstance;!r||r.contains(document.activeElement)||n&&n.contains(document.activeElement)||t.props.onBlur&&t.props.onBlur(e)},0)},this.getRootDOMNode=function(){return d.a.findDOMNode(t)},this.openTimePicker=function(){t.onPanelChange(null,"time")},this.closeTimePicker=function(){t.onPanelChange(null,"date")},this.goTime=function(e,n){t.setValue(function(t,e,n){return t.clone().add(e,n)}(t.state.value,e,n))}};Object(C.polyfill)(Qt);var xt,_t,Dt,Mt=function(t){var e,n;return n=e=function(e){function n(){var t,r,i;a()(this,n);for(var o=arguments.length,s=Array(o),l=0;l<o;l++)s[l]=arguments[l];return t=r=A()(this,e.call.apply(e,[this].concat(s))),r.onSelect=function(t,e){t&&r.setValue(t),r.setSelectedValue(t,e)},r.renderRoot=function(t){var e,n=r.props,i=n.prefixCls,o=((e={})[i]=1,e[i+"-hidden"]=!n.visible,e[n.className]=!!n.className,e[t.className]=!!t.className,e);return g.a.createElement("div",{ref:r.saveRoot,className:""+Q()(o),style:r.props.style,tabIndex:"0",onKeyDown:r.onKeyDown,onBlur:r.onBlur},t.children)},r.setSelectedValue=function(t,e){"selectedValue"in r.props||r.setState({selectedValue:t}),r.props.onSelect&&r.props.onSelect(t,e)},r.setValue=function(t){var e=r.state.value;"value"in r.props||r.setState({value:t}),(e&&t&&!e.isSame(t)||!e&&t||e&&!t)&&r.props.onChange(t)},r.isAllowedDate=function(t){return k(t,r.props.disabledDate,r.props.disabledTime)},i=t,A()(r,i)}return c()(n,e),n.getDerivedStateFromProps=function(e,n){if(t.getDerivedStateFromProps)return t.getDerivedStateFromProps(e,n);var r=e.value,i=e.selectedValue,o={};return"value"in e&&(o.value=r||e.defaultValue||ut(n.value)),"selectedValue"in e&&(o.selectedValue=i),o},n}(t),e.displayName="CalendarMixinWrapper",e.defaultProps=t.defaultProps,n}((Dt=_t=function(t){function e(){var n,r,i;a()(this,e);for(var o=arguments.length,s=Array(o),l=0;l<o;l++)s[l]=arguments[l];return n=r=A()(this,t.call.apply(t,[this].concat(s))),r.getFormat=function(){var t=r.props.format,e=r.props,n=e.locale,i=e.timePicker;return t||(t=i?n.dateTimeFormat:n.dateFormat),t},r.focus=function(){r.focusElement?r.focusElement.focus():r.rootInstance&&r.rootInstance.focus()},r.saveFocusElement=function(t){r.focusElement=t},r.saveRoot=function(t){r.rootInstance=t},i=n,A()(r,i)}return c()(e,t),e.prototype.shouldComponentUpdate=function(t){return this.props.visible||t.visible},e}(xt=Qt),_t.displayName="CommonMixinWrapper",_t.defaultProps=xt.defaultProps,_t.getDerivedStateFromProps=xt.getDerivedStateFromProps,Dt));e.default=Mt},function(t,e,n){"use strict";n.r(e);var r=n(0),i=n.n(r);function o(t,e){return!e||"object"!=typeof e&&"function"!=typeof e?function(t){if(void 0===t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return t}(t):e}function a(t){return(a=Object.setPrototypeOf?Object.getPrototypeOf:function(t){return t.__proto__||Object.getPrototypeOf(t)})(t)}function s(t,e){return(s=Object.setPrototypeOf||function(t,e){return t.__proto__=e,t})(t,e)}var A=function(t){function e(){return function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}(this,e),o(this,a(e).apply(this,arguments))}return function(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function");t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,writable:!0,configurable:!0}}),e&&s(t,e)}(e,r["Component"]),e}();A.isSelectOptGroup=!0;var l=n(1),c=n.n(l);function u(t,e){return!e||"object"!=typeof e&&"function"!=typeof e?function(t){if(void 0===t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return t}(t):e}function g(t){return(g=Object.setPrototypeOf?Object.getPrototypeOf:function(t){return t.__proto__||Object.getPrototypeOf(t)})(t)}function h(t,e){return(h=Object.setPrototypeOf||function(t,e){return t.__proto__=e,t})(t,e)}var d=function(t){function e(){return function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}(this,e),u(this,g(e).apply(this,arguments))}return function(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function");t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,writable:!0,configurable:!0}}),e&&h(t,e)}(e,r["Component"]),e}();function f(t){return function(t){if(Array.isArray(t)){for(var e=0,n=new Array(t.length);e<t.length;e++)n[e]=t[e];return n}}(t)||function(t){if(Symbol.iterator in Object(t)||"[object Arguments]"===Object.prototype.toString.call(t))return Array.from(t)}(t)||function(){throw new TypeError("Invalid attempt to spread non-iterable instance")}()}function p(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];var r=e[0],i=e[1],o=e[2],a=e.slice(3),s=l.oneOfType([l.string,l.number]),A=l.shape({key:s.isRequired,label:l.node});return r.labelInValue?l.oneOfType([l.arrayOf(A),A]).apply(void 0,[r,i,o].concat(f(a)))?new Error("Invalid prop `".concat(i,"` supplied to `").concat(o,"`, ")+"when you set `labelInValue` to `true`, `".concat(i,"` should in ")+"shape of `{ key: string | number, label?: ReactNode }`."):null:("multiple"===r.mode||"tags"===r.mode||r.multiple||r.tags)&&""===r[i]?new Error("Invalid prop `".concat(i,"` of type `string` supplied to `").concat(o,"`, ")+"expected `array` when `multiple` or `tags` is `true`."):l.oneOfType([l.arrayOf(s),s]).apply(void 0,[r,i,o].concat(f(a)))}d.propTypes={value:l.oneOfType([l.string,l.number])},d.isSelectOption=!0;var I={id:l.string,defaultActiveFirstOption:l.bool,multiple:l.bool,filterOption:l.any,children:l.any,showSearch:l.bool,disabled:l.bool,allowClear:l.bool,showArrow:l.bool,tags:l.bool,prefixCls:l.string,className:l.string,transitionName:l.string,optionLabelProp:l.string,optionFilterProp:l.string,animation:l.string,choiceTransitionName:l.string,open:l.bool,defaultOpen:l.bool,onChange:l.func,onBlur:l.func,onFocus:l.func,onSelect:l.func,onSearch:l.func,onPopupScroll:l.func,onMouseEnter:l.func,onMouseLeave:l.func,onInputKeyDown:l.func,placeholder:l.any,onDeselect:l.func,labelInValue:l.bool,loading:l.bool,value:p,defaultValue:p,dropdownStyle:l.object,maxTagTextLength:l.number,maxTagCount:l.number,maxTagPlaceholder:l.oneOfType([l.node,l.func]),tokenSeparators:l.arrayOf(l.string),getInputElement:l.func,showAction:l.arrayOf(l.string),clearIcon:l.node,inputIcon:l.node,removeIcon:l.node,menuItemSelectedIcon:l.oneOfType([l.func,l.node]),dropdownRender:l.func},C=n(4),m=n.n(C),y=n(202),E=n.n(y),v=n(117),B=n(13),b=n.n(B),Q=n(57),w=n.n(Q),x=n(18),_=n.n(x),D=n(14),M=n.n(D),k=n(19),S=n.n(k),L=n(102),R=n(31);function N(){var t=[].slice.call(arguments,0);return 1===t.length?t[0]:function(){for(var e=0;e<t.length;e++)t[e]&&t[e].apply&&t[e].apply(this,arguments)}}var F=n(1269);function T(){}function U(t,e,n){var r=e||"";return t.key||r+"item_"+n}function O(t){return t+"-menu-"}function G(t,e){var n=-1;i.a.Children.forEach(t,function(t){n++,t&&t.type&&t.type.isMenuItemGroup?i.a.Children.forEach(t.props.children,function(t){e(t,++n)}):e(t,n)})}var H=["defaultSelectedKeys","selectedKeys","defaultOpenKeys","openKeys","mode","getPopupContainer","onSelect","onDeselect","onDestroy","openTransitionName","openAnimation","subMenuOpenDelay","subMenuCloseDelay","forceSubMenuRender","triggerSubMenuAction","level","selectable","multiple","onOpenChange","visible","focusable","defaultActiveFirst","prefixCls","inlineIndent","parentMenu","title","rootPrefixCls","eventKey","active","onItemHover","onTitleMouseEnter","onTitleMouseLeave","onTitleClick","popupAlign","popupOffset","isOpen","renderMenuItem","manualRef","subMenuKey","disabled","index","isSelected","store","activeKey","builtinPlacements","overflowedIndicator","attribute","value","popupClassName","inlineCollapsed","menu","theme","itemIcon","expandIcon"],j=function(t){var e=t&&"function"==typeof t.getBoundingClientRect&&t.getBoundingClientRect().width;return e&&(e=+e.toFixed(6)),e||0},Y=function(t,e,n){t&&"object"==typeof t.style&&(t.style[e]=n)},P=n(16),K=n.n(P),z=n(257),J=n(116),q={adjustX:1,adjustY:1},V={topLeft:{points:["bl","tl"],overflow:q,offset:[0,-7]},bottomLeft:{points:["tl","bl"],overflow:q,offset:[0,7]},leftTop:{points:["tr","tl"],overflow:q,offset:[-4,0]},rightTop:{points:["tl","tr"],overflow:q,offset:[4,0]}},W=0,X={horizontal:"bottomLeft",vertical:"rightTop","vertical-left":"rightTop","vertical-right":"leftTop"},Z=function(t,e,n){var r,i=O(e),o=t.getState();t.setState({defaultActiveFirst:b()({},o.defaultActiveFirst,(r={},r[i]=n,r))})},$=function(t){function e(n){_()(this,e);var r=M()(this,t.call(this,n));tt.call(r);var i=n.store,o=n.eventKey,a=i.getState().defaultActiveFirst;r.isRootMenu=!1;var s=!1;return a&&(s=a[o]),Z(i,o,s),r}return S()(e,t),e.prototype.componentDidMount=function(){this.componentDidUpdate()},e.prototype.componentDidUpdate=function(){var t=this,e=this.props,n=e.mode,r=e.parentMenu,i=e.manualRef;i&&i(this),"horizontal"===n&&r.isRootMenu&&this.props.isOpen&&(this.minWidthTimeout=setTimeout(function(){return t.adjustWidth()},0))},e.prototype.componentWillUnmount=function(){var t=this.props,e=t.onDestroy,n=t.eventKey;e&&e(n),this.minWidthTimeout&&clearTimeout(this.minWidthTimeout),this.mouseenterTimeout&&clearTimeout(this.mouseenterTimeout)},e.prototype.renderChildren=function(t){var e=this.props,n={mode:"horizontal"===e.mode?"vertical":e.mode,visible:this.props.isOpen,level:e.level+1,inlineIndent:e.inlineIndent,focusable:!1,onClick:this.onSubMenuClick,onSelect:this.onSelect,onDeselect:this.onDeselect,onDestroy:this.onDestroy,selectedKeys:e.selectedKeys,eventKey:e.eventKey+"-menu-",openKeys:e.openKeys,openTransitionName:e.openTransitionName,openAnimation:e.openAnimation,onOpenChange:this.onOpenChange,subMenuOpenDelay:e.subMenuOpenDelay,parentMenu:this,subMenuCloseDelay:e.subMenuCloseDelay,forceSubMenuRender:e.forceSubMenuRender,triggerSubMenuAction:e.triggerSubMenuAction,builtinPlacements:e.builtinPlacements,defaultActiveFirst:e.store.getState().defaultActiveFirst[O(e.eventKey)],multiple:e.multiple,prefixCls:e.rootPrefixCls,id:this._menuId,manualRef:this.saveMenuInstance,itemIcon:e.itemIcon,expandIcon:e.expandIcon},r=this.haveRendered;if(this.haveRendered=!0,this.haveOpened=this.haveOpened||n.visible||n.forceSubMenuRender,!this.haveOpened)return i.a.createElement("div",null);var o=r||!n.visible||"inline"!==n.mode;n.className=" "+n.prefixCls+"-sub";var a={};return n.openTransitionName?a.transitionName=n.openTransitionName:"object"==typeof n.openAnimation&&(a.animation=b()({},n.openAnimation),o||delete a.animation.appear),i.a.createElement(v.a,b()({},a,{showProp:"visible",component:"",transitionAppear:o}),i.a.createElement(dt,b()({},n,{id:this._menuId}),t))},e.prototype.render=function(){var t,e=b()({},this.props),n=e.isOpen,r=this.getPrefixCls(),o="inline"===e.mode,a=m()(r,r+"-"+e.mode,((t={})[e.className]=!!e.className,t[this.getOpenClassName()]=n,t[this.getActiveClassName()]=e.active||n&&!o,t[this.getDisabledClassName()]=e.disabled,t[this.getSelectedClassName()]=this.isChildrenSelected(),t));this._menuId||(e.eventKey?this._menuId=e.eventKey+"$Menu":this._menuId="$__$"+ ++W+"$Menu");var s={},A={},l={};e.disabled||(s={onMouseLeave:this.onMouseLeave,onMouseEnter:this.onMouseEnter},A={onClick:this.onTitleClick},l={onMouseEnter:this.onTitleMouseEnter,onMouseLeave:this.onTitleMouseLeave});var c={};o&&(c.paddingLeft=e.inlineIndent*e.level);var u={};this.props.isOpen&&(u={"aria-owns":this._menuId});var g=null;"horizontal"!==e.mode&&(g=this.props.expandIcon,"function"==typeof this.props.expandIcon&&(g=i.a.createElement(this.props.expandIcon,b()({},this.props))));var h=i.a.createElement("div",b()({ref:this.saveSubMenuTitle,style:c,className:r+"-title"},l,A,{"aria-expanded":n},u,{"aria-haspopup":"true",title:"string"==typeof e.title?e.title:void 0}),e.title,g||i.a.createElement("i",{className:r+"-arrow"})),d=this.renderChildren(e.children),f=e.parentMenu.isRootMenu?e.parentMenu.props.getPopupContainer:function(t){return t.parentNode},p=X[e.mode],I=e.popupOffset?{offset:e.popupOffset}:{},C="inline"===e.mode?"":e.popupClassName,y=e.disabled,E=e.triggerSubMenuAction,v=e.subMenuOpenDelay,B=e.forceSubMenuRender,Q=e.subMenuCloseDelay,w=e.builtinPlacements;return H.forEach(function(t){return delete e[t]}),delete e.onClick,i.a.createElement("li",b()({},e,s,{className:a,role:"menuitem"}),o&&h,o&&d,!o&&i.a.createElement(J.default,{prefixCls:r,popupClassName:r+"-popup "+C,getPopupContainer:f,builtinPlacements:b()({},V,w),popupPlacement:p,popupVisible:n,popupAlign:I,popup:d,action:y?[]:[E],mouseEnterDelay:v,mouseLeaveDelay:Q,onPopupVisibleChange:this.onPopupVisibleChange,forceRender:B},h))},e}(i.a.Component);$.propTypes={parentMenu:c.a.object,title:c.a.node,children:c.a.any,selectedKeys:c.a.array,openKeys:c.a.array,onClick:c.a.func,onOpenChange:c.a.func,rootPrefixCls:c.a.string,eventKey:c.a.string,multiple:c.a.bool,active:c.a.bool,onItemHover:c.a.func,onSelect:c.a.func,triggerSubMenuAction:c.a.string,onDeselect:c.a.func,onDestroy:c.a.func,onMouseEnter:c.a.func,onMouseLeave:c.a.func,onTitleMouseEnter:c.a.func,onTitleMouseLeave:c.a.func,onTitleClick:c.a.func,popupOffset:c.a.array,isOpen:c.a.bool,store:c.a.object,mode:c.a.oneOf(["horizontal","vertical","vertical-left","vertical-right","inline"]),manualRef:c.a.func,itemIcon:c.a.oneOfType([c.a.func,c.a.node]),expandIcon:c.a.oneOfType([c.a.func,c.a.node])},$.defaultProps={onMouseEnter:T,onMouseLeave:T,onTitleMouseEnter:T,onTitleMouseLeave:T,onTitleClick:T,manualRef:T,mode:"vertical",title:""};var tt=function(){var t=this;this.onDestroy=function(e){t.props.onDestroy(e)},this.onKeyDown=function(e){var n=e.keyCode,r=t.menuInstance,i=t.props,o=i.isOpen,a=i.store;if(n===R.a.ENTER)return t.onTitleClick(e),Z(a,t.props.eventKey,!0),!0;if(n===R.a.RIGHT)return o?r.onKeyDown(e):(t.triggerOpenChange(!0),Z(a,t.props.eventKey,!0)),!0;if(n===R.a.LEFT){var s=void 0;if(!o)return;return(s=r.onKeyDown(e))||(t.triggerOpenChange(!1),s=!0),s}return!o||n!==R.a.UP&&n!==R.a.DOWN?void 0:r.onKeyDown(e)},this.onOpenChange=function(e){t.props.onOpenChange(e)},this.onPopupVisibleChange=function(e){t.triggerOpenChange(e,e?"mouseenter":"mouseleave")},this.onMouseEnter=function(e){var n=t.props,r=n.eventKey,i=n.onMouseEnter,o=n.store;Z(o,t.props.eventKey,!1),i({key:r,domEvent:e})},this.onMouseLeave=function(e){var n=t.props,r=n.parentMenu,i=n.eventKey,o=n.onMouseLeave;r.subMenuInstance=t,o({key:i,domEvent:e})},this.onTitleMouseEnter=function(e){var n=t.props,r=n.eventKey,i=n.onItemHover,o=n.onTitleMouseEnter;i({key:r,hover:!0}),o({key:r,domEvent:e})},this.onTitleMouseLeave=function(e){var n=t.props,r=n.parentMenu,i=n.eventKey,o=n.onItemHover,a=n.onTitleMouseLeave;r.subMenuInstance=t,o({key:i,hover:!1}),a({key:i,domEvent:e})},this.onTitleClick=function(e){var n=t.props;n.onTitleClick({key:n.eventKey,domEvent:e}),"hover"!==n.triggerSubMenuAction&&(t.triggerOpenChange(!n.isOpen,"click"),Z(n.store,t.props.eventKey,!1))},this.onSubMenuClick=function(e){"function"==typeof t.props.onClick&&t.props.onClick(t.addKeyPath(e))},this.onSelect=function(e){t.props.onSelect(e)},this.onDeselect=function(e){t.props.onDeselect(e)},this.getPrefixCls=function(){return t.props.rootPrefixCls+"-submenu"},this.getActiveClassName=function(){return t.getPrefixCls()+"-active"},this.getDisabledClassName=function(){return t.getPrefixCls()+"-disabled"},this.getSelectedClassName=function(){return t.getPrefixCls()+"-selected"},this.getOpenClassName=function(){return t.props.rootPrefixCls+"-submenu-open"},this.saveMenuInstance=function(e){t.menuInstance=e},this.addKeyPath=function(e){return b()({},e,{keyPath:(e.keyPath||[]).concat(t.props.eventKey)})},this.triggerOpenChange=function(e,n){var r=t.props.eventKey,i=function(){t.onOpenChange({key:r,item:t,trigger:n,open:e})};"mouseenter"===n?t.mouseenterTimeout=setTimeout(function(){i()},0):i()},this.isChildrenSelected=function(){var e={find:!1};return function t(e,n,r){e&&!r.find&&i.a.Children.forEach(e,function(e){if(e){var i=e.type;if(!i||!(i.isSubMenu||i.isMenuItem||i.isMenuItemGroup))return;-1!==n.indexOf(e.key)?r.find=!0:e.props.children&&t(e.props.children,n,r)}})}(t.props.children,t.props.selectedKeys,e),e.find},this.isOpen=function(){return-1!==t.props.openKeys.indexOf(t.props.eventKey)},this.adjustWidth=function(){if(t.subMenuTitle&&t.menuInstance){var e=K.a.findDOMNode(t.menuInstance);e.offsetWidth>=t.subMenuTitle.offsetWidth||(e.style.minWidth=t.subMenuTitle.offsetWidth+"px")}},this.saveSubMenuTitle=function(e){t.subMenuTitle=e}},et=Object(L.connect)(function(t,e){var n=t.openKeys,r=t.activeKey,i=t.selectedKeys,o=e.eventKey,a=e.subMenuKey;return{isOpen:n.indexOf(o)>-1,active:r[a]===o,selectedKeys:i}})($);et.isSubMenu=!0;var nt=et,rt=!("undefined"==typeof window||!window.document||!window.document.createElement),it="menuitem-overflowed",ot=.5;rt&&n(1270);var at=function(t){function e(){var n,r,o;_()(this,e);for(var a=arguments.length,s=Array(a),A=0;A<a;A++)s[A]=arguments[A];return n=r=M()(this,t.call.apply(t,[this].concat(s))),r.state={lastVisibleIndex:void 0},r.getMenuItemNodes=function(){var t=r.props.prefixCls,e=K.a.findDOMNode(r);return e?[].slice.call(e.children).filter(function(e){return e.className.split(" ").indexOf(t+"-overflowed-submenu")<0}):[]},r.getOverflowedSubMenuItem=function(t,e,n){var o=r.props,a=o.overflowedIndicator,s=o.level,A=o.mode,l=o.prefixCls,c=o.theme;if(1!==s||"horizontal"!==A)return null;var u=r.props.children[0].props,g=(u.children,u.title,u.style),h=w()(u,["children","title","style"]),d=b()({},g),f=t+"-overflowed-indicator",p=t+"-overflowed-indicator";0===e.length&&!0!==n?d=b()({},d,{display:"none"}):n&&(d=b()({},d,{visibility:"hidden",position:"absolute"}),f+="-placeholder",p+="-placeholder");var I=c?l+"-"+c:"",C={};return H.forEach(function(t){void 0!==h[t]&&(C[t]=h[t])}),i.a.createElement(nt,b()({title:a,className:l+"-overflowed-submenu",popupClassName:I},C,{key:f,eventKey:p,disabled:!1,style:d}),e)},r.setChildrenWidthAndResize=function(){if("horizontal"===r.props.mode){var t=K.a.findDOMNode(r);if(t){var e=t.children;if(e&&0!==e.length){var n=t.children[e.length-1];Y(n,"display","inline-block");var i=r.getMenuItemNodes(),o=i.filter(function(t){return t.className.split(" ").indexOf(it)>=0});o.forEach(function(t){Y(t,"display","inline-block")}),r.menuItemSizes=i.map(function(t){return j(t)}),o.forEach(function(t){Y(t,"display","none")}),r.overflowedIndicatorWidth=j(t.children[t.children.length-1]),r.originalTotalWidth=r.menuItemSizes.reduce(function(t,e){return t+e},0),r.handleResize(),Y(n,"display","none")}}}},r.resizeObserver=null,r.mutationObserver=null,r.originalTotalWidth=0,r.overflowedItems=[],r.menuItemSizes=[],r.handleResize=function(){if("horizontal"===r.props.mode){var t=K.a.findDOMNode(r);if(t){var e=j(t);r.overflowedItems=[];var n=0,i=void 0;r.originalTotalWidth>e+ot&&(i=-1,r.menuItemSizes.forEach(function(t){(n+=t)+r.overflowedIndicatorWidth<=e&&i++})),r.setState({lastVisibleIndex:i})}}},o=n,M()(r,o)}return S()(e,t),e.prototype.componentDidMount=function(){var t=this;if(this.setChildrenWidthAndResize(),1===this.props.level&&"horizontal"===this.props.mode){var e=K.a.findDOMNode(this);if(!e)return;this.resizeObserver=new z.a(function(e){e.forEach(t.setChildrenWidthAndResize)}),[].slice.call(e.children).concat(e).forEach(function(e){t.resizeObserver.observe(e)}),"undefined"!=typeof MutationObserver&&(this.mutationObserver=new MutationObserver(function(){t.resizeObserver.disconnect(),[].slice.call(e.children).concat(e).forEach(function(e){t.resizeObserver.observe(e)}),t.setChildrenWidthAndResize()}),this.mutationObserver.observe(e,{attributes:!1,childList:!0,subTree:!1}))}},e.prototype.componentWillUnmount=function(){this.resizeObserver&&this.resizeObserver.disconnect(),this.mutationObserver&&this.resizeObserver.disconnect()},e.prototype.renderChildren=function(t){var e=this,n=this.state.lastVisibleIndex;return(t||[]).reduce(function(r,o,a){var s=o;if("horizontal"===e.props.mode){var A=e.getOverflowedSubMenuItem(o.props.eventKey,[]);void 0!==n&&-1!==e.props.className.indexOf(e.props.prefixCls+"-root")&&(a>n&&(s=i.a.cloneElement(o,{style:{display:"none"},eventKey:o.props.eventKey+"-hidden",className:o.className+" "+it})),a===n+1&&(e.overflowedItems=t.slice(n+1).map(function(t){return i.a.cloneElement(t,{key:t.props.eventKey,mode:"vertical-left"})}),A=e.getOverflowedSubMenuItem(o.props.eventKey,e.overflowedItems)));var l=[].concat(r,[A,s]);return a===t.length-1&&l.push(e.getOverflowedSubMenuItem(o.props.eventKey,[],!0)),l}return[].concat(r,[s])},[])},e.prototype.render=function(){var t=this.props,e=t.hiddenClassName,n=t.visible,r=(t.prefixCls,t.overflowedIndicator,t.mode,t.level,t.tag),o=(t.children,t.theme,w()(t,["hiddenClassName","visible","prefixCls","overflowedIndicator","mode","level","tag","children","theme"]));return n||(o.className+=" "+e),i.a.createElement(r,o,this.renderChildren(this.props.children))},e}(i.a.Component);at.propTypes={className:c.a.string,children:c.a.node,mode:c.a.oneOf(["horizontal","vertical","vertical-left","vertical-right","inline"]),prefixCls:c.a.string,level:c.a.number,theme:c.a.string,overflowedIndicator:c.a.node,visible:c.a.bool,hiddenClassName:c.a.string,tag:c.a.string,style:c.a.object},at.defaultProps={tag:"div",className:""};var st=at;function At(t,e,n){var r,i=t.getState();t.setState({activeKey:b()({},i.activeKey,(r={},r[e]=n,r))})}function lt(t){return t.eventKey||"0-menu-"}function ct(t,e){var n=e,r=t.children,i=t.eventKey;if(n){var o=void 0;if(G(r,function(t,e){t&&t.props&&!t.props.disabled&&n===U(t,i,e)&&(o=!0)}),o)return n}return n=null,t.defaultActiveFirst?(G(r,function(t,e){n||!t||t.props.disabled||(n=U(t,i,e))}),n):n}function ut(t){if(t){var e=this.instanceArray.indexOf(t);-1!==e?this.instanceArray[e]=t:this.instanceArray.push(t)}}var gt=function(t){function e(n){var r;_()(this,e);var i=M()(this,t.call(this,n));return ht.call(i),n.store.setState({activeKey:b()({},n.store.getState().activeKey,(r={},r[n.eventKey]=ct(n,n.activeKey),r))}),i.instanceArray=[],i}return S()(e,t),e.prototype.componentDidMount=function(){this.props.manualRef&&this.props.manualRef(this)},e.prototype.shouldComponentUpdate=function(t){return this.props.visible||t.visible},e.prototype.componentDidUpdate=function(t){var e=this.props,n="activeKey"in e?e.activeKey:e.store.getState().activeKey[lt(e)],r=ct(e,n);if(r!==n)At(e.store,lt(e),r);else if("activeKey"in t){r!==ct(t,t.activeKey)&&At(e.store,lt(e),r)}},e.prototype.render=function(){var t=this,e=w()(this.props,[]);this.instanceArray=[];var n={className:m()(e.prefixCls,e.className,e.prefixCls+"-"+e.mode),role:e.role||"menu"};e.id&&(n.id=e.id),e.focusable&&(n.tabIndex="0",n.onKeyDown=this.onKeyDown);var r=e.prefixCls,o=e.eventKey,a=e.visible,s=e.level,A=e.mode,l=e.overflowedIndicator,c=e.theme;return H.forEach(function(t){return delete e[t]}),delete e.onClick,i.a.createElement(st,b()({},e,{prefixCls:r,mode:A,tag:"ul",level:s,theme:c,hiddenClassName:r+"-hidden",visible:a,overflowedIndicator:l},n),i.a.Children.map(e.children,function(e,n){return t.renderMenuItem(e,n,o||"0-menu-")}))},e}(i.a.Component);gt.propTypes={onSelect:c.a.func,onClick:c.a.func,onDeselect:c.a.func,onOpenChange:c.a.func,onDestroy:c.a.func,openTransitionName:c.a.string,openAnimation:c.a.oneOfType([c.a.string,c.a.object]),openKeys:c.a.arrayOf(c.a.string),visible:c.a.bool,children:c.a.any,parentMenu:c.a.object,eventKey:c.a.string,store:c.a.shape({getState:c.a.func,setState:c.a.func}),focusable:c.a.bool,multiple:c.a.bool,style:c.a.object,defaultActiveFirst:c.a.bool,activeKey:c.a.string,selectedKeys:c.a.arrayOf(c.a.string),defaultSelectedKeys:c.a.arrayOf(c.a.string),defaultOpenKeys:c.a.arrayOf(c.a.string),level:c.a.number,mode:c.a.oneOf(["horizontal","vertical","vertical-left","vertical-right","inline"]),triggerSubMenuAction:c.a.oneOf(["click","hover"]),inlineIndent:c.a.oneOfType([c.a.number,c.a.string]),manualRef:c.a.func,itemIcon:c.a.oneOfType([c.a.func,c.a.node]),expandIcon:c.a.oneOfType([c.a.func,c.a.node])},gt.defaultProps={prefixCls:"rc-menu",className:"",mode:"vertical",level:1,inlineIndent:24,visible:!0,focusable:!0,style:{},manualRef:T};var ht=function(){var t=this;this.onKeyDown=function(e,n){var r=e.keyCode,i=void 0;if(t.getFlatInstanceArray().forEach(function(t){t&&t.props.active&&t.onKeyDown&&(i=t.onKeyDown(e))}),i)return 1;var o=null;return r!==R.a.UP&&r!==R.a.DOWN||(o=t.step(r===R.a.UP?-1:1)),o?(e.preventDefault(),At(t.props.store,lt(t.props),o.props.eventKey),"function"==typeof n&&n(o),1):void 0},this.onItemHover=function(e){var n=e.key,r=e.hover;At(t.props.store,lt(t.props),r?n:null)},this.onDeselect=function(e){t.props.onDeselect(e)},this.onSelect=function(e){t.props.onSelect(e)},this.onClick=function(e){t.props.onClick(e)},this.onOpenChange=function(e){t.props.onOpenChange(e)},this.onDestroy=function(e){t.props.onDestroy(e)},this.getFlatInstanceArray=function(){return t.instanceArray},this.getOpenTransitionName=function(){return t.props.openTransitionName},this.step=function(e){var n=t.getFlatInstanceArray(),r=t.props.store.getState().activeKey[lt(t.props)],i=n.length;if(!i)return null;e<0&&(n=n.concat().reverse());var o=-1;if(n.every(function(t,e){return!t||t.props.eventKey!==r||(o=e,!1)}),t.props.defaultActiveFirst||-1===o||(a=n.slice(o,i-1)).length&&!a.every(function(t){return!!t.props.disabled})){var a,s=(o+1)%i,A=s;do{var l=n[A];if(l&&!l.props.disabled)return l;A=(A+1)%i}while(A!==s);return null}},this.renderCommonMenuItem=function(e,n,r){var o=t.props.store.getState(),a=t.props,s=U(e,a.eventKey,n),A=e.props;if(!A||"string"==typeof e.type)return e;var l=s===o.activeKey,c=b()({mode:A.mode||a.mode,level:a.level,inlineIndent:a.inlineIndent,renderMenuItem:t.renderMenuItem,rootPrefixCls:a.prefixCls,index:n,parentMenu:a.parentMenu,manualRef:A.disabled?void 0:N(e.ref,ut.bind(t)),eventKey:s,active:!A.disabled&&l,multiple:a.multiple,onClick:function(e){(A.onClick||T)(e),t.onClick(e)},onItemHover:t.onItemHover,openTransitionName:t.getOpenTransitionName(),openAnimation:a.openAnimation,subMenuOpenDelay:a.subMenuOpenDelay,subMenuCloseDelay:a.subMenuCloseDelay,forceSubMenuRender:a.forceSubMenuRender,onOpenChange:t.onOpenChange,onDeselect:t.onDeselect,onSelect:t.onSelect,builtinPlacements:a.builtinPlacements,itemIcon:A.itemIcon||t.props.itemIcon,expandIcon:A.expandIcon||t.props.expandIcon},r);return("inline"===a.mode||F.any)&&(c.triggerSubMenuAction="click"),i.a.cloneElement(e,c)},this.renderMenuItem=function(e,n,r){if(!e)return null;var i=t.props.store.getState(),o={openKeys:i.openKeys,selectedKeys:i.selectedKeys,triggerSubMenuAction:t.props.triggerSubMenuAction,subMenuKey:r};return t.renderCommonMenuItem(e,n,o)}},dt=Object(L.connect)()(gt),ft=function(t){function e(n){_()(this,e);var r=M()(this,t.call(this,n));pt.call(r),r.isRootMenu=!0;var i=n.defaultSelectedKeys,o=n.defaultOpenKeys;return"selectedKeys"in n&&(i=n.selectedKeys||[]),"openKeys"in n&&(o=n.openKeys||[]),r.store=Object(L.create)({selectedKeys:i,openKeys:o,activeKey:{"0-menu-":ct(n,n.activeKey)}}),r}return S()(e,t),e.prototype.componentDidMount=function(){this.updateMiniStore()},e.prototype.componentDidUpdate=function(){this.updateMiniStore()},e.prototype.updateMiniStore=function(){"selectedKeys"in this.props&&this.store.setState({selectedKeys:this.props.selectedKeys||[]}),"openKeys"in this.props&&this.store.setState({openKeys:this.props.openKeys||[]})},e.prototype.render=function(){var t=this,e=w()(this.props,[]);return e.className+=" "+e.prefixCls+"-root",e=b()({},e,{onClick:this.onClick,onOpenChange:this.onOpenChange,onDeselect:this.onDeselect,onSelect:this.onSelect,openTransitionName:this.getOpenTransitionName(),parentMenu:this}),i.a.createElement(L.Provider,{store:this.store},i.a.createElement(dt,b()({},e,{ref:function(e){return t.innerMenu=e}}),this.props.children))},e}(i.a.Component);ft.propTypes={defaultSelectedKeys:c.a.arrayOf(c.a.string),defaultActiveFirst:c.a.bool,selectedKeys:c.a.arrayOf(c.a.string),defaultOpenKeys:c.a.arrayOf(c.a.string),openKeys:c.a.arrayOf(c.a.string),mode:c.a.oneOf(["horizontal","vertical","vertical-left","vertical-right","inline"]),getPopupContainer:c.a.func,onClick:c.a.func,onSelect:c.a.func,onDeselect:c.a.func,onDestroy:c.a.func,openTransitionName:c.a.string,openAnimation:c.a.oneOfType([c.a.string,c.a.object]),subMenuOpenDelay:c.a.number,subMenuCloseDelay:c.a.number,forceSubMenuRender:c.a.bool,triggerSubMenuAction:c.a.string,level:c.a.number,selectable:c.a.bool,multiple:c.a.bool,children:c.a.any,className:c.a.string,style:c.a.object,activeKey:c.a.string,prefixCls:c.a.string,builtinPlacements:c.a.object,itemIcon:c.a.oneOfType([c.a.func,c.a.node]),expandIcon:c.a.oneOfType([c.a.func,c.a.node]),overflowedIndicator:c.a.node},ft.defaultProps={selectable:!0,onClick:T,onSelect:T,onOpenChange:T,onDeselect:T,defaultSelectedKeys:[],defaultOpenKeys:[],subMenuOpenDelay:.1,subMenuCloseDelay:.1,triggerSubMenuAction:"hover",prefixCls:"rc-menu",className:"",mode:"vertical",style:{},builtinPlacements:{},overflowedIndicator:i.a.createElement("span",null,"···")};var pt=function(){var t=this;this.onSelect=function(e){var n=t.props;if(n.selectable){var r=t.store.getState().selectedKeys,i=e.key;r=n.multiple?r.concat([i]):[i],"selectedKeys"in n||t.store.setState({selectedKeys:r}),n.onSelect(b()({},e,{selectedKeys:r}))}},this.onClick=function(e){t.props.onClick(e)},this.onKeyDown=function(e,n){t.innerMenu.getWrappedInstance().onKeyDown(e,n)},this.onOpenChange=function(e){var n=t.props,r=t.store.getState().openKeys.concat(),i=!1,o=function(t){var e=!1;if(t.open)(e=-1===r.indexOf(t.key))&&r.push(t.key);else{var n=r.indexOf(t.key);(e=-1!==n)&&r.splice(n,1)}i=i||e};Array.isArray(e)?e.forEach(o):o(e),i&&("openKeys"in t.props||t.store.setState({openKeys:r}),n.onOpenChange(r))},this.onDeselect=function(e){var n=t.props;if(n.selectable){var r=t.store.getState().selectedKeys.concat(),i=e.key,o=r.indexOf(i);-1!==o&&r.splice(o,1),"selectedKeys"in n||t.store.setState({selectedKeys:r}),n.onDeselect(b()({},e,{selectedKeys:r}))}},this.getOpenTransitionName=function(){var e=t.props,n=e.openTransitionName,r=e.openAnimation;return n||"string"!=typeof r||(n=e.prefixCls+"-open-"+r),n}},It=ft,Ct=n(263),mt=n.n(Ct),yt=function(t){function e(n){_()(this,e);var r=M()(this,t.call(this,n));return r.onKeyDown=function(t){if(t.keyCode===R.a.ENTER)return r.onClick(t),!0},r.onMouseLeave=function(t){var e=r.props,n=e.eventKey,i=e.onItemHover,o=e.onMouseLeave;i({key:n,hover:!1}),o({key:n,domEvent:t})},r.onMouseEnter=function(t){var e=r.props,n=e.eventKey,i=e.onItemHover,o=e.onMouseEnter;i({key:n,hover:!0}),o({key:n,domEvent:t})},r.onClick=function(t){var e=r.props,n=e.eventKey,i=e.multiple,o=e.onClick,a=e.onSelect,s=e.onDeselect,A=e.isSelected,l={key:n,keyPath:[n],item:r,domEvent:t};o(l),i?A?s(l):a(l):A||a(l)},r}return S()(e,t),e.prototype.componentDidMount=function(){this.callRef()},e.prototype.componentDidUpdate=function(){this.props.active&&mt()(K.a.findDOMNode(this),K.a.findDOMNode(this.props.parentMenu),{onlyScrollIfNeeded:!0}),this.callRef()},e.prototype.componentWillUnmount=function(){var t=this.props;t.onDestroy&&t.onDestroy(t.eventKey)},e.prototype.getPrefixCls=function(){return this.props.rootPrefixCls+"-item"},e.prototype.getActiveClassName=function(){return this.getPrefixCls()+"-active"},e.prototype.getSelectedClassName=function(){return this.getPrefixCls()+"-selected"},e.prototype.getDisabledClassName=function(){return this.getPrefixCls()+"-disabled"},e.prototype.callRef=function(){this.props.manualRef&&this.props.manualRef(this)},e.prototype.render=function(){var t,e=b()({},this.props),n=m()(this.getPrefixCls(),e.className,((t={})[this.getActiveClassName()]=!e.disabled&&e.active,t[this.getSelectedClassName()]=e.isSelected,t[this.getDisabledClassName()]=e.disabled,t)),r=b()({},e.attribute,{title:e.title,className:n,role:e.role||"menuitem","aria-disabled":e.disabled});"option"===e.role?r=b()({},r,{role:"option","aria-selected":e.isSelected}):null!==e.role&&"none"!==e.role||(r.role="none");var o={onClick:e.disabled?null:this.onClick,onMouseLeave:e.disabled?null:this.onMouseLeave,onMouseEnter:e.disabled?null:this.onMouseEnter},a=b()({},e.style);"inline"===e.mode&&(a.paddingLeft=e.inlineIndent*e.level),H.forEach(function(t){return delete e[t]});var s=this.props.itemIcon;return"function"==typeof this.props.itemIcon&&(s=i.a.createElement(this.props.itemIcon,this.props)),i.a.createElement("li",b()({},e,r,o,{style:a}),e.children,s)},e}(i.a.Component);yt.propTypes={attribute:c.a.object,rootPrefixCls:c.a.string,eventKey:c.a.string,active:c.a.bool,children:c.a.any,selectedKeys:c.a.array,disabled:c.a.bool,title:c.a.string,onItemHover:c.a.func,onSelect:c.a.func,onClick:c.a.func,onDeselect:c.a.func,parentMenu:c.a.object,onDestroy:c.a.func,onMouseEnter:c.a.func,onMouseLeave:c.a.func,multiple:c.a.bool,isSelected:c.a.bool,manualRef:c.a.func,itemIcon:c.a.oneOfType([c.a.func,c.a.node])},yt.defaultProps={onSelect:T,onMouseEnter:T,onMouseLeave:T,manualRef:T},yt.isMenuItem=!0;var Et=Object(L.connect)(function(t,e){var n=t.activeKey,r=t.selectedKeys,i=e.eventKey;return{active:n[e.subMenuKey]===i,isSelected:-1!==r.indexOf(i)}})(yt),vt=function(t){function e(){var n,r,i;_()(this,e);for(var o=arguments.length,a=Array(o),s=0;s<o;s++)a[s]=arguments[s];return n=r=M()(this,t.call.apply(t,[this].concat(a))),r.renderInnerMenuItem=function(t){var e=r.props;return(0,e.renderMenuItem)(t,e.index,r.props.subMenuKey)},i=n,M()(r,i)}return S()(e,t),e.prototype.render=function(){var t=w()(this.props,[]),e=t.className,n=void 0===e?"":e,r=t.rootPrefixCls,o=r+"-item-group-title",a=r+"-item-group-list",s=t.title,A=t.children;return H.forEach(function(e){return delete t[e]}),delete t.onClick,i.a.createElement("li",b()({},t,{className:n+" "+r+"-item-group"}),i.a.createElement("div",{className:o,title:"string"==typeof s?s:void 0},s),i.a.createElement("ul",{className:a},i.a.Children.map(A,this.renderInnerMenuItem)))},e}(i.a.Component);vt.propTypes={renderMenuItem:c.a.func,index:c.a.number,className:c.a.string,subMenuKey:c.a.string,rootPrefixCls:c.a.string},vt.defaultProps={disabled:!0},vt.isMenuItemGroup=!0;var Bt=vt,bt=function(t){function e(){return _()(this,e),M()(this,t.apply(this,arguments))}return S()(e,t),e.prototype.render=function(){var t=this.props,e=t.className,n=t.rootPrefixCls,r=t.style;return i.a.createElement("li",{className:e+" "+n+"-item-divider",style:r})},e}(i.a.Component);bt.propTypes={className:c.a.string,rootPrefixCls:c.a.string,style:c.a.object},bt.defaultProps={disabled:!0,className:"",style:{}};var Qt=It;function wt(t){var e=[];return i.a.Children.forEach(t,function(t){e.push(t)}),e}var xt=n(67),_t=n(80),Dt=n.n(_t),Mt=n(162),kt=n.n(Mt);function St(t){return"string"==typeof t?t:""}function Lt(t){if(!t)return null;var e=t.props;if("value"in e)return e.value;if(t.key)return t.key;if(t.type&&t.type.isSelectOptGroup&&e.label)return e.label;throw new Error("Need at least a key or a value or a label (only for OptGroup) for ".concat(t))}function Rt(t,e){return"value"===e?Lt(t):t.props[e]}function Nt(t){return t.combobox}function Ft(t){return t.multiple||t.tags}function Tt(t){return Ft(t)||Nt(t)}function Ut(t){return!Tt(t)}function Ot(t){var e=t;return void 0===t?e=[]:Array.isArray(t)||(e=[t]),e}function Gt(t){return"".concat(typeof t,"-").concat(t)}function Ht(t){t.preventDefault()}function jt(t,e){var n=-1;if(t)for(var r=0;r<t.length;r++)if(t[r]===e){n=r;break}return n}function Yt(t,e){var n;if(t=Ot(t))for(var r=0;r<t.length;r++)if(t[r].key===e){n=t[r].label;break}return n}var Pt={userSelect:"none",WebkitUserSelect:"none"},Kt={unselectable:"on"};function zt(t,e){return!e.props.disabled&&Ot(Rt(e,this.props.optionFilterProp)).join("").toLowerCase().indexOf(t.toLowerCase())>-1}function Jt(t,e){return function(n){t[e]=n}}function qt(){return(qt=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}function Vt(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}function Wt(t){return(Wt=Object.setPrototypeOf?Object.getPrototypeOf:function(t){return t.__proto__||Object.getPrototypeOf(t)})(t)}function Xt(t){if(void 0===t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return t}function Zt(t,e){return(Zt=Object.setPrototypeOf||function(t,e){return t.__proto__=e,t})(t,e)}var $t=function(t){function e(t){var n;return function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}(this,e),(n=function(t,e){return!e||"object"!=typeof e&&"function"!=typeof e?Xt(t):e}(this,Wt(e).call(this,t))).rafInstance=null,n.lastVisible=!1,n.scrollActiveItemToView=function(){var t=Object(P.findDOMNode)(n.firstActiveItem),e=n.props,r=e.visible,i=e.firstActiveValue,o=n.props.value;if(t&&r){var a={onlyScrollIfNeeded:!0};o&&0!==o.length||!i||(a.alignWithTop=!0),n.rafInstance=kt()(function(){mt()(t,Object(P.findDOMNode)(n.menuRef),a)})}},n.renderMenu=function(){var t=n.props,e=t.menuItems,o=t.menuItemSelectedIcon,a=t.defaultActiveFirstOption,s=t.prefixCls,A=t.multiple,l=t.onMenuSelect,c=t.inputValue,u=t.backfillValue,g=t.onMenuDeselect,h=t.visible,d=n.props.firstActiveValue;if(e&&e.length){var f={};A?(f.onDeselect=g,f.onSelect=l):f.onClick=l;var p=n.props.value,I=function t(e,n){if(null==n)return[];var r=[];return i.a.Children.forEach(e,function(e){if(e.type.isMenuItemGroup)r=r.concat(t(e.props.children,n));else{var i=Lt(e),o=e.key;-1!==jt(n,i)&&o&&r.push(o)}}),r}(e,p),C={},m=a,y=e;if(I.length||d){h&&!n.lastVisible?C.activeKey=I[0]||d:h||(I[0]&&(m=!1),C.activeKey=void 0);var E=!1,v=function(t){var e=t.key;return!E&&-1!==I.indexOf(e)||!E&&!I.length&&-1!==d.indexOf(t.key)?(E=!0,r.cloneElement(t,{ref:function(t){n.firstActiveItem=t}})):t};y=e.map(function(t){if(t.type.isMenuItemGroup){var e=wt(t.props.children).map(v);return r.cloneElement(t,{},e)}return v(t)})}else n.firstActiveItem=null;var B=p&&p[p.length-1];return c===n.lastInputValue||B&&B===u||(C.activeKey=""),r.createElement(Qt,qt({ref:n.saveMenuRef,style:n.props.dropdownMenuStyle,defaultActiveFirst:m,role:"listbox",itemIcon:A?o:null},C,{multiple:A},f,{selectedKeys:I,prefixCls:"".concat(s,"-menu")}),y)}return null},n.lastInputValue=t.inputValue,n.saveMenuRef=Jt(Xt(n),"menuRef"),n}var n,o,a;return function(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function");t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,writable:!0,configurable:!0}}),e&&Zt(t,e)}(e,r["Component"]),n=e,(o=[{key:"componentDidMount",value:function(){this.scrollActiveItemToView(),this.lastVisible=this.props.visible}},{key:"shouldComponentUpdate",value:function(t){return t.visible||(this.lastVisible=!1),this.props.visible&&!t.visible||t.visible||t.inputValue!==this.props.inputValue}},{key:"componentDidUpdate",value:function(t){var e=this.props;!t.visible&&e.visible&&this.scrollActiveItemToView(),this.lastVisible=e.visible,this.lastInputValue=e.inputValue}},{key:"componentWillUnmount",value:function(){this.rafInstance&&kt.a.cancel(this.rafInstance)}},{key:"render",value:function(){var t=this.renderMenu();return t?r.createElement("div",{style:{overflow:"auto",transform:"translateZ(0)"},id:this.props.ariaId,onFocus:this.props.onPopupFocus,onMouseDown:Ht,onScroll:this.props.onPopupScroll},t):null}}])&&Vt(n.prototype,o),a&&Vt(n,a),e}();function te(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t}function ee(){return(ee=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}function ne(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}function re(t){return(re=Object.setPrototypeOf?Object.getPrototypeOf:function(t){return t.__proto__||Object.getPrototypeOf(t)})(t)}function ie(t){if(void 0===t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return t}function oe(t,e){return(oe=Object.setPrototypeOf||function(t,e){return t.__proto__=e,t})(t,e)}$t.displayName="DropdownMenu",$t.propTypes={ariaId:l.string,defaultActiveFirstOption:l.bool,value:l.any,dropdownMenuStyle:l.object,multiple:l.bool,onPopupFocus:l.func,onPopupScroll:l.func,onMenuDeSelect:l.func,onMenuSelect:l.func,prefixCls:l.string,menuItems:l.any,inputValue:l.string,visible:l.bool,firstActiveValue:l.string,menuItemSelectedIcon:l.oneOfType([l.func,l.node])};var ae=function(t,e){var n={};for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&e.indexOf(r)<0&&(n[r]=t[r]);if(null!=t&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(t);i<r.length;i++)e.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(t,r[i])&&(n[r[i]]=t[r[i]])}return n};J.default.displayName="Trigger";var se={bottomLeft:{points:["tl","bl"],offset:[0,4],overflow:{adjustX:0,adjustY:1}},topLeft:{points:["bl","tl"],offset:[0,-4],overflow:{adjustX:0,adjustY:1}}},Ae=function(t){function e(t){var n;return function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}(this,e),(n=function(t,e){return!e||"object"!=typeof e&&"function"!=typeof e?ie(t):e}(this,re(e).call(this,t))).dropdownMenuRef=null,n.rafInstance=null,n.setDropdownWidth=function(){n.cancelRafInstance(),n.rafInstance=kt()(function(){var t=P.findDOMNode(ie(n)).offsetWidth;t!==n.state.dropdownWidth&&n.setState({dropdownWidth:t})})},n.cancelRafInstance=function(){n.rafInstance&&kt.a.cancel(n.rafInstance)},n.getInnerMenu=function(){return n.dropdownMenuRef&&n.dropdownMenuRef.menuRef},n.getPopupDOMNode=function(){return n.triggerRef.getPopupDomNode()},n.getDropdownElement=function(t){var e=n.props,i=e.dropdownRender,o=e.ariaId,a=r.createElement($t,ee({ref:n.saveDropdownMenuRef},t,{ariaId:o,prefixCls:n.getDropdownPrefixCls(),onMenuSelect:e.onMenuSelect,onMenuDeselect:e.onMenuDeselect,onPopupScroll:e.onPopupScroll,value:e.value,backfillValue:e.backfillValue,firstActiveValue:e.firstActiveValue,defaultActiveFirstOption:e.defaultActiveFirstOption,dropdownMenuStyle:e.dropdownMenuStyle,menuItemSelectedIcon:e.menuItemSelectedIcon}));return i?i(a,e):null},n.getDropdownTransitionName=function(){var t=n.props,e=t.transitionName;return!e&&t.animation&&(e="".concat(n.getDropdownPrefixCls(),"-").concat(t.animation)),e},n.getDropdownPrefixCls=function(){return"".concat(n.props.prefixCls,"-dropdown")},n.saveDropdownMenuRef=Jt(ie(n),"dropdownMenuRef"),n.saveTriggerRef=Jt(ie(n),"triggerRef"),n.state={dropdownWidth:0},n}var n,i,o;return function(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function");t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,writable:!0,configurable:!0}}),e&&oe(t,e)}(e,r["Component"]),n=e,(i=[{key:"componentDidMount",value:function(){this.setDropdownWidth()}},{key:"componentDidUpdate",value:function(){this.setDropdownWidth()}},{key:"componentWillUnmount",value:function(){this.cancelRafInstance()}},{key:"render",value:function(){var t,e,n=this.props,i=n.onPopupFocus,o=n.empty,a=ae(n,["onPopupFocus","empty"]),s=a.multiple,A=a.visible,l=a.inputValue,c=a.dropdownAlign,u=a.disabled,g=a.showSearch,h=a.dropdownClassName,d=a.dropdownStyle,f=a.dropdownMatchSelectWidth,p=this.getDropdownPrefixCls(),I=(te(t={},h,!!h),te(t,"".concat(p,"--").concat(s?"multiple":"single"),1),te(t,"".concat(p,"--empty"),o),t),C=this.getDropdownElement({menuItems:a.options,onPopupFocus:i,multiple:s,inputValue:l,visible:A});e=u?[]:Ut(a)&&!g?["click"]:["blur"];var y=ee({},d),E=f?"width":"minWidth";return this.state.dropdownWidth&&(y[E]="".concat(this.state.dropdownWidth,"px")),r.createElement(J.default,ee({},a,{showAction:u?[]:this.props.showAction,hideAction:e,ref:this.saveTriggerRef,popupPlacement:"bottomLeft",builtinPlacements:se,prefixCls:p,popupTransitionName:this.getDropdownTransitionName(),onPopupVisibleChange:a.onDropdownVisibleChange,popup:C,popupAlign:c,popupVisible:A,getPopupContainer:a.getPopupContainer,popupClassName:m()(I),popupStyle:y}),a.children)}}])&&ne(n.prototype,i),o&&ne(n,o),e}();function le(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t}function ce(){return(ce=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}function ue(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}function ge(t){return(ge=Object.setPrototypeOf?Object.getPrototypeOf:function(t){return t.__proto__||Object.getPrototypeOf(t)})(t)}function he(t){if(void 0===t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return t}function de(t,e){return(de=Object.setPrototypeOf||function(t,e){return t.__proto__=e,t})(t,e)}Ae.defaultProps={dropdownRender:function(t){return t}},Ae.propTypes={onPopupFocus:l.func,onPopupScroll:l.func,dropdownMatchSelectWidth:l.bool,dropdownAlign:l.object,visible:l.bool,disabled:l.bool,showSearch:l.bool,dropdownClassName:l.string,multiple:l.bool,inputValue:l.string,filterOption:l.any,options:l.any,prefixCls:l.string,popupClassName:l.string,children:l.any,showAction:l.arrayOf(l.string),menuItemSelectedIcon:l.oneOfType([l.func,l.node]),dropdownRender:l.func,ariaId:l.string},Ae.displayName="SelectTrigger";var fe="RC_SELECT_EMPTY_VALUE_KEY",pe=function(){return null};function Ie(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];return function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];for(var i=0;i<e.length;i++)e[i]&&"function"==typeof e[i]&&e[i].apply(Ie,n)}}var Ce=function(t){function e(t){var n;!function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}(this,e),(n=function(t,e){return!e||"object"!=typeof e&&"function"!=typeof e?he(t):e}(this,ge(e).call(this,t))).inputRef=null,n.inputMirrorRef=null,n.topCtrlRef=null,n.selectTriggerRef=null,n.rootRef=null,n.selectionRef=null,n.dropdownContainer=null,n.blurTimer=null,n.focusTimer=null,n.comboboxTimer=null,n._focused=!1,n._mouseDown=!1,n._options=[],n._empty=!1,n.onInputChange=function(t){var e=n.props.tokenSeparators,r=t.target.value;if(Ft(n.props)&&e.length&&function(t,e){for(var n=0;n<e.length;++n)if(t.lastIndexOf(e[n])>0)return!0;return!1}(r,e)){var i=n.getValueByInput(r);return void 0!==i&&n.fireChange(i),n.setOpenState(!1,{needFocus:!0}),void n.setInputValue("",!1)}n.setInputValue(r),n.setState({open:!0}),Nt(n.props)&&n.fireChange([r])},n.onDropdownVisibleChange=function(t){t&&!n._focused&&(n.clearBlurTime(),n.timeoutFocus(),n._focused=!0,n.updateFocusClassName()),n.setOpenState(t)},n.onKeyDown=function(t){var e=n.state.open;if(!n.props.disabled){var r=t.keyCode;e&&!n.getInputDOMNode()?n.onInputKeyDown(t):r===R.a.ENTER||r===R.a.DOWN?(e||n.setOpenState(!0),t.preventDefault()):r===R.a.SPACE&&(e||(n.setOpenState(!0),t.preventDefault()))}},n.onInputKeyDown=function(t){var e=n.props,r=e.disabled,i=e.combobox,o=e.defaultActiveFirstOption;if(!r){var a=n.state,s=n.getRealOpenState(a),A=t.keyCode;if(!Ft(n.props)||t.target.value||A!==R.a.BACKSPACE){if(A===R.a.DOWN){if(!a.open)return n.openIfHasChildren(),t.preventDefault(),void t.stopPropagation()}else if(A===R.a.ENTER&&a.open)!s&&i||t.preventDefault(),s&&i&&!1===o&&(n.comboboxTimer=setTimeout(function(){n.setOpenState(!1)}));else if(A===R.a.ESC)return void(a.open&&(n.setOpenState(!1),t.preventDefault(),t.stopPropagation()));if(s&&n.selectTriggerRef){var l=n.selectTriggerRef.getInnerMenu();l&&l.onKeyDown(t,n.handleBackfill)&&(t.preventDefault(),t.stopPropagation())}}else{t.preventDefault();var c=a.value;c.length&&n.removeSelected(c[c.length-1])}}},n.onMenuSelect=function(t){var e=t.item;if(e){var r=n.state.value,i=n.props,o=Lt(e),a=r[r.length-1];if(n.fireSelect(o),Ft(i)){if(-1!==jt(r,o))return;r=r.concat([o])}else{if(!Nt(i)&&void 0!==a&&a===o&&o!==n.state.backfillValue)return void n.setOpenState(!1,{needFocus:!0,fireSearch:!1});r=[o],n.setOpenState(!1,{needFocus:!0,fireSearch:!1})}n.fireChange(r);var s=Nt(i)?Rt(e,i.optionLabelProp):"";i.autoClearSearchValue&&n.setInputValue(s,!1)}},n.onMenuDeselect=function(t){var e=t.item,r=t.domEvent;"keydown"!==r.type||r.keyCode!==R.a.ENTER?("click"===r.type&&n.removeSelected(Lt(e)),n.props.autoClearSearchValue&&n.setInputValue("")):n.removeSelected(Lt(e))},n.onArrowClick=function(t){t.stopPropagation(),t.preventDefault(),n.props.disabled||n.setOpenState(!n.state.open,{needFocus:!n.state.open})},n.onPlaceholderClick=function(){n.getInputDOMNode&&n.getInputDOMNode()&&n.getInputDOMNode().focus()},n.onOuterFocus=function(t){if(n.props.disabled)t.preventDefault();else{n.clearBlurTime();var e=n.getInputDOMNode();e&&t.target===n.rootRef||(Tt(n.props)||t.target!==e)&&(n._focused||(n._focused=!0,n.updateFocusClassName(),Ft(n.props)&&n._mouseDown||n.timeoutFocus()))}},n.onPopupFocus=function(){n.maybeFocus(!0,!0)},n.onOuterBlur=function(t){n.props.disabled?t.preventDefault():n.blurTimer=window.setTimeout(function(){n._focused=!1,n.updateFocusClassName();var t=n.props,e=n.state.value,r=n.state.inputValue;if(Ut(t)&&t.showSearch&&r&&t.defaultActiveFirstOption){var i=n._options||[];if(i.length){var o=function t(e){for(var n=0;n<e.length;n++){var r=e[n];if(r.type.isMenuItemGroup){var i=t(r.props.children);if(i)return i}else if(!r.props.disabled)return r}return null}(i);o&&(e=[Lt(o)],n.fireChange(e))}}else if(Ft(t)&&r){n._mouseDown?n.setInputValue(""):(n.state.inputValue="",n.getInputDOMNode&&n.getInputDOMNode()&&(n.getInputDOMNode().value=""));var a=n.getValueByInput(r);void 0!==a&&(e=a,n.fireChange(e))}if(Ft(t)&&n._mouseDown)return n.maybeFocus(!0,!0),void(n._mouseDown=!1);n.setOpenState(!1),t.onBlur&&t.onBlur(n.getVLForOnChange(e))},10)},n.onClearSelection=function(t){var e=n.props,r=n.state;if(!e.disabled){var i=r.inputValue,o=r.value;t.stopPropagation(),(i||o.length)&&(o.length&&n.fireChange([]),n.setOpenState(!1,{needFocus:!0}),i&&n.setInputValue(""))}},n.onChoiceAnimationLeave=function(){n.forcePopupAlign()},n.getOptionInfoBySingleValue=function(t,e){var i;if((e=e||n.state.optionsInfo)[Gt(t)]&&(i=e[Gt(t)]),i)return i;var o=t;if(n.props.labelInValue){var a=Yt(n.props.value,t),s=Yt(n.props.defaultValue,t);void 0!==a?o=a:void 0!==s&&(o=s)}return{option:r.createElement(d,{value:t,key:t},t),value:t,label:o}},n.getOptionBySingleValue=function(t){return n.getOptionInfoBySingleValue(t).option},n.getOptionsBySingleValue=function(t){return t.map(function(t){return n.getOptionBySingleValue(t)})},n.getValueByLabel=function(t){if(void 0===t)return null;var e=null;return Object.keys(n.state.optionsInfo).forEach(function(r){var i=n.state.optionsInfo[r];if(!i.disabled){var o=Ot(i.label);o&&o.join("")===t&&(e=i.value)}}),e},n.getVLBySingleValue=function(t){return n.props.labelInValue?{key:t,label:n.getLabelBySingleValue(t)}:t},n.getVLForOnChange=function(t){var e=t;return void 0!==e?(e=n.props.labelInValue?e.map(function(t){return{key:t,label:n.getLabelBySingleValue(t)}}):e.map(function(t){return t}),Ft(n.props)?e:e[0]):e},n.getLabelBySingleValue=function(t,e){return n.getOptionInfoBySingleValue(t,e).label},n.getDropdownContainer=function(){return n.dropdownContainer||(n.dropdownContainer=document.createElement("div"),document.body.appendChild(n.dropdownContainer)),n.dropdownContainer},n.getPlaceholderElement=function(){var t=n.props,e=n.state,i=!1;e.inputValue&&(i=!0);var o=e.value;o.length&&(i=!0),Nt(t)&&1===o.length&&e.value&&!e.value[0]&&(i=!1);var a=t.placeholder;return a?r.createElement("div",ce({onMouseDown:Ht,style:ce({display:i?"none":"block"},Pt)},Kt,{onClick:n.onPlaceholderClick,className:"".concat(t.prefixCls,"-selection__placeholder")}),a):null},n.getInputElement=function(){var t=n.props,e=r.createElement("input",{id:t.id,autoComplete:"off"}),i=t.getInputElement?t.getInputElement():e,o=m()(i.props.className,le({},"".concat(t.prefixCls,"-search__field"),!0));return r.createElement("div",{className:"".concat(t.prefixCls,"-search__field__wrap")},r.cloneElement(i,{ref:n.saveInputRef,onChange:n.onInputChange,onKeyDown:Ie(n.onInputKeyDown,i.props.onKeyDown,n.props.onInputKeyDown),value:n.state.inputValue,disabled:t.disabled,className:o}),r.createElement("span",{ref:n.saveInputMirrorRef,className:"".concat(t.prefixCls,"-search__field__mirror")},n.state.inputValue," "))},n.getInputDOMNode=function(){return n.topCtrlRef?n.topCtrlRef.querySelector("input,textarea,div[contentEditable]"):n.inputRef},n.getInputMirrorDOMNode=function(){return n.inputMirrorRef},n.getPopupDOMNode=function(){if(n.selectTriggerRef)return n.selectTriggerRef.getPopupDOMNode()},n.getPopupMenuComponent=function(){if(n.selectTriggerRef)return n.selectTriggerRef.getInnerMenu()},n.setOpenState=function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=e.needFocus,i=e.fireSearch,o=n.props,a=n.state;if(a.open!==t){n.props.onDropdownVisibleChange&&n.props.onDropdownVisibleChange(t);var s={open:t,backfillValue:""};!t&&Ut(o)&&o.showSearch&&n.setInputValue("",i),t||n.maybeFocus(t,!!r),n.setState(ce({open:t},s),function(){t&&n.maybeFocus(t,!!r)})}else n.maybeFocus(t,!!r)},n.setInputValue=function(t){var e=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],r=n.props.onSearch;t!==n.state.inputValue&&n.setState(function(n){return e&&t!==n.inputValue&&r&&r(t),{inputValue:t}},n.forcePopupAlign)},n.getValueByInput=function(t){var e=n.props,r=e.multiple,i=e.tokenSeparators,o=n.state.value,a=!1;return function(t,e){var n=new RegExp("[".concat(e.join(),"]"));return t.split(n).filter(function(t){return t})}(t,i).forEach(function(t){var e=[t];if(r){var i=n.getValueByLabel(t);i&&-1===jt(o,i)&&(o=o.concat(i),a=!0,n.fireSelect(i))}else-1===jt(o,t)&&(o=o.concat(e),a=!0,n.fireSelect(t))}),a?o:void 0},n.getRealOpenState=function(t){var e=n.props.open;if("boolean"==typeof e)return e;var r=(t||n.state).open,i=n._options||[];return!Tt(n.props)&&n.props.showSearch||r&&!i.length&&(r=!1),r},n.markMouseDown=function(){n._mouseDown=!0},n.markMouseLeave=function(){n._mouseDown=!1},n.handleBackfill=function(t){if(n.props.backfill&&(Ut(n.props)||Nt(n.props))){var e=Lt(t);Nt(n.props)&&n.setInputValue(e,!1),n.setState({value:[e],backfillValue:e})}},n.filterOption=function(t,e){var r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:zt,i=n.state.value,o=i[i.length-1];if(!t||o&&o===n.state.backfillValue)return!0;var a=n.props.filterOption;return"filterOption"in n.props?!0===a&&(a=r.bind(he(n))):a=r.bind(he(n)),!a||("function"==typeof a?a.call(he(n),t,e):!e.props.disabled)},n.timeoutFocus=function(){var t=n.props.onFocus;n.focusTimer&&n.clearFocusTime(),n.focusTimer=window.setTimeout(function(){t&&t()},10)},n.clearFocusTime=function(){n.focusTimer&&(clearTimeout(n.focusTimer),n.focusTimer=null)},n.clearBlurTime=function(){n.blurTimer&&(clearTimeout(n.blurTimer),n.blurTimer=null)},n.clearComboboxTime=function(){n.comboboxTimer&&(clearTimeout(n.comboboxTimer),n.comboboxTimer=null)},n.updateFocusClassName=function(){var t=n.rootRef,e=n.props;n._focused?E()(t).add("".concat(e.prefixCls,"-focused")):E()(t).remove("".concat(e.prefixCls,"-focused"))},n.maybeFocus=function(t,e){if(e||t){var r=n.getInputDOMNode(),i=document.activeElement;r&&(t||Tt(n.props))?i!==r&&(r.focus(),n._focused=!0):i!==n.selectionRef&&n.selectionRef&&(n.selectionRef.focus(),n._focused=!0)}},n.removeSelected=function(t,e){var r=n.props;if(!r.disabled&&!n.isChildDisabled(t)){e&&e.stopPropagation&&e.stopPropagation();var i=n.state.value.filter(function(e){return e!==t});if(Ft(r)){var o=t;r.labelInValue&&(o={key:t,label:n.getLabelBySingleValue(t)}),r.onDeselect&&r.onDeselect(o,n.getOptionBySingleValue(t))}n.fireChange(i)}},n.openIfHasChildren=function(){var t=n.props;(r.Children.count(t.children)||Ut(t))&&n.setOpenState(!0)},n.fireSelect=function(t){n.props.onSelect&&n.props.onSelect(n.getVLBySingleValue(t),n.getOptionBySingleValue(t))},n.fireChange=function(t){var e=n.props;"value"in e||n.setState({value:t},n.forcePopupAlign);var r=n.getVLForOnChange(t),i=n.getOptionsBySingleValue(t);e.onChange&&e.onChange(r,Ft(n.props)?i:i[0])},n.isChildDisabled=function(t){return wt(n.props.children).some(function(e){return Lt(e)===t&&e.props&&e.props.disabled})},n.forcePopupAlign=function(){n.state.open&&n.selectTriggerRef&&n.selectTriggerRef.triggerRef&&n.selectTriggerRef.triggerRef.forcePopupAlign()},n.renderFilterOptions=function(){var t=n.state.inputValue,e=n.props,i=e.children,o=e.tags,a=e.notFoundContent,s=[],A=[],l=!1,c=n.renderFilterOptionsFromChildren(i,A,s);if(o){var u=n.state.value;(u=u.filter(function(e){return-1===A.indexOf(e)&&(!t||String(e).indexOf(String(t))>-1)})).sort(function(t,e){return t.length-e.length}),u.forEach(function(t){var e=t,n=r.createElement(Et,{style:Pt,role:"option",attribute:Kt,value:e,key:e},e);c.push(n),s.push(n)}),t&&s.every(function(e){return Lt(e)!==t})&&c.unshift(r.createElement(Et,{style:Pt,role:"option",attribute:Kt,value:t,key:t},t))}return!c.length&&a&&(l=!0,c=[r.createElement(Et,{style:Pt,attribute:Kt,disabled:!0,role:"option",value:"NOT_FOUND",key:"NOT_FOUND"},a)]),{empty:l,options:c}},n.renderFilterOptionsFromChildren=function(t,e,i){var o=[],a=n.props,s=n.state.inputValue,A=a.tags;return r.Children.forEach(t,function(t){if(t){var a=t.type;if(a.isSelectOptGroup){var l=t.props.label,c=t.key;if(c||"string"!=typeof l?!l&&c&&(l=c):c=l,s&&n.filterOption(s,t)){var u=wt(t.props.children).map(function(t){var e=Lt(t)||t.key;return r.createElement(Et,ce({key:e,value:e},t.props))});o.push(r.createElement(Bt,{key:c,title:l},u))}else{var g=n.renderFilterOptionsFromChildren(t.props.children,e,i);g.length&&o.push(r.createElement(Bt,{key:c,title:l},g))}}else{Dt()(a.isSelectOption,"the children of `Select` should be `Select.Option` or `Select.OptGroup`, "+"instead of `".concat(a.name||a.displayName||t.type,"`."));var h=Lt(t);if(function(t,e){if(!Ut(e)&&!function(t){return t.multiple}(e)&&"string"!=typeof t)throw new Error("Invalid `value` of type `".concat(typeof t,"` supplied to Option, ")+"expected `string` when `tags/combobox` is `true`.")}(h,n.props),n.filterOption(s,t)){var d=r.createElement(Et,ce({style:Pt,attribute:Kt,value:h,key:h,role:"option"},t.props));o.push(d),i.push(d)}A&&e.push(h)}}}),o},n.renderTopControlNode=function(){var t=n.state,e=t.open,i=t.inputValue,o=n.state.value,a=n.props,s=a.choiceTransitionName,A=a.prefixCls,l=a.maxTagTextLength,c=a.maxTagCount,u=a.showSearch,g=a.removeIcon,h=a.maxTagPlaceholder,d="".concat(A,"-selection__rendered"),f=null;if(Ut(a)){var p=null;if(o.length){var I=!1,C=1;u&&e?(I=!i)&&(C=.4):I=!0;var m=o[0],y=n.getOptionInfoBySingleValue(m),E=y.label,B=y.title;p=r.createElement("div",{key:"value",className:"".concat(A,"-selection-selected-value"),title:St(B||E),style:{display:I?"block":"none",opacity:C}},E)}f=u?[p,r.createElement("div",{className:"".concat(A,"-search ").concat(A,"-search--inline"),key:"input",style:{display:e?"block":"none"}},n.getInputElement())]:[p]}else{var b,Q=[],w=o;if(void 0!==c&&o.length>c){w=w.slice(0,c);var x=n.getVLForOnChange(o.slice(c,o.length)),_="+ ".concat(o.length-c," ...");h&&(_="function"==typeof h?h(x):h),b=r.createElement("li",ce({style:Pt},Kt,{role:"presentation",onMouseDown:Ht,className:"".concat(A,"-selection__choice ").concat(A,"-selection__choice__disabled"),key:"maxTagPlaceholder",title:St(_)}),r.createElement("div",{className:"".concat(A,"-selection__choice__content")},_))}Ft(a)&&(Q=w.map(function(t){var e=n.getOptionInfoBySingleValue(t),i=e.label,o=e.title||i;l&&"string"==typeof i&&i.length>l&&(i="".concat(i.slice(0,l),"..."));var a=n.isChildDisabled(t),s=a?"".concat(A,"-selection__choice ").concat(A,"-selection__choice__disabled"):"".concat(A,"-selection__choice");return r.createElement("li",ce({style:Pt},Kt,{onMouseDown:Ht,className:s,role:"presentation",key:t||fe,title:St(o)}),r.createElement("div",{className:"".concat(A,"-selection__choice__content")},i),a?null:r.createElement("span",{onClick:function(e){n.removeSelected(t,e)},className:"".concat(A,"-selection__choice__remove")},g||r.createElement("i",{className:"".concat(A,"-selection__choice__remove-icon")},"×")))})),b&&Q.push(b),Q.push(r.createElement("li",{className:"".concat(A,"-search ").concat(A,"-search--inline"),key:"__input"},n.getInputElement())),f=Ft(a)&&s?r.createElement(v.a,{onLeave:n.onChoiceAnimationLeave,component:"ul",transitionName:s},Q):r.createElement("ul",null,Q)}return r.createElement("div",{className:d,ref:n.saveTopCtrlRef},n.getPlaceholderElement(),f)};var i=e.getOptionsInfoFromProps(t);if(t.tags&&"function"!=typeof t.filterOption){var o=Object.keys(i).some(function(t){return i[t].disabled});Dt()(!o,"Please avoid setting option to disabled in tags mode since user can always type text as tag.")}return n.state={value:e.getValueFromProps(t,!0),inputValue:t.combobox?e.getInputValueForCombobox(t,i,!0):"",open:t.defaultOpen,optionsInfo:i,backfillValue:"",skipBuildOptionsInfo:!0,ariaId:""},n.saveInputRef=Jt(he(n),"inputRef"),n.saveInputMirrorRef=Jt(he(n),"inputMirrorRef"),n.saveTopCtrlRef=Jt(he(n),"topCtrlRef"),n.saveSelectTriggerRef=Jt(he(n),"selectTriggerRef"),n.saveRootRef=Jt(he(n),"rootRef"),n.saveSelectionRef=Jt(he(n),"selectionRef"),n}var n,i,o;return function(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function");t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,writable:!0,configurable:!0}}),e&&de(t,e)}(e,r["Component"]),n=e,(i=[{key:"componentDidMount",value:function(){var t;(this.props.autoFocus||this.state.open)&&this.focus(),this.setState({ariaId:(t=(new Date).getTime(),"xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g,function(e){var n=(t+16*Math.random())%16|0;return t=Math.floor(t/16),("x"===e?n:7&n|8).toString(16)}))})}},{key:"componentDidUpdate",value:function(){if(Ft(this.props)){var t=this.getInputDOMNode(),e=this.getInputMirrorDOMNode();t&&t.value&&e?(t.style.width="",t.style.width="".concat(e.clientWidth,"px")):t&&(t.style.width="")}this.forcePopupAlign()}},{key:"componentWillUnmount",value:function(){this.clearFocusTime(),this.clearBlurTime(),this.clearComboboxTime(),this.dropdownContainer&&(P.unmountComponentAtNode(this.dropdownContainer),document.body.removeChild(this.dropdownContainer),this.dropdownContainer=null)}},{key:"focus",value:function(){Ut(this.props)&&this.selectionRef?this.selectionRef.focus():this.getInputDOMNode()&&this.getInputDOMNode().focus()}},{key:"blur",value:function(){Ut(this.props)&&this.selectionRef?this.selectionRef.blur():this.getInputDOMNode()&&this.getInputDOMNode().blur()}},{key:"renderArrow",value:function(t){var e=this.props,n=e.showArrow,i=void 0===n?!t:n,o=e.loading,a=e.inputIcon,s=e.prefixCls;if(!i&&!o)return null;var A=o?r.createElement("i",{className:"".concat(s,"-arrow-loading")}):r.createElement("i",{className:"".concat(s,"-arrow-icon")});return r.createElement("span",ce({key:"arrow",className:"".concat(s,"-arrow"),style:Pt},Kt,{onClick:this.onArrowClick}),a||A)}},{key:"renderClear",value:function(){var t=this.props,e=t.prefixCls,n=t.allowClear,i=t.clearIcon,o=this.state.inputValue,a=this.state.value,s=r.createElement("span",ce({key:"clear",className:"".concat(e,"-selection__clear"),onMouseDown:Ht,style:Pt},Kt,{onClick:this.onClearSelection}),i||r.createElement("i",{className:"".concat(e,"-selection__clear-icon")},"×"));return n?Nt(this.props)?o?s:null:o||a.length?s:null:null}},{key:"render",value:function(){var t,e=this.props,n=Ft(e),i=e.showArrow,o=void 0===i||i,a=this.state,s=e.className,A=e.disabled,l=e.prefixCls,c=e.loading,u=this.renderTopControlNode(),g=this.state,h=g.open,d=g.ariaId;if(h){var f=this.renderFilterOptions();this._empty=f.empty,this._options=f.options}var p=this.getRealOpenState(),I=this._empty,C=this._options||[],y={};Object.keys(e).forEach(function(t){!Object.prototype.hasOwnProperty.call(e,t)||"data-"!==t.substr(0,5)&&"aria-"!==t.substr(0,5)&&"role"!==t||(y[t]=e[t])});var E=ce({},y);Tt(e)||(E=ce({},E,{onKeyDown:this.onKeyDown,tabIndex:e.disabled?-1:e.tabIndex}));var v=(le(t={},s,!!s),le(t,l,1),le(t,"".concat(l,"-open"),h),le(t,"".concat(l,"-focused"),h||!!this._focused),le(t,"".concat(l,"-combobox"),Nt(e)),le(t,"".concat(l,"-disabled"),A),le(t,"".concat(l,"-enabled"),!A),le(t,"".concat(l,"-allow-clear"),!!e.allowClear),le(t,"".concat(l,"-no-arrow"),!o),le(t,"".concat(l,"-loading"),!!c),t);return r.createElement(Ae,{onPopupFocus:this.onPopupFocus,onMouseEnter:this.props.onMouseEnter,onMouseLeave:this.props.onMouseLeave,dropdownAlign:e.dropdownAlign,dropdownClassName:e.dropdownClassName,dropdownMatchSelectWidth:e.dropdownMatchSelectWidth,defaultActiveFirstOption:e.defaultActiveFirstOption,dropdownMenuStyle:e.dropdownMenuStyle,transitionName:e.transitionName,animation:e.animation,prefixCls:e.prefixCls,dropdownStyle:e.dropdownStyle,combobox:e.combobox,showSearch:e.showSearch,options:C,empty:I,multiple:n,disabled:A,visible:p,inputValue:a.inputValue,value:a.value,backfillValue:a.backfillValue,firstActiveValue:e.firstActiveValue,onDropdownVisibleChange:this.onDropdownVisibleChange,getPopupContainer:e.getPopupContainer,onMenuSelect:this.onMenuSelect,onMenuDeselect:this.onMenuDeselect,onPopupScroll:e.onPopupScroll,showAction:e.showAction,ref:this.saveSelectTriggerRef,menuItemSelectedIcon:e.menuItemSelectedIcon,dropdownRender:e.dropdownRender,ariaId:d},r.createElement("div",{id:e.id,style:e.style,ref:this.saveRootRef,onBlur:this.onOuterBlur,onFocus:this.onOuterFocus,className:m()(v),onMouseDown:this.markMouseDown,onMouseUp:this.markMouseLeave,onMouseOut:this.markMouseLeave},r.createElement("div",ce({ref:this.saveSelectionRef,key:"selection",className:"".concat(l,"-selection\n            ").concat(l,"-selection--").concat(n?"multiple":"single"),role:"combobox","aria-autocomplete":"list","aria-haspopup":"true","aria-controls":d,"aria-expanded":p},E),u,this.renderClear(),this.renderArrow(!!n))))}}])&&ue(n.prototype,i),o&&ue(n,o),e}();Ce.propTypes=I,Ce.defaultProps={prefixCls:"rc-select",defaultOpen:!1,labelInValue:!1,defaultActiveFirstOption:!0,showSearch:!0,allowClear:!1,placeholder:"",onChange:pe,onFocus:pe,onBlur:pe,onSelect:pe,onSearch:pe,onDeselect:pe,onInputKeyDown:pe,dropdownMatchSelectWidth:!0,dropdownStyle:{},dropdownMenuStyle:{},optionFilterProp:"value",optionLabelProp:"value",notFoundContent:"Not Found",backfill:!1,showAction:["click"],tokenSeparators:[],autoClearSearchValue:!0,tabIndex:0,dropdownRender:function(t){return t}},Ce.getDerivedStateFromProps=function(t,e){var n=e.skipBuildOptionsInfo?e.optionsInfo:Ce.getOptionsInfoFromProps(t,e),r={optionsInfo:n,skipBuildOptionsInfo:!1};if("open"in t&&(r.open=t.open),"value"in t){var i=Ce.getValueFromProps(t);r.value=i,t.combobox&&(r.inputValue=Ce.getInputValueForCombobox(t,n))}return r},Ce.getOptionsFromChildren=function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:[];return r.Children.forEach(t,function(t){t&&(t.type.isSelectOptGroup?Ce.getOptionsFromChildren(t.props.children,e):e.push(t))}),e},Ce.getInputValueForCombobox=function(t,e,n){var r=[];if("value"in t&&!n&&(r=Ot(t.value)),"defaultValue"in t&&n&&(r=Ot(t.defaultValue)),!r.length)return"";var i=r=r[0];return t.labelInValue?i=r.label:e[Gt(r)]&&(i=e[Gt(r)].label),void 0===i&&(i=""),i},Ce.getLabelFromOption=function(t,e){return Rt(e,t.optionLabelProp)},Ce.getOptionsInfoFromProps=function(t,e){var n=Ce.getOptionsFromChildren(t.children),r={};if(n.forEach(function(e){var n=Lt(e);r[Gt(n)]={option:e,value:n,label:Ce.getLabelFromOption(t,e),title:e.props.title,disabled:e.props.disabled}}),e){var i=e.optionsInfo,o=e.value;o&&o.forEach(function(t){var e=Gt(t);r[e]||void 0===i[e]||(r[e]=i[e])})}return r},Ce.getValueFromProps=function(t,e){var n=[];return"value"in t&&!e&&(n=Ot(t.value)),"defaultValue"in t&&e&&(n=Ot(t.defaultValue)),t.labelInValue&&(n=n.map(function(t){return t.key})),n},Ce.displayName="Select",Object(xt.polyfill)(Ce);var me=Ce;n.d(e,"Option",function(){return d}),n.d(e,"OptGroup",function(){return A}),n.d(e,"SelectPropTypes",function(){return I}),me.Option=d,me.OptGroup=A;e.default=me},function(t,e,n){"use strict";n.r(e);var r=n(612),i=n(40),o=n(626),a=n(5),s=n(46),A=n(10),l=n(0),c=n.n(l);const u=l.createElement("p",null,"A trusted Jupyter notebook may execute hidden malicious code when you open it.",l.createElement("br",null),"Selecting trust will re-render this notebook in a trusted state.",l.createElement("br",null),"For more information, see the",l.createElement("a",{href:"https://jupyter-notebook.readthedocs.io/en/stable/security.html"},"Jupyter security documentation")),g="application/vnd.jupyter.cells";class h{static get executed(){return f.executed}constructor(){}}var d,f;(d=h||(h={})).splitCell=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.deselectAll();const n=t.model,r=t.activeCellIndex,i=t.widgets[r],o=i.editor,a=o.getCursorPosition(),s=o.getOffsetAt(a),A=i.model.value.text,l=f.cloneCell(n,i.model),c=f.cloneCell(n,i.model);"code"===l.type&&l.outputs.clear(),l.value.text=A.slice(0,s).replace(/^\n+/,"").replace(/\n+$/,""),c.value.text=A.slice(s).replace(/^\n+/,"").replace(/\n+$/,"");const u=n.cells;u.beginCompoundOperation(),u.set(r,l),u.insert(r+1,c),u.endCompoundOperation(),t.activeCellIndex++,f.handleState(t,e)},d.mergeCells=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t),n=[],r=[],i=t.model,a=i.cells,s=t.activeCell,A=t.activeCellIndex;if(t.widgets.forEach((e,i)=>{t.isSelectedOrActive(e)&&(n.push(e.model.value.text),i!==A&&r.push(e.model))}),1===n.length){if(A===a.length-1)return;const t=a.get(A+1);n.push(t.value.text),r.push(t)}t.deselectAll();const l=f.cloneCell(i,s.model);l.value.text=n.join("\n\n"),"code"===l.type&&l.outputs.clear(),a.beginCompoundOperation(),a.set(A,l),r.forEach(t=>{a.removeValue(t)}),a.endCompoundOperation(),s instanceof o.c&&(t.activeCell.rendered=!1),f.handleState(t,e)},d.deleteCells=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);f.deleteCells(t),f.handleState(t,e)},d.insertAbove=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t),n=t.model,r=n.contentFactory.createCell(t.notebookConfig.defaultCell,{}),i=t.activeCellIndex;n.cells.insert(i,r),t.activeCellIndex=i,t.deselectAll(),f.handleState(t,e,!0)},d.insertBelow=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t),n=t.model,r=n.contentFactory.createCell(t.notebookConfig.defaultCell,{});n.cells.insert(t.activeCellIndex+1,r),t.activeCellIndex++,t.deselectAll(),f.handleState(t,e,!0)},d.moveDown=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t),n=t.model.cells,r=t.widgets;n.beginCompoundOperation();for(let e=n.length-2;e>-1;e--)t.isSelectedOrActive(r[e])&&(t.isSelectedOrActive(r[e+1])||(n.move(e,e+1),t.activeCellIndex===e&&t.activeCellIndex++,t.select(r[e+1]),t.deselect(r[e])));n.endCompoundOperation(),f.handleState(t,e,!0)},d.moveUp=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t),n=t.model.cells,r=t.widgets;n.beginCompoundOperation();for(let e=1;e<n.length;e++)t.isSelectedOrActive(r[e])&&(t.isSelectedOrActive(r[e-1])||(n.move(e,e-1),t.activeCellIndex===e&&t.activeCellIndex--,t.select(r[e-1]),t.deselect(r[e])));n.endCompoundOperation(),f.handleState(t,e,!0)},d.changeCellType=function(t,e){if(!t.model||!t.activeCell)return;const n=f.getState(t);f.changeCellType(t,e),f.handleState(t,n)},d.run=function(t,e){if(!t.model||!t.activeCell)return Promise.resolve(!1);const n=f.getState(t),r=f.runSelected(t,e);return f.handleRunState(t,n,!1),r},d.runAndAdvance=function(t,e){if(!t.model||!t.activeCell)return Promise.resolve(!1);const n=f.getState(t),r=f.runSelected(t,e),i=t.model;if(t.activeCellIndex===t.widgets.length-1){const e=i.contentFactory.createCell(t.notebookConfig.defaultCell,{});i.cells.push(e),t.activeCellIndex++,t.mode="edit"}else t.activeCellIndex++;return f.handleRunState(t,n,!0),r},d.runAndInsert=function(t,e){if(!t.model||!t.activeCell)return Promise.resolve(!1);const n=f.getState(t),r=f.runSelected(t,e),i=t.model,o=i.contentFactory.createCell(t.notebookConfig.defaultCell,{});return i.cells.insert(t.activeCellIndex+1,o),t.activeCellIndex++,t.mode="edit",f.handleRunState(t,n,!0),r},d.runAll=function(t,e){if(!t.model||!t.activeCell)return Promise.resolve(!1);const n=f.getState(t);t.widgets.forEach(e=>{t.select(e)});const r=f.runSelected(t,e);return f.handleRunState(t,n,!0),r},d.renderAllMarkdown=function(t,e){if(!t.model||!t.activeCell)return Promise.resolve(!1);const n=t.activeCellIndex,r=f.getState(t);if(t.widgets.forEach((e,n)=>{"markdown"===e.model.type&&(t.select(e),t.activeCellIndex=n)}),"markdown"!==t.activeCell.model.type)return Promise.resolve(!0);const i=f.runSelected(t,e);return t.activeCellIndex=n,f.handleRunState(t,r,!0),i},d.runAllAbove=function(t,e){const{activeCell:n,activeCellIndex:r,model:i}=t;if(!i||!n||r<1)return Promise.resolve(!1);const o=f.getState(t);t.activeCellIndex--,t.deselectAll();for(let e=0;e<t.activeCellIndex;++e)t.select(t.widgets[e]);const a=f.runSelected(t,e);return t.activeCellIndex++,f.handleRunState(t,o,!0),a},d.runAllBelow=function(t,e){if(!t.model||!t.activeCell)return Promise.resolve(!1);const n=f.getState(t);t.deselectAll();for(let e=t.activeCellIndex;e<t.widgets.length;++e)t.select(t.widgets[e]);const r=f.runSelected(t,e);return f.handleRunState(t,n,!0),r},d.selectAbove=function(t){if(!t.model||!t.activeCell)return;if(0===t.activeCellIndex)return;let e=t.activeCellIndex-1;if("edit"===t.mode)for(;t.widgets[e].inputHidden;){if(0===e)return;e-=1}const n=f.getState(t);t.activeCellIndex=e,t.deselectAll(),f.handleState(t,n,!0)},d.selectBelow=function(t){if(!t.model||!t.activeCell)return;const e=t.widgets.length-1;if(t.activeCellIndex===e)return;let n=t.activeCellIndex+1;if("edit"===t.mode)for(;t.widgets[n].inputHidden;){if(n===e)return;n+=1}const r=f.getState(t);t.activeCellIndex=n,t.deselectAll(),f.handleState(t,r,!0)},d.extendSelectionAbove=function(t){if(!t.model||!t.activeCell)return;if(0===t.activeCellIndex)return;const e=f.getState(t);t.mode="command",t.extendContiguousSelectionTo(t.activeCellIndex-1),f.handleState(t,e,!0)},d.extendSelectionBelow=function(t){if(!t.model||!t.activeCell)return;if(t.activeCellIndex===t.widgets.length-1)return;const e=f.getState(t);t.mode="command",t.extendContiguousSelectionTo(t.activeCellIndex+1),f.handleState(t,e,!0)},d.selectAll=function(t){t.model&&t.activeCell&&t.widgets.forEach(e=>{t.select(e)})},d.deselectAll=function(t){t.model&&t.activeCell&&t.deselectAll()},d.copy=function(t){f.copyOrCut(t,!1)},d.cut=function(t){f.copyOrCut(t,!0)},d.paste=function(t,e="below"){if(!t.model||!t.activeCell)return;const n=r.a.getInstance();if(!n.hasData(g))return;const i=f.getState(t),o=n.getData(g),a=t.model;t.mode="command";const s=o.map(t=>{switch(t.cell_type){case"code":return a.contentFactory.createCodeCell({cell:t});case"markdown":return a.contentFactory.createMarkdownCell({cell:t});default:return a.contentFactory.createRawCell({cell:t})}}),A=t.model.cells;let l;switch(A.beginCompoundOperation(),e){case"below":l=t.activeCellIndex;break;case"above":l=t.activeCellIndex-1;break;case"replace":const n=[];t.widgets.forEach((e,r)=>{const i=!1!==e.model.metadata.get("deletable");t.isSelectedOrActive(e)&&i&&n.push(r)}),n.length>0&&n.reverse().forEach(t=>{A.remove(t)}),l=n[0]}s.forEach(t=>{A.insert(++l,t)}),A.endCompoundOperation(),t.activeCellIndex+=s.length,t.deselectAll(),f.handleState(t,i)},d.undo=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.mode="command",t.model.cells.undo(),t.deselectAll(),f.handleState(t,e)},d.redo=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.mode="command",t.model.cells.redo(),t.deselectAll(),f.handleState(t,e)},d.toggleAllLineNumbers=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t),n=t.editorConfig,r=!(n.code.lineNumbers&&n.markdown.lineNumbers&&n.raw.lineNumbers),i={code:Object.assign({},n.code,{lineNumbers:r}),markdown:Object.assign({},n.markdown,{lineNumbers:r}),raw:Object.assign({},n.raw,{lineNumbers:r})};t.editorConfig=i,f.handleState(t,e)},d.clearOutputs=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);Object(a.each)(t.model.cells,(e,n)=>{const r=t.widgets[n];t.isSelectedOrActive(r)&&"code"===e.type&&(e.outputs.clear(),r.outputHidden=!1,e.executionCount=null)}),f.handleState(t,e)},d.clearAllOutputs=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);Object(a.each)(t.model.cells,(e,n)=>{const r=t.widgets[n];"code"===e.type&&(e.outputs.clear(),e.executionCount=null,r.outputHidden=!1)}),f.handleState(t,e)},d.hideCode=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.widgets.forEach(e=>{t.isSelectedOrActive(e)&&"code"===e.model.type&&(e.inputHidden=!0)}),f.handleState(t,e)},d.showCode=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.widgets.forEach(e=>{t.isSelectedOrActive(e)&&"code"===e.model.type&&(e.inputHidden=!1)}),f.handleState(t,e)},d.hideAllCode=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.widgets.forEach(t=>{"code"===t.model.type&&(t.inputHidden=!0)}),f.handleState(t,e)},d.showAllCode=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.widgets.forEach(t=>{"code"===t.model.type&&(t.inputHidden=!1)}),f.handleState(t,e)},d.hideOutput=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.widgets.forEach(e=>{t.isSelectedOrActive(e)&&"code"===e.model.type&&(e.outputHidden=!0)}),f.handleState(t,e)},d.showOutput=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.widgets.forEach(e=>{t.isSelectedOrActive(e)&&"code"===e.model.type&&(e.outputHidden=!1)}),f.handleState(t,e)},d.hideAllOutputs=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.widgets.forEach(t=>{"code"===t.model.type&&(t.outputHidden=!0)}),f.handleState(t,e)},d.showAllOutputs=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.widgets.forEach(t=>{"code"===t.model.type&&(t.outputHidden=!1)}),f.handleState(t,e)},d.enableOutputScrolling=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.widgets.forEach(e=>{t.isSelectedOrActive(e)&&"code"===e.model.type&&(e.outputsScrolled=!0)}),f.handleState(t,e)},d.disableOutputScrolling=function(t){if(!t.model||!t.activeCell)return;const e=f.getState(t);t.widgets.forEach(e=>{t.isSelectedOrActive(e)&&"code"===e.model.type&&(e.outputsScrolled=!1)}),f.handleState(t,e)},d.setMarkdownHeader=function(t,e){if(!t.model||!t.activeCell)return;const n=f.getState(t),r=t.model.cells;e=Math.min(Math.max(e,1),6),t.widgets.forEach((n,i)=>{t.isSelectedOrActive(n)&&f.setMarkdownHeader(r.get(i),e)}),f.changeCellType(t,"markdown"),f.handleState(t,n)},d.trust=function(t){if(!t.model)return Promise.resolve();const e=Object(a.toArray)(t.model.cells);return e.every(t=>t.trusted)?Object(i.b)({body:"Notebook is already trusted",buttons:[i.a.okButton()]}).then(()=>void 0):Object(i.b)({body:u,title:"Trust this notebook?",buttons:[i.a.cancelButton(),i.a.warnButton()]}).then(t=>{t.button.accept&&e.forEach(t=>{t.trusted=!0})})},function(t){function e(t){return{wasFocused:t.node.contains(document.activeElement),activeCell:t.activeCell}}function n(t,e,n=!1){const{activeCell:r,node:i}=t;(e.wasFocused||"edit"===t.mode)&&t.activate(),n&&s.ElementExt.scrollIntoViewIfNeeded(i,r.node)}function i(t){const e=t.model,n=e.cells,r=[];t.mode="command",t.widgets.forEach((e,n)=>{const i=!1!==e.model.metadata.get("deletable");t.isSelectedOrActive(e)&&i&&(r.push(n),t.model.deletedCells.push(e.model.id))}),r.length>0&&(n.beginCompoundOperation(),r.reverse().forEach(t=>{n.remove(t)}),n.length||n.push(e.contentFactory.createCell(t.notebookConfig.defaultCell,{})),n.endCompoundOperation(),t.activeCellIndex=r[0]-r.length+1),t.deselectAll()}t.executed=new A.Signal({}),t.getState=e,t.handleState=n,t.handleRunState=function(t,e,n=!1){if((e.wasFocused||"edit"===t.mode)&&t.activate(),n){const n=e.activeCell.inputArea.node.getBoundingClientRect();t.scrollToPosition(n.bottom,45)}},t.cloneCell=function(t,e){switch(e.type){case"code":return t.contentFactory.createCodeCell({cell:e.toJSON()});case"markdown":return t.contentFactory.createMarkdownCell({cell:e.toJSON()});default:return t.contentFactory.createRawCell({cell:e.toJSON()})}},t.runSelected=function(e,n){e.mode="command";let r=e.activeCellIndex;const i=e.widgets.filter((t,n)=>{const i=e.isSelectedOrActive(t);return i&&(r=n),i});return e.activeCellIndex=r,e.deselectAll(),Promise.all(i.map(r=>(function(e,n,r){switch(n.model.type){case"markdown":n.rendered=!0,n.inputHidden=!1,t.executed.emit({notebook:e,cell:n});break;case"code":if(r)return o.b.execute(n,r,{deletedCells:e.model.deletedCells}).then(t=>{if(e.model.deletedCells.splice(0,e.model.deletedCells.length),n.isDisposed)return!1;if(!t)return!0;if("ok"===t.content.status){const r=t.content;return r.payload&&r.payload.length&&function(t,e,n){const r=t.payload.filter(t=>"set_next_input"===t.source)[0];if(!r)return;const i=r.text;if(r.replace)return void(n.model.value.text=i);const o=e.model.contentFactory.createCodeCell({}),s=e.model.cells,A=a.ArrayExt.firstIndexOf(Object(a.toArray)(s),n.model);o.value.text=i,-1===A?s.push(o):s.insert(A+1,o)}(r,e,n),!0}throw new Error("KernelReplyNotOK")}).catch(t=>{if("Canceled"!==t.message)throw t;return!1}).then(r=>(r&&t.executed.emit({notebook:e,cell:n}),r));n.model.executionCount=null}return Promise.resolve(!0)})(e,r,n))).then(t=>!e.isDisposed&&(e.update(),t.every(t=>t))).catch(t=>{if("KernelReplyNotOK"!==t.message)throw t;return i.map(t=>{"code"===t.model.type&&null==t.model.executionCount&&t.setPrompt("")}),e.update(),!1})},t.copyOrCut=function(t,o){if(!t.model||!t.activeCell)return;const a=e(t),s=r.a.getInstance();t.mode="command",s.clear();let A=t.widgets.filter(e=>t.isSelectedOrActive(e)).map(t=>t.model.toJSON()).map(t=>(void 0!==t.metadata.deletable&&delete t.metadata.deletable,t));s.setData(g,A),o?i(t):t.deselectAll(),n(t,a)},t.changeCellType=function(t,e){const n=t.model,r=n.cells;r.beginCompoundOperation(),t.widgets.forEach((i,o)=>{if(t.isSelectedOrActive(i)){if(i.model.type!==e){const t=i.model.toJSON();let a;switch(e){case"code":a=n.contentFactory.createCodeCell({cell:t});break;case"markdown":a=n.contentFactory.createMarkdownCell({cell:t}),"code"===i.model.type&&(a.trusted=!1);break;default:a=n.contentFactory.createRawCell({cell:t}),"code"===i.model.type&&(a.trusted=!1)}r.set(o,a)}"markdown"===e&&((i=t.widgets[o]).rendered=!1)}}),r.endCompoundOperation(),t.deselectAll()},t.deleteCells=i,t.setMarkdownHeader=function(t,e){let n=t.value.text;const r=Array(e+1).join("#")+" ",i=/^(#+\s*)|^(\s*)/.exec(n);i&&(n=n.slice(i[0].length)),t.value.text=r+n}}(f||(f={}));var p=n(245),I=n(76),C=n(1454);const m="jp-SaveIcon",y="jp-AddIcon",E="jp-CutIcon",v="jp-CopyIcon",B="jp-PasteIcon",b="jp-RunIcon",Q="jp-Notebook-toolbarCellType",w="jp-Notebook-toolbarCellTypeDropdown";var x;!function(t){function e(t){function e(){if(t.context.model.readOnly)return Object(i.b)({title:"Cannot Save",body:"Document is read-only",buttons:[i.a.okButton()]});t.context.save().then(()=>{if(!t.isDisposed)return t.context.createCheckpoint()})}return Object(p.g)(I.a.create(l.createElement(I.b,{signal:t.context.fileChanged},()=>l.createElement(p.e,{iconClassName:m,onClick:e,tooltip:"Save the notebook contents and create checkpoint",enabled:!!(t&&t.context&&t.context.contentsModel&&t.context.contentsModel.writable)}))))}function n(t){return new p.d({iconClassName:y,onClick:()=>{h.insertBelow(t.content)},tooltip:"Insert a cell below"})}function r(t){return new p.d({iconClassName:E,onClick:()=>{h.cut(t.content)},tooltip:"Cut the selected cells"})}function o(t){return new p.d({iconClassName:v,onClick:()=>{h.copy(t.content)},tooltip:"Copy the selected cells"})}function a(t){return new p.d({iconClassName:B,onClick:()=>{h.paste(t.content)},tooltip:"Paste cells from the clipboard"})}function s(t){return new p.d({iconClassName:b,onClick:()=>{h.runAndAdvance(t.content,t.session)},tooltip:"Run the selected cells and advance"})}function A(t){return new _(t.content)}t.createSaveButton=e,t.createInsertButton=n,t.createCutButton=r,t.createCopyButton=o,t.createPasteButton=a,t.createRunButton=s,t.createCellTypeItem=A,t.getDefaultItems=function(t){return[{name:"save",widget:e(t)},{name:"insert",widget:n(t)},{name:"cut",widget:r(t)},{name:"copy",widget:o(t)},{name:"paste",widget:a(t)},{name:"run",widget:s(t)},{name:"interrupt",widget:p.c.createInterruptButton(t.session)},{name:"restart",widget:p.c.createRestartButton(t.session)},{name:"cellType",widget:A(t)},{name:"spacer",widget:p.c.createSpacerItem()},{name:"kernelName",widget:p.c.createKernelNameItem(t.session)},{name:"kernelStatus",widget:p.c.createKernelStatusItem(t.session)}]}}(x||(x={}));class _ extends I.a{constructor(t){super(),this.handleChange=t=>{"-"!==t.target.value&&(h.changeCellType(this._notebook,t.target.value),this._notebook.activate())},this.handleKeyDown=t=>{13===t.keyCode&&this._notebook.activate()},this._notebook=null,this.addClass(Q),this._notebook=t,t.model&&this.update(),t.activeCellChanged.connect(this.update,this),t.selectionChanged.connect(this.update,this)}render(){let t="-";this._notebook.activeCell&&(t=this._notebook.activeCell.model.type);for(let e of this._notebook.widgets)if(this._notebook.isSelectedOrActive(e)&&e.model.type!==t){t="-";break}return l.createElement(C.b,{className:w,onChange:this.handleChange,onKeyDown:this.handleKeyDown,value:t,iconProps:{icon:l.createElement("span",{className:"jp-MaterialIcon jp-DownCaretIcon bp3-icon"})},"aria-label":"Cell type",minimal:!0},l.createElement("option",{value:"-"},"-"),l.createElement("option",{value:"code"},"Code"),l.createElement("option",{value:"markdown"},"Markdown"),l.createElement("option",{value:"raw"},"Raw"))}}n(72);var D=n(614),M=n(200),k=n(17);class S extends M.a.Model{constructor(t,e){super({modelDB:e}),this._defaultLang="",this._dirty=!1,this._readOnly=!1,this._contentChanged=new A.Signal(this),this._stateChanged=new A.Signal(this),this._defaultLang=t||"",this.value.changed.connect(this.triggerContentChange,this)}get contentChanged(){return this._contentChanged}get stateChanged(){return this._stateChanged}get dirty(){return this._dirty}set dirty(t){if(t===this._dirty)return;let e=this._dirty;this._dirty=t,this.triggerStateChange({name:"dirty",oldValue:e,newValue:t})}get readOnly(){return this._readOnly}set readOnly(t){if(t===this._readOnly)return;let e=this._readOnly;this._readOnly=t,this.triggerStateChange({name:"readOnly",oldValue:e,newValue:t})}get defaultKernelName(){return""}get defaultKernelLanguage(){return this._defaultLang}toString(){return this.value.text}fromString(t){this.value.text=t}toJSON(){return JSON.parse(this.value.text||"null")}fromJSON(t){this.fromString(JSON.stringify(t))}initialize(){}triggerStateChange(t){this._stateChanged.emit(t)}triggerContentChange(){this._contentChanged.emit(void 0),this.dirty=!0}}class L{constructor(t){this._isDisposed=!1,this._widgetCreated=new A.Signal(this),this._name=t.name,this._readOnly=void 0!==t.readOnly&&t.readOnly,this._defaultFor=t.defaultFor?t.defaultFor.slice():[],this._defaultRendered=(t.defaultRendered||[]).slice(),this._fileTypes=t.fileTypes.slice(),this._modelName=t.modelName||"text",this._preferKernel=!!t.preferKernel,this._canStartKernel=!!t.canStartKernel,this._shutdownOnClose=!!t.shutdownOnClose,this._toolbarFactory=t.toolbarFactory}get widgetCreated(){return this._widgetCreated}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed=!0}get readOnly(){return this._readOnly}get name(){return this._name}get fileTypes(){return this._fileTypes.slice()}get modelName(){return this._modelName}get defaultFor(){return this._defaultFor.slice()}get defaultRendered(){return this._defaultRendered.slice()}get preferKernel(){return this._preferKernel}get canStartKernel(){return this._canStartKernel}get shutdownOnClose(){return this._shutdownOnClose}set shutdownOnClose(t){this._shutdownOnClose=t}createNew(t,e){const n=this.createNewWidget(t,e);let r;return(r=this._toolbarFactory?this._toolbarFactory(n):this.defaultToolbarFactory(n)).forEach(({name:t,widget:e})=>{n.toolbar.addItem(t,e)}),this._widgetCreated.emit(n),n}defaultToolbarFactory(t){return[]}}const R="jp-mod-dirty";class N extends D.a{constructor(t){t.reveal=Promise.all([t.reveal,t.context.ready]),super(t),this.context=t.context,this.context.pathChanged.connect(this._onPathChanged,this),this._onPathChanged(this.context,this.context.path),this.context.model.stateChanged.connect(this._onModelStateChanged,this),this.context.ready.then(()=>{this._handleDirtyState()})}setFragment(t){}_onPathChanged(t,e){this.title.label=k.PathExt.basename(t.localPath)}_onModelStateChanged(t,e){"dirty"===e.name&&this._handleDirtyState()}_handleDirtyState(){this.context.model.dirty?this.title.className+=` ${R}`:this.title.className=this.title.className.replace(R,"")}}var F=n(625),T=n(9),U=n(201);class O{constructor(t,e){this._isDisposed=!1,this._cellOrder=null,this._cellMap=null,this._changed=new A.Signal(this),this._factory=null,this._factory=e,this._cellOrder=t.createList("cellOrder"),this._cellMap=new U.a,this._cellOrder.changed.connect(this._onOrderChanged,this)}get changed(){return this._changed}get isDisposed(){return this._isDisposed}get isEmpty(){return 0===this._cellOrder.length}get length(){return this._cellOrder.length}iter(){let t=[];for(let e of Object(a.toArray)(this._cellOrder))t.push(this._cellMap.get(e));return new a.ArrayIterator(t)}dispose(){if(!this._isDisposed){this._isDisposed=!0,A.Signal.clearData(this);for(let t of this._cellMap.values())t.dispose();this._cellMap.dispose(),this._cellOrder.dispose()}}get(t){return this._cellMap.get(this._cellOrder.get(t))}set(t,e){this._cellMap.set(e.id,e),this._cellOrder.set(t,e.id)}push(t){return this._cellMap.set(t.id,t),this._cellOrder.push(t.id)}insert(t,e){this._cellMap.set(e.id,e),this._cellOrder.insert(t,e.id)}removeValue(t){let e=a.ArrayExt.findFirstIndex(Object(a.toArray)(this._cellOrder),e=>this._cellMap.get(e)===t);return this.remove(e),e}remove(t){let e=this._cellOrder.get(t);return this._cellOrder.remove(t),this._cellMap.get(e)}clear(){this._cellOrder.clear()}move(t,e){this._cellOrder.move(t,e)}pushAll(t){let e=Object(a.toArray)(t);return Object(a.each)(e,t=>{this._cellMap.set(t.id,t),this._cellOrder.push(t.id)}),this.length}insertAll(t,e){let n=Object(a.toArray)(e);return Object(a.each)(n,e=>{this._cellMap.set(e.id,e),this._cellOrder.beginCompoundOperation(),this._cellOrder.insert(t++,e.id),this._cellOrder.endCompoundOperation()}),this.length}removeRange(t,e){return this._cellOrder.removeRange(t,e),this.length}get canRedo(){return this._cellOrder.canRedo}get canUndo(){return this._cellOrder.canUndo}beginCompoundOperation(t){this._cellOrder.beginCompoundOperation(t)}endCompoundOperation(){this._cellOrder.endCompoundOperation()}undo(){this._cellOrder.undo()}redo(){this._cellOrder.redo()}clearUndo(){for(let t of this._cellMap.keys())if(-1===a.ArrayExt.findFirstIndex(Object(a.toArray)(this._cellOrder),e=>e===t)){this._cellMap.get(t).dispose(),this._cellMap.delete(t)}this._cellOrder.clearUndo()}_onOrderChanged(t,e){"add"!==e.type&&"set"!==e.type||Object(a.each)(e.newValues,t=>{if(!this._cellMap.has(t)){let e;switch(this._factory.modelDB.createValue(t+".type").get()){case"code":e=this._factory.createCodeCell({id:t});break;case"markdown":e=this._factory.createMarkdownCell({id:t});break;default:e=this._factory.createRawCell({id:t})}this._cellMap.set(t,e)}});let n=[],r=[];Object(a.each)(e.newValues,t=>{n.push(this._cellMap.get(t))}),Object(a.each)(e.oldValues,t=>{r.push(this._cellMap.get(t))}),this._changed.emit({type:e.type,oldIndex:e.oldIndex,newIndex:e.newIndex,oldValues:r,newValues:n})}}class G extends S{constructor(t={}){super(t.languagePreference,t.modelDB),this._nbformat=k.nbformat.MAJOR_VERSION,this._nbformatMinor=k.nbformat.MINOR_VERSION;let e=t.contentFactory||G.defaultContentFactory;this.contentFactory=e.clone(this.modelDB.view("cells")),this._cells=new O(this.modelDB,this.contentFactory),this._cells.changed.connect(this._onCellsChanged,this);let n=this.modelDB.createMap("metadata");if(!n.has("language_info")){let e=t.languagePreference||"";n.set("language_info",{name:e})}this._ensureMetadata(),n.changed.connect(this.triggerContentChange,this),this._deletedCells=[]}get metadata(){return this.modelDB.get("metadata")}get cells(){return this._cells}get nbformat(){return this._nbformat}get nbformatMinor(){return this._nbformatMinor}get defaultKernelName(){let t=this.metadata.get("kernelspec");return t?t.name:""}get deletedCells(){return this._deletedCells}get defaultKernelLanguage(){let t=this.metadata.get("language_info");return t?t.name:""}dispose(){if(null===this.cells)return;let t=this.cells;this._cells=null,t.dispose(),super.dispose()}toString(){return JSON.stringify(this.toJSON())}fromString(t){this.fromJSON(JSON.parse(t))}toJSON(){let t=[];for(let e=0;e<this.cells.length;e++){let n=this.cells.get(e);t.push(n.toJSON())}this._ensureMetadata();let e=Object.create(null);for(let t of this.metadata.keys())e[t]=JSON.parse(JSON.stringify(this.metadata.get(t)));return{metadata:e,nbformat_minor:this._nbformatMinor,nbformat:this._nbformat,cells:t}}fromJSON(t){let e=[],n=this.contentFactory;for(let r of t.cells)switch(r.cell_type){case"code":e.push(n.createCodeCell({cell:r}));break;case"markdown":e.push(n.createMarkdownCell({cell:r}));break;case"raw":e.push(n.createRawCell({cell:r}));break;default:continue}this.cells.beginCompoundOperation(),this.cells.clear(),this.cells.pushAll(e),this.cells.endCompoundOperation();let r=0,o=0;this._nbformatMinor=k.nbformat.MINOR_VERSION,this._nbformat=k.nbformat.MAJOR_VERSION;const a=t.metadata.orig_nbformat;if(t.nbformat!==this._nbformat&&(r=this._nbformat,this._nbformat=o=t.nbformat,this.triggerStateChange({name:"nbformat",oldValue:r,newValue:o})),t.nbformat_minor>this._nbformatMinor&&(r=this._nbformatMinor,this._nbformatMinor=o=t.nbformat_minor,this.triggerStateChange({name:"nbformatMinor",oldValue:r,newValue:o})),void 0!==a&&this._nbformat!==a){const t=this._nbformat>a,e=`This notebook has been converted from ${t?"an older":"a newer"} notebook format (v${a}) to the current notebook format (v${this._nbformat}). The next time you save this notebook, the current notebook format (v${this._nbformat}) will be used. ${t?"Older versions of Jupyter may not be able to read the new format.":"Some features of the original notebook may not be available."}  To preserve the original format version, close the notebook without saving it.`;Object(i.b)({title:"Notebook converted",body:e,buttons:[i.a.okButton()]})}this.metadata.clear();let s=t.metadata;for(let t in s)"orig_nbformat"!==t&&this.metadata.set(t,s[t]);this._ensureMetadata(),this.dirty=!0}initialize(){super.initialize(),this.cells.clearUndo()}_onCellsChanged(t,e){switch(e.type){case"add":e.newValues.forEach(t=>{t.contentChanged.connect(this.triggerContentChange,this)});break;case"remove":break;case"set":e.newValues.forEach(t=>{t.contentChanged.connect(this.triggerContentChange,this)})}this.triggerContentChange()}_ensureMetadata(){let t=this.metadata;t.has("language_info")||t.set("language_info",{name:""}),t.has("kernelspec")||t.set("kernelspec",{name:"",display_name:""})}}!function(t){class e{constructor(t){this.codeCellContentFactory=t.codeCellContentFactory||F.c.defaultContentFactory,this.modelDB=t.modelDB}createCell(t,e){switch(t){case"code":return this.createCodeCell(e);case"markdown":return this.createMarkdownCell(e);case"raw":default:return this.createRawCell(e)}}createCodeCell(t){return t.contentFactory&&(t.contentFactory=this.codeCellContentFactory),this.modelDB&&(t.id||(t.id=T.UUID.uuid4()),t.modelDB=this.modelDB.view(t.id)),new F.c(t)}createMarkdownCell(t){return this.modelDB&&(t.id||(t.id=T.UUID.uuid4()),t.modelDB=this.modelDB.view(t.id)),new F.d(t)}createRawCell(t){return this.modelDB&&(t.id||(t.id=T.UUID.uuid4()),t.modelDB=this.modelDB.view(t.id)),new F.e(t)}clone(t){return new e({modelDB:t,codeCellContentFactory:this.codeCellContentFactory})}}t.ContentFactory=e,t.defaultContentFactory=new e({})}(G||(G={}));class H{constructor(t){this._disposed=!1;let e=t.codeCellContentFactory;this.contentFactory=t.contentFactory||new G.ContentFactory({codeCellContentFactory:e})}get name(){return"notebook"}get contentType(){return"notebook"}get fileFormat(){return"json"}get isDisposed(){return this._disposed}dispose(){this._disposed=!0}createNew(t,e){let n=this.contentFactory;return new G({languagePreference:t,contentFactory:n,modelDB:e})}preferredLanguage(t){return""}}var j,Y,P=n(25),K=n(27),z=n(6),J=n(613),q=n(115),V=n(384),W=n(627),X=n(204);class Z extends z.Widget{constructor(){super(),this._items=[],this.layout=new z.PanelLayout,this.addClass("jp-RankedPanel")}addWidget(t,e){const n={widget:t,rank:e},r=a.ArrayExt.upperBound(this._items,n,j.itemCmp);a.ArrayExt.insert(this._items,r,n),this.layout.insertWidget(r,t)}onChildRemoved(t){let e=a.ArrayExt.findFirstIndex(this._items,e=>e.widget===t.child);-1!==e&&a.ArrayExt.removeAt(this._items,e)}}class $ extends z.Widget{constructor(t){super(),this.addClass("jp-NotebookTools"),this._commonTools=new Z,this._advancedTools=new Z,this._advancedTools.title.label="Advanced Tools";const e=this.layout=new z.PanelLayout;e.addWidget(this._commonTools),e.addWidget(new J.a({widget:this._advancedTools})),this._tracker=t.tracker,this._tracker.currentChanged.connect(this._onActiveNotebookPanelChanged,this),this._tracker.activeCellChanged.connect(this._onActiveCellChanged,this),this._tracker.selectionChanged.connect(this._onSelectionChanged,this),this._onActiveNotebookPanelChanged(),this._onActiveCellChanged(),this._onSelectionChanged()}get activeCell(){return this._tracker.activeCell}get selectedCells(){const t=this._tracker.currentWidget;if(!t)return[];const e=t.content;return e.widgets.filter(t=>e.isSelectedOrActive(t))}get activeNotebookPanel(){return this._tracker.currentWidget}addItem(t){let e,n=t.tool,r="rank"in t?t.rank:100;e="advanced"===t.section?this._advancedTools:this._commonTools,n.addClass("jp-NotebookTools-tool"),e.addWidget(n,r),n.notebookTools=this,P.MessageLoop.sendMessage(n,$.ActiveNotebookPanelMessage),P.MessageLoop.sendMessage(n,$.ActiveCellMessage)}_onActiveNotebookPanelChanged(){this._prevActiveNotebookModel&&!this._prevActiveNotebookModel.isDisposed&&this._prevActiveNotebookModel.metadata.changed.disconnect(this._onActiveNotebookPanelMetadataChanged,this);const t=this.activeNotebookPanel&&this.activeNotebookPanel.content?this.activeNotebookPanel.content.model:null;this._prevActiveNotebookModel=t,t&&t.metadata.changed.connect(this._onActiveNotebookPanelMetadataChanged,this),Object(a.each)(this._toolChildren(),t=>{P.MessageLoop.sendMessage(t,$.ActiveNotebookPanelMessage)})}_onActiveCellChanged(){this._prevActiveCell&&!this._prevActiveCell.isDisposed&&this._prevActiveCell.metadata.changed.disconnect(this._onActiveCellMetadataChanged,this);const t=this.activeCell?this.activeCell.model:null;this._prevActiveCell=t,t&&t.metadata.changed.connect(this._onActiveCellMetadataChanged,this),Object(a.each)(this._toolChildren(),t=>{P.MessageLoop.sendMessage(t,$.ActiveCellMessage)})}_onSelectionChanged(){Object(a.each)(this._toolChildren(),t=>{P.MessageLoop.sendMessage(t,$.SelectionMessage)})}_onActiveNotebookPanelMetadataChanged(t,e){let n=new X.a.ChangeMessage("activenotebookpanel-metadata-changed",e);Object(a.each)(this._toolChildren(),t=>{P.MessageLoop.sendMessage(t,n)})}_onActiveCellMetadataChanged(t,e){let n=new X.a.ChangeMessage("activecell-metadata-changed",e);Object(a.each)(this._toolChildren(),t=>{P.MessageLoop.sendMessage(t,n)})}_toolChildren(){return Object(a.chain)(this._commonTools.children(),this._advancedTools.children())}}!function(t){t.ActiveNotebookPanelMessage=new P.ConflatableMessage("activenotebookpanel-changed"),t.ActiveCellMessage=new P.ConflatableMessage("activecell-changed"),t.SelectionMessage=new P.ConflatableMessage("selection-changed");class e extends z.Widget{dispose(){super.dispose(),this.notebookTools=null}processMessage(t){switch(super.processMessage(t),t.type){case"activenotebookpanel-changed":this.onActiveNotebookPanelChanged(t);break;case"activecell-changed":this.onActiveCellChanged(t);break;case"selection-changed":this.onSelectionChanged(t);break;case"activecell-metadata-changed":this.onActiveCellMetadataChanged(t);break;case"activenotebookpanel-metadata-changed":this.onActiveNotebookPanelMetadataChanged(t)}}onActiveNotebookPanelChanged(t){}onActiveCellChanged(t){}onSelectionChanged(t){}onActiveCellMetadataChanged(t){}onActiveNotebookPanelMetadataChanged(t){}}t.Tool=e;t.ActiveCellTool=class extends e{constructor(){super(),this._model=new M.a.Model,this.addClass("jp-ActiveCellTool"),this.addClass("jp-InputArea"),this.layout=new z.PanelLayout}dispose(){null!==this._model&&(this._model.dispose(),this._model=null,super.dispose())}onActiveCellChanged(){let t=this.notebookTools.activeCell,e=this.layout,n=e.widgets.length;for(let t=0;t<n;t++)e.widgets[0].dispose();if(this._cellModel&&!this._cellModel.isDisposed&&(this._cellModel.value.changed.disconnect(this._onValueChanged,this),this._cellModel.mimeTypeChanged.disconnect(this._onMimeTypeChanged,this)),!t){let t=new z.Widget;return t.addClass("jp-InputArea-editor"),t.addClass("jp-InputArea-editor"),e.addWidget(t),void(this._cellModel=null)}let r=t.promptNode?t.promptNode.cloneNode(!0):null,i=new z.Widget({node:r}),o=t.contentFactory.editorFactory,a=this._cellModel=t.model;a.value.changed.connect(this._onValueChanged,this),a.mimeTypeChanged.connect(this._onMimeTypeChanged,this),this._model.value.text=a.value.text.split("\n")[0],this._model.mimeType=a.mimeType;let s=this._model,A=new V.a({model:s,factory:o});A.addClass("jp-InputArea-editor"),A.addClass("jp-InputArea-editor"),A.editor.setOption("readOnly",!0),e.addWidget(i),e.addWidget(A)}_onValueChanged(){this._model.value.text=this._cellModel.value.text.split("\n")[0]}_onMimeTypeChanged(){this._model.mimeType=this._cellModel.mimeType}};class n extends e{constructor(t){super();const{editorFactory:e}=t;this.addClass("jp-MetadataEditorTool");let n=this.layout=new z.PanelLayout;this.editor=new W.a({editorFactory:e}),this.editor.title.label=t.label||"Edit Metadata";const r=new z.Widget({node:document.createElement("label")});r.node.textContent=t.label||"Edit Metadata",n.addWidget(r),n.addWidget(this.editor)}}t.MetadataEditorTool=n;t.NotebookMetadataEditorTool=class extends n{constructor(t){t.label=t.label||"Notebook Metadata",super(t)}onActiveNotebookPanelChanged(t){this._update()}onActiveNotebookPanelMetadataChanged(t){this._update()}_update(){const t=this.notebookTools.activeNotebookPanel&&this.notebookTools.activeNotebookPanel.content;this.editor.source=t?t.model.metadata:null}};t.CellMetadataEditorTool=class extends n{constructor(t){t.label=t.label||"Cell Metadata",super(t)}onActiveCellChanged(t){this._update()}onActiveCellMetadataChanged(t){this._update()}_update(){let t=this.notebookTools.activeCell;this.editor.source=t?t.model.metadata:null}};class r extends e{constructor(t){super({node:j.createSelectorNode(t)}),this._getValue=t=>{let e=t.model.metadata.get(this.key);return void 0===e&&(e=this._default),e},this._setValue=(t,e)=>{e===this._default?t.model.metadata.delete(this.key):t.model.metadata.set(this.key,e)},this._changeGuard=!1,this.addClass("jp-KeySelector"),this.key=t.key,this._default=t.default,this._validCellTypes=t.validCellTypes||[],this._getter=t.getter||this._getValue,this._setter=t.setter||this._setValue}get selectNode(){return this.node.getElementsByTagName("select")[0]}handleEvent(t){switch(t.type){case"change":this.onValueChanged()}}onAfterAttach(t){this.selectNode.addEventListener("change",this)}onBeforeDetach(t){this.selectNode.removeEventListener("change",this)}onActiveCellChanged(t){let e=this.selectNode,n=this.notebookTools.activeCell;if(!n)return e.disabled=!0,void(e.value="");let r=n.model.type;if(this._validCellTypes.length&&-1===this._validCellTypes.indexOf(r))return e.value=void 0,void(e.disabled=!0);e.disabled=!1,this._changeGuard=!0;let i=this._getter;e.value=JSON.stringify(i(n)),this._changeGuard=!1}onActiveCellMetadataChanged(t){if(this._changeGuard)return;let e=this.selectNode,n=this.notebookTools.activeCell;if(t.args.key===this.key&&n){this._changeGuard=!0;let t=this._getter;e.value=JSON.stringify(t(n)),this._changeGuard=!1}}onValueChanged(){let t=this.notebookTools.activeCell;if(!t||this._changeGuard)return;this._changeGuard=!0;let e=this.selectNode;(0,this._setter)(t,JSON.parse(e.value)),this._changeGuard=!1}}t.KeySelector=r,t.createSlideShowSelector=function(){return new r({key:"slideshow",title:"Slide Type",optionsMap:{"-":null,Slide:"slide","Sub-Slide":"subslide",Fragment:"fragment",Skip:"skip",Notes:"notes"},getter:t=>{let e=t.model.metadata.get("slideshow");return e&&e.slide_type},setter:(t,e)=>{let n=t.model.metadata.get("slideshow")||Object.create(null);null===e?delete(n=Object.assign({},n)).slide_type:n=Object.assign({},n,{slide_type:e}),Object.keys(n).length>0?t.model.metadata.set("slideshow",n):t.model.metadata.delete("slideshow")}})},t.createNBConvertSelector=function(t){return new r({key:"raw_mimetype",title:"Raw NBConvert Format",optionsMap:t,validCellTypes:["raw"]})}}($||($={})),(Y=j||(j={})).itemCmp=function(t,e){return t.rank-e.rank},Y.createSelectorNode=function(t){let e=t.key,n=t.title||e[0].toLocaleUpperCase()+e.slice(1),r=[];for(let e in t.optionsMap){let n=JSON.stringify(t.optionsMap[e]);r.push(K.h.option({value:n},e))}let i=K.VirtualDOM.realize(K.h.div({},K.h.label(n,K.h.select({},r))));return q.a.styleNode(i),i};var tt=n(195),et=n(43),nt=n(127);const rt="jpKernelUser",it="jpCodeRunner",ot="jpUndoer",at="jp-Notebook",st="jp-Notebook-cell",At="jp-mod-editMode",lt="jp-mod-commandMode",ct="jp-mod-active",ut="jp-mod-selected",gt="jp-mod-multiSelected",ht="jp-mod-unconfined",dt="jp-mod-dropTarget",ft="jp-mod-dropSource",pt="jp-dragImage",It="jp-dragImage-singlePrompt",Ct="jp-dragImage-content",mt="jp-dragImage-prompt",yt="jp-dragImage-multipleBack",Et="application/vnd.jupyter.cells",vt=5;class Bt extends z.Widget{constructor(t){super(),this._editorConfig=Bt.defaultEditorConfig,this._notebookConfig=Bt.defaultNotebookConfig,this._mimetype="text/plain",this._model=null,this._modelChanged=new A.Signal(this),this._modelContentChanged=new A.Signal(this),this.addClass(at),this.node.dataset[rt]="true",this.node.dataset[ot]="true",this.rendermime=t.rendermime,this.layout=new Qt.NotebookPanelLayout,this.contentFactory=t.contentFactory||Bt.defaultContentFactory,this.editorConfig=t.editorConfig||Bt.defaultEditorConfig,this.notebookConfig=t.notebookConfig||Bt.defaultNotebookConfig,this._mimetypeService=t.mimeTypeService}get modelChanged(){return this._modelChanged}get modelContentChanged(){return this._modelContentChanged}get model(){return this._model}set model(t){if(t=t||null,this._model===t)return;let e=this._model;this._model=t,e&&e.modelDB.isCollaborative&&e.modelDB.connected.then(()=>{e.modelDB.collaborators.changed.disconnect(this._onCollaboratorsChanged,this)}),t&&t.modelDB.isCollaborative&&t.modelDB.connected.then(()=>{t.modelDB.collaborators.changed.connect(this._onCollaboratorsChanged,this)}),this._onModelChanged(e,t),this.onModelChanged(e,t),this._modelChanged.emit(void 0)}get codeMimetype(){return this._mimetype}get widgets(){return this.layout.widgets}get editorConfig(){return this._editorConfig}set editorConfig(t){this._editorConfig=t,this._updateEditorConfig()}get notebookConfig(){return this._notebookConfig}set notebookConfig(t){this._notebookConfig=t,this._updateNotebookConfig()}dispose(){this.isDisposed||(this._model=null,super.dispose())}onModelChanged(t,e){}onModelContentChanged(t,e){this._modelContentChanged.emit(void 0)}onMetadataChanged(t,e){switch(e.key){case"language_info":this._updateMimetype()}}onCellInserted(t,e){}onCellMoved(t,e){}onCellRemoved(t,e){}_onModelChanged(t,e){let n=this.layout;if(t)for(t.cells.changed.disconnect(this._onCellsChanged,this),t.metadata.changed.disconnect(this.onMetadataChanged,this),t.contentChanged.disconnect(this.onModelContentChanged,this);n.widgets.length;)this._removeCell(0);if(!e)return void(this._mimetype="text/plain");this._updateMimetype();let r=e.cells;r.length||r.push(e.contentFactory.createCell(this.notebookConfig.defaultCell,{})),Object(a.each)(r,(t,e)=>{this._insertCell(e,t)}),r.changed.connect(this._onCellsChanged,this),e.contentChanged.connect(this.onModelContentChanged,this),e.metadata.changed.connect(this.onMetadataChanged,this)}_onCellsChanged(t,e){let n=0;switch(e.type){case"add":n=e.newIndex,Object(a.each)(e.newValues,t=>{this._insertCell(n++,t)});break;case"move":this._moveCell(e.oldIndex,e.newIndex);break;case"remove":if(Object(a.each)(e.oldValues,t=>{this._removeCell(e.oldIndex)}),!t.length){const t=this.model;requestAnimationFrame(()=>{t.isDisposed||t.cells.length||t.cells.push(t.contentFactory.createCell(this.notebookConfig.defaultCell,{}))})}break;case"set":n=e.newIndex,Object(a.each)(e.newValues,t=>{this._insertCell(n,t),this._removeCell(n+1),n++});break;default:return}}_insertCell(t,e){let n;switch(e.type){case"code":(n=this._createCodeCell(e)).model.mimeType=this._mimetype;break;case"markdown":n=this._createMarkdownCell(e),""===e.value.text&&(n.rendered=!1);break;default:n=this._createRawCell(e)}n.addClass(st),this.layout.insertWidget(t,n),this.onCellInserted(t,n)}_createCodeCell(t){let e=this.rendermime,n=this.contentFactory;let r={editorConfig:this.editorConfig.code,model:t,rendermime:e,contentFactory:n,updateEditorOnShow:!1};const i=this.contentFactory.createCodeCell(r,this);return i.syncCollapse=!0,i.syncEditable=!0,i.syncScrolled=!0,i}_createMarkdownCell(t){let e=this.rendermime,n=this.contentFactory;let r={editorConfig:this.editorConfig.markdown,model:t,rendermime:e,contentFactory:n,updateEditorOnShow:!1};const i=this.contentFactory.createMarkdownCell(r,this);return i.syncCollapse=!0,i.syncEditable=!0,i}_createRawCell(t){let e=this.contentFactory;let n={editorConfig:this.editorConfig.raw,model:t,contentFactory:e,updateEditorOnShow:!1};const r=this.contentFactory.createRawCell(n,this);return r.syncCollapse=!0,r.syncEditable=!0,r}_moveCell(t,e){let n=this.layout;n.insertWidget(e,n.widgets[t]),this.onCellMoved(t,e)}_removeCell(t){let e=this.layout.widgets[t];e.parent=null,this.onCellRemoved(t,e),e.dispose()}_updateMimetype(){let t=this._model.metadata.get("language_info");t&&(this._mimetype=this._mimetypeService.getMimeTypeByLanguage(t),Object(a.each)(this.widgets,t=>{"code"===t.model.type&&(t.model.mimeType=this._mimetype)}))}_onCollaboratorsChanged(){for(let t=0;t<this.widgets.length;t++){let e=this.widgets[t];for(let t of e.model.selections.keys())this._model.modelDB.collaborators.has(t)||e.model.selections.delete(t)}}_updateEditorConfig(){for(let t=0;t<this.widgets.length;t++){const e=this.widgets[t];let n;switch(e.model.type){case"code":n=this._editorConfig.code;break;case"markdown":n=this._editorConfig.markdown;break;default:n=this._editorConfig.raw}Object.keys(n).forEach(t=>{e.editor.setOption(t,n[t])}),e.editor.refresh()}}_updateNotebookConfig(){this.toggleClass("jp-mod-scrollPastEnd",this._notebookConfig.scrollPastEnd)}}!function(t){t.defaultEditorConfig={code:Object.assign({},M.a.defaultConfig,{lineWrap:"off",matchBrackets:!0,autoClosingBrackets:!0}),markdown:Object.assign({},M.a.defaultConfig,{lineWrap:"on",matchBrackets:!1,autoClosingBrackets:!1}),raw:Object.assign({},M.a.defaultConfig,{lineWrap:"on",matchBrackets:!1,autoClosingBrackets:!1})},t.defaultNotebookConfig={scrollPastEnd:!0,defaultCell:"code"};class e extends o.a.ContentFactory{createCodeCell(t,e){return t.contentFactory||(t.contentFactory=this),new o.b(t).initializeState()}createMarkdownCell(t,e){return t.contentFactory||(t.contentFactory=this),new o.c(t).initializeState()}createRawCell(t,e){return t.contentFactory||(t.contentFactory=this),new o.d(t).initializeState()}}t.ContentFactory=e,t.defaultContentFactory=new e}(Bt||(Bt={}));class bt extends Bt{constructor(t){super(Qt.processNotebookOptions(t)),this._activeCellIndex=-1,this._activeCell=null,this._mode="command",this._drag=null,this._fragment="",this._dragData=null,this._mouseMode=null,this._activeCellChanged=new A.Signal(this),this._stateChanged=new A.Signal(this),this._selectionChanged=new A.Signal(this),this._checkCacheOnNextResize=!1,this.node.tabIndex=-1,this.node.setAttribute("data-p-dragscroll","true")}get activeCellChanged(){return this._activeCellChanged}get stateChanged(){return this._stateChanged}get selectionChanged(){return this._selectionChanged}get mode(){return this._mode}set mode(t){let e=this.activeCell;if(e||(t="command"),t===this._mode)return void this._ensureFocus();this.update();let n=this._mode;this._mode=t,"edit"===t?(Object(a.each)(this.widgets,t=>{this.deselect(t)}),e instanceof o.c&&(e.rendered=!1),e.inputHidden=!1):this.node.focus(),this._stateChanged.emit({name:"mode",oldValue:n,newValue:t}),this._ensureFocus()}get activeCellIndex(){return this.model&&this.model.cells.length?this._activeCellIndex:-1}set activeCellIndex(t){let e=this._activeCellIndex;this.model&&this.model.cells.length?(t=Math.max(t,0),t=Math.min(t,this.model.cells.length-1)):t=-1,this._activeCellIndex=t;let n=this.widgets[t];n!==this._activeCell&&(this.update(),this._activeCell=n,this._activeCellChanged.emit(n)),"edit"===this.mode&&n instanceof o.c&&(n.rendered=!1),this._ensureFocus(),t!==e&&(this._trimSelections(),this._stateChanged.emit({name:"activeCellIndex",oldValue:e,newValue:t}))}get activeCell(){return this._activeCell}dispose(){this.isDisposed||(this._activeCell=null,super.dispose())}select(t){Qt.selectedProperty.get(t)||(Qt.selectedProperty.set(t,!0),this._selectionChanged.emit(void 0),this.update())}deselect(t){Qt.selectedProperty.get(t)&&(Qt.selectedProperty.set(t,!1),this._selectionChanged.emit(void 0),this.update())}isSelected(t){return Qt.selectedProperty.get(t)}isSelectedOrActive(t){return t===this._activeCell||Qt.selectedProperty.get(t)}deselectAll(){let t=!1;Object(a.each)(this.widgets,e=>{Qt.selectedProperty.get(e)&&(t=!0),Qt.selectedProperty.set(e,!1)}),t&&this._selectionChanged.emit(void 0),this.activeCellIndex=this.activeCellIndex,this.update()}extendContiguousSelectionTo(t){let e,{head:n,anchor:r}=this.getContiguousSelection();if(null===r){if(t===this.activeCellIndex)return;n=this.activeCellIndex,r=this.activeCellIndex}if(this.activeCellIndex=t,(t=this.activeCellIndex)===r)return void this.deselectAll();let i=!1;if(n<t)for(n<r&&(Qt.selectedProperty.set(this.widgets[n],!1),i=!0),e=n+1;e<t;e++)e!==r&&(Qt.selectedProperty.set(this.widgets[e],!Qt.selectedProperty.get(this.widgets[e])),i=!0);else if(t<n)for(r<n&&(Qt.selectedProperty.set(this.widgets[n],!1),i=!0),e=t+1;e<n;e++)e!==r&&(Qt.selectedProperty.set(this.widgets[e],!Qt.selectedProperty.get(this.widgets[e])),i=!0);Qt.selectedProperty.get(this.widgets[r])||(i=!0),Qt.selectedProperty.set(this.widgets[r],!0),Qt.selectedProperty.get(this.widgets[t])||(i=!0),Qt.selectedProperty.set(this.widgets[t],!0),i&&this._selectionChanged.emit(void 0)}getContiguousSelection(){let t=this.widgets,e=a.ArrayExt.findFirstIndex(t,t=>this.isSelected(t));if(-1===e)return{head:null,anchor:null};let n=a.ArrayExt.findLastIndex(t,t=>this.isSelected(t),-1,e);for(let r=e;r<=n;r++)if(!this.isSelected(t[r]))throw new Error("Selection not contiguous");let r=this.activeCellIndex;if(e!==r&&n!==r)throw new Error("Active cell not at endpoint of selection");return e===r?{head:e,anchor:n}:{head:n,anchor:e}}scrollToPosition(t,e=25){let n=this.node,r=n.getBoundingClientRect(),i=t-r.top-r.height/2;Math.abs(i)>r.height*e/100&&(n.scrollTop+=i)}setFragment(t){Promise.all(this.widgets.map(t=>t.ready)).then(()=>{this._fragment=t,this.update()})}handleEvent(t){if(this.model)switch(t.type){case"contextmenu":t.eventPhase===Event.CAPTURING_PHASE&&this._evtContextMenuCapture(t);break;case"mousedown":t.eventPhase===Event.CAPTURING_PHASE?this._evtMouseDownCapture(t):this._evtMouseDown(t);break;case"mouseup":t.currentTarget===document&&this._evtDocumentMouseup(t);break;case"mousemove":t.currentTarget===document&&this._evtDocumentMousemove(t);break;case"keydown":this._ensureFocus(!0);break;case"dblclick":this._evtDblClick(t);break;case"focusin":this._evtFocusIn(t);break;case"focusout":this._evtFocusOut(t);break;case"p-dragenter":this._evtDragEnter(t);break;case"p-dragleave":this._evtDragLeave(t);break;case"p-dragover":this._evtDragOver(t);break;case"p-drop":this._evtDrop(t)}}onAfterAttach(t){super.onAfterAttach(t);let e=this.node;e.addEventListener("contextmenu",this,!0),e.addEventListener("mousedown",this,!0),e.addEventListener("mousedown",this),e.addEventListener("keydown",this),e.addEventListener("dblclick",this),e.addEventListener("focusin",this),e.addEventListener("focusout",this),e.addEventListener("p-dragenter",this,!0),e.addEventListener("p-dragleave",this,!0),e.addEventListener("p-dragover",this,!0),e.addEventListener("p-drop",this,!0)}onBeforeDetach(t){let e=this.node;e.removeEventListener("contextmenu",this,!0),e.removeEventListener("mousedown",this,!0),e.removeEventListener("mousedown",this),e.removeEventListener("keydown",this),e.removeEventListener("dblclick",this),e.removeEventListener("focusin",this),e.removeEventListener("focusout",this),e.removeEventListener("p-dragenter",this,!0),e.removeEventListener("p-dragleave",this,!0),e.removeEventListener("p-dragover",this,!0),e.removeEventListener("p-drop",this,!0),document.removeEventListener("mousemove",this,!0),document.removeEventListener("mouseup",this,!0)}onAfterShow(t){this._checkCacheOnNextResize=!0}onResize(t){if(!this._checkCacheOnNextResize)return super.onResize(t);this._checkCacheOnNextResize=!1;const e=this._cellLayoutStateCache,n=parseInt(this.node.style.width,10);if(!e||n!==e.width){this._cellLayoutStateCache={width:n};for(let t of this.widgets)t instanceof o.a&&t.editorWidget.update()}}onBeforeHide(t){const e=parseInt(this.node.style.width,10);this._cellLayoutStateCache={width:e}}onActivateRequest(t){this._ensureFocus(!0)}onUpdateRequest(t){let e=this.activeCell;"edit"===this.mode?(this.addClass(At),this.removeClass(lt)):(this.addClass(lt),this.removeClass(At)),e&&e.addClass(ct);let n=0;if(Object(a.each)(this.widgets,t=>{t!==e&&t.removeClass(ct),t.removeClass(gt),this.isSelectedOrActive(t)?(t.addClass(ut),n++):t.removeClass(ut)}),n>1&&e.addClass(gt),this._fragment){let t;try{t=this.node.querySelector(this._fragment)}catch(t){console.warn("Unable to set URI fragment identifier",t)}t&&t.scrollIntoView(),this._fragment=""}}onCellInserted(t,e){if(this.model&&this.model.modelDB.isCollaborative){let t=this.model.modelDB;t.connected.then(()=>{if(!e.isDisposed){let n=t.collaborators.localCollaborator;e.editor.uuid=n.sessionId,e.editor.selectionStyle=Object.assign({},M.a.defaultSelectionStyle,{color:n.color})}})}e.editor.edgeRequested.connect(this._onEdgeRequest,this),this.activeCellIndex=t<=this.activeCellIndex?this.activeCellIndex+1:this.activeCellIndex}onCellMoved(t,e){let n=this.activeCellIndex;t===n?this.activeCellIndex=e:t<n&&n<=e?this.activeCellIndex--:e<=n&&n<t&&this.activeCellIndex++}onCellRemoved(t,e){this.activeCellIndex=t<=this.activeCellIndex?this.activeCellIndex-1:this.activeCellIndex,this.isSelected(e)&&this._selectionChanged.emit(void 0)}onModelChanged(t,e){super.onModelChanged(t,e),this.activeCellIndex=0}_onEdgeRequest(t,e){let n=this.activeCellIndex;if("top"===e){if(this.activeCellIndex--,this.activeCellIndex<n){let t=this.activeCell.editor,e=t.lineCount-1;t.setCursorPosition({line:e,column:0})}}else if("bottom"===e&&(this.activeCellIndex++,this.activeCellIndex>n)){this.activeCell.editor.setCursorPosition({line:0,column:0})}this.mode="edit"}_ensureFocus(t=!1){let e=this.activeCell;"edit"===this.mode&&e&&(e.editor.hasFocus()||e.editor.focus()),t&&!this.node.contains(document.activeElement)&&this.node.focus()}_findCell(t){for(;t&&t!==this.node;){if(t.classList.contains(st)){let e=a.ArrayExt.findFirstIndex(this.widgets,e=>e.node===t);if(-1!==e)return e;break}t=t.parentElement}return-1}_evtContextMenuCapture(t){if(t.shiftKey)return;let e=t.target,n=this._findCell(e);-1===n&&(e=document.elementFromPoint(t.clientX,t.clientY),n=this._findCell(e));let r=this.widgets[n];r&&r.editorWidget.node.contains(e)&&t.preventDefault()}_evtMouseDownCapture(t){const{button:e,shiftKey:n}=t;let r=t.target,i=this._findCell(r);-1===i&&(r=document.elementFromPoint(t.clientX,t.clientY),i=this._findCell(r));let o=this.widgets[i];2===e&&!n&&o&&o.editorWidget.node.contains(r)&&(this.mode="command",t.preventDefault())}_evtMouseDown(t){const{button:e,shiftKey:n}=t;if(0!==e&&2!==e)return;if(n&&2===e)return;let r=t.target,i=this._findCell(r);-1===i&&(r=document.elementFromPoint(t.clientX,t.clientY),i=this._findCell(r));let o,a=this.widgets[i];if("input"!==(o=a?a.editorWidget.node.contains(r)?"input":a.promptNode.contains(r)?"prompt":"cell":"notebook")&&(this.mode="command"),"notebook"===o)this.deselectAll();else if("prompt"===o||"cell"===o){const r=""!==window.getSelection().toString();if(0===e&&n&&!r){t.preventDefault();try{this.extendContiguousSelectionTo(i)}catch(t){return console.error(t),void this.deselectAll()}this._mouseMode="select",document.addEventListener("mouseup",this,!0),document.addEventListener("mousemove",this,!0)}else 0!==e||n?2===e&&(this.isSelectedOrActive(a)||(this.deselectAll(),this.activeCellIndex=i),t.preventDefault()):"prompt"===o&&this.isSelectedOrActive(a)?(this._dragData={pressX:t.clientX,pressY:t.clientY,index:i},this._mouseMode="couldDrag",document.addEventListener("mouseup",this,!0),document.addEventListener("mousemove",this,!0),t.preventDefault()):(this.deselectAll(),this.activeCellIndex=i)}else"input"===o&&(2!==e||this.isSelectedOrActive(a)||(this.deselectAll(),this.activeCellIndex=i));this._ensureFocus(!0)}_evtDocumentMouseup(t){if(document.removeEventListener("mousemove",this,!0),document.removeEventListener("mouseup",this,!0),"couldDrag"===this._mouseMode){let e=t.target,n=this._findCell(e);-1===n&&(e=document.elementFromPoint(t.clientX,t.clientY),n=this._findCell(e)),this.deselectAll(),this.activeCellIndex=n}this._mouseMode=null,t.preventDefault(),t.stopPropagation()}_evtDocumentMousemove(t){switch(t.preventDefault(),t.stopPropagation(),this._mouseMode){case"select":let e=t.target,n=this._findCell(e);-1!==n&&this.extendContiguousSelectionTo(n);break;case"couldDrag":let r=this._dragData,i=Math.abs(t.clientX-r.pressX),o=Math.abs(t.clientY-r.pressY);(i>=vt||o>=vt)&&(this._mouseMode=null,this._startDrag(r.index,t.clientX,t.clientY))}}_evtDragEnter(t){if(!t.mimeData.hasData(Et))return;t.preventDefault(),t.stopPropagation();let e=t.target,n=this._findCell(e);-1!==n&&this.layout.widgets[n].node.classList.add(dt)}_evtDragLeave(t){if(!t.mimeData.hasData(Et))return;t.preventDefault(),t.stopPropagation();let e=this.node.getElementsByClassName(dt);e.length&&e[0].classList.remove(dt)}_evtDragOver(t){if(!t.mimeData.hasData(Et))return;t.preventDefault(),t.stopPropagation(),t.dropAction=t.proposedAction;let e=this.node.getElementsByClassName(dt);e.length&&e[0].classList.remove(dt);let n=t.target,r=this._findCell(n);-1!==r&&this.layout.widgets[r].node.classList.add(dt)}_evtDrop(t){if(!t.mimeData.hasData(Et))return;if(t.preventDefault(),t.stopPropagation(),"none"===t.proposedAction)return void(t.dropAction="none");let e=t.target;for(;e&&e.parentElement;){if(e.classList.contains(dt)){e.classList.remove(dt);break}e=e.parentElement}if(t.source===this){t.dropAction="move";let n=t.mimeData.getData("internal:cells"),r=a.ArrayExt.firstIndexOf(this.widgets,n[0]),i=this._findCell(e);if(-1!==i&&i>r?i-=1:-1===i&&(i=this.widgets.length-1),i>=r&&i<r+n.length)return;this.model.cells.beginCompoundOperation(),r<i?Object(a.each)(n,t=>{this.model.cells.move(r,i)}):r>i&&Object(a.each)(n,t=>{this.model.cells.move(r++,i++)}),this.model.cells.endCompoundOperation()}else{t.dropAction="copy";let n=this._findCell(e);-1===n&&(n=this.widgets.length);let r=n,i=this.model,o=t.mimeData.getData(Et),s=i.contentFactory;i.cells.beginCompoundOperation(),Object(a.each)(o,t=>{let e;switch(t.cell_type){case"code":e=s.createCodeCell({cell:t});break;case"markdown":e=s.createMarkdownCell({cell:t});break;default:e=s.createRawCell({cell:t})}i.cells.insert(n++,e)}),i.cells.endCompoundOperation(),this.deselectAll(),this.activeCellIndex=r,this.extendContiguousSelectionTo(n-1)}}_startDrag(t,e,n){let r=this.model.cells,i=[],o=[];Object(a.each)(this.widgets,(t,e)=>{let n=r.get(e);this.isSelectedOrActive(t)&&(t.addClass(ft),i.push(n.toJSON()),o.push(t))});let s,A=this.activeCell,l=null;if("code"===A.model.type){let t=A.model.executionCount;s=" ",t&&(s=t.toString())}else s="";l=Qt.createDragImage(i.length,s,A.model.value.text.split("\n")[0].slice(0,26)),this._drag=new nt.Drag({mimeData:new T.MimeData,dragImage:l,supportedActions:"copy-move",proposedAction:"copy",source:this}),this._drag.mimeData.setData(Et,i),this._drag.mimeData.setData("internal:cells",o);const c=o.map(t=>t.model.value.text).join("\n");this._drag.mimeData.setData("text/plain",c),document.removeEventListener("mousemove",this,!0),document.removeEventListener("mouseup",this,!0),this._mouseMode=null,this._drag.start(e,n).then(t=>{this.isDisposed||(this._drag=null,Object(a.each)(o,t=>{t.removeClass(ft)}))})}_evtFocusIn(t){let e=t.target,n=this._findCell(e);if(-1!==n){let t=this.widgets[n];t.editorWidget.node.contains(e)||(this.mode="command"),this.activeCellIndex=n,t.editorWidget.node.contains(e)&&(this.mode="edit")}else this.mode="command";"command"===this.mode&&e!==this.node?delete this.node.dataset[it]:this.node.dataset[it]="true"}_evtFocusOut(t){let e=t.relatedTarget;if(!e)return;const n=this._findCell(e);if(-1!==n){if(this.widgets[n].editorWidget.node.contains(e))return}"command"!==this.mode&&(this.mode="command",e&&e.focus())}_evtDblClick(t){let e=this.model;if(!e)return;this.deselectAll();let n=t.target,r=this._findCell(n);if(-1===r&&(n=document.elementFromPoint(t.clientX,t.clientY),r=this._findCell(n)),-1!==r)if(this.activeCellIndex=r,"markdown"===e.cells.get(r).type){this.widgets[r].rendered=!1}else"img"===n.localName&&n.classList.toggle(ht)}_trimSelections(){for(let t=0;t<this.widgets.length;t++)if(t!==this._activeCellIndex){let e=this.widgets[t];e.model.selections.delete(e.editor.uuid)}}}var Qt;!function(t){class e extends Bt.ContentFactory{}t.ContentFactory=e,t.defaultContentFactory=new e}(bt||(bt={})),function(t){t.selectedProperty=new et.AttachedProperty({name:"selected",create:()=>!1});t.NotebookPanelLayout=class extends z.PanelLayout{onUpdateRequest(t){}},t.createDragImage=function(t,e,n){return t>1?""!==e?K.VirtualDOM.realize(K.h.div(K.h.div({className:pt},K.h.span({className:mt},"["+e+"]:"),K.h.span({className:Ct},n)),K.h.div({className:yt},""))):K.VirtualDOM.realize(K.h.div(K.h.div({className:pt},K.h.span({className:mt}),K.h.span({className:Ct},n)),K.h.div({className:yt},""))):""!==e?K.VirtualDOM.realize(K.h.div(K.h.div({className:`${pt} ${It}`},K.h.span({className:mt},"["+e+"]:"),K.h.span({className:Ct},n)))):K.VirtualDOM.realize(K.h.div(K.h.div({className:`${pt} ${It}`},K.h.span({className:mt}),K.h.span({className:Ct},n))))},t.processNotebookOptions=function(t){return t.contentFactory?t:{rendermime:t.rendermime,languagePreference:t.languagePreference,contentFactory:bt.defaultContentFactory,mimeTypeService:t.mimeTypeService}}}(Qt||(Qt={}));const wt="jp-NotebookPanel",xt="jp-NotebookPanel-toolbar",_t="jp-NotebookPanel-notebook";class Dt extends N{constructor(t){super(t),this._activated=new A.Signal(this),this._autorestarting=!1,this.addClass(wt),this.toolbar.addClass(xt),this.content.addClass(_t),this.content.model=this.context.model,this.context.session.kernelChanged.connect(this._onKernelChanged,this),this.context.session.statusChanged.connect(this._onSessionStatusChanged,this),this.revealed.then(()=>{if(!this.isDisposed&&1===this.content.widgets.length){let t=this.content.widgets[0].model;"code"===t.type&&""===t.value.text&&(this.content.mode="edit")}})}get activated(){return this._activated}get session(){return this.context.session}get model(){return this.content?this.content.model:null}setConfig(t){this.content.editorConfig=t.editorConfig,this.content.notebookConfig=t.notebookConfig;const e=this.context.session.kernelPreference;this.context.session.kernelPreference=Object.assign({},e,{shutdownOnClose:t.kernelShutdown})}setFragment(t){this.context.ready.then(()=>{this.content.setFragment(t)})}dispose(){this.content.dispose(),super.dispose()}onActivateRequest(t){super.onActivateRequest(t),this._activated.emit(void 0)}[tt.a.symbol](){return async()=>{this.context.model.dirty&&!this.context.model.readOnly&&await this.context.save(),await tt.a.printURL(k.PageConfig.getNBConvertURL({format:"html",download:!1,path:this.context.path}))}}_onKernelChanged(t,e){if(!this.model||!e.newValue)return;let{newValue:n}=e;n.ready.then(()=>{this.model&&this.context.session.kernel===n&&this._updateLanguage(n.info.language_info)}),this._updateSpec(n)}_onSessionStatusChanged(t,e){"autorestarting"!==e||this._autorestarting?"restarting"===e||(this._autorestarting=!1):(Object(i.b)({title:"Kernel Restarting",body:`The kernel for ${this.session.path} appears to have died. It will restart automatically.`,buttons:[i.a.okButton()]}),this._autorestarting=!0)}_updateLanguage(t){this.model.metadata.set("language_info",t)}_updateSpec(t){return t.getSpec().then(e=>{this.isDisposed||this.model.metadata.set("kernelspec",{name:t.name,display_name:e.display_name,language:e.language})})}}!function(t){class e extends bt.ContentFactory{createNotebook(t){return new bt(t)}}t.ContentFactory=e,t.defaultContentFactory=new e,t.IContentFactory=new T.Token("@jupyterlab/notebook:IContentFactory")}(Dt||(Dt={}));var Mt=n(243);class kt extends Mt.a{constructor(){super(...arguments),this._activeCell=null,this._activeCellChanged=new A.Signal(this),this._selectionChanged=new A.Signal(this)}get activeCell(){let t=this.currentWidget;return t&&t.content.activeCell||null}get activeCellChanged(){return this._activeCellChanged}get selectionChanged(){return this._selectionChanged}add(t){const e=super.add(t);return t.content.activeCellChanged.connect(this._onActiveCellChanged,this),t.content.selectionChanged.connect(this._onSelectionChanged,this),e}dispose(){this._activeCell=null,super.dispose()}onCurrentChanged(t){let e=this.activeCell;e&&e===this._activeCell||(this._activeCell=e,t&&this._activeCellChanged.emit(t.content.activeCell||null))}_onActiveCellChanged(t,e){this.currentWidget&&this.currentWidget.content===t&&(this._activeCell=e||null,this._activeCellChanged.emit(this._activeCell))}_onSelectionChanged(t){this.currentWidget&&this.currentWidget.content===t&&this._selectionChanged.emit(void 0)}}class St extends L{constructor(t){super(t),this.rendermime=t.rendermime,this.contentFactory=t.contentFactory||Dt.defaultContentFactory,this.mimeTypeService=t.mimeTypeService,this._editorConfig=t.editorConfig||Bt.defaultEditorConfig,this._notebookConfig=t.notebookConfig||Bt.defaultNotebookConfig}get editorConfig(){return this._editorConfig}set editorConfig(t){this._editorConfig=t}get notebookConfig(){return this._notebookConfig}set notebookConfig(t){this._notebookConfig=t}createNewWidget(t,e){let n={rendermime:e?e.content.rendermime:this.rendermime.clone({resolver:t.urlResolver}),contentFactory:this.contentFactory,mimeTypeService:this.mimeTypeService,editorConfig:e?e.content.editorConfig:this._editorConfig,notebookConfig:e?e.content.notebookConfig:this._notebookConfig},r=this.contentFactory.createNotebook(n);return new Dt({context:t,content:r})}defaultToolbarFactory(t){return x.getDefaultItems(t)}}var Lt=n(164);function Rt(t){return l.createElement(Lt.b,{source:`Mode: ${k.Text.titleCase(t.notebookMode)}`})}class Nt extends I.d{constructor(){super(),this.model=new Nt.Model}render(){return this.model?(this.node.title=`Notebook is in ${this.model.notebookMode} mode`,l.createElement(Rt,{notebookMode:this.model.notebookMode})):null}}(Nt||(Nt={})).Model=class extends I.c{constructor(){super(...arguments),this._onChanged=t=>{const e=this._notebookMode;this._notebook?this._notebookMode=t.mode:this._notebookMode="command",this._triggerChange(e,this._notebookMode)},this._notebookMode="command",this._notebook=null}get notebookMode(){return this._notebookMode}set notebook(t){const e=this._notebook;null!==e&&(e.stateChanged.disconnect(this._onChanged,this),e.activeCellChanged.disconnect(this._onChanged,this),e.modelContentChanged.disconnect(this._onChanged,this));const n=this._notebookMode;this._notebook=t,null===this._notebook?this._notebookMode="command":(this._notebookMode=this._notebook.mode,this._notebook.stateChanged.connect(this._onChanged,this),this._notebook.activeCellChanged.connect(this._onChanged,this),this._notebook.modelContentChanged.connect(this._onChanged,this)),this._triggerChange(n,this._notebookMode)}_triggerChange(t,e){t!==e&&this.stateChanged.emit(void 0)}};var Ft=n(1453);function Tt(t){return t.allCellsTrusted?c.a.createElement(Ft.a,{name:"trusted",top:"2px",kind:"statusBar"}):c.a.createElement(Ft.a,{name:"not-trusted",top:"2px",kind:"statusBar"})}class Ut extends I.d{constructor(){super(),this.model=new Ut.Model}render(){return this.model?(this.node.title=(t=this.model,t.trustedCells===t.totalCells?[`Notebook trusted: ${t.trustedCells} of ${t.totalCells} cells trusted.`,"jp-StatusItem-trusted"]:t.activeCellTrusted?[`Active cell trusted: ${t.trustedCells} of ${t.totalCells} cells trusted. `,"jp-StatusItem-trusted"]:[`Notebook not trusted: ${t.trustedCells} of ${t.totalCells} cells trusted.`,"jp-StatusItem-untrusted"])[0],c.a.createElement("div",null,c.a.createElement(Tt,{allCellsTrusted:this.model.trustedCells===this.model.totalCells,activeCellTrusted:this.model.activeCellTrusted,totalCells:this.model.totalCells,trustedCells:this.model.trustedCells}))):null;var t}}(Ut||(Ut={})).Model=class extends I.c{constructor(){super(...arguments),this._trustedCells=0,this._totalCells=0,this._activeCellTrusted=!1,this._notebook=null}get trustedCells(){return this._trustedCells}get totalCells(){return this._totalCells}get activeCellTrusted(){return this._activeCellTrusted}get notebook(){return this._notebook}set notebook(t){const e=this._notebook;null!==e&&(e.activeCellChanged.disconnect(this._onActiveCellChanged,this),e.modelContentChanged.disconnect(this._onModelChanged,this));const n=this._getAllState();if(this._notebook=t,null===this._notebook)this._trustedCells=0,this._totalCells=0,this._activeCellTrusted=!1;else{this._notebook.activeCellChanged.connect(this._onActiveCellChanged,this),this._notebook.modelContentChanged.connect(this._onModelChanged,this),void 0!==this._notebook.activeCell?this._activeCellTrusted=this._notebook.activeCell.model.trusted:this._activeCellTrusted=!1;const{total:t,trusted:e}=this._deriveCellTrustState(this._notebook.model);this._totalCells=t,this._trustedCells=e}this._triggerChange(n,this._getAllState())}_onModelChanged(t){const e=this._getAllState(),{total:n,trusted:r}=this._deriveCellTrustState(t.model);this._totalCells=n,this._trustedCells=r,this._triggerChange(e,this._getAllState())}_onActiveCellChanged(t,e){const n=this._getAllState();this._activeCellTrusted=!!e&&e.model.trusted,this._triggerChange(n,this._getAllState())}_deriveCellTrustState(t){let e=Object(a.toArray)(t.cells),n=e.reduce((t,e)=>e.trusted?t+1:t,0);return{total:e.length,trusted:n}}_getAllState(){return[this._trustedCells,this._totalCells,this.activeCellTrusted]}_triggerChange(t,e){t[0]===e[0]&&t[1]===e[1]&&t[2]===e[2]||this.stateChanged.emit(void 0)}};var Ot=n(628);n.d(e,"NotebookActions",function(){return h}),n.d(e,"ToolbarItems",function(){return x}),n.d(e,"CellTypeSwitcher",function(){return _}),n.d(e,"NotebookModel",function(){return G}),n.d(e,"NotebookModelFactory",function(){return H}),n.d(e,"NotebookTools",function(){return $}),n.d(e,"NotebookPanel",function(){return Dt}),n.d(e,"NotebookTracker",function(){return kt}),n.d(e,"StaticNotebook",function(){return Bt}),n.d(e,"Notebook",function(){return bt}),n.d(e,"NotebookWidgetFactory",function(){return St}),n.d(e,"CommandEditStatus",function(){return Nt}),n.d(e,"NotebookTrustStatus",function(){return Ut}),n.d(e,"INotebookTools",function(){return Ot.a}),n.d(e,"INotebookTracker",function(){return Ot.b})},function(t,e,n){"use strict";n.r(e);var r={};n.r(r),n.d(r,"ADD",function(){return o}),n.d(r,"ADD_COLUMN_LEFT",function(){return a}),n.d(r,"ADD_COLUMN_RIGHT",function(){return s}),n.d(r,"ADD_ROW_BOTTOM",function(){return A}),n.d(r,"ADD_ROW_TOP",function(){return l}),n.d(r,"ADD_TO_ARTIFACT",function(){return c}),n.d(r,"ADD_TO_FOLDER",function(){return u}),n.d(r,"AIRPLANE",function(){return g}),n.d(r,"ALIGN_CENTER",function(){return h}),n.d(r,"ALIGN_JUSTIFY",function(){return d}),n.d(r,"ALIGN_LEFT",function(){return f}),n.d(r,"ALIGN_RIGHT",function(){return p}),n.d(r,"ALIGNMENT_BOTTOM",function(){return I}),n.d(r,"ALIGNMENT_HORIZONTAL_CENTER",function(){return C}),n.d(r,"ALIGNMENT_LEFT",function(){return m}),n.d(r,"ALIGNMENT_RIGHT",function(){return y}),n.d(r,"ALIGNMENT_TOP",function(){return E}),n.d(r,"ALIGNMENT_VERTICAL_CENTER",function(){return v}),n.d(r,"ANNOTATION",function(){return B}),n.d(r,"APPLICATION",function(){return b}),n.d(r,"APPLICATIONS",function(){return Q}),n.d(r,"ARROW_BOTTOM_LEFT",function(){return w}),n.d(r,"ARROW_BOTTOM_RIGHT",function(){return x}),n.d(r,"ARROW_DOWN",function(){return _}),n.d(r,"ARROW_LEFT",function(){return D}),n.d(r,"ARROW_RIGHT",function(){return M}),n.d(r,"ARROW_TOP_LEFT",function(){return k}),n.d(r,"ARROW_TOP_RIGHT",function(){return S}),n.d(r,"ARROW_UP",function(){return L}),n.d(r,"ARROWS_HORIZONTAL",function(){return R}),n.d(r,"ARROWS_VERTICAL",function(){return N}),n.d(r,"ASTERISK",function(){return F}),n.d(r,"AUTOMATIC_UPDATES",function(){return T}),n.d(r,"BADGE",function(){return U}),n.d(r,"BAN_CIRCLE",function(){return O}),n.d(r,"BANK_ACCOUNT",function(){return G}),n.d(r,"BARCODE",function(){return H}),n.d(r,"BLANK",function(){return j}),n.d(r,"BLOCKED_PERSON",function(){return Y}),n.d(r,"BOLD",function(){return P}),n.d(r,"BOOK",function(){return K}),n.d(r,"BOOKMARK",function(){return z}),n.d(r,"BOX",function(){return J}),n.d(r,"BRIEFCASE",function(){return q}),n.d(r,"BUILD",function(){return V}),n.d(r,"CALCULATOR",function(){return W}),n.d(r,"CALENDAR",function(){return X}),n.d(r,"CAMERA",function(){return Z}),n.d(r,"CARET_DOWN",function(){return $}),n.d(r,"CARET_LEFT",function(){return tt}),n.d(r,"CARET_RIGHT",function(){return et}),n.d(r,"CARET_UP",function(){return nt}),n.d(r,"CELL_TOWER",function(){return rt}),n.d(r,"CHANGES",function(){return it}),n.d(r,"CHART",function(){return ot}),n.d(r,"CHAT",function(){return at}),n.d(r,"CHEVRON_BACKWARD",function(){return st}),n.d(r,"CHEVRON_DOWN",function(){return At}),n.d(r,"CHEVRON_FORWARD",function(){return lt}),n.d(r,"CHEVRON_LEFT",function(){return ct}),n.d(r,"CHEVRON_RIGHT",function(){return ut}),n.d(r,"CHEVRON_UP",function(){return gt}),n.d(r,"CIRCLE",function(){return ht}),n.d(r,"CIRCLE_ARROW_DOWN",function(){return dt}),n.d(r,"CIRCLE_ARROW_LEFT",function(){return ft}),n.d(r,"CIRCLE_ARROW_RIGHT",function(){return pt}),n.d(r,"CIRCLE_ARROW_UP",function(){return It}),n.d(r,"CITATION",function(){return Ct}),n.d(r,"CLEAN",function(){return mt}),n.d(r,"CLIPBOARD",function(){return yt}),n.d(r,"CLOUD",function(){return Et}),n.d(r,"CLOUD_DOWNLOAD",function(){return vt}),n.d(r,"CLOUD_UPLOAD",function(){return Bt}),n.d(r,"CODE",function(){return bt}),n.d(r,"CODE_BLOCK",function(){return Qt}),n.d(r,"COG",function(){return wt}),n.d(r,"COLLAPSE_ALL",function(){return xt}),n.d(r,"COLUMN_LAYOUT",function(){return _t}),n.d(r,"COMMENT",function(){return Dt}),n.d(r,"COMPARISON",function(){return Mt}),n.d(r,"COMPASS",function(){return kt}),n.d(r,"COMPRESSED",function(){return St}),n.d(r,"CONFIRM",function(){return Lt}),n.d(r,"CONSOLE",function(){return Rt}),n.d(r,"CONTRAST",function(){return Nt}),n.d(r,"CONTROL",function(){return Ft}),n.d(r,"CREDIT_CARD",function(){return Tt}),n.d(r,"CROSS",function(){return Ut}),n.d(r,"CROWN",function(){return Ot}),n.d(r,"CUBE",function(){return Gt}),n.d(r,"CUBE_ADD",function(){return Ht}),n.d(r,"CUBE_REMOVE",function(){return jt}),n.d(r,"CURVED_RANGE_CHART",function(){return Yt}),n.d(r,"CUT",function(){return Pt}),n.d(r,"DASHBOARD",function(){return Kt}),n.d(r,"DATABASE",function(){return zt}),n.d(r,"DELETE",function(){return Jt}),n.d(r,"DELTA",function(){return qt}),n.d(r,"DERIVE_COLUMN",function(){return Vt}),n.d(r,"DESKTOP",function(){return Wt}),n.d(r,"DIAGRAM_TREE",function(){return Xt}),n.d(r,"DIRECTION_LEFT",function(){return Zt}),n.d(r,"DIRECTION_RIGHT",function(){return $t}),n.d(r,"DISABLE",function(){return te}),n.d(r,"DOCUMENT",function(){return ee}),n.d(r,"DOCUMENT_OPEN",function(){return ne}),n.d(r,"DOCUMENT_SHARE",function(){return re}),n.d(r,"DOLLAR",function(){return ie}),n.d(r,"DOT",function(){return oe}),n.d(r,"DOUBLE_CARET_HORIZONTAL",function(){return ae}),n.d(r,"DOUBLE_CARET_VERTICAL",function(){return se}),n.d(r,"DOUBLE_CHEVRON_DOWN",function(){return Ae}),n.d(r,"DOUBLE_CHEVRON_LEFT",function(){return le}),n.d(r,"DOUBLE_CHEVRON_RIGHT",function(){return ce}),n.d(r,"DOUBLE_CHEVRON_UP",function(){return ue}),n.d(r,"DOUGHNUT_CHART",function(){return ge}),n.d(r,"DOWNLOAD",function(){return he}),n.d(r,"DRAG_HANDLE_HORIZONTAL",function(){return de}),n.d(r,"DRAG_HANDLE_VERTICAL",function(){return fe}),n.d(r,"DRAW",function(){return pe}),n.d(r,"DRIVE_TIME",function(){return Ie}),n.d(r,"DUPLICATE",function(){return Ce}),n.d(r,"EDIT",function(){return me}),n.d(r,"EJECT",function(){return ye}),n.d(r,"ENDORSED",function(){return Ee}),n.d(r,"ENVELOPE",function(){return ve}),n.d(r,"EQUALS",function(){return Be}),n.d(r,"ERASER",function(){return be}),n.d(r,"ERROR",function(){return Qe}),n.d(r,"EURO",function(){return we}),n.d(r,"EXCHANGE",function(){return xe}),n.d(r,"EXCLUDE_ROW",function(){return _e}),n.d(r,"EXPAND_ALL",function(){return De}),n.d(r,"EXPORT",function(){return Me}),n.d(r,"EYE_OFF",function(){return ke}),n.d(r,"EYE_ON",function(){return Se}),n.d(r,"EYE_OPEN",function(){return Le}),n.d(r,"FAST_BACKWARD",function(){return Re}),n.d(r,"FAST_FORWARD",function(){return Ne}),n.d(r,"FEED",function(){return Fe}),n.d(r,"FEED_SUBSCRIBED",function(){return Te}),n.d(r,"FILM",function(){return Ue}),n.d(r,"FILTER",function(){return Oe}),n.d(r,"FILTER_KEEP",function(){return Ge}),n.d(r,"FILTER_LIST",function(){return He}),n.d(r,"FILTER_OPEN",function(){return je}),n.d(r,"FILTER_REMOVE",function(){return Ye}),n.d(r,"FLAG",function(){return Pe}),n.d(r,"FLAME",function(){return Ke}),n.d(r,"FLASH",function(){return ze}),n.d(r,"FLOPPY_DISK",function(){return Je}),n.d(r,"FLOW_BRANCH",function(){return qe}),n.d(r,"FLOW_END",function(){return Ve}),n.d(r,"FLOW_LINEAR",function(){return We}),n.d(r,"FLOW_REVIEW",function(){return Xe}),n.d(r,"FLOW_REVIEW_BRANCH",function(){return Ze}),n.d(r,"FLOWS",function(){return $e}),n.d(r,"FOLDER_CLOSE",function(){return tn}),n.d(r,"FOLDER_NEW",function(){return en}),n.d(r,"FOLDER_OPEN",function(){return nn}),n.d(r,"FOLDER_SHARED",function(){return rn}),n.d(r,"FOLDER_SHARED_OPEN",function(){return on}),n.d(r,"FOLLOWER",function(){return an}),n.d(r,"FOLLOWING",function(){return sn}),n.d(r,"FONT",function(){return An}),n.d(r,"FORK",function(){return ln}),n.d(r,"FORM",function(){return cn}),n.d(r,"FULL_CIRCLE",function(){return un}),n.d(r,"FULL_STACKED_CHART",function(){return gn}),n.d(r,"FULLSCREEN",function(){return hn}),n.d(r,"FUNCTION",function(){return dn}),n.d(r,"GANTT_CHART",function(){return fn}),n.d(r,"GEOLOCATION",function(){return pn}),n.d(r,"GEOSEARCH",function(){return In}),n.d(r,"GIT_BRANCH",function(){return Cn}),n.d(r,"GIT_COMMIT",function(){return mn}),n.d(r,"GIT_MERGE",function(){return yn}),n.d(r,"GIT_NEW_BRANCH",function(){return En}),n.d(r,"GIT_PULL",function(){return vn}),n.d(r,"GIT_PUSH",function(){return Bn}),n.d(r,"GIT_REPO",function(){return bn}),n.d(r,"GLASS",function(){return Qn}),n.d(r,"GLOBE",function(){return wn}),n.d(r,"GLOBE_NETWORK",function(){return xn}),n.d(r,"GRAPH",function(){return _n}),n.d(r,"GRAPH_REMOVE",function(){return Dn}),n.d(r,"GREATER_THAN",function(){return Mn}),n.d(r,"GREATER_THAN_OR_EQUAL_TO",function(){return kn}),n.d(r,"GRID",function(){return Sn}),n.d(r,"GRID_VIEW",function(){return Ln}),n.d(r,"GROUP_OBJECTS",function(){return Rn}),n.d(r,"GROUPED_BAR_CHART",function(){return Nn}),n.d(r,"HAND",function(){return Fn}),n.d(r,"HAND_DOWN",function(){return Tn}),n.d(r,"HAND_LEFT",function(){return Un}),n.d(r,"HAND_RIGHT",function(){return On}),n.d(r,"HAND_UP",function(){return Gn}),n.d(r,"HEADER",function(){return Hn}),n.d(r,"HEADER_ONE",function(){return jn}),n.d(r,"HEADER_TWO",function(){return Yn}),n.d(r,"HEADSET",function(){return Pn}),n.d(r,"HEART",function(){return Kn}),n.d(r,"HEART_BROKEN",function(){return zn}),n.d(r,"HEAT_GRID",function(){return Jn}),n.d(r,"HEATMAP",function(){return qn}),n.d(r,"HELP",function(){return Vn}),n.d(r,"HELPER_MANAGEMENT",function(){return Wn}),n.d(r,"HIGHLIGHT",function(){return Xn}),n.d(r,"HISTORY",function(){return Zn}),n.d(r,"HOME",function(){return $n}),n.d(r,"HORIZONTAL_BAR_CHART",function(){return tr}),n.d(r,"HORIZONTAL_BAR_CHART_ASC",function(){return er}),n.d(r,"HORIZONTAL_BAR_CHART_DESC",function(){return nr}),n.d(r,"HORIZONTAL_DISTRIBUTION",function(){return rr}),n.d(r,"ID_NUMBER",function(){return ir}),n.d(r,"IMAGE_ROTATE_LEFT",function(){return or}),n.d(r,"IMAGE_ROTATE_RIGHT",function(){return ar}),n.d(r,"IMPORT",function(){return sr}),n.d(r,"INBOX",function(){return Ar}),n.d(r,"INBOX_FILTERED",function(){return lr}),n.d(r,"INBOX_GEO",function(){return cr}),n.d(r,"INBOX_SEARCH",function(){return ur}),n.d(r,"INBOX_UPDATE",function(){return gr}),n.d(r,"INFO_SIGN",function(){return hr}),n.d(r,"INHERITANCE",function(){return dr}),n.d(r,"INNER_JOIN",function(){return fr}),n.d(r,"INSERT",function(){return pr}),n.d(r,"INTERSECTION",function(){return Ir}),n.d(r,"IP_ADDRESS",function(){return Cr}),n.d(r,"ISSUE",function(){return mr}),n.d(r,"ISSUE_CLOSED",function(){return yr}),n.d(r,"ISSUE_NEW",function(){return Er}),n.d(r,"ITALIC",function(){return vr}),n.d(r,"JOIN_TABLE",function(){return Br}),n.d(r,"KEY",function(){return br}),n.d(r,"KEY_BACKSPACE",function(){return Qr}),n.d(r,"KEY_COMMAND",function(){return wr}),n.d(r,"KEY_CONTROL",function(){return xr}),n.d(r,"KEY_DELETE",function(){return _r}),n.d(r,"KEY_ENTER",function(){return Dr}),n.d(r,"KEY_ESCAPE",function(){return Mr}),n.d(r,"KEY_OPTION",function(){return kr}),n.d(r,"KEY_SHIFT",function(){return Sr}),n.d(r,"KEY_TAB",function(){return Lr}),n.d(r,"KNOWN_VEHICLE",function(){return Rr}),n.d(r,"LABEL",function(){return Nr}),n.d(r,"LAYER",function(){return Fr}),n.d(r,"LAYERS",function(){return Tr}),n.d(r,"LAYOUT",function(){return Ur}),n.d(r,"LAYOUT_AUTO",function(){return Or}),n.d(r,"LAYOUT_BALLOON",function(){return Gr}),n.d(r,"LAYOUT_CIRCLE",function(){return Hr}),n.d(r,"LAYOUT_GRID",function(){return jr}),n.d(r,"LAYOUT_GROUP_BY",function(){return Yr}),n.d(r,"LAYOUT_HIERARCHY",function(){return Pr}),n.d(r,"LAYOUT_LINEAR",function(){return Kr}),n.d(r,"LAYOUT_SKEW_GRID",function(){return zr}),n.d(r,"LAYOUT_SORTED_CLUSTERS",function(){return Jr}),n.d(r,"LEFT_JOIN",function(){return qr}),n.d(r,"LESS_THAN",function(){return Vr}),n.d(r,"LESS_THAN_OR_EQUAL_TO",function(){return Wr}),n.d(r,"LIFESAVER",function(){return Xr}),n.d(r,"LIGHTBULB",function(){return Zr}),n.d(r,"LINK",function(){return $r}),n.d(r,"LIST",function(){return ti}),n.d(r,"LIST_COLUMNS",function(){return ei}),n.d(r,"LIST_DETAIL_VIEW",function(){return ni}),n.d(r,"LOCATE",function(){return ri}),n.d(r,"LOCK",function(){return ii}),n.d(r,"LOG_IN",function(){return oi}),n.d(r,"LOG_OUT",function(){return ai}),n.d(r,"MANUAL",function(){return si}),n.d(r,"MANUALLY_ENTERED_DATA",function(){return Ai}),n.d(r,"MAP",function(){return li}),n.d(r,"MAP_CREATE",function(){return ci}),n.d(r,"MAP_MARKER",function(){return ui}),n.d(r,"MAXIMIZE",function(){return gi}),n.d(r,"MEDIA",function(){return hi}),n.d(r,"MENU",function(){return di}),n.d(r,"MENU_CLOSED",function(){return fi}),n.d(r,"MENU_OPEN",function(){return pi}),n.d(r,"MERGE_COLUMNS",function(){return Ii}),n.d(r,"MERGE_LINKS",function(){return Ci}),n.d(r,"MINIMIZE",function(){return mi}),n.d(r,"MINUS",function(){return yi}),n.d(r,"MOBILE_PHONE",function(){return Ei}),n.d(r,"MOBILE_VIDEO",function(){return vi}),n.d(r,"MOON",function(){return Bi}),n.d(r,"MORE",function(){return bi}),n.d(r,"MOUNTAIN",function(){return Qi}),n.d(r,"MOVE",function(){return wi}),n.d(r,"MUGSHOT",function(){return xi}),n.d(r,"MULTI_SELECT",function(){return _i}),n.d(r,"MUSIC",function(){return Di}),n.d(r,"NEW_GRID_ITEM",function(){return Mi}),n.d(r,"NEW_LINK",function(){return ki}),n.d(r,"NEW_OBJECT",function(){return Si}),n.d(r,"NEW_PERSON",function(){return Li}),n.d(r,"NEW_PRESCRIPTION",function(){return Ri}),n.d(r,"NEW_TEXT_BOX",function(){return Ni}),n.d(r,"NINJA",function(){return Fi}),n.d(r,"NOT_EQUAL_TO",function(){return Ti}),n.d(r,"NOTIFICATIONS",function(){return Ui}),n.d(r,"NOTIFICATIONS_UPDATED",function(){return Oi}),n.d(r,"NUMBERED_LIST",function(){return Gi}),n.d(r,"NUMERICAL",function(){return Hi}),n.d(r,"OFFICE",function(){return ji}),n.d(r,"OFFLINE",function(){return Yi}),n.d(r,"OIL_FIELD",function(){return Pi}),n.d(r,"ONE_COLUMN",function(){return Ki}),n.d(r,"OUTDATED",function(){return zi}),n.d(r,"PAGE_LAYOUT",function(){return Ji}),n.d(r,"PANEL_STATS",function(){return qi}),n.d(r,"PANEL_TABLE",function(){return Vi}),n.d(r,"PAPERCLIP",function(){return Wi}),n.d(r,"PARAGRAPH",function(){return Xi}),n.d(r,"PATH",function(){return Zi}),n.d(r,"PATH_SEARCH",function(){return $i}),n.d(r,"PAUSE",function(){return to}),n.d(r,"PEOPLE",function(){return eo}),n.d(r,"PERCENTAGE",function(){return no}),n.d(r,"PERSON",function(){return ro}),n.d(r,"PHONE",function(){return io}),n.d(r,"PIE_CHART",function(){return oo}),n.d(r,"PIN",function(){return ao}),n.d(r,"PIVOT",function(){return so}),n.d(r,"PIVOT_TABLE",function(){return Ao}),n.d(r,"PLAY",function(){return lo}),n.d(r,"PLUS",function(){return co}),n.d(r,"POLYGON_FILTER",function(){return uo}),n.d(r,"POWER",function(){return go}),n.d(r,"PREDICTIVE_ANALYSIS",function(){return ho}),n.d(r,"PRESCRIPTION",function(){return fo}),n.d(r,"PRESENTATION",function(){return po}),n.d(r,"PRINT",function(){return Io}),n.d(r,"PROJECTS",function(){return Co}),n.d(r,"PROPERTIES",function(){return mo}),n.d(r,"PROPERTY",function(){return yo}),n.d(r,"PUBLISH_FUNCTION",function(){return Eo}),n.d(r,"PULSE",function(){return vo}),n.d(r,"RANDOM",function(){return Bo}),n.d(r,"RECORD",function(){return bo}),n.d(r,"REDO",function(){return Qo}),n.d(r,"REFRESH",function(){return wo}),n.d(r,"REGRESSION_CHART",function(){return xo}),n.d(r,"REMOVE",function(){return _o}),n.d(r,"REMOVE_COLUMN",function(){return Do}),n.d(r,"REMOVE_COLUMN_LEFT",function(){return Mo}),n.d(r,"REMOVE_COLUMN_RIGHT",function(){return ko}),n.d(r,"REMOVE_ROW_BOTTOM",function(){return So}),n.d(r,"REMOVE_ROW_TOP",function(){return Lo}),n.d(r,"REPEAT",function(){return Ro}),n.d(r,"RESET",function(){return No}),n.d(r,"RESOLVE",function(){return Fo}),n.d(r,"RIG",function(){return To}),n.d(r,"RIGHT_JOIN",function(){return Uo}),n.d(r,"RING",function(){return Oo}),n.d(r,"ROTATE_DOCUMENT",function(){return Go}),n.d(r,"ROTATE_PAGE",function(){return Ho}),n.d(r,"SATELLITE",function(){return jo}),n.d(r,"SAVED",function(){return Yo}),n.d(r,"SCATTER_PLOT",function(){return Po}),n.d(r,"SEARCH",function(){return Ko}),n.d(r,"SEARCH_AROUND",function(){return zo}),n.d(r,"SEARCH_TEMPLATE",function(){return Jo}),n.d(r,"SEARCH_TEXT",function(){return qo}),n.d(r,"SEGMENTED_CONTROL",function(){return Vo}),n.d(r,"SELECT",function(){return Wo}),n.d(r,"SELECTION",function(){return Xo}),n.d(r,"SEND_TO",function(){return Zo}),n.d(r,"SEND_TO_GRAPH",function(){return $o}),n.d(r,"SEND_TO_MAP",function(){return ta}),n.d(r,"SERIES_ADD",function(){return ea}),n.d(r,"SERIES_CONFIGURATION",function(){return na}),n.d(r,"SERIES_DERIVED",function(){return ra}),n.d(r,"SERIES_FILTERED",function(){return ia}),n.d(r,"SERIES_SEARCH",function(){return oa}),n.d(r,"SETTINGS",function(){return aa}),n.d(r,"SHARE",function(){return sa}),n.d(r,"SHIELD",function(){return Aa}),n.d(r,"SHOP",function(){return la}),n.d(r,"SHOPPING_CART",function(){return ca}),n.d(r,"SIM_CARD",function(){return ua}),n.d(r,"SLASH",function(){return ga}),n.d(r,"SMALL_CROSS",function(){return ha}),n.d(r,"SMALL_MINUS",function(){return da}),n.d(r,"SMALL_PLUS",function(){return fa}),n.d(r,"SMALL_TICK",function(){return pa}),n.d(r,"SNOWFLAKE",function(){return Ia}),n.d(r,"SOCIAL_MEDIA",function(){return Ca}),n.d(r,"SORT",function(){return ma}),n.d(r,"SORT_ALPHABETICAL",function(){return ya}),n.d(r,"SORT_ALPHABETICAL_DESC",function(){return Ea}),n.d(r,"SORT_ASC",function(){return va}),n.d(r,"SORT_DESC",function(){return Ba}),n.d(r,"SORT_NUMERICAL",function(){return ba}),n.d(r,"SORT_NUMERICAL_DESC",function(){return Qa}),n.d(r,"SPLIT_COLUMNS",function(){return wa}),n.d(r,"SQUARE",function(){return xa}),n.d(r,"STACKED_CHART",function(){return _a}),n.d(r,"STAR",function(){return Da}),n.d(r,"STAR_EMPTY",function(){return Ma}),n.d(r,"STEP_BACKWARD",function(){return ka}),n.d(r,"STEP_CHART",function(){return Sa}),n.d(r,"STEP_FORWARD",function(){return La}),n.d(r,"STOP",function(){return Ra}),n.d(r,"STRIKETHROUGH",function(){return Na}),n.d(r,"STYLE",function(){return Fa}),n.d(r,"SWAP_HORIZONTAL",function(){return Ta}),n.d(r,"SWAP_VERTICAL",function(){return Ua}),n.d(r,"SYMBOL_CIRCLE",function(){return Oa}),n.d(r,"SYMBOL_CROSS",function(){return Ga}),n.d(r,"SYMBOL_DIAMOND",function(){return Ha}),n.d(r,"SYMBOL_SQUARE",function(){return ja}),n.d(r,"SYMBOL_TRIANGLE_DOWN",function(){return Ya}),n.d(r,"SYMBOL_TRIANGLE_UP",function(){return Pa}),n.d(r,"TAG",function(){return Ka}),n.d(r,"TAKE_ACTION",function(){return za}),n.d(r,"TAXI",function(){return Ja}),n.d(r,"TEXT_HIGHLIGHT",function(){return qa}),n.d(r,"TH",function(){return Va}),n.d(r,"TH_DERIVED",function(){return Wa}),n.d(r,"TH_DISCONNECT",function(){return Xa}),n.d(r,"TH_FILTERED",function(){return Za}),n.d(r,"TH_LIST",function(){return $a}),n.d(r,"THUMBS_DOWN",function(){return ts}),n.d(r,"THUMBS_UP",function(){return es}),n.d(r,"TICK",function(){return ns}),n.d(r,"TICK_CIRCLE",function(){return rs}),n.d(r,"TIME",function(){return is}),n.d(r,"TIMELINE_AREA_CHART",function(){return os}),n.d(r,"TIMELINE_BAR_CHART",function(){return as}),n.d(r,"TIMELINE_EVENTS",function(){return ss}),n.d(r,"TIMELINE_LINE_CHART",function(){return As}),n.d(r,"TINT",function(){return ls}),n.d(r,"TORCH",function(){return cs}),n.d(r,"TRAIN",function(){return us}),n.d(r,"TRANSLATE",function(){return gs}),n.d(r,"TRASH",function(){return hs}),n.d(r,"TREE",function(){return ds}),n.d(r,"TRENDING_DOWN",function(){return fs}),n.d(r,"TRENDING_UP",function(){return ps}),n.d(r,"TWO_COLUMNS",function(){return Is}),n.d(r,"UNDERLINE",function(){return Cs}),n.d(r,"UNDO",function(){return ms}),n.d(r,"UNGROUP_OBJECTS",function(){return ys}),n.d(r,"UNKNOWN_VEHICLE",function(){return Es}),n.d(r,"UNLOCK",function(){return vs}),n.d(r,"UNPIN",function(){return Bs}),n.d(r,"UNRESOLVE",function(){return bs}),n.d(r,"UPDATED",function(){return Qs}),n.d(r,"UPLOAD",function(){return ws}),n.d(r,"USER",function(){return xs}),n.d(r,"VARIABLE",function(){return _s}),n.d(r,"VERTICAL_BAR_CHART_ASC",function(){return Ds}),n.d(r,"VERTICAL_BAR_CHART_DESC",function(){return Ms}),n.d(r,"VERTICAL_DISTRIBUTION",function(){return ks}),n.d(r,"VIDEO",function(){return Ss}),n.d(r,"VOLUME_DOWN",function(){return Ls}),n.d(r,"VOLUME_OFF",function(){return Rs}),n.d(r,"VOLUME_UP",function(){return Ns}),n.d(r,"WALK",function(){return Fs}),n.d(r,"WARNING_SIGN",function(){return Ts}),n.d(r,"WATERFALL_CHART",function(){return Us}),n.d(r,"WIDGET",function(){return Os}),n.d(r,"WIDGET_BUTTON",function(){return Gs}),n.d(r,"WIDGET_FOOTER",function(){return Hs}),n.d(r,"WIDGET_HEADER",function(){return js}),n.d(r,"WRENCH",function(){return Ys}),n.d(r,"ZOOM_IN",function(){return Ps}),n.d(r,"ZOOM_OUT",function(){return Ks}),n.d(r,"ZOOM_TO_FIT",function(){return zs});var i={};n.r(i),n.d(i,"ADD",function(){return Js}),n.d(i,"ADD_COLUMN_LEFT",function(){return qs}),n.d(i,"ADD_COLUMN_RIGHT",function(){return Vs}),n.d(i,"ADD_ROW_BOTTOM",function(){return Ws}),n.d(i,"ADD_ROW_TOP",function(){return Xs}),n.d(i,"ADD_TO_ARTIFACT",function(){return Zs}),n.d(i,"ADD_TO_FOLDER",function(){return $s}),n.d(i,"AIRPLANE",function(){return tA}),n.d(i,"ALIGN_CENTER",function(){return eA}),n.d(i,"ALIGN_JUSTIFY",function(){return nA}),n.d(i,"ALIGN_LEFT",function(){return rA}),n.d(i,"ALIGN_RIGHT",function(){return iA}),n.d(i,"ALIGNMENT_BOTTOM",function(){return oA}),n.d(i,"ALIGNMENT_HORIZONTAL_CENTER",function(){return aA}),n.d(i,"ALIGNMENT_LEFT",function(){return sA}),n.d(i,"ALIGNMENT_RIGHT",function(){return AA}),n.d(i,"ALIGNMENT_TOP",function(){return lA}),n.d(i,"ALIGNMENT_VERTICAL_CENTER",function(){return cA}),n.d(i,"ANNOTATION",function(){return uA}),n.d(i,"APPLICATION",function(){return gA}),n.d(i,"APPLICATIONS",function(){return hA}),n.d(i,"ARROW_BOTTOM_LEFT",function(){return dA}),n.d(i,"ARROW_BOTTOM_RIGHT",function(){return fA}),n.d(i,"ARROW_DOWN",function(){return pA}),n.d(i,"ARROW_LEFT",function(){return IA}),n.d(i,"ARROW_RIGHT",function(){return CA}),n.d(i,"ARROW_TOP_LEFT",function(){return mA}),n.d(i,"ARROW_TOP_RIGHT",function(){return yA}),n.d(i,"ARROW_UP",function(){return EA}),n.d(i,"ARROWS_HORIZONTAL",function(){return vA}),n.d(i,"ARROWS_VERTICAL",function(){return BA}),n.d(i,"ASTERISK",function(){return bA}),n.d(i,"AUTOMATIC_UPDATES",function(){return QA}),n.d(i,"BADGE",function(){return wA}),n.d(i,"BAN_CIRCLE",function(){return xA}),n.d(i,"BANK_ACCOUNT",function(){return _A}),n.d(i,"BARCODE",function(){return DA}),n.d(i,"BLANK",function(){return MA}),n.d(i,"BLOCKED_PERSON",function(){return kA}),n.d(i,"BOLD",function(){return SA}),n.d(i,"BOOK",function(){return LA}),n.d(i,"BOOKMARK",function(){return RA}),n.d(i,"BOX",function(){return NA}),n.d(i,"BRIEFCASE",function(){return FA}),n.d(i,"BUILD",function(){return TA}),n.d(i,"CALCULATOR",function(){return UA}),n.d(i,"CALENDAR",function(){return OA}),n.d(i,"CAMERA",function(){return GA}),n.d(i,"CARET_DOWN",function(){return HA}),n.d(i,"CARET_LEFT",function(){return jA}),n.d(i,"CARET_RIGHT",function(){return YA}),n.d(i,"CARET_UP",function(){return PA}),n.d(i,"CELL_TOWER",function(){return KA}),n.d(i,"CHANGES",function(){return zA}),n.d(i,"CHART",function(){return JA}),n.d(i,"CHAT",function(){return qA}),n.d(i,"CHEVRON_BACKWARD",function(){return VA}),n.d(i,"CHEVRON_DOWN",function(){return WA}),n.d(i,"CHEVRON_FORWARD",function(){return XA}),n.d(i,"CHEVRON_LEFT",function(){return ZA}),n.d(i,"CHEVRON_RIGHT",function(){return $A}),n.d(i,"CHEVRON_UP",function(){return tl}),n.d(i,"CIRCLE",function(){return el}),n.d(i,"CIRCLE_ARROW_DOWN",function(){return nl}),n.d(i,"CIRCLE_ARROW_LEFT",function(){return rl}),n.d(i,"CIRCLE_ARROW_RIGHT",function(){return il}),n.d(i,"CIRCLE_ARROW_UP",function(){return ol}),n.d(i,"CITATION",function(){return al}),n.d(i,"CLEAN",function(){return sl}),n.d(i,"CLIPBOARD",function(){return Al}),n.d(i,"CLOUD",function(){return ll}),n.d(i,"CLOUD_DOWNLOAD",function(){return cl}),n.d(i,"CLOUD_UPLOAD",function(){return ul}),n.d(i,"CODE",function(){return gl}),n.d(i,"CODE_BLOCK",function(){return hl}),n.d(i,"COG",function(){return dl}),n.d(i,"COLLAPSE_ALL",function(){return fl}),n.d(i,"COLUMN_LAYOUT",function(){return pl}),n.d(i,"COMMENT",function(){return Il}),n.d(i,"COMPARISON",function(){return Cl}),n.d(i,"COMPASS",function(){return ml}),n.d(i,"COMPRESSED",function(){return yl}),n.d(i,"CONFIRM",function(){return El}),n.d(i,"CONSOLE",function(){return vl}),n.d(i,"CONTRAST",function(){return Bl}),n.d(i,"CONTROL",function(){return bl}),n.d(i,"CREDIT_CARD",function(){return Ql}),n.d(i,"CROSS",function(){return wl}),n.d(i,"CROWN",function(){return xl}),n.d(i,"CUBE",function(){return _l}),n.d(i,"CUBE_ADD",function(){return Dl}),n.d(i,"CUBE_REMOVE",function(){return Ml}),n.d(i,"CURVED_RANGE_CHART",function(){return kl}),n.d(i,"CUT",function(){return Sl}),n.d(i,"DASHBOARD",function(){return Ll}),n.d(i,"DATABASE",function(){return Rl}),n.d(i,"DELETE",function(){return Nl}),n.d(i,"DELTA",function(){return Fl}),n.d(i,"DERIVE_COLUMN",function(){return Tl}),n.d(i,"DESKTOP",function(){return Ul}),n.d(i,"DIAGRAM_TREE",function(){return Ol}),n.d(i,"DIRECTION_LEFT",function(){return Gl}),n.d(i,"DIRECTION_RIGHT",function(){return Hl}),n.d(i,"DISABLE",function(){return jl}),n.d(i,"DOCUMENT",function(){return Yl}),n.d(i,"DOCUMENT_OPEN",function(){return Pl}),n.d(i,"DOCUMENT_SHARE",function(){return Kl}),n.d(i,"DOLLAR",function(){return zl}),n.d(i,"DOT",function(){return Jl}),n.d(i,"DOUBLE_CARET_HORIZONTAL",function(){return ql}),n.d(i,"DOUBLE_CARET_VERTICAL",function(){return Vl}),n.d(i,"DOUBLE_CHEVRON_DOWN",function(){return Wl}),n.d(i,"DOUBLE_CHEVRON_LEFT",function(){return Xl}),n.d(i,"DOUBLE_CHEVRON_RIGHT",function(){return Zl}),n.d(i,"DOUBLE_CHEVRON_UP",function(){return $l}),n.d(i,"DOUGHNUT_CHART",function(){return tc}),n.d(i,"DOWNLOAD",function(){return ec}),n.d(i,"DRAG_HANDLE_HORIZONTAL",function(){return nc}),n.d(i,"DRAG_HANDLE_VERTICAL",function(){return rc}),n.d(i,"DRAW",function(){return ic}),n.d(i,"DRIVE_TIME",function(){return oc}),n.d(i,"DUPLICATE",function(){return ac}),n.d(i,"EDIT",function(){return sc}),n.d(i,"EJECT",function(){return Ac}),n.d(i,"ENDORSED",function(){return lc}),n.d(i,"ENVELOPE",function(){return cc}),n.d(i,"EQUALS",function(){return uc}),n.d(i,"ERASER",function(){return gc}),n.d(i,"ERROR",function(){return hc}),n.d(i,"EURO",function(){return dc}),n.d(i,"EXCHANGE",function(){return fc}),n.d(i,"EXCLUDE_ROW",function(){return pc}),n.d(i,"EXPAND_ALL",function(){return Ic}),n.d(i,"EXPORT",function(){return Cc}),n.d(i,"EYE_OFF",function(){return mc}),n.d(i,"EYE_ON",function(){return yc}),n.d(i,"EYE_OPEN",function(){return Ec}),n.d(i,"FAST_BACKWARD",function(){return vc}),n.d(i,"FAST_FORWARD",function(){return Bc}),n.d(i,"FEED",function(){return bc}),n.d(i,"FEED_SUBSCRIBED",function(){return Qc}),n.d(i,"FILM",function(){return wc}),n.d(i,"FILTER",function(){return xc}),n.d(i,"FILTER_KEEP",function(){return _c}),n.d(i,"FILTER_LIST",function(){return Dc}),n.d(i,"FILTER_OPEN",function(){return Mc}),n.d(i,"FILTER_REMOVE",function(){return kc}),n.d(i,"FLAG",function(){return Sc}),n.d(i,"FLAME",function(){return Lc}),n.d(i,"FLASH",function(){return Rc}),n.d(i,"FLOPPY_DISK",function(){return Nc}),n.d(i,"FLOW_BRANCH",function(){return Fc}),n.d(i,"FLOW_END",function(){return Tc}),n.d(i,"FLOW_LINEAR",function(){return Uc}),n.d(i,"FLOW_REVIEW",function(){return Oc}),n.d(i,"FLOW_REVIEW_BRANCH",function(){return Gc}),n.d(i,"FLOWS",function(){return Hc}),n.d(i,"FOLDER_CLOSE",function(){return jc}),n.d(i,"FOLDER_NEW",function(){return Yc}),n.d(i,"FOLDER_OPEN",function(){return Pc}),n.d(i,"FOLDER_SHARED",function(){return Kc}),n.d(i,"FOLDER_SHARED_OPEN",function(){return zc}),n.d(i,"FOLLOWER",function(){return Jc}),n.d(i,"FOLLOWING",function(){return qc}),n.d(i,"FONT",function(){return Vc}),n.d(i,"FORK",function(){return Wc}),n.d(i,"FORM",function(){return Xc}),n.d(i,"FULL_CIRCLE",function(){return Zc}),n.d(i,"FULL_STACKED_CHART",function(){return $c}),n.d(i,"FULLSCREEN",function(){return tu}),n.d(i,"FUNCTION",function(){return eu}),n.d(i,"GANTT_CHART",function(){return nu}),n.d(i,"GEOLOCATION",function(){return ru}),n.d(i,"GEOSEARCH",function(){return iu}),n.d(i,"GIT_BRANCH",function(){return ou}),n.d(i,"GIT_COMMIT",function(){return au}),n.d(i,"GIT_MERGE",function(){return su}),n.d(i,"GIT_NEW_BRANCH",function(){return Au}),n.d(i,"GIT_PULL",function(){return lu}),n.d(i,"GIT_PUSH",function(){return cu}),n.d(i,"GIT_REPO",function(){return uu}),n.d(i,"GLASS",function(){return gu}),n.d(i,"GLOBE",function(){return hu}),n.d(i,"GLOBE_NETWORK",function(){return du}),n.d(i,"GRAPH",function(){return fu}),n.d(i,"GRAPH_REMOVE",function(){return pu}),n.d(i,"GREATER_THAN",function(){return Iu}),n.d(i,"GREATER_THAN_OR_EQUAL_TO",function(){return Cu}),n.d(i,"GRID",function(){return mu}),n.d(i,"GRID_VIEW",function(){return yu}),n.d(i,"GROUP_OBJECTS",function(){return Eu}),n.d(i,"GROUPED_BAR_CHART",function(){return vu}),n.d(i,"HAND",function(){return Bu}),n.d(i,"HAND_DOWN",function(){return bu}),n.d(i,"HAND_LEFT",function(){return Qu}),n.d(i,"HAND_RIGHT",function(){return wu}),n.d(i,"HAND_UP",function(){return xu}),n.d(i,"HEADER",function(){return _u}),n.d(i,"HEADER_ONE",function(){return Du}),n.d(i,"HEADER_TWO",function(){return Mu}),n.d(i,"HEADSET",function(){return ku}),n.d(i,"HEART",function(){return Su}),n.d(i,"HEART_BROKEN",function(){return Lu}),n.d(i,"HEAT_GRID",function(){return Ru}),n.d(i,"HEATMAP",function(){return Nu}),n.d(i,"HELP",function(){return Fu}),n.d(i,"HELPER_MANAGEMENT",function(){return Tu}),n.d(i,"HIGHLIGHT",function(){return Uu}),n.d(i,"HISTORY",function(){return Ou}),n.d(i,"HOME",function(){return Gu}),n.d(i,"HORIZONTAL_BAR_CHART",function(){return Hu}),n.d(i,"HORIZONTAL_BAR_CHART_ASC",function(){return ju}),n.d(i,"HORIZONTAL_BAR_CHART_DESC",function(){return Yu}),n.d(i,"HORIZONTAL_DISTRIBUTION",function(){return Pu}),n.d(i,"ID_NUMBER",function(){return Ku}),n.d(i,"IMAGE_ROTATE_LEFT",function(){return zu}),n.d(i,"IMAGE_ROTATE_RIGHT",function(){return Ju}),n.d(i,"IMPORT",function(){return qu}),n.d(i,"INBOX",function(){return Vu}),n.d(i,"INBOX_FILTERED",function(){return Wu}),n.d(i,"INBOX_GEO",function(){return Xu}),n.d(i,"INBOX_SEARCH",function(){return Zu}),n.d(i,"INBOX_UPDATE",function(){return $u}),n.d(i,"INFO_SIGN",function(){return tg}),n.d(i,"INHERITANCE",function(){return eg}),n.d(i,"INNER_JOIN",function(){return ng}),n.d(i,"INSERT",function(){return rg}),n.d(i,"INTERSECTION",function(){return ig}),n.d(i,"IP_ADDRESS",function(){return og}),n.d(i,"ISSUE",function(){return ag}),n.d(i,"ISSUE_CLOSED",function(){return sg}),n.d(i,"ISSUE_NEW",function(){return Ag}),n.d(i,"ITALIC",function(){return lg}),n.d(i,"JOIN_TABLE",function(){return cg}),n.d(i,"KEY",function(){return ug}),n.d(i,"KEY_BACKSPACE",function(){return gg}),n.d(i,"KEY_COMMAND",function(){return hg}),n.d(i,"KEY_CONTROL",function(){return dg}),n.d(i,"KEY_DELETE",function(){return fg}),n.d(i,"KEY_ENTER",function(){return pg}),n.d(i,"KEY_ESCAPE",function(){return Ig}),n.d(i,"KEY_OPTION",function(){return Cg}),n.d(i,"KEY_SHIFT",function(){return mg}),n.d(i,"KEY_TAB",function(){return yg}),n.d(i,"KNOWN_VEHICLE",function(){return Eg}),n.d(i,"LABEL",function(){return vg}),n.d(i,"LAYER",function(){return Bg}),n.d(i,"LAYERS",function(){return bg}),n.d(i,"LAYOUT",function(){return Qg}),n.d(i,"LAYOUT_AUTO",function(){return wg}),n.d(i,"LAYOUT_BALLOON",function(){return xg}),n.d(i,"LAYOUT_CIRCLE",function(){return _g}),n.d(i,"LAYOUT_GRID",function(){return Dg}),n.d(i,"LAYOUT_GROUP_BY",function(){return Mg}),n.d(i,"LAYOUT_HIERARCHY",function(){return kg}),n.d(i,"LAYOUT_LINEAR",function(){return Sg}),n.d(i,"LAYOUT_SKEW_GRID",function(){return Lg}),n.d(i,"LAYOUT_SORTED_CLUSTERS",function(){return Rg}),n.d(i,"LEFT_JOIN",function(){return Ng}),n.d(i,"LESS_THAN",function(){return Fg}),n.d(i,"LESS_THAN_OR_EQUAL_TO",function(){return Tg}),n.d(i,"LIFESAVER",function(){return Ug}),n.d(i,"LIGHTBULB",function(){return Og}),n.d(i,"LINK",function(){return Gg}),n.d(i,"LIST",function(){return Hg}),n.d(i,"LIST_COLUMNS",function(){return jg}),n.d(i,"LIST_DETAIL_VIEW",function(){return Yg}),n.d(i,"LOCATE",function(){return Pg}),n.d(i,"LOCK",function(){return Kg}),n.d(i,"LOG_IN",function(){return zg}),n.d(i,"LOG_OUT",function(){return Jg}),n.d(i,"MANUAL",function(){return qg}),n.d(i,"MANUALLY_ENTERED_DATA",function(){return Vg}),n.d(i,"MAP",function(){return Wg}),n.d(i,"MAP_CREATE",function(){return Xg}),n.d(i,"MAP_MARKER",function(){return Zg}),n.d(i,"MAXIMIZE",function(){return $g}),n.d(i,"MEDIA",function(){return th}),n.d(i,"MENU",function(){return eh}),n.d(i,"MENU_CLOSED",function(){return nh}),n.d(i,"MENU_OPEN",function(){return rh}),n.d(i,"MERGE_COLUMNS",function(){return ih}),n.d(i,"MERGE_LINKS",function(){return oh}),n.d(i,"MINIMIZE",function(){return ah}),n.d(i,"MINUS",function(){return sh}),n.d(i,"MOBILE_PHONE",function(){return Ah}),n.d(i,"MOBILE_VIDEO",function(){return lh}),n.d(i,"MOON",function(){return ch}),n.d(i,"MORE",function(){return uh}),n.d(i,"MOUNTAIN",function(){return gh}),n.d(i,"MOVE",function(){return hh}),n.d(i,"MUGSHOT",function(){return dh}),n.d(i,"MULTI_SELECT",function(){return fh}),n.d(i,"MUSIC",function(){return ph}),n.d(i,"NEW_GRID_ITEM",function(){return Ih}),n.d(i,"NEW_LINK",function(){return Ch}),n.d(i,"NEW_OBJECT",function(){return mh}),n.d(i,"NEW_PERSON",function(){return yh}),n.d(i,"NEW_PRESCRIPTION",function(){return Eh}),n.d(i,"NEW_TEXT_BOX",function(){return vh}),n.d(i,"NINJA",function(){return Bh}),n.d(i,"NOT_EQUAL_TO",function(){return bh}),n.d(i,"NOTIFICATIONS",function(){return Qh}),n.d(i,"NOTIFICATIONS_UPDATED",function(){return wh}),n.d(i,"NUMBERED_LIST",function(){return xh}),n.d(i,"NUMERICAL",function(){return _h}),n.d(i,"OFFICE",function(){return Dh}),n.d(i,"OFFLINE",function(){return Mh}),n.d(i,"OIL_FIELD",function(){return kh}),n.d(i,"ONE_COLUMN",function(){return Sh}),n.d(i,"OUTDATED",function(){return Lh}),n.d(i,"PAGE_LAYOUT",function(){return Rh}),n.d(i,"PANEL_STATS",function(){return Nh}),n.d(i,"PANEL_TABLE",function(){return Fh}),n.d(i,"PAPERCLIP",function(){return Th}),n.d(i,"PARAGRAPH",function(){return Uh}),n.d(i,"PATH",function(){return Oh}),n.d(i,"PATH_SEARCH",function(){return Gh}),n.d(i,"PAUSE",function(){return Hh}),n.d(i,"PEOPLE",function(){return jh}),n.d(i,"PERCENTAGE",function(){return Yh}),n.d(i,"PERSON",function(){return Ph}),n.d(i,"PHONE",function(){return Kh}),n.d(i,"PIE_CHART",function(){return zh}),n.d(i,"PIN",function(){return Jh}),n.d(i,"PIVOT",function(){return qh}),n.d(i,"PIVOT_TABLE",function(){return Vh}),n.d(i,"PLAY",function(){return Wh}),n.d(i,"PLUS",function(){return Xh}),n.d(i,"POLYGON_FILTER",function(){return Zh}),n.d(i,"POWER",function(){return $h}),n.d(i,"PREDICTIVE_ANALYSIS",function(){return td}),n.d(i,"PRESCRIPTION",function(){return ed}),n.d(i,"PRESENTATION",function(){return nd}),n.d(i,"PRINT",function(){return rd}),n.d(i,"PROJECTS",function(){return id}),n.d(i,"PROPERTIES",function(){return od}),n.d(i,"PROPERTY",function(){return ad}),n.d(i,"PUBLISH_FUNCTION",function(){return sd}),n.d(i,"PULSE",function(){return Ad}),n.d(i,"RANDOM",function(){return ld}),n.d(i,"RECORD",function(){return cd}),n.d(i,"REDO",function(){return ud}),n.d(i,"REFRESH",function(){return gd}),n.d(i,"REGRESSION_CHART",function(){return hd}),n.d(i,"REMOVE",function(){return dd}),n.d(i,"REMOVE_COLUMN",function(){return fd}),n.d(i,"REMOVE_COLUMN_LEFT",function(){return pd}),n.d(i,"REMOVE_COLUMN_RIGHT",function(){return Id}),n.d(i,"REMOVE_ROW_BOTTOM",function(){return Cd}),n.d(i,"REMOVE_ROW_TOP",function(){return md}),n.d(i,"REPEAT",function(){return yd}),n.d(i,"RESET",function(){return Ed}),n.d(i,"RESOLVE",function(){return vd}),n.d(i,"RIG",function(){return Bd}),n.d(i,"RIGHT_JOIN",function(){return bd}),n.d(i,"RING",function(){return Qd}),n.d(i,"ROTATE_DOCUMENT",function(){return wd}),n.d(i,"ROTATE_PAGE",function(){return xd}),n.d(i,"SATELLITE",function(){return _d}),n.d(i,"SAVED",function(){return Dd}),n.d(i,"SCATTER_PLOT",function(){return Md}),n.d(i,"SEARCH",function(){return kd}),n.d(i,"SEARCH_AROUND",function(){return Sd}),n.d(i,"SEARCH_TEMPLATE",function(){return Ld}),n.d(i,"SEARCH_TEXT",function(){return Rd}),n.d(i,"SEGMENTED_CONTROL",function(){return Nd}),n.d(i,"SELECT",function(){return Fd}),n.d(i,"SELECTION",function(){return Td}),n.d(i,"SEND_TO",function(){return Ud}),n.d(i,"SEND_TO_GRAPH",function(){return Od}),n.d(i,"SEND_TO_MAP",function(){return Gd}),n.d(i,"SERIES_ADD",function(){return Hd}),n.d(i,"SERIES_CONFIGURATION",function(){return jd}),n.d(i,"SERIES_DERIVED",function(){return Yd}),n.d(i,"SERIES_FILTERED",function(){return Pd}),n.d(i,"SERIES_SEARCH",function(){return Kd}),n.d(i,"SETTINGS",function(){return zd}),n.d(i,"SHARE",function(){return Jd}),n.d(i,"SHIELD",function(){return qd}),n.d(i,"SHOP",function(){return Vd}),n.d(i,"SHOPPING_CART",function(){return Wd}),n.d(i,"SIM_CARD",function(){return Xd}),n.d(i,"SLASH",function(){return Zd}),n.d(i,"SMALL_CROSS",function(){return $d}),n.d(i,"SMALL_MINUS",function(){return tf}),n.d(i,"SMALL_PLUS",function(){return ef}),n.d(i,"SMALL_TICK",function(){return nf}),n.d(i,"SNOWFLAKE",function(){return rf}),n.d(i,"SOCIAL_MEDIA",function(){return of}),n.d(i,"SORT",function(){return af}),n.d(i,"SORT_ALPHABETICAL",function(){return sf}),n.d(i,"SORT_ALPHABETICAL_DESC",function(){return Af}),n.d(i,"SORT_ASC",function(){return lf}),n.d(i,"SORT_DESC",function(){return cf}),n.d(i,"SORT_NUMERICAL",function(){return uf}),n.d(i,"SORT_NUMERICAL_DESC",function(){return gf}),n.d(i,"SPLIT_COLUMNS",function(){return hf}),n.d(i,"SQUARE",function(){return df}),n.d(i,"STACKED_CHART",function(){return ff}),n.d(i,"STAR",function(){return pf}),n.d(i,"STAR_EMPTY",function(){return If}),n.d(i,"STEP_BACKWARD",function(){return Cf}),n.d(i,"STEP_CHART",function(){return mf}),n.d(i,"STEP_FORWARD",function(){return yf}),n.d(i,"STOP",function(){return Ef}),n.d(i,"STRIKETHROUGH",function(){return vf}),n.d(i,"STYLE",function(){return Bf}),n.d(i,"SWAP_HORIZONTAL",function(){return bf}),n.d(i,"SWAP_VERTICAL",function(){return Qf}),n.d(i,"SYMBOL_CIRCLE",function(){return wf}),n.d(i,"SYMBOL_CROSS",function(){return xf}),n.d(i,"SYMBOL_DIAMOND",function(){return _f}),n.d(i,"SYMBOL_SQUARE",function(){return Df}),n.d(i,"SYMBOL_TRIANGLE_DOWN",function(){return Mf}),n.d(i,"SYMBOL_TRIANGLE_UP",function(){return kf}),n.d(i,"TAG",function(){return Sf}),n.d(i,"TAKE_ACTION",function(){return Lf}),n.d(i,"TAXI",function(){return Rf}),n.d(i,"TEXT_HIGHLIGHT",function(){return Nf}),n.d(i,"TH",function(){return Ff}),n.d(i,"TH_DERIVED",function(){return Tf}),n.d(i,"TH_DISCONNECT",function(){return Uf}),n.d(i,"TH_FILTERED",function(){return Of}),n.d(i,"TH_LIST",function(){return Gf}),n.d(i,"THUMBS_DOWN",function(){return Hf}),n.d(i,"THUMBS_UP",function(){return jf}),n.d(i,"TICK",function(){return Yf}),n.d(i,"TICK_CIRCLE",function(){return Pf}),n.d(i,"TIME",function(){return Kf}),n.d(i,"TIMELINE_AREA_CHART",function(){return zf}),n.d(i,"TIMELINE_BAR_CHART",function(){return Jf}),n.d(i,"TIMELINE_EVENTS",function(){return qf}),n.d(i,"TIMELINE_LINE_CHART",function(){return Vf}),n.d(i,"TINT",function(){return Wf}),n.d(i,"TORCH",function(){return Xf}),n.d(i,"TRAIN",function(){return Zf}),n.d(i,"TRANSLATE",function(){return $f}),n.d(i,"TRASH",function(){return tp}),n.d(i,"TREE",function(){return ep}),n.d(i,"TRENDING_DOWN",function(){return np}),n.d(i,"TRENDING_UP",function(){return rp}),n.d(i,"TWO_COLUMNS",function(){return ip}),n.d(i,"UNDERLINE",function(){return op}),n.d(i,"UNDO",function(){return ap}),n.d(i,"UNGROUP_OBJECTS",function(){return sp}),n.d(i,"UNKNOWN_VEHICLE",function(){return Ap}),n.d(i,"UNLOCK",function(){return lp}),n.d(i,"UNPIN",function(){return cp}),n.d(i,"UNRESOLVE",function(){return up}),n.d(i,"UPDATED",function(){return gp}),n.d(i,"UPLOAD",function(){return hp}),n.d(i,"USER",function(){return dp}),n.d(i,"VARIABLE",function(){return fp}),n.d(i,"VERTICAL_BAR_CHART_ASC",function(){return pp}),n.d(i,"VERTICAL_BAR_CHART_DESC",function(){return Ip}),n.d(i,"VERTICAL_DISTRIBUTION",function(){return Cp}),n.d(i,"VIDEO",function(){return mp}),n.d(i,"VOLUME_DOWN",function(){return yp}),n.d(i,"VOLUME_OFF",function(){return Ep}),n.d(i,"VOLUME_UP",function(){return vp}),n.d(i,"WALK",function(){return Bp}),n.d(i,"WARNING_SIGN",function(){return bp}),n.d(i,"WATERFALL_CHART",function(){return Qp}),n.d(i,"WIDGET",function(){return wp}),n.d(i,"WIDGET_BUTTON",function(){return xp}),n.d(i,"WIDGET_FOOTER",function(){return _p}),n.d(i,"WIDGET_HEADER",function(){return Dp}),n.d(i,"WRENCH",function(){return Mp}),n.d(i,"ZOOM_IN",function(){return kp}),n.d(i,"ZOOM_OUT",function(){return Sp}),n.d(i,"ZOOM_TO_FIT",function(){return Lp});var o="",a="",s="",A="",l="",c="",u="",g="",h="",d="",f="",p="",I="",C="",m="",y="",E="",v="",B="",b="",Q="",w="↙",x="↘",_="↓ ",D="←",M="→",k="↖",S="↗",L="↑ ",R="↔ ",N="↕ ",F="*",T="",U="",O="",G="",H="",j="",Y="",P="",K="",z="",J="",q="",V="",W="",X="",Z="",$="⌄",tt="〈",et="〉",nt="⌃",rt="",it="",ot="",at="",st="",At="",lt="",ct="",ut="",gt="",ht="",dt="",ft="",pt="",It="",Ct="",mt="",yt="",Et="☁",vt="",Bt="",bt="",Qt="",wt="",xt="",_t="",Dt="",Mt="",kt="",St="",Lt="",Rt="",Nt="",Ft="",Tt="",Ut="✗",Ot="",Gt="",Ht="",jt="",Yt="",Pt="",Kt="",zt="",Jt="",qt="Δ",Vt="",Wt="",Xt="",Zt="",$t="",te="",ee="",ne="",re="",ie="$",oe="•",ae="",se="",Ae="",le="",ce="",ue="",ge="",he="",de="",fe="",pe="",Ie="",Ce="",me="✎",ye="⏏",Ee="",ve="✉",Be="",be="",Qe="",we="€",xe="",_e="",De="",Me="",ke="",Se="",Le="",Re="",Ne="",Fe="",Te="",Ue="",Oe="",Ge="",He="",je="",Ye="",Pe="⚑",Ke="",ze="",Je="",qe="",Ve="",We="",Xe="",Ze="",$e="",tn="",en="",nn="",rn="",on="",an="",sn="",An="",ln="",cn="",un="",gn="",hn="",dn="",fn="",pn="",In="",Cn="",mn="",yn="",En="",vn="",Bn="",bn="",Qn="",wn="",xn="",_n="",Dn="",Mn="",kn="",Sn="",Ln="",Rn="",Nn="",Fn="",Tn="",Un="",On="",Gn="",Hn="",jn="",Yn="",Pn="",Kn="♥",zn="",Jn="",qn="",Vn="?",Wn="",Xn="",Zn="",$n="⌂",tr="",er="",nr="",rr="",ir="",or="",ar="",sr="",Ar="",lr="",cr="",ur="",gr="",hr="ℹ",dr="",fr="",pr="",Ir="",Cr="",mr="",yr="",Er="",vr="",Br="",br="",Qr="",wr="",xr="",_r="",Dr="",Mr="",kr="",Sr="",Lr="",Rr="",Nr="",Fr="",Tr="",Ur="",Or="",Gr="",Hr="",jr="",Yr="",Pr="",Kr="",zr="",Jr="",qr="",Vr="",Wr="",Xr="",Zr="",$r="",ti="☰",ei="",ni="",ri="",ii="",oi="",ai="",si="",Ai="",li="",ci="",ui="",gi="",hi="",di="",fi="",pi="",Ii="",Ci="",mi="",yi="−",Ei="",vi="",Bi="",bi="",Qi="",wi="",xi="",_i="",Di="",Mi="",ki="",Si="",Li="",Ri="",Ni="",Fi="",Ti="",Ui="",Oi="",Gi="",Hi="",ji="",Yi="",Pi="",Ki="",zi="",Ji="",qi="",Vi="",Wi="",Xi="",Zi="",$i="",to="",eo="",no="",ro="",io="☎",oo="",ao="",so="",Ao="",lo="",co="+",uo="",go="",ho="",fo="",po="",Io="⎙",Co="",mo="",yo="",Eo="",vo="",Bo="",bo="",Qo="",wo="",xo="",_o="",Do="",Mo="",ko="",So="",Lo="",Ro="",No="",Fo="",To="",Uo="",Oo="",Go="",Ho="",jo="",Yo="",Po="",Ko="",zo="",Jo="",qo="",Vo="",Wo="",Xo="⦿",Zo="",$o="",ta="",ea="",na="",ra="",ia="",oa="",aa="",sa="",Aa="",la="",ca="",ua="",ga="",ha="",da="",fa="",pa="",Ia="",Ca="",ma="",ya="",Ea="",va="",Ba="",ba="",Qa="",wa="",xa="",_a="",Da="★",Ma="☆",ka="",Sa="",La="",Ra="",Na="",Fa="",Ta="",Ua="",Oa="",Ga="",Ha="",ja="",Ya="",Pa="",Ka="",za="",Ja="",qa="",Va="",Wa="",Xa="",Za="",$a="",ts="",es="",ns="✓",rs="",is="⏲",os="",as="",ss="",As="",ls="",cs="",us="",gs="",hs="",ds="",fs="",ps="",Is="",Cs="⎁",ms="⎌",ys="",Es="",vs="",Bs="",bs="",Qs="",ws="",xs="",_s="",Ds="",Ms="",ks="",Ss="",Ls="",Rs="",Ns="",Fs="",Ts="",Us="",Os="",Gs="",Hs="",js="",Ys="",Ps="",Ks="",zs="",Js="add",qs="add-column-left",Vs="add-column-right",Ws="add-row-bottom",Xs="add-row-top",Zs="add-to-artifact",$s="add-to-folder",tA="airplane",eA="align-center",nA="align-justify",rA="align-left",iA="align-right",oA="alignment-bottom",aA="alignment-horizontal-center",sA="alignment-left",AA="alignment-right",lA="alignment-top",cA="alignment-vertical-center",uA="annotation",gA="application",hA="applications",dA="arrow-bottom-left",fA="arrow-bottom-right",pA="arrow-down",IA="arrow-left",CA="arrow-right",mA="arrow-top-left",yA="arrow-top-right",EA="arrow-up",vA="arrows-horizontal",BA="arrows-vertical",bA="asterisk",QA="automatic-updates",wA="badge",xA="ban-circle",_A="bank-account",DA="barcode",MA="blank",kA="blocked-person",SA="bold",LA="book",RA="bookmark",NA="box",FA="briefcase",TA="build",UA="calculator",OA="calendar",GA="camera",HA="caret-down",jA="caret-left",YA="caret-right",PA="caret-up",KA="cell-tower",zA="changes",JA="chart",qA="chat",VA="chevron-backward",WA="chevron-down",XA="chevron-forward",ZA="chevron-left",$A="chevron-right",tl="chevron-up",el="circle",nl="circle-arrow-down",rl="circle-arrow-left",il="circle-arrow-right",ol="circle-arrow-up",al="citation",sl="clean",Al="clipboard",ll="cloud",cl="cloud-download",ul="cloud-upload",gl="code",hl="code-block",dl="cog",fl="collapse-all",pl="column-layout",Il="comment",Cl="comparison",ml="compass",yl="compressed",El="confirm",vl="console",Bl="contrast",bl="control",Ql="credit-card",wl="cross",xl="crown",_l="cube",Dl="cube-add",Ml="cube-remove",kl="curved-range-chart",Sl="cut",Ll="dashboard",Rl="database",Nl="delete",Fl="delta",Tl="derive-column",Ul="desktop",Ol="diagram-tree",Gl="direction-left",Hl="direction-right",jl="disable",Yl="document",Pl="document-open",Kl="document-share",zl="dollar",Jl="dot",ql="double-caret-horizontal",Vl="double-caret-vertical",Wl="double-chevron-down",Xl="double-chevron-left",Zl="double-chevron-right",$l="double-chevron-up",tc="doughnut-chart",ec="download",nc="drag-handle-horizontal",rc="drag-handle-vertical",ic="draw",oc="drive-time",ac="duplicate",sc="edit",Ac="eject",lc="endorsed",cc="envelope",uc="equals",gc="eraser",hc="error",dc="euro",fc="exchange",pc="exclude-row",Ic="expand-all",Cc="export",mc="eye-off",yc="eye-on",Ec="eye-open",vc="fast-backward",Bc="fast-forward",bc="feed",Qc="feed-subscribed",wc="film",xc="filter",_c="filter-keep",Dc="filter-list",Mc="filter-open",kc="filter-remove",Sc="flag",Lc="flame",Rc="flash",Nc="floppy-disk",Fc="flow-branch",Tc="flow-end",Uc="flow-linear",Oc="flow-review",Gc="flow-review-branch",Hc="flows",jc="folder-close",Yc="folder-new",Pc="folder-open",Kc="folder-shared",zc="folder-shared-open",Jc="follower",qc="following",Vc="font",Wc="fork",Xc="form",Zc="full-circle",$c="full-stacked-chart",tu="fullscreen",eu="function",nu="gantt-chart",ru="geolocation",iu="geosearch",ou="git-branch",au="git-commit",su="git-merge",Au="git-new-branch",lu="git-pull",cu="git-push",uu="git-repo",gu="glass",hu="globe",du="globe-network",fu="graph",pu="graph-remove",Iu="greater-than",Cu="greater-than-or-equal-to",mu="grid",yu="grid-view",Eu="group-objects",vu="grouped-bar-chart",Bu="hand",bu="hand-down",Qu="hand-left",wu="hand-right",xu="hand-up",_u="header",Du="header-one",Mu="header-two",ku="headset",Su="heart",Lu="heart-broken",Ru="heat-grid",Nu="heatmap",Fu="help",Tu="helper-management",Uu="highlight",Ou="history",Gu="home",Hu="horizontal-bar-chart",ju="horizontal-bar-chart-asc",Yu="horizontal-bar-chart-desc",Pu="horizontal-distribution",Ku="id-number",zu="image-rotate-left",Ju="image-rotate-right",qu="import",Vu="inbox",Wu="inbox-filtered",Xu="inbox-geo",Zu="inbox-search",$u="inbox-update",tg="info-sign",eg="inheritance",ng="inner-join",rg="insert",ig="intersection",og="ip-address",ag="issue",sg="issue-closed",Ag="issue-new",lg="italic",cg="join-table",ug="key",gg="key-backspace",hg="key-command",dg="key-control",fg="key-delete",pg="key-enter",Ig="key-escape",Cg="key-option",mg="key-shift",yg="key-tab",Eg="known-vehicle",vg="label",Bg="layer",bg="layers",Qg="layout",wg="layout-auto",xg="layout-balloon",_g="layout-circle",Dg="layout-grid",Mg="layout-group-by",kg="layout-hierarchy",Sg="layout-linear",Lg="layout-skew-grid",Rg="layout-sorted-clusters",Ng="left-join",Fg="less-than",Tg="less-than-or-equal-to",Ug="lifesaver",Og="lightbulb",Gg="link",Hg="list",jg="list-columns",Yg="list-detail-view",Pg="locate",Kg="lock",zg="log-in",Jg="log-out",qg="manual",Vg="manually-entered-data",Wg="map",Xg="map-create",Zg="map-marker",$g="maximize",th="media",eh="menu",nh="menu-closed",rh="menu-open",ih="merge-columns",oh="merge-links",ah="minimize",sh="minus",Ah="mobile-phone",lh="mobile-video",ch="moon",uh="more",gh="mountain",hh="move",dh="mugshot",fh="multi-select",ph="music",Ih="new-grid-item",Ch="new-link",mh="new-object",yh="new-person",Eh="new-prescription",vh="new-text-box",Bh="ninja",bh="not-equal-to",Qh="notifications",wh="notifications-updated",xh="numbered-list",_h="numerical",Dh="office",Mh="offline",kh="oil-field",Sh="one-column",Lh="outdated",Rh="page-layout",Nh="panel-stats",Fh="panel-table",Th="paperclip",Uh="paragraph",Oh="path",Gh="path-search",Hh="pause",jh="people",Yh="percentage",Ph="person",Kh="phone",zh="pie-chart",Jh="pin",qh="pivot",Vh="pivot-table",Wh="play",Xh="plus",Zh="polygon-filter",$h="power",td="predictive-analysis",ed="prescription",nd="presentation",rd="print",id="projects",od="properties",ad="property",sd="publish-function",Ad="pulse",ld="random",cd="record",ud="redo",gd="refresh",hd="regression-chart",dd="remove",fd="remove-column",pd="remove-column-left",Id="remove-column-right",Cd="remove-row-bottom",md="remove-row-top",yd="repeat",Ed="reset",vd="resolve",Bd="rig",bd="right-join",Qd="ring",wd="rotate-document",xd="rotate-page",_d="satellite",Dd="saved",Md="scatter-plot",kd="search",Sd="search-around",Ld="search-template",Rd="search-text",Nd="segmented-control",Fd="select",Td="selection",Ud="send-to",Od="send-to-graph",Gd="send-to-map",Hd="series-add",jd="series-configuration",Yd="series-derived",Pd="series-filtered",Kd="series-search",zd="settings",Jd="share",qd="shield",Vd="shop",Wd="shopping-cart",Xd="sim-card",Zd="slash",$d="small-cross",tf="small-minus",ef="small-plus",nf="small-tick",rf="snowflake",of="social-media",af="sort",sf="sort-alphabetical",Af="sort-alphabetical-desc",lf="sort-asc",cf="sort-desc",uf="sort-numerical",gf="sort-numerical-desc",hf="split-columns",df="square",ff="stacked-chart",pf="star",If="star-empty",Cf="step-backward",mf="step-chart",yf="step-forward",Ef="stop",vf="strikethrough",Bf="style",bf="swap-horizontal",Qf="swap-vertical",wf="symbol-circle",xf="symbol-cross",_f="symbol-diamond",Df="symbol-square",Mf="symbol-triangle-down",kf="symbol-triangle-up",Sf="tag",Lf="take-action",Rf="taxi",Nf="text-highlight",Ff="th",Tf="th-derived",Uf="th-disconnect",Of="th-filtered",Gf="th-list",Hf="thumbs-down",jf="thumbs-up",Yf="tick",Pf="tick-circle",Kf="time",zf="timeline-area-chart",Jf="timeline-bar-chart",qf="timeline-events",Vf="timeline-line-chart",Wf="tint",Xf="torch",Zf="train",$f="translate",tp="trash",ep="tree",np="trending-down",rp="trending-up",ip="two-columns",op="underline",ap="undo",sp="ungroup-objects",Ap="unknown-vehicle",lp="unlock",cp="unpin",up="unresolve",gp="updated",hp="upload",dp="user",fp="variable",pp="vertical-bar-chart-asc",Ip="vertical-bar-chart-desc",Cp="vertical-distribution",mp="video",yp="volume-down",Ep="volume-off",vp="volume-up",Bp="walk",bp="warning-sign",Qp="waterfall-chart",wp="widget",xp="widget-button",_p="widget-footer",Dp="widget-header",Mp="wrench",kp="zoom-in",Sp="zoom-out",Lp="zoom-to-fit",Rp=n(307);n.d(e,"IconContents",function(){return r}),n.d(e,"IconNames",function(){return i}),n.d(e,"IconSvgPaths16",function(){return Rp.a}),n.d(e,"IconSvgPaths20",function(){return Rp.b})},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoCiAgICAgICAgY2xhc3M9ImpwLWljb24wIgogICAgICAgIGZpbGw9IiMwMDAiCiAgICAgICAgZD0iTTI0IDIwLjE4OGwtOC4zMTUtOC4yMDkgOC4yLTguMjgyLTMuNjk3LTMuNjk3LTguMjEyIDguMzE4LTguMzEtOC4yMDMtMy42NjYgMy42NjYgOC4zMjEgOC4yNC04LjIwNiA4LjMxMyAzLjY2NiAzLjY2NiA4LjIzNy04LjMxOCA4LjI4NSA4LjIwM3oiCiAgICAvPgo8L3N2Zz4K"},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoCiAgICAgICAgZmlsbD0iIzAwMCIKICAgICAgICBmaWxsLW9wYWNpdHk9IjAuMCIKICAgICAgICBkPSJNMjQgMjAuMTg4bC04LjMxNS04LjIwOSA4LjItOC4yODItMy42OTctMy42OTctOC4yMTIgOC4zMTgtOC4zMS04LjIwMy0zLjY2NiAzLjY2NiA4LjMyMSA4LjI0LTguMjA2IDguMzEzIDMuNjY2IDMuNjY2IDguMjM3LTguMzE4IDguMjg1IDguMjAzeiIKICAgIC8+Cjwvc3ZnPgo="},function(t,e,n){(function(e){var n=1/0,r="[object Symbol]",i=/[&<>"'`]/g,o=RegExp(i.source),a="object"==typeof e&&e&&e.Object===Object&&e,s="object"==typeof self&&self&&self.Object===Object&&self,A=a||s||Function("return this")();var l=function(t){return function(e){return null==t?void 0:t[e]}}({"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;","`":"&#96;"}),c=Object.prototype.toString,u=A.Symbol,g=u?u.prototype:void 0,h=g?g.toString:void 0;function d(t){if("string"==typeof t)return t;if(function(t){return"symbol"==typeof t||function(t){return!!t&&"object"==typeof t}(t)&&c.call(t)==r}(t))return h?h.call(t):"";var e=t+"";return"0"==e&&1/t==-n?"-0":e}t.exports=function(t){var e;return(t=null==(e=t)?"":d(e))&&o.test(t)?t.replace(i,l):t}}).call(this,n(37))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2),i=r.__importStar(n(0)),o=n(177),a=n(1137),s=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.__extends(e,t),e.prototype.render=function(){return i.createElement("button",r.__assign({type:"button"},o.removeNonHTMLProps(this.props),this.getCommonButtonProps()),this.renderChildren())},e.displayName=o.DISPLAYNAME_PREFIX+".Button",e}(a.AbstractButton);e.Button=s;var A=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.__extends(e,t),e.prototype.render=function(){var t=this.props,e=t.href,n=t.tabIndex,a=void 0===n?0:n,s=this.getCommonButtonProps();return i.createElement("a",r.__assign({role:"button"},o.removeNonHTMLProps(this.props),s,{href:s.disabled?void 0:e,tabIndex:s.disabled?-1:a}),this.renderChildren())},e.displayName=o.DISPLAYNAME_PREFIX+".AnchorButton",e}(a.AbstractButton);e.AnchorButton=A},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r,i=n(2),o=i.__importDefault(n(4)),a=i.__importStar(n(0)),s=n(308),A=i.__importStar(n(139)),l=n(177);!function(t){t[t.OPEN_START=0]="OPEN_START",t[t.OPENING=1]="OPENING",t[t.OPEN=2]="OPEN",t[t.CLOSING_START=3]="CLOSING_START",t[t.CLOSING=4]="CLOSING",t[t.CLOSED=5]="CLOSED"}(r=e.AnimationStates||(e.AnimationStates={}));var c=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={animationState:e.props.isOpen?r.OPEN:r.CLOSED,height:"0px"},e.height=0,e.contentsRefHandler=function(t){e.contents=t,null!=t&&(e.height=e.contents.clientHeight,e.setState({animationState:e.props.isOpen?r.OPEN:r.CLOSED,height:e.height+"px"}))},e}return i.__extends(e,t),e.prototype.componentWillReceiveProps=function(t){this.props.isOpen!==t.isOpen&&(this.clearTimeouts(),this.state.animationState===r.CLOSED||t.isOpen?this.state.animationState!==r.OPEN&&t.isOpen&&this.setState({animationState:r.OPEN_START}):this.setState({animationState:r.CLOSING_START,height:this.height+"px"}))},e.prototype.render=function(){var t=this.state.animationState!==r.CLOSED,e=t||this.props.keepChildrenMounted,n=t&&this.state.animationState!==r.CLOSING,i="auto"===this.state.height,s={height:t?this.state.height:void 0,overflowY:i?"visible":void 0,transition:i?"none":void 0},l={transform:n?"translateY(0)":"translateY(-"+this.height+"px)",transition:i?"none":void 0};return a.createElement(this.props.component,{className:o.default(A.COLLAPSE,this.props.className),style:s},a.createElement("div",{className:A.COLLAPSE_BODY,ref:this.contentsRefHandler,style:l,"aria-hidden":!t&&this.props.keepChildrenMounted},e?this.props.children:null))},e.prototype.componentDidMount=function(){this.forceUpdate(),this.props.isOpen?this.setState({animationState:r.OPEN,height:"auto"}):this.setState({animationState:r.CLOSED})},e.prototype.componentDidUpdate=function(){var t=this;null!=this.contents&&0!==this.contents.clientHeight&&(this.height=this.contents.clientHeight),this.state.animationState===r.CLOSING_START&&(this.setTimeout(function(){return t.setState({animationState:r.CLOSING,height:"0px"})}),this.setTimeout(function(){return t.onDelayedStateChange()},this.props.transitionDuration)),this.state.animationState===r.OPEN_START&&(this.setState({animationState:r.OPENING,height:this.height+"px"}),this.setTimeout(function(){return t.onDelayedStateChange()},this.props.transitionDuration))},e.prototype.onDelayedStateChange=function(){switch(this.state.animationState){case r.OPENING:this.setState({animationState:r.OPEN,height:"auto"});break;case r.CLOSING:this.setState({animationState:r.CLOSED})}},e.displayName=l.DISPLAYNAME_PREFIX+".Collapse",e.defaultProps={component:"div",isOpen:!1,keepChildrenMounted:!1,transitionDuration:200},e}(s.AbstractPureComponent);e.Collapse=c},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2),i=r.__importDefault(n(4)),o=r.__importStar(n(0)),a=n(139),s=n(190),A=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.__extends(e,t),e.prototype.render=function(){var t,e=this.props,n=e.className,A=e.disabled,l=e.elementRef,c=e.fill,u=e.iconProps,g=e.large,h=e.minimal,d=e.options,f=void 0===d?[]:d,p=r.__rest(e,["className","disabled","elementRef","fill","iconProps","large","minimal","options"]),I=i.default(a.HTML_SELECT,((t={})[a.DISABLED]=A,t[a.FILL]=c,t[a.LARGE]=g,t[a.MINIMAL]=h,t),n),C=f.map(function(t){var e="object"==typeof t?t:{value:t};return o.createElement("option",r.__assign({},e,{key:e.value,children:e.label||e.value}))});return o.createElement("div",{className:I},o.createElement("select",r.__assign({disabled:A,ref:l},p,{multiple:!1}),C,p.children),o.createElement(s.Icon,r.__assign({icon:"double-caret-vertical"},u)))},e}(o.PureComponent);e.HTMLSelect=A},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2),i=r.__importDefault(n(4)),o=r.__importStar(n(0)),a=n(242),s=n(624),A=n(1162),l=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={isOpen:!1},e.TypedQueryList=A.QueryList.ofType(),e.input=null,e.queryList=null,e.refHandlers={input:function(t){e.input=t,a.Utils.safeInvokeMember(e.props.inputProps,"inputRef",t)},queryList:function(t){return e.queryList=t}},e.renderQueryList=function(t){var n=e.props,A=n.filterable,l=void 0===A||A,c=n.disabled,u=void 0!==c&&c,g=n.inputProps,h=void 0===g?{}:g,d=n.popoverProps,f=void 0===d?{}:d,p=o.createElement(a.InputGroup,r.__assign({leftIcon:"search",placeholder:"Filter...",rightElement:e.maybeRenderClearButton(t.query)},h,{inputRef:e.refHandlers.input,onChange:t.handleQueryChange,value:t.query})),I=t.handleKeyDown,C=t.handleKeyUp;return o.createElement(a.Popover,r.__assign({autoFocus:!1,enforceFocus:!1,isOpen:e.state.isOpen,disabled:u,position:a.Position.BOTTOM_LEFT},f,{className:i.default(t.className,f.className),onInteraction:e.handlePopoverInteraction,popoverClassName:i.default(s.Classes.SELECT_POPOVER,f.popoverClassName),onOpening:e.handlePopoverOpening,onOpened:e.handlePopoverOpened,onClosing:e.handlePopoverClosing}),o.createElement("div",{onKeyDown:e.state.isOpen?I:e.handleTargetKeyDown,onKeyUp:e.state.isOpen?C:void 0},e.props.children),o.createElement("div",{onKeyDown:I,onKeyUp:C},l?p:void 0,t.itemList))},e.handleTargetKeyDown=function(t){t.which!==a.Keys.ARROW_UP&&t.which!==a.Keys.ARROW_DOWN||(t.preventDefault(),e.setState({isOpen:!0}))},e.handleItemSelect=function(t,n){e.setState({isOpen:!1}),a.Utils.safeInvoke(e.props.onItemSelect,t,n)},e.handlePopoverInteraction=function(t){e.setState({isOpen:t}),a.Utils.safeInvokeMember(e.props.popoverProps,"onInteraction",t)},e.handlePopoverOpening=function(t){e.previousFocusedElement=document.activeElement,e.props.resetOnClose&&e.resetQuery(),a.Utils.safeInvokeMember(e.props.popoverProps,"onOpening",t)},e.handlePopoverOpened=function(t){null!=e.queryList&&e.queryList.scrollActiveItemIntoView(),requestAnimationFrame(function(){var t=e.props.inputProps;!1!==(void 0===t?{}:t).autoFocus&&null!=e.input&&e.input.focus()}),a.Utils.safeInvokeMember(e.props.popoverProps,"onOpened",t)},e.handlePopoverClosing=function(t){requestAnimationFrame(function(){void 0!==e.previousFocusedElement&&(e.previousFocusedElement.focus(),e.previousFocusedElement=void 0)}),a.Utils.safeInvokeMember(e.props.popoverProps,"onClosing",t)},e.resetQuery=function(){return e.queryList&&e.queryList.setQuery("",!0)},e}return r.__extends(e,t),e.ofType=function(){return e},e.prototype.render=function(){var t=this.props,e=(t.filterable,t.inputProps,t.popoverProps,r.__rest(t,["filterable","inputProps","popoverProps"]));return o.createElement(this.TypedQueryList,r.__assign({},e,{onItemSelect:this.handleItemSelect,ref:this.refHandlers.queryList,renderer:this.renderQueryList}))},e.prototype.componentDidUpdate=function(t,e){this.state.isOpen&&!e.isOpen&&null!=this.queryList&&this.queryList.scrollActiveItemIntoView()},e.prototype.maybeRenderClearButton=function(t){return t.length>0?o.createElement(a.Button,{icon:"cross",minimal:!0,onClick:this.resetQuery}):void 0},e.displayName=a.DISPLAYNAME_PREFIX+".Select",e}(o.PureComponent);e.Select=l},function(t,e){t.exports=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);return i}},function(t,e,n){"use strict";e.__esModule=!0;var r=o(n(0)),i=o(n(1146));function o(t){return t&&t.__esModule?t:{default:t}}e.default=r.default.createContext||i.default,t.exports=e.default},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.default=function(t,e,n,r){function i(e){var r=new o.default(e);n.call(t,r)}if(t.addEventListener){var a=(s=!1,"object"==typeof r?s=r.capture||!1:"boolean"==typeof r&&(s=r),t.addEventListener(e,i,r||!1),{v:{remove:function(){t.removeEventListener(e,i,s)}}});if("object"==typeof a)return a.v}else if(t.attachEvent)return t.attachEvent("on"+e,i),{remove:function(){t.detachEvent("on"+e,i)}};var s};var r,i=n(1224),o=(r=i)&&r.__esModule?r:{default:r};t.exports=e.default},function(t,e,n){var r=n(1338);t.exports=function(t,e){return r(t,e)}},function(t,e,n){var r=n(182),i=n(1404),o=n(183),a="[object Object]",s=Function.prototype,A=Object.prototype,l=s.toString,c=A.hasOwnProperty,u=l.call(Object);t.exports=function(t){if(!o(t)||r(t)!=a)return!1;var e=i(t);if(null===e)return!0;var n=c.call(e,"constructor")&&e.constructor;return"function"==typeof n&&n instanceof n&&l.call(n)==u}},function(t,e,n){"use strict";n.d(e,"a",function(){return s}),n.d(e,"b",function(){return A});var r=n(201),i=n(381),o=n(382),a=n(10);class s{constructor(t={}){if(this._map=new r.a,this._isDisposed=!1,this._stateChanged=new a.Signal(this),this._changed=new a.Signal(this),this._modelDB=null,this._serialized=null,this._changeGuard=!1,this.contentFactory=t.contentFactory||s.defaultContentFactory,t.values)for(let e of Object.keys(t.values))this.set(e,t.values[e]);this._map.changed.connect(this._onMapChanged,this),t.modelDB&&(this._modelDB=t.modelDB,this._serialized=this._modelDB.createValue("attachments"),this._serialized.get()?this.fromJSON(this._serialized.get()):this._serialized.set(this.toJSON()),this._serialized.changed.connect(this._onSerializedChanged,this))}get stateChanged(){return this._stateChanged}get changed(){return this._changed}get keys(){return this._map.keys()}get length(){return this._map.keys().length}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,this._map.dispose(),a.Signal.clearData(this))}has(t){return this._map.has(t)}get(t){return this._map.get(t)}set(t,e){let n=this._createItem({value:e});this._map.set(t,n)}clear(){this._map.values().forEach(t=>{t.dispose()}),this._map.clear()}fromJSON(t){this.clear(),Object.keys(t).forEach(e=>{this.set(e,t[e])})}toJSON(){let t={};for(let e of this._map.keys())t[e]=this._map.get(e).toJSON();return t}_createItem(t){let e=this.contentFactory.createAttachmentModel(t);return e.changed.connect(this._onGenericChange,this),e}_onMapChanged(t,e){this._serialized&&!this._changeGuard&&(this._changeGuard=!0,this._serialized.set(this.toJSON()),this._changeGuard=!1),this._changed.emit(e),this._stateChanged.emit(void 0)}_onSerializedChanged(t,e){this._changeGuard||(this._changeGuard=!0,this.fromJSON(e.newValue),this._changeGuard=!1)}_onGenericChange(){this._stateChanged.emit(void 0)}}!function(t){class e{createAttachmentModel(t){return new i.a(t)}}t.ContentFactory=e,t.defaultContentFactory=new e}(s||(s={}));class A{constructor(t){this._parent=t.parent||null,this._model=t.model}resolveUrl(t){return this._parent&&!t.startsWith("attachment:")?this._parent.resolveUrl(t):Promise.resolve(t)}getDownloadUrl(t){if(this._parent&&!t.startsWith("attachment:"))return this._parent.getDownloadUrl(t);const e=t.slice("attachment:".length);if(!this._model.has(e))return Promise.resolve(t);const{data:n}=this._model.get(e),r=Object.keys(n)[0];if(-1===o.b.mimeTypes.indexOf(r))return Promise.reject(`Cannot render unknown image mime type "${r}".`);const i=`data:${r};base64,${n[r]}`;return Promise.resolve(i)}isLocal(t){return!(this._parent&&!t.startsWith("attachment:"))||this._parent.isLocal(t)}}},function(t,e,n){!function(t){"use strict";function e(t){for(var e={},n=0;n<t.length;++n)e[t[n].toLowerCase()]=!0;return e}t.defineMode("css",function(e,n){var r=n.inline;n.propertyKeywords||(n=t.resolveMode("text/css"));var i,o,a=e.indentUnit,s=n.tokenHooks,A=n.documentTypes||{},l=n.mediaTypes||{},c=n.mediaFeatures||{},u=n.mediaValueKeywords||{},g=n.propertyKeywords||{},h=n.nonStandardPropertyKeywords||{},d=n.fontProperties||{},f=n.counterDescriptors||{},p=n.colorKeywords||{},I=n.valueKeywords||{},C=n.allowNested,m=n.lineComment,y=!0===n.supportsAtComponent;function E(t,e){return i=e,t}function v(t){return function(e,n){for(var r,i=!1;null!=(r=e.next());){if(r==t&&!i){")"==t&&e.backUp(1);break}i=!i&&"\\"==r}return(r==t||!i&&")"!=t)&&(n.tokenize=null),E("string","string")}}function B(t,e){return t.next(),t.match(/\s*[\"\')]/,!1)?e.tokenize=null:e.tokenize=v(")"),E(null,"(")}function b(t,e,n){this.type=t,this.indent=e,this.prev=n}function Q(t,e,n,r){return t.context=new b(n,e.indentation()+(!1===r?0:a),t.context),n}function w(t){return t.context.prev&&(t.context=t.context.prev),t.context.type}function x(t,e,n){return M[n.context.type](t,e,n)}function _(t,e,n,r){for(var i=r||1;i>0;i--)n.context=n.context.prev;return x(t,e,n)}function D(t){var e=t.current().toLowerCase();o=I.hasOwnProperty(e)?"atom":p.hasOwnProperty(e)?"keyword":"variable"}var M={top:function(t,e,n){if("{"==t)return Q(n,e,"block");if("}"==t&&n.context.prev)return w(n);if(y&&/@component/i.test(t))return Q(n,e,"atComponentBlock");if(/^@(-moz-)?document$/i.test(t))return Q(n,e,"documentTypes");if(/^@(media|supports|(-moz-)?document|import)$/i.test(t))return Q(n,e,"atBlock");if(/^@(font-face|counter-style)/i.test(t))return n.stateArg=t,"restricted_atBlock_before";if(/^@(-(moz|ms|o|webkit)-)?keyframes$/i.test(t))return"keyframes";if(t&&"@"==t.charAt(0))return Q(n,e,"at");if("hash"==t)o="builtin";else if("word"==t)o="tag";else{if("variable-definition"==t)return"maybeprop";if("interpolation"==t)return Q(n,e,"interpolation");if(":"==t)return"pseudo";if(C&&"("==t)return Q(n,e,"parens")}return n.context.type},block:function(t,e,n){if("word"==t){var r=e.current().toLowerCase();return g.hasOwnProperty(r)?(o="property","maybeprop"):h.hasOwnProperty(r)?(o="string-2","maybeprop"):C?(o=e.match(/^\s*:(?:\s|$)/,!1)?"property":"tag","block"):(o+=" error","maybeprop")}return"meta"==t?"block":C||"hash"!=t&&"qualifier"!=t?M.top(t,e,n):(o="error","block")},maybeprop:function(t,e,n){return":"==t?Q(n,e,"prop"):x(t,e,n)},prop:function(t,e,n){if(";"==t)return w(n);if("{"==t&&C)return Q(n,e,"propBlock");if("}"==t||"{"==t)return _(t,e,n);if("("==t)return Q(n,e,"parens");if("hash"!=t||/^#([0-9a-fA-f]{3,4}|[0-9a-fA-f]{6}|[0-9a-fA-f]{8})$/.test(e.current())){if("word"==t)D(e);else if("interpolation"==t)return Q(n,e,"interpolation")}else o+=" error";return"prop"},propBlock:function(t,e,n){return"}"==t?w(n):"word"==t?(o="property","maybeprop"):n.context.type},parens:function(t,e,n){return"{"==t||"}"==t?_(t,e,n):")"==t?w(n):"("==t?Q(n,e,"parens"):"interpolation"==t?Q(n,e,"interpolation"):("word"==t&&D(e),"parens")},pseudo:function(t,e,n){return"meta"==t?"pseudo":"word"==t?(o="variable-3",n.context.type):x(t,e,n)},documentTypes:function(t,e,n){return"word"==t&&A.hasOwnProperty(e.current())?(o="tag",n.context.type):M.atBlock(t,e,n)},atBlock:function(t,e,n){if("("==t)return Q(n,e,"atBlock_parens");if("}"==t||";"==t)return _(t,e,n);if("{"==t)return w(n)&&Q(n,e,C?"block":"top");if("interpolation"==t)return Q(n,e,"interpolation");if("word"==t){var r=e.current().toLowerCase();o="only"==r||"not"==r||"and"==r||"or"==r?"keyword":l.hasOwnProperty(r)?"attribute":c.hasOwnProperty(r)?"property":u.hasOwnProperty(r)?"keyword":g.hasOwnProperty(r)?"property":h.hasOwnProperty(r)?"string-2":I.hasOwnProperty(r)?"atom":p.hasOwnProperty(r)?"keyword":"error"}return n.context.type},atComponentBlock:function(t,e,n){return"}"==t?_(t,e,n):"{"==t?w(n)&&Q(n,e,C?"block":"top",!1):("word"==t&&(o="error"),n.context.type)},atBlock_parens:function(t,e,n){return")"==t?w(n):"{"==t||"}"==t?_(t,e,n,2):M.atBlock(t,e,n)},restricted_atBlock_before:function(t,e,n){return"{"==t?Q(n,e,"restricted_atBlock"):"word"==t&&"@counter-style"==n.stateArg?(o="variable","restricted_atBlock_before"):x(t,e,n)},restricted_atBlock:function(t,e,n){return"}"==t?(n.stateArg=null,w(n)):"word"==t?(o="@font-face"==n.stateArg&&!d.hasOwnProperty(e.current().toLowerCase())||"@counter-style"==n.stateArg&&!f.hasOwnProperty(e.current().toLowerCase())?"error":"property","maybeprop"):"restricted_atBlock"},keyframes:function(t,e,n){return"word"==t?(o="variable","keyframes"):"{"==t?Q(n,e,"top"):x(t,e,n)},at:function(t,e,n){return";"==t?w(n):"{"==t||"}"==t?_(t,e,n):("word"==t?o="tag":"hash"==t&&(o="builtin"),"at")},interpolation:function(t,e,n){return"}"==t?w(n):"{"==t||";"==t?_(t,e,n):("word"==t?o="variable":"variable"!=t&&"("!=t&&")"!=t&&(o="error"),"interpolation")}};return{startState:function(t){return{tokenize:null,state:r?"block":"top",stateArg:null,context:new b(r?"block":"top",t||0,null)}},token:function(t,e){if(!e.tokenize&&t.eatSpace())return null;var n=(e.tokenize||function(t,e){var n=t.next();if(s[n]){var r=s[n](t,e);if(!1!==r)return r}return"@"==n?(t.eatWhile(/[\w\\\-]/),E("def",t.current())):"="==n||("~"==n||"|"==n)&&t.eat("=")?E(null,"compare"):'"'==n||"'"==n?(e.tokenize=v(n),e.tokenize(t,e)):"#"==n?(t.eatWhile(/[\w\\\-]/),E("atom","hash")):"!"==n?(t.match(/^\s*\w*/),E("keyword","important")):/\d/.test(n)||"."==n&&t.eat(/\d/)?(t.eatWhile(/[\w.%]/),E("number","unit")):"-"!==n?/[,+>*\/]/.test(n)?E(null,"select-op"):"."==n&&t.match(/^-?[_a-z][_a-z0-9-]*/i)?E("qualifier","qualifier"):/[:;{}\[\]\(\)]/.test(n)?E(null,n):t.match(/[\w-.]+(?=\()/)?(/^(url(-prefix)?|domain|regexp)$/.test(t.current().toLowerCase())&&(e.tokenize=B),E("variable callee","variable")):/[\w\\\-]/.test(n)?(t.eatWhile(/[\w\\\-]/),E("property","word")):E(null,null):/[\d.]/.test(t.peek())?(t.eatWhile(/[\w.%]/),E("number","unit")):t.match(/^-[\w\\\-]*/)?(t.eatWhile(/[\w\\\-]/),t.match(/^\s*:/,!1)?E("variable-2","variable-definition"):E("variable-2","variable")):t.match(/^\w+-/)?E("meta","meta"):void 0})(t,e);return n&&"object"==typeof n&&(i=n[1],n=n[0]),o=n,"comment"!=i&&(e.state=M[e.state](i,t,e)),o},indent:function(t,e){var n=t.context,r=e&&e.charAt(0),i=n.indent;return"prop"!=n.type||"}"!=r&&")"!=r||(n=n.prev),n.prev&&("}"!=r||"block"!=n.type&&"top"!=n.type&&"interpolation"!=n.type&&"restricted_atBlock"!=n.type?(")"!=r||"parens"!=n.type&&"atBlock_parens"!=n.type)&&("{"!=r||"at"!=n.type&&"atBlock"!=n.type)||(i=Math.max(0,n.indent-a)):(n=n.prev,i=n.indent)),i},electricChars:"}",blockCommentStart:"/*",blockCommentEnd:"*/",blockCommentContinue:" * ",lineComment:m,fold:"brace"}});var n=["domain","regexp","url","url-prefix"],r=e(n),i=["all","aural","braille","handheld","print","projection","screen","tty","tv","embossed"],o=e(i),a=["width","min-width","max-width","height","min-height","max-height","device-width","min-device-width","max-device-width","device-height","min-device-height","max-device-height","aspect-ratio","min-aspect-ratio","max-aspect-ratio","device-aspect-ratio","min-device-aspect-ratio","max-device-aspect-ratio","color","min-color","max-color","color-index","min-color-index","max-color-index","monochrome","min-monochrome","max-monochrome","resolution","min-resolution","max-resolution","scan","grid","orientation","device-pixel-ratio","min-device-pixel-ratio","max-device-pixel-ratio","pointer","any-pointer","hover","any-hover"],s=e(a),A=["landscape","portrait","none","coarse","fine","on-demand","hover","interlace","progressive"],l=e(A),c=["align-content","align-items","align-self","alignment-adjust","alignment-baseline","anchor-point","animation","animation-delay","animation-direction","animation-duration","animation-fill-mode","animation-iteration-count","animation-name","animation-play-state","animation-timing-function","appearance","azimuth","backface-visibility","background","background-attachment","background-blend-mode","background-clip","background-color","background-image","background-origin","background-position","background-repeat","background-size","baseline-shift","binding","bleed","bookmark-label","bookmark-level","bookmark-state","bookmark-target","border","border-bottom","border-bottom-color","border-bottom-left-radius","border-bottom-right-radius","border-bottom-style","border-bottom-width","border-collapse","border-color","border-image","border-image-outset","border-image-repeat","border-image-slice","border-image-source","border-image-width","border-left","border-left-color","border-left-style","border-left-width","border-radius","border-right","border-right-color","border-right-style","border-right-width","border-spacing","border-style","border-top","border-top-color","border-top-left-radius","border-top-right-radius","border-top-style","border-top-width","border-width","bottom","box-decoration-break","box-shadow","box-sizing","break-after","break-before","break-inside","caption-side","caret-color","clear","clip","color","color-profile","column-count","column-fill","column-gap","column-rule","column-rule-color","column-rule-style","column-rule-width","column-span","column-width","columns","content","counter-increment","counter-reset","crop","cue","cue-after","cue-before","cursor","direction","display","dominant-baseline","drop-initial-after-adjust","drop-initial-after-align","drop-initial-before-adjust","drop-initial-before-align","drop-initial-size","drop-initial-value","elevation","empty-cells","fit","fit-position","flex","flex-basis","flex-direction","flex-flow","flex-grow","flex-shrink","flex-wrap","float","float-offset","flow-from","flow-into","font","font-feature-settings","font-family","font-kerning","font-language-override","font-size","font-size-adjust","font-stretch","font-style","font-synthesis","font-variant","font-variant-alternates","font-variant-caps","font-variant-east-asian","font-variant-ligatures","font-variant-numeric","font-variant-position","font-weight","grid","grid-area","grid-auto-columns","grid-auto-flow","grid-auto-rows","grid-column","grid-column-end","grid-column-gap","grid-column-start","grid-gap","grid-row","grid-row-end","grid-row-gap","grid-row-start","grid-template","grid-template-areas","grid-template-columns","grid-template-rows","hanging-punctuation","height","hyphens","icon","image-orientation","image-rendering","image-resolution","inline-box-align","justify-content","justify-items","justify-self","left","letter-spacing","line-break","line-height","line-stacking","line-stacking-ruby","line-stacking-shift","line-stacking-strategy","list-style","list-style-image","list-style-position","list-style-type","margin","margin-bottom","margin-left","margin-right","margin-top","marks","marquee-direction","marquee-loop","marquee-play-count","marquee-speed","marquee-style","max-height","max-width","min-height","min-width","mix-blend-mode","move-to","nav-down","nav-index","nav-left","nav-right","nav-up","object-fit","object-position","opacity","order","orphans","outline","outline-color","outline-offset","outline-style","outline-width","overflow","overflow-style","overflow-wrap","overflow-x","overflow-y","padding","padding-bottom","padding-left","padding-right","padding-top","page","page-break-after","page-break-before","page-break-inside","page-policy","pause","pause-after","pause-before","perspective","perspective-origin","pitch","pitch-range","place-content","place-items","place-self","play-during","position","presentation-level","punctuation-trim","quotes","region-break-after","region-break-before","region-break-inside","region-fragment","rendering-intent","resize","rest","rest-after","rest-before","richness","right","rotation","rotation-point","ruby-align","ruby-overhang","ruby-position","ruby-span","shape-image-threshold","shape-inside","shape-margin","shape-outside","size","speak","speak-as","speak-header","speak-numeral","speak-punctuation","speech-rate","stress","string-set","tab-size","table-layout","target","target-name","target-new","target-position","text-align","text-align-last","text-decoration","text-decoration-color","text-decoration-line","text-decoration-skip","text-decoration-style","text-emphasis","text-emphasis-color","text-emphasis-position","text-emphasis-style","text-height","text-indent","text-justify","text-outline","text-overflow","text-shadow","text-size-adjust","text-space-collapse","text-transform","text-underline-position","text-wrap","top","transform","transform-origin","transform-style","transition","transition-delay","transition-duration","transition-property","transition-timing-function","unicode-bidi","user-select","vertical-align","visibility","voice-balance","voice-duration","voice-family","voice-pitch","voice-range","voice-rate","voice-stress","voice-volume","volume","white-space","widows","width","will-change","word-break","word-spacing","word-wrap","z-index","clip-path","clip-rule","mask","enable-background","filter","flood-color","flood-opacity","lighting-color","stop-color","stop-opacity","pointer-events","color-interpolation","color-interpolation-filters","color-rendering","fill","fill-opacity","fill-rule","image-rendering","marker","marker-end","marker-mid","marker-start","shape-rendering","stroke","stroke-dasharray","stroke-dashoffset","stroke-linecap","stroke-linejoin","stroke-miterlimit","stroke-opacity","stroke-width","text-rendering","baseline-shift","dominant-baseline","glyph-orientation-horizontal","glyph-orientation-vertical","text-anchor","writing-mode"],u=e(c),g=["scrollbar-arrow-color","scrollbar-base-color","scrollbar-dark-shadow-color","scrollbar-face-color","scrollbar-highlight-color","scrollbar-shadow-color","scrollbar-3d-light-color","scrollbar-track-color","shape-inside","searchfield-cancel-button","searchfield-decoration","searchfield-results-button","searchfield-results-decoration","zoom"],h=e(g),d=e(["font-family","src","unicode-range","font-variant","font-feature-settings","font-stretch","font-weight","font-style"]),f=e(["additive-symbols","fallback","negative","pad","prefix","range","speak-as","suffix","symbols","system"]),p=["aliceblue","antiquewhite","aqua","aquamarine","azure","beige","bisque","black","blanchedalmond","blue","blueviolet","brown","burlywood","cadetblue","chartreuse","chocolate","coral","cornflowerblue","cornsilk","crimson","cyan","darkblue","darkcyan","darkgoldenrod","darkgray","darkgreen","darkkhaki","darkmagenta","darkolivegreen","darkorange","darkorchid","darkred","darksalmon","darkseagreen","darkslateblue","darkslategray","darkturquoise","darkviolet","deeppink","deepskyblue","dimgray","dodgerblue","firebrick","floralwhite","forestgreen","fuchsia","gainsboro","ghostwhite","gold","goldenrod","gray","grey","green","greenyellow","honeydew","hotpink","indianred","indigo","ivory","khaki","lavender","lavenderblush","lawngreen","lemonchiffon","lightblue","lightcoral","lightcyan","lightgoldenrodyellow","lightgray","lightgreen","lightpink","lightsalmon","lightseagreen","lightskyblue","lightslategray","lightsteelblue","lightyellow","lime","limegreen","linen","magenta","maroon","mediumaquamarine","mediumblue","mediumorchid","mediumpurple","mediumseagreen","mediumslateblue","mediumspringgreen","mediumturquoise","mediumvioletred","midnightblue","mintcream","mistyrose","moccasin","navajowhite","navy","oldlace","olive","olivedrab","orange","orangered","orchid","palegoldenrod","palegreen","paleturquoise","palevioletred","papayawhip","peachpuff","peru","pink","plum","powderblue","purple","rebeccapurple","red","rosybrown","royalblue","saddlebrown","salmon","sandybrown","seagreen","seashell","sienna","silver","skyblue","slateblue","slategray","snow","springgreen","steelblue","tan","teal","thistle","tomato","turquoise","violet","wheat","white","whitesmoke","yellow","yellowgreen"],I=e(p),C=["above","absolute","activeborder","additive","activecaption","afar","after-white-space","ahead","alias","all","all-scroll","alphabetic","alternate","always","amharic","amharic-abegede","antialiased","appworkspace","arabic-indic","armenian","asterisks","attr","auto","auto-flow","avoid","avoid-column","avoid-page","avoid-region","background","backwards","baseline","below","bidi-override","binary","bengali","blink","block","block-axis","bold","bolder","border","border-box","both","bottom","break","break-all","break-word","bullets","button","button-bevel","buttonface","buttonhighlight","buttonshadow","buttontext","calc","cambodian","capitalize","caps-lock-indicator","caption","captiontext","caret","cell","center","checkbox","circle","cjk-decimal","cjk-earthly-branch","cjk-heavenly-stem","cjk-ideographic","clear","clip","close-quote","col-resize","collapse","color","color-burn","color-dodge","column","column-reverse","compact","condensed","contain","content","contents","content-box","context-menu","continuous","copy","counter","counters","cover","crop","cross","crosshair","currentcolor","cursive","cyclic","darken","dashed","decimal","decimal-leading-zero","default","default-button","dense","destination-atop","destination-in","destination-out","destination-over","devanagari","difference","disc","discard","disclosure-closed","disclosure-open","document","dot-dash","dot-dot-dash","dotted","double","down","e-resize","ease","ease-in","ease-in-out","ease-out","element","ellipse","ellipsis","embed","end","ethiopic","ethiopic-abegede","ethiopic-abegede-am-et","ethiopic-abegede-gez","ethiopic-abegede-ti-er","ethiopic-abegede-ti-et","ethiopic-halehame-aa-er","ethiopic-halehame-aa-et","ethiopic-halehame-am-et","ethiopic-halehame-gez","ethiopic-halehame-om-et","ethiopic-halehame-sid-et","ethiopic-halehame-so-et","ethiopic-halehame-ti-er","ethiopic-halehame-ti-et","ethiopic-halehame-tig","ethiopic-numeric","ew-resize","exclusion","expanded","extends","extra-condensed","extra-expanded","fantasy","fast","fill","fixed","flat","flex","flex-end","flex-start","footnotes","forwards","from","geometricPrecision","georgian","graytext","grid","groove","gujarati","gurmukhi","hand","hangul","hangul-consonant","hard-light","hebrew","help","hidden","hide","higher","highlight","highlighttext","hiragana","hiragana-iroha","horizontal","hsl","hsla","hue","icon","ignore","inactiveborder","inactivecaption","inactivecaptiontext","infinite","infobackground","infotext","inherit","initial","inline","inline-axis","inline-block","inline-flex","inline-grid","inline-table","inset","inside","intrinsic","invert","italic","japanese-formal","japanese-informal","justify","kannada","katakana","katakana-iroha","keep-all","khmer","korean-hangul-formal","korean-hanja-formal","korean-hanja-informal","landscape","lao","large","larger","left","level","lighter","lighten","line-through","linear","linear-gradient","lines","list-item","listbox","listitem","local","logical","loud","lower","lower-alpha","lower-armenian","lower-greek","lower-hexadecimal","lower-latin","lower-norwegian","lower-roman","lowercase","ltr","luminosity","malayalam","match","matrix","matrix3d","media-controls-background","media-current-time-display","media-fullscreen-button","media-mute-button","media-play-button","media-return-to-realtime-button","media-rewind-button","media-seek-back-button","media-seek-forward-button","media-slider","media-sliderthumb","media-time-remaining-display","media-volume-slider","media-volume-slider-container","media-volume-sliderthumb","medium","menu","menulist","menulist-button","menulist-text","menulist-textfield","menutext","message-box","middle","min-intrinsic","mix","mongolian","monospace","move","multiple","multiply","myanmar","n-resize","narrower","ne-resize","nesw-resize","no-close-quote","no-drop","no-open-quote","no-repeat","none","normal","not-allowed","nowrap","ns-resize","numbers","numeric","nw-resize","nwse-resize","oblique","octal","opacity","open-quote","optimizeLegibility","optimizeSpeed","oriya","oromo","outset","outside","outside-shape","overlay","overline","padding","padding-box","painted","page","paused","persian","perspective","plus-darker","plus-lighter","pointer","polygon","portrait","pre","pre-line","pre-wrap","preserve-3d","progress","push-button","radial-gradient","radio","read-only","read-write","read-write-plaintext-only","rectangle","region","relative","repeat","repeating-linear-gradient","repeating-radial-gradient","repeat-x","repeat-y","reset","reverse","rgb","rgba","ridge","right","rotate","rotate3d","rotateX","rotateY","rotateZ","round","row","row-resize","row-reverse","rtl","run-in","running","s-resize","sans-serif","saturation","scale","scale3d","scaleX","scaleY","scaleZ","screen","scroll","scrollbar","scroll-position","se-resize","searchfield","searchfield-cancel-button","searchfield-decoration","searchfield-results-button","searchfield-results-decoration","self-start","self-end","semi-condensed","semi-expanded","separate","serif","show","sidama","simp-chinese-formal","simp-chinese-informal","single","skew","skewX","skewY","skip-white-space","slide","slider-horizontal","slider-vertical","sliderthumb-horizontal","sliderthumb-vertical","slow","small","small-caps","small-caption","smaller","soft-light","solid","somali","source-atop","source-in","source-out","source-over","space","space-around","space-between","space-evenly","spell-out","square","square-button","start","static","status-bar","stretch","stroke","sub","subpixel-antialiased","super","sw-resize","symbolic","symbols","system-ui","table","table-caption","table-cell","table-column","table-column-group","table-footer-group","table-header-group","table-row","table-row-group","tamil","telugu","text","text-bottom","text-top","textarea","textfield","thai","thick","thin","threeddarkshadow","threedface","threedhighlight","threedlightshadow","threedshadow","tibetan","tigre","tigrinya-er","tigrinya-er-abegede","tigrinya-et","tigrinya-et-abegede","to","top","trad-chinese-formal","trad-chinese-informal","transform","translate","translate3d","translateX","translateY","translateZ","transparent","ultra-condensed","ultra-expanded","underline","unset","up","upper-alpha","upper-armenian","upper-greek","upper-hexadecimal","upper-latin","upper-norwegian","upper-roman","uppercase","urdu","url","var","vertical","vertical-text","visible","visibleFill","visiblePainted","visibleStroke","visual","w-resize","wait","wave","wider","window","windowframe","windowtext","words","wrap","wrap-reverse","x-large","x-small","xor","xx-large","xx-small"],m=e(C),y=n.concat(i).concat(a).concat(A).concat(c).concat(g).concat(p).concat(C);function E(t,e){for(var n,r=!1;null!=(n=t.next());){if(r&&"/"==n){e.tokenize=null;break}r="*"==n}return["comment","comment"]}t.registerHelper("hintWords","css",y),t.defineMIME("text/css",{documentTypes:r,mediaTypes:o,mediaFeatures:s,mediaValueKeywords:l,propertyKeywords:u,nonStandardPropertyKeywords:h,fontProperties:d,counterDescriptors:f,colorKeywords:I,valueKeywords:m,tokenHooks:{"/":function(t,e){return!!t.eat("*")&&(e.tokenize=E,E(t,e))}},name:"css"}),t.defineMIME("text/x-scss",{mediaTypes:o,mediaFeatures:s,mediaValueKeywords:l,propertyKeywords:u,nonStandardPropertyKeywords:h,colorKeywords:I,valueKeywords:m,fontProperties:d,allowNested:!0,lineComment:"//",tokenHooks:{"/":function(t,e){return t.eat("/")?(t.skipToEnd(),["comment","comment"]):t.eat("*")?(e.tokenize=E,E(t,e)):["operator","operator"]},":":function(t){return!!t.match(/\s*\{/,!1)&&[null,null]},$:function(t){return t.match(/^[\w-]+/),t.match(/^\s*:/,!1)?["variable-2","variable-definition"]:["variable-2","variable"]},"#":function(t){return!!t.eat("{")&&[null,"interpolation"]}},name:"css",helperType:"scss"}),t.defineMIME("text/x-less",{mediaTypes:o,mediaFeatures:s,mediaValueKeywords:l,propertyKeywords:u,nonStandardPropertyKeywords:h,colorKeywords:I,valueKeywords:m,fontProperties:d,allowNested:!0,lineComment:"//",tokenHooks:{"/":function(t,e){return t.eat("/")?(t.skipToEnd(),["comment","comment"]):t.eat("*")?(e.tokenize=E,E(t,e)):["operator","operator"]},"@":function(t){return t.eat("{")?[null,"interpolation"]:!t.match(/^(charset|document|font-face|import|(-(moz|ms|o|webkit)-)?keyframes|media|namespace|page|supports)\b/i,!1)&&(t.eatWhile(/[\w\\\-]/),t.match(/^\s*:/,!1)?["variable-2","variable-definition"]:["variable-2","variable"])},"&":function(){return["atom","atom"]}},name:"css",helperType:"less"}),t.defineMIME("text/x-gss",{documentTypes:r,mediaTypes:o,mediaFeatures:s,propertyKeywords:u,nonStandardPropertyKeywords:h,fontProperties:d,counterDescriptors:f,colorKeywords:I,valueKeywords:m,supportsAtComponent:!0,tokenHooks:{"/":function(t,e){return!!t.eat("*")&&(e.tokenize=E,E(t,e))}},name:"css",helperType:"gss"})}(n(15))},function(t,e,n){!function(t){"use strict";var e={autoSelfClosers:{area:!0,base:!0,br:!0,col:!0,command:!0,embed:!0,frame:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0,menuitem:!0},implicitlyClosed:{dd:!0,li:!0,optgroup:!0,option:!0,p:!0,rp:!0,rt:!0,tbody:!0,td:!0,tfoot:!0,th:!0,tr:!0},contextGrabbers:{dd:{dd:!0,dt:!0},dt:{dd:!0,dt:!0},li:{li:!0},option:{option:!0,optgroup:!0},optgroup:{optgroup:!0},p:{address:!0,article:!0,aside:!0,blockquote:!0,dir:!0,div:!0,dl:!0,fieldset:!0,footer:!0,form:!0,h1:!0,h2:!0,h3:!0,h4:!0,h5:!0,h6:!0,header:!0,hgroup:!0,hr:!0,menu:!0,nav:!0,ol:!0,p:!0,pre:!0,section:!0,table:!0,ul:!0},rp:{rp:!0,rt:!0},rt:{rp:!0,rt:!0},tbody:{tbody:!0,tfoot:!0},td:{td:!0,th:!0},tfoot:{tbody:!0},th:{td:!0,th:!0},thead:{tbody:!0,tfoot:!0},tr:{tr:!0}},doNotIndent:{pre:!0},allowUnquoted:!0,allowMissing:!0,caseFold:!0},n={autoSelfClosers:{},implicitlyClosed:{},contextGrabbers:{},doNotIndent:{},allowUnquoted:!1,allowMissing:!1,allowMissingTagName:!1,caseFold:!1};t.defineMode("xml",function(r,i){var o,a,s=r.indentUnit,A={},l=i.htmlMode?e:n;for(var c in l)A[c]=l[c];for(var c in i)A[c]=i[c];function u(t,e){function n(n){return e.tokenize=n,n(t,e)}var r=t.next();return"<"==r?t.eat("!")?t.eat("[")?t.match("CDATA[")?n(h("atom","]]>")):null:t.match("--")?n(h("comment","--\x3e")):t.match("DOCTYPE",!0,!0)?(t.eatWhile(/[\w\._\-]/),n(function t(e){return function(n,r){for(var i;null!=(i=n.next());){if("<"==i)return r.tokenize=t(e+1),r.tokenize(n,r);if(">"==i){if(1==e){r.tokenize=u;break}return r.tokenize=t(e-1),r.tokenize(n,r)}}return"meta"}}(1))):null:t.eat("?")?(t.eatWhile(/[\w\._\-]/),e.tokenize=h("meta","?>"),"meta"):(o=t.eat("/")?"closeTag":"openTag",e.tokenize=g,"tag bracket"):"&"==r?(t.eat("#")?t.eat("x")?t.eatWhile(/[a-fA-F\d]/)&&t.eat(";"):t.eatWhile(/[\d]/)&&t.eat(";"):t.eatWhile(/[\w\.\-:]/)&&t.eat(";"))?"atom":"error":(t.eatWhile(/[^&<]/),null)}function g(t,e){var n,r,i=t.next();if(">"==i||"/"==i&&t.eat(">"))return e.tokenize=u,o=">"==i?"endTag":"selfcloseTag","tag bracket";if("="==i)return o="equals",null;if("<"==i){e.tokenize=u,e.state=I,e.tagName=e.tagStart=null;var a=e.tokenize(t,e);return a?a+" tag error":"tag error"}return/[\'\"]/.test(i)?(e.tokenize=(n=i,(r=function(t,e){for(;!t.eol();)if(t.next()==n){e.tokenize=g;break}return"string"}).isInAttribute=!0,r),e.stringStartCol=t.column(),e.tokenize(t,e)):(t.match(/^[^\s\u00a0=<>\"\']*[^\s\u00a0=<>\"\'\/]/),"word")}function h(t,e){return function(n,r){for(;!n.eol();){if(n.match(e)){r.tokenize=u;break}n.next()}return t}}function d(t,e,n){this.prev=t.context,this.tagName=e,this.indent=t.indented,this.startOfLine=n,(A.doNotIndent.hasOwnProperty(e)||t.context&&t.context.noIndent)&&(this.noIndent=!0)}function f(t){t.context&&(t.context=t.context.prev)}function p(t,e){for(var n;;){if(!t.context)return;if(n=t.context.tagName,!A.contextGrabbers.hasOwnProperty(n)||!A.contextGrabbers[n].hasOwnProperty(e))return;f(t)}}function I(t,e,n){return"openTag"==t?(n.tagStart=e.column(),C):"closeTag"==t?m:I}function C(t,e,n){return"word"==t?(n.tagName=e.current(),a="tag",v):A.allowMissingTagName&&"endTag"==t?(a="tag bracket",v(t,0,n)):(a="error",C)}function m(t,e,n){if("word"==t){var r=e.current();return n.context&&n.context.tagName!=r&&A.implicitlyClosed.hasOwnProperty(n.context.tagName)&&f(n),n.context&&n.context.tagName==r||!1===A.matchClosing?(a="tag",y):(a="tag error",E)}return A.allowMissingTagName&&"endTag"==t?(a="tag bracket",y(t,0,n)):(a="error",E)}function y(t,e,n){return"endTag"!=t?(a="error",y):(f(n),I)}function E(t,e,n){return a="error",y(t,0,n)}function v(t,e,n){if("word"==t)return a="attribute",B;if("endTag"==t||"selfcloseTag"==t){var r=n.tagName,i=n.tagStart;return n.tagName=n.tagStart=null,"selfcloseTag"==t||A.autoSelfClosers.hasOwnProperty(r)?p(n,r):(p(n,r),n.context=new d(n,r,i==n.indented)),I}return a="error",v}function B(t,e,n){return"equals"==t?b:(A.allowMissing||(a="error"),v(t,0,n))}function b(t,e,n){return"string"==t?Q:"word"==t&&A.allowUnquoted?(a="string",v):(a="error",v(t,0,n))}function Q(t,e,n){return"string"==t?Q:v(t,0,n)}return u.isInText=!0,{startState:function(t){var e={tokenize:u,state:I,indented:t||0,tagName:null,tagStart:null,context:null};return null!=t&&(e.baseIndent=t),e},token:function(t,e){if(!e.tagName&&t.sol()&&(e.indented=t.indentation()),t.eatSpace())return null;o=null;var n=e.tokenize(t,e);return(n||o)&&"comment"!=n&&(a=null,e.state=e.state(o||n,t,e),a&&(n="error"==a?n+" error":a)),n},indent:function(e,n,r){var i=e.context;if(e.tokenize.isInAttribute)return e.tagStart==e.indented?e.stringStartCol+1:e.indented+s;if(i&&i.noIndent)return t.Pass;if(e.tokenize!=g&&e.tokenize!=u)return r?r.match(/^(\s*)/)[0].length:0;if(e.tagName)return!1!==A.multilineTagIndentPastTag?e.tagStart+e.tagName.length+2:e.tagStart+s*(A.multilineTagIndentFactor||1);if(A.alignCDATA&&/<!\[CDATA\[/.test(n))return 0;var o=n&&/^<(\/)?([\w_:\.-]*)/.exec(n);if(o&&o[1])for(;i;){if(i.tagName==o[2]){i=i.prev;break}if(!A.implicitlyClosed.hasOwnProperty(i.tagName))break;i=i.prev}else if(o)for(;i;){var a=A.contextGrabbers[i.tagName];if(!a||!a.hasOwnProperty(o[2]))break;i=i.prev}for(;i&&i.prev&&!i.startOfLine;)i=i.prev;return i?i.indent+s:e.baseIndent||0},electricInput:/<\/[\s\w:]+>$/,blockCommentStart:"\x3c!--",blockCommentEnd:"--\x3e",configuration:A.htmlMode?"html":"xml",helperType:A.htmlMode?"html":"xml",skipAttribute:function(t){t.state==b&&(t.state=v)}}}),t.defineMIME("text/xml","xml"),t.defineMIME("application/xml","xml"),t.mimeModes.hasOwnProperty("text/html")||t.defineMIME("text/html",{name:"xml",htmlMode:!0})}(n(15))},function(t,e,n){!function(t){"use strict";t.overlayMode=function(e,n,r){return{startState:function(){return{base:t.startState(e),overlay:t.startState(n),basePos:0,baseCur:null,overlayPos:0,overlayCur:null,streamSeen:null}},copyState:function(r){return{base:t.copyState(e,r.base),overlay:t.copyState(n,r.overlay),basePos:r.basePos,baseCur:null,overlayPos:r.overlayPos,overlayCur:null}},token:function(t,i){return(t!=i.streamSeen||Math.min(i.basePos,i.overlayPos)<t.start)&&(i.streamSeen=t,i.basePos=i.overlayPos=t.start),t.start==i.basePos&&(i.baseCur=e.token(t,i.base),i.basePos=t.pos),t.start==i.overlayPos&&(t.pos=t.start,i.overlayCur=n.token(t,i.overlay),i.overlayPos=t.pos),t.pos=Math.min(i.basePos,i.overlayPos),null==i.overlayCur?i.baseCur:null!=i.baseCur&&i.overlay.combineTokens||r&&null==i.overlay.combineTokens?i.baseCur+" "+i.overlayCur:i.overlayCur},indent:e.indent&&function(t,n,r){return e.indent(t.base,n,r)},electricChars:e.electricChars,innerMode:function(t){return{state:t.base,mode:e}},blankLine:function(t){var i,o;return e.blankLine&&(i=e.blankLine(t.base)),n.blankLine&&(o=n.blankLine(t.overlay)),null==o?i:r&&null!=i?i+" "+o:o}}}}(n(15))},function(t,e,n){!function(t){"use strict";function e(t,e,n,r,i,o){this.indented=t,this.column=e,this.type=n,this.info=r,this.align=i,this.prev=o}function n(t,n,r,i){var o=t.indented;return t.context&&"statement"==t.context.type&&"statement"!=r&&(o=t.context.indented),t.context=new e(o,n,r,i,null,t.context)}function r(t){var e=t.context.type;return")"!=e&&"]"!=e&&"}"!=e||(t.indented=t.context.indented),t.context=t.context.prev}function i(t,e,n){return"variable"==e.prevToken||"type"==e.prevToken||!!/\S(?:[^- ]>|[*\]])\s*$|\*$/.test(t.string.slice(0,n))||!(!e.typeAtEndOfLine||t.column()!=t.indentation())||void 0}function o(t){for(;;){if(!t||"top"==t.type)return!0;if("}"==t.type&&"namespace"!=t.prev.info)return!1;t=t.prev}}function a(t){for(var e={},n=t.split(" "),r=0;r<n.length;++r)e[n[r]]=!0;return e}function s(t,e){return"function"==typeof t?t(e):t.propertyIsEnumerable(e)}t.defineMode("clike",function(a,A){var l,c,u=a.indentUnit,g=A.statementIndentUnit||u,h=A.dontAlignCalls,d=A.keywords||{},f=A.types||{},p=A.builtin||{},I=A.blockKeywords||{},C=A.defKeywords||{},m=A.atoms||{},y=A.hooks||{},E=A.multiLineStrings,v=!1!==A.indentStatements,B=!1!==A.indentSwitch,b=A.namespaceSeparator,Q=A.isPunctuationChar||/[\[\]{}\(\),;\:\.]/,w=A.numberStart||/[\d\.]/,x=A.number||/^(?:0x[a-f\d]+|0b[01]+|(?:\d+\.?\d*|\.\d+)(?:e[-+]?\d+)?)(u|ll?|l|f)?/i,_=A.isOperatorChar||/[+\-*&%=<>!?|\/]/,D=A.isIdentifierChar||/[\w\$_\xa1-\uffff]/,M=A.isReservedIdentifier||!1;function k(t,e){var n,r=t.next();if(y[r]){var i=y[r](t,e);if(!1!==i)return i}if('"'==r||"'"==r)return e.tokenize=(n=r,function(t,e){for(var r,i=!1,o=!1;null!=(r=t.next());){if(r==n&&!i){o=!0;break}i=!i&&"\\"==r}return(o||!i&&!E)&&(e.tokenize=null),"string"}),e.tokenize(t,e);if(Q.test(r))return l=r,null;if(w.test(r)){if(t.backUp(1),t.match(x))return"number";t.next()}if("/"==r){if(t.eat("*"))return e.tokenize=S,S(t,e);if(t.eat("/"))return t.skipToEnd(),"comment"}if(_.test(r)){for(;!t.match(/^\/[\/*]/,!1)&&t.eat(_););return"operator"}if(t.eatWhile(D),b)for(;t.match(b);)t.eatWhile(D);var o=t.current();return s(d,o)?(s(I,o)&&(l="newstatement"),s(C,o)&&(c=!0),"keyword"):s(f,o)?"type":s(p,o)||M&&M(o)?(s(I,o)&&(l="newstatement"),"builtin"):s(m,o)?"atom":"variable"}function S(t,e){for(var n,r=!1;n=t.next();){if("/"==n&&r){e.tokenize=null;break}r="*"==n}return"comment"}function L(t,e){A.typeFirstDefinitions&&t.eol()&&o(e.context)&&(e.typeAtEndOfLine=i(t,e,t.pos))}return{startState:function(t){return{tokenize:null,context:new e((t||0)-u,0,"top",null,!1),indented:0,startOfLine:!0,prevToken:null}},token:function(t,e){var a=e.context;if(t.sol()&&(null==a.align&&(a.align=!1),e.indented=t.indentation(),e.startOfLine=!0),t.eatSpace())return L(t,e),null;l=c=null;var s=(e.tokenize||k)(t,e);if("comment"==s||"meta"==s)return s;if(null==a.align&&(a.align=!0),";"==l||":"==l||","==l&&t.match(/^\s*(?:\/\/.*)?$/,!1))for(;"statement"==e.context.type;)r(e);else if("{"==l)n(e,t.column(),"}");else if("["==l)n(e,t.column(),"]");else if("("==l)n(e,t.column(),")");else if("}"==l){for(;"statement"==a.type;)a=r(e);for("}"==a.type&&(a=r(e));"statement"==a.type;)a=r(e)}else l==a.type?r(e):v&&(("}"==a.type||"top"==a.type)&&";"!=l||"statement"==a.type&&"newstatement"==l)&&n(e,t.column(),"statement",t.current());if("variable"==s&&("def"==e.prevToken||A.typeFirstDefinitions&&i(t,e,t.start)&&o(e.context)&&t.match(/^\s*\(/,!1))&&(s="def"),y.token){var u=y.token(t,e,s);void 0!==u&&(s=u)}return"def"==s&&!1===A.styleDefs&&(s="variable"),e.startOfLine=!1,e.prevToken=c?"def":s||l,L(t,e),s},indent:function(e,n){if(e.tokenize!=k&&null!=e.tokenize||e.typeAtEndOfLine)return t.Pass;var r=e.context,i=n&&n.charAt(0),o=i==r.type;if("statement"==r.type&&"}"==i&&(r=r.prev),A.dontIndentStatements)for(;"statement"==r.type&&A.dontIndentStatements.test(r.info);)r=r.prev;if(y.indent){var a=y.indent(e,r,n,u);if("number"==typeof a)return a}var s=r.prev&&"switch"==r.prev.info;if(A.allmanIndentation&&/[{(]/.test(i)){for(;"top"!=r.type&&"}"!=r.type;)r=r.prev;return r.indented}return"statement"==r.type?r.indented+("{"==i?0:g):!r.align||h&&")"==r.type?")"!=r.type||o?r.indented+(o?0:u)+(o||!s||/^(?:case|default)\b/.test(n)?0:u):r.indented+g:r.column+(o?0:1)},electricInput:B?/^\s*(?:case .*?:|default:|\{\}?|\})$/:/^\s*[{}]$/,blockCommentStart:"/*",blockCommentEnd:"*/",blockCommentContinue:" * ",lineComment:"//",fold:"brace"}});var A="auto if break case register continue return default do sizeof static else struct switch extern typedef union for goto while enum const volatile inline restrict asm fortran",l=a("int long char short double float unsigned signed void bool"),c=a("SEL instancetype id Class Protocol BOOL");function u(t){return s(l,t)||/.+_t$/.test(t)}var g="case do else for if switch while struct enum union";function h(t,e){if(!e.startOfLine)return!1;for(var n,r=null;n=t.peek();){if("\\"==n&&t.match(/^.$/)){r=h;break}if("/"==n&&t.match(/^\/[\/\*]/,!1))break;t.next()}return e.tokenize=r,"meta"}function d(t,e){return"type"==e.prevToken&&"type"}function f(t){return!(!t||t.length<2||"_"!=t[0]||"_"!=t[1]&&t[1]===t[1].toLowerCase())}function p(t){return t.eatWhile(/[\w\.']/),"number"}function I(t,e){if(t.backUp(1),t.match(/(R|u8R|uR|UR|LR)/)){var n=t.match(/"([^\s\\()]{0,16})\(/);return!!n&&(e.cpp11RawStringDelim=n[1],e.tokenize=m,m(t,e))}return t.match(/(u8|u|U|L)/)?!!t.match(/["']/,!1)&&"string":(t.next(),!1)}function C(t,e){for(var n;null!=(n=t.next());)if('"'==n&&!t.eat('"')){e.tokenize=null;break}return"string"}function m(t,e){var n=e.cpp11RawStringDelim.replace(/[^\w\s]/g,"\\$&"),r=t.match(new RegExp(".*?\\)"+n+'"'));return r?e.tokenize=null:t.skipToEnd(),"string"}function y(e,n){"string"==typeof e&&(e=[e]);var r=[];function i(t){if(t)for(var e in t)t.hasOwnProperty(e)&&r.push(e)}i(n.keywords),i(n.types),i(n.builtin),i(n.atoms),r.length&&(n.helperType=e[0],t.registerHelper("hintWords",e[0],r));for(var o=0;o<e.length;++o)t.defineMIME(e[o],n)}function E(t,e){for(var n=!1;!t.eol();){if(!n&&t.match('"""')){e.tokenize=null;break}n="\\"==t.next()&&!n}return"string"}y(["text/x-csrc","text/x-c","text/x-chdr"],{name:"clike",keywords:a(A),types:u,blockKeywords:a(g),defKeywords:a("struct enum union"),typeFirstDefinitions:!0,atoms:a("NULL true false"),isReservedIdentifier:f,hooks:{"#":h,"*":d},modeProps:{fold:["brace","include"]}}),y(["text/x-c++src","text/x-c++hdr"],{name:"clike",keywords:a(A+"alignas alignof and and_eq audit axiom bitand bitor catch class compl concept constexpr const_cast decltype delete dynamic_cast explicit export final friend import module mutable namespace new noexcept not not_eq operator or or_eq override private protected public reinterpret_cast requires static_assert static_cast template this thread_local throw try typeid typename using virtual xor xor_eq"),types:u,blockKeywords:a(g+" class try catch"),defKeywords:a("struct enum union class namespace"),typeFirstDefinitions:!0,atoms:a("true false NULL nullptr"),dontIndentStatements:/^template$/,isIdentifierChar:/[\w\$_~\xa1-\uffff]/,isReservedIdentifier:f,hooks:{"#":h,"*":d,u:I,U:I,L:I,R:I,0:p,1:p,2:p,3:p,4:p,5:p,6:p,7:p,8:p,9:p,token:function(t,e,n){if("variable"==n&&"("==t.peek()&&(";"==e.prevToken||null==e.prevToken||"}"==e.prevToken)&&(r=t.current(),(i=/(\w+)::~?(\w+)$/.exec(r))&&i[1]==i[2]))return"def";var r,i}},namespaceSeparator:"::",modeProps:{fold:["brace","include"]}}),y("text/x-java",{name:"clike",keywords:a("abstract assert break case catch class const continue default do else enum extends final finally float for goto if implements import instanceof interface native new package private protected public return static strictfp super switch synchronized this throw throws transient try volatile while @interface"),types:a("byte short int long float double boolean char void Boolean Byte Character Double Float Integer Long Number Object Short String StringBuffer StringBuilder Void"),blockKeywords:a("catch class do else finally for if switch try while"),defKeywords:a("class interface enum @interface"),typeFirstDefinitions:!0,atoms:a("true false null"),number:/^(?:0x[a-f\d_]+|0b[01_]+|(?:[\d_]+\.?\d*|\.\d+)(?:e[-+]?[\d_]+)?)(u|ll?|l|f)?/i,hooks:{"@":function(t){return!t.match("interface",!1)&&(t.eatWhile(/[\w\$_]/),"meta")}},modeProps:{fold:["brace","import"]}}),y("text/x-csharp",{name:"clike",keywords:a("abstract as async await base break case catch checked class const continue default delegate do else enum event explicit extern finally fixed for foreach goto if implicit in interface internal is lock namespace new operator out override params private protected public readonly ref return sealed sizeof stackalloc static struct switch this throw try typeof unchecked unsafe using virtual void volatile while add alias ascending descending dynamic from get global group into join let orderby partial remove select set value var yield"),types:a("Action Boolean Byte Char DateTime DateTimeOffset Decimal Double Func Guid Int16 Int32 Int64 Object SByte Single String Task TimeSpan UInt16 UInt32 UInt64 bool byte char decimal double short int long object sbyte float string ushort uint ulong"),blockKeywords:a("catch class do else finally for foreach if struct switch try while"),defKeywords:a("class interface namespace struct var"),typeFirstDefinitions:!0,atoms:a("true false null"),hooks:{"@":function(t,e){return t.eat('"')?(e.tokenize=C,C(t,e)):(t.eatWhile(/[\w\$_]/),"meta")}}}),y("text/x-scala",{name:"clike",keywords:a("abstract case catch class def do else extends final finally for forSome if implicit import lazy match new null object override package private protected return sealed super this throw trait try type val var while with yield _ assert assume require print println printf readLine readBoolean readByte readShort readChar readInt readLong readFloat readDouble"),types:a("AnyVal App Application Array BufferedIterator BigDecimal BigInt Char Console Either Enumeration Equiv Error Exception Fractional Function IndexedSeq Int Integral Iterable Iterator List Map Numeric Nil NotNull Option Ordered Ordering PartialFunction PartialOrdering Product Proxy Range Responder Seq Serializable Set Specializable Stream StringBuilder StringContext Symbol Throwable Traversable TraversableOnce Tuple Unit Vector Boolean Byte Character CharSequence Class ClassLoader Cloneable Comparable Compiler Double Exception Float Integer Long Math Number Object Package Pair Process Runtime Runnable SecurityManager Short StackTraceElement StrictMath String StringBuffer System Thread ThreadGroup ThreadLocal Throwable Triple Void"),multiLineStrings:!0,blockKeywords:a("catch class enum do else finally for forSome if match switch try while"),defKeywords:a("class enum def object package trait type val var"),atoms:a("true false null"),indentStatements:!1,indentSwitch:!1,isOperatorChar:/[+\-*&%=<>!?|\/#:@]/,hooks:{"@":function(t){return t.eatWhile(/[\w\$_]/),"meta"},'"':function(t,e){return!!t.match('""')&&(e.tokenize=E,e.tokenize(t,e))},"'":function(t){return t.eatWhile(/[\w\$_\xa1-\uffff]/),"atom"},"=":function(t,n){var r=n.context;return!("}"!=r.type||!r.align||!t.eat(">"))&&(n.context=new e(r.indented,r.column,r.type,r.info,null,r.prev),"operator")},"/":function(t,e){return!!t.eat("*")&&(e.tokenize=function t(e){return function(n,r){for(var i;i=n.next();){if("*"==i&&n.eat("/")){if(1==e){r.tokenize=null;break}return r.tokenize=t(e-1),r.tokenize(n,r)}if("/"==i&&n.eat("*"))return r.tokenize=t(e+1),r.tokenize(n,r)}return"comment"}}(1),e.tokenize(t,e))}},modeProps:{closeBrackets:{pairs:'()[]{}""',triples:'"'}}}),y("text/x-kotlin",{name:"clike",keywords:a("package as typealias class interface this super val operator var fun for is in This throw return annotation break continue object if else while do try when !in !is as? file import where by get set abstract enum open inner override private public internal protected catch finally out final vararg reified dynamic companion constructor init sealed field property receiver param sparam lateinit data inline noinline tailrec external annotation crossinline const operator infix suspend actual expect setparam"),types:a("Boolean Byte Character CharSequence Class ClassLoader Cloneable Comparable Compiler Double Exception Float Integer Long Math Number Object Package Pair Process Runtime Runnable SecurityManager Short StackTraceElement StrictMath String StringBuffer System Thread ThreadGroup ThreadLocal Throwable Triple Void Annotation Any BooleanArray ByteArray Char CharArray DeprecationLevel DoubleArray Enum FloatArray Function Int IntArray Lazy LazyThreadSafetyMode LongArray Nothing ShortArray Unit"),intendSwitch:!1,indentStatements:!1,multiLineStrings:!0,number:/^(?:0x[a-f\d_]+|0b[01_]+|(?:[\d_]+(\.\d+)?|\.\d+)(?:e[-+]?[\d_]+)?)(u|ll?|l|f)?/i,blockKeywords:a("catch class do else finally for if where try while enum"),defKeywords:a("class val var object interface fun"),atoms:a("true false null this"),hooks:{"@":function(t){return t.eatWhile(/[\w\$_]/),"meta"},"*":function(t,e){return"."==e.prevToken?"variable":"operator"},'"':function(t,e){var n;return e.tokenize=(n=t.match('""'),function(t,e){for(var r,i=!1,o=!1;!t.eol();){if(!n&&!i&&t.match('"')){o=!0;break}if(n&&t.match('"""')){o=!0;break}r=t.next(),!i&&"$"==r&&t.match("{")&&t.skipTo("}"),i=!i&&"\\"==r&&!n}return!o&&n||(e.tokenize=null),"string"}),e.tokenize(t,e)},indent:function(t,e,n,r){var i=n&&n.charAt(0);return"}"!=t.prevToken&&")"!=t.prevToken||""!=n?"operator"==t.prevToken&&"}"!=n||"variable"==t.prevToken&&"."==i||("}"==t.prevToken||")"==t.prevToken)&&"."==i?2*r+e.indented:e.align&&"}"==e.type?e.indented+(t.context.type==(n||"").charAt(0)?0:r):void 0:t.indented}},modeProps:{closeBrackets:{triples:'"'}}}),y(["x-shader/x-vertex","x-shader/x-fragment"],{name:"clike",keywords:a("sampler1D sampler2D sampler3D samplerCube sampler1DShadow sampler2DShadow const attribute uniform varying break continue discard return for while do if else struct in out inout"),types:a("float int bool void vec2 vec3 vec4 ivec2 ivec3 ivec4 bvec2 bvec3 bvec4 mat2 mat3 mat4"),blockKeywords:a("for while do if else struct"),builtin:a("radians degrees sin cos tan asin acos atan pow exp log exp2 sqrt inversesqrt abs sign floor ceil fract mod min max clamp mix step smoothstep length distance dot cross normalize ftransform faceforward reflect refract matrixCompMult lessThan lessThanEqual greaterThan greaterThanEqual equal notEqual any all not texture1D texture1DProj texture1DLod texture1DProjLod texture2D texture2DProj texture2DLod texture2DProjLod texture3D texture3DProj texture3DLod texture3DProjLod textureCube textureCubeLod shadow1D shadow2D shadow1DProj shadow2DProj shadow1DLod shadow2DLod shadow1DProjLod shadow2DProjLod dFdx dFdy fwidth noise1 noise2 noise3 noise4"),atoms:a("true false gl_FragColor gl_SecondaryColor gl_Normal gl_Vertex gl_MultiTexCoord0 gl_MultiTexCoord1 gl_MultiTexCoord2 gl_MultiTexCoord3 gl_MultiTexCoord4 gl_MultiTexCoord5 gl_MultiTexCoord6 gl_MultiTexCoord7 gl_FogCoord gl_PointCoord gl_Position gl_PointSize gl_ClipVertex gl_FrontColor gl_BackColor gl_FrontSecondaryColor gl_BackSecondaryColor gl_TexCoord gl_FogFragCoord gl_FragCoord gl_FrontFacing gl_FragData gl_FragDepth gl_ModelViewMatrix gl_ProjectionMatrix gl_ModelViewProjectionMatrix gl_TextureMatrix gl_NormalMatrix gl_ModelViewMatrixInverse gl_ProjectionMatrixInverse gl_ModelViewProjectionMatrixInverse gl_TexureMatrixTranspose gl_ModelViewMatrixInverseTranspose gl_ProjectionMatrixInverseTranspose gl_ModelViewProjectionMatrixInverseTranspose gl_TextureMatrixInverseTranspose gl_NormalScale gl_DepthRange gl_ClipPlane gl_Point gl_FrontMaterial gl_BackMaterial gl_LightSource gl_LightModel gl_FrontLightModelProduct gl_BackLightModelProduct gl_TextureColor gl_EyePlaneS gl_EyePlaneT gl_EyePlaneR gl_EyePlaneQ gl_FogParameters gl_MaxLights gl_MaxClipPlanes gl_MaxTextureUnits gl_MaxTextureCoords gl_MaxVertexAttribs gl_MaxVertexUniformComponents gl_MaxVaryingFloats gl_MaxVertexTextureImageUnits gl_MaxTextureImageUnits gl_MaxFragmentUniformComponents gl_MaxCombineTextureImageUnits gl_MaxDrawBuffers"),indentSwitch:!1,hooks:{"#":h},modeProps:{fold:["brace","include"]}}),y("text/x-nesc",{name:"clike",keywords:a(A+" as atomic async call command component components configuration event generic implementation includes interface module new norace nx_struct nx_union post provides signal task uses abstract extends"),types:u,blockKeywords:a(g),atoms:a("null true false"),hooks:{"#":h},modeProps:{fold:["brace","include"]}}),y("text/x-objectivec",{name:"clike",keywords:a(A+" bycopy byref in inout oneway out self super atomic nonatomic retain copy readwrite readonly strong weak assign typeof nullable nonnull null_resettable _cmd @interface @implementation @end @protocol @encode @property @synthesize @dynamic @class @public @package @private @protected @required @optional @try @catch @finally @import @selector @encode @defs @synchronized @autoreleasepool @compatibility_alias @available"),types:function(t){return u(t)||s(c,t)},builtin:a("FOUNDATION_EXPORT FOUNDATION_EXTERN NS_INLINE NS_FORMAT_FUNCTION NS_RETURNS_RETAINED NS_ERROR_ENUM NS_RETURNS_NOT_RETAINED NS_RETURNS_INNER_POINTER NS_DESIGNATED_INITIALIZER NS_ENUM NS_OPTIONS NS_REQUIRES_NIL_TERMINATION NS_ASSUME_NONNULL_BEGIN NS_ASSUME_NONNULL_END NS_SWIFT_NAME NS_REFINED_FOR_SWIFT"),blockKeywords:a(g+" @synthesize @try @catch @finally @autoreleasepool @synchronized"),defKeywords:a("struct enum union @interface @implementation @protocol @class"),dontIndentStatements:/^@.*$/,typeFirstDefinitions:!0,atoms:a("YES NO NULL Nil nil true false nullptr"),isReservedIdentifier:f,hooks:{"#":h,"*":d},modeProps:{fold:["brace","include"]}}),y("text/x-squirrel",{name:"clike",keywords:a("base break clone continue const default delete enum extends function in class foreach local resume return this throw typeof yield constructor instanceof static"),types:u,blockKeywords:a("case catch class else for foreach if switch try while"),defKeywords:a("function local class"),typeFirstDefinitions:!0,atoms:a("true false null"),hooks:{"#":h},modeProps:{fold:["brace","include"]}});var v=null;y("text/x-ceylon",{name:"clike",keywords:a("abstracts alias assembly assert assign break case catch class continue dynamic else exists extends finally for function given if import in interface is let module new nonempty object of out outer package return satisfies super switch then this throw try value void while"),types:function(t){var e=t.charAt(0);return e===e.toUpperCase()&&e!==e.toLowerCase()},blockKeywords:a("case catch class dynamic else finally for function if interface module new object switch try while"),defKeywords:a("class dynamic function interface module object package value"),builtin:a("abstract actual aliased annotation by default deprecated doc final formal late license native optional sealed see serializable shared suppressWarnings tagged throws variable"),isPunctuationChar:/[\[\]{}\(\),;\:\.`]/,isOperatorChar:/[+\-*&%=<>!?|^~:\/]/,numberStart:/[\d#$]/,number:/^(?:#[\da-fA-F_]+|\$[01_]+|[\d_]+[kMGTPmunpf]?|[\d_]+\.[\d_]+(?:[eE][-+]?\d+|[kMGTPmunpf]|)|)/i,multiLineStrings:!0,typeFirstDefinitions:!0,atoms:a("true false null larger smaller equal empty finished"),indentSwitch:!1,styleDefs:!1,hooks:{"@":function(t){return t.eatWhile(/[\w\$_]/),"meta"},'"':function(t,e){return e.tokenize=function t(e){return function(n,r){for(var i,o=!1,a=!1;!n.eol();){if(!o&&n.match('"')&&("single"==e||n.match('""'))){a=!0;break}if(!o&&n.match("``")){v=t(e),a=!0;break}i=n.next(),o="single"==e&&!o&&"\\"==i}return a&&(r.tokenize=null),"string"}}(t.match('""')?"triple":"single"),e.tokenize(t,e)},"`":function(t,e){return!(!v||!t.match("`"))&&(e.tokenize=v,v=null,e.tokenize(t,e))},"'":function(t){return t.eatWhile(/[\w\$_\xa1-\uffff]/),"atom"},token:function(t,e,n){if(("variable"==n||"type"==n)&&"."==e.prevToken)return"variable-2"}},modeProps:{fold:["brace","import"],closeBrackets:{triples:'"'}}})}(n(15))},function(t,e,n){!function(t){"use strict";function e(t){return new RegExp("^(("+t.join(")|(")+"))\\b")}var n,r=e(["and","or","not","is"]),i=["as","assert","break","class","continue","def","del","elif","else","except","finally","for","from","global","if","import","lambda","pass","raise","return","try","while","with","yield","in"],o=["abs","all","any","bin","bool","bytearray","callable","chr","classmethod","compile","complex","delattr","dict","dir","divmod","enumerate","eval","filter","float","format","frozenset","getattr","globals","hasattr","hash","help","hex","id","input","int","isinstance","issubclass","iter","len","list","locals","map","max","memoryview","min","next","object","oct","open","ord","pow","property","range","repr","reversed","round","set","setattr","slice","sorted","staticmethod","str","sum","super","tuple","type","vars","zip","__import__","NotImplemented","Ellipsis","__debug__"];function a(t){return t.scopes[t.scopes.length-1]}t.registerHelper("hintWords","python",i.concat(o)),t.defineMode("python",function(n,s){for(var A="error",l=s.delimiters||s.singleDelimiters||/^[\(\)\[\]\{\}@,:`=;\.\\]/,c=[s.singleOperators,s.doubleOperators,s.doubleDelimiters,s.tripleDelimiters,s.operators||/^([-+*\/%\/&|^]=?|[<>=]+|\/\/=?|\*\*=?|!=|[~!@]|\.\.\.)/],u=0;u<c.length;u++)c[u]||c.splice(u--,1);var g=s.hangingIndent||n.indentUnit,h=i,d=o;null!=s.extra_keywords&&(h=h.concat(s.extra_keywords)),null!=s.extra_builtins&&(d=d.concat(s.extra_builtins));var f=!(s.version&&Number(s.version)<3);if(f){var p=s.identifiers||/^[_A-Za-z\u00A1-\uFFFF][_A-Za-z0-9\u00A1-\uFFFF]*/;h=h.concat(["nonlocal","False","True","None","async","await"]),d=d.concat(["ascii","bytes","exec","print"]);var I=new RegExp("^(([rbuf]|(br)|(fr))?('{3}|\"{3}|['\"]))","i")}else{var p=s.identifiers||/^[_A-Za-z][_A-Za-z0-9]*/;h=h.concat(["exec","print"]),d=d.concat(["apply","basestring","buffer","cmp","coerce","execfile","file","intern","long","raw_input","reduce","reload","unichr","unicode","xrange","False","True","None"]);var I=new RegExp("^(([rubf]|(ur)|(br))?('{3}|\"{3}|['\"]))","i")}var C=e(h),m=e(d);function y(t,e){var n=t.sol()&&"\\"!=e.lastToken;if(n&&(e.indent=t.indentation()),n&&"py"==a(e).type){var r=a(e).offset;if(t.eatSpace()){var i=t.indentation();return i>r?v(e):i<r&&B(t,e)&&"#"!=t.peek()&&(e.errorToken=!0),null}var o=E(t,e);return r>0&&B(t,e)&&(o+=" "+A),o}return E(t,e)}function E(t,e){if(t.eatSpace())return null;if(t.match(/^#.*/))return"comment";if(t.match(/^[0-9\.]/,!1)){var n=!1;if(t.match(/^[\d_]*\.\d+(e[\+\-]?\d+)?/i)&&(n=!0),t.match(/^[\d_]+\.\d*/)&&(n=!0),t.match(/^\.\d+/)&&(n=!0),n)return t.eat(/J/i),"number";var i=!1;if(t.match(/^0x[0-9a-f_]+/i)&&(i=!0),t.match(/^0b[01_]+/i)&&(i=!0),t.match(/^0o[0-7_]+/i)&&(i=!0),t.match(/^[1-9][\d_]*(e[\+\-]?[\d_]+)?/)&&(t.eat(/J/i),i=!0),t.match(/^0(?![\dx])/i)&&(i=!0),i)return t.eat(/L/i),"number"}if(t.match(I)){var o=-1!==t.current().toLowerCase().indexOf("f");return o?(e.tokenize=function(t,e){for(;"rubf".indexOf(t.charAt(0).toLowerCase())>=0;)t=t.substr(1);var n=1==t.length,r="string";function i(t){return function(e,n){var r=E(e,n);return"punctuation"==r&&("{"==e.current()?n.tokenize=i(t+1):"}"==e.current()&&(n.tokenize=t>1?i(t-1):o)),r}}function o(o,a){for(;!o.eol();)if(o.eatWhile(/[^'"\{\}\\]/),o.eat("\\")){if(o.next(),n&&o.eol())return r}else{if(o.match(t))return a.tokenize=e,r;if(o.match("{{"))return r;if(o.match("{",!1))return a.tokenize=i(0),o.current()?r:a.tokenize(o,a);if(o.match("}}"))return r;if(o.match("}"))return A;o.eat(/['"]/)}if(n){if(s.singleLineStringErrors)return A;a.tokenize=e}return r}return o.isString=!0,o}(t.current(),e.tokenize),e.tokenize(t,e)):(e.tokenize=function(t,e){for(;"rubf".indexOf(t.charAt(0).toLowerCase())>=0;)t=t.substr(1);var n=1==t.length,r="string";function i(i,o){for(;!i.eol();)if(i.eatWhile(/[^'"\\]/),i.eat("\\")){if(i.next(),n&&i.eol())return r}else{if(i.match(t))return o.tokenize=e,r;i.eat(/['"]/)}if(n){if(s.singleLineStringErrors)return A;o.tokenize=e}return r}return i.isString=!0,i}(t.current(),e.tokenize),e.tokenize(t,e))}for(var a=0;a<c.length;a++)if(t.match(c[a]))return"operator";return t.match(l)?"punctuation":"."==e.lastToken&&t.match(p)?"property":t.match(C)||t.match(r)?"keyword":t.match(m)?"builtin":t.match(/^(self|cls)\b/)?"variable-2":t.match(p)?"def"==e.lastToken||"class"==e.lastToken?"def":"variable":(t.next(),A)}function v(t){for(;"py"!=a(t).type;)t.scopes.pop();t.scopes.push({offset:a(t).offset+n.indentUnit,type:"py",align:null})}function B(t,e){for(var n=t.indentation();e.scopes.length>1&&a(e).offset>n;){if("py"!=a(e).type)return!0;e.scopes.pop()}return a(e).offset!=n}function b(t,e){t.sol()&&(e.beginningOfLine=!0);var n=e.tokenize(t,e),r=t.current();if(e.beginningOfLine&&"@"==r)return t.match(p,!1)?"meta":f?"operator":A;if(/\S/.test(r)&&(e.beginningOfLine=!1),"variable"!=n&&"builtin"!=n||"meta"!=e.lastToken||(n="meta"),"pass"!=r&&"return"!=r||(e.dedent+=1),"lambda"==r&&(e.lambda=!0),":"!=r||e.lambda||"py"!=a(e).type||v(e),1==r.length&&!/string|comment/.test(n)){var i="[({".indexOf(r);if(-1!=i&&function(t,e,n){var r=t.match(/^([\s\[\{\(]|#.*)*$/,!1)?null:t.column()+1;e.scopes.push({offset:e.indent+g,type:n,align:r})}(t,e,"])}".slice(i,i+1)),-1!=(i="])}".indexOf(r))){if(a(e).type!=r)return A;e.indent=e.scopes.pop().offset-g}}return e.dedent>0&&t.eol()&&"py"==a(e).type&&(e.scopes.length>1&&e.scopes.pop(),e.dedent-=1),n}var Q={startState:function(t){return{tokenize:y,scopes:[{offset:t||0,type:"py",align:null}],indent:t||0,lastToken:null,lambda:!1,dedent:0}},token:function(t,e){var n=e.errorToken;n&&(e.errorToken=!1);var r=b(t,e);return r&&"comment"!=r&&(e.lastToken="keyword"==r||"punctuation"==r?t.current():r),"punctuation"==r&&(r=null),t.eol()&&e.lambda&&(e.lambda=!1),n?r+" "+A:r},indent:function(e,n){if(e.tokenize!=y)return e.tokenize.isString?t.Pass:0;var r=a(e),i=r.type==n.charAt(0);return null!=r.align?r.align-(i?1:0):r.offset-(i?g:0)},electricInput:/^\s*[\}\]\)]$/,closeBrackets:{triples:"'\""},lineComment:"#",fold:"indent"};return Q}),t.defineMIME("text/x-python","python"),t.defineMIME("text/x-cython",{name:"python",extra_keywords:(n="by cdef cimport cpdef ctypedef enum except extern gil include nogil property public readonly struct union DEF IF ELIF ELSE",n.split(" "))})}(n(15))},function(t,e,n){!function(t){"use strict";t.defineMode("stex",function(t,e){function n(t,e){t.cmdState.push(e)}function r(t){return t.cmdState.length>0?t.cmdState[t.cmdState.length-1]:null}function i(t,e,n){return function(){this.name=t,this.bracketNo=0,this.style=e,this.styles=n,this.argument=null,this.styleIdentifier=function(){return this.styles[this.bracketNo-1]||null},this.openBracket=function(){return this.bracketNo++,"bracket"},this.closeBracket=function(){}}}var o={};function a(t,e){t.f=e}function s(t,e){var i;if(t.match(/^\\[a-zA-Z@]+/)){var s=t.current().slice(1);return i=new(i=o[s]||o.DEFAULT),n(e,i),a(e,l),i.style}if(t.match(/^\\[$&%#{}_]/))return"tag";if(t.match(/^\\[,;!\/\\]/))return"tag";if(t.match("\\["))return a(e,function(t,e){return A(t,e,"\\]")}),"keyword";if(t.match("\\("))return a(e,function(t,e){return A(t,e,"\\)")}),"keyword";if(t.match("$$"))return a(e,function(t,e){return A(t,e,"$$")}),"keyword";if(t.match("$"))return a(e,function(t,e){return A(t,e,"$")}),"keyword";var c=t.next();return"%"==c?(t.skipToEnd(),"comment"):"}"==c||"]"==c?(i=r(e))?(i.closeBracket(c),a(e,l),"bracket"):"error":"{"==c||"["==c?(i=new(i=o.DEFAULT),n(e,i),"bracket"):/\d/.test(c)?(t.eatWhile(/[\w.%]/),"atom"):(t.eatWhile(/[\w\-_]/),"begin"==(i=function(t){for(var e=t.cmdState,n=e.length-1;n>=0;n--){var r=e[n];if("DEFAULT"!=r.name)return r}return{styleIdentifier:function(){return null}}}(e)).name&&(i.argument=t.current()),i.styleIdentifier())}function A(t,e,n){if(t.eatSpace())return null;if(n&&t.match(n))return a(e,s),"keyword";if(t.match(/^\\[a-zA-Z@]+/))return"tag";if(t.match(/^[a-zA-Z]+/))return"variable-2";if(t.match(/^\\[$&%#{}_]/))return"tag";if(t.match(/^\\[,;!\/]/))return"tag";if(t.match(/^[\^_&]/))return"tag";if(t.match(/^[+\-<>|=,\/@!*:;'"`~#?]/))return null;if(t.match(/^(\d+\.\d*|\d*\.\d+|\d+)/))return"number";var r=t.next();return"{"==r||"}"==r||"["==r||"]"==r||"("==r||")"==r?"bracket":"%"==r?(t.skipToEnd(),"comment"):"error"}function l(t,e){var n=t.peek();return"{"==n||"["==n?(r(e).openBracket(n),t.eat(n),a(e,s),"bracket"):/[ \t\r]/.test(n)?(t.eat(n),null):(a(e,s),function(t){var e=t.cmdState.pop();e&&e.closeBracket()}(e),s(t,e))}return o.importmodule=i("importmodule","tag",["string","builtin"]),o.documentclass=i("documentclass","tag",["","atom"]),o.usepackage=i("usepackage","tag",["atom"]),o.begin=i("begin","tag",["atom"]),o.end=i("end","tag",["atom"]),o.label=i("label","tag",["atom"]),o.ref=i("ref","tag",["atom"]),o.eqref=i("eqref","tag",["atom"]),o.cite=i("cite","tag",["atom"]),o.bibitem=i("bibitem","tag",["atom"]),o.Bibitem=i("Bibitem","tag",["atom"]),o.RBibitem=i("RBibitem","tag",["atom"]),o.DEFAULT=function(){this.name="DEFAULT",this.style="tag",this.styleIdentifier=this.openBracket=this.closeBracket=function(){}},{startState:function(){var t=e.inMathMode?function(t,e){return A(t,e)}:s;return{cmdState:[],f:t}},copyState:function(t){return{cmdState:t.cmdState.slice(),f:t.f}},token:function(t,e){return e.f(t,e)},blankLine:function(t){t.f=s,t.cmdState.length=0},lineComment:"%"}}),t.defineMIME("text/x-stex","stex"),t.defineMIME("text/x-latex","stex")}(n(15))},function(t,e,n){!function(t){"use strict";var e=/^((?:(?:aaas?|about|acap|adiumxtra|af[ps]|aim|apt|attachment|aw|beshare|bitcoin|bolo|callto|cap|chrome(?:-extension)?|cid|coap|com-eventbrite-attendee|content|crid|cvs|data|dav|dict|dlna-(?:playcontainer|playsingle)|dns|doi|dtn|dvb|ed2k|facetime|feed|file|finger|fish|ftp|geo|gg|git|gizmoproject|go|gopher|gtalk|h323|hcp|https?|iax|icap|icon|im|imap|info|ipn|ipp|irc[6s]?|iris(?:\.beep|\.lwz|\.xpc|\.xpcs)?|itms|jar|javascript|jms|keyparc|lastfm|ldaps?|magnet|mailto|maps|market|message|mid|mms|ms-help|msnim|msrps?|mtqp|mumble|mupdate|mvn|news|nfs|nih?|nntp|notes|oid|opaquelocktoken|palm|paparazzi|platform|pop|pres|proxy|psyc|query|res(?:ource)?|rmi|rsync|rtmp|rtsp|secondlife|service|session|sftp|sgn|shttp|sieve|sips?|skype|sm[bs]|snmp|soap\.beeps?|soldat|spotify|ssh|steam|svn|tag|teamspeak|tel(?:net)?|tftp|things|thismessage|tip|tn3270|tv|udp|unreal|urn|ut2004|vemmi|ventrilo|view-source|webcal|wss?|wtai|wyciwyg|xcon(?:-userid)?|xfire|xmlrpc\.beeps?|xmpp|xri|ymsgr|z39\.50[rs]?):(?:\/{1,3}|[a-z0-9%])|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}\/)(?:[^\s()<>]|\([^\s()<>]*\))+(?:\([^\s()<>]*\)|[^\s`*!()\[\]{};:'".,<>?«»“”‘’]))/i;t.defineMode("gfm",function(n,r){var i=0,o={startState:function(){return{code:!1,codeBlock:!1,ateSpace:!1}},copyState:function(t){return{code:t.code,codeBlock:t.codeBlock,ateSpace:t.ateSpace}},token:function(t,n){if(n.combineTokens=null,n.codeBlock)return t.match(/^```+/)?(n.codeBlock=!1,null):(t.skipToEnd(),null);if(t.sol()&&(n.code=!1),t.sol()&&t.match(/^```+/))return t.skipToEnd(),n.codeBlock=!0,null;if("`"===t.peek()){t.next();var o=t.pos;t.eatWhile("`");var a=1+t.pos-o;return n.code?a===i&&(n.code=!1):(i=a,n.code=!0),null}if(n.code)return t.next(),null;if(t.eatSpace())return n.ateSpace=!0,null;if((t.sol()||n.ateSpace)&&(n.ateSpace=!1,!1!==r.gitHubSpice)){if(t.match(/^(?:[a-zA-Z0-9\-_]+\/)?(?:[a-zA-Z0-9\-_]+@)?(?=.{0,6}\d)(?:[a-f0-9]{7,40}\b)/))return n.combineTokens=!0,"link";if(t.match(/^(?:[a-zA-Z0-9\-_]+\/)?(?:[a-zA-Z0-9\-_]+)?#[0-9]+\b/))return n.combineTokens=!0,"link"}return t.match(e)&&"]("!=t.string.slice(t.start-2,t.start)&&(0==t.start||/\W/.test(t.string.charAt(t.start-1)))?(n.combineTokens=!0,"link"):(t.next(),null)},blankLine:function(t){return t.code=!1,null}},a={taskLists:!0,strikethrough:!0,emoji:!0};for(var s in r)a[s]=r[s];return a.name="markdown",t.overlayMode(t.getMode(n,a),o)},"markdown"),t.defineMIME("text/x-gfm","gfm")}(n(15),n(392),n(732))},function(t,e,n){!function(t){"use strict";t.defineMode("julia",function(e,n){function r(t,e){return void 0===e&&(e="\\b"),new RegExp("^(("+t.join(")|(")+"))"+e)}var i=n.operators||r(["[<>]:","[<>=]=","<<=?",">>>?=?","=>","->","\\/\\/","[\\\\%*+\\-<>!=\\/^|&\\u00F7\\u22BB]=?","\\?","\\$","~",":","\\u00D7","\\u2208","\\u2209","\\u220B","\\u220C","\\u2218","\\u221A","\\u221B","\\u2229","\\u222A","\\u2260","\\u2264","\\u2265","\\u2286","\\u2288","\\u228A","\\u22C5","\\b(in|isa)\\b(?!.?\\()"],""),o=n.delimiters||/^[;,()[\]{}]/,a=n.identifiers||/^[_A-Za-z\u00A1-\u2217\u2219-\uFFFF][\w\u00A1-\u2217\u2219-\uFFFF]*!*/,s=r(["\\\\[0-7]{1,3}","\\\\x[A-Fa-f0-9]{1,2}","\\\\[abefnrtv0%?'\"\\\\]","([^\\u0027\\u005C\\uD800-\\uDFFF]|[\\uD800-\\uDFFF][\\uDC00-\\uDFFF])"],"'"),A=["if","else","elseif","while","for","begin","let","end","do","try","catch","finally","return","break","continue","global","local","const","export","import","importall","using","function","where","macro","module","baremodule","struct","type","mutable","immutable","quote","typealias","abstract","primitive","bitstype"],l=["true","false","nothing","NaN","Inf"];t.registerHelper("hintWords","julia",A.concat(l));var c=r(["begin","function","type","struct","immutable","let","macro","for","while","quote","if","else","elseif","try","finally","catch","do"]),u=r(["end","else","elseif","catch","finally"]),g=r(A),h=r(l),d=/^@[_A-Za-z][\w]*/,f=/^:[_A-Za-z\u00A1-\uFFFF][\w\u00A1-\uFFFF]*!*/,p=/^(`|([_A-Za-z\u00A1-\uFFFF]*"("")?))/;function I(t){return t.nestedArrays>0}function C(t,e){return void 0===e&&(e=0),t.scopes.length<=e?null:t.scopes[t.scopes.length-(e+1)]}function m(t,e){if(t.match(/^#=/,!1))return e.tokenize=v,e.tokenize(t,e);var n=e.leavingExpr;if(t.sol()&&(n=!1),e.leavingExpr=!1,n&&t.match(/^'+/))return"operator";if(t.match(/\.{4,}/))return"error";if(t.match(/\.{1,3}/))return"operator";if(t.eatSpace())return null;var r,s,A=t.peek();if("#"===A)return t.skipToEnd(),"comment";if("["===A&&(e.scopes.push("["),e.nestedArrays++),"("===A&&(e.scopes.push("("),e.nestedGenerators++),I(e)&&"]"===A){for("if"===C(e)&&e.scopes.pop();"for"===C(e);)e.scopes.pop();e.scopes.pop(),e.nestedArrays--,e.leavingExpr=!0}if(function(t){return t.nestedGenerators>0}(e)&&")"===A){for("if"===C(e)&&e.scopes.pop();"for"===C(e);)e.scopes.pop();e.scopes.pop(),e.nestedGenerators--,e.leavingExpr=!0}if(I(e)){if("end"==e.lastToken&&t.match(/^:/))return"operator";if(t.match(/^end/))return"number"}if((r=t.match(c,!1))&&e.scopes.push(r[0]),t.match(u,!1)&&e.scopes.pop(),t.match(/^::(?![:\$])/))return e.tokenize=E,e.tokenize(t,e);if(!n&&t.match(f)||t.match(/:([<>]:|<<=?|>>>?=?|->|\/\/|\.{2,3}|[\.\\%*+\-<>!\/^|&]=?|[~\?\$])/))return"builtin";if(t.match(i))return"operator";if(t.match(/^\.?\d/,!1)){var l=RegExp(/^im\b/),b=!1;if(t.match(/^\d*\.(?!\.)\d*([Eef][\+\-]?\d+)?/i)&&(b=!0),t.match(/^\d+\.(?!\.)\d*/)&&(b=!0),t.match(/^\.\d+/)&&(b=!0),t.match(/^0x\.[0-9a-f]+p[\+\-]?\d+/i)&&(b=!0),t.match(/^0x[0-9a-f]+/i)&&(b=!0),t.match(/^0b[01]+/i)&&(b=!0),t.match(/^0o[0-7]+/i)&&(b=!0),t.match(/^[1-9]\d*(e[\+\-]?\d+)?/)&&(b=!0),t.match(/^0(?![\dx])/i)&&(b=!0),b)return t.match(l),e.leavingExpr=!0,"number"}if(t.match(/^'/))return e.tokenize=B,e.tokenize(t,e);if(t.match(p))return e.tokenize=('"""'===(s=t.current()).substr(-3)?s='"""':'"'===s.substr(-1)&&(s='"'),function(t,e){if(t.eat("\\"))t.next();else{if(t.match(s))return e.tokenize=m,e.leavingExpr=!0,"string";t.eat(/[`"]/)}return t.eatWhile(/[^\\`"]/),"string"}),e.tokenize(t,e);if(t.match(d))return"meta";if(t.match(o))return null;if(t.match(g))return"keyword";if(t.match(h))return"builtin";var Q=e.isDefinition||"function"==e.lastToken||"macro"==e.lastToken||"type"==e.lastToken||"struct"==e.lastToken||"immutable"==e.lastToken;return t.match(a)?Q?"."===t.peek()?(e.isDefinition=!0,"variable"):(e.isDefinition=!1,"def"):t.match(/^({[^}]*})*\(/,!1)?(e.tokenize=y,e.tokenize(t,e)):(e.leavingExpr=!0,"variable"):(t.next(),"error")}function y(t,e){var n=t.match(/^(\(\s*)/);if(n&&(e.firstParenPos<0&&(e.firstParenPos=e.scopes.length),e.scopes.push("("),e.charsAdvanced+=n[1].length),"("==C(e)&&t.match(/^\)/)&&(e.scopes.pop(),e.charsAdvanced+=1,e.scopes.length<=e.firstParenPos)){var r=t.match(/^(\s*where\s+[^\s=]+)*\s*?=(?!=)/,!1);return t.backUp(e.charsAdvanced),e.firstParenPos=-1,e.charsAdvanced=0,e.tokenize=m,r?"def":"builtin"}if(t.match(/^$/g,!1)){for(t.backUp(e.charsAdvanced);e.scopes.length>e.firstParenPos;)e.scopes.pop();return e.firstParenPos=-1,e.charsAdvanced=0,e.tokenize=m,"builtin"}return e.charsAdvanced+=t.match(/^([^()]*)/)[1].length,e.tokenize(t,e)}function E(t,e){return t.match(/.*?(?=,|;|{|}|\(|\)|=|$|\s)/),t.match(/^{/)?e.nestedParameters++:t.match(/^}/)&&e.nestedParameters>0&&e.nestedParameters--,e.nestedParameters>0?t.match(/.*?(?={|})/)||t.next():0==e.nestedParameters&&(e.tokenize=m),"builtin"}function v(t,e){return t.match(/^#=/)&&e.nestedComments++,t.match(/.*?(?=(#=|=#))/)||t.skipToEnd(),t.match(/^=#/)&&(e.nestedComments--,0==e.nestedComments&&(e.tokenize=m)),"comment"}function B(t,e){var n,r=!1;if(t.match(s))r=!0;else if(n=t.match(/\\u([a-f0-9]{1,4})(?=')/i)){var i=parseInt(n[1],16);(i<=55295||i>=57344)&&(r=!0,t.next())}else if(n=t.match(/\\U([A-Fa-f0-9]{5,8})(?=')/)){var i=parseInt(n[1],16);i<=1114111&&(r=!0,t.next())}return r?(e.leavingExpr=!0,e.tokenize=m,"string"):(t.match(/^[^']+(?=')/)||t.skipToEnd(),t.match(/^'/)&&(e.tokenize=m),"error")}var b={startState:function(){return{tokenize:m,scopes:[],lastToken:null,leavingExpr:!1,isDefinition:!1,nestedArrays:0,nestedComments:0,nestedGenerators:0,nestedParameters:0,charsAdvanced:0,firstParenPos:-1}},token:function(t,e){var n=e.tokenize(t,e),r=t.current();return r&&n&&(e.lastToken=r),n},indent:function(t,n){var r=0;return"]"!==n&&")"!==n&&"end"!==n&&"else"!==n&&"catch"!==n&&"elseif"!==n&&"finally"!==n||(r=-1),(t.scopes.length+r)*e.indentUnit},electricInput:/\b(end|else|catch|finally)\b/,blockCommentStart:"#=",blockCommentEnd:"=#",lineComment:"#",closeBrackets:'()[]{}""',fold:"indent"};return b}),t.defineMIME("text/x-julia","julia")}(n(15))},function(t,e,n){!function(t){"use strict";t.registerHelper("wordChars","r",/[\w.]/),t.defineMode("r",function(e){function n(t){for(var e={},n=0;n<t.length;++n)e[t[n]]=!0;return e}var r=["NULL","NA","Inf","NaN","NA_integer_","NA_real_","NA_complex_","NA_character_","TRUE","FALSE"],i=["list","quote","bquote","eval","return","call","parse","deparse"],o=["if","else","repeat","while","function","for","in","next","break"];t.registerHelper("hintWords","r",r.concat(i,o));var a,s=n(r),A=n(i),l=n(o),c=n(["if","else","repeat","while","function","for"]),u=/[+\-*\/^<>=!&|~$:]/;function g(t,e){a=null;var n,r=t.next();if("#"==r)return t.skipToEnd(),"comment";if("0"==r&&t.eat("x"))return t.eatWhile(/[\da-f]/i),"number";if("."==r&&t.eat(/\d/))return t.match(/\d*(?:e[+\-]?\d+)?/),"number";if(/\d/.test(r))return t.match(/\d*(?:\.\d+)?(?:e[+\-]\d+)?L?/),"number";if("'"==r||'"'==r)return e.tokenize=(n=r,function(t,e){if(t.eat("\\")){var r=t.next();return"x"==r?t.match(/^[a-f0-9]{2}/i):("u"==r||"U"==r)&&t.eat("{")&&t.skipTo("}")?t.next():"u"==r?t.match(/^[a-f0-9]{4}/i):"U"==r?t.match(/^[a-f0-9]{8}/i):/[0-7]/.test(r)&&t.match(/^[0-7]{1,2}/),"string-2"}for(var i;null!=(i=t.next());){if(i==n){e.tokenize=g;break}if("\\"==i){t.backUp(1);break}}return"string"}),"string";if("`"==r)return t.match(/[^`]+`/),"variable-3";if("."==r&&t.match(/.[.\d]+/))return"keyword";if(/[\w\.]/.test(r)&&"_"!=r){t.eatWhile(/[\w\.]/);var i=t.current();return s.propertyIsEnumerable(i)?"atom":l.propertyIsEnumerable(i)?(c.propertyIsEnumerable(i)&&!t.match(/\s*if(\s+|$)/,!1)&&(a="block"),"keyword"):A.propertyIsEnumerable(i)?"builtin":"variable"}return"%"==r?(t.skipTo("%")&&t.next(),"operator variable-2"):"<"==r&&t.eat("-")||"<"==r&&t.match("<-")||"-"==r&&t.match(/>>?/)?"operator arrow":"="==r&&e.ctx.argList?"arg-is":u.test(r)?"$"==r?"operator dollar":(t.eatWhile(u),"operator"):/[\(\){}\[\];]/.test(r)?(a=r,";"==r?"semi":null):null}function h(t,e,n){t.ctx={type:e,indent:t.indent,flags:0,column:n.column(),prev:t.ctx}}function d(t,e){var n=t.ctx;t.ctx={type:n.type,indent:n.indent,flags:n.flags|e,column:n.column,prev:n.prev}}function f(t){t.indent=t.ctx.indent,t.ctx=t.ctx.prev}return{startState:function(){return{tokenize:g,ctx:{type:"top",indent:-e.indentUnit,flags:2},indent:0,afterIdent:!1}},token:function(t,e){if(t.sol()&&(0==(3&e.ctx.flags)&&(e.ctx.flags|=2),4&e.ctx.flags&&f(e),e.indent=t.indentation()),t.eatSpace())return null;var n=e.tokenize(t,e);return"comment"!=n&&0==(2&e.ctx.flags)&&d(e,1),";"!=a&&"{"!=a&&"}"!=a||"block"!=e.ctx.type||f(e),"{"==a?h(e,"}",t):"("==a?(h(e,")",t),e.afterIdent&&(e.ctx.argList=!0)):"["==a?h(e,"]",t):"block"==a?h(e,"block",t):a==e.ctx.type?f(e):"block"==e.ctx.type&&"comment"!=n&&d(e,4),e.afterIdent="variable"==n||"keyword"==n,n},indent:function(t,n){if(t.tokenize!=g)return 0;var r=n&&n.charAt(0),i=t.ctx,o=r==i.type;return 4&i.flags&&(i=i.prev),"block"==i.type?i.indent+("{"==r?0:e.indentUnit):1&i.flags?i.column+(o?0:1):i.indent+(o?0:e.indentUnit)},lineComment:"#"}}),t.defineMIME("text/x-rsrc","r")}(n(15))},function(t,e,n){!function(t){"use strict";t.defineMode("shell",function(){var e={};function n(t,n){for(var r=0;r<n.length;r++)e[n[r]]=t}var r=["true","false"],i=["if","then","do","else","elif","while","until","for","in","esac","fi","fin","fil","done","exit","set","unset","export","function"],o=["ab","awk","bash","beep","cat","cc","cd","chown","chmod","chroot","clear","cp","curl","cut","diff","echo","find","gawk","gcc","get","git","grep","hg","kill","killall","ln","ls","make","mkdir","openssl","mv","nc","nl","node","npm","ping","ps","restart","rm","rmdir","sed","service","sh","shopt","shred","source","sort","sleep","ssh","start","stop","su","sudo","svn","tee","telnet","top","touch","vi","vim","wall","wc","wget","who","write","yes","zsh"];function a(t,e){var n="("==t?")":"{"==t?"}":t;return function(r,i){for(var o,c=!1;null!=(o=r.next());){if(o===n&&!c){i.tokens.shift();break}if("$"===o&&!c&&"'"!==t&&r.peek()!=n){c=!0,r.backUp(1),i.tokens.unshift(A);break}if(!c&&t!==n&&o===t)return i.tokens.unshift(a(t,e)),l(r,i);if(!c&&/['"]/.test(o)&&!/['"]/.test(t)){i.tokens.unshift(s(o,"string")),r.backUp(1);break}c=!c&&"\\"===o}return e}}function s(t,e){return function(n,r){return r.tokens[0]=a(t,e),n.next(),l(n,r)}}t.registerHelper("hintWords","shell",r.concat(i,o)),n("atom",r),n("keyword",i),n("builtin",o);var A=function(t,e){e.tokens.length>1&&t.eat("$");var n=t.next();return/['"({]/.test(n)?(e.tokens[0]=a(n,"("==n?"quote":"{"==n?"def":"string"),l(t,e)):(/\d/.test(n)||t.eatWhile(/\w/),e.tokens.shift(),"def")};function l(t,n){return(n.tokens[0]||function(t,n){if(t.eatSpace())return null;var r=t.sol(),i=t.next();if("\\"===i)return t.next(),null;if("'"===i||'"'===i||"`"===i)return n.tokens.unshift(a(i,"`"===i?"quote":"string")),l(t,n);if("#"===i)return r&&t.eat("!")?(t.skipToEnd(),"meta"):(t.skipToEnd(),"comment");if("$"===i)return n.tokens.unshift(A),l(t,n);if("+"===i||"="===i)return"operator";if("-"===i)return t.eat("-"),t.eatWhile(/\w/),"attribute";if(/\d/.test(i)&&(t.eatWhile(/\d/),t.eol()||!/\w/.test(t.peek())))return"number";t.eatWhile(/[\w-]/);var o=t.current();return"="===t.peek()&&/\w+/.test(o)?"def":e.hasOwnProperty(o)?e[o]:null})(t,n)}return{startState:function(){return{tokens:[]}},token:function(t,e){return l(t,e)},closeBrackets:"()[]{}''\"\"``",lineComment:"#",fold:"brace"}}),t.defineMIME("text/x-sh","shell"),t.defineMIME("application/x-sh","shell")}(n(15))},function(t,e,n){!function(t){"use strict";function e(t){for(var e;null!=(e=t.next());)if("`"==e&&!t.eat("`"))return"variable-2";return t.backUp(t.current().length-1),t.eatWhile(/\w/)?"variable-2":null}function n(t){return t.eat("@")&&(t.match(/^session\./),t.match(/^local\./),t.match(/^global\./)),t.eat("'")?(t.match(/^.*'/),"variable-2"):t.eat('"')?(t.match(/^.*"/),"variable-2"):t.eat("`")?(t.match(/^.*`/),"variable-2"):t.match(/^[0-9a-zA-Z$\.\_]+/)?"variable-2":null}function r(t){return t.eat("N")?"atom":t.match(/^[a-zA-Z.#!?]/)?"variable-2":null}t.defineMode("sql",function(e,n){var r=n.client||{},s=n.atoms||{false:!0,true:!0,null:!0},A=n.builtin||o(a),l=n.keywords||o(i),c=n.operatorChars||/^[*+\-%<>!=&|~^\/]/,u=n.support||{},g=n.hooks||{},h=n.dateSQL||{date:!0,time:!0,timestamp:!0},d=!1!==n.backslashStringEscapes,f=n.brackets||/^[\{}\(\)\[\]]/,p=n.punctuation||/^[;.,:]/;function I(t,e){var n,i=t.next();if(g[i]){var o=g[i](t,e);if(!1!==o)return o}if(u.hexNumber&&("0"==i&&t.match(/^[xX][0-9a-fA-F]+/)||("x"==i||"X"==i)&&t.match(/^'[0-9a-fA-F]+'/)))return"number";if(u.binaryNumber&&(("b"==i||"B"==i)&&t.match(/^'[01]+'/)||"0"==i&&t.match(/^b[01]+/)))return"number";if(i.charCodeAt(0)>47&&i.charCodeAt(0)<58)return t.match(/^[0-9]*(\.[0-9]+)?([eE][-+]?[0-9]+)?/),u.decimallessFloat&&t.match(/^\.(?!\.)/),"number";if("?"==i&&(t.eatSpace()||t.eol()||t.eat(";")))return"variable-3";if("'"==i||'"'==i&&u.doubleQuote)return e.tokenize=(n=i,function(t,e){for(var r,i=!1;null!=(r=t.next());){if(r==n&&!i){e.tokenize=I;break}i=d&&!i&&"\\"==r}return"string"}),e.tokenize(t,e);if((u.nCharCast&&("n"==i||"N"==i)||u.charsetCast&&"_"==i&&t.match(/[a-z][a-z0-9]*/i))&&("'"==t.peek()||'"'==t.peek()))return"keyword";if(u.commentSlashSlash&&"/"==i&&t.eat("/"))return t.skipToEnd(),"comment";if(u.commentHash&&"#"==i||"-"==i&&t.eat("-")&&(!u.commentSpaceRequired||t.eat(" ")))return t.skipToEnd(),"comment";if("/"==i&&t.eat("*"))return e.tokenize=function t(e){return function(n,r){var i=n.match(/^.*?(\/\*|\*\/)/);return i?"/*"==i[1]?r.tokenize=t(e+1):r.tokenize=e>1?t(e-1):I:n.skipToEnd(),"comment"}}(1),e.tokenize(t,e);if("."!=i){if(c.test(i))return t.eatWhile(c),"operator";if(f.test(i))return"bracket";if(p.test(i))return t.eatWhile(p),"punctuation";if("{"==i&&(t.match(/^( )*(d|D|t|T|ts|TS)( )*'[^']*'( )*}/)||t.match(/^( )*(d|D|t|T|ts|TS)( )*"[^"]*"( )*}/)))return"number";t.eatWhile(/^[_\w\d]/);var a=t.current().toLowerCase();return h.hasOwnProperty(a)&&(t.match(/^( )+'[^']*'/)||t.match(/^( )+"[^"]*"/))?"number":s.hasOwnProperty(a)?"atom":A.hasOwnProperty(a)?"builtin":l.hasOwnProperty(a)?"keyword":r.hasOwnProperty(a)?"string-2":null}return u.zerolessFloat&&t.match(/^(?:\d+(?:e[+-]?\d+)?)/i)?"number":t.match(/^\.+/)?null:u.ODBCdotTable&&t.match(/^[\w\d_]+/)?"variable-2":void 0}function C(t,e,n){e.context={prev:e.context,indent:t.indentation(),col:t.column(),type:n}}return{startState:function(){return{tokenize:I,context:null}},token:function(t,e){if(t.sol()&&e.context&&null==e.context.align&&(e.context.align=!1),e.tokenize==I&&t.eatSpace())return null;var n=e.tokenize(t,e);if("comment"==n)return n;e.context&&null==e.context.align&&(e.context.align=!0);var r=t.current();return"("==r?C(t,e,")"):"["==r?C(t,e,"]"):e.context&&e.context.type==r&&function(t){t.indent=t.context.indent,t.context=t.context.prev}(e),n},indent:function(n,r){var i=n.context;if(!i)return t.Pass;var o=r.charAt(0)==i.type;return i.align?i.col+(o?0:1):i.indent+(o?0:e.indentUnit)},blockCommentStart:"/*",blockCommentEnd:"*/",lineComment:u.commentSlashSlash?"//":u.commentHash?"#":"--",closeBrackets:"()[]{}''\"\"``"}});var i="alter and as asc between by count create delete desc distinct drop from group having in insert into is join like not on or order select set table union update values where limit ";function o(t){for(var e={},n=t.split(" "),r=0;r<n.length;++r)e[n[r]]=!0;return e}var a="bool boolean bit blob enum long longblob longtext medium mediumblob mediumint mediumtext time timestamp tinyblob tinyint tinytext text bigint int int1 int2 int3 int4 int8 integer float float4 float8 double char varbinary varchar varcharacter precision real date datetime year unsigned signed decimal numeric";t.defineMIME("text/x-sql",{name:"sql",keywords:o(i+"begin"),builtin:o(a),atoms:o("false true null unknown"),dateSQL:o("date time timestamp"),support:o("ODBCdotTable doubleQuote binaryNumber hexNumber")}),t.defineMIME("text/x-mssql",{name:"sql",client:o("$partition binary_checksum checksum connectionproperty context_info current_request_id error_line error_message error_number error_procedure error_severity error_state formatmessage get_filestream_transaction_context getansinull host_id host_name isnull isnumeric min_active_rowversion newid newsequentialid rowcount_big xact_state object_id"),keywords:o(i+"begin trigger proc view index for add constraint key primary foreign collate clustered nonclustered declare exec go if use index holdlock nolock nowait paglock readcommitted readcommittedlock readpast readuncommitted repeatableread rowlock serializable snapshot tablock tablockx updlock with"),builtin:o("bigint numeric bit smallint decimal smallmoney int tinyint money float real char varchar text nchar nvarchar ntext binary varbinary image cursor timestamp hierarchyid uniqueidentifier sql_variant xml table "),atoms:o("is not null like and or in left right between inner outer join all any some cross unpivot pivot exists"),operatorChars:/^[*+\-%<>!=^\&|\/]/,brackets:/^[\{}\(\)]/,punctuation:/^[;.,:\/]/,backslashStringEscapes:!1,dateSQL:o("date datetimeoffset datetime2 smalldatetime datetime time"),hooks:{"@":n}}),t.defineMIME("text/x-mysql",{name:"sql",client:o("charset clear connect edit ego exit go help nopager notee nowarning pager print prompt quit rehash source status system tee"),keywords:o(i+"accessible action add after algorithm all analyze asensitive at authors auto_increment autocommit avg avg_row_length before binary binlog both btree cache call cascade cascaded case catalog_name chain change changed character check checkpoint checksum class_origin client_statistics close coalesce code collate collation collations column columns comment commit committed completion concurrent condition connection consistent constraint contains continue contributors convert cross current current_date current_time current_timestamp current_user cursor data database databases day_hour day_microsecond day_minute day_second deallocate dec declare default delay_key_write delayed delimiter des_key_file describe deterministic dev_pop dev_samp deviance diagnostics directory disable discard distinctrow div dual dumpfile each elseif enable enclosed end ends engine engines enum errors escape escaped even event events every execute exists exit explain extended fast fetch field fields first flush for force foreign found_rows full fulltext function general get global grant grants group group_concat handler hash help high_priority hosts hour_microsecond hour_minute hour_second if ignore ignore_server_ids import index index_statistics infile inner innodb inout insensitive insert_method install interval invoker isolation iterate key keys kill language last leading leave left level limit linear lines list load local localtime localtimestamp lock logs low_priority master master_heartbeat_period master_ssl_verify_server_cert masters match max max_rows maxvalue message_text middleint migrate min min_rows minute_microsecond minute_second mod mode modifies modify mutex mysql_errno natural next no no_write_to_binlog offline offset one online open optimize option optionally out outer outfile pack_keys parser partition partitions password phase plugin plugins prepare preserve prev primary privileges procedure processlist profile profiles purge query quick range read read_write reads real rebuild recover references regexp relaylog release remove rename reorganize repair repeatable replace require resignal restrict resume return returns revoke right rlike rollback rollup row row_format rtree savepoint schedule schema schema_name schemas second_microsecond security sensitive separator serializable server session share show signal slave slow smallint snapshot soname spatial specific sql sql_big_result sql_buffer_result sql_cache sql_calc_found_rows sql_no_cache sql_small_result sqlexception sqlstate sqlwarning ssl start starting starts status std stddev stddev_pop stddev_samp storage straight_join subclass_origin sum suspend table_name table_statistics tables tablespace temporary terminated to trailing transaction trigger triggers truncate uncommitted undo uninstall unique unlock upgrade usage use use_frm user user_resources user_statistics using utc_date utc_time utc_timestamp value variables varying view views warnings when while with work write xa xor year_month zerofill begin do then else loop repeat"),builtin:o("bool boolean bit blob decimal double float long longblob longtext medium mediumblob mediumint mediumtext time timestamp tinyblob tinyint tinytext text bigint int int1 int2 int3 int4 int8 integer float float4 float8 double char varbinary varchar varcharacter precision date datetime year unsigned signed numeric"),atoms:o("false true null unknown"),operatorChars:/^[*+\-%<>!=&|^]/,dateSQL:o("date time timestamp"),support:o("ODBCdotTable decimallessFloat zerolessFloat binaryNumber hexNumber doubleQuote nCharCast charsetCast commentHash commentSpaceRequired"),hooks:{"@":n,"`":e,"\\":r}}),t.defineMIME("text/x-mariadb",{name:"sql",client:o("charset clear connect edit ego exit go help nopager notee nowarning pager print prompt quit rehash source status system tee"),keywords:o(i+"accessible action add after algorithm all always analyze asensitive at authors auto_increment autocommit avg avg_row_length before binary binlog both btree cache call cascade cascaded case catalog_name chain change changed character check checkpoint checksum class_origin client_statistics close coalesce code collate collation collations column columns comment commit committed completion concurrent condition connection consistent constraint contains continue contributors convert cross current current_date current_time current_timestamp current_user cursor data database databases day_hour day_microsecond day_minute day_second deallocate dec declare default delay_key_write delayed delimiter des_key_file describe deterministic dev_pop dev_samp deviance diagnostics directory disable discard distinctrow div dual dumpfile each elseif enable enclosed end ends engine engines enum errors escape escaped even event events every execute exists exit explain extended fast fetch field fields first flush for force foreign found_rows full fulltext function general generated get global grant grants group groupby_concat handler hard hash help high_priority hosts hour_microsecond hour_minute hour_second if ignore ignore_server_ids import index index_statistics infile inner innodb inout insensitive insert_method install interval invoker isolation iterate key keys kill language last leading leave left level limit linear lines list load local localtime localtimestamp lock logs low_priority master master_heartbeat_period master_ssl_verify_server_cert masters match max max_rows maxvalue message_text middleint migrate min min_rows minute_microsecond minute_second mod mode modifies modify mutex mysql_errno natural next no no_write_to_binlog offline offset one online open optimize option optionally out outer outfile pack_keys parser partition partitions password persistent phase plugin plugins prepare preserve prev primary privileges procedure processlist profile profiles purge query quick range read read_write reads real rebuild recover references regexp relaylog release remove rename reorganize repair repeatable replace require resignal restrict resume return returns revoke right rlike rollback rollup row row_format rtree savepoint schedule schema schema_name schemas second_microsecond security sensitive separator serializable server session share show shutdown signal slave slow smallint snapshot soft soname spatial specific sql sql_big_result sql_buffer_result sql_cache sql_calc_found_rows sql_no_cache sql_small_result sqlexception sqlstate sqlwarning ssl start starting starts status std stddev stddev_pop stddev_samp storage straight_join subclass_origin sum suspend table_name table_statistics tables tablespace temporary terminated to trailing transaction trigger triggers truncate uncommitted undo uninstall unique unlock upgrade usage use use_frm user user_resources user_statistics using utc_date utc_time utc_timestamp value variables varying view views virtual warnings when while with work write xa xor year_month zerofill begin do then else loop repeat"),builtin:o("bool boolean bit blob decimal double float long longblob longtext medium mediumblob mediumint mediumtext time timestamp tinyblob tinyint tinytext text bigint int int1 int2 int3 int4 int8 integer float float4 float8 double char varbinary varchar varcharacter precision date datetime year unsigned signed numeric"),atoms:o("false true null unknown"),operatorChars:/^[*+\-%<>!=&|^]/,dateSQL:o("date time timestamp"),support:o("ODBCdotTable decimallessFloat zerolessFloat binaryNumber hexNumber doubleQuote nCharCast charsetCast commentHash commentSpaceRequired"),hooks:{"@":n,"`":e,"\\":r}}),t.defineMIME("text/x-sqlite",{name:"sql",client:o("auth backup bail binary changes check clone databases dbinfo dump echo eqp exit explain fullschema headers help import imposter indexes iotrace limit lint load log mode nullvalue once open output print prompt quit read restore save scanstats schema separator session shell show stats system tables testcase timeout timer trace vfsinfo vfslist vfsname width"),keywords:o(i+"abort action add after all analyze attach autoincrement before begin cascade case cast check collate column commit conflict constraint cross current_date current_time current_timestamp database default deferrable deferred detach each else end escape except exclusive exists explain fail for foreign full glob if ignore immediate index indexed initially inner instead intersect isnull key left limit match natural no notnull null of offset outer plan pragma primary query raise recursive references regexp reindex release rename replace restrict right rollback row savepoint temp temporary then to transaction trigger unique using vacuum view virtual when with without"),builtin:o("bool boolean bit blob decimal double float long longblob longtext medium mediumblob mediumint mediumtext time timestamp tinyblob tinyint tinytext text clob bigint int int2 int8 integer float double char varchar date datetime year unsigned signed numeric real"),atoms:o("null current_date current_time current_timestamp"),operatorChars:/^[*+\-%<>!=&|\/~]/,dateSQL:o("date time timestamp datetime"),support:o("decimallessFloat zerolessFloat"),identifierQuote:'"',hooks:{"@":n,":":n,"?":n,$:n,'"':function(t){for(var e;null!=(e=t.next());)if('"'==e&&!t.eat('"'))return"variable-2";return t.backUp(t.current().length-1),t.eatWhile(/\w/)?"variable-2":null},"`":e}}),t.defineMIME("text/x-cassandra",{name:"sql",client:{},keywords:o("add all allow alter and any apply as asc authorize batch begin by clustering columnfamily compact consistency count create custom delete desc distinct drop each_quorum exists filtering from grant if in index insert into key keyspace keyspaces level limit local_one local_quorum modify nan norecursive nosuperuser not of on one order password permission permissions primary quorum rename revoke schema select set storage superuser table three to token truncate ttl two type unlogged update use user users using values where with writetime"),builtin:o("ascii bigint blob boolean counter decimal double float frozen inet int list map static text timestamp timeuuid tuple uuid varchar varint"),atoms:o("false true infinity NaN"),operatorChars:/^[<>=]/,dateSQL:{},support:o("commentSlashSlash decimallessFloat"),hooks:{}}),t.defineMIME("text/x-plsql",{name:"sql",client:o("appinfo arraysize autocommit autoprint autorecovery autotrace blockterminator break btitle cmdsep colsep compatibility compute concat copycommit copytypecheck define describe echo editfile embedded escape exec execute feedback flagger flush heading headsep instance linesize lno loboffset logsource long longchunksize markup native newpage numformat numwidth pagesize pause pno recsep recsepchar release repfooter repheader serveroutput shiftinout show showmode size spool sqlblanklines sqlcase sqlcode sqlcontinue sqlnumber sqlpluscompatibility sqlprefix sqlprompt sqlterminator suffix tab term termout time timing trimout trimspool ttitle underline verify version wrap"),keywords:o("abort accept access add all alter and any array arraylen as asc assert assign at attributes audit authorization avg base_table begin between binary_integer body boolean by case cast char char_base check close cluster clusters colauth column comment commit compress connect connected constant constraint crash create current currval cursor data_base database date dba deallocate debugoff debugon decimal declare default definition delay delete desc digits dispose distinct do drop else elseif elsif enable end entry escape exception exception_init exchange exclusive exists exit external fast fetch file for force form from function generic goto grant group having identified if immediate in increment index indexes indicator initial initrans insert interface intersect into is key level library like limited local lock log logging long loop master maxextents maxtrans member minextents minus mislabel mode modify multiset new next no noaudit nocompress nologging noparallel not nowait number_base object of off offline on online only open option or order out package parallel partition pctfree pctincrease pctused pls_integer positive positiven pragma primary prior private privileges procedure public raise range raw read rebuild record ref references refresh release rename replace resource restrict return returning returns reverse revoke rollback row rowid rowlabel rownum rows run savepoint schema segment select separate session set share snapshot some space split sql start statement storage subtype successful synonym tabauth table tables tablespace task terminate then to trigger truncate type union unique unlimited unrecoverable unusable update use using validate value values variable view views when whenever where while with work"),builtin:o("abs acos add_months ascii asin atan atan2 average bfile bfilename bigserial bit blob ceil character chartorowid chr clob concat convert cos cosh count dec decode deref dual dump dup_val_on_index empty error exp false float floor found glb greatest hextoraw initcap instr instrb int integer isopen last_day least length lengthb ln lower lpad ltrim lub make_ref max min mlslabel mod months_between natural naturaln nchar nclob new_time next_day nextval nls_charset_decl_len nls_charset_id nls_charset_name nls_initcap nls_lower nls_sort nls_upper nlssort no_data_found notfound null number numeric nvarchar2 nvl others power rawtohex real reftohex round rowcount rowidtochar rowtype rpad rtrim serial sign signtype sin sinh smallint soundex sqlcode sqlerrm sqrt stddev string substr substrb sum sysdate tan tanh to_char text to_date to_label to_multi_byte to_number to_single_byte translate true trunc uid unlogged upper user userenv varchar varchar2 variance varying vsize xml"),operatorChars:/^[*\/+\-%<>!=~]/,dateSQL:o("date time timestamp"),support:o("doubleQuote nCharCast zerolessFloat binaryNumber hexNumber")}),t.defineMIME("text/x-hive",{name:"sql",keywords:o("select alter $elem$ $key$ $value$ add after all analyze and archive as asc before between binary both bucket buckets by cascade case cast change cluster clustered clusterstatus collection column columns comment compute concatenate continue create cross cursor data database databases dbproperties deferred delete delimited desc describe directory disable distinct distribute drop else enable end escaped exclusive exists explain export extended external fetch fields fileformat first format formatted from full function functions grant group having hold_ddltime idxproperties if import in index indexes inpath inputdriver inputformat insert intersect into is items join keys lateral left like limit lines load local location lock locks mapjoin materialized minus msck no_drop nocompress not of offline on option or order out outer outputdriver outputformat overwrite partition partitioned partitions percent plus preserve procedure purge range rcfile read readonly reads rebuild recordreader recordwriter recover reduce regexp rename repair replace restrict revoke right rlike row schema schemas semi sequencefile serde serdeproperties set shared show show_database sort sorted ssl statistics stored streamtable table tables tablesample tblproperties temporary terminated textfile then tmp to touch transform trigger unarchive undo union uniquejoin unlock update use using utc utc_tmestamp view when where while with admin authorization char compact compactions conf cube current current_date current_timestamp day decimal defined dependency directories elem_type exchange file following for grouping hour ignore inner interval jar less logical macro minute month more none noscan over owner partialscan preceding pretty principals protection reload rewrite role roles rollup rows second server sets skewed transactions truncate unbounded unset uri user values window year"),builtin:o("bool boolean long timestamp tinyint smallint bigint int float double date datetime unsigned string array struct map uniontype key_type utctimestamp value_type varchar"),atoms:o("false true null unknown"),operatorChars:/^[*+\-%<>!=]/,dateSQL:o("date timestamp"),support:o("ODBCdotTable doubleQuote binaryNumber hexNumber")}),t.defineMIME("text/x-pgsql",{name:"sql",client:o("source"),keywords:o(i+"a abort abs absent absolute access according action ada add admin after aggregate alias all allocate also alter always analyse analyze and any are array array_agg array_max_cardinality as asc asensitive assert assertion assignment asymmetric at atomic attach attribute attributes authorization avg backward base64 before begin begin_frame begin_partition bernoulli between bigint binary bit bit_length blob blocked bom boolean both breadth by c cache call called cardinality cascade cascaded case cast catalog catalog_name ceil ceiling chain char char_length character character_length character_set_catalog character_set_name character_set_schema characteristics characters check checkpoint class class_origin clob close cluster coalesce cobol collate collation collation_catalog collation_name collation_schema collect column column_name columns command_function command_function_code comment comments commit committed concurrently condition condition_number configuration conflict connect connection connection_name constant constraint constraint_catalog constraint_name constraint_schema constraints constructor contains content continue control conversion convert copy corr corresponding cost count covar_pop covar_samp create cross csv cube cume_dist current current_catalog current_date current_default_transform_group current_path current_role current_row current_schema current_time current_timestamp current_transform_group_for_type current_user cursor cursor_name cycle data database datalink datatype date datetime_interval_code datetime_interval_precision day db deallocate debug dec decimal declare default defaults deferrable deferred defined definer degree delete delimiter delimiters dense_rank depends depth deref derived desc describe descriptor detach detail deterministic diagnostics dictionary disable discard disconnect dispatch distinct dlnewcopy dlpreviouscopy dlurlcomplete dlurlcompleteonly dlurlcompletewrite dlurlpath dlurlpathonly dlurlpathwrite dlurlscheme dlurlserver dlvalue do document domain double drop dump dynamic dynamic_function dynamic_function_code each element else elseif elsif empty enable encoding encrypted end end_frame end_partition endexec enforced enum equals errcode error escape event every except exception exclude excluding exclusive exec execute exists exit exp explain expression extension external extract false family fetch file filter final first first_value flag float floor following for force foreach foreign fortran forward found frame_row free freeze from fs full function functions fusion g general generated get global go goto grant granted greatest group grouping groups handler having header hex hierarchy hint hold hour id identity if ignore ilike immediate immediately immutable implementation implicit import in include including increment indent index indexes indicator info inherit inherits initially inline inner inout input insensitive insert instance instantiable instead int integer integrity intersect intersection interval into invoker is isnull isolation join k key key_member key_type label lag language large last last_value lateral lead leading leakproof least left length level library like like_regex limit link listen ln load local localtime localtimestamp location locator lock locked log logged loop lower m map mapping match matched materialized max max_cardinality maxvalue member merge message message_length message_octet_length message_text method min minute minvalue mod mode modifies module month more move multiset mumps name names namespace national natural nchar nclob nesting new next nfc nfd nfkc nfkd nil no none normalize normalized not nothing notice notify notnull nowait nth_value ntile null nullable nullif nulls number numeric object occurrences_regex octet_length octets of off offset oids old on only open operator option options or order ordering ordinality others out outer output over overlaps overlay overriding owned owner p pad parallel parameter parameter_mode parameter_name parameter_ordinal_position parameter_specific_catalog parameter_specific_name parameter_specific_schema parser partial partition pascal passing passthrough password path percent percent_rank percentile_cont percentile_disc perform period permission pg_context pg_datatype_name pg_exception_context pg_exception_detail pg_exception_hint placing plans pli policy portion position position_regex power precedes preceding precision prepare prepared preserve primary print_strict_params prior privileges procedural procedure procedures program public publication query quote raise range rank read reads real reassign recheck recovery recursive ref references referencing refresh regr_avgx regr_avgy regr_count regr_intercept regr_r2 regr_slope regr_sxx regr_sxy regr_syy reindex relative release rename repeatable replace replica requiring reset respect restart restore restrict result result_oid return returned_cardinality returned_length returned_octet_length returned_sqlstate returning returns reverse revoke right role rollback rollup routine routine_catalog routine_name routine_schema routines row row_count row_number rows rowtype rule savepoint scale schema schema_name schemas scope scope_catalog scope_name scope_schema scroll search second section security select selective self sensitive sequence sequences serializable server server_name session session_user set setof sets share show similar simple size skip slice smallint snapshot some source space specific specific_name specifictype sql sqlcode sqlerror sqlexception sqlstate sqlwarning sqrt stable stacked standalone start state statement static statistics stddev_pop stddev_samp stdin stdout storage strict strip structure style subclass_origin submultiset subscription substring substring_regex succeeds sum symmetric sysid system system_time system_user t table table_name tables tablesample tablespace temp template temporary text then ties time timestamp timezone_hour timezone_minute to token top_level_count trailing transaction transaction_active transactions_committed transactions_rolled_back transform transforms translate translate_regex translation treat trigger trigger_catalog trigger_name trigger_schema trim trim_array true truncate trusted type types uescape unbounded uncommitted under unencrypted union unique unknown unlink unlisten unlogged unnamed unnest until untyped update upper uri usage use_column use_variable user user_defined_type_catalog user_defined_type_code user_defined_type_name user_defined_type_schema using vacuum valid validate validator value value_of values var_pop var_samp varbinary varchar variable_conflict variadic varying verbose version versioning view views volatile warning when whenever where while whitespace width_bucket window with within without work wrapper write xml xmlagg xmlattributes xmlbinary xmlcast xmlcomment xmlconcat xmldeclaration xmldocument xmlelement xmlexists xmlforest xmliterate xmlnamespaces xmlparse xmlpi xmlquery xmlroot xmlschema xmlserialize xmltable xmltext xmlvalidate year yes zone"),builtin:o("bigint int8 bigserial serial8 bit varying varbit boolean bool box bytea character char varchar cidr circle date double precision float8 inet integer int int4 interval json jsonb line lseg macaddr macaddr8 money numeric decimal path pg_lsn point polygon real float4 smallint int2 smallserial serial2 serial serial4 text time without zone with timetz timestamp timestamptz tsquery tsvector txid_snapshot uuid xml"),atoms:o("false true null unknown"),operatorChars:/^[*\/+\-%<>!=&|^\/#@?~]/,dateSQL:o("date time timestamp"),support:o("ODBCdotTable decimallessFloat zerolessFloat binaryNumber hexNumber nCharCast charsetCast")}),t.defineMIME("text/x-gql",{name:"sql",keywords:o("ancestor and asc by contains desc descendant distinct from group has in is limit offset on order select superset where"),atoms:o("false true"),builtin:o("blob datetime first key __key__ string integer double boolean null"),operatorChars:/^[*+\-%<>!=]/}),t.defineMIME("text/x-gpsql",{name:"sql",client:o("source"),keywords:o("abort absolute access action active add admin after aggregate all also alter always analyse analyze and any array as asc assertion assignment asymmetric at authorization backward before begin between bigint binary bit boolean both by cache called cascade cascaded case cast chain char character characteristics check checkpoint class close cluster coalesce codegen collate column comment commit committed concurrency concurrently configuration connection constraint constraints contains content continue conversion copy cost cpu_rate_limit create createdb createexttable createrole createuser cross csv cube current current_catalog current_date current_role current_schema current_time current_timestamp current_user cursor cycle data database day deallocate dec decimal declare decode default defaults deferrable deferred definer delete delimiter delimiters deny desc dictionary disable discard distinct distributed do document domain double drop dxl each else enable encoding encrypted end enum errors escape every except exchange exclude excluding exclusive execute exists explain extension external extract false family fetch fields filespace fill filter first float following for force foreign format forward freeze from full function global grant granted greatest group group_id grouping handler hash having header hold host hour identity if ignore ilike immediate immutable implicit in including inclusive increment index indexes inherit inherits initially inline inner inout input insensitive insert instead int integer intersect interval into invoker is isnull isolation join key language large last leading least left level like limit list listen load local localtime localtimestamp location lock log login mapping master match maxvalue median merge minute minvalue missing mode modifies modify month move name names national natural nchar new newline next no nocreatedb nocreateexttable nocreaterole nocreateuser noinherit nologin none noovercommit nosuperuser not nothing notify notnull nowait null nullif nulls numeric object of off offset oids old on only operator option options or order ordered others out outer over overcommit overlaps overlay owned owner parser partial partition partitions passing password percent percentile_cont percentile_disc placing plans position preceding precision prepare prepared preserve primary prior privileges procedural procedure protocol queue quote randomly range read readable reads real reassign recheck recursive ref references reindex reject relative release rename repeatable replace replica reset resource restart restrict returning returns revoke right role rollback rollup rootpartition row rows rule savepoint scatter schema scroll search second security segment select sequence serializable session session_user set setof sets share show similar simple smallint some split sql stable standalone start statement statistics stdin stdout storage strict strip subpartition subpartitions substring superuser symmetric sysid system table tablespace temp template temporary text then threshold ties time timestamp to trailing transaction treat trigger trim true truncate trusted type unbounded uncommitted unencrypted union unique unknown unlisten until update user using vacuum valid validation validator value values varchar variadic varying verbose version view volatile web when where whitespace window with within without work writable write xml xmlattributes xmlconcat xmlelement xmlexists xmlforest xmlparse xmlpi xmlroot xmlserialize year yes zone"),builtin:o("bigint int8 bigserial serial8 bit varying varbit boolean bool box bytea character char varchar cidr circle date double precision float float8 inet integer int int4 interval json jsonb line lseg macaddr macaddr8 money numeric decimal path pg_lsn point polygon real float4 smallint int2 smallserial serial2 serial serial4 text time without zone with timetz timestamp timestamptz tsquery tsvector txid_snapshot uuid xml"),atoms:o("false true null unknown"),operatorChars:/^[*+\-%<>!=&|^\/#@?~]/,dateSQL:o("date time timestamp"),support:o("ODBCdotTable decimallessFloat zerolessFloat binaryNumber hexNumber nCharCast charsetCast")}),t.defineMIME("text/x-sparksql",{name:"sql",keywords:o("add after all alter analyze and anti archive array as asc at between bucket buckets by cache cascade case cast change clear cluster clustered codegen collection column columns comment commit compact compactions compute concatenate cost create cross cube current current_date current_timestamp database databases datata dbproperties defined delete delimited deny desc describe dfs directories distinct distribute drop else end escaped except exchange exists explain export extended external false fields fileformat first following for format formatted from full function functions global grant group grouping having if ignore import in index indexes inner inpath inputformat insert intersect interval into is items join keys last lateral lazy left like limit lines list load local location lock locks logical macro map minus msck natural no not null nulls of on optimize option options or order out outer outputformat over overwrite partition partitioned partitions percent preceding principals purge range recordreader recordwriter recover reduce refresh regexp rename repair replace reset restrict revoke right rlike role roles rollback rollup row rows schema schemas select semi separated serde serdeproperties set sets show skewed sort sorted start statistics stored stratify struct table tables tablesample tblproperties temp temporary terminated then to touch transaction transactions transform true truncate unarchive unbounded uncache union unlock unset use using values view when where window with"),builtin:o("tinyint smallint int bigint boolean float double string binary timestamp decimal array map struct uniontype delimited serde sequencefile textfile rcfile inputformat outputformat"),atoms:o("false true null"),operatorChars:/^[*\/+\-%<>!=~&|^]/,dateSQL:o("date time timestamp"),support:o("ODBCdotTable doubleQuote zerolessFloat")}),t.defineMIME("text/x-esper",{name:"sql",client:o("source"),keywords:o("alter and as asc between by count create delete desc distinct drop from group having in insert into is join like not on or order select set table union update values where limit after all and as at asc avedev avg between by case cast coalesce count create current_timestamp day days delete define desc distinct else end escape events every exists false first from full group having hour hours in inner insert instanceof into irstream is istream join last lastweekday left limit like max match_recognize matches median measures metadatasql min minute minutes msec millisecond milliseconds not null offset on or order outer output partition pattern prev prior regexp retain-union retain-intersection right rstream sec second seconds select set some snapshot sql stddev sum then true unidirectional until update variable weekday when where window"),builtin:{},atoms:o("false true null"),operatorChars:/^[*+\-%<>!=&|^\/#@?~]/,dateSQL:o("time"),support:o("decimallessFloat zerolessFloat binaryNumber hexNumber")})}(n(15))},,function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),n(743),n(906);const r=n(1025),i=n(6),o=n(66),a=n(17);!function(){if(!(""!==a.PageConfig.getOption("hub_user")))return;const t=window.location.href,e=a.URLExt.join(a.PageConfig.getOption("hub_host"),a.PageConfig.getOption("hub_prefix"),"user-redirect"),n=t.replace(a.PageConfig.getBaseUrl(),"");window.history.replaceState(null,"",a.URLExt.join("/",e,n))}();const s=document.getElementById("loadingSpinny");i.Widget.attach(o.HoverManager.GetManager(),document.body);const A=new r.MainApp,l=document.getElementsByTagName("MainApp")[0];i.Widget.attach(A,l),s.remove(),window.addEventListener("resize",()=>{A.update()})},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(17);n.p=r.URLExt.join(r.PageConfig.getBaseUrl(),"viewer/static/")},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(10);e.ActivityMonitor=class{constructor(t){this._timer=-1,this._timeout=-1,this._isDisposed=!1,this._activityStopped=new r.Signal(this),t.signal.connect(this._onSignalFired,this),this._timeout=t.timeout||1e3}get activityStopped(){return this._activityStopped}get timeout(){return this._timeout}set timeout(t){this._timeout=t}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this._isDisposed=!0,r.Signal.clearData(this))}_onSignalFired(t,e){clearTimeout(this._timer),this._sender=t,this._args=e,this._timer=setTimeout(()=>{this._activityStopped.emit({sender:this._sender,args:this._args})},this._timeout)}}},function(t,e,n){(function(t,e){!function(t,n){"use strict";if(!t.setImmediate){var r,i,o,a,s,A=1,l={},c=!1,u=t.document,g=Object.getPrototypeOf&&Object.getPrototypeOf(t);g=g&&g.setTimeout?g:t,"[object process]"==={}.toString.call(t.process)?r=function(t){e.nextTick(function(){d(t)})}:!function(){if(t.postMessage&&!t.importScripts){var e=!0,n=t.onmessage;return t.onmessage=function(){e=!1},t.postMessage("","*"),t.onmessage=n,e}}()?t.MessageChannel?((o=new MessageChannel).port1.onmessage=function(t){d(t.data)},r=function(t){o.port2.postMessage(t)}):u&&"onreadystatechange"in u.createElement("script")?(i=u.documentElement,r=function(t){var e=u.createElement("script");e.onreadystatechange=function(){d(t),e.onreadystatechange=null,i.removeChild(e),e=null},i.appendChild(e)}):r=function(t){setTimeout(d,0,t)}:(a="setImmediate$"+Math.random()+"$",s=function(e){e.source===t&&"string"==typeof e.data&&0===e.data.indexOf(a)&&d(+e.data.slice(a.length))},t.addEventListener?t.addEventListener("message",s,!1):t.attachEvent("onmessage",s),r=function(e){t.postMessage(a+e,"*")}),g.setImmediate=function(t){"function"!=typeof t&&(t=new Function(""+t));for(var e=new Array(arguments.length-1),n=0;n<e.length;n++)e[n]=arguments[n+1];var i={callback:t,args:e};return l[A]=i,r(A),A++},g.clearImmediate=h}function h(t){delete l[t]}function d(t){if(c)setTimeout(d,0,t);else{var e=l[t];if(e){c=!0;try{!function(t){var e=t.callback,r=t.args;switch(r.length){case 0:e();break;case 1:e(r[0]);break;case 2:e(r[0],r[1]);break;case 3:e(r[0],r[1],r[2]);break;default:e.apply(n,r)}}(e)}finally{h(t),c=!1}}}}}("undefined"==typeof self?void 0===t?this:t:self)}).call(this,n(37),n(58))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){function e(t,e,n,r){void 0===n&&(n=0),void 0===r&&(r=-1);var i,o=t.length;if(0===o)return-1;n=n<0?Math.max(0,n+o):Math.min(n,o-1),i=(r=r<0?Math.max(0,r+o):Math.min(r,o-1))<n?r+1+(o-n):r-n+1;for(var a=0;a<i;++a){var s=(n+a)%o;if(t[s]===e)return s}return-1}function n(t,e,n,r){void 0===n&&(n=-1),void 0===r&&(r=0);var i,o=t.length;if(0===o)return-1;i=(n=n<0?Math.max(0,n+o):Math.min(n,o-1))<(r=r<0?Math.max(0,r+o):Math.min(r,o-1))?n+1+(o-r):n-r+1;for(var a=0;a<i;++a){var s=(n-a+o)%o;if(t[s]===e)return s}return-1}function r(t,e,n,r){void 0===n&&(n=0),void 0===r&&(r=-1);var i,o=t.length;if(0===o)return-1;n=n<0?Math.max(0,n+o):Math.min(n,o-1),i=(r=r<0?Math.max(0,r+o):Math.min(r,o-1))<n?r+1+(o-n):r-n+1;for(var a=0;a<i;++a){var s=(n+a)%o;if(e(t[s],s))return s}return-1}function i(t,e,n,r){void 0===n&&(n=-1),void 0===r&&(r=0);var i,o=t.length;if(0===o)return-1;i=(n=n<0?Math.max(0,n+o):Math.min(n,o-1))<(r=r<0?Math.max(0,r+o):Math.min(r,o-1))?n+1+(o-r):n-r+1;for(var a=0;a<i;++a){var s=(n-a+o)%o;if(e(t[s],s))return s}return-1}function o(t,e,n){void 0===e&&(e=0),void 0===n&&(n=-1);var r=t.length;if(!(r<=1))for(e=e<0?Math.max(0,e+r):Math.min(e,r-1),n=n<0?Math.max(0,n+r):Math.min(n,r-1);e<n;){var i=t[e],o=t[n];t[e++]=o,t[n--]=i}}function a(t,e){var n=t.length;if(e<0&&(e+=n),!(e<0||e>=n)){for(var r=t[e],i=e+1;i<n;++i)t[i-1]=t[i];return t.length=n-1,r}}t.firstIndexOf=e,t.lastIndexOf=n,t.findFirstIndex=r,t.findLastIndex=i,t.findFirstValue=function(t,e,n,i){void 0===n&&(n=0),void 0===i&&(i=-1);var o=r(t,e,n,i);return-1!==o?t[o]:void 0},t.findLastValue=function(t,e,n,r){void 0===n&&(n=-1),void 0===r&&(r=0);var o=i(t,e,n,r);return-1!==o?t[o]:void 0},t.lowerBound=function(t,e,n,r,i){void 0===r&&(r=0),void 0===i&&(i=-1);var o=t.length;if(0===o)return 0;for(var a=r=r<0?Math.max(0,r+o):Math.min(r,o-1),s=(i=i<0?Math.max(0,i+o):Math.min(i,o-1))-r+1;s>0;){var A=s>>1,l=a+A;n(t[l],e)<0?(a=l+1,s-=A+1):s=A}return a},t.upperBound=function(t,e,n,r,i){void 0===r&&(r=0),void 0===i&&(i=-1);var o=t.length;if(0===o)return 0;for(var a=r=r<0?Math.max(0,r+o):Math.min(r,o-1),s=(i=i<0?Math.max(0,i+o):Math.min(i,o-1))-r+1;s>0;){var A=s>>1,l=a+A;n(t[l],e)>0?s=A:(a=l+1,s-=A+1)}return a},t.shallowEqual=function(t,e,n){if(t===e)return!0;if(t.length!==e.length)return!1;for(var r=0,i=t.length;r<i;++r)if(n?!n(t[r],e[r]):t[r]!==e[r])return!1;return!0},t.slice=function(t,e){void 0===e&&(e={});var n=e.start,r=e.stop,i=e.step;if(void 0===i&&(i=1),0===i)throw new Error("Slice `step` cannot be zero.");var o,a=t.length;void 0===n?n=i<0?a-1:0:n<0?n=Math.max(n+a,i<0?-1:0):n>=a&&(n=i<0?a-1:a),void 0===r?r=i<0?-1:a:r<0?r=Math.max(r+a,i<0?-1:0):r>=a&&(r=i<0?a-1:a),o=i<0&&r>=n||i>0&&n>=r?0:i<0?Math.floor((r-n+1)/i+1):Math.floor((r-n-1)/i+1);for(var s=[],A=0;A<o;++A)s[A]=t[n+A*i];return s},t.move=function(t,e,n){var r=t.length;if(!(r<=1)&&(e=e<0?Math.max(0,e+r):Math.min(e,r-1))!==(n=n<0?Math.max(0,n+r):Math.min(n,r-1))){for(var i=t[e],o=e<n?1:-1,a=e;a!==n;a+=o)t[a]=t[a+o];t[n]=i}},t.reverse=o,t.rotate=function(t,e,n,r){void 0===n&&(n=0),void 0===r&&(r=-1);var i=t.length;if(!(i<=1||(n=n<0?Math.max(0,n+i):Math.min(n,i-1))>=(r=r<0?Math.max(0,r+i):Math.min(r,i-1)))){var a=r-n+1;if(e>0?e%=a:e<0&&(e=(e%a+a)%a),0!==e){var s=n+e;o(t,n,s-1),o(t,s,r),o(t,n,r)}}},t.fill=function(t,e,n,r){void 0===n&&(n=0),void 0===r&&(r=-1);var i=t.length;if(0!==i){var o;n=n<0?Math.max(0,n+i):Math.min(n,i-1),o=(r=r<0?Math.max(0,r+i):Math.min(r,i-1))<n?r+1+(i-n):r-n+1;for(var a=0;a<o;++a)t[(n+a)%i]=e}},t.insert=function(t,e,n){var r=t.length;e=e<0?Math.max(0,e+r):Math.min(e,r);for(var i=r;i>e;--i)t[i]=t[i-1];t[e]=n},t.removeAt=a,t.removeFirstOf=function(t,n,r,i){void 0===r&&(r=0),void 0===i&&(i=-1);var o=e(t,n,r,i);return-1!==o&&a(t,o),o},t.removeLastOf=function(t,e,r,i){void 0===r&&(r=-1),void 0===i&&(i=0);var o=n(t,e,r,i);return-1!==o&&a(t,o),o},t.removeAllOf=function(t,e,n,r){void 0===n&&(n=0),void 0===r&&(r=-1);var i=t.length;if(0===i)return 0;n=n<0?Math.max(0,n+i):Math.min(n,i-1),r=r<0?Math.max(0,r+i):Math.min(r,i-1);for(var o=0,a=0;a<i;++a)n<=r&&a>=n&&a<=r&&t[a]===e?o++:r<n&&(a<=r||a>=n)&&t[a]===e?o++:o>0&&(t[a-o]=t[a]);return o>0&&(t.length=i-o),o},t.removeFirstWhere=function(t,e,n,i){var o;void 0===n&&(n=0),void 0===i&&(i=-1);var s=r(t,e,n,i);return-1!==s&&(o=a(t,s)),{index:s,value:o}},t.removeLastWhere=function(t,e,n,r){var o;void 0===n&&(n=-1),void 0===r&&(r=0);var s=i(t,e,n,r);return-1!==s&&(o=a(t,s)),{index:s,value:o}},t.removeAllWhere=function(t,e,n,r){void 0===n&&(n=0),void 0===r&&(r=-1);var i=t.length;if(0===i)return 0;n=n<0?Math.max(0,n+i):Math.min(n,i-1),r=r<0?Math.max(0,r+i):Math.min(r,i-1);for(var o=0,a=0;a<i;++a)n<=r&&a>=n&&a<=r&&e(t[a],a)?o++:r<n&&(a<=r||a>=n)&&e(t[a],a)?o++:o>0&&(t[a-o]=t[a]);return o>0&&(t.length=i-o),o}}(e.ArrayExt||(e.ArrayExt={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(82);e.chain=function(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return new i(r.iter(t.map(r.iter)))};var i=function(){function t(t){this._cloned=!1,this._source=t,this._active=void 0}return t.prototype.iter=function(){return this},t.prototype.clone=function(){var e=new t(this._source.clone());return e._active=this._active&&this._active.clone(),e._cloned=!0,this._cloned=!0,e},t.prototype.next=function(){if(void 0===this._active){var t=this._source.next();if(void 0===t)return;this._active=this._cloned?t.clone():t}var e=this._active.next();return void 0!==e?e:(this._active=void 0,this.next())},t}();e.ChainIterator=i},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.empty=function(){return new r};var r=function(){function t(){}return t.prototype.iter=function(){return this},t.prototype.clone=function(){return new t},t.prototype.next=function(){},t}();e.EmptyIterator=r},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(82);e.enumerate=function(t,e){return void 0===e&&(e=0),new i(r.iter(t),e)};var i=function(){function t(t,e){this._source=t,this._index=e}return t.prototype.iter=function(){return this},t.prototype.clone=function(){return new t(this._source.clone(),this._index)},t.prototype.next=function(){var t=this._source.next();if(void 0!==t)return[this._index++,t]},t}();e.EnumerateIterator=i},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(82);e.filter=function(t,e){return new i(r.iter(t),e)};var i=function(){function t(t,e){this._index=0,this._source=t,this._fn=e}return t.prototype.iter=function(){return this},t.prototype.clone=function(){var e=new t(this._source.clone(),this._fn);return e._index=this._index,e},t.prototype.next=function(){for(var t,e=this._fn,n=this._source;void 0!==(t=n.next());)if(e(t,this._index++))return t},t}();e.FilterIterator=i},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(82);e.find=function(t,e){for(var n,i=0,o=r.iter(t);void 0!==(n=o.next());)if(e(n,i++))return n},e.findIndex=function(t,e){for(var n,i=0,o=r.iter(t);void 0!==(n=o.next());)if(e(n,i++))return i-1;return-1},e.min=function(t,e){var n=r.iter(t),i=n.next();if(void 0!==i){for(var o=i;void 0!==(i=n.next());)e(i,o)<0&&(o=i);return o}},e.max=function(t,e){var n=r.iter(t),i=n.next();if(void 0!==i){for(var o=i;void 0!==(i=n.next());)e(i,o)>0&&(o=i);return o}},e.minmax=function(t,e){var n=r.iter(t),i=n.next();if(void 0!==i){for(var o=i,a=i;void 0!==(i=n.next());)e(i,o)<0?o=i:e(i,a)>0&&(a=i);return[o,a]}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(82);e.map=function(t,e){return new i(r.iter(t),e)};var i=function(){function t(t,e){this._index=0,this._source=t,this._fn=e}return t.prototype.iter=function(){return this},t.prototype.clone=function(){var e=new t(this._source.clone(),this._fn);return e._index=this._index,e},t.prototype.next=function(){var t=this._source.next();if(void 0!==t)return this._fn.call(void 0,t,this._index++)},t}();e.MapIterator=i},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.range=function(t,e,n){return void 0===e?new i(0,t,1):new i(t,e,void 0===n?1:n)};var r,i=function(){function t(t,e,n){this._index=0,this._start=t,this._stop=e,this._step=n,this._length=r.rangeLength(t,e,n)}return t.prototype.iter=function(){return this},t.prototype.clone=function(){var e=new t(this._start,this._stop,this._step);return e._index=this._index,e},t.prototype.next=function(){if(!(this._index>=this._length))return this._start+this._step*this._index++},t}();e.RangeIterator=i,function(t){t.rangeLength=function(t,e,n){return 0===n?1/0:t>e&&n>0?0:t<e&&n<0?0:Math.ceil((e-t)/n)}}(r||(r={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(82);e.reduce=function(t,e,n){var i=0,o=r.iter(t),a=o.next();if(void 0===a&&void 0===n)throw new TypeError("Reduce of empty iterable with no initial value.");if(void 0===a)return n;var s,A,l=o.next();if(void 0===l&&void 0===n)return a;if(void 0===l)return e(n,a,i++);for(s=e(void 0===n?a:e(n,a,i++),l,i++);void 0!==(A=o.next());)s=e(s,A,i++);return s}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.repeat=function(t,e){return new r(t,e)},e.once=function(t){return new r(t,1)};var r=function(){function t(t,e){this._value=t,this._count=e}return t.prototype.iter=function(){return this},t.prototype.clone=function(){return new t(this._value,this._count)},t.prototype.next=function(){if(!(this._count<=0))return this._count--,this._value},t}();e.RepeatIterator=r},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.retro=function(t){return"function"==typeof t.retro?t.retro():new r(t)};var r=function(){function t(t){this._source=t,this._index=t.length-1}return t.prototype.iter=function(){return this},t.prototype.clone=function(){var e=new t(this._source);return e._index=this._index,e},t.prototype.next=function(){if(!(this._index<0||this._index>=this._source.length))return this._source[this._index--]},t}();e.RetroArrayIterator=r},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(82);e.topologicSort=function(t){var e=[],n=new Set,i=new Map;return r.each(t,function(t){var e=t[0],n=t[1],r=i.get(n);r?r.push(e):i.set(n,[e])}),i.forEach(function(t,e){o(e)}),e;function o(t){if(!n.has(t)){n.add(t);var r=i.get(t);r&&r.forEach(o),e.push(t)}}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(82);e.stride=function(t,e){return new i(r.iter(t),e)};var i=function(){function t(t,e){this._source=t,this._step=e}return t.prototype.iter=function(){return this},t.prototype.clone=function(){return new t(this._source.clone(),this._step)},t.prototype.next=function(){for(var t=this._source.next(),e=this._step-1;e>0;--e)this._source.next();return t},t}();e.StrideIterator=i},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){function e(t,e,n){void 0===n&&(n=0);for(var r=new Array(e.length),i=0,o=n,a=e.length;i<a;++i,++o){if(-1===(o=t.indexOf(e[i],o)))return null;r[i]=o}return r}t.findIndices=e,t.matchSumOfSquares=function(t,n,r){void 0===r&&(r=0);var i=e(t,n,r);if(!i)return null;for(var o=0,a=0,s=i.length;a<s;++a){var A=i[a]-r;o+=A*A}return{score:o,indices:i}},t.matchSumOfDeltas=function(t,n,r){void 0===r&&(r=0);var i=e(t,n,r);if(!i)return null;for(var o=0,a=r-1,s=0,A=i.length;s<A;++s){var l=i[s];o+=l-a-1,a=l}return{score:o,indices:i}},t.highlight=function(t,e,n){for(var r=[],i=0,o=0,a=e.length;i<a;){for(var s=e[i],A=e[i];++i<a&&e[i]===A+1;)A++;o<s&&r.push(t.slice(o,s)),s<A+1&&r.push(n(t.slice(s,A+1))),o=A+1}return o<t.length&&r.push(t.slice(o)),r},t.cmp=function(t,e){return t<e?-1:t>e?1:0}}(e.StringExt||(e.StringExt={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(82);e.take=function(t,e){return new i(r.iter(t),e)};var i=function(){function t(t,e){this._source=t,this._count=e}return t.prototype.iter=function(){return this},t.prototype.clone=function(){return new t(this._source.clone(),this._count)},t.prototype.next=function(){if(!(this._count<=0)){var t=this._source.next();if(void 0!==t)return this._count--,t}},t}();e.TakeIterator=i},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(82);e.zip=function(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return new i(t.map(r.iter))};var i=function(){function t(t){this._source=t}return t.prototype.iter=function(){return this},t.prototype.clone=function(){return new t(this._source.map(function(t){return t.clone()}))},t.prototype.next=function(){for(var t=new Array(this._source.length),e=0,n=this._source.length;e<n;++e){var r=this._source[e].next();if(void 0===r)return;t[e]=r}return t},t}();e.ZipIterator=i},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});e.DataConnector=class{async list(t){throw new Error("DataConnector#list method has not been implemented.")}async remove(t){throw new Error("DataConnector#remove method has not been implemented.")}async save(t,e){throw new Error("DataConnector#save method has not been implemented.")}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){t.CODE_BLOCK_MARKER="```";const e=[".markdown",".mdown",".mkdn",".md",".mkd",".mdwn",".mdtxt",".mdtext",".text",".txt",".Rmd"];class n{constructor(t){this.startLine=t,this.code="",this.endLine=-1}}t.MarkdownCodeBlock=n,t.isMarkdown=function(t){return e.indexOf(t)>-1},t.findMarkdownCodeBlocks=function(e){if(!e||""===e)return[];const r=e.split("\n"),i=[];let o=null;for(let e=0;e<r.length;e++){const a=r[e],s=0===a.indexOf(t.CODE_BLOCK_MARKER),A=null!=o;if(s||A)if(A)o&&(s?(o.endLine=e-1,i.push(o),o=null):o.code+=a+"\n");else{o=new n(e);const r=a.indexOf(t.CODE_BLOCK_MARKER),s=a.lastIndexOf(t.CODE_BLOCK_MARKER);r!==s&&(o.code=a.substring(r+t.CODE_BLOCK_MARKER.length,s),o.endLine=e,i.push(o),o=null)}}return i}}(e.MarkdownCodeBlocks||(e.MarkdownCodeBlocks={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9);!function(t){t.MAJOR_VERSION=4,t.MINOR_VERSION=4,t.validateMimeValue=function(t,e){const n="application/json"===t||/^application\/(.*?)+\+json$/.test(t);let i=t=>"[object String]"===Object.prototype.toString.call(t);if(Array.isArray(e)){if(n)return!1;let t=!0;return e.forEach(e=>{i(e)||(t=!1)}),t}return i(e)?!n:!!n&&r.JSONExt.isObject(e)},t.isRaw=function(t){return"raw"===t.cell_type},t.isMarkdown=function(t){return"markdown"===t.cell_type},t.isCode=function(t){return"code"===t.cell_type},t.isExecuteResult=function(t){return"execute_result"===t.output_type},t.isDisplayData=function(t){return"display_data"===t.output_type},t.isDisplayUpdate=function(t){return"update_display_data"===t.output_type},t.isStream=function(t){return"stream"===t.output_type},t.isError=function(t){return"error"===t.output_type}}(e.nbformat||(e.nbformat={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){function e(t){return null===t||"boolean"==typeof t||"number"==typeof t||"string"==typeof t}function n(t){return Array.isArray(t)}function r(t,i){if(t===i)return!0;if(e(t)||e(i))return!1;var o=n(t),a=n(i);return o===a&&(o&&a?function(t,e){if(t===e)return!0;if(t.length!==e.length)return!1;for(var n=0,i=t.length;n<i;++n)if(!r(t[n],e[n]))return!1;return!0}(t,i):function(t,e){if(t===e)return!0;for(var n in t)if(!(n in e))return!1;for(var n in e)if(!(n in t))return!1;for(var n in t)if(!r(t[n],e[n]))return!1;return!0}(t,i))}function i(t){return e(t)?t:n(t)?function(t){for(var e=new Array(t.length),n=0,r=t.length;n<r;++n)e[n]=i(t[n]);return e}(t):function(t){var e={};for(var n in t)e[n]=i(t[n]);return e}(t)}t.emptyObject=Object.freeze({}),t.emptyArray=Object.freeze([]),t.isPrimitive=e,t.isArray=n,t.isObject=function(t){return!e(t)&&!n(t)},t.deepEqual=r,t.deepCopy=i}(e.JSONExt||(e.JSONExt={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=function(){function t(){this._types=[],this._values=[]}return t.prototype.types=function(){return this._types.slice()},t.prototype.hasData=function(t){return-1!==this._types.indexOf(t)},t.prototype.getData=function(t){var e=this._types.indexOf(t);return-1!==e?this._values[e]:void 0},t.prototype.setData=function(t,e){this.clearData(t),this._types.push(t),this._values.push(e)},t.prototype.clearData=function(t){var e=this._types.indexOf(t);-1!==e&&(this._types.splice(e,1),this._values.splice(e,1))},t.prototype.clear=function(){this._types.length=0,this._values.length=0},t}();e.MimeData=r},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=function(){function t(){var t=this;this.promise=new Promise(function(e,n){t._resolve=e,t._reject=n})}return t.prototype.resolve=function(t){(0,this._resolve)(t)},t.prototype.reject=function(t){(0,this._reject)(t)},t}();e.PromiseDelegate=r},function(t,e,n){"use strict";e.randomBytes=e.rng=e.pseudoRandomBytes=e.prng=n(135),e.createHash=e.Hash=n(165),e.createHmac=e.Hmac=n(403);var r=n(787),i=Object.keys(r),o=["sha1","sha224","sha256","sha384","sha512","md5","rmd160"].concat(i);e.getHashes=function(){return o};var a=n(406);e.pbkdf2=a.pbkdf2,e.pbkdf2Sync=a.pbkdf2Sync;var s=n(789);e.Cipher=s.Cipher,e.createCipher=s.createCipher,e.Cipheriv=s.Cipheriv,e.createCipheriv=s.createCipheriv,e.Decipher=s.Decipher,e.createDecipher=s.createDecipher,e.Decipheriv=s.Decipheriv,e.createDecipheriv=s.createDecipheriv,e.getCiphers=s.getCiphers,e.listCiphers=s.listCiphers;var A=n(806);e.DiffieHellmanGroup=A.DiffieHellmanGroup,e.createDiffieHellmanGroup=A.createDiffieHellmanGroup,e.getDiffieHellman=A.getDiffieHellman,e.createDiffieHellman=A.createDiffieHellman,e.DiffieHellman=A.DiffieHellman;var l=n(811);e.createSign=l.createSign,e.Sign=l.Sign,e.createVerify=l.createVerify,e.Verify=l.Verify,e.createECDH=n(845);var c=n(846);e.publicEncrypt=c.publicEncrypt,e.privateEncrypt=c.privateEncrypt,e.publicDecrypt=c.publicDecrypt,e.privateDecrypt=c.privateDecrypt;var u=n(849);e.randomFill=u.randomFill,e.randomFillSync=u.randomFillSync,e.createCredentials=function(){throw new Error(["sorry, createCredentials is not implemented yet","we accept pull requests","https://github.com/crypto-browserify/crypto-browserify"].join("\n"))},e.constants={DH_CHECK_P_NOT_SAFE_PRIME:2,DH_CHECK_P_NOT_PRIME:1,DH_UNABLE_TO_CHECK_GENERATOR:4,DH_NOT_SUITABLE_GENERATOR:8,NPN_ENABLED:1,ALPN_ENABLED:1,RSA_PKCS1_PADDING:1,RSA_SSLV23_PADDING:2,RSA_NO_PADDING:3,RSA_PKCS1_OAEP_PADDING:4,RSA_X931_PADDING:5,RSA_PKCS1_PSS_PADDING:6,POINT_CONVERSION_COMPRESSED:2,POINT_CONVERSION_UNCOMPRESSED:4,POINT_CONVERSION_HYBRID:6}},function(t,e){e.read=function(t,e,n,r,i){var o,a,s=8*i-r-1,A=(1<<s)-1,l=A>>1,c=-7,u=n?i-1:0,g=n?-1:1,h=t[e+u];for(u+=g,o=h&(1<<-c)-1,h>>=-c,c+=s;c>0;o=256*o+t[e+u],u+=g,c-=8);for(a=o&(1<<-c)-1,o>>=-c,c+=r;c>0;a=256*a+t[e+u],u+=g,c-=8);if(0===o)o=1-l;else{if(o===A)return a?NaN:1/0*(h?-1:1);a+=Math.pow(2,r),o-=l}return(h?-1:1)*a*Math.pow(2,o-r)},e.write=function(t,e,n,r,i,o){var a,s,A,l=8*o-i-1,c=(1<<l)-1,u=c>>1,g=23===i?Math.pow(2,-24)-Math.pow(2,-77):0,h=r?0:o-1,d=r?1:-1,f=e<0||0===e&&1/e<0?1:0;for(e=Math.abs(e),isNaN(e)||e===1/0?(s=isNaN(e)?1:0,a=c):(a=Math.floor(Math.log(e)/Math.LN2),e*(A=Math.pow(2,-a))<1&&(a--,A*=2),(e+=a+u>=1?g/A:g*Math.pow(2,1-u))*A>=2&&(a++,A/=2),a+u>=c?(s=0,a=c):a+u>=1?(s=(e*A-1)*Math.pow(2,i),a+=u):(s=e*Math.pow(2,u-1)*Math.pow(2,i),a=0));i>=8;t[n+h]=255&s,h+=d,s/=256,i-=8);for(a=a<<i|s,l+=i;l>0;t[n+h]=255&a,h+=d,a/=256,l-=8);t[n+h-d]|=128*f}},function(t,e){var n={}.toString;t.exports=Array.isArray||function(t){return"[object Array]"==n.call(t)}},function(t,e){var n={}.toString;t.exports=Array.isArray||function(t){return"[object Array]"==n.call(t)}},function(t,e){},function(t,e,n){"use strict";var r=n(270).Buffer,i=n(774);t.exports=function(){function t(){!function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}(this,t),this.head=null,this.tail=null,this.length=0}return t.prototype.push=function(t){var e={data:t,next:null};this.length>0?this.tail.next=e:this.head=e,this.tail=e,++this.length},t.prototype.unshift=function(t){var e={data:t,next:this.head};0===this.length&&(this.tail=e),this.head=e,++this.length},t.prototype.shift=function(){if(0!==this.length){var t=this.head.data;return 1===this.length?this.head=this.tail=null:this.head=this.head.next,--this.length,t}},t.prototype.clear=function(){this.head=this.tail=null,this.length=0},t.prototype.join=function(t){if(0===this.length)return"";for(var e=this.head,n=""+e.data;e=e.next;)n+=t+e.data;return n},t.prototype.concat=function(t){if(0===this.length)return r.alloc(0);if(1===this.length)return this.head.data;for(var e,n,i,o=r.allocUnsafe(t>>>0),a=this.head,s=0;a;)e=a.data,n=o,i=s,e.copy(n,i),s+=a.data.length,a=a.next;return o},t}(),i&&i.inspect&&i.inspect.custom&&(t.exports.prototype[i.inspect.custom]=function(){var t=i.inspect({length:this.length});return this.constructor.name+" "+t})},function(t,e){},function(t,e,n){(function(e){function n(t){try{if(!e.localStorage)return!1}catch(t){return!1}var n=e.localStorage[t];return null!=n&&"true"===String(n).toLowerCase()}t.exports=function(t,e){if(n("noDeprecation"))return t;var r=!1;return function(){if(!r){if(n("throwDeprecation"))throw new Error(e);n("traceDeprecation")?console.trace(e):console.warn(e),r=!0}return t.apply(this,arguments)}}}).call(this,n(37))},function(t,e,n){var r=n(42),i=r.Buffer;function o(t,e){for(var n in t)e[n]=t[n]}function a(t,e,n){return i(t,e,n)}i.from&&i.alloc&&i.allocUnsafe&&i.allocUnsafeSlow?t.exports=r:(o(r,e),e.Buffer=a),o(i,a),a.from=function(t,e,n){if("number"==typeof t)throw new TypeError("Argument must not be a number");return i(t,e,n)},a.alloc=function(t,e,n){if("number"!=typeof t)throw new TypeError("Argument must be a number");var r=i(t);return void 0!==e?"string"==typeof n?r.fill(e,n):r.fill(e):r.fill(0),r},a.allocUnsafe=function(t){if("number"!=typeof t)throw new TypeError("Argument must be a number");return i(t)},a.allocUnsafeSlow=function(t){if("number"!=typeof t)throw new TypeError("Argument must be a number");return r.SlowBuffer(t)}},function(t,e,n){"use strict";t.exports=o;var r=n(400),i=n(166);function o(t){if(!(this instanceof o))return new o(t);r.call(this,t)}i.inherits=n(30),i.inherits(o,r),o.prototype._transform=function(t,e,n){n(null,t)}},function(t,e,n){t.exports=n(271)},function(t,e,n){t.exports=n(119)},function(t,e,n){t.exports=n(269).Transform},function(t,e,n){t.exports=n(269).PassThrough},function(t,e,n){var r=n(30),i=n(136),o=n(35).Buffer,a=[1518500249,1859775393,-1894007588,-899497514],s=new Array(80);function A(){this.init(),this._w=s,i.call(this,64,56)}function l(t){return t<<30|t>>>2}function c(t,e,n,r){return 0===t?e&n|~e&r:2===t?e&n|e&r|n&r:e^n^r}r(A,i),A.prototype.init=function(){return this._a=1732584193,this._b=4023233417,this._c=2562383102,this._d=271733878,this._e=3285377520,this},A.prototype._update=function(t){for(var e,n=this._w,r=0|this._a,i=0|this._b,o=0|this._c,s=0|this._d,A=0|this._e,u=0;u<16;++u)n[u]=t.readInt32BE(4*u);for(;u<80;++u)n[u]=n[u-3]^n[u-8]^n[u-14]^n[u-16];for(var g=0;g<80;++g){var h=~~(g/20),d=0|((e=r)<<5|e>>>27)+c(h,i,o,s)+A+n[g]+a[h];A=s,s=o,o=l(i),i=r,r=d}this._a=r+this._a|0,this._b=i+this._b|0,this._c=o+this._c|0,this._d=s+this._d|0,this._e=A+this._e|0},A.prototype._hash=function(){var t=o.allocUnsafe(20);return t.writeInt32BE(0|this._a,0),t.writeInt32BE(0|this._b,4),t.writeInt32BE(0|this._c,8),t.writeInt32BE(0|this._d,12),t.writeInt32BE(0|this._e,16),t},t.exports=A},function(t,e,n){var r=n(30),i=n(136),o=n(35).Buffer,a=[1518500249,1859775393,-1894007588,-899497514],s=new Array(80);function A(){this.init(),this._w=s,i.call(this,64,56)}function l(t){return t<<5|t>>>27}function c(t){return t<<30|t>>>2}function u(t,e,n,r){return 0===t?e&n|~e&r:2===t?e&n|e&r|n&r:e^n^r}r(A,i),A.prototype.init=function(){return this._a=1732584193,this._b=4023233417,this._c=2562383102,this._d=271733878,this._e=3285377520,this},A.prototype._update=function(t){for(var e,n=this._w,r=0|this._a,i=0|this._b,o=0|this._c,s=0|this._d,A=0|this._e,g=0;g<16;++g)n[g]=t.readInt32BE(4*g);for(;g<80;++g)n[g]=(e=n[g-3]^n[g-8]^n[g-14]^n[g-16])<<1|e>>>31;for(var h=0;h<80;++h){var d=~~(h/20),f=l(r)+u(d,i,o,s)+A+n[h]+a[d]|0;A=s,s=o,o=c(i),i=r,r=f}this._a=r+this._a|0,this._b=i+this._b|0,this._c=o+this._c|0,this._d=s+this._d|0,this._e=A+this._e|0},A.prototype._hash=function(){var t=o.allocUnsafe(20);return t.writeInt32BE(0|this._a,0),t.writeInt32BE(0|this._b,4),t.writeInt32BE(0|this._c,8),t.writeInt32BE(0|this._d,12),t.writeInt32BE(0|this._e,16),t},t.exports=A},function(t,e,n){var r=n(30),i=n(401),o=n(136),a=n(35).Buffer,s=new Array(64);function A(){this.init(),this._w=s,o.call(this,64,56)}r(A,i),A.prototype.init=function(){return this._a=3238371032,this._b=914150663,this._c=812702999,this._d=4144912697,this._e=4290775857,this._f=1750603025,this._g=1694076839,this._h=3204075428,this},A.prototype._hash=function(){var t=a.allocUnsafe(28);return t.writeInt32BE(this._a,0),t.writeInt32BE(this._b,4),t.writeInt32BE(this._c,8),t.writeInt32BE(this._d,12),t.writeInt32BE(this._e,16),t.writeInt32BE(this._f,20),t.writeInt32BE(this._g,24),t},t.exports=A},function(t,e,n){var r=n(30),i=n(402),o=n(136),a=n(35).Buffer,s=new Array(160);function A(){this.init(),this._w=s,o.call(this,128,112)}r(A,i),A.prototype.init=function(){return this._ah=3418070365,this._bh=1654270250,this._ch=2438529370,this._dh=355462360,this._eh=1731405415,this._fh=2394180231,this._gh=3675008525,this._hh=1203062813,this._al=3238371032,this._bl=914150663,this._cl=812702999,this._dl=4144912697,this._el=4290775857,this._fl=1750603025,this._gl=1694076839,this._hl=3204075428,this},A.prototype._hash=function(){var t=a.allocUnsafe(48);function e(e,n,r){t.writeInt32BE(e,r),t.writeInt32BE(n,r+4)}return e(this._ah,this._al,0),e(this._bh,this._bl,8),e(this._ch,this._cl,16),e(this._dh,this._dl,24),e(this._eh,this._el,32),e(this._fh,this._fl,40),t},t.exports=A},function(t,e,n){"use strict";var r=n(30),i=n(35).Buffer,o=n(105),a=i.alloc(128),s=64;function A(t,e){o.call(this,"digest"),"string"==typeof e&&(e=i.from(e)),this._alg=t,this._key=e,e.length>s?e=t(e):e.length<s&&(e=i.concat([e,a],s));for(var n=this._ipad=i.allocUnsafe(s),r=this._opad=i.allocUnsafe(s),A=0;A<s;A++)n[A]=54^e[A],r[A]=92^e[A];this._hash=[n]}r(A,o),A.prototype._update=function(t){this._hash.push(t)},A.prototype._final=function(){var t=this._alg(i.concat(this._hash));return this._alg(i.concat([this._opad,t]))},t.exports=A},function(t,e,n){t.exports=n(405)},function(t,e,n){(function(e,r){var i,o=n(407),a=n(408),s=n(409),A=n(35).Buffer,l=e.crypto&&e.crypto.subtle,c={sha:"SHA-1","sha-1":"SHA-1",sha1:"SHA-1",sha256:"SHA-256","sha-256":"SHA-256",sha384:"SHA-384","sha-384":"SHA-384","sha-512":"SHA-512",sha512:"SHA-512"},u=[];function g(t,e,n,r,i){return l.importKey("raw",t,{name:"PBKDF2"},!1,["deriveBits"]).then(function(t){return l.deriveBits({name:"PBKDF2",salt:e,iterations:n,hash:{name:i}},t,r<<3)}).then(function(t){return A.from(t)})}t.exports=function(t,n,h,d,f,p){"function"==typeof f&&(p=f,f=void 0);var I=c[(f=f||"sha1").toLowerCase()];if(!I||"function"!=typeof e.Promise)return r.nextTick(function(){var e;try{e=s(t,n,h,d,f)}catch(t){return p(t)}p(null,e)});if(o(t,n,h,d),"function"!=typeof p)throw new Error("No callback provided to pbkdf2");A.isBuffer(t)||(t=A.from(t,a)),A.isBuffer(n)||(n=A.from(n,a)),function(t,e){t.then(function(t){r.nextTick(function(){e(null,t)})},function(t){r.nextTick(function(){e(t)})})}(function(t){if(e.process&&!e.process.browser)return Promise.resolve(!1);if(!l||!l.importKey||!l.deriveBits)return Promise.resolve(!1);if(void 0!==u[t])return u[t];var n=g(i=i||A.alloc(8),i,10,128,t).then(function(){return!0}).catch(function(){return!1});return u[t]=n,n}(I).then(function(e){return e?g(t,n,h,d,I):s(t,n,h,d,f)}),p)}}).call(this,n(37),n(58))},function(t,e,n){var r=n(790),i=n(275),o=n(276),a=n(805),s=n(212);function A(t,e,n){if(t=t.toLowerCase(),o[t])return i.createCipheriv(t,e,n);if(a[t])return new r({key:e,iv:n,mode:t});throw new TypeError("invalid suite type")}function l(t,e,n){if(t=t.toLowerCase(),o[t])return i.createDecipheriv(t,e,n);if(a[t])return new r({key:e,iv:n,mode:t,decrypt:!0});throw new TypeError("invalid suite type")}e.createCipher=e.Cipher=function(t,e){var n,r;if(t=t.toLowerCase(),o[t])n=o[t].key,r=o[t].iv;else{if(!a[t])throw new TypeError("invalid suite type");n=8*a[t].key,r=a[t].iv}var i=s(e,!1,n,r);return A(t,i.key,i.iv)},e.createCipheriv=e.Cipheriv=A,e.createDecipher=e.Decipher=function(t,e){var n,r;if(t=t.toLowerCase(),o[t])n=o[t].key,r=o[t].iv;else{if(!a[t])throw new TypeError("invalid suite type");n=8*a[t].key,r=a[t].iv}var i=s(e,!1,n,r);return l(t,i.key,i.iv)},e.createDecipheriv=e.Decipheriv=l,e.listCiphers=e.getCiphers=function(){return Object.keys(a).concat(i.getCiphers())}},function(t,e,n){var r=n(105),i=n(274),o=n(30),a=n(35).Buffer,s={"des-ede3-cbc":i.CBC.instantiate(i.EDE),"des-ede3":i.EDE,"des-ede-cbc":i.CBC.instantiate(i.EDE),"des-ede":i.EDE,"des-cbc":i.CBC.instantiate(i.DES),"des-ecb":i.DES};function A(t){r.call(this);var e,n=t.mode.toLowerCase(),i=s[n];e=t.decrypt?"decrypt":"encrypt";var o=t.key;a.isBuffer(o)||(o=a.from(o)),"des-ede"!==n&&"des-ede-cbc"!==n||(o=a.concat([o,o.slice(0,8)]));var A=t.iv;a.isBuffer(A)||(A=a.from(A)),this._des=i.create({key:o,iv:A,type:e})}s.des=s["des-cbc"],s.des3=s["des-ede3-cbc"],t.exports=A,o(A,r),A.prototype._update=function(t){return a.from(this._des.update(t))},A.prototype._final=function(){return a.from(this._des.final())}},function(t,e,n){"use strict";e.readUInt32BE=function(t,e){return(t[0+e]<<24|t[1+e]<<16|t[2+e]<<8|t[3+e])>>>0},e.writeUInt32BE=function(t,e,n){t[0+n]=e>>>24,t[1+n]=e>>>16&255,t[2+n]=e>>>8&255,t[3+n]=255&e},e.ip=function(t,e,n,r){for(var i=0,o=0,a=6;a>=0;a-=2){for(var s=0;s<=24;s+=8)i<<=1,i|=e>>>s+a&1;for(s=0;s<=24;s+=8)i<<=1,i|=t>>>s+a&1}for(a=6;a>=0;a-=2){for(s=1;s<=25;s+=8)o<<=1,o|=e>>>s+a&1;for(s=1;s<=25;s+=8)o<<=1,o|=t>>>s+a&1}n[r+0]=i>>>0,n[r+1]=o>>>0},e.rip=function(t,e,n,r){for(var i=0,o=0,a=0;a<4;a++)for(var s=24;s>=0;s-=8)i<<=1,i|=e>>>s+a&1,i<<=1,i|=t>>>s+a&1;for(a=4;a<8;a++)for(s=24;s>=0;s-=8)o<<=1,o|=e>>>s+a&1,o<<=1,o|=t>>>s+a&1;n[r+0]=i>>>0,n[r+1]=o>>>0},e.pc1=function(t,e,n,r){for(var i=0,o=0,a=7;a>=5;a--){for(var s=0;s<=24;s+=8)i<<=1,i|=e>>s+a&1;for(s=0;s<=24;s+=8)i<<=1,i|=t>>s+a&1}for(s=0;s<=24;s+=8)i<<=1,i|=e>>s+a&1;for(a=1;a<=3;a++){for(s=0;s<=24;s+=8)o<<=1,o|=e>>s+a&1;for(s=0;s<=24;s+=8)o<<=1,o|=t>>s+a&1}for(s=0;s<=24;s+=8)o<<=1,o|=t>>s+a&1;n[r+0]=i>>>0,n[r+1]=o>>>0},e.r28shl=function(t,e){return t<<e&268435455|t>>>28-e};var r=[14,11,17,4,27,23,25,0,13,22,7,18,5,9,16,24,2,20,12,21,1,8,15,26,15,4,25,19,9,1,26,16,5,11,23,8,12,7,17,0,22,3,10,14,6,20,27,24];e.pc2=function(t,e,n,i){for(var o=0,a=0,s=r.length>>>1,A=0;A<s;A++)o<<=1,o|=t>>>r[A]&1;for(A=s;A<r.length;A++)a<<=1,a|=e>>>r[A]&1;n[i+0]=o>>>0,n[i+1]=a>>>0},e.expand=function(t,e,n){var r=0,i=0;r=(1&t)<<5|t>>>27;for(var o=23;o>=15;o-=4)r<<=6,r|=t>>>o&63;for(o=11;o>=3;o-=4)i|=t>>>o&63,i<<=6;i|=(31&t)<<1|t>>>31,e[n+0]=r>>>0,e[n+1]=i>>>0};var i=[14,0,4,15,13,7,1,4,2,14,15,2,11,13,8,1,3,10,10,6,6,12,12,11,5,9,9,5,0,3,7,8,4,15,1,12,14,8,8,2,13,4,6,9,2,1,11,7,15,5,12,11,9,3,7,14,3,10,10,0,5,6,0,13,15,3,1,13,8,4,14,7,6,15,11,2,3,8,4,14,9,12,7,0,2,1,13,10,12,6,0,9,5,11,10,5,0,13,14,8,7,10,11,1,10,3,4,15,13,4,1,2,5,11,8,6,12,7,6,12,9,0,3,5,2,14,15,9,10,13,0,7,9,0,14,9,6,3,3,4,15,6,5,10,1,2,13,8,12,5,7,14,11,12,4,11,2,15,8,1,13,1,6,10,4,13,9,0,8,6,15,9,3,8,0,7,11,4,1,15,2,14,12,3,5,11,10,5,14,2,7,12,7,13,13,8,14,11,3,5,0,6,6,15,9,0,10,3,1,4,2,7,8,2,5,12,11,1,12,10,4,14,15,9,10,3,6,15,9,0,0,6,12,10,11,1,7,13,13,8,15,9,1,4,3,5,14,11,5,12,2,7,8,2,4,14,2,14,12,11,4,2,1,12,7,4,10,7,11,13,6,1,8,5,5,0,3,15,15,10,13,3,0,9,14,8,9,6,4,11,2,8,1,12,11,7,10,1,13,14,7,2,8,13,15,6,9,15,12,0,5,9,6,10,3,4,0,5,14,3,12,10,1,15,10,4,15,2,9,7,2,12,6,9,8,5,0,6,13,1,3,13,4,14,14,0,7,11,5,3,11,8,9,4,14,3,15,2,5,12,2,9,8,5,12,15,3,10,7,11,0,14,4,1,10,7,1,6,13,0,11,8,6,13,4,13,11,0,2,11,14,7,15,4,0,9,8,1,13,10,3,14,12,3,9,5,7,12,5,2,10,15,6,8,1,6,1,6,4,11,11,13,13,8,12,1,3,4,7,10,14,7,10,9,15,5,6,0,8,15,0,14,5,2,9,3,2,12,13,1,2,15,8,13,4,8,6,10,15,3,11,7,1,4,10,12,9,5,3,6,14,11,5,0,0,14,12,9,7,2,7,2,11,1,4,14,1,7,9,4,12,10,14,8,2,13,0,15,6,12,10,9,13,0,15,3,3,5,5,6,8,11];e.substitute=function(t,e){for(var n=0,r=0;r<4;r++){n<<=4,n|=i[64*r+(t>>>18-6*r&63)]}for(r=0;r<4;r++){n<<=4,n|=i[256+64*r+(e>>>18-6*r&63)]}return n>>>0};var o=[16,25,12,11,3,20,4,15,31,17,9,6,27,14,1,22,30,24,8,18,0,5,29,23,13,19,2,26,10,21,28,7];e.permute=function(t){for(var e=0,n=0;n<o.length;n++)e<<=1,e|=t>>>o[n]&1;return e>>>0},e.padSplit=function(t,e,n){for(var r=t.toString(2);r.length<e;)r="0"+r;for(var i=[],o=0;o<e;o+=n)i.push(r.slice(o,o+n));return i.join(" ")}},function(t,e,n){"use strict";var r=n(77);function i(t){this.options=t,this.type=this.options.type,this.blockSize=8,this._init(),this.buffer=new Array(this.blockSize),this.bufferOff=0}t.exports=i,i.prototype._init=function(){},i.prototype.update=function(t){return 0===t.length?[]:"decrypt"===this.type?this._updateDecrypt(t):this._updateEncrypt(t)},i.prototype._buffer=function(t,e){for(var n=Math.min(this.buffer.length-this.bufferOff,t.length-e),r=0;r<n;r++)this.buffer[this.bufferOff+r]=t[e+r];return this.bufferOff+=n,n},i.prototype._flushBuffer=function(t,e){return this._update(this.buffer,0,t,e),this.bufferOff=0,this.blockSize},i.prototype._updateEncrypt=function(t){var e=0,n=0,r=(this.bufferOff+t.length)/this.blockSize|0,i=new Array(r*this.blockSize);0!==this.bufferOff&&(e+=this._buffer(t,e),this.bufferOff===this.buffer.length&&(n+=this._flushBuffer(i,n)));for(var o=t.length-(t.length-e)%this.blockSize;e<o;e+=this.blockSize)this._update(t,e,i,n),n+=this.blockSize;for(;e<t.length;e++,this.bufferOff++)this.buffer[this.bufferOff]=t[e];return i},i.prototype._updateDecrypt=function(t){for(var e=0,n=0,r=Math.ceil((this.bufferOff+t.length)/this.blockSize)-1,i=new Array(r*this.blockSize);r>0;r--)e+=this._buffer(t,e),n+=this._flushBuffer(i,n);return e+=this._buffer(t,e),i},i.prototype.final=function(t){var e,n;return t&&(e=this.update(t)),n="encrypt"===this.type?this._finalEncrypt():this._finalDecrypt(),e?e.concat(n):n},i.prototype._pad=function(t,e){if(0===e)return!1;for(;e<t.length;)t[e++]=0;return!0},i.prototype._finalEncrypt=function(){if(!this._pad(this.buffer,this.bufferOff))return[];var t=new Array(this.blockSize);return this._update(this.buffer,0,t,0),t},i.prototype._unpad=function(t){return t},i.prototype._finalDecrypt=function(){r.equal(this.bufferOff,this.blockSize,"Not enough data to decrypt");var t=new Array(this.blockSize);return this._flushBuffer(t,0),this._unpad(t)}},function(t,e,n){"use strict";var r=n(77),i=n(30),o=n(274),a=o.utils,s=o.Cipher;function A(){this.tmp=new Array(2),this.keys=null}function l(t){s.call(this,t);var e=new A;this._desState=e,this.deriveKeys(e,t.key)}i(l,s),t.exports=l,l.create=function(t){return new l(t)};var c=[1,1,2,2,2,2,2,2,1,2,2,2,2,2,2,1];l.prototype.deriveKeys=function(t,e){t.keys=new Array(32),r.equal(e.length,this.blockSize,"Invalid key length");var n=a.readUInt32BE(e,0),i=a.readUInt32BE(e,4);a.pc1(n,i,t.tmp,0),n=t.tmp[0],i=t.tmp[1];for(var o=0;o<t.keys.length;o+=2){var s=c[o>>>1];n=a.r28shl(n,s),i=a.r28shl(i,s),a.pc2(n,i,t.keys,o)}},l.prototype._update=function(t,e,n,r){var i=this._desState,o=a.readUInt32BE(t,e),s=a.readUInt32BE(t,e+4);a.ip(o,s,i.tmp,0),o=i.tmp[0],s=i.tmp[1],"encrypt"===this.type?this._encrypt(i,o,s,i.tmp,0):this._decrypt(i,o,s,i.tmp,0),o=i.tmp[0],s=i.tmp[1],a.writeUInt32BE(n,o,r),a.writeUInt32BE(n,s,r+4)},l.prototype._pad=function(t,e){for(var n=t.length-e,r=e;r<t.length;r++)t[r]=n;return!0},l.prototype._unpad=function(t){for(var e=t[t.length-1],n=t.length-e;n<t.length;n++)r.equal(t[n],e);return t.slice(0,t.length-e)},l.prototype._encrypt=function(t,e,n,r,i){for(var o=e,s=n,A=0;A<t.keys.length;A+=2){var l=t.keys[A],c=t.keys[A+1];a.expand(s,t.tmp,0),l^=t.tmp[0],c^=t.tmp[1];var u=a.substitute(l,c),g=s;s=(o^a.permute(u))>>>0,o=g}a.rip(s,o,r,i)},l.prototype._decrypt=function(t,e,n,r,i){for(var o=n,s=e,A=t.keys.length-2;A>=0;A-=2){var l=t.keys[A],c=t.keys[A+1];a.expand(o,t.tmp,0),l^=t.tmp[0],c^=t.tmp[1];var u=a.substitute(l,c),g=o;o=(s^a.permute(u))>>>0,s=g}a.rip(o,s,r,i)}},function(t,e,n){"use strict";var r=n(77),i=n(30),o={};function a(t){r.equal(t.length,8,"Invalid IV length"),this.iv=new Array(8);for(var e=0;e<this.iv.length;e++)this.iv[e]=t[e]}e.instantiate=function(t){function e(e){t.call(this,e),this._cbcInit()}i(e,t);for(var n=Object.keys(o),r=0;r<n.length;r++){var a=n[r];e.prototype[a]=o[a]}return e.create=function(t){return new e(t)},e},o._cbcInit=function(){var t=new a(this.options.iv);this._cbcState=t},o._update=function(t,e,n,r){var i=this._cbcState,o=this.constructor.super_.prototype,a=i.iv;if("encrypt"===this.type){for(var s=0;s<this.blockSize;s++)a[s]^=t[e+s];o._update.call(this,a,0,n,r);for(s=0;s<this.blockSize;s++)a[s]=n[r+s]}else{o._update.call(this,t,e,n,r);for(s=0;s<this.blockSize;s++)n[r+s]^=a[s];for(s=0;s<this.blockSize;s++)a[s]=t[e+s]}}},function(t,e,n){"use strict";var r=n(77),i=n(30),o=n(274),a=o.Cipher,s=o.DES;function A(t,e){r.equal(e.length,24,"Invalid key length");var n=e.slice(0,8),i=e.slice(8,16),o=e.slice(16,24);this.ciphers="encrypt"===t?[s.create({type:"encrypt",key:n}),s.create({type:"decrypt",key:i}),s.create({type:"encrypt",key:o})]:[s.create({type:"decrypt",key:o}),s.create({type:"encrypt",key:i}),s.create({type:"decrypt",key:n})]}function l(t){a.call(this,t);var e=new A(this.type,this.options.key);this._edeState=e}i(l,a),t.exports=l,l.create=function(t){return new l(t)},l.prototype._update=function(t,e,n,r){var i=this._edeState;i.ciphers[0]._update(t,e,n,r),i.ciphers[1]._update(n,r,n,r),i.ciphers[2]._update(n,r,n,r)},l.prototype._pad=s.prototype._pad,l.prototype._unpad=s.prototype._unpad},function(t,e,n){var r=n(276),i=n(413),o=n(35).Buffer,a=n(414),s=n(105),A=n(211),l=n(212);function c(t,e,n){s.call(this),this._cache=new g,this._cipher=new A.AES(e),this._prev=o.from(n),this._mode=t,this._autopadding=!0}n(30)(c,s),c.prototype._update=function(t){var e,n;this._cache.add(t);for(var r=[];e=this._cache.get();)n=this._mode.encrypt(this,e),r.push(n);return o.concat(r)};var u=o.alloc(16,16);function g(){this.cache=o.allocUnsafe(0)}function h(t,e,n){var s=r[t.toLowerCase()];if(!s)throw new TypeError("invalid suite type");if("string"==typeof e&&(e=o.from(e)),e.length!==s.key/8)throw new TypeError("invalid key length "+e.length);if("string"==typeof n&&(n=o.from(n)),"GCM"!==s.mode&&n.length!==s.iv)throw new TypeError("invalid iv length "+n.length);return"stream"===s.type?new a(s.module,e,n):"auth"===s.type?new i(s.module,e,n):new c(s.module,e,n)}c.prototype._final=function(){var t=this._cache.flush();if(this._autopadding)return t=this._mode.encrypt(this,t),this._cipher.scrub(),t;if(!t.equals(u))throw this._cipher.scrub(),new Error("data not multiple of block length")},c.prototype.setAutoPadding=function(t){return this._autopadding=!!t,this},g.prototype.add=function(t){this.cache=o.concat([this.cache,t])},g.prototype.get=function(){if(this.cache.length>15){var t=this.cache.slice(0,16);return this.cache=this.cache.slice(16),t}return null},g.prototype.flush=function(){for(var t=16-this.cache.length,e=o.allocUnsafe(t),n=-1;++n<t;)e.writeUInt8(t,n);return o.concat([this.cache,e])},e.createCipheriv=h,e.createCipher=function(t,e){var n=r[t.toLowerCase()];if(!n)throw new TypeError("invalid suite type");var i=l(e,!1,n.key,n.iv);return h(t,i.key,i.iv)}},function(t,e){e.encrypt=function(t,e){return t._cipher.encryptBlock(e)},e.decrypt=function(t,e){return t._cipher.decryptBlock(e)}},function(t,e,n){var r=n(167);e.encrypt=function(t,e){var n=r(e,t._prev);return t._prev=t._cipher.encryptBlock(n),t._prev},e.decrypt=function(t,e){var n=t._prev;t._prev=e;var i=t._cipher.decryptBlock(e);return r(i,n)}},function(t,e,n){var r=n(35).Buffer,i=n(167);function o(t,e,n){var o=e.length,a=i(e,t._cache);return t._cache=t._cache.slice(o),t._prev=r.concat([t._prev,n?e:a]),a}e.encrypt=function(t,e,n){for(var i,a=r.allocUnsafe(0);e.length;){if(0===t._cache.length&&(t._cache=t._cipher.encryptBlock(t._prev),t._prev=r.allocUnsafe(0)),!(t._cache.length<=e.length)){a=r.concat([a,o(t,e,n)]);break}i=t._cache.length,a=r.concat([a,o(t,e.slice(0,i),n)]),e=e.slice(i)}return a}},function(t,e,n){var r=n(35).Buffer;function i(t,e,n){var i=t._cipher.encryptBlock(t._prev)[0]^e;return t._prev=r.concat([t._prev.slice(1),r.from([n?e:i])]),i}e.encrypt=function(t,e,n){for(var o=e.length,a=r.allocUnsafe(o),s=-1;++s<o;)a[s]=i(t,e[s],n);return a}},function(t,e,n){var r=n(35).Buffer;function i(t,e,n){for(var r,i,a=-1,s=0;++a<8;)r=e&1<<7-a?128:0,s+=(128&(i=t._cipher.encryptBlock(t._prev)[0]^r))>>a%8,t._prev=o(t._prev,n?r:i);return s}function o(t,e){var n=t.length,i=-1,o=r.allocUnsafe(t.length);for(t=r.concat([t,r.from([e])]);++i<n;)o[i]=t[i]<<1|t[i+1]>>7;return o}e.encrypt=function(t,e,n){for(var o=e.length,a=r.allocUnsafe(o),s=-1;++s<o;)a[s]=i(t,e[s],n);return a}},function(t,e,n){(function(t){var r=n(167);function i(t){return t._prev=t._cipher.encryptBlock(t._prev),t._prev}e.encrypt=function(e,n){for(;e._cache.length<n.length;)e._cache=t.concat([e._cache,i(e)]);var o=e._cache.slice(0,n.length);return e._cache=e._cache.slice(n.length),r(n,o)}}).call(this,n(42).Buffer)},function(t,e,n){var r=n(35).Buffer,i=r.alloc(16,0);function o(t){var e=r.allocUnsafe(16);return e.writeUInt32BE(t[0]>>>0,0),e.writeUInt32BE(t[1]>>>0,4),e.writeUInt32BE(t[2]>>>0,8),e.writeUInt32BE(t[3]>>>0,12),e}function a(t){this.h=t,this.state=r.alloc(16,0),this.cache=r.allocUnsafe(0)}a.prototype.ghash=function(t){for(var e=-1;++e<t.length;)this.state[e]^=t[e];this._multiply()},a.prototype._multiply=function(){for(var t,e,n,r=[(t=this.h).readUInt32BE(0),t.readUInt32BE(4),t.readUInt32BE(8),t.readUInt32BE(12)],i=[0,0,0,0],a=-1;++a<128;){for(0!=(this.state[~~(a/8)]&1<<7-a%8)&&(i[0]^=r[0],i[1]^=r[1],i[2]^=r[2],i[3]^=r[3]),n=0!=(1&r[3]),e=3;e>0;e--)r[e]=r[e]>>>1|(1&r[e-1])<<31;r[0]=r[0]>>>1,n&&(r[0]=r[0]^225<<24)}this.state=o(i)},a.prototype.update=function(t){var e;for(this.cache=r.concat([this.cache,t]);this.cache.length>=16;)e=this.cache.slice(0,16),this.cache=this.cache.slice(16),this.ghash(e)},a.prototype.final=function(t,e){return this.cache.length&&this.ghash(r.concat([this.cache,i],16)),this.ghash(o([0,t,0,e])),this.state},t.exports=a},function(t,e,n){var r=n(413),i=n(35).Buffer,o=n(276),a=n(414),s=n(105),A=n(211),l=n(212);function c(t,e,n){s.call(this),this._cache=new u,this._last=void 0,this._cipher=new A.AES(e),this._prev=i.from(n),this._mode=t,this._autopadding=!0}function u(){this.cache=i.allocUnsafe(0)}function g(t,e,n){var s=o[t.toLowerCase()];if(!s)throw new TypeError("invalid suite type");if("string"==typeof n&&(n=i.from(n)),"GCM"!==s.mode&&n.length!==s.iv)throw new TypeError("invalid iv length "+n.length);if("string"==typeof e&&(e=i.from(e)),e.length!==s.key/8)throw new TypeError("invalid key length "+e.length);return"stream"===s.type?new a(s.module,e,n,!0):"auth"===s.type?new r(s.module,e,n,!0):new c(s.module,e,n)}n(30)(c,s),c.prototype._update=function(t){var e,n;this._cache.add(t);for(var r=[];e=this._cache.get(this._autopadding);)n=this._mode.decrypt(this,e),r.push(n);return i.concat(r)},c.prototype._final=function(){var t=this._cache.flush();if(this._autopadding)return function(t){var e=t[15];if(e<1||e>16)throw new Error("unable to decrypt data");var n=-1;for(;++n<e;)if(t[n+(16-e)]!==e)throw new Error("unable to decrypt data");if(16===e)return;return t.slice(0,16-e)}(this._mode.decrypt(this,t));if(t)throw new Error("data not multiple of block length")},c.prototype.setAutoPadding=function(t){return this._autopadding=!!t,this},u.prototype.add=function(t){this.cache=i.concat([this.cache,t])},u.prototype.get=function(t){var e;if(t){if(this.cache.length>16)return e=this.cache.slice(0,16),this.cache=this.cache.slice(16),e}else if(this.cache.length>=16)return e=this.cache.slice(0,16),this.cache=this.cache.slice(16),e;return null},u.prototype.flush=function(){if(this.cache.length)return this.cache},e.createDecipher=function(t,e){var n=o[t.toLowerCase()];if(!n)throw new TypeError("invalid suite type");var r=l(e,!1,n.key,n.iv);return g(t,r.key,r.iv)},e.createDecipheriv=g},function(t,e){e["des-ecb"]={key:8,iv:0},e["des-cbc"]=e.des={key:8,iv:8},e["des-ede3-cbc"]=e.des3={key:24,iv:8},e["des-ede3"]={key:24,iv:0},e["des-ede-cbc"]={key:16,iv:8},e["des-ede"]={key:16,iv:0}},function(t,e,n){(function(t){var r=n(415),i=n(809),o=n(810);var a={binary:!0,hex:!0,base64:!0};e.DiffieHellmanGroup=e.createDiffieHellmanGroup=e.getDiffieHellman=function(e){var n=new t(i[e].prime,"hex"),r=new t(i[e].gen,"hex");return new o(n,r)},e.createDiffieHellman=e.DiffieHellman=function e(n,i,s,A){return t.isBuffer(i)||void 0===a[i]?e(n,"binary",i,s):(i=i||"binary",A=A||"binary",s=s||new t([2]),t.isBuffer(s)||(s=new t(s,A)),"number"==typeof n?new o(r(n,s),s,!0):(t.isBuffer(n)||(n=new t(n,i)),new o(n,s,!0)))}}).call(this,n(42).Buffer)},function(t,e){},function(t,e){},function(t){t.exports=JSON.parse('{"modp1":{"gen":"02","prime":"ffffffffffffffffc90fdaa22168c234c4c6628b80dc1cd129024e088a67cc74020bbea63b139b22514a08798e3404ddef9519b3cd3a431b302b0a6df25f14374fe1356d6d51c245e485b576625e7ec6f44c42e9a63a3620ffffffffffffffff"},"modp2":{"gen":"02","prime":"ffffffffffffffffc90fdaa22168c234c4c6628b80dc1cd129024e088a67cc74020bbea63b139b22514a08798e3404ddef9519b3cd3a431b302b0a6df25f14374fe1356d6d51c245e485b576625e7ec6f44c42e9a637ed6b0bff5cb6f406b7edee386bfb5a899fa5ae9f24117c4b1fe649286651ece65381ffffffffffffffff"},"modp5":{"gen":"02","prime":"ffffffffffffffffc90fdaa22168c234c4c6628b80dc1cd129024e088a67cc74020bbea63b139b22514a08798e3404ddef9519b3cd3a431b302b0a6df25f14374fe1356d6d51c245e485b576625e7ec6f44c42e9a637ed6b0bff5cb6f406b7edee386bfb5a899fa5ae9f24117c4b1fe649286651ece45b3dc2007cb8a163bf0598da48361c55d39a69163fa8fd24cf5f83655d23dca3ad961c62f356208552bb9ed529077096966d670c354e4abc9804f1746c08ca237327ffffffffffffffff"},"modp14":{"gen":"02","prime":"ffffffffffffffffc90fdaa22168c234c4c6628b80dc1cd129024e088a67cc74020bbea63b139b22514a08798e3404ddef9519b3cd3a431b302b0a6df25f14374fe1356d6d51c245e485b576625e7ec6f44c42e9a637ed6b0bff5cb6f406b7edee386bfb5a899fa5ae9f24117c4b1fe649286651ece45b3dc2007cb8a163bf0598da48361c55d39a69163fa8fd24cf5f83655d23dca3ad961c62f356208552bb9ed529077096966d670c354e4abc9804f1746c08ca18217c32905e462e36ce3be39e772c180e86039b2783a2ec07a28fb5c55df06f4c52c9de2bcbf6955817183995497cea956ae515d2261898fa051015728e5a8aacaa68ffffffffffffffff"},"modp15":{"gen":"02","prime":"ffffffffffffffffc90fdaa22168c234c4c6628b80dc1cd129024e088a67cc74020bbea63b139b22514a08798e3404ddef9519b3cd3a431b302b0a6df25f14374fe1356d6d51c245e485b576625e7ec6f44c42e9a637ed6b0bff5cb6f406b7edee386bfb5a899fa5ae9f24117c4b1fe649286651ece45b3dc2007cb8a163bf0598da48361c55d39a69163fa8fd24cf5f83655d23dca3ad961c62f356208552bb9ed529077096966d670c354e4abc9804f1746c08ca18217c32905e462e36ce3be39e772c180e86039b2783a2ec07a28fb5c55df06f4c52c9de2bcbf6955817183995497cea956ae515d2261898fa051015728e5a8aaac42dad33170d04507a33a85521abdf1cba64ecfb850458dbef0a8aea71575d060c7db3970f85a6e1e4c7abf5ae8cdb0933d71e8c94e04a25619dcee3d2261ad2ee6bf12ffa06d98a0864d87602733ec86a64521f2b18177b200cbbe117577a615d6c770988c0bad946e208e24fa074e5ab3143db5bfce0fd108e4b82d120a93ad2caffffffffffffffff"},"modp16":{"gen":"02","prime":"ffffffffffffffffc90fdaa22168c234c4c6628b80dc1cd129024e088a67cc74020bbea63b139b22514a08798e3404ddef9519b3cd3a431b302b0a6df25f14374fe1356d6d51c245e485b576625e7ec6f44c42e9a637ed6b0bff5cb6f406b7edee386bfb5a899fa5ae9f24117c4b1fe649286651ece45b3dc2007cb8a163bf0598da48361c55d39a69163fa8fd24cf5f83655d23dca3ad961c62f356208552bb9ed529077096966d670c354e4abc9804f1746c08ca18217c32905e462e36ce3be39e772c180e86039b2783a2ec07a28fb5c55df06f4c52c9de2bcbf6955817183995497cea956ae515d2261898fa051015728e5a8aaac42dad33170d04507a33a85521abdf1cba64ecfb850458dbef0a8aea71575d060c7db3970f85a6e1e4c7abf5ae8cdb0933d71e8c94e04a25619dcee3d2261ad2ee6bf12ffa06d98a0864d87602733ec86a64521f2b18177b200cbbe117577a615d6c770988c0bad946e208e24fa074e5ab3143db5bfce0fd108e4b82d120a92108011a723c12a787e6d788719a10bdba5b2699c327186af4e23c1a946834b6150bda2583e9ca2ad44ce8dbbbc2db04de8ef92e8efc141fbecaa6287c59474e6bc05d99b2964fa090c3a2233ba186515be7ed1f612970cee2d7afb81bdd762170481cd0069127d5b05aa993b4ea988d8fddc186ffb7dc90a6c08f4df435c934063199ffffffffffffffff"},"modp17":{"gen":"02","prime":"ffffffffffffffffc90fdaa22168c234c4c6628b80dc1cd129024e088a67cc74020bbea63b139b22514a08798e3404ddef9519b3cd3a431b302b0a6df25f14374fe1356d6d51c245e485b576625e7ec6f44c42e9a637ed6b0bff5cb6f406b7edee386bfb5a899fa5ae9f24117c4b1fe649286651ece45b3dc2007cb8a163bf0598da48361c55d39a69163fa8fd24cf5f83655d23dca3ad961c62f356208552bb9ed529077096966d670c354e4abc9804f1746c08ca18217c32905e462e36ce3be39e772c180e86039b2783a2ec07a28fb5c55df06f4c52c9de2bcbf6955817183995497cea956ae515d2261898fa051015728e5a8aaac42dad33170d04507a33a85521abdf1cba64ecfb850458dbef0a8aea71575d060c7db3970f85a6e1e4c7abf5ae8cdb0933d71e8c94e04a25619dcee3d2261ad2ee6bf12ffa06d98a0864d87602733ec86a64521f2b18177b200cbbe117577a615d6c770988c0bad946e208e24fa074e5ab3143db5bfce0fd108e4b82d120a92108011a723c12a787e6d788719a10bdba5b2699c327186af4e23c1a946834b6150bda2583e9ca2ad44ce8dbbbc2db04de8ef92e8efc141fbecaa6287c59474e6bc05d99b2964fa090c3a2233ba186515be7ed1f612970cee2d7afb81bdd762170481cd0069127d5b05aa993b4ea988d8fddc186ffb7dc90a6c08f4df435c93402849236c3fab4d27c7026c1d4dcb2602646dec9751e763dba37bdf8ff9406ad9e530ee5db382f413001aeb06a53ed9027d831179727b0865a8918da3edbebcf9b14ed44ce6cbaced4bb1bdb7f1447e6cc254b332051512bd7af426fb8f401378cd2bf5983ca01c64b92ecf032ea15d1721d03f482d7ce6e74fef6d55e702f46980c82b5a84031900b1c9e59e7c97fbec7e8f323a97a7e36cc88be0f1d45b7ff585ac54bd407b22b4154aacc8f6d7ebf48e1d814cc5ed20f8037e0a79715eef29be32806a1d58bb7c5da76f550aa3d8a1fbff0eb19ccb1a313d55cda56c9ec2ef29632387fe8d76e3c0468043e8f663f4860ee12bf2d5b0b7474d6e694f91e6dcc4024ffffffffffffffff"},"modp18":{"gen":"02","prime":"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"}}')},function(t,e,n){(function(e){var r=n(53),i=new(n(416)),o=new r(24),a=new r(11),s=new r(10),A=new r(3),l=new r(7),c=n(415),u=n(135);function g(t,n){return n=n||"utf8",e.isBuffer(t)||(t=new e(t,n)),this._pub=new r(t),this}function h(t,n){return n=n||"utf8",e.isBuffer(t)||(t=new e(t,n)),this._priv=new r(t),this}t.exports=f;var d={};function f(t,e,n){this.setGenerator(e),this.__prime=new r(t),this._prime=r.mont(this.__prime),this._primeLen=t.length,this._pub=void 0,this._priv=void 0,this._primeCode=void 0,n?(this.setPublicKey=g,this.setPrivateKey=h):this._primeCode=8}function p(t,n){var r=new e(t.toArray());return n?r.toString(n):r}Object.defineProperty(f.prototype,"verifyError",{enumerable:!0,get:function(){return"number"!=typeof this._primeCode&&(this._primeCode=function(t,e){var n=e.toString("hex"),r=[n,t.toString(16)].join("_");if(r in d)return d[r];var u,g=0;if(t.isEven()||!c.simpleSieve||!c.fermatTest(t)||!i.test(t))return g+=1,g+="02"===n||"05"===n?8:4,d[r]=g,g;switch(i.test(t.shrn(1))||(g+=2),n){case"02":t.mod(o).cmp(a)&&(g+=8);break;case"05":(u=t.mod(s)).cmp(A)&&u.cmp(l)&&(g+=8);break;default:g+=4}return d[r]=g,g}(this.__prime,this.__gen)),this._primeCode}}),f.prototype.generateKeys=function(){return this._priv||(this._priv=new r(u(this._primeLen))),this._pub=this._gen.toRed(this._prime).redPow(this._priv).fromRed(),this.getPublicKey()},f.prototype.computeSecret=function(t){var n=(t=(t=new r(t)).toRed(this._prime)).redPow(this._priv).fromRed(),i=new e(n.toArray()),o=this.getPrime();if(i.length<o.length){var a=new e(o.length-i.length);a.fill(0),i=e.concat([a,i])}return i},f.prototype.getPublicKey=function(t){return p(this._pub,t)},f.prototype.getPrivateKey=function(t){return p(this._priv,t)},f.prototype.getPrime=function(t){return p(this.__prime,t)},f.prototype.getGenerator=function(t){return p(this._gen,t)},f.prototype.setGenerator=function(t,n){return n=n||"utf8",e.isBuffer(t)||(t=new e(t,n)),this.__gen=t,this._gen=new r(t),this}}).call(this,n(42).Buffer)},function(t,e,n){(function(e){var r=n(165),i=n(268),o=n(30),a=n(812),s=n(844),A=n(405);function l(t){i.Writable.call(this);var e=A[t];if(!e)throw new Error("Unknown message digest");this._hashType=e.hash,this._hash=r(e.hash),this._tag=e.id,this._signType=e.sign}function c(t){i.Writable.call(this);var e=A[t];if(!e)throw new Error("Unknown message digest");this._hash=r(e.hash),this._tag=e.id,this._signType=e.sign}function u(t){return new l(t)}function g(t){return new c(t)}Object.keys(A).forEach(function(t){A[t].id=new e(A[t].id,"hex"),A[t.toLowerCase()]=A[t]}),o(l,i.Writable),l.prototype._write=function(t,e,n){this._hash.update(t),n()},l.prototype.update=function(t,n){return"string"==typeof t&&(t=new e(t,n)),this._hash.update(t),this},l.prototype.sign=function(t,e){this.end();var n=this._hash.digest(),r=a(n,t,this._hashType,this._signType,this._tag);return e?r.toString(e):r},o(c,i.Writable),c.prototype._write=function(t,e,n){this._hash.update(t),n()},c.prototype.update=function(t,n){return"string"==typeof t&&(t=new e(t,n)),this._hash.update(t),this},c.prototype.verify=function(t,n,r){"string"==typeof n&&(n=new e(n,r)),this.end();var i=this._hash.digest();return s(n,i,t,this._signType,this._tag)},t.exports={Sign:u,Verify:g,createSign:u,createVerify:g}}).call(this,n(42).Buffer)},function(t,e,n){(function(e){var r=n(403),i=n(278),o=n(279).ec,a=n(53),s=n(214),A=n(426);function l(t,n,i,o){if((t=new e(t.toArray())).length<n.byteLength()){var a=new e(n.byteLength()-t.length);a.fill(0),t=e.concat([a,t])}var s=i.length,A=function(t,n){t=(t=c(t,n)).mod(n);var r=new e(t.toArray());if(r.length<n.byteLength()){var i=new e(n.byteLength()-r.length);i.fill(0),r=e.concat([i,r])}return r}(i,n),l=new e(s);l.fill(1);var u=new e(s);return u.fill(0),u=r(o,u).update(l).update(new e([0])).update(t).update(A).digest(),l=r(o,u).update(l).digest(),{k:u=r(o,u).update(l).update(new e([1])).update(t).update(A).digest(),v:l=r(o,u).update(l).digest()}}function c(t,e){var n=new a(t),r=(t.length<<3)-e.bitLength();return r>0&&n.ishrn(r),n}function u(t,n,i){var o,a;do{for(o=new e(0);8*o.length<t.bitLength();)n.v=r(i,n.k).update(n.v).digest(),o=e.concat([o,n.v]);a=c(o,t),n.k=r(i,n.k).update(n.v).update(new e([0])).digest(),n.v=r(i,n.k).update(n.v).digest()}while(-1!==a.cmp(t));return a}function g(t,e,n,r){return t.toRed(a.mont(n)).redPow(e).fromRed().mod(r)}t.exports=function(t,n,r,h,d){var f=s(n);if(f.curve){if("ecdsa"!==h&&"ecdsa/rsa"!==h)throw new Error("wrong private key type");return function(t,n){var r=A[n.curve.join(".")];if(!r)throw new Error("unknown curve "+n.curve.join("."));var i=new o(r).keyFromPrivate(n.privateKey).sign(t);return new e(i.toDER())}(t,f)}if("dsa"===f.type){if("dsa"!==h)throw new Error("wrong private key type");return function(t,n,r){for(var i,o=n.params.priv_key,s=n.params.p,A=n.params.q,h=n.params.g,d=new a(0),f=c(t,A).mod(A),p=!1,I=l(o,A,t,r);!1===p;)i=u(A,I,r),d=g(h,i,s,A),0===(p=i.invm(A).imul(f.add(o.mul(d))).mod(A)).cmpn(0)&&(p=!1,d=new a(0));return function(t,n){t=t.toArray(),n=n.toArray(),128&t[0]&&(t=[0].concat(t)),128&n[0]&&(n=[0].concat(n));var r=[48,t.length+n.length+4,2,t.length];return r=r.concat(t,[2,n.length],n),new e(r)}(d,p)}(t,f,r)}if("rsa"!==h&&"ecdsa/rsa"!==h)throw new Error("wrong private key type");t=e.concat([d,t]);for(var p=f.modulus.byteLength(),I=[0,1];t.length+I.length+1<p;)I.push(255);I.push(0);for(var C=-1;++C<t.length;)I.push(t[C]);return i(I,f)},t.exports.getKey=l,t.exports.makeKey=u}).call(this,n(42).Buffer)},function(t){t.exports=JSON.parse('{"name":"elliptic","version":"6.5.0","description":"EC cryptography","main":"lib/elliptic.js","files":["lib"],"scripts":{"jscs":"jscs benchmarks/*.js lib/*.js lib/**/*.js lib/**/**/*.js test/index.js","jshint":"jscs benchmarks/*.js lib/*.js lib/**/*.js lib/**/**/*.js test/index.js","lint":"npm run jscs && npm run jshint","unit":"istanbul test _mocha --reporter=spec test/index.js","test":"npm run lint && npm run unit","version":"grunt dist && git add dist/"},"repository":{"type":"git","url":"git@github.com:indutny/elliptic"},"keywords":["EC","Elliptic","curve","Cryptography"],"author":"Fedor Indutny <fedor@indutny.com>","license":"MIT","bugs":{"url":"https://github.com/indutny/elliptic/issues"},"homepage":"https://github.com/indutny/elliptic","devDependencies":{"brfs":"^1.4.3","coveralls":"^2.11.3","grunt":"^0.4.5","grunt-browserify":"^5.0.0","grunt-cli":"^1.2.0","grunt-contrib-connect":"^1.0.0","grunt-contrib-copy":"^1.0.0","grunt-contrib-uglify":"^1.0.1","grunt-mocha-istanbul":"^3.0.1","grunt-saucelabs":"^8.6.2","istanbul":"^0.4.2","jscs":"^2.9.0","jshint":"^2.6.0","mocha":"^2.1.0"},"dependencies":{"bn.js":"^4.4.0","brorand":"^1.0.1","hash.js":"^1.0.0","hmac-drbg":"^1.0.0","inherits":"^2.0.1","minimalistic-assert":"^1.0.0","minimalistic-crypto-utils":"^1.0.0"}}')},function(t,e,n){"use strict";var r=n(78),i=n(53),o=n(30),a=n(213),s=r.assert;function A(t){a.call(this,"short",t),this.a=new i(t.a,16).toRed(this.red),this.b=new i(t.b,16).toRed(this.red),this.tinv=this.two.redInvm(),this.zeroA=0===this.a.fromRed().cmpn(0),this.threeA=0===this.a.fromRed().sub(this.p).cmpn(-3),this.endo=this._getEndomorphism(t),this._endoWnafT1=new Array(4),this._endoWnafT2=new Array(4)}function l(t,e,n,r){a.BasePoint.call(this,t,"affine"),null===e&&null===n?(this.x=null,this.y=null,this.inf=!0):(this.x=new i(e,16),this.y=new i(n,16),r&&(this.x.forceRed(this.curve.red),this.y.forceRed(this.curve.red)),this.x.red||(this.x=this.x.toRed(this.curve.red)),this.y.red||(this.y=this.y.toRed(this.curve.red)),this.inf=!1)}function c(t,e,n,r){a.BasePoint.call(this,t,"jacobian"),null===e&&null===n&&null===r?(this.x=this.curve.one,this.y=this.curve.one,this.z=new i(0)):(this.x=new i(e,16),this.y=new i(n,16),this.z=new i(r,16)),this.x.red||(this.x=this.x.toRed(this.curve.red)),this.y.red||(this.y=this.y.toRed(this.curve.red)),this.z.red||(this.z=this.z.toRed(this.curve.red)),this.zOne=this.z===this.curve.one}o(A,a),t.exports=A,A.prototype._getEndomorphism=function(t){if(this.zeroA&&this.g&&this.n&&1===this.p.modn(3)){var e,n;if(t.beta)e=new i(t.beta,16).toRed(this.red);else{var r=this._getEndoRoots(this.p);e=(e=r[0].cmp(r[1])<0?r[0]:r[1]).toRed(this.red)}if(t.lambda)n=new i(t.lambda,16);else{var o=this._getEndoRoots(this.n);0===this.g.mul(o[0]).x.cmp(this.g.x.redMul(e))?n=o[0]:(n=o[1],s(0===this.g.mul(n).x.cmp(this.g.x.redMul(e))))}return{beta:e,lambda:n,basis:t.basis?t.basis.map(function(t){return{a:new i(t.a,16),b:new i(t.b,16)}}):this._getEndoBasis(n)}}},A.prototype._getEndoRoots=function(t){var e=t===this.p?this.red:i.mont(t),n=new i(2).toRed(e).redInvm(),r=n.redNeg(),o=new i(3).toRed(e).redNeg().redSqrt().redMul(n);return[r.redAdd(o).fromRed(),r.redSub(o).fromRed()]},A.prototype._getEndoBasis=function(t){for(var e,n,r,o,a,s,A,l,c,u=this.n.ushrn(Math.floor(this.n.bitLength()/2)),g=t,h=this.n.clone(),d=new i(1),f=new i(0),p=new i(0),I=new i(1),C=0;0!==g.cmpn(0);){var m=h.div(g);l=h.sub(m.mul(g)),c=p.sub(m.mul(d));var y=I.sub(m.mul(f));if(!r&&l.cmp(u)<0)e=A.neg(),n=d,r=l.neg(),o=c;else if(r&&2==++C)break;A=l,h=g,g=l,p=d,d=c,I=f,f=y}a=l.neg(),s=c;var E=r.sqr().add(o.sqr());return a.sqr().add(s.sqr()).cmp(E)>=0&&(a=e,s=n),r.negative&&(r=r.neg(),o=o.neg()),a.negative&&(a=a.neg(),s=s.neg()),[{a:r,b:o},{a:a,b:s}]},A.prototype._endoSplit=function(t){var e=this.endo.basis,n=e[0],r=e[1],i=r.b.mul(t).divRound(this.n),o=n.b.neg().mul(t).divRound(this.n),a=i.mul(n.a),s=o.mul(r.a),A=i.mul(n.b),l=o.mul(r.b);return{k1:t.sub(a).sub(s),k2:A.add(l).neg()}},A.prototype.pointFromX=function(t,e){(t=new i(t,16)).red||(t=t.toRed(this.red));var n=t.redSqr().redMul(t).redIAdd(t.redMul(this.a)).redIAdd(this.b),r=n.redSqrt();if(0!==r.redSqr().redSub(n).cmp(this.zero))throw new Error("invalid point");var o=r.fromRed().isOdd();return(e&&!o||!e&&o)&&(r=r.redNeg()),this.point(t,r)},A.prototype.validate=function(t){if(t.inf)return!0;var e=t.x,n=t.y,r=this.a.redMul(e),i=e.redSqr().redMul(e).redIAdd(r).redIAdd(this.b);return 0===n.redSqr().redISub(i).cmpn(0)},A.prototype._endoWnafMulAdd=function(t,e,n){for(var r=this._endoWnafT1,i=this._endoWnafT2,o=0;o<t.length;o++){var a=this._endoSplit(e[o]),s=t[o],A=s._getBeta();a.k1.negative&&(a.k1.ineg(),s=s.neg(!0)),a.k2.negative&&(a.k2.ineg(),A=A.neg(!0)),r[2*o]=s,r[2*o+1]=A,i[2*o]=a.k1,i[2*o+1]=a.k2}for(var l=this._wnafMulAdd(1,r,i,2*o,n),c=0;c<2*o;c++)r[c]=null,i[c]=null;return l},o(l,a.BasePoint),A.prototype.point=function(t,e,n){return new l(this,t,e,n)},A.prototype.pointFromJSON=function(t,e){return l.fromJSON(this,t,e)},l.prototype._getBeta=function(){if(this.curve.endo){var t=this.precomputed;if(t&&t.beta)return t.beta;var e=this.curve.point(this.x.redMul(this.curve.endo.beta),this.y);if(t){var n=this.curve,r=function(t){return n.point(t.x.redMul(n.endo.beta),t.y)};t.beta=e,e.precomputed={beta:null,naf:t.naf&&{wnd:t.naf.wnd,points:t.naf.points.map(r)},doubles:t.doubles&&{step:t.doubles.step,points:t.doubles.points.map(r)}}}return e}},l.prototype.toJSON=function(){return this.precomputed?[this.x,this.y,this.precomputed&&{doubles:this.precomputed.doubles&&{step:this.precomputed.doubles.step,points:this.precomputed.doubles.points.slice(1)},naf:this.precomputed.naf&&{wnd:this.precomputed.naf.wnd,points:this.precomputed.naf.points.slice(1)}}]:[this.x,this.y]},l.fromJSON=function(t,e,n){"string"==typeof e&&(e=JSON.parse(e));var r=t.point(e[0],e[1],n);if(!e[2])return r;function i(e){return t.point(e[0],e[1],n)}var o=e[2];return r.precomputed={beta:null,doubles:o.doubles&&{step:o.doubles.step,points:[r].concat(o.doubles.points.map(i))},naf:o.naf&&{wnd:o.naf.wnd,points:[r].concat(o.naf.points.map(i))}},r},l.prototype.inspect=function(){return this.isInfinity()?"<EC Point Infinity>":"<EC Point x: "+this.x.fromRed().toString(16,2)+" y: "+this.y.fromRed().toString(16,2)+">"},l.prototype.isInfinity=function(){return this.inf},l.prototype.add=function(t){if(this.inf)return t;if(t.inf)return this;if(this.eq(t))return this.dbl();if(this.neg().eq(t))return this.curve.point(null,null);if(0===this.x.cmp(t.x))return this.curve.point(null,null);var e=this.y.redSub(t.y);0!==e.cmpn(0)&&(e=e.redMul(this.x.redSub(t.x).redInvm()));var n=e.redSqr().redISub(this.x).redISub(t.x),r=e.redMul(this.x.redSub(n)).redISub(this.y);return this.curve.point(n,r)},l.prototype.dbl=function(){if(this.inf)return this;var t=this.y.redAdd(this.y);if(0===t.cmpn(0))return this.curve.point(null,null);var e=this.curve.a,n=this.x.redSqr(),r=t.redInvm(),i=n.redAdd(n).redIAdd(n).redIAdd(e).redMul(r),o=i.redSqr().redISub(this.x.redAdd(this.x)),a=i.redMul(this.x.redSub(o)).redISub(this.y);return this.curve.point(o,a)},l.prototype.getX=function(){return this.x.fromRed()},l.prototype.getY=function(){return this.y.fromRed()},l.prototype.mul=function(t){return t=new i(t,16),this._hasDoubles(t)?this.curve._fixedNafMul(this,t):this.curve.endo?this.curve._endoWnafMulAdd([this],[t]):this.curve._wnafMul(this,t)},l.prototype.mulAdd=function(t,e,n){var r=[this,e],i=[t,n];return this.curve.endo?this.curve._endoWnafMulAdd(r,i):this.curve._wnafMulAdd(1,r,i,2)},l.prototype.jmulAdd=function(t,e,n){var r=[this,e],i=[t,n];return this.curve.endo?this.curve._endoWnafMulAdd(r,i,!0):this.curve._wnafMulAdd(1,r,i,2,!0)},l.prototype.eq=function(t){return this===t||this.inf===t.inf&&(this.inf||0===this.x.cmp(t.x)&&0===this.y.cmp(t.y))},l.prototype.neg=function(t){if(this.inf)return this;var e=this.curve.point(this.x,this.y.redNeg());if(t&&this.precomputed){var n=this.precomputed,r=function(t){return t.neg()};e.precomputed={naf:n.naf&&{wnd:n.naf.wnd,points:n.naf.points.map(r)},doubles:n.doubles&&{step:n.doubles.step,points:n.doubles.points.map(r)}}}return e},l.prototype.toJ=function(){return this.inf?this.curve.jpoint(null,null,null):this.curve.jpoint(this.x,this.y,this.curve.one)},o(c,a.BasePoint),A.prototype.jpoint=function(t,e,n){return new c(this,t,e,n)},c.prototype.toP=function(){if(this.isInfinity())return this.curve.point(null,null);var t=this.z.redInvm(),e=t.redSqr(),n=this.x.redMul(e),r=this.y.redMul(e).redMul(t);return this.curve.point(n,r)},c.prototype.neg=function(){return this.curve.jpoint(this.x,this.y.redNeg(),this.z)},c.prototype.add=function(t){if(this.isInfinity())return t;if(t.isInfinity())return this;var e=t.z.redSqr(),n=this.z.redSqr(),r=this.x.redMul(e),i=t.x.redMul(n),o=this.y.redMul(e.redMul(t.z)),a=t.y.redMul(n.redMul(this.z)),s=r.redSub(i),A=o.redSub(a);if(0===s.cmpn(0))return 0!==A.cmpn(0)?this.curve.jpoint(null,null,null):this.dbl();var l=s.redSqr(),c=l.redMul(s),u=r.redMul(l),g=A.redSqr().redIAdd(c).redISub(u).redISub(u),h=A.redMul(u.redISub(g)).redISub(o.redMul(c)),d=this.z.redMul(t.z).redMul(s);return this.curve.jpoint(g,h,d)},c.prototype.mixedAdd=function(t){if(this.isInfinity())return t.toJ();if(t.isInfinity())return this;var e=this.z.redSqr(),n=this.x,r=t.x.redMul(e),i=this.y,o=t.y.redMul(e).redMul(this.z),a=n.redSub(r),s=i.redSub(o);if(0===a.cmpn(0))return 0!==s.cmpn(0)?this.curve.jpoint(null,null,null):this.dbl();var A=a.redSqr(),l=A.redMul(a),c=n.redMul(A),u=s.redSqr().redIAdd(l).redISub(c).redISub(c),g=s.redMul(c.redISub(u)).redISub(i.redMul(l)),h=this.z.redMul(a);return this.curve.jpoint(u,g,h)},c.prototype.dblp=function(t){if(0===t)return this;if(this.isInfinity())return this;if(!t)return this.dbl();if(this.curve.zeroA||this.curve.threeA){for(var e=this,n=0;n<t;n++)e=e.dbl();return e}var r=this.curve.a,i=this.curve.tinv,o=this.x,a=this.y,s=this.z,A=s.redSqr().redSqr(),l=a.redAdd(a);for(n=0;n<t;n++){var c=o.redSqr(),u=l.redSqr(),g=u.redSqr(),h=c.redAdd(c).redIAdd(c).redIAdd(r.redMul(A)),d=o.redMul(u),f=h.redSqr().redISub(d.redAdd(d)),p=d.redISub(f),I=h.redMul(p);I=I.redIAdd(I).redISub(g);var C=l.redMul(s);n+1<t&&(A=A.redMul(g)),o=f,s=C,l=I}return this.curve.jpoint(o,l.redMul(i),s)},c.prototype.dbl=function(){return this.isInfinity()?this:this.curve.zeroA?this._zeroDbl():this.curve.threeA?this._threeDbl():this._dbl()},c.prototype._zeroDbl=function(){var t,e,n;if(this.zOne){var r=this.x.redSqr(),i=this.y.redSqr(),o=i.redSqr(),a=this.x.redAdd(i).redSqr().redISub(r).redISub(o);a=a.redIAdd(a);var s=r.redAdd(r).redIAdd(r),A=s.redSqr().redISub(a).redISub(a),l=o.redIAdd(o);l=(l=l.redIAdd(l)).redIAdd(l),t=A,e=s.redMul(a.redISub(A)).redISub(l),n=this.y.redAdd(this.y)}else{var c=this.x.redSqr(),u=this.y.redSqr(),g=u.redSqr(),h=this.x.redAdd(u).redSqr().redISub(c).redISub(g);h=h.redIAdd(h);var d=c.redAdd(c).redIAdd(c),f=d.redSqr(),p=g.redIAdd(g);p=(p=p.redIAdd(p)).redIAdd(p),t=f.redISub(h).redISub(h),e=d.redMul(h.redISub(t)).redISub(p),n=(n=this.y.redMul(this.z)).redIAdd(n)}return this.curve.jpoint(t,e,n)},c.prototype._threeDbl=function(){var t,e,n;if(this.zOne){var r=this.x.redSqr(),i=this.y.redSqr(),o=i.redSqr(),a=this.x.redAdd(i).redSqr().redISub(r).redISub(o);a=a.redIAdd(a);var s=r.redAdd(r).redIAdd(r).redIAdd(this.curve.a),A=s.redSqr().redISub(a).redISub(a);t=A;var l=o.redIAdd(o);l=(l=l.redIAdd(l)).redIAdd(l),e=s.redMul(a.redISub(A)).redISub(l),n=this.y.redAdd(this.y)}else{var c=this.z.redSqr(),u=this.y.redSqr(),g=this.x.redMul(u),h=this.x.redSub(c).redMul(this.x.redAdd(c));h=h.redAdd(h).redIAdd(h);var d=g.redIAdd(g),f=(d=d.redIAdd(d)).redAdd(d);t=h.redSqr().redISub(f),n=this.y.redAdd(this.z).redSqr().redISub(u).redISub(c);var p=u.redSqr();p=(p=(p=p.redIAdd(p)).redIAdd(p)).redIAdd(p),e=h.redMul(d.redISub(t)).redISub(p)}return this.curve.jpoint(t,e,n)},c.prototype._dbl=function(){var t=this.curve.a,e=this.x,n=this.y,r=this.z,i=r.redSqr().redSqr(),o=e.redSqr(),a=n.redSqr(),s=o.redAdd(o).redIAdd(o).redIAdd(t.redMul(i)),A=e.redAdd(e),l=(A=A.redIAdd(A)).redMul(a),c=s.redSqr().redISub(l.redAdd(l)),u=l.redISub(c),g=a.redSqr();g=(g=(g=g.redIAdd(g)).redIAdd(g)).redIAdd(g);var h=s.redMul(u).redISub(g),d=n.redAdd(n).redMul(r);return this.curve.jpoint(c,h,d)},c.prototype.trpl=function(){if(!this.curve.zeroA)return this.dbl().add(this);var t=this.x.redSqr(),e=this.y.redSqr(),n=this.z.redSqr(),r=e.redSqr(),i=t.redAdd(t).redIAdd(t),o=i.redSqr(),a=this.x.redAdd(e).redSqr().redISub(t).redISub(r),s=(a=(a=(a=a.redIAdd(a)).redAdd(a).redIAdd(a)).redISub(o)).redSqr(),A=r.redIAdd(r);A=(A=(A=A.redIAdd(A)).redIAdd(A)).redIAdd(A);var l=i.redIAdd(a).redSqr().redISub(o).redISub(s).redISub(A),c=e.redMul(l);c=(c=c.redIAdd(c)).redIAdd(c);var u=this.x.redMul(s).redISub(c);u=(u=u.redIAdd(u)).redIAdd(u);var g=this.y.redMul(l.redMul(A.redISub(l)).redISub(a.redMul(s)));g=(g=(g=g.redIAdd(g)).redIAdd(g)).redIAdd(g);var h=this.z.redAdd(a).redSqr().redISub(n).redISub(s);return this.curve.jpoint(u,g,h)},c.prototype.mul=function(t,e){return t=new i(t,e),this.curve._wnafMul(this,t)},c.prototype.eq=function(t){if("affine"===t.type)return this.eq(t.toJ());if(this===t)return!0;var e=this.z.redSqr(),n=t.z.redSqr();if(0!==this.x.redMul(n).redISub(t.x.redMul(e)).cmpn(0))return!1;var r=e.redMul(this.z),i=n.redMul(t.z);return 0===this.y.redMul(i).redISub(t.y.redMul(r)).cmpn(0)},c.prototype.eqXToP=function(t){var e=this.z.redSqr(),n=t.toRed(this.curve.red).redMul(e);if(0===this.x.cmp(n))return!0;for(var r=t.clone(),i=this.curve.redN.redMul(e);;){if(r.iadd(this.curve.n),r.cmp(this.curve.p)>=0)return!1;if(n.redIAdd(i),0===this.x.cmp(n))return!0}},c.prototype.inspect=function(){return this.isInfinity()?"<EC JPoint Infinity>":"<EC JPoint x: "+this.x.toString(16,2)+" y: "+this.y.toString(16,2)+" z: "+this.z.toString(16,2)+">"},c.prototype.isInfinity=function(){return 0===this.z.cmpn(0)}},function(t,e,n){"use strict";var r=n(53),i=n(30),o=n(213),a=n(78);function s(t){o.call(this,"mont",t),this.a=new r(t.a,16).toRed(this.red),this.b=new r(t.b,16).toRed(this.red),this.i4=new r(4).toRed(this.red).redInvm(),this.two=new r(2).toRed(this.red),this.a24=this.i4.redMul(this.a.redAdd(this.two))}function A(t,e,n){o.BasePoint.call(this,t,"projective"),null===e&&null===n?(this.x=this.curve.one,this.z=this.curve.zero):(this.x=new r(e,16),this.z=new r(n,16),this.x.red||(this.x=this.x.toRed(this.curve.red)),this.z.red||(this.z=this.z.toRed(this.curve.red)))}i(s,o),t.exports=s,s.prototype.validate=function(t){var e=t.normalize().x,n=e.redSqr(),r=n.redMul(e).redAdd(n.redMul(this.a)).redAdd(e);return 0===r.redSqrt().redSqr().cmp(r)},i(A,o.BasePoint),s.prototype.decodePoint=function(t,e){return this.point(a.toArray(t,e),1)},s.prototype.point=function(t,e){return new A(this,t,e)},s.prototype.pointFromJSON=function(t){return A.fromJSON(this,t)},A.prototype.precompute=function(){},A.prototype._encode=function(){return this.getX().toArray("be",this.curve.p.byteLength())},A.fromJSON=function(t,e){return new A(t,e[0],e[1]||t.one)},A.prototype.inspect=function(){return this.isInfinity()?"<EC Point Infinity>":"<EC Point x: "+this.x.fromRed().toString(16,2)+" z: "+this.z.fromRed().toString(16,2)+">"},A.prototype.isInfinity=function(){return 0===this.z.cmpn(0)},A.prototype.dbl=function(){var t=this.x.redAdd(this.z).redSqr(),e=this.x.redSub(this.z).redSqr(),n=t.redSub(e),r=t.redMul(e),i=n.redMul(e.redAdd(this.curve.a24.redMul(n)));return this.curve.point(r,i)},A.prototype.add=function(){throw new Error("Not supported on Montgomery curve")},A.prototype.diffAdd=function(t,e){var n=this.x.redAdd(this.z),r=this.x.redSub(this.z),i=t.x.redAdd(t.z),o=t.x.redSub(t.z).redMul(n),a=i.redMul(r),s=e.z.redMul(o.redAdd(a).redSqr()),A=e.x.redMul(o.redISub(a).redSqr());return this.curve.point(s,A)},A.prototype.mul=function(t){for(var e=t.clone(),n=this,r=this.curve.point(null,null),i=[];0!==e.cmpn(0);e.iushrn(1))i.push(e.andln(1));for(var o=i.length-1;o>=0;o--)0===i[o]?(n=n.diffAdd(r,this),r=r.dbl()):(r=n.diffAdd(r,this),n=n.dbl());return r},A.prototype.mulAdd=function(){throw new Error("Not supported on Montgomery curve")},A.prototype.jumlAdd=function(){throw new Error("Not supported on Montgomery curve")},A.prototype.eq=function(t){return 0===this.getX().cmp(t.getX())},A.prototype.normalize=function(){return this.x=this.x.redMul(this.z.redInvm()),this.z=this.curve.one,this},A.prototype.getX=function(){return this.normalize(),this.x.fromRed()}},function(t,e,n){"use strict";var r=n(78),i=n(53),o=n(30),a=n(213),s=r.assert;function A(t){this.twisted=1!=(0|t.a),this.mOneA=this.twisted&&-1==(0|t.a),this.extended=this.mOneA,a.call(this,"edwards",t),this.a=new i(t.a,16).umod(this.red.m),this.a=this.a.toRed(this.red),this.c=new i(t.c,16).toRed(this.red),this.c2=this.c.redSqr(),this.d=new i(t.d,16).toRed(this.red),this.dd=this.d.redAdd(this.d),s(!this.twisted||0===this.c.fromRed().cmpn(1)),this.oneC=1==(0|t.c)}function l(t,e,n,r,o){a.BasePoint.call(this,t,"projective"),null===e&&null===n&&null===r?(this.x=this.curve.zero,this.y=this.curve.one,this.z=this.curve.one,this.t=this.curve.zero,this.zOne=!0):(this.x=new i(e,16),this.y=new i(n,16),this.z=r?new i(r,16):this.curve.one,this.t=o&&new i(o,16),this.x.red||(this.x=this.x.toRed(this.curve.red)),this.y.red||(this.y=this.y.toRed(this.curve.red)),this.z.red||(this.z=this.z.toRed(this.curve.red)),this.t&&!this.t.red&&(this.t=this.t.toRed(this.curve.red)),this.zOne=this.z===this.curve.one,this.curve.extended&&!this.t&&(this.t=this.x.redMul(this.y),this.zOne||(this.t=this.t.redMul(this.z.redInvm()))))}o(A,a),t.exports=A,A.prototype._mulA=function(t){return this.mOneA?t.redNeg():this.a.redMul(t)},A.prototype._mulC=function(t){return this.oneC?t:this.c.redMul(t)},A.prototype.jpoint=function(t,e,n,r){return this.point(t,e,n,r)},A.prototype.pointFromX=function(t,e){(t=new i(t,16)).red||(t=t.toRed(this.red));var n=t.redSqr(),r=this.c2.redSub(this.a.redMul(n)),o=this.one.redSub(this.c2.redMul(this.d).redMul(n)),a=r.redMul(o.redInvm()),s=a.redSqrt();if(0!==s.redSqr().redSub(a).cmp(this.zero))throw new Error("invalid point");var A=s.fromRed().isOdd();return(e&&!A||!e&&A)&&(s=s.redNeg()),this.point(t,s)},A.prototype.pointFromY=function(t,e){(t=new i(t,16)).red||(t=t.toRed(this.red));var n=t.redSqr(),r=n.redSub(this.c2),o=n.redMul(this.d).redMul(this.c2).redSub(this.a),a=r.redMul(o.redInvm());if(0===a.cmp(this.zero)){if(e)throw new Error("invalid point");return this.point(this.zero,t)}var s=a.redSqrt();if(0!==s.redSqr().redSub(a).cmp(this.zero))throw new Error("invalid point");return s.fromRed().isOdd()!==e&&(s=s.redNeg()),this.point(s,t)},A.prototype.validate=function(t){if(t.isInfinity())return!0;t.normalize();var e=t.x.redSqr(),n=t.y.redSqr(),r=e.redMul(this.a).redAdd(n),i=this.c2.redMul(this.one.redAdd(this.d.redMul(e).redMul(n)));return 0===r.cmp(i)},o(l,a.BasePoint),A.prototype.pointFromJSON=function(t){return l.fromJSON(this,t)},A.prototype.point=function(t,e,n,r){return new l(this,t,e,n,r)},l.fromJSON=function(t,e){return new l(t,e[0],e[1],e[2])},l.prototype.inspect=function(){return this.isInfinity()?"<EC Point Infinity>":"<EC Point x: "+this.x.fromRed().toString(16,2)+" y: "+this.y.fromRed().toString(16,2)+" z: "+this.z.fromRed().toString(16,2)+">"},l.prototype.isInfinity=function(){return 0===this.x.cmpn(0)&&(0===this.y.cmp(this.z)||this.zOne&&0===this.y.cmp(this.curve.c))},l.prototype._extDbl=function(){var t=this.x.redSqr(),e=this.y.redSqr(),n=this.z.redSqr();n=n.redIAdd(n);var r=this.curve._mulA(t),i=this.x.redAdd(this.y).redSqr().redISub(t).redISub(e),o=r.redAdd(e),a=o.redSub(n),s=r.redSub(e),A=i.redMul(a),l=o.redMul(s),c=i.redMul(s),u=a.redMul(o);return this.curve.point(A,l,u,c)},l.prototype._projDbl=function(){var t,e,n,r=this.x.redAdd(this.y).redSqr(),i=this.x.redSqr(),o=this.y.redSqr();if(this.curve.twisted){var a=(l=this.curve._mulA(i)).redAdd(o);if(this.zOne)t=r.redSub(i).redSub(o).redMul(a.redSub(this.curve.two)),e=a.redMul(l.redSub(o)),n=a.redSqr().redSub(a).redSub(a);else{var s=this.z.redSqr(),A=a.redSub(s).redISub(s);t=r.redSub(i).redISub(o).redMul(A),e=a.redMul(l.redSub(o)),n=a.redMul(A)}}else{var l=i.redAdd(o);s=this.curve._mulC(this.z).redSqr(),A=l.redSub(s).redSub(s);t=this.curve._mulC(r.redISub(l)).redMul(A),e=this.curve._mulC(l).redMul(i.redISub(o)),n=l.redMul(A)}return this.curve.point(t,e,n)},l.prototype.dbl=function(){return this.isInfinity()?this:this.curve.extended?this._extDbl():this._projDbl()},l.prototype._extAdd=function(t){var e=this.y.redSub(this.x).redMul(t.y.redSub(t.x)),n=this.y.redAdd(this.x).redMul(t.y.redAdd(t.x)),r=this.t.redMul(this.curve.dd).redMul(t.t),i=this.z.redMul(t.z.redAdd(t.z)),o=n.redSub(e),a=i.redSub(r),s=i.redAdd(r),A=n.redAdd(e),l=o.redMul(a),c=s.redMul(A),u=o.redMul(A),g=a.redMul(s);return this.curve.point(l,c,g,u)},l.prototype._projAdd=function(t){var e,n,r=this.z.redMul(t.z),i=r.redSqr(),o=this.x.redMul(t.x),a=this.y.redMul(t.y),s=this.curve.d.redMul(o).redMul(a),A=i.redSub(s),l=i.redAdd(s),c=this.x.redAdd(this.y).redMul(t.x.redAdd(t.y)).redISub(o).redISub(a),u=r.redMul(A).redMul(c);return this.curve.twisted?(e=r.redMul(l).redMul(a.redSub(this.curve._mulA(o))),n=A.redMul(l)):(e=r.redMul(l).redMul(a.redSub(o)),n=this.curve._mulC(A).redMul(l)),this.curve.point(u,e,n)},l.prototype.add=function(t){return this.isInfinity()?t:t.isInfinity()?this:this.curve.extended?this._extAdd(t):this._projAdd(t)},l.prototype.mul=function(t){return this._hasDoubles(t)?this.curve._fixedNafMul(this,t):this.curve._wnafMul(this,t)},l.prototype.mulAdd=function(t,e,n){return this.curve._wnafMulAdd(1,[this,e],[t,n],2,!1)},l.prototype.jmulAdd=function(t,e,n){return this.curve._wnafMulAdd(1,[this,e],[t,n],2,!0)},l.prototype.normalize=function(){if(this.zOne)return this;var t=this.z.redInvm();return this.x=this.x.redMul(t),this.y=this.y.redMul(t),this.t&&(this.t=this.t.redMul(t)),this.z=this.curve.one,this.zOne=!0,this},l.prototype.neg=function(){return this.curve.point(this.x.redNeg(),this.y,this.z,this.t&&this.t.redNeg())},l.prototype.getX=function(){return this.normalize(),this.x.fromRed()},l.prototype.getY=function(){return this.normalize(),this.y.fromRed()},l.prototype.eq=function(t){return this===t||0===this.getX().cmp(t.getX())&&0===this.getY().cmp(t.getY())},l.prototype.eqXToP=function(t){var e=t.toRed(this.curve.red).redMul(this.z);if(0===this.x.cmp(e))return!0;for(var n=t.clone(),r=this.curve.redN.redMul(this.z);;){if(n.iadd(this.curve.n),n.cmp(this.curve.p)>=0)return!1;if(e.redIAdd(r),0===this.x.cmp(e))return!0}},l.prototype.toP=l.prototype.normalize,l.prototype.mixedAdd=l.prototype.add},function(t,e,n){"use strict";e.sha1=n(818),e.sha224=n(819),e.sha256=n(420),e.sha384=n(820),e.sha512=n(421)},function(t,e,n){"use strict";var r=n(87),i=n(168),o=n(419),a=r.rotl32,s=r.sum32,A=r.sum32_5,l=o.ft_1,c=i.BlockHash,u=[1518500249,1859775393,2400959708,3395469782];function g(){if(!(this instanceof g))return new g;c.call(this),this.h=[1732584193,4023233417,2562383102,271733878,3285377520],this.W=new Array(80)}r.inherits(g,c),t.exports=g,g.blockSize=512,g.outSize=160,g.hmacStrength=80,g.padLength=64,g.prototype._update=function(t,e){for(var n=this.W,r=0;r<16;r++)n[r]=t[e+r];for(;r<n.length;r++)n[r]=a(n[r-3]^n[r-8]^n[r-14]^n[r-16],1);var i=this.h[0],o=this.h[1],c=this.h[2],g=this.h[3],h=this.h[4];for(r=0;r<n.length;r++){var d=~~(r/20),f=A(a(i,5),l(d,o,c,g),h,n[r],u[d]);h=g,g=c,c=a(o,30),o=i,i=f}this.h[0]=s(this.h[0],i),this.h[1]=s(this.h[1],o),this.h[2]=s(this.h[2],c),this.h[3]=s(this.h[3],g),this.h[4]=s(this.h[4],h)},g.prototype._digest=function(t){return"hex"===t?r.toHex32(this.h,"big"):r.split32(this.h,"big")}},function(t,e,n){"use strict";var r=n(87),i=n(420);function o(){if(!(this instanceof o))return new o;i.call(this),this.h=[3238371032,914150663,812702999,4144912697,4290775857,1750603025,1694076839,3204075428]}r.inherits(o,i),t.exports=o,o.blockSize=512,o.outSize=224,o.hmacStrength=192,o.padLength=64,o.prototype._digest=function(t){return"hex"===t?r.toHex32(this.h.slice(0,7),"big"):r.split32(this.h.slice(0,7),"big")}},function(t,e,n){"use strict";var r=n(87),i=n(421);function o(){if(!(this instanceof o))return new o;i.call(this),this.h=[3418070365,3238371032,1654270250,914150663,2438529370,812702999,355462360,4144912697,1731405415,4290775857,2394180231,1750603025,3675008525,1694076839,1203062813,3204075428]}r.inherits(o,i),t.exports=o,o.blockSize=1024,o.outSize=384,o.hmacStrength=192,o.padLength=128,o.prototype._digest=function(t){return"hex"===t?r.toHex32(this.h.slice(0,12),"big"):r.split32(this.h.slice(0,12),"big")}},function(t,e,n){"use strict";var r=n(87),i=n(168),o=r.rotl32,a=r.sum32,s=r.sum32_3,A=r.sum32_4,l=i.BlockHash;function c(){if(!(this instanceof c))return new c;l.call(this),this.h=[1732584193,4023233417,2562383102,271733878,3285377520],this.endian="little"}function u(t,e,n,r){return t<=15?e^n^r:t<=31?e&n|~e&r:t<=47?(e|~n)^r:t<=63?e&r|n&~r:e^(n|~r)}function g(t){return t<=15?0:t<=31?1518500249:t<=47?1859775393:t<=63?2400959708:2840853838}function h(t){return t<=15?1352829926:t<=31?1548603684:t<=47?1836072691:t<=63?2053994217:0}r.inherits(c,l),e.ripemd160=c,c.blockSize=512,c.outSize=160,c.hmacStrength=192,c.padLength=64,c.prototype._update=function(t,e){for(var n=this.h[0],r=this.h[1],i=this.h[2],l=this.h[3],c=this.h[4],C=n,m=r,y=i,E=l,v=c,B=0;B<80;B++){var b=a(o(A(n,u(B,r,i,l),t[d[B]+e],g(B)),p[B]),c);n=c,c=l,l=o(i,10),i=r,r=b,b=a(o(A(C,u(79-B,m,y,E),t[f[B]+e],h(B)),I[B]),v),C=v,v=E,E=o(y,10),y=m,m=b}b=s(this.h[1],i,E),this.h[1]=s(this.h[2],l,v),this.h[2]=s(this.h[3],c,C),this.h[3]=s(this.h[4],n,m),this.h[4]=s(this.h[0],r,y),this.h[0]=b},c.prototype._digest=function(t){return"hex"===t?r.toHex32(this.h,"little"):r.split32(this.h,"little")};var d=[0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,7,4,13,1,10,6,15,3,12,0,9,5,2,14,11,8,3,10,14,4,9,15,8,1,2,7,0,6,13,11,5,12,1,9,11,10,0,8,12,4,13,3,7,15,14,5,6,2,4,0,5,9,7,12,2,10,14,1,3,8,11,6,15,13],f=[5,14,7,0,9,2,11,4,13,6,15,8,1,10,3,12,6,11,3,7,0,13,5,10,14,15,8,12,4,9,1,2,15,5,1,3,7,14,6,9,11,8,12,2,10,0,4,13,8,6,4,1,3,11,15,0,5,12,2,13,9,7,10,14,12,15,10,4,1,5,8,7,6,2,13,14,0,3,9,11],p=[11,14,15,12,5,8,7,9,11,13,14,15,6,7,9,8,7,6,8,13,11,9,7,15,7,12,15,9,11,7,13,12,11,13,6,7,14,9,13,15,14,8,13,6,5,12,7,5,11,12,14,15,14,15,9,8,9,14,5,6,8,6,5,12,9,15,5,11,6,8,13,12,5,12,13,14,11,8,5,6],I=[8,9,9,11,13,15,15,5,7,7,8,11,14,14,12,6,9,13,15,7,12,8,9,11,7,7,12,7,6,15,13,11,9,7,15,11,8,6,6,14,12,13,5,14,13,13,7,5,15,5,8,11,14,14,6,14,6,9,12,9,12,5,15,8,8,5,12,9,12,5,14,6,8,13,6,5,15,13,11,11]},function(t,e,n){"use strict";var r=n(87),i=n(77);function o(t,e,n){if(!(this instanceof o))return new o(t,e,n);this.Hash=t,this.blockSize=t.blockSize/8,this.outSize=t.outSize/8,this.inner=null,this.outer=null,this._init(r.toArray(e,n))}t.exports=o,o.prototype._init=function(t){t.length>this.blockSize&&(t=(new this.Hash).update(t).digest()),i(t.length<=this.blockSize);for(var e=t.length;e<this.blockSize;e++)t.push(0);for(e=0;e<t.length;e++)t[e]^=54;for(this.inner=(new this.Hash).update(t),e=0;e<t.length;e++)t[e]^=106;this.outer=(new this.Hash).update(t)},o.prototype.update=function(t,e){return this.inner.update(t,e),this},o.prototype.digest=function(t){return this.outer.update(this.inner.digest()),this.outer.digest(t)}},function(t,e){t.exports={doubles:{step:4,points:[["e60fce93b59e9ec53011aabc21c23e97b2a31369b87a5ae9c44ee89e2a6dec0a","f7e3507399e595929db99f34f57937101296891e44d23f0be1f32cce69616821"],["8282263212c609d9ea2a6e3e172de238d8c39cabd5ac1ca10646e23fd5f51508","11f8a8098557dfe45e8256e830b60ace62d613ac2f7b17bed31b6eaff6e26caf"],["175e159f728b865a72f99cc6c6fc846de0b93833fd2222ed73fce5b551e5b739","d3506e0d9e3c79eba4ef97a51ff71f5eacb5955add24345c6efa6ffee9fed695"],["363d90d447b00c9c99ceac05b6262ee053441c7e55552ffe526bad8f83ff4640","4e273adfc732221953b445397f3363145b9a89008199ecb62003c7f3bee9de9"],["8b4b5f165df3c2be8c6244b5b745638843e4a781a15bcd1b69f79a55dffdf80c","4aad0a6f68d308b4b3fbd7813ab0da04f9e336546162ee56b3eff0c65fd4fd36"],["723cbaa6e5db996d6bf771c00bd548c7b700dbffa6c0e77bcb6115925232fcda","96e867b5595cc498a921137488824d6e2660a0653779494801dc069d9eb39f5f"],["eebfa4d493bebf98ba5feec812c2d3b50947961237a919839a533eca0e7dd7fa","5d9a8ca3970ef0f269ee7edaf178089d9ae4cdc3a711f712ddfd4fdae1de8999"],["100f44da696e71672791d0a09b7bde459f1215a29b3c03bfefd7835b39a48db0","cdd9e13192a00b772ec8f3300c090666b7ff4a18ff5195ac0fbd5cd62bc65a09"],["e1031be262c7ed1b1dc9227a4a04c017a77f8d4464f3b3852c8acde6e534fd2d","9d7061928940405e6bb6a4176597535af292dd419e1ced79a44f18f29456a00d"],["feea6cae46d55b530ac2839f143bd7ec5cf8b266a41d6af52d5e688d9094696d","e57c6b6c97dce1bab06e4e12bf3ecd5c981c8957cc41442d3155debf18090088"],["da67a91d91049cdcb367be4be6ffca3cfeed657d808583de33fa978bc1ec6cb1","9bacaa35481642bc41f463f7ec9780e5dec7adc508f740a17e9ea8e27a68be1d"],["53904faa0b334cdda6e000935ef22151ec08d0f7bb11069f57545ccc1a37b7c0","5bc087d0bc80106d88c9eccac20d3c1c13999981e14434699dcb096b022771c8"],["8e7bcd0bd35983a7719cca7764ca906779b53a043a9b8bcaeff959f43ad86047","10b7770b2a3da4b3940310420ca9514579e88e2e47fd68b3ea10047e8460372a"],["385eed34c1cdff21e6d0818689b81bde71a7f4f18397e6690a841e1599c43862","283bebc3e8ea23f56701de19e9ebf4576b304eec2086dc8cc0458fe5542e5453"],["6f9d9b803ecf191637c73a4413dfa180fddf84a5947fbc9c606ed86c3fac3a7","7c80c68e603059ba69b8e2a30e45c4d47ea4dd2f5c281002d86890603a842160"],["3322d401243c4e2582a2147c104d6ecbf774d163db0f5e5313b7e0e742d0e6bd","56e70797e9664ef5bfb019bc4ddaf9b72805f63ea2873af624f3a2e96c28b2a0"],["85672c7d2de0b7da2bd1770d89665868741b3f9af7643397721d74d28134ab83","7c481b9b5b43b2eb6374049bfa62c2e5e77f17fcc5298f44c8e3094f790313a6"],["948bf809b1988a46b06c9f1919413b10f9226c60f668832ffd959af60c82a0a","53a562856dcb6646dc6b74c5d1c3418c6d4dff08c97cd2bed4cb7f88d8c8e589"],["6260ce7f461801c34f067ce0f02873a8f1b0e44dfc69752accecd819f38fd8e8","bc2da82b6fa5b571a7f09049776a1ef7ecd292238051c198c1a84e95b2b4ae17"],["e5037de0afc1d8d43d8348414bbf4103043ec8f575bfdc432953cc8d2037fa2d","4571534baa94d3b5f9f98d09fb990bddbd5f5b03ec481f10e0e5dc841d755bda"],["e06372b0f4a207adf5ea905e8f1771b4e7e8dbd1c6a6c5b725866a0ae4fce725","7a908974bce18cfe12a27bb2ad5a488cd7484a7787104870b27034f94eee31dd"],["213c7a715cd5d45358d0bbf9dc0ce02204b10bdde2a3f58540ad6908d0559754","4b6dad0b5ae462507013ad06245ba190bb4850f5f36a7eeddff2c27534b458f2"],["4e7c272a7af4b34e8dbb9352a5419a87e2838c70adc62cddf0cc3a3b08fbd53c","17749c766c9d0b18e16fd09f6def681b530b9614bff7dd33e0b3941817dcaae6"],["fea74e3dbe778b1b10f238ad61686aa5c76e3db2be43057632427e2840fb27b6","6e0568db9b0b13297cf674deccb6af93126b596b973f7b77701d3db7f23cb96f"],["76e64113f677cf0e10a2570d599968d31544e179b760432952c02a4417bdde39","c90ddf8dee4e95cf577066d70681f0d35e2a33d2b56d2032b4b1752d1901ac01"],["c738c56b03b2abe1e8281baa743f8f9a8f7cc643df26cbee3ab150242bcbb891","893fb578951ad2537f718f2eacbfbbbb82314eef7880cfe917e735d9699a84c3"],["d895626548b65b81e264c7637c972877d1d72e5f3a925014372e9f6588f6c14b","febfaa38f2bc7eae728ec60818c340eb03428d632bb067e179363ed75d7d991f"],["b8da94032a957518eb0f6433571e8761ceffc73693e84edd49150a564f676e03","2804dfa44805a1e4d7c99cc9762808b092cc584d95ff3b511488e4e74efdf6e7"],["e80fea14441fb33a7d8adab9475d7fab2019effb5156a792f1a11778e3c0df5d","eed1de7f638e00771e89768ca3ca94472d155e80af322ea9fcb4291b6ac9ec78"],["a301697bdfcd704313ba48e51d567543f2a182031efd6915ddc07bbcc4e16070","7370f91cfb67e4f5081809fa25d40f9b1735dbf7c0a11a130c0d1a041e177ea1"],["90ad85b389d6b936463f9d0512678de208cc330b11307fffab7ac63e3fb04ed4","e507a3620a38261affdcbd9427222b839aefabe1582894d991d4d48cb6ef150"],["8f68b9d2f63b5f339239c1ad981f162ee88c5678723ea3351b7b444c9ec4c0da","662a9f2dba063986de1d90c2b6be215dbbea2cfe95510bfdf23cbf79501fff82"],["e4f3fb0176af85d65ff99ff9198c36091f48e86503681e3e6686fd5053231e11","1e63633ad0ef4f1c1661a6d0ea02b7286cc7e74ec951d1c9822c38576feb73bc"],["8c00fa9b18ebf331eb961537a45a4266c7034f2f0d4e1d0716fb6eae20eae29e","efa47267fea521a1a9dc343a3736c974c2fadafa81e36c54e7d2a4c66702414b"],["e7a26ce69dd4829f3e10cec0a9e98ed3143d084f308b92c0997fddfc60cb3e41","2a758e300fa7984b471b006a1aafbb18d0a6b2c0420e83e20e8a9421cf2cfd51"],["b6459e0ee3662ec8d23540c223bcbdc571cbcb967d79424f3cf29eb3de6b80ef","67c876d06f3e06de1dadf16e5661db3c4b3ae6d48e35b2ff30bf0b61a71ba45"],["d68a80c8280bb840793234aa118f06231d6f1fc67e73c5a5deda0f5b496943e8","db8ba9fff4b586d00c4b1f9177b0e28b5b0e7b8f7845295a294c84266b133120"],["324aed7df65c804252dc0270907a30b09612aeb973449cea4095980fc28d3d5d","648a365774b61f2ff130c0c35aec1f4f19213b0c7e332843967224af96ab7c84"],["4df9c14919cde61f6d51dfdbe5fee5dceec4143ba8d1ca888e8bd373fd054c96","35ec51092d8728050974c23a1d85d4b5d506cdc288490192ebac06cad10d5d"],["9c3919a84a474870faed8a9c1cc66021523489054d7f0308cbfc99c8ac1f98cd","ddb84f0f4a4ddd57584f044bf260e641905326f76c64c8e6be7e5e03d4fc599d"],["6057170b1dd12fdf8de05f281d8e06bb91e1493a8b91d4cc5a21382120a959e5","9a1af0b26a6a4807add9a2daf71df262465152bc3ee24c65e899be932385a2a8"],["a576df8e23a08411421439a4518da31880cef0fba7d4df12b1a6973eecb94266","40a6bf20e76640b2c92b97afe58cd82c432e10a7f514d9f3ee8be11ae1b28ec8"],["7778a78c28dec3e30a05fe9629de8c38bb30d1f5cf9a3a208f763889be58ad71","34626d9ab5a5b22ff7098e12f2ff580087b38411ff24ac563b513fc1fd9f43ac"],["928955ee637a84463729fd30e7afd2ed5f96274e5ad7e5cb09eda9c06d903ac","c25621003d3f42a827b78a13093a95eeac3d26efa8a8d83fc5180e935bcd091f"],["85d0fef3ec6db109399064f3a0e3b2855645b4a907ad354527aae75163d82751","1f03648413a38c0be29d496e582cf5663e8751e96877331582c237a24eb1f962"],["ff2b0dce97eece97c1c9b6041798b85dfdfb6d8882da20308f5404824526087e","493d13fef524ba188af4c4dc54d07936c7b7ed6fb90e2ceb2c951e01f0c29907"],["827fbbe4b1e880ea9ed2b2e6301b212b57f1ee148cd6dd28780e5e2cf856e241","c60f9c923c727b0b71bef2c67d1d12687ff7a63186903166d605b68baec293ec"],["eaa649f21f51bdbae7be4ae34ce6e5217a58fdce7f47f9aa7f3b58fa2120e2b3","be3279ed5bbbb03ac69a80f89879aa5a01a6b965f13f7e59d47a5305ba5ad93d"],["e4a42d43c5cf169d9391df6decf42ee541b6d8f0c9a137401e23632dda34d24f","4d9f92e716d1c73526fc99ccfb8ad34ce886eedfa8d8e4f13a7f7131deba9414"],["1ec80fef360cbdd954160fadab352b6b92b53576a88fea4947173b9d4300bf19","aeefe93756b5340d2f3a4958a7abbf5e0146e77f6295a07b671cdc1cc107cefd"],["146a778c04670c2f91b00af4680dfa8bce3490717d58ba889ddb5928366642be","b318e0ec3354028add669827f9d4b2870aaa971d2f7e5ed1d0b297483d83efd0"],["fa50c0f61d22e5f07e3acebb1aa07b128d0012209a28b9776d76a8793180eef9","6b84c6922397eba9b72cd2872281a68a5e683293a57a213b38cd8d7d3f4f2811"],["da1d61d0ca721a11b1a5bf6b7d88e8421a288ab5d5bba5220e53d32b5f067ec2","8157f55a7c99306c79c0766161c91e2966a73899d279b48a655fba0f1ad836f1"],["a8e282ff0c9706907215ff98e8fd416615311de0446f1e062a73b0610d064e13","7f97355b8db81c09abfb7f3c5b2515888b679a3e50dd6bd6cef7c73111f4cc0c"],["174a53b9c9a285872d39e56e6913cab15d59b1fa512508c022f382de8319497c","ccc9dc37abfc9c1657b4155f2c47f9e6646b3a1d8cb9854383da13ac079afa73"],["959396981943785c3d3e57edf5018cdbe039e730e4918b3d884fdff09475b7ba","2e7e552888c331dd8ba0386a4b9cd6849c653f64c8709385e9b8abf87524f2fd"],["d2a63a50ae401e56d645a1153b109a8fcca0a43d561fba2dbb51340c9d82b151","e82d86fb6443fcb7565aee58b2948220a70f750af484ca52d4142174dcf89405"],["64587e2335471eb890ee7896d7cfdc866bacbdbd3839317b3436f9b45617e073","d99fcdd5bf6902e2ae96dd6447c299a185b90a39133aeab358299e5e9faf6589"],["8481bde0e4e4d885b3a546d3e549de042f0aa6cea250e7fd358d6c86dd45e458","38ee7b8cba5404dd84a25bf39cecb2ca900a79c42b262e556d64b1b59779057e"],["13464a57a78102aa62b6979ae817f4637ffcfed3c4b1ce30bcd6303f6caf666b","69be159004614580ef7e433453ccb0ca48f300a81d0942e13f495a907f6ecc27"],["bc4a9df5b713fe2e9aef430bcc1dc97a0cd9ccede2f28588cada3a0d2d83f366","d3a81ca6e785c06383937adf4b798caa6e8a9fbfa547b16d758d666581f33c1"],["8c28a97bf8298bc0d23d8c749452a32e694b65e30a9472a3954ab30fe5324caa","40a30463a3305193378fedf31f7cc0eb7ae784f0451cb9459e71dc73cbef9482"],["8ea9666139527a8c1dd94ce4f071fd23c8b350c5a4bb33748c4ba111faccae0","620efabbc8ee2782e24e7c0cfb95c5d735b783be9cf0f8e955af34a30e62b945"],["dd3625faef5ba06074669716bbd3788d89bdde815959968092f76cc4eb9a9787","7a188fa3520e30d461da2501045731ca941461982883395937f68d00c644a573"],["f710d79d9eb962297e4f6232b40e8f7feb2bc63814614d692c12de752408221e","ea98e67232d3b3295d3b535532115ccac8612c721851617526ae47a9c77bfc82"]]},naf:{wnd:7,points:[["f9308a019258c31049344f85f89d5229b531c845836f99b08601f113bce036f9","388f7b0f632de8140fe337e62a37f3566500a99934c2231b6cb9fd7584b8e672"],["2f8bde4d1a07209355b4a7250a5c5128e88b84bddc619ab7cba8d569b240efe4","d8ac222636e5e3d6d4dba9dda6c9c426f788271bab0d6840dca87d3aa6ac62d6"],["5cbdf0646e5db4eaa398f365f2ea7a0e3d419b7e0330e39ce92bddedcac4f9bc","6aebca40ba255960a3178d6d861a54dba813d0b813fde7b5a5082628087264da"],["acd484e2f0c7f65309ad178a9f559abde09796974c57e714c35f110dfc27ccbe","cc338921b0a7d9fd64380971763b61e9add888a4375f8e0f05cc262ac64f9c37"],["774ae7f858a9411e5ef4246b70c65aac5649980be5c17891bbec17895da008cb","d984a032eb6b5e190243dd56d7b7b365372db1e2dff9d6a8301d74c9c953c61b"],["f28773c2d975288bc7d1d205c3748651b075fbc6610e58cddeeddf8f19405aa8","ab0902e8d880a89758212eb65cdaf473a1a06da521fa91f29b5cb52db03ed81"],["d7924d4f7d43ea965a465ae3095ff41131e5946f3c85f79e44adbcf8e27e080e","581e2872a86c72a683842ec228cc6defea40af2bd896d3a5c504dc9ff6a26b58"],["defdea4cdb677750a420fee807eacf21eb9898ae79b9768766e4faa04a2d4a34","4211ab0694635168e997b0ead2a93daeced1f4a04a95c0f6cfb199f69e56eb77"],["2b4ea0a797a443d293ef5cff444f4979f06acfebd7e86d277475656138385b6c","85e89bc037945d93b343083b5a1c86131a01f60c50269763b570c854e5c09b7a"],["352bbf4a4cdd12564f93fa332ce333301d9ad40271f8107181340aef25be59d5","321eb4075348f534d59c18259dda3e1f4a1b3b2e71b1039c67bd3d8bcf81998c"],["2fa2104d6b38d11b0230010559879124e42ab8dfeff5ff29dc9cdadd4ecacc3f","2de1068295dd865b64569335bd5dd80181d70ecfc882648423ba76b532b7d67"],["9248279b09b4d68dab21a9b066edda83263c3d84e09572e269ca0cd7f5453714","73016f7bf234aade5d1aa71bdea2b1ff3fc0de2a887912ffe54a32ce97cb3402"],["daed4f2be3a8bf278e70132fb0beb7522f570e144bf615c07e996d443dee8729","a69dce4a7d6c98e8d4a1aca87ef8d7003f83c230f3afa726ab40e52290be1c55"],["c44d12c7065d812e8acf28d7cbb19f9011ecd9e9fdf281b0e6a3b5e87d22e7db","2119a460ce326cdc76c45926c982fdac0e106e861edf61c5a039063f0e0e6482"],["6a245bf6dc698504c89a20cfded60853152b695336c28063b61c65cbd269e6b4","e022cf42c2bd4a708b3f5126f16a24ad8b33ba48d0423b6efd5e6348100d8a82"],["1697ffa6fd9de627c077e3d2fe541084ce13300b0bec1146f95ae57f0d0bd6a5","b9c398f186806f5d27561506e4557433a2cf15009e498ae7adee9d63d01b2396"],["605bdb019981718b986d0f07e834cb0d9deb8360ffb7f61df982345ef27a7479","2972d2de4f8d20681a78d93ec96fe23c26bfae84fb14db43b01e1e9056b8c49"],["62d14dab4150bf497402fdc45a215e10dcb01c354959b10cfe31c7e9d87ff33d","80fc06bd8cc5b01098088a1950eed0db01aa132967ab472235f5642483b25eaf"],["80c60ad0040f27dade5b4b06c408e56b2c50e9f56b9b8b425e555c2f86308b6f","1c38303f1cc5c30f26e66bad7fe72f70a65eed4cbe7024eb1aa01f56430bd57a"],["7a9375ad6167ad54aa74c6348cc54d344cc5dc9487d847049d5eabb0fa03c8fb","d0e3fa9eca8726909559e0d79269046bdc59ea10c70ce2b02d499ec224dc7f7"],["d528ecd9b696b54c907a9ed045447a79bb408ec39b68df504bb51f459bc3ffc9","eecf41253136e5f99966f21881fd656ebc4345405c520dbc063465b521409933"],["49370a4b5f43412ea25f514e8ecdad05266115e4a7ecb1387231808f8b45963","758f3f41afd6ed428b3081b0512fd62a54c3f3afbb5b6764b653052a12949c9a"],["77f230936ee88cbbd73df930d64702ef881d811e0e1498e2f1c13eb1fc345d74","958ef42a7886b6400a08266e9ba1b37896c95330d97077cbbe8eb3c7671c60d6"],["f2dac991cc4ce4b9ea44887e5c7c0bce58c80074ab9d4dbaeb28531b7739f530","e0dedc9b3b2f8dad4da1f32dec2531df9eb5fbeb0598e4fd1a117dba703a3c37"],["463b3d9f662621fb1b4be8fbbe2520125a216cdfc9dae3debcba4850c690d45b","5ed430d78c296c3543114306dd8622d7c622e27c970a1de31cb377b01af7307e"],["f16f804244e46e2a09232d4aff3b59976b98fac14328a2d1a32496b49998f247","cedabd9b82203f7e13d206fcdf4e33d92a6c53c26e5cce26d6579962c4e31df6"],["caf754272dc84563b0352b7a14311af55d245315ace27c65369e15f7151d41d1","cb474660ef35f5f2a41b643fa5e460575f4fa9b7962232a5c32f908318a04476"],["2600ca4b282cb986f85d0f1709979d8b44a09c07cb86d7c124497bc86f082120","4119b88753c15bd6a693b03fcddbb45d5ac6be74ab5f0ef44b0be9475a7e4b40"],["7635ca72d7e8432c338ec53cd12220bc01c48685e24f7dc8c602a7746998e435","91b649609489d613d1d5e590f78e6d74ecfc061d57048bad9e76f302c5b9c61"],["754e3239f325570cdbbf4a87deee8a66b7f2b33479d468fbc1a50743bf56cc18","673fb86e5bda30fb3cd0ed304ea49a023ee33d0197a695d0c5d98093c536683"],["e3e6bd1071a1e96aff57859c82d570f0330800661d1c952f9fe2694691d9b9e8","59c9e0bba394e76f40c0aa58379a3cb6a5a2283993e90c4167002af4920e37f5"],["186b483d056a033826ae73d88f732985c4ccb1f32ba35f4b4cc47fdcf04aa6eb","3b952d32c67cf77e2e17446e204180ab21fb8090895138b4a4a797f86e80888b"],["df9d70a6b9876ce544c98561f4be4f725442e6d2b737d9c91a8321724ce0963f","55eb2dafd84d6ccd5f862b785dc39d4ab157222720ef9da217b8c45cf2ba2417"],["5edd5cc23c51e87a497ca815d5dce0f8ab52554f849ed8995de64c5f34ce7143","efae9c8dbc14130661e8cec030c89ad0c13c66c0d17a2905cdc706ab7399a868"],["290798c2b6476830da12fe02287e9e777aa3fba1c355b17a722d362f84614fba","e38da76dcd440621988d00bcf79af25d5b29c094db2a23146d003afd41943e7a"],["af3c423a95d9f5b3054754efa150ac39cd29552fe360257362dfdecef4053b45","f98a3fd831eb2b749a93b0e6f35cfb40c8cd5aa667a15581bc2feded498fd9c6"],["766dbb24d134e745cccaa28c99bf274906bb66b26dcf98df8d2fed50d884249a","744b1152eacbe5e38dcc887980da38b897584a65fa06cedd2c924f97cbac5996"],["59dbf46f8c94759ba21277c33784f41645f7b44f6c596a58ce92e666191abe3e","c534ad44175fbc300f4ea6ce648309a042ce739a7919798cd85e216c4a307f6e"],["f13ada95103c4537305e691e74e9a4a8dd647e711a95e73cb62dc6018cfd87b8","e13817b44ee14de663bf4bc808341f326949e21a6a75c2570778419bdaf5733d"],["7754b4fa0e8aced06d4167a2c59cca4cda1869c06ebadfb6488550015a88522c","30e93e864e669d82224b967c3020b8fa8d1e4e350b6cbcc537a48b57841163a2"],["948dcadf5990e048aa3874d46abef9d701858f95de8041d2a6828c99e2262519","e491a42537f6e597d5d28a3224b1bc25df9154efbd2ef1d2cbba2cae5347d57e"],["7962414450c76c1689c7b48f8202ec37fb224cf5ac0bfa1570328a8a3d7c77ab","100b610ec4ffb4760d5c1fc133ef6f6b12507a051f04ac5760afa5b29db83437"],["3514087834964b54b15b160644d915485a16977225b8847bb0dd085137ec47ca","ef0afbb2056205448e1652c48e8127fc6039e77c15c2378b7e7d15a0de293311"],["d3cc30ad6b483e4bc79ce2c9dd8bc54993e947eb8df787b442943d3f7b527eaf","8b378a22d827278d89c5e9be8f9508ae3c2ad46290358630afb34db04eede0a4"],["1624d84780732860ce1c78fcbfefe08b2b29823db913f6493975ba0ff4847610","68651cf9b6da903e0914448c6cd9d4ca896878f5282be4c8cc06e2a404078575"],["733ce80da955a8a26902c95633e62a985192474b5af207da6df7b4fd5fc61cd4","f5435a2bd2badf7d485a4d8b8db9fcce3e1ef8e0201e4578c54673bc1dc5ea1d"],["15d9441254945064cf1a1c33bbd3b49f8966c5092171e699ef258dfab81c045c","d56eb30b69463e7234f5137b73b84177434800bacebfc685fc37bbe9efe4070d"],["a1d0fcf2ec9de675b612136e5ce70d271c21417c9d2b8aaaac138599d0717940","edd77f50bcb5a3cab2e90737309667f2641462a54070f3d519212d39c197a629"],["e22fbe15c0af8ccc5780c0735f84dbe9a790badee8245c06c7ca37331cb36980","a855babad5cd60c88b430a69f53a1a7a38289154964799be43d06d77d31da06"],["311091dd9860e8e20ee13473c1155f5f69635e394704eaa74009452246cfa9b3","66db656f87d1f04fffd1f04788c06830871ec5a64feee685bd80f0b1286d8374"],["34c1fd04d301be89b31c0442d3e6ac24883928b45a9340781867d4232ec2dbdf","9414685e97b1b5954bd46f730174136d57f1ceeb487443dc5321857ba73abee"],["f219ea5d6b54701c1c14de5b557eb42a8d13f3abbcd08affcc2a5e6b049b8d63","4cb95957e83d40b0f73af4544cccf6b1f4b08d3c07b27fb8d8c2962a400766d1"],["d7b8740f74a8fbaab1f683db8f45de26543a5490bca627087236912469a0b448","fa77968128d9c92ee1010f337ad4717eff15db5ed3c049b3411e0315eaa4593b"],["32d31c222f8f6f0ef86f7c98d3a3335ead5bcd32abdd94289fe4d3091aa824bf","5f3032f5892156e39ccd3d7915b9e1da2e6dac9e6f26e961118d14b8462e1661"],["7461f371914ab32671045a155d9831ea8793d77cd59592c4340f86cbc18347b5","8ec0ba238b96bec0cbdddcae0aa442542eee1ff50c986ea6b39847b3cc092ff6"],["ee079adb1df1860074356a25aa38206a6d716b2c3e67453d287698bad7b2b2d6","8dc2412aafe3be5c4c5f37e0ecc5f9f6a446989af04c4e25ebaac479ec1c8c1e"],["16ec93e447ec83f0467b18302ee620f7e65de331874c9dc72bfd8616ba9da6b5","5e4631150e62fb40d0e8c2a7ca5804a39d58186a50e497139626778e25b0674d"],["eaa5f980c245f6f038978290afa70b6bd8855897f98b6aa485b96065d537bd99","f65f5d3e292c2e0819a528391c994624d784869d7e6ea67fb18041024edc07dc"],["78c9407544ac132692ee1910a02439958ae04877151342ea96c4b6b35a49f51","f3e0319169eb9b85d5404795539a5e68fa1fbd583c064d2462b675f194a3ddb4"],["494f4be219a1a77016dcd838431aea0001cdc8ae7a6fc688726578d9702857a5","42242a969283a5f339ba7f075e36ba2af925ce30d767ed6e55f4b031880d562c"],["a598a8030da6d86c6bc7f2f5144ea549d28211ea58faa70ebf4c1e665c1fe9b5","204b5d6f84822c307e4b4a7140737aec23fc63b65b35f86a10026dbd2d864e6b"],["c41916365abb2b5d09192f5f2dbeafec208f020f12570a184dbadc3e58595997","4f14351d0087efa49d245b328984989d5caf9450f34bfc0ed16e96b58fa9913"],["841d6063a586fa475a724604da03bc5b92a2e0d2e0a36acfe4c73a5514742881","73867f59c0659e81904f9a1c7543698e62562d6744c169ce7a36de01a8d6154"],["5e95bb399a6971d376026947f89bde2f282b33810928be4ded112ac4d70e20d5","39f23f366809085beebfc71181313775a99c9aed7d8ba38b161384c746012865"],["36e4641a53948fd476c39f8a99fd974e5ec07564b5315d8bf99471bca0ef2f66","d2424b1b1abe4eb8164227b085c9aa9456ea13493fd563e06fd51cf5694c78fc"],["336581ea7bfbbb290c191a2f507a41cf5643842170e914faeab27c2c579f726","ead12168595fe1be99252129b6e56b3391f7ab1410cd1e0ef3dcdcabd2fda224"],["8ab89816dadfd6b6a1f2634fcf00ec8403781025ed6890c4849742706bd43ede","6fdcef09f2f6d0a044e654aef624136f503d459c3e89845858a47a9129cdd24e"],["1e33f1a746c9c5778133344d9299fcaa20b0938e8acff2544bb40284b8c5fb94","60660257dd11b3aa9c8ed618d24edff2306d320f1d03010e33a7d2057f3b3b6"],["85b7c1dcb3cec1b7ee7f30ded79dd20a0ed1f4cc18cbcfcfa410361fd8f08f31","3d98a9cdd026dd43f39048f25a8847f4fcafad1895d7a633c6fed3c35e999511"],["29df9fbd8d9e46509275f4b125d6d45d7fbe9a3b878a7af872a2800661ac5f51","b4c4fe99c775a606e2d8862179139ffda61dc861c019e55cd2876eb2a27d84b"],["a0b1cae06b0a847a3fea6e671aaf8adfdfe58ca2f768105c8082b2e449fce252","ae434102edde0958ec4b19d917a6a28e6b72da1834aff0e650f049503a296cf2"],["4e8ceafb9b3e9a136dc7ff67e840295b499dfb3b2133e4ba113f2e4c0e121e5","cf2174118c8b6d7a4b48f6d534ce5c79422c086a63460502b827ce62a326683c"],["d24a44e047e19b6f5afb81c7ca2f69080a5076689a010919f42725c2b789a33b","6fb8d5591b466f8fc63db50f1c0f1c69013f996887b8244d2cdec417afea8fa3"],["ea01606a7a6c9cdd249fdfcfacb99584001edd28abbab77b5104e98e8e3b35d4","322af4908c7312b0cfbfe369f7a7b3cdb7d4494bc2823700cfd652188a3ea98d"],["af8addbf2b661c8a6c6328655eb96651252007d8c5ea31be4ad196de8ce2131f","6749e67c029b85f52a034eafd096836b2520818680e26ac8f3dfbcdb71749700"],["e3ae1974566ca06cc516d47e0fb165a674a3dabcfca15e722f0e3450f45889","2aeabe7e4531510116217f07bf4d07300de97e4874f81f533420a72eeb0bd6a4"],["591ee355313d99721cf6993ffed1e3e301993ff3ed258802075ea8ced397e246","b0ea558a113c30bea60fc4775460c7901ff0b053d25ca2bdeee98f1a4be5d196"],["11396d55fda54c49f19aa97318d8da61fa8584e47b084945077cf03255b52984","998c74a8cd45ac01289d5833a7beb4744ff536b01b257be4c5767bea93ea57a4"],["3c5d2a1ba39c5a1790000738c9e0c40b8dcdfd5468754b6405540157e017aa7a","b2284279995a34e2f9d4de7396fc18b80f9b8b9fdd270f6661f79ca4c81bd257"],["cc8704b8a60a0defa3a99a7299f2e9c3fbc395afb04ac078425ef8a1793cc030","bdd46039feed17881d1e0862db347f8cf395b74fc4bcdc4e940b74e3ac1f1b13"],["c533e4f7ea8555aacd9777ac5cad29b97dd4defccc53ee7ea204119b2889b197","6f0a256bc5efdf429a2fb6242f1a43a2d9b925bb4a4b3a26bb8e0f45eb596096"],["c14f8f2ccb27d6f109f6d08d03cc96a69ba8c34eec07bbcf566d48e33da6593","c359d6923bb398f7fd4473e16fe1c28475b740dd098075e6c0e8649113dc3a38"],["a6cbc3046bc6a450bac24789fa17115a4c9739ed75f8f21ce441f72e0b90e6ef","21ae7f4680e889bb130619e2c0f95a360ceb573c70603139862afd617fa9b9f"],["347d6d9a02c48927ebfb86c1359b1caf130a3c0267d11ce6344b39f99d43cc38","60ea7f61a353524d1c987f6ecec92f086d565ab687870cb12689ff1e31c74448"],["da6545d2181db8d983f7dcb375ef5866d47c67b1bf31c8cf855ef7437b72656a","49b96715ab6878a79e78f07ce5680c5d6673051b4935bd897fea824b77dc208a"],["c40747cc9d012cb1a13b8148309c6de7ec25d6945d657146b9d5994b8feb1111","5ca560753be2a12fc6de6caf2cb489565db936156b9514e1bb5e83037e0fa2d4"],["4e42c8ec82c99798ccf3a610be870e78338c7f713348bd34c8203ef4037f3502","7571d74ee5e0fb92a7a8b33a07783341a5492144cc54bcc40a94473693606437"],["3775ab7089bc6af823aba2e1af70b236d251cadb0c86743287522a1b3b0dedea","be52d107bcfa09d8bcb9736a828cfa7fac8db17bf7a76a2c42ad961409018cf7"],["cee31cbf7e34ec379d94fb814d3d775ad954595d1314ba8846959e3e82f74e26","8fd64a14c06b589c26b947ae2bcf6bfa0149ef0be14ed4d80f448a01c43b1c6d"],["b4f9eaea09b6917619f6ea6a4eb5464efddb58fd45b1ebefcdc1a01d08b47986","39e5c9925b5a54b07433a4f18c61726f8bb131c012ca542eb24a8ac07200682a"],["d4263dfc3d2df923a0179a48966d30ce84e2515afc3dccc1b77907792ebcc60e","62dfaf07a0f78feb30e30d6295853ce189e127760ad6cf7fae164e122a208d54"],["48457524820fa65a4f8d35eb6930857c0032acc0a4a2de422233eeda897612c4","25a748ab367979d98733c38a1fa1c2e7dc6cc07db2d60a9ae7a76aaa49bd0f77"],["dfeeef1881101f2cb11644f3a2afdfc2045e19919152923f367a1767c11cceda","ecfb7056cf1de042f9420bab396793c0c390bde74b4bbdff16a83ae09a9a7517"],["6d7ef6b17543f8373c573f44e1f389835d89bcbc6062ced36c82df83b8fae859","cd450ec335438986dfefa10c57fea9bcc521a0959b2d80bbf74b190dca712d10"],["e75605d59102a5a2684500d3b991f2e3f3c88b93225547035af25af66e04541f","f5c54754a8f71ee540b9b48728473e314f729ac5308b06938360990e2bfad125"],["eb98660f4c4dfaa06a2be453d5020bc99a0c2e60abe388457dd43fefb1ed620c","6cb9a8876d9cb8520609af3add26cd20a0a7cd8a9411131ce85f44100099223e"],["13e87b027d8514d35939f2e6892b19922154596941888336dc3563e3b8dba942","fef5a3c68059a6dec5d624114bf1e91aac2b9da568d6abeb2570d55646b8adf1"],["ee163026e9fd6fe017c38f06a5be6fc125424b371ce2708e7bf4491691e5764a","1acb250f255dd61c43d94ccc670d0f58f49ae3fa15b96623e5430da0ad6c62b2"],["b268f5ef9ad51e4d78de3a750c2dc89b1e626d43505867999932e5db33af3d80","5f310d4b3c99b9ebb19f77d41c1dee018cf0d34fd4191614003e945a1216e423"],["ff07f3118a9df035e9fad85eb6c7bfe42b02f01ca99ceea3bf7ffdba93c4750d","438136d603e858a3a5c440c38eccbaddc1d2942114e2eddd4740d098ced1f0d8"],["8d8b9855c7c052a34146fd20ffb658bea4b9f69e0d825ebec16e8c3ce2b526a1","cdb559eedc2d79f926baf44fb84ea4d44bcf50fee51d7ceb30e2e7f463036758"],["52db0b5384dfbf05bfa9d472d7ae26dfe4b851ceca91b1eba54263180da32b63","c3b997d050ee5d423ebaf66a6db9f57b3180c902875679de924b69d84a7b375"],["e62f9490d3d51da6395efd24e80919cc7d0f29c3f3fa48c6fff543becbd43352","6d89ad7ba4876b0b22c2ca280c682862f342c8591f1daf5170e07bfd9ccafa7d"],["7f30ea2476b399b4957509c88f77d0191afa2ff5cb7b14fd6d8e7d65aaab1193","ca5ef7d4b231c94c3b15389a5f6311e9daff7bb67b103e9880ef4bff637acaec"],["5098ff1e1d9f14fb46a210fada6c903fef0fb7b4a1dd1d9ac60a0361800b7a00","9731141d81fc8f8084d37c6e7542006b3ee1b40d60dfe5362a5b132fd17ddc0"],["32b78c7de9ee512a72895be6b9cbefa6e2f3c4ccce445c96b9f2c81e2778ad58","ee1849f513df71e32efc3896ee28260c73bb80547ae2275ba497237794c8753c"],["e2cb74fddc8e9fbcd076eef2a7c72b0ce37d50f08269dfc074b581550547a4f7","d3aa2ed71c9dd2247a62df062736eb0baddea9e36122d2be8641abcb005cc4a4"],["8438447566d4d7bedadc299496ab357426009a35f235cb141be0d99cd10ae3a8","c4e1020916980a4da5d01ac5e6ad330734ef0d7906631c4f2390426b2edd791f"],["4162d488b89402039b584c6fc6c308870587d9c46f660b878ab65c82c711d67e","67163e903236289f776f22c25fb8a3afc1732f2b84b4e95dbda47ae5a0852649"],["3fad3fa84caf0f34f0f89bfd2dcf54fc175d767aec3e50684f3ba4a4bf5f683d","cd1bc7cb6cc407bb2f0ca647c718a730cf71872e7d0d2a53fa20efcdfe61826"],["674f2600a3007a00568c1a7ce05d0816c1fb84bf1370798f1c69532faeb1a86b","299d21f9413f33b3edf43b257004580b70db57da0b182259e09eecc69e0d38a5"],["d32f4da54ade74abb81b815ad1fb3b263d82d6c692714bcff87d29bd5ee9f08f","f9429e738b8e53b968e99016c059707782e14f4535359d582fc416910b3eea87"],["30e4e670435385556e593657135845d36fbb6931f72b08cb1ed954f1e3ce3ff6","462f9bce619898638499350113bbc9b10a878d35da70740dc695a559eb88db7b"],["be2062003c51cc3004682904330e4dee7f3dcd10b01e580bf1971b04d4cad297","62188bc49d61e5428573d48a74e1c655b1c61090905682a0d5558ed72dccb9bc"],["93144423ace3451ed29e0fb9ac2af211cb6e84a601df5993c419859fff5df04a","7c10dfb164c3425f5c71a3f9d7992038f1065224f72bb9d1d902a6d13037b47c"],["b015f8044f5fcbdcf21ca26d6c34fb8197829205c7b7d2a7cb66418c157b112c","ab8c1e086d04e813744a655b2df8d5f83b3cdc6faa3088c1d3aea1454e3a1d5f"],["d5e9e1da649d97d89e4868117a465a3a4f8a18de57a140d36b3f2af341a21b52","4cb04437f391ed73111a13cc1d4dd0db1693465c2240480d8955e8592f27447a"],["d3ae41047dd7ca065dbf8ed77b992439983005cd72e16d6f996a5316d36966bb","bd1aeb21ad22ebb22a10f0303417c6d964f8cdd7df0aca614b10dc14d125ac46"],["463e2763d885f958fc66cdd22800f0a487197d0a82e377b49f80af87c897b065","bfefacdb0e5d0fd7df3a311a94de062b26b80c61fbc97508b79992671ef7ca7f"],["7985fdfd127c0567c6f53ec1bb63ec3158e597c40bfe747c83cddfc910641917","603c12daf3d9862ef2b25fe1de289aed24ed291e0ec6708703a5bd567f32ed03"],["74a1ad6b5f76e39db2dd249410eac7f99e74c59cb83d2d0ed5ff1543da7703e9","cc6157ef18c9c63cd6193d83631bbea0093e0968942e8c33d5737fd790e0db08"],["30682a50703375f602d416664ba19b7fc9bab42c72747463a71d0896b22f6da3","553e04f6b018b4fa6c8f39e7f311d3176290d0e0f19ca73f17714d9977a22ff8"],["9e2158f0d7c0d5f26c3791efefa79597654e7a2b2464f52b1ee6c1347769ef57","712fcdd1b9053f09003a3481fa7762e9ffd7c8ef35a38509e2fbf2629008373"],["176e26989a43c9cfeba4029c202538c28172e566e3c4fce7322857f3be327d66","ed8cc9d04b29eb877d270b4878dc43c19aefd31f4eee09ee7b47834c1fa4b1c3"],["75d46efea3771e6e68abb89a13ad747ecf1892393dfc4f1b7004788c50374da8","9852390a99507679fd0b86fd2b39a868d7efc22151346e1a3ca4726586a6bed8"],["809a20c67d64900ffb698c4c825f6d5f2310fb0451c869345b7319f645605721","9e994980d9917e22b76b061927fa04143d096ccc54963e6a5ebfa5f3f8e286c1"],["1b38903a43f7f114ed4500b4eac7083fdefece1cf29c63528d563446f972c180","4036edc931a60ae889353f77fd53de4a2708b26b6f5da72ad3394119daf408f9"]]}}},function(t,e,n){"use strict";var r=n(53),i=n(825),o=n(78),a=n(280),s=n(277),A=o.assert,l=n(826),c=n(827);function u(t){if(!(this instanceof u))return new u(t);"string"==typeof t&&(A(a.hasOwnProperty(t),"Unknown curve "+t),t=a[t]),t instanceof a.PresetCurve&&(t={curve:t}),this.curve=t.curve.curve,this.n=this.curve.n,this.nh=this.n.ushrn(1),this.g=this.curve.g,this.g=t.curve.g,this.g.precompute(t.curve.n.bitLength()+1),this.hash=t.hash||t.curve.hash}t.exports=u,u.prototype.keyPair=function(t){return new l(this,t)},u.prototype.keyFromPrivate=function(t,e){return l.fromPrivate(this,t,e)},u.prototype.keyFromPublic=function(t,e){return l.fromPublic(this,t,e)},u.prototype.genKeyPair=function(t){t||(t={});for(var e=new i({hash:this.hash,pers:t.pers,persEnc:t.persEnc||"utf8",entropy:t.entropy||s(this.hash.hmacStrength),entropyEnc:t.entropy&&t.entropyEnc||"utf8",nonce:this.n.toArray()}),n=this.n.byteLength(),o=this.n.sub(new r(2));;){var a=new r(e.generate(n));if(!(a.cmp(o)>0))return a.iaddn(1),this.keyFromPrivate(a)}},u.prototype._truncateToN=function(t,e){var n=8*t.byteLength()-this.n.bitLength();return n>0&&(t=t.ushrn(n)),!e&&t.cmp(this.n)>=0?t.sub(this.n):t},u.prototype.sign=function(t,e,n,o){"object"==typeof n&&(o=n,n=null),o||(o={}),e=this.keyFromPrivate(e,n),t=this._truncateToN(new r(t,16));for(var a=this.n.byteLength(),s=e.getPrivate().toArray("be",a),A=t.toArray("be",a),l=new i({hash:this.hash,entropy:s,nonce:A,pers:o.pers,persEnc:o.persEnc||"utf8"}),u=this.n.sub(new r(1)),g=0;;g++){var h=o.k?o.k(g):new r(l.generate(this.n.byteLength()));if(!((h=this._truncateToN(h,!0)).cmpn(1)<=0||h.cmp(u)>=0)){var d=this.g.mul(h);if(!d.isInfinity()){var f=d.getX(),p=f.umod(this.n);if(0!==p.cmpn(0)){var I=h.invm(this.n).mul(p.mul(e.getPrivate()).iadd(t));if(0!==(I=I.umod(this.n)).cmpn(0)){var C=(d.getY().isOdd()?1:0)|(0!==f.cmp(p)?2:0);return o.canonical&&I.cmp(this.nh)>0&&(I=this.n.sub(I),C^=1),new c({r:p,s:I,recoveryParam:C})}}}}}},u.prototype.verify=function(t,e,n,i){t=this._truncateToN(new r(t,16)),n=this.keyFromPublic(n,i);var o=(e=new c(e,"hex")).r,a=e.s;if(o.cmpn(1)<0||o.cmp(this.n)>=0)return!1;if(a.cmpn(1)<0||a.cmp(this.n)>=0)return!1;var s,A=a.invm(this.n),l=A.mul(t).umod(this.n),u=A.mul(o).umod(this.n);return this.curve._maxwellTrick?!(s=this.g.jmulAdd(l,n.getPublic(),u)).isInfinity()&&s.eqXToP(o):!(s=this.g.mulAdd(l,n.getPublic(),u)).isInfinity()&&0===s.getX().umod(this.n).cmp(o)},u.prototype.recoverPubKey=function(t,e,n,i){A((3&n)===n,"The recovery param is more than two bits"),e=new c(e,i);var o=this.n,a=new r(t),s=e.r,l=e.s,u=1&n,g=n>>1;if(s.cmp(this.curve.p.umod(this.curve.n))>=0&&g)throw new Error("Unable to find sencond key candinate");s=g?this.curve.pointFromX(s.add(this.curve.n),u):this.curve.pointFromX(s,u);var h=e.r.invm(o),d=o.sub(a).mul(h).umod(o),f=l.mul(h).umod(o);return this.g.mulAdd(d,s,f)},u.prototype.getKeyRecoveryParam=function(t,e,n,r){if(null!==(e=new c(e,r)).recoveryParam)return e.recoveryParam;for(var i=0;i<4;i++){var o;try{o=this.recoverPubKey(t,e,i)}catch(t){continue}if(o.eq(n))return i}throw new Error("Unable to find valid recovery factor")}},function(t,e,n){"use strict";var r=n(281),i=n(417),o=n(77);function a(t){if(!(this instanceof a))return new a(t);this.hash=t.hash,this.predResist=!!t.predResist,this.outLen=this.hash.outSize,this.minEntropy=t.minEntropy||this.hash.hmacStrength,this._reseed=null,this.reseedInterval=null,this.K=null,this.V=null;var e=i.toArray(t.entropy,t.entropyEnc||"hex"),n=i.toArray(t.nonce,t.nonceEnc||"hex"),r=i.toArray(t.pers,t.persEnc||"hex");o(e.length>=this.minEntropy/8,"Not enough entropy. Minimum is: "+this.minEntropy+" bits"),this._init(e,n,r)}t.exports=a,a.prototype._init=function(t,e,n){var r=t.concat(e).concat(n);this.K=new Array(this.outLen/8),this.V=new Array(this.outLen/8);for(var i=0;i<this.V.length;i++)this.K[i]=0,this.V[i]=1;this._update(r),this._reseed=1,this.reseedInterval=281474976710656},a.prototype._hmac=function(){return new r.hmac(this.hash,this.K)},a.prototype._update=function(t){var e=this._hmac().update(this.V).update([0]);t&&(e=e.update(t)),this.K=e.digest(),this.V=this._hmac().update(this.V).digest(),t&&(this.K=this._hmac().update(this.V).update([1]).update(t).digest(),this.V=this._hmac().update(this.V).digest())},a.prototype.reseed=function(t,e,n,r){"string"!=typeof e&&(r=n,n=e,e=null),t=i.toArray(t,e),n=i.toArray(n,r),o(t.length>=this.minEntropy/8,"Not enough entropy. Minimum is: "+this.minEntropy+" bits"),this._update(t.concat(n||[])),this._reseed=1},a.prototype.generate=function(t,e,n,r){if(this._reseed>this.reseedInterval)throw new Error("Reseed is required");"string"!=typeof e&&(r=n,n=e,e=null),n&&(n=i.toArray(n,r||"hex"),this._update(n));for(var o=[];o.length<t;)this.V=this._hmac().update(this.V).digest(),o=o.concat(this.V);var a=o.slice(0,t);return this._update(n),this._reseed++,i.encode(a,e)}},function(t,e,n){"use strict";var r=n(53),i=n(78).assert;function o(t,e){this.ec=t,this.priv=null,this.pub=null,e.priv&&this._importPrivate(e.priv,e.privEnc),e.pub&&this._importPublic(e.pub,e.pubEnc)}t.exports=o,o.fromPublic=function(t,e,n){return e instanceof o?e:new o(t,{pub:e,pubEnc:n})},o.fromPrivate=function(t,e,n){return e instanceof o?e:new o(t,{priv:e,privEnc:n})},o.prototype.validate=function(){var t=this.getPublic();return t.isInfinity()?{result:!1,reason:"Invalid public key"}:t.validate()?t.mul(this.ec.curve.n).isInfinity()?{result:!0,reason:null}:{result:!1,reason:"Public key * N != O"}:{result:!1,reason:"Public key is not a point"}},o.prototype.getPublic=function(t,e){return"string"==typeof t&&(e=t,t=null),this.pub||(this.pub=this.ec.g.mul(this.priv)),e?this.pub.encode(e,t):this.pub},o.prototype.getPrivate=function(t){return"hex"===t?this.priv.toString(16,2):this.priv},o.prototype._importPrivate=function(t,e){this.priv=new r(t,e||16),this.priv=this.priv.umod(this.ec.curve.n)},o.prototype._importPublic=function(t,e){if(t.x||t.y)return"mont"===this.ec.curve.type?i(t.x,"Need x coordinate"):"short"!==this.ec.curve.type&&"edwards"!==this.ec.curve.type||i(t.x&&t.y,"Need both x and y coordinate"),void(this.pub=this.ec.curve.point(t.x,t.y));this.pub=this.ec.curve.decodePoint(t,e)},o.prototype.derive=function(t){return t.mul(this.priv).getX()},o.prototype.sign=function(t,e,n){return this.ec.sign(t,this,e,n)},o.prototype.verify=function(t,e){return this.ec.verify(t,e,this)},o.prototype.inspect=function(){return"<Key priv: "+(this.priv&&this.priv.toString(16,2))+" pub: "+(this.pub&&this.pub.inspect())+" >"}},function(t,e,n){"use strict";var r=n(53),i=n(78),o=i.assert;function a(t,e){if(t instanceof a)return t;this._importDER(t,e)||(o(t.r&&t.s,"Signature without r or s"),this.r=new r(t.r,16),this.s=new r(t.s,16),void 0===t.recoveryParam?this.recoveryParam=null:this.recoveryParam=t.recoveryParam)}function s(){this.place=0}function A(t,e){var n=t[e.place++];if(!(128&n))return n;for(var r=15&n,i=0,o=0,a=e.place;o<r;o++,a++)i<<=8,i|=t[a];return e.place=a,i}function l(t){for(var e=0,n=t.length-1;!t[e]&&!(128&t[e+1])&&e<n;)e++;return 0===e?t:t.slice(e)}function c(t,e){if(e<128)t.push(e);else{var n=1+(Math.log(e)/Math.LN2>>>3);for(t.push(128|n);--n;)t.push(e>>>(n<<3)&255);t.push(e)}}t.exports=a,a.prototype._importDER=function(t,e){t=i.toArray(t,e);var n=new s;if(48!==t[n.place++])return!1;if(A(t,n)+n.place!==t.length)return!1;if(2!==t[n.place++])return!1;var o=A(t,n),a=t.slice(n.place,o+n.place);if(n.place+=o,2!==t[n.place++])return!1;var l=A(t,n);if(t.length!==l+n.place)return!1;var c=t.slice(n.place,l+n.place);return 0===a[0]&&128&a[1]&&(a=a.slice(1)),0===c[0]&&128&c[1]&&(c=c.slice(1)),this.r=new r(a),this.s=new r(c),this.recoveryParam=null,!0},a.prototype.toDER=function(t){var e=this.r.toArray(),n=this.s.toArray();for(128&e[0]&&(e=[0].concat(e)),128&n[0]&&(n=[0].concat(n)),e=l(e),n=l(n);!(n[0]||128&n[1]);)n=n.slice(1);var r=[2];c(r,e.length),(r=r.concat(e)).push(2),c(r,n.length);var o=r.concat(n),a=[48];return c(a,o.length),a=a.concat(o),i.encode(a,t)}},function(t,e,n){"use strict";var r=n(281),i=n(280),o=n(78),a=o.assert,s=o.parseBytes,A=n(829),l=n(830);function c(t){if(a("ed25519"===t,"only tested with ed25519 so far"),!(this instanceof c))return new c(t);t=i[t].curve;this.curve=t,this.g=t.g,this.g.precompute(t.n.bitLength()+1),this.pointClass=t.point().constructor,this.encodingLength=Math.ceil(t.n.bitLength()/8),this.hash=r.sha512}t.exports=c,c.prototype.sign=function(t,e){t=s(t);var n=this.keyFromSecret(e),r=this.hashInt(n.messagePrefix(),t),i=this.g.mul(r),o=this.encodePoint(i),a=this.hashInt(o,n.pubBytes(),t).mul(n.priv()),A=r.add(a).umod(this.curve.n);return this.makeSignature({R:i,S:A,Rencoded:o})},c.prototype.verify=function(t,e,n){t=s(t),e=this.makeSignature(e);var r=this.keyFromPublic(n),i=this.hashInt(e.Rencoded(),r.pubBytes(),t),o=this.g.mul(e.S());return e.R().add(r.pub().mul(i)).eq(o)},c.prototype.hashInt=function(){for(var t=this.hash(),e=0;e<arguments.length;e++)t.update(arguments[e]);return o.intFromLE(t.digest()).umod(this.curve.n)},c.prototype.keyFromPublic=function(t){return A.fromPublic(this,t)},c.prototype.keyFromSecret=function(t){return A.fromSecret(this,t)},c.prototype.makeSignature=function(t){return t instanceof l?t:new l(this,t)},c.prototype.encodePoint=function(t){var e=t.getY().toArray("le",this.encodingLength);return e[this.encodingLength-1]|=t.getX().isOdd()?128:0,e},c.prototype.decodePoint=function(t){var e=(t=o.parseBytes(t)).length-1,n=t.slice(0,e).concat(-129&t[e]),r=0!=(128&t[e]),i=o.intFromLE(n);return this.curve.pointFromY(i,r)},c.prototype.encodeInt=function(t){return t.toArray("le",this.encodingLength)},c.prototype.decodeInt=function(t){return o.intFromLE(t)},c.prototype.isPoint=function(t){return t instanceof this.pointClass}},function(t,e,n){"use strict";var r=n(78),i=r.assert,o=r.parseBytes,a=r.cachedProperty;function s(t,e){this.eddsa=t,this._secret=o(e.secret),t.isPoint(e.pub)?this._pub=e.pub:this._pubBytes=o(e.pub)}s.fromPublic=function(t,e){return e instanceof s?e:new s(t,{pub:e})},s.fromSecret=function(t,e){return e instanceof s?e:new s(t,{secret:e})},s.prototype.secret=function(){return this._secret},a(s,"pubBytes",function(){return this.eddsa.encodePoint(this.pub())}),a(s,"pub",function(){return this._pubBytes?this.eddsa.decodePoint(this._pubBytes):this.eddsa.g.mul(this.priv())}),a(s,"privBytes",function(){var t=this.eddsa,e=this.hash(),n=t.encodingLength-1,r=e.slice(0,t.encodingLength);return r[0]&=248,r[n]&=127,r[n]|=64,r}),a(s,"priv",function(){return this.eddsa.decodeInt(this.privBytes())}),a(s,"hash",function(){return this.eddsa.hash().update(this.secret()).digest()}),a(s,"messagePrefix",function(){return this.hash().slice(this.eddsa.encodingLength)}),s.prototype.sign=function(t){return i(this._secret,"KeyPair can only verify"),this.eddsa.sign(t,this)},s.prototype.verify=function(t,e){return this.eddsa.verify(t,e,this)},s.prototype.getSecret=function(t){return i(this._secret,"KeyPair is public only"),r.encode(this.secret(),t)},s.prototype.getPublic=function(t){return r.encode(this.pubBytes(),t)},t.exports=s},function(t,e,n){"use strict";var r=n(53),i=n(78),o=i.assert,a=i.cachedProperty,s=i.parseBytes;function A(t,e){this.eddsa=t,"object"!=typeof e&&(e=s(e)),Array.isArray(e)&&(e={R:e.slice(0,t.encodingLength),S:e.slice(t.encodingLength)}),o(e.R&&e.S,"Signature without R or S"),t.isPoint(e.R)&&(this._R=e.R),e.S instanceof r&&(this._S=e.S),this._Rencoded=Array.isArray(e.R)?e.R:e.Rencoded,this._Sencoded=Array.isArray(e.S)?e.S:e.Sencoded}a(A,"S",function(){return this.eddsa.decodeInt(this.Sencoded())}),a(A,"R",function(){return this.eddsa.decodePoint(this.Rencoded())}),a(A,"Rencoded",function(){return this.eddsa.encodePoint(this.R())}),a(A,"Sencoded",function(){return this.eddsa.encodeInt(this.S())}),A.prototype.toBytes=function(){return this.Rencoded().concat(this.Sencoded())},A.prototype.toHex=function(){return i.encode(this.toBytes(),"hex").toUpperCase()},t.exports=A},function(t,e,n){"use strict";var r=n(169);e.certificate=n(841);var i=r.define("RSAPrivateKey",function(){this.seq().obj(this.key("version").int(),this.key("modulus").int(),this.key("publicExponent").int(),this.key("privateExponent").int(),this.key("prime1").int(),this.key("prime2").int(),this.key("exponent1").int(),this.key("exponent2").int(),this.key("coefficient").int())});e.RSAPrivateKey=i;var o=r.define("RSAPublicKey",function(){this.seq().obj(this.key("modulus").int(),this.key("publicExponent").int())});e.RSAPublicKey=o;var a=r.define("SubjectPublicKeyInfo",function(){this.seq().obj(this.key("algorithm").use(s),this.key("subjectPublicKey").bitstr())});e.PublicKey=a;var s=r.define("AlgorithmIdentifier",function(){this.seq().obj(this.key("algorithm").objid(),this.key("none").null_().optional(),this.key("curve").objid().optional(),this.key("params").seq().obj(this.key("p").int(),this.key("q").int(),this.key("g").int()).optional())}),A=r.define("PrivateKeyInfo",function(){this.seq().obj(this.key("version").int(),this.key("algorithm").use(s),this.key("subjectPrivateKey").octstr())});e.PrivateKey=A;var l=r.define("EncryptedPrivateKeyInfo",function(){this.seq().obj(this.key("algorithm").seq().obj(this.key("id").objid(),this.key("decrypt").seq().obj(this.key("kde").seq().obj(this.key("id").objid(),this.key("kdeparams").seq().obj(this.key("salt").octstr(),this.key("iters").int())),this.key("cipher").seq().obj(this.key("algo").objid(),this.key("iv").octstr()))),this.key("subjectPrivateKey").octstr())});e.EncryptedPrivateKey=l;var c=r.define("DSAPrivateKey",function(){this.seq().obj(this.key("version").int(),this.key("p").int(),this.key("q").int(),this.key("g").int(),this.key("pub_key").int(),this.key("priv_key").int())});e.DSAPrivateKey=c,e.DSAparam=r.define("DSAparam",function(){this.int()});var u=r.define("ECPrivateKey",function(){this.seq().obj(this.key("version").int(),this.key("privateKey").octstr(),this.key("parameters").optional().explicit(0).use(g),this.key("publicKey").optional().explicit(1).bitstr())});e.ECPrivateKey=u;var g=r.define("ECParameters",function(){this.choice({namedCurve:this.objid()})});e.signature=r.define("signature",function(){this.seq().obj(this.key("r").int(),this.key("s").int())})},function(t,e,n){var r=n(169),i=n(30);function o(t,e){this.name=t,this.body=e,this.decoders={},this.encoders={}}e.define=function(t,e){return new o(t,e)},o.prototype._createNamed=function(t){var e;try{e=n(833).runInThisContext("(function "+this.name+"(entity) {\n  this._initNamed(entity);\n})")}catch(t){e=function(t){this._initNamed(t)}}return i(e,t),e.prototype._initNamed=function(e){t.call(this,e)},new e(this)},o.prototype._getDecoder=function(t){return t=t||"der",this.decoders.hasOwnProperty(t)||(this.decoders[t]=this._createNamed(r.decoders[t])),this.decoders[t]},o.prototype.decode=function(t,e,n){return this._getDecoder(e).decode(t,n)},o.prototype._getEncoder=function(t){return t=t||"der",this.encoders.hasOwnProperty(t)||(this.encoders[t]=this._createNamed(r.encoders[t])),this.encoders[t]},o.prototype.encode=function(t,e,n){return this._getEncoder(e).encode(t,n)}},function(module,exports){var indexOf=function(t,e){if(t.indexOf)return t.indexOf(e);for(var n=0;n<t.length;n++)if(t[n]===e)return n;return-1},Object_keys=function(t){if(Object.keys)return Object.keys(t);var e=[];for(var n in t)e.push(n);return e},forEach=function(t,e){if(t.forEach)return t.forEach(e);for(var n=0;n<t.length;n++)e(t[n],n,t)},defineProp=function(){try{return Object.defineProperty({},"_",{}),function(t,e,n){Object.defineProperty(t,e,{writable:!0,enumerable:!1,configurable:!0,value:n})}}catch(t){return function(t,e,n){t[e]=n}}}(),globals=["Array","Boolean","Date","Error","EvalError","Function","Infinity","JSON","Math","NaN","Number","Object","RangeError","ReferenceError","RegExp","String","SyntaxError","TypeError","URIError","decodeURI","decodeURIComponent","encodeURI","encodeURIComponent","escape","eval","isFinite","isNaN","parseFloat","parseInt","undefined","unescape"];function Context(){}Context.prototype={};var Script=exports.Script=function(t){if(!(this instanceof Script))return new Script(t);this.code=t};Script.prototype.runInContext=function(t){if(!(t instanceof Context))throw new TypeError("needs a 'context' argument.");var e=document.createElement("iframe");e.style||(e.style={}),e.style.display="none",document.body.appendChild(e);var n=e.contentWindow,r=n.eval,i=n.execScript;!r&&i&&(i.call(n,"null"),r=n.eval),forEach(Object_keys(t),function(e){n[e]=t[e]}),forEach(globals,function(e){t[e]&&(n[e]=t[e])});var o=Object_keys(n),a=r.call(n,this.code);return forEach(Object_keys(n),function(e){(e in t||-1===indexOf(o,e))&&(t[e]=n[e])}),forEach(globals,function(e){e in t||defineProp(t,e,n[e])}),document.body.removeChild(e),a},Script.prototype.runInThisContext=function(){return eval(this.code)},Script.prototype.runInNewContext=function(t){var e=Script.createContext(t),n=this.runInContext(e);return t&&forEach(Object_keys(e),function(n){t[n]=e[n]}),n},forEach(Object_keys(Script.prototype),function(t){exports[t]=Script[t]=function(e){var n=Script(e);return n[t].apply(n,[].slice.call(arguments,1))}}),exports.isContext=function(t){return t instanceof Context},exports.createScript=function(t){return exports.Script(t)},exports.createContext=Script.createContext=function(t){var e=new Context;return"object"==typeof t&&forEach(Object_keys(t),function(n){e[n]=t[n]}),e}},function(t,e,n){var r=n(30);function i(t){this._reporterState={obj:null,path:[],options:t||{},errors:[]}}function o(t,e){this.path=t,this.rethrow(e)}e.Reporter=i,i.prototype.isError=function(t){return t instanceof o},i.prototype.save=function(){var t=this._reporterState;return{obj:t.obj,pathLen:t.path.length}},i.prototype.restore=function(t){var e=this._reporterState;e.obj=t.obj,e.path=e.path.slice(0,t.pathLen)},i.prototype.enterKey=function(t){return this._reporterState.path.push(t)},i.prototype.exitKey=function(t){var e=this._reporterState;e.path=e.path.slice(0,t-1)},i.prototype.leaveKey=function(t,e,n){var r=this._reporterState;this.exitKey(t),null!==r.obj&&(r.obj[e]=n)},i.prototype.path=function(){return this._reporterState.path.join("/")},i.prototype.enterObject=function(){var t=this._reporterState,e=t.obj;return t.obj={},e},i.prototype.leaveObject=function(t){var e=this._reporterState,n=e.obj;return e.obj=t,n},i.prototype.error=function(t){var e,n=this._reporterState,r=t instanceof o;if(e=r?t:new o(n.path.map(function(t){return"["+JSON.stringify(t)+"]"}).join(""),t.message||t,t.stack),!n.options.partial)throw e;return r||n.errors.push(e),e},i.prototype.wrapResult=function(t){var e=this._reporterState;return e.options.partial?{result:this.isError(t)?null:t,errors:e.errors}:t},r(o,Error),o.prototype.rethrow=function(t){if(this.message=t+" at: "+(this.path||"(shallow)"),Error.captureStackTrace&&Error.captureStackTrace(this,o),!this.stack)try{throw new Error(this.message)}catch(t){this.stack=t.stack}return this}},function(t,e,n){var r=n(170).Reporter,i=n(170).EncoderBuffer,o=n(170).DecoderBuffer,a=n(77),s=["seq","seqof","set","setof","objid","bool","gentime","utctime","null_","enum","int","objDesc","bitstr","bmpstr","charstr","genstr","graphstr","ia5str","iso646str","numstr","octstr","printstr","t61str","unistr","utf8str","videostr"],A=["key","obj","use","optional","explicit","implicit","def","choice","any","contains"].concat(s);function l(t,e){var n={};this._baseState=n,n.enc=t,n.parent=e||null,n.children=null,n.tag=null,n.args=null,n.reverseArgs=null,n.choice=null,n.optional=!1,n.any=!1,n.obj=!1,n.use=null,n.useDecoder=null,n.key=null,n.default=null,n.explicit=null,n.implicit=null,n.contains=null,n.parent||(n.children=[],this._wrap())}t.exports=l;var c=["enc","parent","children","tag","args","reverseArgs","choice","optional","any","obj","use","alteredUse","key","default","explicit","implicit","contains"];l.prototype.clone=function(){var t=this._baseState,e={};c.forEach(function(n){e[n]=t[n]});var n=new this.constructor(e.parent);return n._baseState=e,n},l.prototype._wrap=function(){var t=this._baseState;A.forEach(function(e){this[e]=function(){var n=new this.constructor(this);return t.children.push(n),n[e].apply(n,arguments)}},this)},l.prototype._init=function(t){var e=this._baseState;a(null===e.parent),t.call(this),e.children=e.children.filter(function(t){return t._baseState.parent===this},this),a.equal(e.children.length,1,"Root node can have only one child")},l.prototype._useArgs=function(t){var e=this._baseState,n=t.filter(function(t){return t instanceof this.constructor},this);t=t.filter(function(t){return!(t instanceof this.constructor)},this),0!==n.length&&(a(null===e.children),e.children=n,n.forEach(function(t){t._baseState.parent=this},this)),0!==t.length&&(a(null===e.args),e.args=t,e.reverseArgs=t.map(function(t){if("object"!=typeof t||t.constructor!==Object)return t;var e={};return Object.keys(t).forEach(function(n){n==(0|n)&&(n|=0);var r=t[n];e[r]=n}),e}))},["_peekTag","_decodeTag","_use","_decodeStr","_decodeObjid","_decodeTime","_decodeNull","_decodeInt","_decodeBool","_decodeList","_encodeComposite","_encodeStr","_encodeObjid","_encodeTime","_encodeNull","_encodeInt","_encodeBool"].forEach(function(t){l.prototype[t]=function(){var e=this._baseState;throw new Error(t+" not implemented for encoding: "+e.enc)}}),s.forEach(function(t){l.prototype[t]=function(){var e=this._baseState,n=Array.prototype.slice.call(arguments);return a(null===e.tag),e.tag=t,this._useArgs(n),this}}),l.prototype.use=function(t){a(t);var e=this._baseState;return a(null===e.use),e.use=t,this},l.prototype.optional=function(){return this._baseState.optional=!0,this},l.prototype.def=function(t){var e=this._baseState;return a(null===e.default),e.default=t,e.optional=!0,this},l.prototype.explicit=function(t){var e=this._baseState;return a(null===e.explicit&&null===e.implicit),e.explicit=t,this},l.prototype.implicit=function(t){var e=this._baseState;return a(null===e.explicit&&null===e.implicit),e.implicit=t,this},l.prototype.obj=function(){var t=this._baseState,e=Array.prototype.slice.call(arguments);return t.obj=!0,0!==e.length&&this._useArgs(e),this},l.prototype.key=function(t){var e=this._baseState;return a(null===e.key),e.key=t,this},l.prototype.any=function(){return this._baseState.any=!0,this},l.prototype.choice=function(t){var e=this._baseState;return a(null===e.choice),e.choice=t,this._useArgs(Object.keys(t).map(function(e){return t[e]})),this},l.prototype.contains=function(t){var e=this._baseState;return a(null===e.use),e.contains=t,this},l.prototype._decode=function(t,e){var n=this._baseState;if(null===n.parent)return t.wrapResult(n.children[0]._decode(t,e));var r,i=n.default,a=!0,s=null;if(null!==n.key&&(s=t.enterKey(n.key)),n.optional){var A=null;if(null!==n.explicit?A=n.explicit:null!==n.implicit?A=n.implicit:null!==n.tag&&(A=n.tag),null!==A||n.any){if(a=this._peekTag(t,A,n.any),t.isError(a))return a}else{var l=t.save();try{null===n.choice?this._decodeGeneric(n.tag,t,e):this._decodeChoice(t,e),a=!0}catch(t){a=!1}t.restore(l)}}if(n.obj&&a&&(r=t.enterObject()),a){if(null!==n.explicit){var c=this._decodeTag(t,n.explicit);if(t.isError(c))return c;t=c}var u=t.offset;if(null===n.use&&null===n.choice){if(n.any)l=t.save();var g=this._decodeTag(t,null!==n.implicit?n.implicit:n.tag,n.any);if(t.isError(g))return g;n.any?i=t.raw(l):t=g}if(e&&e.track&&null!==n.tag&&e.track(t.path(),u,t.length,"tagged"),e&&e.track&&null!==n.tag&&e.track(t.path(),t.offset,t.length,"content"),i=n.any?i:null===n.choice?this._decodeGeneric(n.tag,t,e):this._decodeChoice(t,e),t.isError(i))return i;if(n.any||null!==n.choice||null===n.children||n.children.forEach(function(n){n._decode(t,e)}),n.contains&&("octstr"===n.tag||"bitstr"===n.tag)){var h=new o(i);i=this._getUse(n.contains,t._reporterState.obj)._decode(h,e)}}return n.obj&&a&&(i=t.leaveObject(r)),null===n.key||null===i&&!0!==a?null!==s&&t.exitKey(s):t.leaveKey(s,n.key,i),i},l.prototype._decodeGeneric=function(t,e,n){var r=this._baseState;return"seq"===t||"set"===t?null:"seqof"===t||"setof"===t?this._decodeList(e,t,r.args[0],n):/str$/.test(t)?this._decodeStr(e,t,n):"objid"===t&&r.args?this._decodeObjid(e,r.args[0],r.args[1],n):"objid"===t?this._decodeObjid(e,null,null,n):"gentime"===t||"utctime"===t?this._decodeTime(e,t,n):"null_"===t?this._decodeNull(e,n):"bool"===t?this._decodeBool(e,n):"objDesc"===t?this._decodeStr(e,t,n):"int"===t||"enum"===t?this._decodeInt(e,r.args&&r.args[0],n):null!==r.use?this._getUse(r.use,e._reporterState.obj)._decode(e,n):e.error("unknown tag: "+t)},l.prototype._getUse=function(t,e){var n=this._baseState;return n.useDecoder=this._use(t,e),a(null===n.useDecoder._baseState.parent),n.useDecoder=n.useDecoder._baseState.children[0],n.implicit!==n.useDecoder._baseState.implicit&&(n.useDecoder=n.useDecoder.clone(),n.useDecoder._baseState.implicit=n.implicit),n.useDecoder},l.prototype._decodeChoice=function(t,e){var n=this._baseState,r=null,i=!1;return Object.keys(n.choice).some(function(o){var a=t.save(),s=n.choice[o];try{var A=s._decode(t,e);if(t.isError(A))return!1;r={type:o,value:A},i=!0}catch(e){return t.restore(a),!1}return!0},this),i?r:t.error("Choice not matched")},l.prototype._createEncoderBuffer=function(t){return new i(t,this.reporter)},l.prototype._encode=function(t,e,n){var r=this._baseState;if(null===r.default||r.default!==t){var i=this._encodeValue(t,e,n);if(void 0!==i&&!this._skipDefault(i,e,n))return i}},l.prototype._encodeValue=function(t,e,n){var i=this._baseState;if(null===i.parent)return i.children[0]._encode(t,e||new r);var o=null;if(this.reporter=e,i.optional&&void 0===t){if(null===i.default)return;t=i.default}var a=null,s=!1;if(i.any)o=this._createEncoderBuffer(t);else if(i.choice)o=this._encodeChoice(t,e);else if(i.contains)a=this._getUse(i.contains,n)._encode(t,e),s=!0;else if(i.children)a=i.children.map(function(n){if("null_"===n._baseState.tag)return n._encode(null,e,t);if(null===n._baseState.key)return e.error("Child should have a key");var r=e.enterKey(n._baseState.key);if("object"!=typeof t)return e.error("Child expected, but input is not object");var i=n._encode(t[n._baseState.key],e,t);return e.leaveKey(r),i},this).filter(function(t){return t}),a=this._createEncoderBuffer(a);else if("seqof"===i.tag||"setof"===i.tag){if(!i.args||1!==i.args.length)return e.error("Too many args for : "+i.tag);if(!Array.isArray(t))return e.error("seqof/setof, but data is not Array");var A=this.clone();A._baseState.implicit=null,a=this._createEncoderBuffer(t.map(function(n){var r=this._baseState;return this._getUse(r.args[0],t)._encode(n,e)},A))}else null!==i.use?o=this._getUse(i.use,n)._encode(t,e):(a=this._encodePrimitive(i.tag,t),s=!0);if(!i.any&&null===i.choice){var l=null!==i.implicit?i.implicit:i.tag,c=null===i.implicit?"universal":"context";null===l?null===i.use&&e.error("Tag could be omitted only for .use()"):null===i.use&&(o=this._encodeComposite(l,s,c,a))}return null!==i.explicit&&(o=this._encodeComposite(i.explicit,!1,"context",o)),o},l.prototype._encodeChoice=function(t,e){var n=this._baseState,r=n.choice[t.type];return r||a(!1,t.type+" not found in "+JSON.stringify(Object.keys(n.choice))),r._encode(t.value,e)},l.prototype._encodePrimitive=function(t,e){var n=this._baseState;if(/str$/.test(t))return this._encodeStr(e,t);if("objid"===t&&n.args)return this._encodeObjid(e,n.reverseArgs[0],n.args[1]);if("objid"===t)return this._encodeObjid(e,null,null);if("gentime"===t||"utctime"===t)return this._encodeTime(e,t);if("null_"===t)return this._encodeNull();if("int"===t||"enum"===t)return this._encodeInt(e,n.args&&n.reverseArgs[0]);if("bool"===t)return this._encodeBool(e);if("objDesc"===t)return this._encodeStr(e,t);throw new Error("Unsupported tag: "+t)},l.prototype._isNumstr=function(t){return/^[0-9 ]*$/.test(t)},l.prototype._isPrintstr=function(t){return/^[A-Za-z0-9 '\(\)\+,\-\.\/:=\?]*$/.test(t)}},function(t,e,n){var r=n(423);e.tagClass={0:"universal",1:"application",2:"context",3:"private"},e.tagClassByName=r._reverse(e.tagClass),e.tag={0:"end",1:"bool",2:"int",3:"bitstr",4:"octstr",5:"null_",6:"objid",7:"objDesc",8:"external",9:"real",10:"enum",11:"embed",12:"utf8str",13:"relativeOid",16:"seq",17:"set",18:"numstr",19:"printstr",20:"t61str",21:"videostr",22:"ia5str",23:"utctime",24:"gentime",25:"graphstr",26:"iso646str",27:"genstr",28:"unistr",29:"charstr",30:"bmpstr"},e.tagByName=r._reverse(e.tag)},function(t,e,n){var r=e;r.der=n(424),r.pem=n(838)},function(t,e,n){var r=n(30),i=n(42).Buffer,o=n(424);function a(t){o.call(this,t),this.enc="pem"}r(a,o),t.exports=a,a.prototype.decode=function(t,e){for(var n=t.toString().split(/[\r\n]+/g),r=e.label.toUpperCase(),a=/^-----(BEGIN|END) ([^-]+)-----$/,s=-1,A=-1,l=0;l<n.length;l++){var c=n[l].match(a);if(null!==c&&c[2]===r){if(-1!==s){if("END"!==c[1])break;A=l;break}if("BEGIN"!==c[1])break;s=l}}if(-1===s||-1===A)throw new Error("PEM section not found for: "+r);var u=n.slice(s+1,A).join("");u.replace(/[^a-z0-9\+\/=]+/gi,"");var g=new i(u,"base64");return o.prototype.decode.call(this,g,e)}},function(t,e,n){var r=e;r.der=n(425),r.pem=n(840)},function(t,e,n){var r=n(30),i=n(425);function o(t){i.call(this,t),this.enc="pem"}r(o,i),t.exports=o,o.prototype.encode=function(t,e){for(var n=i.prototype.encode.call(this,t).toString("base64"),r=["-----BEGIN "+e.label+"-----"],o=0;o<n.length;o+=64)r.push(n.slice(o,o+64));return r.push("-----END "+e.label+"-----"),r.join("\n")}},function(t,e,n){"use strict";var r=n(169),i=r.define("Time",function(){this.choice({utcTime:this.utctime(),generalTime:this.gentime()})}),o=r.define("AttributeTypeValue",function(){this.seq().obj(this.key("type").objid(),this.key("value").any())}),a=r.define("AlgorithmIdentifier",function(){this.seq().obj(this.key("algorithm").objid(),this.key("parameters").optional(),this.key("curve").objid().optional())}),s=r.define("SubjectPublicKeyInfo",function(){this.seq().obj(this.key("algorithm").use(a),this.key("subjectPublicKey").bitstr())}),A=r.define("RelativeDistinguishedName",function(){this.setof(o)}),l=r.define("RDNSequence",function(){this.seqof(A)}),c=r.define("Name",function(){this.choice({rdnSequence:this.use(l)})}),u=r.define("Validity",function(){this.seq().obj(this.key("notBefore").use(i),this.key("notAfter").use(i))}),g=r.define("Extension",function(){this.seq().obj(this.key("extnID").objid(),this.key("critical").bool().def(!1),this.key("extnValue").octstr())}),h=r.define("TBSCertificate",function(){this.seq().obj(this.key("version").explicit(0).int().optional(),this.key("serialNumber").int(),this.key("signature").use(a),this.key("issuer").use(c),this.key("validity").use(u),this.key("subject").use(c),this.key("subjectPublicKeyInfo").use(s),this.key("issuerUniqueID").implicit(1).bitstr().optional(),this.key("subjectUniqueID").implicit(2).bitstr().optional(),this.key("extensions").explicit(3).seqof(g).optional())}),d=r.define("X509Certificate",function(){this.seq().obj(this.key("tbsCertificate").use(h),this.key("signatureAlgorithm").use(a),this.key("signatureValue").bitstr())});t.exports=d},function(t){t.exports=JSON.parse('{"2.16.840.1.101.3.4.1.1":"aes-128-ecb","2.16.840.1.101.3.4.1.2":"aes-128-cbc","2.16.840.1.101.3.4.1.3":"aes-128-ofb","2.16.840.1.101.3.4.1.4":"aes-128-cfb","2.16.840.1.101.3.4.1.21":"aes-192-ecb","2.16.840.1.101.3.4.1.22":"aes-192-cbc","2.16.840.1.101.3.4.1.23":"aes-192-ofb","2.16.840.1.101.3.4.1.24":"aes-192-cfb","2.16.840.1.101.3.4.1.41":"aes-256-ecb","2.16.840.1.101.3.4.1.42":"aes-256-cbc","2.16.840.1.101.3.4.1.43":"aes-256-ofb","2.16.840.1.101.3.4.1.44":"aes-256-cfb"}')},function(t,e,n){var r=/Proc-Type: 4,ENCRYPTED[\n\r]+DEK-Info: AES-((?:128)|(?:192)|(?:256))-CBC,([0-9A-H]+)[\n\r]+([0-9A-z\n\r\+\/\=]+)[\n\r]+/m,i=/^-----BEGIN ((?:.*? KEY)|CERTIFICATE)-----/m,o=/^-----BEGIN ((?:.*? KEY)|CERTIFICATE)-----([0-9A-z\n\r\+\/\=]+)-----END \1-----$/m,a=n(212),s=n(275),A=n(35).Buffer;t.exports=function(t,e){var n,l=t.toString(),c=l.match(r);if(c){var u="aes"+c[1],g=A.from(c[2],"hex"),h=A.from(c[3].replace(/[\r\n]/g,""),"base64"),d=a(e,g.slice(0,8),parseInt(c[1],10)).key,f=[],p=s.createDecipheriv(u,d,g);f.push(p.update(h)),f.push(p.final()),n=A.concat(f)}else{var I=l.match(o);n=new A(I[2].replace(/[\r\n]/g,""),"base64")}return{tag:l.match(i)[1],data:n}}},function(t,e,n){(function(e){var r=n(53),i=n(279).ec,o=n(214),a=n(426);function s(t,e){if(t.cmpn(0)<=0)throw new Error("invalid sig");if(t.cmp(e)>=e)throw new Error("invalid sig")}t.exports=function(t,n,A,l,c){var u=o(A);if("ec"===u.type){if("ecdsa"!==l&&"ecdsa/rsa"!==l)throw new Error("wrong public key type");return function(t,e,n){var r=a[n.data.algorithm.curve.join(".")];if(!r)throw new Error("unknown curve "+n.data.algorithm.curve.join("."));var o=new i(r),s=n.data.subjectPrivateKey.data;return o.verify(e,t,s)}(t,n,u)}if("dsa"===u.type){if("dsa"!==l)throw new Error("wrong public key type");return function(t,e,n){var i=n.data.p,a=n.data.q,A=n.data.g,l=n.data.pub_key,c=o.signature.decode(t,"der"),u=c.s,g=c.r;s(u,a),s(g,a);var h=r.mont(i),d=u.invm(a);return 0===A.toRed(h).redPow(new r(e).mul(d).mod(a)).fromRed().mul(l.toRed(h).redPow(g.mul(d).mod(a)).fromRed()).mod(i).mod(a).cmp(g)}(t,n,u)}if("rsa"!==l&&"ecdsa/rsa"!==l)throw new Error("wrong public key type");n=e.concat([c,n]);for(var g=u.modulus.byteLength(),h=[1],d=0;n.length+h.length+2<g;)h.push(255),d++;h.push(0);for(var f=-1;++f<n.length;)h.push(n[f]);h=new e(h);var p=r.mont(u.modulus);t=(t=new r(t).toRed(p)).redPow(new r(u.publicExponent)),t=new e(t.fromRed().toArray());var I=d<8?1:0;for(g=Math.min(t.length,h.length),t.length!==h.length&&(I=1),f=-1;++f<g;)I|=t[f]^h[f];return 0===I}}).call(this,n(42).Buffer)},function(t,e,n){(function(e){var r=n(279),i=n(53);t.exports=function(t){return new a(t)};var o={secp256k1:{name:"secp256k1",byteLength:32},secp224r1:{name:"p224",byteLength:28},prime256v1:{name:"p256",byteLength:32},prime192v1:{name:"p192",byteLength:24},ed25519:{name:"ed25519",byteLength:32},secp384r1:{name:"p384",byteLength:48},secp521r1:{name:"p521",byteLength:66}};function a(t){this.curveType=o[t],this.curveType||(this.curveType={name:t}),this.curve=new r.ec(this.curveType.name),this.keys=void 0}function s(t,n,r){Array.isArray(t)||(t=t.toArray());var i=new e(t);if(r&&i.length<r){var o=new e(r-i.length);o.fill(0),i=e.concat([o,i])}return n?i.toString(n):i}o.p224=o.secp224r1,o.p256=o.secp256r1=o.prime256v1,o.p192=o.secp192r1=o.prime192v1,o.p384=o.secp384r1,o.p521=o.secp521r1,a.prototype.generateKeys=function(t,e){return this.keys=this.curve.genKeyPair(),this.getPublicKey(t,e)},a.prototype.computeSecret=function(t,n,r){return n=n||"utf8",e.isBuffer(t)||(t=new e(t,n)),s(this.curve.keyFromPublic(t).getPublic().mul(this.keys.getPrivate()).getX(),r,this.curveType.byteLength)},a.prototype.getPublicKey=function(t,e){var n=this.keys.getPublic("compressed"===e,!0);return"hybrid"===e&&(n[n.length-1]%2?n[0]=7:n[0]=6),s(n,t)},a.prototype.getPrivateKey=function(t){return s(this.keys.getPrivate(),t)},a.prototype.setPublicKey=function(t,n){return n=n||"utf8",e.isBuffer(t)||(t=new e(t,n)),this.keys._importPublic(t),this},a.prototype.setPrivateKey=function(t,n){n=n||"utf8",e.isBuffer(t)||(t=new e(t,n));var r=new i(t);return r=r.toString(16),this.keys=this.curve.genKeyPair(),this.keys._importPrivate(r),this}}).call(this,n(42).Buffer)},function(t,e,n){e.publicEncrypt=n(847),e.privateDecrypt=n(848),e.privateEncrypt=function(t,n){return e.publicEncrypt(t,n,!0)},e.publicDecrypt=function(t,n){return e.privateDecrypt(t,n,!0)}},function(t,e,n){var r=n(214),i=n(135),o=n(165),a=n(427),s=n(428),A=n(53),l=n(429),c=n(278),u=n(35).Buffer;t.exports=function(t,e,n){var g;g=t.padding?t.padding:n?1:4;var h,d=r(t);if(4===g)h=function(t,e){var n=t.modulus.byteLength(),r=e.length,l=o("sha1").update(u.alloc(0)).digest(),c=l.length,g=2*c;if(r>n-g-2)throw new Error("message too long");var h=u.alloc(n-r-g-2),d=n-c-1,f=i(c),p=s(u.concat([l,h,u.alloc(1,1),e],d),a(f,d)),I=s(f,a(p,c));return new A(u.concat([u.alloc(1),I,p],n))}(d,e);else if(1===g)h=function(t,e,n){var r,o=e.length,a=t.modulus.byteLength();if(o>a-11)throw new Error("message too long");r=n?u.alloc(a-o-3,255):function(t){var e,n=u.allocUnsafe(t),r=0,o=i(2*t),a=0;for(;r<t;)a===o.length&&(o=i(2*t),a=0),(e=o[a++])&&(n[r++]=e);return n}(a-o-3);return new A(u.concat([u.from([0,n?1:2]),r,u.alloc(1),e],a))}(d,e,n);else{if(3!==g)throw new Error("unknown padding");if((h=new A(e)).cmp(d.modulus)>=0)throw new Error("data too long for modulus")}return n?c(h,d):l(h,d)}},function(t,e,n){var r=n(214),i=n(427),o=n(428),a=n(53),s=n(278),A=n(165),l=n(429),c=n(35).Buffer;t.exports=function(t,e,n){var u;u=t.padding?t.padding:n?1:4;var g,h=r(t),d=h.modulus.byteLength();if(e.length>d||new a(e).cmp(h.modulus)>=0)throw new Error("decryption error");g=n?l(new a(e),h):s(e,h);var f=c.alloc(d-g.length);if(g=c.concat([f,g],d),4===u)return function(t,e){var n=t.modulus.byteLength(),r=A("sha1").update(c.alloc(0)).digest(),a=r.length;if(0!==e[0])throw new Error("decryption error");var s=e.slice(1,a+1),l=e.slice(a+1),u=o(s,i(l,a)),g=o(l,i(u,n-a-1));if(function(t,e){t=c.from(t),e=c.from(e);var n=0,r=t.length;t.length!==e.length&&(n++,r=Math.min(t.length,e.length));var i=-1;for(;++i<r;)n+=t[i]^e[i];return n}(r,g.slice(0,a)))throw new Error("decryption error");var h=a;for(;0===g[h];)h++;if(1!==g[h++])throw new Error("decryption error");return g.slice(h)}(h,g);if(1===u)return function(t,e,n){var r=e.slice(0,2),i=2,o=0;for(;0!==e[i++];)if(i>=e.length){o++;break}var a=e.slice(2,i-1);("0002"!==r.toString("hex")&&!n||"0001"!==r.toString("hex")&&n)&&o++;a.length<8&&o++;if(o)throw new Error("decryption error");return e.slice(i)}(0,g,n);if(3===u)return g;throw new Error("unknown padding")}},function(t,e,n){"use strict";(function(t,r){function i(){throw new Error("secure random number generation not supported by this browser\nuse chrome, FireFox or Internet Explorer 11")}var o=n(35),a=n(135),s=o.Buffer,A=o.kMaxLength,l=t.crypto||t.msCrypto,c=Math.pow(2,32)-1;function u(t,e){if("number"!=typeof t||t!=t)throw new TypeError("offset must be a number");if(t>c||t<0)throw new TypeError("offset must be a uint32");if(t>A||t>e)throw new RangeError("offset out of range")}function g(t,e,n){if("number"!=typeof t||t!=t)throw new TypeError("size must be a number");if(t>c||t<0)throw new TypeError("size must be a uint32");if(t+e>n||t>A)throw new RangeError("buffer too small")}function h(t,e,n,i){if(r.browser){var o=t.buffer,s=new Uint8Array(o,e,n);return l.getRandomValues(s),i?void r.nextTick(function(){i(null,t)}):t}if(!i)return a(n).copy(t,e),t;a(n,function(n,r){if(n)return i(n);r.copy(t,e),i(null,t)})}l&&l.getRandomValues||!r.browser?(e.randomFill=function(e,n,r,i){if(!(s.isBuffer(e)||e instanceof t.Uint8Array))throw new TypeError('"buf" argument must be a Buffer or Uint8Array');if("function"==typeof n)i=n,n=0,r=e.length;else if("function"==typeof r)i=r,r=e.length-n;else if("function"!=typeof i)throw new TypeError('"cb" argument must be a function');return u(n,e.length),g(r,n,e.length),h(e,n,r,i)},e.randomFillSync=function(e,n,r){void 0===n&&(n=0);if(!(s.isBuffer(e)||e instanceof t.Uint8Array))throw new TypeError('"buf" argument must be a Buffer or Uint8Array');u(n,e.length),void 0===r&&(r=e.length-n);return g(r,n,e.length),h(e,n,r)}):(e.randomFill=i,e.randomFillSync=i)}).call(this,n(37),n(58))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=function(t){this.name=t,this._tokenStructuralPropertyT=null};e.Token=r},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(395);!function(t){t.uuid4=function(){for(var t=new Uint8Array(16),e=new Array(256),n=0;n<16;++n)e[n]="0"+n.toString(16);for(n=16;n<256;++n)e[n]=n.toString(16);return function(){return r.Random.getRandomValues(t),t[6]=64|15&t[6],t[8]=128|63&t[8],e[t[0]]+e[t[1]]+e[t[2]]+e[t[3]]+"-"+e[t[4]]+e[t[5]]+"-"+e[t[6]]+e[t[7]]+"-"+e[t[8]]+e[t[9]]+"-"+e[t[10]]+e[t[11]]+e[t[12]]+e[t[13]]+e[t[14]]+e[t[15]]}}()}(e.UUID||(e.UUID={}))},function(module,exports,__webpack_require__){"use strict";(function(process){var __importDefault=this&&this.__importDefault||function(t){return t&&t.__esModule?t:{default:t}};Object.defineProperty(exports,"__esModule",{value:!0});const coreutils_1=__webpack_require__(9),minimist_1=__importDefault(__webpack_require__(853)),url_1=__webpack_require__(430);var PageConfig,Private;!function(PageConfig){function getOption(name){if(configData)return configData[name]||Private.getBodyData(name);configData=Object.create(null);let found=!1;if("undefined"!=typeof document&&document){const t=document.getElementById("jupyter-config-data");t&&(configData=JSON.parse(t.textContent||""),found=!0)}if(!found&&void 0!==process)try{const cli=minimist_1.default(process.argv.slice(2)),path=__webpack_require__(171);let fullPath="";"jupyter-config-data"in cli?fullPath=path.resolve(cli["jupyter-config-data"]):"JUPYTER_CONFIG_DATA"in process.env&&(fullPath=path.resolve(process.env.JUPYTER_CONFIG_DATA)),fullPath&&(configData=eval("require")(fullPath))}catch(t){console.error(t)}if(coreutils_1.JSONExt.isObject(configData))for(let t in configData)"string"!=typeof configData[t]&&(configData[t]=JSON.stringify(configData[t]));else configData=Object.create(null);return configData[name]||Private.getBodyData(name)}function setOption(t,e){const n=getOption(t);return configData[t]=e,n}function getBaseUrl(){return url_1.URLExt.normalize(getOption("baseUrl")||"/")}function getTreeUrl(){return url_1.URLExt.join(getBaseUrl(),getOption("treeUrl"))}function getWsUrl(t){let e=getOption("wsUrl");if(!e){if(0!==(t=t?url_1.URLExt.normalize(t):getBaseUrl()).indexOf("http"))return"";e="ws"+t.slice(4)}return url_1.URLExt.normalize(e)}function getNBConvertURL({path:t,format:e,download:n}){const r=url_1.URLExt.encodeParts(t),i=url_1.URLExt.join(getBaseUrl(),"nbconvert",e,r);return n?i+"?download=true":i}function getToken(){return getOption("token")||Private.getBodyData("jupyterApiToken")}function getNotebookVersion(){const t=getOption("notebookVersion");return""===t?[0,0,0]:JSON.parse(t)}PageConfig.getOption=getOption,PageConfig.setOption=setOption,PageConfig.getBaseUrl=getBaseUrl,PageConfig.getTreeUrl=getTreeUrl,PageConfig.getWsUrl=getWsUrl,PageConfig.getNBConvertURL=getNBConvertURL,PageConfig.getToken=getToken,PageConfig.getNotebookVersion=getNotebookVersion;let configData=null}(PageConfig=exports.PageConfig||(exports.PageConfig={})),function(t){t.getBodyData=function(t){if("undefined"==typeof document||!document.body)return"";let e=document.body.dataset[t];return void 0===e?"":decodeURIComponent(e)}}(Private||(Private={}))}).call(this,__webpack_require__(58))},function(t,e){function n(t){return"number"==typeof t||(!!/^0x[0-9a-f]+$/i.test(t)||/^[-+]?(?:\d+(?:\.\d*)?|\.\d+)(e[-+]?\d+)?$/.test(t))}t.exports=function(t,e){e||(e={});var r={bools:{},strings:{},unknownFn:null};"function"==typeof e.unknown&&(r.unknownFn=e.unknown),"boolean"==typeof e.boolean&&e.boolean?r.allBools=!0:[].concat(e.boolean).filter(Boolean).forEach(function(t){r.bools[t]=!0});var i={};Object.keys(e.alias||{}).forEach(function(t){i[t]=[].concat(e.alias[t]),i[t].forEach(function(e){i[e]=[t].concat(i[t].filter(function(t){return e!==t}))})}),[].concat(e.string).filter(Boolean).forEach(function(t){r.strings[t]=!0,i[t]&&(r.strings[i[t]]=!0)});var o=e.default||{},a={_:[]};Object.keys(r.bools).forEach(function(t){A(t,void 0!==o[t]&&o[t])});var s=[];function A(t,e,o){if(!o||!r.unknownFn||function(t,e){return r.allBools&&/^--[^=]+$/.test(e)||r.strings[t]||r.bools[t]||i[t]}(t,o)||!1!==r.unknownFn(o)){var s=!r.strings[t]&&n(e)?Number(e):e;l(a,t.split("."),s),(i[t]||[]).forEach(function(t){l(a,t.split("."),s)})}}function l(t,e,n){var i=t;e.slice(0,-1).forEach(function(t){void 0===i[t]&&(i[t]={}),i=i[t]});var o=e[e.length-1];void 0===i[o]||r.bools[o]||"boolean"==typeof i[o]?i[o]=n:Array.isArray(i[o])?i[o].push(n):i[o]=[i[o],n]}function c(t){return i[t].some(function(t){return r.bools[t]})}-1!==t.indexOf("--")&&(s=t.slice(t.indexOf("--")+1),t=t.slice(0,t.indexOf("--")));for(var u=0;u<t.length;u++){var g=t[u];if(/^--.+=/.test(g)){var h=g.match(/^--([^=]+)=([\s\S]*)$/),d=h[1],f=h[2];r.bools[d]&&(f="false"!==f),A(d,f,g)}else if(/^--no-.+/.test(g)){A(d=g.match(/^--no-(.+)/)[1],!1,g)}else if(/^--.+/.test(g)){d=g.match(/^--(.+)/)[1];void 0===(m=t[u+1])||/^-/.test(m)||r.bools[d]||r.allBools||i[d]&&c(d)?/^(true|false)$/.test(m)?(A(d,"true"===m,g),u++):A(d,!r.strings[d]||"",g):(A(d,m,g),u++)}else if(/^-[^-]+/.test(g)){for(var p=g.slice(1,-1).split(""),I=!1,C=0;C<p.length;C++){var m;if("-"!==(m=g.slice(C+2))){if(/[A-Za-z]/.test(p[C])&&/=/.test(m)){A(p[C],m.split("=")[1],g),I=!0;break}if(/[A-Za-z]/.test(p[C])&&/-?\d+(\.\d*)?(e-?\d+)?$/.test(m)){A(p[C],m,g),I=!0;break}if(p[C+1]&&p[C+1].match(/\W/)){A(p[C],g.slice(C+2),g),I=!0;break}A(p[C],!r.strings[p[C]]||"",g)}else A(p[C],m,g)}d=g.slice(-1)[0];I||"-"===d||(!t[u+1]||/^(-|--)[^-]/.test(t[u+1])||r.bools[d]||i[d]&&c(d)?t[u+1]&&/true|false/.test(t[u+1])?(A(d,"true"===t[u+1],g),u++):A(d,!r.strings[d]||"",g):(A(d,t[u+1],g),u++))}else if(r.unknownFn&&!1===r.unknownFn(g)||a._.push(r.strings._||!n(g)?g:Number(g)),e.stopEarly){a._.push.apply(a._,t.slice(u+1));break}}return Object.keys(o).forEach(function(t){var e,n,r;e=a,n=t.split("."),r=e,n.slice(0,-1).forEach(function(t){r=r[t]||{}}),n[n.length-1]in r||(l(a,t.split("."),o[t]),(i[t]||[]).forEach(function(e){l(a,e.split("."),o[t])}))}),e["--"]?(a["--"]=new Array,s.forEach(function(t){a["--"].push(t)})):s.forEach(function(t){a._.push(t)}),a}},function(t,e,n){"use strict";(function(e){var r=n(855),i=n(856),o=/^[A-Za-z][A-Za-z0-9+-.]*:\/\//,a=/^([a-z][a-z0-9.+-]*:)?(\/\/)?([\S\s]*)/i,s=new RegExp("^[\\x09\\x0A\\x0B\\x0C\\x0D\\x20\\xA0\\u1680\\u180E\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200A\\u202F\\u205F\\u3000\\u2028\\u2029\\uFEFF]+");function A(t){return(t||"").toString().replace(s,"")}var l=[["#","hash"],["?","query"],function(t){return t.replace("\\","/")},["/","pathname"],["@","auth",1],[NaN,"host",void 0,1,1],[/:(\d+)$/,"port",void 0,1],[NaN,"hostname",void 0,1,1]],c={hash:1,query:1};function u(t){var n,r=("undefined"!=typeof window?window:void 0!==e?e:"undefined"!=typeof self?self:{}).location||{},i={},a=typeof(t=t||r);if("blob:"===t.protocol)i=new h(unescape(t.pathname),{});else if("string"===a)for(n in i=new h(t,{}),c)delete i[n];else if("object"===a){for(n in t)n in c||(i[n]=t[n]);void 0===i.slashes&&(i.slashes=o.test(t.href))}return i}function g(t){t=A(t);var e=a.exec(t);return{protocol:e[1]?e[1].toLowerCase():"",slashes:!!e[2],rest:e[3]}}function h(t,e,n){if(t=A(t),!(this instanceof h))return new h(t,e,n);var o,a,s,c,d,f,p=l.slice(),I=typeof e,C=this,m=0;for("object"!==I&&"string"!==I&&(n=e,e=null),n&&"function"!=typeof n&&(n=i.parse),e=u(e),o=!(a=g(t||"")).protocol&&!a.slashes,C.slashes=a.slashes||o&&e.slashes,C.protocol=a.protocol||e.protocol||"",t=a.rest,a.slashes||(p[3]=[/(.*)/,"pathname"]);m<p.length;m++)"function"!=typeof(c=p[m])?(s=c[0],f=c[1],s!=s?C[f]=t:"string"==typeof s?~(d=t.indexOf(s))&&("number"==typeof c[2]?(C[f]=t.slice(0,d),t=t.slice(d+c[2])):(C[f]=t.slice(d),t=t.slice(0,d))):(d=s.exec(t))&&(C[f]=d[1],t=t.slice(0,d.index)),C[f]=C[f]||o&&c[3]&&e[f]||"",c[4]&&(C[f]=C[f].toLowerCase())):t=c(t);n&&(C.query=n(C.query)),o&&e.slashes&&"/"!==C.pathname.charAt(0)&&(""!==C.pathname||""!==e.pathname)&&(C.pathname=function(t,e){if(""===t)return e;for(var n=(e||"/").split("/").slice(0,-1).concat(t.split("/")),r=n.length,i=n[r-1],o=!1,a=0;r--;)"."===n[r]?n.splice(r,1):".."===n[r]?(n.splice(r,1),a++):a&&(0===r&&(o=!0),n.splice(r,1),a--);return o&&n.unshift(""),"."!==i&&".."!==i||n.push(""),n.join("/")}(C.pathname,e.pathname)),r(C.port,C.protocol)||(C.host=C.hostname,C.port=""),C.username=C.password="",C.auth&&(c=C.auth.split(":"),C.username=c[0]||"",C.password=c[1]||""),C.origin=C.protocol&&C.host&&"file:"!==C.protocol?C.protocol+"//"+C.host:"null",C.href=C.toString()}h.prototype={set:function(t,e,n){var o=this;switch(t){case"query":"string"==typeof e&&e.length&&(e=(n||i.parse)(e)),o[t]=e;break;case"port":o[t]=e,r(e,o.protocol)?e&&(o.host=o.hostname+":"+e):(o.host=o.hostname,o[t]="");break;case"hostname":o[t]=e,o.port&&(e+=":"+o.port),o.host=e;break;case"host":o[t]=e,/:\d+$/.test(e)?(e=e.split(":"),o.port=e.pop(),o.hostname=e.join(":")):(o.hostname=e,o.port="");break;case"protocol":o.protocol=e.toLowerCase(),o.slashes=!n;break;case"pathname":case"hash":if(e){var a="pathname"===t?"/":"#";o[t]=e.charAt(0)!==a?a+e:e}else o[t]=e;break;default:o[t]=e}for(var s=0;s<l.length;s++){var A=l[s];A[4]&&(o[A[1]]=o[A[1]].toLowerCase())}return o.origin=o.protocol&&o.host&&"file:"!==o.protocol?o.protocol+"//"+o.host:"null",o.href=o.toString(),o},toString:function(t){t&&"function"==typeof t||(t=i.stringify);var e,n=this,r=n.protocol;r&&":"!==r.charAt(r.length-1)&&(r+=":");var o=r+(n.slashes?"//":"");return n.username&&(o+=n.username,n.password&&(o+=":"+n.password),o+="@"),o+=n.host+n.pathname,(e="object"==typeof n.query?t(n.query):n.query)&&(o+="?"!==e.charAt(0)?"?"+e:e),n.hash&&(o+=n.hash),o}},h.extractProtocol=g,h.location=u,h.trimLeft=A,h.qs=i,t.exports=h}).call(this,n(37))},function(t,e,n){"use strict";t.exports=function(t,e){if(e=e.split(":")[0],!(t=+t))return!1;switch(e){case"http":case"ws":return 80!==t;case"https":case"wss":return 443!==t;case"ftp":return 21!==t;case"gopher":return 70!==t;case"file":return!1}return 0!==t}},function(t,e,n){"use strict";var r,i=Object.prototype.hasOwnProperty;function o(t){try{return decodeURIComponent(t.replace(/\+/g," "))}catch(t){return null}}e.stringify=function(t,e){e=e||"";var n,o,a=[];for(o in"string"!=typeof e&&(e="?"),t)if(i.call(t,o)){if((n=t[o])||null!==n&&n!==r&&!isNaN(n)||(n=""),o=encodeURIComponent(o),n=encodeURIComponent(n),null===o||null===n)continue;a.push(o+"="+n)}return a.length?e+a.join("&"):""},e.parse=function(t){for(var e,n=/([^=?&]+)=?([^&]*)/g,r={};e=n.exec(t);){var i=o(e[1]),a=o(e[2]);null===i||null===a||i in r||(r[i]=a)}return r}},function(t,e,n){"use strict";var r=this&&this.__importStar||function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)Object.hasOwnProperty.call(t,n)&&(e[n]=t[n]);return e.default=t,e};Object.defineProperty(e,"__esModule",{value:!0});const i=r(n(858));!function(t){function e(t){return 0===t.indexOf("/")&&(t=t.slice(1)),t}t.join=function(...t){return"."===i.join(...t)?"":e(i.join(...t))},t.basename=function(t,e){return i.basename(t,e)},t.dirname=function(t){let n=e(i.dirname(t));return"."===n?"":n},t.extname=function(t){return i.extname(t)},t.stem=function(t){return t.split("\\").pop().split("/").pop().split(".").shift()},t.normalize=function(t){return""===t?"":e(i.normalize(t))},t.resolve=function(...t){return e(i.resolve(...t))},t.relative=function(t,n){return e(i.relative(t,n))},t.normalizeExtension=function(t){return t.length>0&&0!==t.indexOf(".")&&(t=`.${t}`),t},t.removeSlash=e}(e.PathExt||(e.PathExt={}))},function(t,e,n){"use strict";(function(e){var r=n(859),i=function(t){return"string"==typeof t};function o(t,e){for(var n=[],r=0;r<t.length;r++){var i=t[r];i&&"."!==i&&(".."===i?n.length&&".."!==n[n.length-1]?n.pop():e&&n.push(".."):n.push(i))}return n}var a=/^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/,s={};function A(t){return a.exec(t).slice(1)}s.resolve=function(){for(var t="",n=!1,r=arguments.length-1;r>=-1&&!n;r--){var a=r>=0?arguments[r]:e.cwd();if(!i(a))throw new TypeError("Arguments to path.resolve must be strings");a&&(t=a+"/"+t,n="/"===a.charAt(0))}return(n?"/":"")+(t=o(t.split("/"),!n).join("/"))||"."},s.normalize=function(t){var e=s.isAbsolute(t),n="/"===t.substr(-1);return(t=o(t.split("/"),!e).join("/"))||e||(t="."),t&&n&&(t+="/"),(e?"/":"")+t},s.isAbsolute=function(t){return"/"===t.charAt(0)},s.join=function(){for(var t="",e=0;e<arguments.length;e++){var n=arguments[e];if(!i(n))throw new TypeError("Arguments to path.join must be strings");n&&(t+=t?"/"+n:n)}return s.normalize(t)},s.relative=function(t,e){function n(t){for(var e=0;e<t.length&&""===t[e];e++);for(var n=t.length-1;n>=0&&""===t[n];n--);return e>n?[]:t.slice(e,n+1)}t=s.resolve(t).substr(1),e=s.resolve(e).substr(1);for(var r=n(t.split("/")),i=n(e.split("/")),o=Math.min(r.length,i.length),a=o,A=0;A<o;A++)if(r[A]!==i[A]){a=A;break}var l=[];for(A=a;A<r.length;A++)l.push("..");return(l=l.concat(i.slice(a))).join("/")},s._makeLong=function(t){return t},s.dirname=function(t){var e=A(t),n=e[0],r=e[1];return n||r?(r&&(r=r.substr(0,r.length-1)),n+r):"."},s.basename=function(t,e){var n=A(t)[2];return e&&n.substr(-1*e.length)===e&&(n=n.substr(0,n.length-e.length)),n},s.extname=function(t){return A(t)[3]},s.format=function(t){if(!r.isObject(t))throw new TypeError("Parameter 'pathObject' must be an object, not "+typeof t);var e=t.root||"";if(!i(e))throw new TypeError("'pathObject.root' must be a string or undefined, not "+typeof t.root);return(t.dir?t.dir+s.sep:"")+(t.base||"")},s.parse=function(t){if(!i(t))throw new TypeError("Parameter 'pathString' must be a string, not "+typeof t);var e=A(t);if(!e||4!==e.length)throw new TypeError("Invalid path '"+t+"'");return e[1]=e[1]||"",e[2]=e[2]||"",e[3]=e[3]||"",{root:e[0],dir:e[0]+e[1].slice(0,e[1].length-1),base:e[2],ext:e[3],name:e[2].slice(0,e[2].length-e[3].length)}},s.sep="/",s.delimiter=":",t.exports=s}).call(this,n(58))},function(t,e,n){(function(t){var r=Object.getOwnPropertyDescriptors||function(t){for(var e=Object.keys(t),n={},r=0;r<e.length;r++)n[e[r]]=Object.getOwnPropertyDescriptor(t,e[r]);return n},i=/%[sdj%]/g;e.format=function(t){if(!I(t)){for(var e=[],n=0;n<arguments.length;n++)e.push(s(arguments[n]));return e.join(" ")}n=1;for(var r=arguments,o=r.length,a=String(t).replace(i,function(t){if("%%"===t)return"%";if(n>=o)return t;switch(t){case"%s":return String(r[n++]);case"%d":return Number(r[n++]);case"%j":try{return JSON.stringify(r[n++])}catch(t){return"[Circular]"}default:return t}}),A=r[n];n<o;A=r[++n])f(A)||!y(A)?a+=" "+A:a+=" "+s(A);return a},e.deprecate=function(n,r){if(void 0!==t&&!0===t.noDeprecation)return n;if(void 0===t)return function(){return e.deprecate(n,r).apply(this,arguments)};var i=!1;return function(){if(!i){if(t.throwDeprecation)throw new Error(r);t.traceDeprecation?console.trace(r):console.error(r),i=!0}return n.apply(this,arguments)}};var o,a={};function s(t,n){var r={seen:[],stylize:l};return arguments.length>=3&&(r.depth=arguments[2]),arguments.length>=4&&(r.colors=arguments[3]),d(n)?r.showHidden=n:n&&e._extend(r,n),C(r.showHidden)&&(r.showHidden=!1),C(r.depth)&&(r.depth=2),C(r.colors)&&(r.colors=!1),C(r.customInspect)&&(r.customInspect=!0),r.colors&&(r.stylize=A),c(r,t,r.depth)}function A(t,e){var n=s.styles[e];return n?"["+s.colors[n][0]+"m"+t+"["+s.colors[n][1]+"m":t}function l(t,e){return t}function c(t,n,r){if(t.customInspect&&n&&B(n.inspect)&&n.inspect!==e.inspect&&(!n.constructor||n.constructor.prototype!==n)){var i=n.inspect(r,t);return I(i)||(i=c(t,i,r)),i}var o=function(t,e){if(C(e))return t.stylize("undefined","undefined");if(I(e)){var n="'"+JSON.stringify(e).replace(/^"|"$/g,"").replace(/'/g,"\\'").replace(/\\"/g,'"')+"'";return t.stylize(n,"string")}if(p(e))return t.stylize(""+e,"number");if(d(e))return t.stylize(""+e,"boolean");if(f(e))return t.stylize("null","null")}(t,n);if(o)return o;var a=Object.keys(n),s=function(t){var e={};return t.forEach(function(t,n){e[t]=!0}),e}(a);if(t.showHidden&&(a=Object.getOwnPropertyNames(n)),v(n)&&(a.indexOf("message")>=0||a.indexOf("description")>=0))return u(n);if(0===a.length){if(B(n)){var A=n.name?": "+n.name:"";return t.stylize("[Function"+A+"]","special")}if(m(n))return t.stylize(RegExp.prototype.toString.call(n),"regexp");if(E(n))return t.stylize(Date.prototype.toString.call(n),"date");if(v(n))return u(n)}var l,y="",b=!1,Q=["{","}"];(h(n)&&(b=!0,Q=["[","]"]),B(n))&&(y=" [Function"+(n.name?": "+n.name:"")+"]");return m(n)&&(y=" "+RegExp.prototype.toString.call(n)),E(n)&&(y=" "+Date.prototype.toUTCString.call(n)),v(n)&&(y=" "+u(n)),0!==a.length||b&&0!=n.length?r<0?m(n)?t.stylize(RegExp.prototype.toString.call(n),"regexp"):t.stylize("[Object]","special"):(t.seen.push(n),l=b?function(t,e,n,r,i){for(var o=[],a=0,s=e.length;a<s;++a)_(e,String(a))?o.push(g(t,e,n,r,String(a),!0)):o.push("");return i.forEach(function(i){i.match(/^\d+$/)||o.push(g(t,e,n,r,i,!0))}),o}(t,n,r,s,a):a.map(function(e){return g(t,n,r,s,e,b)}),t.seen.pop(),function(t,e,n){if(t.reduce(function(t,e){return 0,e.indexOf("\n")>=0&&0,t+e.replace(/\u001b\[\d\d?m/g,"").length+1},0)>60)return n[0]+(""===e?"":e+"\n ")+" "+t.join(",\n  ")+" "+n[1];return n[0]+e+" "+t.join(", ")+" "+n[1]}(l,y,Q)):Q[0]+y+Q[1]}function u(t){return"["+Error.prototype.toString.call(t)+"]"}function g(t,e,n,r,i,o){var a,s,A;if((A=Object.getOwnPropertyDescriptor(e,i)||{value:e[i]}).get?s=A.set?t.stylize("[Getter/Setter]","special"):t.stylize("[Getter]","special"):A.set&&(s=t.stylize("[Setter]","special")),_(r,i)||(a="["+i+"]"),s||(t.seen.indexOf(A.value)<0?(s=f(n)?c(t,A.value,null):c(t,A.value,n-1)).indexOf("\n")>-1&&(s=o?s.split("\n").map(function(t){return"  "+t}).join("\n").substr(2):"\n"+s.split("\n").map(function(t){return"   "+t}).join("\n")):s=t.stylize("[Circular]","special")),C(a)){if(o&&i.match(/^\d+$/))return s;(a=JSON.stringify(""+i)).match(/^"([a-zA-Z_][a-zA-Z_0-9]*)"$/)?(a=a.substr(1,a.length-2),a=t.stylize(a,"name")):(a=a.replace(/'/g,"\\'").replace(/\\"/g,'"').replace(/(^"|"$)/g,"'"),a=t.stylize(a,"string"))}return a+": "+s}function h(t){return Array.isArray(t)}function d(t){return"boolean"==typeof t}function f(t){return null===t}function p(t){return"number"==typeof t}function I(t){return"string"==typeof t}function C(t){return void 0===t}function m(t){return y(t)&&"[object RegExp]"===b(t)}function y(t){return"object"==typeof t&&null!==t}function E(t){return y(t)&&"[object Date]"===b(t)}function v(t){return y(t)&&("[object Error]"===b(t)||t instanceof Error)}function B(t){return"function"==typeof t}function b(t){return Object.prototype.toString.call(t)}function Q(t){return t<10?"0"+t.toString(10):t.toString(10)}e.debuglog=function(n){if(C(o)&&(o=t.env.NODE_DEBUG||""),n=n.toUpperCase(),!a[n])if(new RegExp("\\b"+n+"\\b","i").test(o)){var r=t.pid;a[n]=function(){var t=e.format.apply(e,arguments);console.error("%s %d: %s",n,r,t)}}else a[n]=function(){};return a[n]},e.inspect=s,s.colors={bold:[1,22],italic:[3,23],underline:[4,24],inverse:[7,27],white:[37,39],grey:[90,39],black:[30,39],blue:[34,39],cyan:[36,39],green:[32,39],magenta:[35,39],red:[31,39],yellow:[33,39]},s.styles={special:"cyan",number:"yellow",boolean:"yellow",undefined:"grey",null:"bold",string:"green",date:"magenta",regexp:"red"},e.isArray=h,e.isBoolean=d,e.isNull=f,e.isNullOrUndefined=function(t){return null==t},e.isNumber=p,e.isString=I,e.isSymbol=function(t){return"symbol"==typeof t},e.isUndefined=C,e.isRegExp=m,e.isObject=y,e.isDate=E,e.isError=v,e.isFunction=B,e.isPrimitive=function(t){return null===t||"boolean"==typeof t||"number"==typeof t||"string"==typeof t||"symbol"==typeof t||void 0===t},e.isBuffer=n(860);var w=["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"];function x(){var t=new Date,e=[Q(t.getHours()),Q(t.getMinutes()),Q(t.getSeconds())].join(":");return[t.getDate(),w[t.getMonth()],e].join(" ")}function _(t,e){return Object.prototype.hasOwnProperty.call(t,e)}e.log=function(){console.log("%s - %s",x(),e.format.apply(e,arguments))},e.inherits=n(861),e._extend=function(t,e){if(!e||!y(e))return t;for(var n=Object.keys(e),r=n.length;r--;)t[n[r]]=e[n[r]];return t};var D="undefined"!=typeof Symbol?Symbol("util.promisify.custom"):void 0;function M(t,e){if(!t){var n=new Error("Promise was rejected with a falsy value");n.reason=t,t=n}return e(t)}e.promisify=function(t){if("function"!=typeof t)throw new TypeError('The "original" argument must be of type Function');if(D&&t[D]){var e;if("function"!=typeof(e=t[D]))throw new TypeError('The "util.promisify.custom" argument must be of type Function');return Object.defineProperty(e,D,{value:e,enumerable:!1,writable:!1,configurable:!0}),e}function e(){for(var e,n,r=new Promise(function(t,r){e=t,n=r}),i=[],o=0;o<arguments.length;o++)i.push(arguments[o]);i.push(function(t,r){t?n(t):e(r)});try{t.apply(this,i)}catch(t){n(t)}return r}return Object.setPrototypeOf(e,Object.getPrototypeOf(t)),D&&Object.defineProperty(e,D,{value:e,enumerable:!1,writable:!1,configurable:!0}),Object.defineProperties(e,r(t))},e.promisify.custom=D,e.callbackify=function(e){if("function"!=typeof e)throw new TypeError('The "original" argument must be of type Function');function n(){for(var n=[],r=0;r<arguments.length;r++)n.push(arguments[r]);var i=n.pop();if("function"!=typeof i)throw new TypeError("The last argument must be of type Function");var o=this,a=function(){return i.apply(o,arguments)};e.apply(this,n).then(function(e){t.nextTick(a,null,e)},function(e){t.nextTick(M,e,a)})}return Object.setPrototypeOf(n,Object.getPrototypeOf(e)),Object.defineProperties(n,r(e)),n}}).call(this,n(58))},function(t,e){t.exports=function(t){return t&&"object"==typeof t&&"function"==typeof t.copy&&"function"==typeof t.fill&&"function"==typeof t.readUInt8}},function(t,e){"function"==typeof Object.create?t.exports=function(t,e){t.super_=e,t.prototype=Object.create(e.prototype,{constructor:{value:t,enumerable:!1,writable:!0,configurable:!0}})}:t.exports=function(t,e){t.super_=e;var n=function(){};n.prototype=e.prototype,t.prototype=new n,t.prototype.constructor=t}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9),i=n(431);class o{constructor(t,e=500){this.payload=null,this.limit=e,this.poll=new i.Poll({auto:!1,factory:async()=>await t(),frequency:{backoff:!1,interval:i.Poll.NEVER,max:i.Poll.NEVER},standby:"never"}),this.payload=new r.PromiseDelegate,this.poll.ticked.connect((t,e)=>{const{payload:n}=this;return"resolved"===e.phase?(this.payload=new r.PromiseDelegate,void n.resolve(e.payload||void 0)):"rejected"===e.phase||"stopped"===e.phase?(this.payload=new r.PromiseDelegate,n.promise.catch(t=>void 0),void n.reject(e.payload)):void 0},this)}get isDisposed(){return null===this.payload}dispose(){this.isDisposed||(this.payload=null,this.poll.dispose())}async stop(){return this.poll.stop()}}e.RateLimiter=o;e.Debouncer=class extends o{async invoke(){return this.poll.schedule({interval:this.limit,phase:"invoked"}),this.payload.promise}};e.Throttler=class extends o{async invoke(){return"invoked"!==this.poll.state.phase&&this.poll.schedule({interval:this.limit,phase:"invoked"}),this.payload.promise}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9),i=n(43),o=n(10);var a;e.RestorablePool=class{constructor(t){this._added=new o.Signal(this),this._current=null,this._currentChanged=new o.Signal(this),this._hasRestored=!1,this._isDisposed=!1,this._objects=new Set,this._restore=null,this._restored=new r.PromiseDelegate,this._updated=new o.Signal(this),this.namespace=t.namespace}get added(){return this._added}get current(){return this._current}set current(t){this._current!==t&&this._objects.has(t)&&(this._current=t,this._currentChanged.emit(this._current))}get currentChanged(){return this._currentChanged}get isDisposed(){return this._isDisposed}get restored(){return this._restored.promise}get size(){return this._objects.size}get updated(){return this._updated}async add(t){if(t.isDisposed){const e="A disposed object cannot be added.";throw console.warn(e,t),new Error(e)}if(this._objects.has(t)){const e="This object already exists in the pool.";throw console.warn(e,t),new Error(e)}if(this._objects.add(t),t.disposed.connect(this._onInstanceDisposed,this),!a.injectedProperty.get(t)){if(this._restore){const{connector:e}=this._restore,n=this._restore.name(t);if(n){const r=`${this.namespace}:${n}`,i=this._restore.args(t);a.nameProperty.set(t,r),await e.save(r,{data:i})}}this._added.emit(t)}}dispose(){this.isDisposed||(this._current=null,this._isDisposed=!0,this._objects.clear(),o.Signal.clearData(this))}find(t){const e=this._objects.values();for(let n of e)if(t(n))return n}forEach(t){this._objects.forEach(t)}filter(t){const e=[];return this.forEach(n=>{t(n)&&e.push(n)}),e}inject(t){return a.injectedProperty.set(t,!0),this.add(t)}has(t){return this._objects.has(t)}async restore(t){if(this._hasRestored)throw new Error("This pool has already been restored.");this._hasRestored=!0;const{command:e,connector:n,registry:r,when:i}=t,o=this.namespace,a=i?[n.list(o)].concat(i):[n.list(o)];this._restore=t;const[s]=await Promise.all(a),A=await Promise.all(s.ids.map(async(t,i)=>{const o=s.values[i],a=o&&o.data;return void 0===a?n.remove(t):r.execute(e,a).catch(()=>n.remove(t))}));return this._restored.resolve(),A}async save(t){const e=a.injectedProperty.get(t);if(!this._restore||!this.has(t)||e)return;const{connector:n}=this._restore,r=this._restore.name(t),i=a.nameProperty.get(t),o=r?`${this.namespace}:${r}`:"";if(i&&i!==o&&await n.remove(i),a.nameProperty.set(t,o),o){const e=this._restore.args(t);await n.save(o,{data:e})}i!==o&&this._updated.emit(t)}_onInstanceDisposed(t){if(this._objects.delete(t),t===this._current&&(this._current=null,this._currentChanged.emit(this._current)),a.injectedProperty.get(t))return;if(!this._restore)return;const{connector:e}=this._restore,n=a.nameProperty.get(t);n&&e.remove(n)}},function(t){t.injectedProperty=new i.AttachedProperty({name:"injected",create:()=>!1}),t.nameProperty=new i.AttachedProperty({name:"name",create:()=>""})}(a||(a={}))},function(t,e,n){"use strict";var r=this&&this.__importDefault||function(t){return t&&t.__esModule?t:{default:t}},i=this&&this.__importStar||function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)Object.hasOwnProperty.call(t,n)&&(e[n]=t[n]);return e.default=t,e};Object.defineProperty(e,"__esModule",{value:!0});const o=r(n(865)),a=i(n(899)),s=n(9),A=n(50),l=n(10),c=r(n(900)),u=s.JSONExt.deepCopy,g=7e3,h=String.fromCharCode(30);class d{constructor(){this._composer=new o.default({useDefaults:!0}),this._validator=new o.default,this._composer.addSchema(c.default,"jupyterlab-plugin-schema"),this._validator.addSchema(c.default,"jupyterlab-plugin-schema")}validateData(t,e=!0){const n=this._validator.getSchema(t.id),r=this._composer.getSchema(t.id);if(!n||!r){if("object"!==t.schema.type){return[{dataPath:"type",keyword:"schema",schemaPath:"",message:"Setting registry schemas' root-level type must be "+`'object', rejecting type: ${t.schema.type}`}]}return this._addSchema(t.id,t.schema)||this.validateData(t)}let i;try{i=a.parse(t.raw,null)}catch(t){if(t instanceof SyntaxError)return[{dataPath:"",keyword:"syntax",schemaPath:"",message:t.message}];const{column:e,description:n}=t;return[{dataPath:"",keyword:"parse",schemaPath:"",message:`${n} (line ${t.lineNumber} column ${e})`}]}if(!n(i))return n.errors;const o=u(i);return r(o)?(e&&(t.data={composite:o,user:i}),null):r.errors}_addSchema(t,e){const n=this._composer,r=this._validator,i=r.getSchema("jupyterlab-plugin-schema");return i(e)?r.validateSchema(e)?(n.removeSchema(t),r.removeSchema(t),n.addSchema(e,t),r.addSchema(e,t),null):r.errors:i.errors}}e.DefaultSchemaValidator=d;class f{constructor(t){this.schema=c.default,this.plugins=Object.create(null),this._pluginChanged=new l.Signal(this),this._ready=Promise.resolve(),this._transformers=Object.create(null),this.connector=t.connector,this.validator=t.validator||new d,this._timeout=t.timeout||g,t.plugins&&(this._ready=this._preload(t.plugins))}get pluginChanged(){return this._pluginChanged}async get(t,e){await this._ready;const n=this.plugins;if(t in n){const{composite:r,user:i}=n[t].data;return{composite:e in r?u(r[e]):void 0,user:e in i?u(i[e]):void 0}}return this.load(t).then(()=>this.get(t,e))}async load(t){await this._ready;const e=this.plugins,n=this;return t in e?new p({plugin:e[t],registry:n}):this.reload(t)}async reload(t){await this._ready;const e=await this.connector.fetch(t),n=this.plugins;return await this._load(await this._transform("fetch",e)),this._pluginChanged.emit(t),new p({plugin:n[t],registry:this})}async remove(t,e){await this._ready;const n=this.plugins;if(!(t in n))return;const r=a.parse(n[t].raw,null);return delete r[e],delete r[`// ${e}`],n[t].raw=I.annotatedPlugin(n[t],r),this._save(t)}async set(t,e,n){await this._ready;const r=this.plugins;if(!(t in r))return this.load(t).then(()=>this.set(t,e,n));const i=a.parse(r[t].raw,null);return r[t].raw=I.annotatedPlugin(r[t],Object.assign({},i,{[e]:n})),this._save(t)}transform(t,e){const n=this._transformers;if(t in n)throw new Error(`${t} already has a transformer.`);return n[t]={fetch:e.fetch||(t=>t),compose:e.compose||(t=>t)},new A.DisposableDelegate(()=>{delete n[t]})}async upload(t,e){await this._ready;const n=this.plugins;return t in n?(n[t].raw=e,this._save(t)):this.load(t).then(()=>this.upload(t,e))}async _load(t){const e=t.id;try{await this._validate(t)}catch(t){const n=[`Validating ${e} failed:`];throw t.forEach((t,e)=>{const{dataPath:r,schemaPath:i,keyword:o,message:a}=t;(r||i)&&n.push(`${e} - schema @ ${i}, data @ ${r}`),n.push(`{${o}} ${a}`)}),console.warn(n.join("\n")),t}}async _preload(t){await Promise.all(t.map(async t=>{try{await this._load(await this._transform("fetch",t))}catch(t){console.log("Ignored setting registry preload errors.",t)}}))}async _save(t){const e=this.plugins;if(!(t in e))throw new Error(`${t} does not exist in setting registry.`);try{await this._validate(e[t])}catch(e){throw console.warn(`${t} validation errors:`,e),new Error(`${t} failed to validate; check console.`)}await this.connector.save(t,e[t].raw);const n=await this.connector.fetch(t);await this._load(await this._transform("fetch",n)),this._pluginChanged.emit(t)}async _transform(t,e,n=(new Date).getTime()){const r=(new Date).getTime()-n,i=e.id,o=this._transformers,a=this._timeout;if(!e.schema["jupyter.lab.transform"])return e;if(i in o){const n=o[i][t].call(null,e);if(n.id!==i)throw[{dataPath:"",keyword:"id",message:"Plugin transformations cannot change plugin IDs.",schemaPath:""}];return n}if(r<a)return await new Promise(t=>{setTimeout(()=>{t()},250)}),this._transform(t,e,n);throw[{dataPath:"",keyword:"timeout",message:`Transforming ${e.id} timed out.`,schemaPath:""}]}async _validate(t){const e=this.validator.validateData(t);if(e)throw e;this.plugins[t.id]=await this._transform("compose",t)}}e.SettingRegistry=f;class p{constructor(t){this._changed=new l.Signal(this),this._isDisposed=!1,this.id=t.plugin.id,this.registry=t.registry,this.registry.pluginChanged.connect(this._onPluginChanged,this)}get changed(){return this._changed}get composite(){return this.plugin.data.composite}get isDisposed(){return this._isDisposed}get plugin(){return this.registry.plugins[this.id]}get schema(){return this.plugin.schema}get raw(){return this.plugin.raw}get user(){return this.plugin.data.user}get version(){return this.plugin.version}annotatedDefaults(){return I.annotatedDefaults(this.schema,this.id)}default(t){return I.reifyDefault(this.schema,t)}dispose(){this._isDisposed||(this._isDisposed=!0,l.Signal.clearData(this))}get(t){const{composite:e,user:n}=this;return{composite:t in e?u(e[t]):void 0,user:t in n?u(n[t]):void 0}}remove(t){return this.registry.remove(this.plugin.id,t)}save(t){return this.registry.upload(this.plugin.id,t)}set(t,e){return this.registry.set(this.plugin.id,t,e)}validate(t){const{id:e,schema:n}=this.plugin,r=this.registry.validator,i=this.version;return r.validateData({data:{composite:{},user:{}},id:e,raw:t,schema:n,version:i},!1)}_onPluginChanged(t,e){e===this.plugin.id&&this._changed.emit(void 0)}}var I;e.Settings=p,function(t){t.reconcileShortcuts=function(t,e){const n={};return e=e.filter(t=>{const e=t.keys.join(h),{selector:r}=t;return e?(e in n||(n[e]={}),r in n[e]?(console.warn("Shortcut skipped due to collision.",t),!1):(n[e][r]=!1,!0)):(console.warn('Shortcut skipped because `keys` are [""].',t),!1)}),t=t.filter(t=>{const{disabled:e}=t,r=t.keys.join(h);if(e||!r)return!1;r in n||(n[r]={});const{selector:i}=t;return i in n[r]?(n[r][i]&&console.warn("Shortcut skipped due to collision.",t),!1):(n[r][i]=!0,!0)}),e.filter(t=>!t.disabled).concat(t)}}(f=e.SettingRegistry||(e.SettingRegistry={})),function(t){const e="    ",n="[missing schema description]",r="[missing schema title]";function i(t){return t.reduce((e,n,r)=>{const i=n.split("\n");return e+n+(0===i[i.length-1].trim().indexOf("//")||r===t.length-1?"":",")+(r===t.length-1?"":"\n\n")},"")}function o(t,n=`${e}// `){return n+t.split("\n").join(`\n${n}`)}function a(t,e){if(!("default"in(t=(e?t.properties[e]:t)||{}))||"object"!==t.type)return t.default;const n=s.JSONExt.deepCopy(t.default);for(let e in t.properties||{})n[e]=a(t.properties[e]);return n}t.annotatedDefaults=function(t,s){const{description:A,properties:l,title:c}=t,u=Object.keys(l).sort((t,e)=>t.localeCompare(e)),g=Math.max((A||n).length,s.length);return["{",o(`${c||r}`),o(s),o(A||n),o("*".repeat(g)),"",i(u.map(r=>(function(t,r){const i=t.properties&&t.properties[r]||{},s=i.type,A=i.description||n,l=i.title||"",c=a(t,r),u=e.length,g=void 0!==c?o(`"${r}": ${JSON.stringify(c,null,u)}`,e):o(`"${r}": ${s}`);return[o(l),o(A),g].filter(t=>t.length).join("\n")})(t,r))),"}"].join("\n")},t.annotatedPlugin=function(t,a){const{description:s,title:A}=t.schema,l=Object.keys(a).sort((t,e)=>t.localeCompare(e)),c=Math.max((s||n).length,t.id.length);return["{",o(`${A||r}`),o(t.id),o(s||n),o("*".repeat(c)),"",i(l.map(i=>(function(t,i,a){const s=t.properties&&t.properties[i],A=s&&s.description||n,l=s&&s.title||r,c=e.length,u=o(`"${i}": ${JSON.stringify(a,null,c)}`,e);return[o(l),o(A),u].join("\n")})(t.schema,i,a[i]))),"}"].join("\n")},t.reifyDefault=a}(I=e.Private||(e.Private={}))},function(t,e,n){"use strict";var r=n(866),i=n(282),o=n(870),a=n(432),s=n(433),A=n(871),l=n(872),c=n(893),u=n(137);t.exports=I,I.prototype.validate=function(t,e){var n;if("string"==typeof t){if(!(n=this.getSchema(t)))throw new Error('no schema with key or ref "'+t+'"')}else{var r=this._addSchema(t);n=r.validate||this._compile(r)}var i=n(e);!0!==n.$async&&(this.errors=n.errors);return i},I.prototype.compile=function(t,e){var n=this._addSchema(t,void 0,e);return n.validate||this._compile(n)},I.prototype.addSchema=function(t,e,n,r){if(Array.isArray(t)){for(var o=0;o<t.length;o++)this.addSchema(t[o],void 0,n,r);return this}var a=this._getId(t);if(void 0!==a&&"string"!=typeof a)throw new Error("schema id must be string");return B(this,e=i.normalizeId(e||a)),this._schemas[e]=this._addSchema(t,n,r,!0),this},I.prototype.addMetaSchema=function(t,e,n){return this.addSchema(t,e,n,!0),this},I.prototype.validateSchema=function(t,e){var n=t.$schema;if(void 0!==n&&"string"!=typeof n)throw new Error("$schema must be a string");if(!(n=n||this._opts.defaultMeta||function(t){var e=t._opts.meta;return t._opts.defaultMeta="object"==typeof e?t._getId(e)||e:t.getSchema(d)?d:void 0,t._opts.defaultMeta}(this)))return this.logger.warn("meta-schema not available"),this.errors=null,!0;var r=this.validate(n,t);if(!r&&e){var i="schema is invalid: "+this.errorsText();if("log"!=this._opts.validateSchema)throw new Error(i);this.logger.error(i)}return r},I.prototype.getSchema=function(t){var e=C(this,t);switch(typeof e){case"object":return e.validate||this._compile(e);case"string":return this.getSchema(e);case"undefined":return function(t,e){var n=i.schema.call(t,{schema:{}},e);if(n){var o=n.schema,s=n.root,A=n.baseId,l=r.call(t,o,s,void 0,A);return t._fragments[e]=new a({ref:e,fragment:!0,schema:o,root:s,baseId:A,validate:l}),l}}(this,t)}},I.prototype.removeSchema=function(t){if(t instanceof RegExp)return m(this,this._schemas,t),m(this,this._refs,t),this;switch(typeof t){case"undefined":return m(this,this._schemas),m(this,this._refs),this._cache.clear(),this;case"string":var e=C(this,t);return e&&this._cache.del(e.cacheKey),delete this._schemas[t],delete this._refs[t],this;case"object":var n=this._opts.serialize,r=n?n(t):t;this._cache.del(r);var o=this._getId(t);o&&(o=i.normalizeId(o),delete this._schemas[o],delete this._refs[o])}return this},I.prototype.addFormat=function(t,e){"string"==typeof e&&(e=new RegExp(e));return this._formats[t]=e,this},I.prototype.errorsText=function(t,e){if(!(t=t||this.errors))return"No errors";for(var n=void 0===(e=e||{}).separator?", ":e.separator,r=void 0===e.dataVar?"data":e.dataVar,i="",o=0;o<t.length;o++){var a=t[o];a&&(i+=r+a.dataPath+" "+a.message+n)}return i.slice(0,-n.length)},I.prototype._addSchema=function(t,e,n,r){if("object"!=typeof t&&"boolean"!=typeof t)throw new Error("schema should be object or boolean");var o=this._opts.serialize,s=o?o(t):t,A=this._cache.get(s);if(A)return A;r=r||!1!==this._opts.addUsedSchema;var l=i.normalizeId(this._getId(t));l&&r&&B(this,l);var c,u=!1!==this._opts.validateSchema&&!e;u&&!(c=l&&l==i.normalizeId(t.$schema))&&this.validateSchema(t,!0);var g=i.ids.call(this,t),h=new a({id:l,schema:t,localRefs:g,cacheKey:s,meta:n});"#"!=l[0]&&r&&(this._refs[l]=h);this._cache.put(s,h),u&&c&&this.validateSchema(t,!0);return h},I.prototype._compile=function(t,e){if(t.compiling)return t.validate=o,o.schema=t.schema,o.errors=null,o.root=e||o,!0===t.schema.$async&&(o.$async=!0),o;var n,i;t.compiling=!0,t.meta&&(n=this._opts,this._opts=this._metaOpts);try{i=r.call(this,t.schema,e,t.localRefs)}catch(e){throw delete t.validate,e}finally{t.compiling=!1,t.meta&&(this._opts=n)}return t.validate=i,t.refs=i.refs,t.refVal=i.refVal,t.root=i.root,i;function o(){var e=t.validate,n=e.apply(this,arguments);return o.errors=e.errors,n}},I.prototype.compileAsync=n(894);var g=n(895);I.prototype.addKeyword=g.add,I.prototype.getKeyword=g.get,I.prototype.removeKeyword=g.remove,I.prototype.validateKeyword=g.validate;var h=n(284);I.ValidationError=h.Validation,I.MissingRefError=h.MissingRef,I.$dataMetaSchema=c;var d="http://json-schema.org/draft-07/schema",f=["removeAdditional","useDefaults","coerceTypes","strictDefaults"],p=["/properties"];function I(t){if(!(this instanceof I))return new I(t);t=this._opts=u.copy(t)||{},function(t){var e=t._opts.logger;if(!1===e)t.logger={log:b,warn:b,error:b};else{if(void 0===e&&(e=console),!("object"==typeof e&&e.log&&e.warn&&e.error))throw new Error("logger must implement log, warn and error methods");t.logger=e}}(this),this._schemas={},this._refs={},this._fragments={},this._formats=A(t.format),this._cache=t.cache||new o,this._loadingSchemas={},this._compilations=[],this.RULES=l(),this._getId=function(t){switch(t.schemaId){case"auto":return v;case"id":return y;default:return E}}(t),t.loopRequired=t.loopRequired||1/0,"property"==t.errorDataPath&&(t._errorDataPathProperty=!0),void 0===t.serialize&&(t.serialize=s),this._metaOpts=function(t){for(var e=u.copy(t._opts),n=0;n<f.length;n++)delete e[f[n]];return e}(this),t.formats&&function(t){for(var e in t._opts.formats){var n=t._opts.formats[e];t.addFormat(e,n)}}(this),function(t){var e;t._opts.$data&&(e=n(898),t.addMetaSchema(e,e.$id,!0));if(!1===t._opts.meta)return;var r=n(439);t._opts.$data&&(r=c(r,p));t.addMetaSchema(r,d,!0),t._refs["http://json-schema.org/schema"]=d}(this),"object"==typeof t.meta&&this.addMetaSchema(t.meta),t.nullable&&this.addKeyword("nullable",{metaSchema:{type:"boolean"}}),function(t){var e=t._opts.schemas;if(!e)return;if(Array.isArray(e))t.addSchema(e);else for(var n in e)t.addSchema(e[n],n)}(this)}function C(t,e){return e=i.normalizeId(e),t._schemas[e]||t._refs[e]||t._fragments[e]}function m(t,e,n){for(var r in e){var i=e[r];i.meta||n&&!n.test(r)||(t._cache.del(i.cacheKey),delete e[r])}}function y(t){return t.$id&&this.logger.warn("schema $id ignored",t.$id),t.id}function E(t){return t.id&&this.logger.warn("schema id ignored",t.id),t.$id}function v(t){if(t.$id&&t.id&&t.$id!=t.id)throw new Error("schema $id is different from id");return t.$id||t.id}function B(t,e){if(t._schemas[e]||t._refs[e])throw new Error('schema with key or id "'+e+'" already exists')}function b(){}},function(t,e,n){"use strict";var r=n(282),i=n(137),o=n(284),a=n(433),s=n(434),A=i.ucs2length,l=n(283),c=o.Validation;function u(t,e,n){for(var r=0;r<this._compilations.length;r++){var i=this._compilations[r];if(i.schema==t&&i.root==e&&i.baseId==n)return r}return-1}function g(t,e){return"var pattern"+t+" = new RegExp("+i.toQuotedString(e[t])+");"}function h(t){return"var default"+t+" = defaults["+t+"];"}function d(t,e){return void 0===e[t]?"":"var refVal"+t+" = refVal["+t+"];"}function f(t){return"var customRule"+t+" = customRules["+t+"];"}function p(t,e){if(!t.length)return"";for(var n="",r=0;r<t.length;r++)n+=e(r,t);return n}t.exports=function t(e,n,I,C){var m=this,y=this._opts,E=[void 0],v={},B=[],b={},Q=[],w={},x=[];n=n||{schema:e,refVal:E,refs:v};var _=function(t,e,n){var r=u.call(this,t,e,n);return r>=0?{index:r,compiling:!0}:(r=this._compilations.length,this._compilations[r]={schema:t,root:e,baseId:n},{index:r,compiling:!1})}.call(this,e,n,C);var D=this._compilations[_.index];if(_.compiling)return D.callValidate=function t(){var e=D.validate;var n=e.apply(this,arguments);t.errors=e.errors;return n};var M=this._formats;var k=this.RULES;try{var S=R(e,n,I,C);D.validate=S;var L=D.callValidate;return L&&(L.schema=S.schema,L.errors=null,L.refs=S.refs,L.refVal=S.refVal,L.root=S.root,L.$async=S.$async,y.sourceCode&&(L.source=S.source)),S}finally{(function(t,e,n){var r=u.call(this,t,e,n);r>=0&&this._compilations.splice(r,1)}).call(this,e,n,C)}function R(e,a,u,I){var C=!a||a&&a.schema==e;if(a.schema!=n.schema)return t.call(m,e,a,u,I);var b,w=!0===e.$async,_=s({isTop:!0,schema:e,isRoot:C,baseId:I,root:a,schemaPath:"",errSchemaPath:"#",errorPath:'""',MissingRefError:o.MissingRef,RULES:k,validate:s,util:i,resolve:r,resolveRef:N,usePattern:U,useDefault:O,useCustomRule:G,opts:y,formats:M,logger:m.logger,self:m});_=p(E,d)+p(B,g)+p(Q,h)+p(x,f)+_,y.processCode&&(_=y.processCode(_));try{var D=new Function("self","RULES","formats","root","refVal","defaults","customRules","equal","ucs2length","ValidationError",_);b=D(m,k,M,n,E,Q,x,l,A,c),E[0]=b}catch(t){throw m.logger.error("Error compiling schema, function code:",_),t}return b.schema=e,b.errors=null,b.refs=v,b.refVal=E,b.root=C?b:a,w&&(b.$async=!0),!0===y.sourceCode&&(b.source={code:_,patterns:B,defaults:Q}),b}function N(e,i,o){i=r.url(e,i);var a,s,A=v[i];if(void 0!==A)return T(a=E[A],s="refVal["+A+"]");if(!o&&n.refs){var l=n.refs[i];if(void 0!==l)return a=n.refVal[l],s=F(i,a),T(a,s)}s=F(i);var c=r.call(m,R,n,i);if(void 0===c){var u=I&&I[i];u&&(c=r.inlineRef(u,y.inlineRefs)?u:t.call(m,u,n,I,e))}if(void 0!==c)return function(t,e){var n=v[t];E[n]=e}(i,c),T(c,s);!function(t){delete v[t]}(i)}function F(t,e){var n=E.length;return E[n]=e,v[t]=n,"refVal"+n}function T(t,e){return"object"==typeof t||"boolean"==typeof t?{code:e,schema:t,inline:!0}:{code:e,$async:t&&!!t.$async}}function U(t){var e=b[t];return void 0===e&&(e=b[t]=B.length,B[e]=t),"pattern"+e}function O(t){switch(typeof t){case"boolean":case"number":return""+t;case"string":return i.toQuotedString(t);case"object":if(null===t)return"null";var e=a(t),n=w[e];return void 0===n&&(n=w[e]=Q.length,Q[n]=t),"default"+n}}function G(t,e,n,r){if(!1!==m._opts.validateSchema){var i=t.definition.dependencies;if(i&&!i.every(function(t){return Object.prototype.hasOwnProperty.call(n,t)}))throw new Error("parent schema must have all required keywords: "+i.join(","));var o=t.definition.validateSchema;if(o){var a=o(e);if(!a){var s="keyword schema is invalid: "+m.errorsText(o.errors);if("log"!=m._opts.validateSchema)throw new Error(s);m.logger.error(s)}}}var A,l=t.definition.compile,c=t.definition.inline,u=t.definition.macro;if(l)A=l.call(m,e,n,r);else if(u)A=u.call(m,e,n,r),!1!==y.validateSchema&&m.validateSchema(A,!0);else if(c)A=c.call(m,r,t.keyword,e,n);else if(!(A=t.definition.validate))return;if(void 0===A)throw new Error('custom keyword "'+t.keyword+'"failed to compile');var g=x.length;return x[g]=A,{code:"customRule"+g,validate:A}}}},function(t,e,n){
/** @license URI.js v4.2.1 (c) 2011 Gary Court. License: http://github.com/garycourt/uri-js */
!function(t){"use strict";function e(){for(var t=arguments.length,e=Array(t),n=0;n<t;n++)e[n]=arguments[n];if(e.length>1){e[0]=e[0].slice(0,-1);for(var r=e.length-1,i=1;i<r;++i)e[i]=e[i].slice(1,-1);return e[r]=e[r].slice(1),e.join("")}return e[0]}function n(t){return"(?:"+t+")"}function r(t){return void 0===t?"undefined":null===t?"null":Object.prototype.toString.call(t).split(" ").pop().split("]").shift().toLowerCase()}function i(t){return t.toUpperCase()}function o(t){var r=e("[0-9]","[A-Fa-f]"),i=n(n("%[EFef]"+r+"%"+r+r+"%"+r+r)+"|"+n("%[89A-Fa-f]"+r+"%"+r+r)+"|"+n("%"+r+r)),o="[\\!\\$\\&\\'\\(\\)\\*\\+\\,\\;\\=]",a=e("[\\:\\/\\?\\#\\[\\]\\@]",o),s=t?"[\\xA0-\\u200D\\u2010-\\u2029\\u202F-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFEF]":"[]",A=t?"[\\uE000-\\uF8FF]":"[]",l=e("[A-Za-z]","[0-9]","[\\-\\.\\_\\~]",s),c=n("[A-Za-z]"+e("[A-Za-z]","[0-9]","[\\+\\-\\.]")+"*"),u=n(n(i+"|"+e(l,o,"[\\:]"))+"*"),g=(n(n("25[0-5]")+"|"+n("2[0-4][0-9]")+"|"+n("1[0-9][0-9]")+"|"+n("[1-9][0-9]")+"|[0-9]"),n(n("25[0-5]")+"|"+n("2[0-4][0-9]")+"|"+n("1[0-9][0-9]")+"|"+n("0?[1-9][0-9]")+"|0?0?[0-9]")),h=n(g+"\\."+g+"\\."+g+"\\."+g),d=n(r+"{1,4}"),f=n(n(d+"\\:"+d)+"|"+h),p=n(n(d+"\\:")+"{6}"+f),I=n("\\:\\:"+n(d+"\\:")+"{5}"+f),C=n(n(d)+"?\\:\\:"+n(d+"\\:")+"{4}"+f),m=n(n(n(d+"\\:")+"{0,1}"+d)+"?\\:\\:"+n(d+"\\:")+"{3}"+f),y=n(n(n(d+"\\:")+"{0,2}"+d)+"?\\:\\:"+n(d+"\\:")+"{2}"+f),E=n(n(n(d+"\\:")+"{0,3}"+d)+"?\\:\\:"+d+"\\:"+f),v=n(n(n(d+"\\:")+"{0,4}"+d)+"?\\:\\:"+f),B=n(n(n(d+"\\:")+"{0,5}"+d)+"?\\:\\:"+d),b=n(n(n(d+"\\:")+"{0,6}"+d)+"?\\:\\:"),Q=n([p,I,C,m,y,E,v,B,b].join("|")),w=n(n(l+"|"+i)+"+"),x=(n(Q+"\\%25"+w),n(Q+n("\\%25|\\%(?!"+r+"{2})")+w)),_=n("[vV]"+r+"+\\."+e(l,o,"[\\:]")+"+"),D=n("\\["+n(x+"|"+Q+"|"+_)+"\\]"),M=n(n(i+"|"+e(l,o))+"*"),k=n(D+"|"+h+"(?!"+M+")|"+M),S=n("[0-9]*"),L=n(n(u+"@")+"?"+k+n("\\:"+S)+"?"),R=n(i+"|"+e(l,o,"[\\:\\@]")),N=n(R+"*"),F=n(R+"+"),T=n(n(i+"|"+e(l,o,"[\\@]"))+"+"),U=n(n("\\/"+N)+"*"),O=n("\\/"+n(F+U)+"?"),G=n(T+U),H=n(F+U),j="(?!"+R+")",Y=(n(U+"|"+O+"|"+G+"|"+H+"|"+j),n(n(R+"|"+e("[\\/\\?]",A))+"*")),P=n(n(R+"|[\\/\\?]")+"*"),K=n(n("\\/\\/"+L+U)+"|"+O+"|"+H+"|"+j),z=n(c+"\\:"+K+n("\\?"+Y)+"?"+n("\\#"+P)+"?"),J=n(n("\\/\\/"+L+U)+"|"+O+"|"+G+"|"+j),q=n(J+n("\\?"+Y)+"?"+n("\\#"+P)+"?");return n(z+"|"+q),n(c+"\\:"+K+n("\\?"+Y)+"?"),n(n("\\/\\/("+n("("+u+")@")+"?("+k+")"+n("\\:("+S+")")+"?)")+"?("+U+"|"+O+"|"+H+"|"+j+")"),n("\\?("+Y+")"),n("\\#("+P+")"),n(n("\\/\\/("+n("("+u+")@")+"?("+k+")"+n("\\:("+S+")")+"?)")+"?("+U+"|"+O+"|"+G+"|"+j+")"),n("\\?("+Y+")"),n("\\#("+P+")"),n(n("\\/\\/("+n("("+u+")@")+"?("+k+")"+n("\\:("+S+")")+"?)")+"?("+U+"|"+O+"|"+H+"|"+j+")"),n("\\?("+Y+")"),n("\\#("+P+")"),n("("+u+")@"),n("\\:("+S+")"),{NOT_SCHEME:new RegExp(e("[^]","[A-Za-z]","[0-9]","[\\+\\-\\.]"),"g"),NOT_USERINFO:new RegExp(e("[^\\%\\:]",l,o),"g"),NOT_HOST:new RegExp(e("[^\\%\\[\\]\\:]",l,o),"g"),NOT_PATH:new RegExp(e("[^\\%\\/\\:\\@]",l,o),"g"),NOT_PATH_NOSCHEME:new RegExp(e("[^\\%\\/\\@]",l,o),"g"),NOT_QUERY:new RegExp(e("[^\\%]",l,o,"[\\:\\@\\/\\?]",A),"g"),NOT_FRAGMENT:new RegExp(e("[^\\%]",l,o,"[\\:\\@\\/\\?]"),"g"),ESCAPE:new RegExp(e("[^]",l,o),"g"),UNRESERVED:new RegExp(l,"g"),OTHER_CHARS:new RegExp(e("[^\\%]",l,a),"g"),PCT_ENCODED:new RegExp(i,"g"),IPV4ADDRESS:new RegExp("^("+h+")$"),IPV6ADDRESS:new RegExp("^\\[?("+Q+")"+n(n("\\%25|\\%(?!"+r+"{2})")+"("+w+")")+"?\\]?$")}}var a=o(!1),s=o(!0),A=function(t,e){if(Array.isArray(t))return t;if(Symbol.iterator in Object(t))return function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!e||n.length!==e);r=!0);}catch(t){i=!0,o=t}finally{try{!r&&s.return&&s.return()}finally{if(i)throw o}}return n}(t,e);throw new TypeError("Invalid attempt to destructure non-iterable instance")},l=2147483647,c=/^xn--/,u=/[^\0-\x7E]/,g=/[\x2E\u3002\uFF0E\uFF61]/g,h={overflow:"Overflow: input needs wider integers to process","not-basic":"Illegal input >= 0x80 (not a basic code point)","invalid-input":"Invalid input"},d=Math.floor,f=String.fromCharCode;function p(t){throw new RangeError(h[t])}function I(t,e){var n=t.split("@"),r="";n.length>1&&(r=n[0]+"@",t=n[1]);var i=(t=t.replace(g,".")).split("."),o=function(t,e){for(var n=[],r=t.length;r--;)n[r]=e(t[r]);return n}(i,e).join(".");return r+o}function C(t){for(var e=[],n=0,r=t.length;n<r;){var i=t.charCodeAt(n++);if(i>=55296&&i<=56319&&n<r){var o=t.charCodeAt(n++);56320==(64512&o)?e.push(((1023&i)<<10)+(1023&o)+65536):(e.push(i),n--)}else e.push(i)}return e}var m=function(t,e){return t+22+75*(t<26)-((0!=e)<<5)},y=function(t,e,n){var r=0;for(t=n?d(t/700):t>>1,t+=d(t/e);t>455;r+=36)t=d(t/35);return d(r+36*t/(t+38))},E=function(t){var e,n=[],r=t.length,i=0,o=128,a=72,s=t.lastIndexOf("-");s<0&&(s=0);for(var A=0;A<s;++A)t.charCodeAt(A)>=128&&p("not-basic"),n.push(t.charCodeAt(A));for(var c=s>0?s+1:0;c<r;){for(var u=i,g=1,h=36;;h+=36){c>=r&&p("invalid-input");var f=(e=t.charCodeAt(c++))-48<10?e-22:e-65<26?e-65:e-97<26?e-97:36;(f>=36||f>d((l-i)/g))&&p("overflow"),i+=f*g;var I=h<=a?1:h>=a+26?26:h-a;if(f<I)break;var C=36-I;g>d(l/C)&&p("overflow"),g*=C}var m=n.length+1;a=y(i-u,m,0==u),d(i/m)>l-o&&p("overflow"),o+=d(i/m),i%=m,n.splice(i++,0,o)}return String.fromCodePoint.apply(String,n)},v=function(t){var e=[],n=(t=C(t)).length,r=128,i=0,o=72,a=!0,s=!1,A=void 0;try{for(var c,u=t[Symbol.iterator]();!(a=(c=u.next()).done);a=!0){var g=c.value;g<128&&e.push(f(g))}}catch(t){s=!0,A=t}finally{try{!a&&u.return&&u.return()}finally{if(s)throw A}}var h=e.length,I=h;for(h&&e.push("-");I<n;){var E=l,v=!0,B=!1,b=void 0;try{for(var Q,w=t[Symbol.iterator]();!(v=(Q=w.next()).done);v=!0){var x=Q.value;x>=r&&x<E&&(E=x)}}catch(t){B=!0,b=t}finally{try{!v&&w.return&&w.return()}finally{if(B)throw b}}var _=I+1;E-r>d((l-i)/_)&&p("overflow"),i+=(E-r)*_,r=E;var D=!0,M=!1,k=void 0;try{for(var S,L=t[Symbol.iterator]();!(D=(S=L.next()).done);D=!0){var R=S.value;if(R<r&&++i>l&&p("overflow"),R==r){for(var N=i,F=36;;F+=36){var T=F<=o?1:F>=o+26?26:F-o;if(N<T)break;var U=N-T,O=36-T;e.push(f(m(T+U%O,0))),N=d(U/O)}e.push(f(m(N,0))),o=y(i,_,I==h),i=0,++I}}}catch(t){M=!0,k=t}finally{try{!D&&L.return&&L.return()}finally{if(M)throw k}}++i,++r}return e.join("")},B={version:"2.1.0",ucs2:{decode:C,encode:function(t){return String.fromCodePoint.apply(String,function(t){if(Array.isArray(t)){for(var e=0,n=Array(t.length);e<t.length;e++)n[e]=t[e];return n}return Array.from(t)}(t))}},decode:E,encode:v,toASCII:function(t){return I(t,function(t){return u.test(t)?"xn--"+v(t):t})},toUnicode:function(t){return I(t,function(t){return c.test(t)?E(t.slice(4).toLowerCase()):t})}},b={};function Q(t){var e=t.charCodeAt(0);return e<16?"%0"+e.toString(16).toUpperCase():e<128?"%"+e.toString(16).toUpperCase():e<2048?"%"+(e>>6|192).toString(16).toUpperCase()+"%"+(63&e|128).toString(16).toUpperCase():"%"+(e>>12|224).toString(16).toUpperCase()+"%"+(e>>6&63|128).toString(16).toUpperCase()+"%"+(63&e|128).toString(16).toUpperCase()}function w(t){for(var e="",n=0,r=t.length;n<r;){var i=parseInt(t.substr(n+1,2),16);if(i<128)e+=String.fromCharCode(i),n+=3;else if(i>=194&&i<224){if(r-n>=6){var o=parseInt(t.substr(n+4,2),16);e+=String.fromCharCode((31&i)<<6|63&o)}else e+=t.substr(n,6);n+=6}else if(i>=224){if(r-n>=9){var a=parseInt(t.substr(n+4,2),16),s=parseInt(t.substr(n+7,2),16);e+=String.fromCharCode((15&i)<<12|(63&a)<<6|63&s)}else e+=t.substr(n,9);n+=9}else e+=t.substr(n,3),n+=3}return e}function x(t,e){function n(t){var n=w(t);return n.match(e.UNRESERVED)?n:t}return t.scheme&&(t.scheme=String(t.scheme).replace(e.PCT_ENCODED,n).toLowerCase().replace(e.NOT_SCHEME,"")),void 0!==t.userinfo&&(t.userinfo=String(t.userinfo).replace(e.PCT_ENCODED,n).replace(e.NOT_USERINFO,Q).replace(e.PCT_ENCODED,i)),void 0!==t.host&&(t.host=String(t.host).replace(e.PCT_ENCODED,n).toLowerCase().replace(e.NOT_HOST,Q).replace(e.PCT_ENCODED,i)),void 0!==t.path&&(t.path=String(t.path).replace(e.PCT_ENCODED,n).replace(t.scheme?e.NOT_PATH:e.NOT_PATH_NOSCHEME,Q).replace(e.PCT_ENCODED,i)),void 0!==t.query&&(t.query=String(t.query).replace(e.PCT_ENCODED,n).replace(e.NOT_QUERY,Q).replace(e.PCT_ENCODED,i)),void 0!==t.fragment&&(t.fragment=String(t.fragment).replace(e.PCT_ENCODED,n).replace(e.NOT_FRAGMENT,Q).replace(e.PCT_ENCODED,i)),t}function _(t){return t.replace(/^0*(.*)/,"$1")||"0"}function D(t,e){var n=t.match(e.IPV4ADDRESS)||[],r=A(n,2),i=r[1];return i?i.split(".").map(_).join("."):t}function M(t,e){var n=t.match(e.IPV6ADDRESS)||[],r=A(n,3),i=r[1],o=r[2];if(i){for(var a=i.toLowerCase().split("::").reverse(),s=A(a,2),l=s[0],c=s[1],u=c?c.split(":").map(_):[],g=l.split(":").map(_),h=e.IPV4ADDRESS.test(g[g.length-1]),d=h?7:8,f=g.length-d,p=Array(d),I=0;I<d;++I)p[I]=u[I]||g[f+I]||"";h&&(p[d-1]=D(p[d-1],e));var C=p.reduce(function(t,e,n){if(!e||"0"===e){var r=t[t.length-1];r&&r.index+r.length===n?r.length++:t.push({index:n,length:1})}return t},[]),m=C.sort(function(t,e){return e.length-t.length})[0],y=void 0;if(m&&m.length>1){var E=p.slice(0,m.index),v=p.slice(m.index+m.length);y=E.join(":")+"::"+v.join(":")}else y=p.join(":");return o&&(y+="%"+o),y}return t}var k=/^(?:([^:\/?#]+):)?(?:\/\/((?:([^\/?#@]*)@)?(\[[^\/?#\]]+\]|[^\/?#:]*)(?:\:(\d*))?))?([^?#]*)(?:\?([^#]*))?(?:#((?:.|\n|\r)*))?/i,S=void 0==="".match(/(){0}/)[1];function L(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n={},r=!1!==e.iri?s:a;"suffix"===e.reference&&(t=(e.scheme?e.scheme+":":"")+"//"+t);var i=t.match(k);if(i){S?(n.scheme=i[1],n.userinfo=i[3],n.host=i[4],n.port=parseInt(i[5],10),n.path=i[6]||"",n.query=i[7],n.fragment=i[8],isNaN(n.port)&&(n.port=i[5])):(n.scheme=i[1]||void 0,n.userinfo=-1!==t.indexOf("@")?i[3]:void 0,n.host=-1!==t.indexOf("//")?i[4]:void 0,n.port=parseInt(i[5],10),n.path=i[6]||"",n.query=-1!==t.indexOf("?")?i[7]:void 0,n.fragment=-1!==t.indexOf("#")?i[8]:void 0,isNaN(n.port)&&(n.port=t.match(/\/\/(?:.|\n)*\:(?:\/|\?|\#|$)/)?i[4]:void 0)),n.host&&(n.host=M(D(n.host,r),r)),void 0!==n.scheme||void 0!==n.userinfo||void 0!==n.host||void 0!==n.port||n.path||void 0!==n.query?void 0===n.scheme?n.reference="relative":void 0===n.fragment?n.reference="absolute":n.reference="uri":n.reference="same-document",e.reference&&"suffix"!==e.reference&&e.reference!==n.reference&&(n.error=n.error||"URI is not a "+e.reference+" reference.");var o=b[(e.scheme||n.scheme||"").toLowerCase()];if(e.unicodeSupport||o&&o.unicodeSupport)x(n,r);else{if(n.host&&(e.domainHost||o&&o.domainHost))try{n.host=B.toASCII(n.host.replace(r.PCT_ENCODED,w).toLowerCase())}catch(t){n.error=n.error||"Host's domain name can not be converted to ASCII via punycode: "+t}x(n,a)}o&&o.parse&&o.parse(n,e)}else n.error=n.error||"URI can not be parsed.";return n}var R=/^\.\.?\//,N=/^\/\.(\/|$)/,F=/^\/\.\.(\/|$)/,T=/^\/?(?:.|\n)*?(?=\/|$)/;function U(t){for(var e=[];t.length;)if(t.match(R))t=t.replace(R,"");else if(t.match(N))t=t.replace(N,"/");else if(t.match(F))t=t.replace(F,"/"),e.pop();else if("."===t||".."===t)t="";else{var n=t.match(T);if(!n)throw new Error("Unexpected dot segment condition");var r=n[0];t=t.slice(r.length),e.push(r)}return e.join("")}function O(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=e.iri?s:a,r=[],i=b[(e.scheme||t.scheme||"").toLowerCase()];if(i&&i.serialize&&i.serialize(t,e),t.host)if(n.IPV6ADDRESS.test(t.host));else if(e.domainHost||i&&i.domainHost)try{t.host=e.iri?B.toUnicode(t.host):B.toASCII(t.host.replace(n.PCT_ENCODED,w).toLowerCase())}catch(n){t.error=t.error||"Host's domain name can not be converted to "+(e.iri?"Unicode":"ASCII")+" via punycode: "+n}x(t,n),"suffix"!==e.reference&&t.scheme&&(r.push(t.scheme),r.push(":"));var o=function(t,e){var n=!1!==e.iri?s:a,r=[];return void 0!==t.userinfo&&(r.push(t.userinfo),r.push("@")),void 0!==t.host&&r.push(M(D(String(t.host),n),n).replace(n.IPV6ADDRESS,function(t,e,n){return"["+e+(n?"%25"+n:"")+"]"})),"number"==typeof t.port&&(r.push(":"),r.push(t.port.toString(10))),r.length?r.join(""):void 0}(t,e);if(void 0!==o&&("suffix"!==e.reference&&r.push("//"),r.push(o),t.path&&"/"!==t.path.charAt(0)&&r.push("/")),void 0!==t.path){var A=t.path;e.absolutePath||i&&i.absolutePath||(A=U(A)),void 0===o&&(A=A.replace(/^\/\//,"/%2F")),r.push(A)}return void 0!==t.query&&(r.push("?"),r.push(t.query)),void 0!==t.fragment&&(r.push("#"),r.push(t.fragment)),r.join("")}function G(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=arguments[3],i={};return r||(t=L(O(t,n),n),e=L(O(e,n),n)),!(n=n||{}).tolerant&&e.scheme?(i.scheme=e.scheme,i.userinfo=e.userinfo,i.host=e.host,i.port=e.port,i.path=U(e.path||""),i.query=e.query):(void 0!==e.userinfo||void 0!==e.host||void 0!==e.port?(i.userinfo=e.userinfo,i.host=e.host,i.port=e.port,i.path=U(e.path||""),i.query=e.query):(e.path?("/"===e.path.charAt(0)?i.path=U(e.path):(void 0===t.userinfo&&void 0===t.host&&void 0===t.port||t.path?t.path?i.path=t.path.slice(0,t.path.lastIndexOf("/")+1)+e.path:i.path=e.path:i.path="/"+e.path,i.path=U(i.path)),i.query=e.query):(i.path=t.path,void 0!==e.query?i.query=e.query:i.query=t.query),i.userinfo=t.userinfo,i.host=t.host,i.port=t.port),i.scheme=t.scheme),i.fragment=e.fragment,i}function H(t,e){return t&&t.toString().replace(e&&e.iri?s.PCT_ENCODED:a.PCT_ENCODED,w)}var j={scheme:"http",domainHost:!0,parse:function(t,e){return t.host||(t.error=t.error||"HTTP URIs must have a host."),t},serialize:function(t,e){return t.port!==("https"!==String(t.scheme).toLowerCase()?80:443)&&""!==t.port||(t.port=void 0),t.path||(t.path="/"),t}},Y={scheme:"https",domainHost:j.domainHost,parse:j.parse,serialize:j.serialize},P={},K="[A-Za-z0-9\\-\\.\\_\\~\\xA0-\\u200D\\u2010-\\u2029\\u202F-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFEF]",z="[0-9A-Fa-f]",J=n(n("%[EFef][0-9A-Fa-f]%"+z+z+"%"+z+z)+"|"+n("%[89A-Fa-f][0-9A-Fa-f]%"+z+z)+"|"+n("%"+z+z)),q=e("[\\!\\$\\%\\'\\(\\)\\*\\+\\,\\-\\.0-9\\<\\>A-Z\\x5E-\\x7E]",'[\\"\\\\]'),V=new RegExp(K,"g"),W=new RegExp(J,"g"),X=new RegExp(e("[^]","[A-Za-z0-9\\!\\$\\%\\'\\*\\+\\-\\^\\_\\`\\{\\|\\}\\~]","[\\.]",'[\\"]',q),"g"),Z=new RegExp(e("[^]",K,"[\\!\\$\\'\\(\\)\\*\\+\\,\\;\\:\\@]"),"g"),$=Z;function tt(t){var e=w(t);return e.match(V)?e:t}var et={scheme:"mailto",parse:function(t,e){var n=t,r=n.to=n.path?n.path.split(","):[];if(n.path=void 0,n.query){for(var i=!1,o={},a=n.query.split("&"),s=0,A=a.length;s<A;++s){var l=a[s].split("=");switch(l[0]){case"to":for(var c=l[1].split(","),u=0,g=c.length;u<g;++u)r.push(c[u]);break;case"subject":n.subject=H(l[1],e);break;case"body":n.body=H(l[1],e);break;default:i=!0,o[H(l[0],e)]=H(l[1],e)}}i&&(n.headers=o)}n.query=void 0;for(var h=0,d=r.length;h<d;++h){var f=r[h].split("@");if(f[0]=H(f[0]),e.unicodeSupport)f[1]=H(f[1],e).toLowerCase();else try{f[1]=B.toASCII(H(f[1],e).toLowerCase())}catch(t){n.error=n.error||"Email address's domain name can not be converted to ASCII via punycode: "+t}r[h]=f.join("@")}return n},serialize:function(t,e){var n,r=t,o=null!=(n=t.to)?n instanceof Array?n:"number"!=typeof n.length||n.split||n.setInterval||n.call?[n]:Array.prototype.slice.call(n):[];if(o){for(var a=0,s=o.length;a<s;++a){var A=String(o[a]),l=A.lastIndexOf("@"),c=A.slice(0,l).replace(W,tt).replace(W,i).replace(X,Q),u=A.slice(l+1);try{u=e.iri?B.toUnicode(u):B.toASCII(H(u,e).toLowerCase())}catch(t){r.error=r.error||"Email address's domain name can not be converted to "+(e.iri?"Unicode":"ASCII")+" via punycode: "+t}o[a]=c+"@"+u}r.path=o.join(",")}var g=t.headers=t.headers||{};t.subject&&(g.subject=t.subject),t.body&&(g.body=t.body);var h=[];for(var d in g)g[d]!==P[d]&&h.push(d.replace(W,tt).replace(W,i).replace(Z,Q)+"="+g[d].replace(W,tt).replace(W,i).replace($,Q));return h.length&&(r.query=h.join("&")),r}},nt=/^([^\:]+)\:(.*)/,rt={scheme:"urn",parse:function(t,e){var n=t.path&&t.path.match(nt),r=t;if(n){var i=e.scheme||r.scheme||"urn",o=n[1].toLowerCase(),a=n[2],s=i+":"+(e.nid||o),A=b[s];r.nid=o,r.nss=a,r.path=void 0,A&&(r=A.parse(r,e))}else r.error=r.error||"URN can not be parsed.";return r},serialize:function(t,e){var n=e.scheme||t.scheme||"urn",r=t.nid,i=n+":"+(e.nid||r),o=b[i];o&&(t=o.serialize(t,e));var a=t,s=t.nss;return a.path=(r||e.nid)+":"+s,a}},it=/^[0-9A-Fa-f]{8}(?:\-[0-9A-Fa-f]{4}){3}\-[0-9A-Fa-f]{12}$/,ot={scheme:"urn:uuid",parse:function(t,e){var n=t;return n.uuid=n.nss,n.nss=void 0,e.tolerant||n.uuid&&n.uuid.match(it)||(n.error=n.error||"UUID is not valid."),n},serialize:function(t,e){var n=t;return n.nss=(t.uuid||"").toLowerCase(),n}};b[j.scheme]=j,b[Y.scheme]=Y,b[et.scheme]=et,b[rt.scheme]=rt,b[ot.scheme]=ot,t.SCHEMES=b,t.pctEncChar=Q,t.pctDecChars=w,t.parse=L,t.removeDotSegments=U,t.serialize=O,t.resolveComponents=G,t.resolve=function(t,e,n){var r=function(t,e){var n=t;if(e)for(var r in e)n[r]=e[r];return n}({scheme:"null"},n);return O(G(L(t,r),L(e,r),r,!0),r)},t.normalize=function(t,e){return"string"==typeof t?t=O(L(t,e),e):"object"===r(t)&&(t=L(O(t,e),e)),t},t.equal=function(t,e,n){return"string"==typeof t?t=O(L(t,n),n):"object"===r(t)&&(t=O(t,n)),"string"==typeof e?e=O(L(e,n),n):"object"===r(e)&&(e=O(e,n)),t===e},t.escapeComponent=function(t,e){return t&&t.toString().replace(e&&e.iri?s.ESCAPE:a.ESCAPE,Q)},t.unescapeComponent=H,Object.defineProperty(t,"__esModule",{value:!0})}(e)},function(t,e,n){"use strict";t.exports=function(t){for(var e,n=0,r=t.length,i=0;i<r;)n++,(e=t.charCodeAt(i++))>=55296&&e<=56319&&i<r&&56320==(64512&(e=t.charCodeAt(i)))&&i++;return n}},function(t,e,n){"use strict";var r=t.exports=function(t,e,n){"function"==typeof e&&(n=e,e={}),function t(e,n,i,o,a,s,A,l,c,u){if(o&&"object"==typeof o&&!Array.isArray(o)){for(var g in n(o,a,s,A,l,c,u),o){var h=o[g];if(Array.isArray(h)){if(g in r.arrayKeywords)for(var d=0;d<h.length;d++)t(e,n,i,h[d],a+"/"+g+"/"+d,s,a,g,o,d)}else if(g in r.propsKeywords){if(h&&"object"==typeof h)for(var f in h)t(e,n,i,h[f],a+"/"+g+"/"+f.replace(/~/g,"~0").replace(/\//g,"~1"),s,a,g,o,f)}else(g in r.keywords||e.allKeys&&!(g in r.skipKeywords))&&t(e,n,i,h,a+"/"+g,s,a,g,o)}i(o,a,s,A,l,c,u)}}(e,"function"==typeof(n=e.cb||n)?n:n.pre||function(){},n.post||function(){},t,"",t)};r.keywords={additionalItems:!0,items:!0,contains:!0,additionalProperties:!0,propertyNames:!0,not:!0},r.arrayKeywords={items:!0,allOf:!0,anyOf:!0,oneOf:!0},r.propsKeywords={definitions:!0,properties:!0,patternProperties:!0,dependencies:!0},r.skipKeywords={default:!0,enum:!0,const:!0,required:!0,maximum:!0,minimum:!0,exclusiveMaximum:!0,exclusiveMinimum:!0,multipleOf:!0,maxLength:!0,minLength:!0,pattern:!0,format:!0,maxItems:!0,minItems:!0,uniqueItems:!0,maxProperties:!0,minProperties:!0}},function(t,e,n){"use strict";var r=t.exports=function(){this._cache={}};r.prototype.put=function(t,e){this._cache[t]=e},r.prototype.get=function(t){return this._cache[t]},r.prototype.del=function(t){delete this._cache[t]},r.prototype.clear=function(){this._cache={}}},function(t,e,n){"use strict";var r=n(137),i=/^(\d\d\d\d)-(\d\d)-(\d\d)$/,o=[0,31,28,31,30,31,30,31,31,30,31,30,31],a=/^(\d\d):(\d\d):(\d\d)(\.\d+)?(z|[+-]\d\d:\d\d)?$/i,s=/^[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?(?:\.[a-z0-9](?:[-0-9a-z]{0,61}[0-9a-z])?)*$/i,A=/^(?:[a-z][a-z0-9+\-.]*:)(?:\/?\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:]|%[0-9a-f]{2})*@)?(?:\[(?:(?:(?:(?:[0-9a-f]{1,4}:){6}|::(?:[0-9a-f]{1,4}:){5}|(?:[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){4}|(?:(?:[0-9a-f]{1,4}:){0,1}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){3}|(?:(?:[0-9a-f]{1,4}:){0,2}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){2}|(?:(?:[0-9a-f]{1,4}:){0,3}[0-9a-f]{1,4})?::[0-9a-f]{1,4}:|(?:(?:[0-9a-f]{1,4}:){0,4}[0-9a-f]{1,4})?::)(?:[0-9a-f]{1,4}:[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?))|(?:(?:[0-9a-f]{1,4}:){0,5}[0-9a-f]{1,4})?::[0-9a-f]{1,4}|(?:(?:[0-9a-f]{1,4}:){0,6}[0-9a-f]{1,4})?::)|[Vv][0-9a-f]+\.[a-z0-9\-._~!$&'()*+,;=:]+)\]|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|(?:[a-z0-9\-._~!$&'()*+,;=]|%[0-9a-f]{2})*)(?::\d*)?(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*|\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*)?|(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*)(?:\?(?:[a-z0-9\-._~!$&'()*+,;=:@\/?]|%[0-9a-f]{2})*)?(?:#(?:[a-z0-9\-._~!$&'()*+,;=:@\/?]|%[0-9a-f]{2})*)?$/i,l=/^(?:(?:[^\x00-\x20"'<>%\\^`{|}]|%[0-9a-f]{2})|\{[+#.\/;?&=,!@|]?(?:[a-z0-9_]|%[0-9a-f]{2})+(?::[1-9][0-9]{0,3}|\*)?(?:,(?:[a-z0-9_]|%[0-9a-f]{2})+(?::[1-9][0-9]{0,3}|\*)?)*\})*$/i,c=/^(?:(?:http[s\u017F]?|ftp):\/\/)(?:(?:[\0-\x08\x0E-\x1F!-\x9F\xA1-\u167F\u1681-\u1FFF\u200B-\u2027\u202A-\u202E\u2030-\u205E\u2060-\u2FFF\u3001-\uD7FF\uE000-\uFEFE\uFF00-\uFFFF]|[\uD800-\uDBFF][\uDC00-\uDFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+(?::(?:[\0-\x08\x0E-\x1F!-\x9F\xA1-\u167F\u1681-\u1FFF\u200B-\u2027\u202A-\u202E\u2030-\u205E\u2060-\u2FFF\u3001-\uD7FF\uE000-\uFEFE\uFF00-\uFFFF]|[\uD800-\uDBFF][\uDC00-\uDFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])*)?@)?(?:(?!10(?:\.[0-9]{1,3}){3})(?!127(?:\.[0-9]{1,3}){3})(?!169\.254(?:\.[0-9]{1,3}){2})(?!192\.168(?:\.[0-9]{1,3}){2})(?!172\.(?:1[6-9]|2[0-9]|3[01])(?:\.[0-9]{1,3}){2})(?:[1-9][0-9]?|1[0-9][0-9]|2[01][0-9]|22[0-3])(?:\.(?:1?[0-9]{1,2}|2[0-4][0-9]|25[0-5])){2}(?:\.(?:[1-9][0-9]?|1[0-9][0-9]|2[0-4][0-9]|25[0-4]))|(?:(?:(?:[0-9KSa-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+-?)*(?:[0-9KSa-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+)(?:\.(?:(?:[0-9KSa-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+-?)*(?:[0-9KSa-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+)*(?:\.(?:(?:[KSa-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]){2,})))(?::[0-9]{2,5})?(?:\/(?:[\0-\x08\x0E-\x1F!-\x9F\xA1-\u167F\u1681-\u1FFF\u200B-\u2027\u202A-\u202E\u2030-\u205E\u2060-\u2FFF\u3001-\uD7FF\uE000-\uFEFE\uFF00-\uFFFF]|[\uD800-\uDBFF][\uDC00-\uDFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])*)?$/i,u=/^(?:urn:uuid:)?[0-9a-f]{8}-(?:[0-9a-f]{4}-){3}[0-9a-f]{12}$/i,g=/^(?:\/(?:[^~\/]|~0|~1)*)*$/,h=/^#(?:\/(?:[a-z0-9_\-.!$&'()*+,;:=@]|%[0-9a-f]{2}|~0|~1)*)*$/i,d=/^(?:0|[1-9][0-9]*)(?:#|(?:\/(?:[^~\/]|~0|~1)*)*)$/;function f(t){return t="full"==t?"full":"fast",r.copy(f[t])}function p(t){var e=t.match(i);if(!e)return!1;var n=+e[1],r=+e[2],a=+e[3];return r>=1&&r<=12&&a>=1&&a<=(2==r&&function(t){return t%4==0&&(t%100!=0||t%400==0)}(n)?29:o[r])}function I(t,e){var n=t.match(a);if(!n)return!1;var r=n[1],i=n[2],o=n[3],s=n[5];return(r<=23&&i<=59&&o<=59||23==r&&59==i&&60==o)&&(!e||s)}t.exports=f,f.fast={date:/^\d\d\d\d-[0-1]\d-[0-3]\d$/,time:/^(?:[0-2]\d:[0-5]\d:[0-5]\d|23:59:60)(?:\.\d+)?(?:z|[+-]\d\d:\d\d)?$/i,"date-time":/^\d\d\d\d-[0-1]\d-[0-3]\d[t\s](?:[0-2]\d:[0-5]\d:[0-5]\d|23:59:60)(?:\.\d+)?(?:z|[+-]\d\d:\d\d)$/i,uri:/^(?:[a-z][a-z0-9+-.]*:)(?:\/?\/)?[^\s]*$/i,"uri-reference":/^(?:(?:[a-z][a-z0-9+-.]*:)?\/?\/)?(?:[^\\\s#][^\s#]*)?(?:#[^\\\s]*)?$/i,"uri-template":l,url:c,email:/^[a-z0-9.!#$%&'*+\/=?^_`{|}~-]+@[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?(?:\.[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?)*$/i,hostname:s,ipv4:/^(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)$/,ipv6:/^\s*(?:(?:(?:[0-9a-f]{1,4}:){7}(?:[0-9a-f]{1,4}|:))|(?:(?:[0-9a-f]{1,4}:){6}(?::[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){5}(?:(?:(?::[0-9a-f]{1,4}){1,2})|:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){4}(?:(?:(?::[0-9a-f]{1,4}){1,3})|(?:(?::[0-9a-f]{1,4})?:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){3}(?:(?:(?::[0-9a-f]{1,4}){1,4})|(?:(?::[0-9a-f]{1,4}){0,2}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){2}(?:(?:(?::[0-9a-f]{1,4}){1,5})|(?:(?::[0-9a-f]{1,4}){0,3}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){1}(?:(?:(?::[0-9a-f]{1,4}){1,6})|(?:(?::[0-9a-f]{1,4}){0,4}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?::(?:(?:(?::[0-9a-f]{1,4}){1,7})|(?:(?::[0-9a-f]{1,4}){0,5}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(?:%.+)?\s*$/i,regex:E,uuid:u,"json-pointer":g,"json-pointer-uri-fragment":h,"relative-json-pointer":d},f.full={date:p,time:I,"date-time":function(t){var e=t.split(C);return 2==e.length&&p(e[0])&&I(e[1],!0)},uri:function(t){return m.test(t)&&A.test(t)},"uri-reference":/^(?:[a-z][a-z0-9+\-.]*:)?(?:\/?\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:]|%[0-9a-f]{2})*@)?(?:\[(?:(?:(?:(?:[0-9a-f]{1,4}:){6}|::(?:[0-9a-f]{1,4}:){5}|(?:[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){4}|(?:(?:[0-9a-f]{1,4}:){0,1}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){3}|(?:(?:[0-9a-f]{1,4}:){0,2}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){2}|(?:(?:[0-9a-f]{1,4}:){0,3}[0-9a-f]{1,4})?::[0-9a-f]{1,4}:|(?:(?:[0-9a-f]{1,4}:){0,4}[0-9a-f]{1,4})?::)(?:[0-9a-f]{1,4}:[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?))|(?:(?:[0-9a-f]{1,4}:){0,5}[0-9a-f]{1,4})?::[0-9a-f]{1,4}|(?:(?:[0-9a-f]{1,4}:){0,6}[0-9a-f]{1,4})?::)|[Vv][0-9a-f]+\.[a-z0-9\-._~!$&'()*+,;=:]+)\]|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|(?:[a-z0-9\-._~!$&'"()*+,;=]|%[0-9a-f]{2})*)(?::\d*)?(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*|\/(?:(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*)?|(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*)?(?:\?(?:[a-z0-9\-._~!$&'"()*+,;=:@\/?]|%[0-9a-f]{2})*)?(?:#(?:[a-z0-9\-._~!$&'"()*+,;=:@\/?]|%[0-9a-f]{2})*)?$/i,"uri-template":l,url:c,email:/^[a-z0-9!#$%&'*+\/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+\/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$/i,hostname:function(t){return t.length<=255&&s.test(t)},ipv4:/^(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)$/,ipv6:/^\s*(?:(?:(?:[0-9a-f]{1,4}:){7}(?:[0-9a-f]{1,4}|:))|(?:(?:[0-9a-f]{1,4}:){6}(?::[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){5}(?:(?:(?::[0-9a-f]{1,4}){1,2})|:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){4}(?:(?:(?::[0-9a-f]{1,4}){1,3})|(?:(?::[0-9a-f]{1,4})?:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){3}(?:(?:(?::[0-9a-f]{1,4}){1,4})|(?:(?::[0-9a-f]{1,4}){0,2}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){2}(?:(?:(?::[0-9a-f]{1,4}){1,5})|(?:(?::[0-9a-f]{1,4}){0,3}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){1}(?:(?:(?::[0-9a-f]{1,4}){1,6})|(?:(?::[0-9a-f]{1,4}){0,4}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?::(?:(?:(?::[0-9a-f]{1,4}){1,7})|(?:(?::[0-9a-f]{1,4}){0,5}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(?:%.+)?\s*$/i,regex:E,uuid:u,"json-pointer":g,"json-pointer-uri-fragment":h,"relative-json-pointer":d};var C=/t|\s/i;var m=/\/|:/;var y=/[^\\]\\Z/;function E(t){if(y.test(t))return!1;try{return new RegExp(t),!0}catch(t){return!1}}},function(t,e,n){"use strict";var r=n(873),i=n(137).toHash;t.exports=function(){var t=[{type:"number",rules:[{maximum:["exclusiveMaximum"]},{minimum:["exclusiveMinimum"]},"multipleOf","format"]},{type:"string",rules:["maxLength","minLength","pattern","format"]},{type:"array",rules:["maxItems","minItems","items","contains","uniqueItems"]},{type:"object",rules:["maxProperties","minProperties","required","dependencies","propertyNames",{properties:["additionalProperties","patternProperties"]}]},{rules:["$ref","const","enum","not","anyOf","oneOf","allOf","if"]}],e=["type","$comment"];return t.all=i(e),t.types=i(["number","integer","string","array","object","boolean","null"]),t.forEach(function(n){n.rules=n.rules.map(function(n){var i;if("object"==typeof n){var o=Object.keys(n)[0];i=n[o],n=o,i.forEach(function(n){e.push(n),t.all[n]=!0})}return e.push(n),t.all[n]={keyword:n,code:r[n],implements:i}}),t.all.$comment={keyword:"$comment",code:r.$comment},n.type&&(t.types[n.type]=n)}),t.keywords=i(e.concat(["$schema","$id","id","$data","$async","title","description","default","definitions","examples","readOnly","writeOnly","contentMediaType","contentEncoding","additionalItems","then","else"])),t.custom={},t}},function(t,e,n){"use strict";t.exports={$ref:n(874),allOf:n(875),anyOf:n(876),$comment:n(877),const:n(878),contains:n(879),dependencies:n(880),enum:n(881),format:n(882),if:n(883),items:n(884),maximum:n(435),minimum:n(435),maxItems:n(436),minItems:n(436),maxLength:n(437),minLength:n(437),maxProperties:n(438),minProperties:n(438),multipleOf:n(885),not:n(886),oneOf:n(887),pattern:n(888),properties:n(889),propertyNames:n(890),required:n(891),uniqueItems:n(892),validate:n(434)}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r,i,o=" ",a=t.level,s=t.dataLevel,A=t.schema[e],l=t.errSchemaPath+"/"+e,c=!t.opts.allErrors,u="data"+(s||""),g="valid"+a;if("#"==A||"#/"==A)t.isRoot?(r=t.async,i="validate"):(r=!0===t.root.schema.$async,i="root.refVal[0]");else{var h=t.resolveRef(t.baseId,A,t.isRoot);if(void 0===h){var d=t.MissingRefError.message(t.baseId,A);if("fail"==t.opts.missingRefs){t.logger.error(d),(C=C||[]).push(o),o="",!1!==t.createErrors?(o+=" { keyword: '$ref' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(l)+" , params: { ref: '"+t.util.escapeQuotes(A)+"' } ",!1!==t.opts.messages&&(o+=" , message: 'can\\'t resolve reference "+t.util.escapeQuotes(A)+"' "),t.opts.verbose&&(o+=" , schema: "+t.util.toQuotedString(A)+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+u+" "),o+=" } "):o+=" {} ";var f=o;o=C.pop(),!t.compositeRule&&c?t.async?o+=" throw new ValidationError(["+f+"]); ":o+=" validate.errors = ["+f+"]; return false; ":o+=" var err = "+f+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",c&&(o+=" if (false) { ")}else{if("ignore"!=t.opts.missingRefs)throw new t.MissingRefError(t.baseId,A,d);t.logger.warn(d),c&&(o+=" if (true) { ")}}else if(h.inline){var p=t.util.copy(t);p.level++;var I="valid"+p.level;p.schema=h.schema,p.schemaPath="",p.errSchemaPath=A,o+=" "+t.validate(p).replace(/validate\.schema/g,h.code)+" ",c&&(o+=" if ("+I+") { ")}else r=!0===h.$async||t.async&&!1!==h.$async,i=h.code}if(i){var C;(C=C||[]).push(o),o="",t.opts.passContext?o+=" "+i+".call(this, ":o+=" "+i+"( ",o+=" "+u+", (dataPath || '')",'""'!=t.errorPath&&(o+=" + "+t.errorPath);var m=o+=" , "+(s?"data"+(s-1||""):"parentData")+" , "+(s?t.dataPathArr[s]:"parentDataProperty")+", rootData)  ";if(o=C.pop(),r){if(!t.async)throw new Error("async schema referenced by sync schema");c&&(o+=" var "+g+"; "),o+=" try { await "+m+"; ",c&&(o+=" "+g+" = true; "),o+=" } catch (e) { if (!(e instanceof ValidationError)) throw e; if (vErrors === null) vErrors = e.errors; else vErrors = vErrors.concat(e.errors); errors = vErrors.length; ",c&&(o+=" "+g+" = false; "),o+=" } ",c&&(o+=" if ("+g+") { ")}else o+=" if (!"+m+") { if (vErrors === null) vErrors = "+i+".errors; else vErrors = vErrors.concat("+i+".errors); errors = vErrors.length; } ",c&&(o+=" else { ")}return o}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.schema[e],o=t.schemaPath+t.util.getProperty(e),a=t.errSchemaPath+"/"+e,s=!t.opts.allErrors,A=t.util.copy(t),l="";A.level++;var c="valid"+A.level,u=A.baseId,g=!0,h=i;if(h)for(var d,f=-1,p=h.length-1;f<p;)d=h[f+=1],t.util.schemaHasRules(d,t.RULES.all)&&(g=!1,A.schema=d,A.schemaPath=o+"["+f+"]",A.errSchemaPath=a+"/"+f,r+="  "+t.validate(A)+" ",A.baseId=u,s&&(r+=" if ("+c+") { ",l+="}"));return s&&(r+=g?" if (true) { ":" "+l.slice(0,-1)+" "),r=t.util.cleanUpCode(r)}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||""),u="valid"+i,g="errs__"+i,h=t.util.copy(t),d="";h.level++;var f="valid"+h.level;if(a.every(function(e){return t.util.schemaHasRules(e,t.RULES.all)})){var p=h.baseId;r+=" var "+g+" = errors; var "+u+" = false;  ";var I=t.compositeRule;t.compositeRule=h.compositeRule=!0;var C=a;if(C)for(var m,y=-1,E=C.length-1;y<E;)m=C[y+=1],h.schema=m,h.schemaPath=s+"["+y+"]",h.errSchemaPath=A+"/"+y,r+="  "+t.validate(h)+" ",h.baseId=p,r+=" "+u+" = "+u+" || "+f+"; if (!"+u+") { ",d+="}";t.compositeRule=h.compositeRule=I,r+=" "+d+" if (!"+u+") {   var err =   ",!1!==t.createErrors?(r+=" { keyword: 'anyOf' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: {} ",!1!==t.opts.messages&&(r+=" , message: 'should match some schema in anyOf' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",!t.compositeRule&&l&&(t.async?r+=" throw new ValidationError(vErrors); ":r+=" validate.errors = vErrors; return false; "),r+=" } else {  errors = "+g+"; if (vErrors !== null) { if ("+g+") vErrors.length = "+g+"; else vErrors = null; } ",t.opts.allErrors&&(r+=" } "),r=t.util.cleanUpCode(r)}else l&&(r+=" if (true) { ");return r}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.schema[e],o=t.errSchemaPath+"/"+e,a=(t.opts.allErrors,t.util.toQuotedString(i));return!0===t.opts.$comment?r+=" console.log("+a+");":"function"==typeof t.opts.$comment&&(r+=" self._opts.$comment("+a+", "+t.util.toQuotedString(o)+", validate.root.schema);"),r}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||""),u="valid"+i,g=t.opts.$data&&a&&a.$data;g&&(r+=" var schema"+i+" = "+t.util.getData(a.$data,o,t.dataPathArr)+"; "),g||(r+=" var schema"+i+" = validate.schema"+s+";"),r+="var "+u+" = equal("+c+", schema"+i+"); if (!"+u+") {   ";var h=h||[];h.push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'const' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { allowedValue: schema"+i+" } ",!1!==t.opts.messages&&(r+=" , message: 'should be equal to constant' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var d=r;return r=h.pop(),!t.compositeRule&&l?t.async?r+=" throw new ValidationError(["+d+"]); ":r+=" validate.errors = ["+d+"]; return false; ":r+=" var err = "+d+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" }",l&&(r+=" else { "),r}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||""),u="valid"+i,g="errs__"+i,h=t.util.copy(t);h.level++;var d="valid"+h.level,f="i"+i,p=h.dataLevel=t.dataLevel+1,I="data"+p,C=t.baseId,m=t.util.schemaHasRules(a,t.RULES.all);if(r+="var "+g+" = errors;var "+u+";",m){var y=t.compositeRule;t.compositeRule=h.compositeRule=!0,h.schema=a,h.schemaPath=s,h.errSchemaPath=A,r+=" var "+d+" = false; for (var "+f+" = 0; "+f+" < "+c+".length; "+f+"++) { ",h.errorPath=t.util.getPathExpr(t.errorPath,f,t.opts.jsonPointers,!0);var E=c+"["+f+"]";h.dataPathArr[p]=f;var v=t.validate(h);h.baseId=C,t.util.varOccurences(v,I)<2?r+=" "+t.util.varReplace(v,I,E)+" ":r+=" var "+I+" = "+E+"; "+v+" ",r+=" if ("+d+") break; }  ",t.compositeRule=h.compositeRule=y,r+="  if (!"+d+") {"}else r+=" if ("+c+".length == 0) {";var B=B||[];B.push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'contains' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: {} ",!1!==t.opts.messages&&(r+=" , message: 'should contain a valid item' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var b=r;return r=B.pop(),!t.compositeRule&&l?t.async?r+=" throw new ValidationError(["+b+"]); ":r+=" validate.errors = ["+b+"]; return false; ":r+=" var err = "+b+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } else { ",m&&(r+="  errors = "+g+"; if (vErrors !== null) { if ("+g+") vErrors.length = "+g+"; else vErrors = null; } "),t.opts.allErrors&&(r+=" } "),r=t.util.cleanUpCode(r)}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||""),u="errs__"+i,g=t.util.copy(t),h="";g.level++;var d="valid"+g.level,f={},p={},I=t.opts.ownProperties;for(E in a){var C=a[E],m=Array.isArray(C)?p:f;m[E]=C}r+="var "+u+" = errors;";var y=t.errorPath;for(var E in r+="var missing"+i+";",p)if((m=p[E]).length){if(r+=" if ( "+c+t.util.getProperty(E)+" !== undefined ",I&&(r+=" && Object.prototype.hasOwnProperty.call("+c+", '"+t.util.escapeQuotes(E)+"') "),l){r+=" && ( ";var v=m;if(v)for(var B=-1,b=v.length-1;B<b;){M=v[B+=1],B&&(r+=" || "),r+=" ( ( "+(R=c+(L=t.util.getProperty(M)))+" === undefined ",I&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", '"+t.util.escapeQuotes(M)+"') "),r+=") && (missing"+i+" = "+t.util.toQuotedString(t.opts.jsonPointers?M:L)+") ) "}r+=")) {  ";var Q="missing"+i,w="' + "+Q+" + '";t.opts._errorDataPathProperty&&(t.errorPath=t.opts.jsonPointers?t.util.getPathExpr(y,Q,!0):y+" + "+Q);var x=x||[];x.push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'dependencies' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { property: '"+t.util.escapeQuotes(E)+"', missingProperty: '"+w+"', depsCount: "+m.length+", deps: '"+t.util.escapeQuotes(1==m.length?m[0]:m.join(", "))+"' } ",!1!==t.opts.messages&&(r+=" , message: 'should have ",1==m.length?r+="property "+t.util.escapeQuotes(m[0]):r+="properties "+t.util.escapeQuotes(m.join(", ")),r+=" when property "+t.util.escapeQuotes(E)+" is present' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var _=r;r=x.pop(),!t.compositeRule&&l?t.async?r+=" throw new ValidationError(["+_+"]); ":r+=" validate.errors = ["+_+"]; return false; ":r+=" var err = "+_+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; "}else{r+=" ) { ";var D=m;if(D)for(var M,k=-1,S=D.length-1;k<S;){M=D[k+=1];var L=t.util.getProperty(M),R=(w=t.util.escapeQuotes(M),c+L);t.opts._errorDataPathProperty&&(t.errorPath=t.util.getPath(y,M,t.opts.jsonPointers)),r+=" if ( "+R+" === undefined ",I&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", '"+t.util.escapeQuotes(M)+"') "),r+=") {  var err =   ",!1!==t.createErrors?(r+=" { keyword: 'dependencies' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { property: '"+t.util.escapeQuotes(E)+"', missingProperty: '"+w+"', depsCount: "+m.length+", deps: '"+t.util.escapeQuotes(1==m.length?m[0]:m.join(", "))+"' } ",!1!==t.opts.messages&&(r+=" , message: 'should have ",1==m.length?r+="property "+t.util.escapeQuotes(m[0]):r+="properties "+t.util.escapeQuotes(m.join(", ")),r+=" when property "+t.util.escapeQuotes(E)+" is present' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; } "}}r+=" }   ",l&&(h+="}",r+=" else { ")}t.errorPath=y;var N=g.baseId;for(var E in f){C=f[E];t.util.schemaHasRules(C,t.RULES.all)&&(r+=" "+d+" = true; if ( "+c+t.util.getProperty(E)+" !== undefined ",I&&(r+=" && Object.prototype.hasOwnProperty.call("+c+", '"+t.util.escapeQuotes(E)+"') "),r+=") { ",g.schema=C,g.schemaPath=s+t.util.getProperty(E),g.errSchemaPath=A+"/"+t.util.escapeFragment(E),r+="  "+t.validate(g)+" ",g.baseId=N,r+=" }  ",l&&(r+=" if ("+d+") { ",h+="}"))}return l&&(r+="   "+h+" if ("+u+" == errors) {"),r=t.util.cleanUpCode(r)}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||""),u="valid"+i,g=t.opts.$data&&a&&a.$data;g&&(r+=" var schema"+i+" = "+t.util.getData(a.$data,o,t.dataPathArr)+"; ");var h="i"+i,d="schema"+i;g||(r+=" var "+d+" = validate.schema"+s+";"),r+="var "+u+";",g&&(r+=" if (schema"+i+" === undefined) "+u+" = true; else if (!Array.isArray(schema"+i+")) "+u+" = false; else {"),r+=u+" = false;for (var "+h+"=0; "+h+"<"+d+".length; "+h+"++) if (equal("+c+", "+d+"["+h+"])) { "+u+" = true; break; }",g&&(r+="  }  "),r+=" if (!"+u+") {   ";var f=f||[];f.push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'enum' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { allowedValues: schema"+i+" } ",!1!==t.opts.messages&&(r+=" , message: 'should be equal to one of the allowed values' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var p=r;return r=f.pop(),!t.compositeRule&&l?t.async?r+=" throw new ValidationError(["+p+"]); ":r+=" validate.errors = ["+p+"]; return false; ":r+=" var err = "+p+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" }",l&&(r+=" else { "),r}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||"");if(!1===t.opts.format)return l&&(r+=" if (true) { "),r;var u,g=t.opts.$data&&a&&a.$data;g?(r+=" var schema"+i+" = "+t.util.getData(a.$data,o,t.dataPathArr)+"; ",u="schema"+i):u=a;var h=t.opts.unknownFormats,d=Array.isArray(h);if(g){r+=" var "+(f="format"+i)+" = formats["+u+"]; var "+(p="isObject"+i)+" = typeof "+f+" == 'object' && !("+f+" instanceof RegExp) && "+f+".validate; var "+(I="formatType"+i)+" = "+p+" && "+f+".type || 'string'; if ("+p+") { ",t.async&&(r+=" var async"+i+" = "+f+".async; "),r+=" "+f+" = "+f+".validate; } if (  ",g&&(r+=" ("+u+" !== undefined && typeof "+u+" != 'string') || "),r+=" (","ignore"!=h&&(r+=" ("+u+" && !"+f+" ",d&&(r+=" && self._opts.unknownFormats.indexOf("+u+") == -1 "),r+=") || "),r+=" ("+f+" && "+I+" == '"+n+"' && !(typeof "+f+" == 'function' ? ",t.async?r+=" (async"+i+" ? await "+f+"("+c+") : "+f+"("+c+")) ":r+=" "+f+"("+c+") ",r+=" : "+f+".test("+c+"))))) {"}else{var f;if(!(f=t.formats[a])){if("ignore"==h)return t.logger.warn('unknown format "'+a+'" ignored in schema at path "'+t.errSchemaPath+'"'),l&&(r+=" if (true) { "),r;if(d&&h.indexOf(a)>=0)return l&&(r+=" if (true) { "),r;throw new Error('unknown format "'+a+'" is used in schema at path "'+t.errSchemaPath+'"')}var p,I=(p="object"==typeof f&&!(f instanceof RegExp)&&f.validate)&&f.type||"string";if(p){var C=!0===f.async;f=f.validate}if(I!=n)return l&&(r+=" if (true) { "),r;if(C){if(!t.async)throw new Error("async format in sync schema");r+=" if (!(await "+(m="formats"+t.util.getProperty(a)+".validate")+"("+c+"))) { "}else{r+=" if (! ";var m="formats"+t.util.getProperty(a);p&&(m+=".validate"),r+="function"==typeof f?" "+m+"("+c+") ":" "+m+".test("+c+") ",r+=") { "}}var y=y||[];y.push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'format' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { format:  ",r+=g?""+u:""+t.util.toQuotedString(a),r+="  } ",!1!==t.opts.messages&&(r+=" , message: 'should match format \"",r+=g?"' + "+u+" + '":""+t.util.escapeQuotes(a),r+="\"' "),t.opts.verbose&&(r+=" , schema:  ",r+=g?"validate.schema"+s:""+t.util.toQuotedString(a),r+="         , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var E=r;return r=y.pop(),!t.compositeRule&&l?t.async?r+=" throw new ValidationError(["+E+"]); ":r+=" validate.errors = ["+E+"]; return false; ":r+=" var err = "+E+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } ",l&&(r+=" else { "),r}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||""),u="valid"+i,g="errs__"+i,h=t.util.copy(t);h.level++;var d="valid"+h.level,f=t.schema.then,p=t.schema.else,I=void 0!==f&&t.util.schemaHasRules(f,t.RULES.all),C=void 0!==p&&t.util.schemaHasRules(p,t.RULES.all),m=h.baseId;if(I||C){var y;h.createErrors=!1,h.schema=a,h.schemaPath=s,h.errSchemaPath=A,r+=" var "+g+" = errors; var "+u+" = true;  ";var E=t.compositeRule;t.compositeRule=h.compositeRule=!0,r+="  "+t.validate(h)+" ",h.baseId=m,h.createErrors=!0,r+="  errors = "+g+"; if (vErrors !== null) { if ("+g+") vErrors.length = "+g+"; else vErrors = null; }  ",t.compositeRule=h.compositeRule=E,I?(r+=" if ("+d+") {  ",h.schema=t.schema.then,h.schemaPath=t.schemaPath+".then",h.errSchemaPath=t.errSchemaPath+"/then",r+="  "+t.validate(h)+" ",h.baseId=m,r+=" "+u+" = "+d+"; ",I&&C?r+=" var "+(y="ifClause"+i)+" = 'then'; ":y="'then'",r+=" } ",C&&(r+=" else { ")):r+=" if (!"+d+") { ",C&&(h.schema=t.schema.else,h.schemaPath=t.schemaPath+".else",h.errSchemaPath=t.errSchemaPath+"/else",r+="  "+t.validate(h)+" ",h.baseId=m,r+=" "+u+" = "+d+"; ",I&&C?r+=" var "+(y="ifClause"+i)+" = 'else'; ":y="'else'",r+=" } "),r+=" if (!"+u+") {   var err =   ",!1!==t.createErrors?(r+=" { keyword: 'if' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { failingKeyword: "+y+" } ",!1!==t.opts.messages&&(r+=" , message: 'should match \"' + "+y+" + '\" schema' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",!t.compositeRule&&l&&(t.async?r+=" throw new ValidationError(vErrors); ":r+=" validate.errors = vErrors; return false; "),r+=" }   ",l&&(r+=" else { "),r=t.util.cleanUpCode(r)}else l&&(r+=" if (true) { ");return r}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||""),u="valid"+i,g="errs__"+i,h=t.util.copy(t),d="";h.level++;var f="valid"+h.level,p="i"+i,I=h.dataLevel=t.dataLevel+1,C="data"+I,m=t.baseId;if(r+="var "+g+" = errors;var "+u+";",Array.isArray(a)){var y=t.schema.additionalItems;if(!1===y){r+=" "+u+" = "+c+".length <= "+a.length+"; ";var E=A;A=t.errSchemaPath+"/additionalItems",r+="  if (!"+u+") {   ";var v=v||[];v.push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'additionalItems' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { limit: "+a.length+" } ",!1!==t.opts.messages&&(r+=" , message: 'should NOT have more than "+a.length+" items' "),t.opts.verbose&&(r+=" , schema: false , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var B=r;r=v.pop(),!t.compositeRule&&l?t.async?r+=" throw new ValidationError(["+B+"]); ":r+=" validate.errors = ["+B+"]; return false; ":r+=" var err = "+B+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } ",A=E,l&&(d+="}",r+=" else { ")}var b=a;if(b)for(var Q,w=-1,x=b.length-1;w<x;)if(Q=b[w+=1],t.util.schemaHasRules(Q,t.RULES.all)){r+=" "+f+" = true; if ("+c+".length > "+w+") { ";var _=c+"["+w+"]";h.schema=Q,h.schemaPath=s+"["+w+"]",h.errSchemaPath=A+"/"+w,h.errorPath=t.util.getPathExpr(t.errorPath,w,t.opts.jsonPointers,!0),h.dataPathArr[I]=w;var D=t.validate(h);h.baseId=m,t.util.varOccurences(D,C)<2?r+=" "+t.util.varReplace(D,C,_)+" ":r+=" var "+C+" = "+_+"; "+D+" ",r+=" }  ",l&&(r+=" if ("+f+") { ",d+="}")}if("object"==typeof y&&t.util.schemaHasRules(y,t.RULES.all)){h.schema=y,h.schemaPath=t.schemaPath+".additionalItems",h.errSchemaPath=t.errSchemaPath+"/additionalItems",r+=" "+f+" = true; if ("+c+".length > "+a.length+") {  for (var "+p+" = "+a.length+"; "+p+" < "+c+".length; "+p+"++) { ",h.errorPath=t.util.getPathExpr(t.errorPath,p,t.opts.jsonPointers,!0);_=c+"["+p+"]";h.dataPathArr[I]=p;D=t.validate(h);h.baseId=m,t.util.varOccurences(D,C)<2?r+=" "+t.util.varReplace(D,C,_)+" ":r+=" var "+C+" = "+_+"; "+D+" ",l&&(r+=" if (!"+f+") break; "),r+=" } }  ",l&&(r+=" if ("+f+") { ",d+="}")}}else if(t.util.schemaHasRules(a,t.RULES.all)){h.schema=a,h.schemaPath=s,h.errSchemaPath=A,r+="  for (var "+p+" = 0; "+p+" < "+c+".length; "+p+"++) { ",h.errorPath=t.util.getPathExpr(t.errorPath,p,t.opts.jsonPointers,!0);_=c+"["+p+"]";h.dataPathArr[I]=p;D=t.validate(h);h.baseId=m,t.util.varOccurences(D,C)<2?r+=" "+t.util.varReplace(D,C,_)+" ":r+=" var "+C+" = "+_+"; "+D+" ",l&&(r+=" if (!"+f+") break; "),r+=" }"}return l&&(r+=" "+d+" if ("+g+" == errors) {"),r=t.util.cleanUpCode(r)}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r,i=" ",o=t.level,a=t.dataLevel,s=t.schema[e],A=t.schemaPath+t.util.getProperty(e),l=t.errSchemaPath+"/"+e,c=!t.opts.allErrors,u="data"+(a||""),g=t.opts.$data&&s&&s.$data;g?(i+=" var schema"+o+" = "+t.util.getData(s.$data,a,t.dataPathArr)+"; ",r="schema"+o):r=s,i+="var division"+o+";if (",g&&(i+=" "+r+" !== undefined && ( typeof "+r+" != 'number' || "),i+=" (division"+o+" = "+u+" / "+r+", ",t.opts.multipleOfPrecision?i+=" Math.abs(Math.round(division"+o+") - division"+o+") > 1e-"+t.opts.multipleOfPrecision+" ":i+=" division"+o+" !== parseInt(division"+o+") ",i+=" ) ",g&&(i+="  )  "),i+=" ) {   ";var h=h||[];h.push(i),i="",!1!==t.createErrors?(i+=" { keyword: 'multipleOf' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(l)+" , params: { multipleOf: "+r+" } ",!1!==t.opts.messages&&(i+=" , message: 'should be multiple of ",i+=g?"' + "+r:r+"'"),t.opts.verbose&&(i+=" , schema:  ",i+=g?"validate.schema"+A:""+s,i+="         , parentSchema: validate.schema"+t.schemaPath+" , data: "+u+" "),i+=" } "):i+=" {} ";var d=i;return i=h.pop(),!t.compositeRule&&c?t.async?i+=" throw new ValidationError(["+d+"]); ":i+=" validate.errors = ["+d+"]; return false; ":i+=" var err = "+d+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+="} ",c&&(i+=" else { "),i}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||""),u="errs__"+i,g=t.util.copy(t);g.level++;var h="valid"+g.level;if(t.util.schemaHasRules(a,t.RULES.all)){g.schema=a,g.schemaPath=s,g.errSchemaPath=A,r+=" var "+u+" = errors;  ";var d,f=t.compositeRule;t.compositeRule=g.compositeRule=!0,g.createErrors=!1,g.opts.allErrors&&(d=g.opts.allErrors,g.opts.allErrors=!1),r+=" "+t.validate(g)+" ",g.createErrors=!0,d&&(g.opts.allErrors=d),t.compositeRule=g.compositeRule=f,r+=" if ("+h+") {   ";var p=p||[];p.push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'not' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: {} ",!1!==t.opts.messages&&(r+=" , message: 'should NOT be valid' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var I=r;r=p.pop(),!t.compositeRule&&l?t.async?r+=" throw new ValidationError(["+I+"]); ":r+=" validate.errors = ["+I+"]; return false; ":r+=" var err = "+I+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } else {  errors = "+u+"; if (vErrors !== null) { if ("+u+") vErrors.length = "+u+"; else vErrors = null; } ",t.opts.allErrors&&(r+=" } ")}else r+="  var err =   ",!1!==t.createErrors?(r+=" { keyword: 'not' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: {} ",!1!==t.opts.messages&&(r+=" , message: 'should NOT be valid' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",l&&(r+=" if (false) { ");return r}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||""),u="valid"+i,g="errs__"+i,h=t.util.copy(t),d="";h.level++;var f="valid"+h.level,p=h.baseId,I="prevValid"+i,C="passingSchemas"+i;r+="var "+g+" = errors , "+I+" = false , "+u+" = false , "+C+" = null; ";var m=t.compositeRule;t.compositeRule=h.compositeRule=!0;var y=a;if(y)for(var E,v=-1,B=y.length-1;v<B;)E=y[v+=1],t.util.schemaHasRules(E,t.RULES.all)?(h.schema=E,h.schemaPath=s+"["+v+"]",h.errSchemaPath=A+"/"+v,r+="  "+t.validate(h)+" ",h.baseId=p):r+=" var "+f+" = true; ",v&&(r+=" if ("+f+" && "+I+") { "+u+" = false; "+C+" = ["+C+", "+v+"]; } else { ",d+="}"),r+=" if ("+f+") { "+u+" = "+I+" = true; "+C+" = "+v+"; }";return t.compositeRule=h.compositeRule=m,r+=d+"if (!"+u+") {   var err =   ",!1!==t.createErrors?(r+=" { keyword: 'oneOf' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { passingSchemas: "+C+" } ",!1!==t.opts.messages&&(r+=" , message: 'should match exactly one schema in oneOf' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",!t.compositeRule&&l&&(t.async?r+=" throw new ValidationError(vErrors); ":r+=" validate.errors = vErrors; return false; "),r+="} else {  errors = "+g+"; if (vErrors !== null) { if ("+g+") vErrors.length = "+g+"; else vErrors = null; }",t.opts.allErrors&&(r+=" } "),r}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r,i=" ",o=t.level,a=t.dataLevel,s=t.schema[e],A=t.schemaPath+t.util.getProperty(e),l=t.errSchemaPath+"/"+e,c=!t.opts.allErrors,u="data"+(a||""),g=t.opts.$data&&s&&s.$data;g?(i+=" var schema"+o+" = "+t.util.getData(s.$data,a,t.dataPathArr)+"; ",r="schema"+o):r=s,i+="if ( ",g&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'string') || "),i+=" !"+(g?"(new RegExp("+r+"))":t.usePattern(s))+".test("+u+") ) {   ";var h=h||[];h.push(i),i="",!1!==t.createErrors?(i+=" { keyword: 'pattern' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(l)+" , params: { pattern:  ",i+=g?""+r:""+t.util.toQuotedString(s),i+="  } ",!1!==t.opts.messages&&(i+=" , message: 'should match pattern \"",i+=g?"' + "+r+" + '":""+t.util.escapeQuotes(s),i+="\"' "),t.opts.verbose&&(i+=" , schema:  ",i+=g?"validate.schema"+A:""+t.util.toQuotedString(s),i+="         , parentSchema: validate.schema"+t.schemaPath+" , data: "+u+" "),i+=" } "):i+=" {} ";var d=i;return i=h.pop(),!t.compositeRule&&c?t.async?i+=" throw new ValidationError(["+d+"]); ":i+=" validate.errors = ["+d+"]; return false; ":i+=" var err = "+d+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+="} ",c&&(i+=" else { "),i}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||""),u="errs__"+i,g=t.util.copy(t),h="";g.level++;var d="valid"+g.level,f="key"+i,p="idx"+i,I=g.dataLevel=t.dataLevel+1,C="data"+I,m="dataProperties"+i,y=Object.keys(a||{}),E=t.schema.patternProperties||{},v=Object.keys(E),B=t.schema.additionalProperties,b=y.length||v.length,Q=!1===B,w="object"==typeof B&&Object.keys(B).length,x=t.opts.removeAdditional,_=Q||w||x,D=t.opts.ownProperties,M=t.baseId,k=t.schema.required;if(k&&(!t.opts.$data||!k.$data)&&k.length<t.opts.loopRequired)var S=t.util.toHash(k);if(r+="var "+u+" = errors;var "+d+" = true;",D&&(r+=" var "+m+" = undefined;"),_){if(r+=D?" "+m+" = "+m+" || Object.keys("+c+"); for (var "+p+"=0; "+p+"<"+m+".length; "+p+"++) { var "+f+" = "+m+"["+p+"]; ":" for (var "+f+" in "+c+") { ",b){if(r+=" var isAdditional"+i+" = !(false ",y.length)if(y.length>8)r+=" || validate.schema"+s+".hasOwnProperty("+f+") ";else{var L=y;if(L)for(var R=-1,N=L.length-1;R<N;)q=L[R+=1],r+=" || "+f+" == "+t.util.toQuotedString(q)+" "}if(v.length){var F=v;if(F)for(var T=-1,U=F.length-1;T<U;)it=F[T+=1],r+=" || "+t.usePattern(it)+".test("+f+") "}r+=" ); if (isAdditional"+i+") { "}if("all"==x)r+=" delete "+c+"["+f+"]; ";else{var O=t.errorPath,G="' + "+f+" + '";if(t.opts._errorDataPathProperty&&(t.errorPath=t.util.getPathExpr(t.errorPath,f,t.opts.jsonPointers)),Q)if(x)r+=" delete "+c+"["+f+"]; ";else{r+=" "+d+" = false; ";var H=A;A=t.errSchemaPath+"/additionalProperties",(et=et||[]).push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'additionalProperties' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { additionalProperty: '"+G+"' } ",!1!==t.opts.messages&&(r+=" , message: '",t.opts._errorDataPathProperty?r+="is an invalid additional property":r+="should NOT have additional properties",r+="' "),t.opts.verbose&&(r+=" , schema: false , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var j=r;r=et.pop(),!t.compositeRule&&l?t.async?r+=" throw new ValidationError(["+j+"]); ":r+=" validate.errors = ["+j+"]; return false; ":r+=" var err = "+j+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",A=H,l&&(r+=" break; ")}else if(w)if("failing"==x){r+=" var "+u+" = errors;  ";var Y=t.compositeRule;t.compositeRule=g.compositeRule=!0,g.schema=B,g.schemaPath=t.schemaPath+".additionalProperties",g.errSchemaPath=t.errSchemaPath+"/additionalProperties",g.errorPath=t.opts._errorDataPathProperty?t.errorPath:t.util.getPathExpr(t.errorPath,f,t.opts.jsonPointers);var P=c+"["+f+"]";g.dataPathArr[I]=f;var K=t.validate(g);g.baseId=M,t.util.varOccurences(K,C)<2?r+=" "+t.util.varReplace(K,C,P)+" ":r+=" var "+C+" = "+P+"; "+K+" ",r+=" if (!"+d+") { errors = "+u+"; if (validate.errors !== null) { if (errors) validate.errors.length = errors; else validate.errors = null; } delete "+c+"["+f+"]; }  ",t.compositeRule=g.compositeRule=Y}else{g.schema=B,g.schemaPath=t.schemaPath+".additionalProperties",g.errSchemaPath=t.errSchemaPath+"/additionalProperties",g.errorPath=t.opts._errorDataPathProperty?t.errorPath:t.util.getPathExpr(t.errorPath,f,t.opts.jsonPointers);P=c+"["+f+"]";g.dataPathArr[I]=f;K=t.validate(g);g.baseId=M,t.util.varOccurences(K,C)<2?r+=" "+t.util.varReplace(K,C,P)+" ":r+=" var "+C+" = "+P+"; "+K+" ",l&&(r+=" if (!"+d+") break; ")}t.errorPath=O}b&&(r+=" } "),r+=" }  ",l&&(r+=" if ("+d+") { ",h+="}")}var z=t.opts.useDefaults&&!t.compositeRule;if(y.length){var J=y;if(J)for(var q,V=-1,W=J.length-1;V<W;){var X=a[q=J[V+=1]];if(t.util.schemaHasRules(X,t.RULES.all)){var Z=t.util.getProperty(q),$=(P=c+Z,z&&void 0!==X.default);g.schema=X,g.schemaPath=s+Z,g.errSchemaPath=A+"/"+t.util.escapeFragment(q),g.errorPath=t.util.getPath(t.errorPath,q,t.opts.jsonPointers),g.dataPathArr[I]=t.util.toQuotedString(q);K=t.validate(g);if(g.baseId=M,t.util.varOccurences(K,C)<2){K=t.util.varReplace(K,C,P);var tt=P}else{tt=C;r+=" var "+C+" = "+P+"; "}if($)r+=" "+K+" ";else{if(S&&S[q]){r+=" if ( "+tt+" === undefined ",D&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", '"+t.util.escapeQuotes(q)+"') "),r+=") { "+d+" = false; ";O=t.errorPath,H=A;var et,nt=t.util.escapeQuotes(q);t.opts._errorDataPathProperty&&(t.errorPath=t.util.getPath(O,q,t.opts.jsonPointers)),A=t.errSchemaPath+"/required",(et=et||[]).push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'required' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { missingProperty: '"+nt+"' } ",!1!==t.opts.messages&&(r+=" , message: '",t.opts._errorDataPathProperty?r+="is a required property":r+="should have required property \\'"+nt+"\\'",r+="' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";j=r;r=et.pop(),!t.compositeRule&&l?t.async?r+=" throw new ValidationError(["+j+"]); ":r+=" validate.errors = ["+j+"]; return false; ":r+=" var err = "+j+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",A=H,t.errorPath=O,r+=" } else { "}else l?(r+=" if ( "+tt+" === undefined ",D&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", '"+t.util.escapeQuotes(q)+"') "),r+=") { "+d+" = true; } else { "):(r+=" if ("+tt+" !== undefined ",D&&(r+=" &&   Object.prototype.hasOwnProperty.call("+c+", '"+t.util.escapeQuotes(q)+"') "),r+=" ) { ");r+=" "+K+" } "}}l&&(r+=" if ("+d+") { ",h+="}")}}if(v.length){var rt=v;if(rt)for(var it,ot=-1,at=rt.length-1;ot<at;){X=E[it=rt[ot+=1]];if(t.util.schemaHasRules(X,t.RULES.all)){g.schema=X,g.schemaPath=t.schemaPath+".patternProperties"+t.util.getProperty(it),g.errSchemaPath=t.errSchemaPath+"/patternProperties/"+t.util.escapeFragment(it),r+=D?" "+m+" = "+m+" || Object.keys("+c+"); for (var "+p+"=0; "+p+"<"+m+".length; "+p+"++) { var "+f+" = "+m+"["+p+"]; ":" for (var "+f+" in "+c+") { ",r+=" if ("+t.usePattern(it)+".test("+f+")) { ",g.errorPath=t.util.getPathExpr(t.errorPath,f,t.opts.jsonPointers);P=c+"["+f+"]";g.dataPathArr[I]=f;K=t.validate(g);g.baseId=M,t.util.varOccurences(K,C)<2?r+=" "+t.util.varReplace(K,C,P)+" ":r+=" var "+C+" = "+P+"; "+K+" ",l&&(r+=" if (!"+d+") break; "),r+=" } ",l&&(r+=" else "+d+" = true; "),r+=" }  ",l&&(r+=" if ("+d+") { ",h+="}")}}}return l&&(r+=" "+h+" if ("+u+" == errors) {"),r=t.util.cleanUpCode(r)}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||""),u="errs__"+i,g=t.util.copy(t);g.level++;var h="valid"+g.level;if(r+="var "+u+" = errors;",t.util.schemaHasRules(a,t.RULES.all)){g.schema=a,g.schemaPath=s,g.errSchemaPath=A;var d="key"+i,f="idx"+i,p="i"+i,I="' + "+d+" + '",C="data"+(g.dataLevel=t.dataLevel+1),m="dataProperties"+i,y=t.opts.ownProperties,E=t.baseId;y&&(r+=" var "+m+" = undefined; "),r+=y?" "+m+" = "+m+" || Object.keys("+c+"); for (var "+f+"=0; "+f+"<"+m+".length; "+f+"++) { var "+d+" = "+m+"["+f+"]; ":" for (var "+d+" in "+c+") { ",r+=" var startErrs"+i+" = errors; ";var v=d,B=t.compositeRule;t.compositeRule=g.compositeRule=!0;var b=t.validate(g);g.baseId=E,t.util.varOccurences(b,C)<2?r+=" "+t.util.varReplace(b,C,v)+" ":r+=" var "+C+" = "+v+"; "+b+" ",t.compositeRule=g.compositeRule=B,r+=" if (!"+h+") { for (var "+p+"=startErrs"+i+"; "+p+"<errors; "+p+"++) { vErrors["+p+"].propertyName = "+d+"; }   var err =   ",!1!==t.createErrors?(r+=" { keyword: 'propertyNames' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { propertyName: '"+I+"' } ",!1!==t.opts.messages&&(r+=" , message: 'property name \\'"+I+"\\' is invalid' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",!t.compositeRule&&l&&(t.async?r+=" throw new ValidationError(vErrors); ":r+=" validate.errors = vErrors; return false; "),l&&(r+=" break; "),r+=" } }"}return l&&(r+="  if ("+u+" == errors) {"),r=t.util.cleanUpCode(r)}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r=" ",i=t.level,o=t.dataLevel,a=t.schema[e],s=t.schemaPath+t.util.getProperty(e),A=t.errSchemaPath+"/"+e,l=!t.opts.allErrors,c="data"+(o||""),u="valid"+i,g=t.opts.$data&&a&&a.$data;g&&(r+=" var schema"+i+" = "+t.util.getData(a.$data,o,t.dataPathArr)+"; ");var h="schema"+i;if(!g)if(a.length<t.opts.loopRequired&&t.schema.properties&&Object.keys(t.schema.properties).length){var d=[],f=a;if(f)for(var p,I=-1,C=f.length-1;I<C;){p=f[I+=1];var m=t.schema.properties[p];m&&t.util.schemaHasRules(m,t.RULES.all)||(d[d.length]=p)}}else d=a;if(g||d.length){var y=t.errorPath,E=g||d.length>=t.opts.loopRequired,v=t.opts.ownProperties;if(l)if(r+=" var missing"+i+"; ",E){g||(r+=" var "+h+" = validate.schema"+s+"; ");var B="' + "+(D="schema"+i+"["+(w="i"+i)+"]")+" + '";t.opts._errorDataPathProperty&&(t.errorPath=t.util.getPathExpr(y,D,t.opts.jsonPointers)),r+=" var "+u+" = true; ",g&&(r+=" if (schema"+i+" === undefined) "+u+" = true; else if (!Array.isArray(schema"+i+")) "+u+" = false; else {"),r+=" for (var "+w+" = 0; "+w+" < "+h+".length; "+w+"++) { "+u+" = "+c+"["+h+"["+w+"]] !== undefined ",v&&(r+=" &&   Object.prototype.hasOwnProperty.call("+c+", "+h+"["+w+"]) "),r+="; if (!"+u+") break; } ",g&&(r+="  }  "),r+="  if (!"+u+") {   ",(_=_||[]).push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'required' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { missingProperty: '"+B+"' } ",!1!==t.opts.messages&&(r+=" , message: '",t.opts._errorDataPathProperty?r+="is a required property":r+="should have required property \\'"+B+"\\'",r+="' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var b=r;r=_.pop(),!t.compositeRule&&l?t.async?r+=" throw new ValidationError(["+b+"]); ":r+=" validate.errors = ["+b+"]; return false; ":r+=" var err = "+b+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } else { "}else{r+=" if ( ";var Q=d;if(Q)for(var w=-1,x=Q.length-1;w<x;){k=Q[w+=1],w&&(r+=" || "),r+=" ( ( "+(N=c+(R=t.util.getProperty(k)))+" === undefined ",v&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", '"+t.util.escapeQuotes(k)+"') "),r+=") && (missing"+i+" = "+t.util.toQuotedString(t.opts.jsonPointers?k:R)+") ) "}r+=") {  ";var _;B="' + "+(D="missing"+i)+" + '";t.opts._errorDataPathProperty&&(t.errorPath=t.opts.jsonPointers?t.util.getPathExpr(y,D,!0):y+" + "+D),(_=_||[]).push(r),r="",!1!==t.createErrors?(r+=" { keyword: 'required' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { missingProperty: '"+B+"' } ",!1!==t.opts.messages&&(r+=" , message: '",t.opts._errorDataPathProperty?r+="is a required property":r+="should have required property \\'"+B+"\\'",r+="' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";b=r;r=_.pop(),!t.compositeRule&&l?t.async?r+=" throw new ValidationError(["+b+"]); ":r+=" validate.errors = ["+b+"]; return false; ":r+=" var err = "+b+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } else { "}else if(E){g||(r+=" var "+h+" = validate.schema"+s+"; ");var D;B="' + "+(D="schema"+i+"["+(w="i"+i)+"]")+" + '";t.opts._errorDataPathProperty&&(t.errorPath=t.util.getPathExpr(y,D,t.opts.jsonPointers)),g&&(r+=" if ("+h+" && !Array.isArray("+h+")) {  var err =   ",!1!==t.createErrors?(r+=" { keyword: 'required' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { missingProperty: '"+B+"' } ",!1!==t.opts.messages&&(r+=" , message: '",t.opts._errorDataPathProperty?r+="is a required property":r+="should have required property \\'"+B+"\\'",r+="' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; } else if ("+h+" !== undefined) { "),r+=" for (var "+w+" = 0; "+w+" < "+h+".length; "+w+"++) { if ("+c+"["+h+"["+w+"]] === undefined ",v&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", "+h+"["+w+"]) "),r+=") {  var err =   ",!1!==t.createErrors?(r+=" { keyword: 'required' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { missingProperty: '"+B+"' } ",!1!==t.opts.messages&&(r+=" , message: '",t.opts._errorDataPathProperty?r+="is a required property":r+="should have required property \\'"+B+"\\'",r+="' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; } } ",g&&(r+="  }  ")}else{var M=d;if(M)for(var k,S=-1,L=M.length-1;S<L;){k=M[S+=1];var R=t.util.getProperty(k),N=(B=t.util.escapeQuotes(k),c+R);t.opts._errorDataPathProperty&&(t.errorPath=t.util.getPath(y,k,t.opts.jsonPointers)),r+=" if ( "+N+" === undefined ",v&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", '"+t.util.escapeQuotes(k)+"') "),r+=") {  var err =   ",!1!==t.createErrors?(r+=" { keyword: 'required' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(A)+" , params: { missingProperty: '"+B+"' } ",!1!==t.opts.messages&&(r+=" , message: '",t.opts._errorDataPathProperty?r+="is a required property":r+="should have required property \\'"+B+"\\'",r+="' "),t.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; } "}}t.errorPath=y}else l&&(r+=" if (true) {");return r}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r,i=" ",o=t.level,a=t.dataLevel,s=t.schema[e],A=t.schemaPath+t.util.getProperty(e),l=t.errSchemaPath+"/"+e,c=!t.opts.allErrors,u="data"+(a||""),g="valid"+o,h=t.opts.$data&&s&&s.$data;if(h?(i+=" var schema"+o+" = "+t.util.getData(s.$data,a,t.dataPathArr)+"; ",r="schema"+o):r=s,(s||h)&&!1!==t.opts.uniqueItems){h&&(i+=" var "+g+"; if ("+r+" === false || "+r+" === undefined) "+g+" = true; else if (typeof "+r+" != 'boolean') "+g+" = false; else { "),i+=" var i = "+u+".length , "+g+" = true , j; if (i > 1) { ";var d=t.schema.items&&t.schema.items.type,f=Array.isArray(d);if(!d||"object"==d||"array"==d||f&&(d.indexOf("object")>=0||d.indexOf("array")>=0))i+=" outer: for (;i--;) { for (j = i; j--;) { if (equal("+u+"[i], "+u+"[j])) { "+g+" = false; break outer; } } } ";else{i+=" var itemIndices = {}, item; for (;i--;) { var item = "+u+"[i]; ";var p="checkDataType"+(f?"s":"");i+=" if ("+t.util[p](d,"item",!0)+") continue; ",f&&(i+=" if (typeof item == 'string') item = '\"' + item; "),i+=" if (typeof itemIndices[item] == 'number') { "+g+" = false; j = itemIndices[item]; break; } itemIndices[item] = i; } "}i+=" } ",h&&(i+="  }  "),i+=" if (!"+g+") {   ";var I=I||[];I.push(i),i="",!1!==t.createErrors?(i+=" { keyword: 'uniqueItems' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(l)+" , params: { i: i, j: j } ",!1!==t.opts.messages&&(i+=" , message: 'should NOT have duplicate items (items ## ' + j + ' and ' + i + ' are identical)' "),t.opts.verbose&&(i+=" , schema:  ",i+=h?"validate.schema"+A:""+s,i+="         , parentSchema: validate.schema"+t.schemaPath+" , data: "+u+" "),i+=" } "):i+=" {} ";var C=i;i=I.pop(),!t.compositeRule&&c?t.async?i+=" throw new ValidationError(["+C+"]); ":i+=" validate.errors = ["+C+"]; return false; ":i+=" var err = "+C+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+=" } ",c&&(i+=" else { ")}else c&&(i+=" if (true) { ");return i}},function(t,e,n){"use strict";var r=["multipleOf","maximum","exclusiveMaximum","minimum","exclusiveMinimum","maxLength","minLength","pattern","additionalItems","maxItems","minItems","uniqueItems","maxProperties","minProperties","required","additionalProperties","enum","format","const"];t.exports=function(t,e){for(var n=0;n<e.length;n++){t=JSON.parse(JSON.stringify(t));var i,o=e[n].split("/"),a=t;for(i=1;i<o.length;i++)a=a[o[i]];for(i=0;i<r.length;i++){var s=r[i],A=a[s];A&&(a[s]={anyOf:[A,{$ref:"https://raw.githubusercontent.com/epoberezkin/ajv/master/lib/refs/data.json#"}]})}}return t}},function(t,e,n){"use strict";var r=n(284).MissingRef;t.exports=function t(e,n,i){var o=this;if("function"!=typeof this._opts.loadSchema)throw new Error("options.loadSchema should be a function");"function"==typeof n&&(i=n,n=void 0);var a=s(e).then(function(){var t=o._addSchema(e,void 0,n);return t.validate||function t(e){try{return o._compile(e)}catch(t){if(t instanceof r)return function i(t){var i=t.missingSchema;if(l(i))throw new Error("Schema "+i+" is loaded but "+t.missingRef+" cannot be resolved");var a=o._loadingSchemas[i];a||(a=o._loadingSchemas[i]=o._opts.loadSchema(i)).then(A,A);return a.then(function(t){if(!l(i))return s(t).then(function(){l(i)||o.addSchema(t,i,void 0,n)})}).then(function(){return function t(e){try{return o._compile(e)}catch(t){if(t instanceof r)return i(t);throw t}function i(r){var i=r.missingSchema;if(l(i))throw new Error("Schema "+i+" is loaded but "+r.missingRef+" cannot be resolved");var a=o._loadingSchemas[i];return a||(a=o._loadingSchemas[i]=o._opts.loadSchema(i)).then(A,A),a.then(function(t){if(!l(i))return s(t).then(function(){l(i)||o.addSchema(t,i,void 0,n)})}).then(function(){return t(e)});function A(){delete o._loadingSchemas[i]}function l(t){return o._refs[t]||o._schemas[t]}}}(e)});function A(){delete o._loadingSchemas[i]}function l(t){return o._refs[t]||o._schemas[t]}}(t);throw t}function i(r){var i=r.missingSchema;if(l(i))throw new Error("Schema "+i+" is loaded but "+r.missingRef+" cannot be resolved");var a=o._loadingSchemas[i];return a||(a=o._loadingSchemas[i]=o._opts.loadSchema(i)).then(A,A),a.then(function(t){if(!l(i))return s(t).then(function(){l(i)||o.addSchema(t,i,void 0,n)})}).then(function(){return t(e)});function A(){delete o._loadingSchemas[i]}function l(t){return o._refs[t]||o._schemas[t]}}}(t)});i&&a.then(function(t){i(null,t)},i);return a;function s(e){var n=e.$schema;return n&&!o.getSchema(n)?t.call(o,{$ref:n},!0):Promise.resolve()}}},function(t,e,n){"use strict";var r=/^[a-z_$][a-z0-9_$-]*$/i,i=n(896),o=n(897);t.exports={add:function(t,e){var n=this.RULES;if(n.keywords[t])throw new Error("Keyword "+t+" is already defined");if(!r.test(t))throw new Error("Keyword "+t+" is not a valid identifier");if(e){this.validateKeyword(e,!0);var o=e.type;if(Array.isArray(o))for(var a=0;a<o.length;a++)A(t,o[a],e);else A(t,o,e);var s=e.metaSchema;s&&(e.$data&&this._opts.$data&&(s={anyOf:[s,{$ref:"https://raw.githubusercontent.com/epoberezkin/ajv/master/lib/refs/data.json#"}]}),e.validateSchema=this.compile(s,!0))}function A(t,e,r){for(var o,a=0;a<n.length;a++){var s=n[a];if(s.type==e){o=s;break}}o||(o={type:e,rules:[]},n.push(o));var A={keyword:t,definition:r,custom:!0,code:i,implements:r.implements};o.rules.push(A),n.custom[t]=A}return n.keywords[t]=n.all[t]=!0,this},get:function(t){var e=this.RULES.custom[t];return e?e.definition:this.RULES.keywords[t]||!1},remove:function(t){var e=this.RULES;delete e.keywords[t],delete e.all[t],delete e.custom[t];for(var n=0;n<e.length;n++)for(var r=e[n].rules,i=0;i<r.length;i++)if(r[i].keyword==t){r.splice(i,1);break}return this},validate:function t(e,n){t.errors=null;var r=this._validateKeyword=this._validateKeyword||this.compile(o,!0);if(r(e))return!0;t.errors=r.errors;if(n)throw new Error("custom keyword definition is invalid: "+this.errorsText(r.errors));return!1}}},function(t,e,n){"use strict";t.exports=function(t,e,n){var r,i,o=" ",a=t.level,s=t.dataLevel,A=t.schema[e],l=t.schemaPath+t.util.getProperty(e),c=t.errSchemaPath+"/"+e,u=!t.opts.allErrors,g="data"+(s||""),h="valid"+a,d="errs__"+a,f=t.opts.$data&&A&&A.$data;f?(o+=" var schema"+a+" = "+t.util.getData(A.$data,s,t.dataPathArr)+"; ",i="schema"+a):i=A;var p,I,C,m,y,E="definition"+a,v=this.definition,B="";if(f&&v.$data){y="keywordValidate"+a;var b=v.validateSchema;o+=" var "+E+" = RULES.custom['"+e+"'].definition; var "+y+" = "+E+".validate;"}else{if(!(m=t.useCustomRule(this,A,t.schema,t)))return;i="validate.schema"+l,y=m.code,p=v.compile,I=v.inline,C=v.macro}var Q=y+".errors",w="i"+a,x="ruleErr"+a,_=v.async;if(_&&!t.async)throw new Error("async keyword in sync schema");if(I||C||(o+=Q+" = null;"),o+="var "+d+" = errors;var "+h+";",f&&v.$data&&(B+="}",o+=" if ("+i+" === undefined) { "+h+" = true; } else { ",b&&(B+="}",o+=" "+h+" = "+E+".validateSchema("+i+"); if ("+h+") { ")),I)v.statements?o+=" "+m.validate+" ":o+=" "+h+" = "+m.validate+"; ";else if(C){var D=t.util.copy(t);B="";D.level++;var M="valid"+D.level;D.schema=m.validate,D.schemaPath="";var k=t.compositeRule;t.compositeRule=D.compositeRule=!0;var S=t.validate(D).replace(/validate\.schema/g,y);t.compositeRule=D.compositeRule=k,o+=" "+S}else{(F=F||[]).push(o),o="",o+="  "+y+".call( ",t.opts.passContext?o+="this":o+="self",p||!1===v.schema?o+=" , "+g+" ":o+=" , "+i+" , "+g+" , validate.schema"+t.schemaPath+" ",o+=" , (dataPath || '')",'""'!=t.errorPath&&(o+=" + "+t.errorPath);var L=s?"data"+(s-1||""):"parentData",R=s?t.dataPathArr[s]:"parentDataProperty",N=o+=" , "+L+" , "+R+" , rootData )  ";o=F.pop(),!1===v.errors?(o+=" "+h+" = ",_&&(o+="await "),o+=N+"; "):o+=_?" var "+(Q="customErrors"+a)+" = null; try { "+h+" = await "+N+"; } catch (e) { "+h+" = false; if (e instanceof ValidationError) "+Q+" = e.errors; else throw e; } ":" "+Q+" = null; "+h+" = "+N+"; "}if(v.modifying&&(o+=" if ("+L+") "+g+" = "+L+"["+R+"];"),o+=""+B,v.valid)u&&(o+=" if (true) { ");else{var F;o+=" if ( ",void 0===v.valid?(o+=" !",o+=C?""+M:""+h):o+=" "+!v.valid+" ",o+=") { ",r=this.keyword,(F=F||[]).push(o),o="",(F=F||[]).push(o),o="",!1!==t.createErrors?(o+=" { keyword: '"+(r||"custom")+"' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(c)+" , params: { keyword: '"+this.keyword+"' } ",!1!==t.opts.messages&&(o+=" , message: 'should pass \""+this.keyword+"\" keyword validation' "),t.opts.verbose&&(o+=" , schema: validate.schema"+l+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+g+" "),o+=" } "):o+=" {} ";var T=o;o=F.pop(),!t.compositeRule&&u?t.async?o+=" throw new ValidationError(["+T+"]); ":o+=" validate.errors = ["+T+"]; return false; ":o+=" var err = "+T+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ";var U=o;o=F.pop(),I?v.errors?"full"!=v.errors&&(o+="  for (var "+w+"="+d+"; "+w+"<errors; "+w+"++) { var "+x+" = vErrors["+w+"]; if ("+x+".dataPath === undefined) "+x+".dataPath = (dataPath || '') + "+t.errorPath+"; if ("+x+".schemaPath === undefined) { "+x+'.schemaPath = "'+c+'"; } ',t.opts.verbose&&(o+=" "+x+".schema = "+i+"; "+x+".data = "+g+"; "),o+=" } "):!1===v.errors?o+=" "+U+" ":(o+=" if ("+d+" == errors) { "+U+" } else {  for (var "+w+"="+d+"; "+w+"<errors; "+w+"++) { var "+x+" = vErrors["+w+"]; if ("+x+".dataPath === undefined) "+x+".dataPath = (dataPath || '') + "+t.errorPath+"; if ("+x+".schemaPath === undefined) { "+x+'.schemaPath = "'+c+'"; } ',t.opts.verbose&&(o+=" "+x+".schema = "+i+"; "+x+".data = "+g+"; "),o+=" } } "):C?(o+="   var err =   ",!1!==t.createErrors?(o+=" { keyword: '"+(r||"custom")+"' , dataPath: (dataPath || '') + "+t.errorPath+" , schemaPath: "+t.util.toQuotedString(c)+" , params: { keyword: '"+this.keyword+"' } ",!1!==t.opts.messages&&(o+=" , message: 'should pass \""+this.keyword+"\" keyword validation' "),t.opts.verbose&&(o+=" , schema: validate.schema"+l+" , parentSchema: validate.schema"+t.schemaPath+" , data: "+g+" "),o+=" } "):o+=" {} ",o+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",!t.compositeRule&&u&&(t.async?o+=" throw new ValidationError(vErrors); ":o+=" validate.errors = vErrors; return false; ")):!1===v.errors?o+=" "+U+" ":(o+=" if (Array.isArray("+Q+")) { if (vErrors === null) vErrors = "+Q+"; else vErrors = vErrors.concat("+Q+"); errors = vErrors.length;  for (var "+w+"="+d+"; "+w+"<errors; "+w+"++) { var "+x+" = vErrors["+w+"]; if ("+x+".dataPath === undefined) "+x+".dataPath = (dataPath || '') + "+t.errorPath+";  "+x+'.schemaPath = "'+c+'";  ',t.opts.verbose&&(o+=" "+x+".schema = "+i+"; "+x+".data = "+g+"; "),o+=" } } else { "+U+" } "),o+=" } ",u&&(o+=" else { ")}return o}},function(t,e,n){"use strict";var r=n(439);t.exports={$id:"https://github.com/epoberezkin/ajv/blob/master/lib/definition_schema.js",definitions:{simpleTypes:r.definitions.simpleTypes},type:"object",dependencies:{schema:["validate"],$data:["validate"],statements:["inline"],valid:{not:{required:["macro"]}}},properties:{type:r.properties.type,schema:{type:"boolean"},statements:{type:"boolean"},dependencies:{type:"array",items:{type:"string"}},metaSchema:{type:"object"},modifying:{type:"boolean"},valid:{type:"boolean"},$data:{type:"boolean"},async:{type:"boolean"},errors:{anyOf:[{type:"boolean"},{const:"full"}]}}}},function(t){t.exports=JSON.parse('{"$schema":"http://json-schema.org/draft-07/schema#","$id":"https://raw.githubusercontent.com/epoberezkin/ajv/master/lib/refs/data.json#","description":"Meta-schema for $data reference (JSON Schema extension proposal)","type":"object","required":["$data"],"properties":{"$data":{"type":"string","anyOf":[{"format":"relative-json-pointer"},{"format":"json-pointer"}]}},"additionalProperties":false}')},function(t,e,n){t.exports=function(){"use strict";function t(t,e){return t(e={exports:{}},e.exports),e.exports}var e=t(function(t){var e=t.exports="undefined"!=typeof window&&window.Math==Math?window:"undefined"!=typeof self&&self.Math==Math?self:Function("return this")();"number"==typeof __g&&(__g=e)}),n=t(function(t){var e=t.exports={version:"2.5.7"};"number"==typeof __e&&(__e=e)}),r=(n.version,function(t){return"object"==typeof t?null!==t:"function"==typeof t}),i=function(t){if(!r(t))throw TypeError(t+" is not an object!");return t},o=function(t){try{return!!t()}catch(t){return!0}},a=!o(function(){return 7!=Object.defineProperty({},"a",{get:function(){return 7}}).a}),s=e.document,A=r(s)&&r(s.createElement),l=!a&&!o(function(){return 7!=Object.defineProperty((t="div",A?s.createElement(t):{}),"a",{get:function(){return 7}}).a;var t}),c=Object.defineProperty,u={f:a?Object.defineProperty:function(t,e,n){if(i(t),e=function(t,e){if(!r(t))return t;var n,i;if(e&&"function"==typeof(n=t.toString)&&!r(i=n.call(t)))return i;if("function"==typeof(n=t.valueOf)&&!r(i=n.call(t)))return i;if(!e&&"function"==typeof(n=t.toString)&&!r(i=n.call(t)))return i;throw TypeError("Can't convert object to primitive value")}(e,!0),i(n),l)try{return c(t,e,n)}catch(t){}if("get"in n||"set"in n)throw TypeError("Accessors not supported!");return"value"in n&&(t[e]=n.value),t}},g=a?function(t,e,n){return u.f(t,e,function(t,e){return{enumerable:!(1&t),configurable:!(2&t),writable:!(4&t),value:e}}(1,n))}:function(t,e,n){return t[e]=n,t},h={}.hasOwnProperty,d=function(t,e){return h.call(t,e)},f=0,p=Math.random(),I=t(function(t){var r=function(t){return"Symbol(".concat(void 0===t?"":t,")_",(++f+p).toString(36))}("src"),i=Function.toString,o=(""+i).split("toString");n.inspectSource=function(t){return i.call(t)},(t.exports=function(t,n,i,a){var s="function"==typeof i;s&&(d(i,"name")||g(i,"name",n)),t[n]!==i&&(s&&(d(i,r)||g(i,r,t[n]?""+t[n]:o.join(String(n)))),t===e?t[n]=i:a?t[n]?t[n]=i:g(t,n,i):(delete t[n],g(t,n,i)))})(Function.prototype,"toString",function(){return"function"==typeof this&&this[r]||i.call(this)})}),C=function(t,e,n){if(function(t){if("function"!=typeof t)throw TypeError(t+" is not a function!")}(t),void 0===e)return t;switch(n){case 1:return function(n){return t.call(e,n)};case 2:return function(n,r){return t.call(e,n,r)};case 3:return function(n,r,i){return t.call(e,n,r,i)}}return function(){return t.apply(e,arguments)}},m=function(t,r,i){var o,a,s,A,l=t&m.F,c=t&m.G,u=t&m.S,h=t&m.P,d=t&m.B,f=c?e:u?e[r]||(e[r]={}):(e[r]||{}).prototype,p=c?n:n[r]||(n[r]={}),y=p.prototype||(p.prototype={});for(o in c&&(i=r),i)a=!l&&f&&void 0!==f[o],s=(a?f:i)[o],A=d&&a?C(s,e):h&&"function"==typeof s?C(Function.call,s):s,f&&I(f,o,s,t&m.U),p[o]!=s&&g(p,o,A),h&&y[o]!=s&&(y[o]=s)};e.core=n,m.F=1,m.G=2,m.S=4,m.P=8,m.B=16,m.W=32,m.U=64,m.R=128;var y,E=m,v=Math.ceil,B=Math.floor,b=function(t){return isNaN(t=+t)?0:(t>0?B:v)(t)},Q=(y=!1,function(t,e){var n,r,i=String(function(t){if(null==t)throw TypeError("Can't call method on  "+t);return t}(t)),o=b(e),a=i.length;return o<0||o>=a?y?"":void 0:(n=i.charCodeAt(o))<55296||n>56319||o+1===a||(r=i.charCodeAt(o+1))<56320||r>57343?y?i.charAt(o):n:y?i.slice(o,o+2):r-56320+(n-55296<<10)+65536});E(E.P,"String",{codePointAt:function(t){return Q(this,t)}}),n.String.codePointAt;var w=Math.max,x=Math.min,_=function(t,e){return(t=b(t))<0?w(t+e,0):x(t,e)},D=String.fromCharCode,M=String.fromCodePoint;E(E.S+E.F*(!!M&&1!=M.length),"String",{fromCodePoint:function(t){for(var e,n=arguments,r=[],i=arguments.length,o=0;i>o;){if(e=+n[o++],_(e,1114111)!==e)throw RangeError(e+" is not a valid code point");r.push(e<65536?D(e):D(55296+((e-=65536)>>10),e%1024+56320))}return r.join("")}}),n.String.fromCodePoint;var k,S,L,R,N,F,T,U,O,G,H,j,Y,P,K={Space_Separator:/[\u1680\u2000-\u200A\u202F\u205F\u3000]/,ID_Start:/[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0370-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u05D0-\u05EA\u05F0-\u05F2\u0620-\u064A\u066E\u066F\u0671-\u06D3\u06D5\u06E5\u06E6\u06EE\u06EF\u06FA-\u06FC\u06FF\u0710\u0712-\u072F\u074D-\u07A5\u07B1\u07CA-\u07EA\u07F4\u07F5\u07FA\u0800-\u0815\u081A\u0824\u0828\u0840-\u0858\u0860-\u086A\u08A0-\u08B4\u08B6-\u08BD\u0904-\u0939\u093D\u0950\u0958-\u0961\u0971-\u0980\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BD\u09CE\u09DC\u09DD\u09DF-\u09E1\u09F0\u09F1\u09FC\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A59-\u0A5C\u0A5E\u0A72-\u0A74\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABD\u0AD0\u0AE0\u0AE1\u0AF9\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3D\u0B5C\u0B5D\u0B5F-\u0B61\u0B71\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BD0\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D\u0C58-\u0C5A\u0C60\u0C61\u0C80\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBD\u0CDE\u0CE0\u0CE1\u0CF1\u0CF2\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D\u0D4E\u0D54-\u0D56\u0D5F-\u0D61\u0D7A-\u0D7F\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0E01-\u0E30\u0E32\u0E33\u0E40-\u0E46\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB0\u0EB2\u0EB3\u0EBD\u0EC0-\u0EC4\u0EC6\u0EDC-\u0EDF\u0F00\u0F40-\u0F47\u0F49-\u0F6C\u0F88-\u0F8C\u1000-\u102A\u103F\u1050-\u1055\u105A-\u105D\u1061\u1065\u1066\u106E-\u1070\u1075-\u1081\u108E\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176C\u176E-\u1770\u1780-\u17B3\u17D7\u17DC\u1820-\u1877\u1880-\u1884\u1887-\u18A8\u18AA\u18B0-\u18F5\u1900-\u191E\u1950-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u1A00-\u1A16\u1A20-\u1A54\u1AA7\u1B05-\u1B33\u1B45-\u1B4B\u1B83-\u1BA0\u1BAE\u1BAF\u1BBA-\u1BE5\u1C00-\u1C23\u1C4D-\u1C4F\u1C5A-\u1C7D\u1C80-\u1C88\u1CE9-\u1CEC\u1CEE-\u1CF1\u1CF5\u1CF6\u1D00-\u1DBF\u1E00-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u2071\u207F\u2090-\u209C\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CEE\u2CF2\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2E2F\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303C\u3041-\u3096\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312E\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FEA\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA61F\uA62A\uA62B\uA640-\uA66E\uA67F-\uA69D\uA6A0-\uA6EF\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AE\uA7B0-\uA7B7\uA7F7-\uA801\uA803-\uA805\uA807-\uA80A\uA80C-\uA822\uA840-\uA873\uA882-\uA8B3\uA8F2-\uA8F7\uA8FB\uA8FD\uA90A-\uA925\uA930-\uA946\uA960-\uA97C\uA984-\uA9B2\uA9CF\uA9E0-\uA9E4\uA9E6-\uA9EF\uA9FA-\uA9FE\uAA00-\uAA28\uAA40-\uAA42\uAA44-\uAA4B\uAA60-\uAA76\uAA7A\uAA7E-\uAAAF\uAAB1\uAAB5\uAAB6\uAAB9-\uAABD\uAAC0\uAAC2\uAADB-\uAADD\uAAE0-\uAAEA\uAAF2-\uAAF4\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABE2\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D\uFB1F-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE70-\uFE74\uFE76-\uFEFC\uFF21-\uFF3A\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDD40-\uDD74\uDE80-\uDE9C\uDEA0-\uDED0\uDF00-\uDF1F\uDF2D-\uDF4A\uDF50-\uDF75\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF\uDFD1-\uDFD5]|\uD801[\uDC00-\uDC9D\uDCB0-\uDCD3\uDCD8-\uDCFB\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDCE0-\uDCF2\uDCF4\uDCF5\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00\uDE10-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE4\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48\uDC80-\uDCB2\uDCC0-\uDCF2]|\uD804[\uDC03-\uDC37\uDC83-\uDCAF\uDCD0-\uDCE8\uDD03-\uDD26\uDD50-\uDD72\uDD76\uDD83-\uDDB2\uDDC1-\uDDC4\uDDDA\uDDDC\uDE00-\uDE11\uDE13-\uDE2B\uDE80-\uDE86\uDE88\uDE8A-\uDE8D\uDE8F-\uDE9D\uDE9F-\uDEA8\uDEB0-\uDEDE\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3D\uDF50\uDF5D-\uDF61]|\uD805[\uDC00-\uDC34\uDC47-\uDC4A\uDC80-\uDCAF\uDCC4\uDCC5\uDCC7\uDD80-\uDDAE\uDDD8-\uDDDB\uDE00-\uDE2F\uDE44\uDE80-\uDEAA\uDF00-\uDF19]|\uD806[\uDCA0-\uDCDF\uDCFF\uDE00\uDE0B-\uDE32\uDE3A\uDE50\uDE5C-\uDE83\uDE86-\uDE89\uDEC0-\uDEF8]|\uD807[\uDC00-\uDC08\uDC0A-\uDC2E\uDC40\uDC72-\uDC8F\uDD00-\uDD06\uDD08\uDD09\uDD0B-\uDD30\uDD46]|\uD808[\uDC00-\uDF99]|\uD809[\uDC00-\uDC6E\uDC80-\uDD43]|[\uD80C\uD81C-\uD820\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872\uD874-\uD879][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD811[\uDC00-\uDE46]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDED0-\uDEED\uDF00-\uDF2F\uDF40-\uDF43\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50\uDF93-\uDF9F\uDFE0\uDFE1]|\uD821[\uDC00-\uDFEC]|\uD822[\uDC00-\uDEF2]|\uD82C[\uDC00-\uDD1E\uDD70-\uDEFB]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB]|\uD83A[\uDC00-\uDCC4\uDD00-\uDD43]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1\uDEB0-\uDFFF]|\uD87A[\uDC00-\uDFE0]|\uD87E[\uDC00-\uDE1D]/,ID_Continue:/[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0300-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u0483-\u0487\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u0591-\u05BD\u05BF\u05C1\u05C2\u05C4\u05C5\u05C7\u05D0-\u05EA\u05F0-\u05F2\u0610-\u061A\u0620-\u0669\u066E-\u06D3\u06D5-\u06DC\u06DF-\u06E8\u06EA-\u06FC\u06FF\u0710-\u074A\u074D-\u07B1\u07C0-\u07F5\u07FA\u0800-\u082D\u0840-\u085B\u0860-\u086A\u08A0-\u08B4\u08B6-\u08BD\u08D4-\u08E1\u08E3-\u0963\u0966-\u096F\u0971-\u0983\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BC-\u09C4\u09C7\u09C8\u09CB-\u09CE\u09D7\u09DC\u09DD\u09DF-\u09E3\u09E6-\u09F1\u09FC\u0A01-\u0A03\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A3C\u0A3E-\u0A42\u0A47\u0A48\u0A4B-\u0A4D\u0A51\u0A59-\u0A5C\u0A5E\u0A66-\u0A75\u0A81-\u0A83\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABC-\u0AC5\u0AC7-\u0AC9\u0ACB-\u0ACD\u0AD0\u0AE0-\u0AE3\u0AE6-\u0AEF\u0AF9-\u0AFF\u0B01-\u0B03\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3C-\u0B44\u0B47\u0B48\u0B4B-\u0B4D\u0B56\u0B57\u0B5C\u0B5D\u0B5F-\u0B63\u0B66-\u0B6F\u0B71\u0B82\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BBE-\u0BC2\u0BC6-\u0BC8\u0BCA-\u0BCD\u0BD0\u0BD7\u0BE6-\u0BEF\u0C00-\u0C03\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D-\u0C44\u0C46-\u0C48\u0C4A-\u0C4D\u0C55\u0C56\u0C58-\u0C5A\u0C60-\u0C63\u0C66-\u0C6F\u0C80-\u0C83\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBC-\u0CC4\u0CC6-\u0CC8\u0CCA-\u0CCD\u0CD5\u0CD6\u0CDE\u0CE0-\u0CE3\u0CE6-\u0CEF\u0CF1\u0CF2\u0D00-\u0D03\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D44\u0D46-\u0D48\u0D4A-\u0D4E\u0D54-\u0D57\u0D5F-\u0D63\u0D66-\u0D6F\u0D7A-\u0D7F\u0D82\u0D83\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0DCA\u0DCF-\u0DD4\u0DD6\u0DD8-\u0DDF\u0DE6-\u0DEF\u0DF2\u0DF3\u0E01-\u0E3A\u0E40-\u0E4E\u0E50-\u0E59\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB9\u0EBB-\u0EBD\u0EC0-\u0EC4\u0EC6\u0EC8-\u0ECD\u0ED0-\u0ED9\u0EDC-\u0EDF\u0F00\u0F18\u0F19\u0F20-\u0F29\u0F35\u0F37\u0F39\u0F3E-\u0F47\u0F49-\u0F6C\u0F71-\u0F84\u0F86-\u0F97\u0F99-\u0FBC\u0FC6\u1000-\u1049\u1050-\u109D\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u135D-\u135F\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1714\u1720-\u1734\u1740-\u1753\u1760-\u176C\u176E-\u1770\u1772\u1773\u1780-\u17D3\u17D7\u17DC\u17DD\u17E0-\u17E9\u180B-\u180D\u1810-\u1819\u1820-\u1877\u1880-\u18AA\u18B0-\u18F5\u1900-\u191E\u1920-\u192B\u1930-\u193B\u1946-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u19D0-\u19D9\u1A00-\u1A1B\u1A20-\u1A5E\u1A60-\u1A7C\u1A7F-\u1A89\u1A90-\u1A99\u1AA7\u1AB0-\u1ABD\u1B00-\u1B4B\u1B50-\u1B59\u1B6B-\u1B73\u1B80-\u1BF3\u1C00-\u1C37\u1C40-\u1C49\u1C4D-\u1C7D\u1C80-\u1C88\u1CD0-\u1CD2\u1CD4-\u1CF9\u1D00-\u1DF9\u1DFB-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u203F\u2040\u2054\u2071\u207F\u2090-\u209C\u20D0-\u20DC\u20E1\u20E5-\u20F0\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D7F-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2DE0-\u2DFF\u2E2F\u3005-\u3007\u3021-\u302F\u3031-\u3035\u3038-\u303C\u3041-\u3096\u3099\u309A\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312E\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FEA\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA62B\uA640-\uA66F\uA674-\uA67D\uA67F-\uA6F1\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AE\uA7B0-\uA7B7\uA7F7-\uA827\uA840-\uA873\uA880-\uA8C5\uA8D0-\uA8D9\uA8E0-\uA8F7\uA8FB\uA8FD\uA900-\uA92D\uA930-\uA953\uA960-\uA97C\uA980-\uA9C0\uA9CF-\uA9D9\uA9E0-\uA9FE\uAA00-\uAA36\uAA40-\uAA4D\uAA50-\uAA59\uAA60-\uAA76\uAA7A-\uAAC2\uAADB-\uAADD\uAAE0-\uAAEF\uAAF2-\uAAF6\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABEA\uABEC\uABED\uABF0-\uABF9\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE00-\uFE0F\uFE20-\uFE2F\uFE33\uFE34\uFE4D-\uFE4F\uFE70-\uFE74\uFE76-\uFEFC\uFF10-\uFF19\uFF21-\uFF3A\uFF3F\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDD40-\uDD74\uDDFD\uDE80-\uDE9C\uDEA0-\uDED0\uDEE0\uDF00-\uDF1F\uDF2D-\uDF4A\uDF50-\uDF7A\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF\uDFD1-\uDFD5]|\uD801[\uDC00-\uDC9D\uDCA0-\uDCA9\uDCB0-\uDCD3\uDCD8-\uDCFB\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDCE0-\uDCF2\uDCF4\uDCF5\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00-\uDE03\uDE05\uDE06\uDE0C-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE38-\uDE3A\uDE3F\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE6\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48\uDC80-\uDCB2\uDCC0-\uDCF2]|\uD804[\uDC00-\uDC46\uDC66-\uDC6F\uDC7F-\uDCBA\uDCD0-\uDCE8\uDCF0-\uDCF9\uDD00-\uDD34\uDD36-\uDD3F\uDD50-\uDD73\uDD76\uDD80-\uDDC4\uDDCA-\uDDCC\uDDD0-\uDDDA\uDDDC\uDE00-\uDE11\uDE13-\uDE37\uDE3E\uDE80-\uDE86\uDE88\uDE8A-\uDE8D\uDE8F-\uDE9D\uDE9F-\uDEA8\uDEB0-\uDEEA\uDEF0-\uDEF9\uDF00-\uDF03\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3C-\uDF44\uDF47\uDF48\uDF4B-\uDF4D\uDF50\uDF57\uDF5D-\uDF63\uDF66-\uDF6C\uDF70-\uDF74]|\uD805[\uDC00-\uDC4A\uDC50-\uDC59\uDC80-\uDCC5\uDCC7\uDCD0-\uDCD9\uDD80-\uDDB5\uDDB8-\uDDC0\uDDD8-\uDDDD\uDE00-\uDE40\uDE44\uDE50-\uDE59\uDE80-\uDEB7\uDEC0-\uDEC9\uDF00-\uDF19\uDF1D-\uDF2B\uDF30-\uDF39]|\uD806[\uDCA0-\uDCE9\uDCFF\uDE00-\uDE3E\uDE47\uDE50-\uDE83\uDE86-\uDE99\uDEC0-\uDEF8]|\uD807[\uDC00-\uDC08\uDC0A-\uDC36\uDC38-\uDC40\uDC50-\uDC59\uDC72-\uDC8F\uDC92-\uDCA7\uDCA9-\uDCB6\uDD00-\uDD06\uDD08\uDD09\uDD0B-\uDD36\uDD3A\uDD3C\uDD3D\uDD3F-\uDD47\uDD50-\uDD59]|\uD808[\uDC00-\uDF99]|\uD809[\uDC00-\uDC6E\uDC80-\uDD43]|[\uD80C\uD81C-\uD820\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872\uD874-\uD879][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD811[\uDC00-\uDE46]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDE60-\uDE69\uDED0-\uDEED\uDEF0-\uDEF4\uDF00-\uDF36\uDF40-\uDF43\uDF50-\uDF59\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50-\uDF7E\uDF8F-\uDF9F\uDFE0\uDFE1]|\uD821[\uDC00-\uDFEC]|\uD822[\uDC00-\uDEF2]|\uD82C[\uDC00-\uDD1E\uDD70-\uDEFB]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99\uDC9D\uDC9E]|\uD834[\uDD65-\uDD69\uDD6D-\uDD72\uDD7B-\uDD82\uDD85-\uDD8B\uDDAA-\uDDAD\uDE42-\uDE44]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB\uDFCE-\uDFFF]|\uD836[\uDE00-\uDE36\uDE3B-\uDE6C\uDE75\uDE84\uDE9B-\uDE9F\uDEA1-\uDEAF]|\uD838[\uDC00-\uDC06\uDC08-\uDC18\uDC1B-\uDC21\uDC23\uDC24\uDC26-\uDC2A]|\uD83A[\uDC00-\uDCC4\uDCD0-\uDCD6\uDD00-\uDD4A\uDD50-\uDD59]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1\uDEB0-\uDFFF]|\uD87A[\uDC00-\uDFE0]|\uD87E[\uDC00-\uDE1D]|\uDB40[\uDD00-\uDDEF]/},z={isSpaceSeparator:function(t){return K.Space_Separator.test(t)},isIdStartChar:function(t){return t>="a"&&t<="z"||t>="A"&&t<="Z"||"$"===t||"_"===t||K.ID_Start.test(t)},isIdContinueChar:function(t){return t>="a"&&t<="z"||t>="A"&&t<="Z"||t>="0"&&t<="9"||"$"===t||"_"===t||"‌"===t||"‍"===t||K.ID_Continue.test(t)},isDigit:function(t){return/[0-9]/.test(t)},isHexDigit:function(t){return/[0-9A-Fa-f]/.test(t)}};function J(){for(G="default",H="",j=!1,Y=1;;){P=q();var t=W[G]();if(t)return t}}function q(){if(k[R])return String.fromCodePoint(k.codePointAt(R))}function V(){var t=q();return"\n"===t?(N++,F=0):t?F+=t.length:F++,t&&(R+=t.length),t}var W={default:function(){switch(P){case"\t":case"\v":case"\f":case" ":case" ":case"\ufeff":case"\n":case"\r":case"\u2028":case"\u2029":return void V();case"/":return V(),void(G="comment");case void 0:return V(),X("eof")}if(!z.isSpaceSeparator(P))return W[S]();V()},comment:function(){switch(P){case"*":return V(),void(G="multiLineComment");case"/":return V(),void(G="singleLineComment")}throw rt(V())},multiLineComment:function(){switch(P){case"*":return V(),void(G="multiLineCommentAsterisk");case void 0:throw rt(V())}V()},multiLineCommentAsterisk:function(){switch(P){case"*":return void V();case"/":return V(),void(G="default");case void 0:throw rt(V())}V(),G="multiLineComment"},singleLineComment:function(){switch(P){case"\n":case"\r":case"\u2028":case"\u2029":return V(),void(G="default");case void 0:return V(),X("eof")}V()},value:function(){switch(P){case"{":case"[":return X("punctuator",V());case"n":return V(),Z("ull"),X("null",null);case"t":return V(),Z("rue"),X("boolean",!0);case"f":return V(),Z("alse"),X("boolean",!1);case"-":case"+":return"-"===V()&&(Y=-1),void(G="sign");case".":return H=V(),void(G="decimalPointLeading");case"0":return H=V(),void(G="zero");case"1":case"2":case"3":case"4":case"5":case"6":case"7":case"8":case"9":return H=V(),void(G="decimalInteger");case"I":return V(),Z("nfinity"),X("numeric",1/0);case"N":return V(),Z("aN"),X("numeric",NaN);case'"':case"'":return j='"'===V(),H="",void(G="string")}throw rt(V())},identifierNameStartEscape:function(){if("u"!==P)throw rt(V());V();var t=$();switch(t){case"$":case"_":break;default:if(!z.isIdStartChar(t))throw ot()}H+=t,G="identifierName"},identifierName:function(){switch(P){case"$":case"_":case"‌":case"‍":return void(H+=V());case"\\":return V(),void(G="identifierNameEscape")}if(!z.isIdContinueChar(P))return X("identifier",H);H+=V()},identifierNameEscape:function(){if("u"!==P)throw rt(V());V();var t=$();switch(t){case"$":case"_":case"‌":case"‍":break;default:if(!z.isIdContinueChar(t))throw ot()}H+=t,G="identifierName"},sign:function(){switch(P){case".":return H=V(),void(G="decimalPointLeading");case"0":return H=V(),void(G="zero");case"1":case"2":case"3":case"4":case"5":case"6":case"7":case"8":case"9":return H=V(),void(G="decimalInteger");case"I":return V(),Z("nfinity"),X("numeric",Y*(1/0));case"N":return V(),Z("aN"),X("numeric",NaN)}throw rt(V())},zero:function(){switch(P){case".":return H+=V(),void(G="decimalPoint");case"e":case"E":return H+=V(),void(G="decimalExponent");case"x":case"X":return H+=V(),void(G="hexadecimal")}return X("numeric",0*Y)},decimalInteger:function(){switch(P){case".":return H+=V(),void(G="decimalPoint");case"e":case"E":return H+=V(),void(G="decimalExponent")}if(!z.isDigit(P))return X("numeric",Y*Number(H));H+=V()},decimalPointLeading:function(){if(z.isDigit(P))return H+=V(),void(G="decimalFraction");throw rt(V())},decimalPoint:function(){switch(P){case"e":case"E":return H+=V(),void(G="decimalExponent")}return z.isDigit(P)?(H+=V(),void(G="decimalFraction")):X("numeric",Y*Number(H))},decimalFraction:function(){switch(P){case"e":case"E":return H+=V(),void(G="decimalExponent")}if(!z.isDigit(P))return X("numeric",Y*Number(H));H+=V()},decimalExponent:function(){switch(P){case"+":case"-":return H+=V(),void(G="decimalExponentSign")}if(z.isDigit(P))return H+=V(),void(G="decimalExponentInteger");throw rt(V())},decimalExponentSign:function(){if(z.isDigit(P))return H+=V(),void(G="decimalExponentInteger");throw rt(V())},decimalExponentInteger:function(){if(!z.isDigit(P))return X("numeric",Y*Number(H));H+=V()},hexadecimal:function(){if(z.isHexDigit(P))return H+=V(),void(G="hexadecimalInteger");throw rt(V())},hexadecimalInteger:function(){if(!z.isHexDigit(P))return X("numeric",Y*Number(H));H+=V()},string:function(){switch(P){case"\\":return V(),void(H+=function(){switch(q()){case"b":return V(),"\b";case"f":return V(),"\f";case"n":return V(),"\n";case"r":return V(),"\r";case"t":return V(),"\t";case"v":return V(),"\v";case"0":if(V(),z.isDigit(q()))throw rt(V());return"\0";case"x":return V(),function(){var t="",e=q();if(!z.isHexDigit(e))throw rt(V());if(t+=V(),e=q(),!z.isHexDigit(e))throw rt(V());return t+=V(),String.fromCodePoint(parseInt(t,16))}();case"u":return V(),$();case"\n":case"\u2028":case"\u2029":return V(),"";case"\r":return V(),"\n"===q()&&V(),"";case"1":case"2":case"3":case"4":case"5":case"6":case"7":case"8":case"9":case void 0:throw rt(V())}return V()}());case'"':return j?(V(),X("string",H)):void(H+=V());case"'":return j?void(H+=V()):(V(),X("string",H));case"\n":case"\r":throw rt(V());case"\u2028":case"\u2029":!function(t){console.warn("JSON5: '"+at(t)+"' in strings is not valid ECMAScript; consider escaping")}(P);break;case void 0:throw rt(V())}H+=V()},start:function(){switch(P){case"{":case"[":return X("punctuator",V())}G="value"},beforePropertyName:function(){switch(P){case"$":case"_":return H=V(),void(G="identifierName");case"\\":return V(),void(G="identifierNameStartEscape");case"}":return X("punctuator",V());case'"':case"'":return j='"'===V(),void(G="string")}if(z.isIdStartChar(P))return H+=V(),void(G="identifierName");throw rt(V())},afterPropertyName:function(){if(":"===P)return X("punctuator",V());throw rt(V())},beforePropertyValue:function(){G="value"},afterPropertyValue:function(){switch(P){case",":case"}":return X("punctuator",V())}throw rt(V())},beforeArrayValue:function(){if("]"===P)return X("punctuator",V());G="value"},afterArrayValue:function(){switch(P){case",":case"]":return X("punctuator",V())}throw rt(V())},end:function(){throw rt(V())}};function X(t,e){return{type:t,value:e,line:N,column:F}}function Z(t){for(var e=0,n=t;e<n.length;e+=1){var r=n[e],i=q();if(i!==r)throw rt(V());V()}}function $(){for(var t="",e=4;e-- >0;){var n=q();if(!z.isHexDigit(n))throw rt(V());t+=V()}return String.fromCodePoint(parseInt(t,16))}var tt={start:function(){if("eof"===T.type)throw it();et()},beforePropertyName:function(){switch(T.type){case"identifier":case"string":return U=T.value,void(S="afterPropertyName");case"punctuator":return void nt();case"eof":throw it()}},afterPropertyName:function(){if("eof"===T.type)throw it();S="beforePropertyValue"},beforePropertyValue:function(){if("eof"===T.type)throw it();et()},beforeArrayValue:function(){if("eof"===T.type)throw it();"punctuator"!==T.type||"]"!==T.value?et():nt()},afterPropertyValue:function(){if("eof"===T.type)throw it();switch(T.value){case",":return void(S="beforePropertyName");case"}":nt()}},afterArrayValue:function(){if("eof"===T.type)throw it();switch(T.value){case",":return void(S="beforeArrayValue");case"]":nt()}},end:function(){}};function et(){var t;switch(T.type){case"punctuator":switch(T.value){case"{":t={};break;case"[":t=[]}break;case"null":case"boolean":case"numeric":case"string":t=T.value}if(void 0===O)O=t;else{var e=L[L.length-1];Array.isArray(e)?e.push(t):e[U]=t}if(null!==t&&"object"==typeof t)L.push(t),S=Array.isArray(t)?"beforeArrayValue":"beforePropertyName";else{var n=L[L.length-1];S=null==n?"end":Array.isArray(n)?"afterArrayValue":"afterPropertyValue"}}function nt(){L.pop();var t=L[L.length-1];S=null==t?"end":Array.isArray(t)?"afterArrayValue":"afterPropertyValue"}function rt(t){return st(void 0===t?"JSON5: invalid end of input at "+N+":"+F:"JSON5: invalid character '"+at(t)+"' at "+N+":"+F)}function it(){return st("JSON5: invalid end of input at "+N+":"+F)}function ot(){return st("JSON5: invalid identifier character at "+N+":"+(F-=5))}function at(t){var e={"'":"\\'",'"':'\\"',"\\":"\\\\","\b":"\\b","\f":"\\f","\n":"\\n","\r":"\\r","\t":"\\t","\v":"\\v","\0":"\\0","\u2028":"\\u2028","\u2029":"\\u2029"};if(e[t])return e[t];if(t<" "){var n=t.charCodeAt(0).toString(16);return"\\x"+("00"+n).substring(n.length)}return t}function st(t){var e=new SyntaxError(t);return e.lineNumber=N,e.columnNumber=F,e}return{parse:function(t,e){k=String(t),S="start",L=[],R=0,N=1,F=0,T=void 0,U=void 0,O=void 0;do{T=J(),tt[S]()}while("eof"!==T.type);return"function"==typeof e?function t(e,n,r){var i=e[n];if(null!=i&&"object"==typeof i)for(var o in i){var a=t(i,o,r);void 0===a?delete i[o]:i[o]=a}return r.call(e,n,i)}({"":O},"",e):O},stringify:function(t,e,n){var r,i,o,a=[],s="",A="";if(null==e||"object"!=typeof e||Array.isArray(e)||(n=e.space,o=e.quote,e=e.replacer),"function"==typeof e)i=e;else if(Array.isArray(e)){r=[];for(var l=0,c=e;l<c.length;l+=1){var u=c[l],g=void 0;"string"==typeof u?g=u:("number"==typeof u||u instanceof String||u instanceof Number)&&(g=String(u)),void 0!==g&&r.indexOf(g)<0&&r.push(g)}}return n instanceof Number?n=Number(n):n instanceof String&&(n=String(n)),"number"==typeof n?n>0&&(n=Math.min(10,Math.floor(n)),A="          ".substr(0,n)):"string"==typeof n&&(A=n.substr(0,10)),h("",{"":t});function h(t,e){var n=e[t];switch(null!=n&&("function"==typeof n.toJSON5?n=n.toJSON5(t):"function"==typeof n.toJSON&&(n=n.toJSON(t))),i&&(n=i.call(e,t,n)),n instanceof Number?n=Number(n):n instanceof String?n=String(n):n instanceof Boolean&&(n=n.valueOf()),n){case null:return"null";case!0:return"true";case!1:return"false"}return"string"==typeof n?d(n):"number"==typeof n?String(n):"object"==typeof n?Array.isArray(n)?function(t){if(a.indexOf(t)>=0)throw TypeError("Converting circular structure to JSON5");a.push(t);var e=s;s+=A;for(var n,r=[],i=0;i<t.length;i++){var o=h(String(i),t);r.push(void 0!==o?o:"null")}if(0===r.length)n="[]";else if(""===A){var l=r.join(",");n="["+l+"]"}else{var c=",\n"+s,u=r.join(c);n="[\n"+s+u+",\n"+e+"]"}return a.pop(),s=e,n}(n):function(t){if(a.indexOf(t)>=0)throw TypeError("Converting circular structure to JSON5");a.push(t);var e=s;s+=A;for(var n,i,o=r||Object.keys(t),l=[],c=0,u=o;c<u.length;c+=1){var g=u[c],d=h(g,t);if(void 0!==d){var p=f(g)+":";""!==A&&(p+=" "),p+=d,l.push(p)}}if(0===l.length)n="{}";else if(""===A)i=l.join(","),n="{"+i+"}";else{var I=",\n"+s;i=l.join(I),n="{\n"+s+i+",\n"+e+"}"}return a.pop(),s=e,n}(n):void 0}function d(t){for(var e={"'":.1,'"':.2},n={"'":"\\'",'"':'\\"',"\\":"\\\\","\b":"\\b","\f":"\\f","\n":"\\n","\r":"\\r","\t":"\\t","\v":"\\v","\0":"\\0","\u2028":"\\u2028","\u2029":"\\u2029"},r="",i=0,a=t;i<a.length;i+=1){var s=a[i];switch(s){case"'":case'"':e[s]++,r+=s;continue}if(n[s])r+=n[s];else if(s<" "){var A=s.charCodeAt(0).toString(16);r+="\\x"+("00"+A).substring(A.length)}else r+=s}var l=o||Object.keys(e).reduce(function(t,n){return e[t]<e[n]?t:n});return r=r.replace(new RegExp(l,"g"),n[l]),l+r+l}function f(t){if(0===t.length)return d(t);var e=String.fromCodePoint(t.codePointAt(0));if(!z.isIdStartChar(e))return d(t);for(var n=e.length;n<t.length;n++)if(!z.isIdContinueChar(String.fromCodePoint(t.codePointAt(n))))return d(t);return t}}}}()},function(t){t.exports=JSON.parse('{"$schema":"http://json-schema.org/draft-07/schema","title":"JupyterLab Plugin Settings/Preferences Schema","description":"JupyterLab plugin settings/preferences schema","version":"1.0.0","type":"object","additionalProperties":true,"properties":{"jupyter.lab.setting-deprecated":{"type":"boolean","default":false},"jupyter.lab.setting-icon-class":{"type":"string","default":"jp-SettingsIcon"},"jupyter.lab.setting-icon-label":{"type":"string","default":"Plugin"},"jupyter.lab.shortcuts":{"items":{"$ref":"#/definitions/shortcut"},"type":"array","default":[]},"jupyter.lab.transform":{"type":"boolean","default":false}},"definitions":{"shortcut":{"properties":{"args":{"type":"object"},"command":{"type":"string"},"disabled":{"type":"boolean","default":false},"keys":{"items":{"type":"string"},"minItems":1,"type":"array"},"selector":{"type":"string"}},"required":["command","keys","selector"],"type":"object"}}}')},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(10);class i{constructor(t={}){this._changed=new r.Signal(this);const{connector:e,transform:n}=t;this._connector=e||new i.Connector,this._ready=(n||Promise.resolve(null)).then(t=>{if(!t)return;const{contents:e,type:n}=t;switch(n){case"cancel":return;case"clear":return this._clear();case"merge":return this._merge(e||{});case"overwrite":return this._overwrite(e||{});default:return}})}get changed(){return this._changed}async clear(){await this._ready,await this._clear()}async fetch(t){return await this._ready,this._fetch(t)}async list(t){return await this._ready,this._list(t)}async remove(t){await this._ready,await this._remove(t),this._changed.emit({id:t,type:"remove"})}async save(t,e){await this._ready,await this._save(t,e),this._changed.emit({id:t,type:"save"})}async toJSON(){await this._ready;const{ids:t,values:e}=await this._list();return e.reduce((e,n,r)=>(e[t[r]]=n,e),{})}async _clear(){await Promise.all((await this._list()).ids.map(t=>this._remove(t)))}async _fetch(t){const e=await this._connector.fetch(t);if(e)return JSON.parse(e).v}async _list(t){const{ids:e,values:n}=await this._connector.list(t);return{ids:e,values:n.map(t=>JSON.parse(t).v)}}async _merge(t){await Promise.all(Object.keys(t).map(e=>this._save(e,t[e])))}async _overwrite(t){await this._clear(),await this._merge(t)}async _remove(t){return this._connector.remove(t)}async _save(t,e){return this._connector.save(t,JSON.stringify({v:e}))}}e.StateDB=i,function(t){t.Connector=class{constructor(){this._storage={}}async fetch(t){return this._storage[t]}async list(t=""){return Object.keys(this._storage).reduce((e,n)=>(n&&0===n.indexOf(t)&&(e.ids.push(n),e.values.push(this._storage[n])),e),{ids:[],values:[]})}async remove(t){delete this._storage[t]}async save(t,e){this._storage[t]=e}}}(i=e.StateDB||(e.StateDB={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){const e="𝐚".length>1;t.jsIndexToCharIndex=function(t,n){if(e)return t;let r=t;for(let e=0;e+1<n.length&&e<t;e++){let t=n.charCodeAt(e);if(t>=55296&&t<=56319){let t=n.charCodeAt(e+1);t>=56320&&t<=57343&&(r--,e++)}}return r},t.charIndexToJsIndex=function(t,n){if(e)return t;let r=t;for(let t=0;t+1<n.length&&t<r;t++){let e=n.charCodeAt(t);if(e>=55296&&e<=56319){let e=n.charCodeAt(t+1);e>=56320&&e<=57343&&(r++,t++)}}return r},t.camelCase=function(t,e=!1){return t.replace(/(?:^\w|[A-Z]|\b\w|\s+|-+|_+)/g,function(t,n){return 0==+t||"-"===t[0]?"":0!==n||e?t.toUpperCase():t.toLowerCase()})},t.titleCase=function(t){return t.toLowerCase().split(" ").map(t=>t.charAt(0).toUpperCase()+t.slice(1)).join(" ")}}(e.Text||(e.Text={}))},function(t,e,n){"use strict";var r=this&&this.__importDefault||function(t){return t&&t.__esModule?t:{default:t}};Object.defineProperty(e,"__esModule",{value:!0});const i=r(n(7));!function(t){t.formatHuman=function(t){let e=i.default(t).fromNow();return e="a few seconds ago"===e?"seconds ago":e},t.format=function(t,e="YYYY-MM-DD HH:mm"){return i.default(t).format(e)}}(e.Time||(e.Time={}))},function(t,e,n){var r={"./af":440,"./af.js":440,"./ar":441,"./ar-dz":442,"./ar-dz.js":442,"./ar-kw":443,"./ar-kw.js":443,"./ar-ly":444,"./ar-ly.js":444,"./ar-ma":445,"./ar-ma.js":445,"./ar-sa":446,"./ar-sa.js":446,"./ar-tn":447,"./ar-tn.js":447,"./ar.js":441,"./az":448,"./az.js":448,"./be":449,"./be.js":449,"./bg":450,"./bg.js":450,"./bm":451,"./bm.js":451,"./bn":452,"./bn.js":452,"./bo":453,"./bo.js":453,"./br":454,"./br.js":454,"./bs":455,"./bs.js":455,"./ca":456,"./ca.js":456,"./cs":457,"./cs.js":457,"./cv":458,"./cv.js":458,"./cy":459,"./cy.js":459,"./da":460,"./da.js":460,"./de":461,"./de-at":462,"./de-at.js":462,"./de-ch":463,"./de-ch.js":463,"./de.js":461,"./dv":464,"./dv.js":464,"./el":465,"./el.js":465,"./en-SG":466,"./en-SG.js":466,"./en-au":467,"./en-au.js":467,"./en-ca":468,"./en-ca.js":468,"./en-gb":469,"./en-gb.js":469,"./en-ie":470,"./en-ie.js":470,"./en-il":471,"./en-il.js":471,"./en-nz":472,"./en-nz.js":472,"./eo":473,"./eo.js":473,"./es":474,"./es-do":475,"./es-do.js":475,"./es-us":476,"./es-us.js":476,"./es.js":474,"./et":477,"./et.js":477,"./eu":478,"./eu.js":478,"./fa":479,"./fa.js":479,"./fi":480,"./fi.js":480,"./fo":481,"./fo.js":481,"./fr":482,"./fr-ca":483,"./fr-ca.js":483,"./fr-ch":484,"./fr-ch.js":484,"./fr.js":482,"./fy":485,"./fy.js":485,"./ga":486,"./ga.js":486,"./gd":487,"./gd.js":487,"./gl":488,"./gl.js":488,"./gom-latn":489,"./gom-latn.js":489,"./gu":490,"./gu.js":490,"./he":491,"./he.js":491,"./hi":492,"./hi.js":492,"./hr":493,"./hr.js":493,"./hu":494,"./hu.js":494,"./hy-am":495,"./hy-am.js":495,"./id":496,"./id.js":496,"./is":497,"./is.js":497,"./it":498,"./it-ch":499,"./it-ch.js":499,"./it.js":498,"./ja":500,"./ja.js":500,"./jv":501,"./jv.js":501,"./ka":502,"./ka.js":502,"./kk":503,"./kk.js":503,"./km":504,"./km.js":504,"./kn":505,"./kn.js":505,"./ko":506,"./ko.js":506,"./ku":507,"./ku.js":507,"./ky":508,"./ky.js":508,"./lb":509,"./lb.js":509,"./lo":510,"./lo.js":510,"./lt":511,"./lt.js":511,"./lv":512,"./lv.js":512,"./me":513,"./me.js":513,"./mi":514,"./mi.js":514,"./mk":515,"./mk.js":515,"./ml":516,"./ml.js":516,"./mn":517,"./mn.js":517,"./mr":518,"./mr.js":518,"./ms":519,"./ms-my":520,"./ms-my.js":520,"./ms.js":519,"./mt":521,"./mt.js":521,"./my":522,"./my.js":522,"./nb":523,"./nb.js":523,"./ne":524,"./ne.js":524,"./nl":525,"./nl-be":526,"./nl-be.js":526,"./nl.js":525,"./nn":527,"./nn.js":527,"./pa-in":528,"./pa-in.js":528,"./pl":529,"./pl.js":529,"./pt":530,"./pt-br":531,"./pt-br.js":531,"./pt.js":530,"./ro":532,"./ro.js":532,"./ru":533,"./ru.js":533,"./sd":534,"./sd.js":534,"./se":535,"./se.js":535,"./si":536,"./si.js":536,"./sk":537,"./sk.js":537,"./sl":538,"./sl.js":538,"./sq":539,"./sq.js":539,"./sr":540,"./sr-cyrl":541,"./sr-cyrl.js":541,"./sr.js":540,"./ss":542,"./ss.js":542,"./sv":543,"./sv.js":543,"./sw":544,"./sw.js":544,"./ta":545,"./ta.js":545,"./te":546,"./te.js":546,"./tet":547,"./tet.js":547,"./tg":548,"./tg.js":548,"./th":549,"./th.js":549,"./tl-ph":550,"./tl-ph.js":550,"./tlh":551,"./tlh.js":551,"./tr":552,"./tr.js":552,"./tzl":553,"./tzl.js":553,"./tzm":554,"./tzm-latn":555,"./tzm-latn.js":555,"./tzm.js":554,"./ug-cn":556,"./ug-cn.js":556,"./uk":557,"./uk.js":557,"./ur":558,"./ur.js":558,"./uz":559,"./uz-latn":560,"./uz-latn.js":560,"./uz.js":559,"./vi":561,"./vi.js":561,"./x-pseudo":562,"./x-pseudo.js":562,"./yo":563,"./yo.js":563,"./zh-cn":564,"./zh-cn.js":564,"./zh-hk":565,"./zh-hk.js":565,"./zh-tw":566,"./zh-tw.js":566};function i(t){var e=o(t);return n(e)}function o(t){if(!n.o(r,t)){var e=new Error("Cannot find module '"+t+"'");throw e.code="MODULE_NOT_FOUND",e}return r[t]}i.keys=function(){return Object.keys(r)},i.resolve=o,t.exports=i,i.id=904},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9);e.ISettingRegistry=new r.Token("@jupyterlab/coreutils:ISettingRegistry"),e.IStateDB=new r.Token("@jupyterlab/coreutils:IStateDB")},function(t,e,n){var r=n(907);"string"==typeof r&&(r=[[t.i,r,""]]);var i={hmr:!0,transform:void 0,insertInto:void 0};n(287)(r,i);r.locals&&(t.exports=r.locals)},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(908),""),e.i(n(909),""),e.i(n(910),""),e.i(n(1003),""),e.i(n(286),""),e.i(n(285),""),e.i(n(569),""),e.i(n(1022),""),e.push([t.i,'/* Taken from the Dashbook proposal mockup: https://codepen.io/anon/pen/LmMzNv?editors=1100 */\nbody {\n  padding: 0;\n  margin: 0;\n  font-family: "Segoe UI", "sans-serif";\n  background-color: white;\n}\nmainapp > .m-Viewer {\n  position: absolute;\n  left: 0;\n  right: 0;\n  top: 0;\n  bottom: 0;\n}\n.m-Viewer[data-display="compressed"] .m-OutputPanel-Wrapper {\n  padding: 0;\n}\n.m-Viewer[data-display="compressed"] .m-OutputPanel-Wrapper .m-OutputPanel {\n  box-shadow: unset;\n}\n.m-OutputPanel-Wrapper {\n  padding: 0.5em;\n}\n.m-OutputPanel {\n  box-shadow: 3px 3px 0.25em 0 rgba(0, 0, 0, 0.5);\n}\n.maven_toolbar {\n  background: #65c1e8;\n  height: 1.875em;\n  color: white;\n  line-height: 1.875em;\n  padding: 0 0.5em;\n}\n.maven_toolbar .kernel_status {\n  float: right;\n}\n.maven_toolbar.state_kernel_busy {\n  background: #f6b731;\n}\n.maven_toolbar.state_kernel_error {\n  background: #bc2c1a;\n}\n.m-StatusToolbar-Overlay {\n  color: white;\n  font-size: 3em;\n  position: absolute;\n  left: 0;\n  top: 0;\n  right: 0;\n  bottom: 0;\n  background-color: rgba(0, 0, 0, 0.5);\n  text-align: center;\n  vertical-align: middle;\n  line-height: 100vh;\n}\n.m-RenderedLayout {\n  height: 100%;\n}\n',""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,'/*-----------------------------------------------------------------------------\r\n| Copyright (c) Jupyter Development Team.\r\n| Distributed under the terms of the Modified BSD License.\r\n|----------------------------------------------------------------------------*/\r\n\r\n/*\r\nThe following CSS variables define the main, public API for styling JupyterLab.\r\nThese variables should be used by all plugins wherever possible. In other\r\nwords, plugins should not define custom colors, sizes, etc unless absolutely\r\nnecessary. This enables users to change the visual theme of JupyterLab\r\nby changing these variables.\r\nMany variables appear in an ordered sequence (0,1,2,3). These sequences\r\nare designed to work well together, so for example, `--jp-border-color1` should\r\nbe used with `--jp-layout-color1`. The numbers have the following meanings:\r\n* 0: super-primary, reserved for special emphasis\r\n* 1: primary, most important under normal situations\r\n* 2: secondary, next most important under normal situations\r\n* 3: tertiary, next most important under normal situations\r\nThroughout JupyterLab, we are mostly following principles from Google\'s\r\nMaterial Design when selecting colors. We are not, however, following\r\nall of MD as it is not optimized for dense, information rich UIs.\r\n*/\r\n\r\n\r\n:root {\r\n\r\n    /* Elevation\r\n     *\r\n     * We style box-shadows using Material Design\'s idea of elevation. These particular numbers are taken from here:\r\n     *\r\n     * https://github.com/material-components/material-components-web\r\n     * https://material-components-web.appspot.com/elevation.html\r\n     */\r\n\r\n    --jp-shadow-base-lightness: 0;\r\n    --jp-shadow-umbra-color: rgba(var(--jp-shadow-base-lightness), var(--jp-shadow-base-lightness), var(--jp-shadow-base-lightness), 0.2);\r\n    --jp-shadow-penumbra-color: rgba(var(--jp-shadow-base-lightness), var(--jp-shadow-base-lightness), var(--jp-shadow-base-lightness), 0.14);\r\n    --jp-shadow-ambient-color: rgba(var(--jp-shadow-base-lightness), var(--jp-shadow-base-lightness), var(--jp-shadow-base-lightness), 0.12);\r\n    --jp-elevation-z0: none;\r\n    --jp-elevation-z1: 0px 2px 1px -1px var(--jp-shadow-umbra-color),\r\n    0px 1px 1px 0px var(--jp-shadow-penumbra-color),\r\n    0px 1px 3px 0px var(--jp-shadow-ambient-color);\r\n    --jp-elevation-z2: 0px 3px 1px -2px var(--jp-shadow-umbra-color),\r\n    0px 2px 2px 0px var(--jp-shadow-penumbra-color),\r\n    0px 1px 5px 0px var(--jp-shadow-ambient-color);\r\n    --jp-elevation-z4: 0px 2px 4px -1px var(--jp-shadow-umbra-color),\r\n    0px 4px 5px 0px var(--jp-shadow-penumbra-color),\r\n    0px 1px 10px 0px var(--jp-shadow-ambient-color);\r\n    --jp-elevation-z6: 0px 3px 5px -1px var(--jp-shadow-umbra-color),\r\n    0px 6px 10px 0px var(--jp-shadow-penumbra-color),\r\n    0px 1px 18px 0px var(--jp-shadow-ambient-color);\r\n    --jp-elevation-z8: 0px 5px 5px -3px var(--jp-shadow-umbra-color),\r\n    0px 8px 10px 1px var(--jp-shadow-penumbra-color),\r\n    0px 3px 14px 2px var(--jp-shadow-ambient-color);\r\n    --jp-elevation-z12: 0px 7px 8px -4px var(--jp-shadow-umbra-color),\r\n    0px 12px 17px 2px var(--jp-shadow-penumbra-color),\r\n    0px 5px 22px 4px var(--jp-shadow-ambient-color);\r\n    --jp-elevation-z16: 0px 8px 10px -5px var(--jp-shadow-umbra-color),\r\n    0px 16px 24px 2px var(--jp-shadow-penumbra-color),\r\n    0px 6px 30px 5px var(--jp-shadow-ambient-color);\r\n    --jp-elevation-z20: 0px 10px 13px -6px var(--jp-shadow-umbra-color),\r\n    0px 20px 31px 3px var(--jp-shadow-penumbra-color),\r\n    0px 8px 38px 7px var(--jp-shadow-ambient-color);\r\n    --jp-elevation-z24: 0px 11px 15px -7px var(--jp-shadow-umbra-color),\r\n    0px 24px 38px 3px var(--jp-shadow-penumbra-color),\r\n    0px 9px 46px 8px var(--jp-shadow-ambient-color);\r\n\r\n    /* Borders\r\n     *\r\n     * The following variables, specify the visual styling of borders in JupyterLab.\r\n     */\r\n\r\n    --jp-border-width: 1px;\r\n    --jp-border-color0: var(--md-grey-400);\r\n    --jp-border-color1: var(--md-grey-400);\r\n    --jp-border-color2: var(--md-grey-300);\r\n    --jp-border-color3: var(--md-grey-200);\r\n    --jp-border-radius: 2px;\r\n\r\n    /* UI Fonts\r\n     *\r\n     * The UI font CSS variables are used for the typography all of the JupyterLab\r\n     * user interface elements that are not directly user generated content.\r\n     *\r\n     * The font sizing here is done assuming that the body font size of --jp-ui-font-size1\r\n     * is applied to a parent element. When children elements, such as headings, are sized\r\n     * in em all things will be computed relative to that body size.\r\n     */\r\n\r\n    --jp-ui-font-scale-factor: 1.2;\r\n    --jp-ui-font-size0: 0.83333em;\r\n    --jp-ui-font-size1: 13px; /* Base font size */\r\n    --jp-ui-font-size2: 1.2em;\r\n    --jp-ui-font-size3: 1.44em;\r\n\r\n    --jp-ui-font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";\r\n\r\n    /*\r\n     * Use these font colors against the corresponding main layout colors.\r\n     * In a light theme, these go from dark to light.\r\n     */\r\n\r\n    /* Defaults use Material Design specification */\r\n    --jp-ui-font-color0: rgba(0,0,0,1.0);\r\n    --jp-ui-font-color1: rgba(0,0,0,0.87);\r\n    --jp-ui-font-color2: rgba(0,0,0,0.54);\r\n    --jp-ui-font-color3: rgba(0,0,0,0.38);\r\n\r\n    /*\r\n     * Use these against the brand/accent/warn/error colors.\r\n     * These will typically go from light to darker, in both a dark and light theme.\r\n     */\r\n\r\n    --jp-ui-inverse-font-color0: rgba(255,255,255,1.0);\r\n    --jp-ui-inverse-font-color1: rgba(255,255,255,1.0);\r\n    --jp-ui-inverse-font-color2: rgba(255,255,255,0.7);\r\n    --jp-ui-inverse-font-color3: rgba(255,255,255,0.5);\r\n\r\n    /* Content Fonts\r\n     *\r\n     * Content font variables are used for typography of user generated content.\r\n     *\r\n     * The font sizing here is done assuming that the body font size of --jp-content-font-size1\r\n     * is applied to a parent element. When children elements, such as headings, are sized\r\n     * in em all things will be computed relative to that body size.\r\n     */\r\n\r\n    --jp-content-line-height: 1.6;\r\n    --jp-content-font-scale-factor: 1.2;\r\n    --jp-content-font-size0: 0.83333em;\r\n    --jp-content-font-size1: 14px; /* Base font size */\r\n    --jp-content-font-size2: 1.2em;\r\n    --jp-content-font-size3: 1.44em;\r\n    --jp-content-font-size4: 1.728em;\r\n    --jp-content-font-size5: 2.0736em;\r\n\r\n    /* This gives a magnification of about 125% in presentation mode over normal. */\r\n    --jp-content-presentation-font-size1: 17px;\r\n\r\n    --jp-content-heading-line-height: 1.0;\r\n    --jp-content-heading-margin-top: 1.2em;\r\n    --jp-content-heading-margin-bottom: 0.8em;\r\n    --jp-content-heading-font-weight: 500;\r\n\r\n    /* Defaults use Material Design specification */\r\n    --jp-content-font-color0: rgba(0,0,0,1.0);\r\n    --jp-content-font-color1: rgba(0,0,0,0.87);\r\n    --jp-content-font-color2: rgba(0,0,0,0.54);\r\n    --jp-content-font-color3: rgba(0,0,0,0.38);\r\n\r\n    --jp-content-link-color: var(--md-blue-700);\r\n\r\n    --jp-content-font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";\r\n\r\n    /*\r\n     * Code Fonts\r\n     *\r\n     * Code font variables are used for typography of code and other monospaces content.\r\n     */\r\n\r\n    --jp-code-font-size: 13px;\r\n    --jp-code-line-height: 1.3077; /* 17px for 13px base */\r\n    --jp-code-padding: 0.385em; /* 5px for 13px base */\r\n    --jp-code-font-family: \'Source Code Pro\', monospace;\r\n\r\n    /* This gives a magnification of about 125% in presentation mode over normal. */\r\n    --jp-code-presentation-font-size: 16px;\r\n\r\n    /* Layout\r\n     *\r\n     * The following are the main layout colors use in JupyterLab. In a light\r\n     * theme these would go from light to dark.\r\n     */\r\n\r\n    --jp-layout-color0: white;\r\n    --jp-layout-color1: white;\r\n    --jp-layout-color2: var(--md-grey-200);\r\n    --jp-layout-color3: var(--md-grey-400);\r\n    --jp-layout-color4: var(--md-grey-600);\r\n\r\n    /* Brand/accent */\r\n\r\n    --jp-brand-color0: var(--md-blue-700);\r\n    --jp-brand-color1: var(--md-blue-500);\r\n    --jp-brand-color2: var(--md-blue-300);\r\n    --jp-brand-color3: var(--md-blue-100);\r\n\r\n    --jp-accent-color0: var(--md-green-700);\r\n    --jp-accent-color1: var(--md-green-500);\r\n    --jp-accent-color2: var(--md-green-300);\r\n    --jp-accent-color3: var(--md-green-100);\r\n\r\n    /* State colors (warn, error, success, info) */\r\n\r\n    --jp-warn-color0: var(--md-orange-700);\r\n    --jp-warn-color1: var(--md-orange-500);\r\n    --jp-warn-color2: var(--md-orange-300);\r\n    --jp-warn-color3: var(--md-orange-100);\r\n\r\n    --jp-error-color0: var(--md-red-700);\r\n    --jp-error-color1: var(--md-red-500);\r\n    --jp-error-color2: var(--md-red-300);\r\n    --jp-error-color3: var(--md-red-100);\r\n\r\n    --jp-success-color0: var(--md-green-700);\r\n    --jp-success-color1: var(--md-green-500);\r\n    --jp-success-color2: var(--md-green-300);\r\n    --jp-success-color3: var(--md-green-100);\r\n\r\n    --jp-info-color0: var(--md-cyan-700);\r\n    --jp-info-color1: var(--md-cyan-500);\r\n    --jp-info-color2: var(--md-cyan-300);\r\n    --jp-info-color3: var(--md-cyan-100);\r\n\r\n    /* Cell specific styles */\r\n\r\n    --jp-cell-padding: 5px;\r\n\r\n    --jp-cell-collapser-width: 8px;\r\n    --jp-cell-collapser-min-height: 20px;\r\n    --jp-cell-collapser-not-active-hover-opacity: 0.6;\r\n\r\n    --jp-cell-editor-background: var(--md-grey-100);\r\n    --jp-cell-editor-border-color: var(--md-grey-300);\r\n    --jp-cell-editor-box-shadow: inset 0 0 2px var(--md-blue-300);\r\n    --jp-cell-editor-active-background: var(--jp-layout-color0);\r\n    --jp-cell-editor-active-border-color: var(--jp-brand-color1);\r\n\r\n    --jp-cell-prompt-width: 90px;\r\n    --jp-cell-prompt-font-family: \'Source Code Pro\', monospace;\r\n    --jp-cell-prompt-letter-spacing: 0px;\r\n    --jp-cell-prompt-opacity: 1.0;\r\n    --jp-cell-prompt-not-active-opacity: 0.5;\r\n    --jp-cell-prompt-not-active-font-color: var(--md-grey-700);\r\n    /* A custom blend of MD grey and blue 600\r\n     * See https://meyerweb.com/eric/tools/color-blend/#546E7A:1E88E5:5:hex */\r\n    --jp-cell-inprompt-font-color: #307FC1;\r\n    /* A custom blend of MD grey and orange 600\r\n     * https://meyerweb.com/eric/tools/color-blend/#546E7A:F4511E:5:hex */\r\n    --jp-cell-outprompt-font-color: #BF5B3D;\r\n\r\n    /* Notebook specific styles */\r\n\r\n    --jp-notebook-padding: 10px;\r\n    --jp-notebook-scroll-padding: 100px;\r\n    --jp-notebook-select-background: var(--jp-layout-color1);\r\n    --jp-notebook-multiselected-color: var(--md-blue-50);\r\n\r\n    /* Rendermime styles */\r\n\r\n    --jp-rendermime-error-background: #fdd;\r\n    --jp-rendermime-table-row-background: var(--md-grey-100);\r\n    --jp-rendermime-table-row-hover-background: var(--md-light-blue-50);\r\n\r\n    /* Dialog specific styles */\r\n\r\n    --jp-dialog-background: rgba(0,0,0,.25);\r\n\r\n    /* Console specific styles */\r\n\r\n    --jp-console-padding: 10px;\r\n\r\n    /* Toolbar specific styles */\r\n\r\n    --jp-toolbar-border-color: var(--jp-border-color1);\r\n    --jp-toolbar-micro-height: 8px;\r\n    --jp-toolbar-background: var(--jp-layout-color1);\r\n    --jp-toolbar-box-shadow: 0px 0px 2px 0px rgba(0,0,0,0.24);\r\n    --jp-toolbar-header-margin: 4px 4px 0px 4px;\r\n    --jp-toolbar-active-background: var(--md-grey-300);\r\n\r\n    /* Input field styles */\r\n\r\n    --jp-input-box-shadow: inset 0 0 2px var(--md-blue-300);\r\n    --jp-input-active-background: var(--jp-layout-color1);\r\n    --jp-input-hover-background: var(--jp-layout-color1);\r\n    --jp-input-background: var(--md-grey-100);\r\n    --jp-input-border-color: var(--jp-border-color1);\r\n    --jp-input-active-border-color: var(--jp-brand-color1);\r\n\r\n    /* General editor styles */\r\n\r\n    --jp-editor-selected-background: #d9d9d9;\r\n    --jp-editor-selected-focused-background: #d7d4f0;\r\n    --jp-editor-cursor-color: var(--jp-ui-font-color0);\r\n\r\n    /* Code mirror specific styles */\r\n\r\n    --jp-mirror-editor-keyword-color: #008000;\r\n    --jp-mirror-editor-atom-color: #88F;\r\n    --jp-mirror-editor-number-color: #080;\r\n    --jp-mirror-editor-def-color: #00f;\r\n    --jp-mirror-editor-variable-color: var(--md-grey-900);\r\n    --jp-mirror-editor-variable-2-color: var(--md-grey-800);\r\n    --jp-mirror-editor-variable-3-color: var(--md-grey-700);\r\n    --jp-mirror-editor-punctuation-color: #05a;\r\n    --jp-mirror-editor-property-color: #05a;\r\n    --jp-mirror-editor-operator-color: #AA22FF;\r\n    --jp-mirror-editor-comment-color: #408080;\r\n    --jp-mirror-editor-string-color: #BA2121;\r\n    --jp-mirror-editor-string-2-color: #f50;\r\n    --jp-mirror-editor-meta-color: #AA22FF;\r\n    --jp-mirror-editor-qualifier-color: #555;\r\n    --jp-mirror-editor-builtin-color: #008000;\r\n    --jp-mirror-editor-bracket-color: #997;\r\n    --jp-mirror-editor-tag-color: #170;\r\n    --jp-mirror-editor-attribute-color: #00c;\r\n    --jp-mirror-editor-header-color: blue;\r\n    --jp-mirror-editor-quote-color: #090;\r\n    --jp-mirror-editor-link-color: #00c;\r\n    --jp-mirror-editor-error-color: #f00;\r\n    --jp-mirror-editor-hr-color: #999;\r\n\r\n    /* Vega extension styles */\r\n\r\n    --jp-vega-background: white;\r\n\r\n}',""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/* This is a set of overrides for variables defined in ./jupyter-vars.css. This is to wrangle the layout into something\r\n * that's pretty.\r\n */\r\n\r\n:root {\r\n    --jp-cell-prompt-width: 0;\r\n}\r\n\r\n.jp-Dialog-content {\r\n    max-width: 98vw;\r\n    max-height: 98vh;\r\n}\r\n",""])},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(911),""),e.i(n(927),""),e.i(n(935),""),e.i(n(996),""),e.i(n(997),""),e.i(n(998),""),e.i(n(1002),""),e.push([t.i,"@font-face {\n  font-family: 'Material Icons';\n  font-style: normal;\n  font-weight: 400;\n  src: url(https://fonts.gstatic.com/s/materialicons/v48/flUhRq6tzZclQEJ-Vdg-IuiaDsNZ.ttf) format('truetype');\n}\n.material-icons {\n  font-family: 'Material Icons';\n  font-weight: normal;\n  font-style: normal;\n  font-size: 24px;\n  line-height: 1;\n  letter-spacing: normal;\n  text-transform: none;\n  display: inline-block;\n  white-space: nowrap;\n  word-wrap: normal;\n  direction: ltr;\n}\n.m-RenderedLayout {\n  min-height: 500px;\n  width: 100%;\n}\n/* Color is MD-500 Light Blue\n * Inspired by a false color image of polar auras on Jupiter,\n * showing influence from three of it's many moons:\n * https://www.jpl.nasa.gov/spaceimages/details.php?id=PIA03155 */\n.m-PanelIcon {\n  font-size: 14px;\n  line-height: 24px;\n  color: var(--md-light-blue-500);\n}\n.m-FileIcon {\n  font-size: 18px;\n  color: var(--md-light-blue-500);\n  width: 20px;\n  height: 17px;\n  text-align: center;\n  vertical-align: middle;\n}\n.jp-mod-selected > .m-FileIcon {\n  color: var(--md-grey-50);\n}\n.jp-mod-running > .m-FileIcon {\n  position: relative;\n  left: -4px;\n}\n.jp-mod-running > .m-FileIcon::before {\n  /* The !important is needed to reliably override the file picker styles\n     * -4px = super - 2\n     * -6px = super + 2 (also expressed as super[bottom] - 2) */\n  left: -4px !important;\n  top: -6px !important;\n}\n.m-LauncherIcon::after {\n  font-size: 52px;\n  color: var(--md-light-blue-500);\n}\n.m-LauncherIcon.m-Icon-Dashboard::after {\n  content: \"dashboard\";\n}\n.m-LauncherIcon.m-Icon-Part::after {\n  content: \"insert_chart\";\n}\n/* Blueprint override override override (...override?) */\n.jp-ToolbarButton > .fa.minimal {\n  color: black !important;\n  padding: 0px 6px !important;\n}\n.jp-MainAreaWidget.m-DashboardEditor .jp-ToolbarButtonComponent .jp-ToolbarButtonComponent-label {\n  display: none;\n}\n",""])},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(912),""),e.i(n(920),""),e.i(n(921),""),e.i(n(922),""),e.i(n(923),""),e.i(n(924),""),e.i(n(925),""),e.i(n(926),""),e.push([t.i,'.m-TreeView {\n  padding-right: 3px;\n}\n.m-TreeView > .m-TreeView-Level > .m-TreeView-Node {\n  margin-left: 0;\n}\n.m-TreeView > .m-TreeView-Level > .m-TreeView-Node[data-children="0"] {\n  margin-left: 16px;\n}\n.m-TreeView .m-TreeView-Level {\n  padding-inline-start: 0px;\n  list-style-type: none;\n  margin-block-start: 0;\n  margin-block-end: 0;\n}\n.m-TreeView .m-TreeView-Node-row {\n  display: inline-block;\n  width: 100%;\n  height: 24px;\n  padding: 0;\n  padding-left: 3px;\n  user-select: none;\n}\n.m-TreeView .m-TreeView-Node-row .m-TreeView-Node-content {\n  overflow: hidden;\n}\n.m-TreeView .m-TreeView-Node-row:hover:not(.m-selected) {\n  background: var(--m-inactive-ui-color);\n  color: var(--m-inactive-font-color);\n}\n.m-TreeView .m-TreeView-Node-row:hover.m-selected {\n  background: var(--m-hover-selected-ui-color);\n}\n.m-TreeView .m-TreeView-Node-row.m-selected {\n  background: var(--m-selected-ui-color);\n  color: var(--m-active-font-color);\n}\n.m-TreeView .m-TreeView-Node-row.m-selected .m-FlippyTriangle {\n  border-left-color: var(--m-active-font-color);\n}\n.m-TreeView .m-TreeView-Node {\n  margin-left: 20px;\n}\n.m-TreeView .m-TreeView-Node[data-children="0"] {\n  margin-left: 36px;\n}\n.m-TreeView .m-TreeView-Node[data-preview-validity="valid"] {\n  border-color: var(--m-valid-ui-color);\n}\n.m-TreeView .m-TreeView-Node[data-preview-validity="invalid"] {\n  border-color: var(--m-warn-ui-color);\n}\n.m-TreeView .m-TreeView-Node[data-preview="insert-above"] {\n  border-top-width: 2px;\n  border-top-style: solid;\n}\n.m-TreeView .m-TreeView-Node[data-preview="insert-below"] {\n  border-bottom-width: 2px;\n  border-bottom-style: solid;\n}\n.m-TreeView .m-TreeView-Node[data-preview="insert-into"] > .m-TreeView-Node-row::after {\n  content: "...";\n  padding-left: 3px;\n  height: 24px;\n  margin-left: 36px;\n  display: block;\n  color: var(--m-active-font-color);\n}\n.m-TreeView .m-TreeView-Node[data-preview="insert-into"][data-preview-validity="valid"] > .m-TreeView-Node-row::after {\n  background: var(--m-valid-ui-color);\n}\n.m-TreeView .m-TreeView-Node[data-preview="insert-into"][data-preview-validity="invalid"] > .m-TreeView-Node-row::after {\n  background: var(--m-warn-ui-color);\n}\n.m-TreeList-drag {\n  background: var(--m-active-ui-color);\n  color: var(--m-active-font-color);\n}\n.p-Widget.m-Hover {\n  position: absolute;\n  background-color: white;\n  z-index: 3;\n  /* create a new stacking context and move this to the top of it */\n}\n.p-Widget.m-Hover[data-mode="dialog"] {\n  box-shadow: var(--jp-elevation-z12);\n}\n.m-HoverManager {\n  z-index: 3;\n  position: absolute;\n  isolation: isolate;\n  width: 100%;\n  height: 100%;\n  pointer-events: none;\n}\n.m-HoverManager > * {\n  pointer-events: auto;\n}\n.m-Dialog-titlebar {\n  background-color: var(--jp-layout-color4);\n  border-bottom: 1px solid var(--m-border-color, lightgrey);\n  color: var(--m-active-font-color, white);\n  user-select: none;\n  font-size: 12pt;\n}\n.m-Dialog-titlebar span {\n  pointer-events: none;\n  padding-left: 5px;\n}\n.m-Dialog-titlebar .m-Button {\n  border: none;\n  background: none;\n  padding: 0;\n  border-radius: 100px;\n  color: var(--m-active-font-color, white);\n}\n.m-Dialog-titlebar .m-Button .m-Button:hover {\n  background-color: var(--m-hover-active-ui-color, grey);\n}\n.m-Dialog-Btn button {\n  border: none;\n  background-color: var(--m-neutral-ui-color);\n  color: var(--m-active-font-color, white);\n}\n.m-Dialog-Btn button:hover {\n  background-color: var(--m-hover-neutral-ui-color);\n}\n.m-Dialog-Btn button:focus {\n  border: 5px solid var(--m-hover-neutral-ui-color);\n}\n.m-Dialog-Btn[data-type="accept"] button {\n  background-color: var(--m-valid-ui-color);\n}\n.m-Dialog-Btn[data-type="accept"] button:hover {\n  background-color: var(--m-hover-valid-ui-color);\n}\n.m-Dialog-Btn[data-type="accept"] button:focus {\n  border-color: var(--m-hover-valid-ui-color);\n}\n.m-Dialog-Btn[data-type="warn"] button {\n  background-color: var(--m-warn-ui-color);\n}\n.m-Dialog-Btn[data-type="warn"] button:hover {\n  background-color: var(--m-hover-warn-ui-color);\n}\n.m-Dialog-Btn[data-type="warn"] button:focus {\n  border-color: var(--m-hover-valid-ui-color);\n}\n.m-Hover:not([data-mode="dialog"]) .p-Widget {\n  width: 100%;\n  height: 100%;\n  position: relative;\n}\nbody {\n  height: 100vh;\n}\n.m-rw-inline-hack .rw-widget-container {\n  border-radius: 0;\n}\n.m-rw-inline-hack .rw-widget-picker {\n  height: unset;\n  display: flex;\n  flex-direction: row;\n}\n.m-rw-inline-hack .rw-widget-picker > .rw-select {\n  flex: 0 0 1.9em;\n}\n.m-rw-inline-hack .rw-widget-picker > .rw-select > * {\n  height: 15px;\n  top: -6px;\n}\n.m-rw-inline-hack .rw-widget-picker .rw-select:not(.rw-select-bordered) {\n  cursor: pointer;\n  border: none;\n  border-left: #ccc 1px solid;\n}\n.m-rw-inline-hack .rw-widget-picker > input.rw-input {\n  width: 0px;\n  flex: 1 1;\n}\n.m-rw-inline-hack .rw-widget-picker > div.rw-input {\n  max-width: unset;\n}\n',""])},function(t,e,n){var r=n(172);(t.exports=n(12)(!1)).push([t.i,'.rw-btn,.rw-input-reset,.rw-input,.rw-dropdown-list-autofill,.rw-filter-input{color:inherit;padding:0;margin:0;border:none;box-shadow:none;background:none;background-image:none;font-family:inherit;font-size:inherit;line-height:inherit;-ms-touch-action:manipulation;touch-action:manipulation}.rw-btn::-moz-focus-inner{padding:0;border:0}select.rw-input{text-transform:none}html input[type="button"].rw-input{-webkit-appearance:button;cursor:pointer}textarea.rw-input{overflow:auto;resize:vertical}button[disabled].rw-input,fieldset[disabled] .rw-input,html input[disabled].rw-input{cursor:not-allowed}button.rw-input::-moz-focus-inner,input.rw-input::-moz-focus-inner{border:0;padding:0}input[type="checkbox"],input[type="radio"]{box-sizing:border-box;padding:0}@font-face{font-family:\'RwWidgets\';font-weight:normal;font-style:normal;src:url('+r(n(913))+");src:url("+r(n(914))+"?#iefix&v=4.1.0) format('embedded-opentype'),url("+r(n(915))+") format('woff'),url("+r(n(916))+") format('truetype'),url("+r(n(917))+"#fontawesomeregular) format('svg')}.rw-i{display:inline-block;color:inherit;font-family:RwWidgets;font-style:normal;font-weight:normal;font-variant:normal;text-transform:none;-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased}.rw-i-caret-down:before{content:'\\E803'}.rw-i-caret-up:before{content:'\\E800'}.rw-i-chevron-left:before{content:'\\F104'}.rw-i-chevron-right:before{content:'\\F105'}.rw-i-clock-o:before{content:'\\E805'}.rw-i-calendar:before{content:'\\E804'}.rw-i-search:before{content:'\\E801'}.rw-btn{position:relative;color:#333;display:inline-block;text-align:center;vertical-align:middle;border:1px solid transparent;cursor:pointer;outline:none}.rw-state-readonly .rw-btn,.rw-state-disabled .rw-btn{cursor:not-allowed}.rw-btn-select{opacity:.75;transition:opacity 150ms ease-in}.rw-btn-select:hover,.rw-state-focus .rw-btn-select,:hover>.rw-btn-select{opacity:1}.rw-btn-primary{width:100%;white-space:normal;line-height:2em}.rw-btn-primary:hover{background-color:#e6e6e6}.rw-btn-select[disabled],.rw-btn-primary[disabled],fieldset[disabled] .rw-btn-select,fieldset[disabled] .rw-btn-primary{box-shadow:none;cursor:not-allowed;opacity:.65;pointer-events:none}.rw-sr{position:absolute;width:1px;height:1px;margin:-1px;padding:0;overflow:hidden;clip:rect(0, 0, 0, 0);border:0}.rw-widget{background-clip:border-box;border:none;color:#333;font-size:1em;font-family:inherit;outline:none;position:relative}.rw-widget,.rw-widget *{box-sizing:border-box}.rw-widget:before,.rw-widget *:before,.rw-widget:after,.rw-widget *:after{box-sizing:border-box}.rw-widget>.rw-widget-container{width:100%;margin:0}.rw-widget-container{background-color:#fff;border:#ccc 1px solid;border-radius:4px}.rw-widget-container.rw-state-focus,.rw-state-focus>.rw-widget-container,.rw-widget-container.rw-state-focus:hover,.rw-state-focus>.rw-widget-container:hover{background-color:#fff;border-color:#66afe9;box-shadow:0 0 8px rgba(102,175,233,0.6)}.rw-widget-container.rw-state-readonly,.rw-state-readonly>.rw-widget-container{cursor:not-allowed}.rw-widget-container.rw-state-disabled,.rw-state-disabled>.rw-widget-container,fieldset[disabled] .rw-widget-container,.rw-widget-container.rw-state-disabled:hover,.rw-state-disabled>.rw-widget-container:hover,fieldset[disabled] .rw-widget-container:hover,.rw-widget-container.rw-state-disabled:active,.rw-state-disabled>.rw-widget-container:active,fieldset[disabled] .rw-widget-container:active{box-shadow:none;cursor:not-allowed}.rw-widget-picker{position:relative;overflow:hidden;border-collapse:separate;display:inline-table;height:2.429em}.rw-widget-picker>*{position:relative;border:none;outline:none;width:100%;height:100%;display:table-cell}.rw-widget-picker>.rw-select{width:1%;white-space:nowrap}.rw-open>.rw-widget-picker{border-bottom-right-radius:0;border-bottom-left-radius:0}.rw-open-up>.rw-widget-picker{border-top-right-radius:0;border-top-left-radius:0}fieldset[disabled] .rw-widget-picker,.rw-state-disabled>.rw-widget-picker{background-color:#eee}.rw-multiselect>.rw-widget-picker{height:auto}.rw-select{cursor:pointer}.rw-select>*{width:1.9em;height:100%}.rw-state-readonly .rw-select,.rw-state-disabled .rw-select{cursor:not-allowed}.rw-select-bordered{cursor:pointer;border:none;border-left:#ccc 1px solid}.rw-select-bordered:hover,.rw-select-bordered:active{background-color:#e6e6e6}.rw-select-bordered:active{box-shadow:inset 0 3px 5px rgba(0,0,0,0.125)}.rw-state-disabled .rw-select-bordered,.rw-state-readonly .rw-select-bordered,fieldset[disabled] .rw-select-bordered,.rw-state-disabled .rw-select-bordered:hover,.rw-state-readonly .rw-select-bordered:hover,fieldset[disabled] .rw-select-bordered:hover,.rw-state-disabled .rw-select-bordered:active,.rw-state-readonly .rw-select-bordered:active,fieldset[disabled] .rw-select-bordered:active{cursor:not-allowed;background-color:inherit;background-image:none;box-shadow:none}.rw-rtl .rw-select-bordered{border-right:#ccc 1px solid;border-left:none}.rw-rtl{direction:rtl}.rw-input-reset,.rw-input,.rw-dropdown-list-autofill,.rw-filter-input{outline:0}.rw-input-reset::-moz-placeholder{color:#999;opacity:1}.rw-input-reset:-ms-input-placeholder{color:#999}.rw-input-reset::-webkit-input-placeholder{color:#999}.rw-input,.rw-dropdown-list-autofill,.rw-filter-input{color:#555;padding:0 .857em;background-color:#fff}.rw-input[type=\"text\"]::-ms-clear{display:none}.rw-input[disabled],fieldset[disabled] .rw-input{box-shadow:none;cursor:not-allowed;opacity:1;background-color:#eee;border-color:#ccc}.rw-input[readonly]{cursor:not-allowed}.rw-i.rw-loading{display:block;background:url("+r(n(918))+") no-repeat center;min-width:16px;width:1.9em;height:16px}.rw-i.rw-loading:before{content:\"\"}.rw-placeholder{color:#999}.rw-detect-autofill:-webkit-autofill{animation-name:react-widgets-autofill-start;transition:background-color 50000s ease-in-out 0s}.rw-detect-autofill:not(:-webkit-autofill){animation-name:react-widgets-autofill-cancel}.rw-webkit-autofill .rw-widget-container,.rw-input:-webkit-autofill{background-color:#faffbd !important;background-image:none !important;color:#000 !important}.rw-widget-input,.rw-filter-input{box-shadow:inset 0 1px 1px rgba(0,0,0,0.075)}.rw-widget-input.rw-state-focus{box-shadow:0 0 8px rgba(102, 175, 233, 0.6), inset 0 1px 1px rgba(0, 0, 0, 0.075)}.rw-list{margin:0;padding:0;list-style:none;font-size:1em;outline:0;overflow:auto;max-height:200px}.rw-list-option{-ms-user-select:none;user-select:none;color:#333;cursor:pointer;border:1px solid transparent}.rw-list-option.rw-state-focus,.rw-list-option.rw-state-focus:hover{background-color:transparent;border-color:#66afe9;color:#333}.rw-list-option:hover,.rw-list-option:hover.rw-state-focus{background-color:#e6e6e6;border-color:#e6e6e6;color:#333}.rw-list-option.rw-state-selected,.rw-list-option.rw-state-selected:hover{background-color:#337ab7;border-color:#337ab7;color:white}fieldset[disabled] .rw-list-option,.rw-list-option.rw-state-disabled,.rw-list-option.rw-state-readonly{box-shadow:none;cursor:not-allowed;color:#999;opacity:.7}fieldset[disabled] .rw-list-option:hover,.rw-list-option.rw-state-disabled:hover,.rw-list-option.rw-state-readonly:hover{background:none;border-color:transparent}.rw-list-empty,.rw-list-option,.rw-list-optgroup{padding:.143em .75em;outline:0}.rw-list-optgroup{font-weight:bold;padding-top:7px}.rw-list-option-create{border-top:1px #ccc solid}.rw-dropdown-list-autofill{padding:0}.rw-dropdown-list-input{background-color:transparent;vertical-align:middle;padding-right:0;max-width:1px;text-overflow:ellipsis;white-space:nowrap;overflow:hidden}.rw-rtl .rw-dropdown-list-input{padding-right:.857em;padding-left:0}.rw-filter-input{position:relative;margin:4px;padding-right:0}.rw-filter-input .rw-rtl{padding-right:.857em;padding-left:0}.rw-filter-input .rw-select,.rw-filter-input .rw-btn{opacity:.75;cursor:text}.rw-filter-input>.rw-select,.rw-filter-input>.rw-select:active,.rw-filter-input>.rw-select:hover{background:none;cursor:initial;box-shadow:none}.rw-number-picker .rw-btn{cursor:pointer;height:calc(1.2145em - 1px);margin-top:-1px\\9;height:1.2145em\\9;line-height:1.2145em;line-height:calc(1.2145em - 1px);display:block;border:none}.rw-number-picker .rw-btn:hover,.rw-number-picker .rw-btn:active{background-color:#e6e6e6}.rw-number-picker .rw-btn:active{box-shadow:inset 0 3px 5px rgba(0,0,0,0.125)}.rw-state-disabled .rw-number-picker .rw-btn,.rw-state-readonly .rw-number-picker .rw-btn,fieldset[disabled] .rw-number-picker .rw-btn,.rw-state-disabled .rw-number-picker .rw-btn:hover,.rw-state-readonly .rw-number-picker .rw-btn:hover,fieldset[disabled] .rw-number-picker .rw-btn:hover,.rw-state-disabled .rw-number-picker .rw-btn:active,.rw-state-readonly .rw-number-picker .rw-btn:active,fieldset[disabled] .rw-number-picker .rw-btn:active{cursor:not-allowed;background-color:inherit;background-image:none;box-shadow:none}.rw-number-picker .rw-select{vertical-align:middle}.rw-number-picker .rw-select,.rw-number-picker .rw-select:hover,.rw-number-picker .rw-select:active{box-shadow:none}.rw-calendar-popup{right:auto;min-width:0;width:18em}.rw-calendar{border-radius:4px;background-color:#fff;border:#ccc 1px solid;overflow:hidden}.rw-calendar.rw-popup{border-color:#ccc}.rw-calendar-now{font-weight:bold}.rw-calendar-btn-left,.rw-calendar-btn-right{width:12.5%}.rw-calendar-btn-view{width:75%}.rw-calendar-footer{border-top:1px solid #ccc}.rw-calendar-grid{outline:none;height:14.28571429em;table-layout:fixed;border-collapse:separate;border-spacing:0;width:100%;background-color:#fff}.rw-head-cell{text-align:center;border-bottom:1px solid #ccc;padding:.25em}.rw-cell{color:#333;border-radius:4px;cursor:pointer;line-height:normal;text-align:center;border:1px solid transparent;padding:.25em}.rw-cell:hover{background-color:#e6e6e6;border-color:#e6e6e6;color:#333}.rw-cell.rw-state-focus,.rw-cell.rw-state-focus:hover{background-color:transparent;border-color:#66afe9;color:#333}.rw-cell.rw-state-selected,.rw-cell.rw-state-selected:hover{background-color:#337ab7;border-color:#337ab7;color:white}.rw-cell.rw-state-disabled{color:#999;opacity:.7}.rw-cell.rw-state-disabled:hover{background:none;border-color:transparent}.rw-calendar-month .rw-cell{text-align:center}.rw-cell-off-range{color:#999}.rw-calendar-transition-group{position:relative}.rw-calendar-transition{transition:transform 300ms;overflow:hidden}.rw-calendar-transition-top{-ms-transform:translateY(-100%);transform:translateY(-100%)}.rw-calendar-transition-bottom{-ms-transform:translateY(100%);transform:translateY(100%)}.rw-calendar-transition-right{-ms-transform:translateX(-100%);transform:translateX(-100%)}.rw-calendar-transition-left{-ms-transform:translateX(100%);transform:translateX(100%)}.rw-calendar-transition-entering.rw-calendar-transition-top,.rw-calendar-transition-entered.rw-calendar-transition-top,.rw-calendar-transition-entering.rw-calendar-transition-bottom,.rw-calendar-transition-entered.rw-calendar-transition-bottom{-ms-transform:translateY(0);transform:translateY(0)}.rw-calendar-transition-entering.rw-calendar-transition-right,.rw-calendar-transition-entered.rw-calendar-transition-right,.rw-calendar-transition-entering.rw-calendar-transition-left,.rw-calendar-transition-entered.rw-calendar-transition-left{-ms-transform:translateX(0);transform:translateX(0)}.rw-calendar-transition-exiting.rw-calendar-transition-top{-ms-transform:translateY(100%);transform:translateY(100%)}.rw-calendar-transition-exiting.rw-calendar-transition-bottom{-ms-transform:translateY(-100%);transform:translateY(-100%)}.rw-calendar-transition-exiting.rw-calendar-transition-right{-ms-transform:translateX(100%);transform:translateX(100%)}.rw-calendar-transition-exiting.rw-calendar-transition-left{-ms-transform:translateX(-100%);transform:translateX(-100%)}.rw-select-list{overflow:auto;position:relative}.rw-select-list .rw-list{max-height:none;font-size:1em}.rw-select-list-label{display:block;position:relative;font-weight:normal;cursor:inherit;padding-left:20px;margin:0}.rw-rtl .rw-select-list-label{padding-left:0;padding-right:20px}input.rw-select-list-input{position:absolute;left:0;top:50%;-ms-transform:translateY(-50%);transform:translateY(-50%);top:0.1em /9;margin:0;line-height:normal;cursor:inherit}.rw-rtl input.rw-select-list-input{left:auto;right:0}.rw-loading-mask{content:'';background:url("+r(n(919))+") no-repeat center;position:absolute;background-color:#fff;border-radius:4px;opacity:.7;top:0;left:0;height:100%;width:100%}.rw-multiselect{cursor:text}.rw-multiselect .rw-input-reset{height:calc(2.429em - 2px);margin-top:-2px\\9;height:2.429em\\9;border-width:0;width:auto;max-width:100%;padding:0 .857em}.rw-multiselect .rw-select{vertical-align:middle}.rw-multiselect .rw-select,.rw-multiselect .rw-select:hover,.rw-multiselect .rw-select:active{box-shadow:none;background:none}.rw-multiselect-taglist{margin:0;padding:0;list-style:none;display:inline-block;vertical-align:0;outline:none}.rw-multiselect-tag{display:inline-table;color:inherit;padding:0 .35em 0 .35em;margin-left:calc(0.279335em - 1px);margin-top:.279335em;margin-top:calc(0.279335em - 1px);height:1.87033em;border-radius:3px;background-color:#eee;border:1px solid #ccc;cursor:default;vertical-align:top;text-align:center;overflow:hidden;max-width:100%}.rw-multiselect-tag>*{display:table-cell;vertical-align:middle;height:100%}.rw-rtl .rw-multiselect-tag{margin-left:0;margin-right:calc(0.279335em - 1px);padding:0 .35em 0 .35em}.rw-multiselect-tag.rw-state-focus,.rw-multiselect-tag.rw-state-focus:hover{background-color:transparent;border-color:#66afe9;color:#333}.rw-multiselect-tag.rw-state-readonly,.rw-multiselect-tag.rw-state-disabled,.rw-state-readonly .rw-multiselect-tag,.rw-state-disabled .rw-multiselect-tag,fieldset[disabled] .rw-multiselect-tag{cursor:not-allowed}.rw-multiselect-tag.rw-state-disabled,.rw-state-disabled .rw-multiselect-tag,fieldset[disabled] .rw-multiselect-tag{opacity:.65}fieldset[disabled] .rw-multiselect-tag{box-shadow:none;cursor:not-allowed}.rw-multiselect-tag-btn{color:inherit;margin-left:.25em}.rw-rtl .rw-multiselect-tag-btn{margin-left:0;margin-right:.25em}.rw-autocomplete .rw-select{position:absolute;display:block;width:auto;top:0;bottom:0;right:0}.rw-popup-container{position:absolute;z-index:1005;top:100%;left:-6px;right:-6px}.rw-popup-container.rw-dropup{top:auto;bottom:100%}.rw-state-focus .rw-popup-container{z-index:1006}.rw-popup-transition{width:100%;margin-bottom:6px;padding:0 6px}.rw-dropup>.rw-popup-transition{margin-bottom:0;margin-top:6px}.rw-popup{border-top-right-radius:0;border-top-left-radius:0;border-bottom-right-radius:3px;border-bottom-left-radius:3px;box-shadow:0 5px 6px rgba(0,0,0,0.2);border:#ccc 1px solid;background:#fff}.rw-dropup .rw-popup{border-bottom-right-radius:0;border-bottom-left-radius:0;border-top-right-radius:3px;border-top-left-radius:3px;box-shadow:0 -2px 6px rgba(0,0,0,0.2)}.rw-popup-transition{transition:transform 200ms}.rw-popup-transition-entering{overflow:hidden}.rw-popup-transition-entering .rw-popup-transition{-ms-transform:translateY(0);transform:translateY(0);transition-timing-function:ease-out}.rw-popup-transition-exiting .rw-popup-transition{transition-timing-function:ease-in}.rw-popup-transition-exiting,.rw-popup-transition-exited{overflow:hidden}.rw-popup-transition-exiting .rw-popup-transition,.rw-popup-transition-exited .rw-popup-transition{-ms-transform:translateY(-100%);transform:translateY(-100%)}.rw-popup-transition-exiting.rw-dropup .rw-popup-transition,.rw-popup-transition-exited.rw-dropup .rw-popup-transition{-ms-transform:translateY(100%);transform:translateY(100%)}.rw-popup-transition-exited{display:none}.rw-state-disabled{box-shadow:none;cursor:not-allowed}",""])},function(t,e,n){t.exports=n.p+"bc7c4a59f924cf037aad6e1f9edba366.eot"},function(t,e,n){t.exports=n.p+"bc7c4a59f924cf037aad6e1f9edba366.eot"},function(t,e){t.exports="data:application/font-woff;base64, "},function(t,e){t.exports="data:application/octet-stream;base64, "},function(t,e){t.exports="data:image/svg+xml;base64,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"},function(t,e,n){t.exports=n.p+"33a752211d05af6684e26ec63c2ed965.gif"},function(t,e,n){t.exports=n.p+"a770b6797b68e3f8920e473eb824bac0.gif"},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,":root {\r\n    /**\r\n     * Variable naming scheme:\r\n     * - All vars begin with the prefix `m`\r\n     * - Last word indicates the intended use (eg, `color`, `font`, `width`)\r\n     * - UI colors are backgrounds or otherwise dominant colors of UI elements\r\n     *      `-ui-color`\r\n     * - Font colors are font colors or secondary colors of UI elements\r\n     *      `-font-color`\r\n     * - Border colors are used either as callouts or to soften a visual transition\r\n     *      `-border-color`\r\n     */\r\n    --m-active-ui-color: hsl(0, 0%, 48%);\r\n    --m-active-font-color: white;\r\n    --m-border-color: #bdbdbd;\r\n    --m-inactive-ui-color: hsl(0, 0%, 90%);\r\n    --m-inactive-font-color: black;\r\n\r\n    /** All these should have active-font-color */\r\n    --m-selected-ui-color: hsl(199, 98%, 38%);\r\n    --m-valid-ui-color: hsl(199, 98%, 58%);\r\n    --m-neutral-ui-color: hsl(0, 0%, 48%);\r\n    /* warn is 20% darker to ensure that it has a perceptibly different luminance */\r\n    --m-warn-ui-color: hsl(0, 98%, 38%);\r\n    /* a subtle callout color for inline examples */\r\n    --m-callout-ui-color: hsl(199, 98%, 98%);\r\n\r\n    /** A hover should darken by 10% */\r\n    --m-hover-active-ui-color: hsl(0, 0%, 38%);\r\n    --m-hover-ui-color: hsl(0, 0%, 90%);\r\n    --m-hover-selected-ui-color: hsl(199, 98%, 28%);\r\n    --m-hover-valid-ui-color: hsl(199, 98%, 48%);\r\n    --m-hover-neutral-ui-color: hsl(0, 0%, 38%);\r\n    --m-hover-warn-ui-color: hsl(0, 98%, 28%);\r\n}",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".m-ListBox {\r\n    list-style-type: none;\r\n    margin: 0;\r\n    padding: 0;\r\n}\r\n\r\n.m-ListBox-item {\r\n    height: 1em;\r\n    line-height: 1em;\r\n    padding: 10px;\r\n    display: flex;\r\n}\r\n\r\n.m-ListBox-item:hover:not(.m-selected) {\r\n    background: hsl(0, 0%, 90%);\r\n}\r\n\r\n.m-ListBox-item:hover.m-selected {\r\n    background-color: hsl(199, 98%, 28%)\r\n}\r\n\r\n.m-selected {\r\n    background:hsl(199, 98%, 38%);\r\n    color: white;\r\n}\r\n\r\n.m-ListBox-item.m-editing {\r\n    padding: 5px;\r\n    height: calc(1em + 10px); /* padding top and bottom */\r\n}\r\n\r\n.m-editing > input {\r\n    flex: 1 0 0;\r\n    padding: 5px;\r\n}",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,'.m-FallbackDetailEditor {\r\n    height: 100%;\r\n    width: 100%;\r\n}\r\n\r\n.m-TypeEditor {\r\n    display: flex;\r\n    flex-direction: row;\r\n    margin-bottom: -1px;\r\n}\r\n\r\n.m-TypeEditor>:first-child {\r\n    margin-right: 24px;\r\n}\r\n\r\n.m-TypeEditor>input[type="text"] {\r\n    width: 0; /* Force unset the width so that the flexbox can set the width */\r\n}\r\n\r\n.m-TypeEditor .m-TypeEditor-DetailEditorBtn {\r\n    position: relative;\r\n    width: 18px;\r\n    height: 15px;\r\n    background: none;\r\n    border: 1px solid grey;\r\n    padding: 2px;\r\n    margin: 3px;\r\n    margin-left: -21px;\r\n    display: none;\r\n    box-sizing: border-box;\r\n}\r\n\r\n.m-TypeEditor:focus-within {\r\n    z-index: 1; /* move up in the stacking order to make the callout visible */\r\n}\r\n\r\n.m-TypeEditor:hover .m-TypeEditor-DetailEditorBtn,\r\n.m-TypeEditor:focus-within .m-TypeEditor-DetailEditorBtn  {\r\n    display: inline;\r\n}\r\n\r\n.m-TypeEditor .m-TypeEditor-DetailEditorBtn:hover {\r\n    background: rgb(201, 201, 201);\r\n}\r\n\r\n.m-TypeEditor .m-TypeEditor-DetailEditorBtn:active {\r\n    background: grey;\r\n    color: white;\r\n}\r\n\r\n.m-TypeEditor>*:not(button) {\r\n    flex-grow: 1;\r\n}\r\n\r\n.m-FallbackEditor-Number {\r\n    text-align: right;\r\n}\r\n\r\n.m-TypeEditor-Unsupported {\r\n    background-color: whitesmoke;\r\n    color: grey;\r\n}\r\n\r\n.m-FallbackEditor-invalid {\r\n    border-color: red;\r\n    --jp-content-font-color1: red;\r\n}\r\n',""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".m-InlineColorEditor {\r\n    display: flex;\r\n    flex-direction: row;\r\n}\r\n\r\n.m-InlineColorEditor .m-InlineColorEditor__colorSwatch {\r\n    flex: 0 0 30px;\r\n    margin: 3px;\r\n    border: 1px solid black;\r\n}\r\n\r\n.m-InlineColorEditor .m-InlineColorEditor__input {\r\n    flex: 1 0;\r\n}\r\n\r\n.m-InlineColorEditor__invalid {\r\n    border: 1px solid red;\r\n}\r\n\r\n.m-ColorPicker .m-ValueSlider {\r\n    background-image: linear-gradient(to right, black, white);\r\n    color: white;\r\n    display: flex;\r\n    padding: 2px;\r\n    vertical-align: middle;\r\n    line-height: 26px;\r\n}\r\n\r\n.m-ColorPicker .m-ValueSlider input {\r\n    flex-grow: 1;\r\n    margin-left: 20px;\r\n    margin-right: 20px;\r\n}\r\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,'.m-PropertiesEditor {\r\n    width: 100%;\r\n    position: relative;\r\n    --indent-width: 14px;\r\n    margin-bottom: 1px;\r\n}\r\n\r\n.m-PropertiesEditor-Property {\r\n    width: 100%;\r\n    position: relative;\r\n    display: flex;\r\n}\r\n\r\n.m-PropertiesEditor-PropertyLabel {\r\n    display: inline-block;\r\n    padding-left: calc(var(--indent-width) * (var(--indent) + 1));\r\n    flex: 0 0 calc(35% - (var(--indent-width) * (var(--indent) + 1)));\r\n    white-space: nowrap;\r\n    text-overflow: ellipsis;\r\n    overflow: hidden;\r\n}\r\n\r\n.m-PropertiesEditor-PropertyLabel.m-PropertiesEditor-PropertyGroup {\r\n    font-weight: bold;\r\n    padding-left: calc(var(--indent-width) * var(--indent));\r\n}\r\n\r\n.m-PropertiesEditor-Property>:last-child {\r\n    flex: 1 0 auto;\r\n}\r\n\r\n.m-FlippyTriangle {\r\n    width: 0;\r\n    height: 0;\r\n    margin: 0;\r\n    padding: 0;\r\n    background: transparent;\r\n    border: 8px solid transparent;\r\n    border-left: 8px solid black;\r\n    position: relative;\r\n    top: -4px;\r\n}\r\n\r\n.m-FlippyTriangle[data-collapsed="true"] {\r\n    transform: rotate(0)\r\n}\r\n\r\n.m-FlippyTriangle[data-collapsed="false"] {\r\n    transform: rotate(0.25turn)\r\n}',""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".m-Progress {\r\n    width: 100%;\r\n    position: relative;\r\n}\r\n\r\n.m-Progress-flavorText {\r\n    width: 100%;\r\n    text-align: center;\r\n    font-family: 'Segoe UI', Tahoma, Geneva, Verdana, sans-serif;\r\n}\r\n\r\n.m-Progress-bar {\r\n    -webkit-appearance: none;\r\n    background-color: lightgrey;\r\n    border-radius: 50px;\r\n    width: 100%;\r\n    margin-bottom: 20px;\r\n}\r\n\r\n.m-Progress-bar::-webkit-progress-bar {\r\n    background-color: lightgrey;\r\n    border-radius: 50px;\r\n}\r\n\r\n.m-Progress-bar::-moz-progress-bar {\r\n    background-color: #03A9F4;\r\n    border-radius: 50px;\r\n}\r\n\r\n.m-Progress-bar.m-Progress-failure::-moz-progress-bar {\r\n    background-color: red;\r\n}\r\n\r\n.m-Progress-bar.m-Progress-ok::-moz-progress-bar {\r\n    background-color: green;\r\n}\r\n\r\n/* Webkit needs this in a separate rule */\r\n.m-Progress-bar::-webkit-progress-value {\r\n    background-color: #03A9F4;\r\n    border-radius: 50px;\r\n    margin-bottom: 20px;\r\n}\r\n\r\n/* Webkit doesn't let us animate pseudo elements. So instead, display a static\r\n barber pole and leave it at that for now. */\r\n.m-Progress-bar-fake-indeterminate {\r\n    width: 100%;\r\n    height: 1em;\r\n    display: inline-block;\r\n    border-radius: 50px;\r\n    animation: m-Progress-indeterminate-bounce 10s linear 0s infinite;\r\n    background: repeating-linear-gradient(\r\n        -45deg,\r\n        #02A8F5,\r\n        #02A8F5 10px,\r\n        #027CB5 10px,\r\n        #027CB5 20px\r\n    );\r\n    background-size: 200% 200%;\r\n}\r\n\r\n.m-Progress-bar.m-Progress-failure::-webkit-progress-value {\r\n    background-color: red;\r\n}\r\n\r\n.m-Progress-bar.m-Progress-ok::-webkit-progress-value {\r\n    background-color: green;\r\n}\r\n\r\n@keyframes m-Progress-indeterminate-bounce {\r\n    to {\r\n        background-size: 100% 100%;\r\n    }\r\n}\r\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"div.rc-select-dropdown {\r\n    z-index: 100000;\r\n    max-height: 300px;\r\n    overflow-y: scroll;\r\n}\r\n\r\n.m-Button {\r\n    width: 100%;\r\n    height: 100%;\r\n    /* this override is required because of blueprint.js */\r\n    box-sizing: border-box;\r\n}\r\n\r\n.m-TextInput>span {\r\n    min-height: 22px;\r\n    display: flex;\r\n}\r\n\r\n.m-TextInput>span>label {\r\n    flex: 0 0 35%;\r\n}\r\n\r\n.m-TextInput>span>input {\r\n    width: 0;\r\n    flex: 1 0 60%;\r\n}\r\n\r\n.m-ResizeGrippy {\r\n    width:10px;\r\n    height:10px;\r\n    right:0;\r\n    bottom:0;\r\n    position:absolute !important;\r\n    float:left;\r\n    cursor: nwse-resize !important;\r\n    z-index:1000;\r\n}",""])},function(t,e,n){var r=n(172);(e=t.exports=n(12)(!1)).i(n(928),""),e.i(n(929),""),e.i(n(930),""),e.push([t.i,".m-SlickGridPart {\n  font-size: 12px;\n  font-family: sans-serif;\n  background-color: white;\n}\n.m-SlickGridPart .column-group-level0 {\n  background-color: orange;\n  color: white;\n}\n.m-SlickGridPart .column-group-level1 {\n  background-color: green;\n  color: white;\n}\n.m-SlickGridPart .slick-viewport {\n  background-color: #e4e4e4;\n}\n.m-SlickGridPart .toggle {\n  height: 9px;\n  width: 9px;\n  display: inline-block;\n}\n.m-SlickGridPart .toggle.expand {\n  background: no-repeat center center url(data:image/gif;base64,R0lGODlhCQAJAPcAAAFGeoCAgNXz/+v5/+v6/+z5/+36//L7//X8//j9/wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACQAJAAAIOAADCBxIUIDBgwIEChgwAECBAgQUFjAAQIABAwoBaNSIMYCAAwIqGlSIAEHFkiQTIBCgkqDLAAEBADs=);\n}\n.m-SlickGridPart .toggle.collapse {\n  background: no-repeat center center url(data:image/gif;base64,R0lGODlhCQAJAPcAAAFGeoCAgNXz/+v5/+v6/+z5/+36//L7//X8//j9/wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACQAJAAAIMwADCBxIUIDBgwIEChgwwECBAgQUFjBAkaJCABgxGlB4AGHCAAIQiBypEEECkScJqgwQEAA7);\n}\n.m-SlickGridPart .slick-header-column {\n  font-family: 'Arial', sans-serif !important;\n}\n.m-SlickGridPart .slick-header-column:hover {\n  transition: 0.3s;\n}\n.m-SlickGridPart .slick-header-columns-left:first-child {\n  font-style: italic;\n}\n.m-SlickGridPart .slick-row {\n  background: #dddddd !important;\n  transition: 0.09s;\n}\n.m-SlickGridPart .cell-hover {\n  position: fixed;\n  background: rgba(221, 221, 221, 0.2) !important;\n  outline: solid rgba(0, 0, 0, 0.2) 1px;\n  z-index: 9999;\n  left: 0;\n  top: 0;\n  right: 0;\n  pointer-events: none;\n  display: none;\n}\n.m-SlickGridPart .slick-range-decorator {\n  background-color: rgba(162, 199, 255, 0.2);\n  /*transition: 0.05s;*/\n  z-index: 9999;\n}\n.m-SlickGridPart .grid-canvas .flashing {\n  background-color: rgba(255, 149, 148, 0.5);\n}\n.m-SlickGridPart .tree-spacer {\n  content: \"\";\n  z-index: 1;\n  top: 50%;\n  border: 1px dotted #AAA;\n}\n.m-SlickGridPart .spark-line {\n  fill: none;\n  stroke: steelblue;\n  stroke-width: 1.5px;\n}\n.m-SlickGridPart .row-detail-image {\n  background: url("+r(n(931))+") no-repeat left center;\n  width: 25px;\n  height: 100%;\n  float: left;\n}\n.m-SlickGridPart .table-hover-image {\n  background: url("+r(n(932))+") no-repeat left center;\n  width: 25px;\n  height: 100%;\n  float: left;\n  padding-left: 16px;\n}\n.m-SlickGridPart .dashboard-link-image {\n  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAACXBIWXMAAAsTAAALEwEAmpwYAAAABGdBTUEAALGOfPtRkwAAACBjSFJNAAB6JQAAgIMAAPn/AACA6QAAdTAAAOpgAAA6mAAAF2+SX8VGAAABVElEQVR42qSTvUpDQRCFv927MSIKNjYhja8gNqKFICiIsRUkpT5BQLBSrK210ErzABILSSOCaBHBl9AIElAQkvu3PxYXr/khmpCp5jCz55yd3RHOOUYJkSs9FYAKiD4dIITAWIcnBc4YFBYtvM368fy1mnp/qUwuLg+t/Pb4VAGEAvis3WGt62cA5UmMc3hC4KxBCctEow6soqIvw/3Jwr+KxkJsIbIQatguXQGgAHK5HGfnz2nz7s4cmcPXFDf288xcqhTXCpowigCQg9zXj3uxEKI/QWw7cbOLIDS/uWq3bV1y2I+hvpcn1NDS0IrhYV0TGIhNp0BKoEtrAHiAf1RldqOcNt2Wi2RvsmR+ZrISpjU5iOXA9J+PhORZ2qPVRRD9QaAAfA0fB1V8DYGGIIbqRZHIgHG9tpMflsxfjckmS1unDLNUQkrU+HSSj7qNkhHjewCTLY4jzXD4MwAAAABJRU5ErkJggg==) no-repeat left center;\n  width: 25px;\n  height: 100%;\n  float: left;\n}\n.m-SlickGridPart #search-frame {\n  position: absolute;\n  bottom: 0;\n  left: 0;\n  right: 0;\n  background-color: #e2e2e2;\n  border-top: 1px solid #c6c6c6;\n  font-size: 12px;\n  z-index: 500;\n  display: none;\n}\n.m-SlickGridPart .slick-cell-checkbox {\n  padding-right: 2px;\n  margin-top: 1px;\n  height: 1.2em;\n}\n.m-SlickGridPart .slick-cell-radio {\n  padding-right: 2px;\n  margin-top: 1px;\n  height: 1.2em;\n}\n.m-SlickGridPart .slick-header-columns {\n  border-bottom: 1px solid silver;\n}\n.m-SlickGridPart .slick-header-column {\n  border-right: 1px solid silver;\n}\n.m-SlickGridPart .slick-headerrow {\n  background: #fafafa;\n}\n.m-SlickGridPart .slick-headerrow-column {\n  background: #fafafa;\n  border-bottom: 0;\n  height: 100%;\n}\n.m-SlickGridPart .slick-row {\n  background: white;\n  transition: 0.09s;\n  font-family: 'Arial', sans-serif !important;\n}\n.m-SlickGridPart .slick-row.ui-state-active {\n  background: #F5F7D7;\n}\n.m-SlickGridPart .slick-row.selected {\n  z-index: 10;\n  background: #DFE8F6;\n}\n.m-SlickGridPart .slick-cell {\n  padding-left: 4px;\n  padding-right: 4px;\n  font-size: 11px;\n}\n.m-SlickGridPart .slick-cell.selected {\n  color: white !important;\n  background: #77abed !important;\n  transition: 0.08s;\n}\n.m-SlickGridPart .slick-cell.editable {\n  background: white;\n  border: 1px solid black;\n}\n.m-SlickGridPart .slick-cell.flashing {\n  border: 1px solid red !important;\n}\n.m-SlickGridPart .slick-cell.active {\n  outline: dotted gray 1px;\n}\n.m-SlickGridPart .slick-cell.search-match {\n  background: yellow !important;\n  color: black !important;\n}\n.m-SlickGridPart .slick-cell.invalid {\n  border-color: red;\n  animation-duration: 0.2s;\n  animation-name: slickgrid-invalid-hilite;\n}\n.m-SlickGridPart .slick-row.odd {\n  background: #fafafa;\n}\n.m-SlickGridPart .slick-row.ui-state-active {\n  background: #F5F7D7;\n}\n.m-SlickGridPart .slick-row.loading {\n  opacity: 0.5;\n  filter: alpha(opacity=50);\n}\n.m-SlickGridPart .slick-group {\n  border-bottom: 2px solid silver;\n}\n.m-SlickGridPart .slick-group-toggle {\n  width: 9px;\n  height: 9px;\n  margin-right: 5px;\n}\n.m-SlickGridPart .slick-resizable-handle-hover {\n  background-color: #ccc;\n}\n.m-SlickGridPart .slick-sortable-placeholder {\n  background: silver;\n}\n.m-SlickGridPart .slick-group-toggle.expanded {\n  background: url("+r(n(933))+") no-repeat center center;\n}\n.m-SlickGridPart .slick-group-toggle.collapsed {\n  background: url("+r(n(934))+') no-repeat center center;\n}\n.m-SlickGridPart .slick-group-totals {\n  color: gray;\n  background: white;\n}\n.m-SlickGridPart .slick-sortable-placeholder {\n  background: silver !important;\n}\n@keyframes fadeitup {\n  0% {\n    background-color: green;\n  }\n}\n@keyframes fadeitdown {\n  0% {\n    background-color: red;\n  }\n}\n.m-SlickGridPart .changeup {\n  animation: fadeitup 5s ease-out;\n}\n.m-SlickGridPart .changedown {\n  animation: fadeitdown 5s ease-out;\n}\n@keyframes fadeituparrow {\n  0% {\n    color: green;\n    content: "\\25B2";\n  }\n  100% {\n    color: transparent;\n    content: "\\25B2";\n  }\n}\n@keyframes fadeitdownarrow {\n  0% {\n    color: red;\n    content: "\\25BC";\n  }\n  100% {\n    color: transparent;\n    content: "\\25BC";\n  }\n}\n.m-SlickGridPart .changeuparrow::before {\n  content: "";\n  animation: fadeituparrow 5s ease-out;\n}\n.m-SlickGridPart .changedownarrow::before {\n  content: "";\n  animation: fadeitdownarrow 5s ease-out;\n}\n.m-SlickGridPart.m-TableEditorPart .slick-row {\n  background-color: #ffffff !important;\n}\n.m-SlickGridPart.m-TableEditorPart .slick-cell.active.editable {\n  margin: 0;\n  padding: 0;\n}\n.m-SlickGridPart.m-TableEditorPart .slick-cell {\n  font-size: 13px;\n}\n.m-SlickGridPart.m-TableEditorPart .m-TypeEditor input {\n  font-size: 13px;\n}\n.m-SlickGridPart.m-TableHover {\n  border: 1px solid black;\n}\n.m-SlickGridPart.m-TableHover .slick-viewport {\n  overflow: hidden !important;\n}\n.m-SlickGridPart.m-TablePopup {\n  padding: 0 !important;\n}\n.m-SlickGridPart .m-cell-error {\n  left: 0;\n  right: 0;\n  position: absolute;\n  font-size: 17px;\n  color: red;\n  font-weight: bold;\n  text-align: center;\n}\n.m-error-detail-hover {\n  margin: 0;\n}\n.m-Hover pre.p-Widget.m-error-detail-hover {\n  overflow: auto;\n  margin: 0;\n}\n',""])},function(t,e,n){(e=t.exports=n(12)(!1)).push([t.i,"@import url(https://fonts.googleapis.com/css?family=Material+Icons);",""]),e.push([t.i,"@import url(https://fonts.googleapis.com/css?family=Open+Sans);",""]),e.push([t.i,"@import url(https://fonts.googleapis.com/css?family=Roboto+Mono);",""]),e.push([t.i,'/******************************************************************************\n *\n * Copyright (c) 2017, the Perspective Authors.\n *\n * This file is part of the Perspective library, distributed under the terms of\n * the Apache License 2.0.  The full license can be found in the LICENSE file.\n *\n */\n/******************************************************************************\n *\n * Copyright (c) 2017, the Perspective Authors.\n *\n * This file is part of the Perspective library, distributed under the terms of\n * the Apache License 2.0.  The full license can be found in the LICENSE file.\n *\n */\nperspective-viewer {\n  --interface--font-family: "Open Sans";\n  --interface-monospace--font-family: "Roboto Mono";\n  --inactive-column-selector--content: "\\E835";\n  --active-column-selector--content: "\\E834";\n  --active-column-selector--color: #1a7da1;\n  --active-column--background: none;\n  --column--height: 21px;\n  --active-column--height: 51px;\n  --inactive-column--height: 24px;\n  --active-columns--justify-content: space-evenly;\n  --inactive-column--opacity: 0.3;\n  --column_type-float--color: #1a7da1;\n  --column_type-string--color: #ff8888;\n  --config_button--before: "more_vert";\n  --button--font-family: "Material Icons";\n  --copy_button--before: "file_copy";\n  --download_button--before: "save_alt";\n  --reset_button--before: "refresh";\n  --row_draggable-border-last-child-width: 0;\n  --row_pivots--before: "Group By";\n  --column_pivots--before: "Split By";\n  --sort-order-asc--before: "arrow_upward";\n  --sort-order-desc--before: "arrow_downward";\n  --sort-order-none--before: "remove";\n  --sort-order-col-asc--before: "arrow_back";\n  --sort-order-col-desc--before: "arrow_forward";\n  --sort_order-padding: 0 0 0 0px;\n  --select--background-color: none;\n  --select--padding: 0px;\n  --column-drop-container--background: none;\n  --column-drop-container--padding: 0px;\n  --column-drop-container--margin: 24px 24px 0px 0px;\n  --column-drop-label--margin: -12px 0px 0px 0px;\n  --column-drop-label--font-size: 10px;\n  --column-drop-label--display: inline-block;\n  --column-container--margin: 24px 0px 0px 0px;\n  --column-add--font-family: "Material Icons";\n  --column-add--before: "add";\n  --column_type--padding: 0px 0px 0px 0px;\n  --column_type--width: 25px;\n  --column-selector--font-family: "Material Icons";\n  --active-column-selector--height: 45px;\n  --column_selector--width: 31px;\n  --column_selector--font-size: 16px;\n  --hypergrid--padding: 24px;\n  --side_panel--padding: 24px 24px 24px 17px;\n  --button--padding: 24px 17px 0px 17px;\n  --button--font-size: 16px;\n  --button-hover--color: #1a7da1;\n  --top_panel--padding: 0px 0px 24px 0px;\n  --hypergrid-scrollbar--width: 8px;\n  --hypergrid-scrollbar--border-radius: 5px;\n  --hypergrid-positive--color: #1078d1;\n  --hypergrid-negative--color: #de3838;\n  --hypergrid--font-size: 12px;\n  --hypergrid-header--font-size: 12px;\n  --hypergrid--font-family: "Open Sans";\n  --hypergrid-header--font-family: "Open Sans";\n  --d3fc-treedata-axis--lines: none;\n}',""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*\r\nIMPORTANT:\r\nIn order to preserve the uniform grid appearance, all cell styles need to have padding, margin and border sizes.\r\nNo built-in (selected, editable, highlight, flashing, invalid, loading, :focus) or user-specified CSS\r\nclasses should alter those!\r\n*/\r\n\r\n#slickgrid-container {\r\n    position: absolute;\r\n    top: 0;\r\n    bottom: 0;\r\n    left: 0;\r\n    right: 0;\r\n}\r\n\r\n.toggle {\r\n    display: inline-block;\r\n    /*position: relative;*/\r\n    /*top: -0.2em;*/\r\n}\r\n\r\n.row-selector {\r\n    text-align: center;\r\n    background: #FFF !important;\r\n    border: 1px solid black;\r\n}\r\n\r\n.cell-hover {\r\n    position: absolute;\r\n    z-index: 9999;\r\n    left: 0;\r\n    top: 0;\r\n    right: 0;\r\n    pointer-events: none;\r\n    display: none;\r\n}\r\n\r\n.slick-header.ui-state-default, .slick-headerrow.ui-state-default {\r\n    width: 100%;\r\n    overflow: hidden;\r\n    border-left: 0;\r\n}\r\n\r\n.slick-column-name {\r\n    padding-left: 4px;\r\n    padding-right: 4px;\r\n}\r\n\r\n.slick-header-columns, .slick-headerrow-columns {\r\n    position: relative;\r\n    white-space: nowrap;\r\n    cursor: default;\r\n    overflow: hidden;\r\n}\r\n\r\n.slick-header-column.ui-state-default {\r\n    position: relative;\r\n    display: inline-block;\r\n    overflow: hidden;\r\n    -o-text-overflow: ellipsis;\r\n    text-overflow: ellipsis;\r\n    margin: 0;\r\n    border-left: 0;\r\n    border-top: 0;\r\n    border-bottom: 0;\r\n    float: left;\r\n    height: 16px;\r\n    line-height: 16px;\r\n    /*padding: 4px;*/\r\n    padding-bottom: 4px;\r\n}\r\n\r\n.slick-headerrow-column.ui-state-default {\r\n    /*padding: 4px;*/\r\n}\r\n\r\n.slick-header-column-sorted {\r\n    font-style: italic;\r\n}\r\n\r\n.slick-sort-indicator {\r\n    display: inline-block;\r\n    width: 8px;\r\n    height: 5px;\r\n    margin-left: 4px;\r\n    margin-top: 6px;\r\n    float: right;\r\n    position: absolute;\r\n    bottom: 7px;\r\n    right: 5px;\r\n}\r\n\r\n.slick-sort-indicator-desc {\r\n    background: url(data:image/gif;base64,R0lGODlhDQAFAIcAAGGQzUD/QOPu+wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAMAAAEALAAAAAANAAUAAAgeAAUAGEgQgIAACBEKLHgwYcKFBh1KFNhQosOKEgMCADs=);\r\n}\r\n\r\n.slick-sort-indicator-asc {\r\n    background: url(data:image/gif;base64,R0lGODlhDQAFAIcAAGGQzUD/QOPu+wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAMAAAEALAAAAAANAAUAAAgbAAMIDABgoEGDABIeRJhQ4cKGEA8KmEiRosGAADs=);\r\n}\r\n\r\n.slick-resizable-handle {\r\n    position: absolute;\r\n    font-size: 0.1px;\r\n    display: block;\r\n    cursor: col-resize;\r\n    width: 4px;\r\n    right: 0;\r\n    top: 0;\r\n    height: 100%;\r\n}\r\n\r\n.grid-canvas {\r\n    position: relative;\r\n    outline: 0;\r\n    overflow: hidden;\r\n}\r\n\r\n.slick-row {\r\n    position: absolute;\r\n    border: 0px;\r\n    line-height: 17px;\r\n}\r\n\r\n.slick-row.ui-widget-content, .slick-row.ui-state-active {\r\n    position: absolute;\r\n    border: 0;\r\n    width: 100%;\r\n}\r\n\r\n.slick-cell, .slick-headerrow-column {\r\n    position: absolute;\r\n    /*border: 1px solid transparent;*/\r\n    border-right: 1px solid silver;\r\n    border-bottom-color: silver;\r\n    overflow: hidden;\r\n    -o-text-overflow: ellipsis;\r\n    text-overflow: ellipsis;\r\n    vertical-align: middle;\r\n    z-index: 1;\r\n    padding: 1px 0px 3px 3px;\r\n    margin: 0;\r\n    white-space: nowrap;\r\n    cursor: default;\r\n}\r\n\r\n.slick-group {\r\n}\r\n\r\n.slick-group-toggle {\r\n    display: inline-block;\r\n}\r\n\r\n.slick-cell.editable {\r\n    z-index: 11;\r\n    overflow: visible;\r\n}\r\n\r\n.slick-cell:focus {\r\n    outline: none;\r\n}\r\n\r\n.slick-reorder-proxy {\r\n    display: inline-block;\r\n    background: blue;\r\n    opacity: 0.15;\r\n    filter: alpha(opacity=15);\r\n    cursor: move;\r\n}\r\n\r\n.slick-reorder-guide {\r\n    display: inline-block;\r\n    height: 2px;\r\n    background: blue;\r\n    opacity: 0.7;\r\n    filter: alpha(opacity=70);\r\n}\r\n\r\n.slick-selection {\r\n    z-index: 10;\r\n    position: absolute;\r\n    border: 2px dashed black;\r\n}\r\n\r\n.slick-pane {\r\n    position: absolute;\r\n    outline: 0;\r\n    overflow: hidden;\r\n    width: 100%;\r\n}\r\n\r\n.slick-pane-header {\r\n    display: block;\r\n}\r\n\r\n.slick-header {\r\n    overflow: hidden;\r\n    position: relative;\r\n}\r\n\r\n.slick-headerrow {\r\n    overflow: hidden;\r\n    position: relative;\r\n}\r\n\r\n.slick-top-panel-scroller {\r\n    overflow: hidden;\r\n    position: relative;\r\n}\r\n\r\n.slick-top-panel {\r\n    width: 10000px\r\n}\r\n\r\n.slick-viewport {\r\n    position: relative;\r\n    outline: 0;\r\n    width: 100%;\r\n}\r\n\r\n@-moz-keyframes slickgrid-invalid-hilite {\r\n    from {\r\n        box-shadow: 0 0 6px red;\r\n    }\r\n    to {\r\n        box-shadow: none;\r\n    }\r\n}\r\n\r\n@-webkit-keyframes slickgrid-invalid-hilite {\r\n    from {\r\n        box-shadow: 0 0 6px red;\r\n    }\r\n    to {\r\n        box-shadow: none;\r\n    }\r\n}\r\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,'.slick-path-button {\r\n    display: inline-block;\r\n    float: left;\r\n    vertical-align: top;\r\n    /*margin: 1px;*/\r\n    /**\r\n    * This makes all "float:right" elements after it that spill over to the next line\r\n    * display way below the lower boundary of the column thus hiding them.\r\n    */\r\n    height: 100%;\r\n    width: 15px;\r\n    background: no-repeat center center;\r\n    cursor: pointer;\r\n    font-style: normal !important;\r\n    border-left: 1px solid gray;\r\n    text-align: center;\r\n}\r\n\r\n.slick-path-button:hover {\r\n    background-color: #d3d3d3;\r\n}\r\n\r\n.slick-path-button-hidden {\r\n    width: 0;\r\n\r\n    -webkit-transition: 0.2s width;\r\n    -ms-transition: 0.2s width;\r\n    transition: 0.2s width;\r\n}\r\n\r\n.slick-header-column:hover > .slick-path-button {\r\n    width: 15px;\r\n}',""])},function(t,e,n){t.exports=n.p+"908310e51073da9ae4ac3537c5eaf4ac.png"},function(t,e,n){t.exports=n.p+"721776b186c661a5439a22beb10937d3.png"},function(t,e,n){t.exports=n.p+"6ccc1ed85229f7c88b07bbbeefe5214d.gif"},function(t,e,n){t.exports=n.p+"b2c868bfc2cab12122861c8214e38724.gif"},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(936),""),e.i(n(937),""),e.i(n(938),""),e.i(n(285),""),e.i(n(994),""),e.i(n(995),""),e.push([t.i,"/* Repeat the selectors to bump up the specificity.\r\n *\r\n * Yes. This works.\r\n */\r\n.cm-s-jupyter.cm-s-jupyter .CodeMirror-cursor.CodeMirror-cursor {\r\n    border-left: var(--jp-code-cursor-width0, 1px) solid var(--jp-editor-cursor-color, black)\r\n}\r\n\r\n/* Codemirror autocomplete styles */\r\n.CodeMirror-hint-container { \r\n    display:flex;\r\n}\r\n.CodeMirror-hint-signature { \r\n    color:lightgray;\r\n}\r\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".rc-calendar {\n  box-sizing: border-box;\n}\n.rc-calendar * {\n  box-sizing: border-box;\n}\n.rc-calendar-hidden {\n  display: none;\n}\n.rc-calendar-input-wrap {\n  position: relative;\n  padding: 6px;\n  border-bottom: 1px solid #e9e9e9;\n}\n.rc-calendar-input-wrap:after {\n  content: '';\n  clear: both;\n}\n.rc-calendar-date-input-wrap {\n  overflow: hidden;\n}\n.rc-calendar-time-picker {\n  position: absolute;\n  width: 100%;\n  top: 34px;\n  background-color: white;\n  height: 217px;\n}\n.rc-calendar-time-picker-panel {\n  width: 100%;\n  position: relative;\n}\n.rc-calendar-time-picker-panel .rc-time-picker-panel-input-wrap {\n  display: none;\n}\n.rc-calendar-time-picker-panel .rc-time-picker-panel-inner {\n  border: none;\n  box-shadow: none;\n}\n.rc-calendar-time-picker-panel .rc-time-picker-panel-select {\n  width: 84px;\n  max-height: 217px;\n}\n.rc-calendar-time-picker-panel .rc-time-picker-panel-select li {\n  text-align: center;\n  padding: 0;\n}\n.rc-calendar-time-picker-wrap {\n  float: left;\n  width: 100%;\n}\n.rc-calendar-time-picker-wrap .rc-time-picker {\n  width: 100%;\n}\n.rc-calendar-time-picker-wrap .rc-time-picker-input {\n  padding: 0;\n  border: 1px solid transparent;\n  outline: 0;\n  height: 22px;\n}\n.rc-calendar-time-picker-wrap .rc-time-picker-icon {\n  display: none;\n}\n.rc-calendar-input {\n  border: 1px solid transparent;\n  width: 100%;\n  color: #666;\n  cursor: text;\n  line-height: 1.5;\n  outline: 0;\n  height: 22px;\n}\n.rc-calendar-input-invalid {\n  border-color: red;\n}\n.rc-calendar-clear-btn {\n  z-index: 9999;\n  position: absolute;\n  right: 6px;\n  cursor: pointer;\n  overflow: hidden;\n  width: 20px;\n  height: 20px;\n  text-align: center;\n  line-height: 20px;\n  top: 6px;\n  margin: 0;\n}\n.rc-calendar-clear-btn:after {\n  content: \"x\";\n  font-size: 12px;\n  color: #aaa;\n  display: inline-block;\n  line-height: 1;\n  width: 20px;\n  transition: color 0.3s ease;\n}\n.rc-calendar-clear-btn:hover:after {\n  color: #666;\n}\n.rc-calendar-picker {\n  position: absolute;\n  left: -9999px;\n  top: -9999px;\n  z-index: 1000;\n}\n.rc-calendar-picker-hidden {\n  display: none;\n}\n.rc-calendar-picker-slide-up-enter {\n  animation-duration: .3s;\n  animation-fill-mode: both;\n  transform-origin: 0 0;\n  display: block !important;\n  opacity: 0;\n  animation-timing-function: cubic-bezier(0.08, 0.82, 0.17, 1);\n  animation-play-state: paused;\n}\n.rc-calendar-picker-slide-up-appear {\n  animation-duration: .3s;\n  animation-fill-mode: both;\n  transform-origin: 0 0;\n  display: block !important;\n  opacity: 0;\n  animation-timing-function: cubic-bezier(0.08, 0.82, 0.17, 1);\n  animation-play-state: paused;\n}\n.rc-calendar-picker-slide-up-leave {\n  animation-duration: .3s;\n  animation-fill-mode: both;\n  transform-origin: 0 0;\n  display: block !important;\n  opacity: 1;\n  animation-timing-function: cubic-bezier(0.6, 0.04, 0.98, 0.34);\n  animation-play-state: paused;\n}\n.rc-calendar-picker-slide-up-enter.rc-calendar-picker-slide-up-enter-active.rc-calendar-picker-placement-bottomLeft,\n.rc-calendar-picker-slide-up-enter.rc-calendar-picker-slide-up-enter-active.rc-calendar-picker-placement-bottomRight,\n.rc-calendar-picker-slide-up-appear.rc-calendar-picker-slide-up-appear-active.rc-calendar-picker-placement-bottomLeft,\n.rc-calendar-picker-slide-up-appear.rc-calendar-picker-slide-up-appear-active.rc-calendar-picker-placement-bottomRight {\n  animation-name: rcDropdownSlideUpIn;\n  animation-play-state: running;\n}\n.rc-calendar-picker-slide-up-enter.rc-calendar-picker-slide-up-enter-active.rc-calendar-picker-placement-topLeft,\n.rc-calendar-picker-slide-up-enter.rc-calendar-picker-slide-up-enter-active.rc-calendar-picker-placement-topRight,\n.rc-calendar-picker-slide-up-appear.rc-calendar-picker-slide-up-appear-active.rc-calendar-picker-placement-topLeft,\n.rc-calendar-picker-slide-up-appear.rc-calendar-picker-slide-up-appear-active.rc-calendar-picker-placement-topRight {\n  animation-name: rcDropdownSlideDownIn;\n  animation-play-state: running;\n}\n.rc-calendar-picker-slide-up-leave.rc-calendar-picker-slide-up-leave-active.rc-calendar-picker-placement-bottomLeft,\n.rc-calendar-picker-slide-up-leave.rc-calendar-picker-slide-up-leave-active.rc-calendar-picker-placement-bottomRight {\n  animation-name: rcDropdownSlideUpOut;\n  animation-play-state: running;\n}\n.rc-calendar-picker-slide-up-leave.rc-calendar-picker-slide-up-leave-active.rc-calendar-picker-placement-topLeft,\n.rc-calendar-picker-slide-up-leave.rc-calendar-picker-slide-up-leave-active.rc-calendar-picker-placement-topRight {\n  animation-name: rcDropdownSlideDownOut;\n  animation-play-state: running;\n}\n@keyframes rcDropdownSlideUpIn {\n  0% {\n    opacity: 0;\n    transform-origin: 0% 0%;\n    transform: scaleY(0);\n  }\n  100% {\n    opacity: 1;\n    transform-origin: 0% 0%;\n    transform: scaleY(1);\n  }\n}\n@keyframes rcDropdownSlideUpOut {\n  0% {\n    opacity: 1;\n    transform-origin: 0% 0%;\n    transform: scaleY(1);\n  }\n  100% {\n    opacity: 0;\n    transform-origin: 0% 0%;\n    transform: scaleY(0);\n  }\n}\n@keyframes rcDropdownSlideDownIn {\n  0% {\n    opacity: 0;\n    transform-origin: 100% 100%;\n    transform: scaleY(0);\n  }\n  100% {\n    opacity: 1;\n    transform-origin: 100% 100%;\n    transform: scaleY(1);\n  }\n}\n@keyframes rcDropdownSlideDownOut {\n  0% {\n    opacity: 1;\n    transform-origin: 100% 100%;\n    transform: scaleY(1);\n  }\n  100% {\n    opacity: 0;\n    transform-origin: 100% 100%;\n    transform: scaleY(0);\n  }\n}\n.rc-calendar {\n  position: relative;\n  outline: none;\n  font-family: Arial, \"Hiragino Sans GB\", \"Microsoft Yahei\", \"Microsoft Sans Serif\", \"WenQuanYi Micro Hei\", sans-serif;\n  width: 253px;\n  list-style: none;\n  font-size: 12px;\n  text-align: left;\n  background-color: #fff;\n  border-radius: 3px;\n  box-shadow: 0 1px 5px #ccc;\n  background-clip: padding-box;\n  border: 1px solid #ccc;\n  line-height: 1.5;\n}\n.rc-calendar-date-panel,\n.rc-calendar-panel {\n  position: relative;\n  outline: none;\n}\n.rc-calendar-week-number {\n  width: 286px;\n}\n.rc-calendar-week-number-cell {\n  text-align: center;\n}\n.rc-calendar-header {\n  padding: 0 10px;\n  height: 34px;\n  line-height: 30px;\n  text-align: center;\n  -moz-user-select: none;\n   -ms-user-select: none;\n       user-select: none;\n  -webkit-user-select: none;\n  border-bottom: 1px solid #ccc;\n}\n.rc-calendar-header > a {\n  font-weight: bold;\n  display: inline-block;\n  padding: 0px 5px;\n  line-height: 34px;\n  text-align: center;\n  width: 30px;\n}\n.rc-calendar-header > a:hover {\n  cursor: pointer;\n  color: #23c0fa;\n}\n.rc-calendar-header .rc-calendar-prev-month-btn {\n  position: absolute;\n  left: 25px;\n}\n.rc-calendar-header .rc-calendar-prev-month-btn:after {\n  content: '\\2039';\n}\n.rc-calendar-header .rc-calendar-next-month-btn {\n  position: absolute;\n  right: 25px;\n}\n.rc-calendar-header .rc-calendar-next-month-btn:after {\n  content: '\\203A';\n}\n.rc-calendar-year-select,\n.rc-calendar-month-select,\n.rc-calendar-day-select {\n  display: inline-block;\n  font-size: 12px;\n  font-weight: bold;\n  color: #666;\n  padding: 0 8px;\n  line-height: 34px;\n}\n.rc-calendar-year-select:hover,\n.rc-calendar-month-select:hover,\n.rc-calendar-day-select:hover {\n  cursor: pointer;\n  color: #23c0fa;\n}\n.rc-calendar-year-select.rc-calendar-time-status:hover,\n.rc-calendar-month-select.rc-calendar-time-status:hover,\n.rc-calendar-day-select.rc-calendar-time-status:hover {\n  cursor: pointer;\n  color: #666;\n}\n.rc-calendar-prev-month-btn,\n.rc-calendar-next-month-btn,\n.rc-calendar-prev-year-btn,\n.rc-calendar-next-year-btn {\n  position: absolute;\n  top: 0;\n  cursor: pointer;\n  color: #999;\n  font-family: Arial, \"Hiragino Sans GB\", \"Microsoft Yahei\", \"Microsoft Sans Serif\", sans-serif;\n  padding: 0 5px;\n  font-size: 16px;\n  display: inline-block;\n  line-height: 34px;\n}\n.rc-calendar-prev-month-btn:hover,\n.rc-calendar-next-month-btn:hover,\n.rc-calendar-prev-year-btn:hover,\n.rc-calendar-next-year-btn:hover {\n  color: #23c0fa;\n}\n.rc-calendar-next-year-btn {\n  right: 0;\n}\n.rc-calendar-next-year-btn:after {\n  content: '\\BB';\n}\n.rc-calendar-prev-year-btn {\n  left: 0;\n}\n.rc-calendar-prev-year-btn:after {\n  content: '\\AB';\n}\n.rc-calendar-body {\n  padding: 9px 10px 10px;\n  height: 217px;\n}\n.rc-calendar table {\n  border-collapse: collapse;\n  max-width: 100%;\n  background-color: transparent;\n  width: 100%;\n}\n.rc-calendar table,\n.rc-calendar td,\n.rc-calendar th,\n.rc-calendar td {\n  border: none;\n}\n.rc-calendar-table {\n  border-spacing: 0;\n  margin-bottom: 0;\n}\n.rc-calendar-column-header {\n  line-height: 18px;\n  padding: 6px 0;\n  width: 33px;\n  text-align: center;\n}\n.rc-calendar-column-header .rc-calendar-column-header-inner {\n  display: block;\n  font-weight: normal;\n}\n.rc-calendar-week-number-header .rc-calendar-column-header-inner {\n  display: none;\n}\n.rc-calendar-cell {\n  padding: 1px 0;\n}\n.rc-calendar-date {\n  display: block;\n  margin: 0 auto;\n  color: #666;\n  border-radius: 4px 4px;\n  width: 26px;\n  height: 26px;\n  padding: 0;\n  background: transparent;\n  line-height: 26px;\n  text-align: center;\n}\n.rc-calendar-date:hover {\n  background: #ebfaff;\n  cursor: pointer;\n}\n.rc-calendar-selected-day .rc-calendar-date {\n  background: #d9f4fe;\n}\n.rc-calendar-selected-date .rc-calendar-date {\n  background: #3fc7fa;\n  color: #fff;\n}\n.rc-calendar-selected-date .rc-calendar-date:hover {\n  background: #3fc7fa;\n}\n.rc-calendar-today .rc-calendar-date {\n  border: 1px solid #3fc7fa;\n}\n.rc-calendar-disabled-cell .rc-calendar-date {\n  cursor: not-allowed;\n  color: #bcbcbc;\n  background: #f3f3f3;\n  border-radius: 0;\n  width: auto;\n}\n.rc-calendar-disabled-cell .rc-calendar-date:hover {\n  background: #f3f3f3;\n}\n.rc-calendar-disabled-cell-first-of-row .rc-calendar-date {\n  border-top-left-radius: 4px;\n  border-bottom-left-radius: 4px;\n}\n.rc-calendar-disabled-cell-last-of-row .rc-calendar-date {\n  border-top-right-radius: 4px;\n  border-bottom-right-radius: 4px;\n}\n.rc-calendar-last-month-cell .rc-calendar-date,\n.rc-calendar-next-month-btn-day .rc-calendar-date {\n  color: #bbb;\n}\n.rc-calendar-footer {\n  border-top: 1px solid #ccc;\n  padding: 10px 0;\n  text-align: center;\n  position: relative;\n}\n.rc-calendar-footer .rc-time-picker {\n  width: 90px;\n}\n.rc-calendar-footer .rc-time-picker-input {\n  height: 24px;\n}\n.rc-calendar-footer-show-ok {\n  text-align: right;\n}\n.rc-calendar-footer-show-ok .rc-calendar-footer-btn {\n  padding-right: 12px;\n}\n.rc-calendar-footer-show-ok .rc-calendar-time-picker-btn {\n  margin-left: 0;\n  padding: 0 12px;\n}\n.rc-calendar-footer-show-ok .rc-calendar-today-btn {\n  float: left;\n  padding-left: 12px;\n}\n.rc-calendar-footer-btn {\n  margin-top: 2px;\n}\n.rc-calendar-footer-btn:after {\n  content: 'x';\n  height: 0;\n  font-size: 0;\n  overflow: hidden;\n  clear: both;\n}\n.rc-calendar-time-picker-btn {\n  margin-left: 10px;\n}\n.rc-calendar-today-btn,\n.rc-calendar-ok-btn,\n.rc-calendar-time-picker-btn {\n  display: inline-block;\n  text-align: center;\n  color: #f46830;\n}\n.rc-calendar-today-btn:hover,\n.rc-calendar-ok-btn:hover,\n.rc-calendar-time-picker-btn:hover {\n  cursor: pointer;\n  color: #23c0fa;\n}\n.rc-calendar-today-btn-disabled,\n.rc-calendar-ok-btn-disabled,\n.rc-calendar-time-picker-btn-disabled {\n  color: #bbb;\n}\n.rc-calendar-today-btn-disabled:hover,\n.rc-calendar-ok-btn-disabled:hover,\n.rc-calendar-time-picker-btn-disabled:hover {\n  color: #bbb;\n}\n.rc-calendar-today-btn {\n  padding-left: 10px;\n}\n.rc-calendar-time-input {\n  height: 25px;\n  position: relative;\n  display: inline-block;\n  margin: 0 0;\n  padding: 4px 10px;\n  border-radius: 6px 6px;\n  border: 1px solid #d9d9d9;\n  background-color: #ffffff;\n  color: #666;\n  line-height: 1.5;\n  transform: border 0.3s cubic-bezier(0.35, 0, 0.25, 1), background 0.3s cubic-bezier(0.35, 0, 0.25, 1), box-shadow 0.3s cubic-bezier(0.35, 0, 0.25, 1);\n  width: 40px;\n}\n.rc-calendar-time-input:hover {\n  border-color: #23c0fa;\n}\n.rc-calendar-time-input:focus {\n  border-color: #23c0fa;\n  box-shadow: 0 0 3px #23c0fa;\n}\n.rc-calendar-time-panel {\n  left: 0;\n  top: 0;\n  bottom: 0;\n  right: 0;\n  background: #ffffff;\n  z-index: 10;\n  position: absolute;\n  outline: none;\n}\n.rc-calendar-time-panel-header {\n  padding: 0 10px;\n  height: 34px;\n  line-height: 34px;\n  position: relative;\n  text-align: center;\n  -moz-user-select: none;\n   -ms-user-select: none;\n       user-select: none;\n  -webkit-user-select: none;\n  border-bottom: 1px solid #ccc;\n}\n.rc-calendar-time-panel-body {\n  padding: 9px 10px 10px;\n}\n.rc-calendar-time-panel-title {\n  width: 180px;\n  font-weight: bold;\n  display: inline-block;\n  padding: 4px 5px;\n  text-align: center;\n  height: 30px;\n  line-height: 22px;\n  border-radius: 4px;\n}\n.rc-calendar-time-panel-table {\n  table-layout: fixed;\n  width: 100%;\n  height: 255px;\n  border-collapse: separate;\n}\n.rc-calendar-time-panel-cell {\n  text-align: center;\n  height: 42px;\n  vertical-align: middle;\n}\n.rc-calendar-time-panel-time {\n  line-height: 26px;\n  display: block;\n  border-radius: 4px;\n  width: 26px;\n  margin: 0 auto;\n}\n.rc-calendar-time-panel-time:hover {\n  background: #ebfaff;\n  cursor: pointer;\n}\n.rc-calendar-time-panel-selected-cell .rc-calendar-time-panel-time {\n  background: #3fc7fa;\n  color: #fff;\n}\n.rc-calendar-time-panel-selected-cell .rc-calendar-time-panel-time:hover {\n  background: #3fc7fa;\n  color: #fff;\n}\n.rc-calendar-month-panel {\n  left: 0;\n  top: 0;\n  bottom: 0;\n  right: 0;\n  background: #ffffff;\n  z-index: 10;\n  position: absolute;\n  outline: none;\n}\n.rc-calendar-month-panel > div {\n  height: 100%;\n  display: -ms-flexbox;\n  display: flex;\n  -ms-flex-direction: column;\n      flex-direction: column;\n}\n.rc-calendar-month-panel-hidden {\n  display: none;\n}\n.rc-calendar-month-panel-header {\n  padding: 0 10px;\n  height: 34px;\n  line-height: 30px;\n  position: relative;\n  text-align: center;\n  -moz-user-select: none;\n   -ms-user-select: none;\n       user-select: none;\n  -webkit-user-select: none;\n  border-bottom: 1px solid #ccc;\n}\n.rc-calendar-month-panel-header > a {\n  font-weight: bold;\n  display: inline-block;\n  padding: 4px 5px;\n  text-align: center;\n  width: 30px;\n}\n.rc-calendar-month-panel-header > a:hover {\n  cursor: pointer;\n  color: #23c0fa;\n}\n.rc-calendar-month-panel-prev-year-btn,\n.rc-calendar-month-panel-next-year-btn {\n  position: absolute;\n  top: 0;\n}\n.rc-calendar-month-panel-next-year-btn:after {\n  content: '\\BB';\n}\n.rc-calendar-month-panel-prev-year-btn {\n  -webkit-user-select: none;\n     -moz-user-select: none;\n      -ms-user-select: none;\n          user-select: none;\n  left: 0;\n}\n.rc-calendar-month-panel-prev-year-btn:after {\n  content: '\\AB';\n}\n.rc-calendar-month-panel .rc-calendar-month-panel-year-select {\n  width: 180px;\n}\n.rc-calendar-month-panel-year-select-arrow {\n  display: none;\n}\n.rc-calendar-month-panel-next-year-btn {\n  -webkit-user-select: none;\n     -moz-user-select: none;\n      -ms-user-select: none;\n          user-select: none;\n  right: 0;\n}\n.rc-calendar-month-panel-body {\n  -ms-flex: 1;\n      flex: 1;\n  padding: 9px 10px 10px;\n}\n.rc-calendar-month-panel-footer {\n  border-top: 1px solid #ccc;\n  line-height: 38px;\n}\n.rc-calendar-month-panel-table {\n  table-layout: fixed;\n  width: 100%;\n  height: 100%;\n  border-collapse: separate;\n}\n.rc-calendar-month-panel-cell {\n  text-align: center;\n}\n.rc-calendar-month-panel-cell .rc-calendar-month-panel-month {\n  display: block;\n  width: 46px;\n  margin: 0 auto;\n  color: #666;\n  border-radius: 4px 4px;\n  height: 36px;\n  padding: 0;\n  background: transparent;\n  line-height: 36px;\n  text-align: center;\n}\n.rc-calendar-month-panel-cell .rc-calendar-month-panel-month:hover {\n  background: #ebfaff;\n  cursor: pointer;\n}\n.rc-calendar-month-panel-cell-disabled .rc-calendar-month-panel-month {\n  color: #bfbfbf;\n}\n.rc-calendar-month-panel-cell-disabled .rc-calendar-month-panel-month:hover {\n  background: white;\n  cursor: not-allowed;\n}\n.rc-calendar-month-panel-selected-cell .rc-calendar-month-panel-month {\n  background: #3fc7fa;\n  color: #fff;\n}\n.rc-calendar-month-panel-selected-cell .rc-calendar-month-panel-month:hover {\n  background: #3fc7fa;\n  color: #fff;\n}\n.rc-calendar-month-header-wrap {\n  position: relative;\n  height: 308px;\n}\n.rc-calendar-year-panel {\n  left: 0;\n  top: 0;\n  bottom: 0;\n  right: 0;\n  background: #ffffff;\n  z-index: 10;\n  position: absolute;\n  outline: none;\n}\n.rc-calendar-year-panel > div {\n  height: 100%;\n  display: -ms-flexbox;\n  display: flex;\n  -ms-flex-direction: column;\n      flex-direction: column;\n}\n.rc-calendar-year-panel-hidden {\n  display: none;\n}\n.rc-calendar-year-panel-header {\n  padding: 0 10px;\n  height: 34px;\n  line-height: 30px;\n  position: relative;\n  text-align: center;\n  -moz-user-select: none;\n   -ms-user-select: none;\n       user-select: none;\n  -webkit-user-select: none;\n  border-bottom: 1px solid #ccc;\n}\n.rc-calendar-year-panel-header > a {\n  font-weight: bold;\n  display: inline-block;\n  padding: 4px 5px;\n  text-align: center;\n  width: 30px;\n}\n.rc-calendar-year-panel-header > a:hover {\n  cursor: pointer;\n  color: #23c0fa;\n}\n.rc-calendar-year-panel-prev-decade-btn,\n.rc-calendar-year-panel-next-decade-btn {\n  position: absolute;\n  top: 0;\n}\n.rc-calendar-year-panel-next-decade-btn:after {\n  content: '\\BB';\n}\n.rc-calendar-year-panel-prev-decade-btn {\n  -webkit-user-select: none;\n     -moz-user-select: none;\n      -ms-user-select: none;\n          user-select: none;\n  left: 0;\n}\n.rc-calendar-year-panel-prev-decade-btn:after {\n  content: '\\AB';\n}\n.rc-calendar-year-panel .rc-calendar-year-panel-decade-select {\n  width: 180px;\n}\n.rc-calendar-year-panel-decade-select-arrow {\n  display: none;\n}\n.rc-calendar-year-panel-next-decade-btn {\n  -webkit-user-select: none;\n     -moz-user-select: none;\n      -ms-user-select: none;\n          user-select: none;\n  right: 0;\n}\n.rc-calendar-year-panel-body {\n  -ms-flex: 1;\n      flex: 1;\n  padding: 9px 10px 10px;\n}\n.rc-calendar-year-panel-footer {\n  border-top: 1px solid #ccc;\n  line-height: 38px;\n}\n.rc-calendar-year-panel-table {\n  table-layout: fixed;\n  width: 100%;\n  height: 100%;\n  border-collapse: separate;\n}\n.rc-calendar-year-panel-cell {\n  text-align: center;\n}\n.rc-calendar-year-panel-year {\n  display: block;\n  width: 46px;\n  margin: 0 auto;\n  color: #666;\n  border-radius: 4px 4px;\n  height: 36px;\n  padding: 0;\n  background: transparent;\n  line-height: 36px;\n  text-align: center;\n}\n.rc-calendar-year-panel-year:hover {\n  background: #ebfaff;\n  cursor: pointer;\n}\n.rc-calendar-year-panel-selected-cell .rc-calendar-year-panel-year {\n  background: #3fc7fa;\n  color: #fff;\n}\n.rc-calendar-year-panel-selected-cell .rc-calendar-year-panel-year:hover {\n  background: #3fc7fa;\n  color: #fff;\n}\n.rc-calendar-year-panel-last-decade-cell .rc-calendar-year-panel-year,\n.rc-calendar-year-panel-next-decade-cell .rc-calendar-year-panel-year {\n  -moz-user-select: none;\n   -ms-user-select: none;\n       user-select: none;\n  -webkit-user-select: none;\n  color: rgba(0, 0, 0, 0.25);\n}\n.rc-calendar-decade-panel {\n  left: 0;\n  top: 0;\n  bottom: 0;\n  right: 0;\n  background: #ffffff;\n  z-index: 10;\n  position: absolute;\n  outline: none;\n  display: -ms-flexbox;\n  display: flex;\n  -ms-flex-direction: column;\n      flex-direction: column;\n}\n.rc-calendar-decade-panel-hidden {\n  display: none;\n}\n.rc-calendar-decade-panel-header {\n  padding: 0 10px;\n  height: 34px;\n  line-height: 34px;\n  position: relative;\n  text-align: center;\n  -moz-user-select: none;\n   -ms-user-select: none;\n       user-select: none;\n  -webkit-user-select: none;\n  border-bottom: 1px solid #ccc;\n}\n.rc-calendar-decade-panel-header > a {\n  font-weight: bold;\n  display: inline-block;\n  padding: 1px 5px;\n  text-align: center;\n  width: 30px;\n}\n.rc-calendar-decade-panel-header > a:hover {\n  cursor: pointer;\n  color: #23c0fa;\n}\n.rc-calendar-decade-panel-prev-century-btn,\n.rc-calendar-decade-panel-next-century-btn {\n  position: absolute;\n  top: 0;\n}\n.rc-calendar-decade-panel-next-century-btn:after {\n  content: '\\BB';\n}\n.rc-calendar-decade-panel-prev-century-btn {\n  -webkit-user-select: none;\n     -moz-user-select: none;\n      -ms-user-select: none;\n          user-select: none;\n  left: 0;\n}\n.rc-calendar-decade-panel-prev-century-btn:after {\n  content: '\\AB';\n}\n.rc-calendar-decade-panel-next-century-btn {\n  -webkit-user-select: none;\n     -moz-user-select: none;\n      -ms-user-select: none;\n          user-select: none;\n  right: 0;\n}\n.rc-calendar-decade-panel-body {\n  -ms-flex: 1;\n      flex: 1;\n  padding: 9px 10px 10px;\n}\n.rc-calendar-decade-panel-footer {\n  border-top: 1px solid #ccc;\n  line-height: 38px;\n}\n.rc-calendar-decade-panel-table {\n  table-layout: fixed;\n  width: 100%;\n  height: 100%;\n  border-collapse: separate;\n}\n.rc-calendar-decade-panel-cell {\n  text-align: center;\n}\n.rc-calendar-decade-panel-decade {\n  display: block;\n  margin: 0 auto;\n  color: #666;\n  border-radius: 4px 4px;\n  height: 36px;\n  padding: 0;\n  background: transparent;\n  line-height: 36px;\n  text-align: center;\n}\n.rc-calendar-decade-panel-decade:hover {\n  background: #ebfaff;\n  cursor: pointer;\n}\n.rc-calendar-decade-panel-selected-cell .rc-calendar-decade-panel-decade {\n  background: #3fc7fa;\n  color: #fff;\n}\n.rc-calendar-decade-panel-selected-cell .rc-calendar-decade-panel-decade:hover {\n  background: #3fc7fa;\n  color: #fff;\n}\n.rc-calendar-decade-panel-last-century-cell .rc-calendar-decade-panel-decade,\n.rc-calendar-decade-panel-next-century-cell .rc-calendar-decade-panel-decade {\n  -moz-user-select: none;\n   -ms-user-select: none;\n       user-select: none;\n  -webkit-user-select: none;\n  color: rgba(0, 0, 0, 0.25);\n}\n.rc-calendar-range {\n  width: 502px;\n  overflow: hidden;\n}\n.rc-calendar-range-part {\n  width: 250px;\n  position: relative;\n}\n.rc-calendar-range-part .rc-calendar-time-picker {\n  top: 69px;\n}\n.rc-calendar-range-part .rc-calendar-time-picker-panel-select {\n  width: 77px;\n}\n.rc-calendar-range-left {\n  float: left;\n}\n.rc-calendar-range-left .rc-calendar-time-picker-panel-select:last-child {\n  border-right: 1px solid #e9e9e9;\n}\n.rc-calendar-range-right {\n  float: right;\n}\n.rc-calendar-range-right .rc-calendar-time-picker-panel {\n  left: 21px;\n}\n.rc-calendar-range-right .rc-calendar-time-picker-panel-select:first-child {\n  border-left: 1px solid #e9e9e9;\n}\n.rc-calendar-range-middle {\n  position: absolute;\n  margin-left: -10px;\n  text-align: center;\n  height: 35px;\n  line-height: 35px;\n}\n.rc-calendar-range .rc-calendar-date-panel::after {\n  content: \".\";\n  display: block;\n  height: 0;\n  clear: both;\n  visibility: hidden;\n}\n.rc-calendar-range .rc-calendar-input-wrap {\n  height: 35px;\n}\n.rc-calendar-range .rc-calendar-input,\n.rc-calendar-range .rc-time-picker-input {\n  padding: 1px 7px;\n  height: 22px;\n}\n.rc-calendar-range .rc-calendar-body,\n.rc-calendar-range .rc-calendar-decade-panel-body,\n.rc-calendar-range .rc-calendar-year-panel-body,\n.rc-calendar-range .rc-calendar-month-panel-body {\n  border-bottom: 1px solid #e9e9e9;\n}\n.rc-calendar-range.rc-calendar-week-number {\n  width: 574px;\n}\n.rc-calendar-range.rc-calendar-week-number .rc-calendar-range-part {\n  width: 286px;\n}\n.rc-calendar-range.rc-calendar-week-number .rc-calendar-range-part .rc-calendar-time-picker {\n  top: 69px;\n}\n.rc-calendar-range.rc-calendar-week-number .rc-calendar-range-part .rc-calendar-time-picker-panel-select {\n  width: 89px;\n}\n.rc-calendar-range.rc-calendar-week-number .rc-calendar-range-right .rc-calendar-time-picker-panel {\n  left: 36px;\n}\n.rc-calendar-range .rc-calendar-year-panel,\n.rc-calendar-range .rc-calendar-month-panel,\n.rc-calendar-range .rc-calendar-decade-panel {\n  top: 35px;\n}\n.rc-calendar-range .rc-calendar-month-panel .rc-calendar-year-panel {\n  top: 0;\n}\n.rc-calendar-range .rc-calendar-decade-panel-table,\n.rc-calendar-range .rc-calendar-year-panel-table,\n.rc-calendar-range .rc-calendar-month-panel-table {\n  height: 198px;\n}\n.rc-calendar-range .rc-calendar-in-range-cell {\n  background: #ebf4f8;\n  border-radius: 0;\n}\n.rc-calendar-range-bottom {\n  text-align: right;\n}\n.rc-calendar-range .rc-calendar-footer {\n  border-top: none;\n  padding: 0;\n}\n.rc-calendar-range .rc-calendar-footer-btn {\n  padding: 10px 12px 10px 0;\n}\n.rc-calendar-range .rc-calendar-ok-btn {\n  position: static;\n}\n.rc-calendar-range .rc-calendar-today-btn {\n  float: left;\n}\n.rc-calendar-full {\n  width: 275px;\n}\n.rc-calendar-full-header {\n  padding: 5px 10px;\n  text-align: center;\n  -moz-user-select: none;\n   -ms-user-select: none;\n       user-select: none;\n  -webkit-user-select: none;\n  border-bottom: 1px solid #ccc;\n  overflow: hidden;\n}\n.rc-calendar-full-header-month-select,\n.rc-calendar-full-header-year-select {\n  width: 70px;\n  float: right;\n  margin-right: 5px;\n}\n.rc-calendar-full-header-switcher {\n  float: right;\n  display: inline-block;\n}\n.rc-calendar-full-header-switcher-normal:hover {\n  border-color: #23c0fa;\n  box-shadow: 0 0 2px rgba(45, 183, 245, 0.8);\n  cursor: pointer;\n}\n.rc-calendar-full-header-switcher-focus {\n  border-color: #3fc7fa;\n  background-color: #3fc7fa;\n  color: #fff;\n}\n.rc-calendar-full-header-switcher > span {\n  float: left;\n  height: 28px;\n  line-height: 24px;\n  border: 1px solid #d9d9d9;\n  padding: 0 10px;\n  color: #666;\n}\n.rc-calendar-full-header-switcher > span:first-child {\n  border-top-left-radius: 4px;\n  border-bottom-left-radius: 4px;\n  border-right: none;\n}\n.rc-calendar-full-header-switcher > span:last-child {\n  border-top-right-radius: 4px;\n  border-bottom-right-radius: 4px;\n  border-left: none;\n}\n.rc-calendar-fullscreen {\n  width: auto;\n}\n.rc-calendar-fullscreen .rc-calendar-full-header {\n  border-bottom: none;\n}\n.rc-calendar-fullscreen .rc-calendar-column-header {\n  text-align: right;\n  padding-right: 12px;\n}\n.rc-calendar-fullscreen .rc-calendar-cell {\n  padding: 0;\n}\n.rc-calendar-fullscreen .rc-calendar-cell .rc-calendar-date,\n.rc-calendar-fullscreen .rc-calendar-month-panel-cell .rc-calendar-month-panel-month {\n  display: block;\n  height: 116px;\n  width: auto;\n  border-radius: 0;\n  margin: 0 4px;\n  border: none;\n  border-top: 2px solid #eee;\n  text-align: right;\n  padding-right: 8px;\n}\n.rc-calendar-fullscreen .rc-calendar-selected-day .rc-calendar-date,\n.rc-calendar-fullscreen .rc-calendar-month-panel-selected-cell .rc-calendar-month-panel-month {\n  background-color: #ebfaff;\n  color: #666;\n}\n.rc-calendar-fullscreen .rc-calendar-today .rc-calendar-date,\n.rc-calendar-fullscreen .rc-calendar-month-panel-selected-cell .rc-calendar-month-panel-month {\n  border-top-color: #3FC7FA;\n  color: #3FC7FA;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"@-webkit-keyframes select-ring {\n  0% {\n    transform: rotate(0deg);\n  }\n  100% {\n    transform: rotate(360deg);\n  }\n}\n@keyframes select-ring {\n  0% {\n    transform: rotate(0deg);\n  }\n  100% {\n    transform: rotate(360deg);\n  }\n}\n.rc-select {\n  box-sizing: border-box;\n  display: inline-block;\n  position: relative;\n  vertical-align: middle;\n  color: #666;\n  line-height: 28px;\n}\n.rc-select ul,\n.rc-select li {\n  margin: 0;\n  padding: 0;\n  list-style: none;\n}\n.rc-select > ul > li > a {\n  padding: 0;\n  background-color: #fff;\n}\n.rc-select-arrow {\n  height: 26px;\n  position: absolute;\n  top: 1px;\n  right: 1px;\n  width: 20px;\n  outline: none;\n}\n.rc-select-arrow .rc-select-arrow-loading {\n  display: inline-block;\n  width: 18px;\n  height: 18px;\n  margin-top: 6px;\n  margin-left: -4px;\n}\n.rc-select-arrow .rc-select-arrow-loading:after {\n  content: ' ';\n  display: block;\n  width: 12px;\n  height: 12px;\n  margin: 2px;\n  border-radius: 50%;\n  border: 2px solid #999999;\n  border-color: #999999 transparent #999999 transparent;\n  -webkit-animation: select-ring 1.2s linear infinite;\n          animation: select-ring 1.2s linear infinite;\n}\n.rc-select-arrow .rc-select-arrow-icon {\n  border-color: #999999 transparent transparent transparent;\n  border-style: solid;\n  border-width: 5px 4px 0 4px;\n  height: 0;\n  width: 0;\n  margin-left: -4px;\n  margin-top: -2px;\n  position: absolute;\n  top: 50%;\n  left: 50%;\n}\n.rc-select:not(.rc-select-no-arrow) .rc-select-selection--multiple .rc-select-selection__clear,\n.rc-select-loading .rc-select-selection--multiple .rc-select-selection__clear {\n  right: 20px;\n}\n.rc-select-selection {\n  outline: none;\n  -moz-user-select: none;\n   -ms-user-select: none;\n       user-select: none;\n  -webkit-user-select: none;\n  box-sizing: border-box;\n  display: block;\n  background-color: #fff;\n  border-radius: 6px;\n  border: 1px solid #d9d9d9;\n}\n.rc-select-selection__placeholder {\n  position: absolute;\n  top: 0;\n  color: #aaa;\n}\n.rc-select-selection__clear {\n  font-weight: bold;\n  position: absolute;\n  line-height: 28px;\n}\n.rc-select-selection__clear-icon {\n  font-style: normal;\n}\n.rc-select-focused .rc-select-selection {\n  border-color: #23c0fa;\n  box-shadow: 0 0 2px rgba(45, 183, 245, 0.8);\n}\n.rc-select-enabled .rc-select-selection:hover {\n  border-color: #23c0fa;\n  box-shadow: 0 0 2px rgba(45, 183, 245, 0.8);\n}\n.rc-select-enabled .rc-select-selection:active {\n  border-color: #2db7f5;\n}\n.rc-select-selection--single {\n  height: 28px;\n  line-height: 28px;\n  cursor: pointer;\n  position: relative;\n}\n.rc-select-selection--single .rc-select-selection-selected-value {\n  pointer-events: none;\n  position: absolute;\n  left: 0;\n  top: 0;\n}\n.rc-select-selection--single .rc-select-selection__rendered {\n  height: 28px;\n  position: relative;\n  display: block;\n  overflow: hidden;\n  white-space: nowrap;\n  text-overflow: ellipsis;\n  margin-left: 10px;\n  line-height: 28px;\n}\n.rc-select-selection--single .rc-select-selection__clear {\n  top: 0;\n  right: 20px;\n}\n.rc-select-disabled {\n  color: #ccc;\n  cursor: not-allowed;\n}\n.rc-select-disabled .rc-select-selection--single,\n.rc-select-disabled .rc-select-selection__choice__remove {\n  cursor: not-allowed;\n  color: #ccc;\n}\n.rc-select-disabled .rc-select-selection--single:hover,\n.rc-select-disabled .rc-select-selection__choice__remove:hover {\n  cursor: not-allowed;\n  color: #ccc;\n}\n.rc-select-search__field__wrap {\n  display: inline-block;\n}\n.rc-select-search__field__placeholder {\n  position: absolute;\n  top: 0;\n  left: 3px;\n  color: #aaa;\n}\n.rc-select-search--inline {\n  width: 100%;\n}\n.rc-select-search--inline .rc-select-search__field__wrap {\n  width: 100%;\n}\n.rc-select-search--inline .rc-select-search__field {\n  border: none;\n  font-size: 100%;\n  background: transparent;\n  outline: 0;\n  width: 100%;\n}\n.rc-select-search--inline .rc-select-search__field::-ms-clear {\n  display: none;\n}\n.rc-select-search--inline .rc-select-search__field__mirror {\n  position: absolute;\n  top: -999px;\n  left: 0;\n  white-space: pre;\n}\n.rc-select-search--inline > i {\n  float: right;\n}\n.rc-select-enabled.rc-select-selection--multiple {\n  cursor: text;\n}\n.rc-select-selection--multiple {\n  min-height: 28px;\n}\n.rc-select-selection--multiple .rc-select-search--inline {\n  float: left;\n  width: auto;\n}\n.rc-select-selection--multiple .rc-select-search--inline .rc-select-search__field {\n  width: 0.75em;\n}\n.rc-select-selection--multiple .rc-select-search--inline .rc-select-search__field__wrap {\n  width: auto;\n}\n.rc-select-selection--multiple .rc-select-search__field__placeholder {\n  top: 5px;\n  left: 8px;\n}\n.rc-select-selection--multiple .rc-select-selection__rendered {\n  position: relative;\n  overflow: hidden;\n  text-overflow: ellipsis;\n  margin-left: 8px;\n  padding-bottom: 2px;\n}\n.rc-select-selection--multiple .rc-select-selection__rendered .rc-select-selection__choice {\n  margin-top: 4px;\n  line-height: 20px;\n}\n.rc-select-selection--multiple .rc-select-selection__clear {\n  top: 1px;\n  right: 8px;\n}\n.rc-select-enabled .rc-select-selection__choice {\n  cursor: default;\n}\n.rc-select-enabled .rc-select-selection__choice:hover .rc-select-selection__choice__remove {\n  opacity: 1;\n  transform: scale(1);\n}\n.rc-select-enabled .rc-select-selection__choice:hover .rc-select-selection__choice__content {\n  margin-left: -8px;\n  margin-right: 8px;\n}\n.rc-select-enabled .rc-select-selection__choice__disabled {\n  cursor: not-allowed;\n}\n.rc-select-enabled .rc-select-selection__choice__disabled:hover .rc-select-selection__choice__content {\n  margin-left: 0;\n  margin-right: 0;\n}\n.rc-select .rc-select-selection__choice {\n  background-color: #f3f3f3;\n  border-radius: 4px;\n  float: left;\n  padding: 0 15px;\n  margin-right: 4px;\n  position: relative;\n  overflow: hidden;\n  transition: padding 0.3s cubic-bezier(0.6, -0.28, 0.735, 0.045), width 0.3s cubic-bezier(0.6, -0.28, 0.735, 0.045);\n}\n.rc-select .rc-select-selection__choice__content {\n  margin-left: 0;\n  margin-right: 0;\n  transition: margin 0.3s cubic-bezier(0.165, 0.84, 0.44, 1);\n}\n.rc-select .rc-select-selection__choice-zoom-enter,\n.rc-select .rc-select-selection__choice-zoom-appear,\n.rc-select .rc-select-selection__choice-zoom-leave {\n  -webkit-animation-duration: 0.3s;\n          animation-duration: 0.3s;\n  -webkit-animation-fill-mode: both;\n          animation-fill-mode: both;\n  transform-origin: 0 0;\n  opacity: 0;\n  -webkit-animation-play-state: paused;\n          animation-play-state: paused;\n  -webkit-animation-timing-function: cubic-bezier(0.175, 0.885, 0.32, 1.275);\n          animation-timing-function: cubic-bezier(0.175, 0.885, 0.32, 1.275);\n}\n.rc-select .rc-select-selection__choice-zoom-leave {\n  opacity: 1;\n  -webkit-animation-timing-function: cubic-bezier(0.6, -0.28, 0.735, 0.045);\n          animation-timing-function: cubic-bezier(0.6, -0.28, 0.735, 0.045);\n}\n.rc-select .rc-select-selection__choice-zoom-enter.rc-select-selection__choice-zoom-enter-active,\n.rc-select .rc-select-selection__choice-zoom-appear.rc-select-selection__choice-zoom-appear-active {\n  -webkit-animation-play-state: running;\n          animation-play-state: running;\n  -webkit-animation-name: rcSelectChoiceZoomIn;\n          animation-name: rcSelectChoiceZoomIn;\n}\n.rc-select .rc-select-selection__choice-zoom-leave.rc-select-selection__choice-zoom-leave-active {\n  -webkit-animation-play-state: running;\n          animation-play-state: running;\n  -webkit-animation-name: rcSelectChoiceZoomOut;\n          animation-name: rcSelectChoiceZoomOut;\n}\n@-webkit-keyframes rcSelectChoiceZoomIn {\n  0% {\n    transform: scale(0.6);\n    opacity: 0;\n  }\n  100% {\n    transform: scale(1);\n    opacity: 1;\n  }\n}\n@keyframes rcSelectChoiceZoomIn {\n  0% {\n    transform: scale(0.6);\n    opacity: 0;\n  }\n  100% {\n    transform: scale(1);\n    opacity: 1;\n  }\n}\n@-webkit-keyframes rcSelectChoiceZoomOut {\n  to {\n    transform: scale(0);\n    opacity: 0;\n  }\n}\n@keyframes rcSelectChoiceZoomOut {\n  to {\n    transform: scale(0);\n    opacity: 0;\n  }\n}\n.rc-select .rc-select-selection__choice__remove {\n  color: #919191;\n  cursor: pointer;\n  font-weight: bold;\n  padding: 0 0 0 8px;\n  position: absolute;\n  opacity: 0;\n  transform: scale(0);\n  top: 0;\n  right: 2px;\n  transition: opacity 0.3s, transform 0.3s;\n}\n.rc-select .rc-select-selection__choice__remove-icon {\n  font-style: normal;\n}\n.rc-select .rc-select-selection__choice__remove:hover {\n  color: #333;\n}\n.rc-select-dropdown {\n  background-color: white;\n  border: 1px solid #d9d9d9;\n  box-shadow: 0 0px 4px #d9d9d9;\n  border-radius: 4px;\n  box-sizing: border-box;\n  z-index: 100;\n  left: -9999px;\n  top: -9999px;\n  position: absolute;\n  outline: none;\n}\n.rc-select-dropdown:empty,\n.rc-select-dropdown-hidden {\n  display: none;\n}\n.rc-select-dropdown-menu {\n  outline: none;\n  margin: 0;\n  padding: 0;\n  list-style: none;\n  z-index: 9999;\n}\n.rc-select-dropdown-menu > li {\n  margin: 0;\n  padding: 0;\n}\n.rc-select-dropdown-menu-item-group-list {\n  margin: 0;\n  padding: 0;\n}\n.rc-select-dropdown-menu-item-group-list > li.rc-select-menu-item {\n  padding-left: 20px;\n}\n.rc-select-dropdown-menu-item-group-title {\n  color: #999;\n  line-height: 1.5;\n  padding: 8px 10px;\n  border-bottom: 1px solid #dedede;\n}\nli.rc-select-dropdown-menu-item {\n  margin: 0;\n  position: relative;\n  display: block;\n  padding: 7px 10px;\n  font-weight: normal;\n  color: #666;\n  white-space: nowrap;\n}\nli.rc-select-dropdown-menu-item-disabled {\n  color: #ccc;\n  cursor: not-allowed;\n}\nli.rc-select-dropdown-menu-item-selected {\n  color: #666;\n  background-color: #ddd;\n}\nli.rc-select-dropdown-menu-item-active {\n  background-color: #5897fb;\n  color: white;\n  cursor: pointer;\n}\nli.rc-select-dropdown-menu-item-divider {\n  height: 1px;\n  margin: 1px 0;\n  overflow: hidden;\n  background-color: #e5e5e5;\n  line-height: 0;\n}\n.rc-select-dropdown-slide-up-enter,\n.rc-select-dropdown-slide-up-appear {\n  -webkit-animation-duration: 0.3s;\n          animation-duration: 0.3s;\n  -webkit-animation-fill-mode: both;\n          animation-fill-mode: both;\n  transform-origin: 0 0;\n  opacity: 0;\n  -webkit-animation-timing-function: cubic-bezier(0.08, 0.82, 0.17, 1);\n          animation-timing-function: cubic-bezier(0.08, 0.82, 0.17, 1);\n  -webkit-animation-play-state: paused;\n          animation-play-state: paused;\n}\n.rc-select-dropdown-slide-up-leave {\n  -webkit-animation-duration: 0.3s;\n          animation-duration: 0.3s;\n  -webkit-animation-fill-mode: both;\n          animation-fill-mode: both;\n  transform-origin: 0 0;\n  opacity: 1;\n  -webkit-animation-timing-function: cubic-bezier(0.6, 0.04, 0.98, 0.34);\n          animation-timing-function: cubic-bezier(0.6, 0.04, 0.98, 0.34);\n  -webkit-animation-play-state: paused;\n          animation-play-state: paused;\n}\n.rc-select-dropdown-slide-up-enter.rc-select-dropdown-slide-up-enter-active.rc-select-dropdown-placement-bottomLeft,\n.rc-select-dropdown-slide-up-appear.rc-select-dropdown-slide-up-appear-active.rc-select-dropdown-placement-bottomLeft {\n  -webkit-animation-name: rcSelectDropdownSlideUpIn;\n          animation-name: rcSelectDropdownSlideUpIn;\n  -webkit-animation-play-state: running;\n          animation-play-state: running;\n}\n.rc-select-dropdown-slide-up-leave.rc-select-dropdown-slide-up-leave-active.rc-select-dropdown-placement-bottomLeft {\n  -webkit-animation-name: rcSelectDropdownSlideUpOut;\n          animation-name: rcSelectDropdownSlideUpOut;\n  -webkit-animation-play-state: running;\n          animation-play-state: running;\n}\n.rc-select-dropdown-slide-up-enter.rc-select-dropdown-slide-up-enter-active.rc-select-dropdown-placement-topLeft,\n.rc-select-dropdown-slide-up-appear.rc-select-dropdown-slide-up-appear-active.rc-select-dropdown-placement-topLeft {\n  -webkit-animation-name: rcSelectDropdownSlideDownIn;\n          animation-name: rcSelectDropdownSlideDownIn;\n  -webkit-animation-play-state: running;\n          animation-play-state: running;\n}\n.rc-select-dropdown-slide-up-leave.rc-select-dropdown-slide-up-leave-active.rc-select-dropdown-placement-topLeft {\n  -webkit-animation-name: rcSelectDropdownSlideDownOut;\n          animation-name: rcSelectDropdownSlideDownOut;\n  -webkit-animation-play-state: running;\n          animation-play-state: running;\n}\n@-webkit-keyframes rcSelectDropdownSlideUpIn {\n  0% {\n    opacity: 0;\n    transform-origin: 0% 0%;\n    transform: scaleY(0);\n  }\n  100% {\n    opacity: 1;\n    transform-origin: 0% 0%;\n    transform: scaleY(1);\n  }\n}\n@keyframes rcSelectDropdownSlideUpIn {\n  0% {\n    opacity: 0;\n    transform-origin: 0% 0%;\n    transform: scaleY(0);\n  }\n  100% {\n    opacity: 1;\n    transform-origin: 0% 0%;\n    transform: scaleY(1);\n  }\n}\n@-webkit-keyframes rcSelectDropdownSlideUpOut {\n  0% {\n    opacity: 1;\n    transform-origin: 0% 0%;\n    transform: scaleY(1);\n  }\n  100% {\n    opacity: 0;\n    transform-origin: 0% 0%;\n    transform: scaleY(0);\n  }\n}\n@keyframes rcSelectDropdownSlideUpOut {\n  0% {\n    opacity: 1;\n    transform-origin: 0% 0%;\n    transform: scaleY(1);\n  }\n  100% {\n    opacity: 0;\n    transform-origin: 0% 0%;\n    transform: scaleY(0);\n  }\n}\n@-webkit-keyframes rcSelectDropdownSlideDownIn {\n  0% {\n    opacity: 0;\n    transform-origin: 0% 100%;\n    transform: scaleY(0);\n  }\n  100% {\n    opacity: 1;\n    transform-origin: 0% 100%;\n    transform: scaleY(1);\n  }\n}\n@keyframes rcSelectDropdownSlideDownIn {\n  0% {\n    opacity: 0;\n    transform-origin: 0% 100%;\n    transform: scaleY(0);\n  }\n  100% {\n    opacity: 1;\n    transform-origin: 0% 100%;\n    transform: scaleY(1);\n  }\n}\n@-webkit-keyframes rcSelectDropdownSlideDownOut {\n  0% {\n    opacity: 1;\n    transform-origin: 0% 100%;\n    transform: scaleY(1);\n  }\n  100% {\n    opacity: 0;\n    transform-origin: 0% 100%;\n    transform: scaleY(0);\n  }\n}\n@keyframes rcSelectDropdownSlideDownOut {\n  0% {\n    opacity: 1;\n    transform-origin: 0% 100%;\n    transform: scaleY(1);\n  }\n  100% {\n    opacity: 0;\n    transform-origin: 0% 100%;\n    transform: scaleY(0);\n  }\n}\n.rc-select-open .rc-select-arrow b {\n  border-color: transparent transparent #888 transparent;\n  border-width: 0 4px 5px 4px;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".rc-slider {\n  position: relative;\n  height: 14px;\n  padding: 5px 0;\n  width: 100%;\n  border-radius: 6px;\n  -ms-touch-action: none;\n      touch-action: none;\n  box-sizing: border-box;\n  -webkit-tap-highlight-color: rgba(0, 0, 0, 0);\n}\n.rc-slider * {\n  box-sizing: border-box;\n  -webkit-tap-highlight-color: rgba(0, 0, 0, 0);\n}\n.rc-slider-rail {\n  position: absolute;\n  width: 100%;\n  background-color: #e9e9e9;\n  height: 4px;\n  border-radius: 6px;\n}\n.rc-slider-track {\n  position: absolute;\n  left: 0;\n  height: 4px;\n  border-radius: 6px;\n  background-color: #abe2fb;\n}\n.rc-slider-handle {\n  position: absolute;\n  margin-left: -7px;\n  margin-top: -5px;\n  width: 14px;\n  height: 14px;\n  cursor: pointer;\n  cursor: -webkit-grab;\n  cursor: grab;\n  border-radius: 50%;\n  border: solid 2px #96dbfa;\n  background-color: #fff;\n  -ms-touch-action: pan-x;\n      touch-action: pan-x;\n}\n.rc-slider-handle:focus {\n  border-color: #57c5f7;\n  box-shadow: 0 0 0 5px #96dbfa;\n  outline: none;\n}\n.rc-slider-handle-click-focused:focus {\n  border-color: #96dbfa;\n  box-shadow: unset;\n}\n.rc-slider-handle:hover {\n  border-color: #57c5f7;\n}\n.rc-slider-handle:active {\n  border-color: #57c5f7;\n  box-shadow: 0 0 5px #57c5f7;\n  cursor: -webkit-grabbing;\n  cursor: grabbing;\n}\n.rc-slider-mark {\n  position: absolute;\n  top: 18px;\n  left: 0;\n  width: 100%;\n  font-size: 12px;\n}\n.rc-slider-mark-text {\n  position: absolute;\n  display: inline-block;\n  vertical-align: middle;\n  text-align: center;\n  cursor: pointer;\n  color: #999;\n}\n.rc-slider-mark-text-active {\n  color: #666;\n}\n.rc-slider-step {\n  position: absolute;\n  width: 100%;\n  height: 4px;\n  background: transparent;\n}\n.rc-slider-dot {\n  position: absolute;\n  bottom: -2px;\n  margin-left: -4px;\n  width: 8px;\n  height: 8px;\n  border: 2px solid #e9e9e9;\n  background-color: #fff;\n  cursor: pointer;\n  border-radius: 50%;\n  vertical-align: middle;\n}\n.rc-slider-dot-active {\n  border-color: #96dbfa;\n}\n.rc-slider-disabled {\n  background-color: #e9e9e9;\n}\n.rc-slider-disabled .rc-slider-track {\n  background-color: #ccc;\n}\n.rc-slider-disabled .rc-slider-handle,\n.rc-slider-disabled .rc-slider-dot {\n  border-color: #ccc;\n  box-shadow: none;\n  background-color: #fff;\n  cursor: not-allowed;\n}\n.rc-slider-disabled .rc-slider-mark-text,\n.rc-slider-disabled .rc-slider-dot {\n  cursor: not-allowed !important;\n}\n.rc-slider-vertical {\n  width: 14px;\n  height: 100%;\n  padding: 0 5px;\n}\n.rc-slider-vertical .rc-slider-rail {\n  height: 100%;\n  width: 4px;\n}\n.rc-slider-vertical .rc-slider-track {\n  left: 5px;\n  bottom: 0;\n  width: 4px;\n}\n.rc-slider-vertical .rc-slider-handle {\n  margin-left: -5px;\n  margin-bottom: -7px;\n  -ms-touch-action: pan-y;\n      touch-action: pan-y;\n}\n.rc-slider-vertical .rc-slider-mark {\n  top: 0;\n  left: 18px;\n  height: 100%;\n}\n.rc-slider-vertical .rc-slider-step {\n  height: 100%;\n  width: 4px;\n}\n.rc-slider-vertical .rc-slider-dot {\n  left: 2px;\n  margin-bottom: -4px;\n}\n.rc-slider-vertical .rc-slider-dot:first-child {\n  margin-bottom: -4px;\n}\n.rc-slider-vertical .rc-slider-dot:last-child {\n  margin-bottom: -4px;\n}\n.rc-slider-tooltip-zoom-down-enter,\n.rc-slider-tooltip-zoom-down-appear {\n  animation-duration: .3s;\n  animation-fill-mode: both;\n  display: block !important;\n  animation-play-state: paused;\n}\n.rc-slider-tooltip-zoom-down-leave {\n  animation-duration: .3s;\n  animation-fill-mode: both;\n  display: block !important;\n  animation-play-state: paused;\n}\n.rc-slider-tooltip-zoom-down-enter.rc-slider-tooltip-zoom-down-enter-active,\n.rc-slider-tooltip-zoom-down-appear.rc-slider-tooltip-zoom-down-appear-active {\n  animation-name: rcSliderTooltipZoomDownIn;\n  animation-play-state: running;\n}\n.rc-slider-tooltip-zoom-down-leave.rc-slider-tooltip-zoom-down-leave-active {\n  animation-name: rcSliderTooltipZoomDownOut;\n  animation-play-state: running;\n}\n.rc-slider-tooltip-zoom-down-enter,\n.rc-slider-tooltip-zoom-down-appear {\n  transform: scale(0, 0);\n  animation-timing-function: cubic-bezier(0.23, 1, 0.32, 1);\n}\n.rc-slider-tooltip-zoom-down-leave {\n  animation-timing-function: cubic-bezier(0.755, 0.05, 0.855, 0.06);\n}\n@keyframes rcSliderTooltipZoomDownIn {\n  0% {\n    opacity: 0;\n    transform-origin: 50% 100%;\n    transform: scale(0, 0);\n  }\n  100% {\n    transform-origin: 50% 100%;\n    transform: scale(1, 1);\n  }\n}\n@keyframes rcSliderTooltipZoomDownOut {\n  0% {\n    transform-origin: 50% 100%;\n    transform: scale(1, 1);\n  }\n  100% {\n    opacity: 0;\n    transform-origin: 50% 100%;\n    transform: scale(0, 0);\n  }\n}\n.rc-slider-tooltip {\n  position: absolute;\n  left: -9999px;\n  top: -9999px;\n  visibility: visible;\n  box-sizing: border-box;\n  -webkit-tap-highlight-color: rgba(0, 0, 0, 0);\n}\n.rc-slider-tooltip * {\n  box-sizing: border-box;\n  -webkit-tap-highlight-color: rgba(0, 0, 0, 0);\n}\n.rc-slider-tooltip-hidden {\n  display: none;\n}\n.rc-slider-tooltip-placement-top {\n  padding: 4px 0 8px 0;\n}\n.rc-slider-tooltip-inner {\n  padding: 6px 2px;\n  min-width: 24px;\n  height: 24px;\n  font-size: 12px;\n  line-height: 1;\n  color: #fff;\n  text-align: center;\n  text-decoration: none;\n  background-color: #6c6c6c;\n  border-radius: 6px;\n  box-shadow: 0 0 4px #d9d9d9;\n}\n.rc-slider-tooltip-arrow {\n  position: absolute;\n  width: 0;\n  height: 0;\n  border-color: transparent;\n  border-style: solid;\n}\n.rc-slider-tooltip-placement-top .rc-slider-tooltip-arrow {\n  bottom: 4px;\n  left: 50%;\n  margin-left: -4px;\n  border-width: 4px 4px 0;\n  border-top-color: #6c6c6c;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n.p-Widget {\n  box-sizing: border-box;\n  position: relative;\n  overflow: hidden;\n  cursor: default;\n}\n\n\n.p-Widget.p-mod-hidden {\n  display: none !important;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n.p-CommandPalette {\n  display: flex;\n  flex-direction: column;\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n}\n\n\n.p-CommandPalette-search {\n  flex: 0 0 auto;\n}\n\n\n.p-CommandPalette-content {\n  flex: 1 1 auto;\n  margin: 0;\n  padding: 0;\n  min-height: 0;\n  overflow: auto;\n  list-style-type: none;\n}\n\n\n.p-CommandPalette-header {\n  overflow: hidden;\n  white-space: nowrap;\n  text-overflow: ellipsis;\n}\n\n\n.p-CommandPalette-item {\n  display: flex;\n  flex-direction: row;\n}\n\n\n.p-CommandPalette-itemIcon {\n  flex: 0 0 auto;\n}\n\n\n.p-CommandPalette-itemContent {\n  flex: 1 1 auto;\n  overflow: hidden;\n}\n\n\n.p-CommandPalette-itemShortcut {\n  flex: 0 0 auto;\n}\n\n\n.p-CommandPalette-itemLabel {\n  overflow: hidden;\n  white-space: nowrap;\n  text-overflow: ellipsis;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n.p-DockPanel {\n  z-index: 0;\n}\n\n\n.p-DockPanel-widget {\n  z-index: 0;\n}\n\n\n.p-DockPanel-tabBar {\n  z-index: 1;\n}\n\n\n.p-DockPanel-handle {\n  z-index: 2;\n}\n\n\n.p-DockPanel-handle.p-mod-hidden {\n  display: none !important;\n}\n\n\n.p-DockPanel-handle:after {\n  position: absolute;\n  top: 0;\n  left: 0;\n  width: 100%;\n  height: 100%;\n  content: '';\n}\n\n\n.p-DockPanel-handle[data-orientation='horizontal'] {\n  cursor: ew-resize;\n}\n\n\n.p-DockPanel-handle[data-orientation='vertical'] {\n  cursor: ns-resize;\n}\n\n\n.p-DockPanel-handle[data-orientation='horizontal']:after {\n  left: 50%;\n  min-width: 8px;\n  transform: translateX(-50%);\n}\n\n\n.p-DockPanel-handle[data-orientation='vertical']:after {\n  top: 50%;\n  min-height: 8px;\n  transform: translateY(-50%);\n}\n\n\n.p-DockPanel-overlay {\n  z-index: 3;\n  box-sizing: border-box;\n  pointer-events: none;\n}\n\n\n.p-DockPanel-overlay.p-mod-hidden {\n  display: none !important;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n.p-Menu {\n  z-index: 10000;\n  position: absolute;\n  white-space: nowrap;\n  overflow-x: hidden;\n  overflow-y: auto;\n  outline: none;\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n}\n\n\n.p-Menu-content {\n  margin: 0;\n  padding: 0;\n  display: table;\n  list-style-type: none;\n}\n\n\n.p-Menu-item {\n  display: table-row;\n}\n\n\n.p-Menu-item.p-mod-hidden,\n.p-Menu-item.p-mod-collapsed {\n  display: none !important;\n}\n\n\n.p-Menu-itemIcon,\n.p-Menu-itemSubmenuIcon {\n  display: table-cell;\n  text-align: center;\n}\n\n\n.p-Menu-itemLabel {\n  display: table-cell;\n  text-align: left;\n}\n\n\n.p-Menu-itemShortcut {\n  display: table-cell;\n  text-align: right;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n.p-MenuBar {\n  outline: none;\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n}\n\n\n.p-MenuBar-content {\n  margin: 0;\n  padding: 0;\n  display: flex;\n  flex-direction: row;\n  list-style-type: none;\n}\n\n\n.p-MenuBar-item {\n  box-sizing: border-box;\n}\n\n\n.p-MenuBar-itemIcon,\n.p-MenuBar-itemLabel {\n  display: inline-block;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n.p-ScrollBar {\n  display: flex;\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n}\n\n\n.p-ScrollBar[data-orientation='horizontal'] {\n  flex-direction: row;\n}\n\n\n.p-ScrollBar[data-orientation='vertical'] {\n  flex-direction: column;\n}\n\n\n.p-ScrollBar-button {\n  box-sizing: border-box;\n  flex: 0 0 auto;\n}\n\n\n.p-ScrollBar-track {\n  box-sizing: border-box;\n  position: relative;\n  overflow: hidden;\n  flex: 1 1 auto;\n}\n\n\n.p-ScrollBar-thumb {\n  box-sizing: border-box;\n  position: absolute;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n.p-SplitPanel-child {\n  z-index: 0;\n}\n\n\n.p-SplitPanel-handle {\n  z-index: 1;\n}\n\n\n.p-SplitPanel-handle.p-mod-hidden {\n  display: none !important;\n}\n\n\n.p-SplitPanel-handle:after {\n  position: absolute;\n  top: 0;\n  left: 0;\n  width: 100%;\n  height: 100%;\n  content: '';\n}\n\n\n.p-SplitPanel[data-orientation='horizontal'] > .p-SplitPanel-handle {\n  cursor: ew-resize;\n}\n\n\n.p-SplitPanel[data-orientation='vertical'] > .p-SplitPanel-handle {\n  cursor: ns-resize;\n}\n\n\n.p-SplitPanel[data-orientation='horizontal'] > .p-SplitPanel-handle:after {\n  left: 50%;\n  min-width: 8px;\n  transform: translateX(-50%);\n}\n\n\n.p-SplitPanel[data-orientation='vertical'] > .p-SplitPanel-handle:after {\n  top: 50%;\n  min-height: 8px;\n  transform: translateY(-50%);\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n.p-TabBar {\n  display: flex;\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n}\n\n\n.p-TabBar[data-orientation='horizontal'] {\n  flex-direction: row;\n}\n\n\n.p-TabBar[data-orientation='vertical'] {\n  flex-direction: column;\n}\n\n\n.p-TabBar-content {\n  margin: 0;\n  padding: 0;\n  display: flex;\n  flex: 1 1 auto;\n  list-style-type: none;\n}\n\n\n.p-TabBar[data-orientation='horizontal'] > .p-TabBar-content {\n  flex-direction: row;\n}\n\n\n.p-TabBar[data-orientation='vertical'] > .p-TabBar-content {\n  flex-direction: column;\n}\n\n\n.p-TabBar-tab {\n  display: flex;\n  flex-direction: row;\n  box-sizing: border-box;\n  overflow: hidden;\n}\n\n\n.p-TabBar-tabIcon,\n.p-TabBar-tabCloseIcon {\n  flex: 0 0 auto;\n}\n\n\n.p-TabBar-tabLabel {\n  flex: 1 1 auto;\n  overflow: hidden;\n  white-space: nowrap;\n}\n\n\n.p-TabBar-tab.p-mod-hidden {\n  display: none !important;\n}\n\n\n.p-TabBar.p-mod-dragging .p-TabBar-tab {\n  position: relative;\n}\n\n\n.p-TabBar.p-mod-dragging[data-orientation='horizontal'] .p-TabBar-tab {\n  left: 0;\n  transition: left 150ms ease;\n}\n\n\n.p-TabBar.p-mod-dragging[data-orientation='vertical'] .p-TabBar-tab {\n  top: 0;\n  transition: top 150ms ease;\n}\n\n\n.p-TabBar.p-mod-dragging .p-TabBar-tab.p-mod-dragging {\n  transition: none;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n.p-TabPanel-tabBar {\n  z-index: 1;\n}\n\n\n.p-TabPanel-stackedPanel {\n  z-index: 0;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,'@charset "UTF-8";\n/*!\n\nCopyright 2015-present Palantir Technologies, Inc. All rights reserved.\nLicensed under the Apache License, Version 2.0.\n\n*/\nhtml{\n  -webkit-box-sizing:border-box;\n          box-sizing:border-box; }\n\n*,\n*::before,\n*::after{\n  -webkit-box-sizing:inherit;\n          box-sizing:inherit; }\n\nbody{\n  text-transform:none;\n  line-height:1.28581;\n  letter-spacing:0;\n  font-size:14px;\n  font-weight:400;\n  color:#182026;\n  font-family:-apple-system, "BlinkMacSystemFont", "Segoe UI", "Roboto", "Oxygen", "Ubuntu", "Cantarell", "Open Sans", "Helvetica Neue", "Icons16", sans-serif; }\n\np{\n  margin-top:0;\n  margin-bottom:10px; }\n\nsmall{\n  font-size:12px; }\n\nstrong{\n  font-weight:600; }\n\n::-moz-selection{\n  background:rgba(125, 188, 255, 0.6); }\n\n::selection{\n  background:rgba(125, 188, 255, 0.6); }\n.bp3-heading{\n  color:#182026;\n  font-weight:600;\n  margin:0 0 10px;\n  padding:0; }\n  .bp3-dark .bp3-heading{\n    color:#f5f8fa; }\n\nh1.bp3-heading, .bp3-running-text h1{\n  line-height:40px;\n  font-size:36px; }\n\nh2.bp3-heading, .bp3-running-text h2{\n  line-height:32px;\n  font-size:28px; }\n\nh3.bp3-heading, .bp3-running-text h3{\n  line-height:25px;\n  font-size:22px; }\n\nh4.bp3-heading, .bp3-running-text h4{\n  line-height:21px;\n  font-size:18px; }\n\nh5.bp3-heading, .bp3-running-text h5{\n  line-height:19px;\n  font-size:16px; }\n\nh6.bp3-heading, .bp3-running-text h6{\n  line-height:16px;\n  font-size:14px; }\n.bp3-ui-text{\n  text-transform:none;\n  line-height:1.28581;\n  letter-spacing:0;\n  font-size:14px;\n  font-weight:400; }\n\n.bp3-monospace-text{\n  text-transform:none;\n  font-family:monospace; }\n\n.bp3-text-muted{\n  color:#5c7080; }\n  .bp3-dark .bp3-text-muted{\n    color:#a7b6c2; }\n\n.bp3-text-disabled{\n  color:rgba(92, 112, 128, 0.6); }\n  .bp3-dark .bp3-text-disabled{\n    color:rgba(167, 182, 194, 0.6); }\n\n.bp3-text-overflow-ellipsis{\n  overflow:hidden;\n  text-overflow:ellipsis;\n  white-space:nowrap;\n  word-wrap:normal; }\n.bp3-running-text{\n  line-height:1.5;\n  font-size:14px; }\n  .bp3-running-text h1{\n    color:#182026;\n    font-weight:600;\n    margin-top:40px;\n    margin-bottom:20px; }\n    .bp3-dark .bp3-running-text h1{\n      color:#f5f8fa; }\n  .bp3-running-text h2{\n    color:#182026;\n    font-weight:600;\n    margin-top:40px;\n    margin-bottom:20px; }\n    .bp3-dark .bp3-running-text h2{\n      color:#f5f8fa; }\n  .bp3-running-text h3{\n    color:#182026;\n    font-weight:600;\n    margin-top:40px;\n    margin-bottom:20px; }\n    .bp3-dark .bp3-running-text h3{\n      color:#f5f8fa; }\n  .bp3-running-text h4{\n    color:#182026;\n    font-weight:600;\n    margin-top:40px;\n    margin-bottom:20px; }\n    .bp3-dark .bp3-running-text h4{\n      color:#f5f8fa; }\n  .bp3-running-text h5{\n    color:#182026;\n    font-weight:600;\n    margin-top:40px;\n    margin-bottom:20px; }\n    .bp3-dark .bp3-running-text h5{\n      color:#f5f8fa; }\n  .bp3-running-text h6{\n    color:#182026;\n    font-weight:600;\n    margin-top:40px;\n    margin-bottom:20px; }\n    .bp3-dark .bp3-running-text h6{\n      color:#f5f8fa; }\n  .bp3-running-text hr{\n    margin:20px 0;\n    border:none;\n    border-bottom:1px solid rgba(16, 22, 26, 0.15); }\n    .bp3-dark .bp3-running-text hr{\n      border-color:rgba(255, 255, 255, 0.15); }\n  .bp3-running-text p{\n    margin:0 0 10px;\n    padding:0; }\n\n.bp3-text-large{\n  font-size:16px; }\n\n.bp3-text-small{\n  font-size:12px; }\na{\n  text-decoration:none;\n  color:#106ba3; }\n  a:hover{\n    cursor:pointer;\n    text-decoration:underline;\n    color:#106ba3; }\n  a .bp3-icon, a .bp3-icon-standard, a .bp3-icon-large{\n    color:inherit; }\n  a code,\n  .bp3-dark a code{\n    color:inherit; }\n  .bp3-dark a,\n  .bp3-dark a:hover{\n    color:#48aff0; }\n    .bp3-dark a .bp3-icon, .bp3-dark a .bp3-icon-standard, .bp3-dark a .bp3-icon-large,\n    .bp3-dark a:hover .bp3-icon,\n    .bp3-dark a:hover .bp3-icon-standard,\n    .bp3-dark a:hover .bp3-icon-large{\n      color:inherit; }\n.bp3-running-text code, .bp3-code{\n  text-transform:none;\n  font-family:monospace;\n  border-radius:3px;\n  -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2);\n          box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2);\n  background:rgba(255, 255, 255, 0.7);\n  padding:2px 5px;\n  color:#5c7080;\n  font-size:smaller; }\n  .bp3-dark .bp3-running-text code, .bp3-running-text .bp3-dark code, .bp3-dark .bp3-code{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4);\n    background:rgba(16, 22, 26, 0.3);\n    color:#a7b6c2; }\n  .bp3-running-text a > code, a > .bp3-code{\n    color:#137cbd; }\n    .bp3-dark .bp3-running-text a > code, .bp3-running-text .bp3-dark a > code, .bp3-dark a > .bp3-code{\n      color:inherit; }\n\n.bp3-running-text pre, .bp3-code-block{\n  text-transform:none;\n  font-family:monospace;\n  display:block;\n  margin:10px 0;\n  border-radius:3px;\n  -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.15);\n          box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.15);\n  background:rgba(255, 255, 255, 0.7);\n  padding:13px 15px 12px;\n  line-height:1.4;\n  color:#182026;\n  font-size:13px;\n  word-break:break-all;\n  word-wrap:break-word; }\n  .bp3-dark .bp3-running-text pre, .bp3-running-text .bp3-dark pre, .bp3-dark .bp3-code-block{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4);\n    background:rgba(16, 22, 26, 0.3);\n    color:#f5f8fa; }\n  .bp3-running-text pre > code, .bp3-code-block > code{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:none;\n    padding:0;\n    color:inherit;\n    font-size:inherit; }\n\n.bp3-running-text kbd, .bp3-key{\n  display:-webkit-inline-box;\n  display:-webkit-inline-flex;\n  display:-ms-inline-flexbox;\n  display:inline-flex;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  -webkit-box-pack:center;\n  -webkit-justify-content:center;\n      -ms-flex-pack:center;\n          justify-content:center;\n  border-radius:3px;\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.2);\n  background:#ffffff;\n  min-width:24px;\n  height:24px;\n  padding:3px 6px;\n  vertical-align:middle;\n  line-height:24px;\n  color:#5c7080;\n  font-family:inherit;\n  font-size:12px; }\n  .bp3-running-text kbd .bp3-icon, .bp3-key .bp3-icon, .bp3-running-text kbd .bp3-icon-standard, .bp3-key .bp3-icon-standard, .bp3-running-text kbd .bp3-icon-large, .bp3-key .bp3-icon-large{\n    margin-right:5px; }\n  .bp3-dark .bp3-running-text kbd, .bp3-running-text .bp3-dark kbd, .bp3-dark .bp3-key{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.4);\n    background:#394b59;\n    color:#a7b6c2; }\n.bp3-running-text blockquote, .bp3-blockquote{\n  margin:0 0 10px;\n  border-left:solid 4px rgba(167, 182, 194, 0.5);\n  padding:0 20px; }\n  .bp3-dark .bp3-running-text blockquote, .bp3-running-text .bp3-dark blockquote, .bp3-dark .bp3-blockquote{\n    border-color:rgba(115, 134, 148, 0.5); }\n.bp3-running-text ul,\n.bp3-running-text ol, .bp3-list{\n  margin:10px 0;\n  padding-left:30px; }\n  .bp3-running-text ul li:not(:last-child), .bp3-running-text ol li:not(:last-child), .bp3-list li:not(:last-child){\n    margin-bottom:5px; }\n  .bp3-running-text ul ol, .bp3-running-text ol ol, .bp3-list ol,\n  .bp3-running-text ul ul,\n  .bp3-running-text ol ul,\n  .bp3-list ul{\n    margin-top:5px; }\n\n.bp3-list-unstyled{\n  margin:0;\n  padding:0;\n  list-style:none; }\n  .bp3-list-unstyled li{\n    padding:0; }\n.bp3-rtl{\n  text-align:right; }\n\n.bp3-dark{\n  color:#f5f8fa; }\n\n:focus{\n  outline:rgba(19, 124, 189, 0.6) auto 2px;\n  outline-offset:2px;\n  -moz-outline-radius:6px; }\n\n.bp3-focus-disabled :focus{\n  outline:none !important; }\n  .bp3-focus-disabled :focus ~ .bp3-control-indicator{\n    outline:none !important; }\n\n.bp3-alert{\n  max-width:400px;\n  padding:20px; }\n\n.bp3-alert-body{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex; }\n  .bp3-alert-body .bp3-icon{\n    margin-top:0;\n    margin-right:20px;\n    font-size:40px; }\n\n.bp3-alert-footer{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-orient:horizontal;\n  -webkit-box-direction:reverse;\n  -webkit-flex-direction:row-reverse;\n      -ms-flex-direction:row-reverse;\n          flex-direction:row-reverse;\n  margin-top:10px; }\n  .bp3-alert-footer .bp3-button{\n    margin-left:10px; }\n.bp3-breadcrumbs{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-flex-wrap:wrap;\n      -ms-flex-wrap:wrap;\n          flex-wrap:wrap;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  margin:0;\n  cursor:default;\n  height:30px;\n  padding:0;\n  list-style:none; }\n  .bp3-breadcrumbs > li{\n    display:-webkit-box;\n    display:-webkit-flex;\n    display:-ms-flexbox;\n    display:flex;\n    -webkit-box-align:center;\n    -webkit-align-items:center;\n        -ms-flex-align:center;\n            align-items:center; }\n    .bp3-breadcrumbs > li::after{\n      display:block;\n      margin:0 5px;\n      background:url("data:image/svg+xml,%3csvg xmlns=\'http://www.w3.org/2000/svg\' viewBox=\'0 0 16 16\'%3e%3cpath fill-rule=\'evenodd\' clip-rule=\'evenodd\' d=\'M10.71 7.29l-4-4a1.003 1.003 0 0 0-1.42 1.42L8.59 8 5.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 0 0 1.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z\' fill=\'%235C7080\'/%3e%3c/svg%3e");\n      width:16px;\n      height:16px;\n      content:""; }\n    .bp3-breadcrumbs > li:last-of-type::after{\n      display:none; }\n\n.bp3-breadcrumb,\n.bp3-breadcrumb-current,\n.bp3-breadcrumbs-collapsed{\n  display:inline-block;\n  font-size:16px; }\n\n.bp3-breadcrumb,\n.bp3-breadcrumbs-collapsed{\n  color:#5c7080; }\n\n.bp3-breadcrumb:hover{\n  text-decoration:none; }\n\n.bp3-breadcrumb.bp3-disabled{\n  cursor:not-allowed;\n  color:rgba(92, 112, 128, 0.6); }\n\n.bp3-breadcrumb-current{\n  color:inherit;\n  font-weight:600; }\n  .bp3-breadcrumb-current .bp3-input{\n    vertical-align:baseline;\n    font-size:inherit;\n    font-weight:inherit; }\n\n.bp3-breadcrumbs-collapsed{\n  margin-right:2px;\n  border:none;\n  border-radius:3px;\n  background:#ced9e0;\n  cursor:pointer;\n  padding:1px 5px;\n  vertical-align:text-bottom; }\n  .bp3-breadcrumbs-collapsed::before{\n    display:block;\n    background:url("data:image/svg+xml,%3csvg xmlns=\'http://www.w3.org/2000/svg\' viewBox=\'0 0 16 16\'%3e%3cg fill=\'%235C7080\'%3e%3ccircle cx=\'2\' cy=\'8.03\' r=\'2\'/%3e%3ccircle cx=\'14\' cy=\'8.03\' r=\'2\'/%3e%3ccircle cx=\'8\' cy=\'8.03\' r=\'2\'/%3e%3c/g%3e%3c/svg%3e") center no-repeat;\n    width:16px;\n    height:16px;\n    content:""; }\n  .bp3-breadcrumbs-collapsed:hover{\n    background:#bfccd6;\n    text-decoration:none;\n    color:#182026; }\n\n.bp3-dark .bp3-breadcrumb,\n.bp3-dark .bp3-breadcrumbs-collapsed{\n  color:#a7b6c2; }\n\n.bp3-dark .bp3-breadcrumbs > li::after{\n  color:#a7b6c2; }\n\n.bp3-dark .bp3-breadcrumb.bp3-disabled{\n  color:rgba(167, 182, 194, 0.6); }\n\n.bp3-dark .bp3-breadcrumb-current{\n  color:#f5f8fa; }\n\n.bp3-dark .bp3-breadcrumbs-collapsed{\n  background:rgba(16, 22, 26, 0.4); }\n  .bp3-dark .bp3-breadcrumbs-collapsed:hover{\n    background:rgba(16, 22, 26, 0.6);\n    color:#f5f8fa; }\n.bp3-button{\n  display:-webkit-inline-box;\n  display:-webkit-inline-flex;\n  display:-ms-inline-flexbox;\n  display:inline-flex;\n  -webkit-box-orient:horizontal;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:row;\n      -ms-flex-direction:row;\n          flex-direction:row;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  -webkit-box-pack:center;\n  -webkit-justify-content:center;\n      -ms-flex-pack:center;\n          justify-content:center;\n  border:none;\n  border-radius:3px;\n  cursor:pointer;\n  padding:5px 10px;\n  vertical-align:middle;\n  text-align:left;\n  font-size:14px;\n  min-width:30px;\n  min-height:30px; }\n  .bp3-button > *{\n    -webkit-box-flex:0;\n    -webkit-flex-grow:0;\n        -ms-flex-positive:0;\n            flex-grow:0;\n    -webkit-flex-shrink:0;\n        -ms-flex-negative:0;\n            flex-shrink:0; }\n  .bp3-button > .bp3-fill{\n    -webkit-box-flex:1;\n    -webkit-flex-grow:1;\n        -ms-flex-positive:1;\n            flex-grow:1;\n    -webkit-flex-shrink:1;\n        -ms-flex-negative:1;\n            flex-shrink:1; }\n  .bp3-button::before,\n  .bp3-button > *{\n    margin-right:7px; }\n  .bp3-button:empty::before,\n  .bp3-button > :last-child{\n    margin-right:0; }\n  .bp3-button:empty{\n    padding:0 !important; }\n  .bp3-button:disabled, .bp3-button.bp3-disabled{\n    cursor:not-allowed; }\n  .bp3-button.bp3-fill{\n    display:-webkit-box;\n    display:-webkit-flex;\n    display:-ms-flexbox;\n    display:flex;\n    width:100%; }\n  .bp3-button.bp3-align-right,\n  .bp3-align-right .bp3-button{\n    text-align:right; }\n  .bp3-button:not([class*="bp3-intent-"]){\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n    background-color:#f5f8fa;\n    background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.8)), to(rgba(255, 255, 255, 0)));\n    background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.8), rgba(255, 255, 255, 0));\n    color:#182026; }\n    .bp3-button:not([class*="bp3-intent-"]):hover{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n      background-clip:padding-box;\n      background-color:#ebf1f5; }\n    .bp3-button:not([class*="bp3-intent-"]):active, .bp3-button:not([class*="bp3-intent-"]).bp3-active{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n      background-color:#d8e1e8;\n      background-image:none; }\n    .bp3-button:not([class*="bp3-intent-"]):disabled, .bp3-button:not([class*="bp3-intent-"]).bp3-disabled{\n      outline:none;\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background-color:rgba(206, 217, 224, 0.5);\n      background-image:none;\n      cursor:not-allowed;\n      color:rgba(92, 112, 128, 0.6); }\n      .bp3-button:not([class*="bp3-intent-"]):disabled.bp3-active, .bp3-button:not([class*="bp3-intent-"]):disabled.bp3-active:hover, .bp3-button:not([class*="bp3-intent-"]).bp3-disabled.bp3-active, .bp3-button:not([class*="bp3-intent-"]).bp3-disabled.bp3-active:hover{\n        background:rgba(206, 217, 224, 0.7); }\n  .bp3-button.bp3-intent-primary{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n    background-color:#137cbd;\n    background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.1)), to(rgba(255, 255, 255, 0)));\n    background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.1), rgba(255, 255, 255, 0));\n    color:#ffffff; }\n    .bp3-button.bp3-intent-primary:hover, .bp3-button.bp3-intent-primary:active, .bp3-button.bp3-intent-primary.bp3-active{\n      color:#ffffff; }\n    .bp3-button.bp3-intent-primary:hover{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n      background-color:#106ba3; }\n    .bp3-button.bp3-intent-primary:active, .bp3-button.bp3-intent-primary.bp3-active{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n      background-color:#0e5a8a;\n      background-image:none; }\n    .bp3-button.bp3-intent-primary:disabled, .bp3-button.bp3-intent-primary.bp3-disabled{\n      border-color:transparent;\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background-color:rgba(19, 124, 189, 0.5);\n      background-image:none;\n      color:rgba(255, 255, 255, 0.6); }\n  .bp3-button.bp3-intent-success{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n    background-color:#0f9960;\n    background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.1)), to(rgba(255, 255, 255, 0)));\n    background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.1), rgba(255, 255, 255, 0));\n    color:#ffffff; }\n    .bp3-button.bp3-intent-success:hover, .bp3-button.bp3-intent-success:active, .bp3-button.bp3-intent-success.bp3-active{\n      color:#ffffff; }\n    .bp3-button.bp3-intent-success:hover{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n      background-color:#0d8050; }\n    .bp3-button.bp3-intent-success:active, .bp3-button.bp3-intent-success.bp3-active{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n      background-color:#0a6640;\n      background-image:none; }\n    .bp3-button.bp3-intent-success:disabled, .bp3-button.bp3-intent-success.bp3-disabled{\n      border-color:transparent;\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background-color:rgba(15, 153, 96, 0.5);\n      background-image:none;\n      color:rgba(255, 255, 255, 0.6); }\n  .bp3-button.bp3-intent-warning{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n    background-color:#d9822b;\n    background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.1)), to(rgba(255, 255, 255, 0)));\n    background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.1), rgba(255, 255, 255, 0));\n    color:#ffffff; }\n    .bp3-button.bp3-intent-warning:hover, .bp3-button.bp3-intent-warning:active, .bp3-button.bp3-intent-warning.bp3-active{\n      color:#ffffff; }\n    .bp3-button.bp3-intent-warning:hover{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n      background-color:#bf7326; }\n    .bp3-button.bp3-intent-warning:active, .bp3-button.bp3-intent-warning.bp3-active{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n      background-color:#a66321;\n      background-image:none; }\n    .bp3-button.bp3-intent-warning:disabled, .bp3-button.bp3-intent-warning.bp3-disabled{\n      border-color:transparent;\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background-color:rgba(217, 130, 43, 0.5);\n      background-image:none;\n      color:rgba(255, 255, 255, 0.6); }\n  .bp3-button.bp3-intent-danger{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n    background-color:#db3737;\n    background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.1)), to(rgba(255, 255, 255, 0)));\n    background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.1), rgba(255, 255, 255, 0));\n    color:#ffffff; }\n    .bp3-button.bp3-intent-danger:hover, .bp3-button.bp3-intent-danger:active, .bp3-button.bp3-intent-danger.bp3-active{\n      color:#ffffff; }\n    .bp3-button.bp3-intent-danger:hover{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n      background-color:#c23030; }\n    .bp3-button.bp3-intent-danger:active, .bp3-button.bp3-intent-danger.bp3-active{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n      background-color:#a82a2a;\n      background-image:none; }\n    .bp3-button.bp3-intent-danger:disabled, .bp3-button.bp3-intent-danger.bp3-disabled{\n      border-color:transparent;\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background-color:rgba(219, 55, 55, 0.5);\n      background-image:none;\n      color:rgba(255, 255, 255, 0.6); }\n  .bp3-button[class*="bp3-intent-"] .bp3-button-spinner .bp3-spinner-head{\n    stroke:#ffffff; }\n  .bp3-button.bp3-large,\n  .bp3-large .bp3-button{\n    min-width:40px;\n    min-height:40px;\n    padding:5px 15px;\n    font-size:16px; }\n    .bp3-button.bp3-large::before,\n    .bp3-button.bp3-large > *,\n    .bp3-large .bp3-button::before,\n    .bp3-large .bp3-button > *{\n      margin-right:10px; }\n    .bp3-button.bp3-large:empty::before,\n    .bp3-button.bp3-large > :last-child,\n    .bp3-large .bp3-button:empty::before,\n    .bp3-large .bp3-button > :last-child{\n      margin-right:0; }\n  .bp3-button.bp3-small,\n  .bp3-small .bp3-button{\n    min-width:24px;\n    min-height:24px;\n    padding:0 7px; }\n  .bp3-button.bp3-loading{\n    position:relative; }\n    .bp3-button.bp3-loading[class*="bp3-icon-"]::before{\n      visibility:hidden; }\n    .bp3-button.bp3-loading .bp3-button-spinner{\n      position:absolute;\n      margin:0; }\n    .bp3-button.bp3-loading > :not(.bp3-button-spinner){\n      visibility:hidden; }\n  .bp3-button[class*="bp3-icon-"]::before{\n    line-height:1;\n    font-family:"Icons16", sans-serif;\n    font-size:16px;\n    font-weight:400;\n    font-style:normal;\n    -moz-osx-font-smoothing:grayscale;\n    -webkit-font-smoothing:antialiased;\n    color:#5c7080; }\n  .bp3-button .bp3-icon, .bp3-button .bp3-icon-standard, .bp3-button .bp3-icon-large{\n    color:#5c7080; }\n    .bp3-button .bp3-icon.bp3-align-right, .bp3-button .bp3-icon-standard.bp3-align-right, .bp3-button .bp3-icon-large.bp3-align-right{\n      margin-left:7px; }\n  .bp3-button .bp3-icon:first-child:last-child,\n  .bp3-button .bp3-spinner + .bp3-icon:last-child{\n    margin:0 -7px; }\n  .bp3-dark .bp3-button:not([class*="bp3-intent-"]){\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n    background-color:#394b59;\n    background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.05)), to(rgba(255, 255, 255, 0)));\n    background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.05), rgba(255, 255, 255, 0));\n    color:#f5f8fa; }\n    .bp3-dark .bp3-button:not([class*="bp3-intent-"]):hover, .bp3-dark .bp3-button:not([class*="bp3-intent-"]):active, .bp3-dark .bp3-button:not([class*="bp3-intent-"]).bp3-active{\n      color:#f5f8fa; }\n    .bp3-dark .bp3-button:not([class*="bp3-intent-"]):hover{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n      background-color:#30404d; }\n    .bp3-dark .bp3-button:not([class*="bp3-intent-"]):active, .bp3-dark .bp3-button:not([class*="bp3-intent-"]).bp3-active{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.6), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.6), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n      background-color:#202b33;\n      background-image:none; }\n    .bp3-dark .bp3-button:not([class*="bp3-intent-"]):disabled, .bp3-dark .bp3-button:not([class*="bp3-intent-"]).bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background-color:rgba(57, 75, 89, 0.5);\n      background-image:none;\n      color:rgba(167, 182, 194, 0.6); }\n      .bp3-dark .bp3-button:not([class*="bp3-intent-"]):disabled.bp3-active, .bp3-dark .bp3-button:not([class*="bp3-intent-"]).bp3-disabled.bp3-active{\n        background:rgba(57, 75, 89, 0.7); }\n    .bp3-dark .bp3-button:not([class*="bp3-intent-"]) .bp3-button-spinner .bp3-spinner-head{\n      background:rgba(16, 22, 26, 0.5);\n      stroke:#8a9ba8; }\n    .bp3-dark .bp3-button:not([class*="bp3-intent-"])[class*="bp3-icon-"]::before{\n      color:#a7b6c2; }\n    .bp3-dark .bp3-button:not([class*="bp3-intent-"]) .bp3-icon, .bp3-dark .bp3-button:not([class*="bp3-intent-"]) .bp3-icon-standard, .bp3-dark .bp3-button:not([class*="bp3-intent-"]) .bp3-icon-large{\n      color:#a7b6c2; }\n  .bp3-dark .bp3-button[class*="bp3-intent-"]{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4); }\n    .bp3-dark .bp3-button[class*="bp3-intent-"]:hover{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4); }\n    .bp3-dark .bp3-button[class*="bp3-intent-"]:active, .bp3-dark .bp3-button[class*="bp3-intent-"].bp3-active{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2); }\n    .bp3-dark .bp3-button[class*="bp3-intent-"]:disabled, .bp3-dark .bp3-button[class*="bp3-intent-"].bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background-image:none;\n      color:rgba(255, 255, 255, 0.3); }\n    .bp3-dark .bp3-button[class*="bp3-intent-"] .bp3-button-spinner .bp3-spinner-head{\n      stroke:#8a9ba8; }\n  .bp3-button:disabled::before,\n  .bp3-button:disabled .bp3-icon, .bp3-button:disabled .bp3-icon-standard, .bp3-button:disabled .bp3-icon-large, .bp3-button.bp3-disabled::before,\n  .bp3-button.bp3-disabled .bp3-icon, .bp3-button.bp3-disabled .bp3-icon-standard, .bp3-button.bp3-disabled .bp3-icon-large, .bp3-button[class*="bp3-intent-"]::before,\n  .bp3-button[class*="bp3-intent-"] .bp3-icon, .bp3-button[class*="bp3-intent-"] .bp3-icon-standard, .bp3-button[class*="bp3-intent-"] .bp3-icon-large{\n    color:inherit !important; }\n  .bp3-button.bp3-minimal{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:none; }\n    .bp3-button.bp3-minimal:hover{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:rgba(167, 182, 194, 0.3);\n      text-decoration:none;\n      color:#182026; }\n    .bp3-button.bp3-minimal:active, .bp3-button.bp3-minimal.bp3-active{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:rgba(115, 134, 148, 0.3);\n      color:#182026; }\n    .bp3-button.bp3-minimal:disabled, .bp3-button.bp3-minimal:disabled:hover, .bp3-button.bp3-minimal.bp3-disabled, .bp3-button.bp3-minimal.bp3-disabled:hover{\n      background:none;\n      cursor:not-allowed;\n      color:rgba(92, 112, 128, 0.6); }\n      .bp3-button.bp3-minimal:disabled.bp3-active, .bp3-button.bp3-minimal:disabled:hover.bp3-active, .bp3-button.bp3-minimal.bp3-disabled.bp3-active, .bp3-button.bp3-minimal.bp3-disabled:hover.bp3-active{\n        background:rgba(115, 134, 148, 0.3); }\n    .bp3-dark .bp3-button.bp3-minimal{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:none;\n      color:inherit; }\n      .bp3-dark .bp3-button.bp3-minimal:hover, .bp3-dark .bp3-button.bp3-minimal:active, .bp3-dark .bp3-button.bp3-minimal.bp3-active{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background:none; }\n      .bp3-dark .bp3-button.bp3-minimal:hover{\n        background:rgba(138, 155, 168, 0.15); }\n      .bp3-dark .bp3-button.bp3-minimal:active, .bp3-dark .bp3-button.bp3-minimal.bp3-active{\n        background:rgba(138, 155, 168, 0.3);\n        color:#f5f8fa; }\n      .bp3-dark .bp3-button.bp3-minimal:disabled, .bp3-dark .bp3-button.bp3-minimal:disabled:hover, .bp3-dark .bp3-button.bp3-minimal.bp3-disabled, .bp3-dark .bp3-button.bp3-minimal.bp3-disabled:hover{\n        background:none;\n        cursor:not-allowed;\n        color:rgba(167, 182, 194, 0.6); }\n        .bp3-dark .bp3-button.bp3-minimal:disabled.bp3-active, .bp3-dark .bp3-button.bp3-minimal:disabled:hover.bp3-active, .bp3-dark .bp3-button.bp3-minimal.bp3-disabled.bp3-active, .bp3-dark .bp3-button.bp3-minimal.bp3-disabled:hover.bp3-active{\n          background:rgba(138, 155, 168, 0.3); }\n    .bp3-button.bp3-minimal.bp3-intent-primary{\n      color:#106ba3; }\n      .bp3-button.bp3-minimal.bp3-intent-primary:hover, .bp3-button.bp3-minimal.bp3-intent-primary:active, .bp3-button.bp3-minimal.bp3-intent-primary.bp3-active{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background:none;\n        color:#106ba3; }\n      .bp3-button.bp3-minimal.bp3-intent-primary:hover{\n        background:rgba(19, 124, 189, 0.15);\n        color:#106ba3; }\n      .bp3-button.bp3-minimal.bp3-intent-primary:active, .bp3-button.bp3-minimal.bp3-intent-primary.bp3-active{\n        background:rgba(19, 124, 189, 0.3);\n        color:#106ba3; }\n      .bp3-button.bp3-minimal.bp3-intent-primary:disabled, .bp3-button.bp3-minimal.bp3-intent-primary.bp3-disabled{\n        background:none;\n        color:rgba(16, 107, 163, 0.5); }\n        .bp3-button.bp3-minimal.bp3-intent-primary:disabled.bp3-active, .bp3-button.bp3-minimal.bp3-intent-primary.bp3-disabled.bp3-active{\n          background:rgba(19, 124, 189, 0.3); }\n      .bp3-button.bp3-minimal.bp3-intent-primary .bp3-button-spinner .bp3-spinner-head{\n        stroke:#106ba3; }\n      .bp3-dark .bp3-button.bp3-minimal.bp3-intent-primary{\n        color:#48aff0; }\n        .bp3-dark .bp3-button.bp3-minimal.bp3-intent-primary:hover{\n          background:rgba(19, 124, 189, 0.2);\n          color:#48aff0; }\n        .bp3-dark .bp3-button.bp3-minimal.bp3-intent-primary:active, .bp3-dark .bp3-button.bp3-minimal.bp3-intent-primary.bp3-active{\n          background:rgba(19, 124, 189, 0.3);\n          color:#48aff0; }\n        .bp3-dark .bp3-button.bp3-minimal.bp3-intent-primary:disabled, .bp3-dark .bp3-button.bp3-minimal.bp3-intent-primary.bp3-disabled{\n          background:none;\n          color:rgba(72, 175, 240, 0.5); }\n          .bp3-dark .bp3-button.bp3-minimal.bp3-intent-primary:disabled.bp3-active, .bp3-dark .bp3-button.bp3-minimal.bp3-intent-primary.bp3-disabled.bp3-active{\n            background:rgba(19, 124, 189, 0.3); }\n    .bp3-button.bp3-minimal.bp3-intent-success{\n      color:#0d8050; }\n      .bp3-button.bp3-minimal.bp3-intent-success:hover, .bp3-button.bp3-minimal.bp3-intent-success:active, .bp3-button.bp3-minimal.bp3-intent-success.bp3-active{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background:none;\n        color:#0d8050; }\n      .bp3-button.bp3-minimal.bp3-intent-success:hover{\n        background:rgba(15, 153, 96, 0.15);\n        color:#0d8050; }\n      .bp3-button.bp3-minimal.bp3-intent-success:active, .bp3-button.bp3-minimal.bp3-intent-success.bp3-active{\n        background:rgba(15, 153, 96, 0.3);\n        color:#0d8050; }\n      .bp3-button.bp3-minimal.bp3-intent-success:disabled, .bp3-button.bp3-minimal.bp3-intent-success.bp3-disabled{\n        background:none;\n        color:rgba(13, 128, 80, 0.5); }\n        .bp3-button.bp3-minimal.bp3-intent-success:disabled.bp3-active, .bp3-button.bp3-minimal.bp3-intent-success.bp3-disabled.bp3-active{\n          background:rgba(15, 153, 96, 0.3); }\n      .bp3-button.bp3-minimal.bp3-intent-success .bp3-button-spinner .bp3-spinner-head{\n        stroke:#0d8050; }\n      .bp3-dark .bp3-button.bp3-minimal.bp3-intent-success{\n        color:#3dcc91; }\n        .bp3-dark .bp3-button.bp3-minimal.bp3-intent-success:hover{\n          background:rgba(15, 153, 96, 0.2);\n          color:#3dcc91; }\n        .bp3-dark .bp3-button.bp3-minimal.bp3-intent-success:active, .bp3-dark .bp3-button.bp3-minimal.bp3-intent-success.bp3-active{\n          background:rgba(15, 153, 96, 0.3);\n          color:#3dcc91; }\n        .bp3-dark .bp3-button.bp3-minimal.bp3-intent-success:disabled, .bp3-dark .bp3-button.bp3-minimal.bp3-intent-success.bp3-disabled{\n          background:none;\n          color:rgba(61, 204, 145, 0.5); }\n          .bp3-dark .bp3-button.bp3-minimal.bp3-intent-success:disabled.bp3-active, .bp3-dark .bp3-button.bp3-minimal.bp3-intent-success.bp3-disabled.bp3-active{\n            background:rgba(15, 153, 96, 0.3); }\n    .bp3-button.bp3-minimal.bp3-intent-warning{\n      color:#bf7326; }\n      .bp3-button.bp3-minimal.bp3-intent-warning:hover, .bp3-button.bp3-minimal.bp3-intent-warning:active, .bp3-button.bp3-minimal.bp3-intent-warning.bp3-active{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background:none;\n        color:#bf7326; }\n      .bp3-button.bp3-minimal.bp3-intent-warning:hover{\n        background:rgba(217, 130, 43, 0.15);\n        color:#bf7326; }\n      .bp3-button.bp3-minimal.bp3-intent-warning:active, .bp3-button.bp3-minimal.bp3-intent-warning.bp3-active{\n        background:rgba(217, 130, 43, 0.3);\n        color:#bf7326; }\n      .bp3-button.bp3-minimal.bp3-intent-warning:disabled, .bp3-button.bp3-minimal.bp3-intent-warning.bp3-disabled{\n        background:none;\n        color:rgba(191, 115, 38, 0.5); }\n        .bp3-button.bp3-minimal.bp3-intent-warning:disabled.bp3-active, .bp3-button.bp3-minimal.bp3-intent-warning.bp3-disabled.bp3-active{\n          background:rgba(217, 130, 43, 0.3); }\n      .bp3-button.bp3-minimal.bp3-intent-warning .bp3-button-spinner .bp3-spinner-head{\n        stroke:#bf7326; }\n      .bp3-dark .bp3-button.bp3-minimal.bp3-intent-warning{\n        color:#ffb366; }\n        .bp3-dark .bp3-button.bp3-minimal.bp3-intent-warning:hover{\n          background:rgba(217, 130, 43, 0.2);\n          color:#ffb366; }\n        .bp3-dark .bp3-button.bp3-minimal.bp3-intent-warning:active, .bp3-dark .bp3-button.bp3-minimal.bp3-intent-warning.bp3-active{\n          background:rgba(217, 130, 43, 0.3);\n          color:#ffb366; }\n        .bp3-dark .bp3-button.bp3-minimal.bp3-intent-warning:disabled, .bp3-dark .bp3-button.bp3-minimal.bp3-intent-warning.bp3-disabled{\n          background:none;\n          color:rgba(255, 179, 102, 0.5); }\n          .bp3-dark .bp3-button.bp3-minimal.bp3-intent-warning:disabled.bp3-active, .bp3-dark .bp3-button.bp3-minimal.bp3-intent-warning.bp3-disabled.bp3-active{\n            background:rgba(217, 130, 43, 0.3); }\n    .bp3-button.bp3-minimal.bp3-intent-danger{\n      color:#c23030; }\n      .bp3-button.bp3-minimal.bp3-intent-danger:hover, .bp3-button.bp3-minimal.bp3-intent-danger:active, .bp3-button.bp3-minimal.bp3-intent-danger.bp3-active{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background:none;\n        color:#c23030; }\n      .bp3-button.bp3-minimal.bp3-intent-danger:hover{\n        background:rgba(219, 55, 55, 0.15);\n        color:#c23030; }\n      .bp3-button.bp3-minimal.bp3-intent-danger:active, .bp3-button.bp3-minimal.bp3-intent-danger.bp3-active{\n        background:rgba(219, 55, 55, 0.3);\n        color:#c23030; }\n      .bp3-button.bp3-minimal.bp3-intent-danger:disabled, .bp3-button.bp3-minimal.bp3-intent-danger.bp3-disabled{\n        background:none;\n        color:rgba(194, 48, 48, 0.5); }\n        .bp3-button.bp3-minimal.bp3-intent-danger:disabled.bp3-active, .bp3-button.bp3-minimal.bp3-intent-danger.bp3-disabled.bp3-active{\n          background:rgba(219, 55, 55, 0.3); }\n      .bp3-button.bp3-minimal.bp3-intent-danger .bp3-button-spinner .bp3-spinner-head{\n        stroke:#c23030; }\n      .bp3-dark .bp3-button.bp3-minimal.bp3-intent-danger{\n        color:#ff7373; }\n        .bp3-dark .bp3-button.bp3-minimal.bp3-intent-danger:hover{\n          background:rgba(219, 55, 55, 0.2);\n          color:#ff7373; }\n        .bp3-dark .bp3-button.bp3-minimal.bp3-intent-danger:active, .bp3-dark .bp3-button.bp3-minimal.bp3-intent-danger.bp3-active{\n          background:rgba(219, 55, 55, 0.3);\n          color:#ff7373; }\n        .bp3-dark .bp3-button.bp3-minimal.bp3-intent-danger:disabled, .bp3-dark .bp3-button.bp3-minimal.bp3-intent-danger.bp3-disabled{\n          background:none;\n          color:rgba(255, 115, 115, 0.5); }\n          .bp3-dark .bp3-button.bp3-minimal.bp3-intent-danger:disabled.bp3-active, .bp3-dark .bp3-button.bp3-minimal.bp3-intent-danger.bp3-disabled.bp3-active{\n            background:rgba(219, 55, 55, 0.3); }\n\na.bp3-button{\n  text-align:center;\n  text-decoration:none;\n  -webkit-transition:none;\n  transition:none; }\n  a.bp3-button, a.bp3-button:hover, a.bp3-button:active{\n    color:#182026; }\n  a.bp3-button.bp3-disabled{\n    color:rgba(92, 112, 128, 0.6); }\n\n.bp3-button-text{\n  -webkit-box-flex:0;\n  -webkit-flex:0 1 auto;\n      -ms-flex:0 1 auto;\n          flex:0 1 auto; }\n\n.bp3-button.bp3-align-left .bp3-button-text, .bp3-button.bp3-align-right .bp3-button-text,\n.bp3-button-group.bp3-align-left .bp3-button-text,\n.bp3-button-group.bp3-align-right .bp3-button-text{\n  -webkit-box-flex:1;\n  -webkit-flex:1 1 auto;\n      -ms-flex:1 1 auto;\n          flex:1 1 auto; }\n.bp3-button-group{\n  display:-webkit-inline-box;\n  display:-webkit-inline-flex;\n  display:-ms-inline-flexbox;\n  display:inline-flex; }\n  .bp3-button-group .bp3-button{\n    -webkit-box-flex:0;\n    -webkit-flex:0 0 auto;\n        -ms-flex:0 0 auto;\n            flex:0 0 auto;\n    position:relative;\n    z-index:4; }\n    .bp3-button-group .bp3-button:focus{\n      z-index:5; }\n    .bp3-button-group .bp3-button:hover{\n      z-index:6; }\n    .bp3-button-group .bp3-button:active, .bp3-button-group .bp3-button.bp3-active{\n      z-index:7; }\n    .bp3-button-group .bp3-button:disabled, .bp3-button-group .bp3-button.bp3-disabled{\n      z-index:3; }\n    .bp3-button-group .bp3-button[class*="bp3-intent-"]{\n      z-index:9; }\n      .bp3-button-group .bp3-button[class*="bp3-intent-"]:focus{\n        z-index:10; }\n      .bp3-button-group .bp3-button[class*="bp3-intent-"]:hover{\n        z-index:11; }\n      .bp3-button-group .bp3-button[class*="bp3-intent-"]:active, .bp3-button-group .bp3-button[class*="bp3-intent-"].bp3-active{\n        z-index:12; }\n      .bp3-button-group .bp3-button[class*="bp3-intent-"]:disabled, .bp3-button-group .bp3-button[class*="bp3-intent-"].bp3-disabled{\n        z-index:8; }\n  .bp3-button-group:not(.bp3-minimal) > .bp3-popover-wrapper:not(:first-child) .bp3-button,\n  .bp3-button-group:not(.bp3-minimal) > .bp3-button:not(:first-child){\n    border-top-left-radius:0;\n    border-bottom-left-radius:0; }\n  .bp3-button-group:not(.bp3-minimal) > .bp3-popover-wrapper:not(:last-child) .bp3-button,\n  .bp3-button-group:not(.bp3-minimal) > .bp3-button:not(:last-child){\n    margin-right:-1px;\n    border-top-right-radius:0;\n    border-bottom-right-radius:0; }\n  .bp3-button-group.bp3-minimal .bp3-button{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:none; }\n    .bp3-button-group.bp3-minimal .bp3-button:hover{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:rgba(167, 182, 194, 0.3);\n      text-decoration:none;\n      color:#182026; }\n    .bp3-button-group.bp3-minimal .bp3-button:active, .bp3-button-group.bp3-minimal .bp3-button.bp3-active{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:rgba(115, 134, 148, 0.3);\n      color:#182026; }\n    .bp3-button-group.bp3-minimal .bp3-button:disabled, .bp3-button-group.bp3-minimal .bp3-button:disabled:hover, .bp3-button-group.bp3-minimal .bp3-button.bp3-disabled, .bp3-button-group.bp3-minimal .bp3-button.bp3-disabled:hover{\n      background:none;\n      cursor:not-allowed;\n      color:rgba(92, 112, 128, 0.6); }\n      .bp3-button-group.bp3-minimal .bp3-button:disabled.bp3-active, .bp3-button-group.bp3-minimal .bp3-button:disabled:hover.bp3-active, .bp3-button-group.bp3-minimal .bp3-button.bp3-disabled.bp3-active, .bp3-button-group.bp3-minimal .bp3-button.bp3-disabled:hover.bp3-active{\n        background:rgba(115, 134, 148, 0.3); }\n    .bp3-dark .bp3-button-group.bp3-minimal .bp3-button{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:none;\n      color:inherit; }\n      .bp3-dark .bp3-button-group.bp3-minimal .bp3-button:hover, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button:active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-active{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background:none; }\n      .bp3-dark .bp3-button-group.bp3-minimal .bp3-button:hover{\n        background:rgba(138, 155, 168, 0.15); }\n      .bp3-dark .bp3-button-group.bp3-minimal .bp3-button:active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-active{\n        background:rgba(138, 155, 168, 0.3);\n        color:#f5f8fa; }\n      .bp3-dark .bp3-button-group.bp3-minimal .bp3-button:disabled, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button:disabled:hover, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-disabled, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-disabled:hover{\n        background:none;\n        cursor:not-allowed;\n        color:rgba(167, 182, 194, 0.6); }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button:disabled.bp3-active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button:disabled:hover.bp3-active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-disabled.bp3-active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-disabled:hover.bp3-active{\n          background:rgba(138, 155, 168, 0.3); }\n    .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary{\n      color:#106ba3; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary:hover, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary:active, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary.bp3-active{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background:none;\n        color:#106ba3; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary:hover{\n        background:rgba(19, 124, 189, 0.15);\n        color:#106ba3; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary:active, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary.bp3-active{\n        background:rgba(19, 124, 189, 0.3);\n        color:#106ba3; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary:disabled, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary.bp3-disabled{\n        background:none;\n        color:rgba(16, 107, 163, 0.5); }\n        .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary:disabled.bp3-active, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary.bp3-disabled.bp3-active{\n          background:rgba(19, 124, 189, 0.3); }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary .bp3-button-spinner .bp3-spinner-head{\n        stroke:#106ba3; }\n      .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary{\n        color:#48aff0; }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary:hover{\n          background:rgba(19, 124, 189, 0.2);\n          color:#48aff0; }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary:active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary.bp3-active{\n          background:rgba(19, 124, 189, 0.3);\n          color:#48aff0; }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary:disabled, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary.bp3-disabled{\n          background:none;\n          color:rgba(72, 175, 240, 0.5); }\n          .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary:disabled.bp3-active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-primary.bp3-disabled.bp3-active{\n            background:rgba(19, 124, 189, 0.3); }\n    .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success{\n      color:#0d8050; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success:hover, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success:active, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success.bp3-active{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background:none;\n        color:#0d8050; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success:hover{\n        background:rgba(15, 153, 96, 0.15);\n        color:#0d8050; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success:active, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success.bp3-active{\n        background:rgba(15, 153, 96, 0.3);\n        color:#0d8050; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success:disabled, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success.bp3-disabled{\n        background:none;\n        color:rgba(13, 128, 80, 0.5); }\n        .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success:disabled.bp3-active, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success.bp3-disabled.bp3-active{\n          background:rgba(15, 153, 96, 0.3); }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success .bp3-button-spinner .bp3-spinner-head{\n        stroke:#0d8050; }\n      .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success{\n        color:#3dcc91; }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success:hover{\n          background:rgba(15, 153, 96, 0.2);\n          color:#3dcc91; }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success:active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success.bp3-active{\n          background:rgba(15, 153, 96, 0.3);\n          color:#3dcc91; }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success:disabled, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success.bp3-disabled{\n          background:none;\n          color:rgba(61, 204, 145, 0.5); }\n          .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success:disabled.bp3-active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-success.bp3-disabled.bp3-active{\n            background:rgba(15, 153, 96, 0.3); }\n    .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning{\n      color:#bf7326; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning:hover, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning:active, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning.bp3-active{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background:none;\n        color:#bf7326; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning:hover{\n        background:rgba(217, 130, 43, 0.15);\n        color:#bf7326; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning:active, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning.bp3-active{\n        background:rgba(217, 130, 43, 0.3);\n        color:#bf7326; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning:disabled, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning.bp3-disabled{\n        background:none;\n        color:rgba(191, 115, 38, 0.5); }\n        .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning:disabled.bp3-active, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning.bp3-disabled.bp3-active{\n          background:rgba(217, 130, 43, 0.3); }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning .bp3-button-spinner .bp3-spinner-head{\n        stroke:#bf7326; }\n      .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning{\n        color:#ffb366; }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning:hover{\n          background:rgba(217, 130, 43, 0.2);\n          color:#ffb366; }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning:active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning.bp3-active{\n          background:rgba(217, 130, 43, 0.3);\n          color:#ffb366; }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning:disabled, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning.bp3-disabled{\n          background:none;\n          color:rgba(255, 179, 102, 0.5); }\n          .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning:disabled.bp3-active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-warning.bp3-disabled.bp3-active{\n            background:rgba(217, 130, 43, 0.3); }\n    .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger{\n      color:#c23030; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger:hover, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger:active, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger.bp3-active{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background:none;\n        color:#c23030; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger:hover{\n        background:rgba(219, 55, 55, 0.15);\n        color:#c23030; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger:active, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger.bp3-active{\n        background:rgba(219, 55, 55, 0.3);\n        color:#c23030; }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger:disabled, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger.bp3-disabled{\n        background:none;\n        color:rgba(194, 48, 48, 0.5); }\n        .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger:disabled.bp3-active, .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger.bp3-disabled.bp3-active{\n          background:rgba(219, 55, 55, 0.3); }\n      .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger .bp3-button-spinner .bp3-spinner-head{\n        stroke:#c23030; }\n      .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger{\n        color:#ff7373; }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger:hover{\n          background:rgba(219, 55, 55, 0.2);\n          color:#ff7373; }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger:active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger.bp3-active{\n          background:rgba(219, 55, 55, 0.3);\n          color:#ff7373; }\n        .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger:disabled, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger.bp3-disabled{\n          background:none;\n          color:rgba(255, 115, 115, 0.5); }\n          .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger:disabled.bp3-active, .bp3-dark .bp3-button-group.bp3-minimal .bp3-button.bp3-intent-danger.bp3-disabled.bp3-active{\n            background:rgba(219, 55, 55, 0.3); }\n  .bp3-button-group .bp3-popover-wrapper,\n  .bp3-button-group .bp3-popover-target{\n    display:-webkit-box;\n    display:-webkit-flex;\n    display:-ms-flexbox;\n    display:flex;\n    -webkit-box-flex:1;\n    -webkit-flex:1 1 auto;\n        -ms-flex:1 1 auto;\n            flex:1 1 auto; }\n  .bp3-button-group.bp3-fill{\n    display:-webkit-box;\n    display:-webkit-flex;\n    display:-ms-flexbox;\n    display:flex;\n    width:100%; }\n  .bp3-button-group .bp3-button.bp3-fill,\n  .bp3-button-group.bp3-fill .bp3-button:not(.bp3-fixed){\n    -webkit-box-flex:1;\n    -webkit-flex:1 1 auto;\n        -ms-flex:1 1 auto;\n            flex:1 1 auto; }\n  .bp3-button-group.bp3-vertical{\n    -webkit-box-orient:vertical;\n    -webkit-box-direction:normal;\n    -webkit-flex-direction:column;\n        -ms-flex-direction:column;\n            flex-direction:column;\n    -webkit-box-align:stretch;\n    -webkit-align-items:stretch;\n        -ms-flex-align:stretch;\n            align-items:stretch;\n    vertical-align:top; }\n    .bp3-button-group.bp3-vertical.bp3-fill{\n      width:unset;\n      height:100%; }\n    .bp3-button-group.bp3-vertical .bp3-button{\n      margin-right:0 !important;\n      width:100%; }\n    .bp3-button-group.bp3-vertical:not(.bp3-minimal) > .bp3-popover-wrapper:first-child .bp3-button,\n    .bp3-button-group.bp3-vertical:not(.bp3-minimal) > .bp3-button:first-child{\n      border-radius:3px 3px 0 0; }\n    .bp3-button-group.bp3-vertical:not(.bp3-minimal) > .bp3-popover-wrapper:last-child .bp3-button,\n    .bp3-button-group.bp3-vertical:not(.bp3-minimal) > .bp3-button:last-child{\n      border-radius:0 0 3px 3px; }\n    .bp3-button-group.bp3-vertical:not(.bp3-minimal) > .bp3-popover-wrapper:not(:last-child) .bp3-button,\n    .bp3-button-group.bp3-vertical:not(.bp3-minimal) > .bp3-button:not(:last-child){\n      margin-bottom:-1px; }\n  .bp3-button-group.bp3-align-left .bp3-button{\n    text-align:left; }\n  .bp3-dark .bp3-button-group:not(.bp3-minimal) > .bp3-popover-wrapper:not(:last-child) .bp3-button,\n  .bp3-dark .bp3-button-group:not(.bp3-minimal) > .bp3-button:not(:last-child){\n    margin-right:1px; }\n  .bp3-dark .bp3-button-group.bp3-vertical > .bp3-popover-wrapper:not(:last-child) .bp3-button,\n  .bp3-dark .bp3-button-group.bp3-vertical > .bp3-button:not(:last-child){\n    margin-bottom:1px; }\n.bp3-callout{\n  line-height:1.5;\n  font-size:14px;\n  position:relative;\n  border-radius:3px;\n  background-color:rgba(138, 155, 168, 0.15);\n  width:100%;\n  padding:10px 12px 9px; }\n  .bp3-callout[class*="bp3-icon-"]{\n    padding-left:40px; }\n    .bp3-callout[class*="bp3-icon-"]::before{\n      line-height:1;\n      font-family:"Icons20", sans-serif;\n      font-size:20px;\n      font-weight:400;\n      font-style:normal;\n      -moz-osx-font-smoothing:grayscale;\n      -webkit-font-smoothing:antialiased;\n      position:absolute;\n      top:10px;\n      left:10px;\n      color:#5c7080; }\n  .bp3-callout.bp3-callout-icon{\n    padding-left:40px; }\n    .bp3-callout.bp3-callout-icon > .bp3-icon:first-child{\n      position:absolute;\n      top:10px;\n      left:10px;\n      color:#5c7080; }\n  .bp3-callout .bp3-heading{\n    margin-top:0;\n    margin-bottom:5px;\n    line-height:20px; }\n    .bp3-callout .bp3-heading:last-child{\n      margin-bottom:0; }\n  .bp3-dark .bp3-callout{\n    background-color:rgba(138, 155, 168, 0.2); }\n    .bp3-dark .bp3-callout[class*="bp3-icon-"]::before{\n      color:#a7b6c2; }\n  .bp3-callout.bp3-intent-primary{\n    background-color:rgba(19, 124, 189, 0.15); }\n    .bp3-callout.bp3-intent-primary[class*="bp3-icon-"]::before,\n    .bp3-callout.bp3-intent-primary > .bp3-icon:first-child,\n    .bp3-callout.bp3-intent-primary .bp3-heading{\n      color:#106ba3; }\n    .bp3-dark .bp3-callout.bp3-intent-primary{\n      background-color:rgba(19, 124, 189, 0.25); }\n      .bp3-dark .bp3-callout.bp3-intent-primary[class*="bp3-icon-"]::before,\n      .bp3-dark .bp3-callout.bp3-intent-primary > .bp3-icon:first-child,\n      .bp3-dark .bp3-callout.bp3-intent-primary .bp3-heading{\n        color:#48aff0; }\n  .bp3-callout.bp3-intent-success{\n    background-color:rgba(15, 153, 96, 0.15); }\n    .bp3-callout.bp3-intent-success[class*="bp3-icon-"]::before,\n    .bp3-callout.bp3-intent-success > .bp3-icon:first-child,\n    .bp3-callout.bp3-intent-success .bp3-heading{\n      color:#0d8050; }\n    .bp3-dark .bp3-callout.bp3-intent-success{\n      background-color:rgba(15, 153, 96, 0.25); }\n      .bp3-dark .bp3-callout.bp3-intent-success[class*="bp3-icon-"]::before,\n      .bp3-dark .bp3-callout.bp3-intent-success > .bp3-icon:first-child,\n      .bp3-dark .bp3-callout.bp3-intent-success .bp3-heading{\n        color:#3dcc91; }\n  .bp3-callout.bp3-intent-warning{\n    background-color:rgba(217, 130, 43, 0.15); }\n    .bp3-callout.bp3-intent-warning[class*="bp3-icon-"]::before,\n    .bp3-callout.bp3-intent-warning > .bp3-icon:first-child,\n    .bp3-callout.bp3-intent-warning .bp3-heading{\n      color:#bf7326; }\n    .bp3-dark .bp3-callout.bp3-intent-warning{\n      background-color:rgba(217, 130, 43, 0.25); }\n      .bp3-dark .bp3-callout.bp3-intent-warning[class*="bp3-icon-"]::before,\n      .bp3-dark .bp3-callout.bp3-intent-warning > .bp3-icon:first-child,\n      .bp3-dark .bp3-callout.bp3-intent-warning .bp3-heading{\n        color:#ffb366; }\n  .bp3-callout.bp3-intent-danger{\n    background-color:rgba(219, 55, 55, 0.15); }\n    .bp3-callout.bp3-intent-danger[class*="bp3-icon-"]::before,\n    .bp3-callout.bp3-intent-danger > .bp3-icon:first-child,\n    .bp3-callout.bp3-intent-danger .bp3-heading{\n      color:#c23030; }\n    .bp3-dark .bp3-callout.bp3-intent-danger{\n      background-color:rgba(219, 55, 55, 0.25); }\n      .bp3-dark .bp3-callout.bp3-intent-danger[class*="bp3-icon-"]::before,\n      .bp3-dark .bp3-callout.bp3-intent-danger > .bp3-icon:first-child,\n      .bp3-dark .bp3-callout.bp3-intent-danger .bp3-heading{\n        color:#ff7373; }\n  .bp3-running-text .bp3-callout{\n    margin:20px 0; }\n.bp3-card{\n  border-radius:3px;\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.15), 0 0 0 rgba(16, 22, 26, 0), 0 0 0 rgba(16, 22, 26, 0);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.15), 0 0 0 rgba(16, 22, 26, 0), 0 0 0 rgba(16, 22, 26, 0);\n  background-color:#ffffff;\n  padding:20px;\n  -webkit-transition:-webkit-transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9), -webkit-box-shadow 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:-webkit-transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9), -webkit-box-shadow 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9), box-shadow 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9), box-shadow 200ms cubic-bezier(0.4, 1, 0.75, 0.9), -webkit-transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9), -webkit-box-shadow 200ms cubic-bezier(0.4, 1, 0.75, 0.9); }\n  .bp3-card.bp3-dark,\n  .bp3-dark .bp3-card{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4), 0 0 0 rgba(16, 22, 26, 0), 0 0 0 rgba(16, 22, 26, 0);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4), 0 0 0 rgba(16, 22, 26, 0), 0 0 0 rgba(16, 22, 26, 0);\n    background-color:#30404d; }\n\n.bp3-elevation-0{\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.15), 0 0 0 rgba(16, 22, 26, 0), 0 0 0 rgba(16, 22, 26, 0);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.15), 0 0 0 rgba(16, 22, 26, 0), 0 0 0 rgba(16, 22, 26, 0); }\n  .bp3-elevation-0.bp3-dark,\n  .bp3-dark .bp3-elevation-0{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4), 0 0 0 rgba(16, 22, 26, 0), 0 0 0 rgba(16, 22, 26, 0);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4), 0 0 0 rgba(16, 22, 26, 0), 0 0 0 rgba(16, 22, 26, 0); }\n\n.bp3-elevation-1{\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.2); }\n  .bp3-elevation-1.bp3-dark,\n  .bp3-dark .bp3-elevation-1{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.4); }\n\n.bp3-elevation-2{\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 1px 1px rgba(16, 22, 26, 0.2), 0 2px 6px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 1px 1px rgba(16, 22, 26, 0.2), 0 2px 6px rgba(16, 22, 26, 0.2); }\n  .bp3-elevation-2.bp3-dark,\n  .bp3-dark .bp3-elevation-2{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 1px 1px rgba(16, 22, 26, 0.4), 0 2px 6px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 1px 1px rgba(16, 22, 26, 0.4), 0 2px 6px rgba(16, 22, 26, 0.4); }\n\n.bp3-elevation-3{\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2); }\n  .bp3-elevation-3.bp3-dark,\n  .bp3-dark .bp3-elevation-3{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4); }\n\n.bp3-elevation-4{\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 4px 8px rgba(16, 22, 26, 0.2), 0 18px 46px 6px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 4px 8px rgba(16, 22, 26, 0.2), 0 18px 46px 6px rgba(16, 22, 26, 0.2); }\n  .bp3-elevation-4.bp3-dark,\n  .bp3-dark .bp3-elevation-4{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 4px 8px rgba(16, 22, 26, 0.4), 0 18px 46px 6px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 4px 8px rgba(16, 22, 26, 0.4), 0 18px 46px 6px rgba(16, 22, 26, 0.4); }\n\n.bp3-card.bp3-interactive:hover{\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2);\n  cursor:pointer; }\n  .bp3-card.bp3-interactive:hover.bp3-dark,\n  .bp3-dark .bp3-card.bp3-interactive:hover{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4); }\n\n.bp3-card.bp3-interactive:active{\n  opacity:0.9;\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.2);\n  -webkit-transition-duration:0;\n          transition-duration:0; }\n  .bp3-card.bp3-interactive:active.bp3-dark,\n  .bp3-dark .bp3-card.bp3-interactive:active{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.4); }\n\n.bp3-collapse{\n  height:0;\n  overflow-y:hidden;\n  -webkit-transition:height 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:height 200ms cubic-bezier(0.4, 1, 0.75, 0.9); }\n  .bp3-collapse .bp3-collapse-body{\n    -webkit-transition:-webkit-transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n    transition:-webkit-transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n    transition:transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n    transition:transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9), -webkit-transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9); }\n    .bp3-collapse .bp3-collapse-body[aria-hidden="true"]{\n      display:none; }\n\n.bp3-context-menu .bp3-popover-target{\n  display:block; }\n\n.bp3-context-menu-popover-target{\n  position:fixed; }\n\n.bp3-divider{\n  margin:5px;\n  border-right:1px solid rgba(16, 22, 26, 0.15);\n  border-bottom:1px solid rgba(16, 22, 26, 0.15); }\n  .bp3-dark .bp3-divider{\n    border-color:rgba(16, 22, 26, 0.4); }\n.bp3-dialog-container{\n  opacity:1;\n  -webkit-transform:scale(1);\n          transform:scale(1);\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  -webkit-box-pack:center;\n  -webkit-justify-content:center;\n      -ms-flex-pack:center;\n          justify-content:center;\n  width:100%;\n  min-height:100%;\n  pointer-events:none;\n  -webkit-user-select:none;\n     -moz-user-select:none;\n      -ms-user-select:none;\n          user-select:none; }\n  .bp3-dialog-container.bp3-overlay-enter > .bp3-dialog, .bp3-dialog-container.bp3-overlay-appear > .bp3-dialog{\n    opacity:0;\n    -webkit-transform:scale(0.5);\n            transform:scale(0.5); }\n  .bp3-dialog-container.bp3-overlay-enter-active > .bp3-dialog, .bp3-dialog-container.bp3-overlay-appear-active > .bp3-dialog{\n    opacity:1;\n    -webkit-transform:scale(1);\n            transform:scale(1);\n    -webkit-transition-property:opacity, -webkit-transform;\n    transition-property:opacity, -webkit-transform;\n    transition-property:opacity, transform;\n    transition-property:opacity, transform, -webkit-transform;\n    -webkit-transition-duration:300ms;\n            transition-duration:300ms;\n    -webkit-transition-timing-function:cubic-bezier(0.54, 1.12, 0.38, 1.11);\n            transition-timing-function:cubic-bezier(0.54, 1.12, 0.38, 1.11);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n  .bp3-dialog-container.bp3-overlay-exit > .bp3-dialog{\n    opacity:1;\n    -webkit-transform:scale(1);\n            transform:scale(1); }\n  .bp3-dialog-container.bp3-overlay-exit-active > .bp3-dialog{\n    opacity:0;\n    -webkit-transform:scale(0.5);\n            transform:scale(0.5);\n    -webkit-transition-property:opacity, -webkit-transform;\n    transition-property:opacity, -webkit-transform;\n    transition-property:opacity, transform;\n    transition-property:opacity, transform, -webkit-transform;\n    -webkit-transition-duration:300ms;\n            transition-duration:300ms;\n    -webkit-transition-timing-function:cubic-bezier(0.54, 1.12, 0.38, 1.11);\n            transition-timing-function:cubic-bezier(0.54, 1.12, 0.38, 1.11);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n\n.bp3-dialog{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-orient:vertical;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:column;\n      -ms-flex-direction:column;\n          flex-direction:column;\n  margin:30px 0;\n  border-radius:6px;\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 4px 8px rgba(16, 22, 26, 0.2), 0 18px 46px 6px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 4px 8px rgba(16, 22, 26, 0.2), 0 18px 46px 6px rgba(16, 22, 26, 0.2);\n  background:#ebf1f5;\n  width:500px;\n  padding-bottom:20px;\n  pointer-events:all;\n  -webkit-user-select:text;\n     -moz-user-select:text;\n      -ms-user-select:text;\n          user-select:text; }\n  .bp3-dialog:focus{\n    outline:0; }\n  .bp3-dialog.bp3-dark,\n  .bp3-dark .bp3-dialog{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 4px 8px rgba(16, 22, 26, 0.4), 0 18px 46px 6px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 4px 8px rgba(16, 22, 26, 0.4), 0 18px 46px 6px rgba(16, 22, 26, 0.4);\n    background:#293742;\n    color:#f5f8fa; }\n\n.bp3-dialog-header{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-flex:0;\n  -webkit-flex:0 0 auto;\n      -ms-flex:0 0 auto;\n          flex:0 0 auto;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  border-radius:6px 6px 0 0;\n  -webkit-box-shadow:0 1px 0 rgba(16, 22, 26, 0.15);\n          box-shadow:0 1px 0 rgba(16, 22, 26, 0.15);\n  background:#ffffff;\n  min-height:40px;\n  padding-right:5px;\n  padding-left:20px; }\n  .bp3-dialog-header .bp3-icon-large,\n  .bp3-dialog-header .bp3-icon{\n    -webkit-box-flex:0;\n    -webkit-flex:0 0 auto;\n        -ms-flex:0 0 auto;\n            flex:0 0 auto;\n    margin-right:10px;\n    color:#5c7080; }\n  .bp3-dialog-header .bp3-heading{\n    overflow:hidden;\n    text-overflow:ellipsis;\n    white-space:nowrap;\n    word-wrap:normal;\n    -webkit-box-flex:1;\n    -webkit-flex:1 1 auto;\n        -ms-flex:1 1 auto;\n            flex:1 1 auto;\n    margin:0;\n    line-height:inherit; }\n    .bp3-dialog-header .bp3-heading:last-child{\n      margin-right:20px; }\n  .bp3-dark .bp3-dialog-header{\n    -webkit-box-shadow:0 1px 0 rgba(16, 22, 26, 0.4);\n            box-shadow:0 1px 0 rgba(16, 22, 26, 0.4);\n    background:#30404d; }\n    .bp3-dark .bp3-dialog-header .bp3-icon-large,\n    .bp3-dark .bp3-dialog-header .bp3-icon{\n      color:#a7b6c2; }\n\n.bp3-dialog-body{\n  -webkit-box-flex:1;\n  -webkit-flex:1 1 auto;\n      -ms-flex:1 1 auto;\n          flex:1 1 auto;\n  margin:20px;\n  line-height:18px; }\n\n.bp3-dialog-footer{\n  -webkit-box-flex:0;\n  -webkit-flex:0 0 auto;\n      -ms-flex:0 0 auto;\n          flex:0 0 auto;\n  margin:0 20px; }\n\n.bp3-dialog-footer-actions{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-pack:end;\n  -webkit-justify-content:flex-end;\n      -ms-flex-pack:end;\n          justify-content:flex-end; }\n  .bp3-dialog-footer-actions .bp3-button{\n    margin-left:10px; }\n.bp3-drawer{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-orient:vertical;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:column;\n      -ms-flex-direction:column;\n          flex-direction:column;\n  margin:0;\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 4px 8px rgba(16, 22, 26, 0.2), 0 18px 46px 6px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 4px 8px rgba(16, 22, 26, 0.2), 0 18px 46px 6px rgba(16, 22, 26, 0.2);\n  background:#ffffff;\n  padding:0; }\n  .bp3-drawer:focus{\n    outline:0; }\n  .bp3-drawer.bp3-position-top{\n    top:0;\n    right:0;\n    left:0;\n    height:50%; }\n    .bp3-drawer.bp3-position-top.bp3-overlay-enter, .bp3-drawer.bp3-position-top.bp3-overlay-appear{\n      -webkit-transform:translateY(-100%);\n              transform:translateY(-100%); }\n    .bp3-drawer.bp3-position-top.bp3-overlay-enter-active, .bp3-drawer.bp3-position-top.bp3-overlay-appear-active{\n      -webkit-transform:translateY(0);\n              transform:translateY(0);\n      -webkit-transition-property:-webkit-transform;\n      transition-property:-webkit-transform;\n      transition-property:transform;\n      transition-property:transform, -webkit-transform;\n      -webkit-transition-duration:200ms;\n              transition-duration:200ms;\n      -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n              transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n      -webkit-transition-delay:0;\n              transition-delay:0; }\n    .bp3-drawer.bp3-position-top.bp3-overlay-exit{\n      -webkit-transform:translateY(0);\n              transform:translateY(0); }\n    .bp3-drawer.bp3-position-top.bp3-overlay-exit-active{\n      -webkit-transform:translateY(-100%);\n              transform:translateY(-100%);\n      -webkit-transition-property:-webkit-transform;\n      transition-property:-webkit-transform;\n      transition-property:transform;\n      transition-property:transform, -webkit-transform;\n      -webkit-transition-duration:100ms;\n              transition-duration:100ms;\n      -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n              transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n      -webkit-transition-delay:0;\n              transition-delay:0; }\n  .bp3-drawer.bp3-position-bottom{\n    right:0;\n    bottom:0;\n    left:0;\n    height:50%; }\n    .bp3-drawer.bp3-position-bottom.bp3-overlay-enter, .bp3-drawer.bp3-position-bottom.bp3-overlay-appear{\n      -webkit-transform:translateY(100%);\n              transform:translateY(100%); }\n    .bp3-drawer.bp3-position-bottom.bp3-overlay-enter-active, .bp3-drawer.bp3-position-bottom.bp3-overlay-appear-active{\n      -webkit-transform:translateY(0);\n              transform:translateY(0);\n      -webkit-transition-property:-webkit-transform;\n      transition-property:-webkit-transform;\n      transition-property:transform;\n      transition-property:transform, -webkit-transform;\n      -webkit-transition-duration:200ms;\n              transition-duration:200ms;\n      -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n              transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n      -webkit-transition-delay:0;\n              transition-delay:0; }\n    .bp3-drawer.bp3-position-bottom.bp3-overlay-exit{\n      -webkit-transform:translateY(0);\n              transform:translateY(0); }\n    .bp3-drawer.bp3-position-bottom.bp3-overlay-exit-active{\n      -webkit-transform:translateY(100%);\n              transform:translateY(100%);\n      -webkit-transition-property:-webkit-transform;\n      transition-property:-webkit-transform;\n      transition-property:transform;\n      transition-property:transform, -webkit-transform;\n      -webkit-transition-duration:100ms;\n              transition-duration:100ms;\n      -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n              transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n      -webkit-transition-delay:0;\n              transition-delay:0; }\n  .bp3-drawer.bp3-position-left{\n    top:0;\n    bottom:0;\n    left:0;\n    width:50%; }\n    .bp3-drawer.bp3-position-left.bp3-overlay-enter, .bp3-drawer.bp3-position-left.bp3-overlay-appear{\n      -webkit-transform:translateX(-100%);\n              transform:translateX(-100%); }\n    .bp3-drawer.bp3-position-left.bp3-overlay-enter-active, .bp3-drawer.bp3-position-left.bp3-overlay-appear-active{\n      -webkit-transform:translateX(0);\n              transform:translateX(0);\n      -webkit-transition-property:-webkit-transform;\n      transition-property:-webkit-transform;\n      transition-property:transform;\n      transition-property:transform, -webkit-transform;\n      -webkit-transition-duration:200ms;\n              transition-duration:200ms;\n      -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n              transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n      -webkit-transition-delay:0;\n              transition-delay:0; }\n    .bp3-drawer.bp3-position-left.bp3-overlay-exit{\n      -webkit-transform:translateX(0);\n              transform:translateX(0); }\n    .bp3-drawer.bp3-position-left.bp3-overlay-exit-active{\n      -webkit-transform:translateX(-100%);\n              transform:translateX(-100%);\n      -webkit-transition-property:-webkit-transform;\n      transition-property:-webkit-transform;\n      transition-property:transform;\n      transition-property:transform, -webkit-transform;\n      -webkit-transition-duration:100ms;\n              transition-duration:100ms;\n      -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n              transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n      -webkit-transition-delay:0;\n              transition-delay:0; }\n  .bp3-drawer.bp3-position-right{\n    top:0;\n    right:0;\n    bottom:0;\n    width:50%; }\n    .bp3-drawer.bp3-position-right.bp3-overlay-enter, .bp3-drawer.bp3-position-right.bp3-overlay-appear{\n      -webkit-transform:translateX(100%);\n              transform:translateX(100%); }\n    .bp3-drawer.bp3-position-right.bp3-overlay-enter-active, .bp3-drawer.bp3-position-right.bp3-overlay-appear-active{\n      -webkit-transform:translateX(0);\n              transform:translateX(0);\n      -webkit-transition-property:-webkit-transform;\n      transition-property:-webkit-transform;\n      transition-property:transform;\n      transition-property:transform, -webkit-transform;\n      -webkit-transition-duration:200ms;\n              transition-duration:200ms;\n      -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n              transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n      -webkit-transition-delay:0;\n              transition-delay:0; }\n    .bp3-drawer.bp3-position-right.bp3-overlay-exit{\n      -webkit-transform:translateX(0);\n              transform:translateX(0); }\n    .bp3-drawer.bp3-position-right.bp3-overlay-exit-active{\n      -webkit-transform:translateX(100%);\n              transform:translateX(100%);\n      -webkit-transition-property:-webkit-transform;\n      transition-property:-webkit-transform;\n      transition-property:transform;\n      transition-property:transform, -webkit-transform;\n      -webkit-transition-duration:100ms;\n              transition-duration:100ms;\n      -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n              transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n      -webkit-transition-delay:0;\n              transition-delay:0; }\n  .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n  .bp3-position-right):not(.bp3-vertical){\n    top:0;\n    right:0;\n    bottom:0;\n    width:50%; }\n    .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n    .bp3-position-right):not(.bp3-vertical).bp3-overlay-enter, .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n    .bp3-position-right):not(.bp3-vertical).bp3-overlay-appear{\n      -webkit-transform:translateX(100%);\n              transform:translateX(100%); }\n    .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n    .bp3-position-right):not(.bp3-vertical).bp3-overlay-enter-active, .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n    .bp3-position-right):not(.bp3-vertical).bp3-overlay-appear-active{\n      -webkit-transform:translateX(0);\n              transform:translateX(0);\n      -webkit-transition-property:-webkit-transform;\n      transition-property:-webkit-transform;\n      transition-property:transform;\n      transition-property:transform, -webkit-transform;\n      -webkit-transition-duration:200ms;\n              transition-duration:200ms;\n      -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n              transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n      -webkit-transition-delay:0;\n              transition-delay:0; }\n    .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n    .bp3-position-right):not(.bp3-vertical).bp3-overlay-exit{\n      -webkit-transform:translateX(0);\n              transform:translateX(0); }\n    .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n    .bp3-position-right):not(.bp3-vertical).bp3-overlay-exit-active{\n      -webkit-transform:translateX(100%);\n              transform:translateX(100%);\n      -webkit-transition-property:-webkit-transform;\n      transition-property:-webkit-transform;\n      transition-property:transform;\n      transition-property:transform, -webkit-transform;\n      -webkit-transition-duration:100ms;\n              transition-duration:100ms;\n      -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n              transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n      -webkit-transition-delay:0;\n              transition-delay:0; }\n  .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n  .bp3-position-right).bp3-vertical{\n    right:0;\n    bottom:0;\n    left:0;\n    height:50%; }\n    .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n    .bp3-position-right).bp3-vertical.bp3-overlay-enter, .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n    .bp3-position-right).bp3-vertical.bp3-overlay-appear{\n      -webkit-transform:translateY(100%);\n              transform:translateY(100%); }\n    .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n    .bp3-position-right).bp3-vertical.bp3-overlay-enter-active, .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n    .bp3-position-right).bp3-vertical.bp3-overlay-appear-active{\n      -webkit-transform:translateY(0);\n              transform:translateY(0);\n      -webkit-transition-property:-webkit-transform;\n      transition-property:-webkit-transform;\n      transition-property:transform;\n      transition-property:transform, -webkit-transform;\n      -webkit-transition-duration:200ms;\n              transition-duration:200ms;\n      -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n              transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n      -webkit-transition-delay:0;\n              transition-delay:0; }\n    .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n    .bp3-position-right).bp3-vertical.bp3-overlay-exit{\n      -webkit-transform:translateY(0);\n              transform:translateY(0); }\n    .bp3-drawer:not(.bp3-position-top):not(.bp3-position-bottom):not(.bp3-position-left):not(\n    .bp3-position-right).bp3-vertical.bp3-overlay-exit-active{\n      -webkit-transform:translateY(100%);\n              transform:translateY(100%);\n      -webkit-transition-property:-webkit-transform;\n      transition-property:-webkit-transform;\n      transition-property:transform;\n      transition-property:transform, -webkit-transform;\n      -webkit-transition-duration:100ms;\n              transition-duration:100ms;\n      -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n              transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n      -webkit-transition-delay:0;\n              transition-delay:0; }\n  .bp3-drawer.bp3-dark,\n  .bp3-dark .bp3-drawer{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 4px 8px rgba(16, 22, 26, 0.4), 0 18px 46px 6px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 4px 8px rgba(16, 22, 26, 0.4), 0 18px 46px 6px rgba(16, 22, 26, 0.4);\n    background:#30404d;\n    color:#f5f8fa; }\n\n.bp3-drawer-header{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-flex:0;\n  -webkit-flex:0 0 auto;\n      -ms-flex:0 0 auto;\n          flex:0 0 auto;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  position:relative;\n  border-radius:0;\n  -webkit-box-shadow:0 1px 0 rgba(16, 22, 26, 0.15);\n          box-shadow:0 1px 0 rgba(16, 22, 26, 0.15);\n  min-height:40px;\n  padding:5px;\n  padding-left:20px; }\n  .bp3-drawer-header .bp3-icon-large,\n  .bp3-drawer-header .bp3-icon{\n    -webkit-box-flex:0;\n    -webkit-flex:0 0 auto;\n        -ms-flex:0 0 auto;\n            flex:0 0 auto;\n    margin-right:10px;\n    color:#5c7080; }\n  .bp3-drawer-header .bp3-heading{\n    overflow:hidden;\n    text-overflow:ellipsis;\n    white-space:nowrap;\n    word-wrap:normal;\n    -webkit-box-flex:1;\n    -webkit-flex:1 1 auto;\n        -ms-flex:1 1 auto;\n            flex:1 1 auto;\n    margin:0;\n    line-height:inherit; }\n    .bp3-drawer-header .bp3-heading:last-child{\n      margin-right:20px; }\n  .bp3-dark .bp3-drawer-header{\n    -webkit-box-shadow:0 1px 0 rgba(16, 22, 26, 0.4);\n            box-shadow:0 1px 0 rgba(16, 22, 26, 0.4); }\n    .bp3-dark .bp3-drawer-header .bp3-icon-large,\n    .bp3-dark .bp3-drawer-header .bp3-icon{\n      color:#a7b6c2; }\n\n.bp3-drawer-body{\n  -webkit-box-flex:1;\n  -webkit-flex:1 1 auto;\n      -ms-flex:1 1 auto;\n          flex:1 1 auto;\n  overflow:auto;\n  line-height:18px; }\n\n.bp3-drawer-footer{\n  -webkit-box-flex:0;\n  -webkit-flex:0 0 auto;\n      -ms-flex:0 0 auto;\n          flex:0 0 auto;\n  position:relative;\n  -webkit-box-shadow:inset 0 1px 0 rgba(16, 22, 26, 0.15);\n          box-shadow:inset 0 1px 0 rgba(16, 22, 26, 0.15);\n  padding:10px 20px; }\n  .bp3-dark .bp3-drawer-footer{\n    -webkit-box-shadow:inset 0 1px 0 rgba(16, 22, 26, 0.4);\n            box-shadow:inset 0 1px 0 rgba(16, 22, 26, 0.4); }\n.bp3-editable-text{\n  display:inline-block;\n  position:relative;\n  cursor:text;\n  max-width:100%;\n  vertical-align:top;\n  white-space:nowrap; }\n  .bp3-editable-text::before{\n    position:absolute;\n    top:-3px;\n    right:-3px;\n    bottom:-3px;\n    left:-3px;\n    border-radius:3px;\n    content:"";\n    -webkit-transition:background-color 100ms cubic-bezier(0.4, 1, 0.75, 0.9), -webkit-box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n    transition:background-color 100ms cubic-bezier(0.4, 1, 0.75, 0.9), -webkit-box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n    transition:background-color 100ms cubic-bezier(0.4, 1, 0.75, 0.9), box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n    transition:background-color 100ms cubic-bezier(0.4, 1, 0.75, 0.9), box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9), -webkit-box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9); }\n  .bp3-editable-text:hover::before{\n    -webkit-box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(16, 22, 26, 0.15);\n            box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(16, 22, 26, 0.15); }\n  .bp3-editable-text.bp3-editable-text-editing::before{\n    -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n    background-color:#ffffff; }\n  .bp3-editable-text.bp3-disabled::before{\n    -webkit-box-shadow:none;\n            box-shadow:none; }\n  .bp3-editable-text.bp3-intent-primary .bp3-editable-text-input,\n  .bp3-editable-text.bp3-intent-primary .bp3-editable-text-content{\n    color:#137cbd; }\n  .bp3-editable-text.bp3-intent-primary:hover::before{\n    -webkit-box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(19, 124, 189, 0.4);\n            box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(19, 124, 189, 0.4); }\n  .bp3-editable-text.bp3-intent-primary.bp3-editable-text-editing::before{\n    -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n  .bp3-editable-text.bp3-intent-success .bp3-editable-text-input,\n  .bp3-editable-text.bp3-intent-success .bp3-editable-text-content{\n    color:#0f9960; }\n  .bp3-editable-text.bp3-intent-success:hover::before{\n    -webkit-box-shadow:0 0 0 0 rgba(15, 153, 96, 0), 0 0 0 0 rgba(15, 153, 96, 0), inset 0 0 0 1px rgba(15, 153, 96, 0.4);\n            box-shadow:0 0 0 0 rgba(15, 153, 96, 0), 0 0 0 0 rgba(15, 153, 96, 0), inset 0 0 0 1px rgba(15, 153, 96, 0.4); }\n  .bp3-editable-text.bp3-intent-success.bp3-editable-text-editing::before{\n    -webkit-box-shadow:0 0 0 1px #0f9960, 0 0 0 3px rgba(15, 153, 96, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px #0f9960, 0 0 0 3px rgba(15, 153, 96, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n  .bp3-editable-text.bp3-intent-warning .bp3-editable-text-input,\n  .bp3-editable-text.bp3-intent-warning .bp3-editable-text-content{\n    color:#d9822b; }\n  .bp3-editable-text.bp3-intent-warning:hover::before{\n    -webkit-box-shadow:0 0 0 0 rgba(217, 130, 43, 0), 0 0 0 0 rgba(217, 130, 43, 0), inset 0 0 0 1px rgba(217, 130, 43, 0.4);\n            box-shadow:0 0 0 0 rgba(217, 130, 43, 0), 0 0 0 0 rgba(217, 130, 43, 0), inset 0 0 0 1px rgba(217, 130, 43, 0.4); }\n  .bp3-editable-text.bp3-intent-warning.bp3-editable-text-editing::before{\n    -webkit-box-shadow:0 0 0 1px #d9822b, 0 0 0 3px rgba(217, 130, 43, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px #d9822b, 0 0 0 3px rgba(217, 130, 43, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n  .bp3-editable-text.bp3-intent-danger .bp3-editable-text-input,\n  .bp3-editable-text.bp3-intent-danger .bp3-editable-text-content{\n    color:#db3737; }\n  .bp3-editable-text.bp3-intent-danger:hover::before{\n    -webkit-box-shadow:0 0 0 0 rgba(219, 55, 55, 0), 0 0 0 0 rgba(219, 55, 55, 0), inset 0 0 0 1px rgba(219, 55, 55, 0.4);\n            box-shadow:0 0 0 0 rgba(219, 55, 55, 0), 0 0 0 0 rgba(219, 55, 55, 0), inset 0 0 0 1px rgba(219, 55, 55, 0.4); }\n  .bp3-editable-text.bp3-intent-danger.bp3-editable-text-editing::before{\n    -webkit-box-shadow:0 0 0 1px #db3737, 0 0 0 3px rgba(219, 55, 55, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px #db3737, 0 0 0 3px rgba(219, 55, 55, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n  .bp3-dark .bp3-editable-text:hover::before{\n    -webkit-box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(255, 255, 255, 0.15);\n            box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(255, 255, 255, 0.15); }\n  .bp3-dark .bp3-editable-text.bp3-editable-text-editing::before{\n    -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n    background-color:rgba(16, 22, 26, 0.3); }\n  .bp3-dark .bp3-editable-text.bp3-disabled::before{\n    -webkit-box-shadow:none;\n            box-shadow:none; }\n  .bp3-dark .bp3-editable-text.bp3-intent-primary .bp3-editable-text-content{\n    color:#48aff0; }\n  .bp3-dark .bp3-editable-text.bp3-intent-primary:hover::before{\n    -webkit-box-shadow:0 0 0 0 rgba(72, 175, 240, 0), 0 0 0 0 rgba(72, 175, 240, 0), inset 0 0 0 1px rgba(72, 175, 240, 0.4);\n            box-shadow:0 0 0 0 rgba(72, 175, 240, 0), 0 0 0 0 rgba(72, 175, 240, 0), inset 0 0 0 1px rgba(72, 175, 240, 0.4); }\n  .bp3-dark .bp3-editable-text.bp3-intent-primary.bp3-editable-text-editing::before{\n    -webkit-box-shadow:0 0 0 1px #48aff0, 0 0 0 3px rgba(72, 175, 240, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px #48aff0, 0 0 0 3px rgba(72, 175, 240, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n  .bp3-dark .bp3-editable-text.bp3-intent-success .bp3-editable-text-content{\n    color:#3dcc91; }\n  .bp3-dark .bp3-editable-text.bp3-intent-success:hover::before{\n    -webkit-box-shadow:0 0 0 0 rgba(61, 204, 145, 0), 0 0 0 0 rgba(61, 204, 145, 0), inset 0 0 0 1px rgba(61, 204, 145, 0.4);\n            box-shadow:0 0 0 0 rgba(61, 204, 145, 0), 0 0 0 0 rgba(61, 204, 145, 0), inset 0 0 0 1px rgba(61, 204, 145, 0.4); }\n  .bp3-dark .bp3-editable-text.bp3-intent-success.bp3-editable-text-editing::before{\n    -webkit-box-shadow:0 0 0 1px #3dcc91, 0 0 0 3px rgba(61, 204, 145, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px #3dcc91, 0 0 0 3px rgba(61, 204, 145, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n  .bp3-dark .bp3-editable-text.bp3-intent-warning .bp3-editable-text-content{\n    color:#ffb366; }\n  .bp3-dark .bp3-editable-text.bp3-intent-warning:hover::before{\n    -webkit-box-shadow:0 0 0 0 rgba(255, 179, 102, 0), 0 0 0 0 rgba(255, 179, 102, 0), inset 0 0 0 1px rgba(255, 179, 102, 0.4);\n            box-shadow:0 0 0 0 rgba(255, 179, 102, 0), 0 0 0 0 rgba(255, 179, 102, 0), inset 0 0 0 1px rgba(255, 179, 102, 0.4); }\n  .bp3-dark .bp3-editable-text.bp3-intent-warning.bp3-editable-text-editing::before{\n    -webkit-box-shadow:0 0 0 1px #ffb366, 0 0 0 3px rgba(255, 179, 102, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px #ffb366, 0 0 0 3px rgba(255, 179, 102, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n  .bp3-dark .bp3-editable-text.bp3-intent-danger .bp3-editable-text-content{\n    color:#ff7373; }\n  .bp3-dark .bp3-editable-text.bp3-intent-danger:hover::before{\n    -webkit-box-shadow:0 0 0 0 rgba(255, 115, 115, 0), 0 0 0 0 rgba(255, 115, 115, 0), inset 0 0 0 1px rgba(255, 115, 115, 0.4);\n            box-shadow:0 0 0 0 rgba(255, 115, 115, 0), 0 0 0 0 rgba(255, 115, 115, 0), inset 0 0 0 1px rgba(255, 115, 115, 0.4); }\n  .bp3-dark .bp3-editable-text.bp3-intent-danger.bp3-editable-text-editing::before{\n    -webkit-box-shadow:0 0 0 1px #ff7373, 0 0 0 3px rgba(255, 115, 115, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px #ff7373, 0 0 0 3px rgba(255, 115, 115, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n\n.bp3-editable-text-input,\n.bp3-editable-text-content{\n  display:inherit;\n  position:relative;\n  min-width:inherit;\n  max-width:inherit;\n  vertical-align:top;\n  text-transform:inherit;\n  letter-spacing:inherit;\n  color:inherit;\n  font:inherit;\n  resize:none; }\n\n.bp3-editable-text-input{\n  border:none;\n  -webkit-box-shadow:none;\n          box-shadow:none;\n  background:none;\n  width:100%;\n  padding:0;\n  white-space:pre-wrap; }\n  .bp3-editable-text-input::-webkit-input-placeholder{\n    opacity:1;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-editable-text-input::-ms-input-placeholder{\n    opacity:1;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-editable-text-input::placeholder{\n    opacity:1;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-editable-text-input:focus{\n    outline:none; }\n  .bp3-editable-text-input::-ms-clear{\n    display:none; }\n\n.bp3-editable-text-content{\n  overflow:hidden;\n  padding-right:2px;\n  text-overflow:ellipsis;\n  white-space:pre; }\n  .bp3-editable-text-editing > .bp3-editable-text-content{\n    position:absolute;\n    left:0;\n    visibility:hidden; }\n  .bp3-editable-text-placeholder > .bp3-editable-text-content{\n    color:rgba(92, 112, 128, 0.6); }\n    .bp3-dark .bp3-editable-text-placeholder > .bp3-editable-text-content{\n      color:rgba(167, 182, 194, 0.6); }\n\n.bp3-editable-text.bp3-multiline{\n  display:block; }\n  .bp3-editable-text.bp3-multiline .bp3-editable-text-content{\n    overflow:auto;\n    white-space:pre-wrap;\n    word-wrap:break-word; }\n.bp3-control-group{\n  -webkit-transform:translateZ(0);\n          transform:translateZ(0);\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-orient:horizontal;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:row;\n      -ms-flex-direction:row;\n          flex-direction:row;\n  -webkit-box-align:stretch;\n  -webkit-align-items:stretch;\n      -ms-flex-align:stretch;\n          align-items:stretch; }\n  .bp3-control-group > *{\n    -webkit-box-flex:0;\n    -webkit-flex-grow:0;\n        -ms-flex-positive:0;\n            flex-grow:0;\n    -webkit-flex-shrink:0;\n        -ms-flex-negative:0;\n            flex-shrink:0; }\n  .bp3-control-group > .bp3-fill{\n    -webkit-box-flex:1;\n    -webkit-flex-grow:1;\n        -ms-flex-positive:1;\n            flex-grow:1;\n    -webkit-flex-shrink:1;\n        -ms-flex-negative:1;\n            flex-shrink:1; }\n  .bp3-control-group .bp3-button,\n  .bp3-control-group .bp3-html-select,\n  .bp3-control-group .bp3-input,\n  .bp3-control-group .bp3-select{\n    position:relative; }\n  .bp3-control-group .bp3-input{\n    z-index:2;\n    border-radius:inherit; }\n    .bp3-control-group .bp3-input:focus{\n      z-index:14;\n      border-radius:3px; }\n    .bp3-control-group .bp3-input[class*="bp3-intent"]{\n      z-index:13; }\n      .bp3-control-group .bp3-input[class*="bp3-intent"]:focus{\n        z-index:15; }\n    .bp3-control-group .bp3-input[readonly], .bp3-control-group .bp3-input:disabled, .bp3-control-group .bp3-input.bp3-disabled{\n      z-index:1; }\n  .bp3-control-group .bp3-input-group[class*="bp3-intent"] .bp3-input{\n    z-index:13; }\n    .bp3-control-group .bp3-input-group[class*="bp3-intent"] .bp3-input:focus{\n      z-index:15; }\n  .bp3-control-group .bp3-button,\n  .bp3-control-group .bp3-html-select select,\n  .bp3-control-group .bp3-select select{\n    -webkit-transform:translateZ(0);\n            transform:translateZ(0);\n    z-index:4;\n    border-radius:inherit; }\n    .bp3-control-group .bp3-button:focus,\n    .bp3-control-group .bp3-html-select select:focus,\n    .bp3-control-group .bp3-select select:focus{\n      z-index:5; }\n    .bp3-control-group .bp3-button:hover,\n    .bp3-control-group .bp3-html-select select:hover,\n    .bp3-control-group .bp3-select select:hover{\n      z-index:6; }\n    .bp3-control-group .bp3-button:active,\n    .bp3-control-group .bp3-html-select select:active,\n    .bp3-control-group .bp3-select select:active{\n      z-index:7; }\n    .bp3-control-group .bp3-button[readonly], .bp3-control-group .bp3-button:disabled, .bp3-control-group .bp3-button.bp3-disabled,\n    .bp3-control-group .bp3-html-select select[readonly],\n    .bp3-control-group .bp3-html-select select:disabled,\n    .bp3-control-group .bp3-html-select select.bp3-disabled,\n    .bp3-control-group .bp3-select select[readonly],\n    .bp3-control-group .bp3-select select:disabled,\n    .bp3-control-group .bp3-select select.bp3-disabled{\n      z-index:3; }\n    .bp3-control-group .bp3-button[class*="bp3-intent"],\n    .bp3-control-group .bp3-html-select select[class*="bp3-intent"],\n    .bp3-control-group .bp3-select select[class*="bp3-intent"]{\n      z-index:9; }\n      .bp3-control-group .bp3-button[class*="bp3-intent"]:focus,\n      .bp3-control-group .bp3-html-select select[class*="bp3-intent"]:focus,\n      .bp3-control-group .bp3-select select[class*="bp3-intent"]:focus{\n        z-index:10; }\n      .bp3-control-group .bp3-button[class*="bp3-intent"]:hover,\n      .bp3-control-group .bp3-html-select select[class*="bp3-intent"]:hover,\n      .bp3-control-group .bp3-select select[class*="bp3-intent"]:hover{\n        z-index:11; }\n      .bp3-control-group .bp3-button[class*="bp3-intent"]:active,\n      .bp3-control-group .bp3-html-select select[class*="bp3-intent"]:active,\n      .bp3-control-group .bp3-select select[class*="bp3-intent"]:active{\n        z-index:12; }\n      .bp3-control-group .bp3-button[class*="bp3-intent"][readonly], .bp3-control-group .bp3-button[class*="bp3-intent"]:disabled, .bp3-control-group .bp3-button[class*="bp3-intent"].bp3-disabled,\n      .bp3-control-group .bp3-html-select select[class*="bp3-intent"][readonly],\n      .bp3-control-group .bp3-html-select select[class*="bp3-intent"]:disabled,\n      .bp3-control-group .bp3-html-select select[class*="bp3-intent"].bp3-disabled,\n      .bp3-control-group .bp3-select select[class*="bp3-intent"][readonly],\n      .bp3-control-group .bp3-select select[class*="bp3-intent"]:disabled,\n      .bp3-control-group .bp3-select select[class*="bp3-intent"].bp3-disabled{\n        z-index:8; }\n  .bp3-control-group .bp3-input-group > .bp3-icon,\n  .bp3-control-group .bp3-input-group > .bp3-button,\n  .bp3-control-group .bp3-input-group > .bp3-input-action{\n    z-index:16; }\n  .bp3-control-group .bp3-select::after,\n  .bp3-control-group .bp3-html-select::after,\n  .bp3-control-group .bp3-select > .bp3-icon,\n  .bp3-control-group .bp3-html-select > .bp3-icon{\n    z-index:17; }\n  .bp3-control-group:not(.bp3-vertical) > *{\n    margin-right:-1px; }\n  .bp3-dark .bp3-control-group:not(.bp3-vertical) > *{\n    margin-right:0; }\n  .bp3-dark .bp3-control-group:not(.bp3-vertical) > .bp3-button + .bp3-button{\n    margin-left:1px; }\n  .bp3-control-group .bp3-popover-wrapper,\n  .bp3-control-group .bp3-popover-target{\n    border-radius:inherit; }\n  .bp3-control-group > :first-child{\n    border-radius:3px 0 0 3px; }\n  .bp3-control-group > :last-child{\n    margin-right:0;\n    border-radius:0 3px 3px 0; }\n  .bp3-control-group > :only-child{\n    margin-right:0;\n    border-radius:3px; }\n  .bp3-control-group .bp3-input-group .bp3-button{\n    border-radius:3px; }\n  .bp3-control-group > .bp3-fill{\n    -webkit-box-flex:1;\n    -webkit-flex:1 1 auto;\n        -ms-flex:1 1 auto;\n            flex:1 1 auto; }\n  .bp3-control-group.bp3-fill > *:not(.bp3-fixed){\n    -webkit-box-flex:1;\n    -webkit-flex:1 1 auto;\n        -ms-flex:1 1 auto;\n            flex:1 1 auto; }\n  .bp3-control-group.bp3-vertical{\n    -webkit-box-orient:vertical;\n    -webkit-box-direction:normal;\n    -webkit-flex-direction:column;\n        -ms-flex-direction:column;\n            flex-direction:column; }\n    .bp3-control-group.bp3-vertical > *{\n      margin-top:-1px; }\n    .bp3-control-group.bp3-vertical > :first-child{\n      margin-top:0;\n      border-radius:3px 3px 0 0; }\n    .bp3-control-group.bp3-vertical > :last-child{\n      border-radius:0 0 3px 3px; }\n.bp3-control{\n  display:block;\n  position:relative;\n  margin-bottom:10px;\n  cursor:pointer;\n  text-transform:none; }\n  .bp3-control input:checked ~ .bp3-control-indicator{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n    background-color:#137cbd;\n    background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.1)), to(rgba(255, 255, 255, 0)));\n    background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.1), rgba(255, 255, 255, 0));\n    color:#ffffff; }\n  .bp3-control:hover input:checked ~ .bp3-control-indicator{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n    background-color:#106ba3; }\n  .bp3-control input:not(:disabled):active:checked ~ .bp3-control-indicator{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n    background:#0e5a8a; }\n  .bp3-control input:disabled:checked ~ .bp3-control-indicator{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:rgba(19, 124, 189, 0.5); }\n  .bp3-dark .bp3-control input:checked ~ .bp3-control-indicator{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4); }\n  .bp3-dark .bp3-control:hover input:checked ~ .bp3-control-indicator{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n    background-color:#106ba3; }\n  .bp3-dark .bp3-control input:not(:disabled):active:checked ~ .bp3-control-indicator{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n    background-color:#0e5a8a; }\n  .bp3-dark .bp3-control input:disabled:checked ~ .bp3-control-indicator{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:rgba(14, 90, 138, 0.5); }\n  .bp3-control:not(.bp3-align-right){\n    padding-left:26px; }\n    .bp3-control:not(.bp3-align-right) .bp3-control-indicator{\n      margin-left:-26px; }\n  .bp3-control.bp3-align-right{\n    padding-right:26px; }\n    .bp3-control.bp3-align-right .bp3-control-indicator{\n      margin-right:-26px; }\n  .bp3-control.bp3-disabled{\n    cursor:not-allowed;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-control.bp3-inline{\n    display:inline-block;\n    margin-right:20px; }\n  .bp3-control input{\n    position:absolute;\n    top:0;\n    left:0;\n    opacity:0;\n    z-index:-1; }\n  .bp3-control .bp3-control-indicator{\n    display:inline-block;\n    position:relative;\n    margin-top:-3px;\n    margin-right:10px;\n    border:none;\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n    background-clip:padding-box;\n    background-color:#f5f8fa;\n    background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.8)), to(rgba(255, 255, 255, 0)));\n    background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.8), rgba(255, 255, 255, 0));\n    cursor:pointer;\n    width:1em;\n    height:1em;\n    vertical-align:middle;\n    font-size:16px;\n    -webkit-user-select:none;\n       -moz-user-select:none;\n        -ms-user-select:none;\n            user-select:none; }\n    .bp3-control .bp3-control-indicator::before{\n      display:block;\n      width:1em;\n      height:1em;\n      content:""; }\n  .bp3-control:hover .bp3-control-indicator{\n    background-color:#ebf1f5; }\n  .bp3-control input:not(:disabled):active ~ .bp3-control-indicator{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n    background:#d8e1e8; }\n  .bp3-control input:disabled ~ .bp3-control-indicator{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:rgba(206, 217, 224, 0.5);\n    cursor:not-allowed; }\n  .bp3-control input:focus ~ .bp3-control-indicator{\n    outline:rgba(19, 124, 189, 0.6) auto 2px;\n    outline-offset:2px;\n    -moz-outline-radius:6px; }\n  .bp3-control.bp3-align-right .bp3-control-indicator{\n    float:right;\n    margin-top:1px;\n    margin-left:10px; }\n  .bp3-control.bp3-large{\n    font-size:16px; }\n    .bp3-control.bp3-large:not(.bp3-align-right){\n      padding-left:30px; }\n      .bp3-control.bp3-large:not(.bp3-align-right) .bp3-control-indicator{\n        margin-left:-30px; }\n    .bp3-control.bp3-large.bp3-align-right{\n      padding-right:30px; }\n      .bp3-control.bp3-large.bp3-align-right .bp3-control-indicator{\n        margin-right:-30px; }\n    .bp3-control.bp3-large .bp3-control-indicator{\n      font-size:20px; }\n    .bp3-control.bp3-large.bp3-align-right .bp3-control-indicator{\n      margin-top:0; }\n  .bp3-control.bp3-checkbox input:indeterminate ~ .bp3-control-indicator{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n    background-color:#137cbd;\n    background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.1)), to(rgba(255, 255, 255, 0)));\n    background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.1), rgba(255, 255, 255, 0));\n    color:#ffffff; }\n  .bp3-control.bp3-checkbox:hover input:indeterminate ~ .bp3-control-indicator{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 -1px 0 rgba(16, 22, 26, 0.2);\n    background-color:#106ba3; }\n  .bp3-control.bp3-checkbox input:not(:disabled):active:indeterminate ~ .bp3-control-indicator{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n    background:#0e5a8a; }\n  .bp3-control.bp3-checkbox input:disabled:indeterminate ~ .bp3-control-indicator{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:rgba(19, 124, 189, 0.5); }\n  .bp3-dark .bp3-control.bp3-checkbox input:indeterminate ~ .bp3-control-indicator{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4); }\n  .bp3-dark .bp3-control.bp3-checkbox:hover input:indeterminate ~ .bp3-control-indicator{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n    background-color:#106ba3; }\n  .bp3-dark .bp3-control.bp3-checkbox input:not(:disabled):active:indeterminate ~ .bp3-control-indicator{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n    background-color:#0e5a8a; }\n  .bp3-dark .bp3-control.bp3-checkbox input:disabled:indeterminate ~ .bp3-control-indicator{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:rgba(14, 90, 138, 0.5); }\n  .bp3-control.bp3-checkbox .bp3-control-indicator{\n    border-radius:3px; }\n  .bp3-control.bp3-checkbox input:checked ~ .bp3-control-indicator::before{\n    background-image:url("data:image/svg+xml,%3csvg xmlns=\'http://www.w3.org/2000/svg\' viewBox=\'0 0 16 16\'%3e%3cpath fill-rule=\'evenodd\' clip-rule=\'evenodd\' d=\'M12 5c-.28 0-.53.11-.71.29L7 9.59l-2.29-2.3a1.003 1.003 0 0 0-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l5-5A1.003 1.003 0 0 0 12 5z\' fill=\'white\'/%3e%3c/svg%3e"); }\n  .bp3-control.bp3-checkbox input:indeterminate ~ .bp3-control-indicator::before{\n    background-image:url("data:image/svg+xml,%3csvg xmlns=\'http://www.w3.org/2000/svg\' viewBox=\'0 0 16 16\'%3e%3cpath fill-rule=\'evenodd\' clip-rule=\'evenodd\' d=\'M11 7H5c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1z\' fill=\'white\'/%3e%3c/svg%3e"); }\n  .bp3-control.bp3-radio .bp3-control-indicator{\n    border-radius:50%; }\n  .bp3-control.bp3-radio input:checked ~ .bp3-control-indicator::before{\n    background-image:radial-gradient(#ffffff, #ffffff 28%, transparent 32%); }\n  .bp3-control.bp3-radio input:checked:disabled ~ .bp3-control-indicator::before{\n    opacity:0.5; }\n  .bp3-control.bp3-radio input:focus ~ .bp3-control-indicator{\n    -moz-outline-radius:16px; }\n  .bp3-control.bp3-switch input ~ .bp3-control-indicator{\n    background:rgba(167, 182, 194, 0.5); }\n  .bp3-control.bp3-switch:hover input ~ .bp3-control-indicator{\n    background:rgba(115, 134, 148, 0.5); }\n  .bp3-control.bp3-switch input:not(:disabled):active ~ .bp3-control-indicator{\n    background:rgba(92, 112, 128, 0.5); }\n  .bp3-control.bp3-switch input:disabled ~ .bp3-control-indicator{\n    background:rgba(206, 217, 224, 0.5); }\n  .bp3-control.bp3-switch input:checked ~ .bp3-control-indicator{\n    background:#137cbd; }\n  .bp3-control.bp3-switch:hover input:checked ~ .bp3-control-indicator{\n    background:#106ba3; }\n  .bp3-control.bp3-switch input:checked:not(:disabled):active ~ .bp3-control-indicator{\n    background:#0e5a8a; }\n  .bp3-control.bp3-switch input:checked:disabled ~ .bp3-control-indicator{\n    background:rgba(19, 124, 189, 0.5); }\n  .bp3-control.bp3-switch:not(.bp3-align-right){\n    padding-left:38px; }\n    .bp3-control.bp3-switch:not(.bp3-align-right) .bp3-control-indicator{\n      margin-left:-38px; }\n  .bp3-control.bp3-switch.bp3-align-right{\n    padding-right:38px; }\n    .bp3-control.bp3-switch.bp3-align-right .bp3-control-indicator{\n      margin-right:-38px; }\n  .bp3-control.bp3-switch .bp3-control-indicator{\n    border:none;\n    border-radius:1.75em;\n    -webkit-box-shadow:none !important;\n            box-shadow:none !important;\n    width:auto;\n    min-width:1.75em;\n    -webkit-transition:background-color 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n    transition:background-color 100ms cubic-bezier(0.4, 1, 0.75, 0.9); }\n    .bp3-control.bp3-switch .bp3-control-indicator::before{\n      position:absolute;\n      left:0;\n      margin:2px;\n      border-radius:50%;\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 1px 1px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 1px 1px rgba(16, 22, 26, 0.2);\n      background:#ffffff;\n      width:calc(1em - 4px);\n      height:calc(1em - 4px);\n      -webkit-transition:left 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n      transition:left 100ms cubic-bezier(0.4, 1, 0.75, 0.9); }\n  .bp3-control.bp3-switch input:checked ~ .bp3-control-indicator::before{\n    left:calc(100% - 1em); }\n  .bp3-control.bp3-switch.bp3-large:not(.bp3-align-right){\n    padding-left:45px; }\n    .bp3-control.bp3-switch.bp3-large:not(.bp3-align-right) .bp3-control-indicator{\n      margin-left:-45px; }\n  .bp3-control.bp3-switch.bp3-large.bp3-align-right{\n    padding-right:45px; }\n    .bp3-control.bp3-switch.bp3-large.bp3-align-right .bp3-control-indicator{\n      margin-right:-45px; }\n  .bp3-dark .bp3-control.bp3-switch input ~ .bp3-control-indicator{\n    background:rgba(16, 22, 26, 0.5); }\n  .bp3-dark .bp3-control.bp3-switch:hover input ~ .bp3-control-indicator{\n    background:rgba(16, 22, 26, 0.7); }\n  .bp3-dark .bp3-control.bp3-switch input:not(:disabled):active ~ .bp3-control-indicator{\n    background:rgba(16, 22, 26, 0.9); }\n  .bp3-dark .bp3-control.bp3-switch input:disabled ~ .bp3-control-indicator{\n    background:rgba(57, 75, 89, 0.5); }\n  .bp3-dark .bp3-control.bp3-switch input:checked ~ .bp3-control-indicator{\n    background:#137cbd; }\n  .bp3-dark .bp3-control.bp3-switch:hover input:checked ~ .bp3-control-indicator{\n    background:#106ba3; }\n  .bp3-dark .bp3-control.bp3-switch input:checked:not(:disabled):active ~ .bp3-control-indicator{\n    background:#0e5a8a; }\n  .bp3-dark .bp3-control.bp3-switch input:checked:disabled ~ .bp3-control-indicator{\n    background:rgba(14, 90, 138, 0.5); }\n  .bp3-dark .bp3-control.bp3-switch .bp3-control-indicator::before{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n    background:#394b59; }\n  .bp3-dark .bp3-control.bp3-switch input:checked ~ .bp3-control-indicator::before{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4); }\n  .bp3-control.bp3-switch .bp3-switch-inner-text{\n    text-align:center;\n    font-size:0.7em; }\n  .bp3-control.bp3-switch .bp3-control-indicator-child:first-child{\n    visibility:hidden;\n    margin-right:1.2em;\n    margin-left:0.5em;\n    line-height:0; }\n  .bp3-control.bp3-switch .bp3-control-indicator-child:last-child{\n    visibility:visible;\n    margin-right:0.5em;\n    margin-left:1.2em;\n    line-height:1em; }\n  .bp3-control.bp3-switch input:checked ~ .bp3-control-indicator .bp3-control-indicator-child:first-child{\n    visibility:visible;\n    line-height:1em; }\n  .bp3-control.bp3-switch input:checked ~ .bp3-control-indicator .bp3-control-indicator-child:last-child{\n    visibility:hidden;\n    line-height:0; }\n  .bp3-dark .bp3-control{\n    color:#f5f8fa; }\n    .bp3-dark .bp3-control.bp3-disabled{\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark .bp3-control .bp3-control-indicator{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n      background-color:#394b59;\n      background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.05)), to(rgba(255, 255, 255, 0)));\n      background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.05), rgba(255, 255, 255, 0)); }\n    .bp3-dark .bp3-control:hover .bp3-control-indicator{\n      background-color:#30404d; }\n    .bp3-dark .bp3-control input:not(:disabled):active ~ .bp3-control-indicator{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.6), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.6), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n      background:#202b33; }\n    .bp3-dark .bp3-control input:disabled ~ .bp3-control-indicator{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:rgba(57, 75, 89, 0.5);\n      cursor:not-allowed; }\n    .bp3-dark .bp3-control.bp3-checkbox input:disabled:checked ~ .bp3-control-indicator, .bp3-dark .bp3-control.bp3-checkbox input:disabled:indeterminate ~ .bp3-control-indicator{\n      color:rgba(167, 182, 194, 0.6); }\n.bp3-file-input{\n  display:inline-block;\n  position:relative;\n  cursor:pointer;\n  height:30px; }\n  .bp3-file-input input{\n    opacity:0;\n    margin:0;\n    min-width:200px; }\n    .bp3-file-input input:disabled + .bp3-file-upload-input,\n    .bp3-file-input input.bp3-disabled + .bp3-file-upload-input{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:rgba(206, 217, 224, 0.5);\n      cursor:not-allowed;\n      color:rgba(92, 112, 128, 0.6);\n      resize:none; }\n      .bp3-file-input input:disabled + .bp3-file-upload-input::after,\n      .bp3-file-input input.bp3-disabled + .bp3-file-upload-input::after{\n        outline:none;\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background-color:rgba(206, 217, 224, 0.5);\n        background-image:none;\n        cursor:not-allowed;\n        color:rgba(92, 112, 128, 0.6); }\n        .bp3-file-input input:disabled + .bp3-file-upload-input::after.bp3-active, .bp3-file-input input:disabled + .bp3-file-upload-input::after.bp3-active:hover,\n        .bp3-file-input input.bp3-disabled + .bp3-file-upload-input::after.bp3-active,\n        .bp3-file-input input.bp3-disabled + .bp3-file-upload-input::after.bp3-active:hover{\n          background:rgba(206, 217, 224, 0.7); }\n      .bp3-dark .bp3-file-input input:disabled + .bp3-file-upload-input, .bp3-dark\n      .bp3-file-input input.bp3-disabled + .bp3-file-upload-input{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background:rgba(57, 75, 89, 0.5);\n        color:rgba(167, 182, 194, 0.6); }\n        .bp3-dark .bp3-file-input input:disabled + .bp3-file-upload-input::after, .bp3-dark\n        .bp3-file-input input.bp3-disabled + .bp3-file-upload-input::after{\n          -webkit-box-shadow:none;\n                  box-shadow:none;\n          background-color:rgba(57, 75, 89, 0.5);\n          background-image:none;\n          color:rgba(167, 182, 194, 0.6); }\n          .bp3-dark .bp3-file-input input:disabled + .bp3-file-upload-input::after.bp3-active, .bp3-dark\n          .bp3-file-input input.bp3-disabled + .bp3-file-upload-input::after.bp3-active{\n            background:rgba(57, 75, 89, 0.7); }\n  .bp3-file-input.bp3-file-input-has-selection .bp3-file-upload-input{\n    color:#182026; }\n  .bp3-dark .bp3-file-input.bp3-file-input-has-selection .bp3-file-upload-input{\n    color:#f5f8fa; }\n  .bp3-file-input.bp3-fill{\n    width:100%; }\n  .bp3-file-input.bp3-large,\n  .bp3-large .bp3-file-input{\n    height:40px; }\n\n.bp3-file-upload-input{\n  outline:none;\n  border:none;\n  border-radius:3px;\n  -webkit-box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n  background:#ffffff;\n  height:30px;\n  padding:0 10px;\n  vertical-align:middle;\n  line-height:30px;\n  color:#182026;\n  font-size:14px;\n  font-weight:400;\n  -webkit-transition:-webkit-box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:-webkit-box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9), -webkit-box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  -webkit-appearance:none;\n     -moz-appearance:none;\n          appearance:none;\n  overflow:hidden;\n  text-overflow:ellipsis;\n  white-space:nowrap;\n  word-wrap:normal;\n  position:absolute;\n  top:0;\n  right:0;\n  left:0;\n  padding-right:80px;\n  color:rgba(92, 112, 128, 0.6);\n  -webkit-user-select:none;\n     -moz-user-select:none;\n      -ms-user-select:none;\n          user-select:none; }\n  .bp3-file-upload-input::-webkit-input-placeholder{\n    opacity:1;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-file-upload-input::-ms-input-placeholder{\n    opacity:1;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-file-upload-input::placeholder{\n    opacity:1;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-file-upload-input:focus, .bp3-file-upload-input.bp3-active{\n    -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n  .bp3-file-upload-input[type="search"], .bp3-file-upload-input.bp3-round{\n    border-radius:30px;\n    -webkit-box-sizing:border-box;\n            box-sizing:border-box;\n    padding-left:10px; }\n  .bp3-file-upload-input[readonly]{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.15);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.15); }\n  .bp3-file-upload-input:disabled, .bp3-file-upload-input.bp3-disabled{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:rgba(206, 217, 224, 0.5);\n    cursor:not-allowed;\n    color:rgba(92, 112, 128, 0.6);\n    resize:none; }\n  .bp3-file-upload-input::after{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n    background-color:#f5f8fa;\n    background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.8)), to(rgba(255, 255, 255, 0)));\n    background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.8), rgba(255, 255, 255, 0));\n    color:#182026;\n    min-width:24px;\n    min-height:24px;\n    position:absolute;\n    top:0;\n    right:0;\n    margin:3px;\n    border-radius:3px;\n    width:70px;\n    text-align:center;\n    line-height:24px;\n    content:"Browse"; }\n    .bp3-file-upload-input::after:hover{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n      background-clip:padding-box;\n      background-color:#ebf1f5; }\n    .bp3-file-upload-input::after:active, .bp3-file-upload-input::after.bp3-active{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n      background-color:#d8e1e8;\n      background-image:none; }\n    .bp3-file-upload-input::after:disabled, .bp3-file-upload-input::after.bp3-disabled{\n      outline:none;\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background-color:rgba(206, 217, 224, 0.5);\n      background-image:none;\n      cursor:not-allowed;\n      color:rgba(92, 112, 128, 0.6); }\n      .bp3-file-upload-input::after:disabled.bp3-active, .bp3-file-upload-input::after:disabled.bp3-active:hover, .bp3-file-upload-input::after.bp3-disabled.bp3-active, .bp3-file-upload-input::after.bp3-disabled.bp3-active:hover{\n        background:rgba(206, 217, 224, 0.7); }\n  .bp3-file-upload-input:hover::after{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n    background-clip:padding-box;\n    background-color:#ebf1f5; }\n  .bp3-file-upload-input:active::after{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n    background-color:#d8e1e8;\n    background-image:none; }\n  .bp3-large .bp3-file-upload-input{\n    height:40px;\n    line-height:40px;\n    font-size:16px;\n    padding-right:95px; }\n    .bp3-large .bp3-file-upload-input[type="search"], .bp3-large .bp3-file-upload-input.bp3-round{\n      padding:0 15px; }\n    .bp3-large .bp3-file-upload-input::after{\n      min-width:30px;\n      min-height:30px;\n      margin:5px;\n      width:85px;\n      line-height:30px; }\n  .bp3-dark .bp3-file-upload-input{\n    -webkit-box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n    background:rgba(16, 22, 26, 0.3);\n    color:#f5f8fa;\n    color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark .bp3-file-upload-input::-webkit-input-placeholder{\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark .bp3-file-upload-input::-ms-input-placeholder{\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark .bp3-file-upload-input::placeholder{\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark .bp3-file-upload-input:focus{\n      -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 1px #137cbd, 0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n    .bp3-dark .bp3-file-upload-input[readonly]{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4); }\n    .bp3-dark .bp3-file-upload-input:disabled, .bp3-dark .bp3-file-upload-input.bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:rgba(57, 75, 89, 0.5);\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark .bp3-file-upload-input::after{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n      background-color:#394b59;\n      background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.05)), to(rgba(255, 255, 255, 0)));\n      background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.05), rgba(255, 255, 255, 0));\n      color:#f5f8fa; }\n      .bp3-dark .bp3-file-upload-input::after:hover, .bp3-dark .bp3-file-upload-input::after:active, .bp3-dark .bp3-file-upload-input::after.bp3-active{\n        color:#f5f8fa; }\n      .bp3-dark .bp3-file-upload-input::after:hover{\n        -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n                box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n        background-color:#30404d; }\n      .bp3-dark .bp3-file-upload-input::after:active, .bp3-dark .bp3-file-upload-input::after.bp3-active{\n        -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.6), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n                box-shadow:0 0 0 1px rgba(16, 22, 26, 0.6), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n        background-color:#202b33;\n        background-image:none; }\n      .bp3-dark .bp3-file-upload-input::after:disabled, .bp3-dark .bp3-file-upload-input::after.bp3-disabled{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background-color:rgba(57, 75, 89, 0.5);\n        background-image:none;\n        color:rgba(167, 182, 194, 0.6); }\n        .bp3-dark .bp3-file-upload-input::after:disabled.bp3-active, .bp3-dark .bp3-file-upload-input::after.bp3-disabled.bp3-active{\n          background:rgba(57, 75, 89, 0.7); }\n      .bp3-dark .bp3-file-upload-input::after .bp3-button-spinner .bp3-spinner-head{\n        background:rgba(16, 22, 26, 0.5);\n        stroke:#8a9ba8; }\n    .bp3-dark .bp3-file-upload-input:hover::after{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n      background-color:#30404d; }\n    .bp3-dark .bp3-file-upload-input:active::after{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.6), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.6), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n      background-color:#202b33;\n      background-image:none; }\n\n.bp3-file-upload-input::after{\n  -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n          box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1); }\n.bp3-form-group{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-orient:vertical;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:column;\n      -ms-flex-direction:column;\n          flex-direction:column;\n  margin:0 0 15px; }\n  .bp3-form-group label.bp3-label{\n    margin-bottom:5px; }\n  .bp3-form-group .bp3-control{\n    margin-top:7px; }\n  .bp3-form-group .bp3-form-helper-text{\n    margin-top:5px;\n    color:#5c7080;\n    font-size:12px; }\n  .bp3-form-group.bp3-intent-primary .bp3-form-helper-text{\n    color:#106ba3; }\n  .bp3-form-group.bp3-intent-success .bp3-form-helper-text{\n    color:#0d8050; }\n  .bp3-form-group.bp3-intent-warning .bp3-form-helper-text{\n    color:#bf7326; }\n  .bp3-form-group.bp3-intent-danger .bp3-form-helper-text{\n    color:#c23030; }\n  .bp3-form-group.bp3-inline{\n    -webkit-box-orient:horizontal;\n    -webkit-box-direction:normal;\n    -webkit-flex-direction:row;\n        -ms-flex-direction:row;\n            flex-direction:row;\n    -webkit-box-align:start;\n    -webkit-align-items:flex-start;\n        -ms-flex-align:start;\n            align-items:flex-start; }\n    .bp3-form-group.bp3-inline.bp3-large label.bp3-label{\n      margin:0 10px 0 0;\n      line-height:40px; }\n    .bp3-form-group.bp3-inline label.bp3-label{\n      margin:0 10px 0 0;\n      line-height:30px; }\n  .bp3-form-group.bp3-disabled .bp3-label,\n  .bp3-form-group.bp3-disabled .bp3-text-muted,\n  .bp3-form-group.bp3-disabled .bp3-form-helper-text{\n    color:rgba(92, 112, 128, 0.6) !important; }\n  .bp3-dark .bp3-form-group.bp3-intent-primary .bp3-form-helper-text{\n    color:#48aff0; }\n  .bp3-dark .bp3-form-group.bp3-intent-success .bp3-form-helper-text{\n    color:#3dcc91; }\n  .bp3-dark .bp3-form-group.bp3-intent-warning .bp3-form-helper-text{\n    color:#ffb366; }\n  .bp3-dark .bp3-form-group.bp3-intent-danger .bp3-form-helper-text{\n    color:#ff7373; }\n  .bp3-dark .bp3-form-group .bp3-form-helper-text{\n    color:#a7b6c2; }\n  .bp3-dark .bp3-form-group.bp3-disabled .bp3-label,\n  .bp3-dark .bp3-form-group.bp3-disabled .bp3-text-muted,\n  .bp3-dark .bp3-form-group.bp3-disabled .bp3-form-helper-text{\n    color:rgba(167, 182, 194, 0.6) !important; }\n.bp3-input-group{\n  display:block;\n  position:relative; }\n  .bp3-input-group .bp3-input{\n    position:relative;\n    width:100%; }\n    .bp3-input-group .bp3-input:not(:first-child){\n      padding-left:30px; }\n    .bp3-input-group .bp3-input:not(:last-child){\n      padding-right:30px; }\n  .bp3-input-group .bp3-input-action,\n  .bp3-input-group > .bp3-button,\n  .bp3-input-group > .bp3-icon{\n    position:absolute;\n    top:0; }\n    .bp3-input-group .bp3-input-action:first-child,\n    .bp3-input-group > .bp3-button:first-child,\n    .bp3-input-group > .bp3-icon:first-child{\n      left:0; }\n    .bp3-input-group .bp3-input-action:last-child,\n    .bp3-input-group > .bp3-button:last-child,\n    .bp3-input-group > .bp3-icon:last-child{\n      right:0; }\n  .bp3-input-group .bp3-button{\n    min-width:24px;\n    min-height:24px;\n    margin:3px;\n    padding:0 7px; }\n    .bp3-input-group .bp3-button:empty{\n      padding:0; }\n  .bp3-input-group > .bp3-icon{\n    z-index:1;\n    color:#5c7080; }\n    .bp3-input-group > .bp3-icon:empty{\n      line-height:1;\n      font-family:"Icons16", sans-serif;\n      font-size:16px;\n      font-weight:400;\n      font-style:normal;\n      -moz-osx-font-smoothing:grayscale;\n      -webkit-font-smoothing:antialiased; }\n  .bp3-input-group > .bp3-icon,\n  .bp3-input-group .bp3-input-action > .bp3-spinner{\n    margin:7px; }\n  .bp3-input-group .bp3-tag{\n    margin:5px; }\n  .bp3-input-group .bp3-input:not(:focus) + .bp3-button.bp3-minimal:not(:hover):not(:focus),\n  .bp3-input-group .bp3-input:not(:focus) + .bp3-input-action .bp3-button.bp3-minimal:not(:hover):not(:focus){\n    color:#5c7080; }\n    .bp3-dark .bp3-input-group .bp3-input:not(:focus) + .bp3-button.bp3-minimal:not(:hover):not(:focus), .bp3-dark\n    .bp3-input-group .bp3-input:not(:focus) + .bp3-input-action .bp3-button.bp3-minimal:not(:hover):not(:focus){\n      color:#a7b6c2; }\n    .bp3-input-group .bp3-input:not(:focus) + .bp3-button.bp3-minimal:not(:hover):not(:focus) .bp3-icon, .bp3-input-group .bp3-input:not(:focus) + .bp3-button.bp3-minimal:not(:hover):not(:focus) .bp3-icon-standard, .bp3-input-group .bp3-input:not(:focus) + .bp3-button.bp3-minimal:not(:hover):not(:focus) .bp3-icon-large,\n    .bp3-input-group .bp3-input:not(:focus) + .bp3-input-action .bp3-button.bp3-minimal:not(:hover):not(:focus) .bp3-icon,\n    .bp3-input-group .bp3-input:not(:focus) + .bp3-input-action .bp3-button.bp3-minimal:not(:hover):not(:focus) .bp3-icon-standard,\n    .bp3-input-group .bp3-input:not(:focus) + .bp3-input-action .bp3-button.bp3-minimal:not(:hover):not(:focus) .bp3-icon-large{\n      color:#5c7080; }\n  .bp3-input-group .bp3-input:not(:focus) + .bp3-button.bp3-minimal:disabled,\n  .bp3-input-group .bp3-input:not(:focus) + .bp3-input-action .bp3-button.bp3-minimal:disabled{\n    color:rgba(92, 112, 128, 0.6) !important; }\n    .bp3-input-group .bp3-input:not(:focus) + .bp3-button.bp3-minimal:disabled .bp3-icon, .bp3-input-group .bp3-input:not(:focus) + .bp3-button.bp3-minimal:disabled .bp3-icon-standard, .bp3-input-group .bp3-input:not(:focus) + .bp3-button.bp3-minimal:disabled .bp3-icon-large,\n    .bp3-input-group .bp3-input:not(:focus) + .bp3-input-action .bp3-button.bp3-minimal:disabled .bp3-icon,\n    .bp3-input-group .bp3-input:not(:focus) + .bp3-input-action .bp3-button.bp3-minimal:disabled .bp3-icon-standard,\n    .bp3-input-group .bp3-input:not(:focus) + .bp3-input-action .bp3-button.bp3-minimal:disabled .bp3-icon-large{\n      color:rgba(92, 112, 128, 0.6) !important; }\n  .bp3-input-group.bp3-disabled{\n    cursor:not-allowed; }\n    .bp3-input-group.bp3-disabled .bp3-icon{\n      color:rgba(92, 112, 128, 0.6); }\n  .bp3-input-group.bp3-large .bp3-button{\n    min-width:30px;\n    min-height:30px;\n    margin:5px; }\n  .bp3-input-group.bp3-large > .bp3-icon,\n  .bp3-input-group.bp3-large .bp3-input-action > .bp3-spinner{\n    margin:12px; }\n  .bp3-input-group.bp3-large .bp3-input{\n    height:40px;\n    line-height:40px;\n    font-size:16px; }\n    .bp3-input-group.bp3-large .bp3-input[type="search"], .bp3-input-group.bp3-large .bp3-input.bp3-round{\n      padding:0 15px; }\n    .bp3-input-group.bp3-large .bp3-input:not(:first-child){\n      padding-left:40px; }\n    .bp3-input-group.bp3-large .bp3-input:not(:last-child){\n      padding-right:40px; }\n  .bp3-input-group.bp3-small .bp3-button{\n    min-width:20px;\n    min-height:20px;\n    margin:2px; }\n  .bp3-input-group.bp3-small .bp3-tag{\n    min-width:20px;\n    min-height:20px;\n    margin:2px; }\n  .bp3-input-group.bp3-small > .bp3-icon,\n  .bp3-input-group.bp3-small .bp3-input-action > .bp3-spinner{\n    margin:4px; }\n  .bp3-input-group.bp3-small .bp3-input{\n    height:24px;\n    padding-right:8px;\n    padding-left:8px;\n    line-height:24px;\n    font-size:12px; }\n    .bp3-input-group.bp3-small .bp3-input[type="search"], .bp3-input-group.bp3-small .bp3-input.bp3-round{\n      padding:0 12px; }\n    .bp3-input-group.bp3-small .bp3-input:not(:first-child){\n      padding-left:24px; }\n    .bp3-input-group.bp3-small .bp3-input:not(:last-child){\n      padding-right:24px; }\n  .bp3-input-group.bp3-fill{\n    -webkit-box-flex:1;\n    -webkit-flex:1 1 auto;\n        -ms-flex:1 1 auto;\n            flex:1 1 auto;\n    width:100%; }\n  .bp3-input-group.bp3-round .bp3-button,\n  .bp3-input-group.bp3-round .bp3-input,\n  .bp3-input-group.bp3-round .bp3-tag{\n    border-radius:30px; }\n  .bp3-dark .bp3-input-group .bp3-icon{\n    color:#a7b6c2; }\n  .bp3-dark .bp3-input-group.bp3-disabled .bp3-icon{\n    color:rgba(167, 182, 194, 0.6); }\n  .bp3-input-group.bp3-intent-primary .bp3-input{\n    -webkit-box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px #137cbd, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px #137cbd, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input-group.bp3-intent-primary .bp3-input:focus{\n      -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input-group.bp3-intent-primary .bp3-input[readonly]{\n      -webkit-box-shadow:inset 0 0 0 1px #137cbd;\n              box-shadow:inset 0 0 0 1px #137cbd; }\n    .bp3-input-group.bp3-intent-primary .bp3-input:disabled, .bp3-input-group.bp3-intent-primary .bp3-input.bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none; }\n  .bp3-input-group.bp3-intent-primary > .bp3-icon{\n    color:#106ba3; }\n    .bp3-dark .bp3-input-group.bp3-intent-primary > .bp3-icon{\n      color:#48aff0; }\n  .bp3-input-group.bp3-intent-success .bp3-input{\n    -webkit-box-shadow:0 0 0 0 rgba(15, 153, 96, 0), 0 0 0 0 rgba(15, 153, 96, 0), inset 0 0 0 1px #0f9960, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 0 rgba(15, 153, 96, 0), 0 0 0 0 rgba(15, 153, 96, 0), inset 0 0 0 1px #0f9960, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input-group.bp3-intent-success .bp3-input:focus{\n      -webkit-box-shadow:0 0 0 1px #0f9960, 0 0 0 3px rgba(15, 153, 96, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px #0f9960, 0 0 0 3px rgba(15, 153, 96, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input-group.bp3-intent-success .bp3-input[readonly]{\n      -webkit-box-shadow:inset 0 0 0 1px #0f9960;\n              box-shadow:inset 0 0 0 1px #0f9960; }\n    .bp3-input-group.bp3-intent-success .bp3-input:disabled, .bp3-input-group.bp3-intent-success .bp3-input.bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none; }\n  .bp3-input-group.bp3-intent-success > .bp3-icon{\n    color:#0d8050; }\n    .bp3-dark .bp3-input-group.bp3-intent-success > .bp3-icon{\n      color:#3dcc91; }\n  .bp3-input-group.bp3-intent-warning .bp3-input{\n    -webkit-box-shadow:0 0 0 0 rgba(217, 130, 43, 0), 0 0 0 0 rgba(217, 130, 43, 0), inset 0 0 0 1px #d9822b, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 0 rgba(217, 130, 43, 0), 0 0 0 0 rgba(217, 130, 43, 0), inset 0 0 0 1px #d9822b, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input-group.bp3-intent-warning .bp3-input:focus{\n      -webkit-box-shadow:0 0 0 1px #d9822b, 0 0 0 3px rgba(217, 130, 43, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px #d9822b, 0 0 0 3px rgba(217, 130, 43, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input-group.bp3-intent-warning .bp3-input[readonly]{\n      -webkit-box-shadow:inset 0 0 0 1px #d9822b;\n              box-shadow:inset 0 0 0 1px #d9822b; }\n    .bp3-input-group.bp3-intent-warning .bp3-input:disabled, .bp3-input-group.bp3-intent-warning .bp3-input.bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none; }\n  .bp3-input-group.bp3-intent-warning > .bp3-icon{\n    color:#bf7326; }\n    .bp3-dark .bp3-input-group.bp3-intent-warning > .bp3-icon{\n      color:#ffb366; }\n  .bp3-input-group.bp3-intent-danger .bp3-input{\n    -webkit-box-shadow:0 0 0 0 rgba(219, 55, 55, 0), 0 0 0 0 rgba(219, 55, 55, 0), inset 0 0 0 1px #db3737, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 0 rgba(219, 55, 55, 0), 0 0 0 0 rgba(219, 55, 55, 0), inset 0 0 0 1px #db3737, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input-group.bp3-intent-danger .bp3-input:focus{\n      -webkit-box-shadow:0 0 0 1px #db3737, 0 0 0 3px rgba(219, 55, 55, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px #db3737, 0 0 0 3px rgba(219, 55, 55, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input-group.bp3-intent-danger .bp3-input[readonly]{\n      -webkit-box-shadow:inset 0 0 0 1px #db3737;\n              box-shadow:inset 0 0 0 1px #db3737; }\n    .bp3-input-group.bp3-intent-danger .bp3-input:disabled, .bp3-input-group.bp3-intent-danger .bp3-input.bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none; }\n  .bp3-input-group.bp3-intent-danger > .bp3-icon{\n    color:#c23030; }\n    .bp3-dark .bp3-input-group.bp3-intent-danger > .bp3-icon{\n      color:#ff7373; }\n.bp3-input{\n  outline:none;\n  border:none;\n  border-radius:3px;\n  -webkit-box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n  background:#ffffff;\n  height:30px;\n  padding:0 10px;\n  vertical-align:middle;\n  line-height:30px;\n  color:#182026;\n  font-size:14px;\n  font-weight:400;\n  -webkit-transition:-webkit-box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:-webkit-box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9), -webkit-box-shadow 100ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  -webkit-appearance:none;\n     -moz-appearance:none;\n          appearance:none; }\n  .bp3-input::-webkit-input-placeholder{\n    opacity:1;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-input::-ms-input-placeholder{\n    opacity:1;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-input::placeholder{\n    opacity:1;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-input:focus, .bp3-input.bp3-active{\n    -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n  .bp3-input[type="search"], .bp3-input.bp3-round{\n    border-radius:30px;\n    -webkit-box-sizing:border-box;\n            box-sizing:border-box;\n    padding-left:10px; }\n  .bp3-input[readonly]{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.15);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.15); }\n  .bp3-input:disabled, .bp3-input.bp3-disabled{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:rgba(206, 217, 224, 0.5);\n    cursor:not-allowed;\n    color:rgba(92, 112, 128, 0.6);\n    resize:none; }\n  .bp3-input.bp3-large{\n    height:40px;\n    line-height:40px;\n    font-size:16px; }\n    .bp3-input.bp3-large[type="search"], .bp3-input.bp3-large.bp3-round{\n      padding:0 15px; }\n  .bp3-input.bp3-small{\n    height:24px;\n    padding-right:8px;\n    padding-left:8px;\n    line-height:24px;\n    font-size:12px; }\n    .bp3-input.bp3-small[type="search"], .bp3-input.bp3-small.bp3-round{\n      padding:0 12px; }\n  .bp3-input.bp3-fill{\n    -webkit-box-flex:1;\n    -webkit-flex:1 1 auto;\n        -ms-flex:1 1 auto;\n            flex:1 1 auto;\n    width:100%; }\n  .bp3-dark .bp3-input{\n    -webkit-box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n    background:rgba(16, 22, 26, 0.3);\n    color:#f5f8fa; }\n    .bp3-dark .bp3-input::-webkit-input-placeholder{\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark .bp3-input::-ms-input-placeholder{\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark .bp3-input::placeholder{\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark .bp3-input:focus{\n      -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 1px #137cbd, 0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n    .bp3-dark .bp3-input[readonly]{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4); }\n    .bp3-dark .bp3-input:disabled, .bp3-dark .bp3-input.bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:rgba(57, 75, 89, 0.5);\n      color:rgba(167, 182, 194, 0.6); }\n  .bp3-input.bp3-intent-primary{\n    -webkit-box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px #137cbd, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px #137cbd, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input.bp3-intent-primary:focus{\n      -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input.bp3-intent-primary[readonly]{\n      -webkit-box-shadow:inset 0 0 0 1px #137cbd;\n              box-shadow:inset 0 0 0 1px #137cbd; }\n    .bp3-input.bp3-intent-primary:disabled, .bp3-input.bp3-intent-primary.bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none; }\n    .bp3-dark .bp3-input.bp3-intent-primary{\n      -webkit-box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px #137cbd, inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px #137cbd, inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n      .bp3-dark .bp3-input.bp3-intent-primary:focus{\n        -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n                box-shadow:0 0 0 1px #137cbd, 0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n      .bp3-dark .bp3-input.bp3-intent-primary[readonly]{\n        -webkit-box-shadow:inset 0 0 0 1px #137cbd;\n                box-shadow:inset 0 0 0 1px #137cbd; }\n      .bp3-dark .bp3-input.bp3-intent-primary:disabled, .bp3-dark .bp3-input.bp3-intent-primary.bp3-disabled{\n        -webkit-box-shadow:none;\n                box-shadow:none; }\n  .bp3-input.bp3-intent-success{\n    -webkit-box-shadow:0 0 0 0 rgba(15, 153, 96, 0), 0 0 0 0 rgba(15, 153, 96, 0), inset 0 0 0 1px #0f9960, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 0 rgba(15, 153, 96, 0), 0 0 0 0 rgba(15, 153, 96, 0), inset 0 0 0 1px #0f9960, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input.bp3-intent-success:focus{\n      -webkit-box-shadow:0 0 0 1px #0f9960, 0 0 0 3px rgba(15, 153, 96, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px #0f9960, 0 0 0 3px rgba(15, 153, 96, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input.bp3-intent-success[readonly]{\n      -webkit-box-shadow:inset 0 0 0 1px #0f9960;\n              box-shadow:inset 0 0 0 1px #0f9960; }\n    .bp3-input.bp3-intent-success:disabled, .bp3-input.bp3-intent-success.bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none; }\n    .bp3-dark .bp3-input.bp3-intent-success{\n      -webkit-box-shadow:0 0 0 0 rgba(15, 153, 96, 0), 0 0 0 0 rgba(15, 153, 96, 0), 0 0 0 0 rgba(15, 153, 96, 0), inset 0 0 0 1px #0f9960, inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 0 rgba(15, 153, 96, 0), 0 0 0 0 rgba(15, 153, 96, 0), 0 0 0 0 rgba(15, 153, 96, 0), inset 0 0 0 1px #0f9960, inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n      .bp3-dark .bp3-input.bp3-intent-success:focus{\n        -webkit-box-shadow:0 0 0 1px #0f9960, 0 0 0 1px #0f9960, 0 0 0 3px rgba(15, 153, 96, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n                box-shadow:0 0 0 1px #0f9960, 0 0 0 1px #0f9960, 0 0 0 3px rgba(15, 153, 96, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n      .bp3-dark .bp3-input.bp3-intent-success[readonly]{\n        -webkit-box-shadow:inset 0 0 0 1px #0f9960;\n                box-shadow:inset 0 0 0 1px #0f9960; }\n      .bp3-dark .bp3-input.bp3-intent-success:disabled, .bp3-dark .bp3-input.bp3-intent-success.bp3-disabled{\n        -webkit-box-shadow:none;\n                box-shadow:none; }\n  .bp3-input.bp3-intent-warning{\n    -webkit-box-shadow:0 0 0 0 rgba(217, 130, 43, 0), 0 0 0 0 rgba(217, 130, 43, 0), inset 0 0 0 1px #d9822b, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 0 rgba(217, 130, 43, 0), 0 0 0 0 rgba(217, 130, 43, 0), inset 0 0 0 1px #d9822b, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input.bp3-intent-warning:focus{\n      -webkit-box-shadow:0 0 0 1px #d9822b, 0 0 0 3px rgba(217, 130, 43, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px #d9822b, 0 0 0 3px rgba(217, 130, 43, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input.bp3-intent-warning[readonly]{\n      -webkit-box-shadow:inset 0 0 0 1px #d9822b;\n              box-shadow:inset 0 0 0 1px #d9822b; }\n    .bp3-input.bp3-intent-warning:disabled, .bp3-input.bp3-intent-warning.bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none; }\n    .bp3-dark .bp3-input.bp3-intent-warning{\n      -webkit-box-shadow:0 0 0 0 rgba(217, 130, 43, 0), 0 0 0 0 rgba(217, 130, 43, 0), 0 0 0 0 rgba(217, 130, 43, 0), inset 0 0 0 1px #d9822b, inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 0 rgba(217, 130, 43, 0), 0 0 0 0 rgba(217, 130, 43, 0), 0 0 0 0 rgba(217, 130, 43, 0), inset 0 0 0 1px #d9822b, inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n      .bp3-dark .bp3-input.bp3-intent-warning:focus{\n        -webkit-box-shadow:0 0 0 1px #d9822b, 0 0 0 1px #d9822b, 0 0 0 3px rgba(217, 130, 43, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n                box-shadow:0 0 0 1px #d9822b, 0 0 0 1px #d9822b, 0 0 0 3px rgba(217, 130, 43, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n      .bp3-dark .bp3-input.bp3-intent-warning[readonly]{\n        -webkit-box-shadow:inset 0 0 0 1px #d9822b;\n                box-shadow:inset 0 0 0 1px #d9822b; }\n      .bp3-dark .bp3-input.bp3-intent-warning:disabled, .bp3-dark .bp3-input.bp3-intent-warning.bp3-disabled{\n        -webkit-box-shadow:none;\n                box-shadow:none; }\n  .bp3-input.bp3-intent-danger{\n    -webkit-box-shadow:0 0 0 0 rgba(219, 55, 55, 0), 0 0 0 0 rgba(219, 55, 55, 0), inset 0 0 0 1px #db3737, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 0 rgba(219, 55, 55, 0), 0 0 0 0 rgba(219, 55, 55, 0), inset 0 0 0 1px #db3737, inset 0 0 0 1px rgba(16, 22, 26, 0.15), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input.bp3-intent-danger:focus{\n      -webkit-box-shadow:0 0 0 1px #db3737, 0 0 0 3px rgba(219, 55, 55, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px #db3737, 0 0 0 3px rgba(219, 55, 55, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2); }\n    .bp3-input.bp3-intent-danger[readonly]{\n      -webkit-box-shadow:inset 0 0 0 1px #db3737;\n              box-shadow:inset 0 0 0 1px #db3737; }\n    .bp3-input.bp3-intent-danger:disabled, .bp3-input.bp3-intent-danger.bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none; }\n    .bp3-dark .bp3-input.bp3-intent-danger{\n      -webkit-box-shadow:0 0 0 0 rgba(219, 55, 55, 0), 0 0 0 0 rgba(219, 55, 55, 0), 0 0 0 0 rgba(219, 55, 55, 0), inset 0 0 0 1px #db3737, inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 0 rgba(219, 55, 55, 0), 0 0 0 0 rgba(219, 55, 55, 0), 0 0 0 0 rgba(219, 55, 55, 0), inset 0 0 0 1px #db3737, inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n      .bp3-dark .bp3-input.bp3-intent-danger:focus{\n        -webkit-box-shadow:0 0 0 1px #db3737, 0 0 0 1px #db3737, 0 0 0 3px rgba(219, 55, 55, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n                box-shadow:0 0 0 1px #db3737, 0 0 0 1px #db3737, 0 0 0 3px rgba(219, 55, 55, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n      .bp3-dark .bp3-input.bp3-intent-danger[readonly]{\n        -webkit-box-shadow:inset 0 0 0 1px #db3737;\n                box-shadow:inset 0 0 0 1px #db3737; }\n      .bp3-dark .bp3-input.bp3-intent-danger:disabled, .bp3-dark .bp3-input.bp3-intent-danger.bp3-disabled{\n        -webkit-box-shadow:none;\n                box-shadow:none; }\n  .bp3-input::-ms-clear{\n    display:none; }\ntextarea.bp3-input{\n  max-width:100%;\n  padding:10px; }\n  textarea.bp3-input, textarea.bp3-input.bp3-large, textarea.bp3-input.bp3-small{\n    height:auto;\n    line-height:inherit; }\n  textarea.bp3-input.bp3-small{\n    padding:8px; }\n  .bp3-dark textarea.bp3-input{\n    -webkit-box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), 0 0 0 0 rgba(19, 124, 189, 0), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n    background:rgba(16, 22, 26, 0.3);\n    color:#f5f8fa; }\n    .bp3-dark textarea.bp3-input::-webkit-input-placeholder{\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark textarea.bp3-input::-ms-input-placeholder{\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark textarea.bp3-input::placeholder{\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark textarea.bp3-input:focus{\n      -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 1px #137cbd, 0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4); }\n    .bp3-dark textarea.bp3-input[readonly]{\n      -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4);\n              box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.4); }\n    .bp3-dark textarea.bp3-input:disabled, .bp3-dark textarea.bp3-input.bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:rgba(57, 75, 89, 0.5);\n      color:rgba(167, 182, 194, 0.6); }\nlabel.bp3-label{\n  display:block;\n  margin-top:0;\n  margin-bottom:15px; }\n  label.bp3-label .bp3-html-select,\n  label.bp3-label .bp3-input,\n  label.bp3-label .bp3-select,\n  label.bp3-label .bp3-slider,\n  label.bp3-label .bp3-popover-wrapper{\n    display:block;\n    margin-top:5px;\n    text-transform:none; }\n  label.bp3-label .bp3-select select,\n  label.bp3-label .bp3-html-select select{\n    width:100%;\n    vertical-align:top;\n    font-weight:400; }\n  label.bp3-label.bp3-disabled,\n  label.bp3-label.bp3-disabled .bp3-text-muted{\n    color:rgba(92, 112, 128, 0.6); }\n  label.bp3-label.bp3-inline{\n    line-height:30px; }\n    label.bp3-label.bp3-inline .bp3-html-select,\n    label.bp3-label.bp3-inline .bp3-input,\n    label.bp3-label.bp3-inline .bp3-input-group,\n    label.bp3-label.bp3-inline .bp3-select,\n    label.bp3-label.bp3-inline .bp3-popover-wrapper{\n      display:inline-block;\n      margin:0 0 0 5px;\n      vertical-align:top; }\n    label.bp3-label.bp3-inline .bp3-input-group .bp3-input{\n      margin-left:0; }\n    label.bp3-label.bp3-inline.bp3-large{\n      line-height:40px; }\n  label.bp3-label:not(.bp3-inline) .bp3-popover-target{\n    display:block; }\n  .bp3-dark label.bp3-label{\n    color:#f5f8fa; }\n    .bp3-dark label.bp3-label.bp3-disabled,\n    .bp3-dark label.bp3-label.bp3-disabled .bp3-text-muted{\n      color:rgba(167, 182, 194, 0.6); }\n.bp3-numeric-input .bp3-button-group.bp3-vertical > .bp3-button{\n  -webkit-box-flex:1;\n  -webkit-flex:1 1 14px;\n      -ms-flex:1 1 14px;\n          flex:1 1 14px;\n  width:30px;\n  min-height:0;\n  padding:0; }\n  .bp3-numeric-input .bp3-button-group.bp3-vertical > .bp3-button:first-child{\n    border-radius:0 3px 0 0; }\n  .bp3-numeric-input .bp3-button-group.bp3-vertical > .bp3-button:last-child{\n    border-radius:0 0 3px 0; }\n\n.bp3-numeric-input .bp3-button-group.bp3-vertical:first-child > .bp3-button:first-child{\n  border-radius:3px 0 0 0; }\n\n.bp3-numeric-input .bp3-button-group.bp3-vertical:first-child > .bp3-button:last-child{\n  border-radius:0 0 0 3px; }\n\n.bp3-numeric-input.bp3-large .bp3-button-group.bp3-vertical > .bp3-button{\n  width:40px; }\n\nform{\n  display:block; }\n.bp3-html-select select,\n.bp3-select select{\n  display:-webkit-inline-box;\n  display:-webkit-inline-flex;\n  display:-ms-inline-flexbox;\n  display:inline-flex;\n  -webkit-box-orient:horizontal;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:row;\n      -ms-flex-direction:row;\n          flex-direction:row;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  -webkit-box-pack:center;\n  -webkit-justify-content:center;\n      -ms-flex-pack:center;\n          justify-content:center;\n  border:none;\n  border-radius:3px;\n  cursor:pointer;\n  padding:5px 10px;\n  vertical-align:middle;\n  text-align:left;\n  font-size:14px;\n  -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n          box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n  background-color:#f5f8fa;\n  background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.8)), to(rgba(255, 255, 255, 0)));\n  background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.8), rgba(255, 255, 255, 0));\n  color:#182026;\n  border-radius:3px;\n  width:100%;\n  height:30px;\n  padding:0 25px 0 10px;\n  -moz-appearance:none;\n  -webkit-appearance:none; }\n  .bp3-html-select select > *, .bp3-select select > *{\n    -webkit-box-flex:0;\n    -webkit-flex-grow:0;\n        -ms-flex-positive:0;\n            flex-grow:0;\n    -webkit-flex-shrink:0;\n        -ms-flex-negative:0;\n            flex-shrink:0; }\n  .bp3-html-select select > .bp3-fill, .bp3-select select > .bp3-fill{\n    -webkit-box-flex:1;\n    -webkit-flex-grow:1;\n        -ms-flex-positive:1;\n            flex-grow:1;\n    -webkit-flex-shrink:1;\n        -ms-flex-negative:1;\n            flex-shrink:1; }\n  .bp3-html-select select::before,\n  .bp3-select select::before, .bp3-html-select select > *, .bp3-select select > *{\n    margin-right:7px; }\n  .bp3-html-select select:empty::before,\n  .bp3-select select:empty::before,\n  .bp3-html-select select > :last-child,\n  .bp3-select select > :last-child{\n    margin-right:0; }\n  .bp3-html-select select:hover,\n  .bp3-select select:hover{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n    background-clip:padding-box;\n    background-color:#ebf1f5; }\n  .bp3-html-select select:active,\n  .bp3-select select:active, .bp3-html-select select.bp3-active,\n  .bp3-select select.bp3-active{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n    background-color:#d8e1e8;\n    background-image:none; }\n  .bp3-html-select select:disabled,\n  .bp3-select select:disabled, .bp3-html-select select.bp3-disabled,\n  .bp3-select select.bp3-disabled{\n    outline:none;\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background-color:rgba(206, 217, 224, 0.5);\n    background-image:none;\n    cursor:not-allowed;\n    color:rgba(92, 112, 128, 0.6); }\n    .bp3-html-select select:disabled.bp3-active,\n    .bp3-select select:disabled.bp3-active, .bp3-html-select select:disabled.bp3-active:hover,\n    .bp3-select select:disabled.bp3-active:hover, .bp3-html-select select.bp3-disabled.bp3-active,\n    .bp3-select select.bp3-disabled.bp3-active, .bp3-html-select select.bp3-disabled.bp3-active:hover,\n    .bp3-select select.bp3-disabled.bp3-active:hover{\n      background:rgba(206, 217, 224, 0.7); }\n\n.bp3-html-select.bp3-minimal select,\n.bp3-select.bp3-minimal select{\n  -webkit-box-shadow:none;\n          box-shadow:none;\n  background:none; }\n  .bp3-html-select.bp3-minimal select:hover,\n  .bp3-select.bp3-minimal select:hover{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:rgba(167, 182, 194, 0.3);\n    text-decoration:none;\n    color:#182026; }\n  .bp3-html-select.bp3-minimal select:active,\n  .bp3-select.bp3-minimal select:active, .bp3-html-select.bp3-minimal select.bp3-active,\n  .bp3-select.bp3-minimal select.bp3-active{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:rgba(115, 134, 148, 0.3);\n    color:#182026; }\n  .bp3-html-select.bp3-minimal select:disabled,\n  .bp3-select.bp3-minimal select:disabled, .bp3-html-select.bp3-minimal select:disabled:hover,\n  .bp3-select.bp3-minimal select:disabled:hover, .bp3-html-select.bp3-minimal select.bp3-disabled,\n  .bp3-select.bp3-minimal select.bp3-disabled, .bp3-html-select.bp3-minimal select.bp3-disabled:hover,\n  .bp3-select.bp3-minimal select.bp3-disabled:hover{\n    background:none;\n    cursor:not-allowed;\n    color:rgba(92, 112, 128, 0.6); }\n    .bp3-html-select.bp3-minimal select:disabled.bp3-active,\n    .bp3-select.bp3-minimal select:disabled.bp3-active, .bp3-html-select.bp3-minimal select:disabled:hover.bp3-active,\n    .bp3-select.bp3-minimal select:disabled:hover.bp3-active, .bp3-html-select.bp3-minimal select.bp3-disabled.bp3-active,\n    .bp3-select.bp3-minimal select.bp3-disabled.bp3-active, .bp3-html-select.bp3-minimal select.bp3-disabled:hover.bp3-active,\n    .bp3-select.bp3-minimal select.bp3-disabled:hover.bp3-active{\n      background:rgba(115, 134, 148, 0.3); }\n  .bp3-dark .bp3-html-select.bp3-minimal select, .bp3-html-select.bp3-minimal .bp3-dark select,\n  .bp3-dark .bp3-select.bp3-minimal select, .bp3-select.bp3-minimal .bp3-dark select{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:none;\n    color:inherit; }\n    .bp3-dark .bp3-html-select.bp3-minimal select:hover, .bp3-html-select.bp3-minimal .bp3-dark select:hover,\n    .bp3-dark .bp3-select.bp3-minimal select:hover, .bp3-select.bp3-minimal .bp3-dark select:hover, .bp3-dark .bp3-html-select.bp3-minimal select:active, .bp3-html-select.bp3-minimal .bp3-dark select:active,\n    .bp3-dark .bp3-select.bp3-minimal select:active, .bp3-select.bp3-minimal .bp3-dark select:active, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-active,\n    .bp3-dark .bp3-select.bp3-minimal select.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-active{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:none; }\n    .bp3-dark .bp3-html-select.bp3-minimal select:hover, .bp3-html-select.bp3-minimal .bp3-dark select:hover,\n    .bp3-dark .bp3-select.bp3-minimal select:hover, .bp3-select.bp3-minimal .bp3-dark select:hover{\n      background:rgba(138, 155, 168, 0.15); }\n    .bp3-dark .bp3-html-select.bp3-minimal select:active, .bp3-html-select.bp3-minimal .bp3-dark select:active,\n    .bp3-dark .bp3-select.bp3-minimal select:active, .bp3-select.bp3-minimal .bp3-dark select:active, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-active,\n    .bp3-dark .bp3-select.bp3-minimal select.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-active{\n      background:rgba(138, 155, 168, 0.3);\n      color:#f5f8fa; }\n    .bp3-dark .bp3-html-select.bp3-minimal select:disabled, .bp3-html-select.bp3-minimal .bp3-dark select:disabled,\n    .bp3-dark .bp3-select.bp3-minimal select:disabled, .bp3-select.bp3-minimal .bp3-dark select:disabled, .bp3-dark .bp3-html-select.bp3-minimal select:disabled:hover, .bp3-html-select.bp3-minimal .bp3-dark select:disabled:hover,\n    .bp3-dark .bp3-select.bp3-minimal select:disabled:hover, .bp3-select.bp3-minimal .bp3-dark select:disabled:hover, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-disabled, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-disabled,\n    .bp3-dark .bp3-select.bp3-minimal select.bp3-disabled, .bp3-select.bp3-minimal .bp3-dark select.bp3-disabled, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-disabled:hover, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-disabled:hover,\n    .bp3-dark .bp3-select.bp3-minimal select.bp3-disabled:hover, .bp3-select.bp3-minimal .bp3-dark select.bp3-disabled:hover{\n      background:none;\n      cursor:not-allowed;\n      color:rgba(167, 182, 194, 0.6); }\n      .bp3-dark .bp3-html-select.bp3-minimal select:disabled.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select:disabled.bp3-active,\n      .bp3-dark .bp3-select.bp3-minimal select:disabled.bp3-active, .bp3-select.bp3-minimal .bp3-dark select:disabled.bp3-active, .bp3-dark .bp3-html-select.bp3-minimal select:disabled:hover.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select:disabled:hover.bp3-active,\n      .bp3-dark .bp3-select.bp3-minimal select:disabled:hover.bp3-active, .bp3-select.bp3-minimal .bp3-dark select:disabled:hover.bp3-active, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-disabled.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-disabled.bp3-active,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-disabled.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-disabled.bp3-active, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-disabled:hover.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-disabled:hover.bp3-active,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-disabled:hover.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-disabled:hover.bp3-active{\n        background:rgba(138, 155, 168, 0.3); }\n  .bp3-html-select.bp3-minimal select.bp3-intent-primary,\n  .bp3-select.bp3-minimal select.bp3-intent-primary{\n    color:#106ba3; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-primary:hover,\n    .bp3-select.bp3-minimal select.bp3-intent-primary:hover, .bp3-html-select.bp3-minimal select.bp3-intent-primary:active,\n    .bp3-select.bp3-minimal select.bp3-intent-primary:active, .bp3-html-select.bp3-minimal select.bp3-intent-primary.bp3-active,\n    .bp3-select.bp3-minimal select.bp3-intent-primary.bp3-active{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:none;\n      color:#106ba3; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-primary:hover,\n    .bp3-select.bp3-minimal select.bp3-intent-primary:hover{\n      background:rgba(19, 124, 189, 0.15);\n      color:#106ba3; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-primary:active,\n    .bp3-select.bp3-minimal select.bp3-intent-primary:active, .bp3-html-select.bp3-minimal select.bp3-intent-primary.bp3-active,\n    .bp3-select.bp3-minimal select.bp3-intent-primary.bp3-active{\n      background:rgba(19, 124, 189, 0.3);\n      color:#106ba3; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-primary:disabled,\n    .bp3-select.bp3-minimal select.bp3-intent-primary:disabled, .bp3-html-select.bp3-minimal select.bp3-intent-primary.bp3-disabled,\n    .bp3-select.bp3-minimal select.bp3-intent-primary.bp3-disabled{\n      background:none;\n      color:rgba(16, 107, 163, 0.5); }\n      .bp3-html-select.bp3-minimal select.bp3-intent-primary:disabled.bp3-active,\n      .bp3-select.bp3-minimal select.bp3-intent-primary:disabled.bp3-active, .bp3-html-select.bp3-minimal select.bp3-intent-primary.bp3-disabled.bp3-active,\n      .bp3-select.bp3-minimal select.bp3-intent-primary.bp3-disabled.bp3-active{\n        background:rgba(19, 124, 189, 0.3); }\n    .bp3-html-select.bp3-minimal select.bp3-intent-primary .bp3-button-spinner .bp3-spinner-head, .bp3-select.bp3-minimal select.bp3-intent-primary .bp3-button-spinner .bp3-spinner-head{\n      stroke:#106ba3; }\n    .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-primary, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-primary,\n    .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-primary, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-primary{\n      color:#48aff0; }\n      .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-primary:hover, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-primary:hover,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-primary:hover, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-primary:hover{\n        background:rgba(19, 124, 189, 0.2);\n        color:#48aff0; }\n      .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-primary:active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-primary:active,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-primary:active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-primary:active, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-primary.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-primary.bp3-active,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-primary.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-primary.bp3-active{\n        background:rgba(19, 124, 189, 0.3);\n        color:#48aff0; }\n      .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-primary:disabled, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-primary:disabled,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-primary:disabled, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-primary:disabled, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-primary.bp3-disabled, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-primary.bp3-disabled,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-primary.bp3-disabled, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-primary.bp3-disabled{\n        background:none;\n        color:rgba(72, 175, 240, 0.5); }\n        .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-primary:disabled.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-primary:disabled.bp3-active,\n        .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-primary:disabled.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-primary:disabled.bp3-active, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-primary.bp3-disabled.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-primary.bp3-disabled.bp3-active,\n        .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-primary.bp3-disabled.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-primary.bp3-disabled.bp3-active{\n          background:rgba(19, 124, 189, 0.3); }\n  .bp3-html-select.bp3-minimal select.bp3-intent-success,\n  .bp3-select.bp3-minimal select.bp3-intent-success{\n    color:#0d8050; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-success:hover,\n    .bp3-select.bp3-minimal select.bp3-intent-success:hover, .bp3-html-select.bp3-minimal select.bp3-intent-success:active,\n    .bp3-select.bp3-minimal select.bp3-intent-success:active, .bp3-html-select.bp3-minimal select.bp3-intent-success.bp3-active,\n    .bp3-select.bp3-minimal select.bp3-intent-success.bp3-active{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:none;\n      color:#0d8050; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-success:hover,\n    .bp3-select.bp3-minimal select.bp3-intent-success:hover{\n      background:rgba(15, 153, 96, 0.15);\n      color:#0d8050; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-success:active,\n    .bp3-select.bp3-minimal select.bp3-intent-success:active, .bp3-html-select.bp3-minimal select.bp3-intent-success.bp3-active,\n    .bp3-select.bp3-minimal select.bp3-intent-success.bp3-active{\n      background:rgba(15, 153, 96, 0.3);\n      color:#0d8050; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-success:disabled,\n    .bp3-select.bp3-minimal select.bp3-intent-success:disabled, .bp3-html-select.bp3-minimal select.bp3-intent-success.bp3-disabled,\n    .bp3-select.bp3-minimal select.bp3-intent-success.bp3-disabled{\n      background:none;\n      color:rgba(13, 128, 80, 0.5); }\n      .bp3-html-select.bp3-minimal select.bp3-intent-success:disabled.bp3-active,\n      .bp3-select.bp3-minimal select.bp3-intent-success:disabled.bp3-active, .bp3-html-select.bp3-minimal select.bp3-intent-success.bp3-disabled.bp3-active,\n      .bp3-select.bp3-minimal select.bp3-intent-success.bp3-disabled.bp3-active{\n        background:rgba(15, 153, 96, 0.3); }\n    .bp3-html-select.bp3-minimal select.bp3-intent-success .bp3-button-spinner .bp3-spinner-head, .bp3-select.bp3-minimal select.bp3-intent-success .bp3-button-spinner .bp3-spinner-head{\n      stroke:#0d8050; }\n    .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-success, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-success,\n    .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-success, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-success{\n      color:#3dcc91; }\n      .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-success:hover, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-success:hover,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-success:hover, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-success:hover{\n        background:rgba(15, 153, 96, 0.2);\n        color:#3dcc91; }\n      .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-success:active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-success:active,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-success:active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-success:active, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-success.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-success.bp3-active,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-success.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-success.bp3-active{\n        background:rgba(15, 153, 96, 0.3);\n        color:#3dcc91; }\n      .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-success:disabled, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-success:disabled,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-success:disabled, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-success:disabled, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-success.bp3-disabled, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-success.bp3-disabled,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-success.bp3-disabled, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-success.bp3-disabled{\n        background:none;\n        color:rgba(61, 204, 145, 0.5); }\n        .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-success:disabled.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-success:disabled.bp3-active,\n        .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-success:disabled.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-success:disabled.bp3-active, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-success.bp3-disabled.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-success.bp3-disabled.bp3-active,\n        .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-success.bp3-disabled.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-success.bp3-disabled.bp3-active{\n          background:rgba(15, 153, 96, 0.3); }\n  .bp3-html-select.bp3-minimal select.bp3-intent-warning,\n  .bp3-select.bp3-minimal select.bp3-intent-warning{\n    color:#bf7326; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-warning:hover,\n    .bp3-select.bp3-minimal select.bp3-intent-warning:hover, .bp3-html-select.bp3-minimal select.bp3-intent-warning:active,\n    .bp3-select.bp3-minimal select.bp3-intent-warning:active, .bp3-html-select.bp3-minimal select.bp3-intent-warning.bp3-active,\n    .bp3-select.bp3-minimal select.bp3-intent-warning.bp3-active{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:none;\n      color:#bf7326; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-warning:hover,\n    .bp3-select.bp3-minimal select.bp3-intent-warning:hover{\n      background:rgba(217, 130, 43, 0.15);\n      color:#bf7326; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-warning:active,\n    .bp3-select.bp3-minimal select.bp3-intent-warning:active, .bp3-html-select.bp3-minimal select.bp3-intent-warning.bp3-active,\n    .bp3-select.bp3-minimal select.bp3-intent-warning.bp3-active{\n      background:rgba(217, 130, 43, 0.3);\n      color:#bf7326; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-warning:disabled,\n    .bp3-select.bp3-minimal select.bp3-intent-warning:disabled, .bp3-html-select.bp3-minimal select.bp3-intent-warning.bp3-disabled,\n    .bp3-select.bp3-minimal select.bp3-intent-warning.bp3-disabled{\n      background:none;\n      color:rgba(191, 115, 38, 0.5); }\n      .bp3-html-select.bp3-minimal select.bp3-intent-warning:disabled.bp3-active,\n      .bp3-select.bp3-minimal select.bp3-intent-warning:disabled.bp3-active, .bp3-html-select.bp3-minimal select.bp3-intent-warning.bp3-disabled.bp3-active,\n      .bp3-select.bp3-minimal select.bp3-intent-warning.bp3-disabled.bp3-active{\n        background:rgba(217, 130, 43, 0.3); }\n    .bp3-html-select.bp3-minimal select.bp3-intent-warning .bp3-button-spinner .bp3-spinner-head, .bp3-select.bp3-minimal select.bp3-intent-warning .bp3-button-spinner .bp3-spinner-head{\n      stroke:#bf7326; }\n    .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-warning, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-warning,\n    .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-warning, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-warning{\n      color:#ffb366; }\n      .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-warning:hover, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-warning:hover,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-warning:hover, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-warning:hover{\n        background:rgba(217, 130, 43, 0.2);\n        color:#ffb366; }\n      .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-warning:active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-warning:active,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-warning:active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-warning:active, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-warning.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-warning.bp3-active,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-warning.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-warning.bp3-active{\n        background:rgba(217, 130, 43, 0.3);\n        color:#ffb366; }\n      .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-warning:disabled, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-warning:disabled,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-warning:disabled, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-warning:disabled, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-warning.bp3-disabled, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-warning.bp3-disabled,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-warning.bp3-disabled, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-warning.bp3-disabled{\n        background:none;\n        color:rgba(255, 179, 102, 0.5); }\n        .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-warning:disabled.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-warning:disabled.bp3-active,\n        .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-warning:disabled.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-warning:disabled.bp3-active, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-warning.bp3-disabled.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-warning.bp3-disabled.bp3-active,\n        .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-warning.bp3-disabled.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-warning.bp3-disabled.bp3-active{\n          background:rgba(217, 130, 43, 0.3); }\n  .bp3-html-select.bp3-minimal select.bp3-intent-danger,\n  .bp3-select.bp3-minimal select.bp3-intent-danger{\n    color:#c23030; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-danger:hover,\n    .bp3-select.bp3-minimal select.bp3-intent-danger:hover, .bp3-html-select.bp3-minimal select.bp3-intent-danger:active,\n    .bp3-select.bp3-minimal select.bp3-intent-danger:active, .bp3-html-select.bp3-minimal select.bp3-intent-danger.bp3-active,\n    .bp3-select.bp3-minimal select.bp3-intent-danger.bp3-active{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background:none;\n      color:#c23030; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-danger:hover,\n    .bp3-select.bp3-minimal select.bp3-intent-danger:hover{\n      background:rgba(219, 55, 55, 0.15);\n      color:#c23030; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-danger:active,\n    .bp3-select.bp3-minimal select.bp3-intent-danger:active, .bp3-html-select.bp3-minimal select.bp3-intent-danger.bp3-active,\n    .bp3-select.bp3-minimal select.bp3-intent-danger.bp3-active{\n      background:rgba(219, 55, 55, 0.3);\n      color:#c23030; }\n    .bp3-html-select.bp3-minimal select.bp3-intent-danger:disabled,\n    .bp3-select.bp3-minimal select.bp3-intent-danger:disabled, .bp3-html-select.bp3-minimal select.bp3-intent-danger.bp3-disabled,\n    .bp3-select.bp3-minimal select.bp3-intent-danger.bp3-disabled{\n      background:none;\n      color:rgba(194, 48, 48, 0.5); }\n      .bp3-html-select.bp3-minimal select.bp3-intent-danger:disabled.bp3-active,\n      .bp3-select.bp3-minimal select.bp3-intent-danger:disabled.bp3-active, .bp3-html-select.bp3-minimal select.bp3-intent-danger.bp3-disabled.bp3-active,\n      .bp3-select.bp3-minimal select.bp3-intent-danger.bp3-disabled.bp3-active{\n        background:rgba(219, 55, 55, 0.3); }\n    .bp3-html-select.bp3-minimal select.bp3-intent-danger .bp3-button-spinner .bp3-spinner-head, .bp3-select.bp3-minimal select.bp3-intent-danger .bp3-button-spinner .bp3-spinner-head{\n      stroke:#c23030; }\n    .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-danger, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-danger,\n    .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-danger, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-danger{\n      color:#ff7373; }\n      .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-danger:hover, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-danger:hover,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-danger:hover, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-danger:hover{\n        background:rgba(219, 55, 55, 0.2);\n        color:#ff7373; }\n      .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-danger:active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-danger:active,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-danger:active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-danger:active, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-danger.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-danger.bp3-active,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-danger.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-danger.bp3-active{\n        background:rgba(219, 55, 55, 0.3);\n        color:#ff7373; }\n      .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-danger:disabled, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-danger:disabled,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-danger:disabled, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-danger:disabled, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-danger.bp3-disabled, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-danger.bp3-disabled,\n      .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-danger.bp3-disabled, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-danger.bp3-disabled{\n        background:none;\n        color:rgba(255, 115, 115, 0.5); }\n        .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-danger:disabled.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-danger:disabled.bp3-active,\n        .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-danger:disabled.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-danger:disabled.bp3-active, .bp3-dark .bp3-html-select.bp3-minimal select.bp3-intent-danger.bp3-disabled.bp3-active, .bp3-html-select.bp3-minimal .bp3-dark select.bp3-intent-danger.bp3-disabled.bp3-active,\n        .bp3-dark .bp3-select.bp3-minimal select.bp3-intent-danger.bp3-disabled.bp3-active, .bp3-select.bp3-minimal .bp3-dark select.bp3-intent-danger.bp3-disabled.bp3-active{\n          background:rgba(219, 55, 55, 0.3); }\n\n.bp3-html-select.bp3-large select,\n.bp3-select.bp3-large select{\n  height:40px;\n  padding-right:35px;\n  font-size:16px; }\n\n.bp3-dark .bp3-html-select select, .bp3-dark .bp3-select select{\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n  background-color:#394b59;\n  background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.05)), to(rgba(255, 255, 255, 0)));\n  background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.05), rgba(255, 255, 255, 0));\n  color:#f5f8fa; }\n  .bp3-dark .bp3-html-select select:hover, .bp3-dark .bp3-select select:hover, .bp3-dark .bp3-html-select select:active, .bp3-dark .bp3-select select:active, .bp3-dark .bp3-html-select select.bp3-active, .bp3-dark .bp3-select select.bp3-active{\n    color:#f5f8fa; }\n  .bp3-dark .bp3-html-select select:hover, .bp3-dark .bp3-select select:hover{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n    background-color:#30404d; }\n  .bp3-dark .bp3-html-select select:active, .bp3-dark .bp3-select select:active, .bp3-dark .bp3-html-select select.bp3-active, .bp3-dark .bp3-select select.bp3-active{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.6), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.6), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n    background-color:#202b33;\n    background-image:none; }\n  .bp3-dark .bp3-html-select select:disabled, .bp3-dark .bp3-select select:disabled, .bp3-dark .bp3-html-select select.bp3-disabled, .bp3-dark .bp3-select select.bp3-disabled{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background-color:rgba(57, 75, 89, 0.5);\n    background-image:none;\n    color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark .bp3-html-select select:disabled.bp3-active, .bp3-dark .bp3-select select:disabled.bp3-active, .bp3-dark .bp3-html-select select.bp3-disabled.bp3-active, .bp3-dark .bp3-select select.bp3-disabled.bp3-active{\n      background:rgba(57, 75, 89, 0.7); }\n  .bp3-dark .bp3-html-select select .bp3-button-spinner .bp3-spinner-head, .bp3-dark .bp3-select select .bp3-button-spinner .bp3-spinner-head{\n    background:rgba(16, 22, 26, 0.5);\n    stroke:#8a9ba8; }\n\n.bp3-html-select select:disabled,\n.bp3-select select:disabled{\n  -webkit-box-shadow:none;\n          box-shadow:none;\n  background-color:rgba(206, 217, 224, 0.5);\n  cursor:not-allowed;\n  color:rgba(92, 112, 128, 0.6); }\n\n.bp3-html-select .bp3-icon,\n.bp3-select .bp3-icon, .bp3-select::after{\n  position:absolute;\n  top:7px;\n  right:7px;\n  color:#5c7080;\n  pointer-events:none; }\n  .bp3-html-select .bp3-disabled.bp3-icon,\n  .bp3-select .bp3-disabled.bp3-icon, .bp3-disabled.bp3-select::after{\n    color:rgba(92, 112, 128, 0.6); }\n.bp3-html-select,\n.bp3-select{\n  display:inline-block;\n  position:relative;\n  vertical-align:middle;\n  letter-spacing:normal; }\n  .bp3-html-select select::-ms-expand,\n  .bp3-select select::-ms-expand{\n    display:none; }\n  .bp3-html-select .bp3-icon,\n  .bp3-select .bp3-icon{\n    color:#5c7080; }\n    .bp3-html-select .bp3-icon:hover,\n    .bp3-select .bp3-icon:hover{\n      color:#182026; }\n    .bp3-dark .bp3-html-select .bp3-icon, .bp3-dark\n    .bp3-select .bp3-icon{\n      color:#a7b6c2; }\n      .bp3-dark .bp3-html-select .bp3-icon:hover, .bp3-dark\n      .bp3-select .bp3-icon:hover{\n        color:#f5f8fa; }\n  .bp3-html-select.bp3-large::after,\n  .bp3-html-select.bp3-large .bp3-icon,\n  .bp3-select.bp3-large::after,\n  .bp3-select.bp3-large .bp3-icon{\n    top:12px;\n    right:12px; }\n  .bp3-html-select.bp3-fill,\n  .bp3-html-select.bp3-fill select,\n  .bp3-select.bp3-fill,\n  .bp3-select.bp3-fill select{\n    width:100%; }\n  .bp3-dark .bp3-html-select option, .bp3-dark\n  .bp3-select option{\n    background-color:#30404d;\n    color:#f5f8fa; }\n  .bp3-dark .bp3-html-select::after, .bp3-dark\n  .bp3-select::after{\n    color:#a7b6c2; }\n\n.bp3-select::after{\n  line-height:1;\n  font-family:"Icons16", sans-serif;\n  font-size:16px;\n  font-weight:400;\n  font-style:normal;\n  -moz-osx-font-smoothing:grayscale;\n  -webkit-font-smoothing:antialiased;\n  content:"\\E6C6"; }\n.bp3-running-text table, table.bp3-html-table{\n  border-spacing:0;\n  font-size:14px; }\n  .bp3-running-text table th, table.bp3-html-table th,\n  .bp3-running-text table td,\n  table.bp3-html-table td{\n    padding:11px;\n    vertical-align:top;\n    text-align:left; }\n  .bp3-running-text table th, table.bp3-html-table th{\n    color:#182026;\n    font-weight:600; }\n  \n  .bp3-running-text table td,\n  table.bp3-html-table td{\n    color:#182026; }\n  .bp3-running-text table tbody tr:first-child th, table.bp3-html-table tbody tr:first-child th,\n  .bp3-running-text table tbody tr:first-child td,\n  table.bp3-html-table tbody tr:first-child td{\n    -webkit-box-shadow:inset 0 1px 0 0 rgba(16, 22, 26, 0.15);\n            box-shadow:inset 0 1px 0 0 rgba(16, 22, 26, 0.15); }\n  .bp3-dark .bp3-running-text table th, .bp3-running-text .bp3-dark table th, .bp3-dark table.bp3-html-table th{\n    color:#f5f8fa; }\n  .bp3-dark .bp3-running-text table td, .bp3-running-text .bp3-dark table td, .bp3-dark table.bp3-html-table td{\n    color:#f5f8fa; }\n  .bp3-dark .bp3-running-text table tbody tr:first-child th, .bp3-running-text .bp3-dark table tbody tr:first-child th, .bp3-dark table.bp3-html-table tbody tr:first-child th,\n  .bp3-dark .bp3-running-text table tbody tr:first-child td,\n  .bp3-running-text .bp3-dark table tbody tr:first-child td,\n  .bp3-dark table.bp3-html-table tbody tr:first-child td{\n    -webkit-box-shadow:inset 0 1px 0 0 rgba(255, 255, 255, 0.15);\n            box-shadow:inset 0 1px 0 0 rgba(255, 255, 255, 0.15); }\n\ntable.bp3-html-table.bp3-html-table-condensed th,\ntable.bp3-html-table.bp3-html-table-condensed td, table.bp3-html-table.bp3-small th,\ntable.bp3-html-table.bp3-small td{\n  padding-top:6px;\n  padding-bottom:6px; }\n\ntable.bp3-html-table.bp3-html-table-striped tbody tr:nth-child(odd) td{\n  background:rgba(191, 204, 214, 0.15); }\n\ntable.bp3-html-table.bp3-html-table-bordered th:not(:first-child){\n  -webkit-box-shadow:inset 1px 0 0 0 rgba(16, 22, 26, 0.15);\n          box-shadow:inset 1px 0 0 0 rgba(16, 22, 26, 0.15); }\n\ntable.bp3-html-table.bp3-html-table-bordered tbody tr td{\n  -webkit-box-shadow:inset 0 1px 0 0 rgba(16, 22, 26, 0.15);\n          box-shadow:inset 0 1px 0 0 rgba(16, 22, 26, 0.15); }\n  table.bp3-html-table.bp3-html-table-bordered tbody tr td:not(:first-child){\n    -webkit-box-shadow:inset 1px 1px 0 0 rgba(16, 22, 26, 0.15);\n            box-shadow:inset 1px 1px 0 0 rgba(16, 22, 26, 0.15); }\n\ntable.bp3-html-table.bp3-html-table-bordered.bp3-html-table-striped tbody tr:not(:first-child) td{\n  -webkit-box-shadow:none;\n          box-shadow:none; }\n  table.bp3-html-table.bp3-html-table-bordered.bp3-html-table-striped tbody tr:not(:first-child) td:not(:first-child){\n    -webkit-box-shadow:inset 1px 0 0 0 rgba(16, 22, 26, 0.15);\n            box-shadow:inset 1px 0 0 0 rgba(16, 22, 26, 0.15); }\n\ntable.bp3-html-table.bp3-interactive tbody tr:hover td{\n  background-color:rgba(191, 204, 214, 0.3);\n  cursor:pointer; }\n\ntable.bp3-html-table.bp3-interactive tbody tr:active td{\n  background-color:rgba(191, 204, 214, 0.4); }\n\n.bp3-dark table.bp3-html-table.bp3-html-table-striped tbody tr:nth-child(odd) td{\n  background:rgba(92, 112, 128, 0.15); }\n\n.bp3-dark table.bp3-html-table.bp3-html-table-bordered th:not(:first-child){\n  -webkit-box-shadow:inset 1px 0 0 0 rgba(255, 255, 255, 0.15);\n          box-shadow:inset 1px 0 0 0 rgba(255, 255, 255, 0.15); }\n\n.bp3-dark table.bp3-html-table.bp3-html-table-bordered tbody tr td{\n  -webkit-box-shadow:inset 0 1px 0 0 rgba(255, 255, 255, 0.15);\n          box-shadow:inset 0 1px 0 0 rgba(255, 255, 255, 0.15); }\n  .bp3-dark table.bp3-html-table.bp3-html-table-bordered tbody tr td:not(:first-child){\n    -webkit-box-shadow:inset 1px 1px 0 0 rgba(255, 255, 255, 0.15);\n            box-shadow:inset 1px 1px 0 0 rgba(255, 255, 255, 0.15); }\n\n.bp3-dark table.bp3-html-table.bp3-html-table-bordered.bp3-html-table-striped tbody tr:not(:first-child) td{\n  -webkit-box-shadow:inset 1px 0 0 0 rgba(255, 255, 255, 0.15);\n          box-shadow:inset 1px 0 0 0 rgba(255, 255, 255, 0.15); }\n  .bp3-dark table.bp3-html-table.bp3-html-table-bordered.bp3-html-table-striped tbody tr:not(:first-child) td:first-child{\n    -webkit-box-shadow:none;\n            box-shadow:none; }\n\n.bp3-dark table.bp3-html-table.bp3-interactive tbody tr:hover td{\n  background-color:rgba(92, 112, 128, 0.3);\n  cursor:pointer; }\n\n.bp3-dark table.bp3-html-table.bp3-interactive tbody tr:active td{\n  background-color:rgba(92, 112, 128, 0.4); }\n\n.bp3-key-combo{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-orient:horizontal;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:row;\n      -ms-flex-direction:row;\n          flex-direction:row;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center; }\n  .bp3-key-combo > *{\n    -webkit-box-flex:0;\n    -webkit-flex-grow:0;\n        -ms-flex-positive:0;\n            flex-grow:0;\n    -webkit-flex-shrink:0;\n        -ms-flex-negative:0;\n            flex-shrink:0; }\n  .bp3-key-combo > .bp3-fill{\n    -webkit-box-flex:1;\n    -webkit-flex-grow:1;\n        -ms-flex-positive:1;\n            flex-grow:1;\n    -webkit-flex-shrink:1;\n        -ms-flex-negative:1;\n            flex-shrink:1; }\n  .bp3-key-combo::before,\n  .bp3-key-combo > *{\n    margin-right:5px; }\n  .bp3-key-combo:empty::before,\n  .bp3-key-combo > :last-child{\n    margin-right:0; }\n\n.bp3-hotkey-dialog{\n  top:40px;\n  padding-bottom:0; }\n  .bp3-hotkey-dialog .bp3-dialog-body{\n    margin:0;\n    padding:0; }\n  .bp3-hotkey-dialog .bp3-hotkey-label{\n    -webkit-box-flex:1;\n    -webkit-flex-grow:1;\n        -ms-flex-positive:1;\n            flex-grow:1; }\n\n.bp3-hotkey-column{\n  margin:auto;\n  max-height:80vh;\n  overflow-y:auto;\n  padding:30px; }\n  .bp3-hotkey-column .bp3-heading{\n    margin-bottom:20px; }\n    .bp3-hotkey-column .bp3-heading:not(:first-child){\n      margin-top:40px; }\n\n.bp3-hotkey{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  -webkit-box-pack:justify;\n  -webkit-justify-content:space-between;\n      -ms-flex-pack:justify;\n          justify-content:space-between;\n  margin-right:0;\n  margin-left:0; }\n  .bp3-hotkey:not(:last-child){\n    margin-bottom:10px; }\n.bp3-icon{\n  display:inline-block;\n  -webkit-box-flex:0;\n  -webkit-flex:0 0 auto;\n      -ms-flex:0 0 auto;\n          flex:0 0 auto;\n  vertical-align:text-bottom; }\n  .bp3-icon:not(:empty)::before{\n    content:"" !important;\n    content:unset !important; }\n  .bp3-icon > svg{\n    display:block; }\n    .bp3-icon > svg:not([fill]){\n      fill:currentColor; }\n\n.bp3-icon.bp3-intent-primary, .bp3-icon-standard.bp3-intent-primary, .bp3-icon-large.bp3-intent-primary{\n  color:#106ba3; }\n  .bp3-dark .bp3-icon.bp3-intent-primary, .bp3-dark .bp3-icon-standard.bp3-intent-primary, .bp3-dark .bp3-icon-large.bp3-intent-primary{\n    color:#48aff0; }\n\n.bp3-icon.bp3-intent-success, .bp3-icon-standard.bp3-intent-success, .bp3-icon-large.bp3-intent-success{\n  color:#0d8050; }\n  .bp3-dark .bp3-icon.bp3-intent-success, .bp3-dark .bp3-icon-standard.bp3-intent-success, .bp3-dark .bp3-icon-large.bp3-intent-success{\n    color:#3dcc91; }\n\n.bp3-icon.bp3-intent-warning, .bp3-icon-standard.bp3-intent-warning, .bp3-icon-large.bp3-intent-warning{\n  color:#bf7326; }\n  .bp3-dark .bp3-icon.bp3-intent-warning, .bp3-dark .bp3-icon-standard.bp3-intent-warning, .bp3-dark .bp3-icon-large.bp3-intent-warning{\n    color:#ffb366; }\n\n.bp3-icon.bp3-intent-danger, .bp3-icon-standard.bp3-intent-danger, .bp3-icon-large.bp3-intent-danger{\n  color:#c23030; }\n  .bp3-dark .bp3-icon.bp3-intent-danger, .bp3-dark .bp3-icon-standard.bp3-intent-danger, .bp3-dark .bp3-icon-large.bp3-intent-danger{\n    color:#ff7373; }\n\nspan.bp3-icon-standard{\n  line-height:1;\n  font-family:"Icons16", sans-serif;\n  font-size:16px;\n  font-weight:400;\n  font-style:normal;\n  -moz-osx-font-smoothing:grayscale;\n  -webkit-font-smoothing:antialiased;\n  display:inline-block; }\n\nspan.bp3-icon-large{\n  line-height:1;\n  font-family:"Icons20", sans-serif;\n  font-size:20px;\n  font-weight:400;\n  font-style:normal;\n  -moz-osx-font-smoothing:grayscale;\n  -webkit-font-smoothing:antialiased;\n  display:inline-block; }\n\nspan.bp3-icon:empty{\n  line-height:1;\n  font-family:"Icons20";\n  font-size:inherit;\n  font-weight:400;\n  font-style:normal; }\n  span.bp3-icon:empty::before{\n    -moz-osx-font-smoothing:grayscale;\n    -webkit-font-smoothing:antialiased; }\n\n.bp3-icon-add::before{\n  content:"\\E63E"; }\n\n.bp3-icon-add-column-left::before{\n  content:"\\E6F9"; }\n\n.bp3-icon-add-column-right::before{\n  content:"\\E6FA"; }\n\n.bp3-icon-add-row-bottom::before{\n  content:"\\E6F8"; }\n\n.bp3-icon-add-row-top::before{\n  content:"\\E6F7"; }\n\n.bp3-icon-add-to-artifact::before{\n  content:"\\E67C"; }\n\n.bp3-icon-add-to-folder::before{\n  content:"\\E6D2"; }\n\n.bp3-icon-airplane::before{\n  content:"\\E74B"; }\n\n.bp3-icon-align-center::before{\n  content:"\\E603"; }\n\n.bp3-icon-align-justify::before{\n  content:"\\E605"; }\n\n.bp3-icon-align-left::before{\n  content:"\\E602"; }\n\n.bp3-icon-align-right::before{\n  content:"\\E604"; }\n\n.bp3-icon-alignment-bottom::before{\n  content:"\\E727"; }\n\n.bp3-icon-alignment-horizontal-center::before{\n  content:"\\E726"; }\n\n.bp3-icon-alignment-left::before{\n  content:"\\E722"; }\n\n.bp3-icon-alignment-right::before{\n  content:"\\E724"; }\n\n.bp3-icon-alignment-top::before{\n  content:"\\E725"; }\n\n.bp3-icon-alignment-vertical-center::before{\n  content:"\\E723"; }\n\n.bp3-icon-annotation::before{\n  content:"\\E6F0"; }\n\n.bp3-icon-application::before{\n  content:"\\E735"; }\n\n.bp3-icon-applications::before{\n  content:"\\E621"; }\n\n.bp3-icon-arrow-bottom-left::before{\n  content:"\\2199"; }\n\n.bp3-icon-arrow-bottom-right::before{\n  content:"\\2198"; }\n\n.bp3-icon-arrow-down::before{\n  content:"\\2193"; }\n\n.bp3-icon-arrow-left::before{\n  content:"\\2190"; }\n\n.bp3-icon-arrow-right::before{\n  content:"\\2192"; }\n\n.bp3-icon-arrow-top-left::before{\n  content:"\\2196"; }\n\n.bp3-icon-arrow-top-right::before{\n  content:"\\2197"; }\n\n.bp3-icon-arrow-up::before{\n  content:"\\2191"; }\n\n.bp3-icon-arrows-horizontal::before{\n  content:"\\2194"; }\n\n.bp3-icon-arrows-vertical::before{\n  content:"\\2195"; }\n\n.bp3-icon-asterisk::before{\n  content:"*"; }\n\n.bp3-icon-automatic-updates::before{\n  content:"\\E65F"; }\n\n.bp3-icon-badge::before{\n  content:"\\E6E3"; }\n\n.bp3-icon-ban-circle::before{\n  content:"\\E69D"; }\n\n.bp3-icon-bank-account::before{\n  content:"\\E76F"; }\n\n.bp3-icon-barcode::before{\n  content:"\\E676"; }\n\n.bp3-icon-blank::before{\n  content:"\\E900"; }\n\n.bp3-icon-blocked-person::before{\n  content:"\\E768"; }\n\n.bp3-icon-bold::before{\n  content:"\\E606"; }\n\n.bp3-icon-book::before{\n  content:"\\E6B8"; }\n\n.bp3-icon-bookmark::before{\n  content:"\\E61A"; }\n\n.bp3-icon-box::before{\n  content:"\\E6BF"; }\n\n.bp3-icon-briefcase::before{\n  content:"\\E674"; }\n\n.bp3-icon-build::before{\n  content:"\\E72D"; }\n\n.bp3-icon-calculator::before{\n  content:"\\E70B"; }\n\n.bp3-icon-calendar::before{\n  content:"\\E62B"; }\n\n.bp3-icon-camera::before{\n  content:"\\E69E"; }\n\n.bp3-icon-caret-down::before{\n  content:"\\2304"; }\n\n.bp3-icon-caret-left::before{\n  content:"\\2329"; }\n\n.bp3-icon-caret-right::before{\n  content:"\\232A"; }\n\n.bp3-icon-caret-up::before{\n  content:"\\2303"; }\n\n.bp3-icon-cell-tower::before{\n  content:"\\E770"; }\n\n.bp3-icon-changes::before{\n  content:"\\E623"; }\n\n.bp3-icon-chart::before{\n  content:"\\E67E"; }\n\n.bp3-icon-chat::before{\n  content:"\\E689"; }\n\n.bp3-icon-chevron-backward::before{\n  content:"\\E6DF"; }\n\n.bp3-icon-chevron-down::before{\n  content:"\\E697"; }\n\n.bp3-icon-chevron-forward::before{\n  content:"\\E6E0"; }\n\n.bp3-icon-chevron-left::before{\n  content:"\\E694"; }\n\n.bp3-icon-chevron-right::before{\n  content:"\\E695"; }\n\n.bp3-icon-chevron-up::before{\n  content:"\\E696"; }\n\n.bp3-icon-circle::before{\n  content:"\\E66A"; }\n\n.bp3-icon-circle-arrow-down::before{\n  content:"\\E68E"; }\n\n.bp3-icon-circle-arrow-left::before{\n  content:"\\E68C"; }\n\n.bp3-icon-circle-arrow-right::before{\n  content:"\\E68B"; }\n\n.bp3-icon-circle-arrow-up::before{\n  content:"\\E68D"; }\n\n.bp3-icon-citation::before{\n  content:"\\E61B"; }\n\n.bp3-icon-clean::before{\n  content:"\\E7C5"; }\n\n.bp3-icon-clipboard::before{\n  content:"\\E61D"; }\n\n.bp3-icon-cloud::before{\n  content:"\\2601"; }\n\n.bp3-icon-cloud-download::before{\n  content:"\\E690"; }\n\n.bp3-icon-cloud-upload::before{\n  content:"\\E691"; }\n\n.bp3-icon-code::before{\n  content:"\\E661"; }\n\n.bp3-icon-code-block::before{\n  content:"\\E6C5"; }\n\n.bp3-icon-cog::before{\n  content:"\\E645"; }\n\n.bp3-icon-collapse-all::before{\n  content:"\\E763"; }\n\n.bp3-icon-column-layout::before{\n  content:"\\E6DA"; }\n\n.bp3-icon-comment::before{\n  content:"\\E68A"; }\n\n.bp3-icon-comparison::before{\n  content:"\\E637"; }\n\n.bp3-icon-compass::before{\n  content:"\\E79C"; }\n\n.bp3-icon-compressed::before{\n  content:"\\E6C0"; }\n\n.bp3-icon-confirm::before{\n  content:"\\E639"; }\n\n.bp3-icon-console::before{\n  content:"\\E79B"; }\n\n.bp3-icon-contrast::before{\n  content:"\\E6CB"; }\n\n.bp3-icon-control::before{\n  content:"\\E67F"; }\n\n.bp3-icon-credit-card::before{\n  content:"\\E649"; }\n\n.bp3-icon-cross::before{\n  content:"\\2717"; }\n\n.bp3-icon-crown::before{\n  content:"\\E7B4"; }\n\n.bp3-icon-cube::before{\n  content:"\\E7C8"; }\n\n.bp3-icon-cube-add::before{\n  content:"\\E7C9"; }\n\n.bp3-icon-cube-remove::before{\n  content:"\\E7D0"; }\n\n.bp3-icon-curved-range-chart::before{\n  content:"\\E71B"; }\n\n.bp3-icon-cut::before{\n  content:"\\E6EF"; }\n\n.bp3-icon-dashboard::before{\n  content:"\\E751"; }\n\n.bp3-icon-database::before{\n  content:"\\E683"; }\n\n.bp3-icon-delete::before{\n  content:"\\E644"; }\n\n.bp3-icon-delta::before{\n  content:"\\394"; }\n\n.bp3-icon-derive-column::before{\n  content:"\\E739"; }\n\n.bp3-icon-desktop::before{\n  content:"\\E6AF"; }\n\n.bp3-icon-diagram-tree::before{\n  content:"\\E7B3"; }\n\n.bp3-icon-direction-left::before{\n  content:"\\E681"; }\n\n.bp3-icon-direction-right::before{\n  content:"\\E682"; }\n\n.bp3-icon-disable::before{\n  content:"\\E600"; }\n\n.bp3-icon-document::before{\n  content:"\\E630"; }\n\n.bp3-icon-document-open::before{\n  content:"\\E71E"; }\n\n.bp3-icon-document-share::before{\n  content:"\\E71F"; }\n\n.bp3-icon-dollar::before{\n  content:"$"; }\n\n.bp3-icon-dot::before{\n  content:"\\2022"; }\n\n.bp3-icon-double-caret-horizontal::before{\n  content:"\\E6C7"; }\n\n.bp3-icon-double-caret-vertical::before{\n  content:"\\E6C6"; }\n\n.bp3-icon-double-chevron-down::before{\n  content:"\\E703"; }\n\n.bp3-icon-double-chevron-left::before{\n  content:"\\E6FF"; }\n\n.bp3-icon-double-chevron-right::before{\n  content:"\\E701"; }\n\n.bp3-icon-double-chevron-up::before{\n  content:"\\E702"; }\n\n.bp3-icon-doughnut-chart::before{\n  content:"\\E6CE"; }\n\n.bp3-icon-download::before{\n  content:"\\E62F"; }\n\n.bp3-icon-drag-handle-horizontal::before{\n  content:"\\E716"; }\n\n.bp3-icon-drag-handle-vertical::before{\n  content:"\\E715"; }\n\n.bp3-icon-draw::before{\n  content:"\\E66B"; }\n\n.bp3-icon-drive-time::before{\n  content:"\\E615"; }\n\n.bp3-icon-duplicate::before{\n  content:"\\E69C"; }\n\n.bp3-icon-edit::before{\n  content:"\\270E"; }\n\n.bp3-icon-eject::before{\n  content:"\\23CF"; }\n\n.bp3-icon-endorsed::before{\n  content:"\\E75F"; }\n\n.bp3-icon-envelope::before{\n  content:"\\2709"; }\n\n.bp3-icon-equals::before{\n  content:"\\E7D9"; }\n\n.bp3-icon-eraser::before{\n  content:"\\E773"; }\n\n.bp3-icon-error::before{\n  content:"\\E648"; }\n\n.bp3-icon-euro::before{\n  content:"\\20AC"; }\n\n.bp3-icon-exchange::before{\n  content:"\\E636"; }\n\n.bp3-icon-exclude-row::before{\n  content:"\\E6EA"; }\n\n.bp3-icon-expand-all::before{\n  content:"\\E764"; }\n\n.bp3-icon-export::before{\n  content:"\\E633"; }\n\n.bp3-icon-eye-off::before{\n  content:"\\E6CC"; }\n\n.bp3-icon-eye-on::before{\n  content:"\\E75A"; }\n\n.bp3-icon-eye-open::before{\n  content:"\\E66F"; }\n\n.bp3-icon-fast-backward::before{\n  content:"\\E6A8"; }\n\n.bp3-icon-fast-forward::before{\n  content:"\\E6AC"; }\n\n.bp3-icon-feed::before{\n  content:"\\E656"; }\n\n.bp3-icon-feed-subscribed::before{\n  content:"\\E78F"; }\n\n.bp3-icon-film::before{\n  content:"\\E6A1"; }\n\n.bp3-icon-filter::before{\n  content:"\\E638"; }\n\n.bp3-icon-filter-keep::before{\n  content:"\\E78C"; }\n\n.bp3-icon-filter-list::before{\n  content:"\\E6EE"; }\n\n.bp3-icon-filter-open::before{\n  content:"\\E7D7"; }\n\n.bp3-icon-filter-remove::before{\n  content:"\\E78D"; }\n\n.bp3-icon-flag::before{\n  content:"\\2691"; }\n\n.bp3-icon-flame::before{\n  content:"\\E7A9"; }\n\n.bp3-icon-flash::before{\n  content:"\\E6B3"; }\n\n.bp3-icon-floppy-disk::before{\n  content:"\\E6B7"; }\n\n.bp3-icon-flow-branch::before{\n  content:"\\E7C1"; }\n\n.bp3-icon-flow-end::before{\n  content:"\\E7C4"; }\n\n.bp3-icon-flow-linear::before{\n  content:"\\E7C0"; }\n\n.bp3-icon-flow-review::before{\n  content:"\\E7C2"; }\n\n.bp3-icon-flow-review-branch::before{\n  content:"\\E7C3"; }\n\n.bp3-icon-flows::before{\n  content:"\\E659"; }\n\n.bp3-icon-folder-close::before{\n  content:"\\E652"; }\n\n.bp3-icon-folder-new::before{\n  content:"\\E7B0"; }\n\n.bp3-icon-folder-open::before{\n  content:"\\E651"; }\n\n.bp3-icon-folder-shared::before{\n  content:"\\E653"; }\n\n.bp3-icon-folder-shared-open::before{\n  content:"\\E670"; }\n\n.bp3-icon-follower::before{\n  content:"\\E760"; }\n\n.bp3-icon-following::before{\n  content:"\\E761"; }\n\n.bp3-icon-font::before{\n  content:"\\E6B4"; }\n\n.bp3-icon-fork::before{\n  content:"\\E63A"; }\n\n.bp3-icon-form::before{\n  content:"\\E795"; }\n\n.bp3-icon-full-circle::before{\n  content:"\\E685"; }\n\n.bp3-icon-full-stacked-chart::before{\n  content:"\\E75E"; }\n\n.bp3-icon-fullscreen::before{\n  content:"\\E699"; }\n\n.bp3-icon-function::before{\n  content:"\\E6E5"; }\n\n.bp3-icon-gantt-chart::before{\n  content:"\\E6F4"; }\n\n.bp3-icon-geolocation::before{\n  content:"\\E640"; }\n\n.bp3-icon-geosearch::before{\n  content:"\\E613"; }\n\n.bp3-icon-git-branch::before{\n  content:"\\E72A"; }\n\n.bp3-icon-git-commit::before{\n  content:"\\E72B"; }\n\n.bp3-icon-git-merge::before{\n  content:"\\E729"; }\n\n.bp3-icon-git-new-branch::before{\n  content:"\\E749"; }\n\n.bp3-icon-git-pull::before{\n  content:"\\E728"; }\n\n.bp3-icon-git-push::before{\n  content:"\\E72C"; }\n\n.bp3-icon-git-repo::before{\n  content:"\\E748"; }\n\n.bp3-icon-glass::before{\n  content:"\\E6B1"; }\n\n.bp3-icon-globe::before{\n  content:"\\E666"; }\n\n.bp3-icon-globe-network::before{\n  content:"\\E7B5"; }\n\n.bp3-icon-graph::before{\n  content:"\\E673"; }\n\n.bp3-icon-graph-remove::before{\n  content:"\\E609"; }\n\n.bp3-icon-greater-than::before{\n  content:"\\E7E1"; }\n\n.bp3-icon-greater-than-or-equal-to::before{\n  content:"\\E7E2"; }\n\n.bp3-icon-grid::before{\n  content:"\\E6D0"; }\n\n.bp3-icon-grid-view::before{\n  content:"\\E6E4"; }\n\n.bp3-icon-group-objects::before{\n  content:"\\E60A"; }\n\n.bp3-icon-grouped-bar-chart::before{\n  content:"\\E75D"; }\n\n.bp3-icon-hand::before{\n  content:"\\E6DE"; }\n\n.bp3-icon-hand-down::before{\n  content:"\\E6BB"; }\n\n.bp3-icon-hand-left::before{\n  content:"\\E6BC"; }\n\n.bp3-icon-hand-right::before{\n  content:"\\E6B9"; }\n\n.bp3-icon-hand-up::before{\n  content:"\\E6BA"; }\n\n.bp3-icon-header::before{\n  content:"\\E6B5"; }\n\n.bp3-icon-header-one::before{\n  content:"\\E793"; }\n\n.bp3-icon-header-two::before{\n  content:"\\E794"; }\n\n.bp3-icon-headset::before{\n  content:"\\E6DC"; }\n\n.bp3-icon-heart::before{\n  content:"\\2665"; }\n\n.bp3-icon-heart-broken::before{\n  content:"\\E7A2"; }\n\n.bp3-icon-heat-grid::before{\n  content:"\\E6F3"; }\n\n.bp3-icon-heatmap::before{\n  content:"\\E614"; }\n\n.bp3-icon-help::before{\n  content:"?"; }\n\n.bp3-icon-helper-management::before{\n  content:"\\E66D"; }\n\n.bp3-icon-highlight::before{\n  content:"\\E6ED"; }\n\n.bp3-icon-history::before{\n  content:"\\E64A"; }\n\n.bp3-icon-home::before{\n  content:"\\2302"; }\n\n.bp3-icon-horizontal-bar-chart::before{\n  content:"\\E70C"; }\n\n.bp3-icon-horizontal-bar-chart-asc::before{\n  content:"\\E75C"; }\n\n.bp3-icon-horizontal-bar-chart-desc::before{\n  content:"\\E71D"; }\n\n.bp3-icon-horizontal-distribution::before{\n  content:"\\E720"; }\n\n.bp3-icon-id-number::before{\n  content:"\\E771"; }\n\n.bp3-icon-image-rotate-left::before{\n  content:"\\E73A"; }\n\n.bp3-icon-image-rotate-right::before{\n  content:"\\E73B"; }\n\n.bp3-icon-import::before{\n  content:"\\E632"; }\n\n.bp3-icon-inbox::before{\n  content:"\\E629"; }\n\n.bp3-icon-inbox-filtered::before{\n  content:"\\E7D1"; }\n\n.bp3-icon-inbox-geo::before{\n  content:"\\E7D2"; }\n\n.bp3-icon-inbox-search::before{\n  content:"\\E7D3"; }\n\n.bp3-icon-inbox-update::before{\n  content:"\\E7D4"; }\n\n.bp3-icon-info-sign::before{\n  content:"\\2139"; }\n\n.bp3-icon-inheritance::before{\n  content:"\\E7D5"; }\n\n.bp3-icon-inner-join::before{\n  content:"\\E7A3"; }\n\n.bp3-icon-insert::before{\n  content:"\\E66C"; }\n\n.bp3-icon-intersection::before{\n  content:"\\E765"; }\n\n.bp3-icon-ip-address::before{\n  content:"\\E772"; }\n\n.bp3-icon-issue::before{\n  content:"\\E774"; }\n\n.bp3-icon-issue-closed::before{\n  content:"\\E776"; }\n\n.bp3-icon-issue-new::before{\n  content:"\\E775"; }\n\n.bp3-icon-italic::before{\n  content:"\\E607"; }\n\n.bp3-icon-join-table::before{\n  content:"\\E738"; }\n\n.bp3-icon-key::before{\n  content:"\\E78E"; }\n\n.bp3-icon-key-backspace::before{\n  content:"\\E707"; }\n\n.bp3-icon-key-command::before{\n  content:"\\E705"; }\n\n.bp3-icon-key-control::before{\n  content:"\\E704"; }\n\n.bp3-icon-key-delete::before{\n  content:"\\E708"; }\n\n.bp3-icon-key-enter::before{\n  content:"\\E70A"; }\n\n.bp3-icon-key-escape::before{\n  content:"\\E709"; }\n\n.bp3-icon-key-option::before{\n  content:"\\E742"; }\n\n.bp3-icon-key-shift::before{\n  content:"\\E706"; }\n\n.bp3-icon-key-tab::before{\n  content:"\\E757"; }\n\n.bp3-icon-known-vehicle::before{\n  content:"\\E73C"; }\n\n.bp3-icon-label::before{\n  content:"\\E665"; }\n\n.bp3-icon-layer::before{\n  content:"\\E6CF"; }\n\n.bp3-icon-layers::before{\n  content:"\\E618"; }\n\n.bp3-icon-layout::before{\n  content:"\\E60C"; }\n\n.bp3-icon-layout-auto::before{\n  content:"\\E60D"; }\n\n.bp3-icon-layout-balloon::before{\n  content:"\\E6D3"; }\n\n.bp3-icon-layout-circle::before{\n  content:"\\E60E"; }\n\n.bp3-icon-layout-grid::before{\n  content:"\\E610"; }\n\n.bp3-icon-layout-group-by::before{\n  content:"\\E611"; }\n\n.bp3-icon-layout-hierarchy::before{\n  content:"\\E60F"; }\n\n.bp3-icon-layout-linear::before{\n  content:"\\E6C3"; }\n\n.bp3-icon-layout-skew-grid::before{\n  content:"\\E612"; }\n\n.bp3-icon-layout-sorted-clusters::before{\n  content:"\\E6D4"; }\n\n.bp3-icon-left-join::before{\n  content:"\\E7A4"; }\n\n.bp3-icon-less-than::before{\n  content:"\\E7E3"; }\n\n.bp3-icon-less-than-or-equal-to::before{\n  content:"\\E7E4"; }\n\n.bp3-icon-lifesaver::before{\n  content:"\\E7C7"; }\n\n.bp3-icon-lightbulb::before{\n  content:"\\E6B0"; }\n\n.bp3-icon-link::before{\n  content:"\\E62D"; }\n\n.bp3-icon-list::before{\n  content:"\\2630"; }\n\n.bp3-icon-list-columns::before{\n  content:"\\E7B9"; }\n\n.bp3-icon-list-detail-view::before{\n  content:"\\E743"; }\n\n.bp3-icon-locate::before{\n  content:"\\E619"; }\n\n.bp3-icon-lock::before{\n  content:"\\E625"; }\n\n.bp3-icon-log-in::before{\n  content:"\\E69A"; }\n\n.bp3-icon-log-out::before{\n  content:"\\E64C"; }\n\n.bp3-icon-manual::before{\n  content:"\\E6F6"; }\n\n.bp3-icon-manually-entered-data::before{\n  content:"\\E74A"; }\n\n.bp3-icon-map::before{\n  content:"\\E662"; }\n\n.bp3-icon-map-create::before{\n  content:"\\E741"; }\n\n.bp3-icon-map-marker::before{\n  content:"\\E67D"; }\n\n.bp3-icon-maximize::before{\n  content:"\\E635"; }\n\n.bp3-icon-media::before{\n  content:"\\E62C"; }\n\n.bp3-icon-menu::before{\n  content:"\\E762"; }\n\n.bp3-icon-menu-closed::before{\n  content:"\\E655"; }\n\n.bp3-icon-menu-open::before{\n  content:"\\E654"; }\n\n.bp3-icon-merge-columns::before{\n  content:"\\E74F"; }\n\n.bp3-icon-merge-links::before{\n  content:"\\E60B"; }\n\n.bp3-icon-minimize::before{\n  content:"\\E634"; }\n\n.bp3-icon-minus::before{\n  content:"\\2212"; }\n\n.bp3-icon-mobile-phone::before{\n  content:"\\E717"; }\n\n.bp3-icon-mobile-video::before{\n  content:"\\E69F"; }\n\n.bp3-icon-moon::before{\n  content:"\\E754"; }\n\n.bp3-icon-more::before{\n  content:"\\E62A"; }\n\n.bp3-icon-mountain::before{\n  content:"\\E7B1"; }\n\n.bp3-icon-move::before{\n  content:"\\E693"; }\n\n.bp3-icon-mugshot::before{\n  content:"\\E6DB"; }\n\n.bp3-icon-multi-select::before{\n  content:"\\E680"; }\n\n.bp3-icon-music::before{\n  content:"\\E6A6"; }\n\n.bp3-icon-new-grid-item::before{\n  content:"\\E747"; }\n\n.bp3-icon-new-link::before{\n  content:"\\E65C"; }\n\n.bp3-icon-new-object::before{\n  content:"\\E65D"; }\n\n.bp3-icon-new-person::before{\n  content:"\\E6E9"; }\n\n.bp3-icon-new-prescription::before{\n  content:"\\E78B"; }\n\n.bp3-icon-new-text-box::before{\n  content:"\\E65B"; }\n\n.bp3-icon-ninja::before{\n  content:"\\E675"; }\n\n.bp3-icon-not-equal-to::before{\n  content:"\\E7E0"; }\n\n.bp3-icon-notifications::before{\n  content:"\\E624"; }\n\n.bp3-icon-notifications-updated::before{\n  content:"\\E7B8"; }\n\n.bp3-icon-numbered-list::before{\n  content:"\\E746"; }\n\n.bp3-icon-numerical::before{\n  content:"\\E756"; }\n\n.bp3-icon-office::before{\n  content:"\\E69B"; }\n\n.bp3-icon-offline::before{\n  content:"\\E67A"; }\n\n.bp3-icon-oil-field::before{\n  content:"\\E73F"; }\n\n.bp3-icon-one-column::before{\n  content:"\\E658"; }\n\n.bp3-icon-outdated::before{\n  content:"\\E7A8"; }\n\n.bp3-icon-page-layout::before{\n  content:"\\E660"; }\n\n.bp3-icon-panel-stats::before{\n  content:"\\E777"; }\n\n.bp3-icon-panel-table::before{\n  content:"\\E778"; }\n\n.bp3-icon-paperclip::before{\n  content:"\\E664"; }\n\n.bp3-icon-paragraph::before{\n  content:"\\E76C"; }\n\n.bp3-icon-path::before{\n  content:"\\E753"; }\n\n.bp3-icon-path-search::before{\n  content:"\\E65E"; }\n\n.bp3-icon-pause::before{\n  content:"\\E6A9"; }\n\n.bp3-icon-people::before{\n  content:"\\E63D"; }\n\n.bp3-icon-percentage::before{\n  content:"\\E76A"; }\n\n.bp3-icon-person::before{\n  content:"\\E63C"; }\n\n.bp3-icon-phone::before{\n  content:"\\260E"; }\n\n.bp3-icon-pie-chart::before{\n  content:"\\E684"; }\n\n.bp3-icon-pin::before{\n  content:"\\E646"; }\n\n.bp3-icon-pivot::before{\n  content:"\\E6F1"; }\n\n.bp3-icon-pivot-table::before{\n  content:"\\E6EB"; }\n\n.bp3-icon-play::before{\n  content:"\\E6AB"; }\n\n.bp3-icon-plus::before{\n  content:"+"; }\n\n.bp3-icon-polygon-filter::before{\n  content:"\\E6D1"; }\n\n.bp3-icon-power::before{\n  content:"\\E6D9"; }\n\n.bp3-icon-predictive-analysis::before{\n  content:"\\E617"; }\n\n.bp3-icon-prescription::before{\n  content:"\\E78A"; }\n\n.bp3-icon-presentation::before{\n  content:"\\E687"; }\n\n.bp3-icon-print::before{\n  content:"\\2399"; }\n\n.bp3-icon-projects::before{\n  content:"\\E622"; }\n\n.bp3-icon-properties::before{\n  content:"\\E631"; }\n\n.bp3-icon-property::before{\n  content:"\\E65A"; }\n\n.bp3-icon-publish-function::before{\n  content:"\\E752"; }\n\n.bp3-icon-pulse::before{\n  content:"\\E6E8"; }\n\n.bp3-icon-random::before{\n  content:"\\E698"; }\n\n.bp3-icon-record::before{\n  content:"\\E6AE"; }\n\n.bp3-icon-redo::before{\n  content:"\\E6C4"; }\n\n.bp3-icon-refresh::before{\n  content:"\\E643"; }\n\n.bp3-icon-regression-chart::before{\n  content:"\\E758"; }\n\n.bp3-icon-remove::before{\n  content:"\\E63F"; }\n\n.bp3-icon-remove-column::before{\n  content:"\\E755"; }\n\n.bp3-icon-remove-column-left::before{\n  content:"\\E6FD"; }\n\n.bp3-icon-remove-column-right::before{\n  content:"\\E6FE"; }\n\n.bp3-icon-remove-row-bottom::before{\n  content:"\\E6FC"; }\n\n.bp3-icon-remove-row-top::before{\n  content:"\\E6FB"; }\n\n.bp3-icon-repeat::before{\n  content:"\\E692"; }\n\n.bp3-icon-reset::before{\n  content:"\\E7D6"; }\n\n.bp3-icon-resolve::before{\n  content:"\\E672"; }\n\n.bp3-icon-rig::before{\n  content:"\\E740"; }\n\n.bp3-icon-right-join::before{\n  content:"\\E7A5"; }\n\n.bp3-icon-ring::before{\n  content:"\\E6F2"; }\n\n.bp3-icon-rotate-document::before{\n  content:"\\E6E1"; }\n\n.bp3-icon-rotate-page::before{\n  content:"\\E6E2"; }\n\n.bp3-icon-satellite::before{\n  content:"\\E76B"; }\n\n.bp3-icon-saved::before{\n  content:"\\E6B6"; }\n\n.bp3-icon-scatter-plot::before{\n  content:"\\E73E"; }\n\n.bp3-icon-search::before{\n  content:"\\E64B"; }\n\n.bp3-icon-search-around::before{\n  content:"\\E608"; }\n\n.bp3-icon-search-template::before{\n  content:"\\E628"; }\n\n.bp3-icon-search-text::before{\n  content:"\\E663"; }\n\n.bp3-icon-segmented-control::before{\n  content:"\\E6EC"; }\n\n.bp3-icon-select::before{\n  content:"\\E616"; }\n\n.bp3-icon-selection::before{\n  content:"\\29BF"; }\n\n.bp3-icon-send-to::before{\n  content:"\\E66E"; }\n\n.bp3-icon-send-to-graph::before{\n  content:"\\E736"; }\n\n.bp3-icon-send-to-map::before{\n  content:"\\E737"; }\n\n.bp3-icon-series-add::before{\n  content:"\\E796"; }\n\n.bp3-icon-series-configuration::before{\n  content:"\\E79A"; }\n\n.bp3-icon-series-derived::before{\n  content:"\\E799"; }\n\n.bp3-icon-series-filtered::before{\n  content:"\\E798"; }\n\n.bp3-icon-series-search::before{\n  content:"\\E797"; }\n\n.bp3-icon-settings::before{\n  content:"\\E6A2"; }\n\n.bp3-icon-share::before{\n  content:"\\E62E"; }\n\n.bp3-icon-shield::before{\n  content:"\\E7B2"; }\n\n.bp3-icon-shop::before{\n  content:"\\E6C2"; }\n\n.bp3-icon-shopping-cart::before{\n  content:"\\E6C1"; }\n\n.bp3-icon-sim-card::before{\n  content:"\\E718"; }\n\n.bp3-icon-slash::before{\n  content:"\\E769"; }\n\n.bp3-icon-small-cross::before{\n  content:"\\E6D7"; }\n\n.bp3-icon-small-minus::before{\n  content:"\\E70E"; }\n\n.bp3-icon-small-plus::before{\n  content:"\\E70D"; }\n\n.bp3-icon-small-tick::before{\n  content:"\\E6D8"; }\n\n.bp3-icon-snowflake::before{\n  content:"\\E7B6"; }\n\n.bp3-icon-social-media::before{\n  content:"\\E671"; }\n\n.bp3-icon-sort::before{\n  content:"\\E64F"; }\n\n.bp3-icon-sort-alphabetical::before{\n  content:"\\E64D"; }\n\n.bp3-icon-sort-alphabetical-desc::before{\n  content:"\\E6C8"; }\n\n.bp3-icon-sort-asc::before{\n  content:"\\E6D5"; }\n\n.bp3-icon-sort-desc::before{\n  content:"\\E6D6"; }\n\n.bp3-icon-sort-numerical::before{\n  content:"\\E64E"; }\n\n.bp3-icon-sort-numerical-desc::before{\n  content:"\\E6C9"; }\n\n.bp3-icon-split-columns::before{\n  content:"\\E750"; }\n\n.bp3-icon-square::before{\n  content:"\\E686"; }\n\n.bp3-icon-stacked-chart::before{\n  content:"\\E6E7"; }\n\n.bp3-icon-star::before{\n  content:"\\2605"; }\n\n.bp3-icon-star-empty::before{\n  content:"\\2606"; }\n\n.bp3-icon-step-backward::before{\n  content:"\\E6A7"; }\n\n.bp3-icon-step-chart::before{\n  content:"\\E70F"; }\n\n.bp3-icon-step-forward::before{\n  content:"\\E6AD"; }\n\n.bp3-icon-stop::before{\n  content:"\\E6AA"; }\n\n.bp3-icon-strikethrough::before{\n  content:"\\E7A6"; }\n\n.bp3-icon-style::before{\n  content:"\\E601"; }\n\n.bp3-icon-swap-horizontal::before{\n  content:"\\E745"; }\n\n.bp3-icon-swap-vertical::before{\n  content:"\\E744"; }\n\n.bp3-icon-symbol-circle::before{\n  content:"\\E72E"; }\n\n.bp3-icon-symbol-cross::before{\n  content:"\\E731"; }\n\n.bp3-icon-symbol-diamond::before{\n  content:"\\E730"; }\n\n.bp3-icon-symbol-square::before{\n  content:"\\E72F"; }\n\n.bp3-icon-symbol-triangle-down::before{\n  content:"\\E733"; }\n\n.bp3-icon-symbol-triangle-up::before{\n  content:"\\E732"; }\n\n.bp3-icon-tag::before{\n  content:"\\E61C"; }\n\n.bp3-icon-take-action::before{\n  content:"\\E6CA"; }\n\n.bp3-icon-taxi::before{\n  content:"\\E79E"; }\n\n.bp3-icon-text-highlight::before{\n  content:"\\E6DD"; }\n\n.bp3-icon-th::before{\n  content:"\\E667"; }\n\n.bp3-icon-th-derived::before{\n  content:"\\E669"; }\n\n.bp3-icon-th-disconnect::before{\n  content:"\\E7D8"; }\n\n.bp3-icon-th-filtered::before{\n  content:"\\E7C6"; }\n\n.bp3-icon-th-list::before{\n  content:"\\E668"; }\n\n.bp3-icon-thumbs-down::before{\n  content:"\\E6BE"; }\n\n.bp3-icon-thumbs-up::before{\n  content:"\\E6BD"; }\n\n.bp3-icon-tick::before{\n  content:"\\2713"; }\n\n.bp3-icon-tick-circle::before{\n  content:"\\E779"; }\n\n.bp3-icon-time::before{\n  content:"\\23F2"; }\n\n.bp3-icon-timeline-area-chart::before{\n  content:"\\E6CD"; }\n\n.bp3-icon-timeline-bar-chart::before{\n  content:"\\E620"; }\n\n.bp3-icon-timeline-events::before{\n  content:"\\E61E"; }\n\n.bp3-icon-timeline-line-chart::before{\n  content:"\\E61F"; }\n\n.bp3-icon-tint::before{\n  content:"\\E6B2"; }\n\n.bp3-icon-torch::before{\n  content:"\\E677"; }\n\n.bp3-icon-train::before{\n  content:"\\E79F"; }\n\n.bp3-icon-translate::before{\n  content:"\\E759"; }\n\n.bp3-icon-trash::before{\n  content:"\\E63B"; }\n\n.bp3-icon-tree::before{\n  content:"\\E7B7"; }\n\n.bp3-icon-trending-down::before{\n  content:"\\E71A"; }\n\n.bp3-icon-trending-up::before{\n  content:"\\E719"; }\n\n.bp3-icon-two-columns::before{\n  content:"\\E657"; }\n\n.bp3-icon-underline::before{\n  content:"\\2381"; }\n\n.bp3-icon-undo::before{\n  content:"\\238C"; }\n\n.bp3-icon-ungroup-objects::before{\n  content:"\\E688"; }\n\n.bp3-icon-unknown-vehicle::before{\n  content:"\\E73D"; }\n\n.bp3-icon-unlock::before{\n  content:"\\E626"; }\n\n.bp3-icon-unpin::before{\n  content:"\\E650"; }\n\n.bp3-icon-unresolve::before{\n  content:"\\E679"; }\n\n.bp3-icon-updated::before{\n  content:"\\E7A7"; }\n\n.bp3-icon-upload::before{\n  content:"\\E68F"; }\n\n.bp3-icon-user::before{\n  content:"\\E627"; }\n\n.bp3-icon-variable::before{\n  content:"\\E6F5"; }\n\n.bp3-icon-vertical-bar-chart-asc::before{\n  content:"\\E75B"; }\n\n.bp3-icon-vertical-bar-chart-desc::before{\n  content:"\\E71C"; }\n\n.bp3-icon-vertical-distribution::before{\n  content:"\\E721"; }\n\n.bp3-icon-video::before{\n  content:"\\E6A0"; }\n\n.bp3-icon-volume-down::before{\n  content:"\\E6A4"; }\n\n.bp3-icon-volume-off::before{\n  content:"\\E6A3"; }\n\n.bp3-icon-volume-up::before{\n  content:"\\E6A5"; }\n\n.bp3-icon-walk::before{\n  content:"\\E79D"; }\n\n.bp3-icon-warning-sign::before{\n  content:"\\E647"; }\n\n.bp3-icon-waterfall-chart::before{\n  content:"\\E6E6"; }\n\n.bp3-icon-widget::before{\n  content:"\\E678"; }\n\n.bp3-icon-widget-button::before{\n  content:"\\E790"; }\n\n.bp3-icon-widget-footer::before{\n  content:"\\E792"; }\n\n.bp3-icon-widget-header::before{\n  content:"\\E791"; }\n\n.bp3-icon-wrench::before{\n  content:"\\E734"; }\n\n.bp3-icon-zoom-in::before{\n  content:"\\E641"; }\n\n.bp3-icon-zoom-out::before{\n  content:"\\E642"; }\n\n.bp3-icon-zoom-to-fit::before{\n  content:"\\E67B"; }\n.bp3-submenu > .bp3-popover-wrapper{\n  display:block; }\n\n.bp3-submenu .bp3-popover-target{\n  display:block; }\n\n.bp3-submenu.bp3-popover{\n  -webkit-box-shadow:none;\n          box-shadow:none;\n  padding:0 5px; }\n  .bp3-submenu.bp3-popover > .bp3-popover-content{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2); }\n  .bp3-dark .bp3-submenu.bp3-popover, .bp3-submenu.bp3-popover.bp3-dark{\n    -webkit-box-shadow:none;\n            box-shadow:none; }\n    .bp3-dark .bp3-submenu.bp3-popover > .bp3-popover-content, .bp3-submenu.bp3-popover.bp3-dark > .bp3-popover-content{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4); }\n.bp3-menu{\n  margin:0;\n  border-radius:3px;\n  background:#ffffff;\n  min-width:180px;\n  padding:5px;\n  list-style:none;\n  text-align:left;\n  color:#182026; }\n\n.bp3-menu-divider{\n  display:block;\n  margin:5px;\n  border-top:1px solid rgba(16, 22, 26, 0.15); }\n  .bp3-dark .bp3-menu-divider{\n    border-top-color:rgba(255, 255, 255, 0.15); }\n\n.bp3-menu-item{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-orient:horizontal;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:row;\n      -ms-flex-direction:row;\n          flex-direction:row;\n  -webkit-box-align:start;\n  -webkit-align-items:flex-start;\n      -ms-flex-align:start;\n          align-items:flex-start;\n  border-radius:2px;\n  padding:5px 7px;\n  text-decoration:none;\n  line-height:20px;\n  color:inherit;\n  -webkit-user-select:none;\n     -moz-user-select:none;\n      -ms-user-select:none;\n          user-select:none; }\n  .bp3-menu-item > *{\n    -webkit-box-flex:0;\n    -webkit-flex-grow:0;\n        -ms-flex-positive:0;\n            flex-grow:0;\n    -webkit-flex-shrink:0;\n        -ms-flex-negative:0;\n            flex-shrink:0; }\n  .bp3-menu-item > .bp3-fill{\n    -webkit-box-flex:1;\n    -webkit-flex-grow:1;\n        -ms-flex-positive:1;\n            flex-grow:1;\n    -webkit-flex-shrink:1;\n        -ms-flex-negative:1;\n            flex-shrink:1; }\n  .bp3-menu-item::before,\n  .bp3-menu-item > *{\n    margin-right:7px; }\n  .bp3-menu-item:empty::before,\n  .bp3-menu-item > :last-child{\n    margin-right:0; }\n  .bp3-menu-item > .bp3-fill{\n    word-break:break-word; }\n  .bp3-menu-item:hover, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-menu-item{\n    background-color:rgba(167, 182, 194, 0.3);\n    cursor:pointer;\n    text-decoration:none; }\n  .bp3-menu-item.bp3-disabled{\n    background-color:inherit;\n    cursor:not-allowed;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-dark .bp3-menu-item{\n    color:inherit; }\n    .bp3-dark .bp3-menu-item:hover, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-menu-item, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-menu-item{\n      background-color:rgba(138, 155, 168, 0.15);\n      color:inherit; }\n    .bp3-dark .bp3-menu-item.bp3-disabled{\n      background-color:inherit;\n      color:rgba(167, 182, 194, 0.6); }\n  .bp3-menu-item.bp3-intent-primary{\n    color:#106ba3; }\n    .bp3-menu-item.bp3-intent-primary .bp3-icon{\n      color:inherit; }\n    .bp3-menu-item.bp3-intent-primary::before, .bp3-menu-item.bp3-intent-primary::after,\n    .bp3-menu-item.bp3-intent-primary .bp3-menu-item-label{\n      color:#106ba3; }\n    .bp3-menu-item.bp3-intent-primary:hover, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item, .bp3-menu-item.bp3-intent-primary.bp3-active{\n      background-color:#137cbd; }\n    .bp3-menu-item.bp3-intent-primary:active{\n      background-color:#106ba3; }\n    .bp3-menu-item.bp3-intent-primary:hover, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item, .bp3-menu-item.bp3-intent-primary:hover::before, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item::before, .bp3-menu-item.bp3-intent-primary:hover::after, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item::after,\n    .bp3-menu-item.bp3-intent-primary:hover .bp3-menu-item-label,\n    .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item .bp3-menu-item-label, .bp3-menu-item.bp3-intent-primary:active, .bp3-menu-item.bp3-intent-primary:active::before, .bp3-menu-item.bp3-intent-primary:active::after,\n    .bp3-menu-item.bp3-intent-primary:active .bp3-menu-item-label, .bp3-menu-item.bp3-intent-primary.bp3-active, .bp3-menu-item.bp3-intent-primary.bp3-active::before, .bp3-menu-item.bp3-intent-primary.bp3-active::after,\n    .bp3-menu-item.bp3-intent-primary.bp3-active .bp3-menu-item-label{\n      color:#ffffff; }\n  .bp3-menu-item.bp3-intent-success{\n    color:#0d8050; }\n    .bp3-menu-item.bp3-intent-success .bp3-icon{\n      color:inherit; }\n    .bp3-menu-item.bp3-intent-success::before, .bp3-menu-item.bp3-intent-success::after,\n    .bp3-menu-item.bp3-intent-success .bp3-menu-item-label{\n      color:#0d8050; }\n    .bp3-menu-item.bp3-intent-success:hover, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item, .bp3-menu-item.bp3-intent-success.bp3-active{\n      background-color:#0f9960; }\n    .bp3-menu-item.bp3-intent-success:active{\n      background-color:#0d8050; }\n    .bp3-menu-item.bp3-intent-success:hover, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item, .bp3-menu-item.bp3-intent-success:hover::before, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item::before, .bp3-menu-item.bp3-intent-success:hover::after, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item::after,\n    .bp3-menu-item.bp3-intent-success:hover .bp3-menu-item-label,\n    .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item .bp3-menu-item-label, .bp3-menu-item.bp3-intent-success:active, .bp3-menu-item.bp3-intent-success:active::before, .bp3-menu-item.bp3-intent-success:active::after,\n    .bp3-menu-item.bp3-intent-success:active .bp3-menu-item-label, .bp3-menu-item.bp3-intent-success.bp3-active, .bp3-menu-item.bp3-intent-success.bp3-active::before, .bp3-menu-item.bp3-intent-success.bp3-active::after,\n    .bp3-menu-item.bp3-intent-success.bp3-active .bp3-menu-item-label{\n      color:#ffffff; }\n  .bp3-menu-item.bp3-intent-warning{\n    color:#bf7326; }\n    .bp3-menu-item.bp3-intent-warning .bp3-icon{\n      color:inherit; }\n    .bp3-menu-item.bp3-intent-warning::before, .bp3-menu-item.bp3-intent-warning::after,\n    .bp3-menu-item.bp3-intent-warning .bp3-menu-item-label{\n      color:#bf7326; }\n    .bp3-menu-item.bp3-intent-warning:hover, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item, .bp3-menu-item.bp3-intent-warning.bp3-active{\n      background-color:#d9822b; }\n    .bp3-menu-item.bp3-intent-warning:active{\n      background-color:#bf7326; }\n    .bp3-menu-item.bp3-intent-warning:hover, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item, .bp3-menu-item.bp3-intent-warning:hover::before, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item::before, .bp3-menu-item.bp3-intent-warning:hover::after, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item::after,\n    .bp3-menu-item.bp3-intent-warning:hover .bp3-menu-item-label,\n    .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item .bp3-menu-item-label, .bp3-menu-item.bp3-intent-warning:active, .bp3-menu-item.bp3-intent-warning:active::before, .bp3-menu-item.bp3-intent-warning:active::after,\n    .bp3-menu-item.bp3-intent-warning:active .bp3-menu-item-label, .bp3-menu-item.bp3-intent-warning.bp3-active, .bp3-menu-item.bp3-intent-warning.bp3-active::before, .bp3-menu-item.bp3-intent-warning.bp3-active::after,\n    .bp3-menu-item.bp3-intent-warning.bp3-active .bp3-menu-item-label{\n      color:#ffffff; }\n  .bp3-menu-item.bp3-intent-danger{\n    color:#c23030; }\n    .bp3-menu-item.bp3-intent-danger .bp3-icon{\n      color:inherit; }\n    .bp3-menu-item.bp3-intent-danger::before, .bp3-menu-item.bp3-intent-danger::after,\n    .bp3-menu-item.bp3-intent-danger .bp3-menu-item-label{\n      color:#c23030; }\n    .bp3-menu-item.bp3-intent-danger:hover, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item, .bp3-menu-item.bp3-intent-danger.bp3-active{\n      background-color:#db3737; }\n    .bp3-menu-item.bp3-intent-danger:active{\n      background-color:#c23030; }\n    .bp3-menu-item.bp3-intent-danger:hover, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item, .bp3-menu-item.bp3-intent-danger:hover::before, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item::before, .bp3-menu-item.bp3-intent-danger:hover::after, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item::after,\n    .bp3-menu-item.bp3-intent-danger:hover .bp3-menu-item-label,\n    .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item .bp3-menu-item-label, .bp3-menu-item.bp3-intent-danger:active, .bp3-menu-item.bp3-intent-danger:active::before, .bp3-menu-item.bp3-intent-danger:active::after,\n    .bp3-menu-item.bp3-intent-danger:active .bp3-menu-item-label, .bp3-menu-item.bp3-intent-danger.bp3-active, .bp3-menu-item.bp3-intent-danger.bp3-active::before, .bp3-menu-item.bp3-intent-danger.bp3-active::after,\n    .bp3-menu-item.bp3-intent-danger.bp3-active .bp3-menu-item-label{\n      color:#ffffff; }\n  .bp3-menu-item::before{\n    line-height:1;\n    font-family:"Icons16", sans-serif;\n    font-size:16px;\n    font-weight:400;\n    font-style:normal;\n    -moz-osx-font-smoothing:grayscale;\n    -webkit-font-smoothing:antialiased;\n    margin-right:7px; }\n  .bp3-menu-item::before,\n  .bp3-menu-item > .bp3-icon{\n    margin-top:2px;\n    color:#5c7080; }\n  .bp3-menu-item .bp3-menu-item-label{\n    color:#5c7080; }\n  .bp3-menu-item:hover, .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-menu-item{\n    color:inherit; }\n  .bp3-menu-item.bp3-active, .bp3-menu-item:active{\n    background-color:rgba(115, 134, 148, 0.3); }\n  .bp3-menu-item.bp3-disabled{\n    outline:none !important;\n    background-color:inherit !important;\n    cursor:not-allowed !important;\n    color:rgba(92, 112, 128, 0.6) !important; }\n    .bp3-menu-item.bp3-disabled::before,\n    .bp3-menu-item.bp3-disabled > .bp3-icon,\n    .bp3-menu-item.bp3-disabled .bp3-menu-item-label{\n      color:rgba(92, 112, 128, 0.6) !important; }\n  .bp3-large .bp3-menu-item{\n    padding:9px 7px;\n    line-height:22px;\n    font-size:16px; }\n    .bp3-large .bp3-menu-item .bp3-icon{\n      margin-top:3px; }\n    .bp3-large .bp3-menu-item::before{\n      line-height:1;\n      font-family:"Icons20", sans-serif;\n      font-size:20px;\n      font-weight:400;\n      font-style:normal;\n      -moz-osx-font-smoothing:grayscale;\n      -webkit-font-smoothing:antialiased;\n      margin-top:1px;\n      margin-right:10px; }\n\nbutton.bp3-menu-item{\n  border:none;\n  background:none;\n  width:100%;\n  text-align:left; }\n.bp3-menu-header{\n  display:block;\n  margin:5px;\n  border-top:1px solid rgba(16, 22, 26, 0.15);\n  cursor:default;\n  padding-left:2px; }\n  .bp3-dark .bp3-menu-header{\n    border-top-color:rgba(255, 255, 255, 0.15); }\n  .bp3-menu-header:first-of-type{\n    border-top:none; }\n  .bp3-menu-header > h6{\n    color:#182026;\n    font-weight:600;\n    overflow:hidden;\n    text-overflow:ellipsis;\n    white-space:nowrap;\n    word-wrap:normal;\n    margin:0;\n    padding:10px 7px 0 1px;\n    line-height:17px; }\n    .bp3-dark .bp3-menu-header > h6{\n      color:#f5f8fa; }\n  .bp3-menu-header:first-of-type > h6{\n    padding-top:0; }\n  .bp3-large .bp3-menu-header > h6{\n    padding-top:15px;\n    padding-bottom:5px;\n    font-size:18px; }\n  .bp3-large .bp3-menu-header:first-of-type > h6{\n    padding-top:0; }\n\n.bp3-dark .bp3-menu{\n  background:#30404d;\n  color:#f5f8fa; }\n\n.bp3-dark .bp3-menu-item.bp3-intent-primary{\n  color:#48aff0; }\n  .bp3-dark .bp3-menu-item.bp3-intent-primary .bp3-icon{\n    color:inherit; }\n  .bp3-dark .bp3-menu-item.bp3-intent-primary::before, .bp3-dark .bp3-menu-item.bp3-intent-primary::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-primary .bp3-menu-item-label{\n    color:#48aff0; }\n  .bp3-dark .bp3-menu-item.bp3-intent-primary:hover, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item, .bp3-dark .bp3-menu-item.bp3-intent-primary.bp3-active{\n    background-color:#137cbd; }\n  .bp3-dark .bp3-menu-item.bp3-intent-primary:active{\n    background-color:#106ba3; }\n  .bp3-dark .bp3-menu-item.bp3-intent-primary:hover, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item, .bp3-dark .bp3-menu-item.bp3-intent-primary:hover::before, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item::before, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item::before, .bp3-dark .bp3-menu-item.bp3-intent-primary:hover::after, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item::after, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-primary:hover .bp3-menu-item-label,\n  .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item .bp3-menu-item-label,\n  .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-primary.bp3-menu-item .bp3-menu-item-label, .bp3-dark .bp3-menu-item.bp3-intent-primary:active, .bp3-dark .bp3-menu-item.bp3-intent-primary:active::before, .bp3-dark .bp3-menu-item.bp3-intent-primary:active::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-primary:active .bp3-menu-item-label, .bp3-dark .bp3-menu-item.bp3-intent-primary.bp3-active, .bp3-dark .bp3-menu-item.bp3-intent-primary.bp3-active::before, .bp3-dark .bp3-menu-item.bp3-intent-primary.bp3-active::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-primary.bp3-active .bp3-menu-item-label{\n    color:#ffffff; }\n\n.bp3-dark .bp3-menu-item.bp3-intent-success{\n  color:#3dcc91; }\n  .bp3-dark .bp3-menu-item.bp3-intent-success .bp3-icon{\n    color:inherit; }\n  .bp3-dark .bp3-menu-item.bp3-intent-success::before, .bp3-dark .bp3-menu-item.bp3-intent-success::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-success .bp3-menu-item-label{\n    color:#3dcc91; }\n  .bp3-dark .bp3-menu-item.bp3-intent-success:hover, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item, .bp3-dark .bp3-menu-item.bp3-intent-success.bp3-active{\n    background-color:#0f9960; }\n  .bp3-dark .bp3-menu-item.bp3-intent-success:active{\n    background-color:#0d8050; }\n  .bp3-dark .bp3-menu-item.bp3-intent-success:hover, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item, .bp3-dark .bp3-menu-item.bp3-intent-success:hover::before, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item::before, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item::before, .bp3-dark .bp3-menu-item.bp3-intent-success:hover::after, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item::after, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-success:hover .bp3-menu-item-label,\n  .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item .bp3-menu-item-label,\n  .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-success.bp3-menu-item .bp3-menu-item-label, .bp3-dark .bp3-menu-item.bp3-intent-success:active, .bp3-dark .bp3-menu-item.bp3-intent-success:active::before, .bp3-dark .bp3-menu-item.bp3-intent-success:active::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-success:active .bp3-menu-item-label, .bp3-dark .bp3-menu-item.bp3-intent-success.bp3-active, .bp3-dark .bp3-menu-item.bp3-intent-success.bp3-active::before, .bp3-dark .bp3-menu-item.bp3-intent-success.bp3-active::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-success.bp3-active .bp3-menu-item-label{\n    color:#ffffff; }\n\n.bp3-dark .bp3-menu-item.bp3-intent-warning{\n  color:#ffb366; }\n  .bp3-dark .bp3-menu-item.bp3-intent-warning .bp3-icon{\n    color:inherit; }\n  .bp3-dark .bp3-menu-item.bp3-intent-warning::before, .bp3-dark .bp3-menu-item.bp3-intent-warning::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-warning .bp3-menu-item-label{\n    color:#ffb366; }\n  .bp3-dark .bp3-menu-item.bp3-intent-warning:hover, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item, .bp3-dark .bp3-menu-item.bp3-intent-warning.bp3-active{\n    background-color:#d9822b; }\n  .bp3-dark .bp3-menu-item.bp3-intent-warning:active{\n    background-color:#bf7326; }\n  .bp3-dark .bp3-menu-item.bp3-intent-warning:hover, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item, .bp3-dark .bp3-menu-item.bp3-intent-warning:hover::before, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item::before, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item::before, .bp3-dark .bp3-menu-item.bp3-intent-warning:hover::after, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item::after, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-warning:hover .bp3-menu-item-label,\n  .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item .bp3-menu-item-label,\n  .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-warning.bp3-menu-item .bp3-menu-item-label, .bp3-dark .bp3-menu-item.bp3-intent-warning:active, .bp3-dark .bp3-menu-item.bp3-intent-warning:active::before, .bp3-dark .bp3-menu-item.bp3-intent-warning:active::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-warning:active .bp3-menu-item-label, .bp3-dark .bp3-menu-item.bp3-intent-warning.bp3-active, .bp3-dark .bp3-menu-item.bp3-intent-warning.bp3-active::before, .bp3-dark .bp3-menu-item.bp3-intent-warning.bp3-active::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-warning.bp3-active .bp3-menu-item-label{\n    color:#ffffff; }\n\n.bp3-dark .bp3-menu-item.bp3-intent-danger{\n  color:#ff7373; }\n  .bp3-dark .bp3-menu-item.bp3-intent-danger .bp3-icon{\n    color:inherit; }\n  .bp3-dark .bp3-menu-item.bp3-intent-danger::before, .bp3-dark .bp3-menu-item.bp3-intent-danger::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-danger .bp3-menu-item-label{\n    color:#ff7373; }\n  .bp3-dark .bp3-menu-item.bp3-intent-danger:hover, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item, .bp3-dark .bp3-menu-item.bp3-intent-danger.bp3-active{\n    background-color:#db3737; }\n  .bp3-dark .bp3-menu-item.bp3-intent-danger:active{\n    background-color:#c23030; }\n  .bp3-dark .bp3-menu-item.bp3-intent-danger:hover, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item, .bp3-dark .bp3-menu-item.bp3-intent-danger:hover::before, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item::before, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item::before, .bp3-dark .bp3-menu-item.bp3-intent-danger:hover::after, .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item::after, .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-danger:hover .bp3-menu-item-label,\n  .bp3-dark .bp3-submenu .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item .bp3-menu-item-label,\n  .bp3-submenu .bp3-dark .bp3-popover-target.bp3-popover-open > .bp3-intent-danger.bp3-menu-item .bp3-menu-item-label, .bp3-dark .bp3-menu-item.bp3-intent-danger:active, .bp3-dark .bp3-menu-item.bp3-intent-danger:active::before, .bp3-dark .bp3-menu-item.bp3-intent-danger:active::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-danger:active .bp3-menu-item-label, .bp3-dark .bp3-menu-item.bp3-intent-danger.bp3-active, .bp3-dark .bp3-menu-item.bp3-intent-danger.bp3-active::before, .bp3-dark .bp3-menu-item.bp3-intent-danger.bp3-active::after,\n  .bp3-dark .bp3-menu-item.bp3-intent-danger.bp3-active .bp3-menu-item-label{\n    color:#ffffff; }\n\n.bp3-dark .bp3-menu-item::before,\n.bp3-dark .bp3-menu-item > .bp3-icon{\n  color:#a7b6c2; }\n\n.bp3-dark .bp3-menu-item .bp3-menu-item-label{\n  color:#a7b6c2; }\n\n.bp3-dark .bp3-menu-item.bp3-active, .bp3-dark .bp3-menu-item:active{\n  background-color:rgba(138, 155, 168, 0.3); }\n\n.bp3-dark .bp3-menu-item.bp3-disabled{\n  color:rgba(167, 182, 194, 0.6) !important; }\n  .bp3-dark .bp3-menu-item.bp3-disabled::before,\n  .bp3-dark .bp3-menu-item.bp3-disabled > .bp3-icon,\n  .bp3-dark .bp3-menu-item.bp3-disabled .bp3-menu-item-label{\n    color:rgba(167, 182, 194, 0.6) !important; }\n\n.bp3-dark .bp3-menu-divider,\n.bp3-dark .bp3-menu-header{\n  border-color:rgba(255, 255, 255, 0.15); }\n\n.bp3-dark .bp3-menu-header > h6{\n  color:#f5f8fa; }\n\n.bp3-label .bp3-menu{\n  margin-top:5px; }\n.bp3-navbar{\n  position:relative;\n  z-index:10;\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.2);\n  background-color:#ffffff;\n  width:100%;\n  height:50px;\n  padding:0 15px; }\n  .bp3-navbar.bp3-dark,\n  .bp3-dark .bp3-navbar{\n    background-color:#394b59; }\n  .bp3-navbar.bp3-dark{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.4); }\n  .bp3-dark .bp3-navbar{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 0 0 rgba(16, 22, 26, 0), 0 1px 1px rgba(16, 22, 26, 0.4); }\n  .bp3-navbar.bp3-fixed-top{\n    position:fixed;\n    top:0;\n    right:0;\n    left:0; }\n\n.bp3-navbar-heading{\n  margin-right:15px;\n  font-size:16px; }\n\n.bp3-navbar-group{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  height:50px; }\n  .bp3-navbar-group.bp3-align-left{\n    float:left; }\n  .bp3-navbar-group.bp3-align-right{\n    float:right; }\n\n.bp3-navbar-divider{\n  margin:0 10px;\n  border-left:1px solid rgba(16, 22, 26, 0.15);\n  height:20px; }\n  .bp3-dark .bp3-navbar-divider{\n    border-left-color:rgba(255, 255, 255, 0.15); }\n.bp3-non-ideal-state{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-orient:vertical;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:column;\n      -ms-flex-direction:column;\n          flex-direction:column;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  -webkit-box-pack:center;\n  -webkit-justify-content:center;\n      -ms-flex-pack:center;\n          justify-content:center;\n  width:100%;\n  height:100%;\n  text-align:center; }\n  .bp3-non-ideal-state > *{\n    -webkit-box-flex:0;\n    -webkit-flex-grow:0;\n        -ms-flex-positive:0;\n            flex-grow:0;\n    -webkit-flex-shrink:0;\n        -ms-flex-negative:0;\n            flex-shrink:0; }\n  .bp3-non-ideal-state > .bp3-fill{\n    -webkit-box-flex:1;\n    -webkit-flex-grow:1;\n        -ms-flex-positive:1;\n            flex-grow:1;\n    -webkit-flex-shrink:1;\n        -ms-flex-negative:1;\n            flex-shrink:1; }\n  .bp3-non-ideal-state::before,\n  .bp3-non-ideal-state > *{\n    margin-bottom:20px; }\n  .bp3-non-ideal-state:empty::before,\n  .bp3-non-ideal-state > :last-child{\n    margin-bottom:0; }\n  .bp3-non-ideal-state > *{\n    max-width:400px; }\n\n.bp3-non-ideal-state-visual{\n  color:rgba(92, 112, 128, 0.6);\n  font-size:60px; }\n  .bp3-dark .bp3-non-ideal-state-visual{\n    color:rgba(167, 182, 194, 0.6); }\n\n.bp3-overflow-list{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-flex-wrap:nowrap;\n      -ms-flex-wrap:nowrap;\n          flex-wrap:nowrap;\n  min-width:0; }\n\n.bp3-overflow-list-spacer{\n  -webkit-flex-shrink:1;\n      -ms-flex-negative:1;\n          flex-shrink:1;\n  width:1px; }\n\nbody.bp3-overlay-open{\n  overflow:hidden; }\n\n.bp3-overlay{\n  position:static;\n  top:0;\n  right:0;\n  bottom:0;\n  left:0;\n  z-index:20; }\n  .bp3-overlay:not(.bp3-overlay-open){\n    pointer-events:none; }\n  .bp3-overlay.bp3-overlay-container{\n    position:fixed;\n    overflow:hidden; }\n    .bp3-overlay.bp3-overlay-container.bp3-overlay-inline{\n      position:absolute; }\n  .bp3-overlay.bp3-overlay-scroll-container{\n    position:fixed;\n    overflow:auto; }\n    .bp3-overlay.bp3-overlay-scroll-container.bp3-overlay-inline{\n      position:absolute; }\n  .bp3-overlay.bp3-overlay-inline{\n    display:inline;\n    overflow:visible; }\n\n.bp3-overlay-content{\n  position:fixed;\n  z-index:20; }\n  .bp3-overlay-inline .bp3-overlay-content,\n  .bp3-overlay-scroll-container .bp3-overlay-content{\n    position:absolute; }\n\n.bp3-overlay-backdrop{\n  position:fixed;\n  top:0;\n  right:0;\n  bottom:0;\n  left:0;\n  opacity:1;\n  z-index:20;\n  background-color:rgba(16, 22, 26, 0.7);\n  overflow:auto;\n  -webkit-user-select:none;\n     -moz-user-select:none;\n      -ms-user-select:none;\n          user-select:none; }\n  .bp3-overlay-backdrop.bp3-overlay-enter, .bp3-overlay-backdrop.bp3-overlay-appear{\n    opacity:0; }\n  .bp3-overlay-backdrop.bp3-overlay-enter-active, .bp3-overlay-backdrop.bp3-overlay-appear-active{\n    opacity:1;\n    -webkit-transition-property:opacity;\n    transition-property:opacity;\n    -webkit-transition-duration:200ms;\n            transition-duration:200ms;\n    -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n            transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n  .bp3-overlay-backdrop.bp3-overlay-exit{\n    opacity:1; }\n  .bp3-overlay-backdrop.bp3-overlay-exit-active{\n    opacity:0;\n    -webkit-transition-property:opacity;\n    transition-property:opacity;\n    -webkit-transition-duration:200ms;\n            transition-duration:200ms;\n    -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n            transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n  .bp3-overlay-backdrop:focus{\n    outline:none; }\n  .bp3-overlay-inline .bp3-overlay-backdrop{\n    position:absolute; }\n.bp3-panel-stack{\n  position:relative;\n  overflow:hidden; }\n\n.bp3-panel-stack-header{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-flex-shrink:0;\n      -ms-flex-negative:0;\n          flex-shrink:0;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  z-index:1;\n  -webkit-box-shadow:0 1px rgba(16, 22, 26, 0.15);\n          box-shadow:0 1px rgba(16, 22, 26, 0.15);\n  height:30px; }\n  .bp3-dark .bp3-panel-stack-header{\n    -webkit-box-shadow:0 1px rgba(255, 255, 255, 0.15);\n            box-shadow:0 1px rgba(255, 255, 255, 0.15); }\n  .bp3-panel-stack-header > span{\n    display:-webkit-box;\n    display:-webkit-flex;\n    display:-ms-flexbox;\n    display:flex;\n    -webkit-box-flex:1;\n    -webkit-flex:1;\n        -ms-flex:1;\n            flex:1;\n    -webkit-box-align:stretch;\n    -webkit-align-items:stretch;\n        -ms-flex-align:stretch;\n            align-items:stretch; }\n  .bp3-panel-stack-header .bp3-heading{\n    margin:0 5px; }\n\n.bp3-button.bp3-panel-stack-header-back{\n  margin-left:5px;\n  padding-left:0;\n  white-space:nowrap; }\n  .bp3-button.bp3-panel-stack-header-back .bp3-icon{\n    margin:0 2px; }\n\n.bp3-panel-stack-view{\n  position:absolute;\n  top:0;\n  right:0;\n  bottom:0;\n  left:0;\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-orient:vertical;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:column;\n      -ms-flex-direction:column;\n          flex-direction:column;\n  margin-right:-1px;\n  border-right:1px solid rgba(16, 22, 26, 0.15);\n  background-color:#ffffff;\n  overflow-y:auto; }\n  .bp3-dark .bp3-panel-stack-view{\n    background-color:#30404d; }\n\n.bp3-panel-stack-push .bp3-panel-stack-enter, .bp3-panel-stack-push .bp3-panel-stack-appear{\n  -webkit-transform:translateX(100%);\n          transform:translateX(100%);\n  opacity:0; }\n\n.bp3-panel-stack-push .bp3-panel-stack-enter-active, .bp3-panel-stack-push .bp3-panel-stack-appear-active{\n  -webkit-transform:translate(0%);\n          transform:translate(0%);\n  opacity:1;\n  -webkit-transition-property:opacity, -webkit-transform;\n  transition-property:opacity, -webkit-transform;\n  transition-property:transform, opacity;\n  transition-property:transform, opacity, -webkit-transform;\n  -webkit-transition-duration:400ms;\n          transition-duration:400ms;\n  -webkit-transition-timing-function:ease;\n          transition-timing-function:ease;\n  -webkit-transition-delay:0;\n          transition-delay:0; }\n\n.bp3-panel-stack-push .bp3-panel-stack-exit{\n  -webkit-transform:translate(0%);\n          transform:translate(0%);\n  opacity:1; }\n\n.bp3-panel-stack-push .bp3-panel-stack-exit-active{\n  -webkit-transform:translateX(-50%);\n          transform:translateX(-50%);\n  opacity:0;\n  -webkit-transition-property:opacity, -webkit-transform;\n  transition-property:opacity, -webkit-transform;\n  transition-property:transform, opacity;\n  transition-property:transform, opacity, -webkit-transform;\n  -webkit-transition-duration:400ms;\n          transition-duration:400ms;\n  -webkit-transition-timing-function:ease;\n          transition-timing-function:ease;\n  -webkit-transition-delay:0;\n          transition-delay:0; }\n\n.bp3-panel-stack-pop .bp3-panel-stack-enter, .bp3-panel-stack-pop .bp3-panel-stack-appear{\n  -webkit-transform:translateX(-50%);\n          transform:translateX(-50%);\n  opacity:0; }\n\n.bp3-panel-stack-pop .bp3-panel-stack-enter-active, .bp3-panel-stack-pop .bp3-panel-stack-appear-active{\n  -webkit-transform:translate(0%);\n          transform:translate(0%);\n  opacity:1;\n  -webkit-transition-property:opacity, -webkit-transform;\n  transition-property:opacity, -webkit-transform;\n  transition-property:transform, opacity;\n  transition-property:transform, opacity, -webkit-transform;\n  -webkit-transition-duration:400ms;\n          transition-duration:400ms;\n  -webkit-transition-timing-function:ease;\n          transition-timing-function:ease;\n  -webkit-transition-delay:0;\n          transition-delay:0; }\n\n.bp3-panel-stack-pop .bp3-panel-stack-exit{\n  -webkit-transform:translate(0%);\n          transform:translate(0%);\n  opacity:1; }\n\n.bp3-panel-stack-pop .bp3-panel-stack-exit-active{\n  -webkit-transform:translateX(100%);\n          transform:translateX(100%);\n  opacity:0;\n  -webkit-transition-property:opacity, -webkit-transform;\n  transition-property:opacity, -webkit-transform;\n  transition-property:transform, opacity;\n  transition-property:transform, opacity, -webkit-transform;\n  -webkit-transition-duration:400ms;\n          transition-duration:400ms;\n  -webkit-transition-timing-function:ease;\n          transition-timing-function:ease;\n  -webkit-transition-delay:0;\n          transition-delay:0; }\n.bp3-popover{\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2);\n  -webkit-transform:scale(1);\n          transform:scale(1);\n  display:inline-block;\n  z-index:20;\n  border-radius:3px; }\n  .bp3-popover .bp3-popover-arrow{\n    position:absolute;\n    width:30px;\n    height:30px; }\n    .bp3-popover .bp3-popover-arrow::before{\n      margin:5px;\n      width:20px;\n      height:20px; }\n  .bp3-tether-element-attached-bottom.bp3-tether-target-attached-top > .bp3-popover{\n    margin-top:-17px;\n    margin-bottom:17px; }\n    .bp3-tether-element-attached-bottom.bp3-tether-target-attached-top > .bp3-popover > .bp3-popover-arrow{\n      bottom:-11px; }\n      .bp3-tether-element-attached-bottom.bp3-tether-target-attached-top > .bp3-popover > .bp3-popover-arrow svg{\n        -webkit-transform:rotate(-90deg);\n                transform:rotate(-90deg); }\n  .bp3-tether-element-attached-left.bp3-tether-target-attached-right > .bp3-popover{\n    margin-left:17px; }\n    .bp3-tether-element-attached-left.bp3-tether-target-attached-right > .bp3-popover > .bp3-popover-arrow{\n      left:-11px; }\n      .bp3-tether-element-attached-left.bp3-tether-target-attached-right > .bp3-popover > .bp3-popover-arrow svg{\n        -webkit-transform:rotate(0);\n                transform:rotate(0); }\n  .bp3-tether-element-attached-top.bp3-tether-target-attached-bottom > .bp3-popover{\n    margin-top:17px; }\n    .bp3-tether-element-attached-top.bp3-tether-target-attached-bottom > .bp3-popover > .bp3-popover-arrow{\n      top:-11px; }\n      .bp3-tether-element-attached-top.bp3-tether-target-attached-bottom > .bp3-popover > .bp3-popover-arrow svg{\n        -webkit-transform:rotate(90deg);\n                transform:rotate(90deg); }\n  .bp3-tether-element-attached-right.bp3-tether-target-attached-left > .bp3-popover{\n    margin-right:17px;\n    margin-left:-17px; }\n    .bp3-tether-element-attached-right.bp3-tether-target-attached-left > .bp3-popover > .bp3-popover-arrow{\n      right:-11px; }\n      .bp3-tether-element-attached-right.bp3-tether-target-attached-left > .bp3-popover > .bp3-popover-arrow svg{\n        -webkit-transform:rotate(180deg);\n                transform:rotate(180deg); }\n  .bp3-tether-element-attached-middle > .bp3-popover > .bp3-popover-arrow{\n    top:50%;\n    -webkit-transform:translateY(-50%);\n            transform:translateY(-50%); }\n  .bp3-tether-element-attached-center > .bp3-popover > .bp3-popover-arrow{\n    right:50%;\n    -webkit-transform:translateX(50%);\n            transform:translateX(50%); }\n  .bp3-tether-element-attached-top.bp3-tether-target-attached-top > .bp3-popover > .bp3-popover-arrow{\n    top:-0.3934px; }\n  .bp3-tether-element-attached-right.bp3-tether-target-attached-right > .bp3-popover > .bp3-popover-arrow{\n    right:-0.3934px; }\n  .bp3-tether-element-attached-left.bp3-tether-target-attached-left > .bp3-popover > .bp3-popover-arrow{\n    left:-0.3934px; }\n  .bp3-tether-element-attached-bottom.bp3-tether-target-attached-bottom > .bp3-popover > .bp3-popover-arrow{\n    bottom:-0.3934px; }\n  .bp3-tether-element-attached-top.bp3-tether-element-attached-left > .bp3-popover{\n    -webkit-transform-origin:top left;\n            transform-origin:top left; }\n  .bp3-tether-element-attached-top.bp3-tether-element-attached-center > .bp3-popover{\n    -webkit-transform-origin:top center;\n            transform-origin:top center; }\n  .bp3-tether-element-attached-top.bp3-tether-element-attached-right > .bp3-popover{\n    -webkit-transform-origin:top right;\n            transform-origin:top right; }\n  .bp3-tether-element-attached-middle.bp3-tether-element-attached-left > .bp3-popover{\n    -webkit-transform-origin:center left;\n            transform-origin:center left; }\n  .bp3-tether-element-attached-middle.bp3-tether-element-attached-center > .bp3-popover{\n    -webkit-transform-origin:center center;\n            transform-origin:center center; }\n  .bp3-tether-element-attached-middle.bp3-tether-element-attached-right > .bp3-popover{\n    -webkit-transform-origin:center right;\n            transform-origin:center right; }\n  .bp3-tether-element-attached-bottom.bp3-tether-element-attached-left > .bp3-popover{\n    -webkit-transform-origin:bottom left;\n            transform-origin:bottom left; }\n  .bp3-tether-element-attached-bottom.bp3-tether-element-attached-center > .bp3-popover{\n    -webkit-transform-origin:bottom center;\n            transform-origin:bottom center; }\n  .bp3-tether-element-attached-bottom.bp3-tether-element-attached-right > .bp3-popover{\n    -webkit-transform-origin:bottom right;\n            transform-origin:bottom right; }\n  .bp3-popover .bp3-popover-content{\n    background:#ffffff;\n    color:inherit; }\n  .bp3-popover .bp3-popover-arrow::before{\n    -webkit-box-shadow:1px 1px 6px rgba(16, 22, 26, 0.2);\n            box-shadow:1px 1px 6px rgba(16, 22, 26, 0.2); }\n  .bp3-popover .bp3-popover-arrow-border{\n    fill:#10161a;\n    fill-opacity:0.1; }\n  .bp3-popover .bp3-popover-arrow-fill{\n    fill:#ffffff; }\n  .bp3-popover-enter > .bp3-popover, .bp3-popover-appear > .bp3-popover{\n    -webkit-transform:scale(0.3);\n            transform:scale(0.3); }\n  .bp3-popover-enter-active > .bp3-popover, .bp3-popover-appear-active > .bp3-popover{\n    -webkit-transform:scale(1);\n            transform:scale(1);\n    -webkit-transition-property:-webkit-transform;\n    transition-property:-webkit-transform;\n    transition-property:transform;\n    transition-property:transform, -webkit-transform;\n    -webkit-transition-duration:300ms;\n            transition-duration:300ms;\n    -webkit-transition-timing-function:cubic-bezier(0.54, 1.12, 0.38, 1.11);\n            transition-timing-function:cubic-bezier(0.54, 1.12, 0.38, 1.11);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n  .bp3-popover-exit > .bp3-popover{\n    -webkit-transform:scale(1);\n            transform:scale(1); }\n  .bp3-popover-exit-active > .bp3-popover{\n    -webkit-transform:scale(0.3);\n            transform:scale(0.3);\n    -webkit-transition-property:-webkit-transform;\n    transition-property:-webkit-transform;\n    transition-property:transform;\n    transition-property:transform, -webkit-transform;\n    -webkit-transition-duration:300ms;\n            transition-duration:300ms;\n    -webkit-transition-timing-function:cubic-bezier(0.54, 1.12, 0.38, 1.11);\n            transition-timing-function:cubic-bezier(0.54, 1.12, 0.38, 1.11);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n  .bp3-popover .bp3-popover-content{\n    position:relative;\n    border-radius:3px; }\n  .bp3-popover.bp3-popover-content-sizing .bp3-popover-content{\n    max-width:350px;\n    padding:20px; }\n  .bp3-popover-target + .bp3-overlay .bp3-popover.bp3-popover-content-sizing{\n    width:350px; }\n  .bp3-popover.bp3-minimal{\n    margin:0 !important; }\n    .bp3-popover.bp3-minimal .bp3-popover-arrow{\n      display:none; }\n    .bp3-popover.bp3-minimal.bp3-popover{\n      -webkit-transform:scale(1);\n              transform:scale(1); }\n      .bp3-popover-enter > .bp3-popover.bp3-minimal.bp3-popover, .bp3-popover-appear > .bp3-popover.bp3-minimal.bp3-popover{\n        -webkit-transform:scale(1);\n                transform:scale(1); }\n      .bp3-popover-enter-active > .bp3-popover.bp3-minimal.bp3-popover, .bp3-popover-appear-active > .bp3-popover.bp3-minimal.bp3-popover{\n        -webkit-transform:scale(1);\n                transform:scale(1);\n        -webkit-transition-property:-webkit-transform;\n        transition-property:-webkit-transform;\n        transition-property:transform;\n        transition-property:transform, -webkit-transform;\n        -webkit-transition-duration:100ms;\n                transition-duration:100ms;\n        -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n                transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n        -webkit-transition-delay:0;\n                transition-delay:0; }\n      .bp3-popover-exit > .bp3-popover.bp3-minimal.bp3-popover{\n        -webkit-transform:scale(1);\n                transform:scale(1); }\n      .bp3-popover-exit-active > .bp3-popover.bp3-minimal.bp3-popover{\n        -webkit-transform:scale(1);\n                transform:scale(1);\n        -webkit-transition-property:-webkit-transform;\n        transition-property:-webkit-transform;\n        transition-property:transform;\n        transition-property:transform, -webkit-transform;\n        -webkit-transition-duration:100ms;\n                transition-duration:100ms;\n        -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n                transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n        -webkit-transition-delay:0;\n                transition-delay:0; }\n  .bp3-popover.bp3-dark,\n  .bp3-dark .bp3-popover{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4); }\n    .bp3-popover.bp3-dark .bp3-popover-content,\n    .bp3-dark .bp3-popover .bp3-popover-content{\n      background:#30404d;\n      color:inherit; }\n    .bp3-popover.bp3-dark .bp3-popover-arrow::before,\n    .bp3-dark .bp3-popover .bp3-popover-arrow::before{\n      -webkit-box-shadow:1px 1px 6px rgba(16, 22, 26, 0.4);\n              box-shadow:1px 1px 6px rgba(16, 22, 26, 0.4); }\n    .bp3-popover.bp3-dark .bp3-popover-arrow-border,\n    .bp3-dark .bp3-popover .bp3-popover-arrow-border{\n      fill:#10161a;\n      fill-opacity:0.2; }\n    .bp3-popover.bp3-dark .bp3-popover-arrow-fill,\n    .bp3-dark .bp3-popover .bp3-popover-arrow-fill{\n      fill:#30404d; }\n\n.bp3-popover-arrow::before{\n  display:block;\n  position:absolute;\n  -webkit-transform:rotate(45deg);\n          transform:rotate(45deg);\n  border-radius:2px;\n  content:""; }\n\n.bp3-tether-pinned .bp3-popover-arrow{\n  display:none; }\n\n.bp3-popover-backdrop{\n  background:rgba(255, 255, 255, 0); }\n\n.bp3-transition-container{\n  opacity:1;\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  z-index:20; }\n  .bp3-transition-container.bp3-popover-enter, .bp3-transition-container.bp3-popover-appear{\n    opacity:0; }\n  .bp3-transition-container.bp3-popover-enter-active, .bp3-transition-container.bp3-popover-appear-active{\n    opacity:1;\n    -webkit-transition-property:opacity;\n    transition-property:opacity;\n    -webkit-transition-duration:100ms;\n            transition-duration:100ms;\n    -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n            transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n  .bp3-transition-container.bp3-popover-exit{\n    opacity:1; }\n  .bp3-transition-container.bp3-popover-exit-active{\n    opacity:0;\n    -webkit-transition-property:opacity;\n    transition-property:opacity;\n    -webkit-transition-duration:100ms;\n            transition-duration:100ms;\n    -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n            transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n  .bp3-transition-container:focus{\n    outline:none; }\n  .bp3-transition-container.bp3-popover-leave .bp3-popover-content{\n    pointer-events:none; }\n  .bp3-transition-container[data-x-out-of-boundaries]{\n    display:none; }\n\nspan.bp3-popover-target{\n  display:inline-block; }\n\n.bp3-popover-wrapper.bp3-fill{\n  width:100%; }\n\n.bp3-portal{\n  position:absolute;\n  top:0;\n  right:0;\n  left:0; }\n@-webkit-keyframes linear-progress-bar-stripes{\n  from{\n    background-position:0 0; }\n  to{\n    background-position:30px 0; } }\n@keyframes linear-progress-bar-stripes{\n  from{\n    background-position:0 0; }\n  to{\n    background-position:30px 0; } }\n\n.bp3-progress-bar{\n  display:block;\n  position:relative;\n  border-radius:40px;\n  background:rgba(92, 112, 128, 0.2);\n  width:100%;\n  height:8px;\n  overflow:hidden; }\n  .bp3-progress-bar .bp3-progress-meter{\n    position:absolute;\n    border-radius:40px;\n    background:linear-gradient(-45deg, rgba(255, 255, 255, 0.2) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.2) 50%, rgba(255, 255, 255, 0.2) 75%, transparent 75%);\n    background-color:rgba(92, 112, 128, 0.8);\n    background-size:30px 30px;\n    width:100%;\n    height:100%;\n    -webkit-transition:width 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n    transition:width 200ms cubic-bezier(0.4, 1, 0.75, 0.9); }\n  .bp3-progress-bar:not(.bp3-no-animation):not(.bp3-no-stripes) .bp3-progress-meter{\n    animation:linear-progress-bar-stripes 300ms linear infinite reverse; }\n  .bp3-progress-bar.bp3-no-stripes .bp3-progress-meter{\n    background-image:none; }\n\n.bp3-dark .bp3-progress-bar{\n  background:rgba(16, 22, 26, 0.5); }\n  .bp3-dark .bp3-progress-bar .bp3-progress-meter{\n    background-color:#8a9ba8; }\n\n.bp3-progress-bar.bp3-intent-primary .bp3-progress-meter{\n  background-color:#137cbd; }\n\n.bp3-progress-bar.bp3-intent-success .bp3-progress-meter{\n  background-color:#0f9960; }\n\n.bp3-progress-bar.bp3-intent-warning .bp3-progress-meter{\n  background-color:#d9822b; }\n\n.bp3-progress-bar.bp3-intent-danger .bp3-progress-meter{\n  background-color:#db3737; }\n@-webkit-keyframes skeleton-glow{\n  from{\n    border-color:rgba(206, 217, 224, 0.2);\n    background:rgba(206, 217, 224, 0.2); }\n  to{\n    border-color:rgba(92, 112, 128, 0.2);\n    background:rgba(92, 112, 128, 0.2); } }\n@keyframes skeleton-glow{\n  from{\n    border-color:rgba(206, 217, 224, 0.2);\n    background:rgba(206, 217, 224, 0.2); }\n  to{\n    border-color:rgba(92, 112, 128, 0.2);\n    background:rgba(92, 112, 128, 0.2); } }\n.bp3-skeleton{\n  border-color:rgba(206, 217, 224, 0.2) !important;\n  border-radius:2px;\n  -webkit-box-shadow:none !important;\n          box-shadow:none !important;\n  background:rgba(206, 217, 224, 0.2);\n  background-clip:padding-box !important;\n  cursor:default;\n  color:transparent !important;\n  -webkit-animation:1000ms linear infinite alternate skeleton-glow;\n          animation:1000ms linear infinite alternate skeleton-glow;\n  pointer-events:none;\n  -webkit-user-select:none;\n     -moz-user-select:none;\n      -ms-user-select:none;\n          user-select:none; }\n  .bp3-skeleton::before, .bp3-skeleton::after,\n  .bp3-skeleton *{\n    visibility:hidden !important; }\n.bp3-slider{\n  width:100%;\n  min-width:150px;\n  height:40px;\n  position:relative;\n  outline:none;\n  cursor:default;\n  -webkit-user-select:none;\n     -moz-user-select:none;\n      -ms-user-select:none;\n          user-select:none; }\n  .bp3-slider:hover{\n    cursor:pointer; }\n  .bp3-slider:active{\n    cursor:-webkit-grabbing;\n    cursor:grabbing; }\n  .bp3-slider.bp3-disabled{\n    opacity:0.5;\n    cursor:not-allowed; }\n  .bp3-slider.bp3-slider-unlabeled{\n    height:16px; }\n\n.bp3-slider-track,\n.bp3-slider-progress{\n  top:5px;\n  right:0;\n  left:0;\n  height:6px;\n  position:absolute; }\n\n.bp3-slider-track{\n  border-radius:3px;\n  overflow:hidden; }\n\n.bp3-slider-progress{\n  background:rgba(92, 112, 128, 0.2); }\n  .bp3-dark .bp3-slider-progress{\n    background:rgba(16, 22, 26, 0.5); }\n  .bp3-slider-progress.bp3-intent-primary{\n    background-color:#137cbd; }\n  .bp3-slider-progress.bp3-intent-success{\n    background-color:#0f9960; }\n  .bp3-slider-progress.bp3-intent-warning{\n    background-color:#d9822b; }\n  .bp3-slider-progress.bp3-intent-danger{\n    background-color:#db3737; }\n\n.bp3-slider-handle{\n  -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n          box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n  background-color:#f5f8fa;\n  background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.8)), to(rgba(255, 255, 255, 0)));\n  background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.8), rgba(255, 255, 255, 0));\n  color:#182026;\n  position:absolute;\n  top:0;\n  left:0;\n  border-radius:3px;\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 1px 1px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 1px 1px rgba(16, 22, 26, 0.2);\n  cursor:pointer;\n  width:16px;\n  height:16px; }\n  .bp3-slider-handle:hover{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n    background-clip:padding-box;\n    background-color:#ebf1f5; }\n  .bp3-slider-handle:active, .bp3-slider-handle.bp3-active{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n    background-color:#d8e1e8;\n    background-image:none; }\n  .bp3-slider-handle:disabled, .bp3-slider-handle.bp3-disabled{\n    outline:none;\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background-color:rgba(206, 217, 224, 0.5);\n    background-image:none;\n    cursor:not-allowed;\n    color:rgba(92, 112, 128, 0.6); }\n    .bp3-slider-handle:disabled.bp3-active, .bp3-slider-handle:disabled.bp3-active:hover, .bp3-slider-handle.bp3-disabled.bp3-active, .bp3-slider-handle.bp3-disabled.bp3-active:hover{\n      background:rgba(206, 217, 224, 0.7); }\n  .bp3-slider-handle:focus{\n    z-index:1; }\n  .bp3-slider-handle:hover{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 -1px 0 rgba(16, 22, 26, 0.1);\n    background-clip:padding-box;\n    background-color:#ebf1f5;\n    z-index:2;\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 1px 1px rgba(16, 22, 26, 0.2);\n    cursor:-webkit-grab;\n    cursor:grab; }\n  .bp3-slider-handle.bp3-active{\n    -webkit-box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n            box-shadow:inset 0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n    background-color:#d8e1e8;\n    background-image:none;\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 1px rgba(16, 22, 26, 0.1);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), inset 0 1px 1px rgba(16, 22, 26, 0.1);\n    cursor:-webkit-grabbing;\n    cursor:grabbing; }\n  .bp3-disabled .bp3-slider-handle{\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:#bfccd6;\n    pointer-events:none; }\n  .bp3-dark .bp3-slider-handle{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n    background-color:#394b59;\n    background-image:-webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.05)), to(rgba(255, 255, 255, 0)));\n    background-image:linear-gradient(to bottom, rgba(255, 255, 255, 0.05), rgba(255, 255, 255, 0));\n    color:#f5f8fa; }\n    .bp3-dark .bp3-slider-handle:hover, .bp3-dark .bp3-slider-handle:active, .bp3-dark .bp3-slider-handle.bp3-active{\n      color:#f5f8fa; }\n    .bp3-dark .bp3-slider-handle:hover{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.4);\n      background-color:#30404d; }\n    .bp3-dark .bp3-slider-handle:active, .bp3-dark .bp3-slider-handle.bp3-active{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.6), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.6), inset 0 1px 2px rgba(16, 22, 26, 0.2);\n      background-color:#202b33;\n      background-image:none; }\n    .bp3-dark .bp3-slider-handle:disabled, .bp3-dark .bp3-slider-handle.bp3-disabled{\n      -webkit-box-shadow:none;\n              box-shadow:none;\n      background-color:rgba(57, 75, 89, 0.5);\n      background-image:none;\n      color:rgba(167, 182, 194, 0.6); }\n      .bp3-dark .bp3-slider-handle:disabled.bp3-active, .bp3-dark .bp3-slider-handle.bp3-disabled.bp3-active{\n        background:rgba(57, 75, 89, 0.7); }\n    .bp3-dark .bp3-slider-handle .bp3-button-spinner .bp3-spinner-head{\n      background:rgba(16, 22, 26, 0.5);\n      stroke:#8a9ba8; }\n    .bp3-dark .bp3-slider-handle, .bp3-dark .bp3-slider-handle:hover{\n      background-color:#394b59; }\n    .bp3-dark .bp3-slider-handle.bp3-active{\n      background-color:#293742; }\n  .bp3-dark .bp3-disabled .bp3-slider-handle{\n    border-color:#5c7080;\n    -webkit-box-shadow:none;\n            box-shadow:none;\n    background:#5c7080; }\n  .bp3-slider-handle .bp3-slider-label{\n    margin-left:8px;\n    border-radius:3px;\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2);\n    background:#394b59;\n    color:#f5f8fa; }\n    .bp3-dark .bp3-slider-handle .bp3-slider-label{\n      -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4);\n              box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4);\n      background:#e1e8ed;\n      color:#394b59; }\n    .bp3-disabled .bp3-slider-handle .bp3-slider-label{\n      -webkit-box-shadow:none;\n              box-shadow:none; }\n  .bp3-slider-handle.bp3-start, .bp3-slider-handle.bp3-end{\n    width:8px; }\n  .bp3-slider-handle.bp3-start{\n    border-top-right-radius:0;\n    border-bottom-right-radius:0; }\n  .bp3-slider-handle.bp3-end{\n    margin-left:8px;\n    border-top-left-radius:0;\n    border-bottom-left-radius:0; }\n    .bp3-slider-handle.bp3-end .bp3-slider-label{\n      margin-left:0; }\n\n.bp3-slider-label{\n  -webkit-transform:translate(-50%, 20px);\n          transform:translate(-50%, 20px);\n  display:inline-block;\n  position:absolute;\n  padding:2px 5px;\n  vertical-align:top;\n  line-height:1;\n  font-size:12px; }\n\n.bp3-slider.bp3-vertical{\n  width:40px;\n  min-width:40px;\n  height:150px; }\n  .bp3-slider.bp3-vertical .bp3-slider-track,\n  .bp3-slider.bp3-vertical .bp3-slider-progress{\n    top:0;\n    bottom:0;\n    left:5px;\n    width:6px;\n    height:auto; }\n  .bp3-slider.bp3-vertical .bp3-slider-progress{\n    top:auto; }\n  .bp3-slider.bp3-vertical .bp3-slider-label{\n    -webkit-transform:translate(20px, 50%);\n            transform:translate(20px, 50%); }\n  .bp3-slider.bp3-vertical .bp3-slider-handle{\n    top:auto; }\n    .bp3-slider.bp3-vertical .bp3-slider-handle .bp3-slider-label{\n      margin-top:-8px;\n      margin-left:0; }\n    .bp3-slider.bp3-vertical .bp3-slider-handle.bp3-end, .bp3-slider.bp3-vertical .bp3-slider-handle.bp3-start{\n      margin-left:0;\n      width:16px;\n      height:8px; }\n    .bp3-slider.bp3-vertical .bp3-slider-handle.bp3-start{\n      border-top-left-radius:0;\n      border-bottom-right-radius:3px; }\n      .bp3-slider.bp3-vertical .bp3-slider-handle.bp3-start .bp3-slider-label{\n        -webkit-transform:translate(20px);\n                transform:translate(20px); }\n    .bp3-slider.bp3-vertical .bp3-slider-handle.bp3-end{\n      margin-bottom:8px;\n      border-top-left-radius:3px;\n      border-bottom-left-radius:0;\n      border-bottom-right-radius:0; }\n\n@-webkit-keyframes pt-spinner-animation{\n  from{\n    -webkit-transform:rotate(0deg);\n            transform:rotate(0deg); }\n  to{\n    -webkit-transform:rotate(360deg);\n            transform:rotate(360deg); } }\n\n@keyframes pt-spinner-animation{\n  from{\n    -webkit-transform:rotate(0deg);\n            transform:rotate(0deg); }\n  to{\n    -webkit-transform:rotate(360deg);\n            transform:rotate(360deg); } }\n\n.bp3-spinner{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  -webkit-box-pack:center;\n  -webkit-justify-content:center;\n      -ms-flex-pack:center;\n          justify-content:center;\n  overflow:visible;\n  vertical-align:middle; }\n  .bp3-spinner svg{\n    display:block; }\n  .bp3-spinner path{\n    fill-opacity:0; }\n  .bp3-spinner .bp3-spinner-head{\n    -webkit-transform-origin:center;\n            transform-origin:center;\n    -webkit-transition:stroke-dashoffset 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n    transition:stroke-dashoffset 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n    stroke:rgba(92, 112, 128, 0.8);\n    stroke-linecap:round; }\n  .bp3-spinner .bp3-spinner-track{\n    stroke:rgba(92, 112, 128, 0.2); }\n\n.bp3-spinner-animation{\n  -webkit-animation:pt-spinner-animation 500ms linear infinite;\n          animation:pt-spinner-animation 500ms linear infinite; }\n  .bp3-no-spin > .bp3-spinner-animation{\n    -webkit-animation:none;\n            animation:none; }\n\n.bp3-dark .bp3-spinner .bp3-spinner-head{\n  stroke:#8a9ba8; }\n\n.bp3-dark .bp3-spinner .bp3-spinner-track{\n  stroke:rgba(16, 22, 26, 0.5); }\n\n.bp3-spinner.bp3-intent-primary .bp3-spinner-head{\n  stroke:#137cbd; }\n\n.bp3-spinner.bp3-intent-success .bp3-spinner-head{\n  stroke:#0f9960; }\n\n.bp3-spinner.bp3-intent-warning .bp3-spinner-head{\n  stroke:#d9822b; }\n\n.bp3-spinner.bp3-intent-danger .bp3-spinner-head{\n  stroke:#db3737; }\n.bp3-tabs.bp3-vertical{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex; }\n  .bp3-tabs.bp3-vertical > .bp3-tab-list{\n    -webkit-box-orient:vertical;\n    -webkit-box-direction:normal;\n    -webkit-flex-direction:column;\n        -ms-flex-direction:column;\n            flex-direction:column;\n    -webkit-box-align:start;\n    -webkit-align-items:flex-start;\n        -ms-flex-align:start;\n            align-items:flex-start; }\n    .bp3-tabs.bp3-vertical > .bp3-tab-list .bp3-tab{\n      border-radius:3px;\n      width:100%;\n      padding:0 10px; }\n      .bp3-tabs.bp3-vertical > .bp3-tab-list .bp3-tab[aria-selected="true"]{\n        -webkit-box-shadow:none;\n                box-shadow:none;\n        background-color:rgba(19, 124, 189, 0.2); }\n    .bp3-tabs.bp3-vertical > .bp3-tab-list .bp3-tab-indicator-wrapper .bp3-tab-indicator{\n      top:0;\n      right:0;\n      bottom:0;\n      left:0;\n      border-radius:3px;\n      background-color:rgba(19, 124, 189, 0.2);\n      height:auto; }\n  .bp3-tabs.bp3-vertical > .bp3-tab-panel{\n    margin-top:0;\n    padding-left:20px; }\n\n.bp3-tab-list{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-flex:0;\n  -webkit-flex:0 0 auto;\n      -ms-flex:0 0 auto;\n          flex:0 0 auto;\n  -webkit-box-align:end;\n  -webkit-align-items:flex-end;\n      -ms-flex-align:end;\n          align-items:flex-end;\n  position:relative;\n  margin:0;\n  border:none;\n  padding:0;\n  list-style:none; }\n  .bp3-tab-list > *:not(:last-child){\n    margin-right:20px; }\n\n.bp3-tab{\n  overflow:hidden;\n  text-overflow:ellipsis;\n  white-space:nowrap;\n  word-wrap:normal;\n  -webkit-box-flex:0;\n  -webkit-flex:0 0 auto;\n      -ms-flex:0 0 auto;\n          flex:0 0 auto;\n  position:relative;\n  cursor:pointer;\n  max-width:100%;\n  vertical-align:top;\n  line-height:30px;\n  color:#182026;\n  font-size:14px; }\n  .bp3-tab a{\n    display:block;\n    text-decoration:none;\n    color:inherit; }\n  .bp3-tab-indicator-wrapper ~ .bp3-tab{\n    -webkit-box-shadow:none !important;\n            box-shadow:none !important;\n    background-color:transparent !important; }\n  .bp3-tab[aria-disabled="true"]{\n    cursor:not-allowed;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-tab[aria-selected="true"]{\n    border-radius:0;\n    -webkit-box-shadow:inset 0 -3px 0 #106ba3;\n            box-shadow:inset 0 -3px 0 #106ba3; }\n  .bp3-tab[aria-selected="true"], .bp3-tab:not([aria-disabled="true"]):hover{\n    color:#106ba3; }\n  .bp3-tab:focus{\n    -moz-outline-radius:0; }\n  .bp3-large > .bp3-tab{\n    line-height:40px;\n    font-size:16px; }\n\n.bp3-tab-panel{\n  margin-top:20px; }\n  .bp3-tab-panel[aria-hidden="true"]{\n    display:none; }\n\n.bp3-tab-indicator-wrapper{\n  position:absolute;\n  top:0;\n  left:0;\n  -webkit-transform:translateX(0), translateY(0);\n          transform:translateX(0), translateY(0);\n  -webkit-transition:height, width, -webkit-transform;\n  transition:height, width, -webkit-transform;\n  transition:height, transform, width;\n  transition:height, transform, width, -webkit-transform;\n  -webkit-transition-duration:200ms;\n          transition-duration:200ms;\n  -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n          transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n  pointer-events:none; }\n  .bp3-tab-indicator-wrapper .bp3-tab-indicator{\n    position:absolute;\n    right:0;\n    bottom:0;\n    left:0;\n    background-color:#106ba3;\n    height:3px; }\n  .bp3-tab-indicator-wrapper.bp3-no-animation{\n    -webkit-transition:none;\n    transition:none; }\n\n.bp3-dark .bp3-tab{\n  color:#f5f8fa; }\n  .bp3-dark .bp3-tab[aria-disabled="true"]{\n    color:rgba(167, 182, 194, 0.6); }\n  .bp3-dark .bp3-tab[aria-selected="true"]{\n    -webkit-box-shadow:inset 0 -3px 0 #48aff0;\n            box-shadow:inset 0 -3px 0 #48aff0; }\n  .bp3-dark .bp3-tab[aria-selected="true"], .bp3-dark .bp3-tab:not([aria-disabled="true"]):hover{\n    color:#48aff0; }\n\n.bp3-dark .bp3-tab-indicator{\n  background-color:#48aff0; }\n\n.bp3-flex-expander{\n  -webkit-box-flex:1;\n  -webkit-flex:1 1;\n      -ms-flex:1 1;\n          flex:1 1; }\n.bp3-tag{\n  display:-webkit-inline-box;\n  display:-webkit-inline-flex;\n  display:-ms-inline-flexbox;\n  display:inline-flex;\n  -webkit-box-orient:horizontal;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:row;\n      -ms-flex-direction:row;\n          flex-direction:row;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  position:relative;\n  border:none;\n  border-radius:3px;\n  -webkit-box-shadow:none;\n          box-shadow:none;\n  background-color:#5c7080;\n  min-width:20px;\n  max-width:100%;\n  min-height:20px;\n  padding:2px 6px;\n  line-height:16px;\n  color:#f5f8fa;\n  font-size:12px; }\n  .bp3-tag.bp3-interactive{\n    cursor:pointer; }\n    .bp3-tag.bp3-interactive:hover{\n      background-color:rgba(92, 112, 128, 0.85); }\n    .bp3-tag.bp3-interactive.bp3-active, .bp3-tag.bp3-interactive:active{\n      background-color:rgba(92, 112, 128, 0.7); }\n  .bp3-tag > *{\n    -webkit-box-flex:0;\n    -webkit-flex-grow:0;\n        -ms-flex-positive:0;\n            flex-grow:0;\n    -webkit-flex-shrink:0;\n        -ms-flex-negative:0;\n            flex-shrink:0; }\n  .bp3-tag > .bp3-fill{\n    -webkit-box-flex:1;\n    -webkit-flex-grow:1;\n        -ms-flex-positive:1;\n            flex-grow:1;\n    -webkit-flex-shrink:1;\n        -ms-flex-negative:1;\n            flex-shrink:1; }\n  .bp3-tag::before,\n  .bp3-tag > *{\n    margin-right:4px; }\n  .bp3-tag:empty::before,\n  .bp3-tag > :last-child{\n    margin-right:0; }\n  .bp3-tag:focus{\n    outline:rgba(19, 124, 189, 0.6) auto 2px;\n    outline-offset:0;\n    -moz-outline-radius:6px; }\n  .bp3-tag.bp3-round{\n    border-radius:30px;\n    padding-right:8px;\n    padding-left:8px; }\n  .bp3-dark .bp3-tag{\n    background-color:#bfccd6;\n    color:#182026; }\n    .bp3-dark .bp3-tag.bp3-interactive{\n      cursor:pointer; }\n      .bp3-dark .bp3-tag.bp3-interactive:hover{\n        background-color:rgba(191, 204, 214, 0.85); }\n      .bp3-dark .bp3-tag.bp3-interactive.bp3-active, .bp3-dark .bp3-tag.bp3-interactive:active{\n        background-color:rgba(191, 204, 214, 0.7); }\n    .bp3-dark .bp3-tag > .bp3-icon, .bp3-dark .bp3-tag .bp3-icon-standard, .bp3-dark .bp3-tag .bp3-icon-large{\n      fill:currentColor; }\n  .bp3-tag > .bp3-icon, .bp3-tag .bp3-icon-standard, .bp3-tag .bp3-icon-large{\n    fill:#ffffff; }\n  .bp3-tag.bp3-large,\n  .bp3-large .bp3-tag{\n    min-width:30px;\n    min-height:30px;\n    padding:0 10px;\n    line-height:20px;\n    font-size:14px; }\n    .bp3-tag.bp3-large::before,\n    .bp3-tag.bp3-large > *,\n    .bp3-large .bp3-tag::before,\n    .bp3-large .bp3-tag > *{\n      margin-right:7px; }\n    .bp3-tag.bp3-large:empty::before,\n    .bp3-tag.bp3-large > :last-child,\n    .bp3-large .bp3-tag:empty::before,\n    .bp3-large .bp3-tag > :last-child{\n      margin-right:0; }\n    .bp3-tag.bp3-large.bp3-round,\n    .bp3-large .bp3-tag.bp3-round{\n      padding-right:12px;\n      padding-left:12px; }\n  .bp3-tag.bp3-intent-primary{\n    background:#137cbd;\n    color:#ffffff; }\n    .bp3-tag.bp3-intent-primary.bp3-interactive{\n      cursor:pointer; }\n      .bp3-tag.bp3-intent-primary.bp3-interactive:hover{\n        background-color:rgba(19, 124, 189, 0.85); }\n      .bp3-tag.bp3-intent-primary.bp3-interactive.bp3-active, .bp3-tag.bp3-intent-primary.bp3-interactive:active{\n        background-color:rgba(19, 124, 189, 0.7); }\n  .bp3-tag.bp3-intent-success{\n    background:#0f9960;\n    color:#ffffff; }\n    .bp3-tag.bp3-intent-success.bp3-interactive{\n      cursor:pointer; }\n      .bp3-tag.bp3-intent-success.bp3-interactive:hover{\n        background-color:rgba(15, 153, 96, 0.85); }\n      .bp3-tag.bp3-intent-success.bp3-interactive.bp3-active, .bp3-tag.bp3-intent-success.bp3-interactive:active{\n        background-color:rgba(15, 153, 96, 0.7); }\n  .bp3-tag.bp3-intent-warning{\n    background:#d9822b;\n    color:#ffffff; }\n    .bp3-tag.bp3-intent-warning.bp3-interactive{\n      cursor:pointer; }\n      .bp3-tag.bp3-intent-warning.bp3-interactive:hover{\n        background-color:rgba(217, 130, 43, 0.85); }\n      .bp3-tag.bp3-intent-warning.bp3-interactive.bp3-active, .bp3-tag.bp3-intent-warning.bp3-interactive:active{\n        background-color:rgba(217, 130, 43, 0.7); }\n  .bp3-tag.bp3-intent-danger{\n    background:#db3737;\n    color:#ffffff; }\n    .bp3-tag.bp3-intent-danger.bp3-interactive{\n      cursor:pointer; }\n      .bp3-tag.bp3-intent-danger.bp3-interactive:hover{\n        background-color:rgba(219, 55, 55, 0.85); }\n      .bp3-tag.bp3-intent-danger.bp3-interactive.bp3-active, .bp3-tag.bp3-intent-danger.bp3-interactive:active{\n        background-color:rgba(219, 55, 55, 0.7); }\n  .bp3-tag.bp3-fill{\n    display:-webkit-box;\n    display:-webkit-flex;\n    display:-ms-flexbox;\n    display:flex;\n    width:100%; }\n  .bp3-tag.bp3-minimal > .bp3-icon, .bp3-tag.bp3-minimal .bp3-icon-standard, .bp3-tag.bp3-minimal .bp3-icon-large{\n    fill:#5c7080; }\n  .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]){\n    background-color:rgba(138, 155, 168, 0.2);\n    color:#182026; }\n    .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]).bp3-interactive{\n      cursor:pointer; }\n      .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]).bp3-interactive:hover{\n        background-color:rgba(92, 112, 128, 0.3); }\n      .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]).bp3-interactive.bp3-active, .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]).bp3-interactive:active{\n        background-color:rgba(92, 112, 128, 0.4); }\n    .bp3-dark .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]){\n      color:#f5f8fa; }\n      .bp3-dark .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]).bp3-interactive{\n        cursor:pointer; }\n        .bp3-dark .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]).bp3-interactive:hover{\n          background-color:rgba(191, 204, 214, 0.3); }\n        .bp3-dark .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]).bp3-interactive.bp3-active, .bp3-dark .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]).bp3-interactive:active{\n          background-color:rgba(191, 204, 214, 0.4); }\n      .bp3-dark .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]) > .bp3-icon, .bp3-dark .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]) .bp3-icon-standard, .bp3-dark .bp3-tag.bp3-minimal:not([class*="bp3-intent-"]) .bp3-icon-large{\n        fill:#a7b6c2; }\n  .bp3-tag.bp3-minimal.bp3-intent-primary{\n    background-color:rgba(19, 124, 189, 0.15);\n    color:#106ba3; }\n    .bp3-tag.bp3-minimal.bp3-intent-primary.bp3-interactive{\n      cursor:pointer; }\n      .bp3-tag.bp3-minimal.bp3-intent-primary.bp3-interactive:hover{\n        background-color:rgba(19, 124, 189, 0.25); }\n      .bp3-tag.bp3-minimal.bp3-intent-primary.bp3-interactive.bp3-active, .bp3-tag.bp3-minimal.bp3-intent-primary.bp3-interactive:active{\n        background-color:rgba(19, 124, 189, 0.35); }\n    .bp3-tag.bp3-minimal.bp3-intent-primary > .bp3-icon, .bp3-tag.bp3-minimal.bp3-intent-primary .bp3-icon-standard, .bp3-tag.bp3-minimal.bp3-intent-primary .bp3-icon-large{\n      fill:#137cbd; }\n    .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-primary{\n      background-color:rgba(19, 124, 189, 0.25);\n      color:#48aff0; }\n      .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-primary.bp3-interactive{\n        cursor:pointer; }\n        .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-primary.bp3-interactive:hover{\n          background-color:rgba(19, 124, 189, 0.35); }\n        .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-primary.bp3-interactive.bp3-active, .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-primary.bp3-interactive:active{\n          background-color:rgba(19, 124, 189, 0.45); }\n  .bp3-tag.bp3-minimal.bp3-intent-success{\n    background-color:rgba(15, 153, 96, 0.15);\n    color:#0d8050; }\n    .bp3-tag.bp3-minimal.bp3-intent-success.bp3-interactive{\n      cursor:pointer; }\n      .bp3-tag.bp3-minimal.bp3-intent-success.bp3-interactive:hover{\n        background-color:rgba(15, 153, 96, 0.25); }\n      .bp3-tag.bp3-minimal.bp3-intent-success.bp3-interactive.bp3-active, .bp3-tag.bp3-minimal.bp3-intent-success.bp3-interactive:active{\n        background-color:rgba(15, 153, 96, 0.35); }\n    .bp3-tag.bp3-minimal.bp3-intent-success > .bp3-icon, .bp3-tag.bp3-minimal.bp3-intent-success .bp3-icon-standard, .bp3-tag.bp3-minimal.bp3-intent-success .bp3-icon-large{\n      fill:#0f9960; }\n    .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-success{\n      background-color:rgba(15, 153, 96, 0.25);\n      color:#3dcc91; }\n      .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-success.bp3-interactive{\n        cursor:pointer; }\n        .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-success.bp3-interactive:hover{\n          background-color:rgba(15, 153, 96, 0.35); }\n        .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-success.bp3-interactive.bp3-active, .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-success.bp3-interactive:active{\n          background-color:rgba(15, 153, 96, 0.45); }\n  .bp3-tag.bp3-minimal.bp3-intent-warning{\n    background-color:rgba(217, 130, 43, 0.15);\n    color:#bf7326; }\n    .bp3-tag.bp3-minimal.bp3-intent-warning.bp3-interactive{\n      cursor:pointer; }\n      .bp3-tag.bp3-minimal.bp3-intent-warning.bp3-interactive:hover{\n        background-color:rgba(217, 130, 43, 0.25); }\n      .bp3-tag.bp3-minimal.bp3-intent-warning.bp3-interactive.bp3-active, .bp3-tag.bp3-minimal.bp3-intent-warning.bp3-interactive:active{\n        background-color:rgba(217, 130, 43, 0.35); }\n    .bp3-tag.bp3-minimal.bp3-intent-warning > .bp3-icon, .bp3-tag.bp3-minimal.bp3-intent-warning .bp3-icon-standard, .bp3-tag.bp3-minimal.bp3-intent-warning .bp3-icon-large{\n      fill:#d9822b; }\n    .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-warning{\n      background-color:rgba(217, 130, 43, 0.25);\n      color:#ffb366; }\n      .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-warning.bp3-interactive{\n        cursor:pointer; }\n        .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-warning.bp3-interactive:hover{\n          background-color:rgba(217, 130, 43, 0.35); }\n        .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-warning.bp3-interactive.bp3-active, .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-warning.bp3-interactive:active{\n          background-color:rgba(217, 130, 43, 0.45); }\n  .bp3-tag.bp3-minimal.bp3-intent-danger{\n    background-color:rgba(219, 55, 55, 0.15);\n    color:#c23030; }\n    .bp3-tag.bp3-minimal.bp3-intent-danger.bp3-interactive{\n      cursor:pointer; }\n      .bp3-tag.bp3-minimal.bp3-intent-danger.bp3-interactive:hover{\n        background-color:rgba(219, 55, 55, 0.25); }\n      .bp3-tag.bp3-minimal.bp3-intent-danger.bp3-interactive.bp3-active, .bp3-tag.bp3-minimal.bp3-intent-danger.bp3-interactive:active{\n        background-color:rgba(219, 55, 55, 0.35); }\n    .bp3-tag.bp3-minimal.bp3-intent-danger > .bp3-icon, .bp3-tag.bp3-minimal.bp3-intent-danger .bp3-icon-standard, .bp3-tag.bp3-minimal.bp3-intent-danger .bp3-icon-large{\n      fill:#db3737; }\n    .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-danger{\n      background-color:rgba(219, 55, 55, 0.25);\n      color:#ff7373; }\n      .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-danger.bp3-interactive{\n        cursor:pointer; }\n        .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-danger.bp3-interactive:hover{\n          background-color:rgba(219, 55, 55, 0.35); }\n        .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-danger.bp3-interactive.bp3-active, .bp3-dark .bp3-tag.bp3-minimal.bp3-intent-danger.bp3-interactive:active{\n          background-color:rgba(219, 55, 55, 0.45); }\n\n.bp3-tag-remove{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  opacity:0.5;\n  margin-top:-2px;\n  margin-right:-6px !important;\n  margin-bottom:-2px;\n  border:none;\n  background:none;\n  cursor:pointer;\n  padding:2px;\n  padding-left:0;\n  color:inherit; }\n  .bp3-tag-remove:hover{\n    opacity:0.8;\n    background:none;\n    text-decoration:none; }\n  .bp3-tag-remove:active{\n    opacity:1; }\n  .bp3-tag-remove:empty::before{\n    line-height:1;\n    font-family:"Icons16", sans-serif;\n    font-size:16px;\n    font-weight:400;\n    font-style:normal;\n    -moz-osx-font-smoothing:grayscale;\n    -webkit-font-smoothing:antialiased;\n    content:"\\E6D7"; }\n  .bp3-large .bp3-tag-remove{\n    margin-right:-10px !important;\n    padding:5px;\n    padding-left:0; }\n    .bp3-large .bp3-tag-remove:empty::before{\n      line-height:1;\n      font-family:"Icons20", sans-serif;\n      font-size:20px;\n      font-weight:400;\n      font-style:normal; }\n.bp3-tag-input{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-orient:horizontal;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:row;\n      -ms-flex-direction:row;\n          flex-direction:row;\n  -webkit-box-align:start;\n  -webkit-align-items:flex-start;\n      -ms-flex-align:start;\n          align-items:flex-start;\n  cursor:text;\n  height:auto;\n  min-height:30px;\n  padding-right:0;\n  padding-left:5px;\n  line-height:inherit; }\n  .bp3-tag-input > *{\n    -webkit-box-flex:0;\n    -webkit-flex-grow:0;\n        -ms-flex-positive:0;\n            flex-grow:0;\n    -webkit-flex-shrink:0;\n        -ms-flex-negative:0;\n            flex-shrink:0; }\n  .bp3-tag-input > .bp3-tag-input-values{\n    -webkit-box-flex:1;\n    -webkit-flex-grow:1;\n        -ms-flex-positive:1;\n            flex-grow:1;\n    -webkit-flex-shrink:1;\n        -ms-flex-negative:1;\n            flex-shrink:1; }\n  .bp3-tag-input .bp3-tag-input-icon{\n    margin-top:7px;\n    margin-right:7px;\n    margin-left:2px;\n    color:#5c7080; }\n  .bp3-tag-input .bp3-tag-input-values{\n    display:-webkit-box;\n    display:-webkit-flex;\n    display:-ms-flexbox;\n    display:flex;\n    -webkit-box-orient:horizontal;\n    -webkit-box-direction:normal;\n    -webkit-flex-direction:row;\n        -ms-flex-direction:row;\n            flex-direction:row;\n    -webkit-flex-wrap:wrap;\n        -ms-flex-wrap:wrap;\n            flex-wrap:wrap;\n    -webkit-box-align:center;\n    -webkit-align-items:center;\n        -ms-flex-align:center;\n            align-items:center;\n    -webkit-align-self:stretch;\n        -ms-flex-item-align:stretch;\n            align-self:stretch;\n    margin-top:5px;\n    margin-right:7px;\n    min-width:0; }\n    .bp3-tag-input .bp3-tag-input-values > *{\n      -webkit-box-flex:0;\n      -webkit-flex-grow:0;\n          -ms-flex-positive:0;\n              flex-grow:0;\n      -webkit-flex-shrink:0;\n          -ms-flex-negative:0;\n              flex-shrink:0; }\n    .bp3-tag-input .bp3-tag-input-values > .bp3-fill{\n      -webkit-box-flex:1;\n      -webkit-flex-grow:1;\n          -ms-flex-positive:1;\n              flex-grow:1;\n      -webkit-flex-shrink:1;\n          -ms-flex-negative:1;\n              flex-shrink:1; }\n    .bp3-tag-input .bp3-tag-input-values::before,\n    .bp3-tag-input .bp3-tag-input-values > *{\n      margin-right:5px; }\n    .bp3-tag-input .bp3-tag-input-values:empty::before,\n    .bp3-tag-input .bp3-tag-input-values > :last-child{\n      margin-right:0; }\n    .bp3-tag-input .bp3-tag-input-values:first-child .bp3-input-ghost:first-child{\n      padding-left:5px; }\n    .bp3-tag-input .bp3-tag-input-values > *{\n      margin-bottom:5px; }\n  .bp3-tag-input .bp3-tag{\n    overflow-wrap:break-word; }\n    .bp3-tag-input .bp3-tag.bp3-active{\n      outline:rgba(19, 124, 189, 0.6) auto 2px;\n      outline-offset:0;\n      -moz-outline-radius:6px; }\n  .bp3-tag-input .bp3-input-ghost{\n    -webkit-box-flex:1;\n    -webkit-flex:1 1 auto;\n        -ms-flex:1 1 auto;\n            flex:1 1 auto;\n    width:80px;\n    line-height:20px; }\n    .bp3-tag-input .bp3-input-ghost:disabled, .bp3-tag-input .bp3-input-ghost.bp3-disabled{\n      cursor:not-allowed; }\n  .bp3-tag-input .bp3-button,\n  .bp3-tag-input .bp3-spinner{\n    margin:3px;\n    margin-left:0; }\n  .bp3-tag-input .bp3-button{\n    min-width:24px;\n    min-height:24px;\n    padding:0 7px; }\n  .bp3-tag-input.bp3-large{\n    height:auto;\n    min-height:40px; }\n    .bp3-tag-input.bp3-large::before,\n    .bp3-tag-input.bp3-large > *{\n      margin-right:10px; }\n    .bp3-tag-input.bp3-large:empty::before,\n    .bp3-tag-input.bp3-large > :last-child{\n      margin-right:0; }\n    .bp3-tag-input.bp3-large .bp3-tag-input-icon{\n      margin-top:10px;\n      margin-left:5px; }\n    .bp3-tag-input.bp3-large .bp3-input-ghost{\n      line-height:30px; }\n    .bp3-tag-input.bp3-large .bp3-button{\n      min-width:30px;\n      min-height:30px;\n      padding:5px 10px;\n      margin:5px;\n      margin-left:0; }\n    .bp3-tag-input.bp3-large .bp3-spinner{\n      margin:8px;\n      margin-left:0; }\n  .bp3-tag-input.bp3-active{\n    -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n            box-shadow:0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.2);\n    background-color:#ffffff; }\n  .bp3-dark .bp3-tag-input .bp3-tag-input-icon, .bp3-tag-input.bp3-dark .bp3-tag-input-icon{\n    color:#a7b6c2; }\n  .bp3-dark .bp3-tag-input .bp3-input-ghost, .bp3-tag-input.bp3-dark .bp3-input-ghost{\n    color:#f5f8fa; }\n    .bp3-dark .bp3-tag-input .bp3-input-ghost::-webkit-input-placeholder, .bp3-tag-input.bp3-dark .bp3-input-ghost::-webkit-input-placeholder{\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark .bp3-tag-input .bp3-input-ghost::-ms-input-placeholder, .bp3-tag-input.bp3-dark .bp3-input-ghost::-ms-input-placeholder{\n      color:rgba(167, 182, 194, 0.6); }\n    .bp3-dark .bp3-tag-input .bp3-input-ghost::placeholder, .bp3-tag-input.bp3-dark .bp3-input-ghost::placeholder{\n      color:rgba(167, 182, 194, 0.6); }\n  .bp3-dark .bp3-tag-input.bp3-active, .bp3-tag-input.bp3-dark.bp3-active{\n    -webkit-box-shadow:0 0 0 1px #137cbd, 0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px #137cbd, 0 0 0 1px #137cbd, 0 0 0 3px rgba(19, 124, 189, 0.3), inset 0 0 0 1px rgba(16, 22, 26, 0.3), inset 0 1px 1px rgba(16, 22, 26, 0.4);\n    background-color:rgba(16, 22, 26, 0.3); }\n\n.bp3-input-ghost{\n  border:none;\n  -webkit-box-shadow:none;\n          box-shadow:none;\n  background:none;\n  padding:0; }\n  .bp3-input-ghost::-webkit-input-placeholder{\n    opacity:1;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-input-ghost::-ms-input-placeholder{\n    opacity:1;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-input-ghost::placeholder{\n    opacity:1;\n    color:rgba(92, 112, 128, 0.6); }\n  .bp3-input-ghost:focus{\n    outline:none !important; }\n.bp3-toast{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-align:start;\n  -webkit-align-items:flex-start;\n      -ms-flex-align:start;\n          align-items:flex-start;\n  position:relative !important;\n  margin:20px 0 0;\n  border-radius:3px;\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2);\n  background-color:#ffffff;\n  min-width:300px;\n  max-width:500px;\n  pointer-events:all; }\n  .bp3-toast.bp3-toast-enter, .bp3-toast.bp3-toast-appear{\n    -webkit-transform:translateY(-40px);\n            transform:translateY(-40px); }\n  .bp3-toast.bp3-toast-enter-active, .bp3-toast.bp3-toast-appear-active{\n    -webkit-transform:translateY(0);\n            transform:translateY(0);\n    -webkit-transition-property:-webkit-transform;\n    transition-property:-webkit-transform;\n    transition-property:transform;\n    transition-property:transform, -webkit-transform;\n    -webkit-transition-duration:300ms;\n            transition-duration:300ms;\n    -webkit-transition-timing-function:cubic-bezier(0.54, 1.12, 0.38, 1.11);\n            transition-timing-function:cubic-bezier(0.54, 1.12, 0.38, 1.11);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n  .bp3-toast.bp3-toast-enter ~ .bp3-toast, .bp3-toast.bp3-toast-appear ~ .bp3-toast{\n    -webkit-transform:translateY(-40px);\n            transform:translateY(-40px); }\n  .bp3-toast.bp3-toast-enter-active ~ .bp3-toast, .bp3-toast.bp3-toast-appear-active ~ .bp3-toast{\n    -webkit-transform:translateY(0);\n            transform:translateY(0);\n    -webkit-transition-property:-webkit-transform;\n    transition-property:-webkit-transform;\n    transition-property:transform;\n    transition-property:transform, -webkit-transform;\n    -webkit-transition-duration:300ms;\n            transition-duration:300ms;\n    -webkit-transition-timing-function:cubic-bezier(0.54, 1.12, 0.38, 1.11);\n            transition-timing-function:cubic-bezier(0.54, 1.12, 0.38, 1.11);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n  .bp3-toast.bp3-toast-exit{\n    opacity:1;\n    -webkit-filter:blur(0);\n            filter:blur(0); }\n  .bp3-toast.bp3-toast-exit-active{\n    opacity:0;\n    -webkit-filter:blur(10px);\n            filter:blur(10px);\n    -webkit-transition-property:opacity, -webkit-filter;\n    transition-property:opacity, -webkit-filter;\n    transition-property:opacity, filter;\n    transition-property:opacity, filter, -webkit-filter;\n    -webkit-transition-duration:300ms;\n            transition-duration:300ms;\n    -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n            transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n  .bp3-toast.bp3-toast-exit ~ .bp3-toast{\n    -webkit-transform:translateY(0);\n            transform:translateY(0); }\n  .bp3-toast.bp3-toast-exit-active ~ .bp3-toast{\n    -webkit-transform:translateY(-40px);\n            transform:translateY(-40px);\n    -webkit-transition-property:-webkit-transform;\n    transition-property:-webkit-transform;\n    transition-property:transform;\n    transition-property:transform, -webkit-transform;\n    -webkit-transition-duration:100ms;\n            transition-duration:100ms;\n    -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n            transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n    -webkit-transition-delay:50ms;\n            transition-delay:50ms; }\n  .bp3-toast .bp3-button-group{\n    -webkit-box-flex:0;\n    -webkit-flex:0 0 auto;\n        -ms-flex:0 0 auto;\n            flex:0 0 auto;\n    padding:5px;\n    padding-left:0; }\n  .bp3-toast > .bp3-icon{\n    margin:12px;\n    margin-right:0;\n    color:#5c7080; }\n  .bp3-toast.bp3-dark,\n  .bp3-dark .bp3-toast{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4);\n    background-color:#394b59; }\n    .bp3-toast.bp3-dark > .bp3-icon,\n    .bp3-dark .bp3-toast > .bp3-icon{\n      color:#a7b6c2; }\n  .bp3-toast[class*="bp3-intent-"] a{\n    color:rgba(255, 255, 255, 0.7); }\n    .bp3-toast[class*="bp3-intent-"] a:hover{\n      color:#ffffff; }\n  .bp3-toast[class*="bp3-intent-"] > .bp3-icon{\n    color:#ffffff; }\n  .bp3-toast[class*="bp3-intent-"] .bp3-button, .bp3-toast[class*="bp3-intent-"] .bp3-button::before,\n  .bp3-toast[class*="bp3-intent-"] .bp3-button .bp3-icon, .bp3-toast[class*="bp3-intent-"] .bp3-button:active{\n    color:rgba(255, 255, 255, 0.7) !important; }\n  .bp3-toast[class*="bp3-intent-"] .bp3-button:focus{\n    outline-color:rgba(255, 255, 255, 0.5); }\n  .bp3-toast[class*="bp3-intent-"] .bp3-button:hover{\n    background-color:rgba(255, 255, 255, 0.15) !important;\n    color:#ffffff !important; }\n  .bp3-toast[class*="bp3-intent-"] .bp3-button:active{\n    background-color:rgba(255, 255, 255, 0.3) !important;\n    color:#ffffff !important; }\n  .bp3-toast[class*="bp3-intent-"] .bp3-button::after{\n    background:rgba(255, 255, 255, 0.3) !important; }\n  .bp3-toast.bp3-intent-primary{\n    background-color:#137cbd;\n    color:#ffffff; }\n  .bp3-toast.bp3-intent-success{\n    background-color:#0f9960;\n    color:#ffffff; }\n  .bp3-toast.bp3-intent-warning{\n    background-color:#d9822b;\n    color:#ffffff; }\n  .bp3-toast.bp3-intent-danger{\n    background-color:#db3737;\n    color:#ffffff; }\n\n.bp3-toast-message{\n  -webkit-box-flex:1;\n  -webkit-flex:1 1 auto;\n      -ms-flex:1 1 auto;\n          flex:1 1 auto;\n  padding:11px;\n  word-break:break-word; }\n\n.bp3-toast-container{\n  display:-webkit-box !important;\n  display:-webkit-flex !important;\n  display:-ms-flexbox !important;\n  display:flex !important;\n  -webkit-box-orient:vertical;\n  -webkit-box-direction:normal;\n  -webkit-flex-direction:column;\n      -ms-flex-direction:column;\n          flex-direction:column;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  position:fixed;\n  right:0;\n  left:0;\n  z-index:40;\n  overflow:hidden;\n  padding:0 20px 20px;\n  pointer-events:none; }\n  .bp3-toast-container.bp3-toast-container-top{\n    top:0;\n    bottom:auto; }\n  .bp3-toast-container.bp3-toast-container-bottom{\n    -webkit-box-orient:vertical;\n    -webkit-box-direction:reverse;\n    -webkit-flex-direction:column-reverse;\n        -ms-flex-direction:column-reverse;\n            flex-direction:column-reverse;\n    top:auto;\n    bottom:0; }\n  .bp3-toast-container.bp3-toast-container-left{\n    -webkit-box-align:start;\n    -webkit-align-items:flex-start;\n        -ms-flex-align:start;\n            align-items:flex-start; }\n  .bp3-toast-container.bp3-toast-container-right{\n    -webkit-box-align:end;\n    -webkit-align-items:flex-end;\n        -ms-flex-align:end;\n            align-items:flex-end; }\n\n.bp3-toast-container-bottom .bp3-toast.bp3-toast-enter:not(.bp3-toast-enter-active),\n.bp3-toast-container-bottom .bp3-toast.bp3-toast-enter:not(.bp3-toast-enter-active) ~ .bp3-toast,\n.bp3-toast-container-bottom .bp3-toast.bp3-toast-leave-active ~ .bp3-toast{\n  -webkit-transform:translateY(60px);\n          transform:translateY(60px); }\n.bp3-tooltip{\n  -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2);\n          box-shadow:0 0 0 1px rgba(16, 22, 26, 0.1), 0 2px 4px rgba(16, 22, 26, 0.2), 0 8px 24px rgba(16, 22, 26, 0.2);\n  -webkit-transform:scale(1);\n          transform:scale(1); }\n  .bp3-tooltip .bp3-popover-arrow{\n    position:absolute;\n    width:22px;\n    height:22px; }\n    .bp3-tooltip .bp3-popover-arrow::before{\n      margin:4px;\n      width:14px;\n      height:14px; }\n  .bp3-tether-element-attached-bottom.bp3-tether-target-attached-top > .bp3-tooltip{\n    margin-top:-11px;\n    margin-bottom:11px; }\n    .bp3-tether-element-attached-bottom.bp3-tether-target-attached-top > .bp3-tooltip > .bp3-popover-arrow{\n      bottom:-8px; }\n      .bp3-tether-element-attached-bottom.bp3-tether-target-attached-top > .bp3-tooltip > .bp3-popover-arrow svg{\n        -webkit-transform:rotate(-90deg);\n                transform:rotate(-90deg); }\n  .bp3-tether-element-attached-left.bp3-tether-target-attached-right > .bp3-tooltip{\n    margin-left:11px; }\n    .bp3-tether-element-attached-left.bp3-tether-target-attached-right > .bp3-tooltip > .bp3-popover-arrow{\n      left:-8px; }\n      .bp3-tether-element-attached-left.bp3-tether-target-attached-right > .bp3-tooltip > .bp3-popover-arrow svg{\n        -webkit-transform:rotate(0);\n                transform:rotate(0); }\n  .bp3-tether-element-attached-top.bp3-tether-target-attached-bottom > .bp3-tooltip{\n    margin-top:11px; }\n    .bp3-tether-element-attached-top.bp3-tether-target-attached-bottom > .bp3-tooltip > .bp3-popover-arrow{\n      top:-8px; }\n      .bp3-tether-element-attached-top.bp3-tether-target-attached-bottom > .bp3-tooltip > .bp3-popover-arrow svg{\n        -webkit-transform:rotate(90deg);\n                transform:rotate(90deg); }\n  .bp3-tether-element-attached-right.bp3-tether-target-attached-left > .bp3-tooltip{\n    margin-right:11px;\n    margin-left:-11px; }\n    .bp3-tether-element-attached-right.bp3-tether-target-attached-left > .bp3-tooltip > .bp3-popover-arrow{\n      right:-8px; }\n      .bp3-tether-element-attached-right.bp3-tether-target-attached-left > .bp3-tooltip > .bp3-popover-arrow svg{\n        -webkit-transform:rotate(180deg);\n                transform:rotate(180deg); }\n  .bp3-tether-element-attached-middle > .bp3-tooltip > .bp3-popover-arrow{\n    top:50%;\n    -webkit-transform:translateY(-50%);\n            transform:translateY(-50%); }\n  .bp3-tether-element-attached-center > .bp3-tooltip > .bp3-popover-arrow{\n    right:50%;\n    -webkit-transform:translateX(50%);\n            transform:translateX(50%); }\n  .bp3-tether-element-attached-top.bp3-tether-target-attached-top > .bp3-tooltip > .bp3-popover-arrow{\n    top:-0.22183px; }\n  .bp3-tether-element-attached-right.bp3-tether-target-attached-right > .bp3-tooltip > .bp3-popover-arrow{\n    right:-0.22183px; }\n  .bp3-tether-element-attached-left.bp3-tether-target-attached-left > .bp3-tooltip > .bp3-popover-arrow{\n    left:-0.22183px; }\n  .bp3-tether-element-attached-bottom.bp3-tether-target-attached-bottom > .bp3-tooltip > .bp3-popover-arrow{\n    bottom:-0.22183px; }\n  .bp3-tether-element-attached-top.bp3-tether-element-attached-left > .bp3-tooltip{\n    -webkit-transform-origin:top left;\n            transform-origin:top left; }\n  .bp3-tether-element-attached-top.bp3-tether-element-attached-center > .bp3-tooltip{\n    -webkit-transform-origin:top center;\n            transform-origin:top center; }\n  .bp3-tether-element-attached-top.bp3-tether-element-attached-right > .bp3-tooltip{\n    -webkit-transform-origin:top right;\n            transform-origin:top right; }\n  .bp3-tether-element-attached-middle.bp3-tether-element-attached-left > .bp3-tooltip{\n    -webkit-transform-origin:center left;\n            transform-origin:center left; }\n  .bp3-tether-element-attached-middle.bp3-tether-element-attached-center > .bp3-tooltip{\n    -webkit-transform-origin:center center;\n            transform-origin:center center; }\n  .bp3-tether-element-attached-middle.bp3-tether-element-attached-right > .bp3-tooltip{\n    -webkit-transform-origin:center right;\n            transform-origin:center right; }\n  .bp3-tether-element-attached-bottom.bp3-tether-element-attached-left > .bp3-tooltip{\n    -webkit-transform-origin:bottom left;\n            transform-origin:bottom left; }\n  .bp3-tether-element-attached-bottom.bp3-tether-element-attached-center > .bp3-tooltip{\n    -webkit-transform-origin:bottom center;\n            transform-origin:bottom center; }\n  .bp3-tether-element-attached-bottom.bp3-tether-element-attached-right > .bp3-tooltip{\n    -webkit-transform-origin:bottom right;\n            transform-origin:bottom right; }\n  .bp3-tooltip .bp3-popover-content{\n    background:#394b59;\n    color:#f5f8fa; }\n  .bp3-tooltip .bp3-popover-arrow::before{\n    -webkit-box-shadow:1px 1px 6px rgba(16, 22, 26, 0.2);\n            box-shadow:1px 1px 6px rgba(16, 22, 26, 0.2); }\n  .bp3-tooltip .bp3-popover-arrow-border{\n    fill:#10161a;\n    fill-opacity:0.1; }\n  .bp3-tooltip .bp3-popover-arrow-fill{\n    fill:#394b59; }\n  .bp3-popover-enter > .bp3-tooltip, .bp3-popover-appear > .bp3-tooltip{\n    -webkit-transform:scale(0.8);\n            transform:scale(0.8); }\n  .bp3-popover-enter-active > .bp3-tooltip, .bp3-popover-appear-active > .bp3-tooltip{\n    -webkit-transform:scale(1);\n            transform:scale(1);\n    -webkit-transition-property:-webkit-transform;\n    transition-property:-webkit-transform;\n    transition-property:transform;\n    transition-property:transform, -webkit-transform;\n    -webkit-transition-duration:100ms;\n            transition-duration:100ms;\n    -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n            transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n  .bp3-popover-exit > .bp3-tooltip{\n    -webkit-transform:scale(1);\n            transform:scale(1); }\n  .bp3-popover-exit-active > .bp3-tooltip{\n    -webkit-transform:scale(0.8);\n            transform:scale(0.8);\n    -webkit-transition-property:-webkit-transform;\n    transition-property:-webkit-transform;\n    transition-property:transform;\n    transition-property:transform, -webkit-transform;\n    -webkit-transition-duration:100ms;\n            transition-duration:100ms;\n    -webkit-transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n            transition-timing-function:cubic-bezier(0.4, 1, 0.75, 0.9);\n    -webkit-transition-delay:0;\n            transition-delay:0; }\n  .bp3-tooltip .bp3-popover-content{\n    padding:10px 12px; }\n  .bp3-tooltip.bp3-dark,\n  .bp3-dark .bp3-tooltip{\n    -webkit-box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4);\n            box-shadow:0 0 0 1px rgba(16, 22, 26, 0.2), 0 2px 4px rgba(16, 22, 26, 0.4), 0 8px 24px rgba(16, 22, 26, 0.4); }\n    .bp3-tooltip.bp3-dark .bp3-popover-content,\n    .bp3-dark .bp3-tooltip .bp3-popover-content{\n      background:#e1e8ed;\n      color:#394b59; }\n    .bp3-tooltip.bp3-dark .bp3-popover-arrow::before,\n    .bp3-dark .bp3-tooltip .bp3-popover-arrow::before{\n      -webkit-box-shadow:1px 1px 6px rgba(16, 22, 26, 0.4);\n              box-shadow:1px 1px 6px rgba(16, 22, 26, 0.4); }\n    .bp3-tooltip.bp3-dark .bp3-popover-arrow-border,\n    .bp3-dark .bp3-tooltip .bp3-popover-arrow-border{\n      fill:#10161a;\n      fill-opacity:0.2; }\n    .bp3-tooltip.bp3-dark .bp3-popover-arrow-fill,\n    .bp3-dark .bp3-tooltip .bp3-popover-arrow-fill{\n      fill:#e1e8ed; }\n  .bp3-tooltip.bp3-intent-primary .bp3-popover-content{\n    background:#137cbd;\n    color:#ffffff; }\n  .bp3-tooltip.bp3-intent-primary .bp3-popover-arrow-fill{\n    fill:#137cbd; }\n  .bp3-tooltip.bp3-intent-success .bp3-popover-content{\n    background:#0f9960;\n    color:#ffffff; }\n  .bp3-tooltip.bp3-intent-success .bp3-popover-arrow-fill{\n    fill:#0f9960; }\n  .bp3-tooltip.bp3-intent-warning .bp3-popover-content{\n    background:#d9822b;\n    color:#ffffff; }\n  .bp3-tooltip.bp3-intent-warning .bp3-popover-arrow-fill{\n    fill:#d9822b; }\n  .bp3-tooltip.bp3-intent-danger .bp3-popover-content{\n    background:#db3737;\n    color:#ffffff; }\n  .bp3-tooltip.bp3-intent-danger .bp3-popover-arrow-fill{\n    fill:#db3737; }\n\n.bp3-tooltip-indicator{\n  border-bottom:dotted 1px;\n  cursor:help; }\n.bp3-tree .bp3-icon, .bp3-tree .bp3-icon-standard, .bp3-tree .bp3-icon-large{\n  color:#5c7080; }\n  .bp3-tree .bp3-icon.bp3-intent-primary, .bp3-tree .bp3-icon-standard.bp3-intent-primary, .bp3-tree .bp3-icon-large.bp3-intent-primary{\n    color:#137cbd; }\n  .bp3-tree .bp3-icon.bp3-intent-success, .bp3-tree .bp3-icon-standard.bp3-intent-success, .bp3-tree .bp3-icon-large.bp3-intent-success{\n    color:#0f9960; }\n  .bp3-tree .bp3-icon.bp3-intent-warning, .bp3-tree .bp3-icon-standard.bp3-intent-warning, .bp3-tree .bp3-icon-large.bp3-intent-warning{\n    color:#d9822b; }\n  .bp3-tree .bp3-icon.bp3-intent-danger, .bp3-tree .bp3-icon-standard.bp3-intent-danger, .bp3-tree .bp3-icon-large.bp3-intent-danger{\n    color:#db3737; }\n\n.bp3-tree-node-list{\n  margin:0;\n  padding-left:0;\n  list-style:none; }\n\n.bp3-tree-root{\n  position:relative;\n  background-color:transparent;\n  cursor:default;\n  padding-left:0; }\n\n.bp3-tree-node-content-0{\n  padding-left:0px; }\n\n.bp3-tree-node-content-1{\n  padding-left:23px; }\n\n.bp3-tree-node-content-2{\n  padding-left:46px; }\n\n.bp3-tree-node-content-3{\n  padding-left:69px; }\n\n.bp3-tree-node-content-4{\n  padding-left:92px; }\n\n.bp3-tree-node-content-5{\n  padding-left:115px; }\n\n.bp3-tree-node-content-6{\n  padding-left:138px; }\n\n.bp3-tree-node-content-7{\n  padding-left:161px; }\n\n.bp3-tree-node-content-8{\n  padding-left:184px; }\n\n.bp3-tree-node-content-9{\n  padding-left:207px; }\n\n.bp3-tree-node-content-10{\n  padding-left:230px; }\n\n.bp3-tree-node-content-11{\n  padding-left:253px; }\n\n.bp3-tree-node-content-12{\n  padding-left:276px; }\n\n.bp3-tree-node-content-13{\n  padding-left:299px; }\n\n.bp3-tree-node-content-14{\n  padding-left:322px; }\n\n.bp3-tree-node-content-15{\n  padding-left:345px; }\n\n.bp3-tree-node-content-16{\n  padding-left:368px; }\n\n.bp3-tree-node-content-17{\n  padding-left:391px; }\n\n.bp3-tree-node-content-18{\n  padding-left:414px; }\n\n.bp3-tree-node-content-19{\n  padding-left:437px; }\n\n.bp3-tree-node-content-20{\n  padding-left:460px; }\n\n.bp3-tree-node-content{\n  display:-webkit-box;\n  display:-webkit-flex;\n  display:-ms-flexbox;\n  display:flex;\n  -webkit-box-align:center;\n  -webkit-align-items:center;\n      -ms-flex-align:center;\n          align-items:center;\n  width:100%;\n  height:30px;\n  padding-right:5px; }\n  .bp3-tree-node-content:hover{\n    background-color:rgba(191, 204, 214, 0.4); }\n\n.bp3-tree-node-caret,\n.bp3-tree-node-caret-none{\n  min-width:30px; }\n\n.bp3-tree-node-caret{\n  color:#5c7080;\n  -webkit-transform:rotate(0deg);\n          transform:rotate(0deg);\n  cursor:pointer;\n  padding:7px;\n  -webkit-transition:-webkit-transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:-webkit-transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9);\n  transition:transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9), -webkit-transform 200ms cubic-bezier(0.4, 1, 0.75, 0.9); }\n  .bp3-tree-node-caret:hover{\n    color:#182026; }\n  .bp3-dark .bp3-tree-node-caret{\n    color:#a7b6c2; }\n    .bp3-dark .bp3-tree-node-caret:hover{\n      color:#f5f8fa; }\n  .bp3-tree-node-caret.bp3-tree-node-caret-open{\n    -webkit-transform:rotate(90deg);\n            transform:rotate(90deg); }\n  .bp3-tree-node-caret.bp3-icon-standard::before{\n    content:"\\E695"; }\n\n.bp3-tree-node-icon{\n  position:relative;\n  margin-right:7px; }\n\n.bp3-tree-node-label{\n  overflow:hidden;\n  text-overflow:ellipsis;\n  white-space:nowrap;\n  word-wrap:normal;\n  -webkit-box-flex:1;\n  -webkit-flex:1 1 auto;\n      -ms-flex:1 1 auto;\n          flex:1 1 auto;\n  position:relative;\n  -webkit-user-select:none;\n     -moz-user-select:none;\n      -ms-user-select:none;\n          user-select:none; }\n  .bp3-tree-node-label span{\n    display:inline; }\n\n.bp3-tree-node-secondary-label{\n  padding:0 5px;\n  -webkit-user-select:none;\n     -moz-user-select:none;\n      -ms-user-select:none;\n          user-select:none; }\n  .bp3-tree-node-secondary-label .bp3-popover-wrapper,\n  .bp3-tree-node-secondary-label .bp3-popover-target{\n    display:-webkit-box;\n    display:-webkit-flex;\n    display:-ms-flexbox;\n    display:flex;\n    -webkit-box-align:center;\n    -webkit-align-items:center;\n        -ms-flex-align:center;\n            align-items:center; }\n\n.bp3-tree-node.bp3-disabled .bp3-tree-node-content{\n  background-color:inherit;\n  cursor:not-allowed;\n  color:rgba(92, 112, 128, 0.6); }\n\n.bp3-tree-node.bp3-disabled .bp3-tree-node-caret,\n.bp3-tree-node.bp3-disabled .bp3-tree-node-icon{\n  cursor:not-allowed;\n  color:rgba(92, 112, 128, 0.6); }\n\n.bp3-tree-node.bp3-tree-node-selected > .bp3-tree-node-content{\n  background-color:#137cbd; }\n  .bp3-tree-node.bp3-tree-node-selected > .bp3-tree-node-content,\n  .bp3-tree-node.bp3-tree-node-selected > .bp3-tree-node-content .bp3-icon, .bp3-tree-node.bp3-tree-node-selected > .bp3-tree-node-content .bp3-icon-standard, .bp3-tree-node.bp3-tree-node-selected > .bp3-tree-node-content .bp3-icon-large{\n    color:#ffffff; }\n  .bp3-tree-node.bp3-tree-node-selected > .bp3-tree-node-content .bp3-tree-node-caret::before{\n    color:rgba(255, 255, 255, 0.7); }\n  .bp3-tree-node.bp3-tree-node-selected > .bp3-tree-node-content .bp3-tree-node-caret:hover::before{\n    color:#ffffff; }\n\n.bp3-dark .bp3-tree-node-content:hover{\n  background-color:rgba(92, 112, 128, 0.3); }\n\n.bp3-dark .bp3-tree .bp3-icon, .bp3-dark .bp3-tree .bp3-icon-standard, .bp3-dark .bp3-tree .bp3-icon-large{\n  color:#a7b6c2; }\n  .bp3-dark .bp3-tree .bp3-icon.bp3-intent-primary, .bp3-dark .bp3-tree .bp3-icon-standard.bp3-intent-primary, .bp3-dark .bp3-tree .bp3-icon-large.bp3-intent-primary{\n    color:#137cbd; }\n  .bp3-dark .bp3-tree .bp3-icon.bp3-intent-success, .bp3-dark .bp3-tree .bp3-icon-standard.bp3-intent-success, .bp3-dark .bp3-tree .bp3-icon-large.bp3-intent-success{\n    color:#0f9960; }\n  .bp3-dark .bp3-tree .bp3-icon.bp3-intent-warning, .bp3-dark .bp3-tree .bp3-icon-standard.bp3-intent-warning, .bp3-dark .bp3-tree .bp3-icon-large.bp3-intent-warning{\n    color:#d9822b; }\n  .bp3-dark .bp3-tree .bp3-icon.bp3-intent-danger, .bp3-dark .bp3-tree .bp3-icon-standard.bp3-intent-danger, .bp3-dark .bp3-tree .bp3-icon-large.bp3-intent-danger{\n    color:#db3737; }\n\n.bp3-dark .bp3-tree-node.bp3-tree-node-selected > .bp3-tree-node-content{\n  background-color:#137cbd; }',""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,'/*!\n\nCopyright 2017-present Palantir Technologies, Inc. All rights reserved.\nLicensed under the Apache License, Version 2.0.\n\n*/\n/*\nA mixin to generate the classes for a React CSSTransition which animates any number of CSS\nproperties at once.\n\nTransitioned properties are specificed as a map of property names to lists of (inital value, final\nvalue). For enter & appear transitions, each property will transition from its initial to its final\nvalue. For exit transitions, each property will transition in reverse, from final to initial.\n\n**Simple example:**\n`@include react-transition("popover", (opacity: 0 1), $before: "&");`\nTransition named "popover" moves opacity from 0 to 1. `"&"` indicates that the\nTransition classes are expected to be applied to this element, where the mixin is invoked.\n\n**Params:**\n$name: React transitionName prop\n$properties: map of CSS property to (initial, final) values\n$duration: transition duration\n$easing: transition easing function\n$delay: transition delay\n$before: selector text to insert before transition name (often to select self: &)\n$after: selector text to insert after transiton name (to select children)\n*/\n/*\nA mixin to generate the classes for one phase of a React CSSTransition.\n`$phase` must be `appear` or `enter` or `exit`.\nIf `enter` phase is given then `appear` phase will be generated at the same time.\nIf `exit` phase is given then property values are animated in reverse, from final to initial.\n\n**Example:**\n@include react-transition-phase(toast, enter, $enter-translate, $before: "&");\n@include react-transition-phase(toast, leave, $leave-blur, $pt-transition-duration * 3, $before: "&");\n*/\n/*\nGiven map of properties to values, set each property to the value at the given index.\n(remember that sass indices are 1-based).\n\nExample: `each-prop((opacity: 0 1), 2)` will print "opacity: 1"\n*/\n/*\nFormat transition class name with all the bits.\n"enter" phase will include "appear" phase in returned name.\n*/\n.bp3-omnibar {\n  filter: blur(0);\n  opacity: 1;\n  top: 20vh;\n  left: calc(50% - 250px);\n  z-index: 21;\n  border-radius: 3px;\n  box-shadow: 0 0 0 1px rgba(16, 22, 26, 0.1), 0 4px 8px rgba(16, 22, 26, 0.2), 0 18px 46px 6px rgba(16, 22, 26, 0.2);\n  background-color: #ffffff;\n  width: 500px; }\n  .bp3-omnibar.bp3-overlay-enter, .bp3-omnibar.bp3-overlay-appear {\n    filter: blur(20px);\n    opacity: 0.2; }\n  .bp3-omnibar.bp3-overlay-enter-active, .bp3-omnibar.bp3-overlay-appear-active {\n    filter: blur(0);\n    opacity: 1;\n    transition-property: filter, opacity;\n    transition-duration: 200ms;\n    transition-timing-function: cubic-bezier(0.4, 1, 0.75, 0.9);\n    transition-delay: 0; }\n  .bp3-omnibar.bp3-overlay-exit {\n    filter: blur(0);\n    opacity: 1; }\n  .bp3-omnibar.bp3-overlay-exit-active {\n    filter: blur(20px);\n    opacity: 0.2;\n    transition-property: filter, opacity;\n    transition-duration: 200ms;\n    transition-timing-function: cubic-bezier(0.4, 1, 0.75, 0.9);\n    transition-delay: 0; }\n  .bp3-omnibar .bp3-input {\n    border-radius: 0;\n    background-color: transparent; }\n    .bp3-omnibar .bp3-input, .bp3-omnibar .bp3-input:focus {\n      box-shadow: none; }\n  .bp3-omnibar .bp3-menu {\n    border-radius: 0;\n    box-shadow: inset 0 1px 0 rgba(16, 22, 26, 0.15);\n    background-color: transparent;\n    max-height: calc(60vh - 40px);\n    overflow: auto; }\n    .bp3-omnibar .bp3-menu:empty {\n      display: none; }\n  .bp3-dark .bp3-omnibar, .bp3-omnibar.bp3-dark {\n    box-shadow: 0 0 0 1px rgba(16, 22, 26, 0.2), 0 4px 8px rgba(16, 22, 26, 0.4), 0 18px 46px 6px rgba(16, 22, 26, 0.4);\n    background-color: #30404d; }\n\n.bp3-omnibar-overlay .bp3-overlay-backdrop {\n  background-color: rgba(16, 22, 26, 0.2); }\n\n.bp3-select-popover .bp3-popover-content {\n  padding: 5px; }\n\n.bp3-select-popover .bp3-input-group {\n  margin-bottom: 0; }\n\n.bp3-select-popover .bp3-menu {\n  max-width: 400px;\n  max-height: 300px;\n  overflow: auto;\n  padding: 0; }\n  .bp3-select-popover .bp3-menu:not(:first-child) {\n    padding-top: 5px; }\n\n.bp3-multi-select {\n  min-width: 150px; }\n\n.bp3-multi-select-popover .bp3-menu {\n  max-width: 400px;\n  max-height: 300px;\n  overflow: auto; }\n\n.bp3-select-popover .bp3-popover-content {\n  padding: 5px; }\n\n.bp3-select-popover .bp3-input-group {\n  margin-bottom: 0; }\n\n.bp3-select-popover .bp3-menu {\n  max-width: 400px;\n  max-height: 300px;\n  overflow: auto;\n  padding: 0; }\n  .bp3-select-popover .bp3-menu:not(:first-child) {\n    padding-top: 5px; }',""])},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(951),""),e.i(n(952),""),e.push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/* Override Blueprint's _reset.scss styles */\nhtml {\n  box-sizing: unset;\n}\n\n*,\n*::before,\n*::after {\n  box-sizing: unset;\n}\n\nbody {\n  color: unset;\n  font-family: var(--jp-ui-font-family);\n}\n\np {\n  margin-top: unset;\n  margin-bottom: unset;\n}\n\nsmall {\n  font-size: unset;\n}\n\nstrong {\n  font-weight: unset;\n}\n\n/* Override Blueprint's _typography.scss styles */\na {\n  text-decoration: unset;\n  color: unset;\n}\na:hover {\n  text-decoration: unset;\n  color: unset;\n}\n\n/* Override Blueprint's _accessibility.scss styles */\n:focus {\n  outline: unset;\n  outline-offset: unset;\n  -moz-outline-radius: unset;\n}\n\n/* Styles for ui-components */\n.jp-Button {\n  border-radius: var(--jp-border-radius);\n  padding: 0px 12px;\n  font-size: var(--jp-ui-font-size1);\n}\n\n/* Use our own theme for hover styles */\nbutton.jp-Button.bp3-button.bp3-minimal:hover {\n  background-color: var(--jp-layout-color2);\n}\n.jp-Button.minimal {\n  color: unset !important;\n}\n\n.jp-Button.jp-ToolbarButtonComponent {\n  text-transform: none;\n}\n\n.jp-InputGroup input {\n  box-sizing: border-box;\n  border-radius: 0;\n  background-color: transparent;\n  color: var(--jp-ui-font-color0);\n  box-shadow: inset 0 0 0 var(--jp-border-width) var(--jp-input-border-color);\n}\n\n.jp-InputGroup input:focus {\n  box-shadow: inset 0 0 0 var(--jp-border-width)\n      var(--jp-input-active-box-shadow-color),\n    inset 0 0 0 3px var(--jp-input-active-box-shadow-color);\n}\n\n.jp-InputGroup input::placeholder,\ninput::placeholder {\n  color: var(--jp-ui-font-color3);\n}\n\n.jp-Icon {\n  color: var(--jp-layout-color4);\n}\n\n.jp-InputGroupAction {\n  padding: 6px;\n}\n\n.jp-HTMLSelect.bp3-html-select.bp3-minimal select {\n  height: 24;\n  font-size: var(--jp-ui-font-size1);\n  line-height: 14px;\n  border-radius: 0;\n  display: block;\n  color: var(--jp-ui-font-color0);\n}\n\n/* Use our own theme for hover styles */\n.jp-HTMLSelect.bp3-html-select.bp3-minimal select:hover {\n  color: var(--jp-ui-font-color0);\n  background-color: var(--jp-layout-color2);\n}\nselect {\n  box-sizing: border-box;\n}\n\n/* Sibling imports */\n",""])},function(t,e,n){var r=n(172);(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/* This file was auto-generated by ensureUiComponents() in @jupyterlab/buildutils */\n\n/**\n * (DEPRECATED) Support for consuming icons as CSS background images\n */\n\n/* Icons urls */\n\n:root {\n  --jp-icon-file: url("+r(n(350))+");\n  --jp-icon-folder: url("+r(n(351))+");\n  --jp-icon-html5: url("+r(n(352))+");\n  --jp-icon-image: url("+r(n(353))+");\n  --jp-icon-json: url("+r(n(354))+");\n  --jp-icon-markdown: url("+r(n(355))+");\n  --jp-icon-notebook: url("+r(n(356))+");\n  --jp-icon-python: url("+r(n(357))+");\n  --jp-icon-r-kernel: url("+r(n(358))+");\n  --jp-icon-react: url("+r(n(359))+");\n  --jp-icon-spreadsheet: url("+r(n(360))+");\n  --jp-icon-yaml: url("+r(n(361))+");\n  --jp-icon-build: url("+r(n(362))+");\n  --jp-icon-extension: url("+r(n(363))+");\n  --jp-icon-palette: url("+r(n(364))+");\n  --jp-icon-running: url("+r(n(365))+");\n  --jp-icon-tab: url("+r(n(366))+");\n  --jp-icon-jupyter-favicon: url("+r(n(367))+");\n  --jp-icon-kernel: url("+r(n(368))+");\n  --jp-icon-line-form: url("+r(n(369))+");\n  --jp-icon-not-trusted: url("+r(n(370))+");\n  --jp-icon-terminal: url("+r(n(371))+");\n  --jp-icon-trusted: url("+r(n(372))+");\n  --jp-icon-add: url("+r(n(373))+");\n  --jp-icon-copy: url("+r(n(374))+");\n  --jp-icon-cut: url("+r(n(375))+");\n  --jp-icon-paste: url("+r(n(376))+");\n  --jp-icon-refresh: url("+r(n(377))+");\n  --jp-icon-run: url("+r(n(378))+");\n  --jp-icon-save: url("+r(n(379))+");\n  --jp-icon-stop: url("+r(n(380))+");\n}\n\n/* Icon CSS class declarations */\n\n.jp-FileIcon {\n  background-image: var(--jp-icon-file);\n}\n.jp-FolderIcon {\n  background-image: var(--jp-icon-folder);\n}\n.jp-Html5Icon {\n  background-image: var(--jp-icon-html5);\n}\n.jp-ImageIcon {\n  background-image: var(--jp-icon-image);\n}\n.jp-JsonIcon {\n  background-image: var(--jp-icon-json);\n}\n.jp-MarkdownIcon {\n  background-image: var(--jp-icon-markdown);\n}\n.jp-NotebookIcon {\n  background-image: var(--jp-icon-notebook);\n}\n.jp-PythonIcon {\n  background-image: var(--jp-icon-python);\n}\n.jp-RKernelIcon {\n  background-image: var(--jp-icon-r-kernel);\n}\n.jp-ReactIcon {\n  background-image: var(--jp-icon-react);\n}\n.jp-SpreadsheetIcon {\n  background-image: var(--jp-icon-spreadsheet);\n}\n.jp-YamlIcon {\n  background-image: var(--jp-icon-yaml);\n}\n.jp-BuildIcon {\n  background-image: var(--jp-icon-build);\n}\n.jp-ExtensionIcon {\n  background-image: var(--jp-icon-extension);\n}\n.jp-PaletteIcon {\n  background-image: var(--jp-icon-palette);\n}\n.jp-RunningIcon {\n  background-image: var(--jp-icon-running);\n}\n.jp-TabIcon {\n  background-image: var(--jp-icon-tab);\n}\n.jp-JupyterFaviconIcon {\n  background-image: var(--jp-icon-jupyter-favicon);\n}\n.jp-KernelIcon {\n  background-image: var(--jp-icon-kernel);\n}\n.jp-LineFormIcon {\n  background-image: var(--jp-icon-line-form);\n}\n.jp-NotTrustedIcon {\n  background-image: var(--jp-icon-not-trusted);\n}\n.jp-TerminalIcon {\n  background-image: var(--jp-icon-terminal);\n}\n.jp-TrustedIcon {\n  background-image: var(--jp-icon-trusted);\n}\n.jp-AddIcon {\n  background-image: var(--jp-icon-add);\n}\n.jp-CopyIcon {\n  background-image: var(--jp-icon-copy);\n}\n.jp-CutIcon {\n  background-image: var(--jp-icon-cut);\n}\n.jp-PasteIcon {\n  background-image: var(--jp-icon-paste);\n}\n.jp-RefreshIcon {\n  background-image: var(--jp-icon-refresh);\n}\n.jp-RunIcon {\n  background-image: var(--jp-icon-run);\n}\n.jp-SaveIcon {\n  background-image: var(--jp-icon-save);\n}\n.jp-StopIcon {\n  background-image: var(--jp-icon-stop);\n}\n",""])},function(t,e,n){var r=n(172);(t.exports=n(12)(!1)).push([t.i,'/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/**\n * This file contains support for deprecated CSS not covered in the\n * auto-generated `deprecated.css`.\n */\n\n/* vars for consuming "selected" icons as CSS background images */\n\n:root {\n  --jp-icon-file-selected: url('+r(n(953))+");\n  --jp-icon-folder-selected: url("+r(n(954))+");\n  --jp-icon-image-selected: url("+r(n(955))+");\n  --jp-icon-json-selected: url("+r(n(956))+");\n  --jp-icon-kernel-selected: url("+r(n(957))+");\n  --jp-icon-markdown-selected: url("+r(n(958))+");\n  --jp-icon-notebook-selected: url("+r(n(959))+");\n  --jp-icon-python-selected: url("+r(n(960))+");\n  --jp-icon-r-selected: url("+r(n(961))+");\n  --jp-icon-spreadsheet-selected: url("+r(n(962))+");\n  --jp-icon-yaml-selected: url("+r(n(963))+");\n}\n",""])},function(t,e){t.exports="data:image/svg+xml;base64, "},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyBmaWxsPSIjZmZmZmZmIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij48cGF0aCBkPSJNMTAgNEg0Yy0xLjEgMC0xLjk5LjktMS45OSAyTDIgMThjMCAxLjEuOSAyIDIgMmgxNmMxLjEgMCAyLS45IDItMlY4YzAtMS4xLS45LTItMi0yaC04bC0yLTJ6Ii8+PC9zdmc+"},function(t,e){t.exports="data:image/svg+xml;base64, "},function(t,e){t.exports="data:image/svg+xml;base64, "},function(t,e){t.exports="data:image/svg+xml;base64,PHN2ZyBmaWxsPSIjZmZmZmZmIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij48cGF0aCBkPSJNMTUgOUg5djZoNlY5em0tMiA0aC0ydi0yaDJ2MnptOC0yVjloLTJWN2MwLTEuMS0uOS0yLTItMmgtMlYzaC0ydjJoLTJWM0g5djJIN2MtMS4xIDAtMiAuOS0yIDJ2MkgzdjJoMnYySDN2MmgydjJjMCAxLjEuOSAyIDIgMmgydjJoMnYtMmgydjJoMnYtMmgyYzEuMSAwIDItLjkgMi0ydi0yaDJ2LTJoLTJ2LTJoMnptLTQgNkg3VjdoMTB2MTB6Ii8+PC9zdmc+"},function(t,e){t.exports="data:image/svg+xml;base64, "},function(t,e){t.exports="data:image/svg+xml;base64, "},function(t,e){t.exports="data:image/svg+xml;base64, "},function(t,e){t.exports="data:image/svg+xml;base64, "},function(t,e){t.exports="data:image/svg+xml;base64, "},function(t,e){t.exports="data:image/svg+xml;base64, "},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(965),""),e.i(n(966),""),e.i(n(967),""),e.i(n(968),""),e.i(n(969),""),e.i(n(970),""),e.i(n(971),""),e.i(n(972),""),e.i(n(973),""),e.i(n(974),""),e.push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, Jupyter Development Team.\n|\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n.jp-Collapse {\n  display: flex;\n  flex-direction: column;\n  align-items: stretch;\n  border-top: 1px solid var(--jp-border-color2);\n  border-bottom: 1px solid var(--jp-border-color2);\n}\n\n.jp-Collapse-header {\n  padding: 4px 12px 4px 12px;\n  color: var(--jp-ui-font-color1);\n  background-color: var(--jp-layout-color1);\n  font-size: var(--jp-ui-font-size2);\n  background-image: var(--jp-icon-caretdown);\n  background-repeat: no-repeat;\n  background-position: right 12px top 50%;\n}\n\n.jp-Collapse-header:hover {\n  background-color: var(--jp-layout-color2);\n}\n\n.jp-Collapse-open > .jp-Collapse-header {\n  background-image: var(--jp-icon-caretup);\n}\n\n.jp-Collapse-contents {\n  padding: 0px 12px 0px 12px;\n  background-color: var(--jp-layout-color1);\n  color: var(--jp-ui-font-color1);\n  overflow: auto;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/*-----------------------------------------------------------------------------\n| Variables\n|----------------------------------------------------------------------------*/\n\n:root {\n  --jp-private-commandpalette-search-height: 28px;\n}\n\n/*-----------------------------------------------------------------------------\n| Overall styles\n|----------------------------------------------------------------------------*/\n\n.p-CommandPalette {\n  padding-bottom: 0px;\n  color: var(--jp-ui-font-color1);\n  background: var(--jp-layout-color1);\n  /* This is needed so that all font sizing of children done in ems is\n   * relative to this base size */\n  font-size: var(--jp-ui-font-size1);\n}\n\n/*-----------------------------------------------------------------------------\n| Search\n|----------------------------------------------------------------------------*/\n\n.p-CommandPalette-search {\n  padding: 4px;\n  background-color: var(--jp-layout-color1);\n  z-index: 2;\n}\n\n.p-CommandPalette-wrapper {\n  overflow: overlay;\n  padding: 0px 9px;\n  background-color: var(--jp-input-active-background);\n  height: 30px;\n  box-shadow: inset 0 0 0 var(--jp-border-width) var(--jp-input-border-color);\n}\n\n.p-CommandPalette.p-mod-focused .p-CommandPalette-wrapper {\n  box-shadow: inset 0 0 0 1px var(--jp-input-active-box-shadow-color),\n    inset 0 0 0 3px var(--jp-input-active-box-shadow-color);\n}\n\n.p-CommandPalette-wrapper::after {\n  content: ' ';\n  color: white;\n  background-color: var(--jp-brand-color1);\n  position: absolute;\n  top: 4px;\n  right: 4px;\n  height: 30px;\n  width: 10px;\n  padding: 0px 10px;\n  background-image: var(--jp-icon-search-white);\n  background-size: 20px;\n  background-repeat: no-repeat;\n  background-position: center;\n}\n\n.p-CommandPalette-input {\n  background: transparent;\n  width: calc(100% - 18px);\n  float: left;\n  border: none;\n  outline: none;\n  font-size: var(--jp-ui-font-size1);\n  color: var(--jp-ui-font-color0);\n  line-height: var(--jp-private-commandpalette-search-height);\n}\n\n.p-CommandPalette-input::-webkit-input-placeholder,\n.p-CommandPalette-input::-moz-placeholder,\n.p-CommandPalette-input:-ms-input-placeholder {\n  color: var(--jp-ui-font-color3);\n  font-size: var(--jp-ui-font-size1);\n}\n\n/*-----------------------------------------------------------------------------\n| Results\n|----------------------------------------------------------------------------*/\n\n.p-CommandPalette-header:first-child {\n  margin-top: 0px;\n}\n\n.p-CommandPalette-header {\n  margin-top: 8px;\n  padding: 8px 0 8px 12px;\n  color: var(--jp-ui-font-color1);\n  font-size: var(--jp-ui-font-size0);\n  font-weight: 600;\n  cursor: pointer;\n  text-transform: uppercase;\n  border-bottom: solid var(--jp-border-width) var(--jp-border-color2);\n  letter-spacing: 1px;\n}\n\n.p-CommandPalette-header.p-mod-active {\n  background: var(--jp-layout-color2);\n}\n\n.p-CommandPalette-header:hover::after {\n  content: '';\n  min-width: 12px;\n  min-height: 12px;\n  background-size: 14px;\n  margin-right: 16px;\n  float: right;\n  display: inline-block;\n  vertical-align: middle;\n  background-repeat: no-repeat;\n  background-position: center;\n  background-image: var(--jp-icon-filter-list);\n}\n\n.p-CommandPalette-header > mark {\n  background-color: transparent;\n  font-weight: bold;\n  color: var(--jp-ui-font-color1);\n}\n\n.p-CommandPalette-item {\n  padding: 4px 12px 4px 4px;\n  color: var(--jp-ui-font-color1);\n  font-size: var(--jp-ui-font-size1);\n  font-weight: 400;\n  display: flex;\n}\n\n.p-CommandPalette-item.p-mod-disabled {\n  color: var(--jp-ui-font-color3);\n}\n\n.p-CommandPalette-item.p-mod-active {\n  background: var(--jp-layout-color3);\n}\n\n.p-CommandPalette-item.p-mod-active:hover:not(.p-mod-disabled) {\n  background: var(--jp-layout-color4);\n}\n\n.p-CommandPalette-item:hover:not(.p-mod-active):not(.p-mod-disabled) {\n  background: var(--jp-layout-color2);\n}\n\n.p-CommandPalette-itemContent {\n  overflow: hidden;\n}\n\n.p-CommandPalette-itemLabel > mark {\n  color: var(--jp-ui-font-color0);\n  background-color: transparent;\n  font-weight: bold;\n}\n\n.p-CommandPalette-item.p-mod-disabled mark {\n  color: var(--jp-ui-font-color3);\n}\n\n.p-CommandPalette-item.p-mod-toggled .p-CommandPalette-itemIcon {\n  background-image: var(--jp-icon-checkmark);\n  background-size: 16px;\n  background-repeat: no-repeat;\n}\n\n.p-CommandPalette-item .p-CommandPalette-itemIcon {\n  padding: 0px 0px 0px 4px;\n  position: relative;\n  width: 16px;\n  top: 2px;\n  flex: 0 0 auto;\n}\n\n.p-CommandPalette-item.p-mod-disabled .p-CommandPalette-itemIcon {\n  opacity: 0.4;\n}\n\n.p-CommandPalette-item .p-CommandPalette-itemShortcut {\n  flex: 0 0 auto;\n}\n\n.p-CommandPalette-itemCaption {\n  display: none;\n}\n\n.p-CommandPalette-content {\n  background-color: var(--jp-layout-color1);\n}\n\n.p-CommandPalette-content:empty:after {\n  content: 'No results';\n  margin: auto;\n  margin-top: 20px;\n  width: 100px;\n  display: block;\n  font-size: var(--jp-ui-font-size2);\n  font-family: var(--jp-ui-font-family);\n  font-weight: lighter;\n}\n\n.p-CommandPalette-emptyMessage {\n  text-align: center;\n  margin-top: 24px;\n  line-height: 1.32;\n  padding: 0px 8px;\n  color: var(--jp-content-font-color3);\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, Jupyter Development Team.\n|\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n.jp-Dialog {\n  position: absolute;\n  z-index: 10000;\n  display: flex;\n  flex-direction: column;\n  align-items: center;\n  justify-content: center;\n  top: 0px;\n  left: 0px;\n  margin: 0;\n  padding: 0;\n  width: 100%;\n  height: 100%;\n  background: var(--jp-dialog-background);\n}\n\n.jp-Dialog-content {\n  display: flex;\n  flex-direction: column;\n  margin-left: auto;\n  margin-right: auto;\n  background: var(--jp-layout-color1);\n  padding: 24px;\n  padding-bottom: 12px;\n  min-width: 300px;\n  min-height: 150px;\n  max-width: 500px;\n  max-height: 500px;\n  box-sizing: border-box;\n  box-shadow: var(--jp-elevation-z20);\n  word-wrap: break-word;\n  border-radius: var(--jp-border-radius);\n  /* This is needed so that all font sizing of children done in ems is\n   * relative to this base size */\n  font-size: var(--jp-ui-font-size1);\n  color: var(--jp-ui-font-color1);\n}\n\n.jp-Dialog-button {\n  overflow: visible;\n}\n\nbutton.jp-Dialog-button:focus {\n  outline: 1px solid var(--jp-brand-color1);\n  outline-offset: 4px;\n  -moz-outline-radius: 0px;\n}\n\nbutton.jp-Dialog-button:focus::-moz-focus-inner {\n  border: 0;\n}\n\n.jp-Dialog-header {\n  flex: 0 0 auto;\n  padding-bottom: 12px;\n  font-size: var(--jp-ui-font-size3);\n  font-weight: 400;\n  color: var(--jp-ui-font-color0);\n}\n\n.jp-Dialog-body {\n  display: flex;\n  flex-direction: column;\n  flex: 1 1 auto;\n  font-size: var(--jp-ui-font-size1);\n  background: var(--jp-layout-color1);\n  overflow: auto;\n}\n\n.jp-Dialog-footer {\n  display: flex;\n  flex-direction: row;\n  justify-content: flex-end;\n  flex: 0 0 auto;\n  margin-left: -12px;\n  margin-right: -12px;\n  padding: 12px;\n}\n\n.jp-Dialog-title {\n  overflow: hidden;\n  white-space: nowrap;\n  text-overflow: ellipsis;\n}\n\n.jp-Dialog-body > .jp-select-wrapper {\n  width: 100%;\n}\n\n.jp-Dialog-body > button {\n  padding: 0px 16px;\n}\n\n.jp-Dialog-body > label {\n  padding-top: 4px;\n  padding-bottom: 8px;\n  line-height: 1.4;\n  color: var(--jp-ui-font-color0);\n}\n\n.jp-Dialog-button.jp-mod-styled:not(:last-child) {\n  margin-right: 12px;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2016, Jupyter Development Team.\n|\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n.jp-HoverBox {\n  position: fixed;\n}\n\n.jp-HoverBox.jp-mod-outofview {\n  display: none;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n.jp-IFrame {\n  width: 100%;\n  height: 100%;\n}\n\n.jp-IFrame > iframe {\n  border: none;\n}\n\n/*\nWhen drag events occur, `p-mod-override-cursor` is added to the body.\nBecause iframes steal all cursor events, the following two rules are necessary\nto suppress pointer events while resize drags are occurring. There may be a\nbetter solution to this problem.\n*/\nbody.p-mod-override-cursor .jp-IFrame {\n  position: relative;\n}\n\nbody.p-mod-override-cursor .jp-IFrame:before {\n  content: '';\n  position: absolute;\n  top: 0;\n  left: 0;\n  right: 0;\n  bottom: 0;\n  background: transparent;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2016, Jupyter Development Team.\n|\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n.jp-MainAreaWidget > :focus {\n  outline: none;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/**\n * google-material-color v1.2.6\n * https://github.com/danlevan/google-material-color\n */\n:root {\n  --md-red-50: #ffebee;\n  --md-red-100: #ffcdd2;\n  --md-red-200: #ef9a9a;\n  --md-red-300: #e57373;\n  --md-red-400: #ef5350;\n  --md-red-500: #f44336;\n  --md-red-600: #e53935;\n  --md-red-700: #d32f2f;\n  --md-red-800: #c62828;\n  --md-red-900: #b71c1c;\n  --md-red-A100: #ff8a80;\n  --md-red-A200: #ff5252;\n  --md-red-A400: #ff1744;\n  --md-red-A700: #d50000;\n\n  --md-pink-50: #fce4ec;\n  --md-pink-100: #f8bbd0;\n  --md-pink-200: #f48fb1;\n  --md-pink-300: #f06292;\n  --md-pink-400: #ec407a;\n  --md-pink-500: #e91e63;\n  --md-pink-600: #d81b60;\n  --md-pink-700: #c2185b;\n  --md-pink-800: #ad1457;\n  --md-pink-900: #880e4f;\n  --md-pink-A100: #ff80ab;\n  --md-pink-A200: #ff4081;\n  --md-pink-A400: #f50057;\n  --md-pink-A700: #c51162;\n\n  --md-purple-50: #f3e5f5;\n  --md-purple-100: #e1bee7;\n  --md-purple-200: #ce93d8;\n  --md-purple-300: #ba68c8;\n  --md-purple-400: #ab47bc;\n  --md-purple-500: #9c27b0;\n  --md-purple-600: #8e24aa;\n  --md-purple-700: #7b1fa2;\n  --md-purple-800: #6a1b9a;\n  --md-purple-900: #4a148c;\n  --md-purple-A100: #ea80fc;\n  --md-purple-A200: #e040fb;\n  --md-purple-A400: #d500f9;\n  --md-purple-A700: #aa00ff;\n\n  --md-deep-purple-50: #ede7f6;\n  --md-deep-purple-100: #d1c4e9;\n  --md-deep-purple-200: #b39ddb;\n  --md-deep-purple-300: #9575cd;\n  --md-deep-purple-400: #7e57c2;\n  --md-deep-purple-500: #673ab7;\n  --md-deep-purple-600: #5e35b1;\n  --md-deep-purple-700: #512da8;\n  --md-deep-purple-800: #4527a0;\n  --md-deep-purple-900: #311b92;\n  --md-deep-purple-A100: #b388ff;\n  --md-deep-purple-A200: #7c4dff;\n  --md-deep-purple-A400: #651fff;\n  --md-deep-purple-A700: #6200ea;\n\n  --md-indigo-50: #e8eaf6;\n  --md-indigo-100: #c5cae9;\n  --md-indigo-200: #9fa8da;\n  --md-indigo-300: #7986cb;\n  --md-indigo-400: #5c6bc0;\n  --md-indigo-500: #3f51b5;\n  --md-indigo-600: #3949ab;\n  --md-indigo-700: #303f9f;\n  --md-indigo-800: #283593;\n  --md-indigo-900: #1a237e;\n  --md-indigo-A100: #8c9eff;\n  --md-indigo-A200: #536dfe;\n  --md-indigo-A400: #3d5afe;\n  --md-indigo-A700: #304ffe;\n\n  --md-blue-50: #e3f2fd;\n  --md-blue-100: #bbdefb;\n  --md-blue-200: #90caf9;\n  --md-blue-300: #64b5f6;\n  --md-blue-400: #42a5f5;\n  --md-blue-500: #2196f3;\n  --md-blue-600: #1e88e5;\n  --md-blue-700: #1976d2;\n  --md-blue-800: #1565c0;\n  --md-blue-900: #0d47a1;\n  --md-blue-A100: #82b1ff;\n  --md-blue-A200: #448aff;\n  --md-blue-A400: #2979ff;\n  --md-blue-A700: #2962ff;\n\n  --md-light-blue-50: #e1f5fe;\n  --md-light-blue-100: #b3e5fc;\n  --md-light-blue-200: #81d4fa;\n  --md-light-blue-300: #4fc3f7;\n  --md-light-blue-400: #29b6f6;\n  --md-light-blue-500: #03a9f4;\n  --md-light-blue-600: #039be5;\n  --md-light-blue-700: #0288d1;\n  --md-light-blue-800: #0277bd;\n  --md-light-blue-900: #01579b;\n  --md-light-blue-A100: #80d8ff;\n  --md-light-blue-A200: #40c4ff;\n  --md-light-blue-A400: #00b0ff;\n  --md-light-blue-A700: #0091ea;\n\n  --md-cyan-50: #e0f7fa;\n  --md-cyan-100: #b2ebf2;\n  --md-cyan-200: #80deea;\n  --md-cyan-300: #4dd0e1;\n  --md-cyan-400: #26c6da;\n  --md-cyan-500: #00bcd4;\n  --md-cyan-600: #00acc1;\n  --md-cyan-700: #0097a7;\n  --md-cyan-800: #00838f;\n  --md-cyan-900: #006064;\n  --md-cyan-A100: #84ffff;\n  --md-cyan-A200: #18ffff;\n  --md-cyan-A400: #00e5ff;\n  --md-cyan-A700: #00b8d4;\n\n  --md-teal-50: #e0f2f1;\n  --md-teal-100: #b2dfdb;\n  --md-teal-200: #80cbc4;\n  --md-teal-300: #4db6ac;\n  --md-teal-400: #26a69a;\n  --md-teal-500: #009688;\n  --md-teal-600: #00897b;\n  --md-teal-700: #00796b;\n  --md-teal-800: #00695c;\n  --md-teal-900: #004d40;\n  --md-teal-A100: #a7ffeb;\n  --md-teal-A200: #64ffda;\n  --md-teal-A400: #1de9b6;\n  --md-teal-A700: #00bfa5;\n\n  --md-green-50: #e8f5e9;\n  --md-green-100: #c8e6c9;\n  --md-green-200: #a5d6a7;\n  --md-green-300: #81c784;\n  --md-green-400: #66bb6a;\n  --md-green-500: #4caf50;\n  --md-green-600: #43a047;\n  --md-green-700: #388e3c;\n  --md-green-800: #2e7d32;\n  --md-green-900: #1b5e20;\n  --md-green-A100: #b9f6ca;\n  --md-green-A200: #69f0ae;\n  --md-green-A400: #00e676;\n  --md-green-A700: #00c853;\n\n  --md-light-green-50: #f1f8e9;\n  --md-light-green-100: #dcedc8;\n  --md-light-green-200: #c5e1a5;\n  --md-light-green-300: #aed581;\n  --md-light-green-400: #9ccc65;\n  --md-light-green-500: #8bc34a;\n  --md-light-green-600: #7cb342;\n  --md-light-green-700: #689f38;\n  --md-light-green-800: #558b2f;\n  --md-light-green-900: #33691e;\n  --md-light-green-A100: #ccff90;\n  --md-light-green-A200: #b2ff59;\n  --md-light-green-A400: #76ff03;\n  --md-light-green-A700: #64dd17;\n\n  --md-lime-50: #f9fbe7;\n  --md-lime-100: #f0f4c3;\n  --md-lime-200: #e6ee9c;\n  --md-lime-300: #dce775;\n  --md-lime-400: #d4e157;\n  --md-lime-500: #cddc39;\n  --md-lime-600: #c0ca33;\n  --md-lime-700: #afb42b;\n  --md-lime-800: #9e9d24;\n  --md-lime-900: #827717;\n  --md-lime-A100: #f4ff81;\n  --md-lime-A200: #eeff41;\n  --md-lime-A400: #c6ff00;\n  --md-lime-A700: #aeea00;\n\n  --md-yellow-50: #fffde7;\n  --md-yellow-100: #fff9c4;\n  --md-yellow-200: #fff59d;\n  --md-yellow-300: #fff176;\n  --md-yellow-400: #ffee58;\n  --md-yellow-500: #ffeb3b;\n  --md-yellow-600: #fdd835;\n  --md-yellow-700: #fbc02d;\n  --md-yellow-800: #f9a825;\n  --md-yellow-900: #f57f17;\n  --md-yellow-A100: #ffff8d;\n  --md-yellow-A200: #ffff00;\n  --md-yellow-A400: #ffea00;\n  --md-yellow-A700: #ffd600;\n\n  --md-amber-50: #fff8e1;\n  --md-amber-100: #ffecb3;\n  --md-amber-200: #ffe082;\n  --md-amber-300: #ffd54f;\n  --md-amber-400: #ffca28;\n  --md-amber-500: #ffc107;\n  --md-amber-600: #ffb300;\n  --md-amber-700: #ffa000;\n  --md-amber-800: #ff8f00;\n  --md-amber-900: #ff6f00;\n  --md-amber-A100: #ffe57f;\n  --md-amber-A200: #ffd740;\n  --md-amber-A400: #ffc400;\n  --md-amber-A700: #ffab00;\n\n  --md-orange-50: #fff3e0;\n  --md-orange-100: #ffe0b2;\n  --md-orange-200: #ffcc80;\n  --md-orange-300: #ffb74d;\n  --md-orange-400: #ffa726;\n  --md-orange-500: #ff9800;\n  --md-orange-600: #fb8c00;\n  --md-orange-700: #f57c00;\n  --md-orange-800: #ef6c00;\n  --md-orange-900: #e65100;\n  --md-orange-A100: #ffd180;\n  --md-orange-A200: #ffab40;\n  --md-orange-A400: #ff9100;\n  --md-orange-A700: #ff6d00;\n\n  --md-deep-orange-50: #fbe9e7;\n  --md-deep-orange-100: #ffccbc;\n  --md-deep-orange-200: #ffab91;\n  --md-deep-orange-300: #ff8a65;\n  --md-deep-orange-400: #ff7043;\n  --md-deep-orange-500: #ff5722;\n  --md-deep-orange-600: #f4511e;\n  --md-deep-orange-700: #e64a19;\n  --md-deep-orange-800: #d84315;\n  --md-deep-orange-900: #bf360c;\n  --md-deep-orange-A100: #ff9e80;\n  --md-deep-orange-A200: #ff6e40;\n  --md-deep-orange-A400: #ff3d00;\n  --md-deep-orange-A700: #dd2c00;\n\n  --md-brown-50: #efebe9;\n  --md-brown-100: #d7ccc8;\n  --md-brown-200: #bcaaa4;\n  --md-brown-300: #a1887f;\n  --md-brown-400: #8d6e63;\n  --md-brown-500: #795548;\n  --md-brown-600: #6d4c41;\n  --md-brown-700: #5d4037;\n  --md-brown-800: #4e342e;\n  --md-brown-900: #3e2723;\n\n  --md-grey-50: #fafafa;\n  --md-grey-100: #f5f5f5;\n  --md-grey-200: #eeeeee;\n  --md-grey-300: #e0e0e0;\n  --md-grey-400: #bdbdbd;\n  --md-grey-500: #9e9e9e;\n  --md-grey-600: #757575;\n  --md-grey-700: #616161;\n  --md-grey-800: #424242;\n  --md-grey-900: #212121;\n\n  --md-blue-grey-50: #eceff1;\n  --md-blue-grey-100: #cfd8dc;\n  --md-blue-grey-200: #b0bec5;\n  --md-blue-grey-300: #90a4ae;\n  --md-blue-grey-400: #78909c;\n  --md-blue-grey-500: #607d8b;\n  --md-blue-grey-600: #546e7a;\n  --md-blue-grey-700: #455a64;\n  --md-blue-grey-800: #37474f;\n  --md-blue-grey-900: #263238;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2017, Jupyter Development Team.\n|\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n.jp-Spinner {\n  position: absolute;\n  display: flex;\n  justify-content: center;\n  align-items: center;\n  z-index: 10;\n  left: 0;\n  top: 0;\n  width: 100%;\n  height: 100%;\n  background: var(--jp-layout-color0);\n  outline: none;\n}\n\n.jp-SpinnerContent {\n  font-size: 10px;\n  margin: 50px auto;\n  text-indent: -9999em;\n  width: 3em;\n  height: 3em;\n  border-radius: 50%;\n  background: var(--jp-brand-color3);\n  background: linear-gradient(\n    to right,\n    #f37626 10%,\n    rgba(255, 255, 255, 0) 42%\n  );\n  position: relative;\n  animation: load3 1s infinite linear, fadeIn 1s;\n}\n\n.jp-SpinnerContent:before {\n  width: 50%;\n  height: 50%;\n  background: #f37626;\n  border-radius: 100% 0 0 0;\n  position: absolute;\n  top: 0;\n  left: 0;\n  content: '';\n}\n\n.jp-SpinnerContent:after {\n  background: var(--jp-layout-color0);\n  width: 75%;\n  height: 75%;\n  border-radius: 50%;\n  content: '';\n  margin: auto;\n  position: absolute;\n  top: 0;\n  left: 0;\n  bottom: 0;\n  right: 0;\n}\n\n@keyframes fadeIn {\n  0% {\n    opacity: 0;\n  }\n  100% {\n    opacity: 1;\n  }\n}\n\n@keyframes load3 {\n  0% {\n    transform: rotate(0deg);\n  }\n  100% {\n    transform: rotate(360deg);\n  }\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2017, Jupyter Development Team.\n|\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\nbutton.jp-mod-styled {\n  font-size: var(--jp-ui-font-size1);\n  color: var(--jp-ui-font-color0);\n  border: none;\n  box-sizing: border-box;\n  text-align: center;\n  line-height: 32px;\n  height: 32px;\n  padding: 0px 12px;\n  letter-spacing: 0.8px;\n  outline: none;\n  appearance: none;\n  -webkit-appearance: none;\n  -moz-appearance: none;\n}\n\ninput.jp-mod-styled {\n  background: var(--jp-input-background);\n  height: 28px;\n  box-sizing: border-box;\n  border: var(--jp-border-width) solid var(--jp-border-color1);\n  padding-left: 7px;\n  padding-right: 7px;\n  font-size: var(--jp-ui-font-size2);\n  color: var(--jp-ui-font-color0);\n  outline: none;\n  appearance: none;\n  -webkit-appearance: none;\n  -moz-appearance: none;\n}\n\ninput.jp-mod-styled:focus {\n  border: var(--jp-border-width) solid var(--md-blue-500);\n  box-shadow: inset 0 0 4px var(--md-blue-300);\n}\n\n.jp-select-wrapper {\n  display: flex;\n  flex-direction: column;\n  padding: 1px;\n  background-color: var(--jp-layout-color1);\n  height: 28px;\n  box-sizing: border-box;\n  margin-bottom: 12px;\n}\n\n.jp-select-wrapper.jp-mod-focused select.jp-mod-styled {\n  border: var(--jp-border-width) solid var(--jp-input-active-border-color);\n  box-shadow: var(--jp-input-box-shadow);\n  background-color: var(--jp-input-active-background);\n}\n\nselect.jp-mod-styled:hover {\n  background-color: var(--jp-layout-color1);\n  cursor: pointer;\n  color: var(--jp-ui-font-color0);\n  background-color: var(--jp-input-hover-background);\n  box-shadow: inset 0 0px 1px rgba(0, 0, 0, 0.5);\n}\n\nselect.jp-mod-styled {\n  flex: 1 1 auto;\n  height: 32px;\n  width: 100%;\n  font-size: var(--jp-ui-font-size2);\n  background: var(--jp-input-background);\n  color: var(--jp-ui-font-color0);\n  padding-left: 8px;\n  padding-right: 8px;\n  background-image: var(--jp-ui-select-caret);\n  background-repeat: no-repeat;\n  background-position: 99% center;\n  background-size: 18px;\n  border: var(--jp-border-width) solid var(--jp-input-border-color);\n  border-radius: 0px;\n  outline: none;\n  appearance: none;\n  -webkit-appearance: none;\n  -moz-appearance: none;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2016, Jupyter Development Team.\n|\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n:root {\n  --jp-private-toolbar-height: calc(\n    28px + var(--jp-border-width)\n  ); /* leave 28px for content */\n}\n\n.jp-Toolbar {\n  color: var(--jp-ui-font-color1);\n  flex: 0 0 auto;\n  display: flex;\n  flex-direction: row;\n  border-bottom: var(--jp-border-width) solid var(--jp-toolbar-border-color);\n  box-shadow: var(--jp-toolbar-box-shadow);\n  background: var(--jp-toolbar-background);\n  min-height: var(--jp-toolbar-micro-height);\n  padding: 2px;\n  z-index: 1;\n}\n\n/* Toolbar items */\n\n.jp-Toolbar > .jp-Toolbar-item.jp-Toolbar-spacer {\n  flex-grow: 1;\n  flex-shrink: 1;\n}\n\n.jp-Toolbar-item.jp-Toolbar-kernelStatus {\n  display: inline-block;\n  width: 32px;\n  background-repeat: no-repeat;\n  background-position: center;\n  background-size: 16px;\n}\n\n.jp-Toolbar > .jp-Toolbar-item {\n  flex: 0 0 auto;\n  display: flex;\n  padding-left: 1px;\n  padding-right: 1px;\n  font-size: var(--jp-ui-font-size1);\n  line-height: var(--jp-private-toolbar-height);\n  height: 100%;\n}\n\n/* Toolbar buttons */\n\n/* This is the div we use to wrap the react component into a Widget */\ndiv.jp-ToolbarButton {\n  color: transparent;\n  border: none;\n  box-sizing: border-box;\n  outline: none;\n  appearance: none;\n  -webkit-appearance: none;\n  -moz-appearance: none;\n  padding: 0px;\n  margin: 0px;\n}\n\nbutton.jp-ToolbarButtonComponent {\n  background: var(--jp-layout-color1);\n  border: none;\n  box-sizing: border-box;\n  outline: none;\n  appearance: none;\n  -webkit-appearance: none;\n  -moz-appearance: none;\n  padding: 0px 6px;\n  margin: 0px;\n  height: 24px;\n  border-radius: var(--jp-border-radius);\n  display: flex;\n  align-items: center;\n  text-align: center;\n  font-size: 14px;\n  min-width: unset;\n  min-height: unset;\n}\n\nbutton.jp-ToolbarButtonComponent:disabled {\n  opacity: 0.4;\n}\n\nbutton.jp-ToolbarButtonComponent span {\n  padding: 0px;\n  flex: 0 0 auto;\n}\n\nbutton.jp-ToolbarButtonComponent .jp-ToolbarButtonComponent-label {\n  font-size: var(--jp-ui-font-size1);\n  line-height: 100%;\n  padding-left: 2px;\n  color: var(--jp-ui-font-color1);\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) 2014-2016, Jupyter Development Team.\n|\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n.jp-JSONEditor {\n  display: flex;\n  flex-direction: column;\n  width: 100%;\n}\n\n.jp-JSONEditor-host {\n  flex: 1 1 auto;\n  border: var(--jp-border-width) solid var(--jp-input-border-color);\n  border-radius: 0px;\n  background: var(--jp-layout-color0);\n  min-height: 50px;\n  padding: 1px;\n}\n\n.jp-JSONEditor.jp-mod-error .jp-JSONEditor-host {\n  border-color: red;\n  outline-color: red;\n}\n\n.jp-JSONEditor-header {\n  display: flex;\n  flex: 1 0 auto;\n  padding: 0 0 0 12px;\n}\n\n.jp-JSONEditor-header label {\n  flex: 0 0 auto;\n}\n\n.jp-JSONEditor-commitButton,\n.jp-JSONEditor-revertButton {\n  height: 16px;\n  width: 16px;\n  background-size: 18px;\n  background-repeat: no-repeat;\n  background-position: center;\n}\n\n.jp-JSONEditor-commitButton {\n  background-image: var(--jp-icon-checkmark);\n  margin-left: 8px;\n}\n\n.jp-JSONEditor-revertButton {\n  background-image: var(--jp-icon-undo);\n}\n\n.jp-JSONEditor-host.jp-mod-focused {\n  background-color: var(--jp-input-active-background);\n  border: 1px solid var(--jp-input-active-border-color);\n  box-shadow: var(--jp-input-box-shadow);\n}\n\n.jp-Editor.jp-mod-dropTarget {\n  border: var(--jp-border-width) solid var(--jp-input-active-border-color);\n  box-shadow: var(--jp-input-box-shadow);\n}\n",""])},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(106),""),e.i(n(173),""),e.i(n(286),""),e.push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/* This file was auto-generated by ensurePackage() in @jupyterlab/buildutils */\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/* BASICS */\n\n.CodeMirror {\n  /* Set height, width, borders, and global font properties here */\n  font-family: monospace;\n  height: 300px;\n  color: black;\n  direction: ltr;\n}\n\n/* PADDING */\n\n.CodeMirror-lines {\n  padding: 4px 0; /* Vertical padding around content */\n}\n.CodeMirror pre {\n  padding: 0 4px; /* Horizontal padding of content */\n}\n\n.CodeMirror-scrollbar-filler, .CodeMirror-gutter-filler {\n  background-color: white; /* The little square between H and V scrollbars */\n}\n\n/* GUTTER */\n\n.CodeMirror-gutters {\n  border-right: 1px solid #ddd;\n  background-color: #f7f7f7;\n  white-space: nowrap;\n}\n.CodeMirror-linenumbers {}\n.CodeMirror-linenumber {\n  padding: 0 3px 0 5px;\n  min-width: 20px;\n  text-align: right;\n  color: #999;\n  white-space: nowrap;\n}\n\n.CodeMirror-guttermarker { color: black; }\n.CodeMirror-guttermarker-subtle { color: #999; }\n\n/* CURSOR */\n\n.CodeMirror-cursor {\n  border-left: 1px solid black;\n  border-right: none;\n  width: 0;\n}\n/* Shown when moving in bi-directional text */\n.CodeMirror div.CodeMirror-secondarycursor {\n  border-left: 1px solid silver;\n}\n.cm-fat-cursor .CodeMirror-cursor {\n  width: auto;\n  border: 0 !important;\n  background: #7e7;\n}\n.cm-fat-cursor div.CodeMirror-cursors {\n  z-index: 1;\n}\n.cm-fat-cursor-mark {\n  background-color: rgba(20, 255, 20, 0.5);\n  -webkit-animation: blink 1.06s steps(1) infinite;\n  -moz-animation: blink 1.06s steps(1) infinite;\n  animation: blink 1.06s steps(1) infinite;\n}\n.cm-animate-fat-cursor {\n  width: auto;\n  border: 0;\n  -webkit-animation: blink 1.06s steps(1) infinite;\n  -moz-animation: blink 1.06s steps(1) infinite;\n  animation: blink 1.06s steps(1) infinite;\n  background-color: #7e7;\n}\n@-moz-keyframes blink {\n  0% {}\n  50% { background-color: transparent; }\n  100% {}\n}\n@-webkit-keyframes blink {\n  0% {}\n  50% { background-color: transparent; }\n  100% {}\n}\n@keyframes blink {\n  0% {}\n  50% { background-color: transparent; }\n  100% {}\n}\n\n/* Can style cursor different in overwrite (non-insert) mode */\n.CodeMirror-overwrite .CodeMirror-cursor {}\n\n.cm-tab { display: inline-block; text-decoration: inherit; }\n\n.CodeMirror-rulers {\n  position: absolute;\n  left: 0; right: 0; top: -50px; bottom: -20px;\n  overflow: hidden;\n}\n.CodeMirror-ruler {\n  border-left: 1px solid #ccc;\n  top: 0; bottom: 0;\n  position: absolute;\n}\n\n/* DEFAULT THEME */\n\n.cm-s-default .cm-header {color: blue;}\n.cm-s-default .cm-quote {color: #090;}\n.cm-negative {color: #d44;}\n.cm-positive {color: #292;}\n.cm-header, .cm-strong {font-weight: bold;}\n.cm-em {font-style: italic;}\n.cm-link {text-decoration: underline;}\n.cm-strikethrough {text-decoration: line-through;}\n\n.cm-s-default .cm-keyword {color: #708;}\n.cm-s-default .cm-atom {color: #219;}\n.cm-s-default .cm-number {color: #164;}\n.cm-s-default .cm-def {color: #00f;}\n.cm-s-default .cm-variable,\n.cm-s-default .cm-punctuation,\n.cm-s-default .cm-property,\n.cm-s-default .cm-operator {}\n.cm-s-default .cm-variable-2 {color: #05a;}\n.cm-s-default .cm-variable-3, .cm-s-default .cm-type {color: #085;}\n.cm-s-default .cm-comment {color: #a50;}\n.cm-s-default .cm-string {color: #a11;}\n.cm-s-default .cm-string-2 {color: #f50;}\n.cm-s-default .cm-meta {color: #555;}\n.cm-s-default .cm-qualifier {color: #555;}\n.cm-s-default .cm-builtin {color: #30a;}\n.cm-s-default .cm-bracket {color: #997;}\n.cm-s-default .cm-tag {color: #170;}\n.cm-s-default .cm-attribute {color: #00c;}\n.cm-s-default .cm-hr {color: #999;}\n.cm-s-default .cm-link {color: #00c;}\n\n.cm-s-default .cm-error {color: #f00;}\n.cm-invalidchar {color: #f00;}\n\n.CodeMirror-composing { border-bottom: 2px solid; }\n\n/* Default styles for common addons */\n\ndiv.CodeMirror span.CodeMirror-matchingbracket {color: #0b0;}\ndiv.CodeMirror span.CodeMirror-nonmatchingbracket {color: #a22;}\n.CodeMirror-matchingtag { background: rgba(255, 150, 0, .3); }\n.CodeMirror-activeline-background {background: #e8f2ff;}\n\n/* STOP */\n\n/* The rest of this file contains styles related to the mechanics of\n   the editor. You probably shouldn't touch them. */\n\n.CodeMirror {\n  position: relative;\n  overflow: hidden;\n  background: white;\n}\n\n.CodeMirror-scroll {\n  overflow: scroll !important; /* Things will break if this is overridden */\n  /* 30px is the magic margin used to hide the element's real scrollbars */\n  /* See overflow: hidden in .CodeMirror */\n  margin-bottom: -30px; margin-right: -30px;\n  padding-bottom: 30px;\n  height: 100%;\n  outline: none; /* Prevent dragging from highlighting the element */\n  position: relative;\n}\n.CodeMirror-sizer {\n  position: relative;\n  border-right: 30px solid transparent;\n}\n\n/* The fake, visible scrollbars. Used to force redraw during scrolling\n   before actual scrolling happens, thus preventing shaking and\n   flickering artifacts. */\n.CodeMirror-vscrollbar, .CodeMirror-hscrollbar, .CodeMirror-scrollbar-filler, .CodeMirror-gutter-filler {\n  position: absolute;\n  z-index: 6;\n  display: none;\n}\n.CodeMirror-vscrollbar {\n  right: 0; top: 0;\n  overflow-x: hidden;\n  overflow-y: scroll;\n}\n.CodeMirror-hscrollbar {\n  bottom: 0; left: 0;\n  overflow-y: hidden;\n  overflow-x: scroll;\n}\n.CodeMirror-scrollbar-filler {\n  right: 0; bottom: 0;\n}\n.CodeMirror-gutter-filler {\n  left: 0; bottom: 0;\n}\n\n.CodeMirror-gutters {\n  position: absolute; left: 0; top: 0;\n  min-height: 100%;\n  z-index: 3;\n}\n.CodeMirror-gutter {\n  white-space: normal;\n  height: 100%;\n  display: inline-block;\n  vertical-align: top;\n  margin-bottom: -30px;\n}\n.CodeMirror-gutter-wrapper {\n  position: absolute;\n  z-index: 4;\n  background: none !important;\n  border: none !important;\n}\n.CodeMirror-gutter-background {\n  position: absolute;\n  top: 0; bottom: 0;\n  z-index: 4;\n}\n.CodeMirror-gutter-elt {\n  position: absolute;\n  cursor: default;\n  z-index: 4;\n}\n.CodeMirror-gutter-wrapper ::selection { background-color: transparent }\n.CodeMirror-gutter-wrapper ::-moz-selection { background-color: transparent }\n\n.CodeMirror-lines {\n  cursor: text;\n  min-height: 1px; /* prevents collapsing before first draw */\n}\n.CodeMirror pre {\n  /* Reset some styles that the rest of the page might have set */\n  -moz-border-radius: 0; -webkit-border-radius: 0; border-radius: 0;\n  border-width: 0;\n  background: transparent;\n  font-family: inherit;\n  font-size: inherit;\n  margin: 0;\n  white-space: pre;\n  word-wrap: normal;\n  line-height: inherit;\n  color: inherit;\n  z-index: 2;\n  position: relative;\n  overflow: visible;\n  -webkit-tap-highlight-color: transparent;\n  -webkit-font-variant-ligatures: contextual;\n  font-variant-ligatures: contextual;\n}\n.CodeMirror-wrap pre {\n  word-wrap: break-word;\n  white-space: pre-wrap;\n  word-break: normal;\n}\n\n.CodeMirror-linebackground {\n  position: absolute;\n  left: 0; right: 0; top: 0; bottom: 0;\n  z-index: 0;\n}\n\n.CodeMirror-linewidget {\n  position: relative;\n  z-index: 2;\n  padding: 0.1px; /* Force widget margins to stay inside of the container */\n}\n\n.CodeMirror-widget {}\n\n.CodeMirror-rtl pre { direction: rtl; }\n\n.CodeMirror-code {\n  outline: none;\n}\n\n/* Force content-box sizing for the elements where we expect it */\n.CodeMirror-scroll,\n.CodeMirror-sizer,\n.CodeMirror-gutter,\n.CodeMirror-gutters,\n.CodeMirror-linenumber {\n  -moz-box-sizing: content-box;\n  box-sizing: content-box;\n}\n\n.CodeMirror-measure {\n  position: absolute;\n  width: 100%;\n  height: 0;\n  overflow: hidden;\n  visibility: hidden;\n}\n\n.CodeMirror-cursor {\n  position: absolute;\n  pointer-events: none;\n}\n.CodeMirror-measure pre { position: static; }\n\ndiv.CodeMirror-cursors {\n  visibility: hidden;\n  position: relative;\n  z-index: 3;\n}\ndiv.CodeMirror-dragcursors {\n  visibility: visible;\n}\n\n.CodeMirror-focused div.CodeMirror-cursors {\n  visibility: visible;\n}\n\n.CodeMirror-selected { background: #d9d9d9; }\n.CodeMirror-focused .CodeMirror-selected { background: #d7d4f0; }\n.CodeMirror-crosshair { cursor: crosshair; }\n.CodeMirror-line::selection, .CodeMirror-line > span::selection, .CodeMirror-line > span > span::selection { background: #d7d4f0; }\n.CodeMirror-line::-moz-selection, .CodeMirror-line > span::-moz-selection, .CodeMirror-line > span > span::-moz-selection { background: #d7d4f0; }\n\n.cm-searching {\n  background-color: #ffa;\n  background-color: rgba(255, 255, 0, .4);\n}\n\n/* Used to force a border model for a node */\n.cm-force-border { padding-right: .1px; }\n\n@media print {\n  /* Hide the cursor when printing */\n  .CodeMirror div.CodeMirror-cursors {\n    visibility: hidden;\n  }\n}\n\n/* See issue #2901 */\n.cm-tab-wrap-hack:after { content: ''; }\n\n/* Help users use markselection to safely style text background */\nspan.CodeMirror-selectedtext { background: none; }\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".CodeMirror-dialog {\n  position: absolute;\n  left: 0; right: 0;\n  background: inherit;\n  z-index: 15;\n  padding: .1em .8em;\n  overflow: hidden;\n  color: inherit;\n}\n\n.CodeMirror-dialog-top {\n  border-bottom: 1px solid #eee;\n  top: 0;\n}\n\n.CodeMirror-dialog-bottom {\n  border-top: 1px solid #eee;\n  bottom: 0;\n}\n\n.CodeMirror-dialog input {\n  border: none;\n  outline: none;\n  background: transparent;\n  width: 20em;\n  color: inherit;\n  font-family: monospace;\n}\n\n.CodeMirror-dialog button {\n  font-size: 70%;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,'.CodeMirror-foldmarker {\n  color: blue;\n  text-shadow: #b9f 1px 1px 2px, #b9f -1px -1px 2px, #b9f 1px -1px 2px, #b9f -1px 1px 2px;\n  font-family: arial;\n  line-height: .3;\n  cursor: pointer;\n}\n.CodeMirror-foldgutter {\n  width: .7em;\n}\n.CodeMirror-foldgutter-open,\n.CodeMirror-foldgutter-folded {\n  cursor: pointer;\n}\n.CodeMirror-foldgutter-open:after {\n  content: "\\25BE";\n}\n.CodeMirror-foldgutter-folded:after {\n  content: "\\25B8";\n}\n',""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*\n\n    Name:       material\n    Author:     Michael Kaminsky (http://github.com/mkaminsky11)\n\n    Original material color scheme by Mattia Astorino (https://github.com/equinusocio/material-theme)\n\n*/\n\n.cm-s-material.CodeMirror {\n  background-color: #263238;\n  color: rgba(233, 237, 237, 1);\n}\n.cm-s-material .CodeMirror-gutters {\n  background: #263238;\n  color: rgb(83,127,126);\n  border: none;\n}\n.cm-s-material .CodeMirror-guttermarker, .cm-s-material .CodeMirror-guttermarker-subtle, .cm-s-material .CodeMirror-linenumber { color: rgb(83,127,126); }\n.cm-s-material .CodeMirror-cursor { border-left: 1px solid #f8f8f0; }\n.cm-s-material div.CodeMirror-selected { background: rgba(255, 255, 255, 0.15); }\n.cm-s-material.CodeMirror-focused div.CodeMirror-selected { background: rgba(255, 255, 255, 0.10); }\n.cm-s-material .CodeMirror-line::selection, .cm-s-material .CodeMirror-line > span::selection, .cm-s-material .CodeMirror-line > span > span::selection { background: rgba(255, 255, 255, 0.10); }\n.cm-s-material .CodeMirror-line::-moz-selection, .cm-s-material .CodeMirror-line > span::-moz-selection, .cm-s-material .CodeMirror-line > span > span::-moz-selection { background: rgba(255, 255, 255, 0.10); }\n\n.cm-s-material .CodeMirror-activeline-background { background: rgba(0, 0, 0, 0); }\n.cm-s-material .cm-keyword { color: rgba(199, 146, 234, 1); }\n.cm-s-material .cm-operator { color: rgba(233, 237, 237, 1); }\n.cm-s-material .cm-variable-2 { color: #80CBC4; }\n.cm-s-material .cm-variable-3, .cm-s-material .cm-type { color: #82B1FF; }\n.cm-s-material .cm-builtin { color: #DECB6B; }\n.cm-s-material .cm-atom { color: #F77669; }\n.cm-s-material .cm-number { color: #F77669; }\n.cm-s-material .cm-def { color: rgba(233, 237, 237, 1); }\n.cm-s-material .cm-string { color: #C3E88D; }\n.cm-s-material .cm-string-2 { color: #80CBC4; }\n.cm-s-material .cm-comment { color: #546E7A; }\n.cm-s-material .cm-variable { color: #82B1FF; }\n.cm-s-material .cm-tag { color: #80CBC4; }\n.cm-s-material .cm-meta { color: #80CBC4; }\n.cm-s-material .cm-attribute { color: #FFCB6B; }\n.cm-s-material .cm-property { color: #80CBAE; }\n.cm-s-material .cm-qualifier { color: #DECB6B; }\n.cm-s-material .cm-variable-3, .cm-s-material .cm-type { color: #DECB6B; }\n.cm-s-material .cm-tag { color: rgba(255, 83, 112, 1); }\n.cm-s-material .cm-error {\n  color: rgba(255, 255, 255, 1.0);\n  background-color: #EC5F67;\n}\n.cm-s-material .CodeMirror-matchingbracket {\n  text-decoration: underline;\n  color: white !important;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,'/**\n * "\n *  Using Zenburn color palette from the Emacs Zenburn Theme\n *  https://github.com/bbatsov/zenburn-emacs/blob/master/zenburn-theme.el\n *\n *  Also using parts of https://github.com/xavi/coderay-lighttable-theme\n * "\n * From: https://github.com/wisenomad/zenburn-lighttable-theme/blob/master/zenburn.css\n */\n\n.cm-s-zenburn .CodeMirror-gutters { background: #3f3f3f !important; }\n.cm-s-zenburn .CodeMirror-foldgutter-open, .CodeMirror-foldgutter-folded { color: #999; }\n.cm-s-zenburn .CodeMirror-cursor { border-left: 1px solid white; }\n.cm-s-zenburn { background-color: #3f3f3f; color: #dcdccc; }\n.cm-s-zenburn span.cm-builtin { color: #dcdccc; font-weight: bold; }\n.cm-s-zenburn span.cm-comment { color: #7f9f7f; }\n.cm-s-zenburn span.cm-keyword { color: #f0dfaf; font-weight: bold; }\n.cm-s-zenburn span.cm-atom { color: #bfebbf; }\n.cm-s-zenburn span.cm-def { color: #dcdccc; }\n.cm-s-zenburn span.cm-variable { color: #dfaf8f; }\n.cm-s-zenburn span.cm-variable-2 { color: #dcdccc; }\n.cm-s-zenburn span.cm-string { color: #cc9393; }\n.cm-s-zenburn span.cm-string-2 { color: #cc9393; }\n.cm-s-zenburn span.cm-number { color: #dcdccc; }\n.cm-s-zenburn span.cm-tag { color: #93e0e3; }\n.cm-s-zenburn span.cm-property { color: #dfaf8f; }\n.cm-s-zenburn span.cm-attribute { color: #dfaf8f; }\n.cm-s-zenburn span.cm-qualifier { color: #7cb8bb; }\n.cm-s-zenburn span.cm-meta { color: #f0dfaf; }\n.cm-s-zenburn span.cm-header { color: #f0efd0; }\n.cm-s-zenburn span.cm-operator { color: #f0efd0; }\n.cm-s-zenburn span.CodeMirror-matchingbracket { box-sizing: border-box; background: transparent; border-bottom: 1px solid; }\n.cm-s-zenburn span.CodeMirror-nonmatchingbracket { border-bottom: 1px solid; background: none; }\n.cm-s-zenburn .CodeMirror-activeline { background: #000000; }\n.cm-s-zenburn .CodeMirror-activeline-background { background: #000000; }\n.cm-s-zenburn div.CodeMirror-selected { background: #545454; }\n.cm-s-zenburn .CodeMirror-focused div.CodeMirror-selected { background: #4f4f4f; }\n',""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".cm-s-abcdef.CodeMirror { background: #0f0f0f; color: #defdef; }\n.cm-s-abcdef div.CodeMirror-selected { background: #515151; }\n.cm-s-abcdef .CodeMirror-line::selection, .cm-s-abcdef .CodeMirror-line > span::selection, .cm-s-abcdef .CodeMirror-line > span > span::selection { background: rgba(56, 56, 56, 0.99); }\n.cm-s-abcdef .CodeMirror-line::-moz-selection, .cm-s-abcdef .CodeMirror-line > span::-moz-selection, .cm-s-abcdef .CodeMirror-line > span > span::-moz-selection { background: rgba(56, 56, 56, 0.99); }\n.cm-s-abcdef .CodeMirror-gutters { background: #555; border-right: 2px solid #314151; }\n.cm-s-abcdef .CodeMirror-guttermarker { color: #222; }\n.cm-s-abcdef .CodeMirror-guttermarker-subtle { color: azure; }\n.cm-s-abcdef .CodeMirror-linenumber { color: #FFFFFF; }\n.cm-s-abcdef .CodeMirror-cursor { border-left: 1px solid #00FF00; }\n\n.cm-s-abcdef span.cm-keyword { color: darkgoldenrod; font-weight: bold; }\n.cm-s-abcdef span.cm-atom { color: #77F; }\n.cm-s-abcdef span.cm-number { color: violet; }\n.cm-s-abcdef span.cm-def { color: #fffabc; }\n.cm-s-abcdef span.cm-variable { color: #abcdef; }\n.cm-s-abcdef span.cm-variable-2 { color: #cacbcc; }\n.cm-s-abcdef span.cm-variable-3, .cm-s-abcdef span.cm-type { color: #def; }\n.cm-s-abcdef span.cm-property { color: #fedcba; }\n.cm-s-abcdef span.cm-operator { color: #ff0; }\n.cm-s-abcdef span.cm-comment { color: #7a7b7c; font-style: italic;}\n.cm-s-abcdef span.cm-string { color: #2b4; }\n.cm-s-abcdef span.cm-meta { color: #C9F; }\n.cm-s-abcdef span.cm-qualifier { color: #FFF700; }\n.cm-s-abcdef span.cm-builtin { color: #30aabc; }\n.cm-s-abcdef span.cm-bracket { color: #8a8a8a; }\n.cm-s-abcdef span.cm-tag { color: #FFDD44; }\n.cm-s-abcdef span.cm-attribute { color: #DDFF00; }\n.cm-s-abcdef span.cm-error { color: #FF0000; }\n.cm-s-abcdef span.cm-header { color: aquamarine; font-weight: bold; }\n.cm-s-abcdef span.cm-link { color: blueviolet; }\n\n.cm-s-abcdef .CodeMirror-activeline-background { background: #314151; }\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*\n\n    Name:       Base16 Default Light\n    Author:     Chris Kempson (http://chriskempson.com)\n\n    CodeMirror template by Jan T. Sott (https://github.com/idleberg/base16-codemirror)\n    Original Base16 color scheme by Chris Kempson (https://github.com/chriskempson/base16)\n\n*/\n\n.cm-s-base16-light.CodeMirror { background: #f5f5f5; color: #202020; }\n.cm-s-base16-light div.CodeMirror-selected { background: #e0e0e0; }\n.cm-s-base16-light .CodeMirror-line::selection, .cm-s-base16-light .CodeMirror-line > span::selection, .cm-s-base16-light .CodeMirror-line > span > span::selection { background: #e0e0e0; }\n.cm-s-base16-light .CodeMirror-line::-moz-selection, .cm-s-base16-light .CodeMirror-line > span::-moz-selection, .cm-s-base16-light .CodeMirror-line > span > span::-moz-selection { background: #e0e0e0; }\n.cm-s-base16-light .CodeMirror-gutters { background: #f5f5f5; border-right: 0px; }\n.cm-s-base16-light .CodeMirror-guttermarker { color: #ac4142; }\n.cm-s-base16-light .CodeMirror-guttermarker-subtle { color: #b0b0b0; }\n.cm-s-base16-light .CodeMirror-linenumber { color: #b0b0b0; }\n.cm-s-base16-light .CodeMirror-cursor { border-left: 1px solid #505050; }\n\n.cm-s-base16-light span.cm-comment { color: #8f5536; }\n.cm-s-base16-light span.cm-atom { color: #aa759f; }\n.cm-s-base16-light span.cm-number { color: #aa759f; }\n\n.cm-s-base16-light span.cm-property, .cm-s-base16-light span.cm-attribute { color: #90a959; }\n.cm-s-base16-light span.cm-keyword { color: #ac4142; }\n.cm-s-base16-light span.cm-string { color: #f4bf75; }\n\n.cm-s-base16-light span.cm-variable { color: #90a959; }\n.cm-s-base16-light span.cm-variable-2 { color: #6a9fb5; }\n.cm-s-base16-light span.cm-def { color: #d28445; }\n.cm-s-base16-light span.cm-bracket { color: #202020; }\n.cm-s-base16-light span.cm-tag { color: #ac4142; }\n.cm-s-base16-light span.cm-link { color: #aa759f; }\n.cm-s-base16-light span.cm-error { background: #ac4142; color: #505050; }\n\n.cm-s-base16-light .CodeMirror-activeline-background { background: #DDDCDC; }\n.cm-s-base16-light .CodeMirror-matchingbracket { color: #f5f5f5 !important; background-color: #6A9FB5 !important}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*\n\n    Name:       Base16 Default Dark\n    Author:     Chris Kempson (http://chriskempson.com)\n\n    CodeMirror template by Jan T. Sott (https://github.com/idleberg/base16-codemirror)\n    Original Base16 color scheme by Chris Kempson (https://github.com/chriskempson/base16)\n\n*/\n\n.cm-s-base16-dark.CodeMirror { background: #151515; color: #e0e0e0; }\n.cm-s-base16-dark div.CodeMirror-selected { background: #303030; }\n.cm-s-base16-dark .CodeMirror-line::selection, .cm-s-base16-dark .CodeMirror-line > span::selection, .cm-s-base16-dark .CodeMirror-line > span > span::selection { background: rgba(48, 48, 48, .99); }\n.cm-s-base16-dark .CodeMirror-line::-moz-selection, .cm-s-base16-dark .CodeMirror-line > span::-moz-selection, .cm-s-base16-dark .CodeMirror-line > span > span::-moz-selection { background: rgba(48, 48, 48, .99); }\n.cm-s-base16-dark .CodeMirror-gutters { background: #151515; border-right: 0px; }\n.cm-s-base16-dark .CodeMirror-guttermarker { color: #ac4142; }\n.cm-s-base16-dark .CodeMirror-guttermarker-subtle { color: #505050; }\n.cm-s-base16-dark .CodeMirror-linenumber { color: #505050; }\n.cm-s-base16-dark .CodeMirror-cursor { border-left: 1px solid #b0b0b0; }\n\n.cm-s-base16-dark span.cm-comment { color: #8f5536; }\n.cm-s-base16-dark span.cm-atom { color: #aa759f; }\n.cm-s-base16-dark span.cm-number { color: #aa759f; }\n\n.cm-s-base16-dark span.cm-property, .cm-s-base16-dark span.cm-attribute { color: #90a959; }\n.cm-s-base16-dark span.cm-keyword { color: #ac4142; }\n.cm-s-base16-dark span.cm-string { color: #f4bf75; }\n\n.cm-s-base16-dark span.cm-variable { color: #90a959; }\n.cm-s-base16-dark span.cm-variable-2 { color: #6a9fb5; }\n.cm-s-base16-dark span.cm-def { color: #d28445; }\n.cm-s-base16-dark span.cm-bracket { color: #e0e0e0; }\n.cm-s-base16-dark span.cm-tag { color: #ac4142; }\n.cm-s-base16-dark span.cm-link { color: #aa759f; }\n.cm-s-base16-dark span.cm-error { background: #ac4142; color: #b0b0b0; }\n\n.cm-s-base16-dark .CodeMirror-activeline-background { background: #202020; }\n.cm-s-base16-dark .CodeMirror-matchingbracket { text-decoration: underline; color: white !important; }\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*\n\n    Name:       dracula\n    Author:     Michael Kaminsky (http://github.com/mkaminsky11)\n\n    Original dracula color scheme by Zeno Rocha (https://github.com/zenorocha/dracula-theme)\n\n*/\n\n\n.cm-s-dracula.CodeMirror, .cm-s-dracula .CodeMirror-gutters {\n  background-color: #282a36 !important;\n  color: #f8f8f2 !important;\n  border: none;\n}\n.cm-s-dracula .CodeMirror-gutters { color: #282a36; }\n.cm-s-dracula .CodeMirror-cursor { border-left: solid thin #f8f8f0; }\n.cm-s-dracula .CodeMirror-linenumber { color: #6D8A88; }\n.cm-s-dracula .CodeMirror-selected { background: rgba(255, 255, 255, 0.10); }\n.cm-s-dracula .CodeMirror-line::selection, .cm-s-dracula .CodeMirror-line > span::selection, .cm-s-dracula .CodeMirror-line > span > span::selection { background: rgba(255, 255, 255, 0.10); }\n.cm-s-dracula .CodeMirror-line::-moz-selection, .cm-s-dracula .CodeMirror-line > span::-moz-selection, .cm-s-dracula .CodeMirror-line > span > span::-moz-selection { background: rgba(255, 255, 255, 0.10); }\n.cm-s-dracula span.cm-comment { color: #6272a4; }\n.cm-s-dracula span.cm-string, .cm-s-dracula span.cm-string-2 { color: #f1fa8c; }\n.cm-s-dracula span.cm-number { color: #bd93f9; }\n.cm-s-dracula span.cm-variable { color: #50fa7b; }\n.cm-s-dracula span.cm-variable-2 { color: white; }\n.cm-s-dracula span.cm-def { color: #50fa7b; }\n.cm-s-dracula span.cm-operator { color: #ff79c6; }\n.cm-s-dracula span.cm-keyword { color: #ff79c6; }\n.cm-s-dracula span.cm-atom { color: #bd93f9; }\n.cm-s-dracula span.cm-meta { color: #f8f8f2; }\n.cm-s-dracula span.cm-tag { color: #ff79c6; }\n.cm-s-dracula span.cm-attribute { color: #50fa7b; }\n.cm-s-dracula span.cm-qualifier { color: #50fa7b; }\n.cm-s-dracula span.cm-property { color: #66d9ef; }\n.cm-s-dracula span.cm-builtin { color: #50fa7b; }\n.cm-s-dracula span.cm-variable-3, .cm-s-dracula span.cm-type { color: #ffb86c; }\n\n.cm-s-dracula .CodeMirror-activeline-background { background: rgba(255,255,255,0.1); }\n.cm-s-dracula .CodeMirror-matchingbracket { text-decoration: underline; color: white !important; }\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*\n\n    Name:       Hopscotch\n    Author:     Jan T. Sott\n\n    CodeMirror template by Jan T. Sott (https://github.com/idleberg/base16-codemirror)\n    Original Base16 color scheme by Chris Kempson (https://github.com/chriskempson/base16)\n\n*/\n\n.cm-s-hopscotch.CodeMirror {background: #322931; color: #d5d3d5;}\n.cm-s-hopscotch div.CodeMirror-selected {background: #433b42 !important;}\n.cm-s-hopscotch .CodeMirror-gutters {background: #322931; border-right: 0px;}\n.cm-s-hopscotch .CodeMirror-linenumber {color: #797379;}\n.cm-s-hopscotch .CodeMirror-cursor {border-left: 1px solid #989498 !important;}\n\n.cm-s-hopscotch span.cm-comment {color: #b33508;}\n.cm-s-hopscotch span.cm-atom {color: #c85e7c;}\n.cm-s-hopscotch span.cm-number {color: #c85e7c;}\n\n.cm-s-hopscotch span.cm-property, .cm-s-hopscotch span.cm-attribute {color: #8fc13e;}\n.cm-s-hopscotch span.cm-keyword {color: #dd464c;}\n.cm-s-hopscotch span.cm-string {color: #fdcc59;}\n\n.cm-s-hopscotch span.cm-variable {color: #8fc13e;}\n.cm-s-hopscotch span.cm-variable-2 {color: #1290bf;}\n.cm-s-hopscotch span.cm-def {color: #fd8b19;}\n.cm-s-hopscotch span.cm-error {background: #dd464c; color: #989498;}\n.cm-s-hopscotch span.cm-bracket {color: #d5d3d5;}\n.cm-s-hopscotch span.cm-tag {color: #dd464c;}\n.cm-s-hopscotch span.cm-link {color: #c85e7c;}\n\n.cm-s-hopscotch .CodeMirror-matchingbracket { text-decoration: underline; color: white !important;}\n.cm-s-hopscotch .CodeMirror-activeline-background { background: #302020; }\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/****************************************************************/\n/*   Based on mbonaci's Brackets mbo theme                      */\n/*   https://github.com/mbonaci/global/blob/master/Mbo.tmTheme  */\n/*   Create your own: http://tmtheme-editor.herokuapp.com       */\n/****************************************************************/\n\n.cm-s-mbo.CodeMirror { background: #2c2c2c; color: #ffffec; }\n.cm-s-mbo div.CodeMirror-selected { background: #716C62; }\n.cm-s-mbo .CodeMirror-line::selection, .cm-s-mbo .CodeMirror-line > span::selection, .cm-s-mbo .CodeMirror-line > span > span::selection { background: rgba(113, 108, 98, .99); }\n.cm-s-mbo .CodeMirror-line::-moz-selection, .cm-s-mbo .CodeMirror-line > span::-moz-selection, .cm-s-mbo .CodeMirror-line > span > span::-moz-selection { background: rgba(113, 108, 98, .99); }\n.cm-s-mbo .CodeMirror-gutters { background: #4e4e4e; border-right: 0px; }\n.cm-s-mbo .CodeMirror-guttermarker { color: white; }\n.cm-s-mbo .CodeMirror-guttermarker-subtle { color: grey; }\n.cm-s-mbo .CodeMirror-linenumber { color: #dadada; }\n.cm-s-mbo .CodeMirror-cursor { border-left: 1px solid #ffffec; }\n\n.cm-s-mbo span.cm-comment { color: #95958a; }\n.cm-s-mbo span.cm-atom { color: #00a8c6; }\n.cm-s-mbo span.cm-number { color: #00a8c6; }\n\n.cm-s-mbo span.cm-property, .cm-s-mbo span.cm-attribute { color: #9ddfe9; }\n.cm-s-mbo span.cm-keyword { color: #ffb928; }\n.cm-s-mbo span.cm-string { color: #ffcf6c; }\n.cm-s-mbo span.cm-string.cm-property { color: #ffffec; }\n\n.cm-s-mbo span.cm-variable { color: #ffffec; }\n.cm-s-mbo span.cm-variable-2 { color: #00a8c6; }\n.cm-s-mbo span.cm-def { color: #ffffec; }\n.cm-s-mbo span.cm-bracket { color: #fffffc; font-weight: bold; }\n.cm-s-mbo span.cm-tag { color: #9ddfe9; }\n.cm-s-mbo span.cm-link { color: #f54b07; }\n.cm-s-mbo span.cm-error { border-bottom: #636363; color: #ffffec; }\n.cm-s-mbo span.cm-qualifier { color: #ffffec; }\n\n.cm-s-mbo .CodeMirror-activeline-background { background: #494b41; }\n.cm-s-mbo .CodeMirror-matchingbracket { color: #ffb928 !important; }\n.cm-s-mbo .CodeMirror-matchingtag { background: rgba(255, 255, 255, .37); }\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*\n  MDN-LIKE Theme - Mozilla\n  Ported to CodeMirror by Peter Kroon <plakroon@gmail.com>\n  Report bugs/issues here: https://github.com/codemirror/CodeMirror/issues\n  GitHub: @peterkroon\n\n  The mdn-like theme is inspired on the displayed code examples at: https://developer.mozilla.org/en-US/docs/Web/CSS/animation\n\n*/\n.cm-s-mdn-like.CodeMirror { color: #999; background-color: #fff; }\n.cm-s-mdn-like div.CodeMirror-selected { background: #cfc; }\n.cm-s-mdn-like .CodeMirror-line::selection, .cm-s-mdn-like .CodeMirror-line > span::selection, .cm-s-mdn-like .CodeMirror-line > span > span::selection { background: #cfc; }\n.cm-s-mdn-like .CodeMirror-line::-moz-selection, .cm-s-mdn-like .CodeMirror-line > span::-moz-selection, .cm-s-mdn-like .CodeMirror-line > span > span::-moz-selection { background: #cfc; }\n\n.cm-s-mdn-like .CodeMirror-gutters { background: #f8f8f8; border-left: 6px solid rgba(0,83,159,0.65); color: #333; }\n.cm-s-mdn-like .CodeMirror-linenumber { color: #aaa; padding-left: 8px; }\n.cm-s-mdn-like .CodeMirror-cursor { border-left: 2px solid #222; }\n\n.cm-s-mdn-like .cm-keyword { color: #6262FF; }\n.cm-s-mdn-like .cm-atom { color: #F90; }\n.cm-s-mdn-like .cm-number { color:  #ca7841; }\n.cm-s-mdn-like .cm-def { color: #8DA6CE; }\n.cm-s-mdn-like span.cm-variable-2, .cm-s-mdn-like span.cm-tag { color: #690; }\n.cm-s-mdn-like span.cm-variable-3, .cm-s-mdn-like span.cm-def, .cm-s-mdn-like span.cm-type { color: #07a; }\n\n.cm-s-mdn-like .cm-variable { color: #07a; }\n.cm-s-mdn-like .cm-property { color: #905; }\n.cm-s-mdn-like .cm-qualifier { color: #690; }\n\n.cm-s-mdn-like .cm-operator { color: #cda869; }\n.cm-s-mdn-like .cm-comment { color:#777; font-weight:normal; }\n.cm-s-mdn-like .cm-string { color:#07a; font-style:italic; }\n.cm-s-mdn-like .cm-string-2 { color:#bd6b18; } /*?*/\n.cm-s-mdn-like .cm-meta { color: #000; } /*?*/\n.cm-s-mdn-like .cm-builtin { color: #9B7536; } /*?*/\n.cm-s-mdn-like .cm-tag { color: #997643; }\n.cm-s-mdn-like .cm-attribute { color: #d6bb6d; } /*?*/\n.cm-s-mdn-like .cm-header { color: #FF6400; }\n.cm-s-mdn-like .cm-hr { color: #AEAEAE; }\n.cm-s-mdn-like .cm-link { color:#ad9361; font-style:italic; text-decoration:none; }\n.cm-s-mdn-like .cm-error { border-bottom: 1px solid red; }\n\ndiv.cm-s-mdn-like .CodeMirror-activeline-background { background: #efefff; }\ndiv.cm-s-mdn-like span.CodeMirror-matchingbracket { outline:1px solid grey; color: inherit; }\n\n.cm-s-mdn-like.CodeMirror { background-image: url(data:image/png;base64,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); }\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*\n\n    Name:       seti\n    Author:     Michael Kaminsky (http://github.com/mkaminsky11)\n\n    Original seti color scheme by Jesse Weed (https://github.com/jesseweed/seti-syntax)\n\n*/\n\n\n.cm-s-seti.CodeMirror {\n  background-color: #151718 !important;\n  color: #CFD2D1 !important;\n  border: none;\n}\n.cm-s-seti .CodeMirror-gutters {\n  color: #404b53;\n  background-color: #0E1112;\n  border: none;\n}\n.cm-s-seti .CodeMirror-cursor { border-left: solid thin #f8f8f0; }\n.cm-s-seti .CodeMirror-linenumber { color: #6D8A88; }\n.cm-s-seti.CodeMirror-focused div.CodeMirror-selected { background: rgba(255, 255, 255, 0.10); }\n.cm-s-seti .CodeMirror-line::selection, .cm-s-seti .CodeMirror-line > span::selection, .cm-s-seti .CodeMirror-line > span > span::selection { background: rgba(255, 255, 255, 0.10); }\n.cm-s-seti .CodeMirror-line::-moz-selection, .cm-s-seti .CodeMirror-line > span::-moz-selection, .cm-s-seti .CodeMirror-line > span > span::-moz-selection { background: rgba(255, 255, 255, 0.10); }\n.cm-s-seti span.cm-comment { color: #41535b; }\n.cm-s-seti span.cm-string, .cm-s-seti span.cm-string-2 { color: #55b5db; }\n.cm-s-seti span.cm-number { color: #cd3f45; }\n.cm-s-seti span.cm-variable { color: #55b5db; }\n.cm-s-seti span.cm-variable-2 { color: #a074c4; }\n.cm-s-seti span.cm-def { color: #55b5db; }\n.cm-s-seti span.cm-keyword { color: #ff79c6; }\n.cm-s-seti span.cm-operator { color: #9fca56; }\n.cm-s-seti span.cm-keyword { color: #e6cd69; }\n.cm-s-seti span.cm-atom { color: #cd3f45; }\n.cm-s-seti span.cm-meta { color: #55b5db; }\n.cm-s-seti span.cm-tag { color: #55b5db; }\n.cm-s-seti span.cm-attribute { color: #9fca56; }\n.cm-s-seti span.cm-qualifier { color: #9fca56; }\n.cm-s-seti span.cm-property { color: #a074c4; }\n.cm-s-seti span.cm-variable-3, .cm-s-seti span.cm-type { color: #9fca56; }\n.cm-s-seti span.cm-builtin { color: #9fca56; }\n.cm-s-seti .CodeMirror-activeline-background { background: #101213; }\n.cm-s-seti .CodeMirror-matchingbracket { text-decoration: underline; color: white !important; }\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*\nSolarized theme for code-mirror\nhttp://ethanschoonover.com/solarized\n*/\n\n/*\nSolarized color palette\nhttp://ethanschoonover.com/solarized/img/solarized-palette.png\n*/\n\n.solarized.base03 { color: #002b36; }\n.solarized.base02 { color: #073642; }\n.solarized.base01 { color: #586e75; }\n.solarized.base00 { color: #657b83; }\n.solarized.base0 { color: #839496; }\n.solarized.base1 { color: #93a1a1; }\n.solarized.base2 { color: #eee8d5; }\n.solarized.base3  { color: #fdf6e3; }\n.solarized.solar-yellow  { color: #b58900; }\n.solarized.solar-orange  { color: #cb4b16; }\n.solarized.solar-red { color: #dc322f; }\n.solarized.solar-magenta { color: #d33682; }\n.solarized.solar-violet  { color: #6c71c4; }\n.solarized.solar-blue { color: #268bd2; }\n.solarized.solar-cyan { color: #2aa198; }\n.solarized.solar-green { color: #859900; }\n\n/* Color scheme for code-mirror */\n\n.cm-s-solarized {\n  line-height: 1.45em;\n  color-profile: sRGB;\n  rendering-intent: auto;\n}\n.cm-s-solarized.cm-s-dark {\n  color: #839496;\n  background-color: #002b36;\n  text-shadow: #002b36 0 1px;\n}\n.cm-s-solarized.cm-s-light {\n  background-color: #fdf6e3;\n  color: #657b83;\n  text-shadow: #eee8d5 0 1px;\n}\n\n.cm-s-solarized .CodeMirror-widget {\n  text-shadow: none;\n}\n\n.cm-s-solarized .cm-header { color: #586e75; }\n.cm-s-solarized .cm-quote { color: #93a1a1; }\n\n.cm-s-solarized .cm-keyword { color: #cb4b16; }\n.cm-s-solarized .cm-atom { color: #d33682; }\n.cm-s-solarized .cm-number { color: #d33682; }\n.cm-s-solarized .cm-def { color: #2aa198; }\n\n.cm-s-solarized .cm-variable { color: #839496; }\n.cm-s-solarized .cm-variable-2 { color: #b58900; }\n.cm-s-solarized .cm-variable-3, .cm-s-solarized .cm-type { color: #6c71c4; }\n\n.cm-s-solarized .cm-property { color: #2aa198; }\n.cm-s-solarized .cm-operator { color: #6c71c4; }\n\n.cm-s-solarized .cm-comment { color: #586e75; font-style:italic; }\n\n.cm-s-solarized .cm-string { color: #859900; }\n.cm-s-solarized .cm-string-2 { color: #b58900; }\n\n.cm-s-solarized .cm-meta { color: #859900; }\n.cm-s-solarized .cm-qualifier { color: #b58900; }\n.cm-s-solarized .cm-builtin { color: #d33682; }\n.cm-s-solarized .cm-bracket { color: #cb4b16; }\n.cm-s-solarized .CodeMirror-matchingbracket { color: #859900; }\n.cm-s-solarized .CodeMirror-nonmatchingbracket { color: #dc322f; }\n.cm-s-solarized .cm-tag { color: #93a1a1; }\n.cm-s-solarized .cm-attribute { color: #2aa198; }\n.cm-s-solarized .cm-hr {\n  color: transparent;\n  border-top: 1px solid #586e75;\n  display: block;\n}\n.cm-s-solarized .cm-link { color: #93a1a1; cursor: pointer; }\n.cm-s-solarized .cm-special { color: #6c71c4; }\n.cm-s-solarized .cm-em {\n  color: #999;\n  text-decoration: underline;\n  text-decoration-style: dotted;\n}\n.cm-s-solarized .cm-error,\n.cm-s-solarized .cm-invalidchar {\n  color: #586e75;\n  border-bottom: 1px dotted #dc322f;\n}\n\n.cm-s-solarized.cm-s-dark div.CodeMirror-selected { background: #073642; }\n.cm-s-solarized.cm-s-dark.CodeMirror ::selection { background: rgba(7, 54, 66, 0.99); }\n.cm-s-solarized.cm-s-dark .CodeMirror-line::-moz-selection, .cm-s-dark .CodeMirror-line > span::-moz-selection, .cm-s-dark .CodeMirror-line > span > span::-moz-selection { background: rgba(7, 54, 66, 0.99); }\n\n.cm-s-solarized.cm-s-light div.CodeMirror-selected { background: #eee8d5; }\n.cm-s-solarized.cm-s-light .CodeMirror-line::selection, .cm-s-light .CodeMirror-line > span::selection, .cm-s-light .CodeMirror-line > span > span::selection { background: #eee8d5; }\n.cm-s-solarized.cm-s-light .CodeMirror-line::-moz-selection, .cm-s-ligh .CodeMirror-line > span::-moz-selection, .cm-s-ligh .CodeMirror-line > span > span::-moz-selection { background: #eee8d5; }\n\n/* Editor styling */\n\n\n\n/* Little shadow on the view-port of the buffer view */\n.cm-s-solarized.CodeMirror {\n  -moz-box-shadow: inset 7px 0 12px -6px #000;\n  -webkit-box-shadow: inset 7px 0 12px -6px #000;\n  box-shadow: inset 7px 0 12px -6px #000;\n}\n\n/* Remove gutter border */\n.cm-s-solarized .CodeMirror-gutters {\n  border-right: 0;\n}\n\n/* Gutter colors and line number styling based of color scheme (dark / light) */\n\n/* Dark */\n.cm-s-solarized.cm-s-dark .CodeMirror-gutters {\n  background-color: #073642;\n}\n\n.cm-s-solarized.cm-s-dark .CodeMirror-linenumber {\n  color: #586e75;\n  text-shadow: #021014 0 -1px;\n}\n\n/* Light */\n.cm-s-solarized.cm-s-light .CodeMirror-gutters {\n  background-color: #eee8d5;\n}\n\n.cm-s-solarized.cm-s-light .CodeMirror-linenumber {\n  color: #839496;\n}\n\n/* Common */\n.cm-s-solarized .CodeMirror-linenumber {\n  padding: 0 5px;\n}\n.cm-s-solarized .CodeMirror-guttermarker-subtle { color: #586e75; }\n.cm-s-solarized.cm-s-dark .CodeMirror-guttermarker { color: #ddd; }\n.cm-s-solarized.cm-s-light .CodeMirror-guttermarker { color: #cb4b16; }\n\n.cm-s-solarized .CodeMirror-gutter .CodeMirror-gutter-text {\n  color: #586e75;\n}\n\n/* Cursor */\n.cm-s-solarized .CodeMirror-cursor { border-left: 1px solid #819090; }\n\n/* Fat cursor */\n.cm-s-solarized.cm-s-light.cm-fat-cursor .CodeMirror-cursor { background: #77ee77; }\n.cm-s-solarized.cm-s-light .cm-animate-fat-cursor { background-color: #77ee77; }\n.cm-s-solarized.cm-s-dark.cm-fat-cursor .CodeMirror-cursor { background: #586e75; }\n.cm-s-solarized.cm-s-dark .cm-animate-fat-cursor { background-color: #586e75; }\n\n/* Active line */\n.cm-s-solarized.cm-s-dark .CodeMirror-activeline-background {\n  background: rgba(255, 255, 255, 0.06);\n}\n.cm-s-solarized.cm-s-light .CodeMirror-activeline-background {\n  background: rgba(0, 0, 0, 0.06);\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".cm-s-the-matrix.CodeMirror { background: #000000; color: #00FF00; }\n.cm-s-the-matrix div.CodeMirror-selected { background: #2D2D2D; }\n.cm-s-the-matrix .CodeMirror-line::selection, .cm-s-the-matrix .CodeMirror-line > span::selection, .cm-s-the-matrix .CodeMirror-line > span > span::selection { background: rgba(45, 45, 45, 0.99); }\n.cm-s-the-matrix .CodeMirror-line::-moz-selection, .cm-s-the-matrix .CodeMirror-line > span::-moz-selection, .cm-s-the-matrix .CodeMirror-line > span > span::-moz-selection { background: rgba(45, 45, 45, 0.99); }\n.cm-s-the-matrix .CodeMirror-gutters { background: #060; border-right: 2px solid #00FF00; }\n.cm-s-the-matrix .CodeMirror-guttermarker { color: #0f0; }\n.cm-s-the-matrix .CodeMirror-guttermarker-subtle { color: white; }\n.cm-s-the-matrix .CodeMirror-linenumber { color: #FFFFFF; }\n.cm-s-the-matrix .CodeMirror-cursor { border-left: 1px solid #00FF00; }\n\n.cm-s-the-matrix span.cm-keyword { color: #008803; font-weight: bold; }\n.cm-s-the-matrix span.cm-atom { color: #3FF; }\n.cm-s-the-matrix span.cm-number { color: #FFB94F; }\n.cm-s-the-matrix span.cm-def { color: #99C; }\n.cm-s-the-matrix span.cm-variable { color: #F6C; }\n.cm-s-the-matrix span.cm-variable-2 { color: #C6F; }\n.cm-s-the-matrix span.cm-variable-3, .cm-s-the-matrix span.cm-type { color: #96F; }\n.cm-s-the-matrix span.cm-property { color: #62FFA0; }\n.cm-s-the-matrix span.cm-operator { color: #999; }\n.cm-s-the-matrix span.cm-comment { color: #CCCCCC; }\n.cm-s-the-matrix span.cm-string { color: #39C; }\n.cm-s-the-matrix span.cm-meta { color: #C9F; }\n.cm-s-the-matrix span.cm-qualifier { color: #FFF700; }\n.cm-s-the-matrix span.cm-builtin { color: #30a; }\n.cm-s-the-matrix span.cm-bracket { color: #cc7; }\n.cm-s-the-matrix span.cm-tag { color: #FFBD40; }\n.cm-s-the-matrix span.cm-attribute { color: #FFF700; }\n.cm-s-the-matrix span.cm-error { color: #FF0000; }\n\n.cm-s-the-matrix .CodeMirror-activeline-background { background: #040; }\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,'/*\nCopyright (C) 2011 by MarkLogic Corporation\nAuthor: Mike Brevoort <mike@brevoort.com>\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.\n*/\n.cm-s-xq-light span.cm-keyword { line-height: 1em; font-weight: bold; color: #5A5CAD; }\n.cm-s-xq-light span.cm-atom { color: #6C8CD5; }\n.cm-s-xq-light span.cm-number { color: #164; }\n.cm-s-xq-light span.cm-def { text-decoration:underline; }\n.cm-s-xq-light span.cm-variable { color: black; }\n.cm-s-xq-light span.cm-variable-2 { color:black; }\n.cm-s-xq-light span.cm-variable-3, .cm-s-xq-light span.cm-type { color: black; }\n.cm-s-xq-light span.cm-property {}\n.cm-s-xq-light span.cm-operator {}\n.cm-s-xq-light span.cm-comment { color: #0080FF; font-style: italic; }\n.cm-s-xq-light span.cm-string { color: red; }\n.cm-s-xq-light span.cm-meta { color: yellow; }\n.cm-s-xq-light span.cm-qualifier { color: grey; }\n.cm-s-xq-light span.cm-builtin { color: #7EA656; }\n.cm-s-xq-light span.cm-bracket { color: #cc7; }\n.cm-s-xq-light span.cm-tag { color: #3F7F7F; }\n.cm-s-xq-light span.cm-attribute { color: #7F007F; }\n.cm-s-xq-light span.cm-error { color: #f00; }\n\n.cm-s-xq-light .CodeMirror-activeline-background { background: #e8f2ff; }\n.cm-s-xq-light .CodeMirror-matchingbracket { outline:1px solid grey;color:black !important;background:yellow; }\n',""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n.CodeMirror {\n  line-height: var(--jp-code-line-height);\n  font-size: var(--jp-code-font-size);\n  font-family: var(--jp-code-font-family);\n  border: 0;\n  border-radius: 0;\n  height: auto;\n  /* Changed to auto to autogrow */\n}\n\n.CodeMirror pre {\n  padding: 0 var(--jp-code-padding);\n}\n\n.jp-CodeMirrorEditor[data-type='inline'] .CodeMirror-dialog {\n  background-color: var(--jp-layout-color0);\n  color: var(--jp-content-font-color1);\n}\n\n/* This causes https://github.com/jupyter/jupyterlab/issues/522 */\n/* May not cause it not because we changed it! */\n.CodeMirror-lines {\n  padding: var(--jp-code-padding) 0;\n}\n\n.CodeMirror-linenumber {\n  padding: 0 8px;\n}\n\n.jp-CodeMirrorEditor-static {\n  margin: var(--jp-code-padding);\n}\n\n.jp-CodeMirrorEditor,\n.jp-CodeMirrorEditor-static {\n  cursor: text;\n}\n\n.jp-CodeMirrorEditor[data-type='inline'] .CodeMirror-cursor {\n  border-left: var(--jp-code-cursor-width0) solid var(--jp-editor-cursor-color);\n}\n\n/* When zoomed out 67% and 33% on a screen of 1440 width x 900 height */\n@media screen and (min-width: 2138px) and (max-width: 4319px) {\n  .jp-CodeMirrorEditor[data-type='inline'] .CodeMirror-cursor {\n    border-left: var(--jp-code-cursor-width1) solid\n      var(--jp-editor-cursor-color);\n  }\n}\n\n/* When zoomed out less than 33% */\n@media screen and (min-width: 4320px) {\n  .jp-CodeMirrorEditor[data-type='inline'] .CodeMirror-cursor {\n    border-left: var(--jp-code-cursor-width2) solid\n      var(--jp-editor-cursor-color);\n  }\n}\n\n.CodeMirror.jp-mod-readOnly .CodeMirror-cursor {\n  display: none;\n}\n\n.CodeMirror-gutters {\n  border-right: 1px solid var(--jp-border-color2);\n  background-color: var(--jp-layout-color0);\n}\n\n.jp-CollaboratorCursor {\n  border-left: 5px solid transparent;\n  border-right: 5px solid transparent;\n  border-top: none;\n  border-bottom: 3px solid;\n  background-clip: content-box;\n  margin-left: -5px;\n  margin-right: -5px;\n}\n\n.CodeMirror-selectedtext.cm-searching {\n  background-color: var(--jp-search-selected-match-background-color) !important;\n  color: var(--jp-search-selected-match-color) !important;\n}\n\n.cm-searching {\n  background-color: var(\n    --jp-search-unselected-match-background-color\n  ) !important;\n  color: var(--jp-search-unselected-match-color) !important;\n}\n\n.CodeMirror-focused .CodeMirror-selected {\n  background-color: var(--jp-editor-selected-focused-background);\n}\n\n.CodeMirror-selected {\n  background-color: var(--jp-editor-selected-background);\n}\n\n.jp-CollaboratorCursor-hover {\n  position: absolute;\n  z-index: 1;\n  transform: translateX(-50%);\n  color: white;\n  border-radius: 3px;\n  padding-left: 4px;\n  padding-right: 4px;\n  padding-top: 1px;\n  padding-bottom: 1px;\n  text-align: center;\n  font-size: var(--jp-ui-font-size1);\n  white-space: nowrap;\n}\n\n.jp-CodeMirror-ruler {\n  border-left: 1px dashed var(--jp-border-color2);\n}\n\n/**\n * Here is our jupyter theme for CodeMirror syntax highlighting\n * This is used in our marked.js syntax highlighting and CodeMirror itself\n * The string \"jupyter\" is set in ../codemirror/widget.DEFAULT_CODEMIRROR_THEME\n * This came from the classic notebook, which came form highlight.js/GitHub\n */\n\n/**\n * CodeMirror themes are handling the background/color in this way. This works\n * fine for CodeMirror editors outside the notebook, but the notebook styles\n * these things differently.\n */\n.CodeMirror.cm-s-jupyter {\n  background: var(--jp-layout-color0);\n  color: var(--jp-content-font-color1);\n}\n\n/* In the notebook, we want this styling to be handled by its container */\n.jp-CodeConsole .CodeMirror.cm-s-jupyter,\n.jp-Notebook .CodeMirror.cm-s-jupyter {\n  background: transparent;\n}\n\n.cm-s-jupyter .CodeMirror-cursor {\n  border-left: var(--jp-code-cursor-width0) solid var(--jp-editor-cursor-color);\n}\n.cm-s-jupyter span.cm-keyword {\n  color: var(--jp-mirror-editor-keyword-color);\n  font-weight: bold;\n}\n.cm-s-jupyter span.cm-atom {\n  color: var(--jp-mirror-editor-atom-color);\n}\n.cm-s-jupyter span.cm-number {\n  color: var(--jp-mirror-editor-number-color);\n}\n.cm-s-jupyter span.cm-def {\n  color: var(--jp-mirror-editor-def-color);\n}\n.cm-s-jupyter span.cm-variable {\n  color: var(--jp-mirror-editor-variable-color);\n}\n.cm-s-jupyter span.cm-variable-2 {\n  color: var(--jp-mirror-editor-variable-2-color);\n}\n.cm-s-jupyter span.cm-variable-3 {\n  color: var(--jp-mirror-editor-variable-3-color);\n}\n.cm-s-jupyter span.cm-punctuation {\n  color: var(--jp-mirror-editor-punctuation-color);\n}\n.cm-s-jupyter span.cm-property {\n  color: var(--jp-mirror-editor-property-color);\n}\n.cm-s-jupyter span.cm-operator {\n  color: var(--jp-mirror-editor-operator-color);\n  font-weight: bold;\n}\n.cm-s-jupyter span.cm-comment {\n  color: var(--jp-mirror-editor-comment-color);\n  font-style: italic;\n}\n.cm-s-jupyter span.cm-string {\n  color: var(--jp-mirror-editor-string-color);\n}\n.cm-s-jupyter span.cm-string-2 {\n  color: var(--jp-mirror-editor-string-2-color);\n}\n.cm-s-jupyter span.cm-meta {\n  color: var(--jp-mirror-editor-meta-color);\n}\n.cm-s-jupyter span.cm-qualifier {\n  color: var(--jp-mirror-editor-qualifier-color);\n}\n.cm-s-jupyter span.cm-builtin {\n  color: var(--jp-mirror-editor-builtin-color);\n}\n.cm-s-jupyter span.cm-bracket {\n  color: var(--jp-mirror-editor-bracket-color);\n}\n.cm-s-jupyter span.cm-tag {\n  color: var(--jp-mirror-editor-tag-color);\n}\n.cm-s-jupyter span.cm-attribute {\n  color: var(--jp-mirror-editor-attribute-color);\n}\n.cm-s-jupyter span.cm-header {\n  color: var(--jp-mirror-editor-header-color);\n}\n.cm-s-jupyter span.cm-quote {\n  color: var(--jp-mirror-editor-quote-color);\n}\n.cm-s-jupyter span.cm-link {\n  color: var(--jp-mirror-editor-link-color);\n}\n.cm-s-jupyter span.cm-error {\n  color: var(--jp-mirror-editor-error-color);\n}\n.cm-s-jupyter span.cm-hr {\n  color: #999;\n}\n\n.cm-s-jupyter span.cm-tab {\n  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAADAAAAAMCAYAAAAkuj5RAAAAAXNSR0IArs4c6QAAAGFJREFUSMft1LsRQFAQheHPowAKoACx3IgEKtaEHujDjORSgWTH/ZOdnZOcM/sgk/kFFWY0qV8foQwS4MKBCS3qR6ixBJvElOobYAtivseIE120FaowJPN75GMu8j/LfMwNjh4HUpwg4LUAAAAASUVORK5CYII=);\n  background-position: right;\n  background-repeat: no-repeat;\n}\n\n.cm-s-jupyter .CodeMirror-activeline-background,\n.cm-s-jupyter .CodeMirror-gutter {\n  background-color: var(--jp-layout-color2);\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,'/* The lint marker gutter */\n.CodeMirror-lint-markers {\n  width: 16px;\n}\n\n.CodeMirror-lint-tooltip {\n  background-color: #ffd;\n  border: 1px solid black;\n  border-radius: 4px 4px 4px 4px;\n  color: black;\n  font-family: monospace;\n  font-size: 10pt;\n  overflow: hidden;\n  padding: 2px 5px;\n  position: fixed;\n  white-space: pre;\n  white-space: pre-wrap;\n  z-index: 100;\n  max-width: 600px;\n  opacity: 0;\n  transition: opacity .4s;\n  -moz-transition: opacity .4s;\n  -webkit-transition: opacity .4s;\n  -o-transition: opacity .4s;\n  -ms-transition: opacity .4s;\n}\n\n.CodeMirror-lint-mark-error, .CodeMirror-lint-mark-warning {\n  background-position: left bottom;\n  background-repeat: repeat-x;\n}\n\n.CodeMirror-lint-mark-error {\n  background-image:\n  url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAQAAAADCAYAAAC09K7GAAAAAXNSR0IArs4c6QAAAAZiS0dEAP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9sJDw4cOCW1/KIAAAAZdEVYdENvbW1lbnQAQ3JlYXRlZCB3aXRoIEdJTVBXgQ4XAAAAHElEQVQI12NggIL/DAz/GdA5/xkY/qPKMDAwAADLZwf5rvm+LQAAAABJRU5ErkJggg==")\n  ;\n}\n\n.CodeMirror-lint-mark-warning {\n  background-image: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAQAAAADCAYAAAC09K7GAAAAAXNSR0IArs4c6QAAAAZiS0dEAP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9sJFhQXEbhTg7YAAAAZdEVYdENvbW1lbnQAQ3JlYXRlZCB3aXRoIEdJTVBXgQ4XAAAAMklEQVQI12NkgIIvJ3QXMjAwdDN+OaEbysDA4MPAwNDNwMCwiOHLCd1zX07o6kBVGQEAKBANtobskNMAAAAASUVORK5CYII=");\n}\n\n.CodeMirror-lint-marker-error, .CodeMirror-lint-marker-warning {\n  background-position: center center;\n  background-repeat: no-repeat;\n  cursor: pointer;\n  display: inline-block;\n  height: 16px;\n  width: 16px;\n  vertical-align: middle;\n  position: relative;\n}\n\n.CodeMirror-lint-message-error, .CodeMirror-lint-message-warning {\n  padding-left: 18px;\n  background-position: top left;\n  background-repeat: no-repeat;\n}\n\n.CodeMirror-lint-marker-error, .CodeMirror-lint-message-error {\n  background-image: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAAoLQ9TAAAAHlBMVEW7AAC7AACxAAC7AAC7AAAAAAC4AAC5AAD///+7AAAUdclpAAAABnRSTlMXnORSiwCK0ZKSAAAATUlEQVR42mWPOQ7AQAgDuQLx/z8csYRmPRIFIwRGnosRrpamvkKi0FTIiMASR3hhKW+hAN6/tIWhu9PDWiTGNEkTtIOucA5Oyr9ckPgAWm0GPBog6v4AAAAASUVORK5CYII=");\n}\n\n.CodeMirror-lint-marker-warning, .CodeMirror-lint-message-warning {\n  background-image: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAAoLQ9TAAAANlBMVEX/uwDvrwD/uwD/uwD/uwD/uwD/uwD/uwD/uwD6twD/uwAAAADurwD2tQD7uAD+ugAAAAD/uwDhmeTRAAAADHRSTlMJ8mN1EYcbmiixgACm7WbuAAAAVklEQVR42n3PUQqAIBBFUU1LLc3u/jdbOJoW1P08DA9Gba8+YWJ6gNJoNYIBzAA2chBth5kLmG9YUoG0NHAUwFXwO9LuBQL1giCQb8gC9Oro2vp5rncCIY8L8uEx5ZkAAAAASUVORK5CYII=");\n}\n\n.CodeMirror-lint-marker-multiple {\n  background-image: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAcAAAAHCAMAAADzjKfhAAAACVBMVEUAAAAAAAC/v7914kyHAAAAAXRSTlMAQObYZgAAACNJREFUeNo1ioEJAAAIwmz/H90iFFSGJgFMe3gaLZ0od+9/AQZ0ADosbYraAAAAAElFTkSuQmCC");\n  background-repeat: no-repeat;\n  background-position: right bottom;\n  width: 100%; height: 100%;\n}\n',""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".CodeMirror-hints {\n  position: absolute;\n  z-index: 10;\n  overflow: hidden;\n  list-style: none;\n\n  margin: 0;\n  padding: 2px;\n\n  -webkit-box-shadow: 2px 3px 5px rgba(0,0,0,.2);\n  -moz-box-shadow: 2px 3px 5px rgba(0,0,0,.2);\n  box-shadow: 2px 3px 5px rgba(0,0,0,.2);\n  border-radius: 3px;\n  border: 1px solid silver;\n\n  background: white;\n  font-size: 90%;\n  font-family: monospace;\n\n  max-height: 20em;\n  overflow-y: auto;\n}\n\n.CodeMirror-hint {\n  margin: 0;\n  padding: 0 4px;\n  border-radius: 2px;\n  white-space: pre;\n  color: black;\n  cursor: pointer;\n}\n\nli.CodeMirror-hint-active {\n  background: #08f;\n  color: white;\n}\n",""])},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(568),""),e.push([t.i,'.maven_dashboard {\n  position: relative;\n  box-sizing: border-box;\n}\n.maven_stackpanel {\n  overflow: hidden;\n}\n.maven_stackpanel .m-StackPanel-splitter {\n  background-color: var(--jp-layout-color3);\n  float: left;\n}\n.maven_stackpanel .m-StackPanel-accordion {\n  float: left;\n  background: var(--m-inactive-ui-color);\n  color: var(--m-inactive-font-color);\n}\n.maven_stackpanel .m-StackPanel-accordion .m-FlippyTriangle {\n  margin: 3px;\n  display: inline-block;\n}\n.maven_stackpanel.maven_layout_horizontal > .p-Panel > div.m-StackPanel-splitter {\n  cursor: ew-resize;\n}\n.maven_stackpanel.maven_layout_horizontal > .p-Panel > div.m-StackPanel-accordion {\n  writing-mode: vertical-lr;\n  padding-top: 5px;\n  line-height: 23px;\n  border-left: 1px solid var(--m-border-color);\n  border-right: 1px solid var(--m-border-color);\n}\n.maven_stackpanel.maven_layout_horizontal > .p-Panel > div.m-StackPanel-accordion .m-FlippyTriangle[data-collapsed="false"] {\n  transform: rotate(0);\n}\n.maven_stackpanel.maven_layout_horizontal > .p-Panel > div.m-StackPanel-accordion .m-FlippyTriangle[data-collapsed="true"] {\n  transform: rotate(0.25turn);\n}\n.maven_stackpanel.maven_layout_vertical > .p-Panel > div.m-StackPanel-splitter {\n  cursor: ns-resize;\n}\n.maven_stackpanel.maven_layout_vertical > .p-Panel > div.m-StackPanel-accordion {\n  padding-left: 5px;\n  line-height: 24px;\n  border-top: 1px solid var(--m-border-color);\n  border-bottom: 1px solid var(--m-border-color);\n}\n.m-GridPanel-panel > .maven_dashboard,\n.m-CanvasPanel-canvas > .maven_dashboard {\n  position: absolute;\n}\n.m-Region-Content {\n  flex: 1 0;\n}\n.no-stack {\n  z-index: unset !important;\n}\n.m-Region-Maximized {\n  position: fixed !important;\n  z-index: 10 !important;\n  background: white !important;\n}\n.maven_dashboard {\n  display: flex;\n  flex-direction: column;\n}\n.maven_layout_horizontal > .p-Panel > .maven_dashboard {\n  float: left;\n}\n.maven_layout_vertical > .p-Panel > .maven_dashboard {\n  vertical-align: top;\n  width: 100%;\n}\n.maven_tabpanel_panel {\n  border: 1px solid var(--jp-border-color0);\n  border-top: 0;\n}\n.maven_tabpanel_panel > * {\n  height: 100%;\n  width: 100%;\n}\n.m-TitleBar-Wrapper {\n  flex: 0 0 calc(1em + 10px);\n}\n.m-TitleBar {\n  width: calc(100% - 10px);\n  height: calc(100% - 11px);\n  /* padding*2 + border */\n  padding: 5px;\n  border-bottom: 1px solid var(--jp-border-color0);\n  line-height: 1em;\n  background-color: var(--jp-layout-color2);\n  display: inline-block;\n  user-select: none;\n}\n.m-TitleBar-editableInput {\n  width: 100%;\n  height: 100%;\n  font: inherit;\n  padding: 0;\n  border: none;\n  background: transparent;\n  margin: 0;\n  color: unset;\n  outline: none;\n  overflow: visible;\n}\n.m-TitleBar-Wrapper.m-mod-active > .m-TitleBar {\n  background-color: var(--jp-layout-color4);\n  color: var(--jp-ui-inverse-font-color0);\n}\n.m-AddPartOverlay {\n  height: 100%;\n  width: 100%;\n  background-color: rgba(0, 0, 0, 0.5);\n}\n.m-AddPartOverlay .m-AddPartOverlay-btn {\n  position: absolute;\n  top: calc(50% - 50px);\n  width: 100%;\n  height: 100px;\n  padding: 20px;\n}\n.m-AddPartOverlay .m-AddPartOverlay-btn button {\n  background: none;\n  border: 5px solid white;\n  color: white;\n  font-size: 2rem;\n}\n.m-AddPartOverlay .m-AddPartOverlay-btn button:hover {\n  border-color: lightgrey;\n  color: lightgrey;\n}\n.temp-disable-pointer {\n  pointer-events: none;\n}\n.m-DragShadow {\n  font-size: 1em;\n  background-color: var(--jp-layout-color4, "darkgrey");\n  color: var(--jp-ui-inverse-font-color0, "white");\n  padding: 5px;\n  padding-left: 20px;\n}\n.m-DockPreview {\n  pointer-events: none;\n}\n.m-DockPreview .m-DockPreview__overlay {\n  background-color: rgba(54, 194, 246, 0.25);\n  border: 1px dashed white;\n  position: absolute;\n}\ndiv.dropzone {\n  width: 60px;\n  height: 60px;\n  background: white;\n  padding: 10px;\n  position: absolute;\n  left: calc(50% - 20px);\n  top: calc(50% - 20px);\n  z-index: 5;\n}\ndiv.dropzone .dropzone__preview {\n  border: 2px dashed lightblue;\n  width: 36px;\n  height: 36px;\n  pointer-events: none;\n}\ndiv.dropzone:hover .dropzone__preview {\n  border-color: darkblue;\n}\n.dropzone[data-zone^="inner"] {\n  --mod: -50px;\n}\n.dropzone[data-zone^="outer"] {\n  --mod: -100px;\n}\n.dropzone[data-zone$="left"] {\n  left: calc(50% - (-1 * var(--mod) + 20px));\n}\n.dropzone[data-zone$="left"] .dropzone__preview {\n  border-left-width: 20px;\n  border-left-style: solid;\n  width: 18px;\n  height: 36px;\n}\n.dropzone[data-zone="far-outer-left"] {\n  left: 0;\n}\n.dropzone[data-zone$="right"] {\n  left: calc(50% - (1 * var(--mod) + 20px));\n}\n.dropzone[data-zone$="right"] .dropzone__preview {\n  border-right-width: 20px;\n  border-right-style: solid;\n  width: 18px;\n  height: 36px;\n}\n.dropzone[data-zone="far-outer-right"] {\n  right: 0;\n}\n.dropzone[data-zone$="top"] {\n  top: calc(50% - (-1 * var(--mod) + 20px));\n}\n.dropzone[data-zone$="top"] .dropzone__preview {\n  border-top-width: 20px;\n  border-top-style: solid;\n  width: 36px;\n  height: 18px;\n}\n.dropzone[data-zone="far-outer-top"] {\n  top: 0;\n}\n.dropzone[data-zone$="bottom"] {\n  top: calc(50% - (1 * var(--mod) + 20px));\n}\n.dropzone[data-zone$="bottom"] .dropzone__preview {\n  border-bottom-width: 20px;\n  border-bottom-style: solid;\n  width: 36px;\n  height: 18px;\n}\n.dropzone[data-zone="far-outer-bottom"] {\n  bottom: 0;\n}\n',""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".m-Part {\r\n    padding: 0;\r\n    margin: 0;\r\n}\r\n\r\n.m-PartOverlay {\r\n    background: rgba(0,0,0,0.5);\r\n    color: white;\r\n    text-align: center;\r\n    display: flex;\r\n    flex-direction: column;\r\n    animation: m-overlay-fadein 0.25s ease-in;\r\n}\r\n\r\n@keyframes m-overlay-fadein {\r\n    from {\r\n        opacity: 0;\r\n    }\r\n\r\n    to {\r\n        opacity: 1;\r\n    }\r\n}\r\n\r\n.m-PartOverlay-Header {\r\n    display: flex;\r\n    flex-direction: row;\r\n}\r\n\r\n.m-PartOverlay .m-PartOverlay-Button {\r\n    background: transparent;\r\n    color: white;\r\n    /* approx stroke width of fa-times symbol */\r\n    border: 5px solid white;\r\n    padding: 0.25em;\r\n    font-size: 1.5em;\r\n    opacity: 0.75;\r\n    min-width: 125px;\r\n    width: 30%;\r\n    display: block;\r\n    margin: auto;\r\n    margin-top: 0.5em;\r\n}\r\n\r\n.m-PartOverlay .m-PartOverlay-center {\r\n    width: 100%;\r\n    text-align: center;\r\n}\r\n\r\n.m-PartOverlay .errorDetail {\r\n    background: white;\r\n    color: red;\r\n    padding: 10px;\r\n}\r\n\r\n.m-PartOverlay .m-PartOverlay-BigButton {\r\n    font-size: 3em;\r\n    min-width: 256px;\r\n}\r\n\r\n.m-PartOverlay .m-PartOverlay-Button:hover {\r\n    opacity: 1;\r\n}\r\n\r\n.m-PartOverlay .m-PartOverlay-ButtonContent {\r\n    margin-left: 0.25em;\r\n}\r\n\r\n.m-PartOverlay-State {\r\n    font-size: 3em;\r\n    font-weight: bold;\r\n    margin-bottom: 10px;\r\n}\r\n\r\n.m-PartOverlay > * {\r\n    text-align: left;\r\n    overflow: auto;\r\n    padding: 0.25em;\r\n}\r\n",""])},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(999),""),e.i(n(1e3),""),e.i(n(1001),""),e.push([t.i,".m-OptionsEditor {\n  min-width: 700px;\n  min-height: 500px;\n}\n.m-OptionsEditor .m-OptionsEditor-Field {\n  flex: 0 0 33%;\n}\n.m-OptionsEditor .m-OptionsEditor-Header {\n  height: 3em;\n  font-weight: var(--jp-ui-font-size2);\n  color: var(--jp-ui-font-color0);\n  display: flex;\n  flex-direction: row;\n}\n.m-OptionsEditor .m-OptionsEditor-Body .m-OptionsEditor-Row {\n  height: 2em;\n  display: flex;\n  flex-direction: row;\n  margin: 2px;\n  margin-bottom: 4px;\n}\n.m-OptionsEditor .m-OptionsEditor-Body .m-OptionsEditor-Row .m-OptionsEditor-ValueField {\n  display: flex;\n}\n.m-OptionsEditor .m-OptionsEditor-Body .m-OptionsEditor-Row .m-OptionsEditor-ValueField .m-TypeEditor {\n  flex: 1 0;\n  width: 100%;\n}\n.m-OptionsEditor .m-OptionsEditor-Body .m-OptionsEditor-Row .m-OptionsEditor-icon {\n  flex: 0 1 2em;\n  width: 2em;\n  height: 2em;\n  padding: 0;\n  margin: 0;\n  text-align: center;\n  line-height: 2em;\n}\n.m-PartEditor > .p-Widget {\n  width: 100%;\n  height: 100%;\n}\n.m-PartEditor .m-Part {\n  width: 100%;\n  height: 100%;\n}\n.m-PartEditor .m-PartEditor-PreviewPanel {\n  display: flex;\n  flex-direction: column;\n}\n/* CSS styling fix */\n.p-SplitPanel-handle {\n  background-color: var(--jp-layout-color3);\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".m-GlobalsEditor {\r\n    min-width: 20vw;\r\n    min-height: 20vh;\r\n}\r\n\r\n.m-GlobalsEditor-header {\r\n    display: flex;\r\n}\r\n\r\n.m-GlobalsEditor-headerCell {\r\n    flex-grow: 1;\r\n    flex-shrink: 0;\r\n    flex-basis: auto;\r\n}\r\n\r\n.m-GlobalsEditor .m-PropertiesEditor .m-PropertiesEditor-Property {\r\n    margin-top: 5px;\r\n}",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".m-PartProperty {\r\n    display: flex;\r\n}\r\n\r\n.m-PartProperty-BindingPicker {\r\n    flex: 0 1 auto;\r\n    margin-right: -1px;\r\n    margin-bottom: -1px;\r\n}\r\n\r\n.m-PartProperty>:last-child {\r\n    flex: 1 1 auto;\r\n}",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".m-VisualEditor-dialog {\r\n    width: 100%;\r\n    height: 100%;\r\n    display: flex;\r\n    padding: 3px;\r\n}\r\n\r\n.m-VisualEditor-column {\r\n    flex: 1 1 33%;\r\n    height: 100%;\r\n    float: left;\r\n}\r\n\r\n.m-VisualEditor-column:last-of-type {\r\n    flex-basis: 50%;\r\n}\r\n\r\n.m-RegionPropertiesEditor {\r\n    height: 100%;\r\n    overflow-y: auto;\r\n}\r\n\r\n.m-TreeList {\r\n    height: 100%;\r\n    width: 100%;\r\n    overflow-y: auto;\r\n}\r\n.m-VisualTreeNode .m-VisualTreeNode-caption {\r\n    max-width: calc(100% - 75px);\r\n    text-overflow: ellipsis;\r\n    overflow: hidden;\r\n    white-space: nowrap;\r\n    display: inline-block;\r\n    vertical-align: middle;\r\n}\r\n\r\n.m-TreeView-Node button.m-VisualTreeNode-delete {\r\n    background: none;\r\n    border: none;\r\n    padding: 0;\r\n    margin-left: 10px;\r\n    vertical-align: middle;\r\n    color: inherit;\r\n    visibility: hidden;\r\n}\r\n\r\n.m-TreeView-Node-row:hover button.m-VisualTreeNode-delete {\r\n    visibility: visible;\r\n}\r\n\r\n.m-TreeView-Node button.m-VisualTreeNode-delete:hover {\r\n    color: var(--m-warn-ui-color);\r\n}",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,".m-Notebook-ContentMaximized .jp-Notebook::after {\r\n    min-height: 0;\r\n    background-color: var(--jp-layout-color2);\r\n    transition: background-color 600ms ease-in;\r\n}\r\n\r\n.m-Notebook-ContentMaximized .jp-Notebook {\r\n    background-color: transparent;\r\n}\r\n\r\n.m-NotebookCell-Maximized {\r\n    position: relative;\r\n    box-shadow: var(--jp-elevation-z8);\r\n    background-color: var(--jp-layout-color0);\r\n    transition: box-shadow 600ms ease-in, background-color 600ms ease-in;\r\n}\r\n\r\n.m-NotebookCell-Maximized .jp-Cell-outputWrapper {\r\n    height: calc(100% - 2 * var(--jp-notebook-padding) - var(--jp-cell-padding));\r\n}\r\n\r\n.m-NotebookCell-Maximized,\r\n.m-NotebookCell-Maximized .jp-OutputArea,\r\n.m-NotebookCell-Maximized .jp-OutputArea-output,\r\n.m-NotebookCell-Maximized .jp-OutputArea-child {\r\n    height: 100%;\r\n}",""])},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(106),""),e.i(n(567),""),e.i(n(173),""),e.i(n(569),""),e.i(n(1005),""),e.i(n(1012),""),e.push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/* This file was auto-generated by ensurePackage() in @jupyterlab/buildutils */\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n.jp-MimeDocument {\n  outline: none;\n}\n",""])},function(t,e,n){var r=n(172);(t.exports=n(12)(!1)).push([t.i,"/*!\n *  Font Awesome 4.7.0 by @davegandy - http://fontawesome.io - @fontawesome\n *  License - http://fontawesome.io/license (Font: SIL OFL 1.1, CSS: MIT License)\n */@font-face{font-family:'FontAwesome';src:url("+r(n(1006))+");src:url("+r(n(1007))+"?#iefix&v=4.7.0) format('embedded-opentype'),url("+r(n(1008))+") format('woff2'),url("+r(n(1009))+") format('woff'),url("+r(n(1010))+") format('truetype'),url("+r(n(1011))+'#fontawesomeregular) format(\'svg\');font-weight:normal;font-style:normal}.fa{display:inline-block;font:normal normal normal 14px/1 FontAwesome;font-size:inherit;text-rendering:auto;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.fa-lg{font-size:1.33333333em;line-height:.75em;vertical-align:-15%}.fa-2x{font-size:2em}.fa-3x{font-size:3em}.fa-4x{font-size:4em}.fa-5x{font-size:5em}.fa-fw{width:1.28571429em;text-align:center}.fa-ul{padding-left:0;margin-left:2.14285714em;list-style-type:none}.fa-ul>li{position:relative}.fa-li{position:absolute;left:-2.14285714em;width:2.14285714em;top:.14285714em;text-align:center}.fa-li.fa-lg{left:-1.85714286em}.fa-border{padding:.2em .25em .15em;border:solid .08em #eee;border-radius:.1em}.fa-pull-left{float:left}.fa-pull-right{float:right}.fa.fa-pull-left{margin-right:.3em}.fa.fa-pull-right{margin-left:.3em}.pull-right{float:right}.pull-left{float:left}.fa.pull-left{margin-right:.3em}.fa.pull-right{margin-left:.3em}.fa-spin{-webkit-animation:fa-spin 2s infinite linear;animation:fa-spin 2s infinite linear}.fa-pulse{-webkit-animation:fa-spin 1s infinite steps(8);animation:fa-spin 1s infinite steps(8)}@-webkit-keyframes fa-spin{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}100%{-webkit-transform:rotate(359deg);transform:rotate(359deg)}}@keyframes fa-spin{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}100%{-webkit-transform:rotate(359deg);transform:rotate(359deg)}}.fa-rotate-90{-ms-filter:"progid:DXImageTransform.Microsoft.BasicImage(rotation=1)";-webkit-transform:rotate(90deg);-ms-transform:rotate(90deg);transform:rotate(90deg)}.fa-rotate-180{-ms-filter:"progid:DXImageTransform.Microsoft.BasicImage(rotation=2)";-webkit-transform:rotate(180deg);-ms-transform:rotate(180deg);transform:rotate(180deg)}.fa-rotate-270{-ms-filter:"progid:DXImageTransform.Microsoft.BasicImage(rotation=3)";-webkit-transform:rotate(270deg);-ms-transform:rotate(270deg);transform:rotate(270deg)}.fa-flip-horizontal{-ms-filter:"progid:DXImageTransform.Microsoft.BasicImage(rotation=0, mirror=1)";-webkit-transform:scale(-1, 1);-ms-transform:scale(-1, 1);transform:scale(-1, 1)}.fa-flip-vertical{-ms-filter:"progid:DXImageTransform.Microsoft.BasicImage(rotation=2, mirror=1)";-webkit-transform:scale(1, -1);-ms-transform:scale(1, -1);transform:scale(1, -1)}:root .fa-rotate-90,:root .fa-rotate-180,:root .fa-rotate-270,:root .fa-flip-horizontal,:root .fa-flip-vertical{filter:none}.fa-stack{position:relative;display:inline-block;width:2em;height:2em;line-height:2em;vertical-align:middle}.fa-stack-1x,.fa-stack-2x{position:absolute;left:0;width:100%;text-align:center}.fa-stack-1x{line-height:inherit}.fa-stack-2x{font-size:2em}.fa-inverse{color:#fff}.fa-glass:before{content:"\\F000"}.fa-music:before{content:"\\F001"}.fa-search:before{content:"\\F002"}.fa-envelope-o:before{content:"\\F003"}.fa-heart:before{content:"\\F004"}.fa-star:before{content:"\\F005"}.fa-star-o:before{content:"\\F006"}.fa-user:before{content:"\\F007"}.fa-film:before{content:"\\F008"}.fa-th-large:before{content:"\\F009"}.fa-th:before{content:"\\F00A"}.fa-th-list:before{content:"\\F00B"}.fa-check:before{content:"\\F00C"}.fa-remove:before,.fa-close:before,.fa-times:before{content:"\\F00D"}.fa-search-plus:before{content:"\\F00E"}.fa-search-minus:before{content:"\\F010"}.fa-power-off:before{content:"\\F011"}.fa-signal:before{content:"\\F012"}.fa-gear:before,.fa-cog:before{content:"\\F013"}.fa-trash-o:before{content:"\\F014"}.fa-home:before{content:"\\F015"}.fa-file-o:before{content:"\\F016"}.fa-clock-o:before{content:"\\F017"}.fa-road:before{content:"\\F018"}.fa-download:before{content:"\\F019"}.fa-arrow-circle-o-down:before{content:"\\F01A"}.fa-arrow-circle-o-up:before{content:"\\F01B"}.fa-inbox:before{content:"\\F01C"}.fa-play-circle-o:before{content:"\\F01D"}.fa-rotate-right:before,.fa-repeat:before{content:"\\F01E"}.fa-refresh:before{content:"\\F021"}.fa-list-alt:before{content:"\\F022"}.fa-lock:before{content:"\\F023"}.fa-flag:before{content:"\\F024"}.fa-headphones:before{content:"\\F025"}.fa-volume-off:before{content:"\\F026"}.fa-volume-down:before{content:"\\F027"}.fa-volume-up:before{content:"\\F028"}.fa-qrcode:before{content:"\\F029"}.fa-barcode:before{content:"\\F02A"}.fa-tag:before{content:"\\F02B"}.fa-tags:before{content:"\\F02C"}.fa-book:before{content:"\\F02D"}.fa-bookmark:before{content:"\\F02E"}.fa-print:before{content:"\\F02F"}.fa-camera:before{content:"\\F030"}.fa-font:before{content:"\\F031"}.fa-bold:before{content:"\\F032"}.fa-italic:before{content:"\\F033"}.fa-text-height:before{content:"\\F034"}.fa-text-width:before{content:"\\F035"}.fa-align-left:before{content:"\\F036"}.fa-align-center:before{content:"\\F037"}.fa-align-right:before{content:"\\F038"}.fa-align-justify:before{content:"\\F039"}.fa-list:before{content:"\\F03A"}.fa-dedent:before,.fa-outdent:before{content:"\\F03B"}.fa-indent:before{content:"\\F03C"}.fa-video-camera:before{content:"\\F03D"}.fa-photo:before,.fa-image:before,.fa-picture-o:before{content:"\\F03E"}.fa-pencil:before{content:"\\F040"}.fa-map-marker:before{content:"\\F041"}.fa-adjust:before{content:"\\F042"}.fa-tint:before{content:"\\F043"}.fa-edit:before,.fa-pencil-square-o:before{content:"\\F044"}.fa-share-square-o:before{content:"\\F045"}.fa-check-square-o:before{content:"\\F046"}.fa-arrows:before{content:"\\F047"}.fa-step-backward:before{content:"\\F048"}.fa-fast-backward:before{content:"\\F049"}.fa-backward:before{content:"\\F04A"}.fa-play:before{content:"\\F04B"}.fa-pause:before{content:"\\F04C"}.fa-stop:before{content:"\\F04D"}.fa-forward:before{content:"\\F04E"}.fa-fast-forward:before{content:"\\F050"}.fa-step-forward:before{content:"\\F051"}.fa-eject:before{content:"\\F052"}.fa-chevron-left:before{content:"\\F053"}.fa-chevron-right:before{content:"\\F054"}.fa-plus-circle:before{content:"\\F055"}.fa-minus-circle:before{content:"\\F056"}.fa-times-circle:before{content:"\\F057"}.fa-check-circle:before{content:"\\F058"}.fa-question-circle:before{content:"\\F059"}.fa-info-circle:before{content:"\\F05A"}.fa-crosshairs:before{content:"\\F05B"}.fa-times-circle-o:before{content:"\\F05C"}.fa-check-circle-o:before{content:"\\F05D"}.fa-ban:before{content:"\\F05E"}.fa-arrow-left:before{content:"\\F060"}.fa-arrow-right:before{content:"\\F061"}.fa-arrow-up:before{content:"\\F062"}.fa-arrow-down:before{content:"\\F063"}.fa-mail-forward:before,.fa-share:before{content:"\\F064"}.fa-expand:before{content:"\\F065"}.fa-compress:before{content:"\\F066"}.fa-plus:before{content:"\\F067"}.fa-minus:before{content:"\\F068"}.fa-asterisk:before{content:"\\F069"}.fa-exclamation-circle:before{content:"\\F06A"}.fa-gift:before{content:"\\F06B"}.fa-leaf:before{content:"\\F06C"}.fa-fire:before{content:"\\F06D"}.fa-eye:before{content:"\\F06E"}.fa-eye-slash:before{content:"\\F070"}.fa-warning:before,.fa-exclamation-triangle:before{content:"\\F071"}.fa-plane:before{content:"\\F072"}.fa-calendar:before{content:"\\F073"}.fa-random:before{content:"\\F074"}.fa-comment:before{content:"\\F075"}.fa-magnet:before{content:"\\F076"}.fa-chevron-up:before{content:"\\F077"}.fa-chevron-down:before{content:"\\F078"}.fa-retweet:before{content:"\\F079"}.fa-shopping-cart:before{content:"\\F07A"}.fa-folder:before{content:"\\F07B"}.fa-folder-open:before{content:"\\F07C"}.fa-arrows-v:before{content:"\\F07D"}.fa-arrows-h:before{content:"\\F07E"}.fa-bar-chart-o:before,.fa-bar-chart:before{content:"\\F080"}.fa-twitter-square:before{content:"\\F081"}.fa-facebook-square:before{content:"\\F082"}.fa-camera-retro:before{content:"\\F083"}.fa-key:before{content:"\\F084"}.fa-gears:before,.fa-cogs:before{content:"\\F085"}.fa-comments:before{content:"\\F086"}.fa-thumbs-o-up:before{content:"\\F087"}.fa-thumbs-o-down:before{content:"\\F088"}.fa-star-half:before{content:"\\F089"}.fa-heart-o:before{content:"\\F08A"}.fa-sign-out:before{content:"\\F08B"}.fa-linkedin-square:before{content:"\\F08C"}.fa-thumb-tack:before{content:"\\F08D"}.fa-external-link:before{content:"\\F08E"}.fa-sign-in:before{content:"\\F090"}.fa-trophy:before{content:"\\F091"}.fa-github-square:before{content:"\\F092"}.fa-upload:before{content:"\\F093"}.fa-lemon-o:before{content:"\\F094"}.fa-phone:before{content:"\\F095"}.fa-square-o:before{content:"\\F096"}.fa-bookmark-o:before{content:"\\F097"}.fa-phone-square:before{content:"\\F098"}.fa-twitter:before{content:"\\F099"}.fa-facebook-f:before,.fa-facebook:before{content:"\\F09A"}.fa-github:before{content:"\\F09B"}.fa-unlock:before{content:"\\F09C"}.fa-credit-card:before{content:"\\F09D"}.fa-feed:before,.fa-rss:before{content:"\\F09E"}.fa-hdd-o:before{content:"\\F0A0"}.fa-bullhorn:before{content:"\\F0A1"}.fa-bell:before{content:"\\F0F3"}.fa-certificate:before{content:"\\F0A3"}.fa-hand-o-right:before{content:"\\F0A4"}.fa-hand-o-left:before{content:"\\F0A5"}.fa-hand-o-up:before{content:"\\F0A6"}.fa-hand-o-down:before{content:"\\F0A7"}.fa-arrow-circle-left:before{content:"\\F0A8"}.fa-arrow-circle-right:before{content:"\\F0A9"}.fa-arrow-circle-up:before{content:"\\F0AA"}.fa-arrow-circle-down:before{content:"\\F0AB"}.fa-globe:before{content:"\\F0AC"}.fa-wrench:before{content:"\\F0AD"}.fa-tasks:before{content:"\\F0AE"}.fa-filter:before{content:"\\F0B0"}.fa-briefcase:before{content:"\\F0B1"}.fa-arrows-alt:before{content:"\\F0B2"}.fa-group:before,.fa-users:before{content:"\\F0C0"}.fa-chain:before,.fa-link:before{content:"\\F0C1"}.fa-cloud:before{content:"\\F0C2"}.fa-flask:before{content:"\\F0C3"}.fa-cut:before,.fa-scissors:before{content:"\\F0C4"}.fa-copy:before,.fa-files-o:before{content:"\\F0C5"}.fa-paperclip:before{content:"\\F0C6"}.fa-save:before,.fa-floppy-o:before{content:"\\F0C7"}.fa-square:before{content:"\\F0C8"}.fa-navicon:before,.fa-reorder:before,.fa-bars:before{content:"\\F0C9"}.fa-list-ul:before{content:"\\F0CA"}.fa-list-ol:before{content:"\\F0CB"}.fa-strikethrough:before{content:"\\F0CC"}.fa-underline:before{content:"\\F0CD"}.fa-table:before{content:"\\F0CE"}.fa-magic:before{content:"\\F0D0"}.fa-truck:before{content:"\\F0D1"}.fa-pinterest:before{content:"\\F0D2"}.fa-pinterest-square:before{content:"\\F0D3"}.fa-google-plus-square:before{content:"\\F0D4"}.fa-google-plus:before{content:"\\F0D5"}.fa-money:before{content:"\\F0D6"}.fa-caret-down:before{content:"\\F0D7"}.fa-caret-up:before{content:"\\F0D8"}.fa-caret-left:before{content:"\\F0D9"}.fa-caret-right:before{content:"\\F0DA"}.fa-columns:before{content:"\\F0DB"}.fa-unsorted:before,.fa-sort:before{content:"\\F0DC"}.fa-sort-down:before,.fa-sort-desc:before{content:"\\F0DD"}.fa-sort-up:before,.fa-sort-asc:before{content:"\\F0DE"}.fa-envelope:before{content:"\\F0E0"}.fa-linkedin:before{content:"\\F0E1"}.fa-rotate-left:before,.fa-undo:before{content:"\\F0E2"}.fa-legal:before,.fa-gavel:before{content:"\\F0E3"}.fa-dashboard:before,.fa-tachometer:before{content:"\\F0E4"}.fa-comment-o:before{content:"\\F0E5"}.fa-comments-o:before{content:"\\F0E6"}.fa-flash:before,.fa-bolt:before{content:"\\F0E7"}.fa-sitemap:before{content:"\\F0E8"}.fa-umbrella:before{content:"\\F0E9"}.fa-paste:before,.fa-clipboard:before{content:"\\F0EA"}.fa-lightbulb-o:before{content:"\\F0EB"}.fa-exchange:before{content:"\\F0EC"}.fa-cloud-download:before{content:"\\F0ED"}.fa-cloud-upload:before{content:"\\F0EE"}.fa-user-md:before{content:"\\F0F0"}.fa-stethoscope:before{content:"\\F0F1"}.fa-suitcase:before{content:"\\F0F2"}.fa-bell-o:before{content:"\\F0A2"}.fa-coffee:before{content:"\\F0F4"}.fa-cutlery:before{content:"\\F0F5"}.fa-file-text-o:before{content:"\\F0F6"}.fa-building-o:before{content:"\\F0F7"}.fa-hospital-o:before{content:"\\F0F8"}.fa-ambulance:before{content:"\\F0F9"}.fa-medkit:before{content:"\\F0FA"}.fa-fighter-jet:before{content:"\\F0FB"}.fa-beer:before{content:"\\F0FC"}.fa-h-square:before{content:"\\F0FD"}.fa-plus-square:before{content:"\\F0FE"}.fa-angle-double-left:before{content:"\\F100"}.fa-angle-double-right:before{content:"\\F101"}.fa-angle-double-up:before{content:"\\F102"}.fa-angle-double-down:before{content:"\\F103"}.fa-angle-left:before{content:"\\F104"}.fa-angle-right:before{content:"\\F105"}.fa-angle-up:before{content:"\\F106"}.fa-angle-down:before{content:"\\F107"}.fa-desktop:before{content:"\\F108"}.fa-laptop:before{content:"\\F109"}.fa-tablet:before{content:"\\F10A"}.fa-mobile-phone:before,.fa-mobile:before{content:"\\F10B"}.fa-circle-o:before{content:"\\F10C"}.fa-quote-left:before{content:"\\F10D"}.fa-quote-right:before{content:"\\F10E"}.fa-spinner:before{content:"\\F110"}.fa-circle:before{content:"\\F111"}.fa-mail-reply:before,.fa-reply:before{content:"\\F112"}.fa-github-alt:before{content:"\\F113"}.fa-folder-o:before{content:"\\F114"}.fa-folder-open-o:before{content:"\\F115"}.fa-smile-o:before{content:"\\F118"}.fa-frown-o:before{content:"\\F119"}.fa-meh-o:before{content:"\\F11A"}.fa-gamepad:before{content:"\\F11B"}.fa-keyboard-o:before{content:"\\F11C"}.fa-flag-o:before{content:"\\F11D"}.fa-flag-checkered:before{content:"\\F11E"}.fa-terminal:before{content:"\\F120"}.fa-code:before{content:"\\F121"}.fa-mail-reply-all:before,.fa-reply-all:before{content:"\\F122"}.fa-star-half-empty:before,.fa-star-half-full:before,.fa-star-half-o:before{content:"\\F123"}.fa-location-arrow:before{content:"\\F124"}.fa-crop:before{content:"\\F125"}.fa-code-fork:before{content:"\\F126"}.fa-unlink:before,.fa-chain-broken:before{content:"\\F127"}.fa-question:before{content:"\\F128"}.fa-info:before{content:"\\F129"}.fa-exclamation:before{content:"\\F12A"}.fa-superscript:before{content:"\\F12B"}.fa-subscript:before{content:"\\F12C"}.fa-eraser:before{content:"\\F12D"}.fa-puzzle-piece:before{content:"\\F12E"}.fa-microphone:before{content:"\\F130"}.fa-microphone-slash:before{content:"\\F131"}.fa-shield:before{content:"\\F132"}.fa-calendar-o:before{content:"\\F133"}.fa-fire-extinguisher:before{content:"\\F134"}.fa-rocket:before{content:"\\F135"}.fa-maxcdn:before{content:"\\F136"}.fa-chevron-circle-left:before{content:"\\F137"}.fa-chevron-circle-right:before{content:"\\F138"}.fa-chevron-circle-up:before{content:"\\F139"}.fa-chevron-circle-down:before{content:"\\F13A"}.fa-html5:before{content:"\\F13B"}.fa-css3:before{content:"\\F13C"}.fa-anchor:before{content:"\\F13D"}.fa-unlock-alt:before{content:"\\F13E"}.fa-bullseye:before{content:"\\F140"}.fa-ellipsis-h:before{content:"\\F141"}.fa-ellipsis-v:before{content:"\\F142"}.fa-rss-square:before{content:"\\F143"}.fa-play-circle:before{content:"\\F144"}.fa-ticket:before{content:"\\F145"}.fa-minus-square:before{content:"\\F146"}.fa-minus-square-o:before{content:"\\F147"}.fa-level-up:before{content:"\\F148"}.fa-level-down:before{content:"\\F149"}.fa-check-square:before{content:"\\F14A"}.fa-pencil-square:before{content:"\\F14B"}.fa-external-link-square:before{content:"\\F14C"}.fa-share-square:before{content:"\\F14D"}.fa-compass:before{content:"\\F14E"}.fa-toggle-down:before,.fa-caret-square-o-down:before{content:"\\F150"}.fa-toggle-up:before,.fa-caret-square-o-up:before{content:"\\F151"}.fa-toggle-right:before,.fa-caret-square-o-right:before{content:"\\F152"}.fa-euro:before,.fa-eur:before{content:"\\F153"}.fa-gbp:before{content:"\\F154"}.fa-dollar:before,.fa-usd:before{content:"\\F155"}.fa-rupee:before,.fa-inr:before{content:"\\F156"}.fa-cny:before,.fa-rmb:before,.fa-yen:before,.fa-jpy:before{content:"\\F157"}.fa-ruble:before,.fa-rouble:before,.fa-rub:before{content:"\\F158"}.fa-won:before,.fa-krw:before{content:"\\F159"}.fa-bitcoin:before,.fa-btc:before{content:"\\F15A"}.fa-file:before{content:"\\F15B"}.fa-file-text:before{content:"\\F15C"}.fa-sort-alpha-asc:before{content:"\\F15D"}.fa-sort-alpha-desc:before{content:"\\F15E"}.fa-sort-amount-asc:before{content:"\\F160"}.fa-sort-amount-desc:before{content:"\\F161"}.fa-sort-numeric-asc:before{content:"\\F162"}.fa-sort-numeric-desc:before{content:"\\F163"}.fa-thumbs-up:before{content:"\\F164"}.fa-thumbs-down:before{content:"\\F165"}.fa-youtube-square:before{content:"\\F166"}.fa-youtube:before{content:"\\F167"}.fa-xing:before{content:"\\F168"}.fa-xing-square:before{content:"\\F169"}.fa-youtube-play:before{content:"\\F16A"}.fa-dropbox:before{content:"\\F16B"}.fa-stack-overflow:before{content:"\\F16C"}.fa-instagram:before{content:"\\F16D"}.fa-flickr:before{content:"\\F16E"}.fa-adn:before{content:"\\F170"}.fa-bitbucket:before{content:"\\F171"}.fa-bitbucket-square:before{content:"\\F172"}.fa-tumblr:before{content:"\\F173"}.fa-tumblr-square:before{content:"\\F174"}.fa-long-arrow-down:before{content:"\\F175"}.fa-long-arrow-up:before{content:"\\F176"}.fa-long-arrow-left:before{content:"\\F177"}.fa-long-arrow-right:before{content:"\\F178"}.fa-apple:before{content:"\\F179"}.fa-windows:before{content:"\\F17A"}.fa-android:before{content:"\\F17B"}.fa-linux:before{content:"\\F17C"}.fa-dribbble:before{content:"\\F17D"}.fa-skype:before{content:"\\F17E"}.fa-foursquare:before{content:"\\F180"}.fa-trello:before{content:"\\F181"}.fa-female:before{content:"\\F182"}.fa-male:before{content:"\\F183"}.fa-gittip:before,.fa-gratipay:before{content:"\\F184"}.fa-sun-o:before{content:"\\F185"}.fa-moon-o:before{content:"\\F186"}.fa-archive:before{content:"\\F187"}.fa-bug:before{content:"\\F188"}.fa-vk:before{content:"\\F189"}.fa-weibo:before{content:"\\F18A"}.fa-renren:before{content:"\\F18B"}.fa-pagelines:before{content:"\\F18C"}.fa-stack-exchange:before{content:"\\F18D"}.fa-arrow-circle-o-right:before{content:"\\F18E"}.fa-arrow-circle-o-left:before{content:"\\F190"}.fa-toggle-left:before,.fa-caret-square-o-left:before{content:"\\F191"}.fa-dot-circle-o:before{content:"\\F192"}.fa-wheelchair:before{content:"\\F193"}.fa-vimeo-square:before{content:"\\F194"}.fa-turkish-lira:before,.fa-try:before{content:"\\F195"}.fa-plus-square-o:before{content:"\\F196"}.fa-space-shuttle:before{content:"\\F197"}.fa-slack:before{content:"\\F198"}.fa-envelope-square:before{content:"\\F199"}.fa-wordpress:before{content:"\\F19A"}.fa-openid:before{content:"\\F19B"}.fa-institution:before,.fa-bank:before,.fa-university:before{content:"\\F19C"}.fa-mortar-board:before,.fa-graduation-cap:before{content:"\\F19D"}.fa-yahoo:before{content:"\\F19E"}.fa-google:before{content:"\\F1A0"}.fa-reddit:before{content:"\\F1A1"}.fa-reddit-square:before{content:"\\F1A2"}.fa-stumbleupon-circle:before{content:"\\F1A3"}.fa-stumbleupon:before{content:"\\F1A4"}.fa-delicious:before{content:"\\F1A5"}.fa-digg:before{content:"\\F1A6"}.fa-pied-piper-pp:before{content:"\\F1A7"}.fa-pied-piper-alt:before{content:"\\F1A8"}.fa-drupal:before{content:"\\F1A9"}.fa-joomla:before{content:"\\F1AA"}.fa-language:before{content:"\\F1AB"}.fa-fax:before{content:"\\F1AC"}.fa-building:before{content:"\\F1AD"}.fa-child:before{content:"\\F1AE"}.fa-paw:before{content:"\\F1B0"}.fa-spoon:before{content:"\\F1B1"}.fa-cube:before{content:"\\F1B2"}.fa-cubes:before{content:"\\F1B3"}.fa-behance:before{content:"\\F1B4"}.fa-behance-square:before{content:"\\F1B5"}.fa-steam:before{content:"\\F1B6"}.fa-steam-square:before{content:"\\F1B7"}.fa-recycle:before{content:"\\F1B8"}.fa-automobile:before,.fa-car:before{content:"\\F1B9"}.fa-cab:before,.fa-taxi:before{content:"\\F1BA"}.fa-tree:before{content:"\\F1BB"}.fa-spotify:before{content:"\\F1BC"}.fa-deviantart:before{content:"\\F1BD"}.fa-soundcloud:before{content:"\\F1BE"}.fa-database:before{content:"\\F1C0"}.fa-file-pdf-o:before{content:"\\F1C1"}.fa-file-word-o:before{content:"\\F1C2"}.fa-file-excel-o:before{content:"\\F1C3"}.fa-file-powerpoint-o:before{content:"\\F1C4"}.fa-file-photo-o:before,.fa-file-picture-o:before,.fa-file-image-o:before{content:"\\F1C5"}.fa-file-zip-o:before,.fa-file-archive-o:before{content:"\\F1C6"}.fa-file-sound-o:before,.fa-file-audio-o:before{content:"\\F1C7"}.fa-file-movie-o:before,.fa-file-video-o:before{content:"\\F1C8"}.fa-file-code-o:before{content:"\\F1C9"}.fa-vine:before{content:"\\F1CA"}.fa-codepen:before{content:"\\F1CB"}.fa-jsfiddle:before{content:"\\F1CC"}.fa-life-bouy:before,.fa-life-buoy:before,.fa-life-saver:before,.fa-support:before,.fa-life-ring:before{content:"\\F1CD"}.fa-circle-o-notch:before{content:"\\F1CE"}.fa-ra:before,.fa-resistance:before,.fa-rebel:before{content:"\\F1D0"}.fa-ge:before,.fa-empire:before{content:"\\F1D1"}.fa-git-square:before{content:"\\F1D2"}.fa-git:before{content:"\\F1D3"}.fa-y-combinator-square:before,.fa-yc-square:before,.fa-hacker-news:before{content:"\\F1D4"}.fa-tencent-weibo:before{content:"\\F1D5"}.fa-qq:before{content:"\\F1D6"}.fa-wechat:before,.fa-weixin:before{content:"\\F1D7"}.fa-send:before,.fa-paper-plane:before{content:"\\F1D8"}.fa-send-o:before,.fa-paper-plane-o:before{content:"\\F1D9"}.fa-history:before{content:"\\F1DA"}.fa-circle-thin:before{content:"\\F1DB"}.fa-header:before{content:"\\F1DC"}.fa-paragraph:before{content:"\\F1DD"}.fa-sliders:before{content:"\\F1DE"}.fa-share-alt:before{content:"\\F1E0"}.fa-share-alt-square:before{content:"\\F1E1"}.fa-bomb:before{content:"\\F1E2"}.fa-soccer-ball-o:before,.fa-futbol-o:before{content:"\\F1E3"}.fa-tty:before{content:"\\F1E4"}.fa-binoculars:before{content:"\\F1E5"}.fa-plug:before{content:"\\F1E6"}.fa-slideshare:before{content:"\\F1E7"}.fa-twitch:before{content:"\\F1E8"}.fa-yelp:before{content:"\\F1E9"}.fa-newspaper-o:before{content:"\\F1EA"}.fa-wifi:before{content:"\\F1EB"}.fa-calculator:before{content:"\\F1EC"}.fa-paypal:before{content:"\\F1ED"}.fa-google-wallet:before{content:"\\F1EE"}.fa-cc-visa:before{content:"\\F1F0"}.fa-cc-mastercard:before{content:"\\F1F1"}.fa-cc-discover:before{content:"\\F1F2"}.fa-cc-amex:before{content:"\\F1F3"}.fa-cc-paypal:before{content:"\\F1F4"}.fa-cc-stripe:before{content:"\\F1F5"}.fa-bell-slash:before{content:"\\F1F6"}.fa-bell-slash-o:before{content:"\\F1F7"}.fa-trash:before{content:"\\F1F8"}.fa-copyright:before{content:"\\F1F9"}.fa-at:before{content:"\\F1FA"}.fa-eyedropper:before{content:"\\F1FB"}.fa-paint-brush:before{content:"\\F1FC"}.fa-birthday-cake:before{content:"\\F1FD"}.fa-area-chart:before{content:"\\F1FE"}.fa-pie-chart:before{content:"\\F200"}.fa-line-chart:before{content:"\\F201"}.fa-lastfm:before{content:"\\F202"}.fa-lastfm-square:before{content:"\\F203"}.fa-toggle-off:before{content:"\\F204"}.fa-toggle-on:before{content:"\\F205"}.fa-bicycle:before{content:"\\F206"}.fa-bus:before{content:"\\F207"}.fa-ioxhost:before{content:"\\F208"}.fa-angellist:before{content:"\\F209"}.fa-cc:before{content:"\\F20A"}.fa-shekel:before,.fa-sheqel:before,.fa-ils:before{content:"\\F20B"}.fa-meanpath:before{content:"\\F20C"}.fa-buysellads:before{content:"\\F20D"}.fa-connectdevelop:before{content:"\\F20E"}.fa-dashcube:before{content:"\\F210"}.fa-forumbee:before{content:"\\F211"}.fa-leanpub:before{content:"\\F212"}.fa-sellsy:before{content:"\\F213"}.fa-shirtsinbulk:before{content:"\\F214"}.fa-simplybuilt:before{content:"\\F215"}.fa-skyatlas:before{content:"\\F216"}.fa-cart-plus:before{content:"\\F217"}.fa-cart-arrow-down:before{content:"\\F218"}.fa-diamond:before{content:"\\F219"}.fa-ship:before{content:"\\F21A"}.fa-user-secret:before{content:"\\F21B"}.fa-motorcycle:before{content:"\\F21C"}.fa-street-view:before{content:"\\F21D"}.fa-heartbeat:before{content:"\\F21E"}.fa-venus:before{content:"\\F221"}.fa-mars:before{content:"\\F222"}.fa-mercury:before{content:"\\F223"}.fa-intersex:before,.fa-transgender:before{content:"\\F224"}.fa-transgender-alt:before{content:"\\F225"}.fa-venus-double:before{content:"\\F226"}.fa-mars-double:before{content:"\\F227"}.fa-venus-mars:before{content:"\\F228"}.fa-mars-stroke:before{content:"\\F229"}.fa-mars-stroke-v:before{content:"\\F22A"}.fa-mars-stroke-h:before{content:"\\F22B"}.fa-neuter:before{content:"\\F22C"}.fa-genderless:before{content:"\\F22D"}.fa-facebook-official:before{content:"\\F230"}.fa-pinterest-p:before{content:"\\F231"}.fa-whatsapp:before{content:"\\F232"}.fa-server:before{content:"\\F233"}.fa-user-plus:before{content:"\\F234"}.fa-user-times:before{content:"\\F235"}.fa-hotel:before,.fa-bed:before{content:"\\F236"}.fa-viacoin:before{content:"\\F237"}.fa-train:before{content:"\\F238"}.fa-subway:before{content:"\\F239"}.fa-medium:before{content:"\\F23A"}.fa-yc:before,.fa-y-combinator:before{content:"\\F23B"}.fa-optin-monster:before{content:"\\F23C"}.fa-opencart:before{content:"\\F23D"}.fa-expeditedssl:before{content:"\\F23E"}.fa-battery-4:before,.fa-battery:before,.fa-battery-full:before{content:"\\F240"}.fa-battery-3:before,.fa-battery-three-quarters:before{content:"\\F241"}.fa-battery-2:before,.fa-battery-half:before{content:"\\F242"}.fa-battery-1:before,.fa-battery-quarter:before{content:"\\F243"}.fa-battery-0:before,.fa-battery-empty:before{content:"\\F244"}.fa-mouse-pointer:before{content:"\\F245"}.fa-i-cursor:before{content:"\\F246"}.fa-object-group:before{content:"\\F247"}.fa-object-ungroup:before{content:"\\F248"}.fa-sticky-note:before{content:"\\F249"}.fa-sticky-note-o:before{content:"\\F24A"}.fa-cc-jcb:before{content:"\\F24B"}.fa-cc-diners-club:before{content:"\\F24C"}.fa-clone:before{content:"\\F24D"}.fa-balance-scale:before{content:"\\F24E"}.fa-hourglass-o:before{content:"\\F250"}.fa-hourglass-1:before,.fa-hourglass-start:before{content:"\\F251"}.fa-hourglass-2:before,.fa-hourglass-half:before{content:"\\F252"}.fa-hourglass-3:before,.fa-hourglass-end:before{content:"\\F253"}.fa-hourglass:before{content:"\\F254"}.fa-hand-grab-o:before,.fa-hand-rock-o:before{content:"\\F255"}.fa-hand-stop-o:before,.fa-hand-paper-o:before{content:"\\F256"}.fa-hand-scissors-o:before{content:"\\F257"}.fa-hand-lizard-o:before{content:"\\F258"}.fa-hand-spock-o:before{content:"\\F259"}.fa-hand-pointer-o:before{content:"\\F25A"}.fa-hand-peace-o:before{content:"\\F25B"}.fa-trademark:before{content:"\\F25C"}.fa-registered:before{content:"\\F25D"}.fa-creative-commons:before{content:"\\F25E"}.fa-gg:before{content:"\\F260"}.fa-gg-circle:before{content:"\\F261"}.fa-tripadvisor:before{content:"\\F262"}.fa-odnoklassniki:before{content:"\\F263"}.fa-odnoklassniki-square:before{content:"\\F264"}.fa-get-pocket:before{content:"\\F265"}.fa-wikipedia-w:before{content:"\\F266"}.fa-safari:before{content:"\\F267"}.fa-chrome:before{content:"\\F268"}.fa-firefox:before{content:"\\F269"}.fa-opera:before{content:"\\F26A"}.fa-internet-explorer:before{content:"\\F26B"}.fa-tv:before,.fa-television:before{content:"\\F26C"}.fa-contao:before{content:"\\F26D"}.fa-500px:before{content:"\\F26E"}.fa-amazon:before{content:"\\F270"}.fa-calendar-plus-o:before{content:"\\F271"}.fa-calendar-minus-o:before{content:"\\F272"}.fa-calendar-times-o:before{content:"\\F273"}.fa-calendar-check-o:before{content:"\\F274"}.fa-industry:before{content:"\\F275"}.fa-map-pin:before{content:"\\F276"}.fa-map-signs:before{content:"\\F277"}.fa-map-o:before{content:"\\F278"}.fa-map:before{content:"\\F279"}.fa-commenting:before{content:"\\F27A"}.fa-commenting-o:before{content:"\\F27B"}.fa-houzz:before{content:"\\F27C"}.fa-vimeo:before{content:"\\F27D"}.fa-black-tie:before{content:"\\F27E"}.fa-fonticons:before{content:"\\F280"}.fa-reddit-alien:before{content:"\\F281"}.fa-edge:before{content:"\\F282"}.fa-credit-card-alt:before{content:"\\F283"}.fa-codiepie:before{content:"\\F284"}.fa-modx:before{content:"\\F285"}.fa-fort-awesome:before{content:"\\F286"}.fa-usb:before{content:"\\F287"}.fa-product-hunt:before{content:"\\F288"}.fa-mixcloud:before{content:"\\F289"}.fa-scribd:before{content:"\\F28A"}.fa-pause-circle:before{content:"\\F28B"}.fa-pause-circle-o:before{content:"\\F28C"}.fa-stop-circle:before{content:"\\F28D"}.fa-stop-circle-o:before{content:"\\F28E"}.fa-shopping-bag:before{content:"\\F290"}.fa-shopping-basket:before{content:"\\F291"}.fa-hashtag:before{content:"\\F292"}.fa-bluetooth:before{content:"\\F293"}.fa-bluetooth-b:before{content:"\\F294"}.fa-percent:before{content:"\\F295"}.fa-gitlab:before{content:"\\F296"}.fa-wpbeginner:before{content:"\\F297"}.fa-wpforms:before{content:"\\F298"}.fa-envira:before{content:"\\F299"}.fa-universal-access:before{content:"\\F29A"}.fa-wheelchair-alt:before{content:"\\F29B"}.fa-question-circle-o:before{content:"\\F29C"}.fa-blind:before{content:"\\F29D"}.fa-audio-description:before{content:"\\F29E"}.fa-volume-control-phone:before{content:"\\F2A0"}.fa-braille:before{content:"\\F2A1"}.fa-assistive-listening-systems:before{content:"\\F2A2"}.fa-asl-interpreting:before,.fa-american-sign-language-interpreting:before{content:"\\F2A3"}.fa-deafness:before,.fa-hard-of-hearing:before,.fa-deaf:before{content:"\\F2A4"}.fa-glide:before{content:"\\F2A5"}.fa-glide-g:before{content:"\\F2A6"}.fa-signing:before,.fa-sign-language:before{content:"\\F2A7"}.fa-low-vision:before{content:"\\F2A8"}.fa-viadeo:before{content:"\\F2A9"}.fa-viadeo-square:before{content:"\\F2AA"}.fa-snapchat:before{content:"\\F2AB"}.fa-snapchat-ghost:before{content:"\\F2AC"}.fa-snapchat-square:before{content:"\\F2AD"}.fa-pied-piper:before{content:"\\F2AE"}.fa-first-order:before{content:"\\F2B0"}.fa-yoast:before{content:"\\F2B1"}.fa-themeisle:before{content:"\\F2B2"}.fa-google-plus-circle:before,.fa-google-plus-official:before{content:"\\F2B3"}.fa-fa:before,.fa-font-awesome:before{content:"\\F2B4"}.fa-handshake-o:before{content:"\\F2B5"}.fa-envelope-open:before{content:"\\F2B6"}.fa-envelope-open-o:before{content:"\\F2B7"}.fa-linode:before{content:"\\F2B8"}.fa-address-book:before{content:"\\F2B9"}.fa-address-book-o:before{content:"\\F2BA"}.fa-vcard:before,.fa-address-card:before{content:"\\F2BB"}.fa-vcard-o:before,.fa-address-card-o:before{content:"\\F2BC"}.fa-user-circle:before{content:"\\F2BD"}.fa-user-circle-o:before{content:"\\F2BE"}.fa-user-o:before{content:"\\F2C0"}.fa-id-badge:before{content:"\\F2C1"}.fa-drivers-license:before,.fa-id-card:before{content:"\\F2C2"}.fa-drivers-license-o:before,.fa-id-card-o:before{content:"\\F2C3"}.fa-quora:before{content:"\\F2C4"}.fa-free-code-camp:before{content:"\\F2C5"}.fa-telegram:before{content:"\\F2C6"}.fa-thermometer-4:before,.fa-thermometer:before,.fa-thermometer-full:before{content:"\\F2C7"}.fa-thermometer-3:before,.fa-thermometer-three-quarters:before{content:"\\F2C8"}.fa-thermometer-2:before,.fa-thermometer-half:before{content:"\\F2C9"}.fa-thermometer-1:before,.fa-thermometer-quarter:before{content:"\\F2CA"}.fa-thermometer-0:before,.fa-thermometer-empty:before{content:"\\F2CB"}.fa-shower:before{content:"\\F2CC"}.fa-bathtub:before,.fa-s15:before,.fa-bath:before{content:"\\F2CD"}.fa-podcast:before{content:"\\F2CE"}.fa-window-maximize:before{content:"\\F2D0"}.fa-window-minimize:before{content:"\\F2D1"}.fa-window-restore:before{content:"\\F2D2"}.fa-times-rectangle:before,.fa-window-close:before{content:"\\F2D3"}.fa-times-rectangle-o:before,.fa-window-close-o:before{content:"\\F2D4"}.fa-bandcamp:before{content:"\\F2D5"}.fa-grav:before{content:"\\F2D6"}.fa-etsy:before{content:"\\F2D7"}.fa-imdb:before{content:"\\F2D8"}.fa-ravelry:before{content:"\\F2D9"}.fa-eercast:before{content:"\\F2DA"}.fa-microchip:before{content:"\\F2DB"}.fa-snowflake-o:before{content:"\\F2DC"}.fa-superpowers:before{content:"\\F2DD"}.fa-wpexplorer:before{content:"\\F2DE"}.fa-meetup:before{content:"\\F2E0"}.sr-only{position:absolute;width:1px;height:1px;padding:0;margin:-1px;overflow:hidden;clip:rect(0, 0, 0, 0);border:0}.sr-only-focusable:active,.sr-only-focusable:focus{position:static;width:auto;height:auto;margin:0;overflow:visible;clip:auto}\n',""])},function(t,e,n){t.exports=n.p+"674f50d287a8c48dc19ba404d20fe713.eot"},function(t,e,n){t.exports=n.p+"674f50d287a8c48dc19ba404d20fe713.eot"},function(t,e,n){t.exports=n.p+"af7ae505a9eed503f8b8e6982036873e.woff2"},function(t,e,n){t.exports=n.p+"fee66e712a8a08eef5805a46892932ad.woff"},function(t,e,n){t.exports=n.p+"b06871f281fee6b241d60582ae9369b9.ttf"},function(t,e,n){t.exports=n.p+"912ec66d7572ff821749319396470bde.svg"},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(1013),""),e.i(n(1014),""),e.i(n(1015),""),e.i(n(1016),""),e.i(n(1017),""),e.i(n(1018),""),e.i(n(1019),""),e.i(n(1020),""),e.i(n(1021),""),e.push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n.p-Widget.p-mod-hidden {\n  display: none !important;\n}\n\nbody {\n  font-family: var(--jp-ui-font-family);\n  background: var(--jp-layout-color3);\n  margin: 0;\n  padding: 0;\n  overflow: hidden;\n}\n\n.jp-LabShell {\n  position: absolute;\n  top: 0;\n  left: 0;\n  right: 0;\n  bottom: 0;\n}\n\n.jp-LabShell.jp-mod-devMode {\n  border-top: 4px solid red;\n}\n\n#jp-main-dock-panel {\n  padding: 5px;\n}\n\n#jp-top-panel {\n  border-bottom: var(--jp-border-width) solid var(--jp-border-color0);\n  background: var(--jp-layout-color1);\n  display: flex;\n  min-height: var(--jp-private-menubar-height);\n}\n\n#jp-bottom-panel {\n  background: var(--jp-layout-color1);\n  display: flex;\n}\n\n/* Sibling imports */\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n.p-DataGrid {\n  min-width: 64px;\n  min-height: 64px;\n  border: 1px solid #a0a0a0;\n}\n\n.p-DataGrid-scrollCorner {\n  background-color: #f0f0f0;\n}\n\n.p-DataGrid-scrollCorner::after {\n  content: '';\n  position: absolute;\n  top: 0;\n  left: 0;\n  width: 1px;\n  height: 1px;\n  background-color: #a0a0a0;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/*-----------------------------------------------------------------------------\n| Variables\n|----------------------------------------------------------------------------*/\n\n/*-----------------------------------------------------------------------------\n| DockPanel\n|----------------------------------------------------------------------------*/\n\n.p-DockPanel-widget {\n  background: var(--jp-layout-color0);\n  border-left: var(--jp-border-width) solid var(--jp-border-color1);\n  border-right: var(--jp-border-width) solid var(--jp-border-color1);\n  border-bottom: var(--jp-border-width) solid var(--jp-border-color1);\n}\n\n.p-DockPanel-overlay {\n  background: rgba(33, 150, 243, 0.1);\n  border: var(--jp-border-width) dashed var(--jp-brand-color1);\n  transition-property: top, left, right, bottom;\n  transition-duration: 150ms;\n  transition-timing-function: ease;\n}\n\n.p-DockPanel-overlay.p-mod-root-top,\n.p-DockPanel-overlay.p-mod-root-left,\n.p-DockPanel-overlay.p-mod-root-right,\n.p-DockPanel-overlay.p-mod-root-bottom,\n.p-DockPanel-overlay.p-mod-root-center {\n  border-width: 2px;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/* CSS for specific icons */\n\n.jp-ImageJupyterLab {\n  background-image: var(--jp-image-jupyterlab);\n}\n\n.jp-ImageJupyterLab-wordmark {\n  background-image: var(--jp-image-jupyterlab-wordmark);\n}\n\n.jp-JupyterIcon {\n  background-image: var(--jp-image-jupyter);\n  background-size: 16px;\n  margin: 2px;\n  margin-left: 8px;\n  margin-right: 0px;\n  padding-right: 0px;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/**\n * Support for icons as inline SVG HTMLElements\n */\n\n/* recolor the primary elements of an icon */\n.jp-icon0[fill] {\n  fill: var(--jp-inverse-layout-color0);\n}\n.jp-icon1[fill] {\n  fill: var(--jp-inverse-layout-color1);\n}\n.jp-icon2[fill] {\n  fill: var(--jp-inverse-layout-color2);\n}\n.jp-icon3[fill] {\n  fill: var(--jp-inverse-layout-color3);\n}\n.jp-icon4[fill] {\n  fill: var(--jp-inverse-layout-color4);\n}\n\n.jp-icon0[stroke] {\n  stroke: var(--jp-inverse-layout-color0);\n}\n.jp-icon1[stroke] {\n  stroke: var(--jp-inverse-layout-color1);\n}\n.jp-icon2[stroke] {\n  stroke: var(--jp-inverse-layout-color2);\n}\n.jp-icon3[stroke] {\n  stroke: var(--jp-inverse-layout-color3);\n}\n.jp-icon4[stroke] {\n  stroke: var(--jp-inverse-layout-color4);\n}\n\n/* recolor the accent elements of an icon */\n.jp-icon-accent0[fill] {\n  fill: var(--jp-layout-color0);\n}\n.jp-icon-accent1[fill] {\n  fill: var(--jp-layout-color1);\n}\n.jp-icon-accent2[fill] {\n  fill: var(--jp-layout-color2);\n}\n.jp-icon-accent3[fill] {\n  fill: var(--jp-layout-color3);\n}\n.jp-icon-accent4[fill] {\n  fill: var(--jp-layout-color4);\n}\n\n.jp-icon-accent0[stroke] {\n  stroke: var(--jp-layout-color0);\n}\n.jp-icon-accent1[stroke] {\n  stroke: var(--jp-layout-color1);\n}\n.jp-icon-accent2[stroke] {\n  stroke: var(--jp-layout-color2);\n}\n.jp-icon-accent3[stroke] {\n  stroke: var(--jp-layout-color3);\n}\n.jp-icon-accent4[stroke] {\n  stroke: var(--jp-layout-color4);\n}\n\n/* brand icon colors. Same for light and dark */\n.jp-icon-brand0[fill] {\n  fill: var(--jp-brand-color0);\n}\n.jp-icon-brand1[fill] {\n  fill: var(--jp-brand-color1);\n}\n.jp-icon-brand2[fill] {\n  fill: var(--jp-brand-color2);\n}\n.jp-icon-brand3[fill] {\n  fill: var(--jp-brand-color3);\n}\n.jp-icon-brand4[fill] {\n  fill: var(--jp-brand-color4);\n}\n\n.jp-icon-brand0[stroke] {\n  stroke: var(--jp-brand-color0);\n}\n.jp-icon-brand1[stroke] {\n  stroke: var(--jp-brand-color1);\n}\n.jp-icon-brand2[stroke] {\n  stroke: var(--jp-brand-color2);\n}\n.jp-icon-brand3[stroke] {\n  stroke: var(--jp-brand-color3);\n}\n.jp-icon-brand4[stroke] {\n  stroke: var(--jp-brand-color4);\n}\n\n/* warn icon colors. Same for light and dark */\n.jp-icon-warn0[fill] {\n  fill: var(--jp-warn-color0);\n}\n.jp-icon-warn1[fill] {\n  fill: var(--jp-warn-color1);\n}\n.jp-icon-warn2[fill] {\n  fill: var(--jp-warn-color2);\n}\n.jp-icon-warn3[fill] {\n  fill: var(--jp-warn-color3);\n}\n\n.jp-icon-warn0[stroke] {\n  stroke: var(--jp-warn-color0);\n}\n.jp-icon-warn1[stroke] {\n  stroke: var(--jp-warn-color1);\n}\n.jp-icon-warn2[stroke] {\n  stroke: var(--jp-warn-color2);\n}\n.jp-icon-warn3[stroke] {\n  stroke: var(--jp-warn-color3);\n}\n\n/* icon colors that contrast well with each other and most backgrounds */\n.jp-icon-contrast0[fill] {\n  fill: var(--jp-icon-contrast-color0);\n}\n.jp-icon-contrast1[fill] {\n  fill: var(--jp-icon-contrast-color1);\n}\n.jp-icon-contrast2[fill] {\n  fill: var(--jp-icon-contrast-color2);\n}\n.jp-icon-contrast3[fill] {\n  fill: var(--jp-icon-contrast-color3);\n}\n\n.jp-icon-contrast0[stroke] {\n  stroke: var(--jp-icon-contrast-color0);\n}\n.jp-icon-contrast1[stroke] {\n  stroke: var(--jp-icon-contrast-color1);\n}\n.jp-icon-contrast2[stroke] {\n  stroke: var(--jp-icon-contrast-color2);\n}\n.jp-icon-contrast3[stroke] {\n  stroke: var(--jp-icon-contrast-color3);\n}\n\n/* CSS for icons in selected items in the settings editor */\n#setting-editor .jp-PluginList .jp-mod-selected .jp-icon-selectable[fill] {\n  fill: white;\n}\n\n#setting-editor\n  .jp-PluginList\n  .jp-mod-selected\n  .jp-icon-selectable-inverse[fill] {\n  fill: var(--jp-brand-color1);\n}\n\n/* CSS for icons in selected filebrowser listing items */\n.jp-DirListing-item.jp-mod-selected .jp-icon-selectable[fill] {\n  fill: white;\n}\n\n.jp-DirListing-item.jp-mod-selected .jp-icon-selectable-inverse[fill] {\n  fill: var(--jp-brand-color1);\n}\n\n/* CSS for icons in selected tabs in the sidebar tab manager */\n#tab-manager .p-TabBar-tab.jp-mod-active .jp-icon-selectable[fill] {\n  fill: white;\n}\n\n#tab-manager .p-TabBar-tab.jp-mod-active .jp-icon-selectable-inverse[fill] {\n  fill: var(--jp-brand-color1);\n}\n\n/* special handling for splash icon CSS. While the theme CSS reloads during\n   splash, the splash icon can loose theming. To prevent that, we set a\n   default for its color variable */\n:root {\n  --jp-warn-color0: var(--md-orange-700);\n}\n\n/**\n * (DEPRECATED) Support for icons as CSS `background-images`\n */\n\n.jp-MaterialIcon {\n  min-width: 16px;\n  min-height: 16px;\n  background-size: 16px;\n  display: inline-block;\n  vertical-align: middle;\n  background-repeat: no-repeat;\n  background-position: center;\n}\n\n.jp-Icon {\n  display: inline-block;\n  vertical-align: middle;\n  background-repeat: no-repeat;\n  background-position: center;\n  margin: auto;\n}\n\n.jp-Icon-16 {\n  min-width: 16px;\n  min-height: 16px;\n  background-size: 16px;\n}\n\n.jp-Icon-18 {\n  min-width: 18px;\n  min-height: 18px;\n  background-size: 18px;\n}\n\n.jp-Icon-20 {\n  min-width: 20px;\n  min-height: 20px;\n  background-size: 20px;\n}\n\n.jp-BugIcon {\n  background-image: var(--jp-icon-bug);\n}\n\n.jp-BuildIcon {\n  background-image: var(--jp-icon-build);\n}\n\n.jp-CircleIcon {\n  background-image: var(--jp-icon-kernel-running);\n}\n\n.jp-CloseIcon {\n  background-image: var(--jp-icon-close);\n}\n\n.jp-CodeConsoleIcon {\n  background-image: var(--jp-icon-console);\n}\n\n.jp-DirectionsRunIcon {\n  background-image: var(--jp-icon-directions-run);\n}\n\n.jp-DownCaretIcon {\n  background-image: var(--jp-ui-select-caret);\n}\n\n.jp-DownloadIcon {\n  background-image: var(--jp-icon-download);\n}\n\n.jp-DragIcon {\n  margin-right: 4px;\n}\n\n.jp-EditIcon {\n  background-image: var(--jp-icon-edit);\n}\n\n.jp-EllipsesIcon {\n  background-image: var(--jp-icon-ellipses);\n}\n\n.jp-FileUploadIcon {\n  background-image: var(--jp-icon-file-upload);\n}\n\n.jp-FilledCircleIcon {\n  background-image: var(--jp-icon-circle);\n}\n\n.jp-FilterListIcon {\n  background-image: var(--jp-icon-filter-list);\n}\n\n.jp-HomeIcon {\n  background-image: var(--jp-icon-home);\n  min-width: 16px;\n  min-height: 16px;\n  background-size: 16px;\n  vertical-align: sub;\n}\n\n.jp-JupyterIcon {\n  background-image: var(--jp-image-jupyter);\n  background-size: 16px;\n  margin: 2px;\n  margin-left: 8px;\n  margin-right: 0px;\n  padding-right: 0px;\n}\n\n.jp-KeyboardIcon {\n  background-image: var(--jp-icon-keyboard);\n}\n\n.jp-LauncherIcon {\n  background-image: var(--jp-icon-launcher);\n}\n\n.jp-LinkIcon {\n  background-image: var(--jp-icon-link);\n}\n\n.jp-MainAreaLandscapeIcon {\n  background-repeat: no-repeat;\n  margin-right: 2px;\n  width: 22px;\n}\n\n.jp-MainAreaPortraitIcon {\n  background-position: 0 2px;\n  background-repeat: no-repeat;\n  margin-right: 2px;\n  width: 20px;\n}\n\n.jp-MoreHorizIcon {\n  background-image: var(--jp-icon-more);\n}\n\n.jp-NewFolderIcon {\n  background-image: var(--jp-icon-new-folder);\n}\n\n.jp-SettingsIcon {\n  background-image: var(--jp-icon-settings);\n}\n\n.jp-TextEditorIcon {\n  background-image: var(--jp-icon-text-editor);\n}\n\n.jp-UndoIcon {\n  background-image: var(--jp-icon-undo);\n}\n\n.jp-VegaIcon {\n  background-image: var(--jp-icon-vega);\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/*-----------------------------------------------------------------------------\n| Variables\n|----------------------------------------------------------------------------*/\n\n:root {\n  --jp-private-menubar-height: 28px;\n  --jp-private-menu-item-height: 24px;\n}\n\n/*-----------------------------------------------------------------------------\n| MenuBar\n|----------------------------------------------------------------------------*/\n\n.p-MenuBar {\n  padding-left: 5px;\n  background: var(--jp-layout-color1);\n  color: var(--jp-ui-font-color1);\n  font-size: var(--jp-ui-font-size1);\n}\n\n.p-MenuBar-menu {\n  transform: translateY(calc(-2 * var(--jp-border-width)));\n}\n\n.p-MenuBar-item {\n  padding: 0px 8px;\n  border-left: var(--jp-border-width) solid transparent;\n  border-right: var(--jp-border-width) solid transparent;\n  line-height: var(--jp-private-menubar-height);\n}\n\n.p-MenuBar-item.p-mod-active {\n  background: var(--jp-layout-color2);\n}\n\n.p-MenuBar.p-mod-active .p-MenuBar-item.p-mod-active {\n  z-index: 10001;\n  background: var(--jp-layout-color0);\n  border-left: var(--jp-border-width) solid var(--jp-border-color1);\n  border-right: var(--jp-border-width) solid var(--jp-border-color1);\n  box-shadow: var(--jp-elevation-z6);\n}\n\n.p-MenuBar-item.p-mod-disabled {\n  color: var(--jp-ui-font-color3);\n}\n\n.p-MenuBar-item.p-type-separator {\n  margin: 2px;\n  padding: 0;\n  border: none;\n  border-left: var(--jp-border-width) solid var(--jp-border-color2);\n}\n\n.p-MenuBar-itemMnemonic {\n  text-decoration: underline;\n}\n\n/*-----------------------------------------------------------------------------\n| Menu\n|----------------------------------------------------------------------------*/\n\n.p-Menu {\n  z-index: 10000;\n  padding: 4px 0px;\n  background: var(--jp-layout-color0);\n  color: var(--jp-ui-font-color1);\n  border: var(--jp-border-width) solid var(--jp-border-color1);\n  font-size: var(--jp-ui-font-size1);\n  box-shadow: var(--jp-elevation-z6);\n}\n\n.p-Menu-item {\n  min-height: var(--jp-private-menu-item-height);\n  max-height: var(--jp-private-menu-item-height);\n  padding: 0px;\n  line-height: var(--jp-private-menu-item-height);\n}\n\n.p-Menu-item.p-mod-active {\n  background: var(--jp-layout-color2);\n}\n\n.p-Menu-item.p-mod-disabled {\n  color: var(--jp-ui-font-color3);\n}\n\n.p-Menu-itemIcon {\n  width: 21px;\n  padding: 0px 2px 0px 4px;\n  margin-top: -2px;\n}\n\n.p-Menu-itemLabel {\n  padding: 0px 32px 0px 2px;\n}\n\n.p-Menu-itemMnemonic {\n  text-decoration: underline;\n}\n\n.p-Menu-itemShortcut {\n  padding: 0px;\n}\n\n.p-Menu-itemSubmenuIcon {\n  width: 18px;\n  padding: 0px 4px 0px 0px;\n}\n\n.p-Menu-item[data-type='separator'] > div {\n  padding: 0;\n  height: 9px;\n}\n\n.p-Menu-item[data-type='separator'] > div::after {\n  content: '';\n  display: block;\n  position: relative;\n  top: 4px;\n  border-top: var(--jp-border-width) solid var(--jp-layout-color2);\n}\n\n.p-Menu-itemIcon,\n.p-Menu-itemSubmenuIcon {\n  background-size: 16px;\n  background-repeat: no-repeat;\n  background-position: center;\n}\n\n.p-Menu-item.p-mod-toggled > .p-Menu-itemIcon {\n  background-image: var(--jp-icon-checkmark);\n}\n\n.p-Menu-item.p-mod-toggled.p-mod-disabled > .p-Menu-itemIcon {\n  background-image: var(--jp-icon-checkmark-disabled);\n}\n\n.p-Menu-item[data-type='submenu'] > .p-Menu-itemSubmenuIcon {\n  background-image: var(--jp-icon-caretright);\n  background-size: 18px;\n  background-repeat: no-repeat;\n  background-position: left;\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/*\n * Mozilla scrollbar styling\n */\n\n/* use standard opaque scrollbars for most nodes */\n[data-jp-theme-scrollbars='true'] {\n  scrollbar-color: rgb(var(--jp-scrollbar-thumb-color))\n    var(--jp-scrollbar-background-color);\n}\n\n/* for code nodes, use a transparent style of scrollbar. These selectors\n * will match lower in the tree, and so will override the above */\n[data-jp-theme-scrollbars='true'] .CodeMirror-hscrollbar,\n[data-jp-theme-scrollbars='true'] .CodeMirror-vscrollbar {\n  scrollbar-color: rgba(var(--jp-scrollbar-thumb-color), 0.5) transparent;\n}\n\n/*\n * Webkit scrollbar styling\n */\n\n/* use standard opaque scrollbars for most nodes */\n\n[data-jp-theme-scrollbars='true'] ::-webkit-scrollbar,\n[data-jp-theme-scrollbars='true'] ::-webkit-scrollbar-corner {\n  background: var(--jp-scrollbar-background-color);\n}\n\n[data-jp-theme-scrollbars='true'] ::-webkit-scrollbar-thumb {\n  background: rgb(var(--jp-scrollbar-thumb-color));\n  border: var(--jp-scrollbar-thumb-margin) solid transparent;\n  background-clip: content-box;\n  border-radius: var(--jp-scrollbar-thumb-radius);\n}\n\n[data-jp-theme-scrollbars='true'] ::-webkit-scrollbar-track:horizontal {\n  border-left: var(--jp-scrollbar-endpad) solid\n    var(--jp-scrollbar-background-color);\n  border-right: var(--jp-scrollbar-endpad) solid\n    var(--jp-scrollbar-background-color);\n}\n\n[data-jp-theme-scrollbars='true'] ::-webkit-scrollbar-track:vertical {\n  border-top: var(--jp-scrollbar-endpad) solid\n    var(--jp-scrollbar-background-color);\n  border-bottom: var(--jp-scrollbar-endpad) solid\n    var(--jp-scrollbar-background-color);\n}\n\n/* for code nodes, use a transparent style of scrollbar */\n\n[data-jp-theme-scrollbars='true'] .CodeMirror-hscrollbar::-webkit-scrollbar,\n[data-jp-theme-scrollbars='true'] .CodeMirror-vscrollbar::-webkit-scrollbar,\n[data-jp-theme-scrollbars='true']\n  .CodeMirror-hscrollbar::-webkit-scrollbar-corner,\n[data-jp-theme-scrollbars='true']\n  .CodeMirror-vscrollbar::-webkit-scrollbar-corner {\n  background-color: transparent;\n}\n\n[data-jp-theme-scrollbars='true']\n  .CodeMirror-hscrollbar::-webkit-scrollbar-thumb,\n[data-jp-theme-scrollbars='true']\n  .CodeMirror-vscrollbar::-webkit-scrollbar-thumb {\n  background: rgba(var(--jp-scrollbar-thumb-color), 0.5);\n  border: var(--jp-scrollbar-thumb-margin) solid transparent;\n  background-clip: content-box;\n  border-radius: var(--jp-scrollbar-thumb-radius);\n}\n\n[data-jp-theme-scrollbars='true']\n  .CodeMirror-hscrollbar::-webkit-scrollbar-track:horizontal {\n  border-left: var(--jp-scrollbar-endpad) solid transparent;\n  border-right: var(--jp-scrollbar-endpad) solid transparent;\n}\n\n[data-jp-theme-scrollbars='true']\n  .CodeMirror-vscrollbar::-webkit-scrollbar-track:vertical {\n  border-top: var(--jp-scrollbar-endpad) solid transparent;\n  border-bottom: var(--jp-scrollbar-endpad) solid transparent;\n}\n\n/*\n * Phosphor\n */\n\n.p-ScrollBar[data-orientation='horizontal'] {\n  min-height: 16px;\n  max-height: 16px;\n  min-width: 45px;\n  border-top: 1px solid #a0a0a0;\n}\n\n.p-ScrollBar[data-orientation='vertical'] {\n  min-width: 16px;\n  max-width: 16px;\n  min-height: 45px;\n  border-left: 1px solid #a0a0a0;\n}\n\n.p-ScrollBar-button {\n  background-color: #f0f0f0;\n  background-position: center center;\n  min-height: 15px;\n  max-height: 15px;\n  min-width: 15px;\n  max-width: 15px;\n}\n\n.p-ScrollBar-button:hover {\n  background-color: #dadada;\n}\n\n.p-ScrollBar-button.p-mod-active {\n  background-color: #cdcdcd;\n}\n\n.p-ScrollBar-track {\n  background: #f0f0f0;\n}\n\n.p-ScrollBar-thumb {\n  background: #cdcdcd;\n}\n\n.p-ScrollBar-thumb:hover {\n  background: #bababa;\n}\n\n.p-ScrollBar-thumb.p-mod-active {\n  background: #a0a0a0;\n}\n\n.p-ScrollBar[data-orientation='horizontal'] .p-ScrollBar-thumb {\n  height: 100%;\n  min-width: 15px;\n  border-left: 1px solid #a0a0a0;\n  border-right: 1px solid #a0a0a0;\n}\n\n.p-ScrollBar[data-orientation='vertical'] .p-ScrollBar-thumb {\n  width: 100%;\n  min-height: 15px;\n  border-top: 1px solid #a0a0a0;\n  border-bottom: 1px solid #a0a0a0;\n}\n\n.p-ScrollBar[data-orientation='horizontal']\n  .p-ScrollBar-button[data-action='decrement'] {\n  background-image: var(--jp-image-caretleft);\n}\n\n.p-ScrollBar[data-orientation='horizontal']\n  .p-ScrollBar-button[data-action='increment'] {\n  background-image: var(--jp-image-caretright);\n}\n\n.p-ScrollBar[data-orientation='vertical']\n  .p-ScrollBar-button[data-action='decrement'] {\n  background-image: var(--jp-image-caretup);\n}\n\n.p-ScrollBar[data-orientation='vertical']\n  .p-ScrollBar-button[data-action='increment'] {\n  background-image: var(--jp-image-caretdown);\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/*-----------------------------------------------------------------------------\n| Variables\n|----------------------------------------------------------------------------*/\n\n:root {\n  /* These need to be root because tabs get attached to the body during dragging. */\n  --jp-private-horizontal-tab-height: 24px;\n  --jp-private-horizontal-tab-width: 216px;\n  --jp-private-horizontal-tab-active-top-border: 2px;\n}\n\n/*-----------------------------------------------------------------------------\n| Tabs in the dock panel\n|----------------------------------------------------------------------------*/\n\n.p-DockPanel-tabBar {\n  border-bottom: var(--jp-border-width) solid var(--jp-border-color1);\n  overflow: visible;\n  color: var(--jp-ui-font-color1);\n  font-size: var(--jp-ui-font-size1);\n}\n\n.p-DockPanel-tabBar[data-orientation='horizontal'] {\n  min-height: calc(\n    var(--jp-private-horizontal-tab-height) + 2 * var(--jp-border-width)\n  );\n}\n\n.p-DockPanel-tabBar[data-orientation='vertical'] {\n  min-width: 80px;\n}\n\n.p-DockPanel-tabBar > .p-TabBar-content {\n  align-items: flex-end;\n  min-width: 0;\n  min-height: 0;\n}\n\n.p-DockPanel-tabBar .p-TabBar-tab {\n  flex: 0 1 var(--jp-private-horizontal-tab-width);\n  min-height: calc(\n    var(--jp-private-horizontal-tab-height) + var(--jp-border-width)\n  );\n  min-width: 0px;\n  margin-left: calc(-1 * var(--jp-border-width));\n  line-height: var(--jp-private-horizontal-tab-height);\n  padding: 0px 8px;\n  background: var(--jp-layout-color2);\n  border: var(--jp-border-width) solid var(--jp-border-color1);\n  border-bottom: none;\n  position: relative;\n}\n\n.p-DockPanel-tabBar .p-TabBar-tab:hover:not(.p-mod-current) {\n  background: var(--jp-layout-color1);\n}\n\n.p-DockPanel-tabBar .p-TabBar-tab:first-child {\n  margin-left: 0;\n}\n\n/* This is a current tab of a tab bar in the dock panel: each tab bar has 1. */\n.p-DockPanel-tabBar .p-TabBar-tab.p-mod-current {\n  background: var(--jp-layout-color1);\n  color: var(--jp-ui-font-color0);\n  min-height: calc(\n    var(--jp-private-horizontal-tab-height) + 2 * var(--jp-border-width)\n  );\n  transform: translateY(var(--jp-border-width));\n}\n\n/* This is the main application level current tab: only 1 exists. */\n.p-DockPanel-tabBar .p-TabBar-tab.jp-mod-current:before {\n  position: absolute;\n  top: calc(-1 * var(--jp-border-width));\n  left: calc(-1 * var(--jp-border-width));\n  content: '';\n  height: var(--jp-private-horizontal-tab-active-top-border);\n  width: calc(100% + 2 * var(--jp-border-width));\n  background: var(--jp-brand-color1);\n}\n\n/* This is the left tab bar current tab: only 1 exists. */\n.p-TabBar-tab.p-mod-current {\n  color: var(--jp-ui-font-color0);\n  background: var(--jp-layout-color1);\n}\n\n.p-TabBar-tab.p-mod-current:hover {\n  background: var(--jp-layout-color1);\n}\n\n.p-DockPanel-tabBar .p-TabBar.p-mod-left .p-TabBar-tab,\n.p-DockPanel-tabBar .p-TabBar.p-mod-right .p-TabBar-tab {\n  flex: 0 1 40px;\n  margin-top: -1px;\n  line-height: 40px;\n}\n\n.p-DockPanel-tabBar .p-TabBar.p-mod-left .p-TabBar-tab {\n  border-right: none;\n}\n\n.p-DockPanel-tabBar .p-TabBar.p-mod-right .p-TabBar-tab {\n  border-left: none;\n}\n\n.p-DockPanel-tabBar .p-TabBar.p-mod-left .p-TabBar-tab:first-child,\n.p-DockPanel-tabBar .p-TabBar.p-mod-right .p-TabBar-tab:first-child {\n  margin-top: 0;\n}\n\n.p-DockPanel-tabBar .p-TabBar.p-mod-left .p-TabBar-tab.p-mod-current,\n.p-DockPanel-tabBar .p-TabBar.p-mod-right .p-TabBar-tab.p-mod-current {\n  min-width: 80px;\n  max-width: 80px;\n}\n\n.p-DockPanel-tabBar .p-TabBar.p-mod-right .p-TabBar-tab.p-mod-current {\n  transform: translateX(-1px);\n}\n\n.p-DockPanel-tabBar .p-TabBar-tabIcon,\n.p-DockPanel-tabBar .p-TabBar-tabLabel,\n.p-DockPanel-tabBar .p-TabBar-tabCloseIcon {\n  display: inline-block;\n}\n\n.p-DockPanel-tabBar .p-TabBar-tab .p-TabBar-tabIcon,\n.p-TabBar-tab.p-mod-drag-image .p-TabBar-tabIcon {\n  width: 14px;\n  background-position: left center;\n  background-repeat: no-repeat;\n  background-size: 14px;\n  margin-right: 4px;\n}\n\n.p-DockPanel-tabBar .p-TabBar-tab.p-mod-current .p-TabBar-tabIcon {\n  margin-bottom: var(--jp-border-width);\n}\n\n.p-TabBar-WindowTabLabel {\n  margin-top: 4px;\n}\n\n.p-DockPanel-tabBar .p-TabBar-tab.p-mod-closable > .p-TabBar-tabCloseIcon {\n  margin-left: 4px;\n  padding-top: 8px;\n  background-size: 16px;\n  height: 16px;\n  width: 16px;\n  background-image: var(--jp-icon-close);\n  background-position: center;\n  background-repeat: no-repeat;\n}\n\n.p-DockPanel-tabBar\n  .p-TabBar-tab.p-mod-closable.jp-mod-dirty\n  > .p-TabBar-tabCloseIcon {\n  background-size: 10px;\n  background-image: var(--jp-icon-circle);\n}\n\n.p-DockPanel-tabBar\n  .p-TabBar-tab.p-mod-closable\n  > .p-TabBar-tabCloseIcon:hover {\n  background-size: 16px;\n  background-image: var(--jp-icon-close-circle);\n}\n\n.p-TabBar-tab.p-mod-drag-image {\n  background: var(--jp-layout-color1);\n  border: var(--jp-border-width) solid var(--jp-border-color1);\n  border-top: var(--jp-border-width) solid var(--jp-brand-color1);\n  box-shadow: var(--jp-elevation-z4);\n  color: var(--jp-ui-font-color1);\n  font-size: var(--jp-ui-font-size1);\n  line-height: var(--jp-private-horizontal-tab-height);\n  min-height: var(--jp-private-horizontal-tab-height);\n  min-width: var(--jp-private-horizontal-tab-width);\n  padding: 0px 10px;\n  transform: translateX(-40%) translateY(-58%);\n}\n\n/* .p-TabBar-tab.p-mod-drag-image .p-TabBar-tabLabel {\n  margin-left: 4px;\n} */\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Variables\n|----------------------------------------------------------------------------*/\n\n:root {\n  --jp-flat-button-height: 24px;\n  --jp-flat-button-padding: 8px 12px;\n}\n\n/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\nbutton {\n  border-radius: var(--jp-border-radius);\n}\n\nbutton.jp-mod-styled.jp-mod-accept {\n  background: var(--md-blue-500);\n  border: 0;\n  color: white;\n}\n\nbutton.jp-mod-styled.jp-mod-accept:hover {\n  background: var(--md-blue-600);\n}\n\nbutton.jp-mod-styled.jp-mod-accept:active {\n  background: var(--md-blue-700);\n}\n\nbutton.jp-mod-styled.jp-mod-reject {\n  background: var(--md-grey-500);\n  border: 0;\n  color: white;\n}\n\nbutton.jp-mod-styled.jp-mod-reject:hover {\n  background: var(--md-grey-600);\n}\n\nbutton.jp-mod-styled.jp-mod-reject:active {\n  background: var(--md-grey-700);\n}\n\nbutton.jp-mod-styled.jp-mod-warn {\n  background: var(--jp-error-color1);\n  border: 0;\n  color: white;\n}\n\nbutton.jp-mod-styled.jp-mod-warn:hover {\n  background: var(--md-red-600);\n}\n\nbutton.jp-mod-styled.jp-mod-warn:active {\n  background: var(--md-red-700);\n}\n\n.jp-Button-flat {\n  text-decoration: none;\n  padding: var(--jp-flat-button-padding);\n  color: var(--jp-warn-color1);\n  font-weight: 500;\n  background-color: transparent;\n  height: var(--jp-private-running-shutdown-button-height);\n  line-height: var(--jp-private-running-shutdown-button-height);\n  transition: background-color 0.1s ease;\n  border-radius: 2px;\n}\n\n.jp-Button-flat:hover {\n  background-color: rgba(153, 153, 153, 0.1);\n}\n\n.jp-Button-flat:focus {\n  border: none;\n  box-shadow: none;\n  background-color: rgba(153, 153, 153, 0.2);\n}\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/*-----------------------------------------------------------------------------\n| Variables\n|----------------------------------------------------------------------------*/\n\n:root {\n  --jp-private-sidebar-tab-width: 32px;\n}\n\n/*-----------------------------------------------------------------------------\n| SideBar\n|----------------------------------------------------------------------------*/\n\n.jp-SideBar {\n  /* This is needed so that all font sizing of children done in ems is\n   * relative to this base size */\n  font-size: var(--jp-ui-font-size1);\n}\n\n.jp-SideBar.p-TabBar {\n  color: var(--jp-ui-font-color1);\n  background: var(--jp-layout-color2);\n  font-size: var(--jp-ui-font-size1);\n  min-width: calc(var(--jp-private-sidebar-tab-width) + var(--jp-border-width));\n  max-width: calc(var(--jp-private-sidebar-tab-width) + var(--jp-border-width));\n  overflow: visible;\n  display: block;\n}\n\n.jp-SideBar .p-TabBar-content,\n.jp-SideBar .p-TabBar-content {\n  margin: 0;\n  padding: 0;\n  display: flex;\n  align-items: stretch;\n  list-style-type: none;\n  height: var(--jp-private-sidebar-tab-width);\n  transform-origin: 0 0 0;\n}\n\n.jp-SideBar .p-TabBar-tab {\n  padding: 0 16px;\n  border: none;\n  overflow: visible;\n}\n\n.jp-SideBar .p-TabBar-tab.p-mod-current {\n  min-height: calc(\n    var(--jp-private-sidebar-tab-width) + var(--jp-border-width)\n  );\n  max-height: calc(\n    var(--jp-private-sidebar-tab-width) + var(--jp-border-width)\n  );\n  /* transform: translateY(var(--jp-border-width)); */\n}\n\n.jp-SideBar .p-TabBar-tab:not(.p-mod-current) {\n  background: var(--jp-layout-color2);\n}\n\n.jp-SideBar .p-TabBar-tabIcon.jp-SideBar-tabIcon {\n  min-width: 20px;\n  min-height: 20px;\n  background-size: 20px;\n  display: inline-block;\n  vertical-align: middle;\n  background-repeat: no-repeat;\n  background-position: center;\n}\n\n.jp-SideBar .p-TabBar-tabLabel {\n  line-height: var(--jp-private-sidebar-tab-width);\n}\n\n.jp-SideBar .p-TabBar-tab:hover:not(.p-mod-current) {\n  background: var(--jp-layout-color1);\n}\n\n/* Left */\n\n/* Borders */\n\n.jp-SideBar.p-TabBar.jp-mod-left {\n  border-right: var(--jp-border-width) solid var(--jp-border-color0);\n}\n\n.jp-SideBar.p-TabBar.jp-mod-left .p-TabBar-tab + .p-TabBar-tab {\n  border-right: var(--jp-border-width) solid var(--jp-layout-color2);\n}\n\n.jp-SideBar.p-TabBar.jp-mod-left .p-TabBar-tab.p-mod-current + .p-TabBar-tab {\n  border-right: var(--jp-border-width) solid var(--jp-border-color0);\n}\n\n.jp-SideBar.p-TabBar.jp-mod-left .p-TabBar-tab + .p-TabBar-tab.p-mod-current {\n  border-right: var(--jp-border-width) solid var(--jp-border-color0);\n}\n\n.jp-SideBar.p-TabBar.jp-mod-left .p-TabBar-tab.p-mod-current:last-child {\n  border-left: var(--jp-border-width) solid var(--jp-border-color0);\n}\n\n/* Transforms */\n\n.jp-SideBar.p-TabBar.jp-mod-left .p-TabBar-content {\n  flex-direction: row-reverse;\n  transform: rotate(-90deg) translateX(-100%);\n}\n\n.jp-SideBar.p-TabBar.jp-mod-left\n  .p-TabBar-tab:not(.p-mod-current)\n  .p-TabBar-tabIcon {\n  transform: rotate(90deg);\n}\n\n.jp-SideBar.p-TabBar.jp-mod-left .p-TabBar-tab.p-mod-current .p-TabBar-tabIcon {\n  transform: rotate(90deg)\n    translate(\n      calc(-0.5 * var(--jp-border-width)),\n      calc(-0.5 * var(--jp-border-width))\n    );\n}\n\n/* Right */\n\n/* Borders */\n\n.jp-SideBar.p-TabBar.jp-mod-right {\n  border-left: var(--jp-border-width) solid var(--jp-border-color0);\n}\n\n.jp-SideBar.p-TabBar.jp-mod-right .p-TabBar-tab + .p-TabBar-tab {\n  border-left: var(--jp-border-width) solid var(--jp-layout-color2);\n}\n\n.jp-SideBar.p-TabBar.jp-mod-right .p-TabBar-tab.p-mod-current + .p-TabBar-tab {\n  border-left: var(--jp-border-width) solid var(--jp-border-color0);\n}\n\n.jp-SideBar.p-TabBar.jp-mod-right .p-TabBar-tab + .p-TabBar-tab.p-mod-current {\n  border-left: var(--jp-border-width) solid var(--jp-border-color0);\n}\n\n.jp-SideBar.p-TabBar.jp-mod-right .p-TabBar-tab.p-mod-current:last-child {\n  border-right: var(--jp-border-width) solid var(--jp-border-color0);\n}\n\n/* Transforms */\n\n.jp-SideBar.p-TabBar.jp-mod-right .p-TabBar-content {\n  flex-direction: row;\n  transform: rotate(90deg) translateY(-100%);\n}\n\n.jp-SideBar.p-TabBar.jp-mod-right\n  .p-TabBar-tab:not(.p-mod-current)\n  .p-TabBar-tabIcon {\n  transform: rotate(-90deg);\n}\n\n.jp-SideBar.p-TabBar.jp-mod-right\n  .p-TabBar-tab.p-mod-current\n  .p-TabBar-tabIcon {\n  transform: rotate(-90deg)\n    translate(\n      calc(0.5 * var(--jp-border-width)),\n      calc(-0.5 * var(--jp-border-width))\n    );\n}\n\n/* Stack panels */\n\n#jp-left-stack > .p-Widget,\n#jp-right-stack > .p-Widget {\n  min-width: var(--jp-sidebar-min-width);\n}\n\n#jp-right-stack {\n  border-left: var(--jp-border-width) solid var(--jp-border-color1);\n}\n\n#jp-left-stack {\n  border-right: var(--jp-border-width) solid var(--jp-border-color1);\n}\n",""])},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(106),""),e.i(n(173),""),e.i(n(285),""),e.i(n(1023),""),e.push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/* This file was auto-generated by ensurePackage() in @jupyterlab/buildutils */\n",""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/*-----------------------------------------------------------------------------\n| RenderedText\n|----------------------------------------------------------------------------*/\n\n.jp-RenderedText {\n  text-align: left;\n  padding-left: var(--jp-code-padding);\n  line-height: var(--jp-code-line-height);\n  font-family: var(--jp-code-font-family);\n}\n\n.jp-RenderedText pre,\n.jp-RenderedJavaScript pre,\n.jp-RenderedHTMLCommon pre {\n  color: var(--jp-content-font-color1);\n  font-size: var(--jp-code-font-size);\n  border: none;\n  margin: 0px;\n  padding: 0px;\n}\n\n/* console foregrounds and backgrounds */\n.jp-RenderedText pre .ansi-black-fg {\n  color: #3e424d;\n}\n.jp-RenderedText pre .ansi-red-fg {\n  color: #e75c58;\n}\n.jp-RenderedText pre .ansi-green-fg {\n  color: #00a250;\n}\n.jp-RenderedText pre .ansi-yellow-fg {\n  color: #ddb62b;\n}\n.jp-RenderedText pre .ansi-blue-fg {\n  color: #208ffb;\n}\n.jp-RenderedText pre .ansi-magenta-fg {\n  color: #d160c4;\n}\n.jp-RenderedText pre .ansi-cyan-fg {\n  color: #60c6c8;\n}\n.jp-RenderedText pre .ansi-white-fg {\n  color: #c5c1b4;\n}\n\n.jp-RenderedText pre .ansi-black-bg {\n  background-color: #3e424d;\n}\n.jp-RenderedText pre .ansi-red-bg {\n  background-color: #e75c58;\n}\n.jp-RenderedText pre .ansi-green-bg {\n  background-color: #00a250;\n}\n.jp-RenderedText pre .ansi-yellow-bg {\n  background-color: #ddb62b;\n}\n.jp-RenderedText pre .ansi-blue-bg {\n  background-color: #208ffb;\n}\n.jp-RenderedText pre .ansi-magenta-bg {\n  background-color: #d160c4;\n}\n.jp-RenderedText pre .ansi-cyan-bg {\n  background-color: #60c6c8;\n}\n.jp-RenderedText pre .ansi-white-bg {\n  background-color: #c5c1b4;\n}\n\n.jp-RenderedText pre .ansi-black-intense-fg {\n  color: #282c36;\n}\n.jp-RenderedText pre .ansi-red-intense-fg {\n  color: #b22b31;\n}\n.jp-RenderedText pre .ansi-green-intense-fg {\n  color: #007427;\n}\n.jp-RenderedText pre .ansi-yellow-intense-fg {\n  color: #b27d12;\n}\n.jp-RenderedText pre .ansi-blue-intense-fg {\n  color: #0065ca;\n}\n.jp-RenderedText pre .ansi-magenta-intense-fg {\n  color: #a03196;\n}\n.jp-RenderedText pre .ansi-cyan-intense-fg {\n  color: #258f8f;\n}\n.jp-RenderedText pre .ansi-white-intense-fg {\n  color: #a1a6b2;\n}\n\n.jp-RenderedText pre .ansi-black-intense-bg {\n  background-color: #282c36;\n}\n.jp-RenderedText pre .ansi-red-intense-bg {\n  background-color: #b22b31;\n}\n.jp-RenderedText pre .ansi-green-intense-bg {\n  background-color: #007427;\n}\n.jp-RenderedText pre .ansi-yellow-intense-bg {\n  background-color: #b27d12;\n}\n.jp-RenderedText pre .ansi-blue-intense-bg {\n  background-color: #0065ca;\n}\n.jp-RenderedText pre .ansi-magenta-intense-bg {\n  background-color: #a03196;\n}\n.jp-RenderedText pre .ansi-cyan-intense-bg {\n  background-color: #258f8f;\n}\n.jp-RenderedText pre .ansi-white-intense-bg {\n  background-color: #a1a6b2;\n}\n\n.jp-RenderedText pre .ansi-default-inverse-fg {\n  color: var(--jp-ui-inverse-font-color0);\n}\n.jp-RenderedText pre .ansi-default-inverse-bg {\n  background-color: var(--jp-inverse-layout-color0);\n}\n\n.jp-RenderedText pre .ansi-bold {\n  font-weight: bold;\n}\n.jp-RenderedText pre .ansi-underline {\n  text-decoration: underline;\n}\n\n.jp-RenderedText[data-mime-type='application/vnd.jupyter.stderr'] {\n  background: var(--jp-rendermime-error-background);\n  padding-top: var(--jp-code-padding);\n}\n\n/*-----------------------------------------------------------------------------\n| RenderedLatex\n|----------------------------------------------------------------------------*/\n\n.jp-RenderedLatex {\n  color: var(--jp-content-font-color1);\n  font-size: var(--jp-content-font-size1);\n  line-height: var(--jp-content-line-height);\n}\n\n/* Left-justify outputs.*/\n.jp-OutputArea-output.jp-RenderedLatex {\n  padding: var(--jp-code-padding);\n  text-align: left;\n}\n\n/*-----------------------------------------------------------------------------\n| RenderedHTML\n|----------------------------------------------------------------------------*/\n\n.jp-RenderedHTMLCommon {\n  color: var(--jp-content-font-color1);\n  font-family: var(--jp-content-font-family);\n  font-size: var(--jp-content-font-size1);\n  line-height: var(--jp-content-line-height);\n  /* Give a bit more R padding on Markdown text to keep line lengths reasonable */\n  padding-right: 20px;\n}\n\n.jp-RenderedHTMLCommon em {\n  font-style: italic;\n}\n\n.jp-RenderedHTMLCommon strong {\n  font-weight: bold;\n}\n\n.jp-RenderedHTMLCommon u {\n  text-decoration: underline;\n}\n\n.jp-RenderedHTMLCommon a:link {\n  text-decoration: none;\n  color: var(--jp-content-link-color);\n}\n\n.jp-RenderedHTMLCommon a:hover {\n  text-decoration: underline;\n  color: var(--jp-content-link-color);\n}\n\n.jp-RenderedHTMLCommon a:visited {\n  text-decoration: none;\n  color: var(--jp-content-link-color);\n}\n\n/* Headings */\n\n.jp-RenderedHTMLCommon h1,\n.jp-RenderedHTMLCommon h2,\n.jp-RenderedHTMLCommon h3,\n.jp-RenderedHTMLCommon h4,\n.jp-RenderedHTMLCommon h5,\n.jp-RenderedHTMLCommon h6 {\n  line-height: var(--jp-content-heading-line-height);\n  font-weight: var(--jp-content-heading-font-weight);\n  font-style: normal;\n  margin: var(--jp-content-heading-margin-top) 0\n    var(--jp-content-heading-margin-bottom) 0;\n}\n\n.jp-RenderedHTMLCommon h1:first-child,\n.jp-RenderedHTMLCommon h2:first-child,\n.jp-RenderedHTMLCommon h3:first-child,\n.jp-RenderedHTMLCommon h4:first-child,\n.jp-RenderedHTMLCommon h5:first-child,\n.jp-RenderedHTMLCommon h6:first-child {\n  margin-top: calc(0.5 * var(--jp-content-heading-margin-top));\n}\n\n.jp-RenderedHTMLCommon h1:last-child,\n.jp-RenderedHTMLCommon h2:last-child,\n.jp-RenderedHTMLCommon h3:last-child,\n.jp-RenderedHTMLCommon h4:last-child,\n.jp-RenderedHTMLCommon h5:last-child,\n.jp-RenderedHTMLCommon h6:last-child {\n  margin-bottom: calc(0.5 * var(--jp-content-heading-margin-bottom));\n}\n\n.jp-RenderedHTMLCommon h1 {\n  font-size: var(--jp-content-font-size5);\n}\n\n.jp-RenderedHTMLCommon h2 {\n  font-size: var(--jp-content-font-size4);\n}\n\n.jp-RenderedHTMLCommon h3 {\n  font-size: var(--jp-content-font-size3);\n}\n\n.jp-RenderedHTMLCommon h4 {\n  font-size: var(--jp-content-font-size2);\n}\n\n.jp-RenderedHTMLCommon h5 {\n  font-size: var(--jp-content-font-size1);\n}\n\n.jp-RenderedHTMLCommon h6 {\n  font-size: var(--jp-content-font-size0);\n}\n\n/* Lists */\n\n.jp-RenderedHTMLCommon ul:not(.list-inline),\n.jp-RenderedHTMLCommon ol:not(.list-inline) {\n  padding-left: 2em;\n}\n\n.jp-RenderedHTMLCommon ul {\n  list-style: disc;\n}\n\n.jp-RenderedHTMLCommon ul ul {\n  list-style: square;\n}\n\n.jp-RenderedHTMLCommon ul ul ul {\n  list-style: circle;\n}\n\n.jp-RenderedHTMLCommon ol {\n  list-style: decimal;\n}\n\n.jp-RenderedHTMLCommon ol ol {\n  list-style: upper-alpha;\n}\n\n.jp-RenderedHTMLCommon ol ol ol {\n  list-style: lower-alpha;\n}\n\n.jp-RenderedHTMLCommon ol ol ol ol {\n  list-style: lower-roman;\n}\n\n.jp-RenderedHTMLCommon ol ol ol ol ol {\n  list-style: decimal;\n}\n\n.jp-RenderedHTMLCommon ol,\n.jp-RenderedHTMLCommon ul {\n  margin-bottom: 1em;\n}\n\n.jp-RenderedHTMLCommon ul ul,\n.jp-RenderedHTMLCommon ul ol,\n.jp-RenderedHTMLCommon ol ul,\n.jp-RenderedHTMLCommon ol ol {\n  margin-bottom: 0em;\n}\n\n.jp-RenderedHTMLCommon hr {\n  color: var(--jp-border-color2);\n  background-color: var(--jp-border-color1);\n  margin-top: 1em;\n  margin-bottom: 1em;\n}\n\n.jp-RenderedHTMLCommon > pre {\n  margin: 1.5em 2em;\n}\n\n.jp-RenderedHTMLCommon pre,\n.jp-RenderedHTMLCommon code {\n  border: 0;\n  background-color: var(--jp-layout-color0);\n  color: var(--jp-content-font-color1);\n  font-family: var(--jp-code-font-family);\n  font-size: inherit;\n  line-height: var(--jp-code-line-height);\n  padding: 0;\n  white-space: pre-wrap;\n}\n\n.jp-RenderedHTMLCommon :not(pre) > code {\n  background-color: var(--jp-layout-color2);\n  padding: 1px 5px;\n}\n\n/* Tables */\n\n.jp-RenderedHTMLCommon table {\n  border-collapse: collapse;\n  border-spacing: 0;\n  border: none;\n  color: var(--jp-ui-font-color1);\n  font-size: 12px;\n  table-layout: fixed;\n  margin-left: auto;\n  margin-right: auto;\n}\n\n.jp-RenderedHTMLCommon thead {\n  border-bottom: var(--jp-border-width) solid var(--jp-border-color1);\n  vertical-align: bottom;\n}\n\n.jp-RenderedHTMLCommon td,\n.jp-RenderedHTMLCommon th,\n.jp-RenderedHTMLCommon tr {\n  vertical-align: middle;\n  padding: 0.5em 0.5em;\n  line-height: normal;\n  white-space: normal;\n  max-width: none;\n  border: none;\n}\n\n.jp-RenderedMarkdown.jp-RenderedHTMLCommon td,\n.jp-RenderedMarkdown.jp-RenderedHTMLCommon th {\n  max-width: none;\n}\n\n:not(.jp-RenderedMarkdown).jp-RenderedHTMLCommon td,\n:not(.jp-RenderedMarkdown).jp-RenderedHTMLCommon th,\n:not(.jp-RenderedMarkdown).jp-RenderedHTMLCommon tr {\n  text-align: right;\n}\n\n.jp-RenderedHTMLCommon th {\n  font-weight: bold;\n}\n\n.jp-RenderedHTMLCommon tbody tr:nth-child(odd) {\n  background: var(--jp-layout-color0);\n}\n\n.jp-RenderedHTMLCommon tbody tr:nth-child(even) {\n  background: var(--jp-rendermime-table-row-background);\n}\n\n.jp-RenderedHTMLCommon tbody tr:hover {\n  background: var(--jp-rendermime-table-row-hover-background);\n}\n\n.jp-RenderedHTMLCommon table {\n  margin-bottom: 1em;\n}\n\n.jp-RenderedHTMLCommon p {\n  text-align: left;\n  margin: 0px;\n}\n\n.jp-RenderedHTMLCommon p {\n  margin-bottom: 1em;\n}\n\n.jp-RenderedHTMLCommon img {\n  -moz-force-broken-image-icon: 1;\n}\n\n/* Restrict to direct children as other images could be nested in other content. */\n.jp-RenderedHTMLCommon > img {\n  display: block;\n  margin-left: auto;\n  margin-right: auto;\n  margin-bottom: 1em;\n}\n\n/* Change color behind transparent images if they need it... */\n[data-jp-theme-light='false'] .jp-RenderedImage img.jp-needs-light-background {\n  background-color: var(--jp-inverse-layout-color1);\n}\n[data-jp-theme-light='true'] .jp-RenderedImage img.jp-needs-dark-background {\n  background-color: var(--jp-inverse-layout-color1);\n}\n/* ...or leave it untouched if they don't */\n[data-jp-theme-light='false'] .jp-RenderedImage img.jp-needs-dark-background {\n}\n[data-jp-theme-light='true'] .jp-RenderedImage img.jp-needs-light-background {\n}\n\n.jp-RenderedHTMLCommon img,\n.jp-RenderedImage img,\n.jp-RenderedHTMLCommon svg,\n.jp-RenderedSVG svg {\n  max-width: 100%;\n  height: auto;\n}\n\n.jp-RenderedHTMLCommon img.jp-mod-unconfined,\n.jp-RenderedImage img.jp-mod-unconfined,\n.jp-RenderedHTMLCommon svg.jp-mod-unconfined,\n.jp-RenderedSVG svg.jp-mod-unconfined {\n  max-width: none;\n}\n\n.jp-RenderedHTMLCommon .alert {\n  padding: var(--jp-notebook-padding);\n  border: var(--jp-border-width) solid transparent;\n  border-radius: var(--jp-border-radius);\n  margin-bottom: 1em;\n}\n\n.jp-RenderedHTMLCommon .alert-info {\n  color: var(--jp-info-color0);\n  background-color: var(--jp-info-color3);\n  border-color: var(--jp-info-color2);\n}\n.jp-RenderedHTMLCommon .alert-info hr {\n  border-color: var(--jp-info-color3);\n}\n.jp-RenderedHTMLCommon .alert-info > p:last-child,\n.jp-RenderedHTMLCommon .alert-info > ul:last-child {\n  margin-bottom: 0;\n}\n\n.jp-RenderedHTMLCommon .alert-warning {\n  color: var(--jp-warn-color0);\n  background-color: var(--jp-warn-color3);\n  border-color: var(--jp-warn-color2);\n}\n.jp-RenderedHTMLCommon .alert-warning hr {\n  border-color: var(--jp-warn-color3);\n}\n.jp-RenderedHTMLCommon .alert-warning > p:last-child,\n.jp-RenderedHTMLCommon .alert-warning > ul:last-child {\n  margin-bottom: 0;\n}\n\n.jp-RenderedHTMLCommon .alert-success {\n  color: var(--jp-success-color0);\n  background-color: var(--jp-success-color3);\n  border-color: var(--jp-success-color2);\n}\n.jp-RenderedHTMLCommon .alert-success hr {\n  border-color: var(--jp-success-color3);\n}\n.jp-RenderedHTMLCommon .alert-success > p:last-child,\n.jp-RenderedHTMLCommon .alert-success > ul:last-child {\n  margin-bottom: 0;\n}\n\n.jp-RenderedHTMLCommon .alert-danger {\n  color: var(--jp-error-color0);\n  background-color: var(--jp-error-color3);\n  border-color: var(--jp-error-color2);\n}\n.jp-RenderedHTMLCommon .alert-danger hr {\n  border-color: var(--jp-error-color3);\n}\n.jp-RenderedHTMLCommon .alert-danger > p:last-child,\n.jp-RenderedHTMLCommon .alert-danger > ul:last-child {\n  margin-bottom: 0;\n}\n\n.jp-RenderedHTMLCommon blockquote {\n  margin: 1em 2em;\n  padding: 0 1em;\n  border-left: 5px solid var(--jp-border-color2);\n}\n\na.jp-InternalAnchorLink {\n  visibility: hidden;\n  margin-left: 8px;\n  color: var(--md-blue-800);\n}\n\nh1:hover .jp-InternalAnchorLink,\nh2:hover .jp-InternalAnchorLink,\nh3:hover .jp-InternalAnchorLink,\nh4:hover .jp-InternalAnchorLink,\nh5:hover .jp-InternalAnchorLink,\nh6:hover .jp-InternalAnchorLink {\n  visibility: visible;\n}\n\n.jp-RenderedHTMLCommon kbd {\n  background-color: var(--jp-rendermime-table-row-background);\n  border: 1px solid var(--jp-border-color0);\n  border-bottom-color: var(--jp-border-color2);\n  border-radius: 3px;\n  box-shadow: inset 0 -1px 0 rgba(0, 0, 0, 0.25);\n  display: inline-block;\n  font-size: 0.8em;\n  line-height: 1em;\n  padding: 0.2em 0.5em;\n}\n\n/* Most direct children of .jp-RenderedHTMLCommon have a margin-bottom of 1.0.\n * At the bottom of cells this is a bit too much as there is also spacing\n * between cells. Going all the way to 0 gets too tight between markdown and\n * code cells.\n */\n.jp-RenderedHTMLCommon > *:last-child {\n  margin-bottom: 0.5em;\n}\n",""])},function(t,e){t.exports=function(t){var e="undefined"!=typeof window&&window.location;if(!e)throw new Error("fixUrls requires window.location");if(!t||"string"!=typeof t)return t;var n=e.protocol+"//"+e.host,r=n+e.pathname.replace(/\/[^\/]*$/,"/");return t.replace(/url\s*\(((?:[^)(]|\((?:[^)(]+|\([^)(]*\))*\))*)\)/gi,function(t,e){var i,o=e.trim().replace(/^"(.*)"$/,function(t,e){return e}).replace(/^'(.*)'$/,function(t,e){return e});return/^(#|data:|http:\/\/|https:\/\/|file:\/\/\/|\s*$)/i.test(o)?t:(i=0===o.indexOf("//")?o:0===o.indexOf("/")?n+o:r+o.replace(/^\.\//,""),"url("+JSON.stringify(i)+")")})}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(6),i=n(10),o=n(126),a=n(72),s=n(60),A=n(49),l=n(189),c=n(715),u=n(17);var g=o.RenderMimeRegistry.UrlResolver;const h=n(1163),d=n(1164),f=n(310),p=n(9),I=n(48);n(234);const C=n(310),m=n(1290),y=n(1326),E=n(66),v=n(1329);e.MainApp=class extends r.Widget{constructor(){super(),this.notebookFactory=new c.NotebookModelFactory({}),this.isEmbedded=!1,this.addClass("m-Viewer"),this.params=new f.UrlParametersManager({filterParameter:(t,e)=>"embed"===t&&""!==e&&(this.isEmbedded=!0)}),this.params.fetchParameters(),this.conn_opts=s.ServerConnection.makeSettings(),this.sessionManager=new s.SessionManager({serverSettings:this.conn_opts});const t=this.session=new A.ClientSession({manager:this.sessionManager});this.factory=new I.PartFactory,m.default.map(t=>t.activate(this,this.factory)),y.default.activate(this,this.factory),this.layout=new r.BoxLayout({spacing:0});const e=this.mimeTypeService=new a.CodeMirrorMimeTypeService,n=this.contents=new s.ContentsManager({serverSettings:this.conn_opts}),i=this.rendermime=new o.RenderMimeRegistry({resolver:new g({contents:n,session:t}),latexTypesetter:new d.MathJaxTypesetter({url:"https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.5/MathJax.js",config:"TeX-AMS-MML_HTMLorMML-full,Safe"}),initialFactories:o.standardRendererFactories}),l=this.dataset.display=this.isEmbedded?"compressed":"expanded";this.content=new c.Notebook({mimeTypeService:e,rendermime:i}),this.outputPanel=new r.BoxPanel;const u=new r.Widget;(u.layout=new r.BoxLayout).addWidget(this.outputPanel),this.outputPanel.addClass("m-OutputPanel"),u.addClass("m-OutputPanel-Wrapper"),this.toolbar=new h.StatusToolbar({clientSession:this.session,showOverlay:"compressed"===l}),r.BoxLayout.setSizeBasis(this.toolbar,32),r.BoxLayout.setStretch(this.toolbar,0),r.BoxLayout.setStretch(u,1),this.layout.addWidget(this.toolbar),this.layout.addWidget(u),"compressed"===l&&this.toolbar.hide();const p=E.typeEditorFactoryPlugin.activate(this);E.defaultTypeEditors.activate(this,p);const B=v.DisplayHandleManager.GetManager(t),b=new v.SyncMetadata(t,this.factory,B),Q=v.registerUDPs(this.factory,t.path);this.ready=Promise.all([b.ready,Q]).then(()=>void 0),i.addFactory({safe:!1,mimeTypes:[C.DashboardSerializer.MAVEN_LAYOUT_MIME_TYPE],defaultRank:75,createRenderer:()=>new v.RenderedDashboard({factory:this.factory,rendermime:i,session:t,ready:Promise.all([b.ready,Q]).then(()=>void 0)})}),this.disposed.connect(()=>{i.removeMimeType(C.DashboardSerializer.MAVEN_LAYOUT_MIME_TYPE),b.dispose()}),this.loadFromUrl()}async loadFromUrl(){let t=this.getPathFromUrl();const e=await this.loadNotebookModel(t);null!=this.notebookModel&&this.notebookModel.dispose();const r=null==e.metadata.language_info?void 0:e.metadata.language_info.name;return this.notebookModel=this.notebookFactory.createNew(r),this.notebookModel.fromJSON(e),this.notebookModel.dirty=!1,this.notebookModel.initialize(),await Promise.resolve().then(()=>n(1337)).then(t=>t.registerWidgets(this,{session:this.session,saveState:new i.Signal(this),model:this.notebookModel},this.rendermime)),this.executeNotebook()}getPathFromUrl(){let t=window.location.pathname;return-1===t.indexOf("/view")?(console.log("defaulting to fallback"),t="/demos/MavenGlobals"):t=t.slice(t.indexOf("/view")+5),decodeURI(t)}dispose(){this.isDisposed||(null!=this.notebookModel&&this.notebookModel.dispose(),this.notebookFactory.dispose(),this.content.dispose(),super.dispose())}async loadNotebookModel(t){const e=await this.contents.get(t);if("notebook"===e.type)return e.content;const n="json"===e.format?e.content:JSON.parse(e.content),r={cells:[],metadata:{kernelspec:{name:"python3",language:"python",display_name:"Python 3"},orig_nbformat:u.nbformat.MAJOR_VERSION,globals:n.globals||{}},nbformat:u.nbformat.MAJOR_VERSION,nbformat_minor:u.nbformat.MINOR_VERSION};if(null!=this.sessionManager.specs){const t=this.sessionManager.specs.default,e=this.sessionManager.specs.kernelspecs[t];r.metadata.kernelspec=e}const i={cell_type:"code",execution_count:null,source:(n.init||[]).join("\n"),metadata:{},outputs:[]},o={cell_type:"code",execution_count:null,source:v.RenderedDashboard.getPythonCode(n),metadata:{showinviewer:"true"},outputs:[{output_type:"execute_result",execution_count:null,data:{[C.DashboardSerializer.MAVEN_LAYOUT_MIME_TYPE]:C.DashboardSerializer.DEFAULT_DASHBOARD},metadata:{}}]};return r.cells.push(i,o),r}executeNotebook(){if(performance.mark("beginExecute"),null==this.notebookModel)return;let t=this.getPathFromUrl();this.session.setPath(t+"/viewer-"+p.UUID.uuid4()),this.session.kernelPreference={...this.notebookModel.metadata.get("kernelspec")},this.session.initialize().then(async()=>{performance.mark("sessionInitialized"),performance.measure("Initializing Session","beginExecute","sessionInitialized");const t=this.session.kernel;t.ready.then(()=>{performance.mark("kernelReady"),performance.measure("Kernel Initializing","sessionInitialized","kernelReady")}),await this.ready,t.getSpec().then(t=>this.toolbar.setKernelLanguage(t.display_name)),this.content.model=this.notebookModel;const e=this.notebookModel.cells;console.log("Executing",e.length,"cells"),this.toolbar.setKernelStatus(1);for(let t=0;t<e.length;t++){performance.mark("beginExecuteCell");const e=this.content.widgets[t];if(e.hasClass("jp-CodeCell")){const t=await l.CodeCell.execute(e,this.session);if(t&&"error"===t.content.status)throw Error(t.content.traceback.join("\n"))}if(performance.mark("endExecuteCell"),performance.measure("Executing Cell","beginExecuteCell","endExecuteCell"),"true"===e.model.metadata.get("showinviewer")&&e instanceof l.CodeCell){const t=e.outputArea.widgets[0];if(null==t||t.widgets.length<2)continue;this.outputPanel.addWidget(t.widgets[1])}}performance.mark("runAllCellsComplete"),performance.measure("Executing Notebook","beginExecute","runAllCellsComplete")}).then(()=>{this.toolbar.setKernelStatus(0);for(const t of this.outputPanel.widgets)t instanceof v.RenderedDashboard&&this.params.applyParameters(t.dashboard.globals);this.params.paramsDidChange.subscribe(()=>{for(const t of this.outputPanel.widgets)t instanceof v.RenderedDashboard&&this.params.applyParameters(t.dashboard.globals)})}).catch(t=>{console.error("Failed to run cells"),console.error(t),this.toolbar.setKernelStatus(2)});for(let t=0;t<this.notebookModel.cells.length;t++){const e=this.notebookModel.cells.get(t);null!=e&&e instanceof l.CodeCellModel&&(e.outputs.clear(),e.trusted=!0)}}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){t.copyText=function(t){var e=document.body,n=function(r){r.preventDefault(),r.stopPropagation(),r.clipboardData.setData("text",t),e.removeEventListener("copy",n,!0)};e.addEventListener("copy",n,!0),document.execCommand("copy")}}(e.ClipboardExt||(e.ClipboardExt={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){t.boxSizing=function(t){var e=window.getComputedStyle(t),n=parseFloat(e.borderTopWidth)||0,r=parseFloat(e.borderLeftWidth)||0,i=parseFloat(e.borderRightWidth)||0,o=parseFloat(e.borderBottomWidth)||0,a=parseFloat(e.paddingTop)||0,s=parseFloat(e.paddingLeft)||0,A=parseFloat(e.paddingRight)||0,l=parseFloat(e.paddingBottom)||0;return{borderTop:n,borderLeft:r,borderRight:i,borderBottom:o,paddingTop:a,paddingLeft:s,paddingRight:A,paddingBottom:l,horizontalSum:r+s+A+i,verticalSum:n+a+l+o}},t.sizeLimits=function(t){var e=window.getComputedStyle(t),n=parseFloat(e.minWidth)||0,r=parseFloat(e.minHeight)||0,i=parseFloat(e.maxWidth)||1/0,o=parseFloat(e.maxHeight)||1/0;return{minWidth:n,minHeight:r,maxWidth:i=Math.max(n,i),maxHeight:o=Math.max(r,o)}},t.hitTest=function(t,e,n){var r=t.getBoundingClientRect();return e>=r.left&&e<r.right&&n>=r.top&&n<r.bottom},t.scrollIntoViewIfNeeded=function(t,e){var n=t.getBoundingClientRect(),r=e.getBoundingClientRect();r.top<=n.top&&r.bottom>=n.bottom||(r.top<n.top&&r.height<=n.height?t.scrollTop-=n.top-r.top:r.bottom>n.bottom&&r.height>=n.height?t.scrollTop-=n.top-r.top:r.top<n.top&&r.height>n.height?t.scrollTop-=n.bottom-r.bottom:r.bottom>n.bottom&&r.height<n.height&&(t.scrollTop-=n.bottom-r.bottom))}}(e.ElementExt||(e.ElementExt={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){t.IS_MAC=!!navigator.platform.match(/Mac/i),t.IS_WIN=!!navigator.platform.match(/Win/i),t.IS_IE=/Trident/.test(navigator.userAgent),t.IS_EDGE=/Edge/.test(navigator.userAgent),t.accelKey=function(e){return t.IS_MAC?e.metaKey:e.ctrlKey}}(e.Platform||(e.Platform={}))},function(t,e,n){"use strict";var r;Object.defineProperty(e,"__esModule",{value:!0}),function(t){t.calculateSpecificity=function(t){if(t in r.specificityCache)return r.specificityCache[t];var e=r.calculateSingle(t);return r.specificityCache[t]=e},t.isValid=function(t){if(t in r.validityCache)return r.validityCache[t];var e=!0;try{r.testElem.querySelector(t)}catch(t){e=!1}return r.validityCache[t]=e},t.matches=function(t,e){return r.protoMatchFunc.call(t,e)}}(e.Selector||(e.Selector={})),function(t){var e;t.specificityCache=Object.create(null),t.validityCache=Object.create(null),t.testElem=document.createElement("div"),t.protoMatchFunc=(e=Element.prototype).matches||e.matchesSelector||e.mozMatchesSelector||e.msMatchesSelector||e.oMatchesSelector||e.webkitMatchesSelector||function(t){var e=this.ownerDocument.querySelectorAll(t);return-1!==Array.prototype.indexOf.call(e,this)},t.calculateSingle=function(t){var e=0,c=0,u=0;function g(e){var n=t.match(e);return null!==n&&(t=t.slice(n[0].length),!0)}for(t=(t=t.split(",",1)[0]).replace(l," $1 ");t.length>0;)if(g(n))e++;else if(g(r))c++;else if(g(i))c++;else if(g(a))u++;else if(g(s))c++;else if(g(o))u++;else if(!g(A))return 0;return(e=Math.min(e,255))<<16|(c=Math.min(c,255))<<8|(u=Math.min(u,255))};var n=/^#[^\s\+>~#\.\[:]+/,r=/^\.[^\s\+>~#\.\[:]+/,i=/^\[[^\]]+\]/,o=/^[^\s\+>~#\.\[:]+/,a=/^(::[^\s\+>~#\.\[:]+|:first-line|:first-letter|:before|:after)/,s=/^:[^\s\+>~#\.\[:]+/,A=/^[\s\+>~\*]+/,l=/:not\(([^\)]+)\)/g}(r||(r={}))},function(t,e,n){"use strict";function r(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}Object.defineProperty(e,"__esModule",{value:!0}),r(n(1031)),r(n(1032))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r,i=n(5),o=function(){function t(t){this._root=new r.LeafNode,this.cmp=t}return Object.defineProperty(t.prototype,"isEmpty",{get:function(){return 0===this._root.size},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"size",{get:function(){return this._root.size},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"first",{get:function(){var t=r.firstLeaf(this._root);return t.size>0?t.items[0]:void 0},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"last",{get:function(){var t=r.lastLeaf(this._root);return t.size>0?t.items[t.size-1]:void 0},enumerable:!0,configurable:!0}),t.prototype.iter=function(){return r.iterItems(this._root)},t.prototype.retro=function(){return r.retroItems(this._root)},t.prototype.slice=function(t,e){return r.sliceItems(this._root,t,e)},t.prototype.retroSlice=function(t,e){return r.retroSliceItems(this._root,t,e)},t.prototype.at=function(t){return r.itemAt(this._root,t)},t.prototype.has=function(t,e){return r.hasItem(this._root,t,e)},t.prototype.indexOf=function(t,e){return r.indexOf(this._root,t,e)},t.prototype.get=function(t,e){return r.getItem(this._root,t,e)},t.prototype.assign=function(t){this.clear(),this.update(t)},t.prototype.insert=function(t){var e=r.insertItem(this._root,t,this.cmp);return this._root=r.maybeSplitRoot(this._root),e},t.prototype.update=function(t){var e=this;i.each(t,function(t){e.insert(t)})},t.prototype.delete=function(t,e){var n=r.deleteItem(this._root,t,e);return this._root=r.maybeExtractRoot(this._root),n},t.prototype.remove=function(t){var e=r.removeItem(this._root,t);return this._root=r.maybeExtractRoot(this._root),e},t.prototype.clear=function(){r.clear(this._root),this._root=new r.LeafNode},t}();e.BPlusTree=o,function(t){t.from=function(e,n){var r=new t(n);return r.assign(e),r}}(o=e.BPlusTree||(e.BPlusTree={})),e.BPlusTree=o,function(t){var e=function(){function t(){this.items=[],this.sizes=[],this.children=[]}return Object.defineProperty(t.prototype,"type",{get:function(){return 0},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"size",{get:function(){return this.sizes[this.sizes.length-1]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return this.children.length},enumerable:!0,configurable:!0}),t}();t.BranchNode=e;var n=function(){function t(){this.next=null,this.prev=null,this.items=[]}return Object.defineProperty(t.prototype,"type",{get:function(){return 1},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"size",{get:function(){return this.items.length},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return this.items.length},enumerable:!0,configurable:!0}),t}();function r(t){for(;0===t.type;)t=t.children[0];return t}function o(t){for(;0===t.type;)t=t.children[t.children.length-1];return t}function a(t){0===t.type?(i.each(t.children,a),t.children.length=0,t.sizes.length=0,t.items.length=0):(t.items.length=0,t.next=null,t.prev=null)}t.LeafNode=n,t.firstLeaf=r,t.lastLeaf=o,t.iterItems=function(t){var e=r(t);return new l(e,0,-1)},t.retroItems=function(t){var e=o(t);return new c(e,e.size-1,-1)},t.sliceItems=function(t,e,n){e=void 0===e?0:e<0?Math.max(0,e+t.size):Math.min(e,t.size),n=void 0===n?t.size:n<0?Math.max(0,n+t.size):Math.min(n,t.size);var r=Math.max(0,n-e);if(0===r)return i.empty();for(;0===t.type;){var o=u(t.sizes,e);o>0&&(e-=t.sizes[o-1]),t=t.children[o]}return new l(t,e,r)},t.retroSliceItems=function(t,e,n){e=void 0===e?t.size-1:e<0?Math.max(-1,e+t.size):Math.min(e,t.size-1),n=void 0===n?-1:n<0?Math.max(-1,n+t.size):Math.min(n,t.size-1);var r=Math.max(0,e-n);if(0===r)return i.empty();for(;0===t.type;){var o=u(t.sizes,e);o>0&&(e-=t.sizes[o-1]),t=t.children[o]}return new c(t,e,r)},t.itemAt=function(t,e){if(e<0&&(e+=t.size),!(e<0||e>=t.size)){for(;0===t.type;){var n=u(t.sizes,e);n>0&&(e-=t.sizes[n-1]),t=t.children[n]}return t.items[e]}},t.hasItem=function(t,e,n){for(;0===t.type;){var r=g(t.items,e,n);t=t.children[r]}return h(t.items,e,n)>=0},t.indexOf=function(t,e,n){for(var r=0;0===t.type;){var i=g(t.items,e,n);i>0&&(r+=t.sizes[i-1]),t=t.children[i]}var o=h(t.items,e,n);return o>=0?r+o:-r+o},t.getItem=function(t,e,n){for(;0===t.type;){var r=g(t.items,e,n);t=t.children[r]}var i=h(t.items,e,n);return i>=0?t.items[i]:void 0},t.insertItem=function t(e,n,r){if(1===e.type){var o,a=h(e.items,n,r);return a>=0?(o=e.items[a],e.items[a]=n):(o=void 0,i.ArrayExt.insert(e.items,-a-1,n)),o}var A=g(e.items,n,r),l=e.children[A],c=l.size,u=t(l,n,r),p=l.size;if(e.items[A]=l.items[0],c===p)return u;if(l.width>s){var I=f(l);i.ArrayExt.insert(e.children,A+1,I),i.ArrayExt.insert(e.items,A+1,I.items[0])}return d(e,A),u},t.deleteItem=function t(e,n,r){if(1===e.type){var o=h(e.items,n,r);if(o<0)return;return i.ArrayExt.removeAt(e.items,o)}var a=g(e.items,n,r),s=e.children[a],l=s.size,c=t(s,n,r);return l===s.size?c:(e.items[a]=s.items[0],s.width<A&&(a=p(e,a)),d(e,a),c)},t.removeItem=function t(e,n){if(n<0&&(n+=e.size),!(n<0||n>=e.size)){if(1===e.type)return i.ArrayExt.removeAt(e.items,n);var r=u(e.sizes,n);r>0&&(n-=e.sizes[r]);var o=e.children[r],a=t(o,n);return e.items[r]=o.items[0],o.width<A&&(r=p(e,r)),d(e,r),a}},t.clear=a,t.maybeSplitRoot=function(t){if(t.width<=s)return t;var n=new e,r=f(t);return n.sizes[0]=t.size,n.sizes[1]=t.size+r.size,n.children[0]=t,n.children[1]=r,n.items[0]=t.items[0],n.items[1]=r.items[0],n},t.maybeExtractRoot=function(t){if(1===t.type)return t;if(t.children.length>1)return t;var e=t.children.pop();return a(t),e};var s=32,A=s>>1,l=function(){function t(t,e,n){this._node=t,this._index=e,this._count=n}return t.prototype.iter=function(){return this},t.prototype.clone=function(){return new t(this._node,this._index,this._count)},t.prototype.next=function(){if(null!==this._node&&0!==this._count)return this._index>=this._node.size?(this._node=this._node.next,this._index=0,this.next()):(this._count>0&&this._count--,this._node.items[this._index++])},t}(),c=function(){function t(t,e,n){this._node=t,this._index=e,this._count=n}return t.prototype.iter=function(){return this},t.prototype.clone=function(){return new t(this._node,this._index,this._count)},t.prototype.next=function(){if(null!==this._node&&0!==this._count)return this._index>=this._node.size&&(this._index=this._node.size-1),this._index<0?(this._node=this._node.prev,this._index=this._node?this._node.size-1:-1,this.next()):(this._count>0&&this._count--,this._node.items[this._index--])},t}();function u(t,e){for(var n=t.length,r=0;r<n;++r)if(t[r]>e)return r;return n-1}function g(t,e,n){for(var r=t.length,i=1;i<r;++i)if(n(t[i],e)>0)return i-1;return r-1}function h(t,e,n){for(var r=t.length,i=0;i<r;++i){var o=n(t[i],e);if(0===o)return i;if(o>0)return-i-1}return-r-1}function d(t,e){for(var n=t.sizes,r=t.children,i=e>0?n[e-1]:0,o=r.length;e<o;++e)i=n[e]=i+r[e].size;n.length=r.length}function f(t){if(1===t.type){for(var r=new n,i=t.items,o=r.items,a=A,s=i.length;a<s;++a)o.push(i[a]);return i.length=A,t.next&&(t.next.prev=r),r.next=t.next,r.prev=t,t.next=r,r}var l=new e,c=t.children,u=l.children;for(a=A,s=c.length;a<s;++a)u.push(c[a]);c.length=A;var g=t.items,h=l.items;for(a=A,s=g.length;a<s;++a)h.push(g[a]);return g.length=A,d(t,A),d(l,0),l}function p(t,e){var n,r,o,s,l,c,u=t.children[e],g=0===e?t.children[e+1]:t.children[e-1],h=0===e,f=1===u.type,p=g.width>A;if(f&&p&&h){var I=g;return(C=u).items.push(I.items.shift()),t.items[e+1]=I.items[0],e}if(f&&p&&!h){I=g;return(C=u).items.unshift(I.items.pop()),t.items[e]=C.items[0],e-1}if(f&&!p&&h){var C=u;return(n=(I=g).items).unshift.apply(n,C.items),i.ArrayExt.removeAt(t.children,e),i.ArrayExt.removeAt(t.items,e+1),C.prev&&(C.prev.next=I),I.prev=C.prev,a(C),e}if(f&&!p&&!h){C=u;return(r=(I=g).items).push.apply(r,C.items),i.ArrayExt.removeAt(t.children,e),i.ArrayExt.removeAt(t.items,e),C.next&&(C.next.prev=I),I.next=C.next,a(C),e-1}if(!f&&p&&h){I=g;return(C=u).children.push(I.children.shift()),C.items.push(I.items.shift()),t.items[e+1]=I.items[0],d(C,C.width-1),d(I,0),e}if(!f&&p&&!h){I=g;return(C=u).children.unshift(I.children.pop()),C.items.unshift(I.items.pop()),t.items[e]=C.items[0],d(C,0),d(I,I.width-1),e-1}if(!f&&!p&&h){C=u;return(o=(I=g).children).unshift.apply(o,C.children),(s=I.items).unshift.apply(s,C.items),i.ArrayExt.removeAt(t.children,e),i.ArrayExt.removeAt(t.items,e+1),d(I,0),C.children.length=0,a(C),e}if(!f&&!p&&!h){C=u;return(l=(I=g).children).push.apply(l,C.children),(c=I.items).push.apply(c,C.items),i.ArrayExt.removeAt(t.children,e),i.ArrayExt.removeAt(t.items,e),d(I,0),C.children.length=0,a(C),e-1}throw"unreachable"}}(r||(r={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r,i=n(5),o=function(){function t(){this._first=null,this._last=null,this._size=0}return Object.defineProperty(t.prototype,"isEmpty",{get:function(){return 0===this._size},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"size",{get:function(){return this._size},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"length",{get:function(){return this._size},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"first",{get:function(){return this._first?this._first.value:void 0},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"last",{get:function(){return this._last?this._last.value:void 0},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"firstNode",{get:function(){return this._first},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lastNode",{get:function(){return this._last},enumerable:!0,configurable:!0}),t.prototype.iter=function(){return new t.ForwardValueIterator(this._first)},t.prototype.retro=function(){return new t.RetroValueIterator(this._last)},t.prototype.nodes=function(){return new t.ForwardNodeIterator(this._first)},t.prototype.retroNodes=function(){return new t.RetroNodeIterator(this._last)},t.prototype.assign=function(t){var e=this;this.clear(),i.each(t,function(t){e.addLast(t)})},t.prototype.push=function(t){this.addLast(t)},t.prototype.pop=function(){return this.removeLast()},t.prototype.shift=function(t){this.addFirst(t)},t.prototype.unshift=function(){return this.removeFirst()},t.prototype.addFirst=function(t){var e=new r.LinkedListNode(this,t);return this._first?(e.next=this._first,this._first.prev=e,this._first=e):(this._first=e,this._last=e),this._size++,e},t.prototype.addLast=function(t){var e=new r.LinkedListNode(this,t);return this._last?(e.prev=this._last,this._last.next=e,this._last=e):(this._first=e,this._last=e),this._size++,e},t.prototype.insertBefore=function(t,e){if(!e||e===this._first)return this.addFirst(t);if(!(e instanceof r.LinkedListNode)||e.list!==this)throw new Error("Reference node is not owned by the list.");var n=new r.LinkedListNode(this,t),i=e,o=i.prev;return n.next=i,n.prev=o,i.prev=n,o.next=n,this._size++,n},t.prototype.insertAfter=function(t,e){if(!e||e===this._last)return this.addLast(t);if(!(e instanceof r.LinkedListNode)||e.list!==this)throw new Error("Reference node is not owned by the list.");var n=new r.LinkedListNode(this,t),i=e,o=i.next;return n.next=o,n.prev=i,i.next=n,o.prev=n,this._size++,n},t.prototype.removeFirst=function(){var t=this._first;if(t)return t===this._last?(this._first=null,this._last=null):(this._first=t.next,this._first.prev=null),t.list=null,t.next=null,t.prev=null,this._size--,t.value},t.prototype.removeLast=function(){var t=this._last;if(t)return t===this._first?(this._first=null,this._last=null):(this._last=t.prev,this._last.next=null),t.list=null,t.next=null,t.prev=null,this._size--,t.value},t.prototype.removeNode=function(t){if(!(t instanceof r.LinkedListNode)||t.list!==this)throw new Error("Node is not owned by the list.");var e=t;e===this._first&&e===this._last?(this._first=null,this._last=null):e===this._first?(this._first=e.next,this._first.prev=null):e===this._last?(this._last=e.prev,this._last.next=null):(e.next.prev=e.prev,e.prev.next=e.next),e.list=null,e.next=null,e.prev=null,this._size--},t.prototype.clear=function(){for(var t=this._first;t;){var e=t.next;t.list=null,t.prev=null,t.next=null,t=e}this._first=null,this._last=null,this._size=0},t}();e.LinkedList=o,function(t){t.from=function(e){var n=new t;return n.assign(e),n};var e=function(){function t(t){this._node=t}return t.prototype.iter=function(){return this},t.prototype.clone=function(){return new t(this._node)},t.prototype.next=function(){if(this._node){var t=this._node;return this._node=t.next,t.value}},t}();t.ForwardValueIterator=e;var n=function(){function t(t){this._node=t}return t.prototype.iter=function(){return this},t.prototype.clone=function(){return new t(this._node)},t.prototype.next=function(){if(this._node){var t=this._node;return this._node=t.prev,t.value}},t}();t.RetroValueIterator=n;var r=function(){function t(t){this._node=t}return t.prototype.iter=function(){return this},t.prototype.clone=function(){return new t(this._node)},t.prototype.next=function(){if(this._node){var t=this._node;return this._node=t.next,t}},t}();t.ForwardNodeIterator=r;var i=function(){function t(t){this._node=t}return t.prototype.iter=function(){return this},t.prototype.clone=function(){return new t(this._node)},t.prototype.next=function(){if(this._node){var t=this._node;return this._node=t.prev,t}},t}();t.RetroNodeIterator=i}(o=e.LinkedList||(e.LinkedList={})),e.LinkedList=o,function(t){var e=function(t,e){this.list=null,this.next=null,this.prev=null,this.list=t,this.value=e};t.LinkedListNode=e}(r||(r={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(288),s=function(t){function e(e){void 0===e&&(e={});var n=t.call(this,{layout:o.createLayout(e)})||this;return n.addClass("p-BoxPanel"),n}return i(e,t),Object.defineProperty(e.prototype,"direction",{get:function(){return this.layout.direction},set:function(t){this.layout.direction=t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"alignment",{get:function(){return this.layout.alignment},set:function(t){this.layout.alignment=t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"spacing",{get:function(){return this.layout.spacing},set:function(t){this.layout.spacing=t},enumerable:!0,configurable:!0}),e.prototype.onChildAdded=function(t){t.child.addClass("p-BoxPanel-child")},e.prototype.onChildRemoved=function(t){t.child.removeClass("p-BoxPanel-child")},e}(n(215).Panel);e.BoxPanel=s,function(t){t.getStretch=function(t){return a.BoxLayout.getStretch(t)},t.setStretch=function(t,e){a.BoxLayout.setStretch(t,e)},t.getSizeBasis=function(t){return a.BoxLayout.getSizeBasis(t)},t.setSizeBasis=function(t,e){a.BoxLayout.setSizeBasis(t,e)}}(s=e.BoxPanel||(e.BoxPanel={})),e.BoxPanel=s,function(t){t.createLayout=function(t){return t.layout||new a.BoxLayout(t)}}(o||(o={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)}),o=this&&this.__assign||function(){return(o=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)};Object.defineProperty(e,"__esModule",{value:!0});var a,s=n(5),A=n(9),l=n(570),c=n(46),u=n(27),g=n(59),h=function(t){function e(n){var r=t.call(this,{node:a.createNode()})||this;return r._activeIndex=-1,r._items=[],r._results=null,r.addClass("p-CommandPalette"),r.setFlag(g.Widget.Flag.DisallowLayout),r.commands=n.commands,r.renderer=n.renderer||e.defaultRenderer,r.commands.commandChanged.connect(r._onGenericChange,r),r.commands.keyBindingChanged.connect(r._onGenericChange,r),r}return i(e,t),e.prototype.dispose=function(){this._items.length=0,this._results=null,t.prototype.dispose.call(this)},Object.defineProperty(e.prototype,"searchNode",{get:function(){return this.node.getElementsByClassName("p-CommandPalette-search")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"inputNode",{get:function(){return this.node.getElementsByClassName("p-CommandPalette-input")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"contentNode",{get:function(){return this.node.getElementsByClassName("p-CommandPalette-content")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"items",{get:function(){return this._items},enumerable:!0,configurable:!0}),e.prototype.addItem=function(t){var e=a.createItem(this.commands,t);return this._items.push(e),this.refresh(),e},e.prototype.removeItem=function(t){this.removeItemAt(this._items.indexOf(t))},e.prototype.removeItemAt=function(t){s.ArrayExt.removeAt(this._items,t)&&this.refresh()},e.prototype.clearItems=function(){0!==this._items.length&&(this._items.length=0,this.refresh())},e.prototype.refresh=function(){this._results=null,this.update()},e.prototype.handleEvent=function(t){switch(t.type){case"click":this._evtClick(t);break;case"keydown":this._evtKeyDown(t);break;case"input":this.refresh();break;case"focus":case"blur":this._toggleFocused()}},e.prototype.onBeforeAttach=function(t){this.node.addEventListener("click",this),this.node.addEventListener("keydown",this),this.node.addEventListener("input",this),this.node.addEventListener("focus",this,!0),this.node.addEventListener("blur",this,!0)},e.prototype.onAfterDetach=function(t){this.node.removeEventListener("click",this),this.node.removeEventListener("keydown",this),this.node.removeEventListener("input",this),this.node.removeEventListener("focus",this,!0),this.node.removeEventListener("blur",this,!0)},e.prototype.onActivateRequest=function(t){if(this.isAttached){var e=this.inputNode;e.focus(),e.select()}},e.prototype.onUpdateRequest=function(t){var e=this.inputNode.value,n=this.contentNode,r=this._results;if(r||(r=this._results=a.search(this._items,e),this._activeIndex=e?s.ArrayExt.findFirstIndex(r,a.canActivate):-1),e||0!==r.length)if(e&&0===r.length){var i=this.renderer.renderEmptyMessage({query:e});u.VirtualDOM.render(i,n)}else{for(var o=this.renderer,A=this._activeIndex,l=new Array(r.length),g=0,h=r.length;g<h;++g){var d=r[g];if("header"===d.type){var f=d.indices,p=d.category;l[g]=o.renderHeader({category:p,indices:f})}else{var I=d.item,C=(f=d.indices,g===A);l[g]=o.renderItem({item:I,indices:f,active:C})}}if(u.VirtualDOM.render(l,n),A<0||A>=r.length)n.scrollTop=0;else{var m=n.children[A];c.ElementExt.scrollIntoViewIfNeeded(n,m)}}else u.VirtualDOM.render(null,n)},e.prototype._evtClick=function(t){if(0===t.button){var e=s.ArrayExt.findFirstIndex(this.contentNode.children,function(e){return e.contains(t.target)});-1!==e&&(t.preventDefault(),t.stopPropagation(),this._execute(e))}},e.prototype._evtKeyDown=function(t){if(!(t.altKey||t.ctrlKey||t.metaKey||t.shiftKey))switch(t.keyCode){case 13:t.preventDefault(),t.stopPropagation(),this._execute(this._activeIndex);break;case 38:t.preventDefault(),t.stopPropagation(),this._activatePreviousItem();break;case 40:t.preventDefault(),t.stopPropagation(),this._activateNextItem()}},e.prototype._activateNextItem=function(){if(this._results&&0!==this._results.length){var t=this._activeIndex,e=this._results.length,n=t<e-1?t+1:0,r=0===n?e-1:n-1;this._activeIndex=s.ArrayExt.findFirstIndex(this._results,a.canActivate,n,r),this.update()}},e.prototype._activatePreviousItem=function(){if(this._results&&0!==this._results.length){var t=this._activeIndex,e=this._results.length,n=t<=0?e-1:t-1,r=n===e-1?0:n+1;this._activeIndex=s.ArrayExt.findLastIndex(this._results,a.canActivate,n,r),this.update()}},e.prototype._execute=function(t){if(this._results){var e=this._results[t];if(e){if("header"===e.type){var n=this.inputNode;return n.value=e.category.toLowerCase()+" ",n.focus(),void this.refresh()}e.item.isEnabled&&(this.commands.execute(e.item.command,e.item.args),this.inputNode.value="",this.refresh())}}},e.prototype._toggleFocused=function(){var t=document.activeElement===this.inputNode;this.toggleClass("p-mod-focused",t)},e.prototype._onGenericChange=function(){this.refresh()},e}(g.Widget);e.CommandPalette=h,function(t){var e=function(){function t(){}return t.prototype.renderHeader=function(t){var e=this.formatHeader(t);return u.h.li({className:"p-CommandPalette-header"},e)},t.prototype.renderItem=function(t){var e=this.createItemClass(t),n=this.createItemDataset(t);return u.h.li({className:e,dataset:n},this.renderItemIcon(t),this.renderItemContent(t),this.renderItemShortcut(t))},t.prototype.renderEmptyMessage=function(t){var e=this.formatEmptyMessage(t);return u.h.li({className:"p-CommandPalette-emptyMessage"},e)},t.prototype.renderItemIcon=function(t){var e=this.createIconClass(t);return u.h.div({className:e},t.item.iconLabel)},t.prototype.renderItemContent=function(t){return u.h.div({className:"p-CommandPalette-itemContent"},this.renderItemLabel(t),this.renderItemCaption(t))},t.prototype.renderItemLabel=function(t){var e=this.formatItemLabel(t);return u.h.div({className:"p-CommandPalette-itemLabel"},e)},t.prototype.renderItemCaption=function(t){var e=this.formatItemCaption(t);return u.h.div({className:"p-CommandPalette-itemCaption"},e)},t.prototype.renderItemShortcut=function(t){var e=this.formatItemShortcut(t);return u.h.div({className:"p-CommandPalette-itemShortcut"},e)},t.prototype.createItemClass=function(t){var e="p-CommandPalette-item";t.item.isEnabled||(e+=" p-mod-disabled"),t.item.isToggled&&(e+=" p-mod-toggled"),t.active&&(e+=" p-mod-active");var n=t.item.className;return n&&(e+=" "+n),e},t.prototype.createItemDataset=function(t){return o({},t.item.dataset,{command:t.item.command})},t.prototype.createIconClass=function(t){var e="p-CommandPalette-itemIcon",n=t.item.iconClass;return n?e+" "+n:e},t.prototype.formatHeader=function(t){return t.indices&&0!==t.indices.length?s.StringExt.highlight(t.category,t.indices,u.h.mark):t.category},t.prototype.formatEmptyMessage=function(t){return"No commands found that match '"+t.query+"'"},t.prototype.formatItemShortcut=function(t){var e=t.item.keyBinding;return e?e.keys.map(l.CommandRegistry.formatKeystroke).join(", "):null},t.prototype.formatItemLabel=function(t){return t.indices&&0!==t.indices.length?s.StringExt.highlight(t.item.label,t.indices,u.h.mark):t.item.label},t.prototype.formatItemCaption=function(t){return t.item.caption},t}();t.Renderer=e,t.defaultRenderer=new e}(h=e.CommandPalette||(e.CommandPalette={})),e.CommandPalette=h,function(t){function e(t,e){for(var n=t.category.toLowerCase(),r=n+" "+t.label.toLowerCase(),i=1/0,o=null,a=/\b\w/g;;){var A=a.exec(r);if(!A)break;var l=s.StringExt.matchSumOfDeltas(r,e,A.index);if(!l)break;l&&l.score<=i&&(i=l.score,o=l.indices)}if(!o||i===1/0)return null;for(var c=n.length+1,u=s.ArrayExt.lowerBound(o,c,function(t,e){return t-e}),g=o.slice(0,u),h=o.slice(u),d=0,f=h.length;d<f;++d)h[d]-=c;return 0===g.length?{matchType:0,categoryIndices:null,labelIndices:h,score:i,item:t}:0===h.length?{matchType:1,categoryIndices:g,labelIndices:null,score:i,item:t}:{matchType:2,categoryIndices:g,labelIndices:h,score:i,item:t}}function n(t,e){var n=t.matchType-e.matchType;if(0!==n)return n;var r=t.score-e.score;if(0!==r)return r;var i=0,o=0;switch(t.matchType){case 0:i=t.labelIndices[0],o=e.labelIndices[0];break;case 1:case 2:i=t.categoryIndices[0],o=e.categoryIndices[0]}if(i!==o)return i-o;var a=t.item.category.localeCompare(e.item.category);if(0!==a)return a;var s=t.item.rank,A=e.item.rank;return s!==A?s<A?-1:1:t.item.label.localeCompare(e.item.label)}t.createNode=function(){var t=document.createElement("div"),e=document.createElement("div"),n=document.createElement("div"),r=document.createElement("input"),i=document.createElement("ul");return e.className="p-CommandPalette-search",n.className="p-CommandPalette-wrapper",r.className="p-CommandPalette-input",i.className="p-CommandPalette-content",r.spellcheck=!1,n.appendChild(r),e.appendChild(n),t.appendChild(e),t.appendChild(i),t},t.createItem=function(t,e){return new r(t,e)},t.search=function(t,r){var i=function(t,n){n=n.replace(/\s+/g,"").toLowerCase();for(var r=[],i=0,o=t.length;i<o;++i){var a=t[i];if(a.isVisible)if(n){var s=e(a,n);s&&(a.isEnabled||(s.score+=1e3),r.push(s))}else r.push({matchType:3,categoryIndices:null,labelIndices:null,score:0,item:a})}return r}(t,r);return i.sort(n),function(t){var e=new Array(t.length);s.ArrayExt.fill(e,!1);for(var n=[],r=0,i=t.length;r<i;++r)if(!e[r]){var o=t[r],a=o.item,A=o.categoryIndices,l=a.category;n.push({type:"header",category:l,indices:A});for(var c=r;c<i;++c)if(!e[c]){var u=t[c],g=u.item,h=u.labelIndices;g.category===l&&(n.push({type:"item",item:g,indices:h}),e[c]=!0)}}return n}(i)},t.canActivate=function(t){return"item"===t.type&&t.item.isEnabled};var r=function(){function t(t,e){this._commands=t,this.category=e.category.trim().replace(/\s+/g," "),this.command=e.command,this.args=e.args||A.JSONExt.emptyObject,this.rank=void 0!==e.rank?e.rank:1/0}return Object.defineProperty(t.prototype,"label",{get:function(){return this._commands.label(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"iconClass",{get:function(){return this._commands.iconClass(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"iconLabel",{get:function(){return this._commands.iconLabel(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"caption",{get:function(){return this._commands.caption(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"className",{get:function(){return this._commands.className(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dataset",{get:function(){return this._commands.dataset(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isEnabled",{get:function(){return this._commands.isEnabled(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isToggled",{get:function(){return this._commands.isToggled(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isVisible",{get:function(){return this._commands.isVisible(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"keyBinding",{get:function(){var t=this.command,e=this.args;return s.ArrayExt.findLastValue(this._commands.keyBindings,function(n){return n.command===t&&A.JSONExt.deepEqual(n.args,e)})||null},enumerable:!0,configurable:!0}),t}()}(a||(a={}))},function(t,e,n){"use strict";var r=this&&this.__assign||function(){return(r=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)};Object.defineProperty(e,"__esModule",{value:!0});var i,o=n(5),a=n(50),s=n(46),A=n(571),l=function(){function t(t){this._idTick=0,this._items=[],this.menu=new A.Menu(t)}return t.prototype.addItem=function(t){var e=this,n=i.createItem(t,this._idTick++);return this._items.push(n),new a.DisposableDelegate(function(){o.ArrayExt.removeFirstOf(e._items,n)})},t.prototype.open=function(t){var e=this;if(this.menu.clearItems(),0===this._items.length)return!1;var n=i.matchItems(this._items,t);return!(!n||0===n.length)&&(o.each(n,function(t){e.menu.addItem(t)}),this.menu.open(t.clientX,t.clientY),!0)},t}();e.ContextMenu=l,function(t){function e(t,e){var n=s.Selector.calculateSpecificity(t.selector),r=s.Selector.calculateSpecificity(e.selector);if(n!==r)return r-n;var i=t.rank,o=e.rank;return i!==o?i<o?-1:1:t.id-e.id}t.createItem=function(t,e){var n=function(t){if(-1!==t.indexOf(","))throw new Error("Selector cannot contain commas: "+t);if(!s.Selector.isValid(t))throw new Error("Invalid selector: "+t);return t}(t.selector),i=void 0!==t.rank?t.rank:1/0;return r({},t,{selector:n,rank:i,id:e})},t.matchItems=function(t,n){var r=n.target;if(!r)return null;var i=n.currentTarget;if(!i)return null;if(!(i.contains(r)||(r=document.elementFromPoint(n.clientX,n.clientY))&&i.contains(r)))return null;for(var o=[],a=t.slice();null!==r;){for(var A=[],l=0,c=a.length;l<c;++l){var u=a[l];u&&s.Selector.matches(r,u.selector)&&(A.push(u),a[l]=null)}if(0!==A.length&&(A.sort(e),o.push.apply(o,A)),r===i)break;r=r.parentElement}return o}}(i||(i={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(5),s=n(9),A=n(46),l=n(127),c=n(25),u=n(43),g=n(10),h=n(572),d=n(291),f=n(59),p=function(t){function e(n){void 0===n&&(n={});var r=t.call(this)||this;r._drag=null,r._pressData=null,r._layoutModified=new g.Signal(r),r.addClass("p-DockPanel"),r._mode=n.mode||"multiple-document",r._renderer=n.renderer||e.defaultRenderer,r._edges=n.edges||o.DEFAULT_EDGES,r.dataset.mode=r._mode;var i={createTabBar:function(){return r._createTabBar()},createHandle:function(){return r._createHandle()}};return r.layout=new h.DockLayout({renderer:i,spacing:n.spacing}),r.overlay=n.overlay||new e.Overlay,r.node.appendChild(r.overlay.node),r}return i(e,t),e.prototype.dispose=function(){this._releaseMouse(),this.overlay.hide(0),this._drag&&this._drag.dispose(),t.prototype.dispose.call(this)},Object.defineProperty(e.prototype,"layoutModified",{get:function(){return this._layoutModified},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"renderer",{get:function(){return this.layout.renderer},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"spacing",{get:function(){return this.layout.spacing},set:function(t){this.layout.spacing=t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"mode",{get:function(){return this._mode},set:function(t){if(this._mode!==t){this._mode=t,this.dataset.mode=t;var e=this.layout;switch(t){case"multiple-document":a.each(e.tabBars(),function(t){t.show()});break;case"single-document":e.restoreLayout(o.createSingleDocumentConfig(this));break;default:throw"unreachable"}c.MessageLoop.postMessage(this,o.LayoutModified)}},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isEmpty",{get:function(){return this.layout.isEmpty},enumerable:!0,configurable:!0}),e.prototype.widgets=function(){return this.layout.widgets()},e.prototype.selectedWidgets=function(){return this.layout.selectedWidgets()},e.prototype.tabBars=function(){return this.layout.tabBars()},e.prototype.handles=function(){return this.layout.handles()},e.prototype.selectWidget=function(t){var e=a.find(this.tabBars(),function(e){return-1!==e.titles.indexOf(t.title)});if(!e)throw new Error("Widget is not contained in the dock panel.");e.currentTitle=t.title},e.prototype.activateWidget=function(t){this.selectWidget(t),t.activate()},e.prototype.saveLayout=function(){return this.layout.saveLayout()},e.prototype.restoreLayout=function(t){this._mode="multiple-document",this.layout.restoreLayout(t),(A.Platform.IS_EDGE||A.Platform.IS_IE)&&c.MessageLoop.flush(),c.MessageLoop.postMessage(this,o.LayoutModified)},e.prototype.addWidget=function(t,e){void 0===e&&(e={}),"single-document"===this._mode?this.layout.addWidget(t):this.layout.addWidget(t,e),c.MessageLoop.postMessage(this,o.LayoutModified)},e.prototype.processMessage=function(e){"layout-modified"===e.type?this._layoutModified.emit(void 0):t.prototype.processMessage.call(this,e)},e.prototype.handleEvent=function(t){switch(t.type){case"p-dragenter":this._evtDragEnter(t);break;case"p-dragleave":this._evtDragLeave(t);break;case"p-dragover":this._evtDragOver(t);break;case"p-drop":this._evtDrop(t);break;case"mousedown":this._evtMouseDown(t);break;case"mousemove":this._evtMouseMove(t);break;case"mouseup":this._evtMouseUp(t);break;case"keydown":this._evtKeyDown(t);break;case"contextmenu":t.preventDefault(),t.stopPropagation()}},e.prototype.onBeforeAttach=function(t){this.node.addEventListener("p-dragenter",this),this.node.addEventListener("p-dragleave",this),this.node.addEventListener("p-dragover",this),this.node.addEventListener("p-drop",this),this.node.addEventListener("mousedown",this)},e.prototype.onAfterDetach=function(t){this.node.removeEventListener("p-dragenter",this),this.node.removeEventListener("p-dragleave",this),this.node.removeEventListener("p-dragover",this),this.node.removeEventListener("p-drop",this),this.node.removeEventListener("mousedown",this),this._releaseMouse()},e.prototype.onChildAdded=function(t){o.isGeneratedTabBarProperty.get(t.child)||t.child.addClass("p-DockPanel-widget")},e.prototype.onChildRemoved=function(t){o.isGeneratedTabBarProperty.get(t.child)||(t.child.removeClass("p-DockPanel-widget"),c.MessageLoop.postMessage(this,o.LayoutModified))},e.prototype._evtDragEnter=function(t){t.mimeData.hasData("application/vnd.phosphor.widget-factory")&&(t.preventDefault(),t.stopPropagation())},e.prototype._evtDragLeave=function(t){t.preventDefault(),t.stopPropagation(),this.overlay.hide(1)},e.prototype._evtDragOver=function(t){t.preventDefault(),t.stopPropagation(),"invalid"===this._showOverlay(t.clientX,t.clientY)?t.dropAction="none":t.dropAction=t.proposedAction},e.prototype._evtDrop=function(t){if(t.preventDefault(),t.stopPropagation(),this.overlay.hide(0),"none"!==t.proposedAction){var e=t.clientX,n=t.clientY,r=o.findDropTarget(this,e,n,this._edges),i=r.zone,a=r.target;if("invalid"!==i){var s=t.mimeData.getData("application/vnd.phosphor.widget-factory");if("function"==typeof s){var A=s();if(A instanceof f.Widget)if(A.contains(this))t.dropAction="none";else{var l=a?o.getDropRef(a.tabBar):null;switch(i){case"root-all":this.addWidget(A);break;case"root-top":this.addWidget(A,{mode:"split-top"});break;case"root-left":this.addWidget(A,{mode:"split-left"});break;case"root-right":this.addWidget(A,{mode:"split-right"});break;case"root-bottom":this.addWidget(A,{mode:"split-bottom"});break;case"widget-all":this.addWidget(A,{mode:"tab-after",ref:l});break;case"widget-top":this.addWidget(A,{mode:"split-top",ref:l});break;case"widget-left":this.addWidget(A,{mode:"split-left",ref:l});break;case"widget-right":this.addWidget(A,{mode:"split-right",ref:l});break;case"widget-bottom":this.addWidget(A,{mode:"split-bottom",ref:l});break;case"widget-tab":this.addWidget(A,{mode:"tab-after",ref:l});break;default:throw"unreachable"}t.dropAction=t.proposedAction,this.activateWidget(A)}else t.dropAction="none"}else t.dropAction="none"}else t.dropAction="none"}else t.dropAction="none"},e.prototype._evtKeyDown=function(t){t.preventDefault(),t.stopPropagation(),27===t.keyCode&&(this._releaseMouse(),c.MessageLoop.postMessage(this,o.LayoutModified))},e.prototype._evtMouseDown=function(t){if(0===t.button){var e=this.layout,n=t.target,r=a.find(e.handles(),function(t){return t.contains(n)});if(r){t.preventDefault(),t.stopPropagation(),document.addEventListener("keydown",this,!0),document.addEventListener("mouseup",this,!0),document.addEventListener("mousemove",this,!0),document.addEventListener("contextmenu",this,!0);var i=r.getBoundingClientRect(),o=t.clientX-i.left,s=t.clientY-i.top,A=window.getComputedStyle(r),c=l.Drag.overrideCursor(A.cursor);this._pressData={handle:r,deltaX:o,deltaY:s,override:c}}}},e.prototype._evtMouseMove=function(t){if(this._pressData){t.preventDefault(),t.stopPropagation();var e=this.node.getBoundingClientRect(),n=t.clientX-e.left-this._pressData.deltaX,r=t.clientY-e.top-this._pressData.deltaY;this.layout.moveHandle(this._pressData.handle,n,r)}},e.prototype._evtMouseUp=function(t){0===t.button&&(t.preventDefault(),t.stopPropagation(),this._releaseMouse(),c.MessageLoop.postMessage(this,o.LayoutModified))},e.prototype._releaseMouse=function(){this._pressData&&(this._pressData.override.dispose(),this._pressData=null,document.removeEventListener("keydown",this,!0),document.removeEventListener("mouseup",this,!0),document.removeEventListener("mousemove",this,!0),document.removeEventListener("contextmenu",this,!0))},e.prototype._showOverlay=function(t,e){var n,r,i,a,s=o.findDropTarget(this,t,e,this._edges),l=s.zone,c=s.target;if("invalid"===l)return this.overlay.hide(100),l;var u=A.ElementExt.boxSizing(this.node),g=this.node.getBoundingClientRect();switch(l){case"root-all":n=u.paddingTop,r=u.paddingLeft,i=u.paddingRight,a=u.paddingBottom;break;case"root-top":n=u.paddingTop,r=u.paddingLeft,i=u.paddingRight,a=g.height*o.GOLDEN_RATIO;break;case"root-left":n=u.paddingTop,r=u.paddingLeft,i=g.width*o.GOLDEN_RATIO,a=u.paddingBottom;break;case"root-right":n=u.paddingTop,r=g.width*o.GOLDEN_RATIO,i=u.paddingRight,a=u.paddingBottom;break;case"root-bottom":n=g.height*o.GOLDEN_RATIO,r=u.paddingLeft,i=u.paddingRight,a=u.paddingBottom;break;case"widget-all":n=c.top,r=c.left,i=c.right,a=c.bottom;break;case"widget-top":n=c.top,r=c.left,i=c.right,a=c.bottom+c.height/2;break;case"widget-left":n=c.top,r=c.left,i=c.right+c.width/2,a=c.bottom;break;case"widget-right":n=c.top,r=c.left+c.width/2,i=c.right,a=c.bottom;break;case"widget-bottom":n=c.top+c.height/2,r=c.left,i=c.right,a=c.bottom;break;case"widget-tab":var h=c.tabBar.node.getBoundingClientRect().height;n=c.top,r=c.left,i=c.right,a=c.bottom+c.height-h;break;default:throw"unreachable"}return this.overlay.show({top:n,left:r,right:i,bottom:a}),l},e.prototype._createTabBar=function(){var t=this._renderer.createTabBar();return o.isGeneratedTabBarProperty.set(t,!0),"single-document"===this._mode&&t.hide(),t.tabsMovable=!0,t.allowDeselect=!1,t.removeBehavior="select-previous-tab",t.insertBehavior="select-tab-if-needed",t.tabMoved.connect(this._onTabMoved,this),t.currentChanged.connect(this._onCurrentChanged,this),t.tabCloseRequested.connect(this._onTabCloseRequested,this),t.tabDetachRequested.connect(this._onTabDetachRequested,this),t.tabActivateRequested.connect(this._onTabActivateRequested,this),t},e.prototype._createHandle=function(){return this._renderer.createHandle()},e.prototype._onTabMoved=function(){c.MessageLoop.postMessage(this,o.LayoutModified)},e.prototype._onCurrentChanged=function(t,e){var n=e.previousTitle,r=e.currentTitle;n&&n.owner.hide(),r&&r.owner.show(),(A.Platform.IS_EDGE||A.Platform.IS_IE)&&c.MessageLoop.flush(),c.MessageLoop.postMessage(this,o.LayoutModified)},e.prototype._onTabActivateRequested=function(t,e){e.title.owner.activate()},e.prototype._onTabCloseRequested=function(t,e){e.title.owner.close()},e.prototype._onTabDetachRequested=function(t,e){var n=this;if(!this._drag){t.releaseMouse();var r=e.title,i=e.tab,o=e.clientX,a=e.clientY,A=new s.MimeData;A.setData("application/vnd.phosphor.widget-factory",function(){return r.owner});var c=i.cloneNode(!0);this._drag=new l.Drag({mimeData:A,dragImage:c,proposedAction:"move",supportedActions:"move"}),i.classList.add("p-mod-hidden");this._drag.start(o,a).then(function(){n._drag=null,i.classList.remove("p-mod-hidden")})}},e}(f.Widget);e.DockPanel=p,function(t){var e=function(){function t(){this._timer=-1,this._hidden=!0,this.node=document.createElement("div"),this.node.classList.add("p-DockPanel-overlay"),this.node.classList.add("p-mod-hidden"),this.node.style.position="absolute"}return t.prototype.show=function(t){var e=this.node.style;e.top=t.top+"px",e.left=t.left+"px",e.right=t.right+"px",e.bottom=t.bottom+"px",clearTimeout(this._timer),this._timer=-1,this._hidden&&(this._hidden=!1,this.node.classList.remove("p-mod-hidden"))},t.prototype.hide=function(t){var e=this;if(!this._hidden)return t<=0?(clearTimeout(this._timer),this._timer=-1,this._hidden=!0,void this.node.classList.add("p-mod-hidden")):void(-1===this._timer&&(this._timer=window.setTimeout(function(){e._timer=-1,e._hidden=!0,e.node.classList.add("p-mod-hidden")},t)))},t}();t.Overlay=e;var n=function(){function t(){}return t.prototype.createTabBar=function(){var t=new d.TabBar;return t.addClass("p-DockPanel-tabBar"),t},t.prototype.createHandle=function(){var t=document.createElement("div");return t.className="p-DockPanel-handle",t},t}();t.Renderer=n,t.defaultRenderer=new n}(p=e.DockPanel||(e.DockPanel={})),e.DockPanel=p,function(t){t.GOLDEN_RATIO=.618,t.DEFAULT_EDGES={top:12,right:40,bottom:40,left:40},t.LayoutModified=new c.ConflatableMessage("layout-modified"),t.isGeneratedTabBarProperty=new u.AttachedProperty({name:"isGeneratedTabBar",create:function(){return!1}}),t.createSingleDocumentConfig=function(t){if(t.isEmpty)return{main:null};var e=a.toArray(t.widgets()),n=t.selectedWidgets().next(),r=n?e.indexOf(n):-1;return{main:{type:"tab-area",widgets:e,currentIndex:r}}},t.findDropTarget=function(t,e,n,r){if(!A.ElementExt.hitTest(t.node,e,n))return{zone:"invalid",target:null};var i=t.layout;if(i.isEmpty)return{zone:"root-all",target:null};if("multiple-document"===t.mode){var o=t.node.getBoundingClientRect(),a=e-o.left+1,s=n-o.top+1,l=o.right-e,c=o.bottom-n;switch(Math.min(s,l,c,a)){case s:if(s<r.top)return{zone:"root-top",target:null};break;case l:if(l<r.right)return{zone:"root-right",target:null};break;case c:if(c<r.bottom)return{zone:"root-bottom",target:null};break;case a:if(a<r.left)return{zone:"root-left",target:null};break;default:throw"unreachable"}}var u=i.hitTestTabAreas(e,n);if(!u)return{zone:"invalid",target:null};if("single-document"===t.mode)return{zone:"widget-all",target:u};var g=u.x-u.left+1,h=u.y-u.top+1,d=u.left+u.width-u.x,f=u.top+u.height-u.y;if(h<u.tabBar.node.getBoundingClientRect().height)return{zone:"widget-tab",target:u};var p,I=Math.round(u.width/3),C=Math.round(u.height/3);if(g>I&&d>I&&h>C&&f>C)return{zone:"widget-all",target:u};switch(g/=I,h/=C,d/=I,f/=C,Math.min(g,h,d,f)){case g:p="widget-left";break;case h:p="widget-top";break;case d:p="widget-right";break;case f:p="widget-bottom";break;default:throw"unreachable"}return{zone:p,target:u}},t.getDropRef=function(t){return 0===t.titles.length?null:t.currentTitle?t.currentTitle.owner:t.titles[t.titles.length-1].owner}}(o||(o={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(5),i=n(10),o=function(){function t(){this._counter=0,this._widgets=[],this._activeWidget=null,this._currentWidget=null,this._numbers=new Map,this._nodes=new Map,this._activeChanged=new i.Signal(this),this._currentChanged=new i.Signal(this)}return t.prototype.dispose=function(){var t=this;this._counter<0||(this._counter=-1,i.Signal.clearData(this),r.each(this._widgets,function(e){e.node.removeEventListener("focus",t,!0),e.node.removeEventListener("blur",t,!0)}),this._activeWidget=null,this._currentWidget=null,this._nodes.clear(),this._numbers.clear(),this._widgets.length=0)},Object.defineProperty(t.prototype,"currentChanged",{get:function(){return this._currentChanged},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"activeChanged",{get:function(){return this._activeChanged},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isDisposed",{get:function(){return this._counter<0},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"currentWidget",{get:function(){return this._currentWidget},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"activeWidget",{get:function(){return this._activeWidget},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"widgets",{get:function(){return this._widgets},enumerable:!0,configurable:!0}),t.prototype.focusNumber=function(t){var e=this._numbers.get(t);return void 0===e?-1:e},t.prototype.has=function(t){return this._numbers.has(t)},t.prototype.add=function(t){if(!this._numbers.has(t)){var e=t.node.contains(document.activeElement),n=e?this._counter++:-1;this._widgets.push(t),this._numbers.set(t,n),this._nodes.set(t.node,t),t.node.addEventListener("focus",this,!0),t.node.addEventListener("blur",this,!0),t.disposed.connect(this._onWidgetDisposed,this),e&&this._setWidgets(t,t)}},t.prototype.remove=function(t){var e=this;if(this._numbers.has(t)&&(t.disposed.disconnect(this._onWidgetDisposed,this),t.node.removeEventListener("focus",this,!0),t.node.removeEventListener("blur",this,!0),r.ArrayExt.removeFirstOf(this._widgets,t),this._nodes.delete(t.node),this._numbers.delete(t),this._currentWidget===t)){var n=r.filter(this._widgets,function(t){return-1!==e._numbers.get(t)}),i=r.max(n,function(t,n){return e._numbers.get(t)-e._numbers.get(n)})||null;this._setWidgets(i,null)}},t.prototype.handleEvent=function(t){switch(t.type){case"focus":this._evtFocus(t);break;case"blur":this._evtBlur(t)}},t.prototype._setWidgets=function(t,e){var n=this._currentWidget;this._currentWidget=t;var r=this._activeWidget;this._activeWidget=e,n!==t&&this._currentChanged.emit({oldValue:n,newValue:t}),r!==e&&this._activeChanged.emit({oldValue:r,newValue:e})},t.prototype._evtFocus=function(t){var e=this._nodes.get(t.currentTarget);e!==this._currentWidget&&this._numbers.set(e,this._counter++),this._setWidgets(e,e)},t.prototype._evtBlur=function(t){var e=this._nodes.get(t.currentTarget),n=t.relatedTarget;n&&(e.node.contains(n)||r.find(this._widgets,function(t){return t.node.contains(n)}))||this._setWidgets(this._currentWidget,null)},t.prototype._onWidgetDisposed=function(t){this.remove(t)},t}();e.FocusTracker=o},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(5),s=n(46),A=n(25),l=n(43),c=n(174),u=n(107),g=n(59),h=function(t){function e(e){void 0===e&&(e={});var n=t.call(this,e)||this;return n._dirty=!1,n._rowSpacing=4,n._columnSpacing=4,n._items=[],n._rowStarts=[],n._columnStarts=[],n._rowSizers=[new c.BoxSizer],n._columnSizers=[new c.BoxSizer],n._box=null,void 0!==e.rowCount&&o.reallocSizers(n._rowSizers,e.rowCount),void 0!==e.columnCount&&o.reallocSizers(n._columnSizers,e.columnCount),void 0!==e.rowSpacing&&(n._rowSpacing=o.clampValue(e.rowSpacing)),void 0!==e.columnSpacing&&(n._columnSpacing=o.clampValue(e.columnSpacing)),n}return i(e,t),e.prototype.dispose=function(){a.each(this._items,function(t){var e=t.widget;t.dispose(),e.dispose()}),this._box=null,this._items.length=0,this._rowStarts.length=0,this._rowSizers.length=0,this._columnStarts.length=0,this._columnSizers.length=0,t.prototype.dispose.call(this)},Object.defineProperty(e.prototype,"rowCount",{get:function(){return this._rowSizers.length},set:function(t){t!==this.rowCount&&(o.reallocSizers(this._rowSizers,t),this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"columnCount",{get:function(){return this._columnSizers.length},set:function(t){t!==this.columnCount&&(o.reallocSizers(this._columnSizers,t),this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"rowSpacing",{get:function(){return this._rowSpacing},set:function(t){t=o.clampValue(t),this._rowSpacing!==t&&(this._rowSpacing=t,this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"columnSpacing",{get:function(){return this._columnSpacing},set:function(t){t=o.clampValue(t),this._columnSpacing!==t&&(this._columnSpacing=t,this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),e.prototype.rowStretch=function(t){var e=this._rowSizers[t];return e?e.stretch:-1},e.prototype.setRowStretch=function(t,e){var n=this._rowSizers[t];n&&(e=o.clampValue(e),n.stretch!==e&&(n.stretch=e,this.parent&&this.parent.update()))},e.prototype.columnStretch=function(t){var e=this._columnSizers[t];return e?e.stretch:-1},e.prototype.setColumnStretch=function(t,e){var n=this._columnSizers[t];n&&(e=o.clampValue(e),n.stretch!==e&&(n.stretch=e,this.parent&&this.parent.update()))},e.prototype.iter=function(){return a.map(this._items,function(t){return t.widget})},e.prototype.addWidget=function(t){-1===a.ArrayExt.findFirstIndex(this._items,function(e){return e.widget===t})&&(this._items.push(new u.LayoutItem(t)),this.parent&&this.attachWidget(t))},e.prototype.removeWidget=function(t){var e=a.ArrayExt.findFirstIndex(this._items,function(e){return e.widget===t});if(-1===e){var n=a.ArrayExt.removeAt(this._items,e);this.parent&&this.detachWidget(t),n.dispose()}},e.prototype.init=function(){var e=this;t.prototype.init.call(this),a.each(this,function(t){e.attachWidget(t)})},e.prototype.attachWidget=function(t){this.parent.isAttached&&A.MessageLoop.sendMessage(t,g.Widget.Msg.BeforeAttach),this.parent.node.appendChild(t.node),this.parent.isAttached&&A.MessageLoop.sendMessage(t,g.Widget.Msg.AfterAttach),this.parent.fit()},e.prototype.detachWidget=function(t){this.parent.isAttached&&A.MessageLoop.sendMessage(t,g.Widget.Msg.BeforeDetach),this.parent.node.removeChild(t.node),this.parent.isAttached&&A.MessageLoop.sendMessage(t,g.Widget.Msg.AfterDetach),this.parent.fit()},e.prototype.onBeforeShow=function(e){t.prototype.onBeforeShow.call(this,e),this.parent.update()},e.prototype.onBeforeAttach=function(e){t.prototype.onBeforeAttach.call(this,e),this.parent.fit()},e.prototype.onChildShown=function(t){this.parent.fit()},e.prototype.onChildHidden=function(t){this.parent.fit()},e.prototype.onResize=function(t){this.parent.isVisible&&this._update(t.width,t.height)},e.prototype.onUpdateRequest=function(t){this.parent.isVisible&&this._update(-1,-1)},e.prototype.onFitRequest=function(t){this.parent.isAttached&&this._fit()},e.prototype._fit=function(){for(var t=0,n=this.rowCount;t<n;++t)this._rowSizers[t].minSize=0;for(t=0,n=this.columnCount;t<n;++t)this._columnSizers[t].minSize=0;var r=this._items.filter(function(t){return!t.isHidden});for(t=0,n=r.length;t<n;++t)r[t].fit();var i=this.rowCount-1,a=this.columnCount-1;r.sort(o.rowSpanCmp);for(t=0,n=r.length;t<n;++t){var l=r[t],c=e.getCellConfig(l.widget),u=Math.min(c.row,i),h=Math.min(c.row+c.rowSpan-1,i);o.distributeMin(this._rowSizers,u,h,l.minHeight)}r.sort(o.columnSpanCmp);for(t=0,n=r.length;t<n;++t){l=r[t],c=e.getCellConfig(l.widget);var d=Math.min(c.column,a),f=Math.min(c.column+c.columnSpan-1,a);o.distributeMin(this._columnSizers,d,f,l.minWidth)}if("set-no-constraint"!==this.fitPolicy){var p=i*this._rowSpacing,I=a*this._columnSpacing;for(t=0,n=this.rowCount;t<n;++t)p+=this._rowSizers[t].minSize;for(t=0,n=this.columnCount;t<n;++t)I+=this._columnSizers[t].minSize;var C=this._box=s.ElementExt.boxSizing(this.parent.node);I+=C.horizontalSum,p+=C.verticalSum;var m=this.parent.node.style;m.minWidth=I+"px",m.minHeight=p+"px",this._dirty=!0,this.parent.parent&&A.MessageLoop.sendMessage(this.parent.parent,g.Widget.Msg.FitRequest),this._dirty&&A.MessageLoop.sendMessage(this.parent,g.Widget.Msg.UpdateRequest)}else A.MessageLoop.sendMessage(this.parent,g.Widget.Msg.UpdateRequest)},e.prototype._update=function(t,n){this._dirty=!1,t<0&&(t=this.parent.node.offsetWidth),n<0&&(n=this.parent.node.offsetHeight),this._box||(this._box=s.ElementExt.boxSizing(this.parent.node));var r=this._box.paddingTop,i=this._box.paddingLeft,o=t-this._box.horizontalSum,a=n-this._box.verticalSum,A=this.rowCount-1,l=this.columnCount-1,u=A*this._rowSpacing,g=l*this._columnSpacing;c.BoxEngine.calc(this._rowSizers,Math.max(0,a-u)),c.BoxEngine.calc(this._columnSizers,Math.max(0,o-g));for(var h=0,d=r,f=this.rowCount;h<f;++h)this._rowStarts[h]=d,d+=this._rowSizers[h].size+this._rowSpacing;for(h=0,d=i,f=this.columnCount;h<f;++h)this._columnStarts[h]=d,d+=this._columnSizers[h].size+this._columnSpacing;for(h=0,f=this._items.length;h<f;++h){var p=this._items[h];if(!p.isHidden){var I=e.getCellConfig(p.widget),C=Math.min(I.row,A),m=Math.min(I.column,l),y=Math.min(I.row+I.rowSpan-1,A),E=Math.min(I.column+I.columnSpan-1,l),v=this._columnStarts[m],B=this._rowStarts[C],b=this._columnStarts[E]+this._columnSizers[E].size-v,Q=this._rowStarts[y]+this._rowSizers[y].size-B;p.update(v,B,b,Q)}}},e}(u.Layout);e.GridLayout=h,function(t){t.getCellConfig=function(t){return o.cellConfigProperty.get(t)},t.setCellConfig=function(t,e){o.cellConfigProperty.set(t,o.normalizeConfig(e))}}(h=e.GridLayout||(e.GridLayout={})),e.GridLayout=h,function(t){t.cellConfigProperty=new l.AttachedProperty({name:"cellConfig",create:function(){return{row:0,column:0,rowSpan:1,columnSpan:1}},changed:function(t){t.parent&&t.parent.layout instanceof h&&t.parent.fit()}}),t.normalizeConfig=function(t){return{row:Math.max(0,Math.floor(t.row||0)),column:Math.max(0,Math.floor(t.column||0)),rowSpan:Math.max(1,Math.floor(t.rowSpan||0)),columnSpan:Math.max(1,Math.floor(t.columnSpan||0))}},t.clampValue=function(t){return Math.max(0,Math.floor(t))},t.rowSpanCmp=function(e,n){var r=t.cellConfigProperty.get(e.widget),i=t.cellConfigProperty.get(n.widget);return r.rowSpan-i.rowSpan},t.columnSpanCmp=function(e,n){var r=t.cellConfigProperty.get(e.widget),i=t.cellConfigProperty.get(n.widget);return r.columnSpan-i.columnSpan},t.reallocSizers=function(t,e){for(e=Math.max(1,Math.floor(e));t.length<e;)t.push(new c.BoxSizer);t.length<e&&(t.length=e)},t.distributeMin=function(t,e,n,r){if(!(n<e))if(e!==n){for(var i=0,o=e;o<=n;++o)i+=t[o].minSize;if(!(i>=r)){var a=(r-i)/(n-e+1);for(o=e;o<=n;++o)t[o].minSize+=a}}else{var s=t[e];s.minSize=Math.max(s.minSize,r)}}}(o||(o={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(5),s=n(46),A=n(290),l=n(25),c=n(27),u=n(59),g=function(t){function e(n){void 0===n&&(n={});var r=t.call(this,{node:o.createNode()})||this;return r._activeIndex=-1,r._menus=[],r._childMenu=null,r.addClass("p-MenuBar"),r.setFlag(u.Widget.Flag.DisallowLayout),r.renderer=n.renderer||e.defaultRenderer,r}return i(e,t),e.prototype.dispose=function(){this._closeChildMenu(),this._menus.length=0,t.prototype.dispose.call(this)},Object.defineProperty(e.prototype,"childMenu",{get:function(){return this._childMenu},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"contentNode",{get:function(){return this.node.getElementsByClassName("p-MenuBar-content")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"activeMenu",{get:function(){return this._menus[this._activeIndex]||null},set:function(t){this.activeIndex=t?this._menus.indexOf(t):-1},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"activeIndex",{get:function(){return this._activeIndex},set:function(t){(t<0||t>=this._menus.length)&&(t=-1),this._activeIndex!==t&&(this._activeIndex=t,this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"menus",{get:function(){return this._menus},enumerable:!0,configurable:!0}),e.prototype.openActiveMenu=function(){-1!==this._activeIndex&&(this._openChildMenu(),this._childMenu&&(this._childMenu.activeIndex=-1,this._childMenu.activateNextItem()))},e.prototype.addMenu=function(t){this.insertMenu(this._menus.length,t)},e.prototype.insertMenu=function(t,e){this._closeChildMenu();var n=this._menus.indexOf(e),r=Math.max(0,Math.min(t,this._menus.length));if(-1===n)return a.ArrayExt.insert(this._menus,r,e),e.addClass("p-MenuBar-menu"),e.aboutToClose.connect(this._onMenuAboutToClose,this),e.menuRequested.connect(this._onMenuMenuRequested,this),e.title.changed.connect(this._onTitleChanged,this),void this.update();r===this._menus.length&&r--,n!==r&&(a.ArrayExt.move(this._menus,n,r),this.update())},e.prototype.removeMenu=function(t){this.removeMenuAt(this._menus.indexOf(t))},e.prototype.removeMenuAt=function(t){this._closeChildMenu();var e=a.ArrayExt.removeAt(this._menus,t);e&&(e.aboutToClose.disconnect(this._onMenuAboutToClose,this),e.menuRequested.disconnect(this._onMenuMenuRequested,this),e.title.changed.disconnect(this._onTitleChanged,this),e.removeClass("p-MenuBar-menu"),this.update())},e.prototype.clearMenus=function(){if(0!==this._menus.length){this._closeChildMenu();for(var t=0,e=this._menus;t<e.length;t++){var n=e[t];n.aboutToClose.disconnect(this._onMenuAboutToClose,this),n.menuRequested.disconnect(this._onMenuMenuRequested,this),n.title.changed.disconnect(this._onTitleChanged,this),n.removeClass("p-MenuBar-menu")}this._menus.length=0,this.update()}},e.prototype.handleEvent=function(t){switch(t.type){case"keydown":this._evtKeyDown(t);break;case"mousedown":this._evtMouseDown(t);break;case"mousemove":this._evtMouseMove(t);break;case"mouseleave":this._evtMouseLeave(t);break;case"contextmenu":t.preventDefault(),t.stopPropagation()}},e.prototype.onBeforeAttach=function(t){this.node.addEventListener("keydown",this),this.node.addEventListener("mousedown",this),this.node.addEventListener("mousemove",this),this.node.addEventListener("mouseleave",this),this.node.addEventListener("contextmenu",this)},e.prototype.onAfterDetach=function(t){this.node.removeEventListener("keydown",this),this.node.removeEventListener("mousedown",this),this.node.removeEventListener("mousemove",this),this.node.removeEventListener("mouseleave",this),this.node.removeEventListener("contextmenu",this),this._closeChildMenu()},e.prototype.onActivateRequest=function(t){this.isAttached&&this.node.focus()},e.prototype.onUpdateRequest=function(t){for(var e=this._menus,n=this.renderer,r=this._activeIndex,i=new Array(e.length),o=0,a=e.length;o<a;++o){var s=e[o].title,A=o===r;i[o]=n.renderItem({title:s,active:A})}c.VirtualDOM.render(i,this.contentNode)},e.prototype._evtKeyDown=function(t){t.preventDefault(),t.stopPropagation();var e=t.keyCode;if(13!==e&&38!==e&&40!==e){if(27===e)return this._closeChildMenu(),this.activeIndex=-1,void this.node.blur();if(37!==e)if(39!==e){var n=A.getKeyboardLayout().keyForKeydownEvent(t);if(n){var r=this._activeIndex+1,i=o.findMnemonic(this._menus,n,r);-1===i.index||i.multiple?-1!==i.index?this.activeIndex=i.index:-1!==i.auto&&(this.activeIndex=i.auto):(this.activeIndex=i.index,this.openActiveMenu())}}else{a=this._activeIndex,s=this._menus.length;this.activeIndex=a===s-1?0:a+1}else{var a=this._activeIndex,s=this._menus.length;this.activeIndex=0===a?s-1:a-1}}else this.openActiveMenu()},e.prototype._evtMouseDown=function(t){if(s.ElementExt.hitTest(this.node,t.clientX,t.clientY)){t.preventDefault(),t.stopPropagation(),t.stopImmediatePropagation();var e=a.ArrayExt.findFirstIndex(this.contentNode.children,function(e){return s.ElementExt.hitTest(e,t.clientX,t.clientY)});-1!==e?0===t.button&&(this._childMenu?(this._closeChildMenu(),this.activeIndex=e):(this.activeIndex=e,this._openChildMenu())):this._closeChildMenu()}},e.prototype._evtMouseMove=function(t){var e=a.ArrayExt.findFirstIndex(this.contentNode.children,function(e){return s.ElementExt.hitTest(e,t.clientX,t.clientY)});e!==this._activeIndex&&(-1===e&&this._childMenu||(this.activeIndex=e,this._childMenu&&this._openChildMenu()))},e.prototype._evtMouseLeave=function(t){this._childMenu||(this.activeIndex=-1)},e.prototype._openChildMenu=function(){var t=this.activeMenu;if(t){var e=this._childMenu;if(e!==t){this._childMenu=t,e?e.close():(this.addClass("p-mod-active"),document.addEventListener("mousedown",this,!0)),l.MessageLoop.sendMessage(this,u.Widget.Msg.UpdateRequest);var n=this.contentNode.children[this._activeIndex].getBoundingClientRect(),r=n.left,i=n.bottom;t.open(r,i,{forceX:!0,forceY:!0})}}else this._closeChildMenu()},e.prototype._closeChildMenu=function(){if(this._childMenu){this.removeClass("p-mod-active"),document.removeEventListener("mousedown",this,!0);var t=this._childMenu;this._childMenu=null,t.close(),this.activeIndex=-1}},e.prototype._onMenuAboutToClose=function(t){t===this._childMenu&&(this.removeClass("p-mod-active"),document.removeEventListener("mousedown",this,!0),this._childMenu=null,this.activeIndex=-1)},e.prototype._onMenuMenuRequested=function(t,e){if(t===this._childMenu){var n=this._activeIndex,r=this._menus.length;switch(e){case"next":this.activeIndex=n===r-1?0:n+1;break;case"previous":this.activeIndex=0===n?r-1:n-1}this.openActiveMenu()}},e.prototype._onTitleChanged=function(){this.update()},e}(u.Widget);e.MenuBar=g,function(t){var e=function(){function t(){}return t.prototype.renderItem=function(t){var e=this.createItemClass(t),n=this.createItemDataset(t);return c.h.li({className:e,dataset:n},this.renderIcon(t),this.renderLabel(t))},t.prototype.renderIcon=function(t){var e=this.createIconClass(t);return c.h.div({className:e},t.title.iconLabel)},t.prototype.renderLabel=function(t){var e=this.formatLabel(t);return c.h.div({className:"p-MenuBar-itemLabel"},e)},t.prototype.createItemClass=function(t){var e="p-MenuBar-item";return t.title.className&&(e+=" "+t.title.className),t.active&&(e+=" p-mod-active"),e},t.prototype.createItemDataset=function(t){return t.title.dataset},t.prototype.createIconClass=function(t){var e="p-MenuBar-itemIcon",n=t.title.iconClass;return n?e+" "+n:e},t.prototype.formatLabel=function(t){var e=t.title,n=e.label,r=e.mnemonic;if(r<0||r>=n.length)return n;var i=n.slice(0,r),o=n.slice(r+1),a=n[r];return[i,c.h.span({className:"p-MenuBar-itemMnemonic"},a),o]},t}();t.Renderer=e,t.defaultRenderer=new e}(g=e.MenuBar||(e.MenuBar={})),e.MenuBar=g,function(t){t.createNode=function(){var t=document.createElement("div"),e=document.createElement("ul");return e.className="p-MenuBar-content",t.appendChild(e),t.tabIndex=-1,t},t.findMnemonic=function(t,e,n){for(var r=-1,i=-1,o=!1,a=e.toUpperCase(),s=0,A=t.length;s<A;++s){var l=(s+n)%A,c=t[l].title;if(0!==c.label.length){var u=c.mnemonic;u>=0&&u<c.label.length?c.label[u].toUpperCase()===a&&(-1===r?r=l:o=!0):-1===i&&c.label[0].toUpperCase()===a&&(i=l)}}return{index:r,multiple:o,auto:i}}}(o||(o={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(46),s=n(127),A=n(10),l=n(59),c=function(t){function e(e){void 0===e&&(e={});var n=t.call(this,{node:o.createNode()})||this;return n._onRepeat=function(){if(n._repeatTimer=-1,n._pressData){var t=n._pressData.part;if("thumb"!==t){n._repeatTimer=window.setTimeout(n._onRepeat,20);var e=n._pressData.mouseX,r=n._pressData.mouseY;if("decrement"!==t)if("increment"!==t){if("track"===t){if(!a.ElementExt.hitTest(n.trackNode,e,r))return;var i=n.thumbNode;if(a.ElementExt.hitTest(i,e,r))return;var o=i.getBoundingClientRect(),s=void 0;return s="horizontal"===n._orientation?e<o.left?"decrement":"increment":r<o.top?"decrement":"increment",void n._pageRequested.emit(s)}}else{if(!a.ElementExt.hitTest(n.incrementNode,e,r))return;n._stepRequested.emit("increment")}else{if(!a.ElementExt.hitTest(n.decrementNode,e,r))return;n._stepRequested.emit("decrement")}}}},n._value=0,n._page=10,n._maximum=100,n._repeatTimer=-1,n._pressData=null,n._thumbMoved=new A.Signal(n),n._stepRequested=new A.Signal(n),n._pageRequested=new A.Signal(n),n.addClass("p-ScrollBar"),n.setFlag(l.Widget.Flag.DisallowLayout),n._orientation=e.orientation||"vertical",n.dataset.orientation=n._orientation,void 0!==e.maximum&&(n._maximum=Math.max(0,e.maximum)),void 0!==e.page&&(n._page=Math.max(0,e.page)),void 0!==e.value&&(n._value=Math.max(0,Math.min(e.value,n._maximum))),n}return i(e,t),Object.defineProperty(e.prototype,"thumbMoved",{get:function(){return this._thumbMoved},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"stepRequested",{get:function(){return this._stepRequested},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"pageRequested",{get:function(){return this._pageRequested},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"orientation",{get:function(){return this._orientation},set:function(t){this._orientation!==t&&(this._releaseMouse(),this._orientation=t,this.dataset.orientation=t,this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"value",{get:function(){return this._value},set:function(t){t=Math.max(0,Math.min(t,this._maximum)),this._value!==t&&(this._value=t,this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"page",{get:function(){return this._page},set:function(t){t=Math.max(0,t),this._page!==t&&(this._page=t,this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"maximum",{get:function(){return this._maximum},set:function(t){t=Math.max(0,t),this._maximum!==t&&(this._maximum=t,this._value=Math.min(this._value,t),this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"decrementNode",{get:function(){return this.node.getElementsByClassName("p-ScrollBar-button")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"incrementNode",{get:function(){return this.node.getElementsByClassName("p-ScrollBar-button")[1]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"trackNode",{get:function(){return this.node.getElementsByClassName("p-ScrollBar-track")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"thumbNode",{get:function(){return this.node.getElementsByClassName("p-ScrollBar-thumb")[0]},enumerable:!0,configurable:!0}),e.prototype.handleEvent=function(t){switch(t.type){case"mousedown":this._evtMouseDown(t);break;case"mousemove":this._evtMouseMove(t);break;case"mouseup":this._evtMouseUp(t);break;case"keydown":this._evtKeyDown(t);break;case"contextmenu":t.preventDefault(),t.stopPropagation()}},e.prototype.onBeforeAttach=function(t){this.node.addEventListener("mousedown",this),this.update()},e.prototype.onAfterDetach=function(t){this.node.removeEventListener("mousedown",this),this._releaseMouse()},e.prototype.onUpdateRequest=function(t){var e=100*this._value/this._maximum,n=100*this._page/(this._page+this._maximum);e=Math.max(0,Math.min(e,100)),n=Math.max(0,Math.min(n,100));var r=this.thumbNode.style;"horizontal"===this._orientation?(r.top="",r.height="",r.left=e+"%",r.width=n+"%",r.transform="translate("+-e+"%, 0%)"):(r.left="",r.width="",r.top=e+"%",r.height=n+"%",r.transform="translate(0%, "+-e+"%)")},e.prototype._evtKeyDown=function(t){if(t.preventDefault(),t.stopPropagation(),27===t.keyCode){var e=this._pressData?this._pressData.value:-1;this._releaseMouse(),-1!==e&&this._moveThumb(e)}},e.prototype._evtMouseDown=function(t){if(0===t.button&&(this.activate(),!this._pressData)){var e=o.findPart(this,t.target);if(e){t.preventDefault(),t.stopPropagation();var n=s.Drag.overrideCursor("default");if(this._pressData={part:e,override:n,delta:-1,value:-1,mouseX:t.clientX,mouseY:t.clientY},document.addEventListener("mousemove",this,!0),document.addEventListener("mouseup",this,!0),document.addEventListener("keydown",this,!0),document.addEventListener("contextmenu",this,!0),"thumb"===e){var r=this.thumbNode,i=r.getBoundingClientRect();return"horizontal"===this._orientation?this._pressData.delta=t.clientX-i.left:this._pressData.delta=t.clientY-i.top,r.classList.add("p-mod-active"),void(this._pressData.value=this._value)}if("track"===e){i=this.thumbNode.getBoundingClientRect();var a=void 0;return a="horizontal"===this._orientation?t.clientX<i.left?"decrement":"increment":t.clientY<i.top?"decrement":"increment",this._repeatTimer=window.setTimeout(this._onRepeat,350),void this._pageRequested.emit(a)}return"decrement"===e?(this.decrementNode.classList.add("p-mod-active"),this._repeatTimer=window.setTimeout(this._onRepeat,350),void this._stepRequested.emit("decrement")):"increment"===e?(this.incrementNode.classList.add("p-mod-active"),this._repeatTimer=window.setTimeout(this._onRepeat,350),void this._stepRequested.emit("increment")):void 0}}},e.prototype._evtMouseMove=function(t){if(this._pressData&&(t.preventDefault(),t.stopPropagation(),this._pressData.mouseX=t.clientX,this._pressData.mouseY=t.clientY,"thumb"===this._pressData.part)){var e,n,r=this.thumbNode.getBoundingClientRect(),i=this.trackNode.getBoundingClientRect();"horizontal"===this._orientation?(e=t.clientX-i.left-this._pressData.delta,n=i.width-r.width):(e=t.clientY-i.top-this._pressData.delta,n=i.height-r.height);var o=0===n?0:e*this._maximum/n;this._moveThumb(o)}},e.prototype._evtMouseUp=function(t){0===t.button&&(t.preventDefault(),t.stopPropagation(),this._releaseMouse())},e.prototype._releaseMouse=function(){this._pressData&&(clearTimeout(this._repeatTimer),this._repeatTimer=-1,this._pressData.override.dispose(),this._pressData=null,document.removeEventListener("mousemove",this,!0),document.removeEventListener("mouseup",this,!0),document.removeEventListener("keydown",this,!0),document.removeEventListener("contextmenu",this,!0),this.thumbNode.classList.remove("p-mod-active"),this.decrementNode.classList.remove("p-mod-active"),this.incrementNode.classList.remove("p-mod-active"))},e.prototype._moveThumb=function(t){t=Math.max(0,Math.min(t,this._maximum)),this._value!==t&&(this._value=t,this.update(),this._thumbMoved.emit(t))},e}(l.Widget);e.ScrollBar=c,function(t){t.createNode=function(){var t=document.createElement("div"),e=document.createElement("div"),n=document.createElement("div"),r=document.createElement("div"),i=document.createElement("div");return e.className="p-ScrollBar-button",n.className="p-ScrollBar-button",e.dataset.action="decrement",n.dataset.action="increment",r.className="p-ScrollBar-track",i.className="p-ScrollBar-thumb",r.appendChild(i),t.appendChild(e),t.appendChild(r),t.appendChild(n),t},t.findPart=function(t,e){return t.thumbNode.contains(e)?"thumb":t.trackNode.contains(e)?"track":t.decrementNode.contains(e)?"decrement":t.incrementNode.contains(e)?"increment":null}}(o||(o={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o=n(5),a=n(25),s=n(107),A=n(59),l=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._widget=null,e}return i(e,t),e.prototype.dispose=function(){if(this._widget){var e=this._widget;this._widget=null,e.dispose()}t.prototype.dispose.call(this)},Object.defineProperty(e.prototype,"widget",{get:function(){return this._widget},set:function(t){t&&(t.parent=this.parent),this._widget!==t&&(this._widget&&this._widget.dispose(),this._widget=t,this.parent&&t&&this.attachWidget(t))},enumerable:!0,configurable:!0}),e.prototype.iter=function(){return this._widget?o.once(this._widget):o.empty()},e.prototype.removeWidget=function(t){this._widget===t&&(this._widget=null,this.parent&&this.detachWidget(t))},e.prototype.init=function(){var e=this;t.prototype.init.call(this),o.each(this,function(t){e.attachWidget(t)})},e.prototype.attachWidget=function(t){this.parent.isAttached&&a.MessageLoop.sendMessage(t,A.Widget.Msg.BeforeAttach),this.parent.node.appendChild(t.node),this.parent.isAttached&&a.MessageLoop.sendMessage(t,A.Widget.Msg.AfterAttach)},e.prototype.detachWidget=function(t){this.parent.isAttached&&a.MessageLoop.sendMessage(t,A.Widget.Msg.BeforeDetach),this.parent.node.removeChild(t.node),this.parent.isAttached&&a.MessageLoop.sendMessage(t,A.Widget.Msg.AfterDetach)},e}(s.Layout);e.SingletonLayout=l},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(5),s=n(127),A=n(215),l=n(573),c=function(t){function e(e){void 0===e&&(e={});var n=t.call(this,{layout:o.createLayout(e)})||this;return n._pressData=null,n.addClass("p-SplitPanel"),n}return i(e,t),e.prototype.dispose=function(){this._releaseMouse(),t.prototype.dispose.call(this)},Object.defineProperty(e.prototype,"orientation",{get:function(){return this.layout.orientation},set:function(t){this.layout.orientation=t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"alignment",{get:function(){return this.layout.alignment},set:function(t){this.layout.alignment=t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"spacing",{get:function(){return this.layout.spacing},set:function(t){this.layout.spacing=t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"renderer",{get:function(){return this.layout.renderer},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"handles",{get:function(){return this.layout.handles},enumerable:!0,configurable:!0}),e.prototype.relativeSizes=function(){return this.layout.relativeSizes()},e.prototype.setRelativeSizes=function(t){this.layout.setRelativeSizes(t)},e.prototype.handleEvent=function(t){switch(t.type){case"mousedown":this._evtMouseDown(t);break;case"mousemove":this._evtMouseMove(t);break;case"mouseup":this._evtMouseUp(t);break;case"keydown":this._evtKeyDown(t);break;case"contextmenu":t.preventDefault(),t.stopPropagation()}},e.prototype.onBeforeAttach=function(t){this.node.addEventListener("mousedown",this)},e.prototype.onAfterDetach=function(t){this.node.removeEventListener("mousedown",this),this._releaseMouse()},e.prototype.onChildAdded=function(t){t.child.addClass("p-SplitPanel-child"),this._releaseMouse()},e.prototype.onChildRemoved=function(t){t.child.removeClass("p-SplitPanel-child"),this._releaseMouse()},e.prototype._evtKeyDown=function(t){t.preventDefault(),t.stopPropagation(),27===t.keyCode&&this._releaseMouse()},e.prototype._evtMouseDown=function(t){if(0===t.button){var e=this.layout,n=a.ArrayExt.findFirstIndex(e.handles,function(e){return e.contains(t.target)});if(-1!==n){var r;t.preventDefault(),t.stopPropagation(),document.addEventListener("mouseup",this,!0),document.addEventListener("mousemove",this,!0),document.addEventListener("keydown",this,!0),document.addEventListener("contextmenu",this,!0);var i=e.handles[n],o=i.getBoundingClientRect();r="horizontal"===e.orientation?t.clientX-o.left:t.clientY-o.top;var A=window.getComputedStyle(i),l=s.Drag.overrideCursor(A.cursor);this._pressData={index:n,delta:r,override:l}}}},e.prototype._evtMouseMove=function(t){var e;t.preventDefault(),t.stopPropagation();var n=this.layout,r=this.node.getBoundingClientRect();e="horizontal"===n.orientation?t.clientX-r.left-this._pressData.delta:t.clientY-r.top-this._pressData.delta,n.moveHandle(this._pressData.index,e)},e.prototype._evtMouseUp=function(t){0===t.button&&(t.preventDefault(),t.stopPropagation(),this._releaseMouse())},e.prototype._releaseMouse=function(){this._pressData&&(this._pressData.override.dispose(),this._pressData=null,document.removeEventListener("mouseup",this,!0),document.removeEventListener("mousemove",this,!0),document.removeEventListener("keydown",this,!0),document.removeEventListener("contextmenu",this,!0))},e}(A.Panel);e.SplitPanel=c,function(t){var e=function(){function t(){}return t.prototype.createHandle=function(){var t=document.createElement("div");return t.className="p-SplitPanel-handle",t},t}();t.Renderer=e,t.defaultRenderer=new e,t.getStretch=function(t){return l.SplitLayout.getStretch(t)},t.setStretch=function(t,e){l.SplitLayout.setStretch(t,e)}}(c=e.SplitPanel||(e.SplitPanel={})),e.SplitPanel=c,function(t){t.createLayout=function(t){return t.layout||new l.SplitLayout({renderer:t.renderer||c.defaultRenderer,orientation:t.orientation,alignment:t.alignment,spacing:t.spacing})}}(o||(o={}))},function(t,e,n){"use strict";var r,i=this&&this.__extends||(r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)});Object.defineProperty(e,"__esModule",{value:!0});var o,a=n(46),s=n(25),A=n(10),l=n(288),c=n(575),u=n(291),g=function(t){function e(e){void 0===e&&(e={});var n=t.call(this)||this;n._currentChanged=new A.Signal(n),n.addClass("p-TabPanel"),n.tabBar=new u.TabBar(e),n.tabBar.addClass("p-TabPanel-tabBar"),n.stackedPanel=new c.StackedPanel,n.stackedPanel.addClass("p-TabPanel-stackedPanel"),n.tabBar.tabMoved.connect(n._onTabMoved,n),n.tabBar.currentChanged.connect(n._onCurrentChanged,n),n.tabBar.tabCloseRequested.connect(n._onTabCloseRequested,n),n.tabBar.tabActivateRequested.connect(n._onTabActivateRequested,n),n.stackedPanel.widgetRemoved.connect(n._onWidgetRemoved,n),n._tabPlacement=e.tabPlacement||"top";var r=o.directionFromPlacement(n._tabPlacement),i=o.orientationFromPlacement(n._tabPlacement);n.tabBar.orientation=i,n.tabBar.dataset.placement=n._tabPlacement;var a=new l.BoxLayout({direction:r,spacing:0});return l.BoxLayout.setStretch(n.tabBar,0),l.BoxLayout.setStretch(n.stackedPanel,1),a.addWidget(n.tabBar),a.addWidget(n.stackedPanel),n.layout=a,n}return i(e,t),Object.defineProperty(e.prototype,"currentChanged",{get:function(){return this._currentChanged},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"currentIndex",{get:function(){return this.tabBar.currentIndex},set:function(t){this.tabBar.currentIndex=t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"currentWidget",{get:function(){var t=this.tabBar.currentTitle;return t?t.owner:null},set:function(t){this.tabBar.currentTitle=t?t.title:null},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"tabsMovable",{get:function(){return this.tabBar.tabsMovable},set:function(t){this.tabBar.tabsMovable=t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"tabPlacement",{get:function(){return this._tabPlacement},set:function(t){if(this._tabPlacement!==t){this._tabPlacement=t;var e=o.directionFromPlacement(t),n=o.orientationFromPlacement(t);this.tabBar.orientation=n,this.tabBar.dataset.placement=t,this.layout.direction=e}},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"widgets",{get:function(){return this.stackedPanel.widgets},enumerable:!0,configurable:!0}),e.prototype.addWidget=function(t){this.insertWidget(this.widgets.length,t)},e.prototype.insertWidget=function(t,e){e!==this.currentWidget&&e.hide(),this.stackedPanel.insertWidget(t,e),this.tabBar.insertTab(t,e.title)},e.prototype._onCurrentChanged=function(t,e){var n=e.previousIndex,r=e.previousTitle,i=e.currentIndex,o=e.currentTitle,A=r?r.owner:null,l=o?o.owner:null;A&&A.hide(),l&&l.show(),this._currentChanged.emit({previousIndex:n,previousWidget:A,currentIndex:i,currentWidget:l}),(a.Platform.IS_EDGE||a.Platform.IS_IE)&&s.MessageLoop.flush()},e.prototype._onTabActivateRequested=function(t,e){e.title.owner.activate()},e.prototype._onTabCloseRequested=function(t,e){e.title.owner.close()},e.prototype._onTabMoved=function(t,e){this.stackedPanel.insertWidget(e.toIndex,e.title.owner)},e.prototype._onWidgetRemoved=function(t,e){this.tabBar.removeTab(e.title)},e}(n(59).Widget);e.TabPanel=g,function(t){t.orientationFromPlacement=function(t){return e[t]},t.directionFromPlacement=function(t){return n[t]};var e={top:"horizontal",left:"vertical",right:"vertical",bottom:"horizontal"},n={top:"top-to-bottom",left:"left-to-right",right:"right-to-left",bottom:"bottom-to-top"}}(o||(o={}))},function(t,e,n){!function(t){"use strict";t.runMode=function(e,n,r,i){var o=t.getMode(t.defaults,n),a=/MSIE \d/.test(navigator.userAgent),s=a&&(null==document.documentMode||document.documentMode<9);if(r.appendChild){var A=i&&i.tabSize||t.defaults.tabSize,l=r,c=0;l.innerHTML="",r=function(t,e){if("\n"==t)return l.appendChild(document.createTextNode(s?"\r":t)),void(c=0);for(var n="",r=0;;){var i=t.indexOf("\t",r);if(-1==i){n+=t.slice(r),c+=t.length-r;break}c+=i-r,n+=t.slice(r,i);var o=A-c%A;c+=o;for(var a=0;a<o;++a)n+=" ";r=i+1}if(e){var u=l.appendChild(document.createElement("span"));u.className="cm-"+e.replace(/ +/g," cm-"),u.appendChild(document.createTextNode(n))}else l.appendChild(document.createTextNode(n))}}for(var u=t.splitLines(e),g=i&&i.state||t.startState(o),h=0,d=u.length;h<d;++h){h&&r("\n");var f=new t.StringStream(u[h]);for(!f.string&&o.blankLine&&o.blankLine(g);!f.eol();){var p=o.token(f,g);r(f.current(),p,h,f.start,g),f.start=f.pos}}}}(n(15))},function(t,e,n){!function(t){"use strict";var e={},n=/[^\s\u00a0]/,r=t.Pos;function i(t){var e=t.search(n);return-1==e?0:e}function o(t,e){var n=t.getMode();return!1!==n.useInnerComments&&n.innerMode?t.getModeAt(e):n}t.commands.toggleComment=function(t){t.toggleComment()},t.defineExtension("toggleComment",function(t){t||(t=e);for(var n=1/0,i=this.listSelections(),o=null,a=i.length-1;a>=0;a--){var s=i[a].from(),A=i[a].to();s.line>=n||(A.line>=n&&(A=r(n,0)),n=s.line,null==o?this.uncomment(s,A,t)?o="un":(this.lineComment(s,A,t),o="line"):"un"==o?this.uncomment(s,A,t):this.lineComment(s,A,t))}}),t.defineExtension("lineComment",function(t,a,s){s||(s=e);var A=this,l=o(A,t),c=A.getLine(t.line);if(null!=c&&(u=t,g=c,!/\bstring\b/.test(A.getTokenTypeAt(r(u.line,0)))||/^[\'\"\`]/.test(g))){var u,g,h=s.lineComment||l.lineComment;if(h){var d=Math.min(0!=a.ch||a.line==t.line?a.line+1:a.line,A.lastLine()+1),f=null==s.padding?" ":s.padding,p=s.commentBlankLines||t.line==a.line;A.operation(function(){if(s.indent){for(var e=null,o=t.line;o<d;++o){var a=A.getLine(o),l=a.slice(0,i(a));(null==e||e.length>l.length)&&(e=l)}for(var o=t.line;o<d;++o){var a=A.getLine(o),c=e.length;(p||n.test(a))&&(a.slice(0,c)!=e&&(c=i(a)),A.replaceRange(e+h+f,r(o,0),r(o,c)))}}else for(var o=t.line;o<d;++o)(p||n.test(A.getLine(o)))&&A.replaceRange(h+f,r(o,0))})}else(s.blockCommentStart||l.blockCommentStart)&&(s.fullLines=!0,A.blockComment(t,a,s))}}),t.defineExtension("blockComment",function(t,i,a){a||(a=e);var s=this,A=o(s,t),l=a.blockCommentStart||A.blockCommentStart,c=a.blockCommentEnd||A.blockCommentEnd;if(l&&c){if(!/\bcomment\b/.test(s.getTokenTypeAt(r(t.line,0)))){var u=Math.min(i.line,s.lastLine());u!=t.line&&0==i.ch&&n.test(s.getLine(u))&&--u;var g=null==a.padding?" ":a.padding;t.line>u||s.operation(function(){if(0!=a.fullLines){var e=n.test(s.getLine(u));s.replaceRange(g+c,r(u)),s.replaceRange(l+g,r(t.line,0));var o=a.blockCommentLead||A.blockCommentLead;if(null!=o)for(var h=t.line+1;h<=u;++h)(h!=u||e)&&s.replaceRange(o+g,r(h,0))}else s.replaceRange(c,i),s.replaceRange(l,t)})}}else(a.lineComment||A.lineComment)&&0!=a.fullLines&&s.lineComment(t,i,a)}),t.defineExtension("uncomment",function(t,i,a){a||(a=e);var s,A=this,l=o(A,t),c=Math.min(0!=i.ch||i.line==t.line?i.line:i.line-1,A.lastLine()),u=Math.min(t.line,c),g=a.lineComment||l.lineComment,h=[],d=null==a.padding?" ":a.padding;t:if(g){for(var f=u;f<=c;++f){var p=A.getLine(f),I=p.indexOf(g);if(I>-1&&!/comment/.test(A.getTokenTypeAt(r(f,I+1)))&&(I=-1),-1==I&&n.test(p))break t;if(I>-1&&n.test(p.slice(0,I)))break t;h.push(p)}if(A.operation(function(){for(var t=u;t<=c;++t){var e=h[t-u],n=e.indexOf(g),i=n+g.length;n<0||(e.slice(i,i+d.length)==d&&(i+=d.length),s=!0,A.replaceRange("",r(t,n),r(t,i)))}}),s)return!0}var C=a.blockCommentStart||l.blockCommentStart,m=a.blockCommentEnd||l.blockCommentEnd;if(!C||!m)return!1;var y=a.blockCommentLead||l.blockCommentLead,E=A.getLine(u),v=E.indexOf(C);if(-1==v)return!1;var B=c==u?E:A.getLine(c),b=B.indexOf(m,c==u?v+C.length:0),Q=r(u,v+1),w=r(c,b+1);if(-1==b||!/comment/.test(A.getTokenTypeAt(Q))||!/comment/.test(A.getTokenTypeAt(w))||A.getRange(Q,w,"\n").indexOf(m)>-1)return!1;var x=E.lastIndexOf(C,t.ch),_=-1==x?-1:E.slice(0,t.ch).indexOf(m,x+C.length);if(-1!=x&&-1!=_&&_+m.length!=t.ch)return!1;_=B.indexOf(m,i.ch);var D=B.slice(i.ch).lastIndexOf(C,_-i.ch);return x=-1==_||-1==D?-1:i.ch+D,(-1==_||-1==x||x==i.ch)&&(A.operation(function(){A.replaceRange("",r(c,b-(d&&B.slice(b-d.length,b)==d?d.length:0)),r(c,b+m.length));var t=v+C.length;if(d&&E.slice(t,t+d.length)==d&&(t+=d.length),A.replaceRange("",r(u,v),r(u,t)),y)for(var e=u+1;e<=c;++e){var i=A.getLine(e),o=i.indexOf(y);if(-1!=o&&!n.test(i.slice(0,o))){var a=o+y.length;d&&i.slice(a,a+d.length)==d&&(a+=d.length),A.replaceRange("",r(e,o),r(e,a))}}}),!0)})}(n(15))},function(t,e,n){!function(t){"use strict";function e(e){e.state.rulerDiv.textContent="";var n=e.getOption("rulers"),r=e.defaultCharWidth(),i=e.charCoords(t.Pos(e.firstLine(),0),"div").left;e.state.rulerDiv.style.minHeight=e.display.scroller.offsetHeight+30+"px";for(var o=0;o<n.length;o++){var a=document.createElement("div");a.className="CodeMirror-ruler";var s,A=n[o];"number"==typeof A?s=A:(s=A.column,A.className&&(a.className+=" "+A.className),A.color&&(a.style.borderColor=A.color),A.lineStyle&&(a.style.borderLeftStyle=A.lineStyle),A.width&&(a.style.borderLeftWidth=A.width)),a.style.left=i+s*r+"px",e.state.rulerDiv.appendChild(a)}}t.defineOption("rulers",!1,function(t,n){t.state.rulerDiv&&(t.state.rulerDiv.parentElement.removeChild(t.state.rulerDiv),t.state.rulerDiv=null,t.off("refresh",e)),n&&n.length&&(t.state.rulerDiv=t.display.lineSpace.parentElement.insertBefore(document.createElement("div"),t.display.lineSpace),t.state.rulerDiv.className="CodeMirror-rulers",e(t),t.on("refresh",e))})}(n(15))},function(t,e,n){!function(t){var e={pairs:"()[]{}''\"\"",closeBefore:")]}'\":;>",triples:"",explode:"[]{}"},n=t.Pos;function r(t,n){return"pairs"==n&&"string"==typeof t?t:"object"==typeof t&&null!=t[n]?t[n]:e[n]}t.defineOption("autoCloseBrackets",!1,function(e,n,a){a&&a!=t.Init&&(e.removeKeyMap(i),e.state.closeBrackets=null),n&&(o(r(n,"pairs")),e.state.closeBrackets=n,e.addKeyMap(i))});var i={Backspace:function(e){var i=s(e);if(!i||e.getOption("disableInput"))return t.Pass;for(var o=r(i,"pairs"),a=e.listSelections(),l=0;l<a.length;l++){if(!a[l].empty())return t.Pass;var c=A(e,a[l].head);if(!c||o.indexOf(c)%2!=0)return t.Pass}for(var l=a.length-1;l>=0;l--){var u=a[l].head;e.replaceRange("",n(u.line,u.ch-1),n(u.line,u.ch+1),"+delete")}},Enter:function(e){var n=s(e),i=n&&r(n,"explode");if(!i||e.getOption("disableInput"))return t.Pass;for(var o=e.listSelections(),a=0;a<o.length;a++){if(!o[a].empty())return t.Pass;var l=A(e,o[a].head);if(!l||i.indexOf(l)%2!=0)return t.Pass}e.operation(function(){var t=e.lineSeparator()||"\n";e.replaceSelection(t+t,null),e.execCommand("goCharLeft"),o=e.listSelections();for(var n=0;n<o.length;n++){var r=o[n].head.line;e.indentLine(r,null,!0),e.indentLine(r+1,null,!0)}})}};function o(t){for(var e=0;e<t.length;e++){var n=t.charAt(e),r="'"+n+"'";i[r]||(i[r]=a(n))}}function a(e){return function(i){return function(e,i){var o=s(e);if(!o||e.getOption("disableInput"))return t.Pass;var a=r(o,"pairs"),A=a.indexOf(i);if(-1==A)return t.Pass;for(var c,u=r(o,"closeBefore"),g=r(o,"triples"),h=a.charAt(A+1)==i,d=e.listSelections(),f=A%2==0,p=0;p<d.length;p++){var I,C=d[p],m=C.head,y=e.getRange(m,n(m.line,m.ch+1));if(f&&!C.empty())I="surround";else if(!h&&f||y!=i)if(h&&m.ch>1&&g.indexOf(i)>=0&&e.getRange(n(m.line,m.ch-2),m)==i+i){if(m.ch>2&&/\bstring/.test(e.getTokenTypeAt(n(m.line,m.ch-2))))return t.Pass;I="addFour"}else if(h){var E=0==m.ch?" ":e.getRange(n(m.line,m.ch-1),m);if(t.isWordChar(y)||E==i||t.isWordChar(E))return t.Pass;I="both"}else{if(!f||!(0===y.length||/\s/.test(y)||u.indexOf(y)>-1))return t.Pass;I="both"}else I=h&&l(e,m)?"both":g.indexOf(i)>=0&&e.getRange(m,n(m.line,m.ch+3))==i+i+i?"skipThree":"skip";if(c){if(c!=I)return t.Pass}else c=I}var v=A%2?a.charAt(A-1):i,B=A%2?i:a.charAt(A+1);e.operation(function(){if("skip"==c)e.execCommand("goCharRight");else if("skipThree"==c)for(var r=0;r<3;r++)e.execCommand("goCharRight");else if("surround"==c){for(var i=e.getSelections(),r=0;r<i.length;r++)i[r]=v+i[r]+B;e.replaceSelections(i,"around"),i=e.listSelections().slice();for(var r=0;r<i.length;r++)i[r]=(o=i[r],a=void 0,a=t.cmpPos(o.anchor,o.head)>0,{anchor:new n(o.anchor.line,o.anchor.ch+(a?-1:1)),head:new n(o.head.line,o.head.ch+(a?1:-1))});e.setSelections(i)}else"both"==c?(e.replaceSelection(v+B,null),e.triggerElectric(v+B),e.execCommand("goCharLeft")):"addFour"==c&&(e.replaceSelection(v+v+v+v,"before"),e.execCommand("goCharRight"));var o,a})}(i,e)}}function s(t){var e=t.state.closeBrackets;if(!e||e.override)return e;var n=t.getModeAt(t.getCursor());return n.closeBrackets||e}function A(t,e){var r=t.getRange(n(e.line,e.ch-1),n(e.line,e.ch+1));return 2==r.length?r:null}function l(t,e){var r=t.getTokenAt(n(e.line,e.ch+1));return/\bstring/.test(r.type)&&r.start==e.ch&&(0==e.ch||!/\bstring/.test(t.getTokenTypeAt(e)))}o(e.pairs+"`")}(n(15))},function(t,e,n){!function(t){"use strict";t.defineOption("foldGutter",!1,function(e,r,i){var o;i&&i!=t.Init&&(e.clearGutter(e.state.foldGutter.options.gutter),e.state.foldGutter=null,e.off("gutterClick",s),e.off("change",A),e.off("viewportChange",l),e.off("fold",c),e.off("unfold",c),e.off("swapDoc",A)),r&&(e.state.foldGutter=new n((!0===(o=r)&&(o={}),null==o.gutter&&(o.gutter="CodeMirror-foldgutter"),null==o.indicatorOpen&&(o.indicatorOpen="CodeMirror-foldgutter-open"),null==o.indicatorFolded&&(o.indicatorFolded="CodeMirror-foldgutter-folded"),o)),a(e),e.on("gutterClick",s),e.on("change",A),e.on("viewportChange",l),e.on("fold",c),e.on("unfold",c),e.on("swapDoc",A))});var e=t.Pos;function n(t){this.options=t,this.from=this.to=0}function r(t,n){for(var r=t.findMarks(e(n,0),e(n+1,0)),i=0;i<r.length;++i)if(r[i].__isFold&&r[i].find().from.line==n)return r[i]}function i(t){if("string"==typeof t){var e=document.createElement("div");return e.className=t+" CodeMirror-guttermarker-subtle",e}return t.cloneNode(!0)}function o(t,n,o){var a=t.state.foldGutter.options,s=n,A=t.foldOption(a,"minFoldSize"),l=t.foldOption(a,"rangeFinder");t.eachLine(n,o,function(n){var o=null;if(r(t,s))o=i(a.indicatorFolded);else{var c=e(s,0),u=l&&l(t,c);u&&u.to.line-u.from.line>=A&&(o=i(a.indicatorOpen))}t.setGutterMarker(n,a.gutter,o),++s})}function a(t){var e=t.getViewport(),n=t.state.foldGutter;n&&(t.operation(function(){o(t,e.from,e.to)}),n.from=e.from,n.to=e.to)}function s(t,n,i){var o=t.state.foldGutter;if(o){var a=o.options;if(i==a.gutter){var s=r(t,n);s?s.clear():t.foldCode(e(n,0),a.rangeFinder)}}}function A(t){var e=t.state.foldGutter;if(e){var n=e.options;e.from=e.to=0,clearTimeout(e.changeUpdate),e.changeUpdate=setTimeout(function(){a(t)},n.foldOnChangeTimeSpan||600)}}function l(t){var e=t.state.foldGutter;if(e){var n=e.options;clearTimeout(e.changeUpdate),e.changeUpdate=setTimeout(function(){var n=t.getViewport();e.from==e.to||n.from-e.to>20||e.from-n.to>20?a(t):t.operation(function(){n.from<e.from&&(o(t,n.from,e.from),e.from=n.from),n.to>e.to&&(o(t,e.to,n.to),e.to=n.to)})},n.updateViewportTimeSpan||400)}}function c(t,e){var n=t.state.foldGutter;if(n){var r=e.line;r>=n.from&&r<n.to&&o(t,r,r+1)}}}(n(15),n(578))},function(t,e,n){!function(t){"use strict";t.registerHelper("fold","brace",function(e,n){var r,i=n.line,o=e.getLine(i);function a(a){for(var s=n.ch,A=0;;){var l=s<=0?-1:o.lastIndexOf(a,s-1);if(-1!=l){if(1==A&&l<n.ch)break;if(r=e.getTokenTypeAt(t.Pos(i,l+1)),!/^(comment|string)/.test(r))return l+1;s=l-1}else{if(1==A)break;A=1,s=o.length}}}var s="{",A="}",l=a("{");if(null==l&&(s="[",A="]",l=a("[")),null!=l){var c,u,g=1,h=e.lastLine();t:for(var d=i;d<=h;++d)for(var f=e.getLine(d),p=d==i?l:0;;){var I=f.indexOf(s,p),C=f.indexOf(A,p);if(I<0&&(I=f.length),C<0&&(C=f.length),(p=Math.min(I,C))==f.length)break;if(e.getTokenTypeAt(t.Pos(d,p+1))==r)if(p==I)++g;else if(!--g){c=d,u=p;break t}++p}if(null!=c&&i!=c)return{from:t.Pos(i,l),to:t.Pos(c,u)}}}),t.registerHelper("fold","import",function(e,n){function r(n){if(n<e.firstLine()||n>e.lastLine())return null;var r=e.getTokenAt(t.Pos(n,1));if(/\S/.test(r.string)||(r=e.getTokenAt(t.Pos(n,r.end+1))),"keyword"!=r.type||"import"!=r.string)return null;for(var i=n,o=Math.min(e.lastLine(),n+10);i<=o;++i){var a=e.getLine(i),s=a.indexOf(";");if(-1!=s)return{startCh:r.end,end:t.Pos(i,s)}}}var i,o=n.line,a=r(o);if(!a||r(o-1)||(i=r(o-2))&&i.end.line==o-1)return null;for(var s=a.end;;){var A=r(s.line+1);if(null==A)break;s=A.end}return{from:e.clipPos(t.Pos(o,a.startCh+1)),to:s}}),t.registerHelper("fold","include",function(e,n){function r(n){if(n<e.firstLine()||n>e.lastLine())return null;var r=e.getTokenAt(t.Pos(n,1));return/\S/.test(r.string)||(r=e.getTokenAt(t.Pos(n,r.end+1))),"meta"==r.type&&"#include"==r.string.slice(0,8)?r.start+8:void 0}var i=n.line,o=r(i);if(null==o||null!=r(i-1))return null;for(var a=i;;){var s=r(a+1);if(null==s)break;++a}return{from:t.Pos(i,o+1),to:e.clipPos(t.Pos(a))}})}(n(15))},function(t,e,n){!function(t){"use strict";function e(e,n){var r=e.getLine(n),i=r.search(/\S/);return-1==i||/\bcomment\b/.test(e.getTokenTypeAt(t.Pos(n,i+1)))?-1:t.countColumn(r,null,e.getOption("tabSize"))}t.registerHelper("fold","indent",function(n,r){var i=e(n,r.line);if(!(i<0)){for(var o=null,a=r.line+1,s=n.lastLine();a<=s;++a){var A=e(n,a);if(-1==A);else{if(!(A>i))break;o=a}}return o?{from:t.Pos(r.line,n.getLine(r.line).length),to:t.Pos(o,n.getLine(o).length)}:void 0}})}(n(15))},function(t,e,n){!function(t){"use strict";t.registerHelper("fold","markdown",function(e,n){var r=100;function i(n){var r=e.getTokenTypeAt(t.Pos(n,0));return r&&/\bheader\b/.test(r)}function o(t,e,n){var o=e&&e.match(/^#+/);return o&&i(t)?o[0].length:(o=n&&n.match(/^[=\-]+\s*$/))&&i(t+1)?"="==n[0]?1:2:r}var a=e.getLine(n.line),s=e.getLine(n.line+1),A=o(n.line,a,s);if(A!==r){for(var l=e.lastLine(),c=n.line,u=e.getLine(c+2);c<l&&!(o(c+1,s,u)<=A);)++c,s=u,u=e.getLine(c+2);return{from:t.Pos(n.line,a.length),to:t.Pos(c,e.getLine(c).length)}}})}(n(15))},function(t,e,n){!function(t){"use strict";var e=t.Pos;function n(t,e){return t.line-e.line||t.ch-e.ch}var r="A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD",i=new RegExp("<(/?)(["+r+"]["+r+"-:.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*)","g");function o(t,e,n,r){this.line=e,this.ch=n,this.cm=t,this.text=t.getLine(e),this.min=r?Math.max(r.from,t.firstLine()):t.firstLine(),this.max=r?Math.min(r.to-1,t.lastLine()):t.lastLine()}function a(t,n){var r=t.cm.getTokenTypeAt(e(t.line,n));return r&&/\btag\b/.test(r)}function s(t){if(!(t.line>=t.max))return t.ch=0,t.text=t.cm.getLine(++t.line),!0}function A(t){if(!(t.line<=t.min))return t.text=t.cm.getLine(--t.line),t.ch=t.text.length,!0}function l(t){for(;;){var e=t.text.indexOf(">",t.ch);if(-1==e){if(s(t))continue;return}if(a(t,e+1)){var n=t.text.lastIndexOf("/",e),r=n>-1&&!/\S/.test(t.text.slice(n+1,e));return t.ch=e+1,r?"selfClose":"regular"}t.ch=e+1}}function c(t){for(;;){var e=t.ch?t.text.lastIndexOf("<",t.ch-1):-1;if(-1==e){if(A(t))continue;return}if(a(t,e+1)){i.lastIndex=e,t.ch=e;var n=i.exec(t.text);if(n&&n.index==e)return n}else t.ch=e}}function u(t){for(;;){i.lastIndex=t.ch;var e=i.exec(t.text);if(!e){if(s(t))continue;return}if(a(t,e.index+1))return t.ch=e.index+e[0].length,e;t.ch=e.index+1}}function g(t){for(;;){var e=t.ch?t.text.lastIndexOf(">",t.ch-1):-1;if(-1==e){if(A(t))continue;return}if(a(t,e+1)){var n=t.text.lastIndexOf("/",e),r=n>-1&&!/\S/.test(t.text.slice(n+1,e));return t.ch=e+1,r?"selfClose":"regular"}t.ch=e}}function h(t,n){for(var r=[];;){var i,o=u(t),a=t.line,s=t.ch-(o?o[0].length:0);if(!o||!(i=l(t)))return;if("selfClose"!=i)if(o[1]){for(var A=r.length-1;A>=0;--A)if(r[A]==o[2]){r.length=A;break}if(A<0&&(!n||n==o[2]))return{tag:o[2],from:e(a,s),to:e(t.line,t.ch)}}else r.push(o[2])}}function d(t,n){for(var r=[];;){var i=g(t);if(!i)return;if("selfClose"!=i){var o=t.line,a=t.ch,s=c(t);if(!s)return;if(s[1])r.push(s[2]);else{for(var A=r.length-1;A>=0;--A)if(r[A]==s[2]){r.length=A;break}if(A<0&&(!n||n==s[2]))return{tag:s[2],from:e(t.line,t.ch),to:e(o,a)}}}else c(t)}}t.registerHelper("fold","xml",function(t,r){for(var i=new o(t,r.line,0);;){var a=u(i);if(!a||i.line!=r.line)return;var s=l(i);if(!s)return;if(!a[1]&&"selfClose"!=s){var A=e(i.line,i.ch),c=h(i,a[2]);return c&&n(c.from,A)>0?{from:A,to:c.from}:null}}}),t.findMatchingTag=function(t,r,i){var a=new o(t,r.line,r.ch,i);if(-1!=a.text.indexOf(">")||-1!=a.text.indexOf("<")){var s=l(a),A=s&&e(a.line,a.ch),u=s&&c(a);if(s&&u&&!(n(a,r)>0)){var g={from:e(a.line,a.ch),to:A,tag:u[2]};return"selfClose"==s?{open:g,close:null,at:"open"}:u[1]?{open:d(a,u[2]),close:g,at:"close"}:(a=new o(t,A.line,A.ch,i),{open:g,close:h(a,u[2]),at:"open"})}}},t.findEnclosingTag=function(t,e,n,r){for(var i=new o(t,e.line,e.ch,n);;){var a=d(i,r);if(!a)break;var s=new o(t,e.line,e.ch,n),A=h(s,a.tag);if(A)return{open:a,close:A}}},t.scanForClosingTag=function(t,e,n,r){var i=new o(t,e.line,e.ch,r?{from:0,to:r}:null);return h(i,n)}}(n(15))},function(t,e,n){!function(t){"use strict";t.registerGlobalHelper("fold","comment",function(t){return t.blockCommentStart&&t.blockCommentEnd},function(e,n){var r=e.getModeAt(n),i=r.blockCommentStart,o=r.blockCommentEnd;if(i&&o){for(var a,s=n.line,A=e.getLine(s),l=n.ch,c=0;;){var u=l<=0?-1:A.lastIndexOf(i,l-1);if(-1!=u){if(1==c&&u<n.ch)return;if(/comment/.test(e.getTokenTypeAt(t.Pos(s,u+1)))&&(0==u||A.slice(u-o.length,u)==o||!/comment/.test(e.getTokenTypeAt(t.Pos(s,u))))){a=u+i.length;break}l=u-1}else{if(1==c)return;c=1,l=A.length}}var g,h,d=1,f=e.lastLine();t:for(var p=s;p<=f;++p)for(var I=e.getLine(p),C=p==s?a:0;;){var m=I.indexOf(i,C),y=I.indexOf(o,C);if(m<0&&(m=I.length),y<0&&(y=I.length),(C=Math.min(m,y))==I.length)break;if(C==m)++d;else if(!--d){g=p,h=C;break t}++C}if(null!=g&&(s!=g||h!=a))return{from:t.Pos(s,a),to:t.Pos(g,h)}}})}(n(15))},function(t,e,n){!function(t){"use strict";function e(e,r){t.changeEnd(r).line==e.lastLine()&&n(e)}function n(t){var e="";if(t.lineCount()>1){var r=t.display.scroller.clientHeight-30,i=t.getLineHandle(t.lastLine()).height;e=r-i+"px"}t.state.scrollPastEndPadding!=e&&(t.state.scrollPastEndPadding=e,t.display.lineSpace.parentNode.style.paddingBottom=e,t.off("refresh",n),t.setSize(),t.on("refresh",n))}t.defineOption("scrollPastEnd",!1,function(r,i,o){o&&o!=t.Init&&(r.off("change",e),r.off("refresh",n),r.display.lineSpace.parentNode.style.paddingBottom="",r.state.scrollPastEndPadding=null),i&&(r.on("change",e),r.on("refresh",n),n(r))})}(n(15))},function(t,e,n){!function(t){"use strict";function e(){this.posFrom=this.posTo=this.lastQuery=this.query=null,this.overlay=null}function n(t){return t.state.search||(t.state.search=new e)}function r(t){return"string"==typeof t&&t==t.toLowerCase()}function i(t,e,n){return t.getSearchCursor(e,n,{caseFold:r(e),multiline:!0})}function o(t,e,n,r,i){t.openDialog?t.openDialog(e,i,{value:r,selectValueOnOpen:!0}):i(prompt(n,r))}function a(t){return t.replace(/\\(.)/g,function(t,e){return"n"==e?"\n":"r"==e?"\r":e})}function s(t){var e=t.match(/^\/(.*)\/([a-z]*)$/);if(e)try{t=new RegExp(e[1],-1==e[2].indexOf("i")?"":"i")}catch(t){}else t=a(t);return("string"==typeof t?""==t:t.test(""))&&(t=/x^/),t}function A(t,e,n){e.queryText=n,e.query=s(n),t.removeOverlay(e.overlay,r(e.query)),e.overlay=function(t,e){return"string"==typeof t?t=new RegExp(t.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g,"\\$&"),e?"gi":"g"):t.global||(t=new RegExp(t.source,t.ignoreCase?"gi":"g")),{token:function(e){t.lastIndex=e.pos;var n=t.exec(e.string);if(n&&n.index==e.pos)return e.pos+=n[0].length||1,"searching";n?e.pos=n.index:e.skipToEnd()}}}(e.query,r(e.query)),t.addOverlay(e.overlay),t.showMatchesOnScrollbar&&(e.annotate&&(e.annotate.clear(),e.annotate=null),e.annotate=t.showMatchesOnScrollbar(e.query,r(e.query)))}function l(e,r,i,a){var s=n(e);if(s.query)return c(e,r);var l=e.getSelection()||s.lastQuery;if(l instanceof RegExp&&"x^"==l.source&&(l=null),i&&e.openDialog){var h=null,d=function(n,r){t.e_stop(r),n&&(n!=s.queryText&&(A(e,s,n),s.posFrom=s.posTo=e.getCursor()),h&&(h.style.opacity=1),c(e,r.shiftKey,function(t,n){var r;n.line<3&&document.querySelector&&(r=e.display.wrapper.querySelector(".CodeMirror-dialog"))&&r.getBoundingClientRect().bottom-4>e.cursorCoords(n,"window").top&&((h=r).style.opacity=.4)}))};!function(t,e,n,r,i){t.openDialog(e,r,{value:n,selectValueOnOpen:!0,closeOnEnter:!1,onClose:function(){u(t)},onKeyDown:i})}(e,g(e),l,d,function(r,i){var o=t.keyName(r),a=e.getOption("extraKeys"),s=a&&a[o]||t.keyMap[e.getOption("keyMap")][o];"findNext"==s||"findPrev"==s||"findPersistentNext"==s||"findPersistentPrev"==s?(t.e_stop(r),A(e,n(e),i),e.execCommand(s)):"find"!=s&&"findPersistent"!=s||(t.e_stop(r),d(i,r))}),a&&l&&(A(e,s,l),c(e,r))}else o(e,g(e),"Search for:",l,function(t){t&&!s.query&&e.operation(function(){A(e,s,t),s.posFrom=s.posTo=e.getCursor(),c(e,r)})})}function c(e,r,o){e.operation(function(){var a=n(e),s=i(e,a.query,r?a.posFrom:a.posTo);(s.find(r)||(s=i(e,a.query,r?t.Pos(e.lastLine()):t.Pos(e.firstLine(),0))).find(r))&&(e.setSelection(s.from(),s.to()),e.scrollIntoView({from:s.from(),to:s.to()},20),a.posFrom=s.from(),a.posTo=s.to(),o&&o(s.from(),s.to()))})}function u(t){t.operation(function(){var e=n(t);e.lastQuery=e.query,e.query&&(e.query=e.queryText=null,t.removeOverlay(e.overlay),e.annotate&&(e.annotate.clear(),e.annotate=null))})}function g(t){return'<span class="CodeMirror-search-label">'+t.phrase("Search:")+'</span> <input type="text" style="width: 10em" class="CodeMirror-search-field"/> <span style="color: #888" class="CodeMirror-search-hint">'+t.phrase("(Use /re/ syntax for regexp search)")+"</span>"}function h(t,e,n){t.operation(function(){for(var r=i(t,e);r.findNext();)if("string"!=typeof e){var o=t.getRange(r.from(),r.to()).match(e);r.replace(n.replace(/\$(\d)/g,function(t,e){return o[e]}))}else r.replace(n)})}function d(t,e){if(!t.getOption("readOnly")){var r=t.getSelection()||n(t).lastQuery,A='<span class="CodeMirror-search-label">'+(e?t.phrase("Replace all:"):t.phrase("Replace:"))+"</span>";o(t,A+function(t){return' <input type="text" style="width: 10em" class="CodeMirror-search-field"/> <span style="color: #888" class="CodeMirror-search-hint">'+t.phrase("(Use /re/ syntax for regexp search)")+"</span>"}(t),A,r,function(n){n&&(n=s(n),o(t,function(t){return'<span class="CodeMirror-search-label">'+t.phrase("With:")+'</span> <input type="text" style="width: 10em" class="CodeMirror-search-field"/>'}(t),t.phrase("Replace with:"),"",function(r){if(r=a(r),e)h(t,n,r);else{u(t);var o=i(t,n,t.getCursor("from")),s=function(){var e,a=o.from();!(e=o.findNext())&&(o=i(t,n),!(e=o.findNext())||a&&o.from().line==a.line&&o.from().ch==a.ch)||(t.setSelection(o.from(),o.to()),t.scrollIntoView({from:o.from(),to:o.to()}),function(t,e,n,r){t.openConfirm?t.openConfirm(e,r):confirm(n)&&r[0]()}(t,function(t){return'<span class="CodeMirror-search-label">'+t.phrase("Replace?")+"</span> <button>"+t.phrase("Yes")+"</button> <button>"+t.phrase("No")+"</button> <button>"+t.phrase("All")+"</button> <button>"+t.phrase("Stop")+"</button> "}(t),t.phrase("Replace?"),[function(){A(e)},s,function(){h(t,n,r)}]))},A=function(t){o.replace("string"==typeof n?r:r.replace(/\$(\d)/g,function(e,n){return t[n]})),s()};s()}}))})}}t.commands.find=function(t){u(t),l(t)},t.commands.findPersistent=function(t){u(t),l(t,!1,!0)},t.commands.findPersistentNext=function(t){l(t,!1,!0,!0)},t.commands.findPersistentPrev=function(t){l(t,!0,!0,!0)},t.commands.findNext=l,t.commands.findPrev=function(t){l(t,!0)},t.commands.clearSearch=u,t.commands.replace=d,t.commands.replaceAll=function(t){d(t,!0)}}(n(15),n(293),n(579))},function(t,e,n){!function(t){"use strict";function e(t,e){var n=Number(e);return/^[-+]/.test(e)?t.getCursor().line+n:n-1}t.commands.jumpToLine=function(t){var n=t.getCursor();!function(t,e,n,r,i){t.openDialog?t.openDialog(e,i,{value:r,selectValueOnOpen:!0}):i(prompt(n,r))}(t,function(t){return t.phrase("Jump to line:")+' <input type="text" style="width: 10em" class="CodeMirror-search-field"/> <span style="color: #888" class="CodeMirror-search-hint">'+t.phrase("(Use line:column or scroll% syntax)")+"</span>"}(t),t.phrase("Jump to line:"),n.line+1+":"+n.ch,function(r){var i;if(r)if(i=/^\s*([\+\-]?\d+)\s*\:\s*(\d+)\s*$/.exec(r))t.setCursor(e(t,i[1]),Number(i[2]));else if(i=/^\s*([\+\-]?\d+(\.\d+)?)\%\s*/.exec(r)){var o=Math.round(t.lineCount()*Number(i[1])/100);/^[-+]/.test(i[1])&&(o=n.line+o+1),t.setCursor(o-1,n.ch)}else(i=/^\s*\:?\s*([\+\-]?\d+)\s*/.exec(r))&&t.setCursor(e(t,i[1]),n.ch)})},t.keyMap.default["Alt-G"]="jumpToLine"}(n(15),n(579))},function(t,e,n){!function(t){"use strict";var e="CodeMirror-activeline",n="CodeMirror-activeline-background",r="CodeMirror-activeline-gutter";function i(t){for(var i=0;i<t.state.activeLines.length;i++)t.removeLineClass(t.state.activeLines[i],"wrap",e),t.removeLineClass(t.state.activeLines[i],"background",n),t.removeLineClass(t.state.activeLines[i],"gutter",r)}function o(t,o){for(var a=[],s=0;s<o.length;s++){var A=o[s],l=t.getOption("styleActiveLine");if("object"==typeof l&&l.nonEmpty?A.anchor.line==A.head.line:A.empty()){var c=t.getLineHandleVisualStart(A.head.line);a[a.length-1]!=c&&a.push(c)}}(function(t,e){if(t.length!=e.length)return!1;for(var n=0;n<t.length;n++)if(t[n]!=e[n])return!1;return!0})(t.state.activeLines,a)||t.operation(function(){i(t);for(var o=0;o<a.length;o++)t.addLineClass(a[o],"wrap",e),t.addLineClass(a[o],"background",n),t.addLineClass(a[o],"gutter",r);t.state.activeLines=a})}function a(t,e){o(t,e.ranges)}t.defineOption("styleActiveLine",!1,function(e,n,r){var s=r!=t.Init&&r;n!=s&&(s&&(e.off("beforeSelectionChange",a),i(e),delete e.state.activeLines),n&&(e.state.activeLines=[],o(e,e.listSelections()),e.on("beforeSelectionChange",a)))})}(n(15))},function(t,e,n){!function(t){"use strict";function e(t){t.state.markedSelection&&t.operation(function(){!function(t){if(!t.somethingSelected())return s(t);if(t.listSelections().length>1)return A(t);var e=t.getCursor("start"),n=t.getCursor("end"),i=t.state.markedSelection;if(!i.length)return a(t,e,n);var l=i[0].find(),c=i[i.length-1].find();if(!l||!c||n.line-e.line<=r||o(e,c.to)>=0||o(n,l.from)<=0)return A(t);for(;o(e,l.from)>0;)i.shift().clear(),l=i[0].find();for(o(e,l.from)<0&&(l.to.line-e.line<r?(i.shift().clear(),a(t,e,l.to,0)):a(t,e,l.from,0));o(n,c.to)<0;)i.pop().clear(),c=i[i.length-1].find();o(n,c.to)>0&&(n.line-c.from.line<r?(i.pop().clear(),a(t,c.from,n)):a(t,c.to,n))}(t)})}function n(t){t.state.markedSelection&&t.state.markedSelection.length&&t.operation(function(){s(t)})}t.defineOption("styleSelectedText",!1,function(r,i,o){var a=o&&o!=t.Init;i&&!a?(r.state.markedSelection=[],r.state.markedSelectionStyle="string"==typeof i?i:"CodeMirror-selectedtext",A(r),r.on("cursorActivity",e),r.on("change",n)):!i&&a&&(r.off("cursorActivity",e),r.off("change",n),s(r),r.state.markedSelection=r.state.markedSelectionStyle=null)});var r=8,i=t.Pos,o=t.cmpPos;function a(t,e,n,a){if(0!=o(e,n))for(var s=t.state.markedSelection,A=t.state.markedSelectionStyle,l=e.line;;){var c=l==e.line?e:i(l,0),u=l+r,g=u>=n.line,h=g?n:i(u,0),d=t.markText(c,h,{className:A});if(null==a?s.push(d):s.splice(a++,0,d),g)break;l=u}}function s(t){for(var e=t.state.markedSelection,n=0;n<e.length;++n)e[n].clear();e.length=0}function A(t){s(t);for(var e=t.listSelections(),n=0;n<e.length;n++)a(t,e[n].from(),e[n].to())}}(n(15))},function(t,e,n){!function(t){"use strict";function e(t){t.state.selectionPointer.rects=null,n(t)}function n(t){t.state.selectionPointer.willUpdate||(t.state.selectionPointer.willUpdate=!0,setTimeout(function(){!function(t){var e=t.state.selectionPointer;if(e){if(null==e.rects&&null!=e.mouseX&&(e.rects=[],t.somethingSelected()))for(var n=t.display.selectionDiv.firstChild;n;n=n.nextSibling)e.rects.push(n.getBoundingClientRect());var r=!1;if(null!=e.mouseX)for(var i=0;i<e.rects.length;i++){var o=e.rects[i];o.left<=e.mouseX&&o.right>=e.mouseX&&o.top<=e.mouseY&&o.bottom>=e.mouseY&&(r=!0)}var a=r?e.value:"";t.display.lineDiv.style.cursor!=a&&(t.display.lineDiv.style.cursor=a)}}(t),t.state.selectionPointer.willUpdate=!1},50))}t.defineOption("selectionPointer",!1,function(r,i){var o=r.state.selectionPointer;o&&(t.off(r.getWrapperElement(),"mousemove",o.mousemove),t.off(r.getWrapperElement(),"mouseout",o.mouseout),t.off(window,"scroll",o.windowScroll),r.off("cursorActivity",e),r.off("scroll",e),r.state.selectionPointer=null,r.display.lineDiv.style.cursor=""),i&&(o=r.state.selectionPointer={value:"string"==typeof i?i:"default",mousemove:function(t){!function(t,e){var r=t.state.selectionPointer;(null==e.buttons?e.which:e.buttons)?r.mouseX=r.mouseY=null:(r.mouseX=e.clientX,r.mouseY=e.clientY),n(t)}(r,t)},mouseout:function(t){!function(t,e){if(!t.getWrapperElement().contains(e.relatedTarget)){var r=t.state.selectionPointer;r.mouseX=r.mouseY=null,n(t)}}(r,t)},windowScroll:function(){e(r)},rects:null,mouseX:null,mouseY:null,willUpdate:!1},t.on(r.getWrapperElement(),"mousemove",o.mousemove),t.on(r.getWrapperElement(),"mouseout",o.mouseout),t.on(window,"scroll",o.windowScroll),r.on("cursorActivity",e),r.on("scroll",e))})}(n(15))},function(t,e,n){!function(t){"use strict";var e=t.Pos;function n(t,e){return t.line==e.line&&t.ch==e.ch}var r=[];function i(t){r.push(t),r.length>50&&r.shift()}function o(t){return r[r.length-(t?Math.min(t,1):1)]||""}var a=null;function s(t,e,o,s,A){null==A&&(A=t.getRange(e,o)),"grow"==s&&a&&a.cm==t&&n(e,a.pos)&&t.isClean(a.gen)?function(t){if(!r.length)return i(t);r[r.length-1]+=t}(A):!1!==s&&i(A),t.replaceRange("",e,o,"+delete"),a="grow"==s?{cm:t,pos:e,gen:t.changeGeneration()}:null}function A(t,e,n){return t.findPosH(e,n,"char",!0)}function l(t,e,n){return t.findPosH(e,n,"word",!0)}function c(t,e,n){return t.findPosV(e,n,"line",t.doc.sel.goalColumn)}function u(t,e,n){return t.findPosV(e,n,"page",t.doc.sel.goalColumn)}function g(t,n,r){for(var i=n.line,o=t.getLine(i),a=/\S/.test(r<0?o.slice(0,n.ch):o.slice(n.ch)),s=t.firstLine(),A=t.lastLine();;){if((i+=r)<s||i>A)return t.clipPos(e(i-r,r<0?0:null));o=t.getLine(i);var l=/\S/.test(o);if(l)a=!0;else if(a)return e(i,0)}}function h(t,n,r){for(var i=n.line,o=n.ch,a=t.getLine(n.line),s=!1;;){var A=a.charAt(o+(r<0?-1:0));if(A){if(s&&/[!?.]/.test(A))return e(i,o+(r>0?1:0));s||(s=/\w/.test(A)),o+=r}else{if(i==(r<0?t.firstLine():t.lastLine()))return e(i,o);if(a=t.getLine(i+r),!/\S/.test(a))return e(i,o);i+=r,o=r<0?a.length:0}}}function d(t,r,i){var o;if(t.findMatchingBracket&&(o=t.findMatchingBracket(r,{strict:!0}))&&o.match&&(o.forward?1:-1)==i)return i>0?e(o.to.line,o.to.ch+1):o.to;for(var a=!0;;a=!1){var s=t.getTokenAt(r),A=e(r.line,i<0?s.start:s.end);if(!(a&&i>0&&s.end==r.ch)&&/\w/.test(s.string))return A;var l=t.findPosH(A,i,"char");if(n(A,l))return r;r=l}}function f(t,e){var n=t.state.emacsPrefix;return n?(b(t),"-"==n?-1:Number(n)):e?null:1}function p(t){var e="string"==typeof t?function(e){e.execCommand(t)}:t;return function(t){var n=f(t);e(t);for(var r=1;r<n;++r)e(t)}}function I(t,e,r,i){var o=f(t);o<0&&(i=-i,o=-o);for(var a=0;a<o;++a){var s=r(t,e,i);if(n(s,e))break;e=s}return e}function C(t,e){var n=function(n){n.extendSelection(I(n,n.getCursor(),t,e))};return n.motion=!0,n}function m(t,e,n,r){for(var i,o=t.listSelections(),a=o.length;a--;)i=o[a].head,s(t,i,I(t,i,e,n),r)}function y(t,e){if(t.somethingSelected()){for(var n,r=t.listSelections(),i=r.length;i--;)n=r[i],s(t,n.anchor,n.head,e);return!0}}function E(t,e){t.state.emacsPrefix?"-"!=e&&(t.state.emacsPrefix+=e):(t.state.emacsPrefix=e,t.on("keyHandled",B),t.on("inputRead",Q))}var v={"Alt-G":!0,"Ctrl-X":!0,"Ctrl-Q":!0,"Ctrl-U":!0};function B(t,e){t.state.emacsPrefixMap||v.hasOwnProperty(e)||b(t)}function b(t){t.state.emacsPrefix=null,t.off("keyHandled",B),t.off("inputRead",Q)}function Q(t,e){var n=f(t);if(n>1&&"+input"==e.origin){for(var r=e.text.join("\n"),i="",o=1;o<n;++o)i+=r;t.replaceSelection(i)}}function w(t,e){("string"!=typeof e||!/^\d$/.test(e)&&"Ctrl-U"!=e)&&(t.removeKeyMap(k),t.state.emacsPrefixMap=!1,t.off("keyHandled",w),t.off("inputRead",w))}function x(t){t.setCursor(t.getCursor()),t.setExtending(!t.getExtending()),t.on("change",function(){t.setExtending(!1)})}function _(t){t.setExtending(!1),t.setCursor(t.getCursor())}function D(t,e){var n=t.getCursor(),r=t.findPosH(n,1,"word");t.replaceRange(e(t.getRange(n,r)),n,r),t.setCursor(r)}t.emacs={kill:s,killRegion:y,repeated:p};var M=t.keyMap.emacs=t.normalizeKeyMap({"Ctrl-W":function(t){s(t,t.getCursor("start"),t.getCursor("end"),!0)},"Ctrl-K":p(function(t){var n=t.getCursor(),r=t.clipPos(e(n.line)),i=t.getRange(n,r);/\S/.test(i)||(i+="\n",r=e(n.line+1,0)),s(t,n,r,"grow",i)}),"Alt-W":function(t){i(t.getSelection()),_(t)},"Ctrl-Y":function(t){var e=t.getCursor();t.replaceRange(o(f(t)),e,e,"paste"),t.setSelection(e,t.getCursor())},"Alt-Y":function(t){t.replaceSelection((r.length>1&&r.pop(),o()),"around","paste")},"Ctrl-Space":x,"Ctrl-Shift-2":x,"Ctrl-F":C(A,1),"Ctrl-B":C(A,-1),Right:C(A,1),Left:C(A,-1),"Ctrl-D":function(t){m(t,A,1,!1)},Delete:function(t){y(t,!1)||m(t,A,1,!1)},"Ctrl-H":function(t){m(t,A,-1,!1)},Backspace:function(t){y(t,!1)||m(t,A,-1,!1)},"Alt-F":C(l,1),"Alt-B":C(l,-1),"Alt-Right":C(l,1),"Alt-Left":C(l,-1),"Alt-D":function(t){m(t,l,1,"grow")},"Alt-Backspace":function(t){m(t,l,-1,"grow")},"Ctrl-N":C(c,1),"Ctrl-P":C(c,-1),Down:C(c,1),Up:C(c,-1),"Ctrl-A":"goLineStart","Ctrl-E":"goLineEnd",End:"goLineEnd",Home:"goLineStart","Alt-V":C(u,-1),"Ctrl-V":C(u,1),PageUp:C(u,-1),PageDown:C(u,1),"Ctrl-Up":C(g,-1),"Ctrl-Down":C(g,1),"Alt-A":C(h,-1),"Alt-E":C(h,1),"Alt-K":function(t){m(t,h,1,"grow")},"Ctrl-Alt-K":function(t){m(t,d,1,"grow")},"Ctrl-Alt-Backspace":function(t){m(t,d,-1,"grow")},"Ctrl-Alt-F":C(d,1),"Ctrl-Alt-B":C(d,-1),"Shift-Ctrl-Alt-2":function(t){var e=t.getCursor();t.setSelection(I(t,e,d,1),e)},"Ctrl-Alt-T":function(t){var e=d(t,t.getCursor(),-1),n=d(t,e,1),r=d(t,n,1),i=d(t,r,-1);t.replaceRange(t.getRange(i,r)+t.getRange(n,i)+t.getRange(e,n),e,r)},"Ctrl-Alt-U":p(function(t){for(var n=t.getCursor(),r=n.line,i=n.ch,o=[];r>=t.firstLine();){for(var a=t.getLine(r),s=null==i?a.length:i;s>0;){var i=a.charAt(--s);if(")"==i)o.push("(");else if("]"==i)o.push("[");else if("}"==i)o.push("{");else if(/[\(\{\[]/.test(i)&&(!o.length||o.pop()!=i))return t.extendSelection(e(r,s))}--r,i=null}}),"Alt-Space":function(t){for(var n=t.getCursor(),r=n.ch,i=n.ch,o=t.getLine(n.line);r&&/\s/.test(o.charAt(r-1));)--r;for(;i<o.length&&/\s/.test(o.charAt(i));)++i;t.replaceRange(" ",e(n.line,r),e(n.line,i))},"Ctrl-O":p(function(t){t.replaceSelection("\n","start")}),"Ctrl-T":p(function(t){t.execCommand("transposeChars")}),"Alt-C":p(function(t){D(t,function(t){var e=t.search(/\w/);return-1==e?t:t.slice(0,e)+t.charAt(e).toUpperCase()+t.slice(e+1).toLowerCase()})}),"Alt-U":p(function(t){D(t,function(t){return t.toUpperCase()})}),"Alt-L":p(function(t){D(t,function(t){return t.toLowerCase()})}),"Alt-;":"toggleComment","Ctrl-/":p("undo"),"Shift-Ctrl--":p("undo"),"Ctrl-Z":p("undo"),"Cmd-Z":p("undo"),"Shift-Ctrl-Z":"redo","Shift-Alt-,":"goDocStart","Shift-Alt-.":"goDocEnd","Ctrl-S":"findPersistentNext","Ctrl-R":"findPersistentPrev","Ctrl-G":function(t){t.execCommand("clearSearch"),_(t)},"Shift-Alt-5":"replace","Alt-/":"autocomplete",Enter:"newlineAndIndent","Ctrl-J":p(function(t){t.replaceSelection("\n","end")}),Tab:"indentAuto","Alt-G G":function(t){var e=f(t,!0);if(null!=e&&e>0)return t.setCursor(e-1);!function(t,e,n){t.openDialog?t.openDialog(e+': <input type="text" style="width: 10em"/>',n,{bottom:!0}):n(prompt(e,""))}(t,"Goto line",function(e){var n;e&&!isNaN(n=Number(e))&&n==(0|n)&&n>0&&t.setCursor(n-1)})},"Ctrl-X Tab":function(t){t.indentSelection(f(t,!0)||t.getOption("indentUnit"))},"Ctrl-X Ctrl-X":function(t){t.setSelection(t.getCursor("head"),t.getCursor("anchor"))},"Ctrl-X Ctrl-S":"save","Ctrl-X Ctrl-W":"save","Ctrl-X S":"saveAll","Ctrl-X F":"open","Ctrl-X U":p("undo"),"Ctrl-X K":"close","Ctrl-X Delete":function(t){s(t,t.getCursor(),h(t,t.getCursor(),1),"grow")},"Ctrl-X H":"selectAll","Ctrl-Q Tab":p("insertTab"),"Ctrl-U":function(t){t.state.emacsPrefixMap=!0,t.addKeyMap(k),t.on("keyHandled",w),t.on("inputRead",w)}}),k={"Ctrl-G":b};function S(t){k[t]=function(e){E(e,t)},M["Ctrl-"+t]=function(e){E(e,t)},v["Ctrl-"+t]=!0}for(var L=0;L<10;++L)S(String(L));S("-")}(n(15))},function(t,e,n){!function(t){"use strict";var e=t.commands,n=t.Pos;function r(e,r){e.extendSelectionsBy(function(i){return e.display.shift||e.doc.extend||i.empty()?function(e,r,i){if(i<0&&0==r.ch)return e.clipPos(n(r.line-1));var o=e.getLine(r.line);if(i>0&&r.ch>=o.length)return e.clipPos(n(r.line+1,0));for(var a,s="start",A=r.ch,l=i<0?0:o.length,c=0;A!=l;A+=i,c++){var u=o.charAt(i<0?A-1:A),g="_"!=u&&t.isWordChar(u)?"w":"o";if("w"==g&&u.toUpperCase()==u&&(g="W"),"start"==s)"o"!=g&&(s="in",a=g);else if("in"==s&&a!=g){if("w"==a&&"W"==g&&i<0&&A--,"W"==a&&"w"==g&&i>0){a="w";continue}break}}return n(r.line,A)}(e.doc,i.head,r):r<0?i.from():i.to()})}function i(e,r){if(e.isReadOnly())return t.Pass;e.operation(function(){for(var t=e.listSelections().length,i=[],o=-1,a=0;a<t;a++){var s=e.listSelections()[a].head;if(!(s.line<=o)){var A=n(s.line+(r?0:1),0);e.replaceRange("\n",A,null,"+insertLine"),e.indentLine(A.line,null,!0),i.push({head:A,anchor:A}),o=s.line+1}}e.setSelections(i)}),e.execCommand("indentAuto")}function o(e,r){for(var i=r.ch,o=i,a=e.getLine(r.line);i&&t.isWordChar(a.charAt(i-1));)--i;for(;o<a.length&&t.isWordChar(a.charAt(o));)++o;return{from:n(r.line,i),to:n(r.line,o),word:a.slice(i,o)}}function a(t,e){for(var n=t.listSelections(),r=[],i=0;i<n.length;i++){var o=n[i],a=t.findPosV(o.anchor,e,"line",o.anchor.goalColumn),s=t.findPosV(o.head,e,"line",o.head.goalColumn);a.goalColumn=null!=o.anchor.goalColumn?o.anchor.goalColumn:t.cursorCoords(o.anchor,"div").left,s.goalColumn=null!=o.head.goalColumn?o.head.goalColumn:t.cursorCoords(o.head,"div").left;var A={anchor:a,head:s};r.push(o),r.push(A)}t.setSelections(r)}e.goSubwordLeft=function(t){r(t,-1)},e.goSubwordRight=function(t){r(t,1)},e.scrollLineUp=function(t){var e=t.getScrollInfo();if(!t.somethingSelected()){var n=t.lineAtHeight(e.top+e.clientHeight,"local");t.getCursor().line>=n&&t.execCommand("goLineUp")}t.scrollTo(null,e.top-t.defaultTextHeight())},e.scrollLineDown=function(t){var e=t.getScrollInfo();if(!t.somethingSelected()){var n=t.lineAtHeight(e.top,"local")+1;t.getCursor().line<=n&&t.execCommand("goLineDown")}t.scrollTo(null,e.top+t.defaultTextHeight())},e.splitSelectionByLine=function(t){for(var e=t.listSelections(),r=[],i=0;i<e.length;i++)for(var o=e[i].from(),a=e[i].to(),s=o.line;s<=a.line;++s)a.line>o.line&&s==a.line&&0==a.ch||r.push({anchor:s==o.line?o:n(s,0),head:s==a.line?a:n(s)});t.setSelections(r,0)},e.singleSelectionTop=function(t){var e=t.listSelections()[0];t.setSelection(e.anchor,e.head,{scroll:!1})},e.selectLine=function(t){for(var e=t.listSelections(),r=[],i=0;i<e.length;i++){var o=e[i];r.push({anchor:n(o.from().line,0),head:n(o.to().line+1,0)})}t.setSelections(r)},e.insertLineAfter=function(t){return i(t,!1)},e.insertLineBefore=function(t){return i(t,!0)},e.selectNextOccurrence=function(e){var r=e.getCursor("from"),i=e.getCursor("to"),a=e.state.sublimeFindFullWord==e.doc.sel;if(0==t.cmpPos(r,i)){var s=o(e,r);if(!s.word)return;e.setSelection(s.from,s.to),a=!0}else{var A=e.getRange(r,i),l=a?new RegExp("\\b"+A+"\\b"):A,c=e.getSearchCursor(l,i),u=c.findNext();if(u||(c=e.getSearchCursor(l,n(e.firstLine(),0)),u=c.findNext()),!u||function(t,e,n){for(var r=0;r<t.length;r++)if(t[r].from()==e&&t[r].to()==n)return!0;return!1}(e.listSelections(),c.from(),c.to()))return t.Pass;e.addSelection(c.from(),c.to())}a&&(e.state.sublimeFindFullWord=e.doc.sel)},e.addCursorToPrevLine=function(t){a(t,-1)},e.addCursorToNextLine=function(t){a(t,1)};var s="(){}[]";function A(e){for(var r=e.listSelections(),i=[],o=0;o<r.length;o++){var a=r[o],A=a.head,l=e.scanForBracket(A,-1);if(!l)return!1;for(;;){var c=e.scanForBracket(A,1);if(!c)return!1;if(c.ch==s.charAt(s.indexOf(l.ch)+1)){var u=n(l.pos.line,l.pos.ch+1);if(0!=t.cmpPos(u,a.from())||0!=t.cmpPos(c.pos,a.to())){i.push({anchor:u,head:c.pos});break}if(!(l=e.scanForBracket(l.pos,-1)))return!1}A=n(c.pos.line,c.pos.ch+1)}}return e.setSelections(i),!0}function l(e,r){if(e.isReadOnly())return t.Pass;for(var i,o=e.listSelections(),a=[],s=0;s<o.length;s++){var A=o[s];if(!A.empty()){for(var l=A.from().line,c=A.to().line;s<o.length-1&&o[s+1].from().line==c;)c=o[++s].to().line;o[s].to().ch||c--,a.push(l,c)}}a.length?i=!0:a.push(e.firstLine(),e.lastLine()),e.operation(function(){for(var t=[],o=0;o<a.length;o+=2){var s=a[o],A=a[o+1],l=n(s,0),c=n(A),u=e.getRange(l,c,!1);r?u.sort():u.sort(function(t,e){var n=t.toUpperCase(),r=e.toUpperCase();return n!=r&&(t=n,e=r),t<e?-1:t==e?0:1}),e.replaceRange(u,l,c),i&&t.push({anchor:l,head:n(A+1,0)})}i&&e.setSelections(t,0)})}function c(e,n){e.operation(function(){for(var r=e.listSelections(),i=[],a=[],s=0;s<r.length;s++){var A=r[s];A.empty()?(i.push(s),a.push("")):a.push(n(e.getRange(A.from(),A.to())))}e.replaceSelections(a,"around","case");for(var l,s=i.length-1;s>=0;s--){var A=r[i[s]];if(!(l&&t.cmpPos(A.head,l)>0)){var c=o(e,A.head);l=c.from,e.replaceRange(n(c.word),c.from,c.to)}}})}function u(e){var n=e.getCursor("from"),r=e.getCursor("to");if(0==t.cmpPos(n,r)){var i=o(e,n);if(!i.word)return;n=i.from,r=i.to}return{from:n,to:r,query:e.getRange(n,r),word:i}}function g(t,e){var r=u(t);if(r){var i=r.query,o=t.getSearchCursor(i,e?r.to:r.from);(e?o.findNext():o.findPrevious())?t.setSelection(o.from(),o.to()):(o=t.getSearchCursor(i,e?n(t.firstLine(),0):t.clipPos(n(t.lastLine()))),(e?o.findNext():o.findPrevious())?t.setSelection(o.from(),o.to()):r.word&&t.setSelection(r.from,r.to))}}e.selectScope=function(t){A(t)||t.execCommand("selectAll")},e.selectBetweenBrackets=function(e){if(!A(e))return t.Pass},e.goToBracket=function(e){e.extendSelectionsBy(function(r){var i=e.scanForBracket(r.head,1);if(i&&0!=t.cmpPos(i.pos,r.head))return i.pos;var o=e.scanForBracket(r.head,-1);return o&&n(o.pos.line,o.pos.ch+1)||r.head})},e.swapLineUp=function(e){if(e.isReadOnly())return t.Pass;for(var r=e.listSelections(),i=[],o=e.firstLine()-1,a=[],s=0;s<r.length;s++){var A=r[s],l=A.from().line-1,c=A.to().line;a.push({anchor:n(A.anchor.line-1,A.anchor.ch),head:n(A.head.line-1,A.head.ch)}),0!=A.to().ch||A.empty()||--c,l>o?i.push(l,c):i.length&&(i[i.length-1]=c),o=c}e.operation(function(){for(var t=0;t<i.length;t+=2){var r=i[t],o=i[t+1],s=e.getLine(r);e.replaceRange("",n(r,0),n(r+1,0),"+swapLine"),o>e.lastLine()?e.replaceRange("\n"+s,n(e.lastLine()),null,"+swapLine"):e.replaceRange(s+"\n",n(o,0),null,"+swapLine")}e.setSelections(a),e.scrollIntoView()})},e.swapLineDown=function(e){if(e.isReadOnly())return t.Pass;for(var r=e.listSelections(),i=[],o=e.lastLine()+1,a=r.length-1;a>=0;a--){var s=r[a],A=s.to().line+1,l=s.from().line;0!=s.to().ch||s.empty()||A--,A<o?i.push(A,l):i.length&&(i[i.length-1]=l),o=l}e.operation(function(){for(var t=i.length-2;t>=0;t-=2){var r=i[t],o=i[t+1],a=e.getLine(r);r==e.lastLine()?e.replaceRange("",n(r-1),n(r),"+swapLine"):e.replaceRange("",n(r,0),n(r+1,0),"+swapLine"),e.replaceRange(a+"\n",n(o,0),null,"+swapLine")}e.scrollIntoView()})},e.toggleCommentIndented=function(t){t.toggleComment({indent:!0})},e.joinLines=function(t){for(var e=t.listSelections(),r=[],i=0;i<e.length;i++){for(var o=e[i],a=o.from(),s=a.line,A=o.to().line;i<e.length-1&&e[i+1].from().line==A;)A=e[++i].to().line;r.push({start:s,end:A,anchor:!o.empty()&&a})}t.operation(function(){for(var e=0,i=[],o=0;o<r.length;o++){for(var a,s=r[o],A=s.anchor&&n(s.anchor.line-e,s.anchor.ch),l=s.start;l<=s.end;l++){var c=l-e;l==s.end&&(a=n(c,t.getLine(c).length+1)),c<t.lastLine()&&(t.replaceRange(" ",n(c),n(c+1,/^\s*/.exec(t.getLine(c+1))[0].length)),++e)}i.push({anchor:A||a,head:a})}t.setSelections(i,0)})},e.duplicateLine=function(t){t.operation(function(){for(var e=t.listSelections().length,r=0;r<e;r++){var i=t.listSelections()[r];i.empty()?t.replaceRange(t.getLine(i.head.line)+"\n",n(i.head.line,0)):t.replaceRange(t.getRange(i.from(),i.to()),i.from())}t.scrollIntoView()})},e.sortLines=function(t){l(t,!0)},e.sortLinesInsensitive=function(t){l(t,!1)},e.nextBookmark=function(t){var e=t.state.sublimeBookmarks;if(e)for(;e.length;){var n=e.shift(),r=n.find();if(r)return e.push(n),t.setSelection(r.from,r.to)}},e.prevBookmark=function(t){var e=t.state.sublimeBookmarks;if(e)for(;e.length;){e.unshift(e.pop());var n=e[e.length-1].find();if(n)return t.setSelection(n.from,n.to);e.pop()}},e.toggleBookmark=function(t){for(var e=t.listSelections(),n=t.state.sublimeBookmarks||(t.state.sublimeBookmarks=[]),r=0;r<e.length;r++){for(var i=e[r].from(),o=e[r].to(),a=e[r].empty()?t.findMarksAt(i):t.findMarks(i,o),s=0;s<a.length;s++)if(a[s].sublimeBookmark){a[s].clear();for(var A=0;A<n.length;A++)n[A]==a[s]&&n.splice(A--,1);break}s==a.length&&n.push(t.markText(i,o,{sublimeBookmark:!0,clearWhenEmpty:!1}))}},e.clearBookmarks=function(t){var e=t.state.sublimeBookmarks;if(e)for(var n=0;n<e.length;n++)e[n].clear();e.length=0},e.selectBookmarks=function(t){var e=t.state.sublimeBookmarks,n=[];if(e)for(var r=0;r<e.length;r++){var i=e[r].find();i?n.push({anchor:i.from,head:i.to}):e.splice(r--,0)}n.length&&t.setSelections(n,0)},e.smartBackspace=function(e){if(e.somethingSelected())return t.Pass;e.operation(function(){for(var r=e.listSelections(),i=e.getOption("indentUnit"),o=r.length-1;o>=0;o--){var a=r[o].head,s=e.getRange({line:a.line,ch:0},a),A=t.countColumn(s,null,e.getOption("tabSize")),l=e.findPosH(a,-1,"char",!1);if(s&&!/\S/.test(s)&&A%i==0){var c=new n(a.line,t.findColumn(s,A-i,i));c.ch!=a.ch&&(l=c)}e.replaceRange("",l,a,"+delete")}})},e.delLineRight=function(t){t.operation(function(){for(var e=t.listSelections(),r=e.length-1;r>=0;r--)t.replaceRange("",e[r].anchor,n(e[r].to().line),"+delete");t.scrollIntoView()})},e.upcaseAtCursor=function(t){c(t,function(t){return t.toUpperCase()})},e.downcaseAtCursor=function(t){c(t,function(t){return t.toLowerCase()})},e.setSublimeMark=function(t){t.state.sublimeMark&&t.state.sublimeMark.clear(),t.state.sublimeMark=t.setBookmark(t.getCursor())},e.selectToSublimeMark=function(t){var e=t.state.sublimeMark&&t.state.sublimeMark.find();e&&t.setSelection(t.getCursor(),e)},e.deleteToSublimeMark=function(e){var n=e.state.sublimeMark&&e.state.sublimeMark.find();if(n){var r=e.getCursor(),i=n;if(t.cmpPos(r,i)>0){var o=i;i=r,r=o}e.state.sublimeKilled=e.getRange(r,i),e.replaceRange("",r,i)}},e.swapWithSublimeMark=function(t){var e=t.state.sublimeMark&&t.state.sublimeMark.find();e&&(t.state.sublimeMark.clear(),t.state.sublimeMark=t.setBookmark(t.getCursor()),t.setCursor(e))},e.sublimeYank=function(t){null!=t.state.sublimeKilled&&t.replaceSelection(t.state.sublimeKilled,null,"paste")},e.showInCenter=function(t){var e=t.cursorCoords(null,"local");t.scrollTo(null,(e.top+e.bottom)/2-t.getScrollInfo().clientHeight/2)},e.findUnder=function(t){g(t,!0)},e.findUnderPrevious=function(t){g(t,!1)},e.findAllUnder=function(t){var e=u(t);if(e){for(var n=t.getSearchCursor(e.query),r=[],i=-1;n.findNext();)r.push({anchor:n.from(),head:n.to()}),n.from().line<=e.from.line&&n.from().ch<=e.from.ch&&i++;t.setSelections(r,i)}};var h=t.keyMap;h.macSublime={"Cmd-Left":"goLineStartSmart","Shift-Tab":"indentLess","Shift-Ctrl-K":"deleteLine","Alt-Q":"wrapLines","Ctrl-Left":"goSubwordLeft","Ctrl-Right":"goSubwordRight","Ctrl-Alt-Up":"scrollLineUp","Ctrl-Alt-Down":"scrollLineDown","Cmd-L":"selectLine","Shift-Cmd-L":"splitSelectionByLine",Esc:"singleSelectionTop","Cmd-Enter":"insertLineAfter","Shift-Cmd-Enter":"insertLineBefore","Cmd-D":"selectNextOccurrence","Shift-Cmd-Space":"selectScope","Shift-Cmd-M":"selectBetweenBrackets","Cmd-M":"goToBracket","Cmd-Ctrl-Up":"swapLineUp","Cmd-Ctrl-Down":"swapLineDown","Cmd-/":"toggleCommentIndented","Cmd-J":"joinLines","Shift-Cmd-D":"duplicateLine",F5:"sortLines","Cmd-F5":"sortLinesInsensitive",F2:"nextBookmark","Shift-F2":"prevBookmark","Cmd-F2":"toggleBookmark","Shift-Cmd-F2":"clearBookmarks","Alt-F2":"selectBookmarks",Backspace:"smartBackspace","Cmd-K Cmd-K":"delLineRight","Cmd-K Cmd-U":"upcaseAtCursor","Cmd-K Cmd-L":"downcaseAtCursor","Cmd-K Cmd-Space":"setSublimeMark","Cmd-K Cmd-A":"selectToSublimeMark","Cmd-K Cmd-W":"deleteToSublimeMark","Cmd-K Cmd-X":"swapWithSublimeMark","Cmd-K Cmd-Y":"sublimeYank","Cmd-K Cmd-C":"showInCenter","Cmd-K Cmd-G":"clearBookmarks","Cmd-K Cmd-Backspace":"delLineLeft","Cmd-K Cmd-0":"unfoldAll","Cmd-K Cmd-J":"unfoldAll","Ctrl-Shift-Up":"addCursorToPrevLine","Ctrl-Shift-Down":"addCursorToNextLine","Cmd-F3":"findUnder","Shift-Cmd-F3":"findUnderPrevious","Alt-F3":"findAllUnder","Shift-Cmd-[":"fold","Shift-Cmd-]":"unfold","Cmd-I":"findIncremental","Shift-Cmd-I":"findIncrementalReverse","Cmd-H":"replace",F3:"findNext","Shift-F3":"findPrev",fallthrough:"macDefault"},t.normalizeKeyMap(h.macSublime),h.pcSublime={"Shift-Tab":"indentLess","Shift-Ctrl-K":"deleteLine","Alt-Q":"wrapLines","Ctrl-T":"transposeChars","Alt-Left":"goSubwordLeft","Alt-Right":"goSubwordRight","Ctrl-Up":"scrollLineUp","Ctrl-Down":"scrollLineDown","Ctrl-L":"selectLine","Shift-Ctrl-L":"splitSelectionByLine",Esc:"singleSelectionTop","Ctrl-Enter":"insertLineAfter","Shift-Ctrl-Enter":"insertLineBefore","Ctrl-D":"selectNextOccurrence","Shift-Ctrl-Space":"selectScope","Shift-Ctrl-M":"selectBetweenBrackets","Ctrl-M":"goToBracket","Shift-Ctrl-Up":"swapLineUp","Shift-Ctrl-Down":"swapLineDown","Ctrl-/":"toggleCommentIndented","Ctrl-J":"joinLines","Shift-Ctrl-D":"duplicateLine",F9:"sortLines","Ctrl-F9":"sortLinesInsensitive",F2:"nextBookmark","Shift-F2":"prevBookmark","Ctrl-F2":"toggleBookmark","Shift-Ctrl-F2":"clearBookmarks","Alt-F2":"selectBookmarks",Backspace:"smartBackspace","Ctrl-K Ctrl-K":"delLineRight","Ctrl-K Ctrl-U":"upcaseAtCursor","Ctrl-K Ctrl-L":"downcaseAtCursor","Ctrl-K Ctrl-Space":"setSublimeMark","Ctrl-K Ctrl-A":"selectToSublimeMark","Ctrl-K Ctrl-W":"deleteToSublimeMark","Ctrl-K Ctrl-X":"swapWithSublimeMark","Ctrl-K Ctrl-Y":"sublimeYank","Ctrl-K Ctrl-C":"showInCenter","Ctrl-K Ctrl-G":"clearBookmarks","Ctrl-K Ctrl-Backspace":"delLineLeft","Ctrl-K Ctrl-0":"unfoldAll","Ctrl-K Ctrl-J":"unfoldAll","Ctrl-Alt-Up":"addCursorToPrevLine","Ctrl-Alt-Down":"addCursorToNextLine","Ctrl-F3":"findUnder","Shift-Ctrl-F3":"findUnderPrevious","Alt-F3":"findAllUnder","Shift-Ctrl-[":"fold","Shift-Ctrl-]":"unfold","Ctrl-I":"findIncremental","Shift-Ctrl-I":"findIncrementalReverse","Ctrl-H":"replace",F3:"findNext","Shift-F3":"findPrev",fallthrough:"pcDefault"},t.normalizeKeyMap(h.pcSublime);var d=h.default==h.macDefault;h.sublime=d?h.macSublime:h.pcSublime}(n(15),n(293),n(577))},function(t,e,n){"use strict";
/** @license React v16.8.6
 * react.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var r=n(294),i="function"==typeof Symbol&&Symbol.for,o=i?Symbol.for("react.element"):60103,a=i?Symbol.for("react.portal"):60106,s=i?Symbol.for("react.fragment"):60107,A=i?Symbol.for("react.strict_mode"):60108,l=i?Symbol.for("react.profiler"):60114,c=i?Symbol.for("react.provider"):60109,u=i?Symbol.for("react.context"):60110,g=i?Symbol.for("react.concurrent_mode"):60111,h=i?Symbol.for("react.forward_ref"):60112,d=i?Symbol.for("react.suspense"):60113,f=i?Symbol.for("react.memo"):60115,p=i?Symbol.for("react.lazy"):60116,I="function"==typeof Symbol&&Symbol.iterator;function C(t){for(var e=arguments.length-1,n="https://reactjs.org/docs/error-decoder.html?invariant="+t,r=0;r<e;r++)n+="&args[]="+encodeURIComponent(arguments[r+1]);!function(t,e,n,r,i,o,a,s){if(!t){if(t=void 0,void 0===e)t=Error("Minified exception occurred; use the non-minified dev environment for the full error message and additional helpful warnings.");else{var A=[n,r,i,o,a,s],l=0;(t=Error(e.replace(/%s/g,function(){return A[l++]}))).name="Invariant Violation"}throw t.framesToPop=1,t}}(!1,"Minified React error #"+t+"; visit %s for the full message or use the non-minified dev environment for full errors and additional helpful warnings. ",n)}var m={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},y={};function E(t,e,n){this.props=t,this.context=e,this.refs=y,this.updater=n||m}function v(){}function B(t,e,n){this.props=t,this.context=e,this.refs=y,this.updater=n||m}E.prototype.isReactComponent={},E.prototype.setState=function(t,e){"object"!=typeof t&&"function"!=typeof t&&null!=t&&C("85"),this.updater.enqueueSetState(this,t,e,"setState")},E.prototype.forceUpdate=function(t){this.updater.enqueueForceUpdate(this,t,"forceUpdate")},v.prototype=E.prototype;var b=B.prototype=new v;b.constructor=B,r(b,E.prototype),b.isPureReactComponent=!0;var Q={current:null},w={current:null},x=Object.prototype.hasOwnProperty,_={key:!0,ref:!0,__self:!0,__source:!0};function D(t,e,n){var r=void 0,i={},a=null,s=null;if(null!=e)for(r in void 0!==e.ref&&(s=e.ref),void 0!==e.key&&(a=""+e.key),e)x.call(e,r)&&!_.hasOwnProperty(r)&&(i[r]=e[r]);var A=arguments.length-2;if(1===A)i.children=n;else if(1<A){for(var l=Array(A),c=0;c<A;c++)l[c]=arguments[c+2];i.children=l}if(t&&t.defaultProps)for(r in A=t.defaultProps)void 0===i[r]&&(i[r]=A[r]);return{$$typeof:o,type:t,key:a,ref:s,props:i,_owner:w.current}}function M(t){return"object"==typeof t&&null!==t&&t.$$typeof===o}var k=/\/+/g,S=[];function L(t,e,n,r){if(S.length){var i=S.pop();return i.result=t,i.keyPrefix=e,i.func=n,i.context=r,i.count=0,i}return{result:t,keyPrefix:e,func:n,context:r,count:0}}function R(t){t.result=null,t.keyPrefix=null,t.func=null,t.context=null,t.count=0,10>S.length&&S.push(t)}function N(t,e,n){return null==t?0:function t(e,n,r,i){var s=typeof e;"undefined"!==s&&"boolean"!==s||(e=null);var A=!1;if(null===e)A=!0;else switch(s){case"string":case"number":A=!0;break;case"object":switch(e.$$typeof){case o:case a:A=!0}}if(A)return r(i,e,""===n?"."+F(e,0):n),1;if(A=0,n=""===n?".":n+":",Array.isArray(e))for(var l=0;l<e.length;l++){var c=n+F(s=e[l],l);A+=t(s,c,r,i)}else if(c=null===e||"object"!=typeof e?null:"function"==typeof(c=I&&e[I]||e["@@iterator"])?c:null,"function"==typeof c)for(e=c.call(e),l=0;!(s=e.next()).done;)A+=t(s=s.value,c=n+F(s,l++),r,i);else"object"===s&&C("31","[object Object]"==(r=""+e)?"object with keys {"+Object.keys(e).join(", ")+"}":r,"");return A}(t,"",e,n)}function F(t,e){return"object"==typeof t&&null!==t&&null!=t.key?function(t){var e={"=":"=0",":":"=2"};return"$"+(""+t).replace(/[=:]/g,function(t){return e[t]})}(t.key):e.toString(36)}function T(t,e){t.func.call(t.context,e,t.count++)}function U(t,e,n){var r=t.result,i=t.keyPrefix;t=t.func.call(t.context,e,t.count++),Array.isArray(t)?O(t,r,n,function(t){return t}):null!=t&&(M(t)&&(t=function(t,e){return{$$typeof:o,type:t.type,key:e,ref:t.ref,props:t.props,_owner:t._owner}}(t,i+(!t.key||e&&e.key===t.key?"":(""+t.key).replace(k,"$&/")+"/")+n)),r.push(t))}function O(t,e,n,r,i){var o="";null!=n&&(o=(""+n).replace(k,"$&/")+"/"),N(t,U,e=L(e,o,r,i)),R(e)}function G(){var t=Q.current;return null===t&&C("321"),t}var H={Children:{map:function(t,e,n){if(null==t)return t;var r=[];return O(t,r,null,e,n),r},forEach:function(t,e,n){if(null==t)return t;N(t,T,e=L(null,null,e,n)),R(e)},count:function(t){return N(t,function(){return null},null)},toArray:function(t){var e=[];return O(t,e,null,function(t){return t}),e},only:function(t){return M(t)||C("143"),t}},createRef:function(){return{current:null}},Component:E,PureComponent:B,createContext:function(t,e){return void 0===e&&(e=null),(t={$$typeof:u,_calculateChangedBits:e,_currentValue:t,_currentValue2:t,_threadCount:0,Provider:null,Consumer:null}).Provider={$$typeof:c,_context:t},t.Consumer=t},forwardRef:function(t){return{$$typeof:h,render:t}},lazy:function(t){return{$$typeof:p,_ctor:t,_status:-1,_result:null}},memo:function(t,e){return{$$typeof:f,type:t,compare:void 0===e?null:e}},useCallback:function(t,e){return G().useCallback(t,e)},useContext:function(t,e){return G().useContext(t,e)},useEffect:function(t,e){return G().useEffect(t,e)},useImperativeHandle:function(t,e,n){return G().useImperativeHandle(t,e,n)},useDebugValue:function(){},useLayoutEffect:function(t,e){return G().useLayoutEffect(t,e)},useMemo:function(t,e){return G().useMemo(t,e)},useReducer:function(t,e,n){return G().useReducer(t,e,n)},useRef:function(t){return G().useRef(t)},useState:function(t){return G().useState(t)},Fragment:s,StrictMode:A,Suspense:d,createElement:D,cloneElement:function(t,e,n){null==t&&C("267",t);var i=void 0,a=r({},t.props),s=t.key,A=t.ref,l=t._owner;if(null!=e){void 0!==e.ref&&(A=e.ref,l=w.current),void 0!==e.key&&(s=""+e.key);var c=void 0;for(i in t.type&&t.type.defaultProps&&(c=t.type.defaultProps),e)x.call(e,i)&&!_.hasOwnProperty(i)&&(a[i]=void 0===e[i]&&void 0!==c?c[i]:e[i])}if(1===(i=arguments.length-2))a.children=n;else if(1<i){c=Array(i);for(var u=0;u<i;u++)c[u]=arguments[u+2];a.children=c}return{$$typeof:o,type:t.type,key:s,ref:A,props:a,_owner:l}},createFactory:function(t){var e=D.bind(null,t);return e.type=t,e},isValidElement:M,version:"16.8.6",unstable_ConcurrentMode:g,unstable_Profiler:l,__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED:{ReactCurrentDispatcher:Q,ReactCurrentOwner:w,assign:r}},j={default:H},Y=j&&H||j;t.exports=Y.default||Y},function(t,e,n){"use strict";
/** @license React v16.8.6
 * react-dom.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var r=n(0),i=n(294),o=n(1064);function a(t){for(var e=arguments.length-1,n="https://reactjs.org/docs/error-decoder.html?invariant="+t,r=0;r<e;r++)n+="&args[]="+encodeURIComponent(arguments[r+1]);!function(t,e,n,r,i,o,a,s){if(!t){if(t=void 0,void 0===e)t=Error("Minified exception occurred; use the non-minified dev environment for the full error message and additional helpful warnings.");else{var A=[n,r,i,o,a,s],l=0;(t=Error(e.replace(/%s/g,function(){return A[l++]}))).name="Invariant Violation"}throw t.framesToPop=1,t}}(!1,"Minified React error #"+t+"; visit %s for the full message or use the non-minified dev environment for full errors and additional helpful warnings. ",n)}r||a("227");var s=!1,A=null,l=!1,c=null,u={onError:function(t){s=!0,A=t}};function g(t,e,n,r,i,o,a,l,c){s=!1,A=null,function(t,e,n,r,i,o,a,s,A){var l=Array.prototype.slice.call(arguments,3);try{e.apply(n,l)}catch(t){this.onError(t)}}.apply(u,arguments)}var h=null,d={};function f(){if(h)for(var t in d){var e=d[t],n=h.indexOf(t);if(-1<n||a("96",t),!I[n])for(var r in e.extractEvents||a("97",t),I[n]=e,n=e.eventTypes){var i=void 0,o=n[r],s=e,A=r;C.hasOwnProperty(A)&&a("99",A),C[A]=o;var l=o.phasedRegistrationNames;if(l){for(i in l)l.hasOwnProperty(i)&&p(l[i],s,A);i=!0}else o.registrationName?(p(o.registrationName,s,A),i=!0):i=!1;i||a("98",r,t)}}}function p(t,e,n){m[t]&&a("100",t),m[t]=e,y[t]=e.eventTypes[n].dependencies}var I=[],C={},m={},y={},E=null,v=null,B=null;function b(t,e,n){var r=t.type||"unknown-event";t.currentTarget=B(n),function(t,e,n,r,i,o,u,h,d){if(g.apply(this,arguments),s){if(s){var f=A;s=!1,A=null}else a("198"),f=void 0;l||(l=!0,c=f)}}(r,e,void 0,t),t.currentTarget=null}function Q(t,e){return null==e&&a("30"),null==t?e:Array.isArray(t)?Array.isArray(e)?(t.push.apply(t,e),t):(t.push(e),t):Array.isArray(e)?[t].concat(e):[t,e]}function w(t,e,n){Array.isArray(t)?t.forEach(e,n):t&&e.call(n,t)}var x=null;function _(t){if(t){var e=t._dispatchListeners,n=t._dispatchInstances;if(Array.isArray(e))for(var r=0;r<e.length&&!t.isPropagationStopped();r++)b(t,e[r],n[r]);else e&&b(t,e,n);t._dispatchListeners=null,t._dispatchInstances=null,t.isPersistent()||t.constructor.release(t)}}var D={injectEventPluginOrder:function(t){h&&a("101"),h=Array.prototype.slice.call(t),f()},injectEventPluginsByName:function(t){var e,n=!1;for(e in t)if(t.hasOwnProperty(e)){var r=t[e];d.hasOwnProperty(e)&&d[e]===r||(d[e]&&a("102",e),d[e]=r,n=!0)}n&&f()}};function M(t,e){var n=t.stateNode;if(!n)return null;var r=E(n);if(!r)return null;n=r[e];t:switch(e){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":(r=!r.disabled)||(r=!("button"===(t=t.type)||"input"===t||"select"===t||"textarea"===t)),t=!r;break t;default:t=!1}return t?null:(n&&"function"!=typeof n&&a("231",e,typeof n),n)}function k(t){if(null!==t&&(x=Q(x,t)),t=x,x=null,t&&(w(t,_),x&&a("95"),l))throw t=c,l=!1,c=null,t}var S=Math.random().toString(36).slice(2),L="__reactInternalInstance$"+S,R="__reactEventHandlers$"+S;function N(t){if(t[L])return t[L];for(;!t[L];){if(!t.parentNode)return null;t=t.parentNode}return 5===(t=t[L]).tag||6===t.tag?t:null}function F(t){return!(t=t[L])||5!==t.tag&&6!==t.tag?null:t}function T(t){if(5===t.tag||6===t.tag)return t.stateNode;a("33")}function U(t){return t[R]||null}function O(t){do{t=t.return}while(t&&5!==t.tag);return t||null}function G(t,e,n){(e=M(t,n.dispatchConfig.phasedRegistrationNames[e]))&&(n._dispatchListeners=Q(n._dispatchListeners,e),n._dispatchInstances=Q(n._dispatchInstances,t))}function H(t){if(t&&t.dispatchConfig.phasedRegistrationNames){for(var e=t._targetInst,n=[];e;)n.push(e),e=O(e);for(e=n.length;0<e--;)G(n[e],"captured",t);for(e=0;e<n.length;e++)G(n[e],"bubbled",t)}}function j(t,e,n){t&&n&&n.dispatchConfig.registrationName&&(e=M(t,n.dispatchConfig.registrationName))&&(n._dispatchListeners=Q(n._dispatchListeners,e),n._dispatchInstances=Q(n._dispatchInstances,t))}function Y(t){t&&t.dispatchConfig.registrationName&&j(t._targetInst,null,t)}function P(t){w(t,H)}var K=!("undefined"==typeof window||!window.document||!window.document.createElement);function z(t,e){var n={};return n[t.toLowerCase()]=e.toLowerCase(),n["Webkit"+t]="webkit"+e,n["Moz"+t]="moz"+e,n}var J={animationend:z("Animation","AnimationEnd"),animationiteration:z("Animation","AnimationIteration"),animationstart:z("Animation","AnimationStart"),transitionend:z("Transition","TransitionEnd")},q={},V={};function W(t){if(q[t])return q[t];if(!J[t])return t;var e,n=J[t];for(e in n)if(n.hasOwnProperty(e)&&e in V)return q[t]=n[e];return t}K&&(V=document.createElement("div").style,"AnimationEvent"in window||(delete J.animationend.animation,delete J.animationiteration.animation,delete J.animationstart.animation),"TransitionEvent"in window||delete J.transitionend.transition);var X=W("animationend"),Z=W("animationiteration"),$=W("animationstart"),tt=W("transitionend"),et="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),nt=null,rt=null,it=null;function ot(){if(it)return it;var t,e,n=rt,r=n.length,i="value"in nt?nt.value:nt.textContent,o=i.length;for(t=0;t<r&&n[t]===i[t];t++);var a=r-t;for(e=1;e<=a&&n[r-e]===i[o-e];e++);return it=i.slice(t,1<e?1-e:void 0)}function at(){return!0}function st(){return!1}function At(t,e,n,r){for(var i in this.dispatchConfig=t,this._targetInst=e,this.nativeEvent=n,t=this.constructor.Interface)t.hasOwnProperty(i)&&((e=t[i])?this[i]=e(n):"target"===i?this.target=r:this[i]=n[i]);return this.isDefaultPrevented=(null!=n.defaultPrevented?n.defaultPrevented:!1===n.returnValue)?at:st,this.isPropagationStopped=st,this}function lt(t,e,n,r){if(this.eventPool.length){var i=this.eventPool.pop();return this.call(i,t,e,n,r),i}return new this(t,e,n,r)}function ct(t){t instanceof this||a("279"),t.destructor(),10>this.eventPool.length&&this.eventPool.push(t)}function ut(t){t.eventPool=[],t.getPooled=lt,t.release=ct}i(At.prototype,{preventDefault:function(){this.defaultPrevented=!0;var t=this.nativeEvent;t&&(t.preventDefault?t.preventDefault():"unknown"!=typeof t.returnValue&&(t.returnValue=!1),this.isDefaultPrevented=at)},stopPropagation:function(){var t=this.nativeEvent;t&&(t.stopPropagation?t.stopPropagation():"unknown"!=typeof t.cancelBubble&&(t.cancelBubble=!0),this.isPropagationStopped=at)},persist:function(){this.isPersistent=at},isPersistent:st,destructor:function(){var t,e=this.constructor.Interface;for(t in e)this[t]=null;this.nativeEvent=this._targetInst=this.dispatchConfig=null,this.isPropagationStopped=this.isDefaultPrevented=st,this._dispatchInstances=this._dispatchListeners=null}}),At.Interface={type:null,target:null,currentTarget:function(){return null},eventPhase:null,bubbles:null,cancelable:null,timeStamp:function(t){return t.timeStamp||Date.now()},defaultPrevented:null,isTrusted:null},At.extend=function(t){function e(){}function n(){return r.apply(this,arguments)}var r=this;e.prototype=r.prototype;var o=new e;return i(o,n.prototype),n.prototype=o,n.prototype.constructor=n,n.Interface=i({},r.Interface,t),n.extend=r.extend,ut(n),n},ut(At);var gt=At.extend({data:null}),ht=At.extend({data:null}),dt=[9,13,27,32],ft=K&&"CompositionEvent"in window,pt=null;K&&"documentMode"in document&&(pt=document.documentMode);var It=K&&"TextEvent"in window&&!pt,Ct=K&&(!ft||pt&&8<pt&&11>=pt),mt=String.fromCharCode(32),yt={beforeInput:{phasedRegistrationNames:{bubbled:"onBeforeInput",captured:"onBeforeInputCapture"},dependencies:["compositionend","keypress","textInput","paste"]},compositionEnd:{phasedRegistrationNames:{bubbled:"onCompositionEnd",captured:"onCompositionEndCapture"},dependencies:"blur compositionend keydown keypress keyup mousedown".split(" ")},compositionStart:{phasedRegistrationNames:{bubbled:"onCompositionStart",captured:"onCompositionStartCapture"},dependencies:"blur compositionstart keydown keypress keyup mousedown".split(" ")},compositionUpdate:{phasedRegistrationNames:{bubbled:"onCompositionUpdate",captured:"onCompositionUpdateCapture"},dependencies:"blur compositionupdate keydown keypress keyup mousedown".split(" ")}},Et=!1;function vt(t,e){switch(t){case"keyup":return-1!==dt.indexOf(e.keyCode);case"keydown":return 229!==e.keyCode;case"keypress":case"mousedown":case"blur":return!0;default:return!1}}function Bt(t){return"object"==typeof(t=t.detail)&&"data"in t?t.data:null}var bt=!1;var Qt={eventTypes:yt,extractEvents:function(t,e,n,r){var i=void 0,o=void 0;if(ft)t:{switch(t){case"compositionstart":i=yt.compositionStart;break t;case"compositionend":i=yt.compositionEnd;break t;case"compositionupdate":i=yt.compositionUpdate;break t}i=void 0}else bt?vt(t,n)&&(i=yt.compositionEnd):"keydown"===t&&229===n.keyCode&&(i=yt.compositionStart);return i?(Ct&&"ko"!==n.locale&&(bt||i!==yt.compositionStart?i===yt.compositionEnd&&bt&&(o=ot()):(rt="value"in(nt=r)?nt.value:nt.textContent,bt=!0)),i=gt.getPooled(i,e,n,r),o?i.data=o:null!==(o=Bt(n))&&(i.data=o),P(i),o=i):o=null,(t=It?function(t,e){switch(t){case"compositionend":return Bt(e);case"keypress":return 32!==e.which?null:(Et=!0,mt);case"textInput":return(t=e.data)===mt&&Et?null:t;default:return null}}(t,n):function(t,e){if(bt)return"compositionend"===t||!ft&&vt(t,e)?(t=ot(),it=rt=nt=null,bt=!1,t):null;switch(t){case"paste":return null;case"keypress":if(!(e.ctrlKey||e.altKey||e.metaKey)||e.ctrlKey&&e.altKey){if(e.char&&1<e.char.length)return e.char;if(e.which)return String.fromCharCode(e.which)}return null;case"compositionend":return Ct&&"ko"!==e.locale?null:e.data;default:return null}}(t,n))?((e=ht.getPooled(yt.beforeInput,e,n,r)).data=t,P(e)):e=null,null===o?e:null===e?o:[o,e]}},wt=null,xt=null,_t=null;function Dt(t){if(t=v(t)){"function"!=typeof wt&&a("280");var e=E(t.stateNode);wt(t.stateNode,t.type,e)}}function Mt(t){xt?_t?_t.push(t):_t=[t]:xt=t}function kt(){if(xt){var t=xt,e=_t;if(_t=xt=null,Dt(t),e)for(t=0;t<e.length;t++)Dt(e[t])}}function St(t,e){return t(e)}function Lt(t,e,n){return t(e,n)}function Rt(){}var Nt=!1;function Ft(t,e){if(Nt)return t(e);Nt=!0;try{return St(t,e)}finally{Nt=!1,(null!==xt||null!==_t)&&(Rt(),kt())}}var Tt={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function Ut(t){var e=t&&t.nodeName&&t.nodeName.toLowerCase();return"input"===e?!!Tt[t.type]:"textarea"===e}function Ot(t){return(t=t.target||t.srcElement||window).correspondingUseElement&&(t=t.correspondingUseElement),3===t.nodeType?t.parentNode:t}function Gt(t){if(!K)return!1;var e=(t="on"+t)in document;return e||((e=document.createElement("div")).setAttribute(t,"return;"),e="function"==typeof e[t]),e}function Ht(t){var e=t.type;return(t=t.nodeName)&&"input"===t.toLowerCase()&&("checkbox"===e||"radio"===e)}function jt(t){t._valueTracker||(t._valueTracker=function(t){var e=Ht(t)?"checked":"value",n=Object.getOwnPropertyDescriptor(t.constructor.prototype,e),r=""+t[e];if(!t.hasOwnProperty(e)&&void 0!==n&&"function"==typeof n.get&&"function"==typeof n.set){var i=n.get,o=n.set;return Object.defineProperty(t,e,{configurable:!0,get:function(){return i.call(this)},set:function(t){r=""+t,o.call(this,t)}}),Object.defineProperty(t,e,{enumerable:n.enumerable}),{getValue:function(){return r},setValue:function(t){r=""+t},stopTracking:function(){t._valueTracker=null,delete t[e]}}}}(t))}function Yt(t){if(!t)return!1;var e=t._valueTracker;if(!e)return!0;var n=e.getValue(),r="";return t&&(r=Ht(t)?t.checked?"true":"false":t.value),(t=r)!==n&&(e.setValue(t),!0)}var Pt=r.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED;Pt.hasOwnProperty("ReactCurrentDispatcher")||(Pt.ReactCurrentDispatcher={current:null});var Kt=/^(.*)[\\\/]/,zt="function"==typeof Symbol&&Symbol.for,Jt=zt?Symbol.for("react.element"):60103,qt=zt?Symbol.for("react.portal"):60106,Vt=zt?Symbol.for("react.fragment"):60107,Wt=zt?Symbol.for("react.strict_mode"):60108,Xt=zt?Symbol.for("react.profiler"):60114,Zt=zt?Symbol.for("react.provider"):60109,$t=zt?Symbol.for("react.context"):60110,te=zt?Symbol.for("react.concurrent_mode"):60111,ee=zt?Symbol.for("react.forward_ref"):60112,ne=zt?Symbol.for("react.suspense"):60113,re=zt?Symbol.for("react.memo"):60115,ie=zt?Symbol.for("react.lazy"):60116,oe="function"==typeof Symbol&&Symbol.iterator;function ae(t){return null===t||"object"!=typeof t?null:"function"==typeof(t=oe&&t[oe]||t["@@iterator"])?t:null}function se(t){if(null==t)return null;if("function"==typeof t)return t.displayName||t.name||null;if("string"==typeof t)return t;switch(t){case te:return"ConcurrentMode";case Vt:return"Fragment";case qt:return"Portal";case Xt:return"Profiler";case Wt:return"StrictMode";case ne:return"Suspense"}if("object"==typeof t)switch(t.$$typeof){case $t:return"Context.Consumer";case Zt:return"Context.Provider";case ee:var e=t.render;return e=e.displayName||e.name||"",t.displayName||(""!==e?"ForwardRef("+e+")":"ForwardRef");case re:return se(t.type);case ie:if(t=1===t._status?t._result:null)return se(t)}return null}function Ae(t){var e="";do{t:switch(t.tag){case 3:case 4:case 6:case 7:case 10:case 9:var n="";break t;default:var r=t._debugOwner,i=t._debugSource,o=se(t.type);n=null,r&&(n=se(r.type)),r=o,o="",i?o=" (at "+i.fileName.replace(Kt,"")+":"+i.lineNumber+")":n&&(o=" (created by "+n+")"),n="\n    in "+(r||"Unknown")+o}e+=n,t=t.return}while(t);return e}var le=/^[:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD][:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\-.0-9\u00B7\u0300-\u036F\u203F-\u2040]*$/,ce=Object.prototype.hasOwnProperty,ue={},ge={};function he(t,e,n,r,i){this.acceptsBooleans=2===e||3===e||4===e,this.attributeName=r,this.attributeNamespace=i,this.mustUseProperty=n,this.propertyName=t,this.type=e}var de={};"children dangerouslySetInnerHTML defaultValue defaultChecked innerHTML suppressContentEditableWarning suppressHydrationWarning style".split(" ").forEach(function(t){de[t]=new he(t,0,!1,t,null)}),[["acceptCharset","accept-charset"],["className","class"],["htmlFor","for"],["httpEquiv","http-equiv"]].forEach(function(t){var e=t[0];de[e]=new he(e,1,!1,t[1],null)}),["contentEditable","draggable","spellCheck","value"].forEach(function(t){de[t]=new he(t,2,!1,t.toLowerCase(),null)}),["autoReverse","externalResourcesRequired","focusable","preserveAlpha"].forEach(function(t){de[t]=new he(t,2,!1,t,null)}),"allowFullScreen async autoFocus autoPlay controls default defer disabled formNoValidate hidden loop noModule noValidate open playsInline readOnly required reversed scoped seamless itemScope".split(" ").forEach(function(t){de[t]=new he(t,3,!1,t.toLowerCase(),null)}),["checked","multiple","muted","selected"].forEach(function(t){de[t]=new he(t,3,!0,t,null)}),["capture","download"].forEach(function(t){de[t]=new he(t,4,!1,t,null)}),["cols","rows","size","span"].forEach(function(t){de[t]=new he(t,6,!1,t,null)}),["rowSpan","start"].forEach(function(t){de[t]=new he(t,5,!1,t.toLowerCase(),null)});var fe=/[\-:]([a-z])/g;function pe(t){return t[1].toUpperCase()}function Ie(t,e,n,r){var i=de.hasOwnProperty(e)?de[e]:null;(null!==i?0===i.type:!r&&(2<e.length&&("o"===e[0]||"O"===e[0])&&("n"===e[1]||"N"===e[1])))||(function(t,e,n,r){if(null==e||function(t,e,n,r){if(null!==n&&0===n.type)return!1;switch(typeof e){case"function":case"symbol":return!0;case"boolean":return!r&&(null!==n?!n.acceptsBooleans:"data-"!==(t=t.toLowerCase().slice(0,5))&&"aria-"!==t);default:return!1}}(t,e,n,r))return!0;if(r)return!1;if(null!==n)switch(n.type){case 3:return!e;case 4:return!1===e;case 5:return isNaN(e);case 6:return isNaN(e)||1>e}return!1}(e,n,i,r)&&(n=null),r||null===i?function(t){return!!ce.call(ge,t)||!ce.call(ue,t)&&(le.test(t)?ge[t]=!0:(ue[t]=!0,!1))}(e)&&(null===n?t.removeAttribute(e):t.setAttribute(e,""+n)):i.mustUseProperty?t[i.propertyName]=null===n?3!==i.type&&"":n:(e=i.attributeName,r=i.attributeNamespace,null===n?t.removeAttribute(e):(n=3===(i=i.type)||4===i&&!0===n?"":""+n,r?t.setAttributeNS(r,e,n):t.setAttribute(e,n))))}function Ce(t){switch(typeof t){case"boolean":case"number":case"object":case"string":case"undefined":return t;default:return""}}function me(t,e){var n=e.checked;return i({},e,{defaultChecked:void 0,defaultValue:void 0,value:void 0,checked:null!=n?n:t._wrapperState.initialChecked})}function ye(t,e){var n=null==e.defaultValue?"":e.defaultValue,r=null!=e.checked?e.checked:e.defaultChecked;n=Ce(null!=e.value?e.value:n),t._wrapperState={initialChecked:r,initialValue:n,controlled:"checkbox"===e.type||"radio"===e.type?null!=e.checked:null!=e.value}}function Ee(t,e){null!=(e=e.checked)&&Ie(t,"checked",e,!1)}function ve(t,e){Ee(t,e);var n=Ce(e.value),r=e.type;if(null!=n)"number"===r?(0===n&&""===t.value||t.value!=n)&&(t.value=""+n):t.value!==""+n&&(t.value=""+n);else if("submit"===r||"reset"===r)return void t.removeAttribute("value");e.hasOwnProperty("value")?be(t,e.type,n):e.hasOwnProperty("defaultValue")&&be(t,e.type,Ce(e.defaultValue)),null==e.checked&&null!=e.defaultChecked&&(t.defaultChecked=!!e.defaultChecked)}function Be(t,e,n){if(e.hasOwnProperty("value")||e.hasOwnProperty("defaultValue")){var r=e.type;if(!("submit"!==r&&"reset"!==r||void 0!==e.value&&null!==e.value))return;e=""+t._wrapperState.initialValue,n||e===t.value||(t.value=e),t.defaultValue=e}""!==(n=t.name)&&(t.name=""),t.defaultChecked=!t.defaultChecked,t.defaultChecked=!!t._wrapperState.initialChecked,""!==n&&(t.name=n)}function be(t,e,n){"number"===e&&t.ownerDocument.activeElement===t||(null==n?t.defaultValue=""+t._wrapperState.initialValue:t.defaultValue!==""+n&&(t.defaultValue=""+n))}"accent-height alignment-baseline arabic-form baseline-shift cap-height clip-path clip-rule color-interpolation color-interpolation-filters color-profile color-rendering dominant-baseline enable-background fill-opacity fill-rule flood-color flood-opacity font-family font-size font-size-adjust font-stretch font-style font-variant font-weight glyph-name glyph-orientation-horizontal glyph-orientation-vertical horiz-adv-x horiz-origin-x image-rendering letter-spacing lighting-color marker-end marker-mid marker-start overline-position overline-thickness paint-order panose-1 pointer-events rendering-intent shape-rendering stop-color stop-opacity strikethrough-position strikethrough-thickness stroke-dasharray stroke-dashoffset stroke-linecap stroke-linejoin stroke-miterlimit stroke-opacity stroke-width text-anchor text-decoration text-rendering underline-position underline-thickness unicode-bidi unicode-range units-per-em v-alphabetic v-hanging v-ideographic v-mathematical vector-effect vert-adv-y vert-origin-x vert-origin-y word-spacing writing-mode xmlns:xlink x-height".split(" ").forEach(function(t){var e=t.replace(fe,pe);de[e]=new he(e,1,!1,t,null)}),"xlink:actuate xlink:arcrole xlink:href xlink:role xlink:show xlink:title xlink:type".split(" ").forEach(function(t){var e=t.replace(fe,pe);de[e]=new he(e,1,!1,t,"http://www.w3.org/1999/xlink")}),["xml:base","xml:lang","xml:space"].forEach(function(t){var e=t.replace(fe,pe);de[e]=new he(e,1,!1,t,"http://www.w3.org/XML/1998/namespace")}),["tabIndex","crossOrigin"].forEach(function(t){de[t]=new he(t,1,!1,t.toLowerCase(),null)});var Qe={change:{phasedRegistrationNames:{bubbled:"onChange",captured:"onChangeCapture"},dependencies:"blur change click focus input keydown keyup selectionchange".split(" ")}};function we(t,e,n){return(t=At.getPooled(Qe.change,t,e,n)).type="change",Mt(n),P(t),t}var xe=null,_e=null;function De(t){k(t)}function Me(t){if(Yt(T(t)))return t}function ke(t,e){if("change"===t)return e}var Se=!1;function Le(){xe&&(xe.detachEvent("onpropertychange",Re),_e=xe=null)}function Re(t){"value"===t.propertyName&&Me(_e)&&Ft(De,t=we(_e,t,Ot(t)))}function Ne(t,e,n){"focus"===t?(Le(),_e=n,(xe=e).attachEvent("onpropertychange",Re)):"blur"===t&&Le()}function Fe(t){if("selectionchange"===t||"keyup"===t||"keydown"===t)return Me(_e)}function Te(t,e){if("click"===t)return Me(e)}function Ue(t,e){if("input"===t||"change"===t)return Me(e)}K&&(Se=Gt("input")&&(!document.documentMode||9<document.documentMode));var Oe={eventTypes:Qe,_isInputEventSupported:Se,extractEvents:function(t,e,n,r){var i=e?T(e):window,o=void 0,a=void 0,s=i.nodeName&&i.nodeName.toLowerCase();if("select"===s||"input"===s&&"file"===i.type?o=ke:Ut(i)?Se?o=Ue:(o=Fe,a=Ne):(s=i.nodeName)&&"input"===s.toLowerCase()&&("checkbox"===i.type||"radio"===i.type)&&(o=Te),o&&(o=o(t,e)))return we(o,n,r);a&&a(t,i,e),"blur"===t&&(t=i._wrapperState)&&t.controlled&&"number"===i.type&&be(i,"number",i.value)}},Ge=At.extend({view:null,detail:null}),He={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function je(t){var e=this.nativeEvent;return e.getModifierState?e.getModifierState(t):!!(t=He[t])&&!!e[t]}function Ye(){return je}var Pe=0,Ke=0,ze=!1,Je=!1,qe=Ge.extend({screenX:null,screenY:null,clientX:null,clientY:null,pageX:null,pageY:null,ctrlKey:null,shiftKey:null,altKey:null,metaKey:null,getModifierState:Ye,button:null,buttons:null,relatedTarget:function(t){return t.relatedTarget||(t.fromElement===t.srcElement?t.toElement:t.fromElement)},movementX:function(t){if("movementX"in t)return t.movementX;var e=Pe;return Pe=t.screenX,ze?"mousemove"===t.type?t.screenX-e:0:(ze=!0,0)},movementY:function(t){if("movementY"in t)return t.movementY;var e=Ke;return Ke=t.screenY,Je?"mousemove"===t.type?t.screenY-e:0:(Je=!0,0)}}),Ve=qe.extend({pointerId:null,width:null,height:null,pressure:null,tangentialPressure:null,tiltX:null,tiltY:null,twist:null,pointerType:null,isPrimary:null}),We={mouseEnter:{registrationName:"onMouseEnter",dependencies:["mouseout","mouseover"]},mouseLeave:{registrationName:"onMouseLeave",dependencies:["mouseout","mouseover"]},pointerEnter:{registrationName:"onPointerEnter",dependencies:["pointerout","pointerover"]},pointerLeave:{registrationName:"onPointerLeave",dependencies:["pointerout","pointerover"]}},Xe={eventTypes:We,extractEvents:function(t,e,n,r){var i="mouseover"===t||"pointerover"===t,o="mouseout"===t||"pointerout"===t;if(i&&(n.relatedTarget||n.fromElement)||!o&&!i)return null;if(i=r.window===r?r:(i=r.ownerDocument)?i.defaultView||i.parentWindow:window,o?(o=e,e=(e=n.relatedTarget||n.toElement)?N(e):null):o=null,o===e)return null;var a=void 0,s=void 0,A=void 0,l=void 0;"mouseout"===t||"mouseover"===t?(a=qe,s=We.mouseLeave,A=We.mouseEnter,l="mouse"):"pointerout"!==t&&"pointerover"!==t||(a=Ve,s=We.pointerLeave,A=We.pointerEnter,l="pointer");var c=null==o?i:T(o);if(i=null==e?i:T(e),(t=a.getPooled(s,o,n,r)).type=l+"leave",t.target=c,t.relatedTarget=i,(n=a.getPooled(A,e,n,r)).type=l+"enter",n.target=i,n.relatedTarget=c,r=e,o&&r)t:{for(i=r,l=0,a=e=o;a;a=O(a))l++;for(a=0,A=i;A;A=O(A))a++;for(;0<l-a;)e=O(e),l--;for(;0<a-l;)i=O(i),a--;for(;l--;){if(e===i||e===i.alternate)break t;e=O(e),i=O(i)}e=null}else e=null;for(i=e,e=[];o&&o!==i&&(null===(l=o.alternate)||l!==i);)e.push(o),o=O(o);for(o=[];r&&r!==i&&(null===(l=r.alternate)||l!==i);)o.push(r),r=O(r);for(r=0;r<e.length;r++)j(e[r],"bubbled",t);for(r=o.length;0<r--;)j(o[r],"captured",n);return[t,n]}};function Ze(t,e){return t===e&&(0!==t||1/t==1/e)||t!=t&&e!=e}var $e=Object.prototype.hasOwnProperty;function tn(t,e){if(Ze(t,e))return!0;if("object"!=typeof t||null===t||"object"!=typeof e||null===e)return!1;var n=Object.keys(t),r=Object.keys(e);if(n.length!==r.length)return!1;for(r=0;r<n.length;r++)if(!$e.call(e,n[r])||!Ze(t[n[r]],e[n[r]]))return!1;return!0}function en(t){var e=t;if(t.alternate)for(;e.return;)e=e.return;else{if(0!=(2&e.effectTag))return 1;for(;e.return;)if(0!=(2&(e=e.return).effectTag))return 1}return 3===e.tag?2:3}function nn(t){2!==en(t)&&a("188")}function rn(t){if(!(t=function(t){var e=t.alternate;if(!e)return 3===(e=en(t))&&a("188"),1===e?null:t;for(var n=t,r=e;;){var i=n.return,o=i?i.alternate:null;if(!i||!o)break;if(i.child===o.child){for(var s=i.child;s;){if(s===n)return nn(i),t;if(s===r)return nn(i),e;s=s.sibling}a("188")}if(n.return!==r.return)n=i,r=o;else{s=!1;for(var A=i.child;A;){if(A===n){s=!0,n=i,r=o;break}if(A===r){s=!0,r=i,n=o;break}A=A.sibling}if(!s){for(A=o.child;A;){if(A===n){s=!0,n=o,r=i;break}if(A===r){s=!0,r=o,n=i;break}A=A.sibling}s||a("189")}}n.alternate!==r&&a("190")}return 3!==n.tag&&a("188"),n.stateNode.current===n?t:e}(t)))return null;for(var e=t;;){if(5===e.tag||6===e.tag)return e;if(e.child)e.child.return=e,e=e.child;else{if(e===t)break;for(;!e.sibling;){if(!e.return||e.return===t)return null;e=e.return}e.sibling.return=e.return,e=e.sibling}}return null}var on=At.extend({animationName:null,elapsedTime:null,pseudoElement:null}),an=At.extend({clipboardData:function(t){return"clipboardData"in t?t.clipboardData:window.clipboardData}}),sn=Ge.extend({relatedTarget:null});function An(t){var e=t.keyCode;return"charCode"in t?0===(t=t.charCode)&&13===e&&(t=13):t=e,10===t&&(t=13),32<=t||13===t?t:0}var ln={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},cn={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},un=Ge.extend({key:function(t){if(t.key){var e=ln[t.key]||t.key;if("Unidentified"!==e)return e}return"keypress"===t.type?13===(t=An(t))?"Enter":String.fromCharCode(t):"keydown"===t.type||"keyup"===t.type?cn[t.keyCode]||"Unidentified":""},location:null,ctrlKey:null,shiftKey:null,altKey:null,metaKey:null,repeat:null,locale:null,getModifierState:Ye,charCode:function(t){return"keypress"===t.type?An(t):0},keyCode:function(t){return"keydown"===t.type||"keyup"===t.type?t.keyCode:0},which:function(t){return"keypress"===t.type?An(t):"keydown"===t.type||"keyup"===t.type?t.keyCode:0}}),gn=qe.extend({dataTransfer:null}),hn=Ge.extend({touches:null,targetTouches:null,changedTouches:null,altKey:null,metaKey:null,ctrlKey:null,shiftKey:null,getModifierState:Ye}),dn=At.extend({propertyName:null,elapsedTime:null,pseudoElement:null}),fn=qe.extend({deltaX:function(t){return"deltaX"in t?t.deltaX:"wheelDeltaX"in t?-t.wheelDeltaX:0},deltaY:function(t){return"deltaY"in t?t.deltaY:"wheelDeltaY"in t?-t.wheelDeltaY:"wheelDelta"in t?-t.wheelDelta:0},deltaZ:null,deltaMode:null}),pn=[["abort","abort"],[X,"animationEnd"],[Z,"animationIteration"],[$,"animationStart"],["canplay","canPlay"],["canplaythrough","canPlayThrough"],["drag","drag"],["dragenter","dragEnter"],["dragexit","dragExit"],["dragleave","dragLeave"],["dragover","dragOver"],["durationchange","durationChange"],["emptied","emptied"],["encrypted","encrypted"],["ended","ended"],["error","error"],["gotpointercapture","gotPointerCapture"],["load","load"],["loadeddata","loadedData"],["loadedmetadata","loadedMetadata"],["loadstart","loadStart"],["lostpointercapture","lostPointerCapture"],["mousemove","mouseMove"],["mouseout","mouseOut"],["mouseover","mouseOver"],["playing","playing"],["pointermove","pointerMove"],["pointerout","pointerOut"],["pointerover","pointerOver"],["progress","progress"],["scroll","scroll"],["seeking","seeking"],["stalled","stalled"],["suspend","suspend"],["timeupdate","timeUpdate"],["toggle","toggle"],["touchmove","touchMove"],[tt,"transitionEnd"],["waiting","waiting"],["wheel","wheel"]],In={},Cn={};function mn(t,e){var n=t[0],r="on"+((t=t[1])[0].toUpperCase()+t.slice(1));e={phasedRegistrationNames:{bubbled:r,captured:r+"Capture"},dependencies:[n],isInteractive:e},In[t]=e,Cn[n]=e}[["blur","blur"],["cancel","cancel"],["click","click"],["close","close"],["contextmenu","contextMenu"],["copy","copy"],["cut","cut"],["auxclick","auxClick"],["dblclick","doubleClick"],["dragend","dragEnd"],["dragstart","dragStart"],["drop","drop"],["focus","focus"],["input","input"],["invalid","invalid"],["keydown","keyDown"],["keypress","keyPress"],["keyup","keyUp"],["mousedown","mouseDown"],["mouseup","mouseUp"],["paste","paste"],["pause","pause"],["play","play"],["pointercancel","pointerCancel"],["pointerdown","pointerDown"],["pointerup","pointerUp"],["ratechange","rateChange"],["reset","reset"],["seeked","seeked"],["submit","submit"],["touchcancel","touchCancel"],["touchend","touchEnd"],["touchstart","touchStart"],["volumechange","volumeChange"]].forEach(function(t){mn(t,!0)}),pn.forEach(function(t){mn(t,!1)});var yn={eventTypes:In,isInteractiveTopLevelEventType:function(t){return void 0!==(t=Cn[t])&&!0===t.isInteractive},extractEvents:function(t,e,n,r){var i=Cn[t];if(!i)return null;switch(t){case"keypress":if(0===An(n))return null;case"keydown":case"keyup":t=un;break;case"blur":case"focus":t=sn;break;case"click":if(2===n.button)return null;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":t=qe;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":t=gn;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":t=hn;break;case X:case Z:case $:t=on;break;case tt:t=dn;break;case"scroll":t=Ge;break;case"wheel":t=fn;break;case"copy":case"cut":case"paste":t=an;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":t=Ve;break;default:t=At}return P(e=t.getPooled(i,e,n,r)),e}},En=yn.isInteractiveTopLevelEventType,vn=[];function Bn(t){var e=t.targetInst,n=e;do{if(!n){t.ancestors.push(n);break}var r;for(r=n;r.return;)r=r.return;if(!(r=3!==r.tag?null:r.stateNode.containerInfo))break;t.ancestors.push(n),n=N(r)}while(n);for(n=0;n<t.ancestors.length;n++){e=t.ancestors[n];var i=Ot(t.nativeEvent);r=t.topLevelType;for(var o=t.nativeEvent,a=null,s=0;s<I.length;s++){var A=I[s];A&&(A=A.extractEvents(r,e,o,i))&&(a=Q(a,A))}k(a)}}var bn=!0;function Qn(t,e){if(!e)return null;var n=(En(t)?xn:_n).bind(null,t);e.addEventListener(t,n,!1)}function wn(t,e){if(!e)return null;var n=(En(t)?xn:_n).bind(null,t);e.addEventListener(t,n,!0)}function xn(t,e){Lt(_n,t,e)}function _n(t,e){if(bn){var n=Ot(e);if(null===(n=N(n))||"number"!=typeof n.tag||2===en(n)||(n=null),vn.length){var r=vn.pop();r.topLevelType=t,r.nativeEvent=e,r.targetInst=n,t=r}else t={topLevelType:t,nativeEvent:e,targetInst:n,ancestors:[]};try{Ft(Bn,t)}finally{t.topLevelType=null,t.nativeEvent=null,t.targetInst=null,t.ancestors.length=0,10>vn.length&&vn.push(t)}}}var Dn={},Mn=0,kn="_reactListenersID"+(""+Math.random()).slice(2);function Sn(t){return Object.prototype.hasOwnProperty.call(t,kn)||(t[kn]=Mn++,Dn[t[kn]]={}),Dn[t[kn]]}function Ln(t){if(void 0===(t=t||("undefined"!=typeof document?document:void 0)))return null;try{return t.activeElement||t.body}catch(e){return t.body}}function Rn(t){for(;t&&t.firstChild;)t=t.firstChild;return t}function Nn(t,e){var n,r=Rn(t);for(t=0;r;){if(3===r.nodeType){if(n=t+r.textContent.length,t<=e&&n>=e)return{node:r,offset:e-t};t=n}t:{for(;r;){if(r.nextSibling){r=r.nextSibling;break t}r=r.parentNode}r=void 0}r=Rn(r)}}function Fn(){for(var t=window,e=Ln();e instanceof t.HTMLIFrameElement;){try{var n="string"==typeof e.contentWindow.location.href}catch(t){n=!1}if(!n)break;e=Ln((t=e.contentWindow).document)}return e}function Tn(t){var e=t&&t.nodeName&&t.nodeName.toLowerCase();return e&&("input"===e&&("text"===t.type||"search"===t.type||"tel"===t.type||"url"===t.type||"password"===t.type)||"textarea"===e||"true"===t.contentEditable)}function Un(t){var e=Fn(),n=t.focusedElem,r=t.selectionRange;if(e!==n&&n&&n.ownerDocument&&function t(e,n){return!(!e||!n)&&(e===n||(!e||3!==e.nodeType)&&(n&&3===n.nodeType?t(e,n.parentNode):"contains"in e?e.contains(n):!!e.compareDocumentPosition&&!!(16&e.compareDocumentPosition(n))))}(n.ownerDocument.documentElement,n)){if(null!==r&&Tn(n))if(e=r.start,void 0===(t=r.end)&&(t=e),"selectionStart"in n)n.selectionStart=e,n.selectionEnd=Math.min(t,n.value.length);else if((t=(e=n.ownerDocument||document)&&e.defaultView||window).getSelection){t=t.getSelection();var i=n.textContent.length,o=Math.min(r.start,i);r=void 0===r.end?o:Math.min(r.end,i),!t.extend&&o>r&&(i=r,r=o,o=i),i=Nn(n,o);var a=Nn(n,r);i&&a&&(1!==t.rangeCount||t.anchorNode!==i.node||t.anchorOffset!==i.offset||t.focusNode!==a.node||t.focusOffset!==a.offset)&&((e=e.createRange()).setStart(i.node,i.offset),t.removeAllRanges(),o>r?(t.addRange(e),t.extend(a.node,a.offset)):(e.setEnd(a.node,a.offset),t.addRange(e)))}for(e=[],t=n;t=t.parentNode;)1===t.nodeType&&e.push({element:t,left:t.scrollLeft,top:t.scrollTop});for("function"==typeof n.focus&&n.focus(),n=0;n<e.length;n++)(t=e[n]).element.scrollLeft=t.left,t.element.scrollTop=t.top}}var On=K&&"documentMode"in document&&11>=document.documentMode,Gn={select:{phasedRegistrationNames:{bubbled:"onSelect",captured:"onSelectCapture"},dependencies:"blur contextmenu dragend focus keydown keyup mousedown mouseup selectionchange".split(" ")}},Hn=null,jn=null,Yn=null,Pn=!1;function Kn(t,e){var n=e.window===e?e.document:9===e.nodeType?e:e.ownerDocument;return Pn||null==Hn||Hn!==Ln(n)?null:("selectionStart"in(n=Hn)&&Tn(n)?n={start:n.selectionStart,end:n.selectionEnd}:n={anchorNode:(n=(n.ownerDocument&&n.ownerDocument.defaultView||window).getSelection()).anchorNode,anchorOffset:n.anchorOffset,focusNode:n.focusNode,focusOffset:n.focusOffset},Yn&&tn(Yn,n)?null:(Yn=n,(t=At.getPooled(Gn.select,jn,t,e)).type="select",t.target=Hn,P(t),t))}var zn={eventTypes:Gn,extractEvents:function(t,e,n,r){var i,o=r.window===r?r.document:9===r.nodeType?r:r.ownerDocument;if(!(i=!o)){t:{o=Sn(o),i=y.onSelect;for(var a=0;a<i.length;a++){var s=i[a];if(!o.hasOwnProperty(s)||!o[s]){o=!1;break t}}o=!0}i=!o}if(i)return null;switch(o=e?T(e):window,t){case"focus":(Ut(o)||"true"===o.contentEditable)&&(Hn=o,jn=e,Yn=null);break;case"blur":Yn=jn=Hn=null;break;case"mousedown":Pn=!0;break;case"contextmenu":case"mouseup":case"dragend":return Pn=!1,Kn(n,r);case"selectionchange":if(On)break;case"keydown":case"keyup":return Kn(n,r)}return null}};function Jn(t,e){return t=i({children:void 0},e),(e=function(t){var e="";return r.Children.forEach(t,function(t){null!=t&&(e+=t)}),e}(e.children))&&(t.children=e),t}function qn(t,e,n,r){if(t=t.options,e){e={};for(var i=0;i<n.length;i++)e["$"+n[i]]=!0;for(n=0;n<t.length;n++)i=e.hasOwnProperty("$"+t[n].value),t[n].selected!==i&&(t[n].selected=i),i&&r&&(t[n].defaultSelected=!0)}else{for(n=""+Ce(n),e=null,i=0;i<t.length;i++){if(t[i].value===n)return t[i].selected=!0,void(r&&(t[i].defaultSelected=!0));null!==e||t[i].disabled||(e=t[i])}null!==e&&(e.selected=!0)}}function Vn(t,e){return null!=e.dangerouslySetInnerHTML&&a("91"),i({},e,{value:void 0,defaultValue:void 0,children:""+t._wrapperState.initialValue})}function Wn(t,e){var n=e.value;null==n&&(n=e.defaultValue,null!=(e=e.children)&&(null!=n&&a("92"),Array.isArray(e)&&(1>=e.length||a("93"),e=e[0]),n=e),null==n&&(n="")),t._wrapperState={initialValue:Ce(n)}}function Xn(t,e){var n=Ce(e.value),r=Ce(e.defaultValue);null!=n&&((n=""+n)!==t.value&&(t.value=n),null==e.defaultValue&&t.defaultValue!==n&&(t.defaultValue=n)),null!=r&&(t.defaultValue=""+r)}function Zn(t){var e=t.textContent;e===t._wrapperState.initialValue&&(t.value=e)}D.injectEventPluginOrder("ResponderEventPlugin SimpleEventPlugin EnterLeaveEventPlugin ChangeEventPlugin SelectEventPlugin BeforeInputEventPlugin".split(" ")),E=U,v=F,B=T,D.injectEventPluginsByName({SimpleEventPlugin:yn,EnterLeaveEventPlugin:Xe,ChangeEventPlugin:Oe,SelectEventPlugin:zn,BeforeInputEventPlugin:Qt});var $n={html:"http://www.w3.org/1999/xhtml",mathml:"http://www.w3.org/1998/Math/MathML",svg:"http://www.w3.org/2000/svg"};function tr(t){switch(t){case"svg":return"http://www.w3.org/2000/svg";case"math":return"http://www.w3.org/1998/Math/MathML";default:return"http://www.w3.org/1999/xhtml"}}function er(t,e){return null==t||"http://www.w3.org/1999/xhtml"===t?tr(e):"http://www.w3.org/2000/svg"===t&&"foreignObject"===e?"http://www.w3.org/1999/xhtml":t}var nr=void 0,rr=function(t){return"undefined"!=typeof MSApp&&MSApp.execUnsafeLocalFunction?function(e,n,r,i){MSApp.execUnsafeLocalFunction(function(){return t(e,n)})}:t}(function(t,e){if(t.namespaceURI!==$n.svg||"innerHTML"in t)t.innerHTML=e;else{for((nr=nr||document.createElement("div")).innerHTML="<svg>"+e+"</svg>",e=nr.firstChild;t.firstChild;)t.removeChild(t.firstChild);for(;e.firstChild;)t.appendChild(e.firstChild)}});function ir(t,e){if(e){var n=t.firstChild;if(n&&n===t.lastChild&&3===n.nodeType)return void(n.nodeValue=e)}t.textContent=e}var or={animationIterationCount:!0,borderImageOutset:!0,borderImageSlice:!0,borderImageWidth:!0,boxFlex:!0,boxFlexGroup:!0,boxOrdinalGroup:!0,columnCount:!0,columns:!0,flex:!0,flexGrow:!0,flexPositive:!0,flexShrink:!0,flexNegative:!0,flexOrder:!0,gridArea:!0,gridRow:!0,gridRowEnd:!0,gridRowSpan:!0,gridRowStart:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnSpan:!0,gridColumnStart:!0,fontWeight:!0,lineClamp:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,tabSize:!0,widows:!0,zIndex:!0,zoom:!0,fillOpacity:!0,floodOpacity:!0,stopOpacity:!0,strokeDasharray:!0,strokeDashoffset:!0,strokeMiterlimit:!0,strokeOpacity:!0,strokeWidth:!0},ar=["Webkit","ms","Moz","O"];function sr(t,e,n){return null==e||"boolean"==typeof e||""===e?"":n||"number"!=typeof e||0===e||or.hasOwnProperty(t)&&or[t]?(""+e).trim():e+"px"}function Ar(t,e){for(var n in t=t.style,e)if(e.hasOwnProperty(n)){var r=0===n.indexOf("--"),i=sr(n,e[n],r);"float"===n&&(n="cssFloat"),r?t.setProperty(n,i):t[n]=i}}Object.keys(or).forEach(function(t){ar.forEach(function(e){e=e+t.charAt(0).toUpperCase()+t.substring(1),or[e]=or[t]})});var lr=i({menuitem:!0},{area:!0,base:!0,br:!0,col:!0,embed:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0});function cr(t,e){e&&(lr[t]&&(null!=e.children||null!=e.dangerouslySetInnerHTML)&&a("137",t,""),null!=e.dangerouslySetInnerHTML&&(null!=e.children&&a("60"),"object"==typeof e.dangerouslySetInnerHTML&&"__html"in e.dangerouslySetInnerHTML||a("61")),null!=e.style&&"object"!=typeof e.style&&a("62",""))}function ur(t,e){if(-1===t.indexOf("-"))return"string"==typeof e.is;switch(t){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}function gr(t,e){var n=Sn(t=9===t.nodeType||11===t.nodeType?t:t.ownerDocument);e=y[e];for(var r=0;r<e.length;r++){var i=e[r];if(!n.hasOwnProperty(i)||!n[i]){switch(i){case"scroll":wn("scroll",t);break;case"focus":case"blur":wn("focus",t),wn("blur",t),n.blur=!0,n.focus=!0;break;case"cancel":case"close":Gt(i)&&wn(i,t);break;case"invalid":case"submit":case"reset":break;default:-1===et.indexOf(i)&&Qn(i,t)}n[i]=!0}}}function hr(){}var dr=null,fr=null;function pr(t,e){switch(t){case"button":case"input":case"select":case"textarea":return!!e.autoFocus}return!1}function Ir(t,e){return"textarea"===t||"option"===t||"noscript"===t||"string"==typeof e.children||"number"==typeof e.children||"object"==typeof e.dangerouslySetInnerHTML&&null!==e.dangerouslySetInnerHTML&&null!=e.dangerouslySetInnerHTML.__html}var Cr="function"==typeof setTimeout?setTimeout:void 0,mr="function"==typeof clearTimeout?clearTimeout:void 0,yr=o.unstable_scheduleCallback,Er=o.unstable_cancelCallback;function vr(t){for(t=t.nextSibling;t&&1!==t.nodeType&&3!==t.nodeType;)t=t.nextSibling;return t}function Br(t){for(t=t.firstChild;t&&1!==t.nodeType&&3!==t.nodeType;)t=t.nextSibling;return t}new Set;var br=[],Qr=-1;function wr(t){0>Qr||(t.current=br[Qr],br[Qr]=null,Qr--)}function xr(t,e){br[++Qr]=t.current,t.current=e}var _r={},Dr={current:_r},Mr={current:!1},kr=_r;function Sr(t,e){var n=t.type.contextTypes;if(!n)return _r;var r=t.stateNode;if(r&&r.__reactInternalMemoizedUnmaskedChildContext===e)return r.__reactInternalMemoizedMaskedChildContext;var i,o={};for(i in n)o[i]=e[i];return r&&((t=t.stateNode).__reactInternalMemoizedUnmaskedChildContext=e,t.__reactInternalMemoizedMaskedChildContext=o),o}function Lr(t){return null!=(t=t.childContextTypes)}function Rr(t){wr(Mr),wr(Dr)}function Nr(t){wr(Mr),wr(Dr)}function Fr(t,e,n){Dr.current!==_r&&a("168"),xr(Dr,e),xr(Mr,n)}function Tr(t,e,n){var r=t.stateNode;if(t=e.childContextTypes,"function"!=typeof r.getChildContext)return n;for(var o in r=r.getChildContext())o in t||a("108",se(e)||"Unknown",o);return i({},n,r)}function Ur(t){var e=t.stateNode;return e=e&&e.__reactInternalMemoizedMergedChildContext||_r,kr=Dr.current,xr(Dr,e),xr(Mr,Mr.current),!0}function Or(t,e,n){var r=t.stateNode;r||a("169"),n?(e=Tr(t,e,kr),r.__reactInternalMemoizedMergedChildContext=e,wr(Mr),wr(Dr),xr(Dr,e)):wr(Mr),xr(Mr,n)}var Gr=null,Hr=null;function jr(t){return function(e){try{return t(e)}catch(t){}}}function Yr(t,e,n,r){this.tag=t,this.key=n,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.ref=null,this.pendingProps=e,this.contextDependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=r,this.effectTag=0,this.lastEffect=this.firstEffect=this.nextEffect=null,this.childExpirationTime=this.expirationTime=0,this.alternate=null}function Pr(t,e,n,r){return new Yr(t,e,n,r)}function Kr(t){return!(!(t=t.prototype)||!t.isReactComponent)}function zr(t,e){var n=t.alternate;return null===n?((n=Pr(t.tag,e,t.key,t.mode)).elementType=t.elementType,n.type=t.type,n.stateNode=t.stateNode,n.alternate=t,t.alternate=n):(n.pendingProps=e,n.effectTag=0,n.nextEffect=null,n.firstEffect=null,n.lastEffect=null),n.childExpirationTime=t.childExpirationTime,n.expirationTime=t.expirationTime,n.child=t.child,n.memoizedProps=t.memoizedProps,n.memoizedState=t.memoizedState,n.updateQueue=t.updateQueue,n.contextDependencies=t.contextDependencies,n.sibling=t.sibling,n.index=t.index,n.ref=t.ref,n}function Jr(t,e,n,r,i,o){var s=2;if(r=t,"function"==typeof t)Kr(t)&&(s=1);else if("string"==typeof t)s=5;else t:switch(t){case Vt:return qr(n.children,i,o,e);case te:return Vr(n,3|i,o,e);case Wt:return Vr(n,2|i,o,e);case Xt:return(t=Pr(12,n,e,4|i)).elementType=Xt,t.type=Xt,t.expirationTime=o,t;case ne:return(t=Pr(13,n,e,i)).elementType=ne,t.type=ne,t.expirationTime=o,t;default:if("object"==typeof t&&null!==t)switch(t.$$typeof){case Zt:s=10;break t;case $t:s=9;break t;case ee:s=11;break t;case re:s=14;break t;case ie:s=16,r=null;break t}a("130",null==t?t:typeof t,"")}return(e=Pr(s,n,e,i)).elementType=t,e.type=r,e.expirationTime=o,e}function qr(t,e,n,r){return(t=Pr(7,t,r,e)).expirationTime=n,t}function Vr(t,e,n,r){return t=Pr(8,t,r,e),e=0==(1&e)?Wt:te,t.elementType=e,t.type=e,t.expirationTime=n,t}function Wr(t,e,n){return(t=Pr(6,t,null,e)).expirationTime=n,t}function Xr(t,e,n){return(e=Pr(4,null!==t.children?t.children:[],t.key,e)).expirationTime=n,e.stateNode={containerInfo:t.containerInfo,pendingChildren:null,implementation:t.implementation},e}function Zr(t,e){t.didError=!1;var n=t.earliestPendingTime;0===n?t.earliestPendingTime=t.latestPendingTime=e:n<e?t.earliestPendingTime=e:t.latestPendingTime>e&&(t.latestPendingTime=e),ei(e,t)}function $r(t,e){t.didError=!1,t.latestPingedTime>=e&&(t.latestPingedTime=0);var n=t.earliestPendingTime,r=t.latestPendingTime;n===e?t.earliestPendingTime=r===e?t.latestPendingTime=0:r:r===e&&(t.latestPendingTime=n),n=t.earliestSuspendedTime,r=t.latestSuspendedTime,0===n?t.earliestSuspendedTime=t.latestSuspendedTime=e:n<e?t.earliestSuspendedTime=e:r>e&&(t.latestSuspendedTime=e),ei(e,t)}function ti(t,e){var n=t.earliestPendingTime;return n>e&&(e=n),(t=t.earliestSuspendedTime)>e&&(e=t),e}function ei(t,e){var n=e.earliestSuspendedTime,r=e.latestSuspendedTime,i=e.earliestPendingTime,o=e.latestPingedTime;0===(i=0!==i?i:o)&&(0===t||r<t)&&(i=r),0!==(t=i)&&n>t&&(t=n),e.nextExpirationTimeToWorkOn=i,e.expirationTime=t}function ni(t,e){if(t&&t.defaultProps)for(var n in e=i({},e),t=t.defaultProps)void 0===e[n]&&(e[n]=t[n]);return e}var ri=(new r.Component).refs;function ii(t,e,n,r){n=null==(n=n(r,e=t.memoizedState))?e:i({},e,n),t.memoizedState=n,null!==(r=t.updateQueue)&&0===t.expirationTime&&(r.baseState=n)}var oi={isMounted:function(t){return!!(t=t._reactInternalFiber)&&2===en(t)},enqueueSetState:function(t,e,n){t=t._reactInternalFiber;var r=Es(),i=Vo(r=qa(r,t));i.payload=e,null!=n&&(i.callback=n),ja(),Xo(t,i),Xa(t,r)},enqueueReplaceState:function(t,e,n){t=t._reactInternalFiber;var r=Es(),i=Vo(r=qa(r,t));i.tag=Yo,i.payload=e,null!=n&&(i.callback=n),ja(),Xo(t,i),Xa(t,r)},enqueueForceUpdate:function(t,e){t=t._reactInternalFiber;var n=Es(),r=Vo(n=qa(n,t));r.tag=Po,null!=e&&(r.callback=e),ja(),Xo(t,r),Xa(t,n)}};function ai(t,e,n,r,i,o,a){return"function"==typeof(t=t.stateNode).shouldComponentUpdate?t.shouldComponentUpdate(r,o,a):!e.prototype||!e.prototype.isPureReactComponent||(!tn(n,r)||!tn(i,o))}function si(t,e,n){var r=!1,i=_r,o=e.contextType;return"object"==typeof o&&null!==o?o=Ho(o):(i=Lr(e)?kr:Dr.current,o=(r=null!=(r=e.contextTypes))?Sr(t,i):_r),e=new e(n,o),t.memoizedState=null!==e.state&&void 0!==e.state?e.state:null,e.updater=oi,t.stateNode=e,e._reactInternalFiber=t,r&&((t=t.stateNode).__reactInternalMemoizedUnmaskedChildContext=i,t.__reactInternalMemoizedMaskedChildContext=o),e}function Ai(t,e,n,r){t=e.state,"function"==typeof e.componentWillReceiveProps&&e.componentWillReceiveProps(n,r),"function"==typeof e.UNSAFE_componentWillReceiveProps&&e.UNSAFE_componentWillReceiveProps(n,r),e.state!==t&&oi.enqueueReplaceState(e,e.state,null)}function li(t,e,n,r){var i=t.stateNode;i.props=n,i.state=t.memoizedState,i.refs=ri;var o=e.contextType;"object"==typeof o&&null!==o?i.context=Ho(o):(o=Lr(e)?kr:Dr.current,i.context=Sr(t,o)),null!==(o=t.updateQueue)&&(ea(t,o,n,i,r),i.state=t.memoizedState),"function"==typeof(o=e.getDerivedStateFromProps)&&(ii(t,e,o,n),i.state=t.memoizedState),"function"==typeof e.getDerivedStateFromProps||"function"==typeof i.getSnapshotBeforeUpdate||"function"!=typeof i.UNSAFE_componentWillMount&&"function"!=typeof i.componentWillMount||(e=i.state,"function"==typeof i.componentWillMount&&i.componentWillMount(),"function"==typeof i.UNSAFE_componentWillMount&&i.UNSAFE_componentWillMount(),e!==i.state&&oi.enqueueReplaceState(i,i.state,null),null!==(o=t.updateQueue)&&(ea(t,o,n,i,r),i.state=t.memoizedState)),"function"==typeof i.componentDidMount&&(t.effectTag|=4)}var ci=Array.isArray;function ui(t,e,n){if(null!==(t=n.ref)&&"function"!=typeof t&&"object"!=typeof t){if(n._owner){n=n._owner;var r=void 0;n&&(1!==n.tag&&a("309"),r=n.stateNode),r||a("147",t);var i=""+t;return null!==e&&null!==e.ref&&"function"==typeof e.ref&&e.ref._stringRef===i?e.ref:((e=function(t){var e=r.refs;e===ri&&(e=r.refs={}),null===t?delete e[i]:e[i]=t})._stringRef=i,e)}"string"!=typeof t&&a("284"),n._owner||a("290",t)}return t}function gi(t,e){"textarea"!==t.type&&a("31","[object Object]"===Object.prototype.toString.call(e)?"object with keys {"+Object.keys(e).join(", ")+"}":e,"")}function hi(t){function e(e,n){if(t){var r=e.lastEffect;null!==r?(r.nextEffect=n,e.lastEffect=n):e.firstEffect=e.lastEffect=n,n.nextEffect=null,n.effectTag=8}}function n(n,r){if(!t)return null;for(;null!==r;)e(n,r),r=r.sibling;return null}function r(t,e){for(t=new Map;null!==e;)null!==e.key?t.set(e.key,e):t.set(e.index,e),e=e.sibling;return t}function i(t,e,n){return(t=zr(t,e)).index=0,t.sibling=null,t}function o(e,n,r){return e.index=r,t?null!==(r=e.alternate)?(r=r.index)<n?(e.effectTag=2,n):r:(e.effectTag=2,n):n}function s(e){return t&&null===e.alternate&&(e.effectTag=2),e}function A(t,e,n,r){return null===e||6!==e.tag?((e=Wr(n,t.mode,r)).return=t,e):((e=i(e,n)).return=t,e)}function l(t,e,n,r){return null!==e&&e.elementType===n.type?((r=i(e,n.props)).ref=ui(t,e,n),r.return=t,r):((r=Jr(n.type,n.key,n.props,null,t.mode,r)).ref=ui(t,e,n),r.return=t,r)}function c(t,e,n,r){return null===e||4!==e.tag||e.stateNode.containerInfo!==n.containerInfo||e.stateNode.implementation!==n.implementation?((e=Xr(n,t.mode,r)).return=t,e):((e=i(e,n.children||[])).return=t,e)}function u(t,e,n,r,o){return null===e||7!==e.tag?((e=qr(n,t.mode,r,o)).return=t,e):((e=i(e,n)).return=t,e)}function g(t,e,n){if("string"==typeof e||"number"==typeof e)return(e=Wr(""+e,t.mode,n)).return=t,e;if("object"==typeof e&&null!==e){switch(e.$$typeof){case Jt:return(n=Jr(e.type,e.key,e.props,null,t.mode,n)).ref=ui(t,null,e),n.return=t,n;case qt:return(e=Xr(e,t.mode,n)).return=t,e}if(ci(e)||ae(e))return(e=qr(e,t.mode,n,null)).return=t,e;gi(t,e)}return null}function h(t,e,n,r){var i=null!==e?e.key:null;if("string"==typeof n||"number"==typeof n)return null!==i?null:A(t,e,""+n,r);if("object"==typeof n&&null!==n){switch(n.$$typeof){case Jt:return n.key===i?n.type===Vt?u(t,e,n.props.children,r,i):l(t,e,n,r):null;case qt:return n.key===i?c(t,e,n,r):null}if(ci(n)||ae(n))return null!==i?null:u(t,e,n,r,null);gi(t,n)}return null}function d(t,e,n,r,i){if("string"==typeof r||"number"==typeof r)return A(e,t=t.get(n)||null,""+r,i);if("object"==typeof r&&null!==r){switch(r.$$typeof){case Jt:return t=t.get(null===r.key?n:r.key)||null,r.type===Vt?u(e,t,r.props.children,i,r.key):l(e,t,r,i);case qt:return c(e,t=t.get(null===r.key?n:r.key)||null,r,i)}if(ci(r)||ae(r))return u(e,t=t.get(n)||null,r,i,null);gi(e,r)}return null}function f(i,a,s,A){for(var l=null,c=null,u=a,f=a=0,p=null;null!==u&&f<s.length;f++){u.index>f?(p=u,u=null):p=u.sibling;var I=h(i,u,s[f],A);if(null===I){null===u&&(u=p);break}t&&u&&null===I.alternate&&e(i,u),a=o(I,a,f),null===c?l=I:c.sibling=I,c=I,u=p}if(f===s.length)return n(i,u),l;if(null===u){for(;f<s.length;f++)(u=g(i,s[f],A))&&(a=o(u,a,f),null===c?l=u:c.sibling=u,c=u);return l}for(u=r(i,u);f<s.length;f++)(p=d(u,i,f,s[f],A))&&(t&&null!==p.alternate&&u.delete(null===p.key?f:p.key),a=o(p,a,f),null===c?l=p:c.sibling=p,c=p);return t&&u.forEach(function(t){return e(i,t)}),l}function p(i,s,A,l){var c=ae(A);"function"!=typeof c&&a("150"),null==(A=c.call(A))&&a("151");for(var u=c=null,f=s,p=s=0,I=null,C=A.next();null!==f&&!C.done;p++,C=A.next()){f.index>p?(I=f,f=null):I=f.sibling;var m=h(i,f,C.value,l);if(null===m){f||(f=I);break}t&&f&&null===m.alternate&&e(i,f),s=o(m,s,p),null===u?c=m:u.sibling=m,u=m,f=I}if(C.done)return n(i,f),c;if(null===f){for(;!C.done;p++,C=A.next())null!==(C=g(i,C.value,l))&&(s=o(C,s,p),null===u?c=C:u.sibling=C,u=C);return c}for(f=r(i,f);!C.done;p++,C=A.next())null!==(C=d(f,i,p,C.value,l))&&(t&&null!==C.alternate&&f.delete(null===C.key?p:C.key),s=o(C,s,p),null===u?c=C:u.sibling=C,u=C);return t&&f.forEach(function(t){return e(i,t)}),c}return function(t,r,o,A){var l="object"==typeof o&&null!==o&&o.type===Vt&&null===o.key;l&&(o=o.props.children);var c="object"==typeof o&&null!==o;if(c)switch(o.$$typeof){case Jt:t:{for(c=o.key,l=r;null!==l;){if(l.key===c){if(7===l.tag?o.type===Vt:l.elementType===o.type){n(t,l.sibling),(r=i(l,o.type===Vt?o.props.children:o.props)).ref=ui(t,l,o),r.return=t,t=r;break t}n(t,l);break}e(t,l),l=l.sibling}o.type===Vt?((r=qr(o.props.children,t.mode,A,o.key)).return=t,t=r):((A=Jr(o.type,o.key,o.props,null,t.mode,A)).ref=ui(t,r,o),A.return=t,t=A)}return s(t);case qt:t:{for(l=o.key;null!==r;){if(r.key===l){if(4===r.tag&&r.stateNode.containerInfo===o.containerInfo&&r.stateNode.implementation===o.implementation){n(t,r.sibling),(r=i(r,o.children||[])).return=t,t=r;break t}n(t,r);break}e(t,r),r=r.sibling}(r=Xr(o,t.mode,A)).return=t,t=r}return s(t)}if("string"==typeof o||"number"==typeof o)return o=""+o,null!==r&&6===r.tag?(n(t,r.sibling),(r=i(r,o)).return=t,t=r):(n(t,r),(r=Wr(o,t.mode,A)).return=t,t=r),s(t);if(ci(o))return f(t,r,o,A);if(ae(o))return p(t,r,o,A);if(c&&gi(t,o),void 0===o&&!l)switch(t.tag){case 1:case 0:a("152",(A=t.type).displayName||A.name||"Component")}return n(t,r)}}var di=hi(!0),fi=hi(!1),pi={},Ii={current:pi},Ci={current:pi},mi={current:pi};function yi(t){return t===pi&&a("174"),t}function Ei(t,e){xr(mi,e),xr(Ci,t),xr(Ii,pi);var n=e.nodeType;switch(n){case 9:case 11:e=(e=e.documentElement)?e.namespaceURI:er(null,"");break;default:e=er(e=(n=8===n?e.parentNode:e).namespaceURI||null,n=n.tagName)}wr(Ii),xr(Ii,e)}function vi(t){wr(Ii),wr(Ci),wr(mi)}function Bi(t){yi(mi.current);var e=yi(Ii.current),n=er(e,t.type);e!==n&&(xr(Ci,t),xr(Ii,n))}function bi(t){Ci.current===t&&(wr(Ii),wr(Ci))}var Qi=0,wi=2,xi=4,_i=8,Di=16,Mi=32,ki=64,Si=128,Li=Pt.ReactCurrentDispatcher,Ri=0,Ni=null,Fi=null,Ti=null,Ui=null,Oi=null,Gi=null,Hi=0,ji=null,Yi=0,Pi=!1,Ki=null,zi=0;function Ji(){a("321")}function qi(t,e){if(null===e)return!1;for(var n=0;n<e.length&&n<t.length;n++)if(!Ze(t[n],e[n]))return!1;return!0}function Vi(t,e,n,r,i,o){if(Ri=o,Ni=e,Ti=null!==t?t.memoizedState:null,Li.current=null===Ti?Ao:lo,e=n(r,i),Pi){do{Pi=!1,zi+=1,Ti=null!==t?t.memoizedState:null,Gi=Ui,ji=Oi=Fi=null,Li.current=lo,e=n(r,i)}while(Pi);Ki=null,zi=0}return Li.current=so,(t=Ni).memoizedState=Ui,t.expirationTime=Hi,t.updateQueue=ji,t.effectTag|=Yi,t=null!==Fi&&null!==Fi.next,Ri=0,Gi=Oi=Ui=Ti=Fi=Ni=null,Hi=0,ji=null,Yi=0,t&&a("300"),e}function Wi(){Li.current=so,Ri=0,Gi=Oi=Ui=Ti=Fi=Ni=null,Hi=0,ji=null,Yi=0,Pi=!1,Ki=null,zi=0}function Xi(){var t={memoizedState:null,baseState:null,queue:null,baseUpdate:null,next:null};return null===Oi?Ui=Oi=t:Oi=Oi.next=t,Oi}function Zi(){if(null!==Gi)Gi=(Oi=Gi).next,Ti=null!==(Fi=Ti)?Fi.next:null;else{null===Ti&&a("310");var t={memoizedState:(Fi=Ti).memoizedState,baseState:Fi.baseState,queue:Fi.queue,baseUpdate:Fi.baseUpdate,next:null};Oi=null===Oi?Ui=t:Oi.next=t,Ti=Fi.next}return Oi}function $i(t,e){return"function"==typeof e?e(t):e}function to(t){var e=Zi(),n=e.queue;if(null===n&&a("311"),n.lastRenderedReducer=t,0<zi){var r=n.dispatch;if(null!==Ki){var i=Ki.get(n);if(void 0!==i){Ki.delete(n);var o=e.memoizedState;do{o=t(o,i.action),i=i.next}while(null!==i);return Ze(o,e.memoizedState)||(Eo=!0),e.memoizedState=o,e.baseUpdate===n.last&&(e.baseState=o),n.lastRenderedState=o,[o,r]}}return[e.memoizedState,r]}r=n.last;var s=e.baseUpdate;if(o=e.baseState,null!==s?(null!==r&&(r.next=null),r=s.next):r=null!==r?r.next:null,null!==r){var A=i=null,l=r,c=!1;do{var u=l.expirationTime;u<Ri?(c||(c=!0,A=s,i=o),u>Hi&&(Hi=u)):o=l.eagerReducer===t?l.eagerState:t(o,l.action),s=l,l=l.next}while(null!==l&&l!==r);c||(A=s,i=o),Ze(o,e.memoizedState)||(Eo=!0),e.memoizedState=o,e.baseUpdate=A,e.baseState=i,n.lastRenderedState=o}return[e.memoizedState,n.dispatch]}function eo(t,e,n,r){return t={tag:t,create:e,destroy:n,deps:r,next:null},null===ji?(ji={lastEffect:null}).lastEffect=t.next=t:null===(e=ji.lastEffect)?ji.lastEffect=t.next=t:(n=e.next,e.next=t,t.next=n,ji.lastEffect=t),t}function no(t,e,n,r){var i=Xi();Yi|=t,i.memoizedState=eo(e,n,void 0,void 0===r?null:r)}function ro(t,e,n,r){var i=Zi();r=void 0===r?null:r;var o=void 0;if(null!==Fi){var a=Fi.memoizedState;if(o=a.destroy,null!==r&&qi(r,a.deps))return void eo(Qi,n,o,r)}Yi|=t,i.memoizedState=eo(e,n,o,r)}function io(t,e){return"function"==typeof e?(t=t(),e(t),function(){e(null)}):null!=e?(t=t(),e.current=t,function(){e.current=null}):void 0}function oo(){}function ao(t,e,n){25>zi||a("301");var r=t.alternate;if(t===Ni||null!==r&&r===Ni)if(Pi=!0,t={expirationTime:Ri,action:n,eagerReducer:null,eagerState:null,next:null},null===Ki&&(Ki=new Map),void 0===(n=Ki.get(e)))Ki.set(e,t);else{for(e=n;null!==e.next;)e=e.next;e.next=t}else{ja();var i=Es(),o={expirationTime:i=qa(i,t),action:n,eagerReducer:null,eagerState:null,next:null},s=e.last;if(null===s)o.next=o;else{var A=s.next;null!==A&&(o.next=A),s.next=o}if(e.last=o,0===t.expirationTime&&(null===r||0===r.expirationTime)&&null!==(r=e.lastRenderedReducer))try{var l=e.lastRenderedState,c=r(l,n);if(o.eagerReducer=r,o.eagerState=c,Ze(c,l))return}catch(t){}Xa(t,i)}}var so={readContext:Ho,useCallback:Ji,useContext:Ji,useEffect:Ji,useImperativeHandle:Ji,useLayoutEffect:Ji,useMemo:Ji,useReducer:Ji,useRef:Ji,useState:Ji,useDebugValue:Ji},Ao={readContext:Ho,useCallback:function(t,e){return Xi().memoizedState=[t,void 0===e?null:e],t},useContext:Ho,useEffect:function(t,e){return no(516,Si|ki,t,e)},useImperativeHandle:function(t,e,n){return n=null!=n?n.concat([t]):null,no(4,xi|Mi,io.bind(null,e,t),n)},useLayoutEffect:function(t,e){return no(4,xi|Mi,t,e)},useMemo:function(t,e){var n=Xi();return e=void 0===e?null:e,t=t(),n.memoizedState=[t,e],t},useReducer:function(t,e,n){var r=Xi();return e=void 0!==n?n(e):e,r.memoizedState=r.baseState=e,t=(t=r.queue={last:null,dispatch:null,lastRenderedReducer:t,lastRenderedState:e}).dispatch=ao.bind(null,Ni,t),[r.memoizedState,t]},useRef:function(t){return t={current:t},Xi().memoizedState=t},useState:function(t){var e=Xi();return"function"==typeof t&&(t=t()),e.memoizedState=e.baseState=t,t=(t=e.queue={last:null,dispatch:null,lastRenderedReducer:$i,lastRenderedState:t}).dispatch=ao.bind(null,Ni,t),[e.memoizedState,t]},useDebugValue:oo},lo={readContext:Ho,useCallback:function(t,e){var n=Zi();e=void 0===e?null:e;var r=n.memoizedState;return null!==r&&null!==e&&qi(e,r[1])?r[0]:(n.memoizedState=[t,e],t)},useContext:Ho,useEffect:function(t,e){return ro(516,Si|ki,t,e)},useImperativeHandle:function(t,e,n){return n=null!=n?n.concat([t]):null,ro(4,xi|Mi,io.bind(null,e,t),n)},useLayoutEffect:function(t,e){return ro(4,xi|Mi,t,e)},useMemo:function(t,e){var n=Zi();e=void 0===e?null:e;var r=n.memoizedState;return null!==r&&null!==e&&qi(e,r[1])?r[0]:(t=t(),n.memoizedState=[t,e],t)},useReducer:to,useRef:function(){return Zi().memoizedState},useState:function(t){return to($i)},useDebugValue:oo},co=null,uo=null,go=!1;function ho(t,e){var n=Pr(5,null,null,0);n.elementType="DELETED",n.type="DELETED",n.stateNode=e,n.return=t,n.effectTag=8,null!==t.lastEffect?(t.lastEffect.nextEffect=n,t.lastEffect=n):t.firstEffect=t.lastEffect=n}function fo(t,e){switch(t.tag){case 5:var n=t.type;return null!==(e=1!==e.nodeType||n.toLowerCase()!==e.nodeName.toLowerCase()?null:e)&&(t.stateNode=e,!0);case 6:return null!==(e=""===t.pendingProps||3!==e.nodeType?null:e)&&(t.stateNode=e,!0);case 13:default:return!1}}function po(t){if(go){var e=uo;if(e){var n=e;if(!fo(t,e)){if(!(e=vr(n))||!fo(t,e))return t.effectTag|=2,go=!1,void(co=t);ho(co,n)}co=t,uo=Br(e)}else t.effectTag|=2,go=!1,co=t}}function Io(t){for(t=t.return;null!==t&&5!==t.tag&&3!==t.tag&&18!==t.tag;)t=t.return;co=t}function Co(t){if(t!==co)return!1;if(!go)return Io(t),go=!0,!1;var e=t.type;if(5!==t.tag||"head"!==e&&"body"!==e&&!Ir(e,t.memoizedProps))for(e=uo;e;)ho(t,e),e=vr(e);return Io(t),uo=co?vr(t.stateNode):null,!0}function mo(){uo=co=null,go=!1}var yo=Pt.ReactCurrentOwner,Eo=!1;function vo(t,e,n,r){e.child=null===t?fi(e,null,n,r):di(e,t.child,n,r)}function Bo(t,e,n,r,i){n=n.render;var o=e.ref;return Go(e,i),r=Vi(t,e,n,r,o,i),null===t||Eo?(e.effectTag|=1,vo(t,e,r,i),e.child):(e.updateQueue=t.updateQueue,e.effectTag&=-517,t.expirationTime<=i&&(t.expirationTime=0),So(t,e,i))}function bo(t,e,n,r,i,o){if(null===t){var a=n.type;return"function"!=typeof a||Kr(a)||void 0!==a.defaultProps||null!==n.compare||void 0!==n.defaultProps?((t=Jr(n.type,null,r,null,e.mode,o)).ref=e.ref,t.return=e,e.child=t):(e.tag=15,e.type=a,Qo(t,e,a,r,i,o))}return a=t.child,i<o&&(i=a.memoizedProps,(n=null!==(n=n.compare)?n:tn)(i,r)&&t.ref===e.ref)?So(t,e,o):(e.effectTag|=1,(t=zr(a,r)).ref=e.ref,t.return=e,e.child=t)}function Qo(t,e,n,r,i,o){return null!==t&&tn(t.memoizedProps,r)&&t.ref===e.ref&&(Eo=!1,i<o)?So(t,e,o):xo(t,e,n,r,o)}function wo(t,e){var n=e.ref;(null===t&&null!==n||null!==t&&t.ref!==n)&&(e.effectTag|=128)}function xo(t,e,n,r,i){var o=Lr(n)?kr:Dr.current;return o=Sr(e,o),Go(e,i),n=Vi(t,e,n,r,o,i),null===t||Eo?(e.effectTag|=1,vo(t,e,n,i),e.child):(e.updateQueue=t.updateQueue,e.effectTag&=-517,t.expirationTime<=i&&(t.expirationTime=0),So(t,e,i))}function _o(t,e,n,r,i){if(Lr(n)){var o=!0;Ur(e)}else o=!1;if(Go(e,i),null===e.stateNode)null!==t&&(t.alternate=null,e.alternate=null,e.effectTag|=2),si(e,n,r),li(e,n,r,i),r=!0;else if(null===t){var a=e.stateNode,s=e.memoizedProps;a.props=s;var A=a.context,l=n.contextType;"object"==typeof l&&null!==l?l=Ho(l):l=Sr(e,l=Lr(n)?kr:Dr.current);var c=n.getDerivedStateFromProps,u="function"==typeof c||"function"==typeof a.getSnapshotBeforeUpdate;u||"function"!=typeof a.UNSAFE_componentWillReceiveProps&&"function"!=typeof a.componentWillReceiveProps||(s!==r||A!==l)&&Ai(e,a,r,l),zo=!1;var g=e.memoizedState;A=a.state=g;var h=e.updateQueue;null!==h&&(ea(e,h,r,a,i),A=e.memoizedState),s!==r||g!==A||Mr.current||zo?("function"==typeof c&&(ii(e,n,c,r),A=e.memoizedState),(s=zo||ai(e,n,s,r,g,A,l))?(u||"function"!=typeof a.UNSAFE_componentWillMount&&"function"!=typeof a.componentWillMount||("function"==typeof a.componentWillMount&&a.componentWillMount(),"function"==typeof a.UNSAFE_componentWillMount&&a.UNSAFE_componentWillMount()),"function"==typeof a.componentDidMount&&(e.effectTag|=4)):("function"==typeof a.componentDidMount&&(e.effectTag|=4),e.memoizedProps=r,e.memoizedState=A),a.props=r,a.state=A,a.context=l,r=s):("function"==typeof a.componentDidMount&&(e.effectTag|=4),r=!1)}else a=e.stateNode,s=e.memoizedProps,a.props=e.type===e.elementType?s:ni(e.type,s),A=a.context,"object"==typeof(l=n.contextType)&&null!==l?l=Ho(l):l=Sr(e,l=Lr(n)?kr:Dr.current),(u="function"==typeof(c=n.getDerivedStateFromProps)||"function"==typeof a.getSnapshotBeforeUpdate)||"function"!=typeof a.UNSAFE_componentWillReceiveProps&&"function"!=typeof a.componentWillReceiveProps||(s!==r||A!==l)&&Ai(e,a,r,l),zo=!1,A=e.memoizedState,g=a.state=A,null!==(h=e.updateQueue)&&(ea(e,h,r,a,i),g=e.memoizedState),s!==r||A!==g||Mr.current||zo?("function"==typeof c&&(ii(e,n,c,r),g=e.memoizedState),(c=zo||ai(e,n,s,r,A,g,l))?(u||"function"!=typeof a.UNSAFE_componentWillUpdate&&"function"!=typeof a.componentWillUpdate||("function"==typeof a.componentWillUpdate&&a.componentWillUpdate(r,g,l),"function"==typeof a.UNSAFE_componentWillUpdate&&a.UNSAFE_componentWillUpdate(r,g,l)),"function"==typeof a.componentDidUpdate&&(e.effectTag|=4),"function"==typeof a.getSnapshotBeforeUpdate&&(e.effectTag|=256)):("function"!=typeof a.componentDidUpdate||s===t.memoizedProps&&A===t.memoizedState||(e.effectTag|=4),"function"!=typeof a.getSnapshotBeforeUpdate||s===t.memoizedProps&&A===t.memoizedState||(e.effectTag|=256),e.memoizedProps=r,e.memoizedState=g),a.props=r,a.state=g,a.context=l,r=c):("function"!=typeof a.componentDidUpdate||s===t.memoizedProps&&A===t.memoizedState||(e.effectTag|=4),"function"!=typeof a.getSnapshotBeforeUpdate||s===t.memoizedProps&&A===t.memoizedState||(e.effectTag|=256),r=!1);return Do(t,e,n,r,o,i)}function Do(t,e,n,r,i,o){wo(t,e);var a=0!=(64&e.effectTag);if(!r&&!a)return i&&Or(e,n,!1),So(t,e,o);r=e.stateNode,yo.current=e;var s=a&&"function"!=typeof n.getDerivedStateFromError?null:r.render();return e.effectTag|=1,null!==t&&a?(e.child=di(e,t.child,null,o),e.child=di(e,null,s,o)):vo(t,e,s,o),e.memoizedState=r.state,i&&Or(e,n,!0),e.child}function Mo(t){var e=t.stateNode;e.pendingContext?Fr(0,e.pendingContext,e.pendingContext!==e.context):e.context&&Fr(0,e.context,!1),Ei(t,e.containerInfo)}function ko(t,e,n){var r=e.mode,i=e.pendingProps,o=e.memoizedState;if(0==(64&e.effectTag)){o=null;var a=!1}else o={timedOutAt:null!==o?o.timedOutAt:0},a=!0,e.effectTag&=-65;if(null===t)if(a){var s=i.fallback;t=qr(null,r,0,null),0==(1&e.mode)&&(t.child=null!==e.memoizedState?e.child.child:e.child),r=qr(s,r,n,null),t.sibling=r,(n=t).return=r.return=e}else n=r=fi(e,null,i.children,n);else null!==t.memoizedState?(s=(r=t.child).sibling,a?(n=i.fallback,i=zr(r,r.pendingProps),0==(1&e.mode)&&((a=null!==e.memoizedState?e.child.child:e.child)!==r.child&&(i.child=a)),r=i.sibling=zr(s,n,s.expirationTime),n=i,i.childExpirationTime=0,n.return=r.return=e):n=r=di(e,r.child,i.children,n)):(s=t.child,a?(a=i.fallback,(i=qr(null,r,0,null)).child=s,0==(1&e.mode)&&(i.child=null!==e.memoizedState?e.child.child:e.child),(r=i.sibling=qr(a,r,n,null)).effectTag|=2,n=i,i.childExpirationTime=0,n.return=r.return=e):r=n=di(e,s,i.children,n)),e.stateNode=t.stateNode;return e.memoizedState=o,e.child=n,r}function So(t,e,n){if(null!==t&&(e.contextDependencies=t.contextDependencies),e.childExpirationTime<n)return null;if(null!==t&&e.child!==t.child&&a("153"),null!==e.child){for(n=zr(t=e.child,t.pendingProps,t.expirationTime),e.child=n,n.return=e;null!==t.sibling;)t=t.sibling,(n=n.sibling=zr(t,t.pendingProps,t.expirationTime)).return=e;n.sibling=null}return e.child}function Lo(t,e,n){var r=e.expirationTime;if(null!==t){if(t.memoizedProps!==e.pendingProps||Mr.current)Eo=!0;else if(r<n){switch(Eo=!1,e.tag){case 3:Mo(e),mo();break;case 5:Bi(e);break;case 1:Lr(e.type)&&Ur(e);break;case 4:Ei(e,e.stateNode.containerInfo);break;case 10:Uo(e,e.memoizedProps.value);break;case 13:if(null!==e.memoizedState)return 0!==(r=e.child.childExpirationTime)&&r>=n?ko(t,e,n):null!==(e=So(t,e,n))?e.sibling:null}return So(t,e,n)}}else Eo=!1;switch(e.expirationTime=0,e.tag){case 2:r=e.elementType,null!==t&&(t.alternate=null,e.alternate=null,e.effectTag|=2),t=e.pendingProps;var i=Sr(e,Dr.current);if(Go(e,n),i=Vi(null,e,r,t,i,n),e.effectTag|=1,"object"==typeof i&&null!==i&&"function"==typeof i.render&&void 0===i.$$typeof){if(e.tag=1,Wi(),Lr(r)){var o=!0;Ur(e)}else o=!1;e.memoizedState=null!==i.state&&void 0!==i.state?i.state:null;var s=r.getDerivedStateFromProps;"function"==typeof s&&ii(e,r,s,t),i.updater=oi,e.stateNode=i,i._reactInternalFiber=e,li(e,r,t,n),e=Do(null,e,r,!0,o,n)}else e.tag=0,vo(null,e,i,n),e=e.child;return e;case 16:switch(i=e.elementType,null!==t&&(t.alternate=null,e.alternate=null,e.effectTag|=2),o=e.pendingProps,t=function(t){var e=t._result;switch(t._status){case 1:return e;case 2:case 0:throw e;default:switch(t._status=0,(e=(e=t._ctor)()).then(function(e){0===t._status&&(e=e.default,t._status=1,t._result=e)},function(e){0===t._status&&(t._status=2,t._result=e)}),t._status){case 1:return t._result;case 2:throw t._result}throw t._result=e,e}}(i),e.type=t,i=e.tag=function(t){if("function"==typeof t)return Kr(t)?1:0;if(null!=t){if((t=t.$$typeof)===ee)return 11;if(t===re)return 14}return 2}(t),o=ni(t,o),s=void 0,i){case 0:s=xo(null,e,t,o,n);break;case 1:s=_o(null,e,t,o,n);break;case 11:s=Bo(null,e,t,o,n);break;case 14:s=bo(null,e,t,ni(t.type,o),r,n);break;default:a("306",t,"")}return s;case 0:return r=e.type,i=e.pendingProps,xo(t,e,r,i=e.elementType===r?i:ni(r,i),n);case 1:return r=e.type,i=e.pendingProps,_o(t,e,r,i=e.elementType===r?i:ni(r,i),n);case 3:return Mo(e),null===(r=e.updateQueue)&&a("282"),i=null!==(i=e.memoizedState)?i.element:null,ea(e,r,e.pendingProps,null,n),(r=e.memoizedState.element)===i?(mo(),e=So(t,e,n)):(i=e.stateNode,(i=(null===t||null===t.child)&&i.hydrate)&&(uo=Br(e.stateNode.containerInfo),co=e,i=go=!0),i?(e.effectTag|=2,e.child=fi(e,null,r,n)):(vo(t,e,r,n),mo()),e=e.child),e;case 5:return Bi(e),null===t&&po(e),r=e.type,i=e.pendingProps,o=null!==t?t.memoizedProps:null,s=i.children,Ir(r,i)?s=null:null!==o&&Ir(r,o)&&(e.effectTag|=16),wo(t,e),1!==n&&1&e.mode&&i.hidden?(e.expirationTime=e.childExpirationTime=1,e=null):(vo(t,e,s,n),e=e.child),e;case 6:return null===t&&po(e),null;case 13:return ko(t,e,n);case 4:return Ei(e,e.stateNode.containerInfo),r=e.pendingProps,null===t?e.child=di(e,null,r,n):vo(t,e,r,n),e.child;case 11:return r=e.type,i=e.pendingProps,Bo(t,e,r,i=e.elementType===r?i:ni(r,i),n);case 7:return vo(t,e,e.pendingProps,n),e.child;case 8:case 12:return vo(t,e,e.pendingProps.children,n),e.child;case 10:t:{if(r=e.type._context,i=e.pendingProps,s=e.memoizedProps,Uo(e,o=i.value),null!==s){var A=s.value;if(0===(o=Ze(A,o)?0:0|("function"==typeof r._calculateChangedBits?r._calculateChangedBits(A,o):1073741823))){if(s.children===i.children&&!Mr.current){e=So(t,e,n);break t}}else for(null!==(A=e.child)&&(A.return=e);null!==A;){var l=A.contextDependencies;if(null!==l){s=A.child;for(var c=l.first;null!==c;){if(c.context===r&&0!=(c.observedBits&o)){1===A.tag&&((c=Vo(n)).tag=Po,Xo(A,c)),A.expirationTime<n&&(A.expirationTime=n),null!==(c=A.alternate)&&c.expirationTime<n&&(c.expirationTime=n),c=n;for(var u=A.return;null!==u;){var g=u.alternate;if(u.childExpirationTime<c)u.childExpirationTime=c,null!==g&&g.childExpirationTime<c&&(g.childExpirationTime=c);else{if(!(null!==g&&g.childExpirationTime<c))break;g.childExpirationTime=c}u=u.return}l.expirationTime<n&&(l.expirationTime=n);break}c=c.next}}else s=10===A.tag&&A.type===e.type?null:A.child;if(null!==s)s.return=A;else for(s=A;null!==s;){if(s===e){s=null;break}if(null!==(A=s.sibling)){A.return=s.return,s=A;break}s=s.return}A=s}}vo(t,e,i.children,n),e=e.child}return e;case 9:return i=e.type,r=(o=e.pendingProps).children,Go(e,n),r=r(i=Ho(i,o.unstable_observedBits)),e.effectTag|=1,vo(t,e,r,n),e.child;case 14:return o=ni(i=e.type,e.pendingProps),bo(t,e,i,o=ni(i.type,o),r,n);case 15:return Qo(t,e,e.type,e.pendingProps,r,n);case 17:return r=e.type,i=e.pendingProps,i=e.elementType===r?i:ni(r,i),null!==t&&(t.alternate=null,e.alternate=null,e.effectTag|=2),e.tag=1,Lr(r)?(t=!0,Ur(e)):t=!1,Go(e,n),si(e,r,i),li(e,r,i,n),Do(null,e,r,!0,t,n)}a("156")}var Ro={current:null},No=null,Fo=null,To=null;function Uo(t,e){var n=t.type._context;xr(Ro,n._currentValue),n._currentValue=e}function Oo(t){var e=Ro.current;wr(Ro),t.type._context._currentValue=e}function Go(t,e){No=t,To=Fo=null;var n=t.contextDependencies;null!==n&&n.expirationTime>=e&&(Eo=!0),t.contextDependencies=null}function Ho(t,e){return To!==t&&!1!==e&&0!==e&&("number"==typeof e&&1073741823!==e||(To=t,e=1073741823),e={context:t,observedBits:e,next:null},null===Fo?(null===No&&a("308"),Fo=e,No.contextDependencies={first:e,expirationTime:0}):Fo=Fo.next=e),t._currentValue}var jo=0,Yo=1,Po=2,Ko=3,zo=!1;function Jo(t){return{baseState:t,firstUpdate:null,lastUpdate:null,firstCapturedUpdate:null,lastCapturedUpdate:null,firstEffect:null,lastEffect:null,firstCapturedEffect:null,lastCapturedEffect:null}}function qo(t){return{baseState:t.baseState,firstUpdate:t.firstUpdate,lastUpdate:t.lastUpdate,firstCapturedUpdate:null,lastCapturedUpdate:null,firstEffect:null,lastEffect:null,firstCapturedEffect:null,lastCapturedEffect:null}}function Vo(t){return{expirationTime:t,tag:jo,payload:null,callback:null,next:null,nextEffect:null}}function Wo(t,e){null===t.lastUpdate?t.firstUpdate=t.lastUpdate=e:(t.lastUpdate.next=e,t.lastUpdate=e)}function Xo(t,e){var n=t.alternate;if(null===n){var r=t.updateQueue,i=null;null===r&&(r=t.updateQueue=Jo(t.memoizedState))}else r=t.updateQueue,i=n.updateQueue,null===r?null===i?(r=t.updateQueue=Jo(t.memoizedState),i=n.updateQueue=Jo(n.memoizedState)):r=t.updateQueue=qo(i):null===i&&(i=n.updateQueue=qo(r));null===i||r===i?Wo(r,e):null===r.lastUpdate||null===i.lastUpdate?(Wo(r,e),Wo(i,e)):(Wo(r,e),i.lastUpdate=e)}function Zo(t,e){var n=t.updateQueue;null===(n=null===n?t.updateQueue=Jo(t.memoizedState):$o(t,n)).lastCapturedUpdate?n.firstCapturedUpdate=n.lastCapturedUpdate=e:(n.lastCapturedUpdate.next=e,n.lastCapturedUpdate=e)}function $o(t,e){var n=t.alternate;return null!==n&&e===n.updateQueue&&(e=t.updateQueue=qo(e)),e}function ta(t,e,n,r,o,a){switch(n.tag){case Yo:return"function"==typeof(t=n.payload)?t.call(a,r,o):t;case Ko:t.effectTag=-2049&t.effectTag|64;case jo:if(null==(o="function"==typeof(t=n.payload)?t.call(a,r,o):t))break;return i({},r,o);case Po:zo=!0}return r}function ea(t,e,n,r,i){zo=!1;for(var o=(e=$o(t,e)).baseState,a=null,s=0,A=e.firstUpdate,l=o;null!==A;){var c=A.expirationTime;c<i?(null===a&&(a=A,o=l),s<c&&(s=c)):(l=ta(t,0,A,l,n,r),null!==A.callback&&(t.effectTag|=32,A.nextEffect=null,null===e.lastEffect?e.firstEffect=e.lastEffect=A:(e.lastEffect.nextEffect=A,e.lastEffect=A))),A=A.next}for(c=null,A=e.firstCapturedUpdate;null!==A;){var u=A.expirationTime;u<i?(null===c&&(c=A,null===a&&(o=l)),s<u&&(s=u)):(l=ta(t,0,A,l,n,r),null!==A.callback&&(t.effectTag|=32,A.nextEffect=null,null===e.lastCapturedEffect?e.firstCapturedEffect=e.lastCapturedEffect=A:(e.lastCapturedEffect.nextEffect=A,e.lastCapturedEffect=A))),A=A.next}null===a&&(e.lastUpdate=null),null===c?e.lastCapturedUpdate=null:t.effectTag|=32,null===a&&null===c&&(o=l),e.baseState=o,e.firstUpdate=a,e.firstCapturedUpdate=c,t.expirationTime=s,t.memoizedState=l}function na(t,e,n){null!==e.firstCapturedUpdate&&(null!==e.lastUpdate&&(e.lastUpdate.next=e.firstCapturedUpdate,e.lastUpdate=e.lastCapturedUpdate),e.firstCapturedUpdate=e.lastCapturedUpdate=null),ra(e.firstEffect,n),e.firstEffect=e.lastEffect=null,ra(e.firstCapturedEffect,n),e.firstCapturedEffect=e.lastCapturedEffect=null}function ra(t,e){for(;null!==t;){var n=t.callback;if(null!==n){t.callback=null;var r=e;"function"!=typeof n&&a("191",n),n.call(r)}t=t.nextEffect}}function ia(t,e){return{value:t,source:e,stack:Ae(e)}}function oa(t){t.effectTag|=4}var aa=void 0,sa=void 0,Aa=void 0,la=void 0;aa=function(t,e){for(var n=e.child;null!==n;){if(5===n.tag||6===n.tag)t.appendChild(n.stateNode);else if(4!==n.tag&&null!==n.child){n.child.return=n,n=n.child;continue}if(n===e)break;for(;null===n.sibling;){if(null===n.return||n.return===e)return;n=n.return}n.sibling.return=n.return,n=n.sibling}},sa=function(){},Aa=function(t,e,n,r,o){var a=t.memoizedProps;if(a!==r){var s=e.stateNode;switch(yi(Ii.current),t=null,n){case"input":a=me(s,a),r=me(s,r),t=[];break;case"option":a=Jn(s,a),r=Jn(s,r),t=[];break;case"select":a=i({},a,{value:void 0}),r=i({},r,{value:void 0}),t=[];break;case"textarea":a=Vn(s,a),r=Vn(s,r),t=[];break;default:"function"!=typeof a.onClick&&"function"==typeof r.onClick&&(s.onclick=hr)}cr(n,r),s=n=void 0;var A=null;for(n in a)if(!r.hasOwnProperty(n)&&a.hasOwnProperty(n)&&null!=a[n])if("style"===n){var l=a[n];for(s in l)l.hasOwnProperty(s)&&(A||(A={}),A[s]="")}else"dangerouslySetInnerHTML"!==n&&"children"!==n&&"suppressContentEditableWarning"!==n&&"suppressHydrationWarning"!==n&&"autoFocus"!==n&&(m.hasOwnProperty(n)?t||(t=[]):(t=t||[]).push(n,null));for(n in r){var c=r[n];if(l=null!=a?a[n]:void 0,r.hasOwnProperty(n)&&c!==l&&(null!=c||null!=l))if("style"===n)if(l){for(s in l)!l.hasOwnProperty(s)||c&&c.hasOwnProperty(s)||(A||(A={}),A[s]="");for(s in c)c.hasOwnProperty(s)&&l[s]!==c[s]&&(A||(A={}),A[s]=c[s])}else A||(t||(t=[]),t.push(n,A)),A=c;else"dangerouslySetInnerHTML"===n?(c=c?c.__html:void 0,l=l?l.__html:void 0,null!=c&&l!==c&&(t=t||[]).push(n,""+c)):"children"===n?l===c||"string"!=typeof c&&"number"!=typeof c||(t=t||[]).push(n,""+c):"suppressContentEditableWarning"!==n&&"suppressHydrationWarning"!==n&&(m.hasOwnProperty(n)?(null!=c&&gr(o,n),t||l===c||(t=[])):(t=t||[]).push(n,c))}A&&(t=t||[]).push("style",A),o=t,(e.updateQueue=o)&&oa(e)}},la=function(t,e,n,r){n!==r&&oa(e)};var ca="function"==typeof WeakSet?WeakSet:Set;function ua(t,e){var n=e.source,r=e.stack;null===r&&null!==n&&(r=Ae(n)),null!==n&&se(n.type),e=e.value,null!==t&&1===t.tag&&se(t.type);try{console.error(e)}catch(t){setTimeout(function(){throw t})}}function ga(t){var e=t.ref;if(null!==e)if("function"==typeof e)try{e(null)}catch(e){Ja(t,e)}else e.current=null}function ha(t,e,n){if(null!==(n=null!==(n=n.updateQueue)?n.lastEffect:null)){var r=n=n.next;do{if((r.tag&t)!==Qi){var i=r.destroy;r.destroy=void 0,void 0!==i&&i()}(r.tag&e)!==Qi&&(i=r.create,r.destroy=i()),r=r.next}while(r!==n)}}function da(t){switch("function"==typeof Hr&&Hr(t),t.tag){case 0:case 11:case 14:case 15:var e=t.updateQueue;if(null!==e&&null!==(e=e.lastEffect)){var n=e=e.next;do{var r=n.destroy;if(void 0!==r){var i=t;try{r()}catch(t){Ja(i,t)}}n=n.next}while(n!==e)}break;case 1:if(ga(t),"function"==typeof(e=t.stateNode).componentWillUnmount)try{e.props=t.memoizedProps,e.state=t.memoizedState,e.componentWillUnmount()}catch(e){Ja(t,e)}break;case 5:ga(t);break;case 4:Ia(t)}}function fa(t){return 5===t.tag||3===t.tag||4===t.tag}function pa(t){t:{for(var e=t.return;null!==e;){if(fa(e)){var n=e;break t}e=e.return}a("160"),n=void 0}var r=e=void 0;switch(n.tag){case 5:e=n.stateNode,r=!1;break;case 3:case 4:e=n.stateNode.containerInfo,r=!0;break;default:a("161")}16&n.effectTag&&(ir(e,""),n.effectTag&=-17);t:e:for(n=t;;){for(;null===n.sibling;){if(null===n.return||fa(n.return)){n=null;break t}n=n.return}for(n.sibling.return=n.return,n=n.sibling;5!==n.tag&&6!==n.tag&&18!==n.tag;){if(2&n.effectTag)continue e;if(null===n.child||4===n.tag)continue e;n.child.return=n,n=n.child}if(!(2&n.effectTag)){n=n.stateNode;break t}}for(var i=t;;){if(5===i.tag||6===i.tag)if(n)if(r){var o=e,s=i.stateNode,A=n;8===o.nodeType?o.parentNode.insertBefore(s,A):o.insertBefore(s,A)}else e.insertBefore(i.stateNode,n);else r?(s=e,A=i.stateNode,8===s.nodeType?(o=s.parentNode).insertBefore(A,s):(o=s).appendChild(A),null!=(s=s._reactRootContainer)||null!==o.onclick||(o.onclick=hr)):e.appendChild(i.stateNode);else if(4!==i.tag&&null!==i.child){i.child.return=i,i=i.child;continue}if(i===t)break;for(;null===i.sibling;){if(null===i.return||i.return===t)return;i=i.return}i.sibling.return=i.return,i=i.sibling}}function Ia(t){for(var e=t,n=!1,r=void 0,i=void 0;;){if(!n){n=e.return;t:for(;;){switch(null===n&&a("160"),n.tag){case 5:r=n.stateNode,i=!1;break t;case 3:case 4:r=n.stateNode.containerInfo,i=!0;break t}n=n.return}n=!0}if(5===e.tag||6===e.tag){t:for(var o=e,s=o;;)if(da(s),null!==s.child&&4!==s.tag)s.child.return=s,s=s.child;else{if(s===o)break;for(;null===s.sibling;){if(null===s.return||s.return===o)break t;s=s.return}s.sibling.return=s.return,s=s.sibling}i?(o=r,s=e.stateNode,8===o.nodeType?o.parentNode.removeChild(s):o.removeChild(s)):r.removeChild(e.stateNode)}else if(4===e.tag){if(null!==e.child){r=e.stateNode.containerInfo,i=!0,e.child.return=e,e=e.child;continue}}else if(da(e),null!==e.child){e.child.return=e,e=e.child;continue}if(e===t)break;for(;null===e.sibling;){if(null===e.return||e.return===t)return;4===(e=e.return).tag&&(n=!1)}e.sibling.return=e.return,e=e.sibling}}function Ca(t,e){switch(e.tag){case 0:case 11:case 14:case 15:ha(xi,_i,e);break;case 1:break;case 5:var n=e.stateNode;if(null!=n){var r=e.memoizedProps;t=null!==t?t.memoizedProps:r;var i=e.type,o=e.updateQueue;e.updateQueue=null,null!==o&&function(t,e,n,r,i){t[R]=i,"input"===n&&"radio"===i.type&&null!=i.name&&Ee(t,i),ur(n,r),r=ur(n,i);for(var o=0;o<e.length;o+=2){var a=e[o],s=e[o+1];"style"===a?Ar(t,s):"dangerouslySetInnerHTML"===a?rr(t,s):"children"===a?ir(t,s):Ie(t,a,s,r)}switch(n){case"input":ve(t,i);break;case"textarea":Xn(t,i);break;case"select":e=t._wrapperState.wasMultiple,t._wrapperState.wasMultiple=!!i.multiple,null!=(n=i.value)?qn(t,!!i.multiple,n,!1):e!==!!i.multiple&&(null!=i.defaultValue?qn(t,!!i.multiple,i.defaultValue,!0):qn(t,!!i.multiple,i.multiple?[]:"",!1))}}(n,o,i,t,r)}break;case 6:null===e.stateNode&&a("162"),e.stateNode.nodeValue=e.memoizedProps;break;case 3:case 12:break;case 13:if(n=e.memoizedState,r=void 0,t=e,null===n?r=!1:(r=!0,t=e.child,0===n.timedOutAt&&(n.timedOutAt=Es())),null!==t&&function(t,e){for(var n=t;;){if(5===n.tag){var r=n.stateNode;if(e)r.style.display="none";else{r=n.stateNode;var i=n.memoizedProps.style;i=null!=i&&i.hasOwnProperty("display")?i.display:null,r.style.display=sr("display",i)}}else if(6===n.tag)n.stateNode.nodeValue=e?"":n.memoizedProps;else{if(13===n.tag&&null!==n.memoizedState){(r=n.child.sibling).return=n,n=r;continue}if(null!==n.child){n.child.return=n,n=n.child;continue}}if(n===t)break;for(;null===n.sibling;){if(null===n.return||n.return===t)return;n=n.return}n.sibling.return=n.return,n=n.sibling}}(t,r),null!==(n=e.updateQueue)){e.updateQueue=null;var s=e.stateNode;null===s&&(s=e.stateNode=new ca),n.forEach(function(t){var n=function(t,e){var n=t.stateNode;null!==n&&n.delete(e),e=qa(e=Es(),t),null!==(t=Wa(t,e))&&(Zr(t,e),0!==(e=t.expirationTime)&&vs(t,e))}.bind(null,e,t);s.has(t)||(s.add(t),t.then(n,n))})}break;case 17:break;default:a("163")}}var ma="function"==typeof WeakMap?WeakMap:Map;function ya(t,e,n){(n=Vo(n)).tag=Ko,n.payload={element:null};var r=e.value;return n.callback=function(){ks(r),ua(t,e)},n}function Ea(t,e,n){(n=Vo(n)).tag=Ko;var r=t.type.getDerivedStateFromError;if("function"==typeof r){var i=e.value;n.payload=function(){return r(i)}}var o=t.stateNode;return null!==o&&"function"==typeof o.componentDidCatch&&(n.callback=function(){"function"!=typeof r&&(null===Ta?Ta=new Set([this]):Ta.add(this));var n=e.value,i=e.stack;ua(t,e),this.componentDidCatch(n,{componentStack:null!==i?i:""})}),n}function va(t){switch(t.tag){case 1:Lr(t.type)&&Rr();var e=t.effectTag;return 2048&e?(t.effectTag=-2049&e|64,t):null;case 3:return vi(),Nr(),0!=(64&(e=t.effectTag))&&a("285"),t.effectTag=-2049&e|64,t;case 5:return bi(t),null;case 13:return 2048&(e=t.effectTag)?(t.effectTag=-2049&e|64,t):null;case 18:return null;case 4:return vi(),null;case 10:return Oo(t),null;default:return null}}var Ba=Pt.ReactCurrentDispatcher,ba=Pt.ReactCurrentOwner,Qa=1073741822,wa=!1,xa=null,_a=null,Da=0,Ma=-1,ka=!1,Sa=null,La=!1,Ra=null,Na=null,Fa=null,Ta=null;function Ua(){if(null!==xa)for(var t=xa.return;null!==t;){var e=t;switch(e.tag){case 1:var n=e.type.childContextTypes;null!=n&&Rr();break;case 3:vi(),Nr();break;case 5:bi(e);break;case 4:vi();break;case 10:Oo(e)}t=t.return}_a=null,Da=0,Ma=-1,ka=!1,xa=null}function Oa(){for(;null!==Sa;){var t=Sa.effectTag;if(16&t&&ir(Sa.stateNode,""),128&t){var e=Sa.alternate;null!==e&&(null!==(e=e.ref)&&("function"==typeof e?e(null):e.current=null))}switch(14&t){case 2:pa(Sa),Sa.effectTag&=-3;break;case 6:pa(Sa),Sa.effectTag&=-3,Ca(Sa.alternate,Sa);break;case 4:Ca(Sa.alternate,Sa);break;case 8:Ia(t=Sa),t.return=null,t.child=null,t.memoizedState=null,t.updateQueue=null,null!==(t=t.alternate)&&(t.return=null,t.child=null,t.memoizedState=null,t.updateQueue=null)}Sa=Sa.nextEffect}}function Ga(){for(;null!==Sa;){if(256&Sa.effectTag)t:{var t=Sa.alternate,e=Sa;switch(e.tag){case 0:case 11:case 15:ha(wi,Qi,e);break t;case 1:if(256&e.effectTag&&null!==t){var n=t.memoizedProps,r=t.memoizedState;e=(t=e.stateNode).getSnapshotBeforeUpdate(e.elementType===e.type?n:ni(e.type,n),r),t.__reactInternalSnapshotBeforeUpdate=e}break t;case 3:case 5:case 6:case 4:case 17:break t;default:a("163")}}Sa=Sa.nextEffect}}function Ha(t,e){for(;null!==Sa;){var n=Sa.effectTag;if(36&n){var r=Sa.alternate,i=Sa,o=e;switch(i.tag){case 0:case 11:case 15:ha(Di,Mi,i);break;case 1:var s=i.stateNode;if(4&i.effectTag)if(null===r)s.componentDidMount();else{var A=i.elementType===i.type?r.memoizedProps:ni(i.type,r.memoizedProps);s.componentDidUpdate(A,r.memoizedState,s.__reactInternalSnapshotBeforeUpdate)}null!==(r=i.updateQueue)&&na(0,r,s);break;case 3:if(null!==(r=i.updateQueue)){if(s=null,null!==i.child)switch(i.child.tag){case 5:s=i.child.stateNode;break;case 1:s=i.child.stateNode}na(0,r,s)}break;case 5:o=i.stateNode,null===r&&4&i.effectTag&&pr(i.type,i.memoizedProps)&&o.focus();break;case 6:case 4:case 12:case 13:case 17:break;default:a("163")}}128&n&&(null!==(i=Sa.ref)&&(o=Sa.stateNode,"function"==typeof i?i(o):i.current=o)),512&n&&(Ra=t),Sa=Sa.nextEffect}}function ja(){null!==Na&&Er(Na),null!==Fa&&Fa()}function Ya(t,e){La=wa=!0,t.current===e&&a("177");var n=t.pendingCommitExpirationTime;0===n&&a("261"),t.pendingCommitExpirationTime=0;var r=e.expirationTime,i=e.childExpirationTime;for(function(t,e){if(t.didError=!1,0===e)t.earliestPendingTime=0,t.latestPendingTime=0,t.earliestSuspendedTime=0,t.latestSuspendedTime=0,t.latestPingedTime=0;else{e<t.latestPingedTime&&(t.latestPingedTime=0);var n=t.latestPendingTime;0!==n&&(n>e?t.earliestPendingTime=t.latestPendingTime=0:t.earliestPendingTime>e&&(t.earliestPendingTime=t.latestPendingTime)),0===(n=t.earliestSuspendedTime)?Zr(t,e):e<t.latestSuspendedTime?(t.earliestSuspendedTime=0,t.latestSuspendedTime=0,t.latestPingedTime=0,Zr(t,e)):e>n&&Zr(t,e)}ei(0,t)}(t,i>r?i:r),ba.current=null,r=void 0,1<e.effectTag?null!==e.lastEffect?(e.lastEffect.nextEffect=e,r=e.firstEffect):r=e:r=e.firstEffect,dr=bn,fr=function(){var t=Fn();if(Tn(t)){if("selectionStart"in t)var e={start:t.selectionStart,end:t.selectionEnd};else t:{var n=(e=(e=t.ownerDocument)&&e.defaultView||window).getSelection&&e.getSelection();if(n&&0!==n.rangeCount){e=n.anchorNode;var r=n.anchorOffset,i=n.focusNode;n=n.focusOffset;try{e.nodeType,i.nodeType}catch(t){e=null;break t}var o=0,a=-1,s=-1,A=0,l=0,c=t,u=null;e:for(;;){for(var g;c!==e||0!==r&&3!==c.nodeType||(a=o+r),c!==i||0!==n&&3!==c.nodeType||(s=o+n),3===c.nodeType&&(o+=c.nodeValue.length),null!==(g=c.firstChild);)u=c,c=g;for(;;){if(c===t)break e;if(u===e&&++A===r&&(a=o),u===i&&++l===n&&(s=o),null!==(g=c.nextSibling))break;u=(c=u).parentNode}c=g}e=-1===a||-1===s?null:{start:a,end:s}}else e=null}e=e||{start:0,end:0}}else e=null;return{focusedElem:t,selectionRange:e}}(),bn=!1,Sa=r;null!==Sa;){i=!1;var s=void 0;try{Ga()}catch(t){i=!0,s=t}i&&(null===Sa&&a("178"),Ja(Sa,s),null!==Sa&&(Sa=Sa.nextEffect))}for(Sa=r;null!==Sa;){i=!1,s=void 0;try{Oa()}catch(t){i=!0,s=t}i&&(null===Sa&&a("178"),Ja(Sa,s),null!==Sa&&(Sa=Sa.nextEffect))}for(Un(fr),fr=null,bn=!!dr,dr=null,t.current=e,Sa=r;null!==Sa;){i=!1,s=void 0;try{Ha(t,n)}catch(t){i=!0,s=t}i&&(null===Sa&&a("178"),Ja(Sa,s),null!==Sa&&(Sa=Sa.nextEffect))}if(null!==r&&null!==Ra){var A=function(t,e){Fa=Na=Ra=null;var n=rs;rs=!0;do{if(512&e.effectTag){var r=!1,i=void 0;try{var o=e;ha(Si,Qi,o),ha(Qi,ki,o)}catch(t){r=!0,i=t}r&&Ja(e,i)}e=e.nextEffect}while(null!==e);rs=n,0!==(n=t.expirationTime)&&vs(t,n),ls||rs||xs(1073741823,!1)}.bind(null,t,r);Na=o.unstable_runWithPriority(o.unstable_NormalPriority,function(){return yr(A)}),Fa=A}wa=La=!1,"function"==typeof Gr&&Gr(e.stateNode),n=e.expirationTime,0===(e=(e=e.childExpirationTime)>n?e:n)&&(Ta=null),function(t,e){t.expirationTime=e,t.finishedWork=null}(t,e)}function Pa(t){for(;;){var e=t.alternate,n=t.return,r=t.sibling;if(0==(1024&t.effectTag)){xa=t;t:{var o=e,s=Da,A=(e=t).pendingProps;switch(e.tag){case 2:case 16:break;case 15:case 0:break;case 1:Lr(e.type)&&Rr();break;case 3:vi(),Nr(),(A=e.stateNode).pendingContext&&(A.context=A.pendingContext,A.pendingContext=null),null!==o&&null!==o.child||(Co(e),e.effectTag&=-3),sa(e);break;case 5:bi(e);var l=yi(mi.current);if(s=e.type,null!==o&&null!=e.stateNode)Aa(o,e,s,A,l),o.ref!==e.ref&&(e.effectTag|=128);else if(A){var c=yi(Ii.current);if(Co(e)){o=(A=e).stateNode;var u=A.type,g=A.memoizedProps,h=l;switch(o[L]=A,o[R]=g,s=void 0,l=u){case"iframe":case"object":Qn("load",o);break;case"video":case"audio":for(u=0;u<et.length;u++)Qn(et[u],o);break;case"source":Qn("error",o);break;case"img":case"image":case"link":Qn("error",o),Qn("load",o);break;case"form":Qn("reset",o),Qn("submit",o);break;case"details":Qn("toggle",o);break;case"input":ye(o,g),Qn("invalid",o),gr(h,"onChange");break;case"select":o._wrapperState={wasMultiple:!!g.multiple},Qn("invalid",o),gr(h,"onChange");break;case"textarea":Wn(o,g),Qn("invalid",o),gr(h,"onChange")}for(s in cr(l,g),u=null,g)g.hasOwnProperty(s)&&(c=g[s],"children"===s?"string"==typeof c?o.textContent!==c&&(u=["children",c]):"number"==typeof c&&o.textContent!==""+c&&(u=["children",""+c]):m.hasOwnProperty(s)&&null!=c&&gr(h,s));switch(l){case"input":jt(o),Be(o,g,!0);break;case"textarea":jt(o),Zn(o);break;case"select":case"option":break;default:"function"==typeof g.onClick&&(o.onclick=hr)}s=u,A.updateQueue=s,(A=null!==s)&&oa(e)}else{g=e,h=s,o=A,u=9===l.nodeType?l:l.ownerDocument,c===$n.html&&(c=tr(h)),c===$n.html?"script"===h?((o=u.createElement("div")).innerHTML="<script><\/script>",u=o.removeChild(o.firstChild)):"string"==typeof o.is?u=u.createElement(h,{is:o.is}):(u=u.createElement(h),"select"===h&&(h=u,o.multiple?h.multiple=!0:o.size&&(h.size=o.size))):u=u.createElementNS(c,h),(o=u)[L]=g,o[R]=A,aa(o,e,!1,!1),h=o;var d=l,f=ur(u=s,g=A);switch(u){case"iframe":case"object":Qn("load",h),l=g;break;case"video":case"audio":for(l=0;l<et.length;l++)Qn(et[l],h);l=g;break;case"source":Qn("error",h),l=g;break;case"img":case"image":case"link":Qn("error",h),Qn("load",h),l=g;break;case"form":Qn("reset",h),Qn("submit",h),l=g;break;case"details":Qn("toggle",h),l=g;break;case"input":ye(h,g),l=me(h,g),Qn("invalid",h),gr(d,"onChange");break;case"option":l=Jn(h,g);break;case"select":h._wrapperState={wasMultiple:!!g.multiple},l=i({},g,{value:void 0}),Qn("invalid",h),gr(d,"onChange");break;case"textarea":Wn(h,g),l=Vn(h,g),Qn("invalid",h),gr(d,"onChange");break;default:l=g}cr(u,l),c=void 0;var p=u,I=h,C=l;for(c in C)if(C.hasOwnProperty(c)){var y=C[c];"style"===c?Ar(I,y):"dangerouslySetInnerHTML"===c?null!=(y=y?y.__html:void 0)&&rr(I,y):"children"===c?"string"==typeof y?("textarea"!==p||""!==y)&&ir(I,y):"number"==typeof y&&ir(I,""+y):"suppressContentEditableWarning"!==c&&"suppressHydrationWarning"!==c&&"autoFocus"!==c&&(m.hasOwnProperty(c)?null!=y&&gr(d,c):null!=y&&Ie(I,c,y,f))}switch(u){case"input":jt(h),Be(h,g,!1);break;case"textarea":jt(h),Zn(h);break;case"option":null!=g.value&&h.setAttribute("value",""+Ce(g.value));break;case"select":(l=h).multiple=!!g.multiple,null!=(h=g.value)?qn(l,!!g.multiple,h,!1):null!=g.defaultValue&&qn(l,!!g.multiple,g.defaultValue,!0);break;default:"function"==typeof l.onClick&&(h.onclick=hr)}(A=pr(s,A))&&oa(e),e.stateNode=o}null!==e.ref&&(e.effectTag|=128)}else null===e.stateNode&&a("166");break;case 6:o&&null!=e.stateNode?la(o,e,o.memoizedProps,A):("string"!=typeof A&&(null===e.stateNode&&a("166")),o=yi(mi.current),yi(Ii.current),Co(e)?(s=(A=e).stateNode,o=A.memoizedProps,s[L]=A,(A=s.nodeValue!==o)&&oa(e)):(s=e,(A=(9===o.nodeType?o:o.ownerDocument).createTextNode(A))[L]=e,s.stateNode=A));break;case 11:break;case 13:if(A=e.memoizedState,0!=(64&e.effectTag)){e.expirationTime=s,xa=e;break t}A=null!==A,s=null!==o&&null!==o.memoizedState,null!==o&&!A&&s&&(null!==(o=o.child.sibling)&&(null!==(l=e.firstEffect)?(e.firstEffect=o,o.nextEffect=l):(e.firstEffect=e.lastEffect=o,o.nextEffect=null),o.effectTag=8)),(A||s)&&(e.effectTag|=4);break;case 7:case 8:case 12:break;case 4:vi(),sa(e);break;case 10:Oo(e);break;case 9:case 14:break;case 17:Lr(e.type)&&Rr();break;case 18:break;default:a("156")}xa=null}if(e=t,1===Da||1!==e.childExpirationTime){for(A=0,s=e.child;null!==s;)(o=s.expirationTime)>A&&(A=o),(l=s.childExpirationTime)>A&&(A=l),s=s.sibling;e.childExpirationTime=A}if(null!==xa)return xa;null!==n&&0==(1024&n.effectTag)&&(null===n.firstEffect&&(n.firstEffect=t.firstEffect),null!==t.lastEffect&&(null!==n.lastEffect&&(n.lastEffect.nextEffect=t.firstEffect),n.lastEffect=t.lastEffect),1<t.effectTag&&(null!==n.lastEffect?n.lastEffect.nextEffect=t:n.firstEffect=t,n.lastEffect=t))}else{if(null!==(t=va(t)))return t.effectTag&=1023,t;null!==n&&(n.firstEffect=n.lastEffect=null,n.effectTag|=1024)}if(null!==r)return r;if(null===n)break;t=n}return null}function Ka(t){var e=Lo(t.alternate,t,Da);return t.memoizedProps=t.pendingProps,null===e&&(e=Pa(t)),ba.current=null,e}function za(t,e){wa&&a("243"),ja(),wa=!0;var n=Ba.current;Ba.current=so;var r=t.nextExpirationTimeToWorkOn;r===Da&&t===_a&&null!==xa||(Ua(),Da=r,xa=zr((_a=t).current,null),t.pendingCommitExpirationTime=0);for(var i=!1;;){try{if(e)for(;null!==xa&&!Qs();)xa=Ka(xa);else for(;null!==xa;)xa=Ka(xa)}catch(e){if(To=Fo=No=null,Wi(),null===xa)i=!0,ks(e);else{null===xa&&a("271");var o=xa,s=o.return;if(null!==s){t:{var A=t,l=s,c=o,u=e;if(s=Da,c.effectTag|=1024,c.firstEffect=c.lastEffect=null,null!==u&&"object"==typeof u&&"function"==typeof u.then){var g=u;u=l;var h=-1,d=-1;do{if(13===u.tag){var f=u.alternate;if(null!==f&&null!==(f=f.memoizedState)){d=10*(1073741822-f.timedOutAt);break}"number"==typeof(f=u.pendingProps.maxDuration)&&(0>=f?h=0:(-1===h||f<h)&&(h=f))}u=u.return}while(null!==u);u=l;do{if((f=13===u.tag)&&(f=void 0!==u.memoizedProps.fallback&&null===u.memoizedState),f){if(null===(l=u.updateQueue)?((l=new Set).add(g),u.updateQueue=l):l.add(g),0==(1&u.mode)){u.effectTag|=64,c.effectTag&=-1957,1===c.tag&&(null===c.alternate?c.tag=17:((s=Vo(1073741823)).tag=Po,Xo(c,s))),c.expirationTime=1073741823;break t}l=s;var p=(c=A).pingCache;null===p?(p=c.pingCache=new ma,f=new Set,p.set(g,f)):void 0===(f=p.get(g))&&(f=new Set,p.set(g,f)),f.has(l)||(f.add(l),c=Va.bind(null,c,g,l),g.then(c,c)),-1===h?A=1073741823:(-1===d&&(d=10*(1073741822-ti(A,s))-5e3),A=d+h),0<=A&&Ma<A&&(Ma=A),u.effectTag|=2048,u.expirationTime=s;break t}u=u.return}while(null!==u);u=Error((se(c.type)||"A React component")+" suspended while rendering, but no fallback UI was specified.\n\nAdd a <Suspense fallback=...> component higher in the tree to provide a loading indicator or placeholder to display."+Ae(c))}ka=!0,u=ia(u,c),A=l;do{switch(A.tag){case 3:A.effectTag|=2048,A.expirationTime=s,Zo(A,s=ya(A,u,s));break t;case 1:if(h=u,d=A.type,c=A.stateNode,0==(64&A.effectTag)&&("function"==typeof d.getDerivedStateFromError||null!==c&&"function"==typeof c.componentDidCatch&&(null===Ta||!Ta.has(c)))){A.effectTag|=2048,A.expirationTime=s,Zo(A,s=Ea(A,h,s));break t}}A=A.return}while(null!==A)}xa=Pa(o);continue}i=!0,ks(e)}}break}if(wa=!1,Ba.current=n,To=Fo=No=null,Wi(),i)_a=null,t.finishedWork=null;else if(null!==xa)t.finishedWork=null;else{if(null===(n=t.current.alternate)&&a("281"),_a=null,ka){if(i=t.latestPendingTime,o=t.latestSuspendedTime,s=t.latestPingedTime,0!==i&&i<r||0!==o&&o<r||0!==s&&s<r)return $r(t,r),void ys(t,n,r,t.expirationTime,-1);if(!t.didError&&e)return t.didError=!0,r=t.nextExpirationTimeToWorkOn=r,e=t.expirationTime=1073741823,void ys(t,n,r,e,-1)}e&&-1!==Ma?($r(t,r),(e=10*(1073741822-ti(t,r)))<Ma&&(Ma=e),e=10*(1073741822-Es()),e=Ma-e,ys(t,n,r,t.expirationTime,0>e?0:e)):(t.pendingCommitExpirationTime=r,t.finishedWork=n)}}function Ja(t,e){for(var n=t.return;null!==n;){switch(n.tag){case 1:var r=n.stateNode;if("function"==typeof n.type.getDerivedStateFromError||"function"==typeof r.componentDidCatch&&(null===Ta||!Ta.has(r)))return Xo(n,t=Ea(n,t=ia(e,t),1073741823)),void Xa(n,1073741823);break;case 3:return Xo(n,t=ya(n,t=ia(e,t),1073741823)),void Xa(n,1073741823)}n=n.return}3===t.tag&&(Xo(t,n=ya(t,n=ia(e,t),1073741823)),Xa(t,1073741823))}function qa(t,e){var n=o.unstable_getCurrentPriorityLevel(),r=void 0;if(0==(1&e.mode))r=1073741823;else if(wa&&!La)r=Da;else{switch(n){case o.unstable_ImmediatePriority:r=1073741823;break;case o.unstable_UserBlockingPriority:r=1073741822-10*(1+((1073741822-t+15)/10|0));break;case o.unstable_NormalPriority:r=1073741822-25*(1+((1073741822-t+500)/25|0));break;case o.unstable_LowPriority:case o.unstable_IdlePriority:r=1;break;default:a("313")}null!==_a&&r===Da&&--r}return n===o.unstable_UserBlockingPriority&&(0===as||r<as)&&(as=r),r}function Va(t,e,n){var r=t.pingCache;null!==r&&r.delete(e),null!==_a&&Da===n?_a=null:(e=t.earliestSuspendedTime,r=t.latestSuspendedTime,0!==e&&n<=e&&n>=r&&(t.didError=!1,(0===(e=t.latestPingedTime)||e>n)&&(t.latestPingedTime=n),ei(n,t),0!==(n=t.expirationTime)&&vs(t,n)))}function Wa(t,e){t.expirationTime<e&&(t.expirationTime=e);var n=t.alternate;null!==n&&n.expirationTime<e&&(n.expirationTime=e);var r=t.return,i=null;if(null===r&&3===t.tag)i=t.stateNode;else for(;null!==r;){if(n=r.alternate,r.childExpirationTime<e&&(r.childExpirationTime=e),null!==n&&n.childExpirationTime<e&&(n.childExpirationTime=e),null===r.return&&3===r.tag){i=r.stateNode;break}r=r.return}return i}function Xa(t,e){null!==(t=Wa(t,e))&&(!wa&&0!==Da&&e>Da&&Ua(),Zr(t,e),wa&&!La&&_a===t||vs(t,t.expirationTime),ps>fs&&(ps=0,a("185")))}function Za(t,e,n,r,i){return o.unstable_runWithPriority(o.unstable_ImmediatePriority,function(){return t(e,n,r,i)})}var $a=null,ts=null,es=0,ns=void 0,rs=!1,is=null,os=0,as=0,ss=!1,As=null,ls=!1,cs=!1,us=null,gs=o.unstable_now(),hs=1073741822-(gs/10|0),ds=hs,fs=50,ps=0,Is=null;function Cs(){hs=1073741822-((o.unstable_now()-gs)/10|0)}function ms(t,e){if(0!==es){if(e<es)return;null!==ns&&o.unstable_cancelCallback(ns)}es=e,t=o.unstable_now()-gs,ns=o.unstable_scheduleCallback(ws,{timeout:10*(1073741822-e)-t})}function ys(t,e,n,r,i){t.expirationTime=r,0!==i||Qs()?0<i&&(t.timeoutHandle=Cr(function(t,e,n){t.pendingCommitExpirationTime=n,t.finishedWork=e,Cs(),ds=hs,_s(t,n)}.bind(null,t,e,n),i)):(t.pendingCommitExpirationTime=n,t.finishedWork=e)}function Es(){return rs?ds:(Bs(),0!==os&&1!==os||(Cs(),ds=hs),ds)}function vs(t,e){null===t.nextScheduledRoot?(t.expirationTime=e,null===ts?($a=ts=t,t.nextScheduledRoot=t):(ts=ts.nextScheduledRoot=t).nextScheduledRoot=$a):e>t.expirationTime&&(t.expirationTime=e),rs||(ls?cs&&(is=t,os=1073741823,Ds(t,1073741823,!1)):1073741823===e?xs(1073741823,!1):ms(t,e))}function Bs(){var t=0,e=null;if(null!==ts)for(var n=ts,r=$a;null!==r;){var i=r.expirationTime;if(0===i){if((null===n||null===ts)&&a("244"),r===r.nextScheduledRoot){$a=ts=r.nextScheduledRoot=null;break}if(r===$a)$a=i=r.nextScheduledRoot,ts.nextScheduledRoot=i,r.nextScheduledRoot=null;else{if(r===ts){(ts=n).nextScheduledRoot=$a,r.nextScheduledRoot=null;break}n.nextScheduledRoot=r.nextScheduledRoot,r.nextScheduledRoot=null}r=n.nextScheduledRoot}else{if(i>t&&(t=i,e=r),r===ts)break;if(1073741823===t)break;n=r,r=r.nextScheduledRoot}}is=e,os=t}var bs=!1;function Qs(){return!!bs||!!o.unstable_shouldYield()&&(bs=!0)}function ws(){try{if(!Qs()&&null!==$a){Cs();var t=$a;do{var e=t.expirationTime;0!==e&&hs<=e&&(t.nextExpirationTimeToWorkOn=hs),t=t.nextScheduledRoot}while(t!==$a)}xs(0,!0)}finally{bs=!1}}function xs(t,e){if(Bs(),e)for(Cs(),ds=hs;null!==is&&0!==os&&t<=os&&!(bs&&hs>os);)Ds(is,os,hs>os),Bs(),Cs(),ds=hs;else for(;null!==is&&0!==os&&t<=os;)Ds(is,os,!1),Bs();if(e&&(es=0,ns=null),0!==os&&ms(is,os),ps=0,Is=null,null!==us)for(t=us,us=null,e=0;e<t.length;e++){var n=t[e];try{n._onComplete()}catch(t){ss||(ss=!0,As=t)}}if(ss)throw t=As,As=null,ss=!1,t}function _s(t,e){rs&&a("253"),is=t,os=e,Ds(t,e,!1),xs(1073741823,!1)}function Ds(t,e,n){if(rs&&a("245"),rs=!0,n){var r=t.finishedWork;null!==r?Ms(t,r,e):(t.finishedWork=null,-1!==(r=t.timeoutHandle)&&(t.timeoutHandle=-1,mr(r)),za(t,n),null!==(r=t.finishedWork)&&(Qs()?t.finishedWork=r:Ms(t,r,e)))}else null!==(r=t.finishedWork)?Ms(t,r,e):(t.finishedWork=null,-1!==(r=t.timeoutHandle)&&(t.timeoutHandle=-1,mr(r)),za(t,n),null!==(r=t.finishedWork)&&Ms(t,r,e));rs=!1}function Ms(t,e,n){var r=t.firstBatch;if(null!==r&&r._expirationTime>=n&&(null===us?us=[r]:us.push(r),r._defer))return t.finishedWork=e,void(t.expirationTime=0);t.finishedWork=null,t===Is?ps++:(Is=t,ps=0),o.unstable_runWithPriority(o.unstable_ImmediatePriority,function(){Ya(t,e)})}function ks(t){null===is&&a("246"),is.expirationTime=0,ss||(ss=!0,As=t)}function Ss(t,e){var n=ls;ls=!0;try{return t(e)}finally{(ls=n)||rs||xs(1073741823,!1)}}function Ls(t,e){if(ls&&!cs){cs=!0;try{return t(e)}finally{cs=!1}}return t(e)}function Rs(t,e,n){ls||rs||0===as||(xs(as,!1),as=0);var r=ls;ls=!0;try{return o.unstable_runWithPriority(o.unstable_UserBlockingPriority,function(){return t(e,n)})}finally{(ls=r)||rs||xs(1073741823,!1)}}function Ns(t,e,n,r,i){var o=e.current;t:if(n){e:{2===en(n=n._reactInternalFiber)&&1===n.tag||a("170");var s=n;do{switch(s.tag){case 3:s=s.stateNode.context;break e;case 1:if(Lr(s.type)){s=s.stateNode.__reactInternalMemoizedMergedChildContext;break e}}s=s.return}while(null!==s);a("171"),s=void 0}if(1===n.tag){var A=n.type;if(Lr(A)){n=Tr(n,A,s);break t}}n=s}else n=_r;return null===e.context?e.context=n:e.pendingContext=n,e=i,(i=Vo(r)).payload={element:t},null!==(e=void 0===e?null:e)&&(i.callback=e),ja(),Xo(o,i),Xa(o,r),r}function Fs(t,e,n,r){var i=e.current;return Ns(t,e,n,i=qa(Es(),i),r)}function Ts(t){if(!(t=t.current).child)return null;switch(t.child.tag){case 5:default:return t.child.stateNode}}function Us(t){var e=1073741822-25*(1+((1073741822-Es()+500)/25|0));e>=Qa&&(e=Qa-1),this._expirationTime=Qa=e,this._root=t,this._callbacks=this._next=null,this._hasChildren=this._didComplete=!1,this._children=null,this._defer=!0}function Os(){this._callbacks=null,this._didCommit=!1,this._onCommit=this._onCommit.bind(this)}function Gs(t,e,n){t={current:e=Pr(3,null,null,e?3:0),containerInfo:t,pendingChildren:null,pingCache:null,earliestPendingTime:0,latestPendingTime:0,earliestSuspendedTime:0,latestSuspendedTime:0,latestPingedTime:0,didError:!1,pendingCommitExpirationTime:0,finishedWork:null,timeoutHandle:-1,context:null,pendingContext:null,hydrate:n,nextExpirationTimeToWorkOn:0,expirationTime:0,firstBatch:null,nextScheduledRoot:null},this._internalRoot=e.stateNode=t}function Hs(t){return!(!t||1!==t.nodeType&&9!==t.nodeType&&11!==t.nodeType&&(8!==t.nodeType||" react-mount-point-unstable "!==t.nodeValue))}function js(t,e,n,r,i){var o=n._reactRootContainer;if(o){if("function"==typeof i){var a=i;i=function(){var t=Ts(o._internalRoot);a.call(t)}}null!=t?o.legacy_renderSubtreeIntoContainer(t,e,i):o.render(e,i)}else{if(o=n._reactRootContainer=function(t,e){if(e||(e=!(!(e=t?9===t.nodeType?t.documentElement:t.firstChild:null)||1!==e.nodeType||!e.hasAttribute("data-reactroot"))),!e)for(var n;n=t.lastChild;)t.removeChild(n);return new Gs(t,!1,e)}(n,r),"function"==typeof i){var s=i;i=function(){var t=Ts(o._internalRoot);s.call(t)}}Ls(function(){null!=t?o.legacy_renderSubtreeIntoContainer(t,e,i):o.render(e,i)})}return Ts(o._internalRoot)}function Ys(t,e){var n=2<arguments.length&&void 0!==arguments[2]?arguments[2]:null;return Hs(e)||a("200"),function(t,e,n){var r=3<arguments.length&&void 0!==arguments[3]?arguments[3]:null;return{$$typeof:qt,key:null==r?null:""+r,children:t,containerInfo:e,implementation:n}}(t,e,null,n)}wt=function(t,e,n){switch(e){case"input":if(ve(t,n),e=n.name,"radio"===n.type&&null!=e){for(n=t;n.parentNode;)n=n.parentNode;for(n=n.querySelectorAll("input[name="+JSON.stringify(""+e)+'][type="radio"]'),e=0;e<n.length;e++){var r=n[e];if(r!==t&&r.form===t.form){var i=U(r);i||a("90"),Yt(r),ve(r,i)}}}break;case"textarea":Xn(t,n);break;case"select":null!=(e=n.value)&&qn(t,!!n.multiple,e,!1)}},Us.prototype.render=function(t){this._defer||a("250"),this._hasChildren=!0,this._children=t;var e=this._root._internalRoot,n=this._expirationTime,r=new Os;return Ns(t,e,null,n,r._onCommit),r},Us.prototype.then=function(t){if(this._didComplete)t();else{var e=this._callbacks;null===e&&(e=this._callbacks=[]),e.push(t)}},Us.prototype.commit=function(){var t=this._root._internalRoot,e=t.firstBatch;if(this._defer&&null!==e||a("251"),this._hasChildren){var n=this._expirationTime;if(e!==this){this._hasChildren&&(n=this._expirationTime=e._expirationTime,this.render(this._children));for(var r=null,i=e;i!==this;)r=i,i=i._next;null===r&&a("251"),r._next=i._next,this._next=e,t.firstBatch=this}this._defer=!1,_s(t,n),e=this._next,this._next=null,null!==(e=t.firstBatch=e)&&e._hasChildren&&e.render(e._children)}else this._next=null,this._defer=!1},Us.prototype._onComplete=function(){if(!this._didComplete){this._didComplete=!0;var t=this._callbacks;if(null!==t)for(var e=0;e<t.length;e++)(0,t[e])()}},Os.prototype.then=function(t){if(this._didCommit)t();else{var e=this._callbacks;null===e&&(e=this._callbacks=[]),e.push(t)}},Os.prototype._onCommit=function(){if(!this._didCommit){this._didCommit=!0;var t=this._callbacks;if(null!==t)for(var e=0;e<t.length;e++){var n=t[e];"function"!=typeof n&&a("191",n),n()}}},Gs.prototype.render=function(t,e){var n=this._internalRoot,r=new Os;return null!==(e=void 0===e?null:e)&&r.then(e),Fs(t,n,null,r._onCommit),r},Gs.prototype.unmount=function(t){var e=this._internalRoot,n=new Os;return null!==(t=void 0===t?null:t)&&n.then(t),Fs(null,e,null,n._onCommit),n},Gs.prototype.legacy_renderSubtreeIntoContainer=function(t,e,n){var r=this._internalRoot,i=new Os;return null!==(n=void 0===n?null:n)&&i.then(n),Fs(e,r,t,i._onCommit),i},Gs.prototype.createBatch=function(){var t=new Us(this),e=t._expirationTime,n=this._internalRoot,r=n.firstBatch;if(null===r)n.firstBatch=t,t._next=null;else{for(n=null;null!==r&&r._expirationTime>=e;)n=r,r=r._next;t._next=r,null!==n&&(n._next=t)}return t},St=Ss,Lt=Rs,Rt=function(){rs||0===as||(xs(as,!1),as=0)};var Ps,Ks,zs={createPortal:Ys,findDOMNode:function(t){if(null==t)return null;if(1===t.nodeType)return t;var e=t._reactInternalFiber;return void 0===e&&("function"==typeof t.render?a("188"):a("268",Object.keys(t))),t=null===(t=rn(e))?null:t.stateNode},hydrate:function(t,e,n){return Hs(e)||a("200"),js(null,t,e,!0,n)},render:function(t,e,n){return Hs(e)||a("200"),js(null,t,e,!1,n)},unstable_renderSubtreeIntoContainer:function(t,e,n,r){return Hs(n)||a("200"),(null==t||void 0===t._reactInternalFiber)&&a("38"),js(t,e,n,!1,r)},unmountComponentAtNode:function(t){return Hs(t)||a("40"),!!t._reactRootContainer&&(Ls(function(){js(null,null,t,!1,function(){t._reactRootContainer=null})}),!0)},unstable_createPortal:function(){return Ys.apply(void 0,arguments)},unstable_batchedUpdates:Ss,unstable_interactiveUpdates:Rs,flushSync:function(t,e){rs&&a("187");var n=ls;ls=!0;try{return Za(t,e)}finally{ls=n,xs(1073741823,!1)}},unstable_createRoot:function(t,e){return Hs(t)||a("299","unstable_createRoot"),new Gs(t,!0,null!=e&&!0===e.hydrate)},unstable_flushControlled:function(t){var e=ls;ls=!0;try{Za(t)}finally{(ls=e)||rs||xs(1073741823,!1)}},__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED:{Events:[F,T,U,D.injectEventPluginsByName,C,P,function(t){w(t,Y)},Mt,kt,_n,k]}};Ks=(Ps={findFiberByHostInstance:N,bundleType:0,version:"16.8.6",rendererPackageName:"react-dom"}).findFiberByHostInstance,function(t){if("undefined"==typeof __REACT_DEVTOOLS_GLOBAL_HOOK__)return!1;var e=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(e.isDisabled||!e.supportsFiber)return!0;try{var n=e.inject(t);Gr=jr(function(t){return e.onCommitFiberRoot(n,t)}),Hr=jr(function(t){return e.onCommitFiberUnmount(n,t)})}catch(t){}}(i({},Ps,{overrideProps:null,currentDispatcherRef:Pt.ReactCurrentDispatcher,findHostInstanceByFiber:function(t){return null===(t=rn(t))?null:t.stateNode},findFiberByHostInstance:function(t){return Ks?Ks(t):null}}));var Js={default:zs},qs=Js&&zs||Js;t.exports=qs.default||qs},function(t,e,n){"use strict";t.exports=n(1065)},function(t,e,n){"use strict";(function(t){
/** @license React v0.13.6
 * scheduler.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
Object.defineProperty(e,"__esModule",{value:!0});var n=null,r=!1,i=3,o=-1,a=-1,s=!1,A=!1;function l(){if(!s){var t=n.expirationTime;A?B():A=!0,v(g,t)}}function c(){var t=n,e=n.next;if(n===e)n=null;else{var r=n.previous;n=r.next=e,e.previous=r}t.next=t.previous=null,r=t.callback,e=t.expirationTime,t=t.priorityLevel;var o=i,s=a;i=t,a=e;try{var A=r()}finally{i=o,a=s}if("function"==typeof A)if(A={callback:A,priorityLevel:t,expirationTime:e,next:null,previous:null},null===n)n=A.next=A.previous=A;else{r=null,t=n;do{if(t.expirationTime>=e){r=t;break}t=t.next}while(t!==n);null===r?r=n:r===n&&(n=A,l()),(e=r.previous).next=r.previous=A,A.next=r,A.previous=e}}function u(){if(-1===o&&null!==n&&1===n.priorityLevel){s=!0;try{do{c()}while(null!==n&&1===n.priorityLevel)}finally{s=!1,null!==n?l():A=!1}}}function g(t){s=!0;var i=r;r=t;try{if(t)for(;null!==n;){var o=e.unstable_now();if(!(n.expirationTime<=o))break;do{c()}while(null!==n&&n.expirationTime<=o)}else if(null!==n)do{c()}while(null!==n&&!b())}finally{s=!1,r=i,null!==n?l():A=!1,u()}}var h,d,f=Date,p="function"==typeof setTimeout?setTimeout:void 0,I="function"==typeof clearTimeout?clearTimeout:void 0,C="function"==typeof requestAnimationFrame?requestAnimationFrame:void 0,m="function"==typeof cancelAnimationFrame?cancelAnimationFrame:void 0;function y(t){h=C(function(e){I(d),t(e)}),d=p(function(){m(h),t(e.unstable_now())},100)}if("object"==typeof performance&&"function"==typeof performance.now){var E=performance;e.unstable_now=function(){return E.now()}}else e.unstable_now=function(){return f.now()};var v,B,b,Q=null;if("undefined"!=typeof window?Q=window:void 0!==t&&(Q=t),Q&&Q._schedMock){var w=Q._schedMock;v=w[0],B=w[1],b=w[2],e.unstable_now=w[3]}else if("undefined"==typeof window||"function"!=typeof MessageChannel){var x=null,_=function(t){if(null!==x)try{x(t)}finally{x=null}};v=function(t){null!==x?setTimeout(v,0,t):(x=t,setTimeout(_,0,!1))},B=function(){x=null},b=function(){return!1}}else{"undefined"!=typeof console&&("function"!=typeof C&&console.error("This browser doesn't support requestAnimationFrame. Make sure that you load a polyfill in older browsers. https://fb.me/react-polyfills"),"function"!=typeof m&&console.error("This browser doesn't support cancelAnimationFrame. Make sure that you load a polyfill in older browsers. https://fb.me/react-polyfills"));var D=null,M=!1,k=-1,S=!1,L=!1,R=0,N=33,F=33;b=function(){return R<=e.unstable_now()};var T=new MessageChannel,U=T.port2;T.port1.onmessage=function(){M=!1;var t=D,n=k;D=null,k=-1;var r=e.unstable_now(),i=!1;if(0>=R-r){if(!(-1!==n&&n<=r))return S||(S=!0,y(O)),D=t,void(k=n);i=!0}if(null!==t){L=!0;try{t(i)}finally{L=!1}}};var O=function(t){if(null!==D){y(O);var e=t-R+F;e<F&&N<F?(8>e&&(e=8),F=e<N?N:e):N=e,R=t+F,M||(M=!0,U.postMessage(void 0))}else S=!1};v=function(t,e){D=t,k=e,L||0>e?U.postMessage(void 0):S||(S=!0,y(O))},B=function(){D=null,M=!1,k=-1}}e.unstable_ImmediatePriority=1,e.unstable_UserBlockingPriority=2,e.unstable_NormalPriority=3,e.unstable_IdlePriority=5,e.unstable_LowPriority=4,e.unstable_runWithPriority=function(t,n){switch(t){case 1:case 2:case 3:case 4:case 5:break;default:t=3}var r=i,a=o;i=t,o=e.unstable_now();try{return n()}finally{i=r,o=a,u()}},e.unstable_next=function(t){switch(i){case 1:case 2:case 3:var n=3;break;default:n=i}var r=i,a=o;i=n,o=e.unstable_now();try{return t()}finally{i=r,o=a,u()}},e.unstable_scheduleCallback=function(t,r){var a=-1!==o?o:e.unstable_now();if("object"==typeof r&&null!==r&&"number"==typeof r.timeout)r=a+r.timeout;else switch(i){case 1:r=a+-1;break;case 2:r=a+250;break;case 5:r=a+1073741823;break;case 4:r=a+1e4;break;default:r=a+5e3}if(t={callback:t,priorityLevel:i,expirationTime:r,next:null,previous:null},null===n)n=t.next=t.previous=t,l();else{a=null;var s=n;do{if(s.expirationTime>r){a=s;break}s=s.next}while(s!==n);null===a?a=n:a===n&&(n=t,l()),(r=a.previous).next=a.previous=t,t.next=a,t.previous=r}return t},e.unstable_cancelCallback=function(t){var e=t.next;if(null!==e){if(e===t)n=null;else{t===n&&(n=e);var r=t.previous;r.next=e,e.previous=r}t.next=t.previous=null}},e.unstable_wrapCallback=function(t){var n=i;return function(){var r=i,a=o;i=n,o=e.unstable_now();try{return t.apply(this,arguments)}finally{i=r,o=a,u()}}},e.unstable_getCurrentPriorityLevel=function(){return i},e.unstable_shouldYield=function(){return!r&&(null!==n&&n.expirationTime<a||b())},e.unstable_continueExecution=function(){null!==n&&l()},e.unstable_pauseExecution=function(){},e.unstable_getFirstCallbackNode=function(){return n}}).call(this,n(37))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(580),i=n(1067),o=n(581),a=function(){return r.create(void 0,!0)},s=function(){function t(t){var e=t.autoGenerateTag,n=this;this.cssRaw=function(t){t&&(n._raw+=t||"",n._pendingRawChange=!0,n._styleUpdated())},this.cssRule=function(t){for(var e=[],r=1;r<arguments.length;r++)e[r-1]=arguments[r];var a=i.ensureStringObj(o.extend.apply(void 0,e)).result;n._freeStyle.registerRule(t,a),n._styleUpdated()},this.forceRenderStyles=function(){var t=n._getTag();t&&(t.textContent=n.getStyles())},this.fontFace=function(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];for(var r=n._freeStyle,i=0,o=t;i<o.length;i++){var a=o[i];r.registerRule("@font-face",a)}n._styleUpdated()},this.getStyles=function(){return(n._raw||"")+n._freeStyle.getStyles()},this.keyframes=function(t){var e=i.explodeKeyframes(t),r=e.keyframes,o=e.$debugName,a=n._freeStyle.registerKeyframes(r,o);return n._styleUpdated(),a},this.reinit=function(){var t=a();n._freeStyle=t,n._lastFreeStyleChangeId=t.changeId,n._raw="",n._pendingRawChange=!1;var e=n._getTag();e&&(e.textContent="")},this.setStylesTarget=function(t){n._tag&&(n._tag.textContent=""),n._tag=t,n.forceRenderStyles()},this.stylesheet=function(t){for(var e={},r=0,i=Object.getOwnPropertyNames(t);r<i.length;r++){var o=i[r],a=t[o];a&&(a.$debugName=o,e[o]=n.style(a))}return e};var r=a();this._autoGenerateTag=e,this._freeStyle=r,this._lastFreeStyleChangeId=r.changeId,this._pending=0,this._pendingRawChange=!1,this._raw="",this._tag=void 0,this.style=this.style.bind(this)}return t.prototype._afterAllSync=function(t){var e=this;this._pending++;var n=this._pending;o.raf(function(){n===e._pending&&t()})},t.prototype._getTag=function(){if(this._tag)return this._tag;if(this._autoGenerateTag){var t="undefined"==typeof window?{textContent:""}:document.createElement("style");return"undefined"!=typeof document&&document.head.appendChild(t),this._tag=t,t}},t.prototype._styleUpdated=function(){var t=this,e=this._freeStyle.changeId,n=this._lastFreeStyleChangeId;(this._pendingRawChange||e!==n)&&(this._lastFreeStyleChangeId=e,this._pendingRawChange=!1,this._afterAllSync(function(){return t.forceRenderStyles()}))},t.prototype.style=function(){var t=this._freeStyle,e=i.ensureStringObj(o.extend.apply(void 0,arguments)),n=e.result,r=e.debugName,a=r?t.registerStyle(n,r):t.registerStyle(n);return this._styleUpdated(),a},t}();e.TypeStyle=s},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(580);e.ensureStringObj=function t(e){var n={},i="";for(var o in e){var a=e[o];if("$unique"===o)n[r.IS_UNIQUE]=a;else if("$nest"===o){var s=a;for(var A in s){var l=s[A];n[A]=t(l).result}}else"$debugName"===o?i=a:n[o]=a}return{result:n,debugName:i}},e.explodeKeyframes=function(t){var e={$debugName:void 0,keyframes:{}};for(var n in t){var r=t[n];"$debugName"===n?e.$debugName=r:e.keyframes[n]=r}return e}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0})},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(17),i=n(60);let o="api/config";!function(t){t.create=function(t){let e=new a(t);return e.load().then(()=>e)}}(e.ConfigSection||(e.ConfigSection={}));class a{constructor(t){this._url="unknown";let e=this.serverSettings=t.serverSettings||i.ServerConnection.makeSettings();this._url=r.URLExt.join(e.baseUrl,o,encodeURIComponent(t.name))}get data(){return this._data}load(){return i.ServerConnection.makeRequest(this._url,{},this.serverSettings).then(t=>{if(200!==t.status)throw new i.ServerConnection.ResponseError(t);return t.json()}).then(t=>{this._data=t})}update(t){this._data=Object.assign({},this._data,t);let e={method:"PATCH",body:JSON.stringify(t)};return i.ServerConnection.makeRequest(this._url,e,this.serverSettings).then(t=>{if(200!==t.status)throw new i.ServerConnection.ResponseError(t);return t.json()}).then(t=>(this._data=t,this._data))}}e.ConfigWithDefaults=class{constructor(t){this._className="",this._section=t.section,this._defaults=t.defaults||{},this._className=t.className||""}get(t){let e=this._classData();return t in e?e[t]:this._defaults[t]}set(t,e){let n={};if(n[t]=e,this._className){let t={};return t[this._className]=n,this._section.update(t)}return this._section.update(n)}_classData(){let t=this._section.data;return this._className&&this._className in t?t[this._className]:t}}},function(t,e,n){"use strict";function r(t,e,n){if(!t.hasOwnProperty(e))throw Error(`Missing property '${e}'`);if(void 0!==n){let r=!0,i=t[e];switch(n){case"array":r=Array.isArray(i);break;case"object":r=void 0!==i;break;default:r=typeof i===n}if(!r)throw new Error(`Property '${e}' is not of type '${n}`)}}Object.defineProperty(e,"__esModule",{value:!0}),e.validateContentsModel=function(t){r(t,"name","string"),r(t,"path","string"),r(t,"type","string"),r(t,"created","string"),r(t,"last_modified","string"),r(t,"mimetype","object"),r(t,"content","object"),r(t,"format","object")},e.validateCheckpointModel=function(t){r(t,"id","string"),r(t,"last_modified","string")}},function(t,e,n){"use strict";var r=this&&this.__importStar||function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)Object.hasOwnProperty.call(t,n)&&(e[n]=t[n]);return e.default=t,e};Object.defineProperty(e,"__esModule",{value:!0});const i=n(17),o=n(9),a=n(5),s=n(9),A=n(10),l=n(60),c=n(1072),u=n(216),g=n(1073),h=r(n(1074)),d=r(n(585)),f="api/kernels",p="api/kernelspecs";class I{constructor(t,e){this._createSocket=()=>{if(this.isDisposed)return;let t=this.serverSettings,e=i.URLExt.join(t.wsUrl,f,encodeURIComponent(this._id)),n=e.replace(/^((?:\w+:)?\/\/)(?:[^@\/]+@)/,"$1");console.log("Starting WebSocket:",n);let r=i.URLExt.join(e,"channels?session_id="+encodeURIComponent(this._clientId)),o=t.token;""!==o&&(r+=`&token=${encodeURIComponent(o)}`),this._wsStopped=!1,this._ws=new t.WebSocket(r),this._ws.binaryType="arraybuffer",this._ws.onmessage=this._onWSMessage,this._ws.onopen=this._onWSOpen,this._ws.onclose=this._onWSClose,this._ws.onerror=this._onWSClose},this._onWSOpen=t=>{this._reconnectAttempt=0,this._updateStatus("connected"),this._isReady=!0,this.requestKernelInfo().then(()=>{this._initialized=!0,this._isReady=!0,this._readyPromise.resolve()}).catch(t=>{this._initialized=!0,this._readyPromise.reject(t)}),this._isReady=!1},this._onWSMessage=t=>{if(this._wsStopped)return;let e;try{e=h.deserialize(t.data),d.validateMessage(e)}catch(t){throw t.message=`Kernel message validation error: ${t.message}`,t}this._kernelSession=e.header.session,this._msgChain=this._msgChain.then(()=>this._handleMessage(e)).catch(t=>{console.error(t)}),this._anyMessage.emit({msg:e,direction:"recv"})},this._onWSClose=t=>{if(!this._wsStopped&&this._ws)if(this._clearSocket(),this._reconnectAttempt<this._reconnectLimit){this._updateStatus("reconnecting");let t=Math.pow(2,this._reconnectAttempt);console.error("Connection lost, reconnecting in "+t+" seconds."),setTimeout(this._createSocket,1e3*t),this._reconnectAttempt+=1}else this._updateStatus("dead")},this._id="",this._name="",this._status="unknown",this._kernelSession="",this._clientId="",this._isDisposed=!1,this._wsStopped=!1,this._ws=null,this._username="",this._reconnectLimit=7,this._reconnectAttempt=0,this._isReady=!1,this._readyPromise=new s.PromiseDelegate,this._initialized=!1,this._futures=new Map,this._comms=new Map,this._targetRegistry=Object.create(null),this._info=null,this._pendingMessages=[],this._statusChanged=new A.Signal(this),this._iopubMessage=new A.Signal(this),this._anyMessage=new A.Signal(this),this._unhandledMessage=new A.Signal(this),this._displayIdToParentIds=new Map,this._msgIdToDisplayIds=new Map,this._terminated=new A.Signal(this),this._msgChain=Promise.resolve(),this._noOp=()=>{},this._name=t.name,this._id=e,this.serverSettings=t.serverSettings||l.ServerConnection.makeSettings(),this._clientId=t.clientId||o.UUID.uuid4(),this._username=t.username||"",this.handleComms=void 0===t.handleComms||t.handleComms,this._readyPromise.promise.then(()=>{this._sendPending()}),this._createSocket(),C.runningKernels.push(this)}get terminated(){return this._terminated}get statusChanged(){return this._statusChanged}get iopubMessage(){return this._iopubMessage}get unhandledMessage(){return this._unhandledMessage}get anyMessage(){return this._anyMessage}get id(){return this._id}get name(){return this._name}get model(){return{name:this.name,id:this.id}}get username(){return this._username}get clientId(){return this._clientId}get status(){return this._status}get isDisposed(){return this._isDisposed}get info(){return this._info}get isReady(){return this._isReady}get ready(){return this._readyPromise.promise}getSpec(){return this._specPromise?this._specPromise:(this._specPromise=C.findSpecs(this.serverSettings).then(t=>t.kernelspecs[this._name]),this._specPromise)}clone(t={}){return new I(Object.assign({name:this._name,username:this._username,serverSettings:this.serverSettings,handleComms:this.handleComms},t),this._id)}dispose(){this.isDisposed||(this._isDisposed=!0,this._terminated.emit(),this._status="dead",this._clearState(),this._clearSocket(),this._kernelSession="",this._msgChain=null,a.ArrayExt.removeFirstOf(C.runningKernels,this),A.Signal.clearData(this))}sendShellMessage(t,e=!1,n=!0){return this._sendKernelShellControl(g.KernelShellFutureHandler,t,e,n)}sendControlMessage(t,e=!1,n=!0){return this._sendKernelShellControl(g.KernelControlFutureHandler,t,e,n)}_sendKernelShellControl(t,e,n=!1,r=!0){if("dead"===this.status)throw new Error("Kernel is dead");this._isReady&&this._ws?this._ws.send(h.serialize(e)):this._pendingMessages.push(e),this._anyMessage.emit({msg:e,direction:"send"});let i=new t(()=>{let t=e.header.msg_id;this._futures.delete(t);let n=this._msgIdToDisplayIds.get(t);n&&(n.forEach(e=>{let n=this._displayIdToParentIds.get(e);if(n){let r=n.indexOf(t);if(-1===r)return;1===n.length?this._displayIdToParentIds.delete(e):(n.splice(r,1),this._displayIdToParentIds.set(e,n))}}),this._msgIdToDisplayIds.delete(t))},e,n,r,this);return this._futures.set(e.header.msg_id,i),i}interrupt(){return C.interruptKernel(this,this.serverSettings)}restart(){return C.restartKernel(this,this.serverSettings)}async handleRestart(){await this._clearState(),this._updateStatus("restarting")}reconnect(){return this._clearSocket(),this._updateStatus("reconnecting"),this._createSocket(),this._readyPromise.promise}async shutdown(){if("dead"===this.status)return this._clearSocket(),void await this._clearState();await C.shutdownKernel(this.id,this.serverSettings),await this._clearState(),this._clearSocket()}async requestKernelInfo(){let t=u.KernelMessage.createMessage({msgType:"kernel_info_request",channel:"shell",username:this._username,session:this._clientId,content:{}}),e=await C.handleShellMessage(this,t);if(this.isDisposed)throw new Error("Disposed kernel");if(e.content.status&&"ok"!==e.content.status)throw new Error("Kernel info reply errored");return this._info=e.content,e}requestComplete(t){let e=u.KernelMessage.createMessage({msgType:"complete_request",channel:"shell",username:this._username,session:this._clientId,content:t});return C.handleShellMessage(this,e)}requestInspect(t){let e=u.KernelMessage.createMessage({msgType:"inspect_request",channel:"shell",username:this._username,session:this._clientId,content:t});return C.handleShellMessage(this,e)}requestHistory(t){let e=u.KernelMessage.createMessage({msgType:"history_request",channel:"shell",username:this._username,session:this._clientId,content:t});return C.handleShellMessage(this,e)}requestExecute(t,e=!0,n){let r=u.KernelMessage.createMessage({msgType:"execute_request",channel:"shell",username:this._username,session:this._clientId,content:Object.assign({},{silent:!1,store_history:!0,user_expressions:{},allow_stdin:!0,stop_on_error:!1},t),metadata:n});return this.sendShellMessage(r,!0,e)}requestDebug(t,e=!0){let n=u.KernelMessage.createMessage({msgType:"debug_request",channel:"control",username:this._username,session:this._clientId,content:t});return this.sendControlMessage(n,!0,e)}requestIsComplete(t){let e=u.KernelMessage.createMessage({msgType:"is_complete_request",channel:"shell",username:this._username,session:this._clientId,content:t});return C.handleShellMessage(this,e)}requestCommInfo(t){let e=u.KernelMessage.createMessage({msgType:"comm_info_request",channel:"shell",username:this._username,session:this._clientId,content:t});return C.handleShellMessage(this,e)}sendInputReply(t){if("dead"===this.status)throw new Error("Kernel is dead");let e=u.KernelMessage.createMessage({msgType:"input_reply",channel:"stdin",username:this._username,session:this._clientId,content:t});this._isReady&&this._ws?this._ws.send(h.serialize(e)):this._pendingMessages.push(e),this._anyMessage.emit({msg:e,direction:"send"})}connectToComm(t,e=o.UUID.uuid4()){if(!this.handleComms)throw new Error("Comms are disabled on this kernel connection");if(this._comms.has(e))return this._comms.get(e);let n=new c.CommHandler(t,e,this,()=>{this._unregisterComm(e)});return this._comms.set(e,n),n}registerCommTarget(t,e){this.handleComms&&(this._targetRegistry[t]=e)}removeCommTarget(t,e){this.handleComms&&(this.isDisposed||this._targetRegistry[t]!==e||delete this._targetRegistry[t])}registerMessageHook(t,e){let n=this._futures&&this._futures.get(t);n&&n.registerMessageHook(e)}removeMessageHook(t,e){let n=this._futures&&this._futures.get(t);n&&n.removeMessageHook(e)}async _handleDisplayId(t,e){let n=e.parent_header.msg_id,r=this._displayIdToParentIds.get(t);if(r){let t={header:s.JSONExt.deepCopy(e.header),parent_header:s.JSONExt.deepCopy(e.parent_header),metadata:s.JSONExt.deepCopy(e.metadata),content:s.JSONExt.deepCopy(e.content),channel:e.channel,buffers:e.buffers?e.buffers.slice():[]};t.header.msg_type="update_display_data",await Promise.all(r.map(async e=>{let n=this._futures&&this._futures.get(e);n&&await n.handleMsg(t)}))}if("update_display_data"===e.header.msg_type)return!0;-1===(r=this._displayIdToParentIds.get(t)||[]).indexOf(n)&&r.push(n),this._displayIdToParentIds.set(t,r);let i=this._msgIdToDisplayIds.get(n)||[];return-1===i.indexOf(n)&&i.push(n),this._msgIdToDisplayIds.set(n,i),!1}_clearSocket(){this._wsStopped=!0,null!==this._ws&&(this._ws.onopen=this._noOp,this._ws.onclose=this._noOp,this._ws.onerror=this._noOp,this._ws.onmessage=this._noOp,this._ws.close(),this._ws=null)}_updateStatus(t){switch(t){case"idle":case"busy":!this._isReady&&this._initialized&&(this._isReady=!0,this._readyPromise.resolve());break;case"restarting":this.requestKernelInfo().catch(this._noOp);break;case"starting":case"autorestarting":case"connected":break;case"reconnecting":this._isReady&&(this._isReady=!1,this._readyPromise=new s.PromiseDelegate,this._readyPromise.promise.then(()=>{this._sendPending()}));break;case"dead":this._isReady&&(this._isReady=!1,this._readyPromise=new s.PromiseDelegate),this._readyPromise.promise.catch(this._noOp),this._readyPromise.reject("Kernel is dead");break;default:return void console.error("invalid kernel status:",t)}t!==this._status&&(this._status=t,C.logKernelStatus(this),this._statusChanged.emit(t),"dead"===t&&this.dispose())}_sendPending(){for(;this._ws&&this._pendingMessages.length>0;){let t=h.serialize(this._pendingMessages[0]);this._ws.send(t),this._pendingMessages.shift()}}async _clearState(){this._pendingMessages=[];const t=[];this._futures.forEach(e=>{t.push(e.done.then(this._noOp,this._noOp)),e.dispose()}),this._comms.forEach(t=>{t.dispose()}),this._msgChain=Promise.resolve(),this._kernelSession="",this._futures=new Map,this._comms=new Map,this._displayIdToParentIds.clear(),this._msgIdToDisplayIds.clear(),await Promise.all(t)}_assertCurrentMessage(t){if(this.isDisposed)throw new Error("Kernel object is disposed");if(t.header.session!==this._kernelSession)throw new Error(`Canceling handling of old message: ${t.header.msg_type}`)}async _handleCommOpen(t){this._assertCurrentMessage(t);let e=t.content,n=new c.CommHandler(e.target_name,e.comm_id,this,()=>{this._unregisterComm(e.comm_id)});this._comms.set(e.comm_id,n);try{let r=await C.loadObject(e.target_name,e.target_module,this._targetRegistry);await r(n,t)}catch(t){throw n.close(),console.error("Exception opening new comm"),t}}async _handleCommClose(t){this._assertCurrentMessage(t);let e=t.content,n=this._comms.get(e.comm_id);if(!n)return void console.error("Comm not found for comm id "+e.comm_id);this._unregisterComm(n.commId);let r=n.onClose;r&&await r(t),n.dispose()}async _handleCommMsg(t){this._assertCurrentMessage(t);let e=t.content,n=this._comms.get(e.comm_id);if(!n)return;let r=n.onMsg;r&&await r(t)}_unregisterComm(t){this._comms.delete(t)}async _handleMessage(t){let e=!1;if(t.parent_header&&"iopub"===t.channel&&(u.KernelMessage.isDisplayDataMsg(t)||u.KernelMessage.isUpdateDisplayDataMsg(t)||u.KernelMessage.isExecuteResultMsg(t))){let n=(t.content.transient||{}).display_id;n&&(e=await this._handleDisplayId(n,t),this._assertCurrentMessage(t))}if(!e&&t.parent_header){let e=t.parent_header,n=this._futures&&this._futures.get(e.msg_id);if(n)await n.handleMsg(t),this._assertCurrentMessage(t);else{let n=e.session===this.clientId;"iopub"!==t.channel&&n&&this._unhandledMessage.emit(t)}}if("iopub"===t.channel){switch(t.header.msg_type){case"status":let e=t.content.execution_state;this._updateStatus(e),"restarting"===e&&Promise.resolve().then(async()=>{await this.handleRestart(),this._updateStatus("autorestarting")});break;case"comm_open":this.handleComms&&await this._handleCommOpen(t);break;case"comm_msg":this.handleComms&&await this._handleCommMsg(t);break;case"comm_close":this.handleComms&&await this._handleCommClose(t)}this.isDisposed||(this._assertCurrentMessage(t),this._iopubMessage.emit(t))}}}var C;e.DefaultKernel=I,function(t){t.findById=function(t,e){return C.findById(t,e)},t.getSpecs=function(t){return C.getSpecs(t)},t.listRunning=function(t){return C.listRunning(t)},t.startNew=function(t){return C.startNew(t)},t.connectTo=function(t,e){return C.connectTo(t,e)},t.shutdown=function(t,e){return C.shutdownKernel(t,e)},t.shutdownAll=function(t){return C.shutdownAll(t)}}(I=e.DefaultKernel||(e.DefaultKernel={})),function(t){function e(e){e=e||l.ServerConnection.makeSettings();let n=i.URLExt.join(e.baseUrl,p),r=l.ServerConnection.makeRequest(n,{},e).then(t=>{if(200!==t.status)throw new l.ServerConnection.ResponseError(t);return t.json()}).then(t=>d.validateSpecModels(t));return t.specs[e.baseUrl]=r,r}function n(t){t=t||l.ServerConnection.makeSettings();let e=i.URLExt.join(t.baseUrl,f);return l.ServerConnection.makeRequest(e,{},t).then(t=>{if(200!==t.status)throw new l.ServerConnection.ResponseError(t);return t.json()}).then(t=>{if(!Array.isArray(t))throw new Error("Invalid kernel list");for(let e=0;e<t.length;e++)d.validateModel(t[e]);return r(t)})}function r(e){return a.each(t.runningKernels.slice(),t=>{a.find(e,e=>t.id===e.id)||"dead"===t.status||t.dispose()}),e}async function o(e,n){n=n||l.ServerConnection.makeSettings();let r=i.URLExt.join(n.baseUrl,f,encodeURIComponent(e)),o=await l.ServerConnection.makeRequest(r,{method:"DELETE"},n);if(404===o.status){let t=`The kernel "${e}" does not exist on the server`;console.warn(t)}else if(204!==o.status)throw new l.ServerConnection.ResponseError(o);!function(e){t.runningKernels.slice().forEach(t=>{t.id===e&&t.dispose()})}(e)}async function s(t,e){e=e||l.ServerConnection.makeSettings();let n=i.URLExt.join(e.baseUrl,f,encodeURIComponent(t)),r=await l.ServerConnection.makeRequest(n,{},e);if(200!==r.status)throw new l.ServerConnection.ResponseError(r);let o=await r.json();return d.validateModel(o),o}t.runningKernels=[],t.specs=Object.create(null),t.findById=function(e,n){let r=a.find(t.runningKernels,t=>t.id===e);return r?Promise.resolve(r.model):s(e,n).catch(()=>{throw new Error(`No running kernel with id: ${e}`)})},t.findSpecs=function(n){n=n||l.ServerConnection.makeSettings();let r=t.specs[n.baseUrl];return r||e(n)},t.getSpecs=e,t.listRunning=n,t.updateRunningKernels=r,t.startNew=async function(t){let e=t.serverSettings||l.ServerConnection.makeSettings(),n=i.URLExt.join(e.baseUrl,f),r={method:"POST",body:JSON.stringify({name:t.name})},o=await l.ServerConnection.makeRequest(n,r,e);if(201!==o.status)throw new l.ServerConnection.ResponseError(o);let a=await o.json();return d.validateModel(a),new I(Object.assign({},t,{name:a.name,serverSettings:e}),a.id)},t.connectTo=function(e,n){let r=n||l.ServerConnection.makeSettings(),i=a.find(t.runningKernels,t=>t.id===e.id);if(i){const n=!a.some(t.runningKernels,t=>t.id===e.id&&t.handleComms);return i.clone({handleComms:n})}return new I({name:e.name,serverSettings:r},e.id)},t.restartKernel=async function(e,n){if("dead"===e.status)throw new Error("Kernel is dead");n=n||l.ServerConnection.makeSettings();let r=i.URLExt.join(n.baseUrl,f,encodeURIComponent(e.id),"restart");await Promise.all(t.runningKernels.filter(t=>t.id===e.id).map(t=>t.handleRestart()));let o=await l.ServerConnection.makeRequest(r,{method:"POST"},n);if(200!==o.status)throw new l.ServerConnection.ResponseError(o);let a=await o.json();d.validateModel(a)},t.interruptKernel=async function(t,e){if("dead"===t.status)throw new Error("Kernel is dead");e=e||l.ServerConnection.makeSettings();let n=i.URLExt.join(e.baseUrl,f,encodeURIComponent(t.id),"interrupt"),r=await l.ServerConnection.makeRequest(n,{method:"POST"},e);if(204!==r.status)throw new l.ServerConnection.ResponseError(r)},t.shutdownKernel=o,t.shutdownAll=async function(t){t=t||l.ServerConnection.makeSettings();let e=await n(t);await Promise.all(e.map(e=>o(e.id,t)))},t.getKernelModel=s,t.logKernelStatus=function(t){switch(t.status){case"idle":case"busy":case"unknown":return;default:console.log(`Kernel: ${t.status} (${t.id})`)}},t.handleShellMessage=async function(t,e){return t.sendShellMessage(e,!0).done},t.loadObject=function(t,e,n){return new Promise((r,i)=>{if(e){if("undefined"==typeof requirejs)throw new Error("requirejs not found");requirejs([e],n=>{void 0===n[t]?i(new Error(`Object '${t}' not found in module '${e}'`)):r(n[t])},i)}else n&&n[t]?r(n[t]):i(new Error(`Object '${t}' not found in registry`))})}}(C||(C={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(50),i=n(216);e.CommHandler=class extends r.DisposableDelegate{constructor(t,e,n,r){super(r),this._target="",this._id="",this._id=e,this._target=t,this._kernel=n}get commId(){return this._id}get targetName(){return this._target}get onClose(){return this._onClose}set onClose(t){this._onClose=t}get onMsg(){return this._onMsg}set onMsg(t){this._onMsg=t}open(t,e,n=[]){if(this.isDisposed||this._kernel.isDisposed)throw new Error("Cannot open");let r=i.KernelMessage.createMessage({msgType:"comm_open",channel:"shell",username:this._kernel.username,session:this._kernel.clientId,content:{comm_id:this._id,target_name:this._target,data:t||{}},metadata:e,buffers:n});return this._kernel.sendShellMessage(r,!1,!0)}send(t,e,n=[],r=!0){if(this.isDisposed||this._kernel.isDisposed)throw new Error("Cannot send");let o=i.KernelMessage.createMessage({msgType:"comm_msg",channel:"shell",username:this._kernel.username,session:this._kernel.clientId,content:{comm_id:this._id,data:t},metadata:e,buffers:n});return this._kernel.sendShellMessage(o,!1,!0)}close(t,e,n=[]){if(this.isDisposed||this._kernel.isDisposed)throw new Error("Cannot close");let r=i.KernelMessage.createMessage({msgType:"comm_close",channel:"shell",username:this._kernel.username,session:this._kernel.clientId,content:{comm_id:this._id,data:t||{}},metadata:e,buffers:n}),o=this._kernel.sendShellMessage(r,!1,!0),a=this._onClose;return a&&a(i.KernelMessage.createMessage({msgType:"comm_close",channel:"iopub",username:this._kernel.username,session:this._kernel.clientId,content:{comm_id:this._id,data:t||{}},metadata:e,buffers:n})),this.dispose(),o}}},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0});const r=n(9),i=n(50),o=n(216);class a extends i.DisposableDelegate{constructor(t,e,n,i,o){super(t),this._status=0,this._stdin=s.noOp,this._iopub=s.noOp,this._reply=s.noOp,this._done=new r.PromiseDelegate,this._hooks=new s.HookList,this._disposeOnDone=!0,this._msg=e,n||this._setFlag(s.KernelFutureFlag.GotReply),this._disposeOnDone=i,this._kernel=o}get msg(){return this._msg}get done(){return this._done.promise}get onReply(){return this._reply}set onReply(t){this._reply=t}get onIOPub(){return this._iopub}set onIOPub(t){this._iopub=t}get onStdin(){return this._stdin}set onStdin(t){this._stdin=t}registerMessageHook(t){if(this.isDisposed)throw new Error("Kernel future is disposed");this._hooks.add(t)}removeMessageHook(t){this.isDisposed||this._hooks.remove(t)}sendInputReply(t){this._kernel.sendInputReply(t)}dispose(){this._stdin=s.noOp,this._iopub=s.noOp,this._reply=s.noOp,this._hooks=null,this._testFlag(s.KernelFutureFlag.IsDone)||(this._done.reject(new Error("Canceled")),this._done.promise.catch(()=>{})),super.dispose()}async handleMsg(t){switch(t.channel){case"control":case"shell":t.channel===this.msg.channel&&t.parent_header.msg_id===this.msg.header.msg_id&&await this._handleReply(t);break;case"stdin":await this._handleStdin(t);break;case"iopub":await this._handleIOPub(t)}}async _handleReply(t){let e=this._reply;e&&await e(t),this._replyMsg=t,this._setFlag(s.KernelFutureFlag.GotReply),this._testFlag(s.KernelFutureFlag.GotIdle)&&this._handleDone()}async _handleStdin(t){let e=this._stdin;e&&await e(t)}async _handleIOPub(t){let e=await this._hooks.process(t),n=this._iopub;e&&n&&await n(t),o.KernelMessage.isStatusMsg(t)&&"idle"===t.content.execution_state&&(this._setFlag(s.KernelFutureFlag.GotIdle),this._testFlag(s.KernelFutureFlag.GotReply)&&this._handleDone())}_handleDone(){this._testFlag(s.KernelFutureFlag.IsDone)||(this._setFlag(s.KernelFutureFlag.IsDone),this._done.resolve(this._replyMsg),this._disposeOnDone&&this.dispose())}_testFlag(t){return 0!=(this._status&t)}_setFlag(t){this._status|=t}}e.KernelFutureHandler=a;e.KernelControlFutureHandler=class extends a{};var s;e.KernelShellFutureHandler=class extends a{},function(e){e.noOp=()=>{};const n="function"==typeof requestAnimationFrame?requestAnimationFrame:t;let i;e.HookList=class{constructor(){this._hooks=[]}add(t){this.remove(t),this._hooks.push(t)}remove(t){let e=this._hooks.indexOf(t);e>=0&&(this._hooks[e]=null,this._scheduleCompact())}async process(t){await this._processing;let e,n=new r.PromiseDelegate;this._processing=n.promise;for(let r=this._hooks.length-1;r>=0;r--){let i=this._hooks[r];if(null!==i){try{e=await i(t)}catch(t){e=!0,console.error(t)}if(!1===e)return n.resolve(void 0),!1}}return n.resolve(void 0),!0}_scheduleCompact(){this._compactScheduled||(this._compactScheduled=!0,n(()=>{this._processing=this._processing.then(()=>{this._compactScheduled=!1,this._compact()})}))}_compact(){let t=0;for(let e=0,n=this._hooks.length;e<n;e++){let n=this._hooks[e];null===this._hooks[e]?t++:this._hooks[e-t]=n}this._hooks.length-=t}},function(t){t[t.GotReply=1]="GotReply",t[t.GotIdle=2]="GotIdle",t[t.IsDone=4]="IsDone",t[t.DisposeOnDone=8]="DisposeOnDone"}(i=e.KernelFutureFlag||(e.KernelFutureFlag={}))}(s||(s={}))}).call(this,n(118).setImmediate)},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.deserialize=function(t){let e;return e="string"==typeof t?JSON.parse(t):function(t){let e=new DataView(t),n=e.getUint32(0),r=[];if(n<2)throw new Error("Invalid incoming Kernel Message");for(let t=1;t<=n;t++)r.push(e.getUint32(4*t));let i=new Uint8Array(t.slice(r[0],r[1])),o=JSON.parse(new TextDecoder("utf8").decode(i));o.buffers=[];for(let e=1;e<n;e++){let n=r[e],i=r[e+1]||t.byteLength;o.buffers.push(new DataView(t.slice(n,i)))}return o}(t)},e.serialize=function(t){let e;return e=t.buffers&&t.buffers.length?function(t){let e=[],n=[],r=new TextEncoder,i=[];void 0!==t.buffers&&(i=t.buffers,delete t.buffers);let o=r.encode(JSON.stringify(t));n.push(o.buffer);for(let t=0;t<i.length;t++){let e=i[t];n.push(e instanceof ArrayBuffer?e:e.buffer)}let a=n.length;e.push(4*(a+1));for(let t=0;t+1<n.length;t++)e.push(e[e.length-1]+n[t].byteLength);let s=new Uint8Array(e[e.length-1]+n[n.length-1].byteLength),A=new DataView(s.buffer);A.setUint32(0,a);for(let t=0;t<e.length;t++)A.setUint32(4*(t+1),e[t]);for(let t=0;t<n.length;t++)s.set(new Uint8Array(n[t]),e[t]);return s.buffer}(t):JSON.stringify(t)}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(17),i=n(5),o=n(9),a=n(10),s=n(60),A=n(584);e.KernelManager=class{constructor(t={}){this._isDisposed=!1,this._isReady=!1,this._kernels=new Set,this._models=[],this._runningChanged=new a.Signal(this),this._specs=null,this._specsChanged=new a.Signal(this),this._connectionFailure=new a.Signal(this),this.serverSettings=t.serverSettings||s.ServerConnection.makeSettings(),this._ready=Promise.all([this.requestRunning(),this.requestSpecs()]).then(t=>void 0).catch(t=>void 0).then(()=>{this.isDisposed||(this._isReady=!0)}),this._pollModels=new r.Poll({auto:!1,factory:()=>this.requestRunning(),frequency:{interval:1e4,backoff:!0,max:3e5},name:"@jupyterlab/services:KernelManager#models",standby:t.standby||"when-hidden"}),this._pollSpecs=new r.Poll({auto:!1,factory:()=>this.requestSpecs(),frequency:{interval:61e3,backoff:!0,max:3e5},name:"@jupyterlab/services:KernelManager#specs",standby:t.standby||"when-hidden"}),this.ready.then(()=>{this._pollModels.start(),this._pollSpecs.start()})}get isDisposed(){return this._isDisposed}get isReady(){return this._isReady}get ready(){return this._ready}get runningChanged(){return this._runningChanged}get specs(){return this._specs}get specsChanged(){return this._specsChanged}get connectionFailure(){return this._connectionFailure}connectTo(t){let e=A.Kernel.connectTo(t,this.serverSettings);return this._onStarted(e),e}dispose(){this._isDisposed||(this._isDisposed=!0,this._models.length=0,this._pollModels.dispose(),this._pollSpecs.dispose(),a.Signal.clearData(this))}findById(t){return A.Kernel.findById(t,this.serverSettings)}async refreshRunning(){await this._pollModels.refresh(),await this._pollModels.tick}async refreshSpecs(){await this._pollSpecs.refresh(),await this._pollSpecs.tick}running(){return i.iter(this._models)}async shutdown(t){const e=this._models,n=this._kernels,r=i.ArrayExt.findFirstIndex(e,e=>e.id===t);-1!==r&&(e.splice(r,1),this._runningChanged.emit(e.slice()),n.forEach(e=>{e.id===t&&(n.delete(e),e.dispose())}),await A.Kernel.shutdown(t,this.serverSettings))}async shutdownAll(){try{await this.requestRunning(),await Promise.all(this._models.map(({id:t})=>A.Kernel.shutdown(t,this.serverSettings)))}finally{this._kernels.forEach(t=>{t.dispose()}),this._kernels.clear(),this._models.length&&(this._models.length=0,this._runningChanged.emit([]))}}async startNew(t={}){const e=Object.assign({},t,{serverSettings:this.serverSettings}),n=await A.Kernel.startNew(e);return this._onStarted(n),n}async requestRunning(){const t=await A.Kernel.listRunning(this.serverSettings).catch(t=>{if(t instanceof s.ServerConnection.NetworkError||t.response&&503===t.response.status)return this._connectionFailure.emit(t),[];throw t});if(!this._isDisposed&&!o.JSONExt.deepEqual(t,this._models)){const e=t.map(({id:t})=>t),n=this._kernels;n.forEach(t=>{-1===e.indexOf(t.id)&&(t.dispose(),n.delete(t))}),this._models=t.slice(),this._runningChanged.emit(t)}}async requestSpecs(){const t=await A.Kernel.getSpecs(this.serverSettings);this._isDisposed||o.JSONExt.deepEqual(t,this._specs)||(this._specs=t,this._specsChanged.emit(t))}_onStarted(t){let e=t.id;this._kernels.add(t),-1===i.ArrayExt.findFirstIndex(this._models,t=>t.id===e)&&(this._models.push(t.model),this._runningChanged.emit(this._models.slice())),t.terminated.connect(()=>{this._onTerminated(e)})}_onTerminated(t){let e=i.ArrayExt.findFirstIndex(this._models,e=>e.id===t);-1!==e&&(this._models.splice(e,1),this._runningChanged.emit(this._models.slice()))}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(10),i=n(1077),o=n(586),a=n(583),s=n(587),A=n(589),l=n(590),c=n(120),u=n(591);e.ServiceManager=class{constructor(t={}){this._isDisposed=!1,this._specsChanged=new r.Signal(this),this._connectionFailure=new r.Signal(this),this._isReady=!1;const e=t.defaultDrive,n=t.serverSettings||c.ServerConnection.makeSettings(),g={defaultDrive:e,serverSettings:n,standby:t.standby||"when-hidden"};this.serverSettings=n,this.contents=new a.ContentsManager(g),this.sessions=new s.SessionManager(g),this.settings=new A.SettingManager(g),this.terminals=new l.TerminalManager(g),this.builder=new i.BuildManager(g),this.workspaces=new u.WorkspaceManager(g),this.nbconvert=new o.NbConvertManager(g),this.sessions.specsChanged.connect((t,e)=>{this._specsChanged.emit(e)}),this.sessions.connectionFailure.connect(this._onConnectionFailure,this),this.terminals.connectionFailure.connect(this._onConnectionFailure,this),this._readyPromise=this.sessions.ready.then(()=>{if(this.terminals.isAvailable())return this.terminals.ready}),this._readyPromise.then(()=>{this._isReady=!0})}get specsChanged(){return this._specsChanged}get connectionFailure(){return this._connectionFailure}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,r.Signal.clearData(this),this.contents.dispose(),this.sessions.dispose(),this.terminals.dispose())}get specs(){return this.sessions.specs}get isReady(){return this._isReady}get ready(){return this._readyPromise}_onConnectionFailure(t,e){this._connectionFailure.emit(e)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(17),i=n(120),o="api/build";e.BuildManager=class{constructor(t={}){this._url="",this.serverSettings=t.serverSettings||i.ServerConnection.makeSettings();const{baseUrl:e,appUrl:n}=this.serverSettings;this._url=r.URLExt.join(e,n,o)}get isAvailable(){return"true"===r.PageConfig.getOption("buildAvailable").toLowerCase()}get shouldCheck(){return"true"===r.PageConfig.getOption("buildCheck").toLowerCase()}getStatus(){const{_url:t,serverSettings:e}=this;return i.ServerConnection.makeRequest(t,{},e).then(t=>{if(200!==t.status)throw new i.ServerConnection.ResponseError(t);return t.json()}).then(t=>{if("string"!=typeof t.status)throw new Error("Invalid data");if("string"!=typeof t.message)throw new Error("Invalid data");return t})}build(){const{_url:t,serverSettings:e}=this;return i.ServerConnection.makeRequest(t,{method:"POST"},e).then(t=>{if(400===t.status)throw new i.ServerConnection.ResponseError(t,"Build aborted");if(200!==t.status){let e=`Build failed with ${t.status}, please run 'jupyter lab build' on the server for full output`;throw new i.ServerConnection.ResponseError(t,e)}})}cancel(){const{_url:t,serverSettings:e}=this;return i.ServerConnection.makeRequest(t,{method:"DELETE"},e).then(t=>{if(204!==t.status)throw new i.ServerConnection.ResponseError(t)})}}},function(t,e){},function(t,e){},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(17),i=n(5),o=n(9),a=n(10),s=n(295),A=n(120),l=n(588);e.SessionManager=class{constructor(t={}){this._isDisposed=!1,this._isReady=!1,this._models=[],this._runningChanged=new a.Signal(this),this._connectionFailure=new a.Signal(this),this._sessions=new Set,this._specs=null,this._specsChanged=new a.Signal(this),this.serverSettings=t.serverSettings||A.ServerConnection.makeSettings(),this._ready=Promise.all([this.requestRunning(),this.requestSpecs()]).then(t=>void 0).catch(t=>void 0).then(()=>{this.isDisposed||(this._isReady=!0)}),this._pollModels=new r.Poll({auto:!1,factory:()=>this.requestRunning(),frequency:{interval:1e4,backoff:!0,max:3e5},name:"@jupyterlab/services:SessionManager#models",standby:t.standby||"when-hidden"}),this._pollSpecs=new r.Poll({auto:!1,factory:()=>this.requestSpecs(),frequency:{interval:61e3,backoff:!0,max:3e5},name:"@jupyterlab/services:SessionManager#specs",standby:t.standby||"when-hidden"}),this.ready.then(()=>{this._pollModels.start(),this._pollSpecs.start()})}get specsChanged(){return this._specsChanged}get runningChanged(){return this._runningChanged}get connectionFailure(){return this._connectionFailure}get isDisposed(){return this._isDisposed}get specs(){return this._specs}get isReady(){return this._isReady}get ready(){return this._ready}dispose(){this.isDisposed||(this._isDisposed=!0,this._models.length=0,this._pollModels.dispose(),this._pollSpecs.dispose(),a.Signal.clearData(this))}running(){return i.iter(this._models)}async refreshSpecs(){await this._pollSpecs.refresh(),await this._pollSpecs.tick}async refreshRunning(){await this._pollModels.refresh(),await this._pollModels.tick}async startNew(t){const{serverSettings:e}=this,n=await l.Session.startNew(Object.assign({},t,{serverSettings:e}));return this._onStarted(n),n}async stopIfNeeded(t){try{const e=(await l.Session.listRunning(this.serverSettings)).filter(e=>e.path===t);if(1===e.length){const t=e[0].id;return this.shutdown(t).catch(()=>{})}}catch(t){}}findById(t){return l.Session.findById(t,this.serverSettings)}findByPath(t){return l.Session.findByPath(t,this.serverSettings)}connectTo(t){const e=l.Session.connectTo(t,this.serverSettings);return this._onStarted(e),e}async shutdown(t){const e=this._models,n=this._sessions,r=i.ArrayExt.findFirstIndex(e,e=>e.id===t);-1!==r&&(e.splice(r,1),this._runningChanged.emit(e.slice()),n.forEach(e=>{e.id===t&&(n.delete(e),e.dispose())}),await l.Session.shutdown(t,this.serverSettings))}async shutdownAll(){try{await this.requestRunning(),await Promise.all(this._models.map(({id:t})=>l.Session.shutdown(t,this.serverSettings)))}finally{this._sessions.forEach(t=>{t.dispose()}),this._sessions.clear(),this._models.length&&(this._models.length=0,this._runningChanged.emit([]))}}async requestRunning(){const t=await l.Session.listRunning(this.serverSettings).catch(t=>{if(t instanceof A.ServerConnection.NetworkError||t.response&&503===t.response.status)return this._connectionFailure.emit(t),[];throw t});if(!this.isDisposed&&!o.JSONExt.deepEqual(t,this._models)){const e=t.map(t=>t.id),n=this._sessions;n.forEach(t=>{-1===e.indexOf(t.id)&&(t.dispose(),n.delete(t))}),this._models=t.slice(),this._runningChanged.emit(t)}}async requestSpecs(){const t=await s.Kernel.getSpecs(this.serverSettings);this.isDisposed||o.JSONExt.deepEqual(t,this._specs)||(this._specs=t,this._specsChanged.emit(t))}_onTerminated(t){this.refreshRunning().catch(t=>{if(!this.isDisposed)throw t})}_onStarted(t){let e=t.id,n=i.ArrayExt.findFirstIndex(this._models,t=>t.id===e);this._sessions.add(t),-1===n&&(this._models.push(t.model),this._runningChanged.emit(this._models.slice())),t.terminated.connect(t=>{this._onTerminated(e)}),t.propertyChanged.connect((e,n)=>{this._onChanged(t.model)}),t.kernelChanged.connect(()=>{this._onChanged(t.model)})}_onChanged(t){let e=i.ArrayExt.findFirstIndex(this._models,e=>e.id===t.id);-1!==e&&(this._models[e]=t,this._runningChanged.emit(this._models.slice()))}}},function(t,e,n){"use strict";var r=this&&this.__importStar||function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)Object.hasOwnProperty.call(t,n)&&(e[n]=t[n]);return e.default=t,e};Object.defineProperty(e,"__esModule",{value:!0});const i=n(17),o=n(5),a=n(10),s=n(295),A=n(60),l=r(n(1082)),c="api/sessions";class u{constructor(t,e,n){this._id="",this._path="",this._name="",this._type="",this._isDisposed=!1,this._updating=!1,this._kernelChanged=new a.Signal(this),this._statusChanged=new a.Signal(this),this._iopubMessage=new a.Signal(this),this._unhandledMessage=new a.Signal(this),this._anyMessage=new a.Signal(this),this._propertyChanged=new a.Signal(this),this._terminated=new a.Signal(this),this._id=e,this._path=t.path,this._type=t.type||"file",this._name=t.name||"",this.serverSettings=t.serverSettings||A.ServerConnection.makeSettings(),g.addRunning(this),this.setupKernel(n)}get terminated(){return this._terminated}get kernelChanged(){return this._kernelChanged}get statusChanged(){return this._statusChanged}get iopubMessage(){return this._iopubMessage}get unhandledMessage(){return this._unhandledMessage}get anyMessage(){return this._anyMessage}get propertyChanged(){return this._propertyChanged}get id(){return this._id}get kernel(){return this._kernel}get path(){return this._path}get type(){return this._type}get name(){return this._name}get model(){return{id:this.id,kernel:this.kernel.model,path:this._path,type:this._type,name:this._name}}get status(){return this._kernel?this._kernel.status:"dead"}get isDisposed(){return!0===this._isDisposed}clone(){return new u({path:this._path,name:this._name,type:this._type,serverSettings:this.serverSettings},this._id,this.kernel.model)}update(t){if(this._updating)return;let e=this.model;if(this._path=t.path,this._name=t.name,this._type=t.type,this._kernel.isDisposed||t.kernel.id!==this._kernel.id){let e=this._kernel;this.setupKernel(t.kernel);let n=this._kernel;e.dispose(),this._kernelChanged.emit({oldValue:e,newValue:n})}this._handleModelChange(e)}dispose(){this.isDisposed||(this._isDisposed=!0,this._kernel.dispose(),this._statusChanged.emit("dead"),this._terminated.emit(void 0),g.removeRunning(this),a.Signal.clearData(this))}setPath(t){if(this.isDisposed)return Promise.reject(new Error("Session is disposed"));let e=JSON.stringify({path:t});return this._patch(e).then(()=>{})}setName(t){if(this.isDisposed)return Promise.reject(new Error("Session is disposed"));let e=JSON.stringify({name:t});return this._patch(e).then(()=>{})}setType(t){if(this.isDisposed)return Promise.reject(new Error("Session is disposed"));let e=JSON.stringify({type:t});return this._patch(e).then(()=>{})}changeKernel(t){if(this.isDisposed)return Promise.reject(new Error("Session is disposed"));let e=JSON.stringify({kernel:t});return this._kernel.dispose(),this._statusChanged.emit("restarting"),this._patch(e).then(()=>this.kernel)}shutdown(){return this.isDisposed?Promise.reject(new Error("Session is disposed")):g.shutdownSession(this.id,this.serverSettings)}setupKernel(t){const e=s.Kernel.connectTo(t,this.serverSettings);this._kernel=e,e.statusChanged.connect(this.onKernelStatus,this),e.unhandledMessage.connect(this.onUnhandledMessage,this),e.iopubMessage.connect(this.onIOPubMessage,this),e.anyMessage.connect(this.onAnyMessage,this)}onKernelStatus(t,e){this._statusChanged.emit(e)}onIOPubMessage(t,e){this._iopubMessage.emit(e)}onUnhandledMessage(t,e){this._unhandledMessage.emit(e)}onAnyMessage(t,e){this._anyMessage.emit(e)}_patch(t){this._updating=!0;let e=this.serverSettings,n=g.getSessionUrl(e.baseUrl,this._id),r={method:"PATCH",body:t};return A.ServerConnection.makeRequest(n,r,e).then(t=>{if(this._updating=!1,200!==t.status)throw new A.ServerConnection.ResponseError(t);return t.json()}).then(t=>{let n=l.validateModel(t);return g.updateFromServer(n,e.baseUrl)},t=>{throw this._updating=!1,t})}_handleModelChange(t){t.name!==this._name&&this._propertyChanged.emit("name"),t.type!==this._type&&this._propertyChanged.emit("type"),t.path!==this._path&&this._propertyChanged.emit("path")}}var g;e.DefaultSession=u,function(t){t.listRunning=function(t){return g.listRunning(t)},t.startNew=function(t){return g.startNew(t)},t.findById=function(t,e){return g.findById(t,e)},t.findByPath=function(t,e){return g.findByPath(t,e)},t.connectTo=function(t,e){return g.connectTo(t,e)},t.shutdown=function(t,e){return g.shutdownSession(t,e)},t.shutdownAll=function(t){return g.shutdownAll(t)}}(u=e.DefaultSession||(e.DefaultSession={})),function(t){const e=new Map;function n(t,e){return e=e||A.ServerConnection.makeSettings(),new u({path:t.path,type:t.type,name:t.name,serverSettings:e},t.id,t.kernel)}function r(t,e){let n=a((e=e||A.ServerConnection.makeSettings()).baseUrl,t);return A.ServerConnection.makeRequest(n,{},e).then(t=>{if(200!==t.status)throw new A.ServerConnection.ResponseError(t);return t.json()}).then(t=>{return d(l.validateModel(t),e.baseUrl)})}function a(t,e){return i.URLExt.join(t,c,e)}function s(t){t=t||A.ServerConnection.makeSettings();let e=i.URLExt.join(t.baseUrl,c);return A.ServerConnection.makeRequest(e,{},t).then(t=>{if(200!==t.status)throw new A.ServerConnection.ResponseError(t);return t.json()}).then(e=>{if(!Array.isArray(e))throw new Error("Invalid Session list");for(let t=0;t<e.length;t++)e[t]=l.validateModel(e[t]);return f(e,t.baseUrl)})}function g(t,n){let r=a((n=n||A.ServerConnection.makeSettings()).baseUrl,t);return A.ServerConnection.makeRequest(r,{method:"DELETE"},n).then(r=>{if(404===r.status)return r.json().then(e=>{let n=e.message||`The session "${t}"" does not exist on the server`;console.warn(n)});if(410===r.status)throw new A.ServerConnection.ResponseError(r,"The kernel was deleted but the session was not");if(204!==r.status)throw new A.ServerConnection.ResponseError(r);!function(t,n){let r=e.get(n)||[];o.each(r.slice(),e=>{e.id===t&&e.dispose()})}(t,n.baseUrl)})}function h(t){let e=t.serverSettings||A.ServerConnection.makeSettings(),n={kernel:{name:t.kernelName,id:t.kernelId},path:t.path,type:t.type||"",name:t.name||""},r=i.URLExt.join(e.baseUrl,c),o={method:"POST",body:JSON.stringify(n)};return A.ServerConnection.makeRequest(r,o,e).then(t=>{if(201!==t.status)throw new A.ServerConnection.ResponseError(t);return t.json()}).then(t=>{return d(l.validateModel(t),e.baseUrl)})}function d(t,n){let r=e.get(n)||[];return o.each(r.slice(),e=>{e.id===t.id&&e.update(t)}),t}function f(t,n){let r=e.get(n)||[];return o.each(r.slice(),e=>{o.find(t,t=>e.id===t.id&&(e.update(t),!0))||"dead"===e.status||e.dispose()}),t}t.addRunning=function(t){let n=e.get(t.serverSettings.baseUrl)||[];n.push(t),e.set(t.serverSettings.baseUrl,n)},t.removeRunning=function(t){let n=e.get(t.serverSettings.baseUrl);n&&o.ArrayExt.removeFirstOf(n,t)},t.connectTo=function(t,r){r=r||A.ServerConnection.makeSettings();let i=e.get(r.baseUrl)||[],a=o.find(i,e=>e.id===t.id);return a?a.clone():n(t,r)},t.createSession=n,t.findById=function(t,n){n=n||A.ServerConnection.makeSettings();let i=e.get(n.baseUrl)||[],a=o.find(i,e=>e.id===t);return a?Promise.resolve(a.model):r(t,n).catch(()=>{throw new Error(`No running session for id: ${t}`)})},t.findByPath=function(t,n){n=n||A.ServerConnection.makeSettings();let r=e.get(n.baseUrl)||[],i=o.find(r,e=>e.path===t);return i?Promise.resolve(i.model):s(n).then(e=>{let n=o.find(e,e=>e.path===t);if(n)return n;throw new Error(`No running session for path: ${t}`)})},t.getSessionModel=r,t.getSessionUrl=a,t.listRunning=s,t.shutdownSession=g,t.shutdownAll=async function(t){t=t||A.ServerConnection.makeSettings();const e=await s(t);await Promise.all(e.map(e=>g(e.id,t)))},t.startNew=function(t){return void 0===t.path?Promise.reject(new Error("Must specify a path")):h(t).then(e=>n(e,t.serverSettings))},t.startSession=h,t.updateFromServer=d,t.updateRunningSessions=f}(g||(g={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(585);function i(t,e,n){if(!t.hasOwnProperty(e))throw Error(`Missing property '${e}'`);if(void 0!==n){let r=!0,i=t[e];switch(n){case"array":r=Array.isArray(i);break;case"object":r=void 0!==i;break;default:r=typeof i===n}if(!r)throw new Error(`Property '${e}' is not of type '${n}'`)}}e.validateModel=function(t){let e={id:t.id,kernel:t.kernel,name:t.name,path:t.path,type:t.type};return void 0===t.path&&void 0!==t.notebook&&(e.path=t.notebook.path,e.type="notebook",e.name=""),i(e,"id","string"),i(e,"type","string"),i(e,"name","string"),i(e,"path","string"),i(e,"kernel","object"),r.validateModel(e.kernel),e}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(17),i=n(5),o=n(9),a=n(10),s=n(60),A=n(296);e.TerminalManager=class{constructor(t={}){if(this._isDisposed=!1,this._isReady=!1,this._models=[],this._sessions=new Set,this._runningChanged=new a.Signal(this),this._connectionFailure=new a.Signal(this),this.serverSettings=t.serverSettings||s.ServerConnection.makeSettings(),!A.TerminalSession.isAvailable())return this._ready=Promise.reject("Terminals unavailable"),void this._ready.catch(t=>void 0);this._ready=this.requestRunning().then(t=>void 0).catch(t=>void 0).then(()=>{this.isDisposed||(this._isReady=!0)}),this._pollModels=new r.Poll({auto:!1,factory:()=>this.requestRunning(),frequency:{interval:1e4,backoff:!0,max:3e5},name:"@jupyterlab/services:TerminalManager#models",standby:t.standby||"when-hidden"}),this.ready.then(()=>{this._pollModels.start()})}get runningChanged(){return this._runningChanged}get connectionFailure(){return this._connectionFailure}get isDisposed(){return this._isDisposed}get isReady(){return this._isReady}dispose(){this.isDisposed||(this._isDisposed=!0,this._models.length=0,this._pollModels.dispose(),a.Signal.clearData(this))}get ready(){return this._ready}isAvailable(){return A.TerminalSession.isAvailable()}running(){return i.iter(this._models)}async startNew(t){const e=await A.TerminalSession.startNew(this._getOptions(t));return this._onStarted(e),e}async connectTo(t,e){const n=await A.TerminalSession.connectTo(t,this._getOptions(e));return this._onStarted(n),n}async refreshRunning(){await this._pollModels.refresh(),await this._pollModels.tick}async shutdown(t){const e=this._models,n=this._sessions,r=i.ArrayExt.findFirstIndex(e,e=>e.name===t);-1!==r&&(e.splice(r,1),this._runningChanged.emit(e.slice()),n.forEach(e=>{e.name===t&&(n.delete(e),e.dispose())}),await A.TerminalSession.shutdown(t,this.serverSettings))}async shutdownAll(){try{await this.requestRunning(),await Promise.all(this._models.map(({name:t})=>A.TerminalSession.shutdown(t,this.serverSettings)))}finally{this._sessions.forEach(t=>{t.dispose()}),this._sessions.clear(),this._models.length&&(this._models.length=0,this._runningChanged.emit([]))}}async requestRunning(){const t=await A.TerminalSession.listRunning(this.serverSettings).catch(t=>{if(t instanceof s.ServerConnection.NetworkError||t.response&&503===t.response.status)return this._connectionFailure.emit(t),[];throw t});if(!this.isDisposed&&!o.JSONExt.deepEqual(t,this._models)){const e=t.map(({name:t})=>t),n=this._sessions;n.forEach(t=>{-1===e.indexOf(t.name)&&(t.dispose(),n.delete(t))}),this._models=t.slice(),this._runningChanged.emit(t)}}_getOptions(t={}){return Object.assign({},t,{serverSettings:this.serverSettings})}_onStarted(t){let e=t.name;this._sessions.add(t),-1===i.ArrayExt.findFirstIndex(this._models,t=>t.name===e)&&(this._models.push(t.model),this._runningChanged.emit(this._models.slice())),t.terminated.connect(()=>{this._onTerminated(e)})}_onTerminated(t){let e=i.ArrayExt.findFirstIndex(this._models,e=>e.name===t);-1!==e&&(this._models.splice(e,1),this._runningChanged.emit(this._models.slice()));const n=this._sessions;n.forEach(e=>{e.name===t&&n.delete(e)})}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(17),i=n(5),o=n(10),a=n(60),s=n(296),A="api/terminals";class l{constructor(t,e={}){this._isDisposed=!1,this._isReady=!1,this._messageReceived=new o.Signal(this),this._terminated=new o.Signal(this),this._ws=null,this._noOp=()=>{},this._reconnectLimit=7,this._reconnectAttempt=0,this._disconnected=!1,this._name=t,this.serverSettings=e.serverSettings||a.ServerConnection.makeSettings(),this._readyPromise=this._initializeSocket()}get terminated(){return this._terminated}get messageReceived(){return this._messageReceived}get name(){return this._name}get model(){return{name:this._name}}get isReady(){return this._isReady}get ready(){return this._readyPromise}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this.terminated.emit(void 0),this._isDisposed=!0,this._ws&&(this._ws.close(),this._ws=null),delete c.running[this._url],o.Signal.clearData(this))}send(t){if(this._isDisposed||!t.content)return;const e=[t.type,...t.content],n=this._ws,r=JSON.stringify(e);this._isReady&&n?n.send(r):this.ready.then(()=>{const t=this._ws;t&&t.send(r)})}reconnect(){return this._reconnectAttempt=0,this._readyPromise=this._initializeSocket(),this._readyPromise}shutdown(){const{name:t,serverSettings:e}=this;return l.shutdown(t,e)}clone(){const{name:t,serverSettings:e}=this;return new l(t,{serverSettings:e})}_initializeSocket(){const t=this._name;let e=this._ws;return e&&(e.onopen=this._noOp,e.onclose=this._noOp,e.onerror=this._noOp,e.onmessage=this._noOp,e.close(),this._ws=null),this._isReady=!1,new Promise((n,i)=>{const o=this.serverSettings,a=this.serverSettings.token;this._url=c.getTermUrl(o.baseUrl,this._name),c.running[this._url]=this;let s=r.URLExt.join(o.wsUrl,`terminals/websocket/${t}`);a&&(s+=`?token=${encodeURIComponent(a)}`),(e=this._ws=new o.WebSocket(s)).onmessage=t=>{if(this._isDisposed)return;const e=JSON.parse(t.data);"disconnect"===e[0]&&(this._disconnected=!0),this._reconnectAttempt>0?"setup"===e[0]&&(this._reconnectAttempt=0):this._messageReceived.emit({type:e[0],content:e.slice(1)})},e.onopen=t=>{this._isDisposed||(this._isReady=!0,this._disconnected=!1,n(void 0))},e.onerror=t=>{this._isDisposed||i(t)},e.onclose=t=>{console.warn(`Terminal websocket closed: ${t.code}`),this._disconnected&&this.dispose(),this._reconnectSocket()}})}_reconnectSocket(){if(this._isDisposed||!this._ws||this._disconnected)return;const t=this._reconnectAttempt;if(t>=this._reconnectLimit)return void console.log(`Terminal reconnect aborted: ${t} attempts`);const e=Math.pow(2,t);console.log(`Terminal will attempt to reconnect in ${e}s`),this._isReady=!1,this._reconnectAttempt+=1,setTimeout(()=>{this.isDisposed||this._initializeSocket().then(()=>{console.log("Terminal reconnected")}).catch(t=>{console.warn("Terminal reconnect failed",t)})},1e3*e)}}var c;e.DefaultTerminalSession=l,function(t){function e(t){if(!s.TerminalSession.isAvailable())return Promise.reject(c.unavailableMsg);t=t||a.ServerConnection.makeSettings();let e=c.getServiceUrl(t.baseUrl);return a.ServerConnection.makeRequest(e,{},t).then(t=>{if(200!==t.status)throw new a.ServerConnection.ResponseError(t);return t.json()}).then(t=>{if(!Array.isArray(t))throw new Error("Invalid terminal data");let n=i.toArray(i.map(t,t=>r.URLExt.join(e,t.name)));return i.each(Object.keys(c.running),t=>{if(-1===n.indexOf(t)){c.running[t].dispose()}}),t})}function n(t,e){if(!s.TerminalSession.isAvailable())return Promise.reject(c.unavailableMsg);e=e||a.ServerConnection.makeSettings();let n=c.getTermUrl(e.baseUrl,t);return a.ServerConnection.makeRequest(n,{method:"DELETE"},e).then(t=>{if(404===t.status)return t.json().then(t=>{console.warn(t.message)});if(204!==t.status)throw new a.ServerConnection.ResponseError(t)})}t.isAvailable=function(){return"true"===String(r.PageConfig.getOption("terminalsAvailable")).toLowerCase()},t.startNew=function(e={}){if(!s.TerminalSession.isAvailable())throw c.unavailableMsg;let n=e.serverSettings||a.ServerConnection.makeSettings(),r=c.getServiceUrl(n.baseUrl);return a.ServerConnection.makeRequest(r,{method:"POST"},n).then(t=>{if(200!==t.status)throw new a.ServerConnection.ResponseError(t);return t.json()}).then(r=>{let i=r.name;return new t(i,Object.assign({},e,{serverSettings:n}))})},t.connectTo=function(n,r={}){if(!s.TerminalSession.isAvailable())return Promise.reject(c.unavailableMsg);let o=r.serverSettings||a.ServerConnection.makeSettings(),A=c.getTermUrl(o.baseUrl,n);return A in c.running?Promise.resolve(c.running[A].clone()):e(o).then(e=>{if(-1!==i.ArrayExt.findFirstIndex(e,t=>t.name===n)){let e=new t(n,Object.assign({},r,{serverSettings:o}));return Promise.resolve(e)}return Promise.reject("Could not find session")})},t.listRunning=e,t.shutdown=n,t.shutdownAll=async function(t){t=t||a.ServerConnection.makeSettings();const r=await e(t);await Promise.all(r.map(e=>n(e.name,t)))}}(l=e.DefaultTerminalSession||(e.DefaultTerminalSession={})),function(t){t.running=Object.create(null),t.unavailableMsg="Terminals Unavailable",t.getTermUrl=function(t,e){return r.URLExt.join(t,A,e)},t.getServiceUrl=function(t){return r.URLExt.join(t,A)}}(c||(c={}))},function(t){t.exports=JSON.parse('{"0":65533,"128":8364,"130":8218,"131":402,"132":8222,"133":8230,"134":8224,"135":8225,"136":710,"137":8240,"138":352,"139":8249,"140":338,"142":381,"145":8216,"146":8217,"147":8220,"148":8221,"149":8226,"150":8211,"151":8212,"152":732,"153":8482,"154":353,"155":8250,"156":339,"158":382,"159":376}')},function(t,e,n){var r=n(598),i=t.exports=Object.create(r),o={tagName:"name"};Object.keys(o).forEach(function(t){var e=o[t];Object.defineProperty(i,t,{get:function(){return this[e]||null},set:function(t){return this[e]=t,t}})})},function(t,e,n){var r=n(597),i=n(599);function o(t,e){this.init(t,e)}function a(t,e){return i.getElementsByTagName(t,e,!0)}function s(t,e){return i.getElementsByTagName(t,e,!0,1)[0]}function A(t,e,n){return i.getText(i.getElementsByTagName(t,e,n,1)).trim()}function l(t,e,n,r,i){var o=A(n,r,i);o&&(t[e]=o)}n(30)(o,r),o.prototype.init=r;var c=function(t){return"rss"===t||"feed"===t||"rdf:RDF"===t};o.prototype.onend=function(){var t,e,n={},i=s(c,this.dom);i&&("feed"===i.name?(e=i.children,n.type="atom",l(n,"id","id",e),l(n,"title","title",e),(t=s("link",e))&&(t=t.attribs)&&(t=t.href)&&(n.link=t),l(n,"description","subtitle",e),(t=A("updated",e))&&(n.updated=new Date(t)),l(n,"author","email",e,!0),n.items=a("entry",e).map(function(t){var e,n={};return l(n,"id","id",t=t.children),l(n,"title","title",t),(e=s("link",t))&&(e=e.attribs)&&(e=e.href)&&(n.link=e),(e=A("summary",t)||A("content",t))&&(n.description=e),(e=A("updated",t))&&(n.pubDate=new Date(e)),n})):(e=s("channel",i.children).children,n.type=i.name.substr(0,3),n.id="",l(n,"title","title",e),l(n,"link","link",e),l(n,"description","description",e),(t=A("lastBuildDate",e))&&(n.updated=new Date(t)),l(n,"author","managingEditor",e,!0),n.items=a("item",i.children).map(function(t){var e,n={};return l(n,"id","guid",t=t.children),l(n,"title","title",t),l(n,"link","link",t),l(n,"description","description",t),(e=A("pubDate",t))&&(n.pubDate=new Date(e)),n}))),this.dom=n,r.prototype._handleCallback.call(this,i?null:Error("couldn't find root of feed"))},t.exports=o},function(t,e,n){var r=n(138),i=n(1089),o=r.isTag;t.exports={getInnerHTML:function(t,e){return t.children?t.children.map(function(t){return i(t,e)}).join(""):""},getOuterHTML:i,getText:function t(e){if(Array.isArray(e))return e.map(t).join("");if(o(e))return"br"===e.name?"\n":t(e.children);if(e.type===r.CDATA)return t(e.children);if(e.type===r.Text)return e.data;return""}}},function(t,e,n){var r=n(138),i=n(1090),o={__proto__:null,style:!0,script:!0,xmp:!0,iframe:!0,noembed:!0,noframes:!0,plaintext:!0,noscript:!0};var a={__proto__:null,area:!0,base:!0,basefont:!0,br:!0,col:!0,command:!0,embed:!0,frame:!0,hr:!0,img:!0,input:!0,isindex:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0},s=t.exports=function(t,e){Array.isArray(t)||t.cheerio||(t=[t]),e=e||{};for(var n="",i=0;i<t.length;i++){var o=t[i];"root"===o.type?n+=s(o.children,e):r.isTag(o)?n+=A(o,e):o.type===r.Directive?n+=l(o):o.type===r.Comment?n+=g(o):o.type===r.CDATA?n+=u(o):n+=c(o,e)}return n};function A(t,e){"svg"===t.name&&(e={decodeEntities:e.decodeEntities,xmlMode:!0});var n="<"+t.name,r=function(t,e){if(t){var n,r="";for(var o in t)r&&(r+=" "),r+=o,(null!==(n=t[o])&&""!==n||e.xmlMode)&&(r+='="'+(e.decodeEntities?i.encodeXML(n):n)+'"');return r}}(t.attribs,e);return r&&(n+=" "+r),!e.xmlMode||t.children&&0!==t.children.length?(n+=">",t.children&&(n+=s(t.children,e)),a[t.name]&&!e.xmlMode||(n+="</"+t.name+">")):n+="/>",n}function l(t){return"<"+t.data+">"}function c(t,e){var n=t.data||"";return!e.decodeEntities||t.parent&&t.parent.name in o||(n=i.encodeXML(n)),n}function u(t){return"<![CDATA["+t.children[0].data+"]]>"}function g(t){return"\x3c!--"+t.data+"--\x3e"}},function(t,e,n){var r=n(1091),i=n(1092);e.decode=function(t,e){return(!e||e<=0?i.XML:i.HTML)(t)},e.decodeStrict=function(t,e){return(!e||e<=0?i.XML:i.HTMLStrict)(t)},e.encode=function(t,e){return(!e||e<=0?r.XML:r.HTML)(t)},e.encodeXML=r.XML,e.encodeHTML4=e.encodeHTML5=e.encodeHTML=r.HTML,e.decodeXML=e.decodeXMLStrict=i.XML,e.decodeHTML4=e.decodeHTML5=e.decodeHTML=i.HTML,e.decodeHTML4Strict=e.decodeHTML5Strict=e.decodeHTMLStrict=i.HTMLStrict,e.escape=r.escape},function(t,e,n){var r=s(n(298)),i=A(r);e.XML=h(r,i);var o=s(n(297)),a=A(o);function s(t){return Object.keys(t).sort().reduce(function(e,n){return e[t[n]]="&"+n+";",e},{})}function A(t){var e=[],n=[];return Object.keys(t).forEach(function(t){1===t.length?e.push("\\"+t):n.push(t)}),n.unshift("["+e.join("")+"]"),new RegExp(n.join("|"),"g")}e.HTML=h(o,a);var l=/[^\0-\x7F]/g,c=/[\uD800-\uDBFF][\uDC00-\uDFFF]/g;function u(t){return"&#x"+t.charCodeAt(0).toString(16).toUpperCase()+";"}function g(t){return"&#x"+(1024*(t.charCodeAt(0)-55296)+t.charCodeAt(1)-56320+65536).toString(16).toUpperCase()+";"}function h(t,e){function n(e){return t[e]}return function(t){return t.replace(e,n).replace(c,g).replace(l,u)}}var d=A(r);e.escape=function(t){return t.replace(d,u).replace(c,g).replace(l,u)}},function(t,e,n){var r=n(297),i=n(596),o=n(298),a=n(595),s=l(o),A=l(r);function l(t){var e=Object.keys(t).join("|"),n=g(t),r=new RegExp("&(?:"+(e+="|#[xX][\\da-fA-F]+|#\\d+")+");","g");return function(t){return String(t).replace(r,n)}}var c=function(){for(var t=Object.keys(i).sort(u),e=Object.keys(r).sort(u),n=0,o=0;n<e.length;n++)t[o]===e[n]?(e[n]+=";?",o++):e[n]+=";";var a=new RegExp("&(?:"+e.join("|")+"|#[xX][\\da-fA-F]+;?|#\\d+;?)","g"),s=g(r);function A(t){return";"!==t.substr(-1)&&(t+=";"),s(t)}return function(t){return String(t).replace(a,A)}}();function u(t,e){return t<e?1:-1}function g(t){return function(e){return"#"===e.charAt(1)?"X"===e.charAt(2)||"x"===e.charAt(2)?a(parseInt(e.substr(3),16)):a(parseInt(e.substr(2),10)):t[e.slice(1,-1)]}}t.exports={XML:s,HTML:c,HTMLStrict:A}},function(t,e){var n=e.getChildren=function(t){return t.children},r=e.getParent=function(t){return t.parent};e.getSiblings=function(t){var e=r(t);return e?n(e):[t]},e.getAttributeValue=function(t,e){return t.attribs&&t.attribs[e]},e.hasAttrib=function(t,e){return!!t.attribs&&hasOwnProperty.call(t.attribs,e)},e.getName=function(t){return t.name}},function(t,e){e.removeElement=function(t){if(t.prev&&(t.prev.next=t.next),t.next&&(t.next.prev=t.prev),t.parent){var e=t.parent.children;e.splice(e.lastIndexOf(t),1)}},e.replaceElement=function(t,e){var n=e.prev=t.prev;n&&(n.next=e);var r=e.next=t.next;r&&(r.prev=e);var i=e.parent=t.parent;if(i){var o=i.children;o[o.lastIndexOf(t)]=e}},e.appendChild=function(t,e){if(e.parent=t,1!==t.children.push(e)){var n=t.children[t.children.length-2];n.next=e,e.prev=n,e.next=null}},e.append=function(t,e){var n=t.parent,r=t.next;if(e.next=r,e.prev=t,t.next=e,e.parent=n,r){if(r.prev=e,n){var i=n.children;i.splice(i.lastIndexOf(r),0,e)}}else n&&n.children.push(e)},e.prepend=function(t,e){var n=t.parent;if(n){var r=n.children;r.splice(r.lastIndexOf(t),0,e)}t.prev&&(t.prev.next=e),e.parent=n,e.prev=t.prev,e.next=t,t.prev=e}},function(t,e,n){var r=n(138).isTag;function i(t,e,n,r){for(var o,a=[],s=0,A=e.length;s<A&&!(t(e[s])&&(a.push(e[s]),--r<=0))&&(o=e[s].children,!(n&&o&&o.length>0&&(o=i(t,o,n,r),a=a.concat(o),(r-=o.length)<=0)));s++);return a}t.exports={filter:function(t,e,n,r){Array.isArray(e)||(e=[e]);"number"==typeof r&&isFinite(r)||(r=1/0);return i(t,e,!1!==n,r)},find:i,findOneChild:function(t,e){for(var n=0,r=e.length;n<r;n++)if(t(e[n]))return e[n];return null},findOne:function t(e,n){var i=null;for(var o=0,a=n.length;o<a&&!i;o++)r(n[o])&&(e(n[o])?i=n[o]:n[o].children.length>0&&(i=t(e,n[o].children)));return i},existsOne:function t(e,n){for(var i=0,o=n.length;i<o;i++)if(r(n[i])&&(e(n[i])||n[i].children.length>0&&t(e,n[i].children)))return!0;return!1},findAll:function(t,e){var n=[],i=e.slice();for(;i.length;){var o=i.shift();r(o)&&(o.children&&o.children.length>0&&i.unshift.apply(i,o.children),t(o)&&n.push(o))}return n}}},function(t,e,n){var r=n(138),i=e.isTag=r.isTag;e.testElement=function(t,e){for(var n in t)if(t.hasOwnProperty(n)){if("tag_name"===n){if(!i(e)||!t.tag_name(e.name))return!1}else if("tag_type"===n){if(!t.tag_type(e.type))return!1}else if("tag_contains"===n){if(i(e)||!t.tag_contains(e.data))return!1}else if(!e.attribs||!t[n](e.attribs[n]))return!1}else;return!0};var o={tag_name:function(t){return"function"==typeof t?function(e){return i(e)&&t(e.name)}:"*"===t?i:function(e){return i(e)&&e.name===t}},tag_type:function(t){return"function"==typeof t?function(e){return t(e.type)}:function(e){return e.type===t}},tag_contains:function(t){return"function"==typeof t?function(e){return!i(e)&&t(e.data)}:function(e){return!i(e)&&e.data===t}}};function a(t,e){return"function"==typeof e?function(n){return n.attribs&&e(n.attribs[t])}:function(n){return n.attribs&&n.attribs[t]===e}}function s(t,e){return function(n){return t(n)||e(n)}}e.getElements=function(t,e,n,r){var i=Object.keys(t).map(function(e){var n=t[e];return e in o?o[e](n):a(e,n)});return 0===i.length?[]:this.filter(i.reduce(s),e,n,r)},e.getElementById=function(t,e,n){return Array.isArray(e)||(e=[e]),this.findOne(a("id",t),e,!1!==n)},e.getElementsByTagName=function(t,e,n,r){return this.filter(o.tag_name(t),e,n,r)},e.getElementsByTagType=function(t,e,n,r){return this.filter(o.tag_type(t),e,n,r)}},function(t,e){e.removeSubsets=function(t){for(var e,n,r,i=t.length;--i>-1;){for(e=n=t[i],t[i]=null,r=!0;n;){if(t.indexOf(n)>-1){r=!1,t.splice(i,1);break}n=n.parent}r&&(t[i]=e)}return t};var n=1,r=2,i=4,o=8,a=16,s=e.compareDocumentPosition=function(t,e){var s,A,l,c,u,g,h=[],d=[];if(t===e)return 0;for(s=t;s;)h.unshift(s),s=s.parent;for(s=e;s;)d.unshift(s),s=s.parent;for(g=0;h[g]===d[g];)g++;return 0===g?n:(l=(A=h[g-1]).children,c=h[g],u=d[g],l.indexOf(c)>l.indexOf(u)?A===e?i|a:i:A===t?r|o:r)};e.uniqueSort=function(t){var e,n,o=t.length;for(t=t.slice();--o>-1;)e=t[o],(n=t.indexOf(e))>-1&&n<o&&t.splice(o,1);return t.sort(function(t,e){var n=s(t,e);return n&r?-1:n&i?1:0}),t}},function(t,e,n){t.exports=i;var r=n(600);function i(t){r.call(this,new o(this),t)}function o(t){this.scope=t}n(30)(i,r),i.prototype.readable=!0;var a=n(217).EVENTS;Object.keys(a).forEach(function(t){if(0===a[t])o.prototype["on"+t]=function(){this.scope.emit(t)};else if(1===a[t])o.prototype["on"+t]=function(e){this.scope.emit(t,e)};else{if(2!==a[t])throw Error("wrong number of arguments!");o.prototype["on"+t]=function(e,n){this.scope.emit(t,e,n)}}})},function(t,e){},function(t,e,n){function r(t){this._cbs=t||{}}t.exports=r;var i=n(217).EVENTS;Object.keys(i).forEach(function(t){if(0===i[t])t="on"+t,r.prototype[t]=function(){this._cbs[t]&&this._cbs[t]()};else if(1===i[t])t="on"+t,r.prototype[t]=function(e){this._cbs[t]&&this._cbs[t](e)};else{if(2!==i[t])throw Error("wrong number of arguments");t="on"+t,r.prototype[t]=function(e,n){this._cbs[t]&&this._cbs[t](e,n)}}})},function(t,e,n){function r(t){this._cbs=t||{},this.events=[]}t.exports=r;var i=n(217).EVENTS;Object.keys(i).forEach(function(t){if(0===i[t])t="on"+t,r.prototype[t]=function(){this.events.push([t]),this._cbs[t]&&this._cbs[t]()};else if(1===i[t])t="on"+t,r.prototype[t]=function(e){this.events.push([t,e]),this._cbs[t]&&this._cbs[t](e)};else{if(2!==i[t])throw Error("wrong number of arguments");t="on"+t,r.prototype[t]=function(e,n){this.events.push([t,e,n]),this._cbs[t]&&this._cbs[t](e,n)}}}),r.prototype.onreset=function(){this.events=[],this._cbs.onreset&&this._cbs.onreset()},r.prototype.restart=function(){this._cbs.onreset&&this._cbs.onreset();for(var t=0,e=this.events.length;t<e;t++)if(this._cbs[this.events[t][0]]){var n=this.events[t].length;1===n?this._cbs[this.events[t][0]]():2===n?this._cbs[this.events[t][0]](this.events[t][1]):this._cbs[this.events[t][0]](this.events[t][1],this.events[t][2])}}},function(t,e){t.exports=function(){for(var t={},e=0;e<arguments.length;e++){var r=arguments[e];for(var i in r)n.call(r,i)&&(t[i]=r[i])}return t};var n=Object.prototype.hasOwnProperty},function(t,e,n){(function(e){var n=1/0,r="[object Symbol]",i=/[\\^$.*+?()[\]{}|]/g,o=RegExp(i.source),a="object"==typeof e&&e&&e.Object===Object&&e,s="object"==typeof self&&self&&self.Object===Object&&self,A=a||s||Function("return this")(),l=Object.prototype.toString,c=A.Symbol,u=c?c.prototype:void 0,g=u?u.toString:void 0;function h(t){if("string"==typeof t)return t;if(function(t){return"symbol"==typeof t||function(t){return!!t&&"object"==typeof t}(t)&&l.call(t)==r}(t))return g?g.call(t):"";var e=t+"";return"0"==e&&1/t==-n?"-0":e}t.exports=function(t){var e;return(t=null==(e=t)?"":h(e))&&o.test(t)?t.replace(i,"\\$&"):t}}).call(this,n(37))},function(t,e,n){(function(t,n){var r=200,i="__lodash_hash_undefined__",o=9007199254740991,a="[object Arguments]",s="[object Boolean]",A="[object Date]",l="[object Function]",c="[object GeneratorFunction]",u="[object Map]",g="[object Number]",h="[object Object]",d="[object RegExp]",f="[object Set]",p="[object String]",I="[object Symbol]",C="[object ArrayBuffer]",m="[object DataView]",y="[object Float32Array]",E="[object Float64Array]",v="[object Int8Array]",B="[object Int16Array]",b="[object Int32Array]",Q="[object Uint8Array]",w="[object Uint8ClampedArray]",x="[object Uint16Array]",_="[object Uint32Array]",D=/\w*$/,M=/^\[object .+?Constructor\]$/,k=/^(?:0|[1-9]\d*)$/,S={};S[a]=S["[object Array]"]=S[C]=S[m]=S[s]=S[A]=S[y]=S[E]=S[v]=S[B]=S[b]=S[u]=S[g]=S[h]=S[d]=S[f]=S[p]=S[I]=S[Q]=S[w]=S[x]=S[_]=!0,S["[object Error]"]=S[l]=S["[object WeakMap]"]=!1;var L="object"==typeof t&&t&&t.Object===Object&&t,R="object"==typeof self&&self&&self.Object===Object&&self,N=L||R||Function("return this")(),F=e&&!e.nodeType&&e,T=F&&"object"==typeof n&&n&&!n.nodeType&&n,U=T&&T.exports===F;function O(t,e){return t.set(e[0],e[1]),t}function G(t,e){return t.add(e),t}function H(t,e,n,r){var i=-1,o=t?t.length:0;for(r&&o&&(n=t[++i]);++i<o;)n=e(n,t[i],i,t);return n}function j(t){var e=!1;if(null!=t&&"function"!=typeof t.toString)try{e=!!(t+"")}catch(t){}return e}function Y(t){var e=-1,n=Array(t.size);return t.forEach(function(t,r){n[++e]=[r,t]}),n}function P(t,e){return function(n){return t(e(n))}}function K(t){var e=-1,n=Array(t.size);return t.forEach(function(t){n[++e]=t}),n}var z,J=Array.prototype,q=Function.prototype,V=Object.prototype,W=N["__core-js_shared__"],X=(z=/[^.]+$/.exec(W&&W.keys&&W.keys.IE_PROTO||""))?"Symbol(src)_1."+z:"",Z=q.toString,$=V.hasOwnProperty,tt=V.toString,et=RegExp("^"+Z.call($).replace(/[\\^$.*+?()[\]{}|]/g,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),nt=U?N.Buffer:void 0,rt=N.Symbol,it=N.Uint8Array,ot=P(Object.getPrototypeOf,Object),at=Object.create,st=V.propertyIsEnumerable,At=J.splice,lt=Object.getOwnPropertySymbols,ct=nt?nt.isBuffer:void 0,ut=P(Object.keys,Object),gt=Tt(N,"DataView"),ht=Tt(N,"Map"),dt=Tt(N,"Promise"),ft=Tt(N,"Set"),pt=Tt(N,"WeakMap"),It=Tt(Object,"create"),Ct=jt(gt),mt=jt(ht),yt=jt(dt),Et=jt(ft),vt=jt(pt),Bt=rt?rt.prototype:void 0,bt=Bt?Bt.valueOf:void 0;function Qt(t){var e=-1,n=t?t.length:0;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function wt(t){var e=-1,n=t?t.length:0;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function xt(t){var e=-1,n=t?t.length:0;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function _t(t){this.__data__=new wt(t)}function Dt(t,e){var n=Pt(t)||function(t){return function(t){return function(t){return!!t&&"object"==typeof t}(t)&&Kt(t)}(t)&&$.call(t,"callee")&&(!st.call(t,"callee")||tt.call(t)==a)}(t)?function(t,e){for(var n=-1,r=Array(t);++n<t;)r[n]=e(n);return r}(t.length,String):[],r=n.length,i=!!r;for(var o in t)!e&&!$.call(t,o)||i&&("length"==o||Gt(o,r))||n.push(o);return n}function Mt(t,e,n){var r=t[e];$.call(t,e)&&Yt(r,n)&&(void 0!==n||e in t)||(t[e]=n)}function kt(t,e){for(var n=t.length;n--;)if(Yt(t[n][0],e))return n;return-1}function St(t,e,n,r,i,o,M){var k;if(r&&(k=o?r(t,i,o,M):r(t)),void 0!==k)return k;if(!qt(t))return t;var L=Pt(t);if(L){if(k=function(t){var e=t.length,n=t.constructor(e);e&&"string"==typeof t[0]&&$.call(t,"index")&&(n.index=t.index,n.input=t.input);return n}(t),!e)return function(t,e){var n=-1,r=t.length;e||(e=Array(r));for(;++n<r;)e[n]=t[n];return e}(t,k)}else{var R=Ot(t),N=R==l||R==c;if(zt(t))return function(t,e){if(e)return t.slice();var n=new t.constructor(t.length);return t.copy(n),n}(t,e);if(R==h||R==a||N&&!o){if(j(t))return o?t:{};if(k=function(t){return"function"!=typeof t.constructor||Ht(t)?{}:(e=ot(t),qt(e)?at(e):{});var e}(N?{}:t),!e)return function(t,e){return Nt(t,Ut(t),e)}(t,function(t,e){return t&&Nt(e,Vt(e),t)}(k,t))}else{if(!S[R])return o?t:{};k=function(t,e,n,r){var i=t.constructor;switch(e){case C:return Rt(t);case s:case A:return new i(+t);case m:return function(t,e){var n=e?Rt(t.buffer):t.buffer;return new t.constructor(n,t.byteOffset,t.byteLength)}(t,r);case y:case E:case v:case B:case b:case Q:case w:case x:case _:return function(t,e){var n=e?Rt(t.buffer):t.buffer;return new t.constructor(n,t.byteOffset,t.length)}(t,r);case u:return function(t,e,n){return H(e?n(Y(t),!0):Y(t),O,new t.constructor)}(t,r,n);case g:case p:return new i(t);case d:return function(t){var e=new t.constructor(t.source,D.exec(t));return e.lastIndex=t.lastIndex,e}(t);case f:return function(t,e,n){return H(e?n(K(t),!0):K(t),G,new t.constructor)}(t,r,n);case I:return o=t,bt?Object(bt.call(o)):{}}var o}(t,R,St,e)}}M||(M=new _t);var F=M.get(t);if(F)return F;if(M.set(t,k),!L)var T=n?function(t){return function(t,e,n){var r=e(t);return Pt(t)?r:function(t,e){for(var n=-1,r=e.length,i=t.length;++n<r;)t[i+n]=e[n];return t}(r,n(t))}(t,Vt,Ut)}(t):Vt(t);return function(t,e){for(var n=-1,r=t?t.length:0;++n<r&&!1!==e(t[n],n,t););}(T||t,function(i,o){T&&(i=t[o=i]),Mt(k,o,St(i,e,n,r,o,t,M))}),k}function Lt(t){return!(!qt(t)||(e=t,X&&X in e))&&(Jt(t)||j(t)?et:M).test(jt(t));var e}function Rt(t){var e=new t.constructor(t.byteLength);return new it(e).set(new it(t)),e}function Nt(t,e,n,r){n||(n={});for(var i=-1,o=e.length;++i<o;){var a=e[i],s=r?r(n[a],t[a],a,n,t):void 0;Mt(n,a,void 0===s?t[a]:s)}return n}function Ft(t,e){var n,r,i=t.__data__;return("string"==(r=typeof(n=e))||"number"==r||"symbol"==r||"boolean"==r?"__proto__"!==n:null===n)?i["string"==typeof e?"string":"hash"]:i.map}function Tt(t,e){var n=function(t,e){return null==t?void 0:t[e]}(t,e);return Lt(n)?n:void 0}Qt.prototype.clear=function(){this.__data__=It?It(null):{}},Qt.prototype.delete=function(t){return this.has(t)&&delete this.__data__[t]},Qt.prototype.get=function(t){var e=this.__data__;if(It){var n=e[t];return n===i?void 0:n}return $.call(e,t)?e[t]:void 0},Qt.prototype.has=function(t){var e=this.__data__;return It?void 0!==e[t]:$.call(e,t)},Qt.prototype.set=function(t,e){return this.__data__[t]=It&&void 0===e?i:e,this},wt.prototype.clear=function(){this.__data__=[]},wt.prototype.delete=function(t){var e=this.__data__,n=kt(e,t);return!(n<0||(n==e.length-1?e.pop():At.call(e,n,1),0))},wt.prototype.get=function(t){var e=this.__data__,n=kt(e,t);return n<0?void 0:e[n][1]},wt.prototype.has=function(t){return kt(this.__data__,t)>-1},wt.prototype.set=function(t,e){var n=this.__data__,r=kt(n,t);return r<0?n.push([t,e]):n[r][1]=e,this},xt.prototype.clear=function(){this.__data__={hash:new Qt,map:new(ht||wt),string:new Qt}},xt.prototype.delete=function(t){return Ft(this,t).delete(t)},xt.prototype.get=function(t){return Ft(this,t).get(t)},xt.prototype.has=function(t){return Ft(this,t).has(t)},xt.prototype.set=function(t,e){return Ft(this,t).set(t,e),this},_t.prototype.clear=function(){this.__data__=new wt},_t.prototype.delete=function(t){return this.__data__.delete(t)},_t.prototype.get=function(t){return this.__data__.get(t)},_t.prototype.has=function(t){return this.__data__.has(t)},_t.prototype.set=function(t,e){var n=this.__data__;if(n instanceof wt){var i=n.__data__;if(!ht||i.length<r-1)return i.push([t,e]),this;n=this.__data__=new xt(i)}return n.set(t,e),this};var Ut=lt?P(lt,Object):function(){return[]},Ot=function(t){return tt.call(t)};function Gt(t,e){return!!(e=null==e?o:e)&&("number"==typeof t||k.test(t))&&t>-1&&t%1==0&&t<e}function Ht(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||V)}function jt(t){if(null!=t){try{return Z.call(t)}catch(t){}try{return t+""}catch(t){}}return""}function Yt(t,e){return t===e||t!=t&&e!=e}(gt&&Ot(new gt(new ArrayBuffer(1)))!=m||ht&&Ot(new ht)!=u||dt&&"[object Promise]"!=Ot(dt.resolve())||ft&&Ot(new ft)!=f||pt&&"[object WeakMap]"!=Ot(new pt))&&(Ot=function(t){var e=tt.call(t),n=e==h?t.constructor:void 0,r=n?jt(n):void 0;if(r)switch(r){case Ct:return m;case mt:return u;case yt:return"[object Promise]";case Et:return f;case vt:return"[object WeakMap]"}return e});var Pt=Array.isArray;function Kt(t){return null!=t&&function(t){return"number"==typeof t&&t>-1&&t%1==0&&t<=o}(t.length)&&!Jt(t)}var zt=ct||function(){return!1};function Jt(t){var e=qt(t)?tt.call(t):"";return e==l||e==c}function qt(t){var e=typeof t;return!!t&&("object"==e||"function"==e)}function Vt(t){return Kt(t)?Dt(t):function(t){if(!Ht(t))return ut(t);var e=[];for(var n in Object(t))$.call(t,n)&&"constructor"!=n&&e.push(n);return e}(t)}n.exports=function(t){return St(t,!0,!0)}}).call(this,n(37),n(98)(t))},function(t,e,n){(function(t,n){var r=200,i="__lodash_hash_undefined__",o=800,a=16,s=9007199254740991,A="[object Arguments]",l="[object AsyncFunction]",c="[object Function]",u="[object GeneratorFunction]",g="[object Null]",h="[object Object]",d="[object Proxy]",f="[object Undefined]",p=/^\[object .+?Constructor\]$/,I=/^(?:0|[1-9]\d*)$/,C={};C["[object Float32Array]"]=C["[object Float64Array]"]=C["[object Int8Array]"]=C["[object Int16Array]"]=C["[object Int32Array]"]=C["[object Uint8Array]"]=C["[object Uint8ClampedArray]"]=C["[object Uint16Array]"]=C["[object Uint32Array]"]=!0,C[A]=C["[object Array]"]=C["[object ArrayBuffer]"]=C["[object Boolean]"]=C["[object DataView]"]=C["[object Date]"]=C["[object Error]"]=C[c]=C["[object Map]"]=C["[object Number]"]=C[h]=C["[object RegExp]"]=C["[object Set]"]=C["[object String]"]=C["[object WeakMap]"]=!1;var m="object"==typeof t&&t&&t.Object===Object&&t,y="object"==typeof self&&self&&self.Object===Object&&self,E=m||y||Function("return this")(),v=e&&!e.nodeType&&e,B=v&&"object"==typeof n&&n&&!n.nodeType&&n,b=B&&B.exports===v,Q=b&&m.process,w=function(){try{return Q&&Q.binding&&Q.binding("util")}catch(t){}}(),x=w&&w.isTypedArray;function _(t,e,n){switch(n.length){case 0:return t.call(e);case 1:return t.call(e,n[0]);case 2:return t.call(e,n[0],n[1]);case 3:return t.call(e,n[0],n[1],n[2])}return t.apply(e,n)}function D(t,e){return"__proto__"==e?void 0:t[e]}var M,k,S,L=Array.prototype,R=Function.prototype,N=Object.prototype,F=E["__core-js_shared__"],T=R.toString,U=N.hasOwnProperty,O=(M=/[^.]+$/.exec(F&&F.keys&&F.keys.IE_PROTO||""))?"Symbol(src)_1."+M:"",G=N.toString,H=T.call(Object),j=RegExp("^"+T.call(U).replace(/[\\^$.*+?()[\]{}|]/g,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),Y=b?E.Buffer:void 0,P=E.Symbol,K=E.Uint8Array,z=Y?Y.allocUnsafe:void 0,J=(k=Object.getPrototypeOf,S=Object,function(t){return k(S(t))}),q=Object.create,V=N.propertyIsEnumerable,W=L.splice,X=P?P.toStringTag:void 0,Z=function(){try{var t=Bt(Object,"defineProperty");return t({},"",{}),t}catch(t){}}(),$=Y?Y.isBuffer:void 0,tt=Math.max,et=Date.now,nt=Bt(E,"Map"),rt=Bt(Object,"create"),it=function(){function t(){}return function(e){if(!Rt(e))return{};if(q)return q(e);t.prototype=e;var n=new t;return t.prototype=void 0,n}}();function ot(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function at(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function st(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function At(t){var e=this.__data__=new at(t);this.size=e.size}function lt(t,e){var n=Dt(t),r=!n&&_t(t),i=!n&&!r&&kt(t),o=!n&&!r&&!i&&Ft(t),a=n||r||i||o,s=a?function(t,e){for(var n=-1,r=Array(t);++n<t;)r[n]=e(n);return r}(t.length,String):[],A=s.length;for(var l in t)!e&&!U.call(t,l)||a&&("length"==l||i&&("offset"==l||"parent"==l)||o&&("buffer"==l||"byteLength"==l||"byteOffset"==l)||bt(l,A))||s.push(l);return s}function ct(t,e,n){(void 0===n||xt(t[e],n))&&(void 0!==n||e in t)||ht(t,e,n)}function ut(t,e,n){var r=t[e];U.call(t,e)&&xt(r,n)&&(void 0!==n||e in t)||ht(t,e,n)}function gt(t,e){for(var n=t.length;n--;)if(xt(t[n][0],e))return n;return-1}function ht(t,e,n){"__proto__"==e&&Z?Z(t,e,{configurable:!0,enumerable:!0,value:n,writable:!0}):t[e]=n}ot.prototype.clear=function(){this.__data__=rt?rt(null):{},this.size=0},ot.prototype.delete=function(t){var e=this.has(t)&&delete this.__data__[t];return this.size-=e?1:0,e},ot.prototype.get=function(t){var e=this.__data__;if(rt){var n=e[t];return n===i?void 0:n}return U.call(e,t)?e[t]:void 0},ot.prototype.has=function(t){var e=this.__data__;return rt?void 0!==e[t]:U.call(e,t)},ot.prototype.set=function(t,e){var n=this.__data__;return this.size+=this.has(t)?0:1,n[t]=rt&&void 0===e?i:e,this},at.prototype.clear=function(){this.__data__=[],this.size=0},at.prototype.delete=function(t){var e=this.__data__,n=gt(e,t);return!(n<0||(n==e.length-1?e.pop():W.call(e,n,1),--this.size,0))},at.prototype.get=function(t){var e=this.__data__,n=gt(e,t);return n<0?void 0:e[n][1]},at.prototype.has=function(t){return gt(this.__data__,t)>-1},at.prototype.set=function(t,e){var n=this.__data__,r=gt(n,t);return r<0?(++this.size,n.push([t,e])):n[r][1]=e,this},st.prototype.clear=function(){this.size=0,this.__data__={hash:new ot,map:new(nt||at),string:new ot}},st.prototype.delete=function(t){var e=vt(this,t).delete(t);return this.size-=e?1:0,e},st.prototype.get=function(t){return vt(this,t).get(t)},st.prototype.has=function(t){return vt(this,t).has(t)},st.prototype.set=function(t,e){var n=vt(this,t),r=n.size;return n.set(t,e),this.size+=n.size==r?0:1,this},At.prototype.clear=function(){this.__data__=new at,this.size=0},At.prototype.delete=function(t){var e=this.__data__,n=e.delete(t);return this.size=e.size,n},At.prototype.get=function(t){return this.__data__.get(t)},At.prototype.has=function(t){return this.__data__.has(t)},At.prototype.set=function(t,e){var n=this.__data__;if(n instanceof at){var i=n.__data__;if(!nt||i.length<r-1)return i.push([t,e]),this.size=++n.size,this;n=this.__data__=new st(i)}return n.set(t,e),this.size=n.size,this};var dt,ft=function(t,e,n){for(var r=-1,i=Object(t),o=n(t),a=o.length;a--;){var s=o[dt?a:++r];if(!1===e(i[s],s,i))break}return t};function pt(t){return null==t?void 0===t?f:g:X&&X in Object(t)?function(t){var e=U.call(t,X),n=t[X];try{t[X]=void 0;var r=!0}catch(t){}var i=G.call(t);r&&(e?t[X]=n:delete t[X]);return i}(t):function(t){return G.call(t)}(t)}function It(t){return Nt(t)&&pt(t)==A}function Ct(t){return!(!Rt(t)||function(t){return!!O&&O in t}(t))&&(St(t)?j:p).test(function(t){if(null!=t){try{return T.call(t)}catch(t){}try{return t+""}catch(t){}}return""}(t))}function mt(t){if(!Rt(t))return function(t){var e=[];if(null!=t)for(var n in Object(t))e.push(n);return e}(t);var e=Qt(t),n=[];for(var r in t)("constructor"!=r||!e&&U.call(t,r))&&n.push(r);return n}function yt(t,e,n,r,i){t!==e&&ft(e,function(o,a){if(Rt(o))i||(i=new At),function(t,e,n,r,i,o,a){var s=D(t,n),A=D(e,n),l=a.get(A);if(l)return void ct(t,n,l);var c=o?o(s,A,n+"",t,e,a):void 0,u=void 0===c;if(u){var g=Dt(A),d=!g&&kt(A),f=!g&&!d&&Ft(A);c=A,g||d||f?Dt(s)?c=s:Nt(C=s)&&Mt(C)?c=function(t,e){var n=-1,r=t.length;e||(e=Array(r));for(;++n<r;)e[n]=t[n];return e}(s):d?(u=!1,c=function(t,e){if(e)return t.slice();var n=t.length,r=z?z(n):new t.constructor(n);return t.copy(r),r}(A,!0)):f?(u=!1,p=A,I=!0?function(t){var e=new t.constructor(t.byteLength);return new K(e).set(new K(t)),e}(p.buffer):p.buffer,c=new p.constructor(I,p.byteOffset,p.length)):c=[]:function(t){if(!Nt(t)||pt(t)!=h)return!1;var e=J(t);if(null===e)return!0;var n=U.call(e,"constructor")&&e.constructor;return"function"==typeof n&&n instanceof n&&T.call(n)==H}(A)||_t(A)?(c=s,_t(s)?c=function(t){return function(t,e,n,r){var i=!n;n||(n={});var o=-1,a=e.length;for(;++o<a;){var s=e[o],A=r?r(n[s],t[s],s,n,t):void 0;void 0===A&&(A=t[s]),i?ht(n,s,A):ut(n,s,A)}return n}(t,Tt(t))}(s):(!Rt(s)||r&&St(s))&&(c=function(t){return"function"!=typeof t.constructor||Qt(t)?{}:it(J(t))}(A))):u=!1}var p,I;var C;u&&(a.set(A,c),i(c,A,r,o,a),a.delete(A));ct(t,n,c)}(t,e,a,n,yt,r,i);else{var s=r?r(D(t,a),o,a+"",t,e,i):void 0;void 0===s&&(s=o),ct(t,a,s)}},Tt)}function Et(t,e){return wt(function(t,e,n){return e=tt(void 0===e?t.length-1:e,0),function(){for(var r=arguments,i=-1,o=tt(r.length-e,0),a=Array(o);++i<o;)a[i]=r[e+i];i=-1;for(var s=Array(e+1);++i<e;)s[i]=r[i];return s[e]=n(a),_(t,this,s)}}(t,e,Gt),t+"")}function vt(t,e){var n,r,i=t.__data__;return("string"==(r=typeof(n=e))||"number"==r||"symbol"==r||"boolean"==r?"__proto__"!==n:null===n)?i["string"==typeof e?"string":"hash"]:i.map}function Bt(t,e){var n=function(t,e){return null==t?void 0:t[e]}(t,e);return Ct(n)?n:void 0}function bt(t,e){var n=typeof t;return!!(e=null==e?s:e)&&("number"==n||"symbol"!=n&&I.test(t))&&t>-1&&t%1==0&&t<e}function Qt(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||N)}var wt=function(t){var e=0,n=0;return function(){var r=et(),i=a-(r-n);if(n=r,i>0){if(++e>=o)return arguments[0]}else e=0;return t.apply(void 0,arguments)}}(Z?function(t,e){return Z(t,"toString",{configurable:!0,enumerable:!1,value:(n=e,function(){return n}),writable:!0});var n}:Gt);function xt(t,e){return t===e||t!=t&&e!=e}var _t=It(function(){return arguments}())?It:function(t){return Nt(t)&&U.call(t,"callee")&&!V.call(t,"callee")},Dt=Array.isArray;function Mt(t){return null!=t&&Lt(t.length)&&!St(t)}var kt=$||function(){return!1};function St(t){if(!Rt(t))return!1;var e=pt(t);return e==c||e==u||e==l||e==d}function Lt(t){return"number"==typeof t&&t>-1&&t%1==0&&t<=s}function Rt(t){var e=typeof t;return null!=t&&("object"==e||"function"==e)}function Nt(t){return null!=t&&"object"==typeof t}var Ft=x?function(t){return function(e){return t(e)}}(x):function(t){return Nt(t)&&Lt(t.length)&&!!C[pt(t)]};function Tt(t){return Mt(t)?lt(t,!0):mt(t)}var Ut,Ot=(Ut=function(t,e,n,r){yt(t,e,n,r)},Et(function(t,e){var n=-1,r=e.length,i=r>1?e[r-1]:void 0,o=r>2?e[2]:void 0;for(i=Ut.length>3&&"function"==typeof i?(r--,i):void 0,o&&function(t,e,n){if(!Rt(n))return!1;var r=typeof e;return!!("number"==r?Mt(n)&&bt(e,n.length):"string"==r&&e in n)&&xt(n[e],t)}(e[0],e[1],o)&&(i=r<3?void 0:i,r=1),t=Object(t);++n<r;){var a=e[n];a&&Ut(t,a,n,i)}return t}));function Gt(t){return t}n.exports=Ot}).call(this,n(37),n(98)(t))},function(t,e){var n="[object String]",r=Object.prototype.toString,i=Array.isArray;t.exports=function(t){return"string"==typeof t||!i(t)&&function(t){return!!t&&"object"==typeof t}(t)&&r.call(t)==n}},function(t,e){var n="[object Object]";var r,i,o=Function.prototype,a=Object.prototype,s=o.toString,A=a.hasOwnProperty,l=s.call(Object),c=a.toString,u=(r=Object.getPrototypeOf,i=Object,function(t){return r(i(t))});t.exports=function(t){if(!function(t){return!!t&&"object"==typeof t}(t)||c.call(t)!=n||function(t){var e=!1;if(null!=t&&"function"!=typeof t.toString)try{e=!!(t+"")}catch(t){}return e}(t))return!1;var e=u(t);if(null===e)return!0;var r=A.call(e,"constructor")&&e.constructor;return"function"==typeof r&&r instanceof r&&s.call(r)==l}},function(t,e,n){"use strict";var r=n(1109),i=n(1110),o=/^\d+$/;e.parse=function(t){return(e=t.split(",").map(function(t){var e={};return t.trim().split(/\s+/).forEach(function(t,n){if(0===n)return e.url=t;var i=t.substring(0,t.length-1),a=t[t.length-1],s=parseInt(i,10),A=parseFloat(i);if("w"===a&&o.test(i))e.width=s;else if("h"===a&&o.test(i))e.height=s;else{if("x"!==a||r(A))throw new Error("Invalid srcset descriptor: "+t+".");e.density=A}}),e})).sort().filter(function(t,n){return JSON.stringify(t)!==JSON.stringify(e[n-1])});var e},e.stringify=function(t){return i(t.map(function(t){if(!t.url)throw new Error("URL is required.");var e=[t.url];return t.width&&e.push(t.width+"w"),t.height&&e.push(t.height+"h"),t.density&&e.push(t.density+"x"),e.join(" ")})).join(", ")}},function(t,e,n){"use strict";t.exports=Number.isNaN||function(t){return t!=t}},function(t,e,n){"use strict";(function(e){var n;"Set"in e?"function"==typeof Set.prototype.forEach&&(n=!1,new Set([!0]).forEach(function(t){n=t}),!0===n)?t.exports=function(t){var e=[];return new Set(t).forEach(function(t){e.push(t)}),e}:t.exports=function(t){var e=new Set;return t.filter(function(t){return!e.has(t)&&(e.add(t),!0)})}:t.exports=function(t){for(var e=[],n=0;n<t.length;n++)-1===e.indexOf(t[n])&&e.push(t[n]);return e}}).call(this,n(37))},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=h(n(299)),i=h(n(603)),o=h(n(301)),a=h(n(303)),s=h(n(304)),A=h(n(1130)),l=h(n(302)),c=h(n(609)),u=h(n(306)),g=h(n(610));function h(t){return t&&t.__esModule?t:{default:t}}function d(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];return 1===e.length&&Array.isArray(e[0])&&(e=e[0]),new i.default(e)}d.plugin=function(t,e){function n(){var n=e.apply(void 0,arguments);return n.postcssPlugin=t,n.postcssVersion=(new i.default).version,n}var r;return Object.defineProperty(n,"postcss",{get:function(){return r||(r=n()),r}}),n.process=function(t,e,r){return d([n(r)]).process(t,e)},n},d.stringify=o.default,d.parse=l.default,d.vendor=A.default,d.list=c.default,d.comment=function(t){return new a.default(t)},d.atRule=function(t){return new s.default(t)},d.decl=function(t){return new r.default(t)},d.rule=function(t){return new u.default(t)},d.root=function(t){return new g.default(t)};var f=d;e.default=f,t.exports=e.default},function(t,e){},function(t,e){},function(t,e){},function(t,e,n){"use strict";(function(r){e.__esModule=!0,e.default=void 0;var i=a(n(605)),o=a(n(171));function a(t){return t&&t.__esModule?t:{default:t}}var s=function(){function t(t,e,n){this.stringify=t,this.mapOpts=n.map||{},this.root=e,this.opts=n}var e=t.prototype;return e.isMap=function(){return void 0!==this.opts.map?!!this.opts.map:this.previous().length>0},e.previous=function(){var t=this;return this.previousMaps||(this.previousMaps=[],this.root.walk(function(e){if(e.source&&e.source.input.map){var n=e.source.input.map;-1===t.previousMaps.indexOf(n)&&t.previousMaps.push(n)}})),this.previousMaps},e.isInline=function(){if(void 0!==this.mapOpts.inline)return this.mapOpts.inline;var t=this.mapOpts.annotation;return(void 0===t||!0===t)&&(!this.previous().length||this.previous().some(function(t){return t.inline}))},e.isSourcesContent=function(){return void 0!==this.mapOpts.sourcesContent?this.mapOpts.sourcesContent:!this.previous().length||this.previous().some(function(t){return t.withContent()})},e.clearAnnotation=function(){if(!1!==this.mapOpts.annotation)for(var t,e=this.root.nodes.length-1;e>=0;e--)"comment"===(t=this.root.nodes[e]).type&&0===t.text.indexOf("# sourceMappingURL=")&&this.root.removeChild(e)},e.setSourcesContent=function(){var t=this,e={};this.root.walk(function(n){if(n.source){var r=n.source.input.from;if(r&&!e[r]){e[r]=!0;var i=t.relative(r);t.map.setSourceContent(i,n.source.input.css)}}})},e.applyPrevMaps=function(){var t=this.previous(),e=Array.isArray(t),n=0;for(t=e?t:t[Symbol.iterator]();;){var r;if(e){if(n>=t.length)break;r=t[n++]}else{if((n=t.next()).done)break;r=n.value}var a=r,s=this.relative(a.file),A=a.root||o.default.dirname(a.file),l=void 0;!1===this.mapOpts.sourcesContent?(l=new i.default.SourceMapConsumer(a.text)).sourcesContent&&(l.sourcesContent=l.sourcesContent.map(function(){return null})):l=a.consumer(),this.map.applySourceMap(l,s,this.relative(A))}},e.isAnnotation=function(){return!!this.isInline()||(void 0!==this.mapOpts.annotation?this.mapOpts.annotation:!this.previous().length||this.previous().some(function(t){return t.annotation}))},e.toBase64=function(t){return r?r.from(t).toString("base64"):window.btoa(unescape(encodeURIComponent(t)))},e.addAnnotation=function(){var t;t=this.isInline()?"data:application/json;base64,"+this.toBase64(this.map.toString()):"string"==typeof this.mapOpts.annotation?this.mapOpts.annotation:this.outputFile()+".map";var e="\n";-1!==this.css.indexOf("\r\n")&&(e="\r\n"),this.css+=e+"/*# sourceMappingURL="+t+" */"},e.outputFile=function(){return this.opts.to?this.relative(this.opts.to):this.opts.from?this.relative(this.opts.from):"to.css"},e.generateMap=function(){return this.generateString(),this.isSourcesContent()&&this.setSourcesContent(),this.previous().length>0&&this.applyPrevMaps(),this.isAnnotation()&&this.addAnnotation(),this.isInline()?[this.css]:[this.css,this.map]},e.relative=function(t){if(0===t.indexOf("<"))return t;if(/^\w+:\/\//.test(t))return t;var e=this.opts.to?o.default.dirname(this.opts.to):".";return"string"==typeof this.mapOpts.annotation&&(e=o.default.dirname(o.default.resolve(e,this.mapOpts.annotation))),t=o.default.relative(e,t),"\\"===o.default.sep?t.replace(/\\/g,"/"):t},e.sourcePath=function(t){return this.mapOpts.from?this.mapOpts.from:this.relative(t.source.input.from)},e.generateString=function(){var t=this;this.css="",this.map=new i.default.SourceMapGenerator({file:this.outputFile()});var e,n,r=1,o=1;this.stringify(this.root,function(i,a,s){if(t.css+=i,a&&"end"!==s&&(a.source&&a.source.start?t.map.addMapping({source:t.sourcePath(a),generated:{line:r,column:o-1},original:{line:a.source.start.line,column:a.source.start.column-1}}):t.map.addMapping({source:"<no source>",original:{line:1,column:0},generated:{line:r,column:o-1}})),(e=i.match(/\n/g))?(r+=e.length,n=i.lastIndexOf("\n"),o=i.length-n):o+=i.length,a&&"start"!==s){var A=a.parent||{raws:{}};("decl"!==a.type||a!==A.last||A.raws.semicolon)&&(a.source&&a.source.end?t.map.addMapping({source:t.sourcePath(a),generated:{line:r,column:o-2},original:{line:a.source.end.line,column:a.source.end.column-1}}):t.map.addMapping({source:"<no source>",original:{line:1,column:0},generated:{line:r,column:o-1}}))}})},e.generate=function(){if(this.clearAnnotation(),this.isMap())return this.generateMap();var t="";return this.stringify(this.root,function(e){t+=e}),[t]},t}();e.default=s,t.exports=e.default}).call(this,n(42).Buffer)},function(t,e){var n="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/".split("");e.encode=function(t){if(0<=t&&t<n.length)return n[t];throw new TypeError("Must be between 0 and 63: "+t)},e.decode=function(t){return 65<=t&&t<=90?t-65:97<=t&&t<=122?t-97+26:48<=t&&t<=57?t-48+52:43==t?62:47==t?63:-1}},function(t,e,n){var r=n(176);function i(){this._array=[],this._sorted=!0,this._last={generatedLine:-1,generatedColumn:0}}i.prototype.unsortedForEach=function(t,e){this._array.forEach(t,e)},i.prototype.add=function(t){var e,n,i,o,a,s;e=this._last,n=t,i=e.generatedLine,o=n.generatedLine,a=e.generatedColumn,s=n.generatedColumn,o>i||o==i&&s>=a||r.compareByGeneratedPositionsInflated(e,n)<=0?(this._last=t,this._array.push(t)):(this._sorted=!1,this._array.push(t))},i.prototype.toArray=function(){return this._sorted||(this._array.sort(r.compareByGeneratedPositionsInflated),this._sorted=!0),this._array},e.MappingList=i},function(t,e,n){var r=n(176),i=n(1119),o=n(608).ArraySet,a=n(607),s=n(1120).quickSort;function A(t,e){var n=t;return"string"==typeof t&&(n=r.parseSourceMapInput(t)),null!=n.sections?new u(n,e):new l(n,e)}function l(t,e){var n=t;"string"==typeof t&&(n=r.parseSourceMapInput(t));var i=r.getArg(n,"version"),a=r.getArg(n,"sources"),s=r.getArg(n,"names",[]),A=r.getArg(n,"sourceRoot",null),l=r.getArg(n,"sourcesContent",null),c=r.getArg(n,"mappings"),u=r.getArg(n,"file",null);if(i!=this._version)throw new Error("Unsupported version: "+i);A&&(A=r.normalize(A)),a=a.map(String).map(r.normalize).map(function(t){return A&&r.isAbsolute(A)&&r.isAbsolute(t)?r.relative(A,t):t}),this._names=o.fromArray(s.map(String),!0),this._sources=o.fromArray(a,!0),this._absoluteSources=this._sources.toArray().map(function(t){return r.computeSourceURL(A,t,e)}),this.sourceRoot=A,this.sourcesContent=l,this._mappings=c,this._sourceMapURL=e,this.file=u}function c(){this.generatedLine=0,this.generatedColumn=0,this.source=null,this.originalLine=null,this.originalColumn=null,this.name=null}function u(t,e){var n=t;"string"==typeof t&&(n=r.parseSourceMapInput(t));var i=r.getArg(n,"version"),a=r.getArg(n,"sections");if(i!=this._version)throw new Error("Unsupported version: "+i);this._sources=new o,this._names=new o;var s={line:-1,column:0};this._sections=a.map(function(t){if(t.url)throw new Error("Support for url field in sections not implemented.");var n=r.getArg(t,"offset"),i=r.getArg(n,"line"),o=r.getArg(n,"column");if(i<s.line||i===s.line&&o<s.column)throw new Error("Section offsets must be ordered and non-overlapping.");return s=n,{generatedOffset:{generatedLine:i+1,generatedColumn:o+1},consumer:new A(r.getArg(t,"map"),e)}})}A.fromSourceMap=function(t,e){return l.fromSourceMap(t,e)},A.prototype._version=3,A.prototype.__generatedMappings=null,Object.defineProperty(A.prototype,"_generatedMappings",{configurable:!0,enumerable:!0,get:function(){return this.__generatedMappings||this._parseMappings(this._mappings,this.sourceRoot),this.__generatedMappings}}),A.prototype.__originalMappings=null,Object.defineProperty(A.prototype,"_originalMappings",{configurable:!0,enumerable:!0,get:function(){return this.__originalMappings||this._parseMappings(this._mappings,this.sourceRoot),this.__originalMappings}}),A.prototype._charIsMappingSeparator=function(t,e){var n=t.charAt(e);return";"===n||","===n},A.prototype._parseMappings=function(t,e){throw new Error("Subclasses must implement _parseMappings")},A.GENERATED_ORDER=1,A.ORIGINAL_ORDER=2,A.GREATEST_LOWER_BOUND=1,A.LEAST_UPPER_BOUND=2,A.prototype.eachMapping=function(t,e,n){var i,o=e||null;switch(n||A.GENERATED_ORDER){case A.GENERATED_ORDER:i=this._generatedMappings;break;case A.ORIGINAL_ORDER:i=this._originalMappings;break;default:throw new Error("Unknown order of iteration.")}var a=this.sourceRoot;i.map(function(t){var e=null===t.source?null:this._sources.at(t.source);return{source:e=r.computeSourceURL(a,e,this._sourceMapURL),generatedLine:t.generatedLine,generatedColumn:t.generatedColumn,originalLine:t.originalLine,originalColumn:t.originalColumn,name:null===t.name?null:this._names.at(t.name)}},this).forEach(t,o)},A.prototype.allGeneratedPositionsFor=function(t){var e=r.getArg(t,"line"),n={source:r.getArg(t,"source"),originalLine:e,originalColumn:r.getArg(t,"column",0)};if(n.source=this._findSourceIndex(n.source),n.source<0)return[];var o=[],a=this._findMapping(n,this._originalMappings,"originalLine","originalColumn",r.compareByOriginalPositions,i.LEAST_UPPER_BOUND);if(a>=0){var s=this._originalMappings[a];if(void 0===t.column)for(var A=s.originalLine;s&&s.originalLine===A;)o.push({line:r.getArg(s,"generatedLine",null),column:r.getArg(s,"generatedColumn",null),lastColumn:r.getArg(s,"lastGeneratedColumn",null)}),s=this._originalMappings[++a];else for(var l=s.originalColumn;s&&s.originalLine===e&&s.originalColumn==l;)o.push({line:r.getArg(s,"generatedLine",null),column:r.getArg(s,"generatedColumn",null),lastColumn:r.getArg(s,"lastGeneratedColumn",null)}),s=this._originalMappings[++a]}return o},e.SourceMapConsumer=A,l.prototype=Object.create(A.prototype),l.prototype.consumer=A,l.prototype._findSourceIndex=function(t){var e,n=t;if(null!=this.sourceRoot&&(n=r.relative(this.sourceRoot,n)),this._sources.has(n))return this._sources.indexOf(n);for(e=0;e<this._absoluteSources.length;++e)if(this._absoluteSources[e]==t)return e;return-1},l.fromSourceMap=function(t,e){var n=Object.create(l.prototype),i=n._names=o.fromArray(t._names.toArray(),!0),a=n._sources=o.fromArray(t._sources.toArray(),!0);n.sourceRoot=t._sourceRoot,n.sourcesContent=t._generateSourcesContent(n._sources.toArray(),n.sourceRoot),n.file=t._file,n._sourceMapURL=e,n._absoluteSources=n._sources.toArray().map(function(t){return r.computeSourceURL(n.sourceRoot,t,e)});for(var A=t._mappings.toArray().slice(),u=n.__generatedMappings=[],g=n.__originalMappings=[],h=0,d=A.length;h<d;h++){var f=A[h],p=new c;p.generatedLine=f.generatedLine,p.generatedColumn=f.generatedColumn,f.source&&(p.source=a.indexOf(f.source),p.originalLine=f.originalLine,p.originalColumn=f.originalColumn,f.name&&(p.name=i.indexOf(f.name)),g.push(p)),u.push(p)}return s(n.__originalMappings,r.compareByOriginalPositions),n},l.prototype._version=3,Object.defineProperty(l.prototype,"sources",{get:function(){return this._absoluteSources.slice()}}),l.prototype._parseMappings=function(t,e){for(var n,i,o,A,l,u=1,g=0,h=0,d=0,f=0,p=0,I=t.length,C=0,m={},y={},E=[],v=[];C<I;)if(";"===t.charAt(C))u++,C++,g=0;else if(","===t.charAt(C))C++;else{for((n=new c).generatedLine=u,A=C;A<I&&!this._charIsMappingSeparator(t,A);A++);if(o=m[i=t.slice(C,A)])C+=i.length;else{for(o=[];C<A;)a.decode(t,C,y),l=y.value,C=y.rest,o.push(l);if(2===o.length)throw new Error("Found a source, but no line and column");if(3===o.length)throw new Error("Found a source and line, but no column");m[i]=o}n.generatedColumn=g+o[0],g=n.generatedColumn,o.length>1&&(n.source=f+o[1],f+=o[1],n.originalLine=h+o[2],h=n.originalLine,n.originalLine+=1,n.originalColumn=d+o[3],d=n.originalColumn,o.length>4&&(n.name=p+o[4],p+=o[4])),v.push(n),"number"==typeof n.originalLine&&E.push(n)}s(v,r.compareByGeneratedPositionsDeflated),this.__generatedMappings=v,s(E,r.compareByOriginalPositions),this.__originalMappings=E},l.prototype._findMapping=function(t,e,n,r,o,a){if(t[n]<=0)throw new TypeError("Line must be greater than or equal to 1, got "+t[n]);if(t[r]<0)throw new TypeError("Column must be greater than or equal to 0, got "+t[r]);return i.search(t,e,o,a)},l.prototype.computeColumnSpans=function(){for(var t=0;t<this._generatedMappings.length;++t){var e=this._generatedMappings[t];if(t+1<this._generatedMappings.length){var n=this._generatedMappings[t+1];if(e.generatedLine===n.generatedLine){e.lastGeneratedColumn=n.generatedColumn-1;continue}}e.lastGeneratedColumn=1/0}},l.prototype.originalPositionFor=function(t){var e={generatedLine:r.getArg(t,"line"),generatedColumn:r.getArg(t,"column")},n=this._findMapping(e,this._generatedMappings,"generatedLine","generatedColumn",r.compareByGeneratedPositionsDeflated,r.getArg(t,"bias",A.GREATEST_LOWER_BOUND));if(n>=0){var i=this._generatedMappings[n];if(i.generatedLine===e.generatedLine){var o=r.getArg(i,"source",null);null!==o&&(o=this._sources.at(o),o=r.computeSourceURL(this.sourceRoot,o,this._sourceMapURL));var a=r.getArg(i,"name",null);return null!==a&&(a=this._names.at(a)),{source:o,line:r.getArg(i,"originalLine",null),column:r.getArg(i,"originalColumn",null),name:a}}}return{source:null,line:null,column:null,name:null}},l.prototype.hasContentsOfAllSources=function(){return!!this.sourcesContent&&(this.sourcesContent.length>=this._sources.size()&&!this.sourcesContent.some(function(t){return null==t}))},l.prototype.sourceContentFor=function(t,e){if(!this.sourcesContent)return null;var n=this._findSourceIndex(t);if(n>=0)return this.sourcesContent[n];var i,o=t;if(null!=this.sourceRoot&&(o=r.relative(this.sourceRoot,o)),null!=this.sourceRoot&&(i=r.urlParse(this.sourceRoot))){var a=o.replace(/^file:\/\//,"");if("file"==i.scheme&&this._sources.has(a))return this.sourcesContent[this._sources.indexOf(a)];if((!i.path||"/"==i.path)&&this._sources.has("/"+o))return this.sourcesContent[this._sources.indexOf("/"+o)]}if(e)return null;throw new Error('"'+o+'" is not in the SourceMap.')},l.prototype.generatedPositionFor=function(t){var e=r.getArg(t,"source");if((e=this._findSourceIndex(e))<0)return{line:null,column:null,lastColumn:null};var n={source:e,originalLine:r.getArg(t,"line"),originalColumn:r.getArg(t,"column")},i=this._findMapping(n,this._originalMappings,"originalLine","originalColumn",r.compareByOriginalPositions,r.getArg(t,"bias",A.GREATEST_LOWER_BOUND));if(i>=0){var o=this._originalMappings[i];if(o.source===n.source)return{line:r.getArg(o,"generatedLine",null),column:r.getArg(o,"generatedColumn",null),lastColumn:r.getArg(o,"lastGeneratedColumn",null)}}return{line:null,column:null,lastColumn:null}},e.BasicSourceMapConsumer=l,u.prototype=Object.create(A.prototype),u.prototype.constructor=A,u.prototype._version=3,Object.defineProperty(u.prototype,"sources",{get:function(){for(var t=[],e=0;e<this._sections.length;e++)for(var n=0;n<this._sections[e].consumer.sources.length;n++)t.push(this._sections[e].consumer.sources[n]);return t}}),u.prototype.originalPositionFor=function(t){var e={generatedLine:r.getArg(t,"line"),generatedColumn:r.getArg(t,"column")},n=i.search(e,this._sections,function(t,e){var n=t.generatedLine-e.generatedOffset.generatedLine;return n||t.generatedColumn-e.generatedOffset.generatedColumn}),o=this._sections[n];return o?o.consumer.originalPositionFor({line:e.generatedLine-(o.generatedOffset.generatedLine-1),column:e.generatedColumn-(o.generatedOffset.generatedLine===e.generatedLine?o.generatedOffset.generatedColumn-1:0),bias:t.bias}):{source:null,line:null,column:null,name:null}},u.prototype.hasContentsOfAllSources=function(){return this._sections.every(function(t){return t.consumer.hasContentsOfAllSources()})},u.prototype.sourceContentFor=function(t,e){for(var n=0;n<this._sections.length;n++){var r=this._sections[n].consumer.sourceContentFor(t,!0);if(r)return r}if(e)return null;throw new Error('"'+t+'" is not in the SourceMap.')},u.prototype.generatedPositionFor=function(t){for(var e=0;e<this._sections.length;e++){var n=this._sections[e];if(-1!==n.consumer._findSourceIndex(r.getArg(t,"source"))){var i=n.consumer.generatedPositionFor(t);if(i)return{line:i.line+(n.generatedOffset.generatedLine-1),column:i.column+(n.generatedOffset.generatedLine===i.line?n.generatedOffset.generatedColumn-1:0)}}}return{line:null,column:null}},u.prototype._parseMappings=function(t,e){this.__generatedMappings=[],this.__originalMappings=[];for(var n=0;n<this._sections.length;n++)for(var i=this._sections[n],o=i.consumer._generatedMappings,a=0;a<o.length;a++){var A=o[a],l=i.consumer._sources.at(A.source);l=r.computeSourceURL(i.consumer.sourceRoot,l,this._sourceMapURL),this._sources.add(l),l=this._sources.indexOf(l);var c=null;A.name&&(c=i.consumer._names.at(A.name),this._names.add(c),c=this._names.indexOf(c));var u={source:l,generatedLine:A.generatedLine+(i.generatedOffset.generatedLine-1),generatedColumn:A.generatedColumn+(i.generatedOffset.generatedLine===A.generatedLine?i.generatedOffset.generatedColumn-1:0),originalLine:A.originalLine,originalColumn:A.originalColumn,name:c};this.__generatedMappings.push(u),"number"==typeof u.originalLine&&this.__originalMappings.push(u)}s(this.__generatedMappings,r.compareByGeneratedPositionsDeflated),s(this.__originalMappings,r.compareByOriginalPositions)},e.IndexedSourceMapConsumer=u},function(t,e){e.GREATEST_LOWER_BOUND=1,e.LEAST_UPPER_BOUND=2,e.search=function(t,n,r,i){if(0===n.length)return-1;var o=function t(n,r,i,o,a,s){var A=Math.floor((r-n)/2)+n,l=a(i,o[A],!0);return 0===l?A:l>0?r-A>1?t(A,r,i,o,a,s):s==e.LEAST_UPPER_BOUND?r<o.length?r:-1:A:A-n>1?t(n,A,i,o,a,s):s==e.LEAST_UPPER_BOUND?A:n<0?-1:n}(-1,n.length,t,n,r,i||e.GREATEST_LOWER_BOUND);if(o<0)return-1;for(;o-1>=0&&0===r(n[o],n[o-1],!0);)--o;return o}},function(t,e){function n(t,e,n){var r=t[e];t[e]=t[n],t[n]=r}function r(t,e,i,o){if(i<o){var a=i-1;n(t,(c=i,u=o,Math.round(c+Math.random()*(u-c))),o);for(var s=t[o],A=i;A<o;A++)e(t[A],s)<=0&&n(t,a+=1,A);n(t,a+1,A);var l=a+1;r(t,e,i,l-1),r(t,e,l+1,o)}var c,u}e.quickSort=function(t,e){r(t,e,0,t.length-1)}},function(t,e,n){var r=n(606).SourceMapGenerator,i=n(176),o=/(\r?\n)/,a="$$$isSourceNode$$$";function s(t,e,n,r,i){this.children=[],this.sourceContents={},this.line=null==t?null:t,this.column=null==e?null:e,this.source=null==n?null:n,this.name=null==i?null:i,this[a]=!0,null!=r&&this.add(r)}s.fromStringWithSourceMap=function(t,e,n){var r=new s,a=t.split(o),A=0,l=function(){return t()+(t()||"");function t(){return A<a.length?a[A++]:void 0}},c=1,u=0,g=null;return e.eachMapping(function(t){if(null!==g){if(!(c<t.generatedLine)){var e=(n=a[A]||"").substr(0,t.generatedColumn-u);return a[A]=n.substr(t.generatedColumn-u),u=t.generatedColumn,h(g,e),void(g=t)}h(g,l()),c++,u=0}for(;c<t.generatedLine;)r.add(l()),c++;if(u<t.generatedColumn){var n=a[A]||"";r.add(n.substr(0,t.generatedColumn)),a[A]=n.substr(t.generatedColumn),u=t.generatedColumn}g=t},this),A<a.length&&(g&&h(g,l()),r.add(a.splice(A).join(""))),e.sources.forEach(function(t){var o=e.sourceContentFor(t);null!=o&&(null!=n&&(t=i.join(n,t)),r.setSourceContent(t,o))}),r;function h(t,e){if(null===t||void 0===t.source)r.add(e);else{var o=n?i.join(n,t.source):t.source;r.add(new s(t.originalLine,t.originalColumn,o,e,t.name))}}},s.prototype.add=function(t){if(Array.isArray(t))t.forEach(function(t){this.add(t)},this);else{if(!t[a]&&"string"!=typeof t)throw new TypeError("Expected a SourceNode, string, or an array of SourceNodes and strings. Got "+t);t&&this.children.push(t)}return this},s.prototype.prepend=function(t){if(Array.isArray(t))for(var e=t.length-1;e>=0;e--)this.prepend(t[e]);else{if(!t[a]&&"string"!=typeof t)throw new TypeError("Expected a SourceNode, string, or an array of SourceNodes and strings. Got "+t);this.children.unshift(t)}return this},s.prototype.walk=function(t){for(var e,n=0,r=this.children.length;n<r;n++)(e=this.children[n])[a]?e.walk(t):""!==e&&t(e,{source:this.source,line:this.line,column:this.column,name:this.name})},s.prototype.join=function(t){var e,n,r=this.children.length;if(r>0){for(e=[],n=0;n<r-1;n++)e.push(this.children[n]),e.push(t);e.push(this.children[n]),this.children=e}return this},s.prototype.replaceRight=function(t,e){var n=this.children[this.children.length-1];return n[a]?n.replaceRight(t,e):"string"==typeof n?this.children[this.children.length-1]=n.replace(t,e):this.children.push("".replace(t,e)),this},s.prototype.setSourceContent=function(t,e){this.sourceContents[i.toSetString(t)]=e},s.prototype.walkSourceContents=function(t){for(var e=0,n=this.children.length;e<n;e++)this.children[e][a]&&this.children[e].walkSourceContents(t);var r=Object.keys(this.sourceContents);for(e=0,n=r.length;e<n;e++)t(i.fromSetString(r[e]),this.sourceContents[r[e]])},s.prototype.toString=function(){var t="";return this.walk(function(e){t+=e}),t},s.prototype.toStringWithSourceMap=function(t){var e={code:"",line:1,column:0},n=new r(t),i=!1,o=null,a=null,s=null,A=null;return this.walk(function(t,r){e.code+=t,null!==r.source&&null!==r.line&&null!==r.column?(o===r.source&&a===r.line&&s===r.column&&A===r.name||n.addMapping({source:r.source,original:{line:r.line,column:r.column},generated:{line:e.line,column:e.column},name:r.name}),o=r.source,a=r.line,s=r.column,A=r.name,i=!0):i&&(n.addMapping({generated:{line:e.line,column:e.column}}),o=null,i=!1);for(var l=0,c=t.length;l<c;l++)10===t.charCodeAt(l)?(e.line++,e.column=0,l+1===c?(o=null,i=!1):i&&n.addMapping({source:r.source,original:{line:r.line,column:r.column},generated:{line:e.line,column:e.column},name:r.name})):e.column++}),this.walkSourceContents(function(t,e){n.setSourceContent(t,e)}),{code:e.code,map:n}},e.SourceNode=s},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t){if(r[t])return;r[t]=!0,"undefined"!=typeof console&&console.warn&&console.warn(t)};var r={};t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r,i=(r=n(1124))&&r.__esModule?r:{default:r};function o(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}var a=function(){function t(t,e,n){this.processor=t,this.messages=[],this.root=e,this.opts=n,this.css=void 0,this.map=void 0}var e,n,r,a=t.prototype;return a.toString=function(){return this.css},a.warn=function(t,e){void 0===e&&(e={}),e.plugin||this.lastPlugin&&this.lastPlugin.postcssPlugin&&(e.plugin=this.lastPlugin.postcssPlugin);var n=new i.default(t,e);return this.messages.push(n),n},a.warnings=function(){return this.messages.filter(function(t){return"warning"===t.type})},e=t,(n=[{key:"content",get:function(){return this.css}}])&&o(e.prototype,n),r&&o(e,r),t}();e.default=a,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=function(){function t(t,e){if(void 0===e&&(e={}),this.type="warning",this.text=t,e.node&&e.node.source){var n=e.node.positionBy(e);this.line=n.line,this.column=n.column}for(var r in e)this[r]=e[r]}return t.prototype.toString=function(){return this.node?this.node.error(this.text,{plugin:this.plugin,index:this.index,word:this.word}).message:this.plugin?this.plugin+": "+this.text:this.text},t}();e.default=r,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=l(n(299)),i=l(n(1126)),o=l(n(303)),a=l(n(304)),s=l(n(610)),A=l(n(306));function l(t){return t&&t.__esModule?t:{default:t}}var c=function(){function t(t){this.input=t,this.root=new s.default,this.current=this.root,this.spaces="",this.semicolon=!1,this.createTokenizer(),this.root.source={input:t,start:{line:1,column:1}}}var e=t.prototype;return e.createTokenizer=function(){this.tokenizer=(0,i.default)(this.input)},e.parse=function(){for(var t;!this.tokenizer.endOfFile();)switch((t=this.tokenizer.nextToken())[0]){case"space":this.spaces+=t[1];break;case";":this.freeSemicolon(t);break;case"}":this.end(t);break;case"comment":this.comment(t);break;case"at-word":this.atrule(t);break;case"{":this.emptyRule(t);break;default:this.other(t)}this.endFile()},e.comment=function(t){var e=new o.default;this.init(e,t[2],t[3]),e.source.end={line:t[4],column:t[5]};var n=t[1].slice(2,-2);if(/^\s*$/.test(n))e.text="",e.raws.left=n,e.raws.right="";else{var r=n.match(/^(\s*)([^]*[^\s])(\s*)$/);e.text=r[2],e.raws.left=r[1],e.raws.right=r[3]}},e.emptyRule=function(t){var e=new A.default;this.init(e,t[2],t[3]),e.selector="",e.raws.between="",this.current=e},e.other=function(t){for(var e=!1,n=null,r=!1,i=null,o=[],a=[],s=t;s;){if(n=s[0],a.push(s),"("===n||"["===n)i||(i=s),o.push("("===n?")":"]");else if(0===o.length){if(";"===n){if(r)return void this.decl(a);break}if("{"===n)return void this.rule(a);if("}"===n){this.tokenizer.back(a.pop()),e=!0;break}":"===n&&(r=!0)}else n===o[o.length-1]&&(o.pop(),0===o.length&&(i=null));s=this.tokenizer.nextToken()}if(this.tokenizer.endOfFile()&&(e=!0),o.length>0&&this.unclosedBracket(i),e&&r){for(;a.length&&("space"===(s=a[a.length-1][0])||"comment"===s);)this.tokenizer.back(a.pop());this.decl(a)}else this.unknownWord(a)},e.rule=function(t){t.pop();var e=new A.default;this.init(e,t[0][2],t[0][3]),e.raws.between=this.spacesAndCommentsFromEnd(t),this.raw(e,"selector",t),this.current=e},e.decl=function(t){var e=new r.default;this.init(e);var n,i=t[t.length-1];for(";"===i[0]&&(this.semicolon=!0,t.pop()),i[4]?e.source.end={line:i[4],column:i[5]}:e.source.end={line:i[2],column:i[3]};"word"!==t[0][0];)1===t.length&&this.unknownWord(t),e.raws.before+=t.shift()[1];for(e.source.start={line:t[0][2],column:t[0][3]},e.prop="";t.length;){var o=t[0][0];if(":"===o||"space"===o||"comment"===o)break;e.prop+=t.shift()[1]}for(e.raws.between="";t.length;){if(":"===(n=t.shift())[0]){e.raws.between+=n[1];break}"word"===n[0]&&/\w/.test(n[1])&&this.unknownWord([n]),e.raws.between+=n[1]}"_"!==e.prop[0]&&"*"!==e.prop[0]||(e.raws.before+=e.prop[0],e.prop=e.prop.slice(1)),e.raws.between+=this.spacesAndCommentsFromStart(t),this.precheckMissedSemicolon(t);for(var a=t.length-1;a>0;a--){if("!important"===(n=t[a])[1].toLowerCase()){e.important=!0;var s=this.stringFrom(t,a);" !important"!==(s=this.spacesFromEnd(t)+s)&&(e.raws.important=s);break}if("important"===n[1].toLowerCase()){for(var A=t.slice(0),l="",c=a;c>0;c--){var u=A[c][0];if(0===l.trim().indexOf("!")&&"space"!==u)break;l=A.pop()[1]+l}0===l.trim().indexOf("!")&&(e.important=!0,e.raws.important=l,t=A)}if("space"!==n[0]&&"comment"!==n[0])break}this.raw(e,"value",t),-1!==e.value.indexOf(":")&&this.checkMissedSemicolon(t)},e.atrule=function(t){var e,n,r=new a.default;r.name=t[1].slice(1),""===r.name&&this.unnamedAtrule(r,t),this.init(r,t[2],t[3]);for(var i=!1,o=!1,s=[];!this.tokenizer.endOfFile();){if(";"===(t=this.tokenizer.nextToken())[0]){r.source.end={line:t[2],column:t[3]},this.semicolon=!0;break}if("{"===t[0]){o=!0;break}if("}"===t[0]){if(s.length>0){for(e=s[n=s.length-1];e&&"space"===e[0];)e=s[--n];e&&(r.source.end={line:e[4],column:e[5]})}this.end(t);break}if(s.push(t),this.tokenizer.endOfFile()){i=!0;break}}r.raws.between=this.spacesAndCommentsFromEnd(s),s.length?(r.raws.afterName=this.spacesAndCommentsFromStart(s),this.raw(r,"params",s),i&&(t=s[s.length-1],r.source.end={line:t[4],column:t[5]},this.spaces=r.raws.between,r.raws.between="")):(r.raws.afterName="",r.params=""),o&&(r.nodes=[],this.current=r)},e.end=function(t){this.current.nodes&&this.current.nodes.length&&(this.current.raws.semicolon=this.semicolon),this.semicolon=!1,this.current.raws.after=(this.current.raws.after||"")+this.spaces,this.spaces="",this.current.parent?(this.current.source.end={line:t[2],column:t[3]},this.current=this.current.parent):this.unexpectedClose(t)},e.endFile=function(){this.current.parent&&this.unclosedBlock(),this.current.nodes&&this.current.nodes.length&&(this.current.raws.semicolon=this.semicolon),this.current.raws.after=(this.current.raws.after||"")+this.spaces},e.freeSemicolon=function(t){if(this.spaces+=t[1],this.current.nodes){var e=this.current.nodes[this.current.nodes.length-1];e&&"rule"===e.type&&!e.raws.ownSemicolon&&(e.raws.ownSemicolon=this.spaces,this.spaces="")}},e.init=function(t,e,n){this.current.push(t),t.source={start:{line:e,column:n},input:this.input},t.raws.before=this.spaces,this.spaces="","comment"!==t.type&&(this.semicolon=!1)},e.raw=function(t,e,n){for(var r,i,o,a,s=n.length,A="",l=!0,c=/^([.|#])?([\w])+/i,u=0;u<s;u+=1)"comment"!==(i=(r=n[u])[0])||"rule"!==t.type?"comment"===i||"space"===i&&u===s-1?l=!1:A+=r[1]:(a=n[u-1],o=n[u+1],"space"!==a[0]&&"space"!==o[0]&&c.test(a[1])&&c.test(o[1])?A+=r[1]:l=!1);if(!l){var g=n.reduce(function(t,e){return t+e[1]},"");t.raws[e]={value:A,raw:g}}t[e]=A},e.spacesAndCommentsFromEnd=function(t){for(var e,n="";t.length&&("space"===(e=t[t.length-1][0])||"comment"===e);)n=t.pop()[1]+n;return n},e.spacesAndCommentsFromStart=function(t){for(var e,n="";t.length&&("space"===(e=t[0][0])||"comment"===e);)n+=t.shift()[1];return n},e.spacesFromEnd=function(t){for(var e="";t.length&&"space"===t[t.length-1][0];)e=t.pop()[1]+e;return e},e.stringFrom=function(t,e){for(var n="",r=e;r<t.length;r++)n+=t[r][1];return t.splice(e,t.length-e),n},e.colon=function(t){for(var e,n,r,i=0,o=0;o<t.length;o++){if("("===(n=(e=t[o])[0])&&(i+=1),")"===n&&(i-=1),0===i&&":"===n){if(r){if("word"===r[0]&&"progid"===r[1])continue;return o}this.doubleColon(e)}r=e}return!1},e.unclosedBracket=function(t){throw this.input.error("Unclosed bracket",t[2],t[3])},e.unknownWord=function(t){throw this.input.error("Unknown word",t[0][2],t[0][3])},e.unexpectedClose=function(t){throw this.input.error("Unexpected }",t[2],t[3])},e.unclosedBlock=function(){var t=this.current.source.start;throw this.input.error("Unclosed block",t.line,t.column)},e.doubleColon=function(t){throw this.input.error("Double colon",t[2],t[3])},e.unnamedAtrule=function(t,e){throw this.input.error("At-rule without name",e[2],e[3])},e.precheckMissedSemicolon=function(){},e.checkMissedSemicolon=function(t){var e=this.colon(t);if(!1!==e){for(var n,r=0,i=e-1;i>=0&&("space"===(n=t[i])[0]||2!==(r+=1));i--);throw this.input.error("Missed semicolon",n[2],n[3])}},t}();e.default=c,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t,e){void 0===e&&(e={});var n,w,x,_,D,M,k,S,L,R,N,F,T,U,O=t.css.valueOf(),G=e.ignoreErrors,H=O.length,j=-1,Y=1,P=0,K=[],z=[];function J(e){throw t.error("Unclosed "+e,Y,P-j)}return{back:function(t){z.push(t)},nextToken:function(t){if(z.length)return z.pop();if(P>=H)return;var e=!!t&&t.ignoreUnclosed;((n=O.charCodeAt(P))===s||n===l||n===u&&O.charCodeAt(P+1)!==s)&&(j=P,Y+=1);switch(n){case s:case A:case c:case u:case l:w=P;do{w+=1,(n=O.charCodeAt(w))===s&&(j=w,Y+=1)}while(n===A||n===s||n===c||n===u||n===l);U=["space",O.slice(P,w)],P=w-1;break;case g:case h:case p:case I:case y:case C:case f:var q=String.fromCharCode(n);U=[q,q,Y,P-j];break;case d:if(F=K.length?K.pop()[1]:"",T=O.charCodeAt(P+1),"url"===F&&T!==r&&T!==i&&T!==A&&T!==s&&T!==c&&T!==l&&T!==u){w=P;do{if(R=!1,-1===(w=O.indexOf(")",w+1))){if(G||e){w=P;break}J("bracket")}for(N=w;O.charCodeAt(N-1)===o;)N-=1,R=!R}while(R);U=["brackets",O.slice(P,w+1),Y,P-j,Y,w-j],P=w}else w=O.indexOf(")",P+1),M=O.slice(P,w+1),-1===w||b.test(M)?U=["(","(",Y,P-j]:(U=["brackets",M,Y,P-j,Y,w-j],P=w);break;case r:case i:x=n===r?"'":'"',w=P;do{if(R=!1,-1===(w=O.indexOf(x,w+1))){if(G||e){w=P+1;break}J("string")}for(N=w;O.charCodeAt(N-1)===o;)N-=1,R=!R}while(R);M=O.slice(P,w+1),_=M.split("\n"),(D=_.length-1)>0?(S=Y+D,L=w-_[D].length):(S=Y,L=j),U=["string",O.slice(P,w+1),Y,P-j,S,w-L],j=L,Y=S,P=w;break;case E:v.lastIndex=P+1,v.test(O),w=0===v.lastIndex?O.length-1:v.lastIndex-2,U=["at-word",O.slice(P,w+1),Y,P-j,Y,w-j],P=w;break;case o:for(w=P,k=!0;O.charCodeAt(w+1)===o;)w+=1,k=!k;if(n=O.charCodeAt(w+1),k&&n!==a&&n!==A&&n!==s&&n!==c&&n!==u&&n!==l&&(w+=1,Q.test(O.charAt(w)))){for(;Q.test(O.charAt(w+1));)w+=1;O.charCodeAt(w+1)===A&&(w+=1)}U=["word",O.slice(P,w+1),Y,P-j,Y,w-j],P=w;break;default:n===a&&O.charCodeAt(P+1)===m?(0===(w=O.indexOf("*/",P+2)+1)&&(G||e?w=O.length:J("comment")),M=O.slice(P,w+1),_=M.split("\n"),(D=_.length-1)>0?(S=Y+D,L=w-_[D].length):(S=Y,L=j),U=["comment",M,Y,P-j,S,w-L],j=L,Y=S,P=w):(B.lastIndex=P+1,B.test(O),w=0===B.lastIndex?O.length-1:B.lastIndex-2,U=["word",O.slice(P,w+1),Y,P-j,Y,w-j],K.push(U),P=w)}return P++,U},endOfFile:function(){return 0===z.length&&P>=H},position:function(){return P}}};var r="'".charCodeAt(0),i='"'.charCodeAt(0),o="\\".charCodeAt(0),a="/".charCodeAt(0),s="\n".charCodeAt(0),A=" ".charCodeAt(0),l="\f".charCodeAt(0),c="\t".charCodeAt(0),u="\r".charCodeAt(0),g="[".charCodeAt(0),h="]".charCodeAt(0),d="(".charCodeAt(0),f=")".charCodeAt(0),p="{".charCodeAt(0),I="}".charCodeAt(0),C=";".charCodeAt(0),m="*".charCodeAt(0),y=":".charCodeAt(0),E="@".charCodeAt(0),v=/[ \n\t\r\f{}()'"\\;\/[\]#]/g,B=/[ \n\t\r\f(){}:;@!'"\\\][#]|\/(?=\*)/g,b=/.[\\\/("'\n]/,Q=/[a-f0-9]/i;t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=a(n(171)),i=a(n(601)),o=a(n(1128));function a(t){return t&&t.__esModule?t:{default:t}}function s(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}var A=0,l=function(){function t(t,e){if(void 0===e&&(e={}),null===t||"object"==typeof t&&!t.toString)throw new Error("PostCSS received "+t+" instead of CSS string");this.css=t.toString(),"\ufeff"===this.css[0]||""===this.css[0]?(this.hasBOM=!0,this.css=this.css.slice(1)):this.hasBOM=!1,e.from&&(/^\w+:\/\//.test(e.from)?this.file=e.from:this.file=r.default.resolve(e.from));var n=new o.default(this.css,e);if(n.text){this.map=n;var i=n.consumer().file;!this.file&&i&&(this.file=this.mapResolve(i))}this.file||(A+=1,this.id="<input css "+A+">"),this.map&&(this.map.file=this.from)}var e,n,a,l=t.prototype;return l.error=function(t,e,n,r){var o;void 0===r&&(r={});var a=this.origin(e,n);return(o=a?new i.default(t,a.line,a.column,a.source,a.file,r.plugin):new i.default(t,e,n,this.css,this.file,r.plugin)).input={line:e,column:n,source:this.css},this.file&&(o.input.file=this.file),o},l.origin=function(t,e){if(!this.map)return!1;var n=this.map.consumer(),r=n.originalPositionFor({line:t,column:e});if(!r.source)return!1;var i={file:this.mapResolve(r.source),line:r.line,column:r.column},o=n.sourceContentFor(r.source);return o&&(i.source=o),i},l.mapResolve=function(t){return/^\w+:\/\//.test(t)?t:r.default.resolve(this.map.consumer().sourceRoot||".",t)},e=t,(n=[{key:"from",get:function(){return this.file||this.id}}])&&s(e.prototype,n),a&&s(e,a),t}();e.default=l,t.exports=e.default},function(t,e,n){"use strict";(function(r){e.__esModule=!0,e.default=void 0;var i=s(n(605)),o=s(n(171)),a=s(n(1129));function s(t){return t&&t.__esModule?t:{default:t}}var A=function(){function t(t,e){this.loadAnnotation(t),this.inline=this.startWith(this.annotation,"data:");var n=e.map?e.map.prev:void 0,r=this.loadMap(e.from,n);r&&(this.text=r)}var e=t.prototype;return e.consumer=function(){return this.consumerCache||(this.consumerCache=new i.default.SourceMapConsumer(this.text)),this.consumerCache},e.withContent=function(){return!!(this.consumer().sourcesContent&&this.consumer().sourcesContent.length>0)},e.startWith=function(t,e){return!!t&&t.substr(0,e.length)===e},e.loadAnnotation=function(t){var e=t.match(/\/\*\s*# sourceMappingURL=(.*)\s*\*\//);e&&(this.annotation=e[1].trim())},e.decodeInline=function(t){var e,n="data:application/json,";if(this.startWith(t,n))return decodeURIComponent(t.substr(n.length));if(/^data:application\/json;charset=utf-?8;base64,/.test(t)||/^data:application\/json;base64,/.test(t))return e=t.substr(RegExp.lastMatch.length),r?r.from(e,"base64").toString():window.atob(e);var i=t.match(/data:application\/json;([^,]+),/)[1];throw new Error("Unsupported source map encoding "+i)},e.loadMap=function(t,e){if(!1===e)return!1;if(e){if("string"==typeof e)return e;if("function"==typeof e){var n=e(t);if(n&&a.default.existsSync&&a.default.existsSync(n))return a.default.readFileSync(n,"utf-8").toString().trim();throw new Error("Unable to load previous source map: "+n.toString())}if(e instanceof i.default.SourceMapConsumer)return i.default.SourceMapGenerator.fromSourceMap(e).toString();if(e instanceof i.default.SourceMapGenerator)return e.toString();if(this.isMap(e))return JSON.stringify(e);throw new Error("Unsupported previous source map format: "+e.toString())}if(this.inline)return this.decodeInline(this.annotation);if(this.annotation){var r=this.annotation;return t&&(r=o.default.join(o.default.dirname(t),r)),this.root=o.default.dirname(r),!(!a.default.existsSync||!a.default.existsSync(r))&&a.default.readFileSync(r,"utf-8").toString().trim()}},e.isMap=function(t){return"object"==typeof t&&("string"==typeof t.mappings||"string"==typeof t._mappings)},t}();e.default=A,t.exports=e.default}).call(this,n(42).Buffer)},function(t,e){},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r={prefix:function(t){var e=t.match(/^(-\w+-)/);return e?e[0]:""},unprefixed:function(t){return t.replace(/^-\w+-/,"")}};e.default=r,t.exports=e.default},function(t,e,n){"use strict";var r=n(1132),i=n(1133);function o(){this.protocol=null,this.slashes=null,this.auth=null,this.host=null,this.port=null,this.hostname=null,this.hash=null,this.search=null,this.query=null,this.pathname=null,this.path=null,this.href=null}e.parse=m,e.resolve=function(t,e){return m(t,!1,!0).resolve(e)},e.resolveObject=function(t,e){return t?m(t,!1,!0).resolveObject(e):e},e.format=function(t){i.isString(t)&&(t=m(t));return t instanceof o?t.format():o.prototype.format.call(t)},e.Url=o;var a=/^([a-z0-9.+-]+:)/i,s=/:[0-9]*$/,A=/^(\/\/?(?!\/)[^\?\s]*)(\?[^\s]*)?$/,l=["{","}","|","\\","^","`"].concat(["<",">",'"',"`"," ","\r","\n","\t"]),c=["'"].concat(l),u=["%","/","?",";","#"].concat(c),g=["/","?","#"],h=/^[+a-z0-9A-Z_-]{0,63}$/,d=/^([+a-z0-9A-Z_-]{0,63})(.*)$/,f={javascript:!0,"javascript:":!0},p={javascript:!0,"javascript:":!0},I={http:!0,https:!0,ftp:!0,gopher:!0,file:!0,"http:":!0,"https:":!0,"ftp:":!0,"gopher:":!0,"file:":!0},C=n(1134);function m(t,e,n){if(t&&i.isObject(t)&&t instanceof o)return t;var r=new o;return r.parse(t,e,n),r}o.prototype.parse=function(t,e,n){if(!i.isString(t))throw new TypeError("Parameter 'url' must be a string, not "+typeof t);var o=t.indexOf("?"),s=-1!==o&&o<t.indexOf("#")?"?":"#",l=t.split(s);l[0]=l[0].replace(/\\/g,"/");var m=t=l.join(s);if(m=m.trim(),!n&&1===t.split("#").length){var y=A.exec(m);if(y)return this.path=m,this.href=m,this.pathname=y[1],y[2]?(this.search=y[2],this.query=e?C.parse(this.search.substr(1)):this.search.substr(1)):e&&(this.search="",this.query={}),this}var E=a.exec(m);if(E){var v=(E=E[0]).toLowerCase();this.protocol=v,m=m.substr(E.length)}if(n||E||m.match(/^\/\/[^@\/]+@[^@\/]+/)){var B="//"===m.substr(0,2);!B||E&&p[E]||(m=m.substr(2),this.slashes=!0)}if(!p[E]&&(B||E&&!I[E])){for(var b,Q,w=-1,x=0;x<g.length;x++){-1!==(_=m.indexOf(g[x]))&&(-1===w||_<w)&&(w=_)}-1!==(Q=-1===w?m.lastIndexOf("@"):m.lastIndexOf("@",w))&&(b=m.slice(0,Q),m=m.slice(Q+1),this.auth=decodeURIComponent(b)),w=-1;for(x=0;x<u.length;x++){var _;-1!==(_=m.indexOf(u[x]))&&(-1===w||_<w)&&(w=_)}-1===w&&(w=m.length),this.host=m.slice(0,w),m=m.slice(w),this.parseHost(),this.hostname=this.hostname||"";var D="["===this.hostname[0]&&"]"===this.hostname[this.hostname.length-1];if(!D)for(var M=this.hostname.split(/\./),k=(x=0,M.length);x<k;x++){var S=M[x];if(S&&!S.match(h)){for(var L="",R=0,N=S.length;R<N;R++)S.charCodeAt(R)>127?L+="x":L+=S[R];if(!L.match(h)){var F=M.slice(0,x),T=M.slice(x+1),U=S.match(d);U&&(F.push(U[1]),T.unshift(U[2])),T.length&&(m="/"+T.join(".")+m),this.hostname=F.join(".");break}}}this.hostname.length>255?this.hostname="":this.hostname=this.hostname.toLowerCase(),D||(this.hostname=r.toASCII(this.hostname));var O=this.port?":"+this.port:"",G=this.hostname||"";this.host=G+O,this.href+=this.host,D&&(this.hostname=this.hostname.substr(1,this.hostname.length-2),"/"!==m[0]&&(m="/"+m))}if(!f[v])for(x=0,k=c.length;x<k;x++){var H=c[x];if(-1!==m.indexOf(H)){var j=encodeURIComponent(H);j===H&&(j=escape(H)),m=m.split(H).join(j)}}var Y=m.indexOf("#");-1!==Y&&(this.hash=m.substr(Y),m=m.slice(0,Y));var P=m.indexOf("?");if(-1!==P?(this.search=m.substr(P),this.query=m.substr(P+1),e&&(this.query=C.parse(this.query)),m=m.slice(0,P)):e&&(this.search="",this.query={}),m&&(this.pathname=m),I[v]&&this.hostname&&!this.pathname&&(this.pathname="/"),this.pathname||this.search){O=this.pathname||"";var K=this.search||"";this.path=O+K}return this.href=this.format(),this},o.prototype.format=function(){var t=this.auth||"";t&&(t=(t=encodeURIComponent(t)).replace(/%3A/i,":"),t+="@");var e=this.protocol||"",n=this.pathname||"",r=this.hash||"",o=!1,a="";this.host?o=t+this.host:this.hostname&&(o=t+(-1===this.hostname.indexOf(":")?this.hostname:"["+this.hostname+"]"),this.port&&(o+=":"+this.port)),this.query&&i.isObject(this.query)&&Object.keys(this.query).length&&(a=C.stringify(this.query));var s=this.search||a&&"?"+a||"";return e&&":"!==e.substr(-1)&&(e+=":"),this.slashes||(!e||I[e])&&!1!==o?(o="//"+(o||""),n&&"/"!==n.charAt(0)&&(n="/"+n)):o||(o=""),r&&"#"!==r.charAt(0)&&(r="#"+r),s&&"?"!==s.charAt(0)&&(s="?"+s),e+o+(n=n.replace(/[?#]/g,function(t){return encodeURIComponent(t)}))+(s=s.replace("#","%23"))+r},o.prototype.resolve=function(t){return this.resolveObject(m(t,!1,!0)).format()},o.prototype.resolveObject=function(t){if(i.isString(t)){var e=new o;e.parse(t,!1,!0),t=e}for(var n=new o,r=Object.keys(this),a=0;a<r.length;a++){var s=r[a];n[s]=this[s]}if(n.hash=t.hash,""===t.href)return n.href=n.format(),n;if(t.slashes&&!t.protocol){for(var A=Object.keys(t),l=0;l<A.length;l++){var c=A[l];"protocol"!==c&&(n[c]=t[c])}return I[n.protocol]&&n.hostname&&!n.pathname&&(n.path=n.pathname="/"),n.href=n.format(),n}if(t.protocol&&t.protocol!==n.protocol){if(!I[t.protocol]){for(var u=Object.keys(t),g=0;g<u.length;g++){var h=u[g];n[h]=t[h]}return n.href=n.format(),n}if(n.protocol=t.protocol,t.host||p[t.protocol])n.pathname=t.pathname;else{for(var d=(t.pathname||"").split("/");d.length&&!(t.host=d.shift()););t.host||(t.host=""),t.hostname||(t.hostname=""),""!==d[0]&&d.unshift(""),d.length<2&&d.unshift(""),n.pathname=d.join("/")}if(n.search=t.search,n.query=t.query,n.host=t.host||"",n.auth=t.auth,n.hostname=t.hostname||t.host,n.port=t.port,n.pathname||n.search){var f=n.pathname||"",C=n.search||"";n.path=f+C}return n.slashes=n.slashes||t.slashes,n.href=n.format(),n}var m=n.pathname&&"/"===n.pathname.charAt(0),y=t.host||t.pathname&&"/"===t.pathname.charAt(0),E=y||m||n.host&&t.pathname,v=E,B=n.pathname&&n.pathname.split("/")||[],b=(d=t.pathname&&t.pathname.split("/")||[],n.protocol&&!I[n.protocol]);if(b&&(n.hostname="",n.port=null,n.host&&(""===B[0]?B[0]=n.host:B.unshift(n.host)),n.host="",t.protocol&&(t.hostname=null,t.port=null,t.host&&(""===d[0]?d[0]=t.host:d.unshift(t.host)),t.host=null),E=E&&(""===d[0]||""===B[0])),y)n.host=t.host||""===t.host?t.host:n.host,n.hostname=t.hostname||""===t.hostname?t.hostname:n.hostname,n.search=t.search,n.query=t.query,B=d;else if(d.length)B||(B=[]),B.pop(),B=B.concat(d),n.search=t.search,n.query=t.query;else if(!i.isNullOrUndefined(t.search)){if(b)n.hostname=n.host=B.shift(),(D=!!(n.host&&n.host.indexOf("@")>0)&&n.host.split("@"))&&(n.auth=D.shift(),n.host=n.hostname=D.shift());return n.search=t.search,n.query=t.query,i.isNull(n.pathname)&&i.isNull(n.search)||(n.path=(n.pathname?n.pathname:"")+(n.search?n.search:"")),n.href=n.format(),n}if(!B.length)return n.pathname=null,n.search?n.path="/"+n.search:n.path=null,n.href=n.format(),n;for(var Q=B.slice(-1)[0],w=(n.host||t.host||B.length>1)&&("."===Q||".."===Q)||""===Q,x=0,_=B.length;_>=0;_--)"."===(Q=B[_])?B.splice(_,1):".."===Q?(B.splice(_,1),x++):x&&(B.splice(_,1),x--);if(!E&&!v)for(;x--;x)B.unshift("..");!E||""===B[0]||B[0]&&"/"===B[0].charAt(0)||B.unshift(""),w&&"/"!==B.join("/").substr(-1)&&B.push("");var D,M=""===B[0]||B[0]&&"/"===B[0].charAt(0);b&&(n.hostname=n.host=M?"":B.length?B.shift():"",(D=!!(n.host&&n.host.indexOf("@")>0)&&n.host.split("@"))&&(n.auth=D.shift(),n.host=n.hostname=D.shift()));return(E=E||n.host&&B.length)&&!M&&B.unshift(""),B.length?n.pathname=B.join("/"):(n.pathname=null,n.path=null),i.isNull(n.pathname)&&i.isNull(n.search)||(n.path=(n.pathname?n.pathname:"")+(n.search?n.search:"")),n.auth=t.auth||n.auth,n.slashes=n.slashes||t.slashes,n.href=n.format(),n},o.prototype.parseHost=function(){var t=this.host,e=s.exec(t);e&&(":"!==(e=e[0])&&(this.port=e.substr(1)),t=t.substr(0,t.length-e.length)),t&&(this.hostname=t)}},function(t,e,n){(function(t,r){var i;/*! https://mths.be/punycode v1.4.1 by @mathias */!function(o){e&&e.nodeType,t&&t.nodeType;var a="object"==typeof r&&r;a.global!==a&&a.window!==a&&a.self;var s,A=2147483647,l=36,c=1,u=26,g=38,h=700,d=72,f=128,p="-",I=/^xn--/,C=/[^\x20-\x7E]/,m=/[\x2E\u3002\uFF0E\uFF61]/g,y={overflow:"Overflow: input needs wider integers to process","not-basic":"Illegal input >= 0x80 (not a basic code point)","invalid-input":"Invalid input"},E=l-c,v=Math.floor,B=String.fromCharCode;function b(t){throw new RangeError(y[t])}function Q(t,e){for(var n=t.length,r=[];n--;)r[n]=e(t[n]);return r}function w(t,e){var n=t.split("@"),r="";return n.length>1&&(r=n[0]+"@",t=n[1]),r+Q((t=t.replace(m,".")).split("."),e).join(".")}function x(t){for(var e,n,r=[],i=0,o=t.length;i<o;)(e=t.charCodeAt(i++))>=55296&&e<=56319&&i<o?56320==(64512&(n=t.charCodeAt(i++)))?r.push(((1023&e)<<10)+(1023&n)+65536):(r.push(e),i--):r.push(e);return r}function _(t){return Q(t,function(t){var e="";return t>65535&&(e+=B((t-=65536)>>>10&1023|55296),t=56320|1023&t),e+=B(t)}).join("")}function D(t,e){return t+22+75*(t<26)-((0!=e)<<5)}function M(t,e,n){var r=0;for(t=n?v(t/h):t>>1,t+=v(t/e);t>E*u>>1;r+=l)t=v(t/E);return v(r+(E+1)*t/(t+g))}function k(t){var e,n,r,i,o,a,s,g,h,I,C,m=[],y=t.length,E=0,B=f,Q=d;for((n=t.lastIndexOf(p))<0&&(n=0),r=0;r<n;++r)t.charCodeAt(r)>=128&&b("not-basic"),m.push(t.charCodeAt(r));for(i=n>0?n+1:0;i<y;){for(o=E,a=1,s=l;i>=y&&b("invalid-input"),((g=(C=t.charCodeAt(i++))-48<10?C-22:C-65<26?C-65:C-97<26?C-97:l)>=l||g>v((A-E)/a))&&b("overflow"),E+=g*a,!(g<(h=s<=Q?c:s>=Q+u?u:s-Q));s+=l)a>v(A/(I=l-h))&&b("overflow"),a*=I;Q=M(E-o,e=m.length+1,0==o),v(E/e)>A-B&&b("overflow"),B+=v(E/e),E%=e,m.splice(E++,0,B)}return _(m)}function S(t){var e,n,r,i,o,a,s,g,h,I,C,m,y,E,Q,w=[];for(m=(t=x(t)).length,e=f,n=0,o=d,a=0;a<m;++a)(C=t[a])<128&&w.push(B(C));for(r=i=w.length,i&&w.push(p);r<m;){for(s=A,a=0;a<m;++a)(C=t[a])>=e&&C<s&&(s=C);for(s-e>v((A-n)/(y=r+1))&&b("overflow"),n+=(s-e)*y,e=s,a=0;a<m;++a)if((C=t[a])<e&&++n>A&&b("overflow"),C==e){for(g=n,h=l;!(g<(I=h<=o?c:h>=o+u?u:h-o));h+=l)Q=g-I,E=l-I,w.push(B(D(I+Q%E,0))),g=v(Q/E);w.push(B(D(g,0))),o=M(n,y,r==i),n=0,++r}++n,++e}return w.join("")}s={version:"1.4.1",ucs2:{decode:x,encode:_},decode:k,encode:S,toASCII:function(t){return w(t,function(t){return C.test(t)?"xn--"+S(t):t})},toUnicode:function(t){return w(t,function(t){return I.test(t)?k(t.slice(4).toLowerCase()):t})}},void 0===(i=function(){return s}.call(e,n,e,t))||(t.exports=i)}()}).call(this,n(98)(t),n(37))},function(t,e,n){"use strict";t.exports={isString:function(t){return"string"==typeof t},isObject:function(t){return"object"==typeof t&&null!==t},isNull:function(t){return null===t},isNullOrUndefined:function(t){return null==t}}},function(t,e,n){"use strict";e.decode=e.parse=n(1135),e.encode=e.stringify=n(1136)},function(t,e,n){"use strict";function r(t,e){return Object.prototype.hasOwnProperty.call(t,e)}t.exports=function(t,e,n,o){e=e||"&",n=n||"=";var a={};if("string"!=typeof t||0===t.length)return a;var s=/\+/g;t=t.split(e);var A=1e3;o&&"number"==typeof o.maxKeys&&(A=o.maxKeys);var l=t.length;A>0&&l>A&&(l=A);for(var c=0;c<l;++c){var u,g,h,d,f=t[c].replace(s,"%20"),p=f.indexOf(n);p>=0?(u=f.substr(0,p),g=f.substr(p+1)):(u=f,g=""),h=decodeURIComponent(u),d=decodeURIComponent(g),r(a,h)?i(a[h])?a[h].push(d):a[h]=[a[h],d]:a[h]=d}return a};var i=Array.isArray||function(t){return"[object Array]"===Object.prototype.toString.call(t)}},function(t,e,n){"use strict";var r=function(t){switch(typeof t){case"string":return t;case"boolean":return t?"true":"false";case"number":return isFinite(t)?t:"";default:return""}};t.exports=function(t,e,n,s){return e=e||"&",n=n||"=",null===t&&(t=void 0),"object"==typeof t?o(a(t),function(a){var s=encodeURIComponent(r(a))+n;return i(t[a])?o(t[a],function(t){return s+encodeURIComponent(r(t))}).join(e):s+encodeURIComponent(r(t[a]))}).join(e):s?encodeURIComponent(r(s))+n+encodeURIComponent(r(t)):""};var i=Array.isArray||function(t){return"[object Array]"===Object.prototype.toString.call(t)};function o(t,e){if(t.map)return t.map(e);for(var n=[],r=0;r<t.length;r++)n.push(e(t[r],r));return n}var a=Object.keys||function(t){var e=[];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&e.push(n);return e}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2),i=r.__importDefault(n(4)),o=r.__importStar(n(0)),a=r.__importStar(n(139)),s=r.__importStar(n(618)),A=n(140),l=n(190),c=n(1144),u=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.state={isActive:!1},e.refHandlers={button:function(t){e.buttonRef=t,A.safeInvoke(e.props.elementRef,t)}},e.currentKeyDown=null,e.handleKeyDown=function(t){s.isKeyboardClick(t.which)&&(t.preventDefault(),t.which!==e.currentKeyDown&&e.setState({isActive:!0})),e.currentKeyDown=t.which,A.safeInvoke(e.props.onKeyDown,t)},e.handleKeyUp=function(t){s.isKeyboardClick(t.which)&&(e.setState({isActive:!1}),e.buttonRef.click()),e.currentKeyDown=null,A.safeInvoke(e.props.onKeyUp,t)},e}return r.__extends(e,t),e.prototype.getCommonButtonProps=function(){var t,e=this.props,n=e.alignText,r=e.fill,o=e.large,s=e.loading,A=e.minimal,l=e.small,c=e.tabIndex,u=this.props.disabled||s;return{className:i.default(a.BUTTON,((t={})[a.ACTIVE]=this.state.isActive||this.props.active,t[a.DISABLED]=u,t[a.FILL]=r,t[a.LARGE]=o,t[a.LOADING]=s,t[a.MINIMAL]=A,t[a.SMALL]=l,t),a.alignmentClass(n),a.intentClass(this.props.intent),this.props.className),disabled:u,onClick:u?void 0:this.props.onClick,onKeyDown:this.handleKeyDown,onKeyUp:this.handleKeyUp,ref:this.refHandlers.button,tabIndex:u?-1:c}},e.prototype.renderChildren=function(){var t=this.props,e=t.children,n=t.icon,r=t.loading,i=t.rightIcon,s=t.text;return[r&&o.createElement(c.Spinner,{key:"loading",className:a.BUTTON_SPINNER,size:l.Icon.SIZE_LARGE}),o.createElement(l.Icon,{key:"leftIcon",icon:n}),(!A.isReactNodeEmpty(s)||!A.isReactNodeEmpty(e))&&o.createElement("span",{key:"text",className:a.BUTTON_TEXT},s,e),o.createElement(l.Icon,{key:"rightIcon",icon:i})]},e}(o.PureComponent);e.AbstractButton=u},function(t,e,n){"use strict";function r(t,e,n){return void 0===n&&(n=function(t,e){return t===e}),null==t&&null==e||null!=t&&null!=e&&t.length===e.length&&t.every(function(t,r){return n(t,e[r])})}function i(t,e,n){if(null==t&&null==e)return!0;if(null==t||null==e)return!1;if(Array.isArray(t)||Array.isArray(e))return!1;if(null!=n)return a(t,e,n);var r=Object.keys(t),i=Object.keys(e);return a(t,e,{include:r})&&a(t,e,{include:i})}function o(t,e,n){if(t===e)return!0;if(null==t&&null==e)return!0;if(null==t||null==e)return!1;if(Array.isArray(t)||Array.isArray(e))return r(t,e,o);if(A(t)||A(e))return t===e;if(null!=n)return s(t,e,n);if(t.constructor!==e.constructor)return!1;var i=Object.keys(t),a=Object.keys(e);return null!=i&&null!=a&&(0===i.length&&0===a.length||r(i,a)&&s(t,e,i))}function a(t,e,n){return l(t,e,n).every(function(n){return t.hasOwnProperty(n)===e.hasOwnProperty(n)&&t[n]===e[n]})}function s(t,e,n){return n.every(function(n){return t.hasOwnProperty(n)===e.hasOwnProperty(n)&&o(t[n],e[n])})}function A(t){return"number"==typeof t||"string"==typeof t||"boolean"==typeof t}function l(t,e,n){if(function(t){return null!=t&&null!=t.include}(n))return n.include;var r=Object.keys(t),i=Object.keys(e),o=c(r.concat(i));return n.exclude.forEach(function(t){return delete o[t]}),Object.keys(o)}function c(t){return t.reduce(function(t,e){return t[e]=!0,t},{})}function u(t,e,n,r){return n.filter(function(n){return!r(t,e,n)}).map(function(n){return{key:n,valueA:t[n],valueB:e[n]}})}Object.defineProperty(e,"__esModule",{value:!0}),e.arraysEqual=r,e.shallowCompareKeys=i,e.deepCompareKeys=o,e.getShallowUnequalKeyValues=function(t,e,n){var r=null==t?{}:t,o=null==e?{}:e;return u(r,o,l(r,o,null==n?{exclude:[]}:n),function(t,e,n){return i(t,e,{include:[n]})})},e.getDeepUnequalKeyValues=function(t,e,n){return void 0===t&&(t={}),void 0===e&&(e={}),u(t,e,null==n?function(t,e){var n=Object.keys(t),r=Object.keys(e),i=c(n.concat(r));return Object.keys(i)}(t,e):n,function(t,e,n){return o(t,e,[n])})}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(140);e.safeInvokeMember=function(t,e){for(var n=[],i=2;i<arguments.length;i++)n[i-2]=arguments[i];if(null!=t){var o=t[e];if(r.isFunction(o))return o.apply(void 0,n)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2);r.__exportStar(n(1141),e),r.__exportStar(n(308),e),r.__exportStar(n(615),e),r.__exportStar(n(1142),e),r.__exportStar(n(1143),e),r.__exportStar(n(616),e),r.__exportStar(n(218),e),r.__exportStar(n(617),e),r.__exportStar(n(177),e);var i=r.__importStar(n(139));e.Classes=i;var o=r.__importStar(n(618));e.Keys=o;var a=r.__importStar(n(140));e.Utils=a},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2),i=r.__importStar(n(0)),o=n(140),a=function(t){function e(e,n){var r=t.call(this,e,n)||this;return r.timeoutIds=[],r.clearTimeouts=function(){if(r.timeoutIds.length>0){for(var t=0,e=r.timeoutIds;t<e.length;t++){var n=e[t];window.clearTimeout(n)}r.timeoutIds=[]}},o.isNodeEnv("production")||r.validateProps(r.props),r}return r.__extends(e,t),e.prototype.componentWillReceiveProps=function(t){o.isNodeEnv("production")||this.validateProps(t)},e.prototype.componentWillUnmount=function(){this.clearTimeouts()},e.prototype.setTimeout=function(t,e){var n=window.setTimeout(t,e);return this.timeoutIds.push(n),function(){return window.clearTimeout(n)}},e.prototype.validateProps=function(t){},e}(i.Component);e.AbstractComponent=a},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.Boundary={START:"start",END:"end"}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.Colors={BLACK:"#10161A",BLUE1:"#0E5A8A",BLUE2:"#106BA3",BLUE3:"#137CBD",BLUE4:"#2B95D6",BLUE5:"#48AFF0",COBALT1:"#1F4B99",COBALT2:"#2458B3",COBALT3:"#2965CC",COBALT4:"#4580E6",COBALT5:"#669EFF",DARK_GRAY1:"#182026",DARK_GRAY2:"#202B33",DARK_GRAY3:"#293742",DARK_GRAY4:"#30404D",DARK_GRAY5:"#394B59",FOREST1:"#1D7324",FOREST2:"#238C2C",FOREST3:"#29A634",FOREST4:"#43BF4D",FOREST5:"#62D96B",GOLD1:"#A67908",GOLD2:"#BF8C0A",GOLD3:"#D99E0B",GOLD4:"#F2B824",GOLD5:"#FFC940",GRAY1:"#5C7080",GRAY2:"#738694",GRAY3:"#8A9BA8",GRAY4:"#A7B6C2",GRAY5:"#BFCCD6",GREEN1:"#0A6640",GREEN2:"#0D8050",GREEN3:"#0F9960",GREEN4:"#15B371",GREEN5:"#3DCC91",INDIGO1:"#5642A6",INDIGO2:"#634DBF",INDIGO3:"#7157D9",INDIGO4:"#9179F2",INDIGO5:"#AD99FF",LIGHT_GRAY1:"#CED9E0",LIGHT_GRAY2:"#D8E1E8",LIGHT_GRAY3:"#E1E8ED",LIGHT_GRAY4:"#EBF1F5",LIGHT_GRAY5:"#F5F8FA",LIME1:"#728C23",LIME2:"#87A629",LIME3:"#9BBF30",LIME4:"#B6D94C",LIME5:"#D1F26D",ORANGE1:"#A66321",ORANGE2:"#BF7326",ORANGE3:"#D9822B",ORANGE4:"#F29D49",ORANGE5:"#FFB366",RED1:"#A82A2A",RED2:"#C23030",RED3:"#DB3737",RED4:"#F55656",RED5:"#FF7373",ROSE1:"#A82255",ROSE2:"#C22762",ROSE3:"#DB2C6F",ROSE4:"#F5498B",ROSE5:"#FF66A1",SEPIA1:"#63411E",SEPIA2:"#7D5125",SEPIA3:"#96622D",SEPIA4:"#B07B46",SEPIA5:"#C99765",TURQUOISE1:"#008075",TURQUOISE2:"#00998C",TURQUOISE3:"#00B3A4",TURQUOISE4:"#14CCBD",TURQUOISE5:"#2EE6D6",VERMILION1:"#9E2B0E",VERMILION2:"#B83211",VERMILION3:"#D13913",VERMILION4:"#EB532D",VERMILION5:"#FF6E4A",VIOLET1:"#5C255C",VIOLET2:"#752F75",VIOLET3:"#8F398F",VIOLET4:"#A854A8",VIOLET5:"#C274C2",WHITE:"#FFFFFF"}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2),i=r.__importDefault(n(4)),o=r.__importStar(n(0)),a=n(308),s=r.__importStar(n(139)),A=n(619),l=n(177),c=n(140),u=45,g="M 50,50 m 0,-45 a 45,45 0 1 1 0,90 a 45,45 0 1 1 0,-90",h=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.__extends(e,t),e.prototype.componentDidUpdate=function(t){t.value!==this.props.value&&this.forceUpdate()},e.prototype.render=function(){var t,n=this.props,r=n.className,a=n.intent,A=n.value,l=n.tagName,u=void 0===l?"div":l,h=this.getSize(),d=i.default(s.SPINNER,s.intentClass(a),((t={})[s.SPINNER_NO_SPIN]=null!=A,t),r),f=Math.min(16,4*e.SIZE_LARGE/h),p=280-280*(null==A?.25:c.clamp(A,0,1));return o.createElement(u,{className:d},o.createElement(u,{className:s.SPINNER_ANIMATION},o.createElement("svg",{width:h,height:h,strokeWidth:f.toFixed(2),viewBox:this.getViewBox(f)},o.createElement("path",{className:s.SPINNER_TRACK,d:g}),o.createElement("path",{className:s.SPINNER_HEAD,d:g,pathLength:280,strokeDasharray:"280 280",strokeDashoffset:p}))))},e.prototype.validateProps=function(t){var e=t.className,n=void 0===e?"":e;null!=t.size&&(n.indexOf(s.SMALL)>=0||n.indexOf(s.LARGE)>=0)&&console.warn(A.SPINNER_WARN_CLASSES_SIZE)},e.prototype.getSize=function(){var t=this.props,n=t.className,r=void 0===n?"":n,i=t.size;return null==i?r.indexOf(s.SMALL)>=0?e.SIZE_SMALL:r.indexOf(s.LARGE)>=0?e.SIZE_LARGE:e.SIZE_STANDARD:Math.max(10,i)},e.prototype.getViewBox=function(t){var e=u+t/2,n=(50-e).toFixed(2),r=(2*e).toFixed(2);return n+" "+n+" "+r+" "+r},e.displayName=l.DISPLAYNAME_PREFIX+".Spinner",e.SIZE_SMALL=20,e.SIZE_STANDARD=50,e.SIZE_LARGE=100,e}(a.AbstractPureComponent);e.Spinner=h},function(t,e){
/*!
Copyright (C) 2013-2015 by Andrea Giammarchi - @WebReflection

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
!function(t){"use strict";function e(){return u.createDocumentFragment()}function n(t){return u.createElement(t)}function r(t,e){if(!t)throw new Error("Failed to construct "+e+": 1 argument required, but only 0 present.")}function i(t){if(1===t.length)return o(t[0]);for(var n=e(),r=S.call(t),i=0;i<t.length;i++)n.appendChild(o(r[i]));return n}function o(t){return"object"==typeof t?t:u.createTextNode(t)}for(var a,s,A,l,c,u=t.document,g=Object.prototype.hasOwnProperty,h=Object.defineProperty||function(t,e,n){return g.call(n,"value")?t[e]=n.value:(g.call(n,"get")&&t.__defineGetter__(e,n.get),g.call(n,"set")&&t.__defineSetter__(e,n.set)),t},d=[].indexOf||function(t){for(var e=this.length;e--&&this[e]!==t;);return e},f=function(t){var e=void 0===t.className,n=e?t.getAttribute("class")||"":t.className,r=e||"object"==typeof n,i=(r?e?n:n.baseVal:n).replace(I,"");i.length&&k.push.apply(this,i.split(C)),this._isSVG=r,this._=t},p={get:function(){return new f(this)},set:function(){}},I=/^\s+|\s+$/g,C=/\s+/,m=function(t,e){return this.contains(t)?e||this.remove(t):(void 0===e||e)&&(e=!0,this.add(t)),!!e},y=t.DocumentFragment&&DocumentFragment.prototype,E=t.Node,v=(E||Element).prototype,B=t.CharacterData||E,b=B&&B.prototype,Q=t.DocumentType,w=Q&&Q.prototype,x=(t.Element||E||t.HTMLElement).prototype,_=t.HTMLSelectElement||n("select").constructor,D=_.prototype.remove,M=t.SVGElement,k=["matches",x.matchesSelector||x.webkitMatchesSelector||x.khtmlMatchesSelector||x.mozMatchesSelector||x.msMatchesSelector||x.oMatchesSelector||function(t){var e=this.parentNode;return!!e&&-1<d.call(e.querySelectorAll(t),this)},"closest",function(t){for(var e,n=this;(e=n&&n.matches)&&!n.matches(t);)n=n.parentNode;return e?n:null},"prepend",function(){var t=this.firstChild,e=i(arguments);t?this.insertBefore(e,t):this.appendChild(e)},"append",function(){this.appendChild(i(arguments))},"before",function(){var t=this.parentNode;t&&t.insertBefore(i(arguments),this)},"after",function(){var t=this.parentNode,e=this.nextSibling,n=i(arguments);t&&(e?t.insertBefore(n,e):t.appendChild(n))},"toggleAttribute",function(t,e){var n=this.hasAttribute(t);return 1<arguments.length?n&&!e?this.removeAttribute(t):e&&!n&&this.setAttribute(t,""):n?this.removeAttribute(t):this.setAttribute(t,""),this.hasAttribute(t)},"replace",function(){this.replaceWith.apply(this,arguments)},"replaceWith",function(){var t=this.parentNode;t&&t.replaceChild(i(arguments),this)},"remove",function(){var t=this.parentNode;t&&t.removeChild(this)}],S=k.slice,L=k.length;L;L-=2)if((s=k[L-2])in x||(x[s]=k[L-1]),"remove"!==s||D._dom4||((_.prototype[s]=function(){return 0<arguments.length?D.apply(this,arguments):x.remove.call(this)})._dom4=!0),/^(?:before|after|replace|replaceWith|remove)$/.test(s)&&(!B||s in b||(b[s]=k[L-1]),!Q||s in w||(w[s]=k[L-1])),/^(?:append|prepend)$/.test(s))if(y)s in y||(y[s]=k[L-1]);else try{e().constructor.prototype[s]=k[L-1]}catch(t){}var R;n("a").matches("a")||(x[s]=(R=x[s],function(t){return R.call(this.parentNode?this:e().appendChild(this),t)})),f.prototype={length:0,add:function(){for(var t,e=0;e<arguments.length;e++)t=arguments[e],this.contains(t)||k.push.call(this,s);this._isSVG?this._.setAttribute("class",""+this):this._.className=""+this},contains:function(t){return function(e){return-1<(L=t.call(this,s=function(t){if(!t)throw"SyntaxError";if(C.test(t))throw"InvalidCharacterError";return t}(e)))}}([].indexOf||function(t){for(L=this.length;L--&&this[L]!==t;);return L}),item:function(t){return this[t]||null},remove:function(){for(var t,e=0;e<arguments.length;e++)t=arguments[e],this.contains(t)&&k.splice.call(this,L,1);this._isSVG?this._.setAttribute("class",""+this):this._.className=""+this},toggle:m,toString:function(){return k.join.call(this," ")}},!M||"classList"in M.prototype||h(M.prototype,"classList",p),"classList"in u.documentElement?((l=n("div").classList).add("a","b","a"),"a b"!=l&&("add"in(A=l.constructor.prototype)||(A=t.TemporaryTokenList.prototype),c=function(t){return function(){for(var e=0;e<arguments.length;)t.call(this,arguments[e++])}},A.add=c(A.add),A.remove=c(A.remove),A.toggle=m)):h(x,"classList",p),"contains"in v||h(v,"contains",{value:function(t){for(;t&&t!==this;)t=t.parentNode;return this===t}}),"head"in u||h(u,"head",{get:function(){return a||(a=u.getElementsByTagName("head")[0])}}),function(){for(var e,n=t.requestAnimationFrame,r=t.cancelAnimationFrame,i=["o","ms","moz","webkit"],o=i.length;!r&&o--;)n=n||t[i[o]+"RequestAnimationFrame"],r=t[i[o]+"CancelAnimationFrame"]||t[i[o]+"CancelRequestAnimationFrame"];r||(n?(e=n,n=function(t){var n=!0;return e(function(){n&&t.apply(this,arguments)}),function(){n=!1}},r=function(t){t()}):(n=function(t){return setTimeout(t,15,15)},r=function(t){clearTimeout(t)})),t.requestAnimationFrame=n,t.cancelAnimationFrame=r}();try{new t.CustomEvent("?")}catch(e){t.CustomEvent=function(t,e){function n(t,e,n,r){this.initEvent(t,e,n),this.detail=r}return function(r,i){var o=u.createEvent(t);if("string"!=typeof r)throw new Error("An event name must be provided");return"Event"==t&&(o.initCustomEvent=n),null==i&&(i=e),o.initCustomEvent(r,i.bubbles,i.cancelable,i.detail),o}}(t.CustomEvent?"CustomEvent":"Event",{bubbles:!1,cancelable:!1,detail:null})}try{new Event("_")}catch(e){e=function(t){function e(t,e){r(arguments.length,"Event");var n=u.createEvent("Event");return e||(e={}),n.initEvent(t,!!e.bubbles,!!e.cancelable),n}return e.prototype=t.prototype,e}(t.Event||function(){}),h(t,"Event",{value:e}),Event!==e&&(Event=e)}try{new KeyboardEvent("_",{})}catch(e){e=function(e){var n,i=0,o={char:"",key:"",location:0,ctrlKey:!1,shiftKey:!1,altKey:!1,metaKey:!1,altGraphKey:!1,repeat:!1,locale:navigator.language,detail:0,bubbles:!1,cancelable:!1,keyCode:0,charCode:0,which:0};try{var a=u.createEvent("KeyboardEvent");a.initKeyboardEvent("keyup",!1,!1,t,"+",3,!0,!1,!0,!1,!1),i="+"==(a.keyIdentifier||a.key)&&3==(a.keyLocation||a.location)&&(a.ctrlKey?a.altKey?1:3:a.shiftKey?2:4)||9}catch(t){}function s(t,e,n){try{e[t]=n[t]}catch(t){}}function A(e,a){r(arguments.length,"KeyboardEvent"),a=function(t,e){for(var n in e)e.hasOwnProperty(n)&&!e.hasOwnProperty.call(t,n)&&(t[n]=e[n]);return t}(a||{},o);var A,l=u.createEvent(n),c=a.ctrlKey,g=a.shiftKey,h=a.altKey,d=a.metaKey,f=a.altGraphKey,p=i>3?function(t){for(var e=[],n=["ctrlKey","Control","shiftKey","Shift","altKey","Alt","metaKey","Meta","altGraphKey","AltGraph"],r=0;r<n.length;r+=2)t[n[r]]&&e.push(n[r+1]);return e.join(" ")}(a):null,I=String(a.key),C=String(a.char),m=a.location,y=a.keyCode||(a.keyCode=I)&&I.charCodeAt(0)||0,E=a.charCode||(a.charCode=C)&&C.charCodeAt(0)||0,v=a.bubbles,B=a.cancelable,b=a.repeat,Q=a.locale,w=a.view||t;if(a.which||(a.which=a.keyCode),"initKeyEvent"in l)l.initKeyEvent(e,v,B,w,c,h,g,d,y,E);else if(0<i&&"initKeyboardEvent"in l){switch(A=[e,v,B,w],i){case 1:A.push(I,m,c,g,h,d,f);break;case 2:A.push(c,h,g,d,y,E);break;case 3:A.push(I,m,c,h,g,d,f);break;case 4:A.push(I,m,p,b,Q);break;default:A.push(char,I,m,p,b,Q)}l.initKeyboardEvent.apply(l,A)}else l.initEvent(e,v,B);for(I in l)o.hasOwnProperty(I)&&l[I]!==a[I]&&s(I,l,a);return l}return n=0<i?"KeyboardEvent":"Event",A.prototype=e.prototype,A}(t.KeyboardEvent||function(){}),h(t,"KeyboardEvent",{value:e}),KeyboardEvent!==e&&(KeyboardEvent=e)}try{new MouseEvent("_",{})}catch(e){e=function(e){function n(e,n){r(arguments.length,"MouseEvent");var i=u.createEvent("MouseEvent");return n||(n={}),i.initMouseEvent(e,!!n.bubbles,!!n.cancelable,n.view||t,n.detail||1,n.screenX||0,n.screenY||0,n.clientX||0,n.clientY||0,!!n.ctrlKey,!!n.altKey,!!n.shiftKey,!!n.metaKey,n.button||0,n.relatedTarget||null),i}return n.prototype=e.prototype,n}(t.MouseEvent||function(){}),h(t,"MouseEvent",{value:e}),MouseEvent!==e&&(MouseEvent=e)}u.querySelectorAll("*").forEach||function(){function t(t){var e=t.querySelectorAll;t.querySelectorAll=function(t){var n=e.call(this,t);return n.forEach=Array.prototype.forEach,n}}t(u),t(Element.prototype)}();try{u.querySelector(":scope *")}catch(t){!function(){var t="data-scope-"+(1e9*Math.random()>>>0),e=Element.prototype,n=e.querySelector,r=e.querySelectorAll;function i(e,n,r){e.setAttribute(t,null);var i=n.call(e,String(r).replace(/(^|,\s*)(:scope([ >]|$))/g,function(e,n,r,i){return n+"["+t+"]"+(i||" ")}));return e.removeAttribute(t),i}e.querySelector=function(t){return i(this,n,t)},e.querySelectorAll=function(t){return i(this,r,t)}}()}}(window),function(t){"use strict";var e=t.WeakMap||function(){var t,e=0,n=!1,r=!1;function i(e,i,o){r=o,n=!1,t=void 0,e.dispatchEvent(i)}function o(t){this.value=t}function s(){e++,this.__ce__=new a("@DOMMap:"+e+Math.random())}return o.prototype.handleEvent=function(e){n=!0,r?e.currentTarget.removeEventListener(e.type,this,!1):t=this.value},s.prototype={constructor:s,delete:function(t){return i(t,this.__ce__,!0),n},get:function(e){i(e,this.__ce__,!1);var n=t;return t=void 0,n},has:function(t){return i(t,this.__ce__,!1),n},set:function(t,e){return i(t,this.__ce__,!0),t.addEventListener(this.__ce__.type,new o(e),!1),this}},s}();function n(){}function r(t,e,n){function i(t){i.once&&(t.currentTarget.removeEventListener(t.type,e,i),i.removed=!0),i.passive&&(t.preventDefault=r.preventDefault),"function"==typeof i.callback?i.callback.call(this,t):i.callback&&i.callback.handleEvent(t),i.passive&&delete t.preventDefault}return i.type=t,i.callback=e,i.capture=!!n.capture,i.passive=!!n.passive,i.once=!!n.once,i.removed=!1,i}n.prototype=(Object.create||Object)(null),r.preventDefault=function(){};var i,o,a=t.CustomEvent,s=t.dispatchEvent,A=t.addEventListener,l=t.removeEventListener,c=0,u=function(){c++},g=[].indexOf||function(t){for(var e=this.length;e--&&this[e]!==t;);return e},h=function(t){return"".concat(t.capture?"1":"0",t.passive?"1":"0",t.once?"1":"0")};try{A("_",u,{once:!0}),s(new a("_")),s(new a("_")),l("_",u,{once:!0})}catch(t){}1!==c&&(o=new e,i=function(t){if(t){var e=t.prototype;e.addEventListener=function(t){return function(e,i,a){if(a&&"boolean"!=typeof a){var s,A,l,c=o.get(this),u=h(a);c||o.set(this,c=new n),e in c||(c[e]={handler:[],wrap:[]}),A=c[e],(s=g.call(A.handler,i))<0?(s=A.handler.push(i)-1,A.wrap[s]=l=new n):l=A.wrap[s],u in l||(l[u]=r(e,i,a),t.call(this,e,l[u],l[u].capture))}else t.call(this,e,i,a)}}(e.addEventListener),e.removeEventListener=function(t){return function(e,n,r){if(r&&"boolean"!=typeof r){var i,a,s,A,l=o.get(this);if(l&&e in l&&(s=l[e],-1<(a=g.call(s.handler,n))&&(i=h(r))in(A=s.wrap[a]))){for(i in t.call(this,e,A[i],A[i].capture),delete A[i],A)return;s.handler.splice(a,1),s.wrap.splice(a,1),0===s.handler.length&&delete l[e]}}else t.call(this,e,n,r)}}(e.removeEventListener)}},t.EventTarget?i(EventTarget):(i(t.Text),i(t.Element||t.HTMLElement),i(t.HTMLDocument),i(t.Window||{prototype:t}),i(t.XMLHttpRequest)))}(self)},function(t,e,n){"use strict";e.__esModule=!0;var r=n(0),i=(a(r),a(n(1))),o=a(n(1149));a(n(1150));function a(t){return t&&t.__esModule?t:{default:t}}function s(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}function A(t,e){if(!t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return!e||"object"!=typeof e&&"function"!=typeof e?t:e}function l(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function, not "+typeof e);t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,enumerable:!1,writable:!0,configurable:!0}}),e&&(Object.setPrototypeOf?Object.setPrototypeOf(t,e):t.__proto__=e)}var c=1073741823;e.default=function(t,e){var n,a,u="__create-react-context-"+(0,o.default)()+"__",g=function(t){function n(){var e,r,i,o;s(this,n);for(var a=arguments.length,l=Array(a),c=0;c<a;c++)l[c]=arguments[c];return e=r=A(this,t.call.apply(t,[this].concat(l))),r.emitter=(i=r.props.value,o=[],{on:function(t){o.push(t)},off:function(t){o=o.filter(function(e){return e!==t})},get:function(){return i},set:function(t,e){i=t,o.forEach(function(t){return t(i,e)})}}),A(r,e)}return l(n,t),n.prototype.getChildContext=function(){var t;return(t={})[u]=this.emitter,t},n.prototype.componentWillReceiveProps=function(t){if(this.props.value!==t.value){var n=this.props.value,r=t.value,i=void 0;((o=n)===(a=r)?0!==o||1/o==1/a:o!=o&&a!=a)?i=0:(i="function"==typeof e?e(n,r):c,0!=(i|=0)&&this.emitter.set(t.value,i))}var o,a},n.prototype.render=function(){return this.props.children},n}(r.Component);g.childContextTypes=((n={})[u]=i.default.object.isRequired,n);var h=function(e){function n(){var t,r;s(this,n);for(var i=arguments.length,o=Array(i),a=0;a<i;a++)o[a]=arguments[a];return t=r=A(this,e.call.apply(e,[this].concat(o))),r.state={value:r.getValue()},r.onUpdate=function(t,e){0!=((0|r.observedBits)&e)&&r.setState({value:r.getValue()})},A(r,t)}return l(n,e),n.prototype.componentWillReceiveProps=function(t){var e=t.observedBits;this.observedBits=null==e?c:e},n.prototype.componentDidMount=function(){this.context[u]&&this.context[u].on(this.onUpdate);var t=this.props.observedBits;this.observedBits=null==t?c:t},n.prototype.componentWillUnmount=function(){this.context[u]&&this.context[u].off(this.onUpdate)},n.prototype.getValue=function(){return this.context[u]?this.context[u].get():t},n.prototype.render=function(){return(t=this.props.children,Array.isArray(t)?t[0]:t)(this.state.value);var t},n}(r.Component);return h.contextTypes=((a={})[u]=i.default.object,a),{Provider:g,Consumer:h}},t.exports=e.default},function(t,e,n){"use strict";var r=n(1148);function i(){}function o(){}o.resetWarningCache=i,t.exports=function(){function t(t,e,n,i,o,a){if(a!==r){var s=new Error("Calling PropTypes validators directly is not supported by the `prop-types` package. Use PropTypes.checkPropTypes() to call them. Read more at http://fb.me/use-check-prop-types");throw s.name="Invariant Violation",s}}function e(){return t}t.isRequired=t;var n={array:t,bool:t,func:t,number:t,object:t,string:t,symbol:t,any:t,arrayOf:e,element:t,elementType:t,instanceOf:e,node:t,objectOf:e,oneOf:e,oneOfType:e,shape:e,exact:e,checkPropTypes:o,resetWarningCache:i};return n.PropTypes=n,n}},function(t,e,n){"use strict";t.exports="SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED"},function(t,e,n){"use strict";(function(e){var n="__global_unique_id__";t.exports=function(){return e[n]=(e[n]||0)+1}}).call(this,n(37))},function(t,e,n){"use strict";var r=n(1151);t.exports=r},function(t,e,n){"use strict";function r(t){return function(){return t}}var i=function(){};i.thatReturns=r,i.thatReturnsFalse=r(!1),i.thatReturnsTrue=r(!0),i.thatReturnsNull=r(null),i.thatReturnsThis=function(){return this},i.thatReturnsArgument=function(t){return t},t.exports=i},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;!function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}e.default=t}(n(1));var r=s(n(1153)),i=s(n(1155)),o=s(n(0)),a=s(n(309));n(622);function s(t){return t&&t.__esModule?t:{default:t}}function A(){return(A=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var l=function(t,e){return t&&e&&e.split(" ").forEach(function(e){return(0,r.default)(t,e)})},c=function(t,e){return t&&e&&e.split(" ").forEach(function(e){return(0,i.default)(t,e)})},u=function(t){var e,n;function r(){for(var e,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(e=t.call.apply(t,[this].concat(r))||this).onEnter=function(t,n){var r=e.getClassNames(n?"appear":"enter").className;e.removeClasses(t,"exit"),l(t,r),e.props.onEnter&&e.props.onEnter(t,n)},e.onEntering=function(t,n){var r=e.getClassNames(n?"appear":"enter").activeClassName;e.reflowAndAddClass(t,r),e.props.onEntering&&e.props.onEntering(t,n)},e.onEntered=function(t,n){var r=e.getClassNames("appear").doneClassName,i=e.getClassNames("enter").doneClassName,o=n?r+" "+i:i;e.removeClasses(t,n?"appear":"enter"),l(t,o),e.props.onEntered&&e.props.onEntered(t,n)},e.onExit=function(t){var n=e.getClassNames("exit").className;e.removeClasses(t,"appear"),e.removeClasses(t,"enter"),l(t,n),e.props.onExit&&e.props.onExit(t)},e.onExiting=function(t){var n=e.getClassNames("exit").activeClassName;e.reflowAndAddClass(t,n),e.props.onExiting&&e.props.onExiting(t)},e.onExited=function(t){var n=e.getClassNames("exit").doneClassName;e.removeClasses(t,"exit"),l(t,n),e.props.onExited&&e.props.onExited(t)},e.getClassNames=function(t){var n=e.props.classNames,r="string"==typeof n,i=r?(r&&n?n+"-":"")+t:n[t];return{className:i,activeClassName:r?i+"-active":n[t+"Active"],doneClassName:r?i+"-done":n[t+"Done"]}},e}n=t,(e=r).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n;var i=r.prototype;return i.removeClasses=function(t,e){var n=this.getClassNames(e),r=n.className,i=n.activeClassName,o=n.doneClassName;r&&c(t,r),i&&c(t,i),o&&c(t,o)},i.reflowAndAddClass=function(t,e){e&&(t&&t.scrollTop,l(t,e))},i.render=function(){var t=A({},this.props);return delete t.classNames,o.default.createElement(a.default,A({},t,{onEnter:this.onEnter,onEntered:this.onEntered,onEntering:this.onEntering,onExit:this.onExit,onExiting:this.onExiting,onExited:this.onExited}))},r}(o.default.Component);u.defaultProps={classNames:""},u.propTypes={};var g=u;e.default=g,t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t,e){t.classList?t.classList.add(e):(0,i.default)(t,e)||("string"==typeof t.className?t.className=t.className+" "+e:t.setAttribute("class",(t.className&&t.className.baseVal||"")+" "+e))};var i=r(n(1154));t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t,e){return t.classList?!!e&&t.classList.contains(e):-1!==(" "+(t.className.baseVal||t.className)+" ").indexOf(" "+e+" ")},t.exports=e.default},function(t,e,n){"use strict";function r(t,e){return t.replace(new RegExp("(^|\\s)"+e+"(?:\\s|$)","g"),"$1").replace(/\s+/g," ").replace(/^\s*|\s*$/g,"")}t.exports=function(t,e){t.classList?t.classList.remove(e):"string"==typeof t.className?t.className=r(t.className,e):t.setAttribute("class",r(t.className&&t.className.baseVal||"",e))}},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;a(n(1));var r=a(n(0)),i=n(16),o=a(n(623));function a(t){return t&&t.__esModule?t:{default:t}}var s=function(t){var e,n;function a(){for(var e,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(e=t.call.apply(t,[this].concat(r))||this).handleEnter=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return e.handleLifecycle("onEnter",0,n)},e.handleEntering=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return e.handleLifecycle("onEntering",0,n)},e.handleEntered=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return e.handleLifecycle("onEntered",0,n)},e.handleExit=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return e.handleLifecycle("onExit",1,n)},e.handleExiting=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return e.handleLifecycle("onExiting",1,n)},e.handleExited=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return e.handleLifecycle("onExited",1,n)},e}n=t,(e=a).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n;var s=a.prototype;return s.handleLifecycle=function(t,e,n){var o,a=this.props.children,s=r.default.Children.toArray(a)[e];s.props[t]&&(o=s.props)[t].apply(o,n),this.props[t]&&this.props[t]((0,i.findDOMNode)(this))},s.render=function(){var t=this.props,e=t.children,n=t.in,i=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);return i}(t,["children","in"]),a=r.default.Children.toArray(e),s=a[0],A=a[1];return delete i.onEnter,delete i.onEntering,delete i.onEntered,delete i.onExit,delete i.onExiting,delete i.onExited,r.default.createElement(o.default,i,n?r.default.cloneElement(s,{key:"first",onEnter:this.handleEnter,onEntering:this.handleEntering,onEntered:this.handleEntered}):r.default.cloneElement(A,{key:"second",onEnter:this.handleExit,onEntering:this.handleExiting,onEntered:this.handleExited}))},a}(r.default.Component);s.propTypes={};var A=s;e.default=A,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.getChildMapping=i,e.mergeChildMappings=o,e.getInitialChildMapping=function(t,e){return i(t.children,function(n){return(0,r.cloneElement)(n,{onExited:e.bind(null,n),in:!0,appear:a(n,"appear",t),enter:a(n,"enter",t),exit:a(n,"exit",t)})})},e.getNextChildMapping=function(t,e,n){var s=i(t.children),A=o(e,s);return Object.keys(A).forEach(function(i){var o=A[i];if((0,r.isValidElement)(o)){var l=i in e,c=i in s,u=e[i],g=(0,r.isValidElement)(u)&&!u.props.in;!c||l&&!g?c||!l||g?c&&l&&(0,r.isValidElement)(u)&&(A[i]=(0,r.cloneElement)(o,{onExited:n.bind(null,o),in:u.props.in,exit:a(o,"exit",t),enter:a(o,"enter",t)})):A[i]=(0,r.cloneElement)(o,{in:!1}):A[i]=(0,r.cloneElement)(o,{onExited:n.bind(null,o),in:!0,exit:a(o,"exit",t),enter:a(o,"enter",t)})}}),A};var r=n(0);function i(t,e){var n=Object.create(null);return t&&r.Children.map(t,function(t){return t}).forEach(function(t){n[t.key]=function(t){return e&&(0,r.isValidElement)(t)?e(t):t}(t)}),n}function o(t,e){function n(n){return n in e?e[n]:t[n]}t=t||{},e=e||{};var r,i=Object.create(null),o=[];for(var a in t)a in e?o.length&&(i[a]=o,o=[]):o.push(a);var s={};for(var A in e){if(i[A])for(r=0;r<i[A].length;r++){var l=i[A][r];s[i[A][r]]=n(l)}s[A]=n(A)}for(r=0;r<o.length;r++)s[o[r]]=n(o[r]);return s}function a(t,e,n){return null!=n[e]?n[e]:t.props[e]}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(242).Classes.getClassNamespace();e.MULTISELECT=r+"-multi-select",e.MULTISELECT_POPOVER=e.MULTISELECT+"-popover",e.OMNIBAR=r+"-omnibar",e.OMNIBAR_OVERLAY=e.OMNIBAR+"-overlay",e.SELECT=r+"-select",e.SELECT_POPOVER=e.SELECT+"-popover"},function(t,e,n){"use strict";
/*!
 * Copyright 2017 Palantir Technologies, Inc. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */Object.defineProperty(e,"__esModule",{value:!0}),e.renderFilteredItems=function(t,e,n){if(0===t.query.length&&void 0!==n)return n;var r=t.filteredItems.map(t.renderItem).filter(function(t){return null!=t});return r.length>0?r:e}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(242);e.executeItemsEqual=function(t,e,n){return void 0===t||null==e||null==n?e===n:r.Utils.isFunction(t)?t(e,n):e[t]===n[t]}},function(t,e,n){"use strict";
/*!
 * Copyright 2018 Palantir Technologies, Inc. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */function r(t){if(null==t)return!1;var e=Object.keys(t);return 1===e.length&&"__blueprintCreateNewItemBrand"===e[0]&&"blueprint-create-new-item"===t.__blueprintCreateNewItemBrand}Object.defineProperty(e,"__esModule",{value:!0}),e.getCreateNewItem=function(){return{__blueprintCreateNewItemBrand:"blueprint-create-new-item"}},e.isCreateNewItem=r,e.getActiveItem=function(t){return null==t||r(t)?null:t}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(2),i=r.__importStar(n(0)),o=n(242),a=n(624),s=function(t){function e(e,n){var r=t.call(this,e,n)||this;r.refHandlers={itemsParent:function(t){return r.itemsParentRef=t}},r.shouldCheckActiveItemInViewport=!1,r.expectedNextActiveItem=null,r.renderItemList=function(t){var e=r.props,n=e.initialContent,s=e.noResults,A=r.isCreateItemRendered()?null:s,l=a.renderFilteredItems(t,A,n),c=r.isCreateItemRendered()?r.renderCreateItemMenuItem(r.state.query):null;return null==l&&null==c?null:i.createElement(o.Menu,{ulRef:t.itemsParentRef},l,c)},r.renderItem=function(t,e){if(!0!==r.props.disabled){var n=r.state,i=n.activeItem,o=n.query,s=r.state.filteredItems.indexOf(t)>=0,A={active:a.executeItemsEqual(r.props.itemsEqual,a.getActiveItem(i),t),disabled:u(t,e,r.props.itemDisabled),matchesPredicate:s};return r.props.itemRenderer(t,{handleClick:function(e){return r.handleItemSelect(t,e)},index:e,modifiers:A,query:o})}return null},r.renderCreateItemMenuItem=function(t){var e=r.state.activeItem,n=a.isCreateNewItem(e);return o.Utils.safeInvoke(r.props.createNewItemRenderer,t,n,function(e){r.handleItemCreate(t,e)})},r.handleItemCreate=function(t,e){var n=o.Utils.safeInvoke(r.props.createNewItemFromQuery,t);null!=n&&(o.Utils.safeInvoke(r.props.onItemSelect,n,e),r.setQuery("",!0))},r.handleItemSelect=function(t,e){r.setActiveItem(t),o.Utils.safeInvoke(r.props.onItemSelect,t,e),r.props.resetOnSelect&&r.setQuery("",!0)},r.handlePaste=function(t){for(var e,n=r.props,i=n.createNewItemFromQuery,a=n.onItemsPaste,s=[],A=[],c=0,u=t;c<u.length;c++){var g=u[c],h=l(g,r.props);if(void 0!==h)e=h,A.push(h);else if(r.canCreateItems()){var d=o.Utils.safeInvoke(i,g);void 0!==d&&A.push(d)}else s.push(g)}r.setQuery(s.join(", "),!1),void 0!==e&&r.setActiveItem(e),o.Utils.safeInvoke(a,A)},r.handleKeyDown=function(t){var e=t.keyCode;if(e===o.Keys.ARROW_UP||e===o.Keys.ARROW_DOWN){t.preventDefault();var n=r.getNextActiveItem(e===o.Keys.ARROW_UP?-1:1);null!=n&&r.setActiveItem(n)}o.Utils.safeInvoke(r.props.onKeyDown,t)},r.handleKeyUp=function(t){var e=r.props.onKeyUp,n=r.state.activeItem;t.keyCode===o.Keys.ENTER&&(t.preventDefault(),null==n||a.isCreateNewItem(n)?r.handleItemCreate(r.state.query,t):r.handleItemSelect(n,t)),o.Utils.safeInvoke(e,t)},r.handleQueryChange=function(t){var e=null==t?"":t.target.value;r.setQuery(e),o.Utils.safeInvoke(r.props.onQueryChange,e,t)};var s=e.query,A=void 0===s?"":s,h=o.Utils.safeInvoke(e.createNewItemFromQuery,A),d=c(A,e);return r.state={activeItem:void 0!==r.props.activeItem?r.props.activeItem:g(d,e.itemDisabled),createNewItem:h,filteredItems:d,query:A},r}return r.__extends(e,t),e.ofType=function(){return e},e.prototype.render=function(){var t=this.props,e=t.className,n=t.items,i=t.renderer,o=t.itemListRenderer,a=void 0===o?this.renderItemList:o,s=this.state,A=(s.createNewItem,r.__rest(s,["createNewItem"]));return i(r.__assign({},A,{className:e,handleItemSelect:this.handleItemSelect,handleKeyDown:this.handleKeyDown,handleKeyUp:this.handleKeyUp,handlePaste:this.handlePaste,handleQueryChange:this.handleQueryChange,itemList:a(r.__assign({},A,{items:n,itemsParentRef:this.refHandlers.itemsParent,renderItem:this.renderItem}))}))},e.prototype.componentWillReceiveProps=function(t){void 0!==t.activeItem&&(this.shouldCheckActiveItemInViewport=!0,this.setState({activeItem:t.activeItem})),null!=t.query&&t.query!==this.props.query&&this.setQuery(t.query,t.resetOnQuery,t)},e.prototype.componentDidUpdate=function(t){var e=this;o.Utils.shallowCompareKeys(this.props,t,{include:["items","itemListPredicate","itemPredicate"]})||this.setQuery(this.state.query),this.shouldCheckActiveItemInViewport&&(requestAnimationFrame(function(){return e.scrollActiveItemIntoView()}),this.shouldCheckActiveItemInViewport=!1)},e.prototype.scrollActiveItemIntoView=function(){var t=!1!==this.props.scrollToActiveItem,e=!a.executeItemsEqual(this.props.itemsEqual,a.getActiveItem(this.expectedNextActiveItem),a.getActiveItem(this.props.activeItem));if(this.expectedNextActiveItem=null,t||!e){var n=this.getActiveElement();if(null!=this.itemsParentRef&&null!=n){var r=n.offsetTop,i=n.offsetHeight,o=this.itemsParentRef,s=o.offsetTop,A=o.scrollTop,l=o.clientHeight,c=this.getItemsParentPadding(),u=c.paddingTop,g=r+i+c.paddingBottom-s,h=r-u-s;g>=A+l?this.itemsParentRef.scrollTop=g+i-l:h<=A&&(this.itemsParentRef.scrollTop=h-i)}}},e.prototype.setQuery=function(t,e,n){void 0===e&&(e=this.props.resetOnQuery),void 0===n&&(n=this.props);var r=n.createNewItemFromQuery;this.shouldCheckActiveItemInViewport=!0,t!==this.state.query&&o.Utils.safeInvoke(n.onQueryChange,t);var i=c(t,n),s=null!=r&&""!==t?r(t):void 0;this.setState({createNewItem:s,filteredItems:i,query:t});var A=this.getActiveIndex(i);(e||A<0||u(a.getActiveItem(this.state.activeItem),A,n.itemDisabled))&&this.setActiveItem(g(i,n.itemDisabled))},e.prototype.getActiveElement=function(){var t=this.state.activeItem;if(null!=this.itemsParentRef){if(a.isCreateNewItem(t))return this.itemsParentRef.children.item(this.state.filteredItems.length);var e=this.getActiveIndex();return this.itemsParentRef.children.item(e)}},e.prototype.getActiveIndex=function(t){void 0===t&&(t=this.state.filteredItems);var e=this.state.activeItem;if(null==e||a.isCreateNewItem(e))return-1;for(var n=0;n<t.length;++n)if(a.executeItemsEqual(this.props.itemsEqual,t[n],e))return n;return-1},e.prototype.getItemsParentPadding=function(){var t=getComputedStyle(this.itemsParentRef),e=t.paddingTop;return{paddingBottom:A(t.paddingBottom),paddingTop:A(e)}},e.prototype.getNextActiveItem=function(t,e){if((void 0===e&&(e=this.getActiveIndex()),this.isCreateItemRendered())&&(0===e&&-1===t||e===this.state.filteredItems.length-1&&1===t))return a.getCreateNewItem();return g(this.state.filteredItems,this.props.itemDisabled,t,e)},e.prototype.setActiveItem=function(t){this.expectedNextActiveItem=t,void 0===this.props.activeItem&&(this.shouldCheckActiveItemInViewport=!0,this.setState({activeItem:t})),a.isCreateNewItem(t)?o.Utils.safeInvoke(this.props.onActiveItemChange,null,!0):o.Utils.safeInvoke(this.props.onActiveItemChange,t,!1)},e.prototype.isCreateItemRendered=function(){return this.canCreateItems()&&""!==this.state.query&&!this.wouldCreatedItemMatchSomeExistingItem()},e.prototype.canCreateItems=function(){return null!=this.props.createNewItemFromQuery&&null!=this.props.createNewItemRenderer},e.prototype.wouldCreatedItemMatchSomeExistingItem=function(){var t=this;return this.state.filteredItems.some(function(e){return a.executeItemsEqual(t.props.itemsEqual,e,t.state.createNewItem)})},e.displayName=o.DISPLAYNAME_PREFIX+".QueryList",e.defaultProps={disabled:!1,resetOnQuery:!0},e}(i.Component);function A(t){return null==t?0:parseInt(t.slice(0,-2),10)}function l(t,e){var n=e.items,r=e.itemPredicate;if(o.Utils.isFunction(r))for(var i=0;i<n.length;i++){var a=n[i];if(r(t,a,i,!0))return a}}function c(t,e){var n=e.items,r=e.itemPredicate,i=e.itemListPredicate;return o.Utils.isFunction(i)?i(t,n):o.Utils.isFunction(r)?n.filter(function(e,n){return r(t,e,n)}):n}function u(t,e,n){return null!=n&&null!=t&&(o.Utils.isFunction(n)?n(t,e):!!t[n])}function g(t,e,n,r){if(void 0===n&&(n=1),void 0===r&&(r=t.length-1),0===t.length)return null;var i,o,a,s=r,A=t.length-1;do{if(a=A,!u(t[s=(i=s+n)<(o=0)?a:i>a?o:i],s,e))return t[s]}while(s!==r&&-1!==r);return null}e.QueryList=s,e.getFirstEnabledItem=g},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(6);e.StatusToolbar=class extends r.Widget{constructor(t){super(),this.opts=t,this.addClass("maven_toolbar");const e=document.createElement("span");e.innerText="MavenWorks Dashboard Viewer",this.node.appendChild(e);const n=document.createElement("span");if(n.classList.add("kernel_status"),this.kernelStatus=document.createElement("code"),this.kernelStatus.innerText="Not Connected",n.appendChild(this.kernelStatus),this.node.appendChild(n),t.showOverlay){const t=this.overlay=document.createElement("div");t.classList.add("m-StatusToolbar-Overlay"),t.innerText="Initializing...",document.body.appendChild(t)}}setKernelLanguage(t){this.kernelStatus.innerText=t,null!=this.overlay&&(this.overlay.innerText=`Executing... (${t})`)}setKernelStatus(t){if(this.removeClass("state_kernel_busy"),this.removeClass("state_kernel_error"),null!=this.overlay)switch(t){case 1:break;case 2:this.overlay.innerText="Failed to Execute!",this.overlay.classList.add("m-StatusToolbar-overlay__error");break;case 0:document.body.removeChild(this.overlay)}switch(t){case 0:return;case 1:return void this.addClass("state_kernel_busy");case 2:this.addClass("state_kernel_error")}}}},function(t,e,n){"use strict";n.r(e),n.d(e,"MathJaxTypesetter",function(){return i});var r=n(9);class i{constructor(t){this._initPromise=new r.PromiseDelegate,this._initialized=!1,this._url=t.url,this._config=t.config}typeset(t){this._initialized||this._init(),this._initPromise.promise.then(()=>{MathJax.Hub.Queue(["Typeset",MathJax.Hub,t]);try{MathJax.Hub.Queue(["Require",MathJax.Ajax,"[MathJax]/extensions/TeX/AMSmath.js"],()=>{MathJax.InputJax.TeX.resetEquationNumbers()})}catch(t){console.error("Error queueing resetEquationNumbers:",t)}})}_init(){let t=document.getElementsByTagName("head")[0],e=document.createElement("script");e.type="text/javascript",e.src=`${this._url}?config=${this._config}&amp;delayStartupUntil=configured`,e.charset="utf-8",t.appendChild(e),e.addEventListener("load",()=>{this._onLoad()}),this._initialized=!0}_onLoad(){MathJax.Hub.Config({tex2jax:{inlineMath:[["$","$"],["\\(","\\)"]],displayMath:[["$$","$$"],["\\[","\\]"]],processEscapes:!0,processEnvironments:!0},displayAlign:"center",CommonHTML:{linebreaks:{automatic:!0}},"HTML-CSS":{availableFonts:[],imageFont:null,preferredFont:null,webFont:"STIX-Web",styles:{".MathJax_Display":{margin:0}},linebreaks:{automatic:!0}},skipStartupTypeset:!0}),MathJax.Hub.Configured(),this._initPromise.resolve(void 0)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(6),i=n(629),o=n(64),a=n(680),s=n(9),A=n(113),l=n(28),c=n(681),u=n(48);var g=u.JavascriptEvalPart.generateWrapper;const h=n(338),d=n(66);class f extends r.Widget{constructor({rendermime:t,factory:e,session:n,evaluator:g,baseUrl:d,baseViewUrl:p,externalPartRenderer:I}){super(),this.OnDirtySrc$=new o.Subject,this.isLoading=new l.AsyncTools.Mutex,this._isDirty=!1,this._shouldNotifyDirty=!1,this._uuid=s.UUID.uuid4(),this._localParts={},this.addClass("m-Dashboard"),this.id="dashboard-"+this._uuid,this.OnDirty=this.OnDirtySrc$.pipe(A.delayWhen(()=>o.from(this.isLoading.lock))),this._partUpdateHook=e.OnUpdated.pipe(A.bufferTime(500),A.filter(t=>t.length>0)).subscribe(t=>{if(!this.partManager.hasErrorParts)return;let e=!1;for(const[n,r]of this.partManager)if(r instanceof u.ErrorPart&&r.originalModel&&t.includes(r.originalModel.name)){e=!0;break}if(e){const t=a.DashboardSerializer.toJson(this);this.loadFromModel(t)}}),this.externalPartRenderer=null==I&&null!=t?new f.DefaultExternalPartRenderer(t):I,this.factory=new u.PartFactory(e),this.globals=new h.GlobalsService,this.bindings=new h.BindingsProvider(this.globals,g),this.partManager=new i.PartManager({session:n,rendermime:t,factory:this.factory,globals:this.globals,dashboardId:this.uuid,bindings:this.bindings,baseUrl:d,baseViewUrl:p}),this.layoutManager=new c.LayoutManager({getPartById:t=>{let e=this.partManager.getPartById(t);return null!=e?e:this.externalPartRenderer?this.externalPartRenderer.getPartById(t):null}},{keys:()=>this.factory.keys(),has:t=>this.factory.has(t),get:t=>this.partManager.addPart(t)}),o.merge(this.partManager.OnDirty.pipe(A.mapTo("PartManager")),this.layoutManager.OnDirty.pipe(A.mapTo("LayoutManager")),this.globals.OnDirty.pipe(A.mapTo("Globals"))).subscribe(t=>this.setDirty(t)),(this.layout=new r.BoxLayout).addWidget(this.layoutManager),this.update()}get localParts(){return this._localParts}get uuid(){return this._uuid}get isDirty(){return this._isDirty}get shouldNotifyDirty(){return this._shouldNotifyDirty}get externalParts(){return this.externalPartRenderer}setClean(){this._isDirty=!1,this._shouldNotifyDirty=!1,this.globals.setClean(),this.partManager.setClean(),this.layoutManager.setClean()}async loadFromModel(t){return this.loadFromModelWithOverrides(t,null)}async loadFromModelWithOverrides(t,e){await this.isLoading.aquire();try{this.clearDashboard();for(const e in t.localParts){const n=t.localParts[e];this._localParts[e]=n,this.factory.registerPart(n.name,g(n,"/"+n.name))}const{parts:n,metadata:r,layout:i,globals:o}=t;this.loadGlobals(o),this.applyGlobalOverrides(e),await this.loadParts(n,r),this.layoutManager.initLayout(i),this.setClean()}catch(t){this.clearDashboard(),console.error("[Dashboard] Failed to load from model"),console.error(t),d.HoverManager.Instance.openErrorDialog(t)}finally{this.isLoading.release()}}dispose(){this.isDisposed||(this.layoutManager.dispose(),this.partManager.dispose(),null!=this.externalPartRenderer&&this.externalPartRenderer.dispose(),this.globals.dispose(),this.bindings.dispose(),this.OnDirtySrc$.complete(),this._partUpdateHook.unsubscribe(),super.dispose())}setDirty(t){this._shouldNotifyDirty||("Globals"!==t&&(this._shouldNotifyDirty=!0),this._isDirty||(this._isDirty=!0,this.OnDirtySrc$.next()))}clearDashboard(){this.globals.clearGlobals(),this.partManager.clearParts();for(const t in this.localParts){const e=this.localParts[t];this.factory.unregisterPart(e.name)}this._localParts={},this.layoutManager.initLayout(a.DashboardSerializer.DEFAULT_DASHBOARD.layout)}applyGlobalOverrides(t){if(null!=t){for(let[e,n]of Object.entries(t))this.globals.set(e,n);this.globals.setClean()}}loadGlobals(t){if(null!=t){for(const e of t){const t=l.Types.findType(e.type)||l.Types.Any;let n;n=e.value&&e.value.typeName?e.value:{typeName:e.type,value:e.value},this.globals.addGlobal(e.name,t,l.Converters.deserialize(n))}this.globals.setClean()}}async loadParts(t,e){const n=[];for(const r in t){const i=t[r],o=i[u.PartSerializer.MIMETYPE];null!=o?await this.partManager.addPart(o.name,{name:o.name,options:o.options,id:r}):null!=this.externalPartRenderer&&n.push(this.externalPartRenderer.renderModel(i,(e||{})[r],r))}await Promise.all(n)}}e.Dashboard=f,function(t){t.DefaultExternalPartRenderer=class{constructor(t){this.externalParts=new Map,this.externalData=new Map,this._isDisposed=!1,this.rendermime=t}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this.externalData.clear(),this.externalParts.forEach(t=>t.dispose()),this.externalParts.clear(),this._isDisposed=!0)}getPartById(t){return this.externalParts.get(t)||null}getAllPartIds(){return this.externalParts.keys()}getMetadata(){return[]}async renderModel(t,e,n){this.externalData.set(n,[t,e||null]);const r=this.rendermime.preferredMimeType(t,"any"),i=this.rendermime.createRenderer(r||"text/plain"),o=this.rendermime.createModel({data:t,metadata:e,trusted:!0});this.externalParts.set(n,i),await i.renderModel(o)}serializePart(t){return this.externalData.get(t)}}}(f=e.Dashboard||(e.Dashboard={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(50);!function(t){const e=["mouseenter","mouseover","mousemove","mousedown","mouseup","pointerdown","pointerenter","pointerdown","pointermove","pointerup","pointercancel","pointerout","pointerleave","gotpointercapture","lostpointercapture","auxclick","click","dblclick","contextmenu","wheel","mouseleave","mouseout","select","pointerlockchange","pointerlockerror"],n=["dragstart","drag","dragenter","dragexit","dragleave","dragover","drop","dragend"],i=["keydown","keypress","keyup"],o=["focus","blur","focusin","focusout"];function a(t,e,n,r,i=!1){const o=new e(t,Object.assign({},function(t){const e={};let n=t;do{for(const r of Object.getOwnPropertyNames(n))e[r]=t[r]}while(n=Object.getPrototypeOf(n));return e}(n),i?function(t,e){const n=e.getBoundingClientRect();return{clientX:t.clientX+n.left,clientY:t.clientY+n.top}}(n,r):{}));r.dispatchEvent(o)||n.preventDefault()}t.copyStylesheets=function(t,e){const n=t.querySelectorAll("style, link"),r=t.location.href,i=e.createElement("base");i.href=r,e.head.appendChild(i);for(const t of n.values()){if("LINK"===t.tagName&&"stylesheet"===t.rel){const n=e.createElement("link");n.rel="stylesheet",n.type=t.type,n.href=t.href,e.head.appendChild(n)}if("STYLE"===t.tagName&&"text/css"===t.type){const n=e.createElement("style");n.innerHTML=t.innerHTML,e.head.appendChild(n)}}},t.bubbleEvents=function(t,s){const A=[];for(const n of e){function l(t){a(n,MouseEvent,t,s,!0)}t.addEventListener(n,l),A.push({ev:n,handler:l})}for(const e of n){function c(t){a(e,DragEvent,t,s,!0)}t.addEventListener(e,c),A.push({ev:e,handler:c})}for(const e of i){function u(t){a(e,KeyboardEvent,t,s)}t.addEventListener(e,u),A.push({ev:e,handler:u})}for(const e of o){function g(t){a(e,FocusEvent,t,s)}t.addEventListener(e,g),A.push({ev:e,handler:g})}return new r.DisposableDelegate(()=>{for(const e of A)t.removeEventListener(e.ev,e.handler)})},t.DisableFramePointerEvents=function(){for(const t of Array.from(document.getElementsByTagName("iframe")))t.style.pointerEvents="none"},t.EnableFramePointerEvents=function(){for(const t of Array.from(document.getElementsByTagName("iframe")))t.style.pointerEvents=null}}(e.FrameTools||(e.FrameTools={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){t.map=function*(t,e){for(const n of t)yield e(n)},t.filter=function*(t,e){for(const n of t){let t=!0;try{t=!!e(n)}catch(t){throw new Error("IterationError: Unhandled exception in predicate.\n"+t)}t&&(yield n)}},t.all=function(t){return Promise.all(Array.from(t))},t.dfs_iter=function*t(e,n){for(const i of e){const e=n(i);null==e||(r=e,Array.isArray(r)&&0===r.length)||(yield*t(e,n)),yield i}var r},t.merge=function*(...t){for(const e of t)yield*e},t.mergeUnique=function*(...t){const e=new Set;for(let n=0;n<t.length;n++)for(const r of t[n])e.add(r);yield*e.values(),e.clear()}}(e.IterTools||(e.IterTools={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){let e;t.Clamp=function(t,e,n){return Math.floor(Math.min(Math.max(t,e),n))},t.ClampRectToBounds=function(n,r,i){const o=[t.Clamp(n[0],0,i[0]),t.Clamp(n[1],0,i[1])],a=e.Sub(i,o),s=[t.Clamp(r[0],50,Math.max(50,a[0])),t.Clamp(r[1],50,Math.max(50,a[1]))];return[...o,...s]},function(t){function e(t,e){return t[0]*e[0]+t[1]*e[1]}t.Add=function(t,e){return[t[0]+e[0],t[1]+e[1]]},t.Sub=function(t,e){return[t[0]-e[0],t[1]-e[1]]},t.Dot=e,t.Norm=function(t){return Math.abs(t[0])+Math.abs(t[1])},t.Magnitude=function(t){return Math.sqrt(e(t,t))}}(e=t.Vec2||(t.Vec2={}))}(e.MathTools||(e.MathTools={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9);function i(t,e){const n=r.UUID.uuid4();return performance.mark(n+"start"),console.time(t+" [guid:"+n+"]"),Promise.resolve().then(()=>e()).then(t=>({error:null,result:t,context:null})).catch(t=>({error:t,result:null,context:null})).then(e=>(console.timeEnd(t+" [guid:"+n+"]"),performance.mark(n+"end"),performance.measure(t,n+"start",n+"end"),performance.clearMarks(n+"start"),performance.clearMarks(n+"end"),e))}e.StartTimingWithContext=i,e.StartTiming=function(t,e){return i(t,e).then(t=>null!=t.error?Promise.reject(t.error):t.result)},e.StartTimingSync=function(t,e){const n=r.UUID.uuid4();let i,o;performance.mark(n+"start"),console.time(t+" [guid:"+n+"]");try{i=e()}catch(t){o=t}finally{console.timeEnd(t+" [guid:"+n+"]"),performance.mark(n+"end"),performance.measure(t,n+"start",n+"end"),performance.clearMarks(n+"start"),performance.clearMarks(n+"end")}if(null!=o)throw o;return i},e.StartTimingAsync=async function(t,e){const n=r.UUID.uuid4();let i,o;performance.mark(n+"start"),console.time(t+" [guid:"+n+"]");try{i=await e()}catch(t){o=t}finally{console.timeEnd(t+" [guid:"+n+"]"),performance.mark(n+"end"),performance.measure(t,n+"start",n+"end"),performance.clearMarks(n+"start"),performance.clearMarks(n+"end")}if(null!=o)throw o;return i}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(9).UUID.uuid4;e.CancelError=class extends Error{constructor(t){super(t)}};e.CancelToken=class{constructor(){this.promise=new Promise(t=>this.resolve=t),this.guid=r(),this.isCanceled=!1}onCancel(t){this.promise.then(t)}cancel(){this.isCanceled||(this.isCanceled=!0,this.resolve())}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(220),i=n(311),o=n(631);class a{constructor(t){this._colorRgba=[255,255,255,1],this._colorString="white",this.color=t}get color(){return this._colorString}set color(t){if("string"==typeof t)this._colorRgba=o.get(t.toLowerCase()).value,this._colorString=t;else if(t instanceof Array){this._colorRgba=t;let e=o.to.keyword(t);(t.length>4||1===t[3])&&null!=e?this._colorString=e:this._colorString=o.to.hex(t)}}get rgb(){return o.to.rgb(this._colorRgba)}get hex(){return o.to.hex(this._colorRgba)}get hsl(){return o.to.hsl(this._colorRgba)}}e.Color=a;class s extends i.Converter{isValid(t){return"string"==typeof t&&null!=o.get(t)||t instanceof a}canConvertFrom(t){return"string"==typeof t}convertFrom(t,e){return new a(t)}canStringify(){return!0}toString(t){return t.color}tryFromString(t){return null==o.get(t.toLocaleLowerCase())?void 0:new a(t.toLocaleLowerCase())}serialize(t){return t.color}deserialize(t){return new a(t)}inferInstanceOf(t){return t instanceof a?1:-1}}s.type=r.Types.Color,e.ColorConverter=s,i.Converters.registerConverter(s)},function(t,e,n){"use strict";var r=n(1173),i=Array.prototype.concat,o=Array.prototype.slice,a=t.exports=function(t){for(var e=[],n=0,a=t.length;n<a;n++){var s=t[n];r(s)?e=i.call(e,o.call(s)):e.push(s)}return e};a.wrap=function(t){return function(){return t(a(arguments))}}},function(t,e){t.exports=function(t){return!(!t||"string"==typeof t)&&(t instanceof Array||Array.isArray(t)||t.length>=0&&(t.splice instanceof Function||Object.getOwnPropertyDescriptor(t,t.length-1)&&"String"!==t.constructor.name))}},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0});const r=n(108),i=n(220),o=n(311);let a={};function s(t,e,n){a[t]={serialize:e,deserialize:n}}e.registerType=s;s("Table",function(t,e){return o.Converters.serialize(t,i.Types.Table)},function(t,e){if(!t.hasOwnProperty("value")){console.warn("DEPRECATION WARNING: This dashboard has data in an old format");const e=function(t){return{value:{data:t.data.map(t=>({value:t,typeName:"Any"})),name:t.name,children:(t.children||[]).map(e)},typeName:"Row"}};return o.Converters.deserialize({typeName:"Table",value:{cols:t.cols,rows:t.rows.map(e)}})}return o.Converters.deserialize(t)});s("CachedTable",function(t,e){return o.Converters.serialize(t,i.Types.findType("CachedTable"))},function(t,e){return o.Converters.deserialize(t)});if(s(Date.name,function(t,e){return o.Converters.serialize(t,i.Types.Date)},function(t,e){return new Date(t.date)}),void 0!==t){let e=function(t,e){return{b64:t.toString("base64"),typeName:Uint8Array.name}},n=function(t,e){throw new Error("Buffer is not a valid serialized type and has no deserializer")};s(t.name,e,n)}let A="undefined"!=typeof window?window.btoa:null,l="undefined"!=typeof window?window.atob:null;A||(A=function(e){return new t(e).toString("base64")},l=function(e){return new t(e,"base64").toString()});s(Uint8Array.name,function(t,e){let n=t;if(null==A)throw new Error("Not supported");return{b64:A(String.fromCharCode.apply(null,[...n]))}},function(t,e){if(null==l)throw new Error("Not supported");return new Uint8Array(l(t.b64).split("").map(t=>t.charCodeAt(0)))});let c=function(t,e){let n=t,r=n.stack;return{typeName:"Error",message:n.message,stack:r}},u=function(t,e){let n=new Error(t.message);return n.stack=t.stack,n};e.serialize=function t(e){if(null==e)return null;if("boolean"==typeof e||"number"==typeof e||"string"==typeof e)return e;if(Array.isArray(e))return e.map(e=>t(e));let n=e.constructor.name;if("Object"===n)return r.mapValues(e,e=>t(e));if(e instanceof Error)return c(e);let s=o.Converters.inferType(e);if(s!==i.Types.Any)return o.Converters.serialize(e,s);if(a.hasOwnProperty(n)){let r=a[n].serialize(e,t);return r.typeName||(r.typeName=n),r.deprecatedSerialization||(r.deprecatedSerialization=!0),r}throw Error("Attempted to serialize unsupported type: "+e.constructor.name)},e.deserialize=function t(e){if(null==e)return null;if("boolean"==typeof e||"number"==typeof e||"string"==typeof e)return e;if(Array.isArray(e))return e.map(e=>t(e));let n=e.typeName;if(void 0===n)return r.mapValues(e,e=>t(e));if("Error"===n)return u(e);if(!e.hasOwnProperty("deprecatedSerialization")&&e.hasOwnProperty("value"))return o.Converters.deserialize(e);if(a.hasOwnProperty(n))return a[n].deserialize(e,t);throw Error("Attempted to deserialize unsupported type")}}).call(this,n(42).Buffer)},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(630),i=n(9);class o{constructor(){this.store=new Map,this.locks=new Map}has(t){return this.store.has(t)}get(t,e,n){const r=this.getManagerForKey(t);return r.hasReadLock,e&&this.checkLock(r,n,!1),this.store.get(t)}put(t,e,n,r){const i=this.getManagerForKey(t);n&&this.checkLock(i,r,!0),this.store.set(t,e)}delete(t,e,n){const r=this.getManagerForKey(t);e&&this.checkLock(r,n,!0),this.store.delete(t),this.locks.delete(t)}clear(){this.store.clear(),this.locks.clear()}async lock(t,e){return this.getManagerForKey(t).acquireLock(e)}unlock(t,e){const n=this.getManagerForKey(t).findLock(e);null!=n?n.release():console.warn("[Cache]","Attempted to release an invalid lock")}getLock(t,e){return this.getManagerForKey(t).findLock(e)}checkLock(t,e,n){const r=t.findLock(e);(null==r||r.isFree||r.isWrite!==n)&&console.error("[Cache]","Attempted to perform blocking operation without a lock")}getManagerForKey(t){if(this.locks.has(t))return this.locks.get(t);{const e=new o.LockManager;return this.locks.set(t,e),e}}}e.Cache=o,function(t){t.LockManager=class{constructor(){this.held=[],this.waiting=[],this.nextLockId=1}get hasWriteLock(){return this.held.concat(this.waiting).some(t=>t.isWrite)}get hasReadLock(){return this.held.length+this.waiting.length!==0}findLock(t){return this.held.find(e=>e.lockId===t)||this.waiting.find(e=>e.lockId===t)}async acquireLock(t){let n=t&&0===this.held.length,r=!t&&this.held.every(t=>!t.isWrite);if(n||r){let n=new e(this.nextLockId++,t);return n.released.then(()=>this.onReleaseLock(n)),this.held.push(n),await n.aquire(),n}{let n=new e(this.nextLockId++,t);return n.released.then(()=>this.onReleaseLock(n)),this.waiting.push(n),await n.aquired,n}}async onReleaseLock(t){if(this.held=this.held.filter(e=>e!==t),0===this.waiting.length)return;let e=this.waiting.find(t=>t.isWrite);if(e)await e.aquire(),this.waiting=this.waiting.filter(t=>t!==e),this.held.push(e);else{for(let t of this.waiting)await t.aquire(),this.held.push(t);this.waiting=[]}}};class e extends r.AsyncTools.Mutex{constructor(t,e){super(),this._aquired=new i.PromiseDelegate,this._released=new i.PromiseDelegate,this.lockId=t,this.isWrite=e}get aquired(){return this._aquired.promise}get released(){return this._released.promise}async aquire(){await super.aquire(),this.lock.then(()=>{this._released.resolve(),this._released=new i.PromiseDelegate}),this._aquired.resolve(),this._aquired=new i.PromiseDelegate}}t.CacheLock=e}(o=e.Cache||(e.Cache={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0);e.ListBox=({items:t,selectedKey:e,isEditing:n,onEdit:i,onSelect:o})=>{const a=t.find(t=>t.key===e),[s,A]=r.useState(a?a.label:"");return n||""===s||A(""),r.createElement("ul",{className:"m-ListBox"},t.map(t=>(function(t){let a="m-ListBox-item",l=t.label;return e&&t.key===e&&(a+=" m-selected",n&&(a+=" m-editing",l=r.createElement("input",{type:"text",autoFocus:!0,value:s,onChange:t=>A(t.target.value),onBlur:()=>{i.call(void 0,t.key,s)},onKeyDown:e=>{"Enter"!==e.key&&"Escape"!==e.key||("Escape"===e.key&&(A(t.label),o.call(void 0,null)),e.persist(),setTimeout(()=>e.target.blur()))}}))),r.createElement("li",{key:t.key,"data-list-key":t.key,className:a,onMouseDown:()=>{t.key!==e&&o.call(void 0,t.key)}},l)})(t)))}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0);function i({isCollapsed:t,onCollapse:e}){return r.createElement("button",{className:"m-FlippyTriangle",onClick:()=>e.call(void 0,!t),"data-collapsed":""+t})}function o({prop:t,onCollapse:e}){return r.createElement("span",{className:"m-PropertiesEditor-PropertyLabel\r\n                m-PropertiesEditor-PropertyGroup"},r.createElement(i,{isCollapsed:t.isCollapsed,onCollapse:n=>e.call(void 0,t.key,n)}),t.name)}function a({prop:t,children:e}){return r.createElement(r.Fragment,null,r.createElement("span",{className:"m-PropertiesEditor-PropertyLabel",title:(n=t.metadata,`${n.prettyName}: ${n.documentation||""}\n    Type: ${n.type.serializableName}\n    Default: ${n.default}`)},t.name),e);var n}var s;e.PropertiesEditor=function({properties:t,renderEditor:e,groupOnDots:n=!0}){const[i,A]=r.useState([]),l=Array.from(t).sort((t,e)=>t[1].prettyName.localeCompare(e[1].prettyName));let c;return c=n?s.BuildTree(l,i):l.map(t=>({name:t[0],key:t[0],metadata:t[1],indent:0,isParent:!1,isCollapsed:!1})),r.createElement("div",{className:"m-PropertiesEditor"},c.map(t=>r.createElement("div",{className:"m-PropertiesEditor-Property",key:t.key,style:{"--indent":""+t.indent}},t.isParent?r.createElement(o,{prop:t,onCollapse:(t,e)=>{A(e?[...i,t]:i.filter(e=>e!==t))}}):r.createElement(a,{prop:t},e.call(void 0,t.key,t.metadata)))))},function(t){function*e(t,e,n){const r=t[1].prettyName,i=r.split(".");let o=0;for(let t=0;t<i.length-1&&(null!=n[t]&&n[t]===i[t]);t++)o++;let a=[],s=null;for(let t=0;t<e.length;t++)r.startsWith(e[t])&&(a.push(e[t]),(null==s||e[t].length<s.length)&&(s=e[t]));for(let t=o;t<i.length-1;t++){o++;const e=i.slice(0,t+1).join(".");if(s&&s.length<e.length)return;const n=-1!==a.indexOf(e);if(yield{name:i[t],indent:t,isCollapsed:n,isParent:!0,metadata:null,key:e},n)return}a.length>0||(yield{name:i[i.length-1],indent:o,isCollapsed:!1,isParent:!1,metadata:t[1],key:t[0]})}t.BuildTree=function(t,n){const r=[];let i=[];for(let o=0;o<t.length;o++){const a=t[o];let s=i.slice();for(const t of e(a,n,i))r.push(t),t.isParent&&(s.length>t.indent&&(s=s.slice(0,t.indent)),s.push(t.name));i=s}return r},t.optToProperty=e}(s=e.Private||(e.Private={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0);e.ProgressBar=({value:t,flavorText:e,success:n})=>{const i=t&&Math.min(1,Math.max(0,+t)),o=t?Math.floor(100*t)+"%":"Indeterminate";let a="m-Progress";return null!=n&&(a+=n?"m-Progress-ok":"m-Progress-failure"),null==t?r.createElement("div",{className:a},r.createElement("span",{role:"progressbar",className:"m-Progress-bar-fake-indeterminate",title:o}),r.createElement("span",{className:"m-Progress-flavorText"},e)):r.createElement("div",{className:a},r.createElement("progress",{className:"m-Progress-bar",title:o,value:i},o),r.createElement("span",{className:"m-Progress-flavorText"},e))}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0);e.UncontrolledInput=({value:t,valueChanged:e,className:n})=>{const[i,o]=r.useState(t);return r.createElement("input",{type:"text",className:n,autoComplete:"off",spellCheck:!1,autoCapitalize:"off",autoCorrect:"off",value:i,onChange:t=>{o(t.target.value),e.call(void 0,t.target.value)}})}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(634);e.TreeView=r.TreeView;var i=n(1181);e.TreeModel=i.TreeModel;var o=n(635);e.TreeController=o.TreeController;var a=n(1182);e.TreeList=a.TreeList},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(64),o=n(113);class a{constructor(t){this.idCache=new Map,this._isDisposed=!1,this._onUpdatedSrc$=new i.Subject,this._onUpdated=this._onUpdatedSrc$.asObservable().pipe(o.debounceTime(0,i.animationFrameScheduler)),this.roots=t,this.fillIdCache()}static Create(t){if(!this.VerifyNodeKeyUniqueness(t))throw Error("All TreeNode keys must be unique in the tree!");return new a(this.InsertParentRefsAndDefaults(t))}static VerifyNodeKeyUniqueness(t){const e=new Set;for(const n of r.IterTools.dfs_iter(t,t=>t.children)){if(e.has(n.key))return!1;e.add(n.key)}return!0}static InsertParentRefsAndDefaults(t){const e=t=>{const n=Object.assign({},this.DefaultNode,t);if(!t.children)return n;const r=[];for(const t of n.children){let i=Object.assign({},this.DefaultNode,t);i.children.length>0&&(i=e(t)),i.parent=n,r.push(i)}return n.children=r,n};return t.map(e)}static IsValidUpdate(t){return!("parent"in t||"key"in t||"children"in t)}get isDisposed(){return this._isDisposed}get nodes(){return this.roots}get onUpdated(){return this._onUpdated}get onUpdatedSrc(){return this._onUpdatedSrc$}dispose(){this._isDisposed||(this._onUpdatedSrc$.complete(),this.idCache.clear(),this.roots.splice(0))}[Symbol.iterator](){return r.IterTools.dfs_iter(this.roots,t=>t.children)}get(t){return this.idCache.get(t)}update(t,e){if(!a.IsValidUpdate(e))throw Error("Invalid property override in tree node update");const n=this.idCache.get(t);if(null==n)throw Error("Node with key "+t+" not found!");const r=Object.assign({},n,e);this.replaceNode(n,r)}selectNode(t,e=!1){if(e)for(const t of this)t.isSelected&&this.update(t.key,{isSelected:!1});this.update(t,{isSelected:!0})}addNewNode(t,e,n=0){const r=this.idCache.get(e);if(null==r)throw Error("Parent not found: "+e);t=this.insertNodeIntoParent(t,n,r)}moveNode(t,e,n){const r=null==e?void 0:this.idCache.get(e);this.removeNodeFromParent(t),this.insertNodeIntoParent(t,n,r)}deleteNode(t){this.removeNodeFromParent(t)}removeNodeFromParent(t){const e=t.parent;let n=null!=e?e.children:this.roots;const r=n.indexOf(t);if(-1===r)throw Error("Old parent not found for node: "+t.key);return n=[...n.slice(0,r),...n.slice(r+1)],null==e?this.roots=n:e.children=n,t.parent=void 0,this.idCache.delete(t.key),this._onUpdatedSrc$.next(),t}insertNodeIntoParent(t,e,n){let r=null==n?this.roots:n.children;return r=[...r.slice(0,e),t,...r.slice(e)],null==n?this.roots=r:n.children=r,t.parent=n,this.idCache.set(t.key,t),this._onUpdatedSrc$.next(),t}replaceNode(t,e){const n=t.parent,r=this.indexOfNode(t);this.removeNodeFromParent(t),e=this.insertNodeIntoParent(e,r,n);for(const t of e.children)t.parent=e,this.idCache.set(t.key,t);return e}indexOfNode(t){return(null==t.parent?this.roots:t.parent.children).indexOf(t)}fillIdCache(){for(const t of this)this.idCache.set(t.key,t)}}a.DefaultNode={isCollapsed:!1,isSelected:!1,selectable:!0,children:[]},e.TreeModel=a},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(634),o=n(635),a=n(28),s=n(6),A=n(312),l=n(141),c=n(313);var u;e.TreeList=function({model:t,allowMultiselect:e=!1,showPreviews:n=!0,globalHandler:o=document,controller:g=u.getController(t,{allowOrdering:!0}),onSelected:h=(()=>void 0),dragScope:d="",render:f,renderDragPreview:p}){const[I,C]=r.useState(!1),[m,y]=r.useState(null),[E,v]=r.useState(null),[B,b]=r.useState(null),[Q,w]=r.useState(null),[x,_]=r.useState(null),D=c.usePrevious(I),M=r.useRef(null);return r.useEffect(()=>{if(D!==I){if(I){const e=new class extends l.ReactWrapperWidget{render(){return p&&E?r.createElement("span",null,p.call(void 0,E)):r.createElement("span",null,E?E.key:"...")}};e.addClass("m-TreeList-drag");const n=new s.Widget({node:M.current});function t(t){switch(t.type){case"keydown":return void("Escape"===t.key&&C(!1));case"pointerup":C(!1)}}return e.node.innerHTML=E?E.key:"node",A.HoverManager.GetManager().openHover({hover:e,owner:n,mode:"tooltip",x:0,y:0,width:100,height:24}),o.addEventListener("keydown",t),o.addEventListener("pointerup",t),()=>{o.removeEventListener("keydown",t),o.removeEventListener("pointerup",t),e.dispose(),n.dispose()}}y(null),v(null),b(null),w(null),_(null),u.clearDrag(d)}},[I,M,E]),r.createElement("div",{className:"m-TreeList",ref:M,onPointerDown:e=>{y([e.pageX,e.pageY]);const n=u.getReference(e.target);if(null==n)return;const r=t.get(n);r&&g.canBeginAction(r).then(t=>{t?v(r):y(null)}).catch(console.error)},onPointerMove:async e=>{if(I){const n=u.getReference(e.target);if(null==n||n===E.key)return;const r=E,i=t.get(n),o=g.acceptsChildren(i),a=u.getDropZone(e,o);b(a),w(t.get(n)),await g.canPerformAction(r,i,u.dropZoneToAction(a))?_("valid"):_("invalid")}else{if(null==m)return;const t=a.MathTools.Vec2.Norm(a.MathTools.Vec2.Sub(m,[e.pageX,e.pageY]));Math.abs(t)>5&&(y(null),C(!0))}},onPointerLeave:t=>{I||null==m||y(null),I&&(u.setDrag(d,E),b(null),w(null))},onPointerEnter:t=>{if(!I){const t=u.getDrag(d);if(null==t)return;C(!0),v(t)}},onPointerUp:e=>{y(null);const n=u.getReference(e.target);if(I&&null!=n&&"valid"===x){const r=E,i=t.get(n),o=g.acceptsChildren(i),a=u.getDropZone(e,o);if(null!=t.get(r.key))g.executeAction(r,i,u.dropZoneToAction(a));else{const e="insert-into"===a?i:i.parent;let n=0;n=null==e?t.nodes.indexOf(i):e.children.indexOf(i),"insert-below"===a&&(n+=1),g.executeAction(r,e,"add-new-child",{index:n})}}C(!1)}},r.createElement(i.TreeView,{model:t,onCollapse:(e,n)=>t.update(e,{isCollapsed:n}),onSelect:n=>{t.selectNode(n,!e),h.call(void 0,n)},preview:I&&n?{key:Q&&Q.key||"",action:B||"insert-into",validity:x||"valid",source:E&&E.key||""}:void 0,render:f}))},function(t){function e(t,e){let n=t;for(;null!=n.parentNode;){if(e(n))return n;n=n.parentNode}return null}t.getReference=function t(n){if(!n.matches(".m-TreeView-Node *")&&n.classList.contains("m-TreeView-Level")){const r=e(n,t=>t.classList.contains("m-TreeView-Level"));return null===r||null==r.parentElement?null:t(r.parentElement)}if(n.matches(".m-TreeView-Node *")){const t=e(n,t=>t.classList.contains("m-TreeView-Node"));return null===t?null:t.dataset.key}return null},t.getDropZone=function(t,n=!0){const r=t.target;if(n&&!r.matches(".m-TreeView-Node *")&&r.classList.contains("m-TreeView-Level"))return"insert-into";if(r.matches(".m-TreeView-Node *")){const{clientY:i}=t,o=e(r,t=>t.classList.contains("m-TreeView-Node-row"))||r,{top:a,height:s}=o.getBoundingClientRect(),A=(s+a-i)/s;return n?A<.25?"insert-below":A>.75?"insert-above":"insert-into":A>.5?"insert-above":"insert-below"}return null},t.dropZoneToAction=function(t){switch(t){case"insert-above":return"insert-sibling-above";case"insert-into":return"insert-child-into";case"insert-below":return"insert-sibling-below"}};const n=new WeakMap;t.getController=function(t,e){if(n.has(t))return n.get(t);const r=new o.TreeController(t,e);return n.set(t,r),r};const r=new Map;t.setDrag=function(t,e){r.set(t,e)},t.getDrag=function(t){return r.get(t)||null},t.clearDrag=function(t){r.delete(t)}}(u||(u={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(28),o=n(6),a=n(83),s=n(72),A=n(222);e.FallbackEditor=({type:t,value:e,onValueChanged:n})=>{const i=l.toEditableFormat(e,t),[o,a]=r.useState(e),[s,c]=r.useState(!0);l.toEditableFormat(o,t)!==i&&(a(e),c(!0));let u="m-FallbackEditor-"+t.serializableName;s||(u+=" m-FallbackEditor-invalid");const[g,h,d]=A.useIntermediate(i||"",e=>{try{const r=l.fromEditableFormat(e,t);n.call(void 0,r),c(!0),a(r)}catch(t){c(!1)}});return r.createElement(A.UncontrolledInput,{key:h,className:u,value:g,valueChanged:d})};var l;e.FallbackDetailEditor=class extends o.Widget{constructor({value:t,type:e,onValueChanged:n,metadata:r}){super(),this.editorModel=null,this.editor=null,this.addClass("m-FallbackDetailEditor"),this.layout=new o.BoxLayout,this.value=t,this.type=e,this.mime=r&&r.mime||(l.isEditable(e)?"text/plain":"application/json"),this.onChangeCb=n,this.editorFactory=new s.CodeMirrorEditorFactory,this.setup()}dispose(){this.isDisposed||(this.onChangeCb=()=>void 0,this.editorModel&&this.editorModel.dispose(),super.dispose())}setup(){const t=l.toEditableFormat(this.value,this.type)||"";this.isEditorStale()?this.buildEditor(this.mime,t):(this.editorModel.mimeType=this.mime,this.editorModel.value.text!==t&&(this.editorModel.value.text=t))}isEditorStale(){return null==this.editorModel||null==this.editor}buildEditor(t,e){this.editorModel&&this.editorModel.dispose(),this.editor&&this.editor.dispose(),this.editorModel=new a.CodeEditor.Model({mimeType:t,value:e}),this.editorModel.value.changed.connect(t=>{this.trySetValue(t.text)}),this.editor=new a.CodeEditorWrapper({factory:this.editorFactory.newDocumentEditor,model:this.editorModel}),this.layout.addWidget(this.editor)}trySetValue(t){this.removeClass("m-FallbackEditor-invalid");try{const e=l.fromEditableFormat(t,this.type);this.onChangeCb.call(void 0,e)}catch(t){this.addClass("m-FallbackEditor-invalid")}}},function(t){function e(t,e){return t.toString().padStart(e,"0")}t.isEditable=function(t){switch(t){case i.Types.Boolean:case i.Types.Number:case i.Types.String:case i.Types.Date:case i.Types.DateTime:return!0;default:return!1}},t.toEditableFormat=function(t,n){if(null==t)return null;switch(n){case i.Types.Boolean:case i.Types.Number:case i.Types.String:return""+t;case i.Types.Date:return t instanceof Date||(t=new Date(t)),e((r=t).getUTCFullYear(),4)+"-"+e(r.getUTCMonth()+1,2)+"-"+e(r.getUTCDate(),2);case i.Types.DateTime:return t instanceof Date||(t=new Date(t)),function(t){return e(t.getUTCFullYear(),4)+"-"+e(t.getUTCMonth()+1,2)+"-"+e(t.getUTCDate(),2)+"T"+e(t.getUTCHours(),2)+":"+e(t.getUTCMinutes(),2)+":"+e(t.getUTCSeconds(),2)}(t);default:try{let e=i.Converters.serialize(t,n);t=e&&e.value}catch(e){console.error("Failed to serialize value",t)}return JSON.stringify(t)}var r},t.fromEditableFormat=function(t,e){if(null===t||0===t.length&&e!==i.Types.String)return null;switch(e){case i.Types.Boolean:if(t.match(/^t(?:rue)?$/i))return!0;if(t.match(/^f(?:alse)?$/i))return!1;throw Error("Not a boolean");case i.Types.String:return t;case i.Types.Number:let n=Number.parseFloat(t);if(Number.isNaN(n))throw new Error("Invalid number: "+t);return n;case i.Types.Date:case i.Types.DateTime:let r=new Date(t);if(Number.isNaN(r.getTime()))throw new Error("Invalid date: "+t);return r;default:return i.Converters.deserialize({value:JSON.parse(t),typeName:e.serializableName})}}}(l||(l={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(312),o=n(141),a=n(6),s=n(637),A=n(314);e.TypeEditor=({type:t,value:e,onValueChanged:n,metadata:l,schema:c})=>{const u=s.TypeEditorFactory.Instance,g=u.getEditor(t),h=r.useContext(A.TypeEditorHost.Context),d={type:t,value:e,metadata:l,schema:c,onValueChanged:t=>n.call(void 0,t)};const f=u.isDetailEditorSuppressed(t)?null:r.createElement("button",{className:"m-TypeEditor-DetailEditorBtn fa fa-ellipsis-h",onClick:e=>(function(e){const n=u.getDetailEditor(t);let s;(s=A.isWidgetCtor(n)?new n(d):o.ReactWrapperWidget.Create(r.createElement(n,Object.assign({},d)))).title.label=s.title.label||t.name+" Editor",i.HoverManager.Instance.openHover({hover:s,mode:"dialog",height:300,width:300,owner:h.owner||new a.Widget,x:e.clientX,y:e.clientY})})(e)});return r.createElement("span",{className:"m-TypeEditor"},r.createElement(g,Object.assign({},d)),f)}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(636),i=n(28),o=n(1186),a=n(1228),s=n(1258),A=n(1263);e.defaultTypeEditors={id:"@mavenomics/ui:defaultTypeEditors",autoStart:!0,requires:[r.ITypeEditorFactory],activate:(t,e)=>{e.registerEditor(i.Types.Date,o.DateEditor),e.suppressDetailEditor(i.Types.Date),e.registerEditor(i.Types.DateTime,o.DateEditor),e.suppressDetailEditor(i.Types.DateTime),e.registerEditor(i.Types.Boolean,a.Checkbox),e.suppressDetailEditor(i.Types.Boolean),e.registerEditor(i.Types.Color,s.InlineColor),e.registerDetailEditor(i.Types.Color,s.DetailedColor),e.registerEditor(i.Types.String,A.StringEditor),e.suppressDetailEditor(i.Types.Number)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(7),o=n(713),a=n(648),s=n(314);e.DateEditor=({value:t,onValueChanged:e})=>{const n=i(t||new Date);return r.createElement(s.TypeEditorHost.Context.Consumer,null,t=>r.createElement(a,{value:n,getCalendarContainer:()=>t.portalHostNode||document.body,calendar:r.createElement(o.default,{showOk:!0,onChange:t=>{if(null!=t){const n=new Date(t.format());e.call(void 0,n)}else e.call(void 0,null)}})},({value:t})=>r.createElement("input",{readOnly:!0,value:t&&t.format("MM/DD/YYYY"),placeholder:"mm/dd/yyyy"})))}},function(t,e,n){t.exports={default:n(1188),__esModule:!0}},function(t,e,n){n(1189),t.exports=n(71).Object.assign},function(t,e,n){var r=n(109);r(r.S+r.F,"Object",{assign:n(1191)})},function(t,e){t.exports=function(t){if("function"!=typeof t)throw TypeError(t+" is not a function!");return t}},function(t,e,n){"use strict";var r=n(112),i=n(223),o=n(323),a=n(226),s=n(179),A=n(641),l=Object.assign;t.exports=!l||n(145)(function(){var t={},e={},n=Symbol(),r="abcdefghijklmnopqrst";return t[n]=7,r.split("").forEach(function(t){e[t]=t}),7!=l({},t)[n]||Object.keys(l({},e)).join("")!=r})?function(t,e){for(var n=s(t),l=arguments.length,c=1,u=o.f,g=a.f;l>c;)for(var h,d=A(arguments[c++]),f=u?i(d).concat(u(d)):i(d),p=f.length,I=0;p>I;)h=f[I++],r&&!g.call(d,h)||(n[h]=d[h]);return n}:l},function(t,e,n){var r=n(122),i=n(642),o=n(1193);t.exports=function(t){return function(e,n,a){var s,A=r(e),l=i(A.length),c=o(a,l);if(t&&n!=n){for(;l>c;)if((s=A[c++])!=s)return!0}else for(;l>c;c++)if((t||c in A)&&A[c]===n)return t||c||0;return!t&&-1}}},function(t,e,n){var r=n(319),i=Math.max,o=Math.min;t.exports=function(t,e){return(t=r(t))<0?i(t+e,0):o(t,e)}},function(t,e,n){t.exports={default:n(1195),__esModule:!0}},function(t,e,n){n(643),n(1200),t.exports=n(326).f("iterator")},function(t,e,n){var r=n(319),i=n(318);t.exports=function(t){return function(e,n){var o,a,s=String(i(e)),A=r(n),l=s.length;return A<0||A>=l?t?"":void 0:(o=s.charCodeAt(A))<55296||o>56319||A+1===l||(a=s.charCodeAt(A+1))<56320||a>57343?t?s.charAt(A):o:t?s.slice(A,A+2):a-56320+(o-55296<<10)+65536}}},function(t,e,n){"use strict";var r=n(324),i=n(178),o=n(325),a={};n(142)(a,n(88)("iterator"),function(){return this}),t.exports=function(t,e,n){t.prototype=r(a,{next:i(1,n)}),o(t,e+" Iterator")}},function(t,e,n){var r=n(111),i=n(143),o=n(223);t.exports=n(112)?Object.defineProperties:function(t,e){i(t);for(var n,a=o(e),s=a.length,A=0;s>A;)r.f(t,n=a[A++],e[n]);return t}},function(t,e,n){var r=n(110).document;t.exports=r&&r.documentElement},function(t,e,n){n(1201);for(var r=n(110),i=n(142),o=n(180),a=n(88)("toStringTag"),s="CSSRuleList,CSSStyleDeclaration,CSSValueList,ClientRectList,DOMRectList,DOMStringList,DOMTokenList,DataTransferItemList,FileList,HTMLAllCollection,HTMLCollection,HTMLFormElement,HTMLSelectElement,MediaList,MimeTypeArray,NamedNodeMap,NodeList,PaintRequestList,Plugin,PluginArray,SVGLengthList,SVGNumberList,SVGPathSegList,SVGPointList,SVGStringList,SVGTransformList,SourceBufferList,StyleSheetList,TextTrackCueList,TextTrackList,TouchList".split(","),A=0;A<s.length;A++){var l=s[A],c=r[l],u=c&&c.prototype;u&&!u[a]&&i(u,a,l),o[l]=o.Array}},function(t,e,n){"use strict";var r=n(1202),i=n(1203),o=n(180),a=n(122);t.exports=n(644)(Array,"Array",function(t,e){this._t=a(t),this._i=0,this._k=e},function(){var t=this._t,e=this._k,n=this._i++;return!t||n>=t.length?(this._t=void 0,i(1)):i(0,"keys"==e?n:"values"==e?t[n]:[n,t[n]])},"values"),o.Arguments=o.Array,r("keys"),r("values"),r("entries")},function(t,e){t.exports=function(){}},function(t,e){t.exports=function(t,e){return{value:e,done:!!t}}},function(t,e,n){t.exports={default:n(1205),__esModule:!0}},function(t,e,n){n(1206),n(1211),n(1212),n(1213),t.exports=n(71).Symbol},function(t,e,n){"use strict";var r=n(110),i=n(121),o=n(112),a=n(109),s=n(645),A=n(1207).KEY,l=n(145),c=n(321),u=n(325),g=n(225),h=n(88),d=n(326),f=n(327),p=n(1208),I=n(1209),C=n(143),m=n(144),y=n(179),E=n(122),v=n(316),B=n(178),b=n(324),Q=n(1210),w=n(328),x=n(323),_=n(111),D=n(223),M=w.f,k=_.f,S=Q.f,L=r.Symbol,R=r.JSON,N=R&&R.stringify,F=h("_hidden"),T=h("toPrimitive"),U={}.propertyIsEnumerable,O=c("symbol-registry"),G=c("symbols"),H=c("op-symbols"),j=Object.prototype,Y="function"==typeof L&&!!x.f,P=r.QObject,K=!P||!P.prototype||!P.prototype.findChild,z=o&&l(function(){return 7!=b(k({},"a",{get:function(){return k(this,"a",{value:7}).a}})).a})?function(t,e,n){var r=M(j,e);r&&delete j[e],k(t,e,n),r&&t!==j&&k(j,e,r)}:k,J=function(t){var e=G[t]=b(L.prototype);return e._k=t,e},q=Y&&"symbol"==typeof L.iterator?function(t){return"symbol"==typeof t}:function(t){return t instanceof L},V=function(t,e,n){return t===j&&V(H,e,n),C(t),e=v(e,!0),C(n),i(G,e)?(n.enumerable?(i(t,F)&&t[F][e]&&(t[F][e]=!1),n=b(n,{enumerable:B(0,!1)})):(i(t,F)||k(t,F,B(1,{})),t[F][e]=!0),z(t,e,n)):k(t,e,n)},W=function(t,e){C(t);for(var n,r=p(e=E(e)),i=0,o=r.length;o>i;)V(t,n=r[i++],e[n]);return t},X=function(t){var e=U.call(this,t=v(t,!0));return!(this===j&&i(G,t)&&!i(H,t))&&(!(e||!i(this,t)||!i(G,t)||i(this,F)&&this[F][t])||e)},Z=function(t,e){if(t=E(t),e=v(e,!0),t!==j||!i(G,e)||i(H,e)){var n=M(t,e);return!n||!i(G,e)||i(t,F)&&t[F][e]||(n.enumerable=!0),n}},$=function(t){for(var e,n=S(E(t)),r=[],o=0;n.length>o;)i(G,e=n[o++])||e==F||e==A||r.push(e);return r},tt=function(t){for(var e,n=t===j,r=S(n?H:E(t)),o=[],a=0;r.length>a;)!i(G,e=r[a++])||n&&!i(j,e)||o.push(G[e]);return o};Y||(s((L=function(){if(this instanceof L)throw TypeError("Symbol is not a constructor!");var t=g(arguments.length>0?arguments[0]:void 0),e=function(n){this===j&&e.call(H,n),i(this,F)&&i(this[F],t)&&(this[F][t]=!1),z(this,t,B(1,n))};return o&&K&&z(j,t,{configurable:!0,set:e}),J(t)}).prototype,"toString",function(){return this._k}),w.f=Z,_.f=V,n(647).f=Q.f=$,n(226).f=X,x.f=tt,o&&!n(224)&&s(j,"propertyIsEnumerable",X,!0),d.f=function(t){return J(h(t))}),a(a.G+a.W+a.F*!Y,{Symbol:L});for(var et="hasInstance,isConcatSpreadable,iterator,match,replace,search,species,split,toPrimitive,toStringTag,unscopables".split(","),nt=0;et.length>nt;)h(et[nt++]);for(var rt=D(h.store),it=0;rt.length>it;)f(rt[it++]);a(a.S+a.F*!Y,"Symbol",{for:function(t){return i(O,t+="")?O[t]:O[t]=L(t)},keyFor:function(t){if(!q(t))throw TypeError(t+" is not a symbol!");for(var e in O)if(O[e]===t)return e},useSetter:function(){K=!0},useSimple:function(){K=!1}}),a(a.S+a.F*!Y,"Object",{create:function(t,e){return void 0===e?b(t):W(b(t),e)},defineProperty:V,defineProperties:W,getOwnPropertyDescriptor:Z,getOwnPropertyNames:$,getOwnPropertySymbols:tt});var ot=l(function(){x.f(1)});a(a.S+a.F*ot,"Object",{getOwnPropertySymbols:function(t){return x.f(y(t))}}),R&&a(a.S+a.F*(!Y||l(function(){var t=L();return"[null]"!=N([t])||"{}"!=N({a:t})||"{}"!=N(Object(t))})),"JSON",{stringify:function(t){for(var e,n,r=[t],i=1;arguments.length>i;)r.push(arguments[i++]);if(n=e=r[1],(m(e)||void 0!==t)&&!q(t))return I(e)||(e=function(t,e){if("function"==typeof n&&(e=n.call(this,t,e)),!q(e))return e}),r[1]=e,N.apply(R,r)}}),L.prototype[T]||n(142)(L.prototype,T,L.prototype.valueOf),u(L,"Symbol"),u(Math,"Math",!0),u(r.JSON,"JSON",!0)},function(t,e,n){var r=n(225)("meta"),i=n(144),o=n(121),a=n(111).f,s=0,A=Object.isExtensible||function(){return!0},l=!n(145)(function(){return A(Object.preventExtensions({}))}),c=function(t){a(t,r,{value:{i:"O"+ ++s,w:{}}})},u=t.exports={KEY:r,NEED:!1,fastKey:function(t,e){if(!i(t))return"symbol"==typeof t?t:("string"==typeof t?"S":"P")+t;if(!o(t,r)){if(!A(t))return"F";if(!e)return"E";c(t)}return t[r].i},getWeak:function(t,e){if(!o(t,r)){if(!A(t))return!0;if(!e)return!1;c(t)}return t[r].w},onFreeze:function(t){return l&&u.NEED&&A(t)&&!o(t,r)&&c(t),t}}},function(t,e,n){var r=n(223),i=n(323),o=n(226);t.exports=function(t){var e=r(t),n=i.f;if(n)for(var a,s=n(t),A=o.f,l=0;s.length>l;)A.call(t,a=s[l++])&&e.push(a);return e}},function(t,e,n){var r=n(317);t.exports=Array.isArray||function(t){return"Array"==r(t)}},function(t,e,n){var r=n(122),i=n(647).f,o={}.toString,a="object"==typeof window&&window&&Object.getOwnPropertyNames?Object.getOwnPropertyNames(window):[];t.exports.f=function(t){return a&&"[object Window]"==o.call(t)?function(t){try{return i(t)}catch(t){return a.slice()}}(t):i(r(t))}},function(t,e){},function(t,e,n){n(327)("asyncIterator")},function(t,e,n){n(327)("observable")},function(t,e,n){t.exports={default:n(1215),__esModule:!0}},function(t,e,n){n(1216),t.exports=n(71).Object.setPrototypeOf},function(t,e,n){var r=n(109);r(r.S,"Object",{setPrototypeOf:n(1217).set})},function(t,e,n){var r=n(144),i=n(143),o=function(t,e){if(i(t),!r(e)&&null!==e)throw TypeError(e+": can't set as prototype!")};t.exports={set:Object.setPrototypeOf||("__proto__"in{}?function(t,e,r){try{(r=n(315)(Function.call,n(328).f(Object.prototype,"__proto__").set,2))(t,[]),e=!(t instanceof Array)}catch(t){e=!0}return function(t,n){return o(t,n),e?t.__proto__=n:r(t,n),t}}({},!1):void 0),check:o}},function(t,e,n){t.exports={default:n(1219),__esModule:!0}},function(t,e,n){n(1220);var r=n(71).Object;t.exports=function(t,e){return r.create(t,e)}},function(t,e,n){var r=n(109);r(r.S,"Object",{create:n(324)})},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.default=function(){var t=[].slice.call(arguments,0);if(1===t.length)return t[0];return function(){for(var e=0;e<t.length;e++)t[e]&&t[e].apply&&t[e].apply(this,arguments)}},t.exports=e.default},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r={MAC_ENTER:3,BACKSPACE:8,TAB:9,NUM_CENTER:12,ENTER:13,SHIFT:16,CTRL:17,ALT:18,PAUSE:19,CAPS_LOCK:20,ESC:27,SPACE:32,PAGE_UP:33,PAGE_DOWN:34,END:35,HOME:36,LEFT:37,UP:38,RIGHT:39,DOWN:40,PRINT_SCREEN:44,INSERT:45,DELETE:46,ZERO:48,ONE:49,TWO:50,THREE:51,FOUR:52,FIVE:53,SIX:54,SEVEN:55,EIGHT:56,NINE:57,QUESTION_MARK:63,A:65,B:66,C:67,D:68,E:69,F:70,G:71,H:72,I:73,J:74,K:75,L:76,M:77,N:78,O:79,P:80,Q:81,R:82,S:83,T:84,U:85,V:86,W:87,X:88,Y:89,Z:90,META:91,WIN_KEY_RIGHT:92,CONTEXT_MENU:93,NUM_ZERO:96,NUM_ONE:97,NUM_TWO:98,NUM_THREE:99,NUM_FOUR:100,NUM_FIVE:101,NUM_SIX:102,NUM_SEVEN:103,NUM_EIGHT:104,NUM_NINE:105,NUM_MULTIPLY:106,NUM_PLUS:107,NUM_MINUS:109,NUM_PERIOD:110,NUM_DIVISION:111,F1:112,F2:113,F3:114,F4:115,F5:116,F6:117,F7:118,F8:119,F9:120,F10:121,F11:122,F12:123,NUMLOCK:144,SEMICOLON:186,DASH:189,EQUALS:187,COMMA:188,PERIOD:190,SLASH:191,APOSTROPHE:192,SINGLE_QUOTE:222,OPEN_SQUARE_BRACKET:219,BACKSLASH:220,CLOSE_SQUARE_BRACKET:221,WIN_KEY:224,MAC_FF_META:224,WIN_IME:229,isTextModifyingKeyEvent:function(t){var e=t.keyCode;if(t.altKey&&!t.ctrlKey||t.metaKey||e>=r.F1&&e<=r.F12)return!1;switch(e){case r.ALT:case r.CAPS_LOCK:case r.CONTEXT_MENU:case r.CTRL:case r.DOWN:case r.END:case r.ESC:case r.HOME:case r.INSERT:case r.LEFT:case r.MAC_FF_META:case r.META:case r.NUMLOCK:case r.NUM_CENTER:case r.PAGE_DOWN:case r.PAGE_UP:case r.PAUSE:case r.PRINT_SCREEN:case r.RIGHT:case r.SHIFT:case r.UP:case r.WIN_KEY:case r.WIN_KEY_RIGHT:return!1;default:return!0}},isCharacterKey:function(t){if(t>=r.ZERO&&t<=r.NINE)return!0;if(t>=r.NUM_ZERO&&t<=r.NUM_MULTIPLY)return!0;if(t>=r.A&&t<=r.Z)return!0;if(-1!==window.navigation.userAgent.indexOf("WebKit")&&0===t)return!0;switch(t){case r.SPACE:case r.QUESTION_MARK:case r.NUM_PLUS:case r.NUM_MINUS:case r.NUM_PERIOD:case r.NUM_DIVISION:case r.SEMICOLON:case r.DASH:case r.EQUALS:case r.COMMA:case r.PERIOD:case r.SLASH:case r.APOSTROPHE:case r.SINGLE_QUOTE:case r.OPEN_SQUARE_BRACKET:case r.BACKSLASH:case r.CLOSE_SQUARE_BRACKET:return!0;default:return!1}}};e.default=r,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0;var r={adjustX:1,adjustY:1},i=[0,0],o={bottomLeft:{points:["tl","tl"],overflow:r,offset:[0,-3],targetOffset:i},bottomRight:{points:["tr","tr"],overflow:r,offset:[0,-3],targetOffset:i},topRight:{points:["br","br"],overflow:r,offset:[0,3],targetOffset:i},topLeft:{points:["bl","bl"],overflow:r,offset:[0,3],targetOffset:i}};e.default=o,t.exports=e.default},function(t,e,n){"use strict";function r(t){return t&&t.__esModule?t:{default:t}}Object.defineProperty(e,"__esModule",{value:!0});var i=r(n(1225)),o=r(n(294)),a=!0,s=!1,A=["altKey","bubbles","cancelable","ctrlKey","currentTarget","eventPhase","metaKey","shiftKey","target","timeStamp","view","type"];function l(t){return null==t}var c=[{reg:/^key/,props:["char","charCode","key","keyCode","which"],fix:function(t,e){l(t.which)&&(t.which=l(e.charCode)?e.keyCode:e.charCode),void 0===t.metaKey&&(t.metaKey=t.ctrlKey)}},{reg:/^touch/,props:["touches","changedTouches","targetTouches"]},{reg:/^hashchange$/,props:["newURL","oldURL"]},{reg:/^gesturechange$/i,props:["rotation","scale"]},{reg:/^(mousewheel|DOMMouseScroll)$/,props:[],fix:function(t,e){var n=void 0,r=void 0,i=void 0,o=e.wheelDelta,a=e.axis,s=e.wheelDeltaY,A=e.wheelDeltaX,l=e.detail;o&&(i=o/120),l&&(i=0-(l%3==0?l/3:l)),void 0!==a&&(a===t.HORIZONTAL_AXIS?(r=0,n=0-i):a===t.VERTICAL_AXIS&&(n=0,r=i)),void 0!==s&&(r=s/120),void 0!==A&&(n=-1*A/120),n||r||(r=i),void 0!==n&&(t.deltaX=n),void 0!==r&&(t.deltaY=r),void 0!==i&&(t.delta=i)}},{reg:/^mouse|contextmenu|click|mspointer|(^DOMMouseScroll$)/i,props:["buttons","clientX","clientY","button","offsetX","relatedTarget","which","fromElement","toElement","offsetY","pageX","pageY","screenX","screenY"],fix:function(t,e){var n=void 0,r=void 0,i=void 0,o=t.target,a=e.button;return o&&l(t.pageX)&&!l(e.clientX)&&(r=(n=o.ownerDocument||document).documentElement,i=n.body,t.pageX=e.clientX+(r&&r.scrollLeft||i&&i.scrollLeft||0)-(r&&r.clientLeft||i&&i.clientLeft||0),t.pageY=e.clientY+(r&&r.scrollTop||i&&i.scrollTop||0)-(r&&r.clientTop||i&&i.clientTop||0)),t.which||void 0===a||(t.which=1&a?1:2&a?3:4&a?2:0),!t.relatedTarget&&t.fromElement&&(t.relatedTarget=t.fromElement===o?t.toElement:t.fromElement),t}}];function u(){return a}function g(){return s}function h(t){var e=t.type,n="function"==typeof t.stopPropagation||"boolean"==typeof t.cancelBubble;i.default.call(this),this.nativeEvent=t;var r=g;"defaultPrevented"in t?r=t.defaultPrevented?u:g:"getPreventDefault"in t?r=t.getPreventDefault()?u:g:"returnValue"in t&&(r=t.returnValue===s?u:g),this.isDefaultPrevented=r;var o=[],a=void 0,l=void 0,h=A.concat();for(c.forEach(function(t){e.match(t.reg)&&(h=h.concat(t.props),t.fix&&o.push(t.fix))}),a=h.length;a;)this[l=h[--a]]=t[l];for(!this.target&&n&&(this.target=t.srcElement||document),this.target&&3===this.target.nodeType&&(this.target=this.target.parentNode),a=o.length;a;)(0,o[--a])(this,t);this.timeStamp=t.timeStamp||Date.now()}var d=i.default.prototype;(0,o.default)(h.prototype,d,{constructor:h,preventDefault:function(){var t=this.nativeEvent;t.preventDefault?t.preventDefault():t.returnValue=s,d.preventDefault.call(this)},stopPropagation:function(){var t=this.nativeEvent;t.stopPropagation?t.stopPropagation():t.cancelBubble=a,d.stopPropagation.call(this)}}),e.default=h,t.exports=e.default},function(t,e,n){"use strict";function r(){return!1}function i(){return!0}function o(){this.timeStamp=Date.now(),this.target=void 0,this.currentTarget=void 0}Object.defineProperty(e,"__esModule",{value:!0}),o.prototype={isEventObject:1,constructor:o,isDefaultPrevented:r,isPropagationStopped:r,isImmediatePropagationStopped:r,preventDefault:function(){this.isDefaultPrevented=i},stopPropagation:function(){this.isPropagationStopped=i},stopImmediatePropagation:function(){this.isImmediatePropagationStopped=i,this.stopPropagation()},halt:function(t){t?this.stopImmediatePropagation():this.stopPropagation(),this.preventDefault()}},e.default=o,t.exports=e.default},function(t,e,n){n(1227);var r=n(71).Object;t.exports=function(t,e,n){return r.defineProperty(t,e,n)}},function(t,e,n){var r=n(109);r(r.S+r.F*!n(112),"Object",{defineProperty:n(111).f})},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(651);e.Checkbox=({value:t,onValueChanged:e})=>r.createElement("span",{className:"m-rw-inline-hack"},r.createElement(i,{value:t?"True":"False",data:["True","False"],onChange:t=>{e.call(void 0,"True"===t)}}))},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t){void 0===t&&(t=(0,i.default)());try{return t.activeElement}catch(t){}};var i=r(n(652));t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t){(0,a.default)(t,{componentDidMount:function(){var t=this.props.autoFocus;t&&(this.focus?this.focus():(0,o.findDOMNode)(this).focus())}})},e.PropTypes=void 0;var r,i=n(1),o=n(16),a=(r=n(227))&&r.__esModule?r:{default:r};var s={autoFocus:i.bool};e.PropTypes=s},function(t,e,n){"use strict";e.__esModule=!0,e.callFocusEventHandler=function(t,e,n){var r=t.props[e?"onFocus":"onBlur"];r&&r(n)},e.default=function(t,e){var n,a=void 0===e?{}:e,s=a.willHandle,A=a.didHandle,l=a.onChange,c=a.isDisabled,u=void 0===c?function(){return!!t.props.disabled}:c,g=(0,i.default)(t),h=(0,o.default)(t);function d(e,i){i&&i.persist&&i.persist(),s&&!1===s(e,i)||g.set("focus",function(){(0,r.unstable_batchedUpdates)(function(){e!==n&&(A&&A.call(t,e,i),!h()&&e||(n=e,l&&l(e,i)))})})}return{handleBlur:function(t){u()||d(!1,t)},handleFocus:function(t){u()||d(!0,t)}}};var r=n(16),i=a(n(653)),o=a(n(330));function a(t){return t&&t.__esModule?t:{default:t}}},function(t,e,n){"use strict";function r(){return(r=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}function i(t,e){var n=e.propTypes,i=e.contextTypes,o=e.childContextTypes,a=e.getChildContext,s=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(r=0;r<a.length;r++)n=a[r],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(i[n]=t[n])}return i}(e,["propTypes","contextTypes","childContextTypes","getChildContext"]);if(n&&(t.propTypes=r({},t.propTypes,n)),i&&(t.contextTypes=r({},t.contextTypes,i)),o&&(t.childContextTypes=r({},t.childContextTypes,o)),a){var A=t.prototype.getChildContext;t.prototype.getChildContext=function(){return r({},A&&A.call(this),a.call(this))}}return r(t.prototype,s),t}e.__esModule=!0,e.mixin=i,e.default=function(t){return function(e){return i(e,t)}}},function(t,e,n){"use strict";e.__esModule=!0,e.uncontrolledPropTypes=function(t,e){var n={};return Object.keys(t).forEach(function(t){n[o(t)]=i}),n},e.isProp=function(t,e){return void 0!==t[e]},e.defaultKey=o,e.isReactComponent=function(t){return!!(t&&t.prototype&&t.prototype.isReactComponent)};var r;(r=n(331))&&r.__esModule;var i=function(){};function o(t){return"default"+t.charAt(0).toUpperCase()+t.substr(1)}},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=a(n(0)),i=a(n(1)),o=a(n(4));function a(t){return t&&t.__esModule?t:{default:t}}function s(){return(s=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var A=function(){return r.default.createElement("span",{"aria-hidden":"true",className:"rw-i rw-loading"})},l=function(t){var e,n;function i(){return t.apply(this,arguments)||this}return n=t,(e=i).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n,i.prototype.render=function(){var t=this.props,e=t.className,n=t.disabled,i=t.label,a=t.icon,l=t.busy,c=t.active,u=t.children,g=t.variant,h=void 0===g?"primary":g,d=t.spinner,f=void 0===d?r.default.createElement(A,null):d,p=t.component,I=void 0===p?"button":p,C=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(r=0;r<a.length;r++)n=a[r],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(i[n]=t[n])}return i}(t,["className","disabled","label","icon","busy","active","children","variant","spinner","component"]),m=C.type;return"button"===I&&(m=m||"button"),r.default.createElement(I,s({},C,{tabIndex:"-1",title:i,type:m,disabled:n,"aria-disabled":n,"aria-label":i,className:(0,o.default)(e,"rw-btn",c&&!n&&"rw-state-active",h&&"rw-btn-"+h)}),l?f:a,u)},i}(r.default.Component);l.propTypes={disabled:i.default.bool,label:i.default.string,icon:i.default.node,busy:i.default.bool,active:i.default.bool,variant:i.default.oneOf(["primary","select"]),component:i.default.any,spinner:i.default.node};var c=l;e.default=c,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r,i=g(n(4)),o=g(n(1236)),a=g(n(663)),s=g(n(332)),A=n(665),l=g(n(1)),c=g(n(0)),u=function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}return e.default=t,e}(n(309));function g(t){return t&&t.__esModule?t:{default:t}}function h(){return(h=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var d=((r={})[u.ENTERING]="rw-popup-transition-entering",r[u.EXITING]="rw-popup-transition-exiting",r[u.EXITED]="rw-popup-transition-exited",r),f={in:l.default.bool.isRequired,dropUp:l.default.bool,onEntering:l.default.func,onEntered:l.default.func};var p=function(t){var e,n;function r(){for(var e,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(e=t.call.apply(t,[this].concat(r))||this).setContainerHeight=function(t){t.style.height=e.getHeight()+"px"},e.clearContainerHeight=function(t){t.style.height=""},e.handleEntered=function(t){e.clearContainerHeight(t),e.props.onEntered&&e.props.onEntered()},e.handleEntering=function(){e.props.onEntering&&e.props.onEntering()},e.handleTransitionEnd=function(t,e){var n=function(t){var e=(0,a.default)(t,A.transitionDuration),n=-1===e.indexOf("ms")?1e3:1;return parseFloat(e)*n}(t.lastChild)||0,r=function n(){o.default.off(t,A.transitionEnd,n,!1),e()};setTimeout(r,1.5*n),o.default.on(t,A.transitionEnd,r,!1)},e.attachRef=function(t){return e.element=t},e}n=t,(e=r).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n;var l=r.prototype;return l.getHeight=function(){var t,e=this.element,n=e.firstChild,r=parseInt((0,a.default)(n,"margin-top"),10)+parseInt((0,a.default)(n,"margin-bottom"),10),i=e.style.display;return e.style.display="block",t=((0,s.default)(n)||0)+(isNaN(r)?0:r),e.style.display=i,t},l.render=function(){var t=this,e=this.props,n=e.children,r=e.className,o=e.dropUp;return c.default.createElement(u.default,{appear:!0,in:this.props.in,timeout:5e3,onEnter:this.setContainerHeight,onEntering:this.handleEntering,onEntered:this.handleEntered,onExit:this.setContainerHeight,onExited:this.clearContainerHeight,addEndListener:this.handleTransitionEnd},function(e,a){return c.default.createElement("div",h({},a,{ref:t.attachRef,className:(0,i.default)(r,o&&"rw-dropup",d[e])}),c.default.createElement("div",{className:"rw-popup-transition"},n))})},r}(c.default.Component);p.propTypes=f;var I=p;e.default=I,t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=void 0;var i=r(n(659));e.on=i.default;var o=r(n(660));e.off=o.default;var a=r(n(1237));e.filter=a.default;var s=r(n(1238));e.listen=s.default;var A={on:i.default,off:o.default,filter:a.default,listen:s.default};e.default=A},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t,e){return function(n){var r=n.currentTarget,a=n.target,s=(0,o.default)(r,t);s.some(function(t){return(0,i.default)(t,a)})&&e.call(this,n)}};var i=r(n(661)),o=r(n(662));t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=void 0;var i=r(n(123)),o=r(n(659)),a=r(n(660)),s=function(){};i.default&&(s=function(t,e,n,r){return(0,o.default)(t,e,n,r),function(){(0,a.default)(t,e,n,r)}});var A=s;e.default=A,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t){return t.replace(r,function(t,e){return e.toUpperCase()})};var r=/-(.)/g;t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t){return(0,i.default)(t).replace(o,"-ms-")};var i=r(n(1241)),o=/^ms-/;t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t){return t.replace(r,"-$1").toLowerCase()};var r=/([A-Z])/g;t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t){if(!t)throw new TypeError("No Element passed to `getComputedStyle()`");var e=t.ownerDocument;return"defaultView"in e?e.defaultView.opener?t.ownerDocument.defaultView.getComputedStyle(t,null):window.getComputedStyle(t,null):{getPropertyValue:function(e){var n=t.style;"float"==(e=(0,i.default)(e))&&(e="styleFloat");var r=t.currentStyle[e]||null;if(null==r&&n&&n[e]&&(r=n[e]),a.test(r)&&!o.test(e)){var s=n.left,A=t.runtimeStyle,l=A&&A.left;l&&(A.left=t.currentStyle.left),n.left="fontSize"===e?"1em":r,r=n.pixelLeft+"px",n.left=s,l&&(A.left=l)}return r}}};var i=r(n(664)),o=/^(top|right|bottom|left)$/,a=/^([+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|))(?!px)[a-z%]+$/i;t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t,e){return"removeProperty"in t.style?t.style.removeProperty(e):t.style.removeAttribute(e)},t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=function(t){return!(!t||!r.test(t))};var r=/^((translate|rotate|scale)(X|Y|Z|3d)?|matrix(3d)?|perspective|skew(X|Y)?)$/i;t.exports=e.default},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=a(n(0)),i=n(1246),o=a(n(667));function a(t){return t&&t.__esModule?t:{default:t}}e.default=(0,o.default)(function(t,e,n,o,a){var s=t[e];return r.default.isValidElement(s)?new Error("Invalid "+o+" `"+a+"` of type ReactElement supplied to `"+n+"`,expected an element type (a string , component class, or function component)."):(0,i.isValidElementType)(s)?null:new Error("Invalid "+o+" `"+a+"` of value `"+s+"` supplied to `"+n+"`, expected an element type (a string , component class, or function component).")}),t.exports=e.default},function(t,e,n){"use strict";t.exports=n(1247)},function(t,e,n){"use strict";
/** @license React v16.8.6
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */Object.defineProperty(e,"__esModule",{value:!0});var r="function"==typeof Symbol&&Symbol.for,i=r?Symbol.for("react.element"):60103,o=r?Symbol.for("react.portal"):60106,a=r?Symbol.for("react.fragment"):60107,s=r?Symbol.for("react.strict_mode"):60108,A=r?Symbol.for("react.profiler"):60114,l=r?Symbol.for("react.provider"):60109,c=r?Symbol.for("react.context"):60110,u=r?Symbol.for("react.async_mode"):60111,g=r?Symbol.for("react.concurrent_mode"):60111,h=r?Symbol.for("react.forward_ref"):60112,d=r?Symbol.for("react.suspense"):60113,f=r?Symbol.for("react.memo"):60115,p=r?Symbol.for("react.lazy"):60116;function I(t){if("object"==typeof t&&null!==t){var e=t.$$typeof;switch(e){case i:switch(t=t.type){case u:case g:case a:case A:case s:case d:return t;default:switch(t=t&&t.$$typeof){case c:case h:case l:return t;default:return e}}case p:case f:case o:return e}}}function C(t){return I(t)===g}e.typeOf=I,e.AsyncMode=u,e.ConcurrentMode=g,e.ContextConsumer=c,e.ContextProvider=l,e.Element=i,e.ForwardRef=h,e.Fragment=a,e.Lazy=p,e.Memo=f,e.Portal=o,e.Profiler=A,e.StrictMode=s,e.Suspense=d,e.isValidElementType=function(t){return"string"==typeof t||"function"==typeof t||t===a||t===g||t===A||t===s||t===d||"object"==typeof t&&null!==t&&(t.$$typeof===p||t.$$typeof===f||t.$$typeof===l||t.$$typeof===c||t.$$typeof===h)},e.isAsyncMode=function(t){return C(t)||I(t)===u},e.isConcurrentMode=C,e.isContextConsumer=function(t){return I(t)===c},e.isContextProvider=function(t){return I(t)===l},e.isElement=function(t){return"object"==typeof t&&null!==t&&t.$$typeof===i},e.isForwardRef=function(t){return I(t)===h},e.isFragment=function(t){return I(t)===a},e.isLazy=function(t){return I(t)===p},e.isMemo=function(t){return I(t)===f},e.isPortal=function(t){return I(t)===o},e.isProfiler=function(t){return I(t)===A},e.isStrictMode=function(t){return I(t)===s},e.isSuspense=function(t){return I(t)===d}},function(t,e,n){"use strict";e.__esModule=!0,e.setNumber=function(t){var e=t.format,n=t.parse,r=t.formats,i=t.propType,s=void 0===i?o:i,l=t.decimalChar,c=void 0===l?function(){return"."}:l,u=t.precision,d=void 0===u?function(){return null}:u;h(a,r),A={formats:r,precision:d,decimalChar:c,propType:s,format:g(e),parse:function(t,e,r){var i=n.call(this,t,e,r);return null!=i&&"number"!=typeof i&&invariant(!1),i}}},e.setDate=function(t){var e=t.formats,n=t.format,r=t.parse,i=t.firstOfWeek,a=t.propType,A=void 0===a?o:a;h(s,e),c={formats:e,propType:A,firstOfWeek:i,format:g(n),parse:function(t,e,n){var i=r.call(this,t,e,n);return null==i||i instanceof Date&&!isNaN(i.getTime())||invariant(!1),i}}},e.date=e.number=void 0;i(n(331)),n(229);var r=i(n(1));function i(t){return t&&t.__esModule?t:{default:t}}var o=r.default.oneOfType([r.default.string,r.default.func]),a=["default"],s=["default","date","time","header","footer","weekday","dayOfMonth","month","year","decade","century"],A=d(),l={propType:function(){var t;return(t=A).propType.apply(t,arguments)},getFormat:function(t,e){return e||A.formats[t]},parse:function(){var t;return(t=A).parse.apply(t,arguments)},format:function(){var t;return(t=A).format.apply(t,arguments)},decimalChar:function(){var t;return(t=A).decimalChar.apply(t,arguments)},precision:function(){var t;return(t=A).precision.apply(t,arguments)}};e.number=l;var c=d(),u={propType:function(){var t;return(t=c).propType.apply(t,arguments)},getFormat:function(t,e){return e||c.formats[t]},parse:function(){var t;return(t=c).parse.apply(t,arguments)},format:function(){var t;return(t=c).format.apply(t,arguments)},firstOfWeek:function(){var t;return(t=c).firstOfWeek.apply(t,arguments)}};e.date=u;var g=function(t){return function(e,n,r){var i="function"==typeof n?n(e,r,this):t.call(this,e,n,r);return null!=i&&"string"!=typeof i&&invariant(!1),i}};function h(t,e){0}function d(){return{}}},function(t,e,n){"use strict";t.exports=function(){}},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=a(n(4)),i=a(n(0)),o=a(n(1));function a(t){return t&&t.__esModule?t:{default:t}}var s={className:o.default.string,component:o.default.string};function A(t){var e=t.children,n=t.className,o=t.component,a=void 0===o?"li":o;return i.default.createElement(a,{tabIndex:"-1",role:"separator",className:(0,r.default)(n,"rw-list-optgroup")},e)}A.propTypes=s;var l=A;e.default=l,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}return e.default=t,e}(n(1)),i=s(n(0)),o=s(n(669)),a=s(n(670));function s(t){return t&&t.__esModule?t:{default:t}}function A(){return(A=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var l={searchTerm:r.string,focused:r.bool,onSelect:r.func.isRequired,activeId:r.string};function c(t){var e=t.searchTerm,n=t.onSelect,r=t.focused,s=t.children,l=t.activeId,c=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(r=0;r<a.length;r++)n=a[r],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(i[n]=t[n])}return i}(t,["searchTerm","onSelect","focused","children","activeId"]);return i.default.createElement(o.default,A({},c,{className:"rw-list-option-create"}),i.default.createElement(a.default,{onSelect:n,focused:r,activeId:l,dataItem:e},s))}c.propTypes=l;var u=c;e.default=u,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=A(n(0)),i=A(n(4)),o=A(n(1)),a=function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}return e.default=t,e}(n(146)),s=n(181);function A(t){return t&&t.__esModule?t:{default:t}}var l=function(t){var e,n;function o(){for(var e,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(e=t.call.apply(t,[this].concat(r))||this).state={autofilling:!1},e.handleAutofillDetect=function(t){var n,r=t.animationName;if("react-widgets-autofill-start"===r)n=!0;else{if("react-widgets-autofill-cancel"!==r)return;n=!1}e.setState({autofilling:n}),e.props.onAutofill(n)},e.handleAutofill=function(t){e.setState({autofilling:!1}),e.props.onAutofillChange(t)},e}return n=t,(e=o).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n,o.prototype.render=function(){var t=this.props,e=t.name,n=t.placeholder,o=t.value,a=t.textField,A=t.autoComplete,l=t.valueComponent,c=this.state.autofilling,u=null;c||"off"===A||(u=!o&&n?r.default.createElement("span",{className:"rw-placeholder"},n):l?r.default.createElement(l,{item:o}):(0,s.dataText)(o,a));var g=(0,s.dataValue)(o,a);return r.default.createElement("div",{className:"rw-input rw-dropdown-list-input"},"off"!==A&&r.default.createElement("input",{tabIndex:"-1",name:e,value:null==g?"":g,autoComplete:A,onChange:this.handleAutofill,onAnimationStart:this.handleAutofillDetect,className:(0,i.default)("rw-dropdown-list-autofill rw-detect-autofill",!c&&"rw-sr")}),u)},o}(r.default.Component);l.propTypes={value:o.default.any,placeholder:o.default.string,name:o.default.string,autoComplete:o.default.string,textField:a.accessor,valueComponent:a.elementType,onAutofill:o.default.func.isRequired,onAutofillChange:o.default.func.isRequired};var c=l;e.default=c,t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t,e){if(!i&&o.default){var n=document.body,r=n.matches||n.matchesSelector||n.webkitMatchesSelector||n.mozMatchesSelector||n.msMatchesSelector;i=r?function(t,e){return r.call(t,e)}:s}return i?i(t,e):null};var i,o=r(n(123)),a=r(n(662));function s(t,e){for(var n=(0,a.default)(t.document||t.ownerDocument,e),r=0;n[r]&&n[r]!==t;)r++;return!!n[r]}t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t,e){var n,r,c,u,g,h,d,f=(0,i.default)(t),p={top:0,left:0};if(!t)return;n=e||(0,a.default)(t),u=(0,l.default)(n),r=(0,s.default)(n),h=(0,o.default)(n,!0),(u=(0,l.default)(n))||(p=(0,i.default)(n));f={top:f.top-p.top,left:f.left-p.left,height:f.height,width:f.width},g=f.height,c=f.top+(u?0:r),d=c+g,r=r>c?c:d>r+h?d-h:r;var I=(0,A.default)(function(){return(0,s.default)(n,r)});return function(){return A.default.cancel(I)}};var i=r(n(666)),o=r(n(332)),a=r(n(1255)),s=r(n(1256)),A=r(n(1257)),l=r(n(228));t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t){var e=(0,i.default)(t,"position"),n="absolute"===e,r=t.ownerDocument;if("fixed"===e)return r||document;for(;(t=t.parentNode)&&9!==t.nodeType;){var a=n&&"static"===(0,i.default)(t,"position"),s=(0,i.default)(t,"overflow")+(0,i.default)(t,"overflow-y")+(0,i.default)(t,"overflow-x");if(!a&&(/(auto|scroll)/.test(s)&&(0,o.default)(t)<t.scrollHeight))return t}return document};var i=r(n(663)),o=r(n(332));t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=function(t,e){var n=(0,i.default)(t);if(void 0===e)return n?"pageYOffset"in n?n.pageYOffset:n.document.documentElement.scrollTop:t.scrollTop;n?n.scrollTo("pageXOffset"in n?n.pageXOffset:n.document.documentElement.scrollLeft,e):t.scrollTop=e};var i=r(n(228));t.exports=e.default},function(t,e,n){"use strict";var r=n(54);e.__esModule=!0,e.default=void 0;var i,o=r(n(123)),a="clearTimeout",s=function(t){var e=(new Date).getTime(),n=Math.max(0,16-(e-l)),r=setTimeout(t,n);return l=e,r},A=function(t,e){return t+(t?e[0].toUpperCase()+e.substr(1):e)+"AnimationFrame"};o.default&&["","webkit","moz","o","ms"].some(function(t){var e=A(t,"request");if(e in window)return a=A(t,"cancel"),s=function(t){return window[e](t)}});var l=(new Date).getTime();(i=function(t){return s(t)}).cancel=function(t){window[a]&&"function"==typeof window[a]&&window[a](t)};var c=i;e.default=c,t.exports=e.default},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(28),o=n(6),a=n(1259),s=n(632),A=n(1260),l=Object.freeze([...Object.keys(s),"transparent"].sort());function c({item:t}){return r.createElement("span",{className:"m-InlineColorEditor"},r.createElement("span",{style:{backgroundColor:t},className:"m-InlineColorEditor__colorSwatch"}),t)}e.InlineColor=({value:t,onValueChanged:e})=>{if(!(t instanceof i.Color))try{t=new i.Color(t)}catch(e){t=new i.Color("white")}const[n,o]=r.useState(t.color),[a,s]=r.useState(t.color);t.color!==a&&(o(t.color),s(t.color));const[u,g]=r.useState(!1);return r.createElement("span",{className:"m-InlineColorEditor"},r.createElement("span",{style:{backgroundColor:t.color},className:"m-InlineColorEditor__colorSwatch"}),r.createElement("span",{className:"m-rw-inline-hack m-InlineColorEditor__input "+(u?"m-InlineColorEditor__invalid":"")},r.createElement(A,{defaultValue:t.color,value:n,data:[...l],itemComponent:c,onChange:t=>{const n=t;let r;o(n);try{r=new i.Color(n),s(r.color),g(!1),e.call(void 0,r)}catch(t){g(!0)}}})))};e.DetailedColor=class extends o.Widget{constructor({value:t,onValueChanged:e}){super();const n=this.layout=new o.BoxLayout;if(!(t instanceof i.Color))try{t=new i.Color(t)}catch(e){t=new i.Color("white")}n.addWidget(this.picker=new a.ColorPicker({color:t})),this.cb=e,this.picker.onChange.connect(this.handleChange,this)}handleChange(t,e){this.cb.call(void 0,e)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(6),o=n(631),a=n(10);var s;e.ColorPicker=class extends i.Widget{constructor({color:t=new r.Color("white")}={}){super(),this._onChange=new a.Signal(this),this.addClass("m-ColorPicker"),this._color=t;const e=o.get.rgb(t.color)||[255,255,255,1],n="transparent"===t.color?[0,0,1]:s.rgbToHsv(e[0],e[1],e[2]);this.canvas=new s.CanvasWidget(...n),this.canvas.onChange.connect(this.handleColorChange,this),this.valueSlider=new s.SliderWidget("Brightness: ",n[2]),this.valueSlider.node.title="Value, in the HSV color model",this.alphaSlider=new s.SliderWidget("Transparency: ",null==e[3]?1:e[3]),this.layout=new i.BoxLayout,i.BoxLayout.setSizeBasis(this.valueSlider,30),i.BoxLayout.setSizeBasis(this.alphaSlider,30),i.BoxLayout.setStretch(this.valueSlider,0),i.BoxLayout.setStretch(this.alphaSlider,0),i.BoxLayout.setStretch(this.canvas,1),this.layout.addWidget(this.canvas),this.layout.addWidget(this.valueSlider),this.layout.addWidget(this.alphaSlider)}get onChange(){return this._onChange}get color(){return this._color}set color(t){this._color=t;const e=o.get.rgb(t.color)||[255,255,255,1],n=s.rgbToHsv(e[0],e[1],e[2]);this.canvas.hue=n[0],this.canvas.saturation=n[1],this.valueSlider.value=n[2],this.alphaSlider.value=null==e[3]?1:e[3],this.canvas.value=n[2]}handleEvent(t){switch(t.type){case"m-slider-change":this.handleValueChange(t)}}dispose(){this.isDisposed||(this.canvas.onChange.disconnect(this.handleColorChange,this),super.dispose())}onAfterAttach(){this.node.addEventListener("m-slider-change",this)}onBeforeDetach(){this.node.removeEventListener("m-slider-change",this)}handleValueChange(t){t.preventDefault(),t.stopPropagation(),t.originator===this.valueSlider&&(this.canvas.value=t.value),this.handleColorChange(void 0,[this.canvas.hue,this.canvas.saturation,this.canvas.value,this.alphaSlider.value])}handleColorChange(t,[e,n,i,o]){const a=s.hsvToRgb(e,n,i);this._color=new r.Color("white"),0===this.alphaSlider.value?this._color.color="transparent":this._color.color=[...a,null==o?255:o],this._onChange.emit(this._color)}},function(t){t.CanvasWidget=class extends i.Widget{constructor(t,e,n){super({node:document.createElement("canvas")}),this.onChange=new a.Signal(this);const r=this.node;this.canvas=r,this._hue=t,this._saturation=e,this._value=n,this.ctx=r.getContext("2d"),this.height=0,this.width=0}handleEvent(t){switch(t.type){case"mousedown":this.handleMouseDown(t);break;case"mousemove":this.handleMouseMove(t);break;case"mouseup":case"mouseleave":this.handleMouseUp(t)}}get hue(){return this._hue}set hue(t){this._hue=t,this.redraw()}get saturation(){return this._saturation}set saturation(t){this._saturation=t,this.redraw()}get value(){return this._value}set value(t){this._value=t,this.redraw()}onResize(){this.maybeRedraw()}onUpdateRequest(){this.maybeRedraw()}onAfterAttach(){this.node.addEventListener("mousedown",this)}handleMouseDown(t){this.node.addEventListener("mousemove",this),this.node.addEventListener("mouseleave",this),this.node.addEventListener("mouseup",this),this.handleMouseMove(t)}handleMouseMove(t){const{left:e,top:n}=this.node.getBoundingClientRect(),i=Math.min(this.width,this.height)/2,[o,a]=[t.clientX-e,t.clientY-n],[s,A]=r(o,a,i);this._hue=A,this._saturation=Math.min(s/i,1),this.onChange.emit([this._hue,this._saturation,this._value]),this.redraw()}handleMouseUp(t){this.node.removeEventListener("mousemove",this),this.node.removeEventListener("mouseleave",this),this.node.removeEventListener("mouseup",this)}maybeRedraw(){const{width:t,height:e}=this.node.getBoundingClientRect();t===this.width&&e===this.height||(this.height=e,this.width=t,this.redraw())}redraw(){const t=Math.min(this.width,this.height);this.canvas.height=this.height,this.canvas.width=this.width,this.ctx.restore(),n(this._value,this.ctx,{width:t,height:t}),this.ctx.beginPath();const e=this._saturation*t/2*Math.cos(this._hue*Math.PI/180+Math.PI/2)+t/2,r=this._saturation*t/2*Math.sin(this._hue*Math.PI/180-Math.PI/2)+t/2;this.ctx.arc(e,r,10,0,2*Math.PI,!0);const[i,a,s]=o(this._hue,this._saturation,this._value);this.ctx.fillStyle=`rgb(${i},${a},${s})`,this.ctx.fill(),this._value>=.5?this.ctx.strokeStyle="black":this.ctx.strokeStyle="white",this.ctx.arc(e,r,10,0,2*Math.PI,!0),this.ctx.stroke()}};t.SliderWidget=class extends i.Widget{constructor(t,e){super({node:document.createElement("label")}),this.addClass("m-ValueSlider"),this.node.innerText=t,this.sliderEl=document.createElement("input"),this.sliderEl.type="range",this.sliderEl.min="0",this.sliderEl.max="1",this._value=e,this.sliderEl.value=""+e,this.sliderEl.setAttribute("value",""+e),this.sliderEl.step="0.001",this.node.appendChild(this.sliderEl)}get value(){return this._value}set value(t){this._value=t,this.sliderEl.value=""+t}handleEvent(t){switch(t.type){case"input":this.handleValueChange(t.target.value)}}onAfterAttach(){this.sliderEl.value=""+this._value,this.sliderEl.addEventListener("input",this)}onBeforeDetach(){this.sliderEl.removeEventListener("input",this)}handleValueChange(t){const n=parseFloat(t);if(Number.isNaN(n))return;this._value=n;const r=new e(n,this,{bubbles:!0});this.node.dispatchEvent(r)}};class e extends Event{constructor(t,e,n){super("m-slider-change",n),this.value=t,this.originator=e}}function n(e,n,i){const{width:o,height:a}=i;console.assert(o===a,"Color picker must be square!");const s=Math.min(o,a)/2,A=n.createImageData(o,a),l=A.data;for(let n=0;n<o;n++)for(let i=0;i<a;i++){const[o,A]=r(n,i,s);if(o>s)continue;const c=t.hsvToRgb(A,o/s,e),u=i*a+n,g=4;l[u*g]=c[0],l[u*g+1]=c[1],l[u*g+2]=c[2],l[u*g+3]=255}n.putImageData(A,0,0)}function r(t,e,n){return[Math.sqrt(Math.pow(t-n,2)+Math.pow(e-n,2)),(Math.atan2(t-n,e-n)+Math.PI)/(2*Math.PI)*360]}function o(t,e,n){let r,i,o,a=n*e,s=t/60,A=a*(1-Math.abs(s%2-1));[r,i,o]=s<=1?[a,A,0]:s<=2?[A,a,0]:s<=3?[0,a,A]:s<=4?[0,A,a]:s<=5?[A,0,a]:[a,0,A];let l=n-a,[c,u,g]=[r+l,i+l,o+l];return[Math.round(255*c),Math.round(255*u),Math.round(255*g)]}t.SliderChangeEvent=e,t.drawHsvCircle=n,t.rectToPolar=r,t.hsvToRgb=o,t.rgbToHsv=function(t,e,n){t/=255,e/=255,n/=255;const r=Math.max(t,e,n),i=r-Math.min(t,e,n);let o;return 0===i?o=0:r===t?o=(e-n)/i%6:r===e?o=(n-t)/i+2:r===n&&(o=(t-e)/i+4),[60*o,0===r?0:i/r,r]}}(s||(s={}))},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r,i,o,a,s,A,l,c=R(n(0)),u=R(n(1)),g=R(n(4)),h=n(67),d=R(n(654)),f=R(n(655)),p=R(n(656)),I=R(n(668)),C=R(n(658)),m=R(n(657)),y=R(n(1261)),E=n(335),v=R(n(671)),B=R(n(333)),b=R(n(672)),Q=L(n(146)),w=R(n(673)),x=n(229),_=L(n(230)),D=L(n(334)),M=n(336),k=n(231),S=n(674);function L(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}return e.default=t,e}function R(t){return t&&t.__esModule?t:{default:t}}function N(t,e,n,r){n&&Object.defineProperty(t,e,{enumerable:n.enumerable,configurable:n.configurable,writable:n.writable,value:n.initializer?n.initializer.call(r):void 0})}function F(t){if(void 0===t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return t}function T(t,e,n,r,i){var o={};return Object.keys(r).forEach(function(t){o[t]=r[t]}),o.enumerable=!!o.enumerable,o.configurable=!!o.configurable,("value"in o||o.initializer)&&(o.writable=!0),o=n.slice().reverse().reduce(function(n,r){return r(t,e,n)||n},o),i&&void 0!==o.initializer&&(o.value=o.initializer?o.initializer.call(i):void 0,o.initializer=void 0),void 0===o.initializer&&(Object.defineProperty(t,e,o),o=null),o}function U(){return(U=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var O=U({},D.propTypes,{value:u.default.any,onChange:u.default.func,open:u.default.bool,onToggle:u.default.func,itemComponent:Q.elementType,listComponent:Q.elementType,groupComponent:Q.elementType,groupBy:Q.accessor,data:u.default.array,valueField:Q.accessor,textField:Q.accessor,name:u.default.string,onSelect:u.default.func,autoFocus:u.default.bool,disabled:Q.disabled.acceptsArray,readOnly:Q.disabled,suggest:D.propTypes.filter,busy:u.default.bool,selectIcon:u.default.node,busySpinner:u.default.node,delay:u.default.number,dropUp:u.default.bool,popupTransition:Q.elementType,placeholder:u.default.string,containerClassName:u.default.string,inputProps:u.default.object,listProps:u.default.object,isRtl:u.default.bool,messages:u.default.shape({openCombobox:Q.message,emptyList:Q.message,emptyFilter:Q.message})}),G=(0,h.polyfill)((l=A=function(t){var e,n;function r(e,n){var r;return(r=t.call(this,e,n)||this).handleFocusWillChange=function(t){!t&&r.inputRef&&r.inputRef.accept(),t&&r.focus()},r.handleFocusChanged=function(t){t||r.close()},N(r,"handleSelect",o,F(F(r))),r.handleInputKeyDown=function(t){var e=t.key;r._deleting="Backspace"===e||"Delete"===e,r._isTyping=!0},r.handleInputChange=function(t){var e=r.suggest(t.target.value);r.change(e,!0,t),r.open()},N(r,"handleKeyDown",a,F(F(r))),r.attachListRef=function(t){r.listRef=t},r.attachInputRef=function(t){r.inputRef=t},N(r,"toggle",s,F(F(r))),r.inputId=(0,k.instanceId)(F(F(r)),"_input"),r.listId=(0,k.instanceId)(F(F(r)),"_listbox"),r.activeId=(0,k.instanceId)(F(F(r)),"_listbox_active_option"),r.handleScroll=(0,w.default)(F(F(r))),r.focusManager=(0,v.default)(F(F(r)),{willHandle:r.handleFocusWillChange,didHandle:r.handleFocusChanged}),r.state={isSuggesting:function(){return r.inputRef&&r.inputRef.isSuggesting()}},r}n=t,(e=r).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n;var i=r.prototype;return i.shouldComponentUpdate=function(t,e){var n=e.isSuggesting(),r=!(0,x.isShallowEqual)(e,this.state),i=!(0,x.isShallowEqual)(t,this.props);return n||r||i},r.getDerivedStateFromProps=function(t,e){var n,r=t.value,i=t.data,o=t.messages,a=t.filter,s=t.minLength,A=t.caseSensitive,l=e.focusedItem,c=(0,b.default)(t),u=r!==e.lastValue,g=c.indexOf(i,r),h=-1===g?r:i[g];(-1===g||e.isSuggesting())&&(n=c.text(h)),i=D.filter(i,{filter:a,searchTerm:n,minLength:s,caseSensitive:A,textField:c.text});var d=(0,B.default)(i,e.list,{nextProps:t});-1!==g&&(g=c.indexOf(i,r));var f=c.indexOf(i,l);-1===f&&(f=D.indexOf(i,{searchTerm:n,textField:c.text,filter:a||!0}));var p=i[g],I=null;return I=-1===f?void 0!==p?p:i[0]:i[f],{data:i,list:d,accessors:c,lastValue:r,messages:(0,E.getMessages)(o),selectedItem:u?d.nextEnabled(p):e.selectedItem,focusedItem:u||void 0===l?d.nextEnabled(void 0!==p?p:I):I}},i.renderInput=function(){var t=this.props,e=t.suggest,n=t.filter,r=t.busy,i=t.name,o=t.data,a=t.value,s=t.autoFocus,A=t.tabIndex,l=t.placeholder,u=t.inputProps,g=t.disabled,h=t.readOnly,d=t.open,f=this.state.accessors,p=f.findOrSelf(o,a),I=e?n?"both":"inline":n?"list":"";return c.default.createElement(y.default,U({},u,{role:"combobox",name:i,id:this.inputId,autoFocus:s,tabIndex:A,suggest:e,disabled:!0===g,readOnly:!0===h,"aria-busy":!!r,"aria-owns":this.listId,"aria-autocomplete":I,"aria-activedescendant":d?this.activeId:null,"aria-expanded":d,"aria-haspopup":!0,placeholder:l,value:f.text(p),onChange:this.handleInputChange,onKeyDown:this.handleInputKeyDown,ref:this.attachInputRef}))},i.renderList=function(t){var e=this.activeId,n=this.inputId,r=this.listId,i=this.props,o=i.open,a=i.data,s=i.value,A=i.listProps,l=i.optionComponent,u=i.itemComponent,g=i.groupComponent,h=this.state,d=h.list,f=h.accessors,p=h.focusedItem,I=h.selectedItem,C=h.data,m=this.props.listComponent;return c.default.createElement(m,U({},A,{id:r,activeId:e,data:C,dataState:d.dataState,isDisabled:d.isDisabled,textAccessor:f.text,valueAccessor:f.value,itemComponent:u,groupComponent:g,optionComponent:l,selectedItem:I,focusedItem:o?p:null,searchTerm:f.text(s)||"","aria-hidden":!o,"aria-labelledby":n,"aria-live":o&&"polite",onSelect:this.handleSelect,onMove:this.handleScroll,ref:this.attachListRef,messages:{emptyList:a.length?t.emptyFilter:t.emptyList}}))},i.render=function(){var t=this,e=this.props,n=e.isRtl,r=e.className,i=e.popupTransition,o=e.busy,a=e.dropUp,s=e.open,A=e.selectIcon,l=e.busySpinner,u=e.containerClassName,h=this.state,d=h.focused,I=h.messages,y=!0===this.props.disabled,E=!0===this.props.readOnly,v=_.pickElementProps(this),B=(0,k.isFirstFocusedRender)(this);return c.default.createElement(f.default,U({},v,{open:s,isRtl:n,dropUp:a,focused:d,disabled:y,readOnly:E,onBlur:this.focusManager.handleBlur,onFocus:this.focusManager.handleFocus,onKeyDown:this.handleKeyDown,className:(0,g.default)(r,"rw-combobox")}),c.default.createElement(p.default,{className:u},this.renderInput(),c.default.createElement(m.default,{bordered:!0,busy:o,icon:A,spinner:l,onClick:this.toggle,disabled:y||E,label:I.openCombobox(this.props)})),B&&c.default.createElement(C.default,{open:s,dropUp:a,transition:i,onEntering:function(){return t.listRef.forceUpdate()}},c.default.createElement("div",null,this.renderList(I))))},i.focus=function(){this.inputRef&&this.inputRef.focus()},i.change=function(t,e,n){var r=this.props,i=r.onChange,o=r.value;this._typedChange=!!e,(0,k.notify)(i,[t,{lastValue:o,originalEvent:n}])},i.open=function(){this.props.open||(0,k.notify)(this.props.onToggle,!0)},i.close=function(){this.props.open&&(0,k.notify)(this.props.onToggle,!1)},i.suggest=function(t){var e=this.props,n=e.textField,r=e.suggest,i=e.minLength,o=this.state.data;return this._deleting?t:D.suggest(o,{minLength:i,textField:n,searchTerm:t,filter:r,caseSensitive:!1})},r}(c.default.Component),A.propTypes=O,A.defaultProps={data:[],value:"",open:!1,suggest:!1,filter:!1,delay:500,selectIcon:S.caretDown,listComponent:I.default},o=T((i=l).prototype,"handleSelect",[M.widgetEditable],{enumerable:!0,initializer:function(){var t=this;return function(e,n){t.close(),(0,k.notify)(t.props.onSelect,[e,{originalEvent:n}]),t.change(e,!1,n),t.inputRef&&t.inputRef.accept(!0),t.focus()}}}),a=T(i.prototype,"handleKeyDown",[M.widgetEditable],{enumerable:!0,initializer:function(){var t=this;return function(e){var n=e.key,r=e.altKey,i=t.props,o=i.open,a=i.onKeyDown,s=t.state,A=s.focusedItem,l=s.selectedItem,c=s.list;if((0,k.notify)(a,[e]),!e.defaultPrevented){var u=function(n){return null!=n&&t.handleSelect(n,e)},g=function(e){return t.setState({focusedItem:e})};if("End"===n&&o)e.preventDefault(),g(c.last());else if("Home"===n&&o)e.preventDefault(),g(c.first());else if("Escape"===n&&o)e.preventDefault(),t.close();else if("Enter"===n&&o)e.preventDefault(),u(t.state.focusedItem);else if("Tab"===n)t.inputRef.accept();else if("ArrowDown"===n){if(e.preventDefault(),r)return t.open();o?g(c.next(A)):u(c.next(l))}else if("ArrowUp"===n){if(e.preventDefault(),r)return t.close();o?g(c.prev(A)):u(c.prev(l))}}}}}),s=T(i.prototype,"toggle",[M.widgetEditable],{enumerable:!0,initializer:function(){var t=this;return function(){t.focus(),t.props.open?t.close():t.open()}}}),r=i))||r,H=(0,d.default)(G,{open:"onToggle",value:"onChange"},["focus"]);e.default=H,t.exports=e.default},function(t,e,n){"use strict";e.__esModule=!0,e.default=e.caretSet=void 0;var r=s(n(0)),i=s(n(1)),o=n(16),a=s(n(1262));function s(t){return t&&t.__esModule?t:{default:t}}function A(){return(A=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}var l=function(t,e,n){void 0===n&&(n=e);try{t.setSelectionRange(e,n)}catch(t){}};e.caretSet=l;var c=function(t){var e,n;function i(){for(var e,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(e=t.call.apply(t,[this].concat(r))||this).handleChange=function(t){var n=e.props,r=n.placeholder,i=n.value,o=n.onChange,a=t.target.value;!!r&&!a&&a===(i||"")||(e._last=a,o(t,a))},e}n=t,(e=i).prototype=Object.create(n.prototype),e.prototype.constructor=e,e.__proto__=n;var s=i.prototype;return s.componentDidUpdate=function(){var t=(0,o.findDOMNode)(this),e=this.props.value;if(this.isSuggesting()){var n=e.toLowerCase().indexOf(this._last.toLowerCase())+this._last.length,r=e.length-n;n>=0&&0!==r&&l(t,n,n+r)}},s.accept=function(t){if(void 0===t&&(t=!1),this._last=null,t){var e=(0,o.findDOMNode)(this);l(e,e.value.length)}},s.focus=function(){(0,o.findDOMNode)(this).focus()},s.isSuggesting=function(){var t=this.props,e=t.value;return!!t.suggest&&(null!=this._last&&-1!==e.toLowerCase().indexOf(this._last.toLowerCase()))},s.render=function(){var t=this.props,e=t.onKeyDown,n=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(r=0;r<a.length;r++)n=a[r],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(i[n]=t[n])}return i}(t,["onKeyDown"]);return delete n.suggest,r.default.createElement(a.default,A({},n,{className:"rw-widget-input",onKeyDown:e,onChange:this.handleChange}))},i}(r.default.Component);c.defaultProps={value:""},c.propTypes={value:i.default.string,placeholder:i.default.string,suggest:i.default.bool,onChange:i.default.func.isRequired,onKeyDown:i.default.func};var u=c;e.default=u},function(t,e,n){"use strict";e.__esModule=!0,e.default=void 0;var r=a(n(0)),i=a(n(1)),o=a(n(4));function a(t){return t&&t.__esModule?t:{default:t}}function s(){return(s=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t}).apply(this,arguments)}function A(t){var e=t.className,n=t.disabled,i=t.readOnly,a=t.value,A=t.tabIndex,l=t.nodeRef,c=t.type,u=void 0===c?"text":c,g=t.component,h=void 0===g?"input":g,d=function(t,e){if(null==t)return{};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(r=0;r<a.length;r++)n=a[r],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(i[n]=t[n])}return i}(t,["className","disabled","readOnly","value","tabIndex","nodeRef","type","component"]);return r.default.createElement(h,s({},d,{type:u,ref:l,tabIndex:A||0,autoComplete:"off",disabled:n,readOnly:i,"aria-disabled":n,"aria-readonly":i,value:null==a?"":a,className:(0,o.default)(e,"rw-input")}))}A.propTypes={disabled:i.default.bool,readOnly:i.default.bool,value:i.default.string,type:i.default.string,tabIndex:i.default.string,component:i.default.any,nodeRef:i.default.func};var l=A;e.default=l,t.exports=e.default},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(651),o=n(222);e.StringEditor=({value:t,onValueChanged:e,schema:n})=>{const[a,s,A]=o.useIntermediate(t,e);return null!=n?r.createElement("span",{className:"m-rw-inline-hack"},r.createElement(i,{value:t,data:n.enum,onChange:t=>{e.call(void 0,t)}})):r.createElement(o.UncontrolledInput,{key:s,className:"m-FallbackEditor-String",value:a,valueChanged:t=>A(t)})}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(64),i=n(714),o=n(0),a=n(141),s=n(6),A=n(83),l=n(72),c=n(9);!function(t){function e(t){return class extends t{constructor(){super(...arguments),this._value=null,this._valueDidChangeSrc$=new r.Subject,this._valueDidChange=this._valueDidChangeSrc$.asObservable()}get value(){return this._value}get valueDidChange(){return this._valueDidChange}set value(t){this._value=t,this.update()}dispose(){this.isDisposed||(this._valueDidChangeSrc$.complete(),super.dispose())}setValue(t){this._value=t,this._valueDidChangeSrc$.next(t),this.update()}}}const n=e(a.ReactWrapperWidget),u=e(s.Widget);t.Select=class extends n{constructor(){super(),this._options=[],this.addClass("m-InputWidget-Select")}get options(){return this._options}set options(t){this._options=t.slice(),this.update()}render(){const t=this.options.map(t=>{const e=""+t;if(""!==e)return o.createElement(i.Option,{key:e,value:e},e)});return o.createElement(i.default,{value:""+this.value,onChange:t=>this.setValue(t),style:{width:"100%"}},t)}};t.Code=class extends u{constructor(){super(),this.editorModel=null,this.editor=null,this._mime="text/plain",this._context=null,this.layout=new s.BoxLayout,this.editorFactory=new l.CodeMirrorEditorFactory}get context(){return this._context}set context(t){this._context=t,this.update()}get mime(){return this._mime}set mime(t){this._mime=t,this.update()}dispose(){this.isDisposed||(this.editorModel&&this.editorModel.dispose(),super.dispose())}onUpdateRequest(){const t=""+this.value;this.isEditorStale()?this.buildEditor(this._mime,t):(this.editorModel.mimeType=this._mime,this.editorModel.value.text!==t&&(this.editorModel.value.text=t))}isEditorStale(){return null==this.editorModel||null==this.editor}buildEditor(t,e){this.editorModel&&this.editorModel.dispose(),this.editor&&this.editor.dispose(),this.editorModel=new A.CodeEditor.Model({mimeType:t,value:e}),this.editorModel.value.changed.connect(t=>{this.setValue(t.text)}),this.editor=new A.CodeEditorWrapper({factory:this.editorFactory.newDocumentEditor,model:this.editorModel,config:{lint:{tooltips:!0,options:{esversion:9}},gutters:["CodeMirror-lint-markers"],extraKeys:{"Ctrl-Space":"autocomplete"},hintOptions:{completeSingle:!1},context:this.context}}),this.layout.addWidget(this.editor)}};t.Text=class extends n{constructor(){super(),this._type="text",this._label="",this._name=c.UUID.uuid4(),this.addClass("m-TextInput")}get type(){return this._type}get label(){return this._label}get name(){return this._name}get autocomplete(){return this._autocomplete}set type(t){this._type=t,this.update()}set label(t){this._label=t,this.update()}set name(t){this._name=t,this.update()}set autocomplete(t){this._autocomplete=t,this.update()}render(){return o.createElement("span",null,o.createElement("label",{htmlFor:this.name},this.label),o.createElement("input",{type:this._type,name:this.name,value:this.value||"",autoComplete:this._autocomplete,onChange:t=>this.setValue(t.currentTarget.value)}))}}}(e.InputWidgets||(e.InputWidgets={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r,i=function(){function t(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}return function(e,n,r){return n&&t(e.prototype,n),r&&t(e,r),e}}(),o=n(0),a=((r=o)&&r.__esModule,n(675));var s=function(t){function e(){return function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}(this,e),function(t,e){if(!t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return!e||"object"!=typeof e&&"function"!=typeof e?t:e}(this,(e.__proto__||Object.getPrototypeOf(e)).apply(this,arguments))}return function(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function, not "+typeof e);t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,enumerable:!1,writable:!0,configurable:!0}}),e&&(Object.setPrototypeOf?Object.setPrototypeOf(t,e):t.__proto__=e)}(e,o.Component),i(e,[{key:"getChildContext",value:function(){return{miniStore:this.props.store}}},{key:"render",value:function(){return o.Children.only(this.props.children)}}]),e}();s.propTypes={store:a.storeShape.isRequired},s.childContextTypes={miniStore:a.storeShape.isRequired},e.default=s},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t},i=function(){function t(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}return function(e,n,r){return n&&t(e.prototype,n),r&&t(e,r),e}}();e.default=function(t){var e=!!t,n=t||g;return function(u){var g=function(o){function A(t,e){!function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}(this,A);var r=function(t,e){if(!t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return!e||"object"!=typeof e&&"function"!=typeof e?t:e}(this,(A.__proto__||Object.getPrototypeOf(A)).call(this,t,e));return r.handleChange=function(){if(r.unsubscribe){var t=n(r.store.getState(),r.props);r.setState({subscribed:t})}},r.store=e.miniStore,r.state={subscribed:n(r.store.getState(),t),store:r.store,props:t},r}return function(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function, not "+typeof e);t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,enumerable:!1,writable:!0,configurable:!0}}),e&&(Object.setPrototypeOf?Object.setPrototypeOf(t,e):t.__proto__=e)}(A,o),i(A,null,[{key:"getDerivedStateFromProps",value:function(e,r){return t&&2===t.length&&e!==r.props?{subscribed:n(r.store.getState(),e),props:e}:{props:e}}}]),i(A,[{key:"componentDidMount",value:function(){this.trySubscribe()}},{key:"componentWillUnmount",value:function(){this.tryUnsubscribe()}},{key:"shouldComponentUpdate",value:function(t,e){return!(0,s.default)(this.props,t)||!(0,s.default)(this.state.subscribed,e.subscribed)}},{key:"trySubscribe",value:function(){e&&(this.unsubscribe=this.store.subscribe(this.handleChange),this.handleChange())}},{key:"tryUnsubscribe",value:function(){this.unsubscribe&&(this.unsubscribe(),this.unsubscribe=null)}},{key:"getWrappedInstance",value:function(){return this.wrappedInstance}},{key:"render",value:function(){var t=this,e=r({},this.props,this.state.subscribed,{store:this.store});return u.prototype.render&&(e=r({},e,{ref:function(e){return t.wrappedInstance=e}})),a.default.createElement(u,e)}}]),A}(o.Component);return g.displayName="Connect("+function(t){return t.displayName||t.name||"Component"}(u)+")",g.contextTypes={miniStore:c.storeShape.isRequired},(0,l.polyfill)(g),(0,A.default)(g,u)}};var o=n(0),a=u(o),s=u(n(385)),A=u(n(1267)),l=n(67),c=n(675);function u(t){return t&&t.__esModule?t:{default:t}}var g=function(){return{}}},function(t,e,n){"use strict";var r={childContextTypes:!0,contextTypes:!0,defaultProps:!0,displayName:!0,getDefaultProps:!0,getDerivedStateFromProps:!0,mixins:!0,propTypes:!0,type:!0},i={name:!0,length:!0,prototype:!0,caller:!0,callee:!0,arguments:!0,arity:!0},o=Object.defineProperty,a=Object.getOwnPropertyNames,s=Object.getOwnPropertySymbols,A=Object.getOwnPropertyDescriptor,l=Object.getPrototypeOf,c=l&&l(Object);t.exports=function t(e,n,u){if("string"!=typeof n){if(c){var g=l(n);g&&g!==c&&t(e,g,u)}var h=a(n);s&&(h=h.concat(s(n)));for(var d=0;d<h.length;++d){var f=h[d];if(!(r[f]||i[f]||u&&u[f])){var p=A(n,f);try{o(e,f,p)}catch(t){}}}return e}return e}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t};e.default=function(t){var e=t,n=[];return{setState:function(t){e=r({},e,t);for(var i=0;i<n.length;i++)n[i]()},getState:function(){return e},subscribe:function(t){return n.push(t),function(){var e=n.indexOf(t);n.splice(e,1)}}}}},function(t,e,n){var r,i,o;!function(n){var a=/iPhone/i,s=/iPod/i,A=/iPad/i,l=/\bAndroid(?:.+)Mobile\b/i,c=/Android/i,u=/\bAndroid(?:.+)SD4930UR\b/i,g=/\bAndroid(?:.+)(?:KF[A-Z]{2,4})\b/i,h=/Windows Phone/i,d=/\bWindows(?:.+)ARM\b/i,f=/BlackBerry/i,p=/BB10/i,I=/Opera Mini/i,C=/\b(CriOS|Chrome)(?:.+)Mobile/i,m=/Mobile(?:.+)Firefox\b/i;function y(t,e){return t.test(e)}function E(t){var e=t||("undefined"!=typeof navigator?navigator.userAgent:""),n=e.split("[FBAN");void 0!==n[1]&&(e=n[0]),void 0!==(n=e.split("Twitter"))[1]&&(e=n[0]);var r={apple:{phone:y(a,e)&&!y(h,e),ipod:y(s,e),tablet:!y(a,e)&&y(A,e)&&!y(h,e),device:(y(a,e)||y(s,e)||y(A,e))&&!y(h,e)},amazon:{phone:y(u,e),tablet:!y(u,e)&&y(g,e),device:y(u,e)||y(g,e)},android:{phone:!y(h,e)&&y(u,e)||!y(h,e)&&y(l,e),tablet:!y(h,e)&&!y(u,e)&&!y(l,e)&&(y(g,e)||y(c,e)),device:!y(h,e)&&(y(u,e)||y(g,e)||y(l,e)||y(c,e))||y(/\bokhttp\b/i,e)},windows:{phone:y(h,e),tablet:y(d,e),device:y(h,e)||y(d,e)},other:{blackberry:y(f,e),blackberry10:y(p,e),opera:y(I,e),firefox:y(m,e),chrome:y(C,e),device:y(f,e)||y(p,e)||y(I,e)||y(m,e)||y(C,e)}};return r.any=r.apple.device||r.android.device||r.windows.device||r.other.device,r.phone=r.apple.phone||r.android.phone||r.windows.phone,r.tablet=r.apple.tablet||r.android.tablet||r.windows.tablet,r}t.exports&&"undefined"==typeof window?t.exports=E:t.exports&&"undefined"!=typeof window?(t.exports=E(),t.exports.isMobile=E):(i=[],r=n.isMobile=E(),void 0===(o="function"==typeof r?r.apply(e,i):r)||(t.exports=o))}(this)},function(t,e){window.MutationObserver=window.MutationObserver||function(t){function e(t){this.i=[],this.m=t}function n(e){var n,r={type:null,target:null,addedNodes:[],removedNodes:[],previousSibling:null,nextSibling:null,attributeName:null,attributeNamespace:null,oldValue:null};for(n in e)r[n]!==t&&e[n]!==t&&(r[n]=e[n]);return r}function r(e,r){var s=o(e,r);return function(l){var c,u=l.length;r.a&&3===e.nodeType&&e.nodeValue!==s.a&&l.push(new n({type:"characterData",target:e,oldValue:s.a})),r.b&&s.b&&i(l,e,s.b,r.f),(r.c||r.g)&&(c=function(e,r,o,s){function l(t,r,o,a,A){var l,u,g,h=t.length-1;for(A=-~((h-A)/2);g=t.pop();)l=o[g.j],u=a[g.l],s.c&&A&&Math.abs(g.j-g.l)>=h&&(e.push(n({type:"childList",target:r,addedNodes:[l],removedNodes:[l],nextSibling:l.nextSibling,previousSibling:l.previousSibling})),A--),s.b&&u.b&&i(e,l,u.b,s.f),s.a&&3===l.nodeType&&l.nodeValue!==u.a&&e.push(n({type:"characterData",target:l,oldValue:u.a})),s.g&&c(l,u)}function c(r,o){for(var g,h,d,f,p,I=r.childNodes,C=o.c,m=I.length,y=C?C.length:0,E=0,v=0,B=0;v<m||B<y;)f=I[v],p=(d=C[B])&&d.node,f===p?(s.b&&d.b&&i(e,f,d.b,s.f),s.a&&d.a!==t&&f.nodeValue!==d.a&&e.push(n({type:"characterData",target:f,oldValue:d.a})),h&&l(h,r,I,C,E),s.g&&(f.childNodes.length||d.c&&d.c.length)&&c(f,d),v++,B++):(u=!0,g||(g={},h=[]),f&&(g[d=a(f)]||(g[d]=!0,-1===(d=A(C,f,B,"node"))?s.c&&(e.push(n({type:"childList",target:r,addedNodes:[f],nextSibling:f.nextSibling,previousSibling:f.previousSibling})),E++):h.push({j:v,l:d})),v++),p&&p!==I[v]&&(g[d=a(p)]||(g[d]=!0,-1===(d=A(I,p,v))?s.c&&(e.push(n({type:"childList",target:o.node,removedNodes:[p],nextSibling:C[B+1],previousSibling:C[B-1]})),E--):h.push({j:d,l:B})),B++));h&&l(h,r,I,C,E)}var u;return c(r,o),u}(l,e,s,r)),(c||l.length!==u)&&(s=o(e,r))}}function i(e,r,i,o){for(var a,s,A={},l=r.attributes,u=l.length;u--;)s=(a=l[u]).name,o&&o[s]===t||(c(r,a)!==i[s]&&e.push(n({type:"attributes",target:r,attributeName:s,oldValue:i[s],attributeNamespace:a.namespaceURI})),A[s]=!0);for(s in i)A[s]||e.push(n({target:r,type:"attributes",attributeName:s,oldValue:i[s]}))}function o(t,e){var n=!0;return function t(r){var i={node:r};return!e.a||3!==r.nodeType&&8!==r.nodeType?(e.b&&n&&1===r.nodeType&&(i.b=s(r.attributes,function(t,n){return e.f&&!e.f[n.name]||(t[n.name]=c(r,n)),t})),n&&(e.c||e.a||e.b&&e.g)&&(i.c=function(t,e){for(var n=[],r=0;r<t.length;r++)n[r]=e(t[r],r,t);return n}(r.childNodes,t)),n=e.g):i.a=r.nodeValue,i}(t)}function a(t){try{return t.id||(t.mo_id=t.mo_id||u++)}catch(e){try{return t.nodeValue}catch(t){return u++}}}function s(t,e){for(var n={},r=0;r<t.length;r++)n=e(n,t[r],r,t);return n}function A(t,e,n,r){for(;n<t.length;n++)if((r?t[n][r]:t[n])===e)return n;return-1}e._period=30,e.prototype={observe:function(t,n){for(var i={b:!!(n.attributes||n.attributeFilter||n.attributeOldValue),c:!!n.childList,g:!!n.subtree,a:!(!n.characterData&&!n.characterDataOldValue)},o=this.i,a=0;a<o.length;a++)o[a].s===t&&o.splice(a,1);n.attributeFilter&&(i.f=s(n.attributeFilter,function(t,e){return t[e]=!0,t})),o.push({s:t,o:r(t,i)}),this.h||function(t){!function n(){var r=t.takeRecords();r.length&&t.m(r,t),t.h=setTimeout(n,e._period)}()}(this)},takeRecords:function(){for(var t=[],e=this.i,n=0;n<e.length;n++)e[n].o(t);return t},disconnect:function(){this.i=[],clearTimeout(this.h),this.h=null}};var l=document.createElement("i");l.style.top=0;var c=(l="null"!=l.attributes.style.value)?function(t,e){return e.value}:function(t,e){return"style"!==e.name?e.value:t.style.cssText},u=1;return e}(void 0)},function(t,e,n){"use strict";var r=n(1272);t.exports=function(t,e,n){n=n||{},9===e.nodeType&&(e=r.getWindow(e));var i=n.allowHorizontalScroll,o=n.onlyScrollIfNeeded,a=n.alignWithTop,s=n.alignWithLeft,A=n.offsetTop||0,l=n.offsetLeft||0,c=n.offsetBottom||0,u=n.offsetRight||0;i=void 0===i||i;var g=r.isWindow(e),h=r.offset(t),d=r.outerHeight(t),f=r.outerWidth(t),p=void 0,I=void 0,C=void 0,m=void 0,y=void 0,E=void 0,v=void 0,B=void 0,b=void 0,Q=void 0;g?(v=e,Q=r.height(v),b=r.width(v),B={left:r.scrollLeft(v),top:r.scrollTop(v)},y={left:h.left-B.left-l,top:h.top-B.top-A},E={left:h.left+f-(B.left+b)+u,top:h.top+d-(B.top+Q)+c},m=B):(p=r.offset(e),I=e.clientHeight,C=e.clientWidth,m={left:e.scrollLeft,top:e.scrollTop},y={left:h.left-(p.left+(parseFloat(r.css(e,"borderLeftWidth"))||0))-l,top:h.top-(p.top+(parseFloat(r.css(e,"borderTopWidth"))||0))-A},E={left:h.left+f-(p.left+C+(parseFloat(r.css(e,"borderRightWidth"))||0))+u,top:h.top+d-(p.top+I+(parseFloat(r.css(e,"borderBottomWidth"))||0))+c}),y.top<0||E.top>0?!0===a?r.scrollTop(e,m.top+y.top):!1===a?r.scrollTop(e,m.top+E.top):y.top<0?r.scrollTop(e,m.top+y.top):r.scrollTop(e,m.top+E.top):o||((a=void 0===a||!!a)?r.scrollTop(e,m.top+y.top):r.scrollTop(e,m.top+E.top)),i&&(y.left<0||E.left>0?!0===s?r.scrollLeft(e,m.left+y.left):!1===s?r.scrollLeft(e,m.left+E.left):y.left<0?r.scrollLeft(e,m.left+y.left):r.scrollLeft(e,m.left+E.left):o||((s=void 0===s||!!s)?r.scrollLeft(e,m.left+y.left):r.scrollLeft(e,m.left+E.left)))}},function(t,e,n){"use strict";var r=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t},i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol?"symbol":typeof t};function o(t,e){var n=t["page"+(e?"Y":"X")+"Offset"],r="scroll"+(e?"Top":"Left");if("number"!=typeof n){var i=t.document;"number"!=typeof(n=i.documentElement[r])&&(n=i.body[r])}return n}function a(t){return o(t)}function s(t){return o(t,!0)}function A(t){var e=function(t){var e,n=void 0,r=void 0,i=t.ownerDocument,o=i.body,a=i&&i.documentElement;return n=(e=t.getBoundingClientRect()).left,r=e.top,{left:n-=a.clientLeft||o.clientLeft||0,top:r-=a.clientTop||o.clientTop||0}}(t),n=t.ownerDocument,r=n.defaultView||n.parentWindow;return e.left+=a(r),e.top+=s(r),e}var l=new RegExp("^("+/[\-+]?(?:\d*\.|)\d+(?:[eE][\-+]?\d+|)/.source+")(?!px)[a-z%]+$","i"),c=/^(top|right|bottom|left)$/,u="currentStyle",g="runtimeStyle",h="left",d="px";var f=void 0;function p(t,e){for(var n=0;n<t.length;n++)e(t[n])}function I(t){return"border-box"===f(t,"boxSizing")}"undefined"!=typeof window&&(f=window.getComputedStyle?function(t,e,n){var r="",i=t.ownerDocument,o=n||i.defaultView.getComputedStyle(t,null);return o&&(r=o.getPropertyValue(e)||o[e]),r}:function(t,e){var n=t[u]&&t[u][e];if(l.test(n)&&!c.test(e)){var r=t.style,i=r[h],o=t[g][h];t[g][h]=t[u][h],r[h]="fontSize"===e?"1em":n||0,n=r.pixelLeft+d,r[h]=i,t[g][h]=o}return""===n?"auto":n});var C=["margin","border","padding"],m=-1,y=2,E=1;function v(t,e,n){var r=0,i=void 0,o=void 0,a=void 0;for(o=0;o<e.length;o++)if(i=e[o])for(a=0;a<n.length;a++){var s=void 0;s="border"===i?i+n[a]+"Width":i+n[a],r+=parseFloat(f(t,s))||0}return r}function B(t){return null!=t&&t==t.window}var b={};function Q(t,e,n){if(B(t))return"width"===e?b.viewportWidth(t):b.viewportHeight(t);if(9===t.nodeType)return"width"===e?b.docWidth(t):b.docHeight(t);var r="width"===e?["Left","Right"]:["Top","Bottom"],i="width"===e?t.offsetWidth:t.offsetHeight,o=(f(t),I(t)),a=0;(null==i||i<=0)&&(i=void 0,(null==(a=f(t,e))||Number(a)<0)&&(a=t.style[e]||0),a=parseFloat(a)||0),void 0===n&&(n=o?E:m);var s=void 0!==i||o,A=i||a;if(n===m)return s?A-v(t,["border","padding"],r):a;if(s){var l=n===y?-v(t,["border"],r):v(t,["margin"],r);return A+(n===E?0:l)}return a+v(t,C.slice(n),r)}p(["Width","Height"],function(t){b["doc"+t]=function(e){var n=e.document;return Math.max(n.documentElement["scroll"+t],n.body["scroll"+t],b["viewport"+t](n))},b["viewport"+t]=function(e){var n="client"+t,r=e.document,i=r.body,o=r.documentElement[n];return"CSS1Compat"===r.compatMode&&o||i&&i[n]||o}});var w={position:"absolute",visibility:"hidden",display:"block"};function x(t){var e=void 0,n=arguments;return 0!==t.offsetWidth?e=Q.apply(void 0,n):function(t,e,n){var r={},i=t.style,o=void 0;for(o in e)e.hasOwnProperty(o)&&(r[o]=i[o],i[o]=e[o]);for(o in n.call(t),e)e.hasOwnProperty(o)&&(i[o]=r[o])}(t,w,function(){e=Q.apply(void 0,n)}),e}function _(t,e,n){var r=n;if("object"!==(void 0===e?"undefined":i(e)))return void 0!==r?("number"==typeof r&&(r+="px"),void(t.style[e]=r)):f(t,e);for(var o in e)e.hasOwnProperty(o)&&_(t,o,e[o])}p(["width","height"],function(t){var e=t.charAt(0).toUpperCase()+t.slice(1);b["outer"+e]=function(e,n){return e&&x(e,t,n?0:E)};var n="width"===t?["Left","Right"]:["Top","Bottom"];b[t]=function(e,r){if(void 0===r)return e&&x(e,t,m);if(e){f(e);return I(e)&&(r+=v(e,["padding","border"],n)),_(e,t,r)}}}),t.exports=r({getWindow:function(t){var e=t.ownerDocument||t;return e.defaultView||e.parentWindow},offset:function(t,e){if(void 0===e)return A(t);!function(t,e){"static"===_(t,"position")&&(t.style.position="relative");var n=A(t),r={},i=void 0,o=void 0;for(o in e)e.hasOwnProperty(o)&&(i=parseFloat(_(t,o))||0,r[o]=i+e[o]-n[o]);_(t,r)}(t,e)},isWindow:B,each:p,css:_,clone:function(t){var e={};for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n]);if(t.overflow)for(var n in t)t.hasOwnProperty(n)&&(e.overflow[n]=t.overflow[n]);return e},scrollLeft:function(t,e){if(B(t)){if(void 0===e)return a(t);window.scrollTo(e,s(t))}else{if(void 0===e)return t.scrollLeft;t.scrollLeft=e}},scrollTop:function(t,e){if(B(t)){if(void 0===e)return s(t);window.scrollTo(a(t),e)}else{if(void 0===e)return t.scrollTop;t.scrollTop=e}},viewportWidth:0,viewportHeight:0},b)},function(t,e,n){(function(e){(function(){var n,r,i,o,a,s;"undefined"!=typeof performance&&null!==performance&&performance.now?t.exports=function(){return performance.now()}:null!=e&&e.hrtime?(t.exports=function(){return(n()-a)/1e6},r=e.hrtime,o=(n=function(){var t;return 1e9*(t=r())[0]+t[1]})(),s=1e9*e.uptime(),a=o-s):Date.now?(t.exports=function(){return Date.now()-i},i=Date.now()):(t.exports=function(){return(new Date).getTime()-i},i=(new Date).getTime())}).call(this)}).call(this,n(58))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(66);e.PartOverlay=({partState:t,partStateDetail:e,onRequestRefresh:n,onRequestCancel:o})=>{let a=!1,s=!1,A=!0;switch(t){case"Calculating":a=!0,s=!0;break;case"Error":s=!0;break;case"Canceled":case"WaitForUser":s=!0,A=!1}const l=()=>{if(!(e instanceof Error))return r.createElement("pre",{className:"errorDetail"},(t=>{try{return JSON.stringify(t)}catch(e){return""+t}})(e));let t=r.createElement("pre",{className:"errorDetail"},e.stack);if(null!=e.prettyTraceback){const n=e.prettyTraceback;t=r.createElement("div",{style:{minWidth:"62em",background:"var(--jp-error-color3)"},className:"jp-RenderedText",dangerouslySetInnerHTML:{__html:n}})}return r.createElement("div",null,r.createElement("h2",null,e.message),t)};return r.createElement("div",{className:"m-PartOverlay-Component"},A?(()=>r.createElement("div",{className:"m-PartOverlay-Header"},r.createElement("span",{className:"m-PartOverlay-State m-PartOverlay-center"},t)))():null,(()=>{switch(t){case"Calculating":return r.createElement("div",{className:"m-PartOverlay-center"},r.createElement(i.ProgressBar,null),e.map(t=>r.createElement("p",{key:t},"Evaluating ",t)));case"Rendering":return r.createElement("div",{className:"m-PartOverlay-center"},"...");case"Error":return l();default:return null}})(),s?(t=>{const e=t?"fa-times":"fa-refresh",i="m-PartOverlay-Button"+(A?"":" m-PartOverlay-BigButton");return r.createElement("button",{className:i,onClick:()=>t?o():n()},r.createElement("i",{className:"fa "+e}),r.createElement("span",{className:"m-PartOverlay-ButtonContent"},t?"Cancel":"Refresh"))})(a):null)}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(64),o=n(108),a=n(113);e.OptionsBag=class{constructor(t){this.optionsMap=new Map,this.staleOptions=new Map,this.errorOptions=new Map,this.OnOptionChangedSrc$=new i.Subject,this._isStale=!1,this._isDisposed=!1;for(const e of t)this.optionsMap.set(e.name,Object.freeze({...e}));this.OnOptionChanged=this.OnOptionChangedSrc$.pipe(a.debounceTime(4,i.asyncScheduler))}get isDisposed(){return this._isDisposed}get isStale(){return this._isStale}[Symbol.iterator](){return this.optionsMap.values()}values(){return r.IterTools.map(this.optionsMap.values(),t=>t.value)}setStale(t){if(null!=t)for(const e of t){let[t,n]=e;this.staleOptions.has(t)&&(n=this.mergeStaleness(n)),this.staleOptions.set(t,n)}this._isStale=!0}setAllOptionsStale(){this.setStale(this.optionsMap.entries())}setError(t){for(const e of t)this.errorOptions.set(e,this.getMetadata(e))}clearError(){this.errorOptions.clear()}setFresh(){this.staleOptions.clear(),this._isStale=!1}dispose(){this._isDisposed||(this._isDisposed=!0,this.OnOptionChangedSrc$.complete(),this.staleOptions.clear(),this.errorOptions.clear(),this.optionsMap.clear())}get(t){return this.ensureOption(t),this.optionsMap.get(t).value}set(t,e){this.ensureOption(t);const n=this.optionsMap.get(t);if(o.isEqual(n.value,e))return void console.debug("[OptionsBag]","Ignoring option change- refs match");const r=Object.freeze({...n,value:e});this.optionsMap.set(t,r),this.setStale([[t,n]]),this.OnOptionChangedSrc$.next({option:r,isStale:!0})}setBindingValue(t,e){this.ensureOption(t);const n=this.optionsMap.get(t);if(o.isEqual(n.value,e))return void console.debug("[OptionsBag]","Ignoring option change- refs match");const r=Object.freeze({...n,value:e});this.optionsMap.set(t,r),this.errorOptions.delete(t),this.staleOptions.delete(t),this._isStale=0!==this.staleOptions.size,this.OnOptionChangedSrc$.next({option:r,isStale:!1})}getMetadata(t){return this.ensureOption(t),this.optionsMap.get(t)}setBinding(t,e,n,r){this.ensureOption(t);const i=this.optionsMap.get(t),o={type:e,expr:n,globals:r},a=Object.freeze({...i,binding:Object.freeze(o)});this.optionsMap.set(t,a),this.setStale([[t,i]]),this.OnOptionChangedSrc$.next({option:a,isStale:!0})}clearBinding(t){this.ensureOption(t);const e=this.optionsMap.get(t),n=Object.freeze({...e,binding:void 0});this.optionsMap.set(t,n),this.setStale([[t,e]]),this.OnOptionChangedSrc$.next({option:n,isStale:!0})}getStaleOptions(){return r.IterTools.mergeUnique(this.staleOptions.keys(),this.errorOptions.keys())}*getStaleOptionValues(){for(const t of this.getStaleOptions()){const e=this.staleOptions.has(t)?this.staleOptions:this.errorOptions;yield[t,e.get(t)]}}ensureOption(t){if(!this.optionsMap.has(t))throw Error(t+" is not a valid option!")}mergeStaleness(t){const e=this.staleOptions.get(t.name);return null==e?t:Object.freeze({name:e.name,type:e.type,binding:e.binding,value:t.value})}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9),i=n(43),o=n(676);class a{constructor(){this.globalFactory=new o.PartFactory,this.partFactoryProperty=new i.AttachedProperty({name:"partFactory",create:()=>new o.PartFactory(this.globalFactory)})}get(t){return this.partFactoryProperty.get(t)}registerPart(t,e){this.globalFactory.registerPart(t,e)}get root(){return this.globalFactory}}e.PartFactoryExtension=a,e.IPartFactory=new r.Token("@mavenomics/parts:IPartFactory"),e.factoryExtPlugin={id:"@mavenomics/parts:part-factory-plugin",autoStart:!0,provides:e.IPartFactory,activate:()=>new a}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(337),o=n(108);!function(t){t.MIMETYPE="application/vnd.maven.part+json",t.toJson=function(t,e){if(t instanceof i.ErrorPart)return t.originalModel||{name:t.typeName,id:t.uuid,options:{}};const n={},a=t.constructor.GetMetadata();for(const t of a){const i=e.getMetadata(t.name);null!=i.binding?n[i.name]=i.binding:o.isEqual(i.value,t.value)||(n[i.name]=r.Converters.serialize(i.value,i.type))}return{name:t.getName(),id:t.uuid,options:n}}}(e.PartSerializer||(e.PartSerializer={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(9),i=n(221),o=n(17),a=n(66),s=n(6),A=n(25),l=n(28),c=n(0);!function(t){t.isUDPWrapper=function(t){return null!=t.configObject&&"Part"===t.configObject.typeName};class e extends i.Part{constructor({obj:t,path:e,...n}){super(n),this.onInitialized=new r.PromiseDelegate,this.onFrameSetup=new r.PromiseDelegate,this.hasInitialized=!1,this.hasSetupFrame=!1,this.mustWaitForSetup=!1,this.frameSetupMutex=new l.AsyncTools.Mutex,this.path=e,this.configObject=t,this.title.label=t.name,this.frame=new s.Widget({node:document.createElement("iframe")}),Object.assign(this.frame.node.style,{width:"100%",height:"100%",border:"0"})}async initialize(){null!=this.udpThis?(this.udpThis.context=new n(this.context.hover,this,this.context.baseViewUrl),await this.udpThis.initialize()):this.mustWaitForSetup=!0}async render(t){this.mustWaitForSetup&&(await this.onFrameSetup.promise,this.mustWaitForSetup=!1,this.udpThis.context=new n(this.context.hover,this,this.context.baseViewUrl),await this.udpThis.initialize()),this.udpThis.context.__setOptionsBag(t),await this.udpThis.render([...t.values()])}dispose(){this.isDisposed||(null!=this.bubbleHandler&&this.bubbleHandler.dispose(),delete this.udpThis,this.frame.dispose(),super.dispose())}async onAfterAttach(){this.hasInitialized?await this.maybeRefreshFrame():(this.hasInitialized=!0,await this.frameSetupMutex.aquire(),this.setupFrame());const t=this.frame.node;this.bubbleHandler=l.FrameTools.bubbleEvents(t.contentDocument,t)}onAfterShow(){this.hasInitialized&&this.maybeRefreshFrame()}onBeforeDetach(){null!=this.bubbleHandler&&this.bubbleHandler.dispose()}async maybeRefreshFrame(){if(!this.isFrameValid())if(this.frameSetupMutex.isFree){await this.frameSetupMutex.aquire(),this.frame.dispose(),this.hasSetupFrame&&(this.hasSetupFrame=!1,this.onInitialized=new r.PromiseDelegate,this.onFrameSetup=new r.PromiseDelegate),A.MessageLoop.sendMessage(this,i.Part.Lifecycle.BeforeInitialize),this.frame=new s.Widget({node:document.createElement("iframe")}),Object.assign(this.frame.node.style,{width:"100%",height:"100%",border:"0"}),this.setupFrame();try{await this.onFrameSetup.promise,await this.initialize(),A.MessageLoop.sendMessage(this,i.Part.Lifecycle.AfterInitialize),this.refresh()}catch(t){this.error(t,"init-error")}}else await this.frameSetupMutex.lock}isFrameValid(){const t=this.frame.node;return null!=t.contentDocument&&null!=t.contentWindow&&t.contentWindow.hasOwnProperty("onInitialized")&&t.contentDocument.body.dataset.udpId===this.uuid&&null!=t.contentDocument.getElementById("udp-js")}setupFrame(){this.layout.insertWidget(0,this.frame);const t=this.frame.node;t.contentWindow.onInitialized=this.onInitialized,t.contentDocument.open();let e=(t,e)=>(Object.keys(e).forEach(n=>{t=t.replace(`%${n}%`,e[n])}),t);const n=new URL(this.context.baseUrl),r=o.PathExt.dirname(this.path);let i={PART_DIR:r,ORIGIN:n.origin,BASE_HREF:o.URLExt.join(n.href,r||"")},a=e(this.configObject.data.htmlText,i),s=e(this.configObject.data.cssText,i),A=Object.keys(i).reduce((t,e)=>`${t}window.${e} = "${i[e]}";\n`,"");t.contentDocument.write(`\n<!doctype html>\n<html>\n    <head>\n        <base href="${i.BASE_HREF}/" />\n    </head>\n    <body data-udp-id=${this.uuid}>\n        ${a}\n        <script id="udp-js">\n        ${A}\n        const userThis = {\n            initialize() {\n                console.log("default initialize");\n            },\n            render() {\n                console.log("default render");\n            }\n        };\n        (function() {\n            ${this.configObject.data.jsText};\n        }).call(userThis);\n        window.onInitialized.resolve(userThis);\n        <\/script>\n        <style id="udp-style">${s}</style>\n    </body>\n</html>`),t.contentDocument.close(),this.onInitialized.promise.then(t=>{this.udpThis=t,this.hasSetupFrame=!0,this.frameSetupMutex.release(),this.onFrameSetup.resolve()})}}t.UDPWrapperPart=e,t.generateWrapper=function(t,n){class r extends e{static GetMetadata(){const e=super.GetMetadata();for(const n of t.arguments)e.addOption(n.name,l.Types.findType(n.typeAnnotation)||l.Types.Any,l.Converters.deserialize(n.defaultValue));return e}constructor(e){super({obj:t,path:n,...e})}getName(){return t.name}}return Object.defineProperty(r,"name",{value:t.name,configurable:!1,writable:!1}),r};class n{constructor(t,e,n){this.hover=t,this.parent=e,this.baseViewUrl=n}__setOptionsBag(t){this.optionsBag=t}get(t){return this.optionsBag?this.optionsBag.get(t):null}set(t,e){return this.optionsBag?this.optionsBag.set(t,e):void 0}getFrame(t,e){const n=new s.Widget,r=document.createElement("iframe");return r.src=o.URLExt.join(this.baseViewUrl,t)+"#embed=true;"+e.map(t=>{const e=l.Converters.serialize(t.value,l.Converters.inferType(t.value));return{name:t.name,value:e&&e.value}}).map(t=>`${t.name}=${JSON.stringify(t.value)}`).join(";"),r.style.width="100%",r.style.height="100%",n.node.appendChild(r),n}openHtmlHover(t,e){const n=new s.Widget,r=document.createElement("iframe");r.style.width="100%",r.style.height="100%",r.style.margin="0",r.style.border="0";let i=this.hover.openHover({...e,hover:n,owner:this.parent});return r.onload=()=>{let n=r.contentWindow;n.document.body.style.width="100%",n.document.body.style.height="100%",n.document.body.style.margin="0";let o=n.document;o.write(t),o.close(),o.body.style.margin="0";let a=e.width||o.body.scrollWidth,s=e.height||o.body.scrollHeight;r.width=a.toString(),r.height=s.toString(),this.hover.resizeHover(i,a,s)},n.node.appendChild(r),i}openHover(t){return this.hover.openHover({...t,owner:this.parent})}async openPropertiesEditor(t,e){const n=t.map(t=>{const e=l.Types.findType(t[1].type)||l.Types.Any;return[t[0],{...t[1],type:e}]}),r={...e},i=c.createElement(a.PropertiesEditor,{properties:n,renderEditor:(t,e)=>c.createElement(a.TypeEditor,{key:t,value:r[t],onValueChanged:e=>{r[t]=e},type:e.type})}),o=a.ReactWrapperWidget.Create(i);if(!(await a.HoverManager.Instance.launchDialog(o,this.parent,400,800,"Edit Properties",[{text:"Dismiss"},{text:"Ok",accept:!0}])).accept)throw Error("Cancel");return r}}}(e.JavascriptEvalPart||(e.JavascriptEvalPart={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(6),i=n(0),o=n(66),a=n(221),s=n(25);e.ReactPart=class extends a.Part{constructor(t){super(t),this.container=new A(i.createElement("span")),this.container.addClass("m-ReactPart-Container"),this.layout.insertWidget(0,this.container)}async initialize(){this.container.el=await this.initializeReact(),s.MessageLoop.sendMessage(this.container,r.Widget.Msg.UpdateRequest),s.MessageLoop.flush()}async render(t){this.container.el=await this.renderReact(t),s.MessageLoop.sendMessage(this.container,r.Widget.Msg.UpdateRequest),s.MessageLoop.flush()}dispose(){this.isDisposed||super.dispose()}initializeReact(){return i.createElement("span")}};class A extends o.ReactWrapperWidget{constructor(t){super(),this.el=t}render(){return this.el}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(66);e.PartServices=class{constructor({session:t,rendermime:e,dashboardId:n,baseUrl:i,baseViewUrl:o}){this.isDisposed=!1,this._hover=r.HoverManager.GetManager(),this._session=t,this._rendermime=e,this._dashboardId=n,this._baseUrl=i,this._baseViewUrl=o}get hover(){return this._hover}get session(){return this._session||null}get rendermime(){return this._rendermime||null}get dashboardId(){return this._dashboardId}get baseUrl(){return this._baseUrl}get baseViewUrl(){return this._baseViewUrl}dispose(){this.isDisposed||(this.isDisposed=!0)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(t){t.INPUT_OPTION="Input Table",t.VALUE_OPTION="Value",t.isTablePart=function(e){return null!=e.constructor.GetMetadata().getMetadataForOption(t.INPUT_OPTION)},t.isValuePart=function(e){const n=e.constructor.GetMetadata();return null!=n.getMetadataForOption(t.VALUE_OPTION)||null!=n.getMetadataForOption(t.INPUT_OPTION)},t.getValueOption=function(e){const n=e.constructor.GetMetadata();return n.getMetadataForOption(t.VALUE_OPTION)||n.getMetadataForOption(t.INPUT_OPTION)||null}}(e.PartUtils||(e.PartUtils={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(64),i=n(28),o=n(108);e.GlobalsService=class{constructor(){this.OnChangeSrc$=new r.Subject,this.OnDirtySrc$=new r.Subject,this.globalValues=new Map,this._isDirty=!1,this._isDisposed=!1,this.OnChange=this.OnChangeSrc$.asObservable(),this.OnDirty=this.OnDirtySrc$.asObservable()}get isDirty(){return this._isDirty}get isDisposed(){return this._isDisposed}setClean(){this._isDirty=!1}dispose(){this._isDisposed||(this._isDisposed=!0,this.OnChangeSrc$.complete(),this.OnDirtySrc$.complete(),this.globalValues.clear())}[Symbol.iterator](){return this.globalValues.values()}get(t){return this._ensureGlobalExists(t),this.globalValues.get(t).value}getType(t){return this._ensureGlobalExists(t),this.globalValues.get(t).type}set(t,e){this._ensureGlobalExists(t);const n=this.globalValues.get(t);o.isEqual(n.value,e)?console.debug("[GlobalsService]","Ignoring global change- references match"):this._setOrAddGlobal(t,n.type,e,"update")}has(t){return this.globalValues.has(t)}addGlobal(t,e,n){if(this.globalValues.has(t))throw Error("A global already exists having the name "+t);this._setOrAddGlobal(t,e,n,"add")}removeGlobal(t){this._ensureGlobalExists(t);const e=this.globalValues.get(t);this.globalValues.delete(t),console.debug("[GlobalsService]","Global value deleted"),this.OnChangeSrc$.next(Object.freeze({...e,action:"delete",oldName:e.name,oldType:e.type,oldValue:e.value})),this.setDirty()}changeType(t,e){this._ensureGlobalExists(t);const n=this.globalValues.get(t);if(n.type.equals(e))return;let r=null;try{r=i.Converters.convert(n.value,n.type,e)}catch(r){console.debug("[GlobalsService]",`Cannot convert ${t} from`,`${n.type.serializableName} to ${e.serializableName}, coercing to null`),console.debug(r)}this._setOrAddGlobal(t,e,r,"cast")}renameGlobal(t,e){if(this._ensureGlobalExists(t),this.globalValues.has(e))throw Error("A global already exists having the name "+e);const n=this.globalValues.get(t);this._setOrAddGlobal(e,n.type,n.value,"rename")}clearGlobals(){for(const t of this.globalValues.keys())this.removeGlobal(t)}_ensureGlobalExists(t){if(!this.globalValues.has(t))throw Error("No global named "+t+" exists")}_setOrAddGlobal(t,e,n,r){const i=Object.freeze({name:t,value:n,type:e}),o={...i,action:r};if("add"!==r){const e=this.globalValues.get(t);o.oldName=e.name,o.oldType=e.type,o.oldValue=e.value}this.globalValues.set(t,i);const a=r.endsWith("e")?r+"d":"add"===r?"added":"cast";console.debug("[GlobalsService]","Global value",a),this.OnChangeSrc$.next(Object.freeze(o)),this.setDirty()}setDirty(){this._isDirty||(this._isDirty=!0,this.OnDirtySrc$.next())}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(108),i=n(677),o=n(678);function a(t,e="\\/\\/",n="\\/\\*",r="\\*\\/"){if(t=t.trim(),null!=e){const n=new RegExp(String.raw`^[^${e}]*${e}.*$`);let r=t.match(n);if(r)return r[1].trim()}if(n&&r){const e=new RegExp(String.raw`^[^${n}]*${n}([^${r}]*)${r}`);let i=t.match(e);if(i)return i[1].trim()}return null}e.getCommentLine=a;const s=/[A-Za-z][A-Za-z0-9_]*/,A=new RegExp("@("+s.source+")");function l(t){let e=null,n=[],r=new RegExp(A.source,"g");for(;e=r.exec(t);)n.push(e[1]);return n}e.getGlobalsFromComment=l;class c extends i.Disposable{disposed(){}getMetadata(){}}e.BaseBindingsEvaluator=c;class u extends c{constructor(t,e){super(),this.evaluator=e}async evaluate(t,e,n){return await this.evaluator.evaluate(e,n)}cancel(t){}dispose(){this._isDisposed||(this.evaluator.dispose(),super.dispose())}get name(){return"Eval"}getGlobalsForBinding(t){const e=a(t,"#",null,null);if(null!=e)return l(e);let n,r=new RegExp(A.source,"g");const i=[];for(;null!=(n=r.exec(t||""));)i.push(n[1]);return i}getMetadata(){return this.evaluator.getMetadata()}}class g extends c{constructor(t,e){super(),this.globals=t,this.pool=e}async evaluate(t,e,n){return this.pool.runJs(t,e,n)}cancel(t){this.pool.cancelTasks(t)}getGlobalsForBinding(t){const e=a(t);if(null!=e)return l(e);let n=null;const r=[],i=new RegExp("globals.("+s.source+")","g");for(;n=i.exec(t);)r.push(n[1]);return r}get name(){return"JavaScript"}disposed(){}getMetadata(){return{editorMode:"MQL.js"}}}class h extends c{constructor(t,e){super(),this.globals=t,this.pool=e}async evaluate(t,e,n){return this.pool.runMql(t,e,n)}cancel(t){this.pool.cancelTasks(t)}get name(){return"Mql"}getGlobalsForBinding(t){const e=a(t,null);if(null!=e)return l(e);let n;const r=new RegExp(A.source,"g"),i=new Set;for(;n=r.exec(t);)i.add(n[1]);const o=new RegExp(String.raw`^(?:set|def)\s+${A.source}`,"g");for(;n=o.exec(t);)i.delete(n[1]);return[...i]}disposed(){}getMetadata(){return{editorMode:"mql"}}}class d extends c{constructor(t){super(),this.globals=t}async evaluate(t,e,n){return this.globals.get(e)}get name(){return"Global"}getGlobalsForBinding(t){return[t]}cancel(t){}}class f extends c{evaluate(t,e,n){return Promise.resolve(null)}get name(){return"None"}getGlobalsForBinding(t){return[]}cancel(t){}}e.BindingsProvider=class extends i.Disposable{constructor(t,e){super(),this.workerPool=new o.MqlWorkerPool(t,e,8,15e3);let n=[new d(t),new f,new g(t,this.workerPool),new h(t,this.workerPool)];null!=e&&n.push(new u(t,e)),this.evaluators=n.reduce((t,e)=>(t[e.name]=e,t),{})}getBindingNames(){return r.values(this.evaluators).map(t=>t.name)}getBindingEvaluator(t){return this.evaluators[t]||new p(t)}disposed(){for(let t of r.values(this.evaluators))t.dispose();this.workerPool.dispose()}};class p extends c{constructor(t){super(),this.type=t}get name(){throw new Error("Not Supported")}evaluate(t,e){return Promise.reject("Unknown binding type: "+this.type)}cancel(){}getGlobalsForBinding(t){return[]}}e.ErrorBindingsEvaluator=p},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(64),i=n(1285);e.WorkerWrapper=class{constructor(){this.onMessageSrc=new r.Subject,this._isDisposed=!1,this.worker=new i,this.worker.onmessage=t=>{const e=JSON.parse(t.data);this.onMessageSrc.next(e)}}get isDisposed(){return this._isDisposed}get onMessage(){return this.onMessageSrc}postMessage(t){this.worker.postMessage(JSON.stringify(t))}dispose(){this._isDisposed||(this.worker.terminate(),this.onMessageSrc.complete(),this._isDisposed=!0)}terminate(){this.dispose()}}},function(t,e,n){t.exports=function(){return new Worker(n.p+"320c4e08fbbbd271691a.worker.js")}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(66);function o({caption:t,color:e,background:n}){return r.createElement("span",{className:"m-TitleBar",style:{color:e,background:n}},t)}e.TitleBar=o,function(t){t.EditableTitleBar=({caption:t,color:e,background:n,captionChanged:i,onBlur:o})=>{return r.createElement("span",{className:"m-TitleBar m-TitleBar-editable",style:{color:e,background:n}},r.createElement("input",{type:"text",className:"m-TitleBar-editableInput",value:t,onChange:t=>i.call(void 0,t.target.value),onFocus:({target:t})=>t.select(),onBlur:()=>o.call(void 0),onKeyDown:t=>{"Enter"!==t.key&&"Escape"!==t.key||o.call(void 0)},autoFocus:!0}))};t.WidgetWrapper=class extends i.ReactWrapperWidget{constructor(t){super(),this.dragStartPosition=null,this.editable=!1,this.addClass("m-TitleBar-Wrapper"),this.owner=t,this.widgetTitle=this.owner.title,this.widgetTitle.changed.connect(this.update,this)}dispose(){this.isDisposed||(this.widgetTitle.changed.disconnect(this.update,this),super.dispose())}edit(){this.editable=!0,this.update()}handleEvent(t){switch(t.type){case"pointerdown":this.onStartDrag(t);break;case"pointermove":this.onMove(t);break;case"pointerup":this.endDrag();break;default:throw Error("Unhandled event")}}render(){if(this.isDisposed)throw Error("TitleBarWidget disposed");let e=this.widgetTitle.dataset.color;"transparent"===e&&(e="");let n=this.widgetTitle.dataset.background;return"transparent"===n&&(n=""),this.editable?r.createElement(t.EditableTitleBar,{caption:this.widgetTitle.label,color:e,background:n,captionChanged:t=>{this.owner.setLayoutProperty("caption",t)},onBlur:()=>{this.editable=!1,this.update()}}):r.createElement(t,{caption:this.widgetTitle.label,color:e,background:n})}onAfterAttach(){this.node.addEventListener("pointerdown",this)}onBeforeDetach(){this.node.removeEventListener("pointerdown",this)}onStartDrag(t){"mouse"===t.pointerType&&0!==t.button||this.editable||(this.dragStartPosition=[t.clientX,t.clientY],document.addEventListener("pointermove",this,{capture:!0}),document.addEventListener("pointerup",this,{capture:!0}))}onMove(t){this.node.contains(t.target)?Math.abs(t.clientX-this.dragStartPosition[0])+Math.abs(t.clientY-this.dragStartPosition[1])>2&&(this.endDrag(),this.owner.startDrag(t.clientX,t.clientY)):this.endDrag()}endDrag(){document.removeEventListener("pointerup",this,{capture:!0}),document.removeEventListener("pointermove",this,{capture:!0}),this.dragStartPosition=null}}}(o=e.TitleBar||(e.TitleBar={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(6);var i;e.DockPreview=class extends r.Widget{constructor({owner:t}){super(),this.owner=t,this.addClass("m-DockPreview"),this.previewNode=document.createElement("div"),this.previewNode.classList.add("m-DockPreview__overlay"),this.node.appendChild(this.previewNode)}startPreviewFor(t,e){const n=t.slice(t.lastIndexOf("-")+1);let r=t.startsWith("far")?this.owner.root:this.owner.getRegion(e);if(null==r)throw Error("Cannot display preview for non-existent region "+r);t.startsWith("outer")&&(r=r.parentRegion||r),this.show();const i=r.node.getBoundingClientRect();this.renderPreview(i,n)}stopPreview(){this.hide()}renderPreview(t,e){const n=this.previewNode.style;let{width:r,height:o,top:a,left:s}=t;const A=this.node.getBoundingClientRect();switch(a-=A.top,s-=A.left,e){case i.Direction.Left:r/=2;break;case i.Direction.Right:s+=r/=2;break;case i.Direction.Top:o/=2;break;case i.Direction.Bottom:a+=o/=2}Object.assign(n,{width:r+"px",height:o+"px",top:a+"px",left:s+"px"})}},function(t){let e;!function(t){t.Left="left",t.Right="right",t.Top="top",t.Bottom="bottom"}(e=t.Direction||(t.Direction={}))}(i||(i={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(64),o=n(66);e.UrlParametersManager=class{constructor(t){this.filter=t,this.paramsDidChangeSrc$=new i.ReplaySubject(1),this.params=[],this.paramsDidChange=this.paramsDidChangeSrc$.asObservable(),window.addEventListener("hashchange",()=>{this.fetchParameters(),this.paramsDidChangeSrc$.next()},{capture:!1})}fetchParameters(){this.params=[];let t=decodeURIComponent(window.location.hash);if(t.startsWith("#")&&(t=t.slice(1)),""!==t)for(const e of t.split(";")){const[t,n]=e.split("=");if(""===t||""===n)continue;if(this.filter&&this.filter.filterParameter(t,n))continue;let r=null;try{r=n}catch(e){console.warn("Couldn't parse parameter",t),console.warn(e);continue}this.params.push({name:t,value:r})}}applyParameters(t){const e=[];for(const n of this.params){let i,o;try{i=JSON.parse(n.value)}catch(t){e.push([n.name,t]);continue}try{const a=t.getType(n.name);o=r.Converters.deserialize({typeName:a.serializableName,value:i})}catch(t){e.push([n.name,t]);continue}t.set(n.name,o)}if(e.length>0){const t=e.map(([t,e])=>`<h3>Parsing ${t} failed</h3>\n                <pre>${(e instanceof Error?e:Error(e)).stack}</pre>`).join("");o.HoverManager.Instance.openErrorDialog(t,!0)}}deserializeParameters(t){if(null==t.globals||0===t.globals.length)return{};const e=[];let n={};for(const i of this.params){let o,a;try{o=JSON.parse(i.value)}catch(t){e.push([i.name,t]);continue}try{const s=t.globals.find(t=>t.name===i.name);if(null==s)throw Error("No global named "+i.name+" exists");const A=r.Types.findType(s.type)||r.Types.Any;a=r.Converters.deserialize({typeName:A.serializableName,value:o}),n[s.name]=a}catch(t){e.push([i.name,t]);continue}}if(e.length>0){const t=e.map(([t,e])=>`<h3>Parsing ${t} failed</h3>\n                <pre>${(e instanceof Error?e:Error(e)).stack}</pre>`).join("");o.HoverManager.Instance.openErrorDialog(t,!0)}return n}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(342),o=n(191),a=n(686),s=n(685);class A extends r.PartialConverter{serialize(t){let e=new l(t);const n={rows:new Array(t.rows.length),cols:t.columnNames,types:t.columnTypes.map(t=>t.serializableName)};for(let r=0;r<t.rows.length;r++){const i=t.rows[r];n.rows[r]=e.serialize(i)}return n}deserialize(t){if(t.arrow){const e=a.Table.from([o.toByteArray(t.data.trim())]);return s.ArrowConverter.fromArrow(e)}let e=new i.Table;e.setColumns(t.cols,t.types?t.types.map(t=>r.Types.findType(t)):null);let n=new l(e);for(let r=0;r<t.rows.length;r++){const i=t.rows[r],o=n.deserialize(i.hasOwnProperty("value")?i.value:i);e.appendRow(o)}return e}inferInstanceOf(t){return t instanceof i.Table?1:-1}}A.type=r.Types.Table;class l extends r.PartialConverter{constructor(t,e=null){super(),this._parentTable=t,this._parentRow=e}serialize(t){const e={data:new Array(t._data.length)};null!=t.name&&(e.name=t.name);for(let n=0;n<t._data.length;n++){const i=t._data[n];let o=t.owner.columnTypes[n],a=!0;o===r.Types.Any&&(o=r.Converters.inferType(i),a=!1);const s=r.Converters.serialize(i,o);e.data[n]=a&&null!=s?s.value:s}if(0===t.children.length)return e;e.children=new Array(t.children.length);for(let n=0;n<t.children.length;n++){const r=t.children[n];e.children[n]=this.serialize(r)}return e}deserialize(t){let e=new i.Row(this._parentTable,t.name||null);e._setParent(this._parentRow),null!=this._parentRow&&this._parentRow.appendChild(e);let n=new l(this._parentTable,e);if(null!=t.children)for(let e=0;e<t.children.length;e++){let r,i=t.children[e];r=null!=i&&i.hasOwnProperty("typeName")?i.value:i,n.deserialize(r)}for(let n=0;n<t.data.length;n++){let i=t.data[n],o=i;null==i||i.hasOwnProperty("typeName")||(o={typeName:this._parentTable.columnTypes[n].serializableName,value:i}),e._data[n]=r.Converters.deserialize(o)}return e}inferInstanceOf(t){return t instanceof i.Row?1:-1}}l.type=r.Types.Row,r.Converters.registerConverter(A)},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(1291),i=n(1323);e.default=[r.inputPartsPlugin,i.outputPartsPlugin]},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(48),i=n(1292),o=n(1308),a=n(1309),s=n(1310),A=n(1311),l=n(1312),c=n(1313),u=n(1314),g=n(1315),h=n(1316);e.inputPartsPlugin={id:"@mavenomics/default-parts:input",autoStart:!0,requires:[r.IPartFactory],activate:(t,e)=>{e.registerPart("SliderPart",i.SliderPart),e.registerPart("RangeSliderPart",o.RangeSliderPart),e.registerPart("SelectPart",a.SelectPart),e.registerPart("RadioButtonPart",u.RadioButtonPart),e.registerPart("CheckboxPart",g.CheckboxPart),e.registerPart("DatePickerPart",s.DatePickerPart),e.registerPart("TextInputPart",A.TextInputPart),e.registerPart("TextAreaPart",c.TextAreaPart),e.registerPart("AnimationSliderPart",l.AnimationSliderPart),e.registerPart("CodeEditorPart",h.CodeEditorPart)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(349),o=n(0),a=n(48),s=i.createSliderWithTooltip(i.default);e.SliderPart=class extends a.ReactPart{static GetMetadata(){const t=super.GetMetadata();return t.description="Display a numeric slider",t.addOption("Value",r.Types.Number,5),t.addOption("Min",r.Types.Number,0),t.addOption("Max",r.Types.Number,10),t.addOption("Step",r.Types.Number,1),t}constructor(t){super(t),this.container.node.style.padding="10px 20px"}initializeReact(){return o.createElement(s,{defaultValue:5,min:0,max:10,step:1})}renderReact(t){const e=t.get("Value"),n=t.get("Min"),r=t.get("Max"),i=t.get("Step");let a=i;(r-n)/i>10&&(a*=Math.floor((r-n)/(10*i)));const A={};for(let t=n;t<r;t+=a)A[t]=t.toLocaleString();return A[r]=r.toLocaleString(),o.createElement(s,Object.assign({},{value:e,min:n,max:r,step:i,marks:A},{onChange:e=>{t.set("Value",e)}}))}}},function(t,e,n){t.exports={default:n(1294),__esModule:!0}},function(t,e,n){n(1295),t.exports=n(71).Object.getPrototypeOf},function(t,e,n){var r=n(179),i=n(646);n(687)("getPrototypeOf",function(){return function(t){return i(r(t))}})},function(t,e,n){t.exports={default:n(1297),__esModule:!0}},function(t,e,n){n(1298);var r=n(71).Object;t.exports=function(t,e){return r.getOwnPropertyDescriptor(t,e)}},function(t,e,n){var r=n(122),i=n(328).f;n(687)("getOwnPropertyDescriptor",function(){return function(t,e){return i(r(t),e)}})},function(t,e,n){t.exports={default:n(1300),__esModule:!0}},function(t,e,n){n(643),n(1301),t.exports=n(71).Array.from},function(t,e,n){"use strict";var r=n(315),i=n(109),o=n(179),a=n(1302),s=n(1303),A=n(642),l=n(1304),c=n(1305);i(i.S+i.F*!n(1307)(function(t){Array.from(t)}),"Array",{from:function(t){var e,n,i,u,g=o(t),h="function"==typeof this?this:Array,d=arguments.length,f=d>1?arguments[1]:void 0,p=void 0!==f,I=0,C=c(g);if(p&&(f=r(f,d>2?arguments[2]:void 0,2)),null==C||h==Array&&s(C))for(n=new h(e=A(g.length));e>I;I++)l(n,I,p?f(g[I],I):g[I]);else for(u=C.call(g),n=new h;!(i=u.next()).done;I++)l(n,I,p?a(u,f,[i.value,I],!0):i.value);return n.length=I,n}})},function(t,e,n){var r=n(143);t.exports=function(t,e,n,i){try{return i?e(r(n)[0],n[1]):e(n)}catch(e){var o=t.return;throw void 0!==o&&r(o.call(t)),e}}},function(t,e,n){var r=n(180),i=n(88)("iterator"),o=Array.prototype;t.exports=function(t){return void 0!==t&&(r.Array===t||o[i]===t)}},function(t,e,n){"use strict";var r=n(111),i=n(178);t.exports=function(t,e,n){e in t?r.f(t,e,i(0,n)):t[e]=n}},function(t,e,n){var r=n(1306),i=n(88)("iterator"),o=n(180);t.exports=n(71).getIteratorMethod=function(t){if(null!=t)return t[i]||t["@@iterator"]||o[r(t)]}},function(t,e,n){var r=n(317),i=n(88)("toStringTag"),o="Arguments"==r(function(){return arguments}());t.exports=function(t){var e,n,a;return void 0===t?"Undefined":null===t?"Null":"string"==typeof(n=function(t,e){try{return t[e]}catch(t){}}(e=Object(t),i))?n:o?r(e):"Object"==(a=r(e))&&"function"==typeof e.callee?"Arguments":a}},function(t,e,n){var r=n(88)("iterator"),i=!1;try{var o=[7][r]();o.return=function(){i=!0},Array.from(o,function(){throw 2})}catch(t){}t.exports=function(t,e){if(!e&&!i)return!1;var n=!1;try{var o=[7],a=o[r]();a.next=function(){return{done:n=!0}},o[r]=function(){return a},t(o)}catch(t){}return n}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(48),o=n(349),a=n(0),s=o.createSliderWithTooltip(o.Range);e.RangeSliderPart=class extends i.ReactPart{static GetMetadata(){const t=super.GetMetadata();return t.description="Display a double-headed slider",t.addOption("Start",r.Types.Number,4),t.addOption("End",r.Types.Number,6),t.addOption("Min",r.Types.Number,0),t.addOption("Max",r.Types.Number,10),t.addOption("Step",r.Types.Number,1),t}constructor(t){super(t),this.container.node.style.padding="10px 20px"}initializeReact(){return a.createElement(o.Range,{defaultValue:[4,6],min:0,max:10,step:1})}renderReact(t){const e=[t.get("Start"),t.get("End")],n=t.get("Min"),r=t.get("Max"),i=t.get("Step");let o=i;(r-n)/i>10&&(o*=Math.floor((r-n)/(10*i)));const A={};for(let t=n;t<r;t+=o)A[t]=t.toLocaleString();return A[r]=r.toLocaleString(),a.createElement(s,Object.assign({},{value:e,min:n,max:r,step:i,marks:A},{onChange:e=>{t.set("Start",e[0]),t.set("End",e[1])}}))}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(714),i=n(0),o=n(28),a=n(48);e.SelectPart=class extends a.ReactPart{static GetMetadata(){const t=super.GetMetadata();return t.description="Display a list of options inside a dropdown control",t.addOption("Value",o.Types.String,""),t.addOption("Options",o.Types.Array,[]),t}constructor(t){super(t),this.container.node.style.padding="10px"}initializeReact(){return i.createElement(r.default,{defaultValue:"",style:{width:"100%"}})}renderReact(t){const e=""+t.get("Value"),n=t.get("Options").map(t=>{const e=""+t;if(""!==e)return i.createElement(r.Option,{key:e,value:e},e)});return i.createElement(r.default,{value:e,onChange:e=>t.set("Value",""+e),style:{width:"100%"}},n)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(7),i=n(713),o=n(648),a=n(0),s=n(28),A=n(48);e.DatePickerPart=class extends A.ReactPart{static GetMetadata(){const t=super.GetMetadata();return t.description="Input part with a date editor control",t.addOption("Value",s.Types.Date,new Date,{description:"Defaults to the current day."}),t}renderReact(t){const e=t.get("Value"),n=r(e);return a.createElement(o,{value:n,calendar:a.createElement(i.default,{showOk:!0,value:n,onChange:e=>{if(null==e)return void t.set("Value",null);const n=new Date(e.format());t.set("Value",n)}})},({value:t})=>a.createElement("input",{readOnly:!0,value:t&&t.format("MM/DD/YYYY"),placeholder:"mm/dd/yyyy"}))}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(28),o=n(48);e.TextInputPart=class extends o.ReactPart{constructor(t){super(t),this.lastValue="",this.key=0,this.container.node.style.padding="10px"}static GetMetadata(){const t=super.GetMetadata();return t.description="Edit a string in an HTML `input` element.",t.addOption("Value",i.Types.String,""),t}renderReact(t){const e=""+t.get("Value");return e!==this.lastValue&&(this.lastValue=e,this.key++,this.key===Number.MAX_SAFE_INTEGER&&(this.key=0)),r.createElement(a,{key:this.key,value:""+t.get("Value"),valueChanged:e=>{this.lastValue=e,t.set("Value",e)}})}};const a=({value:t,valueChanged:e})=>{const[n,i]=r.useState(t);return r.createElement("input",{type:"text",autoComplete:"false",autoCapitalize:"false",autoCorrect:"false",style:{width:"100%"},value:n,onChange:t=>{i(t.target.value),e.call(void 0,t.target.value)}})}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(349),i=n(0),o=n(28),a=n(48),s=r.createSliderWithTooltip(r.default);e.AnimationSliderPart=class extends a.ReactPart{constructor(){super(...arguments),this.allowReplay=!1,this.isPlaying=!1,this.lastInterval=100,this.timer=null}static GetMetadata(){const t=super.GetMetadata();return t.description="Part for animating dashboards with a given timestep.",t.remarks="\nOften it is useful to 'animate' a dashboard, either to visualize change over\ntime or just to offer another level of interactivity. The Value option is most\ncommonly bound to a global representing time.\n\nIf you need to use this as a date slider, one way you can accomplish that is\nusing JS bindings. Bind the value of this part to a regular global, and wherever\nyou need time just use that number to 'add' days, hours, seconds, etc. to some\nstart date. For example, if you bound \"Value\" to a global named \"DaysSinceStart\",\nyou can use that in another binding like so:\n\n```js\n/* @DaysSinceStart,@MyStartDate */\n\nconst date = new Date(globals.MyStartDate);\ndate.setDays(date.getDays() + globals.DaysSinceStart);\n\n// now you can use date in your binding\n```\n",t.addOption("Value",o.Types.Number,0,{description:"The value this slider takes on"}),t.addOption("Min",o.Types.Number,0),t.addOption("Max",o.Types.Number,10),t.addOption("Step",o.Types.Number,1,{description:"How much to increment Value by on each timestep."}),t.addOption("Timestep (ms)",o.Types.Number,100,{description:"How frequently to update the value. Note that timesteps under 16ms may not work as expected, and will not be accurate."}),t.addOption("Loop",o.Types.Boolean,!1,{description:"Whether this part should continue by resetting the Value after hitting the Max value."}),t.addOption("Enabled",o.Types.Boolean,!1,{description:"Whether this part is currently animating."}),t}renderReact(t){const e=t.get("Value"),n=t.get("Min"),r=t.get("Max"),o=t.get("Step"),a=t.get("Timestep (ms)"),A=t.get("Loop"),l=t.get("Enabled");l&&!this.isPlaying?this.startInterval(t):this.isPlaying&&!l&&this.pauseInterval(),a!==this.lastInterval&&this.isPlaying&&(this.pauseInterval(),this.startInterval(t)),this.allowReplay=A,this.lastInterval=a;let c=o;(r-n)/o>10&&(c*=Math.floor((r-n)/(10*o)));const u={};for(let t=n;t<r;t+=c)u[t]=t.toLocaleString();return u[r]=r.toLocaleString(),i.createElement("div",{style:{padding:"10px 20px"}},i.createElement(s,Object.assign({style:{marginBottom:"25px"}},{value:e,min:n,max:r,step:o,marks:u},{onChange:e=>{t.set("Value",e)}})),i.createElement("span",null,i.createElement("button",{className:"fa "+(this.isPlaying?"fa-pause":"fa-play"),onClick:()=>{t.set("Enabled",!this.isPlaying)},style:{backgroundColor:this.isPlaying?"black":"",color:this.isPlaying?"white":""}}),i.createElement("button",{className:"fa fa-stop",onClick:()=>this.invalidateInterval(t)}),i.createElement("button",{className:"fa fa-refresh",onClick:()=>{this.allowReplay=!this.allowReplay,t.set("Loop",this.allowReplay)},style:{backgroundColor:this.allowReplay?"black":"",color:this.allowReplay?"white":""}})))}startInterval(t){null!=this.timer&&this.invalidateInterval(t),this.isPlaying=!0,t.get("Enabled")||t.set("Enabled",!0);const e=t.get("Timestep (ms)");this.stepInterval(t),this.timer=window.setInterval(()=>this.stepInterval(t),e)}stepInterval(t){const e=t.get("Min"),n=t.get("Max"),r=t.get("Step");let i=t.get("Value");if((i+=r)>n){if(!this.allowReplay)return void this.invalidateInterval(t);i=e}t.set("Value",i)}pauseInterval(){this.isPlaying=!1,null!=this.timer&&(clearInterval(this.timer),this.timer=null)}invalidateInterval(t){this.isPlaying=!1,t.set("Value",t.get("Min")),t.get("Enabled")&&t.set("Enabled",!1),null!=this.timer&&(clearInterval(this.timer),this.timer=null)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(0),o=n(48);e.TextAreaPart=class extends o.ReactPart{constructor(t){super(t),this.lastValue="",this.key=0,this.container.node.style.padding="10px"}static GetMetadata(){const t=super.GetMetadata();return t.description="Edit multiline text in an HTML5 `textarea` control",t.addOption("Value",r.Types.String,""),t}renderReact(t){let e=""+t.get("Value");return e!==this.lastValue&&(this.lastValue=e,this.key++,this.key===Number.MAX_SAFE_INTEGER&&(this.key=0)),i.createElement(a,{key:this.key,value:e,valueChanged:e=>{this.lastValue=e,t.set("Value",e)}})}};const a=({value:t,valueChanged:e})=>{const[n,r]=i.useState(t);return i.createElement("textarea",{autoComplete:"false",autoCapitalize:"false",autoCorrect:"false",value:n,onChange:t=>{r(t.target.value),e.call(void 0,t.target.value)},style:{width:"100%",height:"100%"}})}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(28),o=n(48);e.RadioButtonPart=class extends o.ReactPart{static GetMetadata(){const t=super.GetMetadata();return t.description="Display a list of strings as a radiobutton control",t.addOption("Value",i.Types.String,""),t.addOption("Options",i.Types.Array,[]),t}renderReact(t){const e=""+t.get("Value"),n=t.get("Options").map(t=>""+t),i=this.uuid,o=n.map(n=>r.createElement("label",{key:n},r.createElement("input",{type:"radio",name:i,checked:e===n,onChange:e=>{e.target.checked&&t.set("Value",n)}}),n));return r.createElement("span",null,o)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(0),i=n(28),o=n(48);e.CheckboxPart=class extends o.ReactPart{static GetMetadata(){const t=super.GetMetadata();return t.description="A part with a simple checkbox and label.",t.addOption("Checked",i.Types.Boolean,!1),t.addOption("Label",i.Types.String,"",{description:"The label appears next to the checkbox, and clicking on it will also toggle the checkbox."}),t}renderReact(t){const e=!!t.get("Checked"),n=""+t.get("Label");return r.createElement("label",null,r.createElement("input",{type:"checkbox",checked:e,onChange:e=>t.set("Checked",e.target.checked)}),n)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(48),i=n(28),o=n(83),a=n(72);n(1317);e.CodeEditorPart=class extends r.Part{constructor(t){super(t),this.editorModel=null,this.editor=null,this.editorFactory=new a.CodeMirrorEditorFactory}static GetMetadata(){const t=super.GetMetadata();return t.description="A multiline text editor part with syntax highlighting",t.remarks='\nInternally this part uses the CodeMirror editor. A list of highlighting modes is\navailable [via the CodeMirror docs](https://codemirror.net/mode/).\nAdditionally, MavenWorks provides highlighters for MQL (`mql` or `text/x-mql`)\nand JS bindings (`jsmql` or `text/x-jsmql`).\n\nThe "Language type" parameter accepts either MIME types or short identifiers,\nwhich are passed to CodeMirror as the `mode` parameter.\n',t.addOption("Code",i.Types.String,""),t.addOption("Language type",i.Types.String,"",{description:"The MIME type or short name of the syntax to highlight with"}),t}initialize(){}render(t){const e=t.get("Language type")||"text/plain",n=t.get("Code")||"";this.isEditorStale(e)?this.buildEditor(e,n,t):(this.editorModel.mimeType=e,this.editorModel.value.text!==n&&(this.editorModel.value.text=n))}isEditorStale(t){return null==this.editorModel||null==this.editor}buildEditor(t,e,n){this.editorModel&&this.editorModel.dispose(),this.editor&&this.editor.dispose(),this.editorModel=new o.CodeEditor.Model({mimeType:t,value:e}),this.editorModel.value.changed.connect(t=>{n.set("Code",t.text)}),this.editor=new o.CodeEditorWrapper({factory:this.editorFactory.newDocumentEditor,model:this.editorModel}),this.layout.insertWidget(0,this.editor)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(15);n(390),n(391),n(1318),n(1319);const i=n(1320);window&&(window.JSHINT=i.JSHINT),n(1321),n(1322).InitAddon(r);const o=n(679),a=n(9),s=n(234),A=n(338),l=n(28);r.defineMode("mql",function(t,e){let n=e.client||{},i=e.atoms||{false:!0,true:!0,null:!0},o=e.builtin||{},a=e.keywords||{},s=e.operatorChars||/^[*+\-%<>!=&|~^]/,A=e.support||{},l=e.hooks||{},c=e.dateSQL||{date:!0,time:!0,timestamp:!0};function u(t,e){let r=t.next();if(l[r]){let n=l[r](t,e);if(!1!==n)return n}if(!0===A.hexNumber&&("0"===r&&t.match(/^[xX][0-9a-fA-F]+/)||("x"===r||"X"===r)&&t.match(/^'[0-9a-fA-F]+'/)))return"number";if(!0===A.binaryNumber&&(("b"===r||"B"===r)&&t.match(/^'[01]+'/)||"0"===r&&t.match(/^b[01]+/)))return"number";if(r.charCodeAt(0)>47&&r.charCodeAt(0)<58)return t.match(/^[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?/),!0===A.decimallessFloat&&t.eat("."),"number";if("?"===r&&(t.eatSpace()||t.eol()||t.eat(";")))return"variable-3";if("'"===r)return e.tokenize=(h=r,function(t,e){let n,r=!1;for(;null!=(n=t.next());){if(n===h&&!r){e.tokenize=u;break}r=!r&&"\\"===n}return"string"}),e.tokenize(t,e);if((!0===A.nCharCast&&("n"===r||"N"===r)||!0===A.charsetCast&&"_"===r&&t.match(/[a-z][a-z0-9]*/i))&&("'"===t.peek()||'"'===t.peek()))return"keyword";if(/^[\(\),\;\[\]]/.test(r))return null;if(A.commentSlashSlash&&"/"===r&&t.eat("/"))return t.skipToEnd(),"comment";if(A.commentHash&&"#"===r||"-"===r&&t.eat("-")&&(!A.commentSpaceRequired||t.eat(" ")))return t.skipToEnd(),"comment";if("/"===r&&t.eat("*"))return e.tokenize=g,e.tokenize(t,e);if("."!==r){if(s.test(r))return t.eatWhile(s),null;if("{"===r&&(t.match(/^( )*(d|D|t|T|ts|TS)( )*'[^']*'( )*}/)||t.match(/^( )*(d|D|t|T|ts|TS)( )*"[^"]*"( )*}/)))return"number";{t.eatWhile(/^[_\w\d]/);let e=t.current().toLowerCase();return c.hasOwnProperty(e)&&(t.match(/^( )+'[^']*'/)||t.match(/^( )+"[^"]*"/))?"number":i.hasOwnProperty(e)?"atom":o.hasOwnProperty(e)?"builtin":a.hasOwnProperty(e)?"keyword":n.hasOwnProperty(e)?"string-2":null}}return!0===A.zerolessFloat&&t.match(/^(?:\d+(?:e[+-]?\d+)?)/i)?"number":!0===A.ODBCdotTable&&t.match(/^[a-zA-Z_]+/)?null:void 0;var h}function g(t,e){for(;;){if(!t.skipTo("*")){t.skipToEnd();break}if(t.next(),t.eat("/")){e.tokenize=u;break}}return"comment"}function h(t,e,n){e.context={prev:e.context,indent:t.indentation(),col:t.column(),type:n}}return{startState:function(){return{tokenize:u,context:null}},token:function(t,e){if(t.sol()&&e.context&&null==e.context.align&&(e.context.align=!1),t.eatSpace())return null;let n=e.tokenize(t,e);if("comment"===n)return n;e.context&&null==e.context.align&&(e.context.align=!0);let r=t.current();return"("===r?h(t,e,")"):"["===r?h(t,e,"]"):e.context&&e.context.type===r&&function(t){t.indent=t.context.indent,t.context=t.context.prev}(e),n},indent:function(e,n){let i=e.context;if(!i)return r.Pass;let o=n.charAt(0)===i.type;return i.align?i.col+(o?0:1):i.indent+(o?0:t.indentUnit)},blockCommentStart:"/*",blockCommentEnd:"*/",lineComment:A.commentSlashSlash?"//":A.commentHash?"#":void 0}}),r.defineMode("jsmql",(t,e)=>{const n=r.getMode(t,{name:"javascript"});let i=r.getMode(t,{name:"text/x-mql"});return r.multiplexingMode(n,{open:"mql`",close:"`",mode:i})});function c(t){let e={},n=t.split(" ");for(let t=0;t<n.length;++t)e[n[t]]=!0;return e}r.defineMIME("text/x-mql",{name:"mql",keywords:c("select from as where group order by def set fun on in asc desc format distinct top with rollup no leaves having"),builtin:c("average wavg single one count max min sum arr_add arr_div arr_log arr_mult arr_sub filter map foldl foldr pow sqrt ifelse"),atoms:c("and not or xor null"),operatorChars:/^[*+\-%<>!=]/,dateSQL:c("date time timestamp"),support:c("ODBCdotTable doubleQuote binaryNumber hexNumber"),hooks:{"@":function(t){return t.eat("@")&&(t.match(/^session\./),t.match(/^local\./),t.match(/^global\./)),t.eat("'")?(t.match(/^.*'/),"variable-2"):t.eat('"')?(t.match(/^.*"/),"variable-2"):t.eat("`")?(t.match(/^.*`/),"variable-2"):t.match(/^[0-9a-zA-Z$\.\_]+/)?"variable-2":null}}}),r.defineMIME("text/x-jsmql","jsmql"),async function(){let t;await new Promise(t=>setTimeout(t)),null!=(t=r.modeInfo)?(t.push({name:"MQL",mime:"text/x-mql",mode:"mql"}),t.push({name:"MQL.js",mime:"text/x-jsmql",mode:"jsmql"})):console.warn("Could not register MQL highlighter with modeInfo- this highlighter won't work")}();const u=new o.WorkerWrapper,g={};u.onMessage.subscribe(t=>{const{id:e}=t;if("parseQueryResult"===t.type&&e){const n=t.data,r=g[e];if(!r)return void console.error("Received bad parse result",r,e,n);try{r.resolve(JSON.parse(n))}catch(t){r.reject(t)}finally{delete g[e]}}else if("getAvailableFunctionsResult"===t.type&&e){const n=t.data,r=g[e];if(!r)return void console.error("Received bad parse result",r,e,n);try{const t=JSON.parse(n).map(t=>({...t,returnType:l.Types.findType(t.returnTypeName)||l.Types.Any,args:t.args.map(t=>({...t,type:l.Types.findType(t.typeName)||l.Types.Any}))}));r.resolve(t)}catch(t){r.reject(t)}finally{delete g[e]}}});const h=(t,e,n,i)=>{(function(t){const e=a.UUID.uuid4();let n,r,i=new Promise((t,e)=>{n=t,r=e});return g[e]={id:e,reject:r,resolve:n},u.postMessage({type:"parseQuery",id:e,codeText:t}),i})(t).then(t=>{i.state.MqlSetsDefs=t;const n=t.errors;if(n.length>0){const t=n[0];let i=t.token&&t.token.text?t.token.text.length:0;e([{message:t.msg,severity:"error",from:r.Pos(t.line-1,t.column),to:r.Pos(t.line-1,t.column+i)}])}else e([])}).catch(t=>{console.error("Unhandled parse error",t),e([])})};h.async=!0,r.registerHelper("lint","mql",h);const d=t=>{const e=t.args.slice(1),n=e,r=e.slice(e.length-t.repeatingArgs),i=n.map(t=>t.name+": "+t.type.name).join(", "),o=n.length>0?", ":"";return`(${i}${r.length>0?` [${o}${r.map(t=>t.name+": "+t.type.name).join(", ")}]+`:""}): ${t.returnTypeName}`},f=t=>`(${t.paramNames.map(t=>`@${t}`).join(", ")})`,p=(t,e,n)=>{const r=document.createElement("div");r.className="CodeMirror-hint-container";const i=document.createElement("div");i.className="CodeMirror-hint-name";const o=document.createElement("div");o.className="CodeMirror-hint-signature",i.appendChild(document.createTextNode(n.name)),o.appendChild(document.createTextNode(n.signature||"")),r.appendChild(i),r.appendChild(o),t.appendChild(r)},I=async(t,e,n)=>{const i=await function(){const t=a.UUID.uuid4();let e,n,r=new Promise((t,r)=>{e=t,n=r});return g[t]={id:t,reject:n,resolve:e},u.postMessage({type:"getAvailableFunctions",id:t}),r}(),o=t.options,s=o&&o.context?o.context.globals:null,l=s?Array.from(s).map(t=>t.name):[],c=(t=>{const e=A.getCommentLine(t);return null!=e?A.getGlobalsFromComment(e):null})(t.getDoc().getValue())||l,h=t.getDoc().getCursor();let I=t.getTokenAt(h);I.end>h.ch&&(I.end=h.ch,I.string=I.string.slice(0,h.ch-I.start));let C=t.state.MqlSetsDefs||{sets:[],defs:[]},m=i.map(t=>({name:t.name,signature:d(t)})),y=C.sets.map(t=>({name:`@${t}`})),E=C.defs.map(t=>({name:`@${t.name}`,signature:f(t)})),v=c.map(t=>({name:`@${t}`})),B=m.concat(y,E,v),b=function(t){return t=t.replace(/^[\.\s]*/g,"")}(I.string);b.length>0&&(B=B.filter(t=>t.name.toLowerCase().startsWith(b.toLowerCase()))),B.sort((t,e)=>t.name.localeCompare(e.name));let Q=b.length>0;e({list:B.map(t=>({name:t.name,signature:t.signature,render:p,text:t.name})),from:Q?r.Pos(h.line,h.ch-b.length):h,to:h})};function C(t){const e=t.options;if(e){let n=e.hintOptions;if(n&&n.globalScope&&n.additionalContext){const r=e&&e.context?e.context.globals:null,i=r?Array.from(r).map(t=>t.name):[];let o=(function(t){const e=A.getCommentLine(t,null);return null!=e?A.getGlobalsFromComment(e):null}(t.getDoc().getValue())||i).reduce((t,e)=>(t[e]="",t),{});n.globalScope.globals=o,n.additionalContext.globals=o}}}I.async=!0,r.registerHelper("hint","mql",I),r.defineInitHook(t=>{r.on(t,"optionChange",(t,e)=>{if("mode"===e)if("text/x-jsmql"===t.getOption("mode")&&t.options&&t.options.hintOptions){t.options.hintOptions.esversion=9,t.options.hintOptions.additionalContext={TableHelper:s.TableHelper};let e={};for(let t of Reflect.ownKeys(window))e[t]=window[t];t.options.hintOptions.globalScope=Object.assign({},e,{TableHelper:s.TableHelper,RunMql:function(){},mql:"",expr:function(){},ImportScriptFromUrl:function(){}}),C(t),t.on("change",C)}else t.off("change",C)})})},function(t,e,n){!function(t){"use strict";var e="CodeMirror-lint-markers";function n(t){t.parentNode&&t.parentNode.removeChild(t)}function r(e,r,i){var o=function(e,n){var r=document.createElement("div");function i(e){if(!r.parentNode)return t.off(document,"mousemove",i);r.style.top=Math.max(0,e.clientY-r.offsetHeight-5)+"px",r.style.left=e.clientX+5+"px"}return r.className="CodeMirror-lint-tooltip",r.appendChild(n.cloneNode(!0)),document.body.appendChild(r),t.on(document,"mousemove",i),i(e),null!=r.style.opacity&&(r.style.opacity=1),r}(e,r);function a(){var e;t.off(i,"mouseout",a),o&&((e=o).parentNode&&(null==e.style.opacity&&n(e),e.style.opacity=0,setTimeout(function(){n(e)},600)),o=null)}var s=setInterval(function(){if(o)for(var t=i;;t=t.parentNode){if(t&&11==t.nodeType&&(t=t.host),t==document.body)return;if(!t){a();break}}if(!o)return clearInterval(s)},400);t.on(i,"mouseout",a)}function i(t,e,n){this.marked=[],this.options=e,this.timeout=null,this.hasGutter=n,this.onMouseOver=function(e){!function(t,e){var n=e.target||e.srcElement;if(/\bCodeMirror-lint-mark-/.test(n.className)){for(var i=n.getBoundingClientRect(),o=(i.left+i.right)/2,a=(i.top+i.bottom)/2,A=t.findMarksAt(t.coordsChar({left:o,top:a},"client")),l=[],c=0;c<A.length;++c){var u=A[c].__annotation;u&&l.push(u)}l.length&&function(t,e){for(var n=e.target||e.srcElement,i=document.createDocumentFragment(),o=0;o<t.length;o++){var a=t[o];i.appendChild(s(a))}r(e,i,n)}(l,e)}}(t,e)},this.waitingFor=0}function o(t){var n=t.state.lint;n.hasGutter&&t.clearGutter(e);for(var r=0;r<n.marked.length;++r)n.marked[r].clear();n.marked.length=0}function a(e,n,i,o){var a=document.createElement("div"),s=a;return a.className="CodeMirror-lint-marker-"+n,i&&((s=a.appendChild(document.createElement("div"))).className="CodeMirror-lint-marker-multiple"),0!=o&&t.on(s,"mouseover",function(t){r(t,e,s)}),a}function s(t){var e=t.severity;e||(e="error");var n=document.createElement("div");return n.className="CodeMirror-lint-message-"+e,void 0!==t.messageHTML?n.innerHTML=t.messageHTML:n.appendChild(document.createTextNode(t.message)),n}function A(e){var n=e.state.lint,r=n.options,i=r.options||r,o=r.getAnnotations||e.getHelper(t.Pos(0,0),"lint");if(o)if(r.async||o.async)!function(e,n,r){var i=e.state.lint,o=++i.waitingFor;function a(){o=-1,e.off("change",a)}e.on("change",a),n(e.getValue(),function(n,r){e.off("change",a),i.waitingFor==o&&(r&&n instanceof t&&(n=r),e.operation(function(){l(e,n)}))},r,e)}(e,o,i);else{var a=o(e.getValue(),i,e);if(!a)return;a.then?a.then(function(t){e.operation(function(){l(e,t)})}):e.operation(function(){l(e,a)})}}function l(t,n){o(t);for(var r=t.state.lint,i=r.options,A=function(t){for(var e=[],n=0;n<t.length;++n){var r=t[n],i=r.from.line;(e[i]||(e[i]=[])).push(r)}return e}(n),l=0;l<A.length;++l){var c=A[l];if(c){for(var u=null,g=r.hasGutter&&document.createDocumentFragment(),h=0;h<c.length;++h){var d=c[h],f=d.severity;f||(f="error"),I=f,u="error"==(p=u)?p:I,i.formatAnnotation&&(d=i.formatAnnotation(d)),r.hasGutter&&g.appendChild(s(d)),d.to&&r.marked.push(t.markText(d.from,d.to,{className:"CodeMirror-lint-mark-"+f,__annotation:d}))}r.hasGutter&&t.setGutterMarker(l,e,a(g,u,c.length>1,r.options.tooltips))}}var p,I;i.onUpdateLinting&&i.onUpdateLinting(n,A,t)}function c(t){var e=t.state.lint;e&&(clearTimeout(e.timeout),e.timeout=setTimeout(function(){A(t)},e.options.delay||500))}t.defineOption("lint",!1,function(n,r,a){if(a&&a!=t.Init&&(o(n),!1!==n.state.lint.options.lintOnChange&&n.off("change",c),t.off(n.getWrapperElement(),"mouseover",n.state.lint.onMouseOver),clearTimeout(n.state.lint.timeout),delete n.state.lint),r){for(var s=n.getOption("gutters"),l=!1,u=0;u<s.length;++u)s[u]==e&&(l=!0);var g=n.state.lint=new i(n,(h=r)instanceof Function?{getAnnotations:h}:(h&&!0!==h||(h={}),h),l);!1!==g.options.lintOnChange&&n.on("change",c),0!=g.options.tooltips&&"gutter"!=g.options.tooltips&&t.on(n.getWrapperElement(),"mouseover",g.onMouseOver),A(n)}var h}),t.defineExtension("performLint",function(){this.state.lint&&A(this)})}(n(15))},function(t,e,n){!function(t){"use strict";var e="CodeMirror-hint",n="CodeMirror-hint-active";function r(t,e){this.cm=t,this.options=e,this.widget=null,this.debounce=0,this.tick=0,this.startPos=this.cm.getCursor("start"),this.startLen=this.cm.getLine(this.startPos.line).length-this.cm.getSelection().length;var n=this;t.on("cursorActivity",this.activityFunc=function(){n.cursorActivity()})}t.showHint=function(t,e,n){if(!e)return t.showHint(n);n&&n.async&&(e.async=!0);var r={hint:e};if(n)for(var i in n)r[i]=n[i];return t.showHint(r)},t.defineExtension("showHint",function(e){e=function(t,e,n){var r=t.options.hintOptions,i={};for(var o in c)i[o]=c[o];if(r)for(var o in r)void 0!==r[o]&&(i[o]=r[o]);if(n)for(var o in n)void 0!==n[o]&&(i[o]=n[o]);return i.hint.resolve&&(i.hint=i.hint.resolve(t,e)),i}(this,this.getCursor("start"),e);var n=this.listSelections();if(!(n.length>1)){if(this.somethingSelected()){if(!e.hint.supportsSelection)return;for(var i=0;i<n.length;i++)if(n[i].head.line!=n[i].anchor.line)return}this.state.completionActive&&this.state.completionActive.close();var o=this.state.completionActive=new r(this,e);o.options.hint&&(t.signal(this,"startCompletion",this),o.update(!0))}}),t.defineExtension("closeHint",function(){this.state.completionActive&&this.state.completionActive.close()});var i=window.requestAnimationFrame||function(t){return setTimeout(t,1e3/60)},o=window.cancelAnimationFrame||clearTimeout;function a(t){return"string"==typeof t?t:t.text}function s(t,e){for(;e&&e!=t;){if("LI"===e.nodeName.toUpperCase()&&e.parentNode==t)return e;e=e.parentNode}}function A(r,i){this.completion=r,this.data=i,this.picked=!1;var o=this,A=r.cm,l=A.getInputField().ownerDocument,c=l.defaultView||l.parentWindow,u=this.hints=l.createElement("ul"),g=r.cm.options.theme;u.className="CodeMirror-hints "+g,this.selectedHint=i.selectedHint||0;for(var h=i.list,d=0;d<h.length;++d){var f=u.appendChild(l.createElement("li")),p=h[d],I=e+(d!=this.selectedHint?"":" "+n);null!=p.className&&(I=p.className+" "+I),f.className=I,p.render?p.render(f,i,p):f.appendChild(l.createTextNode(p.displayText||a(p))),f.hintId=d}var C=r.options.container||l.body,m=A.cursorCoords(r.options.alignWithWord?i.from:null),y=m.left,E=m.bottom,v=!0,B=0,b=0;if(C!==l.body){var Q=-1!==["absolute","relative","fixed"].indexOf(c.getComputedStyle(C).position),w=Q?C:C.offsetParent,x=w.getBoundingClientRect(),_=l.body.getBoundingClientRect();B=x.left-_.left,b=x.top-_.top}u.style.left=y-B+"px",u.style.top=E-b+"px";var D=c.innerWidth||Math.max(l.body.offsetWidth,l.documentElement.offsetWidth),M=c.innerHeight||Math.max(l.body.offsetHeight,l.documentElement.offsetHeight);C.appendChild(u);var k=u.getBoundingClientRect(),S=k.bottom-M,L=u.scrollHeight>u.clientHeight+1,R=A.getScrollInfo();if(S>0){var N=k.bottom-k.top,F=m.top-(m.bottom-k.top);if(F-N>0)u.style.top=(E=m.top-N-b)+"px",v=!1;else if(N>M){u.style.height=M-5+"px",u.style.top=(E=m.bottom-k.top-b)+"px";var T=A.getCursor();i.from.ch!=T.ch&&(m=A.cursorCoords(T),u.style.left=(y=m.left-B)+"px",k=u.getBoundingClientRect())}}var U,O=k.right-D;if(O>0&&(k.right-k.left>D&&(u.style.width=D-5+"px",O-=k.right-k.left-D),u.style.left=(y=m.left-O-B)+"px"),L)for(var G=u.firstChild;G;G=G.nextSibling)G.style.paddingRight=A.display.nativeBarWidth+"px";return A.addKeyMap(this.keyMap=function(t,e){var n={Up:function(){e.moveFocus(-1)},Down:function(){e.moveFocus(1)},PageUp:function(){e.moveFocus(1-e.menuSize(),!0)},PageDown:function(){e.moveFocus(e.menuSize()-1,!0)},Home:function(){e.setFocus(0)},End:function(){e.setFocus(e.length-1)},Enter:e.pick,Tab:e.pick,Esc:e.close};/Mac/.test(navigator.platform)&&(n["Ctrl-P"]=function(){e.moveFocus(-1)},n["Ctrl-N"]=function(){e.moveFocus(1)});var r=t.options.customKeys,i=r?{}:n;function o(t,r){var o;o="string"!=typeof r?function(t){return r(t,e)}:n.hasOwnProperty(r)?n[r]:r,i[t]=o}if(r)for(var a in r)r.hasOwnProperty(a)&&o(a,r[a]);var s=t.options.extraKeys;if(s)for(var a in s)s.hasOwnProperty(a)&&o(a,s[a]);return i}(r,{moveFocus:function(t,e){o.changeActive(o.selectedHint+t,e)},setFocus:function(t){o.changeActive(t)},menuSize:function(){return o.screenAmount()},length:h.length,close:function(){r.close()},pick:function(){o.pick()},data:i})),r.options.closeOnUnfocus&&(A.on("blur",this.onBlur=function(){U=setTimeout(function(){r.close()},100)}),A.on("focus",this.onFocus=function(){clearTimeout(U)})),A.on("scroll",this.onScroll=function(){var t=A.getScrollInfo(),e=A.getWrapperElement().getBoundingClientRect(),n=E+R.top-t.top,i=n-(c.pageYOffset||(l.documentElement||l.body).scrollTop);if(v||(i+=u.offsetHeight),i<=e.top||i>=e.bottom)return r.close();u.style.top=n+"px",u.style.left=y+R.left-t.left+"px"}),t.on(u,"dblclick",function(t){var e=s(u,t.target||t.srcElement);e&&null!=e.hintId&&(o.changeActive(e.hintId),o.pick())}),t.on(u,"click",function(t){var e=s(u,t.target||t.srcElement);e&&null!=e.hintId&&(o.changeActive(e.hintId),r.options.completeOnSingleClick&&o.pick())}),t.on(u,"mousedown",function(){setTimeout(function(){A.focus()},20)}),t.signal(i,"select",h[this.selectedHint],u.childNodes[this.selectedHint]),!0}function l(t,e,n,r){if(t.async)t(e,r,n);else{var i=t(e,n);i&&i.then?i.then(r):r(i)}}r.prototype={close:function(){this.active()&&(this.cm.state.completionActive=null,this.tick=null,this.cm.off("cursorActivity",this.activityFunc),this.widget&&this.data&&t.signal(this.data,"close"),this.widget&&this.widget.close(),t.signal(this.cm,"endCompletion",this.cm))},active:function(){return this.cm.state.completionActive==this},pick:function(e,n){var r=e.list[n];r.hint?r.hint(this.cm,e,r):this.cm.replaceRange(a(r),r.from||e.from,r.to||e.to,"complete"),t.signal(e,"pick",r),this.close()},cursorActivity:function(){this.debounce&&(o(this.debounce),this.debounce=0);var t=this.cm.getCursor(),e=this.cm.getLine(t.line);if(t.line!=this.startPos.line||e.length-t.ch!=this.startLen-this.startPos.ch||t.ch<this.startPos.ch||this.cm.somethingSelected()||!t.ch||this.options.closeCharacters.test(e.charAt(t.ch-1)))this.close();else{var n=this;this.debounce=i(function(){n.update()}),this.widget&&this.widget.disable()}},update:function(t){if(null!=this.tick){var e=this,n=++this.tick;l(this.options.hint,this.cm,this.options,function(r){e.tick==n&&e.finishUpdate(r,t)})}},finishUpdate:function(e,n){this.data&&t.signal(this.data,"update");var r=this.widget&&this.widget.picked||n&&this.options.completeSingle;this.widget&&this.widget.close(),this.data=e,e&&e.list.length&&(r&&1==e.list.length?this.pick(e,0):(this.widget=new A(this,e),t.signal(e,"shown")))}},A.prototype={close:function(){if(this.completion.widget==this){this.completion.widget=null,this.hints.parentNode.removeChild(this.hints),this.completion.cm.removeKeyMap(this.keyMap);var t=this.completion.cm;this.completion.options.closeOnUnfocus&&(t.off("blur",this.onBlur),t.off("focus",this.onFocus)),t.off("scroll",this.onScroll)}},disable:function(){this.completion.cm.removeKeyMap(this.keyMap);var t=this;this.keyMap={Enter:function(){t.picked=!0}},this.completion.cm.addKeyMap(this.keyMap)},pick:function(){this.completion.pick(this.data,this.selectedHint)},changeActive:function(e,r){if(e>=this.data.list.length?e=r?this.data.list.length-1:0:e<0&&(e=r?0:this.data.list.length-1),this.selectedHint!=e){var i=this.hints.childNodes[this.selectedHint];i&&(i.className=i.className.replace(" "+n,"")),(i=this.hints.childNodes[this.selectedHint=e]).className+=" "+n,i.offsetTop<this.hints.scrollTop?this.hints.scrollTop=i.offsetTop-3:i.offsetTop+i.offsetHeight>this.hints.scrollTop+this.hints.clientHeight&&(this.hints.scrollTop=i.offsetTop+i.offsetHeight-this.hints.clientHeight+3),t.signal(this.data,"select",this.data.list[this.selectedHint],i)}},screenAmount:function(){return Math.floor(this.hints.clientHeight/this.hints.firstChild.offsetHeight)||1}},t.registerHelper("hint","auto",{resolve:function(e,n){var r,i=e.getHelpers(n,"hint");if(i.length){var o=function(t,e,n){var r=function(t,e){if(!t.somethingSelected())return e;for(var n=[],r=0;r<e.length;r++)e[r].supportsSelection&&n.push(e[r]);return n}(t,i);!function i(o){if(o==r.length)return e(null);l(r[o],t,n,function(t){t&&t.list.length>0?e(t):i(o+1)})}(0)};return o.async=!0,o.supportsSelection=!0,o}return(r=e.getHelper(e.getCursor(),"hintWords"))?function(e){return t.hint.fromList(e,{words:r})}:t.hint.anyword?function(e,n){return t.hint.anyword(e,n)}:function(){}}}),t.registerHelper("hint","fromList",function(e,n){var r,i=e.getCursor(),o=e.getTokenAt(i),a=t.Pos(i.line,o.start),s=i;o.start<i.ch&&/\w/.test(o.string.charAt(i.ch-o.start-1))?r=o.string.substr(0,i.ch-o.start):(r="",a=i);for(var A=[],l=0;l<n.words.length;l++){var c=n.words[l];c.slice(0,r.length)==r&&A.push(c)}if(A.length)return{list:A,from:a,to:s}}),t.commands.autocomplete=t.showHint;var c={hint:t.hint.auto,completeSingle:!0,alignWithWord:!0,closeCharacters:/[\s()\[\]{};:>,]/,closeOnUnfocus:!0,completeOnSingleClick:!0,container:null,customKeys:null,extraKeys:null};t.defineOption("hintOptions",null)}(n(15))},function(t,e,n){(function(t){var r,i;"undefined"==typeof window&&(window={}),function(){var o;o=function t(e,n,r){function i(s,A){if(!n[s]){if(!e[s]){var l="function"==typeof o&&o;if(!A&&l)return l(s,!0);if(a)return a(s,!0);var c=new Error("Cannot find module '"+s+"'");throw c.code="MODULE_NOT_FOUND",c}var u=n[s]={exports:{}};e[s][0].call(u.exports,function(t){var n=e[s][1][t];return i(n||t)},u,u.exports,t,e,n,r)}return n[s].exports}for(var a="function"==typeof o&&o,s=0;s<r.length;s++)i(r[s]);return i}({1:[function(t,e,n){for(var r=[],i=0;i<128;i++)r[i]=36===i||i>=65&&i<=90||95===i||i>=97&&i<=122;var o=[];for(i=0;i<128;i++)o[i]=r[i]||i>=48&&i<=57;e.exports={asciiIdentifierStartTable:r,asciiIdentifierPartTable:o}},{}],2:[function(t,e,n){e.exports=/^(?:[\$A-Z_a-z\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0370-\u0374\u0376\u0377\u037A-\u037D\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u048A-\u0525\u0531-\u0556\u0559\u0561-\u0587\u05D0-\u05EA\u05F0-\u05F2\u0621-\u064A\u066E\u066F\u0671-\u06D3\u06D5\u06E5\u06E6\u06EE\u06EF\u06FA-\u06FC\u06FF\u0710\u0712-\u072F\u074D-\u07A5\u07B1\u07CA-\u07EA\u07F4\u07F5\u07FA\u0800-\u0815\u081A\u0824\u0828\u0904-\u0939\u093D\u0950\u0958-\u0961\u0971\u0972\u0979-\u097F\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BD\u09CE\u09DC\u09DD\u09DF-\u09E1\u09F0\u09F1\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A59-\u0A5C\u0A5E\u0A72-\u0A74\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABD\u0AD0\u0AE0\u0AE1\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3D\u0B5C\u0B5D\u0B5F-\u0B61\u0B71\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BD0\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C33\u0C35-\u0C39\u0C3D\u0C58\u0C59\u0C60\u0C61\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBD\u0CDE\u0CE0\u0CE1\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D28\u0D2A-\u0D39\u0D3D\u0D60\u0D61\u0D7A-\u0D7F\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0E01-\u0E30\u0E32\u0E33\u0E40-\u0E46\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB0\u0EB2\u0EB3\u0EBD\u0EC0-\u0EC4\u0EC6\u0EDC\u0EDD\u0F00\u0F40-\u0F47\u0F49-\u0F6C\u0F88-\u0F8B\u1000-\u102A\u103F\u1050-\u1055\u105A-\u105D\u1061\u1065\u1066\u106E-\u1070\u1075-\u1081\u108E\u10A0-\u10C5\u10D0-\u10FA\u10FC\u1100-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u1380-\u138F\u13A0-\u13F4\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F0\u1700-\u170C\u170E-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176C\u176E-\u1770\u1780-\u17B3\u17D7\u17DC\u1820-\u1877\u1880-\u18A8\u18AA\u18B0-\u18F5\u1900-\u191C\u1950-\u196D\u1970-\u1974\u1980-\u19AB\u19C1-\u19C7\u1A00-\u1A16\u1A20-\u1A54\u1AA7\u1B05-\u1B33\u1B45-\u1B4B\u1B83-\u1BA0\u1BAE\u1BAF\u1C00-\u1C23\u1C4D-\u1C4F\u1C5A-\u1C7D\u1CE9-\u1CEC\u1CEE-\u1CF1\u1D00-\u1DBF\u1E00-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u2071\u207F\u2090-\u2094\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CEE\u2D00-\u2D25\u2D30-\u2D65\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2E2F\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303C\u3041-\u3096\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31B7\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FCB\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA61F\uA62A\uA62B\uA640-\uA65F\uA662-\uA66E\uA67F-\uA697\uA6A0-\uA6EF\uA717-\uA71F\uA722-\uA788\uA78B\uA78C\uA7FB-\uA801\uA803-\uA805\uA807-\uA80A\uA80C-\uA822\uA840-\uA873\uA882-\uA8B3\uA8F2-\uA8F7\uA8FB\uA90A-\uA925\uA930-\uA946\uA960-\uA97C\uA984-\uA9B2\uA9CF\uAA00-\uAA28\uAA40-\uAA42\uAA44-\uAA4B\uAA60-\uAA76\uAA7A\uAA80-\uAAAF\uAAB1\uAAB5\uAAB6\uAAB9-\uAABD\uAAC0\uAAC2\uAADB-\uAADD\uABC0-\uABE2\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA2D\uFA30-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D\uFB1F-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE70-\uFE74\uFE76-\uFEFC\uFF21-\uFF3A\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC])(?:[\$0-9A-Z_a-z\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0300-\u0374\u0376\u0377\u037A-\u037D\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u0483-\u0487\u048A-\u0525\u0531-\u0556\u0559\u0561-\u0587\u0591-\u05BD\u05BF\u05C1\u05C2\u05C4\u05C5\u05C7\u05D0-\u05EA\u05F0-\u05F2\u0610-\u061A\u0621-\u065E\u0660-\u0669\u066E-\u06D3\u06D5-\u06DC\u06DF-\u06E8\u06EA-\u06FC\u06FF\u0710-\u074A\u074D-\u07B1\u07C0-\u07F5\u07FA\u0800-\u082D\u0900-\u0939\u093C-\u094E\u0950-\u0955\u0958-\u0963\u0966-\u096F\u0971\u0972\u0979-\u097F\u0981-\u0983\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BC-\u09C4\u09C7\u09C8\u09CB-\u09CE\u09D7\u09DC\u09DD\u09DF-\u09E3\u09E6-\u09F1\u0A01-\u0A03\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A3C\u0A3E-\u0A42\u0A47\u0A48\u0A4B-\u0A4D\u0A51\u0A59-\u0A5C\u0A5E\u0A66-\u0A75\u0A81-\u0A83\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABC-\u0AC5\u0AC7-\u0AC9\u0ACB-\u0ACD\u0AD0\u0AE0-\u0AE3\u0AE6-\u0AEF\u0B01-\u0B03\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3C-\u0B44\u0B47\u0B48\u0B4B-\u0B4D\u0B56\u0B57\u0B5C\u0B5D\u0B5F-\u0B63\u0B66-\u0B6F\u0B71\u0B82\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BBE-\u0BC2\u0BC6-\u0BC8\u0BCA-\u0BCD\u0BD0\u0BD7\u0BE6-\u0BEF\u0C01-\u0C03\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C33\u0C35-\u0C39\u0C3D-\u0C44\u0C46-\u0C48\u0C4A-\u0C4D\u0C55\u0C56\u0C58\u0C59\u0C60-\u0C63\u0C66-\u0C6F\u0C82\u0C83\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBC-\u0CC4\u0CC6-\u0CC8\u0CCA-\u0CCD\u0CD5\u0CD6\u0CDE\u0CE0-\u0CE3\u0CE6-\u0CEF\u0D02\u0D03\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D28\u0D2A-\u0D39\u0D3D-\u0D44\u0D46-\u0D48\u0D4A-\u0D4D\u0D57\u0D60-\u0D63\u0D66-\u0D6F\u0D7A-\u0D7F\u0D82\u0D83\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0DCA\u0DCF-\u0DD4\u0DD6\u0DD8-\u0DDF\u0DF2\u0DF3\u0E01-\u0E3A\u0E40-\u0E4E\u0E50-\u0E59\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB9\u0EBB-\u0EBD\u0EC0-\u0EC4\u0EC6\u0EC8-\u0ECD\u0ED0-\u0ED9\u0EDC\u0EDD\u0F00\u0F18\u0F19\u0F20-\u0F29\u0F35\u0F37\u0F39\u0F3E-\u0F47\u0F49-\u0F6C\u0F71-\u0F84\u0F86-\u0F8B\u0F90-\u0F97\u0F99-\u0FBC\u0FC6\u1000-\u1049\u1050-\u109D\u10A0-\u10C5\u10D0-\u10FA\u10FC\u1100-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u135F\u1380-\u138F\u13A0-\u13F4\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F0\u1700-\u170C\u170E-\u1714\u1720-\u1734\u1740-\u1753\u1760-\u176C\u176E-\u1770\u1772\u1773\u1780-\u17B3\u17B6-\u17D3\u17D7\u17DC\u17DD\u17E0-\u17E9\u180B-\u180D\u1810-\u1819\u1820-\u1877\u1880-\u18AA\u18B0-\u18F5\u1900-\u191C\u1920-\u192B\u1930-\u193B\u1946-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u19D0-\u19DA\u1A00-\u1A1B\u1A20-\u1A5E\u1A60-\u1A7C\u1A7F-\u1A89\u1A90-\u1A99\u1AA7\u1B00-\u1B4B\u1B50-\u1B59\u1B6B-\u1B73\u1B80-\u1BAA\u1BAE-\u1BB9\u1C00-\u1C37\u1C40-\u1C49\u1C4D-\u1C7D\u1CD0-\u1CD2\u1CD4-\u1CF2\u1D00-\u1DE6\u1DFD-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u200C\u200D\u203F\u2040\u2054\u2071\u207F\u2090-\u2094\u20D0-\u20DC\u20E1\u20E5-\u20F0\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CF1\u2D00-\u2D25\u2D30-\u2D65\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2DE0-\u2DFF\u2E2F\u3005-\u3007\u3021-\u302F\u3031-\u3035\u3038-\u303C\u3041-\u3096\u3099\u309A\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31B7\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FCB\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA62B\uA640-\uA65F\uA662-\uA66F\uA67C\uA67D\uA67F-\uA697\uA6A0-\uA6F1\uA717-\uA71F\uA722-\uA788\uA78B\uA78C\uA7FB-\uA827\uA840-\uA873\uA880-\uA8C4\uA8D0-\uA8D9\uA8E0-\uA8F7\uA8FB\uA900-\uA92D\uA930-\uA953\uA960-\uA97C\uA980-\uA9C0\uA9CF-\uA9D9\uAA00-\uAA36\uAA40-\uAA4D\uAA50-\uAA59\uAA60-\uAA76\uAA7A\uAA7B\uAA80-\uAAC2\uAADB-\uAADD\uABC0-\uABEA\uABEC\uABED\uABF0-\uABF9\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA2D\uFA30-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE00-\uFE0F\uFE20-\uFE26\uFE33\uFE34\uFE4D-\uFE4F\uFE70-\uFE74\uFE76-\uFEFC\uFF10-\uFF19\uFF21-\uFF3A\uFF3F\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC])*$/},{}],3:[function(t,e,n){var r="183,768,769,770,771,772,773,774,775,776,777,778,779,780,781,782,783,784,785,786,787,788,789,790,791,792,793,794,795,796,797,798,799,800,801,802,803,804,805,806,807,808,809,810,811,812,813,814,815,816,817,818,819,820,821,822,823,824,825,826,827,828,829,830,831,832,833,834,835,836,837,838,839,840,841,842,843,844,845,846,847,848,849,850,851,852,853,854,855,856,857,858,859,860,861,862,863,864,865,866,867,868,869,870,871,872,873,874,875,876,877,878,879,903,1155,1156,1157,1158,1159,1425,1426,1427,1428,1429,1430,1431,1432,1433,1434,1435,1436,1437,1438,1439,1440,1441,1442,1443,1444,1445,1446,1447,1448,1449,1450,1451,1452,1453,1454,1455,1456,1457,1458,1459,1460,1461,1462,1463,1464,1465,1466,1467,1468,1469,1471,1473,1474,1476,1477,1479,1552,1553,1554,1555,1556,1557,1558,1559,1560,1561,1562,1611,1612,1613,1614,1615,1616,1617,1618,1619,1620,1621,1622,1623,1624,1625,1626,1627,1628,1629,1630,1631,1632,1633,1634,1635,1636,1637,1638,1639,1640,1641,1648,1750,1751,1752,1753,1754,1755,1756,1759,1760,1761,1762,1763,1764,1767,1768,1770,1771,1772,1773,1776,1777,1778,1779,1780,1781,1782,1783,1784,1785,1809,1840,1841,1842,1843,1844,1845,1846,1847,1848,1849,1850,1851,1852,1853,1854,1855,1856,1857,1858,1859,1860,1861,1862,1863,1864,1865,1866,1958,1959,1960,1961,1962,1963,1964,1965,1966,1967,1968,1984,1985,1986,1987,1988,1989,1990,1991,1992,1993,2027,2028,2029,2030,2031,2032,2033,2034,2035,2045,2070,2071,2072,2073,2075,2076,2077,2078,2079,2080,2081,2082,2083,2085,2086,2087,2089,2090,2091,2092,2093,2137,2138,2139,2259,2260,2261,2262,2263,2264,2265,2266,2267,2268,2269,2270,2271,2272,2273,2275,2276,2277,2278,2279,2280,2281,2282,2283,2284,2285,2286,2287,2288,2289,2290,2291,2292,2293,2294,2295,2296,2297,2298,2299,2300,2301,2302,2303,2304,2305,2306,2307,2362,2363,2364,2366,2367,2368,2369,2370,2371,2372,2373,2374,2375,2376,2377,2378,2379,2380,2381,2382,2383,2385,2386,2387,2388,2389,2390,2391,2402,2403,2406,2407,2408,2409,2410,2411,2412,2413,2414,2415,2433,2434,2435,2492,2494,2495,2496,2497,2498,2499,2500,2503,2504,2507,2508,2509,2519,2530,2531,2534,2535,2536,2537,2538,2539,2540,2541,2542,2543,2558,2561,2562,2563,2620,2622,2623,2624,2625,2626,2631,2632,2635,2636,2637,2641,2662,2663,2664,2665,2666,2667,2668,2669,2670,2671,2672,2673,2677,2689,2690,2691,2748,2750,2751,2752,2753,2754,2755,2756,2757,2759,2760,2761,2763,2764,2765,2786,2787,2790,2791,2792,2793,2794,2795,2796,2797,2798,2799,2810,2811,2812,2813,2814,2815,2817,2818,2819,2876,2878,2879,2880,2881,2882,2883,2884,2887,2888,2891,2892,2893,2902,2903,2914,2915,2918,2919,2920,2921,2922,2923,2924,2925,2926,2927,2946,3006,3007,3008,3009,3010,3014,3015,3016,3018,3019,3020,3021,3031,3046,3047,3048,3049,3050,3051,3052,3053,3054,3055,3072,3073,3074,3075,3076,3134,3135,3136,3137,3138,3139,3140,3142,3143,3144,3146,3147,3148,3149,3157,3158,3170,3171,3174,3175,3176,3177,3178,3179,3180,3181,3182,3183,3201,3202,3203,3260,3262,3263,3264,3265,3266,3267,3268,3270,3271,3272,3274,3275,3276,3277,3285,3286,3298,3299,3302,3303,3304,3305,3306,3307,3308,3309,3310,3311,3328,3329,3330,3331,3387,3388,3390,3391,3392,3393,3394,3395,3396,3398,3399,3400,3402,3403,3404,3405,3415,3426,3427,3430,3431,3432,3433,3434,3435,3436,3437,3438,3439,3458,3459,3530,3535,3536,3537,3538,3539,3540,3542,3544,3545,3546,3547,3548,3549,3550,3551,3558,3559,3560,3561,3562,3563,3564,3565,3566,3567,3570,3571,3633,3636,3637,3638,3639,3640,3641,3642,3655,3656,3657,3658,3659,3660,3661,3662,3664,3665,3666,3667,3668,3669,3670,3671,3672,3673,3761,3764,3765,3766,3767,3768,3769,3771,3772,3784,3785,3786,3787,3788,3789,3792,3793,3794,3795,3796,3797,3798,3799,3800,3801,3864,3865,3872,3873,3874,3875,3876,3877,3878,3879,3880,3881,3893,3895,3897,3902,3903,3953,3954,3955,3956,3957,3958,3959,3960,3961,3962,3963,3964,3965,3966,3967,3968,3969,3970,3971,3972,3974,3975,3981,3982,3983,3984,3985,3986,3987,3988,3989,3990,3991,3993,3994,3995,3996,3997,3998,3999,4000,4001,4002,4003,4004,4005,4006,4007,4008,4009,4010,4011,4012,4013,4014,4015,4016,4017,4018,4019,4020,4021,4022,4023,4024,4025,4026,4027,4028,4038,4139,4140,4141,4142,4143,4144,4145,4146,4147,4148,4149,4150,4151,4152,4153,4154,4155,4156,4157,4158,4160,4161,4162,4163,4164,4165,4166,4167,4168,4169,4182,4183,4184,4185,4190,4191,4192,4194,4195,4196,4199,4200,4201,4202,4203,4204,4205,4209,4210,4211,4212,4226,4227,4228,4229,4230,4231,4232,4233,4234,4235,4236,4237,4239,4240,4241,4242,4243,4244,4245,4246,4247,4248,4249,4250,4251,4252,4253,4957,4958,4959,4969,4970,4971,4972,4973,4974,4975,4976,4977,5906,5907,5908,5938,5939,5940,5970,5971,6002,6003,6068,6069,6070,6071,6072,6073,6074,6075,6076,6077,6078,6079,6080,6081,6082,6083,6084,6085,6086,6087,6088,6089,6090,6091,6092,6093,6094,6095,6096,6097,6098,6099,6109,6112,6113,6114,6115,6116,6117,6118,6119,6120,6121,6155,6156,6157,6160,6161,6162,6163,6164,6165,6166,6167,6168,6169,6313,6432,6433,6434,6435,6436,6437,6438,6439,6440,6441,6442,6443,6448,6449,6450,6451,6452,6453,6454,6455,6456,6457,6458,6459,6470,6471,6472,6473,6474,6475,6476,6477,6478,6479,6608,6609,6610,6611,6612,6613,6614,6615,6616,6617,6618,6679,6680,6681,6682,6683,6741,6742,6743,6744,6745,6746,6747,6748,6749,6750,6752,6753,6754,6755,6756,6757,6758,6759,6760,6761,6762,6763,6764,6765,6766,6767,6768,6769,6770,6771,6772,6773,6774,6775,6776,6777,6778,6779,6780,6783,6784,6785,6786,6787,6788,6789,6790,6791,6792,6793,6800,6801,6802,6803,6804,6805,6806,6807,6808,6809,6832,6833,6834,6835,6836,6837,6838,6839,6840,6841,6842,6843,6844,6845,6912,6913,6914,6915,6916,6964,6965,6966,6967,6968,6969,6970,6971,6972,6973,6974,6975,6976,6977,6978,6979,6980,6992,6993,6994,6995,6996,6997,6998,6999,7000,7001,7019,7020,7021,7022,7023,7024,7025,7026,7027,7040,7041,7042,7073,7074,7075,7076,7077,7078,7079,7080,7081,7082,7083,7084,7085,7088,7089,7090,7091,7092,7093,7094,7095,7096,7097,7142,7143,7144,7145,7146,7147,7148,7149,7150,7151,7152,7153,7154,7155,7204,7205,7206,7207,7208,7209,7210,7211,7212,7213,7214,7215,7216,7217,7218,7219,7220,7221,7222,7223,7232,7233,7234,7235,7236,7237,7238,7239,7240,7241,7248,7249,7250,7251,7252,7253,7254,7255,7256,7257,7376,7377,7378,7380,7381,7382,7383,7384,7385,7386,7387,7388,7389,7390,7391,7392,7393,7394,7395,7396,7397,7398,7399,7400,7405,7410,7411,7412,7415,7416,7417,7616,7617,7618,7619,7620,7621,7622,7623,7624,7625,7626,7627,7628,7629,7630,7631,7632,7633,7634,7635,7636,7637,7638,7639,7640,7641,7642,7643,7644,7645,7646,7647,7648,7649,7650,7651,7652,7653,7654,7655,7656,7657,7658,7659,7660,7661,7662,7663,7664,7665,7666,7667,7668,7669,7670,7671,7672,7673,7675,7676,7677,7678,7679,8204,8205,8255,8256,8276,8400,8401,8402,8403,8404,8405,8406,8407,8408,8409,8410,8411,8412,8417,8421,8422,8423,8424,8425,8426,8427,8428,8429,8430,8431,8432,11503,11504,11505,11647,11744,11745,11746,11747,11748,11749,11750,11751,11752,11753,11754,11755,11756,11757,11758,11759,11760,11761,11762,11763,11764,11765,11766,11767,11768,11769,11770,11771,11772,11773,11774,11775,12330,12331,12332,12333,12334,12335,12441,12442,42528,42529,42530,42531,42532,42533,42534,42535,42536,42537,42607,42612,42613,42614,42615,42616,42617,42618,42619,42620,42621,42654,42655,42736,42737,43010,43014,43019,43043,43044,43045,43046,43047,43136,43137,43188,43189,43190,43191,43192,43193,43194,43195,43196,43197,43198,43199,43200,43201,43202,43203,43204,43205,43216,43217,43218,43219,43220,43221,43222,43223,43224,43225,43232,43233,43234,43235,43236,43237,43238,43239,43240,43241,43242,43243,43244,43245,43246,43247,43248,43249,43263,43264,43265,43266,43267,43268,43269,43270,43271,43272,43273,43302,43303,43304,43305,43306,43307,43308,43309,43335,43336,43337,43338,43339,43340,43341,43342,43343,43344,43345,43346,43347,43392,43393,43394,43395,43443,43444,43445,43446,43447,43448,43449,43450,43451,43452,43453,43454,43455,43456,43472,43473,43474,43475,43476,43477,43478,43479,43480,43481,43493,43504,43505,43506,43507,43508,43509,43510,43511,43512,43513,43561,43562,43563,43564,43565,43566,43567,43568,43569,43570,43571,43572,43573,43574,43587,43596,43597,43600,43601,43602,43603,43604,43605,43606,43607,43608,43609,43643,43644,43645,43696,43698,43699,43700,43703,43704,43710,43711,43713,43755,43756,43757,43758,43759,43765,43766,44003,44004,44005,44006,44007,44008,44009,44010,44012,44013,44016,44017,44018,44019,44020,44021,44022,44023,44024,44025,64286,65024,65025,65026,65027,65028,65029,65030,65031,65032,65033,65034,65035,65036,65037,65038,65039,65056,65057,65058,65059,65060,65061,65062,65063,65064,65065,65066,65067,65068,65069,65070,65071,65075,65076,65101,65102,65103,65296,65297,65298,65299,65300,65301,65302,65303,65304,65305,65343".split(",").map(function(t){return parseInt(t,10)});e.exports=r},{}],4:[function(t,e,n){var r="170,181,186,192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,208,209,210,211,212,213,214,216,217,218,219,220,221,222,223,224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239,240,241,242,243,244,245,246,248,249,250,251,252,253,254,255,256,257,258,259,260,261,262,263,264,265,266,267,268,269,270,271,272,273,274,275,276,277,278,279,280,281,282,283,284,285,286,287,288,289,290,291,292,293,294,295,296,297,298,299,300,301,302,303,304,305,306,307,308,309,310,311,312,313,314,315,316,317,318,319,320,321,322,323,324,325,326,327,328,329,330,331,332,333,334,335,336,337,338,339,340,341,342,343,344,345,346,347,348,349,350,351,352,353,354,355,356,357,358,359,360,361,362,363,364,365,366,367,368,369,370,371,372,373,374,375,376,377,378,379,380,381,382,383,384,385,386,387,388,389,390,391,392,393,394,395,396,397,398,399,400,401,402,403,404,405,406,407,408,409,410,411,412,413,414,415,416,417,418,419,420,421,422,423,424,425,426,427,428,429,430,431,432,433,434,435,436,437,438,439,440,441,442,443,444,445,446,447,448,449,450,451,452,453,454,455,456,457,458,459,460,461,462,463,464,465,466,467,468,469,470,471,472,473,474,475,476,477,478,479,480,481,482,483,484,485,486,487,488,489,490,491,492,493,494,495,496,497,498,499,500,501,502,503,504,505,506,507,508,509,510,511,512,513,514,515,516,517,518,519,520,521,522,523,524,525,526,527,528,529,530,531,532,533,534,535,536,537,538,539,540,541,542,543,544,545,546,547,548,549,550,551,552,553,554,555,556,557,558,559,560,561,562,563,564,565,566,567,568,569,570,571,572,573,574,575,576,577,578,579,580,581,582,583,584,585,586,587,588,589,590,591,592,593,594,595,596,597,598,599,600,601,602,603,604,605,606,607,608,609,610,611,612,613,614,615,616,617,618,619,620,621,622,623,624,625,626,627,628,629,630,631,632,633,634,635,636,637,638,639,640,641,642,643,644,645,646,647,648,649,650,651,652,653,654,655,656,657,658,659,660,661,662,663,664,665,666,667,668,669,670,671,672,673,674,675,676,677,678,679,680,681,682,683,684,685,686,687,688,689,690,691,692,693,694,695,696,697,698,699,700,701,702,703,704,705,710,711,712,713,714,715,716,717,718,719,720,721,736,737,738,739,740,748,750,880,881,882,883,884,886,887,890,891,892,893,895,902,904,905,906,908,910,911,912,913,914,915,916,917,918,919,920,921,922,923,924,925,926,927,928,929,931,932,933,934,935,936,937,938,939,940,941,942,943,944,945,946,947,948,949,950,951,952,953,954,955,956,957,958,959,960,961,962,963,964,965,966,967,968,969,970,971,972,973,974,975,976,977,978,979,980,981,982,983,984,985,986,987,988,989,990,991,992,993,994,995,996,997,998,999,1000,1001,1002,1003,1004,1005,1006,1007,1008,1009,1010,1011,1012,1013,1015,1016,1017,1018,1019,1020,1021,1022,1023,1024,1025,1026,1027,1028,1029,1030,1031,1032,1033,1034,1035,1036,1037,1038,1039,1040,1041,1042,1043,1044,1045,1046,1047,1048,1049,1050,1051,1052,1053,1054,1055,1056,1057,1058,1059,1060,1061,1062,1063,1064,1065,1066,1067,1068,1069,1070,1071,1072,1073,1074,1075,1076,1077,1078,1079,1080,1081,1082,1083,1084,1085,1086,1087,1088,1089,1090,1091,1092,1093,1094,1095,1096,1097,1098,1099,1100,1101,1102,1103,1104,1105,1106,1107,1108,1109,1110,1111,1112,1113,1114,1115,1116,1117,1118,1119,1120,1121,1122,1123,1124,1125,1126,1127,1128,1129,1130,1131,1132,1133,1134,1135,1136,1137,1138,1139,1140,1141,1142,1143,1144,1145,1146,1147,1148,1149,1150,1151,1152,1153,1162,1163,1164,1165,1166,1167,1168,1169,1170,1171,1172,1173,1174,1175,1176,1177,1178,1179,1180,1181,1182,1183,1184,1185,1186,1187,1188,1189,1190,1191,1192,1193,1194,1195,1196,1197,1198,1199,1200,1201,1202,1203,1204,1205,1206,1207,1208,1209,1210,1211,1212,1213,1214,1215,1216,1217,1218,1219,1220,1221,1222,1223,1224,1225,1226,1227,1228,1229,1230,1231,1232,1233,1234,1235,1236,1237,1238,1239,1240,1241,1242,1243,1244,1245,1246,1247,1248,1249,1250,1251,1252,1253,1254,1255,1256,1257,1258,1259,1260,1261,1262,1263,1264,1265,1266,1267,1268,1269,1270,1271,1272,1273,1274,1275,1276,1277,1278,1279,1280,1281,1282,1283,1284,1285,1286,1287,1288,1289,1290,1291,1292,1293,1294,1295,1296,1297,1298,1299,1300,1301,1302,1303,1304,1305,1306,1307,1308,1309,1310,1311,1312,1313,1314,1315,1316,1317,1318,1319,1320,1321,1322,1323,1324,1325,1326,1327,1329,1330,1331,1332,1333,1334,1335,1336,1337,1338,1339,1340,1341,1342,1343,1344,1345,1346,1347,1348,1349,1350,1351,1352,1353,1354,1355,1356,1357,1358,1359,1360,1361,1362,1363,1364,1365,1366,1369,1376,1377,1378,1379,1380,1381,1382,1383,1384,1385,1386,1387,1388,1389,1390,1391,1392,1393,1394,1395,1396,1397,1398,1399,1400,1401,1402,1403,1404,1405,1406,1407,1408,1409,1410,1411,1412,1413,1414,1415,1416,1488,1489,1490,1491,1492,1493,1494,1495,1496,1497,1498,1499,1500,1501,1502,1503,1504,1505,1506,1507,1508,1509,1510,1511,1512,1513,1514,1519,1520,1521,1522,1568,1569,1570,1571,1572,1573,1574,1575,1576,1577,1578,1579,1580,1581,1582,1583,1584,1585,1586,1587,1588,1589,1590,1591,1592,1593,1594,1595,1596,1597,1598,1599,1600,1601,1602,1603,1604,1605,1606,1607,1608,1609,1610,1646,1647,1649,1650,1651,1652,1653,1654,1655,1656,1657,1658,1659,1660,1661,1662,1663,1664,1665,1666,1667,1668,1669,1670,1671,1672,1673,1674,1675,1676,1677,1678,1679,1680,1681,1682,1683,1684,1685,1686,1687,1688,1689,1690,1691,1692,1693,1694,1695,1696,1697,1698,1699,1700,1701,1702,1703,1704,1705,1706,1707,1708,1709,1710,1711,1712,1713,1714,1715,1716,1717,1718,1719,1720,1721,1722,1723,1724,1725,1726,1727,1728,1729,1730,1731,1732,1733,1734,1735,1736,1737,1738,1739,1740,1741,1742,1743,1744,1745,1746,1747,1749,1765,1766,1774,1775,1786,1787,1788,1791,1808,1810,1811,1812,1813,1814,1815,1816,1817,1818,1819,1820,1821,1822,1823,1824,1825,1826,1827,1828,1829,1830,1831,1832,1833,1834,1835,1836,1837,1838,1839,1869,1870,1871,1872,1873,1874,1875,1876,1877,1878,1879,1880,1881,1882,1883,1884,1885,1886,1887,1888,1889,1890,1891,1892,1893,1894,1895,1896,1897,1898,1899,1900,1901,1902,1903,1904,1905,1906,1907,1908,1909,1910,1911,1912,1913,1914,1915,1916,1917,1918,1919,1920,1921,1922,1923,1924,1925,1926,1927,1928,1929,1930,1931,1932,1933,1934,1935,1936,1937,1938,1939,1940,1941,1942,1943,1944,1945,1946,1947,1948,1949,1950,1951,1952,1953,1954,1955,1956,1957,1969,1994,1995,1996,1997,1998,1999,2000,2001,2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023,2024,2025,2026,2036,2037,2042,2048,2049,2050,2051,2052,2053,2054,2055,2056,2057,2058,2059,2060,2061,2062,2063,2064,2065,2066,2067,2068,2069,2074,2084,2088,2112,2113,2114,2115,2116,2117,2118,2119,2120,2121,2122,2123,2124,2125,2126,2127,2128,2129,2130,2131,2132,2133,2134,2135,2136,2144,2145,2146,2147,2148,2149,2150,2151,2152,2153,2154,2208,2209,2210,2211,2212,2213,2214,2215,2216,2217,2218,2219,2220,2221,2222,2223,2224,2225,2226,2227,2228,2230,2231,2232,2233,2234,2235,2236,2237,2308,2309,2310,2311,2312,2313,2314,2315,2316,2317,2318,2319,2320,2321,2322,2323,2324,2325,2326,2327,2328,2329,2330,2331,2332,2333,2334,2335,2336,2337,2338,2339,2340,2341,2342,2343,2344,2345,2346,2347,2348,2349,2350,2351,2352,2353,2354,2355,2356,2357,2358,2359,2360,2361,2365,2384,2392,2393,2394,2395,2396,2397,2398,2399,2400,2401,2417,2418,2419,2420,2421,2422,2423,2424,2425,2426,2427,2428,2429,2430,2431,2432,2437,2438,2439,2440,2441,2442,2443,2444,2447,2448,2451,2452,2453,2454,2455,2456,2457,2458,2459,2460,2461,2462,2463,2464,2465,2466,2467,2468,2469,2470,2471,2472,2474,2475,2476,2477,2478,2479,2480,2482,2486,2487,2488,2489,2493,2510,2524,2525,2527,2528,2529,2544,2545,2556,2565,2566,2567,2568,2569,2570,2575,2576,2579,2580,2581,2582,2583,2584,2585,2586,2587,2588,2589,2590,2591,2592,2593,2594,2595,2596,2597,2598,2599,2600,2602,2603,2604,2605,2606,2607,2608,2610,2611,2613,2614,2616,2617,2649,2650,2651,2652,2654,2674,2675,2676,2693,2694,2695,2696,2697,2698,2699,2700,2701,2703,2704,2705,2707,2708,2709,2710,2711,2712,2713,2714,2715,2716,2717,2718,2719,2720,2721,2722,2723,2724,2725,2726,2727,2728,2730,2731,2732,2733,2734,2735,2736,2738,2739,2741,2742,2743,2744,2745,2749,2768,2784,2785,2809,2821,2822,2823,2824,2825,2826,2827,2828,2831,2832,2835,2836,2837,2838,2839,2840,2841,2842,2843,2844,2845,2846,2847,2848,2849,2850,2851,2852,2853,2854,2855,2856,2858,2859,2860,2861,2862,2863,2864,2866,2867,2869,2870,2871,2872,2873,2877,2908,2909,2911,2912,2913,2929,2947,2949,2950,2951,2952,2953,2954,2958,2959,2960,2962,2963,2964,2965,2969,2970,2972,2974,2975,2979,2980,2984,2985,2986,2990,2991,2992,2993,2994,2995,2996,2997,2998,2999,3000,3001,3024,3077,3078,3079,3080,3081,3082,3083,3084,3086,3087,3088,3090,3091,3092,3093,3094,3095,3096,3097,3098,3099,3100,3101,3102,3103,3104,3105,3106,3107,3108,3109,3110,3111,3112,3114,3115,3116,3117,3118,3119,3120,3121,3122,3123,3124,3125,3126,3127,3128,3129,3133,3160,3161,3162,3168,3169,3200,3205,3206,3207,3208,3209,3210,3211,3212,3214,3215,3216,3218,3219,3220,3221,3222,3223,3224,3225,3226,3227,3228,3229,3230,3231,3232,3233,3234,3235,3236,3237,3238,3239,3240,3242,3243,3244,3245,3246,3247,3248,3249,3250,3251,3253,3254,3255,3256,3257,3261,3294,3296,3297,3313,3314,3333,3334,3335,3336,3337,3338,3339,3340,3342,3343,3344,3346,3347,3348,3349,3350,3351,3352,3353,3354,3355,3356,3357,3358,3359,3360,3361,3362,3363,3364,3365,3366,3367,3368,3369,3370,3371,3372,3373,3374,3375,3376,3377,3378,3379,3380,3381,3382,3383,3384,3385,3386,3389,3406,3412,3413,3414,3423,3424,3425,3450,3451,3452,3453,3454,3455,3461,3462,3463,3464,3465,3466,3467,3468,3469,3470,3471,3472,3473,3474,3475,3476,3477,3478,3482,3483,3484,3485,3486,3487,3488,3489,3490,3491,3492,3493,3494,3495,3496,3497,3498,3499,3500,3501,3502,3503,3504,3505,3507,3508,3509,3510,3511,3512,3513,3514,3515,3517,3520,3521,3522,3523,3524,3525,3526,3585,3586,3587,3588,3589,3590,3591,3592,3593,3594,3595,3596,3597,3598,3599,3600,3601,3602,3603,3604,3605,3606,3607,3608,3609,3610,3611,3612,3613,3614,3615,3616,3617,3618,3619,3620,3621,3622,3623,3624,3625,3626,3627,3628,3629,3630,3631,3632,3634,3635,3648,3649,3650,3651,3652,3653,3654,3713,3714,3716,3719,3720,3722,3725,3732,3733,3734,3735,3737,3738,3739,3740,3741,3742,3743,3745,3746,3747,3749,3751,3754,3755,3757,3758,3759,3760,3762,3763,3773,3776,3777,3778,3779,3780,3782,3804,3805,3806,3807,3840,3904,3905,3906,3907,3908,3909,3910,3911,3913,3914,3915,3916,3917,3918,3919,3920,3921,3922,3923,3924,3925,3926,3927,3928,3929,3930,3931,3932,3933,3934,3935,3936,3937,3938,3939,3940,3941,3942,3943,3944,3945,3946,3947,3948,3976,3977,3978,3979,3980,4096,4097,4098,4099,4100,4101,4102,4103,4104,4105,4106,4107,4108,4109,4110,4111,4112,4113,4114,4115,4116,4117,4118,4119,4120,4121,4122,4123,4124,4125,4126,4127,4128,4129,4130,4131,4132,4133,4134,4135,4136,4137,4138,4159,4176,4177,4178,4179,4180,4181,4186,4187,4188,4189,4193,4197,4198,4206,4207,4208,4213,4214,4215,4216,4217,4218,4219,4220,4221,4222,4223,4224,4225,4238,4256,4257,4258,4259,4260,4261,4262,4263,4264,4265,4266,4267,4268,4269,4270,4271,4272,4273,4274,4275,4276,4277,4278,4279,4280,4281,4282,4283,4284,4285,4286,4287,4288,4289,4290,4291,4292,4293,4295,4301,4304,4305,4306,4307,4308,4309,4310,4311,4312,4313,4314,4315,4316,4317,4318,4319,4320,4321,4322,4323,4324,4325,4326,4327,4328,4329,4330,4331,4332,4333,4334,4335,4336,4337,4338,4339,4340,4341,4342,4343,4344,4345,4346,4348,4349,4350,4351,4352,4353,4354,4355,4356,4357,4358,4359,4360,4361,4362,4363,4364,4365,4366,4367,4368,4369,4370,4371,4372,4373,4374,4375,4376,4377,4378,4379,4380,4381,4382,4383,4384,4385,4386,4387,4388,4389,4390,4391,4392,4393,4394,4395,4396,4397,4398,4399,4400,4401,4402,4403,4404,4405,4406,4407,4408,4409,4410,4411,4412,4413,4414,4415,4416,4417,4418,4419,4420,4421,4422,4423,4424,4425,4426,4427,4428,4429,4430,4431,4432,4433,4434,4435,4436,4437,4438,4439,4440,4441,4442,4443,4444,4445,4446,4447,4448,4449,4450,4451,4452,4453,4454,4455,4456,4457,4458,4459,4460,4461,4462,4463,4464,4465,4466,4467,4468,4469,4470,4471,4472,4473,4474,4475,4476,4477,4478,4479,4480,4481,4482,4483,4484,4485,4486,4487,4488,4489,4490,4491,4492,4493,4494,4495,4496,4497,4498,4499,4500,4501,4502,4503,4504,4505,4506,4507,4508,4509,4510,4511,4512,4513,4514,4515,4516,4517,4518,4519,4520,4521,4522,4523,4524,4525,4526,4527,4528,4529,4530,4531,4532,4533,4534,4535,4536,4537,4538,4539,4540,4541,4542,4543,4544,4545,4546,4547,4548,4549,4550,4551,4552,4553,4554,4555,4556,4557,4558,4559,4560,4561,4562,4563,4564,4565,4566,4567,4568,4569,4570,4571,4572,4573,4574,4575,4576,4577,4578,4579,4580,4581,4582,4583,4584,4585,4586,4587,4588,4589,4590,4591,4592,4593,4594,4595,4596,4597,4598,4599,4600,4601,4602,4603,4604,4605,4606,4607,4608,4609,4610,4611,4612,4613,4614,4615,4616,4617,4618,4619,4620,4621,4622,4623,4624,4625,4626,4627,4628,4629,4630,4631,4632,4633,4634,4635,4636,4637,4638,4639,4640,4641,4642,4643,4644,4645,4646,4647,4648,4649,4650,4651,4652,4653,4654,4655,4656,4657,4658,4659,4660,4661,4662,4663,4664,4665,4666,4667,4668,4669,4670,4671,4672,4673,4674,4675,4676,4677,4678,4679,4680,4682,4683,4684,4685,4688,4689,4690,4691,4692,4693,4694,4696,4698,4699,4700,4701,4704,4705,4706,4707,4708,4709,4710,4711,4712,4713,4714,4715,4716,4717,4718,4719,4720,4721,4722,4723,4724,4725,4726,4727,4728,4729,4730,4731,4732,4733,4734,4735,4736,4737,4738,4739,4740,4741,4742,4743,4744,4746,4747,4748,4749,4752,4753,4754,4755,4756,4757,4758,4759,4760,4761,4762,4763,4764,4765,4766,4767,4768,4769,4770,4771,4772,4773,4774,4775,4776,4777,4778,4779,4780,4781,4782,4783,4784,4786,4787,4788,4789,4792,4793,4794,4795,4796,4797,4798,4800,4802,4803,4804,4805,4808,4809,4810,4811,4812,4813,4814,4815,4816,4817,4818,4819,4820,4821,4822,4824,4825,4826,4827,4828,4829,4830,4831,4832,4833,4834,4835,4836,4837,4838,4839,4840,4841,4842,4843,4844,4845,4846,4847,4848,4849,4850,4851,4852,4853,4854,4855,4856,4857,4858,4859,4860,4861,4862,4863,4864,4865,4866,4867,4868,4869,4870,4871,4872,4873,4874,4875,4876,4877,4878,4879,4880,4882,4883,4884,4885,4888,4889,4890,4891,4892,4893,4894,4895,4896,4897,4898,4899,4900,4901,4902,4903,4904,4905,4906,4907,4908,4909,4910,4911,4912,4913,4914,4915,4916,4917,4918,4919,4920,4921,4922,4923,4924,4925,4926,4927,4928,4929,4930,4931,4932,4933,4934,4935,4936,4937,4938,4939,4940,4941,4942,4943,4944,4945,4946,4947,4948,4949,4950,4951,4952,4953,4954,4992,4993,4994,4995,4996,4997,4998,4999,5000,5001,5002,5003,5004,5005,5006,5007,5024,5025,5026,5027,5028,5029,5030,5031,5032,5033,5034,5035,5036,5037,5038,5039,5040,5041,5042,5043,5044,5045,5046,5047,5048,5049,5050,5051,5052,5053,5054,5055,5056,5057,5058,5059,5060,5061,5062,5063,5064,5065,5066,5067,5068,5069,5070,5071,5072,5073,5074,5075,5076,5077,5078,5079,5080,5081,5082,5083,5084,5085,5086,5087,5088,5089,5090,5091,5092,5093,5094,5095,5096,5097,5098,5099,5100,5101,5102,5103,5104,5105,5106,5107,5108,5109,5112,5113,5114,5115,5116,5117,5121,5122,5123,5124,5125,5126,5127,5128,5129,5130,5131,5132,5133,5134,5135,5136,5137,5138,5139,5140,5141,5142,5143,5144,5145,5146,5147,5148,5149,5150,5151,5152,5153,5154,5155,5156,5157,5158,5159,5160,5161,5162,5163,5164,5165,5166,5167,5168,5169,5170,5171,5172,5173,5174,5175,5176,5177,5178,5179,5180,5181,5182,5183,5184,5185,5186,5187,5188,5189,5190,5191,5192,5193,5194,5195,5196,5197,5198,5199,5200,5201,5202,5203,5204,5205,5206,5207,5208,5209,5210,5211,5212,5213,5214,5215,5216,5217,5218,5219,5220,5221,5222,5223,5224,5225,5226,5227,5228,5229,5230,5231,5232,5233,5234,5235,5236,5237,5238,5239,5240,5241,5242,5243,5244,5245,5246,5247,5248,5249,5250,5251,5252,5253,5254,5255,5256,5257,5258,5259,5260,5261,5262,5263,5264,5265,5266,5267,5268,5269,5270,5271,5272,5273,5274,5275,5276,5277,5278,5279,5280,5281,5282,5283,5284,5285,5286,5287,5288,5289,5290,5291,5292,5293,5294,5295,5296,5297,5298,5299,5300,5301,5302,5303,5304,5305,5306,5307,5308,5309,5310,5311,5312,5313,5314,5315,5316,5317,5318,5319,5320,5321,5322,5323,5324,5325,5326,5327,5328,5329,5330,5331,5332,5333,5334,5335,5336,5337,5338,5339,5340,5341,5342,5343,5344,5345,5346,5347,5348,5349,5350,5351,5352,5353,5354,5355,5356,5357,5358,5359,5360,5361,5362,5363,5364,5365,5366,5367,5368,5369,5370,5371,5372,5373,5374,5375,5376,5377,5378,5379,5380,5381,5382,5383,5384,5385,5386,5387,5388,5389,5390,5391,5392,5393,5394,5395,5396,5397,5398,5399,5400,5401,5402,5403,5404,5405,5406,5407,5408,5409,5410,5411,5412,5413,5414,5415,5416,5417,5418,5419,5420,5421,5422,5423,5424,5425,5426,5427,5428,5429,5430,5431,5432,5433,5434,5435,5436,5437,5438,5439,5440,5441,5442,5443,5444,5445,5446,5447,5448,5449,5450,5451,5452,5453,5454,5455,5456,5457,5458,5459,5460,5461,5462,5463,5464,5465,5466,5467,5468,5469,5470,5471,5472,5473,5474,5475,5476,5477,5478,5479,5480,5481,5482,5483,5484,5485,5486,5487,5488,5489,5490,5491,5492,5493,5494,5495,5496,5497,5498,5499,5500,5501,5502,5503,5504,5505,5506,5507,5508,5509,5510,5511,5512,5513,5514,5515,5516,5517,5518,5519,5520,5521,5522,5523,5524,5525,5526,5527,5528,5529,5530,5531,5532,5533,5534,5535,5536,5537,5538,5539,5540,5541,5542,5543,5544,5545,5546,5547,5548,5549,5550,5551,5552,5553,5554,5555,5556,5557,5558,5559,5560,5561,5562,5563,5564,5565,5566,5567,5568,5569,5570,5571,5572,5573,5574,5575,5576,5577,5578,5579,5580,5581,5582,5583,5584,5585,5586,5587,5588,5589,5590,5591,5592,5593,5594,5595,5596,5597,5598,5599,5600,5601,5602,5603,5604,5605,5606,5607,5608,5609,5610,5611,5612,5613,5614,5615,5616,5617,5618,5619,5620,5621,5622,5623,5624,5625,5626,5627,5628,5629,5630,5631,5632,5633,5634,5635,5636,5637,5638,5639,5640,5641,5642,5643,5644,5645,5646,5647,5648,5649,5650,5651,5652,5653,5654,5655,5656,5657,5658,5659,5660,5661,5662,5663,5664,5665,5666,5667,5668,5669,5670,5671,5672,5673,5674,5675,5676,5677,5678,5679,5680,5681,5682,5683,5684,5685,5686,5687,5688,5689,5690,5691,5692,5693,5694,5695,5696,5697,5698,5699,5700,5701,5702,5703,5704,5705,5706,5707,5708,5709,5710,5711,5712,5713,5714,5715,5716,5717,5718,5719,5720,5721,5722,5723,5724,5725,5726,5727,5728,5729,5730,5731,5732,5733,5734,5735,5736,5737,5738,5739,5740,5743,5744,5745,5746,5747,5748,5749,5750,5751,5752,5753,5754,5755,5756,5757,5758,5759,5761,5762,5763,5764,5765,5766,5767,5768,5769,5770,5771,5772,5773,5774,5775,5776,5777,5778,5779,5780,5781,5782,5783,5784,5785,5786,5792,5793,5794,5795,5796,5797,5798,5799,5800,5801,5802,5803,5804,5805,5806,5807,5808,5809,5810,5811,5812,5813,5814,5815,5816,5817,5818,5819,5820,5821,5822,5823,5824,5825,5826,5827,5828,5829,5830,5831,5832,5833,5834,5835,5836,5837,5838,5839,5840,5841,5842,5843,5844,5845,5846,5847,5848,5849,5850,5851,5852,5853,5854,5855,5856,5857,5858,5859,5860,5861,5862,5863,5864,5865,5866,5870,5871,5872,5873,5874,5875,5876,5877,5878,5879,5880,5888,5889,5890,5891,5892,5893,5894,5895,5896,5897,5898,5899,5900,5902,5903,5904,5905,5920,5921,5922,5923,5924,5925,5926,5927,5928,5929,5930,5931,5932,5933,5934,5935,5936,5937,5952,5953,5954,5955,5956,5957,5958,5959,5960,5961,5962,5963,5964,5965,5966,5967,5968,5969,5984,5985,5986,5987,5988,5989,5990,5991,5992,5993,5994,5995,5996,5998,5999,6000,6016,6017,6018,6019,6020,6021,6022,6023,6024,6025,6026,6027,6028,6029,6030,6031,6032,6033,6034,6035,6036,6037,6038,6039,6040,6041,6042,6043,6044,6045,6046,6047,6048,6049,6050,6051,6052,6053,6054,6055,6056,6057,6058,6059,6060,6061,6062,6063,6064,6065,6066,6067,6103,6108,6176,6177,6178,6179,6180,6181,6182,6183,6184,6185,6186,6187,6188,6189,6190,6191,6192,6193,6194,6195,6196,6197,6198,6199,6200,6201,6202,6203,6204,6205,6206,6207,6208,6209,6210,6211,6212,6213,6214,6215,6216,6217,6218,6219,6220,6221,6222,6223,6224,6225,6226,6227,6228,6229,6230,6231,6232,6233,6234,6235,6236,6237,6238,6239,6240,6241,6242,6243,6244,6245,6246,6247,6248,6249,6250,6251,6252,6253,6254,6255,6256,6257,6258,6259,6260,6261,6262,6263,6264,6272,6273,6274,6275,6276,6277,6278,6279,6280,6281,6282,6283,6284,6285,6286,6287,6288,6289,6290,6291,6292,6293,6294,6295,6296,6297,6298,6299,6300,6301,6302,6303,6304,6305,6306,6307,6308,6309,6310,6311,6312,6314,6320,6321,6322,6323,6324,6325,6326,6327,6328,6329,6330,6331,6332,6333,6334,6335,6336,6337,6338,6339,6340,6341,6342,6343,6344,6345,6346,6347,6348,6349,6350,6351,6352,6353,6354,6355,6356,6357,6358,6359,6360,6361,6362,6363,6364,6365,6366,6367,6368,6369,6370,6371,6372,6373,6374,6375,6376,6377,6378,6379,6380,6381,6382,6383,6384,6385,6386,6387,6388,6389,6400,6401,6402,6403,6404,6405,6406,6407,6408,6409,6410,6411,6412,6413,6414,6415,6416,6417,6418,6419,6420,6421,6422,6423,6424,6425,6426,6427,6428,6429,6430,6480,6481,6482,6483,6484,6485,6486,6487,6488,6489,6490,6491,6492,6493,6494,6495,6496,6497,6498,6499,6500,6501,6502,6503,6504,6505,6506,6507,6508,6509,6512,6513,6514,6515,6516,6528,6529,6530,6531,6532,6533,6534,6535,6536,6537,6538,6539,6540,6541,6542,6543,6544,6545,6546,6547,6548,6549,6550,6551,6552,6553,6554,6555,6556,6557,6558,6559,6560,6561,6562,6563,6564,6565,6566,6567,6568,6569,6570,6571,6576,6577,6578,6579,6580,6581,6582,6583,6584,6585,6586,6587,6588,6589,6590,6591,6592,6593,6594,6595,6596,6597,6598,6599,6600,6601,6656,6657,6658,6659,6660,6661,6662,6663,6664,6665,6666,6667,6668,6669,6670,6671,6672,6673,6674,6675,6676,6677,6678,6688,6689,6690,6691,6692,6693,6694,6695,6696,6697,6698,6699,6700,6701,6702,6703,6704,6705,6706,6707,6708,6709,6710,6711,6712,6713,6714,6715,6716,6717,6718,6719,6720,6721,6722,6723,6724,6725,6726,6727,6728,6729,6730,6731,6732,6733,6734,6735,6736,6737,6738,6739,6740,6823,6917,6918,6919,6920,6921,6922,6923,6924,6925,6926,6927,6928,6929,6930,6931,6932,6933,6934,6935,6936,6937,6938,6939,6940,6941,6942,6943,6944,6945,6946,6947,6948,6949,6950,6951,6952,6953,6954,6955,6956,6957,6958,6959,6960,6961,6962,6963,6981,6982,6983,6984,6985,6986,6987,7043,7044,7045,7046,7047,7048,7049,7050,7051,7052,7053,7054,7055,7056,7057,7058,7059,7060,7061,7062,7063,7064,7065,7066,7067,7068,7069,7070,7071,7072,7086,7087,7098,7099,7100,7101,7102,7103,7104,7105,7106,7107,7108,7109,7110,7111,7112,7113,7114,7115,7116,7117,7118,7119,7120,7121,7122,7123,7124,7125,7126,7127,7128,7129,7130,7131,7132,7133,7134,7135,7136,7137,7138,7139,7140,7141,7168,7169,7170,7171,7172,7173,7174,7175,7176,7177,7178,7179,7180,7181,7182,7183,7184,7185,7186,7187,7188,7189,7190,7191,7192,7193,7194,7195,7196,7197,7198,7199,7200,7201,7202,7203,7245,7246,7247,7258,7259,7260,7261,7262,7263,7264,7265,7266,7267,7268,7269,7270,7271,7272,7273,7274,7275,7276,7277,7278,7279,7280,7281,7282,7283,7284,7285,7286,7287,7288,7289,7290,7291,7292,7293,7296,7297,7298,7299,7300,7301,7302,7303,7304,7312,7313,7314,7315,7316,7317,7318,7319,7320,7321,7322,7323,7324,7325,7326,7327,7328,7329,7330,7331,7332,7333,7334,7335,7336,7337,7338,7339,7340,7341,7342,7343,7344,7345,7346,7347,7348,7349,7350,7351,7352,7353,7354,7357,7358,7359,7401,7402,7403,7404,7406,7407,7408,7409,7413,7414,7424,7425,7426,7427,7428,7429,7430,7431,7432,7433,7434,7435,7436,7437,7438,7439,7440,7441,7442,7443,7444,7445,7446,7447,7448,7449,7450,7451,7452,7453,7454,7455,7456,7457,7458,7459,7460,7461,7462,7463,7464,7465,7466,7467,7468,7469,7470,7471,7472,7473,7474,7475,7476,7477,7478,7479,7480,7481,7482,7483,7484,7485,7486,7487,7488,7489,7490,7491,7492,7493,7494,7495,7496,7497,7498,7499,7500,7501,7502,7503,7504,7505,7506,7507,7508,7509,7510,7511,7512,7513,7514,7515,7516,7517,7518,7519,7520,7521,7522,7523,7524,7525,7526,7527,7528,7529,7530,7531,7532,7533,7534,7535,7536,7537,7538,7539,7540,7541,7542,7543,7544,7545,7546,7547,7548,7549,7550,7551,7552,7553,7554,7555,7556,7557,7558,7559,7560,7561,7562,7563,7564,7565,7566,7567,7568,7569,7570,7571,7572,7573,7574,7575,7576,7577,7578,7579,7580,7581,7582,7583,7584,7585,7586,7587,7588,7589,7590,7591,7592,7593,7594,7595,7596,7597,7598,7599,7600,7601,7602,7603,7604,7605,7606,7607,7608,7609,7610,7611,7612,7613,7614,7615,7680,7681,7682,7683,7684,7685,7686,7687,7688,7689,7690,7691,7692,7693,7694,7695,7696,7697,7698,7699,7700,7701,7702,7703,7704,7705,7706,7707,7708,7709,7710,7711,7712,7713,7714,7715,7716,7717,7718,7719,7720,7721,7722,7723,7724,7725,7726,7727,7728,7729,7730,7731,7732,7733,7734,7735,7736,7737,7738,7739,7740,7741,7742,7743,7744,7745,7746,7747,7748,7749,7750,7751,7752,7753,7754,7755,7756,7757,7758,7759,7760,7761,7762,7763,7764,7765,7766,7767,7768,7769,7770,7771,7772,7773,7774,7775,7776,7777,7778,7779,7780,7781,7782,7783,7784,7785,7786,7787,7788,7789,7790,7791,7792,7793,7794,7795,7796,7797,7798,7799,7800,7801,7802,7803,7804,7805,7806,7807,7808,7809,7810,7811,7812,7813,7814,7815,7816,7817,7818,7819,7820,7821,7822,7823,7824,7825,7826,7827,7828,7829,7830,7831,7832,7833,7834,7835,7836,7837,7838,7839,7840,7841,7842,7843,7844,7845,7846,7847,7848,7849,7850,7851,7852,7853,7854,7855,7856,7857,7858,7859,7860,7861,7862,7863,7864,7865,7866,7867,7868,7869,7870,7871,7872,7873,7874,7875,7876,7877,7878,7879,7880,7881,7882,7883,7884,7885,7886,7887,7888,7889,7890,7891,7892,7893,7894,7895,7896,7897,7898,7899,7900,7901,7902,7903,7904,7905,7906,7907,7908,7909,7910,7911,7912,7913,7914,7915,7916,7917,7918,7919,7920,7921,7922,7923,7924,7925,7926,7927,7928,7929,7930,7931,7932,7933,7934,7935,7936,7937,7938,7939,7940,7941,7942,7943,7944,7945,7946,7947,7948,7949,7950,7951,7952,7953,7954,7955,7956,7957,7960,7961,7962,7963,7964,7965,7968,7969,7970,7971,7972,7973,7974,7975,7976,7977,7978,7979,7980,7981,7982,7983,7984,7985,7986,7987,7988,7989,7990,7991,7992,7993,7994,7995,7996,7997,7998,7999,8000,8001,8002,8003,8004,8005,8008,8009,8010,8011,8012,8013,8016,8017,8018,8019,8020,8021,8022,8023,8025,8027,8029,8031,8032,8033,8034,8035,8036,8037,8038,8039,8040,8041,8042,8043,8044,8045,8046,8047,8048,8049,8050,8051,8052,8053,8054,8055,8056,8057,8058,8059,8060,8061,8064,8065,8066,8067,8068,8069,8070,8071,8072,8073,8074,8075,8076,8077,8078,8079,8080,8081,8082,8083,8084,8085,8086,8087,8088,8089,8090,8091,8092,8093,8094,8095,8096,8097,8098,8099,8100,8101,8102,8103,8104,8105,8106,8107,8108,8109,8110,8111,8112,8113,8114,8115,8116,8118,8119,8120,8121,8122,8123,8124,8126,8130,8131,8132,8134,8135,8136,8137,8138,8139,8140,8144,8145,8146,8147,8150,8151,8152,8153,8154,8155,8160,8161,8162,8163,8164,8165,8166,8167,8168,8169,8170,8171,8172,8178,8179,8180,8182,8183,8184,8185,8186,8187,8188,8305,8319,8336,8337,8338,8339,8340,8341,8342,8343,8344,8345,8346,8347,8348,8450,8455,8458,8459,8460,8461,8462,8463,8464,8465,8466,8467,8469,8472,8473,8474,8475,8476,8477,8484,8486,8488,8490,8491,8492,8493,8494,8495,8496,8497,8498,8499,8500,8501,8502,8503,8504,8505,8508,8509,8510,8511,8517,8518,8519,8520,8521,8526,8544,8545,8546,8547,8548,8549,8550,8551,8552,8553,8554,8555,8556,8557,8558,8559,8560,8561,8562,8563,8564,8565,8566,8567,8568,8569,8570,8571,8572,8573,8574,8575,8576,8577,8578,8579,8580,8581,8582,8583,8584,11264,11265,11266,11267,11268,11269,11270,11271,11272,11273,11274,11275,11276,11277,11278,11279,11280,11281,11282,11283,11284,11285,11286,11287,11288,11289,11290,11291,11292,11293,11294,11295,11296,11297,11298,11299,11300,11301,11302,11303,11304,11305,11306,11307,11308,11309,11310,11312,11313,11314,11315,11316,11317,11318,11319,11320,11321,11322,11323,11324,11325,11326,11327,11328,11329,11330,11331,11332,11333,11334,11335,11336,11337,11338,11339,11340,11341,11342,11343,11344,11345,11346,11347,11348,11349,11350,11351,11352,11353,11354,11355,11356,11357,11358,11360,11361,11362,11363,11364,11365,11366,11367,11368,11369,11370,11371,11372,11373,11374,11375,11376,11377,11378,11379,11380,11381,11382,11383,11384,11385,11386,11387,11388,11389,11390,11391,11392,11393,11394,11395,11396,11397,11398,11399,11400,11401,11402,11403,11404,11405,11406,11407,11408,11409,11410,11411,11412,11413,11414,11415,11416,11417,11418,11419,11420,11421,11422,11423,11424,11425,11426,11427,11428,11429,11430,11431,11432,11433,11434,11435,11436,11437,11438,11439,11440,11441,11442,11443,11444,11445,11446,11447,11448,11449,11450,11451,11452,11453,11454,11455,11456,11457,11458,11459,11460,11461,11462,11463,11464,11465,11466,11467,11468,11469,11470,11471,11472,11473,11474,11475,11476,11477,11478,11479,11480,11481,11482,11483,11484,11485,11486,11487,11488,11489,11490,11491,11492,11499,11500,11501,11502,11506,11507,11520,11521,11522,11523,11524,11525,11526,11527,11528,11529,11530,11531,11532,11533,11534,11535,11536,11537,11538,11539,11540,11541,11542,11543,11544,11545,11546,11547,11548,11549,11550,11551,11552,11553,11554,11555,11556,11557,11559,11565,11568,11569,11570,11571,11572,11573,11574,11575,11576,11577,11578,11579,11580,11581,11582,11583,11584,11585,11586,11587,11588,11589,11590,11591,11592,11593,11594,11595,11596,11597,11598,11599,11600,11601,11602,11603,11604,11605,11606,11607,11608,11609,11610,11611,11612,11613,11614,11615,11616,11617,11618,11619,11620,11621,11622,11623,11631,11648,11649,11650,11651,11652,11653,11654,11655,11656,11657,11658,11659,11660,11661,11662,11663,11664,11665,11666,11667,11668,11669,11670,11680,11681,11682,11683,11684,11685,11686,11688,11689,11690,11691,11692,11693,11694,11696,11697,11698,11699,11700,11701,11702,11704,11705,11706,11707,11708,11709,11710,11712,11713,11714,11715,11716,11717,11718,11720,11721,11722,11723,11724,11725,11726,11728,11729,11730,11731,11732,11733,11734,11736,11737,11738,11739,11740,11741,11742,12293,12294,12295,12321,12322,12323,12324,12325,12326,12327,12328,12329,12337,12338,12339,12340,12341,12344,12345,12346,12347,12348,12353,12354,12355,12356,12357,12358,12359,12360,12361,12362,12363,12364,12365,12366,12367,12368,12369,12370,12371,12372,12373,12374,12375,12376,12377,12378,12379,12380,12381,12382,12383,12384,12385,12386,12387,12388,12389,12390,12391,12392,12393,12394,12395,12396,12397,12398,12399,12400,12401,12402,12403,12404,12405,12406,12407,12408,12409,12410,12411,12412,12413,12414,12415,12416,12417,12418,12419,12420,12421,12422,12423,12424,12425,12426,12427,12428,12429,12430,12431,12432,12433,12434,12435,12436,12437,12438,12443,12444,12445,12446,12447,12449,12450,12451,12452,12453,12454,12455,12456,12457,12458,12459,12460,12461,12462,12463,12464,12465,12466,12467,12468,12469,12470,12471,12472,12473,12474,12475,12476,12477,12478,12479,12480,12481,12482,12483,12484,12485,12486,12487,12488,12489,12490,12491,12492,12493,12494,12495,12496,12497,12498,12499,12500,12501,12502,12503,12504,12505,12506,12507,12508,12509,12510,12511,12512,12513,12514,12515,12516,12517,12518,12519,12520,12521,12522,12523,12524,12525,12526,12527,12528,12529,12530,12531,12532,12533,12534,12535,12536,12537,12538,12540,12541,12542,12543,12549,12550,12551,12552,12553,12554,12555,12556,12557,12558,12559,12560,12561,12562,12563,12564,12565,12566,12567,12568,12569,12570,12571,12572,12573,12574,12575,12576,12577,12578,12579,12580,12581,12582,12583,12584,12585,12586,12587,12588,12589,12590,12591,12593,12594,12595,12596,12597,12598,12599,12600,12601,12602,12603,12604,12605,12606,12607,12608,12609,12610,12611,12612,12613,12614,12615,12616,12617,12618,12619,12620,12621,12622,12623,12624,12625,12626,12627,12628,12629,12630,12631,12632,12633,12634,12635,12636,12637,12638,12639,12640,12641,12642,12643,12644,12645,12646,12647,12648,12649,12650,12651,12652,12653,12654,12655,12656,12657,12658,12659,12660,12661,12662,12663,12664,12665,12666,12667,12668,12669,12670,12671,12672,12673,12674,12675,12676,12677,12678,12679,12680,12681,12682,12683,12684,12685,12686,12704,12705,12706,12707,12708,12709,12710,12711,12712,12713,12714,12715,12716,12717,12718,12719,12720,12721,12722,12723,12724,12725,12726,12727,12728,12729,12730,12784,12785,12786,12787,12788,12789,12790,12791,12792,12793,12794,12795,12796,12797,12798,12799,13312,13313,13314,13315,13316,13317,13318,13319,13320,13321,13322,13323,13324,13325,13326,13327,13328,13329,13330,13331,13332,13333,13334,13335,13336,13337,13338,13339,13340,13341,13342,13343,13344,13345,13346,13347,13348,13349,13350,13351,13352,13353,13354,13355,13356,13357,13358,13359,13360,13361,13362,13363,13364,13365,13366,13367,13368,13369,13370,13371,13372,13373,13374,13375,13376,13377,13378,13379,13380,13381,13382,13383,13384,13385,13386,13387,13388,13389,13390,13391,13392,13393,13394,13395,13396,13397,13398,13399,13400,13401,13402,13403,13404,13405,13406,13407,13408,13409,13410,13411,13412,13413,13414,13415,13416,13417,13418,13419,13420,13421,13422,13423,13424,13425,13426,13427,13428,13429,13430,13431,13432,13433,13434,13435,13436,13437,13438,13439,13440,13441,13442,13443,13444,13445,13446,13447,13448,13449,13450,13451,13452,13453,13454,13455,13456,13457,13458,13459,13460,13461,13462,13463,13464,13465,13466,13467,13468,13469,13470,13471,13472,13473,13474,13475,13476,13477,13478,13479,13480,13481,13482,13483,13484,13485,13486,13487,13488,13489,13490,13491,13492,13493,13494,13495,13496,13497,13498,13499,13500,13501,13502,13503,13504,13505,13506,13507,13508,13509,13510,13511,13512,13513,13514,13515,13516,13517,13518,13519,13520,13521,13522,13523,13524,13525,13526,13527,13528,13529,13530,13531,13532,13533,13534,13535,13536,13537,13538,13539,13540,13541,13542,13543,13544,13545,13546,13547,13548,13549,13550,13551,13552,13553,13554,13555,13556,13557,13558,13559,13560,13561,13562,13563,13564,13565,13566,13567,13568,13569,13570,13571,13572,13573,13574,13575,13576,13577,13578,13579,13580,13581,13582,13583,13584,13585,13586,13587,13588,13589,13590,13591,13592,13593,13594,13595,13596,13597,13598,13599,13600,13601,13602,13603,13604,13605,13606,13607,13608,13609,13610,13611,13612,13613,13614,13615,13616,13617,13618,13619,13620,13621,13622,13623,13624,13625,13626,13627,13628,13629,13630,13631,13632,13633,13634,13635,13636,13637,13638,13639,13640,13641,13642,13643,13644,13645,13646,13647,13648,13649,13650,13651,13652,13653,13654,13655,13656,13657,13658,13659,13660,13661,13662,13663,13664,13665,13666,13667,13668,13669,13670,13671,13672,13673,13674,13675,13676,13677,13678,13679,13680,13681,13682,13683,13684,13685,13686,13687,13688,13689,13690,13691,13692,13693,13694,13695,13696,13697,13698,13699,13700,13701,13702,13703,13704,13705,13706,13707,13708,13709,13710,13711,13712,13713,13714,13715,13716,13717,13718,13719,13720,13721,13722,13723,13724,13725,13726,13727,13728,13729,13730,13731,13732,13733,13734,13735,13736,13737,13738,13739,13740,13741,13742,13743,13744,13745,13746,13747,13748,13749,13750,13751,13752,13753,13754,13755,13756,13757,13758,13759,13760,13761,13762,13763,13764,13765,13766,13767,13768,13769,13770,13771,13772,13773,13774,13775,13776,13777,13778,13779,13780,13781,13782,13783,13784,13785,13786,13787,13788,13789,13790,13791,13792,13793,13794,13795,13796,13797,13798,13799,13800,13801,13802,13803,13804,13805,13806,13807,13808,13809,13810,13811,13812,13813,13814,13815,13816,13817,13818,13819,13820,13821,13822,13823,13824,13825,13826,13827,13828,13829,13830,13831,13832,13833,13834,13835,13836,13837,13838,13839,13840,13841,13842,13843,13844,13845,13846,13847,13848,13849,13850,13851,13852,13853,13854,13855,13856,13857,13858,13859,13860,13861,13862,13863,13864,13865,13866,13867,13868,13869,13870,13871,13872,13873,13874,13875,13876,13877,13878,13879,13880,13881,13882,13883,13884,13885,13886,13887,13888,13889,13890,13891,13892,13893,13894,13895,13896,13897,13898,13899,13900,13901,13902,13903,13904,13905,13906,13907,13908,13909,13910,13911,13912,13913,13914,13915,13916,13917,13918,13919,13920,13921,13922,13923,13924,13925,13926,13927,13928,13929,13930,13931,13932,13933,13934,13935,13936,13937,13938,13939,13940,13941,13942,13943,13944,13945,13946,13947,13948,13949,13950,13951,13952,13953,13954,13955,13956,13957,13958,13959,13960,13961,13962,13963,13964,13965,13966,13967,13968,13969,13970,13971,13972,13973,13974,13975,13976,13977,13978,13979,13980,13981,13982,13983,13984,13985,13986,13987,13988,13989,13990,13991,13992,13993,13994,13995,13996,13997,13998,13999,14000,14001,14002,14003,14004,14005,14006,14007,14008,14009,14010,14011,14012,14013,14014,14015,14016,14017,14018,14019,14020,14021,14022,14023,14024,14025,14026,14027,14028,14029,14030,14031,14032,14033,14034,14035,14036,14037,14038,14039,14040,14041,14042,14043,14044,14045,14046,14047,14048,14049,14050,14051,14052,14053,14054,14055,14056,14057,14058,14059,14060,14061,14062,14063,14064,14065,14066,14067,14068,14069,14070,14071,14072,14073,14074,14075,14076,14077,14078,14079,14080,14081,14082,14083,14084,14085,14086,14087,14088,14089,14090,14091,14092,14093,14094,14095,14096,14097,14098,14099,14100,14101,14102,14103,14104,14105,14106,14107,14108,14109,14110,14111,14112,14113,14114,14115,14116,14117,14118,14119,14120,14121,14122,14123,14124,14125,14126,14127,14128,14129,14130,14131,14132,14133,14134,14135,14136,14137,14138,14139,14140,14141,14142,14143,14144,14145,14146,14147,14148,14149,14150,14151,14152,14153,14154,14155,14156,14157,14158,14159,14160,14161,14162,14163,14164,14165,14166,14167,14168,14169,14170,14171,14172,14173,14174,14175,14176,14177,14178,14179,14180,14181,14182,14183,14184,14185,14186,14187,14188,14189,14190,14191,14192,14193,14194,14195,14196,14197,14198,14199,14200,14201,14202,14203,14204,14205,14206,14207,14208,14209,14210,14211,14212,14213,14214,14215,14216,14217,14218,14219,14220,14221,14222,14223,14224,14225,14226,14227,14228,14229,14230,14231,14232,14233,14234,14235,14236,14237,14238,14239,14240,14241,14242,14243,14244,14245,14246,14247,14248,14249,14250,14251,14252,14253,14254,14255,14256,14257,14258,14259,14260,14261,14262,14263,14264,14265,14266,14267,14268,14269,14270,14271,14272,14273,14274,14275,14276,14277,14278,14279,14280,14281,14282,14283,14284,14285,14286,14287,14288,14289,14290,14291,14292,14293,14294,14295,14296,14297,14298,14299,14300,14301,14302,14303,14304,14305,14306,14307,14308,14309,14310,14311,14312,14313,14314,14315,14316,14317,14318,14319,14320,14321,14322,14323,14324,14325,14326,14327,14328,14329,14330,14331,14332,14333,14334,14335,14336,14337,14338,14339,14340,14341,14342,14343,14344,14345,14346,14347,14348,14349,14350,14351,14352,14353,14354,14355,14356,14357,14358,14359,14360,14361,14362,14363,14364,14365,14366,14367,14368,14369,14370,14371,14372,14373,14374,14375,14376,14377,14378,14379,14380,14381,14382,14383,14384,14385,14386,14387,14388,14389,14390,14391,14392,14393,14394,14395,14396,14397,14398,14399,14400,14401,14402,14403,14404,14405,14406,14407,14408,14409,14410,14411,14412,14413,14414,14415,14416,14417,14418,14419,14420,14421,14422,14423,14424,14425,14426,14427,14428,14429,14430,14431,14432,14433,14434,14435,14436,14437,14438,14439,14440,14441,14442,14443,14444,14445,14446,14447,14448,14449,14450,14451,14452,14453,14454,14455,14456,14457,14458,14459,14460,14461,14462,14463,14464,14465,14466,14467,14468,14469,14470,14471,14472,14473,14474,14475,14476,14477,14478,14479,14480,14481,14482,14483,14484,14485,14486,14487,14488,14489,14490,14491,14492,14493,14494,14495,14496,14497,14498,14499,14500,14501,14502,14503,14504,14505,14506,14507,14508,14509,14510,14511,14512,14513,14514,14515,14516,14517,14518,14519,14520,14521,14522,14523,14524,14525,14526,14527,14528,14529,14530,14531,14532,14533,14534,14535,14536,14537,14538,14539,14540,14541,14542,14543,14544,14545,14546,14547,14548,14549,14550,14551,14552,14553,14554,14555,14556,14557,14558,14559,14560,14561,14562,14563,14564,14565,14566,14567,14568,14569,14570,14571,14572,14573,14574,14575,14576,14577,14578,14579,14580,14581,14582,14583,14584,14585,14586,14587,14588,14589,14590,14591,14592,14593,14594,14595,14596,14597,14598,14599,14600,14601,14602,14603,14604,14605,14606,14607,14608,14609,14610,14611,14612,14613,14614,14615,14616,14617,14618,14619,14620,14621,14622,14623,14624,14625,14626,14627,14628,14629,14630,14631,14632,14633,14634,14635,14636,14637,14638,14639,14640,14641,14642,14643,14644,14645,14646,14647,14648,14649,14650,14651,14652,14653,14654,14655,14656,14657,14658,14659,14660,14661,14662,14663,14664,14665,14666,14667,14668,14669,14670,14671,14672,14673,14674,14675,14676,14677,14678,14679,14680,14681,14682,14683,14684,14685,14686,14687,14688,14689,14690,14691,14692,14693,14694,14695,14696,14697,14698,14699,14700,14701,14702,14703,14704,14705,14706,14707,14708,14709,14710,14711,14712,14713,14714,14715,14716,14717,14718,14719,14720,14721,14722,14723,14724,14725,14726,14727,14728,14729,14730,14731,14732,14733,14734,14735,14736,14737,14738,14739,14740,14741,14742,14743,14744,14745,14746,14747,14748,14749,14750,14751,14752,14753,14754,14755,14756,14757,14758,14759,14760,14761,14762,14763,14764,14765,14766,14767,14768,14769,14770,14771,14772,14773,14774,14775,14776,14777,14778,14779,14780,14781,14782,14783,14784,14785,14786,14787,14788,14789,14790,14791,14792,14793,14794,14795,14796,14797,14798,14799,14800,14801,14802,14803,14804,14805,14806,14807,14808,14809,14810,14811,14812,14813,14814,14815,14816,14817,14818,14819,14820,14821,14822,14823,14824,14825,14826,14827,14828,14829,14830,14831,14832,14833,14834,14835,14836,14837,14838,14839,14840,14841,14842,14843,14844,14845,14846,14847,14848,14849,14850,14851,14852,14853,14854,14855,14856,14857,14858,14859,14860,14861,14862,14863,14864,14865,14866,14867,14868,14869,14870,14871,14872,14873,14874,14875,14876,14877,14878,14879,14880,14881,14882,14883,14884,14885,14886,14887,14888,14889,14890,14891,14892,14893,14894,14895,14896,14897,14898,14899,14900,14901,14902,14903,14904,14905,14906,14907,14908,14909,14910,14911,14912,14913,14914,14915,14916,14917,14918,14919,14920,14921,14922,14923,14924,14925,14926,14927,14928,14929,14930,14931,14932,14933,14934,14935,14936,14937,14938,14939,14940,14941,14942,14943,14944,14945,14946,14947,14948,14949,14950,14951,14952,14953,14954,14955,14956,14957,14958,14959,14960,14961,14962,14963,14964,14965,14966,14967,14968,14969,14970,14971,14972,14973,14974,14975,14976,14977,14978,14979,14980,14981,14982,14983,14984,14985,14986,14987,14988,14989,14990,14991,14992,14993,14994,14995,14996,14997,14998,14999,15000,15001,15002,15003,15004,15005,15006,15007,15008,15009,15010,15011,15012,15013,15014,15015,15016,15017,15018,15019,15020,15021,15022,15023,15024,15025,15026,15027,15028,15029,15030,15031,15032,15033,15034,15035,15036,15037,15038,15039,15040,15041,15042,15043,15044,15045,15046,15047,15048,15049,15050,15051,15052,15053,15054,15055,15056,15057,15058,15059,15060,15061,15062,15063,15064,15065,15066,15067,15068,15069,15070,15071,15072,15073,15074,15075,15076,15077,15078,15079,15080,15081,15082,15083,15084,15085,15086,15087,15088,15089,15090,15091,15092,15093,15094,15095,15096,15097,15098,15099,15100,15101,15102,15103,15104,15105,15106,15107,15108,15109,15110,15111,15112,15113,15114,15115,15116,15117,15118,15119,15120,15121,15122,15123,15124,15125,15126,15127,15128,15129,15130,15131,15132,15133,15134,15135,15136,15137,15138,15139,15140,15141,15142,15143,15144,15145,15146,15147,15148,15149,15150,15151,15152,15153,15154,15155,15156,15157,15158,15159,15160,15161,15162,15163,15164,15165,15166,15167,15168,15169,15170,15171,15172,15173,15174,15175,15176,15177,15178,15179,15180,15181,15182,15183,15184,15185,15186,15187,15188,15189,15190,15191,15192,15193,15194,15195,15196,15197,15198,15199,15200,15201,15202,15203,15204,15205,15206,15207,15208,15209,15210,15211,15212,15213,15214,15215,15216,15217,15218,15219,15220,15221,15222,15223,15224,15225,15226,15227,15228,15229,15230,15231,15232,15233,15234,15235,15236,15237,15238,15239,15240,15241,15242,15243,15244,15245,15246,15247,15248,15249,15250,15251,15252,15253,15254,15255,15256,15257,15258,15259,15260,15261,15262,15263,15264,15265,15266,15267,15268,15269,15270,15271,15272,15273,15274,15275,15276,15277,15278,15279,15280,15281,15282,15283,15284,15285,15286,15287,15288,15289,15290,15291,15292,15293,15294,15295,15296,15297,15298,15299,15300,15301,15302,15303,15304,15305,15306,15307,15308,15309,15310,15311,15312,15313,15314,15315,15316,15317,15318,15319,15320,15321,15322,15323,15324,15325,15326,15327,15328,15329,15330,15331,15332,15333,15334,15335,15336,15337,15338,15339,15340,15341,15342,15343,15344,15345,15346,15347,15348,15349,15350,15351,15352,15353,15354,15355,15356,15357,15358,15359,15360,15361,15362,15363,15364,15365,15366,15367,15368,15369,15370,15371,15372,15373,15374,15375,15376,15377,15378,15379,15380,15381,15382,15383,15384,15385,15386,15387,15388,15389,15390,15391,15392,15393,15394,15395,15396,15397,15398,15399,15400,15401,15402,15403,15404,15405,15406,15407,15408,15409,15410,15411,15412,15413,15414,15415,15416,15417,15418,15419,15420,15421,15422,15423,15424,15425,15426,15427,15428,15429,15430,15431,15432,15433,15434,15435,15436,15437,15438,15439,15440,15441,15442,15443,15444,15445,15446,15447,15448,15449,15450,15451,15452,15453,15454,15455,15456,15457,15458,15459,15460,15461,15462,15463,15464,15465,15466,15467,15468,15469,15470,15471,15472,15473,15474,15475,15476,15477,15478,15479,15480,15481,15482,15483,15484,15485,15486,15487,15488,15489,15490,15491,15492,15493,15494,15495,15496,15497,15498,15499,15500,15501,15502,15503,15504,15505,15506,15507,15508,15509,15510,15511,15512,15513,15514,15515,15516,15517,15518,15519,15520,15521,15522,15523,15524,15525,15526,15527,15528,15529,15530,15531,15532,15533,15534,15535,15536,15537,15538,15539,15540,15541,15542,15543,15544,15545,15546,15547,15548,15549,15550,15551,15552,15553,15554,15555,15556,15557,15558,15559,15560,15561,15562,15563,15564,15565,15566,15567,15568,15569,15570,15571,15572,15573,15574,15575,15576,15577,15578,15579,15580,15581,15582,15583,15584,15585,15586,15587,15588,15589,15590,15591,15592,15593,15594,15595,15596,15597,15598,15599,15600,15601,15602,15603,15604,15605,15606,15607,15608,15609,15610,15611,15612,15613,15614,15615,15616,15617,15618,15619,15620,15621,15622,15623,15624,15625,15626,15627,15628,15629,15630,15631,15632,15633,15634,15635,15636,15637,15638,15639,15640,15641,15642,15643,15644,15645,15646,15647,15648,15649,15650,15651,15652,15653,15654,15655,15656,15657,15658,15659,15660,15661,15662,15663,15664,15665,15666,15667,15668,15669,15670,15671,15672,15673,15674,15675,15676,15677,15678,15679,15680,15681,15682,15683,15684,15685,15686,15687,15688,15689,15690,15691,15692,15693,15694,15695,15696,15697,15698,15699,15700,15701,15702,15703,15704,15705,15706,15707,15708,15709,15710,15711,15712,15713,15714,15715,15716,15717,15718,15719,15720,15721,15722,15723,15724,15725,15726,15727,15728,15729,15730,15731,15732,15733,15734,15735,15736,15737,15738,15739,15740,15741,15742,15743,15744,15745,15746,15747,15748,15749,15750,15751,15752,15753,15754,15755,15756,15757,15758,15759,15760,15761,15762,15763,15764,15765,15766,15767,15768,15769,15770,15771,15772,15773,15774,15775,15776,15777,15778,15779,15780,15781,15782,15783,15784,15785,15786,15787,15788,15789,15790,15791,15792,15793,15794,15795,15796,15797,15798,15799,15800,15801,15802,15803,15804,15805,15806,15807,15808,15809,15810,15811,15812,15813,15814,15815,15816,15817,15818,15819,15820,15821,15822,15823,15824,15825,15826,15827,15828,15829,15830,15831,15832,15833,15834,15835,15836,15837,15838,15839,15840,15841,15842,15843,15844,15845,15846,15847,15848,15849,15850,15851,15852,15853,15854,15855,15856,15857,15858,15859,15860,15861,15862,15863,15864,15865,15866,15867,15868,15869,15870,15871,15872,15873,15874,15875,15876,15877,15878,15879,15880,15881,15882,15883,15884,15885,15886,15887,15888,15889,15890,15891,15892,15893,15894,15895,15896,15897,15898,15899,15900,15901,15902,15903,15904,15905,15906,15907,15908,15909,15910,15911,15912,15913,15914,15915,15916,15917,15918,15919,15920,15921,15922,15923,15924,15925,15926,15927,15928,15929,15930,15931,15932,15933,15934,15935,15936,15937,15938,15939,15940,15941,15942,15943,15944,15945,15946,15947,15948,15949,15950,15951,15952,15953,15954,15955,15956,15957,15958,15959,15960,15961,15962,15963,15964,15965,15966,15967,15968,15969,15970,15971,15972,15973,15974,15975,15976,15977,15978,15979,15980,15981,15982,15983,15984,15985,15986,15987,15988,15989,15990,15991,15992,15993,15994,15995,15996,15997,15998,15999,16000,16001,16002,16003,16004,16005,16006,16007,16008,16009,16010,16011,16012,16013,16014,16015,16016,16017,16018,16019,16020,16021,16022,16023,16024,16025,16026,16027,16028,16029,16030,16031,16032,16033,16034,16035,16036,16037,16038,16039,16040,16041,16042,16043,16044,16045,16046,16047,16048,16049,16050,16051,16052,16053,16054,16055,16056,16057,16058,16059,16060,16061,16062,16063,16064,16065,16066,16067,16068,16069,16070,16071,16072,16073,16074,16075,16076,16077,16078,16079,16080,16081,16082,16083,16084,16085,16086,16087,16088,16089,16090,16091,16092,16093,16094,16095,16096,16097,16098,16099,16100,16101,16102,16103,16104,16105,16106,16107,16108,16109,16110,16111,16112,16113,16114,16115,16116,16117,16118,16119,16120,16121,16122,16123,16124,16125,16126,16127,16128,16129,16130,16131,16132,16133,16134,16135,16136,16137,16138,16139,16140,16141,16142,16143,16144,16145,16146,16147,16148,16149,16150,16151,16152,16153,16154,16155,16156,16157,16158,16159,16160,16161,16162,16163,16164,16165,16166,16167,16168,16169,16170,16171,16172,16173,16174,16175,16176,16177,16178,16179,16180,16181,16182,16183,16184,16185,16186,16187,16188,16189,16190,16191,16192,16193,16194,16195,16196,16197,16198,16199,16200,16201,16202,16203,16204,16205,16206,16207,16208,16209,16210,16211,16212,16213,16214,16215,16216,16217,16218,16219,16220,16221,16222,16223,16224,16225,16226,16227,16228,16229,16230,16231,16232,16233,16234,16235,16236,16237,16238,16239,16240,16241,16242,16243,16244,16245,16246,16247,16248,16249,16250,16251,16252,16253,16254,16255,16256,16257,16258,16259,16260,16261,16262,16263,16264,16265,16266,16267,16268,16269,16270,16271,16272,16273,16274,16275,16276,16277,16278,16279,16280,16281,16282,16283,16284,16285,16286,16287,16288,16289,16290,16291,16292,16293,16294,16295,16296,16297,16298,16299,16300,16301,16302,16303,16304,16305,16306,16307,16308,16309,16310,16311,16312,16313,16314,16315,16316,16317,16318,16319,16320,16321,16322,16323,16324,16325,16326,16327,16328,16329,16330,16331,16332,16333,16334,16335,16336,16337,16338,16339,16340,16341,16342,16343,16344,16345,16346,16347,16348,16349,16350,16351,16352,16353,16354,16355,16356,16357,16358,16359,16360,16361,16362,16363,16364,16365,16366,16367,16368,16369,16370,16371,16372,16373,16374,16375,16376,16377,16378,16379,16380,16381,16382,16383,16384,16385,16386,16387,16388,16389,16390,16391,16392,16393,16394,16395,16396,16397,16398,16399,16400,16401,16402,16403,16404,16405,16406,16407,16408,16409,16410,16411,16412,16413,16414,16415,16416,16417,16418,16419,16420,16421,16422,16423,16424,16425,16426,16427,16428,16429,16430,16431,16432,16433,16434,16435,16436,16437,16438,16439,16440,16441,16442,16443,16444,16445,16446,16447,16448,16449,16450,16451,16452,16453,16454,16455,16456,16457,16458,16459,16460,16461,16462,16463,16464,16465,16466,16467,16468,16469,16470,16471,16472,16473,16474,16475,16476,16477,16478,16479,16480,16481,16482,16483,16484,16485,16486,16487,16488,16489,16490,16491,16492,16493,16494,16495,16496,16497,16498,16499,16500,16501,16502,16503,16504,16505,16506,16507,16508,16509,16510,16511,16512,16513,16514,16515,16516,16517,16518,16519,16520,16521,16522,16523,16524,16525,16526,16527,16528,16529,16530,16531,16532,16533,16534,16535,16536,16537,16538,16539,16540,16541,16542,16543,16544,16545,16546,16547,16548,16549,16550,16551,16552,16553,16554,16555,16556,16557,16558,16559,16560,16561,16562,16563,16564,16565,16566,16567,16568,16569,16570,16571,16572,16573,16574,16575,16576,16577,16578,16579,16580,16581,16582,16583,16584,16585,16586,16587,16588,16589,16590,16591,16592,16593,16594,16595,16596,16597,16598,16599,16600,16601,16602,16603,16604,16605,16606,16607,16608,16609,16610,16611,16612,16613,16614,16615,16616,16617,16618,16619,16620,16621,16622,16623,16624,16625,16626,16627,16628,16629,16630,16631,16632,16633,16634,16635,16636,16637,16638,16639,16640,16641,16642,16643,16644,16645,16646,16647,16648,16649,16650,16651,16652,16653,16654,16655,16656,16657,16658,16659,16660,16661,16662,16663,16664,16665,16666,16667,16668,16669,16670,16671,16672,16673,16674,16675,16676,16677,16678,16679,16680,16681,16682,16683,16684,16685,16686,16687,16688,16689,16690,16691,16692,16693,16694,16695,16696,16697,16698,16699,16700,16701,16702,16703,16704,16705,16706,16707,16708,16709,16710,16711,16712,16713,16714,16715,16716,16717,16718,16719,16720,16721,16722,16723,16724,16725,16726,16727,16728,16729,16730,16731,16732,16733,16734,16735,16736,16737,16738,16739,16740,16741,16742,16743,16744,16745,16746,16747,16748,16749,16750,16751,16752,16753,16754,16755,16756,16757,16758,16759,16760,16761,16762,16763,16764,16765,16766,16767,16768,16769,16770,16771,16772,16773,16774,16775,16776,16777,16778,16779,16780,16781,16782,16783,16784,16785,16786,16787,16788,16789,16790,16791,16792,16793,16794,16795,16796,16797,16798,16799,16800,16801,16802,16803,16804,16805,16806,16807,16808,16809,16810,16811,16812,16813,16814,16815,16816,16817,16818,16819,16820,16821,16822,16823,16824,16825,16826,16827,16828,16829,16830,16831,16832,16833,16834,16835,16836,16837,16838,16839,16840,16841,16842,16843,16844,16845,16846,16847,16848,16849,16850,16851,16852,16853,16854,16855,16856,16857,16858,16859,16860,16861,16862,16863,16864,16865,16866,16867,16868,16869,16870,16871,16872,16873,16874,16875,16876,16877,16878,16879,16880,16881,16882,16883,16884,16885,16886,16887,16888,16889,16890,16891,16892,16893,16894,16895,16896,16897,16898,16899,16900,16901,16902,16903,16904,16905,16906,16907,16908,16909,16910,16911,16912,16913,16914,16915,16916,16917,16918,16919,16920,16921,16922,16923,16924,16925,16926,16927,16928,16929,16930,16931,16932,16933,16934,16935,16936,16937,16938,16939,16940,16941,16942,16943,16944,16945,16946,16947,16948,16949,16950,16951,16952,16953,16954,16955,16956,16957,16958,16959,16960,16961,16962,16963,16964,16965,16966,16967,16968,16969,16970,16971,16972,16973,16974,16975,16976,16977,16978,16979,16980,16981,16982,16983,16984,16985,16986,16987,16988,16989,16990,16991,16992,16993,16994,16995,16996,16997,16998,16999,17000,17001,17002,17003,17004,17005,17006,17007,17008,17009,17010,17011,17012,17013,17014,17015,17016,17017,17018,17019,17020,17021,17022,17023,17024,17025,17026,17027,17028,17029,17030,17031,17032,17033,17034,17035,17036,17037,17038,17039,17040,17041,17042,17043,17044,17045,17046,17047,17048,17049,17050,17051,17052,17053,17054,17055,17056,17057,17058,17059,17060,17061,17062,17063,17064,17065,17066,17067,17068,17069,17070,17071,17072,17073,17074,17075,17076,17077,17078,17079,17080,17081,17082,17083,17084,17085,17086,17087,17088,17089,17090,17091,17092,17093,17094,17095,17096,17097,17098,17099,17100,17101,17102,17103,17104,17105,17106,17107,17108,17109,17110,17111,17112,17113,17114,17115,17116,17117,17118,17119,17120,17121,17122,17123,17124,17125,17126,17127,17128,17129,17130,17131,17132,17133,17134,17135,17136,17137,17138,17139,17140,17141,17142,17143,17144,17145,17146,17147,17148,17149,17150,17151,17152,17153,17154,17155,17156,17157,17158,17159,17160,17161,17162,17163,17164,17165,17166,17167,17168,17169,17170,17171,17172,17173,17174,17175,17176,17177,17178,17179,17180,17181,17182,17183,17184,17185,17186,17187,17188,17189,17190,17191,17192,17193,17194,17195,17196,17197,17198,17199,17200,17201,17202,17203,17204,17205,17206,17207,17208,17209,17210,17211,17212,17213,17214,17215,17216,17217,17218,17219,17220,17221,17222,17223,17224,17225,17226,17227,17228,17229,17230,17231,17232,17233,17234,17235,17236,17237,17238,17239,17240,17241,17242,17243,17244,17245,17246,17247,17248,17249,17250,17251,17252,17253,17254,17255,17256,17257,17258,17259,17260,17261,17262,17263,17264,17265,17266,17267,17268,17269,17270,17271,17272,17273,17274,17275,17276,17277,17278,17279,17280,17281,17282,17283,17284,17285,17286,17287,17288,17289,17290,17291,17292,17293,17294,17295,17296,17297,17298,17299,17300,17301,17302,17303,17304,17305,17306,17307,17308,17309,17310,17311,17312,17313,17314,17315,17316,17317,17318,17319,17320,17321,17322,17323,17324,17325,17326,17327,17328,17329,17330,17331,17332,17333,17334,17335,17336,17337,17338,17339,17340,17341,17342,17343,17344,17345,17346,17347,17348,17349,17350,17351,17352,17353,17354,17355,17356,17357,17358,17359,17360,17361,17362,17363,17364,17365,17366,17367,17368,17369,17370,17371,17372,17373,17374,17375,17376,17377,17378,17379,17380,17381,17382,17383,17384,17385,17386,17387,17388,17389,17390,17391,17392,17393,17394,17395,17396,17397,17398,17399,17400,17401,17402,17403,17404,17405,17406,17407,17408,17409,17410,17411,17412,17413,17414,17415,17416,17417,17418,17419,17420,17421,17422,17423,17424,17425,17426,17427,17428,17429,17430,17431,17432,17433,17434,17435,17436,17437,17438,17439,17440,17441,17442,17443,17444,17445,17446,17447,17448,17449,17450,17451,17452,17453,17454,17455,17456,17457,17458,17459,17460,17461,17462,17463,17464,17465,17466,17467,17468,17469,17470,17471,17472,17473,17474,17475,17476,17477,17478,17479,17480,17481,17482,17483,17484,17485,17486,17487,17488,17489,17490,17491,17492,17493,17494,17495,17496,17497,17498,17499,17500,17501,17502,17503,17504,17505,17506,17507,17508,17509,17510,17511,17512,17513,17514,17515,17516,17517,17518,17519,17520,17521,17522,17523,17524,17525,17526,17527,17528,17529,17530,17531,17532,17533,17534,17535,17536,17537,17538,17539,17540,17541,17542,17543,17544,17545,17546,17547,17548,17549,17550,17551,17552,17553,17554,17555,17556,17557,17558,17559,17560,17561,17562,17563,17564,17565,17566,17567,17568,17569,17570,17571,17572,17573,17574,17575,17576,17577,17578,17579,17580,17581,17582,17583,17584,17585,17586,17587,17588,17589,17590,17591,17592,17593,17594,17595,17596,17597,17598,17599,17600,17601,17602,17603,17604,17605,17606,17607,17608,17609,17610,17611,17612,17613,17614,17615,17616,17617,17618,17619,17620,17621,17622,17623,17624,17625,17626,17627,17628,17629,17630,17631,17632,17633,17634,17635,17636,17637,17638,17639,17640,17641,17642,17643,17644,17645,17646,17647,17648,17649,17650,17651,17652,17653,17654,17655,17656,17657,17658,17659,17660,17661,17662,17663,17664,17665,17666,17667,17668,17669,17670,17671,17672,17673,17674,17675,17676,17677,17678,17679,17680,17681,17682,17683,17684,17685,17686,17687,17688,17689,17690,17691,17692,17693,17694,17695,17696,17697,17698,17699,17700,17701,17702,17703,17704,17705,17706,17707,17708,17709,17710,17711,17712,17713,17714,17715,17716,17717,17718,17719,17720,17721,17722,17723,17724,17725,17726,17727,17728,17729,17730,17731,17732,17733,17734,17735,17736,17737,17738,17739,17740,17741,17742,17743,17744,17745,17746,17747,17748,17749,17750,17751,17752,17753,17754,17755,17756,17757,17758,17759,17760,17761,17762,17763,17764,17765,17766,17767,17768,17769,17770,17771,17772,17773,17774,17775,17776,17777,17778,17779,17780,17781,17782,17783,17784,17785,17786,17787,17788,17789,17790,17791,17792,17793,17794,17795,17796,17797,17798,17799,17800,17801,17802,17803,17804,17805,17806,17807,17808,17809,17810,17811,17812,17813,17814,17815,17816,17817,17818,17819,17820,17821,17822,17823,17824,17825,17826,17827,17828,17829,17830,17831,17832,17833,17834,17835,17836,17837,17838,17839,17840,17841,17842,17843,17844,17845,17846,17847,17848,17849,17850,17851,17852,17853,17854,17855,17856,17857,17858,17859,17860,17861,17862,17863,17864,17865,17866,17867,17868,17869,17870,17871,17872,17873,17874,17875,17876,17877,17878,17879,17880,17881,17882,17883,17884,17885,17886,17887,17888,17889,17890,17891,17892,17893,17894,17895,17896,17897,17898,17899,17900,17901,17902,17903,17904,17905,17906,17907,17908,17909,17910,17911,17912,17913,17914,17915,17916,17917,17918,17919,17920,17921,17922,17923,17924,17925,17926,17927,17928,17929,17930,17931,17932,17933,17934,17935,17936,17937,17938,17939,17940,17941,17942,17943,17944,17945,17946,17947,17948,17949,17950,17951,17952,17953,17954,17955,17956,17957,17958,17959,17960,17961,17962,17963,17964,17965,17966,17967,17968,17969,17970,17971,17972,17973,17974,17975,17976,17977,17978,17979,17980,17981,17982,17983,17984,17985,17986,17987,17988,17989,17990,17991,17992,17993,17994,17995,17996,17997,17998,17999,18000,18001,18002,18003,18004,18005,18006,18007,18008,18009,18010,18011,18012,18013,18014,18015,18016,18017,18018,18019,18020,18021,18022,18023,18024,18025,18026,18027,18028,18029,18030,18031,18032,18033,18034,18035,18036,18037,18038,18039,18040,18041,18042,18043,18044,18045,18046,18047,18048,18049,18050,18051,18052,18053,18054,18055,18056,18057,18058,18059,18060,18061,18062,18063,18064,18065,18066,18067,18068,18069,18070,18071,18072,18073,18074,18075,18076,18077,18078,18079,18080,18081,18082,18083,18084,18085,18086,18087,18088,18089,18090,18091,18092,18093,18094,18095,18096,18097,18098,18099,18100,18101,18102,18103,18104,18105,18106,18107,18108,18109,18110,18111,18112,18113,18114,18115,18116,18117,18118,18119,18120,18121,18122,18123,18124,18125,18126,18127,18128,18129,18130,18131,18132,18133,18134,18135,18136,18137,18138,18139,18140,18141,18142,18143,18144,18145,18146,18147,18148,18149,18150,18151,18152,18153,18154,18155,18156,18157,18158,18159,18160,18161,18162,18163,18164,18165,18166,18167,18168,18169,18170,18171,18172,18173,18174,18175,18176,18177,18178,18179,18180,18181,18182,18183,18184,18185,18186,18187,18188,18189,18190,18191,18192,18193,18194,18195,18196,18197,18198,18199,18200,18201,18202,18203,18204,18205,18206,18207,18208,18209,18210,18211,18212,18213,18214,18215,18216,18217,18218,18219,18220,18221,18222,18223,18224,18225,18226,18227,18228,18229,18230,18231,18232,18233,18234,18235,18236,18237,18238,18239,18240,18241,18242,18243,18244,18245,18246,18247,18248,18249,18250,18251,18252,18253,18254,18255,18256,18257,18258,18259,18260,18261,18262,18263,18264,18265,18266,18267,18268,18269,18270,18271,18272,18273,18274,18275,18276,18277,18278,18279,18280,18281,18282,18283,18284,18285,18286,18287,18288,18289,18290,18291,18292,18293,18294,18295,18296,18297,18298,18299,18300,18301,18302,18303,18304,18305,18306,18307,18308,18309,18310,18311,18312,18313,18314,18315,18316,18317,18318,18319,18320,18321,18322,18323,18324,18325,18326,18327,18328,18329,18330,18331,18332,18333,18334,18335,18336,18337,18338,18339,18340,18341,18342,18343,18344,18345,18346,18347,18348,18349,18350,18351,18352,18353,18354,18355,18356,18357,18358,18359,18360,18361,18362,18363,18364,18365,18366,18367,18368,18369,18370,18371,18372,18373,18374,18375,18376,18377,18378,18379,18380,18381,18382,18383,18384,18385,18386,18387,18388,18389,18390,18391,18392,18393,18394,18395,18396,18397,18398,18399,18400,18401,18402,18403,18404,18405,18406,18407,18408,18409,18410,18411,18412,18413,18414,18415,18416,18417,18418,18419,18420,18421,18422,18423,18424,18425,18426,18427,18428,18429,18430,18431,18432,18433,18434,18435,18436,18437,18438,18439,18440,18441,18442,18443,18444,18445,18446,18447,18448,18449,18450,18451,18452,18453,18454,18455,18456,18457,18458,18459,18460,18461,18462,18463,18464,18465,18466,18467,18468,18469,18470,18471,18472,18473,18474,18475,18476,18477,18478,18479,18480,18481,18482,18483,18484,18485,18486,18487,18488,18489,18490,18491,18492,18493,18494,18495,18496,18497,18498,18499,18500,18501,18502,18503,18504,18505,18506,18507,18508,18509,18510,18511,18512,18513,18514,18515,18516,18517,18518,18519,18520,18521,18522,18523,18524,18525,18526,18527,18528,18529,18530,18531,18532,18533,18534,18535,18536,18537,18538,18539,18540,18541,18542,18543,18544,18545,18546,18547,18548,18549,18550,18551,18552,18553,18554,18555,18556,18557,18558,18559,18560,18561,18562,18563,18564,18565,18566,18567,18568,18569,18570,18571,18572,18573,18574,18575,18576,18577,18578,18579,18580,18581,18582,18583,18584,18585,18586,18587,18588,18589,18590,18591,18592,18593,18594,18595,18596,18597,18598,18599,18600,18601,18602,18603,18604,18605,18606,18607,18608,18609,18610,18611,18612,18613,18614,18615,18616,18617,18618,18619,18620,18621,18622,18623,18624,18625,18626,18627,18628,18629,18630,18631,18632,18633,18634,18635,18636,18637,18638,18639,18640,18641,18642,18643,18644,18645,18646,18647,18648,18649,18650,18651,18652,18653,18654,18655,18656,18657,18658,18659,18660,18661,18662,18663,18664,18665,18666,18667,18668,18669,18670,18671,18672,18673,18674,18675,18676,18677,18678,18679,18680,18681,18682,18683,18684,18685,18686,18687,18688,18689,18690,18691,18692,18693,18694,18695,18696,18697,18698,18699,18700,18701,18702,18703,18704,18705,18706,18707,18708,18709,18710,18711,18712,18713,18714,18715,18716,18717,18718,18719,18720,18721,18722,18723,18724,18725,18726,18727,18728,18729,18730,18731,18732,18733,18734,18735,18736,18737,18738,18739,18740,18741,18742,18743,18744,18745,18746,18747,18748,18749,18750,18751,18752,18753,18754,18755,18756,18757,18758,18759,18760,18761,18762,18763,18764,18765,18766,18767,18768,18769,18770,18771,18772,18773,18774,18775,18776,18777,18778,18779,18780,18781,18782,18783,18784,18785,18786,18787,18788,18789,18790,18791,18792,18793,18794,18795,18796,18797,18798,18799,18800,18801,18802,18803,18804,18805,18806,18807,18808,18809,18810,18811,18812,18813,18814,18815,18816,18817,18818,18819,18820,18821,18822,18823,18824,18825,18826,18827,18828,18829,18830,18831,18832,18833,18834,18835,18836,18837,18838,18839,18840,18841,18842,18843,18844,18845,18846,18847,18848,18849,18850,18851,18852,18853,18854,18855,18856,18857,18858,18859,18860,18861,18862,18863,18864,18865,18866,18867,18868,18869,18870,18871,18872,18873,18874,18875,18876,18877,18878,18879,18880,18881,18882,18883,18884,18885,18886,18887,18888,18889,18890,18891,18892,18893,18894,18895,18896,18897,18898,18899,18900,18901,18902,18903,18904,18905,18906,18907,18908,18909,18910,18911,18912,18913,18914,18915,18916,18917,18918,18919,18920,18921,18922,18923,18924,18925,18926,18927,18928,18929,18930,18931,18932,18933,18934,18935,18936,18937,18938,18939,18940,18941,18942,18943,18944,18945,18946,18947,18948,18949,18950,18951,18952,18953,18954,18955,18956,18957,18958,18959,18960,18961,18962,18963,18964,18965,18966,18967,18968,18969,18970,18971,18972,18973,18974,18975,18976,18977,18978,18979,18980,18981,18982,18983,18984,18985,18986,18987,18988,18989,18990,18991,18992,18993,18994,18995,18996,18997,18998,18999,19000,19001,19002,19003,19004,19005,19006,19007,19008,19009,19010,19011,19012,19013,19014,19015,19016,19017,19018,19019,19020,19021,19022,19023,19024,19025,19026,19027,19028,19029,19030,19031,19032,19033,19034,19035,19036,19037,19038,19039,19040,19041,19042,19043,19044,19045,19046,19047,19048,19049,19050,19051,19052,19053,19054,19055,19056,19057,19058,19059,19060,19061,19062,19063,19064,19065,19066,19067,19068,19069,19070,19071,19072,19073,19074,19075,19076,19077,19078,19079,19080,19081,19082,19083,19084,19085,19086,19087,19088,19089,19090,19091,19092,19093,19094,19095,19096,19097,19098,19099,19100,19101,19102,19103,19104,19105,19106,19107,19108,19109,19110,19111,19112,19113,19114,19115,19116,19117,19118,19119,19120,19121,19122,19123,19124,19125,19126,19127,19128,19129,19130,19131,19132,19133,19134,19135,19136,19137,19138,19139,19140,19141,19142,19143,19144,19145,19146,19147,19148,19149,19150,19151,19152,19153,19154,19155,19156,19157,19158,19159,19160,19161,19162,19163,19164,19165,19166,19167,19168,19169,19170,19171,19172,19173,19174,19175,19176,19177,19178,19179,19180,19181,19182,19183,19184,19185,19186,19187,19188,19189,19190,19191,19192,19193,19194,19195,19196,19197,19198,19199,19200,19201,19202,19203,19204,19205,19206,19207,19208,19209,19210,19211,19212,19213,19214,19215,19216,19217,19218,19219,19220,19221,19222,19223,19224,19225,19226,19227,19228,19229,19230,19231,19232,19233,19234,19235,19236,19237,19238,19239,19240,19241,19242,19243,19244,19245,19246,19247,19248,19249,19250,19251,19252,19253,19254,19255,19256,19257,19258,19259,19260,19261,19262,19263,19264,19265,19266,19267,19268,19269,19270,19271,19272,19273,19274,19275,19276,19277,19278,19279,19280,19281,19282,19283,19284,19285,19286,19287,19288,19289,19290,19291,19292,19293,19294,19295,19296,19297,19298,19299,19300,19301,19302,19303,19304,19305,19306,19307,19308,19309,19310,19311,19312,19313,19314,19315,19316,19317,19318,19319,19320,19321,19322,19323,19324,19325,19326,19327,19328,19329,19330,19331,19332,19333,19334,19335,19336,19337,19338,19339,19340,19341,19342,19343,19344,19345,19346,19347,19348,19349,19350,19351,19352,19353,19354,19355,19356,19357,19358,19359,19360,19361,19362,19363,19364,19365,19366,19367,19368,19369,19370,19371,19372,19373,19374,19375,19376,19377,19378,19379,19380,19381,19382,19383,19384,19385,19386,19387,19388,19389,19390,19391,19392,19393,19394,19395,19396,19397,19398,19399,19400,19401,19402,19403,19404,19405,19406,19407,19408,19409,19410,19411,19412,19413,19414,19415,19416,19417,19418,19419,19420,19421,19422,19423,19424,19425,19426,19427,19428,19429,19430,19431,19432,19433,19434,19435,19436,19437,19438,19439,19440,19441,19442,19443,19444,19445,19446,19447,19448,19449,19450,19451,19452,19453,19454,19455,19456,19457,19458,19459,19460,19461,19462,19463,19464,19465,19466,19467,19468,19469,19470,19471,19472,19473,19474,19475,19476,19477,19478,19479,19480,19481,19482,19483,19484,19485,19486,19487,19488,19489,19490,19491,19492,19493,19494,19495,19496,19497,19498,19499,19500,19501,19502,19503,19504,19505,19506,19507,19508,19509,19510,19511,19512,19513,19514,19515,19516,19517,19518,19519,19520,19521,19522,19523,19524,19525,19526,19527,19528,19529,19530,19531,19532,19533,19534,19535,19536,19537,19538,19539,19540,19541,19542,19543,19544,19545,19546,19547,19548,19549,19550,19551,19552,19553,19554,19555,19556,19557,19558,19559,19560,19561,19562,19563,19564,19565,19566,19567,19568,19569,19570,19571,19572,19573,19574,19575,19576,19577,19578,19579,19580,19581,19582,19583,19584,19585,19586,19587,19588,19589,19590,19591,19592,19593,19594,19595,19596,19597,19598,19599,19600,19601,19602,19603,19604,19605,19606,19607,19608,19609,19610,19611,19612,19613,19614,19615,19616,19617,19618,19619,19620,19621,19622,19623,19624,19625,19626,19627,19628,19629,19630,19631,19632,19633,19634,19635,19636,19637,19638,19639,19640,19641,19642,19643,19644,19645,19646,19647,19648,19649,19650,19651,19652,19653,19654,19655,19656,19657,19658,19659,19660,19661,19662,19663,19664,19665,19666,19667,19668,19669,19670,19671,19672,19673,19674,19675,19676,19677,19678,19679,19680,19681,19682,19683,19684,19685,19686,19687,19688,19689,19690,19691,19692,19693,19694,19695,19696,19697,19698,19699,19700,19701,19702,19703,19704,19705,19706,19707,19708,19709,19710,19711,19712,19713,19714,19715,19716,19717,19718,19719,19720,19721,19722,19723,19724,19725,19726,19727,19728,19729,19730,19731,19732,19733,19734,19735,19736,19737,19738,19739,19740,19741,19742,19743,19744,19745,19746,19747,19748,19749,19750,19751,19752,19753,19754,19755,19756,19757,19758,19759,19760,19761,19762,19763,19764,19765,19766,19767,19768,19769,19770,19771,19772,19773,19774,19775,19776,19777,19778,19779,19780,19781,19782,19783,19784,19785,19786,19787,19788,19789,19790,19791,19792,19793,19794,19795,19796,19797,19798,19799,19800,19801,19802,19803,19804,19805,19806,19807,19808,19809,19810,19811,19812,19813,19814,19815,19816,19817,19818,19819,19820,19821,19822,19823,19824,19825,19826,19827,19828,19829,19830,19831,19832,19833,19834,19835,19836,19837,19838,19839,19840,19841,19842,19843,19844,19845,19846,19847,19848,19849,19850,19851,19852,19853,19854,19855,19856,19857,19858,19859,19860,19861,19862,19863,19864,19865,19866,19867,19868,19869,19870,19871,19872,19873,19874,19875,19876,19877,19878,19879,19880,19881,19882,19883,19884,19885,19886,19887,19888,19889,19890,19891,19892,19893,19968,19969,19970,19971,19972,19973,19974,19975,19976,19977,19978,19979,19980,19981,19982,19983,19984,19985,19986,19987,19988,19989,19990,19991,19992,19993,19994,19995,19996,19997,19998,19999,20000,20001,20002,20003,20004,20005,20006,20007,20008,20009,20010,20011,20012,20013,20014,20015,20016,20017,20018,20019,20020,20021,20022,20023,20024,20025,20026,20027,20028,20029,20030,20031,20032,20033,20034,20035,20036,20037,20038,20039,20040,20041,20042,20043,20044,20045,20046,20047,20048,20049,20050,20051,20052,20053,20054,20055,20056,20057,20058,20059,20060,20061,20062,20063,20064,20065,20066,20067,20068,20069,20070,20071,20072,20073,20074,20075,20076,20077,20078,20079,20080,20081,20082,20083,20084,20085,20086,20087,20088,20089,20090,20091,20092,20093,20094,20095,20096,20097,20098,20099,20100,20101,20102,20103,20104,20105,20106,20107,20108,20109,20110,20111,20112,20113,20114,20115,20116,20117,20118,20119,20120,20121,20122,20123,20124,20125,20126,20127,20128,20129,20130,20131,20132,20133,20134,20135,20136,20137,20138,20139,20140,20141,20142,20143,20144,20145,20146,20147,20148,20149,20150,20151,20152,20153,20154,20155,20156,20157,20158,20159,20160,20161,20162,20163,20164,20165,20166,20167,20168,20169,20170,20171,20172,20173,20174,20175,20176,20177,20178,20179,20180,20181,20182,20183,20184,20185,20186,20187,20188,20189,20190,20191,20192,20193,20194,20195,20196,20197,20198,20199,20200,20201,20202,20203,20204,20205,20206,20207,20208,20209,20210,20211,20212,20213,20214,20215,20216,20217,20218,20219,20220,20221,20222,20223,20224,20225,20226,20227,20228,20229,20230,20231,20232,20233,20234,20235,20236,20237,20238,20239,20240,20241,20242,20243,20244,20245,20246,20247,20248,20249,20250,20251,20252,20253,20254,20255,20256,20257,20258,20259,20260,20261,20262,20263,20264,20265,20266,20267,20268,20269,20270,20271,20272,20273,20274,20275,20276,20277,20278,20279,20280,20281,20282,20283,20284,20285,20286,20287,20288,20289,20290,20291,20292,20293,20294,20295,20296,20297,20298,20299,20300,20301,20302,20303,20304,20305,20306,20307,20308,20309,20310,20311,20312,20313,20314,20315,20316,20317,20318,20319,20320,20321,20322,20323,20324,20325,20326,20327,20328,20329,20330,20331,20332,20333,20334,20335,20336,20337,20338,20339,20340,20341,20342,20343,20344,20345,20346,20347,20348,20349,20350,20351,20352,20353,20354,20355,20356,20357,20358,20359,20360,20361,20362,20363,20364,20365,20366,20367,20368,20369,20370,20371,20372,20373,20374,20375,20376,20377,20378,20379,20380,20381,20382,20383,20384,20385,20386,20387,20388,20389,20390,20391,20392,20393,20394,20395,20396,20397,20398,20399,20400,20401,20402,20403,20404,20405,20406,20407,20408,20409,20410,20411,20412,20413,20414,20415,20416,20417,20418,20419,20420,20421,20422,20423,20424,20425,20426,20427,20428,20429,20430,20431,20432,20433,20434,20435,20436,20437,20438,20439,20440,20441,20442,20443,20444,20445,20446,20447,20448,20449,20450,20451,20452,20453,20454,20455,20456,20457,20458,20459,20460,20461,20462,20463,20464,20465,20466,20467,20468,20469,20470,20471,20472,20473,20474,20475,20476,20477,20478,20479,20480,20481,20482,20483,20484,20485,20486,20487,20488,20489,20490,20491,20492,20493,20494,20495,20496,20497,20498,20499,20500,20501,20502,20503,20504,20505,20506,20507,20508,20509,20510,20511,20512,20513,20514,20515,20516,20517,20518,20519,20520,20521,20522,20523,20524,20525,20526,20527,20528,20529,20530,20531,20532,20533,20534,20535,20536,20537,20538,20539,20540,20541,20542,20543,20544,20545,20546,20547,20548,20549,20550,20551,20552,20553,20554,20555,20556,20557,20558,20559,20560,20561,20562,20563,20564,20565,20566,20567,20568,20569,20570,20571,20572,20573,20574,20575,20576,20577,20578,20579,20580,20581,20582,20583,20584,20585,20586,20587,20588,20589,20590,20591,20592,20593,20594,20595,20596,20597,20598,20599,20600,20601,20602,20603,20604,20605,20606,20607,20608,20609,20610,20611,20612,20613,20614,20615,20616,20617,20618,20619,20620,20621,20622,20623,20624,20625,20626,20627,20628,20629,20630,20631,20632,20633,20634,20635,20636,20637,20638,20639,20640,20641,20642,20643,20644,20645,20646,20647,20648,20649,20650,20651,20652,20653,20654,20655,20656,20657,20658,20659,20660,20661,20662,20663,20664,20665,20666,20667,20668,20669,20670,20671,20672,20673,20674,20675,20676,20677,20678,20679,20680,20681,20682,20683,20684,20685,20686,20687,20688,20689,20690,20691,20692,20693,20694,20695,20696,20697,20698,20699,20700,20701,20702,20703,20704,20705,20706,20707,20708,20709,20710,20711,20712,20713,20714,20715,20716,20717,20718,20719,20720,20721,20722,20723,20724,20725,20726,20727,20728,20729,20730,20731,20732,20733,20734,20735,20736,20737,20738,20739,20740,20741,20742,20743,20744,20745,20746,20747,20748,20749,20750,20751,20752,20753,20754,20755,20756,20757,20758,20759,20760,20761,20762,20763,20764,20765,20766,20767,20768,20769,20770,20771,20772,20773,20774,20775,20776,20777,20778,20779,20780,20781,20782,20783,20784,20785,20786,20787,20788,20789,20790,20791,20792,20793,20794,20795,20796,20797,20798,20799,20800,20801,20802,20803,20804,20805,20806,20807,20808,20809,20810,20811,20812,20813,20814,20815,20816,20817,20818,20819,20820,20821,20822,20823,20824,20825,20826,20827,20828,20829,20830,20831,20832,20833,20834,20835,20836,20837,20838,20839,20840,20841,20842,20843,20844,20845,20846,20847,20848,20849,20850,20851,20852,20853,20854,20855,20856,20857,20858,20859,20860,20861,20862,20863,20864,20865,20866,20867,20868,20869,20870,20871,20872,20873,20874,20875,20876,20877,20878,20879,20880,20881,20882,20883,20884,20885,20886,20887,20888,20889,20890,20891,20892,20893,20894,20895,20896,20897,20898,20899,20900,20901,20902,20903,20904,20905,20906,20907,20908,20909,20910,20911,20912,20913,20914,20915,20916,20917,20918,20919,20920,20921,20922,20923,20924,20925,20926,20927,20928,20929,20930,20931,20932,20933,20934,20935,20936,20937,20938,20939,20940,20941,20942,20943,20944,20945,20946,20947,20948,20949,20950,20951,20952,20953,20954,20955,20956,20957,20958,20959,20960,20961,20962,20963,20964,20965,20966,20967,20968,20969,20970,20971,20972,20973,20974,20975,20976,20977,20978,20979,20980,20981,20982,20983,20984,20985,20986,20987,20988,20989,20990,20991,20992,20993,20994,20995,20996,20997,20998,20999,21000,21001,21002,21003,21004,21005,21006,21007,21008,21009,21010,21011,21012,21013,21014,21015,21016,21017,21018,21019,21020,21021,21022,21023,21024,21025,21026,21027,21028,21029,21030,21031,21032,21033,21034,21035,21036,21037,21038,21039,21040,21041,21042,21043,21044,21045,21046,21047,21048,21049,21050,21051,21052,21053,21054,21055,21056,21057,21058,21059,21060,21061,21062,21063,21064,21065,21066,21067,21068,21069,21070,21071,21072,21073,21074,21075,21076,21077,21078,21079,21080,21081,21082,21083,21084,21085,21086,21087,21088,21089,21090,21091,21092,21093,21094,21095,21096,21097,21098,21099,21100,21101,21102,21103,21104,21105,21106,21107,21108,21109,21110,21111,21112,21113,21114,21115,21116,21117,21118,21119,21120,21121,21122,21123,21124,21125,21126,21127,21128,21129,21130,21131,21132,21133,21134,21135,21136,21137,21138,21139,21140,21141,21142,21143,21144,21145,21146,21147,21148,21149,21150,21151,21152,21153,21154,21155,21156,21157,21158,21159,21160,21161,21162,21163,21164,21165,21166,21167,21168,21169,21170,21171,21172,21173,21174,21175,21176,21177,21178,21179,21180,21181,21182,21183,21184,21185,21186,21187,21188,21189,21190,21191,21192,21193,21194,21195,21196,21197,21198,21199,21200,21201,21202,21203,21204,21205,21206,21207,21208,21209,21210,21211,21212,21213,21214,21215,21216,21217,21218,21219,21220,21221,21222,21223,21224,21225,21226,21227,21228,21229,21230,21231,21232,21233,21234,21235,21236,21237,21238,21239,21240,21241,21242,21243,21244,21245,21246,21247,21248,21249,21250,21251,21252,21253,21254,21255,21256,21257,21258,21259,21260,21261,21262,21263,21264,21265,21266,21267,21268,21269,21270,21271,21272,21273,21274,21275,21276,21277,21278,21279,21280,21281,21282,21283,21284,21285,21286,21287,21288,21289,21290,21291,21292,21293,21294,21295,21296,21297,21298,21299,21300,21301,21302,21303,21304,21305,21306,21307,21308,21309,21310,21311,21312,21313,21314,21315,21316,21317,21318,21319,21320,21321,21322,21323,21324,21325,21326,21327,21328,21329,21330,21331,21332,21333,21334,21335,21336,21337,21338,21339,21340,21341,21342,21343,21344,21345,21346,21347,21348,21349,21350,21351,21352,21353,21354,21355,21356,21357,21358,21359,21360,21361,21362,21363,21364,21365,21366,21367,21368,21369,21370,21371,21372,21373,21374,21375,21376,21377,21378,21379,21380,21381,21382,21383,21384,21385,21386,21387,21388,21389,21390,21391,21392,21393,21394,21395,21396,21397,21398,21399,21400,21401,21402,21403,21404,21405,21406,21407,21408,21409,21410,21411,21412,21413,21414,21415,21416,21417,21418,21419,21420,21421,21422,21423,21424,21425,21426,21427,21428,21429,21430,21431,21432,21433,21434,21435,21436,21437,21438,21439,21440,21441,21442,21443,21444,21445,21446,21447,21448,21449,21450,21451,21452,21453,21454,21455,21456,21457,21458,21459,21460,21461,21462,21463,21464,21465,21466,21467,21468,21469,21470,21471,21472,21473,21474,21475,21476,21477,21478,21479,21480,21481,21482,21483,21484,21485,21486,21487,21488,21489,21490,21491,21492,21493,21494,21495,21496,21497,21498,21499,21500,21501,21502,21503,21504,21505,21506,21507,21508,21509,21510,21511,21512,21513,21514,21515,21516,21517,21518,21519,21520,21521,21522,21523,21524,21525,21526,21527,21528,21529,21530,21531,21532,21533,21534,21535,21536,21537,21538,21539,21540,21541,21542,21543,21544,21545,21546,21547,21548,21549,21550,21551,21552,21553,21554,21555,21556,21557,21558,21559,21560,21561,21562,21563,21564,21565,21566,21567,21568,21569,21570,21571,21572,21573,21574,21575,21576,21577,21578,21579,21580,21581,21582,21583,21584,21585,21586,21587,21588,21589,21590,21591,21592,21593,21594,21595,21596,21597,21598,21599,21600,21601,21602,21603,21604,21605,21606,21607,21608,21609,21610,21611,21612,21613,21614,21615,21616,21617,21618,21619,21620,21621,21622,21623,21624,21625,21626,21627,21628,21629,21630,21631,21632,21633,21634,21635,21636,21637,21638,21639,21640,21641,21642,21643,21644,21645,21646,21647,21648,21649,21650,21651,21652,21653,21654,21655,21656,21657,21658,21659,21660,21661,21662,21663,21664,21665,21666,21667,21668,21669,21670,21671,21672,21673,21674,21675,21676,21677,21678,21679,21680,21681,21682,21683,21684,21685,21686,21687,21688,21689,21690,21691,21692,21693,21694,21695,21696,21697,21698,21699,21700,21701,21702,21703,21704,21705,21706,21707,21708,21709,21710,21711,21712,21713,21714,21715,21716,21717,21718,21719,21720,21721,21722,21723,21724,21725,21726,21727,21728,21729,21730,21731,21732,21733,21734,21735,21736,21737,21738,21739,21740,21741,21742,21743,21744,21745,21746,21747,21748,21749,21750,21751,21752,21753,21754,21755,21756,21757,21758,21759,21760,21761,21762,21763,21764,21765,21766,21767,21768,21769,21770,21771,21772,21773,21774,21775,21776,21777,21778,21779,21780,21781,21782,21783,21784,21785,21786,21787,21788,21789,21790,21791,21792,21793,21794,21795,21796,21797,21798,21799,21800,21801,21802,21803,21804,21805,21806,21807,21808,21809,21810,21811,21812,21813,21814,21815,21816,21817,21818,21819,21820,21821,21822,21823,21824,21825,21826,21827,21828,21829,21830,21831,21832,21833,21834,21835,21836,21837,21838,21839,21840,21841,21842,21843,21844,21845,21846,21847,21848,21849,21850,21851,21852,21853,21854,21855,21856,21857,21858,21859,21860,21861,21862,21863,21864,21865,21866,21867,21868,21869,21870,21871,21872,21873,21874,21875,21876,21877,21878,21879,21880,21881,21882,21883,21884,21885,21886,21887,21888,21889,21890,21891,21892,21893,21894,21895,21896,21897,21898,21899,21900,21901,21902,21903,21904,21905,21906,21907,21908,21909,21910,21911,21912,21913,21914,21915,21916,21917,21918,21919,21920,21921,21922,21923,21924,21925,21926,21927,21928,21929,21930,21931,21932,21933,21934,21935,21936,21937,21938,21939,21940,21941,21942,21943,21944,21945,21946,21947,21948,21949,21950,21951,21952,21953,21954,21955,21956,21957,21958,21959,21960,21961,21962,21963,21964,21965,21966,21967,21968,21969,21970,21971,21972,21973,21974,21975,21976,21977,21978,21979,21980,21981,21982,21983,21984,21985,21986,21987,21988,21989,21990,21991,21992,21993,21994,21995,21996,21997,21998,21999,22000,22001,22002,22003,22004,22005,22006,22007,22008,22009,22010,22011,22012,22013,22014,22015,22016,22017,22018,22019,22020,22021,22022,22023,22024,22025,22026,22027,22028,22029,22030,22031,22032,22033,22034,22035,22036,22037,22038,22039,22040,22041,22042,22043,22044,22045,22046,22047,22048,22049,22050,22051,22052,22053,22054,22055,22056,22057,22058,22059,22060,22061,22062,22063,22064,22065,22066,22067,22068,22069,22070,22071,22072,22073,22074,22075,22076,22077,22078,22079,22080,22081,22082,22083,22084,22085,22086,22087,22088,22089,22090,22091,22092,22093,22094,22095,22096,22097,22098,22099,22100,22101,22102,22103,22104,22105,22106,22107,22108,22109,22110,22111,22112,22113,22114,22115,22116,22117,22118,22119,22120,22121,22122,22123,22124,22125,22126,22127,22128,22129,22130,22131,22132,22133,22134,22135,22136,22137,22138,22139,22140,22141,22142,22143,22144,22145,22146,22147,22148,22149,22150,22151,22152,22153,22154,22155,22156,22157,22158,22159,22160,22161,22162,22163,22164,22165,22166,22167,22168,22169,22170,22171,22172,22173,22174,22175,22176,22177,22178,22179,22180,22181,22182,22183,22184,22185,22186,22187,22188,22189,22190,22191,22192,22193,22194,22195,22196,22197,22198,22199,22200,22201,22202,22203,22204,22205,22206,22207,22208,22209,22210,22211,22212,22213,22214,22215,22216,22217,22218,22219,22220,22221,22222,22223,22224,22225,22226,22227,22228,22229,22230,22231,22232,22233,22234,22235,22236,22237,22238,22239,22240,22241,22242,22243,22244,22245,22246,22247,22248,22249,22250,22251,22252,22253,22254,22255,22256,22257,22258,22259,22260,22261,22262,22263,22264,22265,22266,22267,22268,22269,22270,22271,22272,22273,22274,22275,22276,22277,22278,22279,22280,22281,22282,22283,22284,22285,22286,22287,22288,22289,22290,22291,22292,22293,22294,22295,22296,22297,22298,22299,22300,22301,22302,22303,22304,22305,22306,22307,22308,22309,22310,22311,22312,22313,22314,22315,22316,22317,22318,22319,22320,22321,22322,22323,22324,22325,22326,22327,22328,22329,22330,22331,22332,22333,22334,22335,22336,22337,22338,22339,22340,22341,22342,22343,22344,22345,22346,22347,22348,22349,22350,22351,22352,22353,22354,22355,22356,22357,22358,22359,22360,22361,22362,22363,22364,22365,22366,22367,22368,22369,22370,22371,22372,22373,22374,22375,22376,22377,22378,22379,22380,22381,22382,22383,22384,22385,22386,22387,22388,22389,22390,22391,22392,22393,22394,22395,22396,22397,22398,22399,22400,22401,22402,22403,22404,22405,22406,22407,22408,22409,22410,22411,22412,22413,22414,22415,22416,22417,22418,22419,22420,22421,22422,22423,22424,22425,22426,22427,22428,22429,22430,22431,22432,22433,22434,22435,22436,22437,22438,22439,22440,22441,22442,22443,22444,22445,22446,22447,22448,22449,22450,22451,22452,22453,22454,22455,22456,22457,22458,22459,22460,22461,22462,22463,22464,22465,22466,22467,22468,22469,22470,22471,22472,22473,22474,22475,22476,22477,22478,22479,22480,22481,22482,22483,22484,22485,22486,22487,22488,22489,22490,22491,22492,22493,22494,22495,22496,22497,22498,22499,22500,22501,22502,22503,22504,22505,22506,22507,22508,22509,22510,22511,22512,22513,22514,22515,22516,22517,22518,22519,22520,22521,22522,22523,22524,22525,22526,22527,22528,22529,22530,22531,22532,22533,22534,22535,22536,22537,22538,22539,22540,22541,22542,22543,22544,22545,22546,22547,22548,22549,22550,22551,22552,22553,22554,22555,22556,22557,22558,22559,22560,22561,22562,22563,22564,22565,22566,22567,22568,22569,22570,22571,22572,22573,22574,22575,22576,22577,22578,22579,22580,22581,22582,22583,22584,22585,22586,22587,22588,22589,22590,22591,22592,22593,22594,22595,22596,22597,22598,22599,22600,22601,22602,22603,22604,22605,22606,22607,22608,22609,22610,22611,22612,22613,22614,22615,22616,22617,22618,22619,22620,22621,22622,22623,22624,22625,22626,22627,22628,22629,22630,22631,22632,22633,22634,22635,22636,22637,22638,22639,22640,22641,22642,22643,22644,22645,22646,22647,22648,22649,22650,22651,22652,22653,22654,22655,22656,22657,22658,22659,22660,22661,22662,22663,22664,22665,22666,22667,22668,22669,22670,22671,22672,22673,22674,22675,22676,22677,22678,22679,22680,22681,22682,22683,22684,22685,22686,22687,22688,22689,22690,22691,22692,22693,22694,22695,22696,22697,22698,22699,22700,22701,22702,22703,22704,22705,22706,22707,22708,22709,22710,22711,22712,22713,22714,22715,22716,22717,22718,22719,22720,22721,22722,22723,22724,22725,22726,22727,22728,22729,22730,22731,22732,22733,22734,22735,22736,22737,22738,22739,22740,22741,22742,22743,22744,22745,22746,22747,22748,22749,22750,22751,22752,22753,22754,22755,22756,22757,22758,22759,22760,22761,22762,22763,22764,22765,22766,22767,22768,22769,22770,22771,22772,22773,22774,22775,22776,22777,22778,22779,22780,22781,22782,22783,22784,22785,22786,22787,22788,22789,22790,22791,22792,22793,22794,22795,22796,22797,22798,22799,22800,22801,22802,22803,22804,22805,22806,22807,22808,22809,22810,22811,22812,22813,22814,22815,22816,22817,22818,22819,22820,22821,22822,22823,22824,22825,22826,22827,22828,22829,22830,22831,22832,22833,22834,22835,22836,22837,22838,22839,22840,22841,22842,22843,22844,22845,22846,22847,22848,22849,22850,22851,22852,22853,22854,22855,22856,22857,22858,22859,22860,22861,22862,22863,22864,22865,22866,22867,22868,22869,22870,22871,22872,22873,22874,22875,22876,22877,22878,22879,22880,22881,22882,22883,22884,22885,22886,22887,22888,22889,22890,22891,22892,22893,22894,22895,22896,22897,22898,22899,22900,22901,22902,22903,22904,22905,22906,22907,22908,22909,22910,22911,22912,22913,22914,22915,22916,22917,22918,22919,22920,22921,22922,22923,22924,22925,22926,22927,22928,22929,22930,22931,22932,22933,22934,22935,22936,22937,22938,22939,22940,22941,22942,22943,22944,22945,22946,22947,22948,22949,22950,22951,22952,22953,22954,22955,22956,22957,22958,22959,22960,22961,22962,22963,22964,22965,22966,22967,22968,22969,22970,22971,22972,22973,22974,22975,22976,22977,22978,22979,22980,22981,22982,22983,22984,22985,22986,22987,22988,22989,22990,22991,22992,22993,22994,22995,22996,22997,22998,22999,23000,23001,23002,23003,23004,23005,23006,23007,23008,23009,23010,23011,23012,23013,23014,23015,23016,23017,23018,23019,23020,23021,23022,23023,23024,23025,23026,23027,23028,23029,23030,23031,23032,23033,23034,23035,23036,23037,23038,23039,23040,23041,23042,23043,23044,23045,23046,23047,23048,23049,23050,23051,23052,23053,23054,23055,23056,23057,23058,23059,23060,23061,23062,23063,23064,23065,23066,23067,23068,23069,23070,23071,23072,23073,23074,23075,23076,23077,23078,23079,23080,23081,23082,23083,23084,23085,23086,23087,23088,23089,23090,23091,23092,23093,23094,23095,23096,23097,23098,23099,23100,23101,23102,23103,23104,23105,23106,23107,23108,23109,23110,23111,23112,23113,23114,23115,23116,23117,23118,23119,23120,23121,23122,23123,23124,23125,23126,23127,23128,23129,23130,23131,23132,23133,23134,23135,23136,23137,23138,23139,23140,23141,23142,23143,23144,23145,23146,23147,23148,23149,23150,23151,23152,23153,23154,23155,23156,23157,23158,23159,23160,23161,23162,23163,23164,23165,23166,23167,23168,23169,23170,23171,23172,23173,23174,23175,23176,23177,23178,23179,23180,23181,23182,23183,23184,23185,23186,23187,23188,23189,23190,23191,23192,23193,23194,23195,23196,23197,23198,23199,23200,23201,23202,23203,23204,23205,23206,23207,23208,23209,23210,23211,23212,23213,23214,23215,23216,23217,23218,23219,23220,23221,23222,23223,23224,23225,23226,23227,23228,23229,23230,23231,23232,23233,23234,23235,23236,23237,23238,23239,23240,23241,23242,23243,23244,23245,23246,23247,23248,23249,23250,23251,23252,23253,23254,23255,23256,23257,23258,23259,23260,23261,23262,23263,23264,23265,23266,23267,23268,23269,23270,23271,23272,23273,23274,23275,23276,23277,23278,23279,23280,23281,23282,23283,23284,23285,23286,23287,23288,23289,23290,23291,23292,23293,23294,23295,23296,23297,23298,23299,23300,23301,23302,23303,23304,23305,23306,23307,23308,23309,23310,23311,23312,23313,23314,23315,23316,23317,23318,23319,23320,23321,23322,23323,23324,23325,23326,23327,23328,23329,23330,23331,23332,23333,23334,23335,23336,23337,23338,23339,23340,23341,23342,23343,23344,23345,23346,23347,23348,23349,23350,23351,23352,23353,23354,23355,23356,23357,23358,23359,23360,23361,23362,23363,23364,23365,23366,23367,23368,23369,23370,23371,23372,23373,23374,23375,23376,23377,23378,23379,23380,23381,23382,23383,23384,23385,23386,23387,23388,23389,23390,23391,23392,23393,23394,23395,23396,23397,23398,23399,23400,23401,23402,23403,23404,23405,23406,23407,23408,23409,23410,23411,23412,23413,23414,23415,23416,23417,23418,23419,23420,23421,23422,23423,23424,23425,23426,23427,23428,23429,23430,23431,23432,23433,23434,23435,23436,23437,23438,23439,23440,23441,23442,23443,23444,23445,23446,23447,23448,23449,23450,23451,23452,23453,23454,23455,23456,23457,23458,23459,23460,23461,23462,23463,23464,23465,23466,23467,23468,23469,23470,23471,23472,23473,23474,23475,23476,23477,23478,23479,23480,23481,23482,23483,23484,23485,23486,23487,23488,23489,23490,23491,23492,23493,23494,23495,23496,23497,23498,23499,23500,23501,23502,23503,23504,23505,23506,23507,23508,23509,23510,23511,23512,23513,23514,23515,23516,23517,23518,23519,23520,23521,23522,23523,23524,23525,23526,23527,23528,23529,23530,23531,23532,23533,23534,23535,23536,23537,23538,23539,23540,23541,23542,23543,23544,23545,23546,23547,23548,23549,23550,23551,23552,23553,23554,23555,23556,23557,23558,23559,23560,23561,23562,23563,23564,23565,23566,23567,23568,23569,23570,23571,23572,23573,23574,23575,23576,23577,23578,23579,23580,23581,23582,23583,23584,23585,23586,23587,23588,23589,23590,23591,23592,23593,23594,23595,23596,23597,23598,23599,23600,23601,23602,23603,23604,23605,23606,23607,23608,23609,23610,23611,23612,23613,23614,23615,23616,23617,23618,23619,23620,23621,23622,23623,23624,23625,23626,23627,23628,23629,23630,23631,23632,23633,23634,23635,23636,23637,23638,23639,23640,23641,23642,23643,23644,23645,23646,23647,23648,23649,23650,23651,23652,23653,23654,23655,23656,23657,23658,23659,23660,23661,23662,23663,23664,23665,23666,23667,23668,23669,23670,23671,23672,23673,23674,23675,23676,23677,23678,23679,23680,23681,23682,23683,23684,23685,23686,23687,23688,23689,23690,23691,23692,23693,23694,23695,23696,23697,23698,23699,23700,23701,23702,23703,23704,23705,23706,23707,23708,23709,23710,23711,23712,23713,23714,23715,23716,23717,23718,23719,23720,23721,23722,23723,23724,23725,23726,23727,23728,23729,23730,23731,23732,23733,23734,23735,23736,23737,23738,23739,23740,23741,23742,23743,23744,23745,23746,23747,23748,23749,23750,23751,23752,23753,23754,23755,23756,23757,23758,23759,23760,23761,23762,23763,23764,23765,23766,23767,23768,23769,23770,23771,23772,23773,23774,23775,23776,23777,23778,23779,23780,23781,23782,23783,23784,23785,23786,23787,23788,23789,23790,23791,23792,23793,23794,23795,23796,23797,23798,23799,23800,23801,23802,23803,23804,23805,23806,23807,23808,23809,23810,23811,23812,23813,23814,23815,23816,23817,23818,23819,23820,23821,23822,23823,23824,23825,23826,23827,23828,23829,23830,23831,23832,23833,23834,23835,23836,23837,23838,23839,23840,23841,23842,23843,23844,23845,23846,23847,23848,23849,23850,23851,23852,23853,23854,23855,23856,23857,23858,23859,23860,23861,23862,23863,23864,23865,23866,23867,23868,23869,23870,23871,23872,23873,23874,23875,23876,23877,23878,23879,23880,23881,23882,23883,23884,23885,23886,23887,23888,23889,23890,23891,23892,23893,23894,23895,23896,23897,23898,23899,23900,23901,23902,23903,23904,23905,23906,23907,23908,23909,23910,23911,23912,23913,23914,23915,23916,23917,23918,23919,23920,23921,23922,23923,23924,23925,23926,23927,23928,23929,23930,23931,23932,23933,23934,23935,23936,23937,23938,23939,23940,23941,23942,23943,23944,23945,23946,23947,23948,23949,23950,23951,23952,23953,23954,23955,23956,23957,23958,23959,23960,23961,23962,23963,23964,23965,23966,23967,23968,23969,23970,23971,23972,23973,23974,23975,23976,23977,23978,23979,23980,23981,23982,23983,23984,23985,23986,23987,23988,23989,23990,23991,23992,23993,23994,23995,23996,23997,23998,23999,24000,24001,24002,24003,24004,24005,24006,24007,24008,24009,24010,24011,24012,24013,24014,24015,24016,24017,24018,24019,24020,24021,24022,24023,24024,24025,24026,24027,24028,24029,24030,24031,24032,24033,24034,24035,24036,24037,24038,24039,24040,24041,24042,24043,24044,24045,24046,24047,24048,24049,24050,24051,24052,24053,24054,24055,24056,24057,24058,24059,24060,24061,24062,24063,24064,24065,24066,24067,24068,24069,24070,24071,24072,24073,24074,24075,24076,24077,24078,24079,24080,24081,24082,24083,24084,24085,24086,24087,24088,24089,24090,24091,24092,24093,24094,24095,24096,24097,24098,24099,24100,24101,24102,24103,24104,24105,24106,24107,24108,24109,24110,24111,24112,24113,24114,24115,24116,24117,24118,24119,24120,24121,24122,24123,24124,24125,24126,24127,24128,24129,24130,24131,24132,24133,24134,24135,24136,24137,24138,24139,24140,24141,24142,24143,24144,24145,24146,24147,24148,24149,24150,24151,24152,24153,24154,24155,24156,24157,24158,24159,24160,24161,24162,24163,24164,24165,24166,24167,24168,24169,24170,24171,24172,24173,24174,24175,24176,24177,24178,24179,24180,24181,24182,24183,24184,24185,24186,24187,24188,24189,24190,24191,24192,24193,24194,24195,24196,24197,24198,24199,24200,24201,24202,24203,24204,24205,24206,24207,24208,24209,24210,24211,24212,24213,24214,24215,24216,24217,24218,24219,24220,24221,24222,24223,24224,24225,24226,24227,24228,24229,24230,24231,24232,24233,24234,24235,24236,24237,24238,24239,24240,24241,24242,24243,24244,24245,24246,24247,24248,24249,24250,24251,24252,24253,24254,24255,24256,24257,24258,24259,24260,24261,24262,24263,24264,24265,24266,24267,24268,24269,24270,24271,24272,24273,24274,24275,24276,24277,24278,24279,24280,24281,24282,24283,24284,24285,24286,24287,24288,24289,24290,24291,24292,24293,24294,24295,24296,24297,24298,24299,24300,24301,24302,24303,24304,24305,24306,24307,24308,24309,24310,24311,24312,24313,24314,24315,24316,24317,24318,24319,24320,24321,24322,24323,24324,24325,24326,24327,24328,24329,24330,24331,24332,24333,24334,24335,24336,24337,24338,24339,24340,24341,24342,24343,24344,24345,24346,24347,24348,24349,24350,24351,24352,24353,24354,24355,24356,24357,24358,24359,24360,24361,24362,24363,24364,24365,24366,24367,24368,24369,24370,24371,24372,24373,24374,24375,24376,24377,24378,24379,24380,24381,24382,24383,24384,24385,24386,24387,24388,24389,24390,24391,24392,24393,24394,24395,24396,24397,24398,24399,24400,24401,24402,24403,24404,24405,24406,24407,24408,24409,24410,24411,24412,24413,24414,24415,24416,24417,24418,24419,24420,24421,24422,24423,24424,24425,24426,24427,24428,24429,24430,24431,24432,24433,24434,24435,24436,24437,24438,24439,24440,24441,24442,24443,24444,24445,24446,24447,24448,24449,24450,24451,24452,24453,24454,24455,24456,24457,24458,24459,24460,24461,24462,24463,24464,24465,24466,24467,24468,24469,24470,24471,24472,24473,24474,24475,24476,24477,24478,24479,24480,24481,24482,24483,24484,24485,24486,24487,24488,24489,24490,24491,24492,24493,24494,24495,24496,24497,24498,24499,24500,24501,24502,24503,24504,24505,24506,24507,24508,24509,24510,24511,24512,24513,24514,24515,24516,24517,24518,24519,24520,24521,24522,24523,24524,24525,24526,24527,24528,24529,24530,24531,24532,24533,24534,24535,24536,24537,24538,24539,24540,24541,24542,24543,24544,24545,24546,24547,24548,24549,24550,24551,24552,24553,24554,24555,24556,24557,24558,24559,24560,24561,24562,24563,24564,24565,24566,24567,24568,24569,24570,24571,24572,24573,24574,24575,24576,24577,24578,24579,24580,24581,24582,24583,24584,24585,24586,24587,24588,24589,24590,24591,24592,24593,24594,24595,24596,24597,24598,24599,24600,24601,24602,24603,24604,24605,24606,24607,24608,24609,24610,24611,24612,24613,24614,24615,24616,24617,24618,24619,24620,24621,24622,24623,24624,24625,24626,24627,24628,24629,24630,24631,24632,24633,24634,24635,24636,24637,24638,24639,24640,24641,24642,24643,24644,24645,24646,24647,24648,24649,24650,24651,24652,24653,24654,24655,24656,24657,24658,24659,24660,24661,24662,24663,24664,24665,24666,24667,24668,24669,24670,24671,24672,24673,24674,24675,24676,24677,24678,24679,24680,24681,24682,24683,24684,24685,24686,24687,24688,24689,24690,24691,24692,24693,24694,24695,24696,24697,24698,24699,24700,24701,24702,24703,24704,24705,24706,24707,24708,24709,24710,24711,24712,24713,24714,24715,24716,24717,24718,24719,24720,24721,24722,24723,24724,24725,24726,24727,24728,24729,24730,24731,24732,24733,24734,24735,24736,24737,24738,24739,24740,24741,24742,24743,24744,24745,24746,24747,24748,24749,24750,24751,24752,24753,24754,24755,24756,24757,24758,24759,24760,24761,24762,24763,24764,24765,24766,24767,24768,24769,24770,24771,24772,24773,24774,24775,24776,24777,24778,24779,24780,24781,24782,24783,24784,24785,24786,24787,24788,24789,24790,24791,24792,24793,24794,24795,24796,24797,24798,24799,24800,24801,24802,24803,24804,24805,24806,24807,24808,24809,24810,24811,24812,24813,24814,24815,24816,24817,24818,24819,24820,24821,24822,24823,24824,24825,24826,24827,24828,24829,24830,24831,24832,24833,24834,24835,24836,24837,24838,24839,24840,24841,24842,24843,24844,24845,24846,24847,24848,24849,24850,24851,24852,24853,24854,24855,24856,24857,24858,24859,24860,24861,24862,24863,24864,24865,24866,24867,24868,24869,24870,24871,24872,24873,24874,24875,24876,24877,24878,24879,24880,24881,24882,24883,24884,24885,24886,24887,24888,24889,24890,24891,24892,24893,24894,24895,24896,24897,24898,24899,24900,24901,24902,24903,24904,24905,24906,24907,24908,24909,24910,24911,24912,24913,24914,24915,24916,24917,24918,24919,24920,24921,24922,24923,24924,24925,24926,24927,24928,24929,24930,24931,24932,24933,24934,24935,24936,24937,24938,24939,24940,24941,24942,24943,24944,24945,24946,24947,24948,24949,24950,24951,24952,24953,24954,24955,24956,24957,24958,24959,24960,24961,24962,24963,24964,24965,24966,24967,24968,24969,24970,24971,24972,24973,24974,24975,24976,24977,24978,24979,24980,24981,24982,24983,24984,24985,24986,24987,24988,24989,24990,24991,24992,24993,24994,24995,24996,24997,24998,24999,25000,25001,25002,25003,25004,25005,25006,25007,25008,25009,25010,25011,25012,25013,25014,25015,25016,25017,25018,25019,25020,25021,25022,25023,25024,25025,25026,25027,25028,25029,25030,25031,25032,25033,25034,25035,25036,25037,25038,25039,25040,25041,25042,25043,25044,25045,25046,25047,25048,25049,25050,25051,25052,25053,25054,25055,25056,25057,25058,25059,25060,25061,25062,25063,25064,25065,25066,25067,25068,25069,25070,25071,25072,25073,25074,25075,25076,25077,25078,25079,25080,25081,25082,25083,25084,25085,25086,25087,25088,25089,25090,25091,25092,25093,25094,25095,25096,25097,25098,25099,25100,25101,25102,25103,25104,25105,25106,25107,25108,25109,25110,25111,25112,25113,25114,25115,25116,25117,25118,25119,25120,25121,25122,25123,25124,25125,25126,25127,25128,25129,25130,25131,25132,25133,25134,25135,25136,25137,25138,25139,25140,25141,25142,25143,25144,25145,25146,25147,25148,25149,25150,25151,25152,25153,25154,25155,25156,25157,25158,25159,25160,25161,25162,25163,25164,25165,25166,25167,25168,25169,25170,25171,25172,25173,25174,25175,25176,25177,25178,25179,25180,25181,25182,25183,25184,25185,25186,25187,25188,25189,25190,25191,25192,25193,25194,25195,25196,25197,25198,25199,25200,25201,25202,25203,25204,25205,25206,25207,25208,25209,25210,25211,25212,25213,25214,25215,25216,25217,25218,25219,25220,25221,25222,25223,25224,25225,25226,25227,25228,25229,25230,25231,25232,25233,25234,25235,25236,25237,25238,25239,25240,25241,25242,25243,25244,25245,25246,25247,25248,25249,25250,25251,25252,25253,25254,25255,25256,25257,25258,25259,25260,25261,25262,25263,25264,25265,25266,25267,25268,25269,25270,25271,25272,25273,25274,25275,25276,25277,25278,25279,25280,25281,25282,25283,25284,25285,25286,25287,25288,25289,25290,25291,25292,25293,25294,25295,25296,25297,25298,25299,25300,25301,25302,25303,25304,25305,25306,25307,25308,25309,25310,25311,25312,25313,25314,25315,25316,25317,25318,25319,25320,25321,25322,25323,25324,25325,25326,25327,25328,25329,25330,25331,25332,25333,25334,25335,25336,25337,25338,25339,25340,25341,25342,25343,25344,25345,25346,25347,25348,25349,25350,25351,25352,25353,25354,25355,25356,25357,25358,25359,25360,25361,25362,25363,25364,25365,25366,25367,25368,25369,25370,25371,25372,25373,25374,25375,25376,25377,25378,25379,25380,25381,25382,25383,25384,25385,25386,25387,25388,25389,25390,25391,25392,25393,25394,25395,25396,25397,25398,25399,25400,25401,25402,25403,25404,25405,25406,25407,25408,25409,25410,25411,25412,25413,25414,25415,25416,25417,25418,25419,25420,25421,25422,25423,25424,25425,25426,25427,25428,25429,25430,25431,25432,25433,25434,25435,25436,25437,25438,25439,25440,25441,25442,25443,25444,25445,25446,25447,25448,25449,25450,25451,25452,25453,25454,25455,25456,25457,25458,25459,25460,25461,25462,25463,25464,25465,25466,25467,25468,25469,25470,25471,25472,25473,25474,25475,25476,25477,25478,25479,25480,25481,25482,25483,25484,25485,25486,25487,25488,25489,25490,25491,25492,25493,25494,25495,25496,25497,25498,25499,25500,25501,25502,25503,25504,25505,25506,25507,25508,25509,25510,25511,25512,25513,25514,25515,25516,25517,25518,25519,25520,25521,25522,25523,25524,25525,25526,25527,25528,25529,25530,25531,25532,25533,25534,25535,25536,25537,25538,25539,25540,25541,25542,25543,25544,25545,25546,25547,25548,25549,25550,25551,25552,25553,25554,25555,25556,25557,25558,25559,25560,25561,25562,25563,25564,25565,25566,25567,25568,25569,25570,25571,25572,25573,25574,25575,25576,25577,25578,25579,25580,25581,25582,25583,25584,25585,25586,25587,25588,25589,25590,25591,25592,25593,25594,25595,25596,25597,25598,25599,25600,25601,25602,25603,25604,25605,25606,25607,25608,25609,25610,25611,25612,25613,25614,25615,25616,25617,25618,25619,25620,25621,25622,25623,25624,25625,25626,25627,25628,25629,25630,25631,25632,25633,25634,25635,25636,25637,25638,25639,25640,25641,25642,25643,25644,25645,25646,25647,25648,25649,25650,25651,25652,25653,25654,25655,25656,25657,25658,25659,25660,25661,25662,25663,25664,25665,25666,25667,25668,25669,25670,25671,25672,25673,25674,25675,25676,25677,25678,25679,25680,25681,25682,25683,25684,25685,25686,25687,25688,25689,25690,25691,25692,25693,25694,25695,25696,25697,25698,25699,25700,25701,25702,25703,25704,25705,25706,25707,25708,25709,25710,25711,25712,25713,25714,25715,25716,25717,25718,25719,25720,25721,25722,25723,25724,25725,25726,25727,25728,25729,25730,25731,25732,25733,25734,25735,25736,25737,25738,25739,25740,25741,25742,25743,25744,25745,25746,25747,25748,25749,25750,25751,25752,25753,25754,25755,25756,25757,25758,25759,25760,25761,25762,25763,25764,25765,25766,25767,25768,25769,25770,25771,25772,25773,25774,25775,25776,25777,25778,25779,25780,25781,25782,25783,25784,25785,25786,25787,25788,25789,25790,25791,25792,25793,25794,25795,25796,25797,25798,25799,25800,25801,25802,25803,25804,25805,25806,25807,25808,25809,25810,25811,25812,25813,25814,25815,25816,25817,25818,25819,25820,25821,25822,25823,25824,25825,25826,25827,25828,25829,25830,25831,25832,25833,25834,25835,25836,25837,25838,25839,25840,25841,25842,25843,25844,25845,25846,25847,25848,25849,25850,25851,25852,25853,25854,25855,25856,25857,25858,25859,25860,25861,25862,25863,25864,25865,25866,25867,25868,25869,25870,25871,25872,25873,25874,25875,25876,25877,25878,25879,25880,25881,25882,25883,25884,25885,25886,25887,25888,25889,25890,25891,25892,25893,25894,25895,25896,25897,25898,25899,25900,25901,25902,25903,25904,25905,25906,25907,25908,25909,25910,25911,25912,25913,25914,25915,25916,25917,25918,25919,25920,25921,25922,25923,25924,25925,25926,25927,25928,25929,25930,25931,25932,25933,25934,25935,25936,25937,25938,25939,25940,25941,25942,25943,25944,25945,25946,25947,25948,25949,25950,25951,25952,25953,25954,25955,25956,25957,25958,25959,25960,25961,25962,25963,25964,25965,25966,25967,25968,25969,25970,25971,25972,25973,25974,25975,25976,25977,25978,25979,25980,25981,25982,25983,25984,25985,25986,25987,25988,25989,25990,25991,25992,25993,25994,25995,25996,25997,25998,25999,26000,26001,26002,26003,26004,26005,26006,26007,26008,26009,26010,26011,26012,26013,26014,26015,26016,26017,26018,26019,26020,26021,26022,26023,26024,26025,26026,26027,26028,26029,26030,26031,26032,26033,26034,26035,26036,26037,26038,26039,26040,26041,26042,26043,26044,26045,26046,26047,26048,26049,26050,26051,26052,26053,26054,26055,26056,26057,26058,26059,26060,26061,26062,26063,26064,26065,26066,26067,26068,26069,26070,26071,26072,26073,26074,26075,26076,26077,26078,26079,26080,26081,26082,26083,26084,26085,26086,26087,26088,26089,26090,26091,26092,26093,26094,26095,26096,26097,26098,26099,26100,26101,26102,26103,26104,26105,26106,26107,26108,26109,26110,26111,26112,26113,26114,26115,26116,26117,26118,26119,26120,26121,26122,26123,26124,26125,26126,26127,26128,26129,26130,26131,26132,26133,26134,26135,26136,26137,26138,26139,26140,26141,26142,26143,26144,26145,26146,26147,26148,26149,26150,26151,26152,26153,26154,26155,26156,26157,26158,26159,26160,26161,26162,26163,26164,26165,26166,26167,26168,26169,26170,26171,26172,26173,26174,26175,26176,26177,26178,26179,26180,26181,26182,26183,26184,26185,26186,26187,26188,26189,26190,26191,26192,26193,26194,26195,26196,26197,26198,26199,26200,26201,26202,26203,26204,26205,26206,26207,26208,26209,26210,26211,26212,26213,26214,26215,26216,26217,26218,26219,26220,26221,26222,26223,26224,26225,26226,26227,26228,26229,26230,26231,26232,26233,26234,26235,26236,26237,26238,26239,26240,26241,26242,26243,26244,26245,26246,26247,26248,26249,26250,26251,26252,26253,26254,26255,26256,26257,26258,26259,26260,26261,26262,26263,26264,26265,26266,26267,26268,26269,26270,26271,26272,26273,26274,26275,26276,26277,26278,26279,26280,26281,26282,26283,26284,26285,26286,26287,26288,26289,26290,26291,26292,26293,26294,26295,26296,26297,26298,26299,26300,26301,26302,26303,26304,26305,26306,26307,26308,26309,26310,26311,26312,26313,26314,26315,26316,26317,26318,26319,26320,26321,26322,26323,26324,26325,26326,26327,26328,26329,26330,26331,26332,26333,26334,26335,26336,26337,26338,26339,26340,26341,26342,26343,26344,26345,26346,26347,26348,26349,26350,26351,26352,26353,26354,26355,26356,26357,26358,26359,26360,26361,26362,26363,26364,26365,26366,26367,26368,26369,26370,26371,26372,26373,26374,26375,26376,26377,26378,26379,26380,26381,26382,26383,26384,26385,26386,26387,26388,26389,26390,26391,26392,26393,26394,26395,26396,26397,26398,26399,26400,26401,26402,26403,26404,26405,26406,26407,26408,26409,26410,26411,26412,26413,26414,26415,26416,26417,26418,26419,26420,26421,26422,26423,26424,26425,26426,26427,26428,26429,26430,26431,26432,26433,26434,26435,26436,26437,26438,26439,26440,26441,26442,26443,26444,26445,26446,26447,26448,26449,26450,26451,26452,26453,26454,26455,26456,26457,26458,26459,26460,26461,26462,26463,26464,26465,26466,26467,26468,26469,26470,26471,26472,26473,26474,26475,26476,26477,26478,26479,26480,26481,26482,26483,26484,26485,26486,26487,26488,26489,26490,26491,26492,26493,26494,26495,26496,26497,26498,26499,26500,26501,26502,26503,26504,26505,26506,26507,26508,26509,26510,26511,26512,26513,26514,26515,26516,26517,26518,26519,26520,26521,26522,26523,26524,26525,26526,26527,26528,26529,26530,26531,26532,26533,26534,26535,26536,26537,26538,26539,26540,26541,26542,26543,26544,26545,26546,26547,26548,26549,26550,26551,26552,26553,26554,26555,26556,26557,26558,26559,26560,26561,26562,26563,26564,26565,26566,26567,26568,26569,26570,26571,26572,26573,26574,26575,26576,26577,26578,26579,26580,26581,26582,26583,26584,26585,26586,26587,26588,26589,26590,26591,26592,26593,26594,26595,26596,26597,26598,26599,26600,26601,26602,26603,26604,26605,26606,26607,26608,26609,26610,26611,26612,26613,26614,26615,26616,26617,26618,26619,26620,26621,26622,26623,26624,26625,26626,26627,26628,26629,26630,26631,26632,26633,26634,26635,26636,26637,26638,26639,26640,26641,26642,26643,26644,26645,26646,26647,26648,26649,26650,26651,26652,26653,26654,26655,26656,26657,26658,26659,26660,26661,26662,26663,26664,26665,26666,26667,26668,26669,26670,26671,26672,26673,26674,26675,26676,26677,26678,26679,26680,26681,26682,26683,26684,26685,26686,26687,26688,26689,26690,26691,26692,26693,26694,26695,26696,26697,26698,26699,26700,26701,26702,26703,26704,26705,26706,26707,26708,26709,26710,26711,26712,26713,26714,26715,26716,26717,26718,26719,26720,26721,26722,26723,26724,26725,26726,26727,26728,26729,26730,26731,26732,26733,26734,26735,26736,26737,26738,26739,26740,26741,26742,26743,26744,26745,26746,26747,26748,26749,26750,26751,26752,26753,26754,26755,26756,26757,26758,26759,26760,26761,26762,26763,26764,26765,26766,26767,26768,26769,26770,26771,26772,26773,26774,26775,26776,26777,26778,26779,26780,26781,26782,26783,26784,26785,26786,26787,26788,26789,26790,26791,26792,26793,26794,26795,26796,26797,26798,26799,26800,26801,26802,26803,26804,26805,26806,26807,26808,26809,26810,26811,26812,26813,26814,26815,26816,26817,26818,26819,26820,26821,26822,26823,26824,26825,26826,26827,26828,26829,26830,26831,26832,26833,26834,26835,26836,26837,26838,26839,26840,26841,26842,26843,26844,26845,26846,26847,26848,26849,26850,26851,26852,26853,26854,26855,26856,26857,26858,26859,26860,26861,26862,26863,26864,26865,26866,26867,26868,26869,26870,26871,26872,26873,26874,26875,26876,26877,26878,26879,26880,26881,26882,26883,26884,26885,26886,26887,26888,26889,26890,26891,26892,26893,26894,26895,26896,26897,26898,26899,26900,26901,26902,26903,26904,26905,26906,26907,26908,26909,26910,26911,26912,26913,26914,26915,26916,26917,26918,26919,26920,26921,26922,26923,26924,26925,26926,26927,26928,26929,26930,26931,26932,26933,26934,26935,26936,26937,26938,26939,26940,26941,26942,26943,26944,26945,26946,26947,26948,26949,26950,26951,26952,26953,26954,26955,26956,26957,26958,26959,26960,26961,26962,26963,26964,26965,26966,26967,26968,26969,26970,26971,26972,26973,26974,26975,26976,26977,26978,26979,26980,26981,26982,26983,26984,26985,26986,26987,26988,26989,26990,26991,26992,26993,26994,26995,26996,26997,26998,26999,27000,27001,27002,27003,27004,27005,27006,27007,27008,27009,27010,27011,27012,27013,27014,27015,27016,27017,27018,27019,27020,27021,27022,27023,27024,27025,27026,27027,27028,27029,27030,27031,27032,27033,27034,27035,27036,27037,27038,27039,27040,27041,27042,27043,27044,27045,27046,27047,27048,27049,27050,27051,27052,27053,27054,27055,27056,27057,27058,27059,27060,27061,27062,27063,27064,27065,27066,27067,27068,27069,27070,27071,27072,27073,27074,27075,27076,27077,27078,27079,27080,27081,27082,27083,27084,27085,27086,27087,27088,27089,27090,27091,27092,27093,27094,27095,27096,27097,27098,27099,27100,27101,27102,27103,27104,27105,27106,27107,27108,27109,27110,27111,27112,27113,27114,27115,27116,27117,27118,27119,27120,27121,27122,27123,27124,27125,27126,27127,27128,27129,27130,27131,27132,27133,27134,27135,27136,27137,27138,27139,27140,27141,27142,27143,27144,27145,27146,27147,27148,27149,27150,27151,27152,27153,27154,27155,27156,27157,27158,27159,27160,27161,27162,27163,27164,27165,27166,27167,27168,27169,27170,27171,27172,27173,27174,27175,27176,27177,27178,27179,27180,27181,27182,27183,27184,27185,27186,27187,27188,27189,27190,27191,27192,27193,27194,27195,27196,27197,27198,27199,27200,27201,27202,27203,27204,27205,27206,27207,27208,27209,27210,27211,27212,27213,27214,27215,27216,27217,27218,27219,27220,27221,27222,27223,27224,27225,27226,27227,27228,27229,27230,27231,27232,27233,27234,27235,27236,27237,27238,27239,27240,27241,27242,27243,27244,27245,27246,27247,27248,27249,27250,27251,27252,27253,27254,27255,27256,27257,27258,27259,27260,27261,27262,27263,27264,27265,27266,27267,27268,27269,27270,27271,27272,27273,27274,27275,27276,27277,27278,27279,27280,27281,27282,27283,27284,27285,27286,27287,27288,27289,27290,27291,27292,27293,27294,27295,27296,27297,27298,27299,27300,27301,27302,27303,27304,27305,27306,27307,27308,27309,27310,27311,27312,27313,27314,27315,27316,27317,27318,27319,27320,27321,27322,27323,27324,27325,27326,27327,27328,27329,27330,27331,27332,27333,27334,27335,27336,27337,27338,27339,27340,27341,27342,27343,27344,27345,27346,27347,27348,27349,27350,27351,27352,27353,27354,27355,27356,27357,27358,27359,27360,27361,27362,27363,27364,27365,27366,27367,27368,27369,27370,27371,27372,27373,27374,27375,27376,27377,27378,27379,27380,27381,27382,27383,27384,27385,27386,27387,27388,27389,27390,27391,27392,27393,27394,27395,27396,27397,27398,27399,27400,27401,27402,27403,27404,27405,27406,27407,27408,27409,27410,27411,27412,27413,27414,27415,27416,27417,27418,27419,27420,27421,27422,27423,27424,27425,27426,27427,27428,27429,27430,27431,27432,27433,27434,27435,27436,27437,27438,27439,27440,27441,27442,27443,27444,27445,27446,27447,27448,27449,27450,27451,27452,27453,27454,27455,27456,27457,27458,27459,27460,27461,27462,27463,27464,27465,27466,27467,27468,27469,27470,27471,27472,27473,27474,27475,27476,27477,27478,27479,27480,27481,27482,27483,27484,27485,27486,27487,27488,27489,27490,27491,27492,27493,27494,27495,27496,27497,27498,27499,27500,27501,27502,27503,27504,27505,27506,27507,27508,27509,27510,27511,27512,27513,27514,27515,27516,27517,27518,27519,27520,27521,27522,27523,27524,27525,27526,27527,27528,27529,27530,27531,27532,27533,27534,27535,27536,27537,27538,27539,27540,27541,27542,27543,27544,27545,27546,27547,27548,27549,27550,27551,27552,27553,27554,27555,27556,27557,27558,27559,27560,27561,27562,27563,27564,27565,27566,27567,27568,27569,27570,27571,27572,27573,27574,27575,27576,27577,27578,27579,27580,27581,27582,27583,27584,27585,27586,27587,27588,27589,27590,27591,27592,27593,27594,27595,27596,27597,27598,27599,27600,27601,27602,27603,27604,27605,27606,27607,27608,27609,27610,27611,27612,27613,27614,27615,27616,27617,27618,27619,27620,27621,27622,27623,27624,27625,27626,27627,27628,27629,27630,27631,27632,27633,27634,27635,27636,27637,27638,27639,27640,27641,27642,27643,27644,27645,27646,27647,27648,27649,27650,27651,27652,27653,27654,27655,27656,27657,27658,27659,27660,27661,27662,27663,27664,27665,27666,27667,27668,27669,27670,27671,27672,27673,27674,27675,27676,27677,27678,27679,27680,27681,27682,27683,27684,27685,27686,27687,27688,27689,27690,27691,27692,27693,27694,27695,27696,27697,27698,27699,27700,27701,27702,27703,27704,27705,27706,27707,27708,27709,27710,27711,27712,27713,27714,27715,27716,27717,27718,27719,27720,27721,27722,27723,27724,27725,27726,27727,27728,27729,27730,27731,27732,27733,27734,27735,27736,27737,27738,27739,27740,27741,27742,27743,27744,27745,27746,27747,27748,27749,27750,27751,27752,27753,27754,27755,27756,27757,27758,27759,27760,27761,27762,27763,27764,27765,27766,27767,27768,27769,27770,27771,27772,27773,27774,27775,27776,27777,27778,27779,27780,27781,27782,27783,27784,27785,27786,27787,27788,27789,27790,27791,27792,27793,27794,27795,27796,27797,27798,27799,27800,27801,27802,27803,27804,27805,27806,27807,27808,27809,27810,27811,27812,27813,27814,27815,27816,27817,27818,27819,27820,27821,27822,27823,27824,27825,27826,27827,27828,27829,27830,27831,27832,27833,27834,27835,27836,27837,27838,27839,27840,27841,27842,27843,27844,27845,27846,27847,27848,27849,27850,27851,27852,27853,27854,27855,27856,27857,27858,27859,27860,27861,27862,27863,27864,27865,27866,27867,27868,27869,27870,27871,27872,27873,27874,27875,27876,27877,27878,27879,27880,27881,27882,27883,27884,27885,27886,27887,27888,27889,27890,27891,27892,27893,27894,27895,27896,27897,27898,27899,27900,27901,27902,27903,27904,27905,27906,27907,27908,27909,27910,27911,27912,27913,27914,27915,27916,27917,27918,27919,27920,27921,27922,27923,27924,27925,27926,27927,27928,27929,27930,27931,27932,27933,27934,27935,27936,27937,27938,27939,27940,27941,27942,27943,27944,27945,27946,27947,27948,27949,27950,27951,27952,27953,27954,27955,27956,27957,27958,27959,27960,27961,27962,27963,27964,27965,27966,27967,27968,27969,27970,27971,27972,27973,27974,27975,27976,27977,27978,27979,27980,27981,27982,27983,27984,27985,27986,27987,27988,27989,27990,27991,27992,27993,27994,27995,27996,27997,27998,27999,28000,28001,28002,28003,28004,28005,28006,28007,28008,28009,28010,28011,28012,28013,28014,28015,28016,28017,28018,28019,28020,28021,28022,28023,28024,28025,28026,28027,28028,28029,28030,28031,28032,28033,28034,28035,28036,28037,28038,28039,28040,28041,28042,28043,28044,28045,28046,28047,28048,28049,28050,28051,28052,28053,28054,28055,28056,28057,28058,28059,28060,28061,28062,28063,28064,28065,28066,28067,28068,28069,28070,28071,28072,28073,28074,28075,28076,28077,28078,28079,28080,28081,28082,28083,28084,28085,28086,28087,28088,28089,28090,28091,28092,28093,28094,28095,28096,28097,28098,28099,28100,28101,28102,28103,28104,28105,28106,28107,28108,28109,28110,28111,28112,28113,28114,28115,28116,28117,28118,28119,28120,28121,28122,28123,28124,28125,28126,28127,28128,28129,28130,28131,28132,28133,28134,28135,28136,28137,28138,28139,28140,28141,28142,28143,28144,28145,28146,28147,28148,28149,28150,28151,28152,28153,28154,28155,28156,28157,28158,28159,28160,28161,28162,28163,28164,28165,28166,28167,28168,28169,28170,28171,28172,28173,28174,28175,28176,28177,28178,28179,28180,28181,28182,28183,28184,28185,28186,28187,28188,28189,28190,28191,28192,28193,28194,28195,28196,28197,28198,28199,28200,28201,28202,28203,28204,28205,28206,28207,28208,28209,28210,28211,28212,28213,28214,28215,28216,28217,28218,28219,28220,28221,28222,28223,28224,28225,28226,28227,28228,28229,28230,28231,28232,28233,28234,28235,28236,28237,28238,28239,28240,28241,28242,28243,28244,28245,28246,28247,28248,28249,28250,28251,28252,28253,28254,28255,28256,28257,28258,28259,28260,28261,28262,28263,28264,28265,28266,28267,28268,28269,28270,28271,28272,28273,28274,28275,28276,28277,28278,28279,28280,28281,28282,28283,28284,28285,28286,28287,28288,28289,28290,28291,28292,28293,28294,28295,28296,28297,28298,28299,28300,28301,28302,28303,28304,28305,28306,28307,28308,28309,28310,28311,28312,28313,28314,28315,28316,28317,28318,28319,28320,28321,28322,28323,28324,28325,28326,28327,28328,28329,28330,28331,28332,28333,28334,28335,28336,28337,28338,28339,28340,28341,28342,28343,28344,28345,28346,28347,28348,28349,28350,28351,28352,28353,28354,28355,28356,28357,28358,28359,28360,28361,28362,28363,28364,28365,28366,28367,28368,28369,28370,28371,28372,28373,28374,28375,28376,28377,28378,28379,28380,28381,28382,28383,28384,28385,28386,28387,28388,28389,28390,28391,28392,28393,28394,28395,28396,28397,28398,28399,28400,28401,28402,28403,28404,28405,28406,28407,28408,28409,28410,28411,28412,28413,28414,28415,28416,28417,28418,28419,28420,28421,28422,28423,28424,28425,28426,28427,28428,28429,28430,28431,28432,28433,28434,28435,28436,28437,28438,28439,28440,28441,28442,28443,28444,28445,28446,28447,28448,28449,28450,28451,28452,28453,28454,28455,28456,28457,28458,28459,28460,28461,28462,28463,28464,28465,28466,28467,28468,28469,28470,28471,28472,28473,28474,28475,28476,28477,28478,28479,28480,28481,28482,28483,28484,28485,28486,28487,28488,28489,28490,28491,28492,28493,28494,28495,28496,28497,28498,28499,28500,28501,28502,28503,28504,28505,28506,28507,28508,28509,28510,28511,28512,28513,28514,28515,28516,28517,28518,28519,28520,28521,28522,28523,28524,28525,28526,28527,28528,28529,28530,28531,28532,28533,28534,28535,28536,28537,28538,28539,28540,28541,28542,28543,28544,28545,28546,28547,28548,28549,28550,28551,28552,28553,28554,28555,28556,28557,28558,28559,28560,28561,28562,28563,28564,28565,28566,28567,28568,28569,28570,28571,28572,28573,28574,28575,28576,28577,28578,28579,28580,28581,28582,28583,28584,28585,28586,28587,28588,28589,28590,28591,28592,28593,28594,28595,28596,28597,28598,28599,28600,28601,28602,28603,28604,28605,28606,28607,28608,28609,28610,28611,28612,28613,28614,28615,28616,28617,28618,28619,28620,28621,28622,28623,28624,28625,28626,28627,28628,28629,28630,28631,28632,28633,28634,28635,28636,28637,28638,28639,28640,28641,28642,28643,28644,28645,28646,28647,28648,28649,28650,28651,28652,28653,28654,28655,28656,28657,28658,28659,28660,28661,28662,28663,28664,28665,28666,28667,28668,28669,28670,28671,28672,28673,28674,28675,28676,28677,28678,28679,28680,28681,28682,28683,28684,28685,28686,28687,28688,28689,28690,28691,28692,28693,28694,28695,28696,28697,28698,28699,28700,28701,28702,28703,28704,28705,28706,28707,28708,28709,28710,28711,28712,28713,28714,28715,28716,28717,28718,28719,28720,28721,28722,28723,28724,28725,28726,28727,28728,28729,28730,28731,28732,28733,28734,28735,28736,28737,28738,28739,28740,28741,28742,28743,28744,28745,28746,28747,28748,28749,28750,28751,28752,28753,28754,28755,28756,28757,28758,28759,28760,28761,28762,28763,28764,28765,28766,28767,28768,28769,28770,28771,28772,28773,28774,28775,28776,28777,28778,28779,28780,28781,28782,28783,28784,28785,28786,28787,28788,28789,28790,28791,28792,28793,28794,28795,28796,28797,28798,28799,28800,28801,28802,28803,28804,28805,28806,28807,28808,28809,28810,28811,28812,28813,28814,28815,28816,28817,28818,28819,28820,28821,28822,28823,28824,28825,28826,28827,28828,28829,28830,28831,28832,28833,28834,28835,28836,28837,28838,28839,28840,28841,28842,28843,28844,28845,28846,28847,28848,28849,28850,28851,28852,28853,28854,28855,28856,28857,28858,28859,28860,28861,28862,28863,28864,28865,28866,28867,28868,28869,28870,28871,28872,28873,28874,28875,28876,28877,28878,28879,28880,28881,28882,28883,28884,28885,28886,28887,28888,28889,28890,28891,28892,28893,28894,28895,28896,28897,28898,28899,28900,28901,28902,28903,28904,28905,28906,28907,28908,28909,28910,28911,28912,28913,28914,28915,28916,28917,28918,28919,28920,28921,28922,28923,28924,28925,28926,28927,28928,28929,28930,28931,28932,28933,28934,28935,28936,28937,28938,28939,28940,28941,28942,28943,28944,28945,28946,28947,28948,28949,28950,28951,28952,28953,28954,28955,28956,28957,28958,28959,28960,28961,28962,28963,28964,28965,28966,28967,28968,28969,28970,28971,28972,28973,28974,28975,28976,28977,28978,28979,28980,28981,28982,28983,28984,28985,28986,28987,28988,28989,28990,28991,28992,28993,28994,28995,28996,28997,28998,28999,29000,29001,29002,29003,29004,29005,29006,29007,29008,29009,29010,29011,29012,29013,29014,29015,29016,29017,29018,29019,29020,29021,29022,29023,29024,29025,29026,29027,29028,29029,29030,29031,29032,29033,29034,29035,29036,29037,29038,29039,29040,29041,29042,29043,29044,29045,29046,29047,29048,29049,29050,29051,29052,29053,29054,29055,29056,29057,29058,29059,29060,29061,29062,29063,29064,29065,29066,29067,29068,29069,29070,29071,29072,29073,29074,29075,29076,29077,29078,29079,29080,29081,29082,29083,29084,29085,29086,29087,29088,29089,29090,29091,29092,29093,29094,29095,29096,29097,29098,29099,29100,29101,29102,29103,29104,29105,29106,29107,29108,29109,29110,29111,29112,29113,29114,29115,29116,29117,29118,29119,29120,29121,29122,29123,29124,29125,29126,29127,29128,29129,29130,29131,29132,29133,29134,29135,29136,29137,29138,29139,29140,29141,29142,29143,29144,29145,29146,29147,29148,29149,29150,29151,29152,29153,29154,29155,29156,29157,29158,29159,29160,29161,29162,29163,29164,29165,29166,29167,29168,29169,29170,29171,29172,29173,29174,29175,29176,29177,29178,29179,29180,29181,29182,29183,29184,29185,29186,29187,29188,29189,29190,29191,29192,29193,29194,29195,29196,29197,29198,29199,29200,29201,29202,29203,29204,29205,29206,29207,29208,29209,29210,29211,29212,29213,29214,29215,29216,29217,29218,29219,29220,29221,29222,29223,29224,29225,29226,29227,29228,29229,29230,29231,29232,29233,29234,29235,29236,29237,29238,29239,29240,29241,29242,29243,29244,29245,29246,29247,29248,29249,29250,29251,29252,29253,29254,29255,29256,29257,29258,29259,29260,29261,29262,29263,29264,29265,29266,29267,29268,29269,29270,29271,29272,29273,29274,29275,29276,29277,29278,29279,29280,29281,29282,29283,29284,29285,29286,29287,29288,29289,29290,29291,29292,29293,29294,29295,29296,29297,29298,29299,29300,29301,29302,29303,29304,29305,29306,29307,29308,29309,29310,29311,29312,29313,29314,29315,29316,29317,29318,29319,29320,29321,29322,29323,29324,29325,29326,29327,29328,29329,29330,29331,29332,29333,29334,29335,29336,29337,29338,29339,29340,29341,29342,29343,29344,29345,29346,29347,29348,29349,29350,29351,29352,29353,29354,29355,29356,29357,29358,29359,29360,29361,29362,29363,29364,29365,29366,29367,29368,29369,29370,29371,29372,29373,29374,29375,29376,29377,29378,29379,29380,29381,29382,29383,29384,29385,29386,29387,29388,29389,29390,29391,29392,29393,29394,29395,29396,29397,29398,29399,29400,29401,29402,29403,29404,29405,29406,29407,29408,29409,29410,29411,29412,29413,29414,29415,29416,29417,29418,29419,29420,29421,29422,29423,29424,29425,29426,29427,29428,29429,29430,29431,29432,29433,29434,29435,29436,29437,29438,29439,29440,29441,29442,29443,29444,29445,29446,29447,29448,29449,29450,29451,29452,29453,29454,29455,29456,29457,29458,29459,29460,29461,29462,29463,29464,29465,29466,29467,29468,29469,29470,29471,29472,29473,29474,29475,29476,29477,29478,29479,29480,29481,29482,29483,29484,29485,29486,29487,29488,29489,29490,29491,29492,29493,29494,29495,29496,29497,29498,29499,29500,29501,29502,29503,29504,29505,29506,29507,29508,29509,29510,29511,29512,29513,29514,29515,29516,29517,29518,29519,29520,29521,29522,29523,29524,29525,29526,29527,29528,29529,29530,29531,29532,29533,29534,29535,29536,29537,29538,29539,29540,29541,29542,29543,29544,29545,29546,29547,29548,29549,29550,29551,29552,29553,29554,29555,29556,29557,29558,29559,29560,29561,29562,29563,29564,29565,29566,29567,29568,29569,29570,29571,29572,29573,29574,29575,29576,29577,29578,29579,29580,29581,29582,29583,29584,29585,29586,29587,29588,29589,29590,29591,29592,29593,29594,29595,29596,29597,29598,29599,29600,29601,29602,29603,29604,29605,29606,29607,29608,29609,29610,29611,29612,29613,29614,29615,29616,29617,29618,29619,29620,29621,29622,29623,29624,29625,29626,29627,29628,29629,29630,29631,29632,29633,29634,29635,29636,29637,29638,29639,29640,29641,29642,29643,29644,29645,29646,29647,29648,29649,29650,29651,29652,29653,29654,29655,29656,29657,29658,29659,29660,29661,29662,29663,29664,29665,29666,29667,29668,29669,29670,29671,29672,29673,29674,29675,29676,29677,29678,29679,29680,29681,29682,29683,29684,29685,29686,29687,29688,29689,29690,29691,29692,29693,29694,29695,29696,29697,29698,29699,29700,29701,29702,29703,29704,29705,29706,29707,29708,29709,29710,29711,29712,29713,29714,29715,29716,29717,29718,29719,29720,29721,29722,29723,29724,29725,29726,29727,29728,29729,29730,29731,29732,29733,29734,29735,29736,29737,29738,29739,29740,29741,29742,29743,29744,29745,29746,29747,29748,29749,29750,29751,29752,29753,29754,29755,29756,29757,29758,29759,29760,29761,29762,29763,29764,29765,29766,29767,29768,29769,29770,29771,29772,29773,29774,29775,29776,29777,29778,29779,29780,29781,29782,29783,29784,29785,29786,29787,29788,29789,29790,29791,29792,29793,29794,29795,29796,29797,29798,29799,29800,29801,29802,29803,29804,29805,29806,29807,29808,29809,29810,29811,29812,29813,29814,29815,29816,29817,29818,29819,29820,29821,29822,29823,29824,29825,29826,29827,29828,29829,29830,29831,29832,29833,29834,29835,29836,29837,29838,29839,29840,29841,29842,29843,29844,29845,29846,29847,29848,29849,29850,29851,29852,29853,29854,29855,29856,29857,29858,29859,29860,29861,29862,29863,29864,29865,29866,29867,29868,29869,29870,29871,29872,29873,29874,29875,29876,29877,29878,29879,29880,29881,29882,29883,29884,29885,29886,29887,29888,29889,29890,29891,29892,29893,29894,29895,29896,29897,29898,29899,29900,29901,29902,29903,29904,29905,29906,29907,29908,29909,29910,29911,29912,29913,29914,29915,29916,29917,29918,29919,29920,29921,29922,29923,29924,29925,29926,29927,29928,29929,29930,29931,29932,29933,29934,29935,29936,29937,29938,29939,29940,29941,29942,29943,29944,29945,29946,29947,29948,29949,29950,29951,29952,29953,29954,29955,29956,29957,29958,29959,29960,29961,29962,29963,29964,29965,29966,29967,29968,29969,29970,29971,29972,29973,29974,29975,29976,29977,29978,29979,29980,29981,29982,29983,29984,29985,29986,29987,29988,29989,29990,29991,29992,29993,29994,29995,29996,29997,29998,29999,30000,30001,30002,30003,30004,30005,30006,30007,30008,30009,30010,30011,30012,30013,30014,30015,30016,30017,30018,30019,30020,30021,30022,30023,30024,30025,30026,30027,30028,30029,30030,30031,30032,30033,30034,30035,30036,30037,30038,30039,30040,30041,30042,30043,30044,30045,30046,30047,30048,30049,30050,30051,30052,30053,30054,30055,30056,30057,30058,30059,30060,30061,30062,30063,30064,30065,30066,30067,30068,30069,30070,30071,30072,30073,30074,30075,30076,30077,30078,30079,30080,30081,30082,30083,30084,30085,30086,30087,30088,30089,30090,30091,30092,30093,30094,30095,30096,30097,30098,30099,30100,30101,30102,30103,30104,30105,30106,30107,30108,30109,30110,30111,30112,30113,30114,30115,30116,30117,30118,30119,30120,30121,30122,30123,30124,30125,30126,30127,30128,30129,30130,30131,30132,30133,30134,30135,30136,30137,30138,30139,30140,30141,30142,30143,30144,30145,30146,30147,30148,30149,30150,30151,30152,30153,30154,30155,30156,30157,30158,30159,30160,30161,30162,30163,30164,30165,30166,30167,30168,30169,30170,30171,30172,30173,30174,30175,30176,30177,30178,30179,30180,30181,30182,30183,30184,30185,30186,30187,30188,30189,30190,30191,30192,30193,30194,30195,30196,30197,30198,30199,30200,30201,30202,30203,30204,30205,30206,30207,30208,30209,30210,30211,30212,30213,30214,30215,30216,30217,30218,30219,30220,30221,30222,30223,30224,30225,30226,30227,30228,30229,30230,30231,30232,30233,30234,30235,30236,30237,30238,30239,30240,30241,30242,30243,30244,30245,30246,30247,30248,30249,30250,30251,30252,30253,30254,30255,30256,30257,30258,30259,30260,30261,30262,30263,30264,30265,30266,30267,30268,30269,30270,30271,30272,30273,30274,30275,30276,30277,30278,30279,30280,30281,30282,30283,30284,30285,30286,30287,30288,30289,30290,30291,30292,30293,30294,30295,30296,30297,30298,30299,30300,30301,30302,30303,30304,30305,30306,30307,30308,30309,30310,30311,30312,30313,30314,30315,30316,30317,30318,30319,30320,30321,30322,30323,30324,30325,30326,30327,30328,30329,30330,30331,30332,30333,30334,30335,30336,30337,30338,30339,30340,30341,30342,30343,30344,30345,30346,30347,30348,30349,30350,30351,30352,30353,30354,30355,30356,30357,30358,30359,30360,30361,30362,30363,30364,30365,30366,30367,30368,30369,30370,30371,30372,30373,30374,30375,30376,30377,30378,30379,30380,30381,30382,30383,30384,30385,30386,30387,30388,30389,30390,30391,30392,30393,30394,30395,30396,30397,30398,30399,30400,30401,30402,30403,30404,30405,30406,30407,30408,30409,30410,30411,30412,30413,30414,30415,30416,30417,30418,30419,30420,30421,30422,30423,30424,30425,30426,30427,30428,30429,30430,30431,30432,30433,30434,30435,30436,30437,30438,30439,30440,30441,30442,30443,30444,30445,30446,30447,30448,30449,30450,30451,30452,30453,30454,30455,30456,30457,30458,30459,30460,30461,30462,30463,30464,30465,30466,30467,30468,30469,30470,30471,30472,30473,30474,30475,30476,30477,30478,30479,30480,30481,30482,30483,30484,30485,30486,30487,30488,30489,30490,30491,30492,30493,30494,30495,30496,30497,30498,30499,30500,30501,30502,30503,30504,30505,30506,30507,30508,30509,30510,30511,30512,30513,30514,30515,30516,30517,30518,30519,30520,30521,30522,30523,30524,30525,30526,30527,30528,30529,30530,30531,30532,30533,30534,30535,30536,30537,30538,30539,30540,30541,30542,30543,30544,30545,30546,30547,30548,30549,30550,30551,30552,30553,30554,30555,30556,30557,30558,30559,30560,30561,30562,30563,30564,30565,30566,30567,30568,30569,30570,30571,30572,30573,30574,30575,30576,30577,30578,30579,30580,30581,30582,30583,30584,30585,30586,30587,30588,30589,30590,30591,30592,30593,30594,30595,30596,30597,30598,30599,30600,30601,30602,30603,30604,30605,30606,30607,30608,30609,30610,30611,30612,30613,30614,30615,30616,30617,30618,30619,30620,30621,30622,30623,30624,30625,30626,30627,30628,30629,30630,30631,30632,30633,30634,30635,30636,30637,30638,30639,30640,30641,30642,30643,30644,30645,30646,30647,30648,30649,30650,30651,30652,30653,30654,30655,30656,30657,30658,30659,30660,30661,30662,30663,30664,30665,30666,30667,30668,30669,30670,30671,30672,30673,30674,30675,30676,30677,30678,30679,30680,30681,30682,30683,30684,30685,30686,30687,30688,30689,30690,30691,30692,30693,30694,30695,30696,30697,30698,30699,30700,30701,30702,30703,30704,30705,30706,30707,30708,30709,30710,30711,30712,30713,30714,30715,30716,30717,30718,30719,30720,30721,30722,30723,30724,30725,30726,30727,30728,30729,30730,30731,30732,30733,30734,30735,30736,30737,30738,30739,30740,30741,30742,30743,30744,30745,30746,30747,30748,30749,30750,30751,30752,30753,30754,30755,30756,30757,30758,30759,30760,30761,30762,30763,30764,30765,30766,30767,30768,30769,30770,30771,30772,30773,30774,30775,30776,30777,30778,30779,30780,30781,30782,30783,30784,30785,30786,30787,30788,30789,30790,30791,30792,30793,30794,30795,30796,30797,30798,30799,30800,30801,30802,30803,30804,30805,30806,30807,30808,30809,30810,30811,30812,30813,30814,30815,30816,30817,30818,30819,30820,30821,30822,30823,30824,30825,30826,30827,30828,30829,30830,30831,30832,30833,30834,30835,30836,30837,30838,30839,30840,30841,30842,30843,30844,30845,30846,30847,30848,30849,30850,30851,30852,30853,30854,30855,30856,30857,30858,30859,30860,30861,30862,30863,30864,30865,30866,30867,30868,30869,30870,30871,30872,30873,30874,30875,30876,30877,30878,30879,30880,30881,30882,30883,30884,30885,30886,30887,30888,30889,30890,30891,30892,30893,30894,30895,30896,30897,30898,30899,30900,30901,30902,30903,30904,30905,30906,30907,30908,30909,30910,30911,30912,30913,30914,30915,30916,30917,30918,30919,30920,30921,30922,30923,30924,30925,30926,30927,30928,30929,30930,30931,30932,30933,30934,30935,30936,30937,30938,30939,30940,30941,30942,30943,30944,30945,30946,30947,30948,30949,30950,30951,30952,30953,30954,30955,30956,30957,30958,30959,30960,30961,30962,30963,30964,30965,30966,30967,30968,30969,30970,30971,30972,30973,30974,30975,30976,30977,30978,30979,30980,30981,30982,30983,30984,30985,30986,30987,30988,30989,30990,30991,30992,30993,30994,30995,30996,30997,30998,30999,31000,31001,31002,31003,31004,31005,31006,31007,31008,31009,31010,31011,31012,31013,31014,31015,31016,31017,31018,31019,31020,31021,31022,31023,31024,31025,31026,31027,31028,31029,31030,31031,31032,31033,31034,31035,31036,31037,31038,31039,31040,31041,31042,31043,31044,31045,31046,31047,31048,31049,31050,31051,31052,31053,31054,31055,31056,31057,31058,31059,31060,31061,31062,31063,31064,31065,31066,31067,31068,31069,31070,31071,31072,31073,31074,31075,31076,31077,31078,31079,31080,31081,31082,31083,31084,31085,31086,31087,31088,31089,31090,31091,31092,31093,31094,31095,31096,31097,31098,31099,31100,31101,31102,31103,31104,31105,31106,31107,31108,31109,31110,31111,31112,31113,31114,31115,31116,31117,31118,31119,31120,31121,31122,31123,31124,31125,31126,31127,31128,31129,31130,31131,31132,31133,31134,31135,31136,31137,31138,31139,31140,31141,31142,31143,31144,31145,31146,31147,31148,31149,31150,31151,31152,31153,31154,31155,31156,31157,31158,31159,31160,31161,31162,31163,31164,31165,31166,31167,31168,31169,31170,31171,31172,31173,31174,31175,31176,31177,31178,31179,31180,31181,31182,31183,31184,31185,31186,31187,31188,31189,31190,31191,31192,31193,31194,31195,31196,31197,31198,31199,31200,31201,31202,31203,31204,31205,31206,31207,31208,31209,31210,31211,31212,31213,31214,31215,31216,31217,31218,31219,31220,31221,31222,31223,31224,31225,31226,31227,31228,31229,31230,31231,31232,31233,31234,31235,31236,31237,31238,31239,31240,31241,31242,31243,31244,31245,31246,31247,31248,31249,31250,31251,31252,31253,31254,31255,31256,31257,31258,31259,31260,31261,31262,31263,31264,31265,31266,31267,31268,31269,31270,31271,31272,31273,31274,31275,31276,31277,31278,31279,31280,31281,31282,31283,31284,31285,31286,31287,31288,31289,31290,31291,31292,31293,31294,31295,31296,31297,31298,31299,31300,31301,31302,31303,31304,31305,31306,31307,31308,31309,31310,31311,31312,31313,31314,31315,31316,31317,31318,31319,31320,31321,31322,31323,31324,31325,31326,31327,31328,31329,31330,31331,31332,31333,31334,31335,31336,31337,31338,31339,31340,31341,31342,31343,31344,31345,31346,31347,31348,31349,31350,31351,31352,31353,31354,31355,31356,31357,31358,31359,31360,31361,31362,31363,31364,31365,31366,31367,31368,31369,31370,31371,31372,31373,31374,31375,31376,31377,31378,31379,31380,31381,31382,31383,31384,31385,31386,31387,31388,31389,31390,31391,31392,31393,31394,31395,31396,31397,31398,31399,31400,31401,31402,31403,31404,31405,31406,31407,31408,31409,31410,31411,31412,31413,31414,31415,31416,31417,31418,31419,31420,31421,31422,31423,31424,31425,31426,31427,31428,31429,31430,31431,31432,31433,31434,31435,31436,31437,31438,31439,31440,31441,31442,31443,31444,31445,31446,31447,31448,31449,31450,31451,31452,31453,31454,31455,31456,31457,31458,31459,31460,31461,31462,31463,31464,31465,31466,31467,31468,31469,31470,31471,31472,31473,31474,31475,31476,31477,31478,31479,31480,31481,31482,31483,31484,31485,31486,31487,31488,31489,31490,31491,31492,31493,31494,31495,31496,31497,31498,31499,31500,31501,31502,31503,31504,31505,31506,31507,31508,31509,31510,31511,31512,31513,31514,31515,31516,31517,31518,31519,31520,31521,31522,31523,31524,31525,31526,31527,31528,31529,31530,31531,31532,31533,31534,31535,31536,31537,31538,31539,31540,31541,31542,31543,31544,31545,31546,31547,31548,31549,31550,31551,31552,31553,31554,31555,31556,31557,31558,31559,31560,31561,31562,31563,31564,31565,31566,31567,31568,31569,31570,31571,31572,31573,31574,31575,31576,31577,31578,31579,31580,31581,31582,31583,31584,31585,31586,31587,31588,31589,31590,31591,31592,31593,31594,31595,31596,31597,31598,31599,31600,31601,31602,31603,31604,31605,31606,31607,31608,31609,31610,31611,31612,31613,31614,31615,31616,31617,31618,31619,31620,31621,31622,31623,31624,31625,31626,31627,31628,31629,31630,31631,31632,31633,31634,31635,31636,31637,31638,31639,31640,31641,31642,31643,31644,31645,31646,31647,31648,31649,31650,31651,31652,31653,31654,31655,31656,31657,31658,31659,31660,31661,31662,31663,31664,31665,31666,31667,31668,31669,31670,31671,31672,31673,31674,31675,31676,31677,31678,31679,31680,31681,31682,31683,31684,31685,31686,31687,31688,31689,31690,31691,31692,31693,31694,31695,31696,31697,31698,31699,31700,31701,31702,31703,31704,31705,31706,31707,31708,31709,31710,31711,31712,31713,31714,31715,31716,31717,31718,31719,31720,31721,31722,31723,31724,31725,31726,31727,31728,31729,31730,31731,31732,31733,31734,31735,31736,31737,31738,31739,31740,31741,31742,31743,31744,31745,31746,31747,31748,31749,31750,31751,31752,31753,31754,31755,31756,31757,31758,31759,31760,31761,31762,31763,31764,31765,31766,31767,31768,31769,31770,31771,31772,31773,31774,31775,31776,31777,31778,31779,31780,31781,31782,31783,31784,31785,31786,31787,31788,31789,31790,31791,31792,31793,31794,31795,31796,31797,31798,31799,31800,31801,31802,31803,31804,31805,31806,31807,31808,31809,31810,31811,31812,31813,31814,31815,31816,31817,31818,31819,31820,31821,31822,31823,31824,31825,31826,31827,31828,31829,31830,31831,31832,31833,31834,31835,31836,31837,31838,31839,31840,31841,31842,31843,31844,31845,31846,31847,31848,31849,31850,31851,31852,31853,31854,31855,31856,31857,31858,31859,31860,31861,31862,31863,31864,31865,31866,31867,31868,31869,31870,31871,31872,31873,31874,31875,31876,31877,31878,31879,31880,31881,31882,31883,31884,31885,31886,31887,31888,31889,31890,31891,31892,31893,31894,31895,31896,31897,31898,31899,31900,31901,31902,31903,31904,31905,31906,31907,31908,31909,31910,31911,31912,31913,31914,31915,31916,31917,31918,31919,31920,31921,31922,31923,31924,31925,31926,31927,31928,31929,31930,31931,31932,31933,31934,31935,31936,31937,31938,31939,31940,31941,31942,31943,31944,31945,31946,31947,31948,31949,31950,31951,31952,31953,31954,31955,31956,31957,31958,31959,31960,31961,31962,31963,31964,31965,31966,31967,31968,31969,31970,31971,31972,31973,31974,31975,31976,31977,31978,31979,31980,31981,31982,31983,31984,31985,31986,31987,31988,31989,31990,31991,31992,31993,31994,31995,31996,31997,31998,31999,32000,32001,32002,32003,32004,32005,32006,32007,32008,32009,32010,32011,32012,32013,32014,32015,32016,32017,32018,32019,32020,32021,32022,32023,32024,32025,32026,32027,32028,32029,32030,32031,32032,32033,32034,32035,32036,32037,32038,32039,32040,32041,32042,32043,32044,32045,32046,32047,32048,32049,32050,32051,32052,32053,32054,32055,32056,32057,32058,32059,32060,32061,32062,32063,32064,32065,32066,32067,32068,32069,32070,32071,32072,32073,32074,32075,32076,32077,32078,32079,32080,32081,32082,32083,32084,32085,32086,32087,32088,32089,32090,32091,32092,32093,32094,32095,32096,32097,32098,32099,32100,32101,32102,32103,32104,32105,32106,32107,32108,32109,32110,32111,32112,32113,32114,32115,32116,32117,32118,32119,32120,32121,32122,32123,32124,32125,32126,32127,32128,32129,32130,32131,32132,32133,32134,32135,32136,32137,32138,32139,32140,32141,32142,32143,32144,32145,32146,32147,32148,32149,32150,32151,32152,32153,32154,32155,32156,32157,32158,32159,32160,32161,32162,32163,32164,32165,32166,32167,32168,32169,32170,32171,32172,32173,32174,32175,32176,32177,32178,32179,32180,32181,32182,32183,32184,32185,32186,32187,32188,32189,32190,32191,32192,32193,32194,32195,32196,32197,32198,32199,32200,32201,32202,32203,32204,32205,32206,32207,32208,32209,32210,32211,32212,32213,32214,32215,32216,32217,32218,32219,32220,32221,32222,32223,32224,32225,32226,32227,32228,32229,32230,32231,32232,32233,32234,32235,32236,32237,32238,32239,32240,32241,32242,32243,32244,32245,32246,32247,32248,32249,32250,32251,32252,32253,32254,32255,32256,32257,32258,32259,32260,32261,32262,32263,32264,32265,32266,32267,32268,32269,32270,32271,32272,32273,32274,32275,32276,32277,32278,32279,32280,32281,32282,32283,32284,32285,32286,32287,32288,32289,32290,32291,32292,32293,32294,32295,32296,32297,32298,32299,32300,32301,32302,32303,32304,32305,32306,32307,32308,32309,32310,32311,32312,32313,32314,32315,32316,32317,32318,32319,32320,32321,32322,32323,32324,32325,32326,32327,32328,32329,32330,32331,32332,32333,32334,32335,32336,32337,32338,32339,32340,32341,32342,32343,32344,32345,32346,32347,32348,32349,32350,32351,32352,32353,32354,32355,32356,32357,32358,32359,32360,32361,32362,32363,32364,32365,32366,32367,32368,32369,32370,32371,32372,32373,32374,32375,32376,32377,32378,32379,32380,32381,32382,32383,32384,32385,32386,32387,32388,32389,32390,32391,32392,32393,32394,32395,32396,32397,32398,32399,32400,32401,32402,32403,32404,32405,32406,32407,32408,32409,32410,32411,32412,32413,32414,32415,32416,32417,32418,32419,32420,32421,32422,32423,32424,32425,32426,32427,32428,32429,32430,32431,32432,32433,32434,32435,32436,32437,32438,32439,32440,32441,32442,32443,32444,32445,32446,32447,32448,32449,32450,32451,32452,32453,32454,32455,32456,32457,32458,32459,32460,32461,32462,32463,32464,32465,32466,32467,32468,32469,32470,32471,32472,32473,32474,32475,32476,32477,32478,32479,32480,32481,32482,32483,32484,32485,32486,32487,32488,32489,32490,32491,32492,32493,32494,32495,32496,32497,32498,32499,32500,32501,32502,32503,32504,32505,32506,32507,32508,32509,32510,32511,32512,32513,32514,32515,32516,32517,32518,32519,32520,32521,32522,32523,32524,32525,32526,32527,32528,32529,32530,32531,32532,32533,32534,32535,32536,32537,32538,32539,32540,32541,32542,32543,32544,32545,32546,32547,32548,32549,32550,32551,32552,32553,32554,32555,32556,32557,32558,32559,32560,32561,32562,32563,32564,32565,32566,32567,32568,32569,32570,32571,32572,32573,32574,32575,32576,32577,32578,32579,32580,32581,32582,32583,32584,32585,32586,32587,32588,32589,32590,32591,32592,32593,32594,32595,32596,32597,32598,32599,32600,32601,32602,32603,32604,32605,32606,32607,32608,32609,32610,32611,32612,32613,32614,32615,32616,32617,32618,32619,32620,32621,32622,32623,32624,32625,32626,32627,32628,32629,32630,32631,32632,32633,32634,32635,32636,32637,32638,32639,32640,32641,32642,32643,32644,32645,32646,32647,32648,32649,32650,32651,32652,32653,32654,32655,32656,32657,32658,32659,32660,32661,32662,32663,32664,32665,32666,32667,32668,32669,32670,32671,32672,32673,32674,32675,32676,32677,32678,32679,32680,32681,32682,32683,32684,32685,32686,32687,32688,32689,32690,32691,32692,32693,32694,32695,32696,32697,32698,32699,32700,32701,32702,32703,32704,32705,32706,32707,32708,32709,32710,32711,32712,32713,32714,32715,32716,32717,32718,32719,32720,32721,32722,32723,32724,32725,32726,32727,32728,32729,32730,32731,32732,32733,32734,32735,32736,32737,32738,32739,32740,32741,32742,32743,32744,32745,32746,32747,32748,32749,32750,32751,32752,32753,32754,32755,32756,32757,32758,32759,32760,32761,32762,32763,32764,32765,32766,32767,32768,32769,32770,32771,32772,32773,32774,32775,32776,32777,32778,32779,32780,32781,32782,32783,32784,32785,32786,32787,32788,32789,32790,32791,32792,32793,32794,32795,32796,32797,32798,32799,32800,32801,32802,32803,32804,32805,32806,32807,32808,32809,32810,32811,32812,32813,32814,32815,32816,32817,32818,32819,32820,32821,32822,32823,32824,32825,32826,32827,32828,32829,32830,32831,32832,32833,32834,32835,32836,32837,32838,32839,32840,32841,32842,32843,32844,32845,32846,32847,32848,32849,32850,32851,32852,32853,32854,32855,32856,32857,32858,32859,32860,32861,32862,32863,32864,32865,32866,32867,32868,32869,32870,32871,32872,32873,32874,32875,32876,32877,32878,32879,32880,32881,32882,32883,32884,32885,32886,32887,32888,32889,32890,32891,32892,32893,32894,32895,32896,32897,32898,32899,32900,32901,32902,32903,32904,32905,32906,32907,32908,32909,32910,32911,32912,32913,32914,32915,32916,32917,32918,32919,32920,32921,32922,32923,32924,32925,32926,32927,32928,32929,32930,32931,32932,32933,32934,32935,32936,32937,32938,32939,32940,32941,32942,32943,32944,32945,32946,32947,32948,32949,32950,32951,32952,32953,32954,32955,32956,32957,32958,32959,32960,32961,32962,32963,32964,32965,32966,32967,32968,32969,32970,32971,32972,32973,32974,32975,32976,32977,32978,32979,32980,32981,32982,32983,32984,32985,32986,32987,32988,32989,32990,32991,32992,32993,32994,32995,32996,32997,32998,32999,33000,33001,33002,33003,33004,33005,33006,33007,33008,33009,33010,33011,33012,33013,33014,33015,33016,33017,33018,33019,33020,33021,33022,33023,33024,33025,33026,33027,33028,33029,33030,33031,33032,33033,33034,33035,33036,33037,33038,33039,33040,33041,33042,33043,33044,33045,33046,33047,33048,33049,33050,33051,33052,33053,33054,33055,33056,33057,33058,33059,33060,33061,33062,33063,33064,33065,33066,33067,33068,33069,33070,33071,33072,33073,33074,33075,33076,33077,33078,33079,33080,33081,33082,33083,33084,33085,33086,33087,33088,33089,33090,33091,33092,33093,33094,33095,33096,33097,33098,33099,33100,33101,33102,33103,33104,33105,33106,33107,33108,33109,33110,33111,33112,33113,33114,33115,33116,33117,33118,33119,33120,33121,33122,33123,33124,33125,33126,33127,33128,33129,33130,33131,33132,33133,33134,33135,33136,33137,33138,33139,33140,33141,33142,33143,33144,33145,33146,33147,33148,33149,33150,33151,33152,33153,33154,33155,33156,33157,33158,33159,33160,33161,33162,33163,33164,33165,33166,33167,33168,33169,33170,33171,33172,33173,33174,33175,33176,33177,33178,33179,33180,33181,33182,33183,33184,33185,33186,33187,33188,33189,33190,33191,33192,33193,33194,33195,33196,33197,33198,33199,33200,33201,33202,33203,33204,33205,33206,33207,33208,33209,33210,33211,33212,33213,33214,33215,33216,33217,33218,33219,33220,33221,33222,33223,33224,33225,33226,33227,33228,33229,33230,33231,33232,33233,33234,33235,33236,33237,33238,33239,33240,33241,33242,33243,33244,33245,33246,33247,33248,33249,33250,33251,33252,33253,33254,33255,33256,33257,33258,33259,33260,33261,33262,33263,33264,33265,33266,33267,33268,33269,33270,33271,33272,33273,33274,33275,33276,33277,33278,33279,33280,33281,33282,33283,33284,33285,33286,33287,33288,33289,33290,33291,33292,33293,33294,33295,33296,33297,33298,33299,33300,33301,33302,33303,33304,33305,33306,33307,33308,33309,33310,33311,33312,33313,33314,33315,33316,33317,33318,33319,33320,33321,33322,33323,33324,33325,33326,33327,33328,33329,33330,33331,33332,33333,33334,33335,33336,33337,33338,33339,33340,33341,33342,33343,33344,33345,33346,33347,33348,33349,33350,33351,33352,33353,33354,33355,33356,33357,33358,33359,33360,33361,33362,33363,33364,33365,33366,33367,33368,33369,33370,33371,33372,33373,33374,33375,33376,33377,33378,33379,33380,33381,33382,33383,33384,33385,33386,33387,33388,33389,33390,33391,33392,33393,33394,33395,33396,33397,33398,33399,33400,33401,33402,33403,33404,33405,33406,33407,33408,33409,33410,33411,33412,33413,33414,33415,33416,33417,33418,33419,33420,33421,33422,33423,33424,33425,33426,33427,33428,33429,33430,33431,33432,33433,33434,33435,33436,33437,33438,33439,33440,33441,33442,33443,33444,33445,33446,33447,33448,33449,33450,33451,33452,33453,33454,33455,33456,33457,33458,33459,33460,33461,33462,33463,33464,33465,33466,33467,33468,33469,33470,33471,33472,33473,33474,33475,33476,33477,33478,33479,33480,33481,33482,33483,33484,33485,33486,33487,33488,33489,33490,33491,33492,33493,33494,33495,33496,33497,33498,33499,33500,33501,33502,33503,33504,33505,33506,33507,33508,33509,33510,33511,33512,33513,33514,33515,33516,33517,33518,33519,33520,33521,33522,33523,33524,33525,33526,33527,33528,33529,33530,33531,33532,33533,33534,33535,33536,33537,33538,33539,33540,33541,33542,33543,33544,33545,33546,33547,33548,33549,33550,33551,33552,33553,33554,33555,33556,33557,33558,33559,33560,33561,33562,33563,33564,33565,33566,33567,33568,33569,33570,33571,33572,33573,33574,33575,33576,33577,33578,33579,33580,33581,33582,33583,33584,33585,33586,33587,33588,33589,33590,33591,33592,33593,33594,33595,33596,33597,33598,33599,33600,33601,33602,33603,33604,33605,33606,33607,33608,33609,33610,33611,33612,33613,33614,33615,33616,33617,33618,33619,33620,33621,33622,33623,33624,33625,33626,33627,33628,33629,33630,33631,33632,33633,33634,33635,33636,33637,33638,33639,33640,33641,33642,33643,33644,33645,33646,33647,33648,33649,33650,33651,33652,33653,33654,33655,33656,33657,33658,33659,33660,33661,33662,33663,33664,33665,33666,33667,33668,33669,33670,33671,33672,33673,33674,33675,33676,33677,33678,33679,33680,33681,33682,33683,33684,33685,33686,33687,33688,33689,33690,33691,33692,33693,33694,33695,33696,33697,33698,33699,33700,33701,33702,33703,33704,33705,33706,33707,33708,33709,33710,33711,33712,33713,33714,33715,33716,33717,33718,33719,33720,33721,33722,33723,33724,33725,33726,33727,33728,33729,33730,33731,33732,33733,33734,33735,33736,33737,33738,33739,33740,33741,33742,33743,33744,33745,33746,33747,33748,33749,33750,33751,33752,33753,33754,33755,33756,33757,33758,33759,33760,33761,33762,33763,33764,33765,33766,33767,33768,33769,33770,33771,33772,33773,33774,33775,33776,33777,33778,33779,33780,33781,33782,33783,33784,33785,33786,33787,33788,33789,33790,33791,33792,33793,33794,33795,33796,33797,33798,33799,33800,33801,33802,33803,33804,33805,33806,33807,33808,33809,33810,33811,33812,33813,33814,33815,33816,33817,33818,33819,33820,33821,33822,33823,33824,33825,33826,33827,33828,33829,33830,33831,33832,33833,33834,33835,33836,33837,33838,33839,33840,33841,33842,33843,33844,33845,33846,33847,33848,33849,33850,33851,33852,33853,33854,33855,33856,33857,33858,33859,33860,33861,33862,33863,33864,33865,33866,33867,33868,33869,33870,33871,33872,33873,33874,33875,33876,33877,33878,33879,33880,33881,33882,33883,33884,33885,33886,33887,33888,33889,33890,33891,33892,33893,33894,33895,33896,33897,33898,33899,33900,33901,33902,33903,33904,33905,33906,33907,33908,33909,33910,33911,33912,33913,33914,33915,33916,33917,33918,33919,33920,33921,33922,33923,33924,33925,33926,33927,33928,33929,33930,33931,33932,33933,33934,33935,33936,33937,33938,33939,33940,33941,33942,33943,33944,33945,33946,33947,33948,33949,33950,33951,33952,33953,33954,33955,33956,33957,33958,33959,33960,33961,33962,33963,33964,33965,33966,33967,33968,33969,33970,33971,33972,33973,33974,33975,33976,33977,33978,33979,33980,33981,33982,33983,33984,33985,33986,33987,33988,33989,33990,33991,33992,33993,33994,33995,33996,33997,33998,33999,34000,34001,34002,34003,34004,34005,34006,34007,34008,34009,34010,34011,34012,34013,34014,34015,34016,34017,34018,34019,34020,34021,34022,34023,34024,34025,34026,34027,34028,34029,34030,34031,34032,34033,34034,34035,34036,34037,34038,34039,34040,34041,34042,34043,34044,34045,34046,34047,34048,34049,34050,34051,34052,34053,34054,34055,34056,34057,34058,34059,34060,34061,34062,34063,34064,34065,34066,34067,34068,34069,34070,34071,34072,34073,34074,34075,34076,34077,34078,34079,34080,34081,34082,34083,34084,34085,34086,34087,34088,34089,34090,34091,34092,34093,34094,34095,34096,34097,34098,34099,34100,34101,34102,34103,34104,34105,34106,34107,34108,34109,34110,34111,34112,34113,34114,34115,34116,34117,34118,34119,34120,34121,34122,34123,34124,34125,34126,34127,34128,34129,34130,34131,34132,34133,34134,34135,34136,34137,34138,34139,34140,34141,34142,34143,34144,34145,34146,34147,34148,34149,34150,34151,34152,34153,34154,34155,34156,34157,34158,34159,34160,34161,34162,34163,34164,34165,34166,34167,34168,34169,34170,34171,34172,34173,34174,34175,34176,34177,34178,34179,34180,34181,34182,34183,34184,34185,34186,34187,34188,34189,34190,34191,34192,34193,34194,34195,34196,34197,34198,34199,34200,34201,34202,34203,34204,34205,34206,34207,34208,34209,34210,34211,34212,34213,34214,34215,34216,34217,34218,34219,34220,34221,34222,34223,34224,34225,34226,34227,34228,34229,34230,34231,34232,34233,34234,34235,34236,34237,34238,34239,34240,34241,34242,34243,34244,34245,34246,34247,34248,34249,34250,34251,34252,34253,34254,34255,34256,34257,34258,34259,34260,34261,34262,34263,34264,34265,34266,34267,34268,34269,34270,34271,34272,34273,34274,34275,34276,34277,34278,34279,34280,34281,34282,34283,34284,34285,34286,34287,34288,34289,34290,34291,34292,34293,34294,34295,34296,34297,34298,34299,34300,34301,34302,34303,34304,34305,34306,34307,34308,34309,34310,34311,34312,34313,34314,34315,34316,34317,34318,34319,34320,34321,34322,34323,34324,34325,34326,34327,34328,34329,34330,34331,34332,34333,34334,34335,34336,34337,34338,34339,34340,34341,34342,34343,34344,34345,34346,34347,34348,34349,34350,34351,34352,34353,34354,34355,34356,34357,34358,34359,34360,34361,34362,34363,34364,34365,34366,34367,34368,34369,34370,34371,34372,34373,34374,34375,34376,34377,34378,34379,34380,34381,34382,34383,34384,34385,34386,34387,34388,34389,34390,34391,34392,34393,34394,34395,34396,34397,34398,34399,34400,34401,34402,34403,34404,34405,34406,34407,34408,34409,34410,34411,34412,34413,34414,34415,34416,34417,34418,34419,34420,34421,34422,34423,34424,34425,34426,34427,34428,34429,34430,34431,34432,34433,34434,34435,34436,34437,34438,34439,34440,34441,34442,34443,34444,34445,34446,34447,34448,34449,34450,34451,34452,34453,34454,34455,34456,34457,34458,34459,34460,34461,34462,34463,34464,34465,34466,34467,34468,34469,34470,34471,34472,34473,34474,34475,34476,34477,34478,34479,34480,34481,34482,34483,34484,34485,34486,34487,34488,34489,34490,34491,34492,34493,34494,34495,34496,34497,34498,34499,34500,34501,34502,34503,34504,34505,34506,34507,34508,34509,34510,34511,34512,34513,34514,34515,34516,34517,34518,34519,34520,34521,34522,34523,34524,34525,34526,34527,34528,34529,34530,34531,34532,34533,34534,34535,34536,34537,34538,34539,34540,34541,34542,34543,34544,34545,34546,34547,34548,34549,34550,34551,34552,34553,34554,34555,34556,34557,34558,34559,34560,34561,34562,34563,34564,34565,34566,34567,34568,34569,34570,34571,34572,34573,34574,34575,34576,34577,34578,34579,34580,34581,34582,34583,34584,34585,34586,34587,34588,34589,34590,34591,34592,34593,34594,34595,34596,34597,34598,34599,34600,34601,34602,34603,34604,34605,34606,34607,34608,34609,34610,34611,34612,34613,34614,34615,34616,34617,34618,34619,34620,34621,34622,34623,34624,34625,34626,34627,34628,34629,34630,34631,34632,34633,34634,34635,34636,34637,34638,34639,34640,34641,34642,34643,34644,34645,34646,34647,34648,34649,34650,34651,34652,34653,34654,34655,34656,34657,34658,34659,34660,34661,34662,34663,34664,34665,34666,34667,34668,34669,34670,34671,34672,34673,34674,34675,34676,34677,34678,34679,34680,34681,34682,34683,34684,34685,34686,34687,34688,34689,34690,34691,34692,34693,34694,34695,34696,34697,34698,34699,34700,34701,34702,34703,34704,34705,34706,34707,34708,34709,34710,34711,34712,34713,34714,34715,34716,34717,34718,34719,34720,34721,34722,34723,34724,34725,34726,34727,34728,34729,34730,34731,34732,34733,34734,34735,34736,34737,34738,34739,34740,34741,34742,34743,34744,34745,34746,34747,34748,34749,34750,34751,34752,34753,34754,34755,34756,34757,34758,34759,34760,34761,34762,34763,34764,34765,34766,34767,34768,34769,34770,34771,34772,34773,34774,34775,34776,34777,34778,34779,34780,34781,34782,34783,34784,34785,34786,34787,34788,34789,34790,34791,34792,34793,34794,34795,34796,34797,34798,34799,34800,34801,34802,34803,34804,34805,34806,34807,34808,34809,34810,34811,34812,34813,34814,34815,34816,34817,34818,34819,34820,34821,34822,34823,34824,34825,34826,34827,34828,34829,34830,34831,34832,34833,34834,34835,34836,34837,34838,34839,34840,34841,34842,34843,34844,34845,34846,34847,34848,34849,34850,34851,34852,34853,34854,34855,34856,34857,34858,34859,34860,34861,34862,34863,34864,34865,34866,34867,34868,34869,34870,34871,34872,34873,34874,34875,34876,34877,34878,34879,34880,34881,34882,34883,34884,34885,34886,34887,34888,34889,34890,34891,34892,34893,34894,34895,34896,34897,34898,34899,34900,34901,34902,34903,34904,34905,34906,34907,34908,34909,34910,34911,34912,34913,34914,34915,34916,34917,34918,34919,34920,34921,34922,34923,34924,34925,34926,34927,34928,34929,34930,34931,34932,34933,34934,34935,34936,34937,34938,34939,34940,34941,34942,34943,34944,34945,34946,34947,34948,34949,34950,34951,34952,34953,34954,34955,34956,34957,34958,34959,34960,34961,34962,34963,34964,34965,34966,34967,34968,34969,34970,34971,34972,34973,34974,34975,34976,34977,34978,34979,34980,34981,34982,34983,34984,34985,34986,34987,34988,34989,34990,34991,34992,34993,34994,34995,34996,34997,34998,34999,35000,35001,35002,35003,35004,35005,35006,35007,35008,35009,35010,35011,35012,35013,35014,35015,35016,35017,35018,35019,35020,35021,35022,35023,35024,35025,35026,35027,35028,35029,35030,35031,35032,35033,35034,35035,35036,35037,35038,35039,35040,35041,35042,35043,35044,35045,35046,35047,35048,35049,35050,35051,35052,35053,35054,35055,35056,35057,35058,35059,35060,35061,35062,35063,35064,35065,35066,35067,35068,35069,35070,35071,35072,35073,35074,35075,35076,35077,35078,35079,35080,35081,35082,35083,35084,35085,35086,35087,35088,35089,35090,35091,35092,35093,35094,35095,35096,35097,35098,35099,35100,35101,35102,35103,35104,35105,35106,35107,35108,35109,35110,35111,35112,35113,35114,35115,35116,35117,35118,35119,35120,35121,35122,35123,35124,35125,35126,35127,35128,35129,35130,35131,35132,35133,35134,35135,35136,35137,35138,35139,35140,35141,35142,35143,35144,35145,35146,35147,35148,35149,35150,35151,35152,35153,35154,35155,35156,35157,35158,35159,35160,35161,35162,35163,35164,35165,35166,35167,35168,35169,35170,35171,35172,35173,35174,35175,35176,35177,35178,35179,35180,35181,35182,35183,35184,35185,35186,35187,35188,35189,35190,35191,35192,35193,35194,35195,35196,35197,35198,35199,35200,35201,35202,35203,35204,35205,35206,35207,35208,35209,35210,35211,35212,35213,35214,35215,35216,35217,35218,35219,35220,35221,35222,35223,35224,35225,35226,35227,35228,35229,35230,35231,35232,35233,35234,35235,35236,35237,35238,35239,35240,35241,35242,35243,35244,35245,35246,35247,35248,35249,35250,35251,35252,35253,35254,35255,35256,35257,35258,35259,35260,35261,35262,35263,35264,35265,35266,35267,35268,35269,35270,35271,35272,35273,35274,35275,35276,35277,35278,35279,35280,35281,35282,35283,35284,35285,35286,35287,35288,35289,35290,35291,35292,35293,35294,35295,35296,35297,35298,35299,35300,35301,35302,35303,35304,35305,35306,35307,35308,35309,35310,35311,35312,35313,35314,35315,35316,35317,35318,35319,35320,35321,35322,35323,35324,35325,35326,35327,35328,35329,35330,35331,35332,35333,35334,35335,35336,35337,35338,35339,35340,35341,35342,35343,35344,35345,35346,35347,35348,35349,35350,35351,35352,35353,35354,35355,35356,35357,35358,35359,35360,35361,35362,35363,35364,35365,35366,35367,35368,35369,35370,35371,35372,35373,35374,35375,35376,35377,35378,35379,35380,35381,35382,35383,35384,35385,35386,35387,35388,35389,35390,35391,35392,35393,35394,35395,35396,35397,35398,35399,35400,35401,35402,35403,35404,35405,35406,35407,35408,35409,35410,35411,35412,35413,35414,35415,35416,35417,35418,35419,35420,35421,35422,35423,35424,35425,35426,35427,35428,35429,35430,35431,35432,35433,35434,35435,35436,35437,35438,35439,35440,35441,35442,35443,35444,35445,35446,35447,35448,35449,35450,35451,35452,35453,35454,35455,35456,35457,35458,35459,35460,35461,35462,35463,35464,35465,35466,35467,35468,35469,35470,35471,35472,35473,35474,35475,35476,35477,35478,35479,35480,35481,35482,35483,35484,35485,35486,35487,35488,35489,35490,35491,35492,35493,35494,35495,35496,35497,35498,35499,35500,35501,35502,35503,35504,35505,35506,35507,35508,35509,35510,35511,35512,35513,35514,35515,35516,35517,35518,35519,35520,35521,35522,35523,35524,35525,35526,35527,35528,35529,35530,35531,35532,35533,35534,35535,35536,35537,35538,35539,35540,35541,35542,35543,35544,35545,35546,35547,35548,35549,35550,35551,35552,35553,35554,35555,35556,35557,35558,35559,35560,35561,35562,35563,35564,35565,35566,35567,35568,35569,35570,35571,35572,35573,35574,35575,35576,35577,35578,35579,35580,35581,35582,35583,35584,35585,35586,35587,35588,35589,35590,35591,35592,35593,35594,35595,35596,35597,35598,35599,35600,35601,35602,35603,35604,35605,35606,35607,35608,35609,35610,35611,35612,35613,35614,35615,35616,35617,35618,35619,35620,35621,35622,35623,35624,35625,35626,35627,35628,35629,35630,35631,35632,35633,35634,35635,35636,35637,35638,35639,35640,35641,35642,35643,35644,35645,35646,35647,35648,35649,35650,35651,35652,35653,35654,35655,35656,35657,35658,35659,35660,35661,35662,35663,35664,35665,35666,35667,35668,35669,35670,35671,35672,35673,35674,35675,35676,35677,35678,35679,35680,35681,35682,35683,35684,35685,35686,35687,35688,35689,35690,35691,35692,35693,35694,35695,35696,35697,35698,35699,35700,35701,35702,35703,35704,35705,35706,35707,35708,35709,35710,35711,35712,35713,35714,35715,35716,35717,35718,35719,35720,35721,35722,35723,35724,35725,35726,35727,35728,35729,35730,35731,35732,35733,35734,35735,35736,35737,35738,35739,35740,35741,35742,35743,35744,35745,35746,35747,35748,35749,35750,35751,35752,35753,35754,35755,35756,35757,35758,35759,35760,35761,35762,35763,35764,35765,35766,35767,35768,35769,35770,35771,35772,35773,35774,35775,35776,35777,35778,35779,35780,35781,35782,35783,35784,35785,35786,35787,35788,35789,35790,35791,35792,35793,35794,35795,35796,35797,35798,35799,35800,35801,35802,35803,35804,35805,35806,35807,35808,35809,35810,35811,35812,35813,35814,35815,35816,35817,35818,35819,35820,35821,35822,35823,35824,35825,35826,35827,35828,35829,35830,35831,35832,35833,35834,35835,35836,35837,35838,35839,35840,35841,35842,35843,35844,35845,35846,35847,35848,35849,35850,35851,35852,35853,35854,35855,35856,35857,35858,35859,35860,35861,35862,35863,35864,35865,35866,35867,35868,35869,35870,35871,35872,35873,35874,35875,35876,35877,35878,35879,35880,35881,35882,35883,35884,35885,35886,35887,35888,35889,35890,35891,35892,35893,35894,35895,35896,35897,35898,35899,35900,35901,35902,35903,35904,35905,35906,35907,35908,35909,35910,35911,35912,35913,35914,35915,35916,35917,35918,35919,35920,35921,35922,35923,35924,35925,35926,35927,35928,35929,35930,35931,35932,35933,35934,35935,35936,35937,35938,35939,35940,35941,35942,35943,35944,35945,35946,35947,35948,35949,35950,35951,35952,35953,35954,35955,35956,35957,35958,35959,35960,35961,35962,35963,35964,35965,35966,35967,35968,35969,35970,35971,35972,35973,35974,35975,35976,35977,35978,35979,35980,35981,35982,35983,35984,35985,35986,35987,35988,35989,35990,35991,35992,35993,35994,35995,35996,35997,35998,35999,36000,36001,36002,36003,36004,36005,36006,36007,36008,36009,36010,36011,36012,36013,36014,36015,36016,36017,36018,36019,36020,36021,36022,36023,36024,36025,36026,36027,36028,36029,36030,36031,36032,36033,36034,36035,36036,36037,36038,36039,36040,36041,36042,36043,36044,36045,36046,36047,36048,36049,36050,36051,36052,36053,36054,36055,36056,36057,36058,36059,36060,36061,36062,36063,36064,36065,36066,36067,36068,36069,36070,36071,36072,36073,36074,36075,36076,36077,36078,36079,36080,36081,36082,36083,36084,36085,36086,36087,36088,36089,36090,36091,36092,36093,36094,36095,36096,36097,36098,36099,36100,36101,36102,36103,36104,36105,36106,36107,36108,36109,36110,36111,36112,36113,36114,36115,36116,36117,36118,36119,36120,36121,36122,36123,36124,36125,36126,36127,36128,36129,36130,36131,36132,36133,36134,36135,36136,36137,36138,36139,36140,36141,36142,36143,36144,36145,36146,36147,36148,36149,36150,36151,36152,36153,36154,36155,36156,36157,36158,36159,36160,36161,36162,36163,36164,36165,36166,36167,36168,36169,36170,36171,36172,36173,36174,36175,36176,36177,36178,36179,36180,36181,36182,36183,36184,36185,36186,36187,36188,36189,36190,36191,36192,36193,36194,36195,36196,36197,36198,36199,36200,36201,36202,36203,36204,36205,36206,36207,36208,36209,36210,36211,36212,36213,36214,36215,36216,36217,36218,36219,36220,36221,36222,36223,36224,36225,36226,36227,36228,36229,36230,36231,36232,36233,36234,36235,36236,36237,36238,36239,36240,36241,36242,36243,36244,36245,36246,36247,36248,36249,36250,36251,36252,36253,36254,36255,36256,36257,36258,36259,36260,36261,36262,36263,36264,36265,36266,36267,36268,36269,36270,36271,36272,36273,36274,36275,36276,36277,36278,36279,36280,36281,36282,36283,36284,36285,36286,36287,36288,36289,36290,36291,36292,36293,36294,36295,36296,36297,36298,36299,36300,36301,36302,36303,36304,36305,36306,36307,36308,36309,36310,36311,36312,36313,36314,36315,36316,36317,36318,36319,36320,36321,36322,36323,36324,36325,36326,36327,36328,36329,36330,36331,36332,36333,36334,36335,36336,36337,36338,36339,36340,36341,36342,36343,36344,36345,36346,36347,36348,36349,36350,36351,36352,36353,36354,36355,36356,36357,36358,36359,36360,36361,36362,36363,36364,36365,36366,36367,36368,36369,36370,36371,36372,36373,36374,36375,36376,36377,36378,36379,36380,36381,36382,36383,36384,36385,36386,36387,36388,36389,36390,36391,36392,36393,36394,36395,36396,36397,36398,36399,36400,36401,36402,36403,36404,36405,36406,36407,36408,36409,36410,36411,36412,36413,36414,36415,36416,36417,36418,36419,36420,36421,36422,36423,36424,36425,36426,36427,36428,36429,36430,36431,36432,36433,36434,36435,36436,36437,36438,36439,36440,36441,36442,36443,36444,36445,36446,36447,36448,36449,36450,36451,36452,36453,36454,36455,36456,36457,36458,36459,36460,36461,36462,36463,36464,36465,36466,36467,36468,36469,36470,36471,36472,36473,36474,36475,36476,36477,36478,36479,36480,36481,36482,36483,36484,36485,36486,36487,36488,36489,36490,36491,36492,36493,36494,36495,36496,36497,36498,36499,36500,36501,36502,36503,36504,36505,36506,36507,36508,36509,36510,36511,36512,36513,36514,36515,36516,36517,36518,36519,36520,36521,36522,36523,36524,36525,36526,36527,36528,36529,36530,36531,36532,36533,36534,36535,36536,36537,36538,36539,36540,36541,36542,36543,36544,36545,36546,36547,36548,36549,36550,36551,36552,36553,36554,36555,36556,36557,36558,36559,36560,36561,36562,36563,36564,36565,36566,36567,36568,36569,36570,36571,36572,36573,36574,36575,36576,36577,36578,36579,36580,36581,36582,36583,36584,36585,36586,36587,36588,36589,36590,36591,36592,36593,36594,36595,36596,36597,36598,36599,36600,36601,36602,36603,36604,36605,36606,36607,36608,36609,36610,36611,36612,36613,36614,36615,36616,36617,36618,36619,36620,36621,36622,36623,36624,36625,36626,36627,36628,36629,36630,36631,36632,36633,36634,36635,36636,36637,36638,36639,36640,36641,36642,36643,36644,36645,36646,36647,36648,36649,36650,36651,36652,36653,36654,36655,36656,36657,36658,36659,36660,36661,36662,36663,36664,36665,36666,36667,36668,36669,36670,36671,36672,36673,36674,36675,36676,36677,36678,36679,36680,36681,36682,36683,36684,36685,36686,36687,36688,36689,36690,36691,36692,36693,36694,36695,36696,36697,36698,36699,36700,36701,36702,36703,36704,36705,36706,36707,36708,36709,36710,36711,36712,36713,36714,36715,36716,36717,36718,36719,36720,36721,36722,36723,36724,36725,36726,36727,36728,36729,36730,36731,36732,36733,36734,36735,36736,36737,36738,36739,36740,36741,36742,36743,36744,36745,36746,36747,36748,36749,36750,36751,36752,36753,36754,36755,36756,36757,36758,36759,36760,36761,36762,36763,36764,36765,36766,36767,36768,36769,36770,36771,36772,36773,36774,36775,36776,36777,36778,36779,36780,36781,36782,36783,36784,36785,36786,36787,36788,36789,36790,36791,36792,36793,36794,36795,36796,36797,36798,36799,36800,36801,36802,36803,36804,36805,36806,36807,36808,36809,36810,36811,36812,36813,36814,36815,36816,36817,36818,36819,36820,36821,36822,36823,36824,36825,36826,36827,36828,36829,36830,36831,36832,36833,36834,36835,36836,36837,36838,36839,36840,36841,36842,36843,36844,36845,36846,36847,36848,36849,36850,36851,36852,36853,36854,36855,36856,36857,36858,36859,36860,36861,36862,36863,36864,36865,36866,36867,36868,36869,36870,36871,36872,36873,36874,36875,36876,36877,36878,36879,36880,36881,36882,36883,36884,36885,36886,36887,36888,36889,36890,36891,36892,36893,36894,36895,36896,36897,36898,36899,36900,36901,36902,36903,36904,36905,36906,36907,36908,36909,36910,36911,36912,36913,36914,36915,36916,36917,36918,36919,36920,36921,36922,36923,36924,36925,36926,36927,36928,36929,36930,36931,36932,36933,36934,36935,36936,36937,36938,36939,36940,36941,36942,36943,36944,36945,36946,36947,36948,36949,36950,36951,36952,36953,36954,36955,36956,36957,36958,36959,36960,36961,36962,36963,36964,36965,36966,36967,36968,36969,36970,36971,36972,36973,36974,36975,36976,36977,36978,36979,36980,36981,36982,36983,36984,36985,36986,36987,36988,36989,36990,36991,36992,36993,36994,36995,36996,36997,36998,36999,37000,37001,37002,37003,37004,37005,37006,37007,37008,37009,37010,37011,37012,37013,37014,37015,37016,37017,37018,37019,37020,37021,37022,37023,37024,37025,37026,37027,37028,37029,37030,37031,37032,37033,37034,37035,37036,37037,37038,37039,37040,37041,37042,37043,37044,37045,37046,37047,37048,37049,37050,37051,37052,37053,37054,37055,37056,37057,37058,37059,37060,37061,37062,37063,37064,37065,37066,37067,37068,37069,37070,37071,37072,37073,37074,37075,37076,37077,37078,37079,37080,37081,37082,37083,37084,37085,37086,37087,37088,37089,37090,37091,37092,37093,37094,37095,37096,37097,37098,37099,37100,37101,37102,37103,37104,37105,37106,37107,37108,37109,37110,37111,37112,37113,37114,37115,37116,37117,37118,37119,37120,37121,37122,37123,37124,37125,37126,37127,37128,37129,37130,37131,37132,37133,37134,37135,37136,37137,37138,37139,37140,37141,37142,37143,37144,37145,37146,37147,37148,37149,37150,37151,37152,37153,37154,37155,37156,37157,37158,37159,37160,37161,37162,37163,37164,37165,37166,37167,37168,37169,37170,37171,37172,37173,37174,37175,37176,37177,37178,37179,37180,37181,37182,37183,37184,37185,37186,37187,37188,37189,37190,37191,37192,37193,37194,37195,37196,37197,37198,37199,37200,37201,37202,37203,37204,37205,37206,37207,37208,37209,37210,37211,37212,37213,37214,37215,37216,37217,37218,37219,37220,37221,37222,37223,37224,37225,37226,37227,37228,37229,37230,37231,37232,37233,37234,37235,37236,37237,37238,37239,37240,37241,37242,37243,37244,37245,37246,37247,37248,37249,37250,37251,37252,37253,37254,37255,37256,37257,37258,37259,37260,37261,37262,37263,37264,37265,37266,37267,37268,37269,37270,37271,37272,37273,37274,37275,37276,37277,37278,37279,37280,37281,37282,37283,37284,37285,37286,37287,37288,37289,37290,37291,37292,37293,37294,37295,37296,37297,37298,37299,37300,37301,37302,37303,37304,37305,37306,37307,37308,37309,37310,37311,37312,37313,37314,37315,37316,37317,37318,37319,37320,37321,37322,37323,37324,37325,37326,37327,37328,37329,37330,37331,37332,37333,37334,37335,37336,37337,37338,37339,37340,37341,37342,37343,37344,37345,37346,37347,37348,37349,37350,37351,37352,37353,37354,37355,37356,37357,37358,37359,37360,37361,37362,37363,37364,37365,37366,37367,37368,37369,37370,37371,37372,37373,37374,37375,37376,37377,37378,37379,37380,37381,37382,37383,37384,37385,37386,37387,37388,37389,37390,37391,37392,37393,37394,37395,37396,37397,37398,37399,37400,37401,37402,37403,37404,37405,37406,37407,37408,37409,37410,37411,37412,37413,37414,37415,37416,37417,37418,37419,37420,37421,37422,37423,37424,37425,37426,37427,37428,37429,37430,37431,37432,37433,37434,37435,37436,37437,37438,37439,37440,37441,37442,37443,37444,37445,37446,37447,37448,37449,37450,37451,37452,37453,37454,37455,37456,37457,37458,37459,37460,37461,37462,37463,37464,37465,37466,37467,37468,37469,37470,37471,37472,37473,37474,37475,37476,37477,37478,37479,37480,37481,37482,37483,37484,37485,37486,37487,37488,37489,37490,37491,37492,37493,37494,37495,37496,37497,37498,37499,37500,37501,37502,37503,37504,37505,37506,37507,37508,37509,37510,37511,37512,37513,37514,37515,37516,37517,37518,37519,37520,37521,37522,37523,37524,37525,37526,37527,37528,37529,37530,37531,37532,37533,37534,37535,37536,37537,37538,37539,37540,37541,37542,37543,37544,37545,37546,37547,37548,37549,37550,37551,37552,37553,37554,37555,37556,37557,37558,37559,37560,37561,37562,37563,37564,37565,37566,37567,37568,37569,37570,37571,37572,37573,37574,37575,37576,37577,37578,37579,37580,37581,37582,37583,37584,37585,37586,37587,37588,37589,37590,37591,37592,37593,37594,37595,37596,37597,37598,37599,37600,37601,37602,37603,37604,37605,37606,37607,37608,37609,37610,37611,37612,37613,37614,37615,37616,37617,37618,37619,37620,37621,37622,37623,37624,37625,37626,37627,37628,37629,37630,37631,37632,37633,37634,37635,37636,37637,37638,37639,37640,37641,37642,37643,37644,37645,37646,37647,37648,37649,37650,37651,37652,37653,37654,37655,37656,37657,37658,37659,37660,37661,37662,37663,37664,37665,37666,37667,37668,37669,37670,37671,37672,37673,37674,37675,37676,37677,37678,37679,37680,37681,37682,37683,37684,37685,37686,37687,37688,37689,37690,37691,37692,37693,37694,37695,37696,37697,37698,37699,37700,37701,37702,37703,37704,37705,37706,37707,37708,37709,37710,37711,37712,37713,37714,37715,37716,37717,37718,37719,37720,37721,37722,37723,37724,37725,37726,37727,37728,37729,37730,37731,37732,37733,37734,37735,37736,37737,37738,37739,37740,37741,37742,37743,37744,37745,37746,37747,37748,37749,37750,37751,37752,37753,37754,37755,37756,37757,37758,37759,37760,37761,37762,37763,37764,37765,37766,37767,37768,37769,37770,37771,37772,37773,37774,37775,37776,37777,37778,37779,37780,37781,37782,37783,37784,37785,37786,37787,37788,37789,37790,37791,37792,37793,37794,37795,37796,37797,37798,37799,37800,37801,37802,37803,37804,37805,37806,37807,37808,37809,37810,37811,37812,37813,37814,37815,37816,37817,37818,37819,37820,37821,37822,37823,37824,37825,37826,37827,37828,37829,37830,37831,37832,37833,37834,37835,37836,37837,37838,37839,37840,37841,37842,37843,37844,37845,37846,37847,37848,37849,37850,37851,37852,37853,37854,37855,37856,37857,37858,37859,37860,37861,37862,37863,37864,37865,37866,37867,37868,37869,37870,37871,37872,37873,37874,37875,37876,37877,37878,37879,37880,37881,37882,37883,37884,37885,37886,37887,37888,37889,37890,37891,37892,37893,37894,37895,37896,37897,37898,37899,37900,37901,37902,37903,37904,37905,37906,37907,37908,37909,37910,37911,37912,37913,37914,37915,37916,37917,37918,37919,37920,37921,37922,37923,37924,37925,37926,37927,37928,37929,37930,37931,37932,37933,37934,37935,37936,37937,37938,37939,37940,37941,37942,37943,37944,37945,37946,37947,37948,37949,37950,37951,37952,37953,37954,37955,37956,37957,37958,37959,37960,37961,37962,37963,37964,37965,37966,37967,37968,37969,37970,37971,37972,37973,37974,37975,37976,37977,37978,37979,37980,37981,37982,37983,37984,37985,37986,37987,37988,37989,37990,37991,37992,37993,37994,37995,37996,37997,37998,37999,38000,38001,38002,38003,38004,38005,38006,38007,38008,38009,38010,38011,38012,38013,38014,38015,38016,38017,38018,38019,38020,38021,38022,38023,38024,38025,38026,38027,38028,38029,38030,38031,38032,38033,38034,38035,38036,38037,38038,38039,38040,38041,38042,38043,38044,38045,38046,38047,38048,38049,38050,38051,38052,38053,38054,38055,38056,38057,38058,38059,38060,38061,38062,38063,38064,38065,38066,38067,38068,38069,38070,38071,38072,38073,38074,38075,38076,38077,38078,38079,38080,38081,38082,38083,38084,38085,38086,38087,38088,38089,38090,38091,38092,38093,38094,38095,38096,38097,38098,38099,38100,38101,38102,38103,38104,38105,38106,38107,38108,38109,38110,38111,38112,38113,38114,38115,38116,38117,38118,38119,38120,38121,38122,38123,38124,38125,38126,38127,38128,38129,38130,38131,38132,38133,38134,38135,38136,38137,38138,38139,38140,38141,38142,38143,38144,38145,38146,38147,38148,38149,38150,38151,38152,38153,38154,38155,38156,38157,38158,38159,38160,38161,38162,38163,38164,38165,38166,38167,38168,38169,38170,38171,38172,38173,38174,38175,38176,38177,38178,38179,38180,38181,38182,38183,38184,38185,38186,38187,38188,38189,38190,38191,38192,38193,38194,38195,38196,38197,38198,38199,38200,38201,38202,38203,38204,38205,38206,38207,38208,38209,38210,38211,38212,38213,38214,38215,38216,38217,38218,38219,38220,38221,38222,38223,38224,38225,38226,38227,38228,38229,38230,38231,38232,38233,38234,38235,38236,38237,38238,38239,38240,38241,38242,38243,38244,38245,38246,38247,38248,38249,38250,38251,38252,38253,38254,38255,38256,38257,38258,38259,38260,38261,38262,38263,38264,38265,38266,38267,38268,38269,38270,38271,38272,38273,38274,38275,38276,38277,38278,38279,38280,38281,38282,38283,38284,38285,38286,38287,38288,38289,38290,38291,38292,38293,38294,38295,38296,38297,38298,38299,38300,38301,38302,38303,38304,38305,38306,38307,38308,38309,38310,38311,38312,38313,38314,38315,38316,38317,38318,38319,38320,38321,38322,38323,38324,38325,38326,38327,38328,38329,38330,38331,38332,38333,38334,38335,38336,38337,38338,38339,38340,38341,38342,38343,38344,38345,38346,38347,38348,38349,38350,38351,38352,38353,38354,38355,38356,38357,38358,38359,38360,38361,38362,38363,38364,38365,38366,38367,38368,38369,38370,38371,38372,38373,38374,38375,38376,38377,38378,38379,38380,38381,38382,38383,38384,38385,38386,38387,38388,38389,38390,38391,38392,38393,38394,38395,38396,38397,38398,38399,38400,38401,38402,38403,38404,38405,38406,38407,38408,38409,38410,38411,38412,38413,38414,38415,38416,38417,38418,38419,38420,38421,38422,38423,38424,38425,38426,38427,38428,38429,38430,38431,38432,38433,38434,38435,38436,38437,38438,38439,38440,38441,38442,38443,38444,38445,38446,38447,38448,38449,38450,38451,38452,38453,38454,38455,38456,38457,38458,38459,38460,38461,38462,38463,38464,38465,38466,38467,38468,38469,38470,38471,38472,38473,38474,38475,38476,38477,38478,38479,38480,38481,38482,38483,38484,38485,38486,38487,38488,38489,38490,38491,38492,38493,38494,38495,38496,38497,38498,38499,38500,38501,38502,38503,38504,38505,38506,38507,38508,38509,38510,38511,38512,38513,38514,38515,38516,38517,38518,38519,38520,38521,38522,38523,38524,38525,38526,38527,38528,38529,38530,38531,38532,38533,38534,38535,38536,38537,38538,38539,38540,38541,38542,38543,38544,38545,38546,38547,38548,38549,38550,38551,38552,38553,38554,38555,38556,38557,38558,38559,38560,38561,38562,38563,38564,38565,38566,38567,38568,38569,38570,38571,38572,38573,38574,38575,38576,38577,38578,38579,38580,38581,38582,38583,38584,38585,38586,38587,38588,38589,38590,38591,38592,38593,38594,38595,38596,38597,38598,38599,38600,38601,38602,38603,38604,38605,38606,38607,38608,38609,38610,38611,38612,38613,38614,38615,38616,38617,38618,38619,38620,38621,38622,38623,38624,38625,38626,38627,38628,38629,38630,38631,38632,38633,38634,38635,38636,38637,38638,38639,38640,38641,38642,38643,38644,38645,38646,38647,38648,38649,38650,38651,38652,38653,38654,38655,38656,38657,38658,38659,38660,38661,38662,38663,38664,38665,38666,38667,38668,38669,38670,38671,38672,38673,38674,38675,38676,38677,38678,38679,38680,38681,38682,38683,38684,38685,38686,38687,38688,38689,38690,38691,38692,38693,38694,38695,38696,38697,38698,38699,38700,38701,38702,38703,38704,38705,38706,38707,38708,38709,38710,38711,38712,38713,38714,38715,38716,38717,38718,38719,38720,38721,38722,38723,38724,38725,38726,38727,38728,38729,38730,38731,38732,38733,38734,38735,38736,38737,38738,38739,38740,38741,38742,38743,38744,38745,38746,38747,38748,38749,38750,38751,38752,38753,38754,38755,38756,38757,38758,38759,38760,38761,38762,38763,38764,38765,38766,38767,38768,38769,38770,38771,38772,38773,38774,38775,38776,38777,38778,38779,38780,38781,38782,38783,38784,38785,38786,38787,38788,38789,38790,38791,38792,38793,38794,38795,38796,38797,38798,38799,38800,38801,38802,38803,38804,38805,38806,38807,38808,38809,38810,38811,38812,38813,38814,38815,38816,38817,38818,38819,38820,38821,38822,38823,38824,38825,38826,38827,38828,38829,38830,38831,38832,38833,38834,38835,38836,38837,38838,38839,38840,38841,38842,38843,38844,38845,38846,38847,38848,38849,38850,38851,38852,38853,38854,38855,38856,38857,38858,38859,38860,38861,38862,38863,38864,38865,38866,38867,38868,38869,38870,38871,38872,38873,38874,38875,38876,38877,38878,38879,38880,38881,38882,38883,38884,38885,38886,38887,38888,38889,38890,38891,38892,38893,38894,38895,38896,38897,38898,38899,38900,38901,38902,38903,38904,38905,38906,38907,38908,38909,38910,38911,38912,38913,38914,38915,38916,38917,38918,38919,38920,38921,38922,38923,38924,38925,38926,38927,38928,38929,38930,38931,38932,38933,38934,38935,38936,38937,38938,38939,38940,38941,38942,38943,38944,38945,38946,38947,38948,38949,38950,38951,38952,38953,38954,38955,38956,38957,38958,38959,38960,38961,38962,38963,38964,38965,38966,38967,38968,38969,38970,38971,38972,38973,38974,38975,38976,38977,38978,38979,38980,38981,38982,38983,38984,38985,38986,38987,38988,38989,38990,38991,38992,38993,38994,38995,38996,38997,38998,38999,39000,39001,39002,39003,39004,39005,39006,39007,39008,39009,39010,39011,39012,39013,39014,39015,39016,39017,39018,39019,39020,39021,39022,39023,39024,39025,39026,39027,39028,39029,39030,39031,39032,39033,39034,39035,39036,39037,39038,39039,39040,39041,39042,39043,39044,39045,39046,39047,39048,39049,39050,39051,39052,39053,39054,39055,39056,39057,39058,39059,39060,39061,39062,39063,39064,39065,39066,39067,39068,39069,39070,39071,39072,39073,39074,39075,39076,39077,39078,39079,39080,39081,39082,39083,39084,39085,39086,39087,39088,39089,39090,39091,39092,39093,39094,39095,39096,39097,39098,39099,39100,39101,39102,39103,39104,39105,39106,39107,39108,39109,39110,39111,39112,39113,39114,39115,39116,39117,39118,39119,39120,39121,39122,39123,39124,39125,39126,39127,39128,39129,39130,39131,39132,39133,39134,39135,39136,39137,39138,39139,39140,39141,39142,39143,39144,39145,39146,39147,39148,39149,39150,39151,39152,39153,39154,39155,39156,39157,39158,39159,39160,39161,39162,39163,39164,39165,39166,39167,39168,39169,39170,39171,39172,39173,39174,39175,39176,39177,39178,39179,39180,39181,39182,39183,39184,39185,39186,39187,39188,39189,39190,39191,39192,39193,39194,39195,39196,39197,39198,39199,39200,39201,39202,39203,39204,39205,39206,39207,39208,39209,39210,39211,39212,39213,39214,39215,39216,39217,39218,39219,39220,39221,39222,39223,39224,39225,39226,39227,39228,39229,39230,39231,39232,39233,39234,39235,39236,39237,39238,39239,39240,39241,39242,39243,39244,39245,39246,39247,39248,39249,39250,39251,39252,39253,39254,39255,39256,39257,39258,39259,39260,39261,39262,39263,39264,39265,39266,39267,39268,39269,39270,39271,39272,39273,39274,39275,39276,39277,39278,39279,39280,39281,39282,39283,39284,39285,39286,39287,39288,39289,39290,39291,39292,39293,39294,39295,39296,39297,39298,39299,39300,39301,39302,39303,39304,39305,39306,39307,39308,39309,39310,39311,39312,39313,39314,39315,39316,39317,39318,39319,39320,39321,39322,39323,39324,39325,39326,39327,39328,39329,39330,39331,39332,39333,39334,39335,39336,39337,39338,39339,39340,39341,39342,39343,39344,39345,39346,39347,39348,39349,39350,39351,39352,39353,39354,39355,39356,39357,39358,39359,39360,39361,39362,39363,39364,39365,39366,39367,39368,39369,39370,39371,39372,39373,39374,39375,39376,39377,39378,39379,39380,39381,39382,39383,39384,39385,39386,39387,39388,39389,39390,39391,39392,39393,39394,39395,39396,39397,39398,39399,39400,39401,39402,39403,39404,39405,39406,39407,39408,39409,39410,39411,39412,39413,39414,39415,39416,39417,39418,39419,39420,39421,39422,39423,39424,39425,39426,39427,39428,39429,39430,39431,39432,39433,39434,39435,39436,39437,39438,39439,39440,39441,39442,39443,39444,39445,39446,39447,39448,39449,39450,39451,39452,39453,39454,39455,39456,39457,39458,39459,39460,39461,39462,39463,39464,39465,39466,39467,39468,39469,39470,39471,39472,39473,39474,39475,39476,39477,39478,39479,39480,39481,39482,39483,39484,39485,39486,39487,39488,39489,39490,39491,39492,39493,39494,39495,39496,39497,39498,39499,39500,39501,39502,39503,39504,39505,39506,39507,39508,39509,39510,39511,39512,39513,39514,39515,39516,39517,39518,39519,39520,39521,39522,39523,39524,39525,39526,39527,39528,39529,39530,39531,39532,39533,39534,39535,39536,39537,39538,39539,39540,39541,39542,39543,39544,39545,39546,39547,39548,39549,39550,39551,39552,39553,39554,39555,39556,39557,39558,39559,39560,39561,39562,39563,39564,39565,39566,39567,39568,39569,39570,39571,39572,39573,39574,39575,39576,39577,39578,39579,39580,39581,39582,39583,39584,39585,39586,39587,39588,39589,39590,39591,39592,39593,39594,39595,39596,39597,39598,39599,39600,39601,39602,39603,39604,39605,39606,39607,39608,39609,39610,39611,39612,39613,39614,39615,39616,39617,39618,39619,39620,39621,39622,39623,39624,39625,39626,39627,39628,39629,39630,39631,39632,39633,39634,39635,39636,39637,39638,39639,39640,39641,39642,39643,39644,39645,39646,39647,39648,39649,39650,39651,39652,39653,39654,39655,39656,39657,39658,39659,39660,39661,39662,39663,39664,39665,39666,39667,39668,39669,39670,39671,39672,39673,39674,39675,39676,39677,39678,39679,39680,39681,39682,39683,39684,39685,39686,39687,39688,39689,39690,39691,39692,39693,39694,39695,39696,39697,39698,39699,39700,39701,39702,39703,39704,39705,39706,39707,39708,39709,39710,39711,39712,39713,39714,39715,39716,39717,39718,39719,39720,39721,39722,39723,39724,39725,39726,39727,39728,39729,39730,39731,39732,39733,39734,39735,39736,39737,39738,39739,39740,39741,39742,39743,39744,39745,39746,39747,39748,39749,39750,39751,39752,39753,39754,39755,39756,39757,39758,39759,39760,39761,39762,39763,39764,39765,39766,39767,39768,39769,39770,39771,39772,39773,39774,39775,39776,39777,39778,39779,39780,39781,39782,39783,39784,39785,39786,39787,39788,39789,39790,39791,39792,39793,39794,39795,39796,39797,39798,39799,39800,39801,39802,39803,39804,39805,39806,39807,39808,39809,39810,39811,39812,39813,39814,39815,39816,39817,39818,39819,39820,39821,39822,39823,39824,39825,39826,39827,39828,39829,39830,39831,39832,39833,39834,39835,39836,39837,39838,39839,39840,39841,39842,39843,39844,39845,39846,39847,39848,39849,39850,39851,39852,39853,39854,39855,39856,39857,39858,39859,39860,39861,39862,39863,39864,39865,39866,39867,39868,39869,39870,39871,39872,39873,39874,39875,39876,39877,39878,39879,39880,39881,39882,39883,39884,39885,39886,39887,39888,39889,39890,39891,39892,39893,39894,39895,39896,39897,39898,39899,39900,39901,39902,39903,39904,39905,39906,39907,39908,39909,39910,39911,39912,39913,39914,39915,39916,39917,39918,39919,39920,39921,39922,39923,39924,39925,39926,39927,39928,39929,39930,39931,39932,39933,39934,39935,39936,39937,39938,39939,39940,39941,39942,39943,39944,39945,39946,39947,39948,39949,39950,39951,39952,39953,39954,39955,39956,39957,39958,39959,39960,39961,39962,39963,39964,39965,39966,39967,39968,39969,39970,39971,39972,39973,39974,39975,39976,39977,39978,39979,39980,39981,39982,39983,39984,39985,39986,39987,39988,39989,39990,39991,39992,39993,39994,39995,39996,39997,39998,39999,40000,40001,40002,40003,40004,40005,40006,40007,40008,40009,40010,40011,40012,40013,40014,40015,40016,40017,40018,40019,40020,40021,40022,40023,40024,40025,40026,40027,40028,40029,40030,40031,40032,40033,40034,40035,40036,40037,40038,40039,40040,40041,40042,40043,40044,40045,40046,40047,40048,40049,40050,40051,40052,40053,40054,40055,40056,40057,40058,40059,40060,40061,40062,40063,40064,40065,40066,40067,40068,40069,40070,40071,40072,40073,40074,40075,40076,40077,40078,40079,40080,40081,40082,40083,40084,40085,40086,40087,40088,40089,40090,40091,40092,40093,40094,40095,40096,40097,40098,40099,40100,40101,40102,40103,40104,40105,40106,40107,40108,40109,40110,40111,40112,40113,40114,40115,40116,40117,40118,40119,40120,40121,40122,40123,40124,40125,40126,40127,40128,40129,40130,40131,40132,40133,40134,40135,40136,40137,40138,40139,40140,40141,40142,40143,40144,40145,40146,40147,40148,40149,40150,40151,40152,40153,40154,40155,40156,40157,40158,40159,40160,40161,40162,40163,40164,40165,40166,40167,40168,40169,40170,40171,40172,40173,40174,40175,40176,40177,40178,40179,40180,40181,40182,40183,40184,40185,40186,40187,40188,40189,40190,40191,40192,40193,40194,40195,40196,40197,40198,40199,40200,40201,40202,40203,40204,40205,40206,40207,40208,40209,40210,40211,40212,40213,40214,40215,40216,40217,40218,40219,40220,40221,40222,40223,40224,40225,40226,40227,40228,40229,40230,40231,40232,40233,40234,40235,40236,40237,40238,40239,40240,40241,40242,40243,40244,40245,40246,40247,40248,40249,40250,40251,40252,40253,40254,40255,40256,40257,40258,40259,40260,40261,40262,40263,40264,40265,40266,40267,40268,40269,40270,40271,40272,40273,40274,40275,40276,40277,40278,40279,40280,40281,40282,40283,40284,40285,40286,40287,40288,40289,40290,40291,40292,40293,40294,40295,40296,40297,40298,40299,40300,40301,40302,40303,40304,40305,40306,40307,40308,40309,40310,40311,40312,40313,40314,40315,40316,40317,40318,40319,40320,40321,40322,40323,40324,40325,40326,40327,40328,40329,40330,40331,40332,40333,40334,40335,40336,40337,40338,40339,40340,40341,40342,40343,40344,40345,40346,40347,40348,40349,40350,40351,40352,40353,40354,40355,40356,40357,40358,40359,40360,40361,40362,40363,40364,40365,40366,40367,40368,40369,40370,40371,40372,40373,40374,40375,40376,40377,40378,40379,40380,40381,40382,40383,40384,40385,40386,40387,40388,40389,40390,40391,40392,40393,40394,40395,40396,40397,40398,40399,40400,40401,40402,40403,40404,40405,40406,40407,40408,40409,40410,40411,40412,40413,40414,40415,40416,40417,40418,40419,40420,40421,40422,40423,40424,40425,40426,40427,40428,40429,40430,40431,40432,40433,40434,40435,40436,40437,40438,40439,40440,40441,40442,40443,40444,40445,40446,40447,40448,40449,40450,40451,40452,40453,40454,40455,40456,40457,40458,40459,40460,40461,40462,40463,40464,40465,40466,40467,40468,40469,40470,40471,40472,40473,40474,40475,40476,40477,40478,40479,40480,40481,40482,40483,40484,40485,40486,40487,40488,40489,40490,40491,40492,40493,40494,40495,40496,40497,40498,40499,40500,40501,40502,40503,40504,40505,40506,40507,40508,40509,40510,40511,40512,40513,40514,40515,40516,40517,40518,40519,40520,40521,40522,40523,40524,40525,40526,40527,40528,40529,40530,40531,40532,40533,40534,40535,40536,40537,40538,40539,40540,40541,40542,40543,40544,40545,40546,40547,40548,40549,40550,40551,40552,40553,40554,40555,40556,40557,40558,40559,40560,40561,40562,40563,40564,40565,40566,40567,40568,40569,40570,40571,40572,40573,40574,40575,40576,40577,40578,40579,40580,40581,40582,40583,40584,40585,40586,40587,40588,40589,40590,40591,40592,40593,40594,40595,40596,40597,40598,40599,40600,40601,40602,40603,40604,40605,40606,40607,40608,40609,40610,40611,40612,40613,40614,40615,40616,40617,40618,40619,40620,40621,40622,40623,40624,40625,40626,40627,40628,40629,40630,40631,40632,40633,40634,40635,40636,40637,40638,40639,40640,40641,40642,40643,40644,40645,40646,40647,40648,40649,40650,40651,40652,40653,40654,40655,40656,40657,40658,40659,40660,40661,40662,40663,40664,40665,40666,40667,40668,40669,40670,40671,40672,40673,40674,40675,40676,40677,40678,40679,40680,40681,40682,40683,40684,40685,40686,40687,40688,40689,40690,40691,40692,40693,40694,40695,40696,40697,40698,40699,40700,40701,40702,40703,40704,40705,40706,40707,40708,40709,40710,40711,40712,40713,40714,40715,40716,40717,40718,40719,40720,40721,40722,40723,40724,40725,40726,40727,40728,40729,40730,40731,40732,40733,40734,40735,40736,40737,40738,40739,40740,40741,40742,40743,40744,40745,40746,40747,40748,40749,40750,40751,40752,40753,40754,40755,40756,40757,40758,40759,40760,40761,40762,40763,40764,40765,40766,40767,40768,40769,40770,40771,40772,40773,40774,40775,40776,40777,40778,40779,40780,40781,40782,40783,40784,40785,40786,40787,40788,40789,40790,40791,40792,40793,40794,40795,40796,40797,40798,40799,40800,40801,40802,40803,40804,40805,40806,40807,40808,40809,40810,40811,40812,40813,40814,40815,40816,40817,40818,40819,40820,40821,40822,40823,40824,40825,40826,40827,40828,40829,40830,40831,40832,40833,40834,40835,40836,40837,40838,40839,40840,40841,40842,40843,40844,40845,40846,40847,40848,40849,40850,40851,40852,40853,40854,40855,40856,40857,40858,40859,40860,40861,40862,40863,40864,40865,40866,40867,40868,40869,40870,40871,40872,40873,40874,40875,40876,40877,40878,40879,40880,40881,40882,40883,40884,40885,40886,40887,40888,40889,40890,40891,40892,40893,40894,40895,40896,40897,40898,40899,40900,40901,40902,40903,40904,40905,40906,40907,40908,40909,40910,40911,40912,40913,40914,40915,40916,40917,40918,40919,40920,40921,40922,40923,40924,40925,40926,40927,40928,40929,40930,40931,40932,40933,40934,40935,40936,40937,40938,40939,40940,40941,40942,40943,40960,40961,40962,40963,40964,40965,40966,40967,40968,40969,40970,40971,40972,40973,40974,40975,40976,40977,40978,40979,40980,40981,40982,40983,40984,40985,40986,40987,40988,40989,40990,40991,40992,40993,40994,40995,40996,40997,40998,40999,41000,41001,41002,41003,41004,41005,41006,41007,41008,41009,41010,41011,41012,41013,41014,41015,41016,41017,41018,41019,41020,41021,41022,41023,41024,41025,41026,41027,41028,41029,41030,41031,41032,41033,41034,41035,41036,41037,41038,41039,41040,41041,41042,41043,41044,41045,41046,41047,41048,41049,41050,41051,41052,41053,41054,41055,41056,41057,41058,41059,41060,41061,41062,41063,41064,41065,41066,41067,41068,41069,41070,41071,41072,41073,41074,41075,41076,41077,41078,41079,41080,41081,41082,41083,41084,41085,41086,41087,41088,41089,41090,41091,41092,41093,41094,41095,41096,41097,41098,41099,41100,41101,41102,41103,41104,41105,41106,41107,41108,41109,41110,41111,41112,41113,41114,41115,41116,41117,41118,41119,41120,41121,41122,41123,41124,41125,41126,41127,41128,41129,41130,41131,41132,41133,41134,41135,41136,41137,41138,41139,41140,41141,41142,41143,41144,41145,41146,41147,41148,41149,41150,41151,41152,41153,41154,41155,41156,41157,41158,41159,41160,41161,41162,41163,41164,41165,41166,41167,41168,41169,41170,41171,41172,41173,41174,41175,41176,41177,41178,41179,41180,41181,41182,41183,41184,41185,41186,41187,41188,41189,41190,41191,41192,41193,41194,41195,41196,41197,41198,41199,41200,41201,41202,41203,41204,41205,41206,41207,41208,41209,41210,41211,41212,41213,41214,41215,41216,41217,41218,41219,41220,41221,41222,41223,41224,41225,41226,41227,41228,41229,41230,41231,41232,41233,41234,41235,41236,41237,41238,41239,41240,41241,41242,41243,41244,41245,41246,41247,41248,41249,41250,41251,41252,41253,41254,41255,41256,41257,41258,41259,41260,41261,41262,41263,41264,41265,41266,41267,41268,41269,41270,41271,41272,41273,41274,41275,41276,41277,41278,41279,41280,41281,41282,41283,41284,41285,41286,41287,41288,41289,41290,41291,41292,41293,41294,41295,41296,41297,41298,41299,41300,41301,41302,41303,41304,41305,41306,41307,41308,41309,41310,41311,41312,41313,41314,41315,41316,41317,41318,41319,41320,41321,41322,41323,41324,41325,41326,41327,41328,41329,41330,41331,41332,41333,41334,41335,41336,41337,41338,41339,41340,41341,41342,41343,41344,41345,41346,41347,41348,41349,41350,41351,41352,41353,41354,41355,41356,41357,41358,41359,41360,41361,41362,41363,41364,41365,41366,41367,41368,41369,41370,41371,41372,41373,41374,41375,41376,41377,41378,41379,41380,41381,41382,41383,41384,41385,41386,41387,41388,41389,41390,41391,41392,41393,41394,41395,41396,41397,41398,41399,41400,41401,41402,41403,41404,41405,41406,41407,41408,41409,41410,41411,41412,41413,41414,41415,41416,41417,41418,41419,41420,41421,41422,41423,41424,41425,41426,41427,41428,41429,41430,41431,41432,41433,41434,41435,41436,41437,41438,41439,41440,41441,41442,41443,41444,41445,41446,41447,41448,41449,41450,41451,41452,41453,41454,41455,41456,41457,41458,41459,41460,41461,41462,41463,41464,41465,41466,41467,41468,41469,41470,41471,41472,41473,41474,41475,41476,41477,41478,41479,41480,41481,41482,41483,41484,41485,41486,41487,41488,41489,41490,41491,41492,41493,41494,41495,41496,41497,41498,41499,41500,41501,41502,41503,41504,41505,41506,41507,41508,41509,41510,41511,41512,41513,41514,41515,41516,41517,41518,41519,41520,41521,41522,41523,41524,41525,41526,41527,41528,41529,41530,41531,41532,41533,41534,41535,41536,41537,41538,41539,41540,41541,41542,41543,41544,41545,41546,41547,41548,41549,41550,41551,41552,41553,41554,41555,41556,41557,41558,41559,41560,41561,41562,41563,41564,41565,41566,41567,41568,41569,41570,41571,41572,41573,41574,41575,41576,41577,41578,41579,41580,41581,41582,41583,41584,41585,41586,41587,41588,41589,41590,41591,41592,41593,41594,41595,41596,41597,41598,41599,41600,41601,41602,41603,41604,41605,41606,41607,41608,41609,41610,41611,41612,41613,41614,41615,41616,41617,41618,41619,41620,41621,41622,41623,41624,41625,41626,41627,41628,41629,41630,41631,41632,41633,41634,41635,41636,41637,41638,41639,41640,41641,41642,41643,41644,41645,41646,41647,41648,41649,41650,41651,41652,41653,41654,41655,41656,41657,41658,41659,41660,41661,41662,41663,41664,41665,41666,41667,41668,41669,41670,41671,41672,41673,41674,41675,41676,41677,41678,41679,41680,41681,41682,41683,41684,41685,41686,41687,41688,41689,41690,41691,41692,41693,41694,41695,41696,41697,41698,41699,41700,41701,41702,41703,41704,41705,41706,41707,41708,41709,41710,41711,41712,41713,41714,41715,41716,41717,41718,41719,41720,41721,41722,41723,41724,41725,41726,41727,41728,41729,41730,41731,41732,41733,41734,41735,41736,41737,41738,41739,41740,41741,41742,41743,41744,41745,41746,41747,41748,41749,41750,41751,41752,41753,41754,41755,41756,41757,41758,41759,41760,41761,41762,41763,41764,41765,41766,41767,41768,41769,41770,41771,41772,41773,41774,41775,41776,41777,41778,41779,41780,41781,41782,41783,41784,41785,41786,41787,41788,41789,41790,41791,41792,41793,41794,41795,41796,41797,41798,41799,41800,41801,41802,41803,41804,41805,41806,41807,41808,41809,41810,41811,41812,41813,41814,41815,41816,41817,41818,41819,41820,41821,41822,41823,41824,41825,41826,41827,41828,41829,41830,41831,41832,41833,41834,41835,41836,41837,41838,41839,41840,41841,41842,41843,41844,41845,41846,41847,41848,41849,41850,41851,41852,41853,41854,41855,41856,41857,41858,41859,41860,41861,41862,41863,41864,41865,41866,41867,41868,41869,41870,41871,41872,41873,41874,41875,41876,41877,41878,41879,41880,41881,41882,41883,41884,41885,41886,41887,41888,41889,41890,41891,41892,41893,41894,41895,41896,41897,41898,41899,41900,41901,41902,41903,41904,41905,41906,41907,41908,41909,41910,41911,41912,41913,41914,41915,41916,41917,41918,41919,41920,41921,41922,41923,41924,41925,41926,41927,41928,41929,41930,41931,41932,41933,41934,41935,41936,41937,41938,41939,41940,41941,41942,41943,41944,41945,41946,41947,41948,41949,41950,41951,41952,41953,41954,41955,41956,41957,41958,41959,41960,41961,41962,41963,41964,41965,41966,41967,41968,41969,41970,41971,41972,41973,41974,41975,41976,41977,41978,41979,41980,41981,41982,41983,41984,41985,41986,41987,41988,41989,41990,41991,41992,41993,41994,41995,41996,41997,41998,41999,42000,42001,42002,42003,42004,42005,42006,42007,42008,42009,42010,42011,42012,42013,42014,42015,42016,42017,42018,42019,42020,42021,42022,42023,42024,42025,42026,42027,42028,42029,42030,42031,42032,42033,42034,42035,42036,42037,42038,42039,42040,42041,42042,42043,42044,42045,42046,42047,42048,42049,42050,42051,42052,42053,42054,42055,42056,42057,42058,42059,42060,42061,42062,42063,42064,42065,42066,42067,42068,42069,42070,42071,42072,42073,42074,42075,42076,42077,42078,42079,42080,42081,42082,42083,42084,42085,42086,42087,42088,42089,42090,42091,42092,42093,42094,42095,42096,42097,42098,42099,42100,42101,42102,42103,42104,42105,42106,42107,42108,42109,42110,42111,42112,42113,42114,42115,42116,42117,42118,42119,42120,42121,42122,42123,42124,42192,42193,42194,42195,42196,42197,42198,42199,42200,42201,42202,42203,42204,42205,42206,42207,42208,42209,42210,42211,42212,42213,42214,42215,42216,42217,42218,42219,42220,42221,42222,42223,42224,42225,42226,42227,42228,42229,42230,42231,42232,42233,42234,42235,42236,42237,42240,42241,42242,42243,42244,42245,42246,42247,42248,42249,42250,42251,42252,42253,42254,42255,42256,42257,42258,42259,42260,42261,42262,42263,42264,42265,42266,42267,42268,42269,42270,42271,42272,42273,42274,42275,42276,42277,42278,42279,42280,42281,42282,42283,42284,42285,42286,42287,42288,42289,42290,42291,42292,42293,42294,42295,42296,42297,42298,42299,42300,42301,42302,42303,42304,42305,42306,42307,42308,42309,42310,42311,42312,42313,42314,42315,42316,42317,42318,42319,42320,42321,42322,42323,42324,42325,42326,42327,42328,42329,42330,42331,42332,42333,42334,42335,42336,42337,42338,42339,42340,42341,42342,42343,42344,42345,42346,42347,42348,42349,42350,42351,42352,42353,42354,42355,42356,42357,42358,42359,42360,42361,42362,42363,42364,42365,42366,42367,42368,42369,42370,42371,42372,42373,42374,42375,42376,42377,42378,42379,42380,42381,42382,42383,42384,42385,42386,42387,42388,42389,42390,42391,42392,42393,42394,42395,42396,42397,42398,42399,42400,42401,42402,42403,42404,42405,42406,42407,42408,42409,42410,42411,42412,42413,42414,42415,42416,42417,42418,42419,42420,42421,42422,42423,42424,42425,42426,42427,42428,42429,42430,42431,42432,42433,42434,42435,42436,42437,42438,42439,42440,42441,42442,42443,42444,42445,42446,42447,42448,42449,42450,42451,42452,42453,42454,42455,42456,42457,42458,42459,42460,42461,42462,42463,42464,42465,42466,42467,42468,42469,42470,42471,42472,42473,42474,42475,42476,42477,42478,42479,42480,42481,42482,42483,42484,42485,42486,42487,42488,42489,42490,42491,42492,42493,42494,42495,42496,42497,42498,42499,42500,42501,42502,42503,42504,42505,42506,42507,42508,42512,42513,42514,42515,42516,42517,42518,42519,42520,42521,42522,42523,42524,42525,42526,42527,42538,42539,42560,42561,42562,42563,42564,42565,42566,42567,42568,42569,42570,42571,42572,42573,42574,42575,42576,42577,42578,42579,42580,42581,42582,42583,42584,42585,42586,42587,42588,42589,42590,42591,42592,42593,42594,42595,42596,42597,42598,42599,42600,42601,42602,42603,42604,42605,42606,42623,42624,42625,42626,42627,42628,42629,42630,42631,42632,42633,42634,42635,42636,42637,42638,42639,42640,42641,42642,42643,42644,42645,42646,42647,42648,42649,42650,42651,42652,42653,42656,42657,42658,42659,42660,42661,42662,42663,42664,42665,42666,42667,42668,42669,42670,42671,42672,42673,42674,42675,42676,42677,42678,42679,42680,42681,42682,42683,42684,42685,42686,42687,42688,42689,42690,42691,42692,42693,42694,42695,42696,42697,42698,42699,42700,42701,42702,42703,42704,42705,42706,42707,42708,42709,42710,42711,42712,42713,42714,42715,42716,42717,42718,42719,42720,42721,42722,42723,42724,42725,42726,42727,42728,42729,42730,42731,42732,42733,42734,42735,42775,42776,42777,42778,42779,42780,42781,42782,42783,42786,42787,42788,42789,42790,42791,42792,42793,42794,42795,42796,42797,42798,42799,42800,42801,42802,42803,42804,42805,42806,42807,42808,42809,42810,42811,42812,42813,42814,42815,42816,42817,42818,42819,42820,42821,42822,42823,42824,42825,42826,42827,42828,42829,42830,42831,42832,42833,42834,42835,42836,42837,42838,42839,42840,42841,42842,42843,42844,42845,42846,42847,42848,42849,42850,42851,42852,42853,42854,42855,42856,42857,42858,42859,42860,42861,42862,42863,42864,42865,42866,42867,42868,42869,42870,42871,42872,42873,42874,42875,42876,42877,42878,42879,42880,42881,42882,42883,42884,42885,42886,42887,42888,42891,42892,42893,42894,42895,42896,42897,42898,42899,42900,42901,42902,42903,42904,42905,42906,42907,42908,42909,42910,42911,42912,42913,42914,42915,42916,42917,42918,42919,42920,42921,42922,42923,42924,42925,42926,42927,42928,42929,42930,42931,42932,42933,42934,42935,42936,42937,42999,43000,43001,43002,43003,43004,43005,43006,43007,43008,43009,43011,43012,43013,43015,43016,43017,43018,43020,43021,43022,43023,43024,43025,43026,43027,43028,43029,43030,43031,43032,43033,43034,43035,43036,43037,43038,43039,43040,43041,43042,43072,43073,43074,43075,43076,43077,43078,43079,43080,43081,43082,43083,43084,43085,43086,43087,43088,43089,43090,43091,43092,43093,43094,43095,43096,43097,43098,43099,43100,43101,43102,43103,43104,43105,43106,43107,43108,43109,43110,43111,43112,43113,43114,43115,43116,43117,43118,43119,43120,43121,43122,43123,43138,43139,43140,43141,43142,43143,43144,43145,43146,43147,43148,43149,43150,43151,43152,43153,43154,43155,43156,43157,43158,43159,43160,43161,43162,43163,43164,43165,43166,43167,43168,43169,43170,43171,43172,43173,43174,43175,43176,43177,43178,43179,43180,43181,43182,43183,43184,43185,43186,43187,43250,43251,43252,43253,43254,43255,43259,43261,43262,43274,43275,43276,43277,43278,43279,43280,43281,43282,43283,43284,43285,43286,43287,43288,43289,43290,43291,43292,43293,43294,43295,43296,43297,43298,43299,43300,43301,43312,43313,43314,43315,43316,43317,43318,43319,43320,43321,43322,43323,43324,43325,43326,43327,43328,43329,43330,43331,43332,43333,43334,43360,43361,43362,43363,43364,43365,43366,43367,43368,43369,43370,43371,43372,43373,43374,43375,43376,43377,43378,43379,43380,43381,43382,43383,43384,43385,43386,43387,43388,43396,43397,43398,43399,43400,43401,43402,43403,43404,43405,43406,43407,43408,43409,43410,43411,43412,43413,43414,43415,43416,43417,43418,43419,43420,43421,43422,43423,43424,43425,43426,43427,43428,43429,43430,43431,43432,43433,43434,43435,43436,43437,43438,43439,43440,43441,43442,43471,43488,43489,43490,43491,43492,43494,43495,43496,43497,43498,43499,43500,43501,43502,43503,43514,43515,43516,43517,43518,43520,43521,43522,43523,43524,43525,43526,43527,43528,43529,43530,43531,43532,43533,43534,43535,43536,43537,43538,43539,43540,43541,43542,43543,43544,43545,43546,43547,43548,43549,43550,43551,43552,43553,43554,43555,43556,43557,43558,43559,43560,43584,43585,43586,43588,43589,43590,43591,43592,43593,43594,43595,43616,43617,43618,43619,43620,43621,43622,43623,43624,43625,43626,43627,43628,43629,43630,43631,43632,43633,43634,43635,43636,43637,43638,43642,43646,43647,43648,43649,43650,43651,43652,43653,43654,43655,43656,43657,43658,43659,43660,43661,43662,43663,43664,43665,43666,43667,43668,43669,43670,43671,43672,43673,43674,43675,43676,43677,43678,43679,43680,43681,43682,43683,43684,43685,43686,43687,43688,43689,43690,43691,43692,43693,43694,43695,43697,43701,43702,43705,43706,43707,43708,43709,43712,43714,43739,43740,43741,43744,43745,43746,43747,43748,43749,43750,43751,43752,43753,43754,43762,43763,43764,43777,43778,43779,43780,43781,43782,43785,43786,43787,43788,43789,43790,43793,43794,43795,43796,43797,43798,43808,43809,43810,43811,43812,43813,43814,43816,43817,43818,43819,43820,43821,43822,43824,43825,43826,43827,43828,43829,43830,43831,43832,43833,43834,43835,43836,43837,43838,43839,43840,43841,43842,43843,43844,43845,43846,43847,43848,43849,43850,43851,43852,43853,43854,43855,43856,43857,43858,43859,43860,43861,43862,43863,43864,43865,43866,43868,43869,43870,43871,43872,43873,43874,43875,43876,43877,43888,43889,43890,43891,43892,43893,43894,43895,43896,43897,43898,43899,43900,43901,43902,43903,43904,43905,43906,43907,43908,43909,43910,43911,43912,43913,43914,43915,43916,43917,43918,43919,43920,43921,43922,43923,43924,43925,43926,43927,43928,43929,43930,43931,43932,43933,43934,43935,43936,43937,43938,43939,43940,43941,43942,43943,43944,43945,43946,43947,43948,43949,43950,43951,43952,43953,43954,43955,43956,43957,43958,43959,43960,43961,43962,43963,43964,43965,43966,43967,43968,43969,43970,43971,43972,43973,43974,43975,43976,43977,43978,43979,43980,43981,43982,43983,43984,43985,43986,43987,43988,43989,43990,43991,43992,43993,43994,43995,43996,43997,43998,43999,44000,44001,44002,44032,44033,44034,44035,44036,44037,44038,44039,44040,44041,44042,44043,44044,44045,44046,44047,44048,44049,44050,44051,44052,44053,44054,44055,44056,44057,44058,44059,44060,44061,44062,44063,44064,44065,44066,44067,44068,44069,44070,44071,44072,44073,44074,44075,44076,44077,44078,44079,44080,44081,44082,44083,44084,44085,44086,44087,44088,44089,44090,44091,44092,44093,44094,44095,44096,44097,44098,44099,44100,44101,44102,44103,44104,44105,44106,44107,44108,44109,44110,44111,44112,44113,44114,44115,44116,44117,44118,44119,44120,44121,44122,44123,44124,44125,44126,44127,44128,44129,44130,44131,44132,44133,44134,44135,44136,44137,44138,44139,44140,44141,44142,44143,44144,44145,44146,44147,44148,44149,44150,44151,44152,44153,44154,44155,44156,44157,44158,44159,44160,44161,44162,44163,44164,44165,44166,44167,44168,44169,44170,44171,44172,44173,44174,44175,44176,44177,44178,44179,44180,44181,44182,44183,44184,44185,44186,44187,44188,44189,44190,44191,44192,44193,44194,44195,44196,44197,44198,44199,44200,44201,44202,44203,44204,44205,44206,44207,44208,44209,44210,44211,44212,44213,44214,44215,44216,44217,44218,44219,44220,44221,44222,44223,44224,44225,44226,44227,44228,44229,44230,44231,44232,44233,44234,44235,44236,44237,44238,44239,44240,44241,44242,44243,44244,44245,44246,44247,44248,44249,44250,44251,44252,44253,44254,44255,44256,44257,44258,44259,44260,44261,44262,44263,44264,44265,44266,44267,44268,44269,44270,44271,44272,44273,44274,44275,44276,44277,44278,44279,44280,44281,44282,44283,44284,44285,44286,44287,44288,44289,44290,44291,44292,44293,44294,44295,44296,44297,44298,44299,44300,44301,44302,44303,44304,44305,44306,44307,44308,44309,44310,44311,44312,44313,44314,44315,44316,44317,44318,44319,44320,44321,44322,44323,44324,44325,44326,44327,44328,44329,44330,44331,44332,44333,44334,44335,44336,44337,44338,44339,44340,44341,44342,44343,44344,44345,44346,44347,44348,44349,44350,44351,44352,44353,44354,44355,44356,44357,44358,44359,44360,44361,44362,44363,44364,44365,44366,44367,44368,44369,44370,44371,44372,44373,44374,44375,44376,44377,44378,44379,44380,44381,44382,44383,44384,44385,44386,44387,44388,44389,44390,44391,44392,44393,44394,44395,44396,44397,44398,44399,44400,44401,44402,44403,44404,44405,44406,44407,44408,44409,44410,44411,44412,44413,44414,44415,44416,44417,44418,44419,44420,44421,44422,44423,44424,44425,44426,44427,44428,44429,44430,44431,44432,44433,44434,44435,44436,44437,44438,44439,44440,44441,44442,44443,44444,44445,44446,44447,44448,44449,44450,44451,44452,44453,44454,44455,44456,44457,44458,44459,44460,44461,44462,44463,44464,44465,44466,44467,44468,44469,44470,44471,44472,44473,44474,44475,44476,44477,44478,44479,44480,44481,44482,44483,44484,44485,44486,44487,44488,44489,44490,44491,44492,44493,44494,44495,44496,44497,44498,44499,44500,44501,44502,44503,44504,44505,44506,44507,44508,44509,44510,44511,44512,44513,44514,44515,44516,44517,44518,44519,44520,44521,44522,44523,44524,44525,44526,44527,44528,44529,44530,44531,44532,44533,44534,44535,44536,44537,44538,44539,44540,44541,44542,44543,44544,44545,44546,44547,44548,44549,44550,44551,44552,44553,44554,44555,44556,44557,44558,44559,44560,44561,44562,44563,44564,44565,44566,44567,44568,44569,44570,44571,44572,44573,44574,44575,44576,44577,44578,44579,44580,44581,44582,44583,44584,44585,44586,44587,44588,44589,44590,44591,44592,44593,44594,44595,44596,44597,44598,44599,44600,44601,44602,44603,44604,44605,44606,44607,44608,44609,44610,44611,44612,44613,44614,44615,44616,44617,44618,44619,44620,44621,44622,44623,44624,44625,44626,44627,44628,44629,44630,44631,44632,44633,44634,44635,44636,44637,44638,44639,44640,44641,44642,44643,44644,44645,44646,44647,44648,44649,44650,44651,44652,44653,44654,44655,44656,44657,44658,44659,44660,44661,44662,44663,44664,44665,44666,44667,44668,44669,44670,44671,44672,44673,44674,44675,44676,44677,44678,44679,44680,44681,44682,44683,44684,44685,44686,44687,44688,44689,44690,44691,44692,44693,44694,44695,44696,44697,44698,44699,44700,44701,44702,44703,44704,44705,44706,44707,44708,44709,44710,44711,44712,44713,44714,44715,44716,44717,44718,44719,44720,44721,44722,44723,44724,44725,44726,44727,44728,44729,44730,44731,44732,44733,44734,44735,44736,44737,44738,44739,44740,44741,44742,44743,44744,44745,44746,44747,44748,44749,44750,44751,44752,44753,44754,44755,44756,44757,44758,44759,44760,44761,44762,44763,44764,44765,44766,44767,44768,44769,44770,44771,44772,44773,44774,44775,44776,44777,44778,44779,44780,44781,44782,44783,44784,44785,44786,44787,44788,44789,44790,44791,44792,44793,44794,44795,44796,44797,44798,44799,44800,44801,44802,44803,44804,44805,44806,44807,44808,44809,44810,44811,44812,44813,44814,44815,44816,44817,44818,44819,44820,44821,44822,44823,44824,44825,44826,44827,44828,44829,44830,44831,44832,44833,44834,44835,44836,44837,44838,44839,44840,44841,44842,44843,44844,44845,44846,44847,44848,44849,44850,44851,44852,44853,44854,44855,44856,44857,44858,44859,44860,44861,44862,44863,44864,44865,44866,44867,44868,44869,44870,44871,44872,44873,44874,44875,44876,44877,44878,44879,44880,44881,44882,44883,44884,44885,44886,44887,44888,44889,44890,44891,44892,44893,44894,44895,44896,44897,44898,44899,44900,44901,44902,44903,44904,44905,44906,44907,44908,44909,44910,44911,44912,44913,44914,44915,44916,44917,44918,44919,44920,44921,44922,44923,44924,44925,44926,44927,44928,44929,44930,44931,44932,44933,44934,44935,44936,44937,44938,44939,44940,44941,44942,44943,44944,44945,44946,44947,44948,44949,44950,44951,44952,44953,44954,44955,44956,44957,44958,44959,44960,44961,44962,44963,44964,44965,44966,44967,44968,44969,44970,44971,44972,44973,44974,44975,44976,44977,44978,44979,44980,44981,44982,44983,44984,44985,44986,44987,44988,44989,44990,44991,44992,44993,44994,44995,44996,44997,44998,44999,45000,45001,45002,45003,45004,45005,45006,45007,45008,45009,45010,45011,45012,45013,45014,45015,45016,45017,45018,45019,45020,45021,45022,45023,45024,45025,45026,45027,45028,45029,45030,45031,45032,45033,45034,45035,45036,45037,45038,45039,45040,45041,45042,45043,45044,45045,45046,45047,45048,45049,45050,45051,45052,45053,45054,45055,45056,45057,45058,45059,45060,45061,45062,45063,45064,45065,45066,45067,45068,45069,45070,45071,45072,45073,45074,45075,45076,45077,45078,45079,45080,45081,45082,45083,45084,45085,45086,45087,45088,45089,45090,45091,45092,45093,45094,45095,45096,45097,45098,45099,45100,45101,45102,45103,45104,45105,45106,45107,45108,45109,45110,45111,45112,45113,45114,45115,45116,45117,45118,45119,45120,45121,45122,45123,45124,45125,45126,45127,45128,45129,45130,45131,45132,45133,45134,45135,45136,45137,45138,45139,45140,45141,45142,45143,45144,45145,45146,45147,45148,45149,45150,45151,45152,45153,45154,45155,45156,45157,45158,45159,45160,45161,45162,45163,45164,45165,45166,45167,45168,45169,45170,45171,45172,45173,45174,45175,45176,45177,45178,45179,45180,45181,45182,45183,45184,45185,45186,45187,45188,45189,45190,45191,45192,45193,45194,45195,45196,45197,45198,45199,45200,45201,45202,45203,45204,45205,45206,45207,45208,45209,45210,45211,45212,45213,45214,45215,45216,45217,45218,45219,45220,45221,45222,45223,45224,45225,45226,45227,45228,45229,45230,45231,45232,45233,45234,45235,45236,45237,45238,45239,45240,45241,45242,45243,45244,45245,45246,45247,45248,45249,45250,45251,45252,45253,45254,45255,45256,45257,45258,45259,45260,45261,45262,45263,45264,45265,45266,45267,45268,45269,45270,45271,45272,45273,45274,45275,45276,45277,45278,45279,45280,45281,45282,45283,45284,45285,45286,45287,45288,45289,45290,45291,45292,45293,45294,45295,45296,45297,45298,45299,45300,45301,45302,45303,45304,45305,45306,45307,45308,45309,45310,45311,45312,45313,45314,45315,45316,45317,45318,45319,45320,45321,45322,45323,45324,45325,45326,45327,45328,45329,45330,45331,45332,45333,45334,45335,45336,45337,45338,45339,45340,45341,45342,45343,45344,45345,45346,45347,45348,45349,45350,45351,45352,45353,45354,45355,45356,45357,45358,45359,45360,45361,45362,45363,45364,45365,45366,45367,45368,45369,45370,45371,45372,45373,45374,45375,45376,45377,45378,45379,45380,45381,45382,45383,45384,45385,45386,45387,45388,45389,45390,45391,45392,45393,45394,45395,45396,45397,45398,45399,45400,45401,45402,45403,45404,45405,45406,45407,45408,45409,45410,45411,45412,45413,45414,45415,45416,45417,45418,45419,45420,45421,45422,45423,45424,45425,45426,45427,45428,45429,45430,45431,45432,45433,45434,45435,45436,45437,45438,45439,45440,45441,45442,45443,45444,45445,45446,45447,45448,45449,45450,45451,45452,45453,45454,45455,45456,45457,45458,45459,45460,45461,45462,45463,45464,45465,45466,45467,45468,45469,45470,45471,45472,45473,45474,45475,45476,45477,45478,45479,45480,45481,45482,45483,45484,45485,45486,45487,45488,45489,45490,45491,45492,45493,45494,45495,45496,45497,45498,45499,45500,45501,45502,45503,45504,45505,45506,45507,45508,45509,45510,45511,45512,45513,45514,45515,45516,45517,45518,45519,45520,45521,45522,45523,45524,45525,45526,45527,45528,45529,45530,45531,45532,45533,45534,45535,45536,45537,45538,45539,45540,45541,45542,45543,45544,45545,45546,45547,45548,45549,45550,45551,45552,45553,45554,45555,45556,45557,45558,45559,45560,45561,45562,45563,45564,45565,45566,45567,45568,45569,45570,45571,45572,45573,45574,45575,45576,45577,45578,45579,45580,45581,45582,45583,45584,45585,45586,45587,45588,45589,45590,45591,45592,45593,45594,45595,45596,45597,45598,45599,45600,45601,45602,45603,45604,45605,45606,45607,45608,45609,45610,45611,45612,45613,45614,45615,45616,45617,45618,45619,45620,45621,45622,45623,45624,45625,45626,45627,45628,45629,45630,45631,45632,45633,45634,45635,45636,45637,45638,45639,45640,45641,45642,45643,45644,45645,45646,45647,45648,45649,45650,45651,45652,45653,45654,45655,45656,45657,45658,45659,45660,45661,45662,45663,45664,45665,45666,45667,45668,45669,45670,45671,45672,45673,45674,45675,45676,45677,45678,45679,45680,45681,45682,45683,45684,45685,45686,45687,45688,45689,45690,45691,45692,45693,45694,45695,45696,45697,45698,45699,45700,45701,45702,45703,45704,45705,45706,45707,45708,45709,45710,45711,45712,45713,45714,45715,45716,45717,45718,45719,45720,45721,45722,45723,45724,45725,45726,45727,45728,45729,45730,45731,45732,45733,45734,45735,45736,45737,45738,45739,45740,45741,45742,45743,45744,45745,45746,45747,45748,45749,45750,45751,45752,45753,45754,45755,45756,45757,45758,45759,45760,45761,45762,45763,45764,45765,45766,45767,45768,45769,45770,45771,45772,45773,45774,45775,45776,45777,45778,45779,45780,45781,45782,45783,45784,45785,45786,45787,45788,45789,45790,45791,45792,45793,45794,45795,45796,45797,45798,45799,45800,45801,45802,45803,45804,45805,45806,45807,45808,45809,45810,45811,45812,45813,45814,45815,45816,45817,45818,45819,45820,45821,45822,45823,45824,45825,45826,45827,45828,45829,45830,45831,45832,45833,45834,45835,45836,45837,45838,45839,45840,45841,45842,45843,45844,45845,45846,45847,45848,45849,45850,45851,45852,45853,45854,45855,45856,45857,45858,45859,45860,45861,45862,45863,45864,45865,45866,45867,45868,45869,45870,45871,45872,45873,45874,45875,45876,45877,45878,45879,45880,45881,45882,45883,45884,45885,45886,45887,45888,45889,45890,45891,45892,45893,45894,45895,45896,45897,45898,45899,45900,45901,45902,45903,45904,45905,45906,45907,45908,45909,45910,45911,45912,45913,45914,45915,45916,45917,45918,45919,45920,45921,45922,45923,45924,45925,45926,45927,45928,45929,45930,45931,45932,45933,45934,45935,45936,45937,45938,45939,45940,45941,45942,45943,45944,45945,45946,45947,45948,45949,45950,45951,45952,45953,45954,45955,45956,45957,45958,45959,45960,45961,45962,45963,45964,45965,45966,45967,45968,45969,45970,45971,45972,45973,45974,45975,45976,45977,45978,45979,45980,45981,45982,45983,45984,45985,45986,45987,45988,45989,45990,45991,45992,45993,45994,45995,45996,45997,45998,45999,46000,46001,46002,46003,46004,46005,46006,46007,46008,46009,46010,46011,46012,46013,46014,46015,46016,46017,46018,46019,46020,46021,46022,46023,46024,46025,46026,46027,46028,46029,46030,46031,46032,46033,46034,46035,46036,46037,46038,46039,46040,46041,46042,46043,46044,46045,46046,46047,46048,46049,46050,46051,46052,46053,46054,46055,46056,46057,46058,46059,46060,46061,46062,46063,46064,46065,46066,46067,46068,46069,46070,46071,46072,46073,46074,46075,46076,46077,46078,46079,46080,46081,46082,46083,46084,46085,46086,46087,46088,46089,46090,46091,46092,46093,46094,46095,46096,46097,46098,46099,46100,46101,46102,46103,46104,46105,46106,46107,46108,46109,46110,46111,46112,46113,46114,46115,46116,46117,46118,46119,46120,46121,46122,46123,46124,46125,46126,46127,46128,46129,46130,46131,46132,46133,46134,46135,46136,46137,46138,46139,46140,46141,46142,46143,46144,46145,46146,46147,46148,46149,46150,46151,46152,46153,46154,46155,46156,46157,46158,46159,46160,46161,46162,46163,46164,46165,46166,46167,46168,46169,46170,46171,46172,46173,46174,46175,46176,46177,46178,46179,46180,46181,46182,46183,46184,46185,46186,46187,46188,46189,46190,46191,46192,46193,46194,46195,46196,46197,46198,46199,46200,46201,46202,46203,46204,46205,46206,46207,46208,46209,46210,46211,46212,46213,46214,46215,46216,46217,46218,46219,46220,46221,46222,46223,46224,46225,46226,46227,46228,46229,46230,46231,46232,46233,46234,46235,46236,46237,46238,46239,46240,46241,46242,46243,46244,46245,46246,46247,46248,46249,46250,46251,46252,46253,46254,46255,46256,46257,46258,46259,46260,46261,46262,46263,46264,46265,46266,46267,46268,46269,46270,46271,46272,46273,46274,46275,46276,46277,46278,46279,46280,46281,46282,46283,46284,46285,46286,46287,46288,46289,46290,46291,46292,46293,46294,46295,46296,46297,46298,46299,46300,46301,46302,46303,46304,46305,46306,46307,46308,46309,46310,46311,46312,46313,46314,46315,46316,46317,46318,46319,46320,46321,46322,46323,46324,46325,46326,46327,46328,46329,46330,46331,46332,46333,46334,46335,46336,46337,46338,46339,46340,46341,46342,46343,46344,46345,46346,46347,46348,46349,46350,46351,46352,46353,46354,46355,46356,46357,46358,46359,46360,46361,46362,46363,46364,46365,46366,46367,46368,46369,46370,46371,46372,46373,46374,46375,46376,46377,46378,46379,46380,46381,46382,46383,46384,46385,46386,46387,46388,46389,46390,46391,46392,46393,46394,46395,46396,46397,46398,46399,46400,46401,46402,46403,46404,46405,46406,46407,46408,46409,46410,46411,46412,46413,46414,46415,46416,46417,46418,46419,46420,46421,46422,46423,46424,46425,46426,46427,46428,46429,46430,46431,46432,46433,46434,46435,46436,46437,46438,46439,46440,46441,46442,46443,46444,46445,46446,46447,46448,46449,46450,46451,46452,46453,46454,46455,46456,46457,46458,46459,46460,46461,46462,46463,46464,46465,46466,46467,46468,46469,46470,46471,46472,46473,46474,46475,46476,46477,46478,46479,46480,46481,46482,46483,46484,46485,46486,46487,46488,46489,46490,46491,46492,46493,46494,46495,46496,46497,46498,46499,46500,46501,46502,46503,46504,46505,46506,46507,46508,46509,46510,46511,46512,46513,46514,46515,46516,46517,46518,46519,46520,46521,46522,46523,46524,46525,46526,46527,46528,46529,46530,46531,46532,46533,46534,46535,46536,46537,46538,46539,46540,46541,46542,46543,46544,46545,46546,46547,46548,46549,46550,46551,46552,46553,46554,46555,46556,46557,46558,46559,46560,46561,46562,46563,46564,46565,46566,46567,46568,46569,46570,46571,46572,46573,46574,46575,46576,46577,46578,46579,46580,46581,46582,46583,46584,46585,46586,46587,46588,46589,46590,46591,46592,46593,46594,46595,46596,46597,46598,46599,46600,46601,46602,46603,46604,46605,46606,46607,46608,46609,46610,46611,46612,46613,46614,46615,46616,46617,46618,46619,46620,46621,46622,46623,46624,46625,46626,46627,46628,46629,46630,46631,46632,46633,46634,46635,46636,46637,46638,46639,46640,46641,46642,46643,46644,46645,46646,46647,46648,46649,46650,46651,46652,46653,46654,46655,46656,46657,46658,46659,46660,46661,46662,46663,46664,46665,46666,46667,46668,46669,46670,46671,46672,46673,46674,46675,46676,46677,46678,46679,46680,46681,46682,46683,46684,46685,46686,46687,46688,46689,46690,46691,46692,46693,46694,46695,46696,46697,46698,46699,46700,46701,46702,46703,46704,46705,46706,46707,46708,46709,46710,46711,46712,46713,46714,46715,46716,46717,46718,46719,46720,46721,46722,46723,46724,46725,46726,46727,46728,46729,46730,46731,46732,46733,46734,46735,46736,46737,46738,46739,46740,46741,46742,46743,46744,46745,46746,46747,46748,46749,46750,46751,46752,46753,46754,46755,46756,46757,46758,46759,46760,46761,46762,46763,46764,46765,46766,46767,46768,46769,46770,46771,46772,46773,46774,46775,46776,46777,46778,46779,46780,46781,46782,46783,46784,46785,46786,46787,46788,46789,46790,46791,46792,46793,46794,46795,46796,46797,46798,46799,46800,46801,46802,46803,46804,46805,46806,46807,46808,46809,46810,46811,46812,46813,46814,46815,46816,46817,46818,46819,46820,46821,46822,46823,46824,46825,46826,46827,46828,46829,46830,46831,46832,46833,46834,46835,46836,46837,46838,46839,46840,46841,46842,46843,46844,46845,46846,46847,46848,46849,46850,46851,46852,46853,46854,46855,46856,46857,46858,46859,46860,46861,46862,46863,46864,46865,46866,46867,46868,46869,46870,46871,46872,46873,46874,46875,46876,46877,46878,46879,46880,46881,46882,46883,46884,46885,46886,46887,46888,46889,46890,46891,46892,46893,46894,46895,46896,46897,46898,46899,46900,46901,46902,46903,46904,46905,46906,46907,46908,46909,46910,46911,46912,46913,46914,46915,46916,46917,46918,46919,46920,46921,46922,46923,46924,46925,46926,46927,46928,46929,46930,46931,46932,46933,46934,46935,46936,46937,46938,46939,46940,46941,46942,46943,46944,46945,46946,46947,46948,46949,46950,46951,46952,46953,46954,46955,46956,46957,46958,46959,46960,46961,46962,46963,46964,46965,46966,46967,46968,46969,46970,46971,46972,46973,46974,46975,46976,46977,46978,46979,46980,46981,46982,46983,46984,46985,46986,46987,46988,46989,46990,46991,46992,46993,46994,46995,46996,46997,46998,46999,47000,47001,47002,47003,47004,47005,47006,47007,47008,47009,47010,47011,47012,47013,47014,47015,47016,47017,47018,47019,47020,47021,47022,47023,47024,47025,47026,47027,47028,47029,47030,47031,47032,47033,47034,47035,47036,47037,47038,47039,47040,47041,47042,47043,47044,47045,47046,47047,47048,47049,47050,47051,47052,47053,47054,47055,47056,47057,47058,47059,47060,47061,47062,47063,47064,47065,47066,47067,47068,47069,47070,47071,47072,47073,47074,47075,47076,47077,47078,47079,47080,47081,47082,47083,47084,47085,47086,47087,47088,47089,47090,47091,47092,47093,47094,47095,47096,47097,47098,47099,47100,47101,47102,47103,47104,47105,47106,47107,47108,47109,47110,47111,47112,47113,47114,47115,47116,47117,47118,47119,47120,47121,47122,47123,47124,47125,47126,47127,47128,47129,47130,47131,47132,47133,47134,47135,47136,47137,47138,47139,47140,47141,47142,47143,47144,47145,47146,47147,47148,47149,47150,47151,47152,47153,47154,47155,47156,47157,47158,47159,47160,47161,47162,47163,47164,47165,47166,47167,47168,47169,47170,47171,47172,47173,47174,47175,47176,47177,47178,47179,47180,47181,47182,47183,47184,47185,47186,47187,47188,47189,47190,47191,47192,47193,47194,47195,47196,47197,47198,47199,47200,47201,47202,47203,47204,47205,47206,47207,47208,47209,47210,47211,47212,47213,47214,47215,47216,47217,47218,47219,47220,47221,47222,47223,47224,47225,47226,47227,47228,47229,47230,47231,47232,47233,47234,47235,47236,47237,47238,47239,47240,47241,47242,47243,47244,47245,47246,47247,47248,47249,47250,47251,47252,47253,47254,47255,47256,47257,47258,47259,47260,47261,47262,47263,47264,47265,47266,47267,47268,47269,47270,47271,47272,47273,47274,47275,47276,47277,47278,47279,47280,47281,47282,47283,47284,47285,47286,47287,47288,47289,47290,47291,47292,47293,47294,47295,47296,47297,47298,47299,47300,47301,47302,47303,47304,47305,47306,47307,47308,47309,47310,47311,47312,47313,47314,47315,47316,47317,47318,47319,47320,47321,47322,47323,47324,47325,47326,47327,47328,47329,47330,47331,47332,47333,47334,47335,47336,47337,47338,47339,47340,47341,47342,47343,47344,47345,47346,47347,47348,47349,47350,47351,47352,47353,47354,47355,47356,47357,47358,47359,47360,47361,47362,47363,47364,47365,47366,47367,47368,47369,47370,47371,47372,47373,47374,47375,47376,47377,47378,47379,47380,47381,47382,47383,47384,47385,47386,47387,47388,47389,47390,47391,47392,47393,47394,47395,47396,47397,47398,47399,47400,47401,47402,47403,47404,47405,47406,47407,47408,47409,47410,47411,47412,47413,47414,47415,47416,47417,47418,47419,47420,47421,47422,47423,47424,47425,47426,47427,47428,47429,47430,47431,47432,47433,47434,47435,47436,47437,47438,47439,47440,47441,47442,47443,47444,47445,47446,47447,47448,47449,47450,47451,47452,47453,47454,47455,47456,47457,47458,47459,47460,47461,47462,47463,47464,47465,47466,47467,47468,47469,47470,47471,47472,47473,47474,47475,47476,47477,47478,47479,47480,47481,47482,47483,47484,47485,47486,47487,47488,47489,47490,47491,47492,47493,47494,47495,47496,47497,47498,47499,47500,47501,47502,47503,47504,47505,47506,47507,47508,47509,47510,47511,47512,47513,47514,47515,47516,47517,47518,47519,47520,47521,47522,47523,47524,47525,47526,47527,47528,47529,47530,47531,47532,47533,47534,47535,47536,47537,47538,47539,47540,47541,47542,47543,47544,47545,47546,47547,47548,47549,47550,47551,47552,47553,47554,47555,47556,47557,47558,47559,47560,47561,47562,47563,47564,47565,47566,47567,47568,47569,47570,47571,47572,47573,47574,47575,47576,47577,47578,47579,47580,47581,47582,47583,47584,47585,47586,47587,47588,47589,47590,47591,47592,47593,47594,47595,47596,47597,47598,47599,47600,47601,47602,47603,47604,47605,47606,47607,47608,47609,47610,47611,47612,47613,47614,47615,47616,47617,47618,47619,47620,47621,47622,47623,47624,47625,47626,47627,47628,47629,47630,47631,47632,47633,47634,47635,47636,47637,47638,47639,47640,47641,47642,47643,47644,47645,47646,47647,47648,47649,47650,47651,47652,47653,47654,47655,47656,47657,47658,47659,47660,47661,47662,47663,47664,47665,47666,47667,47668,47669,47670,47671,47672,47673,47674,47675,47676,47677,47678,47679,47680,47681,47682,47683,47684,47685,47686,47687,47688,47689,47690,47691,47692,47693,47694,47695,47696,47697,47698,47699,47700,47701,47702,47703,47704,47705,47706,47707,47708,47709,47710,47711,47712,47713,47714,47715,47716,47717,47718,47719,47720,47721,47722,47723,47724,47725,47726,47727,47728,47729,47730,47731,47732,47733,47734,47735,47736,47737,47738,47739,47740,47741,47742,47743,47744,47745,47746,47747,47748,47749,47750,47751,47752,47753,47754,47755,47756,47757,47758,47759,47760,47761,47762,47763,47764,47765,47766,47767,47768,47769,47770,47771,47772,47773,47774,47775,47776,47777,47778,47779,47780,47781,47782,47783,47784,47785,47786,47787,47788,47789,47790,47791,47792,47793,47794,47795,47796,47797,47798,47799,47800,47801,47802,47803,47804,47805,47806,47807,47808,47809,47810,47811,47812,47813,47814,47815,47816,47817,47818,47819,47820,47821,47822,47823,47824,47825,47826,47827,47828,47829,47830,47831,47832,47833,47834,47835,47836,47837,47838,47839,47840,47841,47842,47843,47844,47845,47846,47847,47848,47849,47850,47851,47852,47853,47854,47855,47856,47857,47858,47859,47860,47861,47862,47863,47864,47865,47866,47867,47868,47869,47870,47871,47872,47873,47874,47875,47876,47877,47878,47879,47880,47881,47882,47883,47884,47885,47886,47887,47888,47889,47890,47891,47892,47893,47894,47895,47896,47897,47898,47899,47900,47901,47902,47903,47904,47905,47906,47907,47908,47909,47910,47911,47912,47913,47914,47915,47916,47917,47918,47919,47920,47921,47922,47923,47924,47925,47926,47927,47928,47929,47930,47931,47932,47933,47934,47935,47936,47937,47938,47939,47940,47941,47942,47943,47944,47945,47946,47947,47948,47949,47950,47951,47952,47953,47954,47955,47956,47957,47958,47959,47960,47961,47962,47963,47964,47965,47966,47967,47968,47969,47970,47971,47972,47973,47974,47975,47976,47977,47978,47979,47980,47981,47982,47983,47984,47985,47986,47987,47988,47989,47990,47991,47992,47993,47994,47995,47996,47997,47998,47999,48000,48001,48002,48003,48004,48005,48006,48007,48008,48009,48010,48011,48012,48013,48014,48015,48016,48017,48018,48019,48020,48021,48022,48023,48024,48025,48026,48027,48028,48029,48030,48031,48032,48033,48034,48035,48036,48037,48038,48039,48040,48041,48042,48043,48044,48045,48046,48047,48048,48049,48050,48051,48052,48053,48054,48055,48056,48057,48058,48059,48060,48061,48062,48063,48064,48065,48066,48067,48068,48069,48070,48071,48072,48073,48074,48075,48076,48077,48078,48079,48080,48081,48082,48083,48084,48085,48086,48087,48088,48089,48090,48091,48092,48093,48094,48095,48096,48097,48098,48099,48100,48101,48102,48103,48104,48105,48106,48107,48108,48109,48110,48111,48112,48113,48114,48115,48116,48117,48118,48119,48120,48121,48122,48123,48124,48125,48126,48127,48128,48129,48130,48131,48132,48133,48134,48135,48136,48137,48138,48139,48140,48141,48142,48143,48144,48145,48146,48147,48148,48149,48150,48151,48152,48153,48154,48155,48156,48157,48158,48159,48160,48161,48162,48163,48164,48165,48166,48167,48168,48169,48170,48171,48172,48173,48174,48175,48176,48177,48178,48179,48180,48181,48182,48183,48184,48185,48186,48187,48188,48189,48190,48191,48192,48193,48194,48195,48196,48197,48198,48199,48200,48201,48202,48203,48204,48205,48206,48207,48208,48209,48210,48211,48212,48213,48214,48215,48216,48217,48218,48219,48220,48221,48222,48223,48224,48225,48226,48227,48228,48229,48230,48231,48232,48233,48234,48235,48236,48237,48238,48239,48240,48241,48242,48243,48244,48245,48246,48247,48248,48249,48250,48251,48252,48253,48254,48255,48256,48257,48258,48259,48260,48261,48262,48263,48264,48265,48266,48267,48268,48269,48270,48271,48272,48273,48274,48275,48276,48277,48278,48279,48280,48281,48282,48283,48284,48285,48286,48287,48288,48289,48290,48291,48292,48293,48294,48295,48296,48297,48298,48299,48300,48301,48302,48303,48304,48305,48306,48307,48308,48309,48310,48311,48312,48313,48314,48315,48316,48317,48318,48319,48320,48321,48322,48323,48324,48325,48326,48327,48328,48329,48330,48331,48332,48333,48334,48335,48336,48337,48338,48339,48340,48341,48342,48343,48344,48345,48346,48347,48348,48349,48350,48351,48352,48353,48354,48355,48356,48357,48358,48359,48360,48361,48362,48363,48364,48365,48366,48367,48368,48369,48370,48371,48372,48373,48374,48375,48376,48377,48378,48379,48380,48381,48382,48383,48384,48385,48386,48387,48388,48389,48390,48391,48392,48393,48394,48395,48396,48397,48398,48399,48400,48401,48402,48403,48404,48405,48406,48407,48408,48409,48410,48411,48412,48413,48414,48415,48416,48417,48418,48419,48420,48421,48422,48423,48424,48425,48426,48427,48428,48429,48430,48431,48432,48433,48434,48435,48436,48437,48438,48439,48440,48441,48442,48443,48444,48445,48446,48447,48448,48449,48450,48451,48452,48453,48454,48455,48456,48457,48458,48459,48460,48461,48462,48463,48464,48465,48466,48467,48468,48469,48470,48471,48472,48473,48474,48475,48476,48477,48478,48479,48480,48481,48482,48483,48484,48485,48486,48487,48488,48489,48490,48491,48492,48493,48494,48495,48496,48497,48498,48499,48500,48501,48502,48503,48504,48505,48506,48507,48508,48509,48510,48511,48512,48513,48514,48515,48516,48517,48518,48519,48520,48521,48522,48523,48524,48525,48526,48527,48528,48529,48530,48531,48532,48533,48534,48535,48536,48537,48538,48539,48540,48541,48542,48543,48544,48545,48546,48547,48548,48549,48550,48551,48552,48553,48554,48555,48556,48557,48558,48559,48560,48561,48562,48563,48564,48565,48566,48567,48568,48569,48570,48571,48572,48573,48574,48575,48576,48577,48578,48579,48580,48581,48582,48583,48584,48585,48586,48587,48588,48589,48590,48591,48592,48593,48594,48595,48596,48597,48598,48599,48600,48601,48602,48603,48604,48605,48606,48607,48608,48609,48610,48611,48612,48613,48614,48615,48616,48617,48618,48619,48620,48621,48622,48623,48624,48625,48626,48627,48628,48629,48630,48631,48632,48633,48634,48635,48636,48637,48638,48639,48640,48641,48642,48643,48644,48645,48646,48647,48648,48649,48650,48651,48652,48653,48654,48655,48656,48657,48658,48659,48660,48661,48662,48663,48664,48665,48666,48667,48668,48669,48670,48671,48672,48673,48674,48675,48676,48677,48678,48679,48680,48681,48682,48683,48684,48685,48686,48687,48688,48689,48690,48691,48692,48693,48694,48695,48696,48697,48698,48699,48700,48701,48702,48703,48704,48705,48706,48707,48708,48709,48710,48711,48712,48713,48714,48715,48716,48717,48718,48719,48720,48721,48722,48723,48724,48725,48726,48727,48728,48729,48730,48731,48732,48733,48734,48735,48736,48737,48738,48739,48740,48741,48742,48743,48744,48745,48746,48747,48748,48749,48750,48751,48752,48753,48754,48755,48756,48757,48758,48759,48760,48761,48762,48763,48764,48765,48766,48767,48768,48769,48770,48771,48772,48773,48774,48775,48776,48777,48778,48779,48780,48781,48782,48783,48784,48785,48786,48787,48788,48789,48790,48791,48792,48793,48794,48795,48796,48797,48798,48799,48800,48801,48802,48803,48804,48805,48806,48807,48808,48809,48810,48811,48812,48813,48814,48815,48816,48817,48818,48819,48820,48821,48822,48823,48824,48825,48826,48827,48828,48829,48830,48831,48832,48833,48834,48835,48836,48837,48838,48839,48840,48841,48842,48843,48844,48845,48846,48847,48848,48849,48850,48851,48852,48853,48854,48855,48856,48857,48858,48859,48860,48861,48862,48863,48864,48865,48866,48867,48868,48869,48870,48871,48872,48873,48874,48875,48876,48877,48878,48879,48880,48881,48882,48883,48884,48885,48886,48887,48888,48889,48890,48891,48892,48893,48894,48895,48896,48897,48898,48899,48900,48901,48902,48903,48904,48905,48906,48907,48908,48909,48910,48911,48912,48913,48914,48915,48916,48917,48918,48919,48920,48921,48922,48923,48924,48925,48926,48927,48928,48929,48930,48931,48932,48933,48934,48935,48936,48937,48938,48939,48940,48941,48942,48943,48944,48945,48946,48947,48948,48949,48950,48951,48952,48953,48954,48955,48956,48957,48958,48959,48960,48961,48962,48963,48964,48965,48966,48967,48968,48969,48970,48971,48972,48973,48974,48975,48976,48977,48978,48979,48980,48981,48982,48983,48984,48985,48986,48987,48988,48989,48990,48991,48992,48993,48994,48995,48996,48997,48998,48999,49000,49001,49002,49003,49004,49005,49006,49007,49008,49009,49010,49011,49012,49013,49014,49015,49016,49017,49018,49019,49020,49021,49022,49023,49024,49025,49026,49027,49028,49029,49030,49031,49032,49033,49034,49035,49036,49037,49038,49039,49040,49041,49042,49043,49044,49045,49046,49047,49048,49049,49050,49051,49052,49053,49054,49055,49056,49057,49058,49059,49060,49061,49062,49063,49064,49065,49066,49067,49068,49069,49070,49071,49072,49073,49074,49075,49076,49077,49078,49079,49080,49081,49082,49083,49084,49085,49086,49087,49088,49089,49090,49091,49092,49093,49094,49095,49096,49097,49098,49099,49100,49101,49102,49103,49104,49105,49106,49107,49108,49109,49110,49111,49112,49113,49114,49115,49116,49117,49118,49119,49120,49121,49122,49123,49124,49125,49126,49127,49128,49129,49130,49131,49132,49133,49134,49135,49136,49137,49138,49139,49140,49141,49142,49143,49144,49145,49146,49147,49148,49149,49150,49151,49152,49153,49154,49155,49156,49157,49158,49159,49160,49161,49162,49163,49164,49165,49166,49167,49168,49169,49170,49171,49172,49173,49174,49175,49176,49177,49178,49179,49180,49181,49182,49183,49184,49185,49186,49187,49188,49189,49190,49191,49192,49193,49194,49195,49196,49197,49198,49199,49200,49201,49202,49203,49204,49205,49206,49207,49208,49209,49210,49211,49212,49213,49214,49215,49216,49217,49218,49219,49220,49221,49222,49223,49224,49225,49226,49227,49228,49229,49230,49231,49232,49233,49234,49235,49236,49237,49238,49239,49240,49241,49242,49243,49244,49245,49246,49247,49248,49249,49250,49251,49252,49253,49254,49255,49256,49257,49258,49259,49260,49261,49262,49263,49264,49265,49266,49267,49268,49269,49270,49271,49272,49273,49274,49275,49276,49277,49278,49279,49280,49281,49282,49283,49284,49285,49286,49287,49288,49289,49290,49291,49292,49293,49294,49295,49296,49297,49298,49299,49300,49301,49302,49303,49304,49305,49306,49307,49308,49309,49310,49311,49312,49313,49314,49315,49316,49317,49318,49319,49320,49321,49322,49323,49324,49325,49326,49327,49328,49329,49330,49331,49332,49333,49334,49335,49336,49337,49338,49339,49340,49341,49342,49343,49344,49345,49346,49347,49348,49349,49350,49351,49352,49353,49354,49355,49356,49357,49358,49359,49360,49361,49362,49363,49364,49365,49366,49367,49368,49369,49370,49371,49372,49373,49374,49375,49376,49377,49378,49379,49380,49381,49382,49383,49384,49385,49386,49387,49388,49389,49390,49391,49392,49393,49394,49395,49396,49397,49398,49399,49400,49401,49402,49403,49404,49405,49406,49407,49408,49409,49410,49411,49412,49413,49414,49415,49416,49417,49418,49419,49420,49421,49422,49423,49424,49425,49426,49427,49428,49429,49430,49431,49432,49433,49434,49435,49436,49437,49438,49439,49440,49441,49442,49443,49444,49445,49446,49447,49448,49449,49450,49451,49452,49453,49454,49455,49456,49457,49458,49459,49460,49461,49462,49463,49464,49465,49466,49467,49468,49469,49470,49471,49472,49473,49474,49475,49476,49477,49478,49479,49480,49481,49482,49483,49484,49485,49486,49487,49488,49489,49490,49491,49492,49493,49494,49495,49496,49497,49498,49499,49500,49501,49502,49503,49504,49505,49506,49507,49508,49509,49510,49511,49512,49513,49514,49515,49516,49517,49518,49519,49520,49521,49522,49523,49524,49525,49526,49527,49528,49529,49530,49531,49532,49533,49534,49535,49536,49537,49538,49539,49540,49541,49542,49543,49544,49545,49546,49547,49548,49549,49550,49551,49552,49553,49554,49555,49556,49557,49558,49559,49560,49561,49562,49563,49564,49565,49566,49567,49568,49569,49570,49571,49572,49573,49574,49575,49576,49577,49578,49579,49580,49581,49582,49583,49584,49585,49586,49587,49588,49589,49590,49591,49592,49593,49594,49595,49596,49597,49598,49599,49600,49601,49602,49603,49604,49605,49606,49607,49608,49609,49610,49611,49612,49613,49614,49615,49616,49617,49618,49619,49620,49621,49622,49623,49624,49625,49626,49627,49628,49629,49630,49631,49632,49633,49634,49635,49636,49637,49638,49639,49640,49641,49642,49643,49644,49645,49646,49647,49648,49649,49650,49651,49652,49653,49654,49655,49656,49657,49658,49659,49660,49661,49662,49663,49664,49665,49666,49667,49668,49669,49670,49671,49672,49673,49674,49675,49676,49677,49678,49679,49680,49681,49682,49683,49684,49685,49686,49687,49688,49689,49690,49691,49692,49693,49694,49695,49696,49697,49698,49699,49700,49701,49702,49703,49704,49705,49706,49707,49708,49709,49710,49711,49712,49713,49714,49715,49716,49717,49718,49719,49720,49721,49722,49723,49724,49725,49726,49727,49728,49729,49730,49731,49732,49733,49734,49735,49736,49737,49738,49739,49740,49741,49742,49743,49744,49745,49746,49747,49748,49749,49750,49751,49752,49753,49754,49755,49756,49757,49758,49759,49760,49761,49762,49763,49764,49765,49766,49767,49768,49769,49770,49771,49772,49773,49774,49775,49776,49777,49778,49779,49780,49781,49782,49783,49784,49785,49786,49787,49788,49789,49790,49791,49792,49793,49794,49795,49796,49797,49798,49799,49800,49801,49802,49803,49804,49805,49806,49807,49808,49809,49810,49811,49812,49813,49814,49815,49816,49817,49818,49819,49820,49821,49822,49823,49824,49825,49826,49827,49828,49829,49830,49831,49832,49833,49834,49835,49836,49837,49838,49839,49840,49841,49842,49843,49844,49845,49846,49847,49848,49849,49850,49851,49852,49853,49854,49855,49856,49857,49858,49859,49860,49861,49862,49863,49864,49865,49866,49867,49868,49869,49870,49871,49872,49873,49874,49875,49876,49877,49878,49879,49880,49881,49882,49883,49884,49885,49886,49887,49888,49889,49890,49891,49892,49893,49894,49895,49896,49897,49898,49899,49900,49901,49902,49903,49904,49905,49906,49907,49908,49909,49910,49911,49912,49913,49914,49915,49916,49917,49918,49919,49920,49921,49922,49923,49924,49925,49926,49927,49928,49929,49930,49931,49932,49933,49934,49935,49936,49937,49938,49939,49940,49941,49942,49943,49944,49945,49946,49947,49948,49949,49950,49951,49952,49953,49954,49955,49956,49957,49958,49959,49960,49961,49962,49963,49964,49965,49966,49967,49968,49969,49970,49971,49972,49973,49974,49975,49976,49977,49978,49979,49980,49981,49982,49983,49984,49985,49986,49987,49988,49989,49990,49991,49992,49993,49994,49995,49996,49997,49998,49999,50000,50001,50002,50003,50004,50005,50006,50007,50008,50009,50010,50011,50012,50013,50014,50015,50016,50017,50018,50019,50020,50021,50022,50023,50024,50025,50026,50027,50028,50029,50030,50031,50032,50033,50034,50035,50036,50037,50038,50039,50040,50041,50042,50043,50044,50045,50046,50047,50048,50049,50050,50051,50052,50053,50054,50055,50056,50057,50058,50059,50060,50061,50062,50063,50064,50065,50066,50067,50068,50069,50070,50071,50072,50073,50074,50075,50076,50077,50078,50079,50080,50081,50082,50083,50084,50085,50086,50087,50088,50089,50090,50091,50092,50093,50094,50095,50096,50097,50098,50099,50100,50101,50102,50103,50104,50105,50106,50107,50108,50109,50110,50111,50112,50113,50114,50115,50116,50117,50118,50119,50120,50121,50122,50123,50124,50125,50126,50127,50128,50129,50130,50131,50132,50133,50134,50135,50136,50137,50138,50139,50140,50141,50142,50143,50144,50145,50146,50147,50148,50149,50150,50151,50152,50153,50154,50155,50156,50157,50158,50159,50160,50161,50162,50163,50164,50165,50166,50167,50168,50169,50170,50171,50172,50173,50174,50175,50176,50177,50178,50179,50180,50181,50182,50183,50184,50185,50186,50187,50188,50189,50190,50191,50192,50193,50194,50195,50196,50197,50198,50199,50200,50201,50202,50203,50204,50205,50206,50207,50208,50209,50210,50211,50212,50213,50214,50215,50216,50217,50218,50219,50220,50221,50222,50223,50224,50225,50226,50227,50228,50229,50230,50231,50232,50233,50234,50235,50236,50237,50238,50239,50240,50241,50242,50243,50244,50245,50246,50247,50248,50249,50250,50251,50252,50253,50254,50255,50256,50257,50258,50259,50260,50261,50262,50263,50264,50265,50266,50267,50268,50269,50270,50271,50272,50273,50274,50275,50276,50277,50278,50279,50280,50281,50282,50283,50284,50285,50286,50287,50288,50289,50290,50291,50292,50293,50294,50295,50296,50297,50298,50299,50300,50301,50302,50303,50304,50305,50306,50307,50308,50309,50310,50311,50312,50313,50314,50315,50316,50317,50318,50319,50320,50321,50322,50323,50324,50325,50326,50327,50328,50329,50330,50331,50332,50333,50334,50335,50336,50337,50338,50339,50340,50341,50342,50343,50344,50345,50346,50347,50348,50349,50350,50351,50352,50353,50354,50355,50356,50357,50358,50359,50360,50361,50362,50363,50364,50365,50366,50367,50368,50369,50370,50371,50372,50373,50374,50375,50376,50377,50378,50379,50380,50381,50382,50383,50384,50385,50386,50387,50388,50389,50390,50391,50392,50393,50394,50395,50396,50397,50398,50399,50400,50401,50402,50403,50404,50405,50406,50407,50408,50409,50410,50411,50412,50413,50414,50415,50416,50417,50418,50419,50420,50421,50422,50423,50424,50425,50426,50427,50428,50429,50430,50431,50432,50433,50434,50435,50436,50437,50438,50439,50440,50441,50442,50443,50444,50445,50446,50447,50448,50449,50450,50451,50452,50453,50454,50455,50456,50457,50458,50459,50460,50461,50462,50463,50464,50465,50466,50467,50468,50469,50470,50471,50472,50473,50474,50475,50476,50477,50478,50479,50480,50481,50482,50483,50484,50485,50486,50487,50488,50489,50490,50491,50492,50493,50494,50495,50496,50497,50498,50499,50500,50501,50502,50503,50504,50505,50506,50507,50508,50509,50510,50511,50512,50513,50514,50515,50516,50517,50518,50519,50520,50521,50522,50523,50524,50525,50526,50527,50528,50529,50530,50531,50532,50533,50534,50535,50536,50537,50538,50539,50540,50541,50542,50543,50544,50545,50546,50547,50548,50549,50550,50551,50552,50553,50554,50555,50556,50557,50558,50559,50560,50561,50562,50563,50564,50565,50566,50567,50568,50569,50570,50571,50572,50573,50574,50575,50576,50577,50578,50579,50580,50581,50582,50583,50584,50585,50586,50587,50588,50589,50590,50591,50592,50593,50594,50595,50596,50597,50598,50599,50600,50601,50602,50603,50604,50605,50606,50607,50608,50609,50610,50611,50612,50613,50614,50615,50616,50617,50618,50619,50620,50621,50622,50623,50624,50625,50626,50627,50628,50629,50630,50631,50632,50633,50634,50635,50636,50637,50638,50639,50640,50641,50642,50643,50644,50645,50646,50647,50648,50649,50650,50651,50652,50653,50654,50655,50656,50657,50658,50659,50660,50661,50662,50663,50664,50665,50666,50667,50668,50669,50670,50671,50672,50673,50674,50675,50676,50677,50678,50679,50680,50681,50682,50683,50684,50685,50686,50687,50688,50689,50690,50691,50692,50693,50694,50695,50696,50697,50698,50699,50700,50701,50702,50703,50704,50705,50706,50707,50708,50709,50710,50711,50712,50713,50714,50715,50716,50717,50718,50719,50720,50721,50722,50723,50724,50725,50726,50727,50728,50729,50730,50731,50732,50733,50734,50735,50736,50737,50738,50739,50740,50741,50742,50743,50744,50745,50746,50747,50748,50749,50750,50751,50752,50753,50754,50755,50756,50757,50758,50759,50760,50761,50762,50763,50764,50765,50766,50767,50768,50769,50770,50771,50772,50773,50774,50775,50776,50777,50778,50779,50780,50781,50782,50783,50784,50785,50786,50787,50788,50789,50790,50791,50792,50793,50794,50795,50796,50797,50798,50799,50800,50801,50802,50803,50804,50805,50806,50807,50808,50809,50810,50811,50812,50813,50814,50815,50816,50817,50818,50819,50820,50821,50822,50823,50824,50825,50826,50827,50828,50829,50830,50831,50832,50833,50834,50835,50836,50837,50838,50839,50840,50841,50842,50843,50844,50845,50846,50847,50848,50849,50850,50851,50852,50853,50854,50855,50856,50857,50858,50859,50860,50861,50862,50863,50864,50865,50866,50867,50868,50869,50870,50871,50872,50873,50874,50875,50876,50877,50878,50879,50880,50881,50882,50883,50884,50885,50886,50887,50888,50889,50890,50891,50892,50893,50894,50895,50896,50897,50898,50899,50900,50901,50902,50903,50904,50905,50906,50907,50908,50909,50910,50911,50912,50913,50914,50915,50916,50917,50918,50919,50920,50921,50922,50923,50924,50925,50926,50927,50928,50929,50930,50931,50932,50933,50934,50935,50936,50937,50938,50939,50940,50941,50942,50943,50944,50945,50946,50947,50948,50949,50950,50951,50952,50953,50954,50955,50956,50957,50958,50959,50960,50961,50962,50963,50964,50965,50966,50967,50968,50969,50970,50971,50972,50973,50974,50975,50976,50977,50978,50979,50980,50981,50982,50983,50984,50985,50986,50987,50988,50989,50990,50991,50992,50993,50994,50995,50996,50997,50998,50999,51000,51001,51002,51003,51004,51005,51006,51007,51008,51009,51010,51011,51012,51013,51014,51015,51016,51017,51018,51019,51020,51021,51022,51023,51024,51025,51026,51027,51028,51029,51030,51031,51032,51033,51034,51035,51036,51037,51038,51039,51040,51041,51042,51043,51044,51045,51046,51047,51048,51049,51050,51051,51052,51053,51054,51055,51056,51057,51058,51059,51060,51061,51062,51063,51064,51065,51066,51067,51068,51069,51070,51071,51072,51073,51074,51075,51076,51077,51078,51079,51080,51081,51082,51083,51084,51085,51086,51087,51088,51089,51090,51091,51092,51093,51094,51095,51096,51097,51098,51099,51100,51101,51102,51103,51104,51105,51106,51107,51108,51109,51110,51111,51112,51113,51114,51115,51116,51117,51118,51119,51120,51121,51122,51123,51124,51125,51126,51127,51128,51129,51130,51131,51132,51133,51134,51135,51136,51137,51138,51139,51140,51141,51142,51143,51144,51145,51146,51147,51148,51149,51150,51151,51152,51153,51154,51155,51156,51157,51158,51159,51160,51161,51162,51163,51164,51165,51166,51167,51168,51169,51170,51171,51172,51173,51174,51175,51176,51177,51178,51179,51180,51181,51182,51183,51184,51185,51186,51187,51188,51189,51190,51191,51192,51193,51194,51195,51196,51197,51198,51199,51200,51201,51202,51203,51204,51205,51206,51207,51208,51209,51210,51211,51212,51213,51214,51215,51216,51217,51218,51219,51220,51221,51222,51223,51224,51225,51226,51227,51228,51229,51230,51231,51232,51233,51234,51235,51236,51237,51238,51239,51240,51241,51242,51243,51244,51245,51246,51247,51248,51249,51250,51251,51252,51253,51254,51255,51256,51257,51258,51259,51260,51261,51262,51263,51264,51265,51266,51267,51268,51269,51270,51271,51272,51273,51274,51275,51276,51277,51278,51279,51280,51281,51282,51283,51284,51285,51286,51287,51288,51289,51290,51291,51292,51293,51294,51295,51296,51297,51298,51299,51300,51301,51302,51303,51304,51305,51306,51307,51308,51309,51310,51311,51312,51313,51314,51315,51316,51317,51318,51319,51320,51321,51322,51323,51324,51325,51326,51327,51328,51329,51330,51331,51332,51333,51334,51335,51336,51337,51338,51339,51340,51341,51342,51343,51344,51345,51346,51347,51348,51349,51350,51351,51352,51353,51354,51355,51356,51357,51358,51359,51360,51361,51362,51363,51364,51365,51366,51367,51368,51369,51370,51371,51372,51373,51374,51375,51376,51377,51378,51379,51380,51381,51382,51383,51384,51385,51386,51387,51388,51389,51390,51391,51392,51393,51394,51395,51396,51397,51398,51399,51400,51401,51402,51403,51404,51405,51406,51407,51408,51409,51410,51411,51412,51413,51414,51415,51416,51417,51418,51419,51420,51421,51422,51423,51424,51425,51426,51427,51428,51429,51430,51431,51432,51433,51434,51435,51436,51437,51438,51439,51440,51441,51442,51443,51444,51445,51446,51447,51448,51449,51450,51451,51452,51453,51454,51455,51456,51457,51458,51459,51460,51461,51462,51463,51464,51465,51466,51467,51468,51469,51470,51471,51472,51473,51474,51475,51476,51477,51478,51479,51480,51481,51482,51483,51484,51485,51486,51487,51488,51489,51490,51491,51492,51493,51494,51495,51496,51497,51498,51499,51500,51501,51502,51503,51504,51505,51506,51507,51508,51509,51510,51511,51512,51513,51514,51515,51516,51517,51518,51519,51520,51521,51522,51523,51524,51525,51526,51527,51528,51529,51530,51531,51532,51533,51534,51535,51536,51537,51538,51539,51540,51541,51542,51543,51544,51545,51546,51547,51548,51549,51550,51551,51552,51553,51554,51555,51556,51557,51558,51559,51560,51561,51562,51563,51564,51565,51566,51567,51568,51569,51570,51571,51572,51573,51574,51575,51576,51577,51578,51579,51580,51581,51582,51583,51584,51585,51586,51587,51588,51589,51590,51591,51592,51593,51594,51595,51596,51597,51598,51599,51600,51601,51602,51603,51604,51605,51606,51607,51608,51609,51610,51611,51612,51613,51614,51615,51616,51617,51618,51619,51620,51621,51622,51623,51624,51625,51626,51627,51628,51629,51630,51631,51632,51633,51634,51635,51636,51637,51638,51639,51640,51641,51642,51643,51644,51645,51646,51647,51648,51649,51650,51651,51652,51653,51654,51655,51656,51657,51658,51659,51660,51661,51662,51663,51664,51665,51666,51667,51668,51669,51670,51671,51672,51673,51674,51675,51676,51677,51678,51679,51680,51681,51682,51683,51684,51685,51686,51687,51688,51689,51690,51691,51692,51693,51694,51695,51696,51697,51698,51699,51700,51701,51702,51703,51704,51705,51706,51707,51708,51709,51710,51711,51712,51713,51714,51715,51716,51717,51718,51719,51720,51721,51722,51723,51724,51725,51726,51727,51728,51729,51730,51731,51732,51733,51734,51735,51736,51737,51738,51739,51740,51741,51742,51743,51744,51745,51746,51747,51748,51749,51750,51751,51752,51753,51754,51755,51756,51757,51758,51759,51760,51761,51762,51763,51764,51765,51766,51767,51768,51769,51770,51771,51772,51773,51774,51775,51776,51777,51778,51779,51780,51781,51782,51783,51784,51785,51786,51787,51788,51789,51790,51791,51792,51793,51794,51795,51796,51797,51798,51799,51800,51801,51802,51803,51804,51805,51806,51807,51808,51809,51810,51811,51812,51813,51814,51815,51816,51817,51818,51819,51820,51821,51822,51823,51824,51825,51826,51827,51828,51829,51830,51831,51832,51833,51834,51835,51836,51837,51838,51839,51840,51841,51842,51843,51844,51845,51846,51847,51848,51849,51850,51851,51852,51853,51854,51855,51856,51857,51858,51859,51860,51861,51862,51863,51864,51865,51866,51867,51868,51869,51870,51871,51872,51873,51874,51875,51876,51877,51878,51879,51880,51881,51882,51883,51884,51885,51886,51887,51888,51889,51890,51891,51892,51893,51894,51895,51896,51897,51898,51899,51900,51901,51902,51903,51904,51905,51906,51907,51908,51909,51910,51911,51912,51913,51914,51915,51916,51917,51918,51919,51920,51921,51922,51923,51924,51925,51926,51927,51928,51929,51930,51931,51932,51933,51934,51935,51936,51937,51938,51939,51940,51941,51942,51943,51944,51945,51946,51947,51948,51949,51950,51951,51952,51953,51954,51955,51956,51957,51958,51959,51960,51961,51962,51963,51964,51965,51966,51967,51968,51969,51970,51971,51972,51973,51974,51975,51976,51977,51978,51979,51980,51981,51982,51983,51984,51985,51986,51987,51988,51989,51990,51991,51992,51993,51994,51995,51996,51997,51998,51999,52000,52001,52002,52003,52004,52005,52006,52007,52008,52009,52010,52011,52012,52013,52014,52015,52016,52017,52018,52019,52020,52021,52022,52023,52024,52025,52026,52027,52028,52029,52030,52031,52032,52033,52034,52035,52036,52037,52038,52039,52040,52041,52042,52043,52044,52045,52046,52047,52048,52049,52050,52051,52052,52053,52054,52055,52056,52057,52058,52059,52060,52061,52062,52063,52064,52065,52066,52067,52068,52069,52070,52071,52072,52073,52074,52075,52076,52077,52078,52079,52080,52081,52082,52083,52084,52085,52086,52087,52088,52089,52090,52091,52092,52093,52094,52095,52096,52097,52098,52099,52100,52101,52102,52103,52104,52105,52106,52107,52108,52109,52110,52111,52112,52113,52114,52115,52116,52117,52118,52119,52120,52121,52122,52123,52124,52125,52126,52127,52128,52129,52130,52131,52132,52133,52134,52135,52136,52137,52138,52139,52140,52141,52142,52143,52144,52145,52146,52147,52148,52149,52150,52151,52152,52153,52154,52155,52156,52157,52158,52159,52160,52161,52162,52163,52164,52165,52166,52167,52168,52169,52170,52171,52172,52173,52174,52175,52176,52177,52178,52179,52180,52181,52182,52183,52184,52185,52186,52187,52188,52189,52190,52191,52192,52193,52194,52195,52196,52197,52198,52199,52200,52201,52202,52203,52204,52205,52206,52207,52208,52209,52210,52211,52212,52213,52214,52215,52216,52217,52218,52219,52220,52221,52222,52223,52224,52225,52226,52227,52228,52229,52230,52231,52232,52233,52234,52235,52236,52237,52238,52239,52240,52241,52242,52243,52244,52245,52246,52247,52248,52249,52250,52251,52252,52253,52254,52255,52256,52257,52258,52259,52260,52261,52262,52263,52264,52265,52266,52267,52268,52269,52270,52271,52272,52273,52274,52275,52276,52277,52278,52279,52280,52281,52282,52283,52284,52285,52286,52287,52288,52289,52290,52291,52292,52293,52294,52295,52296,52297,52298,52299,52300,52301,52302,52303,52304,52305,52306,52307,52308,52309,52310,52311,52312,52313,52314,52315,52316,52317,52318,52319,52320,52321,52322,52323,52324,52325,52326,52327,52328,52329,52330,52331,52332,52333,52334,52335,52336,52337,52338,52339,52340,52341,52342,52343,52344,52345,52346,52347,52348,52349,52350,52351,52352,52353,52354,52355,52356,52357,52358,52359,52360,52361,52362,52363,52364,52365,52366,52367,52368,52369,52370,52371,52372,52373,52374,52375,52376,52377,52378,52379,52380,52381,52382,52383,52384,52385,52386,52387,52388,52389,52390,52391,52392,52393,52394,52395,52396,52397,52398,52399,52400,52401,52402,52403,52404,52405,52406,52407,52408,52409,52410,52411,52412,52413,52414,52415,52416,52417,52418,52419,52420,52421,52422,52423,52424,52425,52426,52427,52428,52429,52430,52431,52432,52433,52434,52435,52436,52437,52438,52439,52440,52441,52442,52443,52444,52445,52446,52447,52448,52449,52450,52451,52452,52453,52454,52455,52456,52457,52458,52459,52460,52461,52462,52463,52464,52465,52466,52467,52468,52469,52470,52471,52472,52473,52474,52475,52476,52477,52478,52479,52480,52481,52482,52483,52484,52485,52486,52487,52488,52489,52490,52491,52492,52493,52494,52495,52496,52497,52498,52499,52500,52501,52502,52503,52504,52505,52506,52507,52508,52509,52510,52511,52512,52513,52514,52515,52516,52517,52518,52519,52520,52521,52522,52523,52524,52525,52526,52527,52528,52529,52530,52531,52532,52533,52534,52535,52536,52537,52538,52539,52540,52541,52542,52543,52544,52545,52546,52547,52548,52549,52550,52551,52552,52553,52554,52555,52556,52557,52558,52559,52560,52561,52562,52563,52564,52565,52566,52567,52568,52569,52570,52571,52572,52573,52574,52575,52576,52577,52578,52579,52580,52581,52582,52583,52584,52585,52586,52587,52588,52589,52590,52591,52592,52593,52594,52595,52596,52597,52598,52599,52600,52601,52602,52603,52604,52605,52606,52607,52608,52609,52610,52611,52612,52613,52614,52615,52616,52617,52618,52619,52620,52621,52622,52623,52624,52625,52626,52627,52628,52629,52630,52631,52632,52633,52634,52635,52636,52637,52638,52639,52640,52641,52642,52643,52644,52645,52646,52647,52648,52649,52650,52651,52652,52653,52654,52655,52656,52657,52658,52659,52660,52661,52662,52663,52664,52665,52666,52667,52668,52669,52670,52671,52672,52673,52674,52675,52676,52677,52678,52679,52680,52681,52682,52683,52684,52685,52686,52687,52688,52689,52690,52691,52692,52693,52694,52695,52696,52697,52698,52699,52700,52701,52702,52703,52704,52705,52706,52707,52708,52709,52710,52711,52712,52713,52714,52715,52716,52717,52718,52719,52720,52721,52722,52723,52724,52725,52726,52727,52728,52729,52730,52731,52732,52733,52734,52735,52736,52737,52738,52739,52740,52741,52742,52743,52744,52745,52746,52747,52748,52749,52750,52751,52752,52753,52754,52755,52756,52757,52758,52759,52760,52761,52762,52763,52764,52765,52766,52767,52768,52769,52770,52771,52772,52773,52774,52775,52776,52777,52778,52779,52780,52781,52782,52783,52784,52785,52786,52787,52788,52789,52790,52791,52792,52793,52794,52795,52796,52797,52798,52799,52800,52801,52802,52803,52804,52805,52806,52807,52808,52809,52810,52811,52812,52813,52814,52815,52816,52817,52818,52819,52820,52821,52822,52823,52824,52825,52826,52827,52828,52829,52830,52831,52832,52833,52834,52835,52836,52837,52838,52839,52840,52841,52842,52843,52844,52845,52846,52847,52848,52849,52850,52851,52852,52853,52854,52855,52856,52857,52858,52859,52860,52861,52862,52863,52864,52865,52866,52867,52868,52869,52870,52871,52872,52873,52874,52875,52876,52877,52878,52879,52880,52881,52882,52883,52884,52885,52886,52887,52888,52889,52890,52891,52892,52893,52894,52895,52896,52897,52898,52899,52900,52901,52902,52903,52904,52905,52906,52907,52908,52909,52910,52911,52912,52913,52914,52915,52916,52917,52918,52919,52920,52921,52922,52923,52924,52925,52926,52927,52928,52929,52930,52931,52932,52933,52934,52935,52936,52937,52938,52939,52940,52941,52942,52943,52944,52945,52946,52947,52948,52949,52950,52951,52952,52953,52954,52955,52956,52957,52958,52959,52960,52961,52962,52963,52964,52965,52966,52967,52968,52969,52970,52971,52972,52973,52974,52975,52976,52977,52978,52979,52980,52981,52982,52983,52984,52985,52986,52987,52988,52989,52990,52991,52992,52993,52994,52995,52996,52997,52998,52999,53000,53001,53002,53003,53004,53005,53006,53007,53008,53009,53010,53011,53012,53013,53014,53015,53016,53017,53018,53019,53020,53021,53022,53023,53024,53025,53026,53027,53028,53029,53030,53031,53032,53033,53034,53035,53036,53037,53038,53039,53040,53041,53042,53043,53044,53045,53046,53047,53048,53049,53050,53051,53052,53053,53054,53055,53056,53057,53058,53059,53060,53061,53062,53063,53064,53065,53066,53067,53068,53069,53070,53071,53072,53073,53074,53075,53076,53077,53078,53079,53080,53081,53082,53083,53084,53085,53086,53087,53088,53089,53090,53091,53092,53093,53094,53095,53096,53097,53098,53099,53100,53101,53102,53103,53104,53105,53106,53107,53108,53109,53110,53111,53112,53113,53114,53115,53116,53117,53118,53119,53120,53121,53122,53123,53124,53125,53126,53127,53128,53129,53130,53131,53132,53133,53134,53135,53136,53137,53138,53139,53140,53141,53142,53143,53144,53145,53146,53147,53148,53149,53150,53151,53152,53153,53154,53155,53156,53157,53158,53159,53160,53161,53162,53163,53164,53165,53166,53167,53168,53169,53170,53171,53172,53173,53174,53175,53176,53177,53178,53179,53180,53181,53182,53183,53184,53185,53186,53187,53188,53189,53190,53191,53192,53193,53194,53195,53196,53197,53198,53199,53200,53201,53202,53203,53204,53205,53206,53207,53208,53209,53210,53211,53212,53213,53214,53215,53216,53217,53218,53219,53220,53221,53222,53223,53224,53225,53226,53227,53228,53229,53230,53231,53232,53233,53234,53235,53236,53237,53238,53239,53240,53241,53242,53243,53244,53245,53246,53247,53248,53249,53250,53251,53252,53253,53254,53255,53256,53257,53258,53259,53260,53261,53262,53263,53264,53265,53266,53267,53268,53269,53270,53271,53272,53273,53274,53275,53276,53277,53278,53279,53280,53281,53282,53283,53284,53285,53286,53287,53288,53289,53290,53291,53292,53293,53294,53295,53296,53297,53298,53299,53300,53301,53302,53303,53304,53305,53306,53307,53308,53309,53310,53311,53312,53313,53314,53315,53316,53317,53318,53319,53320,53321,53322,53323,53324,53325,53326,53327,53328,53329,53330,53331,53332,53333,53334,53335,53336,53337,53338,53339,53340,53341,53342,53343,53344,53345,53346,53347,53348,53349,53350,53351,53352,53353,53354,53355,53356,53357,53358,53359,53360,53361,53362,53363,53364,53365,53366,53367,53368,53369,53370,53371,53372,53373,53374,53375,53376,53377,53378,53379,53380,53381,53382,53383,53384,53385,53386,53387,53388,53389,53390,53391,53392,53393,53394,53395,53396,53397,53398,53399,53400,53401,53402,53403,53404,53405,53406,53407,53408,53409,53410,53411,53412,53413,53414,53415,53416,53417,53418,53419,53420,53421,53422,53423,53424,53425,53426,53427,53428,53429,53430,53431,53432,53433,53434,53435,53436,53437,53438,53439,53440,53441,53442,53443,53444,53445,53446,53447,53448,53449,53450,53451,53452,53453,53454,53455,53456,53457,53458,53459,53460,53461,53462,53463,53464,53465,53466,53467,53468,53469,53470,53471,53472,53473,53474,53475,53476,53477,53478,53479,53480,53481,53482,53483,53484,53485,53486,53487,53488,53489,53490,53491,53492,53493,53494,53495,53496,53497,53498,53499,53500,53501,53502,53503,53504,53505,53506,53507,53508,53509,53510,53511,53512,53513,53514,53515,53516,53517,53518,53519,53520,53521,53522,53523,53524,53525,53526,53527,53528,53529,53530,53531,53532,53533,53534,53535,53536,53537,53538,53539,53540,53541,53542,53543,53544,53545,53546,53547,53548,53549,53550,53551,53552,53553,53554,53555,53556,53557,53558,53559,53560,53561,53562,53563,53564,53565,53566,53567,53568,53569,53570,53571,53572,53573,53574,53575,53576,53577,53578,53579,53580,53581,53582,53583,53584,53585,53586,53587,53588,53589,53590,53591,53592,53593,53594,53595,53596,53597,53598,53599,53600,53601,53602,53603,53604,53605,53606,53607,53608,53609,53610,53611,53612,53613,53614,53615,53616,53617,53618,53619,53620,53621,53622,53623,53624,53625,53626,53627,53628,53629,53630,53631,53632,53633,53634,53635,53636,53637,53638,53639,53640,53641,53642,53643,53644,53645,53646,53647,53648,53649,53650,53651,53652,53653,53654,53655,53656,53657,53658,53659,53660,53661,53662,53663,53664,53665,53666,53667,53668,53669,53670,53671,53672,53673,53674,53675,53676,53677,53678,53679,53680,53681,53682,53683,53684,53685,53686,53687,53688,53689,53690,53691,53692,53693,53694,53695,53696,53697,53698,53699,53700,53701,53702,53703,53704,53705,53706,53707,53708,53709,53710,53711,53712,53713,53714,53715,53716,53717,53718,53719,53720,53721,53722,53723,53724,53725,53726,53727,53728,53729,53730,53731,53732,53733,53734,53735,53736,53737,53738,53739,53740,53741,53742,53743,53744,53745,53746,53747,53748,53749,53750,53751,53752,53753,53754,53755,53756,53757,53758,53759,53760,53761,53762,53763,53764,53765,53766,53767,53768,53769,53770,53771,53772,53773,53774,53775,53776,53777,53778,53779,53780,53781,53782,53783,53784,53785,53786,53787,53788,53789,53790,53791,53792,53793,53794,53795,53796,53797,53798,53799,53800,53801,53802,53803,53804,53805,53806,53807,53808,53809,53810,53811,53812,53813,53814,53815,53816,53817,53818,53819,53820,53821,53822,53823,53824,53825,53826,53827,53828,53829,53830,53831,53832,53833,53834,53835,53836,53837,53838,53839,53840,53841,53842,53843,53844,53845,53846,53847,53848,53849,53850,53851,53852,53853,53854,53855,53856,53857,53858,53859,53860,53861,53862,53863,53864,53865,53866,53867,53868,53869,53870,53871,53872,53873,53874,53875,53876,53877,53878,53879,53880,53881,53882,53883,53884,53885,53886,53887,53888,53889,53890,53891,53892,53893,53894,53895,53896,53897,53898,53899,53900,53901,53902,53903,53904,53905,53906,53907,53908,53909,53910,53911,53912,53913,53914,53915,53916,53917,53918,53919,53920,53921,53922,53923,53924,53925,53926,53927,53928,53929,53930,53931,53932,53933,53934,53935,53936,53937,53938,53939,53940,53941,53942,53943,53944,53945,53946,53947,53948,53949,53950,53951,53952,53953,53954,53955,53956,53957,53958,53959,53960,53961,53962,53963,53964,53965,53966,53967,53968,53969,53970,53971,53972,53973,53974,53975,53976,53977,53978,53979,53980,53981,53982,53983,53984,53985,53986,53987,53988,53989,53990,53991,53992,53993,53994,53995,53996,53997,53998,53999,54000,54001,54002,54003,54004,54005,54006,54007,54008,54009,54010,54011,54012,54013,54014,54015,54016,54017,54018,54019,54020,54021,54022,54023,54024,54025,54026,54027,54028,54029,54030,54031,54032,54033,54034,54035,54036,54037,54038,54039,54040,54041,54042,54043,54044,54045,54046,54047,54048,54049,54050,54051,54052,54053,54054,54055,54056,54057,54058,54059,54060,54061,54062,54063,54064,54065,54066,54067,54068,54069,54070,54071,54072,54073,54074,54075,54076,54077,54078,54079,54080,54081,54082,54083,54084,54085,54086,54087,54088,54089,54090,54091,54092,54093,54094,54095,54096,54097,54098,54099,54100,54101,54102,54103,54104,54105,54106,54107,54108,54109,54110,54111,54112,54113,54114,54115,54116,54117,54118,54119,54120,54121,54122,54123,54124,54125,54126,54127,54128,54129,54130,54131,54132,54133,54134,54135,54136,54137,54138,54139,54140,54141,54142,54143,54144,54145,54146,54147,54148,54149,54150,54151,54152,54153,54154,54155,54156,54157,54158,54159,54160,54161,54162,54163,54164,54165,54166,54167,54168,54169,54170,54171,54172,54173,54174,54175,54176,54177,54178,54179,54180,54181,54182,54183,54184,54185,54186,54187,54188,54189,54190,54191,54192,54193,54194,54195,54196,54197,54198,54199,54200,54201,54202,54203,54204,54205,54206,54207,54208,54209,54210,54211,54212,54213,54214,54215,54216,54217,54218,54219,54220,54221,54222,54223,54224,54225,54226,54227,54228,54229,54230,54231,54232,54233,54234,54235,54236,54237,54238,54239,54240,54241,54242,54243,54244,54245,54246,54247,54248,54249,54250,54251,54252,54253,54254,54255,54256,54257,54258,54259,54260,54261,54262,54263,54264,54265,54266,54267,54268,54269,54270,54271,54272,54273,54274,54275,54276,54277,54278,54279,54280,54281,54282,54283,54284,54285,54286,54287,54288,54289,54290,54291,54292,54293,54294,54295,54296,54297,54298,54299,54300,54301,54302,54303,54304,54305,54306,54307,54308,54309,54310,54311,54312,54313,54314,54315,54316,54317,54318,54319,54320,54321,54322,54323,54324,54325,54326,54327,54328,54329,54330,54331,54332,54333,54334,54335,54336,54337,54338,54339,54340,54341,54342,54343,54344,54345,54346,54347,54348,54349,54350,54351,54352,54353,54354,54355,54356,54357,54358,54359,54360,54361,54362,54363,54364,54365,54366,54367,54368,54369,54370,54371,54372,54373,54374,54375,54376,54377,54378,54379,54380,54381,54382,54383,54384,54385,54386,54387,54388,54389,54390,54391,54392,54393,54394,54395,54396,54397,54398,54399,54400,54401,54402,54403,54404,54405,54406,54407,54408,54409,54410,54411,54412,54413,54414,54415,54416,54417,54418,54419,54420,54421,54422,54423,54424,54425,54426,54427,54428,54429,54430,54431,54432,54433,54434,54435,54436,54437,54438,54439,54440,54441,54442,54443,54444,54445,54446,54447,54448,54449,54450,54451,54452,54453,54454,54455,54456,54457,54458,54459,54460,54461,54462,54463,54464,54465,54466,54467,54468,54469,54470,54471,54472,54473,54474,54475,54476,54477,54478,54479,54480,54481,54482,54483,54484,54485,54486,54487,54488,54489,54490,54491,54492,54493,54494,54495,54496,54497,54498,54499,54500,54501,54502,54503,54504,54505,54506,54507,54508,54509,54510,54511,54512,54513,54514,54515,54516,54517,54518,54519,54520,54521,54522,54523,54524,54525,54526,54527,54528,54529,54530,54531,54532,54533,54534,54535,54536,54537,54538,54539,54540,54541,54542,54543,54544,54545,54546,54547,54548,54549,54550,54551,54552,54553,54554,54555,54556,54557,54558,54559,54560,54561,54562,54563,54564,54565,54566,54567,54568,54569,54570,54571,54572,54573,54574,54575,54576,54577,54578,54579,54580,54581,54582,54583,54584,54585,54586,54587,54588,54589,54590,54591,54592,54593,54594,54595,54596,54597,54598,54599,54600,54601,54602,54603,54604,54605,54606,54607,54608,54609,54610,54611,54612,54613,54614,54615,54616,54617,54618,54619,54620,54621,54622,54623,54624,54625,54626,54627,54628,54629,54630,54631,54632,54633,54634,54635,54636,54637,54638,54639,54640,54641,54642,54643,54644,54645,54646,54647,54648,54649,54650,54651,54652,54653,54654,54655,54656,54657,54658,54659,54660,54661,54662,54663,54664,54665,54666,54667,54668,54669,54670,54671,54672,54673,54674,54675,54676,54677,54678,54679,54680,54681,54682,54683,54684,54685,54686,54687,54688,54689,54690,54691,54692,54693,54694,54695,54696,54697,54698,54699,54700,54701,54702,54703,54704,54705,54706,54707,54708,54709,54710,54711,54712,54713,54714,54715,54716,54717,54718,54719,54720,54721,54722,54723,54724,54725,54726,54727,54728,54729,54730,54731,54732,54733,54734,54735,54736,54737,54738,54739,54740,54741,54742,54743,54744,54745,54746,54747,54748,54749,54750,54751,54752,54753,54754,54755,54756,54757,54758,54759,54760,54761,54762,54763,54764,54765,54766,54767,54768,54769,54770,54771,54772,54773,54774,54775,54776,54777,54778,54779,54780,54781,54782,54783,54784,54785,54786,54787,54788,54789,54790,54791,54792,54793,54794,54795,54796,54797,54798,54799,54800,54801,54802,54803,54804,54805,54806,54807,54808,54809,54810,54811,54812,54813,54814,54815,54816,54817,54818,54819,54820,54821,54822,54823,54824,54825,54826,54827,54828,54829,54830,54831,54832,54833,54834,54835,54836,54837,54838,54839,54840,54841,54842,54843,54844,54845,54846,54847,54848,54849,54850,54851,54852,54853,54854,54855,54856,54857,54858,54859,54860,54861,54862,54863,54864,54865,54866,54867,54868,54869,54870,54871,54872,54873,54874,54875,54876,54877,54878,54879,54880,54881,54882,54883,54884,54885,54886,54887,54888,54889,54890,54891,54892,54893,54894,54895,54896,54897,54898,54899,54900,54901,54902,54903,54904,54905,54906,54907,54908,54909,54910,54911,54912,54913,54914,54915,54916,54917,54918,54919,54920,54921,54922,54923,54924,54925,54926,54927,54928,54929,54930,54931,54932,54933,54934,54935,54936,54937,54938,54939,54940,54941,54942,54943,54944,54945,54946,54947,54948,54949,54950,54951,54952,54953,54954,54955,54956,54957,54958,54959,54960,54961,54962,54963,54964,54965,54966,54967,54968,54969,54970,54971,54972,54973,54974,54975,54976,54977,54978,54979,54980,54981,54982,54983,54984,54985,54986,54987,54988,54989,54990,54991,54992,54993,54994,54995,54996,54997,54998,54999,55000,55001,55002,55003,55004,55005,55006,55007,55008,55009,55010,55011,55012,55013,55014,55015,55016,55017,55018,55019,55020,55021,55022,55023,55024,55025,55026,55027,55028,55029,55030,55031,55032,55033,55034,55035,55036,55037,55038,55039,55040,55041,55042,55043,55044,55045,55046,55047,55048,55049,55050,55051,55052,55053,55054,55055,55056,55057,55058,55059,55060,55061,55062,55063,55064,55065,55066,55067,55068,55069,55070,55071,55072,55073,55074,55075,55076,55077,55078,55079,55080,55081,55082,55083,55084,55085,55086,55087,55088,55089,55090,55091,55092,55093,55094,55095,55096,55097,55098,55099,55100,55101,55102,55103,55104,55105,55106,55107,55108,55109,55110,55111,55112,55113,55114,55115,55116,55117,55118,55119,55120,55121,55122,55123,55124,55125,55126,55127,55128,55129,55130,55131,55132,55133,55134,55135,55136,55137,55138,55139,55140,55141,55142,55143,55144,55145,55146,55147,55148,55149,55150,55151,55152,55153,55154,55155,55156,55157,55158,55159,55160,55161,55162,55163,55164,55165,55166,55167,55168,55169,55170,55171,55172,55173,55174,55175,55176,55177,55178,55179,55180,55181,55182,55183,55184,55185,55186,55187,55188,55189,55190,55191,55192,55193,55194,55195,55196,55197,55198,55199,55200,55201,55202,55203,55216,55217,55218,55219,55220,55221,55222,55223,55224,55225,55226,55227,55228,55229,55230,55231,55232,55233,55234,55235,55236,55237,55238,55243,55244,55245,55246,55247,55248,55249,55250,55251,55252,55253,55254,55255,55256,55257,55258,55259,55260,55261,55262,55263,55264,55265,55266,55267,55268,55269,55270,55271,55272,55273,55274,55275,55276,55277,55278,55279,55280,55281,55282,55283,55284,55285,55286,55287,55288,55289,55290,55291,63744,63745,63746,63747,63748,63749,63750,63751,63752,63753,63754,63755,63756,63757,63758,63759,63760,63761,63762,63763,63764,63765,63766,63767,63768,63769,63770,63771,63772,63773,63774,63775,63776,63777,63778,63779,63780,63781,63782,63783,63784,63785,63786,63787,63788,63789,63790,63791,63792,63793,63794,63795,63796,63797,63798,63799,63800,63801,63802,63803,63804,63805,63806,63807,63808,63809,63810,63811,63812,63813,63814,63815,63816,63817,63818,63819,63820,63821,63822,63823,63824,63825,63826,63827,63828,63829,63830,63831,63832,63833,63834,63835,63836,63837,63838,63839,63840,63841,63842,63843,63844,63845,63846,63847,63848,63849,63850,63851,63852,63853,63854,63855,63856,63857,63858,63859,63860,63861,63862,63863,63864,63865,63866,63867,63868,63869,63870,63871,63872,63873,63874,63875,63876,63877,63878,63879,63880,63881,63882,63883,63884,63885,63886,63887,63888,63889,63890,63891,63892,63893,63894,63895,63896,63897,63898,63899,63900,63901,63902,63903,63904,63905,63906,63907,63908,63909,63910,63911,63912,63913,63914,63915,63916,63917,63918,63919,63920,63921,63922,63923,63924,63925,63926,63927,63928,63929,63930,63931,63932,63933,63934,63935,63936,63937,63938,63939,63940,63941,63942,63943,63944,63945,63946,63947,63948,63949,63950,63951,63952,63953,63954,63955,63956,63957,63958,63959,63960,63961,63962,63963,63964,63965,63966,63967,63968,63969,63970,63971,63972,63973,63974,63975,63976,63977,63978,63979,63980,63981,63982,63983,63984,63985,63986,63987,63988,63989,63990,63991,63992,63993,63994,63995,63996,63997,63998,63999,64000,64001,64002,64003,64004,64005,64006,64007,64008,64009,64010,64011,64012,64013,64014,64015,64016,64017,64018,64019,64020,64021,64022,64023,64024,64025,64026,64027,64028,64029,64030,64031,64032,64033,64034,64035,64036,64037,64038,64039,64040,64041,64042,64043,64044,64045,64046,64047,64048,64049,64050,64051,64052,64053,64054,64055,64056,64057,64058,64059,64060,64061,64062,64063,64064,64065,64066,64067,64068,64069,64070,64071,64072,64073,64074,64075,64076,64077,64078,64079,64080,64081,64082,64083,64084,64085,64086,64087,64088,64089,64090,64091,64092,64093,64094,64095,64096,64097,64098,64099,64100,64101,64102,64103,64104,64105,64106,64107,64108,64109,64112,64113,64114,64115,64116,64117,64118,64119,64120,64121,64122,64123,64124,64125,64126,64127,64128,64129,64130,64131,64132,64133,64134,64135,64136,64137,64138,64139,64140,64141,64142,64143,64144,64145,64146,64147,64148,64149,64150,64151,64152,64153,64154,64155,64156,64157,64158,64159,64160,64161,64162,64163,64164,64165,64166,64167,64168,64169,64170,64171,64172,64173,64174,64175,64176,64177,64178,64179,64180,64181,64182,64183,64184,64185,64186,64187,64188,64189,64190,64191,64192,64193,64194,64195,64196,64197,64198,64199,64200,64201,64202,64203,64204,64205,64206,64207,64208,64209,64210,64211,64212,64213,64214,64215,64216,64217,64256,64257,64258,64259,64260,64261,64262,64275,64276,64277,64278,64279,64285,64287,64288,64289,64290,64291,64292,64293,64294,64295,64296,64298,64299,64300,64301,64302,64303,64304,64305,64306,64307,64308,64309,64310,64312,64313,64314,64315,64316,64318,64320,64321,64323,64324,64326,64327,64328,64329,64330,64331,64332,64333,64334,64335,64336,64337,64338,64339,64340,64341,64342,64343,64344,64345,64346,64347,64348,64349,64350,64351,64352,64353,64354,64355,64356,64357,64358,64359,64360,64361,64362,64363,64364,64365,64366,64367,64368,64369,64370,64371,64372,64373,64374,64375,64376,64377,64378,64379,64380,64381,64382,64383,64384,64385,64386,64387,64388,64389,64390,64391,64392,64393,64394,64395,64396,64397,64398,64399,64400,64401,64402,64403,64404,64405,64406,64407,64408,64409,64410,64411,64412,64413,64414,64415,64416,64417,64418,64419,64420,64421,64422,64423,64424,64425,64426,64427,64428,64429,64430,64431,64432,64433,64467,64468,64469,64470,64471,64472,64473,64474,64475,64476,64477,64478,64479,64480,64481,64482,64483,64484,64485,64486,64487,64488,64489,64490,64491,64492,64493,64494,64495,64496,64497,64498,64499,64500,64501,64502,64503,64504,64505,64506,64507,64508,64509,64510,64511,64512,64513,64514,64515,64516,64517,64518,64519,64520,64521,64522,64523,64524,64525,64526,64527,64528,64529,64530,64531,64532,64533,64534,64535,64536,64537,64538,64539,64540,64541,64542,64543,64544,64545,64546,64547,64548,64549,64550,64551,64552,64553,64554,64555,64556,64557,64558,64559,64560,64561,64562,64563,64564,64565,64566,64567,64568,64569,64570,64571,64572,64573,64574,64575,64576,64577,64578,64579,64580,64581,64582,64583,64584,64585,64586,64587,64588,64589,64590,64591,64592,64593,64594,64595,64596,64597,64598,64599,64600,64601,64602,64603,64604,64605,64606,64607,64608,64609,64610,64611,64612,64613,64614,64615,64616,64617,64618,64619,64620,64621,64622,64623,64624,64625,64626,64627,64628,64629,64630,64631,64632,64633,64634,64635,64636,64637,64638,64639,64640,64641,64642,64643,64644,64645,64646,64647,64648,64649,64650,64651,64652,64653,64654,64655,64656,64657,64658,64659,64660,64661,64662,64663,64664,64665,64666,64667,64668,64669,64670,64671,64672,64673,64674,64675,64676,64677,64678,64679,64680,64681,64682,64683,64684,64685,64686,64687,64688,64689,64690,64691,64692,64693,64694,64695,64696,64697,64698,64699,64700,64701,64702,64703,64704,64705,64706,64707,64708,64709,64710,64711,64712,64713,64714,64715,64716,64717,64718,64719,64720,64721,64722,64723,64724,64725,64726,64727,64728,64729,64730,64731,64732,64733,64734,64735,64736,64737,64738,64739,64740,64741,64742,64743,64744,64745,64746,64747,64748,64749,64750,64751,64752,64753,64754,64755,64756,64757,64758,64759,64760,64761,64762,64763,64764,64765,64766,64767,64768,64769,64770,64771,64772,64773,64774,64775,64776,64777,64778,64779,64780,64781,64782,64783,64784,64785,64786,64787,64788,64789,64790,64791,64792,64793,64794,64795,64796,64797,64798,64799,64800,64801,64802,64803,64804,64805,64806,64807,64808,64809,64810,64811,64812,64813,64814,64815,64816,64817,64818,64819,64820,64821,64822,64823,64824,64825,64826,64827,64828,64829,64848,64849,64850,64851,64852,64853,64854,64855,64856,64857,64858,64859,64860,64861,64862,64863,64864,64865,64866,64867,64868,64869,64870,64871,64872,64873,64874,64875,64876,64877,64878,64879,64880,64881,64882,64883,64884,64885,64886,64887,64888,64889,64890,64891,64892,64893,64894,64895,64896,64897,64898,64899,64900,64901,64902,64903,64904,64905,64906,64907,64908,64909,64910,64911,64914,64915,64916,64917,64918,64919,64920,64921,64922,64923,64924,64925,64926,64927,64928,64929,64930,64931,64932,64933,64934,64935,64936,64937,64938,64939,64940,64941,64942,64943,64944,64945,64946,64947,64948,64949,64950,64951,64952,64953,64954,64955,64956,64957,64958,64959,64960,64961,64962,64963,64964,64965,64966,64967,65008,65009,65010,65011,65012,65013,65014,65015,65016,65017,65018,65019,65136,65137,65138,65139,65140,65142,65143,65144,65145,65146,65147,65148,65149,65150,65151,65152,65153,65154,65155,65156,65157,65158,65159,65160,65161,65162,65163,65164,65165,65166,65167,65168,65169,65170,65171,65172,65173,65174,65175,65176,65177,65178,65179,65180,65181,65182,65183,65184,65185,65186,65187,65188,65189,65190,65191,65192,65193,65194,65195,65196,65197,65198,65199,65200,65201,65202,65203,65204,65205,65206,65207,65208,65209,65210,65211,65212,65213,65214,65215,65216,65217,65218,65219,65220,65221,65222,65223,65224,65225,65226,65227,65228,65229,65230,65231,65232,65233,65234,65235,65236,65237,65238,65239,65240,65241,65242,65243,65244,65245,65246,65247,65248,65249,65250,65251,65252,65253,65254,65255,65256,65257,65258,65259,65260,65261,65262,65263,65264,65265,65266,65267,65268,65269,65270,65271,65272,65273,65274,65275,65276,65313,65314,65315,65316,65317,65318,65319,65320,65321,65322,65323,65324,65325,65326,65327,65328,65329,65330,65331,65332,65333,65334,65335,65336,65337,65338,65345,65346,65347,65348,65349,65350,65351,65352,65353,65354,65355,65356,65357,65358,65359,65360,65361,65362,65363,65364,65365,65366,65367,65368,65369,65370,65382,65383,65384,65385,65386,65387,65388,65389,65390,65391,65392,65393,65394,65395,65396,65397,65398,65399,65400,65401,65402,65403,65404,65405,65406,65407,65408,65409,65410,65411,65412,65413,65414,65415,65416,65417,65418,65419,65420,65421,65422,65423,65424,65425,65426,65427,65428,65429,65430,65431,65432,65433,65434,65435,65436,65437,65438,65439,65440,65441,65442,65443,65444,65445,65446,65447,65448,65449,65450,65451,65452,65453,65454,65455,65456,65457,65458,65459,65460,65461,65462,65463,65464,65465,65466,65467,65468,65469,65470,65474,65475,65476,65477,65478,65479,65482,65483,65484,65485,65486,65487,65490,65491,65492,65493,65494,65495,65498,65499,65500".split(",").map(function(t){return parseInt(t,10)});e.exports=r},{}],5:[function(t,e,n){var r=t("util/"),i=Array.prototype.slice,o=Object.prototype.hasOwnProperty,a=e.exports=c;function s(t,e){return r.isUndefined(e)?""+e:r.isNumber(e)&&!isFinite(e)?e.toString():r.isFunction(e)||r.isRegExp(e)?e.toString():e}function A(t,e){return r.isString(t)?t.length<e?t:t.slice(0,e):t}function l(t,e,n,r,i){throw new a.AssertionError({message:n,actual:t,expected:e,operator:r,stackStartFunction:i})}function c(t,e){t||l(t,!0,e,"==",a.ok)}function u(t,e){if(t===e)return!0;if(r.isBuffer(t)&&r.isBuffer(e)){if(t.length!=e.length)return!1;for(var n=0;n<t.length;n++)if(t[n]!==e[n])return!1;return!0}return r.isDate(t)&&r.isDate(e)?t.getTime()===e.getTime():r.isRegExp(t)&&r.isRegExp(e)?t.source===e.source&&t.global===e.global&&t.multiline===e.multiline&&t.lastIndex===e.lastIndex&&t.ignoreCase===e.ignoreCase:r.isObject(t)||r.isObject(e)?function(t,e){if(r.isNullOrUndefined(t)||r.isNullOrUndefined(e))return!1;if(t.prototype!==e.prototype)return!1;if(r.isPrimitive(t)||r.isPrimitive(e))return t===e;var n=g(t),o=g(e);if(n&&!o||!n&&o)return!1;if(n)return t=i.call(t),e=i.call(e),u(t,e);var a,s,A=f(t),l=f(e);if(A.length!=l.length)return!1;for(A.sort(),l.sort(),s=A.length-1;s>=0;s--)if(A[s]!=l[s])return!1;for(s=A.length-1;s>=0;s--)if(a=A[s],!u(t[a],e[a]))return!1;return!0}(t,e):t==e}function g(t){return"[object Arguments]"==Object.prototype.toString.call(t)}function h(t,e){return!(!t||!e)&&("[object RegExp]"==Object.prototype.toString.call(e)?e.test(t):t instanceof e||!0===e.call({},t))}function d(t,e,n,i){var o;r.isString(n)&&(i=n,n=null);try{e()}catch(t){o=t}if(i=(n&&n.name?" ("+n.name+").":".")+(i?" "+i:"."),t&&!o&&l(o,n,"Missing expected exception"+i),!t&&h(o,n)&&l(o,n,"Got unwanted exception"+i),t&&o&&n&&!h(o,n)||!t&&o)throw o}a.AssertionError=function(t){this.name="AssertionError",this.actual=t.actual,this.expected=t.expected,this.operator=t.operator,t.message?(this.message=t.message,this.generatedMessage=!1):(this.message=function(t){return A(JSON.stringify(t.actual,s),128)+" "+t.operator+" "+A(JSON.stringify(t.expected,s),128)}(this),this.generatedMessage=!0);var e=t.stackStartFunction||l;if(Error.captureStackTrace)Error.captureStackTrace(this,e);else{var n=new Error;if(n.stack){var r=n.stack,i=e.name,o=r.indexOf("\n"+i);if(o>=0){var a=r.indexOf("\n",o+1);r=r.substring(a+1)}this.stack=r}}},r.inherits(a.AssertionError,Error),a.fail=l,a.ok=c,a.equal=function(t,e,n){t!=e&&l(t,e,n,"==",a.equal)},a.notEqual=function(t,e,n){t==e&&l(t,e,n,"!=",a.notEqual)},a.deepEqual=function(t,e,n){u(t,e)||l(t,e,n,"deepEqual",a.deepEqual)},a.notDeepEqual=function(t,e,n){u(t,e)&&l(t,e,n,"notDeepEqual",a.notDeepEqual)},a.strictEqual=function(t,e,n){t!==e&&l(t,e,n,"===",a.strictEqual)},a.notStrictEqual=function(t,e,n){t===e&&l(t,e,n,"!==",a.notStrictEqual)},a.throws=function(t,e,n){d.apply(this,[!0].concat(i.call(arguments)))},a.doesNotThrow=function(t,e){d.apply(this,[!1].concat(i.call(arguments)))},a.ifError=function(t){if(t)throw t};var f=Object.keys||function(t){var e=[];for(var n in t)o.call(t,n)&&e.push(n);return e}},{"util/":8}],6:[function(t,e,n){"function"==typeof Object.create?e.exports=function(t,e){t.super_=e,t.prototype=Object.create(e.prototype,{constructor:{value:t,enumerable:!1,writable:!0,configurable:!0}})}:e.exports=function(t,e){t.super_=e;var n=function(){};n.prototype=e.prototype,t.prototype=new n,t.prototype.constructor=t}},{}],7:[function(t,e,n){e.exports=function(t){return t&&"object"==typeof t&&"function"==typeof t.copy&&"function"==typeof t.fill&&"function"==typeof t.readUInt8}},{}],8:[function(e,n,r){(function(t,n){var i=/%[sdj%]/g;r.format=function(t){if(!I(t)){for(var e=[],n=0;n<arguments.length;n++)e.push(s(arguments[n]));return e.join(" ")}n=1;for(var r=arguments,o=r.length,a=String(t).replace(i,function(t){if("%%"===t)return"%";if(n>=o)return t;switch(t){case"%s":return String(r[n++]);case"%d":return Number(r[n++]);case"%j":try{return JSON.stringify(r[n++])}catch(t){return"[Circular]"}default:return t}}),A=r[n];n<o;A=r[++n])f(A)||!y(A)?a+=" "+A:a+=" "+s(A);return a},r.deprecate=function(e,i){if(C(n.process))return function(){return r.deprecate(e,i).apply(this,arguments)};if(!0===t.noDeprecation)return e;var o=!1;return function(){if(!o){if(t.throwDeprecation)throw new Error(i);t.traceDeprecation?console.trace(i):console.error(i),o=!0}return e.apply(this,arguments)}};var o,a={};function s(t,e){var n={seen:[],stylize:l};return arguments.length>=3&&(n.depth=arguments[2]),arguments.length>=4&&(n.colors=arguments[3]),d(e)?n.showHidden=e:e&&r._extend(n,e),C(n.showHidden)&&(n.showHidden=!1),C(n.depth)&&(n.depth=2),C(n.colors)&&(n.colors=!1),C(n.customInspect)&&(n.customInspect=!0),n.colors&&(n.stylize=A),c(n,t,n.depth)}function A(t,e){var n=s.styles[e];return n?"["+s.colors[n][0]+"m"+t+"["+s.colors[n][1]+"m":t}function l(t,e){return t}function c(t,e,n){if(t.customInspect&&e&&B(e.inspect)&&e.inspect!==r.inspect&&(!e.constructor||e.constructor.prototype!==e)){var i=e.inspect(n,t);return I(i)||(i=c(t,i,n)),i}var o=function(t,e){if(C(e))return t.stylize("undefined","undefined");if(I(e)){var n="'"+JSON.stringify(e).replace(/^"|"$/g,"").replace(/'/g,"\\'").replace(/\\"/g,'"')+"'";return t.stylize(n,"string")}if(p(e))return t.stylize(""+e,"number");if(d(e))return t.stylize(""+e,"boolean");if(f(e))return t.stylize("null","null")}(t,e);if(o)return o;var a=Object.keys(e),s=function(t){var e={};return t.forEach(function(t,n){e[t]=!0}),e}(a);if(t.showHidden&&(a=Object.getOwnPropertyNames(e)),v(e)&&(a.indexOf("message")>=0||a.indexOf("description")>=0))return u(e);if(0===a.length){if(B(e)){var A=e.name?": "+e.name:"";return t.stylize("[Function"+A+"]","special")}if(m(e))return t.stylize(RegExp.prototype.toString.call(e),"regexp");if(E(e))return t.stylize(Date.prototype.toString.call(e),"date");if(v(e))return u(e)}var l,y="",b=!1,Q=["{","}"];(h(e)&&(b=!0,Q=["[","]"]),B(e))&&(y=" [Function"+(e.name?": "+e.name:"")+"]");return m(e)&&(y=" "+RegExp.prototype.toString.call(e)),E(e)&&(y=" "+Date.prototype.toUTCString.call(e)),v(e)&&(y=" "+u(e)),0!==a.length||b&&0!=e.length?n<0?m(e)?t.stylize(RegExp.prototype.toString.call(e),"regexp"):t.stylize("[Object]","special"):(t.seen.push(e),l=b?function(t,e,n,r,i){for(var o=[],a=0,s=e.length;a<s;++a)_(e,String(a))?o.push(g(t,e,n,r,String(a),!0)):o.push("");return i.forEach(function(i){i.match(/^\d+$/)||o.push(g(t,e,n,r,i,!0))}),o}(t,e,n,s,a):a.map(function(r){return g(t,e,n,s,r,b)}),t.seen.pop(),function(t,e,n){if(t.reduce(function(t,e){return 0,e.indexOf("\n")>=0&&0,t+e.replace(/\u001b\[\d\d?m/g,"").length+1},0)>60)return n[0]+(""===e?"":e+"\n ")+" "+t.join(",\n  ")+" "+n[1];return n[0]+e+" "+t.join(", ")+" "+n[1]}(l,y,Q)):Q[0]+y+Q[1]}function u(t){return"["+Error.prototype.toString.call(t)+"]"}function g(t,e,n,r,i,o){var a,s,A;if((A=Object.getOwnPropertyDescriptor(e,i)||{value:e[i]}).get?s=A.set?t.stylize("[Getter/Setter]","special"):t.stylize("[Getter]","special"):A.set&&(s=t.stylize("[Setter]","special")),_(r,i)||(a="["+i+"]"),s||(t.seen.indexOf(A.value)<0?(s=f(n)?c(t,A.value,null):c(t,A.value,n-1)).indexOf("\n")>-1&&(s=o?s.split("\n").map(function(t){return"  "+t}).join("\n").substr(2):"\n"+s.split("\n").map(function(t){return"   "+t}).join("\n")):s=t.stylize("[Circular]","special")),C(a)){if(o&&i.match(/^\d+$/))return s;(a=JSON.stringify(""+i)).match(/^"([a-zA-Z_][a-zA-Z_0-9]*)"$/)?(a=a.substr(1,a.length-2),a=t.stylize(a,"name")):(a=a.replace(/'/g,"\\'").replace(/\\"/g,'"').replace(/(^"|"$)/g,"'"),a=t.stylize(a,"string"))}return a+": "+s}function h(t){return Array.isArray(t)}function d(t){return"boolean"==typeof t}function f(t){return null===t}function p(t){return"number"==typeof t}function I(t){return"string"==typeof t}function C(t){return void 0===t}function m(t){return y(t)&&"[object RegExp]"===b(t)}function y(t){return"object"==typeof t&&null!==t}function E(t){return y(t)&&"[object Date]"===b(t)}function v(t){return y(t)&&("[object Error]"===b(t)||t instanceof Error)}function B(t){return"function"==typeof t}function b(t){return Object.prototype.toString.call(t)}function Q(t){return t<10?"0"+t.toString(10):t.toString(10)}r.debuglog=function(e){if(C(o)&&(o=t.env.NODE_DEBUG||""),e=e.toUpperCase(),!a[e])if(new RegExp("\\b"+e+"\\b","i").test(o)){var n=t.pid;a[e]=function(){var t=r.format.apply(r,arguments);console.error("%s %d: %s",e,n,t)}}else a[e]=function(){};return a[e]},r.inspect=s,s.colors={bold:[1,22],italic:[3,23],underline:[4,24],inverse:[7,27],white:[37,39],grey:[90,39],black:[30,39],blue:[34,39],cyan:[36,39],green:[32,39],magenta:[35,39],red:[31,39],yellow:[33,39]},s.styles={special:"cyan",number:"yellow",boolean:"yellow",undefined:"grey",null:"bold",string:"green",date:"magenta",regexp:"red"},r.isArray=h,r.isBoolean=d,r.isNull=f,r.isNullOrUndefined=function(t){return null==t},r.isNumber=p,r.isString=I,r.isSymbol=function(t){return"symbol"==typeof t},r.isUndefined=C,r.isRegExp=m,r.isObject=y,r.isDate=E,r.isError=v,r.isFunction=B,r.isPrimitive=function(t){return null===t||"boolean"==typeof t||"number"==typeof t||"string"==typeof t||"symbol"==typeof t||void 0===t},r.isBuffer=e("./support/isBuffer");var w=["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"];function x(){var t=new Date,e=[Q(t.getHours()),Q(t.getMinutes()),Q(t.getSeconds())].join(":");return[t.getDate(),w[t.getMonth()],e].join(" ")}function _(t,e){return Object.prototype.hasOwnProperty.call(t,e)}r.log=function(){console.log("%s - %s",x(),r.format.apply(r,arguments))},r.inherits=e("inherits"),r._extend=function(t,e){if(!e||!y(e))return t;for(var n=Object.keys(e),r=n.length;r--;)t[n[r]]=e[n[r]];return t}}).call(this,e("_process"),void 0!==t?t:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{"./support/isBuffer":7,_process:11,inherits:6}],9:[function(t,e,n){function r(){this._events=this._events||{},this._maxListeners=this._maxListeners||void 0}function i(t){return"function"==typeof t}function o(t){return"object"==typeof t&&null!==t}function a(t){return void 0===t}e.exports=r,r.EventEmitter=r,r.prototype._events=void 0,r.prototype._maxListeners=void 0,r.defaultMaxListeners=10,r.prototype.setMaxListeners=function(t){if("number"!=typeof t||t<0||isNaN(t))throw TypeError("n must be a positive number");return this._maxListeners=t,this},r.prototype.emit=function(t){var e,n,r,s,A,l;if(this._events||(this._events={}),"error"===t&&(!this._events.error||o(this._events.error)&&!this._events.error.length)){if((e=arguments[1])instanceof Error)throw e;throw TypeError('Uncaught, unspecified "error" event.')}if(a(n=this._events[t]))return!1;if(i(n))switch(arguments.length){case 1:n.call(this);break;case 2:n.call(this,arguments[1]);break;case 3:n.call(this,arguments[1],arguments[2]);break;default:for(r=arguments.length,s=new Array(r-1),A=1;A<r;A++)s[A-1]=arguments[A];n.apply(this,s)}else if(o(n)){for(r=arguments.length,s=new Array(r-1),A=1;A<r;A++)s[A-1]=arguments[A];for(r=(l=n.slice()).length,A=0;A<r;A++)l[A].apply(this,s)}return!0},r.prototype.addListener=function(t,e){var n;if(!i(e))throw TypeError("listener must be a function");(this._events||(this._events={}),this._events.newListener&&this.emit("newListener",t,i(e.listener)?e.listener:e),this._events[t]?o(this._events[t])?this._events[t].push(e):this._events[t]=[this._events[t],e]:this._events[t]=e,o(this._events[t])&&!this._events[t].warned)&&((n=a(this._maxListeners)?r.defaultMaxListeners:this._maxListeners)&&n>0&&this._events[t].length>n&&(this._events[t].warned=!0,console.error("(node) warning: possible EventEmitter memory leak detected. %d listeners added. Use emitter.setMaxListeners() to increase limit.",this._events[t].length),"function"==typeof console.trace&&console.trace()));return this},r.prototype.on=r.prototype.addListener,r.prototype.once=function(t,e){if(!i(e))throw TypeError("listener must be a function");var n=!1;function r(){this.removeListener(t,r),n||(n=!0,e.apply(this,arguments))}return r.listener=e,this.on(t,r),this},r.prototype.removeListener=function(t,e){var n,r,a,s;if(!i(e))throw TypeError("listener must be a function");if(!this._events||!this._events[t])return this;if(a=(n=this._events[t]).length,r=-1,n===e||i(n.listener)&&n.listener===e)delete this._events[t],this._events.removeListener&&this.emit("removeListener",t,e);else if(o(n)){for(s=a;s-- >0;)if(n[s]===e||n[s].listener&&n[s].listener===e){r=s;break}if(r<0)return this;1===n.length?(n.length=0,delete this._events[t]):n.splice(r,1),this._events.removeListener&&this.emit("removeListener",t,e)}return this},r.prototype.removeAllListeners=function(t){var e,n;if(!this._events)return this;if(!this._events.removeListener)return 0===arguments.length?this._events={}:this._events[t]&&delete this._events[t],this;if(0===arguments.length){for(e in this._events)"removeListener"!==e&&this.removeAllListeners(e);return this.removeAllListeners("removeListener"),this._events={},this}if(i(n=this._events[t]))this.removeListener(t,n);else for(;n.length;)this.removeListener(t,n[n.length-1]);return delete this._events[t],this},r.prototype.listeners=function(t){return this._events&&this._events[t]?i(this._events[t])?[this._events[t]]:this._events[t].slice():[]},r.listenerCount=function(t,e){return t._events&&t._events[e]?i(t._events[e])?1:t._events[e].length:0}},{}],10:[function(t,e,n){arguments[4][6][0].apply(n,arguments)},{dup:6}],11:[function(t,e,n){var r=e.exports={},i=[],o=!1;function a(){if(!o){var t;o=!0;for(var e=i.length;e;){t=i,i=[];for(var n=-1;++n<e;)t[n]();e=i.length}o=!1}}function s(){}r.nextTick=function(t){i.push(t),o||setTimeout(a,0)},r.title="browser",r.browser=!0,r.env={},r.argv=[],r.version="",r.versions={},r.on=s,r.addListener=s,r.once=s,r.off=s,r.removeListener=s,r.removeAllListeners=s,r.emit=s,r.binding=function(t){throw new Error("process.binding is not supported")},r.cwd=function(){return"/"},r.chdir=function(t){throw new Error("process.chdir is not supported")},r.umask=function(){return 0}},{}],12:[function(t,e,n){arguments[4][7][0].apply(n,arguments)},{dup:7}],13:[function(t,e,n){arguments[4][8][0].apply(n,arguments)},{"./support/isBuffer":12,_process:11,dup:8,inherits:10}],14:[function(e,n,r){(function(t){var r,i=e("util"),o=e("assert"),a=e("date-now"),s=Array.prototype.slice,A={};r=void 0!==t&&t.console?t.console:"undefined"!=typeof window&&window.console?window.console:{};for(var l=[[function(){},"log"],[function(){r.log.apply(r,arguments)},"info"],[function(){r.log.apply(r,arguments)},"warn"],[function(){r.warn.apply(r,arguments)},"error"],[function(t){A[t]=a()},"time"],[function(t){var e=A[t];if(!e)throw new Error("No such label: "+t);var n=a()-e;r.log(t+": "+n+"ms")},"timeEnd"],[function(){var t=new Error;t.name="Trace",t.message=i.format.apply(null,arguments),r.error(t.stack)},"trace"],[function(t){r.log(i.inspect(t)+"\n")},"dir"],[function(t){if(!t){var e=s.call(arguments,1);o.ok(!1,i.format.apply(null,e))}},"assert"]],c=0;c<l.length;c++){var u=l[c],g=u[0],h=u[1];r[h]||(r[h]=g)}n.exports=r}).call(this,void 0!==t?t:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{assert:5,"date-now":15,util:13}],15:[function(t,e,n){e.exports=function(){return(new Date).getTime()}},{}],16:[function(e,i,o){(function(t){(function(){var e,a=200,s="Unsupported core-js use. Try https://npms.io/search?q=ponyfill.",A="Expected a function",l="__lodash_hash_undefined__",c=500,u="__lodash_placeholder__",g=1,h=2,d=4,f=1,p=2,I=1,C=2,m=4,y=8,E=16,v=32,B=64,b=128,Q=256,w=512,x=30,_="...",D=800,M=16,k=1,S=2,L=1/0,R=9007199254740991,N=17976931348623157e292,F=NaN,T=4294967295,U=T-1,O=T>>>1,G=[["ary",b],["bind",I],["bindKey",C],["curry",y],["curryRight",E],["flip",w],["partial",v],["partialRight",B],["rearg",Q]],H="[object Arguments]",j="[object Array]",Y="[object AsyncFunction]",P="[object Boolean]",K="[object Date]",z="[object DOMException]",J="[object Error]",q="[object Function]",V="[object GeneratorFunction]",W="[object Map]",X="[object Number]",Z="[object Null]",$="[object Object]",tt="[object Proxy]",et="[object RegExp]",nt="[object Set]",rt="[object String]",it="[object Symbol]",ot="[object Undefined]",at="[object WeakMap]",st="[object WeakSet]",At="[object ArrayBuffer]",lt="[object DataView]",ct="[object Float32Array]",ut="[object Float64Array]",gt="[object Int8Array]",ht="[object Int16Array]",dt="[object Int32Array]",ft="[object Uint8Array]",pt="[object Uint8ClampedArray]",It="[object Uint16Array]",Ct="[object Uint32Array]",mt=/\b__p \+= '';/g,yt=/\b(__p \+=) '' \+/g,Et=/(__e\(.*?\)|\b__t\)) \+\n'';/g,vt=/&(?:amp|lt|gt|quot|#39);/g,Bt=/[&<>"']/g,bt=RegExp(vt.source),Qt=RegExp(Bt.source),wt=/<%-([\s\S]+?)%>/g,xt=/<%([\s\S]+?)%>/g,_t=/<%=([\s\S]+?)%>/g,Dt=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,Mt=/^\w*$/,kt=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,St=/[\\^$.*+?()[\]{}|]/g,Lt=RegExp(St.source),Rt=/^\s+|\s+$/g,Nt=/^\s+/,Ft=/\s+$/,Tt=/\{(?:\n\/\* \[wrapped with .+\] \*\/)?\n?/,Ut=/\{\n\/\* \[wrapped with (.+)\] \*/,Ot=/,? & /,Gt=/[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g,Ht=/\\(\\)?/g,jt=/\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g,Yt=/\w*$/,Pt=/^[-+]0x[0-9a-f]+$/i,Kt=/^0b[01]+$/i,zt=/^\[object .+?Constructor\]$/,Jt=/^0o[0-7]+$/i,qt=/^(?:0|[1-9]\d*)$/,Vt=/[\xc0-\xd6\xd8-\xf6\xf8-\xff\u0100-\u017f]/g,Wt=/($^)/,Xt=/['\n\r\u2028\u2029\\]/g,Zt="\\u0300-\\u036f\\ufe20-\\ufe2f\\u20d0-\\u20ff",$t="\\xac\\xb1\\xd7\\xf7\\x00-\\x2f\\x3a-\\x40\\x5b-\\x60\\x7b-\\xbf\\u2000-\\u206f \\t\\x0b\\f\\xa0\\ufeff\\n\\r\\u2028\\u2029\\u1680\\u180e\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200a\\u202f\\u205f\\u3000",te="[\\ud800-\\udfff]",ee="["+$t+"]",ne="["+Zt+"]",re="\\d+",ie="[\\u2700-\\u27bf]",oe="[a-z\\xdf-\\xf6\\xf8-\\xff]",ae="[^\\ud800-\\udfff"+$t+re+"\\u2700-\\u27bfa-z\\xdf-\\xf6\\xf8-\\xffA-Z\\xc0-\\xd6\\xd8-\\xde]",se="\\ud83c[\\udffb-\\udfff]",Ae="[^\\ud800-\\udfff]",le="(?:\\ud83c[\\udde6-\\uddff]){2}",ce="[\\ud800-\\udbff][\\udc00-\\udfff]",ue="[A-Z\\xc0-\\xd6\\xd8-\\xde]",ge="(?:"+oe+"|"+ae+")",he="(?:"+ue+"|"+ae+")",de="(?:"+ne+"|"+se+")"+"?",fe="[\\ufe0e\\ufe0f]?"+de+("(?:\\u200d(?:"+[Ae,le,ce].join("|")+")[\\ufe0e\\ufe0f]?"+de+")*"),pe="(?:"+[ie,le,ce].join("|")+")"+fe,Ie="(?:"+[Ae+ne+"?",ne,le,ce,te].join("|")+")",Ce=RegExp("['’]","g"),me=RegExp(ne,"g"),ye=RegExp(se+"(?="+se+")|"+Ie+fe,"g"),Ee=RegExp([ue+"?"+oe+"+(?:['’](?:d|ll|m|re|s|t|ve))?(?="+[ee,ue,"$"].join("|")+")",he+"+(?:['’](?:D|LL|M|RE|S|T|VE))?(?="+[ee,ue+ge,"$"].join("|")+")",ue+"?"+ge+"+(?:['’](?:d|ll|m|re|s|t|ve))?",ue+"+(?:['’](?:D|LL|M|RE|S|T|VE))?","\\d*(?:1ST|2ND|3RD|(?![123])\\dTH)(?=\\b|[a-z_])","\\d*(?:1st|2nd|3rd|(?![123])\\dth)(?=\\b|[A-Z_])",re,pe].join("|"),"g"),ve=RegExp("[\\u200d\\ud800-\\udfff"+Zt+"\\ufe0e\\ufe0f]"),Be=/[a-z][A-Z]|[A-Z]{2}[a-z]|[0-9][a-zA-Z]|[a-zA-Z][0-9]|[^a-zA-Z0-9 ]/,be=["Array","Buffer","DataView","Date","Error","Float32Array","Float64Array","Function","Int8Array","Int16Array","Int32Array","Map","Math","Object","Promise","RegExp","Set","String","Symbol","TypeError","Uint8Array","Uint8ClampedArray","Uint16Array","Uint32Array","WeakMap","_","clearTimeout","isFinite","parseInt","setTimeout"],Qe=-1,we={};we[ct]=we[ut]=we[gt]=we[ht]=we[dt]=we[ft]=we[pt]=we[It]=we[Ct]=!0,we[H]=we[j]=we[At]=we[P]=we[lt]=we[K]=we[J]=we[q]=we[W]=we[X]=we[$]=we[et]=we[nt]=we[rt]=we[at]=!1;var xe={};xe[H]=xe[j]=xe[At]=xe[lt]=xe[P]=xe[K]=xe[ct]=xe[ut]=xe[gt]=xe[ht]=xe[dt]=xe[W]=xe[X]=xe[$]=xe[et]=xe[nt]=xe[rt]=xe[it]=xe[ft]=xe[pt]=xe[It]=xe[Ct]=!0,xe[J]=xe[q]=xe[at]=!1;var _e={"\\":"\\","'":"'","\n":"n","\r":"r","\u2028":"u2028","\u2029":"u2029"},De=parseFloat,Me=parseInt,ke="object"==typeof t&&t&&t.Object===Object&&t,Se="object"==typeof self&&self&&self.Object===Object&&self,Le=ke||Se||Function("return this")(),Re="object"==typeof o&&o&&!o.nodeType&&o,Ne=Re&&"object"==typeof i&&i&&!i.nodeType&&i,Fe=Ne&&Ne.exports===Re,Te=Fe&&ke.process,Ue=function(){try{var t=Ne&&Ne.require&&Ne.require("util").types;return t||Te&&Te.binding&&Te.binding("util")}catch(t){}}(),Oe=Ue&&Ue.isArrayBuffer,Ge=Ue&&Ue.isDate,He=Ue&&Ue.isMap,je=Ue&&Ue.isRegExp,Ye=Ue&&Ue.isSet,Pe=Ue&&Ue.isTypedArray;function Ke(t,e,n){switch(n.length){case 0:return t.call(e);case 1:return t.call(e,n[0]);case 2:return t.call(e,n[0],n[1]);case 3:return t.call(e,n[0],n[1],n[2])}return t.apply(e,n)}function ze(t,e,n,r){for(var i=-1,o=null==t?0:t.length;++i<o;){var a=t[i];e(r,a,n(a),t)}return r}function Je(t,e){for(var n=-1,r=null==t?0:t.length;++n<r&&!1!==e(t[n],n,t););return t}function qe(t,e){for(var n=null==t?0:t.length;n--&&!1!==e(t[n],n,t););return t}function Ve(t,e){for(var n=-1,r=null==t?0:t.length;++n<r;)if(!e(t[n],n,t))return!1;return!0}function We(t,e){for(var n=-1,r=null==t?0:t.length,i=0,o=[];++n<r;){var a=t[n];e(a,n,t)&&(o[i++]=a)}return o}function Xe(t,e){return!!(null==t?0:t.length)&&An(t,e,0)>-1}function Ze(t,e,n){for(var r=-1,i=null==t?0:t.length;++r<i;)if(n(e,t[r]))return!0;return!1}function $e(t,e){for(var n=-1,r=null==t?0:t.length,i=Array(r);++n<r;)i[n]=e(t[n],n,t);return i}function tn(t,e){for(var n=-1,r=e.length,i=t.length;++n<r;)t[i+n]=e[n];return t}function en(t,e,n,r){var i=-1,o=null==t?0:t.length;for(r&&o&&(n=t[++i]);++i<o;)n=e(n,t[i],i,t);return n}function nn(t,e,n,r){var i=null==t?0:t.length;for(r&&i&&(n=t[--i]);i--;)n=e(n,t[i],i,t);return n}function rn(t,e){for(var n=-1,r=null==t?0:t.length;++n<r;)if(e(t[n],n,t))return!0;return!1}var on=gn("length");function an(t,e,n){var r;return n(t,function(t,n,i){if(e(t,n,i))return r=n,!1}),r}function sn(t,e,n,r){for(var i=t.length,o=n+(r?1:-1);r?o--:++o<i;)if(e(t[o],o,t))return o;return-1}function An(t,e,n){return e==e?function(t,e,n){var r=n-1,i=t.length;for(;++r<i;)if(t[r]===e)return r;return-1}(t,e,n):sn(t,cn,n)}function ln(t,e,n,r){for(var i=n-1,o=t.length;++i<o;)if(r(t[i],e))return i;return-1}function cn(t){return t!=t}function un(t,e){var n=null==t?0:t.length;return n?fn(t,e)/n:F}function gn(t){return function(n){return null==n?e:n[t]}}function hn(t){return function(n){return null==t?e:t[n]}}function dn(t,e,n,r,i){return i(t,function(t,i,o){n=r?(r=!1,t):e(n,t,i,o)}),n}function fn(t,n){for(var r,i=-1,o=t.length;++i<o;){var a=n(t[i]);a!==e&&(r=r===e?a:r+a)}return r}function pn(t,e){for(var n=-1,r=Array(t);++n<t;)r[n]=e(n);return r}function In(t){return function(e){return t(e)}}function Cn(t,e){return $e(e,function(e){return t[e]})}function mn(t,e){return t.has(e)}function yn(t,e){for(var n=-1,r=t.length;++n<r&&An(e,t[n],0)>-1;);return n}function En(t,e){for(var n=t.length;n--&&An(e,t[n],0)>-1;);return n}var vn=hn({"À":"A","Á":"A","Â":"A","Ã":"A","Ä":"A","Å":"A","à":"a","á":"a","â":"a","ã":"a","ä":"a","å":"a","Ç":"C","ç":"c","Ð":"D","ð":"d","È":"E","É":"E","Ê":"E","Ë":"E","è":"e","é":"e","ê":"e","ë":"e","Ì":"I","Í":"I","Î":"I","Ï":"I","ì":"i","í":"i","î":"i","ï":"i","Ñ":"N","ñ":"n","Ò":"O","Ó":"O","Ô":"O","Õ":"O","Ö":"O","Ø":"O","ò":"o","ó":"o","ô":"o","õ":"o","ö":"o","ø":"o","Ù":"U","Ú":"U","Û":"U","Ü":"U","ù":"u","ú":"u","û":"u","ü":"u","Ý":"Y","ý":"y","ÿ":"y","Æ":"Ae","æ":"ae","Þ":"Th","þ":"th","ß":"ss","Ā":"A","Ă":"A","Ą":"A","ā":"a","ă":"a","ą":"a","Ć":"C","Ĉ":"C","Ċ":"C","Č":"C","ć":"c","ĉ":"c","ċ":"c","č":"c","Ď":"D","Đ":"D","ď":"d","đ":"d","Ē":"E","Ĕ":"E","Ė":"E","Ę":"E","Ě":"E","ē":"e","ĕ":"e","ė":"e","ę":"e","ě":"e","Ĝ":"G","Ğ":"G","Ġ":"G","Ģ":"G","ĝ":"g","ğ":"g","ġ":"g","ģ":"g","Ĥ":"H","Ħ":"H","ĥ":"h","ħ":"h","Ĩ":"I","Ī":"I","Ĭ":"I","Į":"I","İ":"I","ĩ":"i","ī":"i","ĭ":"i","į":"i","ı":"i","Ĵ":"J","ĵ":"j","Ķ":"K","ķ":"k","ĸ":"k","Ĺ":"L","Ļ":"L","Ľ":"L","Ŀ":"L","Ł":"L","ĺ":"l","ļ":"l","ľ":"l","ŀ":"l","ł":"l","Ń":"N","Ņ":"N","Ň":"N","Ŋ":"N","ń":"n","ņ":"n","ň":"n","ŋ":"n","Ō":"O","Ŏ":"O","Ő":"O","ō":"o","ŏ":"o","ő":"o","Ŕ":"R","Ŗ":"R","Ř":"R","ŕ":"r","ŗ":"r","ř":"r","Ś":"S","Ŝ":"S","Ş":"S","Š":"S","ś":"s","ŝ":"s","ş":"s","š":"s","Ţ":"T","Ť":"T","Ŧ":"T","ţ":"t","ť":"t","ŧ":"t","Ũ":"U","Ū":"U","Ŭ":"U","Ů":"U","Ű":"U","Ų":"U","ũ":"u","ū":"u","ŭ":"u","ů":"u","ű":"u","ų":"u","Ŵ":"W","ŵ":"w","Ŷ":"Y","ŷ":"y","Ÿ":"Y","Ź":"Z","Ż":"Z","Ž":"Z","ź":"z","ż":"z","ž":"z","Ĳ":"IJ","ĳ":"ij","Œ":"Oe","œ":"oe","ŉ":"'n","ſ":"s"}),Bn=hn({"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"});function bn(t){return"\\"+_e[t]}function Qn(t){return ve.test(t)}function wn(t){var e=-1,n=Array(t.size);return t.forEach(function(t,r){n[++e]=[r,t]}),n}function xn(t,e){return function(n){return t(e(n))}}function _n(t,e){for(var n=-1,r=t.length,i=0,o=[];++n<r;){var a=t[n];a!==e&&a!==u||(t[n]=u,o[i++]=n)}return o}function Dn(t){var e=-1,n=Array(t.size);return t.forEach(function(t){n[++e]=t}),n}function Mn(t){var e=-1,n=Array(t.size);return t.forEach(function(t){n[++e]=[t,t]}),n}function kn(t){return Qn(t)?function(t){var e=ye.lastIndex=0;for(;ye.test(t);)++e;return e}(t):on(t)}function Sn(t){return Qn(t)?function(t){return t.match(ye)||[]}(t):function(t){return t.split("")}(t)}var Ln=hn({"&amp;":"&","&lt;":"<","&gt;":">","&quot;":'"',"&#39;":"'"});var Rn=function t(n){var r,i=(n=null==n?Le:Rn.defaults(Le.Object(),n,Rn.pick(Le,be))).Array,o=n.Date,Zt=n.Error,$t=n.Function,te=n.Math,ee=n.Object,ne=n.RegExp,re=n.String,ie=n.TypeError,oe=i.prototype,ae=$t.prototype,se=ee.prototype,Ae=n["__core-js_shared__"],le=ae.toString,ce=se.hasOwnProperty,ue=0,ge=(r=/[^.]+$/.exec(Ae&&Ae.keys&&Ae.keys.IE_PROTO||""))?"Symbol(src)_1."+r:"",he=se.toString,de=le.call(ee),fe=Le._,pe=ne("^"+le.call(ce).replace(St,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),Ie=Fe?n.Buffer:e,ye=n.Symbol,ve=n.Uint8Array,_e=Ie?Ie.allocUnsafe:e,ke=xn(ee.getPrototypeOf,ee),Se=ee.create,Re=se.propertyIsEnumerable,Ne=oe.splice,Te=ye?ye.isConcatSpreadable:e,Ue=ye?ye.iterator:e,on=ye?ye.toStringTag:e,hn=function(){try{var t=Oo(ee,"defineProperty");return t({},"",{}),t}catch(t){}}(),Nn=n.clearTimeout!==Le.clearTimeout&&n.clearTimeout,Fn=o&&o.now!==Le.Date.now&&o.now,Tn=n.setTimeout!==Le.setTimeout&&n.setTimeout,Un=te.ceil,On=te.floor,Gn=ee.getOwnPropertySymbols,Hn=Ie?Ie.isBuffer:e,jn=n.isFinite,Yn=oe.join,Pn=xn(ee.keys,ee),Kn=te.max,zn=te.min,Jn=o.now,qn=n.parseInt,Vn=te.random,Wn=oe.reverse,Xn=Oo(n,"DataView"),Zn=Oo(n,"Map"),$n=Oo(n,"Promise"),tr=Oo(n,"Set"),er=Oo(n,"WeakMap"),nr=Oo(ee,"create"),rr=er&&new er,ir={},or=ua(Xn),ar=ua(Zn),sr=ua($n),Ar=ua(tr),lr=ua(er),cr=ye?ye.prototype:e,ur=cr?cr.valueOf:e,gr=cr?cr.toString:e;function hr(t){if(_s(t)&&!Is(t)&&!(t instanceof Ir)){if(t instanceof pr)return t;if(ce.call(t,"__wrapped__"))return ga(t)}return new pr(t)}var dr=function(){function t(){}return function(n){if(!xs(n))return{};if(Se)return Se(n);t.prototype=n;var r=new t;return t.prototype=e,r}}();function fr(){}function pr(t,n){this.__wrapped__=t,this.__actions__=[],this.__chain__=!!n,this.__index__=0,this.__values__=e}function Ir(t){this.__wrapped__=t,this.__actions__=[],this.__dir__=1,this.__filtered__=!1,this.__iteratees__=[],this.__takeCount__=T,this.__views__=[]}function Cr(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function mr(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function yr(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function Er(t){var e=-1,n=null==t?0:t.length;for(this.__data__=new yr;++e<n;)this.add(t[e])}function vr(t){var e=this.__data__=new mr(t);this.size=e.size}function Br(t,e){var n=Is(t),r=!n&&ps(t),i=!n&&!r&&Es(t),o=!n&&!r&&!i&&Fs(t),a=n||r||i||o,s=a?pn(t.length,re):[],A=s.length;for(var l in t)!e&&!ce.call(t,l)||a&&("length"==l||i&&("offset"==l||"parent"==l)||o&&("buffer"==l||"byteLength"==l||"byteOffset"==l)||zo(l,A))||s.push(l);return s}function br(t){var n=t.length;return n?t[Ei(0,n-1)]:e}function Qr(t,e){return Aa(no(t),Rr(e,0,t.length))}function wr(t){return Aa(no(t))}function xr(t,n,r){(r===e||hs(t[n],r))&&(r!==e||n in t)||Sr(t,n,r)}function _r(t,n,r){var i=t[n];ce.call(t,n)&&hs(i,r)&&(r!==e||n in t)||Sr(t,n,r)}function Dr(t,e){for(var n=t.length;n--;)if(hs(t[n][0],e))return n;return-1}function Mr(t,e,n,r){return Or(t,function(t,i,o){e(r,t,n(t),o)}),r}function kr(t,e){return t&&ro(e,iA(e),t)}function Sr(t,e,n){"__proto__"==e&&hn?hn(t,e,{configurable:!0,enumerable:!0,value:n,writable:!0}):t[e]=n}function Lr(t,n){for(var r=-1,o=n.length,a=i(o),s=null==t;++r<o;)a[r]=s?e:$s(t,n[r]);return a}function Rr(t,n,r){return t==t&&(r!==e&&(t=t<=r?t:r),n!==e&&(t=t>=n?t:n)),t}function Nr(t,n,r,i,o,a){var s,A=n&g,l=n&h,c=n&d;if(r&&(s=o?r(t,i,o,a):r(t)),s!==e)return s;if(!xs(t))return t;var u=Is(t);if(u){if(s=function(t){var e=t.length,n=new t.constructor(e);return e&&"string"==typeof t[0]&&ce.call(t,"index")&&(n.index=t.index,n.input=t.input),n}(t),!A)return no(t,s)}else{var f=jo(t),p=f==q||f==V;if(Es(t))return Wi(t,A);if(f==$||f==H||p&&!o){if(s=l||p?{}:Po(t),!A)return l?function(t,e){return ro(t,Ho(t),e)}(t,function(t,e){return t&&ro(e,oA(e),t)}(s,t)):function(t,e){return ro(t,Go(t),e)}(t,kr(s,t))}else{if(!xe[f])return o?t:{};s=function(t,e,n){var r,i=t.constructor;switch(e){case At:return Xi(t);case P:case K:return new i(+t);case lt:return function(t,e){var n=e?Xi(t.buffer):t.buffer;return new t.constructor(n,t.byteOffset,t.byteLength)}(t,n);case ct:case ut:case gt:case ht:case dt:case ft:case pt:case It:case Ct:return Zi(t,n);case W:return new i;case X:case rt:return new i(t);case et:return function(t){var e=new t.constructor(t.source,Yt.exec(t));return e.lastIndex=t.lastIndex,e}(t);case nt:return new i;case it:return r=t,ur?ee(ur.call(r)):{}}}(t,f,A)}}a||(a=new vr);var I=a.get(t);if(I)return I;if(a.set(t,s),Ls(t))return t.forEach(function(e){s.add(Nr(e,n,r,e,t,a))}),s;if(Ds(t))return t.forEach(function(e,i){s.set(i,Nr(e,n,r,i,t,a))}),s;var C=u?e:(c?l?So:ko:l?oA:iA)(t);return Je(C||t,function(e,i){C&&(e=t[i=e]),_r(s,i,Nr(e,n,r,i,t,a))}),s}function Fr(t,n,r){var i=r.length;if(null==t)return!i;for(t=ee(t);i--;){var o=r[i],a=n[o],s=t[o];if(s===e&&!(o in t)||!a(s))return!1}return!0}function Tr(t,n,r){if("function"!=typeof t)throw new ie(A);return ia(function(){t.apply(e,r)},n)}function Ur(t,e,n,r){var i=-1,o=Xe,s=!0,A=t.length,l=[],c=e.length;if(!A)return l;n&&(e=$e(e,In(n))),r?(o=Ze,s=!1):e.length>=a&&(o=mn,s=!1,e=new Er(e));t:for(;++i<A;){var u=t[i],g=null==n?u:n(u);if(u=r||0!==u?u:0,s&&g==g){for(var h=c;h--;)if(e[h]===g)continue t;l.push(u)}else o(e,g,r)||l.push(u)}return l}hr.templateSettings={escape:wt,evaluate:xt,interpolate:_t,variable:"",imports:{_:hr}},hr.prototype=fr.prototype,hr.prototype.constructor=hr,pr.prototype=dr(fr.prototype),pr.prototype.constructor=pr,Ir.prototype=dr(fr.prototype),Ir.prototype.constructor=Ir,Cr.prototype.clear=function(){this.__data__=nr?nr(null):{},this.size=0},Cr.prototype.delete=function(t){var e=this.has(t)&&delete this.__data__[t];return this.size-=e?1:0,e},Cr.prototype.get=function(t){var n=this.__data__;if(nr){var r=n[t];return r===l?e:r}return ce.call(n,t)?n[t]:e},Cr.prototype.has=function(t){var n=this.__data__;return nr?n[t]!==e:ce.call(n,t)},Cr.prototype.set=function(t,n){var r=this.__data__;return this.size+=this.has(t)?0:1,r[t]=nr&&n===e?l:n,this},mr.prototype.clear=function(){this.__data__=[],this.size=0},mr.prototype.delete=function(t){var e=this.__data__,n=Dr(e,t);return!(n<0||(n==e.length-1?e.pop():Ne.call(e,n,1),--this.size,0))},mr.prototype.get=function(t){var n=this.__data__,r=Dr(n,t);return r<0?e:n[r][1]},mr.prototype.has=function(t){return Dr(this.__data__,t)>-1},mr.prototype.set=function(t,e){var n=this.__data__,r=Dr(n,t);return r<0?(++this.size,n.push([t,e])):n[r][1]=e,this},yr.prototype.clear=function(){this.size=0,this.__data__={hash:new Cr,map:new(Zn||mr),string:new Cr}},yr.prototype.delete=function(t){var e=To(this,t).delete(t);return this.size-=e?1:0,e},yr.prototype.get=function(t){return To(this,t).get(t)},yr.prototype.has=function(t){return To(this,t).has(t)},yr.prototype.set=function(t,e){var n=To(this,t),r=n.size;return n.set(t,e),this.size+=n.size==r?0:1,this},Er.prototype.add=Er.prototype.push=function(t){return this.__data__.set(t,l),this},Er.prototype.has=function(t){return this.__data__.has(t)},vr.prototype.clear=function(){this.__data__=new mr,this.size=0},vr.prototype.delete=function(t){var e=this.__data__,n=e.delete(t);return this.size=e.size,n},vr.prototype.get=function(t){return this.__data__.get(t)},vr.prototype.has=function(t){return this.__data__.has(t)},vr.prototype.set=function(t,e){var n=this.__data__;if(n instanceof mr){var r=n.__data__;if(!Zn||r.length<a-1)return r.push([t,e]),this.size=++n.size,this;n=this.__data__=new yr(r)}return n.set(t,e),this.size=n.size,this};var Or=ao(Jr),Gr=ao(qr,!0);function Hr(t,e){var n=!0;return Or(t,function(t,r,i){return n=!!e(t,r,i)}),n}function jr(t,n,r){for(var i=-1,o=t.length;++i<o;){var a=t[i],s=n(a);if(null!=s&&(A===e?s==s&&!Ns(s):r(s,A)))var A=s,l=a}return l}function Yr(t,e){var n=[];return Or(t,function(t,r,i){e(t,r,i)&&n.push(t)}),n}function Pr(t,e,n,r,i){var o=-1,a=t.length;for(n||(n=Ko),i||(i=[]);++o<a;){var s=t[o];e>0&&n(s)?e>1?Pr(s,e-1,n,r,i):tn(i,s):r||(i[i.length]=s)}return i}var Kr=so(),zr=so(!0);function Jr(t,e){return t&&Kr(t,e,iA)}function qr(t,e){return t&&zr(t,e,iA)}function Vr(t,e){return We(e,function(e){return bs(t[e])})}function Wr(t,n){for(var r=0,i=(n=zi(n,t)).length;null!=t&&r<i;)t=t[ca(n[r++])];return r&&r==i?t:e}function Xr(t,e,n){var r=e(t);return Is(t)?r:tn(r,n(t))}function Zr(t){return null==t?t===e?ot:Z:on&&on in ee(t)?function(t){var n=ce.call(t,on),r=t[on];try{t[on]=e;var i=!0}catch(t){}var o=he.call(t);return i&&(n?t[on]=r:delete t[on]),o}(t):function(t){return he.call(t)}(t)}function $r(t,e){return t>e}function ti(t,e){return null!=t&&ce.call(t,e)}function ei(t,e){return null!=t&&e in ee(t)}function ni(t,n,r){for(var o=r?Ze:Xe,a=t[0].length,s=t.length,A=s,l=i(s),c=1/0,u=[];A--;){var g=t[A];A&&n&&(g=$e(g,In(n))),c=zn(g.length,c),l[A]=!r&&(n||a>=120&&g.length>=120)?new Er(A&&g):e}g=t[0];var h=-1,d=l[0];t:for(;++h<a&&u.length<c;){var f=g[h],p=n?n(f):f;if(f=r||0!==f?f:0,!(d?mn(d,p):o(u,p,r))){for(A=s;--A;){var I=l[A];if(!(I?mn(I,p):o(t[A],p,r)))continue t}d&&d.push(p),u.push(f)}}return u}function ri(t,n,r){var i=null==(t=ea(t,n=zi(n,t)))?t:t[ca(Ba(n))];return null==i?e:Ke(i,t,r)}function ii(t){return _s(t)&&Zr(t)==H}function oi(t,n,r,i,o){return t===n||(null==t||null==n||!_s(t)&&!_s(n)?t!=t&&n!=n:function(t,n,r,i,o,a){var s=Is(t),A=Is(n),l=s?j:jo(t),c=A?j:jo(n),u=(l=l==H?$:l)==$,g=(c=c==H?$:c)==$,h=l==c;if(h&&Es(t)){if(!Es(n))return!1;s=!0,u=!1}if(h&&!u)return a||(a=new vr),s||Fs(t)?Do(t,n,r,i,o,a):function(t,e,n,r,i,o,a){switch(n){case lt:if(t.byteLength!=e.byteLength||t.byteOffset!=e.byteOffset)return!1;t=t.buffer,e=e.buffer;case At:return!(t.byteLength!=e.byteLength||!o(new ve(t),new ve(e)));case P:case K:case X:return hs(+t,+e);case J:return t.name==e.name&&t.message==e.message;case et:case rt:return t==e+"";case W:var s=wn;case nt:var A=r&f;if(s||(s=Dn),t.size!=e.size&&!A)return!1;var l=a.get(t);if(l)return l==e;r|=p,a.set(t,e);var c=Do(s(t),s(e),r,i,o,a);return a.delete(t),c;case it:if(ur)return ur.call(t)==ur.call(e)}return!1}(t,n,l,r,i,o,a);if(!(r&f)){var d=u&&ce.call(t,"__wrapped__"),I=g&&ce.call(n,"__wrapped__");if(d||I){var C=d?t.value():t,m=I?n.value():n;return a||(a=new vr),o(C,m,r,i,a)}}return!!h&&(a||(a=new vr),function(t,n,r,i,o,a){var s=r&f,A=ko(t),l=A.length,c=ko(n).length;if(l!=c&&!s)return!1;for(var u=l;u--;){var g=A[u];if(!(s?g in n:ce.call(n,g)))return!1}var h=a.get(t);if(h&&a.get(n))return h==n;var d=!0;a.set(t,n),a.set(n,t);for(var p=s;++u<l;){g=A[u];var I=t[g],C=n[g];if(i)var m=s?i(C,I,g,n,t,a):i(I,C,g,t,n,a);if(!(m===e?I===C||o(I,C,r,i,a):m)){d=!1;break}p||(p="constructor"==g)}if(d&&!p){var y=t.constructor,E=n.constructor;y!=E&&"constructor"in t&&"constructor"in n&&!("function"==typeof y&&y instanceof y&&"function"==typeof E&&E instanceof E)&&(d=!1)}return a.delete(t),a.delete(n),d}(t,n,r,i,o,a))}(t,n,r,i,oi,o))}function ai(t,n,r,i){var o=r.length,a=o,s=!i;if(null==t)return!a;for(t=ee(t);o--;){var A=r[o];if(s&&A[2]?A[1]!==t[A[0]]:!(A[0]in t))return!1}for(;++o<a;){var l=(A=r[o])[0],c=t[l],u=A[1];if(s&&A[2]){if(c===e&&!(l in t))return!1}else{var g=new vr;if(i)var h=i(c,u,l,t,n,g);if(!(h===e?oi(u,c,f|p,i,g):h))return!1}}return!0}function si(t){return!(!xs(t)||(e=t,ge&&ge in e))&&(bs(t)?pe:zt).test(ua(t));var e}function Ai(t){return"function"==typeof t?t:null==t?MA:"object"==typeof t?Is(t)?di(t[0],t[1]):hi(t):OA(t)}function li(t){if(!Xo(t))return Pn(t);var e=[];for(var n in ee(t))ce.call(t,n)&&"constructor"!=n&&e.push(n);return e}function ci(t){if(!xs(t))return function(t){var e=[];if(null!=t)for(var n in ee(t))e.push(n);return e}(t);var e=Xo(t),n=[];for(var r in t)("constructor"!=r||!e&&ce.call(t,r))&&n.push(r);return n}function ui(t,e){return t<e}function gi(t,e){var n=-1,r=ms(t)?i(t.length):[];return Or(t,function(t,i,o){r[++n]=e(t,i,o)}),r}function hi(t){var e=Uo(t);return 1==e.length&&e[0][2]?$o(e[0][0],e[0][1]):function(n){return n===t||ai(n,t,e)}}function di(t,n){return qo(t)&&Zo(n)?$o(ca(t),n):function(r){var i=$s(r,t);return i===e&&i===n?tA(r,t):oi(n,i,f|p)}}function fi(t,n,r,i,o){t!==n&&Kr(n,function(a,s){if(xs(a))o||(o=new vr),function(t,n,r,i,o,a,s){var A=na(t,r),l=na(n,r),c=s.get(l);if(c)xr(t,r,c);else{var u=a?a(A,l,r+"",t,n,s):e,g=u===e;if(g){var h=Is(l),d=!h&&Es(l),f=!h&&!d&&Fs(l);u=l,h||d||f?Is(A)?u=A:ys(A)?u=no(A):d?(g=!1,u=Wi(l,!0)):f?(g=!1,u=Zi(l,!0)):u=[]:ks(l)||ps(l)?(u=A,ps(A)?u=Ps(A):xs(A)&&!bs(A)||(u=Po(l))):g=!1}g&&(s.set(l,u),o(u,l,i,a,s),s.delete(l)),xr(t,r,u)}}(t,n,s,r,fi,i,o);else{var A=i?i(na(t,s),a,s+"",t,n,o):e;A===e&&(A=a),xr(t,s,A)}},oA)}function pi(t,n){var r=t.length;if(r)return zo(n+=n<0?r:0,r)?t[n]:e}function Ii(t,e,n){var r=-1;return e=$e(e.length?e:[MA],In(Fo())),function(t,e){var n=t.length;for(t.sort(e);n--;)t[n]=t[n].value;return t}(gi(t,function(t,n,i){return{criteria:$e(e,function(e){return e(t)}),index:++r,value:t}}),function(t,e){return function(t,e,n){for(var r=-1,i=t.criteria,o=e.criteria,a=i.length,s=n.length;++r<a;){var A=$i(i[r],o[r]);if(A){if(r>=s)return A;var l=n[r];return A*("desc"==l?-1:1)}}return t.index-e.index}(t,e,n)})}function Ci(t,e,n){for(var r=-1,i=e.length,o={};++r<i;){var a=e[r],s=Wr(t,a);n(s,a)&&wi(o,zi(a,t),s)}return o}function mi(t,e,n,r){var i=r?ln:An,o=-1,a=e.length,s=t;for(t===e&&(e=no(e)),n&&(s=$e(t,In(n)));++o<a;)for(var A=0,l=e[o],c=n?n(l):l;(A=i(s,c,A,r))>-1;)s!==t&&Ne.call(s,A,1),Ne.call(t,A,1);return t}function yi(t,e){for(var n=t?e.length:0,r=n-1;n--;){var i=e[n];if(n==r||i!==o){var o=i;zo(i)?Ne.call(t,i,1):Ui(t,i)}}return t}function Ei(t,e){return t+On(Vn()*(e-t+1))}function vi(t,e){var n="";if(!t||e<1||e>R)return n;do{e%2&&(n+=t),(e=On(e/2))&&(t+=t)}while(e);return n}function Bi(t,e){return oa(ta(t,e,MA),t+"")}function bi(t){return br(hA(t))}function Qi(t,e){var n=hA(t);return Aa(n,Rr(e,0,n.length))}function wi(t,n,r,i){if(!xs(t))return t;for(var o=-1,a=(n=zi(n,t)).length,s=a-1,A=t;null!=A&&++o<a;){var l=ca(n[o]),c=r;if(o!=s){var u=A[l];(c=i?i(u,l,A):e)===e&&(c=xs(u)?u:zo(n[o+1])?[]:{})}_r(A,l,c),A=A[l]}return t}var xi=rr?function(t,e){return rr.set(t,e),t}:MA,_i=hn?function(t,e){return hn(t,"toString",{configurable:!0,enumerable:!1,value:xA(e),writable:!0})}:MA;function Di(t){return Aa(hA(t))}function Mi(t,e,n){var r=-1,o=t.length;e<0&&(e=-e>o?0:o+e),(n=n>o?o:n)<0&&(n+=o),o=e>n?0:n-e>>>0,e>>>=0;for(var a=i(o);++r<o;)a[r]=t[r+e];return a}function ki(t,e){var n;return Or(t,function(t,r,i){return!(n=e(t,r,i))}),!!n}function Si(t,e,n){var r=0,i=null==t?r:t.length;if("number"==typeof e&&e==e&&i<=O){for(;r<i;){var o=r+i>>>1,a=t[o];null!==a&&!Ns(a)&&(n?a<=e:a<e)?r=o+1:i=o}return i}return Li(t,e,MA,n)}function Li(t,n,r,i){n=r(n);for(var o=0,a=null==t?0:t.length,s=n!=n,A=null===n,l=Ns(n),c=n===e;o<a;){var u=On((o+a)/2),g=r(t[u]),h=g!==e,d=null===g,f=g==g,p=Ns(g);if(s)var I=i||f;else I=c?f&&(i||h):A?f&&h&&(i||!d):l?f&&h&&!d&&(i||!p):!d&&!p&&(i?g<=n:g<n);I?o=u+1:a=u}return zn(a,U)}function Ri(t,e){for(var n=-1,r=t.length,i=0,o=[];++n<r;){var a=t[n],s=e?e(a):a;if(!n||!hs(s,A)){var A=s;o[i++]=0===a?0:a}}return o}function Ni(t){return"number"==typeof t?t:Ns(t)?F:+t}function Fi(t){if("string"==typeof t)return t;if(Is(t))return $e(t,Fi)+"";if(Ns(t))return gr?gr.call(t):"";var e=t+"";return"0"==e&&1/t==-L?"-0":e}function Ti(t,e,n){var r=-1,i=Xe,o=t.length,s=!0,A=[],l=A;if(n)s=!1,i=Ze;else if(o>=a){var c=e?null:Bo(t);if(c)return Dn(c);s=!1,i=mn,l=new Er}else l=e?[]:A;t:for(;++r<o;){var u=t[r],g=e?e(u):u;if(u=n||0!==u?u:0,s&&g==g){for(var h=l.length;h--;)if(l[h]===g)continue t;e&&l.push(g),A.push(u)}else i(l,g,n)||(l!==A&&l.push(g),A.push(u))}return A}function Ui(t,e){return null==(t=ea(t,e=zi(e,t)))||delete t[ca(Ba(e))]}function Oi(t,e,n,r){return wi(t,e,n(Wr(t,e)),r)}function Gi(t,e,n,r){for(var i=t.length,o=r?i:-1;(r?o--:++o<i)&&e(t[o],o,t););return n?Mi(t,r?0:o,r?o+1:i):Mi(t,r?o+1:0,r?i:o)}function Hi(t,e){var n=t;return n instanceof Ir&&(n=n.value()),en(e,function(t,e){return e.func.apply(e.thisArg,tn([t],e.args))},n)}function ji(t,e,n){var r=t.length;if(r<2)return r?Ti(t[0]):[];for(var o=-1,a=i(r);++o<r;)for(var s=t[o],A=-1;++A<r;)A!=o&&(a[o]=Ur(a[o]||s,t[A],e,n));return Ti(Pr(a,1),e,n)}function Yi(t,n,r){for(var i=-1,o=t.length,a=n.length,s={};++i<o;){var A=i<a?n[i]:e;r(s,t[i],A)}return s}function Pi(t){return ys(t)?t:[]}function Ki(t){return"function"==typeof t?t:MA}function zi(t,e){return Is(t)?t:qo(t,e)?[t]:la(Ks(t))}var Ji=Bi;function qi(t,n,r){var i=t.length;return r=r===e?i:r,!n&&r>=i?t:Mi(t,n,r)}var Vi=Nn||function(t){return Le.clearTimeout(t)};function Wi(t,e){if(e)return t.slice();var n=t.length,r=_e?_e(n):new t.constructor(n);return t.copy(r),r}function Xi(t){var e=new t.constructor(t.byteLength);return new ve(e).set(new ve(t)),e}function Zi(t,e){var n=e?Xi(t.buffer):t.buffer;return new t.constructor(n,t.byteOffset,t.length)}function $i(t,n){if(t!==n){var r=t!==e,i=null===t,o=t==t,a=Ns(t),s=n!==e,A=null===n,l=n==n,c=Ns(n);if(!A&&!c&&!a&&t>n||a&&s&&l&&!A&&!c||i&&s&&l||!r&&l||!o)return 1;if(!i&&!a&&!c&&t<n||c&&r&&o&&!i&&!a||A&&r&&o||!s&&o||!l)return-1}return 0}function to(t,e,n,r){for(var o=-1,a=t.length,s=n.length,A=-1,l=e.length,c=Kn(a-s,0),u=i(l+c),g=!r;++A<l;)u[A]=e[A];for(;++o<s;)(g||o<a)&&(u[n[o]]=t[o]);for(;c--;)u[A++]=t[o++];return u}function eo(t,e,n,r){for(var o=-1,a=t.length,s=-1,A=n.length,l=-1,c=e.length,u=Kn(a-A,0),g=i(u+c),h=!r;++o<u;)g[o]=t[o];for(var d=o;++l<c;)g[d+l]=e[l];for(;++s<A;)(h||o<a)&&(g[d+n[s]]=t[o++]);return g}function no(t,e){var n=-1,r=t.length;for(e||(e=i(r));++n<r;)e[n]=t[n];return e}function ro(t,n,r,i){var o=!r;r||(r={});for(var a=-1,s=n.length;++a<s;){var A=n[a],l=i?i(r[A],t[A],A,r,t):e;l===e&&(l=t[A]),o?Sr(r,A,l):_r(r,A,l)}return r}function io(t,e){return function(n,r){var i=Is(n)?ze:Mr,o=e?e():{};return i(n,t,Fo(r,2),o)}}function oo(t){return Bi(function(n,r){var i=-1,o=r.length,a=o>1?r[o-1]:e,s=o>2?r[2]:e;for(a=t.length>3&&"function"==typeof a?(o--,a):e,s&&Jo(r[0],r[1],s)&&(a=o<3?e:a,o=1),n=ee(n);++i<o;){var A=r[i];A&&t(n,A,i,a)}return n})}function ao(t,e){return function(n,r){if(null==n)return n;if(!ms(n))return t(n,r);for(var i=n.length,o=e?i:-1,a=ee(n);(e?o--:++o<i)&&!1!==r(a[o],o,a););return n}}function so(t){return function(e,n,r){for(var i=-1,o=ee(e),a=r(e),s=a.length;s--;){var A=a[t?s:++i];if(!1===n(o[A],A,o))break}return e}}function Ao(t){return function(n){var r=Qn(n=Ks(n))?Sn(n):e,i=r?r[0]:n.charAt(0),o=r?qi(r,1).join(""):n.slice(1);return i[t]()+o}}function lo(t){return function(e){return en(bA(pA(e).replace(Ce,"")),t,"")}}function co(t){return function(){var e=arguments;switch(e.length){case 0:return new t;case 1:return new t(e[0]);case 2:return new t(e[0],e[1]);case 3:return new t(e[0],e[1],e[2]);case 4:return new t(e[0],e[1],e[2],e[3]);case 5:return new t(e[0],e[1],e[2],e[3],e[4]);case 6:return new t(e[0],e[1],e[2],e[3],e[4],e[5]);case 7:return new t(e[0],e[1],e[2],e[3],e[4],e[5],e[6])}var n=dr(t.prototype),r=t.apply(n,e);return xs(r)?r:n}}function uo(t){return function(n,r,i){var o=ee(n);if(!ms(n)){var a=Fo(r,3);n=iA(n),r=function(t){return a(o[t],t,o)}}var s=t(n,r,i);return s>-1?o[a?n[s]:s]:e}}function go(t){return Mo(function(n){var r=n.length,i=r,o=pr.prototype.thru;for(t&&n.reverse();i--;){var a=n[i];if("function"!=typeof a)throw new ie(A);if(o&&!s&&"wrapper"==Ro(a))var s=new pr([],!0)}for(i=s?i:r;++i<r;){var l=Ro(a=n[i]),c="wrapper"==l?Lo(a):e;s=c&&Vo(c[0])&&c[1]==(b|y|v|Q)&&!c[4].length&&1==c[9]?s[Ro(c[0])].apply(s,c[3]):1==a.length&&Vo(a)?s[l]():s.thru(a)}return function(){var t=arguments,e=t[0];if(s&&1==t.length&&Is(e))return s.plant(e).value();for(var i=0,o=r?n[i].apply(this,t):e;++i<r;)o=n[i].call(this,o);return o}})}function ho(t,n,r,o,a,s,A,l,c,u){var g=n&b,h=n&I,d=n&C,f=n&(y|E),p=n&w,m=d?e:co(t);return function I(){for(var C=arguments.length,y=i(C),E=C;E--;)y[E]=arguments[E];if(f)var v=No(I),B=function(t,e){for(var n=t.length,r=0;n--;)t[n]===e&&++r;return r}(y,v);if(o&&(y=to(y,o,a,f)),s&&(y=eo(y,s,A,f)),C-=B,f&&C<u){var b=_n(y,v);return Eo(t,n,ho,I.placeholder,r,y,b,l,c,u-C)}var Q=h?r:this,w=d?Q[t]:t;return C=y.length,l?y=function(t,n){for(var r=t.length,i=zn(n.length,r),o=no(t);i--;){var a=n[i];t[i]=zo(a,r)?o[a]:e}return t}(y,l):p&&C>1&&y.reverse(),g&&c<C&&(y.length=c),this&&this!==Le&&this instanceof I&&(w=m||co(w)),w.apply(Q,y)}}function fo(t,e){return function(n,r){return function(t,e,n,r){return Jr(t,function(t,i,o){e(r,n(t),i,o)}),r}(n,t,e(r),{})}}function po(t,n){return function(r,i){var o;if(r===e&&i===e)return n;if(r!==e&&(o=r),i!==e){if(o===e)return i;"string"==typeof r||"string"==typeof i?(r=Fi(r),i=Fi(i)):(r=Ni(r),i=Ni(i)),o=t(r,i)}return o}}function Io(t){return Mo(function(e){return e=$e(e,In(Fo())),Bi(function(n){var r=this;return t(e,function(t){return Ke(t,r,n)})})})}function Co(t,n){var r=(n=n===e?" ":Fi(n)).length;if(r<2)return r?vi(n,t):n;var i=vi(n,Un(t/kn(n)));return Qn(n)?qi(Sn(i),0,t).join(""):i.slice(0,t)}function mo(t){return function(n,r,o){return o&&"number"!=typeof o&&Jo(n,r,o)&&(r=o=e),n=Gs(n),r===e?(r=n,n=0):r=Gs(r),function(t,e,n,r){for(var o=-1,a=Kn(Un((e-t)/(n||1)),0),s=i(a);a--;)s[r?a:++o]=t,t+=n;return s}(n,r,o=o===e?n<r?1:-1:Gs(o),t)}}function yo(t){return function(e,n){return"string"==typeof e&&"string"==typeof n||(e=Ys(e),n=Ys(n)),t(e,n)}}function Eo(t,n,r,i,o,a,s,A,l,c){var u=n&y;n|=u?v:B,(n&=~(u?B:v))&m||(n&=~(I|C));var g=[t,n,o,u?a:e,u?s:e,u?e:a,u?e:s,A,l,c],h=r.apply(e,g);return Vo(t)&&ra(h,g),h.placeholder=i,aa(h,t,n)}function vo(t){var e=te[t];return function(t,n){if(t=Ys(t),n=null==n?0:zn(Hs(n),292)){var r=(Ks(t)+"e").split("e");return+((r=(Ks(e(r[0]+"e"+(+r[1]+n)))+"e").split("e"))[0]+"e"+(+r[1]-n))}return e(t)}}var Bo=tr&&1/Dn(new tr([,-0]))[1]==L?function(t){return new tr(t)}:NA;function bo(t){return function(e){var n=jo(e);return n==W?wn(e):n==nt?Mn(e):function(t,e){return $e(e,function(e){return[e,t[e]]})}(e,t(e))}}function Qo(t,n,r,o,a,s,l,c){var g=n&C;if(!g&&"function"!=typeof t)throw new ie(A);var h=o?o.length:0;if(h||(n&=~(v|B),o=a=e),l=l===e?l:Kn(Hs(l),0),c=c===e?c:Hs(c),h-=a?a.length:0,n&B){var d=o,f=a;o=a=e}var p=g?e:Lo(t),w=[t,n,r,o,a,d,f,s,l,c];if(p&&function(t,e){var n=t[1],r=e[1],i=n|r,o=i<(I|C|b),a=r==b&&n==y||r==b&&n==Q&&t[7].length<=e[8]||r==(b|Q)&&e[7].length<=e[8]&&n==y;if(!o&&!a)return t;r&I&&(t[2]=e[2],i|=n&I?0:m);var s=e[3];if(s){var A=t[3];t[3]=A?to(A,s,e[4]):s,t[4]=A?_n(t[3],u):e[4]}(s=e[5])&&(A=t[5],t[5]=A?eo(A,s,e[6]):s,t[6]=A?_n(t[5],u):e[6]),(s=e[7])&&(t[7]=s),r&b&&(t[8]=null==t[8]?e[8]:zn(t[8],e[8])),null==t[9]&&(t[9]=e[9]),t[0]=e[0],t[1]=i}(w,p),t=w[0],n=w[1],r=w[2],o=w[3],a=w[4],!(c=w[9]=w[9]===e?g?0:t.length:Kn(w[9]-h,0))&&n&(y|E)&&(n&=~(y|E)),n&&n!=I)x=n==y||n==E?function(t,n,r){var o=co(t);return function a(){for(var s=arguments.length,A=i(s),l=s,c=No(a);l--;)A[l]=arguments[l];var u=s<3&&A[0]!==c&&A[s-1]!==c?[]:_n(A,c);return(s-=u.length)<r?Eo(t,n,ho,a.placeholder,e,A,u,e,e,r-s):Ke(this&&this!==Le&&this instanceof a?o:t,this,A)}}(t,n,c):n!=v&&n!=(I|v)||a.length?ho.apply(e,w):function(t,e,n,r){var o=e&I,a=co(t);return function e(){for(var s=-1,A=arguments.length,l=-1,c=r.length,u=i(c+A),g=this&&this!==Le&&this instanceof e?a:t;++l<c;)u[l]=r[l];for(;A--;)u[l++]=arguments[++s];return Ke(g,o?n:this,u)}}(t,n,r,o);else var x=function(t,e,n){var r=e&I,i=co(t);return function e(){return(this&&this!==Le&&this instanceof e?i:t).apply(r?n:this,arguments)}}(t,n,r);return aa((p?xi:ra)(x,w),t,n)}function wo(t,n,r,i){return t===e||hs(t,se[r])&&!ce.call(i,r)?n:t}function xo(t,n,r,i,o,a){return xs(t)&&xs(n)&&(a.set(n,t),fi(t,n,e,xo,a),a.delete(n)),t}function _o(t){return ks(t)?e:t}function Do(t,n,r,i,o,a){var s=r&f,A=t.length,l=n.length;if(A!=l&&!(s&&l>A))return!1;var c=a.get(t);if(c&&a.get(n))return c==n;var u=-1,g=!0,h=r&p?new Er:e;for(a.set(t,n),a.set(n,t);++u<A;){var d=t[u],I=n[u];if(i)var C=s?i(I,d,u,n,t,a):i(d,I,u,t,n,a);if(C!==e){if(C)continue;g=!1;break}if(h){if(!rn(n,function(t,e){if(!mn(h,e)&&(d===t||o(d,t,r,i,a)))return h.push(e)})){g=!1;break}}else if(d!==I&&!o(d,I,r,i,a)){g=!1;break}}return a.delete(t),a.delete(n),g}function Mo(t){return oa(ta(t,e,Ca),t+"")}function ko(t){return Xr(t,iA,Go)}function So(t){return Xr(t,oA,Ho)}var Lo=rr?function(t){return rr.get(t)}:NA;function Ro(t){for(var e=t.name+"",n=ir[e],r=ce.call(ir,e)?n.length:0;r--;){var i=n[r],o=i.func;if(null==o||o==t)return i.name}return e}function No(t){return(ce.call(hr,"placeholder")?hr:t).placeholder}function Fo(){var t=hr.iteratee||kA;return t=t===kA?Ai:t,arguments.length?t(arguments[0],arguments[1]):t}function To(t,e){var n,r,i=t.__data__;return("string"==(r=typeof(n=e))||"number"==r||"symbol"==r||"boolean"==r?"__proto__"!==n:null===n)?i["string"==typeof e?"string":"hash"]:i.map}function Uo(t){for(var e=iA(t),n=e.length;n--;){var r=e[n],i=t[r];e[n]=[r,i,Zo(i)]}return e}function Oo(t,n){var r=function(t,n){return null==t?e:t[n]}(t,n);return si(r)?r:e}var Go=Gn?function(t){return null==t?[]:(t=ee(t),We(Gn(t),function(e){return Re.call(t,e)}))}:jA,Ho=Gn?function(t){for(var e=[];t;)tn(e,Go(t)),t=ke(t);return e}:jA,jo=Zr;function Yo(t,e,n){for(var r=-1,i=(e=zi(e,t)).length,o=!1;++r<i;){var a=ca(e[r]);if(!(o=null!=t&&n(t,a)))break;t=t[a]}return o||++r!=i?o:!!(i=null==t?0:t.length)&&ws(i)&&zo(a,i)&&(Is(t)||ps(t))}function Po(t){return"function"!=typeof t.constructor||Xo(t)?{}:dr(ke(t))}function Ko(t){return Is(t)||ps(t)||!!(Te&&t&&t[Te])}function zo(t,e){var n=typeof t;return!!(e=null==e?R:e)&&("number"==n||"symbol"!=n&&qt.test(t))&&t>-1&&t%1==0&&t<e}function Jo(t,e,n){if(!xs(n))return!1;var r=typeof e;return!!("number"==r?ms(n)&&zo(e,n.length):"string"==r&&e in n)&&hs(n[e],t)}function qo(t,e){if(Is(t))return!1;var n=typeof t;return!("number"!=n&&"symbol"!=n&&"boolean"!=n&&null!=t&&!Ns(t))||Mt.test(t)||!Dt.test(t)||null!=e&&t in ee(e)}function Vo(t){var e=Ro(t),n=hr[e];if("function"!=typeof n||!(e in Ir.prototype))return!1;if(t===n)return!0;var r=Lo(n);return!!r&&t===r[0]}(Xn&&jo(new Xn(new ArrayBuffer(1)))!=lt||Zn&&jo(new Zn)!=W||$n&&"[object Promise]"!=jo($n.resolve())||tr&&jo(new tr)!=nt||er&&jo(new er)!=at)&&(jo=function(t){var n=Zr(t),r=n==$?t.constructor:e,i=r?ua(r):"";if(i)switch(i){case or:return lt;case ar:return W;case sr:return"[object Promise]";case Ar:return nt;case lr:return at}return n});var Wo=Ae?bs:YA;function Xo(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||se)}function Zo(t){return t==t&&!xs(t)}function $o(t,n){return function(r){return null!=r&&r[t]===n&&(n!==e||t in ee(r))}}function ta(t,n,r){return n=Kn(n===e?t.length-1:n,0),function(){for(var e=arguments,o=-1,a=Kn(e.length-n,0),s=i(a);++o<a;)s[o]=e[n+o];o=-1;for(var A=i(n+1);++o<n;)A[o]=e[o];return A[n]=r(s),Ke(t,this,A)}}function ea(t,e){return e.length<2?t:Wr(t,Mi(e,0,-1))}function na(t,e){if("__proto__"!=e)return t[e]}var ra=sa(xi),ia=Tn||function(t,e){return Le.setTimeout(t,e)},oa=sa(_i);function aa(t,e,n){var r=e+"";return oa(t,function(t,e){var n=e.length;if(!n)return t;var r=n-1;return e[r]=(n>1?"& ":"")+e[r],e=e.join(n>2?", ":" "),t.replace(Tt,"{\n/* [wrapped with "+e+"] */\n")}(r,function(t,e){return Je(G,function(n){var r="_."+n[0];e&n[1]&&!Xe(t,r)&&t.push(r)}),t.sort()}(function(t){var e=t.match(Ut);return e?e[1].split(Ot):[]}(r),n)))}function sa(t){var n=0,r=0;return function(){var i=Jn(),o=M-(i-r);if(r=i,o>0){if(++n>=D)return arguments[0]}else n=0;return t.apply(e,arguments)}}function Aa(t,n){var r=-1,i=t.length,o=i-1;for(n=n===e?i:n;++r<n;){var a=Ei(r,o),s=t[a];t[a]=t[r],t[r]=s}return t.length=n,t}var la=function(t){var e=ss(t,function(t){return n.size===c&&n.clear(),t}),n=e.cache;return e}(function(t){var e=[];return 46===t.charCodeAt(0)&&e.push(""),t.replace(kt,function(t,n,r,i){e.push(r?i.replace(Ht,"$1"):n||t)}),e});function ca(t){if("string"==typeof t||Ns(t))return t;var e=t+"";return"0"==e&&1/t==-L?"-0":e}function ua(t){if(null!=t){try{return le.call(t)}catch(t){}try{return t+""}catch(t){}}return""}function ga(t){if(t instanceof Ir)return t.clone();var e=new pr(t.__wrapped__,t.__chain__);return e.__actions__=no(t.__actions__),e.__index__=t.__index__,e.__values__=t.__values__,e}var ha=Bi(function(t,e){return ys(t)?Ur(t,Pr(e,1,ys,!0)):[]}),da=Bi(function(t,n){var r=Ba(n);return ys(r)&&(r=e),ys(t)?Ur(t,Pr(n,1,ys,!0),Fo(r,2)):[]}),fa=Bi(function(t,n){var r=Ba(n);return ys(r)&&(r=e),ys(t)?Ur(t,Pr(n,1,ys,!0),e,r):[]});function pa(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=null==n?0:Hs(n);return i<0&&(i=Kn(r+i,0)),sn(t,Fo(e,3),i)}function Ia(t,n,r){var i=null==t?0:t.length;if(!i)return-1;var o=i-1;return r!==e&&(o=Hs(r),o=r<0?Kn(i+o,0):zn(o,i-1)),sn(t,Fo(n,3),o,!0)}function Ca(t){return null!=t&&t.length?Pr(t,1):[]}function ma(t){return t&&t.length?t[0]:e}var ya=Bi(function(t){var e=$e(t,Pi);return e.length&&e[0]===t[0]?ni(e):[]}),Ea=Bi(function(t){var n=Ba(t),r=$e(t,Pi);return n===Ba(r)?n=e:r.pop(),r.length&&r[0]===t[0]?ni(r,Fo(n,2)):[]}),va=Bi(function(t){var n=Ba(t),r=$e(t,Pi);return(n="function"==typeof n?n:e)&&r.pop(),r.length&&r[0]===t[0]?ni(r,e,n):[]});function Ba(t){var n=null==t?0:t.length;return n?t[n-1]:e}var ba=Bi(Qa);function Qa(t,e){return t&&t.length&&e&&e.length?mi(t,e):t}var wa=Mo(function(t,e){var n=null==t?0:t.length,r=Lr(t,e);return yi(t,$e(e,function(t){return zo(t,n)?+t:t}).sort($i)),r});function xa(t){return null==t?t:Wn.call(t)}var _a=Bi(function(t){return Ti(Pr(t,1,ys,!0))}),Da=Bi(function(t){var n=Ba(t);return ys(n)&&(n=e),Ti(Pr(t,1,ys,!0),Fo(n,2))}),Ma=Bi(function(t){var n=Ba(t);return n="function"==typeof n?n:e,Ti(Pr(t,1,ys,!0),e,n)});function ka(t){if(!t||!t.length)return[];var e=0;return t=We(t,function(t){if(ys(t))return e=Kn(t.length,e),!0}),pn(e,function(e){return $e(t,gn(e))})}function Sa(t,n){if(!t||!t.length)return[];var r=ka(t);return null==n?r:$e(r,function(t){return Ke(n,e,t)})}var La=Bi(function(t,e){return ys(t)?Ur(t,e):[]}),Ra=Bi(function(t){return ji(We(t,ys))}),Na=Bi(function(t){var n=Ba(t);return ys(n)&&(n=e),ji(We(t,ys),Fo(n,2))}),Fa=Bi(function(t){var n=Ba(t);return n="function"==typeof n?n:e,ji(We(t,ys),e,n)}),Ta=Bi(ka);var Ua=Bi(function(t){var n=t.length,r=n>1?t[n-1]:e;return r="function"==typeof r?(t.pop(),r):e,Sa(t,r)});function Oa(t){var e=hr(t);return e.__chain__=!0,e}function Ga(t,e){return e(t)}var Ha=Mo(function(t){var n=t.length,r=n?t[0]:0,i=this.__wrapped__,o=function(e){return Lr(e,t)};return!(n>1||this.__actions__.length)&&i instanceof Ir&&zo(r)?((i=i.slice(r,+r+(n?1:0))).__actions__.push({func:Ga,args:[o],thisArg:e}),new pr(i,this.__chain__).thru(function(t){return n&&!t.length&&t.push(e),t})):this.thru(o)});var ja=io(function(t,e,n){ce.call(t,n)?++t[n]:Sr(t,n,1)});var Ya=uo(pa),Pa=uo(Ia);function Ka(t,e){return(Is(t)?Je:Or)(t,Fo(e,3))}function za(t,e){return(Is(t)?qe:Gr)(t,Fo(e,3))}var Ja=io(function(t,e,n){ce.call(t,n)?t[n].push(e):Sr(t,n,[e])});var qa=Bi(function(t,e,n){var r=-1,o="function"==typeof e,a=ms(t)?i(t.length):[];return Or(t,function(t){a[++r]=o?Ke(e,t,n):ri(t,e,n)}),a}),Va=io(function(t,e,n){Sr(t,n,e)});function Wa(t,e){return(Is(t)?$e:gi)(t,Fo(e,3))}var Xa=io(function(t,e,n){t[n?0:1].push(e)},function(){return[[],[]]});var Za=Bi(function(t,e){if(null==t)return[];var n=e.length;return n>1&&Jo(t,e[0],e[1])?e=[]:n>2&&Jo(e[0],e[1],e[2])&&(e=[e[0]]),Ii(t,Pr(e,1),[])}),$a=Fn||function(){return Le.Date.now()};function ts(t,n,r){return n=r?e:n,n=t&&null==n?t.length:n,Qo(t,b,e,e,e,e,n)}function es(t,n){var r;if("function"!=typeof n)throw new ie(A);return t=Hs(t),function(){return--t>0&&(r=n.apply(this,arguments)),t<=1&&(n=e),r}}var ns=Bi(function(t,e,n){var r=I;if(n.length){var i=_n(n,No(ns));r|=v}return Qo(t,r,e,n,i)}),rs=Bi(function(t,e,n){var r=I|C;if(n.length){var i=_n(n,No(rs));r|=v}return Qo(e,r,t,n,i)});function is(t,n,r){var i,o,a,s,l,c,u=0,g=!1,h=!1,d=!0;if("function"!=typeof t)throw new ie(A);function f(n){var r=i,a=o;return i=o=e,u=n,s=t.apply(a,r)}function p(t){var r=t-c;return c===e||r>=n||r<0||h&&t-u>=a}function I(){var t=$a();if(p(t))return C(t);l=ia(I,function(t){var e=n-(t-c);return h?zn(e,a-(t-u)):e}(t))}function C(t){return l=e,d&&i?f(t):(i=o=e,s)}function m(){var t=$a(),r=p(t);if(i=arguments,o=this,c=t,r){if(l===e)return function(t){return u=t,l=ia(I,n),g?f(t):s}(c);if(h)return l=ia(I,n),f(c)}return l===e&&(l=ia(I,n)),s}return n=Ys(n)||0,xs(r)&&(g=!!r.leading,a=(h="maxWait"in r)?Kn(Ys(r.maxWait)||0,n):a,d="trailing"in r?!!r.trailing:d),m.cancel=function(){l!==e&&Vi(l),u=0,i=c=o=l=e},m.flush=function(){return l===e?s:C($a())},m}var os=Bi(function(t,e){return Tr(t,1,e)}),as=Bi(function(t,e,n){return Tr(t,Ys(e)||0,n)});function ss(t,e){if("function"!=typeof t||null!=e&&"function"!=typeof e)throw new ie(A);var n=function(){var r=arguments,i=e?e.apply(this,r):r[0],o=n.cache;if(o.has(i))return o.get(i);var a=t.apply(this,r);return n.cache=o.set(i,a)||o,a};return n.cache=new(ss.Cache||yr),n}function As(t){if("function"!=typeof t)throw new ie(A);return function(){var e=arguments;switch(e.length){case 0:return!t.call(this);case 1:return!t.call(this,e[0]);case 2:return!t.call(this,e[0],e[1]);case 3:return!t.call(this,e[0],e[1],e[2])}return!t.apply(this,e)}}ss.Cache=yr;var ls=Ji(function(t,e){var n=(e=1==e.length&&Is(e[0])?$e(e[0],In(Fo())):$e(Pr(e,1),In(Fo()))).length;return Bi(function(r){for(var i=-1,o=zn(r.length,n);++i<o;)r[i]=e[i].call(this,r[i]);return Ke(t,this,r)})}),cs=Bi(function(t,n){var r=_n(n,No(cs));return Qo(t,v,e,n,r)}),us=Bi(function(t,n){var r=_n(n,No(us));return Qo(t,B,e,n,r)}),gs=Mo(function(t,n){return Qo(t,Q,e,e,e,n)});function hs(t,e){return t===e||t!=t&&e!=e}var ds=yo($r),fs=yo(function(t,e){return t>=e}),ps=ii(function(){return arguments}())?ii:function(t){return _s(t)&&ce.call(t,"callee")&&!Re.call(t,"callee")},Is=i.isArray,Cs=Oe?In(Oe):function(t){return _s(t)&&Zr(t)==At};function ms(t){return null!=t&&ws(t.length)&&!bs(t)}function ys(t){return _s(t)&&ms(t)}var Es=Hn||YA,vs=Ge?In(Ge):function(t){return _s(t)&&Zr(t)==K};function Bs(t){if(!_s(t))return!1;var e=Zr(t);return e==J||e==z||"string"==typeof t.message&&"string"==typeof t.name&&!ks(t)}function bs(t){if(!xs(t))return!1;var e=Zr(t);return e==q||e==V||e==Y||e==tt}function Qs(t){return"number"==typeof t&&t==Hs(t)}function ws(t){return"number"==typeof t&&t>-1&&t%1==0&&t<=R}function xs(t){var e=typeof t;return null!=t&&("object"==e||"function"==e)}function _s(t){return null!=t&&"object"==typeof t}var Ds=He?In(He):function(t){return _s(t)&&jo(t)==W};function Ms(t){return"number"==typeof t||_s(t)&&Zr(t)==X}function ks(t){if(!_s(t)||Zr(t)!=$)return!1;var e=ke(t);if(null===e)return!0;var n=ce.call(e,"constructor")&&e.constructor;return"function"==typeof n&&n instanceof n&&le.call(n)==de}var Ss=je?In(je):function(t){return _s(t)&&Zr(t)==et};var Ls=Ye?In(Ye):function(t){return _s(t)&&jo(t)==nt};function Rs(t){return"string"==typeof t||!Is(t)&&_s(t)&&Zr(t)==rt}function Ns(t){return"symbol"==typeof t||_s(t)&&Zr(t)==it}var Fs=Pe?In(Pe):function(t){return _s(t)&&ws(t.length)&&!!we[Zr(t)]};var Ts=yo(ui),Us=yo(function(t,e){return t<=e});function Os(t){if(!t)return[];if(ms(t))return Rs(t)?Sn(t):no(t);if(Ue&&t[Ue])return function(t){for(var e,n=[];!(e=t.next()).done;)n.push(e.value);return n}(t[Ue]());var e=jo(t);return(e==W?wn:e==nt?Dn:hA)(t)}function Gs(t){return t?(t=Ys(t))===L||t===-L?(t<0?-1:1)*N:t==t?t:0:0===t?t:0}function Hs(t){var e=Gs(t),n=e%1;return e==e?n?e-n:e:0}function js(t){return t?Rr(Hs(t),0,T):0}function Ys(t){if("number"==typeof t)return t;if(Ns(t))return F;if(xs(t)){var e="function"==typeof t.valueOf?t.valueOf():t;t=xs(e)?e+"":e}if("string"!=typeof t)return 0===t?t:+t;t=t.replace(Rt,"");var n=Kt.test(t);return n||Jt.test(t)?Me(t.slice(2),n?2:8):Pt.test(t)?F:+t}function Ps(t){return ro(t,oA(t))}function Ks(t){return null==t?"":Fi(t)}var zs=oo(function(t,e){if(Xo(e)||ms(e))ro(e,iA(e),t);else for(var n in e)ce.call(e,n)&&_r(t,n,e[n])}),Js=oo(function(t,e){ro(e,oA(e),t)}),qs=oo(function(t,e,n,r){ro(e,oA(e),t,r)}),Vs=oo(function(t,e,n,r){ro(e,iA(e),t,r)}),Ws=Mo(Lr);var Xs=Bi(function(t,n){t=ee(t);var r=-1,i=n.length,o=i>2?n[2]:e;for(o&&Jo(n[0],n[1],o)&&(i=1);++r<i;)for(var a=n[r],s=oA(a),A=-1,l=s.length;++A<l;){var c=s[A],u=t[c];(u===e||hs(u,se[c])&&!ce.call(t,c))&&(t[c]=a[c])}return t}),Zs=Bi(function(t){return t.push(e,xo),Ke(sA,e,t)});function $s(t,n,r){var i=null==t?e:Wr(t,n);return i===e?r:i}function tA(t,e){return null!=t&&Yo(t,e,ei)}var eA=fo(function(t,e,n){null!=e&&"function"!=typeof e.toString&&(e=he.call(e)),t[e]=n},xA(MA)),nA=fo(function(t,e,n){null!=e&&"function"!=typeof e.toString&&(e=he.call(e)),ce.call(t,e)?t[e].push(n):t[e]=[n]},Fo),rA=Bi(ri);function iA(t){return ms(t)?Br(t):li(t)}function oA(t){return ms(t)?Br(t,!0):ci(t)}var aA=oo(function(t,e,n){fi(t,e,n)}),sA=oo(function(t,e,n,r){fi(t,e,n,r)}),AA=Mo(function(t,e){var n={};if(null==t)return n;var r=!1;e=$e(e,function(e){return e=zi(e,t),r||(r=e.length>1),e}),ro(t,So(t),n),r&&(n=Nr(n,g|h|d,_o));for(var i=e.length;i--;)Ui(n,e[i]);return n});var lA=Mo(function(t,e){return null==t?{}:function(t,e){return Ci(t,e,function(e,n){return tA(t,n)})}(t,e)});function cA(t,e){if(null==t)return{};var n=$e(So(t),function(t){return[t]});return e=Fo(e),Ci(t,n,function(t,n){return e(t,n[0])})}var uA=bo(iA),gA=bo(oA);function hA(t){return null==t?[]:Cn(t,iA(t))}var dA=lo(function(t,e,n){return e=e.toLowerCase(),t+(n?fA(e):e)});function fA(t){return BA(Ks(t).toLowerCase())}function pA(t){return(t=Ks(t))&&t.replace(Vt,vn).replace(me,"")}var IA=lo(function(t,e,n){return t+(n?"-":"")+e.toLowerCase()}),CA=lo(function(t,e,n){return t+(n?" ":"")+e.toLowerCase()}),mA=Ao("toLowerCase");var yA=lo(function(t,e,n){return t+(n?"_":"")+e.toLowerCase()});var EA=lo(function(t,e,n){return t+(n?" ":"")+BA(e)});var vA=lo(function(t,e,n){return t+(n?" ":"")+e.toUpperCase()}),BA=Ao("toUpperCase");function bA(t,n,r){return t=Ks(t),(n=r?e:n)===e?function(t){return Be.test(t)}(t)?function(t){return t.match(Ee)||[]}(t):function(t){return t.match(Gt)||[]}(t):t.match(n)||[]}var QA=Bi(function(t,n){try{return Ke(t,e,n)}catch(t){return Bs(t)?t:new Zt(t)}}),wA=Mo(function(t,e){return Je(e,function(e){e=ca(e),Sr(t,e,ns(t[e],t))}),t});function xA(t){return function(){return t}}var _A=go(),DA=go(!0);function MA(t){return t}function kA(t){return Ai("function"==typeof t?t:Nr(t,g))}var SA=Bi(function(t,e){return function(n){return ri(n,t,e)}}),LA=Bi(function(t,e){return function(n){return ri(t,n,e)}});function RA(t,e,n){var r=iA(e),i=Vr(e,r);null!=n||xs(e)&&(i.length||!r.length)||(n=e,e=t,t=this,i=Vr(e,iA(e)));var o=!(xs(n)&&"chain"in n&&!n.chain),a=bs(t);return Je(i,function(n){var r=e[n];t[n]=r,a&&(t.prototype[n]=function(){var e=this.__chain__;if(o||e){var n=t(this.__wrapped__),i=n.__actions__=no(this.__actions__);return i.push({func:r,args:arguments,thisArg:t}),n.__chain__=e,n}return r.apply(t,tn([this.value()],arguments))})}),t}function NA(){}var FA=Io($e),TA=Io(Ve),UA=Io(rn);function OA(t){return qo(t)?gn(ca(t)):function(t){return function(e){return Wr(e,t)}}(t)}var GA=mo(),HA=mo(!0);function jA(){return[]}function YA(){return!1}var PA=po(function(t,e){return t+e},0),KA=vo("ceil"),zA=po(function(t,e){return t/e},1),JA=vo("floor");var qA,VA=po(function(t,e){return t*e},1),WA=vo("round"),XA=po(function(t,e){return t-e},0);return hr.after=function(t,e){if("function"!=typeof e)throw new ie(A);return t=Hs(t),function(){if(--t<1)return e.apply(this,arguments)}},hr.ary=ts,hr.assign=zs,hr.assignIn=Js,hr.assignInWith=qs,hr.assignWith=Vs,hr.at=Ws,hr.before=es,hr.bind=ns,hr.bindAll=wA,hr.bindKey=rs,hr.castArray=function(){if(!arguments.length)return[];var t=arguments[0];return Is(t)?t:[t]},hr.chain=Oa,hr.chunk=function(t,n,r){n=(r?Jo(t,n,r):n===e)?1:Kn(Hs(n),0);var o=null==t?0:t.length;if(!o||n<1)return[];for(var a=0,s=0,A=i(Un(o/n));a<o;)A[s++]=Mi(t,a,a+=n);return A},hr.compact=function(t){for(var e=-1,n=null==t?0:t.length,r=0,i=[];++e<n;){var o=t[e];o&&(i[r++]=o)}return i},hr.concat=function(){var t=arguments.length;if(!t)return[];for(var e=i(t-1),n=arguments[0],r=t;r--;)e[r-1]=arguments[r];return tn(Is(n)?no(n):[n],Pr(e,1))},hr.cond=function(t){var e=null==t?0:t.length,n=Fo();return t=e?$e(t,function(t){if("function"!=typeof t[1])throw new ie(A);return[n(t[0]),t[1]]}):[],Bi(function(n){for(var r=-1;++r<e;){var i=t[r];if(Ke(i[0],this,n))return Ke(i[1],this,n)}})},hr.conforms=function(t){return function(t){var e=iA(t);return function(n){return Fr(n,t,e)}}(Nr(t,g))},hr.constant=xA,hr.countBy=ja,hr.create=function(t,e){var n=dr(t);return null==e?n:kr(n,e)},hr.curry=function t(n,r,i){var o=Qo(n,y,e,e,e,e,e,r=i?e:r);return o.placeholder=t.placeholder,o},hr.curryRight=function t(n,r,i){var o=Qo(n,E,e,e,e,e,e,r=i?e:r);return o.placeholder=t.placeholder,o},hr.debounce=is,hr.defaults=Xs,hr.defaultsDeep=Zs,hr.defer=os,hr.delay=as,hr.difference=ha,hr.differenceBy=da,hr.differenceWith=fa,hr.drop=function(t,n,r){var i=null==t?0:t.length;return i?Mi(t,(n=r||n===e?1:Hs(n))<0?0:n,i):[]},hr.dropRight=function(t,n,r){var i=null==t?0:t.length;return i?Mi(t,0,(n=i-(n=r||n===e?1:Hs(n)))<0?0:n):[]},hr.dropRightWhile=function(t,e){return t&&t.length?Gi(t,Fo(e,3),!0,!0):[]},hr.dropWhile=function(t,e){return t&&t.length?Gi(t,Fo(e,3),!0):[]},hr.fill=function(t,n,r,i){var o=null==t?0:t.length;return o?(r&&"number"!=typeof r&&Jo(t,n,r)&&(r=0,i=o),function(t,n,r,i){var o=t.length;for((r=Hs(r))<0&&(r=-r>o?0:o+r),(i=i===e||i>o?o:Hs(i))<0&&(i+=o),i=r>i?0:js(i);r<i;)t[r++]=n;return t}(t,n,r,i)):[]},hr.filter=function(t,e){return(Is(t)?We:Yr)(t,Fo(e,3))},hr.flatMap=function(t,e){return Pr(Wa(t,e),1)},hr.flatMapDeep=function(t,e){return Pr(Wa(t,e),L)},hr.flatMapDepth=function(t,n,r){return r=r===e?1:Hs(r),Pr(Wa(t,n),r)},hr.flatten=Ca,hr.flattenDeep=function(t){return null!=t&&t.length?Pr(t,L):[]},hr.flattenDepth=function(t,n){return null!=t&&t.length?Pr(t,n=n===e?1:Hs(n)):[]},hr.flip=function(t){return Qo(t,w)},hr.flow=_A,hr.flowRight=DA,hr.fromPairs=function(t){for(var e=-1,n=null==t?0:t.length,r={};++e<n;){var i=t[e];r[i[0]]=i[1]}return r},hr.functions=function(t){return null==t?[]:Vr(t,iA(t))},hr.functionsIn=function(t){return null==t?[]:Vr(t,oA(t))},hr.groupBy=Ja,hr.initial=function(t){return null!=t&&t.length?Mi(t,0,-1):[]},hr.intersection=ya,hr.intersectionBy=Ea,hr.intersectionWith=va,hr.invert=eA,hr.invertBy=nA,hr.invokeMap=qa,hr.iteratee=kA,hr.keyBy=Va,hr.keys=iA,hr.keysIn=oA,hr.map=Wa,hr.mapKeys=function(t,e){var n={};return e=Fo(e,3),Jr(t,function(t,r,i){Sr(n,e(t,r,i),t)}),n},hr.mapValues=function(t,e){var n={};return e=Fo(e,3),Jr(t,function(t,r,i){Sr(n,r,e(t,r,i))}),n},hr.matches=function(t){return hi(Nr(t,g))},hr.matchesProperty=function(t,e){return di(t,Nr(e,g))},hr.memoize=ss,hr.merge=aA,hr.mergeWith=sA,hr.method=SA,hr.methodOf=LA,hr.mixin=RA,hr.negate=As,hr.nthArg=function(t){return t=Hs(t),Bi(function(e){return pi(e,t)})},hr.omit=AA,hr.omitBy=function(t,e){return cA(t,As(Fo(e)))},hr.once=function(t){return es(2,t)},hr.orderBy=function(t,n,r,i){return null==t?[]:(Is(n)||(n=null==n?[]:[n]),Is(r=i?e:r)||(r=null==r?[]:[r]),Ii(t,n,r))},hr.over=FA,hr.overArgs=ls,hr.overEvery=TA,hr.overSome=UA,hr.partial=cs,hr.partialRight=us,hr.partition=Xa,hr.pick=lA,hr.pickBy=cA,hr.property=OA,hr.propertyOf=function(t){return function(n){return null==t?e:Wr(t,n)}},hr.pull=ba,hr.pullAll=Qa,hr.pullAllBy=function(t,e,n){return t&&t.length&&e&&e.length?mi(t,e,Fo(n,2)):t},hr.pullAllWith=function(t,n,r){return t&&t.length&&n&&n.length?mi(t,n,e,r):t},hr.pullAt=wa,hr.range=GA,hr.rangeRight=HA,hr.rearg=gs,hr.reject=function(t,e){return(Is(t)?We:Yr)(t,As(Fo(e,3)))},hr.remove=function(t,e){var n=[];if(!t||!t.length)return n;var r=-1,i=[],o=t.length;for(e=Fo(e,3);++r<o;){var a=t[r];e(a,r,t)&&(n.push(a),i.push(r))}return yi(t,i),n},hr.rest=function(t,n){if("function"!=typeof t)throw new ie(A);return Bi(t,n=n===e?n:Hs(n))},hr.reverse=xa,hr.sampleSize=function(t,n,r){return n=(r?Jo(t,n,r):n===e)?1:Hs(n),(Is(t)?Qr:Qi)(t,n)},hr.set=function(t,e,n){return null==t?t:wi(t,e,n)},hr.setWith=function(t,n,r,i){return i="function"==typeof i?i:e,null==t?t:wi(t,n,r,i)},hr.shuffle=function(t){return(Is(t)?wr:Di)(t)},hr.slice=function(t,n,r){var i=null==t?0:t.length;return i?(r&&"number"!=typeof r&&Jo(t,n,r)?(n=0,r=i):(n=null==n?0:Hs(n),r=r===e?i:Hs(r)),Mi(t,n,r)):[]},hr.sortBy=Za,hr.sortedUniq=function(t){return t&&t.length?Ri(t):[]},hr.sortedUniqBy=function(t,e){return t&&t.length?Ri(t,Fo(e,2)):[]},hr.split=function(t,n,r){return r&&"number"!=typeof r&&Jo(t,n,r)&&(n=r=e),(r=r===e?T:r>>>0)?(t=Ks(t))&&("string"==typeof n||null!=n&&!Ss(n))&&!(n=Fi(n))&&Qn(t)?qi(Sn(t),0,r):t.split(n,r):[]},hr.spread=function(t,e){if("function"!=typeof t)throw new ie(A);return e=null==e?0:Kn(Hs(e),0),Bi(function(n){var r=n[e],i=qi(n,0,e);return r&&tn(i,r),Ke(t,this,i)})},hr.tail=function(t){var e=null==t?0:t.length;return e?Mi(t,1,e):[]},hr.take=function(t,n,r){return t&&t.length?Mi(t,0,(n=r||n===e?1:Hs(n))<0?0:n):[]},hr.takeRight=function(t,n,r){var i=null==t?0:t.length;return i?Mi(t,(n=i-(n=r||n===e?1:Hs(n)))<0?0:n,i):[]},hr.takeRightWhile=function(t,e){return t&&t.length?Gi(t,Fo(e,3),!1,!0):[]},hr.takeWhile=function(t,e){return t&&t.length?Gi(t,Fo(e,3)):[]},hr.tap=function(t,e){return e(t),t},hr.throttle=function(t,e,n){var r=!0,i=!0;if("function"!=typeof t)throw new ie(A);return xs(n)&&(r="leading"in n?!!n.leading:r,i="trailing"in n?!!n.trailing:i),is(t,e,{leading:r,maxWait:e,trailing:i})},hr.thru=Ga,hr.toArray=Os,hr.toPairs=uA,hr.toPairsIn=gA,hr.toPath=function(t){return Is(t)?$e(t,ca):Ns(t)?[t]:no(la(Ks(t)))},hr.toPlainObject=Ps,hr.transform=function(t,e,n){var r=Is(t),i=r||Es(t)||Fs(t);if(e=Fo(e,4),null==n){var o=t&&t.constructor;n=i?r?new o:[]:xs(t)&&bs(o)?dr(ke(t)):{}}return(i?Je:Jr)(t,function(t,r,i){return e(n,t,r,i)}),n},hr.unary=function(t){return ts(t,1)},hr.union=_a,hr.unionBy=Da,hr.unionWith=Ma,hr.uniq=function(t){return t&&t.length?Ti(t):[]},hr.uniqBy=function(t,e){return t&&t.length?Ti(t,Fo(e,2)):[]},hr.uniqWith=function(t,n){return n="function"==typeof n?n:e,t&&t.length?Ti(t,e,n):[]},hr.unset=function(t,e){return null==t||Ui(t,e)},hr.unzip=ka,hr.unzipWith=Sa,hr.update=function(t,e,n){return null==t?t:Oi(t,e,Ki(n))},hr.updateWith=function(t,n,r,i){return i="function"==typeof i?i:e,null==t?t:Oi(t,n,Ki(r),i)},hr.values=hA,hr.valuesIn=function(t){return null==t?[]:Cn(t,oA(t))},hr.without=La,hr.words=bA,hr.wrap=function(t,e){return cs(Ki(e),t)},hr.xor=Ra,hr.xorBy=Na,hr.xorWith=Fa,hr.zip=Ta,hr.zipObject=function(t,e){return Yi(t||[],e||[],_r)},hr.zipObjectDeep=function(t,e){return Yi(t||[],e||[],wi)},hr.zipWith=Ua,hr.entries=uA,hr.entriesIn=gA,hr.extend=Js,hr.extendWith=qs,RA(hr,hr),hr.add=PA,hr.attempt=QA,hr.camelCase=dA,hr.capitalize=fA,hr.ceil=KA,hr.clamp=function(t,n,r){return r===e&&(r=n,n=e),r!==e&&(r=(r=Ys(r))==r?r:0),n!==e&&(n=(n=Ys(n))==n?n:0),Rr(Ys(t),n,r)},hr.clone=function(t){return Nr(t,d)},hr.cloneDeep=function(t){return Nr(t,g|d)},hr.cloneDeepWith=function(t,n){return Nr(t,g|d,n="function"==typeof n?n:e)},hr.cloneWith=function(t,n){return Nr(t,d,n="function"==typeof n?n:e)},hr.conformsTo=function(t,e){return null==e||Fr(t,e,iA(e))},hr.deburr=pA,hr.defaultTo=function(t,e){return null==t||t!=t?e:t},hr.divide=zA,hr.endsWith=function(t,n,r){t=Ks(t),n=Fi(n);var i=t.length,o=r=r===e?i:Rr(Hs(r),0,i);return(r-=n.length)>=0&&t.slice(r,o)==n},hr.eq=hs,hr.escape=function(t){return(t=Ks(t))&&Qt.test(t)?t.replace(Bt,Bn):t},hr.escapeRegExp=function(t){return(t=Ks(t))&&Lt.test(t)?t.replace(St,"\\$&"):t},hr.every=function(t,n,r){var i=Is(t)?Ve:Hr;return r&&Jo(t,n,r)&&(n=e),i(t,Fo(n,3))},hr.find=Ya,hr.findIndex=pa,hr.findKey=function(t,e){return an(t,Fo(e,3),Jr)},hr.findLast=Pa,hr.findLastIndex=Ia,hr.findLastKey=function(t,e){return an(t,Fo(e,3),qr)},hr.floor=JA,hr.forEach=Ka,hr.forEachRight=za,hr.forIn=function(t,e){return null==t?t:Kr(t,Fo(e,3),oA)},hr.forInRight=function(t,e){return null==t?t:zr(t,Fo(e,3),oA)},hr.forOwn=function(t,e){return t&&Jr(t,Fo(e,3))},hr.forOwnRight=function(t,e){return t&&qr(t,Fo(e,3))},hr.get=$s,hr.gt=ds,hr.gte=fs,hr.has=function(t,e){return null!=t&&Yo(t,e,ti)},hr.hasIn=tA,hr.head=ma,hr.identity=MA,hr.includes=function(t,e,n,r){t=ms(t)?t:hA(t),n=n&&!r?Hs(n):0;var i=t.length;return n<0&&(n=Kn(i+n,0)),Rs(t)?n<=i&&t.indexOf(e,n)>-1:!!i&&An(t,e,n)>-1},hr.indexOf=function(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=null==n?0:Hs(n);return i<0&&(i=Kn(r+i,0)),An(t,e,i)},hr.inRange=function(t,n,r){return n=Gs(n),r===e?(r=n,n=0):r=Gs(r),function(t,e,n){return t>=zn(e,n)&&t<Kn(e,n)}(t=Ys(t),n,r)},hr.invoke=rA,hr.isArguments=ps,hr.isArray=Is,hr.isArrayBuffer=Cs,hr.isArrayLike=ms,hr.isArrayLikeObject=ys,hr.isBoolean=function(t){return!0===t||!1===t||_s(t)&&Zr(t)==P},hr.isBuffer=Es,hr.isDate=vs,hr.isElement=function(t){return _s(t)&&1===t.nodeType&&!ks(t)},hr.isEmpty=function(t){if(null==t)return!0;if(ms(t)&&(Is(t)||"string"==typeof t||"function"==typeof t.splice||Es(t)||Fs(t)||ps(t)))return!t.length;var e=jo(t);if(e==W||e==nt)return!t.size;if(Xo(t))return!li(t).length;for(var n in t)if(ce.call(t,n))return!1;return!0},hr.isEqual=function(t,e){return oi(t,e)},hr.isEqualWith=function(t,n,r){var i=(r="function"==typeof r?r:e)?r(t,n):e;return i===e?oi(t,n,e,r):!!i},hr.isError=Bs,hr.isFinite=function(t){return"number"==typeof t&&jn(t)},hr.isFunction=bs,hr.isInteger=Qs,hr.isLength=ws,hr.isMap=Ds,hr.isMatch=function(t,e){return t===e||ai(t,e,Uo(e))},hr.isMatchWith=function(t,n,r){return r="function"==typeof r?r:e,ai(t,n,Uo(n),r)},hr.isNaN=function(t){return Ms(t)&&t!=+t},hr.isNative=function(t){if(Wo(t))throw new Zt(s);return si(t)},hr.isNil=function(t){return null==t},hr.isNull=function(t){return null===t},hr.isNumber=Ms,hr.isObject=xs,hr.isObjectLike=_s,hr.isPlainObject=ks,hr.isRegExp=Ss,hr.isSafeInteger=function(t){return Qs(t)&&t>=-R&&t<=R},hr.isSet=Ls,hr.isString=Rs,hr.isSymbol=Ns,hr.isTypedArray=Fs,hr.isUndefined=function(t){return t===e},hr.isWeakMap=function(t){return _s(t)&&jo(t)==at},hr.isWeakSet=function(t){return _s(t)&&Zr(t)==st},hr.join=function(t,e){return null==t?"":Yn.call(t,e)},hr.kebabCase=IA,hr.last=Ba,hr.lastIndexOf=function(t,n,r){var i=null==t?0:t.length;if(!i)return-1;var o=i;return r!==e&&(o=(o=Hs(r))<0?Kn(i+o,0):zn(o,i-1)),n==n?function(t,e,n){for(var r=n+1;r--;)if(t[r]===e)return r;return r}(t,n,o):sn(t,cn,o,!0)},hr.lowerCase=CA,hr.lowerFirst=mA,hr.lt=Ts,hr.lte=Us,hr.max=function(t){return t&&t.length?jr(t,MA,$r):e},hr.maxBy=function(t,n){return t&&t.length?jr(t,Fo(n,2),$r):e},hr.mean=function(t){return un(t,MA)},hr.meanBy=function(t,e){return un(t,Fo(e,2))},hr.min=function(t){return t&&t.length?jr(t,MA,ui):e},hr.minBy=function(t,n){return t&&t.length?jr(t,Fo(n,2),ui):e},hr.stubArray=jA,hr.stubFalse=YA,hr.stubObject=function(){return{}},hr.stubString=function(){return""},hr.stubTrue=function(){return!0},hr.multiply=VA,hr.nth=function(t,n){return t&&t.length?pi(t,Hs(n)):e},hr.noConflict=function(){return Le._===this&&(Le._=fe),this},hr.noop=NA,hr.now=$a,hr.pad=function(t,e,n){t=Ks(t);var r=(e=Hs(e))?kn(t):0;if(!e||r>=e)return t;var i=(e-r)/2;return Co(On(i),n)+t+Co(Un(i),n)},hr.padEnd=function(t,e,n){t=Ks(t);var r=(e=Hs(e))?kn(t):0;return e&&r<e?t+Co(e-r,n):t},hr.padStart=function(t,e,n){t=Ks(t);var r=(e=Hs(e))?kn(t):0;return e&&r<e?Co(e-r,n)+t:t},hr.parseInt=function(t,e,n){return n||null==e?e=0:e&&(e=+e),qn(Ks(t).replace(Nt,""),e||0)},hr.random=function(t,n,r){if(r&&"boolean"!=typeof r&&Jo(t,n,r)&&(n=r=e),r===e&&("boolean"==typeof n?(r=n,n=e):"boolean"==typeof t&&(r=t,t=e)),t===e&&n===e?(t=0,n=1):(t=Gs(t),n===e?(n=t,t=0):n=Gs(n)),t>n){var i=t;t=n,n=i}if(r||t%1||n%1){var o=Vn();return zn(t+o*(n-t+De("1e-"+((o+"").length-1))),n)}return Ei(t,n)},hr.reduce=function(t,e,n){var r=Is(t)?en:dn,i=arguments.length<3;return r(t,Fo(e,4),n,i,Or)},hr.reduceRight=function(t,e,n){var r=Is(t)?nn:dn,i=arguments.length<3;return r(t,Fo(e,4),n,i,Gr)},hr.repeat=function(t,n,r){return n=(r?Jo(t,n,r):n===e)?1:Hs(n),vi(Ks(t),n)},hr.replace=function(){var t=arguments,e=Ks(t[0]);return t.length<3?e:e.replace(t[1],t[2])},hr.result=function(t,n,r){var i=-1,o=(n=zi(n,t)).length;for(o||(o=1,t=e);++i<o;){var a=null==t?e:t[ca(n[i])];a===e&&(i=o,a=r),t=bs(a)?a.call(t):a}return t},hr.round=WA,hr.runInContext=t,hr.sample=function(t){return(Is(t)?br:bi)(t)},hr.size=function(t){if(null==t)return 0;if(ms(t))return Rs(t)?kn(t):t.length;var e=jo(t);return e==W||e==nt?t.size:li(t).length},hr.snakeCase=yA,hr.some=function(t,n,r){var i=Is(t)?rn:ki;return r&&Jo(t,n,r)&&(n=e),i(t,Fo(n,3))},hr.sortedIndex=function(t,e){return Si(t,e)},hr.sortedIndexBy=function(t,e,n){return Li(t,e,Fo(n,2))},hr.sortedIndexOf=function(t,e){var n=null==t?0:t.length;if(n){var r=Si(t,e);if(r<n&&hs(t[r],e))return r}return-1},hr.sortedLastIndex=function(t,e){return Si(t,e,!0)},hr.sortedLastIndexBy=function(t,e,n){return Li(t,e,Fo(n,2),!0)},hr.sortedLastIndexOf=function(t,e){if(null!=t&&t.length){var n=Si(t,e,!0)-1;if(hs(t[n],e))return n}return-1},hr.startCase=EA,hr.startsWith=function(t,e,n){return t=Ks(t),n=null==n?0:Rr(Hs(n),0,t.length),e=Fi(e),t.slice(n,n+e.length)==e},hr.subtract=XA,hr.sum=function(t){return t&&t.length?fn(t,MA):0},hr.sumBy=function(t,e){return t&&t.length?fn(t,Fo(e,2)):0},hr.template=function(t,n,r){var i=hr.templateSettings;r&&Jo(t,n,r)&&(n=e),t=Ks(t),n=qs({},n,i,wo);var o,a,s=qs({},n.imports,i.imports,wo),A=iA(s),l=Cn(s,A),c=0,u=n.interpolate||Wt,g="__p += '",h=ne((n.escape||Wt).source+"|"+u.source+"|"+(u===_t?jt:Wt).source+"|"+(n.evaluate||Wt).source+"|$","g"),d="//# sourceURL="+("sourceURL"in n?n.sourceURL:"lodash.templateSources["+ ++Qe+"]")+"\n";t.replace(h,function(e,n,r,i,s,A){return r||(r=i),g+=t.slice(c,A).replace(Xt,bn),n&&(o=!0,g+="' +\n__e("+n+") +\n'"),s&&(a=!0,g+="';\n"+s+";\n__p += '"),r&&(g+="' +\n((__t = ("+r+")) == null ? '' : __t) +\n'"),c=A+e.length,e}),g+="';\n";var f=n.variable;f||(g="with (obj) {\n"+g+"\n}\n"),g=(a?g.replace(mt,""):g).replace(yt,"$1").replace(Et,"$1;"),g="function("+(f||"obj")+") {\n"+(f?"":"obj || (obj = {});\n")+"var __t, __p = ''"+(o?", __e = _.escape":"")+(a?", __j = Array.prototype.join;\nfunction print() { __p += __j.call(arguments, '') }\n":";\n")+g+"return __p\n}";var p=QA(function(){return $t(A,d+"return "+g).apply(e,l)});if(p.source=g,Bs(p))throw p;return p},hr.times=function(t,e){if((t=Hs(t))<1||t>R)return[];var n=T,r=zn(t,T);e=Fo(e),t-=T;for(var i=pn(r,e);++n<t;)e(n);return i},hr.toFinite=Gs,hr.toInteger=Hs,hr.toLength=js,hr.toLower=function(t){return Ks(t).toLowerCase()},hr.toNumber=Ys,hr.toSafeInteger=function(t){return t?Rr(Hs(t),-R,R):0===t?t:0},hr.toString=Ks,hr.toUpper=function(t){return Ks(t).toUpperCase()},hr.trim=function(t,n,r){if((t=Ks(t))&&(r||n===e))return t.replace(Rt,"");if(!t||!(n=Fi(n)))return t;var i=Sn(t),o=Sn(n);return qi(i,yn(i,o),En(i,o)+1).join("")},hr.trimEnd=function(t,n,r){if((t=Ks(t))&&(r||n===e))return t.replace(Ft,"");if(!t||!(n=Fi(n)))return t;var i=Sn(t);return qi(i,0,En(i,Sn(n))+1).join("")},hr.trimStart=function(t,n,r){if((t=Ks(t))&&(r||n===e))return t.replace(Nt,"");if(!t||!(n=Fi(n)))return t;var i=Sn(t);return qi(i,yn(i,Sn(n))).join("")},hr.truncate=function(t,n){var r=x,i=_;if(xs(n)){var o="separator"in n?n.separator:o;r="length"in n?Hs(n.length):r,i="omission"in n?Fi(n.omission):i}var a=(t=Ks(t)).length;if(Qn(t)){var s=Sn(t);a=s.length}if(r>=a)return t;var A=r-kn(i);if(A<1)return i;var l=s?qi(s,0,A).join(""):t.slice(0,A);if(o===e)return l+i;if(s&&(A+=l.length-A),Ss(o)){if(t.slice(A).search(o)){var c,u=l;for(o.global||(o=ne(o.source,Ks(Yt.exec(o))+"g")),o.lastIndex=0;c=o.exec(u);)var g=c.index;l=l.slice(0,g===e?A:g)}}else if(t.indexOf(Fi(o),A)!=A){var h=l.lastIndexOf(o);h>-1&&(l=l.slice(0,h))}return l+i},hr.unescape=function(t){return(t=Ks(t))&&bt.test(t)?t.replace(vt,Ln):t},hr.uniqueId=function(t){var e=++ue;return Ks(t)+e},hr.upperCase=vA,hr.upperFirst=BA,hr.each=Ka,hr.eachRight=za,hr.first=ma,RA(hr,(qA={},Jr(hr,function(t,e){ce.call(hr.prototype,e)||(qA[e]=t)}),qA),{chain:!1}),hr.VERSION="4.17.11",Je(["bind","bindKey","curry","curryRight","partial","partialRight"],function(t){hr[t].placeholder=hr}),Je(["drop","take"],function(t,n){Ir.prototype[t]=function(r){r=r===e?1:Kn(Hs(r),0);var i=this.__filtered__&&!n?new Ir(this):this.clone();return i.__filtered__?i.__takeCount__=zn(r,i.__takeCount__):i.__views__.push({size:zn(r,T),type:t+(i.__dir__<0?"Right":"")}),i},Ir.prototype[t+"Right"]=function(e){return this.reverse()[t](e).reverse()}}),Je(["filter","map","takeWhile"],function(t,e){var n=e+1,r=n==k||3==n;Ir.prototype[t]=function(t){var e=this.clone();return e.__iteratees__.push({iteratee:Fo(t,3),type:n}),e.__filtered__=e.__filtered__||r,e}}),Je(["head","last"],function(t,e){var n="take"+(e?"Right":"");Ir.prototype[t]=function(){return this[n](1).value()[0]}}),Je(["initial","tail"],function(t,e){var n="drop"+(e?"":"Right");Ir.prototype[t]=function(){return this.__filtered__?new Ir(this):this[n](1)}}),Ir.prototype.compact=function(){return this.filter(MA)},Ir.prototype.find=function(t){return this.filter(t).head()},Ir.prototype.findLast=function(t){return this.reverse().find(t)},Ir.prototype.invokeMap=Bi(function(t,e){return"function"==typeof t?new Ir(this):this.map(function(n){return ri(n,t,e)})}),Ir.prototype.reject=function(t){return this.filter(As(Fo(t)))},Ir.prototype.slice=function(t,n){t=Hs(t);var r=this;return r.__filtered__&&(t>0||n<0)?new Ir(r):(t<0?r=r.takeRight(-t):t&&(r=r.drop(t)),n!==e&&(r=(n=Hs(n))<0?r.dropRight(-n):r.take(n-t)),r)},Ir.prototype.takeRightWhile=function(t){return this.reverse().takeWhile(t).reverse()},Ir.prototype.toArray=function(){return this.take(T)},Jr(Ir.prototype,function(t,n){var r=/^(?:filter|find|map|reject)|While$/.test(n),i=/^(?:head|last)$/.test(n),o=hr[i?"take"+("last"==n?"Right":""):n],a=i||/^find/.test(n);o&&(hr.prototype[n]=function(){var n=this.__wrapped__,s=i?[1]:arguments,A=n instanceof Ir,l=s[0],c=A||Is(n),u=function(t){var e=o.apply(hr,tn([t],s));return i&&g?e[0]:e};c&&r&&"function"==typeof l&&1!=l.length&&(A=c=!1);var g=this.__chain__,h=!!this.__actions__.length,d=a&&!g,f=A&&!h;if(!a&&c){n=f?n:new Ir(this);var p=t.apply(n,s);return p.__actions__.push({func:Ga,args:[u],thisArg:e}),new pr(p,g)}return d&&f?t.apply(this,s):(p=this.thru(u),d?i?p.value()[0]:p.value():p)})}),Je(["pop","push","shift","sort","splice","unshift"],function(t){var e=oe[t],n=/^(?:push|sort|unshift)$/.test(t)?"tap":"thru",r=/^(?:pop|shift)$/.test(t);hr.prototype[t]=function(){var t=arguments;if(r&&!this.__chain__){var i=this.value();return e.apply(Is(i)?i:[],t)}return this[n](function(n){return e.apply(Is(n)?n:[],t)})}}),Jr(Ir.prototype,function(t,e){var n=hr[e];if(n){var r=n.name+"";(ir[r]||(ir[r]=[])).push({name:e,func:n})}}),ir[ho(e,C).name]=[{name:"wrapper",func:e}],Ir.prototype.clone=function(){var t=new Ir(this.__wrapped__);return t.__actions__=no(this.__actions__),t.__dir__=this.__dir__,t.__filtered__=this.__filtered__,t.__iteratees__=no(this.__iteratees__),t.__takeCount__=this.__takeCount__,t.__views__=no(this.__views__),t},Ir.prototype.reverse=function(){if(this.__filtered__){var t=new Ir(this);t.__dir__=-1,t.__filtered__=!0}else(t=this.clone()).__dir__*=-1;return t},Ir.prototype.value=function(){var t=this.__wrapped__.value(),e=this.__dir__,n=Is(t),r=e<0,i=n?t.length:0,o=function(t,e,n){for(var r=-1,i=n.length;++r<i;){var o=n[r],a=o.size;switch(o.type){case"drop":t+=a;break;case"dropRight":e-=a;break;case"take":e=zn(e,t+a);break;case"takeRight":t=Kn(t,e-a)}}return{start:t,end:e}}(0,i,this.__views__),a=o.start,s=o.end,A=s-a,l=r?s:a-1,c=this.__iteratees__,u=c.length,g=0,h=zn(A,this.__takeCount__);if(!n||!r&&i==A&&h==A)return Hi(t,this.__actions__);var d=[];t:for(;A--&&g<h;){for(var f=-1,p=t[l+=e];++f<u;){var I=c[f],C=I.iteratee,m=I.type,y=C(p);if(m==S)p=y;else if(!y){if(m==k)continue t;break t}}d[g++]=p}return d},hr.prototype.at=Ha,hr.prototype.chain=function(){return Oa(this)},hr.prototype.commit=function(){return new pr(this.value(),this.__chain__)},hr.prototype.next=function(){this.__values__===e&&(this.__values__=Os(this.value()));var t=this.__index__>=this.__values__.length;return{done:t,value:t?e:this.__values__[this.__index__++]}},hr.prototype.plant=function(t){for(var n,r=this;r instanceof fr;){var i=ga(r);i.__index__=0,i.__values__=e,n?o.__wrapped__=i:n=i;var o=i;r=r.__wrapped__}return o.__wrapped__=t,n},hr.prototype.reverse=function(){var t=this.__wrapped__;if(t instanceof Ir){var n=t;return this.__actions__.length&&(n=new Ir(this)),(n=n.reverse()).__actions__.push({func:Ga,args:[xa],thisArg:e}),new pr(n,this.__chain__)}return this.thru(xa)},hr.prototype.toJSON=hr.prototype.valueOf=hr.prototype.value=function(){return Hi(this.__wrapped__,this.__actions__)},hr.prototype.first=hr.prototype.head,Ue&&(hr.prototype[Ue]=function(){return this}),hr}();Le._=Rn,(r=function(){return Rn}.call(o,n,o,i))===e||(i.exports=r)}).call(this)}).call(this,void 0!==t?t:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{}],17:[function(t,e,n){"use strict";var r=t("lodash"),i=t("events"),o=t("./reg.js"),a=t("./state.js").state,s=t("../data/ascii-identifier-data.js"),A=s.asciiIdentifierStartTable,l=s.asciiIdentifierPartTable,c=t("../data/non-ascii-identifier-start.js"),u=t("../data/non-ascii-identifier-part-only.js"),g={Identifier:1,Punctuator:2,NumericLiteral:3,StringLiteral:4,Comment:5,Keyword:6,RegExp:9,TemplateHead:10,TemplateMiddle:11,TemplateTail:12,NoSubstTemplate:13},h={Block:1,Template:2};function d(t){return/^[0-9a-fA-F]+$/.test(t)}function f(t){return 1===t.length&&d(t)}function p(t){var e=t;"string"==typeof e&&(e=e.replace(/\r\n/g,"\n").replace(/\r/g,"\n").split("\n")),e[0]&&"#!"===e[0].substr(0,2)&&(-1!==e[0].indexOf("node")&&(a.option.node=!0),e[0]=""),this.emitter=new i.EventEmitter,this.source=t,this.setLines(e),this.prereg=!0,this.line=0,this.char=1,this.from=1,this.input="",this.inComment=!1,this.context=[],this.templateStarts=[];for(var n=0;n<a.option.indent;n+=1)a.tab+=" "}p.prototype={_lines:[],inContext:function(t){return this.context.length>0&&this.context[this.context.length-1].type===t},pushContext:function(t){this.context.push({type:t})},popContext:function(){return this.context.pop()},currentContext:function(){return this.context.length>0&&this.context[this.context.length-1]},getLines:function(){return this._lines=a.lines,this._lines},setLines:function(t){this._lines=t,a.lines=this._lines},peek:function(t){return this.input.charAt(t||0)},skip:function(t){t=t||1,this.char+=t,this.input=this.input.slice(t)},on:function(t,e){t.split(" ").forEach(function(t){this.emitter.on(t,e)}.bind(this))},trigger:function(){this.emitter.emit.apply(this.emitter,Array.prototype.slice.call(arguments))},triggerAsync:function(t,e,n,r){n.push(function(){r()&&this.trigger(t,e)}.bind(this))},scanPunctuator:function(){var t,e,n,r=this.peek();switch(r){case".":if(/^[0-9]$/.test(this.peek(1)))return null;if("."===this.peek(1)&&"."===this.peek(2))return{type:g.Punctuator,value:"..."};case"(":case")":case";":case",":case"[":case"]":case":":case"~":case"?":return{type:g.Punctuator,value:r};case"{":return this.pushContext(h.Block),{type:g.Punctuator,value:r};case"}":return this.inContext(h.Block)&&this.popContext(),{type:g.Punctuator,value:r};case"#":return{type:g.Punctuator,value:r};case"":return null}return t=this.peek(1),e=this.peek(2),n=this.peek(3),">"===r&&">"===t&&">"===e&&"="===n?{type:g.Punctuator,value:">>>="}:"="===r&&"="===t&&"="===e?{type:g.Punctuator,value:"==="}:"!"===r&&"="===t&&"="===e?{type:g.Punctuator,value:"!=="}:">"===r&&">"===t&&">"===e?{type:g.Punctuator,value:">>>"}:"<"===r&&"<"===t&&"="===e?{type:g.Punctuator,value:"<<="}:">"===r&&">"===t&&"="===e?{type:g.Punctuator,value:">>="}:"="===r&&">"===t?{type:g.Punctuator,value:r+t}:r===t&&"+-<>&|*".indexOf(r)>=0?"*"===r&&"="===e?{type:g.Punctuator,value:r+t+e}:{type:g.Punctuator,value:r+t}:"<>=!+-*%&|^/".indexOf(r)>=0?"="===t?{type:g.Punctuator,value:r+t}:{type:g.Punctuator,value:r}:null},scanComments:function(t){var e=this.peek(),n=this.peek(1),r=this.input.substr(2),i=this.line,a=this.char,s=this;function A(t,e,n){var r=!1,i=t+e,a="plain";return(n=n||{}).isMultiline&&(i+="*/"),e=e.replace(/\n/g," "),"/*"===t&&o.fallsThrough.test(e)&&(r=!0,a="falls through"),["jshint","jshint.unstable","jslint","members","member","globals","global","exported"].forEach(function(n){if(!r&&("//"!==t||"jshint"===n||"jshint.unstable"===n)){" "===e.charAt(n.length)&&e.substr(0,n.length)===n&&(r=!0,t+=n,e=e.substr(n.length)),r||" "!==e.charAt(0)||" "!==e.charAt(n.length+1)||e.substr(1,n.length)!==n||(r=!0,t=t+" "+n,e=e.substr(n.length+1));var i=e.indexOf(n);if(!r&&i>=0&&" "===e.charAt(i+n.length))0===e.substr(0,i).trim().length&&(r=!0,e=e.substr(n.length+i));if(r)switch(n){case"member":a="members";break;case"global":a="globals";break;default:var o=e.split(":").map(function(t){return t.replace(/^\s+/,"").replace(/\s+$/,"")});if(2===o.length)switch(o[0]){case"ignore":switch(o[1]){case"start":s.ignoringLinterErrors=!0,r=!1;break;case"end":s.ignoringLinterErrors=!1,r=!1}}a=n}}}),{type:g.Comment,commentType:a,value:i,body:e,isSpecial:r,isMalformed:n.isMalformed||!1}}if("*"===e&&"/"===n)return this.trigger("error",{code:"E018",line:i,character:a}),this.skip(2),null;if("/"!==e||"*"!==n&&"/"!==n)return null;if("/"===n)return this.skip(this.input.length),A("//",r);var l="";if("*"===n){for(this.inComment=!0,this.skip(2);"*"!==this.peek()||"/"!==this.peek(1);)if(""===this.peek()){if(l+="\n",!this.nextLine(t))return this.trigger("error",{code:"E017",line:i,character:a}),this.inComment=!1,A("/*",l,{isMultiline:!0,isMalformed:!0})}else l+=this.peek(),this.skip();return this.skip(2),this.inComment=!1,A("/*",l,{isMultiline:!0})}},scanKeyword:function(){var t=/^[a-zA-Z_$][a-zA-Z0-9_$]*/.exec(this.input);return t&&["if","in","do","var","for","new","try","let","this","else","case","void","with","enum","while","break","catch","throw","const","yield","class","super","return","typeof","delete","switch","export","import","default","finally","extends","function","continue","debugger","instanceof","true","false","null","async","await"].indexOf(t[0])>=0?{type:g.Keyword,value:t[0]}:null},scanIdentifier:function(e){var n,r,i="",o=0;function s(t){return c.indexOf(t)>-1}function h(t){return s(t)||u.indexOf(t)>-1}var f=function(){if(o+=1,"u"!==this.peek(o))return null;var t,e=this.peek(o+1)+this.peek(o+2)+this.peek(o+3)+this.peek(o+4);return d(e)?(t=parseInt(e,16),l[t]||h(t)?(o+=5,"\\u"+e):null):null}.bind(this),p=function(){var t=this.peek(o),e=t.charCodeAt(0);return 92===e?f():e<128?A[e]?(o+=1,t):null:s(e)?(o+=1,t):null}.bind(this),I=function(){var t=this.peek(o),e=t.charCodeAt(0);return 92===e?f():e<128?l[e]?(o+=1,t):null:h(e)?(o+=1,t):null}.bind(this);if(null===(n=p()))return null;for(i=n;null!==(n=I());)i+=n;return r=function(t){return t.replace(/\\u([0-9a-fA-F]{4})/g,function(t,e){return String.fromCharCode(parseInt(e,16))})}(i),a.inES6(!0)||t("../data/es5-identifier-names.js").test(r)||this.triggerAsync("warning",{code:"W119",line:this.line,character:this.char,data:["unicode 8","6"]},e,function(){return!0}),{type:g.Identifier,value:r,text:i,tokenLength:i.length}},scanNumericLiteral:function(t){var e=0,n="",r=this.input.length,i=this.peek(e),o=l,s=10,A=!1;function l(t){return/^[0-9]$/.test(t)}function c(t){return/^[0-7]$/.test(t)}function u(t){return"$"===t||"_"===t||"\\"===t||t>="a"&&t<="z"||t>="A"&&t<="Z"}if("."!==i&&!l(i))return null;if("."!==i){for(n=this.peek(e),e+=1,i=this.peek(e),"0"===n&&("x"!==i&&"X"!==i||(o=f,s=16,e+=1,n+=i),"o"!==i&&"O"!==i||(o=c,s=8,a.inES6(!0)||this.triggerAsync("warning",{code:"W119",line:this.line,character:this.char,data:["Octal integer literal","6"]},t,function(){return!0}),e+=1,n+=i),"b"!==i&&"B"!==i||(o=function(t){return/^[01]$/.test(t)},s=2,a.inES6(!0)||this.triggerAsync("warning",{code:"W119",line:this.line,character:this.char,data:["Binary integer literal","6"]},t,function(){return!0}),e+=1,n+=i),c(i)&&(o=c,s=8,A=!0,e+=1,n+=i),!c(i)&&l(i)&&(e+=1,n+=i));e<r&&(i=this.peek(e),A&&l(i)||o(i));)n+=i,e+=1;if(o!==l)return!A&&n.length<=2?{type:g.NumericLiteral,value:n,isMalformed:!0}:e<r&&u(i=this.peek(e))?null:{type:g.NumericLiteral,value:n,base:s,isLegacy:A,isMalformed:!1}}if("."===i)for(n+=i,e+=1;e<r&&l(i=this.peek(e));)n+=i,e+=1;if("e"===i||"E"===i){if(n+=i,e+=1,"+"!==(i=this.peek(e))&&"-"!==i||(n+=this.peek(e),e+=1),!l(i=this.peek(e)))return null;for(n+=i,e+=1;e<r&&l(i=this.peek(e));)n+=i,e+=1}return e<r&&u(i=this.peek(e))?null:{type:g.NumericLiteral,value:n,base:s,isMalformed:!isFinite(n)}},scanEscapeSequence:function(t){var e=!1,n=1;this.skip();var r=this.peek();switch(r){case"'":this.triggerAsync("warning",{code:"W114",line:this.line,character:this.char,data:["\\'"]},t,function(){return a.jsonMode});break;case"b":r="\\b";break;case"f":r="\\f";break;case"n":r="\\n";break;case"r":r="\\r";break;case"t":r="\\t";break;case"0":r="\\0";var i=parseInt(this.peek(1),10);this.triggerAsync("warning",{code:"W115",line:this.line,character:this.char},t,function(){return i>=0&&i<=7&&a.isStrict()});break;case"1":case"2":case"3":case"4":case"5":case"6":case"7":r="\\"+r,this.triggerAsync("warning",{code:"W115",line:this.line,character:this.char},t,function(){return a.isStrict()});break;case"u":var o=this.input.substr(1,4),s=parseInt(o,16);d(o)||this.trigger("warning",{code:"W052",line:this.line,character:this.char,data:["u"+o]}),r=String.fromCharCode(s),n=5;break;case"v":this.triggerAsync("warning",{code:"W114",line:this.line,character:this.char,data:["\\v"]},t,function(){return a.jsonMode}),r="\v";break;case"x":var A=parseInt(this.input.substr(1,2),16);this.triggerAsync("warning",{code:"W114",line:this.line,character:this.char,data:["\\x-"]},t,function(){return a.jsonMode}),r=String.fromCharCode(A),n=3;break;case"\\":r="\\\\";break;case'"':r='\\"';break;case"/":break;case"":e=!0,r=""}return{char:r,jump:n,allowNewLine:e}},scanTemplateLiteral:function(t){var e,n,r="",i=this.line,o=this.char,s=this.templateStarts.length;if("`"===this.peek())a.inES6(!0)||this.triggerAsync("warning",{code:"W119",line:this.line,character:this.char,data:["template literal syntax","6"]},t,function(){return!0}),e=g.TemplateHead,this.templateStarts.push({line:this.line,char:this.char}),s=this.templateStarts.length,this.skip(1),this.pushContext(h.Template);else{if(!this.inContext(h.Template)||"}"!==this.peek())return null;e=g.TemplateMiddle}for(;"`"!==this.peek();){for(;""===(n=this.peek());)if(r+="\n",!this.nextLine(t)){var A=this.templateStarts.pop();return this.trigger("error",{code:"E052",line:A.line,character:A.char}),{type:e,value:r,startLine:i,startChar:o,isUnclosed:!0,depth:s,context:this.popContext()}}if("$"===n&&"{"===this.peek(1))return r+="${",this.skip(2),{type:e,value:r,startLine:i,startChar:o,isUnclosed:!1,depth:s,context:this.currentContext()};if("\\"===n){var l=this.scanEscapeSequence(t);r+=l.char,this.skip(l.jump)}else"`"!==n&&(r+=n,this.skip(1))}return e=e===g.TemplateHead?g.NoSubstTemplate:g.TemplateTail,this.skip(1),this.templateStarts.pop(),{type:e,value:r,startLine:i,startChar:o,isUnclosed:!1,depth:s,context:this.popContext()}},scanStringLiteral:function(t){var e=this.peek();if('"'!==e&&"'"!==e)return null;this.triggerAsync("warning",{code:"W108",line:this.line,character:this.char},t,function(){return a.jsonMode&&'"'!==e});var n="",r=this.line,i=this.char,o=!1;for(this.skip();this.peek()!==e;)if(""===this.peek()){if(o?(o=!1,this.triggerAsync("warning",{code:"W043",line:this.line,character:this.char},t,function(){return!a.option.multistr}),this.triggerAsync("warning",{code:"W042",line:this.line,character:this.char},t,function(){return a.jsonMode&&a.option.multistr})):this.trigger("warning",{code:"W112",line:this.line,character:this.char}),!this.nextLine(t))return{type:g.StringLiteral,value:n,startLine:r,startChar:i,isUnclosed:!0,quote:e}}else{o=!1;var s=this.peek(),A=1;if(s<" "&&this.triggerAsync("warning",{code:"W113",line:this.line,character:this.char,data:["<non-printable>"]},t,function(){return!0}),"\\"===s){var l=this.scanEscapeSequence(t);s=l.char,A=l.jump,o=l.allowNewLine}""!==s&&(n+=s,this.skip(A))}return this.skip(),{type:g.StringLiteral,value:n,startLine:r,startChar:i,isUnclosed:!1,quote:e}},scanRegExp:function(t){var e,n,r,i=0,s=this.input.length,A=this.peek(),l=A,c="",u=[],h="",f="",p=!1,I=!1,C=!1,m=!1,y=!1,E="",v=function(){return h.indexOf("u")>-1},B=0,b=function(){var e,n;if(i+=1,A=this.peek(i),o.nonzeroDigit.test(A)){for(n=A,e=this.peek(i+1);o.nonzeroDigit.test(e)||"0"===e;)i+=1,n+=A=e,c+=A,l+=A,e=this.peek(i+1);return u.push(Number(n)),n}if(E+=A,"u"===A&&"{"===this.peek(i+1)){var r=i+2;for(n="u{",e=this.peek(r);d(e);)n+=e,r+=1,e=this.peek(r);if("}"!==e)this.triggerAsync("error",{code:"E016",line:this.line,character:this.char,data:["Invalid Unicode escape sequence"]},t,v);else if(n.length>2)return c+=n+="}",l+=n,i=r+1,n}return A<" "&&(p=!0,this.triggerAsync("warning",{code:"W048",line:this.line,character:this.char},t,function(){return!0})),"<"===A&&(p=!0,this.triggerAsync("warning",{code:"W049",line:this.line,character:this.char,data:[A]},t,function(){return!0})),i+=1,c+=A,l+=A,A}.bind(this),Q=function(){var t,e=i,n="",r="";for(t=this.peek(e+1);o.decimalDigit.test(t);)e+=1,n+=t,t=this.peek(e+1);if(!n)return!1;if("}"===t)return!0;if(","!==t)return!1;for(e+=1,t=this.peek(e+1);o.decimalDigit.test(t);)e+=1,r+=t,t=this.peek(e+1);return"}"===t&&(!r||Number(n)<=Number(r))}.bind(this),w=function(t){return t.replace(/\\u\{([0-9a-fA-F]+)\}|\\u([a-fA-F0-9]{4})/g,function(t,e,n){var r,i=parseInt(e||n,16);return i>1114111?(p=!0,void this.trigger("error",{code:"E016",line:this.line,character:this.char,data:[A]})):(r=String.fromCharCode(i),o.regexpSyntaxChars.test(r)?t:i<=65535?String.fromCharCode(i):"")}.bind(this)).replace(/[\uD800-\uDBFF][\uDC00-\uDFFF]/g,"")}.bind(this);if(!this.prereg||"/"!==A)return null;for(i+=1,n=!1;i<s;)if(C&="-"===A,A=this.peek(i),l+=A,c+=A,I&&("]"===A?"\\"===this.peek(i-1)&&"\\"!==this.peek(i-2)||(I=!1):"-"===A&&(C=!0)),"\\"!==A)if(I)i+=1;else if("{"!==A||y||(y=!Q()),"["!==A){if("("===A)!0,"?"!==this.peek(i+1)||"="!==this.peek(i+2)&&"!"!==this.peek(i+2)||(m=!0);else if(")"===A)m?(m=!1,o.regexpQuantifiers.test(this.peek(i+1))&&this.triggerAsync("error",{code:"E016",line:this.line,character:this.char,data:["Quantified quantifiable"]},t,v)):B+=1,!1;else if("/"===A){c=c.substr(0,c.length-1),n=!0,i+=1;break}i+=1}else I=!0,i+=1;else e=b(),I&&("-"===this.peek(i)||C)&&o.regexpCharClasses.test(e)&&this.triggerAsync("error",{code:"E016",line:this.line,character:this.char,data:["Character class used in range"]},t,v);if(!n)return this.trigger("error",{code:"E015",line:this.line,character:this.from}),void this.trigger("fatal",{line:this.line,from:this.from});for(;i<s&&(A=this.peek(i),/[gimyus]/.test(A));){if("y"===A)a.inES6(!0)||this.triggerAsync("warning",{code:"W119",line:this.line,character:this.char,data:["Sticky RegExp flag","6"]},t,function(){return!0});else if("u"===A){a.inES6(!0)||this.triggerAsync("warning",{code:"W119",line:this.line,character:this.char,data:["Unicode RegExp flag","6"]},t,function(){return!0}),function(t,e,n,r){return!!t.some(function(t){if(t>e)return!0})||!n.split("").every(function(t){return"u"===t||"/"===t||r.regexpCharClasses.test(t)||r.regexpSyntaxChars.test(t)})}(u,B,E,o)?r="Invalid escape":y&&(r="Invalid quantifier"),c=w(c)}else"s"===A?(a.inES9()||this.triggerAsync("warning",{code:"W119",line:this.line,character:this.char,data:["DotAll RegExp flag","9"]},t,function(){return!0}),l.indexOf("s")>-1&&(r="Duplicate RegExp flag")):f+=A;h.indexOf(A)>-1&&(r="Duplicate RegExp flag"),h+=A,l+=A,h+=A,i+=1}-1===h.indexOf("u")&&this.triggerAsync("warning",{code:"W147",line:this.line,character:this.char},t,function(){return a.option.regexpu});try{new RegExp(c,f)}catch(t){r=t.message}return r?(p=!0,this.trigger("error",{code:"E016",line:this.line,character:this.char,data:[r]})):h.indexOf("s")>-1&&!o.regexpDot.test(c)&&this.trigger("warning",{code:"W148",line:this.line,character:this.char}),{type:g.RegExp,value:l,isMalformed:p}},scanNonBreakingSpaces:function(){return a.option.nonbsp?this.input.search(/(\u00A0)/):-1},next:function(t){for(this.from=this.char;o.whitespace.test(this.peek());)this.from+=1,this.skip();var e=this.scanComments(t)||this.scanStringLiteral(t)||this.scanTemplateLiteral(t);return e||((e=this.scanRegExp(t)||this.scanPunctuator()||this.scanKeyword()||this.scanIdentifier(t)||this.scanNumericLiteral(t))?(this.skip(e.tokenLength||e.value.length),e):null)},nextLine:function(t){var e;if(this.line>=this.getLines().length)return!1;this.input=this.getLines()[this.line],this.line+=1,this.char=1,this.from=1;var n=this.input.trim(),i=function(){return r.some(arguments,function(t){return 0===n.indexOf(t)})};(!0===this.ignoringLinterErrors&&(i("/*","//")||this.inComment&&function(){return r.some(arguments,function(t){return-1!==n.indexOf(t,n.length-t.length)})}("*/")||(this.input="")),(e=this.scanNonBreakingSpaces())>=0&&this.triggerAsync("warning",{code:"W125",line:this.line,character:e+1},t,function(){return!0}),this.input=this.input.replace(/\t/g,a.tab),!this.ignoringLinterErrors&&a.option.maxlen&&a.option.maxlen<this.input.length)&&((!(this.inComment||i.call(n,"//")||i.call(n,"/*"))||!o.maxlenException.test(n))&&this.triggerAsync("warning",{code:"W101",line:this.line,character:this.input.length},t,function(){return!0}));return!0},token:function(){for(var t,e,n=(t=[],{push:function(e){t.push(e)},check:function(){for(var e=0;e<t.length;++e)t[e]();t.splice(0,t.length)}}),i=function(t,e,i,o){var s;if("(endline)"!==t&&"(end)"!==t&&(this.prereg=!1),"(punctuator)"===t){switch(e){case".":case")":case"~":case"#":case"]":case"}":case"++":case"--":this.prereg=!1;break;default:this.prereg=!0}s=Object.create(a.syntax[e]||a.syntax["(error)"])}return"(identifier)"===t&&("return"!==e&&"case"!==e&&"yield"!==e&&"typeof"!==e&&"instanceof"!==e&&"void"!==e&&"await"!==e||(this.prereg=!0),r.has(a.syntax,e)&&(s=Object.create(a.syntax[e]||a.syntax["(error)"]))),"(template)"!==t&&"(template middle)"!==t||(this.prereg=!0),s||(s=Object.create(a.syntax[t])),s.identifier="(identifier)"===t,s.type=s.type||t,s.value=e,s.line=this.line,s.character=this.char,s.from=this.from,s.identifier&&o&&(s.raw_text=o.text||o.value),o&&o.startLine&&o.startLine!==this.line&&(s.startLine=o.startLine),o&&o.context&&(s.context=o.context),o&&o.depth&&(s.depth=o.depth),o&&o.isUnclosed&&(s.isUnclosed=o.isUnclosed),i&&s.identifier&&(s.isProperty=i),s.check=n.check,s}.bind(this);;){if(!this.input.length)return this.nextLine(n)?i("(endline)",""):this.exhausted?null:(this.exhausted=!0,i("(end)",""));if(e=this.next(n))switch(e.type){case g.StringLiteral:return this.triggerAsync("String",{line:this.line,char:this.char,from:this.from,startLine:e.startLine,startChar:e.startChar,value:e.value,quote:e.quote},n,function(){return!0}),i("(string)",e.value,null,e);case g.TemplateHead:return this.trigger("TemplateHead",{line:this.line,char:this.char,from:this.from,startLine:e.startLine,startChar:e.startChar,value:e.value}),i("(template)",e.value,null,e);case g.TemplateMiddle:return this.trigger("TemplateMiddle",{line:this.line,char:this.char,from:this.from,startLine:e.startLine,startChar:e.startChar,value:e.value}),i("(template middle)",e.value,null,e);case g.TemplateTail:return this.trigger("TemplateTail",{line:this.line,char:this.char,from:this.from,startLine:e.startLine,startChar:e.startChar,value:e.value}),i("(template tail)",e.value,null,e);case g.NoSubstTemplate:return this.trigger("NoSubstTemplate",{line:this.line,char:this.char,from:this.from,startLine:e.startLine,startChar:e.startChar,value:e.value}),i("(no subst template)",e.value,null,e);case g.Identifier:this.triggerAsync("Identifier",{line:this.line,char:this.char,from:this.from,name:e.value,raw_name:e.text,isProperty:"."===a.tokens.curr.id},n,function(){return!0});case g.Keyword:return i("(identifier)",e.value,"."===a.tokens.curr.id,e);case g.NumericLiteral:return e.isMalformed&&this.trigger("warning",{code:"W045",line:this.line,character:this.char,data:[e.value]}),this.triggerAsync("warning",{code:"W114",line:this.line,character:this.char,data:["0x-"]},n,function(){return 16===e.base&&a.jsonMode}),this.triggerAsync("warning",{code:"W115",line:this.line,character:this.char},n,function(){return a.isStrict()&&8===e.base&&e.isLegacy}),this.trigger("Number",{line:this.line,char:this.char,from:this.from,value:e.value,base:e.base,isMalformed:e.isMalformed}),i("(number)",e.value);case g.RegExp:return i("(regexp)",e.value);case g.Comment:if(e.isSpecial)return{id:"(comment)",value:e.value,body:e.body,type:e.commentType,isSpecial:e.isSpecial,line:this.line,character:this.char,from:this.from};break;default:return i("(punctuator)",e.value)}else this.input.length&&(this.trigger("error",{code:"E024",line:this.line,character:this.char,data:[this.peek()]}),this.input="")}}},n.Lexer=p,n.Context=h},{"../data/ascii-identifier-data.js":1,"../data/es5-identifier-names.js":2,"../data/non-ascii-identifier-part-only.js":3,"../data/non-ascii-identifier-start.js":4,"./reg.js":22,"./state.js":24,events:9,lodash:16}],18:[function(t,e,n){"use strict";var r=t("lodash");n.errors={},n.warnings={},n.info={},r.each({E001:"Bad {a}option: '{b}'.",E002:"Bad option value.",E003:"Expected a JSON value.",E004:"Input is neither a string nor an array of strings.",E005:"Input is empty.",E006:"Unexpected early end of program.",E007:'Missing "use strict" statement.',E008:"Strict violation.",E009:"Option 'validthis' can't be used in a global scope.",E010:"'with' is not allowed in strict mode.",E011:"'{a}' has already been declared.",E012:"const '{a}' is initialized to 'undefined'.",E013:"Attempting to override '{a}' which is a constant.",E014:"A regular expression literal can be confused with '/='.",E015:"Unclosed regular expression.",E016:"Invalid regular expression.",E017:"Unclosed comment.",E018:"Unbegun comment.",E019:"Unmatched '{a}'.",E020:"Expected '{a}' to match '{b}' from line {c} and instead saw '{d}'.",E021:"Expected '{a}' and instead saw '{b}'.",E022:"Line breaking error '{a}'.",E023:"Missing '{a}'.",E024:"Unexpected '{a}'.",E025:"Missing ':' on a case clause.",E026:"Missing '}' to match '{' from line {a}.",E027:"Missing ']' to match '[' from line {a}.",E028:"Illegal comma.",E029:"Unclosed string.",E030:"Expected an identifier and instead saw '{a}'.",E031:"Bad assignment.",E032:"Expected a small integer or 'false' and instead saw '{a}'.",E033:"Expected an operator and instead saw '{a}'.",E034:"get/set are ES5 features.",E035:"Missing property name.",E036:"Expected to see a statement and instead saw a block.",E037:null,E038:null,E039:"Function declarations are not invocable. Wrap the whole function invocation in parens.",E040:"Each value should have its own case label.",E041:"Unrecoverable syntax error.",E042:"Stopping.",E043:"Too many errors.",E044:null,E045:"Invalid for each loop.",E046:"Yield expressions may only occur within generator functions.",E047:null,E048:"{a} declaration not directly within block.",E049:"A {a} cannot be named '{b}'.",E050:"Mozilla requires the yield expression to be parenthesized here.",E051:null,E052:"Unclosed template literal.",E053:"{a} declarations are only allowed at the top level of module scope.",E054:"Class properties must be methods. Expected '(' but instead saw '{a}'.",E055:"The '{a}' option cannot be set after any executable code.",E056:"'{a}' was used before it was declared, which is illegal for '{b}' variables.",E057:"Invalid meta property: '{a}.{b}'.",E058:"Missing semicolon.",E059:"Incompatible values for the '{a}' and '{b}' linting options.",E060:"Non-callable values cannot be used as the second operand to instanceof.",E061:"Invalid position for 'yield' expression (consider wrapping in parenthesis).",E062:"Rest parameter does not a support default value.",E063:"Super property may only be used within method bodies.",E064:"Super call may only be used within class method bodies.",E065:"Functions defined outside of strict mode with non-simple parameter lists may not enable strict mode.",E066:"Asynchronous iteration is only available with for-of loops."},function(t,e){n.errors[e]={code:e,desc:t}}),r.each({W001:"'hasOwnProperty' is a really bad name.",W002:"Value of '{a}' may be overwritten in IE 8 and earlier.",W003:"'{a}' was used before it was defined.",W004:"'{a}' is already defined.",W005:"A dot following a number can be confused with a decimal point.",W006:"Confusing minuses.",W007:"Confusing plusses.",W008:"A leading decimal point can be confused with a dot: '{a}'.",W009:"The array literal notation [] is preferable.",W010:"The object literal notation {} is preferable.",W011:null,W012:null,W013:null,W014:"Misleading line break before '{a}'; readers may interpret this as an expression boundary.",W015:null,W016:"Unexpected use of '{a}'.",W017:"Bad operand.",W018:"Confusing use of '{a}'.",W019:"Use the isNaN function to compare with NaN.",W020:"Read only.",W021:"Reassignment of '{a}', which is a {b}. Use 'var' or 'let' to declare bindings that may change.",W022:"Do not assign to the exception parameter.",W023:null,W024:"Expected an identifier and instead saw '{a}' (a reserved word).",W025:"Missing name in function declaration.",W026:"Inner functions should be listed at the top of the outer function.",W027:"Unreachable '{a}' after '{b}'.",W028:"Label '{a}' on {b} statement.",W030:"Expected an assignment or function call and instead saw an expression.",W031:"Do not use 'new' for side effects.",W032:"Unnecessary semicolon.",W033:"Missing semicolon.",W034:'Unnecessary directive "{a}".',W035:"Empty block.",W036:"Unexpected /*member '{a}'.",W037:"'{a}' is a statement label.",W038:"'{a}' used out of scope.",W039:null,W040:"If a strict mode function is executed using function invocation, its 'this' value will be undefined.",W041:null,W042:"Avoid EOL escaping.",W043:"Bad escaping of EOL. Use option multistr if needed.",W044:"Bad or unnecessary escaping.",W045:"Bad number '{a}'.",W046:"Don't use extra leading zeros '{a}'.",W047:"A trailing decimal point can be confused with a dot: '{a}'.",W048:"Unexpected control character in regular expression.",W049:"Unexpected escaped character '{a}' in regular expression.",W050:"JavaScript URL.",W051:"Variables should not be deleted.",W052:"Unexpected '{a}'.",W053:"Do not use {a} as a constructor.",W054:"The Function constructor is a form of eval.",W055:"A constructor name should start with an uppercase letter.",W056:"Bad constructor.",W057:"Weird construction. Is 'new' necessary?",W058:"Missing '()' invoking a constructor.",W059:"Avoid arguments.{a}.",W060:"document.write can be a form of eval.",W061:"eval can be harmful.",W062:"Wrap an immediate function invocation in parens to assist the reader in understanding that the expression is the result of a function, and not the function itself.",W063:"Math is not a function.",W064:"Missing 'new' prefix when invoking a constructor.",W065:"Missing radix parameter.",W066:"Implied eval. Consider passing a function instead of a string.",W067:"Bad invocation.",W068:"Wrapping non-IIFE function literals in parens is unnecessary.",W069:"['{a}'] is better written in dot notation.",W070:"Extra comma. (it breaks older versions of IE)",W071:"This function has too many statements. ({a})",W072:"This function has too many parameters. ({a})",W073:"Blocks are nested too deeply. ({a})",W074:"This function's cyclomatic complexity is too high. ({a})",W075:"Duplicate {a} '{b}'.",W076:"Unexpected parameter '{a}' in get {b} function.",W077:"Expected a single parameter in set {a} function.",W078:"Setter is defined without getter.",W079:"Redefinition of '{a}'.",W080:"It's not necessary to initialize '{a}' to 'undefined'.",W081:null,W082:"Function declarations should not be placed in blocks. Use a function expression or move the statement to the top of the outer function.",W083:"Functions declared within loops referencing an outer scoped variable may lead to confusing semantics. ({a})",W084:"Expected a conditional expression and instead saw an assignment.",W085:"Don't use 'with'.",W086:"Expected a 'break' statement before '{a}'.",W087:"Forgotten 'debugger' statement?",W088:"Creating global 'for' variable. Should be 'for (var {a} ...'.",W089:"The body of a for in should be wrapped in an if statement to filter unwanted properties from the prototype.",W090:"'{a}' is not a statement label.",W091:null,W093:"Did you mean to return a conditional instead of an assignment?",W094:"Unexpected comma.",W095:"Expected a string and instead saw {a}.",W096:"The '{a}' key may produce unexpected results.",W097:'Use the function form of "use strict".',W098:"'{a}' is defined but never used.",W099:null,W100:null,W101:"Line is too long.",W102:null,W103:"The '{a}' property is deprecated.",W104:"'{a}' is available in ES{b} (use 'esversion: {b}') or Mozilla JS extensions (use moz).",W105:null,W106:"Identifier '{a}' is not in camel case.",W107:"Script URL.",W108:"Strings must use doublequote.",W109:"Strings must use singlequote.",W110:"Mixed double and single quotes.",W112:"Unclosed string.",W113:"Control character in string: {a}.",W114:"Avoid {a}.",W115:"Octal literals are not allowed in strict mode.",W116:"Expected '{a}' and instead saw '{b}'.",W117:"'{a}' is not defined.",W118:"'{a}' is only available in Mozilla JavaScript extensions (use moz option).",W119:"'{a}' is only available in ES{b} (use 'esversion: {b}').",W120:"You might be leaking a variable ({a}) here.",W121:"Extending prototype of native object: '{a}'.",W122:"Invalid typeof value '{a}'",W123:"'{a}' is already defined in outer scope.",W124:"A generator function should contain at least one yield expression.",W125:"This line contains non-breaking spaces: http://jshint.com/docs/options/#nonbsp",W126:"Unnecessary grouping operator.",W127:"Unexpected use of a comma operator.",W128:"Empty array elements require elision=true.",W129:"'{a}' is defined in a future version of JavaScript. Use a different variable name to avoid migration issues.",W130:"Invalid element after rest element.",W131:"Invalid parameter after rest parameter.",W132:"`var` declarations are forbidden. Use `let` or `const` instead.",W133:"Invalid for-{a} loop left-hand-side: {b}.",W134:"The '{a}' option is only available when linting ECMAScript {b} code.",W135:"{a} may not be supported by non-browser environments.",W136:"'{a}' must be in function scope.",W137:"Empty destructuring: this is unnecessary and can be removed.",W138:"Regular parameters should not come after default parameters.",W139:"Function expressions should not be used as the second operand to instanceof.",W140:"Missing comma.",W141:"Empty {a}: this is unnecessary and can be removed.",W142:"Empty {a}: consider replacing with `import '{b}';`.",W143:"Assignment to properties of a mapped arguments object may cause unexpected changes to formal parameters.",W144:"'{a}' is a non-standard language feature. Enable it using the '{b}' unstable option.",W145:"Superfluous 'case' clause.",W146:"Unnecessary `await` expression.",W147:"Regular expressions should include the 'u' flag.",W148:"Unnecessary RegExp 's' flag."},function(t,e){n.warnings[e]={code:e,desc:t}}),r.each({I001:"Comma warnings can be turned off with 'laxcomma'.",I002:null,I003:"ES5 option is now set per default"},function(t,e){n.info[e]={code:e,desc:t}})},{lodash:16}],19:[function(t,e,n){"use strict";function r(){this._stack=[]}Object.defineProperty(r.prototype,"length",{get:function(){return this._stack.length}}),r.prototype.push=function(){this._stack.push(null)},r.prototype.pop=function(){this._stack.pop()},r.prototype.set=function(t){this._stack[this.length-1]=t},r.prototype.infer=function(){var t,e=this._stack[this.length-1],n="";return e&&"class"!==e.type||(e=this._stack[this.length-2]),e?"(string)"!==(t=e.type)&&"(number)"!==t&&"(identifier)"!==t&&"default"!==t?"(expression)":(e.accessorType&&(n=e.accessorType+" "),n+e.value):"(empty)"},e.exports=r},{}],20:[function(t,e,n){"use strict";n.bool={enforcing:{bitwise:!0,freeze:!0,camelcase:!0,curly:!0,eqeqeq:!0,futurehostile:!0,es3:!0,es5:!0,forin:!0,immed:!0,leanswitch:!0,newcap:!0,noarg:!0,nocomma:!0,noempty:!0,nonbsp:!0,nonew:!0,noreturnawait:!0,regexpu:!0,undef:!0,singleGroups:!1,varstmt:!1,enforceall:!1,trailingcomma:!1},relaxing:{asi:!0,multistr:!0,debug:!0,boss:!0,evil:!0,funcscope:!0,globalstrict:!0,iterator:!0,notypeof:!0,plusplus:!0,proto:!0,scripturl:!0,sub:!0,supernew:!0,laxbreak:!0,laxcomma:!0,validthis:!0,withstmt:!0,moz:!0,noyield:!0,eqnull:!0,lastsemic:!0,loopfunc:!0,expr:!0,esnext:!0,elision:!0},environments:{mootools:!0,couch:!0,jasmine:!0,jquery:!0,node:!0,qunit:!0,rhino:!0,shelljs:!0,prototypejs:!0,yui:!0,mocha:!0,module:!0,wsh:!0,worker:!0,nonstandard:!0,browser:!0,browserify:!0,devel:!0,dojo:!0,typed:!0,phantom:!0},obsolete:{onecase:!0,regexp:!0,regexdash:!0}},n.val={maxlen:!1,indent:!1,maxerr:!1,predef:!1,globals:!1,quotmark:!1,scope:!1,maxstatements:!1,maxdepth:!1,maxparams:!1,maxcomplexity:!1,shadow:!1,strict:!0,unused:!0,latedef:!1,ignore:!1,ignoreDelimiters:!1,esversion:5},n.unstable={},n.inverted={bitwise:!0,forin:!0,newcap:!0,plusplus:!0,regexp:!0,undef:!0,eqeqeq:!0,strict:!0},n.validNames=Object.keys(n.val).concat(Object.keys(n.bool.relaxing)).concat(Object.keys(n.bool.enforcing)).concat(Object.keys(n.bool.obsolete)).concat(Object.keys(n.bool.environments)).concat(["unstable"]),n.unstableNames=Object.keys(n.unstable),n.renamed={eqeq:"eqeqeq",windows:"wsh",sloppy:"strict"},n.removed={nomen:!0,onevar:!0,passfail:!0,white:!0,gcl:!0,smarttabs:!0,trailing:!0},n.noenforceall={varstmt:!0,strict:!0,regexpu:!0}},{}],21:[function(t,e,n){e.exports={export:1,noin:2,initial:4,preAsync:8,async:16,tryClause:32}},{}],22:[function(t,e,n){"use strict";n.unsafeString=/@cc|<\/?|script|\]\s*\]|<\s*!|&lt/i,n.needEsc=/[\u0000-\u001f&<"\/\\\u007f-\u009f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/,n.needEscGlobal=/[\u0000-\u001f&<"\/\\\u007f-\u009f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,n.starSlash=/\*\//,n.identifier=/^([a-zA-Z_$][a-zA-Z0-9_$]*)$/,n.javascriptURL=/^(?:javascript|jscript|ecmascript|vbscript|livescript)\s*:/i,n.fallsThrough=/^\s*falls?\sthrough\s*$/,n.maxlenException=/^(?:(?:\/\/|\/\*|\*) ?)?[^ ]+$/,n.whitespace=/[ \f\n\r\t\v\u00a0\u1680\u2000-\u200a\u2028\u2029\u202f\u205f\u3000\ufeff]/,n.nonzeroDigit=/^[1-9]$/,n.decimalDigit=/^[0-9]$/,n.regexpSyntaxChars=/[\^$\\.*+?()[\]{}|]/,n.regexpQuantifiers=/[*+?{]/,n.regexpCharClasses=/[dDsSwW]/,n.regexpDot=/(^|[^\\])(\\\\)*\./},{}],23:[function(t,e,n){"use strict";var r=t("lodash"),i=t("events"),o={};e.exports=function(t,e,n,a){var s,A=[];function l(t){s={"(labels)":Object.create(null),"(usages)":Object.create(null),"(breakLabels)":Object.create(null),"(parent)":s,"(type)":t,"(params)":"functionparams"===t||"catchparams"===t?[]:null},A.push(s)}l("global"),s["(predefined)"]=e;var c=s,u=Object.create(null),g=Object.create(null),h=[],d=new i.EventEmitter;function f(t,e){d.emit("warning",{code:t,token:e,data:r.slice(arguments,2)})}function p(t,e){d.emit("warning",{code:t,token:e,data:r.slice(arguments,2)})}function I(t){s["(usages)"][t]||(s["(usages)"][t]={"(modified)":[],"(reassigned)":[],"(tokens)":[]})}var C=function(e){return void 0===e&&(e=t.option.unused),!0===e&&(e="last-param"),e},m=function(t,e,n,r){var i=e.line,o=e.from,a=e.raw_text||t,s={vars:["var"],"last-param":["var","param"],strict:["var","param","last-param"]};(r=C(r))&&s[r]&&-1!==s[r].indexOf(n)&&f("W098",{line:i,from:o},a),(r||"var"===n)&&h.push({name:t,line:i,character:o})};function y(){if("functionparams"!==s["(type)"]){var e=s["(labels)"];for(var n in e)"exception"!==e[n]["(type)"]&&e[n]["(unused)"]&&m(n,e[n]["(token)"],"var")}else!function(){var e=s["(params)"];if(e)for(var n,r=e.pop();r;){var i=s["(labels)"][r];if(n=C(t.funct["(unusedOption)"]),"undefined"===r)return;if(i["(unused)"])m(r,i["(token)"],"param",t.funct["(unusedOption)"]);else if("last-param"===n)return;r=e.pop()}}()}function E(t){for(var e=A.length-1;e>=0;--e){var n=A[e]["(labels)"];if(n[t])return n}}function v(e,n){if("outer"===t.option.shadow)for(var r="global"===c["(type)"],i="functionparams"===s["(type)"],o=!r,a=0;a<A.length;a++){var l=A[a];i||A[a+1]!==c||(o=!1),o&&l["(labels)"][e]&&f("W123",n,e),l["(breakLabels)"][e]&&f("W123",n,e)}}function B(e,n,r){var i;t.option.latedef&&(i="function"===e||"generator function"===e||"async function"===e,(!0===t.option.latedef&&i||!i)&&f("W003",r,n))}var b={on:function(t,e){t.split(" ").forEach(function(t){d.on(t,e)})},isPredefined:function(t){return!this.has(t)&&r.has(A[0]["(predefined)"],t)},stack:function(t){var e=s;l(t),t||"functionparams"!==e["(type)"]||(s["(isFuncBody)"]=!0,c=s)},unstack:function(){var e,n,i,l=A.length>1?A[A.length-2]:null,h=s===c,d="functionparams"===s["(type)"],I="functionouter"===s["(type)"],C=s["(usages)"],E=s["(labels)"],v=Object.keys(C);for(C.__proto__&&-1===v.indexOf("__proto__")&&v.push("__proto__"),e=0;e<v.length;e++){var B=v[e],b=C[B],Q=E[B];if(Q){var w=Q["(type)"];if(i="const"===w||"import"===w,Q["(useOutsideOfScope)"]&&!t.option.funcscope){var x=b["(tokens)"];for(n=0;n<x.length;n++)Q["(function)"]===x[n]["(function)"]&&p("W038",x[n],B)}if(s["(labels)"][B]["(unused)"]=!1,i&&b["(modified)"])for(n=0;n<b["(modified)"].length;n++)p("E013",b["(modified)"][n],B);if(("function"===w||"generator function"===w||"async function"===w||"class"===w)&&b["(reassigned)"])for(n=0;n<b["(reassigned)"].length;n++)b["(reassigned)"][n].ignoreW021||f("W021",b["(reassigned)"][n],B,w)}else if(l){var _=this.labeltype(B);if(i="const"===_||null===_&&!1===A[0]["(predefined)"][B],I&&!i&&(t.funct["(outerMutables)"]||(t.funct["(outerMutables)"]=[]),t.funct["(outerMutables)"].push(B)),l["(usages)"][B]){var D=l["(usages)"][B];D["(modified)"]=D["(modified)"].concat(b["(modified)"]),D["(tokens)"]=D["(tokens)"].concat(b["(tokens)"]),D["(reassigned)"]=D["(reassigned)"].concat(b["(reassigned)"])}else l["(usages)"][B]=b,h&&(l["(usages)"][B]["(onlyUsedSubFunction)"]=!0)}else if("boolean"==typeof s["(predefined)"][B]){if(delete a[B],u[B]=o,!1===s["(predefined)"][B]&&b["(reassigned)"])for(n=0;n<b["(reassigned)"].length;n++)b["(reassigned)"][n].ignoreW020||f("W020",b["(reassigned)"][n])}else for(n=0;n<b["(tokens)"].length;n++){var M=b["(tokens)"][n];M.forgiveUndef||(t.option.undef&&!M.ignoreUndef&&f("W117",M,B),g[B]?g[B].line.push(M.line):g[B]={name:B,line:[M.line]})}}if(l||Object.keys(a).forEach(function(t){m(t,a[t],"var")}),l&&!h&&!d&&!I){var k=Object.keys(E);for(e=0;e<k.length;e++){var S=k[e],L=E[S];if(!L["(blockscoped)"]&&"exception"!==L["(type)"]){var R=l["(labels)"][S];R?R["(unused)"]&=L["(unused)"]:(L["(useOutsideOfScope)"]="global"!==c["(type)"]&&!this.funct.has(S,{excludeCurrent:!0}),l["(labels)"][S]=L),delete E[S]}}}y(),A.pop(),h&&(c=A[r.findLastIndex(A,function(t){return t["(isFuncBody)"]||"global"===t["(type)"]})]),s=l},addParam:function(e,n,i){if("exception"===(i=i||"param")){var o=this.funct.labeltype(e);o&&"exception"!==o&&(t.option.node||f("W002",t.tokens.next,e)),!t.isStrict()||"arguments"!==e&&"eval"!==e||f("E008",n)}r.has(s["(labels)"],e)?s["(labels)"][e].duplicated=!0:(v(e,n),s["(labels)"][e]={"(type)":i,"(token)":n,"(unused)":!0},s["(params)"].push(e)),r.has(s["(usages)"],e)&&(s["(usages)"][e]["(onlyUsedSubFunction)"]?B(i,e,n):f("E056",n,e,i))},validateParams:function(e){var n=t.isStrict(),r=c["(parent)"],i=t.funct["(hasSimpleParams)"],o=t.funct["(method)"];r["(params)"]&&r["(params)"].forEach(function(a){var s=r["(labels)"][a];s.duplicated&&(n||e||o||!i?f("E011",s["(token)"],a):!0!==t.option.shadow&&f("W004",s["(token)"],a)),!n||"arguments"!==a&&"eval"!==a||f("E008",s["(token)"])})},getUsedOrDefinedGlobals:function(){var t=Object.keys(u);return u.__proto__===o&&-1===t.indexOf("__proto__")&&t.push("__proto__"),t},getImpliedGlobals:function(){var t=r.values(g);return g.__proto__&&(t.some(function(t){return"__proto__"===t.name})||t.push(g.__proto__)),t},getUnuseds:function(){return h},has:function(t){return Boolean(E(t))},labeltype:function(t){var e=E(t);return e?e[t]["(type)"]:null},addExported:function(t){var e=A[0]["(labels)"];if(r.has(a,t))delete a[t];else if(r.has(e,t))e[t]["(unused)"]=!1;else{for(var i=1;i<A.length;i++){var o=A[i];if(o["(type)"])break;if(r.has(o["(labels)"],t)&&!o["(labels)"][t]["(blockscoped)"])return void(o["(labels)"][t]["(unused)"]=!1)}n[t]=!0}},setExported:function(t,e){this.block.use(t,e)},initialize:function(t){s["(labels)"][t]&&(s["(labels)"][t]["(initialized)"]=!0)},addlabel:function(e,i){var a=i.type,l=i.token,g="let"===a||"const"===a||"class"===a||"import"===a||"generator function"===a||"async function"===a||"async generator function"===a,h="function"===a||"generator function"===a||"async function"===a||"import"===a,d="global"===(g?s:c)["(type)"]&&r.has(n,e);if(v(e,l),!t.isStrict()||"arguments"!==e&&"eval"!==e||f("E008",l),g){var p=s["(labels)"][e];p||s!==c||"global"===s["(type)"]||(p=!!c["(parent)"]["(labels)"][e]),!p&&s["(usages)"][e]&&(s["(usages)"][e]["(onlyUsedSubFunction)"]||h?B(a,e,l):h||f("E056",l,e,a)),!p||h&&"global"===s["(type)"]&&"import"!==a?"outer"===t.option.shadow&&b.funct.has(e)&&f("W004",l,e):f("E011",l,e),b.block.add(e,a,l,!d,i.initialized)}else{var I=b.funct.has(e);!I&&function(t){for(var e=A.length-1;e>=0;e--){var n=A[e];if(n["(usages)"][t])return n["(usages)"][t];if(n===c)break}return!1}(e)&&B(a,e,l),b.funct.has(e,{onlyBlockscoped:!0})?f("E011",l,e):!0!==t.option.shadow&&I&&"__proto__"!==e&&"global"!==c["(type)"]&&f("W004",l,e),b.funct.add(e,a,l,!d),"global"!==c["(type)"]||t.impliedClosure()||(u[e]=o)}},funct:{labeltype:function(t,e){for(var n=e&&e.onlyBlockscoped,r=e&&e.excludeParams,i=A.length-(e&&e.excludeCurrent?2:1);i>=0;i--){var o=A[i];if(o["(labels)"][t]&&(!n||o["(labels)"][t]["(blockscoped)"]))return o["(labels)"][t]["(type)"];var a=r?A[i-1]:o;if(a&&"functionparams"===a["(type)"])return null}return null},hasBreakLabel:function(t){for(var e=A.length-1;e>=0;e--){var n=A[e];if(n["(breakLabels)"][t])return!0;if("functionparams"===n["(type)"])return!1}return!1},has:function(t,e){return Boolean(this.labeltype(t,e))},add:function(t,e,n,r){s["(labels)"][t]={"(type)":e,"(token)":n,"(blockscoped)":!1,"(function)":c,"(unused)":r}}},block:{isGlobal:function(){return"global"===s["(type)"]},use:function(e,n){var r=c["(parent)"];r&&r["(labels)"][e]&&"param"===r["(labels)"][e]["(type)"]&&(b.funct.has(e,{excludeParams:!0,onlyBlockscoped:!0})||(r["(labels)"][e]["(unused)"]=!1)),n&&(t.ignored.W117||!1===t.option.undef)&&(n.ignoreUndef=!0),I(e),s["(usages)"][e]["(onlyUsedSubFunction)"]=!1,n&&(n["(function)"]=c,s["(usages)"][e]["(tokens)"].push(n));var i=s["(labels)"][e];i&&i["(blockscoped)"]&&!i["(initialized)"]&&p("E056",n,e,i["(type)"])},reassign:function(e,n){n.ignoreW020=t.ignored.W020,n.ignoreW021=t.ignored.W021,this.modify(e,n),s["(usages)"][e]["(reassigned)"].push(n)},modify:function(t,e){I(t),s["(usages)"][t]["(onlyUsedSubFunction)"]=!1,s["(usages)"][t]["(modified)"].push(e)},add:function(t,e,n,r,i){s["(labels)"][t]={"(type)":e,"(token)":n,"(initialized)":!!i,"(blockscoped)":!0,"(unused)":r}},addBreakLabel:function(e,n){var r=n.token;b.funct.hasBreakLabel(e)?f("E011",r,e):"outer"===t.option.shadow&&(b.funct.has(e)?f("W004",r,e):v(e,r)),s["(breakLabels)"][e]=r}}};return b}},{events:9,lodash:16}],24:[function(t,e,n){"use strict";var r=t("./name-stack.js"),i={syntax:{},isStrict:function(){return this.directive["use strict"]||this.inClassBody||this.option.module||"implied"===this.option.strict},stmtMissingStrict:function(){return"global"===this.option.strict||!1!==this.option.strict&&!!this.option.globalstrict},allowsGlobalUsd:function(){return"global"===this.option.strict||this.option.globalstrict||this.option.module||this.impliedClosure()},impliedClosure:function(){return this.option.node||this.option.phantom||this.option.browserify},inMoz:function(){return this.option.moz},inES9:function(){return this.esVersion>=9},inES8:function(){return this.esVersion>=8},inES7:function(){return this.esVersion>=7},inES6:function(t){return!(t||!this.option.moz)||this.esVersion>=6},inES5:function(){return!this.esVersion||this.esVersion>=5||this.option.moz},inferEsVersion:function(){var t=null;if(this.option.esversion){if(this.option.es3?t="es3":this.option.es5?t="es5":this.option.esnext&&(t="esnext"),t)return t;2015===this.option.esversion?this.esVersion=6:this.esVersion=this.option.esversion}else this.option.es3?this.esVersion=3:this.option.esnext&&(this.esVersion=6);return null},reset:function(){this.tokens={prev:null,next:null,curr:null},this.option={unstable:{}},this.esVersion=5,this.funct=null,this.ignored={},this.directive=Object.create(null),this.jsonMode=!1,this.lines=[],this.tab="",this.cache={},this.ignoredLines={},this.forinifcheckneeded=!1,this.nameStack=new r,this.inClassBody=!1}};n.state=i},{"./name-stack.js":19}],25:[function(t,e,n){"use strict";n.register=function(t){t.on("Identifier",function(e){t.getOption("proto")||"__proto__"===e.name&&t.warn("W103",{line:e.line,char:e.char,data:[e.name,"6"]})}),t.on("Identifier",function(e){t.getOption("iterator")||"__iterator__"===e.name&&t.warn("W103",{line:e.line,char:e.char,data:[e.name]})}),t.on("Identifier",function(e){t.getOption("camelcase")&&e.name.replace(/^_+|_+$/g,"").indexOf("_")>-1&&!e.name.match(/^[A-Z0-9_]*$/)&&t.warn("W106",{line:e.line,char:e.char,data:[e.name]})}),t.on("String",function(e){var n,r=t.getOption("quotmark");r&&("single"===r&&"'"!==e.quote&&(n="W109"),"double"===r&&'"'!==e.quote&&(n="W108"),!0===r&&(t.getCache("quotmark")||t.setCache("quotmark",e.quote),t.getCache("quotmark")!==e.quote&&(n="W110")),n&&t.warn(n,{line:e.line,char:e.char}))}),t.on("Number",function(e){"."===e.value.charAt(0)&&t.warn("W008",{line:e.line,char:e.char,data:[e.value]}),"."===e.value.substr(e.value.length-1)&&t.warn("W047",{line:e.line,char:e.char,data:[e.value]}),/^00+/.test(e.value)&&t.warn("W046",{line:e.line,char:e.char,data:[e.value]})}),t.on("String",function(e){t.getOption("scripturl")||/^(?:javascript|jscript|ecmascript|vbscript|livescript)\s*:/i.test(e.value)&&t.warn("W107",{line:e.line,char:e.char})})}},{}],26:[function(t,e,n){"use strict";n.reservedVars={NaN:!1,undefined:!1},n.ecmaIdentifiers={3:{Array:!1,Boolean:!1,Date:!1,decodeURI:!1,decodeURIComponent:!1,encodeURI:!1,encodeURIComponent:!1,Error:!1,eval:!1,EvalError:!1,Function:!1,hasOwnProperty:!1,Infinity:!1,isFinite:!1,isNaN:!1,Math:!1,Number:!1,Object:!1,parseInt:!1,parseFloat:!1,RangeError:!1,ReferenceError:!1,RegExp:!1,String:!1,SyntaxError:!1,TypeError:!1,URIError:!1},5:{JSON:!1},6:{ArrayBuffer:!1,DataView:!1,Float32Array:!1,Float64Array:!1,Int8Array:!1,Int16Array:!1,Int32Array:!1,Map:!1,Promise:!1,Proxy:!1,Reflect:!1,Set:!1,Symbol:!1,Uint8Array:!1,Uint16Array:!1,Uint32Array:!1,Uint8ClampedArray:!1,WeakMap:!1,WeakSet:!1},8:{Atomics:!1,SharedArrayBuffer:!1}},n.browser={Audio:!1,Blob:!1,addEventListener:!1,applicationCache:!1,atob:!1,blur:!1,btoa:!1,cancelAnimationFrame:!1,CanvasGradient:!1,CanvasPattern:!1,CanvasRenderingContext2D:!1,CSS:!1,CSSImportRule:!1,CSSGroupingRule:!1,CSSMarginRule:!1,CSSMediaRule:!1,CSSNamespaceRule:!1,CSSPageRule:!1,CSSRule:!1,CSSRuleList:!1,CSSStyleDeclaration:!1,CSSStyleRule:!1,CSSStyleSheet:!1,clearInterval:!1,clearTimeout:!1,close:!1,closed:!1,Comment:!1,CompositionEvent:!1,createImageBitmap:!1,CustomEvent:!1,DOMParser:!1,defaultStatus:!1,dispatchEvent:!1,Document:!1,document:!1,DocumentFragment:!1,Element:!1,ElementTimeControl:!1,Event:!1,event:!1,fetch:!1,File:!1,FileList:!1,FileReader:!1,FormData:!1,focus:!1,frames:!1,getComputedStyle:!1,Headers:!1,HTMLAnchorElement:!1,HTMLAreaElement:!1,HTMLAudioElement:!1,HTMLBaseElement:!1,HTMLBlockquoteElement:!1,HTMLBodyElement:!1,HTMLBRElement:!1,HTMLButtonElement:!1,HTMLCanvasElement:!1,HTMLCollection:!1,HTMLDataElement:!1,HTMLDataListElement:!1,HTMLDetailsElement:!1,HTMLDialogElement:!1,HTMLDirectoryElement:!1,HTMLDivElement:!1,HTMLDListElement:!1,HTMLElement:!1,HTMLEmbedElement:!1,HTMLFieldSetElement:!1,HTMLFontElement:!1,HTMLFormElement:!1,HTMLFrameElement:!1,HTMLFrameSetElement:!1,HTMLHeadElement:!1,HTMLHeadingElement:!1,HTMLHRElement:!1,HTMLHtmlElement:!1,HTMLIFrameElement:!1,HTMLImageElement:!1,HTMLInputElement:!1,HTMLIsIndexElement:!1,HTMLLabelElement:!1,HTMLLayerElement:!1,HTMLLegendElement:!1,HTMLLIElement:!1,HTMLLinkElement:!1,HTMLMapElement:!1,HTMLMarqueeElement:!1,HTMLMediaElement:!1,HTMLMenuElement:!1,HTMLMetaElement:!1,HTMLMeterElement:!1,HTMLModElement:!1,HTMLObjectElement:!1,HTMLOListElement:!1,HTMLOptGroupElement:!1,HTMLOptionElement:!1,HTMLParagraphElement:!1,HTMLParamElement:!1,HTMLPictureElement:!1,HTMLPreElement:!1,HTMLProgressElement:!1,HTMLQuoteElement:!1,HTMLScriptElement:!1,HTMLSelectElement:!1,HTMLSlotElement:!1,HTMLSourceElement:!1,HTMLStyleElement:!1,HTMLTableCaptionElement:!1,HTMLTableCellElement:!1,HTMLTableColElement:!1,HTMLTableElement:!1,HTMLTableRowElement:!1,HTMLTableSectionElement:!1,HTMLTemplateElement:!1,HTMLTextAreaElement:!1,HTMLTimeElement:!1,HTMLTitleElement:!1,HTMLTrackElement:!1,HTMLUListElement:!1,HTMLVideoElement:!1,history:!1,Image:!1,IntersectionObserver:!1,Intl:!1,length:!1,localStorage:!1,location:!1,matchMedia:!1,MediaList:!1,MediaRecorder:!1,MessageChannel:!1,MessageEvent:!1,MessagePort:!1,MouseEvent:!1,moveBy:!1,moveTo:!1,MutationObserver:!1,name:!1,Node:!1,NodeFilter:!1,NodeList:!1,Notification:!1,navigator:!1,onbeforeunload:!0,onblur:!0,onerror:!0,onfocus:!0,onload:!0,onresize:!0,onunload:!0,open:!1,openDatabase:!1,opener:!1,Option:!1,origin:!1,parent:!1,performance:!1,print:!1,queueMicrotask:!1,Range:!1,requestAnimationFrame:!1,removeEventListener:!1,Request:!1,resizeBy:!1,resizeTo:!1,Response:!1,screen:!1,scroll:!1,scrollBy:!1,scrollTo:!1,sessionStorage:!1,setInterval:!1,setTimeout:!1,SharedWorker:!1,status:!1,Storage:!1,StyleSheet:!1,SVGAElement:!1,SVGAltGlyphDefElement:!1,SVGAltGlyphElement:!1,SVGAltGlyphItemElement:!1,SVGAngle:!1,SVGAnimateColorElement:!1,SVGAnimateElement:!1,SVGAnimateMotionElement:!1,SVGAnimateTransformElement:!1,SVGAnimatedAngle:!1,SVGAnimatedBoolean:!1,SVGAnimatedEnumeration:!1,SVGAnimatedInteger:!1,SVGAnimatedLength:!1,SVGAnimatedLengthList:!1,SVGAnimatedNumber:!1,SVGAnimatedNumberList:!1,SVGAnimatedPathData:!1,SVGAnimatedPoints:!1,SVGAnimatedPreserveAspectRatio:!1,SVGAnimatedRect:!1,SVGAnimatedString:!1,SVGAnimatedTransformList:!1,SVGAnimationElement:!1,SVGCSSRule:!1,SVGCircleElement:!1,SVGClipPathElement:!1,SVGColor:!1,SVGColorProfileElement:!1,SVGColorProfileRule:!1,SVGComponentTransferFunctionElement:!1,SVGCursorElement:!1,SVGDefsElement:!1,SVGDescElement:!1,SVGDocument:!1,SVGElement:!1,SVGElementInstance:!1,SVGElementInstanceList:!1,SVGEllipseElement:!1,SVGExternalResourcesRequired:!1,SVGFEBlendElement:!1,SVGFEColorMatrixElement:!1,SVGFEComponentTransferElement:!1,SVGFECompositeElement:!1,SVGFEConvolveMatrixElement:!1,SVGFEDiffuseLightingElement:!1,SVGFEDisplacementMapElement:!1,SVGFEDistantLightElement:!1,SVGFEFloodElement:!1,SVGFEFuncAElement:!1,SVGFEFuncBElement:!1,SVGFEFuncGElement:!1,SVGFEFuncRElement:!1,SVGFEGaussianBlurElement:!1,SVGFEImageElement:!1,SVGFEMergeElement:!1,SVGFEMergeNodeElement:!1,SVGFEMorphologyElement:!1,SVGFEOffsetElement:!1,SVGFEPointLightElement:!1,SVGFESpecularLightingElement:!1,SVGFESpotLightElement:!1,SVGFETileElement:!1,SVGFETurbulenceElement:!1,SVGFilterElement:!1,SVGFilterPrimitiveStandardAttributes:!1,SVGFitToViewBox:!1,SVGFontElement:!1,SVGFontFaceElement:!1,SVGFontFaceFormatElement:!1,SVGFontFaceNameElement:!1,SVGFontFaceSrcElement:!1,SVGFontFaceUriElement:!1,SVGForeignObjectElement:!1,SVGGElement:!1,SVGGlyphElement:!1,SVGGlyphRefElement:!1,SVGGradientElement:!1,SVGHKernElement:!1,SVGICCColor:!1,SVGImageElement:!1,SVGLangSpace:!1,SVGLength:!1,SVGLengthList:!1,SVGLineElement:!1,SVGLinearGradientElement:!1,SVGLocatable:!1,SVGMPathElement:!1,SVGMarkerElement:!1,SVGMaskElement:!1,SVGMatrix:!1,SVGMetadataElement:!1,SVGMissingGlyphElement:!1,SVGNumber:!1,SVGNumberList:!1,SVGPaint:!1,SVGPathElement:!1,SVGPathSeg:!1,SVGPathSegArcAbs:!1,SVGPathSegArcRel:!1,SVGPathSegClosePath:!1,SVGPathSegCurvetoCubicAbs:!1,SVGPathSegCurvetoCubicRel:!1,SVGPathSegCurvetoCubicSmoothAbs:!1,SVGPathSegCurvetoCubicSmoothRel:!1,SVGPathSegCurvetoQuadraticAbs:!1,SVGPathSegCurvetoQuadraticRel:!1,SVGPathSegCurvetoQuadraticSmoothAbs:!1,SVGPathSegCurvetoQuadraticSmoothRel:!1,SVGPathSegLinetoAbs:!1,SVGPathSegLinetoHorizontalAbs:!1,SVGPathSegLinetoHorizontalRel:!1,SVGPathSegLinetoRel:!1,SVGPathSegLinetoVerticalAbs:!1,SVGPathSegLinetoVerticalRel:!1,SVGPathSegList:!1,SVGPathSegMovetoAbs:!1,SVGPathSegMovetoRel:!1,SVGPatternElement:!1,SVGPoint:!1,SVGPointList:!1,SVGPolygonElement:!1,SVGPolylineElement:!1,SVGPreserveAspectRatio:!1,SVGRadialGradientElement:!1,SVGRect:!1,SVGRectElement:!1,SVGRenderingIntent:!1,SVGSVGElement:!1,SVGScriptElement:!1,SVGSetElement:!1,SVGStopElement:!1,SVGStringList:!1,SVGStylable:!1,SVGStyleElement:!1,SVGSwitchElement:!1,SVGSymbolElement:!1,SVGTRefElement:!1,SVGTSpanElement:!1,SVGTests:!1,SVGTextContentElement:!1,SVGTextElement:!1,SVGTextPathElement:!1,SVGTextPositioningElement:!1,SVGTitleElement:!1,SVGTransform:!1,SVGTransformList:!1,SVGTransformable:!1,SVGURIReference:!1,SVGUnitTypes:!1,SVGUseElement:!1,SVGVKernElement:!1,SVGViewElement:!1,SVGViewSpec:!1,SVGZoomAndPan:!1,Text:!1,TextDecoder:!1,TextEncoder:!1,TimeEvent:!1,top:!1,URL:!1,WebGLActiveInfo:!1,WebGLBuffer:!1,WebGLContextEvent:!1,WebGLFramebuffer:!1,WebGLProgram:!1,WebGLRenderbuffer:!1,WebGLRenderingContext:!1,WebGLShader:!1,WebGLShaderPrecisionFormat:!1,WebGLTexture:!1,WebGLUniformLocation:!1,WebSocket:!1,window:!1,Window:!1,Worker:!1,XDomainRequest:!1,XMLDocument:!1,XMLHttpRequest:!1,XMLSerializer:!1,XPathEvaluator:!1,XPathException:!1,XPathExpression:!1,XPathNamespace:!1,XPathNSResolver:!1,XPathResult:!1},n.devel={alert:!1,confirm:!1,console:!1,Debug:!1,opera:!1,prompt:!1},n.worker={addEventListener:!0,atob:!0,btoa:!0,clearInterval:!0,clearTimeout:!0,createImageBitmap:!0,dispatchEvent:!0,importScripts:!0,onmessage:!0,origin:!0,postMessage:!0,queueMicrotask:!0,removeEventListener:!0,self:!0,setInterval:!0,setTimeout:!0,FileReaderSync:!0},n.nonstandard={escape:!1,unescape:!1},n.couch={require:!1,respond:!1,getRow:!1,emit:!1,send:!1,start:!1,sum:!1,log:!1,exports:!1,module:!1,provides:!1},n.node={__filename:!1,__dirname:!1,arguments:!1,GLOBAL:!1,global:!1,module:!1,require:!1,Buffer:!0,console:!0,exports:!0,process:!0,setTimeout:!0,clearTimeout:!0,setInterval:!0,clearInterval:!0,setImmediate:!0,clearImmediate:!0},n.browserify={__filename:!1,__dirname:!1,global:!1,module:!1,require:!1,Buffer:!0,exports:!0,process:!0},n.phantom={phantom:!0,require:!0,WebPage:!0,console:!0,exports:!0},n.qunit={asyncTest:!1,deepEqual:!1,equal:!1,expect:!1,module:!1,notDeepEqual:!1,notEqual:!1,notOk:!1,notPropEqual:!1,notStrictEqual:!1,ok:!1,propEqual:!1,QUnit:!1,raises:!1,start:!1,stop:!1,strictEqual:!1,test:!1,throws:!1},n.rhino={arguments:!1,defineClass:!1,deserialize:!1,gc:!1,help:!1,importClass:!1,importPackage:!1,java:!1,load:!1,loadClass:!1,Packages:!1,print:!1,quit:!1,readFile:!1,readUrl:!1,runCommand:!1,seal:!1,serialize:!1,spawn:!1,sync:!1,toint32:!1,version:!1},n.shelljs={target:!1,echo:!1,exit:!1,cd:!1,pwd:!1,ls:!1,find:!1,cp:!1,rm:!1,mv:!1,mkdir:!1,test:!1,cat:!1,sed:!1,grep:!1,which:!1,dirs:!1,pushd:!1,popd:!1,env:!1,exec:!1,chmod:!1,config:!1,error:!1,tempdir:!1},n.typed={ArrayBuffer:!1,ArrayBufferView:!1,DataView:!1,Float32Array:!1,Float64Array:!1,Int16Array:!1,Int32Array:!1,Int8Array:!1,Uint16Array:!1,Uint32Array:!1,Uint8Array:!1,Uint8ClampedArray:!1},n.wsh={ActiveXObject:!0,Enumerator:!0,GetObject:!0,ScriptEngine:!0,ScriptEngineBuildVersion:!0,ScriptEngineMajorVersion:!0,ScriptEngineMinorVersion:!0,VBArray:!0,WSH:!0,WScript:!0,XDomainRequest:!0},n.dojo={dojo:!1,dijit:!1,dojox:!1,define:!1,require:!1},n.jquery={$:!1,jQuery:!1},n.mootools={$:!1,$$:!1,Asset:!1,Browser:!1,Chain:!1,Class:!1,Color:!1,Cookie:!1,Core:!1,Document:!1,DomReady:!1,DOMEvent:!1,DOMReady:!1,Drag:!1,Element:!1,Elements:!1,Event:!1,Events:!1,Fx:!1,Group:!1,Hash:!1,HtmlTable:!1,IFrame:!1,IframeShim:!1,InputValidator:!1,instanceOf:!1,Keyboard:!1,Locale:!1,Mask:!1,MooTools:!1,Native:!1,Options:!1,OverText:!1,Request:!1,Scroller:!1,Slick:!1,Slider:!1,Sortables:!1,Spinner:!1,Swiff:!1,Tips:!1,Type:!1,typeOf:!1,URI:!1,Window:!1},n.prototypejs={$:!1,$$:!1,$A:!1,$F:!1,$H:!1,$R:!1,$break:!1,$continue:!1,$w:!1,Abstract:!1,Ajax:!1,Class:!1,Enumerable:!1,Element:!1,Event:!1,Field:!1,Form:!1,Hash:!1,Insertion:!1,ObjectRange:!1,PeriodicalExecuter:!1,Position:!1,Prototype:!1,Selector:!1,Template:!1,Toggle:!1,Try:!1,Autocompleter:!1,Builder:!1,Control:!1,Draggable:!1,Draggables:!1,Droppables:!1,Effect:!1,Sortable:!1,SortableObserver:!1,Sound:!1,Scriptaculous:!1},n.yui={YUI:!1,Y:!1,YUI_config:!1},n.mocha={mocha:!1,describe:!1,xdescribe:!1,it:!1,xit:!1,context:!1,xcontext:!1,before:!1,after:!1,beforeEach:!1,afterEach:!1,suite:!1,test:!1,setup:!1,teardown:!1,suiteSetup:!1,suiteTeardown:!1},n.jasmine={jasmine:!1,describe:!1,xdescribe:!1,it:!1,xit:!1,beforeEach:!1,afterEach:!1,setFixtures:!1,loadFixtures:!1,spyOn:!1,expect:!1,runs:!1,waitsFor:!1,waits:!1,beforeAll:!1,afterAll:!1,fail:!1,fdescribe:!1,fit:!1,pending:!1,spyOnProperty:!1}},{}],jshint:[function(t,e,n){
/*!
 * JSHint, by JSHint Community.
 *
 * This file (and this file only) is licensed under the same slightly modified
 * MIT license that JSLint is. It stops evil-doers everywhere:
 *
 *   Copyright (c) 2002 Douglas Crockford  (www.JSLint.com)
 *
 *   Permission is hereby granted, free of charge, to any person obtaining
 *   a copy of this software and associated documentation files (the "Software"),
 *   to deal in the Software without restriction, including without limitation
 *   the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *   and/or sell copies of the Software, and to permit persons to whom
 *   the Software is furnished to do so, subject to the following conditions:
 *
 *   The above copyright notice and this permission notice shall be included
 *   in all copies or substantial portions of the Software.
 *
 *   The Software shall be used for Good, not Evil.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *   DEALINGS IN THE SOFTWARE.
 *
 */
var r=t("lodash"),i=t("events"),o=t("./vars.js"),a=t("./messages.js"),s=t("./lex.js").Lexer,A=t("./reg.js"),l=t("./state.js").state,c=t("./style.js"),u=t("./options.js"),g=t("./scope-manager.js"),h=t("./prod-params.js"),d=(t("console-browserify"),function(){"use strict";var t,e,n,f,p,I,C,m,y,E,v,B={"<":!0,"<=":!0,"==":!0,"===":!0,"!==":!0,"!=":!0,">":!0,">=":!0,"+":!0,"-":!0,"*":!0,"/":!0,"%":!0},b=[],Q=new i.EventEmitter;function w(t,e,n){var i,o;return e?(i="",o=u.validNames):(i="unstable ",o=u.unstableNames),t=t.trim(),!!/^[+-]W\d{3}$/g.test(t)||(!(-1===o.indexOf(t)&&"jslint"!==n.type&&!r.has(u.removed,t))||(F("E001",n,i,t),!1))}function x(t,e){return!!t&&!(!t.identifier||t.value!==e)}function _(t,e){if(!e.reserved)return!1;var n=e.meta;if(n&&n.isFutureReservedWord){if(l.inES5()){if(!n.es5)return!1;if(e.isProperty)return!1}}else if(n&&n.es5&&!l.inES5())return!1;return!(n&&n.strictOnly&&l.inES5()&&!l.option.strict&&!l.isStrict())&&!!("await"!==e.id||t&h.async||l.option.module)}function D(t,e){return t.replace(/\{([^{}]*)\}/g,function(t,n){var r=e[n];return"string"==typeof r||"number"==typeof r?r:t})}function M(t,e){Object.keys(e).forEach(function(n){r.has(d.blacklist,n)||(t[n]=e[n])})}function k(){var t;!function(){if(l.option.enforceall){for(var t in u.bool.enforcing)void 0!==l.option[t]||u.noenforceall[t]||(l.option[t]=!0);for(var e in u.bool.relaxing)void 0===l.option[e]&&(l.option[e]=!1)}}(),(t=l.inferEsVersion())&&S("E059",l.tokens.next,"esversion",t),l.inES5()&&M(E,o.ecmaIdentifiers[5]),l.inES6()&&M(E,o.ecmaIdentifiers[6]),l.inES8()&&M(E,o.ecmaIdentifiers[8]),"global"===l.option.strict&&"globalstrict"in l.option&&S("E059",l.tokens.next,"strict","globalstrict"),l.option.module&&(l.inES6()||R("W134",l.tokens.next,"module",6)),l.option.regexpu&&(l.inES6()||R("W134",l.tokens.next,"regexpu",6)),l.option.couch&&M(E,o.couch),l.option.qunit&&M(E,o.qunit),l.option.rhino&&M(E,o.rhino),l.option.shelljs&&(M(E,o.shelljs),M(E,o.node)),l.option.typed&&M(E,o.typed),l.option.phantom&&M(E,o.phantom),l.option.prototypejs&&M(E,o.prototypejs),l.option.node&&(M(E,o.node),M(E,o.typed)),l.option.devel&&M(E,o.devel),l.option.dojo&&M(E,o.dojo),l.option.browser&&(M(E,o.browser),M(E,o.typed)),l.option.browserify&&(M(E,o.browser),M(E,o.typed),M(E,o.browserify)),l.option.nonstandard&&M(E,o.nonstandard),l.option.jasmine&&M(E,o.jasmine),l.option.jquery&&M(E,o.jquery),l.option.mootools&&M(E,o.mootools),l.option.worker&&M(E,o.worker),l.option.wsh&&M(E,o.wsh),l.option.yui&&M(E,o.yui),l.option.mocha&&M(E,o.mocha)}function S(t,e,n,r){var i=Math.floor(e.line/l.lines.length*100),o=a.errors[t].desc,s={name:"JSHintError",line:e.line,character:e.from,message:o+" ("+i+"% scanned).",raw:o,code:t,a:n,b:r};throw s.reason=D(o,s)+" ("+i+"% scanned).",s}function L(){var t=l.ignoredLines;r.isEmpty(t)||(d.errors=r.reject(d.errors,function(e){return t[e.line]}))}function R(t,e,n,r,i,o){var s,A,c,u;if(/^W\d{3}$/.test(t)){if(l.ignored[t])return;u=a.warnings[t]}else/E\d{3}/.test(t)?u=a.errors[t]:/I\d{3}/.test(t)&&(u=a.info[t]);return"(end)"===(e=e||l.tokens.next||{}).id&&(e=l.tokens.curr),A=e.line,s=e.from,(c={id:"(error)",raw:u.desc,code:u.code,evidence:l.lines[A-1]||"",line:A,character:s,scope:d.scope,a:n,b:r,c:i,d:o}).reason=D(u.desc,c),d.errors.push(c),L(),d.errors.length>=l.option.maxerr&&S("E043",e),c}function N(t,e,n,r,i,o,a){return R(t,{line:e,from:n},r,i,o,a)}function F(t,e,n,r,i,o){R(t,e,n,r,i,o)}function T(t,e,n,r,i,o,a){return F(t,{line:e,from:n},r,i,o,a)}function U(t,e){d.internals.push({id:"(internal)",elem:t,token:e,code:e.value.replace(/([^\\])(\\*)\2\\n/g,"$1\n")})}function O(t,n){var i=t.body.split(",").map(function(t){return t.trim()}),o={};if("falls through"!==t.type){if("globals"===t.type)for(var a in i.forEach(function(e,n){var r=(e=e.split(":"))[0].trim(),a=(e[1]||"").trim();if("-"!==r&&r.length)"-"===r.charAt(0)?(r=r.slice(1),a=!1,d.blacklist[r]=r,delete E[r]):o[r]="true"===a;else{if(n>0&&n===i.length-1)return;F("E002",t)}}),M(E,o),o)r.has(o,a)&&(e[a]=t);"exported"===t.type&&i.forEach(function(e,n){if(e.length)l.funct["(scope)"].addExported(e);else{if(n>0&&n===i.length-1)return;F("E002",t)}}),"members"===t.type&&(y=y||{},i.forEach(function(t){var e=t.charAt(0);e!==t.charAt(t.length-1)||'"'!==e&&"'"!==e||(t=t.substr(1,t.length-2).replace('\\"','"')),y[t]=!1}));var s=["maxstatements","maxparams","maxdepth","maxcomplexity","maxerr","maxlen","indent"];"jshint"!==t.type&&"jslint"!==t.type&&"jshint.unstable"!==t.type||(i.forEach(function(e){var n=(e=e.split(":"))[0].trim(),r=(e[1]||"").trim();if(w(n,"jshint.unstable"!==t.type,t))if(s.indexOf(n)>=0)if("false"!==r){if("number"!=typeof(r=+r)||!isFinite(r)||r<=0||Math.floor(r)!==r)return void F("E032",t,e[1].trim());l.option[n]=r}else l.option[n]="indent"===n&&4;else{if("validthis"===n)return l.funct["(global)"]?void F("E009"):"true"!==r&&"false"!==r?void F("E002",t):void(l.option.validthis="true"===r);if("quotmark"!==n)if("shadow"!==n)if("unused"!==n)if("latedef"!==n)if("ignore"!==n)if("strict"!==n)if("module"===n&&(Rt(l.funct)||F("E055",t,"module")),"esversion"!==n){var i,o=/^([+-])(W\d{3})$/g.exec(n);if(o)l.ignored[o[2]]="-"===o[1];else"true"!==r&&"false"!==r?F("E002",t):"jslint"===t.type?(i=u.renamed[n]||n,l.option[i]="true"===r,void 0!==u.inverted[i]&&(l.option[i]=!l.option[i])):"jshint.unstable"===t.type?l.option.unstable[n]="true"===r:l.option[n]="true"===r}else{switch(r){case"3":case"5":case"6":case"7":case"8":case"9":l.option.moz=!1,l.option.esversion=+r;break;case"2015":case"2016":case"2017":case"2018":l.option.moz=!1,l.option.esversion=+r-2009;break;default:F("E002",t)}Rt(l.funct)||F("E055",t,"esversion")}else switch(r){case"true":l.option.strict=!0;break;case"false":l.option.strict=!1;break;case"global":case"implied":l.option.strict=r;break;default:F("E002",t)}else switch(r){case"line":l.ignoredLines[t.line]=!0,L();break;default:F("E002",t)}else switch(r){case"true":l.option.latedef=!0;break;case"false":l.option.latedef=!1;break;case"nofunc":l.option.latedef="nofunc";break;default:F("E002",t)}else switch(r){case"true":l.option.unused=!0;break;case"false":l.option.unused=!1;break;case"vars":case"strict":l.option.unused=r;break;default:F("E002",t)}else switch(r){case"true":l.option.shadow=!0;break;case"outer":l.option.shadow="outer";break;case"false":case"inner":l.option.shadow="inner";break;default:F("E002",t)}else switch(r){case"true":case"false":l.option.quotmark="true"===r;break;case"double":case"single":l.option.quotmark=r;break;default:F("E002",t)}}}),k())}else n.caseFallsThrough=!0}function G(t){var e,n=t||0,r=I.length;if(n<r)return I[n];for(;r<=n;){if(!(e=C.token()))return I.length?I[r-1]:l.tokens.next;I[r]=e,r+=1}return e}function H(){var t,e=0;do{t=G(e++)}while("(endline)"===t.id);return t}function j(t,e){var n=l.tokens.next;switch(l.tokens.curr.id){case"(number)":"."===n.id&&R("W005",l.tokens.curr);break;case"-":"-"!==n.id&&"--"!==n.id||R("W006");break;case"+":"+"!==n.id&&"++"!==n.id||R("W007")}for(t&&n.id!==t&&(e?"(end)"===n.id?F("E019",e,e.id):F("E020",n,t,e.id,e.line,n.value):"(identifier)"===n.type&&n.value===t||F("E021",n,t,n.value)),l.tokens.prev=l.tokens.curr,l.tokens.curr=l.tokens.next;;){if(l.tokens.next=I.shift()||C.token(),l.tokens.next||S("E041",l.tokens.curr),"(end)"===l.tokens.next.id||"(error)"===l.tokens.next.id)return;if(l.tokens.next.check&&l.tokens.next.check(),l.tokens.next.isSpecial)O(l.tokens.next,l.tokens.curr);else if("(endline)"!==l.tokens.next.id)break}}function Y(t,e,n){return arguments.length<=1&&(e=l.tokens.curr,n=l.tokens.next),!!("in"===n.id&&t&h.noin)||(";"===n.id||"}"===n.id||":"===n.id||(n.infix===e.infix||"after"===e.ltBoundary||"before"===n.ltBoundary)&&e.line!==K(n))}function P(t,e){var n,r,i=!1,o=!1,a=t&h.initial;if(t&=~h.initial,l.nameStack.push(),"(end)"===l.tokens.next.id&&F("E006",l.tokens.curr),j(),a&&(l.funct["(verb)"]=l.tokens.curr.value,l.tokens.curr.beginsStmt=!0),r=l.tokens.curr,a&&r.fud&&(!r.useFud||r.useFud(t)))n=l.tokens.curr.fud(t);else for(l.tokens.curr.nud?n=l.tokens.curr.nud(t,e):F("E030",l.tokens.curr,l.tokens.curr.id);e<l.tokens.next.lbp&&!Y(t);)i="Array"===l.tokens.curr.value,o="Object"===l.tokens.curr.value,n&&(n.value||n.first&&n.first.value)&&("new"!==n.value||n.first&&n.first.value&&"."===n.first.value)&&(i=!1,n.value!==l.tokens.curr.value&&(o=!1)),j(),i&&"("===l.tokens.curr.id&&")"===l.tokens.next.id&&R("W009",l.tokens.curr),o&&"("===l.tokens.curr.id&&")"===l.tokens.next.id&&R("W010",l.tokens.curr),n&&l.tokens.curr.led?n=l.tokens.curr.led(t,n):F("E033",l.tokens.curr,l.tokens.curr.id);return l.nameStack.pop(),n}function K(t){return t.startLine||t.line}function z(t,e){l.option.laxbreak||t.line===K(e)||R("W014",e,e.value)}function J(t){(t=t).line!==K(l.tokens.next)&&R("E022",t,t.value)}function q(t,e){t.line!==K(e)&&(l.option.laxcomma||(V.first&&(R("I001"),V.first=!1),R("W014",t,e.value)))}function V(t){if((t=t||{}).peek?q(l.tokens.prev,l.tokens.curr):(q(l.tokens.curr,l.tokens.next),j(",")),l.tokens.next.identifier&&(!t.property||!l.inES5()))switch(l.tokens.next.value){case"break":case"case":case"catch":case"continue":case"default":case"do":case"else":case"finally":case"for":case"if":case"in":case"instanceof":case"return":case"switch":case"throw":case"try":case"var":case"let":case"while":case"with":return F("E024",l.tokens.next,l.tokens.next.value),!1}if("(punctuator)"===l.tokens.next.type)switch(l.tokens.next.value){case"}":case"]":case",":case")":return!!t.allowTrailing||(F("E024",l.tokens.next,l.tokens.next.value),!1)}return!0}function W(t,e){var n=l.syntax[t];return n&&"object"==typeof n||(l.syntax[t]=n={id:t,lbp:e,rbp:e,value:t}),n}function X(t){var e=W(t,0);return e.delim=!0,e}function Z(t,e){var n=X(t);return n.identifier=n.reserved=!0,n.fud=e,n}function $(t,e){var n=Z(t,e);return n.block=!0,n}function tt(t){var e=t.id.charAt(0);return(e>="a"&&e<="z"||e>="A"&&e<="Z")&&(t.identifier=t.reserved=!0),t}function et(t,e){var n=W(t,150);return tt(n),n.nud="function"==typeof e?e:function(t){return this.arity="unary",this.right=P(t,150),"++"!==this.id&&"--"!==this.id||(l.option.plusplus&&R("W016",this,this.id),this.right&&gt(t,this.right,this)),this},n}function nt(t,e){var n=X(t);return n.type=t,n.nud=e,n}function rt(t,e){var n=nt(t,e);return n.identifier=!0,n.reserved=!0,n}function it(t,e){var n=nt(t,e&&e.nud||l.syntax["(identifier)"].nud);return(e=e||{}).isFutureReservedWord=!0,n.value=t,n.identifier=!0,n.reserved=!0,n.meta=e,n}function ot(t,e){return rt(t,function(){return"function"==typeof e&&e(this),this})}function at(t,e,n,r){var i=W(t,n);return tt(i),i.infix=!0,i.led=function(i,o){return r||z(l.tokens.prev,l.tokens.curr),"in"!==t&&"instanceof"!==t||"!"!==o.id||R("W018",o,"!"),"function"==typeof e?e(i,o,this):(this.left=o,this.right=P(i,n),this)},i}function st(t,e){var n=W(t,100);return n.infix=!0,n.led=function(t,n){z(l.tokens.prev,l.tokens.curr),this.left=n;var r=this.right=P(t,100);return x(n,"NaN")||x(r,"NaN")?R("W019",this):e&&e.apply(this,[t,n,r]),n&&r||S("E041",l.tokens.curr),"!"===n.id&&R("W018",n,"!"),"!"===r.id&&R("W018",r,"!"),this},n}function At(t){return"unary"===t.arity&&"++"!==t.id&&"--"!==t.id}var lt={};function ct(t,e,n){var i;return!n.option.notypeof&&(!(!t||!e)&&(i=n.inES6()?lt.es6:lt.es3,"(identifier)"===e.type&&"typeof"===e.value&&"(string)"===t.type&&!r.includes(i,t.value)))}function ut(t,e){var n=!1;return"this"===t.type&&null===e.funct["(context)"]?n=!0:"(identifier)"===t.type&&(e.option.node&&"global"===t.value?n=!0:!e.option.browser||"window"!==t.value&&"document"!==t.value||(n=!0)),n}function gt(t,e,n,r){var i=r&&r.allowDestructuring;if(n=n||e,l.option.freeze){var o=function(t){var e=["Array","ArrayBuffer","Boolean","Collator","DataView","Date","DateTimeFormat","Error","EvalError","Float32Array","Float64Array","Function","Infinity","Intl","Int16Array","Int32Array","Int8Array","Iterator","Number","NumberFormat","Object","RangeError","ReferenceError","RegExp","StopIteration","String","SyntaxError","TypeError","Uint16Array","Uint32Array","Uint8Array","Uint8ClampedArray","URIError"],n=function t(e){if("object"==typeof e)return"prototype"===e.right?e:t(e.left)}(t);if(n)return function(t){for(;!t.identifier&&"object"==typeof t.left;)t=t.left;if(t.identifier&&e.indexOf(t.value)>=0&&l.funct["(scope)"].isPredefined(t.value))return t.value}(n)}(e);o&&R("W121",e,o)}return ne(e,"...")&&(e=e.right),e.identifier&&!e.isMetaProperty&&l.funct["(scope)"].block.reassign(e.value,e),"."===e.id?(e.left&&("arguments"!==e.left.value||l.isStrict())||R("W143",n),l.nameStack.set(l.tokens.prev),!0):"{"===e.id||"["===e.id?(i&&e.destructAssign||("{"!==e.id&&e.left?"arguments"!==e.left.value||l.isStrict()||R("W143",n):R("E031",n)),"["===e.id&&l.nameStack.set(e.right),!0):!e.identifier||_(t,e)||e.isMetaProperty||"eval"===e.value||"arguments"===e.value?(F("E031",n),!1):("exception"===l.funct["(scope)"].labeltype(e.value)&&R("W022",e),l.nameStack.set(e),!0)}function ht(t,e,n){var r=at(t,"function"==typeof e?e:function(t,e,n){return n.left=e,gt(t,e,n,{allowDestructuring:!0}),n.right=P(t,10),n},n);return r.exps=!0,r.assign=!0,r}function dt(t,e,n){var r=W(t,n);return tt(r),r.infix=!0,r.led="function"==typeof e?e:function(t,e){return l.option.bitwise&&R("W016",this,this.id),this.left=e,this.right=P(t,n),this},r}function ft(t){return ht(t,function(t,e,n){return l.option.bitwise&&R("W016",n,n.id),gt(t,e,n),n.right=P(t,10),n},20)}function pt(t){var e=W(t,150);return e.led=function(t,e){return l.option.plusplus&&R("W016",this,this.id),gt(t,e,this),this.left=e,this},e}function It(t,e,n){if(l.tokens.next.identifier){n||j();var r=l.tokens.curr,i=l.tokens.curr.value;return _(t,r)?e&&l.inES5()?i:"undefined"===i?i:(R("W024",l.tokens.curr,l.tokens.curr.id),i):i}}function Ct(t){if(!ne(l.tokens.next,"..."))return!1;if(l.inES6(!0)||R("W119",l.tokens.next,t+" operator","6"),j(),ne(l.tokens.next,"..."))for(R("E024",l.tokens.next,"...");ne(l.tokens.next,"...");)j();return!0}function mt(t,e){var n=It(t,e,!1);if(n)return n;F("E030",l.tokens.next,l.tokens.next.value),";"!==l.tokens.next.id&&j()}function yt(t){var e,n=0;if(";"===l.tokens.next.id&&!t.inBracelessBlock)for(;;){do{e=G(n),n+=1}while("(end)"!==e.id&&"(comment)"===e.id);if(e.reach)return;if("(endline)"!==e.id){if("function"===e.id){!0===l.option.latedef&&R("W026",e);break}R("W027",e,e.value,t.value);break}}}function Et(t){if(";"!==l.tokens.next.id){if(l.tokens.next.isUnclosed)return j();var e=K(l.tokens.next)===l.tokens.curr.line&&"(end)"!==l.tokens.next.id,n=ne(l.tokens.next,"}");!e||n||"do"===t.id&&l.inES6(!0)?l.option.asi||n&&e&&l.option.lastsemic||N("W033",l.tokens.curr.line,l.tokens.curr.character):T("E058",l.tokens.curr.line,l.tokens.curr.character)}else j(";")}function vt(t){var e,n=p,r=l.tokens.next,i=!1;if(t|=h.initial,";"!==r.id){var o=_(t,r);return o&&r.meta&&r.meta.isFutureReservedWord&&!r.fud&&(R("W024",r,r.id),o=!1),r.identifier&&!o&&":"===G().id&&(j(),j(":"),i=!0,l.funct["(scope)"].stack(),l.funct["(scope)"].block.addBreakLabel(r.value,{token:l.tokens.curr}),l.tokens.next.labelled||"{"===l.tokens.next.value||R("W028",l.tokens.next,r.value,l.tokens.next.value),r=l.tokens.next),"{"===r.id?(Qt(t,!0,!0,!1,!1,"case"===l.funct["(verb)"]&&":"===l.tokens.curr.value),void(i&&l.funct["(scope)"].unstack())):(!(e=P(t,0))||e.identifier&&"function"===e.value||"(punctuator)"===e.type&&e.left&&e.left.identifier&&"function"===e.left.value||!l.isStrict()&&l.stmtMissingStrict()&&R("E007"),r.block||(l.option.expr||e&&e.exps?l.option.nonew&&e&&e.left&&"("===e.id&&"new"===e.left.id&&R("W031",r):R("W030",l.tokens.curr),Et(r)),p=n,i&&l.funct["(scope)"].unstack(),e)}j(";")}function Bt(t){for(var e,n=[];!l.tokens.next.reach&&"(end)"!==l.tokens.next.id;)";"===l.tokens.next.id?((!(e=G())||"("!==e.id&&"["!==e.id)&&R("W032"),j(";")):n.push(vt(t));return n}function bt(){for(var t=l.tokens.next;"(string)"===l.tokens.next.id;){var e=H();if(!Y(0,t,e))break;t=e,j();var n=l.tokens.curr.value;(l.directive[n]||"use strict"===n&&"implied"===l.option.strict)&&R("W034",l.tokens.curr,n),"use strict"===n&&l.inES7()&&!l.funct["(global)"]&&!1===l.funct["(hasSimpleParams)"]&&F("E065",l.tokens.curr),l.directive[n]=!0,Et(t)}l.isStrict()&&(l.option.undef=!0)}function Qt(t,e,n,r,i,o){var a,s,A,c,u=f,g=p;f=e,A=l.tokens.next;var d=l.funct["(metrics)"];if(d.nestedBlockDepth+=1,d.verifyMaxNestedBlockDepthPerFunction(),"{"===l.tokens.next.id){if(j("{"),l.funct["(scope)"].stack(),"}"!==l.tokens.next.id){for(p+=l.option.indent;!e&&l.tokens.next.from>p;)p+=l.option.indent;if(r){for(c in s={},l.directive)s[c]=l.directive[c];bt(),l.funct["(isStrict)"]=l.isStrict(),l.option.strict&&l.funct["(context)"]["(global)"]&&(s["use strict"]||l.isStrict()||R("E007"))}a=Bt(t),d.statementCount+=a.length,p-=l.option.indent}else r&&(l.funct["(isStrict)"]=l.isStrict());j("}",A),r&&(l.funct["(scope)"].validateParams(i),s&&(l.directive=s)),l.funct["(scope)"].unstack(),p=g}else if(e){l.funct["(scope)"].stack(),n&&!l.option.curly||R("W116",l.tokens.next,"{",l.tokens.next.value);var I="if"===l.funct["(verb)"]||"else"===l.tokens.curr.id;l.tokens.next.inBracelessBlock=!0,p+=l.option.indent,a=[vt(t)],p-=l.option.indent,!a[0]||!a[0].declaration||I&&"function"===a[0].id||F("E048",a[0],a[0].id[0].toUpperCase()+a[0].id.slice(1)),l.funct["(scope)"].unstack()}else if(r){l.funct["(scope)"].stack(),!n||i||l.inMoz()||F("W118",l.tokens.curr,"function closure expressions"),i&&l.funct["(scope)"].validateParams(!0);var C=P(t,10);l.option.noreturnawait&&t&h.async&&C.identifier&&"await"===C.value&&R("W146",C),l.option.strict&&l.funct["(context)"]["(global)"]&&(l.isStrict()||R("E007")),l.funct["(scope)"].unstack()}else F("E021",l.tokens.next,"{",l.tokens.next.value);switch(l.funct["(verb)"]){case"break":case"continue":case"return":case"throw":if(o)break;default:l.funct["(verb)"]=null}return f=u,!e||!l.option.noempty||a&&0!==a.length||R("W035",l.tokens.prev),d.nestedBlockDepth-=1,a}function wt(t){y&&"boolean"!=typeof y[t]&&R("W036",l.tokens.curr,t),"number"==typeof m[t]?m[t]+=1:m[t]=1}lt.legacy=["xml","unknown"],lt.es3=["undefined","boolean","number","string","function","object"],lt.es3=lt.es3.concat(lt.legacy),lt.es6=lt.es3.concat("symbol"),nt("(number)",function(){return this}),nt("(string)",function(){return this}),l.syntax["(identifier)"]={type:"(identifier)",lbp:0,identifier:!0,nud:function(t){var e=this.value;return"=>"===l.tokens.next.id?this:(_(t,this)?R("W024",this,e):l.funct["(comparray)"].check(e)||l.funct["(scope)"].block.use(e,l.tokens.curr),this)},led:function(){F("E033",l.tokens.next,l.tokens.next.value)}};var xt,_t={identifier:!1,template:!0};l.syntax["(template)"]=r.extend({lbp:155,type:"(template)",nud:Nt,led:Nt,noSubst:!1},_t),l.syntax["(template middle)"]=r.extend({lbp:0,type:"(template middle)",noSubst:!1},_t),l.syntax["(template tail)"]=r.extend({lbp:0,type:"(template tail)",tail:!0,noSubst:!1},_t),l.syntax["(no subst template)"]=r.extend({lbp:155,type:"(template)",nud:Nt,led:Nt,noSubst:!0,tail:!0},_t),nt("(regexp)",function(){return this}),X("(endline)"),(xt=X("(begin)")).line=xt.from=0,X("(end)").reach=!0,X("(error)").reach=!0,X("}").reach=!0,X(")"),X("]"),X('"').reach=!0,X("'").reach=!0,X(";"),X(":").reach=!0,X("#"),rt("else"),rt("case").reach=!0,rt("catch"),rt("default").reach=!0,rt("finally"),rt("true",function(){return this}),rt("false",function(){return this}),ot("null"),ot("this",function(t){l.isStrict()&&!l.funct["(method)"]&&!l.option.validthis&&(l.funct["(statement)"]&&l.funct["(name)"].charAt(0)>"Z"||l.funct["(global)"])&&R("W040",t)}),ot("super",function(t){Wt.call(l.tokens.curr,t)}),ht("=","assign",20),ht("+=","assignadd",20),ht("-=","assignsub",20),ht("*=","assignmult",20),ht("/=","assigndiv",20).nud=function(){F("E014")},ht("%=","assignmod",20),ht("**=",function(t,e,n){return l.inES7()||R("W119",n,"Exponentiation operator","7"),n.left=e,gt(t,e,n),n.right=P(t,10),n},20),ft("&="),ft("|="),ft("^="),ft("<<="),ft(">>="),ft(">>>="),at(",",function(t,e,n){var r;if(n.exprs=[e],l.option.nocomma&&R("W127"),!V({peek:!0}))return n;for(;(r=P(t,10))&&(n.exprs.push(r),","===l.tokens.next.value&&V()););return n},10,!0),at("?",function(t,e,n){return Ut(),n.left=e,n.right=P(t&~h.noin,10),j(":"),P(t,10),n},30);function Dt(t,e){var n,r,i,o,a,s,A=Object.create(null);for("{"===l.tokens.next.value?j("{"):(R("W116",l.tokens.curr,"identifier",l.tokens.next.type),j());"}"!==l.tokens.next.value;)switch(o=!1,a=!1,e&=~h.preAsync,"static"===l.tokens.next.value&&(o=!0,j()),"async"===l.tokens.next.value&&(ne(G(),"(")||(e|=h.preAsync,j(),J(l.tokens.curr),ne(l.tokens.next,"*")&&(a=!0,j("*"),l.inES9()||R("W119",l.tokens.next,"async generators","9")),l.inES8()||R("W119",l.tokens.curr,"async functions","8"))),"*"===l.tokens.next.value&&(a=!0,j()),(i=l.tokens.next).value){case";":R("W032",i),j();break;case"constructor":o?(Zt(A,n=St(e),i,!0,o),Mt(t,e,n,a)):((a||e&h.preAsync)&&F("E024",i,i.value),s&&F("E024",i,i.value),j(),Mt(t,e,l.nameStack.infer()),s=!0);break;case"set":case"get":a&&F("E024",i,i.value),r=i.value,j(),"["===l.tokens.next.value?Mt(t,e,n=te(e),!1):("prototype"!==(n=St(e))&&"constructor"!==n||F("E049",l.tokens.curr,"class "+r+"ter method",n),$t(r,A,n,l.tokens.curr,!0,o),Mt(t,e,l.nameStack.infer(),!1));break;case"[":Mt(t,e,n=te(e),a);break;default:if(!(n=St(e))){F("E024",i,i.value),j();break}"prototype"===n&&F("E049",i,"class method",n),Zt(A,n,i,!0,o),Mt(t,e,n,a)}j("}"),Gt(A),l.inClassBody=!1,l.funct["(scope)"].unstack()}function Mt(t,e,n,r){if(r&&(l.inES6()||R("W119",l.tokens.curr,"function*","6")),"("!==l.tokens.next.value){if(F("E054",l.tokens.next,l.tokens.next.value),j(),"{"===l.tokens.next.value)return j(),void("}"===l.tokens.next.value&&(R("W116",l.tokens.next,"(",l.tokens.next.value),j(),mt(e),j()));for(;"("!==l.tokens.next.value;)j()}Ft(e,{name:n,type:r?"generator":null,isMethod:!0,statement:t})}function kt(t){var e,n=l.tokens.next,r=-1;do{"("===n.value?t+=1:")"===n.value&&(t-=1),e=n,n=G(r+=1)}while((0!==t||")"!==e.value)&&"(end)"!==n.type);return n}function St(t,e){var n,r=!0;return(n="object"==typeof e?e:It(t,!0,r=e))?"object"==typeof n&&("(string)"===n.id||"(identifier)"===n.id?n=n.value:"(number)"===n.id&&(n=n.value.toString())):"(string)"===l.tokens.next.id?(n=l.tokens.next.value,r||j()):"(number)"===l.tokens.next.id&&(n=l.tokens.next.value.toString(),r||j()),"hasOwnProperty"===n&&R("W001"),n}function Lt(t,e,n){var i={"(name)":t,"(breakage)":0,"(loopage)":0,"(isStrict)":"unknown","(global)":!1,"(line)":null,"(character)":null,"(metrics)":null,"(statement)":null,"(context)":null,"(scope)":null,"(comparray)":null,"(generator)":null,"(arrow)":null,"(async)":null,"(params)":null};return e&&r.extend(i,{"(line)":e.line,"(character)":e.character,"(metrics)":Tt(e)}),r.extend(i,n),i["(context)"]&&(i["(scope)"]=i["(context)"]["(scope)"],i["(comparray)"]=i["(context)"]["(comparray)"]),i}function Rt(t){return t["(global)"]&&!t["(verb)"]}function Nt(t,e){var n=this.context,r=this.noSubst,i=this.depth,o="number"==typeof e?null:e;if(!r)for(;!a();)!l.tokens.next.template||l.tokens.next.depth>i?P(t,0):j();return{id:"(template)",type:"(template)",tag:o};function a(){if(l.tokens.curr.template&&l.tokens.curr.tail&&l.tokens.curr.context===n)return!0;var t=l.tokens.next.template&&l.tokens.next.tail&&l.tokens.next.context===n;return t&&j(),t||l.tokens.next.isUnclosed}}function Ft(t,e){var i,o,a,s,A,c,u,g,d,f=l.option,p=l.ignored,I=t&h.preAsync;e&&(a=e.name,s=e.statement,A=e.classExprBinding,c="generator"===e.type,u="arrow"===e.type,g=e.isMethod,d=e.ignoreLoopFunc),t&=~h.noin,t&=~h.tryClause,I?t|=h.async:t&=~h.async,t&=~h.preAsync,l.option=Object.create(l.option),l.ignored=Object.create(l.ignored),l.funct=Lt(a||l.nameStack.infer(),l.tokens.next,{"(statement)":s,"(context)":l.funct,"(arrow)":u,"(method)":g,"(generator)":c,"(async)":I}),i=l.funct,o=l.tokens.curr,n.push(l.funct),l.funct["(scope)"].stack("functionouter");var C=!g&&(a||A);C&&l.funct["(scope)"].block.add(C,A?"class":"function",l.tokens.curr,!1),u||l.funct["(scope)"].funct.add("arguments","var",o,!1),l.funct["(scope)"].stack("functionparams");var m=function(t,e){var n,i,o,a=[],s=[],A=!1,c=!1,u=0,g=e&&e.loneArg,h=!1;if(g&&!0===g.identifier)return l.funct["(scope)"].addParam(g.value,g),{arity:1,params:[g.value],isSimple:!0};if(n=l.tokens.next,e&&e.parsedOpening||j("("),")"!==l.tokens.next.id)for(;;){u++;var d=[];if(r.includes(["{","["],l.tokens.next.id))for(o in h=!0,s=Ht(t))(o=s[o]).id&&(a.push(o.id),d.push([o.id,o.token]));else if(c=Ct("rest"),i=mt(t))a.push(i),d.push([i,l.tokens.curr]);else for(;!ee(l.tokens.next,[",",")"]);)j();if(A&&"="!==l.tokens.next.id&&F("W138",l.tokens.curr),"="===l.tokens.next.id&&(l.inES6()||R("W119",l.tokens.next,"default parameters","6"),c&&F("E062",l.tokens.next),j("="),A=!0,P(t,10)),d.forEach(f),","===l.tokens.next.id&&(c&&R("W131",l.tokens.next),V({allowTrailing:!0})),")"===l.tokens.next.id)return","!==l.tokens.curr.id||l.inES8()||R("W119",l.tokens.curr,"Trailing comma in function parameters","8"),j(")",n),{arity:u,params:a,isSimple:!h&&!c&&!A}}else j(")");function f(t){l.funct["(scope)"].addParam.apply(l.funct["(scope)"],t)}}(t,e);return m?(l.funct["(params)"]=m.params,l.funct["(hasSimpleParams)"]=m.isSimple,l.funct["(metrics)"].arity=m.arity,l.funct["(metrics)"].verifyMaxParametersPerFunction()):(l.funct["(params)"]=[],l.funct["(metrics)"].arity=0,l.funct["(hasSimpleParams)"]=!0),u&&(l.inES6(!0)||R("W119",l.tokens.curr,"arrow function syntax (=>)","6"),e.loneArg||j("=>")),Qt(t,!1,!0,!0,u),!l.option.noyield&&c&&"yielded"!==l.funct["(generator)"]&&R("W124",l.tokens.curr),l.funct["(metrics)"].verifyMaxStatementsPerFunction(),l.funct["(metrics)"].verifyMaxComplexityPerFunction(),l.funct["(unusedOption)"]=l.option.unused,l.option=f,l.ignored=p,l.funct["(last)"]=l.tokens.curr.line,l.funct["(lastcharacter)"]=l.tokens.curr.character,l.funct["(scope)"].unstack(),l.funct["(scope)"].unstack(),l.funct=l.funct["(context)"],d||l.option.loopfunc||!l.funct["(loopage)"]||i["(outerMutables)"]&&R("W083",o,i["(outerMutables)"].join(", ")),i}function Tt(t){return{statementCount:0,nestedBlockDepth:-1,ComplexityCount:1,arity:0,verifyMaxStatementsPerFunction:function(){l.option.maxstatements&&this.statementCount>l.option.maxstatements&&R("W071",t,this.statementCount)},verifyMaxParametersPerFunction:function(){r.isNumber(l.option.maxparams)&&this.arity>l.option.maxparams&&R("W072",t,this.arity)},verifyMaxNestedBlockDepthPerFunction:function(){l.option.maxdepth&&this.nestedBlockDepth>0&&this.nestedBlockDepth===l.option.maxdepth+1&&R("W073",null,this.nestedBlockDepth)},verifyMaxComplexityPerFunction:function(){var e=l.option.maxcomplexity,n=this.ComplexityCount;e&&n>e&&R("W074",t,n)}}}function Ut(){l.funct["(metrics)"].ComplexityCount+=1}function Ot(t){var e,n;switch(t&&(e=t.id,n=t.paren,","===e&&(t=t.exprs[t.exprs.length-1])&&(e=t.id,n=n||t.paren)),e){case"=":case"+=":case"-=":case"*=":case"%=":case"&=":case"|=":case"^=":case"/=":n||l.option.boss||R("W084")}}function Gt(t){if(l.inES5())for(var e in t)t[e]&&t[e].setterToken&&!t[e].getterToken&&!t[e].static&&R("W078",t[e].setterToken)}function Ht(t,e){var n=e&&e.assignment;return t&=~h.noin,l.inES6()||R("W104",l.tokens.curr,n?"destructuring assignment":"destructuring binding","6"),function t(e,n){var r,i;var o=[];var a=n&&n.openingParsed;var s=n&&n.assignment;var A=s?{assignment:s}:null;var c=a?l.tokens.curr:l.tokens.next;var u=function(){var n;if(ee(l.tokens.next,["[","{"]))for(r=t(e,A),i=0;i<r.length;i++)o.push({id:r[i].id,token:r[i].token});else if(ne(l.tokens.next,","))o.push({id:null,token:l.tokens.curr});else if(ne(l.tokens.next,"("))j("("),u(),j(")");else{if(s){var a=P(e,20);a&&(gt(e,a),a.identifier&&(n=a.value))}else n=mt(e);n&&o.push({id:n,token:l.tokens.curr})}};var g=function(t){var e,n;if(ne(l.tokens.next,"["))j("["),P(t,10),j("]"),j(":"),u();else if("(string)"===l.tokens.next.id||"(number)"===l.tokens.next.id)j(),j(":"),u();else{var r=Ct("rest");r?(l.inES9()||R("W119",l.tokens.next,"object rest property","9"),"(identifier)"!==(n=P(t,10)).type&&F("E030",n,n.value)):e=mt(t),!r&&ne(l.tokens.next,":")?(j(":"),u()):e&&(s&&gt(t,l.tokens.curr),o.push({id:e,token:l.tokens.curr})),r&&ne(l.tokens.next,",")&&R("W130",l.tokens.next)}};var h,d;if(ne(c,"[")){a||j("["),ne(l.tokens.next,"]")&&R("W137",l.tokens.curr);for(var f=!1;!ne(l.tokens.next,"]");){var p=Ct("rest");u(),p&&!f&&ne(l.tokens.next,",")&&(R("W130",l.tokens.next),f=!0),!p&&ne(l.tokens.next,"=")&&(ne(l.tokens.prev,"...")?j("]"):j("="),h=l.tokens.prev,(d=P(e,10))&&d.identifier&&"undefined"===d.value&&R("W080",h,h.value)),ne(l.tokens.next,"]")||j(",")}j("]")}else if(ne(c,"{")){for(a||j("{"),ne(l.tokens.next,"}")&&R("W137",l.tokens.curr);!ne(l.tokens.next,"}")&&(g(e),ne(l.tokens.next,"=")&&(j("="),h=l.tokens.prev,(d=P(e,10))&&d.identifier&&"undefined"===d.value&&R("W080",h,h.value)),ne(l.tokens.next,"}")||(j(","),!ne(l.tokens.next,"}"))););j("}")}return o}(t,e)}function jt(t,e){var n=e.first;n&&r.zip(t,Array.isArray(n)?n:[n]).forEach(function(t){var e=t[0],n=t[1];e&&n?e.first=n:e&&e.first&&!n&&R("W080",e.first,e.first.value)})}function Yt(t,e,n){var i,o,a,s,A=n&h.noin,c=n&h.export,u="let"===t,g="const"===t;for(l.inES6()||R("W104",l.tokens.curr,t,"6"),u&&Kt()&&(j("("),l.funct["(scope)"].stack(),s=!0,e.declaration=!1),e.first=[];;){var d=[];for(var f in r.includes(["{","["],l.tokens.next.value)?(i=Ht(n),o=!1):(i=[{id:mt(n),token:l.tokens.curr}],o=!0),!A&&g&&"="!==l.tokens.next.id&&R("E012",l.tokens.curr,l.tokens.curr.value),i)i.hasOwnProperty(f)&&("let"===(f=i[f]).id&&R("W024",f.token,f.id),l.funct["(scope)"].block.isGlobal()&&!1===E[f.id]&&R("W079",f.token,f.id),f.id&&(l.funct["(scope)"].addlabel(f.id,{type:t,token:f.token}),d.push(f.token)));if("="===l.tokens.next.id){e.hasInitializer=!0,j("="),!A&&"="===G(0).id&&l.tokens.next.identifier&&R("W120",l.tokens.next,l.tokens.next.value);var p=l.tokens.prev;(a=P(n,10))&&a.identifier&&"undefined"===a.value&&R("W080",p,p.value),o||jt(d,a)}if("in"!==l.tokens.next.value&&"of"!==l.tokens.next.value)for(f in i)i.hasOwnProperty(f)&&(f=i[f],l.funct["(scope)"].initialize(f.id),o&&c&&l.funct["(scope)"].setExported(f.token.value,f.token));if(e.first=e.first.concat(d),","!==l.tokens.next.id)break;e.hasComma=!0,V()}return s&&(j(")"),Qt(n,!0,!0),e.block=!0,l.funct["(scope)"].unstack()),e}at("||",function(t,e,n){return Ut(),n.left=e,n.right=P(t,40),n},40),at("&&","and",50),at("**",function(t,e,n){return l.inES7()||R("W119",n,"Exponentiation operator","7"),!e.paren&&At(e)&&F("E024",n,"**"),n.left=e,n.right=P(t,n.rbp),n},150),l.syntax["**"].rbp=140,dt("|","bitor",70),dt("^","bitxor",80),dt("&","bitand",90),st("==",function(t,e,n){switch(!0){case!(l.option.eqnull&&("null"===(e&&e.value)||"null"===(n&&n.value)))&&l.option.eqeqeq:this.from=this.character,R("W116",this,"===","==");break;case ct(n,e,l):R("W122",this,n.value);break;case ct(e,n,l):R("W122",this,e.value)}return this}),st("===",function(t,e,n){return ct(n,e,l)?R("W122",this,n.value):ct(e,n,l)&&R("W122",this,e.value),this}),st("!=",function(t,e,n){return!(l.option.eqnull&&("null"===(e&&e.value)||"null"===(n&&n.value)))&&l.option.eqeqeq?(this.from=this.character,R("W116",this,"!==","!=")):ct(n,e,l)?R("W122",this,n.value):ct(e,n,l)&&R("W122",this,e.value),this}),st("!==",function(t,e,n){return ct(n,e,l)?R("W122",this,n.value):ct(e,n,l)&&R("W122",this,e.value),this}),st("<"),st(">"),st("<="),st(">="),dt("<<","shiftleft",120),dt(">>","shiftright",120),dt(">>>","shiftrightunsigned",120),at("in","in",120),at("instanceof",function(t,e,n){var r,i=l.funct["(scope)"];return n.left=e,n.right=r=P(t,120),r?(("(number)"===r.id||"(string)"===r.id||"null"===r.value||"undefined"===r.value&&!i.has("undefined")||"unary"===r.arity||"{"===r.id||"["===r.id&&!r.right||"(regexp)"===r.id||"(template)"===r.id&&!r.tag)&&F("E060"),"function"===r.id&&R("W139"),n):n},120),at("+",function(t,e,n){var r;return n.left=e,n.right=r=P(t,130),e&&r&&"(string)"===e.id&&"(string)"===r.id?(e.value+=r.value,e.character=r.character,!l.option.scripturl&&A.javascriptURL.test(e.value)&&R("W050",e),e):n},130),et("+","num"),at("-","sub",130),et("-","neg"),at("*","mult",140),at("/","div",140),at("%","mod",140),pt("++"),et("++","preinc"),l.syntax["++"].exps=!0,l.syntax["++"].ltBoundary="before",pt("--"),et("--","predec"),l.syntax["--"].exps=!0,l.syntax["--"].ltBoundary="before",et("delete",function(t){this.arity="unary";var e=P(t,150);return e?("."!==e.id&&"["!==e.id&&R("W051"),this.first=e,e.identifier&&!l.isStrict()&&(e.forgiveUndef=!0),this):this}).exps=!0,et("~",function(t){return l.option.bitwise&&R("W016",this,"~"),this.arity="unary",this.right=P(t,150),this}),at("..."),et("!",function(t){return this.arity="unary",this.right=P(t,150),this.right||S("E041",this),!0===B[this.right.id]&&R("W018",this,"!"),this}),et("typeof",function(t){this.arity="unary";var e=P(t,150);return this.first=this.right=e,e||S("E041",this),e.identifier&&(e.forgiveUndef=!0),this}),et("new",function(t){var e=function(t,e,n){if(ne(l.tokens.next,".")){var r=l.tokens.curr.id;j(".");var i=mt(t);return l.tokens.curr.isMetaProperty=!0,e!==i?F("E057",l.tokens.prev,r,i):n(),l.tokens.curr}}(t,"target",function(){l.inES6(!0)||R("W119",l.tokens.prev,"new.target","6");for(var t,e=l.funct;e&&(t=!e["(global)"],e["(arrow)"]);)e=e["(context)"];t||R("W136",l.tokens.prev,"new.target")});if(e)return e;var n,r=P(t,155);if(r&&"function"!==r.id)if(r.identifier)switch(r.value){case"Number":case"String":case"Boolean":case"Math":case"JSON":R("W053",l.tokens.prev,r.value);break;case"Symbol":l.inES6()&&R("W053",l.tokens.prev,r.value);break;case"Function":l.option.evil||R("W054");break;case"Date":case"RegExp":case"this":break;default:"function"!==r.id&&(n=r.value.substr(0,1),l.option.newcap&&(n<"A"||n>"Z")&&!l.funct["(scope)"].isPredefined(r.value)&&R("W055",l.tokens.curr))}else"."!==r.id&&"["!==r.id&&"("!==r.id&&R("W056",l.tokens.curr);else l.option.supernew||R("W057",this);return"("===l.tokens.next.id||l.option.supernew||R("W058",l.tokens.curr,l.tokens.curr.value),this.first=this.right=r,this}),l.syntax.new.exps=!0,$("class",function(t){var e,n,r=t&h.export;return l.inES6()||R("W104",l.tokens.curr,"class","6"),l.inClassBody=!0,l.tokens.next.identifier&&"extends"!==l.tokens.next.value&&(e=(n=l.tokens.next).value,mt(t),l.funct["(scope)"].addlabel(e,{type:"class",initialized:!1,token:n})),"extends"===l.tokens.next.value&&(j("extends"),P(t,0)),n&&(this.name=e,l.funct["(scope)"].initialize(e),r&&l.funct["(scope)"].setExported(e,n)),l.funct["(scope)"].stack(),Dt(this,t),this}).exps=!0,et("class",function(t){var e,n;return l.inES6()||R("W104",l.tokens.curr,"class","6"),l.inClassBody=!0,l.tokens.next.identifier&&"extends"!==l.tokens.next.value&&(e=(n=l.tokens.next).value,mt(t)),"extends"===l.tokens.next.value&&(j("extends"),P(t,0)),l.funct["(scope)"].stack(),n&&(this.name=e,l.funct["(scope)"].addlabel(e,{type:"class",initialized:!0,token:n}),l.funct["(scope)"].block.use(e,n)),Dt(this,t),this}),et("void").exps=!0,at(".",function(t,e,n){var r=mt(t,!0);return"string"==typeof r&&wt(r),n.left=e,n.right=r,r&&"hasOwnProperty"===r&&"="===l.tokens.next.value&&R("W001"),!e||"arguments"!==e.value||"callee"!==r&&"caller"!==r?l.option.evil||!e||"document"!==e.value||"write"!==r&&"writeln"!==r||R("W060",e):l.option.noarg?R("W059",e,r):l.isStrict()&&F("E008"),l.option.evil||"eval"!==r&&"execScript"!==r||ut(e,l)&&R("W061"),n},160,!0),at("(",function(t,e,n){l.option.immed&&e&&!e.immed&&"function"===e.id&&R("W062"),l.option.asi&&ee(l.tokens.prev,[")","]"])&&l.tokens.prev.line!==K(l.tokens.curr)&&R("W014",l.tokens.curr,l.tokens.curr.id);var r=0,i=[];if(e&&"(identifier)"===e.type&&e.value.match(/^[A-Z]([A-Z0-9_$]*[a-z][A-Za-z0-9_$]*)?$/)&&-1==="Array Number String Boolean Date Object Error Symbol".indexOf(e.value)&&("Math"===e.value?R("W063",e):l.option.newcap&&R("W064",e)),")"!==l.tokens.next.id)for(;Ct("spread"),i[i.length]=P(t,10),r+=1,","===l.tokens.next.id;)if(V({allowTrailing:!0}),")"===l.tokens.next.id){l.inES8()||R("W119",l.tokens.curr,"Trailing comma in arguments lists","8");break}return j(")"),"object"==typeof e&&(l.inES5()||"parseInt"!==e.value||1!==r||R("W065",l.tokens.curr),l.option.evil||("eval"===e.value||"Function"===e.value||"execScript"===e.value?R("W061",e):(!i[0]||"(string)"!==i[0].id||"setTimeout"!==e.value&&"setInterval"!==e.value)&&(!i[0]||"(string)"!==i[0].id||"."!==e.value||"window"!==e.left.value||"setTimeout"!==e.right&&"setInterval"!==e.right)||(R("W066",e),U(e,i[0]))),e.identifier||"."===e.id||"["===e.id||"=>"===e.id||"("===e.id||"&&"===e.id||"||"===e.id||"?"===e.id||"async"===e.id||l.inES6()&&e["(name)"]||R("W067",n)),n.left=e,n},155,!0).exps=!0,et("(",function(t,e){var n,r,i,o,a=l.tokens.curr,s=l.tokens.prev,A=!l.option.singleGroups,c=kt(1);if("function"===l.tokens.next.id&&(r=l.tokens.next.immed=!0),"=>"===c.value)return c.funct=Ft(t,{type:"arrow",parsedOpening:!0}),c;var u,g=[];if(")"!==l.tokens.next.id)for(;g.push(P(t,10)),","===l.tokens.next.id;)l.option.nocomma&&R("W127"),V();return j(")",this),l.option.immed&&g[0]&&"function"===g[0].id&&"("!==l.tokens.next.id&&"."!==l.tokens.next.id&&"["!==l.tokens.next.id&&R("W068",this),g.length?(g.length>1?((n=Object.create(l.syntax[","])).exprs=g,i=g[0],o=g[g.length-1]):(n=i=o=g[0],A||(A=a.beginsStmt&&("{"===n.id||r)||r&&(!Y()||"}"!==l.tokens.prev.id)||"=>"===n.id&&!Y()||"{"===n.id&&"=>"===s.id||At(n)&&"**"===l.tokens.next.id||"(number)"===n.type&&ne(c,".")&&/^\d+$/.test(n.value)||a.beginsStmt&&"="===n.id&&"{"===n.left.id)),n&&(!A&&((u=i).first||u.right||u.left||"yield"===u.id||"await"===u.id||n.exprs)&&(A=e>i.lbp||e>0&&e===i.lbp||!Y()&&o.rbp<l.tokens.next.lbp),A||R("W126",a),n.paren=!0),n):void 0}),function(t){var e=W(t,42);e.infix=!0,e.led=function(t,e){return z(l.tokens.prev,l.tokens.curr),this.left=e,this.right=Ft(t,{type:"arrow",loneArg:e}),this}}("=>"),at("[",function(t,e,n){var r,i;return l.option.asi&&ee(l.tokens.prev,[")","]"])&&l.tokens.prev.line!==K(l.tokens.curr)&&R("W014",l.tokens.curr,l.tokens.curr.id),(r=P(t&~h.noin,10))&&"(string)"===r.type&&(l.option.evil||"eval"!==r.value&&"execScript"!==r.value||ut(e,l)&&R("W061"),wt(r.value),!l.option.sub&&A.identifier.test(r.value)&&((i=l.syntax[r.value])?!_(t,i):"eval"!==r.value&&"arguments"!==r.value)&&R("W069",l.tokens.prev,r.value)),j("]",n),r&&"hasOwnProperty"===r.value&&"="===l.tokens.next.value&&R("W001"),n.left=e,n.right=r,n},160,!0),et("[",function(t){var e=Xt();if(e.isCompArray)return l.option.esnext||l.inMoz()||R("W118",l.tokens.curr,"array comprehension"),function(t){var e={exps:!0};l.funct["(comparray)"].stack();var n=!1;return"for"!==l.tokens.next.value&&(n=!0,l.inMoz()||R("W116",l.tokens.next,"for",l.tokens.next.value),l.funct["(comparray)"].setState("use"),e.right=P(t,10)),j("for"),"each"===l.tokens.next.value&&(j("each"),l.inMoz()||R("W118",l.tokens.curr,"for each")),j("("),l.funct["(comparray)"].setState("define"),e.left=P(t,130),r.includes(["in","of"],l.tokens.next.value)?j():F("E045",l.tokens.curr),l.funct["(comparray)"].setState("generate"),P(t,10),j(")"),"if"===l.tokens.next.value&&(j("if"),j("("),l.funct["(comparray)"].setState("filter"),P(t,10),j(")")),n||(l.funct["(comparray)"].setState("use"),e.right=P(t,10)),j("]"),l.funct["(comparray)"].unstack(),e}(t);if(e.isDestAssign)return this.destructAssign=Ht(t,{openingParsed:!0,assignment:!0}),this;var n=l.tokens.curr.line!==K(l.tokens.next);for(this.first=[],n&&(p+=l.option.indent,l.tokens.next.from===p+l.option.indent&&(p+=l.option.indent));"(end)"!==l.tokens.next.id;){for(;","===l.tokens.next.id;){if(!l.option.elision){if(l.inES5()){R("W128");do{j(",")}while(","===l.tokens.next.id);continue}R("W070")}j(",")}if("]"===l.tokens.next.id)break;if(Ct("spread"),this.first.push(P(t,10)),","!==l.tokens.next.id){l.option.trailingcomma&&l.inES5()&&N("W140",l.tokens.curr.line,l.tokens.curr.character);break}if(V({allowTrailing:!0}),"]"===l.tokens.next.id&&!l.inES5()){R("W070",l.tokens.curr);break}}return n&&(p-=l.option.indent),j("]",this),this}),function(t){t.nud=function(t){var e,n,r,i,o,a,s=!1,A=Object.create(null),c=!1;if((e=l.tokens.curr.line!==K(l.tokens.next))&&(p+=l.option.indent,l.tokens.next.from===p+l.option.indent&&(p+=l.option.indent)),Xt().isDestAssign)return this.destructAssign=Ht(t,{openingParsed:!0,assignment:!0}),this;for(l.inObjectBody=!0;"}"!==l.tokens.next.id;){if(a=l.tokens.next.value,!l.tokens.next.identifier||","!==H().id&&"}"!==H().id)if(":"===G().id||"get"!==a&&"set"!==a)if(Ct("spread"))l.inES9()||R("W119",l.tokens.next,"object spread property","9"),P(t,10);else{if("async"!==l.tokens.next.id||ee(G(),["(",":"])?c=!1:(l.inES8()||R("W119",l.tokens.next,"async functions","8"),c=!0,j(),J(l.tokens.curr)),"*"===l.tokens.next.value&&"(punctuator)"===l.tokens.next.type?(c&&!l.inES9()?R("W119",l.tokens.next,"async generators","9"):l.inES6()||R("W104",l.tokens.next,"generator functions","6"),j("*"),s=!0):s=!1,"["===l.tokens.next.id)r=te(t),l.nameStack.set(r);else if(l.nameStack.set(l.tokens.next),Zt(A,r=St(t),l.tokens.next),"string"!=typeof r)break;"("===l.tokens.next.value?(l.inES6()||R("W104",l.tokens.curr,"concise methods","6"),Ft(c?t|h.preAsync:t,{isMethod:!0,type:s?"generator":null})):(j(":"),P(t,10))}else j(a),l.inES5()||F("E034"),"["===l.tokens.next.id?r=te():(r=St(t))||l.inES6()||F("E035"),r&&$t(a,A,r,l.tokens.curr),o=l.tokens.next,i=(n=Ft(t,{isMethod:!0}))["(params)"],"get"===a&&r&&i.length?R("W076",o,i[0],r):"set"===a&&r&&1!==n["(metrics)"].arity&&R("W077",o,r);else l.inES6()||R("W104",l.tokens.next,"object short notation","6"),Zt(A,r=St(t,!0),l.tokens.next),P(t,10);if(wt(r),","!==l.tokens.next.id){l.option.trailingcomma&&l.inES5()&&N("W140",l.tokens.curr.line,l.tokens.curr.character);break}V({allowTrailing:!0,property:!0}),","===l.tokens.next.id?R("W070",l.tokens.curr):"}"!==l.tokens.next.id||l.inES5()||R("W070",l.tokens.curr)}return e&&(p-=l.option.indent),j("}",this),Gt(A),l.inObjectBody=!1,this},t.fud=function(){F("E036",l.tokens.curr)}}(X("{"));var Pt=Z("const",function(t){return Yt("const",this,t)});function Kt(){return"("===l.tokens.next.id&&l.inMoz()}Pt.exps=!0,Pt.declaration=!0;var zt,Jt=Z("let",function(t){return Yt("let",this,t)});Jt.nud=function(t,e){if(!Kt())return this.exps=!1,l.syntax["(identifier)"].nud.apply(this,arguments);l.funct["(scope)"].stack(),j("("),l.tokens.prev.fud(t),j(")"),P(t,e),l.funct["(scope)"].unstack()},Jt.meta={es5:!0,isFutureReservedWord:!1,strictOnly:!0},Jt.exps=!0,Jt.declaration=!0,Jt.useFud=function(t){var e=l.tokens.next;return!(this.line!==e.line&&!l.inES6())&&(e.identifier&&(!_(t,e)||"let"===e.id)||ee(e,["{","["])||Kt())},Z("var",function(t){var e,n,i,a,s=t&h.noin,A=t&h.export;for(this.first=[];;){var c=[];for(var u in r.includes(["{","["],l.tokens.next.value)?(e=Ht(t),n=!1):(e=[],(a=mt(t))&&e.push({id:a,token:l.tokens.curr}),n=!0),l.option.varstmt&&R("W132",this),e)e.hasOwnProperty(u)&&(u=e[u],l.funct["(global)"]&&!l.impliedClosure()&&(!1===E[u.id]?R("W079",u.token,u.id):!1===l.option.futurehostile&&(!l.inES5()&&!1===o.ecmaIdentifiers[5][u.id]||!l.inES6()&&!1===o.ecmaIdentifiers[6][u.id])&&R("W129",u.token,u.id)),u.id&&(l.funct["(scope)"].addlabel(u.id,{type:"var",token:u.token}),n&&A&&l.funct["(scope)"].setExported(u.id,u.token),c.push(u.token)));if("="===l.tokens.next.id&&(this.hasInitializer=!0,l.nameStack.set(l.tokens.curr),j("="),"="===G(0).id&&l.tokens.next.identifier&&((s||l.funct["(params)"])&&-1!==l.funct["(params)"].indexOf(l.tokens.next.value)||R("W120",l.tokens.next,l.tokens.next.value)),a=l.tokens.prev,(i=P(t,10))&&!l.funct["(loopage)"]&&i.identifier&&"undefined"===i.value&&R("W080",a,a.value),n||jt(c,i)),this.first=this.first.concat(c),","!==l.tokens.next.id)break;this.hasComma=!0,V()}return this}).exps=!0,$("function",function(t){var e=t&h.export,n=!1,r=t&h.preAsync,i="";r&&(i="async "),"*"===l.tokens.next.value&&(r&&!l.inES9()?R("W119",l.tokens.prev,"async generators","9"):r||l.inES6(!0)||R("W119",l.tokens.next,"function*","6"),j("*"),i+="generator ",n=!0),i+="function",f&&R("W082",l.tokens.curr);var o=It(t)?l.tokens.curr:null;o?(l.funct["(scope)"].addlabel(o.value,{type:i,token:l.tokens.curr,initialized:!0}),e&&l.funct["(scope)"].setExported(o.value,l.tokens.prev)):e||R("W025");var a=Ft(t,{name:o&&o.value,statement:this,type:n?"generator":null,ignoreLoopFunc:f}),s=a["(isStrict)"]&&!l.isStrict();return o&&("arguments"===a["(name)"]||"eval"===a["(name)"])&&s&&F("E008",o),"("===l.tokens.next.id&&l.tokens.next.line===l.tokens.curr.line&&F("E039"),this}).declaration=!0,et("function",function(t){var e=!1,n=t&h.preAsync;"*"===l.tokens.next.value&&(n&&!l.inES9()?R("W119",l.tokens.prev,"async generators","9"):n||l.inES6(!0)||R("W119",l.tokens.curr,"function*","6"),j("*"),e=!0);var r=It(n?t|h.async:t)?l.tokens.curr:null,i=Ft(t,{name:r&&r.value,type:e?"generator":null});return r&&("arguments"===i["(name)"]||"eval"===i["(name)"])&&i["(isStrict)"]&&F("E008",r),this}),$("if",function(t){var e=l.tokens.next;Ut(),l.condition=!0,j("(");var n=P(t,0);n||S("E041",this),Ot(n);var r=null;l.option.forin&&l.forinifcheckneeded&&(l.forinifcheckneeded=!1,r=l.forinifchecks[l.forinifchecks.length-1],"(punctuator)"===n.type&&"!"===n.value?r.type="(negative)":r.type="(positive)"),j(")",e),l.condition=!1;var i=Qt(t,!0,!0);return r&&"(negative)"===r.type&&i&&i[0]&&"(identifier)"===i[0].type&&"continue"===i[0].value&&(r.type="(negative-with-continue)"),"else"===l.tokens.next.id&&(j("else"),"if"===l.tokens.next.id||"switch"===l.tokens.next.id?vt(t):Qt(t,!0,!0)),this}),$("try",function(t){var e;function n(){if(j("catch"),j("("),l.funct["(scope)"].stack("catchparams"),ee(l.tokens.next,["[","{"])){var e=Ht(t);r.each(e,function(t){t.id&&l.funct["(scope)"].addParam(t.id,t,"exception")})}else"(identifier)"!==l.tokens.next.type?R("E030",l.tokens.next,l.tokens.next.value):l.funct["(scope)"].addParam(mt(t),l.tokens.curr,"exception");"if"===l.tokens.next.value&&(l.inMoz()||R("W118",l.tokens.curr,"catch filter"),j("if"),P(t,0)),j(")"),Qt(t,!1),l.funct["(scope)"].unstack()}for(Qt(t|h.tryClause,!0);"catch"===l.tokens.next.id;)Ut(),e&&!l.inMoz()&&R("W118",l.tokens.next,"multiple catch blocks"),n(),e=!0;return"finally"===l.tokens.next.id?(j("finally"),void Qt(t,!0)):(e||F("E021",l.tokens.next,"catch",l.tokens.next.value),this)}),$("while",function(t){var e=l.tokens.next;return l.funct["(breakage)"]+=1,l.funct["(loopage)"]+=1,Ut(),j("("),Ot(P(t,0)),j(")",e),Qt(t,!0,!0),l.funct["(breakage)"]-=1,l.funct["(loopage)"]-=1,this}).labelled=!0,$("with",function(t){var e=l.tokens.next;return l.isStrict()?F("E010",l.tokens.curr):l.option.withstmt||R("W085",l.tokens.curr),j("("),P(t,0),j(")",e),Qt(t,!0,!0),this}),$("switch",function(t){var e=l.tokens.next,n=!1,r=!1;for(l.funct["(breakage)"]+=1,j("("),Ot(P(t,0)),j(")",e),e=l.tokens.next,j("{"),l.funct["(scope)"].stack(),l.tokens.next.from===p&&(r=!0),r||(p+=l.option.indent),this.cases=[];;)switch(l.tokens.next.id){case"case":switch(l.funct["(verb)"]){case"yield":case"break":case"case":case"continue":case"return":case"switch":case"throw":break;case"default":l.option.leanswitch&&R("W145",l.tokens.next);break;default:l.tokens.curr.caseFallsThrough||R("W086",l.tokens.curr,"case")}j("case"),this.cases.push(P(t,0)),Ut(),n=!0,j(":"),l.funct["(verb)"]="case";break;case"default":switch(l.funct["(verb)"]){case"yield":case"break":case"continue":case"return":case"throw":break;case"case":l.option.leanswitch&&R("W145",l.tokens.curr);break;default:this.cases.length&&(l.tokens.curr.caseFallsThrough||R("W086",l.tokens.curr,"default"))}j("default"),n=!0,j(":"),l.funct["(verb)"]="default";break;case"}":return r||(p-=l.option.indent),j("}",e),l.funct["(scope)"].unstack(),l.funct["(breakage)"]-=1,void(l.funct["(verb)"]=void 0);case"(end)":return void F("E023",l.tokens.next,"}");default:if(p+=l.option.indent,n)switch(l.tokens.curr.id){case",":return void F("E040");case":":n=!1,Bt(t);break;default:return void F("E025",l.tokens.curr)}else{if(":"!==l.tokens.curr.id)return void F("E021",l.tokens.next,"case",l.tokens.next.value);j(":"),F("E024",l.tokens.curr,":"),Bt(t)}p-=l.option.indent}}).labelled=!0,Z("debugger",function(){return l.option.debug||R("W087",this),this}).exps=!0,function(){var t=Z("do",function(t){l.funct["(breakage)"]+=1,l.funct["(loopage)"]+=1,Ut(),this.first=Qt(t,!0,!0),j("while");var e=l.tokens.next;return j("("),Ot(P(t,0)),j(")",e),l.funct["(breakage)"]-=1,l.funct["(loopage)"]-=1,this});t.labelled=!0,t.exps=!0}(),$("for",function(t){var e,n,i,o,a,s,A,c,u=l.tokens.next,g=!1,d=!1,f=null;"each"===u.value&&(f=u,j("each"),l.inMoz()||R("W118",l.tokens.curr,"for each")),l.tokens.next.identifier&&"await"===l.tokens.next.value&&(j("await"),d=!0,t&h.async?l.inES9()||R("W119",l.tokens.curr,"asynchronous iteration","9"):F("E024",l.tokens.curr,"await")),Ut(),j("(");var p=G(),I=t|h.noin;if("var"===l.tokens.next.id)j("var"),i=(c=l.tokens.curr.fud(I)).hasComma?c:null,o=c.hasInitializer?c:null;else if("const"===l.tokens.next.id||"let"===l.tokens.next.id&&(p.identifier&&"in"!==p.id||ee(p,["{","["])))j(l.tokens.next.id),g=!0,l.funct["(scope)"].stack(),i=(c=l.tokens.curr.fud(I)).hasComma?c:null,o=c.hasInitializer?c:null;else if(!ne(l.tokens.next,";")){for(s=[];"in"!==l.tokens.next.value&&"of"!==l.tokens.next.value&&!ne(l.tokens.next,";");)ee(l.tokens.next,["{","["])?(Ht(I,{assignment:!0}).forEach(function(t){this.push(t.token)},s),ne(l.tokens.next,"=")&&(j("="),o=l.tokens.curr,P(I,10))):(A=P(I,10))&&("(identifier)"===A.type?s.push(A):ne(A,"=")&&(o=A,s.push(A))),ne(l.tokens.next,",")&&(j(","),i||(i=l.tokens.curr));o||i||s.forEach(function(t){l.funct["(scope)"].has(t.value)||R("W088",t,t.value)})}if(n=l.tokens.next,d&&"of"!==n.value&&F("E066",n),r.includes(["in","of"],n.value)){if("of"===n.value?(a=20,l.inES6()||R("W104",n,"for of","6")):a=0,i&&F("W133",i,n.value,"more than one ForBinding"),o&&F("W133",o,n.value,"initializer is forbidden"),!A||i||o||gt(t,A,n),j(n.value),P(t,a),j(")",u),"in"===n.value&&l.option.forin&&(l.forinifcheckneeded=!0,void 0===l.forinifchecks&&(l.forinifchecks=[]),l.forinifchecks.push({type:"(none)"})),l.funct["(breakage)"]+=1,l.funct["(loopage)"]+=1,e=Qt(t,!0,!0),"in"===n.value&&l.option.forin){if(l.forinifchecks&&l.forinifchecks.length>0){var C=l.forinifchecks.pop();(e&&e.length>0&&("object"!=typeof e[0]||"if"!==e[0].value)||"(positive)"===C.type&&e.length>1||"(negative)"===C.type)&&R("W089",this)}l.forinifcheckneeded=!1}l.funct["(breakage)"]-=1,l.funct["(loopage)"]-=1}else{if(f&&F("E045",f),J(l.tokens.curr),j(";"),c&&("const"!==c.value||c.hasInitializer||R("E012",c,c.first[0].value),c.first.forEach(function(t){l.funct["(scope)"].initialize(t.value)})),l.funct["(loopage)"]+=1,";"!==l.tokens.next.id&&Ot(P(t,0)),J(l.tokens.curr),j(";"),";"===l.tokens.next.id&&F("E021",l.tokens.next,")",";"),")"!==l.tokens.next.id)for(;P(t,0),","===l.tokens.next.id;)V();j(")",u),l.funct["(breakage)"]+=1,Qt(t,!0,!0),l.funct["(breakage)"]-=1,l.funct["(loopage)"]-=1}return g&&l.funct["(scope)"].unstack(),this}).labelled=!0,Z("break",function(){var t=l.tokens.next.value;return l.option.asi||J(this),";"===l.tokens.next.id||l.tokens.next.reach||l.tokens.curr.line!==K(l.tokens.next)?0===l.funct["(breakage)"]&&R("W052",l.tokens.next,this.value):(l.funct["(scope)"].funct.hasBreakLabel(t)||R("W090",l.tokens.next,t),this.first=l.tokens.next,j()),yt(this),this}).exps=!0,Z("continue",function(){var t=l.tokens.next.value;return 0!==l.funct["(breakage)"]&&l.funct["(loopage)"]||R("W052",l.tokens.next,this.value),l.option.asi||J(this),";"===l.tokens.next.id||l.tokens.next.reach||l.tokens.curr.line===K(l.tokens.next)&&(l.funct["(scope)"].funct.hasBreakLabel(t)||R("W090",l.tokens.next,t),this.first=l.tokens.next,j()),yt(this),this}).exps=!0,Z("return",function(t){return this.line===K(l.tokens.next)?";"===l.tokens.next.id||l.tokens.next.reach||(this.first=P(t,0),!this.first||"(punctuator)"!==this.first.type||"="!==this.first.value||this.first.paren||l.option.boss||N("W093",this.first.line,this.first.character),l.option.noreturnawait&&t&h.async&&!(t&h.tryClause)&&this.first.identifier&&"await"===this.first.value&&R("W146",this.first)):"(punctuator)"===l.tokens.next.type&&["[","{","+","-"].indexOf(l.tokens.next.value)>-1&&J(this),yt(this),this}).exps=!0,et("await",function(t){return t&h.async?(l.funct["(params)"]||F("E024",this,"await"),P(t,0),this):(this.exps=!1,l.syntax["(identifier)"].nud.apply(this,arguments))}).exps=!0,(zt=et("async",function(t,e){return this.isFunc(t)?(l.inES8()||R("W119",this,"async functions","8"),t|=h.preAsync,this.func=P(t,e),this.identifier=!1,this):(this.exps=!1,l.syntax["(identifier)"].nud.apply(this,arguments))})).meta={es5:!0,isFutureReservedWord:!0,strictOnly:!0},zt.isFunc=function(){var t=l.tokens.next;return this.line===t.line&&("function"===t.id||("("===t.id?"=>"===kt(0).id:!!t.identifier&&"=>"===G().id))},zt.useFud=zt.isFunc,zt.fud=function(t){return l.inES8()||R("W119",this,"async functions","8"),t|=h.preAsync,t|=h.initial,this.func=P(t,0),this.block=this.func.block,this.exps=this.func.exps,this},zt.exps=!0,delete zt.reserved,function(t){t.exps=!0,t.lbp=t.rbp=25,t.ltBoundary="after"}(et("yield",function(t){if(l.inMoz())return qt.call(this,t);var e=l.tokens.prev;return l.funct["(params)"]||F("E024",this,"yield"),!this.beginsStmt&&e.lbp>30&&!ee(e,["("])&&F("E061",this),l.inES6(!0)&&!l.funct["(generator)"]?"(catch)"===l.funct["(name)"]&&l.funct["(context)"]["(generator)"]||F("E046",l.tokens.curr,"yield"):l.inES6()||R("W104",l.tokens.curr,"yield","6"),l.funct["(generator)"]="yielded","*"===l.tokens.next.value&&j("*"),Y()||","===l.tokens.next.id||(l.tokens.next.nud?(z(l.tokens.curr,l.tokens.next),this.first=P(t,10),"(punctuator)"!==this.first.type||"="!==this.first.value||this.first.paren||l.option.boss||N("W093",this.first.line,this.first.character)):l.tokens.next.led&&","!==l.tokens.next.id&&F("W017",l.tokens.next)),this}));var qt=function(t){var e=l.tokens.prev;l.inES6(!0)&&!l.funct["(generator)"]&&("(catch)"===l.funct["(name)"]&&l.funct["(context)"]["(generator)"]||F("E046",l.tokens.curr,"yield")),l.funct["(generator)"]="yielded";var n=!1;return"*"===l.tokens.next.value&&(n=!0,j("*")),this.line===K(l.tokens.next)?((n||";"!==l.tokens.next.id&&!l.option.asi&&!l.tokens.next.reach&&l.tokens.next.nud)&&(z(l.tokens.curr,l.tokens.next),this.first=P(t,10),"(punctuator)"!==this.first.type||"="!==this.first.value||this.first.paren||l.option.boss||N("W093",this.first.line,this.first.character)),")"!==l.tokens.next.id&&(e.lbp>30||!e.assign&&!Y()||"yield"===e.id)&&F("E050",this)):l.option.asi||J(this),this};function Vt(t,e){return("call"!==t||!e["(async)"])&&(!("property"!==t||!e["(method)"])||(!("call"!==t||!e["(statement)"]||"class"!==e["(statement)"].id)||!!e["(arrow)"]&&Vt(t,e["(context)"])))}Z("throw",function(t){return J(this),this.first=P(t,20),yt(this),this}).exps=!0,Z("import",function(t){if(l.funct["(scope)"].block.isGlobal()||F("E053",l.tokens.curr,"Import"),l.inES6()||R("W119",l.tokens.curr,"import","6"),"(string)"===l.tokens.next.type)return j("(string)"),this;if(l.tokens.next.identifier){if(this.name=mt(t),l.funct["(scope)"].addlabel(this.name,{type:"import",initialized:!0,token:l.tokens.curr}),","!==l.tokens.next.value)return j("from"),j("(string)"),this;j(",")}if("*"===l.tokens.next.id)j("*"),j("as"),l.tokens.next.identifier&&(this.name=mt(t),l.funct["(scope)"].addlabel(this.name,{type:"import",initialized:!0,token:l.tokens.curr}));else for(j("{");;){if("}"===l.tokens.next.value){j("}");break}var e;if("default"===l.tokens.next.type?(e="default",j("default")):e=mt(t),"as"===l.tokens.next.value&&(j("as"),e=mt(t)),l.funct["(scope)"].addlabel(e,{type:"import",initialized:!0,token:l.tokens.curr}),","!==l.tokens.next.value){if("}"===l.tokens.next.value){j("}");break}F("E024",l.tokens.next,l.tokens.next.value);break}j(",")}return j("from"),j("(string)"),this}).exps=!0,Z("export",function(t){var e,n,r,i=!0;if(t|=h.export,l.inES6()||(R("W119",l.tokens.curr,"export","6"),i=!1),l.funct["(scope)"].block.isGlobal()||(F("E053",l.tokens.curr,"Export"),i=!1),"*"===l.tokens.next.value)return j("*"),j("from"),j("(string)"),this;if("default"===l.tokens.next.type){l.nameStack.set(l.tokens.next),j("default");var o=l.tokens.next.id;return"function"===o?(this.block=!0,j("function"),l.syntax.function.fud(t)):"async"===o&&"function"===G().id?(this.block=!0,j("async"),j("function"),l.syntax.function.fud(t|h.preAsync)):"class"===o?(this.block=!0,j("class"),l.syntax.class.fud(t)):(e=P(t,10)).identifier&&(n=e.value,l.funct["(scope)"].setExported(n,e)),this}if("{"===l.tokens.next.value){j("{");for(var a=[];!ne(l.tokens.next,"}");)l.tokens.next.identifier||F("E030",l.tokens.next,l.tokens.next.value),j(),a.push(l.tokens.curr),"as"===l.tokens.next.value&&(j("as"),l.tokens.next.identifier||F("E030",l.tokens.next,l.tokens.next.value),j()),ne(l.tokens.next,"}")||j(",");return j("}"),"from"===l.tokens.next.value?(j("from"),r=l.tokens.next,j("(string)")):i&&a.forEach(function(t){l.funct["(scope)"].setExported(t.value,t)}),0===a.length&&(r?R("W142",this,"export",r.value):R("W141",this,"export")),this}return"var"===l.tokens.next.id?(j("var"),l.tokens.curr.fud(t)):"let"===l.tokens.next.id?(j("let"),l.tokens.curr.fud(t)):"const"===l.tokens.next.id?(j("const"),l.tokens.curr.fud(t)):"function"===l.tokens.next.id?(this.block=!0,j("function"),l.syntax.function.fud(t)):"async"===l.tokens.next.id&&"function"===G().id?(this.block=!0,j("async"),j("function"),l.syntax.function.fud(t|h.preAsync)):"class"===l.tokens.next.id?(this.block=!0,j("class"),l.syntax.class.fud(t)):F("E024",l.tokens.next,l.tokens.next.value),this}).exps=!0;var Wt=function(){var t=l.tokens.next;return ee(t,["[","."])?Vt("property",l.funct)||F("E063",this):ne(t,"(")?Vt("call",l.funct)||F("E064",this):F("E024",t,t.value||t.id),this};it("abstract"),it("boolean"),it("byte"),it("char"),it("double"),it("enum",{es5:!0}),it("export",{es5:!0}),it("extends",{es5:!0}),it("final"),it("float"),it("goto"),it("implements",{es5:!0,strictOnly:!0}),it("import",{es5:!0}),it("int"),it("interface",{es5:!0,strictOnly:!0}),it("long"),it("native"),it("package",{es5:!0,strictOnly:!0}),it("private",{es5:!0,strictOnly:!0}),it("protected",{es5:!0,strictOnly:!0}),it("public",{es5:!0,strictOnly:!0}),it("short"),it("static",{es5:!0,strictOnly:!0}),it("synchronized"),it("transient"),it("volatile");var Xt=function(){var t,e,n,r=-1,i=0,o={};ee(l.tokens.curr,["[","{"])&&(i+=1);do{if(n=-1===r?l.tokens.curr:t,t=-1===r?l.tokens.next:G(r),e=G(r+1),r+=1,ee(t,["[","{"])?i+=1:ee(t,["]","}"])&&(i-=1),1===i&&t.identifier&&"for"===t.value&&!ne(n,".")){o.isCompArray=!0,o.notJson=!0;break}if(0===i&&ee(t,["}","]"])){if("="===e.value){o.isDestAssign=!0,o.notJson=!0;break}if("."===e.value){o.notJson=!0;break}}ne(t,";")&&(o.notJson=!0)}while(i>0&&"(end)"!==t.id);return o};function Zt(t,e,n,r,i,o){n.identifier&&(e=n.value);var a=e;if(r&&i&&(a="static "+e),t[a]&&"__proto__"!==e&&!o){var s=["key","class method","static class method"];s=s[(r||!1)+(i||!1)],R("W075",l.tokens.next,s,e)}else t[a]=Object.create(null);t[a].basic=!0,t[a].basictkn=n}function $t(t,e,n,r,i,o){var a="get"===t?"getterToken":"setterToken",s=n;if(l.tokens.curr.accessorType=t,l.nameStack.set(r),i&&o&&(s="static "+n),e[s]){if((e[s].basic||e[s][a])&&"__proto__"!==n){var A="";i?(o&&(A+="static "),A+=t+"ter method"):A="key",R("W075",l.tokens.next,A,n)}}else e[s]=Object.create(null);e[s][a]=r,o&&(e[s].static=!0)}function te(t){j("["),l.tokens.curr.delim=!0,l.tokens.curr.lbp=0,l.inES6()||R("W119",l.tokens.curr,"computed property names","6");var e=P(t&~h.noin,10);return j("]"),e}function ee(t,e){return"(punctuator)"===t.type&&r.includes(e,t.value)}function ne(t,e){return"(punctuator)"===t.type&&t.value===e}function re(t){var e=Xt();e.notJson?(!l.inES6()&&e.isDestAssign&&R("W104",l.tokens.curr,"destructuring assignment","6"),Bt(t)):(l.option.laxbreak=!0,l.jsonMode=!0,function t(){function e(){var e={},n=l.tokens.next;if(j("{"),"}"!==l.tokens.next.id)for(;;){if("(end)"===l.tokens.next.id)F("E026",l.tokens.next,n.line);else{if("}"===l.tokens.next.id){R("W094",l.tokens.curr);break}","===l.tokens.next.id?F("E028",l.tokens.next):"(string)"!==l.tokens.next.id&&R("W095",l.tokens.next,l.tokens.next.value)}if(!0===e[l.tokens.next.value]?R("W075",l.tokens.next,"key",l.tokens.next.value):"__proto__"===l.tokens.next.value&&!l.option.proto||"__iterator__"===l.tokens.next.value&&!l.option.iterator?R("W096",l.tokens.next,l.tokens.next.value):e[l.tokens.next.value]=!0,j(),j(":"),t(),","!==l.tokens.next.id)break;j(",")}j("}")}switch(l.tokens.next.id){case"{":!function e(){var t={},e=l.tokens.next;if(j("{"),"}"!==l.tokens.next.id)for(;;){if("(end)"===l.tokens.next.id)F("E026",l.tokens.next,e.line);else{if("}"===l.tokens.next.id){R("W094",l.tokens.curr);break}","===l.tokens.next.id?F("E028",l.tokens.next):"(string)"!==l.tokens.next.id&&R("W095",l.tokens.next,l.tokens.next.value)}if(!0===t[l.tokens.next.value]?R("W075",l.tokens.next,"key",l.tokens.next.value):"__proto__"===l.tokens.next.value&&!l.option.proto||"__iterator__"===l.tokens.next.value&&!l.option.iterator?R("W096",l.tokens.next,l.tokens.next.value):t[l.tokens.next.value]=!0,j(),j(":"),function t(){function e(){var e={},n=l.tokens.next;if(j("{"),"}"!==l.tokens.next.id)for(;;){if("(end)"===l.tokens.next.id)F("E026",l.tokens.next,n.line);else{if("}"===l.tokens.next.id){R("W094",l.tokens.curr);break}","===l.tokens.next.id?F("E028",l.tokens.next):"(string)"!==l.tokens.next.id&&R("W095",l.tokens.next,l.tokens.next.value)}if(!0===e[l.tokens.next.value]?R("W075",l.tokens.next,"key",l.tokens.next.value):"__proto__"===l.tokens.next.value&&!l.option.proto||"__iterator__"===l.tokens.next.value&&!l.option.iterator?R("W096",l.tokens.next,l.tokens.next.value):e[l.tokens.next.value]=!0,j(),j(":"),t(),","!==l.tokens.next.id)break;j(",")}j("}")}switch(l.tokens.next.id){case"{":e();break;case"[":!function(){var e=l.tokens.next;if(j("["),"]"!==l.tokens.next.id)for(;;){if("(end)"===l.tokens.next.id)F("E027",l.tokens.next,e.line);else{if("]"===l.tokens.next.id){R("W094",l.tokens.curr);break}","===l.tokens.next.id&&F("E028",l.tokens.next)}if(t(),","!==l.tokens.next.id)break;j(",")}j("]")}();break;case"true":case"false":case"null":case"(number)":case"(string)":j();break;case"-":j("-"),j("(number)");break;default:F("E003",l.tokens.next)}}(),","!==l.tokens.next.id)break;j(",")}j("}")}();break;case"[":!function(){var e=l.tokens.next;if(j("["),"]"!==l.tokens.next.id)for(;;){if("(end)"===l.tokens.next.id)F("E027",l.tokens.next,e.line);else{if("]"===l.tokens.next.id){R("W094",l.tokens.curr);break}","===l.tokens.next.id&&F("E028",l.tokens.next)}if(t(),","!==l.tokens.next.id)break;j(",")}j("]")}();break;case"true":case"false":case"null":case"(number)":case"(string)":j();break;case"-":j("-"),j("(number)");break;default:F("E003",l.tokens.next)}}())}var ie=function(){var t,e=function(){this.mode="use",this.variables=[]},n=[];function i(e){return 0===t.variables.filter(function(t){if(t.value===e&&!t.undef)return!0===t.unused&&(t.unused=!1),e}).length}return{stack:function(){t=new e,n.push(t)},unstack:function(){t.variables.filter(function(t){t.unused&&R("W098",t.token,t.token.raw_text||t.value),t.undef&&l.funct["(scope)"].block.use(t.value,t.token)}),n.splice(-1,1),t=n[n.length-1]},setState:function(e){r.includes(["use","define","generate","filter"],e)&&(t.mode=e)},check:function(e){if(t)return t&&"use"===t.mode?(i(e)&&t.variables.push({token:l.tokens.curr,value:e,undef:!0,unused:!1}),!0):t&&"define"===t.mode?(function(e){return 0!==t.variables.filter(function(t){if(t.value===e)return t.undef=!1,e}).length}(e)||t.variables.push({token:l.tokens.curr,value:e,undef:!1,unused:!0}),!0):t&&"generate"===t.mode?(l.funct["(scope)"].block.use(e,l.tokens.curr),!0):!(!t||"filter"!==t.mode)&&(i(e)&&l.funct["(scope)"].block.use(e,l.tokens.curr),!0)}}};function oe(){function t(){var t={},e=l.tokens.next;if(j("{"),"}"!==l.tokens.next.id){for(;;){if(l.tokens.next.id==="(end)"){F("E026",l.tokens.next,e.line)}else if(l.tokens.next.id==="}"){R("W094",l.tokens.curr);break}else if(l.tokens.next.id===","){F("E028",l.tokens.next)}else if(l.tokens.next.id!=="(string)"){R("W095",l.tokens.next,l.tokens.next.value)}if(t[l.tokens.next.value]===true){R("W075",l.tokens.next,"key",l.tokens.next.value)}else if(l.tokens.next.value==="__proto__"&&!l.option.proto||l.tokens.next.value==="__iterator__"&&!l.option.iterator){R("W096",l.tokens.next,l.tokens.next.value)}else{t[l.tokens.next.value]=true}j();j(":");oe();if(l.tokens.next.id!==","){break}j(",")}}j("}")}function e(){var t=l.tokens.next;if(j("["),"]"!==l.tokens.next.id){for(;;){if(l.tokens.next.id==="(end)"){F("E027",l.tokens.next,t.line)}else if(l.tokens.next.id==="]"){R("W094",l.tokens.curr);break}else if(l.tokens.next.id===","){F("E028",l.tokens.next)}oe();if(l.tokens.next.id!==","){break}j(",")}}j("]")}switch(l.tokens.next.id){case"{":t();break;case"[":e();break;case"true":case"false":case"null":case"(number)":case"(string)":j();break;case"-":j("-");j("(number)");break;default:F("E003",l.tokens.next)}}var ae=function(t){return t.replace(/[-\/\\^$*+?.()|[\]{}]/g,"\\$&")},se=function(i,a,A){var c,u,h,B,x,_;a=r.clone(a),l.reset(),x=l.option,_=l.ignored,a&&a.scope?d.scope=a.scope:(d.errors=[],d.internals=[],d.blacklist={},d.scope="(main)"),M(E=Object.create(null),o.ecmaIdentifiers[3]),M(E,o.reservedVars),e=Object.create(null);var D=Object.create(null);function L(t,e){t&&(Array.isArray(t)||"object"!=typeof t||(t=Object.keys(t)),t.forEach(e))}if(a)for(L([a.predef,a.globals],function(t){L(t,function(e){var n,r;"-"===e[0]?(n=e.slice(1),d.blacklist[n]=n,delete E[n]):(r=Object.getOwnPropertyDescriptor(t,e),E[e]=!!r&&r.value)})}),L(a.exported||null,function(t){D[t]=!0}),delete a.predef,delete a.exported,B=Object.keys(a),c=0;c<B.length;c++)if(/^-W\d{3}$/g.test(B[c]))_[B[c].slice(1)]=!0;else{var U=B[c];x[U]=a[U]}l.option=x,l.ignored=_,l.option.indent=l.option.indent||4,l.option.maxerr=l.option.maxerr||50,p=1;var O,G,H=g(l,E,D,e);if(H.on("warning",function(t){R.apply(null,[t.code,t.token].concat(t.data))}),H.on("error",function(t){F.apply(null,[t.code,t.token].concat(t.data))}),l.funct=Lt("(global)",null,{"(global)":!0,"(scope)":H,"(comparray)":ie(),"(metrics)":Tt(l.tokens.next)}),n=[l.funct],v=[],m={},y=null,f=!1,I=[],O=i,"[object String]"!==Object.prototype.toString.call(O)&&!Array.isArray(i))return T("E004",0),!1;for(G in t={get isJSON(){return l.jsonMode},getOption:function(t){return l.option[t]||null},getCache:function(t){return l.cache[t]},setCache:function(t,e){l.cache[t]=e},warn:function(t,e){N.apply(null,[t,e.line,e.char].concat(e.data))},on:function(t,e){t.split(" ").forEach(function(t){Q.on(t,e)}.bind(this))}},Q.removeAllListeners(),(b||[]).forEach(function(e){e(t)}),l.tokens.prev=l.tokens.curr=l.tokens.next=l.syntax["(begin)"],a&&a.ignoreDelimiters&&(Array.isArray(a.ignoreDelimiters)||(a.ignoreDelimiters=[a.ignoreDelimiters]),a.ignoreDelimiters.forEach(function(t){t.start&&t.end&&(u=ae(t.start)+"[\\s\\S]*?"+ae(t.end),h=new RegExp(u,"ig"),i=i.replace(h,function(t){return t.replace(/./g," ")}))})),(C=new s(i)).on("warning",function(t){N.apply(null,[t.code,t.line,t.character].concat(t.data))}),C.on("error",function(t){T.apply(null,[t.code,t.line,t.character].concat(t.data))}),C.on("fatal",function(t){S("E041",t)}),C.on("Identifier",function(t){Q.emit("Identifier",t)}),C.on("String",function(t){Q.emit("String",t)}),C.on("Number",function(t){Q.emit("Number",t)}),a)r.has(a,G)&&w(G,!0,l.tokens.curr);if(a)for(G in a.unstable)r.has(a.unstable,G)&&w(G,!1,l.tokens.curr);try{switch(k(),M(E,A||{}),V.first=!0,j(),l.tokens.next.id){case"{":case"[":re(0);break;default:bt(),l.directive["use strict"]&&(l.allowsGlobalUsd()||R("W097",l.tokens.prev)),Bt(0)}"(end)"!==l.tokens.next.id&&S("E041",l.tokens.curr),l.funct["(scope)"].unstack()}catch(t){if(!t||"JSHintError"!==t.name)throw t;var Y=l.tokens.next||{};d.errors.push({scope:"(main)",raw:t.raw,code:t.code,reason:t.reason,line:t.line||Y.line,character:t.character||Y.from})}return"(main)"===d.scope&&function(t,e,n){var r,i,o,a;for(i=0;i<t.length;i+=1)for(a=t[i],e.scope=a.elem,r=d.errors.length,se(a.code,e,n),o=r;o<d.errors.length;o+=1)d.errors[o].line+=a.token.line-1}(d.internals,a||{},A),0===d.errors.length};return se.addModule=function(t){b.push(t)},se.addModule(c.register),se.data=function(){var t,e,r,i,o,a={functions:[],options:l.option};se.errors.length&&(a.errors=se.errors),l.jsonMode&&(a.json=!0);var s=l.funct["(scope)"].getImpliedGlobals();for(s.length>0&&(a.implieds=s),v.length>0&&(a.urls=v),(o=l.funct["(scope)"].getUsedOrDefinedGlobals()).length>0&&(a.globals=o),r=1;r<n.length;r+=1)e=n[r],(t={}).name=e["(name)"],t.param=e["(params)"],t.line=e["(line)"],t.character=e["(character)"],t.last=e["(last)"],t.lastcharacter=e["(lastcharacter)"],t.metrics={complexity:e["(metrics)"].ComplexityCount,parameters:e["(metrics)"].arity,statements:e["(metrics)"].statementCount},a.functions.push(t);var A=l.funct["(scope)"].getUnuseds();for(i in A.length>0&&(a.unused=A),m)if("number"==typeof m[i]){a.member=m;break}return a},se.jshint=se,se}());"object"==typeof n&&n&&(n.JSHINT=d)},{"./lex.js":17,"./messages.js":18,"./options.js":20,"./prod-params.js":21,"./reg.js":22,"./scope-manager.js":23,"./state.js":24,"./style.js":25,"./vars.js":26,"console-browserify":14,events:9,lodash:16}]},{},[]),i=o("jshint").JSHINT,e&&(e.JSHINT=i)}()}).call(this,n(37))},function(t,e,n){!function(t){"use strict";t.registerHelper("lint","javascript",function(e,n){if(!window.JSHINT)return window.console&&window.console.error("Error: window.JSHINT not defined, CodeMirror JavaScript linting cannot run."),[];n.indent||(n.indent=1),JSHINT(e,n,n.globals);var r=JSHINT.data().errors,i=[];return r&&function(e,n){for(var r=0;r<e.length;r++){var i=e[r];if(i){if(i.line<=0){window.console&&window.console.warn("Cannot display JSHint error (invalid line "+i.line+")",i);continue}var o=i.character-1,a=o+1;if(i.evidence){var s=i.evidence.substring(o).search(/.\b/);s>-1&&(a+=s)}var A={message:i.reason,severity:i.code&&i.code.startsWith("W")?"warning":"error",from:t.Pos(i.line-1,o),to:t.Pos(i.line-1,a)};n.push(A)}}}(r,i),i})}(n(15))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.InitAddon=function(t){let e=t.Pos;function n(t,e){for(let n=0,r=t.length;n<r;++n)e(t[n])}function r(n,r,i,o){let a=n.getCursor(),s=i(n,a);if(/\b(?:string|comment)\b/.test(s.type))return;let A=t.innerMode(n.getMode(),s.state);if("json"===A.mode.helperType)return;s.state=A.state,/^[\w$_]*$/.test(s.string)?s.end>a.ch&&(s.end=a.ch,s.string=s.string.slice(0,a.ch-s.start)):s={start:a.ch,end:a.ch,string:"",state:s.state,type:"."===s.string?"property":null};let l=void 0,u=s;for(;"property"===u.type;){if("."!==(u=i(n,e(a.line,u.start))).string)return;u=i(n,e(a.line,u.start)),l||(l=[]),l.push(u)}return{list:c(s,l,r,o),from:e(a.line,s.start),to:e(a.line,s.end)}}function i(t,e){let n=t.getTokenAt(e);return e.ch===n.start+1&&"."===n.string.charAt(0)?(n.end=n.start,n.string=".",n.type="property"):/^\.[\w$_]*$/.test(n.string)&&(n.type="property",n.start++,n.string=n.string.replace(/\./,"")),n}t.registerHelper("hint","javascript",function(t,e){return r(t,A,function(t,e){return t.getTokenAt(e)},e)}),t.registerHelper("hint","coffeescript",function(t,e){return r(t,l,i,e)});const o="charAt charCodeAt indexOf lastIndexOf substring substr slice trim trimLeft trimRight toUpperCase toLowerCase split concat match replace search".split(" "),a="length concat join splice push pop shift unshift slice reverse sort indexOf lastIndexOf every some filter forEach map reduce reduceRight ".split(" "),s="prototype apply call bind".split(" "),A="break case catch class const continue debugger default delete do else export extends false finally for function if in import instanceof new null return super switch this throw true try typeof var void while with yield".split(" "),l="and break catch class continue delete do else extends false finally for if ininstanceof isnt new no not null of off on or return switch then throw true try typeof until void while with yes".split(" ");function c(t,e,r,i){let A=[],l=t.string,c=i&&i.globalScope||window;function u(t){0!==t.lastIndexOf(l,0)||function(t,e){if(!Array.prototype.indexOf){let n=t.length;for(;n--;)if(t[n]===e)return!0;return!1}return-1!==t.indexOf(e)}(A,t)||A.push(t)}function g(t){"string"==typeof t?n(o,u):t instanceof Array?n(a,u):t instanceof Function&&n(s,u),function(t,e){if(Object.getOwnPropertyNames&&Object.getPrototypeOf)for(let n=t;n;n=Object.getPrototypeOf(n))Object.getOwnPropertyNames(n).forEach(e);else for(let n in t)e(n)}(t,u)}if(e&&e.length){let t,n=e.pop(),r=n.type&&0===n.type.indexOf("variable"),o=n.type&&0===n.type.indexOf("keyword")&&"this"===n.string;for(r||o?(i&&i.additionalContext&&(t=i.additionalContext[n.string]),i&&!1===i.useGlobalScope||(t=t||c[n.string])):"string"===n.type?t="":"atom"===n.type?t=1:"function"===n.type&&(null==c.jQuery||"$"!==n.string&&"jQuery"!==n.string||"function"!=typeof c.jQuery?null!=c._&&"_"===n.string&&"function"==typeof c._&&(t=c._()):t=c.jQuery());null!=t&&e.length;)t=t[e.pop().string];null!=t&&g(t)}else{for(let e=t.state.localVars;e;e=e.next)u(e.name);for(let e=t.state.globalVars;e;e=e.next)u(e.name);i&&!1===i.useGlobalScope||g(c),n(r,u)}return A}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(48),i=n(1324),o=n(1325);e.outputPartsPlugin={id:"@mavenomics/default-parts:output",autoStart:!0,requires:[r.IPartFactory],activate:(t,e)=>{e.registerPart("RowViewPart",i.RowViewPart),e.registerPart("LabelPart",o.LabelPart)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(48),o=n(234),a=n(0);e.RowViewPart=class extends i.ReactPart{static GetMetadata(){const t=super.GetMetadata();return t.description="Display a single row from a table",t.remarks="This part will _only_ display the first row of the given table.",t.addOption("Table",r.Types.Table,new o.Table),t}renderReact(t){const e=t.get("Table");if(!(e instanceof o.Table))return a.createElement("div",null);const n=e.rows[0],r=[];for(const t of e.columnNames){const e=n.getValue(t);null!=e&&r.push(a.createElement("tr",{id:t},a.createElement("td",null,t),a.createElement("td",null,e)))}return a.createElement("table",null,a.createElement("thead",null,a.createElement("tr",null,a.createElement("th",null,"Column"),a.createElement("th",null,"Value"))),r)}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(6),o=n(48);e.LabelPart=class extends o.Part{constructor(){super(...arguments),this.content=new i.Widget}static GetMetadata(){const t=super.GetMetadata();return t.description="Display a value or statistic, with a caption to contextualize it",t.remarks="\nSometimes you may want to echo a global, selection, or some other state from a\ndashboard. Or maybe you want to display a statistic or result. LabelParts are\nconvenient for this.\n\n> #### Note\n>\n> The LabelPart Value is typed as _`Any`_, which means that the inline editor\n> on the Part Properties dialog won't accept an unquoted string. Use quotes if\n> you intend to display a static string as the value.\n",t.addOption("Value",r.Types.Any,null),t.addOption("Caption",r.Types.String,""),t}async initialize(){this.content.addClass("m-LabelPart"),this.layout.addWidget(this.content)}async render(t){const e=""+t.get("Caption"),n=""+t.get("Value");this.content.node.innerHTML=`<p style="text-align:center;">\n            ${e}\n        </p>\n        <h1 style="text-align:center;">${n}</h1>`}}},function(module,exports,__webpack_require__){var require,t;window,t=function(__WEBPACK_EXTERNAL_MODULE__25__,__WEBPACK_EXTERNAL_MODULE__26__,__WEBPACK_EXTERNAL_MODULE__28__,__WEBPACK_EXTERNAL_MODULE__41__,__WEBPACK_EXTERNAL_MODULE__64__,__WEBPACK_EXTERNAL_MODULE__91__,__WEBPACK_EXTERNAL_MODULE__93__,__WEBPACK_EXTERNAL_MODULE__154__,__WEBPACK_EXTERNAL_MODULE__156__,__WEBPACK_EXTERNAL_MODULE__284__,__WEBPACK_EXTERNAL_MODULE__285__){return function(t){var e={};function n(r){if(e[r])return e[r].exports;var i=e[r]={i:r,l:!1,exports:{}};return t[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=t,n.c=e,n.d=function(t,e,r){n.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:r})},n.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},n.t=function(t,e){if(1&e&&(t=n(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var i in t)n.d(r,i,function(e){return t[e]}.bind(null,i));return r},n.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(e,"a",e),e},n.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},n.p="",n(n.s=129)}([function(t,e,n){"use strict";n.r(e);var r="http://www.w3.org/1999/xhtml",i={svg:"http://www.w3.org/2000/svg",xhtml:r,xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"},o=function(t){var e=t+="",n=e.indexOf(":");return n>=0&&"xmlns"!==(e=t.slice(0,n))&&(t=t.slice(n+1)),i.hasOwnProperty(e)?{space:i[e],local:t}:t},a=function(t){var e=o(t);return(e.local?function(t){return function(){return this.ownerDocument.createElementNS(t.space,t.local)}}:function(t){return function(){var e=this.ownerDocument,n=this.namespaceURI;return n===r&&e.documentElement.namespaceURI===r?e.createElement(t):e.createElementNS(n,t)}})(e)};function s(){}var A=function(t){return null==t?s:function(){return this.querySelector(t)}};function l(){return[]}var c=function(t){return null==t?l:function(){return this.querySelectorAll(t)}},u=function(t){return function(){return this.matches(t)}},g=function(t){return new Array(t.length)};function h(t,e){this.ownerDocument=t.ownerDocument,this.namespaceURI=t.namespaceURI,this._next=null,this._parent=t,this.__data__=e}h.prototype={constructor:h,appendChild:function(t){return this._parent.insertBefore(t,this._next)},insertBefore:function(t,e){return this._parent.insertBefore(t,e)},querySelector:function(t){return this._parent.querySelector(t)},querySelectorAll:function(t){return this._parent.querySelectorAll(t)}};var d="$";function f(t,e,n,r,i,o){for(var a,s=0,A=e.length,l=o.length;s<l;++s)(a=e[s])?(a.__data__=o[s],r[s]=a):n[s]=new h(t,o[s]);for(;s<A;++s)(a=e[s])&&(i[s]=a)}function p(t,e,n,r,i,o,a){var s,A,l,c={},u=e.length,g=o.length,f=new Array(u);for(s=0;s<u;++s)(A=e[s])&&(f[s]=l=d+a.call(A,A.__data__,s,e),l in c?i[s]=A:c[l]=A);for(s=0;s<g;++s)(A=c[l=d+a.call(t,o[s],s,o)])?(r[s]=A,A.__data__=o[s],c[l]=null):n[s]=new h(t,o[s]);for(s=0;s<u;++s)(A=e[s])&&c[f[s]]===A&&(i[s]=A)}function I(t,e){return t<e?-1:t>e?1:t>=e?0:NaN}function C(t){return function(){this.removeAttribute(t)}}function m(t){return function(){this.removeAttributeNS(t.space,t.local)}}function y(t,e){return function(){this.setAttribute(t,e)}}function E(t,e){return function(){this.setAttributeNS(t.space,t.local,e)}}function v(t,e){return function(){var n=e.apply(this,arguments);null==n?this.removeAttribute(t):this.setAttribute(t,n)}}function B(t,e){return function(){var n=e.apply(this,arguments);null==n?this.removeAttributeNS(t.space,t.local):this.setAttributeNS(t.space,t.local,n)}}var b=function(t){return t.ownerDocument&&t.ownerDocument.defaultView||t.document&&t||t.defaultView};function Q(t){return function(){this.style.removeProperty(t)}}function w(t,e,n){return function(){this.style.setProperty(t,e,n)}}function x(t,e,n){return function(){var r=e.apply(this,arguments);null==r?this.style.removeProperty(t):this.style.setProperty(t,r,n)}}function _(t,e){return t.style.getPropertyValue(e)||b(t).getComputedStyle(t,null).getPropertyValue(e)}function D(t){return function(){delete this[t]}}function M(t,e){return function(){this[t]=e}}function k(t,e){return function(){var n=e.apply(this,arguments);null==n?delete this[t]:this[t]=n}}function S(t){return t.trim().split(/^|\s+/)}function L(t){return t.classList||new R(t)}function R(t){this._node=t,this._names=S(t.getAttribute("class")||"")}function N(t,e){for(var n=L(t),r=-1,i=e.length;++r<i;)n.add(e[r])}function F(t,e){for(var n=L(t),r=-1,i=e.length;++r<i;)n.remove(e[r])}function T(t){return function(){N(this,t)}}function U(t){return function(){F(this,t)}}function O(t,e){return function(){(e.apply(this,arguments)?N:F)(this,t)}}function G(){this.textContent=""}function H(t){return function(){this.textContent=t}}function j(t){return function(){var e=t.apply(this,arguments);this.textContent=null==e?"":e}}function Y(){this.innerHTML=""}function P(t){return function(){this.innerHTML=t}}function K(t){return function(){var e=t.apply(this,arguments);this.innerHTML=null==e?"":e}}function z(){this.nextSibling&&this.parentNode.appendChild(this)}function J(){this.previousSibling&&this.parentNode.insertBefore(this,this.parentNode.firstChild)}function q(){return null}function V(){var t=this.parentNode;t&&t.removeChild(this)}function W(){return this.parentNode.insertBefore(this.cloneNode(!1),this.nextSibling)}function X(){return this.parentNode.insertBefore(this.cloneNode(!0),this.nextSibling)}R.prototype={add:function(t){this._names.indexOf(t)<0&&(this._names.push(t),this._node.setAttribute("class",this._names.join(" ")))},remove:function(t){var e=this._names.indexOf(t);e>=0&&(this._names.splice(e,1),this._node.setAttribute("class",this._names.join(" ")))},contains:function(t){return this._names.indexOf(t)>=0}};var Z={},$=null;function tt(t,e,n){return t=et(t,e,n),function(e){var n=e.relatedTarget;n&&(n===this||8&n.compareDocumentPosition(this))||t.call(this,e)}}function et(t,e,n){return function(r){var i=$;$=r;try{t.call(this,this.__data__,e,n)}finally{$=i}}}function nt(t){return t.trim().split(/^|\s+/).map(function(t){var e="",n=t.indexOf(".");return n>=0&&(e=t.slice(n+1),t=t.slice(0,n)),{type:t,name:e}})}function rt(t){return function(){var e=this.__on;if(e){for(var n,r=0,i=-1,o=e.length;r<o;++r)n=e[r],t.type&&n.type!==t.type||n.name!==t.name?e[++i]=n:this.removeEventListener(n.type,n.listener,n.capture);++i?e.length=i:delete this.__on}}}function it(t,e,n){var r=Z.hasOwnProperty(t.type)?tt:et;return function(i,o,a){var s,A=this.__on,l=r(e,o,a);if(A)for(var c=0,u=A.length;c<u;++c)if((s=A[c]).type===t.type&&s.name===t.name)return this.removeEventListener(s.type,s.listener,s.capture),this.addEventListener(s.type,s.listener=l,s.capture=n),void(s.value=e);this.addEventListener(t.type,l,n),s={type:t.type,name:t.name,value:e,listener:l,capture:n},A?A.push(s):this.__on=[s]}}function ot(t,e,n,r){var i=$;t.sourceEvent=$,$=t;try{return e.apply(n,r)}finally{$=i}}function at(t,e,n){var r=b(t),i=r.CustomEvent;"function"==typeof i?i=new i(e,n):(i=r.document.createEvent("Event"),n?(i.initEvent(e,n.bubbles,n.cancelable),i.detail=n.detail):i.initEvent(e,!1,!1)),t.dispatchEvent(i)}"undefined"!=typeof document&&("onmouseenter"in document.documentElement||(Z={mouseenter:"mouseover",mouseleave:"mouseout"}));var st=[null];function At(t,e){this._groups=t,this._parents=e}function lt(){return new At([[document.documentElement]],st)}At.prototype=lt.prototype={constructor:At,select:function(t){"function"!=typeof t&&(t=A(t));for(var e=this._groups,n=e.length,r=new Array(n),i=0;i<n;++i)for(var o,a,s=e[i],l=s.length,c=r[i]=new Array(l),u=0;u<l;++u)(o=s[u])&&(a=t.call(o,o.__data__,u,s))&&("__data__"in o&&(a.__data__=o.__data__),c[u]=a);return new At(r,this._parents)},selectAll:function(t){"function"!=typeof t&&(t=c(t));for(var e=this._groups,n=e.length,r=[],i=[],o=0;o<n;++o)for(var a,s=e[o],A=s.length,l=0;l<A;++l)(a=s[l])&&(r.push(t.call(a,a.__data__,l,s)),i.push(a));return new At(r,i)},filter:function(t){"function"!=typeof t&&(t=u(t));for(var e=this._groups,n=e.length,r=new Array(n),i=0;i<n;++i)for(var o,a=e[i],s=a.length,A=r[i]=[],l=0;l<s;++l)(o=a[l])&&t.call(o,o.__data__,l,a)&&A.push(o);return new At(r,this._parents)},data:function(t,e){if(!t)return d=new Array(this.size()),c=-1,this.each(function(t){d[++c]=t}),d;var n,r=e?p:f,i=this._parents,o=this._groups;"function"!=typeof t&&(n=t,t=function(){return n});for(var a=o.length,s=new Array(a),A=new Array(a),l=new Array(a),c=0;c<a;++c){var u=i[c],g=o[c],h=g.length,d=t.call(u,u&&u.__data__,c,i),I=d.length,C=A[c]=new Array(I),m=s[c]=new Array(I);r(u,g,C,m,l[c]=new Array(h),d,e);for(var y,E,v=0,B=0;v<I;++v)if(y=C[v]){for(v>=B&&(B=v+1);!(E=m[B])&&++B<I;);y._next=E||null}}return(s=new At(s,i))._enter=A,s._exit=l,s},enter:function(){return new At(this._enter||this._groups.map(g),this._parents)},exit:function(){return new At(this._exit||this._groups.map(g),this._parents)},join:function(t,e,n){var r=this.enter(),i=this,o=this.exit();return r="function"==typeof t?t(r):r.append(t+""),null!=e&&(i=e(i)),null==n?o.remove():n(o),r&&i?r.merge(i).order():i},merge:function(t){for(var e=this._groups,n=t._groups,r=e.length,i=n.length,o=Math.min(r,i),a=new Array(r),s=0;s<o;++s)for(var A,l=e[s],c=n[s],u=l.length,g=a[s]=new Array(u),h=0;h<u;++h)(A=l[h]||c[h])&&(g[h]=A);for(;s<r;++s)a[s]=e[s];return new At(a,this._parents)},order:function(){for(var t=this._groups,e=-1,n=t.length;++e<n;)for(var r,i=t[e],o=i.length-1,a=i[o];--o>=0;)(r=i[o])&&(a&&4^r.compareDocumentPosition(a)&&a.parentNode.insertBefore(r,a),a=r);return this},sort:function(t){function e(e,n){return e&&n?t(e.__data__,n.__data__):!e-!n}t||(t=I);for(var n=this._groups,r=n.length,i=new Array(r),o=0;o<r;++o){for(var a,s=n[o],A=s.length,l=i[o]=new Array(A),c=0;c<A;++c)(a=s[c])&&(l[c]=a);l.sort(e)}return new At(i,this._parents).order()},call:function(){var t=arguments[0];return arguments[0]=this,t.apply(null,arguments),this},nodes:function(){var t=new Array(this.size()),e=-1;return this.each(function(){t[++e]=this}),t},node:function(){for(var t=this._groups,e=0,n=t.length;e<n;++e)for(var r=t[e],i=0,o=r.length;i<o;++i){var a=r[i];if(a)return a}return null},size:function(){var t=0;return this.each(function(){++t}),t},empty:function(){return!this.node()},each:function(t){for(var e=this._groups,n=0,r=e.length;n<r;++n)for(var i,o=e[n],a=0,s=o.length;a<s;++a)(i=o[a])&&t.call(i,i.__data__,a,o);return this},attr:function(t,e){var n=o(t);if(arguments.length<2){var r=this.node();return n.local?r.getAttributeNS(n.space,n.local):r.getAttribute(n)}return this.each((null==e?n.local?m:C:"function"==typeof e?n.local?B:v:n.local?E:y)(n,e))},style:function(t,e,n){return arguments.length>1?this.each((null==e?Q:"function"==typeof e?x:w)(t,e,null==n?"":n)):_(this.node(),t)},property:function(t,e){return arguments.length>1?this.each((null==e?D:"function"==typeof e?k:M)(t,e)):this.node()[t]},classed:function(t,e){var n=S(t+"");if(arguments.length<2){for(var r=L(this.node()),i=-1,o=n.length;++i<o;)if(!r.contains(n[i]))return!1;return!0}return this.each(("function"==typeof e?O:e?T:U)(n,e))},text:function(t){return arguments.length?this.each(null==t?G:("function"==typeof t?j:H)(t)):this.node().textContent},html:function(t){return arguments.length?this.each(null==t?Y:("function"==typeof t?K:P)(t)):this.node().innerHTML},raise:function(){return this.each(z)},lower:function(){return this.each(J)},append:function(t){var e="function"==typeof t?t:a(t);return this.select(function(){return this.appendChild(e.apply(this,arguments))})},insert:function(t,e){var n="function"==typeof t?t:a(t),r=null==e?q:"function"==typeof e?e:A(e);return this.select(function(){return this.insertBefore(n.apply(this,arguments),r.apply(this,arguments)||null)})},remove:function(){return this.each(V)},clone:function(t){return this.select(t?X:W)},datum:function(t){return arguments.length?this.property("__data__",t):this.node().__data__},on:function(t,e,n){var r,i,o=nt(t+""),a=o.length;if(!(arguments.length<2)){for(s=e?it:rt,null==n&&(n=!1),r=0;r<a;++r)this.each(s(o[r],e,n));return this}var s=this.node().__on;if(s)for(var A,l=0,c=s.length;l<c;++l)for(r=0,A=s[l];r<a;++r)if((i=o[r]).type===A.type&&i.name===A.name)return A.value},dispatch:function(t,e){return this.each(("function"==typeof e?function(t,e){return function(){return at(this,t,e.apply(this,arguments))}}:function(t,e){return function(){return at(this,t,e)}})(t,e))}};var ct=lt,ut=function(t){return"string"==typeof t?new At([[document.querySelector(t)]],[document.documentElement]):new At([[t]],st)},gt=function(t){return ut(a(t).call(document.documentElement))},ht=0;function dt(){return new ft}function ft(){this._="@"+(++ht).toString(36)}ft.prototype=dt.prototype={constructor:ft,get:function(t){for(var e=this._;!(e in t);)if(!(t=t.parentNode))return;return t[e]},set:function(t,e){return t[this._]=e},remove:function(t){return this._ in t&&delete t[this._]},toString:function(){return this._}};var pt=function(){for(var t,e=$;t=e.sourceEvent;)e=t;return e},It=function(t,e){var n=t.ownerSVGElement||t;if(n.createSVGPoint){var r=n.createSVGPoint();return r.x=e.clientX,r.y=e.clientY,[(r=r.matrixTransform(t.getScreenCTM().inverse())).x,r.y]}var i=t.getBoundingClientRect();return[e.clientX-i.left-t.clientLeft,e.clientY-i.top-t.clientTop]},Ct=function(t){var e=pt();return e.changedTouches&&(e=e.changedTouches[0]),It(t,e)},mt=function(t){return"string"==typeof t?new At([document.querySelectorAll(t)],[document.documentElement]):new At([null==t?[]:t],st)},yt=function(t,e,n){arguments.length<3&&(n=e,e=pt().changedTouches);for(var r,i=0,o=e?e.length:0;i<o;++i)if((r=e[i]).identifier===n)return It(t,r);return null},Et=function(t,e){null==e&&(e=pt().touches);for(var n=0,r=e?e.length:0,i=new Array(r);n<r;++n)i[n]=It(t,e[n]);return i};n.d(e,"create",function(){return gt}),n.d(e,"creator",function(){return a}),n.d(e,"local",function(){return dt}),n.d(e,"matcher",function(){return u}),n.d(e,"mouse",function(){return Ct}),n.d(e,"namespace",function(){return o}),n.d(e,"namespaces",function(){return i}),n.d(e,"clientPoint",function(){return It}),n.d(e,"select",function(){return ut}),n.d(e,"selectAll",function(){return mt}),n.d(e,"selection",function(){return ct}),n.d(e,"selector",function(){return A}),n.d(e,"selectorAll",function(){return c}),n.d(e,"style",function(){return _}),n.d(e,"touch",function(){return yt}),n.d(e,"touches",function(){return Et}),n.d(e,"window",function(){return b}),n.d(e,"event",function(){return $}),n.d(e,"customEvent",function(){return ot})},function(t,e,n){"use strict";n.r(e);var r=new Date,i=new Date;function o(t,e,n,a){function s(e){return t(e=new Date(+e)),e}return s.floor=s,s.ceil=function(n){return t(n=new Date(n-1)),e(n,1),t(n),n},s.round=function(t){var e=s(t),n=s.ceil(t);return t-e<n-t?e:n},s.offset=function(t,n){return e(t=new Date(+t),null==n?1:Math.floor(n)),t},s.range=function(n,r,i){var o,a=[];if(n=s.ceil(n),i=null==i?1:Math.floor(i),!(n<r&&i>0))return a;do{a.push(o=new Date(+n)),e(n,i),t(n)}while(o<n&&n<r);return a},s.filter=function(n){return o(function(e){if(e>=e)for(;t(e),!n(e);)e.setTime(e-1)},function(t,r){if(t>=t)if(r<0)for(;++r<=0;)for(;e(t,-1),!n(t););else for(;--r>=0;)for(;e(t,1),!n(t););})},n&&(s.count=function(e,o){return r.setTime(+e),i.setTime(+o),t(r),t(i),Math.floor(n(r,i))},s.every=function(t){return t=Math.floor(t),isFinite(t)&&t>0?t>1?s.filter(a?function(e){return a(e)%t==0}:function(e){return s.count(0,e)%t==0}):s:null}),s}var a=o(function(){},function(t,e){t.setTime(+t+e)},function(t,e){return e-t});a.every=function(t){return t=Math.floor(t),isFinite(t)&&t>0?t>1?o(function(e){e.setTime(Math.floor(e/t)*t)},function(e,n){e.setTime(+e+n*t)},function(e,n){return(n-e)/t}):a:null};var s=a,A=a.range,l=6e4,c=6048e5,u=o(function(t){t.setTime(t-t.getMilliseconds())},function(t,e){t.setTime(+t+1e3*e)},function(t,e){return(e-t)/1e3},function(t){return t.getUTCSeconds()}),g=u,h=u.range,d=o(function(t){t.setTime(t-t.getMilliseconds()-1e3*t.getSeconds())},function(t,e){t.setTime(+t+e*l)},function(t,e){return(e-t)/l},function(t){return t.getMinutes()}),f=d,p=d.range,I=o(function(t){t.setTime(t-t.getMilliseconds()-1e3*t.getSeconds()-t.getMinutes()*l)},function(t,e){t.setTime(+t+36e5*e)},function(t,e){return(e-t)/36e5},function(t){return t.getHours()}),C=I,m=I.range,y=o(function(t){t.setHours(0,0,0,0)},function(t,e){t.setDate(t.getDate()+e)},function(t,e){return(e-t-(e.getTimezoneOffset()-t.getTimezoneOffset())*l)/864e5},function(t){return t.getDate()-1}),E=y,v=y.range;function B(t){return o(function(e){e.setDate(e.getDate()-(e.getDay()+7-t)%7),e.setHours(0,0,0,0)},function(t,e){t.setDate(t.getDate()+7*e)},function(t,e){return(e-t-(e.getTimezoneOffset()-t.getTimezoneOffset())*l)/c})}var b=B(0),Q=B(1),w=B(2),x=B(3),_=B(4),D=B(5),M=B(6),k=b.range,S=Q.range,L=w.range,R=x.range,N=_.range,F=D.range,T=M.range,U=o(function(t){t.setDate(1),t.setHours(0,0,0,0)},function(t,e){t.setMonth(t.getMonth()+e)},function(t,e){return e.getMonth()-t.getMonth()+12*(e.getFullYear()-t.getFullYear())},function(t){return t.getMonth()}),O=U,G=U.range,H=o(function(t){t.setMonth(0,1),t.setHours(0,0,0,0)},function(t,e){t.setFullYear(t.getFullYear()+e)},function(t,e){return e.getFullYear()-t.getFullYear()},function(t){return t.getFullYear()});H.every=function(t){return isFinite(t=Math.floor(t))&&t>0?o(function(e){e.setFullYear(Math.floor(e.getFullYear()/t)*t),e.setMonth(0,1),e.setHours(0,0,0,0)},function(e,n){e.setFullYear(e.getFullYear()+n*t)}):null};var j=H,Y=H.range,P=o(function(t){t.setUTCSeconds(0,0)},function(t,e){t.setTime(+t+e*l)},function(t,e){return(e-t)/l},function(t){return t.getUTCMinutes()}),K=P,z=P.range,J=o(function(t){t.setUTCMinutes(0,0,0)},function(t,e){t.setTime(+t+36e5*e)},function(t,e){return(e-t)/36e5},function(t){return t.getUTCHours()}),q=J,V=J.range,W=o(function(t){t.setUTCHours(0,0,0,0)},function(t,e){t.setUTCDate(t.getUTCDate()+e)},function(t,e){return(e-t)/864e5},function(t){return t.getUTCDate()-1}),X=W,Z=W.range;function $(t){return o(function(e){e.setUTCDate(e.getUTCDate()-(e.getUTCDay()+7-t)%7),e.setUTCHours(0,0,0,0)},function(t,e){t.setUTCDate(t.getUTCDate()+7*e)},function(t,e){return(e-t)/c})}var tt=$(0),et=$(1),nt=$(2),rt=$(3),it=$(4),ot=$(5),at=$(6),st=tt.range,At=et.range,lt=nt.range,ct=rt.range,ut=it.range,gt=ot.range,ht=at.range,dt=o(function(t){t.setUTCDate(1),t.setUTCHours(0,0,0,0)},function(t,e){t.setUTCMonth(t.getUTCMonth()+e)},function(t,e){return e.getUTCMonth()-t.getUTCMonth()+12*(e.getUTCFullYear()-t.getUTCFullYear())},function(t){return t.getUTCMonth()}),ft=dt,pt=dt.range,It=o(function(t){t.setUTCMonth(0,1),t.setUTCHours(0,0,0,0)},function(t,e){t.setUTCFullYear(t.getUTCFullYear()+e)},function(t,e){return e.getUTCFullYear()-t.getUTCFullYear()},function(t){return t.getUTCFullYear()});It.every=function(t){return isFinite(t=Math.floor(t))&&t>0?o(function(e){e.setUTCFullYear(Math.floor(e.getUTCFullYear()/t)*t),e.setUTCMonth(0,1),e.setUTCHours(0,0,0,0)},function(e,n){e.setUTCFullYear(e.getUTCFullYear()+n*t)}):null};var Ct=It,mt=It.range;n.d(e,"timeInterval",function(){return o}),n.d(e,"timeMillisecond",function(){return s}),n.d(e,"timeMilliseconds",function(){return A}),n.d(e,"utcMillisecond",function(){return s}),n.d(e,"utcMilliseconds",function(){return A}),n.d(e,"timeSecond",function(){return g}),n.d(e,"timeSeconds",function(){return h}),n.d(e,"utcSecond",function(){return g}),n.d(e,"utcSeconds",function(){return h}),n.d(e,"timeMinute",function(){return f}),n.d(e,"timeMinutes",function(){return p}),n.d(e,"timeHour",function(){return C}),n.d(e,"timeHours",function(){return m}),n.d(e,"timeDay",function(){return E}),n.d(e,"timeDays",function(){return v}),n.d(e,"timeWeek",function(){return b}),n.d(e,"timeWeeks",function(){return k}),n.d(e,"timeSunday",function(){return b}),n.d(e,"timeSundays",function(){return k}),n.d(e,"timeMonday",function(){return Q}),n.d(e,"timeMondays",function(){return S}),n.d(e,"timeTuesday",function(){return w}),n.d(e,"timeTuesdays",function(){return L}),n.d(e,"timeWednesday",function(){return x}),n.d(e,"timeWednesdays",function(){return R}),n.d(e,"timeThursday",function(){return _}),n.d(e,"timeThursdays",function(){return N}),n.d(e,"timeFriday",function(){return D}),n.d(e,"timeFridays",function(){return F}),n.d(e,"timeSaturday",function(){return M}),n.d(e,"timeSaturdays",function(){return T}),n.d(e,"timeMonth",function(){return O}),n.d(e,"timeMonths",function(){return G}),n.d(e,"timeYear",function(){return j}),n.d(e,"timeYears",function(){return Y}),n.d(e,"utcMinute",function(){return K}),n.d(e,"utcMinutes",function(){return z}),n.d(e,"utcHour",function(){return q}),n.d(e,"utcHours",function(){return V}),n.d(e,"utcDay",function(){return X}),n.d(e,"utcDays",function(){return Z}),n.d(e,"utcWeek",function(){return tt}),n.d(e,"utcWeeks",function(){return st}),n.d(e,"utcSunday",function(){return tt}),n.d(e,"utcSundays",function(){return st}),n.d(e,"utcMonday",function(){return et}),n.d(e,"utcMondays",function(){return At}),n.d(e,"utcTuesday",function(){return nt}),n.d(e,"utcTuesdays",function(){return lt}),n.d(e,"utcWednesday",function(){return rt}),n.d(e,"utcWednesdays",function(){return ct}),n.d(e,"utcThursday",function(){return it}),n.d(e,"utcThursdays",function(){return ut}),n.d(e,"utcFriday",function(){return ot}),n.d(e,"utcFridays",function(){return gt}),n.d(e,"utcSaturday",function(){return at}),n.d(e,"utcSaturdays",function(){return ht}),n.d(e,"utcMonth",function(){return ft}),n.d(e,"utcMonths",function(){return pt}),n.d(e,"utcYear",function(){return Ct}),n.d(e,"utcYears",function(){return mt})},function(t,e,n){"use strict";n.r(e);var r=function(t,e){return t<e?-1:t>e?1:t>=e?0:NaN},i=function(t){var e;return 1===t.length&&(e=t,t=function(t,n){return r(e(t),n)}),{left:function(e,n,r,i){for(null==r&&(r=0),null==i&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)<0?r=o+1:i=o}return r},right:function(e,n,r,i){for(null==r&&(r=0),null==i&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)>0?i=o:r=o+1}return r}}},o=i(r),a=o.right,s=o.left,A=a,l=function(t,e){null==e&&(e=c);for(var n=0,r=t.length-1,i=t[0],o=new Array(r<0?0:r);n<r;)o[n]=e(i,i=t[++n]);return o};function c(t,e){return[t,e]}var u=function(t,e,n){var r,i,o,a,s=t.length,A=e.length,l=new Array(s*A);for(null==n&&(n=c),r=o=0;r<s;++r)for(a=t[r],i=0;i<A;++i,++o)l[o]=n(a,e[i]);return l},g=function(t,e){return e<t?-1:e>t?1:e>=t?0:NaN},h=function(t){return null===t?NaN:+t},d=function(t,e){var n,r,i=t.length,o=0,a=-1,s=0,A=0;if(null==e)for(;++a<i;)isNaN(n=h(t[a]))||(A+=(r=n-s)*(n-(s+=r/++o)));else for(;++a<i;)isNaN(n=h(e(t[a],a,t)))||(A+=(r=n-s)*(n-(s+=r/++o)));if(o>1)return A/(o-1)},f=function(t,e){var n=d(t,e);return n?Math.sqrt(n):n},p=function(t,e){var n,r,i,o=t.length,a=-1;if(null==e){for(;++a<o;)if(null!=(n=t[a])&&n>=n)for(r=i=n;++a<o;)null!=(n=t[a])&&(r>n&&(r=n),i<n&&(i=n))}else for(;++a<o;)if(null!=(n=e(t[a],a,t))&&n>=n)for(r=i=n;++a<o;)null!=(n=e(t[a],a,t))&&(r>n&&(r=n),i<n&&(i=n));return[r,i]},I=Array.prototype,C=I.slice,m=I.map,y=function(t){return function(){return t}},E=function(t){return t},v=function(t,e,n){t=+t,e=+e,n=(i=arguments.length)<2?(e=t,t=0,1):i<3?1:+n;for(var r=-1,i=0|Math.max(0,Math.ceil((e-t)/n)),o=new Array(i);++r<i;)o[r]=t+r*n;return o},B=Math.sqrt(50),b=Math.sqrt(10),Q=Math.sqrt(2),w=function(t,e,n){var r,i,o,a,s=-1;if(n=+n,(t=+t)==(e=+e)&&n>0)return[t];if((r=e<t)&&(i=t,t=e,e=i),0===(a=x(t,e,n))||!isFinite(a))return[];if(a>0)for(t=Math.ceil(t/a),e=Math.floor(e/a),o=new Array(i=Math.ceil(e-t+1));++s<i;)o[s]=(t+s)*a;else for(t=Math.floor(t*a),e=Math.ceil(e*a),o=new Array(i=Math.ceil(t-e+1));++s<i;)o[s]=(t-s)/a;return r&&o.reverse(),o};function x(t,e,n){var r=(e-t)/Math.max(0,n),i=Math.floor(Math.log(r)/Math.LN10),o=r/Math.pow(10,i);return i>=0?(o>=B?10:o>=b?5:o>=Q?2:1)*Math.pow(10,i):-Math.pow(10,-i)/(o>=B?10:o>=b?5:o>=Q?2:1)}function _(t,e,n){var r=Math.abs(e-t)/Math.max(0,n),i=Math.pow(10,Math.floor(Math.log(r)/Math.LN10)),o=r/i;return o>=B?i*=10:o>=b?i*=5:o>=Q&&(i*=2),e<t?-i:i}var D=function(t){return Math.ceil(Math.log(t.length)/Math.LN2)+1},M=function(){var t=E,e=p,n=D;function r(r){var i,o,a=r.length,s=new Array(a);for(i=0;i<a;++i)s[i]=t(r[i],i,r);var l=e(s),c=l[0],u=l[1],g=n(s,c,u);Array.isArray(g)||(g=_(c,u,g),g=v(Math.ceil(c/g)*g,u,g));for(var h=g.length;g[0]<=c;)g.shift(),--h;for(;g[h-1]>u;)g.pop(),--h;var d,f=new Array(h+1);for(i=0;i<=h;++i)(d=f[i]=[]).x0=i>0?g[i-1]:c,d.x1=i<h?g[i]:u;for(i=0;i<a;++i)c<=(o=s[i])&&o<=u&&f[A(g,o,0,h)].push(r[i]);return f}return r.value=function(e){return arguments.length?(t="function"==typeof e?e:y(e),r):t},r.domain=function(t){return arguments.length?(e="function"==typeof t?t:y([t[0],t[1]]),r):e},r.thresholds=function(t){return arguments.length?(n="function"==typeof t?t:Array.isArray(t)?y(C.call(t)):y(t),r):n},r},k=function(t,e,n){if(null==n&&(n=h),r=t.length){if((e=+e)<=0||r<2)return+n(t[0],0,t);if(e>=1)return+n(t[r-1],r-1,t);var r,i=(r-1)*e,o=Math.floor(i),a=+n(t[o],o,t);return a+(+n(t[o+1],o+1,t)-a)*(i-o)}},S=function(t,e,n){return t=m.call(t,h).sort(r),Math.ceil((n-e)/(2*(k(t,.75)-k(t,.25))*Math.pow(t.length,-1/3)))},L=function(t,e,n){return Math.ceil((n-e)/(3.5*f(t)*Math.pow(t.length,-1/3)))},R=function(t,e){var n,r,i=t.length,o=-1;if(null==e){for(;++o<i;)if(null!=(n=t[o])&&n>=n)for(r=n;++o<i;)null!=(n=t[o])&&n>r&&(r=n)}else for(;++o<i;)if(null!=(n=e(t[o],o,t))&&n>=n)for(r=n;++o<i;)null!=(n=e(t[o],o,t))&&n>r&&(r=n);return r},N=function(t,e){var n,r=t.length,i=r,o=-1,a=0;if(null==e)for(;++o<r;)isNaN(n=h(t[o]))?--i:a+=n;else for(;++o<r;)isNaN(n=h(e(t[o],o,t)))?--i:a+=n;if(i)return a/i},F=function(t,e){var n,i=t.length,o=-1,a=[];if(null==e)for(;++o<i;)isNaN(n=h(t[o]))||a.push(n);else for(;++o<i;)isNaN(n=h(e(t[o],o,t)))||a.push(n);return k(a.sort(r),.5)},T=function(t){for(var e,n,r,i=t.length,o=-1,a=0;++o<i;)a+=t[o].length;for(n=new Array(a);--i>=0;)for(e=(r=t[i]).length;--e>=0;)n[--a]=r[e];return n},U=function(t,e){var n,r,i=t.length,o=-1;if(null==e){for(;++o<i;)if(null!=(n=t[o])&&n>=n)for(r=n;++o<i;)null!=(n=t[o])&&r>n&&(r=n)}else for(;++o<i;)if(null!=(n=e(t[o],o,t))&&n>=n)for(r=n;++o<i;)null!=(n=e(t[o],o,t))&&r>n&&(r=n);return r},O=function(t,e){for(var n=e.length,r=new Array(n);n--;)r[n]=t[e[n]];return r},G=function(t,e){if(n=t.length){var n,i,o=0,a=0,s=t[a];for(null==e&&(e=r);++o<n;)(e(i=t[o],s)<0||0!==e(s,s))&&(s=i,a=o);return 0===e(s,s)?a:void 0}},H=function(t,e,n){for(var r,i,o=(null==n?t.length:n)-(e=null==e?0:+e);o;)i=Math.random()*o--|0,r=t[o+e],t[o+e]=t[i+e],t[i+e]=r;return t},j=function(t,e){var n,r=t.length,i=-1,o=0;if(null==e)for(;++i<r;)(n=+t[i])&&(o+=n);else for(;++i<r;)(n=+e(t[i],i,t))&&(o+=n);return o},Y=function(t){if(!(i=t.length))return[];for(var e=-1,n=U(t,P),r=new Array(n);++e<n;)for(var i,o=-1,a=r[e]=new Array(i);++o<i;)a[o]=t[o][e];return r};function P(t){return t.length}var K=function(){return Y(arguments)};n.d(e,"bisect",function(){return A}),n.d(e,"bisectRight",function(){return a}),n.d(e,"bisectLeft",function(){return s}),n.d(e,"ascending",function(){return r}),n.d(e,"bisector",function(){return i}),n.d(e,"cross",function(){return u}),n.d(e,"descending",function(){return g}),n.d(e,"deviation",function(){return f}),n.d(e,"extent",function(){return p}),n.d(e,"histogram",function(){return M}),n.d(e,"thresholdFreedmanDiaconis",function(){return S}),n.d(e,"thresholdScott",function(){return L}),n.d(e,"thresholdSturges",function(){return D}),n.d(e,"max",function(){return R}),n.d(e,"mean",function(){return N}),n.d(e,"median",function(){return F}),n.d(e,"merge",function(){return T}),n.d(e,"min",function(){return U}),n.d(e,"pairs",function(){return l}),n.d(e,"permute",function(){return O}),n.d(e,"quantile",function(){return k}),n.d(e,"range",function(){return v}),n.d(e,"scan",function(){return G}),n.d(e,"shuffle",function(){return H}),n.d(e,"sum",function(){return j}),n.d(e,"ticks",function(){return w}),n.d(e,"tickIncrement",function(){return x}),n.d(e,"tickStep",function(){return _}),n.d(e,"transpose",function(){return Y}),n.d(e,"variance",function(){return d}),n.d(e,"zip",function(){return K})},function(t,e,n){"use strict";var r=function(t,e,n){t.prototype=e.prototype=n,n.constructor=t};function i(t,e){var n=Object.create(t.prototype);for(var r in e)n[r]=e[r];return n}function o(){}var a="\\s*([+-]?\\d+)\\s*",s="\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*",A="\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*",l=/^#([0-9a-f]{3})$/,c=/^#([0-9a-f]{6})$/,u=new RegExp("^rgb\\("+[a,a,a]+"\\)$"),g=new RegExp("^rgb\\("+[A,A,A]+"\\)$"),h=new RegExp("^rgba\\("+[a,a,a,s]+"\\)$"),d=new RegExp("^rgba\\("+[A,A,A,s]+"\\)$"),f=new RegExp("^hsl\\("+[s,A,A]+"\\)$"),p=new RegExp("^hsla\\("+[s,A,A,s]+"\\)$"),I={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074};function C(t){var e;return t=(t+"").trim().toLowerCase(),(e=l.exec(t))?new B((e=parseInt(e[1],16))>>8&15|e>>4&240,e>>4&15|240&e,(15&e)<<4|15&e,1):(e=c.exec(t))?m(parseInt(e[1],16)):(e=u.exec(t))?new B(e[1],e[2],e[3],1):(e=g.exec(t))?new B(255*e[1]/100,255*e[2]/100,255*e[3]/100,1):(e=h.exec(t))?y(e[1],e[2],e[3],e[4]):(e=d.exec(t))?y(255*e[1]/100,255*e[2]/100,255*e[3]/100,e[4]):(e=f.exec(t))?Q(e[1],e[2]/100,e[3]/100,1):(e=p.exec(t))?Q(e[1],e[2]/100,e[3]/100,e[4]):I.hasOwnProperty(t)?m(I[t]):"transparent"===t?new B(NaN,NaN,NaN,0):null}function m(t){return new B(t>>16&255,t>>8&255,255&t,1)}function y(t,e,n,r){return r<=0&&(t=e=n=NaN),new B(t,e,n,r)}function E(t){return t instanceof o||(t=C(t)),t?new B((t=t.rgb()).r,t.g,t.b,t.opacity):new B}function v(t,e,n,r){return 1===arguments.length?E(t):new B(t,e,n,null==r?1:r)}function B(t,e,n,r){this.r=+t,this.g=+e,this.b=+n,this.opacity=+r}function b(t){return((t=Math.max(0,Math.min(255,Math.round(t)||0)))<16?"0":"")+t.toString(16)}function Q(t,e,n,r){return r<=0?t=e=n=NaN:n<=0||n>=1?t=e=NaN:e<=0&&(t=NaN),new x(t,e,n,r)}function w(t,e,n,r){return 1===arguments.length?function(t){if(t instanceof x)return new x(t.h,t.s,t.l,t.opacity);if(t instanceof o||(t=C(t)),!t)return new x;if(t instanceof x)return t;var e=(t=t.rgb()).r/255,n=t.g/255,r=t.b/255,i=Math.min(e,n,r),a=Math.max(e,n,r),s=NaN,A=a-i,l=(a+i)/2;return A?(s=e===a?(n-r)/A+6*(n<r):n===a?(r-e)/A+2:(e-n)/A+4,A/=l<.5?a+i:2-a-i,s*=60):A=l>0&&l<1?0:s,new x(s,A,l,t.opacity)}(t):new x(t,e,n,null==r?1:r)}function x(t,e,n,r){this.h=+t,this.s=+e,this.l=+n,this.opacity=+r}function _(t,e,n){return 255*(t<60?e+(n-e)*t/60:t<180?n:t<240?e+(n-e)*(240-t)/60:e)}r(o,C,{displayable:function(){return this.rgb().displayable()},hex:function(){return this.rgb().hex()},toString:function(){return this.rgb()+""}}),r(B,v,i(o,{brighter:function(t){return t=null==t?1/.7:Math.pow(1/.7,t),new B(this.r*t,this.g*t,this.b*t,this.opacity)},darker:function(t){return t=null==t?.7:Math.pow(.7,t),new B(this.r*t,this.g*t,this.b*t,this.opacity)},rgb:function(){return this},displayable:function(){return-.5<=this.r&&this.r<255.5&&-.5<=this.g&&this.g<255.5&&-.5<=this.b&&this.b<255.5&&0<=this.opacity&&this.opacity<=1},hex:function(){return"#"+b(this.r)+b(this.g)+b(this.b)},toString:function(){var t=this.opacity;return(1===(t=isNaN(t)?1:Math.max(0,Math.min(1,t)))?"rgb(":"rgba(")+Math.max(0,Math.min(255,Math.round(this.r)||0))+", "+Math.max(0,Math.min(255,Math.round(this.g)||0))+", "+Math.max(0,Math.min(255,Math.round(this.b)||0))+(1===t?")":", "+t+")")}})),r(x,w,i(o,{brighter:function(t){return t=null==t?1/.7:Math.pow(1/.7,t),new x(this.h,this.s,this.l*t,this.opacity)},darker:function(t){return t=null==t?.7:Math.pow(.7,t),new x(this.h,this.s,this.l*t,this.opacity)},rgb:function(){var t=this.h%360+360*(this.h<0),e=isNaN(t)||isNaN(this.s)?0:this.s,n=this.l,r=n+(n<.5?n:1-n)*e,i=2*n-r;return new B(_(t>=240?t-240:t+120,i,r),_(t,i,r),_(t<120?t+240:t-120,i,r),this.opacity)},displayable:function(){return(0<=this.s&&this.s<=1||isNaN(this.s))&&0<=this.l&&this.l<=1&&0<=this.opacity&&this.opacity<=1}}));var D=Math.PI/180,M=180/Math.PI,k=.96422,S=1,L=.82521,R=4/29,N=6/29,F=3*N*N,T=N*N*N;function U(t){if(t instanceof H)return new H(t.l,t.a,t.b,t.opacity);if(t instanceof V)return W(t);t instanceof B||(t=E(t));var e,n,r=K(t.r),i=K(t.g),o=K(t.b),a=j((.2225045*r+.7168786*i+.0606169*o)/S);return r===i&&i===o?e=n=a:(e=j((.4360747*r+.3850649*i+.1430804*o)/k),n=j((.0139322*r+.0971045*i+.7141733*o)/L)),new H(116*a-16,500*(e-a),200*(a-n),t.opacity)}function O(t,e){return new H(t,0,0,null==e?1:e)}function G(t,e,n,r){return 1===arguments.length?U(t):new H(t,e,n,null==r?1:r)}function H(t,e,n,r){this.l=+t,this.a=+e,this.b=+n,this.opacity=+r}function j(t){return t>T?Math.pow(t,1/3):t/F+R}function Y(t){return t>N?t*t*t:F*(t-R)}function P(t){return 255*(t<=.0031308?12.92*t:1.055*Math.pow(t,1/2.4)-.055)}function K(t){return(t/=255)<=.04045?t/12.92:Math.pow((t+.055)/1.055,2.4)}function z(t){if(t instanceof V)return new V(t.h,t.c,t.l,t.opacity);if(t instanceof H||(t=U(t)),0===t.a&&0===t.b)return new V(NaN,0<t.l&&t.l<100?0:NaN,t.l,t.opacity);var e=Math.atan2(t.b,t.a)*M;return new V(e<0?e+360:e,Math.sqrt(t.a*t.a+t.b*t.b),t.l,t.opacity)}function J(t,e,n,r){return 1===arguments.length?z(t):new V(n,e,t,null==r?1:r)}function q(t,e,n,r){return 1===arguments.length?z(t):new V(t,e,n,null==r?1:r)}function V(t,e,n,r){this.h=+t,this.c=+e,this.l=+n,this.opacity=+r}function W(t){if(isNaN(t.h))return new H(t.l,0,0,t.opacity);var e=t.h*D;return new H(t.l,Math.cos(e)*t.c,Math.sin(e)*t.c,t.opacity)}r(H,G,i(o,{brighter:function(t){return new H(this.l+18*(null==t?1:t),this.a,this.b,this.opacity)},darker:function(t){return new H(this.l-18*(null==t?1:t),this.a,this.b,this.opacity)},rgb:function(){var t=(this.l+16)/116,e=isNaN(this.a)?t:t+this.a/500,n=isNaN(this.b)?t:t-this.b/200;return new B(P(3.1338561*(e=k*Y(e))-1.6168667*(t=S*Y(t))-.4906146*(n=L*Y(n))),P(-.9787684*e+1.9161415*t+.033454*n),P(.0719453*e-.2289914*t+1.4052427*n),this.opacity)}})),r(V,q,i(o,{brighter:function(t){return new V(this.h,this.c,this.l+18*(null==t?1:t),this.opacity)},darker:function(t){return new V(this.h,this.c,this.l-18*(null==t?1:t),this.opacity)},rgb:function(){return W(this).rgb()}}));var X=-.14861,Z=1.78277,$=-.29227,tt=-.90649,et=1.97294,nt=et*tt,rt=et*Z,it=Z*$-tt*X;function ot(t,e,n,r){return 1===arguments.length?function(t){if(t instanceof at)return new at(t.h,t.s,t.l,t.opacity);t instanceof B||(t=E(t));var e=t.r/255,n=t.g/255,r=t.b/255,i=(it*r+nt*e-rt*n)/(it+nt-rt),o=r-i,a=(et*(n-i)-$*o)/tt,s=Math.sqrt(a*a+o*o)/(et*i*(1-i)),A=s?Math.atan2(a,o)*M-120:NaN;return new at(A<0?A+360:A,s,i,t.opacity)}(t):new at(t,e,n,null==r?1:r)}function at(t,e,n,r){this.h=+t,this.s=+e,this.l=+n,this.opacity=+r}r(at,ot,i(o,{brighter:function(t){return t=null==t?1/.7:Math.pow(1/.7,t),new at(this.h,this.s,this.l*t,this.opacity)},darker:function(t){return t=null==t?.7:Math.pow(.7,t),new at(this.h,this.s,this.l*t,this.opacity)},rgb:function(){var t=isNaN(this.h)?0:(this.h+120)*D,e=+this.l,n=isNaN(this.s)?0:this.s*e*(1-e),r=Math.cos(t),i=Math.sin(t);return new B(255*(e+n*(X*r+Z*i)),255*(e+n*($*r+tt*i)),255*(e+n*(et*r)),this.opacity)}})),n.d(e,"a",function(){return C}),n.d(e,"h",function(){return v}),n.d(e,"e",function(){return w}),n.d(e,"f",function(){return G}),n.d(e,"d",function(){return q}),n.d(e,"g",function(){return J}),n.d(e,"c",function(){return O}),n.d(e,"b",function(){return ot})},function(t,e,n){"use strict";var r=n(3);function i(t,e,n,r,i){var o=t*t,a=o*t;return((1-3*t+3*o-a)*e+(4-6*o+3*a)*n+(1+3*t+3*o-3*a)*r+a*i)/6}var o=function(t){var e=t.length-1;return function(n){var r=n<=0?n=0:n>=1?(n=1,e-1):Math.floor(n*e),o=t[r],a=t[r+1],s=r>0?t[r-1]:2*o-a,A=r<e-1?t[r+2]:2*a-o;return i((n-r/e)*e,s,o,a,A)}},a=function(t){var e=t.length;return function(n){var r=Math.floor(((n%=1)<0?++n:n)*e),o=t[(r+e-1)%e],a=t[r%e],s=t[(r+1)%e],A=t[(r+2)%e];return i((n-r/e)*e,o,a,s,A)}},s=function(t){return function(){return t}};function A(t,e){return function(n){return t+n*e}}function l(t,e){var n=e-t;return n?A(t,n>180||n<-180?n-360*Math.round(n/360):n):s(isNaN(t)?e:t)}function c(t,e){var n=e-t;return n?A(t,n):s(isNaN(t)?e:t)}var u=function t(e){var n=function(t){return 1==(t=+t)?c:function(e,n){return n-e?function(t,e,n){return t=Math.pow(t,n),e=Math.pow(e,n)-t,n=1/n,function(r){return Math.pow(t+r*e,n)}}(e,n,t):s(isNaN(e)?n:e)}}(e);function i(t,e){var i=n((t=Object(r.h)(t)).r,(e=Object(r.h)(e)).r),o=n(t.g,e.g),a=n(t.b,e.b),s=c(t.opacity,e.opacity);return function(e){return t.r=i(e),t.g=o(e),t.b=a(e),t.opacity=s(e),t+""}}return i.gamma=t,i}(1);function g(t){return function(e){var n,i,o=e.length,a=new Array(o),s=new Array(o),A=new Array(o);for(n=0;n<o;++n)i=Object(r.h)(e[n]),a[n]=i.r||0,s[n]=i.g||0,A[n]=i.b||0;return a=t(a),s=t(s),A=t(A),i.opacity=1,function(t){return i.r=a(t),i.g=s(t),i.b=A(t),i+""}}}var h,d,f,p,I=g(o),C=g(a),m=function(t,e){var n,r=e?e.length:0,i=t?Math.min(r,t.length):0,o=new Array(i),a=new Array(r);for(n=0;n<i;++n)o[n]=w(t[n],e[n]);for(;n<r;++n)a[n]=e[n];return function(t){for(n=0;n<i;++n)a[n]=o[n](t);return a}},y=function(t,e){var n=new Date;return e-=t=+t,function(r){return n.setTime(t+e*r),n}},E=function(t,e){return e-=t=+t,function(n){return t+e*n}},v=function(t,e){var n,r={},i={};for(n in null!==t&&"object"==typeof t||(t={}),null!==e&&"object"==typeof e||(e={}),e)n in t?r[n]=w(t[n],e[n]):i[n]=e[n];return function(t){for(n in r)i[n]=r[n](t);return i}},B=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,b=new RegExp(B.source,"g"),Q=function(t,e){var n,r,i,o=B.lastIndex=b.lastIndex=0,a=-1,s=[],A=[];for(t+="",e+="";(n=B.exec(t))&&(r=b.exec(e));)(i=r.index)>o&&(i=e.slice(o,i),s[a]?s[a]+=i:s[++a]=i),(n=n[0])===(r=r[0])?s[a]?s[a]+=r:s[++a]=r:(s[++a]=null,A.push({i:a,x:E(n,r)})),o=b.lastIndex;return o<e.length&&(i=e.slice(o),s[a]?s[a]+=i:s[++a]=i),s.length<2?A[0]?function(t){return function(e){return t(e)+""}}(A[0].x):function(t){return function(){return t}}(e):(e=A.length,function(t){for(var n,r=0;r<e;++r)s[(n=A[r]).i]=n.x(t);return s.join("")})},w=function(t,e){var n,i=typeof e;return null==e||"boolean"===i?s(e):("number"===i?E:"string"===i?(n=Object(r.a)(e))?(e=n,u):Q:e instanceof r.a?u:e instanceof Date?y:Array.isArray(e)?m:"function"!=typeof e.valueOf&&"function"!=typeof e.toString||isNaN(e)?v:E)(t,e)},x=function(t){var e=t.length;return function(n){return t[Math.max(0,Math.min(e-1,Math.floor(n*e)))]}},_=function(t,e){var n=l(+t,+e);return function(t){var e=n(t);return e-360*Math.floor(e/360)}},D=function(t,e){return e-=t=+t,function(n){return Math.round(t+e*n)}},M=180/Math.PI,k={translateX:0,translateY:0,rotate:0,skewX:0,scaleX:1,scaleY:1},S=function(t,e,n,r,i,o){var a,s,A;return(a=Math.sqrt(t*t+e*e))&&(t/=a,e/=a),(A=t*n+e*r)&&(n-=t*A,r-=e*A),(s=Math.sqrt(n*n+r*r))&&(n/=s,r/=s,A/=s),t*r<e*n&&(t=-t,e=-e,A=-A,a=-a),{translateX:i,translateY:o,rotate:Math.atan2(e,t)*M,skewX:Math.atan(A)*M,scaleX:a,scaleY:s}};function L(t,e,n,r){function i(t){return t.length?t.pop()+" ":""}return function(o,a){var s=[],A=[];return o=t(o),a=t(a),function(t,r,i,o,a,s){if(t!==i||r!==o){var A=a.push("translate(",null,e,null,n);s.push({i:A-4,x:E(t,i)},{i:A-2,x:E(r,o)})}else(i||o)&&a.push("translate("+i+e+o+n)}(o.translateX,o.translateY,a.translateX,a.translateY,s,A),function(t,e,n,o){t!==e?(t-e>180?e+=360:e-t>180&&(t+=360),A.push({i:n.push(i(n)+"rotate(",null,r)-2,x:E(t,e)})):e&&n.push(i(n)+"rotate("+e+r)}(o.rotate,a.rotate,s),function(t,e,n,o){t!==e?A.push({i:n.push(i(n)+"skewX(",null,r)-2,x:E(t,e)}):e&&n.push(i(n)+"skewX("+e+r)}(o.skewX,a.skewX,s),function(t,e,n,r,o,a){if(t!==n||e!==r){var s=o.push(i(o)+"scale(",null,",",null,")");a.push({i:s-4,x:E(t,n)},{i:s-2,x:E(e,r)})}else 1===n&&1===r||o.push(i(o)+"scale("+n+","+r+")")}(o.scaleX,o.scaleY,a.scaleX,a.scaleY,s,A),o=a=null,function(t){for(var e,n=-1,r=A.length;++n<r;)s[(e=A[n]).i]=e.x(t);return s.join("")}}}var R=L(function(t){return"none"===t?k:(h||(h=document.createElement("DIV"),d=document.documentElement,f=document.defaultView),h.style.transform=t,t=f.getComputedStyle(d.appendChild(h),null).getPropertyValue("transform"),d.removeChild(h),t=t.slice(7,-1).split(","),S(+t[0],+t[1],+t[2],+t[3],+t[4],+t[5]))},"px, ","px)","deg)"),N=L(function(t){return null==t?k:(p||(p=document.createElementNS("http://www.w3.org/2000/svg","g")),p.setAttribute("transform",t),(t=p.transform.baseVal.consolidate())?(t=t.matrix,S(t.a,t.b,t.c,t.d,t.e,t.f)):k)},", ",")",")"),F=Math.SQRT2;function T(t){return((t=Math.exp(t))+1/t)/2}var U=function(t,e){var n,r,i=t[0],o=t[1],a=t[2],s=e[0],A=e[1],l=e[2],c=s-i,u=A-o,g=c*c+u*u;if(g<1e-12)r=Math.log(l/a)/F,n=function(t){return[i+t*c,o+t*u,a*Math.exp(F*t*r)]};else{var h=Math.sqrt(g),d=(l*l-a*a+4*g)/(2*a*2*h),f=(l*l-a*a-4*g)/(2*l*2*h),p=Math.log(Math.sqrt(d*d+1)-d),I=Math.log(Math.sqrt(f*f+1)-f);r=(I-p)/F,n=function(t){var e,n=t*r,s=T(p),A=a/(2*h)*(s*(e=F*n+p,((e=Math.exp(2*e))-1)/(e+1))-function(t){return((t=Math.exp(t))-1/t)/2}(p));return[i+A*c,o+A*u,a*s/T(F*n+p)]}}return n.duration=1e3*r,n};function O(t){return function(e,n){var i=t((e=Object(r.e)(e)).h,(n=Object(r.e)(n)).h),o=c(e.s,n.s),a=c(e.l,n.l),s=c(e.opacity,n.opacity);return function(t){return e.h=i(t),e.s=o(t),e.l=a(t),e.opacity=s(t),e+""}}}var G=O(l),H=O(c);function j(t,e){var n=c((t=Object(r.f)(t)).l,(e=Object(r.f)(e)).l),i=c(t.a,e.a),o=c(t.b,e.b),a=c(t.opacity,e.opacity);return function(e){return t.l=n(e),t.a=i(e),t.b=o(e),t.opacity=a(e),t+""}}function Y(t){return function(e,n){var i=t((e=Object(r.d)(e)).h,(n=Object(r.d)(n)).h),o=c(e.c,n.c),a=c(e.l,n.l),s=c(e.opacity,n.opacity);return function(t){return e.h=i(t),e.c=o(t),e.l=a(t),e.opacity=s(t),e+""}}}var P=Y(l),K=Y(c);function z(t){return function e(n){function i(e,i){var o=t((e=Object(r.b)(e)).h,(i=Object(r.b)(i)).h),a=c(e.s,i.s),s=c(e.l,i.l),A=c(e.opacity,i.opacity);return function(t){return e.h=o(t),e.s=a(t),e.l=s(Math.pow(t,n)),e.opacity=A(t),e+""}}return n=+n,i.gamma=e,i}(1)}var J=z(l),q=z(c);function V(t,e){for(var n=0,r=e.length-1,i=e[0],o=new Array(r<0?0:r);n<r;)o[n]=t(i,i=e[++n]);return function(t){var e=Math.max(0,Math.min(r-1,Math.floor(t*=r)));return o[e](t-e)}}var W=function(t,e){for(var n=new Array(e),r=0;r<e;++r)n[r]=t(r/(e-1));return n};n.d(e,"a",function(){return w}),n.d(e,"b",function(){return m}),n.d(e,"c",function(){return o}),n.d(e,"d",function(){return a}),n.d(e,"g",function(){return y}),n.d(e,"h",function(){return x}),n.d(e,"m",function(){return _}),n.d(e,"o",function(){return E}),n.d(e,"p",function(){return v}),n.d(e,"t",function(){return D}),n.d(e,"u",function(){return Q}),n.d(e,"v",function(){return R}),n.d(e,"w",function(){return N}),n.d(e,"x",function(){return U}),n.d(e,"q",function(){return u}),n.d(e,"r",function(){return I}),n.d(e,"s",function(){return C}),n.d(e,"k",function(){return G}),n.d(e,"l",function(){return H}),n.d(e,"n",function(){return j}),n.d(e,"i",function(){return P}),n.d(e,"j",function(){return K}),n.d(e,"e",function(){return J}),n.d(e,"f",function(){return q}),n.d(e,"y",function(){return V}),n.d(e,"z",function(){return W})},function(t,e,n){var r,i,o;
/*!
 * jQuery JavaScript Library v1.11.0
 * http://jquery.com/
 *
 * Includes Sizzle.js
 * http://sizzlejs.com/
 *
 * Copyright 2005, 2014 jQuery Foundation, Inc. and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2014-01-23T21:02Z
 */
/*!
 * jQuery JavaScript Library v1.11.0
 * http://jquery.com/
 *
 * Includes Sizzle.js
 * http://sizzlejs.com/
 *
 * Copyright 2005, 2014 jQuery Foundation, Inc. and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2014-01-23T21:02Z
 */i="undefined"!=typeof window?window:this,o=function(n,i){var o=[],a=o.slice,s=o.concat,A=o.push,l=o.indexOf,c={},u=c.toString,g=c.hasOwnProperty,h="".trim,d={},f=function(t,e){return new f.fn.init(t,e)},p=/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,I=/^-ms-/,C=/-([\da-z])/gi,m=function(t,e){return e.toUpperCase()};function y(t){var e=t.length,n=f.type(t);return"function"!==n&&!f.isWindow(t)&&(!(1!==t.nodeType||!e)||"array"===n||0===e||"number"==typeof e&&e>0&&e-1 in t)}f.fn=f.prototype={jquery:"1.11.0",constructor:f,selector:"",length:0,toArray:function(){return a.call(this)},get:function(t){return null!=t?t<0?this[t+this.length]:this[t]:a.call(this)},pushStack:function(t){var e=f.merge(this.constructor(),t);return e.prevObject=this,e.context=this.context,e},each:function(t,e){return f.each(this,t,e)},map:function(t){return this.pushStack(f.map(this,function(e,n){return t.call(e,n,e)}))},slice:function(){return this.pushStack(a.apply(this,arguments))},first:function(){return this.eq(0)},last:function(){return this.eq(-1)},eq:function(t){var e=this.length,n=+t+(t<0?e:0);return this.pushStack(n>=0&&n<e?[this[n]]:[])},end:function(){return this.prevObject||this.constructor(null)},push:A,sort:o.sort,splice:o.splice},f.extend=f.fn.extend=function(){var t,e,n,r,i,o,a=arguments[0]||{},s=1,A=arguments.length,l=!1;for("boolean"==typeof a&&(l=a,a=arguments[s]||{},s++),"object"==typeof a||f.isFunction(a)||(a={}),s===A&&(a=this,s--);s<A;s++)if(null!=(i=arguments[s]))for(r in i)t=a[r],a!==(n=i[r])&&(l&&n&&(f.isPlainObject(n)||(e=f.isArray(n)))?(e?(e=!1,o=t&&f.isArray(t)?t:[]):o=t&&f.isPlainObject(t)?t:{},a[r]=f.extend(l,o,n)):void 0!==n&&(a[r]=n));return a},f.extend({expando:"jQuery"+("1.11.0"+Math.random()).replace(/\D/g,""),isReady:!0,error:function(t){throw new Error(t)},noop:function(){},isFunction:function(t){return"function"===f.type(t)},isArray:Array.isArray||function(t){return"array"===f.type(t)},isWindow:function(t){return null!=t&&t==t.window},isNumeric:function(t){return t-parseFloat(t)>=0},isEmptyObject:function(t){var e;for(e in t)return!1;return!0},isPlainObject:function(t){var e;if(!t||"object"!==f.type(t)||t.nodeType||f.isWindow(t))return!1;try{if(t.constructor&&!g.call(t,"constructor")&&!g.call(t.constructor.prototype,"isPrototypeOf"))return!1}catch(t){return!1}if(d.ownLast)for(e in t)return g.call(t,e);for(e in t);return void 0===e||g.call(t,e)},type:function(t){return null==t?t+"":"object"==typeof t||"function"==typeof t?c[u.call(t)]||"object":typeof t},globalEval:function(t){t&&f.trim(t)&&(n.execScript||function(t){n.eval.call(n,t)})(t)},camelCase:function(t){return t.replace(I,"ms-").replace(C,m)},nodeName:function(t,e){return t.nodeName&&t.nodeName.toLowerCase()===e.toLowerCase()},each:function(t,e,n){var r=0,i=t.length,o=y(t);if(n){if(o)for(;r<i&&!1!==e.apply(t[r],n);r++);else for(r in t)if(!1===e.apply(t[r],n))break}else if(o)for(;r<i&&!1!==e.call(t[r],r,t[r]);r++);else for(r in t)if(!1===e.call(t[r],r,t[r]))break;return t},trim:h&&!h.call("\ufeff ")?function(t){return null==t?"":h.call(t)}:function(t){return null==t?"":(t+"").replace(p,"")},makeArray:function(t,e){var n=e||[];return null!=t&&(y(Object(t))?f.merge(n,"string"==typeof t?[t]:t):A.call(n,t)),n},inArray:function(t,e,n){var r;if(e){if(l)return l.call(e,t,n);for(r=e.length,n=n?n<0?Math.max(0,r+n):n:0;n<r;n++)if(n in e&&e[n]===t)return n}return-1},merge:function(t,e){for(var n=+e.length,r=0,i=t.length;r<n;)t[i++]=e[r++];if(n!=n)for(;void 0!==e[r];)t[i++]=e[r++];return t.length=i,t},grep:function(t,e,n){for(var r=[],i=0,o=t.length,a=!n;i<o;i++)!e(t[i],i)!==a&&r.push(t[i]);return r},map:function(t,e,n){var r,i=0,o=t.length,a=[];if(y(t))for(;i<o;i++)null!=(r=e(t[i],i,n))&&a.push(r);else for(i in t)null!=(r=e(t[i],i,n))&&a.push(r);return s.apply([],a)},guid:1,proxy:function(t,e){var n,r,i;if("string"==typeof e&&(i=t[e],e=t,t=i),f.isFunction(t))return n=a.call(arguments,2),(r=function(){return t.apply(e||this,n.concat(a.call(arguments)))}).guid=t.guid=t.guid||f.guid++,r},now:function(){return+new Date},support:d}),f.each("Boolean Number String Function Array Date RegExp Object Error".split(" "),function(t,e){c["[object "+e+"]"]=e.toLowerCase()});var E=
/*!
 * Sizzle CSS Selector Engine v1.10.16
 * http://sizzlejs.com/
 *
 * Copyright 2013 jQuery Foundation, Inc. and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2014-01-13
 */
function(t){var e,n,r,i,o,a,s,A,l,c,u,g,h,d,f,p,I,C="sizzle"+-new Date,m=t.document,y=0,E=0,v=rt(),B=rt(),b=rt(),Q=function(t,e){return t===e&&(l=!0),0},w="undefined",x=1<<31,_={}.hasOwnProperty,D=[],M=D.pop,k=D.push,S=D.push,L=D.slice,R=D.indexOf||function(t){for(var e=0,n=this.length;e<n;e++)if(this[e]===t)return e;return-1},N="checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",F="[\\x20\\t\\r\\n\\f]",T="(?:\\\\.|[\\w-]|[^\\x00-\\xa0])+",U=T.replace("w","w#"),O="\\["+F+"*("+T+")"+F+"*(?:([*^$|!~]?=)"+F+"*(?:(['\"])((?:\\\\.|[^\\\\])*?)\\3|("+U+")|)|)"+F+"*\\]",G=":("+T+")(?:\\(((['\"])((?:\\\\.|[^\\\\])*?)\\3|((?:\\\\.|[^\\\\()[\\]]|"+O.replace(3,8)+")*)|.*)\\)|)",H=new RegExp("^"+F+"+|((?:^|[^\\\\])(?:\\\\.)*)"+F+"+$","g"),j=new RegExp("^"+F+"*,"+F+"*"),Y=new RegExp("^"+F+"*([>+~]|"+F+")"+F+"*"),P=new RegExp("="+F+"*([^\\]'\"]*?)"+F+"*\\]","g"),K=new RegExp(G),z=new RegExp("^"+U+"$"),J={ID:new RegExp("^#("+T+")"),CLASS:new RegExp("^\\.("+T+")"),TAG:new RegExp("^("+T.replace("w","w*")+")"),ATTR:new RegExp("^"+O),PSEUDO:new RegExp("^"+G),CHILD:new RegExp("^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\("+F+"*(even|odd|(([+-]|)(\\d*)n|)"+F+"*(?:([+-]|)"+F+"*(\\d+)|))"+F+"*\\)|)","i"),bool:new RegExp("^(?:"+N+")$","i"),needsContext:new RegExp("^"+F+"*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\("+F+"*((?:-\\d)?\\d*)"+F+"*\\)|)(?=[^-]|$)","i")},q=/^(?:input|select|textarea|button)$/i,V=/^h\d$/i,W=/^[^{]+\{\s*\[native \w/,X=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,Z=/[+~]/,$=/'|\\/g,tt=new RegExp("\\\\([\\da-f]{1,6}"+F+"?|("+F+")|.)","ig"),et=function(t,e,n){var r="0x"+e-65536;return r!=r||n?e:r<0?String.fromCharCode(r+65536):String.fromCharCode(r>>10|55296,1023&r|56320)};try{S.apply(D=L.call(m.childNodes),m.childNodes),D[m.childNodes.length].nodeType}catch(t){S={apply:D.length?function(t,e){k.apply(t,L.call(e))}:function(t,e){for(var n=t.length,r=0;t[n++]=e[r++];);t.length=n-1}}}function nt(t,e,i,o){var s,A,l,g,f,p,y,E,v,B;if((e?e.ownerDocument||e:m)!==u&&c(e),i=i||[],!t||"string"!=typeof t)return i;if(1!==(g=(e=e||u).nodeType)&&9!==g)return[];if(h&&!o){if(s=X.exec(t))if(l=s[1]){if(9===g){if(!(A=e.getElementById(l))||!A.parentNode)return i;if(A.id===l)return i.push(A),i}else if(e.ownerDocument&&(A=e.ownerDocument.getElementById(l))&&I(e,A)&&A.id===l)return i.push(A),i}else{if(s[2])return S.apply(i,e.getElementsByTagName(t)),i;if((l=s[3])&&n.getElementsByClassName&&e.getElementsByClassName)return S.apply(i,e.getElementsByClassName(l)),i}if(n.qsa&&(!d||!d.test(t))){if(E=y=C,v=e,B=9===g&&t,1===g&&"object"!==e.nodeName.toLowerCase()){for(p=ht(t),(y=e.getAttribute("id"))?E=y.replace($,"\\$&"):e.setAttribute("id",E),E="[id='"+E+"'] ",f=p.length;f--;)p[f]=E+dt(p[f]);v=Z.test(t)&&ut(e.parentNode)||e,B=p.join(",")}if(B)try{return S.apply(i,v.querySelectorAll(B)),i}catch(t){}finally{y||e.removeAttribute("id")}}}return function(t,e,i,o){var s,A,l,c,u,g=ht(t);if(!o&&1===g.length){if((A=g[0]=g[0].slice(0)).length>2&&"ID"===(l=A[0]).type&&n.getById&&9===e.nodeType&&h&&r.relative[A[1].type]){if(!(e=(r.find.ID(l.matches[0].replace(tt,et),e)||[])[0]))return i;t=t.slice(A.shift().value.length)}for(s=J.needsContext.test(t)?0:A.length;s--&&(l=A[s],!r.relative[c=l.type]);)if((u=r.find[c])&&(o=u(l.matches[0].replace(tt,et),Z.test(A[0].type)&&ut(e.parentNode)||e))){if(A.splice(s,1),!(t=o.length&&dt(A)))return S.apply(i,o),i;break}}return a(t,g)(o,e,!h,i,Z.test(t)&&ut(e.parentNode)||e),i}(t.replace(H,"$1"),e,i,o)}function rt(){var t=[];return function e(n,i){return t.push(n+" ")>r.cacheLength&&delete e[t.shift()],e[n+" "]=i}}function it(t){return t[C]=!0,t}function ot(t){var e=u.createElement("div");try{return!!t(e)}catch(t){return!1}finally{e.parentNode&&e.parentNode.removeChild(e),e=null}}function at(t,e){for(var n=t.split("|"),i=t.length;i--;)r.attrHandle[n[i]]=e}function st(t,e){var n=e&&t,r=n&&1===t.nodeType&&1===e.nodeType&&(~e.sourceIndex||x)-(~t.sourceIndex||x);if(r)return r;if(n)for(;n=n.nextSibling;)if(n===e)return-1;return t?1:-1}function At(t){return function(e){return"input"===e.nodeName.toLowerCase()&&e.type===t}}function lt(t){return function(e){var n=e.nodeName.toLowerCase();return("input"===n||"button"===n)&&e.type===t}}function ct(t){return it(function(e){return e=+e,it(function(n,r){for(var i,o=t([],n.length,e),a=o.length;a--;)n[i=o[a]]&&(n[i]=!(r[i]=n[i]))})})}function ut(t){return t&&typeof t.getElementsByTagName!==w&&t}for(e in n=nt.support={},o=nt.isXML=function(t){var e=t&&(t.ownerDocument||t).documentElement;return!!e&&"HTML"!==e.nodeName},c=nt.setDocument=function(t){var e,i=t?t.ownerDocument||t:m,a=i.defaultView;return i!==u&&9===i.nodeType&&i.documentElement?(u=i,g=i.documentElement,h=!o(i),a&&a!==a.top&&(a.addEventListener?a.addEventListener("unload",function(){c()},!1):a.attachEvent&&a.attachEvent("onunload",function(){c()})),n.attributes=ot(function(t){return t.className="i",!t.getAttribute("className")}),n.getElementsByTagName=ot(function(t){return t.appendChild(i.createComment("")),!t.getElementsByTagName("*").length}),n.getElementsByClassName=W.test(i.getElementsByClassName)&&ot(function(t){return t.innerHTML="<div class='a'></div><div class='a i'></div>",t.firstChild.className="i",2===t.getElementsByClassName("i").length}),n.getById=ot(function(t){return g.appendChild(t).id=C,!i.getElementsByName||!i.getElementsByName(C).length}),n.getById?(r.find.ID=function(t,e){if(typeof e.getElementById!==w&&h){var n=e.getElementById(t);return n&&n.parentNode?[n]:[]}},r.filter.ID=function(t){var e=t.replace(tt,et);return function(t){return t.getAttribute("id")===e}}):(delete r.find.ID,r.filter.ID=function(t){var e=t.replace(tt,et);return function(t){var n=typeof t.getAttributeNode!==w&&t.getAttributeNode("id");return n&&n.value===e}}),r.find.TAG=n.getElementsByTagName?function(t,e){if(typeof e.getElementsByTagName!==w)return e.getElementsByTagName(t)}:function(t,e){var n,r=[],i=0,o=e.getElementsByTagName(t);if("*"===t){for(;n=o[i++];)1===n.nodeType&&r.push(n);return r}return o},r.find.CLASS=n.getElementsByClassName&&function(t,e){if(typeof e.getElementsByClassName!==w&&h)return e.getElementsByClassName(t)},f=[],d=[],(n.qsa=W.test(i.querySelectorAll))&&(ot(function(t){t.innerHTML="<select t=''><option selected=''></option></select>",t.querySelectorAll("[t^='']").length&&d.push("[*^$]="+F+"*(?:''|\"\")"),t.querySelectorAll("[selected]").length||d.push("\\["+F+"*(?:value|"+N+")"),t.querySelectorAll(":checked").length||d.push(":checked")}),ot(function(t){var e=i.createElement("input");e.setAttribute("type","hidden"),t.appendChild(e).setAttribute("name","D"),t.querySelectorAll("[name=d]").length&&d.push("name"+F+"*[*^$|!~]?="),t.querySelectorAll(":enabled").length||d.push(":enabled",":disabled"),t.querySelectorAll("*,:x"),d.push(",.*:")})),(n.matchesSelector=W.test(p=g.webkitMatchesSelector||g.mozMatchesSelector||g.oMatchesSelector||g.msMatchesSelector))&&ot(function(t){n.disconnectedMatch=p.call(t,"div"),p.call(t,"[s!='']:x"),f.push("!=",G)}),d=d.length&&new RegExp(d.join("|")),f=f.length&&new RegExp(f.join("|")),e=W.test(g.compareDocumentPosition),I=e||W.test(g.contains)?function(t,e){var n=9===t.nodeType?t.documentElement:t,r=e&&e.parentNode;return t===r||!(!r||1!==r.nodeType||!(n.contains?n.contains(r):t.compareDocumentPosition&&16&t.compareDocumentPosition(r)))}:function(t,e){if(e)for(;e=e.parentNode;)if(e===t)return!0;return!1},Q=e?function(t,e){if(t===e)return l=!0,0;var r=!t.compareDocumentPosition-!e.compareDocumentPosition;return r||(1&(r=(t.ownerDocument||t)===(e.ownerDocument||e)?t.compareDocumentPosition(e):1)||!n.sortDetached&&e.compareDocumentPosition(t)===r?t===i||t.ownerDocument===m&&I(m,t)?-1:e===i||e.ownerDocument===m&&I(m,e)?1:A?R.call(A,t)-R.call(A,e):0:4&r?-1:1)}:function(t,e){if(t===e)return l=!0,0;var n,r=0,o=t.parentNode,a=e.parentNode,s=[t],c=[e];if(!o||!a)return t===i?-1:e===i?1:o?-1:a?1:A?R.call(A,t)-R.call(A,e):0;if(o===a)return st(t,e);for(n=t;n=n.parentNode;)s.unshift(n);for(n=e;n=n.parentNode;)c.unshift(n);for(;s[r]===c[r];)r++;return r?st(s[r],c[r]):s[r]===m?-1:c[r]===m?1:0},i):u},nt.matches=function(t,e){return nt(t,null,null,e)},nt.matchesSelector=function(t,e){if((t.ownerDocument||t)!==u&&c(t),e=e.replace(P,"='$1']"),n.matchesSelector&&h&&(!f||!f.test(e))&&(!d||!d.test(e)))try{var r=p.call(t,e);if(r||n.disconnectedMatch||t.document&&11!==t.document.nodeType)return r}catch(t){}return nt(e,u,null,[t]).length>0},nt.contains=function(t,e){return(t.ownerDocument||t)!==u&&c(t),I(t,e)},nt.attr=function(t,e){(t.ownerDocument||t)!==u&&c(t);var i=r.attrHandle[e.toLowerCase()],o=i&&_.call(r.attrHandle,e.toLowerCase())?i(t,e,!h):void 0;return void 0!==o?o:n.attributes||!h?t.getAttribute(e):(o=t.getAttributeNode(e))&&o.specified?o.value:null},nt.error=function(t){throw new Error("Syntax error, unrecognized expression: "+t)},nt.uniqueSort=function(t){var e,r=[],i=0,o=0;if(l=!n.detectDuplicates,A=!n.sortStable&&t.slice(0),t.sort(Q),l){for(;e=t[o++];)e===t[o]&&(i=r.push(o));for(;i--;)t.splice(r[i],1)}return A=null,t},i=nt.getText=function(t){var e,n="",r=0,o=t.nodeType;if(o){if(1===o||9===o||11===o){if("string"==typeof t.textContent)return t.textContent;for(t=t.firstChild;t;t=t.nextSibling)n+=i(t)}else if(3===o||4===o)return t.nodeValue}else for(;e=t[r++];)n+=i(e);return n},(r=nt.selectors={cacheLength:50,createPseudo:it,match:J,attrHandle:{},find:{},relative:{">":{dir:"parentNode",first:!0}," ":{dir:"parentNode"},"+":{dir:"previousSibling",first:!0},"~":{dir:"previousSibling"}},preFilter:{ATTR:function(t){return t[1]=t[1].replace(tt,et),t[3]=(t[4]||t[5]||"").replace(tt,et),"~="===t[2]&&(t[3]=" "+t[3]+" "),t.slice(0,4)},CHILD:function(t){return t[1]=t[1].toLowerCase(),"nth"===t[1].slice(0,3)?(t[3]||nt.error(t[0]),t[4]=+(t[4]?t[5]+(t[6]||1):2*("even"===t[3]||"odd"===t[3])),t[5]=+(t[7]+t[8]||"odd"===t[3])):t[3]&&nt.error(t[0]),t},PSEUDO:function(t){var e,n=!t[5]&&t[2];return J.CHILD.test(t[0])?null:(t[3]&&void 0!==t[4]?t[2]=t[4]:n&&K.test(n)&&(e=ht(n,!0))&&(e=n.indexOf(")",n.length-e)-n.length)&&(t[0]=t[0].slice(0,e),t[2]=n.slice(0,e)),t.slice(0,3))}},filter:{TAG:function(t){var e=t.replace(tt,et).toLowerCase();return"*"===t?function(){return!0}:function(t){return t.nodeName&&t.nodeName.toLowerCase()===e}},CLASS:function(t){var e=v[t+" "];return e||(e=new RegExp("(^|"+F+")"+t+"("+F+"|$)"))&&v(t,function(t){return e.test("string"==typeof t.className&&t.className||typeof t.getAttribute!==w&&t.getAttribute("class")||"")})},ATTR:function(t,e,n){return function(r){var i=nt.attr(r,t);return null==i?"!="===e:!e||(i+="","="===e?i===n:"!="===e?i!==n:"^="===e?n&&0===i.indexOf(n):"*="===e?n&&i.indexOf(n)>-1:"$="===e?n&&i.slice(-n.length)===n:"~="===e?(" "+i+" ").indexOf(n)>-1:"|="===e&&(i===n||i.slice(0,n.length+1)===n+"-"))}},CHILD:function(t,e,n,r,i){var o="nth"!==t.slice(0,3),a="last"!==t.slice(-4),s="of-type"===e;return 1===r&&0===i?function(t){return!!t.parentNode}:function(e,n,A){var l,c,u,g,h,d,f=o!==a?"nextSibling":"previousSibling",p=e.parentNode,I=s&&e.nodeName.toLowerCase(),m=!A&&!s;if(p){if(o){for(;f;){for(u=e;u=u[f];)if(s?u.nodeName.toLowerCase()===I:1===u.nodeType)return!1;d=f="only"===t&&!d&&"nextSibling"}return!0}if(d=[a?p.firstChild:p.lastChild],a&&m){for(h=(l=(c=p[C]||(p[C]={}))[t]||[])[0]===y&&l[1],g=l[0]===y&&l[2],u=h&&p.childNodes[h];u=++h&&u&&u[f]||(g=h=0)||d.pop();)if(1===u.nodeType&&++g&&u===e){c[t]=[y,h,g];break}}else if(m&&(l=(e[C]||(e[C]={}))[t])&&l[0]===y)g=l[1];else for(;(u=++h&&u&&u[f]||(g=h=0)||d.pop())&&((s?u.nodeName.toLowerCase()!==I:1!==u.nodeType)||!++g||(m&&((u[C]||(u[C]={}))[t]=[y,g]),u!==e)););return(g-=i)===r||g%r==0&&g/r>=0}}},PSEUDO:function(t,e){var n,i=r.pseudos[t]||r.setFilters[t.toLowerCase()]||nt.error("unsupported pseudo: "+t);return i[C]?i(e):i.length>1?(n=[t,t,"",e],r.setFilters.hasOwnProperty(t.toLowerCase())?it(function(t,n){for(var r,o=i(t,e),a=o.length;a--;)t[r=R.call(t,o[a])]=!(n[r]=o[a])}):function(t){return i(t,0,n)}):i}},pseudos:{not:it(function(t){var e=[],n=[],r=a(t.replace(H,"$1"));return r[C]?it(function(t,e,n,i){for(var o,a=r(t,null,i,[]),s=t.length;s--;)(o=a[s])&&(t[s]=!(e[s]=o))}):function(t,i,o){return e[0]=t,r(e,null,o,n),!n.pop()}}),has:it(function(t){return function(e){return nt(t,e).length>0}}),contains:it(function(t){return function(e){return(e.textContent||e.innerText||i(e)).indexOf(t)>-1}}),lang:it(function(t){return z.test(t||"")||nt.error("unsupported lang: "+t),t=t.replace(tt,et).toLowerCase(),function(e){var n;do{if(n=h?e.lang:e.getAttribute("xml:lang")||e.getAttribute("lang"))return(n=n.toLowerCase())===t||0===n.indexOf(t+"-")}while((e=e.parentNode)&&1===e.nodeType);return!1}}),target:function(e){var n=t.location&&t.location.hash;return n&&n.slice(1)===e.id},root:function(t){return t===g},focus:function(t){return t===u.activeElement&&(!u.hasFocus||u.hasFocus())&&!!(t.type||t.href||~t.tabIndex)},enabled:function(t){return!1===t.disabled},disabled:function(t){return!0===t.disabled},checked:function(t){var e=t.nodeName.toLowerCase();return"input"===e&&!!t.checked||"option"===e&&!!t.selected},selected:function(t){return t.parentNode&&t.parentNode.selectedIndex,!0===t.selected},empty:function(t){for(t=t.firstChild;t;t=t.nextSibling)if(t.nodeType<6)return!1;return!0},parent:function(t){return!r.pseudos.empty(t)},header:function(t){return V.test(t.nodeName)},input:function(t){return q.test(t.nodeName)},button:function(t){var e=t.nodeName.toLowerCase();return"input"===e&&"button"===t.type||"button"===e},text:function(t){var e;return"input"===t.nodeName.toLowerCase()&&"text"===t.type&&(null==(e=t.getAttribute("type"))||"text"===e.toLowerCase())},first:ct(function(){return[0]}),last:ct(function(t,e){return[e-1]}),eq:ct(function(t,e,n){return[n<0?n+e:n]}),even:ct(function(t,e){for(var n=0;n<e;n+=2)t.push(n);return t}),odd:ct(function(t,e){for(var n=1;n<e;n+=2)t.push(n);return t}),lt:ct(function(t,e,n){for(var r=n<0?n+e:n;--r>=0;)t.push(r);return t}),gt:ct(function(t,e,n){for(var r=n<0?n+e:n;++r<e;)t.push(r);return t})}}).pseudos.nth=r.pseudos.eq,{radio:!0,checkbox:!0,file:!0,password:!0,image:!0})r.pseudos[e]=At(e);for(e in{submit:!0,reset:!0})r.pseudos[e]=lt(e);function gt(){}function ht(t,e){var n,i,o,a,s,A,l,c=B[t+" "];if(c)return e?0:c.slice(0);for(s=t,A=[],l=r.preFilter;s;){for(a in n&&!(i=j.exec(s))||(i&&(s=s.slice(i[0].length)||s),A.push(o=[])),n=!1,(i=Y.exec(s))&&(n=i.shift(),o.push({value:n,type:i[0].replace(H," ")}),s=s.slice(n.length)),r.filter)!(i=J[a].exec(s))||l[a]&&!(i=l[a](i))||(n=i.shift(),o.push({value:n,type:a,matches:i}),s=s.slice(n.length));if(!n)break}return e?s.length:s?nt.error(t):B(t,A).slice(0)}function dt(t){for(var e=0,n=t.length,r="";e<n;e++)r+=t[e].value;return r}function ft(t,e,n){var r=e.dir,i=n&&"parentNode"===r,o=E++;return e.first?function(e,n,o){for(;e=e[r];)if(1===e.nodeType||i)return t(e,n,o)}:function(e,n,a){var s,A,l=[y,o];if(a){for(;e=e[r];)if((1===e.nodeType||i)&&t(e,n,a))return!0}else for(;e=e[r];)if(1===e.nodeType||i){if((s=(A=e[C]||(e[C]={}))[r])&&s[0]===y&&s[1]===o)return l[2]=s[2];if(A[r]=l,l[2]=t(e,n,a))return!0}}}function pt(t){return t.length>1?function(e,n,r){for(var i=t.length;i--;)if(!t[i](e,n,r))return!1;return!0}:t[0]}function It(t,e,n,r,i){for(var o,a=[],s=0,A=t.length,l=null!=e;s<A;s++)(o=t[s])&&(n&&!n(o,r,i)||(a.push(o),l&&e.push(s)));return a}function Ct(t,e,n,r,i,o){return r&&!r[C]&&(r=Ct(r)),i&&!i[C]&&(i=Ct(i,o)),it(function(o,a,s,A){var l,c,u,g=[],h=[],d=a.length,f=o||function(t,e,n){for(var r=0,i=e.length;r<i;r++)nt(t,e[r],n);return n}(e||"*",s.nodeType?[s]:s,[]),p=!t||!o&&e?f:It(f,g,t,s,A),I=n?i||(o?t:d||r)?[]:a:p;if(n&&n(p,I,s,A),r)for(l=It(I,h),r(l,[],s,A),c=l.length;c--;)(u=l[c])&&(I[h[c]]=!(p[h[c]]=u));if(o){if(i||t){if(i){for(l=[],c=I.length;c--;)(u=I[c])&&l.push(p[c]=u);i(null,I=[],l,A)}for(c=I.length;c--;)(u=I[c])&&(l=i?R.call(o,u):g[c])>-1&&(o[l]=!(a[l]=u))}}else I=It(I===a?I.splice(d,I.length):I),i?i(null,a,I,A):S.apply(a,I)})}function mt(t){for(var e,n,i,o=t.length,a=r.relative[t[0].type],A=a||r.relative[" "],l=a?1:0,c=ft(function(t){return t===e},A,!0),u=ft(function(t){return R.call(e,t)>-1},A,!0),g=[function(t,n,r){return!a&&(r||n!==s)||((e=n).nodeType?c(t,n,r):u(t,n,r))}];l<o;l++)if(n=r.relative[t[l].type])g=[ft(pt(g),n)];else{if((n=r.filter[t[l].type].apply(null,t[l].matches))[C]){for(i=++l;i<o&&!r.relative[t[i].type];i++);return Ct(l>1&&pt(g),l>1&&dt(t.slice(0,l-1).concat({value:" "===t[l-2].type?"*":""})).replace(H,"$1"),n,l<i&&mt(t.slice(l,i)),i<o&&mt(t=t.slice(i)),i<o&&dt(t))}g.push(n)}return pt(g)}return gt.prototype=r.filters=r.pseudos,r.setFilters=new gt,a=nt.compile=function(t,e){var n,i=[],o=[],a=b[t+" "];if(!a){for(e||(e=ht(t)),n=e.length;n--;)(a=mt(e[n]))[C]?i.push(a):o.push(a);a=b(t,function(t,e){var n=e.length>0,i=t.length>0,o=function(o,a,A,l,c){var g,h,d,f=0,p="0",I=o&&[],C=[],m=s,E=o||i&&r.find.TAG("*",c),v=y+=null==m?1:Math.random()||.1,B=E.length;for(c&&(s=a!==u&&a);p!==B&&null!=(g=E[p]);p++){if(i&&g){for(h=0;d=t[h++];)if(d(g,a,A)){l.push(g);break}c&&(y=v)}n&&((g=!d&&g)&&f--,o&&I.push(g))}if(f+=p,n&&p!==f){for(h=0;d=e[h++];)d(I,C,a,A);if(o){if(f>0)for(;p--;)I[p]||C[p]||(C[p]=M.call(l));C=It(C)}S.apply(l,C),c&&!o&&C.length>0&&f+e.length>1&&nt.uniqueSort(l)}return c&&(y=v,s=m),I};return n?it(o):o}(o,i))}return a},n.sortStable=C.split("").sort(Q).join("")===C,n.detectDuplicates=!!l,c(),n.sortDetached=ot(function(t){return 1&t.compareDocumentPosition(u.createElement("div"))}),ot(function(t){return t.innerHTML="<a href='#'></a>","#"===t.firstChild.getAttribute("href")})||at("type|href|height|width",function(t,e,n){if(!n)return t.getAttribute(e,"type"===e.toLowerCase()?1:2)}),n.attributes&&ot(function(t){return t.innerHTML="<input/>",t.firstChild.setAttribute("value",""),""===t.firstChild.getAttribute("value")})||at("value",function(t,e,n){if(!n&&"input"===t.nodeName.toLowerCase())return t.defaultValue}),ot(function(t){return null==t.getAttribute("disabled")})||at(N,function(t,e,n){var r;if(!n)return!0===t[e]?e.toLowerCase():(r=t.getAttributeNode(e))&&r.specified?r.value:null}),nt}(n);f.find=E,f.expr=E.selectors,f.expr[":"]=f.expr.pseudos,f.unique=E.uniqueSort,f.text=E.getText,f.isXMLDoc=E.isXML,f.contains=E.contains;var v=f.expr.match.needsContext,B=/^<(\w+)\s*\/?>(?:<\/\1>|)$/,b=/^.[^:#\[\.,]*$/;function Q(t,e,n){if(f.isFunction(e))return f.grep(t,function(t,r){return!!e.call(t,r,t)!==n});if(e.nodeType)return f.grep(t,function(t){return t===e!==n});if("string"==typeof e){if(b.test(e))return f.filter(e,t,n);e=f.filter(e,t)}return f.grep(t,function(t){return f.inArray(t,e)>=0!==n})}f.filter=function(t,e,n){var r=e[0];return n&&(t=":not("+t+")"),1===e.length&&1===r.nodeType?f.find.matchesSelector(r,t)?[r]:[]:f.find.matches(t,f.grep(e,function(t){return 1===t.nodeType}))},f.fn.extend({find:function(t){var e,n=[],r=this,i=r.length;if("string"!=typeof t)return this.pushStack(f(t).filter(function(){for(e=0;e<i;e++)if(f.contains(r[e],this))return!0}));for(e=0;e<i;e++)f.find(t,r[e],n);return(n=this.pushStack(i>1?f.unique(n):n)).selector=this.selector?this.selector+" "+t:t,n},filter:function(t){return this.pushStack(Q(this,t||[],!1))},not:function(t){return this.pushStack(Q(this,t||[],!0))},is:function(t){return!!Q(this,"string"==typeof t&&v.test(t)?f(t):t||[],!1).length}});var w,x=n.document,_=/^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]*))$/;(f.fn.init=function(t,e){var n,r;if(!t)return this;if("string"==typeof t){if(!(n="<"===t.charAt(0)&&">"===t.charAt(t.length-1)&&t.length>=3?[null,t,null]:_.exec(t))||!n[1]&&e)return!e||e.jquery?(e||w).find(t):this.constructor(e).find(t);if(n[1]){if(e=e instanceof f?e[0]:e,f.merge(this,f.parseHTML(n[1],e&&e.nodeType?e.ownerDocument||e:x,!0)),B.test(n[1])&&f.isPlainObject(e))for(n in e)f.isFunction(this[n])?this[n](e[n]):this.attr(n,e[n]);return this}if((r=x.getElementById(n[2]))&&r.parentNode){if(r.id!==n[2])return w.find(t);this.length=1,this[0]=r}return this.context=x,this.selector=t,this}return t.nodeType?(this.context=this[0]=t,this.length=1,this):f.isFunction(t)?void 0!==w.ready?w.ready(t):t(f):(void 0!==t.selector&&(this.selector=t.selector,this.context=t.context),f.makeArray(t,this))}).prototype=f.fn,w=f(x);var D=/^(?:parents|prev(?:Until|All))/,M={children:!0,contents:!0,next:!0,prev:!0};function k(t,e){do{t=t[e]}while(t&&1!==t.nodeType);return t}f.extend({dir:function(t,e,n){for(var r=[],i=t[e];i&&9!==i.nodeType&&(void 0===n||1!==i.nodeType||!f(i).is(n));)1===i.nodeType&&r.push(i),i=i[e];return r},sibling:function(t,e){for(var n=[];t;t=t.nextSibling)1===t.nodeType&&t!==e&&n.push(t);return n}}),f.fn.extend({has:function(t){var e,n=f(t,this),r=n.length;return this.filter(function(){for(e=0;e<r;e++)if(f.contains(this,n[e]))return!0})},closest:function(t,e){for(var n,r=0,i=this.length,o=[],a=v.test(t)||"string"!=typeof t?f(t,e||this.context):0;r<i;r++)for(n=this[r];n&&n!==e;n=n.parentNode)if(n.nodeType<11&&(a?a.index(n)>-1:1===n.nodeType&&f.find.matchesSelector(n,t))){o.push(n);break}return this.pushStack(o.length>1?f.unique(o):o)},index:function(t){return t?"string"==typeof t?f.inArray(this[0],f(t)):f.inArray(t.jquery?t[0]:t,this):this[0]&&this[0].parentNode?this.first().prevAll().length:-1},add:function(t,e){return this.pushStack(f.unique(f.merge(this.get(),f(t,e))))},addBack:function(t){return this.add(null==t?this.prevObject:this.prevObject.filter(t))}}),f.each({parent:function(t){var e=t.parentNode;return e&&11!==e.nodeType?e:null},parents:function(t){return f.dir(t,"parentNode")},parentsUntil:function(t,e,n){return f.dir(t,"parentNode",n)},next:function(t){return k(t,"nextSibling")},prev:function(t){return k(t,"previousSibling")},nextAll:function(t){return f.dir(t,"nextSibling")},prevAll:function(t){return f.dir(t,"previousSibling")},nextUntil:function(t,e,n){return f.dir(t,"nextSibling",n)},prevUntil:function(t,e,n){return f.dir(t,"previousSibling",n)},siblings:function(t){return f.sibling((t.parentNode||{}).firstChild,t)},children:function(t){return f.sibling(t.firstChild)},contents:function(t){return f.nodeName(t,"iframe")?t.contentDocument||t.contentWindow.document:f.merge([],t.childNodes)}},function(t,e){f.fn[t]=function(n,r){var i=f.map(this,e,n);return"Until"!==t.slice(-5)&&(r=n),r&&"string"==typeof r&&(i=f.filter(r,i)),this.length>1&&(M[t]||(i=f.unique(i)),D.test(t)&&(i=i.reverse())),this.pushStack(i)}});var S,L=/\S+/g,R={};function N(){x.addEventListener?(x.removeEventListener("DOMContentLoaded",F,!1),n.removeEventListener("load",F,!1)):(x.detachEvent("onreadystatechange",F),n.detachEvent("onload",F))}function F(){(x.addEventListener||"load"===event.type||"complete"===x.readyState)&&(N(),f.ready())}f.Callbacks=function(t){t="string"==typeof t?R[t]||function(t){var e=R[t]={};return f.each(t.match(L)||[],function(t,n){e[n]=!0}),e}(t):f.extend({},t);var e,n,r,i,o,a,s=[],A=!t.once&&[],l=function(u){for(n=t.memory&&u,r=!0,o=a||0,a=0,i=s.length,e=!0;s&&o<i;o++)if(!1===s[o].apply(u[0],u[1])&&t.stopOnFalse){n=!1;break}e=!1,s&&(A?A.length&&l(A.shift()):n?s=[]:c.disable())},c={add:function(){if(s){var r=s.length;!function e(n){f.each(n,function(n,r){var i=f.type(r);"function"===i?t.unique&&c.has(r)||s.push(r):r&&r.length&&"string"!==i&&e(r)})}(arguments),e?i=s.length:n&&(a=r,l(n))}return this},remove:function(){return s&&f.each(arguments,function(t,n){for(var r;(r=f.inArray(n,s,r))>-1;)s.splice(r,1),e&&(r<=i&&i--,r<=o&&o--)}),this},has:function(t){return t?f.inArray(t,s)>-1:!(!s||!s.length)},empty:function(){return s=[],i=0,this},disable:function(){return s=A=n=void 0,this},disabled:function(){return!s},lock:function(){return A=void 0,n||c.disable(),this},locked:function(){return!A},fireWith:function(t,n){return!s||r&&!A||(n=[t,(n=n||[]).slice?n.slice():n],e?A.push(n):l(n)),this},fire:function(){return c.fireWith(this,arguments),this},fired:function(){return!!r}};return c},f.extend({Deferred:function(t){var e=[["resolve","done",f.Callbacks("once memory"),"resolved"],["reject","fail",f.Callbacks("once memory"),"rejected"],["notify","progress",f.Callbacks("memory")]],n="pending",r={state:function(){return n},always:function(){return i.done(arguments).fail(arguments),this},then:function(){var t=arguments;return f.Deferred(function(n){f.each(e,function(e,o){var a=f.isFunction(t[e])&&t[e];i[o[1]](function(){var t=a&&a.apply(this,arguments);t&&f.isFunction(t.promise)?t.promise().done(n.resolve).fail(n.reject).progress(n.notify):n[o[0]+"With"](this===r?n.promise():this,a?[t]:arguments)})}),t=null}).promise()},promise:function(t){return null!=t?f.extend(t,r):r}},i={};return r.pipe=r.then,f.each(e,function(t,o){var a=o[2],s=o[3];r[o[1]]=a.add,s&&a.add(function(){n=s},e[1^t][2].disable,e[2][2].lock),i[o[0]]=function(){return i[o[0]+"With"](this===i?r:this,arguments),this},i[o[0]+"With"]=a.fireWith}),r.promise(i),t&&t.call(i,i),i},when:function(t){var e,n,r,i=0,o=a.call(arguments),s=o.length,A=1!==s||t&&f.isFunction(t.promise)?s:0,l=1===A?t:f.Deferred(),c=function(t,n,r){return function(i){n[t]=this,r[t]=arguments.length>1?a.call(arguments):i,r===e?l.notifyWith(n,r):--A||l.resolveWith(n,r)}};if(s>1)for(e=new Array(s),n=new Array(s),r=new Array(s);i<s;i++)o[i]&&f.isFunction(o[i].promise)?o[i].promise().done(c(i,r,o)).fail(l.reject).progress(c(i,n,e)):--A;return A||l.resolveWith(r,o),l.promise()}}),f.fn.ready=function(t){return f.ready.promise().done(t),this},f.extend({isReady:!1,readyWait:1,holdReady:function(t){t?f.readyWait++:f.ready(!0)},ready:function(t){if(!0===t?!--f.readyWait:!f.isReady){if(!x.body)return setTimeout(f.ready);f.isReady=!0,!0!==t&&--f.readyWait>0||(S.resolveWith(x,[f]),f.fn.trigger&&f(x).trigger("ready").off("ready"))}}}),f.ready.promise=function(t){if(!S)if(S=f.Deferred(),"complete"===x.readyState)setTimeout(f.ready);else if(x.addEventListener)x.addEventListener("DOMContentLoaded",F,!1),n.addEventListener("load",F,!1);else{x.attachEvent("onreadystatechange",F),n.attachEvent("onload",F);var e=!1;try{e=null==n.frameElement&&x.documentElement}catch(t){}e&&e.doScroll&&function t(){if(!f.isReady){try{e.doScroll("left")}catch(e){return setTimeout(t,50)}N(),f.ready()}}()}return S.promise(t)};var T,U="undefined";for(T in f(d))break;d.ownLast="0"!==T,d.inlineBlockNeedsLayout=!1,f(function(){var t,e,n=x.getElementsByTagName("body")[0];n&&((t=x.createElement("div")).style.cssText="border:0;width:0;height:0;position:absolute;top:0;left:-9999px;margin-top:1px",e=x.createElement("div"),n.appendChild(t).appendChild(e),typeof e.style.zoom!==U&&(e.style.cssText="border:0;margin:0;width:1px;padding:1px;display:inline;zoom:1",(d.inlineBlockNeedsLayout=3===e.offsetWidth)&&(n.style.zoom=1)),n.removeChild(t),t=e=null)}),function(){var t=x.createElement("div");if(null==d.deleteExpando){d.deleteExpando=!0;try{delete t.test}catch(t){d.deleteExpando=!1}}t=null}(),f.acceptData=function(t){var e=f.noData[(t.nodeName+" ").toLowerCase()],n=+t.nodeType||1;return(1===n||9===n)&&(!e||!0!==e&&t.getAttribute("classid")===e)};var O=/^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,G=/([A-Z])/g;function H(t,e,n){if(void 0===n&&1===t.nodeType){var r="data-"+e.replace(G,"-$1").toLowerCase();if("string"==typeof(n=t.getAttribute(r))){try{n="true"===n||"false"!==n&&("null"===n?null:+n+""===n?+n:O.test(n)?f.parseJSON(n):n)}catch(t){}f.data(t,e,n)}else n=void 0}return n}function j(t){var e;for(e in t)if(("data"!==e||!f.isEmptyObject(t[e]))&&"toJSON"!==e)return!1;return!0}function Y(t,e,n,r){if(f.acceptData(t)){var i,a,s=f.expando,A=t.nodeType,l=A?f.cache:t,c=A?t[s]:t[s]&&s;if(c&&l[c]&&(r||l[c].data)||void 0!==n||"string"!=typeof e)return c||(c=A?t[s]=o.pop()||f.guid++:s),l[c]||(l[c]=A?{}:{toJSON:f.noop}),"object"!=typeof e&&"function"!=typeof e||(r?l[c]=f.extend(l[c],e):l[c].data=f.extend(l[c].data,e)),a=l[c],r||(a.data||(a.data={}),a=a.data),void 0!==n&&(a[f.camelCase(e)]=n),"string"==typeof e?null==(i=a[e])&&(i=a[f.camelCase(e)]):i=a,i}}function P(t,e,n){if(f.acceptData(t)){var r,i,o=t.nodeType,a=o?f.cache:t,s=o?t[f.expando]:f.expando;if(a[s]){if(e&&(r=n?a[s]:a[s].data)){i=(e=f.isArray(e)?e.concat(f.map(e,f.camelCase)):e in r?[e]:(e=f.camelCase(e))in r?[e]:e.split(" ")).length;for(;i--;)delete r[e[i]];if(n?!j(r):!f.isEmptyObject(r))return}(n||(delete a[s].data,j(a[s])))&&(o?f.cleanData([t],!0):d.deleteExpando||a!=a.window?delete a[s]:a[s]=null)}}}f.extend({cache:{},noData:{"applet ":!0,"embed ":!0,"object ":"clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"},hasData:function(t){return!!(t=t.nodeType?f.cache[t[f.expando]]:t[f.expando])&&!j(t)},data:function(t,e,n){return Y(t,e,n)},removeData:function(t,e){return P(t,e)},_data:function(t,e,n){return Y(t,e,n,!0)},_removeData:function(t,e){return P(t,e,!0)}}),f.fn.extend({data:function(t,e){var n,r,i,o=this[0],a=o&&o.attributes;if(void 0===t){if(this.length&&(i=f.data(o),1===o.nodeType&&!f._data(o,"parsedAttrs"))){for(n=a.length;n--;)0===(r=a[n].name).indexOf("data-")&&H(o,r=f.camelCase(r.slice(5)),i[r]);f._data(o,"parsedAttrs",!0)}return i}return"object"==typeof t?this.each(function(){f.data(this,t)}):arguments.length>1?this.each(function(){f.data(this,t,e)}):o?H(o,t,f.data(o,t)):void 0},removeData:function(t){return this.each(function(){f.removeData(this,t)})}}),f.extend({queue:function(t,e,n){var r;if(t)return e=(e||"fx")+"queue",r=f._data(t,e),n&&(!r||f.isArray(n)?r=f._data(t,e,f.makeArray(n)):r.push(n)),r||[]},dequeue:function(t,e){e=e||"fx";var n=f.queue(t,e),r=n.length,i=n.shift(),o=f._queueHooks(t,e);"inprogress"===i&&(i=n.shift(),r--),i&&("fx"===e&&n.unshift("inprogress"),delete o.stop,i.call(t,function(){f.dequeue(t,e)},o)),!r&&o&&o.empty.fire()},_queueHooks:function(t,e){var n=e+"queueHooks";return f._data(t,n)||f._data(t,n,{empty:f.Callbacks("once memory").add(function(){f._removeData(t,e+"queue"),f._removeData(t,n)})})}}),f.fn.extend({queue:function(t,e){var n=2;return"string"!=typeof t&&(e=t,t="fx",n--),arguments.length<n?f.queue(this[0],t):void 0===e?this:this.each(function(){var n=f.queue(this,t,e);f._queueHooks(this,t),"fx"===t&&"inprogress"!==n[0]&&f.dequeue(this,t)})},dequeue:function(t){return this.each(function(){f.dequeue(this,t)})},clearQueue:function(t){return this.queue(t||"fx",[])},promise:function(t,e){var n,r=1,i=f.Deferred(),o=this,a=this.length,s=function(){--r||i.resolveWith(o,[o])};for("string"!=typeof t&&(e=t,t=void 0),t=t||"fx";a--;)(n=f._data(o[a],t+"queueHooks"))&&n.empty&&(r++,n.empty.add(s));return s(),i.promise(e)}});var K=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,z=["Top","Right","Bottom","Left"],J=function(t,e){return t=e||t,"none"===f.css(t,"display")||!f.contains(t.ownerDocument,t)},q=f.access=function(t,e,n,r,i,o,a){var s=0,A=t.length,l=null==n;if("object"===f.type(n))for(s in i=!0,n)f.access(t,e,s,n[s],!0,o,a);else if(void 0!==r&&(i=!0,f.isFunction(r)||(a=!0),l&&(a?(e.call(t,r),e=null):(l=e,e=function(t,e,n){return l.call(f(t),n)})),e))for(;s<A;s++)e(t[s],n,a?r:r.call(t[s],s,e(t[s],n)));return i?t:l?e.call(t):A?e(t[0],n):o},V=/^(?:checkbox|radio)$/i;!function(){var t=x.createDocumentFragment(),e=x.createElement("div"),n=x.createElement("input");if(e.setAttribute("className","t"),e.innerHTML="  <link/><table></table><a href='/a'>a</a>",d.leadingWhitespace=3===e.firstChild.nodeType,d.tbody=!e.getElementsByTagName("tbody").length,d.htmlSerialize=!!e.getElementsByTagName("link").length,d.html5Clone="<:nav></:nav>"!==x.createElement("nav").cloneNode(!0).outerHTML,n.type="checkbox",n.checked=!0,t.appendChild(n),d.appendChecked=n.checked,e.innerHTML="<textarea>x</textarea>",d.noCloneChecked=!!e.cloneNode(!0).lastChild.defaultValue,t.appendChild(e),e.innerHTML="<input type='radio' checked='checked' name='t'/>",d.checkClone=e.cloneNode(!0).cloneNode(!0).lastChild.checked,d.noCloneEvent=!0,e.attachEvent&&(e.attachEvent("onclick",function(){d.noCloneEvent=!1}),e.cloneNode(!0).click()),null==d.deleteExpando){d.deleteExpando=!0;try{delete e.test}catch(t){d.deleteExpando=!1}}t=e=n=null}(),function(){var t,e,r=x.createElement("div");for(t in{submit:!0,change:!0,focusin:!0})e="on"+t,(d[t+"Bubbles"]=e in n)||(r.setAttribute(e,"t"),d[t+"Bubbles"]=!1===r.attributes[e].expando);r=null}();var W=/^(?:input|select|textarea)$/i,X=/^key/,Z=/^(?:mouse|contextmenu)|click/,$=/^(?:focusinfocus|focusoutblur)$/,tt=/^([^.]*)(?:\.(.+)|)$/;function et(){return!0}function nt(){return!1}function rt(){try{return x.activeElement}catch(t){}}function it(t){var e=ot.split("|"),n=t.createDocumentFragment();if(n.createElement)for(;e.length;)n.createElement(e.pop());return n}f.event={global:{},add:function(t,e,n,r,i){var o,a,s,A,l,c,u,g,h,d,p,I=f._data(t);if(I){for(n.handler&&(n=(A=n).handler,i=A.selector),n.guid||(n.guid=f.guid++),(a=I.events)||(a=I.events={}),(c=I.handle)||((c=I.handle=function(t){return typeof f===U||t&&f.event.triggered===t.type?void 0:f.event.dispatch.apply(c.elem,arguments)}).elem=t),s=(e=(e||"").match(L)||[""]).length;s--;)h=p=(o=tt.exec(e[s])||[])[1],d=(o[2]||"").split(".").sort(),h&&(l=f.event.special[h]||{},h=(i?l.delegateType:l.bindType)||h,l=f.event.special[h]||{},u=f.extend({type:h,origType:p,data:r,handler:n,guid:n.guid,selector:i,needsContext:i&&f.expr.match.needsContext.test(i),namespace:d.join(".")},A),(g=a[h])||((g=a[h]=[]).delegateCount=0,l.setup&&!1!==l.setup.call(t,r,d,c)||(t.addEventListener?t.addEventListener(h,c,!1):t.attachEvent&&t.attachEvent("on"+h,c))),l.add&&(l.add.call(t,u),u.handler.guid||(u.handler.guid=n.guid)),i?g.splice(g.delegateCount++,0,u):g.push(u),f.event.global[h]=!0);t=null}},remove:function(t,e,n,r,i){var o,a,s,A,l,c,u,g,h,d,p,I=f.hasData(t)&&f._data(t);if(I&&(c=I.events)){for(l=(e=(e||"").match(L)||[""]).length;l--;)if(h=p=(s=tt.exec(e[l])||[])[1],d=(s[2]||"").split(".").sort(),h){for(u=f.event.special[h]||{},g=c[h=(r?u.delegateType:u.bindType)||h]||[],s=s[2]&&new RegExp("(^|\\.)"+d.join("\\.(?:.*\\.|)")+"(\\.|$)"),A=o=g.length;o--;)a=g[o],!i&&p!==a.origType||n&&n.guid!==a.guid||s&&!s.test(a.namespace)||r&&r!==a.selector&&("**"!==r||!a.selector)||(g.splice(o,1),a.selector&&g.delegateCount--,u.remove&&u.remove.call(t,a));A&&!g.length&&(u.teardown&&!1!==u.teardown.call(t,d,I.handle)||f.removeEvent(t,h,I.handle),delete c[h])}else for(h in c)f.event.remove(t,h+e[l],n,r,!0);f.isEmptyObject(c)&&(delete I.handle,f._removeData(t,"events"))}},trigger:function(t,e,r,i){var o,a,s,A,l,c,u,h=[r||x],d=g.call(t,"type")?t.type:t,p=g.call(t,"namespace")?t.namespace.split("."):[];if(s=c=r=r||x,3!==r.nodeType&&8!==r.nodeType&&!$.test(d+f.event.triggered)&&(d.indexOf(".")>=0&&(p=d.split("."),d=p.shift(),p.sort()),a=d.indexOf(":")<0&&"on"+d,(t=t[f.expando]?t:new f.Event(d,"object"==typeof t&&t)).isTrigger=i?2:3,t.namespace=p.join("."),t.namespace_re=t.namespace?new RegExp("(^|\\.)"+p.join("\\.(?:.*\\.|)")+"(\\.|$)"):null,t.result=void 0,t.target||(t.target=r),e=null==e?[t]:f.makeArray(e,[t]),l=f.event.special[d]||{},i||!l.trigger||!1!==l.trigger.apply(r,e))){if(!i&&!l.noBubble&&!f.isWindow(r)){for(A=l.delegateType||d,$.test(A+d)||(s=s.parentNode);s;s=s.parentNode)h.push(s),c=s;c===(r.ownerDocument||x)&&h.push(c.defaultView||c.parentWindow||n)}for(u=0;(s=h[u++])&&!t.isPropagationStopped();)t.type=u>1?A:l.bindType||d,(o=(f._data(s,"events")||{})[t.type]&&f._data(s,"handle"))&&o.apply(s,e),(o=a&&s[a])&&o.apply&&f.acceptData(s)&&(t.result=o.apply(s,e),!1===t.result&&t.preventDefault());if(t.type=d,!i&&!t.isDefaultPrevented()&&(!l._default||!1===l._default.apply(h.pop(),e))&&f.acceptData(r)&&a&&r[d]&&!f.isWindow(r)){(c=r[a])&&(r[a]=null),f.event.triggered=d;try{r[d]()}catch(t){}f.event.triggered=void 0,c&&(r[a]=c)}return t.result}},dispatch:function(t){t=f.event.fix(t);var e,n,r,i,o,s=[],A=a.call(arguments),l=(f._data(this,"events")||{})[t.type]||[],c=f.event.special[t.type]||{};if(A[0]=t,t.delegateTarget=this,!c.preDispatch||!1!==c.preDispatch.call(this,t)){for(s=f.event.handlers.call(this,t,l),e=0;(i=s[e++])&&!t.isPropagationStopped();)for(t.currentTarget=i.elem,o=0;(r=i.handlers[o++])&&!t.isImmediatePropagationStopped();)t.namespace_re&&!t.namespace_re.test(r.namespace)||(t.handleObj=r,t.data=r.data,void 0!==(n=((f.event.special[r.origType]||{}).handle||r.handler).apply(i.elem,A))&&!1===(t.result=n)&&(t.preventDefault(),t.stopPropagation()));return c.postDispatch&&c.postDispatch.call(this,t),t.result}},handlers:function(t,e){var n,r,i,o,a=[],s=e.delegateCount,A=t.target;if(s&&A.nodeType&&(!t.button||"click"!==t.type))for(;A!=this;A=A.parentNode||this)if(1===A.nodeType&&(!0!==A.disabled||"click"!==t.type)){for(i=[],o=0;o<s;o++)void 0===i[n=(r=e[o]).selector+" "]&&(i[n]=r.needsContext?f(n,this).index(A)>=0:f.find(n,this,null,[A]).length),i[n]&&i.push(r);i.length&&a.push({elem:A,handlers:i})}return s<e.length&&a.push({elem:this,handlers:e.slice(s)}),a},fix:function(t){if(t[f.expando])return t;var e,n,r,i=t.type,o=t,a=this.fixHooks[i];for(a||(this.fixHooks[i]=a=Z.test(i)?this.mouseHooks:X.test(i)?this.keyHooks:{}),r=a.props?this.props.concat(a.props):this.props,t=new f.Event(o),e=r.length;e--;)t[n=r[e]]=o[n];return t.target||(t.target=o.srcElement||x),3===t.target.nodeType&&(t.target=t.target.parentNode),t.metaKey=!!t.metaKey,a.filter?a.filter(t,o):t},props:"altKey bubbles cancelable ctrlKey currentTarget eventPhase metaKey relatedTarget shiftKey target timeStamp view which".split(" "),fixHooks:{},keyHooks:{props:"char charCode key keyCode".split(" "),filter:function(t,e){return null==t.which&&(t.which=null!=e.charCode?e.charCode:e.keyCode),t}},mouseHooks:{props:"button buttons clientX clientY fromElement offsetX offsetY pageX pageY screenX screenY toElement".split(" "),filter:function(t,e){var n,r,i,o=e.button,a=e.fromElement;return null==t.pageX&&null!=e.clientX&&(i=(r=t.target.ownerDocument||x).documentElement,n=r.body,t.pageX=e.clientX+(i&&i.scrollLeft||n&&n.scrollLeft||0)-(i&&i.clientLeft||n&&n.clientLeft||0),t.pageY=e.clientY+(i&&i.scrollTop||n&&n.scrollTop||0)-(i&&i.clientTop||n&&n.clientTop||0)),!t.relatedTarget&&a&&(t.relatedTarget=a===t.target?e.toElement:a),t.which||void 0===o||(t.which=1&o?1:2&o?3:4&o?2:0),t}},special:{load:{noBubble:!0},focus:{trigger:function(){if(this!==rt()&&this.focus)try{return this.focus(),!1}catch(t){}},delegateType:"focusin"},blur:{trigger:function(){if(this===rt()&&this.blur)return this.blur(),!1},delegateType:"focusout"},click:{trigger:function(){if(f.nodeName(this,"input")&&"checkbox"===this.type&&this.click)return this.click(),!1},_default:function(t){return f.nodeName(t.target,"a")}},beforeunload:{postDispatch:function(t){void 0!==t.result&&(t.originalEvent.returnValue=t.result)}}},simulate:function(t,e,n,r){var i=f.extend(new f.Event,n,{type:t,isSimulated:!0,originalEvent:{}});r?f.event.trigger(i,null,e):f.event.dispatch.call(e,i),i.isDefaultPrevented()&&n.preventDefault()}},f.removeEvent=x.removeEventListener?function(t,e,n){t.removeEventListener&&t.removeEventListener(e,n,!1)}:function(t,e,n){var r="on"+e;t.detachEvent&&(typeof t[r]===U&&(t[r]=null),t.detachEvent(r,n))},f.Event=function(t,e){if(!(this instanceof f.Event))return new f.Event(t,e);t&&t.type?(this.originalEvent=t,this.type=t.type,this.isDefaultPrevented=t.defaultPrevented||void 0===t.defaultPrevented&&(!1===t.returnValue||t.getPreventDefault&&t.getPreventDefault())?et:nt):this.type=t,e&&f.extend(this,e),this.timeStamp=t&&t.timeStamp||f.now(),this[f.expando]=!0},f.Event.prototype={isDefaultPrevented:nt,isPropagationStopped:nt,isImmediatePropagationStopped:nt,preventDefault:function(){var t=this.originalEvent;this.isDefaultPrevented=et,t&&(t.preventDefault?t.preventDefault():t.returnValue=!1)},stopPropagation:function(){var t=this.originalEvent;this.isPropagationStopped=et,t&&(t.stopPropagation&&t.stopPropagation(),t.cancelBubble=!0)},stopImmediatePropagation:function(){this.isImmediatePropagationStopped=et,this.stopPropagation()}},f.each({mouseenter:"mouseover",mouseleave:"mouseout"},function(t,e){f.event.special[t]={delegateType:e,bindType:e,handle:function(t){var n,r=this,i=t.relatedTarget,o=t.handleObj;return i&&(i===r||f.contains(r,i))||(t.type=o.origType,n=o.handler.apply(this,arguments),t.type=e),n}}}),d.submitBubbles||(f.event.special.submit={setup:function(){if(f.nodeName(this,"form"))return!1;f.event.add(this,"click._submit keypress._submit",function(t){var e=t.target,n=f.nodeName(e,"input")||f.nodeName(e,"button")?e.form:void 0;n&&!f._data(n,"submitBubbles")&&(f.event.add(n,"submit._submit",function(t){t._submit_bubble=!0}),f._data(n,"submitBubbles",!0))})},postDispatch:function(t){t._submit_bubble&&(delete t._submit_bubble,this.parentNode&&!t.isTrigger&&f.event.simulate("submit",this.parentNode,t,!0))},teardown:function(){if(f.nodeName(this,"form"))return!1;f.event.remove(this,"._submit")}}),d.changeBubbles||(f.event.special.change={setup:function(){if(W.test(this.nodeName))return"checkbox"!==this.type&&"radio"!==this.type||(f.event.add(this,"propertychange._change",function(t){"checked"===t.originalEvent.propertyName&&(this._just_changed=!0)}),f.event.add(this,"click._change",function(t){this._just_changed&&!t.isTrigger&&(this._just_changed=!1),f.event.simulate("change",this,t,!0)})),!1;f.event.add(this,"beforeactivate._change",function(t){var e=t.target;W.test(e.nodeName)&&!f._data(e,"changeBubbles")&&(f.event.add(e,"change._change",function(t){!this.parentNode||t.isSimulated||t.isTrigger||f.event.simulate("change",this.parentNode,t,!0)}),f._data(e,"changeBubbles",!0))})},handle:function(t){var e=t.target;if(this!==e||t.isSimulated||t.isTrigger||"radio"!==e.type&&"checkbox"!==e.type)return t.handleObj.handler.apply(this,arguments)},teardown:function(){return f.event.remove(this,"._change"),!W.test(this.nodeName)}}),d.focusinBubbles||f.each({focus:"focusin",blur:"focusout"},function(t,e){var n=function(t){f.event.simulate(e,t.target,f.event.fix(t),!0)};f.event.special[e]={setup:function(){var r=this.ownerDocument||this,i=f._data(r,e);i||r.addEventListener(t,n,!0),f._data(r,e,(i||0)+1)},teardown:function(){var r=this.ownerDocument||this,i=f._data(r,e)-1;i?f._data(r,e,i):(r.removeEventListener(t,n,!0),f._removeData(r,e))}}}),f.fn.extend({on:function(t,e,n,r,i){var o,a;if("object"==typeof t){for(o in"string"!=typeof e&&(n=n||e,e=void 0),t)this.on(o,e,n,t[o],i);return this}if(null==n&&null==r?(r=e,n=e=void 0):null==r&&("string"==typeof e?(r=n,n=void 0):(r=n,n=e,e=void 0)),!1===r)r=nt;else if(!r)return this;return 1===i&&(a=r,(r=function(t){return f().off(t),a.apply(this,arguments)}).guid=a.guid||(a.guid=f.guid++)),this.each(function(){f.event.add(this,t,r,n,e)})},one:function(t,e,n,r){return this.on(t,e,n,r,1)},off:function(t,e,n){var r,i;if(t&&t.preventDefault&&t.handleObj)return r=t.handleObj,f(t.delegateTarget).off(r.namespace?r.origType+"."+r.namespace:r.origType,r.selector,r.handler),this;if("object"==typeof t){for(i in t)this.off(i,e,t[i]);return this}return!1!==e&&"function"!=typeof e||(n=e,e=void 0),!1===n&&(n=nt),this.each(function(){f.event.remove(this,t,n,e)})},trigger:function(t,e){return this.each(function(){f.event.trigger(t,e,this)})},triggerHandler:function(t,e){var n=this[0];if(n)return f.event.trigger(t,e,n,!0)}});var ot="abbr|article|aside|audio|bdi|canvas|data|datalist|details|figcaption|figure|footer|header|hgroup|mark|meter|nav|output|progress|section|summary|time|video",at=/ jQuery\d+="(?:null|\d+)"/g,st=new RegExp("<(?:"+ot+")[\\s/>]","i"),At=/^\s+/,lt=/<(?!area|br|col|embed|hr|img|input|link|meta|param)(([\w:]+)[^>]*)\/>/gi,ct=/<([\w:]+)/,ut=/<tbody/i,gt=/<|&#?\w+;/,ht=/<(?:script|style|link)/i,dt=/checked\s*(?:[^=]|=\s*.checked.)/i,ft=/^$|\/(?:java|ecma)script/i,pt=/^true\/(.*)/,It=/^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g,Ct={option:[1,"<select multiple='multiple'>","</select>"],legend:[1,"<fieldset>","</fieldset>"],area:[1,"<map>","</map>"],param:[1,"<object>","</object>"],thead:[1,"<table>","</table>"],tr:[2,"<table><tbody>","</tbody></table>"],col:[2,"<table><tbody></tbody><colgroup>","</colgroup></table>"],td:[3,"<table><tbody><tr>","</tr></tbody></table>"],_default:d.htmlSerialize?[0,"",""]:[1,"X<div>","</div>"]},mt=it(x).appendChild(x.createElement("div"));function yt(t,e){var n,r,i=0,o=typeof t.getElementsByTagName!==U?t.getElementsByTagName(e||"*"):typeof t.querySelectorAll!==U?t.querySelectorAll(e||"*"):void 0;if(!o)for(o=[],n=t.childNodes||t;null!=(r=n[i]);i++)!e||f.nodeName(r,e)?o.push(r):f.merge(o,yt(r,e));return void 0===e||e&&f.nodeName(t,e)?f.merge([t],o):o}function Et(t){V.test(t.type)&&(t.defaultChecked=t.checked)}function vt(t,e){return f.nodeName(t,"table")&&f.nodeName(11!==e.nodeType?e:e.firstChild,"tr")?t.getElementsByTagName("tbody")[0]||t.appendChild(t.ownerDocument.createElement("tbody")):t}function Bt(t){return t.type=(null!==f.find.attr(t,"type"))+"/"+t.type,t}function bt(t){var e=pt.exec(t.type);return e?t.type=e[1]:t.removeAttribute("type"),t}function Qt(t,e){for(var n,r=0;null!=(n=t[r]);r++)f._data(n,"globalEval",!e||f._data(e[r],"globalEval"))}function wt(t,e){if(1===e.nodeType&&f.hasData(t)){var n,r,i,o=f._data(t),a=f._data(e,o),s=o.events;if(s)for(n in delete a.handle,a.events={},s)for(r=0,i=s[n].length;r<i;r++)f.event.add(e,n,s[n][r]);a.data&&(a.data=f.extend({},a.data))}}function xt(t,e){var n,r,i;if(1===e.nodeType){if(n=e.nodeName.toLowerCase(),!d.noCloneEvent&&e[f.expando]){for(r in(i=f._data(e)).events)f.removeEvent(e,r,i.handle);e.removeAttribute(f.expando)}"script"===n&&e.text!==t.text?(Bt(e).text=t.text,bt(e)):"object"===n?(e.parentNode&&(e.outerHTML=t.outerHTML),d.html5Clone&&t.innerHTML&&!f.trim(e.innerHTML)&&(e.innerHTML=t.innerHTML)):"input"===n&&V.test(t.type)?(e.defaultChecked=e.checked=t.checked,e.value!==t.value&&(e.value=t.value)):"option"===n?e.defaultSelected=e.selected=t.defaultSelected:"input"!==n&&"textarea"!==n||(e.defaultValue=t.defaultValue)}}Ct.optgroup=Ct.option,Ct.tbody=Ct.tfoot=Ct.colgroup=Ct.caption=Ct.thead,Ct.th=Ct.td,f.extend({clone:function(t,e,n){var r,i,o,a,s,A=f.contains(t.ownerDocument,t);if(d.html5Clone||f.isXMLDoc(t)||!st.test("<"+t.nodeName+">")?o=t.cloneNode(!0):(mt.innerHTML=t.outerHTML,mt.removeChild(o=mt.firstChild)),!(d.noCloneEvent&&d.noCloneChecked||1!==t.nodeType&&11!==t.nodeType||f.isXMLDoc(t)))for(r=yt(o),s=yt(t),a=0;null!=(i=s[a]);++a)r[a]&&xt(i,r[a]);if(e)if(n)for(s=s||yt(t),r=r||yt(o),a=0;null!=(i=s[a]);a++)wt(i,r[a]);else wt(t,o);return(r=yt(o,"script")).length>0&&Qt(r,!A&&yt(t,"script")),r=s=i=null,o},buildFragment:function(t,e,n,r){for(var i,o,a,s,A,l,c,u=t.length,g=it(e),h=[],p=0;p<u;p++)if((o=t[p])||0===o)if("object"===f.type(o))f.merge(h,o.nodeType?[o]:o);else if(gt.test(o)){for(s=s||g.appendChild(e.createElement("div")),A=(ct.exec(o)||["",""])[1].toLowerCase(),c=Ct[A]||Ct._default,s.innerHTML=c[1]+o.replace(lt,"<$1></$2>")+c[2],i=c[0];i--;)s=s.lastChild;if(!d.leadingWhitespace&&At.test(o)&&h.push(e.createTextNode(At.exec(o)[0])),!d.tbody)for(i=(o="table"!==A||ut.test(o)?"<table>"!==c[1]||ut.test(o)?0:s:s.firstChild)&&o.childNodes.length;i--;)f.nodeName(l=o.childNodes[i],"tbody")&&!l.childNodes.length&&o.removeChild(l);for(f.merge(h,s.childNodes),s.textContent="";s.firstChild;)s.removeChild(s.firstChild);s=g.lastChild}else h.push(e.createTextNode(o));for(s&&g.removeChild(s),d.appendChecked||f.grep(yt(h,"input"),Et),p=0;o=h[p++];)if((!r||-1===f.inArray(o,r))&&(a=f.contains(o.ownerDocument,o),s=yt(g.appendChild(o),"script"),a&&Qt(s),n))for(i=0;o=s[i++];)ft.test(o.type||"")&&n.push(o);return s=null,g},cleanData:function(t,e){for(var n,r,i,a,s=0,A=f.expando,l=f.cache,c=d.deleteExpando,u=f.event.special;null!=(n=t[s]);s++)if((e||f.acceptData(n))&&(a=(i=n[A])&&l[i])){if(a.events)for(r in a.events)u[r]?f.event.remove(n,r):f.removeEvent(n,r,a.handle);l[i]&&(delete l[i],c?delete n[A]:typeof n.removeAttribute!==U?n.removeAttribute(A):n[A]=null,o.push(i))}}}),f.fn.extend({text:function(t){return q(this,function(t){return void 0===t?f.text(this):this.empty().append((this[0]&&this[0].ownerDocument||x).createTextNode(t))},null,t,arguments.length)},append:function(){return this.domManip(arguments,function(t){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||vt(this,t).appendChild(t)})},prepend:function(){return this.domManip(arguments,function(t){if(1===this.nodeType||11===this.nodeType||9===this.nodeType){var e=vt(this,t);e.insertBefore(t,e.firstChild)}})},before:function(){return this.domManip(arguments,function(t){this.parentNode&&this.parentNode.insertBefore(t,this)})},after:function(){return this.domManip(arguments,function(t){this.parentNode&&this.parentNode.insertBefore(t,this.nextSibling)})},remove:function(t,e){for(var n,r=t?f.filter(t,this):this,i=0;null!=(n=r[i]);i++)e||1!==n.nodeType||f.cleanData(yt(n)),n.parentNode&&(e&&f.contains(n.ownerDocument,n)&&Qt(yt(n,"script")),n.parentNode.removeChild(n));return this},empty:function(){for(var t,e=0;null!=(t=this[e]);e++){for(1===t.nodeType&&f.cleanData(yt(t,!1));t.firstChild;)t.removeChild(t.firstChild);t.options&&f.nodeName(t,"select")&&(t.options.length=0)}return this},clone:function(t,e){return t=null!=t&&t,e=null==e?t:e,this.map(function(){return f.clone(this,t,e)})},html:function(t){return q(this,function(t){var e=this[0]||{},n=0,r=this.length;if(void 0===t)return 1===e.nodeType?e.innerHTML.replace(at,""):void 0;if("string"==typeof t&&!ht.test(t)&&(d.htmlSerialize||!st.test(t))&&(d.leadingWhitespace||!At.test(t))&&!Ct[(ct.exec(t)||["",""])[1].toLowerCase()]){t=t.replace(lt,"<$1></$2>");try{for(;n<r;n++)1===(e=this[n]||{}).nodeType&&(f.cleanData(yt(e,!1)),e.innerHTML=t);e=0}catch(t){}}e&&this.empty().append(t)},null,t,arguments.length)},replaceWith:function(){var t=arguments[0];return this.domManip(arguments,function(e){t=this.parentNode,f.cleanData(yt(this)),t&&t.replaceChild(e,this)}),t&&(t.length||t.nodeType)?this:this.remove()},detach:function(t){return this.remove(t,!0)},domManip:function(t,e){t=s.apply([],t);var n,r,i,o,a,A,l=0,c=this.length,u=this,g=c-1,h=t[0],p=f.isFunction(h);if(p||c>1&&"string"==typeof h&&!d.checkClone&&dt.test(h))return this.each(function(n){var r=u.eq(n);p&&(t[0]=h.call(this,n,r.html())),r.domManip(t,e)});if(c&&(n=(A=f.buildFragment(t,this[0].ownerDocument,!1,this)).firstChild,1===A.childNodes.length&&(A=n),n)){for(i=(o=f.map(yt(A,"script"),Bt)).length;l<c;l++)r=A,l!==g&&(r=f.clone(r,!0,!0),i&&f.merge(o,yt(r,"script"))),e.call(this[l],r,l);if(i)for(a=o[o.length-1].ownerDocument,f.map(o,bt),l=0;l<i;l++)r=o[l],ft.test(r.type||"")&&!f._data(r,"globalEval")&&f.contains(a,r)&&(r.src?f._evalUrl&&f._evalUrl(r.src):f.globalEval((r.text||r.textContent||r.innerHTML||"").replace(It,"")));A=n=null}return this}}),f.each({appendTo:"append",prependTo:"prepend",insertBefore:"before",insertAfter:"after",replaceAll:"replaceWith"},function(t,e){f.fn[t]=function(t){for(var n,r=0,i=[],o=f(t),a=o.length-1;r<=a;r++)n=r===a?this:this.clone(!0),f(o[r])[e](n),A.apply(i,n.get());return this.pushStack(i)}});var _t,Dt,Mt,kt,St={};function Lt(t,e){var r=f(e.createElement(t)).appendTo(e.body),i=n.getDefaultComputedStyle?n.getDefaultComputedStyle(r[0]).display:f.css(r[0],"display");return r.detach(),i}function Rt(t){var e=x,n=St[t];return n||("none"!==(n=Lt(t,e))&&n||((e=((_t=(_t||f("<iframe frameborder='0' width='0' height='0'/>")).appendTo(e.documentElement))[0].contentWindow||_t[0].contentDocument).document).write(),e.close(),n=Lt(t,e),_t.detach()),St[t]=n),n}(kt=x.createElement("div")).innerHTML="  <link/><table></table><a href='/a'>a</a><input type='checkbox'/>",(Dt=kt.getElementsByTagName("a")[0]).style.cssText="float:left;opacity:.5",d.opacity=/^0.5/.test(Dt.style.opacity),d.cssFloat=!!Dt.style.cssFloat,kt.style.backgroundClip="content-box",kt.cloneNode(!0).style.backgroundClip="",d.clearCloneStyle="content-box"===kt.style.backgroundClip,Dt=kt=null,d.shrinkWrapBlocks=function(){var t,e,n;if(null==Mt){if(!(t=x.getElementsByTagName("body")[0]))return;e=x.createElement("div"),n=x.createElement("div"),t.appendChild(e).appendChild(n),Mt=!1,typeof n.style.zoom!==U&&(n.style.cssText="-webkit-box-sizing:content-box;-moz-box-sizing:content-box;box-sizing:content-box;display:block;padding:0;margin:0;border:0;width:1px;padding:1px;zoom:1",n.innerHTML="<div></div>",n.firstChild.style.width="5px",Mt=3!==n.offsetWidth),t.removeChild(e),t=e=n=null}return Mt};var Nt,Ft,Tt=/^margin/,Ut=new RegExp("^("+K+")(?!px)[a-z%]+$","i"),Ot=/^(top|right|bottom|left)$/;function Gt(t,e){return{get:function(){var n=t();if(null!=n){if(!n)return(this.get=e).apply(this,arguments);delete this.get}}}}n.getComputedStyle?(Nt=function(t){return t.ownerDocument.defaultView.getComputedStyle(t,null)},Ft=function(t,e,n){var r,i,o,a,s=t.style;return a=(n=n||Nt(t))?n.getPropertyValue(e)||n[e]:void 0,n&&(""!==a||f.contains(t.ownerDocument,t)||(a=f.style(t,e)),Ut.test(a)&&Tt.test(e)&&(r=s.width,i=s.minWidth,o=s.maxWidth,s.minWidth=s.maxWidth=s.width=a,a=n.width,s.width=r,s.minWidth=i,s.maxWidth=o)),void 0===a?a:a+""}):x.documentElement.currentStyle&&(Nt=function(t){return t.currentStyle},Ft=function(t,e,n){var r,i,o,a,s=t.style;return null==(a=(n=n||Nt(t))?n[e]:void 0)&&s&&s[e]&&(a=s[e]),Ut.test(a)&&!Ot.test(e)&&(r=s.left,(o=(i=t.runtimeStyle)&&i.left)&&(i.left=t.currentStyle.left),s.left="fontSize"===e?"1em":a,a=s.pixelLeft+"px",s.left=r,o&&(i.left=o)),void 0===a?a:a+""||"auto"}),function(){var t,e,r,i,o,a,s=x.createElement("div"),A="border:0;width:0;height:0;position:absolute;top:0;left:-9999px";function l(){var t,e,s=x.getElementsByTagName("body")[0];s&&(t=x.createElement("div"),e=x.createElement("div"),t.style.cssText=A,s.appendChild(t).appendChild(e),e.style.cssText="-webkit-box-sizing:border-box;-moz-box-sizing:border-box;box-sizing:border-box;position:absolute;display:block;padding:1px;border:1px;width:4px;margin-top:1%;top:1%",f.swap(s,null!=s.style.zoom?{zoom:1}:{},function(){r=4===e.offsetWidth}),i=!0,o=!1,a=!0,n.getComputedStyle&&(o="1%"!==(n.getComputedStyle(e,null)||{}).top,i="4px"===(n.getComputedStyle(e,null)||{width:"4px"}).width),s.removeChild(t),e=s=null)}s.innerHTML="  <link/><table></table><a href='/a'>a</a><input type='checkbox'/>",(t=s.getElementsByTagName("a")[0]).style.cssText="float:left;opacity:.5",d.opacity=/^0.5/.test(t.style.opacity),d.cssFloat=!!t.style.cssFloat,s.style.backgroundClip="content-box",s.cloneNode(!0).style.backgroundClip="",d.clearCloneStyle="content-box"===s.style.backgroundClip,t=s=null,f.extend(d,{reliableHiddenOffsets:function(){if(null!=e)return e;var t,n,r,i=x.createElement("div"),o=x.getElementsByTagName("body")[0];return o?(i.setAttribute("className","t"),i.innerHTML="  <link/><table></table><a href='/a'>a</a><input type='checkbox'/>",(t=x.createElement("div")).style.cssText=A,o.appendChild(t).appendChild(i),i.innerHTML="<table><tr><td></td><td>t</td></tr></table>",(n=i.getElementsByTagName("td"))[0].style.cssText="padding:0;margin:0;border:0;display:none",r=0===n[0].offsetHeight,n[0].style.display="",n[1].style.display="none",e=r&&0===n[0].offsetHeight,o.removeChild(t),i=o=null,e):void 0},boxSizing:function(){return null==r&&l(),r},boxSizingReliable:function(){return null==i&&l(),i},pixelPosition:function(){return null==o&&l(),o},reliableMarginRight:function(){var t,e,r,i;if(null==a&&n.getComputedStyle){if(!(t=x.getElementsByTagName("body")[0]))return;e=x.createElement("div"),r=x.createElement("div"),e.style.cssText=A,t.appendChild(e).appendChild(r),(i=r.appendChild(x.createElement("div"))).style.cssText=r.style.cssText="-webkit-box-sizing:content-box;-moz-box-sizing:content-box;box-sizing:content-box;display:block;padding:0;margin:0;border:0",i.style.marginRight=i.style.width="0",r.style.width="1px",a=!parseFloat((n.getComputedStyle(i,null)||{}).marginRight),t.removeChild(e)}return a}})}(),f.swap=function(t,e,n,r){var i,o,a={};for(o in e)a[o]=t.style[o],t.style[o]=e[o];for(o in i=n.apply(t,r||[]),e)t.style[o]=a[o];return i};var Ht=/alpha\([^)]*\)/i,jt=/opacity\s*=\s*([^)]*)/,Yt=/^(none|table(?!-c[ea]).+)/,Pt=new RegExp("^("+K+")(.*)$","i"),Kt=new RegExp("^([+-])=("+K+")","i"),zt={position:"absolute",visibility:"hidden",display:"block"},Jt={letterSpacing:0,fontWeight:400},qt=["Webkit","O","Moz","ms"];function Vt(t,e){if(e in t)return e;for(var n=e.charAt(0).toUpperCase()+e.slice(1),r=e,i=qt.length;i--;)if((e=qt[i]+n)in t)return e;return r}function Wt(t,e){for(var n,r,i,o=[],a=0,s=t.length;a<s;a++)(r=t[a]).style&&(o[a]=f._data(r,"olddisplay"),n=r.style.display,e?(o[a]||"none"!==n||(r.style.display=""),""===r.style.display&&J(r)&&(o[a]=f._data(r,"olddisplay",Rt(r.nodeName)))):o[a]||(i=J(r),(n&&"none"!==n||!i)&&f._data(r,"olddisplay",i?n:f.css(r,"display"))));for(a=0;a<s;a++)(r=t[a]).style&&(e&&"none"!==r.style.display&&""!==r.style.display||(r.style.display=e?o[a]||"":"none"));return t}function Xt(t,e,n){var r=Pt.exec(e);return r?Math.max(0,r[1]-(n||0))+(r[2]||"px"):e}function Zt(t,e,n,r,i){for(var o=n===(r?"border":"content")?4:"width"===e?1:0,a=0;o<4;o+=2)"margin"===n&&(a+=f.css(t,n+z[o],!0,i)),r?("content"===n&&(a-=f.css(t,"padding"+z[o],!0,i)),"margin"!==n&&(a-=f.css(t,"border"+z[o]+"Width",!0,i))):(a+=f.css(t,"padding"+z[o],!0,i),"padding"!==n&&(a+=f.css(t,"border"+z[o]+"Width",!0,i)));return a}function $t(t,e,n){var r=!0,i="width"===e?t.offsetWidth:t.offsetHeight,o=Nt(t),a=d.boxSizing()&&"border-box"===f.css(t,"boxSizing",!1,o);if(i<=0||null==i){if(((i=Ft(t,e,o))<0||null==i)&&(i=t.style[e]),Ut.test(i))return i;r=a&&(d.boxSizingReliable()||i===t.style[e]),i=parseFloat(i)||0}return i+Zt(t,e,n||(a?"border":"content"),r,o)+"px"}function te(t,e,n,r,i){return new te.prototype.init(t,e,n,r,i)}f.extend({cssHooks:{opacity:{get:function(t,e){if(e){var n=Ft(t,"opacity");return""===n?"1":n}}}},cssNumber:{columnCount:!0,fillOpacity:!0,fontWeight:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,widows:!0,zIndex:!0,zoom:!0},cssProps:{float:d.cssFloat?"cssFloat":"styleFloat"},style:function(t,e,n,r){if(t&&3!==t.nodeType&&8!==t.nodeType&&t.style){var i,o,a,s=f.camelCase(e),A=t.style;if(e=f.cssProps[s]||(f.cssProps[s]=Vt(A,s)),a=f.cssHooks[e]||f.cssHooks[s],void 0===n)return a&&"get"in a&&void 0!==(i=a.get(t,!1,r))?i:A[e];if("string"==(o=typeof n)&&(i=Kt.exec(n))&&(n=(i[1]+1)*i[2]+parseFloat(f.css(t,e)),o="number"),null!=n&&n==n&&("number"!==o||f.cssNumber[s]||(n+="px"),d.clearCloneStyle||""!==n||0!==e.indexOf("background")||(A[e]="inherit"),!(a&&"set"in a&&void 0===(n=a.set(t,n,r)))))try{A[e]="",A[e]=n}catch(t){}}},css:function(t,e,n,r){var i,o,a,s=f.camelCase(e);return e=f.cssProps[s]||(f.cssProps[s]=Vt(t.style,s)),(a=f.cssHooks[e]||f.cssHooks[s])&&"get"in a&&(o=a.get(t,!0,n)),void 0===o&&(o=Ft(t,e,r)),"normal"===o&&e in Jt&&(o=Jt[e]),""===n||n?(i=parseFloat(o),!0===n||f.isNumeric(i)?i||0:o):o}}),f.each(["height","width"],function(t,e){f.cssHooks[e]={get:function(t,n,r){if(n)return 0===t.offsetWidth&&Yt.test(f.css(t,"display"))?f.swap(t,zt,function(){return $t(t,e,r)}):$t(t,e,r)},set:function(t,n,r){var i=r&&Nt(t);return Xt(0,n,r?Zt(t,e,r,d.boxSizing()&&"border-box"===f.css(t,"boxSizing",!1,i),i):0)}}}),d.opacity||(f.cssHooks.opacity={get:function(t,e){return jt.test((e&&t.currentStyle?t.currentStyle.filter:t.style.filter)||"")?.01*parseFloat(RegExp.$1)+"":e?"1":""},set:function(t,e){var n=t.style,r=t.currentStyle,i=f.isNumeric(e)?"alpha(opacity="+100*e+")":"",o=r&&r.filter||n.filter||"";n.zoom=1,(e>=1||""===e)&&""===f.trim(o.replace(Ht,""))&&n.removeAttribute&&(n.removeAttribute("filter"),""===e||r&&!r.filter)||(n.filter=Ht.test(o)?o.replace(Ht,i):o+" "+i)}}),f.cssHooks.marginRight=Gt(d.reliableMarginRight,function(t,e){if(e)return f.swap(t,{display:"inline-block"},Ft,[t,"marginRight"])}),f.each({margin:"",padding:"",border:"Width"},function(t,e){f.cssHooks[t+e]={expand:function(n){for(var r=0,i={},o="string"==typeof n?n.split(" "):[n];r<4;r++)i[t+z[r]+e]=o[r]||o[r-2]||o[0];return i}},Tt.test(t)||(f.cssHooks[t+e].set=Xt)}),f.fn.extend({css:function(t,e){return q(this,function(t,e,n){var r,i,o={},a=0;if(f.isArray(e)){for(r=Nt(t),i=e.length;a<i;a++)o[e[a]]=f.css(t,e[a],!1,r);return o}return void 0!==n?f.style(t,e,n):f.css(t,e)},t,e,arguments.length>1)},show:function(){return Wt(this,!0)},hide:function(){return Wt(this)},toggle:function(t){return"boolean"==typeof t?t?this.show():this.hide():this.each(function(){J(this)?f(this).show():f(this).hide()})}}),f.Tween=te,te.prototype={constructor:te,init:function(t,e,n,r,i,o){this.elem=t,this.prop=n,this.easing=i||"swing",this.options=e,this.start=this.now=this.cur(),this.end=r,this.unit=o||(f.cssNumber[n]?"":"px")},cur:function(){var t=te.propHooks[this.prop];return t&&t.get?t.get(this):te.propHooks._default.get(this)},run:function(t){var e,n=te.propHooks[this.prop];return this.options.duration?this.pos=e=f.easing[this.easing](t,this.options.duration*t,0,1,this.options.duration):this.pos=e=t,this.now=(this.end-this.start)*e+this.start,this.options.step&&this.options.step.call(this.elem,this.now,this),n&&n.set?n.set(this):te.propHooks._default.set(this),this}},te.prototype.init.prototype=te.prototype,te.propHooks={_default:{get:function(t){var e;return null==t.elem[t.prop]||t.elem.style&&null!=t.elem.style[t.prop]?(e=f.css(t.elem,t.prop,""))&&"auto"!==e?e:0:t.elem[t.prop]},set:function(t){f.fx.step[t.prop]?f.fx.step[t.prop](t):t.elem.style&&(null!=t.elem.style[f.cssProps[t.prop]]||f.cssHooks[t.prop])?f.style(t.elem,t.prop,t.now+t.unit):t.elem[t.prop]=t.now}}},te.propHooks.scrollTop=te.propHooks.scrollLeft={set:function(t){t.elem.nodeType&&t.elem.parentNode&&(t.elem[t.prop]=t.now)}},f.easing={linear:function(t){return t},swing:function(t){return.5-Math.cos(t*Math.PI)/2}},f.fx=te.prototype.init,f.fx.step={};var ee,ne,re=/^(?:toggle|show|hide)$/,ie=new RegExp("^(?:([+-])=|)("+K+")([a-z%]*)$","i"),oe=/queueHooks$/,ae=[function(t,e,n){var r,i,o,a,s,A,l,c,u=this,g={},h=t.style,p=t.nodeType&&J(t),I=f._data(t,"fxshow");for(r in n.queue||(null==(s=f._queueHooks(t,"fx")).unqueued&&(s.unqueued=0,A=s.empty.fire,s.empty.fire=function(){s.unqueued||A()}),s.unqueued++,u.always(function(){u.always(function(){s.unqueued--,f.queue(t,"fx").length||s.empty.fire()})})),1===t.nodeType&&("height"in e||"width"in e)&&(n.overflow=[h.overflow,h.overflowX,h.overflowY],l=f.css(t,"display"),c=Rt(t.nodeName),"none"===l&&(l=c),"inline"===l&&"none"===f.css(t,"float")&&(d.inlineBlockNeedsLayout&&"inline"!==c?h.zoom=1:h.display="inline-block")),n.overflow&&(h.overflow="hidden",d.shrinkWrapBlocks()||u.always(function(){h.overflow=n.overflow[0],h.overflowX=n.overflow[1],h.overflowY=n.overflow[2]})),e)if(i=e[r],re.exec(i)){if(delete e[r],o=o||"toggle"===i,i===(p?"hide":"show")){if("show"!==i||!I||void 0===I[r])continue;p=!0}g[r]=I&&I[r]||f.style(t,r)}if(!f.isEmptyObject(g))for(r in I?"hidden"in I&&(p=I.hidden):I=f._data(t,"fxshow",{}),o&&(I.hidden=!p),p?f(t).show():u.done(function(){f(t).hide()}),u.done(function(){var e;for(e in f._removeData(t,"fxshow"),g)f.style(t,e,g[e])}),g)a=ce(p?I[r]:0,r,u),r in I||(I[r]=a.start,p&&(a.end=a.start,a.start="width"===r||"height"===r?1:0))}],se={"*":[function(t,e){var n=this.createTween(t,e),r=n.cur(),i=ie.exec(e),o=i&&i[3]||(f.cssNumber[t]?"":"px"),a=(f.cssNumber[t]||"px"!==o&&+r)&&ie.exec(f.css(n.elem,t)),s=1,A=20;if(a&&a[3]!==o){o=o||a[3],i=i||[],a=+r||1;do{a/=s=s||".5",f.style(n.elem,t,a+o)}while(s!==(s=n.cur()/r)&&1!==s&&--A)}return i&&(a=n.start=+a||+r||0,n.unit=o,n.end=i[1]?a+(i[1]+1)*i[2]:+i[2]),n}]};function Ae(){return setTimeout(function(){ee=void 0}),ee=f.now()}function le(t,e){var n,r={height:t},i=0;for(e=e?1:0;i<4;i+=2-e)r["margin"+(n=z[i])]=r["padding"+n]=t;return e&&(r.opacity=r.width=t),r}function ce(t,e,n){for(var r,i=(se[e]||[]).concat(se["*"]),o=0,a=i.length;o<a;o++)if(r=i[o].call(n,e,t))return r}function ue(t,e,n){var r,i,o=0,a=ae.length,s=f.Deferred().always(function(){delete A.elem}),A=function(){if(i)return!1;for(var e=ee||Ae(),n=Math.max(0,l.startTime+l.duration-e),r=1-(n/l.duration||0),o=0,a=l.tweens.length;o<a;o++)l.tweens[o].run(r);return s.notifyWith(t,[l,r,n]),r<1&&a?n:(s.resolveWith(t,[l]),!1)},l=s.promise({elem:t,props:f.extend({},e),opts:f.extend(!0,{specialEasing:{}},n),originalProperties:e,originalOptions:n,startTime:ee||Ae(),duration:n.duration,tweens:[],createTween:function(e,n){var r=f.Tween(t,l.opts,e,n,l.opts.specialEasing[e]||l.opts.easing);return l.tweens.push(r),r},stop:function(e){var n=0,r=e?l.tweens.length:0;if(i)return this;for(i=!0;n<r;n++)l.tweens[n].run(1);return e?s.resolveWith(t,[l,e]):s.rejectWith(t,[l,e]),this}}),c=l.props;for(function(t,e){var n,r,i,o,a;for(n in t)if(i=e[r=f.camelCase(n)],o=t[n],f.isArray(o)&&(i=o[1],o=t[n]=o[0]),n!==r&&(t[r]=o,delete t[n]),(a=f.cssHooks[r])&&"expand"in a)for(n in o=a.expand(o),delete t[r],o)n in t||(t[n]=o[n],e[n]=i);else e[r]=i}(c,l.opts.specialEasing);o<a;o++)if(r=ae[o].call(l,t,c,l.opts))return r;return f.map(c,ce,l),f.isFunction(l.opts.start)&&l.opts.start.call(t,l),f.fx.timer(f.extend(A,{elem:t,anim:l,queue:l.opts.queue})),l.progress(l.opts.progress).done(l.opts.done,l.opts.complete).fail(l.opts.fail).always(l.opts.always)}f.Animation=f.extend(ue,{tweener:function(t,e){f.isFunction(t)?(e=t,t=["*"]):t=t.split(" ");for(var n,r=0,i=t.length;r<i;r++)n=t[r],se[n]=se[n]||[],se[n].unshift(e)},prefilter:function(t,e){e?ae.unshift(t):ae.push(t)}}),f.speed=function(t,e,n){var r=t&&"object"==typeof t?f.extend({},t):{complete:n||!n&&e||f.isFunction(t)&&t,duration:t,easing:n&&e||e&&!f.isFunction(e)&&e};return r.duration=f.fx.off?0:"number"==typeof r.duration?r.duration:r.duration in f.fx.speeds?f.fx.speeds[r.duration]:f.fx.speeds._default,null!=r.queue&&!0!==r.queue||(r.queue="fx"),r.old=r.complete,r.complete=function(){f.isFunction(r.old)&&r.old.call(this),r.queue&&f.dequeue(this,r.queue)},r},f.fn.extend({fadeTo:function(t,e,n,r){return this.filter(J).css("opacity",0).show().end().animate({opacity:e},t,n,r)},animate:function(t,e,n,r){var i=f.isEmptyObject(t),o=f.speed(e,n,r),a=function(){var e=ue(this,f.extend({},t),o);(i||f._data(this,"finish"))&&e.stop(!0)};return a.finish=a,i||!1===o.queue?this.each(a):this.queue(o.queue,a)},stop:function(t,e,n){var r=function(t){var e=t.stop;delete t.stop,e(n)};return"string"!=typeof t&&(n=e,e=t,t=void 0),e&&!1!==t&&this.queue(t||"fx",[]),this.each(function(){var e=!0,i=null!=t&&t+"queueHooks",o=f.timers,a=f._data(this);if(i)a[i]&&a[i].stop&&r(a[i]);else for(i in a)a[i]&&a[i].stop&&oe.test(i)&&r(a[i]);for(i=o.length;i--;)o[i].elem!==this||null!=t&&o[i].queue!==t||(o[i].anim.stop(n),e=!1,o.splice(i,1));!e&&n||f.dequeue(this,t)})},finish:function(t){return!1!==t&&(t=t||"fx"),this.each(function(){var e,n=f._data(this),r=n[t+"queue"],i=n[t+"queueHooks"],o=f.timers,a=r?r.length:0;for(n.finish=!0,f.queue(this,t,[]),i&&i.stop&&i.stop.call(this,!0),e=o.length;e--;)o[e].elem===this&&o[e].queue===t&&(o[e].anim.stop(!0),o.splice(e,1));for(e=0;e<a;e++)r[e]&&r[e].finish&&r[e].finish.call(this);delete n.finish})}}),f.each(["toggle","show","hide"],function(t,e){var n=f.fn[e];f.fn[e]=function(t,r,i){return null==t||"boolean"==typeof t?n.apply(this,arguments):this.animate(le(e,!0),t,r,i)}}),f.each({slideDown:le("show"),slideUp:le("hide"),slideToggle:le("toggle"),fadeIn:{opacity:"show"},fadeOut:{opacity:"hide"},fadeToggle:{opacity:"toggle"}},function(t,e){f.fn[t]=function(t,n,r){return this.animate(e,t,n,r)}}),f.timers=[],f.fx.tick=function(){var t,e=f.timers,n=0;for(ee=f.now();n<e.length;n++)(t=e[n])()||e[n]!==t||e.splice(n--,1);e.length||f.fx.stop(),ee=void 0},f.fx.timer=function(t){f.timers.push(t),t()?f.fx.start():f.timers.pop()},f.fx.interval=13,f.fx.start=function(){ne||(ne=setInterval(f.fx.tick,f.fx.interval))},f.fx.stop=function(){clearInterval(ne),ne=null},f.fx.speeds={slow:600,fast:200,_default:400},f.fn.delay=function(t,e){return t=f.fx&&f.fx.speeds[t]||t,e=e||"fx",this.queue(e,function(e,n){var r=setTimeout(e,t);n.stop=function(){clearTimeout(r)}})},function(){var t,e,n,r,i=x.createElement("div");i.setAttribute("className","t"),i.innerHTML="  <link/><table></table><a href='/a'>a</a><input type='checkbox'/>",t=i.getElementsByTagName("a")[0],r=(n=x.createElement("select")).appendChild(x.createElement("option")),e=i.getElementsByTagName("input")[0],t.style.cssText="top:1px",d.getSetAttribute="t"!==i.className,d.style=/top/.test(t.getAttribute("style")),d.hrefNormalized="/a"===t.getAttribute("href"),d.checkOn=!!e.value,d.optSelected=r.selected,d.enctype=!!x.createElement("form").enctype,n.disabled=!0,d.optDisabled=!r.disabled,(e=x.createElement("input")).setAttribute("value",""),d.input=""===e.getAttribute("value"),e.value="t",e.setAttribute("type","radio"),d.radioValue="t"===e.value,t=e=n=r=i=null}();var ge=/\r/g;f.fn.extend({val:function(t){var e,n,r,i=this[0];return arguments.length?(r=f.isFunction(t),this.each(function(n){var i;1===this.nodeType&&(null==(i=r?t.call(this,n,f(this).val()):t)?i="":"number"==typeof i?i+="":f.isArray(i)&&(i=f.map(i,function(t){return null==t?"":t+""})),(e=f.valHooks[this.type]||f.valHooks[this.nodeName.toLowerCase()])&&"set"in e&&void 0!==e.set(this,i,"value")||(this.value=i))})):i?(e=f.valHooks[i.type]||f.valHooks[i.nodeName.toLowerCase()])&&"get"in e&&void 0!==(n=e.get(i,"value"))?n:"string"==typeof(n=i.value)?n.replace(ge,""):null==n?"":n:void 0}}),f.extend({valHooks:{option:{get:function(t){var e=f.find.attr(t,"value");return null!=e?e:f.text(t)}},select:{get:function(t){for(var e,n,r=t.options,i=t.selectedIndex,o="select-one"===t.type||i<0,a=o?null:[],s=o?i+1:r.length,A=i<0?s:o?i:0;A<s;A++)if(((n=r[A]).selected||A===i)&&(d.optDisabled?!n.disabled:null===n.getAttribute("disabled"))&&(!n.parentNode.disabled||!f.nodeName(n.parentNode,"optgroup"))){if(e=f(n).val(),o)return e;a.push(e)}return a},set:function(t,e){for(var n,r,i=t.options,o=f.makeArray(e),a=i.length;a--;)if(r=i[a],f.inArray(f.valHooks.option.get(r),o)>=0)try{r.selected=n=!0}catch(t){r.scrollHeight}else r.selected=!1;return n||(t.selectedIndex=-1),i}}}}),f.each(["radio","checkbox"],function(){f.valHooks[this]={set:function(t,e){if(f.isArray(e))return t.checked=f.inArray(f(t).val(),e)>=0}},d.checkOn||(f.valHooks[this].get=function(t){return null===t.getAttribute("value")?"on":t.value})});var he,de,fe=f.expr.attrHandle,pe=/^(?:checked|selected)$/i,Ie=d.getSetAttribute,Ce=d.input;f.fn.extend({attr:function(t,e){return q(this,f.attr,t,e,arguments.length>1)},removeAttr:function(t){return this.each(function(){f.removeAttr(this,t)})}}),f.extend({attr:function(t,e,n){var r,i,o=t.nodeType;if(t&&3!==o&&8!==o&&2!==o)return typeof t.getAttribute===U?f.prop(t,e,n):(1===o&&f.isXMLDoc(t)||(e=e.toLowerCase(),r=f.attrHooks[e]||(f.expr.match.bool.test(e)?de:he)),void 0===n?r&&"get"in r&&null!==(i=r.get(t,e))?i:null==(i=f.find.attr(t,e))?void 0:i:null!==n?r&&"set"in r&&void 0!==(i=r.set(t,n,e))?i:(t.setAttribute(e,n+""),n):void f.removeAttr(t,e))},removeAttr:function(t,e){var n,r,i=0,o=e&&e.match(L);if(o&&1===t.nodeType)for(;n=o[i++];)r=f.propFix[n]||n,f.expr.match.bool.test(n)?Ce&&Ie||!pe.test(n)?t[r]=!1:t[f.camelCase("default-"+n)]=t[r]=!1:f.attr(t,n,""),t.removeAttribute(Ie?n:r)},attrHooks:{type:{set:function(t,e){if(!d.radioValue&&"radio"===e&&f.nodeName(t,"input")){var n=t.value;return t.setAttribute("type",e),n&&(t.value=n),e}}}}}),de={set:function(t,e,n){return!1===e?f.removeAttr(t,n):Ce&&Ie||!pe.test(n)?t.setAttribute(!Ie&&f.propFix[n]||n,n):t[f.camelCase("default-"+n)]=t[n]=!0,n}},f.each(f.expr.match.bool.source.match(/\w+/g),function(t,e){var n=fe[e]||f.find.attr;fe[e]=Ce&&Ie||!pe.test(e)?function(t,e,r){var i,o;return r||(o=fe[e],fe[e]=i,i=null!=n(t,e,r)?e.toLowerCase():null,fe[e]=o),i}:function(t,e,n){if(!n)return t[f.camelCase("default-"+e)]?e.toLowerCase():null}}),Ce&&Ie||(f.attrHooks.value={set:function(t,e,n){if(!f.nodeName(t,"input"))return he&&he.set(t,e,n);t.defaultValue=e}}),Ie||(he={set:function(t,e,n){var r=t.getAttributeNode(n);if(r||t.setAttributeNode(r=t.ownerDocument.createAttribute(n)),r.value=e+="","value"===n||e===t.getAttribute(n))return e}},fe.id=fe.name=fe.coords=function(t,e,n){var r;if(!n)return(r=t.getAttributeNode(e))&&""!==r.value?r.value:null},f.valHooks.button={get:function(t,e){var n=t.getAttributeNode(e);if(n&&n.specified)return n.value},set:he.set},f.attrHooks.contenteditable={set:function(t,e,n){he.set(t,""!==e&&e,n)}},f.each(["width","height"],function(t,e){f.attrHooks[e]={set:function(t,n){if(""===n)return t.setAttribute(e,"auto"),n}}})),d.style||(f.attrHooks.style={get:function(t){return t.style.cssText||void 0},set:function(t,e){return t.style.cssText=e+""}});var me=/^(?:input|select|textarea|button|object)$/i,ye=/^(?:a|area)$/i;f.fn.extend({prop:function(t,e){return q(this,f.prop,t,e,arguments.length>1)},removeProp:function(t){return t=f.propFix[t]||t,this.each(function(){try{this[t]=void 0,delete this[t]}catch(t){}})}}),f.extend({propFix:{for:"htmlFor",class:"className"},prop:function(t,e,n){var r,i,o=t.nodeType;if(t&&3!==o&&8!==o&&2!==o)return(1!==o||!f.isXMLDoc(t))&&(e=f.propFix[e]||e,i=f.propHooks[e]),void 0!==n?i&&"set"in i&&void 0!==(r=i.set(t,n,e))?r:t[e]=n:i&&"get"in i&&null!==(r=i.get(t,e))?r:t[e]},propHooks:{tabIndex:{get:function(t){var e=f.find.attr(t,"tabindex");return e?parseInt(e,10):me.test(t.nodeName)||ye.test(t.nodeName)&&t.href?0:-1}}}}),d.hrefNormalized||f.each(["href","src"],function(t,e){f.propHooks[e]={get:function(t){return t.getAttribute(e,4)}}}),d.optSelected||(f.propHooks.selected={get:function(t){var e=t.parentNode;return e&&(e.selectedIndex,e.parentNode&&e.parentNode.selectedIndex),null}}),f.each(["tabIndex","readOnly","maxLength","cellSpacing","cellPadding","rowSpan","colSpan","useMap","frameBorder","contentEditable"],function(){f.propFix[this.toLowerCase()]=this}),d.enctype||(f.propFix.enctype="encoding");var Ee=/[\t\r\n\f]/g;f.fn.extend({addClass:function(t){var e,n,r,i,o,a,s=0,A=this.length,l="string"==typeof t&&t;if(f.isFunction(t))return this.each(function(e){f(this).addClass(t.call(this,e,this.className))});if(l)for(e=(t||"").match(L)||[];s<A;s++)if(r=1===(n=this[s]).nodeType&&(n.className?(" "+n.className+" ").replace(Ee," "):" ")){for(o=0;i=e[o++];)r.indexOf(" "+i+" ")<0&&(r+=i+" ");a=f.trim(r),n.className!==a&&(n.className=a)}return this},removeClass:function(t){var e,n,r,i,o,a,s=0,A=this.length,l=0===arguments.length||"string"==typeof t&&t;if(f.isFunction(t))return this.each(function(e){f(this).removeClass(t.call(this,e,this.className))});if(l)for(e=(t||"").match(L)||[];s<A;s++)if(r=1===(n=this[s]).nodeType&&(n.className?(" "+n.className+" ").replace(Ee," "):"")){for(o=0;i=e[o++];)for(;r.indexOf(" "+i+" ")>=0;)r=r.replace(" "+i+" "," ");a=t?f.trim(r):"",n.className!==a&&(n.className=a)}return this},toggleClass:function(t,e){var n=typeof t;return"boolean"==typeof e&&"string"===n?e?this.addClass(t):this.removeClass(t):f.isFunction(t)?this.each(function(n){f(this).toggleClass(t.call(this,n,this.className,e),e)}):this.each(function(){if("string"===n)for(var e,r=0,i=f(this),o=t.match(L)||[];e=o[r++];)i.hasClass(e)?i.removeClass(e):i.addClass(e);else n!==U&&"boolean"!==n||(this.className&&f._data(this,"__className__",this.className),this.className=this.className||!1===t?"":f._data(this,"__className__")||"")})},hasClass:function(t){for(var e=" "+t+" ",n=0,r=this.length;n<r;n++)if(1===this[n].nodeType&&(" "+this[n].className+" ").replace(Ee," ").indexOf(e)>=0)return!0;return!1}}),f.each("blur focus focusin focusout load resize scroll unload click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup error contextmenu".split(" "),function(t,e){f.fn[e]=function(t,n){return arguments.length>0?this.on(e,null,t,n):this.trigger(e)}}),f.fn.extend({hover:function(t,e){return this.mouseenter(t).mouseleave(e||t)},bind:function(t,e,n){return this.on(t,null,e,n)},unbind:function(t,e){return this.off(t,null,e)},delegate:function(t,e,n,r){return this.on(e,t,n,r)},undelegate:function(t,e,n){return 1===arguments.length?this.off(t,"**"):this.off(e,t||"**",n)}});var ve=f.now(),Be=/\?/,be=/(,)|(\[|{)|(}|])|"(?:[^"\\\r\n]|\\["\\\/bfnrt]|\\u[\da-fA-F]{4})*"\s*:?|true|false|null|-?(?!0\d)\d+(?:\.\d+|)(?:[eE][+-]?\d+|)/g;f.parseJSON=function(t){if(n.JSON&&n.JSON.parse)return n.JSON.parse(t+"");var e,r=null,i=f.trim(t+"");return i&&!f.trim(i.replace(be,function(t,n,i,o){return e&&n&&(r=0),0===r?t:(e=i||n,r+=!o-!i,"")}))?Function("return "+i)():f.error("Invalid JSON: "+t)},f.parseXML=function(t){var e;if(!t||"string"!=typeof t)return null;try{n.DOMParser?e=(new DOMParser).parseFromString(t,"text/xml"):((e=new ActiveXObject("Microsoft.XMLDOM")).async="false",e.loadXML(t))}catch(t){e=void 0}return e&&e.documentElement&&!e.getElementsByTagName("parsererror").length||f.error("Invalid XML: "+t),e};var Qe,we,xe=/#.*$/,_e=/([?&])_=[^&]*/,De=/^(.*?):[ \t]*([^\r\n]*)\r?$/gm,Me=/^(?:GET|HEAD)$/,ke=/^\/\//,Se=/^([\w.+-]+:)(?:\/\/(?:[^\/?#]*@|)([^\/?#:]*)(?::(\d+)|)|)/,Le={},Re={},Ne="*/".concat("*");try{we=location.href}catch(t){(we=x.createElement("a")).href="",we=we.href}function Fe(t){return function(e,n){"string"!=typeof e&&(n=e,e="*");var r,i=0,o=e.toLowerCase().match(L)||[];if(f.isFunction(n))for(;r=o[i++];)"+"===r.charAt(0)?(r=r.slice(1)||"*",(t[r]=t[r]||[]).unshift(n)):(t[r]=t[r]||[]).push(n)}}function Te(t,e,n,r){var i={},o=t===Re;function a(s){var A;return i[s]=!0,f.each(t[s]||[],function(t,s){var l=s(e,n,r);return"string"!=typeof l||o||i[l]?o?!(A=l):void 0:(e.dataTypes.unshift(l),a(l),!1)}),A}return a(e.dataTypes[0])||!i["*"]&&a("*")}function Ue(t,e){var n,r,i=f.ajaxSettings.flatOptions||{};for(r in e)void 0!==e[r]&&((i[r]?t:n||(n={}))[r]=e[r]);return n&&f.extend(!0,t,n),t}Qe=Se.exec(we.toLowerCase())||[],f.extend({active:0,lastModified:{},etag:{},ajaxSettings:{url:we,type:"GET",isLocal:/^(?:about|app|app-storage|.+-extension|file|res|widget):$/.test(Qe[1]),global:!0,processData:!0,async:!0,contentType:"application/x-www-form-urlencoded; charset=UTF-8",accepts:{"*":Ne,text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript"},contents:{xml:/xml/,html:/html/,json:/json/},responseFields:{xml:"responseXML",text:"responseText",json:"responseJSON"},converters:{"* text":String,"text html":!0,"text json":f.parseJSON,"text xml":f.parseXML},flatOptions:{url:!0,context:!0}},ajaxSetup:function(t,e){return e?Ue(Ue(t,f.ajaxSettings),e):Ue(f.ajaxSettings,t)},ajaxPrefilter:Fe(Le),ajaxTransport:Fe(Re),ajax:function(t,e){"object"==typeof t&&(e=t,t=void 0),e=e||{};var n,r,i,o,a,s,A,l,c=f.ajaxSetup({},e),u=c.context||c,g=c.context&&(u.nodeType||u.jquery)?f(u):f.event,h=f.Deferred(),d=f.Callbacks("once memory"),p=c.statusCode||{},I={},C={},m=0,y="canceled",E={readyState:0,getResponseHeader:function(t){var e;if(2===m){if(!l)for(l={};e=De.exec(o);)l[e[1].toLowerCase()]=e[2];e=l[t.toLowerCase()]}return null==e?null:e},getAllResponseHeaders:function(){return 2===m?o:null},setRequestHeader:function(t,e){var n=t.toLowerCase();return m||(t=C[n]=C[n]||t,I[t]=e),this},overrideMimeType:function(t){return m||(c.mimeType=t),this},statusCode:function(t){var e;if(t)if(m<2)for(e in t)p[e]=[p[e],t[e]];else E.always(t[E.status]);return this},abort:function(t){var e=t||y;return A&&A.abort(e),v(0,e),this}};if(h.promise(E).complete=d.add,E.success=E.done,E.error=E.fail,c.url=((t||c.url||we)+"").replace(xe,"").replace(ke,Qe[1]+"//"),c.type=e.method||e.type||c.method||c.type,c.dataTypes=f.trim(c.dataType||"*").toLowerCase().match(L)||[""],null==c.crossDomain&&(n=Se.exec(c.url.toLowerCase()),c.crossDomain=!(!n||n[1]===Qe[1]&&n[2]===Qe[2]&&(n[3]||("http:"===n[1]?"80":"443"))===(Qe[3]||("http:"===Qe[1]?"80":"443")))),c.data&&c.processData&&"string"!=typeof c.data&&(c.data=f.param(c.data,c.traditional)),Te(Le,c,e,E),2===m)return E;for(r in(s=c.global)&&0==f.active++&&f.event.trigger("ajaxStart"),c.type=c.type.toUpperCase(),c.hasContent=!Me.test(c.type),i=c.url,c.hasContent||(c.data&&(i=c.url+=(Be.test(i)?"&":"?")+c.data,delete c.data),!1===c.cache&&(c.url=_e.test(i)?i.replace(_e,"$1_="+ve++):i+(Be.test(i)?"&":"?")+"_="+ve++)),c.ifModified&&(f.lastModified[i]&&E.setRequestHeader("If-Modified-Since",f.lastModified[i]),f.etag[i]&&E.setRequestHeader("If-None-Match",f.etag[i])),(c.data&&c.hasContent&&!1!==c.contentType||e.contentType)&&E.setRequestHeader("Content-Type",c.contentType),E.setRequestHeader("Accept",c.dataTypes[0]&&c.accepts[c.dataTypes[0]]?c.accepts[c.dataTypes[0]]+("*"!==c.dataTypes[0]?", "+Ne+"; q=0.01":""):c.accepts["*"]),c.headers)E.setRequestHeader(r,c.headers[r]);if(c.beforeSend&&(!1===c.beforeSend.call(u,E,c)||2===m))return E.abort();for(r in y="abort",{success:1,error:1,complete:1})E[r](c[r]);if(A=Te(Re,c,e,E)){E.readyState=1,s&&g.trigger("ajaxSend",[E,c]),c.async&&c.timeout>0&&(a=setTimeout(function(){E.abort("timeout")},c.timeout));try{m=1,A.send(I,v)}catch(t){if(!(m<2))throw t;v(-1,t)}}else v(-1,"No Transport");function v(t,e,n,r){var l,I,C,y,v,B=e;2!==m&&(m=2,a&&clearTimeout(a),A=void 0,o=r||"",E.readyState=t>0?4:0,l=t>=200&&t<300||304===t,n&&(y=function(t,e,n){for(var r,i,o,a,s=t.contents,A=t.dataTypes;"*"===A[0];)A.shift(),void 0===i&&(i=t.mimeType||e.getResponseHeader("Content-Type"));if(i)for(a in s)if(s[a]&&s[a].test(i)){A.unshift(a);break}if(A[0]in n)o=A[0];else{for(a in n){if(!A[0]||t.converters[a+" "+A[0]]){o=a;break}r||(r=a)}o=o||r}if(o)return o!==A[0]&&A.unshift(o),n[o]}(c,E,n)),y=function(t,e,n,r){var i,o,a,s,A,l={},c=t.dataTypes.slice();if(c[1])for(a in t.converters)l[a.toLowerCase()]=t.converters[a];for(o=c.shift();o;)if(t.responseFields[o]&&(n[t.responseFields[o]]=e),!A&&r&&t.dataFilter&&(e=t.dataFilter(e,t.dataType)),A=o,o=c.shift())if("*"===o)o=A;else if("*"!==A&&A!==o){if(!(a=l[A+" "+o]||l["* "+o]))for(i in l)if((s=i.split(" "))[1]===o&&(a=l[A+" "+s[0]]||l["* "+s[0]])){!0===a?a=l[i]:!0!==l[i]&&(o=s[0],c.unshift(s[1]));break}if(!0!==a)if(a&&t.throws)e=a(e);else try{e=a(e)}catch(t){return{state:"parsererror",error:a?t:"No conversion from "+A+" to "+o}}}return{state:"success",data:e}}(c,y,E,l),l?(c.ifModified&&((v=E.getResponseHeader("Last-Modified"))&&(f.lastModified[i]=v),(v=E.getResponseHeader("etag"))&&(f.etag[i]=v)),204===t||"HEAD"===c.type?B="nocontent":304===t?B="notmodified":(B=y.state,I=y.data,l=!(C=y.error))):(C=B,!t&&B||(B="error",t<0&&(t=0))),E.status=t,E.statusText=(e||B)+"",l?h.resolveWith(u,[I,B,E]):h.rejectWith(u,[E,B,C]),E.statusCode(p),p=void 0,s&&g.trigger(l?"ajaxSuccess":"ajaxError",[E,c,l?I:C]),d.fireWith(u,[E,B]),s&&(g.trigger("ajaxComplete",[E,c]),--f.active||f.event.trigger("ajaxStop")))}return E},getJSON:function(t,e,n){return f.get(t,e,n,"json")},getScript:function(t,e){return f.get(t,void 0,e,"script")}}),f.each(["get","post"],function(t,e){f[e]=function(t,n,r,i){return f.isFunction(n)&&(i=i||r,r=n,n=void 0),f.ajax({url:t,type:e,dataType:i,data:n,success:r})}}),f.each(["ajaxStart","ajaxStop","ajaxComplete","ajaxError","ajaxSuccess","ajaxSend"],function(t,e){f.fn[e]=function(t){return this.on(e,t)}}),f._evalUrl=function(t){return f.ajax({url:t,type:"GET",dataType:"script",async:!1,global:!1,throws:!0})},f.fn.extend({wrapAll:function(t){if(f.isFunction(t))return this.each(function(e){f(this).wrapAll(t.call(this,e))});if(this[0]){var e=f(t,this[0].ownerDocument).eq(0).clone(!0);this[0].parentNode&&e.insertBefore(this[0]),e.map(function(){for(var t=this;t.firstChild&&1===t.firstChild.nodeType;)t=t.firstChild;return t}).append(this)}return this},wrapInner:function(t){return f.isFunction(t)?this.each(function(e){f(this).wrapInner(t.call(this,e))}):this.each(function(){var e=f(this),n=e.contents();n.length?n.wrapAll(t):e.append(t)})},wrap:function(t){var e=f.isFunction(t);return this.each(function(n){f(this).wrapAll(e?t.call(this,n):t)})},unwrap:function(){return this.parent().each(function(){f.nodeName(this,"body")||f(this).replaceWith(this.childNodes)}).end()}}),f.expr.filters.hidden=function(t){return t.offsetWidth<=0&&t.offsetHeight<=0||!d.reliableHiddenOffsets()&&"none"===(t.style&&t.style.display||f.css(t,"display"))},f.expr.filters.visible=function(t){return!f.expr.filters.hidden(t)};var Oe=/%20/g,Ge=/\[\]$/,He=/\r?\n/g,je=/^(?:submit|button|image|reset|file)$/i,Ye=/^(?:input|select|textarea|keygen)/i;function Pe(t,e,n,r){var i;if(f.isArray(e))f.each(e,function(e,i){n||Ge.test(t)?r(t,i):Pe(t+"["+("object"==typeof i?e:"")+"]",i,n,r)});else if(n||"object"!==f.type(e))r(t,e);else for(i in e)Pe(t+"["+i+"]",e[i],n,r)}f.param=function(t,e){var n,r=[],i=function(t,e){e=f.isFunction(e)?e():null==e?"":e,r[r.length]=encodeURIComponent(t)+"="+encodeURIComponent(e)};if(void 0===e&&(e=f.ajaxSettings&&f.ajaxSettings.traditional),f.isArray(t)||t.jquery&&!f.isPlainObject(t))f.each(t,function(){i(this.name,this.value)});else for(n in t)Pe(n,t[n],e,i);return r.join("&").replace(Oe,"+")},f.fn.extend({serialize:function(){return f.param(this.serializeArray())},serializeArray:function(){return this.map(function(){var t=f.prop(this,"elements");return t?f.makeArray(t):this}).filter(function(){var t=this.type;return this.name&&!f(this).is(":disabled")&&Ye.test(this.nodeName)&&!je.test(t)&&(this.checked||!V.test(t))}).map(function(t,e){var n=f(this).val();return null==n?null:f.isArray(n)?f.map(n,function(t){return{name:e.name,value:t.replace(He,"\r\n")}}):{name:e.name,value:n.replace(He,"\r\n")}}).get()}}),f.ajaxSettings.xhr=void 0!==n.ActiveXObject?function(){return!this.isLocal&&/^(get|post|head|put|delete|options)$/i.test(this.type)&&qe()||function(){try{return new n.ActiveXObject("Microsoft.XMLHTTP")}catch(t){}}()}:qe;var Ke=0,ze={},Je=f.ajaxSettings.xhr();function qe(){try{return new n.XMLHttpRequest}catch(t){}}n.ActiveXObject&&f(n).on("unload",function(){for(var t in ze)ze[t](void 0,!0)}),d.cors=!!Je&&"withCredentials"in Je,(Je=d.ajax=!!Je)&&f.ajaxTransport(function(t){var e;if(!t.crossDomain||d.cors)return{send:function(n,r){var i,o=t.xhr(),a=++Ke;if(o.open(t.type,t.url,t.async,t.username,t.password),t.xhrFields)for(i in t.xhrFields)o[i]=t.xhrFields[i];for(i in t.mimeType&&o.overrideMimeType&&o.overrideMimeType(t.mimeType),t.crossDomain||n["X-Requested-With"]||(n["X-Requested-With"]="XMLHttpRequest"),n)void 0!==n[i]&&o.setRequestHeader(i,n[i]+"");o.send(t.hasContent&&t.data||null),e=function(n,i){var s,A,l;if(e&&(i||4===o.readyState))if(delete ze[a],e=void 0,o.onreadystatechange=f.noop,i)4!==o.readyState&&o.abort();else{l={},s=o.status,"string"==typeof o.responseText&&(l.text=o.responseText);try{A=o.statusText}catch(t){A=""}s||!t.isLocal||t.crossDomain?1223===s&&(s=204):s=l.text?200:404}l&&r(s,A,l,o.getAllResponseHeaders())},t.async?4===o.readyState?setTimeout(e):o.onreadystatechange=ze[a]=e:e()},abort:function(){e&&e(void 0,!0)}}}),f.ajaxSetup({accepts:{script:"text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"},contents:{script:/(?:java|ecma)script/},converters:{"text script":function(t){return f.globalEval(t),t}}}),f.ajaxPrefilter("script",function(t){void 0===t.cache&&(t.cache=!1),t.crossDomain&&(t.type="GET",t.global=!1)}),f.ajaxTransport("script",function(t){if(t.crossDomain){var e,n=x.head||f("head")[0]||x.documentElement;return{send:function(r,i){(e=x.createElement("script")).async=!0,t.scriptCharset&&(e.charset=t.scriptCharset),e.src=t.url,e.onload=e.onreadystatechange=function(t,n){(n||!e.readyState||/loaded|complete/.test(e.readyState))&&(e.onload=e.onreadystatechange=null,e.parentNode&&e.parentNode.removeChild(e),e=null,n||i(200,"success"))},n.insertBefore(e,n.firstChild)},abort:function(){e&&e.onload(void 0,!0)}}}});var Ve=[],We=/(=)\?(?=&|$)|\?\?/;f.ajaxSetup({jsonp:"callback",jsonpCallback:function(){var t=Ve.pop()||f.expando+"_"+ve++;return this[t]=!0,t}}),f.ajaxPrefilter("json jsonp",function(t,e,r){var i,o,a,s=!1!==t.jsonp&&(We.test(t.url)?"url":"string"==typeof t.data&&!(t.contentType||"").indexOf("application/x-www-form-urlencoded")&&We.test(t.data)&&"data");if(s||"jsonp"===t.dataTypes[0])return i=t.jsonpCallback=f.isFunction(t.jsonpCallback)?t.jsonpCallback():t.jsonpCallback,s?t[s]=t[s].replace(We,"$1"+i):!1!==t.jsonp&&(t.url+=(Be.test(t.url)?"&":"?")+t.jsonp+"="+i),t.converters["script json"]=function(){return a||f.error(i+" was not called"),a[0]},t.dataTypes[0]="json",o=n[i],n[i]=function(){a=arguments},r.always(function(){n[i]=o,t[i]&&(t.jsonpCallback=e.jsonpCallback,Ve.push(i)),a&&f.isFunction(o)&&o(a[0]),a=o=void 0}),"script"}),f.parseHTML=function(t,e,n){if(!t||"string"!=typeof t)return null;"boolean"==typeof e&&(n=e,e=!1),e=e||x;var r=B.exec(t),i=!n&&[];return r?[e.createElement(r[1])]:(r=f.buildFragment([t],e,i),i&&i.length&&f(i).remove(),f.merge([],r.childNodes))};var Xe=f.fn.load;f.fn.load=function(t,e,n){if("string"!=typeof t&&Xe)return Xe.apply(this,arguments);var r,i,o,a=this,s=t.indexOf(" ");return s>=0&&(r=t.slice(s,t.length),t=t.slice(0,s)),f.isFunction(e)?(n=e,e=void 0):e&&"object"==typeof e&&(o="POST"),a.length>0&&f.ajax({url:t,type:o,dataType:"html",data:e}).done(function(t){i=arguments,a.html(r?f("<div>").append(f.parseHTML(t)).find(r):t)}).complete(n&&function(t,e){a.each(n,i||[t.responseText,e,t])}),this},f.expr.filters.animated=function(t){return f.grep(f.timers,function(e){return t===e.elem}).length};var Ze=n.document.documentElement;function $e(t){return f.isWindow(t)?t:9===t.nodeType&&(t.defaultView||t.parentWindow)}f.offset={setOffset:function(t,e,n){var r,i,o,a,s,A,l=f.css(t,"position"),c=f(t),u={};"static"===l&&(t.style.position="relative"),s=c.offset(),o=f.css(t,"top"),A=f.css(t,"left"),("absolute"===l||"fixed"===l)&&f.inArray("auto",[o,A])>-1?(a=(r=c.position()).top,i=r.left):(a=parseFloat(o)||0,i=parseFloat(A)||0),f.isFunction(e)&&(e=e.call(t,n,s)),null!=e.top&&(u.top=e.top-s.top+a),null!=e.left&&(u.left=e.left-s.left+i),"using"in e?e.using.call(t,u):c.css(u)}},f.fn.extend({offset:function(t){if(arguments.length)return void 0===t?this:this.each(function(e){f.offset.setOffset(this,t,e)});var e,n,r={top:0,left:0},i=this[0],o=i&&i.ownerDocument;return o?(e=o.documentElement,f.contains(e,i)?(typeof i.getBoundingClientRect!==U&&(r=i.getBoundingClientRect()),n=$e(o),{top:r.top+(n.pageYOffset||e.scrollTop)-(e.clientTop||0),left:r.left+(n.pageXOffset||e.scrollLeft)-(e.clientLeft||0)}):r):void 0},position:function(){if(this[0]){var t,e,n={top:0,left:0},r=this[0];return"fixed"===f.css(r,"position")?e=r.getBoundingClientRect():(t=this.offsetParent(),e=this.offset(),f.nodeName(t[0],"html")||(n=t.offset()),n.top+=f.css(t[0],"borderTopWidth",!0),n.left+=f.css(t[0],"borderLeftWidth",!0)),{top:e.top-n.top-f.css(r,"marginTop",!0),left:e.left-n.left-f.css(r,"marginLeft",!0)}}},offsetParent:function(){return this.map(function(){for(var t=this.offsetParent||Ze;t&&!f.nodeName(t,"html")&&"static"===f.css(t,"position");)t=t.offsetParent;return t||Ze})}}),f.each({scrollLeft:"pageXOffset",scrollTop:"pageYOffset"},function(t,e){var n=/Y/.test(e);f.fn[t]=function(r){return q(this,function(t,r,i){var o=$e(t);if(void 0===i)return o?e in o?o[e]:o.document.documentElement[r]:t[r];o?o.scrollTo(n?f(o).scrollLeft():i,n?i:f(o).scrollTop()):t[r]=i},t,r,arguments.length,null)}}),f.each(["top","left"],function(t,e){f.cssHooks[e]=Gt(d.pixelPosition,function(t,n){if(n)return n=Ft(t,e),Ut.test(n)?f(t).position()[e]+"px":n})}),f.each({Height:"height",Width:"width"},function(t,e){f.each({padding:"inner"+t,content:e,"":"outer"+t},function(n,r){f.fn[r]=function(r,i){var o=arguments.length&&(n||"boolean"!=typeof r),a=n||(!0===r||!0===i?"margin":"border");return q(this,function(e,n,r){var i;return f.isWindow(e)?e.document.documentElement["client"+t]:9===e.nodeType?(i=e.documentElement,Math.max(e.body["scroll"+t],i["scroll"+t],e.body["offset"+t],i["offset"+t],i["client"+t])):void 0===r?f.css(e,n,a):f.style(e,n,r,a)},e,o?r:void 0,o,null)}})}),f.fn.size=function(){return this.length},f.fn.andSelf=f.fn.addBack,void 0===(r=function(){return f}.apply(e,[]))||(t.exports=r);var tn=n.jQuery,en=n.$;return f.noConflict=function(t){return n.$===f&&(n.$=en),t&&n.jQuery===f&&(n.jQuery=tn),f},typeof i===U&&(n.jQuery=n.$=f),f},"object"==typeof t.exports?t.exports=i.document?o(i,!0):function(t){if(!t.document)throw new Error("jQuery requires a window with a document");return o(t)}:o(i)},function(t,e,n){"use strict";var r=function(t,e){if((n=(t=e?t.toExponential(e-1):t.toExponential()).indexOf("e"))<0)return null;var n,r=t.slice(0,n);return[r.length>1?r[0]+r.slice(2):r,+t.slice(n+1)]},i=function(t){return(t=r(Math.abs(t)))?t[1]:NaN},o=/^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i;function a(t){return new s(t)}function s(t){if(!(e=o.exec(t)))throw new Error("invalid format: "+t);var e;this.fill=e[1]||" ",this.align=e[2]||">",this.sign=e[3]||"-",this.symbol=e[4]||"",this.zero=!!e[5],this.width=e[6]&&+e[6],this.comma=!!e[7],this.precision=e[8]&&+e[8].slice(1),this.trim=!!e[9],this.type=e[10]||""}a.prototype=s.prototype,s.prototype.toString=function(){return this.fill+this.align+this.sign+this.symbol+(this.zero?"0":"")+(null==this.width?"":Math.max(1,0|this.width))+(this.comma?",":"")+(null==this.precision?"":"."+Math.max(0,0|this.precision))+(this.trim?"~":"")+this.type};var A,l,c,u,g=function(t){t:for(var e,n=t.length,r=1,i=-1;r<n;++r)switch(t[r]){case".":i=e=r;break;case"0":0===i&&(i=r),e=r;break;default:if(i>0){if(!+t[r])break t;i=0}}return i>0?t.slice(0,i)+t.slice(e+1):t},h=function(t,e){var n=r(t,e);if(!n)return t+"";var i=n[0],o=n[1];return o<0?"0."+new Array(-o).join("0")+i:i.length>o+1?i.slice(0,o+1)+"."+i.slice(o+1):i+new Array(o-i.length+2).join("0")},d={"%":function(t,e){return(100*t).toFixed(e)},b:function(t){return Math.round(t).toString(2)},c:function(t){return t+""},d:function(t){return Math.round(t).toString(10)},e:function(t,e){return t.toExponential(e)},f:function(t,e){return t.toFixed(e)},g:function(t,e){return t.toPrecision(e)},o:function(t){return Math.round(t).toString(8)},p:function(t,e){return h(100*t,e)},r:h,s:function(t,e){var n=r(t,e);if(!n)return t+"";var i=n[0],o=n[1],a=o-(A=3*Math.max(-8,Math.min(8,Math.floor(o/3))))+1,s=i.length;return a===s?i:a>s?i+new Array(a-s+1).join("0"):a>0?i.slice(0,a)+"."+i.slice(a):"0."+new Array(1-a).join("0")+r(t,Math.max(0,e+a-1))[0]},X:function(t){return Math.round(t).toString(16).toUpperCase()},x:function(t){return Math.round(t).toString(16)}},f=function(t){return t},p=["y","z","a","f","p","n","µ","m","","k","M","G","T","P","E","Z","Y"],I=function(t){var e,n,r=t.grouping&&t.thousands?(e=t.grouping,n=t.thousands,function(t,r){for(var i=t.length,o=[],a=0,s=e[0],A=0;i>0&&s>0&&(A+s+1>r&&(s=Math.max(1,r-A)),o.push(t.substring(i-=s,i+s)),!((A+=s+1)>r));)s=e[a=(a+1)%e.length];return o.reverse().join(n)}):f,o=t.currency,s=t.decimal,l=t.numerals?function(t){return function(e){return e.replace(/[0-9]/g,function(e){return t[+e]})}}(t.numerals):f,c=t.percent||"%";function u(t){var e=(t=a(t)).fill,n=t.align,i=t.sign,u=t.symbol,h=t.zero,f=t.width,I=t.comma,C=t.precision,m=t.trim,y=t.type;"n"===y?(I=!0,y="g"):d[y]||(null==C&&(C=12),m=!0,y="g"),(h||"0"===e&&"="===n)&&(h=!0,e="0",n="=");var E="$"===u?o[0]:"#"===u&&/[boxX]/.test(y)?"0"+y.toLowerCase():"",v="$"===u?o[1]:/[%p]/.test(y)?c:"",B=d[y],b=/[defgprs%]/.test(y);function Q(t){var o,a,c,u=E,d=v;if("c"===y)d=B(t)+d,t="";else{var Q=(t=+t)<0;if(t=B(Math.abs(t),C),m&&(t=g(t)),Q&&0==+t&&(Q=!1),u=(Q?"("===i?i:"-":"-"===i||"("===i?"":i)+u,d=("s"===y?p[8+A/3]:"")+d+(Q&&"("===i?")":""),b)for(o=-1,a=t.length;++o<a;)if(48>(c=t.charCodeAt(o))||c>57){d=(46===c?s+t.slice(o+1):t.slice(o))+d,t=t.slice(0,o);break}}I&&!h&&(t=r(t,1/0));var w=u.length+t.length+d.length,x=w<f?new Array(f-w+1).join(e):"";switch(I&&h&&(t=r(x+t,x.length?f-d.length:1/0),x=""),n){case"<":t=u+t+d+x;break;case"=":t=u+x+t+d;break;case"^":t=x.slice(0,w=x.length>>1)+u+t+d+x.slice(w);break;default:t=x+u+t+d}return l(t)}return C=null==C?6:/[gprs]/.test(y)?Math.max(1,Math.min(21,C)):Math.max(0,Math.min(20,C)),Q.toString=function(){return t+""},Q}return{format:u,formatPrefix:function(t,e){var n=u(((t=a(t)).type="f",t)),r=3*Math.max(-8,Math.min(8,Math.floor(i(e)/3))),o=Math.pow(10,-r),s=p[8+r/3];return function(t){return n(o*t)+s}}}};function C(t){return l=I(t),c=l.format,u=l.formatPrefix,l}C({decimal:".",thousands:",",grouping:[3],currency:["$",""]});var m=function(t){return Math.max(0,-i(Math.abs(t)))},y=function(t,e){return Math.max(0,3*Math.max(-8,Math.min(8,Math.floor(i(e)/3)))-i(Math.abs(t)))},E=function(t,e){return t=Math.abs(t),e=Math.abs(e)-t,Math.max(0,i(e)-i(t))+1};n.d(e,"b",function(){return C}),n.d(e,"a",function(){return c}),n.d(e,"d",function(){return u}),n.d(e,"c",function(){return I}),n.d(e,"e",function(){return a}),n.d(e,"f",function(){return m}),n.d(e,"g",function(){return y}),n.d(e,"h",function(){return E})},function(t,e,n){var r=n(49)("wks"),i=n(36),o=n(13).Symbol,a="function"==typeof o;(t.exports=function(t){return r[t]||(r[t]=a&&o[t]||(a?o:i)("Symbol."+t))}).store=r},function(t,e,n){"use strict";n.r(e);var r=Math.PI,i=2*r,o=i-1e-6;function a(){this._x0=this._y0=this._x1=this._y1=null,this._=""}function s(){return new a}a.prototype=s.prototype={constructor:a,moveTo:function(t,e){this._+="M"+(this._x0=this._x1=+t)+","+(this._y0=this._y1=+e)},closePath:function(){null!==this._x1&&(this._x1=this._x0,this._y1=this._y0,this._+="Z")},lineTo:function(t,e){this._+="L"+(this._x1=+t)+","+(this._y1=+e)},quadraticCurveTo:function(t,e,n,r){this._+="Q"+ +t+","+ +e+","+(this._x1=+n)+","+(this._y1=+r)},bezierCurveTo:function(t,e,n,r,i,o){this._+="C"+ +t+","+ +e+","+ +n+","+ +r+","+(this._x1=+i)+","+(this._y1=+o)},arcTo:function(t,e,n,i,o){t=+t,e=+e,n=+n,i=+i,o=+o;var a=this._x1,s=this._y1,A=n-t,l=i-e,c=a-t,u=s-e,g=c*c+u*u;if(o<0)throw new Error("negative radius: "+o);if(null===this._x1)this._+="M"+(this._x1=t)+","+(this._y1=e);else if(g>1e-6)if(Math.abs(u*A-l*c)>1e-6&&o){var h=n-a,d=i-s,f=A*A+l*l,p=h*h+d*d,I=Math.sqrt(f),C=Math.sqrt(g),m=o*Math.tan((r-Math.acos((f+g-p)/(2*I*C)))/2),y=m/C,E=m/I;Math.abs(y-1)>1e-6&&(this._+="L"+(t+y*c)+","+(e+y*u)),this._+="A"+o+","+o+",0,0,"+ +(u*h>c*d)+","+(this._x1=t+E*A)+","+(this._y1=e+E*l)}else this._+="L"+(this._x1=t)+","+(this._y1=e)},arc:function(t,e,n,a,s,A){t=+t,e=+e;var l=(n=+n)*Math.cos(a),c=n*Math.sin(a),u=t+l,g=e+c,h=1^A,d=A?a-s:s-a;if(n<0)throw new Error("negative radius: "+n);null===this._x1?this._+="M"+u+","+g:(Math.abs(this._x1-u)>1e-6||Math.abs(this._y1-g)>1e-6)&&(this._+="L"+u+","+g),n&&(d<0&&(d=d%i+i),d>o?this._+="A"+n+","+n+",0,1,"+h+","+(t-l)+","+(e-c)+"A"+n+","+n+",0,1,"+h+","+(this._x1=u)+","+(this._y1=g):d>1e-6&&(this._+="A"+n+","+n+",0,"+ +(d>=r)+","+h+","+(this._x1=t+n*Math.cos(s))+","+(this._y1=e+n*Math.sin(s))))},rect:function(t,e,n,r){this._+="M"+(this._x0=this._x1=+t)+","+(this._y0=this._y1=+e)+"h"+ +n+"v"+ +r+"h"+-n+"Z"},toString:function(){return this._}};var A=s;n.d(e,"path",function(){return A})},function(t,e,n){"use strict";function r(){}function i(t,e){var n=new r;if(t instanceof r)t.each(function(t,e){n.set(e,t)});else if(Array.isArray(t)){var i,o=-1,a=t.length;if(null==e)for(;++o<a;)n.set(o,t[o]);else for(;++o<a;)n.set(e(i=t[o],o,t),i)}else if(t)for(var s in t)n.set(s,t[s]);return n}r.prototype=i.prototype={constructor:r,has:function(t){return"$"+t in this},get:function(t){return this["$"+t]},set:function(t,e){return this["$"+t]=e,this},remove:function(t){var e="$"+t;return e in this&&delete this[e]},clear:function(){for(var t in this)"$"===t[0]&&delete this[t]},keys:function(){var t=[];for(var e in this)"$"===e[0]&&t.push(e.slice(1));return t},values:function(){var t=[];for(var e in this)"$"===e[0]&&t.push(this[e]);return t},entries:function(){var t=[];for(var e in this)"$"===e[0]&&t.push({key:e.slice(1),value:this[e]});return t},size:function(){var t=0;for(var e in this)"$"===e[0]&&++t;return t},empty:function(){for(var t in this)if("$"===t[0])return!1;return!0},each:function(t){for(var e in this)"$"===e[0]&&t(this[e],e.slice(1),this)}};var o=i,a=function(){var t,e,n,r=[],i=[];function a(n,i,s,A){if(i>=r.length)return null!=t&&n.sort(t),null!=e?e(n):n;for(var l,c,u,g=-1,h=n.length,d=r[i++],f=o(),p=s();++g<h;)(u=f.get(l=d(c=n[g])+""))?u.push(c):f.set(l,[c]);return f.each(function(t,e){A(p,e,a(t,i,s,A))}),p}return n={object:function(t){return a(t,0,s,A)},map:function(t){return a(t,0,l,c)},entries:function(t){return function t(n,o){if(++o>r.length)return n;var a,s=i[o-1];return null!=e&&o>=r.length?a=n.entries():(a=[],n.each(function(e,n){a.push({key:n,values:t(e,o)})})),null!=s?a.sort(function(t,e){return s(t.key,e.key)}):a}(a(t,0,l,c),0)},key:function(t){return r.push(t),n},sortKeys:function(t){return i[r.length-1]=t,n},sortValues:function(e){return t=e,n},rollup:function(t){return e=t,n}}};function s(){return{}}function A(t,e,n){t[e]=n}function l(){return o()}function c(t,e,n){t.set(e,n)}function u(){}var g=o.prototype;function h(t,e){var n=new u;if(t instanceof u)t.each(function(t){n.add(t)});else if(t){var r=-1,i=t.length;if(null==e)for(;++r<i;)n.add(t[r]);else for(;++r<i;)n.add(e(t[r],r,t))}return n}u.prototype=h.prototype={constructor:u,has:g.has,add:function(t){return this["$"+(t+="")]=t,this},remove:g.remove,clear:g.clear,values:g.keys,size:g.size,empty:g.empty,each:g.each};var d=h,f=function(t){var e=[];for(var n in t)e.push(n);return e},p=function(t){var e=[];for(var n in t)e.push(t[n]);return e},I=function(t){var e=[];for(var n in t)e.push({key:n,value:t[n]});return e};n.d(e,"d",function(){return a}),n.d(e,"e",function(){return d}),n.d(e,"c",function(){return o}),n.d(e,"b",function(){return f}),n.d(e,"f",function(){return p}),n.d(e,"a",function(){return I})},function(t,e,n){"use strict";var r=n(1);function i(t){if(0<=t.y&&t.y<100){var e=new Date(-1,t.m,t.d,t.H,t.M,t.S,t.L);return e.setFullYear(t.y),e}return new Date(t.y,t.m,t.d,t.H,t.M,t.S,t.L)}function o(t){if(0<=t.y&&t.y<100){var e=new Date(Date.UTC(-1,t.m,t.d,t.H,t.M,t.S,t.L));return e.setUTCFullYear(t.y),e}return new Date(Date.UTC(t.y,t.m,t.d,t.H,t.M,t.S,t.L))}function a(t){return{y:t,m:0,d:1,H:0,M:0,S:0,L:0}}function s(t){var e=t.dateTime,n=t.date,s=t.time,A=t.periods,l=t.days,c=t.shortDays,u=t.months,g=t.shortMonths,d=m(A),f=y(A),p=m(l),I=y(l),C=m(c),Bt=y(c),bt=m(u),Qt=y(u),wt=m(g),xt=y(g),_t={a:function(t){return c[t.getDay()]},A:function(t){return l[t.getDay()]},b:function(t){return g[t.getMonth()]},B:function(t){return u[t.getMonth()]},c:null,d:G,e:G,f:K,H:H,I:j,j:Y,L:P,m:z,M:J,p:function(t){return A[+(t.getHours()>=12)]},Q:Et,s:vt,S:q,u:V,U:W,V:X,w:Z,W:$,x:null,X:null,y:tt,Y:et,Z:nt,"%":yt},Dt={a:function(t){return c[t.getUTCDay()]},A:function(t){return l[t.getUTCDay()]},b:function(t){return g[t.getUTCMonth()]},B:function(t){return u[t.getUTCMonth()]},c:null,d:rt,e:rt,f:At,H:it,I:ot,j:at,L:st,m:lt,M:ct,p:function(t){return A[+(t.getUTCHours()>=12)]},Q:Et,s:vt,S:ut,u:gt,U:ht,V:dt,w:ft,W:pt,x:null,X:null,y:It,Y:Ct,Z:mt,"%":yt},Mt={a:function(t,e,n){var r=C.exec(e.slice(n));return r?(t.w=Bt[r[0].toLowerCase()],n+r[0].length):-1},A:function(t,e,n){var r=p.exec(e.slice(n));return r?(t.w=I[r[0].toLowerCase()],n+r[0].length):-1},b:function(t,e,n){var r=wt.exec(e.slice(n));return r?(t.m=xt[r[0].toLowerCase()],n+r[0].length):-1},B:function(t,e,n){var r=bt.exec(e.slice(n));return r?(t.m=Qt[r[0].toLowerCase()],n+r[0].length):-1},c:function(t,n,r){return Lt(t,e,n,r)},d:M,e:M,f:F,H:S,I:S,j:k,L:N,m:D,M:L,p:function(t,e,n){var r=d.exec(e.slice(n));return r?(t.p=f[r[0].toLowerCase()],n+r[0].length):-1},Q:U,s:O,S:R,u:v,U:B,V:b,w:E,W:Q,x:function(t,e,r){return Lt(t,n,e,r)},X:function(t,e,n){return Lt(t,s,e,n)},y:x,Y:w,Z:_,"%":T};function kt(t,e){return function(n){var r,i,o,a=[],s=-1,A=0,l=t.length;for(n instanceof Date||(n=new Date(+n));++s<l;)37===t.charCodeAt(s)&&(a.push(t.slice(A,s)),null!=(i=h[r=t.charAt(++s)])?r=t.charAt(++s):i="e"===r?" ":"0",(o=e[r])&&(r=o(n,i)),a.push(r),A=s+1);return a.push(t.slice(A,s)),a.join("")}}function St(t,e){return function(n){var i,s,A=a(1900);if(Lt(A,t,n+="",0)!=n.length)return null;if("Q"in A)return new Date(A.Q);if("p"in A&&(A.H=A.H%12+12*A.p),"V"in A){if(A.V<1||A.V>53)return null;"w"in A||(A.w=1),"Z"in A?(s=(i=o(a(A.y))).getUTCDay(),i=s>4||0===s?r.utcMonday.ceil(i):Object(r.utcMonday)(i),i=r.utcDay.offset(i,7*(A.V-1)),A.y=i.getUTCFullYear(),A.m=i.getUTCMonth(),A.d=i.getUTCDate()+(A.w+6)%7):(s=(i=e(a(A.y))).getDay(),i=s>4||0===s?r.timeMonday.ceil(i):Object(r.timeMonday)(i),i=r.timeDay.offset(i,7*(A.V-1)),A.y=i.getFullYear(),A.m=i.getMonth(),A.d=i.getDate()+(A.w+6)%7)}else("W"in A||"U"in A)&&("w"in A||(A.w="u"in A?A.u%7:"W"in A?1:0),s="Z"in A?o(a(A.y)).getUTCDay():e(a(A.y)).getDay(),A.m=0,A.d="W"in A?(A.w+6)%7+7*A.W-(s+5)%7:A.w+7*A.U-(s+6)%7);return"Z"in A?(A.H+=A.Z/100|0,A.M+=A.Z%100,o(A)):e(A)}}function Lt(t,e,n,r){for(var i,o,a=0,s=e.length,A=n.length;a<s;){if(r>=A)return-1;if(37===(i=e.charCodeAt(a++))){if(i=e.charAt(a++),!(o=Mt[i in h?e.charAt(a++):i])||(r=o(t,n,r))<0)return-1}else if(i!=n.charCodeAt(r++))return-1}return r}return _t.x=kt(n,_t),_t.X=kt(s,_t),_t.c=kt(e,_t),Dt.x=kt(n,Dt),Dt.X=kt(s,Dt),Dt.c=kt(e,Dt),{format:function(t){var e=kt(t+="",_t);return e.toString=function(){return t},e},parse:function(t){var e=St(t+="",i);return e.toString=function(){return t},e},utcFormat:function(t){var e=kt(t+="",Dt);return e.toString=function(){return t},e},utcParse:function(t){var e=St(t,o);return e.toString=function(){return t},e}}}var A,l,c,u,g,h={"-":"",_:" ",0:"0"},d=/^\s*\d+/,f=/^%/,p=/[\\^$*+?|[\]().{}]/g;function I(t,e,n){var r=t<0?"-":"",i=(r?-t:t)+"",o=i.length;return r+(o<n?new Array(n-o+1).join(e)+i:i)}function C(t){return t.replace(p,"\\$&")}function m(t){return new RegExp("^(?:"+t.map(C).join("|")+")","i")}function y(t){for(var e={},n=-1,r=t.length;++n<r;)e[t[n].toLowerCase()]=n;return e}function E(t,e,n){var r=d.exec(e.slice(n,n+1));return r?(t.w=+r[0],n+r[0].length):-1}function v(t,e,n){var r=d.exec(e.slice(n,n+1));return r?(t.u=+r[0],n+r[0].length):-1}function B(t,e,n){var r=d.exec(e.slice(n,n+2));return r?(t.U=+r[0],n+r[0].length):-1}function b(t,e,n){var r=d.exec(e.slice(n,n+2));return r?(t.V=+r[0],n+r[0].length):-1}function Q(t,e,n){var r=d.exec(e.slice(n,n+2));return r?(t.W=+r[0],n+r[0].length):-1}function w(t,e,n){var r=d.exec(e.slice(n,n+4));return r?(t.y=+r[0],n+r[0].length):-1}function x(t,e,n){var r=d.exec(e.slice(n,n+2));return r?(t.y=+r[0]+(+r[0]>68?1900:2e3),n+r[0].length):-1}function _(t,e,n){var r=/^(Z)|([+-]\d\d)(?::?(\d\d))?/.exec(e.slice(n,n+6));return r?(t.Z=r[1]?0:-(r[2]+(r[3]||"00")),n+r[0].length):-1}function D(t,e,n){var r=d.exec(e.slice(n,n+2));return r?(t.m=r[0]-1,n+r[0].length):-1}function M(t,e,n){var r=d.exec(e.slice(n,n+2));return r?(t.d=+r[0],n+r[0].length):-1}function k(t,e,n){var r=d.exec(e.slice(n,n+3));return r?(t.m=0,t.d=+r[0],n+r[0].length):-1}function S(t,e,n){var r=d.exec(e.slice(n,n+2));return r?(t.H=+r[0],n+r[0].length):-1}function L(t,e,n){var r=d.exec(e.slice(n,n+2));return r?(t.M=+r[0],n+r[0].length):-1}function R(t,e,n){var r=d.exec(e.slice(n,n+2));return r?(t.S=+r[0],n+r[0].length):-1}function N(t,e,n){var r=d.exec(e.slice(n,n+3));return r?(t.L=+r[0],n+r[0].length):-1}function F(t,e,n){var r=d.exec(e.slice(n,n+6));return r?(t.L=Math.floor(r[0]/1e3),n+r[0].length):-1}function T(t,e,n){var r=f.exec(e.slice(n,n+1));return r?n+r[0].length:-1}function U(t,e,n){var r=d.exec(e.slice(n));return r?(t.Q=+r[0],n+r[0].length):-1}function O(t,e,n){var r=d.exec(e.slice(n));return r?(t.Q=1e3*+r[0],n+r[0].length):-1}function G(t,e){return I(t.getDate(),e,2)}function H(t,e){return I(t.getHours(),e,2)}function j(t,e){return I(t.getHours()%12||12,e,2)}function Y(t,e){return I(1+r.timeDay.count(Object(r.timeYear)(t),t),e,3)}function P(t,e){return I(t.getMilliseconds(),e,3)}function K(t,e){return P(t,e)+"000"}function z(t,e){return I(t.getMonth()+1,e,2)}function J(t,e){return I(t.getMinutes(),e,2)}function q(t,e){return I(t.getSeconds(),e,2)}function V(t){var e=t.getDay();return 0===e?7:e}function W(t,e){return I(r.timeSunday.count(Object(r.timeYear)(t),t),e,2)}function X(t,e){var n=t.getDay();return t=n>=4||0===n?Object(r.timeThursday)(t):r.timeThursday.ceil(t),I(r.timeThursday.count(Object(r.timeYear)(t),t)+(4===Object(r.timeYear)(t).getDay()),e,2)}function Z(t){return t.getDay()}function $(t,e){return I(r.timeMonday.count(Object(r.timeYear)(t),t),e,2)}function tt(t,e){return I(t.getFullYear()%100,e,2)}function et(t,e){return I(t.getFullYear()%1e4,e,4)}function nt(t){var e=t.getTimezoneOffset();return(e>0?"-":(e*=-1,"+"))+I(e/60|0,"0",2)+I(e%60,"0",2)}function rt(t,e){return I(t.getUTCDate(),e,2)}function it(t,e){return I(t.getUTCHours(),e,2)}function ot(t,e){return I(t.getUTCHours()%12||12,e,2)}function at(t,e){return I(1+r.utcDay.count(Object(r.utcYear)(t),t),e,3)}function st(t,e){return I(t.getUTCMilliseconds(),e,3)}function At(t,e){return st(t,e)+"000"}function lt(t,e){return I(t.getUTCMonth()+1,e,2)}function ct(t,e){return I(t.getUTCMinutes(),e,2)}function ut(t,e){return I(t.getUTCSeconds(),e,2)}function gt(t){var e=t.getUTCDay();return 0===e?7:e}function ht(t,e){return I(r.utcSunday.count(Object(r.utcYear)(t),t),e,2)}function dt(t,e){var n=t.getUTCDay();return t=n>=4||0===n?Object(r.utcThursday)(t):r.utcThursday.ceil(t),I(r.utcThursday.count(Object(r.utcYear)(t),t)+(4===Object(r.utcYear)(t).getUTCDay()),e,2)}function ft(t){return t.getUTCDay()}function pt(t,e){return I(r.utcMonday.count(Object(r.utcYear)(t),t),e,2)}function It(t,e){return I(t.getUTCFullYear()%100,e,2)}function Ct(t,e){return I(t.getUTCFullYear()%1e4,e,4)}function mt(){return"+0000"}function yt(){return"%"}function Et(t){return+t}function vt(t){return Math.floor(+t/1e3)}function Bt(t){return A=s(t),l=A.format,c=A.parse,u=A.utcFormat,g=A.utcParse,A}Bt({dateTime:"%x, %X",date:"%-m/%-d/%Y",time:"%-I:%M:%S %p",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]});var bt=Date.prototype.toISOString?function(t){return t.toISOString()}:u("%Y-%m-%dT%H:%M:%S.%LZ"),Qt=+new Date("2000-01-01T00:00:00.000Z")?function(t){var e=new Date(t);return isNaN(e)?null:e}:g("%Y-%m-%dT%H:%M:%S.%LZ");n.d(e,"d",function(){return Bt}),n.d(e,"c",function(){return l}),n.d(e,"f",function(){return c}),n.d(e,"g",function(){return u}),n.d(e,"h",function(){return g}),n.d(e,"e",function(){return s}),n.d(e,"a",function(){return bt}),n.d(e,"b",function(){return Qt})},function(t,e,n){var r=n(15);t.exports=function(t){if(!r(t))throw TypeError(t+" is not an object!");return t}},function(t,e,n){"use strict";n.r(e);var r={value:function(){}};function i(){for(var t,e=0,n=arguments.length,r={};e<n;++e){if(!(t=arguments[e]+"")||t in r)throw new Error("illegal type: "+t);r[t]=[]}return new o(r)}function o(t){this._=t}function a(t,e){return t.trim().split(/^|\s+/).map(function(t){var n="",r=t.indexOf(".");if(r>=0&&(n=t.slice(r+1),t=t.slice(0,r)),t&&!e.hasOwnProperty(t))throw new Error("unknown type: "+t);return{type:t,name:n}})}function s(t,e){for(var n,r=0,i=t.length;r<i;++r)if((n=t[r]).name===e)return n.value}function A(t,e,n){for(var i=0,o=t.length;i<o;++i)if(t[i].name===e){t[i]=r,t=t.slice(0,i).concat(t.slice(i+1));break}return null!=n&&t.push({name:e,value:n}),t}o.prototype=i.prototype={constructor:o,on:function(t,e){var n,r=this._,i=a(t+"",r),o=-1,l=i.length;if(!(arguments.length<2)){if(null!=e&&"function"!=typeof e)throw new Error("invalid callback: "+e);for(;++o<l;)if(n=(t=i[o]).type)r[n]=A(r[n],t.name,e);else if(null==e)for(n in r)r[n]=A(r[n],t.name,null);return this}for(;++o<l;)if((n=(t=i[o]).type)&&(n=s(r[n],t.name)))return n},copy:function(){var t={},e=this._;for(var n in e)t[n]=e[n].slice();return new o(t)},call:function(t,e){if((n=arguments.length-2)>0)for(var n,r,i=new Array(n),o=0;o<n;++o)i[o]=arguments[o+2];if(!this._.hasOwnProperty(t))throw new Error("unknown type: "+t);for(o=0,n=(r=this._[t]).length;o<n;++o)r[o].value.apply(e,i)},apply:function(t,e,n){if(!this._.hasOwnProperty(t))throw new Error("unknown type: "+t);for(var r=this._[t],i=0,o=r.length;i<o;++i)r[i].value.apply(e,n)}};var l=i;n.d(e,"dispatch",function(){return l})},function(t,e){var n=t.exports="undefined"!=typeof window&&window.Math==Math?window:"undefined"!=typeof self&&self.Math==Math?self:Function("return this")();"number"==typeof __g&&(__g=n)},function(t,e,n){"use strict";var r,i,o=0,a=0,s=0,A=1e3,l=0,c=0,u=0,g="object"==typeof performance&&performance.now?performance:Date,h="object"==typeof window&&window.requestAnimationFrame?window.requestAnimationFrame.bind(window):function(t){setTimeout(t,17)};function d(){return c||(h(f),c=g.now()+u)}function f(){c=0}function p(){this._call=this._time=this._next=null}function I(t,e,n){var r=new p;return r.restart(t,e,n),r}function C(){d(),++o;for(var t,e=r;e;)(t=c-e._time)>=0&&e._call.call(null,t),e=e._next;--o}function m(){c=(l=g.now())+u,o=a=0;try{C()}finally{o=0,function(){for(var t,e,n=r,o=1/0;n;)n._call?(o>n._time&&(o=n._time),t=n,n=n._next):(e=n._next,n._next=null,n=t?t._next=e:r=e);i=t,E(o)}(),c=0}}function y(){var t=g.now(),e=t-l;e>A&&(u-=e,l=t)}function E(t){o||(a&&(a=clearTimeout(a)),t-c>24?(t<1/0&&(a=setTimeout(m,t-g.now()-u)),s&&(s=clearInterval(s))):(s||(l=g.now(),s=setInterval(y,A)),o=1,h(m)))}p.prototype=I.prototype={constructor:p,restart:function(t,e,n){if("function"!=typeof t)throw new TypeError("callback is not a function");n=(null==n?d():+n)+(null==e?0:+e),this._next||i===this||(i?i._next=this:r=this,i=this),this._call=t,this._time=n,E()},stop:function(){this._call&&(this._call=null,this._time=1/0,E())}};var v=function(t,e,n){var r=new p;return e=null==e?0:+e,r.restart(function(n){r.stop(),t(n+e)},e,n),r},B=function(t,e,n){var r=new p,i=e;return null==e?(r.restart(t,e,n),r):(e=+e,n=null==n?d():+n,r.restart(function o(a){a+=i,r.restart(o,i+=e,n),t(a)},e,n),r)};n.d(e,"b",function(){return d}),n.d(e,"d",function(){return I}),n.d(e,"e",function(){return C}),n.d(e,"c",function(){return v}),n.d(e,"a",function(){return B})},function(t,e){t.exports=function(t){return"object"==typeof t?null!==t:"function"==typeof t}},function(t,e,n){t.exports=!n(17)(function(){return 7!=Object.defineProperty({},"a",{get:function(){return 7}}).a})},function(t,e){t.exports=function(t){try{return!!t()}catch(t){return!0}}},function(t,e,n){var r=n(108),i="object"==typeof self&&self&&self.Object===Object&&self,o=r||i||Function("return this")();t.exports=o},function(t,e,n){var r=n(11),i=n(95),o=n(70),a=Object.defineProperty;e.f=n(16)?Object.defineProperty:function(t,e,n){if(r(t),e=o(e,!0),r(n),i)try{return a(t,e,n)}catch(t){}if("get"in n||"set"in n)throw TypeError("Accessors not supported!");return"value"in n&&(t[e]=n.value),t}},function(t,e,n){"use strict";var r=n(0),i=n(12),o=n(14),a=Object(i.dispatch)("start","end","cancel","interrupt"),s=[],A=0,l=1,c=2,u=3,g=4,h=5,d=6,f=function(t,e,n,r,i,f){var p=t.__transition;if(p){if(n in p)return}else t.__transition={};!function(t,e,n){var r,i=t.__transition;function a(h){var f,p,I,C;if(n.state!==l)return A();for(f in i)if((C=i[f]).name===n.name){if(C.state===u)return Object(o.c)(a);C.state===g?(C.state=d,C.timer.stop(),C.on.call("interrupt",t,t.__data__,C.index,C.group),delete i[f]):+f<e&&(C.state=d,C.timer.stop(),C.on.call("cancel",t,t.__data__,C.index,C.group),delete i[f])}if(Object(o.c)(function(){n.state===u&&(n.state=g,n.timer.restart(s,n.delay,n.time),s(h))}),n.state=c,n.on.call("start",t,t.__data__,n.index,n.group),n.state===c){for(n.state=u,r=new Array(I=n.tween.length),f=0,p=-1;f<I;++f)(C=n.tween[f].value.call(t,t.__data__,n.index,n.group))&&(r[++p]=C);r.length=p+1}}function s(e){for(var i=e<n.duration?n.ease.call(null,e/n.duration):(n.timer.restart(A),n.state=h,1),o=-1,a=r.length;++o<a;)r[o].call(t,i);n.state===h&&(n.on.call("end",t,t.__data__,n.index,n.group),A())}function A(){for(var r in n.state=d,n.timer.stop(),delete i[e],i)return;delete t.__transition}i[e]=n,n.timer=Object(o.d)(function(t){n.state=l,n.timer.restart(a,n.delay,n.time),n.delay<=t&&a(t-n.delay)},0,n.time)}(t,n,{name:e,index:r,group:i,on:a,tween:s,time:f.time,delay:f.delay,duration:f.duration,ease:f.ease,timer:null,state:A})};function p(t,e){var n=C(t,e);if(n.state>A)throw new Error("too late; already scheduled");return n}function I(t,e){var n=C(t,e);if(n.state>u)throw new Error("too late; already running");return n}function C(t,e){var n=t.__transition;if(!n||!(n=n[e]))throw new Error("transition not found");return n}var m=function(t,e){var n,r,i,o=t.__transition,a=!0;if(o){for(i in e=null==e?null:e+"",o)(n=o[i]).name===e?(r=n.state>c&&n.state<h,n.state=d,n.timer.stop(),n.on.call(r?"interrupt":"cancel",t,t.__data__,n.index,n.group),delete o[i]):a=!1;a&&delete t.__transition}},y=n(4);function E(t,e){var n,r;return function(){var i=I(this,t),o=i.tween;if(o!==n)for(var a=0,s=(r=n=o).length;a<s;++a)if(r[a].name===e){(r=r.slice()).splice(a,1);break}i.tween=r}}function v(t,e,n){var r,i;if("function"!=typeof n)throw new Error;return function(){var o=I(this,t),a=o.tween;if(a!==r){i=(r=a).slice();for(var s={name:e,value:n},A=0,l=i.length;A<l;++A)if(i[A].name===e){i[A]=s;break}A===l&&i.push(s)}o.tween=i}}function B(t,e,n){var r=t._id;return t.each(function(){var t=I(this,r);(t.value||(t.value={}))[e]=n.apply(this,arguments)}),function(t){return C(t,r).value[e]}}var b=n(3),Q=function(t,e){var n;return("number"==typeof e?y.o:e instanceof b.a?y.q:(n=Object(b.a)(e))?(e=n,y.q):y.u)(t,e)};function w(t,e){var n,r;function i(){var i=e.apply(this,arguments);return i!==r&&(n=(r=i)&&function(t,e){return function(n){this.setAttributeNS(t.space,t.local,e(n))}}(t,i)),n}return i._value=e,i}function x(t,e){var n,r;function i(){var i=e.apply(this,arguments);return i!==r&&(n=(r=i)&&function(t,e){return function(n){this.setAttribute(t,e(n))}}(t,i)),n}return i._value=e,i}function _(t,e){return function(){p(this,t).delay=+e.apply(this,arguments)}}function D(t,e){return e=+e,function(){p(this,t).delay=e}}function M(t,e){return function(){I(this,t).duration=+e.apply(this,arguments)}}function k(t,e){return e=+e,function(){I(this,t).duration=e}}function S(t,e){if("function"!=typeof e)throw new Error;return function(){I(this,t).ease=e}}function L(t,e,n){var r,i,o=(e+"").trim().split(/^|\s+/).every(function(t){var e=t.indexOf(".");return e>=0&&(t=t.slice(0,e)),!t||"start"===t})?p:I;return function(){var a=o(this,t),s=a.on;s!==r&&(i=(r=s).copy()).on(e,n),a.on=i}}var R=r.selection.prototype.constructor;function N(t){return function(){this.style.removeProperty(t)}}function F(t,e,n){var r,i;function o(){var o=e.apply(this,arguments);return o!==i&&(r=(i=o)&&function(t,e,n){return function(r){this.style.setProperty(t,e(r),n)}}(t,o,n)),r}return o._value=e,o}var T=0;function U(t,e,n,r){this._groups=t,this._parents=e,this._name=n,this._id=r}function O(t){return Object(r.selection)().transition(t)}function G(){return++T}var H=r.selection.prototype;U.prototype=O.prototype={constructor:U,select:function(t){var e=this._name,n=this._id;"function"!=typeof t&&(t=Object(r.selector)(t));for(var i=this._groups,o=i.length,a=new Array(o),s=0;s<o;++s)for(var A,l,c=i[s],u=c.length,g=a[s]=new Array(u),h=0;h<u;++h)(A=c[h])&&(l=t.call(A,A.__data__,h,c))&&("__data__"in A&&(l.__data__=A.__data__),g[h]=l,f(g[h],e,n,h,g,C(A,n)));return new U(a,this._parents,e,n)},selectAll:function(t){var e=this._name,n=this._id;"function"!=typeof t&&(t=Object(r.selectorAll)(t));for(var i=this._groups,o=i.length,a=[],s=[],A=0;A<o;++A)for(var l,c=i[A],u=c.length,g=0;g<u;++g)if(l=c[g]){for(var h,d=t.call(l,l.__data__,g,c),p=C(l,n),I=0,m=d.length;I<m;++I)(h=d[I])&&f(h,e,n,I,d,p);a.push(d),s.push(l)}return new U(a,s,e,n)},filter:function(t){"function"!=typeof t&&(t=Object(r.matcher)(t));for(var e=this._groups,n=e.length,i=new Array(n),o=0;o<n;++o)for(var a,s=e[o],A=s.length,l=i[o]=[],c=0;c<A;++c)(a=s[c])&&t.call(a,a.__data__,c,s)&&l.push(a);return new U(i,this._parents,this._name,this._id)},merge:function(t){if(t._id!==this._id)throw new Error;for(var e=this._groups,n=t._groups,r=e.length,i=n.length,o=Math.min(r,i),a=new Array(r),s=0;s<o;++s)for(var A,l=e[s],c=n[s],u=l.length,g=a[s]=new Array(u),h=0;h<u;++h)(A=l[h]||c[h])&&(g[h]=A);for(;s<r;++s)a[s]=e[s];return new U(a,this._parents,this._name,this._id)},selection:function(){return new R(this._groups,this._parents)},transition:function(){for(var t=this._name,e=this._id,n=G(),r=this._groups,i=r.length,o=0;o<i;++o)for(var a,s=r[o],A=s.length,l=0;l<A;++l)if(a=s[l]){var c=C(a,e);f(a,t,n,l,s,{time:c.time+c.delay+c.duration,delay:0,duration:c.duration,ease:c.ease})}return new U(r,this._parents,t,n)},call:H.call,nodes:H.nodes,node:H.node,size:H.size,empty:H.empty,each:H.each,on:function(t,e){var n=this._id;return arguments.length<2?C(this.node(),n).on.on(t):this.each(L(n,t,e))},attr:function(t,e){var n=Object(r.namespace)(t),i="transform"===n?y.w:Q;return this.attrTween(t,"function"==typeof e?(n.local?function(t,e,n){var r,i,o;return function(){var a,s,A=n(this);if(null!=A)return(a=this.getAttributeNS(t.space,t.local))===(s=A+"")?null:a===r&&s===i?o:(i=s,o=e(r=a,A));this.removeAttributeNS(t.space,t.local)}}:function(t,e,n){var r,i,o;return function(){var a,s,A=n(this);if(null!=A)return(a=this.getAttribute(t))===(s=A+"")?null:a===r&&s===i?o:(i=s,o=e(r=a,A));this.removeAttribute(t)}})(n,i,B(this,"attr."+t,e)):null==e?(n.local?function(t){return function(){this.removeAttributeNS(t.space,t.local)}}:function(t){return function(){this.removeAttribute(t)}})(n):(n.local?function(t,e,n){var r,i,o=n+"";return function(){var a=this.getAttributeNS(t.space,t.local);return a===o?null:a===r?i:i=e(r=a,n)}}:function(t,e,n){var r,i,o=n+"";return function(){var a=this.getAttribute(t);return a===o?null:a===r?i:i=e(r=a,n)}})(n,i,e))},attrTween:function(t,e){var n="attr."+t;if(arguments.length<2)return(n=this.tween(n))&&n._value;if(null==e)return this.tween(n,null);if("function"!=typeof e)throw new Error;var i=Object(r.namespace)(t);return this.tween(n,(i.local?w:x)(i,e))},style:function(t,e,n){var i="transform"==(t+="")?y.v:Q;return null==e?this.styleTween(t,function(t,e){var n,i,o;return function(){var a=Object(r.style)(this,t),s=(this.style.removeProperty(t),Object(r.style)(this,t));return a===s?null:a===n&&s===i?o:o=e(n=a,i=s)}}(t,i)).on("end.style."+t,N(t)):"function"==typeof e?this.styleTween(t,function(t,e,n){var i,o,a;return function(){var s=Object(r.style)(this,t),A=n(this),l=A+"";return null==A&&(this.style.removeProperty(t),l=A=Object(r.style)(this,t)),s===l?null:s===i&&l===o?a:(o=l,a=e(i=s,A))}}(t,i,B(this,"style."+t,e))).each(function(t,e){var n,r,i,o,a="style."+e,s="end."+a;return function(){var A=I(this,t),l=A.on,c=null==A.value[a]?o||(o=N(e)):void 0;l===n&&i===c||(r=(n=l).copy()).on(s,i=c),A.on=r}}(this._id,t)):this.styleTween(t,function(t,e,n){var i,o,a=n+"";return function(){var s=Object(r.style)(this,t);return s===a?null:s===i?o:o=e(i=s,n)}}(t,i,e),n).on("end.style."+t,null)},styleTween:function(t,e,n){var r="style."+(t+="");if(arguments.length<2)return(r=this.tween(r))&&r._value;if(null==e)return this.tween(r,null);if("function"!=typeof e)throw new Error;return this.tween(r,F(t,e,null==n?"":n))},text:function(t){return this.tween("text","function"==typeof t?function(t){return function(){var e=t(this);this.textContent=null==e?"":e}}(B(this,"text",t)):function(t){return function(){this.textContent=t}}(null==t?"":t+""))},remove:function(){return this.on("end.remove",(t=this._id,function(){var e=this.parentNode;for(var n in this.__transition)if(+n!==t)return;e&&e.removeChild(this)}));var t},tween:function(t,e){var n=this._id;if(t+="",arguments.length<2){for(var r,i=C(this.node(),n).tween,o=0,a=i.length;o<a;++o)if((r=i[o]).name===t)return r.value;return null}return this.each((null==e?E:v)(n,t,e))},delay:function(t){var e=this._id;return arguments.length?this.each(("function"==typeof t?_:D)(e,t)):C(this.node(),e).delay},duration:function(t){var e=this._id;return arguments.length?this.each(("function"==typeof t?M:k)(e,t)):C(this.node(),e).duration},ease:function(t){var e=this._id;return arguments.length?this.each(S(e,t)):C(this.node(),e).ease},end:function(){var t,e,n=this,r=n._id,i=n.size();return new Promise(function(o,a){var s={value:a},A={value:function(){0==--i&&o()}};n.each(function(){var n=I(this,r),i=n.on;i!==t&&((e=(t=i).copy())._.cancel.push(s),e._.interrupt.push(s),e._.end.push(A)),n.on=e})})}};var j={time:null,delay:0,duration:250,ease:n(61).o};function Y(t,e){for(var n;!(n=t.__transition)||!(n=n[e]);)if(!(t=t.parentNode))return j.time=Object(o.b)(),j;return n}r.selection.prototype.interrupt=function(t){return this.each(function(){m(this,t)})},r.selection.prototype.transition=function(t){var e,n;t instanceof U?(e=t._id,t=t._name):(e=G(),(n=j).time=Object(o.b)(),t=null==t?null:t+"");for(var r=this._groups,i=r.length,a=0;a<i;++a)for(var s,A=r[a],l=A.length,c=0;c<l;++c)(s=A[c])&&f(s,t,e,c,A,n||Y(s,e));return new U(r,this._parents,t,e)};var P=[null],K=function(t,e){var n,r,i=t.__transition;if(i)for(r in e=null==e?null:e+"",i)if((n=i[r]).state>l&&n.name===e)return new U([[t]],P,e,+r);return null};n.d(e,"c",function(){return O}),n.d(e,"a",function(){return K}),n.d(e,"b",function(){return m})},function(t,e){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(t){"object"==typeof window&&(n=window)}t.exports=n},function(t,e,n){var r=n(19),i=n(51);t.exports=n(16)?function(t,e,n){return r.f(t,e,i(1,n))}:function(t,e,n){return t[e]=n,t}},function(t,e,n){var r=n(13),i=n(22),o=n(27),a=n(36)("src"),s=n(161),A=(""+s).split("toString");n(35).inspectSource=function(t){return s.call(t)},(t.exports=function(t,e,n,s){var l="function"==typeof n;l&&(o(n,"name")||i(n,"name",e)),t[e]!==n&&(l&&(o(n,a)||i(n,a,t[e]?""+t[e]:A.join(String(e)))),t===r?t[e]=n:s?t[e]?t[e]=n:i(t,e,n):(delete t[e],i(t,e,n)))})(Function.prototype,"toString",function(){return"function"==typeof this&&this[a]||s.call(this)})},function(t,e,n){"use strict";var r=n(12),i=n(0);function o(){i.event.stopImmediatePropagation()}var a=function(){i.event.preventDefault(),i.event.stopImmediatePropagation()},s=function(t){var e=t.document.documentElement,n=Object(i.select)(t).on("dragstart.drag",a,!0);"onselectstart"in e?n.on("selectstart.drag",a,!0):(e.__noselect=e.style.MozUserSelect,e.style.MozUserSelect="none")};function A(t,e){var n=t.document.documentElement,r=Object(i.select)(t).on("dragstart.drag",null);e&&(r.on("click.drag",a,!0),setTimeout(function(){r.on("click.drag",null)},0)),"onselectstart"in n?r.on("selectstart.drag",null):(n.style.MozUserSelect=n.__noselect,delete n.__noselect)}var l=function(t){return function(){return t}};function c(t,e,n,r,i,o,a,s,A,l){this.target=t,this.type=e,this.subject=n,this.identifier=r,this.active=i,this.x=o,this.y=a,this.dx=s,this.dy=A,this._=l}function u(){return!i.event.button}function g(){return this.parentNode}function h(t){return null==t?{x:i.event.x,y:i.event.y}:t}function d(){return"ontouchstart"in this}c.prototype.on=function(){var t=this._.on.apply(this._,arguments);return t===this._?this:t};var f=function(){var t,e,n,f,p=u,I=g,C=h,m=d,y={},E=Object(r.dispatch)("start","drag","end"),v=0,B=0;function b(t){t.on("mousedown.drag",Q).filter(m).on("touchstart.drag",_).on("touchmove.drag",D).on("touchend.drag touchcancel.drag",M).style("touch-action","none").style("-webkit-tap-highlight-color","rgba(0,0,0,0)")}function Q(){if(!f&&p.apply(this,arguments)){var r=k("mouse",I.apply(this,arguments),i.mouse,this,arguments);r&&(Object(i.select)(i.event.view).on("mousemove.drag",w,!0).on("mouseup.drag",x,!0),s(i.event.view),o(),n=!1,t=i.event.clientX,e=i.event.clientY,r("start"))}}function w(){if(a(),!n){var r=i.event.clientX-t,o=i.event.clientY-e;n=r*r+o*o>B}y.mouse("drag")}function x(){Object(i.select)(i.event.view).on("mousemove.drag mouseup.drag",null),A(i.event.view,n),a(),y.mouse("end")}function _(){if(p.apply(this,arguments)){var t,e,n=i.event.changedTouches,r=I.apply(this,arguments),a=n.length;for(t=0;t<a;++t)(e=k(n[t].identifier,r,i.touch,this,arguments))&&(o(),e("start"))}}function D(){var t,e,n=i.event.changedTouches,r=n.length;for(t=0;t<r;++t)(e=y[n[t].identifier])&&(a(),e("drag"))}function M(){var t,e,n=i.event.changedTouches,r=n.length;for(f&&clearTimeout(f),f=setTimeout(function(){f=null},500),t=0;t<r;++t)(e=y[n[t].identifier])&&(o(),e("end"))}function k(t,e,n,r,o){var a,s,A,l=n(e,t),u=E.copy();if(Object(i.customEvent)(new c(b,"beforestart",a,t,v,l[0],l[1],0,0,u),function(){return null!=(i.event.subject=a=C.apply(r,o))&&(s=a.x-l[0]||0,A=a.y-l[1]||0,!0)}))return function g(h){var d,f=l;switch(h){case"start":y[t]=g,d=v++;break;case"end":delete y[t],--v;case"drag":l=n(e,t),d=v}Object(i.customEvent)(new c(b,h,a,t,d,l[0]+s,l[1]+A,l[0]-f[0],l[1]-f[1],u),u.apply,u,[h,r,o])}}return b.filter=function(t){return arguments.length?(p="function"==typeof t?t:l(!!t),b):p},b.container=function(t){return arguments.length?(I="function"==typeof t?t:l(t),b):I},b.subject=function(t){return arguments.length?(C="function"==typeof t?t:l(t),b):C},b.touchable=function(t){return arguments.length?(m="function"==typeof t?t:l(!!t),b):m},b.on=function(){var t=E.on.apply(E,arguments);return t===E?b:t},b.clickDistance=function(t){return arguments.length?(B=(t=+t)*t,b):Math.sqrt(B)},b};n.d(e,"a",function(){return f}),n.d(e,"b",function(){return s}),n.d(e,"c",function(){return A})},function(t,e){t.exports=__WEBPACK_EXTERNAL_MODULE__25__},function(t,e){t.exports=__WEBPACK_EXTERNAL_MODULE__26__},function(t,e){var n={}.hasOwnProperty;t.exports=function(t,e){return n.call(t,e)}},function(t,e){t.exports=__WEBPACK_EXTERNAL_MODULE__28__},function(t,e,n){for(var r=n(162),i=n(53),o=n(23),a=n(13),s=n(22),A=n(38),l=n(7),c=l("iterator"),u=l("toStringTag"),g=A.Array,h={CSSRuleList:!0,CSSStyleDeclaration:!1,CSSValueList:!1,ClientRectList:!1,DOMRectList:!1,DOMStringList:!1,DOMTokenList:!0,DataTransferItemList:!1,FileList:!1,HTMLAllCollection:!1,HTMLCollection:!1,HTMLFormElement:!1,HTMLSelectElement:!1,MediaList:!0,MimeTypeArray:!1,NamedNodeMap:!1,NodeList:!0,PaintRequestList:!1,Plugin:!1,PluginArray:!1,SVGLengthList:!1,SVGNumberList:!1,SVGPathSegList:!1,SVGPointList:!1,SVGStringList:!1,SVGTransformList:!1,SourceBufferList:!1,StyleSheetList:!0,TextTrackCueList:!1,TextTrackList:!1,TouchList:!1},d=i(h),f=0;f<d.length;f++){var p,I=d[f],C=h[I],m=a[I],y=m&&m.prototype;if(y&&(y[c]||s(y,c,g),y[u]||s(y,u,I),A[I]=g,C))for(p in r)y[p]||o(y,p,r[p],!0)}},function(t,e,n){var r=n(164),i=n(46);t.exports=function(t){return r(i(t))}},function(t,e,n){var r=n(227),i=n(230);t.exports=function(t,e){var n=i(t,e);return r(n)?n:void 0}},function(t,e,n){"use strict";var r={},i={},o=34,a=10,s=13;function A(t){return new Function("d","return {"+t.map(function(t,e){return JSON.stringify(t)+": d["+e+"]"}).join(",")+"}")}function l(t){var e=Object.create(null),n=[];return t.forEach(function(t){for(var r in t)r in e||n.push(e[r]=r)}),n}function c(t,e){var n=t+"",r=n.length;return r<e?new Array(e-r+1).join(0)+n:n}function u(t){var e,n=t.getUTCHours(),r=t.getUTCMinutes(),i=t.getUTCSeconds(),o=t.getUTCMilliseconds();return isNaN(t)?"Invalid Date":((e=t.getUTCFullYear())<0?"-"+c(-e,6):e>9999?"+"+c(e,6):c(e,4))+"-"+c(t.getUTCMonth()+1,2)+"-"+c(t.getUTCDate(),2)+(o?"T"+c(n,2)+":"+c(r,2)+":"+c(i,2)+"."+c(o,3)+"Z":i?"T"+c(n,2)+":"+c(r,2)+":"+c(i,2)+"Z":r||n?"T"+c(n,2)+":"+c(r,2)+"Z":"")}var g=function(t){var e=new RegExp('["'+t+"\n\r]"),n=t.charCodeAt(0);function c(t,e){var A,l=[],c=t.length,u=0,g=0,h=c<=0,d=!1;function f(){if(h)return i;if(d)return d=!1,r;var e,A,l=u;if(t.charCodeAt(l)===o){for(;u++<c&&t.charCodeAt(u)!==o||t.charCodeAt(++u)===o;);return(e=u)>=c?h=!0:(A=t.charCodeAt(u++))===a?d=!0:A===s&&(d=!0,t.charCodeAt(u)===a&&++u),t.slice(l+1,e-1).replace(/""/g,'"')}for(;u<c;){if((A=t.charCodeAt(e=u++))===a)d=!0;else if(A===s)d=!0,t.charCodeAt(u)===a&&++u;else if(A!==n)continue;return t.slice(l,e)}return h=!0,t.slice(l,c)}for(t.charCodeAt(c-1)===a&&--c,t.charCodeAt(c-1)===s&&--c;(A=f())!==i;){for(var p=[];A!==r&&A!==i;)p.push(A),A=f();e&&null==(p=e(p,g++))||l.push(p)}return l}function g(e,n){return e.map(function(e){return n.map(function(t){return d(e[t])}).join(t)})}function h(e){return e.map(d).join(t)}function d(t){return null==t?"":t instanceof Date?u(t):e.test(t+="")?'"'+t.replace(/"/g,'""')+'"':t}return{parse:function(t,e){var n,r,i=c(t,function(t,i){if(n)return n(t,i-1);r=t,n=e?function(t,e){var n=A(t);return function(r,i){return e(n(r),i,t)}}(t,e):A(t)});return i.columns=r||[],i},parseRows:c,format:function(e,n){return null==n&&(n=l(e)),[n.map(d).join(t)].concat(g(e,n)).join("\n")},formatBody:function(t,e){return null==e&&(e=l(t)),g(t,e).join("\n")},formatRows:function(t){return t.map(h).join("\n")}}},h=g(","),d=h.parse,f=h.parseRows,p=h.format,I=h.formatBody,C=h.formatRows,m=g("\t"),y=m.parse,E=m.parseRows,v=m.format,B=m.formatBody,b=m.formatRows;function Q(t){for(var e in t){var n,r=t[e].trim();if(r)if("true"===r)r=!0;else if("false"===r)r=!1;else if("NaN"===r)r=NaN;else if(isNaN(n=+r)){if(!/^([-+]\d{2})?\d{4}(-\d{2}(-\d{2})?)?(T\d{2}:\d{2}(:\d{2}(\.\d{3})?)?(Z|[-+]\d{2}:\d{2})?)?$/.test(r))continue;r=new Date(r)}else r=n;else r=null;t[e]=r}return t}n.d(e,"g",function(){return g}),n.d(e,"e",function(){return d}),n.d(e,"f",function(){return f}),n.d(e,"b",function(){return p}),n.d(e,"c",function(){return I}),n.d(e,"d",function(){return C}),n.d(e,"k",function(){return y}),n.d(e,"l",function(){return E}),n.d(e,"h",function(){return v}),n.d(e,"i",function(){return B}),n.d(e,"j",function(){return b}),n.d(e,"a",function(){return Q})},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(25),i=n(26);function o(t){var e=t.substring(1,3),n=t.substring(3,5),r=t.substring(5,7);return{r:parseInt(e,16),g:parseInt(n,16),b:parseInt(r,16)}}e.idealTextColor=function(t){var e=o(t);return 255-(.299*e.r+.587*e.g+.114*e.b)<105?"#000000":"#FFFFFF"},e.hex2rgb=o,e.lastSelectionToLinkageExpression=function(t){if("string"==typeof t)try{t=JSON.parse(t)}catch(t){return t}"Path"===t.column&&(t.column="GetName()");var e="("+t.column+") in ( ";return t.list.forEach(function(t,n,r){e+="string"==typeof t?"'"+t+"'":t,n+1<r.length&&(e+=", ")}),e+=" )"},e.findCssBySelector=function t(e,n){var r;if(!0===n||null==n){var i=[];for(r=0;r<document.styleSheets.length;r++){var o=document.styleSheets[r].rules;if(o){var a=t(e,o);a&&i.push(a)}}return i}for(r=0;r<n.length;r++)if(n[r].selectorText===e)return n[r].style},e.isExpression=function(t){return t.match(/[A-z]/i)};const a={"Row Detail.Show Row Detail Button":{default:!1,type:i.Types.Boolean},"ChangeHighlighting.Enabled":{default:!1,type:i.Types.Boolean},"ChangeHighlighting.Style":{default:"GlobalDefault",type:i.Types.String,schema:{enum:["GlobalDefault","Background","Arrow"]}},"ChangeHighlighting.Minimum Change":{default:0,type:i.Types.Number},"ConditionalFormatting.Simple":{default:"None",type:i.Types.String,schema:{enum:["None","NegRed","NegRedPosGreen"]}},"ConditionalFormatting.Condition1. Condition":{default:"null",type:i.Types.String},"ConditionalFormatting.Condition1.ForeColor":{default:new i.Color("black"),type:i.Types.Color},"ConditionalFormatting.Condition1.BackColor":{default:new i.Color("white"),type:i.Types.Color},"ConditionalFormatting.Condition1.IsBold":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition1.IsItalic":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition2. Condition":{default:"null",type:i.Types.String},"ConditionalFormatting.Condition2.ForeColor":{default:new i.Color("black"),type:i.Types.Color},"ConditionalFormatting.Condition2.BackColor":{default:new i.Color("white"),type:i.Types.Color},"ConditionalFormatting.Condition2.IsBold":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition2.IsItalic":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition3. Condition":{default:"null",type:i.Types.String},"ConditionalFormatting.Condition3.ForeColor":{default:new i.Color("black"),type:i.Types.Color},"ConditionalFormatting.Condition3.BackColor":{default:new i.Color("white"),type:i.Types.Color},"ConditionalFormatting.Condition3.IsBold":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition3.IsItalic":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition4. Condition":{default:"null",type:i.Types.String},"ConditionalFormatting.Condition4.ForeColor":{default:new i.Color("black"),type:i.Types.Color},"ConditionalFormatting.Condition4.BackColor":{default:new i.Color("white"),type:i.Types.Color},"ConditionalFormatting.Condition4.IsBold":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition4.IsItalic":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition5. Condition":{default:"null",type:i.Types.String},"ConditionalFormatting.Condition5.ForeColor":{default:new i.Color("black"),type:i.Types.Color},"ConditionalFormatting.Condition5.BackColor":{default:new i.Color("white"),type:i.Types.Color},"ConditionalFormatting.Condition5.IsBold":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition5.IsItalic":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition6. Condition":{default:"null",type:i.Types.String},"ConditionalFormatting.Condition6.ForeColor":{default:new i.Color("black"),type:i.Types.Color},"ConditionalFormatting.Condition6.BackColor":{default:new i.Color("white"),type:i.Types.Color},"ConditionalFormatting.Condition6.IsBold":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition6.IsItalic":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition7. Condition":{default:"null",type:i.Types.String},"ConditionalFormatting.Condition7.ForeColor":{default:new i.Color("black"),type:i.Types.Color},"ConditionalFormatting.Condition7.BackColor":{default:new i.Color("white"),type:i.Types.Color},"ConditionalFormatting.Condition7.IsBold":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition7.IsItalic":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition8. Condition":{default:"null",type:i.Types.String},"ConditionalFormatting.Condition8.ForeColor":{default:new i.Color("black"),type:i.Types.Color},"ConditionalFormatting.Condition8.BackColor":{default:new i.Color("white"),type:i.Types.Color},"ConditionalFormatting.Condition8.IsBold":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition8.IsItalic":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition9. Condition":{default:"null",type:i.Types.String},"ConditionalFormatting.Condition9.ForeColor":{default:new i.Color("black"),type:i.Types.Color},"ConditionalFormatting.Condition9.BackColor":{default:new i.Color("white"),type:i.Types.Color},"ConditionalFormatting.Condition9.IsBold":{default:!1,type:i.Types.Boolean},"ConditionalFormatting.Condition9.IsItalic":{default:!1,type:i.Types.Boolean},"ColumnHeader.ForeColor":{default:new i.Color("black"),type:i.Types.Color},"ColumnHeader.BackColor":{default:new i.Color("white"),type:i.Types.Color},"Heatmap.Minimum.Value":{default:-1,type:i.Types.Number},"Heatmap.Center.Value":{default:0,type:i.Types.Number},"Heatmap.Maximum.Value":{default:1,type:i.Types.Number},"Heatmap.Minimum.Color":{default:new i.Color("red"),type:i.Types.Color},"Heatmap.Center.Color":{default:new i.Color("white"),type:i.Types.Color},"Heatmap.Maximum.Color":{default:new i.Color("blue"),type:i.Types.Color},"Heatmap. Enable":{default:!1,type:i.Types.Boolean},"General.DisplayStyle":{default:"Quantity",type:i.Types.String,schema:{enum:["Quantity","ProgressBar","Sparkline","DashboardLink"]}},"General.NullRule":{default:"Hide",type:i.Types.String,schema:{enum:["Hide","Show"]}},"General.ColumnWidthPixels":{default:59,type:i.Types.Number},"Number.FormatString":{default:"",type:i.Types.String},"ProgressBar.StartColor":{default:new i.Color("blue"),type:i.Types.Color},"ProgressBar.EndColor":{default:new i.Color("yellow"),type:i.Types.Color},"ProgressBar.ShowValue":{default:!0,type:i.Types.Boolean},"ProgressBar.Minimum":{default:0,type:i.Types.Number},"ProgressBar.Maximum":{default:1,type:i.Types.Number}},s=Object.keys(a).map(t=>[t,a[t].default]).reduce((t,e)=>(t[e[0]]=e[1],t),{});e.getFormattingOption=function(t,e){return t&&e in t?t[e]:a[e].default},e.getFormattingWithDefaults=function(t){return Object.assign({},s,t)},e.stripDefaultsFromFormatting=function(t){const e=Object.keys(t),n={};for(let r=0;r<e.length;r++){const i=e[r];t[i]!==a[i].default&&(n[i]=t[i])}return n},e.getFormattingMetadata=function(){return Object.keys(a).map(t=>[t,{default:a[t].default,type:a[t].type,schema:a[t].schema,prettyName:t}])},e.serializeFormatting=function(t){const e=Object.keys(t),n={};for(const r of e){const e=Object.keys(t[r]),o={};for(const n of e){const e=i.Converters.serialize(t[r][n],a[n].type);o[n]=e&&e.value}n[r]=o}return JSON.stringify(n)},e.parseFormatting=function(t){const e=JSON.parse(t),n={},r=Object.keys(e);for(const t of r){const r=e[t],o={},s=Object.keys(r);for(const e of s)if(e in a)try{o[e]=i.Converters.deserialize({typeName:a[e].type.serializableName,value:r[e]})}catch(n){console.warn(`Failed to deserialize formatting option ${t}[${e}], skipping`),console.warn(n),console.warn("Original value",r[e]);continue}n[t]=o}return n},e.tableToResultTable=function(t,e={}){let n=0,i=r.TableHelper.toMQLResultTable(t);for(let t=0;t<i.Columns.length;t++){const n=i.Columns[t];n.FormattingOptions=e[n.Name]||{}}for(let t=0;t<i.Result.Rows.length;t++)i.Result.Rows[t].JavascriptDomIdentifier=""+n++;return i}},function(t,e,n){var r=n(47),i=Math.min;t.exports=function(t){return t>0?i(r(t),9007199254740991):0}},function(t,e){var n=t.exports={version:"2.6.9"};"number"==typeof __e&&(__e=n)},function(t,e){var n=0,r=Math.random();t.exports=function(t){return"Symbol(".concat(void 0===t?"":t,")_",(++n+r).toString(36))}},function(t,e,n){var r=n(13),i=n(35),o=n(22),a=n(23),s=n(52),A=function(t,e,n){var l,c,u,g,h=t&A.F,d=t&A.G,f=t&A.S,p=t&A.P,I=t&A.B,C=d?r:f?r[e]||(r[e]={}):(r[e]||{}).prototype,m=d?i:i[e]||(i[e]={}),y=m.prototype||(m.prototype={});for(l in d&&(n=e),n)u=((c=!h&&C&&void 0!==C[l])?C:n)[l],g=I&&c?s(u,r):p&&"function"==typeof u?s(Function.call,u):u,C&&a(C,l,u,t&A.U),m[l]!=u&&o(m,l,g),p&&y[l]!=u&&(y[l]=u)};r.core=i,A.F=1,A.G=2,A.S=4,A.P=8,A.B=16,A.W=32,A.U=64,A.R=128,t.exports=A},function(t,e){t.exports={}},function(t,e,n){var r=n(81),i=n(192),o=n(193),a=r?r.toStringTag:void 0;t.exports=function(t){return null==t?void 0===t?"[object Undefined]":"[object Null]":a&&a in Object(t)?i(t):o(t)}},function(t,e){t.exports=function(t){return null!=t&&"object"==typeof t}},function(t,e){t.exports=__WEBPACK_EXTERNAL_MODULE__41__},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0});const r=n(62);let i=new WeakMap;function o(t,e,n,r){let o=i.get(t);if(o&&o.version===e)return o.sparks.get(n+"||"+r)}e.GetSparklineFromCache=o,e.SparklineRenderer=function(e,n,a,s,A,{showAxes:l}){!function(t,e){let n=i.get(t);n&&n.version!==e&&(n.sparks=new Map)}(n.grid,n.version);const c=e||t("div");if(!l){const t=o(n.grid,n.version,a,s);if(t)return void c.html(t)}if(c.empty(),null==A)return c;var u=l?50:0,g=l?40:5,h=r.time.scale().range([0,c.width()-u]),d=r.scale.linear().range([c.height()-g,0]);let f=[];if(Array.isArray(A))for(let t=0;t<A.length;++t)f.push({date:A[t][0],value:A[t][1]});else{let t=Object.keys(A);for(let e=0;e<A[t[0]].length;++e)f.push({date:A[t[0]][e],value:A[t[1]][e]})}var p=r.svg.line().x(function(t){return h(t.date)}).y(function(t){return d(t.value)}),I=r.select(c[0]).append("svg").attr("style","width:100%;height:"+c.height()+"px; font-size:10px;font-weight:100;").append("g"),C=0;if(h.domain(r.extent(f,function(t){return t.date})),d.domain(r.extent(f,function(t){return t.value})),!0===l){0===t(c).find("#d3-css").length&&t(c).append(t('<style type="text/css" id="d3-css">').append(".axis path, .axis line {fill:none; stroke:black; shape-rendering:crispEdges}"));var m=r.svg.axis().scale(h).orient("bottom").tickValues(h.domain()).tickFormat(r.time.format("%b '%y")),y=r.svg.axis().scale(d).orient("left").tickValues(d.domain()).tickSize(6,0);I.append("g").attr("class","x axis").attr("transform","translate(30,"+(c.height()-35)+")").call(m),I.append("g").attr("class","y axis").attr("transform","translate(30,5)").call(y),C=31}return I.append("path").datum(f).attr("style","fill:none;stroke:steelblue;stroke-width:1.5px;").attr("transform","translate("+C+", 0)").attr("d",p),l||function(t,e,n,r,o){let a=i.get(t);a||(a={version:e,sparks:new Map},i.set(t,a)),a.version=e,a.sparks.set(n+"||"+r,o)}(n.grid,n.version,a,s,c.html()),c}}).call(this,n(5))},function(t,e){t.exports=function(t){return t.webpackPolyfill||(t.deprecate=function(){},t.paths=[],t.children||(t.children=[]),Object.defineProperty(t,"loaded",{enumerable:!0,get:function(){return t.l}}),Object.defineProperty(t,"id",{enumerable:!0,get:function(){return t.i}}),t.webpackPolyfill=1),t}},function(t,e,n){"use strict";var r=n(11),i=n(45),o=n(34),a=n(47),s=n(65),A=n(66),l=Math.max,c=Math.min,u=Math.floor,g=/\$([$&`']|\d\d?|<[^>]*>)/g,h=/\$([$&`']|\d\d?)/g;n(67)("replace",2,function(t,e,n,d){return[function(r,i){var o=t(this),a=null==r?void 0:r[e];return void 0!==a?a.call(r,o,i):n.call(String(o),r,i)},function(t,e){var i=d(n,t,this,e);if(i.done)return i.value;var u=r(t),g=String(this),h="function"==typeof e;h||(e=String(e));var p=u.global;if(p){var I=u.unicode;u.lastIndex=0}for(var C=[];;){var m=A(u,g);if(null===m)break;if(C.push(m),!p)break;""===String(m[0])&&(u.lastIndex=s(g,o(u.lastIndex),I))}for(var y,E="",v=0,B=0;B<C.length;B++){m=C[B];for(var b=String(m[0]),Q=l(c(a(m.index),g.length),0),w=[],x=1;x<m.length;x++)w.push(void 0===(y=m[x])?y:String(y));var _=m.groups;if(h){var D=[b].concat(w,Q,g);void 0!==_&&D.push(_);var M=String(e.apply(void 0,D))}else M=f(b,g,Q,w,_,e);Q>=v&&(E+=g.slice(v,Q)+M,v=Q+b.length)}return E+g.slice(v)}];function f(t,e,r,o,a,s){var A=r+t.length,l=o.length,c=h;return void 0!==a&&(a=i(a),c=g),n.call(s,c,function(n,i){var s;switch(i.charAt(0)){case"$":return"$";case"&":return t;case"`":return e.slice(0,r);case"'":return e.slice(A);case"<":s=a[i.slice(1,-1)];break;default:var c=+i;if(0===c)return n;if(c>l){var g=u(c/10);return 0===g?n:g<=l?void 0===o[g-1]?i.charAt(1):o[g-1]+i.charAt(1):n}s=o[c-1]}return void 0===s?"":s})}})},function(t,e,n){var r=n(46);t.exports=function(t){return Object(r(t))}},function(t,e){t.exports=function(t){if(null==t)throw TypeError("Can't call method on  "+t);return t}},function(t,e){var n=Math.ceil,r=Math.floor;t.exports=function(t){return isNaN(t=+t)?0:(t>0?r:n)(t)}},function(t,e){var n={}.toString;t.exports=function(t){return n.call(t).slice(8,-1)}},function(t,e,n){var r=n(35),i=n(13),o=i["__core-js_shared__"]||(i["__core-js_shared__"]={});(t.exports=function(t,e){return o[t]||(o[t]=void 0!==e?e:{})})("versions",[]).push({version:r.version,mode:n(50)?"pure":"global",copyright:"© 2019 Denis Pushkarev (zloirock.ru)"})},function(t,e){t.exports=!1},function(t,e){t.exports=function(t,e){return{enumerable:!(1&t),configurable:!(2&t),writable:!(4&t),value:e}}},function(t,e,n){var r=n(71);t.exports=function(t,e,n){if(r(t),void 0===e)return t;switch(n){case 1:return function(n){return t.call(e,n)};case 2:return function(n,r){return t.call(e,n,r)};case 3:return function(n,r,i){return t.call(e,n,r,i)}}return function(){return t.apply(e,arguments)}}},function(t,e,n){var r=n(99),i=n(74);t.exports=Object.keys||function(t){return r(t,i)}},function(t,e,n){var r=n(19).f,i=n(27),o=n(7)("toStringTag");t.exports=function(t,e,n){t&&!i(t=n?t:t.prototype,o)&&r(t,o,{configurable:!0,value:e})}},function(t,e){t.exports=function(t){var e=typeof t;return null!=t&&("object"==e||"function"==e)}},function(t,e,n){var r=n(217),i=n(218),o=n(219),a=n(220),s=n(221);function A(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}A.prototype.clear=r,A.prototype.delete=i,A.prototype.get=o,A.prototype.has=a,A.prototype.set=s,t.exports=A},function(t,e,n){var r=n(115);t.exports=function(t,e){for(var n=t.length;n--;)if(r(t[n][0],e))return n;return-1}},function(t,e,n){var r=n(31)(Object,"create");t.exports=r},function(t,e,n){var r=n(239);t.exports=function(t,e){var n=t.__data__;return r(e)?n["string"==typeof e?"string":"hash"]:n.map}},function(t,e,n){(function(t,e){n(75);const r=n(173).default,i=["perspective.config.js","perspective.config.json","package.json"];function o(t){return t&&"object"==typeof t&&!Array.isArray(t)}function a(t,...e){if(!e.length)return t;const n=e.shift();if(o(t)&&o(n))for(const e in n)o(n[e])?(t[e]||Object.assign(t,{[e]:{}}),a(t[e],n[e])):Object.assign(t,{[e]:n[e]});return a(t,...e)}let s;t.exports.get_type_config=function(e){return t.exports.get_config().types[e]||{}},t.exports.override_config=function(t){s&&console.warn("Config already initialized!"),s=a(r,t)},t.exports.get_config=function(){return s||(s=a(r,"undefined"==typeof window?function(){void 0!==require||t.require;const n=__webpack_require__(171),r=__webpack_require__(1327),[o,...a]=e.cwd().split(n.sep);for(;a.length>0;){for(const t of i){const e="".concat(o).concat(n.sep).concat(n.join(...a,t));if(r.existsSync(e)){if(!t.endsWith("json")){const t=__webpack_require__(1328)(e);return t.default||t}{const n=JSON.parse(r.readFileSync(e));if("package.json"!==t)return n;if(n.perspective)return n.perspective}}}a.pop()}}():{types:{float:{filter_operator:"==",aggregate:"sum",format:{style:"decimal",minimumFractionDigits:2,maximumFractionDigits:2}},string:{filter_operator:"==",aggregate:"count"},integer:{filter_operator:"==",aggregate:"sum",format:{}},boolean:{filter_operator:"==",aggregate:"count"},datetime:{filter_operator:"==",aggregate:"count",format:{week:"numeric",year:"numeric",month:"numeric",day:"numeric",hour:"numeric",minute:"numeric",second:"numeric"}},date:{filter_operator:"==",aggregate:"count",format:{week:"numeric",year:"numeric",month:"numeric",day:"numeric"}}}}||{})),s}}).call(this,n(43)(t),n(100))},function(t,e,n){"use strict";function r(t){return+t}function i(t){return t*t}function o(t){return t*(2-t)}function a(t){return((t*=2)<=1?t*t:--t*(2-t)+1)/2}function s(t){return t*t*t}function A(t){return--t*t*t+1}function l(t){return((t*=2)<=1?t*t*t:(t-=2)*t*t+2)/2}var c=function t(e){function n(t){return Math.pow(t,e)}return e=+e,n.exponent=t,n}(3),u=function t(e){function n(t){return 1-Math.pow(1-t,e)}return e=+e,n.exponent=t,n}(3),g=function t(e){function n(t){return((t*=2)<=1?Math.pow(t,e):2-Math.pow(2-t,e))/2}return e=+e,n.exponent=t,n}(3),h=Math.PI,d=h/2;function f(t){return 1-Math.cos(t*d)}function p(t){return Math.sin(t*d)}function I(t){return(1-Math.cos(h*t))/2}function C(t){return Math.pow(2,10*t-10)}function m(t){return 1-Math.pow(2,-10*t)}function y(t){return((t*=2)<=1?Math.pow(2,10*t-10):2-Math.pow(2,10-10*t))/2}function E(t){return 1-Math.sqrt(1-t*t)}function v(t){return Math.sqrt(1- --t*t)}function B(t){return((t*=2)<=1?1-Math.sqrt(1-t*t):Math.sqrt(1-(t-=2)*t)+1)/2}var b=4/11,Q=6/11,w=8/11,x=.75,_=9/11,D=10/11,M=.9375,k=21/22,S=63/64,L=1/b/b;function R(t){return 1-N(1-t)}function N(t){return(t=+t)<b?L*t*t:t<w?L*(t-=Q)*t+x:t<D?L*(t-=_)*t+M:L*(t-=k)*t+S}function F(t){return((t*=2)<=1?1-N(1-t):N(t-1)+1)/2}var T=function t(e){function n(t){return t*t*((e+1)*t-e)}return e=+e,n.overshoot=t,n}(1.70158),U=function t(e){function n(t){return--t*t*((e+1)*t+e)+1}return e=+e,n.overshoot=t,n}(1.70158),O=function t(e){function n(t){return((t*=2)<1?t*t*((e+1)*t-e):(t-=2)*t*((e+1)*t+e)+2)/2}return e=+e,n.overshoot=t,n}(1.70158),G=2*Math.PI,H=function t(e,n){var r=Math.asin(1/(e=Math.max(1,e)))*(n/=G);function i(t){return e*Math.pow(2,10*--t)*Math.sin((r-t)/n)}return i.amplitude=function(e){return t(e,n*G)},i.period=function(n){return t(e,n)},i}(1,.3),j=function t(e,n){var r=Math.asin(1/(e=Math.max(1,e)))*(n/=G);function i(t){return 1-e*Math.pow(2,-10*(t=+t))*Math.sin((t+r)/n)}return i.amplitude=function(e){return t(e,n*G)},i.period=function(n){return t(e,n)},i}(1,.3),Y=function t(e,n){var r=Math.asin(1/(e=Math.max(1,e)))*(n/=G);function i(t){return((t=2*t-1)<0?e*Math.pow(2,10*t)*Math.sin((r-t)/n):2-e*Math.pow(2,-10*t)*Math.sin((r+t)/n))/2}return i.amplitude=function(e){return t(e,n*G)},i.period=function(n){return t(e,n)},i}(1,.3);n.d(e,"y",function(){return r}),n.d(e,"D",function(){return a}),n.d(e,"E",function(){return i}),n.d(e,"G",function(){return o}),n.d(e,"F",function(){return a}),n.d(e,"m",function(){return l}),n.d(e,"n",function(){return s}),n.d(e,"p",function(){return A}),n.d(e,"o",function(){return l}),n.d(e,"z",function(){return g}),n.d(e,"A",function(){return c}),n.d(e,"C",function(){return u}),n.d(e,"B",function(){return g}),n.d(e,"H",function(){return I}),n.d(e,"I",function(){return f}),n.d(e,"K",function(){return p}),n.d(e,"J",function(){return I}),n.d(e,"u",function(){return y}),n.d(e,"v",function(){return C}),n.d(e,"x",function(){return m}),n.d(e,"w",function(){return y}),n.d(e,"i",function(){return B}),n.d(e,"j",function(){return E}),n.d(e,"l",function(){return v}),n.d(e,"k",function(){return B}),n.d(e,"e",function(){return N}),n.d(e,"f",function(){return R}),n.d(e,"h",function(){return N}),n.d(e,"g",function(){return F}),n.d(e,"a",function(){return O}),n.d(e,"b",function(){return T}),n.d(e,"d",function(){return U}),n.d(e,"c",function(){return O}),n.d(e,"q",function(){return j}),n.d(e,"r",function(){return H}),n.d(e,"t",function(){return j}),n.d(e,"s",function(){return Y})},function(t,e,n){var r,i;!function(){var o={version:"3.5.17"},a=[].slice,s=function(t){return a.call(t)},A=this.document;function l(t){return t&&(t.ownerDocument||t.document||t).documentElement}function c(t){return t&&(t.ownerDocument&&t.ownerDocument.defaultView||t.document&&t||t.defaultView)}if(A)try{s(A.documentElement.childNodes)[0].nodeType}catch(t){s=function(t){for(var e=t.length,n=new Array(e);e--;)n[e]=t[e];return n}}if(Date.now||(Date.now=function(){return+new Date}),A)try{A.createElement("DIV").style.setProperty("opacity",0,"")}catch(t){var u=this.Element.prototype,g=u.setAttribute,h=u.setAttributeNS,d=this.CSSStyleDeclaration.prototype,f=d.setProperty;u.setAttribute=function(t,e){g.call(this,t,e+"")},u.setAttributeNS=function(t,e,n){h.call(this,t,e,n+"")},d.setProperty=function(t,e,n){f.call(this,t,e+"",n)}}function p(t,e){return t<e?-1:t>e?1:t>=e?0:NaN}function I(t){return null===t?NaN:+t}function C(t){return!isNaN(t)}function m(t){return{left:function(e,n,r,i){for(arguments.length<3&&(r=0),arguments.length<4&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)<0?r=o+1:i=o}return r},right:function(e,n,r,i){for(arguments.length<3&&(r=0),arguments.length<4&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)>0?i=o:r=o+1}return r}}}o.ascending=p,o.descending=function(t,e){return e<t?-1:e>t?1:e>=t?0:NaN},o.min=function(t,e){var n,r,i=-1,o=t.length;if(1===arguments.length){for(;++i<o;)if(null!=(r=t[i])&&r>=r){n=r;break}for(;++i<o;)null!=(r=t[i])&&n>r&&(n=r)}else{for(;++i<o;)if(null!=(r=e.call(t,t[i],i))&&r>=r){n=r;break}for(;++i<o;)null!=(r=e.call(t,t[i],i))&&n>r&&(n=r)}return n},o.max=function(t,e){var n,r,i=-1,o=t.length;if(1===arguments.length){for(;++i<o;)if(null!=(r=t[i])&&r>=r){n=r;break}for(;++i<o;)null!=(r=t[i])&&r>n&&(n=r)}else{for(;++i<o;)if(null!=(r=e.call(t,t[i],i))&&r>=r){n=r;break}for(;++i<o;)null!=(r=e.call(t,t[i],i))&&r>n&&(n=r)}return n},o.extent=function(t,e){var n,r,i,o=-1,a=t.length;if(1===arguments.length){for(;++o<a;)if(null!=(r=t[o])&&r>=r){n=i=r;break}for(;++o<a;)null!=(r=t[o])&&(n>r&&(n=r),i<r&&(i=r))}else{for(;++o<a;)if(null!=(r=e.call(t,t[o],o))&&r>=r){n=i=r;break}for(;++o<a;)null!=(r=e.call(t,t[o],o))&&(n>r&&(n=r),i<r&&(i=r))}return[n,i]},o.sum=function(t,e){var n,r=0,i=t.length,o=-1;if(1===arguments.length)for(;++o<i;)C(n=+t[o])&&(r+=n);else for(;++o<i;)C(n=+e.call(t,t[o],o))&&(r+=n);return r},o.mean=function(t,e){var n,r=0,i=t.length,o=-1,a=i;if(1===arguments.length)for(;++o<i;)C(n=I(t[o]))?r+=n:--a;else for(;++o<i;)C(n=I(e.call(t,t[o],o)))?r+=n:--a;if(a)return r/a},o.quantile=function(t,e){var n=(t.length-1)*e+1,r=Math.floor(n),i=+t[r-1],o=n-r;return o?i+o*(t[r]-i):i},o.median=function(t,e){var n,r=[],i=t.length,a=-1;if(1===arguments.length)for(;++a<i;)C(n=I(t[a]))&&r.push(n);else for(;++a<i;)C(n=I(e.call(t,t[a],a)))&&r.push(n);if(r.length)return o.quantile(r.sort(p),.5)},o.variance=function(t,e){var n,r,i=t.length,o=0,a=0,s=-1,A=0;if(1===arguments.length)for(;++s<i;)C(n=I(t[s]))&&(a+=(r=n-o)*(n-(o+=r/++A)));else for(;++s<i;)C(n=I(e.call(t,t[s],s)))&&(a+=(r=n-o)*(n-(o+=r/++A)));if(A>1)return a/(A-1)},o.deviation=function(){var t=o.variance.apply(this,arguments);return t?Math.sqrt(t):t};var y=m(p);function E(t){return t.length}o.bisectLeft=y.left,o.bisect=o.bisectRight=y.right,o.bisector=function(t){return m(1===t.length?function(e,n){return p(t(e),n)}:t)},o.shuffle=function(t,e,n){(o=arguments.length)<3&&(n=t.length,o<2&&(e=0));for(var r,i,o=n-e;o;)i=Math.random()*o--|0,r=t[o+e],t[o+e]=t[i+e],t[i+e]=r;return t},o.permute=function(t,e){for(var n=e.length,r=new Array(n);n--;)r[n]=t[e[n]];return r},o.pairs=function(t){for(var e=0,n=t.length-1,r=t[0],i=new Array(n<0?0:n);e<n;)i[e]=[r,r=t[++e]];return i},o.transpose=function(t){if(!(i=t.length))return[];for(var e=-1,n=o.min(t,E),r=new Array(n);++e<n;)for(var i,a=-1,s=r[e]=new Array(i);++a<i;)s[a]=t[a][e];return r},o.zip=function(){return o.transpose(arguments)},o.keys=function(t){var e=[];for(var n in t)e.push(n);return e},o.values=function(t){var e=[];for(var n in t)e.push(t[n]);return e},o.entries=function(t){var e=[];for(var n in t)e.push({key:n,value:t[n]});return e},o.merge=function(t){for(var e,n,r,i=t.length,o=-1,a=0;++o<i;)a+=t[o].length;for(n=new Array(a);--i>=0;)for(e=(r=t[i]).length;--e>=0;)n[--a]=r[e];return n};var v=Math.abs;function B(t){for(var e=1;t*e%1;)e*=10;return e}function b(t,e){for(var n in e)Object.defineProperty(t.prototype,n,{value:e[n],enumerable:!1})}function Q(){this._=Object.create(null)}o.range=function(t,e,n){if(arguments.length<3&&(n=1,arguments.length<2&&(e=t,t=0)),(e-t)/n==1/0)throw new Error("infinite range");var r,i=[],o=B(v(n)),a=-1;if(t*=o,e*=o,(n*=o)<0)for(;(r=t+n*++a)>e;)i.push(r/o);else for(;(r=t+n*++a)<e;)i.push(r/o);return i},o.map=function(t,e){var n=new Q;if(t instanceof Q)t.forEach(function(t,e){n.set(t,e)});else if(Array.isArray(t)){var r,i=-1,o=t.length;if(1===arguments.length)for(;++i<o;)n.set(i,t[i]);else for(;++i<o;)n.set(e.call(t,r=t[i],i),r)}else for(var a in t)n.set(a,t[a]);return n};var w="__proto__",x="\0";function _(t){return(t+="")===w||t[0]===x?x+t:t}function D(t){return(t+="")[0]===x?t.slice(1):t}function M(t){return _(t)in this._}function k(t){return(t=_(t))in this._&&delete this._[t]}function S(){var t=[];for(var e in this._)t.push(D(e));return t}function L(){var t=0;for(var e in this._)++t;return t}function R(){for(var t in this._)return!1;return!0}function N(){this._=Object.create(null)}function F(t){return t}function T(t,e,n){return function(){var r=n.apply(e,arguments);return r===e?t:r}}function U(t,e){if(e in t)return e;e=e.charAt(0).toUpperCase()+e.slice(1);for(var n=0,r=O.length;n<r;++n){var i=O[n]+e;if(i in t)return i}}b(Q,{has:M,get:function(t){return this._[_(t)]},set:function(t,e){return this._[_(t)]=e},remove:k,keys:S,values:function(){var t=[];for(var e in this._)t.push(this._[e]);return t},entries:function(){var t=[];for(var e in this._)t.push({key:D(e),value:this._[e]});return t},size:L,empty:R,forEach:function(t){for(var e in this._)t.call(this,D(e),this._[e])}}),o.nest=function(){var t,e,n={},r=[],i=[];function a(i,o,s){if(s>=r.length)return e?e.call(n,o):t?o.sort(t):o;for(var A,l,c,u,g=-1,h=o.length,d=r[s++],f=new Q;++g<h;)(u=f.get(A=d(l=o[g])))?u.push(l):f.set(A,[l]);return i?(l=i(),c=function(t,e){l.set(t,a(i,e,s))}):(l={},c=function(t,e){l[t]=a(i,e,s)}),f.forEach(c),l}return n.map=function(t,e){return a(e,t,0)},n.entries=function(t){return function t(e,n){if(n>=r.length)return e;var o=[],a=i[n++];return e.forEach(function(e,r){o.push({key:e,values:t(r,n)})}),a?o.sort(function(t,e){return a(t.key,e.key)}):o}(a(o.map,t,0),0)},n.key=function(t){return r.push(t),n},n.sortKeys=function(t){return i[r.length-1]=t,n},n.sortValues=function(e){return t=e,n},n.rollup=function(t){return e=t,n},n},o.set=function(t){var e=new N;if(t)for(var n=0,r=t.length;n<r;++n)e.add(t[n]);return e},b(N,{has:M,add:function(t){return this._[_(t+="")]=!0,t},remove:k,values:S,size:L,empty:R,forEach:function(t){for(var e in this._)t.call(this,D(e))}}),o.behavior={},o.rebind=function(t,e){for(var n,r=1,i=arguments.length;++r<i;)t[n=arguments[r]]=T(t,e,e[n]);return t};var O=["webkit","ms","moz","Moz","o","O"];function G(){}function H(){}function j(t){var e=[],n=new Q;function r(){for(var n,r=e,i=-1,o=r.length;++i<o;)(n=r[i].on)&&n.apply(this,arguments);return t}return r.on=function(r,i){var o,a=n.get(r);return arguments.length<2?a&&a.on:(a&&(a.on=null,e=e.slice(0,o=e.indexOf(a)).concat(e.slice(o+1)),n.remove(r)),i&&e.push(n.set(r,{on:i})),t)},r}function Y(){o.event.preventDefault()}function P(){for(var t,e=o.event;t=e.sourceEvent;)e=t;return e}function K(t){for(var e=new H,n=0,r=arguments.length;++n<r;)e[arguments[n]]=j(e);return e.of=function(n,r){return function(i){try{var a=i.sourceEvent=o.event;i.target=t,o.event=i,e[i.type].apply(n,r)}finally{o.event=a}}},e}o.dispatch=function(){for(var t=new H,e=-1,n=arguments.length;++e<n;)t[arguments[e]]=j(t);return t},H.prototype.on=function(t,e){var n=t.indexOf("."),r="";if(n>=0&&(r=t.slice(n+1),t=t.slice(0,n)),t)return arguments.length<2?this[t].on(r):this[t].on(r,e);if(2===arguments.length){if(null==e)for(t in this)this.hasOwnProperty(t)&&this[t].on(r,null);return this}},o.event=null,o.requote=function(t){return t.replace(z,"\\$&")};var z=/[\\\^\$\*\+\?\|\[\]\(\)\.\{\}]/g,J={}.__proto__?function(t,e){t.__proto__=e}:function(t,e){for(var n in e)t[n]=e[n]};function q(t){return J(t,Z),t}var V=function(t,e){return e.querySelector(t)},W=function(t,e){return e.querySelectorAll(t)},X=function(t,e){var n=t.matches||t[U(t,"matchesSelector")];return(X=function(t,e){return n.call(t,e)})(t,e)};"function"==typeof Sizzle&&(V=function(t,e){return Sizzle(t,e)[0]||null},W=Sizzle,X=Sizzle.matchesSelector),o.selection=function(){return o.select(A.documentElement)};var Z=o.selection.prototype=[];function $(t){return"function"==typeof t?t:function(){return V(t,this)}}function tt(t){return"function"==typeof t?t:function(){return W(t,this)}}Z.select=function(t){var e,n,r,i,o=[];t=$(t);for(var a=-1,s=this.length;++a<s;){o.push(e=[]),e.parentNode=(r=this[a]).parentNode;for(var A=-1,l=r.length;++A<l;)(i=r[A])?(e.push(n=t.call(i,i.__data__,A,a)),n&&"__data__"in i&&(n.__data__=i.__data__)):e.push(null)}return q(o)},Z.selectAll=function(t){var e,n,r=[];t=tt(t);for(var i=-1,o=this.length;++i<o;)for(var a=this[i],A=-1,l=a.length;++A<l;)(n=a[A])&&(r.push(e=s(t.call(n,n.__data__,A,i))),e.parentNode=n);return q(r)};var et="http://www.w3.org/1999/xhtml",nt={svg:"http://www.w3.org/2000/svg",xhtml:et,xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"};function rt(t,e){return t=o.ns.qualify(t),null==e?t.local?function(){this.removeAttributeNS(t.space,t.local)}:function(){this.removeAttribute(t)}:"function"==typeof e?t.local?function(){var n=e.apply(this,arguments);null==n?this.removeAttributeNS(t.space,t.local):this.setAttributeNS(t.space,t.local,n)}:function(){var n=e.apply(this,arguments);null==n?this.removeAttribute(t):this.setAttribute(t,n)}:t.local?function(){this.setAttributeNS(t.space,t.local,e)}:function(){this.setAttribute(t,e)}}function it(t){return t.trim().replace(/\s+/g," ")}function ot(t){return new RegExp("(?:^|\\s+)"+o.requote(t)+"(?:\\s+|$)","g")}function at(t){return(t+"").trim().split(/^|\s+/)}function st(t,e){var n=(t=at(t).map(At)).length;return"function"==typeof e?function(){for(var r=-1,i=e.apply(this,arguments);++r<n;)t[r](this,i)}:function(){for(var r=-1;++r<n;)t[r](this,e)}}function At(t){var e=ot(t);return function(n,r){if(i=n.classList)return r?i.add(t):i.remove(t);var i=n.getAttribute("class")||"";r?(e.lastIndex=0,e.test(i)||n.setAttribute("class",it(i+" "+t))):n.setAttribute("class",it(i.replace(e," ")))}}function lt(t,e,n){return null==e?function(){this.style.removeProperty(t)}:"function"==typeof e?function(){var r=e.apply(this,arguments);null==r?this.style.removeProperty(t):this.style.setProperty(t,r,n)}:function(){this.style.setProperty(t,e,n)}}function ct(t,e){return null==e?function(){delete this[t]}:"function"==typeof e?function(){var n=e.apply(this,arguments);null==n?delete this[t]:this[t]=n}:function(){this[t]=e}}function ut(t){return"function"==typeof t?t:(t=o.ns.qualify(t)).local?function(){return this.ownerDocument.createElementNS(t.space,t.local)}:function(){var e=this.ownerDocument,n=this.namespaceURI;return n===et&&e.documentElement.namespaceURI===et?e.createElement(t):e.createElementNS(n,t)}}function gt(){var t=this.parentNode;t&&t.removeChild(this)}function ht(t){return{__data__:t}}function dt(t){return function(){return X(this,t)}}function ft(t){return arguments.length||(t=p),function(e,n){return e&&n?t(e.__data__,n.__data__):!e-!n}}function pt(t,e){for(var n=0,r=t.length;n<r;n++)for(var i,o=t[n],a=0,s=o.length;a<s;a++)(i=o[a])&&e(i,a,n);return t}function It(t){return J(t,Ct),t}o.ns={prefix:nt,qualify:function(t){var e=t.indexOf(":"),n=t;return e>=0&&"xmlns"!==(n=t.slice(0,e))&&(t=t.slice(e+1)),nt.hasOwnProperty(n)?{space:nt[n],local:t}:t}},Z.attr=function(t,e){if(arguments.length<2){if("string"==typeof t){var n=this.node();return(t=o.ns.qualify(t)).local?n.getAttributeNS(t.space,t.local):n.getAttribute(t)}for(e in t)this.each(rt(e,t[e]));return this}return this.each(rt(t,e))},Z.classed=function(t,e){if(arguments.length<2){if("string"==typeof t){var n=this.node(),r=(t=at(t)).length,i=-1;if(e=n.classList){for(;++i<r;)if(!e.contains(t[i]))return!1}else for(e=n.getAttribute("class");++i<r;)if(!ot(t[i]).test(e))return!1;return!0}for(e in t)this.each(st(e,t[e]));return this}return this.each(st(t,e))},Z.style=function(t,e,n){var r=arguments.length;if(r<3){if("string"!=typeof t){for(n in r<2&&(e=""),t)this.each(lt(n,t[n],e));return this}if(r<2){var i=this.node();return c(i).getComputedStyle(i,null).getPropertyValue(t)}n=""}return this.each(lt(t,e,n))},Z.property=function(t,e){if(arguments.length<2){if("string"==typeof t)return this.node()[t];for(e in t)this.each(ct(e,t[e]));return this}return this.each(ct(t,e))},Z.text=function(t){return arguments.length?this.each("function"==typeof t?function(){var e=t.apply(this,arguments);this.textContent=null==e?"":e}:null==t?function(){this.textContent=""}:function(){this.textContent=t}):this.node().textContent},Z.html=function(t){return arguments.length?this.each("function"==typeof t?function(){var e=t.apply(this,arguments);this.innerHTML=null==e?"":e}:null==t?function(){this.innerHTML=""}:function(){this.innerHTML=t}):this.node().innerHTML},Z.append=function(t){return t=ut(t),this.select(function(){return this.appendChild(t.apply(this,arguments))})},Z.insert=function(t,e){return t=ut(t),e=$(e),this.select(function(){return this.insertBefore(t.apply(this,arguments),e.apply(this,arguments)||null)})},Z.remove=function(){return this.each(gt)},Z.data=function(t,e){var n,r,i=-1,o=this.length;if(!arguments.length){for(t=new Array(o=(n=this[0]).length);++i<o;)(r=n[i])&&(t[i]=r.__data__);return t}function a(t,n){var r,i,o,a=t.length,c=n.length,u=Math.min(a,c),g=new Array(c),h=new Array(c),d=new Array(a);if(e){var f,p=new Q,I=new Array(a);for(r=-1;++r<a;)(i=t[r])&&(p.has(f=e.call(i,i.__data__,r))?d[r]=i:p.set(f,i),I[r]=f);for(r=-1;++r<c;)(i=p.get(f=e.call(n,o=n[r],r)))?!0!==i&&(g[r]=i,i.__data__=o):h[r]=ht(o),p.set(f,!0);for(r=-1;++r<a;)r in I&&!0!==p.get(I[r])&&(d[r]=t[r])}else{for(r=-1;++r<u;)i=t[r],o=n[r],i?(i.__data__=o,g[r]=i):h[r]=ht(o);for(;r<c;++r)h[r]=ht(n[r]);for(;r<a;++r)d[r]=t[r]}h.update=g,h.parentNode=g.parentNode=d.parentNode=t.parentNode,s.push(h),A.push(g),l.push(d)}var s=It([]),A=q([]),l=q([]);if("function"==typeof t)for(;++i<o;)a(n=this[i],t.call(n,n.parentNode.__data__,i));else for(;++i<o;)a(n=this[i],t);return A.enter=function(){return s},A.exit=function(){return l},A},Z.datum=function(t){return arguments.length?this.property("__data__",t):this.property("__data__")},Z.filter=function(t){var e,n,r,i=[];"function"!=typeof t&&(t=dt(t));for(var o=0,a=this.length;o<a;o++){i.push(e=[]),e.parentNode=(n=this[o]).parentNode;for(var s=0,A=n.length;s<A;s++)(r=n[s])&&t.call(r,r.__data__,s,o)&&e.push(r)}return q(i)},Z.order=function(){for(var t=-1,e=this.length;++t<e;)for(var n,r=this[t],i=r.length-1,o=r[i];--i>=0;)(n=r[i])&&(o&&o!==n.nextSibling&&o.parentNode.insertBefore(n,o),o=n);return this},Z.sort=function(t){t=ft.apply(this,arguments);for(var e=-1,n=this.length;++e<n;)this[e].sort(t);return this.order()},Z.each=function(t){return pt(this,function(e,n,r){t.call(e,e.__data__,n,r)})},Z.call=function(t){var e=s(arguments);return t.apply(e[0]=this,e),this},Z.empty=function(){return!this.node()},Z.node=function(){for(var t=0,e=this.length;t<e;t++)for(var n=this[t],r=0,i=n.length;r<i;r++){var o=n[r];if(o)return o}return null},Z.size=function(){var t=0;return pt(this,function(){++t}),t};var Ct=[];function mt(t){var e,n;return function(r,i,o){var a,s=t[o].update,A=s.length;for(o!=n&&(n=o,e=0),i>=e&&(e=i+1);!(a=s[e])&&++e<A;);return a}}function yt(t,e,n){var r="__on"+t,i=t.indexOf("."),a=vt;i>0&&(t=t.slice(0,i));var A=Et.get(t);function l(){var e=this[r];e&&(this.removeEventListener(t,e,e.$),delete this[r])}return A&&(t=A,a=Bt),i?e?function(){var i=a(e,s(arguments));l.call(this),this.addEventListener(t,this[r]=i,i.$=n),i._=e}:l:e?G:function(){var e,n=new RegExp("^__on([^.]+)"+o.requote(t)+"$");for(var r in this)if(e=r.match(n)){var i=this[r];this.removeEventListener(e[1],i,i.$),delete this[r]}}}o.selection.enter=It,o.selection.enter.prototype=Ct,Ct.append=Z.append,Ct.empty=Z.empty,Ct.node=Z.node,Ct.call=Z.call,Ct.size=Z.size,Ct.select=function(t){for(var e,n,r,i,o,a=[],s=-1,A=this.length;++s<A;){r=(i=this[s]).update,a.push(e=[]),e.parentNode=i.parentNode;for(var l=-1,c=i.length;++l<c;)(o=i[l])?(e.push(r[l]=n=t.call(i.parentNode,o.__data__,l,s)),n.__data__=o.__data__):e.push(null)}return q(a)},Ct.insert=function(t,e){return arguments.length<2&&(e=mt(this)),Z.insert.call(this,t,e)},o.select=function(t){var e;return"string"==typeof t?(e=[V(t,A)]).parentNode=A.documentElement:(e=[t]).parentNode=l(t),q([e])},o.selectAll=function(t){var e;return"string"==typeof t?(e=s(W(t,A))).parentNode=A.documentElement:(e=s(t)).parentNode=null,q([e])},Z.on=function(t,e,n){var r=arguments.length;if(r<3){if("string"!=typeof t){for(n in r<2&&(e=!1),t)this.each(yt(n,t[n],e));return this}if(r<2)return(r=this.node()["__on"+t])&&r._;n=!1}return this.each(yt(t,e,n))};var Et=o.map({mouseenter:"mouseover",mouseleave:"mouseout"});function vt(t,e){return function(n){var r=o.event;o.event=n,e[0]=this.__data__;try{t.apply(this,e)}finally{o.event=r}}}function Bt(t,e){var n=vt(t,e);return function(t){var e=t.relatedTarget;e&&(e===this||8&e.compareDocumentPosition(this))||n.call(this,t)}}A&&Et.forEach(function(t){"on"+t in A&&Et.remove(t)});var bt,Qt=0;function wt(t){var e=".dragsuppress-"+ ++Qt,n="click"+e,r=o.select(c(t)).on("touchmove"+e,Y).on("dragstart"+e,Y).on("selectstart"+e,Y);if(null==bt&&(bt=!("onselectstart"in t)&&U(t.style,"userSelect")),bt){var i=l(t).style,a=i[bt];i[bt]="none"}return function(t){if(r.on(e,null),bt&&(i[bt]=a),t){var o=function(){r.on(n,null)};r.on(n,function(){Y(),o()},!0),setTimeout(o,0)}}}o.mouse=function(t){return _t(t,P())};var xt=this.navigator&&/WebKit/.test(this.navigator.userAgent)?-1:0;function _t(t,e){e.changedTouches&&(e=e.changedTouches[0]);var n=t.ownerSVGElement||t;if(n.createSVGPoint){var r=n.createSVGPoint();if(xt<0){var i=c(t);if(i.scrollX||i.scrollY){var a=(n=o.select("body").append("svg").style({position:"absolute",top:0,left:0,margin:0,padding:0,border:"none"},"important"))[0][0].getScreenCTM();xt=!(a.f||a.e),n.remove()}}return xt?(r.x=e.pageX,r.y=e.pageY):(r.x=e.clientX,r.y=e.clientY),[(r=r.matrixTransform(t.getScreenCTM().inverse())).x,r.y]}var s=t.getBoundingClientRect();return[e.clientX-s.left-t.clientLeft,e.clientY-s.top-t.clientTop]}function Dt(){return o.event.changedTouches[0].identifier}o.touch=function(t,e,n){if(arguments.length<3&&(n=e,e=P().changedTouches),e)for(var r,i=0,o=e.length;i<o;++i)if((r=e[i]).identifier===n)return _t(t,r)},o.behavior.drag=function(){var t=K(i,"drag","dragstart","dragend"),e=null,n=a(G,o.mouse,c,"mousemove","mouseup"),r=a(Dt,o.touch,F,"touchmove","touchend");function i(){this.on("mousedown.drag",n).on("touchstart.drag",r)}function a(n,r,i,a,s){return function(){var A,l=this,c=o.event.target.correspondingElement||o.event.target,u=l.parentNode,g=t.of(l,arguments),h=0,d=n(),f=".drag"+(null==d?"":"-"+d),p=o.select(i(c)).on(a+f,m).on(s+f,y),I=wt(c),C=r(u,d);function m(){var t,e,n=r(u,d);n&&(t=n[0]-C[0],e=n[1]-C[1],h|=t|e,C=n,g({type:"drag",x:n[0]+A[0],y:n[1]+A[1],dx:t,dy:e}))}function y(){r(u,d)&&(p.on(a+f,null).on(s+f,null),I(h),g({type:"dragend"}))}A=e?[(A=e.apply(l,arguments)).x-C[0],A.y-C[1]]:[0,0],g({type:"dragstart"})}}return i.origin=function(t){return arguments.length?(e=t,i):e},o.rebind(i,t,"on")},o.touches=function(t,e){return arguments.length<2&&(e=P().touches),e?s(e).map(function(e){var n=_t(t,e);return n.identifier=e.identifier,n}):[]};var Mt=1e-6,kt=Mt*Mt,St=Math.PI,Lt=2*St,Rt=Lt-Mt,Nt=St/2,Ft=St/180,Tt=180/St;function Ut(t){return t>0?1:t<0?-1:0}function Ot(t,e,n){return(e[0]-t[0])*(n[1]-t[1])-(e[1]-t[1])*(n[0]-t[0])}function Gt(t){return t>1?0:t<-1?St:Math.acos(t)}function Ht(t){return t>1?Nt:t<-1?-Nt:Math.asin(t)}function jt(t){return((t=Math.exp(t))+1/t)/2}function Yt(t){return(t=Math.sin(t/2))*t}var Pt=Math.SQRT2;o.interpolateZoom=function(t,e){var n,r,i=t[0],o=t[1],a=t[2],s=e[0],A=e[1],l=e[2],c=s-i,u=A-o,g=c*c+u*u;if(g<kt)r=Math.log(l/a)/Pt,n=function(t){return[i+t*c,o+t*u,a*Math.exp(Pt*t*r)]};else{var h=Math.sqrt(g),d=(l*l-a*a+4*g)/(2*a*2*h),f=(l*l-a*a-4*g)/(2*l*2*h),p=Math.log(Math.sqrt(d*d+1)-d),I=Math.log(Math.sqrt(f*f+1)-f);r=(I-p)/Pt,n=function(t){var e,n=t*r,s=jt(p),A=a/(2*h)*(s*(e=Pt*n+p,((e=Math.exp(2*e))-1)/(e+1))-function(t){return((t=Math.exp(t))-1/t)/2}(p));return[i+A*c,o+A*u,a*s/jt(Pt*n+p)]}}return n.duration=1e3*r,n},o.behavior.zoom=function(){var t,e,n,r,i,a,s,l,u,g={x:0,y:0,k:1},h=[960,500],d=Jt,f=250,p=0,I="mousedown.zoom",C="mousemove.zoom",m="mouseup.zoom",y="touchstart.zoom",E=K(v,"zoomstart","zoom","zoomend");function v(t){t.on(I,k).on(zt+".zoom",L).on("dblclick.zoom",R).on(y,S)}function B(t){return[(t[0]-g.x)/g.k,(t[1]-g.y)/g.k]}function b(t){g.k=Math.max(d[0],Math.min(d[1],t))}function Q(t,e){e=function(t){return[t[0]*g.k+g.x,t[1]*g.k+g.y]}(e),g.x+=t[0]-e[0],g.y+=t[1]-e[1]}function w(t,n,r,i){t.__chart__={x:g.x,y:g.y,k:g.k},b(Math.pow(2,i)),Q(e=n,r),t=o.select(t),f>0&&(t=t.transition().duration(f)),t.call(v.event)}function x(){s&&s.domain(a.range().map(function(t){return(t-g.x)/g.k}).map(a.invert)),u&&u.domain(l.range().map(function(t){return(t-g.y)/g.k}).map(l.invert))}function _(t){p++||t({type:"zoomstart"})}function D(t){x(),t({type:"zoom",scale:g.k,translate:[g.x,g.y]})}function M(t){--p||(t({type:"zoomend"}),e=null)}function k(){var t=this,e=E.of(t,arguments),n=0,r=o.select(c(t)).on(C,function(){n=1,Q(o.mouse(t),i),D(e)}).on(m,function(){r.on(C,null).on(m,null),a(n),M(e)}),i=B(o.mouse(t)),a=wt(t);Qs.call(t),_(e)}function S(){var t,e=this,n=E.of(e,arguments),r={},a=0,s=".zoom-"+o.event.changedTouches[0].identifier,A="touchmove"+s,l="touchend"+s,c=[],u=o.select(e),h=wt(e);function d(){var n=o.touches(e);return t=g.k,n.forEach(function(t){t.identifier in r&&(r[t.identifier]=B(t))}),n}function f(){var t=o.event.target;o.select(t).on(A,p).on(l,C),c.push(t);for(var n=o.event.changedTouches,s=0,u=n.length;s<u;++s)r[n[s].identifier]=null;var h=d(),f=Date.now();if(1===h.length){if(f-i<500){var I=h[0];w(e,I,r[I.identifier],Math.floor(Math.log(g.k)/Math.LN2)+1),Y()}i=f}else if(h.length>1){I=h[0];var m=h[1],y=I[0]-m[0],E=I[1]-m[1];a=y*y+E*E}}function p(){var s,A,l,c,u=o.touches(e);Qs.call(e);for(var g=0,h=u.length;g<h;++g,c=null)if(l=u[g],c=r[l.identifier]){if(A)break;s=l,A=c}if(c){var d=(d=l[0]-s[0])*d+(d=l[1]-s[1])*d,f=a&&Math.sqrt(d/a);s=[(s[0]+l[0])/2,(s[1]+l[1])/2],A=[(A[0]+c[0])/2,(A[1]+c[1])/2],b(f*t)}i=null,Q(s,A),D(n)}function C(){if(o.event.touches.length){for(var t=o.event.changedTouches,e=0,i=t.length;e<i;++e)delete r[t[e].identifier];for(var a in r)return void d()}o.selectAll(c).on(s,null),u.on(I,k).on(y,S),h(),M(n)}f(),_(n),u.on(I,null).on(y,f)}function L(){var i=E.of(this,arguments);r?clearTimeout(r):(Qs.call(this),t=B(e=n||o.mouse(this)),_(i)),r=setTimeout(function(){r=null,M(i)},50),Y(),b(Math.pow(2,.002*Kt())*g.k),Q(e,t),D(i)}function R(){var t=o.mouse(this),e=Math.log(g.k)/Math.LN2;w(this,t,B(t),o.event.shiftKey?Math.ceil(e)-1:Math.floor(e)+1)}return zt||(zt="onwheel"in A?(Kt=function(){return-o.event.deltaY*(o.event.deltaMode?120:1)},"wheel"):"onmousewheel"in A?(Kt=function(){return o.event.wheelDelta},"mousewheel"):(Kt=function(){return-o.event.detail},"MozMousePixelScroll")),v.event=function(t){t.each(function(){var t=E.of(this,arguments),n=g;_s?o.select(this).transition().each("start.zoom",function(){g=this.__chart__||{x:0,y:0,k:1},_(t)}).tween("zoom:zoom",function(){var r=h[0],i=h[1],a=e?e[0]:r/2,s=e?e[1]:i/2,A=o.interpolateZoom([(a-g.x)/g.k,(s-g.y)/g.k,r/g.k],[(a-n.x)/n.k,(s-n.y)/n.k,r/n.k]);return function(e){var n=A(e),i=r/n[2];this.__chart__=g={x:a-n[0]*i,y:s-n[1]*i,k:i},D(t)}}).each("interrupt.zoom",function(){M(t)}).each("end.zoom",function(){M(t)}):(this.__chart__=g,_(t),D(t),M(t))})},v.translate=function(t){return arguments.length?(g={x:+t[0],y:+t[1],k:g.k},x(),v):[g.x,g.y]},v.scale=function(t){return arguments.length?(g={x:g.x,y:g.y,k:null},b(+t),x(),v):g.k},v.scaleExtent=function(t){return arguments.length?(d=null==t?Jt:[+t[0],+t[1]],v):d},v.center=function(t){return arguments.length?(n=t&&[+t[0],+t[1]],v):n},v.size=function(t){return arguments.length?(h=t&&[+t[0],+t[1]],v):h},v.duration=function(t){return arguments.length?(f=+t,v):f},v.x=function(t){return arguments.length?(s=t,a=t.copy(),g={x:0,y:0,k:1},v):s},v.y=function(t){return arguments.length?(u=t,l=t.copy(),g={x:0,y:0,k:1},v):u},o.rebind(v,E,"on")};var Kt,zt,Jt=[0,1/0];function qt(){}function Vt(t,e,n){return this instanceof Vt?(this.h=+t,this.s=+e,void(this.l=+n)):arguments.length<2?t instanceof Vt?new Vt(t.h,t.s,t.l):pe(""+t,Ie,Vt):new Vt(t,e,n)}o.color=qt,qt.prototype.toString=function(){return this.rgb()+""},o.hsl=Vt;var Wt=Vt.prototype=new qt;function Xt(t,e,n){var r,i;function o(t){return Math.round(255*function(t){return t>360?t-=360:t<0&&(t+=360),t<60?r+(i-r)*t/60:t<180?i:t<240?r+(i-r)*(240-t)/60:r}(t))}return t=isNaN(t)?0:(t%=360)<0?t+360:t,e=isNaN(e)?0:e<0?0:e>1?1:e,r=2*(n=n<0?0:n>1?1:n)-(i=n<=.5?n*(1+e):n+e-n*e),new ue(o(t+120),o(t),o(t-120))}function Zt(t,e,n){return this instanceof Zt?(this.h=+t,this.c=+e,void(this.l=+n)):arguments.length<2?t instanceof Zt?new Zt(t.h,t.c,t.l):function(t,e,n){return t>0?new Zt(Math.atan2(n,e)*Tt,Math.sqrt(e*e+n*n),t):new Zt(NaN,NaN,t)}(t instanceof ee?t.l:(t=Ce((t=o.rgb(t)).r,t.g,t.b)).l,t.a,t.b):new Zt(t,e,n)}Wt.brighter=function(t){return t=Math.pow(.7,arguments.length?t:1),new Vt(this.h,this.s,this.l/t)},Wt.darker=function(t){return t=Math.pow(.7,arguments.length?t:1),new Vt(this.h,this.s,t*this.l)},Wt.rgb=function(){return Xt(this.h,this.s,this.l)},o.hcl=Zt;var $t=Zt.prototype=new qt;function te(t,e,n){return isNaN(t)&&(t=0),isNaN(e)&&(e=0),new ee(n,Math.cos(t*=Ft)*e,Math.sin(t)*e)}function ee(t,e,n){return this instanceof ee?(this.l=+t,this.a=+e,void(this.b=+n)):arguments.length<2?t instanceof ee?new ee(t.l,t.a,t.b):t instanceof Zt?te(t.h,t.c,t.l):Ce((t=ue(t)).r,t.g,t.b):new ee(t,e,n)}$t.brighter=function(t){return new Zt(this.h,this.c,Math.min(100,this.l+ne*(arguments.length?t:1)))},$t.darker=function(t){return new Zt(this.h,this.c,Math.max(0,this.l-ne*(arguments.length?t:1)))},$t.rgb=function(){return te(this.h,this.c,this.l).rgb()},o.lab=ee;var ne=18,re=.95047,ie=1,oe=1.08883,ae=ee.prototype=new qt;function se(t,e,n){var r=(t+16)/116,i=r+e/500,o=r-n/200;return new ue(ce(3.2404542*(i=Ae(i)*re)-1.5371385*(r=Ae(r)*ie)-.4985314*(o=Ae(o)*oe)),ce(-.969266*i+1.8760108*r+.041556*o),ce(.0556434*i-.2040259*r+1.0572252*o))}function Ae(t){return t>.206893034?t*t*t:(t-4/29)/7.787037}function le(t){return t>.008856?Math.pow(t,1/3):7.787037*t+4/29}function ce(t){return Math.round(255*(t<=.00304?12.92*t:1.055*Math.pow(t,1/2.4)-.055))}function ue(t,e,n){return this instanceof ue?(this.r=~~t,this.g=~~e,void(this.b=~~n)):arguments.length<2?t instanceof ue?new ue(t.r,t.g,t.b):pe(""+t,ue,Xt):new ue(t,e,n)}function ge(t){return new ue(t>>16,t>>8&255,255&t)}function he(t){return ge(t)+""}ae.brighter=function(t){return new ee(Math.min(100,this.l+ne*(arguments.length?t:1)),this.a,this.b)},ae.darker=function(t){return new ee(Math.max(0,this.l-ne*(arguments.length?t:1)),this.a,this.b)},ae.rgb=function(){return se(this.l,this.a,this.b)},o.rgb=ue;var de=ue.prototype=new qt;function fe(t){return t<16?"0"+Math.max(0,t).toString(16):Math.min(255,t).toString(16)}function pe(t,e,n){var r,i,o,a=0,s=0,A=0;if(r=/([a-z]+)\((.*)\)/.exec(t=t.toLowerCase()))switch(i=r[2].split(","),r[1]){case"hsl":return n(parseFloat(i[0]),parseFloat(i[1])/100,parseFloat(i[2])/100);case"rgb":return e(ye(i[0]),ye(i[1]),ye(i[2]))}return(o=Ee.get(t))?e(o.r,o.g,o.b):(null==t||"#"!==t.charAt(0)||isNaN(o=parseInt(t.slice(1),16))||(4===t.length?(a=(3840&o)>>4,a|=a>>4,s=240&o,s|=s>>4,A=15&o,A|=A<<4):7===t.length&&(a=(16711680&o)>>16,s=(65280&o)>>8,A=255&o)),e(a,s,A))}function Ie(t,e,n){var r,i,o=Math.min(t/=255,e/=255,n/=255),a=Math.max(t,e,n),s=a-o,A=(a+o)/2;return s?(i=A<.5?s/(a+o):s/(2-a-o),r=t==a?(e-n)/s+(e<n?6:0):e==a?(n-t)/s+2:(t-e)/s+4,r*=60):(r=NaN,i=A>0&&A<1?0:r),new Vt(r,i,A)}function Ce(t,e,n){var r=le((.4124564*(t=me(t))+.3575761*(e=me(e))+.1804375*(n=me(n)))/re),i=le((.2126729*t+.7151522*e+.072175*n)/ie);return ee(116*i-16,500*(r-i),200*(i-le((.0193339*t+.119192*e+.9503041*n)/oe)))}function me(t){return(t/=255)<=.04045?t/12.92:Math.pow((t+.055)/1.055,2.4)}function ye(t){var e=parseFloat(t);return"%"===t.charAt(t.length-1)?Math.round(2.55*e):e}de.brighter=function(t){t=Math.pow(.7,arguments.length?t:1);var e=this.r,n=this.g,r=this.b,i=30;return e||n||r?(e&&e<i&&(e=i),n&&n<i&&(n=i),r&&r<i&&(r=i),new ue(Math.min(255,e/t),Math.min(255,n/t),Math.min(255,r/t))):new ue(i,i,i)},de.darker=function(t){return new ue((t=Math.pow(.7,arguments.length?t:1))*this.r,t*this.g,t*this.b)},de.hsl=function(){return Ie(this.r,this.g,this.b)},de.toString=function(){return"#"+fe(this.r)+fe(this.g)+fe(this.b)};var Ee=o.map({aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074});function ve(t){return"function"==typeof t?t:function(){return t}}function Be(t){return function(e,n,r){return 2===arguments.length&&"function"==typeof n&&(r=n,n=null),be(e,n,t,r)}}function be(t,e,n,r){var i={},a=o.dispatch("beforesend","progress","load","error"),A={},l=new XMLHttpRequest,c=null;function u(){var t,e=l.status;if(!e&&function(t){var e=t.responseType;return e&&"text"!==e?t.response:t.responseText}(l)||e>=200&&e<300||304===e){try{t=n.call(i,l)}catch(t){return void a.error.call(i,t)}a.load.call(i,t)}else a.error.call(i,l)}return!this.XDomainRequest||"withCredentials"in l||!/^(http(s)?:)?\/\//.test(t)||(l=new XDomainRequest),"onload"in l?l.onload=l.onerror=u:l.onreadystatechange=function(){l.readyState>3&&u()},l.onprogress=function(t){var e=o.event;o.event=t;try{a.progress.call(i,l)}finally{o.event=e}},i.header=function(t,e){return t=(t+"").toLowerCase(),arguments.length<2?A[t]:(null==e?delete A[t]:A[t]=e+"",i)},i.mimeType=function(t){return arguments.length?(e=null==t?null:t+"",i):e},i.responseType=function(t){return arguments.length?(c=t,i):c},i.response=function(t){return n=t,i},["get","post"].forEach(function(t){i[t]=function(){return i.send.apply(i,[t].concat(s(arguments)))}}),i.send=function(n,r,o){if(2===arguments.length&&"function"==typeof r&&(o=r,r=null),l.open(n,t,!0),null==e||"accept"in A||(A.accept=e+",*/*"),l.setRequestHeader)for(var s in A)l.setRequestHeader(s,A[s]);return null!=e&&l.overrideMimeType&&l.overrideMimeType(e),null!=c&&(l.responseType=c),null!=o&&i.on("error",o).on("load",function(t){o(null,t)}),a.beforesend.call(i,l),l.send(null==r?null:r),i},i.abort=function(){return l.abort(),i},o.rebind(i,a,"on"),null==r?i:i.get(function(t){return 1===t.length?function(e,n){t(null==e?n:null)}:t}(r))}Ee.forEach(function(t,e){Ee.set(t,ge(e))}),o.functor=ve,o.xhr=Be(F),o.dsv=function(t,e){var n=new RegExp('["'+t+"\n]"),r=t.charCodeAt(0);function i(t,n,r){arguments.length<3&&(r=n,n=null);var i=be(t,e,null==n?o:a(n),r);return i.row=function(t){return arguments.length?i.response(null==(n=t)?o:a(t)):n},i}function o(t){return i.parse(t.responseText)}function a(t){return function(e){return i.parse(e.responseText,t)}}function s(e){return e.map(A).join(t)}function A(t){return n.test(t)?'"'+t.replace(/\"/g,'""')+'"':t}return i.parse=function(t,e){var n;return i.parseRows(t,function(t,r){if(n)return n(t,r-1);var i=new Function("d","return {"+t.map(function(t,e){return JSON.stringify(t)+": d["+e+"]"}).join(",")+"}");n=e?function(t,n){return e(i(t),n)}:i})},i.parseRows=function(t,e){var n,i,o={},a={},s=[],A=t.length,l=0,c=0;function u(){if(l>=A)return a;if(i)return i=!1,o;var e=l;if(34===t.charCodeAt(e)){for(var n=e;n++<A;)if(34===t.charCodeAt(n)){if(34!==t.charCodeAt(n+1))break;++n}return l=n+2,13===(s=t.charCodeAt(n+1))?(i=!0,10===t.charCodeAt(n+2)&&++l):10===s&&(i=!0),t.slice(e+1,n).replace(/""/g,'"')}for(;l<A;){var s,c=1;if(10===(s=t.charCodeAt(l++)))i=!0;else if(13===s)i=!0,10===t.charCodeAt(l)&&(++l,++c);else if(s!==r)continue;return t.slice(e,l-c)}return t.slice(e)}for(;(n=u())!==a;){for(var g=[];n!==o&&n!==a;)g.push(n),n=u();e&&null==(g=e(g,c++))||s.push(g)}return s},i.format=function(e){if(Array.isArray(e[0]))return i.formatRows(e);var n=new N,r=[];return e.forEach(function(t){for(var e in t)n.has(e)||r.push(n.add(e))}),[r.map(A).join(t)].concat(e.map(function(e){return r.map(function(t){return A(e[t])}).join(t)})).join("\n")},i.formatRows=function(t){return t.map(s).join("\n")},i},o.csv=o.dsv(",","text/csv"),o.tsv=o.dsv("\t","text/tab-separated-values");var Qe,we,xe,_e,De=this[U(this,"requestAnimationFrame")]||function(t){setTimeout(t,17)};function Me(t,e,n){var r=arguments.length;r<2&&(e=0),r<3&&(n=Date.now());var i={c:t,t:n+e,n:null};return we?we.n=i:Qe=i,we=i,xe||(_e=clearTimeout(_e),xe=1,De(ke)),i}function ke(){var t=Se(),e=Le()-t;e>24?(isFinite(e)&&(clearTimeout(_e),_e=setTimeout(ke,e)),xe=0):(xe=1,De(ke))}function Se(){for(var t=Date.now(),e=Qe;e;)t>=e.t&&e.c(t-e.t)&&(e.c=null),e=e.n;return t}function Le(){for(var t,e=Qe,n=1/0;e;)e.c?(e.t<n&&(n=e.t),e=(t=e).n):e=t?t.n=e.n:Qe=e.n;return we=t,n}function Re(t,e){return e-(t?Math.ceil(Math.log(t)/Math.LN10):1)}o.timer=function(){Me.apply(this,arguments)},o.timer.flush=function(){Se(),Le()},o.round=function(t,e){return e?Math.round(t*(e=Math.pow(10,e)))/e:Math.round(t)};var Ne=["y","z","a","f","p","n","µ","m","","k","M","G","T","P","E","Z","Y"].map(function(t,e){var n=Math.pow(10,3*v(8-e));return{scale:e>8?function(t){return t/n}:function(t){return t*n},symbol:t}});function Fe(t){var e=t.decimal,n=t.thousands,r=t.grouping,i=t.currency,a=r&&n?function(t,e){for(var i=t.length,o=[],a=0,s=r[0],A=0;i>0&&s>0&&(A+s+1>e&&(s=Math.max(1,e-A)),o.push(t.substring(i-=s,i+s)),!((A+=s+1)>e));)s=r[a=(a+1)%r.length];return o.reverse().join(n)}:F;return function(t){var n=Te.exec(t),r=n[1]||" ",s=n[2]||">",A=n[3]||"-",l=n[4]||"",c=n[5],u=+n[6],g=n[7],h=n[8],d=n[9],f=1,p="",I="",C=!1,m=!0;switch(h&&(h=+h.substring(1)),(c||"0"===r&&"="===s)&&(c=r="0",s="="),d){case"n":g=!0,d="g";break;case"%":f=100,I="%",d="f";break;case"p":f=100,I="%",d="r";break;case"b":case"o":case"x":case"X":"#"===l&&(p="0"+d.toLowerCase());case"c":m=!1;case"d":C=!0,h=0;break;case"s":f=-1,d="r"}"$"===l&&(p=i[0],I=i[1]),"r"!=d||h||(d="g"),null!=h&&("g"==d?h=Math.max(1,Math.min(21,h)):"e"!=d&&"f"!=d||(h=Math.max(0,Math.min(20,h)))),d=Ue.get(d)||Oe;var y=c&&g;return function(t){var n=I;if(C&&t%1)return"";var i=t<0||0===t&&1/t<0?(t=-t,"-"):"-"===A?"":A;if(f<0){var l=o.formatPrefix(t,h);t=l.scale(t),n=l.symbol+I}else t*=f;var E,v,B=(t=d(t,h)).lastIndexOf(".");if(B<0){var b=m?t.lastIndexOf("e"):-1;b<0?(E=t,v=""):(E=t.substring(0,b),v=t.substring(b))}else E=t.substring(0,B),v=e+t.substring(B+1);!c&&g&&(E=a(E,1/0));var Q=p.length+E.length+v.length+(y?0:i.length),w=Q<u?new Array(Q=u-Q+1).join(r):"";return y&&(E=a(w+E,w.length?u-v.length:1/0)),i+=p,t=E+v,("<"===s?i+t+w:">"===s?w+i+t:"^"===s?w.substring(0,Q>>=1)+i+t+w.substring(Q):i+(y?t:w+t))+n}}}o.formatPrefix=function(t,e){var n=0;return(t=+t)&&(t<0&&(t*=-1),e&&(t=o.round(t,Re(t,e))),n=1+Math.floor(1e-12+Math.log(t)/Math.LN10),n=Math.max(-24,Math.min(24,3*Math.floor((n-1)/3)))),Ne[8+n/3]};var Te=/(?:([^{])?([<>=^]))?([+\- ])?([$#])?(0)?(\d+)?(,)?(\.-?\d+)?([a-z%])?/i,Ue=o.map({b:function(t){return t.toString(2)},c:function(t){return String.fromCharCode(t)},o:function(t){return t.toString(8)},x:function(t){return t.toString(16)},X:function(t){return t.toString(16).toUpperCase()},g:function(t,e){return t.toPrecision(e)},e:function(t,e){return t.toExponential(e)},f:function(t,e){return t.toFixed(e)},r:function(t,e){return(t=o.round(t,Re(t,e))).toFixed(Math.max(0,Math.min(20,Re(t*(1+1e-15),e))))}});function Oe(t){return t+""}var Ge=o.time={},He=Date;function je(){this._=new Date(arguments.length>1?Date.UTC.apply(this,arguments):arguments[0])}je.prototype={getDate:function(){return this._.getUTCDate()},getDay:function(){return this._.getUTCDay()},getFullYear:function(){return this._.getUTCFullYear()},getHours:function(){return this._.getUTCHours()},getMilliseconds:function(){return this._.getUTCMilliseconds()},getMinutes:function(){return this._.getUTCMinutes()},getMonth:function(){return this._.getUTCMonth()},getSeconds:function(){return this._.getUTCSeconds()},getTime:function(){return this._.getTime()},getTimezoneOffset:function(){return 0},valueOf:function(){return this._.valueOf()},setDate:function(){Ye.setUTCDate.apply(this._,arguments)},setDay:function(){Ye.setUTCDay.apply(this._,arguments)},setFullYear:function(){Ye.setUTCFullYear.apply(this._,arguments)},setHours:function(){Ye.setUTCHours.apply(this._,arguments)},setMilliseconds:function(){Ye.setUTCMilliseconds.apply(this._,arguments)},setMinutes:function(){Ye.setUTCMinutes.apply(this._,arguments)},setMonth:function(){Ye.setUTCMonth.apply(this._,arguments)},setSeconds:function(){Ye.setUTCSeconds.apply(this._,arguments)},setTime:function(){Ye.setTime.apply(this._,arguments)}};var Ye=Date.prototype;function Pe(t,e,n){function r(e){var n=t(e),r=o(n,1);return e-n<r-e?n:r}function i(n){return e(n=t(new He(n-1)),1),n}function o(t,n){return e(t=new He(+t),n),t}function a(t,r,o){var a=i(t),s=[];if(o>1)for(;a<r;)n(a)%o||s.push(new Date(+a)),e(a,1);else for(;a<r;)s.push(new Date(+a)),e(a,1);return s}t.floor=t,t.round=r,t.ceil=i,t.offset=o,t.range=a;var s=t.utc=Ke(t);return s.floor=s,s.round=Ke(r),s.ceil=Ke(i),s.offset=Ke(o),s.range=function(t,e,n){try{He=je;var r=new je;return r._=t,a(r,e,n)}finally{He=Date}},t}function Ke(t){return function(e,n){try{He=je;var r=new je;return r._=e,t(r,n)._}finally{He=Date}}}function ze(t){var e=t.dateTime,n=t.date,r=t.time,i=t.periods,a=t.days,s=t.shortDays,A=t.months,l=t.shortMonths;function c(t){var e=t.length;function n(n){for(var r,i,o,a=[],s=-1,A=0;++s<e;)37===t.charCodeAt(s)&&(a.push(t.slice(A,s)),null!=(i=Je[r=t.charAt(++s)])&&(r=t.charAt(++s)),(o=E[r])&&(r=o(n,null==i?"e"===r?" ":"0":i)),a.push(r),A=s+1);return a.push(t.slice(A,s)),a.join("")}return n.parse=function(e){var n={y:1900,m:0,d:1,H:0,M:0,S:0,L:0,Z:null};if(u(n,t,e,0)!=e.length)return null;"p"in n&&(n.H=n.H%12+12*n.p);var r=null!=n.Z&&He!==je,i=new(r?je:He);return"j"in n?i.setFullYear(n.y,0,n.j):"W"in n||"U"in n?("w"in n||(n.w="W"in n?1:0),i.setFullYear(n.y,0,1),i.setFullYear(n.y,0,"W"in n?(n.w+6)%7+7*n.W-(i.getDay()+5)%7:n.w+7*n.U-(i.getDay()+6)%7)):i.setFullYear(n.y,n.m,n.d),i.setHours(n.H+(n.Z/100|0),n.M+n.Z%100,n.S,n.L),r?i._:i},n.toString=function(){return t},n}function u(t,e,n,r){for(var i,o,a,s=0,A=e.length,l=n.length;s<A;){if(r>=l)return-1;if(37===(i=e.charCodeAt(s++))){if(a=e.charAt(s++),!(o=v[a in Je?e.charAt(s++):a])||(r=o(t,n,r))<0)return-1}else if(i!=n.charCodeAt(r++))return-1}return r}c.utc=function(t){var e=c(t);function n(t){try{var n=new(He=je);return n._=t,e(n)}finally{He=Date}}return n.parse=function(t){try{He=je;var n=e.parse(t);return n&&n._}finally{He=Date}},n.toString=e.toString,n},c.multi=c.utc.multi=fn;var g=o.map(),h=Xe(a),d=Ze(a),f=Xe(s),p=Ze(s),I=Xe(A),C=Ze(A),m=Xe(l),y=Ze(l);i.forEach(function(t,e){g.set(t.toLowerCase(),e)});var E={a:function(t){return s[t.getDay()]},A:function(t){return a[t.getDay()]},b:function(t){return l[t.getMonth()]},B:function(t){return A[t.getMonth()]},c:c(e),d:function(t,e){return We(t.getDate(),e,2)},e:function(t,e){return We(t.getDate(),e,2)},H:function(t,e){return We(t.getHours(),e,2)},I:function(t,e){return We(t.getHours()%12||12,e,2)},j:function(t,e){return We(1+Ge.dayOfYear(t),e,3)},L:function(t,e){return We(t.getMilliseconds(),e,3)},m:function(t,e){return We(t.getMonth()+1,e,2)},M:function(t,e){return We(t.getMinutes(),e,2)},p:function(t){return i[+(t.getHours()>=12)]},S:function(t,e){return We(t.getSeconds(),e,2)},U:function(t,e){return We(Ge.sundayOfYear(t),e,2)},w:function(t){return t.getDay()},W:function(t,e){return We(Ge.mondayOfYear(t),e,2)},x:c(n),X:c(r),y:function(t,e){return We(t.getFullYear()%100,e,2)},Y:function(t,e){return We(t.getFullYear()%1e4,e,4)},Z:hn,"%":function(){return"%"}},v={a:function(t,e,n){f.lastIndex=0;var r=f.exec(e.slice(n));return r?(t.w=p.get(r[0].toLowerCase()),n+r[0].length):-1},A:function(t,e,n){h.lastIndex=0;var r=h.exec(e.slice(n));return r?(t.w=d.get(r[0].toLowerCase()),n+r[0].length):-1},b:function(t,e,n){m.lastIndex=0;var r=m.exec(e.slice(n));return r?(t.m=y.get(r[0].toLowerCase()),n+r[0].length):-1},B:function(t,e,n){I.lastIndex=0;var r=I.exec(e.slice(n));return r?(t.m=C.get(r[0].toLowerCase()),n+r[0].length):-1},c:function(t,e,n){return u(t,E.c.toString(),e,n)},d:sn,e:sn,H:ln,I:ln,j:An,L:gn,m:an,M:cn,p:function(t,e,n){var r=g.get(e.slice(n,n+=2).toLowerCase());return null==r?-1:(t.p=r,n)},S:un,U:tn,w:$e,W:en,x:function(t,e,n){return u(t,E.x.toString(),e,n)},X:function(t,e,n){return u(t,E.X.toString(),e,n)},y:rn,Y:nn,Z:on,"%":dn};return c}Ge.year=Pe(function(t){return(t=Ge.day(t)).setMonth(0,1),t},function(t,e){t.setFullYear(t.getFullYear()+e)},function(t){return t.getFullYear()}),Ge.years=Ge.year.range,Ge.years.utc=Ge.year.utc.range,Ge.day=Pe(function(t){var e=new He(2e3,0);return e.setFullYear(t.getFullYear(),t.getMonth(),t.getDate()),e},function(t,e){t.setDate(t.getDate()+e)},function(t){return t.getDate()-1}),Ge.days=Ge.day.range,Ge.days.utc=Ge.day.utc.range,Ge.dayOfYear=function(t){var e=Ge.year(t);return Math.floor((t-e-6e4*(t.getTimezoneOffset()-e.getTimezoneOffset()))/864e5)},["sunday","monday","tuesday","wednesday","thursday","friday","saturday"].forEach(function(t,e){e=7-e;var n=Ge[t]=Pe(function(t){return(t=Ge.day(t)).setDate(t.getDate()-(t.getDay()+e)%7),t},function(t,e){t.setDate(t.getDate()+7*Math.floor(e))},function(t){var n=Ge.year(t).getDay();return Math.floor((Ge.dayOfYear(t)+(n+e)%7)/7)-(n!==e)});Ge[t+"s"]=n.range,Ge[t+"s"].utc=n.utc.range,Ge[t+"OfYear"]=function(t){var n=Ge.year(t).getDay();return Math.floor((Ge.dayOfYear(t)+(n+e)%7)/7)}}),Ge.week=Ge.sunday,Ge.weeks=Ge.sunday.range,Ge.weeks.utc=Ge.sunday.utc.range,Ge.weekOfYear=Ge.sundayOfYear;var Je={"-":"",_:" ",0:"0"},qe=/^\s*\d+/,Ve=/^%/;function We(t,e,n){var r=t<0?"-":"",i=(r?-t:t)+"",o=i.length;return r+(o<n?new Array(n-o+1).join(e)+i:i)}function Xe(t){return new RegExp("^(?:"+t.map(o.requote).join("|")+")","i")}function Ze(t){for(var e=new Q,n=-1,r=t.length;++n<r;)e.set(t[n].toLowerCase(),n);return e}function $e(t,e,n){qe.lastIndex=0;var r=qe.exec(e.slice(n,n+1));return r?(t.w=+r[0],n+r[0].length):-1}function tn(t,e,n){qe.lastIndex=0;var r=qe.exec(e.slice(n));return r?(t.U=+r[0],n+r[0].length):-1}function en(t,e,n){qe.lastIndex=0;var r=qe.exec(e.slice(n));return r?(t.W=+r[0],n+r[0].length):-1}function nn(t,e,n){qe.lastIndex=0;var r=qe.exec(e.slice(n,n+4));return r?(t.y=+r[0],n+r[0].length):-1}function rn(t,e,n){qe.lastIndex=0;var r,i=qe.exec(e.slice(n,n+2));return i?(t.y=(r=+i[0])+(r>68?1900:2e3),n+i[0].length):-1}function on(t,e,n){return/^[+-]\d{4}$/.test(e=e.slice(n,n+5))?(t.Z=-e,n+5):-1}function an(t,e,n){qe.lastIndex=0;var r=qe.exec(e.slice(n,n+2));return r?(t.m=r[0]-1,n+r[0].length):-1}function sn(t,e,n){qe.lastIndex=0;var r=qe.exec(e.slice(n,n+2));return r?(t.d=+r[0],n+r[0].length):-1}function An(t,e,n){qe.lastIndex=0;var r=qe.exec(e.slice(n,n+3));return r?(t.j=+r[0],n+r[0].length):-1}function ln(t,e,n){qe.lastIndex=0;var r=qe.exec(e.slice(n,n+2));return r?(t.H=+r[0],n+r[0].length):-1}function cn(t,e,n){qe.lastIndex=0;var r=qe.exec(e.slice(n,n+2));return r?(t.M=+r[0],n+r[0].length):-1}function un(t,e,n){qe.lastIndex=0;var r=qe.exec(e.slice(n,n+2));return r?(t.S=+r[0],n+r[0].length):-1}function gn(t,e,n){qe.lastIndex=0;var r=qe.exec(e.slice(n,n+3));return r?(t.L=+r[0],n+r[0].length):-1}function hn(t){var e=t.getTimezoneOffset(),n=e>0?"-":"+",r=v(e)/60|0,i=v(e)%60;return n+We(r,"0",2)+We(i,"0",2)}function dn(t,e,n){Ve.lastIndex=0;var r=Ve.exec(e.slice(n,n+1));return r?n+r[0].length:-1}function fn(t){for(var e=t.length,n=-1;++n<e;)t[n][0]=this(t[n][0]);return function(e){for(var n=0,r=t[n];!r[1](e);)r=t[++n];return r[0](e)}}o.locale=function(t){return{numberFormat:Fe(t),timeFormat:ze(t)}};var pn=o.locale({decimal:".",thousands:",",grouping:[3],currency:["$",""],dateTime:"%a %b %e %X %Y",date:"%m/%d/%Y",time:"%H:%M:%S",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]});function In(){}o.format=pn.numberFormat,o.geo={},In.prototype={s:0,t:0,add:function(t){mn(t,this.t,Cn),mn(Cn.s,this.s,this),this.s?this.t+=Cn.t:this.s=Cn.t},reset:function(){this.s=this.t=0},valueOf:function(){return this.s}};var Cn=new In;function mn(t,e,n){var r=n.s=t+e,i=r-t,o=r-i;n.t=t-o+(e-i)}function yn(t,e){t&&vn.hasOwnProperty(t.type)&&vn[t.type](t,e)}o.geo.stream=function(t,e){t&&En.hasOwnProperty(t.type)?En[t.type](t,e):yn(t,e)};var En={Feature:function(t,e){yn(t.geometry,e)},FeatureCollection:function(t,e){for(var n=t.features,r=-1,i=n.length;++r<i;)yn(n[r].geometry,e)}},vn={Sphere:function(t,e){e.sphere()},Point:function(t,e){t=t.coordinates,e.point(t[0],t[1],t[2])},MultiPoint:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)t=n[r],e.point(t[0],t[1],t[2])},LineString:function(t,e){Bn(t.coordinates,e,0)},MultiLineString:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)Bn(n[r],e,0)},Polygon:function(t,e){bn(t.coordinates,e)},MultiPolygon:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)bn(n[r],e)},GeometryCollection:function(t,e){for(var n=t.geometries,r=-1,i=n.length;++r<i;)yn(n[r],e)}};function Bn(t,e,n){var r,i=-1,o=t.length-n;for(e.lineStart();++i<o;)r=t[i],e.point(r[0],r[1],r[2]);e.lineEnd()}function bn(t,e){var n=-1,r=t.length;for(e.polygonStart();++n<r;)Bn(t[n],e,1);e.polygonEnd()}o.geo.area=function(t){return Qn=0,o.geo.stream(t,Un),Qn};var Qn,wn,xn,_n,Dn,Mn,kn,Sn,Ln,Rn,Nn,Fn,Tn=new In,Un={sphere:function(){Qn+=4*St},point:G,lineStart:G,lineEnd:G,polygonStart:function(){Tn.reset(),Un.lineStart=On},polygonEnd:function(){var t=2*Tn;Qn+=t<0?4*St+t:t,Un.lineStart=Un.lineEnd=Un.point=G}};function On(){var t,e,n,r,i;function o(t,e){e=e*Ft/2+St/4;var o=(t*=Ft)-n,a=o>=0?1:-1,s=a*o,A=Math.cos(e),l=Math.sin(e),c=i*l,u=r*A+c*Math.cos(s),g=c*a*Math.sin(s);Tn.add(Math.atan2(g,u)),n=t,r=A,i=l}Un.point=function(a,s){Un.point=o,n=(t=a)*Ft,r=Math.cos(s=(e=s)*Ft/2+St/4),i=Math.sin(s)},Un.lineEnd=function(){o(t,e)}}function Gn(t){var e=t[0],n=t[1],r=Math.cos(n);return[r*Math.cos(e),r*Math.sin(e),Math.sin(n)]}function Hn(t,e){return t[0]*e[0]+t[1]*e[1]+t[2]*e[2]}function jn(t,e){return[t[1]*e[2]-t[2]*e[1],t[2]*e[0]-t[0]*e[2],t[0]*e[1]-t[1]*e[0]]}function Yn(t,e){t[0]+=e[0],t[1]+=e[1],t[2]+=e[2]}function Pn(t,e){return[t[0]*e,t[1]*e,t[2]*e]}function Kn(t){var e=Math.sqrt(t[0]*t[0]+t[1]*t[1]+t[2]*t[2]);t[0]/=e,t[1]/=e,t[2]/=e}function zn(t){return[Math.atan2(t[1],t[0]),Ht(t[2])]}function Jn(t,e){return v(t[0]-e[0])<Mt&&v(t[1]-e[1])<Mt}o.geo.bounds=function(){var t,e,n,r,i,a,s,A,l,c,u,g={point:h,lineStart:f,lineEnd:p,polygonStart:function(){g.point=I,g.lineStart=C,g.lineEnd=m,l=0,Un.polygonStart()},polygonEnd:function(){Un.polygonEnd(),g.point=h,g.lineStart=f,g.lineEnd=p,Tn<0?(t=-(n=180),e=-(r=90)):l>Mt?r=90:l<-Mt&&(e=-90),u[0]=t,u[1]=n}};function h(i,o){c.push(u=[t=i,n=i]),o<e&&(e=o),o>r&&(r=o)}function d(o,a){var s=Gn([o*Ft,a*Ft]);if(A){var l=jn(A,s),c=jn([l[1],-l[0],0],l);Kn(c),c=zn(c);var u=o-i,g=u>0?1:-1,d=c[0]*Tt*g,f=v(u)>180;if(f^(g*i<d&&d<g*o))(p=c[1]*Tt)>r&&(r=p);else if(f^(g*i<(d=(d+360)%360-180)&&d<g*o)){var p;(p=-c[1]*Tt)<e&&(e=p)}else a<e&&(e=a),a>r&&(r=a);f?o<i?y(t,o)>y(t,n)&&(n=o):y(o,n)>y(t,n)&&(t=o):n>=t?(o<t&&(t=o),o>n&&(n=o)):o>i?y(t,o)>y(t,n)&&(n=o):y(o,n)>y(t,n)&&(t=o)}else h(o,a);A=s,i=o}function f(){g.point=d}function p(){u[0]=t,u[1]=n,g.point=h,A=null}function I(t,e){if(A){var n=t-i;l+=v(n)>180?n+(n>0?360:-360):n}else a=t,s=e;Un.point(t,e),d(t,e)}function C(){Un.lineStart()}function m(){I(a,s),Un.lineEnd(),v(l)>Mt&&(t=-(n=180)),u[0]=t,u[1]=n,A=null}function y(t,e){return(e-=t)<0?e+360:e}function E(t,e){return t[0]-e[0]}function B(t,e){return e[0]<=e[1]?e[0]<=t&&t<=e[1]:t<e[0]||e[1]<t}return function(i){if(r=n=-(t=e=1/0),c=[],o.geo.stream(i,g),l=c.length){c.sort(E);for(var a=1,s=[f=c[0]];a<l;++a)B((h=c[a])[0],f)||B(h[1],f)?(y(f[0],h[1])>y(f[0],f[1])&&(f[1]=h[1]),y(h[0],f[1])>y(f[0],f[1])&&(f[0]=h[0])):s.push(f=h);for(var A,l,h,d=-1/0,f=(a=0,s[l=s.length-1]);a<=l;f=h,++a)h=s[a],(A=y(f[1],h[0]))>d&&(d=A,t=h[0],n=f[1])}return c=u=null,t===1/0||e===1/0?[[NaN,NaN],[NaN,NaN]]:[[t,e],[n,r]]}}(),o.geo.centroid=function(t){wn=xn=_n=Dn=Mn=kn=Sn=Ln=Rn=Nn=Fn=0,o.geo.stream(t,qn);var e=Rn,n=Nn,r=Fn,i=e*e+n*n+r*r;return i<kt&&(e=kn,n=Sn,r=Ln,xn<Mt&&(e=_n,n=Dn,r=Mn),(i=e*e+n*n+r*r)<kt)?[NaN,NaN]:[Math.atan2(n,e)*Tt,Ht(r/Math.sqrt(i))*Tt]};var qn={sphere:G,point:Vn,lineStart:Xn,lineEnd:Zn,polygonStart:function(){qn.lineStart=$n},polygonEnd:function(){qn.lineStart=Xn}};function Vn(t,e){t*=Ft;var n=Math.cos(e*=Ft);Wn(n*Math.cos(t),n*Math.sin(t),Math.sin(e))}function Wn(t,e,n){_n+=(t-_n)/++wn,Dn+=(e-Dn)/wn,Mn+=(n-Mn)/wn}function Xn(){var t,e,n;function r(r,i){r*=Ft;var o=Math.cos(i*=Ft),a=o*Math.cos(r),s=o*Math.sin(r),A=Math.sin(i),l=Math.atan2(Math.sqrt((l=e*A-n*s)*l+(l=n*a-t*A)*l+(l=t*s-e*a)*l),t*a+e*s+n*A);xn+=l,kn+=l*(t+(t=a)),Sn+=l*(e+(e=s)),Ln+=l*(n+(n=A)),Wn(t,e,n)}qn.point=function(i,o){i*=Ft;var a=Math.cos(o*=Ft);t=a*Math.cos(i),e=a*Math.sin(i),n=Math.sin(o),qn.point=r,Wn(t,e,n)}}function Zn(){qn.point=Vn}function $n(){var t,e,n,r,i;function o(t,e){t*=Ft;var o=Math.cos(e*=Ft),a=o*Math.cos(t),s=o*Math.sin(t),A=Math.sin(e),l=r*A-i*s,c=i*a-n*A,u=n*s-r*a,g=Math.sqrt(l*l+c*c+u*u),h=n*a+r*s+i*A,d=g&&-Gt(h)/g,f=Math.atan2(g,h);Rn+=d*l,Nn+=d*c,Fn+=d*u,xn+=f,kn+=f*(n+(n=a)),Sn+=f*(r+(r=s)),Ln+=f*(i+(i=A)),Wn(n,r,i)}qn.point=function(a,s){t=a,e=s,qn.point=o,a*=Ft;var A=Math.cos(s*=Ft);n=A*Math.cos(a),r=A*Math.sin(a),i=Math.sin(s),Wn(n,r,i)},qn.lineEnd=function(){o(t,e),qn.lineEnd=Zn,qn.point=Vn}}function tr(t,e){function n(n,r){return n=t(n,r),e(n[0],n[1])}return t.invert&&e.invert&&(n.invert=function(n,r){return(n=e.invert(n,r))&&t.invert(n[0],n[1])}),n}function er(){return!0}function nr(t,e,n,r,i){var o=[],a=[];if(t.forEach(function(t){if(!((e=t.length-1)<=0)){var e,n=t[0],r=t[e];if(Jn(n,r)){i.lineStart();for(var s=0;s<e;++s)i.point((n=t[s])[0],n[1]);i.lineEnd()}else{var A=new ir(n,t,null,!0),l=new ir(n,null,A,!1);A.o=l,o.push(A),a.push(l),A=new ir(r,t,null,!1),l=new ir(r,null,A,!0),A.o=l,o.push(A),a.push(l)}}}),a.sort(e),rr(o),rr(a),o.length){for(var s=0,A=n,l=a.length;s<l;++s)a[s].e=A=!A;for(var c,u,g=o[0];;){for(var h=g,d=!0;h.v;)if((h=h.n)===g)return;c=h.z,i.lineStart();do{if(h.v=h.o.v=!0,h.e){if(d)for(s=0,l=c.length;s<l;++s)i.point((u=c[s])[0],u[1]);else r(h.x,h.n.x,1,i);h=h.n}else{if(d)for(s=(c=h.p.z).length-1;s>=0;--s)i.point((u=c[s])[0],u[1]);else r(h.x,h.p.x,-1,i);h=h.p}c=(h=h.o).z,d=!d}while(!h.v);i.lineEnd()}}}function rr(t){if(e=t.length){for(var e,n,r=0,i=t[0];++r<e;)i.n=n=t[r],n.p=i,i=n;i.n=n=t[0],n.p=i}}function ir(t,e,n,r){this.x=t,this.z=e,this.o=n,this.e=r,this.v=!1,this.n=this.p=null}function or(t,e,n,r){return function(i,a){var s,A=e(a),l=i.invert(r[0],r[1]),c={point:u,lineStart:h,lineEnd:d,polygonStart:function(){c.point=y,c.lineStart=E,c.lineEnd=v,s=[],f=[]},polygonEnd:function(){c.point=u,c.lineStart=h,c.lineEnd=d,s=o.merge(s);var t=function(t,e){var n=t[0],r=t[1],i=[Math.sin(n),-Math.cos(n),0],o=0,a=0;Tn.reset();for(var s=0,A=e.length;s<A;++s){var l=e[s],c=l.length;if(c)for(var u=l[0],g=u[0],h=u[1]/2+St/4,d=Math.sin(h),f=Math.cos(h),p=1;;){p===c&&(p=0);var I=(t=l[p])[0],C=t[1]/2+St/4,m=Math.sin(C),y=Math.cos(C),E=I-g,v=E>=0?1:-1,B=v*E,b=B>St,Q=d*m;if(Tn.add(Math.atan2(Q*v*Math.sin(B),f*y+Q*Math.cos(B))),o+=b?E+v*Lt:E,b^g>=n^I>=n){var w=jn(Gn(u),Gn(t));Kn(w);var x=jn(i,w);Kn(x);var _=(b^E>=0?-1:1)*Ht(x[2]);(r>_||r===_&&(w[0]||w[1]))&&(a+=b^E>=0?1:-1)}if(!p++)break;g=I,d=m,f=y,u=t}}return(o<-Mt||o<Mt&&Tn<-Mt)^1&a}(l,f);s.length?(m||(a.polygonStart(),m=!0),nr(s,Ar,t,n,a)):t&&(m||(a.polygonStart(),m=!0),a.lineStart(),n(null,null,1,a),a.lineEnd()),m&&(a.polygonEnd(),m=!1),s=f=null},sphere:function(){a.polygonStart(),a.lineStart(),n(null,null,1,a),a.lineEnd(),a.polygonEnd()}};function u(e,n){var r=i(e,n);t(e=r[0],n=r[1])&&a.point(e,n)}function g(t,e){var n=i(t,e);A.point(n[0],n[1])}function h(){c.point=g,A.lineStart()}function d(){c.point=u,A.lineEnd()}var f,p,I=sr(),C=e(I),m=!1;function y(t,e){p.push([t,e]);var n=i(t,e);C.point(n[0],n[1])}function E(){C.lineStart(),p=[]}function v(){y(p[0][0],p[0][1]),C.lineEnd();var t,e=C.clean(),n=I.buffer(),r=n.length;if(p.pop(),f.push(p),p=null,r)if(1&e){var i,o=-1;if((r=(t=n[0]).length-1)>0){for(m||(a.polygonStart(),m=!0),a.lineStart();++o<r;)a.point((i=t[o])[0],i[1]);a.lineEnd()}}else r>1&&2&e&&n.push(n.pop().concat(n.shift())),s.push(n.filter(ar))}return c}}function ar(t){return t.length>1}function sr(){var t,e=[];return{lineStart:function(){e.push(t=[])},point:function(e,n){t.push([e,n])},lineEnd:G,buffer:function(){var n=e;return e=[],t=null,n},rejoin:function(){e.length>1&&e.push(e.pop().concat(e.shift()))}}}function Ar(t,e){return((t=t.x)[0]<0?t[1]-Nt-Mt:Nt-t[1])-((e=e.x)[0]<0?e[1]-Nt-Mt:Nt-e[1])}var lr=or(er,function(t){var e,n=NaN,r=NaN,i=NaN;return{lineStart:function(){t.lineStart(),e=1},point:function(o,a){var s=o>0?St:-St,A=v(o-n);v(A-St)<Mt?(t.point(n,r=(r+a)/2>0?Nt:-Nt),t.point(i,r),t.lineEnd(),t.lineStart(),t.point(s,r),t.point(o,r),e=0):i!==s&&A>=St&&(v(n-i)<Mt&&(n-=i*Mt),v(o-s)<Mt&&(o-=s*Mt),r=function(t,e,n,r){var i,o,a=Math.sin(t-n);return v(a)>Mt?Math.atan((Math.sin(e)*(o=Math.cos(r))*Math.sin(n)-Math.sin(r)*(i=Math.cos(e))*Math.sin(t))/(i*o*a)):(e+r)/2}(n,r,o,a),t.point(i,r),t.lineEnd(),t.lineStart(),t.point(s,r),e=0),t.point(n=o,r=a),i=s},lineEnd:function(){t.lineEnd(),n=r=NaN},clean:function(){return 2-e}}},function(t,e,n,r){var i;if(null==t)i=n*Nt,r.point(-St,i),r.point(0,i),r.point(St,i),r.point(St,0),r.point(St,-i),r.point(0,-i),r.point(-St,-i),r.point(-St,0),r.point(-St,i);else if(v(t[0]-e[0])>Mt){var o=t[0]<e[0]?St:-St;i=n*o/2,r.point(-o,i),r.point(0,i),r.point(o,i)}else r.point(e[0],e[1])},[-St,-St/2]);function cr(t){var e=Math.cos(t),n=e>0,r=v(e)>Mt;return or(i,function(t){var e,s,A,l,c;return{lineStart:function(){l=A=!1,c=1},point:function(u,g){var h,d=[u,g],f=i(u,g),p=n?f?0:a(u,g):f?a(u+(u<0?St:-St),g):0;if(!e&&(l=A=f)&&t.lineStart(),f!==A&&(h=o(e,d),(Jn(e,h)||Jn(d,h))&&(d[0]+=Mt,d[1]+=Mt,f=i(d[0],d[1]))),f!==A)c=0,f?(t.lineStart(),h=o(d,e),t.point(h[0],h[1])):(h=o(e,d),t.point(h[0],h[1]),t.lineEnd()),e=h;else if(r&&e&&n^f){var I;p&s||!(I=o(d,e,!0))||(c=0,n?(t.lineStart(),t.point(I[0][0],I[0][1]),t.point(I[1][0],I[1][1]),t.lineEnd()):(t.point(I[1][0],I[1][1]),t.lineEnd(),t.lineStart(),t.point(I[0][0],I[0][1])))}!f||e&&Jn(e,d)||t.point(d[0],d[1]),e=d,A=f,s=p},lineEnd:function(){A&&t.lineEnd(),e=null},clean:function(){return c|(l&&A)<<1}}},Jr(t,6*Ft),n?[0,-t]:[-St,t-St]);function i(t,n){return Math.cos(t)*Math.cos(n)>e}function o(t,n,r){var i=[1,0,0],o=jn(Gn(t),Gn(n)),a=Hn(o,o),s=o[0],A=a-s*s;if(!A)return!r&&t;var l=e*a/A,c=-e*s/A,u=jn(i,o),g=Pn(i,l);Yn(g,Pn(o,c));var h=u,d=Hn(g,h),f=Hn(h,h),p=d*d-f*(Hn(g,g)-1);if(!(p<0)){var I=Math.sqrt(p),C=Pn(h,(-d-I)/f);if(Yn(C,g),C=zn(C),!r)return C;var m,y=t[0],E=n[0],B=t[1],b=n[1];E<y&&(m=y,y=E,E=m);var Q=E-y,w=v(Q-St)<Mt;if(!w&&b<B&&(m=B,B=b,b=m),w||Q<Mt?w?B+b>0^C[1]<(v(C[0]-y)<Mt?B:b):B<=C[1]&&C[1]<=b:Q>St^(y<=C[0]&&C[0]<=E)){var x=Pn(h,(-d+I)/f);return Yn(x,g),[C,zn(x)]}}}function a(e,r){var i=n?t:St-t,o=0;return e<-i?o|=1:e>i&&(o|=2),r<-i?o|=4:r>i&&(o|=8),o}}function ur(t,e,n,r){return function(i){var o,a=i.a,s=i.b,A=a.x,l=a.y,c=0,u=1,g=s.x-A,h=s.y-l;if(o=t-A,g||!(o>0)){if(o/=g,g<0){if(o<c)return;o<u&&(u=o)}else if(g>0){if(o>u)return;o>c&&(c=o)}if(o=n-A,g||!(o<0)){if(o/=g,g<0){if(o>u)return;o>c&&(c=o)}else if(g>0){if(o<c)return;o<u&&(u=o)}if(o=e-l,h||!(o>0)){if(o/=h,h<0){if(o<c)return;o<u&&(u=o)}else if(h>0){if(o>u)return;o>c&&(c=o)}if(o=r-l,h||!(o<0)){if(o/=h,h<0){if(o>u)return;o>c&&(c=o)}else if(h>0){if(o<c)return;o<u&&(u=o)}return c>0&&(i.a={x:A+c*g,y:l+c*h}),u<1&&(i.b={x:A+u*g,y:l+u*h}),i}}}}}}var gr=1e9;function hr(t,e,n,r){return function(A){var l,c,u,g,h,d,f,p,I,C,m,y=A,E=sr(),v=ur(t,e,n,r),B={point:w,lineStart:function(){B.point=x,c&&c.push(u=[]),C=!0,I=!1,f=p=NaN},lineEnd:function(){l&&(x(g,h),d&&I&&E.rejoin(),l.push(E.buffer())),B.point=w,I&&A.lineEnd()},polygonStart:function(){A=E,l=[],c=[],m=!0},polygonEnd:function(){A=y,l=o.merge(l);var e=function(t){for(var e=0,n=c.length,r=t[1],i=0;i<n;++i)for(var o,a=1,s=c[i],A=s.length,l=s[0];a<A;++a)o=s[a],l[1]<=r?o[1]>r&&Ot(l,o,t)>0&&++e:o[1]<=r&&Ot(l,o,t)<0&&--e,l=o;return 0!==e}([t,r]),n=m&&e,i=l.length;(n||i)&&(A.polygonStart(),n&&(A.lineStart(),b(null,null,1,A),A.lineEnd()),i&&nr(l,a,e,b,A),A.polygonEnd()),l=c=u=null}};function b(o,a,A,l){var c=0,u=0;if(null==o||(c=i(o,A))!==(u=i(a,A))||s(o,a)<0^A>0)do{l.point(0===c||3===c?t:n,c>1?r:e)}while((c=(c+A+4)%4)!==u);else l.point(a[0],a[1])}function Q(i,o){return t<=i&&i<=n&&e<=o&&o<=r}function w(t,e){Q(t,e)&&A.point(t,e)}function x(t,e){var n=Q(t=Math.max(-gr,Math.min(gr,t)),e=Math.max(-gr,Math.min(gr,e)));if(c&&u.push([t,e]),C)g=t,h=e,d=n,C=!1,n&&(A.lineStart(),A.point(t,e));else if(n&&I)A.point(t,e);else{var r={a:{x:f,y:p},b:{x:t,y:e}};v(r)?(I||(A.lineStart(),A.point(r.a.x,r.a.y)),A.point(r.b.x,r.b.y),n||A.lineEnd(),m=!1):n&&(A.lineStart(),A.point(t,e),m=!1)}f=t,p=e,I=n}return B};function i(r,i){return v(r[0]-t)<Mt?i>0?0:3:v(r[0]-n)<Mt?i>0?2:1:v(r[1]-e)<Mt?i>0?1:0:i>0?3:2}function a(t,e){return s(t.x,e.x)}function s(t,e){var n=i(t,1),r=i(e,1);return n!==r?n-r:0===n?e[1]-t[1]:1===n?t[0]-e[0]:2===n?t[1]-e[1]:e[0]-t[0]}}function dr(t){var e=0,n=St/3,r=Or(t),i=r(e,n);return i.parallels=function(t){return arguments.length?r(e=t[0]*St/180,n=t[1]*St/180):[e/St*180,n/St*180]},i}function fr(t,e){var n=Math.sin(t),r=(n+Math.sin(e))/2,i=1+n*(2*r-n),o=Math.sqrt(i)/r;function a(t,e){var n=Math.sqrt(i-2*r*Math.sin(e))/r;return[n*Math.sin(t*=r),o-n*Math.cos(t)]}return a.invert=function(t,e){var n=o-e;return[Math.atan2(t,n)/r,Ht((i-(t*t+n*n)*r*r)/(2*r))]},a}o.geo.clipExtent=function(){var t,e,n,r,i,o,a={stream:function(t){return i&&(i.valid=!1),(i=o(t)).valid=!0,i},extent:function(s){return arguments.length?(o=hr(t=+s[0][0],e=+s[0][1],n=+s[1][0],r=+s[1][1]),i&&(i.valid=!1,i=null),a):[[t,e],[n,r]]}};return a.extent([[0,0],[960,500]])},(o.geo.conicEqualArea=function(){return dr(fr)}).raw=fr,o.geo.albers=function(){return o.geo.conicEqualArea().rotate([96,0]).center([-.6,38.7]).parallels([29.5,45.5]).scale(1070)},o.geo.albersUsa=function(){var t,e,n,r,i=o.geo.albers(),a=o.geo.conicEqualArea().rotate([154,0]).center([-2,58.5]).parallels([55,65]),s=o.geo.conicEqualArea().rotate([157,0]).center([-3,19.9]).parallels([8,18]),A={point:function(e,n){t=[e,n]}};function l(i){var o=i[0],a=i[1];return t=null,e(o,a),t||(n(o,a),t)||r(o,a),t}return l.invert=function(t){var e=i.scale(),n=i.translate(),r=(t[0]-n[0])/e,o=(t[1]-n[1])/e;return(o>=.12&&o<.234&&r>=-.425&&r<-.214?a:o>=.166&&o<.234&&r>=-.214&&r<-.115?s:i).invert(t)},l.stream=function(t){var e=i.stream(t),n=a.stream(t),r=s.stream(t);return{point:function(t,i){e.point(t,i),n.point(t,i),r.point(t,i)},sphere:function(){e.sphere(),n.sphere(),r.sphere()},lineStart:function(){e.lineStart(),n.lineStart(),r.lineStart()},lineEnd:function(){e.lineEnd(),n.lineEnd(),r.lineEnd()},polygonStart:function(){e.polygonStart(),n.polygonStart(),r.polygonStart()},polygonEnd:function(){e.polygonEnd(),n.polygonEnd(),r.polygonEnd()}}},l.precision=function(t){return arguments.length?(i.precision(t),a.precision(t),s.precision(t),l):i.precision()},l.scale=function(t){return arguments.length?(i.scale(t),a.scale(.35*t),s.scale(t),l.translate(i.translate())):i.scale()},l.translate=function(t){if(!arguments.length)return i.translate();var o=i.scale(),c=+t[0],u=+t[1];return e=i.translate(t).clipExtent([[c-.455*o,u-.238*o],[c+.455*o,u+.238*o]]).stream(A).point,n=a.translate([c-.307*o,u+.201*o]).clipExtent([[c-.425*o+Mt,u+.12*o+Mt],[c-.214*o-Mt,u+.234*o-Mt]]).stream(A).point,r=s.translate([c-.205*o,u+.212*o]).clipExtent([[c-.214*o+Mt,u+.166*o+Mt],[c-.115*o-Mt,u+.234*o-Mt]]).stream(A).point,l},l.scale(1070)};var pr,Ir,Cr,mr,yr,Er,vr={point:G,lineStart:G,lineEnd:G,polygonStart:function(){Ir=0,vr.lineStart=Br},polygonEnd:function(){vr.lineStart=vr.lineEnd=vr.point=G,pr+=v(Ir/2)}};function Br(){var t,e,n,r;function i(t,e){Ir+=r*t-n*e,n=t,r=e}vr.point=function(o,a){vr.point=i,t=n=o,e=r=a},vr.lineEnd=function(){i(t,e)}}var br={point:function(t,e){t<Cr&&(Cr=t),t>yr&&(yr=t),e<mr&&(mr=e),e>Er&&(Er=e)},lineStart:G,lineEnd:G,polygonStart:G,polygonEnd:G};function Qr(){var t=wr(4.5),e=[],n={point:r,lineStart:function(){n.point=i},lineEnd:a,polygonStart:function(){n.lineEnd=s},polygonEnd:function(){n.lineEnd=a,n.point=r},pointRadius:function(e){return t=wr(e),n},result:function(){if(e.length){var t=e.join("");return e=[],t}}};function r(n,r){e.push("M",n,",",r,t)}function i(t,r){e.push("M",t,",",r),n.point=o}function o(t,n){e.push("L",t,",",n)}function a(){n.point=r}function s(){e.push("Z")}return n}function wr(t){return"m0,"+t+"a"+t+","+t+" 0 1,1 0,"+-2*t+"a"+t+","+t+" 0 1,1 0,"+2*t+"z"}var xr,_r={point:Dr,lineStart:Mr,lineEnd:kr,polygonStart:function(){_r.lineStart=Sr},polygonEnd:function(){_r.point=Dr,_r.lineStart=Mr,_r.lineEnd=kr}};function Dr(t,e){_n+=t,Dn+=e,++Mn}function Mr(){var t,e;function n(n,r){var i=n-t,o=r-e,a=Math.sqrt(i*i+o*o);kn+=a*(t+n)/2,Sn+=a*(e+r)/2,Ln+=a,Dr(t=n,e=r)}_r.point=function(r,i){_r.point=n,Dr(t=r,e=i)}}function kr(){_r.point=Dr}function Sr(){var t,e,n,r;function i(t,e){var i=t-n,o=e-r,a=Math.sqrt(i*i+o*o);kn+=a*(n+t)/2,Sn+=a*(r+e)/2,Ln+=a,Rn+=(a=r*t-n*e)*(n+t),Nn+=a*(r+e),Fn+=3*a,Dr(n=t,r=e)}_r.point=function(o,a){_r.point=i,Dr(t=n=o,e=r=a)},_r.lineEnd=function(){i(t,e)}}function Lr(t){var e=4.5,n={point:r,lineStart:function(){n.point=i},lineEnd:a,polygonStart:function(){n.lineEnd=s},polygonEnd:function(){n.lineEnd=a,n.point=r},pointRadius:function(t){return e=t,n},result:G};function r(n,r){t.moveTo(n+e,r),t.arc(n,r,e,0,Lt)}function i(e,r){t.moveTo(e,r),n.point=o}function o(e,n){t.lineTo(e,n)}function a(){n.point=r}function s(){t.closePath()}return n}function Rr(t){var e=.5,n=Math.cos(30*Ft),r=16;function i(e){return(r?function(e){var n,i,a,s,A,l,c,u,g,h,d,f,p={point:I,lineStart:C,lineEnd:y,polygonStart:function(){e.polygonStart(),p.lineStart=E},polygonEnd:function(){e.polygonEnd(),p.lineStart=C}};function I(n,r){n=t(n,r),e.point(n[0],n[1])}function C(){u=NaN,p.point=m,e.lineStart()}function m(n,i){var a=Gn([n,i]),s=t(n,i);o(u,g,c,h,d,f,u=s[0],g=s[1],c=n,h=a[0],d=a[1],f=a[2],r,e),e.point(u,g)}function y(){p.point=I,e.lineEnd()}function E(){C(),p.point=v,p.lineEnd=B}function v(t,e){m(n=t,e),i=u,a=g,s=h,A=d,l=f,p.point=m}function B(){o(u,g,c,h,d,f,i,a,n,s,A,l,r,e),p.lineEnd=y,y()}return p}:function(e){return Tr(e,function(n,r){n=t(n,r),e.point(n[0],n[1])})})(e)}function o(r,i,a,s,A,l,c,u,g,h,d,f,p,I){var C=c-r,m=u-i,y=C*C+m*m;if(y>4*e&&p--){var E=s+h,B=A+d,b=l+f,Q=Math.sqrt(E*E+B*B+b*b),w=Math.asin(b/=Q),x=v(v(b)-1)<Mt||v(a-g)<Mt?(a+g)/2:Math.atan2(B,E),_=t(x,w),D=_[0],M=_[1],k=D-r,S=M-i,L=m*k-C*S;(L*L/y>e||v((C*k+m*S)/y-.5)>.3||s*h+A*d+l*f<n)&&(o(r,i,a,s,A,l,D,M,x,E/=Q,B/=Q,b,p,I),I.point(D,M),o(D,M,x,E,B,b,c,u,g,h,d,f,p,I))}}return i.precision=function(t){return arguments.length?(r=(e=t*t)>0&&16,i):Math.sqrt(e)},i}function Nr(t){var e=Rr(function(e,n){return t([e*Tt,n*Tt])});return function(t){return Gr(e(t))}}function Fr(t){this.stream=t}function Tr(t,e){return{point:e,sphere:function(){t.sphere()},lineStart:function(){t.lineStart()},lineEnd:function(){t.lineEnd()},polygonStart:function(){t.polygonStart()},polygonEnd:function(){t.polygonEnd()}}}function Ur(t){return Or(function(){return t})()}function Or(t){var e,n,r,i,a,s,A=Rr(function(t,n){return[(t=e(t,n))[0]*l+i,a-t[1]*l]}),l=150,c=480,u=250,g=0,h=0,d=0,f=0,p=0,I=lr,C=F,m=null,y=null;function E(t){return[(t=r(t[0]*Ft,t[1]*Ft))[0]*l+i,a-t[1]*l]}function v(t){return(t=r.invert((t[0]-i)/l,(a-t[1])/l))&&[t[0]*Tt,t[1]*Tt]}function B(){r=tr(n=Yr(d,f,p),e);var t=e(g,h);return i=c-t[0]*l,a=u+t[1]*l,b()}function b(){return s&&(s.valid=!1,s=null),E}return E.stream=function(t){return s&&(s.valid=!1),(s=Gr(I(n,A(C(t))))).valid=!0,s},E.clipAngle=function(t){return arguments.length?(I=null==t?(m=t,lr):cr((m=+t)*Ft),b()):m},E.clipExtent=function(t){return arguments.length?(y=t,C=t?hr(t[0][0],t[0][1],t[1][0],t[1][1]):F,b()):y},E.scale=function(t){return arguments.length?(l=+t,B()):l},E.translate=function(t){return arguments.length?(c=+t[0],u=+t[1],B()):[c,u]},E.center=function(t){return arguments.length?(g=t[0]%360*Ft,h=t[1]%360*Ft,B()):[g*Tt,h*Tt]},E.rotate=function(t){return arguments.length?(d=t[0]%360*Ft,f=t[1]%360*Ft,p=t.length>2?t[2]%360*Ft:0,B()):[d*Tt,f*Tt,p*Tt]},o.rebind(E,A,"precision"),function(){return e=t.apply(this,arguments),E.invert=e.invert&&v,B()}}function Gr(t){return Tr(t,function(e,n){t.point(e*Ft,n*Ft)})}function Hr(t,e){return[t,e]}function jr(t,e){return[t>St?t-Lt:t<-St?t+Lt:t,e]}function Yr(t,e,n){return t?e||n?tr(Kr(t),zr(e,n)):Kr(t):e||n?zr(e,n):jr}function Pr(t){return function(e,n){return[(e+=t)>St?e-Lt:e<-St?e+Lt:e,n]}}function Kr(t){var e=Pr(t);return e.invert=Pr(-t),e}function zr(t,e){var n=Math.cos(t),r=Math.sin(t),i=Math.cos(e),o=Math.sin(e);function a(t,e){var a=Math.cos(e),s=Math.cos(t)*a,A=Math.sin(t)*a,l=Math.sin(e),c=l*n+s*r;return[Math.atan2(A*i-c*o,s*n-l*r),Ht(c*i+A*o)]}return a.invert=function(t,e){var a=Math.cos(e),s=Math.cos(t)*a,A=Math.sin(t)*a,l=Math.sin(e),c=l*i-A*o;return[Math.atan2(A*i+l*o,s*n+c*r),Ht(c*n-s*r)]},a}function Jr(t,e){var n=Math.cos(t),r=Math.sin(t);return function(i,o,a,s){var A=a*e;null!=i?(i=qr(n,i),o=qr(n,o),(a>0?i<o:i>o)&&(i+=a*Lt)):(i=t+a*Lt,o=t-.5*A);for(var l,c=i;a>0?c>o:c<o;c-=A)s.point((l=zn([n,-r*Math.cos(c),-r*Math.sin(c)]))[0],l[1])}}function qr(t,e){var n=Gn(e);n[0]-=t,Kn(n);var r=Gt(-n[1]);return((-n[2]<0?-r:r)+2*Math.PI-Mt)%(2*Math.PI)}function Vr(t,e,n){var r=o.range(t,e-Mt,n).concat(e);return function(t){return r.map(function(e){return[t,e]})}}function Wr(t,e,n){var r=o.range(t,e-Mt,n).concat(e);return function(t){return r.map(function(e){return[e,t]})}}function Xr(t){return t.source}function Zr(t){return t.target}o.geo.path=function(){var t,e,n,r,i,a=4.5;function s(t){return t&&("function"==typeof a&&r.pointRadius(+a.apply(this,arguments)),i&&i.valid||(i=n(r)),o.geo.stream(t,i)),r.result()}function A(){return i=null,s}return s.area=function(t){return pr=0,o.geo.stream(t,n(vr)),pr},s.centroid=function(t){return _n=Dn=Mn=kn=Sn=Ln=Rn=Nn=Fn=0,o.geo.stream(t,n(_r)),Fn?[Rn/Fn,Nn/Fn]:Ln?[kn/Ln,Sn/Ln]:Mn?[_n/Mn,Dn/Mn]:[NaN,NaN]},s.bounds=function(t){return yr=Er=-(Cr=mr=1/0),o.geo.stream(t,n(br)),[[Cr,mr],[yr,Er]]},s.projection=function(e){return arguments.length?(n=(t=e)?e.stream||Nr(e):F,A()):t},s.context=function(t){return arguments.length?(r=null==(e=t)?new Qr:new Lr(t),"function"!=typeof a&&r.pointRadius(a),A()):e},s.pointRadius=function(t){return arguments.length?(a="function"==typeof t?t:(r.pointRadius(+t),+t),s):a},s.projection(o.geo.albersUsa()).context(null)},o.geo.transform=function(t){return{stream:function(e){var n=new Fr(e);for(var r in t)n[r]=t[r];return n}}},Fr.prototype={point:function(t,e){this.stream.point(t,e)},sphere:function(){this.stream.sphere()},lineStart:function(){this.stream.lineStart()},lineEnd:function(){this.stream.lineEnd()},polygonStart:function(){this.stream.polygonStart()},polygonEnd:function(){this.stream.polygonEnd()}},o.geo.projection=Ur,o.geo.projectionMutator=Or,(o.geo.equirectangular=function(){return Ur(Hr)}).raw=Hr.invert=Hr,o.geo.rotation=function(t){function e(e){return(e=t(e[0]*Ft,e[1]*Ft))[0]*=Tt,e[1]*=Tt,e}return t=Yr(t[0]%360*Ft,t[1]*Ft,t.length>2?t[2]*Ft:0),e.invert=function(e){return(e=t.invert(e[0]*Ft,e[1]*Ft))[0]*=Tt,e[1]*=Tt,e},e},jr.invert=Hr,o.geo.circle=function(){var t,e,n=[0,0],r=6;function i(){var t="function"==typeof n?n.apply(this,arguments):n,r=Yr(-t[0]*Ft,-t[1]*Ft,0).invert,i=[];return e(null,null,1,{point:function(t,e){i.push(t=r(t,e)),t[0]*=Tt,t[1]*=Tt}}),{type:"Polygon",coordinates:[i]}}return i.origin=function(t){return arguments.length?(n=t,i):n},i.angle=function(n){return arguments.length?(e=Jr((t=+n)*Ft,r*Ft),i):t},i.precision=function(n){return arguments.length?(e=Jr(t*Ft,(r=+n)*Ft),i):r},i.angle(90)},o.geo.distance=function(t,e){var n,r=(e[0]-t[0])*Ft,i=t[1]*Ft,o=e[1]*Ft,a=Math.sin(r),s=Math.cos(r),A=Math.sin(i),l=Math.cos(i),c=Math.sin(o),u=Math.cos(o);return Math.atan2(Math.sqrt((n=u*a)*n+(n=l*c-A*u*s)*n),A*c+l*u*s)},o.geo.graticule=function(){var t,e,n,r,i,a,s,A,l,c,u,g,h=10,d=h,f=90,p=360,I=2.5;function C(){return{type:"MultiLineString",coordinates:m()}}function m(){return o.range(Math.ceil(r/f)*f,n,f).map(u).concat(o.range(Math.ceil(A/p)*p,s,p).map(g)).concat(o.range(Math.ceil(e/h)*h,t,h).filter(function(t){return v(t%f)>Mt}).map(l)).concat(o.range(Math.ceil(a/d)*d,i,d).filter(function(t){return v(t%p)>Mt}).map(c))}return C.lines=function(){return m().map(function(t){return{type:"LineString",coordinates:t}})},C.outline=function(){return{type:"Polygon",coordinates:[u(r).concat(g(s).slice(1),u(n).reverse().slice(1),g(A).reverse().slice(1))]}},C.extent=function(t){return arguments.length?C.majorExtent(t).minorExtent(t):C.minorExtent()},C.majorExtent=function(t){return arguments.length?(r=+t[0][0],n=+t[1][0],A=+t[0][1],s=+t[1][1],r>n&&(t=r,r=n,n=t),A>s&&(t=A,A=s,s=t),C.precision(I)):[[r,A],[n,s]]},C.minorExtent=function(n){return arguments.length?(e=+n[0][0],t=+n[1][0],a=+n[0][1],i=+n[1][1],e>t&&(n=e,e=t,t=n),a>i&&(n=a,a=i,i=n),C.precision(I)):[[e,a],[t,i]]},C.step=function(t){return arguments.length?C.majorStep(t).minorStep(t):C.minorStep()},C.majorStep=function(t){return arguments.length?(f=+t[0],p=+t[1],C):[f,p]},C.minorStep=function(t){return arguments.length?(h=+t[0],d=+t[1],C):[h,d]},C.precision=function(o){return arguments.length?(I=+o,l=Vr(a,i,90),c=Wr(e,t,I),u=Vr(A,s,90),g=Wr(r,n,I),C):I},C.majorExtent([[-180,-90+Mt],[180,90-Mt]]).minorExtent([[-180,-80-Mt],[180,80+Mt]])},o.geo.greatArc=function(){var t,e,n=Xr,r=Zr;function i(){return{type:"LineString",coordinates:[t||n.apply(this,arguments),e||r.apply(this,arguments)]}}return i.distance=function(){return o.geo.distance(t||n.apply(this,arguments),e||r.apply(this,arguments))},i.source=function(e){return arguments.length?(n=e,t="function"==typeof e?null:e,i):n},i.target=function(t){return arguments.length?(r=t,e="function"==typeof t?null:t,i):r},i.precision=function(){return arguments.length?i:0},i},o.geo.interpolate=function(t,e){return function(t,e,n,r){var i=Math.cos(e),o=Math.sin(e),a=Math.cos(r),s=Math.sin(r),A=i*Math.cos(t),l=i*Math.sin(t),c=a*Math.cos(n),u=a*Math.sin(n),g=2*Math.asin(Math.sqrt(Yt(r-e)+i*a*Yt(n-t))),h=1/Math.sin(g),d=g?function(t){var e=Math.sin(t*=g)*h,n=Math.sin(g-t)*h,r=n*A+e*c,i=n*l+e*u,a=n*o+e*s;return[Math.atan2(i,r)*Tt,Math.atan2(a,Math.sqrt(r*r+i*i))*Tt]}:function(){return[t*Tt,e*Tt]};return d.distance=g,d}(t[0]*Ft,t[1]*Ft,e[0]*Ft,e[1]*Ft)},o.geo.length=function(t){return xr=0,o.geo.stream(t,$r),xr};var $r={sphere:G,point:G,lineStart:function(){var t,e,n;function r(r,i){var o=Math.sin(i*=Ft),a=Math.cos(i),s=v((r*=Ft)-t),A=Math.cos(s);xr+=Math.atan2(Math.sqrt((s=a*Math.sin(s))*s+(s=n*o-e*a*A)*s),e*o+n*a*A),t=r,e=o,n=a}$r.point=function(i,o){t=i*Ft,e=Math.sin(o*=Ft),n=Math.cos(o),$r.point=r},$r.lineEnd=function(){$r.point=$r.lineEnd=G}},lineEnd:G,polygonStart:G,polygonEnd:G};function ti(t,e){function n(e,n){var r=Math.cos(e),i=Math.cos(n),o=t(r*i);return[o*i*Math.sin(e),o*Math.sin(n)]}return n.invert=function(t,n){var r=Math.sqrt(t*t+n*n),i=e(r),o=Math.sin(i),a=Math.cos(i);return[Math.atan2(t*o,r*a),Math.asin(r&&n*o/r)]},n}var ei=ti(function(t){return Math.sqrt(2/(1+t))},function(t){return 2*Math.asin(t/2)});(o.geo.azimuthalEqualArea=function(){return Ur(ei)}).raw=ei;var ni=ti(function(t){var e=Math.acos(t);return e&&e/Math.sin(e)},F);function ri(t,e){var n=Math.cos(t),r=function(t){return Math.tan(St/4+t/2)},i=t===e?Math.sin(t):Math.log(n/Math.cos(e))/Math.log(r(e)/r(t)),o=n*Math.pow(r(t),i)/i;if(!i)return ai;function a(t,e){o>0?e<-Nt+Mt&&(e=-Nt+Mt):e>Nt-Mt&&(e=Nt-Mt);var n=o/Math.pow(r(e),i);return[n*Math.sin(i*t),o-n*Math.cos(i*t)]}return a.invert=function(t,e){var n=o-e,r=Ut(i)*Math.sqrt(t*t+n*n);return[Math.atan2(t,n)/i,2*Math.atan(Math.pow(o/r,1/i))-Nt]},a}function ii(t,e){var n=Math.cos(t),r=t===e?Math.sin(t):(n-Math.cos(e))/(e-t),i=n/r+t;if(v(r)<Mt)return Hr;function o(t,e){var n=i-e;return[n*Math.sin(r*t),i-n*Math.cos(r*t)]}return o.invert=function(t,e){var n=i-e;return[Math.atan2(t,n)/r,i-Ut(r)*Math.sqrt(t*t+n*n)]},o}(o.geo.azimuthalEquidistant=function(){return Ur(ni)}).raw=ni,(o.geo.conicConformal=function(){return dr(ri)}).raw=ri,(o.geo.conicEquidistant=function(){return dr(ii)}).raw=ii;var oi=ti(function(t){return 1/t},Math.atan);function ai(t,e){return[t,Math.log(Math.tan(St/4+e/2))]}function si(t){var e,n=Ur(t),r=n.scale,i=n.translate,o=n.clipExtent;return n.scale=function(){var t=r.apply(n,arguments);return t===n?e?n.clipExtent(null):n:t},n.translate=function(){var t=i.apply(n,arguments);return t===n?e?n.clipExtent(null):n:t},n.clipExtent=function(t){var a=o.apply(n,arguments);if(a===n){if(e=null==t){var s=St*r(),A=i();o([[A[0]-s,A[1]-s],[A[0]+s,A[1]+s]])}}else e&&(a=null);return a},n.clipExtent(null)}(o.geo.gnomonic=function(){return Ur(oi)}).raw=oi,ai.invert=function(t,e){return[t,2*Math.atan(Math.exp(e))-Nt]},(o.geo.mercator=function(){return si(ai)}).raw=ai;var Ai=ti(function(){return 1},Math.asin);(o.geo.orthographic=function(){return Ur(Ai)}).raw=Ai;var li=ti(function(t){return 1/(1+t)},function(t){return 2*Math.atan(t)});function ci(t,e){return[Math.log(Math.tan(St/4+e/2)),-t]}function ui(t){return t[0]}function gi(t){return t[1]}function hi(t){for(var e=t.length,n=[0,1],r=2,i=2;i<e;i++){for(;r>1&&Ot(t[n[r-2]],t[n[r-1]],t[i])<=0;)--r;n[r++]=i}return n.slice(0,r)}function di(t,e){return t[0]-e[0]||t[1]-e[1]}(o.geo.stereographic=function(){return Ur(li)}).raw=li,ci.invert=function(t,e){return[-e,2*Math.atan(Math.exp(t))-Nt]},(o.geo.transverseMercator=function(){var t=si(ci),e=t.center,n=t.rotate;return t.center=function(t){return t?e([-t[1],t[0]]):[(t=e())[1],-t[0]]},t.rotate=function(t){return t?n([t[0],t[1],t.length>2?t[2]+90:90]):[(t=n())[0],t[1],t[2]-90]},n([0,0,90])}).raw=ci,o.geom={},o.geom.hull=function(t){var e=ui,n=gi;if(arguments.length)return r(t);function r(t){if(t.length<3)return[];var r,i=ve(e),o=ve(n),a=t.length,s=[],A=[];for(r=0;r<a;r++)s.push([+i.call(this,t[r],r),+o.call(this,t[r],r),r]);for(s.sort(di),r=0;r<a;r++)A.push([s[r][0],-s[r][1]]);var l=hi(s),c=hi(A),u=c[0]===l[0],g=c[c.length-1]===l[l.length-1],h=[];for(r=l.length-1;r>=0;--r)h.push(t[s[l[r]][2]]);for(r=+u;r<c.length-g;++r)h.push(t[s[c[r]][2]]);return h}return r.x=function(t){return arguments.length?(e=t,r):e},r.y=function(t){return arguments.length?(n=t,r):n},r},o.geom.polygon=function(t){return J(t,fi),t};var fi=o.geom.polygon.prototype=[];function pi(t,e,n){return(n[0]-e[0])*(t[1]-e[1])<(n[1]-e[1])*(t[0]-e[0])}function Ii(t,e,n,r){var i=t[0],o=n[0],a=e[0]-i,s=r[0]-o,A=t[1],l=n[1],c=e[1]-A,u=r[1]-l,g=(s*(A-l)-u*(i-o))/(u*a-s*c);return[i+g*a,A+g*c]}function Ci(t){var e=t[0],n=t[t.length-1];return!(e[0]-n[0]||e[1]-n[1])}fi.area=function(){for(var t,e=-1,n=this.length,r=this[n-1],i=0;++e<n;)t=r,r=this[e],i+=t[1]*r[0]-t[0]*r[1];return.5*i},fi.centroid=function(t){var e,n,r=-1,i=this.length,o=0,a=0,s=this[i-1];for(arguments.length||(t=-1/(6*this.area()));++r<i;)e=s,s=this[r],n=e[0]*s[1]-s[0]*e[1],o+=(e[0]+s[0])*n,a+=(e[1]+s[1])*n;return[o*t,a*t]},fi.clip=function(t){for(var e,n,r,i,o,a,s=Ci(t),A=-1,l=this.length-Ci(this),c=this[l-1];++A<l;){for(e=t.slice(),t.length=0,i=this[A],o=e[(r=e.length-s)-1],n=-1;++n<r;)pi(a=e[n],c,i)?(pi(o,c,i)||t.push(Ii(o,a,c,i)),t.push(a)):pi(o,c,i)&&t.push(Ii(o,a,c,i)),o=a;s&&t.push(t[0]),c=i}return t};var mi,yi,Ei,vi,Bi,bi=[],Qi=[];function wi(){Pi(this),this.edge=this.site=this.circle=null}function xi(t){var e=bi.pop()||new wi;return e.site=t,e}function _i(t){Ti(t),Ei.remove(t),bi.push(t),Pi(t)}function Di(t){var e=t.circle,n=e.x,r=e.cy,i={x:n,y:r},o=t.P,a=t.N,s=[t];_i(t);for(var A=o;A.circle&&v(n-A.circle.x)<Mt&&v(r-A.circle.cy)<Mt;)o=A.P,s.unshift(A),_i(A),A=o;s.unshift(A),Ti(A);for(var l=a;l.circle&&v(n-l.circle.x)<Mt&&v(r-l.circle.cy)<Mt;)a=l.N,s.push(l),_i(l),l=a;s.push(l),Ti(l);var c,u=s.length;for(c=1;c<u;++c)l=s[c],A=s[c-1],Hi(l.edge,A.site,l.site,i);A=s[0],(l=s[u-1]).edge=Gi(A.site,l.site,null,i),Fi(A),Fi(l)}function Mi(t){for(var e,n,r,i,o=t.x,a=t.y,s=Ei._;s;)if((r=ki(s,a)-o)>Mt)s=s.L;else{if(!((i=o-Si(s,a))>Mt)){r>-Mt?(e=s.P,n=s):i>-Mt?(e=s,n=s.N):e=n=s;break}if(!s.R){e=s;break}s=s.R}var A=xi(t);if(Ei.insert(e,A),e||n){if(e===n)return Ti(e),n=xi(e.site),Ei.insert(A,n),A.edge=n.edge=Gi(e.site,A.site),Fi(e),void Fi(n);if(n){Ti(e),Ti(n);var l=e.site,c=l.x,u=l.y,g=t.x-c,h=t.y-u,d=n.site,f=d.x-c,p=d.y-u,I=2*(g*p-h*f),C=g*g+h*h,m=f*f+p*p,y={x:(p*C-h*m)/I+c,y:(g*m-f*C)/I+u};Hi(n.edge,l,d,y),A.edge=Gi(l,t,null,y),n.edge=Gi(t,d,null,y),Fi(e),Fi(n)}else A.edge=Gi(e.site,A.site)}}function ki(t,e){var n=t.site,r=n.x,i=n.y,o=i-e;if(!o)return r;var a=t.P;if(!a)return-1/0;var s=(n=a.site).x,A=n.y,l=A-e;if(!l)return s;var c=s-r,u=1/o-1/l,g=c/l;return u?(-g+Math.sqrt(g*g-2*u*(c*c/(-2*l)-A+l/2+i-o/2)))/u+r:(r+s)/2}function Si(t,e){var n=t.N;if(n)return ki(n,e);var r=t.site;return r.y===e?r.x:1/0}function Li(t){this.site=t,this.edges=[]}function Ri(t,e){return e.angle-t.angle}function Ni(){Pi(this),this.x=this.y=this.arc=this.site=this.cy=null}function Fi(t){var e=t.P,n=t.N;if(e&&n){var r=e.site,i=t.site,o=n.site;if(r!==o){var a=i.x,s=i.y,A=r.x-a,l=r.y-s,c=o.x-a,u=2*(A*(p=o.y-s)-l*c);if(!(u>=-kt)){var g=A*A+l*l,h=c*c+p*p,d=(p*g-l*h)/u,f=(A*h-c*g)/u,p=f+s,I=Qi.pop()||new Ni;I.arc=t,I.site=i,I.x=d+a,I.y=p+Math.sqrt(d*d+f*f),I.cy=p,t.circle=I;for(var C=null,m=Bi._;m;)if(I.y<m.y||I.y===m.y&&I.x<=m.x){if(!m.L){C=m.P;break}m=m.L}else{if(!m.R){C=m;break}m=m.R}Bi.insert(C,I),C||(vi=I)}}}}function Ti(t){var e=t.circle;e&&(e.P||(vi=e.N),Bi.remove(e),Qi.push(e),Pi(e),t.circle=null)}function Ui(t,e){var n=t.b;if(n)return!0;var r,i,o=t.a,a=e[0][0],s=e[1][0],A=e[0][1],l=e[1][1],c=t.l,u=t.r,g=c.x,h=c.y,d=u.x,f=u.y,p=(g+d)/2,I=(h+f)/2;if(f===h){if(p<a||p>=s)return;if(g>d){if(o){if(o.y>=l)return}else o={x:p,y:A};n={x:p,y:l}}else{if(o){if(o.y<A)return}else o={x:p,y:l};n={x:p,y:A}}}else if(i=I-(r=(g-d)/(f-h))*p,r<-1||r>1)if(g>d){if(o){if(o.y>=l)return}else o={x:(A-i)/r,y:A};n={x:(l-i)/r,y:l}}else{if(o){if(o.y<A)return}else o={x:(l-i)/r,y:l};n={x:(A-i)/r,y:A}}else if(h<f){if(o){if(o.x>=s)return}else o={x:a,y:r*a+i};n={x:s,y:r*s+i}}else{if(o){if(o.x<a)return}else o={x:s,y:r*s+i};n={x:a,y:r*a+i}}return t.a=o,t.b=n,!0}function Oi(t,e){this.l=t,this.r=e,this.a=this.b=null}function Gi(t,e,n,r){var i=new Oi(t,e);return mi.push(i),n&&Hi(i,t,e,n),r&&Hi(i,e,t,r),yi[t.i].edges.push(new ji(i,t,e)),yi[e.i].edges.push(new ji(i,e,t)),i}function Hi(t,e,n,r){t.a||t.b?t.l===n?t.b=r:t.a=r:(t.a=r,t.l=e,t.r=n)}function ji(t,e,n){var r=t.a,i=t.b;this.edge=t,this.site=e,this.angle=n?Math.atan2(n.y-e.y,n.x-e.x):t.l===e?Math.atan2(i.x-r.x,r.y-i.y):Math.atan2(r.x-i.x,i.y-r.y)}function Yi(){this._=null}function Pi(t){t.U=t.C=t.L=t.R=t.P=t.N=null}function Ki(t,e){var n=e,r=e.R,i=n.U;i?i.L===n?i.L=r:i.R=r:t._=r,r.U=i,n.U=r,n.R=r.L,n.R&&(n.R.U=n),r.L=n}function zi(t,e){var n=e,r=e.L,i=n.U;i?i.L===n?i.L=r:i.R=r:t._=r,r.U=i,n.U=r,n.L=r.R,n.L&&(n.L.U=n),r.R=n}function Ji(t){for(;t.L;)t=t.L;return t}function qi(t,e){var n,r,i,o=t.sort(Vi).pop();for(mi=[],yi=new Array(t.length),Ei=new Yi,Bi=new Yi;;)if(i=vi,o&&(!i||o.y<i.y||o.y===i.y&&o.x<i.x))o.x===n&&o.y===r||(yi[o.i]=new Li(o),Mi(o),n=o.x,r=o.y),o=t.pop();else{if(!i)break;Di(i.arc)}e&&(function(t){for(var e,n=mi,r=ur(t[0][0],t[0][1],t[1][0],t[1][1]),i=n.length;i--;)(!Ui(e=n[i],t)||!r(e)||v(e.a.x-e.b.x)<Mt&&v(e.a.y-e.b.y)<Mt)&&(e.a=e.b=null,n.splice(i,1))}(e),function(t){for(var e,n,r,i,o,a,s,A,l,c,u=t[0][0],g=t[1][0],h=t[0][1],d=t[1][1],f=yi,p=f.length;p--;)if((o=f[p])&&o.prepare())for(A=(s=o.edges).length,a=0;a<A;)r=(c=s[a].end()).x,i=c.y,e=(l=s[++a%A].start()).x,n=l.y,(v(r-e)>Mt||v(i-n)>Mt)&&(s.splice(a,0,new ji((I=o.site,C=c,m=v(r-u)<Mt&&d-i>Mt?{x:u,y:v(e-u)<Mt?n:d}:v(i-d)<Mt&&g-r>Mt?{x:v(n-d)<Mt?e:g,y:d}:v(r-g)<Mt&&i-h>Mt?{x:g,y:v(e-g)<Mt?n:h}:v(i-h)<Mt&&r-u>Mt?{x:v(n-h)<Mt?e:u,y:h}:null,y=void 0,(y=new Oi(I,null)).a=C,y.b=m,mi.push(y),y),o.site,null)),++A);var I,C,m,y}(e));var a={cells:yi,edges:mi};return Ei=Bi=mi=yi=null,a}function Vi(t,e){return e.y-t.y||e.x-t.x}Li.prototype.prepare=function(){for(var t,e=this.edges,n=e.length;n--;)(t=e[n].edge).b&&t.a||e.splice(n,1);return e.sort(Ri),e.length},ji.prototype={start:function(){return this.edge.l===this.site?this.edge.a:this.edge.b},end:function(){return this.edge.l===this.site?this.edge.b:this.edge.a}},Yi.prototype={insert:function(t,e){var n,r,i;if(t){if(e.P=t,e.N=t.N,t.N&&(t.N.P=e),t.N=e,t.R){for(t=t.R;t.L;)t=t.L;t.L=e}else t.R=e;n=t}else this._?(t=Ji(this._),e.P=null,e.N=t,t.P=t.L=e,n=t):(e.P=e.N=null,this._=e,n=null);for(e.L=e.R=null,e.U=n,e.C=!0,t=e;n&&n.C;)n===(r=n.U).L?(i=r.R)&&i.C?(n.C=i.C=!1,r.C=!0,t=r):(t===n.R&&(Ki(this,n),n=(t=n).U),n.C=!1,r.C=!0,zi(this,r)):(i=r.L)&&i.C?(n.C=i.C=!1,r.C=!0,t=r):(t===n.L&&(zi(this,n),n=(t=n).U),n.C=!1,r.C=!0,Ki(this,r)),n=t.U;this._.C=!1},remove:function(t){t.N&&(t.N.P=t.P),t.P&&(t.P.N=t.N),t.N=t.P=null;var e,n,r,i=t.U,o=t.L,a=t.R;if(n=o?a?Ji(a):o:a,i?i.L===t?i.L=n:i.R=n:this._=n,o&&a?(r=n.C,n.C=t.C,n.L=o,o.U=n,n!==a?(i=n.U,n.U=t.U,t=n.R,i.L=t,n.R=a,a.U=n):(n.U=i,i=n,t=n.R)):(r=t.C,t=n),t&&(t.U=i),!r)if(t&&t.C)t.C=!1;else{do{if(t===this._)break;if(t===i.L){if((e=i.R).C&&(e.C=!1,i.C=!0,Ki(this,i),e=i.R),e.L&&e.L.C||e.R&&e.R.C){e.R&&e.R.C||(e.L.C=!1,e.C=!0,zi(this,e),e=i.R),e.C=i.C,i.C=e.R.C=!1,Ki(this,i),t=this._;break}}else if((e=i.L).C&&(e.C=!1,i.C=!0,zi(this,i),e=i.L),e.L&&e.L.C||e.R&&e.R.C){e.L&&e.L.C||(e.R.C=!1,e.C=!0,Ki(this,e),e=i.L),e.C=i.C,i.C=e.L.C=!1,zi(this,i),t=this._;break}e.C=!0,t=i,i=i.U}while(!t.C);t&&(t.C=!1)}}},o.geom.voronoi=function(t){var e=ui,n=gi,r=e,i=n,o=Wi;if(t)return a(t);function a(t){var e=new Array(t.length),n=o[0][0],r=o[0][1],i=o[1][0],a=o[1][1];return qi(s(t),o).cells.forEach(function(o,s){var A=o.edges,l=o.site;(e[s]=A.length?A.map(function(t){var e=t.start();return[e.x,e.y]}):l.x>=n&&l.x<=i&&l.y>=r&&l.y<=a?[[n,a],[i,a],[i,r],[n,r]]:[]).point=t[s]}),e}function s(t){return t.map(function(t,e){return{x:Math.round(r(t,e)/Mt)*Mt,y:Math.round(i(t,e)/Mt)*Mt,i:e}})}return a.links=function(t){return qi(s(t)).edges.filter(function(t){return t.l&&t.r}).map(function(e){return{source:t[e.l.i],target:t[e.r.i]}})},a.triangles=function(t){var e=[];return qi(s(t)).cells.forEach(function(n,r){for(var i,o,a,s,A=n.site,l=n.edges.sort(Ri),c=-1,u=l.length,g=l[u-1].edge,h=g.l===A?g.r:g.l;++c<u;)i=h,h=(g=l[c].edge).l===A?g.r:g.l,r<i.i&&r<h.i&&(a=i,s=h,((o=A).x-s.x)*(a.y-o.y)-(o.x-a.x)*(s.y-o.y)<0)&&e.push([t[r],t[i.i],t[h.i]])}),e},a.x=function(t){return arguments.length?(r=ve(e=t),a):e},a.y=function(t){return arguments.length?(i=ve(n=t),a):n},a.clipExtent=function(t){return arguments.length?(o=null==t?Wi:t,a):o===Wi?null:o},a.size=function(t){return arguments.length?a.clipExtent(t&&[[0,0],t]):o===Wi?null:o&&o[1]},a};var Wi=[[-1e6,-1e6],[1e6,1e6]];function Xi(t){return t.x}function Zi(t){return t.y}function $i(t,e,n,r,i,o){if(!t(e,n,r,i,o)){var a=.5*(n+i),s=.5*(r+o),A=e.nodes;A[0]&&$i(t,A[0],n,r,a,s),A[1]&&$i(t,A[1],a,r,i,s),A[2]&&$i(t,A[2],n,s,a,o),A[3]&&$i(t,A[3],a,s,i,o)}}function to(t,e,n,r,i,o,a){var s,A=1/0;return function t(l,c,u,g,h){if(!(c>o||u>a||g<r||h<i)){if(d=l.point){var d,f=e-l.x,p=n-l.y,I=f*f+p*p;if(I<A){var C=Math.sqrt(A=I);r=e-C,i=n-C,o=e+C,a=n+C,s=d}}for(var m=l.nodes,y=.5*(c+g),E=.5*(u+h),v=(n>=E)<<1|e>=y,B=v+4;v<B;++v)if(l=m[3&v])switch(3&v){case 0:t(l,c,u,y,E);break;case 1:t(l,y,u,g,E);break;case 2:t(l,c,E,y,h);break;case 3:t(l,y,E,g,h)}}}(t,r,i,o,a),s}function eo(t,e){t=o.rgb(t),e=o.rgb(e);var n=t.r,r=t.g,i=t.b,a=e.r-n,s=e.g-r,A=e.b-i;return function(t){return"#"+fe(Math.round(n+a*t))+fe(Math.round(r+s*t))+fe(Math.round(i+A*t))}}function no(t,e){var n,r={},i={};for(n in t)n in e?r[n]=so(t[n],e[n]):i[n]=t[n];for(n in e)n in t||(i[n]=e[n]);return function(t){for(n in r)i[n]=r[n](t);return i}}function ro(t,e){return t=+t,e=+e,function(n){return t*(1-n)+e*n}}function io(t,e){var n,r,i,o=oo.lastIndex=ao.lastIndex=0,a=-1,s=[],A=[];for(t+="",e+="";(n=oo.exec(t))&&(r=ao.exec(e));)(i=r.index)>o&&(i=e.slice(o,i),s[a]?s[a]+=i:s[++a]=i),(n=n[0])===(r=r[0])?s[a]?s[a]+=r:s[++a]=r:(s[++a]=null,A.push({i:a,x:ro(n,r)})),o=ao.lastIndex;return o<e.length&&(i=e.slice(o),s[a]?s[a]+=i:s[++a]=i),s.length<2?A[0]?(e=A[0].x,function(t){return e(t)+""}):function(){return e}:(e=A.length,function(t){for(var n,r=0;r<e;++r)s[(n=A[r]).i]=n.x(t);return s.join("")})}o.geom.delaunay=function(t){return o.geom.voronoi().triangles(t)},o.geom.quadtree=function(t,e,n,r,i){var o,a=ui,s=gi;if(o=arguments.length)return a=Xi,s=Zi,3===o&&(i=n,r=e,n=e=0),A(t);function A(t){var A,l,c,u,g,h,d,f,p,I=ve(a),C=ve(s);if(null!=e)h=e,d=n,f=r,p=i;else if(f=p=-(h=d=1/0),l=[],c=[],g=t.length,o)for(u=0;u<g;++u)(A=t[u]).x<h&&(h=A.x),A.y<d&&(d=A.y),A.x>f&&(f=A.x),A.y>p&&(p=A.y),l.push(A.x),c.push(A.y);else for(u=0;u<g;++u){var m=+I(A=t[u],u),y=+C(A,u);m<h&&(h=m),y<d&&(d=y),m>f&&(f=m),y>p&&(p=y),l.push(m),c.push(y)}var E=f-h,B=p-d;function b(t,e,n,r,i,o,a,s){if(!isNaN(n)&&!isNaN(r))if(t.leaf){var A=t.x,l=t.y;if(null!=A)if(v(A-n)+v(l-r)<.01)Q(t,e,n,r,i,o,a,s);else{var c=t.point;t.x=t.y=t.point=null,Q(t,c,A,l,i,o,a,s),Q(t,e,n,r,i,o,a,s)}else t.x=n,t.y=r,t.point=e}else Q(t,e,n,r,i,o,a,s)}function Q(t,e,n,r,i,o,a,s){var A=.5*(i+a),l=.5*(o+s),c=n>=A,u=r>=l,g=u<<1|c;t.leaf=!1,c?i=A:a=A,u?o=l:s=l,b(t=t.nodes[g]||(t.nodes[g]={leaf:!0,nodes:[],point:null,x:null,y:null}),e,n,r,i,o,a,s)}E>B?p=d+E:f=h+B;var w={leaf:!0,nodes:[],point:null,x:null,y:null,add:function(t){b(w,t,+I(t,++u),+C(t,u),h,d,f,p)},visit:function(t){$i(t,w,h,d,f,p)},find:function(t){return to(w,t[0],t[1],h,d,f,p)}};if(u=-1,null==e){for(;++u<g;)b(w,t[u],l[u],c[u],h,d,f,p);--u}else t.forEach(w.add);return l=c=t=A=null,w}return A.x=function(t){return arguments.length?(a=t,A):a},A.y=function(t){return arguments.length?(s=t,A):s},A.extent=function(t){return arguments.length?(null==t?e=n=r=i=null:(e=+t[0][0],n=+t[0][1],r=+t[1][0],i=+t[1][1]),A):null==e?null:[[e,n],[r,i]]},A.size=function(t){return arguments.length?(null==t?e=n=r=i=null:(e=n=0,r=+t[0],i=+t[1]),A):null==e?null:[r-e,i-n]},A},o.interpolateRgb=eo,o.interpolateObject=no,o.interpolateNumber=ro,o.interpolateString=io;var oo=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,ao=new RegExp(oo.source,"g");function so(t,e){for(var n,r=o.interpolators.length;--r>=0&&!(n=o.interpolators[r](t,e)););return n}function Ao(t,e){var n,r=[],i=[],o=t.length,a=e.length,s=Math.min(t.length,e.length);for(n=0;n<s;++n)r.push(so(t[n],e[n]));for(;n<o;++n)i[n]=t[n];for(;n<a;++n)i[n]=e[n];return function(t){for(n=0;n<s;++n)i[n]=r[n](t);return i}}o.interpolate=so,o.interpolators=[function(t,e){var n=typeof e;return("string"===n?Ee.has(e.toLowerCase())||/^(#|rgb\(|hsl\()/i.test(e)?eo:io:e instanceof qt?eo:Array.isArray(e)?Ao:"object"===n&&isNaN(e)?no:ro)(t,e)}],o.interpolateArray=Ao;var lo=function(){return F},co=o.map({linear:lo,poly:function(t){return function(e){return Math.pow(e,t)}},quad:function(){return po},cubic:function(){return Io},sin:function(){return mo},exp:function(){return yo},circle:function(){return Eo},elastic:function(t,e){var n;return arguments.length<2&&(e=.45),arguments.length?n=e/Lt*Math.asin(1/t):(t=1,n=e/4),function(r){return 1+t*Math.pow(2,-10*r)*Math.sin((r-n)*Lt/e)}},back:function(t){return t||(t=1.70158),function(e){return e*e*((t+1)*e-t)}},bounce:function(){return vo}}),uo=o.map({in:F,out:ho,"in-out":fo,"out-in":function(t){return fo(ho(t))}});function go(t){return function(e){return e<=0?0:e>=1?1:t(e)}}function ho(t){return function(e){return 1-t(1-e)}}function fo(t){return function(e){return.5*(e<.5?t(2*e):2-t(2-2*e))}}function po(t){return t*t}function Io(t){return t*t*t}function Co(t){if(t<=0)return 0;if(t>=1)return 1;var e=t*t,n=e*t;return 4*(t<.5?n:3*(t-e)+n-.75)}function mo(t){return 1-Math.cos(t*Nt)}function yo(t){return Math.pow(2,10*(t-1))}function Eo(t){return 1-Math.sqrt(1-t*t)}function vo(t){return t<1/2.75?7.5625*t*t:t<2/2.75?7.5625*(t-=1.5/2.75)*t+.75:t<2.5/2.75?7.5625*(t-=2.25/2.75)*t+.9375:7.5625*(t-=2.625/2.75)*t+.984375}function Bo(t,e){return e-=t,function(n){return Math.round(t+e*n)}}function bo(t){var e=[t.a,t.b],n=[t.c,t.d],r=wo(e),i=Qo(e,n),o=wo(function(t,e,n){return t[0]+=n*e[0],t[1]+=n*e[1],t}(n,e,-i))||0;e[0]*n[1]<n[0]*e[1]&&(e[0]*=-1,e[1]*=-1,r*=-1,i*=-1),this.rotate=(r?Math.atan2(e[1],e[0]):Math.atan2(-n[0],n[1]))*Tt,this.translate=[t.e,t.f],this.scale=[r,o],this.skew=o?Math.atan2(i,o)*Tt:0}function Qo(t,e){return t[0]*e[0]+t[1]*e[1]}function wo(t){var e=Math.sqrt(Qo(t,t));return e&&(t[0]/=e,t[1]/=e),e}o.ease=function(t){var e=t.indexOf("-"),n=e>=0?t.slice(0,e):t,r=e>=0?t.slice(e+1):"in";return n=co.get(n)||lo,go((r=uo.get(r)||F)(n.apply(null,a.call(arguments,1))))},o.interpolateHcl=function(t,e){t=o.hcl(t),e=o.hcl(e);var n=t.h,r=t.c,i=t.l,a=e.h-n,s=e.c-r,A=e.l-i;return isNaN(s)&&(s=0,r=isNaN(r)?e.c:r),isNaN(a)?(a=0,n=isNaN(n)?e.h:n):a>180?a-=360:a<-180&&(a+=360),function(t){return te(n+a*t,r+s*t,i+A*t)+""}},o.interpolateHsl=function(t,e){t=o.hsl(t),e=o.hsl(e);var n=t.h,r=t.s,i=t.l,a=e.h-n,s=e.s-r,A=e.l-i;return isNaN(s)&&(s=0,r=isNaN(r)?e.s:r),isNaN(a)?(a=0,n=isNaN(n)?e.h:n):a>180?a-=360:a<-180&&(a+=360),function(t){return Xt(n+a*t,r+s*t,i+A*t)+""}},o.interpolateLab=function(t,e){t=o.lab(t),e=o.lab(e);var n=t.l,r=t.a,i=t.b,a=e.l-n,s=e.a-r,A=e.b-i;return function(t){return se(n+a*t,r+s*t,i+A*t)+""}},o.interpolateRound=Bo,o.transform=function(t){var e=A.createElementNS(o.ns.prefix.svg,"g");return(o.transform=function(t){if(null!=t){e.setAttribute("transform",t);var n=e.transform.baseVal.consolidate()}return new bo(n?n.matrix:xo)})(t)},bo.prototype.toString=function(){return"translate("+this.translate+")rotate("+this.rotate+")skewX("+this.skew+")scale("+this.scale+")"};var xo={a:1,b:0,c:0,d:1,e:0,f:0};function _o(t){return t.length?t.pop()+",":""}function Do(t,e){var n=[],r=[];return t=o.transform(t),e=o.transform(e),function(t,e,n,r){if(t[0]!==e[0]||t[1]!==e[1]){var i=n.push("translate(",null,",",null,")");r.push({i:i-4,x:ro(t[0],e[0])},{i:i-2,x:ro(t[1],e[1])})}else(e[0]||e[1])&&n.push("translate("+e+")")}(t.translate,e.translate,n,r),function(t,e,n,r){t!==e?(t-e>180?e+=360:e-t>180&&(t+=360),r.push({i:n.push(_o(n)+"rotate(",null,")")-2,x:ro(t,e)})):e&&n.push(_o(n)+"rotate("+e+")")}(t.rotate,e.rotate,n,r),function(t,e,n,r){t!==e?r.push({i:n.push(_o(n)+"skewX(",null,")")-2,x:ro(t,e)}):e&&n.push(_o(n)+"skewX("+e+")")}(t.skew,e.skew,n,r),function(t,e,n,r){if(t[0]!==e[0]||t[1]!==e[1]){var i=n.push(_o(n)+"scale(",null,",",null,")");r.push({i:i-4,x:ro(t[0],e[0])},{i:i-2,x:ro(t[1],e[1])})}else 1===e[0]&&1===e[1]||n.push(_o(n)+"scale("+e+")")}(t.scale,e.scale,n,r),t=e=null,function(t){for(var e,i=-1,o=r.length;++i<o;)n[(e=r[i]).i]=e.x(t);return n.join("")}}function Mo(t,e){return e=(e-=t=+t)||1/e,function(n){return(n-t)/e}}function ko(t,e){return e=(e-=t=+t)||1/e,function(n){return Math.max(0,Math.min(1,(n-t)/e))}}function So(t){for(var e=t.source,n=t.target,r=function(t,e){if(t===e)return t;for(var n=Lo(t),r=Lo(e),i=n.pop(),o=r.pop(),a=null;i===o;)a=i,i=n.pop(),o=r.pop();return a}(e,n),i=[e];e!==r;)e=e.parent,i.push(e);for(var o=i.length;n!==r;)i.splice(o,0,n),n=n.parent;return i}function Lo(t){for(var e=[],n=t.parent;null!=n;)e.push(t),t=n,n=n.parent;return e.push(t),e}function Ro(t){t.fixed|=2}function No(t){t.fixed&=-7}function Fo(t){t.fixed|=4,t.px=t.x,t.py=t.y}function To(t){t.fixed&=-5}o.interpolateTransform=Do,o.layout={},o.layout.bundle=function(){return function(t){for(var e=[],n=-1,r=t.length;++n<r;)e.push(So(t[n]));return e}},o.layout.chord=function(){var t,e,n,r,i,a,s,A={},l=0;function c(){var A,c,g,h,d,f={},p=[],I=o.range(r),C=[];for(t=[],e=[],A=0,h=-1;++h<r;){for(c=0,d=-1;++d<r;)c+=n[h][d];p.push(c),C.push(o.range(r)),A+=c}for(i&&I.sort(function(t,e){return i(p[t],p[e])}),a&&C.forEach(function(t,e){t.sort(function(t,r){return a(n[e][t],n[e][r])})}),A=(Lt-l*r)/A,c=0,h=-1;++h<r;){for(g=c,d=-1;++d<r;){var m=I[h],y=C[m][d],E=n[m][y],v=c,B=c+=E*A;f[m+"-"+y]={index:m,subindex:y,startAngle:v,endAngle:B,value:E}}e[m]={index:m,startAngle:g,endAngle:c,value:p[m]},c+=l}for(h=-1;++h<r;)for(d=h-1;++d<r;){var b=f[h+"-"+d],Q=f[d+"-"+h];(b.value||Q.value)&&t.push(b.value<Q.value?{source:Q,target:b}:{source:b,target:Q})}s&&u()}function u(){t.sort(function(t,e){return s((t.source.value+t.target.value)/2,(e.source.value+e.target.value)/2)})}return A.matrix=function(i){return arguments.length?(r=(n=i)&&n.length,t=e=null,A):n},A.padding=function(n){return arguments.length?(l=n,t=e=null,A):l},A.sortGroups=function(n){return arguments.length?(i=n,t=e=null,A):i},A.sortSubgroups=function(e){return arguments.length?(a=e,t=null,A):a},A.sortChords=function(e){return arguments.length?(s=e,t&&u(),A):s},A.chords=function(){return t||c(),t},A.groups=function(){return e||c(),e},A},o.layout.force=function(){var t,e,n,r,i,a,s={},A=o.dispatch("start","tick","end"),l=[1,1],c=.9,u=Uo,g=Oo,h=-30,d=Go,f=.1,p=.64,I=[],C=[];function m(t){return function(e,n,r,i){if(e.point!==t){var o=e.cx-t.x,a=e.cy-t.y,s=i-n,A=o*o+a*a;if(s*s/p<A){if(A<d){var l=e.charge/A;t.px-=o*l,t.py-=a*l}return!0}e.point&&A&&A<d&&(l=e.pointCharge/A,t.px-=o*l,t.py-=a*l)}return!e.charge}}function y(t){t.px=o.event.x,t.py=o.event.y,s.resume()}return s.tick=function(){if((n*=.99)<.005)return t=null,A.end({type:"end",alpha:n=0}),!0;var e,s,u,g,d,p,y,E,v,B=I.length,b=C.length;for(s=0;s<b;++s)g=(u=C[s]).source,(p=(E=(d=u.target).x-g.x)*E+(v=d.y-g.y)*v)&&(E*=p=n*i[s]*((p=Math.sqrt(p))-r[s])/p,v*=p,d.x-=E*(y=g.weight+d.weight?g.weight/(g.weight+d.weight):.5),d.y-=v*y,g.x+=E*(y=1-y),g.y+=v*y);if((y=n*f)&&(E=l[0]/2,v=l[1]/2,s=-1,y))for(;++s<B;)(u=I[s]).x+=(E-u.x)*y,u.y+=(v-u.y)*y;if(h)for(function t(e,n,r){var i=0,o=0;if(e.charge=0,!e.leaf)for(var a,s=e.nodes,A=s.length,l=-1;++l<A;)null!=(a=s[l])&&(t(a,n,r),e.charge+=a.charge,i+=a.charge*a.cx,o+=a.charge*a.cy);if(e.point){e.leaf||(e.point.x+=Math.random()-.5,e.point.y+=Math.random()-.5);var c=n*r[e.point.index];e.charge+=e.pointCharge=c,i+=c*e.point.x,o+=c*e.point.y}e.cx=i/e.charge,e.cy=o/e.charge}(e=o.geom.quadtree(I),n,a),s=-1;++s<B;)(u=I[s]).fixed||e.visit(m(u));for(s=-1;++s<B;)(u=I[s]).fixed?(u.x=u.px,u.y=u.py):(u.x-=(u.px-(u.px=u.x))*c,u.y-=(u.py-(u.py=u.y))*c);A.tick({type:"tick",alpha:n})},s.nodes=function(t){return arguments.length?(I=t,s):I},s.links=function(t){return arguments.length?(C=t,s):C},s.size=function(t){return arguments.length?(l=t,s):l},s.linkDistance=function(t){return arguments.length?(u="function"==typeof t?t:+t,s):u},s.distance=s.linkDistance,s.linkStrength=function(t){return arguments.length?(g="function"==typeof t?t:+t,s):g},s.friction=function(t){return arguments.length?(c=+t,s):c},s.charge=function(t){return arguments.length?(h="function"==typeof t?t:+t,s):h},s.chargeDistance=function(t){return arguments.length?(d=t*t,s):Math.sqrt(d)},s.gravity=function(t){return arguments.length?(f=+t,s):f},s.theta=function(t){return arguments.length?(p=t*t,s):Math.sqrt(p)},s.alpha=function(e){return arguments.length?(e=+e,n?e>0?n=e:(t.c=null,t.t=NaN,t=null,A.end({type:"end",alpha:n=0})):e>0&&(A.start({type:"start",alpha:n=e}),t=Me(s.tick)),s):n},s.start=function(){var t,e,n,o=I.length,A=C.length,c=l[0],d=l[1];for(t=0;t<o;++t)(n=I[t]).index=t,n.weight=0;for(t=0;t<A;++t)"number"==typeof(n=C[t]).source&&(n.source=I[n.source]),"number"==typeof n.target&&(n.target=I[n.target]),++n.source.weight,++n.target.weight;for(t=0;t<o;++t)n=I[t],isNaN(n.x)&&(n.x=f("x",c)),isNaN(n.y)&&(n.y=f("y",d)),isNaN(n.px)&&(n.px=n.x),isNaN(n.py)&&(n.py=n.y);if(r=[],"function"==typeof u)for(t=0;t<A;++t)r[t]=+u.call(this,C[t],t);else for(t=0;t<A;++t)r[t]=u;if(i=[],"function"==typeof g)for(t=0;t<A;++t)i[t]=+g.call(this,C[t],t);else for(t=0;t<A;++t)i[t]=g;if(a=[],"function"==typeof h)for(t=0;t<o;++t)a[t]=+h.call(this,I[t],t);else for(t=0;t<o;++t)a[t]=h;function f(n,r){if(!e){for(e=new Array(o),l=0;l<o;++l)e[l]=[];for(l=0;l<A;++l){var i=C[l];e[i.source.index].push(i.target),e[i.target.index].push(i.source)}}for(var a,s=e[t],l=-1,c=s.length;++l<c;)if(!isNaN(a=s[l][n]))return a;return Math.random()*r}return s.resume()},s.resume=function(){return s.alpha(.1)},s.stop=function(){return s.alpha(0)},s.drag=function(){if(e||(e=o.behavior.drag().origin(F).on("dragstart.force",Ro).on("drag.force",y).on("dragend.force",No)),!arguments.length)return e;this.on("mouseover.force",Fo).on("mouseout.force",To).call(e)},o.rebind(s,A,"on")};var Uo=20,Oo=1,Go=1/0;function Ho(t,e){return o.rebind(t,e,"sort","children","value"),t.nodes=t,t.links=Jo,t}function jo(t,e){for(var n=[t];null!=(t=n.pop());)if(e(t),(i=t.children)&&(r=i.length))for(var r,i;--r>=0;)n.push(i[r])}function Yo(t,e){for(var n=[t],r=[];null!=(t=n.pop());)if(r.push(t),(o=t.children)&&(i=o.length))for(var i,o,a=-1;++a<i;)n.push(o[a]);for(;null!=(t=r.pop());)e(t)}function Po(t){return t.children}function Ko(t){return t.value}function zo(t,e){return e.value-t.value}function Jo(t){return o.merge(t.map(function(t){return(t.children||[]).map(function(e){return{source:t,target:e}})}))}o.layout.hierarchy=function(){var t=zo,e=Po,n=Ko;function r(i){var o,a=[i],s=[];for(i.depth=0;null!=(o=a.pop());)if(s.push(o),(l=e.call(r,o,o.depth))&&(A=l.length)){for(var A,l,c;--A>=0;)a.push(c=l[A]),c.parent=o,c.depth=o.depth+1;n&&(o.value=0),o.children=l}else n&&(o.value=+n.call(r,o,o.depth)||0),delete o.children;return Yo(i,function(e){var r,i;t&&(r=e.children)&&r.sort(t),n&&(i=e.parent)&&(i.value+=e.value)}),s}return r.sort=function(e){return arguments.length?(t=e,r):t},r.children=function(t){return arguments.length?(e=t,r):e},r.value=function(t){return arguments.length?(n=t,r):n},r.revalue=function(t){return n&&(jo(t,function(t){t.children&&(t.value=0)}),Yo(t,function(t){var e;t.children||(t.value=+n.call(r,t,t.depth)||0),(e=t.parent)&&(e.value+=t.value)})),t},r},o.layout.partition=function(){var t=o.layout.hierarchy(),e=[1,1];function n(n,r){var i=t.call(this,n,r);return function t(e,n,r,i){var o=e.children;if(e.x=n,e.y=e.depth*i,e.dx=r,e.dy=i,o&&(a=o.length)){var a,s,A,l=-1;for(r=e.value?r/e.value:0;++l<a;)t(s=o[l],n,A=s.value*r,i),n+=A}}(i[0],0,e[0],e[1]/function t(e){var n=e.children,r=0;if(n&&(i=n.length))for(var i,o=-1;++o<i;)r=Math.max(r,t(n[o]));return 1+r}(i[0])),i}return n.size=function(t){return arguments.length?(e=t,n):e},Ho(n,t)},o.layout.pie=function(){var t=Number,e=qo,n=0,r=Lt,i=0;function a(s){var A,l=s.length,c=s.map(function(e,n){return+t.call(a,e,n)}),u=+("function"==typeof n?n.apply(this,arguments):n),g=("function"==typeof r?r.apply(this,arguments):r)-u,h=Math.min(Math.abs(g)/l,+("function"==typeof i?i.apply(this,arguments):i)),d=h*(g<0?-1:1),f=o.sum(c),p=f?(g-l*d)/f:0,I=o.range(l),C=[];return null!=e&&I.sort(e===qo?function(t,e){return c[e]-c[t]}:function(t,n){return e(s[t],s[n])}),I.forEach(function(t){C[t]={data:s[t],value:A=c[t],startAngle:u,endAngle:u+=A*p+d,padAngle:h}}),C}return a.value=function(e){return arguments.length?(t=e,a):t},a.sort=function(t){return arguments.length?(e=t,a):e},a.startAngle=function(t){return arguments.length?(n=t,a):n},a.endAngle=function(t){return arguments.length?(r=t,a):r},a.padAngle=function(t){return arguments.length?(i=t,a):i},a};var qo={};function Vo(t){return t.x}function Wo(t){return t.y}function Xo(t,e,n){t.y0=e,t.y=n}o.layout.stack=function(){var t=F,e=ta,n=ea,r=Xo,i=Vo,a=Wo;function s(A,l){if(!(h=A.length))return A;var c=A.map(function(e,n){return t.call(s,e,n)}),u=c.map(function(t){return t.map(function(t,e){return[i.call(s,t,e),a.call(s,t,e)]})}),g=e.call(s,u,l);c=o.permute(c,g),u=o.permute(u,g);var h,d,f,p,I=n.call(s,u,l),C=c[0].length;for(f=0;f<C;++f)for(r.call(s,c[0][f],p=I[f],u[0][f][1]),d=1;d<h;++d)r.call(s,c[d][f],p+=u[d-1][f][1],u[d][f][1]);return A}return s.values=function(e){return arguments.length?(t=e,s):t},s.order=function(t){return arguments.length?(e="function"==typeof t?t:Zo.get(t)||ta,s):e},s.offset=function(t){return arguments.length?(n="function"==typeof t?t:$o.get(t)||ea,s):n},s.x=function(t){return arguments.length?(i=t,s):i},s.y=function(t){return arguments.length?(a=t,s):a},s.out=function(t){return arguments.length?(r=t,s):r},s};var Zo=o.map({"inside-out":function(t){var e,n,r=t.length,i=t.map(na),a=t.map(ra),s=o.range(r).sort(function(t,e){return i[t]-i[e]}),A=0,l=0,c=[],u=[];for(e=0;e<r;++e)n=s[e],A<l?(A+=a[n],c.push(n)):(l+=a[n],u.push(n));return u.reverse().concat(c)},reverse:function(t){return o.range(t.length).reverse()},default:ta}),$o=o.map({silhouette:function(t){var e,n,r,i=t.length,o=t[0].length,a=[],s=0,A=[];for(n=0;n<o;++n){for(e=0,r=0;e<i;e++)r+=t[e][n][1];r>s&&(s=r),a.push(r)}for(n=0;n<o;++n)A[n]=(s-a[n])/2;return A},wiggle:function(t){var e,n,r,i,o,a,s,A,l,c=t.length,u=t[0],g=u.length,h=[];for(h[0]=A=l=0,n=1;n<g;++n){for(e=0,i=0;e<c;++e)i+=t[e][n][1];for(e=0,o=0,s=u[n][0]-u[n-1][0];e<c;++e){for(r=0,a=(t[e][n][1]-t[e][n-1][1])/(2*s);r<e;++r)a+=(t[r][n][1]-t[r][n-1][1])/s;o+=a*t[e][n][1]}h[n]=A-=i?o/i*s:0,A<l&&(l=A)}for(n=0;n<g;++n)h[n]-=l;return h},expand:function(t){var e,n,r,i=t.length,o=t[0].length,a=1/i,s=[];for(n=0;n<o;++n){for(e=0,r=0;e<i;e++)r+=t[e][n][1];if(r)for(e=0;e<i;e++)t[e][n][1]/=r;else for(e=0;e<i;e++)t[e][n][1]=a}for(n=0;n<o;++n)s[n]=0;return s},zero:ea});function ta(t){return o.range(t.length)}function ea(t){for(var e=-1,n=t[0].length,r=[];++e<n;)r[e]=0;return r}function na(t){for(var e,n=1,r=0,i=t[0][1],o=t.length;n<o;++n)(e=t[n][1])>i&&(r=n,i=e);return r}function ra(t){return t.reduce(ia,0)}function ia(t,e){return t+e[1]}function oa(t,e){return aa(t,Math.ceil(Math.log(e.length)/Math.LN2+1))}function aa(t,e){for(var n=-1,r=+t[0],i=(t[1]-r)/e,o=[];++n<=e;)o[n]=i*n+r;return o}function sa(t){return[o.min(t),o.max(t)]}function Aa(t,e){return t.value-e.value}function la(t,e){var n=t._pack_next;t._pack_next=e,e._pack_prev=t,e._pack_next=n,n._pack_prev=e}function ca(t,e){t._pack_next=e,e._pack_prev=t}function ua(t,e){var n=e.x-t.x,r=e.y-t.y,i=t.r+e.r;return.999*i*i>n*n+r*r}function ga(t){if((e=t.children)&&(A=e.length)){var e,n,r,i,o,a,s,A,l=1/0,c=-1/0,u=1/0,g=-1/0;if(e.forEach(ha),(n=e[0]).x=-n.r,n.y=0,m(n),A>1&&((r=e[1]).x=r.r,r.y=0,m(r),A>2))for(fa(n,r,i=e[2]),m(i),la(n,i),n._pack_prev=i,la(i,r),r=n._pack_next,o=3;o<A;o++){fa(n,r,i=e[o]);var h=0,d=1,f=1;for(a=r._pack_next;a!==r;a=a._pack_next,d++)if(ua(a,i)){h=1;break}if(1==h)for(s=n._pack_prev;s!==a._pack_prev&&!ua(s,i);s=s._pack_prev,f++);h?(d<f||d==f&&r.r<n.r?ca(n,r=a):ca(n=s,r),o--):(la(n,i),r=i,m(i))}var p=(l+c)/2,I=(u+g)/2,C=0;for(o=0;o<A;o++)(i=e[o]).x-=p,i.y-=I,C=Math.max(C,i.r+Math.sqrt(i.x*i.x+i.y*i.y));t.r=C,e.forEach(da)}function m(t){l=Math.min(t.x-t.r,l),c=Math.max(t.x+t.r,c),u=Math.min(t.y-t.r,u),g=Math.max(t.y+t.r,g)}}function ha(t){t._pack_next=t._pack_prev=t}function da(t){delete t._pack_next,delete t._pack_prev}function fa(t,e,n){var r=t.r+n.r,i=e.x-t.x,o=e.y-t.y;if(r&&(i||o)){var a=e.r+n.r,s=i*i+o*o,A=.5+((r*=r)-(a*=a))/(2*s),l=Math.sqrt(Math.max(0,2*a*(r+s)-(r-=s)*r-a*a))/(2*s);n.x=t.x+A*i+l*o,n.y=t.y+A*o-l*i}else n.x=t.x+r,n.y=t.y}function pa(t,e){return t.parent==e.parent?1:2}function Ia(t){var e=t.children;return e.length?e[0]:t.t}function Ca(t){var e,n=t.children;return(e=n.length)?n[e-1]:t.t}function ma(t,e,n){var r=n/(e.i-t.i);e.c-=r,e.s+=n,t.c+=r,e.z+=n,e.m+=n}function ya(t,e,n){return t.a.parent===e.parent?t.a:n}function Ea(t){return{x:t.x,y:t.y,dx:t.dx,dy:t.dy}}function va(t,e){var n=t.x+e[3],r=t.y+e[0],i=t.dx-e[1]-e[3],o=t.dy-e[0]-e[2];return i<0&&(n+=i/2,i=0),o<0&&(r+=o/2,o=0),{x:n,y:r,dx:i,dy:o}}function Ba(t){var e=t[0],n=t[t.length-1];return e<n?[e,n]:[n,e]}function ba(t){return t.rangeExtent?t.rangeExtent():Ba(t.range())}function Qa(t,e,n,r){var i=n(t[0],t[1]),o=r(e[0],e[1]);return function(t){return o(i(t))}}function wa(t,e){var n,r=0,i=t.length-1,o=t[r],a=t[i];return a<o&&(n=r,r=i,i=n,n=o,o=a,a=n),t[r]=e.floor(o),t[i]=e.ceil(a),t}function xa(t){return t?{floor:function(e){return Math.floor(e/t)*t},ceil:function(e){return Math.ceil(e/t)*t}}:_a}o.layout.histogram=function(){var t=!0,e=Number,n=sa,r=oa;function i(i,a){for(var s,A,l=[],c=i.map(e,this),u=n.call(this,c,a),g=r.call(this,u,c,a),h=(a=-1,c.length),d=g.length-1,f=t?1:1/h;++a<d;)(s=l[a]=[]).dx=g[a+1]-(s.x=g[a]),s.y=0;if(d>0)for(a=-1;++a<h;)(A=c[a])>=u[0]&&A<=u[1]&&((s=l[o.bisect(g,A,1,d)-1]).y+=f,s.push(i[a]));return l}return i.value=function(t){return arguments.length?(e=t,i):e},i.range=function(t){return arguments.length?(n=ve(t),i):n},i.bins=function(t){return arguments.length?(r="number"==typeof t?function(e){return aa(e,t)}:ve(t),i):r},i.frequency=function(e){return arguments.length?(t=!!e,i):t},i},o.layout.pack=function(){var t,e=o.layout.hierarchy().sort(Aa),n=0,r=[1,1];function i(i,o){var a=e.call(this,i,o),s=a[0],A=r[0],l=r[1],c=null==t?Math.sqrt:"function"==typeof t?t:function(){return t};if(s.x=s.y=0,Yo(s,function(t){t.r=+c(t.value)}),Yo(s,ga),n){var u=n*(t?1:Math.max(2*s.r/A,2*s.r/l))/2;Yo(s,function(t){t.r+=u}),Yo(s,ga),Yo(s,function(t){t.r-=u})}return function t(e,n,r,i){var o=e.children;if(e.x=n+=i*e.x,e.y=r+=i*e.y,e.r*=i,o)for(var a=-1,s=o.length;++a<s;)t(o[a],n,r,i)}(s,A/2,l/2,t?1:1/Math.max(2*s.r/A,2*s.r/l)),a}return i.size=function(t){return arguments.length?(r=t,i):r},i.radius=function(e){return arguments.length?(t=null==e||"function"==typeof e?e:+e,i):t},i.padding=function(t){return arguments.length?(n=+t,i):n},Ho(i,e)},o.layout.tree=function(){var t=o.layout.hierarchy().sort(null).value(null),e=pa,n=[1,1],r=null;function i(i,o){var l=t.call(this,i,o),c=l[0],u=function(t){for(var e,n={A:null,children:[c]},r=[n];null!=(e=r.pop());)for(var i,o=e.children,a=0,s=o.length;a<s;++a)r.push((o[a]=i={_:o[a],parent:e,children:(i=o[a].children)&&i.slice()||[],A:null,a:null,z:0,m:0,c:0,s:0,t:null,i:a}).a=i);return n.children[0]}();if(Yo(u,a),u.parent.m=-u.z,jo(u,s),r)jo(c,A);else{var g=c,h=c,d=c;jo(c,function(t){t.x<g.x&&(g=t),t.x>h.x&&(h=t),t.depth>d.depth&&(d=t)});var f=e(g,h)/2-g.x,p=n[0]/(h.x+e(h,g)/2+f),I=n[1]/(d.depth||1);jo(c,function(t){t.x=(t.x+f)*p,t.y=t.depth*I})}return l}function a(t){var n=t.children,r=t.parent.children,i=t.i?r[t.i-1]:null;if(n.length){!function(t){for(var e,n=0,r=0,i=t.children,o=i.length;--o>=0;)(e=i[o]).z+=n,e.m+=n,n+=e.s+(r+=e.c)}(t);var o=(n[0].z+n[n.length-1].z)/2;i?(t.z=i.z+e(t._,i._),t.m=t.z-o):t.z=o}else i&&(t.z=i.z+e(t._,i._));t.parent.A=function(t,n,r){if(n){for(var i,o=t,a=t,s=n,A=o.parent.children[0],l=o.m,c=a.m,u=s.m,g=A.m;s=Ca(s),o=Ia(o),s&&o;)A=Ia(A),(a=Ca(a)).a=t,(i=s.z+u-o.z-l+e(s._,o._))>0&&(ma(ya(s,t,r),t,i),l+=i,c+=i),u+=s.m,l+=o.m,g+=A.m,c+=a.m;s&&!Ca(a)&&(a.t=s,a.m+=u-c),o&&!Ia(A)&&(A.t=o,A.m+=l-g,r=t)}return r}(t,i,t.parent.A||r[0])}function s(t){t._.x=t.z+t.parent.m,t.m+=t.parent.m}function A(t){t.x*=n[0],t.y=t.depth*n[1]}return i.separation=function(t){return arguments.length?(e=t,i):e},i.size=function(t){return arguments.length?(r=null==(n=t)?A:null,i):r?null:n},i.nodeSize=function(t){return arguments.length?(r=null==(n=t)?null:A,i):r?n:null},Ho(i,t)},o.layout.cluster=function(){var t=o.layout.hierarchy().sort(null).value(null),e=pa,n=[1,1],r=!1;function i(i,a){var s,A=t.call(this,i,a),l=A[0],c=0;Yo(l,function(t){var n=t.children;n&&n.length?(t.x=function(t){return t.reduce(function(t,e){return t+e.x},0)/t.length}(n),t.y=function(t){return 1+o.max(t,function(t){return t.y})}(n)):(t.x=s?c+=e(t,s):0,t.y=0,s=t)});var u=function t(e){var n=e.children;return n&&n.length?t(n[0]):e}(l),g=function t(e){var n,r=e.children;return r&&(n=r.length)?t(r[n-1]):e}(l),h=u.x-e(u,g)/2,d=g.x+e(g,u)/2;return Yo(l,r?function(t){t.x=(t.x-l.x)*n[0],t.y=(l.y-t.y)*n[1]}:function(t){t.x=(t.x-h)/(d-h)*n[0],t.y=(1-(l.y?t.y/l.y:1))*n[1]}),A}return i.separation=function(t){return arguments.length?(e=t,i):e},i.size=function(t){return arguments.length?(r=null==(n=t),i):r?null:n},i.nodeSize=function(t){return arguments.length?(r=null!=(n=t),i):r?n:null},Ho(i,t)},o.layout.treemap=function(){var t,e=o.layout.hierarchy(),n=Math.round,r=[1,1],i=null,a=Ea,s=!1,A="squarify",l=.5*(1+Math.sqrt(5));function c(t,e){for(var n,r,i=-1,o=t.length;++i<o;)r=(n=t[i]).value*(e<0?0:e),n.area=isNaN(r)||r<=0?0:r}function u(t){var e=t.children;if(e&&e.length){var n,r,i,o=a(t),s=[],l=e.slice(),g=1/0,f="slice"===A?o.dx:"dice"===A?o.dy:"slice-dice"===A?1&t.depth?o.dy:o.dx:Math.min(o.dx,o.dy);for(c(l,o.dx*o.dy/t.value),s.area=0;(i=l.length)>0;)s.push(n=l[i-1]),s.area+=n.area,"squarify"!==A||(r=h(s,f))<=g?(l.pop(),g=r):(s.area-=s.pop().area,d(s,f,o,!1),f=Math.min(o.dx,o.dy),s.length=s.area=0,g=1/0);s.length&&(d(s,f,o,!0),s.length=s.area=0),e.forEach(u)}}function g(t){var e=t.children;if(e&&e.length){var n,r=a(t),i=e.slice(),o=[];for(c(i,r.dx*r.dy/t.value),o.area=0;n=i.pop();)o.push(n),o.area+=n.area,null!=n.z&&(d(o,n.z?r.dx:r.dy,r,!i.length),o.length=o.area=0);e.forEach(g)}}function h(t,e){for(var n,r=t.area,i=0,o=1/0,a=-1,s=t.length;++a<s;)(n=t[a].area)&&(n<o&&(o=n),n>i&&(i=n));return e*=e,(r*=r)?Math.max(e*i*l/r,r/(e*o*l)):1/0}function d(t,e,r,i){var o,a=-1,s=t.length,A=r.x,l=r.y,c=e?n(t.area/e):0;if(e==r.dx){for((i||c>r.dy)&&(c=r.dy);++a<s;)(o=t[a]).x=A,o.y=l,o.dy=c,A+=o.dx=Math.min(r.x+r.dx-A,c?n(o.area/c):0);o.z=!0,o.dx+=r.x+r.dx-A,r.y+=c,r.dy-=c}else{for((i||c>r.dx)&&(c=r.dx);++a<s;)(o=t[a]).x=A,o.y=l,o.dx=c,l+=o.dy=Math.min(r.y+r.dy-l,c?n(o.area/c):0);o.z=!1,o.dy+=r.y+r.dy-l,r.x+=c,r.dx-=c}}function f(n){var i=t||e(n),o=i[0];return o.x=o.y=0,o.value?(o.dx=r[0],o.dy=r[1]):o.dx=o.dy=0,t&&e.revalue(o),c([o],o.dx*o.dy/o.value),(t?g:u)(o),s&&(t=i),i}return f.size=function(t){return arguments.length?(r=t,f):r},f.padding=function(t){if(!arguments.length)return i;function e(e){var n=t.call(f,e,e.depth);return null==n?Ea(e):va(e,"number"==typeof n?[n,n,n,n]:n)}function n(e){return va(e,t)}var r;return a=null==(i=t)?Ea:"function"==(r=typeof t)?e:"number"===r?(t=[t,t,t,t],n):n,f},f.round=function(t){return arguments.length?(n=t?Math.round:Number,f):n!=Number},f.sticky=function(e){return arguments.length?(s=e,t=null,f):s},f.ratio=function(t){return arguments.length?(l=t,f):l},f.mode=function(t){return arguments.length?(A=t+"",f):A},Ho(f,e)},o.random={normal:function(t,e){var n=arguments.length;return n<2&&(e=1),n<1&&(t=0),function(){var n,r,i;do{i=(n=2*Math.random()-1)*n+(r=2*Math.random()-1)*r}while(!i||i>1);return t+e*n*Math.sqrt(-2*Math.log(i)/i)}},logNormal:function(){var t=o.random.normal.apply(o,arguments);return function(){return Math.exp(t())}},bates:function(t){var e=o.random.irwinHall(t);return function(){return e()/t}},irwinHall:function(t){return function(){for(var e=0,n=0;n<t;n++)e+=Math.random();return e}}},o.scale={};var _a={floor:F,ceil:F};function Da(t,e,n,r){var i=[],a=[],s=0,A=Math.min(t.length,e.length)-1;for(t[A]<t[0]&&(t=t.slice().reverse(),e=e.slice().reverse());++s<=A;)i.push(n(t[s-1],t[s])),a.push(r(e[s-1],e[s]));return function(e){var n=o.bisect(t,e,1,A)-1;return a[n](i[n](e))}}function Ma(t,e){return o.rebind(t,e,"range","rangeRound","interpolate","clamp")}function ka(t,e){return wa(t,xa(Sa(t,e)[2])),wa(t,xa(Sa(t,e)[2])),t}function Sa(t,e){null==e&&(e=10);var n=Ba(t),r=n[1]-n[0],i=Math.pow(10,Math.floor(Math.log(r/e)/Math.LN10)),o=e/r*i;return o<=.15?i*=10:o<=.35?i*=5:o<=.75&&(i*=2),n[0]=Math.ceil(n[0]/i)*i,n[1]=Math.floor(n[1]/i)*i+.5*i,n[2]=i,n}function La(t,e){return o.range.apply(o,Sa(t,e))}function Ra(t,e,n){var r=Sa(t,e);if(n){var i=Te.exec(n);if(i.shift(),"s"===i[8]){var a=o.formatPrefix(Math.max(v(r[0]),v(r[1])));return i[7]||(i[7]="."+Fa(a.scale(r[2]))),i[8]="f",n=o.format(i.join("")),function(t){return n(a.scale(t))+a.symbol}}i[7]||(i[7]="."+function(t,e){var n=Fa(e[2]);return t in Na?Math.abs(n-Fa(Math.max(v(e[0]),v(e[1]))))+ +("e"!==t):n-2*("%"===t)}(i[8],r)),n=i.join("")}else n=",."+Fa(r[2])+"f";return o.format(n)}o.scale.linear=function(){return function t(e,n,r,i){var o,a;function s(){var t=Math.min(e.length,n.length)>2?Da:Qa,s=i?ko:Mo;return o=t(e,n,s,r),a=t(n,e,s,so),A}function A(t){return o(t)}return A.invert=function(t){return a(t)},A.domain=function(t){return arguments.length?(e=t.map(Number),s()):e},A.range=function(t){return arguments.length?(n=t,s()):n},A.rangeRound=function(t){return A.range(t).interpolate(Bo)},A.clamp=function(t){return arguments.length?(i=t,s()):i},A.interpolate=function(t){return arguments.length?(r=t,s()):r},A.ticks=function(t){return La(e,t)},A.tickFormat=function(t,n){return Ra(e,t,n)},A.nice=function(t){return ka(e,t),s()},A.copy=function(){return t(e,n,r,i)},s()}([0,1],[0,1],so,!1)};var Na={s:1,g:1,p:1,r:1,e:1};function Fa(t){return-Math.floor(Math.log(t)/Math.LN10+.01)}o.scale.log=function(){return function t(e,n,r,i){function a(t){return(r?Math.log(t<0?0:t):-Math.log(t>0?0:-t))/Math.log(n)}function s(t){return r?Math.pow(n,t):-Math.pow(n,-t)}function A(t){return e(a(t))}return A.invert=function(t){return s(e.invert(t))},A.domain=function(t){return arguments.length?(r=t[0]>=0,e.domain((i=t.map(Number)).map(a)),A):i},A.base=function(t){return arguments.length?(n=+t,e.domain(i.map(a)),A):n},A.nice=function(){var t=wa(i.map(a),r?Math:Ua);return e.domain(t),i=t.map(s),A},A.ticks=function(){var t=Ba(i),e=[],o=t[0],A=t[1],l=Math.floor(a(o)),c=Math.ceil(a(A)),u=n%1?2:n;if(isFinite(c-l)){if(r){for(;l<c;l++)for(var g=1;g<u;g++)e.push(s(l)*g);e.push(s(l))}else for(e.push(s(l));l++<c;)for(g=u-1;g>0;g--)e.push(s(l)*g);for(l=0;e[l]<o;l++);for(c=e.length;e[c-1]>A;c--);e=e.slice(l,c)}return e},A.tickFormat=function(t,e){if(!arguments.length)return Ta;arguments.length<2?e=Ta:"function"!=typeof e&&(e=o.format(e));var r=Math.max(1,n*t/A.ticks().length);return function(t){var i=t/s(Math.round(a(t)));return i*n<n-.5&&(i*=n),i<=r?e(t):""}},A.copy=function(){return t(e.copy(),n,r,i)},Ma(A,e)}(o.scale.linear().domain([0,1]),10,!0,[1,10])};var Ta=o.format(".0e"),Ua={floor:function(t){return-Math.ceil(-t)},ceil:function(t){return-Math.floor(-t)}};function Oa(t){return function(e){return e<0?-Math.pow(-e,t):Math.pow(e,t)}}o.scale.pow=function(){return function t(e,n,r){var i=Oa(n),o=Oa(1/n);function a(t){return e(i(t))}return a.invert=function(t){return o(e.invert(t))},a.domain=function(t){return arguments.length?(e.domain((r=t.map(Number)).map(i)),a):r},a.ticks=function(t){return La(r,t)},a.tickFormat=function(t,e){return Ra(r,t,e)},a.nice=function(t){return a.domain(ka(r,t))},a.exponent=function(t){return arguments.length?(i=Oa(n=t),o=Oa(1/n),e.domain(r.map(i)),a):n},a.copy=function(){return t(e.copy(),n,r)},Ma(a,e)}(o.scale.linear(),1,[0,1])},o.scale.sqrt=function(){return o.scale.pow().exponent(.5)},o.scale.ordinal=function(){return function t(e,n){var r,i,a;function s(t){return i[((r.get(t)||("range"===n.t?r.set(t,e.push(t)):NaN))-1)%i.length]}function A(t,n){return o.range(e.length).map(function(e){return t+n*e})}return s.domain=function(t){if(!arguments.length)return e;e=[],r=new Q;for(var i,o=-1,a=t.length;++o<a;)r.has(i=t[o])||r.set(i,e.push(i));return s[n.t].apply(s,n.a)},s.range=function(t){return arguments.length?(i=t,a=0,n={t:"range",a:arguments},s):i},s.rangePoints=function(t,r){arguments.length<2&&(r=0);var o=t[0],l=t[1],c=e.length<2?(o=(o+l)/2,0):(l-o)/(e.length-1+r);return i=A(o+c*r/2,c),a=0,n={t:"rangePoints",a:arguments},s},s.rangeRoundPoints=function(t,r){arguments.length<2&&(r=0);var o=t[0],l=t[1],c=e.length<2?(o=l=Math.round((o+l)/2),0):(l-o)/(e.length-1+r)|0;return i=A(o+Math.round(c*r/2+(l-o-(e.length-1+r)*c)/2),c),a=0,n={t:"rangeRoundPoints",a:arguments},s},s.rangeBands=function(t,r,o){arguments.length<2&&(r=0),arguments.length<3&&(o=r);var l=t[1]<t[0],c=t[l-0],u=t[1-l],g=(u-c)/(e.length-r+2*o);return i=A(c+g*o,g),l&&i.reverse(),a=g*(1-r),n={t:"rangeBands",a:arguments},s},s.rangeRoundBands=function(t,r,o){arguments.length<2&&(r=0),arguments.length<3&&(o=r);var l=t[1]<t[0],c=t[l-0],u=t[1-l],g=Math.floor((u-c)/(e.length-r+2*o));return i=A(c+Math.round((u-c-(e.length-r)*g)/2),g),l&&i.reverse(),a=Math.round(g*(1-r)),n={t:"rangeRoundBands",a:arguments},s},s.rangeBand=function(){return a},s.rangeExtent=function(){return Ba(n.a[0])},s.copy=function(){return t(e,n)},s.domain(e)}([],{t:"range",a:[[]]})},o.scale.category10=function(){return o.scale.ordinal().range(Ga)},o.scale.category20=function(){return o.scale.ordinal().range(Ha)},o.scale.category20b=function(){return o.scale.ordinal().range(ja)},o.scale.category20c=function(){return o.scale.ordinal().range(Ya)};var Ga=[2062260,16744206,2924588,14034728,9725885,9197131,14907330,8355711,12369186,1556175].map(he),Ha=[2062260,11454440,16744206,16759672,2924588,10018698,14034728,16750742,9725885,12955861,9197131,12885140,14907330,16234194,8355711,13092807,12369186,14408589,1556175,10410725].map(he),ja=[3750777,5395619,7040719,10264286,6519097,9216594,11915115,13556636,9202993,12426809,15186514,15190932,8666169,11356490,14049643,15177372,8077683,10834324,13528509,14589654].map(he),Ya=[3244733,7057110,10406625,13032431,15095053,16616764,16625259,16634018,3253076,7652470,10607003,13101504,7695281,10394312,12369372,14342891,6513507,9868950,12434877,14277081].map(he);function Pa(){return 0}o.scale.quantile=function(){return function t(e,n){var r;function i(){var t=0,i=n.length;for(r=[];++t<i;)r[t-1]=o.quantile(e,t/i);return a}function a(t){if(!isNaN(t=+t))return n[o.bisect(r,t)]}return a.domain=function(t){return arguments.length?(e=t.map(I).filter(C).sort(p),i()):e},a.range=function(t){return arguments.length?(n=t,i()):n},a.quantiles=function(){return r},a.invertExtent=function(t){return(t=n.indexOf(t))<0?[NaN,NaN]:[t>0?r[t-1]:e[0],t<r.length?r[t]:e[e.length-1]]},a.copy=function(){return t(e,n)},i()}([],[])},o.scale.quantize=function(){return function t(e,n,r){var i,o;function a(t){return r[Math.max(0,Math.min(o,Math.floor(i*(t-e))))]}function s(){return i=r.length/(n-e),o=r.length-1,a}return a.domain=function(t){return arguments.length?(e=+t[0],n=+t[t.length-1],s()):[e,n]},a.range=function(t){return arguments.length?(r=t,s()):r},a.invertExtent=function(t){return[t=(t=r.indexOf(t))<0?NaN:t/i+e,t+1/i]},a.copy=function(){return t(e,n,r)},s()}(0,1,[0,1])},o.scale.threshold=function(){return function t(e,n){function r(t){if(t<=t)return n[o.bisect(e,t)]}return r.domain=function(t){return arguments.length?(e=t,r):e},r.range=function(t){return arguments.length?(n=t,r):n},r.invertExtent=function(t){return t=n.indexOf(t),[e[t-1],e[t]]},r.copy=function(){return t(e,n)},r}([.5],[0,1])},o.scale.identity=function(){return function t(e){function n(t){return+t}return n.invert=n,n.domain=n.range=function(t){return arguments.length?(e=t.map(n),n):e},n.ticks=function(t){return La(e,t)},n.tickFormat=function(t,n){return Ra(e,t,n)},n.copy=function(){return t(e)},n}([0,1])},o.svg={},o.svg.arc=function(){var t=za,e=Ja,n=Pa,r=Ka,i=qa,o=Va,a=Wa;function s(){var s=Math.max(0,+t.apply(this,arguments)),l=Math.max(0,+e.apply(this,arguments)),c=i.apply(this,arguments)-Nt,u=o.apply(this,arguments)-Nt,g=Math.abs(u-c),h=c>u?0:1;if(l<s&&(d=l,l=s,s=d),g>=Rt)return A(l,h)+(s?A(s,1-h):"")+"Z";var d,f,p,I,C,m,y,E,v,B,b,Q,w=0,x=0,_=[];if((I=(+a.apply(this,arguments)||0)/2)&&(p=r===Ka?Math.sqrt(s*s+l*l):+r.apply(this,arguments),h||(x*=-1),l&&(x=Ht(p/l*Math.sin(I))),s&&(w=Ht(p/s*Math.sin(I)))),l){C=l*Math.cos(c+x),m=l*Math.sin(c+x),y=l*Math.cos(u-x),E=l*Math.sin(u-x);var D=Math.abs(u-c-2*x)<=St?0:1;if(x&&Xa(C,m,y,E)===h^D){var M=(c+u)/2;C=l*Math.cos(M),m=l*Math.sin(M),y=E=null}}else C=m=0;if(s){v=s*Math.cos(u-w),B=s*Math.sin(u-w),b=s*Math.cos(c+w),Q=s*Math.sin(c+w);var k=Math.abs(c-u+2*w)<=St?0:1;if(w&&Xa(v,B,b,Q)===1-h^k){var S=(c+u)/2;v=s*Math.cos(S),B=s*Math.sin(S),b=Q=null}}else v=B=0;if(g>Mt&&(d=Math.min(Math.abs(l-s)/2,+n.apply(this,arguments)))>.001){f=s<l^h?0:1;var L=d,R=d;if(g<St){var N=null==b?[v,B]:null==y?[C,m]:Ii([C,m],[b,Q],[y,E],[v,B]),F=C-N[0],T=m-N[1],U=y-N[0],O=E-N[1],G=1/Math.sin(Math.acos((F*U+T*O)/(Math.sqrt(F*F+T*T)*Math.sqrt(U*U+O*O)))/2),H=Math.sqrt(N[0]*N[0]+N[1]*N[1]);R=Math.min(d,(s-H)/(G-1)),L=Math.min(d,(l-H)/(G+1))}if(null!=y){var j=Za(null==b?[v,B]:[b,Q],[C,m],l,L,h),Y=Za([y,E],[v,B],l,L,h);d===L?_.push("M",j[0],"A",L,",",L," 0 0,",f," ",j[1],"A",l,",",l," 0 ",1-h^Xa(j[1][0],j[1][1],Y[1][0],Y[1][1]),",",h," ",Y[1],"A",L,",",L," 0 0,",f," ",Y[0]):_.push("M",j[0],"A",L,",",L," 0 1,",f," ",Y[0])}else _.push("M",C,",",m);if(null!=b){var P=Za([C,m],[b,Q],s,-R,h),K=Za([v,B],null==y?[C,m]:[y,E],s,-R,h);d===R?_.push("L",K[0],"A",R,",",R," 0 0,",f," ",K[1],"A",s,",",s," 0 ",h^Xa(K[1][0],K[1][1],P[1][0],P[1][1]),",",1-h," ",P[1],"A",R,",",R," 0 0,",f," ",P[0]):_.push("L",K[0],"A",R,",",R," 0 0,",f," ",P[0])}else _.push("L",v,",",B)}else _.push("M",C,",",m),null!=y&&_.push("A",l,",",l," 0 ",D,",",h," ",y,",",E),_.push("L",v,",",B),null!=b&&_.push("A",s,",",s," 0 ",k,",",1-h," ",b,",",Q);return _.push("Z"),_.join("")}function A(t,e){return"M0,"+t+"A"+t+","+t+" 0 1,"+e+" 0,"+-t+"A"+t+","+t+" 0 1,"+e+" 0,"+t}return s.innerRadius=function(e){return arguments.length?(t=ve(e),s):t},s.outerRadius=function(t){return arguments.length?(e=ve(t),s):e},s.cornerRadius=function(t){return arguments.length?(n=ve(t),s):n},s.padRadius=function(t){return arguments.length?(r=t==Ka?Ka:ve(t),s):r},s.startAngle=function(t){return arguments.length?(i=ve(t),s):i},s.endAngle=function(t){return arguments.length?(o=ve(t),s):o},s.padAngle=function(t){return arguments.length?(a=ve(t),s):a},s.centroid=function(){var n=(+t.apply(this,arguments)+ +e.apply(this,arguments))/2,r=(+i.apply(this,arguments)+ +o.apply(this,arguments))/2-Nt;return[Math.cos(r)*n,Math.sin(r)*n]},s};var Ka="auto";function za(t){return t.innerRadius}function Ja(t){return t.outerRadius}function qa(t){return t.startAngle}function Va(t){return t.endAngle}function Wa(t){return t&&t.padAngle}function Xa(t,e,n,r){return(t-n)*e-(e-r)*t>0?0:1}function Za(t,e,n,r,i){var o=t[0]-e[0],a=t[1]-e[1],s=(i?r:-r)/Math.sqrt(o*o+a*a),A=s*a,l=-s*o,c=t[0]+A,u=t[1]+l,g=e[0]+A,h=e[1]+l,d=(c+g)/2,f=(u+h)/2,p=g-c,I=h-u,C=p*p+I*I,m=n-r,y=c*h-g*u,E=(I<0?-1:1)*Math.sqrt(Math.max(0,m*m*C-y*y)),v=(y*I-p*E)/C,B=(-y*p-I*E)/C,b=(y*I+p*E)/C,Q=(-y*p+I*E)/C,w=v-d,x=B-f,_=b-d,D=Q-f;return w*w+x*x>_*_+D*D&&(v=b,B=Q),[[v-A,B-l],[v*n/m,B*n/m]]}function $a(t){var e=ui,n=gi,r=er,i=es,o=i.key,a=.7;function s(o){var s,A=[],l=[],c=-1,u=o.length,g=ve(e),h=ve(n);function d(){A.push("M",i(t(l),a))}for(;++c<u;)r.call(this,s=o[c],c)?l.push([+g.call(this,s,c),+h.call(this,s,c)]):l.length&&(d(),l=[]);return l.length&&d(),A.length?A.join(""):null}return s.x=function(t){return arguments.length?(e=t,s):e},s.y=function(t){return arguments.length?(n=t,s):n},s.defined=function(t){return arguments.length?(r=t,s):r},s.interpolate=function(t){return arguments.length?(o="function"==typeof t?i=t:(i=ts.get(t)||es).key,s):o},s.tension=function(t){return arguments.length?(a=t,s):a},s}o.svg.line=function(){return $a(F)};var ts=o.map({linear:es,"linear-closed":ns,step:function(t){for(var e=0,n=t.length,r=t[0],i=[r[0],",",r[1]];++e<n;)i.push("H",(r[0]+(r=t[e])[0])/2,"V",r[1]);return n>1&&i.push("H",r[0]),i.join("")},"step-before":rs,"step-after":is,basis:ss,"basis-open":function(t){if(t.length<4)return es(t);for(var e,n=[],r=-1,i=t.length,o=[0],a=[0];++r<3;)e=t[r],o.push(e[0]),a.push(e[1]);for(n.push(As(us,o)+","+As(us,a)),--r;++r<i;)e=t[r],o.shift(),o.push(e[0]),a.shift(),a.push(e[1]),gs(n,o,a);return n.join("")},"basis-closed":function(t){for(var e,n,r=-1,i=t.length,o=i+4,a=[],s=[];++r<4;)n=t[r%i],a.push(n[0]),s.push(n[1]);for(e=[As(us,a),",",As(us,s)],--r;++r<o;)n=t[r%i],a.shift(),a.push(n[0]),s.shift(),s.push(n[1]),gs(e,a,s);return e.join("")},bundle:function(t,e){var n=t.length-1;if(n)for(var r,i,o=t[0][0],a=t[0][1],s=t[n][0]-o,A=t[n][1]-a,l=-1;++l<=n;)i=l/n,(r=t[l])[0]=e*r[0]+(1-e)*(o+i*s),r[1]=e*r[1]+(1-e)*(a+i*A);return ss(t)},cardinal:function(t,e){return t.length<3?es(t):t[0]+os(t,as(t,e))},"cardinal-open":function(t,e){return t.length<4?es(t):t[1]+os(t.slice(1,-1),as(t,e))},"cardinal-closed":function(t,e){return t.length<3?ns(t):t[0]+os((t.push(t[0]),t),as([t[t.length-2]].concat(t,[t[1]]),e))},monotone:function(t){return t.length<3?es(t):t[0]+os(t,function(t){for(var e,n,r,i,o=[],a=function(t){for(var e=0,n=t.length-1,r=[],i=t[0],o=t[1],a=r[0]=hs(i,o);++e<n;)r[e]=(a+(a=hs(i=o,o=t[e+1])))/2;return r[e]=a,r}(t),s=-1,A=t.length-1;++s<A;)e=hs(t[s],t[s+1]),v(e)<Mt?a[s]=a[s+1]=0:(i=(n=a[s]/e)*n+(r=a[s+1]/e)*r)>9&&(i=3*e/Math.sqrt(i),a[s]=i*n,a[s+1]=i*r);for(s=-1;++s<=A;)i=(t[Math.min(A,s+1)][0]-t[Math.max(0,s-1)][0])/(6*(1+a[s]*a[s])),o.push([i||0,a[s]*i||0]);return o}(t))}});function es(t){return t.length>1?t.join("L"):t+"Z"}function ns(t){return t.join("L")+"Z"}function rs(t){for(var e=0,n=t.length,r=t[0],i=[r[0],",",r[1]];++e<n;)i.push("V",(r=t[e])[1],"H",r[0]);return i.join("")}function is(t){for(var e=0,n=t.length,r=t[0],i=[r[0],",",r[1]];++e<n;)i.push("H",(r=t[e])[0],"V",r[1]);return i.join("")}function os(t,e){if(e.length<1||t.length!=e.length&&t.length!=e.length+2)return es(t);var n=t.length!=e.length,r="",i=t[0],o=t[1],a=e[0],s=a,A=1;if(n&&(r+="Q"+(o[0]-2*a[0]/3)+","+(o[1]-2*a[1]/3)+","+o[0]+","+o[1],i=t[1],A=2),e.length>1){s=e[1],o=t[A],A++,r+="C"+(i[0]+a[0])+","+(i[1]+a[1])+","+(o[0]-s[0])+","+(o[1]-s[1])+","+o[0]+","+o[1];for(var l=2;l<e.length;l++,A++)o=t[A],s=e[l],r+="S"+(o[0]-s[0])+","+(o[1]-s[1])+","+o[0]+","+o[1]}if(n){var c=t[A];r+="Q"+(o[0]+2*s[0]/3)+","+(o[1]+2*s[1]/3)+","+c[0]+","+c[1]}return r}function as(t,e){for(var n,r=[],i=(1-e)/2,o=t[0],a=t[1],s=1,A=t.length;++s<A;)n=o,o=a,a=t[s],r.push([i*(a[0]-n[0]),i*(a[1]-n[1])]);return r}function ss(t){if(t.length<3)return es(t);var e=1,n=t.length,r=t[0],i=r[0],o=r[1],a=[i,i,i,(r=t[1])[0]],s=[o,o,o,r[1]],A=[i,",",o,"L",As(us,a),",",As(us,s)];for(t.push(t[n-1]);++e<=n;)r=t[e],a.shift(),a.push(r[0]),s.shift(),s.push(r[1]),gs(A,a,s);return t.pop(),A.push("L",r),A.join("")}function As(t,e){return t[0]*e[0]+t[1]*e[1]+t[2]*e[2]+t[3]*e[3]}ts.forEach(function(t,e){e.key=t,e.closed=/-closed$/.test(t)});var ls=[0,2/3,1/3,0],cs=[0,1/3,2/3,0],us=[0,1/6,2/3,1/6];function gs(t,e,n){t.push("C",As(ls,e),",",As(ls,n),",",As(cs,e),",",As(cs,n),",",As(us,e),",",As(us,n))}function hs(t,e){return(e[1]-t[1])/(e[0]-t[0])}function ds(t){for(var e,n,r,i=-1,o=t.length;++i<o;)n=(e=t[i])[0],r=e[1]-Nt,e[0]=n*Math.cos(r),e[1]=n*Math.sin(r);return t}function fs(t){var e=ui,n=ui,r=0,i=gi,o=er,a=es,s=a.key,A=a,l="L",c=.7;function u(s){var u,g,h,d=[],f=[],p=[],I=-1,C=s.length,m=ve(e),y=ve(r),E=e===n?function(){return g}:ve(n),v=r===i?function(){return h}:ve(i);function B(){d.push("M",a(t(p),c),l,A(t(f.reverse()),c),"Z")}for(;++I<C;)o.call(this,u=s[I],I)?(f.push([g=+m.call(this,u,I),h=+y.call(this,u,I)]),p.push([+E.call(this,u,I),+v.call(this,u,I)])):f.length&&(B(),f=[],p=[]);return f.length&&B(),d.length?d.join(""):null}return u.x=function(t){return arguments.length?(e=n=t,u):n},u.x0=function(t){return arguments.length?(e=t,u):e},u.x1=function(t){return arguments.length?(n=t,u):n},u.y=function(t){return arguments.length?(r=i=t,u):i},u.y0=function(t){return arguments.length?(r=t,u):r},u.y1=function(t){return arguments.length?(i=t,u):i},u.defined=function(t){return arguments.length?(o=t,u):o},u.interpolate=function(t){return arguments.length?(s="function"==typeof t?a=t:(a=ts.get(t)||es).key,A=a.reverse||a,l=a.closed?"M":"L",u):s},u.tension=function(t){return arguments.length?(c=t,u):c},u}function ps(t){return t.radius}function Is(t){return[t.x,t.y]}function Cs(t){return function(){var e=t.apply(this,arguments),n=e[0],r=e[1]-Nt;return[n*Math.cos(r),n*Math.sin(r)]}}function ms(){return 64}function ys(){return"circle"}function Es(t){var e=Math.sqrt(t/St);return"M0,"+e+"A"+e+","+e+" 0 1,1 0,"+-e+"A"+e+","+e+" 0 1,1 0,"+e+"Z"}o.svg.line.radial=function(){var t=$a(ds);return t.radius=t.x,delete t.x,t.angle=t.y,delete t.y,t},rs.reverse=is,is.reverse=rs,o.svg.area=function(){return fs(F)},o.svg.area.radial=function(){var t=fs(ds);return t.radius=t.x,delete t.x,t.innerRadius=t.x0,delete t.x0,t.outerRadius=t.x1,delete t.x1,t.angle=t.y,delete t.y,t.startAngle=t.y0,delete t.y0,t.endAngle=t.y1,delete t.y1,t},o.svg.chord=function(){var t=Xr,e=Zr,n=ps,r=qa,i=Va;function o(n,r){var i,o,l=a(this,t,n,r),c=a(this,e,n,r);return"M"+l.p0+s(l.r,l.p1,l.a1-l.a0)+(o=c,(i=l).a0==o.a0&&i.a1==o.a1?A(l.r,l.p1,l.r,l.p0):A(l.r,l.p1,c.r,c.p0)+s(c.r,c.p1,c.a1-c.a0)+A(c.r,c.p1,l.r,l.p0))+"Z"}function a(t,e,o,a){var s=e.call(t,o,a),A=n.call(t,s,a),l=r.call(t,s,a)-Nt,c=i.call(t,s,a)-Nt;return{r:A,a0:l,a1:c,p0:[A*Math.cos(l),A*Math.sin(l)],p1:[A*Math.cos(c),A*Math.sin(c)]}}function s(t,e,n){return"A"+t+","+t+" 0 "+ +(n>St)+",1 "+e}function A(t,e,n,r){return"Q 0,0 "+r}return o.radius=function(t){return arguments.length?(n=ve(t),o):n},o.source=function(e){return arguments.length?(t=ve(e),o):t},o.target=function(t){return arguments.length?(e=ve(t),o):e},o.startAngle=function(t){return arguments.length?(r=ve(t),o):r},o.endAngle=function(t){return arguments.length?(i=ve(t),o):i},o},o.svg.diagonal=function(){var t=Xr,e=Zr,n=Is;function r(r,i){var o=t.call(this,r,i),a=e.call(this,r,i),s=(o.y+a.y)/2,A=[o,{x:o.x,y:s},{x:a.x,y:s},a];return"M"+(A=A.map(n))[0]+"C"+A[1]+" "+A[2]+" "+A[3]}return r.source=function(e){return arguments.length?(t=ve(e),r):t},r.target=function(t){return arguments.length?(e=ve(t),r):e},r.projection=function(t){return arguments.length?(n=t,r):n},r},o.svg.diagonal.radial=function(){var t=o.svg.diagonal(),e=Is,n=t.projection;return t.projection=function(t){return arguments.length?n(Cs(e=t)):e},t},o.svg.symbol=function(){var t=ys,e=ms;function n(n,r){return(vs.get(t.call(this,n,r))||Es)(e.call(this,n,r))}return n.type=function(e){return arguments.length?(t=ve(e),n):t},n.size=function(t){return arguments.length?(e=ve(t),n):e},n};var vs=o.map({circle:Es,cross:function(t){var e=Math.sqrt(t/5)/2;return"M"+-3*e+","+-e+"H"+-e+"V"+-3*e+"H"+e+"V"+-e+"H"+3*e+"V"+e+"H"+e+"V"+3*e+"H"+-e+"V"+e+"H"+-3*e+"Z"},diamond:function(t){var e=Math.sqrt(t/(2*bs)),n=e*bs;return"M0,"+-e+"L"+n+",0 0,"+e+" "+-n+",0Z"},square:function(t){var e=Math.sqrt(t)/2;return"M"+-e+","+-e+"L"+e+","+-e+" "+e+","+e+" "+-e+","+e+"Z"},"triangle-down":function(t){var e=Math.sqrt(t/Bs),n=e*Bs/2;return"M0,"+n+"L"+e+","+-n+" "+-e+","+-n+"Z"},"triangle-up":function(t){var e=Math.sqrt(t/Bs),n=e*Bs/2;return"M0,"+-n+"L"+e+","+n+" "+-e+","+n+"Z"}});o.svg.symbolTypes=vs.keys();var Bs=Math.sqrt(3),bs=Math.tan(30*Ft);Z.transition=function(t){for(var e,n,r=_s||++ks,i=Rs(t),o=[],a=Ds||{time:Date.now(),ease:Co,delay:0,duration:250},s=-1,A=this.length;++s<A;){o.push(e=[]);for(var l=this[s],c=-1,u=l.length;++c<u;)(n=l[c])&&Ns(n,c,i,r,a),e.push(n)}return xs(o,i,r)},Z.interrupt=function(t){return this.each(null==t?Qs:ws(Rs(t)))};var Qs=ws(Rs());function ws(t){return function(){var e,n,r;(e=this[t])&&(r=e[n=e.active])&&(r.timer.c=null,r.timer.t=NaN,--e.count?delete e[n]:delete this[t],e.active+=.5,r.event&&r.event.interrupt.call(this,this.__data__,r.index))}}function xs(t,e,n){return J(t,Ms),t.namespace=e,t.id=n,t}var _s,Ds,Ms=[],ks=0;function Ss(t,e,n,r){var i=t.id,o=t.namespace;return pt(t,"function"==typeof n?function(t,a,s){t[o][i].tween.set(e,r(n.call(t,t.__data__,a,s)))}:(n=r(n),function(t){t[o][i].tween.set(e,n)}))}function Ls(t){return null==t&&(t=""),function(){this.textContent=t}}function Rs(t){return null==t?"__transition__":"__transition_"+t+"__"}function Ns(t,e,n,r,i){var o,a,s,A,l,c=t[n]||(t[n]={active:0,count:0}),u=c[r];function g(n){var i=c.active,g=c[i];for(var d in g&&(g.timer.c=null,g.timer.t=NaN,--c.count,delete c[i],g.event&&g.event.interrupt.call(t,t.__data__,g.index)),c)if(+d<r){var f=c[d];f.timer.c=null,f.timer.t=NaN,--c.count,delete c[d]}a.c=h,Me(function(){return a.c&&h(n||1)&&(a.c=null,a.t=NaN),1},0,o),c.active=r,u.event&&u.event.start.call(t,t.__data__,e),l=[],u.tween.forEach(function(n,r){(r=r.call(t,t.__data__,e))&&l.push(r)}),A=u.ease,s=u.duration}function h(i){for(var o=i/s,a=A(o),g=l.length;g>0;)l[--g].call(t,a);if(o>=1)return u.event&&u.event.end.call(t,t.__data__,e),--c.count?delete c[r]:delete t[n],1}u||(o=i.time,a=Me(function(t){var e=u.delay;if(a.t=e+o,e<=t)return g(t-e);a.c=g},0,o),u=c[r]={tween:new Q,time:o,timer:a,delay:i.delay,duration:i.duration,ease:i.ease,index:e},i=null,++c.count)}Ms.call=Z.call,Ms.empty=Z.empty,Ms.node=Z.node,Ms.size=Z.size,o.transition=function(t,e){return t&&t.transition?_s?t.transition(e):t:o.selection().transition(t)},o.transition.prototype=Ms,Ms.select=function(t){var e,n,r,i=this.id,o=this.namespace,a=[];t=$(t);for(var s=-1,A=this.length;++s<A;){a.push(e=[]);for(var l=this[s],c=-1,u=l.length;++c<u;)(r=l[c])&&(n=t.call(r,r.__data__,c,s))?("__data__"in r&&(n.__data__=r.__data__),Ns(n,c,o,i,r[o][i]),e.push(n)):e.push(null)}return xs(a,o,i)},Ms.selectAll=function(t){var e,n,r,i,o,a=this.id,s=this.namespace,A=[];t=tt(t);for(var l=-1,c=this.length;++l<c;)for(var u=this[l],g=-1,h=u.length;++g<h;)if(r=u[g]){o=r[s][a],n=t.call(r,r.__data__,g,l),A.push(e=[]);for(var d=-1,f=n.length;++d<f;)(i=n[d])&&Ns(i,d,s,a,o),e.push(i)}return xs(A,s,a)},Ms.filter=function(t){var e,n,r=[];"function"!=typeof t&&(t=dt(t));for(var i=0,o=this.length;i<o;i++){r.push(e=[]);for(var a,s=0,A=(a=this[i]).length;s<A;s++)(n=a[s])&&t.call(n,n.__data__,s,i)&&e.push(n)}return xs(r,this.namespace,this.id)},Ms.tween=function(t,e){var n=this.id,r=this.namespace;return arguments.length<2?this.node()[r][n].tween.get(t):pt(this,null==e?function(e){e[r][n].tween.remove(t)}:function(i){i[r][n].tween.set(t,e)})},Ms.attr=function(t,e){if(arguments.length<2){for(e in t)this.attr(e,t[e]);return this}var n="transform"==t?Do:so,r=o.ns.qualify(t);function i(){this.removeAttribute(r)}function a(){this.removeAttributeNS(r.space,r.local)}function s(t){return null==t?i:(t+="",function(){var e,i=this.getAttribute(r);return i!==t&&(e=n(i,t),function(t){this.setAttribute(r,e(t))})})}function A(t){return null==t?a:(t+="",function(){var e,i=this.getAttributeNS(r.space,r.local);return i!==t&&(e=n(i,t),function(t){this.setAttributeNS(r.space,r.local,e(t))})})}return Ss(this,"attr."+t,e,r.local?A:s)},Ms.attrTween=function(t,e){var n=o.ns.qualify(t);return this.tween("attr."+t,n.local?function(t,r){var i=e.call(this,t,r,this.getAttributeNS(n.space,n.local));return i&&function(t){this.setAttributeNS(n.space,n.local,i(t))}}:function(t,r){var i=e.call(this,t,r,this.getAttribute(n));return i&&function(t){this.setAttribute(n,i(t))}})},Ms.style=function(t,e,n){var r=arguments.length;if(r<3){if("string"!=typeof t){for(n in r<2&&(e=""),t)this.style(n,t[n],e);return this}n=""}function i(){this.style.removeProperty(t)}function o(e){return null==e?i:(e+="",function(){var r,i=c(this).getComputedStyle(this,null).getPropertyValue(t);return i!==e&&(r=so(i,e),function(e){this.style.setProperty(t,r(e),n)})})}return Ss(this,"style."+t,e,o)},Ms.styleTween=function(t,e,n){function r(r,i){var o=e.call(this,r,i,c(this).getComputedStyle(this,null).getPropertyValue(t));return o&&function(e){this.style.setProperty(t,o(e),n)}}return arguments.length<3&&(n=""),this.tween("style."+t,r)},Ms.text=function(t){return Ss(this,"text",t,Ls)},Ms.remove=function(){var t=this.namespace;return this.each("end.transition",function(){var e;this[t].count<2&&(e=this.parentNode)&&e.removeChild(this)})},Ms.ease=function(t){var e=this.id,n=this.namespace;return arguments.length<1?this.node()[n][e].ease:("function"!=typeof t&&(t=o.ease.apply(o,arguments)),pt(this,function(r){r[n][e].ease=t}))},Ms.delay=function(t){var e=this.id,n=this.namespace;return arguments.length<1?this.node()[n][e].delay:pt(this,"function"==typeof t?function(r,i,o){r[n][e].delay=+t.call(r,r.__data__,i,o)}:(t=+t,function(r){r[n][e].delay=t}))},Ms.duration=function(t){var e=this.id,n=this.namespace;return arguments.length<1?this.node()[n][e].duration:pt(this,"function"==typeof t?function(r,i,o){r[n][e].duration=Math.max(1,t.call(r,r.__data__,i,o))}:(t=Math.max(1,t),function(r){r[n][e].duration=t}))},Ms.each=function(t,e){var n=this.id,r=this.namespace;if(arguments.length<2){var i=Ds,a=_s;try{_s=n,pt(this,function(e,i,o){Ds=e[r][n],t.call(e,e.__data__,i,o)})}finally{Ds=i,_s=a}}else pt(this,function(i){var a=i[r][n];(a.event||(a.event=o.dispatch("start","end","interrupt"))).on(t,e)});return this},Ms.transition=function(){for(var t,e,n,r=this.id,i=++ks,o=this.namespace,a=[],s=0,A=this.length;s<A;s++){a.push(t=[]);for(var l,c=0,u=(l=this[s]).length;c<u;c++)(e=l[c])&&Ns(e,c,o,i,{time:(n=e[o][r]).time,ease:n.ease,delay:n.delay+n.duration,duration:n.duration}),t.push(e)}return xs(a,o,i)},o.svg.axis=function(){var t,e=o.scale.linear(),n=Fs,r=6,i=6,a=3,A=[10],l=null;function c(s){s.each(function(){var s,c=o.select(this),u=this.__chart__||e,g=this.__chart__=e.copy(),h=null==l?g.ticks?g.ticks.apply(g,A):g.domain():l,d=null==t?g.tickFormat?g.tickFormat.apply(g,A):F:t,f=c.selectAll(".tick").data(h,g),p=f.enter().insert("g",".domain").attr("class","tick").style("opacity",Mt),I=o.transition(f.exit()).style("opacity",Mt).remove(),C=o.transition(f.order()).style("opacity",1),m=Math.max(r,0)+a,y=ba(g),E=c.selectAll(".domain").data([0]),v=(E.enter().append("path").attr("class","domain"),o.transition(E));p.append("line"),p.append("text");var B,b,Q,w,x=p.select("line"),_=C.select("line"),D=f.select("text").text(d),M=p.select("text"),k=C.select("text"),S="top"===n||"left"===n?-1:1;if("bottom"===n||"top"===n?(s=Us,B="x",Q="y",b="x2",w="y2",D.attr("dy",S<0?"0em":".71em").style("text-anchor","middle"),v.attr("d","M"+y[0]+","+S*i+"V0H"+y[1]+"V"+S*i)):(s=Os,B="y",Q="x",b="y2",w="x2",D.attr("dy",".32em").style("text-anchor",S<0?"end":"start"),v.attr("d","M"+S*i+","+y[0]+"H0V"+y[1]+"H"+S*i)),x.attr(w,S*r),M.attr(Q,S*m),_.attr(b,0).attr(w,S*r),k.attr(B,0).attr(Q,S*m),g.rangeBand){var L=g,R=L.rangeBand()/2;u=g=function(t){return L(t)+R}}else u.rangeBand?u=g:I.call(s,g,u);p.call(s,u,g),C.call(s,g,g)})}return c.scale=function(t){return arguments.length?(e=t,c):e},c.orient=function(t){return arguments.length?(n=t in Ts?t+"":Fs,c):n},c.ticks=function(){return arguments.length?(A=s(arguments),c):A},c.tickValues=function(t){return arguments.length?(l=t,c):l},c.tickFormat=function(e){return arguments.length?(t=e,c):t},c.tickSize=function(t){var e=arguments.length;return e?(r=+t,i=+arguments[e-1],c):r},c.innerTickSize=function(t){return arguments.length?(r=+t,c):r},c.outerTickSize=function(t){return arguments.length?(i=+t,c):i},c.tickPadding=function(t){return arguments.length?(a=+t,c):a},c.tickSubdivide=function(){return arguments.length&&c},c};var Fs="bottom",Ts={top:1,right:1,bottom:1,left:1};function Us(t,e,n){t.attr("transform",function(t){var r=e(t);return"translate("+(isFinite(r)?r:n(t))+",0)"})}function Os(t,e,n){t.attr("transform",function(t){var r=e(t);return"translate(0,"+(isFinite(r)?r:n(t))+")"})}o.svg.brush=function(){var t,e,n=K(g,"brushstart","brush","brushend"),r=null,i=null,a=[0,0],s=[0,0],A=!0,l=!0,u=Hs[0];function g(t){t.each(function(){var t=o.select(this).style("pointer-events","all").style("-webkit-tap-highlight-color","rgba(0,0,0,0)").on("mousedown.brush",p).on("touchstart.brush",p),e=t.selectAll(".background").data([0]);e.enter().append("rect").attr("class","background").style("visibility","hidden").style("cursor","crosshair"),t.selectAll(".extent").data([0]).enter().append("rect").attr("class","extent").style("cursor","move");var n=t.selectAll(".resize").data(u,F);n.exit().remove(),n.enter().append("g").attr("class",function(t){return"resize "+t}).style("cursor",function(t){return Gs[t]}).append("rect").attr("x",function(t){return/[ew]$/.test(t)?-3:null}).attr("y",function(t){return/^[ns]/.test(t)?-3:null}).attr("width",6).attr("height",6).style("visibility","hidden"),n.style("display",g.empty()?"none":null);var a,s=o.transition(t),A=o.transition(e);r&&(a=ba(r),A.attr("x",a[0]).attr("width",a[1]-a[0]),d(s)),i&&(a=ba(i),A.attr("y",a[0]).attr("height",a[1]-a[0]),f(s)),h(s)})}function h(t){t.selectAll(".resize").attr("transform",function(t){return"translate("+a[+/e$/.test(t)]+","+s[+/^s/.test(t)]+")"})}function d(t){t.select(".extent").attr("x",a[0]),t.selectAll(".extent,.n>rect,.s>rect").attr("width",a[1]-a[0])}function f(t){t.select(".extent").attr("y",s[0]),t.selectAll(".extent,.e>rect,.w>rect").attr("height",s[1]-s[0])}function p(){var u,p,I=this,C=o.select(o.event.target),m=n.of(I,arguments),y=o.select(I),E=C.datum(),v=!/^(n|s)$/.test(E)&&r,B=!/^(e|w)$/.test(E)&&i,b=C.classed("extent"),Q=wt(I),w=o.mouse(I),x=o.select(c(I)).on("keydown.brush",function(){32==o.event.keyCode&&(b||(u=null,w[0]-=a[1],w[1]-=s[1],b=2),Y())}).on("keyup.brush",function(){32==o.event.keyCode&&2==b&&(w[0]+=a[1],w[1]+=s[1],b=0,Y())});if(o.event.changedTouches?x.on("touchmove.brush",M).on("touchend.brush",S):x.on("mousemove.brush",M).on("mouseup.brush",S),y.interrupt().selectAll("*").interrupt(),b)w[0]=a[0]-w[0],w[1]=s[0]-w[1];else if(E){var _=+/w$/.test(E),D=+/^n/.test(E);p=[a[1-_]-w[0],s[1-D]-w[1]],w[0]=a[_],w[1]=s[D]}else o.event.altKey&&(u=w.slice());function M(){var t=o.mouse(I),e=!1;p&&(t[0]+=p[0],t[1]+=p[1]),b||(o.event.altKey?(u||(u=[(a[0]+a[1])/2,(s[0]+s[1])/2]),w[0]=a[+(t[0]<u[0])],w[1]=s[+(t[1]<u[1])]):u=null),v&&k(t,r,0)&&(d(y),e=!0),B&&k(t,i,1)&&(f(y),e=!0),e&&(h(y),m({type:"brush",mode:b?"move":"resize"}))}function k(n,r,i){var o,c,g=ba(r),h=g[0],d=g[1],f=w[i],p=i?s:a,I=p[1]-p[0];if(b&&(h-=f,d-=I+f),o=(i?l:A)?Math.max(h,Math.min(d,n[i])):n[i],b?c=(o+=f)+I:(u&&(f=Math.max(h,Math.min(d,2*u[i]-o))),f<o?(c=o,o=f):c=f),p[0]!=o||p[1]!=c)return i?e=null:t=null,p[0]=o,p[1]=c,!0}function S(){M(),y.style("pointer-events","all").selectAll(".resize").style("display",g.empty()?"none":null),o.select("body").style("cursor",null),x.on("mousemove.brush",null).on("mouseup.brush",null).on("touchmove.brush",null).on("touchend.brush",null).on("keydown.brush",null).on("keyup.brush",null),Q(),m({type:"brushend"})}y.style("pointer-events","none").selectAll(".resize").style("display",null),o.select("body").style("cursor",C.style("cursor")),m({type:"brushstart"}),M()}return g.event=function(r){r.each(function(){var r=n.of(this,arguments),i={x:a,y:s,i:t,j:e},A=this.__chart__||i;this.__chart__=i,_s?o.select(this).transition().each("start.brush",function(){t=A.i,e=A.j,a=A.x,s=A.y,r({type:"brushstart"})}).tween("brush:brush",function(){var n=Ao(a,i.x),o=Ao(s,i.y);return t=e=null,function(t){a=i.x=n(t),s=i.y=o(t),r({type:"brush",mode:"resize"})}}).each("end.brush",function(){t=i.i,e=i.j,r({type:"brush",mode:"resize"}),r({type:"brushend"})}):(r({type:"brushstart"}),r({type:"brush",mode:"resize"}),r({type:"brushend"}))})},g.x=function(t){return arguments.length?(u=Hs[!(r=t)<<1|!i],g):r},g.y=function(t){return arguments.length?(u=Hs[!r<<1|!(i=t)],g):i},g.clamp=function(t){return arguments.length?(r&&i?(A=!!t[0],l=!!t[1]):r?A=!!t:i&&(l=!!t),g):r&&i?[A,l]:r?A:i?l:null},g.extent=function(n){var o,A,l,c,u;return arguments.length?(r&&(o=n[0],A=n[1],i&&(o=o[0],A=A[0]),t=[o,A],r.invert&&(o=r(o),A=r(A)),A<o&&(u=o,o=A,A=u),o==a[0]&&A==a[1]||(a=[o,A])),i&&(l=n[0],c=n[1],r&&(l=l[1],c=c[1]),e=[l,c],i.invert&&(l=i(l),c=i(c)),c<l&&(u=l,l=c,c=u),l==s[0]&&c==s[1]||(s=[l,c])),g):(r&&(t?(o=t[0],A=t[1]):(o=a[0],A=a[1],r.invert&&(o=r.invert(o),A=r.invert(A)),A<o&&(u=o,o=A,A=u))),i&&(e?(l=e[0],c=e[1]):(l=s[0],c=s[1],i.invert&&(l=i.invert(l),c=i.invert(c)),c<l&&(u=l,l=c,c=u))),r&&i?[[o,l],[A,c]]:r?[o,A]:i&&[l,c])},g.clear=function(){return g.empty()||(a=[0,0],s=[0,0],t=e=null),g},g.empty=function(){return!!r&&a[0]==a[1]||!!i&&s[0]==s[1]},o.rebind(g,n,"on")};var Gs={n:"ns-resize",e:"ew-resize",s:"ns-resize",w:"ew-resize",nw:"nwse-resize",ne:"nesw-resize",se:"nwse-resize",sw:"nesw-resize"},Hs=[["n","e","s","w","nw","ne","se","sw"],["e","w"],["n","s"],[]],js=Ge.format=pn.timeFormat,Ys=js.utc,Ps=Ys("%Y-%m-%dT%H:%M:%S.%LZ");function Ks(t){return t.toISOString()}function zs(t,e,n){function r(e){return t(e)}function i(t,n){var r=(t[1]-t[0])/n,i=o.bisect(qs,r);return i==qs.length?[e.year,Sa(t.map(function(t){return t/31536e6}),n)[2]]:i?e[r/qs[i-1]<qs[i]/r?i-1:i]:[Xs,Sa(t,n)[2]]}return r.invert=function(e){return Js(t.invert(e))},r.domain=function(e){return arguments.length?(t.domain(e),r):t.domain().map(Js)},r.nice=function(t,e){var n=r.domain(),o=Ba(n),a=null==t?i(o,10):"number"==typeof t&&i(o,t);function s(n){return!isNaN(n)&&!t.range(n,Js(+n+1),e).length}return a&&(t=a[0],e=a[1]),r.domain(wa(n,e>1?{floor:function(e){for(;s(e=t.floor(e));)e=Js(e-1);return e},ceil:function(e){for(;s(e=t.ceil(e));)e=Js(+e+1);return e}}:t))},r.ticks=function(t,e){var n=Ba(r.domain()),o=null==t?i(n,10):"number"==typeof t?i(n,t):!t.range&&[{range:t},e];return o&&(t=o[0],e=o[1]),t.range(n[0],Js(+n[1]+1),e<1?1:e)},r.tickFormat=function(){return n},r.copy=function(){return zs(t.copy(),e,n)},Ma(r,t)}function Js(t){return new Date(t)}js.iso=Date.prototype.toISOString&&+new Date("2000-01-01T00:00:00.000Z")?Ks:Ps,Ks.parse=function(t){var e=new Date(t);return isNaN(e)?null:e},Ks.toString=Ps.toString,Ge.second=Pe(function(t){return new He(1e3*Math.floor(t/1e3))},function(t,e){t.setTime(t.getTime()+1e3*Math.floor(e))},function(t){return t.getSeconds()}),Ge.seconds=Ge.second.range,Ge.seconds.utc=Ge.second.utc.range,Ge.minute=Pe(function(t){return new He(6e4*Math.floor(t/6e4))},function(t,e){t.setTime(t.getTime()+6e4*Math.floor(e))},function(t){return t.getMinutes()}),Ge.minutes=Ge.minute.range,Ge.minutes.utc=Ge.minute.utc.range,Ge.hour=Pe(function(t){var e=t.getTimezoneOffset()/60;return new He(36e5*(Math.floor(t/36e5-e)+e))},function(t,e){t.setTime(t.getTime()+36e5*Math.floor(e))},function(t){return t.getHours()}),Ge.hours=Ge.hour.range,Ge.hours.utc=Ge.hour.utc.range,Ge.month=Pe(function(t){return(t=Ge.day(t)).setDate(1),t},function(t,e){t.setMonth(t.getMonth()+e)},function(t){return t.getMonth()}),Ge.months=Ge.month.range,Ge.months.utc=Ge.month.utc.range;var qs=[1e3,5e3,15e3,3e4,6e4,3e5,9e5,18e5,36e5,108e5,216e5,432e5,864e5,1728e5,6048e5,2592e6,7776e6,31536e6],Vs=[[Ge.second,1],[Ge.second,5],[Ge.second,15],[Ge.second,30],[Ge.minute,1],[Ge.minute,5],[Ge.minute,15],[Ge.minute,30],[Ge.hour,1],[Ge.hour,3],[Ge.hour,6],[Ge.hour,12],[Ge.day,1],[Ge.day,2],[Ge.week,1],[Ge.month,1],[Ge.month,3],[Ge.year,1]],Ws=js.multi([[".%L",function(t){return t.getMilliseconds()}],[":%S",function(t){return t.getSeconds()}],["%I:%M",function(t){return t.getMinutes()}],["%I %p",function(t){return t.getHours()}],["%a %d",function(t){return t.getDay()&&1!=t.getDate()}],["%b %d",function(t){return 1!=t.getDate()}],["%B",function(t){return t.getMonth()}],["%Y",er]]),Xs={range:function(t,e,n){return o.range(Math.ceil(t/n)*n,+e,n).map(Js)},floor:F,ceil:F};Vs.year=Ge.year,Ge.scale=function(){return zs(o.scale.linear(),Vs,Ws)};var Zs=Vs.map(function(t){return[t[0].utc,t[1]]}),$s=Ys.multi([[".%L",function(t){return t.getUTCMilliseconds()}],[":%S",function(t){return t.getUTCSeconds()}],["%I:%M",function(t){return t.getUTCMinutes()}],["%I %p",function(t){return t.getUTCHours()}],["%a %d",function(t){return t.getUTCDay()&&1!=t.getUTCDate()}],["%b %d",function(t){return 1!=t.getUTCDate()}],["%B",function(t){return t.getUTCMonth()}],["%Y",er]]);function tA(t){return JSON.parse(t.responseText)}function eA(t){var e=A.createRange();return e.selectNode(A.body),e.createContextualFragment(t.responseText)}Zs.year=Ge.year.utc,Ge.scale.utc=function(){return zs(o.scale.linear(),Zs,$s)},o.text=Be(function(t){return t.responseText}),o.json=function(t,e){return be(t,"application/json",tA,e)},o.html=function(t,e){return be(t,"text/html",eA,e)},o.xml=Be(function(t){return t.responseXML}),this.d3=o,void 0===(i="function"==typeof(r=o)?r.call(e,n,e,t):r)||(t.exports=i)}()},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(5);e.Tooltip=function(t,e,n){var i={html:void 0,htmlMode:"clear",css:Object.assign({backgroundColor:"white",color:"black",padding:0,fontFamily:"sans-serif",fontSize:"13px",pointerEvents:"none"},e.css),show:null,hide:null,move:null,hideDelay:0,showDelay:0,height:-1,width:400};e=Object.assign(i,e);var o=r(document.createElement("div"));function a(){t.off("mousemove"),t.off("mouseout")}return t.off("mouseenter"),t.off("mouseleave"),t.on({mouseenter:function(i){n.CloseHover(),e.width>-1&&o.width(e.width),e.height>-1&&o.height(e.height);var a="function"==typeof e.html?e.html(t,o):e.html;if(e.html)switch(e.htmlMode){case"clear":o.html(a);break;case"append":o.append(a);break;case"prepend":o.prepend(a)}o.css(r.extend(e.css,{position:"absolute",zIndex:9999999}));var s=r(t)[0].getBoundingClientRect();n.OpenHover(o[0].outerHTML,s.right+1,s.y,e.width,e.height)},mouseleave:function(t){n.CloseHover()},dblclick:function(t){}}),{off:a,disable:a}}},function(t,e){t.exports=__WEBPACK_EXTERNAL_MODULE__64__},function(t,e,n){"use strict";var r=n(159)(!0);t.exports=function(t,e,n){return e+(n?r(t,e).length:1)}},function(t,e,n){"use strict";var r=n(94),i=RegExp.prototype.exec;t.exports=function(t,e){var n=t.exec;if("function"==typeof n){var o=n.call(t,e);if("object"!=typeof o)throw new TypeError("RegExp exec method returned something other than an Object or null");return o}if("RegExp"!==r(t))throw new TypeError("RegExp#exec called on incompatible receiver");return i.call(t,e)}},function(t,e,n){"use strict";n(160);var r=n(23),i=n(22),o=n(17),a=n(46),s=n(7),A=n(68),l=s("species"),c=!o(function(){var t=/./;return t.exec=function(){var t=[];return t.groups={a:"7"},t},"7"!=="".replace(t,"$<a>")}),u=function(){var t=/(?:)/,e=t.exec;t.exec=function(){return e.apply(this,arguments)};var n="ab".split(t);return 2===n.length&&"a"===n[0]&&"b"===n[1]}();t.exports=function(t,e,n){var g=s(t),h=!o(function(){var e={};return e[g]=function(){return 7},7!=""[t](e)}),d=h?!o(function(){var e=!1,n=/a/;return n.exec=function(){return e=!0,null},"split"===t&&(n.constructor={},n.constructor[l]=function(){return n}),n[g](""),!e}):void 0;if(!h||!d||"replace"===t&&!c||"split"===t&&!u){var f=/./[g],p=n(a,g,""[t],function(t,e,n,r,i){return e.exec===A?h&&!i?{done:!0,value:f.call(e,n,r)}:{done:!0,value:t.call(n,e,r)}:{done:!1}}),I=p[0],C=p[1];r(String.prototype,t,I),i(RegExp.prototype,g,2==e?function(t,e){return C.call(t,this,e)}:function(t){return C.call(t,this)})}}},function(t,e,n){"use strict";var r,i,o=n(69),a=RegExp.prototype.exec,s=String.prototype.replace,A=a,l=(r=/a/,i=/b*/g,a.call(r,"a"),a.call(i,"a"),0!==r.lastIndex||0!==i.lastIndex),c=void 0!==/()??/.exec("")[1];(l||c)&&(A=function(t){var e,n,r,i,A=this;return c&&(n=new RegExp("^"+A.source+"$(?!\\s)",o.call(A))),l&&(e=A.lastIndex),r=a.call(A,t),l&&r&&(A.lastIndex=A.global?r.index+r[0].length:e),c&&r&&r.length>1&&s.call(r[0],n,function(){for(i=1;i<arguments.length-2;i++)void 0===arguments[i]&&(r[i]=void 0)}),r}),t.exports=A},function(t,e,n){"use strict";var r=n(11);t.exports=function(){var t=r(this),e="";return t.global&&(e+="g"),t.ignoreCase&&(e+="i"),t.multiline&&(e+="m"),t.unicode&&(e+="u"),t.sticky&&(e+="y"),e}},function(t,e,n){var r=n(15);t.exports=function(t,e){if(!r(t))return t;var n,i;if(e&&"function"==typeof(n=t.toString)&&!r(i=n.call(t)))return i;if("function"==typeof(n=t.valueOf)&&!r(i=n.call(t)))return i;if(!e&&"function"==typeof(n=t.toString)&&!r(i=n.call(t)))return i;throw TypeError("Can't convert object to primitive value")}},function(t,e){t.exports=function(t){if("function"!=typeof t)throw TypeError(t+" is not a function!");return t}},function(t,e,n){var r=n(11),i=n(166),o=n(74),a=n(73)("IE_PROTO"),s=function(){},A=function(){var t,e=n(96)("iframe"),r=o.length;for(e.style.display="none",n(169).appendChild(e),e.src="javascript:",(t=e.contentWindow.document).open(),t.write("<script>document.F=Object<\/script>"),t.close(),A=t.F;r--;)delete A.prototype[o[r]];return A()};t.exports=Object.create||function(t,e){var n;return null!==t?(s.prototype=r(t),n=new s,s.prototype=null,n[a]=t):n=A(),void 0===e?n:i(n,e)}},function(t,e,n){var r=n(49)("keys"),i=n(36);t.exports=function(t){return r[t]||(r[t]=i(t))}},function(t,e){t.exports="constructor,hasOwnProperty,isPrototypeOf,propertyIsEnumerable,toLocaleString,toString,valueOf".split(",")},function(t,e,n){"use strict";var r=n(171),i=n(11),o=n(172),a=n(65),s=n(34),A=n(66),l=n(68),c=n(17),u=Math.min,g=[].push,h=!c(function(){RegExp(4294967295,"y")});n(67)("split",2,function(t,e,n,c){var d;return d="c"=="abbc".split(/(b)*/)[1]||4!="test".split(/(?:)/,-1).length||2!="ab".split(/(?:ab)*/).length||4!=".".split(/(.?)(.?)/).length||".".split(/()()/).length>1||"".split(/.?/).length?function(t,e){var i=String(this);if(void 0===t&&0===e)return[];if(!r(t))return n.call(i,t,e);for(var o,a,s,A=[],c=(t.ignoreCase?"i":"")+(t.multiline?"m":"")+(t.unicode?"u":"")+(t.sticky?"y":""),u=0,h=void 0===e?4294967295:e>>>0,d=new RegExp(t.source,c+"g");(o=l.call(d,i))&&!((a=d.lastIndex)>u&&(A.push(i.slice(u,o.index)),o.length>1&&o.index<i.length&&g.apply(A,o.slice(1)),s=o[0].length,u=a,A.length>=h));)d.lastIndex===o.index&&d.lastIndex++;return u===i.length?!s&&d.test("")||A.push(""):A.push(i.slice(u)),A.length>h?A.slice(0,h):A}:"0".split(void 0,0).length?function(t,e){return void 0===t&&0===e?[]:n.call(this,t,e)}:n,[function(n,r){var i=t(this),o=null==n?void 0:n[e];return void 0!==o?o.call(n,i,r):d.call(String(i),n,r)},function(t,e){var r=c(d,t,this,e,d!==n);if(r.done)return r.value;var l=i(t),g=String(this),f=o(l,RegExp),p=l.unicode,I=(l.ignoreCase?"i":"")+(l.multiline?"m":"")+(l.unicode?"u":"")+(h?"y":"g"),C=new f(h?l:"^(?:"+l.source+")",I),m=void 0===e?4294967295:e>>>0;if(0===m)return[];if(0===g.length)return null===A(C,g)?[g]:[];for(var y=0,E=0,v=[];E<g.length;){C.lastIndex=h?E:0;var B,b=A(C,h?g:g.slice(E));if(null===b||(B=u(s(C.lastIndex+(h?0:E)),g.length))===y)E=a(g,E,p);else{if(v.push(g.slice(y,E)),v.length===m)return v;for(var Q=1;Q<=b.length-1;Q++)if(v.push(b[Q]),v.length===m)return v;E=y=B}}return v.push(g.slice(y)),v}]})},function(t,e,n){"use strict";n(174);var r=n(11),i=n(69),o=n(16),a=/./.toString,s=function(t){n(23)(RegExp.prototype,"toString",t,!0)};n(17)(function(){return"/a/b"!=a.call({source:"a",flags:"b"})})?s(function(){var t=r(this);return"/".concat(t.source,"/","flags"in t?t.flags:!o&&t instanceof RegExp?i.call(t):void 0)}):"toString"!=a.name&&s(function(){return a.call(this)})},function(t,e,n){n(103)("asyncIterator")},function(t,e,n){"use strict";var r=n(13),i=n(27),o=n(16),a=n(37),s=n(23),A=n(79).KEY,l=n(17),c=n(49),u=n(54),g=n(36),h=n(7),d=n(104),f=n(103),p=n(185),I=n(186),C=n(11),m=n(15),y=n(45),E=n(30),v=n(70),B=n(51),b=n(72),Q=n(187),w=n(107),x=n(105),_=n(19),D=n(53),M=w.f,k=_.f,S=Q.f,L=r.Symbol,R=r.JSON,N=R&&R.stringify,F=h("_hidden"),T=h("toPrimitive"),U={}.propertyIsEnumerable,O=c("symbol-registry"),G=c("symbols"),H=c("op-symbols"),j=Object.prototype,Y="function"==typeof L&&!!x.f,P=r.QObject,K=!P||!P.prototype||!P.prototype.findChild,z=o&&l(function(){return 7!=b(k({},"a",{get:function(){return k(this,"a",{value:7}).a}})).a})?function(t,e,n){var r=M(j,e);r&&delete j[e],k(t,e,n),r&&t!==j&&k(j,e,r)}:k,J=function(t){var e=G[t]=b(L.prototype);return e._k=t,e},q=Y&&"symbol"==typeof L.iterator?function(t){return"symbol"==typeof t}:function(t){return t instanceof L},V=function(t,e,n){return t===j&&V(H,e,n),C(t),e=v(e,!0),C(n),i(G,e)?(n.enumerable?(i(t,F)&&t[F][e]&&(t[F][e]=!1),n=b(n,{enumerable:B(0,!1)})):(i(t,F)||k(t,F,B(1,{})),t[F][e]=!0),z(t,e,n)):k(t,e,n)},W=function(t,e){C(t);for(var n,r=p(e=E(e)),i=0,o=r.length;o>i;)V(t,n=r[i++],e[n]);return t},X=function(t){var e=U.call(this,t=v(t,!0));return!(this===j&&i(G,t)&&!i(H,t))&&(!(e||!i(this,t)||!i(G,t)||i(this,F)&&this[F][t])||e)},Z=function(t,e){if(t=E(t),e=v(e,!0),t!==j||!i(G,e)||i(H,e)){var n=M(t,e);return!n||!i(G,e)||i(t,F)&&t[F][e]||(n.enumerable=!0),n}},$=function(t){for(var e,n=S(E(t)),r=[],o=0;n.length>o;)i(G,e=n[o++])||e==F||e==A||r.push(e);return r},tt=function(t){for(var e,n=t===j,r=S(n?H:E(t)),o=[],a=0;r.length>a;)!i(G,e=r[a++])||n&&!i(j,e)||o.push(G[e]);return o};Y||(s((L=function(){if(this instanceof L)throw TypeError("Symbol is not a constructor!");var t=g(arguments.length>0?arguments[0]:void 0),e=function(n){this===j&&e.call(H,n),i(this,F)&&i(this[F],t)&&(this[F][t]=!1),z(this,t,B(1,n))};return o&&K&&z(j,t,{configurable:!0,set:e}),J(t)}).prototype,"toString",function(){return this._k}),w.f=Z,_.f=V,n(106).f=Q.f=$,n(80).f=X,x.f=tt,o&&!n(50)&&s(j,"propertyIsEnumerable",X,!0),d.f=function(t){return J(h(t))}),a(a.G+a.W+a.F*!Y,{Symbol:L});for(var et="hasInstance,isConcatSpreadable,iterator,match,replace,search,species,split,toPrimitive,toStringTag,unscopables".split(","),nt=0;et.length>nt;)h(et[nt++]);for(var rt=D(h.store),it=0;rt.length>it;)f(rt[it++]);a(a.S+a.F*!Y,"Symbol",{for:function(t){return i(O,t+="")?O[t]:O[t]=L(t)},keyFor:function(t){if(!q(t))throw TypeError(t+" is not a symbol!");for(var e in O)if(O[e]===t)return e},useSetter:function(){K=!0},useSimple:function(){K=!1}}),a(a.S+a.F*!Y,"Object",{create:function(t,e){return void 0===e?b(t):W(b(t),e)},defineProperty:V,defineProperties:W,getOwnPropertyDescriptor:Z,getOwnPropertyNames:$,getOwnPropertySymbols:tt});var ot=l(function(){x.f(1)});a(a.S+a.F*ot,"Object",{getOwnPropertySymbols:function(t){return x.f(y(t))}}),R&&a(a.S+a.F*(!Y||l(function(){var t=L();return"[null]"!=N([t])||"{}"!=N({a:t})||"{}"!=N(Object(t))})),"JSON",{stringify:function(t){for(var e,n,r=[t],i=1;arguments.length>i;)r.push(arguments[i++]);if(n=e=r[1],(m(e)||void 0!==t)&&!q(t))return I(e)||(e=function(t,e){if("function"==typeof n&&(e=n.call(this,t,e)),!q(e))return e}),r[1]=e,N.apply(R,r)}}),L.prototype[T]||n(22)(L.prototype,T,L.prototype.valueOf),u(L,"Symbol"),u(Math,"Math",!0),u(r.JSON,"JSON",!0)},function(t,e,n){var r=n(36)("meta"),i=n(15),o=n(27),a=n(19).f,s=0,A=Object.isExtensible||function(){return!0},l=!n(17)(function(){return A(Object.preventExtensions({}))}),c=function(t){a(t,r,{value:{i:"O"+ ++s,w:{}}})},u=t.exports={KEY:r,NEED:!1,fastKey:function(t,e){if(!i(t))return"symbol"==typeof t?t:("string"==typeof t?"S":"P")+t;if(!o(t,r)){if(!A(t))return"F";if(!e)return"E";c(t)}return t[r].i},getWeak:function(t,e){if(!o(t,r)){if(!A(t))return!0;if(!e)return!1;c(t)}return t[r].w},onFreeze:function(t){return l&&u.NEED&&A(t)&&!o(t,r)&&c(t),t}}},function(t,e){e.f={}.propertyIsEnumerable},function(t,e,n){var r=n(18).Symbol;t.exports=r},function(t,e){var n=Array.isArray;t.exports=n},function(t,e,n){var r=n(31)(n(18),"Map");t.exports=r},function(t,e,n){"use strict";n.r(e);var r=n(8),i=function(t){return function(){return t}},o=Math.abs,a=Math.atan2,s=Math.cos,A=Math.max,l=Math.min,c=Math.sin,u=Math.sqrt,g=1e-12,h=Math.PI,d=h/2,f=2*h;function p(t){return t>=1?d:t<=-1?-d:Math.asin(t)}function I(t){return t.innerRadius}function C(t){return t.outerRadius}function m(t){return t.startAngle}function y(t){return t.endAngle}function E(t){return t&&t.padAngle}function v(t,e,n,r,i,o,a){var s=t-n,l=e-r,c=(a?o:-o)/u(s*s+l*l),g=c*l,h=-c*s,d=t+g,f=e+h,p=n+g,I=r+h,C=(d+p)/2,m=(f+I)/2,y=p-d,E=I-f,v=y*y+E*E,B=i-o,b=d*I-p*f,Q=(E<0?-1:1)*u(A(0,B*B*v-b*b)),w=(b*E-y*Q)/v,x=(-b*y-E*Q)/v,_=(b*E+y*Q)/v,D=(-b*y+E*Q)/v,M=w-C,k=x-m,S=_-C,L=D-m;return M*M+k*k>S*S+L*L&&(w=_,x=D),{cx:w,cy:x,x01:-g,y01:-h,x11:w*(i/B-1),y11:x*(i/B-1)}}var B=function(){var t=I,e=C,n=i(0),A=null,B=m,b=y,Q=E,w=null;function x(){var i,I,C,m=+t.apply(this,arguments),y=+e.apply(this,arguments),E=B.apply(this,arguments)-d,x=b.apply(this,arguments)-d,_=o(x-E),D=x>E;if(w||(w=i=Object(r.path)()),y<m&&(I=y,y=m,m=I),y>g)if(_>f-g)w.moveTo(y*s(E),y*c(E)),w.arc(0,0,y,E,x,!D),m>g&&(w.moveTo(m*s(x),m*c(x)),w.arc(0,0,m,x,E,D));else{var M,k,S=E,L=x,R=E,N=x,F=_,T=_,U=Q.apply(this,arguments)/2,O=U>g&&(A?+A.apply(this,arguments):u(m*m+y*y)),G=l(o(y-m)/2,+n.apply(this,arguments)),H=G,j=G;if(O>g){var Y=p(O/m*c(U)),P=p(O/y*c(U));(F-=2*Y)>g?(R+=Y*=D?1:-1,N-=Y):(F=0,R=N=(E+x)/2),(T-=2*P)>g?(S+=P*=D?1:-1,L-=P):(T=0,S=L=(E+x)/2)}var K=y*s(S),z=y*c(S),J=m*s(N),q=m*c(N);if(G>g){var V,W=y*s(L),X=y*c(L),Z=m*s(R),$=m*c(R);if(_<h&&(V=function(t,e,n,r,i,o,a,s){var A=Z-K,l=$-z,c=J-W,u=q-X,h=u*A-c*l;if(!(h*h<g))return[K+(h=(c*(z-X)-u*(K-W))/h)*A,z+h*l]}())){var tt=K-V[0],et=z-V[1],nt=W-V[0],rt=X-V[1],it=1/c(((C=(tt*nt+et*rt)/(u(tt*tt+et*et)*u(nt*nt+rt*rt)))>1?0:C<-1?h:Math.acos(C))/2),ot=u(V[0]*V[0]+V[1]*V[1]);H=l(G,(m-ot)/(it-1)),j=l(G,(y-ot)/(it+1))}}T>g?j>g?(M=v(Z,$,K,z,y,j,D),k=v(W,X,J,q,y,j,D),w.moveTo(M.cx+M.x01,M.cy+M.y01),j<G?w.arc(M.cx,M.cy,j,a(M.y01,M.x01),a(k.y01,k.x01),!D):(w.arc(M.cx,M.cy,j,a(M.y01,M.x01),a(M.y11,M.x11),!D),w.arc(0,0,y,a(M.cy+M.y11,M.cx+M.x11),a(k.cy+k.y11,k.cx+k.x11),!D),w.arc(k.cx,k.cy,j,a(k.y11,k.x11),a(k.y01,k.x01),!D))):(w.moveTo(K,z),w.arc(0,0,y,S,L,!D)):w.moveTo(K,z),m>g&&F>g?H>g?(M=v(J,q,W,X,m,-H,D),k=v(K,z,Z,$,m,-H,D),w.lineTo(M.cx+M.x01,M.cy+M.y01),H<G?w.arc(M.cx,M.cy,H,a(M.y01,M.x01),a(k.y01,k.x01),!D):(w.arc(M.cx,M.cy,H,a(M.y01,M.x01),a(M.y11,M.x11),!D),w.arc(0,0,m,a(M.cy+M.y11,M.cx+M.x11),a(k.cy+k.y11,k.cx+k.x11),D),w.arc(k.cx,k.cy,H,a(k.y11,k.x11),a(k.y01,k.x01),!D))):w.arc(0,0,m,N,R,D):w.lineTo(J,q)}else w.moveTo(0,0);if(w.closePath(),i)return w=null,i+""||null}return x.centroid=function(){var n=(+t.apply(this,arguments)+ +e.apply(this,arguments))/2,r=(+B.apply(this,arguments)+ +b.apply(this,arguments))/2-h/2;return[s(r)*n,c(r)*n]},x.innerRadius=function(e){return arguments.length?(t="function"==typeof e?e:i(+e),x):t},x.outerRadius=function(t){return arguments.length?(e="function"==typeof t?t:i(+t),x):e},x.cornerRadius=function(t){return arguments.length?(n="function"==typeof t?t:i(+t),x):n},x.padRadius=function(t){return arguments.length?(A=null==t?null:"function"==typeof t?t:i(+t),x):A},x.startAngle=function(t){return arguments.length?(B="function"==typeof t?t:i(+t),x):B},x.endAngle=function(t){return arguments.length?(b="function"==typeof t?t:i(+t),x):b},x.padAngle=function(t){return arguments.length?(Q="function"==typeof t?t:i(+t),x):Q},x.context=function(t){return arguments.length?(w=null==t?null:t,x):w},x};function b(t){this._context=t}b.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._point=0},lineEnd:function(){(this._line||0!==this._line&&1===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1,this._line?this._context.lineTo(t,e):this._context.moveTo(t,e);break;case 1:this._point=2;default:this._context.lineTo(t,e)}}};var Q=function(t){return new b(t)};function w(t){return t[0]}function x(t){return t[1]}var _=function(){var t=w,e=x,n=i(!0),o=null,a=Q,s=null;function A(i){var A,l,c,u=i.length,g=!1;for(null==o&&(s=a(c=Object(r.path)())),A=0;A<=u;++A)!(A<u&&n(l=i[A],A,i))===g&&((g=!g)?s.lineStart():s.lineEnd()),g&&s.point(+t(l,A,i),+e(l,A,i));if(c)return s=null,c+""||null}return A.x=function(e){return arguments.length?(t="function"==typeof e?e:i(+e),A):t},A.y=function(t){return arguments.length?(e="function"==typeof t?t:i(+t),A):e},A.defined=function(t){return arguments.length?(n="function"==typeof t?t:i(!!t),A):n},A.curve=function(t){return arguments.length?(a=t,null!=o&&(s=a(o)),A):a},A.context=function(t){return arguments.length?(null==t?o=s=null:s=a(o=t),A):o},A},D=function(){var t=w,e=null,n=i(0),o=x,a=i(!0),s=null,A=Q,l=null;function c(i){var c,u,g,h,d,f=i.length,p=!1,I=new Array(f),C=new Array(f);for(null==s&&(l=A(d=Object(r.path)())),c=0;c<=f;++c){if(!(c<f&&a(h=i[c],c,i))===p)if(p=!p)u=c,l.areaStart(),l.lineStart();else{for(l.lineEnd(),l.lineStart(),g=c-1;g>=u;--g)l.point(I[g],C[g]);l.lineEnd(),l.areaEnd()}p&&(I[c]=+t(h,c,i),C[c]=+n(h,c,i),l.point(e?+e(h,c,i):I[c],o?+o(h,c,i):C[c]))}if(d)return l=null,d+""||null}function u(){return _().defined(a).curve(A).context(s)}return c.x=function(n){return arguments.length?(t="function"==typeof n?n:i(+n),e=null,c):t},c.x0=function(e){return arguments.length?(t="function"==typeof e?e:i(+e),c):t},c.x1=function(t){return arguments.length?(e=null==t?null:"function"==typeof t?t:i(+t),c):e},c.y=function(t){return arguments.length?(n="function"==typeof t?t:i(+t),o=null,c):n},c.y0=function(t){return arguments.length?(n="function"==typeof t?t:i(+t),c):n},c.y1=function(t){return arguments.length?(o=null==t?null:"function"==typeof t?t:i(+t),c):o},c.lineX0=c.lineY0=function(){return u().x(t).y(n)},c.lineY1=function(){return u().x(t).y(o)},c.lineX1=function(){return u().x(e).y(n)},c.defined=function(t){return arguments.length?(a="function"==typeof t?t:i(!!t),c):a},c.curve=function(t){return arguments.length?(A=t,null!=s&&(l=A(s)),c):A},c.context=function(t){return arguments.length?(null==t?s=l=null:l=A(s=t),c):s},c},M=function(t,e){return e<t?-1:e>t?1:e>=t?0:NaN},k=function(t){return t},S=function(){var t=k,e=M,n=null,r=i(0),o=i(f),a=i(0);function s(i){var s,A,l,c,u,g=i.length,h=0,d=new Array(g),p=new Array(g),I=+r.apply(this,arguments),C=Math.min(f,Math.max(-f,o.apply(this,arguments)-I)),m=Math.min(Math.abs(C)/g,a.apply(this,arguments)),y=m*(C<0?-1:1);for(s=0;s<g;++s)(u=p[d[s]=s]=+t(i[s],s,i))>0&&(h+=u);for(null!=e?d.sort(function(t,n){return e(p[t],p[n])}):null!=n&&d.sort(function(t,e){return n(i[t],i[e])}),s=0,l=h?(C-g*y)/h:0;s<g;++s,I=c)A=d[s],c=I+((u=p[A])>0?u*l:0)+y,p[A]={data:i[A],index:s,value:u,startAngle:I,endAngle:c,padAngle:m};return p}return s.value=function(e){return arguments.length?(t="function"==typeof e?e:i(+e),s):t},s.sortValues=function(t){return arguments.length?(e=t,n=null,s):e},s.sort=function(t){return arguments.length?(n=t,e=null,s):n},s.startAngle=function(t){return arguments.length?(r="function"==typeof t?t:i(+t),s):r},s.endAngle=function(t){return arguments.length?(o="function"==typeof t?t:i(+t),s):o},s.padAngle=function(t){return arguments.length?(a="function"==typeof t?t:i(+t),s):a},s},L=N(Q);function R(t){this._curve=t}function N(t){function e(e){return new R(t(e))}return e._curve=t,e}function F(t){var e=t.curve;return t.angle=t.x,delete t.x,t.radius=t.y,delete t.y,t.curve=function(t){return arguments.length?e(N(t)):e()._curve},t}R.prototype={areaStart:function(){this._curve.areaStart()},areaEnd:function(){this._curve.areaEnd()},lineStart:function(){this._curve.lineStart()},lineEnd:function(){this._curve.lineEnd()},point:function(t,e){this._curve.point(e*Math.sin(t),e*-Math.cos(t))}};var T=function(){return F(_().curve(L))},U=function(){var t=D().curve(L),e=t.curve,n=t.lineX0,r=t.lineX1,i=t.lineY0,o=t.lineY1;return t.angle=t.x,delete t.x,t.startAngle=t.x0,delete t.x0,t.endAngle=t.x1,delete t.x1,t.radius=t.y,delete t.y,t.innerRadius=t.y0,delete t.y0,t.outerRadius=t.y1,delete t.y1,t.lineStartAngle=function(){return F(n())},delete t.lineX0,t.lineEndAngle=function(){return F(r())},delete t.lineX1,t.lineInnerRadius=function(){return F(i())},delete t.lineY0,t.lineOuterRadius=function(){return F(o())},delete t.lineY1,t.curve=function(t){return arguments.length?e(N(t)):e()._curve},t},O=function(t,e){return[(e=+e)*Math.cos(t-=Math.PI/2),e*Math.sin(t)]},G=Array.prototype.slice;function H(t){return t.source}function j(t){return t.target}function Y(t){var e=H,n=j,o=w,a=x,s=null;function A(){var i,A=G.call(arguments),l=e.apply(this,A),c=n.apply(this,A);if(s||(s=i=Object(r.path)()),t(s,+o.apply(this,(A[0]=l,A)),+a.apply(this,A),+o.apply(this,(A[0]=c,A)),+a.apply(this,A)),i)return s=null,i+""||null}return A.source=function(t){return arguments.length?(e=t,A):e},A.target=function(t){return arguments.length?(n=t,A):n},A.x=function(t){return arguments.length?(o="function"==typeof t?t:i(+t),A):o},A.y=function(t){return arguments.length?(a="function"==typeof t?t:i(+t),A):a},A.context=function(t){return arguments.length?(s=null==t?null:t,A):s},A}function P(t,e,n,r,i){t.moveTo(e,n),t.bezierCurveTo(e=(e+r)/2,n,e,i,r,i)}function K(t,e,n,r,i){t.moveTo(e,n),t.bezierCurveTo(e,n=(n+i)/2,r,n,r,i)}function z(t,e,n,r,i){var o=O(e,n),a=O(e,n=(n+i)/2),s=O(r,n),A=O(r,i);t.moveTo(o[0],o[1]),t.bezierCurveTo(a[0],a[1],s[0],s[1],A[0],A[1])}function J(){return Y(P)}function q(){return Y(K)}function V(){var t=Y(z);return t.angle=t.x,delete t.x,t.radius=t.y,delete t.y,t}var W={draw:function(t,e){var n=Math.sqrt(e/h);t.moveTo(n,0),t.arc(0,0,n,0,f)}},X={draw:function(t,e){var n=Math.sqrt(e/5)/2;t.moveTo(-3*n,-n),t.lineTo(-n,-n),t.lineTo(-n,-3*n),t.lineTo(n,-3*n),t.lineTo(n,-n),t.lineTo(3*n,-n),t.lineTo(3*n,n),t.lineTo(n,n),t.lineTo(n,3*n),t.lineTo(-n,3*n),t.lineTo(-n,n),t.lineTo(-3*n,n),t.closePath()}},Z=Math.sqrt(1/3),$=2*Z,tt={draw:function(t,e){var n=Math.sqrt(e/$),r=n*Z;t.moveTo(0,-n),t.lineTo(r,0),t.lineTo(0,n),t.lineTo(-r,0),t.closePath()}},et=Math.sin(h/10)/Math.sin(7*h/10),nt=Math.sin(f/10)*et,rt=-Math.cos(f/10)*et,it={draw:function(t,e){var n=Math.sqrt(.8908130915292852*e),r=nt*n,i=rt*n;t.moveTo(0,-n),t.lineTo(r,i);for(var o=1;o<5;++o){var a=f*o/5,s=Math.cos(a),A=Math.sin(a);t.lineTo(A*n,-s*n),t.lineTo(s*r-A*i,A*r+s*i)}t.closePath()}},ot={draw:function(t,e){var n=Math.sqrt(e),r=-n/2;t.rect(r,r,n,n)}},at=Math.sqrt(3),st={draw:function(t,e){var n=-Math.sqrt(e/(3*at));t.moveTo(0,2*n),t.lineTo(-at*n,-n),t.lineTo(at*n,-n),t.closePath()}},At=Math.sqrt(3)/2,lt=1/Math.sqrt(12),ct=3*(lt/2+1),ut={draw:function(t,e){var n=Math.sqrt(e/ct),r=n/2,i=n*lt,o=r,a=n*lt+n,s=-o,A=a;t.moveTo(r,i),t.lineTo(o,a),t.lineTo(s,A),t.lineTo(-.5*r-At*i,At*r+-.5*i),t.lineTo(-.5*o-At*a,At*o+-.5*a),t.lineTo(-.5*s-At*A,At*s+-.5*A),t.lineTo(-.5*r+At*i,-.5*i-At*r),t.lineTo(-.5*o+At*a,-.5*a-At*o),t.lineTo(-.5*s+At*A,-.5*A-At*s),t.closePath()}},gt=[W,X,tt,ot,it,st,ut],ht=function(){var t=i(W),e=i(64),n=null;function o(){var i;if(n||(n=i=Object(r.path)()),t.apply(this,arguments).draw(n,+e.apply(this,arguments)),i)return n=null,i+""||null}return o.type=function(e){return arguments.length?(t="function"==typeof e?e:i(e),o):t},o.size=function(t){return arguments.length?(e="function"==typeof t?t:i(+t),o):e},o.context=function(t){return arguments.length?(n=null==t?null:t,o):n},o},dt=function(){};function ft(t,e,n){t._context.bezierCurveTo((2*t._x0+t._x1)/3,(2*t._y0+t._y1)/3,(t._x0+2*t._x1)/3,(t._y0+2*t._y1)/3,(t._x0+4*t._x1+e)/6,(t._y0+4*t._y1+n)/6)}function pt(t){this._context=t}pt.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=NaN,this._point=0},lineEnd:function(){switch(this._point){case 3:ft(this,this._x1,this._y1);case 2:this._context.lineTo(this._x1,this._y1)}(this._line||0!==this._line&&1===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1,this._line?this._context.lineTo(t,e):this._context.moveTo(t,e);break;case 1:this._point=2;break;case 2:this._point=3,this._context.lineTo((5*this._x0+this._x1)/6,(5*this._y0+this._y1)/6);default:ft(this,t,e)}this._x0=this._x1,this._x1=t,this._y0=this._y1,this._y1=e}};var It=function(t){return new pt(t)};function Ct(t){this._context=t}Ct.prototype={areaStart:dt,areaEnd:dt,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._y0=this._y1=this._y2=this._y3=this._y4=NaN,this._point=0},lineEnd:function(){switch(this._point){case 1:this._context.moveTo(this._x2,this._y2),this._context.closePath();break;case 2:this._context.moveTo((this._x2+2*this._x3)/3,(this._y2+2*this._y3)/3),this._context.lineTo((this._x3+2*this._x2)/3,(this._y3+2*this._y2)/3),this._context.closePath();break;case 3:this.point(this._x2,this._y2),this.point(this._x3,this._y3),this.point(this._x4,this._y4)}},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1,this._x2=t,this._y2=e;break;case 1:this._point=2,this._x3=t,this._y3=e;break;case 2:this._point=3,this._x4=t,this._y4=e,this._context.moveTo((this._x0+4*this._x1+t)/6,(this._y0+4*this._y1+e)/6);break;default:ft(this,t,e)}this._x0=this._x1,this._x1=t,this._y0=this._y1,this._y1=e}};var mt=function(t){return new Ct(t)};function yt(t){this._context=t}yt.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=NaN,this._point=0},lineEnd:function(){(this._line||0!==this._line&&3===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3;var n=(this._x0+4*this._x1+t)/6,r=(this._y0+4*this._y1+e)/6;this._line?this._context.lineTo(n,r):this._context.moveTo(n,r);break;case 3:this._point=4;default:ft(this,t,e)}this._x0=this._x1,this._x1=t,this._y0=this._y1,this._y1=e}};var Et=function(t){return new yt(t)};function vt(t,e){this._basis=new pt(t),this._beta=e}vt.prototype={lineStart:function(){this._x=[],this._y=[],this._basis.lineStart()},lineEnd:function(){var t=this._x,e=this._y,n=t.length-1;if(n>0)for(var r,i=t[0],o=e[0],a=t[n]-i,s=e[n]-o,A=-1;++A<=n;)r=A/n,this._basis.point(this._beta*t[A]+(1-this._beta)*(i+r*a),this._beta*e[A]+(1-this._beta)*(o+r*s));this._x=this._y=null,this._basis.lineEnd()},point:function(t,e){this._x.push(+t),this._y.push(+e)}};var Bt=function t(e){function n(t){return 1===e?new pt(t):new vt(t,e)}return n.beta=function(e){return t(+e)},n}(.85);function bt(t,e,n){t._context.bezierCurveTo(t._x1+t._k*(t._x2-t._x0),t._y1+t._k*(t._y2-t._y0),t._x2+t._k*(t._x1-e),t._y2+t._k*(t._y1-n),t._x2,t._y2)}function Qt(t,e){this._context=t,this._k=(1-e)/6}Qt.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x2,this._y2);break;case 3:bt(this,this._x1,this._y1)}(this._line||0!==this._line&&1===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1,this._line?this._context.lineTo(t,e):this._context.moveTo(t,e);break;case 1:this._point=2,this._x1=t,this._y1=e;break;case 2:this._point=3;default:bt(this,t,e)}this._x0=this._x1,this._x1=this._x2,this._x2=t,this._y0=this._y1,this._y1=this._y2,this._y2=e}};var wt=function t(e){function n(t){return new Qt(t,e)}return n.tension=function(e){return t(+e)},n}(0);function xt(t,e){this._context=t,this._k=(1-e)/6}xt.prototype={areaStart:dt,areaEnd:dt,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._x5=this._y0=this._y1=this._y2=this._y3=this._y4=this._y5=NaN,this._point=0},lineEnd:function(){switch(this._point){case 1:this._context.moveTo(this._x3,this._y3),this._context.closePath();break;case 2:this._context.lineTo(this._x3,this._y3),this._context.closePath();break;case 3:this.point(this._x3,this._y3),this.point(this._x4,this._y4),this.point(this._x5,this._y5)}},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1,this._x3=t,this._y3=e;break;case 1:this._point=2,this._context.moveTo(this._x4=t,this._y4=e);break;case 2:this._point=3,this._x5=t,this._y5=e;break;default:bt(this,t,e)}this._x0=this._x1,this._x1=this._x2,this._x2=t,this._y0=this._y1,this._y1=this._y2,this._y2=e}};var _t=function t(e){function n(t){return new xt(t,e)}return n.tension=function(e){return t(+e)},n}(0);function Dt(t,e){this._context=t,this._k=(1-e)/6}Dt.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._point=0},lineEnd:function(){(this._line||0!==this._line&&3===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3,this._line?this._context.lineTo(this._x2,this._y2):this._context.moveTo(this._x2,this._y2);break;case 3:this._point=4;default:bt(this,t,e)}this._x0=this._x1,this._x1=this._x2,this._x2=t,this._y0=this._y1,this._y1=this._y2,this._y2=e}};var Mt=function t(e){function n(t){return new Dt(t,e)}return n.tension=function(e){return t(+e)},n}(0);function kt(t,e,n){var r=t._x1,i=t._y1,o=t._x2,a=t._y2;if(t._l01_a>g){var s=2*t._l01_2a+3*t._l01_a*t._l12_a+t._l12_2a,A=3*t._l01_a*(t._l01_a+t._l12_a);r=(r*s-t._x0*t._l12_2a+t._x2*t._l01_2a)/A,i=(i*s-t._y0*t._l12_2a+t._y2*t._l01_2a)/A}if(t._l23_a>g){var l=2*t._l23_2a+3*t._l23_a*t._l12_a+t._l12_2a,c=3*t._l23_a*(t._l23_a+t._l12_a);o=(o*l+t._x1*t._l23_2a-e*t._l12_2a)/c,a=(a*l+t._y1*t._l23_2a-n*t._l12_2a)/c}t._context.bezierCurveTo(r,i,o,a,t._x2,t._y2)}function St(t,e){this._context=t,this._alpha=e}St.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._l01_a=this._l12_a=this._l23_a=this._l01_2a=this._l12_2a=this._l23_2a=this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x2,this._y2);break;case 3:this.point(this._x2,this._y2)}(this._line||0!==this._line&&1===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){if(t=+t,e=+e,this._point){var n=this._x2-t,r=this._y2-e;this._l23_a=Math.sqrt(this._l23_2a=Math.pow(n*n+r*r,this._alpha))}switch(this._point){case 0:this._point=1,this._line?this._context.lineTo(t,e):this._context.moveTo(t,e);break;case 1:this._point=2;break;case 2:this._point=3;default:kt(this,t,e)}this._l01_a=this._l12_a,this._l12_a=this._l23_a,this._l01_2a=this._l12_2a,this._l12_2a=this._l23_2a,this._x0=this._x1,this._x1=this._x2,this._x2=t,this._y0=this._y1,this._y1=this._y2,this._y2=e}};var Lt=function t(e){function n(t){return e?new St(t,e):new Qt(t,0)}return n.alpha=function(e){return t(+e)},n}(.5);function Rt(t,e){this._context=t,this._alpha=e}Rt.prototype={areaStart:dt,areaEnd:dt,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._x5=this._y0=this._y1=this._y2=this._y3=this._y4=this._y5=NaN,this._l01_a=this._l12_a=this._l23_a=this._l01_2a=this._l12_2a=this._l23_2a=this._point=0},lineEnd:function(){switch(this._point){case 1:this._context.moveTo(this._x3,this._y3),this._context.closePath();break;case 2:this._context.lineTo(this._x3,this._y3),this._context.closePath();break;case 3:this.point(this._x3,this._y3),this.point(this._x4,this._y4),this.point(this._x5,this._y5)}},point:function(t,e){if(t=+t,e=+e,this._point){var n=this._x2-t,r=this._y2-e;this._l23_a=Math.sqrt(this._l23_2a=Math.pow(n*n+r*r,this._alpha))}switch(this._point){case 0:this._point=1,this._x3=t,this._y3=e;break;case 1:this._point=2,this._context.moveTo(this._x4=t,this._y4=e);break;case 2:this._point=3,this._x5=t,this._y5=e;break;default:kt(this,t,e)}this._l01_a=this._l12_a,this._l12_a=this._l23_a,this._l01_2a=this._l12_2a,this._l12_2a=this._l23_2a,this._x0=this._x1,this._x1=this._x2,this._x2=t,this._y0=this._y1,this._y1=this._y2,this._y2=e}};var Nt=function t(e){function n(t){return e?new Rt(t,e):new xt(t,0)}return n.alpha=function(e){return t(+e)},n}(.5);function Ft(t,e){this._context=t,this._alpha=e}Ft.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._l01_a=this._l12_a=this._l23_a=this._l01_2a=this._l12_2a=this._l23_2a=this._point=0},lineEnd:function(){(this._line||0!==this._line&&3===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){if(t=+t,e=+e,this._point){var n=this._x2-t,r=this._y2-e;this._l23_a=Math.sqrt(this._l23_2a=Math.pow(n*n+r*r,this._alpha))}switch(this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3,this._line?this._context.lineTo(this._x2,this._y2):this._context.moveTo(this._x2,this._y2);break;case 3:this._point=4;default:kt(this,t,e)}this._l01_a=this._l12_a,this._l12_a=this._l23_a,this._l01_2a=this._l12_2a,this._l12_2a=this._l23_2a,this._x0=this._x1,this._x1=this._x2,this._x2=t,this._y0=this._y1,this._y1=this._y2,this._y2=e}};var Tt=function t(e){function n(t){return e?new Ft(t,e):new Dt(t,0)}return n.alpha=function(e){return t(+e)},n}(.5);function Ut(t){this._context=t}Ut.prototype={areaStart:dt,areaEnd:dt,lineStart:function(){this._point=0},lineEnd:function(){this._point&&this._context.closePath()},point:function(t,e){t=+t,e=+e,this._point?this._context.lineTo(t,e):(this._point=1,this._context.moveTo(t,e))}};var Ot=function(t){return new Ut(t)};function Gt(t){return t<0?-1:1}function Ht(t,e,n){var r=t._x1-t._x0,i=e-t._x1,o=(t._y1-t._y0)/(r||i<0&&-0),a=(n-t._y1)/(i||r<0&&-0),s=(o*i+a*r)/(r+i);return(Gt(o)+Gt(a))*Math.min(Math.abs(o),Math.abs(a),.5*Math.abs(s))||0}function jt(t,e){var n=t._x1-t._x0;return n?(3*(t._y1-t._y0)/n-e)/2:e}function Yt(t,e,n){var r=t._x0,i=t._y0,o=t._x1,a=t._y1,s=(o-r)/3;t._context.bezierCurveTo(r+s,i+s*e,o-s,a-s*n,o,a)}function Pt(t){this._context=t}function Kt(t){this._context=new zt(t)}function zt(t){this._context=t}function Jt(t){return new Pt(t)}function qt(t){return new Kt(t)}function Vt(t){this._context=t}function Wt(t){var e,n,r=t.length-1,i=new Array(r),o=new Array(r),a=new Array(r);for(i[0]=0,o[0]=2,a[0]=t[0]+2*t[1],e=1;e<r-1;++e)i[e]=1,o[e]=4,a[e]=4*t[e]+2*t[e+1];for(i[r-1]=2,o[r-1]=7,a[r-1]=8*t[r-1]+t[r],e=1;e<r;++e)n=i[e]/o[e-1],o[e]-=n,a[e]-=n*a[e-1];for(i[r-1]=a[r-1]/o[r-1],e=r-2;e>=0;--e)i[e]=(a[e]-i[e+1])/o[e];for(o[r-1]=(t[r]+i[r-1])/2,e=0;e<r-1;++e)o[e]=2*t[e+1]-i[e+1];return[i,o]}Pt.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=this._t0=NaN,this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x1,this._y1);break;case 3:Yt(this,this._t0,jt(this,this._t0))}(this._line||0!==this._line&&1===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){var n=NaN;if(e=+e,(t=+t)!==this._x1||e!==this._y1){switch(this._point){case 0:this._point=1,this._line?this._context.lineTo(t,e):this._context.moveTo(t,e);break;case 1:this._point=2;break;case 2:this._point=3,Yt(this,jt(this,n=Ht(this,t,e)),n);break;default:Yt(this,this._t0,n=Ht(this,t,e))}this._x0=this._x1,this._x1=t,this._y0=this._y1,this._y1=e,this._t0=n}}},(Kt.prototype=Object.create(Pt.prototype)).point=function(t,e){Pt.prototype.point.call(this,e,t)},zt.prototype={moveTo:function(t,e){this._context.moveTo(e,t)},closePath:function(){this._context.closePath()},lineTo:function(t,e){this._context.lineTo(e,t)},bezierCurveTo:function(t,e,n,r,i,o){this._context.bezierCurveTo(e,t,r,n,o,i)}},Vt.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x=[],this._y=[]},lineEnd:function(){var t=this._x,e=this._y,n=t.length;if(n)if(this._line?this._context.lineTo(t[0],e[0]):this._context.moveTo(t[0],e[0]),2===n)this._context.lineTo(t[1],e[1]);else for(var r=Wt(t),i=Wt(e),o=0,a=1;a<n;++o,++a)this._context.bezierCurveTo(r[0][o],i[0][o],r[1][o],i[1][o],t[a],e[a]);(this._line||0!==this._line&&1===n)&&this._context.closePath(),this._line=1-this._line,this._x=this._y=null},point:function(t,e){this._x.push(+t),this._y.push(+e)}};var Xt=function(t){return new Vt(t)};function Zt(t,e){this._context=t,this._t=e}Zt.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x=this._y=NaN,this._point=0},lineEnd:function(){0<this._t&&this._t<1&&2===this._point&&this._context.lineTo(this._x,this._y),(this._line||0!==this._line&&1===this._point)&&this._context.closePath(),this._line>=0&&(this._t=1-this._t,this._line=1-this._line)},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1,this._line?this._context.lineTo(t,e):this._context.moveTo(t,e);break;case 1:this._point=2;default:if(this._t<=0)this._context.lineTo(this._x,e),this._context.lineTo(t,e);else{var n=this._x*(1-this._t)+t*this._t;this._context.lineTo(n,this._y),this._context.lineTo(n,e)}}this._x=t,this._y=e}};var $t=function(t){return new Zt(t,.5)};function te(t){return new Zt(t,0)}function ee(t){return new Zt(t,1)}var ne=function(t,e){if((i=t.length)>1)for(var n,r,i,o=1,a=t[e[0]],s=a.length;o<i;++o)for(r=a,a=t[e[o]],n=0;n<s;++n)a[n][1]+=a[n][0]=isNaN(r[n][1])?r[n][0]:r[n][1]},re=function(t){for(var e=t.length,n=new Array(e);--e>=0;)n[e]=e;return n};function ie(t,e){return t[e]}var oe=function(){var t=i([]),e=re,n=ne,r=ie;function o(i){var o,a,s=t.apply(this,arguments),A=i.length,l=s.length,c=new Array(l);for(o=0;o<l;++o){for(var u,g=s[o],h=c[o]=new Array(A),d=0;d<A;++d)h[d]=u=[0,+r(i[d],g,d,i)],u.data=i[d];h.key=g}for(o=0,a=e(c);o<l;++o)c[a[o]].index=o;return n(c,a),c}return o.keys=function(e){return arguments.length?(t="function"==typeof e?e:i(G.call(e)),o):t},o.value=function(t){return arguments.length?(r="function"==typeof t?t:i(+t),o):r},o.order=function(t){return arguments.length?(e=null==t?re:"function"==typeof t?t:i(G.call(t)),o):e},o.offset=function(t){return arguments.length?(n=null==t?ne:t,o):n},o},ae=function(t,e){if((r=t.length)>0){for(var n,r,i,o=0,a=t[0].length;o<a;++o){for(i=n=0;n<r;++n)i+=t[n][o][1]||0;if(i)for(n=0;n<r;++n)t[n][o][1]/=i}ne(t,e)}},se=function(t,e){if((s=t.length)>0)for(var n,r,i,o,a,s,A=0,l=t[e[0]].length;A<l;++A)for(o=a=0,n=0;n<s;++n)(i=(r=t[e[n]][A])[1]-r[0])>=0?(r[0]=o,r[1]=o+=i):i<0?(r[1]=a,r[0]=a+=i):r[0]=o},Ae=function(t,e){if((n=t.length)>0){for(var n,r=0,i=t[e[0]],o=i.length;r<o;++r){for(var a=0,s=0;a<n;++a)s+=t[a][r][1]||0;i[r][1]+=i[r][0]=-s/2}ne(t,e)}},le=function(t,e){if((i=t.length)>0&&(r=(n=t[e[0]]).length)>0){for(var n,r,i,o=0,a=1;a<r;++a){for(var s=0,A=0,l=0;s<i;++s){for(var c=t[e[s]],u=c[a][1]||0,g=(u-(c[a-1][1]||0))/2,h=0;h<s;++h){var d=t[e[h]];g+=(d[a][1]||0)-(d[a-1][1]||0)}A+=u,l+=g*u}n[a-1][1]+=n[a-1][0]=o,A&&(o-=l/A)}n[a-1][1]+=n[a-1][0]=o,ne(t,e)}},ce=function(t){var e=t.map(ue);return re(t).sort(function(t,n){return e[t]-e[n]})};function ue(t){for(var e,n=-1,r=0,i=t.length,o=-1/0;++n<i;)(e=+t[n][1])>o&&(o=e,r=n);return r}var ge=function(t){var e=t.map(he);return re(t).sort(function(t,n){return e[t]-e[n]})};function he(t){for(var e,n=0,r=-1,i=t.length;++r<i;)(e=+t[r][1])&&(n+=e);return n}var de=function(t){return ge(t).reverse()},fe=function(t){var e,n,r=t.length,i=t.map(he),o=ce(t),a=0,s=0,A=[],l=[];for(e=0;e<r;++e)n=o[e],a<s?(a+=i[n],A.push(n)):(s+=i[n],l.push(n));return l.reverse().concat(A)},pe=function(t){return re(t).reverse()};n.d(e,"arc",function(){return B}),n.d(e,"area",function(){return D}),n.d(e,"line",function(){return _}),n.d(e,"pie",function(){return S}),n.d(e,"areaRadial",function(){return U}),n.d(e,"radialArea",function(){return U}),n.d(e,"lineRadial",function(){return T}),n.d(e,"radialLine",function(){return T}),n.d(e,"pointRadial",function(){return O}),n.d(e,"linkHorizontal",function(){return J}),n.d(e,"linkVertical",function(){return q}),n.d(e,"linkRadial",function(){return V}),n.d(e,"symbol",function(){return ht}),n.d(e,"symbols",function(){return gt}),n.d(e,"symbolCircle",function(){return W}),n.d(e,"symbolCross",function(){return X}),n.d(e,"symbolDiamond",function(){return tt}),n.d(e,"symbolSquare",function(){return ot}),n.d(e,"symbolStar",function(){return it}),n.d(e,"symbolTriangle",function(){return st}),n.d(e,"symbolWye",function(){return ut}),n.d(e,"curveBasisClosed",function(){return mt}),n.d(e,"curveBasisOpen",function(){return Et}),n.d(e,"curveBasis",function(){return It}),n.d(e,"curveBundle",function(){return Bt}),n.d(e,"curveCardinalClosed",function(){return _t}),n.d(e,"curveCardinalOpen",function(){return Mt}),n.d(e,"curveCardinal",function(){return wt}),n.d(e,"curveCatmullRomClosed",function(){return Nt}),n.d(e,"curveCatmullRomOpen",function(){return Tt}),n.d(e,"curveCatmullRom",function(){return Lt}),n.d(e,"curveLinearClosed",function(){return Ot}),n.d(e,"curveLinear",function(){return Q}),n.d(e,"curveMonotoneX",function(){return Jt}),n.d(e,"curveMonotoneY",function(){return qt}),n.d(e,"curveNatural",function(){return Xt}),n.d(e,"curveStep",function(){return $t}),n.d(e,"curveStepAfter",function(){return ee}),n.d(e,"curveStepBefore",function(){return te}),n.d(e,"stack",function(){return oe}),n.d(e,"stackOffsetExpand",function(){return ae}),n.d(e,"stackOffsetDiverging",function(){return se}),n.d(e,"stackOffsetNone",function(){return ne}),n.d(e,"stackOffsetSilhouette",function(){return Ae}),n.d(e,"stackOffsetWiggle",function(){return le}),n.d(e,"stackOrderAppearance",function(){return ce}),n.d(e,"stackOrderAscending",function(){return ge}),n.d(e,"stackOrderDescending",function(){return de}),n.d(e,"stackOrderInsideOut",function(){return fe}),n.d(e,"stackOrderNone",function(){return re}),n.d(e,"stackOrderReverse",function(){return pe})},function(t,e,n){"use strict";n.r(e);var r=function(){return Math.random()},i=function t(e){function n(t,n){return t=null==t?0:+t,n=null==n?1:+n,1===arguments.length?(n=t,t=0):n-=t,function(){return e()*n+t}}return n.source=t,n}(r),o=function t(e){function n(t,n){var r,i;return t=null==t?0:+t,n=null==n?1:+n,function(){var o;if(null!=r)o=r,r=null;else do{r=2*e()-1,o=2*e()-1,i=r*r+o*o}while(!i||i>1);return t+n*o*Math.sqrt(-2*Math.log(i)/i)}}return n.source=t,n}(r),a=function t(e){function n(){var t=o.source(e).apply(this,arguments);return function(){return Math.exp(t())}}return n.source=t,n}(r),s=function t(e){function n(t){return function(){for(var n=0,r=0;r<t;++r)n+=e();return n}}return n.source=t,n}(r),A=function t(e){function n(t){var n=s.source(e)(t);return function(){return n()/t}}return n.source=t,n}(r),l=function t(e){function n(t){return function(){return-Math.log(1-e())/t}}return n.source=t,n}(r);n.d(e,"randomUniform",function(){return i}),n.d(e,"randomNormal",function(){return o}),n.d(e,"randomLogNormal",function(){return a}),n.d(e,"randomBates",function(){return A}),n.d(e,"randomIrwinHall",function(){return s}),n.d(e,"randomExponential",function(){return l})},function(t,e,n){"use strict";function r(t){if(!t.ok)throw new Error(t.status+" "+t.statusText);return t.blob()}n.r(e);var i=function(t,e){return fetch(t,e).then(r)};function o(t){if(!t.ok)throw new Error(t.status+" "+t.statusText);return t.arrayBuffer()}var a=function(t,e){return fetch(t,e).then(o)},s=n(32);function A(t){if(!t.ok)throw new Error(t.status+" "+t.statusText);return t.text()}var l=function(t,e){return fetch(t,e).then(A)};function c(t){return function(e,n,r){return 2===arguments.length&&"function"==typeof n&&(r=n,n=void 0),l(e,n).then(function(e){return t(e,r)})}}function u(t,e,n,r){3===arguments.length&&"function"==typeof n&&(r=n,n=void 0);var i=Object(s.g)(t);return l(e,n).then(function(t){return i.parse(t,r)})}var g=c(s.e),h=c(s.k),d=function(t,e){return new Promise(function(n,r){var i=new Image;for(var o in e)i[o]=e[o];i.onerror=r,i.onload=function(){n(i)},i.src=t})};function f(t){if(!t.ok)throw new Error(t.status+" "+t.statusText);return t.json()}var p=function(t,e){return fetch(t,e).then(f)};function I(t){return function(e,n){return l(e,n).then(function(e){return(new DOMParser).parseFromString(e,t)})}}var C=I("application/xml"),m=I("text/html"),y=I("image/svg+xml");n.d(e,"blob",function(){return i}),n.d(e,"buffer",function(){return a}),n.d(e,"dsv",function(){return u}),n.d(e,"csv",function(){return g}),n.d(e,"tsv",function(){return h}),n.d(e,"image",function(){return d}),n.d(e,"json",function(){return p}),n.d(e,"text",function(){return l}),n.d(e,"xml",function(){return C}),n.d(e,"html",function(){return m}),n.d(e,"svg",function(){return y})},function(t,e,n){"use strict";n.r(e);var r=n(12),i=n(24),o=n(4),a=n(0),s=n(20),A=function(t){return function(){return t}},l=function(t,e,n){this.target=t,this.type=e,this.selection=n};function c(){a.event.stopImmediatePropagation()}var u=function(){a.event.preventDefault(),a.event.stopImmediatePropagation()},g={name:"drag"},h={name:"space"},d={name:"handle"},f={name:"center"},p={name:"x",handles:["e","w"].map(b),input:function(t,e){return t&&[[t[0],e[0][1]],[t[1],e[1][1]]]},output:function(t){return t&&[t[0][0],t[1][0]]}},I={name:"y",handles:["n","s"].map(b),input:function(t,e){return t&&[[e[0][0],t[0]],[e[1][0],t[1]]]},output:function(t){return t&&[t[0][1],t[1][1]]}},C={name:"xy",handles:["n","e","s","w","nw","ne","se","sw"].map(b),input:function(t){return t},output:function(t){return t}},m={overlay:"crosshair",selection:"move",n:"ns-resize",e:"ew-resize",s:"ns-resize",w:"ew-resize",nw:"nwse-resize",ne:"nesw-resize",se:"nwse-resize",sw:"nesw-resize"},y={e:"w",w:"e",nw:"ne",ne:"nw",se:"sw",sw:"se"},E={n:"s",s:"n",nw:"sw",ne:"se",se:"ne",sw:"nw"},v={overlay:1,selection:1,n:null,e:1,s:null,w:-1,nw:-1,ne:1,se:1,sw:-1},B={overlay:1,selection:1,n:-1,e:null,s:1,w:null,nw:-1,ne:-1,se:1,sw:1};function b(t){return{type:t}}function Q(){return!a.event.button}function w(){var t=this.ownerSVGElement||this;return[[0,0],[t.width.baseVal.value,t.height.baseVal.value]]}function x(t){for(;!t.__brush;)if(!(t=t.parentNode))return;return t.__brush}function _(t){return t[0][0]===t[1][0]||t[0][1]===t[1][1]}function D(t){var e=t.__brush;return e?e.dim.output(e.selection):null}function M(){return L(p)}function k(){return L(I)}var S=function(){return L(C)};function L(t){var e,n=w,C=Q,D=Object(r.dispatch)(k,"start","brush","end"),M=6;function k(e){var n=e.property("__brush",F).selectAll(".overlay").data([b("overlay")]);n.enter().append("rect").attr("class","overlay").attr("pointer-events","all").attr("cursor",m.overlay).merge(n).each(function(){var t=x(this).extent;Object(a.select)(this).attr("x",t[0][0]).attr("y",t[0][1]).attr("width",t[1][0]-t[0][0]).attr("height",t[1][1]-t[0][1])}),e.selectAll(".selection").data([b("selection")]).enter().append("rect").attr("class","selection").attr("cursor",m.selection).attr("fill","#777").attr("fill-opacity",.3).attr("stroke","#fff").attr("shape-rendering","crispEdges");var r=e.selectAll(".handle").data(t.handles,function(t){return t.type});r.exit().remove(),r.enter().append("rect").attr("class",function(t){return"handle handle--"+t.type}).attr("cursor",function(t){return m[t.type]}),e.each(S).attr("fill","none").attr("pointer-events","all").style("-webkit-tap-highlight-color","rgba(0,0,0,0)").on("mousedown.brush touchstart.brush",N)}function S(){var t=Object(a.select)(this),e=x(this).selection;e?(t.selectAll(".selection").style("display",null).attr("x",e[0][0]).attr("y",e[0][1]).attr("width",e[1][0]-e[0][0]).attr("height",e[1][1]-e[0][1]),t.selectAll(".handle").style("display",null).attr("x",function(t){return"e"===t.type[t.type.length-1]?e[1][0]-M/2:e[0][0]-M/2}).attr("y",function(t){return"s"===t.type[0]?e[1][1]-M/2:e[0][1]-M/2}).attr("width",function(t){return"n"===t.type||"s"===t.type?e[1][0]-e[0][0]+M:M}).attr("height",function(t){return"e"===t.type||"w"===t.type?e[1][1]-e[0][1]+M:M})):t.selectAll(".selection,.handle").style("display","none").attr("x",null).attr("y",null).attr("width",null).attr("height",null)}function L(t,e){return t.__brush.emitter||new R(t,e)}function R(t,e){this.that=t,this.args=e,this.state=t.__brush,this.active=0}function N(){if(a.event.touches){if(a.event.changedTouches.length<a.event.touches.length)return u()}else if(e)return;if(C.apply(this,arguments)){var n,r,o,A,l,b,Q,w,D,M,k,R,N,F=this,T=a.event.target.__data__.type,U="selection"===(a.event.metaKey?T="overlay":T)?g:a.event.altKey?f:d,O=t===I?null:v[T],G=t===p?null:B[T],H=x(F),j=H.extent,Y=H.selection,P=j[0][0],K=j[0][1],z=j[1][0],J=j[1][1],q=O&&G&&a.event.shiftKey,V=Object(a.mouse)(F),W=V,X=L(F,arguments).beforestart();"overlay"===T?H.selection=Y=[[n=t===I?P:V[0],o=t===p?K:V[1]],[l=t===I?z:n,Q=t===p?J:o]]:(n=Y[0][0],o=Y[0][1],l=Y[1][0],Q=Y[1][1]),r=n,A=o,b=l,w=Q;var Z=Object(a.select)(F).attr("pointer-events","none"),$=Z.selectAll(".overlay").attr("cursor",m[T]);if(a.event.touches)Z.on("touchmove.brush",et,!0).on("touchend.brush touchcancel.brush",rt,!0);else{var tt=Object(a.select)(a.event.view).on("keydown.brush",function(){switch(a.event.keyCode){case 16:q=O&&G;break;case 18:U===d&&(O&&(l=b-D*O,n=r+D*O),G&&(Q=w-M*G,o=A+M*G),U=f,nt());break;case 32:U!==d&&U!==f||(O<0?l=b-D:O>0&&(n=r-D),G<0?Q=w-M:G>0&&(o=A-M),U=h,$.attr("cursor",m.selection),nt());break;default:return}u()},!0).on("keyup.brush",function(){switch(a.event.keyCode){case 16:q&&(R=N=q=!1,nt());break;case 18:U===f&&(O<0?l=b:O>0&&(n=r),G<0?Q=w:G>0&&(o=A),U=d,nt());break;case 32:U===h&&(a.event.altKey?(O&&(l=b-D*O,n=r+D*O),G&&(Q=w-M*G,o=A+M*G),U=f):(O<0?l=b:O>0&&(n=r),G<0?Q=w:G>0&&(o=A),U=d),$.attr("cursor",m[T]),nt());break;default:return}u()},!0).on("mousemove.brush",et,!0).on("mouseup.brush",rt,!0);Object(i.b)(a.event.view)}c(),Object(s.b)(F),S.call(F),X.start()}function et(){var t=Object(a.mouse)(F);!q||R||N||(Math.abs(t[0]-W[0])>Math.abs(t[1]-W[1])?N=!0:R=!0),W=t,k=!0,u(),nt()}function nt(){var t;switch(D=W[0]-V[0],M=W[1]-V[1],U){case h:case g:O&&(D=Math.max(P-n,Math.min(z-l,D)),r=n+D,b=l+D),G&&(M=Math.max(K-o,Math.min(J-Q,M)),A=o+M,w=Q+M);break;case d:O<0?(D=Math.max(P-n,Math.min(z-n,D)),r=n+D,b=l):O>0&&(D=Math.max(P-l,Math.min(z-l,D)),r=n,b=l+D),G<0?(M=Math.max(K-o,Math.min(J-o,M)),A=o+M,w=Q):G>0&&(M=Math.max(K-Q,Math.min(J-Q,M)),A=o,w=Q+M);break;case f:O&&(r=Math.max(P,Math.min(z,n-D*O)),b=Math.max(P,Math.min(z,l+D*O))),G&&(A=Math.max(K,Math.min(J,o-M*G)),w=Math.max(K,Math.min(J,Q+M*G)))}b<r&&(O*=-1,t=n,n=l,l=t,t=r,r=b,b=t,T in y&&$.attr("cursor",m[T=y[T]])),w<A&&(G*=-1,t=o,o=Q,Q=t,t=A,A=w,w=t,T in E&&$.attr("cursor",m[T=E[T]])),H.selection&&(Y=H.selection),R&&(r=Y[0][0],b=Y[1][0]),N&&(A=Y[0][1],w=Y[1][1]),Y[0][0]===r&&Y[0][1]===A&&Y[1][0]===b&&Y[1][1]===w||(H.selection=[[r,A],[b,w]],S.call(F),X.brush())}function rt(){if(c(),a.event.touches){if(a.event.touches.length)return;e&&clearTimeout(e),e=setTimeout(function(){e=null},500),Z.on("touchmove.brush touchend.brush touchcancel.brush",null)}else Object(i.c)(a.event.view,k),tt.on("keydown.brush keyup.brush mousemove.brush mouseup.brush",null);Z.attr("pointer-events","all"),$.attr("cursor",m.overlay),H.selection&&(Y=H.selection),_(Y)&&(H.selection=null,S.call(F)),X.end()}}function F(){var e=this.__brush||{selection:null};return e.extent=n.apply(this,arguments),e.dim=t,e}return k.move=function(e,n){e.selection?e.on("start.brush",function(){L(this,arguments).beforestart().start()}).on("interrupt.brush end.brush",function(){L(this,arguments).end()}).tween("brush",function(){var e=this,r=e.__brush,i=L(e,arguments),a=r.selection,s=t.input("function"==typeof n?n.apply(this,arguments):n,r.extent),A=Object(o.a)(a,s);function l(t){r.selection=1===t&&_(s)?null:A(t),S.call(e),i.brush()}return a&&s?l:l(1)}):e.each(function(){var e=this,r=arguments,i=e.__brush,o=t.input("function"==typeof n?n.apply(e,r):n,i.extent),a=L(e,r).beforestart();Object(s.b)(e),i.selection=null==o||_(o)?null:o,S.call(e),a.start().brush().end()})},R.prototype={beforestart:function(){return 1==++this.active&&(this.state.emitter=this,this.starting=!0),this},start:function(){return this.starting&&(this.starting=!1,this.emit("start")),this},brush:function(){return this.emit("brush"),this},end:function(){return 0==--this.active&&(delete this.state.emitter,this.emit("end")),this},emit:function(e){Object(a.customEvent)(new l(k,e,t.output(this.state.selection)),D.apply,D,[e,this.that,this.args])}},k.extent=function(t){return arguments.length?(n="function"==typeof t?t:A([[+t[0][0],+t[0][1]],[+t[1][0],+t[1][1]]]),k):n},k.filter=function(t){return arguments.length?(C="function"==typeof t?t:A(!!t),k):C},k.handleSize=function(t){return arguments.length?(M=+t,k):M},k.on=function(){var t=D.on.apply(D,arguments);return t===D?k:t},k}n.d(e,"brush",function(){return S}),n.d(e,"brushX",function(){return M}),n.d(e,"brushY",function(){return k}),n.d(e,"brushSelection",function(){return D})},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(41),i=n(25),o=n(26),a=n(28),s=n(130),A=n(33),l=n(91);class c extends r.Part{constructor(){super(...arguments),this.lastColumn=null,this.gridContext=new u(this.context,this),this.gridContainer=new s.SlickGridWidget(this.gridContext)}static GetMetadata(){const t=super.GetMetadata();return t.description="Advanced data grid for query outputs",t.remarks='\nThe SlickGridPart is a datagrid control, and the most comprehensive part in the\nMavenomics part palette. It features native support for MQL hierarchal rows,\nsparklines, number formatters, conditional formatting, and so much more.\n\n#### Column Properties\n\nColumn formatting can be edited by right-clicking on a column header and\nselecting "Edit Column Properties". Here you can set generic styles, such as the\ncolumn header\'s background color and number formatting, as well as other\nformatters such as the Heatmap and ProgressBar formatters. Formatting is keyed\non column names, so if you rename a column you will lose the formatting.\n\n#### Row Paths\n\nGrouped tables returned by MQL `WITH ROLLUP` queries are natively understood\nby the SlickGrid control. If you want further context on the hierarchies, the\nSlickGrid can also show a "path column" that will let you navigate the rows as\na collapsible tree.\n\n#### Column Grouping\n\nSlickGrid supports primitive column grouping. Columns will be treed on \'dots\',\nwhich means a column named "Foo.Bar.Baz" will have 2 collapsible levels "Foo"\nand "Bar", with a third level having just "Baz". If you have adjacent columns\nwith the same parent, they will be grouped together.',t.addOption(r.PartUtils.INPUT_OPTION,o.Types.Table,i.Table.NullTable(),{description:"The input table for this part. Defaults to a null table."}),t.addOption("Formatting",o.Types.String,"{}",{description:"A JSON string describing the formatting for each column in the grid."}),t.addOption("Checklist. Enabled",o.Types.Boolean,!1,{description:"Whether to enable the Checklist feature."}),t.addOption("Checklist.Column for Checkboxes",o.Types.Number,1,{description:"If [Checklist. Enabled] is true, the index of the column to display the checklists in"}),t.addOption("Checklist.Column for Values",o.Types.Number,1,{description:"If [Checklist. Enabled] is true, the index of the column to pull checklist values from"}),t.addOption("Checklist.Selected Values",o.Types.Array,[],{description:"If [Checklist. Enabled] is true, the values of all the checked rows."}),t.addOption("RadioButtonList. Enabled",o.Types.Boolean,!1,{description:"Whether to enable the Radio Button feature."}),t.addOption("RadioButtonList.Column for Radio Buttons",o.Types.Number,1,{description:"If [RadioButtonList. Enabled] is true, the index of the column to display the radio buttons in."}),t.addOption("RadioButtonList.Column for Values",o.Types.Number,1,{description:"If [RadioButtonList. Enabled] is true, the index of the column to pull radio values from."}),t.addOption("RadioButtonList.Last Selection",o.Types.Any,"",{description:"If [RadioButtonList. Enabled] is true, the most recently selected radio value."}),t.addOption("Number of Frozen Columns",o.Types.Number,0),t.addOption("Show Row Selectors",o.Types.Boolean,!1,{description:"Whether to show Excel-style row numbers on the far left of the grid."}),t.addOption("Show Path Column",o.Types.String,"Automatic",{description:"Whether to show the path of each row on the far left. One of `Always`, `Never`, or `Automatic`. When set to Auto, the Path column will be hidden for flat tables but visible for grouped tables.",schema:{type:"string",enum:["Always","Never","Automatic"]}}),t}async initialize(){this.gridContainer.addClass("m-SlickGridPart"),this.layout.insertWidget(0,this.gridContainer)}async render(t){const e=t.get(r.PartUtils.INPUT_OPTION);if(null==e)throw new Error("Input Table must not be null");if(!(e instanceof i.Table)){if(!e.rows||!e.columnNames)throw new Error("Input Table must be a Table.");console.warn("SlickGrid Input Table looks like a table but isn't an instance of Table"),console.warn("This may lead to errors and undefined behavior.")}if(0===e.columnNames.length)throw new Error("Cannot render an empty table.");this.gridContainer.show();const n=A.parseFormatting(""+t.get("Formatting"));this.gridContext._setBag(t),await o.AsyncTools.waitForFrame(),await o.AsyncTools.wait(),await this.gridContainer.Render(e,n)}getName(){return"SlickGrid"}getColumnFormatting(t){return this.gridContext.getColumnFormatting(t)}setColumnFormatting(t,e){return this.gridContext.setColumnFormatting(t,e)}}e.SlickGridPart=c;class u{constructor(t,e){this.services=t,this.owner=e,this.collapseStates=[],this.pathColumn=-1,this.bag=null,this.hoverHandle=null,this.partId=e.uuid,this.version=l.UUID.uuid4()}get lastColumn(){return this.owner.lastColumn}set lastColumn(t){this.owner.lastColumn=t}_setBag(t){this.bag=t}get(t){if(null!=this.bag){if("Show Path Column"===t){const t=this.bag.get("Show Path Column");if("boolean"==typeof t)return t;if("Automatic"===t){const t=this.bag.get(r.PartUtils.INPUT_OPTION);return t instanceof i.Table&&t.length!==t.rows.length}return"Always"===t}return this.bag.get(t)}}set(t,e){if(null!=this.bag)return this.bag.set(t,e)}OpenHover(t,e,n,r,i){this.CloseHover();const o=new a.Widget;o.node.innerHTML=t,this.hoverHandle=this.services.hover.openHover({height:i,width:r,hover:o,owner:this.owner,mode:"hover",offsetMode:"absolute",x:e,y:n})}OpenTableHover(t,e,n,i,o,a,l){this.CloseHover();const g=new u(this.services,this.owner),h=new r.OptionsBag(c.GetMetadata());h.set(r.PartUtils.INPUT_OPTION,t),h.set("Formatting",e),g._setBag(h);const d=new s.SlickGridWidget(g);d.addClass("m-SlickGridPart"),d.addClass(n?"m-TablePopup":"m-TableHover");const f=A.parseFormatting(""+e);d.Render(t,f);const p=this.services.hover.openHover({height:l,width:a,hover:d,owner:this.owner,mode:n?"dialog":"hover",offsetMode:"absolute",x:i,y:o});n||(this.hoverHandle=p)}OpenDashboardHover(t,e,n,r=500,i=400){const o=this.CreateHoverWidget(t);this.CloseHover(),this.hoverHandle=this.services.hover.openHover({hover:o,owner:this.owner,mode:"hover",width:r,height:i,x:e,y:n,offsetMode:"absolute"})}OpenDashboardPopup(t,e,n,r=500,i=400){const o=["left="+e,"top="+n,"width="+r,"height="+i].join(",");window.open(t,l.UUID.uuid4(),o)}CloseHover(){null!=this.hoverHandle&&this.services.hover.closeHover(this.hoverHandle)}getColumnFormatting(t){return A.parseFormatting(""+this.get("Formatting"))[t]}setColumnFormatting(t,e){const n=A.parseFormatting(""+this.get("Formatting"));n[t]=e,this.set("Formatting",A.serializeFormatting(n))}CreateHoverWidget(t){const e=new a.Widget({node:document.createElement("iframe")});return e.node.src=t,Object.assign(e.node.style,{width:"100%",height:"100%"}),e}}},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0});const r=n(143),i=n(33),o=n(42),a=n(25);function s(t,e){return t.styleFormatter=e||!1,t}function A(t){return"string"==typeof t?t.replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;"):t}e.NullFormatter=s(function(t){return void 0!==t&&"Show"===t.nullRule?'<div style="text-align: right; color: #b6b6b6; font-style: italic;">NULL</div>':""}),e.ErrorFormatter=s(()=>'<span class="m-cell-error">*</span>'),e.BasicValueFormatter=s(function(t,n,r,i,o,s){return r instanceof a.Table?e.TableHoverFormatter(t,n,r,i,o,s):Number.isNaN(Number(r))||"number"!=typeof r&&"string"!=typeof r?null==r?e.NullFormatter(i):r instanceof Error?e.ErrorFormatter():'<span style="position:relative; top:-0.1em">'+r+"</span>":'<span style="float:right;">'+r+"</span>"}),e.PathIndentationFormatter=s(function(e,n,r,i,o,a){null===r&&(r=""),r=A(r);var s=t("<span>").css({display:"inline-block",height:"1px",width:15*o.indent,float:"left"}).attr("depth",o.indent).prop("outerHTML"),l=a.getIdxById(o.id);return a.getItems()[l+1]&&a.getItems()[l+1].indent>a.getItems()[l].indent?o._collapsed?s+" <span class='toggle expand'></span>&nbsp;":s+" <span class='toggle collapse'></span>&nbsp;":s+" <span class='toggle'></span>&nbsp;"},!0),e.CheckBoxColumnFormatter=s(function(t,e,n,r,i,o){var a=i["col"+r.columnNumber+"_checked"];return!0===a?'<input type="checkbox" class="slick-cell-checkbox" checked="checked"/>':"indeterminate"===a?'<input type="checkbox" class="slick-cell-checkbox indeterminate"/>':'<input type="checkbox" class="slick-cell-checkbox"/>'},!0),e.RadioButtonColumnFormatter=s(function(t,e,n,r,i,o){var a=i._radio;return!0===a?'<input type="radio" class="slick-cell-radio" checked="true"/>':!1===a?'<input type="radio" class="slick-cell-radio"/>':void 0},!0),e.HeatmapColumnFormatter=s(function(t,n,r,o,a,s){if(r instanceof Error)return e.ErrorFormatter();if(null!==r){r=A(r);var l=o.heatmap.func(r);a["cssStyle"+n]="background-color: "+l+";color:"+i.idealTextColor(l)+";"}},!0),e.NumberFormatter=s(function(t,n,i,o,a,s){return null===i?e.NullFormatter(o):i instanceof Error?e.ErrorFormatter():i===1/0||i===-1/0?"":Number.isNaN(i)?"":"number"==typeof(i=A(i))?(a["cssStyle"+n]=a["cssStyle"+n]||"",'<span style="float:right">'+r(i).format(o.numberFormat)+"</span>"):void 0}),e.ConditionalFormatter=s(function(t,n,r,i,o,a){if(null!==r){if(r instanceof Error)return e.ErrorFormatter();r=A(r),o["cssStyle"+n]=o["cssStyle"+n]||"",i.conditionalFormatters.forEach(function(t){if(!0===t.compute(o)){var e=t.getOptions();""!==e.BackColor&&(o["cssStyle"+n]+="background-color: "+e.BackColor+";"),""!==e.ForeColor&&(o["cssStyle"+n]+="color: "+e.ForeColor+";"),e.IsBold&&(o["cssStyle"+n]+="font-weight: bold;"),e.IsItalic&&(o["cssStyle"+n]+="font-style: italic;")}})}},!0),e.ProgressBarFormatter=s(function(n,r,i,o,a,s){if(null!==i){if(i instanceof Error)return e.ErrorFormatter();i=A(i);var l=o.progressBar.func(i);l>100&&(l=100);var c="#ffffff",u=t("#row-depth-styles");u.length>0&&a.indent&&(c=u.prop("sheet").cssRules[a.indent].style.backgroundColor);var g=l<100?", "+c+" "+(l+1)+"%":"",h=o.progressBar,d=["background: linear-gradient(to right, "+h.startColor+" 0%,"+h.endColor+" "+l+"%"+g+")"].join(";")+";";a["cssStyle"+r]=a["cssStyle"+r]||"",a["cssStyle"+r]+=d}},!0),e.RowDetailFormatter=s(function(t,n,r,i,o,a){return r instanceof Error?e.ErrorFormatter():'<span class="row-detail row-detail-image"></span>'},!0),e.SparklineLoadingFormatter=function(t){const n=s(function(n,r,i,a,s,A){return i instanceof Error?e.ErrorFormatter():i?o.GetSparklineFromCache(t.grid,t.version,s.rowPath,a.field)||"<i>Loading chart...</i>":void 0},!0);return n.formatName=e.SparklineLoadingFormatter.formatName,n},e.SparklineLoadingFormatter.formatName="SparklineLoadingFormatter",e.DashboardLinkFormatter=s(function(t,n,r,i,o,a){return r instanceof Error?e.ErrorFormatter():"string"==typeof r&&r.length>0?'<span class="dashboard-link-image"></span>':"<span></span>"},!0),e.TableHoverFormatter=s(function(t,n,r,i,o,a){return r instanceof Error?e.ErrorFormatter():`<span class="table-hover table-hover-image">${r.rows.length}</span>`},!0)}).call(this,n(5))},function(t,e,n){(function(t,r){var i;
/**
 * @license
 * Lodash <https://lodash.com/>
 * Copyright JS Foundation and other contributors <https://js.foundation/>
 * Released under MIT license <https://lodash.com/license>
 * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
 * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
 */
/**
 * @license
 * Lodash <https://lodash.com/>
 * Copyright JS Foundation and other contributors <https://js.foundation/>
 * Released under MIT license <https://lodash.com/license>
 * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
 * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
 */
(function(){var o,a=200,s="Expected a function",A="__lodash_hash_undefined__",l="__lodash_placeholder__",c=1,u=2,g=4,h=1,d=2,f=1,p=2,I=4,C=8,m=16,y=32,E=64,v=128,B=256,b=512,Q=800,w=16,x=1/0,_=9007199254740991,D=17976931348623157e292,M=NaN,k=4294967295,S=k-1,L=k>>>1,R=[["ary",v],["bind",f],["bindKey",p],["curry",C],["curryRight",m],["flip",b],["partial",y],["partialRight",E],["rearg",B]],N="[object Arguments]",F="[object Array]",T="[object AsyncFunction]",U="[object Boolean]",O="[object Date]",G="[object DOMException]",H="[object Error]",j="[object Function]",Y="[object GeneratorFunction]",P="[object Map]",K="[object Number]",z="[object Null]",J="[object Object]",q="[object Proxy]",V="[object RegExp]",W="[object Set]",X="[object String]",Z="[object Symbol]",$="[object Undefined]",tt="[object WeakMap]",et="[object ArrayBuffer]",nt="[object DataView]",rt="[object Float32Array]",it="[object Float64Array]",ot="[object Int8Array]",at="[object Int16Array]",st="[object Int32Array]",At="[object Uint8Array]",lt="[object Uint8ClampedArray]",ct="[object Uint16Array]",ut="[object Uint32Array]",gt=/\b__p \+= '';/g,ht=/\b(__p \+=) '' \+/g,dt=/(__e\(.*?\)|\b__t\)) \+\n'';/g,ft=/&(?:amp|lt|gt|quot|#39);/g,pt=/[&<>"']/g,It=RegExp(ft.source),Ct=RegExp(pt.source),mt=/<%-([\s\S]+?)%>/g,yt=/<%([\s\S]+?)%>/g,Et=/<%=([\s\S]+?)%>/g,vt=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,Bt=/^\w*$/,bt=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,Qt=/[\\^$.*+?()[\]{}|]/g,wt=RegExp(Qt.source),xt=/^\s+|\s+$/g,_t=/^\s+/,Dt=/\s+$/,Mt=/\{(?:\n\/\* \[wrapped with .+\] \*\/)?\n?/,kt=/\{\n\/\* \[wrapped with (.+)\] \*/,St=/,? & /,Lt=/[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g,Rt=/\\(\\)?/g,Nt=/\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g,Ft=/\w*$/,Tt=/^[-+]0x[0-9a-f]+$/i,Ut=/^0b[01]+$/i,Ot=/^\[object .+?Constructor\]$/,Gt=/^0o[0-7]+$/i,Ht=/^(?:0|[1-9]\d*)$/,jt=/[\xc0-\xd6\xd8-\xf6\xf8-\xff\u0100-\u017f]/g,Yt=/($^)/,Pt=/['\n\r\u2028\u2029\\]/g,Kt="\\u0300-\\u036f\\ufe20-\\ufe2f\\u20d0-\\u20ff",zt="\\xac\\xb1\\xd7\\xf7\\x00-\\x2f\\x3a-\\x40\\x5b-\\x60\\x7b-\\xbf\\u2000-\\u206f \\t\\x0b\\f\\xa0\\ufeff\\n\\r\\u2028\\u2029\\u1680\\u180e\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200a\\u202f\\u205f\\u3000",Jt="["+zt+"]",qt="["+Kt+"]",Vt="\\d+",Wt="[a-z\\xdf-\\xf6\\xf8-\\xff]",Xt="[^\\ud800-\\udfff"+zt+Vt+"\\u2700-\\u27bfa-z\\xdf-\\xf6\\xf8-\\xffA-Z\\xc0-\\xd6\\xd8-\\xde]",Zt="\\ud83c[\\udffb-\\udfff]",$t="[^\\ud800-\\udfff]",te="(?:\\ud83c[\\udde6-\\uddff]){2}",ee="[\\ud800-\\udbff][\\udc00-\\udfff]",ne="[A-Z\\xc0-\\xd6\\xd8-\\xde]",re="(?:"+Wt+"|"+Xt+")",ie="(?:"+ne+"|"+Xt+")",oe="(?:"+qt+"|"+Zt+")?",ae="[\\ufe0e\\ufe0f]?"+oe+"(?:\\u200d(?:"+[$t,te,ee].join("|")+")[\\ufe0e\\ufe0f]?"+oe+")*",se="(?:"+["[\\u2700-\\u27bf]",te,ee].join("|")+")"+ae,Ae="(?:"+[$t+qt+"?",qt,te,ee,"[\\ud800-\\udfff]"].join("|")+")",le=RegExp("['’]","g"),ce=RegExp(qt,"g"),ue=RegExp(Zt+"(?="+Zt+")|"+Ae+ae,"g"),ge=RegExp([ne+"?"+Wt+"+(?:['’](?:d|ll|m|re|s|t|ve))?(?="+[Jt,ne,"$"].join("|")+")",ie+"+(?:['’](?:D|LL|M|RE|S|T|VE))?(?="+[Jt,ne+re,"$"].join("|")+")",ne+"?"+re+"+(?:['’](?:d|ll|m|re|s|t|ve))?",ne+"+(?:['’](?:D|LL|M|RE|S|T|VE))?","\\d*(?:1ST|2ND|3RD|(?![123])\\dTH)(?=\\b|[a-z_])","\\d*(?:1st|2nd|3rd|(?![123])\\dth)(?=\\b|[A-Z_])",Vt,se].join("|"),"g"),he=RegExp("[\\u200d\\ud800-\\udfff"+Kt+"\\ufe0e\\ufe0f]"),de=/[a-z][A-Z]|[A-Z]{2}[a-z]|[0-9][a-zA-Z]|[a-zA-Z][0-9]|[^a-zA-Z0-9 ]/,fe=["Array","Buffer","DataView","Date","Error","Float32Array","Float64Array","Function","Int8Array","Int16Array","Int32Array","Map","Math","Object","Promise","RegExp","Set","String","Symbol","TypeError","Uint8Array","Uint8ClampedArray","Uint16Array","Uint32Array","WeakMap","_","clearTimeout","isFinite","parseInt","setTimeout"],pe=-1,Ie={};Ie[rt]=Ie[it]=Ie[ot]=Ie[at]=Ie[st]=Ie[At]=Ie[lt]=Ie[ct]=Ie[ut]=!0,Ie[N]=Ie[F]=Ie[et]=Ie[U]=Ie[nt]=Ie[O]=Ie[H]=Ie[j]=Ie[P]=Ie[K]=Ie[J]=Ie[V]=Ie[W]=Ie[X]=Ie[tt]=!1;var Ce={};Ce[N]=Ce[F]=Ce[et]=Ce[nt]=Ce[U]=Ce[O]=Ce[rt]=Ce[it]=Ce[ot]=Ce[at]=Ce[st]=Ce[P]=Ce[K]=Ce[J]=Ce[V]=Ce[W]=Ce[X]=Ce[Z]=Ce[At]=Ce[lt]=Ce[ct]=Ce[ut]=!0,Ce[H]=Ce[j]=Ce[tt]=!1;var me={"\\":"\\","'":"'","\n":"n","\r":"r","\u2028":"u2028","\u2029":"u2029"},ye=parseFloat,Ee=parseInt,ve="object"==typeof t&&t&&t.Object===Object&&t,Be="object"==typeof self&&self&&self.Object===Object&&self,be=ve||Be||Function("return this")(),Qe=e&&!e.nodeType&&e,we=Qe&&"object"==typeof r&&r&&!r.nodeType&&r,xe=we&&we.exports===Qe,_e=xe&&ve.process,De=function(){try{return we&&we.require&&we.require("util").types||_e&&_e.binding&&_e.binding("util")}catch(t){}}(),Me=De&&De.isArrayBuffer,ke=De&&De.isDate,Se=De&&De.isMap,Le=De&&De.isRegExp,Re=De&&De.isSet,Ne=De&&De.isTypedArray;function Fe(t,e,n){switch(n.length){case 0:return t.call(e);case 1:return t.call(e,n[0]);case 2:return t.call(e,n[0],n[1]);case 3:return t.call(e,n[0],n[1],n[2])}return t.apply(e,n)}function Te(t,e,n,r){for(var i=-1,o=null==t?0:t.length;++i<o;){var a=t[i];e(r,a,n(a),t)}return r}function Ue(t,e){for(var n=-1,r=null==t?0:t.length;++n<r&&!1!==e(t[n],n,t););return t}function Oe(t,e){for(var n=-1,r=null==t?0:t.length;++n<r;)if(!e(t[n],n,t))return!1;return!0}function Ge(t,e){for(var n=-1,r=null==t?0:t.length,i=0,o=[];++n<r;){var a=t[n];e(a,n,t)&&(o[i++]=a)}return o}function He(t,e){return!(null==t||!t.length)&&Xe(t,e,0)>-1}function je(t,e,n){for(var r=-1,i=null==t?0:t.length;++r<i;)if(n(e,t[r]))return!0;return!1}function Ye(t,e){for(var n=-1,r=null==t?0:t.length,i=Array(r);++n<r;)i[n]=e(t[n],n,t);return i}function Pe(t,e){for(var n=-1,r=e.length,i=t.length;++n<r;)t[i+n]=e[n];return t}function Ke(t,e,n,r){var i=-1,o=null==t?0:t.length;for(r&&o&&(n=t[++i]);++i<o;)n=e(n,t[i],i,t);return n}function ze(t,e,n,r){var i=null==t?0:t.length;for(r&&i&&(n=t[--i]);i--;)n=e(n,t[i],i,t);return n}function Je(t,e){for(var n=-1,r=null==t?0:t.length;++n<r;)if(e(t[n],n,t))return!0;return!1}var qe=en("length");function Ve(t,e,n){var r;return n(t,function(t,n,i){if(e(t,n,i))return r=n,!1}),r}function We(t,e,n,r){for(var i=t.length,o=n+(r?1:-1);r?o--:++o<i;)if(e(t[o],o,t))return o;return-1}function Xe(t,e,n){return e==e?function(t,e,n){for(var r=n-1,i=t.length;++r<i;)if(t[r]===e)return r;return-1}(t,e,n):We(t,$e,n)}function Ze(t,e,n,r){for(var i=n-1,o=t.length;++i<o;)if(r(t[i],e))return i;return-1}function $e(t){return t!=t}function tn(t,e){var n=null==t?0:t.length;return n?on(t,e)/n:M}function en(t){return function(e){return null==e?o:e[t]}}function nn(t){return function(e){return null==t?o:t[e]}}function rn(t,e,n,r,i){return i(t,function(t,i,o){n=r?(r=!1,t):e(n,t,i,o)}),n}function on(t,e){for(var n,r=-1,i=t.length;++r<i;){var a=e(t[r]);a!==o&&(n=n===o?a:n+a)}return n}function an(t,e){for(var n=-1,r=Array(t);++n<t;)r[n]=e(n);return r}function sn(t){return function(e){return t(e)}}function An(t,e){return Ye(e,function(e){return t[e]})}function ln(t,e){return t.has(e)}function cn(t,e){for(var n=-1,r=t.length;++n<r&&Xe(e,t[n],0)>-1;);return n}function un(t,e){for(var n=t.length;n--&&Xe(e,t[n],0)>-1;);return n}var gn=nn({"À":"A","Á":"A","Â":"A","Ã":"A","Ä":"A","Å":"A","à":"a","á":"a","â":"a","ã":"a","ä":"a","å":"a","Ç":"C","ç":"c","Ð":"D","ð":"d","È":"E","É":"E","Ê":"E","Ë":"E","è":"e","é":"e","ê":"e","ë":"e","Ì":"I","Í":"I","Î":"I","Ï":"I","ì":"i","í":"i","î":"i","ï":"i","Ñ":"N","ñ":"n","Ò":"O","Ó":"O","Ô":"O","Õ":"O","Ö":"O","Ø":"O","ò":"o","ó":"o","ô":"o","õ":"o","ö":"o","ø":"o","Ù":"U","Ú":"U","Û":"U","Ü":"U","ù":"u","ú":"u","û":"u","ü":"u","Ý":"Y","ý":"y","ÿ":"y","Æ":"Ae","æ":"ae","Þ":"Th","þ":"th","ß":"ss","Ā":"A","Ă":"A","Ą":"A","ā":"a","ă":"a","ą":"a","Ć":"C","Ĉ":"C","Ċ":"C","Č":"C","ć":"c","ĉ":"c","ċ":"c","č":"c","Ď":"D","Đ":"D","ď":"d","đ":"d","Ē":"E","Ĕ":"E","Ė":"E","Ę":"E","Ě":"E","ē":"e","ĕ":"e","ė":"e","ę":"e","ě":"e","Ĝ":"G","Ğ":"G","Ġ":"G","Ģ":"G","ĝ":"g","ğ":"g","ġ":"g","ģ":"g","Ĥ":"H","Ħ":"H","ĥ":"h","ħ":"h","Ĩ":"I","Ī":"I","Ĭ":"I","Į":"I","İ":"I","ĩ":"i","ī":"i","ĭ":"i","į":"i","ı":"i","Ĵ":"J","ĵ":"j","Ķ":"K","ķ":"k","ĸ":"k","Ĺ":"L","Ļ":"L","Ľ":"L","Ŀ":"L","Ł":"L","ĺ":"l","ļ":"l","ľ":"l","ŀ":"l","ł":"l","Ń":"N","Ņ":"N","Ň":"N","Ŋ":"N","ń":"n","ņ":"n","ň":"n","ŋ":"n","Ō":"O","Ŏ":"O","Ő":"O","ō":"o","ŏ":"o","ő":"o","Ŕ":"R","Ŗ":"R","Ř":"R","ŕ":"r","ŗ":"r","ř":"r","Ś":"S","Ŝ":"S","Ş":"S","Š":"S","ś":"s","ŝ":"s","ş":"s","š":"s","Ţ":"T","Ť":"T","Ŧ":"T","ţ":"t","ť":"t","ŧ":"t","Ũ":"U","Ū":"U","Ŭ":"U","Ů":"U","Ű":"U","Ų":"U","ũ":"u","ū":"u","ŭ":"u","ů":"u","ű":"u","ų":"u","Ŵ":"W","ŵ":"w","Ŷ":"Y","ŷ":"y","Ÿ":"Y","Ź":"Z","Ż":"Z","Ž":"Z","ź":"z","ż":"z","ž":"z","Ĳ":"IJ","ĳ":"ij","Œ":"Oe","œ":"oe","ŉ":"'n","ſ":"s"}),hn=nn({"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"});function dn(t){return"\\"+me[t]}function fn(t){return he.test(t)}function pn(t){var e=-1,n=Array(t.size);return t.forEach(function(t,r){n[++e]=[r,t]}),n}function In(t,e){return function(n){return t(e(n))}}function Cn(t,e){for(var n=-1,r=t.length,i=0,o=[];++n<r;){var a=t[n];a!==e&&a!==l||(t[n]=l,o[i++]=n)}return o}function mn(t){var e=-1,n=Array(t.size);return t.forEach(function(t){n[++e]=t}),n}function yn(t){return fn(t)?function(t){for(var e=ue.lastIndex=0;ue.test(t);)++e;return e}(t):qe(t)}function En(t){return fn(t)?function(t){return t.match(ue)||[]}(t):function(t){return t.split("")}(t)}var vn=nn({"&amp;":"&","&lt;":"<","&gt;":">","&quot;":'"',"&#39;":"'"}),Bn=function t(e){var n,r=(e=null==e?be:Bn.defaults(be.Object(),e,Bn.pick(be,fe))).Array,i=e.Date,Kt=e.Error,zt=e.Function,Jt=e.Math,qt=e.Object,Vt=e.RegExp,Wt=e.String,Xt=e.TypeError,Zt=r.prototype,$t=zt.prototype,te=qt.prototype,ee=e["__core-js_shared__"],ne=$t.toString,re=te.hasOwnProperty,ie=0,oe=(n=/[^.]+$/.exec(ee&&ee.keys&&ee.keys.IE_PROTO||""))?"Symbol(src)_1."+n:"",ae=te.toString,se=ne.call(qt),Ae=be._,ue=Vt("^"+ne.call(re).replace(Qt,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),he=xe?e.Buffer:o,me=e.Symbol,ve=e.Uint8Array,Be=he?he.allocUnsafe:o,Qe=In(qt.getPrototypeOf,qt),we=qt.create,_e=te.propertyIsEnumerable,De=Zt.splice,qe=me?me.isConcatSpreadable:o,nn=me?me.iterator:o,bn=me?me.toStringTag:o,Qn=function(){try{var t=bo(qt,"defineProperty");return t({},"",{}),t}catch(t){}}(),wn=e.clearTimeout!==be.clearTimeout&&e.clearTimeout,xn=i&&i.now!==be.Date.now&&i.now,_n=e.setTimeout!==be.setTimeout&&e.setTimeout,Dn=Jt.ceil,Mn=Jt.floor,kn=qt.getOwnPropertySymbols,Sn=he?he.isBuffer:o,Ln=e.isFinite,Rn=Zt.join,Nn=In(qt.keys,qt),Fn=Jt.max,Tn=Jt.min,Un=i.now,On=e.parseInt,Gn=Jt.random,Hn=Zt.reverse,jn=bo(e,"DataView"),Yn=bo(e,"Map"),Pn=bo(e,"Promise"),Kn=bo(e,"Set"),zn=bo(e,"WeakMap"),Jn=bo(qt,"create"),qn=zn&&new zn,Vn={},Wn=Wo(jn),Xn=Wo(Yn),Zn=Wo(Pn),$n=Wo(Kn),tr=Wo(zn),er=me?me.prototype:o,nr=er?er.valueOf:o,rr=er?er.toString:o;function ir(t){if(hs(t)&&!ns(t)&&!(t instanceof Ar)){if(t instanceof sr)return t;if(re.call(t,"__wrapped__"))return Xo(t)}return new sr(t)}var or=function(){function t(){}return function(e){if(!gs(e))return{};if(we)return we(e);t.prototype=e;var n=new t;return t.prototype=o,n}}();function ar(){}function sr(t,e){this.__wrapped__=t,this.__actions__=[],this.__chain__=!!e,this.__index__=0,this.__values__=o}function Ar(t){this.__wrapped__=t,this.__actions__=[],this.__dir__=1,this.__filtered__=!1,this.__iteratees__=[],this.__takeCount__=k,this.__views__=[]}function lr(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function cr(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function ur(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function gr(t){var e=-1,n=null==t?0:t.length;for(this.__data__=new ur;++e<n;)this.add(t[e])}function hr(t){var e=this.__data__=new cr(t);this.size=e.size}function dr(t,e){var n=ns(t),r=!n&&es(t),i=!n&&!r&&as(t),o=!n&&!r&&!i&&Es(t),a=n||r||i||o,s=a?an(t.length,Wt):[],A=s.length;for(var l in t)!e&&!re.call(t,l)||a&&("length"==l||i&&("offset"==l||"parent"==l)||o&&("buffer"==l||"byteLength"==l||"byteOffset"==l)||ko(l,A))||s.push(l);return s}function fr(t){var e=t.length;return e?t[li(0,e-1)]:o}function pr(t,e,n){(n===o||Za(t[e],n))&&(n!==o||e in t)||Er(t,e,n)}function Ir(t,e,n){var r=t[e];re.call(t,e)&&Za(r,n)&&(n!==o||e in t)||Er(t,e,n)}function Cr(t,e){for(var n=t.length;n--;)if(Za(t[n][0],e))return n;return-1}function mr(t,e,n,r){return _r(t,function(t,i,o){e(r,t,n(t),o)}),r}function yr(t,e){return t&&ji(e,Ys(e),t)}function Er(t,e,n){"__proto__"==e&&Qn?Qn(t,e,{configurable:!0,enumerable:!0,value:n,writable:!0}):t[e]=n}function vr(t,e){for(var n=-1,i=e.length,a=r(i),s=null==t;++n<i;)a[n]=s?o:Us(t,e[n]);return a}function Br(t,e,n){return t==t&&(n!==o&&(t=t<=n?t:n),e!==o&&(t=t>=e?t:e)),t}function br(t,e,n,r,i,a){var s,A=e&c,l=e&u,h=e&g;if(n&&(s=i?n(t,r,i,a):n(t)),s!==o)return s;if(!gs(t))return t;var d=ns(t);if(d){if(s=function(t){var e=t.length,n=new t.constructor(e);return e&&"string"==typeof t[0]&&re.call(t,"index")&&(n.index=t.index,n.input=t.input),n}(t),!A)return Hi(t,s)}else{var f=xo(t),p=f==j||f==Y;if(as(t))return Ni(t,A);if(f==J||f==N||p&&!i){if(s=l||p?{}:Do(t),!A)return l?function(t,e){return ji(t,wo(t),e)}(t,function(t,e){return t&&ji(e,Ps(e),t)}(s,t)):function(t,e){return ji(t,Qo(t),e)}(t,yr(s,t))}else{if(!Ce[f])return i?t:{};s=function(t,e,n){var r,i=t.constructor;switch(e){case et:return Fi(t);case U:case O:return new i(+t);case nt:return function(t,e){var n=e?Fi(t.buffer):t.buffer;return new t.constructor(n,t.byteOffset,t.byteLength)}(t,n);case rt:case it:case ot:case at:case st:case At:case lt:case ct:case ut:return Ti(t,n);case P:return new i;case K:case X:return new i(t);case V:return function(t){var e=new t.constructor(t.source,Ft.exec(t));return e.lastIndex=t.lastIndex,e}(t);case W:return new i;case Z:return r=t,nr?qt(nr.call(r)):{}}}(t,f,A)}}a||(a=new hr);var I=a.get(t);if(I)return I;if(a.set(t,s),Cs(t))return t.forEach(function(r){s.add(br(r,e,n,r,t,a))}),s;if(ds(t))return t.forEach(function(r,i){s.set(i,br(r,e,n,i,t,a))}),s;var C=d?o:(h?l?Io:po:l?Ps:Ys)(t);return Ue(C||t,function(r,i){C&&(r=t[i=r]),Ir(s,i,br(r,e,n,i,t,a))}),s}function Qr(t,e,n){var r=n.length;if(null==t)return!r;for(t=qt(t);r--;){var i=n[r],a=e[i],s=t[i];if(s===o&&!(i in t)||!a(s))return!1}return!0}function wr(t,e,n){if("function"!=typeof t)throw new Xt(s);return Yo(function(){t.apply(o,n)},e)}function xr(t,e,n,r){var i=-1,o=He,s=!0,A=t.length,l=[],c=e.length;if(!A)return l;n&&(e=Ye(e,sn(n))),r?(o=je,s=!1):e.length>=a&&(o=ln,s=!1,e=new gr(e));t:for(;++i<A;){var u=t[i],g=null==n?u:n(u);if(u=r||0!==u?u:0,s&&g==g){for(var h=c;h--;)if(e[h]===g)continue t;l.push(u)}else o(e,g,r)||l.push(u)}return l}ir.templateSettings={escape:mt,evaluate:yt,interpolate:Et,variable:"",imports:{_:ir}},ir.prototype=ar.prototype,ir.prototype.constructor=ir,sr.prototype=or(ar.prototype),sr.prototype.constructor=sr,Ar.prototype=or(ar.prototype),Ar.prototype.constructor=Ar,lr.prototype.clear=function(){this.__data__=Jn?Jn(null):{},this.size=0},lr.prototype.delete=function(t){var e=this.has(t)&&delete this.__data__[t];return this.size-=e?1:0,e},lr.prototype.get=function(t){var e=this.__data__;if(Jn){var n=e[t];return n===A?o:n}return re.call(e,t)?e[t]:o},lr.prototype.has=function(t){var e=this.__data__;return Jn?e[t]!==o:re.call(e,t)},lr.prototype.set=function(t,e){var n=this.__data__;return this.size+=this.has(t)?0:1,n[t]=Jn&&e===o?A:e,this},cr.prototype.clear=function(){this.__data__=[],this.size=0},cr.prototype.delete=function(t){var e=this.__data__,n=Cr(e,t);return!(n<0||(n==e.length-1?e.pop():De.call(e,n,1),--this.size,0))},cr.prototype.get=function(t){var e=this.__data__,n=Cr(e,t);return n<0?o:e[n][1]},cr.prototype.has=function(t){return Cr(this.__data__,t)>-1},cr.prototype.set=function(t,e){var n=this.__data__,r=Cr(n,t);return r<0?(++this.size,n.push([t,e])):n[r][1]=e,this},ur.prototype.clear=function(){this.size=0,this.__data__={hash:new lr,map:new(Yn||cr),string:new lr}},ur.prototype.delete=function(t){var e=vo(this,t).delete(t);return this.size-=e?1:0,e},ur.prototype.get=function(t){return vo(this,t).get(t)},ur.prototype.has=function(t){return vo(this,t).has(t)},ur.prototype.set=function(t,e){var n=vo(this,t),r=n.size;return n.set(t,e),this.size+=n.size==r?0:1,this},gr.prototype.add=gr.prototype.push=function(t){return this.__data__.set(t,A),this},gr.prototype.has=function(t){return this.__data__.has(t)},hr.prototype.clear=function(){this.__data__=new cr,this.size=0},hr.prototype.delete=function(t){var e=this.__data__,n=e.delete(t);return this.size=e.size,n},hr.prototype.get=function(t){return this.__data__.get(t)},hr.prototype.has=function(t){return this.__data__.has(t)},hr.prototype.set=function(t,e){var n=this.__data__;if(n instanceof cr){var r=n.__data__;if(!Yn||r.length<a-1)return r.push([t,e]),this.size=++n.size,this;n=this.__data__=new ur(r)}return n.set(t,e),this.size=n.size,this};var _r=Ki(Fr),Dr=Ki(Tr,!0);function Mr(t,e){var n=!0;return _r(t,function(t,r,i){return n=!!e(t,r,i)}),n}function kr(t,e,n){for(var r=-1,i=t.length;++r<i;){var a=t[r],s=e(a);if(null!=s&&(A===o?s==s&&!ys(s):n(s,A)))var A=s,l=a}return l}function Sr(t,e){var n=[];return _r(t,function(t,r,i){e(t,r,i)&&n.push(t)}),n}function Lr(t,e,n,r,i){var o=-1,a=t.length;for(n||(n=Mo),i||(i=[]);++o<a;){var s=t[o];e>0&&n(s)?e>1?Lr(s,e-1,n,r,i):Pe(i,s):r||(i[i.length]=s)}return i}var Rr=zi(),Nr=zi(!0);function Fr(t,e){return t&&Rr(t,e,Ys)}function Tr(t,e){return t&&Nr(t,e,Ys)}function Ur(t,e){return Ge(e,function(e){return ls(t[e])})}function Or(t,e){for(var n=0,r=(e=ki(e,t)).length;null!=t&&n<r;)t=t[Vo(e[n++])];return n&&n==r?t:o}function Gr(t,e,n){var r=e(t);return ns(t)?r:Pe(r,n(t))}function Hr(t){return null==t?t===o?$:z:bn&&bn in qt(t)?function(t){var e=re.call(t,bn),n=t[bn];try{t[bn]=o;var r=!0}catch(t){}var i=ae.call(t);return r&&(e?t[bn]=n:delete t[bn]),i}(t):function(t){return ae.call(t)}(t)}function jr(t,e){return t>e}function Yr(t,e){return null!=t&&re.call(t,e)}function Pr(t,e){return null!=t&&e in qt(t)}function Kr(t,e,n){for(var i=n?je:He,a=t[0].length,s=t.length,A=s,l=r(s),c=1/0,u=[];A--;){var g=t[A];A&&e&&(g=Ye(g,sn(e))),c=Tn(g.length,c),l[A]=!n&&(e||a>=120&&g.length>=120)?new gr(A&&g):o}g=t[0];var h=-1,d=l[0];t:for(;++h<a&&u.length<c;){var f=g[h],p=e?e(f):f;if(f=n||0!==f?f:0,!(d?ln(d,p):i(u,p,n))){for(A=s;--A;){var I=l[A];if(!(I?ln(I,p):i(t[A],p,n)))continue t}d&&d.push(p),u.push(f)}}return u}function zr(t,e,n){var r=null==(t=Go(t,e=ki(e,t)))?t:t[Vo(Aa(e))];return null==r?o:Fe(r,t,n)}function Jr(t){return hs(t)&&Hr(t)==N}function qr(t,e,n,r,i){return t===e||(null==t||null==e||!hs(t)&&!hs(e)?t!=t&&e!=e:function(t,e,n,r,i,a){var s=ns(t),A=ns(e),l=s?F:xo(t),c=A?F:xo(e),u=(l=l==N?J:l)==J,g=(c=c==N?J:c)==J,f=l==c;if(f&&as(t)){if(!as(e))return!1;s=!0,u=!1}if(f&&!u)return a||(a=new hr),s||Es(t)?ho(t,e,n,r,i,a):function(t,e,n,r,i,o,a){switch(n){case nt:if(t.byteLength!=e.byteLength||t.byteOffset!=e.byteOffset)return!1;t=t.buffer,e=e.buffer;case et:return!(t.byteLength!=e.byteLength||!o(new ve(t),new ve(e)));case U:case O:case K:return Za(+t,+e);case H:return t.name==e.name&&t.message==e.message;case V:case X:return t==e+"";case P:var s=pn;case W:var A=r&h;if(s||(s=mn),t.size!=e.size&&!A)return!1;var l=a.get(t);if(l)return l==e;r|=d,a.set(t,e);var c=ho(s(t),s(e),r,i,o,a);return a.delete(t),c;case Z:if(nr)return nr.call(t)==nr.call(e)}return!1}(t,e,l,n,r,i,a);if(!(n&h)){var p=u&&re.call(t,"__wrapped__"),I=g&&re.call(e,"__wrapped__");if(p||I){var C=p?t.value():t,m=I?e.value():e;return a||(a=new hr),i(C,m,n,r,a)}}return!!f&&(a||(a=new hr),function(t,e,n,r,i,a){var s=n&h,A=po(t),l=A.length;if(l!=po(e).length&&!s)return!1;for(var c=l;c--;){var u=A[c];if(!(s?u in e:re.call(e,u)))return!1}var g=a.get(t);if(g&&a.get(e))return g==e;var d=!0;a.set(t,e),a.set(e,t);for(var f=s;++c<l;){var p=t[u=A[c]],I=e[u];if(r)var C=s?r(I,p,u,e,t,a):r(p,I,u,t,e,a);if(!(C===o?p===I||i(p,I,n,r,a):C)){d=!1;break}f||(f="constructor"==u)}if(d&&!f){var m=t.constructor,y=e.constructor;m!=y&&"constructor"in t&&"constructor"in e&&!("function"==typeof m&&m instanceof m&&"function"==typeof y&&y instanceof y)&&(d=!1)}return a.delete(t),a.delete(e),d}(t,e,n,r,i,a))}(t,e,n,r,qr,i))}function Vr(t,e,n,r){var i=n.length,a=i,s=!r;if(null==t)return!a;for(t=qt(t);i--;){var A=n[i];if(s&&A[2]?A[1]!==t[A[0]]:!(A[0]in t))return!1}for(;++i<a;){var l=(A=n[i])[0],c=t[l],u=A[1];if(s&&A[2]){if(c===o&&!(l in t))return!1}else{var g=new hr;if(r)var f=r(c,u,l,t,e,g);if(!(f===o?qr(u,c,h|d,r,g):f))return!1}}return!0}function Wr(t){return!(!gs(t)||(e=t,oe&&oe in e))&&(ls(t)?ue:Ot).test(Wo(t));var e}function Xr(t){return"function"==typeof t?t:null==t?fA:"object"==typeof t?ns(t)?ni(t[0],t[1]):ei(t):bA(t)}function Zr(t){if(!Fo(t))return Nn(t);var e=[];for(var n in qt(t))re.call(t,n)&&"constructor"!=n&&e.push(n);return e}function $r(t,e){return t<e}function ti(t,e){var n=-1,i=is(t)?r(t.length):[];return _r(t,function(t,r,o){i[++n]=e(t,r,o)}),i}function ei(t){var e=Bo(t);return 1==e.length&&e[0][2]?Uo(e[0][0],e[0][1]):function(n){return n===t||Vr(n,t,e)}}function ni(t,e){return Lo(t)&&To(e)?Uo(Vo(t),e):function(n){var r=Us(n,t);return r===o&&r===e?Os(n,t):qr(e,r,h|d)}}function ri(t,e,n,r,i){t!==e&&Rr(e,function(a,s){if(gs(a))i||(i=new hr),function(t,e,n,r,i,a,s){var A=Ho(t,n),l=Ho(e,n),c=s.get(l);if(c)pr(t,n,c);else{var u=a?a(A,l,n+"",t,e,s):o,g=u===o;if(g){var h=ns(l),d=!h&&as(l),f=!h&&!d&&Es(l);u=l,h||d||f?ns(A)?u=A:os(A)?u=Hi(A):d?(g=!1,u=Ni(l,!0)):f?(g=!1,u=Ti(l,!0)):u=[]:ps(l)||es(l)?(u=A,es(A)?u=Ds(A):gs(A)&&!ls(A)||(u=Do(l))):g=!1}g&&(s.set(l,u),i(u,l,r,a,s),s.delete(l)),pr(t,n,u)}}(t,e,s,n,ri,r,i);else{var A=r?r(Ho(t,s),a,s+"",t,e,i):o;A===o&&(A=a),pr(t,s,A)}},Ps)}function ii(t,e){var n=t.length;if(n)return ko(e+=e<0?n:0,n)?t[e]:o}function oi(t,e,n){var r=-1;return e=Ye(e.length?e:[fA],sn(Eo())),function(t,e){var n=t.length;for(t.sort(e);n--;)t[n]=t[n].value;return t}(ti(t,function(t,n,i){return{criteria:Ye(e,function(e){return e(t)}),index:++r,value:t}}),function(t,e){return function(t,e,n){for(var r=-1,i=t.criteria,o=e.criteria,a=i.length,s=n.length;++r<a;){var A=Ui(i[r],o[r]);if(A)return r>=s?A:A*("desc"==n[r]?-1:1)}return t.index-e.index}(t,e,n)})}function ai(t,e,n){for(var r=-1,i=e.length,o={};++r<i;){var a=e[r],s=Or(t,a);n(s,a)&&gi(o,ki(a,t),s)}return o}function si(t,e,n,r){var i=r?Ze:Xe,o=-1,a=e.length,s=t;for(t===e&&(e=Hi(e)),n&&(s=Ye(t,sn(n)));++o<a;)for(var A=0,l=e[o],c=n?n(l):l;(A=i(s,c,A,r))>-1;)s!==t&&De.call(s,A,1),De.call(t,A,1);return t}function Ai(t,e){for(var n=t?e.length:0,r=n-1;n--;){var i=e[n];if(n==r||i!==o){var o=i;ko(i)?De.call(t,i,1):Bi(t,i)}}return t}function li(t,e){return t+Mn(Gn()*(e-t+1))}function ci(t,e){var n="";if(!t||e<1||e>_)return n;do{e%2&&(n+=t),(e=Mn(e/2))&&(t+=t)}while(e);return n}function ui(t,e){return Po(Oo(t,e,fA),t+"")}function gi(t,e,n,r){if(!gs(t))return t;for(var i=-1,a=(e=ki(e,t)).length,s=a-1,A=t;null!=A&&++i<a;){var l=Vo(e[i]),c=n;if(i!=s){var u=A[l];(c=r?r(u,l,A):o)===o&&(c=gs(u)?u:ko(e[i+1])?[]:{})}Ir(A,l,c),A=A[l]}return t}var hi=qn?function(t,e){return qn.set(t,e),t}:fA,di=Qn?function(t,e){return Qn(t,"toString",{configurable:!0,enumerable:!1,value:gA(e),writable:!0})}:fA;function fi(t,e,n){var i=-1,o=t.length;e<0&&(e=-e>o?0:o+e),(n=n>o?o:n)<0&&(n+=o),o=e>n?0:n-e>>>0,e>>>=0;for(var a=r(o);++i<o;)a[i]=t[i+e];return a}function pi(t,e){var n;return _r(t,function(t,r,i){return!(n=e(t,r,i))}),!!n}function Ii(t,e,n){var r=0,i=null==t?r:t.length;if("number"==typeof e&&e==e&&i<=L){for(;r<i;){var o=r+i>>>1,a=t[o];null!==a&&!ys(a)&&(n?a<=e:a<e)?r=o+1:i=o}return i}return Ci(t,e,fA,n)}function Ci(t,e,n,r){e=n(e);for(var i=0,a=null==t?0:t.length,s=e!=e,A=null===e,l=ys(e),c=e===o;i<a;){var u=Mn((i+a)/2),g=n(t[u]),h=g!==o,d=null===g,f=g==g,p=ys(g);if(s)var I=r||f;else I=c?f&&(r||h):A?f&&h&&(r||!d):l?f&&h&&!d&&(r||!p):!d&&!p&&(r?g<=e:g<e);I?i=u+1:a=u}return Tn(a,S)}function mi(t,e){for(var n=-1,r=t.length,i=0,o=[];++n<r;){var a=t[n],s=e?e(a):a;if(!n||!Za(s,A)){var A=s;o[i++]=0===a?0:a}}return o}function yi(t){return"number"==typeof t?t:ys(t)?M:+t}function Ei(t){if("string"==typeof t)return t;if(ns(t))return Ye(t,Ei)+"";if(ys(t))return rr?rr.call(t):"";var e=t+"";return"0"==e&&1/t==-x?"-0":e}function vi(t,e,n){var r=-1,i=He,o=t.length,s=!0,A=[],l=A;if(n)s=!1,i=je;else if(o>=a){var c=e?null:so(t);if(c)return mn(c);s=!1,i=ln,l=new gr}else l=e?[]:A;t:for(;++r<o;){var u=t[r],g=e?e(u):u;if(u=n||0!==u?u:0,s&&g==g){for(var h=l.length;h--;)if(l[h]===g)continue t;e&&l.push(g),A.push(u)}else i(l,g,n)||(l!==A&&l.push(g),A.push(u))}return A}function Bi(t,e){return null==(t=Go(t,e=ki(e,t)))||delete t[Vo(Aa(e))]}function bi(t,e,n,r){return gi(t,e,n(Or(t,e)),r)}function Qi(t,e,n,r){for(var i=t.length,o=r?i:-1;(r?o--:++o<i)&&e(t[o],o,t););return n?fi(t,r?0:o,r?o+1:i):fi(t,r?o+1:0,r?i:o)}function wi(t,e){var n=t;return n instanceof Ar&&(n=n.value()),Ke(e,function(t,e){return e.func.apply(e.thisArg,Pe([t],e.args))},n)}function xi(t,e,n){var i=t.length;if(i<2)return i?vi(t[0]):[];for(var o=-1,a=r(i);++o<i;)for(var s=t[o],A=-1;++A<i;)A!=o&&(a[o]=xr(a[o]||s,t[A],e,n));return vi(Lr(a,1),e,n)}function _i(t,e,n){for(var r=-1,i=t.length,a=e.length,s={};++r<i;){var A=r<a?e[r]:o;n(s,t[r],A)}return s}function Di(t){return os(t)?t:[]}function Mi(t){return"function"==typeof t?t:fA}function ki(t,e){return ns(t)?t:Lo(t,e)?[t]:qo(Ms(t))}var Si=ui;function Li(t,e,n){var r=t.length;return n=n===o?r:n,!e&&n>=r?t:fi(t,e,n)}var Ri=wn||function(t){return be.clearTimeout(t)};function Ni(t,e){if(e)return t.slice();var n=t.length,r=Be?Be(n):new t.constructor(n);return t.copy(r),r}function Fi(t){var e=new t.constructor(t.byteLength);return new ve(e).set(new ve(t)),e}function Ti(t,e){var n=e?Fi(t.buffer):t.buffer;return new t.constructor(n,t.byteOffset,t.length)}function Ui(t,e){if(t!==e){var n=t!==o,r=null===t,i=t==t,a=ys(t),s=e!==o,A=null===e,l=e==e,c=ys(e);if(!A&&!c&&!a&&t>e||a&&s&&l&&!A&&!c||r&&s&&l||!n&&l||!i)return 1;if(!r&&!a&&!c&&t<e||c&&n&&i&&!r&&!a||A&&n&&i||!s&&i||!l)return-1}return 0}function Oi(t,e,n,i){for(var o=-1,a=t.length,s=n.length,A=-1,l=e.length,c=Fn(a-s,0),u=r(l+c),g=!i;++A<l;)u[A]=e[A];for(;++o<s;)(g||o<a)&&(u[n[o]]=t[o]);for(;c--;)u[A++]=t[o++];return u}function Gi(t,e,n,i){for(var o=-1,a=t.length,s=-1,A=n.length,l=-1,c=e.length,u=Fn(a-A,0),g=r(u+c),h=!i;++o<u;)g[o]=t[o];for(var d=o;++l<c;)g[d+l]=e[l];for(;++s<A;)(h||o<a)&&(g[d+n[s]]=t[o++]);return g}function Hi(t,e){var n=-1,i=t.length;for(e||(e=r(i));++n<i;)e[n]=t[n];return e}function ji(t,e,n,r){var i=!n;n||(n={});for(var a=-1,s=e.length;++a<s;){var A=e[a],l=r?r(n[A],t[A],A,n,t):o;l===o&&(l=t[A]),i?Er(n,A,l):Ir(n,A,l)}return n}function Yi(t,e){return function(n,r){var i=ns(n)?Te:mr,o=e?e():{};return i(n,t,Eo(r,2),o)}}function Pi(t){return ui(function(e,n){var r=-1,i=n.length,a=i>1?n[i-1]:o,s=i>2?n[2]:o;for(a=t.length>3&&"function"==typeof a?(i--,a):o,s&&So(n[0],n[1],s)&&(a=i<3?o:a,i=1),e=qt(e);++r<i;){var A=n[r];A&&t(e,A,r,a)}return e})}function Ki(t,e){return function(n,r){if(null==n)return n;if(!is(n))return t(n,r);for(var i=n.length,o=e?i:-1,a=qt(n);(e?o--:++o<i)&&!1!==r(a[o],o,a););return n}}function zi(t){return function(e,n,r){for(var i=-1,o=qt(e),a=r(e),s=a.length;s--;){var A=a[t?s:++i];if(!1===n(o[A],A,o))break}return e}}function Ji(t){return function(e){var n=fn(e=Ms(e))?En(e):o,r=n?n[0]:e.charAt(0),i=n?Li(n,1).join(""):e.slice(1);return r[t]()+i}}function qi(t){return function(e){return Ke(lA(eA(e).replace(le,"")),t,"")}}function Vi(t){return function(){var e=arguments;switch(e.length){case 0:return new t;case 1:return new t(e[0]);case 2:return new t(e[0],e[1]);case 3:return new t(e[0],e[1],e[2]);case 4:return new t(e[0],e[1],e[2],e[3]);case 5:return new t(e[0],e[1],e[2],e[3],e[4]);case 6:return new t(e[0],e[1],e[2],e[3],e[4],e[5]);case 7:return new t(e[0],e[1],e[2],e[3],e[4],e[5],e[6])}var n=or(t.prototype),r=t.apply(n,e);return gs(r)?r:n}}function Wi(t){return function(e,n,r){var i=qt(e);if(!is(e)){var a=Eo(n,3);e=Ys(e),n=function(t){return a(i[t],t,i)}}var s=t(e,n,r);return s>-1?i[a?e[s]:s]:o}}function Xi(t){return fo(function(e){var n=e.length,r=n,i=sr.prototype.thru;for(t&&e.reverse();r--;){var a=e[r];if("function"!=typeof a)throw new Xt(s);if(i&&!A&&"wrapper"==mo(a))var A=new sr([],!0)}for(r=A?r:n;++r<n;){var l=mo(a=e[r]),c="wrapper"==l?Co(a):o;A=c&&Ro(c[0])&&c[1]==(v|C|y|B)&&!c[4].length&&1==c[9]?A[mo(c[0])].apply(A,c[3]):1==a.length&&Ro(a)?A[l]():A.thru(a)}return function(){var t=arguments,r=t[0];if(A&&1==t.length&&ns(r))return A.plant(r).value();for(var i=0,o=n?e[i].apply(this,t):r;++i<n;)o=e[i].call(this,o);return o}})}function Zi(t,e,n,i,a,s,A,l,c,u){var g=e&v,h=e&f,d=e&p,I=e&(C|m),y=e&b,E=d?o:Vi(t);return function f(){for(var p=arguments.length,C=r(p),m=p;m--;)C[m]=arguments[m];if(I)var v=yo(f),B=function(t,e){for(var n=t.length,r=0;n--;)t[n]===e&&++r;return r}(C,v);if(i&&(C=Oi(C,i,a,I)),s&&(C=Gi(C,s,A,I)),p-=B,I&&p<u){var b=Cn(C,v);return oo(t,e,Zi,f.placeholder,n,C,b,l,c,u-p)}var Q=h?n:this,w=d?Q[t]:t;return p=C.length,l?C=function(t,e){for(var n=t.length,r=Tn(e.length,n),i=Hi(t);r--;){var a=e[r];t[r]=ko(a,n)?i[a]:o}return t}(C,l):y&&p>1&&C.reverse(),g&&c<p&&(C.length=c),this&&this!==be&&this instanceof f&&(w=E||Vi(w)),w.apply(Q,C)}}function $i(t,e){return function(n,r){return function(t,e,n,r){return Fr(t,function(t,i,o){e(r,n(t),i,o)}),r}(n,t,e(r),{})}}function to(t,e){return function(n,r){var i;if(n===o&&r===o)return e;if(n!==o&&(i=n),r!==o){if(i===o)return r;"string"==typeof n||"string"==typeof r?(n=Ei(n),r=Ei(r)):(n=yi(n),r=yi(r)),i=t(n,r)}return i}}function eo(t){return fo(function(e){return e=Ye(e,sn(Eo())),ui(function(n){var r=this;return t(e,function(t){return Fe(t,r,n)})})})}function no(t,e){var n=(e=e===o?" ":Ei(e)).length;if(n<2)return n?ci(e,t):e;var r=ci(e,Dn(t/yn(e)));return fn(e)?Li(En(r),0,t).join(""):r.slice(0,t)}function ro(t){return function(e,n,i){return i&&"number"!=typeof i&&So(e,n,i)&&(n=i=o),e=Qs(e),n===o?(n=e,e=0):n=Qs(n),function(t,e,n,i){for(var o=-1,a=Fn(Dn((e-t)/(n||1)),0),s=r(a);a--;)s[i?a:++o]=t,t+=n;return s}(e,n,i=i===o?e<n?1:-1:Qs(i),t)}}function io(t){return function(e,n){return"string"==typeof e&&"string"==typeof n||(e=_s(e),n=_s(n)),t(e,n)}}function oo(t,e,n,r,i,a,s,A,l,c){var u=e&C;e|=u?y:E,(e&=~(u?E:y))&I||(e&=~(f|p));var g=[t,e,i,u?a:o,u?s:o,u?o:a,u?o:s,A,l,c],h=n.apply(o,g);return Ro(t)&&jo(h,g),h.placeholder=r,Ko(h,t,e)}function ao(t){var e=Jt[t];return function(t,n){if(t=_s(t),n=null==n?0:Tn(ws(n),292)){var r=(Ms(t)+"e").split("e");return+((r=(Ms(e(r[0]+"e"+(+r[1]+n)))+"e").split("e"))[0]+"e"+(+r[1]-n))}return e(t)}}var so=Kn&&1/mn(new Kn([,-0]))[1]==x?function(t){return new Kn(t)}:yA;function Ao(t){return function(e){var n=xo(e);return n==P?pn(e):n==W?function(t){var e=-1,n=Array(t.size);return t.forEach(function(t){n[++e]=[t,t]}),n}(e):function(t,e){return Ye(e,function(e){return[e,t[e]]})}(e,t(e))}}function lo(t,e,n,i,a,A,c,u){var g=e&p;if(!g&&"function"!=typeof t)throw new Xt(s);var h=i?i.length:0;if(h||(e&=~(y|E),i=a=o),c=c===o?c:Fn(ws(c),0),u=u===o?u:ws(u),h-=a?a.length:0,e&E){var d=i,b=a;i=a=o}var Q=g?o:Co(t),w=[t,e,n,i,a,d,b,A,c,u];if(Q&&function(t,e){var n=t[1],r=e[1],i=n|r,o=i<(f|p|v),a=r==v&&n==C||r==v&&n==B&&t[7].length<=e[8]||r==(v|B)&&e[7].length<=e[8]&&n==C;if(!o&&!a)return t;r&f&&(t[2]=e[2],i|=n&f?0:I);var s=e[3];if(s){var A=t[3];t[3]=A?Oi(A,s,e[4]):s,t[4]=A?Cn(t[3],l):e[4]}(s=e[5])&&(A=t[5],t[5]=A?Gi(A,s,e[6]):s,t[6]=A?Cn(t[5],l):e[6]),(s=e[7])&&(t[7]=s),r&v&&(t[8]=null==t[8]?e[8]:Tn(t[8],e[8])),null==t[9]&&(t[9]=e[9]),t[0]=e[0],t[1]=i}(w,Q),t=w[0],e=w[1],n=w[2],i=w[3],a=w[4],!(u=w[9]=w[9]===o?g?0:t.length:Fn(w[9]-h,0))&&e&(C|m)&&(e&=~(C|m)),e&&e!=f)x=e==C||e==m?function(t,e,n){var i=Vi(t);return function a(){for(var s=arguments.length,A=r(s),l=s,c=yo(a);l--;)A[l]=arguments[l];var u=s<3&&A[0]!==c&&A[s-1]!==c?[]:Cn(A,c);return(s-=u.length)<n?oo(t,e,Zi,a.placeholder,o,A,u,o,o,n-s):Fe(this&&this!==be&&this instanceof a?i:t,this,A)}}(t,e,u):e!=y&&e!=(f|y)||a.length?Zi.apply(o,w):function(t,e,n,i){var o=e&f,a=Vi(t);return function e(){for(var s=-1,A=arguments.length,l=-1,c=i.length,u=r(c+A),g=this&&this!==be&&this instanceof e?a:t;++l<c;)u[l]=i[l];for(;A--;)u[l++]=arguments[++s];return Fe(g,o?n:this,u)}}(t,e,n,i);else var x=function(t,e,n){var r=e&f,i=Vi(t);return function e(){return(this&&this!==be&&this instanceof e?i:t).apply(r?n:this,arguments)}}(t,e,n);return Ko((Q?hi:jo)(x,w),t,e)}function co(t,e,n,r){return t===o||Za(t,te[n])&&!re.call(r,n)?e:t}function uo(t,e,n,r,i,a){return gs(t)&&gs(e)&&(a.set(e,t),ri(t,e,o,uo,a),a.delete(e)),t}function go(t){return ps(t)?o:t}function ho(t,e,n,r,i,a){var s=n&h,A=t.length,l=e.length;if(A!=l&&!(s&&l>A))return!1;var c=a.get(t);if(c&&a.get(e))return c==e;var u=-1,g=!0,f=n&d?new gr:o;for(a.set(t,e),a.set(e,t);++u<A;){var p=t[u],I=e[u];if(r)var C=s?r(I,p,u,e,t,a):r(p,I,u,t,e,a);if(C!==o){if(C)continue;g=!1;break}if(f){if(!Je(e,function(t,e){if(!ln(f,e)&&(p===t||i(p,t,n,r,a)))return f.push(e)})){g=!1;break}}else if(p!==I&&!i(p,I,n,r,a)){g=!1;break}}return a.delete(t),a.delete(e),g}function fo(t){return Po(Oo(t,o,ra),t+"")}function po(t){return Gr(t,Ys,Qo)}function Io(t){return Gr(t,Ps,wo)}var Co=qn?function(t){return qn.get(t)}:yA;function mo(t){for(var e=t.name+"",n=Vn[e],r=re.call(Vn,e)?n.length:0;r--;){var i=n[r],o=i.func;if(null==o||o==t)return i.name}return e}function yo(t){return(re.call(ir,"placeholder")?ir:t).placeholder}function Eo(){var t=ir.iteratee||pA;return t=t===pA?Xr:t,arguments.length?t(arguments[0],arguments[1]):t}function vo(t,e){var n,r,i=t.__data__;return("string"==(r=typeof(n=e))||"number"==r||"symbol"==r||"boolean"==r?"__proto__"!==n:null===n)?i["string"==typeof e?"string":"hash"]:i.map}function Bo(t){for(var e=Ys(t),n=e.length;n--;){var r=e[n],i=t[r];e[n]=[r,i,To(i)]}return e}function bo(t,e){var n=function(t,e){return null==t?o:t[e]}(t,e);return Wr(n)?n:o}var Qo=kn?function(t){return null==t?[]:(t=qt(t),Ge(kn(t),function(e){return _e.call(t,e)}))}:xA,wo=kn?function(t){for(var e=[];t;)Pe(e,Qo(t)),t=Qe(t);return e}:xA,xo=Hr;function _o(t,e,n){for(var r=-1,i=(e=ki(e,t)).length,o=!1;++r<i;){var a=Vo(e[r]);if(!(o=null!=t&&n(t,a)))break;t=t[a]}return o||++r!=i?o:!!(i=null==t?0:t.length)&&us(i)&&ko(a,i)&&(ns(t)||es(t))}function Do(t){return"function"!=typeof t.constructor||Fo(t)?{}:or(Qe(t))}function Mo(t){return ns(t)||es(t)||!!(qe&&t&&t[qe])}function ko(t,e){var n=typeof t;return!!(e=null==e?_:e)&&("number"==n||"symbol"!=n&&Ht.test(t))&&t>-1&&t%1==0&&t<e}function So(t,e,n){if(!gs(n))return!1;var r=typeof e;return!!("number"==r?is(n)&&ko(e,n.length):"string"==r&&e in n)&&Za(n[e],t)}function Lo(t,e){if(ns(t))return!1;var n=typeof t;return!("number"!=n&&"symbol"!=n&&"boolean"!=n&&null!=t&&!ys(t))||Bt.test(t)||!vt.test(t)||null!=e&&t in qt(e)}function Ro(t){var e=mo(t),n=ir[e];if("function"!=typeof n||!(e in Ar.prototype))return!1;if(t===n)return!0;var r=Co(n);return!!r&&t===r[0]}(jn&&xo(new jn(new ArrayBuffer(1)))!=nt||Yn&&xo(new Yn)!=P||Pn&&"[object Promise]"!=xo(Pn.resolve())||Kn&&xo(new Kn)!=W||zn&&xo(new zn)!=tt)&&(xo=function(t){var e=Hr(t),n=e==J?t.constructor:o,r=n?Wo(n):"";if(r)switch(r){case Wn:return nt;case Xn:return P;case Zn:return"[object Promise]";case $n:return W;case tr:return tt}return e});var No=ee?ls:_A;function Fo(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||te)}function To(t){return t==t&&!gs(t)}function Uo(t,e){return function(n){return null!=n&&n[t]===e&&(e!==o||t in qt(n))}}function Oo(t,e,n){return e=Fn(e===o?t.length-1:e,0),function(){for(var i=arguments,o=-1,a=Fn(i.length-e,0),s=r(a);++o<a;)s[o]=i[e+o];o=-1;for(var A=r(e+1);++o<e;)A[o]=i[o];return A[e]=n(s),Fe(t,this,A)}}function Go(t,e){return e.length<2?t:Or(t,fi(e,0,-1))}function Ho(t,e){if("__proto__"!=e)return t[e]}var jo=zo(hi),Yo=_n||function(t,e){return be.setTimeout(t,e)},Po=zo(di);function Ko(t,e,n){var r=e+"";return Po(t,function(t,e){var n=e.length;if(!n)return t;var r=n-1;return e[r]=(n>1?"& ":"")+e[r],e=e.join(n>2?", ":" "),t.replace(Mt,"{\n/* [wrapped with "+e+"] */\n")}(r,function(t,e){return Ue(R,function(n){var r="_."+n[0];e&n[1]&&!He(t,r)&&t.push(r)}),t.sort()}(function(t){var e=r.match(kt);return e?e[1].split(St):[]}(),n)))}function zo(t){var e=0,n=0;return function(){var r=Un(),i=w-(r-n);if(n=r,i>0){if(++e>=Q)return arguments[0]}else e=0;return t.apply(o,arguments)}}function Jo(t,e){var n=-1,r=t.length,i=r-1;for(e=e===o?r:e;++n<e;){var a=li(n,i),s=t[a];t[a]=t[n],t[n]=s}return t.length=e,t}var qo=function(t){var e=za(function(t){var e=[];return 46===t.charCodeAt(0)&&e.push(""),t.replace(bt,function(t,n,r,i){e.push(r?i.replace(Rt,"$1"):n||t)}),e},function(t){return 500===n.size&&n.clear(),t}),n=e.cache;return e}();function Vo(t){if("string"==typeof t||ys(t))return t;var e=t+"";return"0"==e&&1/t==-x?"-0":e}function Wo(t){if(null!=t){try{return ne.call(t)}catch(t){}try{return t+""}catch(t){}}return""}function Xo(t){if(t instanceof Ar)return t.clone();var e=new sr(t.__wrapped__,t.__chain__);return e.__actions__=Hi(t.__actions__),e.__index__=t.__index__,e.__values__=t.__values__,e}var Zo=ui(function(t,e){return os(t)?xr(t,Lr(e,1,os,!0)):[]}),$o=ui(function(t,e){var n=Aa(e);return os(n)&&(n=o),os(t)?xr(t,Lr(e,1,os,!0),Eo(n,2)):[]}),ta=ui(function(t,e){var n=Aa(e);return os(n)&&(n=o),os(t)?xr(t,Lr(e,1,os,!0),o,n):[]});function ea(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=null==n?0:ws(n);return i<0&&(i=Fn(r+i,0)),We(t,Eo(e,3),i)}function na(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=r-1;return n!==o&&(i=ws(n),i=n<0?Fn(r+i,0):Tn(i,r-1)),We(t,Eo(e,3),i,!0)}function ra(t){return null!=t&&t.length?Lr(t,1):[]}function ia(t){return t&&t.length?t[0]:o}var oa=ui(function(t){var e=Ye(t,Di);return e.length&&e[0]===t[0]?Kr(e):[]}),aa=ui(function(t){var e=Aa(t),n=Ye(t,Di);return e===Aa(n)?e=o:n.pop(),n.length&&n[0]===t[0]?Kr(n,Eo(e,2)):[]}),sa=ui(function(t){var e=Aa(t),n=Ye(t,Di);return(e="function"==typeof e?e:o)&&n.pop(),n.length&&n[0]===t[0]?Kr(n,o,e):[]});function Aa(t){var e=null==t?0:t.length;return e?t[e-1]:o}var la=ui(ca);function ca(t,e){return t&&t.length&&e&&e.length?si(t,e):t}var ua=fo(function(t,e){var n=null==t?0:t.length,r=vr(t,e);return Ai(t,Ye(e,function(t){return ko(t,n)?+t:t}).sort(Ui)),r});function ga(t){return null==t?t:Hn.call(t)}var ha=ui(function(t){return vi(Lr(t,1,os,!0))}),da=ui(function(t){var e=Aa(t);return os(e)&&(e=o),vi(Lr(t,1,os,!0),Eo(e,2))}),fa=ui(function(t){var e=Aa(t);return e="function"==typeof e?e:o,vi(Lr(t,1,os,!0),o,e)});function pa(t){if(!t||!t.length)return[];var e=0;return t=Ge(t,function(t){if(os(t))return e=Fn(t.length,e),!0}),an(e,function(e){return Ye(t,en(e))})}function Ia(t,e){if(!t||!t.length)return[];var n=pa(t);return null==e?n:Ye(n,function(t){return Fe(e,o,t)})}var Ca=ui(function(t,e){return os(t)?xr(t,e):[]}),ma=ui(function(t){return xi(Ge(t,os))}),ya=ui(function(t){var e=Aa(t);return os(e)&&(e=o),xi(Ge(t,os),Eo(e,2))}),Ea=ui(function(t){var e=Aa(t);return e="function"==typeof e?e:o,xi(Ge(t,os),o,e)}),va=ui(pa),Ba=ui(function(t){var e=t.length,n=e>1?t[e-1]:o;return n="function"==typeof n?(t.pop(),n):o,Ia(t,n)});function ba(t){var e=ir(t);return e.__chain__=!0,e}function Qa(t,e){return e(t)}var wa=fo(function(t){var e=t.length,n=e?t[0]:0,r=this.__wrapped__,i=function(e){return vr(e,t)};return!(e>1||this.__actions__.length)&&r instanceof Ar&&ko(n)?((r=r.slice(n,+n+(e?1:0))).__actions__.push({func:Qa,args:[i],thisArg:o}),new sr(r,this.__chain__).thru(function(t){return e&&!t.length&&t.push(o),t})):this.thru(i)}),xa=Yi(function(t,e,n){re.call(t,n)?++t[n]:Er(t,n,1)}),_a=Wi(ea),Da=Wi(na);function Ma(t,e){return(ns(t)?Ue:_r)(t,Eo(e,3))}function ka(t,e){return(ns(t)?function(t,e){for(var n=null==t?0:t.length;n--&&!1!==e(t[n],n,t););return t}:Dr)(t,Eo(e,3))}var Sa=Yi(function(t,e,n){re.call(t,n)?t[n].push(e):Er(t,n,[e])}),La=ui(function(t,e,n){var i=-1,o="function"==typeof e,a=is(t)?r(t.length):[];return _r(t,function(t){a[++i]=o?Fe(e,t,n):zr(t,e,n)}),a}),Ra=Yi(function(t,e,n){Er(t,n,e)});function Na(t,e){return(ns(t)?Ye:ti)(t,Eo(e,3))}var Fa=Yi(function(t,e,n){t[n?0:1].push(e)},function(){return[[],[]]}),Ta=ui(function(t,e){if(null==t)return[];var n=e.length;return n>1&&So(t,e[0],e[1])?e=[]:n>2&&So(e[0],e[1],e[2])&&(e=[e[0]]),oi(t,Lr(e,1),[])}),Ua=xn||function(){return be.Date.now()};function Oa(t,e,n){return e=n?o:e,e=t&&null==e?t.length:e,lo(t,v,o,o,o,o,e)}function Ga(t,e){var n;if("function"!=typeof e)throw new Xt(s);return t=ws(t),function(){return--t>0&&(n=e.apply(this,arguments)),t<=1&&(e=o),n}}var Ha=ui(function(t,e,n){var r=f;if(n.length){var i=Cn(n,yo(Ha));r|=y}return lo(t,r,e,n,i)}),ja=ui(function(t,e,n){var r=f|p;if(n.length){var i=Cn(n,yo(ja));r|=y}return lo(e,r,t,n,i)});function Ya(t,e,n){var r,i,a,A,l,c,u=0,g=!1,h=!1,d=!0;if("function"!=typeof t)throw new Xt(s);function f(e){var n=r,a=i;return r=i=o,u=e,A=t.apply(a,n)}function p(t){var n=t-c;return c===o||n>=e||n<0||h&&t-u>=a}function I(){var t=Ua();if(p(t))return C(t);l=Yo(I,function(t){var n=e-(t-c);return h?Tn(n,a-(t-u)):n}(t))}function C(t){return l=o,d&&r?f(t):(r=i=o,A)}function m(){var t=Ua(),n=p(t);if(r=arguments,i=this,c=t,n){if(l===o)return function(t){return u=t,l=Yo(I,e),g?f(t):A}(c);if(h)return l=Yo(I,e),f(c)}return l===o&&(l=Yo(I,e)),A}return e=_s(e)||0,gs(n)&&(g=!!n.leading,a=(h="maxWait"in n)?Fn(_s(n.maxWait)||0,e):a,d="trailing"in n?!!n.trailing:d),m.cancel=function(){l!==o&&Ri(l),u=0,r=c=i=l=o},m.flush=function(){return l===o?A:C(Ua())},m}var Pa=ui(function(t,e){return wr(t,1,e)}),Ka=ui(function(t,e,n){return wr(t,_s(e)||0,n)});function za(t,e){if("function"!=typeof t||null!=e&&"function"!=typeof e)throw new Xt(s);var n=function(){var r=arguments,i=e?e.apply(this,r):r[0],o=n.cache;if(o.has(i))return o.get(i);var a=t.apply(this,r);return n.cache=o.set(i,a)||o,a};return n.cache=new(za.Cache||ur),n}function Ja(t){if("function"!=typeof t)throw new Xt(s);return function(){var e=arguments;switch(e.length){case 0:return!t.call(this);case 1:return!t.call(this,e[0]);case 2:return!t.call(this,e[0],e[1]);case 3:return!t.call(this,e[0],e[1],e[2])}return!t.apply(this,e)}}za.Cache=ur;var qa=Si(function(t,e){var n=(e=1==e.length&&ns(e[0])?Ye(e[0],sn(Eo())):Ye(Lr(e,1),sn(Eo()))).length;return ui(function(r){for(var i=-1,o=Tn(r.length,n);++i<o;)r[i]=e[i].call(this,r[i]);return Fe(t,this,r)})}),Va=ui(function(t,e){var n=Cn(e,yo(Va));return lo(t,y,o,e,n)}),Wa=ui(function(t,e){var n=Cn(e,yo(Wa));return lo(t,E,o,e,n)}),Xa=fo(function(t,e){return lo(t,B,o,o,o,e)});function Za(t,e){return t===e||t!=t&&e!=e}var $a=io(jr),ts=io(function(t,e){return t>=e}),es=Jr(function(){return arguments}())?Jr:function(t){return hs(t)&&re.call(t,"callee")&&!_e.call(t,"callee")},ns=r.isArray,rs=Me?sn(Me):function(t){return hs(t)&&Hr(t)==et};function is(t){return null!=t&&us(t.length)&&!ls(t)}function os(t){return hs(t)&&is(t)}var as=Sn||_A,ss=ke?sn(ke):function(t){return hs(t)&&Hr(t)==O};function As(t){if(!hs(t))return!1;var e=Hr(t);return e==H||e==G||"string"==typeof t.message&&"string"==typeof t.name&&!ps(t)}function ls(t){if(!gs(t))return!1;var e=Hr(t);return e==j||e==Y||e==T||e==q}function cs(t){return"number"==typeof t&&t==ws(t)}function us(t){return"number"==typeof t&&t>-1&&t%1==0&&t<=_}function gs(t){var e=typeof t;return null!=t&&("object"==e||"function"==e)}function hs(t){return null!=t&&"object"==typeof t}var ds=Se?sn(Se):function(t){return hs(t)&&xo(t)==P};function fs(t){return"number"==typeof t||hs(t)&&Hr(t)==K}function ps(t){if(!hs(t)||Hr(t)!=J)return!1;var e=Qe(t);if(null===e)return!0;var n=re.call(e,"constructor")&&e.constructor;return"function"==typeof n&&n instanceof n&&ne.call(n)==se}var Is=Le?sn(Le):function(t){return hs(t)&&Hr(t)==V},Cs=Re?sn(Re):function(t){return hs(t)&&xo(t)==W};function ms(t){return"string"==typeof t||!ns(t)&&hs(t)&&Hr(t)==X}function ys(t){return"symbol"==typeof t||hs(t)&&Hr(t)==Z}var Es=Ne?sn(Ne):function(t){return hs(t)&&us(t.length)&&!!Ie[Hr(t)]},vs=io($r),Bs=io(function(t,e){return t<=e});function bs(t){if(!t)return[];if(is(t))return ms(t)?En(t):Hi(t);if(nn&&t[nn])return function(t){for(var e,n=[];!(e=t.next()).done;)n.push(e.value);return n}(t[nn]());var e=xo(t);return(e==P?pn:e==W?mn:Zs)(t)}function Qs(t){return t?(t=_s(t))===x||t===-x?(t<0?-1:1)*D:t==t?t:0:0===t?t:0}function ws(t){var e=Qs(t),n=e%1;return e==e?n?e-n:e:0}function xs(t){return t?Br(ws(t),0,k):0}function _s(t){if("number"==typeof t)return t;if(ys(t))return M;if(gs(t)){var e="function"==typeof t.valueOf?t.valueOf():t;t=gs(e)?e+"":e}if("string"!=typeof t)return 0===t?t:+t;t=t.replace(xt,"");var n=Ut.test(t);return n||Gt.test(t)?Ee(t.slice(2),n?2:8):Tt.test(t)?M:+t}function Ds(t){return ji(t,Ps(t))}function Ms(t){return null==t?"":Ei(t)}var ks=Pi(function(t,e){if(Fo(e)||is(e))ji(e,Ys(e),t);else for(var n in e)re.call(e,n)&&Ir(t,n,e[n])}),Ss=Pi(function(t,e){ji(e,Ps(e),t)}),Ls=Pi(function(t,e,n,r){ji(e,Ps(e),t,r)}),Rs=Pi(function(t,e,n,r){ji(e,Ys(e),t,r)}),Ns=fo(vr),Fs=ui(function(t,e){t=qt(t);var n=-1,r=e.length,i=r>2?e[2]:o;for(i&&So(e[0],e[1],i)&&(r=1);++n<r;)for(var a=e[n],s=Ps(a),A=-1,l=s.length;++A<l;){var c=s[A],u=t[c];(u===o||Za(u,te[c])&&!re.call(t,c))&&(t[c]=a[c])}return t}),Ts=ui(function(t){return t.push(o,uo),Fe(zs,o,t)});function Us(t,e,n){var r=null==t?o:Or(t,e);return r===o?n:r}function Os(t,e){return null!=t&&_o(t,e,Pr)}var Gs=$i(function(t,e,n){null!=e&&"function"!=typeof e.toString&&(e=ae.call(e)),t[e]=n},gA(fA)),Hs=$i(function(t,e,n){null!=e&&"function"!=typeof e.toString&&(e=ae.call(e)),re.call(t,e)?t[e].push(n):t[e]=[n]},Eo),js=ui(zr);function Ys(t){return is(t)?dr(t):Zr(t)}function Ps(t){return is(t)?dr(t,!0):function(t){if(!gs(t))return function(t){var e=[];if(null!=t)for(var n in qt(t))e.push(n);return e}(t);var e=Fo(t),n=[];for(var r in t)("constructor"!=r||!e&&re.call(t,r))&&n.push(r);return n}(t)}var Ks=Pi(function(t,e,n){ri(t,e,n)}),zs=Pi(function(t,e,n,r){ri(t,e,n,r)}),Js=fo(function(t,e){var n={};if(null==t)return n;var r=!1;e=Ye(e,function(e){return e=ki(e,t),r||(r=e.length>1),e}),ji(t,Io(t),n),r&&(n=br(n,c|u|g,go));for(var i=e.length;i--;)Bi(n,e[i]);return n}),qs=fo(function(t,e){return null==t?{}:function(t,e){return ai(t,e,function(e,n){return Os(t,n)})}(t,e)});function Vs(t,e){if(null==t)return{};var n=Ye(Io(t),function(t){return[t]});return e=Eo(e),ai(t,n,function(t,n){return e(t,n[0])})}var Ws=Ao(Ys),Xs=Ao(Ps);function Zs(t){return null==t?[]:An(t,Ys(t))}var $s=qi(function(t,e,n){return e=e.toLowerCase(),t+(n?tA(e):e)});function tA(t){return AA(Ms(t).toLowerCase())}function eA(t){return(t=Ms(t))&&t.replace(jt,gn).replace(ce,"")}var nA=qi(function(t,e,n){return t+(n?"-":"")+e.toLowerCase()}),rA=qi(function(t,e,n){return t+(n?" ":"")+e.toLowerCase()}),iA=Ji("toLowerCase"),oA=qi(function(t,e,n){return t+(n?"_":"")+e.toLowerCase()}),aA=qi(function(t,e,n){return t+(n?" ":"")+AA(e)}),sA=qi(function(t,e,n){return t+(n?" ":"")+e.toUpperCase()}),AA=Ji("toUpperCase");function lA(t,e,n){return t=Ms(t),(e=n?o:e)===o?function(t){return de.test(t)}(t)?function(t){return t.match(ge)||[]}(t):function(t){return t.match(Lt)||[]}(t):t.match(e)||[]}var cA=ui(function(t,e){try{return Fe(t,o,e)}catch(t){return As(t)?t:new Kt(t)}}),uA=fo(function(t,e){return Ue(e,function(e){e=Vo(e),Er(t,e,Ha(t[e],t))}),t});function gA(t){return function(){return t}}var hA=Xi(),dA=Xi(!0);function fA(t){return t}function pA(t){return Xr("function"==typeof t?t:br(t,c))}var IA=ui(function(t,e){return function(n){return zr(n,t,e)}}),CA=ui(function(t,e){return function(n){return zr(t,n,e)}});function mA(t,e,n){var r=Ys(e),i=Ur(e,r);null!=n||gs(e)&&(i.length||!r.length)||(n=e,e=t,t=this,i=Ur(e,Ys(e)));var o=!(gs(n)&&"chain"in n&&!n.chain),a=ls(t);return Ue(i,function(n){var r=e[n];t[n]=r,a&&(t.prototype[n]=function(){var e=this.__chain__;if(o||e){var n=t(this.__wrapped__),i=n.__actions__=Hi(this.__actions__);return i.push({func:r,args:arguments,thisArg:t}),n.__chain__=e,n}return r.apply(t,Pe([this.value()],arguments))})}),t}function yA(){}var EA=eo(Ye),vA=eo(Oe),BA=eo(Je);function bA(t){return Lo(t)?en(Vo(t)):function(t){return function(e){return Or(e,t)}}(t)}var QA=ro(),wA=ro(!0);function xA(){return[]}function _A(){return!1}var DA,MA=to(function(t,e){return t+e},0),kA=ao("ceil"),SA=to(function(t,e){return t/e},1),LA=ao("floor"),RA=to(function(t,e){return t*e},1),NA=ao("round"),FA=to(function(t,e){return t-e},0);return ir.after=function(t,e){if("function"!=typeof e)throw new Xt(s);return t=ws(t),function(){if(--t<1)return e.apply(this,arguments)}},ir.ary=Oa,ir.assign=ks,ir.assignIn=Ss,ir.assignInWith=Ls,ir.assignWith=Rs,ir.at=Ns,ir.before=Ga,ir.bind=Ha,ir.bindAll=uA,ir.bindKey=ja,ir.castArray=function(){if(!arguments.length)return[];var t=arguments[0];return ns(t)?t:[t]},ir.chain=ba,ir.chunk=function(t,e,n){e=(n?So(t,e,n):e===o)?1:Fn(ws(e),0);var i=null==t?0:t.length;if(!i||e<1)return[];for(var a=0,s=0,A=r(Dn(i/e));a<i;)A[s++]=fi(t,a,a+=e);return A},ir.compact=function(t){for(var e=-1,n=null==t?0:t.length,r=0,i=[];++e<n;){var o=t[e];o&&(i[r++]=o)}return i},ir.concat=function(){var t=arguments.length;if(!t)return[];for(var e=r(t-1),n=arguments[0],i=t;i--;)e[i-1]=arguments[i];return Pe(ns(n)?Hi(n):[n],Lr(e,1))},ir.cond=function(t){var e=null==t?0:t.length,n=Eo();return t=e?Ye(t,function(t){if("function"!=typeof t[1])throw new Xt(s);return[n(t[0]),t[1]]}):[],ui(function(n){for(var r=-1;++r<e;){var i=t[r];if(Fe(i[0],this,n))return Fe(i[1],this,n)}})},ir.conforms=function(t){return function(t){var e=Ys(t);return function(n){return Qr(n,t,e)}}(br(t,c))},ir.constant=gA,ir.countBy=xa,ir.create=function(t,e){var n=or(t);return null==e?n:yr(n,e)},ir.curry=function t(e,n,r){var i=lo(e,C,o,o,o,o,o,n=r?o:n);return i.placeholder=t.placeholder,i},ir.curryRight=function t(e,n,r){var i=lo(e,m,o,o,o,o,o,n=r?o:n);return i.placeholder=t.placeholder,i},ir.debounce=Ya,ir.defaults=Fs,ir.defaultsDeep=Ts,ir.defer=Pa,ir.delay=Ka,ir.difference=Zo,ir.differenceBy=$o,ir.differenceWith=ta,ir.drop=function(t,e,n){var r=null==t?0:t.length;return r?fi(t,(e=n||e===o?1:ws(e))<0?0:e,r):[]},ir.dropRight=function(t,e,n){var r=null==t?0:t.length;return r?fi(t,0,(e=r-(e=n||e===o?1:ws(e)))<0?0:e):[]},ir.dropRightWhile=function(t,e){return t&&t.length?Qi(t,Eo(e,3),!0,!0):[]},ir.dropWhile=function(t,e){return t&&t.length?Qi(t,Eo(e,3),!0):[]},ir.fill=function(t,e,n,r){var i=null==t?0:t.length;return i?(n&&"number"!=typeof n&&So(t,e,n)&&(n=0,r=i),function(t,e,n,r){var i=t.length;for((n=ws(n))<0&&(n=-n>i?0:i+n),(r=r===o||r>i?i:ws(r))<0&&(r+=i),r=n>r?0:xs(r);n<r;)t[n++]=e;return t}(t,e,n,r)):[]},ir.filter=function(t,e){return(ns(t)?Ge:Sr)(t,Eo(e,3))},ir.flatMap=function(t,e){return Lr(Na(t,e),1)},ir.flatMapDeep=function(t,e){return Lr(Na(t,e),x)},ir.flatMapDepth=function(t,e,n){return n=n===o?1:ws(n),Lr(Na(t,e),n)},ir.flatten=ra,ir.flattenDeep=function(t){return null!=t&&t.length?Lr(t,x):[]},ir.flattenDepth=function(t,e){return null!=t&&t.length?Lr(t,e=e===o?1:ws(e)):[]},ir.flip=function(t){return lo(t,b)},ir.flow=hA,ir.flowRight=dA,ir.fromPairs=function(t){for(var e=-1,n=null==t?0:t.length,r={};++e<n;){var i=t[e];r[i[0]]=i[1]}return r},ir.functions=function(t){return null==t?[]:Ur(t,Ys(t))},ir.functionsIn=function(t){return null==t?[]:Ur(t,Ps(t))},ir.groupBy=Sa,ir.initial=function(t){return null!=t&&t.length?fi(t,0,-1):[]},ir.intersection=oa,ir.intersectionBy=aa,ir.intersectionWith=sa,ir.invert=Gs,ir.invertBy=Hs,ir.invokeMap=La,ir.iteratee=pA,ir.keyBy=Ra,ir.keys=Ys,ir.keysIn=Ps,ir.map=Na,ir.mapKeys=function(t,e){var n={};return e=Eo(e,3),Fr(t,function(t,r,i){Er(n,e(t,r,i),t)}),n},ir.mapValues=function(t,e){var n={};return e=Eo(e,3),Fr(t,function(t,r,i){Er(n,r,e(t,r,i))}),n},ir.matches=function(t){return ei(br(t,c))},ir.matchesProperty=function(t,e){return ni(t,br(e,c))},ir.memoize=za,ir.merge=Ks,ir.mergeWith=zs,ir.method=IA,ir.methodOf=CA,ir.mixin=mA,ir.negate=Ja,ir.nthArg=function(t){return t=ws(t),ui(function(e){return ii(e,t)})},ir.omit=Js,ir.omitBy=function(t,e){return Vs(t,Ja(Eo(e)))},ir.once=function(t){return Ga(2,t)},ir.orderBy=function(t,e,n,r){return null==t?[]:(ns(e)||(e=null==e?[]:[e]),ns(n=r?o:n)||(n=null==n?[]:[n]),oi(t,e,n))},ir.over=EA,ir.overArgs=qa,ir.overEvery=vA,ir.overSome=BA,ir.partial=Va,ir.partialRight=Wa,ir.partition=Fa,ir.pick=qs,ir.pickBy=Vs,ir.property=bA,ir.propertyOf=function(t){return function(e){return null==t?o:Or(t,e)}},ir.pull=la,ir.pullAll=ca,ir.pullAllBy=function(t,e,n){return t&&t.length&&e&&e.length?si(t,e,Eo(n,2)):t},ir.pullAllWith=function(t,e,n){return t&&t.length&&e&&e.length?si(t,e,o,n):t},ir.pullAt=ua,ir.range=QA,ir.rangeRight=wA,ir.rearg=Xa,ir.reject=function(t,e){return(ns(t)?Ge:Sr)(t,Ja(Eo(e,3)))},ir.remove=function(t,e){var n=[];if(!t||!t.length)return n;var r=-1,i=[],o=t.length;for(e=Eo(e,3);++r<o;){var a=t[r];e(a,r,t)&&(n.push(a),i.push(r))}return Ai(t,i),n},ir.rest=function(t,e){if("function"!=typeof t)throw new Xt(s);return ui(t,e=e===o?e:ws(e))},ir.reverse=ga,ir.sampleSize=function(t,e,n){return e=(n?So(t,e,n):e===o)?1:ws(e),(ns(t)?function(t,e){return Jo(Hi(t),Br(e,0,t.length))}:function(t,e){var n=Zs(t);return Jo(n,Br(e,0,n.length))})(t,e)},ir.set=function(t,e,n){return null==t?t:gi(t,e,n)},ir.setWith=function(t,e,n,r){return r="function"==typeof r?r:o,null==t?t:gi(t,e,n,r)},ir.shuffle=function(t){return(ns(t)?function(t){return Jo(Hi(t))}:function(t){return Jo(Zs(t))})(t)},ir.slice=function(t,e,n){var r=null==t?0:t.length;return r?(n&&"number"!=typeof n&&So(t,e,n)?(e=0,n=r):(e=null==e?0:ws(e),n=n===o?r:ws(n)),fi(t,e,n)):[]},ir.sortBy=Ta,ir.sortedUniq=function(t){return t&&t.length?mi(t):[]},ir.sortedUniqBy=function(t,e){return t&&t.length?mi(t,Eo(e,2)):[]},ir.split=function(t,e,n){return n&&"number"!=typeof n&&So(t,e,n)&&(e=n=o),(n=n===o?k:n>>>0)?(t=Ms(t))&&("string"==typeof e||null!=e&&!Is(e))&&!(e=Ei(e))&&fn(t)?Li(En(t),0,n):t.split(e,n):[]},ir.spread=function(t,e){if("function"!=typeof t)throw new Xt(s);return e=null==e?0:Fn(ws(e),0),ui(function(n){var r=n[e],i=Li(n,0,e);return r&&Pe(i,r),Fe(t,this,i)})},ir.tail=function(t){var e=null==t?0:t.length;return e?fi(t,1,e):[]},ir.take=function(t,e,n){return t&&t.length?fi(t,0,(e=n||e===o?1:ws(e))<0?0:e):[]},ir.takeRight=function(t,e,n){var r=null==t?0:t.length;return r?fi(t,(e=r-(e=n||e===o?1:ws(e)))<0?0:e,r):[]},ir.takeRightWhile=function(t,e){return t&&t.length?Qi(t,Eo(e,3),!1,!0):[]},ir.takeWhile=function(t,e){return t&&t.length?Qi(t,Eo(e,3)):[]},ir.tap=function(t,e){return e(t),t},ir.throttle=function(t,e,n){var r=!0,i=!0;if("function"!=typeof t)throw new Xt(s);return gs(n)&&(r="leading"in n?!!n.leading:r,i="trailing"in n?!!n.trailing:i),Ya(t,e,{leading:r,maxWait:e,trailing:i})},ir.thru=Qa,ir.toArray=bs,ir.toPairs=Ws,ir.toPairsIn=Xs,ir.toPath=function(t){return ns(t)?Ye(t,Vo):ys(t)?[t]:Hi(qo(Ms(t)))},ir.toPlainObject=Ds,ir.transform=function(t,e,n){var r=ns(t),i=r||as(t)||Es(t);if(e=Eo(e,4),null==n){var o=t&&t.constructor;n=i?r?new o:[]:gs(t)&&ls(o)?or(Qe(t)):{}}return(i?Ue:Fr)(t,function(t,r,i){return e(n,t,r,i)}),n},ir.unary=function(t){return Oa(t,1)},ir.union=ha,ir.unionBy=da,ir.unionWith=fa,ir.uniq=function(t){return t&&t.length?vi(t):[]},ir.uniqBy=function(t,e){return t&&t.length?vi(t,Eo(e,2)):[]},ir.uniqWith=function(t,e){return e="function"==typeof e?e:o,t&&t.length?vi(t,o,e):[]},ir.unset=function(t,e){return null==t||Bi(t,e)},ir.unzip=pa,ir.unzipWith=Ia,ir.update=function(t,e,n){return null==t?t:bi(t,e,Mi(n))},ir.updateWith=function(t,e,n,r){return r="function"==typeof r?r:o,null==t?t:bi(t,e,Mi(n),r)},ir.values=Zs,ir.valuesIn=function(t){return null==t?[]:An(t,Ps(t))},ir.without=Ca,ir.words=lA,ir.wrap=function(t,e){return Va(Mi(e),t)},ir.xor=ma,ir.xorBy=ya,ir.xorWith=Ea,ir.zip=va,ir.zipObject=function(t,e){return _i(t||[],e||[],Ir)},ir.zipObjectDeep=function(t,e){return _i(t||[],e||[],gi)},ir.zipWith=Ba,ir.entries=Ws,ir.entriesIn=Xs,ir.extend=Ss,ir.extendWith=Ls,mA(ir,ir),ir.add=MA,ir.attempt=cA,ir.camelCase=$s,ir.capitalize=tA,ir.ceil=kA,ir.clamp=function(t,e,n){return n===o&&(n=e,e=o),n!==o&&(n=(n=_s(n))==n?n:0),e!==o&&(e=(e=_s(e))==e?e:0),Br(_s(t),e,n)},ir.clone=function(t){return br(t,g)},ir.cloneDeep=function(t){return br(t,c|g)},ir.cloneDeepWith=function(t,e){return br(t,c|g,e="function"==typeof e?e:o)},ir.cloneWith=function(t,e){return br(t,g,e="function"==typeof e?e:o)},ir.conformsTo=function(t,e){return null==e||Qr(t,e,Ys(e))},ir.deburr=eA,ir.defaultTo=function(t,e){return null==t||t!=t?e:t},ir.divide=SA,ir.endsWith=function(t,e,n){t=Ms(t),e=Ei(e);var r=t.length,i=n=n===o?r:Br(ws(n),0,r);return(n-=e.length)>=0&&t.slice(n,i)==e},ir.eq=Za,ir.escape=function(t){return(t=Ms(t))&&Ct.test(t)?t.replace(pt,hn):t},ir.escapeRegExp=function(t){return(t=Ms(t))&&wt.test(t)?t.replace(Qt,"\\$&"):t},ir.every=function(t,e,n){var r=ns(t)?Oe:Mr;return n&&So(t,e,n)&&(e=o),r(t,Eo(e,3))},ir.find=_a,ir.findIndex=ea,ir.findKey=function(t,e){return Ve(t,Eo(e,3),Fr)},ir.findLast=Da,ir.findLastIndex=na,ir.findLastKey=function(t,e){return Ve(t,Eo(e,3),Tr)},ir.floor=LA,ir.forEach=Ma,ir.forEachRight=ka,ir.forIn=function(t,e){return null==t?t:Rr(t,Eo(e,3),Ps)},ir.forInRight=function(t,e){return null==t?t:Nr(t,Eo(e,3),Ps)},ir.forOwn=function(t,e){return t&&Fr(t,Eo(e,3))},ir.forOwnRight=function(t,e){return t&&Tr(t,Eo(e,3))},ir.get=Us,ir.gt=$a,ir.gte=ts,ir.has=function(t,e){return null!=t&&_o(t,e,Yr)},ir.hasIn=Os,ir.head=ia,ir.identity=fA,ir.includes=function(t,e,n,r){t=is(t)?t:Zs(t),n=n&&!r?ws(n):0;var i=t.length;return n<0&&(n=Fn(i+n,0)),ms(t)?n<=i&&t.indexOf(e,n)>-1:!!i&&Xe(t,e,n)>-1},ir.indexOf=function(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=null==n?0:ws(n);return i<0&&(i=Fn(r+i,0)),Xe(t,e,i)},ir.inRange=function(t,e,n){return e=Qs(e),n===o?(n=e,e=0):n=Qs(n),function(t,e,n){return t>=Tn(e,n)&&t<Fn(e,n)}(t=_s(t),e,n)},ir.invoke=js,ir.isArguments=es,ir.isArray=ns,ir.isArrayBuffer=rs,ir.isArrayLike=is,ir.isArrayLikeObject=os,ir.isBoolean=function(t){return!0===t||!1===t||hs(t)&&Hr(t)==U},ir.isBuffer=as,ir.isDate=ss,ir.isElement=function(t){return hs(t)&&1===t.nodeType&&!ps(t)},ir.isEmpty=function(t){if(null==t)return!0;if(is(t)&&(ns(t)||"string"==typeof t||"function"==typeof t.splice||as(t)||Es(t)||es(t)))return!t.length;var e=xo(t);if(e==P||e==W)return!t.size;if(Fo(t))return!Zr(t).length;for(var n in t)if(re.call(t,n))return!1;return!0},ir.isEqual=function(t,e){return qr(t,e)},ir.isEqualWith=function(t,e,n){var r=(n="function"==typeof n?n:o)?n(t,e):o;return r===o?qr(t,e,o,n):!!r},ir.isError=As,ir.isFinite=function(t){return"number"==typeof t&&Ln(t)},ir.isFunction=ls,ir.isInteger=cs,ir.isLength=us,ir.isMap=ds,ir.isMatch=function(t,e){return t===e||Vr(t,e,Bo(e))},ir.isMatchWith=function(t,e,n){return n="function"==typeof n?n:o,Vr(t,e,Bo(e),n)},ir.isNaN=function(t){return fs(t)&&t!=+t},ir.isNative=function(t){if(No(t))throw new Kt("Unsupported core-js use. Try https://npms.io/search?q=ponyfill.");return Wr(t)},ir.isNil=function(t){return null==t},ir.isNull=function(t){return null===t},ir.isNumber=fs,ir.isObject=gs,ir.isObjectLike=hs,ir.isPlainObject=ps,ir.isRegExp=Is,ir.isSafeInteger=function(t){return cs(t)&&t>=-_&&t<=_},ir.isSet=Cs,ir.isString=ms,ir.isSymbol=ys,ir.isTypedArray=Es,ir.isUndefined=function(t){return t===o},ir.isWeakMap=function(t){return hs(t)&&xo(t)==tt},ir.isWeakSet=function(t){return hs(t)&&"[object WeakSet]"==Hr(t)},ir.join=function(t,e){return null==t?"":Rn.call(t,e)},ir.kebabCase=nA,ir.last=Aa,ir.lastIndexOf=function(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=r;return n!==o&&(i=(i=ws(n))<0?Fn(r+i,0):Tn(i,r-1)),e==e?function(t,e,n){for(var r=i+1;r--;)if(t[r]===e)return r;return r}(t,e):We(t,$e,i,!0)},ir.lowerCase=rA,ir.lowerFirst=iA,ir.lt=vs,ir.lte=Bs,ir.max=function(t){return t&&t.length?kr(t,fA,jr):o},ir.maxBy=function(t,e){return t&&t.length?kr(t,Eo(e,2),jr):o},ir.mean=function(t){return tn(t,fA)},ir.meanBy=function(t,e){return tn(t,Eo(e,2))},ir.min=function(t){return t&&t.length?kr(t,fA,$r):o},ir.minBy=function(t,e){return t&&t.length?kr(t,Eo(e,2),$r):o},ir.stubArray=xA,ir.stubFalse=_A,ir.stubObject=function(){return{}},ir.stubString=function(){return""},ir.stubTrue=function(){return!0},ir.multiply=RA,ir.nth=function(t,e){return t&&t.length?ii(t,ws(e)):o},ir.noConflict=function(){return be._===this&&(be._=Ae),this},ir.noop=yA,ir.now=Ua,ir.pad=function(t,e,n){t=Ms(t);var r=(e=ws(e))?yn(t):0;if(!e||r>=e)return t;var i=(e-r)/2;return no(Mn(i),n)+t+no(Dn(i),n)},ir.padEnd=function(t,e,n){t=Ms(t);var r=(e=ws(e))?yn(t):0;return e&&r<e?t+no(e-r,n):t},ir.padStart=function(t,e,n){t=Ms(t);var r=(e=ws(e))?yn(t):0;return e&&r<e?no(e-r,n)+t:t},ir.parseInt=function(t,e,n){return n||null==e?e=0:e&&(e=+e),On(Ms(t).replace(_t,""),e||0)},ir.random=function(t,e,n){if(n&&"boolean"!=typeof n&&So(t,e,n)&&(e=n=o),n===o&&("boolean"==typeof e?(n=e,e=o):"boolean"==typeof t&&(n=t,t=o)),t===o&&e===o?(t=0,e=1):(t=Qs(t),e===o?(e=t,t=0):e=Qs(e)),t>e){var r=t;t=e,e=r}if(n||t%1||e%1){var i=Gn();return Tn(t+i*(e-t+ye("1e-"+((i+"").length-1))),e)}return li(t,e)},ir.reduce=function(t,e,n){var r=ns(t)?Ke:rn,i=arguments.length<3;return r(t,Eo(e,4),n,i,_r)},ir.reduceRight=function(t,e,n){var r=ns(t)?ze:rn,i=arguments.length<3;return r(t,Eo(e,4),n,i,Dr)},ir.repeat=function(t,e,n){return e=(n?So(t,e,n):e===o)?1:ws(e),ci(Ms(t),e)},ir.replace=function(){var t=arguments,e=Ms(t[0]);return t.length<3?e:e.replace(t[1],t[2])},ir.result=function(t,e,n){var r=-1,i=(e=ki(e,t)).length;for(i||(i=1,t=o);++r<i;){var a=null==t?o:t[Vo(e[r])];a===o&&(r=i,a=n),t=ls(a)?a.call(t):a}return t},ir.round=NA,ir.runInContext=t,ir.sample=function(t){return(ns(t)?fr:function(t){return fr(Zs(t))})(t)},ir.size=function(t){if(null==t)return 0;if(is(t))return ms(t)?yn(t):t.length;var e=xo(t);return e==P||e==W?t.size:Zr(t).length},ir.snakeCase=oA,ir.some=function(t,e,n){var r=ns(t)?Je:pi;return n&&So(t,e,n)&&(e=o),r(t,Eo(e,3))},ir.sortedIndex=function(t,e){return Ii(t,e)},ir.sortedIndexBy=function(t,e,n){return Ci(t,e,Eo(n,2))},ir.sortedIndexOf=function(t,e){var n=null==t?0:t.length;if(n){var r=Ii(t,e);if(r<n&&Za(t[r],e))return r}return-1},ir.sortedLastIndex=function(t,e){return Ii(t,e,!0)},ir.sortedLastIndexBy=function(t,e,n){return Ci(t,e,Eo(n,2),!0)},ir.sortedLastIndexOf=function(t,e){if(null!=t&&t.length){var n=Ii(t,e,!0)-1;if(Za(t[n],e))return n}return-1},ir.startCase=aA,ir.startsWith=function(t,e,n){return t=Ms(t),n=null==n?0:Br(ws(n),0,t.length),e=Ei(e),t.slice(n,n+e.length)==e},ir.subtract=FA,ir.sum=function(t){return t&&t.length?on(t,fA):0},ir.sumBy=function(t,e){return t&&t.length?on(t,Eo(e,2)):0},ir.template=function(t,e,n){var r=ir.templateSettings;n&&So(t,e,n)&&(e=o),t=Ms(t),e=Ls({},e,r,co);var i,a,s=Ls({},e.imports,r.imports,co),A=Ys(s),l=An(s,A),c=0,u=e.interpolate||Yt,g="__p += '",h=Vt((e.escape||Yt).source+"|"+u.source+"|"+(u===Et?Nt:Yt).source+"|"+(e.evaluate||Yt).source+"|$","g"),d="//# sourceURL="+("sourceURL"in e?e.sourceURL:"lodash.templateSources["+ ++pe+"]")+"\n";t.replace(h,function(e,n,r,o,s,A){return r||(r=o),g+=t.slice(c,A).replace(Pt,dn),n&&(i=!0,g+="' +\n__e("+n+") +\n'"),s&&(a=!0,g+="';\n"+s+";\n__p += '"),r&&(g+="' +\n((__t = ("+r+")) == null ? '' : __t) +\n'"),c=A+e.length,e}),g+="';\n";var f=e.variable;f||(g="with (obj) {\n"+g+"\n}\n"),g=(a?g.replace(gt,""):g).replace(ht,"$1").replace(dt,"$1;"),g="function("+(f||"obj")+") {\n"+(f?"":"obj || (obj = {});\n")+"var __t, __p = ''"+(i?", __e = _.escape":"")+(a?", __j = Array.prototype.join;\nfunction print() { __p += __j.call(arguments, '') }\n":";\n")+g+"return __p\n}";var p=cA(function(){return zt(A,d+"return "+g).apply(o,l)});if(p.source=g,As(p))throw p;return p},ir.times=function(t,e){if((t=ws(t))<1||t>_)return[];var n=k,r=Tn(t,k);e=Eo(e),t-=k;for(var i=an(r,e);++n<t;)e(n);return i},ir.toFinite=Qs,ir.toInteger=ws,ir.toLength=xs,ir.toLower=function(t){return Ms(t).toLowerCase()},ir.toNumber=_s,ir.toSafeInteger=function(t){return t?Br(ws(t),-_,_):0===t?t:0},ir.toString=Ms,ir.toUpper=function(t){return Ms(t).toUpperCase()},ir.trim=function(t,e,n){if((t=Ms(t))&&(n||e===o))return t.replace(xt,"");if(!t||!(e=Ei(e)))return t;var r=En(t),i=En(e);return Li(r,cn(r,i),un(r,i)+1).join("")},ir.trimEnd=function(t,e,n){if((t=Ms(t))&&(n||e===o))return t.replace(Dt,"");if(!t||!(e=Ei(e)))return t;var r=En(t);return Li(r,0,un(r,En(e))+1).join("")},ir.trimStart=function(t,e,n){if((t=Ms(t))&&(n||e===o))return t.replace(_t,"");if(!t||!(e=Ei(e)))return t;var r=En(t);return Li(r,cn(r,En(e))).join("")},ir.truncate=function(t,e){var n=30,r="...";if(gs(e)){var i="separator"in e?e.separator:i;n="length"in e?ws(e.length):n,r="omission"in e?Ei(e.omission):r}var a=(t=Ms(t)).length;if(fn(t)){var s=En(t);a=s.length}if(n>=a)return t;var A=n-yn(r);if(A<1)return r;var l=s?Li(s,0,A).join(""):t.slice(0,A);if(i===o)return l+r;if(s&&(A+=l.length-A),Is(i)){if(t.slice(A).search(i)){var c,u=l;for(i.global||(i=Vt(i.source,Ms(Ft.exec(i))+"g")),i.lastIndex=0;c=i.exec(u);)var g=c.index;l=l.slice(0,g===o?A:g)}}else if(t.indexOf(Ei(i),A)!=A){var h=l.lastIndexOf(i);h>-1&&(l=l.slice(0,h))}return l+r},ir.unescape=function(t){return(t=Ms(t))&&It.test(t)?t.replace(ft,vn):t},ir.uniqueId=function(t){var e=++ie;return Ms(t)+e},ir.upperCase=sA,ir.upperFirst=AA,ir.each=Ma,ir.eachRight=ka,ir.first=ia,mA(ir,(DA={},Fr(ir,function(t,e){re.call(ir.prototype,e)||(DA[e]=t)}),DA),{chain:!1}),ir.VERSION="4.17.11",Ue(["bind","bindKey","curry","curryRight","partial","partialRight"],function(t){ir[t].placeholder=ir}),Ue(["drop","take"],function(t,e){Ar.prototype[t]=function(n){n=n===o?1:Fn(ws(n),0);var r=this.__filtered__&&!e?new Ar(this):this.clone();return r.__filtered__?r.__takeCount__=Tn(n,r.__takeCount__):r.__views__.push({size:Tn(n,k),type:t+(r.__dir__<0?"Right":"")}),r},Ar.prototype[t+"Right"]=function(e){return this.reverse()[t](e).reverse()}}),Ue(["filter","map","takeWhile"],function(t,e){var n=e+1,r=1==n||3==n;Ar.prototype[t]=function(t){var e=this.clone();return e.__iteratees__.push({iteratee:Eo(t,3),type:n}),e.__filtered__=e.__filtered__||r,e}}),Ue(["head","last"],function(t,e){var n="take"+(e?"Right":"");Ar.prototype[t]=function(){return this[n](1).value()[0]}}),Ue(["initial","tail"],function(t,e){var n="drop"+(e?"":"Right");Ar.prototype[t]=function(){return this.__filtered__?new Ar(this):this[n](1)}}),Ar.prototype.compact=function(){return this.filter(fA)},Ar.prototype.find=function(t){return this.filter(t).head()},Ar.prototype.findLast=function(t){return this.reverse().find(t)},Ar.prototype.invokeMap=ui(function(t,e){return"function"==typeof t?new Ar(this):this.map(function(n){return zr(n,t,e)})}),Ar.prototype.reject=function(t){return this.filter(Ja(Eo(t)))},Ar.prototype.slice=function(t,e){t=ws(t);var n=this;return n.__filtered__&&(t>0||e<0)?new Ar(n):(t<0?n=n.takeRight(-t):t&&(n=n.drop(t)),e!==o&&(n=(e=ws(e))<0?n.dropRight(-e):n.take(e-t)),n)},Ar.prototype.takeRightWhile=function(t){return this.reverse().takeWhile(t).reverse()},Ar.prototype.toArray=function(){return this.take(k)},Fr(Ar.prototype,function(t,e){var n=/^(?:filter|find|map|reject)|While$/.test(e),r=/^(?:head|last)$/.test(e),i=ir[r?"take"+("last"==e?"Right":""):e],a=r||/^find/.test(e);i&&(ir.prototype[e]=function(){var e=this.__wrapped__,s=r?[1]:arguments,A=e instanceof Ar,l=s[0],c=A||ns(e),u=function(t){var e=i.apply(ir,Pe([t],s));return r&&g?e[0]:e};c&&n&&"function"==typeof l&&1!=l.length&&(A=c=!1);var g=this.__chain__,h=!!this.__actions__.length,d=a&&!g,f=A&&!h;if(!a&&c){e=f?e:new Ar(this);var p=t.apply(e,s);return p.__actions__.push({func:Qa,args:[u],thisArg:o}),new sr(p,g)}return d&&f?t.apply(this,s):(p=this.thru(u),d?r?p.value()[0]:p.value():p)})}),Ue(["pop","push","shift","sort","splice","unshift"],function(t){var e=Zt[t],n=/^(?:push|sort|unshift)$/.test(t)?"tap":"thru",r=/^(?:pop|shift)$/.test(t);ir.prototype[t]=function(){var t=arguments;if(r&&!this.__chain__){var i=this.value();return e.apply(ns(i)?i:[],t)}return this[n](function(n){return e.apply(ns(n)?n:[],t)})}}),Fr(Ar.prototype,function(t,e){var n=ir[e];if(n){var r=n.name+"";(Vn[r]||(Vn[r]=[])).push({name:e,func:n})}}),Vn[Zi(o,p).name]=[{name:"wrapper",func:o}],Ar.prototype.clone=function(){var t=new Ar(this.__wrapped__);return t.__actions__=Hi(this.__actions__),t.__dir__=this.__dir__,t.__filtered__=this.__filtered__,t.__iteratees__=Hi(this.__iteratees__),t.__takeCount__=this.__takeCount__,t.__views__=Hi(this.__views__),t},Ar.prototype.reverse=function(){if(this.__filtered__){var t=new Ar(this);t.__dir__=-1,t.__filtered__=!0}else(t=this.clone()).__dir__*=-1;return t},Ar.prototype.value=function(){var t=this.__wrapped__.value(),e=this.__dir__,n=ns(t),r=e<0,i=n?t.length:0,o=function(t,e,n){for(var r=-1,i=n.length;++r<i;){var o=n[r],a=o.size;switch(o.type){case"drop":t+=a;break;case"dropRight":e-=a;break;case"take":e=Tn(e,t+a);break;case"takeRight":t=Fn(t,e-a)}}return{start:t,end:e}}(0,i,this.__views__),a=o.start,s=o.end,A=s-a,l=r?s:a-1,c=this.__iteratees__,u=c.length,g=0,h=Tn(A,this.__takeCount__);if(!n||!r&&i==A&&h==A)return wi(t,this.__actions__);var d=[];t:for(;A--&&g<h;){for(var f=-1,p=t[l+=e];++f<u;){var I=c[f],C=I.iteratee,m=I.type,y=C(p);if(2==m)p=y;else if(!y){if(1==m)continue t;break t}}d[g++]=p}return d},ir.prototype.at=wa,ir.prototype.chain=function(){return ba(this)},ir.prototype.commit=function(){return new sr(this.value(),this.__chain__)},ir.prototype.next=function(){this.__values__===o&&(this.__values__=bs(this.value()));var t=this.__index__>=this.__values__.length;return{done:t,value:t?o:this.__values__[this.__index__++]}},ir.prototype.plant=function(t){for(var e,n=this;n instanceof ar;){var r=Xo(n);r.__index__=0,r.__values__=o,e?i.__wrapped__=r:e=r;var i=r;n=n.__wrapped__}return i.__wrapped__=t,e},ir.prototype.reverse=function(){var t=this.__wrapped__;if(t instanceof Ar){var e=t;return this.__actions__.length&&(e=new Ar(this)),(e=e.reverse()).__actions__.push({func:Qa,args:[ga],thisArg:o}),new sr(e,this.__chain__)}return this.thru(ga)},ir.prototype.toJSON=ir.prototype.valueOf=ir.prototype.value=function(){return wi(this.__wrapped__,this.__actions__)},ir.prototype.first=ir.prototype.head,nn&&(ir.prototype[nn]=function(){return this}),ir}();be._=Bn,(i=function(){return Bn}.call(e,n,e,r))===o||(r.exports=i)}).call(this)}).call(this,n(21),n(43)(t))},function(t,e){t.exports=__WEBPACK_EXTERNAL_MODULE__91__},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(41),i=n(26),o=n(25),a=n(153),s=n(157);e.TableEditorPart=class extends r.Part{constructor(t){super(t),this.bag=null,this.editor=new a.TableEditorWidget,this.editor.onChange.connect(this.handleTableChanged,this),this.layout.insertWidget(0,this.editor)}static GetMetadata(){const t=super.GetMetadata();return t.description="Display a table in an editable view.",t.remarks='\nThis part displays an editable table. When bound to a global, you can use this\nfor two-way editing (where your changes are synced to that global).\n\nYou can also enter tables as CSVs- right-click anywhere on the part and select\n"Edit Table as CSV". This will open a dialog that you can paste CSVs or Excel\ncells into.',t.addOption("Input Data",i.Types.Table,o.Table.NullTable()),t}get activeColumn(){return this.editor.activeColumn}get activeCell(){return this.editor.activeCell}dispose(){this.isDisposed||(this.editor.onChange.disconnect(this.handleTableChanged,this),super.dispose())}async initialize(){}async render(t){const e=t.get("Input Data");this.bag=t,e!==this.editor.table&&(this.editor.table=e)}getName(){return"TableEditor"}getTypeForCol(t){if(!this.editor.table)return i.Types.Any;const e=this.editor.table.columnNames.indexOf(t);return this.editor.table.columnTypes[e]||i.Types.Any}addNewColumn(t,e){if(!this.editor.table)return;const n=this.editor.table.copyTable();n.appendColumn(this.findUniqueColumnName(t),i.Types.findType(e)),this.bag&&this.bag.set("Input Data",n)}changeColumnName(t,e,n){const r=this.editor.table&&this.editor.table.copyTable();if(!r)return;const o=i.Types.findType(n)||i.Types.Any,a=r.columnNames.indexOf(t);let s=r.columnNames,A=r.columnTypes;e!==t&&(s=[...r.columnNames.slice(0,a),this.findUniqueColumnName(e),...r.columnNames.slice(a+1)]),n!==r.columnTypes[a].serializableName&&(A=[...r.columnTypes.slice(0,a),o,...r.columnTypes.slice(a+1)]),r.setColumns(s,A),r.dropInvalidTypes(),this.bag&&this.bag.set("Input Data",r)}deleteColumn(t){const e=this.editor.table;if(null==e)return;const n=new o.Table,r=e.columnNames.indexOf(t);if(-1===r)throw Error("Column '"+t+"' not in table!");n.setColumns([...e.columnNames.slice(0,r),...e.columnNames.slice(r+1)],[...e.columnTypes.slice(0,r),...e.columnTypes.slice(r+1)]);for(let t=0;t<e.rows.length;t++){const r=e.rows[t],i=n.createRow(r.name);for(const t of n.columnNames)i.setValue(t,r.getValue(t));n.appendRow(i)}this.bag&&this.bag.set("Input Data",n)}deleteRow(t){const e=this.editor.table;if(!e)return;const n=o.TableHelper.toMatrixObject(e);n.Rows.splice(t,1);const r=new o.Table;r.setColumns(e.columnNames,e.columnTypes);for(let t=0;t<n.Rows.length;t++){const e=r.createRow(null);for(let r=0;r<n.Cols.length;r++)e.setValue(r,n.Rows[t][r]);r.appendRow(e)}this.bag&&this.bag.set("Input Data",r)}toCsv(){const t=this.editor.table;return t?s.unparse(o.TableHelper.toObjectArray(t),{header:!0,newline:"\n"}):""}fromCsv(t){const e=s.parse(t,{header:!0,dynamicTyping:!0});this.bag&&this.bag.set("Input Data",o.TableHelper.fromObjectArray(e.data))}handleTableChanged(){this.bag&&this.bag.set("Input Data",this.editor.table)}findUniqueColumnName(t){const e=this.editor.table;if(null==e)return t;if(-1===e.columnNames.indexOf(t))return t;let n=0;for(;-1!==e.columnNames.indexOf(t+n);)n++;return t+n}}},function(t,e){t.exports=__WEBPACK_EXTERNAL_MODULE__93__},function(t,e,n){var r=n(48),i=n(7)("toStringTag"),o="Arguments"==r(function(){return arguments}());t.exports=function(t){var e,n,a;return void 0===t?"Undefined":null===t?"Null":"string"==typeof(n=function(t,e){try{return t[e]}catch(t){}}(e=Object(t),i))?n:o?r(e):"Object"==(a=r(e))&&"function"==typeof e.callee?"Arguments":a}},function(t,e,n){t.exports=!n(16)&&!n(17)(function(){return 7!=Object.defineProperty(n(96)("div"),"a",{get:function(){return 7}}).a})},function(t,e,n){var r=n(15),i=n(13).document,o=r(i)&&r(i.createElement);t.exports=function(t){return o?i.createElement(t):{}}},function(t,e){t.exports=function(t,e){return{value:e,done:!!t}}},function(t,e,n){"use strict";var r=n(50),i=n(37),o=n(23),a=n(22),s=n(38),A=n(165),l=n(54),c=n(170),u=n(7)("iterator"),g=!([].keys&&"next"in[].keys()),h=function(){return this};t.exports=function(t,e,n,d,f,p,I){A(n,e,d);var C,m,y,E=function(t){if(!g&&t in Q)return Q[t];switch(t){case"keys":case"values":return function(){return new n(this,t)}}return function(){return new n(this,t)}},v=e+" Iterator",B="values"==f,b=!1,Q=t.prototype,w=Q[u]||Q["@@iterator"]||f&&Q[f],x=w||E(f),_=f?B?E("entries"):x:void 0,D="Array"==e&&Q.entries||w;if(D&&(y=c(D.call(new t)))!==Object.prototype&&y.next&&(l(y,v,!0),r||"function"==typeof y[u]||a(y,u,h)),B&&w&&"values"!==w.name&&(b=!0,x=function(){return w.call(this)}),r&&!I||!g&&!b&&Q[u]||a(Q,u,x),s[e]=x,s[v]=h,f)if(C={values:B?x:E("values"),keys:p?x:E("keys"),entries:_},I)for(m in C)m in Q||o(Q,m,C[m]);else i(i.P+i.F*(g||b),e,C);return C}},function(t,e,n){var r=n(27),i=n(30),o=n(167)(!1),a=n(73)("IE_PROTO");t.exports=function(t,e){var n,s=i(t),A=0,l=[];for(n in s)n!=a&&r(s,n)&&l.push(n);for(;e.length>A;)r(s,n=e[A++])&&(~o(l,n)||l.push(n));return l}},function(t,e){var n,r,i=t.exports={};function o(){throw new Error("setTimeout has not been defined")}function a(){throw new Error("clearTimeout has not been defined")}function s(t){if(n===setTimeout)return setTimeout(t,0);if((n===o||!n)&&setTimeout)return n=setTimeout,setTimeout(t,0);try{return n(t,0)}catch(e){try{return n.call(null,t,0)}catch(e){return n.call(this,t,0)}}}!function(){try{n="function"==typeof setTimeout?setTimeout:o}catch(t){n=o}try{r="function"==typeof clearTimeout?clearTimeout:a}catch(t){r=a}}();var A,l=[],c=!1,u=-1;function g(){c&&A&&(c=!1,A.length?l=A.concat(l):u=-1,l.length&&h())}function h(){if(!c){var t=s(g);c=!0;for(var e=l.length;e;){for(A=l,l=[];++u<e;)A&&A[u].run();u=-1,e=l.length}A=null,c=!1,function(t){if(r===clearTimeout)return clearTimeout(t);if((r===a||!r)&&clearTimeout)return r=clearTimeout,clearTimeout(t);try{r(t)}catch(e){try{return r.call(null,t)}catch(e){return r.call(this,t)}}}(t)}}function d(t,e){this.fun=t,this.array=e}function f(){}i.nextTick=function(t){var e=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)e[n-1]=arguments[n];l.push(new d(t,e)),1!==l.length||c||s(h)},d.prototype.run=function(){this.fun.apply(null,this.array)},i.title="browser",i.browser=!0,i.env={},i.argv=[],i.version="",i.versions={},i.on=f,i.addListener=f,i.once=f,i.off=f,i.removeListener=f,i.removeAllListeners=f,i.emit=f,i.prependListener=f,i.prependOnceListener=f,i.listeners=function(t){return[]},i.binding=function(t){throw new Error("process.binding is not supported")},i.cwd=function(){return"/"},i.chdir=function(t){throw new Error("process.chdir is not supported")},i.umask=function(){return 0}},function(t,e,n){"use strict";var r=n(11),i=n(34),o=n(65),a=n(66);n(67)("match",1,function(t,e,n,s){return[function(n){var r=t(this),i=null==n?void 0:n[e];return void 0!==i?i.call(n,r):new RegExp(n)[e](String(r))},function(t){var e=s(n,t,this);if(e.done)return e.value;var A=r(t),l=String(this);if(!A.global)return a(A,l);var c=A.unicode;A.lastIndex=0;for(var u,g=[],h=0;null!==(u=a(A,l));){var d=String(u[0]);g[h]=d,""===d&&(A.lastIndex=o(l,i(A.lastIndex),c)),h++}return 0===h?null:g}]})},function(t,e){var n=new function(){var t;try{throw new Error}catch(i){var e=i.stack.split("\n"),n=0;for(var r in e)if(e[r].match(/http[s]?:\/\//)){n=Number(r);break}t=e[n].match(/((http[s]?:\/\/.+\/)([^\/]+\.(js|html))).*?:/)}this.fullPath=function(){return t?t[1]:"undefined"!=typeof window?window.location.origin+window.location.pathname:""},this.path=function(){return t?t[2]:"undefined"!=typeof window?window.location.pathname:""},this.host=function(){var t=this.path().match(/.+?\/\/.+?\//);return t?t[0]:"undefined"!=typeof window?window.location.hostname:""},this.file=function(){return t?t[3]:""}};t.exports.host=n.host(),t.exports.path=n.path(),t.exports.isCrossOrigin=function(e){if(e&&e.length){var n=document.createElement("a");return n.href=e,!n.href.startsWith(window.location.origin)}return window.location.origin!==t.exports.host.slice(0,window.location.origin.length)},t.exports.publicPath=function(e){return e&&e.length?e:t.exports.path},t.exports.BlobWorker=function(t,e){var n=new Blob([t]),r=window.URL.createObjectURL(n),i=new Worker(r);e&&e(i)},t.exports.XHRWorker=function(e,n){var r=new XMLHttpRequest;r.addEventListener("load",function(){t.exports.BlobWorker(r.responseText,n)},r),r.open("get",e,!0),r.send()}},function(t,e,n){var r=n(13),i=n(35),o=n(50),a=n(104),s=n(19).f;t.exports=function(t){var e=i.Symbol||(i.Symbol=o?{}:r.Symbol||{});"_"==t.charAt(0)||t in e||s(e,t,{value:a.f(t)})}},function(t,e,n){e.f=n(7)},function(t,e){e.f=Object.getOwnPropertySymbols},function(t,e,n){var r=n(99),i=n(74).concat("length","prototype");e.f=Object.getOwnPropertyNames||function(t){return r(t,i)}},function(t,e,n){var r=n(80),i=n(51),o=n(30),a=n(70),s=n(27),A=n(95),l=Object.getOwnPropertyDescriptor;e.f=n(16)?l:function(t,e){if(t=o(t),e=a(e,!0),A)try{return l(t,e)}catch(t){}if(s(t,e))return i(!r.f.call(t,e),t[e])}},function(t,e,n){(function(e){var n="object"==typeof e&&e&&e.Object===Object&&e;t.exports=n}).call(this,n(21))},function(t,e,n){(function(e){!function(e){"use strict";var n,r=Object.prototype,i=r.hasOwnProperty,o="function"==typeof Symbol?Symbol:{},a=o.iterator||"@@iterator",s=o.asyncIterator||"@@asyncIterator",A=o.toStringTag||"@@toStringTag",l="object"==typeof t,c=e.regeneratorRuntime;if(c)l&&(t.exports=c);else{(c=e.regeneratorRuntime=l?t.exports:{}).wrap=y;var u="suspendedStart",g="suspendedYield",h="executing",d="completed",f={},p={};p[a]=function(){return this};var I=Object.getPrototypeOf,C=I&&I(I(k([])));C&&C!==r&&i.call(C,a)&&(p=C);var m=b.prototype=v.prototype=Object.create(p);B.prototype=m.constructor=b,b.constructor=B,b[A]=B.displayName="GeneratorFunction",c.isGeneratorFunction=function(t){var e="function"==typeof t&&t.constructor;return!!e&&(e===B||"GeneratorFunction"===(e.displayName||e.name))},c.mark=function(t){return Object.setPrototypeOf?Object.setPrototypeOf(t,b):(t.__proto__=b,A in t||(t[A]="GeneratorFunction")),t.prototype=Object.create(m),t},c.awrap=function(t){return{__await:t}},Q(w.prototype),w.prototype[s]=function(){return this},c.AsyncIterator=w,c.async=function(t,e,n,r){var i=new w(y(t,e,n,r));return c.isGeneratorFunction(e)?i:i.next().then(function(t){return t.done?t.value:i.next()})},Q(m),m[A]="Generator",m[a]=function(){return this},m.toString=function(){return"[object Generator]"},c.keys=function(t){var e=[];for(var n in t)e.push(n);return e.reverse(),function n(){for(;e.length;){var r=e.pop();if(r in t)return n.value=r,n.done=!1,n}return n.done=!0,n}},c.values=k,M.prototype={constructor:M,reset:function(t){if(this.prev=0,this.next=0,this.sent=this._sent=n,this.done=!1,this.delegate=null,this.method="next",this.arg=n,this.tryEntries.forEach(D),!t)for(var e in this)"t"===e.charAt(0)&&i.call(this,e)&&!isNaN(+e.slice(1))&&(this[e]=n)},stop:function(){this.done=!0;var t=this.tryEntries[0].completion;if("throw"===t.type)throw t.arg;return this.rval},dispatchException:function(t){if(this.done)throw t;var e=this;function r(r,i){return s.type="throw",s.arg=t,e.next=r,i&&(e.method="next",e.arg=n),!!i}for(var o=this.tryEntries.length-1;o>=0;--o){var a=this.tryEntries[o],s=a.completion;if("root"===a.tryLoc)return r("end");if(a.tryLoc<=this.prev){var A=i.call(a,"catchLoc"),l=i.call(a,"finallyLoc");if(A&&l){if(this.prev<a.catchLoc)return r(a.catchLoc,!0);if(this.prev<a.finallyLoc)return r(a.finallyLoc)}else if(A){if(this.prev<a.catchLoc)return r(a.catchLoc,!0)}else{if(!l)throw new Error("try statement without catch or finally");if(this.prev<a.finallyLoc)return r(a.finallyLoc)}}}},abrupt:function(t,e){for(var n=this.tryEntries.length-1;n>=0;--n){var r=this.tryEntries[n];if(r.tryLoc<=this.prev&&i.call(r,"finallyLoc")&&this.prev<r.finallyLoc){var o=r;break}}o&&("break"===t||"continue"===t)&&o.tryLoc<=e&&e<=o.finallyLoc&&(o=null);var a=o?o.completion:{};return a.type=t,a.arg=e,o?(this.method="next",this.next=o.finallyLoc,f):this.complete(a)},complete:function(t,e){if("throw"===t.type)throw t.arg;return"break"===t.type||"continue"===t.type?this.next=t.arg:"return"===t.type?(this.rval=this.arg=t.arg,this.method="return",this.next="end"):"normal"===t.type&&e&&(this.next=e),f},finish:function(t){for(var e=this.tryEntries.length-1;e>=0;--e){var n=this.tryEntries[e];if(n.finallyLoc===t)return this.complete(n.completion,n.afterLoc),D(n),f}},catch:function(t){for(var e=this.tryEntries.length-1;e>=0;--e){var n=this.tryEntries[e];if(n.tryLoc===t){var r=n.completion;if("throw"===r.type){var i=r.arg;D(n)}return i}}throw new Error("illegal catch attempt")},delegateYield:function(t,e,r){return this.delegate={iterator:k(t),resultName:e,nextLoc:r},"next"===this.method&&(this.arg=n),f}}}function y(t,e,n,r){var i=e&&e.prototype instanceof v?e:v,o=Object.create(i.prototype),a=new M(r||[]);return o._invoke=function(t,e,n){var r=u;return function(i,o){if(r===h)throw new Error("Generator is already running");if(r===d){if("throw"===i)throw o;return S()}for(n.method=i,n.arg=o;;){var a=n.delegate;if(a){var s=x(a,n);if(s){if(s===f)continue;return s}}if("next"===n.method)n.sent=n._sent=n.arg;else if("throw"===n.method){if(r===u)throw r=d,n.arg;n.dispatchException(n.arg)}else"return"===n.method&&n.abrupt("return",n.arg);r=h;var A=E(t,e,n);if("normal"===A.type){if(r=n.done?d:g,A.arg===f)continue;return{value:A.arg,done:n.done}}"throw"===A.type&&(r=d,n.method="throw",n.arg=A.arg)}}}(t,n,a),o}function E(t,e,n){try{return{type:"normal",arg:t.call(e,n)}}catch(t){return{type:"throw",arg:t}}}function v(){}function B(){}function b(){}function Q(t){["next","throw","return"].forEach(function(e){t[e]=function(t){return this._invoke(e,t)}})}function w(t){function n(e,r,o,a){var s=E(t[e],t,r);if("throw"!==s.type){var A=s.arg,l=A.value;return l&&"object"==typeof l&&i.call(l,"__await")?Promise.resolve(l.__await).then(function(t){n("next",t,o,a)},function(t){n("throw",t,o,a)}):Promise.resolve(l).then(function(t){A.value=t,o(A)},a)}a(s.arg)}var r;"object"==typeof e.process&&e.process.domain&&(n=e.process.domain.bind(n)),this._invoke=function(t,e){function i(){return new Promise(function(r,i){n(t,e,r,i)})}return r=r?r.then(i,i):i()}}function x(t,e){var r=t.iterator[e.method];if(r===n){if(e.delegate=null,"throw"===e.method){if(t.iterator.return&&(e.method="return",e.arg=n,x(t,e),"throw"===e.method))return f;e.method="throw",e.arg=new TypeError("The iterator does not provide a 'throw' method")}return f}var i=E(r,t.iterator,e.arg);if("throw"===i.type)return e.method="throw",e.arg=i.arg,e.delegate=null,f;var o=i.arg;return o?o.done?(e[t.resultName]=o.value,e.next=t.nextLoc,"return"!==e.method&&(e.method="next",e.arg=n),e.delegate=null,f):o:(e.method="throw",e.arg=new TypeError("iterator result is not an object"),e.delegate=null,f)}function _(t){var e={tryLoc:t[0]};1 in t&&(e.catchLoc=t[1]),2 in t&&(e.finallyLoc=t[2],e.afterLoc=t[3]),this.tryEntries.push(e)}function D(t){var e=t.completion||{};e.type="normal",delete e.arg,t.completion=e}function M(t){this.tryEntries=[{tryLoc:"root"}],t.forEach(_,this),this.reset(!0)}function k(t){if(t){var e=t[a];if(e)return e.call(t);if("function"==typeof t.next)return t;if(!isNaN(t.length)){var r=-1,o=function e(){for(;++r<t.length;)if(i.call(t,r))return e.value=t[r],e.done=!1,e;return e.value=n,e.done=!0,e};return o.next=o}}return{next:S}}function S(){return{value:n,done:!0}}}("object"==typeof e?e:"object"==typeof window?window:"object"==typeof self?self:this)}).call(this,n(21))},function(t,e,n){var r=n(199),i=n(208),o=n(212);t.exports=function(t){return o(t)?r(t):i(t)}},function(t,e,n){(function(t){var r=n(18),i=n(203),o=e&&!e.nodeType&&e,a=o&&"object"==typeof t&&t&&!t.nodeType&&t,s=a&&a.exports===o?r.Buffer:void 0,A=(s?s.isBuffer:void 0)||i;t.exports=A}).call(this,n(43)(t))},function(t,e,n){var r=n(205),i=n(206),o=n(207),a=o&&o.isTypedArray,s=a?i(a):r;t.exports=s},function(t,e){t.exports=function(t){return"number"==typeof t&&t>-1&&t%1==0&&t<=9007199254740991}},function(t,e,n){var r=n(39),i=n(55);t.exports=function(t){if(!i(t))return!1;var e=r(t);return"[object Function]"==e||"[object GeneratorFunction]"==e||"[object AsyncFunction]"==e||"[object Proxy]"==e}},function(t,e){t.exports=function(t,e){return t===e||t!=t&&e!=e}},function(t,e){var n=Function.prototype.toString;t.exports=function(t){if(null!=t){try{return n.call(t)}catch(t){}try{return t+""}catch(t){}}return""}},function(t,e,n){var r=n(231),i=n(238),o=n(240),a=n(241),s=n(242);function A(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}A.prototype.clear=r,A.prototype.delete=i,A.prototype.get=o,A.prototype.has=a,A.prototype.set=s,t.exports=A},function(t,e,n){var r=n(243),i=n(246),o=n(247);t.exports=function(t,e,n,a,s,A){var l=1&n,c=t.length,u=e.length;if(c!=u&&!(l&&u>c))return!1;var g=A.get(t);if(g&&A.get(e))return g==e;var h=-1,d=!0,f=2&n?new r:void 0;for(A.set(t,e),A.set(e,t);++h<c;){var p=t[h],I=e[h];if(a)var C=l?a(I,p,h,e,t,A):a(p,I,h,t,e,A);if(void 0!==C){if(C)continue;d=!1;break}if(f){if(!i(e,function(t,e){if(!o(f,e)&&(p===t||s(p,t,n,a,A)))return f.push(e)})){d=!1;break}}else if(p!==I&&!s(p,I,n,a,A)){d=!1;break}}return A.delete(t),A.delete(e),d}},function(t,e,n){"use strict";var r=n(37),i=n(71),o=n(45),a=n(17),s=[].sort,A=[1,2,3];r(r.P+r.F*(a(function(){A.sort(void 0)})||!a(function(){A.sort(null)})||!n(267)(s)),"Array",{sort:function(t){return void 0===t?s.call(o(this)):s.call(o(this),i(t))}})},function(t,e,n){"use strict";var r=n(268),i=n(124);t.exports=n(273)("Set",function(t){return function(){return t(this,arguments.length>0?arguments[0]:void 0)}},{add:function(t){return r.def(i(this,"Set"),t=0===t?0:t,t)}},r)},function(t,e,n){var r=n(23);t.exports=function(t,e,n){for(var i in e)r(t,i,e[i],n);return t}},function(t,e){t.exports=function(t,e,n,r){if(!(t instanceof e)||void 0!==r&&r in t)throw TypeError(n+": incorrect invocation!");return t}},function(t,e,n){var r=n(52),i=n(269),o=n(270),a=n(11),s=n(34),A=n(271),l={},c={};(e=t.exports=function(t,e,n,u,g){var h,d,f,p,I=g?function(){return t}:A(t),C=r(n,u,e?2:1),m=0;if("function"!=typeof I)throw TypeError(t+" is not iterable!");if(o(I)){for(h=s(t.length);h>m;m++)if((p=e?C(a(d=t[m])[0],d[1]):C(t[m]))===l||p===c)return p}else for(f=I.call(t);!(d=f.next()).done;)if((p=i(f,C,d.value,e))===l||p===c)return p}).BREAK=l,e.RETURN=c},function(t,e,n){var r=n(15);t.exports=function(t,e){if(!r(t)||t._t!==e)throw TypeError("Incompatible receiver, "+e+" required!");return t}},function(t,e){},function(t,e,n){"use strict";n.r(e);var r={};n.r(r),n.d(r,"CONFIG_ALIASES",function(){return i}),n.d(r,"CONFIG_VALID_KEYS",function(){return o}),n.d(r,"SORT_ORDERS",function(){return A}),n.d(r,"SORT_ORDER_IDS",function(){return l}),n.d(r,"TYPE_AGGREGATES",function(){return c}),n.d(r,"COLUMN_SEPARATOR_STRING",function(){return h}),n.d(r,"TYPE_FILTERS",function(){return d}),n(44),n(29);const i={row_pivot:"row_pivots","row-pivot":"row_pivots","row-pivots":"row_pivots",col_pivot:"column_pivots",col_pivots:"column_pivots",column_pivot:"column_pivots","column-pivot":"column_pivots","column-pivots":"column_pivots",filters:"filter",sorts:"sort"},o=["viewport","row_pivots","column_pivots","aggregates","columns","filter","sort","row_pivot_depth","filter_op"],a=["any","avg","count","distinct count","dominant","first by index","last by index","last","high","low","mean","mean by count","median","pct sum parent","pct sum grand total","sum","sum abs","sum not null","unique"],s=["any","count","distinct count","distinct leaf","dominant","first by index","last by index","last","mean by count","unique"],A=["none","asc","desc","col asc","col desc","asc abs","desc abs","col asc abs","col desc abs"],l=[2,0,1,0,1,3,4,3,4],c={string:s,float:a,integer:a,boolean:["any","count","distinct count","distinct leaf","dominant","first by index","last by index","last","mean by count","unique","and","or"],datetime:s,date:s},u=["<",">","==","<=",">=","!=","is nan","is not nan"],g=["<",">","==","<=",">=","!="],h="|",d={string:["==","contains","!=","in","not in","begins with","ends with"],float:u,integer:u,boolean:["&","|","==","!=","or","and"],datetime:g,date:g};var f=n(60);n(76);const p=new WeakMap;let I=0;function C(t,e){return function(){let n,r=()=>{},i=Array.prototype.slice.call(arguments,0,arguments.length);for(let t=i.length-1;t>=0;t--)"function"==typeof i[t]&&(n=i.splice(t,1)[0]);const o=p.get(n);p.delete(n);let a={cmd:e||"view_method",name:this._name,method:t,args:i,subscribe:!0,callback_id:o};this._worker.post(a,n,r),this._worker.unsubscribe(e,n)}}function m(t,e){return function(){let n,r=()=>{},i=Array.prototype.slice.call(arguments,0,arguments.length);for(let t=i.length-1;t>=0;t--)"function"==typeof i[t]&&(n=i.splice(t,1)[0]);p.set(n,I++);let o={cmd:e||"view_method",name:this._name,method:t,args:i,subscribe:!0,callback_id:I};this._worker.post(o,n,r,!0)}}function y(t,e){return function(){var n=Array.prototype.slice.call(arguments,0,arguments.length);return new Promise(function(r,i){var o={cmd:e||"view_method",name:this._name,method:t,args:n,subscribe:!1};this._worker.post(o,r,i)}.bind(this))}}function E(t){let e=t;do{for(const n of Object.getOwnPropertyNames(e)){const e=t[n];"constructor"!==n&&"function"==typeof e&&(t[n]=e.bind(t))}}while(e=e!==Object&&Object.getPrototypeOf(e))}function v(t,e,n){this._worker=t,this._name=Math.random()+"";var r={cmd:"view",view_name:this._name,table_name:e,config:n};this._worker.post(r),E(this)}function B(t,e){this._worker=t,this._name=e}function b(t,e,n){this._worker=t;let r=n.name||Math.random()+"";if(this._name=r,E(this),e.to_arrow){var i={cmd:"table",name:r,args:[],options:n||{}};this._worker.post(i),e.to_arrow().then(t=>{var i={cmd:"table",name:r,args:[t],options:n||{}};this._worker.post(i),e.on_update(this.update,{mode:"row"})})}else i={cmd:"table",name:r,args:[e],options:n||{}},this._worker.post(i)}function Q(t,e,n){this._worker=t,this._name=Math.random()+"";let r=n;for(let t=0;t<e.length;++t){let n=e[t],r=n.func;"function"==typeof r&&(n.func=r.toString())}var i={cmd:"add_computed",original:r,name:this._name,computed:e};this._worker.post(i)}function w(t,e){this._worker=t,this._name=e}function x(){this._worker={initialized:{value:!1},transferable:!1,msg_id:0,handlers:{},messages:[]},E(this)}n(101),n(175),String.prototype.includes||(String.prototype.includes=function(t,e){return"number"!=typeof e&&(e=0),!(e+t.length>this.length)&&-1!==this.indexOf(t,e)}),Array.prototype.includes||Object.defineProperty(Array.prototype,"includes",{value:function(t,e){if(null==this)throw new TypeError('"this" is null or not defined');var n=Object(this),r=n.length>>>0;if(0===r)return!1;for(var i,o,a=0|e,s=Math.max(a>=0?a:r-Math.abs(a),0);s<r;){if((i=n[s])===(o=t)||"number"==typeof i&&"number"==typeof o&&isNaN(i)&&isNaN(o))return!0;s++}return!1}}),B.prototype=v.prototype,v.prototype.get_config=y("get_config"),v.prototype.to_json=y("to_json"),v.prototype.to_arrow=y("to_arrow"),v.prototype.to_columns=y("to_columns"),v.prototype.to_csv=y("to_csv"),v.prototype.schema=y("schema"),v.prototype.num_columns=y("num_columns"),v.prototype.num_rows=y("num_rows"),v.prototype.set_depth=y("set_depth"),v.prototype.get_row_expanded=y("get_row_expanded"),v.prototype.expand=y("expand"),v.prototype.collapse=y("collapse"),v.prototype.delete=y("delete"),v.prototype.col_to_js_typed_array=y("col_to_js_typed_array"),v.prototype.on_update=m("on_update","view_method"),v.prototype.remove_update=C("remove_update","view_method"),v.prototype.on_delete=m("on_delete","view_method"),v.prototype.remove_delete=C("remove_delete","view_method"),b.prototype.type="table",Q.prototype=b.prototype,w.prototype=b.prototype,b.prototype.add_computed=function(t){return new Q(this._worker,t,this._name)},b.prototype.view=function(t){return new v(this._worker,this._name,t)},b.prototype.schema=y("schema","table_method"),b.prototype.column_metadata=y("column_metadata","table_method"),b.prototype.computed_schema=y("computed_schema","table_method"),b.prototype.is_valid_filter=y("is_valid_filter","table_method"),b.prototype.size=y("size","table_method"),b.prototype.columns=y("columns","table_method"),b.prototype.clear=y("clear","table_method"),b.prototype.replace=y("replace","table_method"),b.prototype.delete=y("delete","table_method"),b.prototype.on_delete=m("on_delete","table_method"),b.prototype.remove=y("remove","table_method"),b.prototype.update=function(t){return new Promise((e,n)=>{var r={name:this._name,cmd:"table_method",method:"update",args:[t]};this._worker.post(r,e,n,!1)})},b.prototype.execute=function(t){var e={cmd:"table_execute",name:this._name,f:t.toString()};this._worker.post(e)},x.prototype.unsubscribe=function(t,e){for(let t of Object.keys(this._worker.handlers))this._worker.handlers[t].resolve===e&&delete this._worker.handlers[t]},x.prototype.post=function(t,e,n,r=!1){(e||n)&&(this._worker.handlers[++this._worker.msg_id]={resolve:e,reject:n,keep_alive:r}),t.id=this._worker.msg_id,this._worker.initialized.value?this.send(t):this._worker.messages.push(()=>this.send(t))},x.prototype.initialize_profile_thread=function(){this._worker.initialized.value?this.send({id:-1,cmd:"init_profile_thread"}):this._worker.messages.push(()=>this.send({id:-1,cmd:"init_profile_thread"}))},x.prototype.send=function(){throw new Error("send() not implemented")},x.prototype.open_table=function(t){return new w(this,t)},x.prototype.open_view=function(t){return new B(this,t)};let _=!1;x.prototype._handle=function(t){if(!this._worker.initialized.value){if(!_&&"undefined"!=typeof document){var e=document.createEvent("Event");e.initEvent("perspective-ready",!1,!0),window.dispatchEvent(e),_=!0}const t=this._worker.messages;if(this._worker.initialized.value=!0,this._worker.messages=[],t)for(const e in t)t.hasOwnProperty(e)&&t[e]()}if(t.data.id){var n=this._worker.handlers[t.data.id];n&&(t.data.error?n.reject(t.data.error):n.resolve(t.data.data),n.keep_alive||delete this._worker.handlers[t.data.id])}},x.prototype.table=function(t,e){return new b(this,t,e||{})},x.prototype.terminate=function(){this._worker.terminate(),this._worker=void 0};var D=n(127),M=n.n(D),k=n(128),S=n.n(k);const L=15e3,R=new class{_fetch(t){return new Promise(e=>{let n=new XMLHttpRequest;n.open("GET",t,!0),n.responseType="arraybuffer",n.onload=()=>{e(n.response)},n.send(null)})}set({wasm:t,worker:e}){this._wasm=t||this._wasm,this._worker=e||this._worker}worker(){return(this._worker||M.a)()}async wasm(){return this._wasm||(this._wasm=await this._fetch(S.a)),this._wasm}};class N extends x{constructor(){super(),this.register()}async register(){let t;const e={cmd:"init",config:Object(f.get_config)()};if("undefined"==typeof WebAssembly||/iPad|iPhone|iPod/.test(navigator.userAgent)&&!window.MSStream)throw new Error("WebAssembly not supported. Support for ASM.JS has been removed as of 0.3.1.");for(var n in[t,e.buffer]=await Promise.all([R.worker(),R.wasm()]),this._worker)t[n]=this._worker[n];this._worker=t,this._worker.addEventListener("message",this._handle.bind(this)),this._worker.postMessage(e),this._detect_transferable()}send(t){this._worker.transferable&&t.args&&t.args[0]instanceof ArrayBuffer?this._worker.postMessage(t,t.args):this._worker.postMessage(t)}terminate(){this._worker.terminate(),this._worker=void 0}_detect_transferable(){var t=new ArrayBuffer(1);this._worker.postMessage(t,[t]),this._worker.transferable=0===t.byteLength,this._worker.transferable?console.log("Transferable support detected"):console.warn("Transferable support not detected")}}class F extends x{constructor(t){super(),this._ws=new WebSocket(t),this._ws.binaryType="arraybuffer",this._ws.onopen=()=>{this.send({id:-1,cmd:"init"})};const e=()=>{this._ws.send("heartbeat"),setTimeout(e,L)};setTimeout(e,15e3),this._ws.onmessage=t=>{"heartbeat"!==t.data&&(this._pending_arrow?(this._handle({data:{id:this._pending_arrow,data:t.data}}),delete this._pending_arrow):(t=JSON.parse(t.data)).is_transferable?this._pending_arrow=t.id:this._handle({data:t}))}}send(t){this._ws.send(JSON.stringify(t))}terminate(){this._ws.close()}}const T=function(){let t,e;return{getInstance:function(n){void 0===t&&(t=new N);const r=JSON.stringify(n);if(r!==e)throw new Error("Confiuration object for shared_worker() has changed - this is probably a bug in your application.");return e=r,t}}}();document.currentScript&&document.currentScript.hasAttribute("preload")&&T.getInstance();const U={override:t=>R.set(t),worker:t=>new N(t),websocket:(t=window.location.origin.replace("http","ws"))=>new F(t),shared_worker:t=>T.getInstance(t)};for(let t of Object.keys(r))U[t]=r[t];e.default=U},function(t,e,n){t.exports=function(){var t=n(102),e=t.publicPath(n.p)+"perspective.wasm.worker.js";return t.isCrossOrigin(n.p)?new Promise(function(n){t.XHRWorker(e,n)}):new Promise(function(t){t(new Worker(e))})}},function(t,e,n){var r=n(102);t.exports=r.publicPath(n.p)+"psp.async.wasm"},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(41),i=n(88),o=n(92),a={id:"@mavenomics/chart-parts:pivot",autoStart:!0,requires:[r.IPartFactory],activate:async(t,e)=>{console.info(`Mavenomics Chart Parts plugin.\nBranch   develop\nBuilt on ${new Date(1572368197671).toLocaleDateString()}\nBuild    #develop\nCommit   ${"develop".substr(0,8)}\n`),e.registerPart("SlickGrid",i.SlickGridPart),e.registerPart("TableEditor",o.TableEditorPart);const{PivotPart:r}=await Promise.resolve().then(()=>n(158));e.registerPart("PivotPart",r)}};e.default=a;var s=n(283);e.RegisterGridCommands=s.RegisterGridCommands},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0}),n(131),n(132),n(133),n(134),n(135),n(136),n(137),n(138),n(139),n(140),n(141);const r=n(142),i=n(28),o=n(25),a=n(90),s=n(62),A=n(33),l=n(91),c=n(26),u=n(150);e.SlickGridWidget=class extends i.Widget{constructor(t){super(),this._hasInitialized=!1,this.resultTable=null,this.dataView=null,this.grid=null,this.maxDepth=0,this.depthColorFunction=null,this.plugins={},this.previousData=[],this.newData=[],this.changes={},this.id="grid-"+t.partId,this.gridContext=t}dispose(){this.isDisposed||this.grid&&this.grid.destroy()}SetupGrid(){this.grid&&this.grid.destroy(),this._hasInitialized=!0;for(const t of this.node.children)t.remove();if(this.dataView=new Slick.Data.DataView,this.dataView.beginUpdate(),this.dataView.setItems(this.newData),this.dataView.setFilter(this.rowFilter.bind(this)),this.dataView.endUpdate(),this.dataView.getItemMetadata=this.rowMetadata.bind(this),this.gridContext.pathColumn>-1)for(var e=0;e<this.maxDepth+1;++e)this.columns[this.gridContext.pathColumn].header.buttons.push({text:String(e+1),command:"path"});let n=-1;n+=this.gridContext.get("Show Row Selectors")?1:0,n+=this.gridContext.get("Show Path Column")?1:0;const r={enableCellNavigation:!0,enableColumnReorder:!1,syncColumnCellResize:!0,asyncEditorLoading:!1,enableAsyncPostRender:!0,rowHeight:17,cellHighlightCssClass:"changed",frozenColumn:this.gridContext.get("Number of Frozen Columns")+n};this.grid=new Slick.Grid(this.node,this.dataView,this.columns,r),this.gridContext.grid=this.grid,this.gridContext.version=l.UUID.uuid4(),this.grid.setSelectionModel(new Slick.CellSelectionModel);const i=document.createElement("div");i.classList.add("cell-hover"),i.style.height=this.grid.getOptions().rowHeight+"px",this.node.appendChild(i),this.rowHover=t(i),this.plugins.tableHoverPlugin=new u.TableHoverPlugin(this.gridContext),this.grid.registerPlugin(this.plugins.tableHoverPlugin),this.plugins.errorDetailPlugin=new u.ErrorDetailHoverPlugin(this.gridContext),this.grid.registerPlugin(this.plugins.errorDetailPlugin),this.plugins.pathButtonsPlugin=new Slick.Plugins.PathButtons,this.grid.registerPlugin(this.plugins.pathButtonsPlugin),this.plugins.autoTooltips=new Slick.AutoTooltips({enableForHeaderCells:!0}),this.grid.registerPlugin(this.plugins.autoTooltips),this.setupCallbacks()}async Render(t,e){if(this.show(),this.resultTable=A.tableToResultTable(t,e),this.gridContext.pathColumn=this.gridContext.get("Show Row Selectors")?1:0,this.gridContext.get("Show Path Column")||(this.gridContext.pathColumn=-1),this.mustRerenderGrid())this.SetupGrid();else{console.time("SlickGrid[id="+this.id+"] Column Diff"),this.setDiffs(),this.dataView.beginUpdate();for(let t=0;t<this.newData.length;t++){const e=this.newData[t];this.dataView.updateItem(e.id,e)}this.dataView.endUpdate(),console.timeEnd("SlickGrid[id="+this.id+"] Column Diff")}this.selectPreviousSelection()}onResize(){this.grid&&this.grid.resizeCanvas()}stringifyCell(t){return t instanceof o.Table&&(t=c.Converters.serialize(t,c.Types.Table)),JSON.stringify(t)}stringifyRow(t){let e={...t};for(let t in e){if(!e.hasOwnProperty(t))continue;let n=e[t];if(n instanceof o.Table)e[t]=c.Converters.serialize(n,c.Types.Table);else switch(typeof n){case"object":e[t]=a.cloneDeep(n)}}return e}mustRerenderGrid(){if(this.columns=r.formatColumns(this.resultTable.Columns,this.gridContext,this.resultTable),this.newData=this.mavenFormatRows(this.columns,this.resultTable.Result.Rows),this.haveColumnsChanged()||this.hasDataChanged()||this.haveOptionsChanged()){this.previousData=new Array(this.newData.length);for(let t=0;t<this.newData.length;t++)this.previousData[t]=this.stringifyRow(this.newData[t]);return this.previousColumns=this.resultTable.Columns,this.previousOptions=this.optionsBagToObject(),!0}return!1}haveColumnsChanged(){return!a.isEqual(this.resultTable.Columns,this.previousColumns)}hasDataChanged(){if(this.newData.length!==this.previousData.length)return!0;for(let t=0;t<this.newData.length;t++)if(this.newData[t].rowPath!==this.previousData[t].rowPath)return!0;return!1}haveOptionsChanged(){return!a.isEqual(this.optionsBagToObject(),this.previousOptions)}optionsBagToObject(){const t=["Checklist. Enabled","Checklist.Column for Checkboxes","Checklist.Column for Values","RadioButtonList. Enabled","RadioButtonList.Column for Radio Buttons","RadioButtonList.Column for Values","Number of Frozen Columns","Show Row Selectors","Show Path Column"],e={};for(let n=0;n<t.length;n++)e[t[n]]=this.gridContext.get(t[n]);return e}setDiffs(){if(null==this.grid||null==this.dataView||null==this.resultTable)return;const t=[];for(let e=0;e<this.resultTable.Columns.length;e++){const n=this.resultTable.Columns[e],r=n.FormattingOptions;A.getFormattingOption(r,"ChangeHighlighting.Enabled")&&t.push(n.Name)}if(0===t.length)return;const e={};for(let t=0;t<this.previousData.length;t++){const n=this.previousData[t];e[n.rowPath+n.id]=t}const n={},r={};for(let e=0;e<this.columns.length;e++){const i=this.columns[e];let o=i.name;null!=i.columnGroups&&i.columnGroups.length>0&&(o=i.columnGroups.join(".")+"."+i.name),t.includes(o)&&(n[i.field]=this.resultTable.Columns.findIndex(t=>t.Name===o),r[i.field]=i.id)}this.changes={};const i=new Array(this.newData.length);for(let t=0;t<this.newData.length;t++){const o=this.newData[t],a=this.previousData[e[o.rowPath+o.id]];if(i[t]=this.stringifyRow(o),null==a)continue;const s=Object.keys(o);for(let t=0;t<s.length;t++){const e=s[t],i=o[e],l=a[e],c=n[e];if(null==c)continue;if(this.stringifyCell(i)===this.stringifyCell(l))continue;const u=this.resultTable.Columns[c],g=r[e],h=u.FormattingOptions,d=A.getFormattingOption(h,"ChangeHighlighting.Style"),f=A.getFormattingOption(h,"ChangeHighlighting.Minimum Change");if(Math.abs(i-l)<+f)continue;null==this.changes[a.id]&&(this.changes[a.id]={});const p=i<l;let I;switch(d){case"GlobalDefault":case"Background":I="change"+(p?"down":"up");break;case"Arrow":default:I="change"+(p?"down":"up")+"arrow"}this.changes[a.id][g]=I}}this.previousData=i}updateCollapseStates(t){clearTimeout(this.h_collapseStates),this.h_collapseStates=setTimeout(()=>{this.gridContext.collapseStates=[],t.forEach(t=>{this.gridContext.collapseStates.push(t._collapsed)})},25)}getColumn(t){if(this.grid){var e=this.grid.getColumns();return-1===this.gridContext.pathColumn?e[t]:e[this.gridContext.pathColumn+t]}}hasChildren(t,e){var n=e[t].indent,r=e[t+1];if(r)return r.indent>n}checkChildren(t,e,n,r){for(var i=n[t].indent,o="col"+e+"_checked",a=t+1;a<n.length;++a)if(n[a].indent>i)n[a][o]=r;else if(n[a].indent<=i)break}checkCheckboxes(t,e,n,r){if(!(t<=-1||null==t)){var i,o=[],a=n[t].indent,s="col"+e+"_checked";for(n[t][s]=r,i=t-1;i>=0;--i)if(n[i].indent===a)o.push(n[i][s]||!1);else if(n[i].indent<a)break;for(i=t;i<n.length;++i)if(n[i].indent===a)o.push(n[i][s]||!1);else if(n[i].indent<a)break;if("indeterminate"!==r)return o.every(A)?this.checkCheckboxes(n[t].parent,e,n,!0):o.some(A)?this.checkCheckboxes(n[t].parent,e,n,"indeterminate"):this.checkCheckboxes(n[t].parent,e,n,!1),o;this.checkCheckboxes(n[t].parent,e,n,"indeterminate")}function A(t){return"indeterminate"!==t&&t}}checkSimilar(t,e,n,r,i){r.forEach((o,a)=>{t===o["col"+e]&&(this.checkCheckboxes(a,n,r,i),this.checkChildren(a,n,r,i))})}radioSimilar(t,e,n){n.forEach(n=>{void 0!==n._radio&&(n._radio=t===n["col"+e])})}getSiblings(t,e){var n,r=[],i=t[e].indent;for(n=e-1;n>=0;--n)if(t[n].indent===i)r.push(t[n]);else if(t[n].indent<i)break;for(n=e;n<t.length;++n)if(t[n].indent===i)r.push(t[n]);else if(t[n].indent<i)break;return r}getImmediateChildren(t,e){var n=[];if(e+1>=t.length)return[];for(var r=t[e].indent+1,i=e+1;i<t.length;++i)if(t[i].indent===r)n.push(t[i]);else if(t[i].indent<r)break;return n}getParent(t,e){for(var n=t[e].indent,r=e;r>-1;--r)if(t[r].indent<n)return t[r]}selectPreviousSelection(){clearTimeout(this.h_checkLastSelection),this.h_checkLastSelection=setTimeout(()=>{const t=this.gridContext,e=this;!function(){let n,r;if(1==!!t.get("Checklist. Enabled")){n=t.get("Checklist.Selected Values"),r=e.getColumn(t.get("Checklist.Column for Values"));var i=e.getColumn(t.get("Checklist.Column for Checkboxes"));let o=new Set(n||[]);e.dataView.getItems().forEach(t=>{t.isLeaf&&e.checkCheckboxes(t.rowNumber,i.columnNumber,e.dataView.getItems(),o.has(t["col"+r.columnNumber]))})}}(),function(){let n,r;if(!0===t.get("RadioButtonList. Enabled")){if(n=t.get("RadioButtonList.Last Selection"),r=e.getColumn(t.get("RadioButtonList.Column for Values")),e.getColumn(t.get("RadioButtonList.Column for Radio Buttons")),void 0===n||"0 = 1"===n)return!1;e.dataView.getItems().forEach(t=>{void 0!==t._radio&&(t._radio=t["col"+r.columnNumber]==n)})}}(),this.grid.invalidate()},25)}mavenFormatRows(e,n){var r=[],i=[],o=0,a=0;this.maxDepth=0;for(let t=0;t<n.length;t++){const e=n[t];e.Depth>this.maxDepth&&(this.maxDepth=e.Depth)}for(let t=0;t<n.length;t++){const s=n[t];var A=Math.abs(a-s.Depth),l=0;if(s.Depth>a){for(l=0;l<A;++l)++o;i.push(t-1)}else if(s.Depth<a)for(l=0;l<A;++l)--o,i.pop();a=s.Depth;let c={id:s.JavascriptDomIdentifier,parent:i.length>0?i[i.length-1]:null,indent:o,rowNumber:t};c.path="null"!=s.Name?s.Name:"",c.rowPath=s.Path,c.rowName=s.Name,s.Depth===this.maxDepth&&(c.radio=!1,c.isLeaf=!0),!0===this.gridContext.collapseStates[t]&&(c._collapsed=!0),this.gridContext.pathColumn>-1&&(e[this.gridContext.pathColumn].hasRadio&&s.Depth===this.maxDepth&&(c._radio=!1),c.col0=c.path);for(let t=0;t<s.RowData.length;t++){let n=s.RowData[t];e[this.gridContext.pathColumn+t+1].hasRadio&&s.Depth===this.maxDepth&&(c._radio=!1),c[e[this.gridContext.pathColumn+t+1].field]=n}r.push(c)}this.depthColorFunction=s.scale.linear().domain([-1,this.maxDepth]).range(["#c3c3c3","white"]).clamp(!0);for(var c=t("<style>").attr("type","text/css").attr("id",this.id+"row-depth-styles"),u=0;u<=this.maxDepth;++u)c.append(`#${this.id} .slick-row-depth${u} { background: ${this.depthColorFunction(u)} !important; }`);return t("#"+this.id+"row-depth-styles").remove(),c.appendTo("head"),r}setupCallbacks(){null!=this.grid&&null!=this.dataView&&(this.grid.onHeaderClick.subscribe((e,n)=>{var r=t(e.target);if(r.hasClass("toggle")){var i=r.data("groupNum");return this.toggleHeader(e,n,i),this.grid.setColumns(this.grid.getColumns()),this.grid.invalidate(),void e.stopImmediatePropagation()}var o=n.column.columnNumber;Number.isNaN(o)?this.grid.getSelectionModel().setSelectedRanges([new Slick.Range(0,1,this.grid.getDataLength()-1,this.grid.getColumns().length-1)]):this.grid.getSelectionModel().setSelectedRanges([new Slick.Range(0,o,this.grid.getDataLength()-1,o)])}),this.grid.onClick.subscribe((e,n)=>{let r=t(e.target),i=t(this.grid.getCellNode(n.row,n.cell));if(r.hasClass("toggle"))return this.toggle(e,n),this.grid.invalidate(),void this.grid.render();var o,a,s,A=this.gridContext.get("Show Row Selectors")?1:0,l=n.cell-A;l<0?this.grid.getSelectionModel().setSelectedRanges([new Slick.Range(n.row,1,n.row,this.grid.getColumns().length-1)]):(i.find(".slick-cell-checkbox").length>0&&(o=this.dataView.getItems(),a=this.dataView.getItem(n.row),s="indeterminate"===(s=a["col"+l+"_checked"]||!1)||!s,clearTimeout(this.h_broadcastCheckboxLinkage),this.h_broadcastCheckboxLinkage=setTimeout(()=>{let t=this.getColumn(this.gridContext.get("Checklist.Column for Values"));if(null!=t){this.checkChildren(a.rowNumber,l,o,s),this.checkCheckboxes(a.rowNumber,l,o,s),a.isLeaf&&this.checkSimilar(a["col"+t.columnNumber],t.columnNumber,l,o,s);var e=[];o.forEach((n,r)=>{if(!0===n["col"+l+"_checked"]){var i=n["col"+t.columnNumber];-1!==e.indexOf(i)||this.hasChildren(r,o)||e.push(i)}}),this.grid.invalidate(),this.gridContext.set("Checklist.Selected Values",e)}},25),this.grid.setActiveCell(n.row,n.cell,!1)),i.find(".slick-cell-radio").length>0&&(o=this.dataView.getItems(),a=this.dataView.getItem(n.row),clearTimeout(this.h_broadcastRadioLinkage),this.h_broadcastRadioLinkage=setTimeout(()=>{var t=this.getColumn(this.gridContext.get("RadioButtonList.Column for Values")),e=a["col"+t.columnNumber];this.radioSimilar(e,t.columnNumber,o),this.gridContext.set("RadioButtonList.Last Selection",e),this.grid.invalidate()},25),this.grid.setActiveCell(n.row,n.cell,!1)),this.grid.setActiveCell(n.row,n.cell))}),this.grid.onMouseEnter.subscribe((t,e)=>{var n=this.grid.getCellFromEvent(t),r=this.grid.getCellNode(n.row,n.cell);const{width:i,left:o,right:a}=this.node.getBoundingClientRect();this.rowHover.css({height:this.grid.getOptions().rowHeight+"px",top:r.offset().top,width:i+"px",left:o+"px",right:a+"px"}).show()}),this.grid.onMouseLeave.subscribe((t,e)=>{this.rowHover.hide()}),this.grid.onDblClick.subscribe((e,n)=>{t(e.target).find(".toggle").length>0&&(this.toggle(e,n),this.grid.invalidate(),this.grid.render())}),this.grid.onHeaderContextMenu.subscribe((t,{column:e})=>{if("path"===e.id)return;let n=this.resultTable.Columns.map(t=>t.Name),r=0;r+=this.gridContext.get("Show Row Selectors")?1:0,r+=this.gridContext.get("Show Path Column")?1:0;let i=this.columns.findIndex(t=>t.id===e.id)-r;this.gridContext.lastColumn=n[i]}),this.grid.onRenderRows.subscribe(()=>{t(this.grid.getContainerNode()).find(".slick-cell-checkbox.indeterminate").prop("indeterminate","yes")}),this.grid.onColumnsResized.subscribe(a.debounce(t=>{if(!this.resultTable||!this.grid)return;const e=this.grid.getColumns().map(t=>t.width);this.resultTable.Columns.map((t,n)=>{let r=e[n+1+this.gridContext.pathColumn];t.FormattingOptions["General.ColumnWidthPixels"]=r,this.gridContext.setColumnFormatting(t.Name,A.stripDefaultsFromFormatting(t.FormattingOptions))})},300)),this.dataView.onRowCountChanged.subscribe((t,e)=>{this.grid&&this.grid.render()}),this.dataView.onRowsChanged.subscribe((t,e)=>{this.grid.removeCellCssStyles("highlight"),this.grid.setCellCssStyles("highlight",this.changes)}),this.plugins.pathButtonsPlugin.onCommand.subscribe((t,e)=>{var n=+e.button.text-1,r=[];this.grid.getData().getItems().forEach(t=>{t.indent>=n?t._collapsed=!0:t._collapsed=void 0,r.push(t)}),this.updateCollapseStates(this.grid.getData().getItems()),this.grid.getData().setItems(r),this.grid.invalidate()}))}areColumnsInSameBucket(t,e,n){if(t.length<n||e.length<n)return!1;for(var r=0;r<t.length&&r<n;r++)if(t[r]!=e[r])return!1;return!0}toggleHeader(t,e,n){for(var r=e.column,i=e.grid.getColumns(),o=r.columnNumber,a=r.columnGroups,s=o+1;s<i.length;s++){var A=i[s].columnGroups;if(!this.areColumnsInSameBucket(a,A,n+1))break;i[s].collapsed?(i[s].collapsed=!1,i[s].resizable=!0,i[s].width=i[s].oldWidth,i[s].minWidth=i[s].oldMinWidth):(i[s].collapsed=!0,i[s].resizable=!1,i[s].oldWidth=i[s].width,i[s].oldMinWidth=i[s].minWidth,i[s].width=0,i[s].minWidth=0)}}toggle(t,e){var n=this.dataView.getItem(e.row);n&&(n._collapsed=!n._collapsed,this.dataView.updateItem(n.id,n)),this.updateCollapseStates(this.grid.getData().getItems()),t.stopImmediatePropagation()}rowFilter(t,e){if(null!=t.parent)for(var n=this.newData[t.parent];n;){if(n._collapsed)return!1;n=this.newData[n.parent]}return!0}rowMetadata(t){if(void 0!==t)return{cssClasses:"slick-row-depth"+this.dataView.getItem(t).indent}}}}).call(this,n(5))},function(t,e,n){(function(t){!function(t){t.fn.drag=function(e,n,r){var i="string"==typeof e?e:"",o=t.isFunction(e)?e:t.isFunction(n)?n:null;return 0!==i.indexOf("drag")&&(i="drag"+i),r=(e==o?n:r)||{},o?this.bind(i,r,o):this.trigger(i)};var e=t.event,n=e.special,r=n.drag={defaults:{which:1,distance:0,not:":input",handle:null,relative:!1,drop:!0,click:!1},datakey:"dragdata",noBubble:!0,add:function(e){var n=t.data(this,r.datakey),i=e.data||{};n.related+=1,t.each(r.defaults,function(t,e){void 0!==i[t]&&(n[t]=i[t])})},remove:function(){t.data(this,r.datakey).related-=1},setup:function(){if(!t.data(this,r.datakey)){var n=t.extend({related:0},r.defaults);t.data(this,r.datakey,n),e.add(this,"touchstart mousedown",r.init,n),this.attachEvent&&this.attachEvent("ondragstart",r.dontstart)}},teardown:function(){(t.data(this,r.datakey)||{}).related||(t.removeData(this,r.datakey),e.remove(this,"touchstart mousedown",r.init),r.textselect(!0),this.detachEvent&&this.detachEvent("ondragstart",r.dontstart))},init:function(i){if(!r.touched){var o,a=i.data;if(!(0!=i.which&&a.which>0&&i.which!=a.which)&&!t(i.target).is(a.not)&&(!a.handle||t(i.target).closest(a.handle,i.currentTarget).length)&&(r.touched="touchstart"==i.type?this:null,a.propagates=1,a.mousedown=this,a.interactions=[r.interaction(this,a)],a.target=i.target,a.pageX=i.pageX,a.pageY=i.pageY,a.dragging=null,o=r.hijack(i,"draginit",a),a.propagates))return(o=r.flatten(o))&&o.length&&(a.interactions=[],t.each(o,function(){a.interactions.push(r.interaction(this,a))})),a.propagates=a.interactions.length,!1!==a.drop&&n.drop&&n.drop.handler(i,a),r.textselect(!1),r.touched?e.add(r.touched,"touchmove touchend",r.handler,a):e.add(document,"mousemove mouseup",r.handler,a),!(!r.touched||a.live)&&void 0}},interaction:function(e,n){var i=t(e)[n.relative?"position":"offset"]()||{top:0,left:0};return{drag:e,callback:new r.callback,droppable:[],offset:i}},handler:function(i){var o=i.data;switch(i.type){case!o.dragging&&"touchmove":i.preventDefault();case!o.dragging&&"mousemove":if(Math.pow(i.pageX-o.pageX,2)+Math.pow(i.pageY-o.pageY,2)<Math.pow(o.distance,2))break;i.target=o.target,r.hijack(i,"dragstart",o),o.propagates&&(o.dragging=!0);case"touchmove":i.preventDefault();case"mousemove":if(o.dragging){if(r.hijack(i,"drag",o),o.propagates){!1!==o.drop&&n.drop&&n.drop.handler(i,o);break}i.type="mouseup"}case"touchend":case"mouseup":default:r.touched?e.remove(r.touched,"touchmove touchend",r.handler):e.remove(document,"mousemove mouseup",r.handler),o.dragging&&(!1!==o.drop&&n.drop&&n.drop.handler(i,o),r.hijack(i,"dragend",o)),r.textselect(!0),!1===o.click&&o.dragging&&t.data(o.mousedown,"suppress.click",(new Date).getTime()+5),o.dragging=r.touched=!1}},hijack:function(n,i,o,a,s){if(o){var A,l,c,u={event:n.originalEvent,type:n.type},g=i.indexOf("drop")?"drag":"drop",h=a||0,d=isNaN(a)?o.interactions.length:a;n.type=i,n.originalEvent=null,o.results=[];do{if(l=o.interactions[h]){if("dragend"!==i&&l.cancelled)continue;c=r.properties(n,o,l),l.results=[],t(s||l[g]||o.droppable).each(function(a,s){if(c.target=s,n.isPropagationStopped=function(){return!1},!1===(A=s?e.dispatch.call(s,n,c):null)?("drag"==g&&(l.cancelled=!0,o.propagates-=1),"drop"==i&&(l[g][a]=null)):"dropinit"==i&&l.droppable.push(r.element(A)||s),"dragstart"==i&&(l.proxy=t(r.element(A)||l.drag)[0]),l.results.push(A),delete n.result,"dropinit"!==i)return A}),o.results[h]=r.flatten(l.results),"dropinit"==i&&(l.droppable=r.flatten(l.droppable)),"dragstart"!=i||l.cancelled||c.update()}}while(++h<d);return n.type=u.type,n.originalEvent=u.event,r.flatten(o.results)}},properties:function(t,e,n){var i=n.callback;return i.drag=n.drag,i.proxy=n.proxy||n.drag,i.startX=e.pageX,i.startY=e.pageY,i.deltaX=t.pageX-e.pageX,i.deltaY=t.pageY-e.pageY,i.originalX=n.offset.left,i.originalY=n.offset.top,i.offsetX=i.originalX+i.deltaX,i.offsetY=i.originalY+i.deltaY,i.drop=r.flatten((n.drop||[]).slice()),i.available=r.flatten((n.droppable||[]).slice()),i},element:function(t){if(t&&(t.jquery||1==t.nodeType))return t},flatten:function(e){return t.map(e,function(e){return e&&e.jquery?t.makeArray(e):e&&e.length?r.flatten(e):e})},textselect:function(e){t(document)[e?"unbind":"bind"]("selectstart",r.dontstart).css("MozUserSelect",e?"":"none"),document.unselectable=e?"off":"on"},dontstart:function(){return!1},callback:function(){}};r.callback.prototype={update:function(){n.drop&&this.available.length&&t.each(this.available,function(t){n.drop.locate(this,t)})}};var i=e.dispatch;e.dispatch=function(e){if(!(t.data(this,"suppress."+e.type)-(new Date).getTime()>0))return i.apply(this,arguments);t.removeData(this,"suppress."+e.type)};var o=e.fixHooks.touchstart=e.fixHooks.touchmove=e.fixHooks.touchend=e.fixHooks.touchcancel={props:"clientX clientY pageX pageY screenX screenY".split(" "),filter:function(e,n){if(n){var r=n.touches&&n.touches[0]||n.changedTouches&&n.changedTouches[0]||null;r&&t.each(o.props,function(t,n){e[n]=r[n]})}return e}};n.draginit=n.dragstart=n.dragend=r}(t)}).call(this,n(5))},function(t,e,n){(function(t){
/*! Copyright (c) 2011 Brandon Aaron (http://brandonaaron.net)
 * Licensed under the MIT License (LICENSE.txt).
 *
 * Thanks to: http://adomas.org/javascript-mouse-wheel/ for some pointers.
 * Thanks to: Mathias Bank(http://www.mathias-bank.de) for a scope bug fix.
 * Thanks to: Seamus Leahy for adding deltaX and deltaY
 *
 * Version: 3.0.6
 * 
 * Requires: 1.2.2+
 */
!function(t){var e=["DOMMouseScroll","mousewheel"];if(t.event.fixHooks)for(var n=e.length;n;)t.event.fixHooks[e[--n]]=t.event.mouseHooks;function r(e){var n=e||window.event,r=[].slice.call(arguments,1),i=0,o=0,a=0;return(e=t.event.fix(n)).type="mousewheel",n.wheelDelta&&(i=n.wheelDelta/120),n.detail&&(i=-n.detail/3),a=i,void 0!==n.axis&&n.axis===n.HORIZONTAL_AXIS&&(a=0,o=-1*i),void 0!==n.wheelDeltaY&&(a=n.wheelDeltaY/120),void 0!==n.wheelDeltaX&&(o=-1*n.wheelDeltaX/120),r.unshift(e,i,o,a),(t.event.dispatch||t.event.handle).apply(this,r)}t.event.special.mousewheel={setup:function(){if(this.addEventListener)for(var t=e.length;t;)this.addEventListener(e[--t],r,!1);else this.onmousewheel=r},teardown:function(){if(this.removeEventListener)for(var t=e.length;t;)this.removeEventListener(e[--t],r,!1);else this.onmousewheel=null}},t.fn.extend({mousewheel:function(t){return t?this.bind("mousewheel",t):this.trigger("mousewheel")},unmousewheel:function(t){return this.unbind("mousewheel",t)}})}(t)}).call(this,n(5))},function(t,e,n){(function(t){!function(e){function n(){var t=!1,e=!1;this.stopPropagation=function(){t=!0},this.isPropagationStopped=function(){return t},this.stopImmediatePropagation=function(){e=!0},this.isImmediatePropagationStopped=function(){return e}}function r(){this.__nonDataRow=!0}function i(){this.__group=!0,this.level=0,this.count=0,this.value=null,this.title=null,this.collapsed=!1,this.totals=null,this.rows=[],this.groups=null,this.groupingKey=null}function o(){this.__groupTotals=!0,this.group=null}function a(){var t=null;this.isActive=function(e){return e?t===e:null!==t},this.activate=function(e){if(e!==t){if(null!==t)throw"SlickGrid.EditorLock.activate: an editController is still active, can't activate another editController";if(!e.commitCurrentEdit)throw"SlickGrid.EditorLock.activate: editController must implement .commitCurrentEdit()";if(!e.cancelCurrentEdit)throw"SlickGrid.EditorLock.activate: editController must implement .cancelCurrentEdit()";t=e}},this.deactivate=function(e){if(t!==e)throw"SlickGrid.EditorLock.deactivate: specified editController is not the currently active one";t=null},this.commitCurrentEdit=function(){return!t||t.commitCurrentEdit()},this.cancelCurrentEdit=function(){return!t||t.cancelCurrentEdit()}}t.extend(!0,window,{Slick:{Event:function(){var t=[];this.subscribe=function(e){t.push(e)},this.unsubscribe=function(e){for(var n=t.length-1;n>=0;n--)t[n]===e&&t.splice(n,1)},this.notify=function(e,r,i){var o;r=r||new n,i=i||this;for(var a=0;a<t.length&&!r.isPropagationStopped()&&!r.isImmediatePropagationStopped();a++)o=t[a].call(i,r,e);return o}},EventData:n,EventHandler:function(){var t=[];this.subscribe=function(e,n){return t.push({event:e,handler:n}),e.subscribe(n),this},this.unsubscribe=function(e,n){for(var r=t.length;r--;)if(t[r].event===e&&t[r].handler===n)return t.splice(r,1),void e.unsubscribe(n);return this},this.unsubscribeAll=function(){for(var e=t.length;e--;)t[e].event.unsubscribe(t[e].handler);return t=[],this}},Range:function(t,e,n,r){void 0===n&&void 0===r&&(n=t,r=e),this.fromRow=Math.min(t,n),this.fromCell=Math.min(e,r),this.toRow=Math.max(t,n),this.toCell=Math.max(e,r),this.isSingleRow=function(){return this.fromRow==this.toRow},this.isSingleCell=function(){return this.fromRow==this.toRow&&this.fromCell==this.toCell},this.contains=function(t,e){return t>=this.fromRow&&t<=this.toRow&&e>=this.fromCell&&e<=this.toCell},this.toString=function(){return this.isSingleCell()?"("+this.fromRow+":"+this.fromCell+")":"("+this.fromRow+":"+this.fromCell+" - "+this.toRow+":"+this.toCell+")"}},NonDataRow:r,Group:i,GroupTotals:o,EditorLock:a,GlobalEditorLock:new a}}),i.prototype=new r,i.prototype.equals=function(t){return this.value===t.value&&this.count===t.count&&this.collapsed===t.collapsed&&this.title===t.title},o.prototype=new r}()}).call(this,n(5))},function(t,e,n){(function(t){var e;(e=t).extend(!0,window,{Slick:{Data:{DataView:function(t){var n,r,i,o,a,s=this,A="id",l=[],c=[],u={},g=null,h=null,d=null,f=!1,p=!0,I={},C={},m=[],y=[],E={getter:null,formatter:null,comparer:function(t,e){return t.value-e.value},predefinedValues:[],aggregators:[],aggregateEmpty:!1,aggregateCollapsed:!1,aggregateChildGroups:!1,collapsed:!1,displayTotalsRow:!0,lazyTotalsCalculation:!1},v=[],B=[],b=[],Q=":|:",w=0,x=0,_=0,D=new Slick.Event,M=new Slick.Event,k=new Slick.Event;function S(t){for(var e,n=t=t||0,r=l.length;n<r;n++){if(void 0===(e=l[n][A]))throw"Each data element must implement a unique 'id' property";u[e]=n}}function L(){var t=w?Math.max(1,Math.ceil(_/w)):1;return{pageSize:w,pageNum:x,totalRows:_,totalPages:t}}function R(t,e){p=e,r=t,n=null,!1===e&&l.reverse(),l.sort(t),!1===e&&l.reverse(),u={},S(),q()}function N(t,e){p=e,n=t,r=null;var i=Object.prototype.toString;Object.prototype.toString="function"==typeof t?t:function(){return this[t]},!1===e&&l.reverse(),l.sort(),Object.prototype.toString=i,!1===e&&l.reverse(),u={},S(),q()}function F(n){t.groupItemMetadataProvider||(t.groupItemMetadataProvider=new Slick.Data.GroupItemMetadataProvider),B=[],b=[],v=(n=n||[])instanceof Array?n:[n];for(var r=0;r<v.length;r++){var i=v[r]=e.extend(!0,{},E,v[r]);i.getterIsAFn="function"==typeof i.getter,i.compiledAccumulators=[];for(var o=i.aggregators.length;o--;)i.compiledAccumulators[o]=Y(i.aggregators[o]);b[r]={}}q()}function T(){if(!g){g={};for(var t=0,e=c.length;t<e;t++)g[c[t][A]]=t}}function U(t,e){if(null==t)for(var n=0;n<v.length;n++)b[n]={},v[n].collapsed=e;else b[t]={},v[t].collapsed=e;q()}function O(t,e,n){b[t][e]=v[t].collapsed^n,q()}function G(t){var e,n=t.group,r=v[n.level],i=n.level==v.length,o=r.aggregators.length;if(!i&&r.aggregateChildGroups)for(var a=n.groups.length;a--;)n.groups[a].initialized||G(n.groups[a]);for(;o--;)(e=r.aggregators[o]).init(),!i&&r.aggregateChildGroups?r.compiledAccumulators[o].call(e,n.groups):r.compiledAccumulators[o].call(e,n.rows),e.storeResult(t);t.initialized=!0}function H(t){var e=v[t.level],n=new Slick.GroupTotals;n.group=t,t.totals=n,e.lazyTotalsCalculation||G(n)}function j(t){var e=t.toString().match(/^function[^(]*\(([^)]*)\)\s*{([\s\S]*)}$/);return{params:e[1].split(","),body:e[2]}}function Y(t){var e=j(t.accumulate),n=new Function("_items","for (var "+e.params[0]+", _i=0, _il=_items.length; _i<_il; _i++) {"+e.params[0]+" = _items[_i]; "+e.body+"}");return n.displayName=n.name="compiledAccumulatorLoop",n}function P(t,e){for(var n=[],r=0,i=0,o=t.length;i<o;i++)h(t[i],e)&&(n[r++]=t[i]);return n}function K(t,e,n){for(var r,i=[],o=0,a=0,s=t.length;a<s;a++)r=t[a],n[a]?i[o++]=r:h(r,e)&&(i[o++]=r,n[a]=!0);return i}function z(e){if(h){var n=t.inlineFilters?o:P,r=t.inlineFilters?a:K;I.isFilterNarrowing?m=n(m,i):I.isFilterExpanding?m=r(e,i,y):I.isFilterUnchanged||(m=n(e,i))}else m=w?e:e.concat();var s;return w?(m.length<x*w&&(x=Math.floor(m.length/w)),s=m.slice(w*x,w*x+w)):s=m,{totalRows:m.length,rows:s}}function J(t){g=null,I.isFilterNarrowing==C.isFilterNarrowing&&I.isFilterExpanding==C.isFilterExpanding||(y=[]);var e=z(t);_=e.totalRows;var n=e.rows;B=[],v.length&&(B=function t(e,n){for(var r,i,o,a=[],s={},A=n?n.level+1:0,l=v[A],c=0,u=l.predefinedValues.length;c<u;c++)(r=s[i=l.predefinedValues[c]])||((r=new Slick.Group).value=i,r.level=A,r.groupingKey=(n?n.groupingKey+Q:"")+i,a[a.length]=r,s[i]=r);for(c=0,u=e.length;c<u;c++)o=e[c],(r=s[i=l.getterIsAFn?l.getter(o):o[l.getter]])||((r=new Slick.Group).value=i,r.level=A,r.groupingKey=(n?n.groupingKey+Q:"")+i,a[a.length]=r,s[i]=r),r.rows[r.count++]=o;if(A<v.length-1)for(c=0;c<a.length;c++)(r=a[c]).groups=t(r.rows,r);return a.sort(v[A].comparer),a}(n)).length&&(function t(e,n){for(var r,i=v[n=n||0],o=i.collapsed,a=b[n],s=e.length;s--;)(r=e[s]).collapsed&&!i.aggregateCollapsed||(r.groups&&t(r.groups,n+1),i.aggregators.length&&(i.aggregateEmpty||r.rows.length||r.groups&&r.groups.length)&&H(r),r.collapsed=o^a[r.groupingKey],r.title=i.formatter?i.formatter(r):r.value)}(B),n=function t(e,n){for(var r,i,o=v[n=n||0],a=[],s=0,A=0,l=e.length;A<l;A++){if(i=e[A],a[s++]=i,!i.collapsed)for(var c=0,u=(r=i.groups?t(i.groups,n+1):i.rows).length;c<u;c++)a[s++]=r[c];i.totals&&o.displayTotalsRow&&(!i.collapsed||o.aggregateCollapsed)&&(a[s++]=i.totals)}return a}(B));var r=function(t,e){var n,r,i,o=[],a=0,s=e.length;I&&I.ignoreDiffsBefore&&(a=Math.max(0,Math.min(e.length,I.ignoreDiffsBefore))),I&&I.ignoreDiffsAfter&&(s=Math.min(e.length,Math.max(0,I.ignoreDiffsAfter)));for(var l=a,c=t.length;l<s;l++)l>=c?o[o.length]=l:(n=e[l],r=t[l],(v.length&&(i=n.__nonDataRow||r.__nonDataRow)&&n.__group!==r.__group||n.__group&&!n.equals(r)||i&&(n.__groupTotals||r.__groupTotals)||n[A]!=r[A]||d&&d[n[A]])&&(o[o.length]=l));return o}(c,n);return c=n,r}function q(){if(!f){var t=c.length,e=_,n=J(l);w&&_<x*w&&(x=Math.max(0,Math.ceil(_/w)-1),n=J(l)),d=null,C=I,I={},e!=_&&k.notify(L(),null,s),t!=c.length&&D.notify({previous:t,current:c.length},null,s),n.length>0&&M.notify({rows:n},null,s)}}t=e.extend(!0,{},{groupItemMetadataProvider:null,inlineFilters:!1},t),e.extend(this,{beginUpdate:function(){f=!0},endUpdate:function(){f=!1,q()},setPagingOptions:function(t){null!=t.pageSize&&(w=t.pageSize,x=w?Math.min(x,Math.max(0,Math.ceil(_/w)-1)):0),null!=t.pageNum&&(x=Math.min(t.pageNum,Math.max(0,Math.ceil(_/w)-1))),k.notify(L(),null,s),q()},getPagingInfo:L,getItems:function(){return l},setItems:function(t,e){void 0!==e&&(A=e),l=m=t,u={},S(),function(){for(var t,e=0,n=l.length;e<n;e++)if(void 0===(t=l[e][A])||u[t]!==e)throw"Each data element must implement a unique 'id' property"}(),q()},setFilter:function(e){h=e,t.inlineFilters&&(o=function(){var t=j(h),e=t.body.replace(/return false\s*([;}]|$)/gi,"{ continue _coreloop; }$1").replace(/return true\s*([;}]|$)/gi,"{ _retval[_idx++] = $item$; continue _coreloop; }$1").replace(/return ([^;}]+?)\s*([;}]|$)/gi,"{ if ($1) { _retval[_idx++] = $item$; }; continue _coreloop; }$2"),n=["var _retval = [], _idx = 0; ","var $item$, $args$ = _args; ","_coreloop: ","for (var _i = 0, _il = _items.length; _i < _il; _i++) { ","$item$ = _items[_i]; ","$filter$; ","} ","return _retval; "].join("");n=(n=(n=n.replace(/\$filter\$/gi,e)).replace(/\$item\$/gi,t.params[0])).replace(/\$args\$/gi,t.params[1]);var r=new Function("_items,_args",n);return r.displayName=r.name="compiledFilter",r}(),a=function(){var t=j(h),e=t.body.replace(/return false\s*([;}]|$)/gi,"{ continue _coreloop; }$1").replace(/return true\s*([;}]|$)/gi,"{ _cache[_i] = true;_retval[_idx++] = $item$; continue _coreloop; }$1").replace(/return ([^;}]+?)\s*([;}]|$)/gi,"{ if ((_cache[_i] = $1)) { _retval[_idx++] = $item$; }; continue _coreloop; }$2"),n=["var _retval = [], _idx = 0; ","var $item$, $args$ = _args; ","_coreloop: ","for (var _i = 0, _il = _items.length; _i < _il; _i++) { ","$item$ = _items[_i]; ","if (_cache[_i]) { ","_retval[_idx++] = $item$; ","continue _coreloop; ","} ","$filter$; ","} ","return _retval; "].join("");n=(n=(n=n.replace(/\$filter\$/gi,e)).replace(/\$item\$/gi,t.params[0])).replace(/\$args\$/gi,t.params[1]);var r=new Function("_items,_args,_cache",n);return r.displayName=r.name="compiledFilterWithCaching",r}()),q()},sort:R,fastSort:N,reSort:function(){r?R(r,p):n&&N(n,p)},setGrouping:F,getGrouping:function(){return v},groupBy:function(t,e,n){F(null!=t?{getter:t,formatter:e,comparer:n}:[])},setAggregators:function(t,e){if(!v.length)throw new Error("At least one grouping must be specified before calling setAggregators().");v[0].aggregators=t,v[0].aggregateCollapsed=e,F(v)},collapseAllGroups:function(t){U(t,!0)},expandAllGroups:function(t){U(t,!1)},collapseGroup:function(t){var e=Array.prototype.slice.call(arguments),n=e[0];1==e.length&&-1!=n.indexOf(Q)?O(n.split(Q).length-1,n,!0):O(e.length-1,e.join(Q),!0)},expandGroup:function(t){var e=Array.prototype.slice.call(arguments),n=e[0];1==e.length&&-1!=n.indexOf(Q)?O(n.split(Q).length-1,n,!1):O(e.length-1,e.join(Q),!1)},getGroups:function(){return B},getIdxById:function(t){return u[t]},getRowById:function(t){return T(),g[t]},getItemById:function(t){return l[u[t]]},getItemByIdx:function(t){return l[t]},mapRowsToIds:function(t){for(var e=[],n=0,r=t.length;n<r;n++)t[n]<c.length&&(e[e.length]=c[t[n]][A]);return e},mapIdsToRows:function(t){var e=[];T();for(var n=0,r=t.length;n<r;n++){var i=g[t[n]];null!=i&&(e[e.length]=i)}return e},setRefreshHints:function(t){I=t},setFilterArgs:function(t){i=t},refresh:q,updateItem:function(t,e){if(void 0===u[t]||t!==e[A])throw"Invalid or non-matching id";l[u[t]]=e,d||(d={}),d[t]=!0,q()},insertItem:function(t,e){l.splice(t,0,e),S(t),q()},addItem:function(t){l.push(t),S(l.length-1),q()},deleteItem:function(t){var e=u[t];if(void 0===e)throw"Invalid id";delete u[t],l.splice(e,1),S(e),q()},syncGridSelection:function(t,n,r){var i,o=this,a=o.mapRowsToIds(t.getSelectedRows()),s=new Slick.Event;function A(e){a.join(",")!=e.join(",")&&(a=e,s.notify({grid:t,ids:a},new Slick.EventData,o))}function l(){if(a.length>0){i=!0;var e=o.mapIdsToRows(a);n||A(o.mapRowsToIds(e)),t.setSelectedRows(e),i=!1}}return t.onSelectedRowsChanged.subscribe(function(n,s){if(!i){var l=o.mapRowsToIds(t.getSelectedRows());r&&t.getOptions().multiSelect?A(e.grep(a,function(t){return void 0===o.getRowById(t)}).concat(l)):A(l)}}),this.onRowsChanged.subscribe(l),this.onRowCountChanged.subscribe(l),s},syncGridCellCssStyles:function(t,e){var n,r;function i(t){for(var e in n={},t){var r=c[e][A];n[r]=t[e]}}function o(){if(n){r=!0,T();var i={};for(var o in n){var a=g[o];null!=a&&(i[a]=n[o])}t.setCellCssStyles(e,i),r=!1}}i(t.getCellCssStyles(e)),t.onCellCssStylesChanged.subscribe(function(t,n){r||e==n.key&&n.hash&&i(n.hash)}),this.onRowsChanged.subscribe(o),this.onRowCountChanged.subscribe(o)},getLength:function(){return c.length},getItem:function(t){var e=c[t];if(e&&e.__group&&e.totals&&!e.totals.initialized){var n=v[e.level];n.displayTotalsRow||(G(e.totals),e.title=n.formatter?n.formatter(e):e.value)}else e&&e.__groupTotals&&!e.initialized&&G(e);return e},getItemMetadata:function(e){var n=c[e];return void 0===n?null:n.__group?t.groupItemMetadataProvider.getGroupRowMetadata(n):n.__groupTotals?t.groupItemMetadataProvider.getTotalsRowMetadata(n):null},onRowCountChanged:D,onRowsChanged:M,onPagingInfoChanged:k})},Aggregators:{Avg:function(t){this.field_=t,this.init=function(){this.count_=0,this.nonNullCount_=0,this.sum_=0},this.accumulate=function(t){var e=t[this.field_];this.count_++,null!=e&&""!==e&&NaN!==e&&(this.nonNullCount_++,this.sum_+=parseFloat(e))},this.storeResult=function(t){t.avg||(t.avg={}),0!=this.nonNullCount_&&(t.avg[this.field_]=this.sum_/this.nonNullCount_)}},Min:function(t){this.field_=t,this.init=function(){this.min_=null},this.accumulate=function(t){var e=t[this.field_];null!=e&&""!==e&&NaN!==e&&(null==this.min_||e<this.min_)&&(this.min_=e)},this.storeResult=function(t){t.min||(t.min={}),t.min[this.field_]=this.min_}},Max:function(t){this.field_=t,this.init=function(){this.max_=null},this.accumulate=function(t){var e=t[this.field_];null!=e&&""!==e&&NaN!==e&&(null==this.max_||e>this.max_)&&(this.max_=e)},this.storeResult=function(t){t.max||(t.max={}),t.max[this.field_]=this.max_}},Sum:function(t){this.field_=t,this.init=function(){this.sum_=null},this.accumulate=function(t){var e=t[this.field_];null!=e&&""!==e&&NaN!==e&&(this.sum_+=parseFloat(e))},this.storeResult=function(t){t.sum||(t.sum={}),t.sum[this.field_]=this.sum_}}}}}})}).call(this,n(5))},function(t,e,n){(function(t){var e;(e=t).extend(!0,window,{Slick:{Editors:{Text:function(t){var n,r;this.init=function(){n=e("<INPUT type=text class='editor-text' />").appendTo(t.container).bind("keydown.nav",function(t){"ArrowLeft"!==t.key&&"ArrowRight"!==t.key||t.stopImmediatePropagation()}).focus().select()},this.destroy=function(){n.remove()},this.focus=function(){n.focus()},this.getValue=function(){return n.val()},this.setValue=function(t){n.val(t)},this.loadValue=function(e){r=e[t.column.field]||"",n.val(r),n[0].defaultValue=r,n.select()},this.serializeValue=function(){return n.val()},this.applyValue=function(e,n){e[t.column.field]=n},this.isValueChanged=function(){return!(""==n.val()&&null==r)&&n.val()!=r},this.validate=function(){if(t.column.validator){var e=t.column.validator(n.val());if(!e.valid)return e}return{valid:!0,msg:null}},this.init()},Integer:function(t){var n,r;this.init=function(){(n=e("<INPUT type=text class='editor-text' />")).bind("keydown.nav",function(t){"ArrowLeft"!==t.key&&"ArrowRight"!==t.key||t.stopImmediatePropagation()}),n.appendTo(t.container),n.focus().select()},this.destroy=function(){n.remove()},this.focus=function(){n.focus()},this.loadValue=function(e){r=e[t.column.field],n.val(r),n[0].defaultValue=r,n.select()},this.serializeValue=function(){return parseInt(n.val(),10)||0},this.applyValue=function(e,n){e[t.column.field]=n},this.isValueChanged=function(){return!(""==n.val()&&null==r)&&n.val()!=r},this.validate=function(){return isNaN(n.val())?{valid:!1,msg:"Please enter a valid integer"}:{valid:!0,msg:null}},this.init()},Date:function(t){var n,r,i=!1;this.init=function(){(n=e("<INPUT type=text class='editor-text' />")).appendTo(t.container),n.focus().select(),n.datepicker({showOn:"button",buttonImageOnly:!0,buttonImage:"../images/calendar.gif",beforeShow:function(){i=!0},onClose:function(){i=!1}}),n.width(n.width()-18)},this.destroy=function(){e.datepicker.dpDiv.stop(!0,!0),n.datepicker("hide"),n.datepicker("destroy"),n.remove()},this.show=function(){i&&e.datepicker.dpDiv.stop(!0,!0).show()},this.hide=function(){i&&e.datepicker.dpDiv.stop(!0,!0).hide()},this.position=function(t){i&&e.datepicker.dpDiv.css("top",t.top+30).css("left",t.left)},this.focus=function(){n.focus()},this.loadValue=function(e){r=e[t.column.field],n.val(r),n[0].defaultValue=r,n.select()},this.serializeValue=function(){return n.val()},this.applyValue=function(e,n){e[t.column.field]=n},this.isValueChanged=function(){return!(""==n.val()&&null==r)&&n.val()!=r},this.validate=function(){return{valid:!0,msg:null}},this.init()},YesNoSelect:function(t){var n,r;this.init=function(){(n=e("<SELECT tabIndex='0' class='editor-yesno'><OPTION value='yes'>Yes</OPTION><OPTION value='no'>No</OPTION></SELECT>")).appendTo(t.container),n.focus()},this.destroy=function(){n.remove()},this.focus=function(){n.focus()},this.loadValue=function(e){n.val((r=e[t.column.field])?"yes":"no"),n.select()},this.serializeValue=function(){return"yes"==n.val()},this.applyValue=function(e,n){e[t.column.field]=n},this.isValueChanged=function(){return n.val()!=r},this.validate=function(){return{valid:!0,msg:null}},this.init()},Checkbox:function(t){var n,r;this.init=function(){(n=e("<INPUT type=checkbox value='true' class='editor-checkbox' hideFocus>")).appendTo(t.container),n.focus()},this.destroy=function(){n.remove()},this.focus=function(){n.focus()},this.loadValue=function(e){(r=!!e[t.column.field])?n.prop("checked",!0):n.prop("checked",!1)},this.serializeValue=function(){return n.prop("checked")},this.applyValue=function(e,n){e[t.column.field]=n},this.isValueChanged=function(){return this.serializeValue()!==r},this.validate=function(){return{valid:!0,msg:null}},this.init()},PercentComplete:function(t){var n,r,i;this.init=function(){(n=e("<INPUT type=text class='editor-percentcomplete' />")).width(e(t.container).innerWidth()-25),n.appendTo(t.container),(r=e("<div class='editor-percentcomplete-picker' />").appendTo(t.container)).append("<div class='editor-percentcomplete-helper'><div class='editor-percentcomplete-wrapper'><div class='editor-percentcomplete-slider' /><div class='editor-percentcomplete-buttons' /></div></div>"),r.find(".editor-percentcomplete-buttons").append("<button val=0>Not started</button><br/><button val=50>In Progress</button><br/><button val=100>Complete</button>"),n.focus().select(),r.find(".editor-percentcomplete-slider").slider({orientation:"vertical",range:"min",value:i,slide:function(t,e){n.val(e.value)}}),r.find(".editor-percentcomplete-buttons button").bind("click",function(t){n.val(e(this).attr("val")),r.find(".editor-percentcomplete-slider").slider("value",e(this).attr("val"))})},this.destroy=function(){n.remove(),r.remove()},this.focus=function(){n.focus()},this.loadValue=function(e){n.val(i=e[t.column.field]),n.select()},this.serializeValue=function(){return parseInt(n.val(),10)||0},this.applyValue=function(e,n){e[t.column.field]=n},this.isValueChanged=function(){return!(""==n.val()&&null==i)&&(parseInt(n.val(),10)||0)!=i},this.validate=function(){return isNaN(parseInt(n.val(),10))?{valid:!1,msg:"Please enter a valid positive number"}:{valid:!0,msg:null}},this.init()},LongText:function(t){var n,r,i,o=this;this.init=function(){var i=e("body");r=e("<DIV style='z-index:10000;position:absolute;background:white;padding:5px;border:3px solid gray; -moz-border-radius:10px; border-radius:10px;'/>").appendTo(i),n=e("<TEXTAREA hidefocus rows=5 style='backround:white;width:250px;height:80px;border:0;outline:0'>").appendTo(r),e("<DIV style='text-align:right'><BUTTON>Save</BUTTON><BUTTON>Cancel</BUTTON></DIV>").appendTo(r),r.find("button:first").bind("click",this.save),r.find("button:last").bind("click",this.cancel),n.bind("keydown",this.handleKeyDown),o.position(t.position),n.focus().select()},this.handleKeyDown=function(e){"Enter"==e.key&&e.ctrlKey?o.save():"Escape"==e.key?(e.preventDefault(),o.cancel()):"Tab"==e.key&&e.shiftKey?(e.preventDefault(),t.grid.navigatePrev()):"Tab"==e.key&&(e.preventDefault(),t.grid.navigateNext())},this.save=function(){t.commitChanges()},this.cancel=function(){n.val(i),t.cancelChanges()},this.hide=function(){r.hide()},this.show=function(){r.show()},this.position=function(t){r.css("top",t.top-5).css("left",t.left-5)},this.destroy=function(){r.remove()},this.focus=function(){n.focus()},this.loadValue=function(e){n.val(i=e[t.column.field]),n.select()},this.serializeValue=function(){return n.val()},this.applyValue=function(e,n){e[t.column.field]=n},this.isValueChanged=function(){return!(""==n.val()&&null==i)&&n.val()!=i},this.validate=function(){return{valid:!0,msg:null}},this.init()}}}})}).call(this,n(5))},function(module,exports,__webpack_require__){(function(jQuery){
/**
 * @license
 * (c) 2009-2013 Michael Leibman
 * michael{dot}leibman{at}gmail{dot}com
 * http://github.com/mleibman/slickgrid
 *
 * Distributed under MIT license.
 * All rights reserved.
 *
 * SlickGrid v2.2
 *
 * NOTES:
 *     Cell/row DOM manipulations are done directly bypassing jQuery's DOM manipulation methods.
 *     This increases the speed dramatically, but can only be done safely because there are no event handlers
 *     or data associated with any cell/row DOM nodes.  Cell editors must make sure they implement .destroy()
 *     and do proper cleanup.
 *
 */
if(void 0===jQuery)throw"SlickGrid requires jquery module to be loaded";if(!jQuery.fn.drag)throw"SlickGrid requires jquery.event.drag module to be loaded";if("undefined"==typeof Slick)throw"slick.core.js not loaded";!function($){var scrollbarDimensions,maxSupportedCssHeight;function SlickGrid(container,data,columns,options){var defaults={explicitInitialization:!1,rowHeight:25,defaultColumnWidth:80,enableAddRow:!1,leaveSpaceForNewRows:!1,editable:!1,autoEdit:!0,enableCellNavigation:!0,enableColumnReorder:!0,asyncEditorLoading:!1,asyncEditorLoadDelay:100,forceFitColumns:!1,enableAsyncPostRender:!1,asyncPostRenderDelay:5,autoHeight:!1,editorLock:Slick.GlobalEditorLock,showHeaderRow:!1,headerRowHeight:25,showTopPanel:!1,topPanelHeight:25,formatterFactory:null,editorFactory:null,cellFlashingCssClass:"flashing",selectedCellCssClass:"selected",multiSelect:!0,enableTextSelectionOnCells:!1,dataItemColumnValueExtractor:null,frozenBottom:!1,frozenColumn:-1,frozenRow:-1,fullWidthRows:!1,multiColumnSort:!1,defaultFormatter:defaultFormatter,forceSyncScrolling:!1,addNewRowCssClass:"new-row",linesPerScroll:3},columnDefaults={name:"",resizable:!0,sortable:!0,minWidth:30,rerenderOnResize:!1,headerCssClass:null,defaultSortAsc:!0,focusable:!0,selectable:!0},th,h,ph,n,cj,page=0,offset=0,vScrollDir=1,initialized=!1,$container,uid="slickgrid_"+Math.round(1e6*Math.random()),self=this,$focusSink,$focusSink2,$headerScroller,$headers,$headerRow,$headerRowScroller,$headerRowSpacerL,$headerRowSpacerR,$topPanelScroller,$topPanel,$viewport,$canvas,$style,$boundAncestors,stylesheet,columnCssRulesL,columnCssRulesR,viewportH,viewportW,canvasWidth,canvasWidthL,canvasWidthR,headersWidth,headersWidthL,headersWidthR,viewportHasHScroll,viewportHasVScroll,headerColumnWidthDiff=0,headerColumnHeightDiff=0,cellWidthDiff=0,cellHeightDiff=0,absoluteColumnMinWidth,hasFrozenRows=!1,frozenRowsHeight=0,actualFrozenRow=-1,paneTopH=0,paneBottomH=0,viewportTopH=0,viewportBottomH=0,topPanelH=0,groupPanelH=0,headerRowH=0,tabbingDirection=1,$activeCanvasNode,$activeViewportNode,activePosX,activeRow,activeCell,activeCellNode=null,currentEditor=null,serializedEditorValue,editController,rowsCache={},renderedRows=0,numVisibleRows=0,prevScrollTop=0,scrollTop=0,lastRenderedScrollTop=0,lastRenderedScrollLeft=0,prevScrollLeft=0,scrollLeft=0,selectionModel,selectedRows=[],plugins=[],cellCssClasses={},columnsById={},sortColumns=[],columnPosLeft=[],columnPosRight=[],h_editorLoader=null,h_render=null,h_postrender=null,postProcessedRows={},postProcessToRow=null,postProcessFromRow=null,counter_rows_rendered=0,counter_rows_removed=0,$paneGrouping,$paneHeaderL,$paneHeaderR,$paneTopL,$paneTopR,$paneBottomL,$paneBottomR,$headerScrollerL,$headerScrollerR,$headerL,$headerR,$headerRowScrollerL,$headerRowScrollerR,$headerRowL,$headerRowR,$topPanelScrollerL,$topPanelScrollerR,$topPanelL,$topPanelR,$viewportTopL,$viewportTopR,$viewportBottomL,$viewportBottomR,$canvasTopL,$canvasTopR,$canvasBottomL,$canvasBottomR,$viewportScrollContainerX,$viewportScrollContainerY,$headerScrollContainer,$headerRowScrollContainer;function init(){if(($container=$(container)).length<1)throw new Error("SlickGrid requires a valid container, "+container+" does not exist in the DOM.");maxSupportedCssHeight=maxSupportedCssHeight||getMaxSupportedCssHeight(),scrollbarDimensions=scrollbarDimensions||measureScrollbar(),options=$.extend({},defaults,options),validateAndEnforceOptions(),columnDefaults.width=options.defaultColumnWidth,columnsById={};for(var t=0;t<columns.length;t++){var e=columns[t]=$.extend({},columnDefaults,columns[t]);columnsById[e.id]=t,e.minWidth&&e.width<e.minWidth&&(e.width=e.minWidth),e.maxWidth&&e.width>e.maxWidth&&(e.width=e.maxWidth)}if(options.enableColumnReorder&&!$.fn.sortable)throw new Error("SlickGrid's 'enableColumnReorder = true' option requires jquery-ui.sortable module to be loaded");editController={commitCurrentEdit:commitCurrentEdit,cancelCurrentEdit:cancelCurrentEdit},$container.empty().css("overflow","hidden").css("outline",0).addClass(uid).addClass("ui-widget"),/relative|absolute|fixed/.test($container.css("position"))||$container.css("position","relative"),$focusSink=$("<div tabIndex='0' hideFocus style='position:fixed;width:0;height:0;top:0;left:0;outline:0;'></div>").appendTo($container),$paneGrouping=$("<div class='slick-pane slick-pane-grouping' tabIndex='0'/>").appendTo($container),$paneHeaderL=$("<div class='slick-pane slick-pane-header slick-pane-left' tabIndex='0' />").appendTo($container),$paneHeaderR=$("<div class='slick-pane slick-pane-header slick-pane-right' tabIndex='0' />").appendTo($container),$paneTopL=$("<div class='slick-pane slick-pane-top slick-pane-left' tabIndex='0' />").appendTo($container),$paneTopR=$("<div class='slick-pane slick-pane-top slick-pane-right' tabIndex='0' />").appendTo($container),$paneBottomL=$("<div class='slick-pane slick-pane-bottom slick-pane-left' tabIndex='0' />").appendTo($container),$paneBottomR=$("<div class='slick-pane slick-pane-bottom slick-pane-right' tabIndex='0' />").appendTo($container),$headerScrollerL=$("<div class='ui-state-default slick-header slick-header-left' />").appendTo($paneHeaderL),$headerScrollerR=$("<div class='ui-state-default slick-header slick-header-right' />").appendTo($paneHeaderR),$headerScroller=$().add($headerScrollerL).add($headerScrollerR),$headerL=$("<div class='slick-header-columns slick-header-columns-left' style='left:-1000px' />").appendTo($headerScrollerL),$headerR=$("<div class='slick-header-columns slick-header-columns-right' style='left:-1000px' />").appendTo($headerScrollerR),$headers=$().add($headerL).add($headerR),$headerRowScrollerL=$("<div class='ui-state-default slick-headerrow' />").appendTo($paneTopL),$headerRowScrollerR=$("<div class='ui-state-default slick-headerrow' />").appendTo($paneTopR),$headerRowScroller=$().add($headerRowScrollerL).add($headerRowScrollerR),$headerRowSpacerL=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").css("width",getCanvasWidth()+scrollbarDimensions.width+"px").appendTo($headerRowScrollerL),$headerRowSpacerR=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").css("width",getCanvasWidth()+scrollbarDimensions.width+"px").appendTo($headerRowScrollerR),$headerRowL=$("<div class='slick-headerrow-columns slick-headerrow-columns-left' />").appendTo($headerRowScrollerL),$headerRowR=$("<div class='slick-headerrow-columns slick-headerrow-columns-right' />").appendTo($headerRowScrollerR),$headerRow=$().add($headerRowL).add($headerRowR),$topPanelScrollerL=$("<div class='ui-state-default slick-top-panel-scroller' />").appendTo($paneTopL),$topPanelScrollerR=$("<div class='ui-state-default slick-top-panel-scroller' />").appendTo($paneTopR),$topPanelScroller=$().add($topPanelScrollerL).add($topPanelScrollerR),$topPanelL=$("<div class='slick-top-panel' style='width:10000px' />").appendTo($topPanelScrollerL),$topPanelR=$("<div class='slick-top-panel' style='width:10000px' />").appendTo($topPanelScrollerR),$topPanel=$().add($topPanelL).add($topPanelR),options.showTopPanel||$topPanelScroller.hide(),options.showHeaderRow||$headerRowScroller.hide(),$viewportTopL=$("<div class='slick-viewport slick-viewport-top slick-viewport-left' tabIndex='0' hideFocus />").appendTo($paneTopL),$viewportTopR=$("<div class='slick-viewport slick-viewport-top slick-viewport-right' tabIndex='0' hideFocus />").appendTo($paneTopR),$viewportBottomL=$("<div class='slick-viewport slick-viewport-bottom slick-viewport-left' tabIndex='0' hideFocus />").appendTo($paneBottomL),$viewportBottomR=$("<div class='slick-viewport slick-viewport-bottom slick-viewport-right' tabIndex='0' hideFocus />").appendTo($paneBottomR),$viewport=$().add($viewportTopL).add($viewportTopR).add($viewportBottomL).add($viewportBottomR),$activeViewportNode=$viewportTopL,$canvasTopL=$("<div class='grid-canvas grid-canvas-top grid-canvas-left' tabIndex='0' hideFocus />").appendTo($viewportTopL),$canvasTopR=$("<div class='grid-canvas grid-canvas-top grid-canvas-right' tabIndex='0' hideFocus />").appendTo($viewportTopR),$canvasBottomL=$("<div class='grid-canvas grid-canvas-bottom grid-canvas-left' tabIndex='0' hideFocus />").appendTo($viewportBottomL),$canvasBottomR=$("<div class='grid-canvas grid-canvas-bottom grid-canvas-right' tabIndex='0' hideFocus />").appendTo($viewportBottomR),$canvas=$().add($canvasTopL).add($canvasTopR).add($canvasBottomL).add($canvasBottomR),$activeCanvasNode=$canvasTopL,$focusSink2=$focusSink.clone().appendTo($container),options.explicitInitialization||finishInitialization()}function finishInitialization(){initialized||(initialized=!0,getViewportWidth(),getViewportHeight(),measureCellPaddingAndBorder(),disableSelection($headers),options.enableTextSelectionOnCells||$viewport.bind("selectstart.ui",function(t){return $(t.target).is("input,textarea")}),setFrozenOptions(),setPaneVisibility(),setScroller(),setOverflow(),updateColumnCaches(),createColumnHeaders(),setupColumnSort(),createCssRules(),resizeCanvas(),bindAncestorScrollEvents(),$container.bind("resize.slickgrid",resizeCanvas),$viewport.bind("scroll",handleScroll),jQuery.fn.mousewheel&&(options.frozenColumn>-1||hasFrozenRows)&&$viewport.bind("mousewheel",handleMouseWheel),$headerScroller.bind("contextmenu",handleHeaderContextMenu).bind("click",handleHeaderClick).delegate(".slick-header-column","mouseenter",handleHeaderMouseEnter).delegate(".slick-header-column","mouseleave",handleHeaderMouseLeave),$headerRowScroller.bind("scroll",handleHeaderRowScroll),$focusSink.add($focusSink2).bind("keydown",handleKeyDown),$canvas.bind("keydown",handleKeyDown).bind("click",handleClick).bind("mousedown",handleMouseDown).bind("dblclick",handleDblClick).bind("contextmenu",handleContextMenu).bind("draginit",handleDragInit).bind("dragstart",{distance:3},handleDragStart).bind("drag",handleDrag).bind("dragend",handleDragEnd).delegate(".slick-cell","mouseenter",handleMouseEnter).delegate(".slick-cell","mouseleave",handleMouseLeave))}function registerPlugin(t){plugins.unshift(t),t.init(self)}function unregisterPlugin(t){for(var e=plugins.length;e>=0;e--)if(plugins[e]===t){plugins[e].destroy&&plugins[e].destroy(),plugins.splice(e,1);break}}function setSelectionModel(t){selectionModel&&(selectionModel.onSelectedRangesChanged.unsubscribe(handleSelectedRangesChanged),selectionModel.destroy&&selectionModel.destroy()),(selectionModel=t)&&(selectionModel.init(self),selectionModel.onSelectedRangesChanged.subscribe(handleSelectedRangesChanged))}function getSelectionModel(){return selectionModel}function getCanvasNode(){return $canvas[0]}function getActiveCanvasNode(t){return setActiveCanvasNode(t),$activeCanvasNode[0]}function getCanvases(){return $canvas}function setActiveCanvasNode(t){t&&($activeCanvasNode=$(t.target).closest(".grid-canvas"))}function getViewportNode(){return $viewport[0]}function getActiveViewportNode(t){return setActiveViewPortNode(t),$activeViewportNode[0]}function setActiveViewportNode(t){t&&($activeViewportNode=$(t.target).closest(".slick-viewport"))}function measureScrollbar(){var t=$("<div style='position:absolute; top:-10000px; left:-10000px; width:100px; height:100px; overflow:scroll;'></div>").appendTo("body"),e={width:t.width()-t[0].clientWidth,height:t.height()-t[0].clientHeight};return t.remove(),e}function getHeadersWidth(){headersWidth=headersWidthL=headersWidthR=0;for(var t=0,e=columns.length;t<e;t++){var n=columns[t].width;options.frozenColumn>-1&&t>options.frozenColumn?headersWidthR+=n:headersWidthL+=n}options.frozenColumn>-1?(headersWidthL+=1e3,headersWidthR=Math.max(headersWidthR,viewportW)+headersWidthL,headersWidthR+=scrollbarDimensions.width):(headersWidthL+=scrollbarDimensions.width,headersWidthL=Math.max(headersWidthL,viewportW)+1e3),headersWidth=headersWidthL+headersWidthR}function getCanvasWidth(){var t=viewportHasVScroll?viewportW-scrollbarDimensions.width:viewportW,e=columns.length;for(canvasWidthL=canvasWidthR=0;e--;)options.frozenColumn>-1&&e>options.frozenColumn?canvasWidthR+=columns[e].width:canvasWidthL+=columns[e].width;var n=canvasWidthL+canvasWidthR;return options.fullWidthRows?Math.max(n,t):n}function updateCanvasWidth(t){var e,n=canvasWidth,r=canvasWidthL,i=canvasWidthR;((e=(canvasWidth=getCanvasWidth())!==n||canvasWidthL!==r||canvasWidthR!==i)||options.frozenColumn>-1||hasFrozenRows)&&($canvasTopL.width(canvasWidthL),getHeadersWidth(),$headerL.width(headersWidthL),$headerR.width(headersWidthR),options.frozenColumn>-1?($canvasTopR.width(canvasWidthR),$paneHeaderL.width(canvasWidthL),$paneHeaderR.css("left",canvasWidthL),$paneHeaderR.css("width",viewportW-canvasWidthL),$paneTopL.width(canvasWidthL-1).css("borderRight","1px solid #696969"),$paneTopR.css("left",canvasWidthL),$paneTopR.css("width",viewportW-canvasWidthL),$headerRowScrollerL.width(canvasWidthL),$headerRowScrollerR.width(viewportW-canvasWidthL),$headerRowL.width(canvasWidthL),$headerRowR.width(canvasWidthR),$viewportTopL.width(canvasWidthL),$viewportTopR.width(viewportW-canvasWidthL),hasFrozenRows&&($paneBottomL.width(canvasWidthL),$paneBottomR.css("left",canvasWidthL),$viewportBottomL.width(canvasWidthL),$viewportBottomR.width(viewportW-canvasWidthL),$canvasBottomL.width(canvasWidthL),$canvasBottomR.width(canvasWidthR))):($paneHeaderL.width("100%"),$paneTopL.width("100%"),$headerRowScrollerL.width("100%"),$headerRowL.width(canvasWidth),$viewportTopL.width("100%"),hasFrozenRows&&($viewportBottomL.width("100%"),$canvasBottomL.width(canvasWidthL))),viewportHasHScroll=canvasWidth>viewportW-scrollbarDimensions.width),$headerRowSpacerL.width(canvasWidth+(viewportHasVScroll?scrollbarDimensions.width:0)),$headerRowSpacerR.width(canvasWidth+(viewportHasVScroll?scrollbarDimensions.width:0)),(e||t)&&applyColumnWidths()}function disableSelection(t){t&&t.jquery&&t.attr("unselectable","on").css("MozUserSelect","none").bind("selectstart.ui",function(){return!1})}function getMaxSupportedCssHeight(){for(var t=1e6,e=navigator.userAgent.toLowerCase().match(/firefox/)?6e6:1e9,n=$("<div style='display:none' />").appendTo(document.body);;){var r=2*t;if(n.css("height",r),r>e||n.height()!==r)break;t=r}return n.remove(),t}function bindAncestorScrollEvents(){for(var t=hasFrozenRows&&!options.frozenBottom?$canvasBottomL[0]:$canvasTopL[0];(t=t.parentNode)!=document.body&&null!=t;)if(t==$viewportTopL[0]||t.scrollWidth!=t.clientWidth||t.scrollHeight!=t.clientHeight){var e=$(t);$boundAncestors=$boundAncestors?$boundAncestors.add(e):e,e.bind("scroll."+uid,handleActiveCellPositionChange)}}function unbindAncestorScrollEvents(){$boundAncestors&&($boundAncestors.unbind("scroll."+uid),$boundAncestors=null)}function updateColumnHeader(t,e,n){if(initialized){var r=getColumnIndex(t);if(null!=r){var i=columns[r],o=$headers.children().eq(r);o&&(void 0!==e&&(columns[r].name=e),void 0!==n&&(columns[r].toolTip=n),trigger(self.onBeforeHeaderCellDestroy,{node:o[0],column:i}),o.attr("title",n||"").children().eq(0).html(e),trigger(self.onHeaderCellRendered,{node:o[0],column:i}))}}}function getHeaderRow(){return options.frozenColumn>-1?$headerRow:$headerRow[0]}function getHeaderRowColumn(t){var e,n=getColumnIndex(t);options.frozenColumn>-1?n<=options.frozenColumn?e=$headerRowL:(e=$headerRowR,n-=options.frozenColumn+1):e=$headerRowL;var r=e.children().eq(n);return r&&r[0]}function createColumnHeaders(){function t(){$(this).addClass("ui-state-hover")}function e(){$(this).removeClass("ui-state-hover")}$headers.find(".slick-header-column").each(function(){var t=$(this).data("column");t&&trigger(self.onBeforeHeaderCellDestroy,{node:this,column:t})}),$headerL.empty(),$headerR.empty(),getHeadersWidth(),$headerL.width(headersWidthL),$headerR.width(headersWidthR),$headerRow.find(".slick-headerrow-column").each(function(){var t=$(this).data("column");t&&trigger(self.onBeforeHeaderRowCellDestroy,{node:this,column:t})}),$headerRowL.empty(),$headerRowR.empty();for(var n=Math.max.apply(null,columns.map(function(t){return null!=t.columnGroups?t.columnGroups.length:0})),r=0;r<columns.length;r++){var i=columns[r],o=options.frozenColumn>-1?r<=options.frozenColumn?$headerL:$headerR:$headerL,a=options.frozenColumn>-1?r<=options.frozenColumn?$headerRowL:$headerRowR:$headerRowL,s=$("<div class='slick-header-columngroup ui-state-default slick-header-column' />").data("column",i).width(i.width-headerColumnWidthDiff).height(18*(n+1)).attr("id",""+uid+i.id).attr("title",i.toolTip||"").addClass(i.headerCssClass||"").appendTo(o);(0==i.width||i.collapsed)&&s.attr("style","display:none;");for(var A,l,c=$("<table></table>").css({borderCollapse:"collapse",paddingTop:"2px",width:"path"===i.id?"":"100%"}).appendTo(s),u=0;u<n;u++){A=$("<tr></tr>").appendTo(c),l=$("<td></td>").height(18).width("path"===i.id?"":i.width-headerColumnWidthDiff).appendTo(A);var g=null!=i.columnGroups&&u<i.columnGroups.length?i.columnGroups[u]:null;if(null!=g){var h=r>0&&null!=columns[r-1].columnGroups?columns[r-1].columnGroups:null,d=(r+1<columns.length&&columns[r+1].columnGroups,null==h||u>=h.length||h[u]!=g);A.addClass("column-group-level"+u),d&&($("<span class='toggle "+(columns[r+1]&&columns[r+1].collapsed?"expand":"collapse")+"'></span>").css({float:"left",paddingRight:5}).data("groupNum",u).appendTo(l),$("<span class='slick-column-name'>"+g+"</span>").appendTo(l))}}if(A=$("<tr></tr>").appendTo(c),l=$("<td></td>").height(18).width("path"===i.id?"":i.width-headerColumnWidthDiff).appendTo(A),$("<span class='slick-column-name'>"+i.name+"</span>").attr("id",""+uid+i.id).attr("title",i.toolTip||"").data("column",i).addClass(i.headerCssClass||"").css("paddingBottom","8px").appendTo(l),(options.enableColumnReorder||i.sortable)&&s.on("mouseenter",t).on("mouseleave",e),i.sortable&&(s.addClass("slick-header-sortable"),s.append("<span class='slick-sort-indicator' />")),trigger(self.onHeaderCellRendered,{node:s[0],column:i}),options.showHeaderRow){var f=$("<div class='ui-state-default slick-headerrow-column l"+r+" r"+r+"'></div>").data("column",i).appendTo(a);trigger(self.onHeaderRowCellRendered,{node:f[0],column:i})}}setSortColumns(sortColumns),setupColumnResize(),options.enableColumnReorder&&setupColumnReorder()}function setupColumnSort(){$headers.click(function(t){if(t.metaKey=t.metaKey||t.ctrlKey,!$(t.target).hasClass("slick-resizable-handle")){var e=$(t.target).closest(".slick-header-column");if(e.length){var n=e.data("column");if(n.sortable){if(!getEditorLock().commitCurrentEdit())return;for(var r=null,i=0;i<sortColumns.length;i++)if(sortColumns[i].columnId==n.id){(r=sortColumns[i]).sortAsc=!r.sortAsc;break}t.metaKey&&options.multiColumnSort?r&&sortColumns.splice(i,1):((t.shiftKey||t.metaKey)&&options.multiColumnSort||(sortColumns=[]),r?0==sortColumns.length&&sortColumns.push(r):(r={columnId:n.id,sortAsc:!0},sortColumns.push(r))),options.multiColumnSort?trigger(self.onSort,{multiColumnSort:!0,sortCols:$.map(sortColumns,function(t){return{sortCol:columns[getColumnIndex(t.columnId)],sortAsc:t.sortAsc}})},t):trigger(self.onSort,{multiColumnSort:!1,sortCol:n,sortAsc:r.sortAsc},t)}}}})}function setupColumnReorder(){$headers.filter(":ui-sortable").sortable("destroy");var t=null;function e(){$viewportScrollContainerX[0].scrollLeft=$viewportScrollContainerX[0].scrollLeft+10}function n(){$viewportScrollContainerX[0].scrollLeft=$viewportScrollContainerX[0].scrollLeft-10}$headers.sortable({containment:"parent",distance:3,axis:"x",cursor:"default",tolerance:"intersection",helper:"clone",placeholder:"slick-sortable-placeholder ui-state-default slick-header-column",start:function(t,e){e.placeholder.width(e.helper.outerWidth()-headerColumnWidthDiff),$(e.helper).addClass("slick-header-column-active")},beforeStop:function(t,e){$(e.helper).removeClass("slick-header-column-active")},sort:function(r,i){r.originalEvent.pageX>$container[0].clientWidth?t||(t=setInterval(e,100)):r.originalEvent.pageX<$viewportScrollContainerX.offset().left?t||(t=setInterval(n,100)):(clearInterval(t),t=null)},stop:function(e){if(clearInterval(t),t=null,getEditorLock().commitCurrentEdit()){var n=$headerL.sortable("toArray");n=n.concat($headerR.sortable("toArray"));for(var r=[],i=0;i<n.length;i++)r.push(columns[getColumnIndex(n[i].replace(uid,""))]);setColumns(r),trigger(self.onColumnsReordered,{}),e.stopPropagation(),setupColumnResize()}else $(this).sortable("cancel")}})}function setupColumnResize(){var t,e,n,r,i,o,a,s;(r=$headers.children()).find(".slick-resizable-handle").remove(),r.each(function(t,e){columns[t].resizable&&(void 0===a&&(a=t),s=t)}),void 0!==a&&r.each(function(A,l){A<a||options.forceFitColumns&&A>=s||($(l),$("<div class='slick-resizable-handle' />").appendTo(l).bind("dragstart",function(a,s){if(!getEditorLock().commitCurrentEdit())return!1;n=a.pageX,$(this).parent().addClass("slick-header-column-active");var l=null,c=null;if(r.each(function(t,e){columns[t].previousWidth=$(e).outerWidth()}),options.forceFitColumns)for(l=0,c=0,t=A+1;t<r.length;t++)(e=columns[t]).resizable&&(null!==c&&(e.maxWidth?c+=e.maxWidth-e.previousWidth:c=null),l+=e.previousWidth-Math.max(e.minWidth||0,absoluteColumnMinWidth));var u=0,g=0;for(t=0;t<=A;t++)(e=columns[t]).resizable&&(null!==g&&(e.maxWidth?g+=e.maxWidth-e.previousWidth:g=null),u+=e.previousWidth-Math.max(e.minWidth||0,absoluteColumnMinWidth));null===l&&(l=1e5),null===u&&(u=1e5),null===c&&(c=1e5),null===g&&(g=1e5),o=n+Math.min(l,g),i=n-Math.min(u,c)}).bind("drag",function(a,s){var l,c,u=Math.min(o,Math.max(i,a.pageX))-n;if(u<0){c=u;var g=0;for(t=A;t>=0;t--)(e=columns[t]).resizable&&(l=Math.max(e.minWidth||0,absoluteColumnMinWidth),c&&e.previousWidth+c<l?(c+=e.previousWidth-l,e.width=l):(e.width=e.previousWidth+c,c=0));for(k=0;k<=A;k++)e=columns[k],options.frozenColumn>-1&&k>options.frozenColumn?e.width:g+=e.width;if(options.forceFitColumns)for(c=-u,t=A+1;t<r.length;t++)(e=columns[t]).resizable&&(c&&e.maxWidth&&e.maxWidth-e.previousWidth<c?(c-=e.maxWidth-e.previousWidth,e.width=e.maxWidth):(e.width=e.previousWidth+c,c=0),options.frozenColumn>-1&&t>options.frozenColumn?e.width:g+=e.width);else for(t=A+1;t<r.length;t++)e=columns[t],options.frozenColumn>-1&&t>options.frozenColumn?e.width:g+=e.width}else{for(c=u,g=0,t=A;t>=0;t--)(e=columns[t]).resizable&&(c&&e.maxWidth&&e.maxWidth-e.previousWidth<c?(c-=e.maxWidth-e.previousWidth,e.width=e.maxWidth):(e.width=e.previousWidth+c,c=0));for(k=0;k<=A;k++)e=columns[k],options.frozenColumn>-1&&k>options.frozenColumn?e.width:g+=e.width;if(options.forceFitColumns)for(c=-u,t=A+1;t<r.length;t++)(e=columns[t]).resizable&&(l=Math.max(e.minWidth||0,absoluteColumnMinWidth),c&&e.previousWidth+c<l?(c+=e.previousWidth-l,e.width=l):(e.width=e.previousWidth+c,c=0),options.frozenColumn>-1&&t>options.frozenColumn?e.width:g+=e.width);else for(t=A+1;t<r.length;t++)e=columns[t],options.frozenColumn>-1&&t>options.frozenColumn?e.width:g+=e.width}options.frozenColumn>-1&&g!=canvasWidthL&&($headerL.width(g+1e3),$paneHeaderR.css("left",g)),applyColumnHeaderWidths(),options.syncColumnCellResize&&(updateCanvasWidth(),applyColumnWidths())}).bind("dragend",function(n,i){var o;for($(this).parent().removeClass("slick-header-column-active"),t=0;t<r.length;t++)e=columns[t],o=$(r[t]).outerWidth(),e.previousWidth!==o&&e.rerenderOnResize&&invalidateAllRows();updateCanvasWidth(!0),render(),trigger(self.onColumnsResized,{})}))})}function getVBoxDelta(t){var e=0;return $.each(["borderTopWidth","borderBottomWidth","paddingTop","paddingBottom"],function(n,r){e+=parseFloat(t.css(r))||0}),e}function setFrozenOptions(){if(options.frozenColumn=options.frozenColumn>=0&&options.frozenColumn<columns.length?parseInt(options.frozenColumn):-1,options.frozenRow=options.frozenRow>=0&&options.frozenRow<numVisibleRows?parseInt(options.frozenRow):-1,options.frozenRow>-1){hasFrozenRows=!0,frozenRowsHeight=options.frozenRow*options.rowHeight;var t=getDataLength()||this.data.length;actualFrozenRow=options.frozenBottom?t-options.frozenRow:options.frozenRow}else hasFrozenRows=!1}function setPaneVisibility(){options.frozenColumn>-1?($paneHeaderR.show(),$paneTopR.show(),hasFrozenRows?($paneBottomL.show(),$paneBottomR.show()):($paneBottomR.hide(),$paneBottomL.hide())):($paneHeaderR.hide(),$paneTopR.hide(),$paneBottomR.hide(),hasFrozenRows?$paneBottomL.show():($paneBottomR.hide(),$paneBottomL.hide()))}function setOverflow(){$viewportTopL.css({"overflow-x":options.frozenColumn>-1?hasFrozenRows?"hidden":"scroll":hasFrozenRows?"hidden":"auto","overflow-y":options.frozenColumn>-1?"hidden":hasFrozenRows?"scroll":"auto"}),$viewportTopR.css({"overflow-x":options.frozenColumn>-1?hasFrozenRows?"hidden":"scroll":hasFrozenRows?"hidden":"auto","overflow-y":(options.frozenColumn,hasFrozenRows?"scroll":"auto")}),$viewportBottomL.css({"overflow-x":options.frozenColumn>-1&&hasFrozenRows?"scroll":"auto","overflow-y":options.frozenColumn>-1?"hidden":hasFrozenRows?"scroll":"auto"}),$viewportBottomR.css({"overflow-x":options.frozenColumn>-1&&hasFrozenRows?"scroll":"auto","overflow-y":(options.frozenColumn,"auto")})}function setScroller(){options.frozenColumn>-1?($headerScrollContainer=$headerScrollerR,$headerRowScrollContainer=$headerRowScrollerR,hasFrozenRows?options.frozenBottom?($viewportScrollContainerX=$viewportBottomR,$viewportScrollContainerY=$viewportTopR):$viewportScrollContainerX=$viewportScrollContainerY=$viewportBottomR:$viewportScrollContainerX=$viewportScrollContainerY=$viewportTopR):($headerScrollContainer=$headerScrollerL,$headerRowScrollContainer=$headerRowScrollerL,hasFrozenRows?options.frozenBottom?($viewportScrollContainerX=$viewportBottomL,$viewportScrollContainerY=$viewportTopL):$viewportScrollContainerX=$viewportScrollContainerY=$viewportBottomL:$viewportScrollContainerX=$viewportScrollContainerY=$viewportTopL)}function measureCellPaddingAndBorder(){var t,e=["borderLeftWidth","borderRightWidth","paddingLeft","paddingRight"],n=["borderTopWidth","borderBottomWidth","paddingTop","paddingBottom"];t=$("<div class='ui-state-default slick-header-column' style='visibility:hidden'>-</div>").appendTo($headers),headerColumnWidthDiff=headerColumnHeightDiff=0,"border-box"!=t.css("box-sizing")&&"border-box"!=t.css("-moz-box-sizing")&&"border-box"!=t.css("-webkit-box-sizing")&&($.each(e,function(e,n){headerColumnWidthDiff+=parseFloat(t.css(n))||0}),$.each(n,function(e,n){headerColumnHeightDiff+=parseFloat(t.css(n))||0})),t.remove();var r=$("<div class='slick-row' />").appendTo($canvas);t=$("<div class='slick-cell' id='' style='visibility:hidden'>-</div>").appendTo(r),cellWidthDiff=cellHeightDiff=0,"border-box"!=t.css("box-sizing")&&"border-box"!=t.css("-moz-box-sizing")&&"border-box"!=t.css("-webkit-box-sizing")&&($.each(e,function(e,n){cellWidthDiff+=parseFloat(t.css(n))||0}),$.each(n,function(e,n){cellHeightDiff+=parseFloat(t.css(n))||0})),r.remove(),absoluteColumnMinWidth=Math.max(headerColumnWidthDiff,cellWidthDiff)}function createCssRules(){$style=$("<style type='text/css' rel='stylesheet' />").appendTo($("head"));for(var t=options.rowHeight-cellHeightDiff,e=["."+uid+" .slick-header-column { left: 1000px; }","."+uid+" .slick-top-panel { height:"+options.topPanelHeight+"px; }","."+uid+" .slick-headerrow-columns { height:"+options.headerRowHeight+"px; }","."+uid+" .slick-cell { height:"+t+"px; }","."+uid+" .slick-row { height:"+options.rowHeight+"px; }"],n=0;n<columns.length;n++)e.push("."+uid+" .l"+n+" { }"),e.push("."+uid+" .r"+n+" { }");$style[0].styleSheet?$style[0].styleSheet.cssText=e.join(" "):$style[0].appendChild(document.createTextNode(e.join(" ")))}function getColumnCssRules(t){var e;if(!stylesheet){var n=document.querySelectorAll("style");for(e=0;e<n.length;e++)if((n[e].sheet.ownerNode||n[e].sheet.owningElement)==$style[0]){stylesheet=n[e].sheet;break}if(!stylesheet)throw new Error("Cannot find stylesheet.");columnCssRulesL=[],columnCssRulesR=[];var r,i,o=stylesheet.cssRules||stylesheet.rules;for(e=0;e<o.length;e++){var a=o[e].selectorText;(r=/\.l\d+/.exec(a))?(i=parseInt(r[0].substr(2,r[0].length-2),10),columnCssRulesL[i]=o[e]):(r=/\.r\d+/.exec(a))&&(i=parseInt(r[0].substr(2,r[0].length-2),10),columnCssRulesR[i]=o[e])}}return{left:columnCssRulesL[t],right:columnCssRulesR[t]}}function removeCssRules(){$style.remove(),stylesheet=null}function destroy(){getEditorLock().cancelCurrentEdit(),trigger(self.onBeforeDestroy,{});for(var t=plugins.length;t--;)unregisterPlugin(plugins[t]);options.enableColumnReorder&&$headers.filter(":ui-sortable").sortable("destroy"),unbindAncestorScrollEvents(),$container.unbind(".slickgrid"),removeCssRules(),$canvas.unbind("draginit dragstart dragend drag"),$container.empty().removeClass(uid)}function trigger(t,e,n){return n=n||new Slick.EventData,(e=e||{}).grid=self,t.notify(e,n,self)}function getEditorLock(){return options.editorLock}function getEditController(){return editController}function getColumnIndex(t){return columnsById[t]}function autosizeColumns(){var t,e,n,r=[],i=0,o=0,a=viewportHasVScroll?viewportW-scrollbarDimensions.width:viewportW;for(t=0;t<columns.length;t++)e=columns[t],r.push(e.width),o+=e.width,e.resizable&&(i+=e.width-Math.max(e.minWidth,absoluteColumnMinWidth));for(n=o;o>a&&i;){var s=(o-a)/i;for(t=0;t<columns.length&&o>a;t++){e=columns[t];var A=r[t];if(!(!e.resizable||A<=e.minWidth||A<=absoluteColumnMinWidth)){var l=Math.max(e.minWidth,absoluteColumnMinWidth),c=Math.floor(s*(A-l))||1;o-=c=Math.min(c,A-l),i-=c,r[t]-=c}}if(n==o)break;n=o}for(n=o;o<a;){var u=a/o;for(t=0;t<columns.length&&o<a;t++)if((e=columns[t]).resizable&&!(e.maxWidth<=e.width)){var g=Math.min(Math.floor(u*e.width)-e.width,e.maxWidth-e.width||1e6)||1;o+=g,r[t]+=g}if(n==o)break;n=o}var h=!1;for(t=0;t<columns.length;t++)columns[t].rerenderOnResize&&columns[t].width!=r[t]&&(h=!0),columns[t].width=r[t];applyColumnHeaderWidths(),updateCanvasWidth(!0),h&&(invalidateAllRows(),render())}function applyColumnHeaderWidths(){if(initialized){for(var t,e=0,n=$headers.children(),r=n.length;e<r;e++)(t=$(n[e])).width()!==columns[e].width-headerColumnWidthDiff&&t.width(columns[e].width-headerColumnWidthDiff);updateColumnCaches()}}function applyColumnWidths(){for(var t,e,n=0,r=0;r<columns.length;r++)t=columns[r].width,(e=getColumnCssRules(r)).left.style.left=n+"px",e.right.style.right=(-1!=options.frozenColumn&&r>options.frozenColumn?canvasWidthR:canvasWidthL)-n-t+"px",options.frozenColumn==r?n=0:n+=columns[r].width}function setSortColumn(t,e){setSortColumns([{columnId:t,sortAsc:e}])}function setSortColumns(t){sortColumns=t;var e=$headers.children();e.removeClass("slick-header-column-sorted").find(".slick-sort-indicator").removeClass("slick-sort-indicator-asc slick-sort-indicator-desc"),$.each(sortColumns,function(t,n){null==n.sortAsc&&(n.sortAsc=!0);var r=getColumnIndex(n.columnId);null!=r&&e.eq(r).addClass("slick-header-column-sorted").find(".slick-sort-indicator").addClass(n.sortAsc?"slick-sort-indicator-asc":"slick-sort-indicator-desc")})}function getSortColumns(){return sortColumns}function handleSelectedRangesChanged(t,e){selectedRows=[];for(var n={},r=0;r<e.length;r++)for(var i=e[r].fromRow;i<=e[r].toRow;i++){n[i]||(selectedRows.push(i),n[i]={});for(var o=e[r].fromCell;o<=e[r].toCell;o++)canCellBeSelected(i,o)&&(n[i][columns[o].id]=options.selectedCellCssClass)}setCellCssStyles(options.selectedCellCssClass,n),trigger(self.onSelectedRowsChanged,{rows:getSelectedRows()},t)}function getColumns(){return columns}function updateColumnCaches(){columnPosLeft=[],columnPosRight=[];for(var t=0,e=0,n=columns.length;e<n;e++)columnPosLeft[e]=t,columnPosRight[e]=t+columns[e].width,options.frozenColumn==e?t=0:t+=columns[e].width}function setColumns(t){columns=t,columnsById={};for(var e=0;e<columns.length;e++){var n=columns[e]=$.extend({},columnDefaults,columns[e]);columnsById[n.id]=e,n.minWidth&&n.width<n.minWidth&&(n.width=n.minWidth),n.maxWidth&&n.width>n.maxWidth&&(n.width=n.maxWidth)}updateColumnCaches(),initialized&&(setPaneVisibility(),setOverflow(),invalidateAllRows(),createColumnHeaders(),removeCssRules(),createCssRules(),resizeCanvas(),updateCanvasWidth(),applyColumnWidths(),handleScroll())}function getOptions(){return options}function setOptions(t){getEditorLock().commitCurrentEdit()&&(makeActiveCellNormal(),options.enableAddRow!==t.enableAddRow&&invalidateRow(getDataLength()),options=$.extend(options,t),validateAndEnforceOptions(),setFrozenOptions(),setScroller(),setColumns(columns),render())}function validateAndEnforceOptions(){options.autoHeight&&(options.leaveSpaceForNewRows=!1)}function setData(t,e){data=t,invalidateAllRows(),updateRowCount(),e&&scrollTo(0)}function getData(){return data}function getDataLength(){return data.getLength?data.getLength():data.length}function getDataLengthIncludingAddNew(){return getDataLength()+(options.enableAddRow?1:0)}function getDataItem(t){return data.getItem?data.getItem(t):data[t]}function getTopPanel(){return $topPanel[0]}function setTopPanelVisibility(t){options.showTopPanel!=t&&(options.showTopPanel=t,t?$topPanelScroller.slideDown("fast",resizeCanvas):$topPanelScroller.slideUp("fast",resizeCanvas))}function setHeaderRowVisibility(t){options.showHeaderRow!=t&&(options.showHeaderRow=t,t?$headerRowScroller.slideDown("fast",resizeCanvas):$headerRowScroller.slideUp("fast",resizeCanvas))}function getContainerNode(){return $container.get(0)}function getRowTop(t){return options.rowHeight*t-offset}function getRowFromPosition(t){return Math.floor((t+offset)/options.rowHeight)}function scrollTo(t){t=Math.max(t,0),t=Math.min(t,th-$viewportScrollContainerY.height()+(viewportHasHScroll||options.frozenColumn>-1?scrollbarDimensions.height:0));var e=offset;page=Math.min(n-1,Math.floor(t/ph));var r=t-(offset=Math.round(page*cj));offset!=e&&(cleanupRows(getVisibleRange(r)),updateRowPositions()),prevScrollTop!=r&&(vScrollDir=prevScrollTop+e<r+offset?1:-1,lastRenderedScrollTop=scrollTop=prevScrollTop=r,options.frozenColumn>-1&&($viewportTopL[0].scrollTop=r),hasFrozenRows&&($viewportBottomL[0].scrollTop=$viewportBottomR[0].scrollTop=r),$viewportScrollContainerY[0].scrollTop=r,trigger(self.onViewportChanged,{}))}function defaultFormatter(t,e,n,r,i){return null==n?"":(n+"").replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;")}function getFormatter(t,e){var n=data.getItemMetadata&&data.getItemMetadata(t),r=n&&n.columns&&(n.columns[e.id]||n.columns[getColumnIndex(e.id)]);return r&&r.formatter||n&&n.formatter||e.formatter||options.formatterFactory&&options.formatterFactory.getFormatter(e)||options.defaultFormatter}function getFormatters(t,e){return e.formatters||[getFormatter(t,e)]}function getEditor(t,e){var n=columns[e],r=data.getItemMetadata&&data.getItemMetadata(t),i=r&&r.columns;return i&&i[n.id]&&void 0!==i[n.id].editor?i[n.id].editor:i&&i[e]&&void 0!==i[e].editor?i[e].editor:n.editor||options.editorFactory&&options.editorFactory.getEditor(n)}function getDataItemValueForColumn(t,e){return options.dataItemColumnValueExtractor?options.dataItemColumnValueExtractor(t,e):t[e.field]}function appendRowHtml(t,e,n,r,i){var o=getDataItem(n),a="slick-row"+(n<i&&!o?" loading":"")+(n===activeRow?" active":"")+(n%2==1?" odd":" even");o||(a+=" "+options.addNewRowCssClass);var s=data.getItemMetadata&&data.getItemMetadata(n);s&&s.cssClasses&&(a+=" "+s.cssClasses);var A,l,c=getFrozenRowOffset(n),u=o&&o.cssStyle||"",g="<div class='ui-widget-content "+a+"' style='top:"+(getRowTop(n)-c)+"px;"+u+"'>";t.push(g),options.frozenColumn>-1&&e.push(g);for(var h=0,d=columns.length;h<d;h++){if(l=columns[h],A=1,s&&s.columns){var f=s.columns[l.id]||s.columns[h];"*"===(A=f&&f.colspan||1)&&(A=d-h)}if(0!=l.width&&!l.collapsed){if(columnPosRight[Math.min(d-1,h+A-1)]>r.leftPx){if(columnPosLeft[h]>r.rightPx)break;options.frozenColumn>-1&&h>options.frozenColumn?appendCellHtml(e,n,h,A,o):appendCellHtml(t,n,h,A,o)}else options.frozenColumn>-1&&h<=options.frozenColumn&&appendCellHtml(t,n,h,A);A>1&&(h+=A-1)}}t.push("</div>"),options.frozenColumn>-1&&e.push("</div>")}function appendCellHtml(t,e,n,r,i){var o=columns[n],a="slick-cell l"+n+" r"+Math.min(columns.length-1,n+r-1);for(var s in e===activeRow&&n===activeCell&&(a+=" active"),i&&(a+=i.cssClass?" "+i.cssClass:""),cellCssClasses)cellCssClasses[s][e]&&cellCssClasses[s][e][o.id]&&(a+=" "+cellCssClasses[s][e][o.id]);var A="",l=[];if(i){void 0!==i["cssClass"+n]&&(a+=" "+i["cssClass"+n]);var c=getDataItemValueForColumn(i,o);getFormatters(e,o).forEach(function(t){var r=t(e,n,c,o,i,data);l.push(r)}),A=i["cssStyle"+n]||""}"ROWSELECTOR"===o.id?(t.push("<div class='"+a+" row-selector'>"),t.push(i.rowNumber+1)):(t.push("<div class='"+a+"' style='"+A+"'>"),l.forEach(function(e){t.push(e)})),t.push("</div>"),rowsCache[e].cellRenderQueue.push(n),rowsCache[e].cellColSpans[n]=r}function cleanupRows(t){for(var e in rowsCache){var n=!0;hasFrozenRows&&(options.frozenBottom&&e>=actualFrozenRow||!options.frozenBottom&&e<=actualFrozenRow)&&(n=!1),(e=parseInt(e,10))!==activeRow&&(e<t.top||e>t.bottom)&&n&&removeRowFromCache(e)}}function invalidate(){updateRowCount(),invalidateAllRows(),render()}function invalidateAllRows(){for(var t in currentEditor&&makeActiveCellNormal(),rowsCache)removeRowFromCache(t)}function removeRowFromCache(t){var e=rowsCache[t];e&&(e.rowNode[0].parentElement.removeChild(e.rowNode[0]),e.rowNode[1]&&e.rowNode[1].parentElement.removeChild(e.rowNode[1]),delete rowsCache[t],delete postProcessedRows[t],renderedRows--,counter_rows_removed++)}function invalidateRows(t){var e,n;if(t&&t.length)for(vScrollDir=0,e=0,n=t.length;e<n;e++)currentEditor&&activeRow===t[e]&&makeActiveCellNormal(),rowsCache[t[e]]&&removeRowFromCache(t[e])}function invalidateRow(t){invalidateRows([t])}function updateCell(t,e){var n=getCellNode(t,e);if(n){var r=columns[e],i=getDataItem(t);currentEditor&&activeRow===t&&activeCell===e?currentEditor.loadValue(i):(n.innerHTML=i?getFormatter(t,r)(t,e,getDataItemValueForColumn(i,r),r,i):"",invalidatePostProcessingResults(t))}}function updateRow(t){var e=rowsCache[t];if(e){ensureCellNodesInRowsCache(t);var n=getDataItem(t);for(var r in e.cellNodesByColumnIdx)if(e.cellNodesByColumnIdx.hasOwnProperty(r)){var i=columns[r|=0],o=e.cellNodesByColumnIdx[r][0];t===activeRow&&r===activeCell&&currentEditor?currentEditor.loadValue(n):o.innerHTML=n?getFormatter(t,i)(t,r,getDataItemValueForColumn(n,i),i,n):""}invalidatePostProcessingResults(t)}}function getViewportHeight(){options.autoHeight?viewportH=options.rowHeight*getDataLengthIncludingAddNew()+(-1==options.frozenColumn?$headers.outerHeight():0):(topPanelH=options.showTopPanel?options.topPanelHeight+getVBoxDelta($topPanelScroller):0,headerRowH=options.showHeaderRow?options.headerRowHeight+getVBoxDelta($headerRowScroller):0,viewportH=parseFloat($.css($container[0],"height",!0))-parseFloat($.css($container[0],"paddingTop",!0))-parseFloat($.css($container[0],"paddingBottom",!0))-parseFloat($.css($headerScroller[0],"height"))-getVBoxDelta($headerScroller)-topPanelH-headerRowH),numVisibleRows=Math.ceil(viewportH/options.rowHeight)}function getViewportWidth(){viewportW=parseFloat($.css($container[0],"width",!0))}function resizeCanvas(){if(initialized){paneTopH=0,paneBottomH=0,viewportTopH=0,viewportBottomH=0,groupPanelH=$paneGrouping.height(),$paneHeaderL.css("top",groupPanelH),$paneHeaderR.css("top",groupPanelH),getViewportWidth(),getViewportHeight(),hasFrozenRows?options.frozenBottom?(paneTopH=viewportH-frozenRowsHeight-scrollbarDimensions.height,paneBottomH=frozenRowsHeight+scrollbarDimensions.height):(paneTopH=frozenRowsHeight,paneBottomH=viewportH-frozenRowsHeight):paneTopH=viewportH,paneTopH+=topPanelH+headerRowH,options.frozenColumn>-1&&options.autoHeight&&(paneTopH+=scrollbarDimensions.height),viewportTopH=paneTopH-topPanelH-headerRowH-groupPanelH,options.autoHeight&&(options.frozenColumn>-1&&$container.height(paneTopH+parseFloat($.css($headerScrollerL[0],"height"))),$paneTopL.css("position","relative")),$paneTopL.css({top:$paneHeaderL.height()+groupPanelH,height:paneTopH});var t=$paneTopL.position().top+paneTopH;$viewportTopL.height(viewportTopH),options.frozenColumn>-1?($paneTopR.css({top:$paneHeaderR.height()+groupPanelH,height:paneTopH}),$viewportTopR.height(viewportTopH),hasFrozenRows&&($paneBottomL.css({top:t,height:paneBottomH}),$paneBottomR.css({top:t,height:paneBottomH}),$viewportBottomR.height(paneBottomH))):hasFrozenRows&&($paneBottomL.css({width:"100%",height:paneBottomH}),$paneBottomL.css("top",t)),hasFrozenRows?($viewportBottomL.height(paneBottomH),options.frozenBottom?($canvasBottomL.height(frozenRowsHeight),options.frozenColumn>-1&&$canvasBottomR.height(frozenRowsHeight)):($canvasTopL.height(frozenRowsHeight),options.frozenColumn>-1&&$canvasTopR.height(frozenRowsHeight))):$viewportTopR.height(viewportTopH),options.forceFitColumns&&autosizeColumns(),updateRowCount(),handleScroll(),lastRenderedScrollLeft=-1,render()}}function updateRowCount(){if(initialized){var t=getDataLengthIncludingAddNew(),e=0,r=hasFrozenRows&&!options.frozenBottom?$canvasBottomL.height():$canvasTopL.height();e=hasFrozenRows&&options.frozenBottom?getDataLength()-options.frozenRow:t+(options.leaveSpaceForNewRows?numVisibleRows-1:0);var i=$viewportScrollContainerY.height(),o=viewportHasVScroll;viewportHasVScroll=!options.autoHeight&&e*options.rowHeight>i;var a=t-1;for(var s in rowsCache)s>=a&&removeRowFromCache(s);th=Math.max(options.rowHeight*e,i-scrollbarDimensions.height),activeCellNode&&activeRow>a&&resetActiveCell(),th<maxSupportedCssHeight?(h=ph=th,n=1,cj=0):(ph=(h=maxSupportedCssHeight)/100,n=Math.floor(th/ph),cj=(th-h)/(n-1)),h!==r&&(hasFrozenRows&&!options.frozenBottom?($canvasBottomL.css("height",h),options.frozenColumn>-1&&$canvasBottomR.css("height",h)):($canvasTopL.css("height",h),$canvasTopR.css("height",h)),scrollTop=$viewportScrollContainerY[0].scrollTop);var A=scrollTop+offset<=th-i;0==th||0==scrollTop?page=offset=0:scrollTo(A?scrollTop+offset:th-i),h!=r&&options.autoHeight&&resizeCanvas(),options.forceFitColumns&&o!=viewportHasVScroll&&autosizeColumns(),updateCanvasWidth(!1)}}function getVisibleRange(t,e){return null==t&&(t=scrollTop),null==e&&(e=scrollLeft),{top:getRowFromPosition(t),bottom:getRowFromPosition(t+viewportH)+1,leftPx:e,rightPx:e+viewportW}}function getRenderedRange(t,e){var n=getVisibleRange(t,e),r=Math.round(viewportH/options.rowHeight);return-1==vScrollDir?(n.top-=r,n.bottom+=3):1==vScrollDir?(n.top-=3,n.bottom+=r):(n.top-=3,n.bottom+=3),n.top=Math.max(0,n.top),n.bottom=Math.min(getDataLengthIncludingAddNew()-1,n.bottom),n.leftPx-=viewportW,n.rightPx+=viewportW,n.leftPx=Math.max(0,n.leftPx),n.rightPx=Math.min(canvasWidth,n.rightPx),n}function ensureCellNodesInRowsCache(t){var e=rowsCache[t];if(e&&e.cellRenderQueue.length)for(var n=e.rowNode.children().last();e.cellRenderQueue.length;){var r=e.cellRenderQueue.pop();e.cellNodesByColumnIdx[r]=n,0==(n=n.prev()).length&&(n=$(e.rowNode[0]).children().last())}}function cleanUpCells(t,e){if(!hasFrozenRows||!(options.frozenBottom&&e>actualFrozenRow||e<=actualFrozenRow)){var n,r=rowsCache[e],i=[];for(var o in r.cellNodesByColumnIdx)if(r.cellNodesByColumnIdx.hasOwnProperty(o)&&!((o|=0)<=options.frozenColumn)){var a=r.cellColSpans[o];(columnPosLeft[o]>t.rightPx||columnPosRight[Math.min(columns.length-1,o+a-1)]<t.leftPx)&&(e==activeRow&&o==activeCell||i.push(o))}for(;null!=(n=i.pop());)r.cellNodesByColumnIdx[n][0].parentElement.removeChild(r.cellNodesByColumnIdx[n][0]),delete r.cellColSpans[n],delete r.cellNodesByColumnIdx[n],postProcessedRows[e]&&delete postProcessedRows[e][n]}}function cleanUpAndRenderCells(t){for(var e,n,r,i=[],o=[],a=t.top,s=t.bottom;a<=s;a++)if(e=rowsCache[a]){ensureCellNodesInRowsCache(a),cleanUpCells(t,a),n=0;var A=data.getItemMetadata&&data.getItemMetadata(a);A=A&&A.columns;for(var l=getDataItem(a),c=0,u=columns.length;c<u&&!(columnPosLeft[c]>t.rightPx);c++)if(0!=columns[c].width&&!columns[c].collapsed)if(null==(r=e.cellColSpans[c])){if(r=1,A){var g=A[columns[c].id]||A[c];"*"===(r=g&&g.colspan||1)&&(r=u-c)}columnPosRight[Math.min(u-1,c+r-1)]>t.leftPx&&(appendCellHtml(i,a,c,r,l),n++),c+=r>1?r-1:0}else c+=r>1?r-1:0;n&&o.push(a)}if(i.length){var h,d,f=document.createElement("div");for(f.innerHTML=i.join("");null!=(h=o.pop());){var p;for(e=rowsCache[h];null!=(p=e.cellRenderQueue.pop());)d=$(f).children().last(),options.frozenColumn>-1&&p>options.frozenColumn?$(e.rowNode[1]).append(d):$(e.rowNode[0]).append(d),e.cellNodesByColumnIdx[p]=d}}}function renderRows(t){for(var e=[],n=[],r=[],i=!1,o=getDataLength(),a=t.top,s=t.bottom;a<=s;a++)rowsCache[a]||hasFrozenRows&&options.frozenBottom&&a==getDataLength()||(renderedRows++,r.push(a),rowsCache[a]={rowNode:null,cellColSpans:[],cellNodesByColumnIdx:[],cellRenderQueue:[]},appendRowHtml(e,n,a,t,o),activeCellNode&&activeRow===a&&(i=!0),counter_rows_rendered++);if(r.length){var A=document.createElement("div"),l=document.createElement("div");for(A.innerHTML=e.join(""),l.innerHTML=n.join(""),a=0,s=r.length;a<s;a++)hasFrozenRows&&r[a]>=actualFrozenRow?options.frozenColumn>-1?rowsCache[r[a]].rowNode=$().add($(A.firstChild).appendTo($canvasBottomL)).add($(l.firstChild).appendTo($canvasBottomR)):rowsCache[r[a]].rowNode=$().add($(A.firstChild).appendTo($canvasBottomL)):options.frozenColumn>-1?rowsCache[r[a]].rowNode=$().add($(A.firstChild).appendTo($canvasTopL)).add($(l.firstChild).appendTo($canvasTopR)):rowsCache[r[a]].rowNode=$().add($(A.firstChild).appendTo($canvasTopL));trigger(self.onRenderRows,{}),i&&(activeCellNode=getCellNode(activeRow,activeCell))}}function startPostProcessing(){options.enableAsyncPostRender&&(clearTimeout(h_postrender),h_postrender=setTimeout(asyncPostProcessRows,options.asyncPostRenderDelay))}function invalidatePostProcessingResults(t){delete postProcessedRows[t],postProcessFromRow=Math.min(postProcessFromRow,t),postProcessToRow=Math.max(postProcessToRow,t),startPostProcessing()}function updateRowPositions(){for(var t in rowsCache)rowsCache[t].rowNode.css("top",getRowTop(t)+"px")}function render(){if(initialized){var t=getVisibleRange(),e=getRenderedRange();cleanupRows(e),lastRenderedScrollLeft!=scrollLeft&&cleanUpAndRenderCells(e),renderRows(e),options.frozenBottom&&renderRows({top:actualFrozenRow,bottom:getDataLength()-1,leftPx:e.leftPx,rightPx:e.rightPx}),postProcessFromRow=t.top,postProcessToRow=Math.min(getDataLengthIncludingAddNew()-1,t.bottom),startPostProcessing(),lastRenderedScrollTop=scrollTop,lastRenderedScrollLeft=scrollLeft,h_render=null}}function handleHeaderRowScroll(){var t=$headerRowScrollContainer[0].scrollLeft;t!=$viewportScrollContainerX[0].scrollLeft&&($viewportScrollContainerX[0].scrollLeft=t)}function handleMouseWheel(t,e,n,r){scrollTop=Math.max(0,$viewportScrollContainerY[0].scrollTop-r*(options.rowHeight*options.linesPerScroll)),scrollLeft=$viewportScrollContainerX[0].scrollLeft+10*n,_handleScroll(!0),t.preventDefault()}function handleScroll(){scrollTop=$viewportScrollContainerY[0].scrollTop,scrollLeft=$viewportScrollContainerX[0].scrollLeft,_handleScroll(!1)}function _handleScroll(t){var e=$viewportScrollContainerY[0].scrollHeight-$viewportScrollContainerY[0].clientHeight,r=$viewportScrollContainerY[0].scrollWidth-$viewportScrollContainerY[0].clientWidth;scrollTop>e&&(scrollTop=e),scrollLeft>r&&(scrollLeft=r);var i=Math.abs(scrollTop-prevScrollTop),o=Math.abs(scrollLeft-prevScrollLeft);if(o&&(prevScrollLeft=scrollLeft,$viewportScrollContainerX[0].scrollLeft=scrollLeft,$headerScrollContainer[0].scrollLeft=scrollLeft,$topPanelScroller[0].scrollLeft=scrollLeft,$headerRowScrollContainer[0].scrollLeft=scrollLeft,options.frozenColumn>-1?hasFrozenRows&&($viewportTopR[0].scrollLeft=scrollLeft):hasFrozenRows&&($viewportTopL[0].scrollLeft=scrollLeft)),i)if(vScrollDir=prevScrollTop<scrollTop?1:-1,prevScrollTop=scrollTop,t&&($viewportScrollContainerY[0].scrollTop=scrollTop),options.frozenColumn>-1&&(hasFrozenRows&&!options.frozenBottom?$viewportBottomL[0].scrollTop=scrollTop:$viewportTopL[0].scrollTop=scrollTop),i<viewportH)scrollTo(scrollTop+offset);else{var a=offset;page=h==viewportH?0:Math.min(n-1,Math.floor(scrollTop*((th-viewportH)/(h-viewportH))*(1/ph))),a!=(offset=Math.round(page*cj))&&invalidateAllRows()}(o||i)&&(h_render&&clearTimeout(h_render),(Math.abs(lastRenderedScrollTop-scrollTop)>20||Math.abs(lastRenderedScrollLeft-scrollLeft)>20)&&(options.forceSyncScrolling||Math.abs(lastRenderedScrollTop-scrollTop)<viewportH&&Math.abs(lastRenderedScrollLeft-scrollLeft)<viewportW?render():h_render=setTimeout(render,5),trigger(self.onViewportChanged,{}))),trigger(self.onScroll,{scrollLeft:scrollLeft,scrollTop:scrollTop})}function asyncPostProcessRows(){for(var t=0,e=performance.now(),n=getDataLength();postProcessFromRow<=postProcessToRow;){var r=vScrollDir>=0?postProcessFromRow++:postProcessToRow--,i=rowsCache[r];if(i&&!(r>=n)){postProcessedRows[r]||(postProcessedRows[r]={});var o=performance.now();for(var a in ensureCellNodesInRowsCache(r),i.cellNodesByColumnIdx)if(i.cellNodesByColumnIdx.hasOwnProperty(a)){var s=columns[a|=0];if(s.asyncPostRender&&!postProcessedRows[r][a]){var A=i.cellNodesByColumnIdx[a];A&&(s.asyncPostRender(A,r,getDataItem(r),s),t=performance.now()-o),postProcessedRows[r][a]=!0}}if(performance.now()-e+t>15)return void(h_postrender=setTimeout(asyncPostProcessRows,options.asyncPostRenderDelay))}}}function updateCellCssStylesOnRenderedRows(t,e){var n,r,i,o;for(var a in rowsCache){if(o=e&&e[a],i=t&&t[a],o)for(r in o)i&&o[r]==i[r]||(n=getCellNode(a,getColumnIndex(r)))&&$(n).removeClass(o[r]);if(i)for(r in i)o&&o[r]==i[r]||(n=getCellNode(a,getColumnIndex(r)))&&$(n).addClass(i[r])}}function addCellCssStyles(t,e){if(cellCssClasses[t])throw"addCellCssStyles: cell CSS hash with key '"+t+"' already exists.";cellCssClasses[t]=e,updateCellCssStylesOnRenderedRows(e,null),trigger(self.onCellCssStylesChanged,{key:t,hash:e})}function removeCellCssStyles(t){cellCssClasses[t]&&(updateCellCssStylesOnRenderedRows(null,cellCssClasses[t]),delete cellCssClasses[t],trigger(self.onCellCssStylesChanged,{key:t,hash:null}))}function setCellCssStyles(t,e){var n=cellCssClasses[t];cellCssClasses[t]=e,updateCellCssStylesOnRenderedRows(e,n),trigger(self.onCellCssStylesChanged,{key:t,hash:e})}function getCellCssStyles(t){return cellCssClasses[t]}function flashCell(t,e,n){if(n=n||100,rowsCache[t]){var r=$(getCellNode(t,e));!function t(e){e&&setTimeout(function(){r.queue(function(){r.toggleClass(options.cellFlashingCssClass).dequeue(),t(e-1)})},n)}(4)}}function handleDragInit(t,e){var n=getCellFromEvent(t);if(!n||!cellExists(n.row,n.cell))return!1;var r=trigger(self.onDragInit,e,t);return!!t.isImmediatePropagationStopped()&&r}function handleDragStart(t,e){var n=getCellFromEvent(t);if(!n||!cellExists(n.row,n.cell))return!1;var r=trigger(self.onDragStart,e,t);return!!t.isImmediatePropagationStopped()&&r}function handleDrag(t,e){return trigger(self.onDrag,e,t)}function handleDragEnd(t,e){trigger(self.onDragEnd,e,t)}function handleKeyDown(t){trigger(self.onKeyDown,{row:activeRow,cell:activeCell},t);var e=t.isImmediatePropagationStopped();if(!e)if(t.shiftKey||t.altKey||t.ctrlKey)9!=t.which||!t.shiftKey||t.ctrlKey||t.altKey||(e=navigatePrev());else if(27==t.which){if(!getEditorLock().isActive())return;cancelEditAndSetFocus()}else 34==t.which?(navigatePageDown(),e=!0):33==t.which?(navigatePageUp(),e=!0):37==t.which?e=navigateLeft():39==t.which?e=navigateRight():38==t.which?e=navigateUp():40==t.which?e=navigateDown():9==t.which?e=navigateNext():13==t.which&&(options.editable&&(currentEditor?activeRow===getDataLength()?navigateDown():commitEditAndSetFocus():getEditorLock().commitCurrentEdit()&&makeActiveCellEditable()),e=!0);if(e){t.stopPropagation(),t.preventDefault();try{t.originalEvent.keyCode=0}catch(t){}}}function handleClick(t){currentEditor||(t.target!=document.activeElement||$(t.target).hasClass("slick-cell"))&&setFocus();var e=getCellFromEvent(t);!e||null!==currentEditor&&activeRow==e.row&&activeCell==e.cell||(trigger(self.onClick,{row:e.row,cell:e.cell},t),t.isImmediatePropagationStopped()||activeCell==e.cell&&activeRow==e.row||!canCellBeActive(e.row,e.cell)||getEditorLock().isActive()&&!getEditorLock().commitCurrentEdit()||hasFrozenRows&&((!options.frozenBottom&&e.row>=actualFrozenRow||options.frozenBottom&&e.row<actualFrozenRow)&&scrollRowIntoView(e.row,!1),setActiveCellInternal(getCellNode(e.row,e.cell))))}function handleMouseDown(t){var e=getCellFromEvent(t);e&&trigger(self.onMouseDown,{row:e.row,cell:e.cell},t)}function handleContextMenu(t){var e=$(t.target).closest(".slick-cell",$canvas);0!==e.length&&(activeCellNode===e[0]&&null!==currentEditor||trigger(self.onContextMenu,{},t))}function handleDblClick(t){var e=getCellFromEvent(t);!e||null!==currentEditor&&activeRow==e.row&&activeCell==e.cell||(trigger(self.onDblClick,{row:e.row,cell:e.cell},t),t.isImmediatePropagationStopped()||options.editable&&gotoCell(e.row,e.cell,!0))}function handleHeaderMouseEnter(t){trigger(self.onHeaderMouseEnter,{column:$(this).data("column")},t)}function handleHeaderMouseLeave(t){trigger(self.onHeaderMouseLeave,{column:$(this).data("column")},t)}function handleHeaderContextMenu(t){var e=$(t.target).closest(".slick-header-column",".slick-header-columns"),n=e&&e.data("column");trigger(self.onHeaderContextMenu,{column:n},t)}function handleHeaderClick(t){var e=$(t.target).closest(".slick-header-column",".slick-header-columns"),n=e&&e.data("column");n&&trigger(self.onHeaderClick,{column:n},t)}function handleMouseEnter(t){trigger(self.onMouseEnter,{},t)}function handleMouseLeave(t){trigger(self.onMouseLeave,{},t)}function cellExists(t,e){return!(t<0||t>=getDataLength()||e<0||e>=columns.length)}function getCellFromPoint(t,e){for(var n=getRowFromPosition(e),r=0,i=0,o=0;o<columns.length&&i<t;o++)i+=columns[o].width,r++;return r<0&&(r=0),{row:n,cell:r-1}}function getCellFromNode(t){var e=/l\d+/.exec(t.className);if(!e)throw"getCellFromNode: cannot get cell - "+t.className;return parseInt(e[0].substr(1,e[0].length-1),10)}function getRowFromNode(t){for(var e in rowsCache)if(rowsCache[e].rowNode[0]===t[0])return 0|e;return null}function getFrozenRowOffset(t){return hasFrozenRows?options.frozenBottom?t>=actualFrozenRow?h<viewportTopH?actualFrozenRow*options.rowHeight:h:0:t>=actualFrozenRow?frozenRowsHeight:0:0}function getCellFromEvent(t){var e=$(t.target).closest(".slick-cell",$canvas);if(!e.length)return null;var n=e.parents(".grid-canvas").offset(),r=0,i=e.parents(".grid-canvas-bottom").length;hasFrozenRows&&i&&(r=options.frozenBottom?$canvasTopL.height():frozenRowsHeight);var o=getCellFromPoint(t.clientX-n.left,t.clientY-n.top+r+$(document).scrollTop()).row,a=getCellFromNode(e[0]);return null==o||null==a?null:{row:o,cell:a}}function getCellNodeBox(t,e){if(!cellExists(t,e))return null;for(var n=getFrozenRowOffset(t),r=getRowTop(t)-n,i=r+options.rowHeight-1,o=0,a=0;a<e;a++)o+=columns[a].width,options.frozenColumn==a&&(o=0);return{top:r,left:o,bottom:i,right:o+columns[e].width}}function resetActiveCell(){setActiveCellInternal(null,!1)}function setFocus(){-1==tabbingDirection?$focusSink[0].focus():$focusSink2[0].focus()}function scrollCellIntoView(t,e,n){if(!(e<=options.frozenColumn)){t<actualFrozenRow&&scrollRowIntoView(t,n);var r=getColspan(t,e),i=columnPosLeft[e],o=columnPosRight[e+(r>1?r-1:0)],a=scrollLeft+$viewportScrollContainerX.width();i<scrollLeft?($viewportScrollContainerX.scrollLeft(i),handleScroll(),render()):o>a&&($viewportScrollContainerX.scrollLeft(Math.min(i,o-$viewportScrollContainerX[0].clientWidth)),handleScroll(),render())}}function setActiveCellInternal(t,e,n){null!==activeCellNode&&(makeActiveCellNormal(),$(activeCellNode).removeClass("active"),rowsCache[activeRow]&&$(rowsCache[activeRow].rowNode).removeClass("active"));var r=activeCellNode!==t;if(null!=(activeCellNode=t)){var i=$(activeCellNode),o=i.offset(),a=Math.floor(i.parents(".grid-canvas").offset().top),s=i.parents(".grid-canvas-bottom").length;hasFrozenRows&&s&&(a-=options.frozenBottom?$canvasTopL.height():frozenRowsHeight),cell=getCellFromPoint(o.left,Math.ceil(o.top)-a),activeRow=cell.row,activeCell=activePosX=activeCell=activePosX=getCellFromNode(activeCellNode[0]),i.addClass("active"),rowsCache[activeRow]&&$(rowsCache[activeRow].rowNode).addClass("active"),null==e&&(e=activeRow==getDataLength()||options.autoEdit),options.editable&&e&&isCellPotentiallyEditable(activeRow,activeCell)&&(clearTimeout(h_editorLoader),options.asyncEditorLoading?h_editorLoader=setTimeout(function(){makeActiveCellEditable()},options.asyncEditorLoadDelay):makeActiveCellEditable())}else activeRow=activeCell=null;r&&(setTimeout(scrollActiveCellIntoView,2),!1!==n&&trigger(self.onActiveCellChanged,getActiveCell()))}function clearTextSelection(){if(document.selection&&document.selection.empty)try{document.selection.empty()}catch(t){}else if(window.getSelection){var t=window.getSelection();t&&t.removeAllRanges&&t.removeAllRanges()}}function isCellPotentiallyEditable(t,e){var n=getDataLength();return!(t<n&&!getDataItem(t)||columns[e].cannotTriggerInsert&&t>=n||!getEditor(t,e))}function makeActiveCellNormal(){if(currentEditor){if(trigger(self.onBeforeCellEditorDestroy,{editor:currentEditor}),currentEditor.destroy(),currentEditor=null,activeCellNode){var t=getDataItem(activeRow);if($(activeCellNode).removeClass("editable invalid"),t){var e=columns[activeCell],n=getFormatter(activeRow,e);activeCellNode[0].innerHTML=n(activeRow,activeCell,getDataItemValueForColumn(t,e),e,t),invalidatePostProcessingResults(activeRow)}}navigator.userAgent.toLowerCase().match(/msie/)&&clearTextSelection(),getEditorLock().deactivate(editController)}}function makeActiveCellEditable(t){if(activeCellNode){if(!options.editable)throw"Grid : makeActiveCellEditable : should never get called when options.editable is false";if(clearTimeout(h_editorLoader),isCellPotentiallyEditable(activeRow,activeCell)){var e=columns[activeCell],n=getDataItem(activeRow);!1!==trigger(self.onBeforeEditCell,{row:activeRow,cell:activeCell,item:n,column:e})?(getEditorLock().activate(editController),$(activeCellNode).addClass("editable"),t||(activeCellNode[0].innerHTML=""),currentEditor=new(t||getEditor(activeRow,activeCell))({grid:self,gridPosition:absBox($container[0]),position:absBox(activeCellNode[0]),container:activeCellNode,column:e,item:n||{},commitChanges:commitEditAndSetFocus,cancelChanges:cancelEditAndSetFocus}),n&&currentEditor.loadValue(n),serializedEditorValue=currentEditor.serializeValue(),currentEditor.position&&handleActiveCellPositionChange()):setFocus()}}}function commitEditAndSetFocus(){getEditorLock().commitCurrentEdit()&&(setFocus(),options.autoEdit&&navigateDown())}function cancelEditAndSetFocus(){getEditorLock().cancelCurrentEdit()&&setFocus()}function absBox(t){var e={top:t.offsetTop,left:t.offsetLeft,bottom:0,right:0,width:$(t).outerWidth(),height:$(t).outerHeight(),visible:!0};e.bottom=e.top+e.height,e.right=e.left+e.width;for(var n=t.offsetParent;(t=t.parentNode)!=document.body;)e.visible&&t.scrollHeight!=t.offsetHeight&&"visible"!=$(t).css("overflowY")&&(e.visible=e.bottom>t.scrollTop&&e.top<t.scrollTop+t.clientHeight),e.visible&&t.scrollWidth!=t.offsetWidth&&"visible"!=$(t).css("overflowX")&&(e.visible=e.right>t.scrollLeft&&e.left<t.scrollLeft+t.clientWidth),e.left-=t.scrollLeft,e.top-=t.scrollTop,t===n&&(e.left+=t.offsetLeft,e.top+=t.offsetTop,n=t.offsetParent),e.bottom=e.top+e.height,e.right=e.left+e.width;return e}function getActiveCellPosition(){return absBox(activeCellNode[0])}function getGridPosition(){return absBox($container[0])}function handleActiveCellPositionChange(){if(activeCellNode&&(trigger(self.onActiveCellPositionChanged,{}),currentEditor)){var t=getActiveCellPosition();currentEditor.show&&currentEditor.hide&&(t.visible?currentEditor.show():currentEditor.hide()),currentEditor.position&&currentEditor.position(t)}}function getCellEditor(){return currentEditor}function getActiveCell(){return activeCellNode?{row:activeRow,cell:activeCell}:null}function getActiveCellNode(){return activeCellNode}function scrollActiveCellIntoView(){null!=activeRow&&null!=activeCell&&scrollCellIntoView(activeRow,activeCell)}function scrollRowIntoView(t,e){hasFrozenRows&&!options.frozenBottom&&(t-=actualFrozenRow);var n=$viewportScrollContainerY.height(),r=t*options.rowHeight,i=(t+4)*options.rowHeight-n+(viewportHasHScroll?scrollbarDimensions.height:0);(t+1)*options.rowHeight>scrollTop+n+offset?(scrollTo(e?r:i),render()):t*options.rowHeight<scrollTop+offset&&(scrollTo(e?i:r),render())}function scrollRowToTop(t){scrollTo(t*options.rowHeight),render()}function scrollPage(t){var e=t*numVisibleRows;if(scrollTo((getRowFromPosition(scrollTop)+e)*options.rowHeight),render(),options.enableCellNavigation&&null!=activeRow){var n=activeRow+e,r=getDataLengthIncludingAddNew();n>=r&&(n=r-1),n<0&&(n=0);for(var i=0,o=null,a=activePosX;i<=activePosX;)canCellBeActive(n,i)&&(o=i),i+=getColspan(n,i);null!==o?(setActiveCellInternal(getCellNode(n,o)),activePosX=a):resetActiveCell()}}function navigatePageDown(){scrollPage(1)}function navigatePageUp(){scrollPage(-1)}function getColspan(t,e){var n=data.getItemMetadata&&data.getItemMetadata(t);if(!n||!n.columns)return 1;var r=n.columns[columns[e].id]||n.columns[e],i=r&&r.colspan;return(i="*"===i?columns.length-e:i||1)||1}function findFirstFocusableCell(t){for(var e=0;e<columns.length;){if(canCellBeActive(t,e))return e;e+=getColspan(t,e)}return null}function findLastFocusableCell(t){for(var e=0,n=null;e<columns.length;)canCellBeActive(t,e)&&(n=e),e+=getColspan(t,e);return n}function gotoRight(t,e,n){if(e>=columns.length)return null;do{e+=getColspan(t,e)}while(e<columns.length&&!canCellBeActive(t,e));return e<columns.length?{row:t,cell:e,posX:e}:null}function gotoLeft(t,e,n){if(e<=0)return null;var r=findFirstFocusableCell(t);if(null===r||r>=e)return null;for(var i,o={row:t,cell:r,posX:r};;){if(!(i=gotoRight(o.row,o.cell,o.posX)))return null;if(i.cell>=e)return o;o=i}}function gotoDown(t,e,n){for(var r,i=getDataLengthIncludingAddNew();;){if(++t>=i)return null;for(r=e=0;e<=n;)r=e,e+=getColspan(t,e);if(canCellBeActive(t,r))return{row:t,cell:r,posX:n}}}function gotoUp(t,e,n){for(var r;;){if(--t<0)return null;for(r=e=0;e<=n;)r=e,e+=getColspan(t,e);if(canCellBeActive(t,r))return{row:t,cell:r,posX:n}}}function gotoNext(t,e,n){if(null==t&&null==e&&canCellBeActive(t=e=n=0,e))return{row:t,cell:e,posX:e};var r=gotoRight(t,e,n);if(r)return r;for(var i=null,o=getDataLengthIncludingAddNew();++t<o;)if(null!==(i=findFirstFocusableCell(t)))return{row:t,cell:i,posX:i};return null}function gotoPrev(t,e,n){if(null==t&&null==e&&canCellBeActive(t=getDataLengthIncludingAddNew()-1,e=n=columns.length-1))return{row:t,cell:e,posX:e};for(var r,i;!r&&!(r=gotoLeft(t,e,n));){if(--t<0)return null;e=0,null!==(i=findLastFocusableCell(t))&&(r={row:t,cell:i,posX:i})}return r}function navigateRight(){return navigate("right")}function navigateLeft(){return navigate("left")}function navigateDown(){return navigate("down")}function navigateUp(){return navigate("up")}function navigateNext(){return navigate("next")}function navigatePrev(){return navigate("prev")}function navigate(t){if(!options.enableCellNavigation)return!1;if(!activeCellNode&&"prev"!=t&&"next"!=t)return!1;if(!getEditorLock().commitCurrentEdit())return!0;setFocus(),tabbingDirection={up:-1,down:1,left:-1,right:1,prev:-1,next:1}[t];var e=(0,{up:gotoUp,down:gotoDown,left:gotoLeft,right:gotoRight,prev:gotoPrev,next:gotoNext}[t])(activeRow,activeCell,activePosX);if(e){if(hasFrozenRows&&options.frozenBottom&e.row==getDataLength())return;var n=e.row==getDataLength();return(!options.frozenBottom&&e.row>=actualFrozenRow||options.frozenBottom&&e.row<actualFrozenRow)&&scrollCellIntoView(e.row,e.cell,!n),setActiveCellInternal(getCellNode(e.row,e.cell)),activePosX=e.posX,!0}return setActiveCellInternal(getCellNode(activeRow,activeCell)),!1}function getCellNode(t,e){return rowsCache[t]?(ensureCellNodesInRowsCache(t),rowsCache[t].cellNodesByColumnIdx[e]):null}function setActiveCell(t,e,n){initialized&&(t>getDataLength()||t<0||e>=columns.length||e<0||options.enableCellNavigation&&(scrollCellIntoView(t,e,!1),setActiveCellInternal(getCellNode(t,e),!1,n)))}function canCellBeActive(t,e){if(!options.enableCellNavigation||t>=getDataLengthIncludingAddNew()||t<0||e>=columns.length||e<0)return!1;var n=data.getItemMetadata&&data.getItemMetadata(t);if(n&&"boolean"==typeof n.focusable)return n.focusable;var r=n&&n.columns;return r&&r[columns[e].id]&&"boolean"==typeof r[columns[e].id].focusable?r[columns[e].id].focusable:r&&r[e]&&"boolean"==typeof r[e].focusable?r[e].focusable:columns[e].focusable}function canCellBeSelected(t,e){if(t>=getDataLength()||t<0||e>=columns.length||e<0)return!1;var n=data.getItemMetadata&&data.getItemMetadata(t);if(n&&"boolean"==typeof n.selectable)return n.selectable;var r=n&&n.columns&&(n.columns[columns[e].id]||n.columns[e]);return r&&"boolean"==typeof r.selectable?r.selectable:columns[e].selectable}function gotoCell(t,e,n){initialized&&canCellBeActive(t,e)&&getEditorLock().commitCurrentEdit()&&(scrollCellIntoView(t,e,!1),setActiveCellInternal(getCellNode(t,e),n||t===getDataLength()||options.autoEdit),currentEditor||setFocus())}function commitCurrentEdit(){var t=getDataItem(activeRow),e=columns[activeCell];if(currentEditor){if(currentEditor.isValueChanged()){var n=currentEditor.validate();if(n.valid){if(activeRow<getDataLength()){var r={row:activeRow,cell:activeCell,editor:currentEditor,serializedValue:currentEditor.serializeValue(),prevSerializedValue:serializedEditorValue,execute:function(){this.editor.applyValue(t,this.serializedValue),updateRow(this.row)},undo:function(){this.editor.applyValue(t,this.prevSerializedValue),updateRow(this.row)}};options.editCommandHandler?(makeActiveCellNormal(),options.editCommandHandler(t,e,r)):(r.execute(),makeActiveCellNormal()),trigger(self.onCellChange,{row:activeRow,cell:activeCell,item:t})}else{var i={};currentEditor.applyValue(i,currentEditor.serializeValue()),makeActiveCellNormal(),trigger(self.onAddNewRow,{item:i,column:e})}return!getEditorLock().isActive()}return $(activeCellNode).removeClass("invalid"),$(activeCellNode).width(),$(activeCellNode).addClass("invalid"),trigger(self.onValidationError,{editor:currentEditor,cellNode:activeCellNode,validationResults:n,row:activeRow,cell:activeCell,column:e}),currentEditor.focus(),!1}makeActiveCellNormal()}return!0}function cancelCurrentEdit(){return makeActiveCellNormal(),!0}function rowsToRanges(t){for(var e=[],n=columns.length-1,r=0;r<t.length;r++)e.push(new Slick.Range(t[r],0,t[r],n));return e}function getSelectedRows(){if(!selectionModel)throw"Selection model is not set";return selectedRows}function setSelectedRows(t){if(!selectionModel)throw"Selection model is not set";selectionModel.setSelectedRanges(rowsToRanges(t))}this.debug=function(){var t="";t+="\ncounter_rows_rendered:  "+counter_rows_rendered,t+="\ncounter_rows_removed:  "+counter_rows_removed,t+="\nrenderedRows:  "+renderedRows,t+="\nnumVisibleRows:  "+numVisibleRows,t+="\nmaxSupportedCssHeight:  "+maxSupportedCssHeight,t+="\nn(umber of pages):  "+n,t+="\n(current) page:  "+page,t+="\npage height (ph):  "+ph,t+="\nvScrollDir:  "+vScrollDir,alert(t)},this.eval=function(expr){return eval(expr)},$.extend(this,{slickGridVersion:"2.1",onScroll:new Slick.Event,onSort:new Slick.Event,onHeaderMouseEnter:new Slick.Event,onHeaderMouseLeave:new Slick.Event,onHeaderContextMenu:new Slick.Event,onHeaderClick:new Slick.Event,onHeaderCellRendered:new Slick.Event,onBeforeHeaderCellDestroy:new Slick.Event,onHeaderRowCellRendered:new Slick.Event,onBeforeHeaderRowCellDestroy:new Slick.Event,onMouseEnter:new Slick.Event,onMouseLeave:new Slick.Event,onClick:new Slick.Event,onMouseDown:new Slick.Event,onDblClick:new Slick.Event,onContextMenu:new Slick.Event,onKeyDown:new Slick.Event,onAddNewRow:new Slick.Event,onValidationError:new Slick.Event,onViewportChanged:new Slick.Event,onColumnsReordered:new Slick.Event,onColumnsResized:new Slick.Event,onCellChange:new Slick.Event,onBeforeEditCell:new Slick.Event,onBeforeCellEditorDestroy:new Slick.Event,onBeforeDestroy:new Slick.Event,onActiveCellChanged:new Slick.Event,onActiveCellPositionChanged:new Slick.Event,onDragInit:new Slick.Event,onDragStart:new Slick.Event,onDrag:new Slick.Event,onDragEnd:new Slick.Event,onSelectedRowsChanged:new Slick.Event,onCellCssStylesChanged:new Slick.Event,onRenderRows:new Slick.Event,registerPlugin:registerPlugin,unregisterPlugin:unregisterPlugin,getColumns:getColumns,setColumns:setColumns,getColumnIndex:getColumnIndex,updateColumnHeader:updateColumnHeader,setSortColumn:setSortColumn,setSortColumns:setSortColumns,getSortColumns:getSortColumns,autosizeColumns:autosizeColumns,getOptions:getOptions,setOptions:setOptions,getData:getData,getDataLength:getDataLength,getDataItem:getDataItem,setData:setData,getSelectionModel:getSelectionModel,setSelectionModel:setSelectionModel,getSelectedRows:getSelectedRows,setSelectedRows:setSelectedRows,getContainerNode:getContainerNode,render:render,invalidate:invalidate,invalidateRow:invalidateRow,invalidateRows:invalidateRows,invalidateAllRows:invalidateAllRows,updateCell:updateCell,updateRow:updateRow,getViewport:getVisibleRange,getRenderedRange:getRenderedRange,resizeCanvas:resizeCanvas,updateRowCount:updateRowCount,scrollRowIntoView:scrollRowIntoView,scrollRowToTop:scrollRowToTop,scrollCellIntoView:scrollCellIntoView,getCanvasNode:getCanvasNode,getCanvases:getCanvases,getActiveCanvasNode:getActiveCanvasNode,setActiveCanvasNode:setActiveCanvasNode,getViewportNode:getViewportNode,getActiveViewportNode:getActiveViewportNode,setActiveViewportNode:setActiveViewportNode,focus:setFocus,getCellFromPoint:getCellFromPoint,getCellFromEvent:getCellFromEvent,getActiveCell:getActiveCell,setActiveCell:setActiveCell,getActiveCellNode:getActiveCellNode,getActiveCellPosition:getActiveCellPosition,resetActiveCell:resetActiveCell,editActiveCell:makeActiveCellEditable,getCellEditor:getCellEditor,getCellNode:getCellNode,getCellNodeBox:getCellNodeBox,canCellBeSelected:canCellBeSelected,canCellBeActive:canCellBeActive,navigatePrev:navigatePrev,navigateNext:navigateNext,navigateUp:navigateUp,navigateDown:navigateDown,navigateLeft:navigateLeft,navigateRight:navigateRight,navigatePageUp:navigatePageUp,navigatePageDown:navigatePageDown,gotoCell:gotoCell,getTopPanel:getTopPanel,setTopPanelVisibility:setTopPanelVisibility,setHeaderRowVisibility:setHeaderRowVisibility,getHeaderRow:getHeaderRow,getHeaderRowColumn:getHeaderRowColumn,getGridPosition:getGridPosition,flashCell:flashCell,addCellCssStyles:addCellCssStyles,setCellCssStyles:setCellCssStyles,removeCellCssStyles:removeCellCssStyles,getCellCssStyles:getCellCssStyles,getFrozenRowOffset:getFrozenRowOffset,init:finishInitialization,destroy:destroy,getEditorLock:getEditorLock,getEditController:getEditController}),init()}$.extend(!0,window,{Slick:{Grid:SlickGrid}})}(jQuery)}).call(this,__webpack_require__(5))},function(t,e,n){(function(t){var e;(e=t).extend(!0,window,{Slick:{CellSelectionModel:function(t){var n,r,i=[],o=this,a=new Slick.CellRangeSelector({selectionCss:{border:"2px solid black"}}),s={selectActiveCell:!0};function A(t){for(var e=[],r=0;r<t.length;r++){var i=t[r];n.canCellBeSelected(i.fromRow,i.fromCell)&&n.canCellBeSelected(i.toRow,i.toCell)&&e.push(i)}return e}function l(t){i=A(t),o.onSelectedRangesChanged.notify(i)}function c(){return i}function u(t,e){if(n.getEditorLock().isActive())return t.stopPropagation(),!1}function g(t,e){l([e.range])}function h(t,e){r.selectActiveCell&&null!=e.row&&null!=e.cell&&l([new Slick.Range(e.row,e.cell)])}function d(t){var e,r,i=n.getActiveCell();if(i&&t.shiftKey&&!t.ctrlKey&&!t.altKey&&(37==t.which||39==t.which||38==t.which||40==t.which)){(e=c()).length||e.push(new Slick.Range(i.row,i.cell)),(r=e.pop()).contains(i.row,i.cell)||(r=new Slick.Range(i.row,i.cell));var o=r.toRow-r.fromRow,a=r.toCell-r.fromCell,s=i.row==r.fromRow?1:-1,u=i.cell==r.fromCell?1:-1;37==t.which?a-=u:39==t.which?a+=u:38==t.which?o-=s:40==t.which&&(o+=s);var g=new Slick.Range(i.row,i.cell,i.row+s*o,i.cell+u*a);if(A([g]).length){e.push(g);var h=s>0?g.toRow:g.fromRow,d=u>0?g.toCell:g.fromCell;n.scrollRowIntoView(h),n.scrollCellIntoView(h,d)}else e.push(r);l(e),t.preventDefault(),t.stopPropagation()}}e.extend(this,{getSelectedRanges:c,setSelectedRanges:l,init:function(i){r=e.extend(!0,{},s,t),(n=i).getActiveCanvasNode(),n.onActiveCellChanged.subscribe(h),n.onKeyDown.subscribe(d),i.registerPlugin(a),a.onCellRangeSelected.subscribe(g),a.onBeforeCellRangeSelected.subscribe(u)},destroy:function(){n.onActiveCellChanged.unsubscribe(h),n.onKeyDown.unsubscribe(d),a.onCellRangeSelected.unsubscribe(g),a.onBeforeCellRangeSelected.unsubscribe(u),n.unregisterPlugin(a)},onSelectedRangesChanged:new Slick.Event})}}})}).call(this,n(5))},function(t,e,n){(function(t){var e;(e=t).extend(!0,window,{Slick:{CellRangeDecorator:function(t,n){var r;n=e.extend(!0,{},{selectionCssClass:"slick-range-decorator",selectionCss:{zIndex:"99999999",border:"2px dashed red"}},n),e.extend(this,{show:function(i,o,a){return t.resetActiveCell(),t.setSelectedRows([]),r||(r=e("<div></div>",{css:n.selectionCss}).addClass(n.selectionCssClass).css("position","absolute").appendTo(i)),a=a||o,void 0!==o&&r.css({top:o.top-1,left:o.left-1,height:a.bottom-o.top-2,width:a.right-o.left-2}),r},hide:function(){r&&(r.remove(),r=null)}})}}})}).call(this,n(5))},function(t,e,n){(function(t){var e;(e=t).extend(!0,window,{Slick:{CellRangeSelector:function(t){var n,r,i,o,a,s,A,l,c=this,u=new Slick.EventHandler,g={selectionCss:{border:"2px dashed blue"}};function h(t,o){(i=e(n.getActiveCanvasNode(t))).offset(),s=0,a=0,l=i.hasClass("grid-canvas-bottom"),r.frozenRow>-1&&l&&(a=r.frozenBottom?e(".grid-canvas-bottom").height():e(".grid-canvas-top").height()),A=i.hasClass("grid-canvas-right"),r.frozenColumn>-1&&A&&(s=e(".grid-canvas-left").width()),t.stopImmediatePropagation()}function d(t){if(null!=t){var e=t;return e.left+=s,e}}function f(t,e){var r=n.getCellFromEvent(t);!1!==c.onBeforeCellRangeSelected.notify(r)&&n.canCellBeSelected(r.row,r.cell)&&(o=!0,t.stopImmediatePropagation()),o&&(n.focus(),e.range={start:r,end:{}},c.onCellRangeSelected.notify({range:new Slick.Range(e.range.start.row,e.range.start.cell,e.range.start.row,e.range.start.cell)}))}function p(t,e){if(o){t.stopImmediatePropagation();var u=n.getCellFromPoint(t.pageX-i.offset().left+s,t.pageY-i.offset().top+a);!n.canCellBeSelected(u.row,u.cell)||!A&&u.cell>r.frozenColumn||A&&u.cell<=r.frozenColumn||!l&&u.row>=r.frozenRow||l&&(u.row,r.frozenRow),e.range.end=u,c.onCellRangeSelected.notify({range:new Slick.Range(e.range.start.row,e.range.start.cell,e.range.end.row,e.range.end.cell)});var g=n.getCellNodeBox(e.range.start.row,e.range.start.cell);d(n.getCellNodeBox(u.row,u.cell)),e.range.start.cell>r.frozenColumn&&(g=d(g)),u.cell>r.frozenColumn&&d(u)}}function I(t,e){o&&(o=!1,t.stopImmediatePropagation(),c.onCellRangeSelected.notify({range:new Slick.Range(e.range.start.row,e.range.start.cell,e.range.end.row,e.range.end.cell)}))}e.extend(this,{init:function(i){t=e.extend(!0,{},g,t),new Slick.CellRangeDecorator(i,t),r=(n=i).getOptions(),u.subscribe(n.onDragInit,h).subscribe(n.onDragStart,f).subscribe(n.onDrag,p).subscribe(n.onDragEnd,I)},destroy:function(){u.unsubscribeAll()},onBeforeCellRangeSelected:new Slick.Event,onCellRangeSelected:new Slick.Event})}}})}).call(this,n(5))},function(t,e,n){(function(t){var e;(e=t).extend(!0,window,{Slick:{Plugins:{PathButtons:function(t){var n,r=this,i=new Slick.EventHandler,o={buttonCssClass:"slick-path-button"};function a(n,r){var i=r.column;if(i.header&&i.header.buttons)for(var o=i.header.buttons.length;o--;){var a=i.header.buttons[o],s=e("<div></div>").addClass(t.buttonCssClass).data("column",i).data("button",a);a.showOnHover&&s.addClass("slick-path-button-hidden"),a.image&&s.css("backgroundImage","url("+a.image+")"),a.cssClass&&s.addClass(a.cssClass),a.tooltip&&s.attr("title",a.tooltip),a.command&&s.data("command",a.command),a.handler&&s.bind("click",a.handler),a.text&&s.text(a.text),o===i.header.buttons.length-1&&s.css({marginRight:10,borderRight:"1px solid gray"}),0===o&&s.css("border-left","none"),s.on("click",A).prependTo(e(r.node).find("tr").last())}}function s(n,r){var i=r.column;i.header&&i.header.buttons&&e(r.node).find("."+t.buttonCssClass).remove()}function A(t){var i=e(this).data("command"),o=e(this).data("column"),a=e(this).data("button");null!=i&&(r.onCommand.notify({grid:n,column:o,command:i,button:a},t,r),n.updateColumnHeader(o.id)),t.preventDefault(),t.stopPropagation()}e.extend(this,{init:function(r){t=e.extend(!0,{},o,t),n=r,i.subscribe(n.onHeaderCellRendered,a).subscribe(n.onBeforeHeaderCellDestroy,s),n.setColumns(n.getColumns())},destroy:function(){i.unsubscribeAll()},onCommand:new Slick.Event})}}}})}).call(this,n(5))},function(t,e,n){(function(t){var e;(e=t).extend(!0,window,{Slick:{AutoTooltips:function(t){var n,r={enableForCells:!0,enableForHeaderCells:!1,maxToolTipLength:null};function i(r){var i=n.getCellFromEvent(r);if(i){var o,a=e(n.getCellNode(i.row,i.cell));a.innerWidth()<a[0].scrollWidth?(o=e.trim(a.text()),t.maxToolTipLength&&o.length>t.maxToolTipLength&&(o=o.substr(0,t.maxToolTipLength-3)+"...")):o="",a.attr("title",o)}}function o(t,n){var r=n.column,i=e(t.target).closest(".slick-header-column");r&&!r.toolTip&&i.attr("title",i.innerWidth()<i[0].scrollWidth?r.name:"")}e.extend(this,{init:function(a){t=e.extend(!0,{},r,t),n=a,t.enableForCells&&n.onMouseEnter.subscribe(i),t.enableForHeaderCells&&n.onHeaderMouseEnter.subscribe(o)},destroy:function(){t.enableForCells&&n.onMouseEnter.unsubscribe(i),t.enableForHeaderCells&&n.onHeaderMouseEnter.unsubscribe(o)}})}}})}).call(this,n(5))},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0});const r=n(89),i=n(33),o=n(144),a=n(149),s=n(62);e.formatColumns=function(e,n,A){let l=[],c={id:"path",name:"Path",field:"path",formatters:[r.PathIndentationFormatter],width:200,columnNumber:0,sortable:!0,headerCssClass:"column-header0",header:{buttons:[]}};const u="grid-"+n.partId;t("#"+u+"column-styles").remove();let g=t("<style>").attr("type","text/css").attr("id",u+"column-styles"),h=n.get("Show Path Column");if(h&&(l.push(c),g.append("#"+u+" .column-header0 {color:black !important; font-style:italic;}")),e.forEach(function(t,e){let c=e+Number(h),d=t.Name.split("."),f=d[d.length-1],p={id:f+c,name:f,field:"col"+c,formatters:[],columnNumber:c,columnDataNumber:e,columnGroups:d.slice(0,d.length-1)};function I(e){return i.getFormattingOption(t.FormattingOptions,e)}g.append("#"+u+" .column-header"+c+" {font-weight:100 !important;");var C=[];if(I("General.ColumnWidthPixels")&&(p.width=I("General.ColumnWidthPixels")),"black"!==I("ColumnHeader.ForeColor").color?C.push("color:"+I("ColumnHeader.ForeColor").color):C.push("color:black !important"),"white"!==I("ColumnHeader.BackColor").color&&C.push("background:"+I("ColumnHeader.BackColor").color+" !important"),"None"!==I("ConditionalFormatting.Simple"))switch(p.conditionalFormatters=[],I("ConditionalFormatting.Simple")){case"NegRed":p.conditionalFormatters.push(new a.ConditionalFormatting("getRowValue('"+p.field+"') < 0",{ForeColor:"red"}));break;case"NegRedPosGreen":p.conditionalFormatters.push(new a.ConditionalFormatting("getRowValue('"+p.field+"') < 0",{ForeColor:"red"}),new a.ConditionalFormatting("getRowValue('"+p.field+"') > 0",{ForeColor:"green"}))}function m(t,e){return I("ConditionalFormatting.Condition"+t+"."+e)}for(let t=1;t<=9;++t)"null"!==m(t," Condition")&&(p.conditionalFormatters=p.conditionalFormatters||[],p.conditionalFormatters.push(new a.ConditionalFormatting(""+m(t," Condition"),{BackColor:m(t,"BackColor").color,ForeColor:m(t,"ForeColor").color,IsBold:m(t,"IsBold"),IsItalic:m(t,"IsItalic")})));void 0!==p.conditionalFormatters&&p.formatters.push(r.ConditionalFormatter),I("Heatmap. Enable")&&(p.heatmap={maxColor:I("Heatmap.Maximum.Color").color,maxValue:+I("Heatmap.Maximum.Value"),minColor:I("Heatmap.Minimum.Color").color,minValue:+I("Heatmap.Minimum.Value"),midColor:I("Heatmap.Center.Color").color,midValue:+I("Heatmap.Center.Value")},p.heatmap.func=s.scale.linear().clamp(!0).domain([p.heatmap.minValue,p.heatmap.midValue,p.heatmap.maxValue]).range([p.heatmap.minColor,p.heatmap.midColor,p.heatmap.maxColor]),p.formatters.push(r.HeatmapColumnFormatter)),"ProgressBar"===I("General.DisplayStyle")&&(p.progressBar={startColor:I("ProgressBar.StartColor").color,endColor:I("ProgressBar.EndColor").color,minValue:+I("ProgressBar.Minimum"),maxValue:+I("ProgressBar.Maximum"),showValue:I("ProgressBar.ShowValue")},p.progressBar.func=s.scale.linear().clamp(!0).domain([p.progressBar.minValue,p.progressBar.maxValue]).range([0,100]),p.formatters.push(r.ProgressBarFormatter));const y=o.AsyncPostRenderers(n,A);I("Row Detail.Show Row Detail Button")&&(p.formatters.push(r.RowDetailFormatter),p.asyncPostRender=y.RowDetail,p.sortable=!1),""!==I("Number.FormatString")&&(p.numberFormat=I("Number.FormatString"),p.formatters&&p.formatters.push(r.NumberFormatter)),"SparkLine"===I("General.DisplayStyle")&&(p.asyncPostRender=y.Sparkline,p.rerenderOnResize=!0,p.sortable=!1,p.formatters=[r.SparklineLoadingFormatter(n)]),"DashboardLink"===I("General.DisplayStyle")&&(p.asyncPostRender=y.DashboardLink,p.sortable=!1,p.formatters=[r.DashboardLinkFormatter]),"Hide"!==I("General.NullRule")&&(p.nullRule="Show"),g.append(C.join(";")+";}"),p.headerCssClass="column-header"+c,l.push(p)}),n.get("Checklist. Enabled")&&l[n.get("Checklist.Column for Checkboxes")].formatters.push(r.CheckBoxColumnFormatter),n.get("RadioButtonList. Enabled")){let t=l[n.get("RadioButtonList.Column for Radio Buttons")];t&&(t.formatters.push(r.RadioButtonColumnFormatter),t.hasRadio=!0)}return h&&l[0].formatters.push(r.BasicValueFormatter),l.forEach(function(t){t.formatters.every(t=>t.styleFormatter)&&t.formatters.push(r.BasicValueFormatter)}),g.appendTo("head"),n.get("Show Row Selectors")&&l.unshift({id:"ROWSELECTOR",name:"#",field:"#",behavior:"select",width:40,cannotTriggerInsert:!0,resizable:!1,selectable:!1}),l}}).call(this,n(5))},function(t,e,n){var r,i;
/*! @preserve
 * numeral.js
 * version : 2.0.6
 * author : Adam Draper
 * license : MIT
 * http://adamwdraper.github.com/Numeral-js/
 */void 0===(i="function"==typeof(r=function(){var t,e,n,r,i,o={},a={},s={currentLocale:"en",zeroFormat:null,nullFormat:null,defaultFormat:"0,0",scalePercentBy100:!0},A={currentLocale:s.currentLocale,zeroFormat:s.zeroFormat,nullFormat:s.nullFormat,defaultFormat:s.defaultFormat,scalePercentBy100:s.scalePercentBy100};function l(t,e){this._input=t,this._value=e}return(t=function(n){var r,i,a,s;if(t.isNumeral(n))r=n.value();else if(0===n||void 0===n)r=0;else if(null===n||e.isNaN(n))r=null;else if("string"==typeof n)if(A.zeroFormat&&n===A.zeroFormat)r=0;else if(A.nullFormat&&n===A.nullFormat||!n.replace(/[^0-9]+/g,"").length)r=null;else{for(i in o)if((s="function"==typeof o[i].regexps.unformat?o[i].regexps.unformat():o[i].regexps.unformat)&&n.match(s)){a=o[i].unformat;break}r=(a=a||t._.stringToNumber)(n)}else r=Number(n)||null;return new l(n,r)}).version="2.0.6",t.isNumeral=function(t){return t instanceof l},t._=e={numberToFormat:function(e,n,r){var i,o,s,A,l,c,u,g,h=a[t.options.currentLocale],d=!1,f=!1,p="",I="",C=!1;if(e=e||0,s=Math.abs(e),t._.includes(n,"(")?(d=!0,n=n.replace(/[\(|\)]/g,"")):(t._.includes(n,"+")||t._.includes(n,"-"))&&(c=t._.includes(n,"+")?n.indexOf("+"):e<0?n.indexOf("-"):-1,n=n.replace(/[\+|\-]/g,"")),t._.includes(n,"a")&&(o=!!(o=n.match(/a(k|m|b|t)?/))&&o[1],t._.includes(n," a")&&(p=" "),n=n.replace(new RegExp(p+"a[kmbt]?"),""),s>=1e12&&!o||"t"===o?(p+=h.abbreviations.trillion,e/=1e12):s<1e12&&s>=1e9&&!o||"b"===o?(p+=h.abbreviations.billion,e/=1e9):s<1e9&&s>=1e6&&!o||"m"===o?(p+=h.abbreviations.million,e/=1e6):(s<1e6&&s>=1e3&&!o||"k"===o)&&(p+=h.abbreviations.thousand,e/=1e3)),t._.includes(n,"[.]")&&(f=!0,n=n.replace("[.]",".")),A=e.toString().split(".")[0],l=n.split(".")[1],u=n.indexOf(","),i=(n.split(".")[0].split(",")[0].match(/0/g)||[]).length,l?(t._.includes(l,"[")?(l=(l=l.replace("]","")).split("["),I=t._.toFixed(e,l[0].length+l[1].length,r,l[1].length)):I=t._.toFixed(e,l.length,r),A=I.split(".")[0],I=t._.includes(I,".")?h.delimiters.decimal+I.split(".")[1]:"",f&&0===Number(I.slice(1))&&(I="")):A=t._.toFixed(e,0,r),p&&!o&&Number(A)>=1e3&&p!==h.abbreviations.trillion)switch(A=String(Number(A)/1e3),p){case h.abbreviations.thousand:p=h.abbreviations.million;break;case h.abbreviations.million:p=h.abbreviations.billion;break;case h.abbreviations.billion:p=h.abbreviations.trillion}if(t._.includes(A,"-")&&(A=A.slice(1),C=!0),A.length<i)for(var m=i-A.length;m>0;m--)A="0"+A;return u>-1&&(A=A.toString().replace(/(\d)(?=(\d{3})+(?!\d))/g,"$1"+h.delimiters.thousands)),0===n.indexOf(".")&&(A=""),g=A+I+(p||""),d?g=(d&&C?"(":"")+g+(d&&C?")":""):c>=0?g=0===c?(C?"-":"+")+g:g+(C?"-":"+"):C&&(g="-"+g),g},stringToNumber:function(t){var e,n,r,i=a[A.currentLocale],o=t,s={thousand:3,million:6,billion:9,trillion:12};if(A.zeroFormat&&t===A.zeroFormat)n=0;else if(A.nullFormat&&t===A.nullFormat||!t.replace(/[^0-9]+/g,"").length)n=null;else{for(e in n=1,"."!==i.delimiters.decimal&&(t=t.replace(/\./g,"").replace(i.delimiters.decimal,".")),s)if(r=new RegExp("[^a-zA-Z]"+i.abbreviations[e]+"(?:\\)|(\\"+i.currency.symbol+")?(?:\\))?)?$"),o.match(r)){n*=Math.pow(10,s[e]);break}n*=(t.split("-").length+Math.min(t.split("(").length-1,t.split(")").length-1))%2?1:-1,t=t.replace(/[^0-9\.]+/g,""),n*=Number(t)}return n},isNaN:function(t){return"number"==typeof t&&isNaN(t)},includes:function(t,e){return-1!==t.indexOf(e)},insert:function(t,e,n){return t.slice(0,n)+e+t.slice(n)},reduce:function(t,e){if(null===this)throw new TypeError("Array.prototype.reduce called on null or undefined");if("function"!=typeof e)throw new TypeError(e+" is not a function");var n,r=Object(t),i=r.length>>>0,o=0;if(3===arguments.length)n=arguments[2];else{for(;o<i&&!(o in r);)o++;if(o>=i)throw new TypeError("Reduce of empty array with no initial value");n=r[o++]}for(;o<i;o++)o in r&&(n=e(n,r[o],o,r));return n},multiplier:function(t){var e=t.toString().split(".");return e.length<2?1:Math.pow(10,e[1].length)},correctionFactor:function(){var t=Array.prototype.slice.call(arguments);return t.reduce(function(t,n){var r=e.multiplier(n);return t>r?t:r},1)},toFixed:function(t,e,n,r){var i,o,a,s,A=t.toString().split("."),l=e-(r||0);return i=2===A.length?Math.min(Math.max(A[1].length,l),e):l,a=Math.pow(10,i),s=(n(t+"e+"+i)/a).toFixed(i),r>e-i&&(o=new RegExp("\\.?0{1,"+(r-(e-i))+"}$"),s=s.replace(o,"")),s}},t.options=A,t.formats=o,t.locales=a,t.locale=function(t){return t&&(A.currentLocale=t.toLowerCase()),A.currentLocale},t.localeData=function(t){if(!t)return a[A.currentLocale];if(t=t.toLowerCase(),!a[t])throw new Error("Unknown locale : "+t);return a[t]},t.reset=function(){for(var t in s)A[t]=s[t]},t.zeroFormat=function(t){A.zeroFormat="string"==typeof t?t:null},t.nullFormat=function(t){A.nullFormat="string"==typeof t?t:null},t.defaultFormat=function(t){A.defaultFormat="string"==typeof t?t:"0.0"},t.register=function(t,e,n){if(e=e.toLowerCase(),this[t+"s"][e])throw new TypeError(e+" "+t+" already registered.");return this[t+"s"][e]=n,n},t.validate=function(e,n){var r,i,o,a,s,A,l,c;if("string"!=typeof e&&(e+="",console.warn&&console.warn("Numeral.js: Value is not string. It has been co-erced to: ",e)),(e=e.trim()).match(/^\d+$/))return!0;if(""===e)return!1;try{l=t.localeData(n)}catch(e){l=t.localeData(t.locale())}return o=l.currency.symbol,s=l.abbreviations,r=l.delimiters.decimal,i="."===l.delimiters.thousands?"\\.":l.delimiters.thousands,!(null!==(c=e.match(/^[^\d]+/))&&(e=e.substr(1),c[0]!==o)||null!==(c=e.match(/[^\d]+$/))&&(e=e.slice(0,-1),c[0]!==s.thousand&&c[0]!==s.million&&c[0]!==s.billion&&c[0]!==s.trillion)||(A=new RegExp(i+"{2}"),e.match(/[^\d.,]/g)||(a=e.split(r)).length>2||(a.length<2?!a[0].match(/^\d+.*\d$/)||a[0].match(A):1===a[0].length?!a[0].match(/^\d+$/)||a[0].match(A)||!a[1].match(/^\d+$/):!a[0].match(/^\d+.*\d$/)||a[0].match(A)||!a[1].match(/^\d+$/))))},t.fn=l.prototype={clone:function(){return t(this)},format:function(e,n){var r,i,a,s=this._value,l=e||A.defaultFormat;if(n=n||Math.round,0===s&&null!==A.zeroFormat)i=A.zeroFormat;else if(null===s&&null!==A.nullFormat)i=A.nullFormat;else{for(r in o)if(l.match(o[r].regexps.format)){a=o[r].format;break}i=(a=a||t._.numberToFormat)(s,l,n)}return i},value:function(){return this._value},input:function(){return this._input},set:function(t){return this._value=Number(t),this},add:function(t){var n=e.correctionFactor.call(null,this._value,t);return this._value=e.reduce([this._value,t],function(t,e,r,i){return t+Math.round(n*e)},0)/n,this},subtract:function(t){var n=e.correctionFactor.call(null,this._value,t);return this._value=e.reduce([t],function(t,e,r,i){return t-Math.round(n*e)},Math.round(this._value*n))/n,this},multiply:function(t){return this._value=e.reduce([this._value,t],function(t,n,r,i){var o=e.correctionFactor(t,n);return Math.round(t*o)*Math.round(n*o)/Math.round(o*o)},1),this},divide:function(t){return this._value=e.reduce([this._value,t],function(t,n,r,i){var o=e.correctionFactor(t,n);return Math.round(t*o)/Math.round(n*o)}),this},difference:function(e){return Math.abs(t(this._value).subtract(e).value())}},t.register("locale","en",{delimiters:{thousands:",",decimal:"."},abbreviations:{thousand:"k",million:"m",billion:"b",trillion:"t"},ordinal:function(t){var e=t%10;return 1==~~(t%100/10)?"th":1===e?"st":2===e?"nd":3===e?"rd":"th"},currency:{symbol:"$"}}),t.register("format","bps",{regexps:{format:/(BPS)/,unformat:/(BPS)/},format:function(e,n,r){var i,o=t._.includes(n," BPS")?" ":"";return e*=1e4,n=n.replace(/\s?BPS/,""),i=t._.numberToFormat(e,n,r),t._.includes(i,")")?((i=i.split("")).splice(-1,0,o+"BPS"),i=i.join("")):i=i+o+"BPS",i},unformat:function(e){return+(1e-4*t._.stringToNumber(e)).toFixed(15)}}),r={base:1024,suffixes:["B","KiB","MiB","GiB","TiB","PiB","EiB","ZiB","YiB"]},i="("+(i=(n={base:1e3,suffixes:["B","KB","MB","GB","TB","PB","EB","ZB","YB"]}).suffixes.concat(r.suffixes.filter(function(t){return n.suffixes.indexOf(t)<0})).join("|")).replace("B","B(?!PS)")+")",t.register("format","bytes",{regexps:{format:/([0\s]i?b)/,unformat:new RegExp(i)},format:function(e,i,o){var a,s,A,l=t._.includes(i,"ib")?r:n,c=t._.includes(i," b")||t._.includes(i," ib")?" ":"";for(i=i.replace(/\s?i?b/,""),a=0;a<=l.suffixes.length;a++)if(s=Math.pow(l.base,a),A=Math.pow(l.base,a+1),null===e||0===e||e>=s&&e<A){c+=l.suffixes[a],s>0&&(e/=s);break}return t._.numberToFormat(e,i,o)+c},unformat:function(e){var i,o,a=t._.stringToNumber(e);if(a){for(i=n.suffixes.length-1;i>=0;i--){if(t._.includes(e,n.suffixes[i])){o=Math.pow(n.base,i);break}if(t._.includes(e,r.suffixes[i])){o=Math.pow(r.base,i);break}}a*=o||1}return a}}),t.register("format","currency",{regexps:{format:/(\$)/},format:function(e,n,r){var i,o,a=t.locales[t.options.currentLocale],s={before:n.match(/^([\+|\-|\(|\s|\$]*)/)[0],after:n.match(/([\+|\-|\)|\s|\$]*)$/)[0]};for(n=n.replace(/\s?\$\s?/,""),i=t._.numberToFormat(e,n,r),e>=0?(s.before=s.before.replace(/[\-\(]/,""),s.after=s.after.replace(/[\-\)]/,"")):e<0&&!t._.includes(s.before,"-")&&!t._.includes(s.before,"(")&&(s.before="-"+s.before),o=0;o<s.before.length;o++)switch(s.before[o]){case"$":i=t._.insert(i,a.currency.symbol,o);break;case" ":i=t._.insert(i," ",o+a.currency.symbol.length-1)}for(o=s.after.length-1;o>=0;o--)switch(s.after[o]){case"$":i=o===s.after.length-1?i+a.currency.symbol:t._.insert(i,a.currency.symbol,-(s.after.length-(1+o)));break;case" ":i=o===s.after.length-1?i+" ":t._.insert(i," ",-(s.after.length-(1+o)+a.currency.symbol.length-1))}return i}}),t.register("format","exponential",{regexps:{format:/(e\+|e-)/,unformat:/(e\+|e-)/},format:function(e,n,r){var i=("number"!=typeof e||t._.isNaN(e)?"0e+0":e.toExponential()).split("e");return n=n.replace(/e[\+|\-]{1}0/,""),t._.numberToFormat(Number(i[0]),n,r)+"e"+i[1]},unformat:function(e){var n=t._.includes(e,"e+")?e.split("e+"):e.split("e-"),r=Number(n[0]),i=Number(n[1]);return i=t._.includes(e,"e-")?i*=-1:i,t._.reduce([r,Math.pow(10,i)],function(e,n,r,i){var o=t._.correctionFactor(e,n);return e*o*(n*o)/(o*o)},1)}}),t.register("format","ordinal",{regexps:{format:/(o)/},format:function(e,n,r){var i=t.locales[t.options.currentLocale],o=t._.includes(n," o")?" ":"";return n=n.replace(/\s?o/,""),o+=i.ordinal(e),t._.numberToFormat(e,n,r)+o}}),t.register("format","percentage",{regexps:{format:/(%)/,unformat:/(%)/},format:function(e,n,r){var i,o=t._.includes(n," %")?" ":"";return t.options.scalePercentBy100&&(e*=100),n=n.replace(/\s?\%/,""),i=t._.numberToFormat(e,n,r),t._.includes(i,")")?((i=i.split("")).splice(-1,0,o+"%"),i=i.join("")):i=i+o+"%",i},unformat:function(e){var n=t._.stringToNumber(e);return t.options.scalePercentBy100?.01*n:n}}),t.register("format","time",{regexps:{format:/(:)/,unformat:/(:)/},format:function(t,e,n){var r=Math.floor(t/60/60),i=Math.floor((t-60*r*60)/60),o=Math.round(t-60*r*60-60*i);return r+":"+(i<10?"0"+i:i)+":"+(o<10?"0"+o:o)},unformat:function(t){var e=t.split(":"),n=0;return 3===e.length?(n+=60*Number(e[0])*60,n+=60*Number(e[1]),n+=Number(e[2])):2===e.length&&(n+=60*Number(e[0]),n+=Number(e[1])),Number(n)}}),t})?r.call(e,n,e,t):r)||(t.exports=i)},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(145);e.AsyncPostRenderers=function(t,e){return{Sparkline:function(n,i,o,a,s){r.SparklineRenderer(n,t,o.rowPath,a.field,o[a.field],{showAxes:s}),r.SparklineTooltip(n,{showAxes:s},a,e.Result.Rows[o.rowNumber],t)},RowDetail:function(n,i,o,a){r.RowDetailTooltip(n,{},t.grid.getColumns(),e.Result.Rows[o.rowNumber],t)},DashboardLink:function(n,i,o,a){r.DashboardLinkTooltip(n,{},t.grid.getColumns(),a,e.Result.Rows[o.rowNumber],t)}}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(146);e.DashboardLinkTooltip=r.DashboardLinkTooltip;var i=n(147);e.RowDetailTooltip=i.RowDetailTooltip;var o=n(148);e.SparklineTooltip=o.SparklineTooltip;var a=n(42);e.SparklineRenderer=a.SparklineRenderer;var s=n(63);e.Tooltip=s.Tooltip},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0}),e.DashboardLinkTooltip=function(e,n,r,i,o,a){if(void 0!==o){let n=o.RowData[i.columnDataNumber];if("string"!=typeof n||n.length<1)return;const[r,s]=n.split("?");let A=new URLSearchParams(s),l=+A.get("width"),c=+A.get("height");A.delete("width"),A.delete("height");let u=r+"?"+A;e.off("mouseenter"),e.off("mouseleave"),e.on({mouseenter:function(n){var r=t(e)[0].getBoundingClientRect();a.OpenDashboardHover(u,r.right+1,r.y,l,c)},mouseleave:function(t){a.CloseHover()},dblclick:function(n){var r=t(e)[0].getBoundingClientRect();a.OpenDashboardPopup(u,r.right+1,r.y,l,c)}})}}}).call(this,n(5))},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0});const r=n(63),i=n(89),o=n(42);e.RowDetailTooltip=function(e,n,a,s,A){var l=t("<table>").css({borderCollapse:"collapse",border:"none",width:"100%",padding:2,marginTop:5});let c=24;a.forEach(function(e,n){if(n>A.pathColumn){let r=t("<tr>").css({borderTop:"1px dashed lightgray"});r.append(t("<td>").text(e.name).css({width:"50%",fontSize:"13px",padding:3,"white-space":"normal"}));let a=t("<td>"),u={},g="";u[e.field]=s.RowData[n-1-A.pathColumn],e.formatters.forEach(function(t){t!==i.CheckBoxColumnFormatter&&t!==i.RadioButtonColumnFormatter&&t!==i.RowDetailFormatter&&t!==i.TableHoverFormatter&&t.formatName!==i.SparklineLoadingFormatter.formatName&&(g+=t(-1,n,s.RowData[n-1-A.pathColumn],e,u)||"")}),a.attr("style",u["cssStyle"+n]||"").css({width:"50%",fontSize:"13px",borderLeft:"1px dashed lightgray",borderRight:"none !important",padding:3}),-1!==e.formatters.findIndex(t=>t.formatName===i.SparklineLoadingFormatter.formatName)?u[e.field]&&(r.height(40),a.height(40),a.width(195),o.SparklineRenderer(a,A,s.Path,"RowDetail",u[e.field],{showAxes:!1}),c+=50):c+=24,a.append(g),r.append(a),l.append(r)}}),r.Tooltip(e,t.extend(n,{html:'<h3 style="text-align:center; padding:0; margin:0;">'+s.Name+"</h3>"+l.prop("outerHTML"),height:c,width:400}),A)}}).call(this,n(5))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(63),i=n(42);e.SparklineTooltip=function(t,e,n,o,a){let s={};if(void 0!==o){let l=o.RowData[n.columnDataNumber-1],c=o.RowData[n.columnDataNumber];var A=l&&l.hasOwnProperty("First")?l:c;if(!A)return;s[n.field]=A,r.Tooltip(t,Object.assign(e,{html:function(t,e){return i.SparklineRenderer(e,a,o.Path,n.field,s[n.field],{showAxes:!0}),'<h3 style="text-align:center; padding:0; margin:0;">'+n.name+" for "+o.Name+"</h3>"},htmlMode:"prepend",height:200}),a)}}},function(module,exports,__webpack_require__){"use strict";Object.defineProperty(exports,"__esModule",{value:!0});class ConditionalFormatting{constructor(t,e){this.condition=t||"",this.options=Object.assign({ForeColor:"",BackColor:"",IsBold:!1,IsItalic:!1},e||{}),this.compileCondition()}getCompiled(){return this.compiled||this.compileCondition(),this.compiled}setCondition(t){return this.condition=t,this.compileCondition(),this}getCondition(){return this.condition}setOption(t){return this.options=Object.assign({},this.options,t),this}getOptions(){return this.options}compute(data){function getRowValue(t){return data[t]}try{return eval(this.compiled)}catch(t){return t}}compileCondition(){this.condition&&(this.compiled=this.condition.replace(/(and)/gi,"&&").replace(/(or)/gi,"||"))}}exports.ConditionalFormatting=ConditionalFormatting},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(151);e.TableHoverPlugin=r.TableHoverPlugin;var i=n(152);e.ErrorDetailHoverPlugin=i.ErrorDetailHoverPlugin},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0});const r=n(25);e.TableHoverPlugin=class{constructor(t){this._handleMouseEnter=this.handleMouseEnter.bind(this),this._handleMouseLeave=this.handleMouseLeave.bind(this),this._handleDblClick=this.handleDblClick.bind(this),this.gridContext=t}init(t){this.grid=t,t.onMouseEnter.subscribe(this._handleMouseEnter),t.onMouseLeave.subscribe(this._handleMouseLeave),t.onDblClick.subscribe(this._handleDblClick)}destroy(){this.grid&&(this.grid.onMouseEnter.unsubscribe(this._handleMouseEnter),this.grid.onMouseLeave.unsubscribe(this._handleMouseLeave),this.grid.onDblClick.unsubscribe(this._handleDblClick))}handleMouseEnter(e,n){const i=this.grid;if(i&&!this.currentCell){var o=i.getCellFromEvent(e);if(o){let n=i.getDataItem(o.row)[i.getColumns()[o.cell].field];if(n instanceof r.Table){this.currentCell=o;let r=i.getCellNode(o.row,o.cell);var a=t(r)[0].getBoundingClientRect();const s=800,A=800,l=25+17*n.rows.length,c=Math.min(58*n.columnNames.length+1+(l>A?18:0),s),u=Math.min(l,A);this.gridContext.OpenTableHover(n,"{}",!1,a.right+1,a.y,c,u),e.preventDefault(),e.stopPropagation()}}}}handleMouseLeave(t,e){this.currentCell&&(this.gridContext.CloseHover(),this.currentCell=null,t.preventDefault(),t.stopPropagation())}handleDblClick(e,n){const i=this.grid;if(i){var o=i.getCellFromEvent(e);if(o){let n=i.getDataItem(o.row)[i.getColumns()[o.cell].field];if(n instanceof r.Table){let r=i.getCellNode(o.row,o.cell);var a=t(r)[0].getBoundingClientRect();const s=800,A=800,l=25+17*n.rows.length,c=Math.min(58*n.columnNames.length+1+(l>A?18:0),s),u=Math.min(l,A);this.gridContext.OpenTableHover(n,"{}",!0,a.right+1,a.y,c,u+30),e.preventDefault(),e.stopPropagation()}}}}}}).call(this,n(5))},function(t,e,n){"use strict";(function(t){Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(64);e.ErrorDetailHoverPlugin=class{constructor(t){this._handleMouseEnter=this.handleMouseEnter.bind(this),this._handleMouseLeave=this.handleMouseLeave.bind(this),this._handleDblClick=this.handleDblClick.bind(this),this.hover=null,this.gridContext=t}init(t){this.grid=t,t.onMouseEnter.subscribe(this._handleMouseEnter),t.onMouseLeave.subscribe(this._handleMouseLeave),t.onDblClick.subscribe(this._handleDblClick)}destroy(){this.grid&&(this.grid.onMouseEnter.unsubscribe(this._handleMouseEnter),this.grid.onMouseLeave.unsubscribe(this._handleMouseLeave),this.grid.onDblClick.unsubscribe(this._handleDblClick)),this.hover&&i.HoverManager.GetManager().closeHover(this.hover)}handleMouseEnter(t){const e=this.grid;if(e&&!this.currentCell){var n=e.getCellFromEvent(t);this.createHover(n)&&(t.preventDefault(),t.stopPropagation())}}createHover(e,n="hover"){this.hover&&i.HoverManager.GetManager().closeHover(this.hover);let o=this.grid;if(null==e||null==o)return!1;let a=o.getDataItem(e.row)[o.getColumns()[e.cell].field];if(a instanceof Error){this.currentCell=e;let A=o.getCellNode(e.row,e.cell);var s=t(A)[0].getBoundingClientRect();const l=800,c=800,u=10+20*(a.stack?a.stack.split("\n").length:2),g=6*Math.max(a.name.length,a.stack?Math.max(...a.stack.split("\n").map(t=>t.length)):30),h=Math.min(g,l),d=Math.min(u,c),f=new r.Widget({node:document.createElement("pre")});f.addClass("m-error-detail-hover"),f.title.label="Cell Error",f.node.innerText=a.stack||a.name+":"+a.message;const p=i.HoverManager.GetManager().openHover({hover:f,height:d,width:h,x:s.right+1,y:s.y,mode:n,offsetMode:"absolute",owner:this.gridContext.owner||f});return"hover"===n&&(this.hover=p),!0}return!1}handleMouseLeave(t){this.currentCell&&(this.hover&&i.HoverManager.GetManager().closeHover(this.hover),this.currentCell=null,t.preventDefault(),t.stopPropagation())}handleDblClick(t){const e=this.grid;if(e){var n=e.getCellFromEvent(t);this.createHover(n,"dialog")&&(t.stopPropagation(),t.preventDefault())}}}}).call(this,n(5))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(154),o=n(155);e.TableEditorWidget=class extends r.Widget{constructor(){super(),this.grid=null,this.rows=[],this._table=null,this._onChange=new i.Signal(this),this._activeColumn=null,this._activeCell=null,this.addClass("m-SlickGridPart"),this.addClass("m-TableEditorPart")}get table(){return this._table}set table(t){this._table=t,this.setupGrid()}get onChange(){return this._onChange}get activeColumn(){return this._activeColumn}get activeCell(){return this._activeCell}dispose(){this.isDisposed||(this.grid&&this.grid.destroy(),super.dispose())}onResize(){this.grid&&this.grid.resizeCanvas()}setupGrid(){if(null!=this.grid&&this.grid.destroy(),null==this._table)return;this.rows=[];for(let t=0;t<this._table.rows.length;t++){const e=this._table.rows[t],n={rowname:e.name,id:"row"+t};for(let t=0;t<this._table.columnNames.length;t++)n[this._table.columnNames[t]]=e.getValue(t);this.rows.push(n)}const t=this.getColumnConfig();this.grid=new Slick.Grid(this.node,this.rows,t,{editable:!0,enableCellNavigation:!0,enableColumnReorder:!1,rowHeight:22,enableAddRow:!0}),this.grid.onCellChange.subscribe((t,{cell:e,row:n,item:r})=>{const i=this.grid.getColumns()[e];this._table=this._table.copyTable(),this._table.rows[n].setValue(e,r[i.field]),this._onChange.emit()}),this.grid.onAddNewRow.subscribe((t,{item:e,column:n})=>{this.grid.invalidateRow(this.rows.length),this.rows.push({rowname:null,...e,id:"row"+this.rows.length}),this._table=this._table.copyTable();const r=this._table.createRow(null);r.setValue(n.field,e[n.field]),this._table.appendRow(r),this._onChange.emit(),this.grid.render()}),this.grid.onHeaderContextMenu.subscribe((t,{column:e})=>{this._activeColumn=null!=e&&e.field||null}),this.grid.onContextMenu.subscribe((t,{grid:e})=>{let n=this.grid.getCellFromEvent(t);this._activeCell=n})}getColumnConfig(){if(null==this._table)return[];const t=[];for(let e=0;e<this._table.columnNames.length;e++){const n=this._table.columnNames[e],r=this._table.columnTypes[e],i={field:n,id:"col"+e,editor:o.SlickGridEditor,type:r,name:n};t.push(i)}return t}}},function(t,e){t.exports=__WEBPACK_EXTERNAL_MODULE__154__},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(93),i=n(156),o=n(64),a=n(26);e.SlickGridEditor=class{constructor(t){this.el=null,this.newValue=null,this.defaultValue=null,this.didChange=!1,this.args=t,this.type=this.args.column.type||a.Types.Any,this.defaultValue=(this.args.item||{})[this.args.column.field],this.newValue=this.defaultValue,this.renderEditor()}init(){this.renderEditor()}destroy(){i.unmountComponentAtNode(this.args.container[0])}focus(){this.args.container[0].focus()}loadValue(t){this.defaultValue=t[this.args.column.field],this.renderEditor()}applyValue(t,e){t[this.args.column.field]=a.Converters.deserialize(JSON.parse(e))}isValueChanged(){return this.didChange}serializeValue(){return JSON.stringify(a.Converters.serialize(this.newValue,this.type))}validate(){return{valid:!0,msg:""}}renderEditor(){const t=this.newValue;this.el=r.createElement(o.TypeEditor,{type:this.type,value:t,onValueChanged:t=>{this.newValue=t,this.didChange=!0,this.renderEditor()}}),i.render(this.el,this.args.container[0],()=>{const t=this.args.container.find("input");t.is(":focus")||t.focus()})}}},function(t,e){t.exports=__WEBPACK_EXTERNAL_MODULE__156__},function(t,e,n){var r,i;
/* @license
Papa Parse
v4.6.3
https://github.com/mholt/PapaParse
License: MIT
*/Array.isArray||(Array.isArray=function(t){return"[object Array]"===Object.prototype.toString.call(t)}),void 0===(i="function"==typeof(r=function(){"use strict";var t,e,n="undefined"!=typeof self?self:"undefined"!=typeof window?window:void 0!==n?n:{},r=!n.document&&!!n.postMessage,i=r&&/(\?|&)papaworker(=|&|$)/.test(n.location.search),o=!1,a={},s=0,A={parse:function(e,r){var i=(r=r||{}).dynamicTyping||!1;if(B(i)&&(r.dynamicTypingFunction=i,i={}),r.dynamicTyping=i,r.transform=!!B(r.transform)&&r.transform,r.worker&&A.WORKERS_SUPPORTED){var l=function(){if(!A.WORKERS_SUPPORTED)return!1;if(!o&&null===A.SCRIPT_PATH)throw new Error("Script path cannot be determined automatically when Papa Parse is loaded asynchronously. You need to set Papa.SCRIPT_PATH manually.");var e=A.SCRIPT_PATH||t;e+=(-1!==e.indexOf("?")?"&":"?")+"papaworker";var r=new n.Worker(e);return r.onmessage=C,r.id=s++,a[r.id]=r}();return l.userStep=r.step,l.userChunk=r.chunk,l.userComplete=r.complete,l.userError=r.error,r.step=B(r.step),r.chunk=B(r.chunk),r.complete=B(r.complete),r.error=B(r.error),delete r.worker,void l.postMessage({input:e,config:r,workerId:l.id})}var c=null;return A.NODE_STREAM_INPUT,"string"==typeof e?c=r.download?new u(r):new h(r):!0===e.readable&&B(e.read)&&B(e.on)?c=new d(r):(n.File&&e instanceof File||e instanceof Object)&&(c=new g(r)),c.stream(e)},unparse:function(t,e){var n=!1,r=!0,i=",",o="\r\n",a='"',s=!1;"object"==typeof e&&("string"!=typeof e.delimiter||A.BAD_DELIMITERS.filter(function(t){return-1!==e.delimiter.indexOf(t)}).length||(i=e.delimiter),("boolean"==typeof e.quotes||Array.isArray(e.quotes))&&(n=e.quotes),"boolean"!=typeof e.skipEmptyLines&&"string"!=typeof e.skipEmptyLines||(s=e.skipEmptyLines),"string"==typeof e.newline&&(o=e.newline),"string"==typeof e.quoteChar&&(a=e.quoteChar),"boolean"==typeof e.header&&(r=e.header));var l=new RegExp(p(a),"g");if("string"==typeof t&&(t=JSON.parse(t)),Array.isArray(t)){if(!t.length||Array.isArray(t[0]))return u(null,t,s);if("object"==typeof t[0])return u(c(t[0]),t,s)}else if("object"==typeof t)return"string"==typeof t.data&&(t.data=JSON.parse(t.data)),Array.isArray(t.data)&&(t.fields||(t.fields=t.meta&&t.meta.fields),t.fields||(t.fields=Array.isArray(t.data[0])?t.fields:c(t.data[0])),Array.isArray(t.data[0])||"object"==typeof t.data[0]||(t.data=[t.data])),u(t.fields||[],t.data||[],s);throw"exception: Unable to serialize unrecognized input";function c(t){if("object"!=typeof t)return[];var e=[];for(var n in t)e.push(n);return e}function u(t,e,n){var a="";"string"==typeof t&&(t=JSON.parse(t)),"string"==typeof e&&(e=JSON.parse(e));var s=Array.isArray(t)&&0<t.length,A=!Array.isArray(e[0]);if(s&&r){for(var l=0;l<t.length;l++)0<l&&(a+=i),a+=g(t[l],l);0<e.length&&(a+=o)}for(var c=0;c<e.length;c++){var u=s?t.length:e[c].length,h=!1,d=s?0===Object.keys(e[c]).length:0===e[c].length;if(n&&!s&&(h="greedy"===n?""===e[c].join("").trim():1===e[c].length&&0===e[c][0].length),"greedy"===n&&s){for(var f=[],p=0;p<u;p++){var I=A?t[p]:p;f.push(e[c][I])}h=""===f.join("").trim()}if(!h){for(var C=0;C<u;C++){0<C&&!d&&(a+=i);var m=s&&A?t[C]:C;a+=g(e[c][m],C)}c<e.length-1&&(!n||0<u&&!d)&&(a+=o)}}return a}function g(t,e){return null==t?"":t.constructor===Date?JSON.stringify(t).slice(1,25):(t=t.toString().replace(l,a+a),"boolean"==typeof n&&n||Array.isArray(n)&&n[e]||function(t,e){for(var n=0;n<e.length;n++)if(-1<t.indexOf(e[n]))return!0;return!1}(t,A.BAD_DELIMITERS)||-1<t.indexOf(i)||" "===t.charAt(0)||" "===t.charAt(t.length-1)?a+t+a:t)}}};if(A.RECORD_SEP=String.fromCharCode(30),A.UNIT_SEP=String.fromCharCode(31),A.BYTE_ORDER_MARK="\ufeff",A.BAD_DELIMITERS=["\r","\n",'"',A.BYTE_ORDER_MARK],A.WORKERS_SUPPORTED=!r&&!!n.Worker,A.SCRIPT_PATH=null,A.NODE_STREAM_INPUT=1,A.LocalChunkSize=10485760,A.RemoteChunkSize=5242880,A.DefaultDelimiter=",",A.Parser=I,A.ParserHandle=f,A.NetworkStreamer=u,A.FileStreamer=g,A.StringStreamer=h,A.ReadableStreamStreamer=d,n.jQuery){var l=n.jQuery;l.fn.parse=function(t){var e=t.config||{},r=[];return this.each(function(t){if("INPUT"!==l(this).prop("tagName").toUpperCase()||"file"!==l(this).attr("type").toLowerCase()||!n.FileReader||!this.files||0===this.files.length)return!0;for(var i=0;i<this.files.length;i++)r.push({file:this.files[i],inputElem:this,instanceConfig:l.extend({},e)})}),i(),this;function i(){if(0!==r.length){var e,n,i,a=r[0];if(B(t.before)){var s=t.before(a.file,a.inputElem);if("object"==typeof s){if("abort"===s.action)return e=a.file,n=a.inputElem,i=s.reason,void(B(t.error)&&t.error({name:"AbortError"},e,n,i));if("skip"===s.action)return void o();"object"==typeof s.config&&(a.instanceConfig=l.extend(a.instanceConfig,s.config))}else if("skip"===s)return void o()}var c=a.instanceConfig.complete;a.instanceConfig.complete=function(t){B(c)&&c(t,a.file,a.inputElem),o()},A.parse(a.file,a.instanceConfig)}else B(t.complete)&&t.complete()}function o(){r.splice(0,1),i()}}}function c(t){this._handle=null,this._finished=!1,this._completed=!1,this._input=null,this._baseIndex=0,this._partialLine="",this._rowCount=0,this._start=0,this._nextChunk=null,this.isFirstChunk=!0,this._completeResults={data:[],errors:[],meta:{}},function(t){var e=E(t);e.chunkSize=parseInt(e.chunkSize),t.step||t.chunk||(e.chunkSize=null),this._handle=new f(e),(this._handle.streamer=this)._config=e}.call(this,t),this.parseChunk=function(t,e){if(this.isFirstChunk&&B(this._config.beforeFirstChunk)){var r=this._config.beforeFirstChunk(t);void 0!==r&&(t=r)}this.isFirstChunk=!1;var o=this._partialLine+t;this._partialLine="";var a=this._handle.parse(o,this._baseIndex,!this._finished);if(!this._handle.paused()&&!this._handle.aborted()){var s=a.meta.cursor;this._finished||(this._partialLine=o.substring(s-this._baseIndex),this._baseIndex=s),a&&a.data&&(this._rowCount+=a.data.length);var l=this._finished||this._config.preview&&this._rowCount>=this._config.preview;if(i)n.postMessage({results:a,workerId:A.WORKER_ID,finished:l});else if(B(this._config.chunk)&&!e){if(this._config.chunk(a,this._handle),this._handle.paused()||this._handle.aborted())return;a=void 0,this._completeResults=void 0}return this._config.step||this._config.chunk||(this._completeResults.data=this._completeResults.data.concat(a.data),this._completeResults.errors=this._completeResults.errors.concat(a.errors),this._completeResults.meta=a.meta),this._completed||!l||!B(this._config.complete)||a&&a.meta.aborted||(this._config.complete(this._completeResults,this._input),this._completed=!0),l||a&&a.meta.paused||this._nextChunk(),a}},this._sendError=function(t){B(this._config.error)?this._config.error(t):i&&this._config.error&&n.postMessage({workerId:A.WORKER_ID,error:t,finished:!1})}}function u(t){var e;(t=t||{}).chunkSize||(t.chunkSize=A.RemoteChunkSize),c.call(this,t),this._nextChunk=r?function(){this._readChunk(),this._chunkLoaded()}:function(){this._readChunk()},this.stream=function(t){this._input=t,this._nextChunk()},this._readChunk=function(){if(this._finished)this._chunkLoaded();else{if(e=new XMLHttpRequest,this._config.withCredentials&&(e.withCredentials=this._config.withCredentials),r||(e.onload=v(this._chunkLoaded,this),e.onerror=v(this._chunkError,this)),e.open("GET",this._input,!r),this._config.downloadRequestHeaders){var t=this._config.downloadRequestHeaders;for(var n in t)e.setRequestHeader(n,t[n])}if(this._config.chunkSize){var i=this._start+this._config.chunkSize-1;e.setRequestHeader("Range","bytes="+this._start+"-"+i),e.setRequestHeader("If-None-Match","webkit-no-cache")}try{e.send()}catch(t){this._chunkError(t.message)}r&&0===e.status?this._chunkError():this._start+=this._config.chunkSize}},this._chunkLoaded=function(){4===e.readyState&&(e.status<200||400<=e.status?this._chunkError():(this._finished=!this._config.chunkSize||this._start>function(t){var e=t.getResponseHeader("Content-Range");return null===e?-1:parseInt(e.substr(e.lastIndexOf("/")+1))}(e),this.parseChunk(e.responseText)))},this._chunkError=function(t){var n=e.statusText||t;this._sendError(new Error(n))}}function g(t){var e,n;(t=t||{}).chunkSize||(t.chunkSize=A.LocalChunkSize),c.call(this,t);var r="undefined"!=typeof FileReader;this.stream=function(t){this._input=t,n=t.slice||t.webkitSlice||t.mozSlice,r?((e=new FileReader).onload=v(this._chunkLoaded,this),e.onerror=v(this._chunkError,this)):e=new FileReaderSync,this._nextChunk()},this._nextChunk=function(){this._finished||this._config.preview&&!(this._rowCount<this._config.preview)||this._readChunk()},this._readChunk=function(){var t=this._input;if(this._config.chunkSize){var i=Math.min(this._start+this._config.chunkSize,this._input.size);t=n.call(t,this._start,i)}var o=e.readAsText(t,this._config.encoding);r||this._chunkLoaded({target:{result:o}})},this._chunkLoaded=function(t){this._start+=this._config.chunkSize,this._finished=!this._config.chunkSize||this._start>=this._input.size,this.parseChunk(t.target.result)},this._chunkError=function(){this._sendError(e.error)}}function h(t){var e;c.call(this,t=t||{}),this.stream=function(t){return e=t,this._nextChunk()},this._nextChunk=function(){if(!this._finished){var t=this._config.chunkSize,n=t?e.substr(0,t):e;return e=t?e.substr(t):"",this._finished=!e,this.parseChunk(n)}}}function d(t){c.call(this,t=t||{});var e=[],n=!0,r=!1;this.pause=function(){c.prototype.pause.apply(this,arguments),this._input.pause()},this.resume=function(){c.prototype.resume.apply(this,arguments),this._input.resume()},this.stream=function(t){this._input=t,this._input.on("data",this._streamData),this._input.on("end",this._streamEnd),this._input.on("error",this._streamError)},this._checkIsFinished=function(){r&&1===e.length&&(this._finished=!0)},this._nextChunk=function(){this._checkIsFinished(),e.length?this.parseChunk(e.shift()):n=!0},this._streamData=v(function(t){try{e.push("string"==typeof t?t:t.toString(this._config.encoding)),n&&(n=!1,this._checkIsFinished(),this.parseChunk(e.shift()))}catch(t){this._streamError(t)}},this),this._streamError=v(function(t){this._streamCleanUp(),this._sendError(t)},this),this._streamEnd=v(function(){this._streamCleanUp(),r=!0,this._streamData("")},this),this._streamCleanUp=v(function(){this._input.removeListener("data",this._streamData),this._input.removeListener("end",this._streamEnd),this._input.removeListener("error",this._streamError)},this)}function f(t){var e,n,r,i=/^\s*-?(\d*\.?\d+|\d+\.?\d*)(e[-+]?\d+)?\s*$/i,o=/(\d{4}-[01]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d\.\d+([+-][0-2]\d:[0-5]\d|Z))|(\d{4}-[01]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d([+-][0-2]\d:[0-5]\d|Z))|(\d{4}-[01]\d-[0-3]\dT[0-2]\d:[0-5]\d([+-][0-2]\d:[0-5]\d|Z))/,a=this,s=0,l=0,c=!1,u=!1,g=[],h={data:[],errors:[],meta:{}};if(B(t.step)){var d=t.step;t.step=function(e){if(h=e,m())C();else{if(C(),0===h.data.length)return;s+=e.data.length,t.preview&&s>t.preview?n.abort():d(h,a)}}}function f(e){return"greedy"===t.skipEmptyLines?""===e.join("").trim():1===e.length&&0===e[0].length}function C(){if(h&&r&&(v("Delimiter","UndetectableDelimiter","Unable to auto-detect delimiting character; defaulted to '"+A.DefaultDelimiter+"'"),r=!1),t.skipEmptyLines)for(var e=0;e<h.data.length;e++)f(h.data[e])&&h.data.splice(e--,1);return m()&&function(){if(h){for(var e=0;m()&&e<h.data.length;e++)for(var n=0;n<h.data[e].length;n++){var r=h.data[e][n];t.trimHeaders&&(r=r.trim()),g.push(r)}h.data.splice(0,1)}}(),function(){if(!h||!t.header&&!t.dynamicTyping&&!t.transform)return h;for(var e=0;e<h.data.length;e++){var n,r=t.header?{}:[];for(n=0;n<h.data[e].length;n++){var i=n,o=h.data[e][n];t.header&&(i=n>=g.length?"__parsed_extra":g[n]),t.transform&&(o=t.transform(o,i)),o=y(i,o),"__parsed_extra"===i?(r[i]=r[i]||[],r[i].push(o)):r[i]=o}h.data[e]=r,t.header&&(n>g.length?v("FieldMismatch","TooManyFields","Too many fields: expected "+g.length+" fields but parsed "+n,l+e):n<g.length&&v("FieldMismatch","TooFewFields","Too few fields: expected "+g.length+" fields but parsed "+n,l+e))}return t.header&&h.meta&&(h.meta.fields=g),l+=h.data.length,h}()}function m(){return t.header&&0===g.length}function y(e,n){return r=e,t.dynamicTypingFunction&&void 0===t.dynamicTyping[r]&&(t.dynamicTyping[r]=t.dynamicTypingFunction(r)),!0===(t.dynamicTyping[r]||t.dynamicTyping)?"true"===n||"TRUE"===n||"false"!==n&&"FALSE"!==n&&(i.test(n)?parseFloat(n):o.test(n)?new Date(n):""===n?null:n):n;var r}function v(t,e,n,r){h.errors.push({type:t,code:e,message:n,row:r})}this.parse=function(i,o,a){var s=t.quoteChar||'"';if(t.newline||(t.newline=function(t,e){t=t.substr(0,1048576);var n=new RegExp(p(e)+"([^]*?)"+p(e),"gm"),r=(t=t.replace(n,"")).split("\r"),i=t.split("\n"),o=1<i.length&&i[0].length<r[0].length;if(1===r.length||o)return"\n";for(var a=0,s=0;s<r.length;s++)"\n"===r[s][0]&&a++;return a>=r.length/2?"\r\n":"\r"}(i,s)),r=!1,t.delimiter)B(t.delimiter)&&(t.delimiter=t.delimiter(i),h.meta.delimiter=t.delimiter);else{var l=function(e,n,r,i){for(var o,a,s,l=[",","\t","|",";",A.RECORD_SEP,A.UNIT_SEP],c=0;c<l.length;c++){var u=l[c],g=0,h=0,d=0;s=void 0;for(var p=new I({comments:i,delimiter:u,newline:n,preview:10}).parse(e),C=0;C<p.data.length;C++)if(r&&f(p.data[C]))d++;else{var m=p.data[C].length;h+=m,void 0!==s?1<m&&(g+=Math.abs(m-s),s=m):s=0}0<p.data.length&&(h/=p.data.length-d),(void 0===a||a<g)&&1.99<h&&(a=g,o=u)}return{successful:!!(t.delimiter=o),bestDelimiter:o}}(i,t.newline,t.skipEmptyLines,t.comments);l.successful?t.delimiter=l.bestDelimiter:(r=!0,t.delimiter=A.DefaultDelimiter),h.meta.delimiter=t.delimiter}var u=E(t);return t.preview&&t.header&&u.preview++,e=i,n=new I(u),h=n.parse(e,o,a),C(),c?{meta:{paused:!0}}:h||{meta:{paused:!1}}},this.paused=function(){return c},this.pause=function(){c=!0,n.abort(),e=e.substr(n.getCharIndex())},this.resume=function(){c=!1,a.streamer.parseChunk(e,!0)},this.aborted=function(){return u},this.abort=function(){u=!0,n.abort(),h.meta.aborted=!0,B(t.complete)&&t.complete(h),e=""}}function p(t){return t.replace(/[.*+?^${}()|[\]\\]/g,"\\$&")}function I(t){var e,n=(t=t||{}).delimiter,r=t.newline,i=t.comments,o=t.step,a=t.preview,s=t.fastMode,l=e=void 0===t.quoteChar?'"':t.quoteChar;if(void 0!==t.escapeChar&&(l=t.escapeChar),("string"!=typeof n||-1<A.BAD_DELIMITERS.indexOf(n))&&(n=","),i===n)throw"Comment character same as delimiter";!0===i?i="#":("string"!=typeof i||-1<A.BAD_DELIMITERS.indexOf(i))&&(i=!1),"\n"!==r&&"\r"!==r&&"\r\n"!==r&&(r="\n");var c=0,u=!1;this.parse=function(t,A,g){if("string"!=typeof t)throw"Input must be a string";var h=t.length,d=n.length,f=r.length,I=i.length,C=B(o),m=[],y=[],E=[],v=c=0;if(!t)return F();if(s||!1!==s&&-1===t.indexOf(e)){for(var b=t.split(r),Q=0;Q<b.length;Q++){if(E=b[Q],c+=E.length,Q!==b.length-1)c+=r.length;else if(g)return F();if(!i||E.substr(0,I)!==i){if(C){if(m=[],S(E.split(n)),T(),u)return F()}else S(E.split(n));if(a&&a<=Q)return m=m.slice(0,a),F(!0)}}return F()}for(var w,x=t.indexOf(n,c),_=t.indexOf(r,c),D=new RegExp(p(l)+p(e),"g");;)if(t[c]!==e)if(i&&0===E.length&&t.substr(c,I)===i){if(-1===_)return F();c=_+f,_=t.indexOf(r,c),x=t.indexOf(n,c)}else if(-1!==x&&(x<_||-1===_))E.push(t.substring(c,x)),c=x+d,x=t.indexOf(n,c);else{if(-1===_)break;if(E.push(t.substring(c,_)),N(_+f),C&&(T(),u))return F();if(a&&m.length>=a)return F(!0)}else for(w=c,c++;;){if(-1===(w=t.indexOf(e,w+1)))return g||y.push({type:"Quotes",code:"MissingQuotes",message:"Quoted field unterminated",row:m.length,index:c}),R();if(w===h-1)return R(t.substring(c,w).replace(D,e));if(e!==l||t[w+1]!==l){if(e===l||0===w||t[w-1]!==l){var M=L(-1===_?x:Math.min(x,_));if(t[w+1+M]===n){E.push(t.substring(c,w).replace(D,e)),c=w+1+M+d,x=t.indexOf(n,c),_=t.indexOf(r,c);break}var k=L(_);if(t.substr(w+1+k,f)===r){if(E.push(t.substring(c,w).replace(D,e)),N(w+1+k+f),x=t.indexOf(n,c),C&&(T(),u))return F();if(a&&m.length>=a)return F(!0);break}y.push({type:"Quotes",code:"InvalidQuotes",message:"Trailing quote on quoted field is malformed",row:m.length,index:c}),w++}}else w++}return R();function S(t){m.push(t),v=c}function L(e){var n=0;if(-1!==e){var r=t.substring(w+1,e);r&&""===r.trim()&&(n=r.length)}return n}function R(e){return g||(void 0===e&&(e=t.substr(c)),E.push(e),c=h,S(E),C&&T()),F()}function N(e){c=e,S(E),E=[],_=t.indexOf(r,c)}function F(t){return{data:m,errors:y,meta:{delimiter:n,linebreak:r,aborted:u,truncated:!!t,cursor:v+(A||0)}}}function T(){o(F()),m=[],y=[]}},this.abort=function(){u=!0},this.getCharIndex=function(){return c}}function C(t){var e=t.data,n=a[e.workerId],r=!1;if(e.error)n.userError(e.error,e.file);else if(e.results&&e.results.data){var i={abort:function(){r=!0,m(e.workerId,{data:[],errors:[],meta:{aborted:!0}})},pause:y,resume:y};if(B(n.userStep)){for(var o=0;o<e.results.data.length&&(n.userStep({data:[e.results.data[o]],errors:e.results.errors,meta:e.results.meta},i),!r);o++);delete e.results}else B(n.userChunk)&&(n.userChunk(e.results,i,e.file),delete e.results)}e.finished&&!r&&m(e.workerId,e.results)}function m(t,e){var n=a[t];B(n.userComplete)&&n.userComplete(e),n.terminate(),delete a[t]}function y(){throw"Not implemented."}function E(t){if("object"!=typeof t||null===t)return t;var e=Array.isArray(t)?[]:{};for(var n in t)e[n]=E(t[n]);return e}function v(t,e){return function(){t.apply(e,arguments)}}function B(t){return"function"==typeof t}return i?n.onmessage=function(t){var e=t.data;if(void 0===A.WORKER_ID&&e&&(A.WORKER_ID=e.workerId),"string"==typeof e.input)n.postMessage({workerId:A.WORKER_ID,results:A.parse(e.input,e.config),finished:!0});else if(n.File&&e.input instanceof File||e.input instanceof Object){var r=A.parse(e.input,e.config);r&&n.postMessage({workerId:A.WORKER_ID,results:r,finished:!0})}}:A.WORKERS_SUPPORTED&&(e=document.getElementsByTagName("script"),t=e.length?e[e.length-1].src:"",document.body?document.addEventListener("DOMContentLoaded",function(){o=!0},!0):o=!0),(u.prototype=Object.create(c.prototype)).constructor=u,(g.prototype=Object.create(c.prototype)).constructor=g,(h.prototype=Object.create(h.prototype)).constructor=h,(d.prototype=Object.create(c.prototype)).constructor=d,A})?r.apply(e,[]):r)||(t.exports=i)},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(41),i=n(26),o=n(25),a=n(28),s=n(90),A=n(126),l=n(176),c=n(182);A.default.override({wasm:l,worker:c}),n(184),n(277),n(279);class u extends r.Part{constructor(t){super(t),this.setConfigCallback=()=>void 0,this.config="{}",this.oldData=[],this.chart=new a.Widget({node:document.createElement("perspective-viewer")}),this.chart.node.addEventListener("perspective-config-update",this),this.notifyResize=s.throttle(()=>{this.chart.node.style.width="100%",this.chart.node.style.height="100%",this.chart.node.notifyResize()},500)}static GetMetadata(){const t=super.GetMetadata();return t.description="Display a part in a Perspective pivot control.",t.remarks='\n[Perspective](https://github.com/finos/perspective/) is a control developed by\nJ.P. Morgan Chase and the [Financial Open Source Foundation (FINOS)](www.finos.org).\n\nPerspective is very powerful, and provides an Excel-like chart pivot experience\nas well as a number of chart visualizations. Using only a simple, drag-n-drop UI,\ndevelopers can quickly express a wide variety of charts with almost no effort.\n\n#### Pivoting\n\nClick on the triple-dot in the upper-left-hand corner of the part to open the\npivot UI. Table columns will appear on the left, and pivot operations (row\ngrouping, column grouping, filtering, sorting) will appear at the top. In the\nupper-left corner, you can pick what type of visualization to display (normally\na HyperGrid).\n\n#### Example\n\nBind the Input Table option to the following MQL query:\n\n```mql\nSELECT\n    x,\n    x * x as [y]\nFROM\n    Lattice(\'x = -10 to 10 step 1\')\n```\n\nThen, open the pivot UI and change the visualization to "Y Line Chart". Drag the\nx column to the "Group By" box at the top. You should now see a parabola in the\nchart.\n',t.addOption("Input Table",i.Types.Table,o.Table.NullTable(),{description:"The table to pivot with. Defaults to a null table."}),t.addOption("Config",i.Types.String,"",{description:"The serialized pivot config for Perspective. Bind this to a global to sync configs between charts."}),t}static MavenToPerspectiveSchema(t){const e={};for(let n=0;n<t.columnTypes.length;n++){let r=t.columnTypes[n];const o=t.columnNames[n];if(r===i.Types.Any&&(r=i.Converters.inferType(t.rows[0].getValue(n))),r!==i.Types.Any)switch(r){case i.Types.String:e[o]="string";break;case i.Types.Boolean:e[o]="boolean";break;case i.Types.Date:e[o]="date";break;case i.Types.DateTime:e[o]="datetime";break;case i.Types.Number:e[o]="float";break;default:e[o]="string"}else e[o]="string"}return e}dispose(){this.chart.node.removeEventListener("perspective-config-update",this),this.chart.node.delete()}handleEvent(t){"perspective-config-update"===t.type&&this.setConfigCallback.call(t)}initialize(){this.layout.insertWidget(0,this.chart)}render(t){this.setConfigCallback=()=>{this.config=JSON.stringify(this.chart.node.save()),t.set("Config",this.config)};const e=t.get("Config");if(""!==e&&e.length>1&&e!==this.config){const t=JSON.parse(this.config),n=JSON.parse(e);if(t.columns===n.columns)return;this.config=e,this.chart.node.restore(n)}const n=t.get("Input Table"),r=o.TableHelper.toObjectArray(n),i=u.MavenToPerspectiveSchema(n);if("{}"===JSON.stringify(i))throw Error("Cannot render empty table");s.isEqual(r,this.oldData)||(this.oldData=r,this.chart.node.load(i),this.chart.node.update(r))}getName(){return"PivotPart"}onResize(){this.isAttached&&this.notifyResize()}onActivateRequest(t){this.isAttached&&this.chart.node.focus({preventScroll:!0}),super.onActivateRequest(t)}onAfterShow(t){this.notifyResize(),super.onAfterShow(t)}}e.PivotPart=u},function(t,e,n){var r=n(47),i=n(46);t.exports=function(t){return function(e,n){var o,a,s=String(i(e)),A=r(n),l=s.length;return A<0||A>=l?t?"":void 0:(o=s.charCodeAt(A))<55296||o>56319||A+1===l||(a=s.charCodeAt(A+1))<56320||a>57343?t?s.charAt(A):o:t?s.slice(A,A+2):a-56320+(o-55296<<10)+65536}}},function(t,e,n){"use strict";var r=n(68);n(37)({target:"RegExp",proto:!0,forced:r!==/./.exec},{exec:r})},function(t,e,n){t.exports=n(49)("native-function-to-string",Function.toString)},function(t,e,n){"use strict";var r=n(163),i=n(97),o=n(38),a=n(30);t.exports=n(98)(Array,"Array",function(t,e){this._t=a(t),this._i=0,this._k=e},function(){var t=this._t,e=this._k,n=this._i++;return!t||n>=t.length?(this._t=void 0,i(1)):i(0,"keys"==e?n:"values"==e?t[n]:[n,t[n]])},"values"),o.Arguments=o.Array,r("keys"),r("values"),r("entries")},function(t,e,n){var r=n(7)("unscopables"),i=Array.prototype;null==i[r]&&n(22)(i,r,{}),t.exports=function(t){i[r][t]=!0}},function(t,e,n){var r=n(48);t.exports=Object("z").propertyIsEnumerable(0)?Object:function(t){return"String"==r(t)?t.split(""):Object(t)}},function(t,e,n){"use strict";var r=n(72),i=n(51),o=n(54),a={};n(22)(a,n(7)("iterator"),function(){return this}),t.exports=function(t,e,n){t.prototype=r(a,{next:i(1,n)}),o(t,e+" Iterator")}},function(t,e,n){var r=n(19),i=n(11),o=n(53);t.exports=n(16)?Object.defineProperties:function(t,e){i(t);for(var n,a=o(e),s=a.length,A=0;s>A;)r.f(t,n=a[A++],e[n]);return t}},function(t,e,n){var r=n(30),i=n(34),o=n(168);t.exports=function(t){return function(e,n,a){var s,A=r(e),l=i(A.length),c=o(a,l);if(t&&n!=n){for(;l>c;)if((s=A[c++])!=s)return!0}else for(;l>c;c++)if((t||c in A)&&A[c]===n)return t||c||0;return!t&&-1}}},function(t,e,n){var r=n(47),i=Math.max,o=Math.min;t.exports=function(t,e){return(t=r(t))<0?i(t+e,0):o(t,e)}},function(t,e,n){var r=n(13).document;t.exports=r&&r.documentElement},function(t,e,n){var r=n(27),i=n(45),o=n(73)("IE_PROTO"),a=Object.prototype;t.exports=Object.getPrototypeOf||function(t){return t=i(t),r(t,o)?t[o]:"function"==typeof t.constructor&&t instanceof t.constructor?t.constructor.prototype:t instanceof Object?a:null}},function(t,e,n){var r=n(15),i=n(48),o=n(7)("match");t.exports=function(t){var e;return r(t)&&(void 0!==(e=t[o])?!!e:"RegExp"==i(t))}},function(t,e,n){var r=n(11),i=n(71),o=n(7)("species");t.exports=function(t,e){var n,a=r(t).constructor;return void 0===a||null==(n=r(a)[o])?e:i(n)}},function(t,e){t.exports.default={types:{float:{filter_operator:"==",aggregate:"sum",format:{style:"decimal",minimumFractionDigits:2,maximumFractionDigits:2}},string:{filter_operator:"==",aggregate:"count"},integer:{filter_operator:"==",aggregate:"sum",format:{}},boolean:{filter_operator:"==",aggregate:"count"},datetime:{filter_operator:"==",aggregate:"count",format:{week:"numeric",year:"numeric",month:"numeric",day:"numeric",hour:"numeric",minute:"numeric",second:"numeric"}},date:{filter_operator:"==",aggregate:"count",format:{week:"numeric",year:"numeric",month:"numeric",day:"numeric"}}}}},function(t,e,n){n(16)&&"g"!=/./g.flags&&n(19).f(RegExp.prototype,"flags",{configurable:!0,get:n(69)})},function(t,e){t.exports=function(){var t=window.navigator.userAgent,e=t.indexOf("MSIE ");if(e>0)return parseInt(t.substring(e+5,t.indexOf(".",e)),10);if(t.indexOf("Trident/")>0){var n=t.indexOf("rv:");return parseInt(t.substring(n+3,t.indexOf(".",n)),10)}var r=t.indexOf("Edge/");return r>0&&parseInt(t.substring(r+5,t.indexOf(".",r)),10)}},function(t,e,n){t.exports=n(177)(" ")},function(t,e,n){"use strict";(function(e){t.exports=function(t){for(var n="undefined"!=typeof window&&"function"==typeof window.atob?window.atob(t):e.from(t,"base64").toString("binary"),r=new Uint8Array(n.length),i=0;i<n.length;++i)r[i]=n.charCodeAt(i);return r.buffer}}).call(this,n(178).Buffer)},function(t,e,n){"use strict";(function(t){
/*!
 * The buffer module from node.js, for the browser.
 *
 * @author   Feross Aboukhadijeh <feross@feross.org> <http://feross.org>
 * @license  MIT
 */
var r=n(179),i=n(180),o=n(181);function a(){return A.TYPED_ARRAY_SUPPORT?2147483647:1073741823}function s(t,e){if(a()<e)throw new RangeError("Invalid typed array length");return A.TYPED_ARRAY_SUPPORT?(t=new Uint8Array(e)).__proto__=A.prototype:(null===t&&(t=new A(e)),t.length=e),t}function A(t,e,n){if(!(A.TYPED_ARRAY_SUPPORT||this instanceof A))return new A(t,e,n);if("number"==typeof t){if("string"==typeof e)throw new Error("If encoding is specified then the first argument must be a string");return u(this,t)}return l(this,t,e,n)}function l(t,e,n,r){if("number"==typeof e)throw new TypeError('"value" argument must not be a number');return"undefined"!=typeof ArrayBuffer&&e instanceof ArrayBuffer?function(t,e,n,r){if(e.byteLength,n<0||e.byteLength<n)throw new RangeError("'offset' is out of bounds");if(e.byteLength<n+(r||0))throw new RangeError("'length' is out of bounds");return e=void 0===n&&void 0===r?new Uint8Array(e):void 0===r?new Uint8Array(e,n):new Uint8Array(e,n,r),A.TYPED_ARRAY_SUPPORT?(t=e).__proto__=A.prototype:t=g(t,e),t}(t,e,n,r):"string"==typeof e?function(t,e,n){if("string"==typeof n&&""!==n||(n="utf8"),!A.isEncoding(n))throw new TypeError('"encoding" must be a valid string encoding');var r=0|d(e,n),i=(t=s(t,r)).write(e,n);return i!==r&&(t=t.slice(0,i)),t}(t,e,n):function(t,e){if(A.isBuffer(e)){var n=0|h(e.length);return 0===(t=s(t,n)).length?t:(e.copy(t,0,0,n),t)}if(e){if("undefined"!=typeof ArrayBuffer&&e.buffer instanceof ArrayBuffer||"length"in e)return"number"!=typeof e.length||(r=e.length)!=r?s(t,0):g(t,e);if("Buffer"===e.type&&o(e.data))return g(t,e.data)}var r;throw new TypeError("First argument must be a string, Buffer, ArrayBuffer, Array, or array-like object.")}(t,e)}function c(t){if("number"!=typeof t)throw new TypeError('"size" argument must be a number');if(t<0)throw new RangeError('"size" argument must not be negative')}function u(t,e){if(c(e),t=s(t,e<0?0:0|h(e)),!A.TYPED_ARRAY_SUPPORT)for(var n=0;n<e;++n)t[n]=0;return t}function g(t,e){var n=e.length<0?0:0|h(e.length);t=s(t,n);for(var r=0;r<n;r+=1)t[r]=255&e[r];return t}function h(t){if(t>=a())throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x"+a().toString(16)+" bytes");return 0|t}function d(t,e){if(A.isBuffer(t))return t.length;if("undefined"!=typeof ArrayBuffer&&"function"==typeof ArrayBuffer.isView&&(ArrayBuffer.isView(t)||t instanceof ArrayBuffer))return t.byteLength;"string"!=typeof t&&(t=""+t);var n=t.length;if(0===n)return 0;for(var r=!1;;)switch(e){case"ascii":case"latin1":case"binary":return n;case"utf8":case"utf-8":case void 0:return H(t).length;case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return 2*n;case"hex":return n>>>1;case"base64":return j(t).length;default:if(r)return H(t).length;e=(""+e).toLowerCase(),r=!0}}function f(t,e,n){var r=!1;if((void 0===e||e<0)&&(e=0),e>this.length)return"";if((void 0===n||n>this.length)&&(n=this.length),n<=0)return"";if((n>>>=0)<=(e>>>=0))return"";for(t||(t="utf8");;)switch(t){case"hex":return M(this,e,n);case"utf8":case"utf-8":return w(this,e,n);case"ascii":return _(this,e,n);case"latin1":case"binary":return D(this,e,n);case"base64":return Q(this,e,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return k(this,e,n);default:if(r)throw new TypeError("Unknown encoding: "+t);t=(t+"").toLowerCase(),r=!0}}function p(t,e,n){var r=t[e];t[e]=t[n],t[n]=r}function I(t,e,n,r,i){if(0===t.length)return-1;if("string"==typeof n?(r=n,n=0):n>2147483647?n=2147483647:n<-2147483648&&(n=-2147483648),n=+n,isNaN(n)&&(n=i?0:t.length-1),n<0&&(n=t.length+n),n>=t.length){if(i)return-1;n=t.length-1}else if(n<0){if(!i)return-1;n=0}if("string"==typeof e&&(e=A.from(e,r)),A.isBuffer(e))return 0===e.length?-1:C(t,e,n,r,i);if("number"==typeof e)return e&=255,A.TYPED_ARRAY_SUPPORT&&"function"==typeof Uint8Array.prototype.indexOf?i?Uint8Array.prototype.indexOf.call(t,e,n):Uint8Array.prototype.lastIndexOf.call(t,e,n):C(t,[e],n,r,i);throw new TypeError("val must be string, number or Buffer")}function C(t,e,n,r,i){var o,a=1,s=t.length,A=e.length;if(void 0!==r&&("ucs2"===(r=String(r).toLowerCase())||"ucs-2"===r||"utf16le"===r||"utf-16le"===r)){if(t.length<2||e.length<2)return-1;a=2,s/=2,A/=2,n/=2}function l(t,e){return 1===a?t[e]:t.readUInt16BE(e*a)}if(i){var c=-1;for(o=n;o<s;o++)if(l(t,o)===l(e,-1===c?0:o-c)){if(-1===c&&(c=o),o-c+1===A)return c*a}else-1!==c&&(o-=o-c),c=-1}else for(n+A>s&&(n=s-A),o=n;o>=0;o--){for(var u=!0,g=0;g<A;g++)if(l(t,o+g)!==l(e,g)){u=!1;break}if(u)return o}return-1}function m(t,e,n,r){n=Number(n)||0;var i=t.length-n;r?(r=Number(r))>i&&(r=i):r=i;var o=e.length;if(o%2!=0)throw new TypeError("Invalid hex string");r>o/2&&(r=o/2);for(var a=0;a<r;++a){var s=parseInt(e.substr(2*a,2),16);if(isNaN(s))return a;t[n+a]=s}return a}function y(t,e,n,r){return Y(H(e,t.length-n),t,n,r)}function E(t,e,n,r){return Y(function(t){for(var e=[],n=0;n<t.length;++n)e.push(255&t.charCodeAt(n));return e}(e),t,n,r)}function v(t,e,n,r){return E(t,e,n,r)}function B(t,e,n,r){return Y(j(e),t,n,r)}function b(t,e,n,r){return Y(function(t,e){for(var n,r,i,o=[],a=0;a<t.length&&!((e-=2)<0);++a)r=(n=t.charCodeAt(a))>>8,i=n%256,o.push(i),o.push(r);return o}(e,t.length-n),t,n,r)}function Q(t,e,n){return 0===e&&n===t.length?r.fromByteArray(t):r.fromByteArray(t.slice(e,n))}function w(t,e,n){n=Math.min(t.length,n);for(var r=[],i=e;i<n;){var o,a,s,A,l=t[i],c=null,u=l>239?4:l>223?3:l>191?2:1;if(i+u<=n)switch(u){case 1:l<128&&(c=l);break;case 2:128==(192&(o=t[i+1]))&&(A=(31&l)<<6|63&o)>127&&(c=A);break;case 3:o=t[i+1],a=t[i+2],128==(192&o)&&128==(192&a)&&(A=(15&l)<<12|(63&o)<<6|63&a)>2047&&(A<55296||A>57343)&&(c=A);break;case 4:o=t[i+1],a=t[i+2],s=t[i+3],128==(192&o)&&128==(192&a)&&128==(192&s)&&(A=(15&l)<<18|(63&o)<<12|(63&a)<<6|63&s)>65535&&A<1114112&&(c=A)}null===c?(c=65533,u=1):c>65535&&(c-=65536,r.push(c>>>10&1023|55296),c=56320|1023&c),r.push(c),i+=u}return function(t){var e=t.length;if(e<=x)return String.fromCharCode.apply(String,t);for(var n="",r=0;r<e;)n+=String.fromCharCode.apply(String,t.slice(r,r+=x));return n}(r)}e.Buffer=A,e.SlowBuffer=function(t){return+t!=t&&(t=0),A.alloc(+t)},e.INSPECT_MAX_BYTES=50,A.TYPED_ARRAY_SUPPORT=void 0!==t.TYPED_ARRAY_SUPPORT?t.TYPED_ARRAY_SUPPORT:function(){try{var t=new Uint8Array(1);return t.__proto__={__proto__:Uint8Array.prototype,foo:function(){return 42}},42===t.foo()&&"function"==typeof t.subarray&&0===t.subarray(1,1).byteLength}catch(t){return!1}}(),e.kMaxLength=a(),A.poolSize=8192,A._augment=function(t){return t.__proto__=A.prototype,t},A.from=function(t,e,n){return l(null,t,e,n)},A.TYPED_ARRAY_SUPPORT&&(A.prototype.__proto__=Uint8Array.prototype,A.__proto__=Uint8Array,"undefined"!=typeof Symbol&&Symbol.species&&A[Symbol.species]===A&&Object.defineProperty(A,Symbol.species,{value:null,configurable:!0})),A.alloc=function(t,e,n){return function(t,e,n,r){return c(e),e<=0?s(t,e):void 0!==n?"string"==typeof r?s(t,e).fill(n,r):s(t,e).fill(n):s(t,e)}(null,t,e,n)},A.allocUnsafe=function(t){return u(null,t)},A.allocUnsafeSlow=function(t){return u(null,t)},A.isBuffer=function(t){return!(null==t||!t._isBuffer)},A.compare=function(t,e){if(!A.isBuffer(t)||!A.isBuffer(e))throw new TypeError("Arguments must be Buffers");if(t===e)return 0;for(var n=t.length,r=e.length,i=0,o=Math.min(n,r);i<o;++i)if(t[i]!==e[i]){n=t[i],r=e[i];break}return n<r?-1:r<n?1:0},A.isEncoding=function(t){switch(String(t).toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"latin1":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return!0;default:return!1}},A.concat=function(t,e){if(!o(t))throw new TypeError('"list" argument must be an Array of Buffers');if(0===t.length)return A.alloc(0);var n;if(void 0===e)for(e=0,n=0;n<t.length;++n)e+=t[n].length;var r=A.allocUnsafe(e),i=0;for(n=0;n<t.length;++n){var a=t[n];if(!A.isBuffer(a))throw new TypeError('"list" argument must be an Array of Buffers');a.copy(r,i),i+=a.length}return r},A.byteLength=d,A.prototype._isBuffer=!0,A.prototype.swap16=function(){var t=this.length;if(t%2!=0)throw new RangeError("Buffer size must be a multiple of 16-bits");for(var e=0;e<t;e+=2)p(this,e,e+1);return this},A.prototype.swap32=function(){var t=this.length;if(t%4!=0)throw new RangeError("Buffer size must be a multiple of 32-bits");for(var e=0;e<t;e+=4)p(this,e,e+3),p(this,e+1,e+2);return this},A.prototype.swap64=function(){var t=this.length;if(t%8!=0)throw new RangeError("Buffer size must be a multiple of 64-bits");for(var e=0;e<t;e+=8)p(this,e,e+7),p(this,e+1,e+6),p(this,e+2,e+5),p(this,e+3,e+4);return this},A.prototype.toString=function(){var t=0|this.length;return 0===t?"":0===arguments.length?w(this,0,t):f.apply(this,arguments)},A.prototype.equals=function(t){if(!A.isBuffer(t))throw new TypeError("Argument must be a Buffer");return this===t||0===A.compare(this,t)},A.prototype.inspect=function(){var t="",n=e.INSPECT_MAX_BYTES;return this.length>0&&(t=this.toString("hex",0,n).match(/.{2}/g).join(" "),this.length>n&&(t+=" ... ")),"<Buffer "+t+">"},A.prototype.compare=function(t,e,n,r,i){if(!A.isBuffer(t))throw new TypeError("Argument must be a Buffer");if(void 0===e&&(e=0),void 0===n&&(n=t?t.length:0),void 0===r&&(r=0),void 0===i&&(i=this.length),e<0||n>t.length||r<0||i>this.length)throw new RangeError("out of range index");if(r>=i&&e>=n)return 0;if(r>=i)return-1;if(e>=n)return 1;if(this===t)return 0;for(var o=(i>>>=0)-(r>>>=0),a=(n>>>=0)-(e>>>=0),s=Math.min(o,a),l=this.slice(r,i),c=t.slice(e,n),u=0;u<s;++u)if(l[u]!==c[u]){o=l[u],a=c[u];break}return o<a?-1:a<o?1:0},A.prototype.includes=function(t,e,n){return-1!==this.indexOf(t,e,n)},A.prototype.indexOf=function(t,e,n){return I(this,t,e,n,!0)},A.prototype.lastIndexOf=function(t,e,n){return I(this,t,e,n,!1)},A.prototype.write=function(t,e,n,r){if(void 0===e)r="utf8",n=this.length,e=0;else if(void 0===n&&"string"==typeof e)r=e,n=this.length,e=0;else{if(!isFinite(e))throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");e|=0,isFinite(n)?(n|=0,void 0===r&&(r="utf8")):(r=n,n=void 0)}var i=this.length-e;if((void 0===n||n>i)&&(n=i),t.length>0&&(n<0||e<0)||e>this.length)throw new RangeError("Attempt to write outside buffer bounds");r||(r="utf8");for(var o=!1;;)switch(r){case"hex":return m(this,t,e,n);case"utf8":case"utf-8":return y(this,t,e,n);case"ascii":return E(this,t,e,n);case"latin1":case"binary":return v(this,t,e,n);case"base64":return B(this,t,e,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return b(this,t,e,n);default:if(o)throw new TypeError("Unknown encoding: "+r);r=(""+r).toLowerCase(),o=!0}},A.prototype.toJSON=function(){return{type:"Buffer",data:Array.prototype.slice.call(this._arr||this,0)}};var x=4096;function _(t,e,n){var r="";n=Math.min(t.length,n);for(var i=e;i<n;++i)r+=String.fromCharCode(127&t[i]);return r}function D(t,e,n){var r="";n=Math.min(t.length,n);for(var i=e;i<n;++i)r+=String.fromCharCode(t[i]);return r}function M(t,e,n){var r=t.length;(!e||e<0)&&(e=0),(!n||n<0||n>r)&&(n=r);for(var i="",o=e;o<n;++o)i+=G(t[o]);return i}function k(t,e,n){for(var r=t.slice(e,n),i="",o=0;o<r.length;o+=2)i+=String.fromCharCode(r[o]+256*r[o+1]);return i}function S(t,e,n){if(t%1!=0||t<0)throw new RangeError("offset is not uint");if(t+e>n)throw new RangeError("Trying to access beyond buffer length")}function L(t,e,n,r,i,o){if(!A.isBuffer(t))throw new TypeError('"buffer" argument must be a Buffer instance');if(e>i||e<o)throw new RangeError('"value" argument is out of bounds');if(n+r>t.length)throw new RangeError("Index out of range")}function R(t,e,n,r){e<0&&(e=65535+e+1);for(var i=0,o=Math.min(t.length-n,2);i<o;++i)t[n+i]=(e&255<<8*(r?i:1-i))>>>8*(r?i:1-i)}function N(t,e,n,r){e<0&&(e=4294967295+e+1);for(var i=0,o=Math.min(t.length-n,4);i<o;++i)t[n+i]=e>>>8*(r?i:3-i)&255}function F(t,e,n,r,i,o){if(n+r>t.length)throw new RangeError("Index out of range");if(n<0)throw new RangeError("Index out of range")}function T(t,e,n,r,o){return o||F(t,0,n,4),i.write(t,e,n,r,23,4),n+4}function U(t,e,n,r,o){return o||F(t,0,n,8),i.write(t,e,n,r,52,8),n+8}A.prototype.slice=function(t,e){var n,r=this.length;if((t=~~t)<0?(t+=r)<0&&(t=0):t>r&&(t=r),(e=void 0===e?r:~~e)<0?(e+=r)<0&&(e=0):e>r&&(e=r),e<t&&(e=t),A.TYPED_ARRAY_SUPPORT)(n=this.subarray(t,e)).__proto__=A.prototype;else{var i=e-t;n=new A(i,void 0);for(var o=0;o<i;++o)n[o]=this[o+t]}return n},A.prototype.readUIntLE=function(t,e,n){t|=0,e|=0,n||S(t,e,this.length);for(var r=this[t],i=1,o=0;++o<e&&(i*=256);)r+=this[t+o]*i;return r},A.prototype.readUIntBE=function(t,e,n){t|=0,e|=0,n||S(t,e,this.length);for(var r=this[t+--e],i=1;e>0&&(i*=256);)r+=this[t+--e]*i;return r},A.prototype.readUInt8=function(t,e){return e||S(t,1,this.length),this[t]},A.prototype.readUInt16LE=function(t,e){return e||S(t,2,this.length),this[t]|this[t+1]<<8},A.prototype.readUInt16BE=function(t,e){return e||S(t,2,this.length),this[t]<<8|this[t+1]},A.prototype.readUInt32LE=function(t,e){return e||S(t,4,this.length),(this[t]|this[t+1]<<8|this[t+2]<<16)+16777216*this[t+3]},A.prototype.readUInt32BE=function(t,e){return e||S(t,4,this.length),16777216*this[t]+(this[t+1]<<16|this[t+2]<<8|this[t+3])},A.prototype.readIntLE=function(t,e,n){t|=0,e|=0,n||S(t,e,this.length);for(var r=this[t],i=1,o=0;++o<e&&(i*=256);)r+=this[t+o]*i;return r>=(i*=128)&&(r-=Math.pow(2,8*e)),r},A.prototype.readIntBE=function(t,e,n){t|=0,e|=0,n||S(t,e,this.length);for(var r=e,i=1,o=this[t+--r];r>0&&(i*=256);)o+=this[t+--r]*i;return o>=(i*=128)&&(o-=Math.pow(2,8*e)),o},A.prototype.readInt8=function(t,e){return e||S(t,1,this.length),128&this[t]?-1*(255-this[t]+1):this[t]},A.prototype.readInt16LE=function(t,e){e||S(t,2,this.length);var n=this[t]|this[t+1]<<8;return 32768&n?4294901760|n:n},A.prototype.readInt16BE=function(t,e){e||S(t,2,this.length);var n=this[t+1]|this[t]<<8;return 32768&n?4294901760|n:n},A.prototype.readInt32LE=function(t,e){return e||S(t,4,this.length),this[t]|this[t+1]<<8|this[t+2]<<16|this[t+3]<<24},A.prototype.readInt32BE=function(t,e){return e||S(t,4,this.length),this[t]<<24|this[t+1]<<16|this[t+2]<<8|this[t+3]},A.prototype.readFloatLE=function(t,e){return e||S(t,4,this.length),i.read(this,t,!0,23,4)},A.prototype.readFloatBE=function(t,e){return e||S(t,4,this.length),i.read(this,t,!1,23,4)},A.prototype.readDoubleLE=function(t,e){return e||S(t,8,this.length),i.read(this,t,!0,52,8)},A.prototype.readDoubleBE=function(t,e){return e||S(t,8,this.length),i.read(this,t,!1,52,8)},A.prototype.writeUIntLE=function(t,e,n,r){t=+t,e|=0,n|=0,r||L(this,t,e,n,Math.pow(2,8*n)-1,0);var i=1,o=0;for(this[e]=255&t;++o<n&&(i*=256);)this[e+o]=t/i&255;return e+n},A.prototype.writeUIntBE=function(t,e,n,r){t=+t,e|=0,n|=0,r||L(this,t,e,n,Math.pow(2,8*n)-1,0);var i=n-1,o=1;for(this[e+i]=255&t;--i>=0&&(o*=256);)this[e+i]=t/o&255;return e+n},A.prototype.writeUInt8=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,1,255,0),A.TYPED_ARRAY_SUPPORT||(t=Math.floor(t)),this[e]=255&t,e+1},A.prototype.writeUInt16LE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,2,65535,0),A.TYPED_ARRAY_SUPPORT?(this[e]=255&t,this[e+1]=t>>>8):R(this,t,e,!0),e+2},A.prototype.writeUInt16BE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,2,65535,0),A.TYPED_ARRAY_SUPPORT?(this[e]=t>>>8,this[e+1]=255&t):R(this,t,e,!1),e+2},A.prototype.writeUInt32LE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,4,4294967295,0),A.TYPED_ARRAY_SUPPORT?(this[e+3]=t>>>24,this[e+2]=t>>>16,this[e+1]=t>>>8,this[e]=255&t):N(this,t,e,!0),e+4},A.prototype.writeUInt32BE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,4,4294967295,0),A.TYPED_ARRAY_SUPPORT?(this[e]=t>>>24,this[e+1]=t>>>16,this[e+2]=t>>>8,this[e+3]=255&t):N(this,t,e,!1),e+4},A.prototype.writeIntLE=function(t,e,n,r){if(t=+t,e|=0,!r){var i=Math.pow(2,8*n-1);L(this,t,e,n,i-1,-i)}var o=0,a=1,s=0;for(this[e]=255&t;++o<n&&(a*=256);)t<0&&0===s&&0!==this[e+o-1]&&(s=1),this[e+o]=(t/a>>0)-s&255;return e+n},A.prototype.writeIntBE=function(t,e,n,r){if(t=+t,e|=0,!r){var i=Math.pow(2,8*n-1);L(this,t,e,n,i-1,-i)}var o=n-1,a=1,s=0;for(this[e+o]=255&t;--o>=0&&(a*=256);)t<0&&0===s&&0!==this[e+o+1]&&(s=1),this[e+o]=(t/a>>0)-s&255;return e+n},A.prototype.writeInt8=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,1,127,-128),A.TYPED_ARRAY_SUPPORT||(t=Math.floor(t)),t<0&&(t=255+t+1),this[e]=255&t,e+1},A.prototype.writeInt16LE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,2,32767,-32768),A.TYPED_ARRAY_SUPPORT?(this[e]=255&t,this[e+1]=t>>>8):R(this,t,e,!0),e+2},A.prototype.writeInt16BE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,2,32767,-32768),A.TYPED_ARRAY_SUPPORT?(this[e]=t>>>8,this[e+1]=255&t):R(this,t,e,!1),e+2},A.prototype.writeInt32LE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,4,2147483647,-2147483648),A.TYPED_ARRAY_SUPPORT?(this[e]=255&t,this[e+1]=t>>>8,this[e+2]=t>>>16,this[e+3]=t>>>24):N(this,t,e,!0),e+4},A.prototype.writeInt32BE=function(t,e,n){return t=+t,e|=0,n||L(this,t,e,4,2147483647,-2147483648),t<0&&(t=4294967295+t+1),A.TYPED_ARRAY_SUPPORT?(this[e]=t>>>24,this[e+1]=t>>>16,this[e+2]=t>>>8,this[e+3]=255&t):N(this,t,e,!1),e+4},A.prototype.writeFloatLE=function(t,e,n){return T(this,t,e,!0,n)},A.prototype.writeFloatBE=function(t,e,n){return T(this,t,e,!1,n)},A.prototype.writeDoubleLE=function(t,e,n){return U(this,t,e,!0,n)},A.prototype.writeDoubleBE=function(t,e,n){return U(this,t,e,!1,n)},A.prototype.copy=function(t,e,n,r){if(n||(n=0),r||0===r||(r=this.length),e>=t.length&&(e=t.length),e||(e=0),r>0&&r<n&&(r=n),r===n)return 0;if(0===t.length||0===this.length)return 0;if(e<0)throw new RangeError("targetStart out of bounds");if(n<0||n>=this.length)throw new RangeError("sourceStart out of bounds");if(r<0)throw new RangeError("sourceEnd out of bounds");r>this.length&&(r=this.length),t.length-e<r-n&&(r=t.length-e+n);var i,o=r-n;if(this===t&&n<e&&e<r)for(i=o-1;i>=0;--i)t[i+e]=this[i+n];else if(o<1e3||!A.TYPED_ARRAY_SUPPORT)for(i=0;i<o;++i)t[i+e]=this[i+n];else Uint8Array.prototype.set.call(t,this.subarray(n,n+o),e);return o},A.prototype.fill=function(t,e,n,r){if("string"==typeof t){if("string"==typeof e?(r=e,e=0,n=this.length):"string"==typeof n&&(r=n,n=this.length),1===t.length){var i=t.charCodeAt(0);i<256&&(t=i)}if(void 0!==r&&"string"!=typeof r)throw new TypeError("encoding must be a string");if("string"==typeof r&&!A.isEncoding(r))throw new TypeError("Unknown encoding: "+r)}else"number"==typeof t&&(t&=255);if(e<0||this.length<e||this.length<n)throw new RangeError("Out of range index");if(n<=e)return this;var o;if(e>>>=0,n=void 0===n?this.length:n>>>0,t||(t=0),"number"==typeof t)for(o=e;o<n;++o)this[o]=t;else{var a=A.isBuffer(t)?t:H(new A(t,r).toString()),s=a.length;for(o=0;o<n-e;++o)this[o+e]=a[o%s]}return this};var O=/[^+\/0-9A-Za-z-_]/g;function G(t){return t<16?"0"+t.toString(16):t.toString(16)}function H(t,e){var n;e=e||1/0;for(var r=t.length,i=null,o=[],a=0;a<r;++a){if((n=t.charCodeAt(a))>55295&&n<57344){if(!i){if(n>56319){(e-=3)>-1&&o.push(239,191,189);continue}if(a+1===r){(e-=3)>-1&&o.push(239,191,189);continue}i=n;continue}if(n<56320){(e-=3)>-1&&o.push(239,191,189),i=n;continue}n=65536+(i-55296<<10|n-56320)}else i&&(e-=3)>-1&&o.push(239,191,189);if(i=null,n<128){if((e-=1)<0)break;o.push(n)}else if(n<2048){if((e-=2)<0)break;o.push(n>>6|192,63&n|128)}else if(n<65536){if((e-=3)<0)break;o.push(n>>12|224,n>>6&63|128,63&n|128)}else{if(!(n<1114112))throw new Error("Invalid code point");if((e-=4)<0)break;o.push(n>>18|240,n>>12&63|128,n>>6&63|128,63&n|128)}}return o}function j(t){return r.toByteArray(function(t){if((t=function(t){return t.trim?t.trim():t.replace(/^\s+|\s+$/g,"")}(t).replace(O,"")).length<2)return"";for(;t.length%4!=0;)t+="=";return t}(t))}function Y(t,e,n,r){for(var i=0;i<r&&!(i+n>=e.length||i>=t.length);++i)e[i+n]=t[i];return i}}).call(this,n(21))},function(t,e,n){"use strict";e.byteLength=function(t){var e=l(t),n=e[0],r=e[1];return 3*(n+r)/4-r},e.toByteArray=function(t){for(var e,n=l(t),r=n[0],a=n[1],s=new o(3*(r+a)/4-a),A=0,c=a>0?r-4:r,u=0;u<c;u+=4)e=i[t.charCodeAt(u)]<<18|i[t.charCodeAt(u+1)]<<12|i[t.charCodeAt(u+2)]<<6|i[t.charCodeAt(u+3)],s[A++]=e>>16&255,s[A++]=e>>8&255,s[A++]=255&e;return 2===a&&(e=i[t.charCodeAt(u)]<<2|i[t.charCodeAt(u+1)]>>4,s[A++]=255&e),1===a&&(e=i[t.charCodeAt(u)]<<10|i[t.charCodeAt(u+1)]<<4|i[t.charCodeAt(u+2)]>>2,s[A++]=e>>8&255,s[A++]=255&e),s},e.fromByteArray=function(t){for(var e,n=t.length,i=n%3,o=[],a=0,s=n-i;a<s;a+=16383)o.push(c(t,a,a+16383>s?s:a+16383));return 1===i?(e=t[n-1],o.push(r[e>>2]+r[e<<4&63]+"==")):2===i&&(e=(t[n-2]<<8)+t[n-1],o.push(r[e>>10]+r[e>>4&63]+r[e<<2&63]+"=")),o.join("")};for(var r=[],i=[],o="undefined"!=typeof Uint8Array?Uint8Array:Array,a="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",s=0,A=a.length;s<A;++s)r[s]=a[s],i[a.charCodeAt(s)]=s;function l(t){var e=t.length;if(e%4>0)throw new Error("Invalid string. Length must be a multiple of 4");var n=t.indexOf("=");return-1===n&&(n=e),[n,n===e?0:4-n%4]}function c(t,e,n){for(var i,o,a=[],s=e;s<n;s+=3)i=(t[s]<<16&16711680)+(t[s+1]<<8&65280)+(255&t[s+2]),a.push(r[(o=i)>>18&63]+r[o>>12&63]+r[o>>6&63]+r[63&o]);return a.join("")}i["-".charCodeAt(0)]=62,i["_".charCodeAt(0)]=63},function(t,e){e.read=function(t,e,n,r,i){var o,a,s=8*i-r-1,A=(1<<s)-1,l=A>>1,c=-7,u=n?i-1:0,g=n?-1:1,h=t[e+u];for(u+=g,o=h&(1<<-c)-1,h>>=-c,c+=s;c>0;o=256*o+t[e+u],u+=g,c-=8);for(a=o&(1<<-c)-1,o>>=-c,c+=r;c>0;a=256*a+t[e+u],u+=g,c-=8);if(0===o)o=1-l;else{if(o===A)return a?NaN:1/0*(h?-1:1);a+=Math.pow(2,r),o-=l}return(h?-1:1)*a*Math.pow(2,o-r)},e.write=function(t,e,n,r,i,o){var a,s,A,l=8*o-i-1,c=(1<<l)-1,u=c>>1,g=23===i?Math.pow(2,-24)-Math.pow(2,-77):0,h=r?0:o-1,d=r?1:-1,f=e<0||0===e&&1/e<0?1:0;for(e=Math.abs(e),isNaN(e)||e===1/0?(s=isNaN(e)?1:0,a=c):(a=Math.floor(Math.log(e)/Math.LN2),e*(A=Math.pow(2,-a))<1&&(a--,A*=2),(e+=a+u>=1?g/A:g*Math.pow(2,1-u))*A>=2&&(a++,A/=2),a+u>=c?(s=0,a=c):a+u>=1?(s=(e*A-1)*Math.pow(2,i),a+=u):(s=e*Math.pow(2,u-1)*Math.pow(2,i),a=0));i>=8;t[n+h]=255&s,h+=d,s/=256,i-=8);for(a=a<<i|s,l+=i;l>0;t[n+h]=255&a,h+=d,a/=256,l-=8);t[n+h-d]|=128*f}},function(t,e){var n={}.toString;t.exports=Array.isArray||function(t){return"[object Array]"==n.call(t)}},function(t,e,n){t.exports=function(){return n(183)('!function(t){var e={};function n(r){if(e[r])return e[r].exports;var i=e[r]={i:r,l:!1,exports:{}};return t[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}n.m=t,n.c=e,n.d=function(t,e,r){n.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:r})},n.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},n.t=function(t,e){if(1&e&&(t=n(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var i in t)n.d(r,i,function(e){return t[e]}.bind(null,i));return r},n.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(e,"a",e),e},n.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},n.p="",n(n.s=1)}([function(t,e){},function(module,exports,__webpack_require__){var require;!function(t){var e={};function n(r){if(e[r])return e[r].exports;var i=e[r]={i:r,l:!1,exports:{}};return t[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}n.m=t,n.c=e,n.d=function(t,e,r){n.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:r})},n.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},n.t=function(t,e){if(1&e&&(t=n(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var i in t)n.d(r,i,function(e){return t[e]}.bind(null,i));return r},n.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(e,"a",e),e},n.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},n.p="",n(n.s=68)}([function(t,e,n){"use strict";n.d(e,"g",function(){return i}),n.d(e,"a",function(){return o}),n.d(e,"f",function(){return a}),n.d(e,"h",function(){return s}),n.d(e,"k",function(){return u}),n.d(e,"i",function(){return c}),n.d(e,"j",function(){return f}),n.d(e,"e",function(){return l}),n.d(e,"c",function(){return h}),n.d(e,"b",function(){return p}),n.d(e,"d",function(){return d});\n/*! *****************************************************************************\nCopyright (c) Microsoft Corporation. All rights reserved.\nLicensed under the Apache License, Version 2.0 (the "License"); you may not use\nthis file except in compliance with the License. You may obtain a copy of the\nLicense at http://www.apache.org/licenses/LICENSE-2.0\n\nTHIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY\nKIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED\nWARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,\nMERCHANTABLITY OR NON-INFRINGEMENT.\n\nSee the Apache Version 2.0 License for specific language governing permissions\nand limitations under the License.\n***************************************************************************** */\nvar r=function(t,e){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)};function i(t,e){function n(){this.constructor=t}r(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)}var o=function(){return(o=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)};function a(t,e,n,r){return new(n||(n=Promise))(function(i,o){function a(t){try{u(r.next(t))}catch(t){o(t)}}function s(t){try{u(r.throw(t))}catch(t){o(t)}}function u(t){t.done?i(t.value):new n(function(e){e(t.value)}).then(a,s)}u((r=r.apply(t,e||[])).next())})}function s(t,e){var n,r,i,o,a={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return o={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(o[Symbol.iterator]=function(){return this}),o;function s(o){return function(s){return function(o){if(n)throw new TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(i=2&o[0]?r.return:o[0]?r.throw||((i=r.return)&&i.call(r),0):r.next)&&!(i=i.call(r,o[1])).done)return i;switch(r=0,i&&(o=[2&o[0],i.value]),o[0]){case 0:case 1:i=o;break;case 4:return a.label++,{value:o[1],done:!1};case 5:a.label++,r=o[1],o=[0];continue;case 7:o=a.ops.pop(),a.trys.pop();continue;default:if(!(i=(i=a.trys).length>0&&i[i.length-1])&&(6===o[0]||2===o[0])){a=0;continue}if(3===o[0]&&(!i||o[1]>i[0]&&o[1]<i[3])){a.label=o[1];break}if(6===o[0]&&a.label<i[1]){a.label=i[1],i=o;break}if(i&&a.label<i[2]){a.label=i[2],a.ops.push(o);break}i[2]&&a.ops.pop(),a.trys.pop();continue}o=e.call(t,a)}catch(t){o=[6,t],r=0}finally{n=i=0}if(5&o[0])throw o[1];return{value:o[0]?o[1]:void 0,done:!0}}([o,s])}}}function u(t){var e="function"==typeof Symbol&&t[Symbol.iterator],n=0;return e?e.call(t):{next:function(){return t&&n>=t.length&&(t=void 0),{value:t&&t[n++],done:!t}}}}function c(t,e){var n="function"==typeof Symbol&&t[Symbol.iterator];if(!n)return t;var r,i,o=n.call(t),a=[];try{for(;(void 0===e||e-- >0)&&!(r=o.next()).done;)a.push(r.value)}catch(t){i={error:t}}finally{try{r&&!r.done&&(n=o.return)&&n.call(o)}finally{if(i)throw i.error}}return a}function f(){for(var t=[],e=0;e<arguments.length;e++)t=t.concat(c(arguments[e]));return t}function l(t){return this instanceof l?(this.v=t,this):new l(t)}function h(t,e,n){if(!Symbol.asyncIterator)throw new TypeError("Symbol.asyncIterator is not defined.");var r,i=n.apply(t,e||[]),o=[];return r={},a("next"),a("throw"),a("return"),r[Symbol.asyncIterator]=function(){return this},r;function a(t){i[t]&&(r[t]=function(e){return new Promise(function(n,r){o.push([t,e,n,r])>1||s(t,e)})})}function s(t,e){try{(n=i[t](e)).value instanceof l?Promise.resolve(n.value.v).then(u,c):f(o[0][2],n)}catch(t){f(o[0][3],t)}var n}function u(t){s("next",t)}function c(t){s("throw",t)}function f(t,e){t(e),o.shift(),o.length&&s(o[0][0],o[0][1])}}function p(t){var e,n;return e={},r("next"),r("throw",function(t){throw t}),r("return"),e[Symbol.iterator]=function(){return this},e;function r(r,i){e[r]=t[r]?function(e){return(n=!n)?{value:l(t[r](e)),done:"return"===r}:i?i(e):e}:i}}function d(t){if(!Symbol.asyncIterator)throw new TypeError("Symbol.asyncIterator is not defined.");var e,n=t[Symbol.asyncIterator];return n?n.call(t):(t=u(t),e={},r("next"),r("throw"),r("return"),e[Symbol.asyncIterator]=function(){return this},e);function r(n){e[n]=t[n]&&function(e){return new Promise(function(r,i){!function(t,e,n,r){Promise.resolve(r).then(function(e){t({value:e,done:n})},e)}(r,i,(e=t[n](e)).done,e.value)})}}}},function(t,e,n){"use strict";n.d(e,"a",function(){return a}),n.d(e,"b",function(){return s}),n.d(e,"g",function(){return u}),n.d(e,"f",function(){return c}),n.d(e,"i",function(){return f}),n.d(e,"j",function(){return l}),n.d(e,"c",function(){return h}),n.d(e,"d",function(){return p}),n.d(e,"e",function(){return d}),n.d(e,"h",function(){return r});var r,i=n(5),o=n(21),a=i.a.apache.arrow.flatbuf.Type,s=i.a.apache.arrow.flatbuf.DateUnit,u=i.a.apache.arrow.flatbuf.TimeUnit,c=i.a.apache.arrow.flatbuf.Precision,f=i.a.apache.arrow.flatbuf.UnionMode,l=i.a.apache.arrow.flatbuf.VectorType,h=i.a.apache.arrow.flatbuf.IntervalUnit,p=o.a.apache.arrow.flatbuf.MessageHeader,d=i.a.apache.arrow.flatbuf.MetadataVersion;!function(t){t[t.NONE=0]="NONE",t[t.Null=1]="Null",t[t.Int=2]="Int",t[t.Float=3]="Float",t[t.Binary=4]="Binary",t[t.Utf8=5]="Utf8",t[t.Bool=6]="Bool",t[t.Decimal=7]="Decimal",t[t.Date=8]="Date",t[t.Time=9]="Time",t[t.Timestamp=10]="Timestamp",t[t.Interval=11]="Interval",t[t.List=12]="List",t[t.Struct=13]="Struct",t[t.Union=14]="Union",t[t.FixedSizeBinary=15]="FixedSizeBinary",t[t.FixedSizeList=16]="FixedSizeList",t[t.Map=17]="Map",t[t.Dictionary=-1]="Dictionary",t[t.Int8=-2]="Int8",t[t.Int16=-3]="Int16",t[t.Int32=-4]="Int32",t[t.Int64=-5]="Int64",t[t.Uint8=-6]="Uint8",t[t.Uint16=-7]="Uint16",t[t.Uint32=-8]="Uint32",t[t.Uint64=-9]="Uint64",t[t.Float16=-10]="Float16",t[t.Float32=-11]="Float32",t[t.Float64=-12]="Float64",t[t.DateDay=-13]="DateDay",t[t.DateMillisecond=-14]="DateMillisecond",t[t.TimestampSecond=-15]="TimestampSecond",t[t.TimestampMillisecond=-16]="TimestampMillisecond",t[t.TimestampMicrosecond=-17]="TimestampMicrosecond",t[t.TimestampNanosecond=-18]="TimestampNanosecond",t[t.TimeSecond=-19]="TimeSecond",t[t.TimeMillisecond=-20]="TimeMillisecond",t[t.TimeMicrosecond=-21]="TimeMicrosecond",t[t.TimeNanosecond=-22]="TimeNanosecond",t[t.DenseUnion=-23]="DenseUnion",t[t.SparseUnion=-24]="SparseUnion",t[t.IntervalDayTime=-25]="IntervalDayTime",t[t.IntervalYearMonth=-26]="IntervalYearMonth"}(r||(r={}))},function(t,e,n){"use strict";n.d(e,"c",function(){return s}),n.d(e,"w",function(){return u}),n.d(e,"o",function(){return c}),n.d(e,"s",function(){return f}),n.d(e,"p",function(){return l}),n.d(e,"q",function(){return h}),n.d(e,"r",function(){return p}),n.d(e,"E",function(){return d}),n.d(e,"B",function(){return y}),n.d(e,"C",function(){return b}),n.d(e,"D",function(){return v}),n.d(e,"k",function(){return m}),n.d(e,"l",function(){return _}),n.d(e,"m",function(){return g}),n.d(e,"n",function(){return w}),n.d(e,"a",function(){return S}),n.d(e,"G",function(){return O}),n.d(e,"b",function(){return T}),n.d(e,"g",function(){return I}),n.d(e,"f",function(){return D}),n.d(e,"d",function(){return A}),n.d(e,"e",function(){return M}),n.d(e,"y",function(){return k}),n.d(e,"z",function(){return j}),n.d(e,"A",function(){return x}),n.d(e,"t",function(){return E}),n.d(e,"u",function(){return B}),n.d(e,"x",function(){return F}),n.d(e,"F",function(){return P}),n.d(e,"i",function(){return L}),n.d(e,"j",function(){return U}),n.d(e,"v",function(){return C}),n.d(e,"h",function(){return N});var r,i,o=n(0),a=n(1),s=function(){function t(t){this._children=t}var e;return t.isNull=function(t){return t&&t.typeId===a.h.Null},t.isInt=function(t){return t&&t.typeId===a.h.Int},t.isFloat=function(t){return t&&t.typeId===a.h.Float},t.isBinary=function(t){return t&&t.typeId===a.h.Binary},t.isUtf8=function(t){return t&&t.typeId===a.h.Utf8},t.isBool=function(t){return t&&t.typeId===a.h.Bool},t.isDecimal=function(t){return t&&t.typeId===a.h.Decimal},t.isDate=function(t){return t&&t.typeId===a.h.Date},t.isTime=function(t){return t&&t.typeId===a.h.Time},t.isTimestamp=function(t){return t&&t.typeId===a.h.Timestamp},t.isInterval=function(t){return t&&t.typeId===a.h.Interval},t.isList=function(t){return t&&t.typeId===a.h.List},t.isStruct=function(t){return t&&t.typeId===a.h.Struct},t.isUnion=function(t){return t&&t.typeId===a.h.Union},t.isFixedSizeBinary=function(t){return t&&t.typeId===a.h.FixedSizeBinary},t.isFixedSizeList=function(t){return t&&t.typeId===a.h.FixedSizeList},t.isMap=function(t){return t&&t.typeId===a.h.Map},t.isDictionary=function(t){return t&&t.typeId===a.h.Dictionary},Object.defineProperty(t.prototype,"children",{get:function(){return this._children},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"typeId",{get:function(){return a.h.NONE},enumerable:!0,configurable:!0}),t[Symbol.toStringTag]=((e=t.prototype).ArrayType=Array,e[Symbol.toStringTag]="DataType"),t}(),u=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return o.g(e,t),e.prototype.toString=function(){return"Null"},Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Null},enumerable:!0,configurable:!0}),e[Symbol.toStringTag]=e.prototype[Symbol.toStringTag]="Null",e}(s),c=function(t){function e(e,n){var r=t.call(this)||this;return r._isSigned=e,r._bitWidth=n,r}return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Int},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isSigned",{get:function(){return this._isSigned},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"bitWidth",{get:function(){return this._bitWidth},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"ArrayType",{get:function(){switch(this._bitWidth){case 8:return this._isSigned?Int8Array:Uint8Array;case 16:return this._isSigned?Int16Array:Uint16Array;case 32:case 64:return this._isSigned?Int32Array:Uint32Array}throw new Error("Unrecognized "+this[Symbol.toStringTag]+" type")},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return(this._isSigned?"I":"Ui")+"nt"+this._bitWidth},e[Symbol.toStringTag]=e.prototype[Symbol.toStringTag]="Int",e}(s),f=function(t){function e(){return t.call(this,!0,8)||this}return o.g(e,t),e}(c),l=function(t){function e(){return t.call(this,!0,16)||this}return o.g(e,t),e}(c),h=function(t){function e(){return t.call(this,!0,32)||this}return o.g(e,t),e}(c),p=function(t){function e(){return t.call(this,!0,64)||this}return o.g(e,t),e}(c),d=function(t){function e(){return t.call(this,!1,8)||this}return o.g(e,t),e}(c),y=function(t){function e(){return t.call(this,!1,16)||this}return o.g(e,t),e}(c),b=function(t){function e(){return t.call(this,!1,32)||this}return o.g(e,t),e}(c),v=function(t){function e(){return t.call(this,!1,64)||this}return o.g(e,t),e}(c),m=function(t){function e(e){var n=t.call(this)||this;return n._precision=e,n}return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Float},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"precision",{get:function(){return this._precision},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"ArrayType",{get:function(){switch(this.precision){case a.f.HALF:return Uint16Array;case a.f.SINGLE:return Float32Array;case a.f.DOUBLE:return Float64Array}throw new Error("Unrecognized "+this[Symbol.toStringTag]+" type")},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"Float"+(this.precision<<5||16)},e[Symbol.toStringTag]=e.prototype[Symbol.toStringTag]="Float",e}(s),_=function(t){function e(){return t.call(this,a.f.HALF)||this}return o.g(e,t),e}(m),g=function(t){function e(){return t.call(this,a.f.SINGLE)||this}return o.g(e,t),e}(m),w=function(t){function e(){return t.call(this,a.f.DOUBLE)||this}return o.g(e,t),e}(m),S=function(t){function e(){return t.call(this)||this}var n;return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Binary},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"Binary"},e[Symbol.toStringTag]=((n=e.prototype).ArrayType=Uint8Array,n[Symbol.toStringTag]="Binary"),e}(s),O=function(t){function e(){return t.call(this)||this}var n;return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Utf8},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"Utf8"},e[Symbol.toStringTag]=((n=e.prototype).ArrayType=Uint8Array,n[Symbol.toStringTag]="Utf8"),e}(s),T=function(t){function e(){return t.call(this)||this}var n;return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Bool},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"Bool"},e[Symbol.toStringTag]=((n=e.prototype).ArrayType=Uint8Array,n[Symbol.toStringTag]="Bool"),e}(s),I=function(t){function e(e,n){var r=t.call(this)||this;return r._scale=e,r._precision=n,r}var n;return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Decimal},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"scale",{get:function(){return this._scale},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"precision",{get:function(){return this._precision},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"Decimal["+this._precision+"e"+(this._scale>0?"+":"")+this._scale+"]"},e[Symbol.toStringTag]=((n=e.prototype).ArrayType=Uint32Array,n[Symbol.toStringTag]="Decimal"),e}(s),D=function(t){function e(e){var n=t.call(this)||this;return n._unit=e,n}var n;return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Date},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"unit",{get:function(){return this._unit},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"Date"+32*(this._unit+1)+"<"+a.b[this._unit]+">"},e[Symbol.toStringTag]=((n=e.prototype).ArrayType=Int32Array,n[Symbol.toStringTag]="Date"),e}(s),A=function(t){function e(){return t.call(this,a.b.DAY)||this}return o.g(e,t),e}(D),M=function(t){function e(){return t.call(this,a.b.MILLISECOND)||this}return o.g(e,t),e}(D),k=function(t){function e(e,n){var r=t.call(this)||this;return r._unit=e,r._bitWidth=n,r}var n;return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Time},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"unit",{get:function(){return this._unit},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"bitWidth",{get:function(){return this._bitWidth},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"Time"+this._bitWidth+"<"+a.g[this._unit]+">"},e[Symbol.toStringTag]=((n=e.prototype).ArrayType=Int32Array,n[Symbol.toStringTag]="Time"),e}(s),j=(i=k,o.g(function(){return i.call(this,a.g.SECOND,32)||this},i),function(t){o.g(function(){return t.call(this,a.g.MILLISECOND,32)||this},t)}(k),function(t){o.g(function(){return t.call(this,a.g.MICROSECOND,64)||this},t)}(k),function(t){o.g(function(){return t.call(this,a.g.NANOSECOND,64)||this},t)}(k),function(t){function e(e,n){var r=t.call(this)||this;return r._unit=e,r._timezone=n,r}var n;return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Timestamp},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"unit",{get:function(){return this._unit},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"timezone",{get:function(){return this._timezone},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"Timestamp<"+a.g[this._unit]+(this._timezone?", "+this._timezone:"")+">"},e[Symbol.toStringTag]=((n=e.prototype).ArrayType=Int32Array,n[Symbol.toStringTag]="Timestamp"),e}(s)),x=function(t){function e(e){return t.call(this,a.g.SECOND,e)||this}return o.g(e,t),e}(j),E=(function(t){o.g(function(e){return t.call(this,a.g.MILLISECOND,e)||this},t)}(j),function(t){o.g(function(e){return t.call(this,a.g.MICROSECOND,e)||this},t)}(j),function(t){o.g(function(e){return t.call(this,a.g.NANOSECOND,e)||this},t)}(j),function(t){function e(e){var n=t.call(this)||this;return n._unit=e,n}var n;return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Interval},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"unit",{get:function(){return this._unit},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"Interval<"+a.c[this._unit]+">"},e[Symbol.toStringTag]=((n=e.prototype).ArrayType=Int32Array,n[Symbol.toStringTag]="Interval"),e}(s)),B=(function(t){o.g(function(){return t.call(this,a.c.DAY_TIME)||this},t)}(E),function(t){o.g(function(){return t.call(this,a.c.YEAR_MONTH)||this},t)}(E),function(t){function e(e){return t.call(this,[e])||this}return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.List},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"List<"+this.valueType+">"},Object.defineProperty(e.prototype,"children",{get:function(){return this._children},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"valueType",{get:function(){return this._children[0].type},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"valueField",{get:function(){return this._children[0]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"ArrayType",{get:function(){return this.valueType.ArrayType},enumerable:!0,configurable:!0}),e[Symbol.toStringTag]=e.prototype[Symbol.toStringTag]="List",e}(s)),F=function(t){function e(e){var n=t.call(this,e)||this;return n._children=e,n}return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Struct},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"children",{get:function(){return this._children},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"Struct<["+this._children.map(function(t){return t.type}).join(", ")+"]>"},e[Symbol.toStringTag]=e.prototype[Symbol.toStringTag]="Struct",e}(s),P=function(t){function e(e,n,r){var i=t.call(this,r)||this;return i._mode=e,i._children=r,i._typeIds=n=Int32Array.from(n),i._typeIdToChildIndex=n.reduce(function(t,e,n){return(t[e]=n)&&t||t},Object.create(null)),i}var n;return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Union},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"mode",{get:function(){return this._mode},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"typeIds",{get:function(){return this._typeIds},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"children",{get:function(){return this._children},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"typeIdToChildIndex",{get:function(){return this._typeIdToChildIndex},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return this[Symbol.toStringTag]+"<"+this._children.map(function(t){return""+t.type}).join(" | ")+">"},e[Symbol.toStringTag]=((n=e.prototype).ArrayType=Int32Array,n[Symbol.toStringTag]="Union"),e}(s),L=(function(t){o.g(function(e,n){return t.call(this,a.i.Dense,e,n)||this},t)}(P),function(t){o.g(function(e,n){return t.call(this,a.i.Sparse,e,n)||this},t)}(P),function(t){function e(e){var n=t.call(this)||this;return n._byteWidth=e,n}var n;return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.FixedSizeBinary},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"byteWidth",{get:function(){return this._byteWidth},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"FixedSizeBinary["+this.byteWidth+"]"},e[Symbol.toStringTag]=((n=e.prototype).ArrayType=Uint8Array,n[Symbol.toStringTag]="FixedSizeBinary"),e}(s)),U=function(t){function e(e,n){var r=t.call(this,[n])||this;return r._listSize=e,r}return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.FixedSizeList},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"listSize",{get:function(){return this._listSize},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"children",{get:function(){return this._children},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"valueType",{get:function(){return this.children[0].type},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"valueField",{get:function(){return this.children[0]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"ArrayType",{get:function(){return this.valueType.ArrayType},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"FixedSizeList["+this._listSize+"]<"+this.valueType+">"},e[Symbol.toStringTag]=e.prototype[Symbol.toStringTag]="FixedSizeList",e}(s),C=function(t){function e(e,n){void 0===n&&(n=!1);var r=t.call(this,e)||this;return r._children=e,r._keysSorted=n,r}return o.g(e,t),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Map},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"children",{get:function(){return this._children},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"keysSorted",{get:function(){return this._keysSorted},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"Map<{"+this._children.map(function(t){return t.name+":"+t.type}).join(", ")+"}>"},e[Symbol.toStringTag]=e.prototype[Symbol.toStringTag]="Map_",e}(s),R=(r=-1,function(){return++r}),N=function(t){function e(e,n,r,i,o){var a=t.call(this)||this;return a._indices=n,a._dictionary=e,a._isOrdered=i||!1,a._dictionaryVector=o,a._id=null==r?R():"number"==typeof r?r:r.low,a}return o.g(e,t),Object.defineProperty(e.prototype,"dictionaryVector",{get:function(){return this._dictionaryVector},set:function(t){this._dictionaryVector=t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"typeId",{get:function(){return a.h.Dictionary},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"id",{get:function(){return this._id},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"indices",{get:function(){return this._indices},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dictionary",{get:function(){return this._dictionary},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isOrdered",{get:function(){return this._isOrdered},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"children",{get:function(){return this.dictionary.children},set:function(t){},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"valueType",{get:function(){return this.dictionary},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"ArrayType",{get:function(){return this.dictionary.ArrayType},enumerable:!0,configurable:!0}),e.prototype.toString=function(){return"Dictionary<"+this.indices+", "+this.dictionary+">"},e[Symbol.toStringTag]=e.prototype[Symbol.toStringTag]="Dictionary",e}(s)},function(t,e,n){"use strict";n.d(e,"b",function(){return f}),n.d(e,"d",function(){return l}),n.d(e,"e",function(){return h}),n.d(e,"g",function(){return d}),n.d(e,"f",function(){return b}),n.d(e,"c",function(){return v}),n.d(e,"a",function(){return m});var r=n(0),i=n(12),o=n(14),a=n(7),s=i.a.ByteBuffer,u="undefined"!=typeof SharedArrayBuffer?SharedArrayBuffer:ArrayBuffer;function c(t,e,n,r){void 0===n&&(n=0),void 0===r&&(r=e.byteLength);var i=t.byteLength,o=new Uint8Array(t.buffer,t.byteOffset,i),a=new Uint8Array(e.buffer,e.byteOffset,Math.min(r,i));return o.set(a,n),t}function f(t,e){for(var n,r,i,o=function(t){for(var e,n,r,i,o=t[0]?[t[0]]:[],a=void 0,s=void 0,u=0,c=0,f=t.length;++u<f;)a=o[c],s=t[u],a&&s&&a.buffer===s.buffer?(s.byteOffset<a.byteOffset&&(a=t[u],s=o[c]),e=a.byteOffset,r=a.byteLength,n=s.byteOffset,i=s.byteLength,e+r<n||n+i<e?s&&(o[++c]=s):o[c]=new Uint8Array(a.buffer,e,n-e+i)):s&&(o[++c]=s);return o}(t),a=o.reduce(function(t,e){return t+e.byteLength},0),s=0,u=-1,f=Math.min(e||1/0,a),l=o.length;++u<l;){if(f<=s+(r=(n=o[u]).subarray(0,Math.min(n.length,f-s))).length){r.length<n.length?o[u]=n.subarray(r.length):r.length===n.length&&u++,i?c(i,r,s):i=r;break}c(i||(i=new Uint8Array(f)),r,s),s+=r.length}return[i||new Uint8Array(0),o.slice(u),a-(i?i.byteLength:0)]}function l(t,e){var n=Object(a.f)(e)?e.value:e;return n?("string"==typeof n&&(n=Object(o.b)(n)),n instanceof t?n.constructor===t?n:new t(n.buffer,n.byteOffset,n.byteLength/t.BYTES_PER_ELEMENT):n instanceof ArrayBuffer?new t(n):n instanceof u?new t(n):n instanceof s?l(t,n.bytes()):ArrayBuffer.isView(n)?n.byteLength<=0?new t(0):new t(n.buffer,n.byteOffset,n.byteLength/t.BYTES_PER_ELEMENT):t.from(n)):new t(0)}var h=function(t){return l(Uint8Array,t)},p=function(t){return t.next(),t};var d=function(t){return function(t,e){var n,i;return r.h(this,function(o){switch(o.label){case 0:return n=function(t){return r.h(this,function(e){switch(e.label){case 0:return[4,t];case 1:return e.sent(),[2]}})},i="string"==typeof e?n(e):ArrayBuffer.isView(e)?n(e):e instanceof ArrayBuffer?n(e):e instanceof u?n(e):Object(a.e)(e)?e:n(e),[5,r.k(p(function(e){var n,i,o;return r.h(this,function(r){switch(r.label){case 0:n=null,r.label=1;case 1:return o=(i=e).next,[4,l(t,n)];case 2:n=o.apply(i,[r.sent()]),r.label=3;case 3:if(!n.done)return[3,1];r.label=4;case 4:return[2]}})}(i[Symbol.iterator]())))];case 1:return o.sent(),[2]}})}(Uint8Array,t)};function y(t,e){return r.c(this,arguments,function(){var n,i,o,s,c;return r.h(this,function(f){switch(f.label){case 0:return Object(a.g)(e)?(n=y,i=[t],[4,r.e(e)]):[3,5];case 1:return[5,r.k(r.b.apply(void 0,[r.d.apply(void 0,[n.apply(void 0,i.concat([f.sent()]))])]))];case 2:case 3:return[4,r.e.apply(void 0,[f.sent()])];case 4:return[2,f.sent()];case 5:return o=function(t){return r.c(this,arguments,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,r.e(t)];case 1:return[4,r.e.apply(void 0,[e.sent()])];case 2:return[4,e.sent()];case 3:return e.sent(),[2]}})})},s=function(t){return r.c(this,arguments,function(){return r.h(this,function(e){switch(e.label){case 0:return[5,r.k(r.b(r.d(p(function(t){var e,n,i;return r.h(this,function(r){switch(r.label){case 0:e=null,r.label=1;case 1:return i=(n=t).next,[4,e&&e.value];case 2:e=i.apply(n,[r.sent()]),r.label=3;case 3:if(!e.done)return[3,1];r.label=4;case 4:return[2]}})}(t[Symbol.iterator]())))))];case 1:return[4,r.e.apply(void 0,[e.sent()])];case 2:return e.sent(),[2]}})})},c="string"==typeof e?o(e):ArrayBuffer.isView(e)?o(e):e instanceof ArrayBuffer?o(e):e instanceof u?o(e):Object(a.e)(e)?s(e):Object(a.b)(e)?e:o(e),[5,r.k(r.b(r.d(p(function(e){return r.c(this,arguments,function(){var n,i,o;return r.h(this,function(a){switch(a.label){case 0:n=null,a.label=1;case 1:return o=(i=e).next,[4,r.e(l(t,n))];case 2:return[4,a.sent()];case 3:return[4,r.e.apply(void 0,[o.apply(i,[a.sent()])])];case 4:n=a.sent(),a.label=5;case 5:if(!n.done)return[3,1];a.label=6;case 6:return[2]}})})}(c[Symbol.asyncIterator]())))))];case 6:return[4,r.e.apply(void 0,[f.sent()])];case 7:return f.sent(),[2]}})})}var b=function(t){return y(Uint8Array,t)};function v(t,e,n){if(0!==t){n=n.slice(0,e+1);for(var r=-1;++r<=e;)n[r]+=t}return n}function m(t,e){var n=0,r=t.length;if(r!==e.length)return!1;if(r>0)do{if(t[n]!==e[n])return!1}while(++n<r);return!0}},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r=function(){}},function(t,e,n){"use strict";var r;n.d(e,"a",function(){return r}),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),(i=r.MetadataVersion||(r.MetadataVersion={}))[i.V1=0]="V1",i[i.V2=1]="V2",i[i.V3=2]="V3",i[i.V4=3]="V4"}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),(i=r.UnionMode||(r.UnionMode={}))[i.Sparse=0]="Sparse",i[i.Dense=1]="Dense"}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),(i=r.Precision||(r.Precision={}))[i.HALF=0]="HALF",i[i.SINGLE=1]="SINGLE",i[i.DOUBLE=2]="DOUBLE"}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),(i=r.DateUnit||(r.DateUnit={}))[i.DAY=0]="DAY",i[i.MILLISECOND=1]="MILLISECOND"}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),(i=r.TimeUnit||(r.TimeUnit={}))[i.SECOND=0]="SECOND",i[i.MILLISECOND=1]="MILLISECOND",i[i.MICROSECOND=2]="MICROSECOND",i[i.NANOSECOND=3]="NANOSECOND"}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),(i=r.IntervalUnit||(r.IntervalUnit={}))[i.YEAR_MONTH=0]="YEAR_MONTH",i[i.DAY_TIME=1]="DAY_TIME"}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),(i=r.Type||(r.Type={}))[i.NONE=0]="NONE",i[i.Null=1]="Null",i[i.Int=2]="Int",i[i.FloatingPoint=3]="FloatingPoint",i[i.Binary=4]="Binary",i[i.Utf8=5]="Utf8",i[i.Bool=6]="Bool",i[i.Decimal=7]="Decimal",i[i.Date=8]="Date",i[i.Time=9]="Time",i[i.Timestamp=10]="Timestamp",i[i.Interval=11]="Interval",i[i.List=12]="List",i[i.Struct_=13]="Struct_",i[i.Union=14]="Union",i[i.FixedSizeBinary=15]="FixedSizeBinary",i[i.FixedSizeList=16]="FixedSizeList",i[i.Map=17]="Map"}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),(i=r.VectorType||(r.VectorType={}))[i.OFFSET=0]="OFFSET",i[i.DATA=1]="DATA",i[i.VALIDITY=2]="VALIDITY",i[i.TYPE=3]="TYPE"}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),(i=r.Endianness||(r.Endianness={}))[i.Little=0]="Little",i[i.Big=1]="Big"}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsNull=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.startNull=function(t){t.startObject(0)},t.endNull=function(t){return t.endObject()},t}(),r.Null=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsStruct_=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.startStruct_=function(t){t.startObject(0)},t.endStruct_=function(t){return t.endObject()},t}(),r.Struct_=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsList=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.startList=function(t){t.startObject(0)},t.endList=function(t){return t.endObject()},t}(),r.List=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsFixedSizeList=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.prototype.listSize=function(){var t=this.bb.__offset(this.bb_pos,4);return t?this.bb.readInt32(this.bb_pos+t):0},t.startFixedSizeList=function(t){t.startObject(1)},t.addListSize=function(t,e){t.addFieldInt32(0,e,0)},t.endFixedSizeList=function(t){return t.endObject()},t}(),r.FixedSizeList=i}(r||(r={})),function(t){var e,n;e=t.apache||(t.apache={}),function(t){var e=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsMap=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.prototype.keysSorted=function(){var t=this.bb.__offset(this.bb_pos,4);return!!t&&!!this.bb.readInt8(this.bb_pos+t)},t.startMap=function(t){t.startObject(1)},t.addKeysSorted=function(t,e){t.addFieldInt8(0,+e,0)},t.endMap=function(t){return t.endObject()},t}();t.Map=e}((n=e.arrow||(e.arrow={})).flatbuf||(n.flatbuf={}))}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsUnion=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.mode=function(){var e=this.bb.__offset(this.bb_pos,4);return e?this.bb.readInt16(this.bb_pos+e):t.apache.arrow.flatbuf.UnionMode.Sparse},e.prototype.typeIds=function(t){var e=this.bb.__offset(this.bb_pos,6);return e?this.bb.readInt32(this.bb.__vector(this.bb_pos+e)+4*t):0},e.prototype.typeIdsLength=function(){var t=this.bb.__offset(this.bb_pos,6);return t?this.bb.__vector_len(this.bb_pos+t):0},e.prototype.typeIdsArray=function(){var t=this.bb.__offset(this.bb_pos,6);return t?new Int32Array(this.bb.bytes().buffer,this.bb.bytes().byteOffset+this.bb.__vector(this.bb_pos+t),this.bb.__vector_len(this.bb_pos+t)):null},e.startUnion=function(t){t.startObject(2)},e.addMode=function(e,n){e.addFieldInt16(0,n,t.apache.arrow.flatbuf.UnionMode.Sparse)},e.addTypeIds=function(t,e){t.addFieldOffset(1,e,0)},e.createTypeIdsVector=function(t,e){t.startVector(4,e.length,4);for(var n=e.length-1;n>=0;n--)t.addInt32(e[n]);return t.endVector()},e.startTypeIdsVector=function(t,e){t.startVector(4,e,4)},e.endUnion=function(t){return t.endObject()},e}(),r.Union=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsInt=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.prototype.bitWidth=function(){var t=this.bb.__offset(this.bb_pos,4);return t?this.bb.readInt32(this.bb_pos+t):0},t.prototype.isSigned=function(){var t=this.bb.__offset(this.bb_pos,6);return!!t&&!!this.bb.readInt8(this.bb_pos+t)},t.startInt=function(t){t.startObject(2)},t.addBitWidth=function(t,e){t.addFieldInt32(0,e,0)},t.addIsSigned=function(t,e){t.addFieldInt8(1,+e,0)},t.endInt=function(t){return t.endObject()},t}(),r.Int=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsFloatingPoint=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.precision=function(){var e=this.bb.__offset(this.bb_pos,4);return e?this.bb.readInt16(this.bb_pos+e):t.apache.arrow.flatbuf.Precision.HALF},e.startFloatingPoint=function(t){t.startObject(1)},e.addPrecision=function(e,n){e.addFieldInt16(0,n,t.apache.arrow.flatbuf.Precision.HALF)},e.endFloatingPoint=function(t){return t.endObject()},e}(),r.FloatingPoint=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsUtf8=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.startUtf8=function(t){t.startObject(0)},t.endUtf8=function(t){return t.endObject()},t}(),r.Utf8=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsBinary=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.startBinary=function(t){t.startObject(0)},t.endBinary=function(t){return t.endObject()},t}(),r.Binary=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsFixedSizeBinary=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.prototype.byteWidth=function(){var t=this.bb.__offset(this.bb_pos,4);return t?this.bb.readInt32(this.bb_pos+t):0},t.startFixedSizeBinary=function(t){t.startObject(1)},t.addByteWidth=function(t,e){t.addFieldInt32(0,e,0)},t.endFixedSizeBinary=function(t){return t.endObject()},t}(),r.FixedSizeBinary=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsBool=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.startBool=function(t){t.startObject(0)},t.endBool=function(t){return t.endObject()},t}(),r.Bool=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsDecimal=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.prototype.precision=function(){var t=this.bb.__offset(this.bb_pos,4);return t?this.bb.readInt32(this.bb_pos+t):0},t.prototype.scale=function(){var t=this.bb.__offset(this.bb_pos,6);return t?this.bb.readInt32(this.bb_pos+t):0},t.startDecimal=function(t){t.startObject(2)},t.addPrecision=function(t,e){t.addFieldInt32(0,e,0)},t.addScale=function(t,e){t.addFieldInt32(1,e,0)},t.endDecimal=function(t){return t.endObject()},t}(),r.Decimal=i}(r||(r={})),function(t){var e,n;e=t.apache||(t.apache={}),function(e){var n=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsDate=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.unit=function(){var e=this.bb.__offset(this.bb_pos,4);return e?this.bb.readInt16(this.bb_pos+e):t.apache.arrow.flatbuf.DateUnit.MILLISECOND},e.startDate=function(t){t.startObject(1)},e.addUnit=function(e,n){e.addFieldInt16(0,n,t.apache.arrow.flatbuf.DateUnit.MILLISECOND)},e.endDate=function(t){return t.endObject()},e}();e.Date=n}((n=e.arrow||(e.arrow={})).flatbuf||(n.flatbuf={}))}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsTime=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.unit=function(){var e=this.bb.__offset(this.bb_pos,4);return e?this.bb.readInt16(this.bb_pos+e):t.apache.arrow.flatbuf.TimeUnit.MILLISECOND},e.prototype.bitWidth=function(){var t=this.bb.__offset(this.bb_pos,6);return t?this.bb.readInt32(this.bb_pos+t):32},e.startTime=function(t){t.startObject(2)},e.addUnit=function(e,n){e.addFieldInt16(0,n,t.apache.arrow.flatbuf.TimeUnit.MILLISECOND)},e.addBitWidth=function(t,e){t.addFieldInt32(1,e,32)},e.endTime=function(t){return t.endObject()},e}(),r.Time=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsTimestamp=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.unit=function(){var e=this.bb.__offset(this.bb_pos,4);return e?this.bb.readInt16(this.bb_pos+e):t.apache.arrow.flatbuf.TimeUnit.SECOND},e.prototype.timezone=function(t){var e=this.bb.__offset(this.bb_pos,6);return e?this.bb.__string(this.bb_pos+e,t):null},e.startTimestamp=function(t){t.startObject(2)},e.addUnit=function(e,n){e.addFieldInt16(0,n,t.apache.arrow.flatbuf.TimeUnit.SECOND)},e.addTimezone=function(t,e){t.addFieldOffset(1,e,0)},e.endTimestamp=function(t){return t.endObject()},e}(),r.Timestamp=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsInterval=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.unit=function(){var e=this.bb.__offset(this.bb_pos,4);return e?this.bb.readInt16(this.bb_pos+e):t.apache.arrow.flatbuf.IntervalUnit.YEAR_MONTH},e.startInterval=function(t){t.startObject(1)},e.addUnit=function(e,n){e.addFieldInt16(0,n,t.apache.arrow.flatbuf.IntervalUnit.YEAR_MONTH)},e.endInterval=function(t){return t.endObject()},e}(),r.Interval=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsVectorLayout=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.bitWidth=function(){var t=this.bb.__offset(this.bb_pos,4);return t?this.bb.readInt16(this.bb_pos+t):0},e.prototype.type=function(){var e=this.bb.__offset(this.bb_pos,6);return e?this.bb.readInt16(this.bb_pos+e):t.apache.arrow.flatbuf.VectorType.OFFSET},e.startVectorLayout=function(t){t.startObject(2)},e.addBitWidth=function(t,e){t.addFieldInt16(0,e,0)},e.addType=function(e,n){e.addFieldInt16(1,n,t.apache.arrow.flatbuf.VectorType.OFFSET)},e.endVectorLayout=function(t){return t.endObject()},e}(),r.VectorLayout=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsKeyValue=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.prototype.key=function(t){var e=this.bb.__offset(this.bb_pos,4);return e?this.bb.__string(this.bb_pos+e,t):null},t.prototype.value=function(t){var e=this.bb.__offset(this.bb_pos,6);return e?this.bb.__string(this.bb_pos+e,t):null},t.startKeyValue=function(t){t.startObject(2)},t.addKey=function(t,e){t.addFieldOffset(0,e,0)},t.addValue=function(t,e){t.addFieldOffset(1,e,0)},t.endKeyValue=function(t){return t.endObject()},t}(),r.KeyValue=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsDictionaryEncoding=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.id=function(){var t=this.bb.__offset(this.bb_pos,4);return t?this.bb.readInt64(this.bb_pos+t):this.bb.createLong(0,0)},e.prototype.indexType=function(e){var n=this.bb.__offset(this.bb_pos,6);return n?(e||new t.apache.arrow.flatbuf.Int).__init(this.bb.__indirect(this.bb_pos+n),this.bb):null},e.prototype.isOrdered=function(){var t=this.bb.__offset(this.bb_pos,8);return!!t&&!!this.bb.readInt8(this.bb_pos+t)},e.startDictionaryEncoding=function(t){t.startObject(3)},e.addId=function(t,e){t.addFieldInt64(0,e,t.createLong(0,0))},e.addIndexType=function(t,e){t.addFieldOffset(1,e,0)},e.addIsOrdered=function(t,e){t.addFieldInt8(2,+e,0)},e.endDictionaryEncoding=function(t){return t.endObject()},e}(),r.DictionaryEncoding=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsField=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.name=function(t){var e=this.bb.__offset(this.bb_pos,4);return e?this.bb.__string(this.bb_pos+e,t):null},e.prototype.nullable=function(){var t=this.bb.__offset(this.bb_pos,6);return!!t&&!!this.bb.readInt8(this.bb_pos+t)},e.prototype.typeType=function(){var e=this.bb.__offset(this.bb_pos,8);return e?this.bb.readUint8(this.bb_pos+e):t.apache.arrow.flatbuf.Type.NONE},e.prototype.type=function(t){var e=this.bb.__offset(this.bb_pos,10);return e?this.bb.__union(t,this.bb_pos+e):null},e.prototype.dictionary=function(e){var n=this.bb.__offset(this.bb_pos,12);return n?(e||new t.apache.arrow.flatbuf.DictionaryEncoding).__init(this.bb.__indirect(this.bb_pos+n),this.bb):null},e.prototype.children=function(e,n){var r=this.bb.__offset(this.bb_pos,14);return r?(n||new t.apache.arrow.flatbuf.Field).__init(this.bb.__indirect(this.bb.__vector(this.bb_pos+r)+4*e),this.bb):null},e.prototype.childrenLength=function(){var t=this.bb.__offset(this.bb_pos,14);return t?this.bb.__vector_len(this.bb_pos+t):0},e.prototype.layout=function(e,n){var r=this.bb.__offset(this.bb_pos,16);return r?(n||new t.apache.arrow.flatbuf.VectorLayout).__init(this.bb.__indirect(this.bb.__vector(this.bb_pos+r)+4*e),this.bb):null},e.prototype.layoutLength=function(){var t=this.bb.__offset(this.bb_pos,16);return t?this.bb.__vector_len(this.bb_pos+t):0},e.prototype.customMetadata=function(e,n){var r=this.bb.__offset(this.bb_pos,18);return r?(n||new t.apache.arrow.flatbuf.KeyValue).__init(this.bb.__indirect(this.bb.__vector(this.bb_pos+r)+4*e),this.bb):null},e.prototype.customMetadataLength=function(){var t=this.bb.__offset(this.bb_pos,18);return t?this.bb.__vector_len(this.bb_pos+t):0},e.startField=function(t){t.startObject(8)},e.addName=function(t,e){t.addFieldOffset(0,e,0)},e.addNullable=function(t,e){t.addFieldInt8(1,+e,0)},e.addTypeType=function(e,n){e.addFieldInt8(2,n,t.apache.arrow.flatbuf.Type.NONE)},e.addType=function(t,e){t.addFieldOffset(3,e,0)},e.addDictionary=function(t,e){t.addFieldOffset(4,e,0)},e.addChildren=function(t,e){t.addFieldOffset(5,e,0)},e.createChildrenVector=function(t,e){t.startVector(4,e.length,4);for(var n=e.length-1;n>=0;n--)t.addOffset(e[n]);return t.endVector()},e.startChildrenVector=function(t,e){t.startVector(4,e,4)},e.addLayout=function(t,e){t.addFieldOffset(6,e,0)},e.createLayoutVector=function(t,e){t.startVector(4,e.length,4);for(var n=e.length-1;n>=0;n--)t.addOffset(e[n]);return t.endVector()},e.startLayoutVector=function(t,e){t.startVector(4,e,4)},e.addCustomMetadata=function(t,e){t.addFieldOffset(7,e,0)},e.createCustomMetadataVector=function(t,e){t.startVector(4,e.length,4);for(var n=e.length-1;n>=0;n--)t.addOffset(e[n]);return t.endVector()},e.startCustomMetadataVector=function(t,e){t.startVector(4,e,4)},e.endField=function(t){return t.endObject()},e}(),r.Field=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.prototype.offset=function(){return this.bb.readInt64(this.bb_pos)},t.prototype.length=function(){return this.bb.readInt64(this.bb_pos+8)},t.createBuffer=function(t,e,n){return t.prep(8,16),t.writeInt64(n),t.writeInt64(e),t.offset()},t}(),r.Buffer=i}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsSchema=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.endianness=function(){var e=this.bb.__offset(this.bb_pos,4);return e?this.bb.readInt16(this.bb_pos+e):t.apache.arrow.flatbuf.Endianness.Little},e.prototype.fields=function(e,n){var r=this.bb.__offset(this.bb_pos,6);return r?(n||new t.apache.arrow.flatbuf.Field).__init(this.bb.__indirect(this.bb.__vector(this.bb_pos+r)+4*e),this.bb):null},e.prototype.fieldsLength=function(){var t=this.bb.__offset(this.bb_pos,6);return t?this.bb.__vector_len(this.bb_pos+t):0},e.prototype.customMetadata=function(e,n){var r=this.bb.__offset(this.bb_pos,8);return r?(n||new t.apache.arrow.flatbuf.KeyValue).__init(this.bb.__indirect(this.bb.__vector(this.bb_pos+r)+4*e),this.bb):null},e.prototype.customMetadataLength=function(){var t=this.bb.__offset(this.bb_pos,8);return t?this.bb.__vector_len(this.bb_pos+t):0},e.startSchema=function(t){t.startObject(3)},e.addEndianness=function(e,n){e.addFieldInt16(0,n,t.apache.arrow.flatbuf.Endianness.Little)},e.addFields=function(t,e){t.addFieldOffset(1,e,0)},e.createFieldsVector=function(t,e){t.startVector(4,e.length,4);for(var n=e.length-1;n>=0;n--)t.addOffset(e[n]);return t.endVector()},e.startFieldsVector=function(t,e){t.startVector(4,e,4)},e.addCustomMetadata=function(t,e){t.addFieldOffset(2,e,0)},e.createCustomMetadataVector=function(t,e){t.startVector(4,e.length,4);for(var n=e.length-1;n>=0;n--)t.addOffset(e[n]);return t.endVector()},e.startCustomMetadataVector=function(t,e){t.startVector(4,e,4)},e.endSchema=function(t){return t.endObject()},e.finishSchemaBuffer=function(t,e){t.finish(e)},e}(),r.Schema=i}(r||(r={}))},function(t,e,n){"use strict";n.d(e,"a",function(){return a});var r=n(10),i=n(3),o=n(1),a=function(){function t(e,n,r,i,a,s){var u;if(this.type=e,this.offset=Math.floor(Math.max(n||0,0)),this.length=Math.floor(Math.max(r||0,0)),this._nullCount=Math.floor(Math.max(i||0,-1)),this.childData=(s||[]).map(function(e){return e instanceof t?e:e.data}),a instanceof t)this.stride=a.stride,this.values=a.values,this.typeIds=a.typeIds,this.nullBitmap=a.nullBitmap,this.valueOffsets=a.valueOffsets;else{a&&((u=a[0])&&(this.valueOffsets=u),(u=a[1])&&(this.values=u),(u=a[2])&&(this.nullBitmap=u),(u=a[3])&&(this.typeIds=u));var c=e;switch(e.typeId){case o.h.Decimal:this.stride=4;break;case o.h.Timestamp:this.stride=2;break;case o.h.Date:case o.h.Interval:this.stride=1+c.unit;break;case o.h.Int:case o.h.Time:this.stride=+(c.bitWidth>32)+1;break;case o.h.FixedSizeList:this.stride=c.listSize;break;case o.h.FixedSizeBinary:this.stride=c.byteWidth;break;default:this.stride=1}}}return Object.defineProperty(t.prototype,"ArrayType",{get:function(){return this.type.ArrayType},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"typeId",{get:function(){return this.type.typeId},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"buffers",{get:function(){return[this.valueOffsets,this.values,this.nullBitmap,this.typeIds]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"nullCount",{get:function(){var t,e=this._nullCount;return e<=-1&&(t=this.nullBitmap)&&(this._nullCount=e=this.length-Object(r.e)(t,this.offset,this.offset+this.length)),e},enumerable:!0,configurable:!0}),t.prototype.clone=function(e,n,r,i,o,a){return void 0===n&&(n=this.offset),void 0===r&&(r=this.length),void 0===i&&(i=this._nullCount),void 0===o&&(o=this),void 0===a&&(a=this.childData),new t(e,n,r,i,o,a)},t.prototype.slice=function(t,e){var n=this.stride,r=this.typeId,i=this.childData,o=+(0===this._nullCount)-1,a=16===r?n:1,s=this._sliceBuffers(t,e,n,r);return this.clone(this.type,this.offset+t,e,o,s,!i.length||this.valueOffsets?i:this._sliceChildren(i,a*t,a*e))},t.prototype._sliceBuffers=function(t,e,n,r){var i,a=this.buffers;return(i=a[o.j.TYPE])&&(a[o.j.TYPE]=i.subarray(t,t+e)),(i=a[o.j.OFFSET])&&(a[o.j.OFFSET]=i.subarray(t,t+e+1))||(i=a[o.j.DATA])&&(a[o.j.DATA]=6===r?i:i.subarray(n*t,n*(t+e))),a},t.prototype._sliceChildren=function(t,e,n){return t.map(function(t){return t.slice(e,n)})},t.Null=function(e,n,r,o,a){return new t(e,n,r,o,[void 0,void 0,Object(i.d)(Uint8Array,a)])},t.Int=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,Object(i.d)(e.ArrayType,s),Object(i.d)(Uint8Array,a)])},t.Dictionary=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,Object(i.d)(e.indices.ArrayType,s),Object(i.d)(Uint8Array,a)])},t.Float=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,Object(i.d)(e.ArrayType,s),Object(i.d)(Uint8Array,a)])},t.Bool=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,Object(i.d)(e.ArrayType,s),Object(i.d)(Uint8Array,a)])},t.Decimal=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,Object(i.d)(e.ArrayType,s),Object(i.d)(Uint8Array,a)])},t.Date=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,Object(i.d)(e.ArrayType,s),Object(i.d)(Uint8Array,a)])},t.Time=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,Object(i.d)(e.ArrayType,s),Object(i.d)(Uint8Array,a)])},t.Timestamp=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,Object(i.d)(e.ArrayType,s),Object(i.d)(Uint8Array,a)])},t.Interval=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,Object(i.d)(e.ArrayType,s),Object(i.d)(Uint8Array,a)])},t.FixedSizeBinary=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,Object(i.d)(e.ArrayType,s),Object(i.d)(Uint8Array,a)])},t.Binary=function(e,n,r,o,a,s,u){return new t(e,n,r,o,[Object(i.d)(Int32Array,s),Object(i.d)(Uint8Array,u),Object(i.d)(Uint8Array,a)])},t.Utf8=function(e,n,r,o,a,s,u){return new t(e,n,r,o,[Object(i.d)(Int32Array,s),Object(i.d)(Uint8Array,u),Object(i.d)(Uint8Array,a)])},t.List=function(e,n,r,o,a,s,u){return new t(e,n,r,o,[Object(i.d)(Int32Array,s),void 0,Object(i.d)(Uint8Array,a)],[u])},t.FixedSizeList=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,void 0,Object(i.d)(Uint8Array,a)],[s])},t.Struct=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,void 0,Object(i.d)(Uint8Array,a)],s)},t.Map=function(e,n,r,o,a,s){return new t(e,n,r,o,[void 0,void 0,Object(i.d)(Uint8Array,a)],s)},t.Union=function(e,n,r,a,s,u,c,f){var l=[void 0,void 0,Object(i.d)(Uint8Array,s),Object(i.d)(e.ArrayType,u)];return e.mode===o.i.Sparse?new t(e,n,r,a,l,c):(l[o.j.OFFSET]=Object(i.d)(Int32Array,c),new t(e,n,r,a,l,f))},t}();a.prototype.childData=Object.freeze([])},function(t,e,n){"use strict";n.d(e,"g",function(){return s}),n.d(e,"e",function(){return u}),n.d(e,"b",function(){return c}),n.d(e,"a",function(){return f}),n.d(e,"f",function(){return l}),n.d(e,"d",function(){return h}),n.d(e,"c",function(){return p}),n.d(e,"j",function(){return d}),n.d(e,"h",function(){return y}),n.d(e,"k",function(){return b}),n.d(e,"i",function(){return v});var r=n(8),i=function(t){return"boolean"==typeof t},o=function(t){return"function"==typeof t},a=function(t){return null!=t&&Object(t)===t},s=function(t){return a(t)&&o(t.then)},u=function(t){return a(t)&&o(t[Symbol.iterator])},c=function(t){return a(t)&&o(t[Symbol.asyncIterator])},f=function(t){return a(t)&&a(t.schema)},l=function(t){return a(t)&&"done"in t&&"value"in t},h=function(t){return a(t)&&o(t.stat)&&function(t){return"number"==typeof t}(t.fd)},p=function(t){return a(t)&&y(t.body)},d=function(t){return a(t)&&o(t.abort)&&o(t.getWriter)&&!(t instanceof r.d)},y=function(t){return a(t)&&o(t.tee)&&o(t.cancel)&&o(t.pipeTo)&&o(t.getReader)&&!(t instanceof r.d)},b=function(t){return a(t)&&o(t.end)&&o(t.write)&&i(t.writable)&&!(t instanceof r.d)},v=function(t){return a(t)&&o(t.read)&&o(t.pipe)&&i(t.readable)&&!(t instanceof r.d)}},function(t,e,n){"use strict";n.d(e,"c",function(){return o}),n.d(e,"a",function(){return a}),n.d(e,"d",function(){return s}),n.d(e,"b",function(){return u});var r=n(0),i=n(11),o=Object.freeze({done:!0,value:void 0}),a=function(){function t(t){this._json=t}return Object.defineProperty(t.prototype,"schema",{get:function(){return this._json.schema},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"batches",{get:function(){return this._json.batches||[]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dictionaries",{get:function(){return this._json.dictionaries||[]},enumerable:!0,configurable:!0}),t}(),s=function(){function t(){}return t.prototype.tee=function(){return this._getDOMStream().tee()},t.prototype.pipe=function(t,e){return this._getNodeStream().pipe(t,e)},t.prototype.pipeTo=function(t,e){return this._getDOMStream().pipeTo(t,e)},t.prototype.pipeThrough=function(t,e){return this._getDOMStream().pipeThrough(t,e)},t.prototype._getDOMStream=function(){return this._DOMStream||(this._DOMStream=this.toDOMStream())},t.prototype._getNodeStream=function(){return this._nodeStream||(this._nodeStream=this.toNodeStream())},t}(),u=function(t){function e(){var e=t.call(this)||this;return e._values=[],e.resolvers=[],e._closedPromise=new Promise(function(t){return e._closedPromiseResolve=t}),e}return r.g(e,t),Object.defineProperty(e.prototype,"closed",{get:function(){return this._closedPromise},enumerable:!0,configurable:!0}),e.prototype.cancel=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,this.return(t)];case 1:return e.sent(),[2]}})})},e.prototype.write=function(t){this._ensureOpen()&&(this.resolvers.length<=0?this._values.push(t):this.resolvers.shift().resolve({done:!1,value:t}))},e.prototype.abort=function(t){this._closedPromiseResolve&&(this.resolvers.length<=0?this._error={error:t}:this.resolvers.shift().reject({done:!0,value:t}))},e.prototype.close=function(){if(this._closedPromiseResolve){for(var t=this.resolvers;t.length>0;)t.shift().resolve(o);this._closedPromiseResolve(),this._closedPromiseResolve=void 0}},e.prototype[Symbol.asyncIterator]=function(){return this},e.prototype.toDOMStream=function(t){return i.a.toDOMStream(this._closedPromiseResolve||this._error?this:this._values,t)},e.prototype.toNodeStream=function(t){return i.a.toNodeStream(this._closedPromiseResolve||this._error?this:this._values,t)},e.prototype.throw=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,this.abort(t)];case 1:return e.sent(),[2,o]}})})},e.prototype.return=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(t){switch(t.label){case 0:return[4,this.close()];case 1:return t.sent(),[2,o]}})})},e.prototype.read=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,this.next(t,"read")];case 1:return[2,e.sent().value]}})})},e.prototype.peek=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,this.next(t,"peek")];case 1:return[2,e.sent().value]}})})},e.prototype.next=function(){for(var t=this,e=[],n=0;n<arguments.length;n++)e[n]=arguments[n];return this._values.length>0?Promise.resolve({done:!1,value:this._values.shift()}):this._error?Promise.reject({done:!0,value:this._error.error}):this._closedPromiseResolve?new Promise(function(e,n){t.resolvers.push({resolve:e,reject:n})}):Promise.resolve(o)},e.prototype._ensureOpen=function(){if(this._closedPromiseResolve)return!0;throw new Error(this+" is closed")},e}(s)},function(t,e,n){"use strict";n.d(e,"a",function(){return u});var r=n(0),i=n(6),o=n(4),a=n(1),s=n(2),u=function(){function t(){}return t.prototype.visitMany=function(t){for(var e=this,n=[],i=1;i<arguments.length;i++)n[i-1]=arguments[i];return t.map(function(t,i){return e.visit.apply(e,r.j([t],n.map(function(t){return t[i]})))})},t.prototype.visit=function(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return this.getVisitFn(t[0],!1).apply(this,t)},t.prototype.getVisitFn=function(t,e){return void 0===e&&(e=!0),function(t,e,n){void 0===n&&(n=!0);var r=null,u=a.h.NONE;switch(e instanceof i.a?u=c(e.type):e instanceof o.a?u=c(e.type):e instanceof s.c?u=c(e):"number"!=typeof(u=e)&&(u=a.h[e]),u){case a.h.Null:r=t.visitNull;break;case a.h.Bool:r=t.visitBool;break;case a.h.Int:r=t.visitInt;break;case a.h.Int8:r=t.visitInt8||t.visitInt;break;case a.h.Int16:r=t.visitInt16||t.visitInt;break;case a.h.Int32:r=t.visitInt32||t.visitInt;break;case a.h.Int64:r=t.visitInt64||t.visitInt;break;case a.h.Uint8:r=t.visitUint8||t.visitInt;break;case a.h.Uint16:r=t.visitUint16||t.visitInt;break;case a.h.Uint32:r=t.visitUint32||t.visitInt;break;case a.h.Uint64:r=t.visitUint64||t.visitInt;break;case a.h.Float:r=t.visitFloat;break;case a.h.Float16:r=t.visitFloat16||t.visitFloat;break;case a.h.Float32:r=t.visitFloat32||t.visitFloat;break;case a.h.Float64:r=t.visitFloat64||t.visitFloat;break;case a.h.Utf8:r=t.visitUtf8;break;case a.h.Binary:r=t.visitBinary;break;case a.h.FixedSizeBinary:r=t.visitFixedSizeBinary;break;case a.h.Date:r=t.visitDate;break;case a.h.DateDay:r=t.visitDateDay||t.visitDate;break;case a.h.DateMillisecond:r=t.visitDateMillisecond||t.visitDate;break;case a.h.Timestamp:r=t.visitTimestamp;break;case a.h.TimestampSecond:r=t.visitTimestampSecond||t.visitTimestamp;break;case a.h.TimestampMillisecond:r=t.visitTimestampMillisecond||t.visitTimestamp;break;case a.h.TimestampMicrosecond:r=t.visitTimestampMicrosecond||t.visitTimestamp;break;case a.h.TimestampNanosecond:r=t.visitTimestampNanosecond||t.visitTimestamp;break;case a.h.Time:r=t.visitTime;break;case a.h.TimeSecond:r=t.visitTimeSecond||t.visitTime;break;case a.h.TimeMillisecond:r=t.visitTimeMillisecond||t.visitTime;break;case a.h.TimeMicrosecond:r=t.visitTimeMicrosecond||t.visitTime;break;case a.h.TimeNanosecond:r=t.visitTimeNanosecond||t.visitTime;break;case a.h.Decimal:r=t.visitDecimal;break;case a.h.List:r=t.visitList;break;case a.h.Struct:r=t.visitStruct;break;case a.h.Union:r=t.visitUnion;break;case a.h.DenseUnion:r=t.visitDenseUnion||t.visitUnion;break;case a.h.SparseUnion:r=t.visitSparseUnion||t.visitUnion;break;case a.h.Dictionary:r=t.visitDictionary;break;case a.h.Interval:r=t.visitInterval;break;case a.h.IntervalDayTime:r=t.visitIntervalDayTime||t.visitInterval;break;case a.h.IntervalYearMonth:r=t.visitIntervalYearMonth||t.visitInterval;break;case a.h.FixedSizeList:r=t.visitFixedSizeList;break;case a.h.Map:r=t.visitMap}if("function"==typeof r)return r;if(!n)return function(){return null};throw new Error("Unrecognized type \'"+a.h[u]+"\'")}(this,t,e)},t.prototype.visitNull=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitBool=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitInt=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitFloat=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitUtf8=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitBinary=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitFixedSizeBinary=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitDate=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitTimestamp=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitTime=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitDecimal=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitList=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitStruct=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitUnion=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitDictionary=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitInterval=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitFixedSizeList=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t.prototype.visitMap=function(t){for(var e=[],n=1;n<arguments.length;n++)e[n-1]=arguments[n];return null},t}();function c(t){switch(t.typeId){case a.h.Null:return a.h.Null;case a.h.Int:var e=t,n=e.bitWidth,r=e.isSigned;switch(n){case 8:return r?a.h.Int8:a.h.Uint8;case 16:return r?a.h.Int16:a.h.Uint16;case 32:return r?a.h.Int32:a.h.Uint32;case 64:return r?a.h.Int64:a.h.Uint64}return a.h.Int;case a.h.Float:switch(t.precision){case a.f.HALF:return a.h.Float16;case a.f.SINGLE:return a.h.Float32;case a.f.DOUBLE:return a.h.Float64}return a.h.Float;case a.h.Binary:return a.h.Binary;case a.h.Utf8:return a.h.Utf8;case a.h.Bool:return a.h.Bool;case a.h.Decimal:return a.h.Decimal;case a.h.Time:switch(t.unit){case a.g.SECOND:return a.h.TimeSecond;case a.g.MILLISECOND:return a.h.TimeMillisecond;case a.g.MICROSECOND:return a.h.TimeMicrosecond;case a.g.NANOSECOND:return a.h.TimeNanosecond}return a.h.Time;case a.h.Timestamp:switch(t.unit){case a.g.SECOND:return a.h.TimestampSecond;case a.g.MILLISECOND:return a.h.TimestampMillisecond;case a.g.MICROSECOND:return a.h.TimestampMicrosecond;case a.g.NANOSECOND:return a.h.TimestampNanosecond}return a.h.Timestamp;case a.h.Date:switch(t.unit){case a.b.DAY:return a.h.DateDay;case a.b.MILLISECOND:return a.h.DateMillisecond}return a.h.Date;case a.h.Interval:switch(t.unit){case a.c.DAY_TIME:return a.h.IntervalDayTime;case a.c.YEAR_MONTH:return a.h.IntervalYearMonth}return a.h.Interval;case a.h.Map:return a.h.Map;case a.h.List:return a.h.List;case a.h.Struct:return a.h.Struct;case a.h.Union:switch(t.mode){case a.i.Dense:return a.h.DenseUnion;case a.i.Sparse:return a.h.SparseUnion}return a.h.Union;case a.h.FixedSizeBinary:return a.h.FixedSizeBinary;case a.h.FixedSizeList:return a.h.FixedSizeList;case a.h.Dictionary:return a.h.Dictionary}throw new Error("Unrecognized type \'"+a.h[t.typeId]+"\'")}u.prototype.visitInt8=null,u.prototype.visitInt16=null,u.prototype.visitInt32=null,u.prototype.visitInt64=null,u.prototype.visitUint8=null,u.prototype.visitUint16=null,u.prototype.visitUint32=null,u.prototype.visitUint64=null,u.prototype.visitFloat16=null,u.prototype.visitFloat32=null,u.prototype.visitFloat64=null,u.prototype.visitDateDay=null,u.prototype.visitDateMillisecond=null,u.prototype.visitTimestampSecond=null,u.prototype.visitTimestampMillisecond=null,u.prototype.visitTimestampMicrosecond=null,u.prototype.visitTimestampNanosecond=null,u.prototype.visitTimeSecond=null,u.prototype.visitTimeMillisecond=null,u.prototype.visitTimeMicrosecond=null,u.prototype.visitTimeNanosecond=null,u.prototype.visitDenseUnion=null,u.prototype.visitSparseUnion=null,u.prototype.visitIntervalDayTime=null,u.prototype.visitIntervalYearMonth=null},function(t,e,n){"use strict";n.d(e,"b",function(){return i}),n.d(e,"a",function(){return o}),n.d(e,"f",function(){return a}),n.d(e,"g",function(){return s}),n.d(e,"d",function(){return u}),n.d(e,"c",function(){return c}),n.d(e,"e",function(){return f});var r=n(0);function i(t,e,n,r){return 0!=(n&1<<r)}function o(t,e,n,r){return(n&1<<r)>>r}function a(t,e,n){return n?!!(t[e>>3]|=1<<e%8)||!0:!(t[e>>3]&=~(1<<e%8))&&!1}function s(t,e,n){var r=n.byteLength+7&-8;if(t>0||n.byteLength<r){var o=new Uint8Array(r);return o.set(t%8==0?n.subarray(t>>3):u(c(n,t,e,null,i))),o}return n}function u(t){var e,n,i=0,o=0,a=[],s=0,u=0;try{for(var c=r.k(t),f=c.next();!f.done;f=c.next())f.value&&(u|=1<<s),8==++s&&(a[o++]=u,u=s=0)}catch(t){e={error:t}}finally{try{f&&!f.done&&(n=c.return)&&n.call(c)}finally{if(e)throw e.error}}if((0===o||s>0)&&(a[o++]=u),o%8&&(i=o+8-o%8))do{a[o]=0}while(++o<i);return new Uint8Array(a)}function c(t,e,n,i,o){var a,s,u,c,f;return r.h(this,function(r){switch(r.label){case 0:a=e%8,s=e>>3,u=0,c=n,r.label=1;case 1:if(!(c>0))return[3,6];f=t[s++],r.label=2;case 2:return[4,o(i,u++,f,a)];case 3:r.sent(),r.label=4;case 4:if(--c>0&&++a<8)return[3,2];r.label=5;case 5:return a=0,[3,1];case 6:return[2]}})}function f(t,e,n){var i,a;if(n-e<=0)return 0;if(n-e<8){var s=0;try{for(var u=r.k(c(t,e,n-e,t,o)),h=u.next();!h.done;h=u.next())s+=h.value}catch(t){i={error:t}}finally{try{h&&!h.done&&(a=u.return)&&a.call(u)}finally{if(i)throw i.error}}return s}var p=n>>3<<3,d=e+(e%8==0?0:8-e%8);return f(t,e,d)+f(t,p,n)+function(t,e,n){for(var r=0,i=0|d>>3,o=new DataView(t.buffer,t.byteOffset,t.byteLength),a=void 0===n?t.byteLength:i+n;a-i>=4;)r+=l(o.getUint32(i)),i+=4;for(;a-i>=2;)r+=l(o.getUint16(i)),i+=2;for(;a-i>=1;)r+=l(o.getUint8(i)),i+=1;return r}(t,0,p-d>>3)}function l(t){var e=0|t;return 16843009*((e=(858993459&(e-=e>>>1&1431655765))+(e>>>2&858993459))+(e>>>4)&252645135)>>>24}},function(t,e,n){"use strict";var r=n(0),i=n(3);e.a={fromIterable:function(t){return o(function(t){function e(){var t;return"peek"===l?Object(i.b)(c,h)[0]:(t=r.i(Object(i.b)(c,h),3),f=t[0],c=t[1],p=t[2],f)}var n,o,a,s,u,c,f,l,h,p,d,y;return r.h(this,function(r){switch(r.label){case 0:return u=!1,c=[],p=0,[4,null];case 1:n=r.sent(),l=n.cmd,h=n.size,d=Object(i.g)(t)[Symbol.iterator](),r.label=2;case 2:r.trys.push([2,9,10,11]),r.label=3;case 3:if(o=isNaN(h-p)?d.next(void 0):d.next(h-p),s=o.done,f=o.value,!s&&f.byteLength>0&&(c.push(f),p+=f.byteLength),!(s||h<=p))return[3,7];r.label=4;case 4:return[4,e()];case 5:a=r.sent(),l=a.cmd,h=a.size,r.label=6;case 6:if(h<p)return[3,4];r.label=7;case 7:if(!s)return[3,3];r.label=8;case 8:return[3,11];case 9:return y=r.sent(),(u=!0)&&"function"==typeof d.throw&&d.throw(y),[3,11];case 10:return!1===u&&"function"==typeof d.return&&d.return(),[7];case 11:return[2]}})}(t))},fromAsyncIterable:function(t){return o(function(t){return r.c(this,arguments,function(){function e(){var t;return"peek"===l?Object(i.b)(c,h)[0]:(t=r.i(Object(i.b)(c,h),3),f=t[0],c=t[1],p=t[2],f)}var n,o,a,s,u,c,f,l,h,p,d,y,b;return r.h(this,function(v){switch(v.label){case 0:return u=!1,c=[],p=0,[4,r.e(null)];case 1:return[4,v.sent()];case 2:n=v.sent(),l=n.cmd,h=n.size,d=Object(i.f)(t)[Symbol.asyncIterator](),v.label=3;case 3:v.trys.push([3,15,18,21]),v.label=4;case 4:return isNaN(h-p)?[4,r.e(d.next(void 0))]:[3,6];case 5:return y=v.sent(),[3,8];case 6:return[4,r.e(d.next(h-p))];case 7:y=v.sent(),v.label=8;case 8:if(s=(o=y).done,f=o.value,!s&&f.byteLength>0&&(c.push(f),p+=f.byteLength),!(s||h<=p))return[3,13];v.label=9;case 9:return[4,r.e(e())];case 10:return[4,v.sent()];case 11:a=v.sent(),l=a.cmd,h=a.size,v.label=12;case 12:if(h<p)return[3,9];v.label=13;case 13:if(!s)return[3,4];v.label=14;case 14:return[3,21];case 15:return b=v.sent(),(u=!0)&&"function"==typeof d.throw?[4,r.e(d.throw(b))]:[3,17];case 16:v.sent(),v.label=17;case 17:return[3,21];case 18:return!1===u&&"function"==typeof d.return?[4,r.e(d.return())]:[3,20];case 19:v.sent(),v.label=20;case 20:return[7];case 21:return[2]}})})}(t))},fromDOMStream:function(t){return o(function(t){return r.c(this,arguments,function(){function e(){var t;return"peek"===h?Object(i.b)(f,p)[0]:(t=r.i(Object(i.b)(f,p),3),l=t[0],f=t[1],d=t[2],l)}var n,o,s,u,c,f,l,h,p,d,y,b,v;return r.h(this,function(m){switch(m.label){case 0:return u=!1,c=!1,f=[],d=0,[4,r.e(null)];case 1:return[4,m.sent()];case 2:n=m.sent(),h=n.cmd,p=n.size,y=new a(t),m.label=3;case 3:m.trys.push([3,15,18,22]),m.label=4;case 4:return isNaN(p-d)?[4,r.e(y.read(void 0))]:[3,6];case 5:return b=m.sent(),[3,8];case 6:return[4,r.e(y.read(p-d))];case 7:b=m.sent(),m.label=8;case 8:if(u=(o=b).done,l=o.value,!u&&l.byteLength>0&&(f.push(Object(i.e)(l)),d+=l.byteLength),!(u||p<=d))return[3,13];m.label=9;case 9:return[4,r.e(e())];case 10:return[4,m.sent()];case 11:s=m.sent(),h=s.cmd,p=s.size,m.label=12;case 12:if(p<d)return[3,9];m.label=13;case 13:if(!u)return[3,4];m.label=14;case 14:return[3,22];case 15:return v=m.sent(),(c=!0)?[4,r.e(y.cancel(v))]:[3,17];case 16:m.sent(),m.label=17;case 17:return[3,22];case 18:return!1!==c?[3,20]:[4,r.e(y.cancel())];case 19:return m.sent(),[3,21];case 20:t.locked&&y.releaseLock(),m.label=21;case 21:return[7];case 22:return[2]}})})}(t))},fromNodeStream:function(t){return o(function(t){return r.c(this,arguments,function(){function e(){var t;return"peek"===p?Object(i.b)(b,d)[0]:(t=r.i(Object(i.b)(b,d),3),v=t[0],b=t[1],y=t[2],v)}function n(e,n){var i=this;return v=b=null,new Promise(function(o,a){return r.f(i,void 0,void 0,function(){var i,s,u,c,f,l,h,p;return r.h(this,function(d){try{for(u=r.k(e),c=u.next();!c.done;c=u.next())f=r.i(c.value,2),l=f[0],h=f[1],t.off(l,h)}catch(t){i={error:t}}finally{try{c&&!c.done&&(s=u.return)&&s.call(u)}finally{if(i)throw i.error}}try{(p=t.destroy)&&p.call(t,n),n=void 0}catch(t){n=t||n}finally{null!=n?a(n):o()}return[2]})})})}var o,a,s,c,f,l,h,p,d,y,b,v;return r.h(this,function(m){switch(m.label){case 0:return c=[],f="error",l=!1,h=null,y=0,b=[],[4,r.e(null)];case 1:return[4,m.sent()];case 2:return o=m.sent(),p=o.cmd,d=o.size,t.isTTY?[4,r.e(new Uint8Array(0))]:[3,6];case 3:return[4,m.sent()];case 4:return[4,r.e.apply(void 0,[m.sent()])];case 5:return[2,m.sent()];case 6:m.trys.push([6,,15,17]),c[0]=u(t,"end"),c[1]=u(t,"error"),m.label=7;case 7:return c[2]=u(t,"readable"),[4,r.e(Promise.race(c.map(function(t){return t[2]})))];case 8:if(a=r.i.apply(void 0,[m.sent(),2]),f=a[0],h=a[1],"error"===f)return[3,14];if((l="end"===f)||(isFinite(d-y)?(v=Object(i.e)(t.read(d-y))).byteLength<d-y&&(v=Object(i.e)(t.read(void 0))):v=Object(i.e)(t.read(void 0)),v.byteLength>0&&(b.push(v),y+=v.byteLength)),!(l||d<=y))return[3,13];m.label=9;case 9:return[4,r.e(e())];case 10:return[4,m.sent()];case 11:s=m.sent(),p=s.cmd,d=s.size,m.label=12;case 12:if(d<y)return[3,9];m.label=13;case 13:if(!l)return[3,7];m.label=14;case 14:return[3,17];case 15:return[4,r.e(n(c,"error"===f?h:null))];case 16:return m.sent(),[7];case 17:return[2]}})})}(t))},toDOMStream:function(t,e){throw new Error(\'"toDOMStream" not available in this environment\')},toNodeStream:function(t,e){throw new Error(\'"toNodeStream" not available in this environment\')}};var o=function(t){return t.next(),t},a=function(){function t(t){this.source=t,this.byobReader=null,this.defaultReader=null;try{this.supportsBYOB=!!(this.reader=this.getBYOBReader())}catch(t){this.supportsBYOB=!(this.reader=this.getDefaultReader())}}return Object.defineProperty(t.prototype,"closed",{get:function(){return this.reader?this.reader.closed.catch(function(){}):Promise.resolve()},enumerable:!0,configurable:!0}),t.prototype.releaseLock=function(){this.reader&&this.reader.releaseLock(),this.reader=this.byobReader=this.defaultReader=null},t.prototype.cancel=function(t){return r.f(this,void 0,void 0,function(){var e,n,i;return r.h(this,function(r){switch(r.label){case 0:return n=(e=this).reader,i=e.source,n?[4,n.cancel(t)]:[3,2];case 1:r.sent(),r.label=2;case 2:return i&&i.locked&&this.releaseLock(),[2]}})})},t.prototype.read=function(t){return r.f(this,void 0,void 0,function(){var e,n;return r.h(this,function(r){switch(r.label){case 0:return 0===t?[2,{done:null==this.reader,value:new Uint8Array(0)}]:this.supportsBYOB&&"number"==typeof t?[3,2]:[4,this.getDefaultReader().read()];case 1:return n=r.sent(),[3,4];case 2:return[4,this.readFromBYOBReader(t)];case 3:n=r.sent(),r.label=4;case 4:return!(e=n).done&&(e.value=Object(i.e)(e)),[2,e]}})})},t.prototype.getDefaultReader=function(){return this.byobReader&&this.releaseLock(),this.defaultReader||(this.defaultReader=this.source.getReader(),this.defaultReader.closed.catch(function(){})),this.reader=this.defaultReader},t.prototype.getBYOBReader=function(){return this.defaultReader&&this.releaseLock(),this.byobReader||(this.byobReader=this.source.getReader({mode:"byob"}),this.byobReader.closed.catch(function(){})),this.reader=this.byobReader},t.prototype.readFromBYOBReader=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,s(this.getBYOBReader(),new ArrayBuffer(t),0,t)];case 1:return[2,e.sent()]}})})},t}();function s(t,e,n,i){return r.f(this,void 0,void 0,function(){var o,a,u;return r.h(this,function(r){switch(r.label){case 0:return n>=i?[2,{done:!1,value:new Uint8Array(e,0,i)}]:[4,t.read(new Uint8Array(e,n,i-n))];case 1:return o=r.sent(),a=o.done,u=o.value,(n+=u.byteLength)<i&&!a?[4,s(t,u.buffer,n,i)]:[3,3];case 2:return[2,r.sent()];case 3:return[2,{done:a,value:new Uint8Array(u.buffer,0,n)}]}})})}var u=function(t,e){var n,r=function(t){return n([e,t])};return[e,r,new Promise(function(i){return(n=i)&&t.once(e,r)})]}},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r={};r.Offset,r.Table,r.SIZEOF_SHORT=2,r.SIZEOF_INT=4,r.FILE_IDENTIFIER_LENGTH=4,r.Encoding={UTF8_BYTES:1,UTF16_STRING:2},r.int32=new Int32Array(2),r.float32=new Float32Array(r.int32.buffer),r.float64=new Float64Array(r.int32.buffer),r.isLittleEndian=1===new Uint16Array(new Uint8Array([1,0]).buffer)[0],r.Long=function(t,e){this.low=0|t,this.high=0|e},r.Long.create=function(t,e){return 0==t&&0==e?r.Long.ZERO:new r.Long(t,e)},r.Long.prototype.toFloat64=function(){return(this.low>>>0)+4294967296*this.high},r.Long.prototype.equals=function(t){return this.low==t.low&&this.high==t.high},r.Long.ZERO=new r.Long(0,0),r.Builder=function(t){if(t)e=t;else var e=1024;this.bb=r.ByteBuffer.allocate(e),this.space=e,this.minalign=1,this.vtable=null,this.vtable_in_use=0,this.isNested=!1,this.object_start=0,this.vtables=[],this.vector_num_elems=0,this.force_defaults=!1},r.Builder.prototype.clear=function(){this.bb.clear(),this.space=this.bb.capacity(),this.minalign=1,this.vtable=null,this.vtable_in_use=0,this.isNested=!1,this.object_start=0,this.vtables=[],this.vector_num_elems=0,this.force_defaults=!1},r.Builder.prototype.forceDefaults=function(t){this.force_defaults=t},r.Builder.prototype.dataBuffer=function(){return this.bb},r.Builder.prototype.asUint8Array=function(){return this.bb.bytes().subarray(this.bb.position(),this.bb.position()+this.offset())},r.Builder.prototype.prep=function(t,e){t>this.minalign&&(this.minalign=t);for(var n=1+~(this.bb.capacity()-this.space+e)&t-1;this.space<n+t+e;){var i=this.bb.capacity();this.bb=r.Builder.growByteBuffer(this.bb),this.space+=this.bb.capacity()-i}this.pad(n)},r.Builder.prototype.pad=function(t){for(var e=0;e<t;e++)this.bb.writeInt8(--this.space,0)},r.Builder.prototype.writeInt8=function(t){this.bb.writeInt8(this.space-=1,t)},r.Builder.prototype.writeInt16=function(t){this.bb.writeInt16(this.space-=2,t)},r.Builder.prototype.writeInt32=function(t){this.bb.writeInt32(this.space-=4,t)},r.Builder.prototype.writeInt64=function(t){this.bb.writeInt64(this.space-=8,t)},r.Builder.prototype.writeFloat32=function(t){this.bb.writeFloat32(this.space-=4,t)},r.Builder.prototype.writeFloat64=function(t){this.bb.writeFloat64(this.space-=8,t)},r.Builder.prototype.addInt8=function(t){this.prep(1,0),this.writeInt8(t)},r.Builder.prototype.addInt16=function(t){this.prep(2,0),this.writeInt16(t)},r.Builder.prototype.addInt32=function(t){this.prep(4,0),this.writeInt32(t)},r.Builder.prototype.addInt64=function(t){this.prep(8,0),this.writeInt64(t)},r.Builder.prototype.addFloat32=function(t){this.prep(4,0),this.writeFloat32(t)},r.Builder.prototype.addFloat64=function(t){this.prep(8,0),this.writeFloat64(t)},r.Builder.prototype.addFieldInt8=function(t,e,n){(this.force_defaults||e!=n)&&(this.addInt8(e),this.slot(t))},r.Builder.prototype.addFieldInt16=function(t,e,n){(this.force_defaults||e!=n)&&(this.addInt16(e),this.slot(t))},r.Builder.prototype.addFieldInt32=function(t,e,n){(this.force_defaults||e!=n)&&(this.addInt32(e),this.slot(t))},r.Builder.prototype.addFieldInt64=function(t,e,n){!this.force_defaults&&e.equals(n)||(this.addInt64(e),this.slot(t))},r.Builder.prototype.addFieldFloat32=function(t,e,n){(this.force_defaults||e!=n)&&(this.addFloat32(e),this.slot(t))},r.Builder.prototype.addFieldFloat64=function(t,e,n){(this.force_defaults||e!=n)&&(this.addFloat64(e),this.slot(t))},r.Builder.prototype.addFieldOffset=function(t,e,n){(this.force_defaults||e!=n)&&(this.addOffset(e),this.slot(t))},r.Builder.prototype.addFieldStruct=function(t,e,n){e!=n&&(this.nested(e),this.slot(t))},r.Builder.prototype.nested=function(t){if(t!=this.offset())throw new Error("FlatBuffers: struct must be serialized inline.")},r.Builder.prototype.notNested=function(){if(this.isNested)throw new Error("FlatBuffers: object serialization must not be nested.")},r.Builder.prototype.slot=function(t){this.vtable[t]=this.offset()},r.Builder.prototype.offset=function(){return this.bb.capacity()-this.space},r.Builder.growByteBuffer=function(t){var e=t.capacity();if(3221225472&e)throw new Error("FlatBuffers: cannot grow buffer beyond 2 gigabytes.");var n=e<<1,i=r.ByteBuffer.allocate(n);return i.setPosition(n-e),i.bytes().set(t.bytes(),n-e),i},r.Builder.prototype.addOffset=function(t){this.prep(r.SIZEOF_INT,0),this.writeInt32(this.offset()-t+r.SIZEOF_INT)},r.Builder.prototype.startObject=function(t){this.notNested(),null==this.vtable&&(this.vtable=[]),this.vtable_in_use=t;for(var e=0;e<t;e++)this.vtable[e]=0;this.isNested=!0,this.object_start=this.offset()},r.Builder.prototype.endObject=function(){if(null==this.vtable||!this.isNested)throw new Error("FlatBuffers: endObject called without startObject");this.addInt32(0);for(var t=this.offset(),e=this.vtable_in_use-1;e>=0&&0==this.vtable[e];e--);for(var n=e+1;e>=0;e--)this.addInt16(0!=this.vtable[e]?t-this.vtable[e]:0);this.addInt16(t-this.object_start);var i=(n+2)*r.SIZEOF_SHORT;this.addInt16(i);var o=0,a=this.space;t:for(e=0;e<this.vtables.length;e++){var s=this.bb.capacity()-this.vtables[e];if(i==this.bb.readInt16(s)){for(var u=r.SIZEOF_SHORT;u<i;u+=r.SIZEOF_SHORT)if(this.bb.readInt16(a+u)!=this.bb.readInt16(s+u))continue t;o=this.vtables[e];break}}return o?(this.space=this.bb.capacity()-t,this.bb.writeInt32(this.space,o-t)):(this.vtables.push(this.offset()),this.bb.writeInt32(this.bb.capacity()-t,this.offset()-t)),this.isNested=!1,t},r.Builder.prototype.finish=function(t,e){if(e){var n=e;if(this.prep(this.minalign,r.SIZEOF_INT+r.FILE_IDENTIFIER_LENGTH),n.length!=r.FILE_IDENTIFIER_LENGTH)throw new Error("FlatBuffers: file identifier must be length "+r.FILE_IDENTIFIER_LENGTH);for(var i=r.FILE_IDENTIFIER_LENGTH-1;i>=0;i--)this.writeInt8(n.charCodeAt(i))}this.prep(this.minalign,r.SIZEOF_INT),this.addOffset(t),this.bb.setPosition(this.space)},r.Builder.prototype.requiredField=function(t,e){var n=this.bb.capacity()-t,r=n-this.bb.readInt32(n);if(0==this.bb.readInt16(r+e))throw new Error("FlatBuffers: field "+e+" must be set")},r.Builder.prototype.startVector=function(t,e,n){this.notNested(),this.vector_num_elems=e,this.prep(r.SIZEOF_INT,t*e),this.prep(n,t*e)},r.Builder.prototype.endVector=function(){return this.writeInt32(this.vector_num_elems),this.offset()},r.Builder.prototype.createString=function(t){if(t instanceof Uint8Array)var e=t;else{e=[];for(var n=0;n<t.length;){var r,i=t.charCodeAt(n++);(r=i<55296||i>=56320?i:(i<<10)+t.charCodeAt(n++)+-56613888)<128?e.push(r):(r<2048?e.push(r>>6&31|192):(r<65536?e.push(r>>12&15|224):e.push(r>>18&7|240,r>>12&63|128),e.push(r>>6&63|128)),e.push(63&r|128))}}this.addInt8(0),this.startVector(1,e.length,1),this.bb.setPosition(this.space-=e.length),n=0;for(var o=this.space,a=this.bb.bytes();n<e.length;n++)a[o++]=e[n];return this.endVector()},r.Builder.prototype.createLong=function(t,e){return r.Long.create(t,e)},r.ByteBuffer=function(t){this.bytes_=t,this.position_=0},r.ByteBuffer.allocate=function(t){return new r.ByteBuffer(new Uint8Array(t))},r.ByteBuffer.prototype.clear=function(){this.position_=0},r.ByteBuffer.prototype.bytes=function(){return this.bytes_},r.ByteBuffer.prototype.position=function(){return this.position_},r.ByteBuffer.prototype.setPosition=function(t){this.position_=t},r.ByteBuffer.prototype.capacity=function(){return this.bytes_.length},r.ByteBuffer.prototype.readInt8=function(t){return this.readUint8(t)<<24>>24},r.ByteBuffer.prototype.readUint8=function(t){return this.bytes_[t]},r.ByteBuffer.prototype.readInt16=function(t){return this.readUint16(t)<<16>>16},r.ByteBuffer.prototype.readUint16=function(t){return this.bytes_[t]|this.bytes_[t+1]<<8},r.ByteBuffer.prototype.readInt32=function(t){return this.bytes_[t]|this.bytes_[t+1]<<8|this.bytes_[t+2]<<16|this.bytes_[t+3]<<24},r.ByteBuffer.prototype.readUint32=function(t){return this.readInt32(t)>>>0},r.ByteBuffer.prototype.readInt64=function(t){return new r.Long(this.readInt32(t),this.readInt32(t+4))},r.ByteBuffer.prototype.readUint64=function(t){return new r.Long(this.readUint32(t),this.readUint32(t+4))},r.ByteBuffer.prototype.readFloat32=function(t){return r.int32[0]=this.readInt32(t),r.float32[0]},r.ByteBuffer.prototype.readFloat64=function(t){return r.int32[r.isLittleEndian?0:1]=this.readInt32(t),r.int32[r.isLittleEndian?1:0]=this.readInt32(t+4),r.float64[0]},r.ByteBuffer.prototype.writeInt8=function(t,e){this.bytes_[t]=e},r.ByteBuffer.prototype.writeUint8=function(t,e){this.bytes_[t]=e},r.ByteBuffer.prototype.writeInt16=function(t,e){this.bytes_[t]=e,this.bytes_[t+1]=e>>8},r.ByteBuffer.prototype.writeUint16=function(t,e){this.bytes_[t]=e,this.bytes_[t+1]=e>>8},r.ByteBuffer.prototype.writeInt32=function(t,e){this.bytes_[t]=e,this.bytes_[t+1]=e>>8,this.bytes_[t+2]=e>>16,this.bytes_[t+3]=e>>24},r.ByteBuffer.prototype.writeUint32=function(t,e){this.bytes_[t]=e,this.bytes_[t+1]=e>>8,this.bytes_[t+2]=e>>16,this.bytes_[t+3]=e>>24},r.ByteBuffer.prototype.writeInt64=function(t,e){this.writeInt32(t,e.low),this.writeInt32(t+4,e.high)},r.ByteBuffer.prototype.writeUint64=function(t,e){this.writeUint32(t,e.low),this.writeUint32(t+4,e.high)},r.ByteBuffer.prototype.writeFloat32=function(t,e){r.float32[0]=e,this.writeInt32(t,r.int32[0])},r.ByteBuffer.prototype.writeFloat64=function(t,e){r.float64[0]=e,this.writeInt32(t,r.int32[r.isLittleEndian?0:1]),this.writeInt32(t+4,r.int32[r.isLittleEndian?1:0])},r.ByteBuffer.prototype.getBufferIdentifier=function(){if(this.bytes_.length<this.position_+r.SIZEOF_INT+r.FILE_IDENTIFIER_LENGTH)throw new Error("FlatBuffers: ByteBuffer is too short to contain an identifier.");for(var t="",e=0;e<r.FILE_IDENTIFIER_LENGTH;e++)t+=String.fromCharCode(this.readInt8(this.position_+r.SIZEOF_INT+e));return t},r.ByteBuffer.prototype.__offset=function(t,e){var n=t-this.readInt32(t);return e<this.readInt16(n)?this.readInt16(n+e):0},r.ByteBuffer.prototype.__union=function(t,e){return t.bb_pos=e+this.readInt32(e),t.bb=this,t},r.ByteBuffer.prototype.__string=function(t,e){t+=this.readInt32(t);var n=this.readInt32(t),i="",o=0;if(t+=r.SIZEOF_INT,e===r.Encoding.UTF8_BYTES)return this.bytes_.subarray(t,t+n);for(;o<n;){var a,s=this.readUint8(t+o++);if(s<192)a=s;else{var u=this.readUint8(t+o++);if(s<224)a=(31&s)<<6|63&u;else{var c=this.readUint8(t+o++);a=s<240?(15&s)<<12|(63&u)<<6|63&c:(7&s)<<18|(63&u)<<12|(63&c)<<6|63&this.readUint8(t+o++)}}a<65536?i+=String.fromCharCode(a):(a-=65536,i+=String.fromCharCode(55296+(a>>10),56320+(1023&a)))}return i},r.ByteBuffer.prototype.__indirect=function(t){return t+this.readInt32(t)},r.ByteBuffer.prototype.__vector=function(t){return t+this.readInt32(t)+r.SIZEOF_INT},r.ByteBuffer.prototype.__vector_len=function(t){return this.readInt32(t+this.readInt32(t))},r.ByteBuffer.prototype.__has_identifier=function(t){if(t.length!=r.FILE_IDENTIFIER_LENGTH)throw new Error("FlatBuffers: file identifier must be length "+r.FILE_IDENTIFIER_LENGTH);for(var e=0;e<r.FILE_IDENTIFIER_LENGTH;e++)if(t.charCodeAt(e)!=this.readInt8(this.position_+r.SIZEOF_INT+e))return!1;return!0},r.ByteBuffer.prototype.createLong=function(t,e){return r.Long.create(t,e)}},function(t,e,n){"use strict";n.r(e),n.d(e,"CONFIG_ALIASES",function(){return r}),n.d(e,"CONFIG_VALID_KEYS",function(){return i}),n.d(e,"SORT_ORDERS",function(){return s}),n.d(e,"SORT_ORDER_IDS",function(){return u}),n.d(e,"TYPE_AGGREGATES",function(){return c}),n.d(e,"COLUMN_SEPARATOR_STRING",function(){return h}),n.d(e,"TYPE_FILTERS",function(){return p});const r={row_pivot:"row_pivots","row-pivot":"row_pivots","row-pivots":"row_pivots",col_pivot:"column_pivots",col_pivots:"column_pivots",column_pivot:"column_pivots","column-pivot":"column_pivots","column-pivots":"column_pivots",filters:"filter",sorts:"sort"},i=["viewport","row_pivots","column_pivots","aggregates","columns","filter","sort","row_pivot_depth","filter_op"],o=["any","avg","count","distinct count","dominant","first by index","last by index","last","high","low","mean","mean by count","median","pct sum parent","pct sum grand total","sum","sum abs","sum not null","unique"],a=["any","count","distinct count","distinct leaf","dominant","first by index","last by index","last","mean by count","unique"],s=["none","asc","desc","col asc","col desc","asc abs","desc abs","col asc abs","col desc abs"],u=[2,0,1,0,1,3,4,3,4],c={string:a,float:o,integer:o,boolean:["any","count","distinct count","distinct leaf","dominant","first by index","last by index","last","mean by count","unique","and","or"],datetime:a,date:a},f=["<",">","==","<=",">=","!=","is nan","is not nan"],l=["<",">","==","<=",">=","!="],h="|",p={string:["==","contains","!=","in","not in","begins with","ends with"],float:f,integer:f,boolean:["&","|","==","!=","or","and"],datetime:l,date:l}},function(t,e,n){"use strict";(function(t){n.d(e,"a",function(){return s}),n.d(e,"b",function(){return u});var r=n(3),i=n(55),o="function"==typeof t?t:null,a="function"==typeof TextDecoder&&"function"==typeof TextEncoder,s=function(t){if(a||!o){var e=new t;return e.decode.bind(e)}return function(t){var e=Object(r.e)(t),n=e.buffer,i=e.byteOffset,a=e.length;return o.from(n,i,a).toString()}}("undefined"!=typeof TextDecoder?TextDecoder:i.a),u=function(t){if(a||!o){var e=new t;return e.encode.bind(e)}return function(t){return void 0===t&&(t=""),Object(r.e)(o.from(t,"utf8"))}}("undefined"!=typeof TextEncoder?TextEncoder:i.b)}).call(this,n(57).Buffer)},function(t,e,n){var r=n(40)("wks"),i=n(42),o=n(18).Symbol,a="function"==typeof o;(t.exports=function(t){return r[t]||(r[t]=a&&o[t]||(a?o:i)("Symbol."+t))}).store=r},function(t,e,n){var r=n(25);t.exports=function(t){if(!r(t))throw TypeError(t+" is not an object!");return t}},function(t,e,n){(function(t){t.exports=function(){"use strict";var e,r;function i(){return e.apply(null,arguments)}function o(t){return t instanceof Array||"[object Array]"===Object.prototype.toString.call(t)}function a(t){return null!=t&&"[object Object]"===Object.prototype.toString.call(t)}function s(t){return void 0===t}function u(t){return"number"==typeof t||"[object Number]"===Object.prototype.toString.call(t)}function c(t){return t instanceof Date||"[object Date]"===Object.prototype.toString.call(t)}function f(t,e){var n,r=[];for(n=0;n<t.length;++n)r.push(e(t[n],n));return r}function l(t,e){return Object.prototype.hasOwnProperty.call(t,e)}function h(t,e){for(var n in e)l(e,n)&&(t[n]=e[n]);return l(e,"toString")&&(t.toString=e.toString),l(e,"valueOf")&&(t.valueOf=e.valueOf),t}function p(t,e,n,r){return je(t,e,n,r,!0).utc()}function d(t){return null==t._pf&&(t._pf={empty:!1,unusedTokens:[],unusedInput:[],overflow:-2,charsLeftOver:0,nullInput:!1,invalidMonth:null,invalidFormat:!1,userInvalidated:!1,iso:!1,parsedDateParts:[],meridiem:null,rfc2822:!1,weekdayMismatch:!1}),t._pf}function y(t){if(null==t._isValid){var e=d(t),n=r.call(e.parsedDateParts,function(t){return null!=t}),i=!isNaN(t._d.getTime())&&e.overflow<0&&!e.empty&&!e.invalidMonth&&!e.invalidWeekday&&!e.weekdayMismatch&&!e.nullInput&&!e.invalidFormat&&!e.userInvalidated&&(!e.meridiem||e.meridiem&&n);if(t._strict&&(i=i&&0===e.charsLeftOver&&0===e.unusedTokens.length&&void 0===e.bigHour),null!=Object.isFrozen&&Object.isFrozen(t))return i;t._isValid=i}return t._isValid}function b(t){var e=p(NaN);return null!=t?h(d(e),t):d(e).userInvalidated=!0,e}r=Array.prototype.some?Array.prototype.some:function(t){for(var e=Object(this),n=e.length>>>0,r=0;r<n;r++)if(r in e&&t.call(this,e[r],r,e))return!0;return!1};var v=i.momentProperties=[];function m(t,e){var n,r,i;if(s(e._isAMomentObject)||(t._isAMomentObject=e._isAMomentObject),s(e._i)||(t._i=e._i),s(e._f)||(t._f=e._f),s(e._l)||(t._l=e._l),s(e._strict)||(t._strict=e._strict),s(e._tzm)||(t._tzm=e._tzm),s(e._isUTC)||(t._isUTC=e._isUTC),s(e._offset)||(t._offset=e._offset),s(e._pf)||(t._pf=d(e)),s(e._locale)||(t._locale=e._locale),v.length>0)for(n=0;n<v.length;n++)s(i=e[r=v[n]])||(t[r]=i);return t}var _=!1;function g(t){m(this,t),this._d=new Date(null!=t._d?t._d.getTime():NaN),this.isValid()||(this._d=new Date(NaN)),!1===_&&(_=!0,i.updateOffset(this),_=!1)}function w(t){return t instanceof g||null!=t&&null!=t._isAMomentObject}function S(t){return t<0?Math.ceil(t)||0:Math.floor(t)}function O(t){var e=+t,n=0;return 0!==e&&isFinite(e)&&(n=S(e)),n}function T(t,e,n){var r,i=Math.min(t.length,e.length),o=Math.abs(t.length-e.length),a=0;for(r=0;r<i;r++)(n&&t[r]!==e[r]||!n&&O(t[r])!==O(e[r]))&&a++;return a+o}function I(t){!1===i.suppressDeprecationWarnings&&"undefined"!=typeof console&&console.warn&&console.warn("Deprecation warning: "+t)}function D(t,e){var n=!0;return h(function(){if(null!=i.deprecationHandler&&i.deprecationHandler(null,t),n){for(var r,o=[],a=0;a<arguments.length;a++){if(r="","object"==typeof arguments[a]){for(var s in r+="\\n["+a+"] ",arguments[0])r+=s+": "+arguments[0][s]+", ";r=r.slice(0,-2)}else r=arguments[a];o.push(r)}I(t+"\\nArguments: "+Array.prototype.slice.call(o).join("")+"\\n"+(new Error).stack),n=!1}return e.apply(this,arguments)},e)}var A,M={};function k(t,e){null!=i.deprecationHandler&&i.deprecationHandler(t,e),M[t]||(I(e),M[t]=!0)}function j(t){return t instanceof Function||"[object Function]"===Object.prototype.toString.call(t)}function x(t,e){var n,r=h({},t);for(n in e)l(e,n)&&(a(t[n])&&a(e[n])?(r[n]={},h(r[n],t[n]),h(r[n],e[n])):null!=e[n]?r[n]=e[n]:delete r[n]);for(n in t)l(t,n)&&!l(e,n)&&a(t[n])&&(r[n]=h({},r[n]));return r}function E(t){null!=t&&this.set(t)}i.suppressDeprecationWarnings=!1,i.deprecationHandler=null,A=Object.keys?Object.keys:function(t){var e,n=[];for(e in t)l(t,e)&&n.push(e);return n};var B={};function F(t,e){var n=t.toLowerCase();B[n]=B[n+"s"]=B[e]=t}function P(t){return"string"==typeof t?B[t]||B[t.toLowerCase()]:void 0}function L(t){var e,n,r={};for(n in t)l(t,n)&&(e=P(n))&&(r[e]=t[n]);return r}var U={};function C(t,e){U[t]=e}function R(t,e,n){var r=""+Math.abs(t),i=e-r.length;return(t>=0?n?"+":"":"-")+Math.pow(10,Math.max(0,i)).toString().substr(1)+r}var N=/(\\[[^\\[]*\\])|(\\\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|YYYYYY|YYYYY|YYYY|YY|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,Y=/(\\[[^\\[]*\\])|(\\\\)?(LTS|LT|LL?L?L?|l{1,4})/g,V={},z={};function W(t,e,n,r){var i=r;"string"==typeof r&&(i=function(){return this[r]()}),t&&(z[t]=i),e&&(z[e[0]]=function(){return R(i.apply(this,arguments),e[1],e[2])}),n&&(z[n]=function(){return this.localeData().ordinal(i.apply(this,arguments),t)})}function H(t,e){return t.isValid()?(e=G(e,t.localeData()),V[e]=V[e]||function(t){var e,n,r,i=t.match(N);for(e=0,n=i.length;e<n;e++)z[i[e]]?i[e]=z[i[e]]:i[e]=(r=i[e]).match(/\\[[\\s\\S]/)?r.replace(/^\\[|\\]$/g,""):r.replace(/\\\\/g,"");return function(e){var r,o="";for(r=0;r<n;r++)o+=j(i[r])?i[r].call(e,t):i[r];return o}}(e),V[e](t)):t.localeData().invalidDate()}function G(t,e){var n=5;function r(t){return e.longDateFormat(t)||t}for(Y.lastIndex=0;n>=0&&Y.test(t);)t=t.replace(Y,r),Y.lastIndex=0,n-=1;return t}var q=/\\d/,Z=/\\d\\d/,J=/\\d{3}/,K=/\\d{4}/,$=/[+-]?\\d{6}/,Q=/\\d\\d?/,X=/\\d\\d\\d\\d?/,tt=/\\d\\d\\d\\d\\d\\d?/,et=/\\d{1,3}/,nt=/\\d{1,4}/,rt=/[+-]?\\d{1,6}/,it=/\\d+/,ot=/[+-]?\\d+/,at=/Z|[+-]\\d\\d:?\\d\\d/gi,st=/Z|[+-]\\d\\d(?::?\\d\\d)?/gi,ut=/[0-9]{0,256}[\'a-z\\u00A0-\\u05FF\\u0700-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFF07\\uFF10-\\uFFEF]{1,256}|[\\u0600-\\u06FF\\/]{1,256}(\\s*?[\\u0600-\\u06FF]{1,256}){1,2}/i,ct={};function ft(t,e,n){ct[t]=j(e)?e:function(t,r){return t&&n?n:e}}function lt(t,e){return l(ct,t)?ct[t](e._strict,e._locale):new RegExp(ht(t.replace("\\\\","").replace(/\\\\(\\[)|\\\\(\\])|\\[([^\\]\\[]*)\\]|\\\\(.)/g,function(t,e,n,r,i){return e||n||r||i})))}function ht(t){return t.replace(/[-\\/\\\\^$*+?.()|[\\]{}]/g,"\\\\$&")}var pt={};function dt(t,e){var n,r=e;for("string"==typeof t&&(t=[t]),u(e)&&(r=function(t,n){n[e]=O(t)}),n=0;n<t.length;n++)pt[t[n]]=r}function yt(t,e){dt(t,function(t,n,r,i){r._w=r._w||{},e(t,r._w,r,i)})}function bt(t,e,n){null!=e&&l(pt,t)&&pt[t](e,n._a,n,t)}var vt=0,mt=1,_t=2,gt=3,wt=4,St=5,Ot=6,Tt=7,It=8;function Dt(t){return At(t)?366:365}function At(t){return t%4==0&&t%100!=0||t%400==0}W("Y",0,0,function(){var t=this.year();return t<=9999?""+t:"+"+t}),W(0,["YY",2],0,function(){return this.year()%100}),W(0,["YYYY",4],0,"year"),W(0,["YYYYY",5],0,"year"),W(0,["YYYYYY",6,!0],0,"year"),F("year","y"),C("year",1),ft("Y",ot),ft("YY",Q,Z),ft("YYYY",nt,K),ft("YYYYY",rt,$),ft("YYYYYY",rt,$),dt(["YYYYY","YYYYYY"],vt),dt("YYYY",function(t,e){e[vt]=2===t.length?i.parseTwoDigitYear(t):O(t)}),dt("YY",function(t,e){e[vt]=i.parseTwoDigitYear(t)}),dt("Y",function(t,e){e[vt]=parseInt(t,10)}),i.parseTwoDigitYear=function(t){return O(t)+(O(t)>68?1900:2e3)};var Mt,kt=jt("FullYear",!0);function jt(t,e){return function(n){return null!=n?(Et(this,t,n),i.updateOffset(this,e),this):xt(this,t)}}function xt(t,e){return t.isValid()?t._d["get"+(t._isUTC?"UTC":"")+e]():NaN}function Et(t,e,n){t.isValid()&&!isNaN(n)&&("FullYear"===e&&At(t.year())&&1===t.month()&&29===t.date()?t._d["set"+(t._isUTC?"UTC":"")+e](n,t.month(),Bt(n,t.month())):t._d["set"+(t._isUTC?"UTC":"")+e](n))}function Bt(t,e){if(isNaN(t)||isNaN(e))return NaN;var n=(e%12+12)%12;return t+=(e-n)/12,1===n?At(t)?29:28:31-n%7%2}Mt=Array.prototype.indexOf?Array.prototype.indexOf:function(t){var e;for(e=0;e<this.length;++e)if(this[e]===t)return e;return-1},W("M",["MM",2],"Mo",function(){return this.month()+1}),W("MMM",0,0,function(t){return this.localeData().monthsShort(this,t)}),W("MMMM",0,0,function(t){return this.localeData().months(this,t)}),F("month","M"),C("month",8),ft("M",Q),ft("MM",Q,Z),ft("MMM",function(t,e){return e.monthsShortRegex(t)}),ft("MMMM",function(t,e){return e.monthsRegex(t)}),dt(["M","MM"],function(t,e){e[mt]=O(t)-1}),dt(["MMM","MMMM"],function(t,e,n,r){var i=n._locale.monthsParse(t,r,n._strict);null!=i?e[mt]=i:d(n).invalidMonth=t});var Ft=/D[oD]?(\\[[^\\[\\]]*\\]|\\s)+MMMM?/,Pt="January_February_March_April_May_June_July_August_September_October_November_December".split("_"),Lt="Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_");function Ut(t,e){var n;if(!t.isValid())return t;if("string"==typeof e)if(/^\\d+$/.test(e))e=O(e);else if(!u(e=t.localeData().monthsParse(e)))return t;return n=Math.min(t.date(),Bt(t.year(),e)),t._d["set"+(t._isUTC?"UTC":"")+"Month"](e,n),t}function Ct(t){return null!=t?(Ut(this,t),i.updateOffset(this,!0),this):xt(this,"Month")}var Rt=ut,Nt=ut;function Yt(){function t(t,e){return e.length-t.length}var e,n,r=[],i=[],o=[];for(e=0;e<12;e++)n=p([2e3,e]),r.push(this.monthsShort(n,"")),i.push(this.months(n,"")),o.push(this.months(n,"")),o.push(this.monthsShort(n,""));for(r.sort(t),i.sort(t),o.sort(t),e=0;e<12;e++)r[e]=ht(r[e]),i[e]=ht(i[e]);for(e=0;e<24;e++)o[e]=ht(o[e]);this._monthsRegex=new RegExp("^("+o.join("|")+")","i"),this._monthsShortRegex=this._monthsRegex,this._monthsStrictRegex=new RegExp("^("+i.join("|")+")","i"),this._monthsShortStrictRegex=new RegExp("^("+r.join("|")+")","i")}function Vt(t){var e;if(t<100&&t>=0){var n=Array.prototype.slice.call(arguments);n[0]=t+400,e=new Date(Date.UTC.apply(null,n)),isFinite(e.getUTCFullYear())&&e.setUTCFullYear(t)}else e=new Date(Date.UTC.apply(null,arguments));return e}function zt(t,e,n){var r=7+e-n;return-((7+Vt(t,0,r).getUTCDay()-e)%7)+r-1}function Wt(t,e,n,r,i){var o,a,s=1+7*(e-1)+(7+n-r)%7+zt(t,r,i);return s<=0?a=Dt(o=t-1)+s:s>Dt(t)?(o=t+1,a=s-Dt(t)):(o=t,a=s),{year:o,dayOfYear:a}}function Ht(t,e,n){var r,i,o=zt(t.year(),e,n),a=Math.floor((t.dayOfYear()-o-1)/7)+1;return a<1?r=a+Gt(i=t.year()-1,e,n):a>Gt(t.year(),e,n)?(r=a-Gt(t.year(),e,n),i=t.year()+1):(i=t.year(),r=a),{week:r,year:i}}function Gt(t,e,n){var r=zt(t,e,n),i=zt(t+1,e,n);return(Dt(t)-r+i)/7}function qt(t,e){return t.slice(e,7).concat(t.slice(0,e))}W("w",["ww",2],"wo","week"),W("W",["WW",2],"Wo","isoWeek"),F("week","w"),F("isoWeek","W"),C("week",5),C("isoWeek",5),ft("w",Q),ft("ww",Q,Z),ft("W",Q),ft("WW",Q,Z),yt(["w","ww","W","WW"],function(t,e,n,r){e[r.substr(0,1)]=O(t)}),W("d",0,"do","day"),W("dd",0,0,function(t){return this.localeData().weekdaysMin(this,t)}),W("ddd",0,0,function(t){return this.localeData().weekdaysShort(this,t)}),W("dddd",0,0,function(t){return this.localeData().weekdays(this,t)}),W("e",0,0,"weekday"),W("E",0,0,"isoWeekday"),F("day","d"),F("weekday","e"),F("isoWeekday","E"),C("day",11),C("weekday",11),C("isoWeekday",11),ft("d",Q),ft("e",Q),ft("E",Q),ft("dd",function(t,e){return e.weekdaysMinRegex(t)}),ft("ddd",function(t,e){return e.weekdaysShortRegex(t)}),ft("dddd",function(t,e){return e.weekdaysRegex(t)}),yt(["dd","ddd","dddd"],function(t,e,n,r){var i=n._locale.weekdaysParse(t,r,n._strict);null!=i?e.d=i:d(n).invalidWeekday=t}),yt(["d","e","E"],function(t,e,n,r){e[r]=O(t)});var Zt="Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),Jt="Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),Kt="Su_Mo_Tu_We_Th_Fr_Sa".split("_"),$t=ut,Qt=ut,Xt=ut;function te(){function t(t,e){return e.length-t.length}var e,n,r,i,o,a=[],s=[],u=[],c=[];for(e=0;e<7;e++)n=p([2e3,1]).day(e),r=this.weekdaysMin(n,""),i=this.weekdaysShort(n,""),o=this.weekdays(n,""),a.push(r),s.push(i),u.push(o),c.push(r),c.push(i),c.push(o);for(a.sort(t),s.sort(t),u.sort(t),c.sort(t),e=0;e<7;e++)s[e]=ht(s[e]),u[e]=ht(u[e]),c[e]=ht(c[e]);this._weekdaysRegex=new RegExp("^("+c.join("|")+")","i"),this._weekdaysShortRegex=this._weekdaysRegex,this._weekdaysMinRegex=this._weekdaysRegex,this._weekdaysStrictRegex=new RegExp("^("+u.join("|")+")","i"),this._weekdaysShortStrictRegex=new RegExp("^("+s.join("|")+")","i"),this._weekdaysMinStrictRegex=new RegExp("^("+a.join("|")+")","i")}function ee(){return this.hours()%12||12}function ne(t,e){W(t,0,0,function(){return this.localeData().meridiem(this.hours(),this.minutes(),e)})}function re(t,e){return e._meridiemParse}W("H",["HH",2],0,"hour"),W("h",["hh",2],0,ee),W("k",["kk",2],0,function(){return this.hours()||24}),W("hmm",0,0,function(){return""+ee.apply(this)+R(this.minutes(),2)}),W("hmmss",0,0,function(){return""+ee.apply(this)+R(this.minutes(),2)+R(this.seconds(),2)}),W("Hmm",0,0,function(){return""+this.hours()+R(this.minutes(),2)}),W("Hmmss",0,0,function(){return""+this.hours()+R(this.minutes(),2)+R(this.seconds(),2)}),ne("a",!0),ne("A",!1),F("hour","h"),C("hour",13),ft("a",re),ft("A",re),ft("H",Q),ft("h",Q),ft("k",Q),ft("HH",Q,Z),ft("hh",Q,Z),ft("kk",Q,Z),ft("hmm",X),ft("hmmss",tt),ft("Hmm",X),ft("Hmmss",tt),dt(["H","HH"],gt),dt(["k","kk"],function(t,e,n){var r=O(t);e[gt]=24===r?0:r}),dt(["a","A"],function(t,e,n){n._isPm=n._locale.isPM(t),n._meridiem=t}),dt(["h","hh"],function(t,e,n){e[gt]=O(t),d(n).bigHour=!0}),dt("hmm",function(t,e,n){var r=t.length-2;e[gt]=O(t.substr(0,r)),e[wt]=O(t.substr(r)),d(n).bigHour=!0}),dt("hmmss",function(t,e,n){var r=t.length-4,i=t.length-2;e[gt]=O(t.substr(0,r)),e[wt]=O(t.substr(r,2)),e[St]=O(t.substr(i)),d(n).bigHour=!0}),dt("Hmm",function(t,e,n){var r=t.length-2;e[gt]=O(t.substr(0,r)),e[wt]=O(t.substr(r))}),dt("Hmmss",function(t,e,n){var r=t.length-4,i=t.length-2;e[gt]=O(t.substr(0,r)),e[wt]=O(t.substr(r,2)),e[St]=O(t.substr(i))});var ie,oe=jt("Hours",!0),ae={calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},longDateFormat:{LTS:"h:mm:ss A",LT:"h:mm A",L:"MM/DD/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},invalidDate:"Invalid date",ordinal:"%d",dayOfMonthOrdinalParse:/\\d{1,2}/,relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},months:Pt,monthsShort:Lt,week:{dow:0,doy:6},weekdays:Zt,weekdaysMin:Kt,weekdaysShort:Jt,meridiemParse:/[ap]\\.?m?\\.?/i},se={},ue={};function ce(t){return t?t.toLowerCase().replace("_","-"):t}function fe(e){var r=null;if(!se[e]&&void 0!==t&&t&&t.exports)try{r=ie._abbr,n(100)("./"+e),le(r)}catch(t){}return se[e]}function le(t,e){var n;return t&&((n=s(e)?pe(t):he(t,e))?ie=n:"undefined"!=typeof console&&console.warn&&console.warn("Locale "+t+" not found. Did you forget to load it?")),ie._abbr}function he(t,e){if(null!==e){var n,r=ae;if(e.abbr=t,null!=se[t])k("defineLocaleOverride","use moment.updateLocale(localeName, config) to change an existing locale. moment.defineLocale(localeName, config) should only be used for creating a new locale See http://momentjs.com/guides/#/warnings/define-locale/ for more info."),r=se[t]._config;else if(null!=e.parentLocale)if(null!=se[e.parentLocale])r=se[e.parentLocale]._config;else{if(null==(n=fe(e.parentLocale)))return ue[e.parentLocale]||(ue[e.parentLocale]=[]),ue[e.parentLocale].push({name:t,config:e}),null;r=n._config}return se[t]=new E(x(r,e)),ue[t]&&ue[t].forEach(function(t){he(t.name,t.config)}),le(t),se[t]}return delete se[t],null}function pe(t){var e;if(t&&t._locale&&t._locale._abbr&&(t=t._locale._abbr),!t)return ie;if(!o(t)){if(e=fe(t))return e;t=[t]}return function(t){for(var e,n,r,i,o=0;o<t.length;){for(e=(i=ce(t[o]).split("-")).length,n=(n=ce(t[o+1]))?n.split("-"):null;e>0;){if(r=fe(i.slice(0,e).join("-")))return r;if(n&&n.length>=e&&T(i,n,!0)>=e-1)break;e--}o++}return ie}(t)}function de(t){var e,n=t._a;return n&&-2===d(t).overflow&&(e=n[mt]<0||n[mt]>11?mt:n[_t]<1||n[_t]>Bt(n[vt],n[mt])?_t:n[gt]<0||n[gt]>24||24===n[gt]&&(0!==n[wt]||0!==n[St]||0!==n[Ot])?gt:n[wt]<0||n[wt]>59?wt:n[St]<0||n[St]>59?St:n[Ot]<0||n[Ot]>999?Ot:-1,d(t)._overflowDayOfYear&&(e<vt||e>_t)&&(e=_t),d(t)._overflowWeeks&&-1===e&&(e=Tt),d(t)._overflowWeekday&&-1===e&&(e=It),d(t).overflow=e),t}function ye(t,e,n){return null!=t?t:null!=e?e:n}function be(t){var e,n,r,o,a,s=[];if(!t._d){for(r=function(t){var e=new Date(i.now());return t._useUTC?[e.getUTCFullYear(),e.getUTCMonth(),e.getUTCDate()]:[e.getFullYear(),e.getMonth(),e.getDate()]}(t),t._w&&null==t._a[_t]&&null==t._a[mt]&&function(t){var e,n,r,i,o,a,s,u;if(null!=(e=t._w).GG||null!=e.W||null!=e.E)o=1,a=4,n=ye(e.GG,t._a[vt],Ht(xe(),1,4).year),r=ye(e.W,1),((i=ye(e.E,1))<1||i>7)&&(u=!0);else{o=t._locale._week.dow,a=t._locale._week.doy;var c=Ht(xe(),o,a);n=ye(e.gg,t._a[vt],c.year),r=ye(e.w,c.week),null!=e.d?((i=e.d)<0||i>6)&&(u=!0):null!=e.e?(i=e.e+o,(e.e<0||e.e>6)&&(u=!0)):i=o}r<1||r>Gt(n,o,a)?d(t)._overflowWeeks=!0:null!=u?d(t)._overflowWeekday=!0:(s=Wt(n,r,i,o,a),t._a[vt]=s.year,t._dayOfYear=s.dayOfYear)}(t),null!=t._dayOfYear&&(a=ye(t._a[vt],r[vt]),(t._dayOfYear>Dt(a)||0===t._dayOfYear)&&(d(t)._overflowDayOfYear=!0),n=Vt(a,0,t._dayOfYear),t._a[mt]=n.getUTCMonth(),t._a[_t]=n.getUTCDate()),e=0;e<3&&null==t._a[e];++e)t._a[e]=s[e]=r[e];for(;e<7;e++)t._a[e]=s[e]=null==t._a[e]?2===e?1:0:t._a[e];24===t._a[gt]&&0===t._a[wt]&&0===t._a[St]&&0===t._a[Ot]&&(t._nextDay=!0,t._a[gt]=0),t._d=(t._useUTC?Vt:function(t,e,n,r,i,o,a){var s;return t<100&&t>=0?(s=new Date(t+400,e,n,r,i,o,a),isFinite(s.getFullYear())&&s.setFullYear(t)):s=new Date(t,e,n,r,i,o,a),s}).apply(null,s),o=t._useUTC?t._d.getUTCDay():t._d.getDay(),null!=t._tzm&&t._d.setUTCMinutes(t._d.getUTCMinutes()-t._tzm),t._nextDay&&(t._a[gt]=24),t._w&&void 0!==t._w.d&&t._w.d!==o&&(d(t).weekdayMismatch=!0)}}var ve=/^\\s*((?:[+-]\\d{6}|\\d{4})-(?:\\d\\d-\\d\\d|W\\d\\d-\\d|W\\d\\d|\\d\\d\\d|\\d\\d))(?:(T| )(\\d\\d(?::\\d\\d(?::\\d\\d(?:[.,]\\d+)?)?)?)([\\+\\-]\\d\\d(?::?\\d\\d)?|\\s*Z)?)?$/,me=/^\\s*((?:[+-]\\d{6}|\\d{4})(?:\\d\\d\\d\\d|W\\d\\d\\d|W\\d\\d|\\d\\d\\d|\\d\\d))(?:(T| )(\\d\\d(?:\\d\\d(?:\\d\\d(?:[.,]\\d+)?)?)?)([\\+\\-]\\d\\d(?::?\\d\\d)?|\\s*Z)?)?$/,_e=/Z|[+-]\\d\\d(?::?\\d\\d)?/,ge=[["YYYYYY-MM-DD",/[+-]\\d{6}-\\d\\d-\\d\\d/],["YYYY-MM-DD",/\\d{4}-\\d\\d-\\d\\d/],["GGGG-[W]WW-E",/\\d{4}-W\\d\\d-\\d/],["GGGG-[W]WW",/\\d{4}-W\\d\\d/,!1],["YYYY-DDD",/\\d{4}-\\d{3}/],["YYYY-MM",/\\d{4}-\\d\\d/,!1],["YYYYYYMMDD",/[+-]\\d{10}/],["YYYYMMDD",/\\d{8}/],["GGGG[W]WWE",/\\d{4}W\\d{3}/],["GGGG[W]WW",/\\d{4}W\\d{2}/,!1],["YYYYDDD",/\\d{7}/]],we=[["HH:mm:ss.SSSS",/\\d\\d:\\d\\d:\\d\\d\\.\\d+/],["HH:mm:ss,SSSS",/\\d\\d:\\d\\d:\\d\\d,\\d+/],["HH:mm:ss",/\\d\\d:\\d\\d:\\d\\d/],["HH:mm",/\\d\\d:\\d\\d/],["HHmmss.SSSS",/\\d\\d\\d\\d\\d\\d\\.\\d+/],["HHmmss,SSSS",/\\d\\d\\d\\d\\d\\d,\\d+/],["HHmmss",/\\d\\d\\d\\d\\d\\d/],["HHmm",/\\d\\d\\d\\d/],["HH",/\\d\\d/]],Se=/^\\/?Date\\((\\-?\\d+)/i;function Oe(t){var e,n,r,i,o,a,s=t._i,u=ve.exec(s)||me.exec(s);if(u){for(d(t).iso=!0,e=0,n=ge.length;e<n;e++)if(ge[e][1].exec(u[1])){i=ge[e][0],r=!1!==ge[e][2];break}if(null==i)return void(t._isValid=!1);if(u[3]){for(e=0,n=we.length;e<n;e++)if(we[e][1].exec(u[3])){o=(u[2]||" ")+we[e][0];break}if(null==o)return void(t._isValid=!1)}if(!r&&null!=o)return void(t._isValid=!1);if(u[4]){if(!_e.exec(u[4]))return void(t._isValid=!1);a="Z"}t._f=i+(o||"")+(a||""),Me(t)}else t._isValid=!1}var Te=/^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\\s)?(\\d{1,2})\\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\\s(\\d{2,4})\\s(\\d\\d):(\\d\\d)(?::(\\d\\d))?\\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\\d{4}))$/;function Ie(t){var e=parseInt(t,10);return e<=49?2e3+e:e<=999?1900+e:e}var De={UT:0,GMT:0,EDT:-240,EST:-300,CDT:-300,CST:-360,MDT:-360,MST:-420,PDT:-420,PST:-480};function Ae(t){var e,n,r,i,o,a,s,u=Te.exec(t._i.replace(/\\([^)]*\\)|[\\n\\t]/g," ").replace(/(\\s\\s+)/g," ").replace(/^\\s\\s*/,"").replace(/\\s\\s*$/,""));if(u){var c=(e=u[4],n=u[3],r=u[2],i=u[5],o=u[6],a=u[7],s=[Ie(e),Lt.indexOf(n),parseInt(r,10),parseInt(i,10),parseInt(o,10)],a&&s.push(parseInt(a,10)),s);if(!function(t,e,n){if(t&&Jt.indexOf(t)!==new Date(e[0],e[1],e[2]).getDay())return d(n).weekdayMismatch=!0,n._isValid=!1,!1;return!0}(u[1],c,t))return;t._a=c,t._tzm=function(t,e,n){if(t)return De[t];if(e)return 0;var r=parseInt(n,10),i=r%100;return 60*((r-i)/100)+i}(u[8],u[9],u[10]),t._d=Vt.apply(null,t._a),t._d.setUTCMinutes(t._d.getUTCMinutes()-t._tzm),d(t).rfc2822=!0}else t._isValid=!1}function Me(t){if(t._f!==i.ISO_8601)if(t._f!==i.RFC_2822){t._a=[],d(t).empty=!0;var e,n,r,o,a,s=""+t._i,u=s.length,c=0;for(r=G(t._f,t._locale).match(N)||[],e=0;e<r.length;e++)o=r[e],(n=(s.match(lt(o,t))||[])[0])&&((a=s.substr(0,s.indexOf(n))).length>0&&d(t).unusedInput.push(a),s=s.slice(s.indexOf(n)+n.length),c+=n.length),z[o]?(n?d(t).empty=!1:d(t).unusedTokens.push(o),bt(o,n,t)):t._strict&&!n&&d(t).unusedTokens.push(o);d(t).charsLeftOver=u-c,s.length>0&&d(t).unusedInput.push(s),t._a[gt]<=12&&!0===d(t).bigHour&&t._a[gt]>0&&(d(t).bigHour=void 0),d(t).parsedDateParts=t._a.slice(0),d(t).meridiem=t._meridiem,t._a[gt]=function(t,e,n){var r;return null==n?e:null!=t.meridiemHour?t.meridiemHour(e,n):null!=t.isPM?((r=t.isPM(n))&&e<12&&(e+=12),r||12!==e||(e=0),e):e}(t._locale,t._a[gt],t._meridiem),be(t),de(t)}else Ae(t);else Oe(t)}function ke(t){var e=t._i,n=t._f;return t._locale=t._locale||pe(t._l),null===e||void 0===n&&""===e?b({nullInput:!0}):("string"==typeof e&&(t._i=e=t._locale.preparse(e)),w(e)?new g(de(e)):(c(e)?t._d=e:o(n)?function(t){var e,n,r,i,o;if(0===t._f.length)return d(t).invalidFormat=!0,void(t._d=new Date(NaN));for(i=0;i<t._f.length;i++)o=0,e=m({},t),null!=t._useUTC&&(e._useUTC=t._useUTC),e._f=t._f[i],Me(e),y(e)&&(o+=d(e).charsLeftOver,o+=10*d(e).unusedTokens.length,d(e).score=o,(null==r||o<r)&&(r=o,n=e));h(t,n||e)}(t):n?Me(t):function(t){var e=t._i;s(e)?t._d=new Date(i.now()):c(e)?t._d=new Date(e.valueOf()):"string"==typeof e?function(t){var e=Se.exec(t._i);null===e?(Oe(t),!1===t._isValid&&(delete t._isValid,Ae(t),!1===t._isValid&&(delete t._isValid,i.createFromInputFallback(t)))):t._d=new Date(+e[1])}(t):o(e)?(t._a=f(e.slice(0),function(t){return parseInt(t,10)}),be(t)):a(e)?function(t){if(!t._d){var e=L(t._i);t._a=f([e.year,e.month,e.day||e.date,e.hour,e.minute,e.second,e.millisecond],function(t){return t&&parseInt(t,10)}),be(t)}}(t):u(e)?t._d=new Date(e):i.createFromInputFallback(t)}(t),y(t)||(t._d=null),t))}function je(t,e,n,r,i){var s,u={};return!0!==n&&!1!==n||(r=n,n=void 0),(a(t)&&function(t){if(Object.getOwnPropertyNames)return 0===Object.getOwnPropertyNames(t).length;var e;for(e in t)if(t.hasOwnProperty(e))return!1;return!0}(t)||o(t)&&0===t.length)&&(t=void 0),u._isAMomentObject=!0,u._useUTC=u._isUTC=i,u._l=n,u._i=t,u._f=e,u._strict=r,(s=new g(de(ke(u))))._nextDay&&(s.add(1,"d"),s._nextDay=void 0),s}function xe(t,e,n,r){return je(t,e,n,r,!1)}i.createFromInputFallback=D("value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are discouraged and will be removed in an upcoming major release. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.",function(t){t._d=new Date(t._i+(t._useUTC?" UTC":""))}),i.ISO_8601=function(){},i.RFC_2822=function(){};var Ee=D("moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var t=xe.apply(null,arguments);return this.isValid()&&t.isValid()?t<this?this:t:b()}),Be=D("moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var t=xe.apply(null,arguments);return this.isValid()&&t.isValid()?t>this?this:t:b()});function Fe(t,e){var n,r;if(1===e.length&&o(e[0])&&(e=e[0]),!e.length)return xe();for(n=e[0],r=1;r<e.length;++r)e[r].isValid()&&!e[r][t](n)||(n=e[r]);return n}var Pe=["year","quarter","month","week","day","hour","minute","second","millisecond"];function Le(t){var e=L(t),n=e.year||0,r=e.quarter||0,i=e.month||0,o=e.week||e.isoWeek||0,a=e.day||0,s=e.hour||0,u=e.minute||0,c=e.second||0,f=e.millisecond||0;this._isValid=function(t){for(var e in t)if(-1===Mt.call(Pe,e)||null!=t[e]&&isNaN(t[e]))return!1;for(var n=!1,r=0;r<Pe.length;++r)if(t[Pe[r]]){if(n)return!1;parseFloat(t[Pe[r]])!==O(t[Pe[r]])&&(n=!0)}return!0}(e),this._milliseconds=+f+1e3*c+6e4*u+1e3*s*60*60,this._days=+a+7*o,this._months=+i+3*r+12*n,this._data={},this._locale=pe(),this._bubble()}function Ue(t){return t instanceof Le}function Ce(t){return t<0?-1*Math.round(-1*t):Math.round(t)}function Re(t,e){W(t,0,0,function(){var t=this.utcOffset(),n="+";return t<0&&(t=-t,n="-"),n+R(~~(t/60),2)+e+R(~~t%60,2)})}Re("Z",":"),Re("ZZ",""),ft("Z",st),ft("ZZ",st),dt(["Z","ZZ"],function(t,e,n){n._useUTC=!0,n._tzm=Ye(st,t)});var Ne=/([\\+\\-]|\\d\\d)/gi;function Ye(t,e){var n=(e||"").match(t);if(null===n)return null;var r=((n[n.length-1]||[])+"").match(Ne)||["-",0,0],i=60*r[1]+O(r[2]);return 0===i?0:"+"===r[0]?i:-i}function Ve(t,e){var n,r;return e._isUTC?(n=e.clone(),r=(w(t)||c(t)?t.valueOf():xe(t).valueOf())-n.valueOf(),n._d.setTime(n._d.valueOf()+r),i.updateOffset(n,!1),n):xe(t).local()}function ze(t){return 15*-Math.round(t._d.getTimezoneOffset()/15)}function We(){return!!this.isValid()&&this._isUTC&&0===this._offset}i.updateOffset=function(){};var He=/^(\\-|\\+)?(?:(\\d*)[. ])?(\\d+)\\:(\\d+)(?:\\:(\\d+)(\\.\\d*)?)?$/,Ge=/^(-|\\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;function qe(t,e){var n,r,i,o,a,s,c=t,f=null;return Ue(t)?c={ms:t._milliseconds,d:t._days,M:t._months}:u(t)?(c={},e?c[e]=t:c.milliseconds=t):(f=He.exec(t))?(n="-"===f[1]?-1:1,c={y:0,d:O(f[_t])*n,h:O(f[gt])*n,m:O(f[wt])*n,s:O(f[St])*n,ms:O(Ce(1e3*f[Ot]))*n}):(f=Ge.exec(t))?(n="-"===f[1]?-1:1,c={y:Ze(f[2],n),M:Ze(f[3],n),w:Ze(f[4],n),d:Ze(f[5],n),h:Ze(f[6],n),m:Ze(f[7],n),s:Ze(f[8],n)}):null==c?c={}:"object"==typeof c&&("from"in c||"to"in c)&&(o=xe(c.from),a=xe(c.to),i=o.isValid()&&a.isValid()?(a=Ve(a,o),o.isBefore(a)?s=Je(o,a):((s=Je(a,o)).milliseconds=-s.milliseconds,s.months=-s.months),s):{milliseconds:0,months:0},(c={}).ms=i.milliseconds,c.M=i.months),r=new Le(c),Ue(t)&&l(t,"_locale")&&(r._locale=t._locale),r}function Ze(t,e){var n=t&&parseFloat(t.replace(",","."));return(isNaN(n)?0:n)*e}function Je(t,e){var n={};return n.months=e.month()-t.month()+12*(e.year()-t.year()),t.clone().add(n.months,"M").isAfter(e)&&--n.months,n.milliseconds=+e-+t.clone().add(n.months,"M"),n}function Ke(t,e){return function(n,r){var i;return null===r||isNaN(+r)||(k(e,"moment()."+e+"(period, number) is deprecated. Please use moment()."+e+"(number, period). See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info."),i=n,n=r,r=i),$e(this,qe(n="string"==typeof n?+n:n,r),t),this}}function $e(t,e,n,r){var o=e._milliseconds,a=Ce(e._days),s=Ce(e._months);t.isValid()&&(r=null==r||r,s&&Ut(t,xt(t,"Month")+s*n),a&&Et(t,"Date",xt(t,"Date")+a*n),o&&t._d.setTime(t._d.valueOf()+o*n),r&&i.updateOffset(t,a||s))}qe.fn=Le.prototype,qe.invalid=function(){return qe(NaN)};var Qe=Ke(1,"add"),Xe=Ke(-1,"subtract");function tn(t,e){var n,r=12*(e.year()-t.year())+(e.month()-t.month()),i=t.clone().add(r,"months");return e-i<0?n=(e-i)/(i-t.clone().add(r-1,"months")):n=(e-i)/(t.clone().add(r+1,"months")-i),-(r+n)||0}function en(t){var e;return void 0===t?this._locale._abbr:(null!=(e=pe(t))&&(this._locale=e),this)}i.defaultFormat="YYYY-MM-DDTHH:mm:ssZ",i.defaultFormatUtc="YYYY-MM-DDTHH:mm:ss[Z]";var nn=D("moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.",function(t){return void 0===t?this.localeData():this.locale(t)});function rn(){return this._locale}var on=126227808e5;function an(t,e){return(t%e+e)%e}function sn(t,e,n){return t<100&&t>=0?new Date(t+400,e,n)-on:new Date(t,e,n).valueOf()}function un(t,e,n){return t<100&&t>=0?Date.UTC(t+400,e,n)-on:Date.UTC(t,e,n)}function cn(t,e){W(0,[t,t.length],0,e)}function fn(t,e,n,r,i){var o;return null==t?Ht(this,r,i).year:(e>(o=Gt(t,r,i))&&(e=o),function(t,e,n,r,i){var o=Wt(t,e,n,r,i),a=Vt(o.year,0,o.dayOfYear);return this.year(a.getUTCFullYear()),this.month(a.getUTCMonth()),this.date(a.getUTCDate()),this}.call(this,t,e,n,r,i))}W(0,["gg",2],0,function(){return this.weekYear()%100}),W(0,["GG",2],0,function(){return this.isoWeekYear()%100}),cn("gggg","weekYear"),cn("ggggg","weekYear"),cn("GGGG","isoWeekYear"),cn("GGGGG","isoWeekYear"),F("weekYear","gg"),F("isoWeekYear","GG"),C("weekYear",1),C("isoWeekYear",1),ft("G",ot),ft("g",ot),ft("GG",Q,Z),ft("gg",Q,Z),ft("GGGG",nt,K),ft("gggg",nt,K),ft("GGGGG",rt,$),ft("ggggg",rt,$),yt(["gggg","ggggg","GGGG","GGGGG"],function(t,e,n,r){e[r.substr(0,2)]=O(t)}),yt(["gg","GG"],function(t,e,n,r){e[r]=i.parseTwoDigitYear(t)}),W("Q",0,"Qo","quarter"),F("quarter","Q"),C("quarter",7),ft("Q",q),dt("Q",function(t,e){e[mt]=3*(O(t)-1)}),W("D",["DD",2],"Do","date"),F("date","D"),C("date",9),ft("D",Q),ft("DD",Q,Z),ft("Do",function(t,e){return t?e._dayOfMonthOrdinalParse||e._ordinalParse:e._dayOfMonthOrdinalParseLenient}),dt(["D","DD"],_t),dt("Do",function(t,e){e[_t]=O(t.match(Q)[0])});var ln=jt("Date",!0);W("DDD",["DDDD",3],"DDDo","dayOfYear"),F("dayOfYear","DDD"),C("dayOfYear",4),ft("DDD",et),ft("DDDD",J),dt(["DDD","DDDD"],function(t,e,n){n._dayOfYear=O(t)}),W("m",["mm",2],0,"minute"),F("minute","m"),C("minute",14),ft("m",Q),ft("mm",Q,Z),dt(["m","mm"],wt);var hn=jt("Minutes",!1);W("s",["ss",2],0,"second"),F("second","s"),C("second",15),ft("s",Q),ft("ss",Q,Z),dt(["s","ss"],St);var pn,dn=jt("Seconds",!1);for(W("S",0,0,function(){return~~(this.millisecond()/100)}),W(0,["SS",2],0,function(){return~~(this.millisecond()/10)}),W(0,["SSS",3],0,"millisecond"),W(0,["SSSS",4],0,function(){return 10*this.millisecond()}),W(0,["SSSSS",5],0,function(){return 100*this.millisecond()}),W(0,["SSSSSS",6],0,function(){return 1e3*this.millisecond()}),W(0,["SSSSSSS",7],0,function(){return 1e4*this.millisecond()}),W(0,["SSSSSSSS",8],0,function(){return 1e5*this.millisecond()}),W(0,["SSSSSSSSS",9],0,function(){return 1e6*this.millisecond()}),F("millisecond","ms"),C("millisecond",16),ft("S",et,q),ft("SS",et,Z),ft("SSS",et,J),pn="SSSS";pn.length<=9;pn+="S")ft(pn,it);function yn(t,e){e[Ot]=O(1e3*("0."+t))}for(pn="S";pn.length<=9;pn+="S")dt(pn,yn);var bn=jt("Milliseconds",!1);W("z",0,0,"zoneAbbr"),W("zz",0,0,"zoneName");var vn=g.prototype;function mn(t){return t}vn.add=Qe,vn.calendar=function(t,e){var n=t||xe(),r=Ve(n,this).startOf("day"),o=i.calendarFormat(this,r)||"sameElse",a=e&&(j(e[o])?e[o].call(this,n):e[o]);return this.format(a||this.localeData().calendar(o,this,xe(n)))},vn.clone=function(){return new g(this)},vn.diff=function(t,e,n){var r,i,o;if(!this.isValid())return NaN;if(!(r=Ve(t,this)).isValid())return NaN;switch(i=6e4*(r.utcOffset()-this.utcOffset()),e=P(e)){case"year":o=tn(this,r)/12;break;case"month":o=tn(this,r);break;case"quarter":o=tn(this,r)/3;break;case"second":o=(this-r)/1e3;break;case"minute":o=(this-r)/6e4;break;case"hour":o=(this-r)/36e5;break;case"day":o=(this-r-i)/864e5;break;case"week":o=(this-r-i)/6048e5;break;default:o=this-r}return n?o:S(o)},vn.endOf=function(t){var e;if(void 0===(t=P(t))||"millisecond"===t||!this.isValid())return this;var n=this._isUTC?un:sn;switch(t){case"year":e=n(this.year()+1,0,1)-1;break;case"quarter":e=n(this.year(),this.month()-this.month()%3+3,1)-1;break;case"month":e=n(this.year(),this.month()+1,1)-1;break;case"week":e=n(this.year(),this.month(),this.date()-this.weekday()+7)-1;break;case"isoWeek":e=n(this.year(),this.month(),this.date()-(this.isoWeekday()-1)+7)-1;break;case"day":case"date":e=n(this.year(),this.month(),this.date()+1)-1;break;case"hour":e=this._d.valueOf(),e+=36e5-an(e+(this._isUTC?0:6e4*this.utcOffset()),36e5)-1;break;case"minute":e=this._d.valueOf(),e+=6e4-an(e,6e4)-1;break;case"second":e=this._d.valueOf(),e+=1e3-an(e,1e3)-1}return this._d.setTime(e),i.updateOffset(this,!0),this},vn.format=function(t){t||(t=this.isUtc()?i.defaultFormatUtc:i.defaultFormat);var e=H(this,t);return this.localeData().postformat(e)},vn.from=function(t,e){return this.isValid()&&(w(t)&&t.isValid()||xe(t).isValid())?qe({to:this,from:t}).locale(this.locale()).humanize(!e):this.localeData().invalidDate()},vn.fromNow=function(t){return this.from(xe(),t)},vn.to=function(t,e){return this.isValid()&&(w(t)&&t.isValid()||xe(t).isValid())?qe({from:this,to:t}).locale(this.locale()).humanize(!e):this.localeData().invalidDate()},vn.toNow=function(t){return this.to(xe(),t)},vn.get=function(t){return j(this[t=P(t)])?this[t]():this},vn.invalidAt=function(){return d(this).overflow},vn.isAfter=function(t,e){var n=w(t)?t:xe(t);return!(!this.isValid()||!n.isValid())&&("millisecond"===(e=P(e)||"millisecond")?this.valueOf()>n.valueOf():n.valueOf()<this.clone().startOf(e).valueOf())},vn.isBefore=function(t,e){var n=w(t)?t:xe(t);return!(!this.isValid()||!n.isValid())&&("millisecond"===(e=P(e)||"millisecond")?this.valueOf()<n.valueOf():this.clone().endOf(e).valueOf()<n.valueOf())},vn.isBetween=function(t,e,n,r){var i=w(t)?t:xe(t),o=w(e)?e:xe(e);return!!(this.isValid()&&i.isValid()&&o.isValid())&&("("===(r=r||"()")[0]?this.isAfter(i,n):!this.isBefore(i,n))&&(")"===r[1]?this.isBefore(o,n):!this.isAfter(o,n))},vn.isSame=function(t,e){var n,r=w(t)?t:xe(t);return!(!this.isValid()||!r.isValid())&&("millisecond"===(e=P(e)||"millisecond")?this.valueOf()===r.valueOf():(n=r.valueOf(),this.clone().startOf(e).valueOf()<=n&&n<=this.clone().endOf(e).valueOf()))},vn.isSameOrAfter=function(t,e){return this.isSame(t,e)||this.isAfter(t,e)},vn.isSameOrBefore=function(t,e){return this.isSame(t,e)||this.isBefore(t,e)},vn.isValid=function(){return y(this)},vn.lang=nn,vn.locale=en,vn.localeData=rn,vn.max=Be,vn.min=Ee,vn.parsingFlags=function(){return h({},d(this))},vn.set=function(t,e){if("object"==typeof t)for(var n=function(t){var e=[];for(var n in t)e.push({unit:n,priority:U[n]});return e.sort(function(t,e){return t.priority-e.priority}),e}(t=L(t)),r=0;r<n.length;r++)this[n[r].unit](t[n[r].unit]);else if(j(this[t=P(t)]))return this[t](e);return this},vn.startOf=function(t){var e;if(void 0===(t=P(t))||"millisecond"===t||!this.isValid())return this;var n=this._isUTC?un:sn;switch(t){case"year":e=n(this.year(),0,1);break;case"quarter":e=n(this.year(),this.month()-this.month()%3,1);break;case"month":e=n(this.year(),this.month(),1);break;case"week":e=n(this.year(),this.month(),this.date()-this.weekday());break;case"isoWeek":e=n(this.year(),this.month(),this.date()-(this.isoWeekday()-1));break;case"day":case"date":e=n(this.year(),this.month(),this.date());break;case"hour":e=this._d.valueOf(),e-=an(e+(this._isUTC?0:6e4*this.utcOffset()),36e5);break;case"minute":e=this._d.valueOf(),e-=an(e,6e4);break;case"second":e=this._d.valueOf(),e-=an(e,1e3)}return this._d.setTime(e),i.updateOffset(this,!0),this},vn.subtract=Xe,vn.toArray=function(){var t=this;return[t.year(),t.month(),t.date(),t.hour(),t.minute(),t.second(),t.millisecond()]},vn.toObject=function(){var t=this;return{years:t.year(),months:t.month(),date:t.date(),hours:t.hours(),minutes:t.minutes(),seconds:t.seconds(),milliseconds:t.milliseconds()}},vn.toDate=function(){return new Date(this.valueOf())},vn.toISOString=function(t){if(!this.isValid())return null;var e=!0!==t,n=e?this.clone().utc():this;return n.year()<0||n.year()>9999?H(n,e?"YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYYYY-MM-DD[T]HH:mm:ss.SSSZ"):j(Date.prototype.toISOString)?e?this.toDate().toISOString():new Date(this.valueOf()+60*this.utcOffset()*1e3).toISOString().replace("Z",H(n,"Z")):H(n,e?"YYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYY-MM-DD[T]HH:mm:ss.SSSZ")},vn.inspect=function(){if(!this.isValid())return"moment.invalid(/* "+this._i+" */)";var t="moment",e="";this.isLocal()||(t=0===this.utcOffset()?"moment.utc":"moment.parseZone",e="Z");var n="["+t+\'("]\',r=0<=this.year()&&this.year()<=9999?"YYYY":"YYYYYY",i=e+\'[")]\';return this.format(n+r+"-MM-DD[T]HH:mm:ss.SSS"+i)},vn.toJSON=function(){return this.isValid()?this.toISOString():null},vn.toString=function(){return this.clone().locale("en").format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ")},vn.unix=function(){return Math.floor(this.valueOf()/1e3)},vn.valueOf=function(){return this._d.valueOf()-6e4*(this._offset||0)},vn.creationData=function(){return{input:this._i,format:this._f,locale:this._locale,isUTC:this._isUTC,strict:this._strict}},vn.year=kt,vn.isLeapYear=function(){return At(this.year())},vn.weekYear=function(t){return fn.call(this,t,this.week(),this.weekday(),this.localeData()._week.dow,this.localeData()._week.doy)},vn.isoWeekYear=function(t){return fn.call(this,t,this.isoWeek(),this.isoWeekday(),1,4)},vn.quarter=vn.quarters=function(t){return null==t?Math.ceil((this.month()+1)/3):this.month(3*(t-1)+this.month()%3)},vn.month=Ct,vn.daysInMonth=function(){return Bt(this.year(),this.month())},vn.week=vn.weeks=function(t){var e=this.localeData().week(this);return null==t?e:this.add(7*(t-e),"d")},vn.isoWeek=vn.isoWeeks=function(t){var e=Ht(this,1,4).week;return null==t?e:this.add(7*(t-e),"d")},vn.weeksInYear=function(){var t=this.localeData()._week;return Gt(this.year(),t.dow,t.doy)},vn.isoWeeksInYear=function(){return Gt(this.year(),1,4)},vn.date=ln,vn.day=vn.days=function(t){if(!this.isValid())return null!=t?this:NaN;var e=this._isUTC?this._d.getUTCDay():this._d.getDay();return null!=t?(t=function(t,e){return"string"!=typeof t?t:isNaN(t)?"number"==typeof(t=e.weekdaysParse(t))?t:null:parseInt(t,10)}(t,this.localeData()),this.add(t-e,"d")):e},vn.weekday=function(t){if(!this.isValid())return null!=t?this:NaN;var e=(this.day()+7-this.localeData()._week.dow)%7;return null==t?e:this.add(t-e,"d")},vn.isoWeekday=function(t){if(!this.isValid())return null!=t?this:NaN;if(null!=t){var e=function(t,e){return"string"==typeof t?e.weekdaysParse(t)%7||7:isNaN(t)?null:t}(t,this.localeData());return this.day(this.day()%7?e:e-7)}return this.day()||7},vn.dayOfYear=function(t){var e=Math.round((this.clone().startOf("day")-this.clone().startOf("year"))/864e5)+1;return null==t?e:this.add(t-e,"d")},vn.hour=vn.hours=oe,vn.minute=vn.minutes=hn,vn.second=vn.seconds=dn,vn.millisecond=vn.milliseconds=bn,vn.utcOffset=function(t,e,n){var r,o=this._offset||0;if(!this.isValid())return null!=t?this:NaN;if(null!=t){if("string"==typeof t){if(null===(t=Ye(st,t)))return this}else Math.abs(t)<16&&!n&&(t*=60);return!this._isUTC&&e&&(r=ze(this)),this._offset=t,this._isUTC=!0,null!=r&&this.add(r,"m"),o!==t&&(!e||this._changeInProgress?$e(this,qe(t-o,"m"),1,!1):this._changeInProgress||(this._changeInProgress=!0,i.updateOffset(this,!0),this._changeInProgress=null)),this}return this._isUTC?o:ze(this)},vn.utc=function(t){return this.utcOffset(0,t)},vn.local=function(t){return this._isUTC&&(this.utcOffset(0,t),this._isUTC=!1,t&&this.subtract(ze(this),"m")),this},vn.parseZone=function(){if(null!=this._tzm)this.utcOffset(this._tzm,!1,!0);else if("string"==typeof this._i){var t=Ye(at,this._i);null!=t?this.utcOffset(t):this.utcOffset(0,!0)}return this},vn.hasAlignedHourOffset=function(t){return!!this.isValid()&&(t=t?xe(t).utcOffset():0,(this.utcOffset()-t)%60==0)},vn.isDST=function(){return this.utcOffset()>this.clone().month(0).utcOffset()||this.utcOffset()>this.clone().month(5).utcOffset()},vn.isLocal=function(){return!!this.isValid()&&!this._isUTC},vn.isUtcOffset=function(){return!!this.isValid()&&this._isUTC},vn.isUtc=We,vn.isUTC=We,vn.zoneAbbr=function(){return this._isUTC?"UTC":""},vn.zoneName=function(){return this._isUTC?"Coordinated Universal Time":""},vn.dates=D("dates accessor is deprecated. Use date instead.",ln),vn.months=D("months accessor is deprecated. Use month instead",Ct),vn.years=D("years accessor is deprecated. Use year instead",kt),vn.zone=D("moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/",function(t,e){return null!=t?("string"!=typeof t&&(t=-t),this.utcOffset(t,e),this):-this.utcOffset()}),vn.isDSTShifted=D("isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information",function(){if(!s(this._isDSTShifted))return this._isDSTShifted;var t={};if(m(t,this),(t=ke(t))._a){var e=t._isUTC?p(t._a):xe(t._a);this._isDSTShifted=this.isValid()&&T(t._a,e.toArray())>0}else this._isDSTShifted=!1;return this._isDSTShifted});var _n=E.prototype;function gn(t,e,n,r){var i=pe(),o=p().set(r,e);return i[n](o,t)}function wn(t,e,n){if(u(t)&&(e=t,t=void 0),t=t||"",null!=e)return gn(t,e,n,"month");var r,i=[];for(r=0;r<12;r++)i[r]=gn(t,r,n,"month");return i}function Sn(t,e,n,r){"boolean"==typeof t?(u(e)&&(n=e,e=void 0),e=e||""):(n=e=t,t=!1,u(e)&&(n=e,e=void 0),e=e||"");var i,o=pe(),a=t?o._week.dow:0;if(null!=n)return gn(e,(n+a)%7,r,"day");var s=[];for(i=0;i<7;i++)s[i]=gn(e,(i+a)%7,r,"day");return s}_n.calendar=function(t,e,n){var r=this._calendar[t]||this._calendar.sameElse;return j(r)?r.call(e,n):r},_n.longDateFormat=function(t){var e=this._longDateFormat[t],n=this._longDateFormat[t.toUpperCase()];return e||!n?e:(this._longDateFormat[t]=n.replace(/MMMM|MM|DD|dddd/g,function(t){return t.slice(1)}),this._longDateFormat[t])},_n.invalidDate=function(){return this._invalidDate},_n.ordinal=function(t){return this._ordinal.replace("%d",t)},_n.preparse=mn,_n.postformat=mn,_n.relativeTime=function(t,e,n,r){var i=this._relativeTime[n];return j(i)?i(t,e,n,r):i.replace(/%d/i,t)},_n.pastFuture=function(t,e){var n=this._relativeTime[t>0?"future":"past"];return j(n)?n(e):n.replace(/%s/i,e)},_n.set=function(t){var e,n;for(n in t)j(e=t[n])?this[n]=e:this["_"+n]=e;this._config=t,this._dayOfMonthOrdinalParseLenient=new RegExp((this._dayOfMonthOrdinalParse.source||this._ordinalParse.source)+"|"+/\\d{1,2}/.source)},_n.months=function(t,e){return t?o(this._months)?this._months[t.month()]:this._months[(this._months.isFormat||Ft).test(e)?"format":"standalone"][t.month()]:o(this._months)?this._months:this._months.standalone},_n.monthsShort=function(t,e){return t?o(this._monthsShort)?this._monthsShort[t.month()]:this._monthsShort[Ft.test(e)?"format":"standalone"][t.month()]:o(this._monthsShort)?this._monthsShort:this._monthsShort.standalone},_n.monthsParse=function(t,e,n){var r,i,o;if(this._monthsParseExact)return function(t,e,n){var r,i,o,a=t.toLocaleLowerCase();if(!this._monthsParse)for(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[],r=0;r<12;++r)o=p([2e3,r]),this._shortMonthsParse[r]=this.monthsShort(o,"").toLocaleLowerCase(),this._longMonthsParse[r]=this.months(o,"").toLocaleLowerCase();return n?"MMM"===e?-1!==(i=Mt.call(this._shortMonthsParse,a))?i:null:-1!==(i=Mt.call(this._longMonthsParse,a))?i:null:"MMM"===e?-1!==(i=Mt.call(this._shortMonthsParse,a))?i:-1!==(i=Mt.call(this._longMonthsParse,a))?i:null:-1!==(i=Mt.call(this._longMonthsParse,a))?i:-1!==(i=Mt.call(this._shortMonthsParse,a))?i:null}.call(this,t,e,n);for(this._monthsParse||(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[]),r=0;r<12;r++){if(i=p([2e3,r]),n&&!this._longMonthsParse[r]&&(this._longMonthsParse[r]=new RegExp("^"+this.months(i,"").replace(".","")+"$","i"),this._shortMonthsParse[r]=new RegExp("^"+this.monthsShort(i,"").replace(".","")+"$","i")),n||this._monthsParse[r]||(o="^"+this.months(i,"")+"|^"+this.monthsShort(i,""),this._monthsParse[r]=new RegExp(o.replace(".",""),"i")),n&&"MMMM"===e&&this._longMonthsParse[r].test(t))return r;if(n&&"MMM"===e&&this._shortMonthsParse[r].test(t))return r;if(!n&&this._monthsParse[r].test(t))return r}},_n.monthsRegex=function(t){return this._monthsParseExact?(l(this,"_monthsRegex")||Yt.call(this),t?this._monthsStrictRegex:this._monthsRegex):(l(this,"_monthsRegex")||(this._monthsRegex=Nt),this._monthsStrictRegex&&t?this._monthsStrictRegex:this._monthsRegex)},_n.monthsShortRegex=function(t){return this._monthsParseExact?(l(this,"_monthsRegex")||Yt.call(this),t?this._monthsShortStrictRegex:this._monthsShortRegex):(l(this,"_monthsShortRegex")||(this._monthsShortRegex=Rt),this._monthsShortStrictRegex&&t?this._monthsShortStrictRegex:this._monthsShortRegex)},_n.week=function(t){return Ht(t,this._week.dow,this._week.doy).week},_n.firstDayOfYear=function(){return this._week.doy},_n.firstDayOfWeek=function(){return this._week.dow},_n.weekdays=function(t,e){var n=o(this._weekdays)?this._weekdays:this._weekdays[t&&!0!==t&&this._weekdays.isFormat.test(e)?"format":"standalone"];return!0===t?qt(n,this._week.dow):t?n[t.day()]:n},_n.weekdaysMin=function(t){return!0===t?qt(this._weekdaysMin,this._week.dow):t?this._weekdaysMin[t.day()]:this._weekdaysMin},_n.weekdaysShort=function(t){return!0===t?qt(this._weekdaysShort,this._week.dow):t?this._weekdaysShort[t.day()]:this._weekdaysShort},_n.weekdaysParse=function(t,e,n){var r,i,o;if(this._weekdaysParseExact)return function(t,e,n){var r,i,o,a=t.toLocaleLowerCase();if(!this._weekdaysParse)for(this._weekdaysParse=[],this._shortWeekdaysParse=[],this._minWeekdaysParse=[],r=0;r<7;++r)o=p([2e3,1]).day(r),this._minWeekdaysParse[r]=this.weekdaysMin(o,"").toLocaleLowerCase(),this._shortWeekdaysParse[r]=this.weekdaysShort(o,"").toLocaleLowerCase(),this._weekdaysParse[r]=this.weekdays(o,"").toLocaleLowerCase();return n?"dddd"===e?-1!==(i=Mt.call(this._weekdaysParse,a))?i:null:"ddd"===e?-1!==(i=Mt.call(this._shortWeekdaysParse,a))?i:null:-1!==(i=Mt.call(this._minWeekdaysParse,a))?i:null:"dddd"===e?-1!==(i=Mt.call(this._weekdaysParse,a))?i:-1!==(i=Mt.call(this._shortWeekdaysParse,a))?i:-1!==(i=Mt.call(this._minWeekdaysParse,a))?i:null:"ddd"===e?-1!==(i=Mt.call(this._shortWeekdaysParse,a))?i:-1!==(i=Mt.call(this._weekdaysParse,a))?i:-1!==(i=Mt.call(this._minWeekdaysParse,a))?i:null:-1!==(i=Mt.call(this._minWeekdaysParse,a))?i:-1!==(i=Mt.call(this._weekdaysParse,a))?i:-1!==(i=Mt.call(this._shortWeekdaysParse,a))?i:null}.call(this,t,e,n);for(this._weekdaysParse||(this._weekdaysParse=[],this._minWeekdaysParse=[],this._shortWeekdaysParse=[],this._fullWeekdaysParse=[]),r=0;r<7;r++){if(i=p([2e3,1]).day(r),n&&!this._fullWeekdaysParse[r]&&(this._fullWeekdaysParse[r]=new RegExp("^"+this.weekdays(i,"").replace(".","\\\\.?")+"$","i"),this._shortWeekdaysParse[r]=new RegExp("^"+this.weekdaysShort(i,"").replace(".","\\\\.?")+"$","i"),this._minWeekdaysParse[r]=new RegExp("^"+this.weekdaysMin(i,"").replace(".","\\\\.?")+"$","i")),this._weekdaysParse[r]||(o="^"+this.weekdays(i,"")+"|^"+this.weekdaysShort(i,"")+"|^"+this.weekdaysMin(i,""),this._weekdaysParse[r]=new RegExp(o.replace(".",""),"i")),n&&"dddd"===e&&this._fullWeekdaysParse[r].test(t))return r;if(n&&"ddd"===e&&this._shortWeekdaysParse[r].test(t))return r;if(n&&"dd"===e&&this._minWeekdaysParse[r].test(t))return r;if(!n&&this._weekdaysParse[r].test(t))return r}},_n.weekdaysRegex=function(t){return this._weekdaysParseExact?(l(this,"_weekdaysRegex")||te.call(this),t?this._weekdaysStrictRegex:this._weekdaysRegex):(l(this,"_weekdaysRegex")||(this._weekdaysRegex=$t),this._weekdaysStrictRegex&&t?this._weekdaysStrictRegex:this._weekdaysRegex)},_n.weekdaysShortRegex=function(t){return this._weekdaysParseExact?(l(this,"_weekdaysRegex")||te.call(this),t?this._weekdaysShortStrictRegex:this._weekdaysShortRegex):(l(this,"_weekdaysShortRegex")||(this._weekdaysShortRegex=Qt),this._weekdaysShortStrictRegex&&t?this._weekdaysShortStrictRegex:this._weekdaysShortRegex)},_n.weekdaysMinRegex=function(t){return this._weekdaysParseExact?(l(this,"_weekdaysRegex")||te.call(this),t?this._weekdaysMinStrictRegex:this._weekdaysMinRegex):(l(this,"_weekdaysMinRegex")||(this._weekdaysMinRegex=Xt),this._weekdaysMinStrictRegex&&t?this._weekdaysMinStrictRegex:this._weekdaysMinRegex)},_n.isPM=function(t){return"p"===(t+"").toLowerCase().charAt(0)},_n.meridiem=function(t,e,n){return t>11?n?"pm":"PM":n?"am":"AM"},le("en",{dayOfMonthOrdinalParse:/\\d{1,2}(th|st|nd|rd)/,ordinal:function(t){var e=t%10;return t+(1===O(t%100/10)?"th":1===e?"st":2===e?"nd":3===e?"rd":"th")}}),i.lang=D("moment.lang is deprecated. Use moment.locale instead.",le),i.langData=D("moment.langData is deprecated. Use moment.localeData instead.",pe);var On=Math.abs;function Tn(t,e,n,r){var i=qe(e,n);return t._milliseconds+=r*i._milliseconds,t._days+=r*i._days,t._months+=r*i._months,t._bubble()}function In(t){return t<0?Math.floor(t):Math.ceil(t)}function Dn(t){return 4800*t/146097}function An(t){return 146097*t/4800}function Mn(t){return function(){return this.as(t)}}var kn=Mn("ms"),jn=Mn("s"),xn=Mn("m"),En=Mn("h"),Bn=Mn("d"),Fn=Mn("w"),Pn=Mn("M"),Ln=Mn("Q"),Un=Mn("y");function Cn(t){return function(){return this.isValid()?this._data[t]:NaN}}var Rn=Cn("milliseconds"),Nn=Cn("seconds"),Yn=Cn("minutes"),Vn=Cn("hours"),zn=Cn("days"),Wn=Cn("months"),Hn=Cn("years"),Gn=Math.round,qn={ss:44,s:45,m:45,h:22,d:26,M:11},Zn=Math.abs;function Jn(t){return(t>0)-(t<0)||+t}function Kn(){if(!this.isValid())return this.localeData().invalidDate();var t,e,n=Zn(this._milliseconds)/1e3,r=Zn(this._days),i=Zn(this._months);t=S(n/60),e=S(t/60),n%=60,t%=60;var o=S(i/12),a=i%=12,s=r,u=e,c=t,f=n?n.toFixed(3).replace(/\\.?0+$/,""):"",l=this.asSeconds();if(!l)return"P0D";var h=l<0?"-":"",p=Jn(this._months)!==Jn(l)?"-":"",d=Jn(this._days)!==Jn(l)?"-":"",y=Jn(this._milliseconds)!==Jn(l)?"-":"";return h+"P"+(o?p+o+"Y":"")+(a?p+a+"M":"")+(s?d+s+"D":"")+(u||c||f?"T":"")+(u?y+u+"H":"")+(c?y+c+"M":"")+(f?y+f+"S":"")}var $n=Le.prototype;return $n.isValid=function(){return this._isValid},$n.abs=function(){var t=this._data;return this._milliseconds=On(this._milliseconds),this._days=On(this._days),this._months=On(this._months),t.milliseconds=On(t.milliseconds),t.seconds=On(t.seconds),t.minutes=On(t.minutes),t.hours=On(t.hours),t.months=On(t.months),t.years=On(t.years),this},$n.add=function(t,e){return Tn(this,t,e,1)},$n.subtract=function(t,e){return Tn(this,t,e,-1)},$n.as=function(t){if(!this.isValid())return NaN;var e,n,r=this._milliseconds;if("month"===(t=P(t))||"quarter"===t||"year"===t)switch(e=this._days+r/864e5,n=this._months+Dn(e),t){case"month":return n;case"quarter":return n/3;case"year":return n/12}else switch(e=this._days+Math.round(An(this._months)),t){case"week":return e/7+r/6048e5;case"day":return e+r/864e5;case"hour":return 24*e+r/36e5;case"minute":return 1440*e+r/6e4;case"second":return 86400*e+r/1e3;case"millisecond":return Math.floor(864e5*e)+r;default:throw new Error("Unknown unit "+t)}},$n.asMilliseconds=kn,$n.asSeconds=jn,$n.asMinutes=xn,$n.asHours=En,$n.asDays=Bn,$n.asWeeks=Fn,$n.asMonths=Pn,$n.asQuarters=Ln,$n.asYears=Un,$n.valueOf=function(){return this.isValid()?this._milliseconds+864e5*this._days+this._months%12*2592e6+31536e6*O(this._months/12):NaN},$n._bubble=function(){var t,e,n,r,i,o=this._milliseconds,a=this._days,s=this._months,u=this._data;return o>=0&&a>=0&&s>=0||o<=0&&a<=0&&s<=0||(o+=864e5*In(An(s)+a),a=0,s=0),u.milliseconds=o%1e3,t=S(o/1e3),u.seconds=t%60,e=S(t/60),u.minutes=e%60,n=S(e/60),u.hours=n%24,a+=S(n/24),s+=i=S(Dn(a)),a-=In(An(i)),r=S(s/12),s%=12,u.days=a,u.months=s,u.years=r,this},$n.clone=function(){return qe(this)},$n.get=function(t){return t=P(t),this.isValid()?this[t+"s"]():NaN},$n.milliseconds=Rn,$n.seconds=Nn,$n.minutes=Yn,$n.hours=Vn,$n.days=zn,$n.weeks=function(){return S(this.days()/7)},$n.months=Wn,$n.years=Hn,$n.humanize=function(t){if(!this.isValid())return this.localeData().invalidDate();var e=this.localeData(),n=function(t,e,n){var r=qe(t).abs(),i=Gn(r.as("s")),o=Gn(r.as("m")),a=Gn(r.as("h")),s=Gn(r.as("d")),u=Gn(r.as("M")),c=Gn(r.as("y")),f=i<=qn.ss&&["s",i]||i<qn.s&&["ss",i]||o<=1&&["m"]||o<qn.m&&["mm",o]||a<=1&&["h"]||a<qn.h&&["hh",a]||s<=1&&["d"]||s<qn.d&&["dd",s]||u<=1&&["M"]||u<qn.M&&["MM",u]||c<=1&&["y"]||["yy",c];return f[2]=e,f[3]=+t>0,f[4]=n,function(t,e,n,r,i){return i.relativeTime(e||1,!!n,t,r)}.apply(null,f)}(this,!t,e);return t&&(n=e.pastFuture(+this,n)),e.postformat(n)},$n.toISOString=Kn,$n.toString=Kn,$n.toJSON=Kn,$n.locale=en,$n.localeData=rn,$n.toIsoString=D("toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)",Kn),$n.lang=nn,W("X",0,0,"unix"),W("x",0,0,"valueOf"),ft("x",ot),ft("X",/[+-]?\\d+(\\.\\d{1,3})?/),dt("X",function(t,e,n){n._d=new Date(1e3*parseFloat(t,10))}),dt("x",function(t,e,n){n._d=new Date(O(t))}),i.version="2.24.0",e=xe,i.fn=vn,i.min=function(){return Fe("isBefore",[].slice.call(arguments,0))},i.max=function(){return Fe("isAfter",[].slice.call(arguments,0))},i.now=function(){return Date.now?Date.now():+new Date},i.utc=p,i.unix=function(t){return xe(1e3*t)},i.months=function(t,e){return wn(t,e,"months")},i.isDate=c,i.locale=le,i.invalid=b,i.duration=qe,i.isMoment=w,i.weekdays=function(t,e,n){return Sn(t,e,n,"weekdays")},i.parseZone=function(){return xe.apply(null,arguments).parseZone()},i.localeData=pe,i.isDuration=Ue,i.monthsShort=function(t,e){return wn(t,e,"monthsShort")},i.weekdaysMin=function(t,e,n){return Sn(t,e,n,"weekdaysMin")},i.defineLocale=he,i.updateLocale=function(t,e){if(null!=e){var n,r,i=ae;null!=(r=fe(t))&&(i=r._config),(n=new E(e=x(i,e))).parentLocale=se[t],se[t]=n,le(t)}else null!=se[t]&&(null!=se[t].parentLocale?se[t]=se[t].parentLocale:null!=se[t]&&delete se[t]);return se[t]},i.locales=function(){return A(se)},i.weekdaysShort=function(t,e,n){return Sn(t,e,n,"weekdaysShort")},i.normalizeUnits=P,i.relativeTimeRounding=function(t){return void 0===t?Gn:"function"==typeof t&&(Gn=t,!0)},i.relativeTimeThreshold=function(t,e){return void 0!==qn[t]&&(void 0===e?qn[t]:(qn[t]=e,"s"===t&&(qn.ss=e-1),!0))},i.calendarFormat=function(t,e){var n=t.diff(e,"days",!0);return n<-6?"sameElse":n<-1?"lastWeek":n<0?"lastDay":n<1?"sameDay":n<2?"nextDay":n<7?"nextWeek":"sameElse"},i.prototype=vn,i.HTML5_FMT={DATETIME_LOCAL:"YYYY-MM-DDTHH:mm",DATETIME_LOCAL_SECONDS:"YYYY-MM-DDTHH:mm:ss",DATETIME_LOCAL_MS:"YYYY-MM-DDTHH:mm:ss.SSS",DATE:"YYYY-MM-DD",TIME:"HH:mm",TIME_SECONDS:"HH:mm:ss",TIME_MS:"HH:mm:ss.SSS",WEEK:"GGGG-[W]WW",MONTH:"YYYY-MM"},i}()}).call(this,n(64)(t))},function(t,e){var n=t.exports="undefined"!=typeof window&&window.Math==Math?window:"undefined"!=typeof self&&self.Math==Math?self:Function("return this")();"number"==typeof __g&&(__g=n)},function(t,e,n){var r=n(46),i=n(60);t.exports=n(29)?function(t,e,n){return r.f(t,e,i(1,n))}:function(t,e,n){return t[e]=n,t}},function(t,e,n){"use strict";function r(t){return t>=1&&t<=4?"integer":19===t?"string":10===t||9===t?"float":11===t?"boolean":12===t?"datetime":13===t?"date":void console.warn("Unknown type for value ".concat(t," with JS type ").concat(typeof t))}function i(t){let e=t;do{for(const n of Object.getOwnPropertyNames(e)){const e=t[n];"constructor"!==n&&"function"==typeof e&&(t[n]=e.bind(t))}}while(e=e!==Object&&Object.getPrototypeOf(e))}n.d(e,"b",function(){return r}),n.d(e,"a",function(){return i}),n(103),n(104),String.prototype.includes||(String.prototype.includes=function(t,e){return"number"!=typeof e&&(e=0),!(e+t.length>this.length)&&-1!==this.indexOf(t,e)}),Array.prototype.includes||Object.defineProperty(Array.prototype,"includes",{value:function(t,e){if(null==this)throw new TypeError(\'"this" is null or not defined\');var n=Object(this),r=n.length>>>0;if(0===r)return!1;for(var i,o,a=0|e,s=Math.max(a>=0?a:r-Math.abs(a),0);s<r;){if((i=n[s])===(o=t)||"number"==typeof i&&"number"==typeof o&&isNaN(i)&&isNaN(o))return!0;s++}return!1}})},function(t,e,n){"use strict";n.d(e,"a",function(){return r});var r,i=n(5);!function(t){var e,n;e=t.apache||(t.apache={}),((n=e.arrow||(e.arrow={})).flatbuf||(n.flatbuf={})).Schema=i.a.apache.arrow.flatbuf.Schema}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),(i=r.MessageHeader||(r.MessageHeader={}))[i.NONE=0]="NONE",i[i.Schema=1]="Schema",i[i.DictionaryBatch=2]="DictionaryBatch",i[i.RecordBatch=3]="RecordBatch",i[i.Tensor=4]="Tensor"}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.prototype.length=function(){return this.bb.readInt64(this.bb_pos)},t.prototype.nullCount=function(){return this.bb.readInt64(this.bb_pos+8)},t.createFieldNode=function(t,e,n){return t.prep(8,16),t.writeInt64(n),t.writeInt64(e),t.offset()},t}(),r.FieldNode=i}(r||(r={})),function(t){var e,n,r,o;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),o=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsRecordBatch=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.length=function(){var t=this.bb.__offset(this.bb_pos,4);return t?this.bb.readInt64(this.bb_pos+t):this.bb.createLong(0,0)},e.prototype.nodes=function(e,n){var r=this.bb.__offset(this.bb_pos,6);return r?(n||new t.apache.arrow.flatbuf.FieldNode).__init(this.bb.__vector(this.bb_pos+r)+16*e,this.bb):null},e.prototype.nodesLength=function(){var t=this.bb.__offset(this.bb_pos,6);return t?this.bb.__vector_len(this.bb_pos+t):0},e.prototype.buffers=function(t,e){var n=this.bb.__offset(this.bb_pos,8);return n?(e||new i.a.apache.arrow.flatbuf.Buffer).__init(this.bb.__vector(this.bb_pos+n)+16*t,this.bb):null},e.prototype.buffersLength=function(){var t=this.bb.__offset(this.bb_pos,8);return t?this.bb.__vector_len(this.bb_pos+t):0},e.startRecordBatch=function(t){t.startObject(3)},e.addLength=function(t,e){t.addFieldInt64(0,e,t.createLong(0,0))},e.addNodes=function(t,e){t.addFieldOffset(1,e,0)},e.startNodesVector=function(t,e){t.startVector(16,e,8)},e.addBuffers=function(t,e){t.addFieldOffset(2,e,0)},e.startBuffersVector=function(t,e){t.startVector(16,e,8)},e.endRecordBatch=function(t){return t.endObject()},e}(),r.RecordBatch=o}(r||(r={})),function(t){var e,n,r,i;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),i=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsDictionaryBatch=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.id=function(){var t=this.bb.__offset(this.bb_pos,4);return t?this.bb.readInt64(this.bb_pos+t):this.bb.createLong(0,0)},e.prototype.data=function(e){var n=this.bb.__offset(this.bb_pos,6);return n?(e||new t.apache.arrow.flatbuf.RecordBatch).__init(this.bb.__indirect(this.bb_pos+n),this.bb):null},e.prototype.isDelta=function(){var t=this.bb.__offset(this.bb_pos,8);return!!t&&!!this.bb.readInt8(this.bb_pos+t)},e.startDictionaryBatch=function(t){t.startObject(3)},e.addId=function(t,e){t.addFieldInt64(0,e,t.createLong(0,0))},e.addData=function(t,e){t.addFieldOffset(1,e,0)},e.addIsDelta=function(t,e){t.addFieldInt8(2,+e,0)},e.endDictionaryBatch=function(t){return t.endObject()},e}(),r.DictionaryBatch=i}(r||(r={})),function(t){var e,n,r,o;e=t.apache||(t.apache={}),n=e.arrow||(e.arrow={}),r=n.flatbuf||(n.flatbuf={}),o=function(){function e(){this.bb_pos=0}return e.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},e.getRootAsMessage=function(t,n){return(n||new e).__init(t.readInt32(t.position())+t.position(),t)},e.prototype.version=function(){var t=this.bb.__offset(this.bb_pos,4);return t?this.bb.readInt16(this.bb_pos+t):i.a.apache.arrow.flatbuf.MetadataVersion.V1},e.prototype.headerType=function(){var e=this.bb.__offset(this.bb_pos,6);return e?this.bb.readUint8(this.bb_pos+e):t.apache.arrow.flatbuf.MessageHeader.NONE},e.prototype.header=function(t){var e=this.bb.__offset(this.bb_pos,8);return e?this.bb.__union(t,this.bb_pos+e):null},e.prototype.bodyLength=function(){var t=this.bb.__offset(this.bb_pos,10);return t?this.bb.readInt64(this.bb_pos+t):this.bb.createLong(0,0)},e.startMessage=function(t){t.startObject(4)},e.addVersion=function(t,e){t.addFieldInt16(0,e,i.a.apache.arrow.flatbuf.MetadataVersion.V1)},e.addHeaderType=function(e,n){e.addFieldInt8(1,n,t.apache.arrow.flatbuf.MessageHeader.NONE)},e.addHeader=function(t,e){t.addFieldOffset(2,e,0)},e.addBodyLength=function(t,e){t.addFieldInt64(3,e,t.createLong(0,0))},e.endMessage=function(t){return t.endObject()},e.finishMessageBuffer=function(t,e){t.finish(e)},e}(),r.Message=o}(r||(r={}))},function(t,e){t.exports=function(t){try{return!!t()}catch(t){return!0}}},function(t,e,n){"use strict";n.d(e,"b",function(){return a}),n.d(e,"a",function(){return s});var r=n(17),i=n.n(r);const o=[i.a.ISO_8601,i.a.RFC_2822,"YYYY-MM-DD\\\\DHH:mm:ss.SSSS","MM-DD-YYYY","MM/DD/YYYY","M/D/YYYY","M/D/YY","DD MMM YYYY","HH:mm:ss.SSS"];function a(t){return i()(t,o,!0).isValid()}class s{constructor(){this.date_types=[],this.date_candidates=o.slice(),this.date_exclusions=[]}parse(t){if(this.date_exclusions.indexOf(t)>-1)return null;{let e=t;const n=typeof e;if(e.getMonth)return e;if("string"===n){if(!(e=i()(t,this.date_types,!0)).isValid()||0===this.date_types.length){for(let n of this.date_candidates)if((e=i()(t,n,!0)).isValid())return this.date_types.push(n),this.date_candidates.splice(this.date_candidates.indexOf(n),1),e.toDate();return this.date_exclusions.push(t),null}return e.toDate()}if("number"===n)return new Date(e);throw new Error("Unparseable date ".concat(e))}}}},function(t,e,n){var r,i;\n/* @license\nPapa Parse\nv4.6.3\nhttps://github.com/mholt/PapaParse\nLicense: MIT\n*/Array.isArray||(Array.isArray=function(t){return"[object Array]"===Object.prototype.toString.call(t)}),void 0===(i="function"==typeof(r=function(){"use strict";var t,e,n="undefined"!=typeof self?self:"undefined"!=typeof window?window:void 0!==n?n:{},r=!n.document&&!!n.postMessage,i=r&&/(\\?|&)papaworker(=|&|$)/.test(n.location.search),o=!1,a={},s=0,u={parse:function(e,r){var i=(r=r||{}).dynamicTyping||!1;if(O(i)&&(r.dynamicTypingFunction=i,i={}),r.dynamicTyping=i,r.transform=!!O(r.transform)&&r.transform,r.worker&&u.WORKERS_SUPPORTED){var c=function(){if(!u.WORKERS_SUPPORTED)return!1;if(!o&&null===u.SCRIPT_PATH)throw new Error("Script path cannot be determined automatically when Papa Parse is loaded asynchronously. You need to set Papa.SCRIPT_PATH manually.");var e=u.SCRIPT_PATH||t;e+=(-1!==e.indexOf("?")?"&":"?")+"papaworker";var r=new n.Worker(e);return r.onmessage=m,r.id=s++,a[r.id]=r}();return c.userStep=r.step,c.userChunk=r.chunk,c.userComplete=r.complete,c.userError=r.error,r.step=O(r.step),r.chunk=O(r.chunk),r.complete=O(r.complete),r.error=O(r.error),delete r.worker,void c.postMessage({input:e,config:r,workerId:c.id})}var f=null;return u.NODE_STREAM_INPUT,"string"==typeof e?f=r.download?new l(r):new p(r):!0===e.readable&&O(e.read)&&O(e.on)?f=new d(r):(n.File&&e instanceof File||e instanceof Object)&&(f=new h(r)),f.stream(e)},unparse:function(t,e){var n=!1,r=!0,i=",",o="\\r\\n",a=\'"\',s=!1;"object"==typeof e&&("string"!=typeof e.delimiter||u.BAD_DELIMITERS.filter(function(t){return-1!==e.delimiter.indexOf(t)}).length||(i=e.delimiter),("boolean"==typeof e.quotes||Array.isArray(e.quotes))&&(n=e.quotes),"boolean"!=typeof e.skipEmptyLines&&"string"!=typeof e.skipEmptyLines||(s=e.skipEmptyLines),"string"==typeof e.newline&&(o=e.newline),"string"==typeof e.quoteChar&&(a=e.quoteChar),"boolean"==typeof e.header&&(r=e.header));var c=new RegExp(b(a),"g");if("string"==typeof t&&(t=JSON.parse(t)),Array.isArray(t)){if(!t.length||Array.isArray(t[0]))return l(null,t,s);if("object"==typeof t[0])return l(f(t[0]),t,s)}else if("object"==typeof t)return"string"==typeof t.data&&(t.data=JSON.parse(t.data)),Array.isArray(t.data)&&(t.fields||(t.fields=t.meta&&t.meta.fields),t.fields||(t.fields=Array.isArray(t.data[0])?t.fields:f(t.data[0])),Array.isArray(t.data[0])||"object"==typeof t.data[0]||(t.data=[t.data])),l(t.fields||[],t.data||[],s);throw"exception: Unable to serialize unrecognized input";function f(t){if("object"!=typeof t)return[];var e=[];for(var n in t)e.push(n);return e}function l(t,e,n){var a="";"string"==typeof t&&(t=JSON.parse(t)),"string"==typeof e&&(e=JSON.parse(e));var s=Array.isArray(t)&&0<t.length,u=!Array.isArray(e[0]);if(s&&r){for(var c=0;c<t.length;c++)0<c&&(a+=i),a+=h(t[c],c);0<e.length&&(a+=o)}for(var f=0;f<e.length;f++){var l=s?t.length:e[f].length,p=!1,d=s?0===Object.keys(e[f]).length:0===e[f].length;if(n&&!s&&(p="greedy"===n?""===e[f].join("").trim():1===e[f].length&&0===e[f][0].length),"greedy"===n&&s){for(var y=[],b=0;b<l;b++){var v=u?t[b]:b;y.push(e[f][v])}p=""===y.join("").trim()}if(!p){for(var m=0;m<l;m++){0<m&&!d&&(a+=i);var _=s&&u?t[m]:m;a+=h(e[f][_],m)}f<e.length-1&&(!n||0<l&&!d)&&(a+=o)}}return a}function h(t,e){return null==t?"":t.constructor===Date?JSON.stringify(t).slice(1,25):(t=t.toString().replace(c,a+a),"boolean"==typeof n&&n||Array.isArray(n)&&n[e]||function(t,e){for(var n=0;n<e.length;n++)if(-1<t.indexOf(e[n]))return!0;return!1}(t,u.BAD_DELIMITERS)||-1<t.indexOf(i)||" "===t.charAt(0)||" "===t.charAt(t.length-1)?a+t+a:t)}}};if(u.RECORD_SEP=String.fromCharCode(30),u.UNIT_SEP=String.fromCharCode(31),u.BYTE_ORDER_MARK="\\ufeff",u.BAD_DELIMITERS=["\\r","\\n",\'"\',u.BYTE_ORDER_MARK],u.WORKERS_SUPPORTED=!r&&!!n.Worker,u.SCRIPT_PATH=null,u.NODE_STREAM_INPUT=1,u.LocalChunkSize=10485760,u.RemoteChunkSize=5242880,u.DefaultDelimiter=",",u.Parser=v,u.ParserHandle=y,u.NetworkStreamer=l,u.FileStreamer=h,u.StringStreamer=p,u.ReadableStreamStreamer=d,n.jQuery){var c=n.jQuery;c.fn.parse=function(t){var e=t.config||{},r=[];return this.each(function(t){if("INPUT"!==c(this).prop("tagName").toUpperCase()||"file"!==c(this).attr("type").toLowerCase()||!n.FileReader||!this.files||0===this.files.length)return!0;for(var i=0;i<this.files.length;i++)r.push({file:this.files[i],inputElem:this,instanceConfig:c.extend({},e)})}),i(),this;function i(){if(0!==r.length){var e,n,i,a=r[0];if(O(t.before)){var s=t.before(a.file,a.inputElem);if("object"==typeof s){if("abort"===s.action)return e=a.file,n=a.inputElem,i=s.reason,void(O(t.error)&&t.error({name:"AbortError"},e,n,i));if("skip"===s.action)return void o();"object"==typeof s.config&&(a.instanceConfig=c.extend(a.instanceConfig,s.config))}else if("skip"===s)return void o()}var f=a.instanceConfig.complete;a.instanceConfig.complete=function(t){O(f)&&f(t,a.file,a.inputElem),o()},u.parse(a.file,a.instanceConfig)}else O(t.complete)&&t.complete()}function o(){r.splice(0,1),i()}}}function f(t){this._handle=null,this._finished=!1,this._completed=!1,this._input=null,this._baseIndex=0,this._partialLine="",this._rowCount=0,this._start=0,this._nextChunk=null,this.isFirstChunk=!0,this._completeResults={data:[],errors:[],meta:{}},function(t){var e=w(t);e.chunkSize=parseInt(e.chunkSize),t.step||t.chunk||(e.chunkSize=null),this._handle=new y(e),(this._handle.streamer=this)._config=e}.call(this,t),this.parseChunk=function(t,e){if(this.isFirstChunk&&O(this._config.beforeFirstChunk)){var r=this._config.beforeFirstChunk(t);void 0!==r&&(t=r)}this.isFirstChunk=!1;var o=this._partialLine+t;this._partialLine="";var a=this._handle.parse(o,this._baseIndex,!this._finished);if(!this._handle.paused()&&!this._handle.aborted()){var s=a.meta.cursor;this._finished||(this._partialLine=o.substring(s-this._baseIndex),this._baseIndex=s),a&&a.data&&(this._rowCount+=a.data.length);var c=this._finished||this._config.preview&&this._rowCount>=this._config.preview;if(i)n.postMessage({results:a,workerId:u.WORKER_ID,finished:c});else if(O(this._config.chunk)&&!e){if(this._config.chunk(a,this._handle),this._handle.paused()||this._handle.aborted())return;a=void 0,this._completeResults=void 0}return this._config.step||this._config.chunk||(this._completeResults.data=this._completeResults.data.concat(a.data),this._completeResults.errors=this._completeResults.errors.concat(a.errors),this._completeResults.meta=a.meta),this._completed||!c||!O(this._config.complete)||a&&a.meta.aborted||(this._config.complete(this._completeResults,this._input),this._completed=!0),c||a&&a.meta.paused||this._nextChunk(),a}},this._sendError=function(t){O(this._config.error)?this._config.error(t):i&&this._config.error&&n.postMessage({workerId:u.WORKER_ID,error:t,finished:!1})}}function l(t){var e;(t=t||{}).chunkSize||(t.chunkSize=u.RemoteChunkSize),f.call(this,t),this._nextChunk=r?function(){this._readChunk(),this._chunkLoaded()}:function(){this._readChunk()},this.stream=function(t){this._input=t,this._nextChunk()},this._readChunk=function(){if(this._finished)this._chunkLoaded();else{if(e=new XMLHttpRequest,this._config.withCredentials&&(e.withCredentials=this._config.withCredentials),r||(e.onload=S(this._chunkLoaded,this),e.onerror=S(this._chunkError,this)),e.open("GET",this._input,!r),this._config.downloadRequestHeaders){var t=this._config.downloadRequestHeaders;for(var n in t)e.setRequestHeader(n,t[n])}if(this._config.chunkSize){var i=this._start+this._config.chunkSize-1;e.setRequestHeader("Range","bytes="+this._start+"-"+i),e.setRequestHeader("If-None-Match","webkit-no-cache")}try{e.send()}catch(t){this._chunkError(t.message)}r&&0===e.status?this._chunkError():this._start+=this._config.chunkSize}},this._chunkLoaded=function(){var t;4===e.readyState&&(e.status<200||400<=e.status?this._chunkError():(this._finished=!this._config.chunkSize||this._start>(null===(t=e.getResponseHeader("Content-Range"))?-1:parseInt(t.substr(t.lastIndexOf("/")+1))),this.parseChunk(e.responseText)))},this._chunkError=function(t){var n=e.statusText||t;this._sendError(new Error(n))}}function h(t){var e,n;(t=t||{}).chunkSize||(t.chunkSize=u.LocalChunkSize),f.call(this,t);var r="undefined"!=typeof FileReader;this.stream=function(t){this._input=t,n=t.slice||t.webkitSlice||t.mozSlice,r?((e=new FileReader).onload=S(this._chunkLoaded,this),e.onerror=S(this._chunkError,this)):e=new FileReaderSync,this._nextChunk()},this._nextChunk=function(){this._finished||this._config.preview&&!(this._rowCount<this._config.preview)||this._readChunk()},this._readChunk=function(){var t=this._input;if(this._config.chunkSize){var i=Math.min(this._start+this._config.chunkSize,this._input.size);t=n.call(t,this._start,i)}var o=e.readAsText(t,this._config.encoding);r||this._chunkLoaded({target:{result:o}})},this._chunkLoaded=function(t){this._start+=this._config.chunkSize,this._finished=!this._config.chunkSize||this._start>=this._input.size,this.parseChunk(t.target.result)},this._chunkError=function(){this._sendError(e.error)}}function p(t){var e;f.call(this,t=t||{}),this.stream=function(t){return e=t,this._nextChunk()},this._nextChunk=function(){if(!this._finished){var t=this._config.chunkSize,n=t?e.substr(0,t):e;return e=t?e.substr(t):"",this._finished=!e,this.parseChunk(n)}}}function d(t){f.call(this,t=t||{});var e=[],n=!0,r=!1;this.pause=function(){f.prototype.pause.apply(this,arguments),this._input.pause()},this.resume=function(){f.prototype.resume.apply(this,arguments),this._input.resume()},this.stream=function(t){this._input=t,this._input.on("data",this._streamData),this._input.on("end",this._streamEnd),this._input.on("error",this._streamError)},this._checkIsFinished=function(){r&&1===e.length&&(this._finished=!0)},this._nextChunk=function(){this._checkIsFinished(),e.length?this.parseChunk(e.shift()):n=!0},this._streamData=S(function(t){try{e.push("string"==typeof t?t:t.toString(this._config.encoding)),n&&(n=!1,this._checkIsFinished(),this.parseChunk(e.shift()))}catch(t){this._streamError(t)}},this),this._streamError=S(function(t){this._streamCleanUp(),this._sendError(t)},this),this._streamEnd=S(function(){this._streamCleanUp(),r=!0,this._streamData("")},this),this._streamCleanUp=S(function(){this._input.removeListener("data",this._streamData),this._input.removeListener("end",this._streamEnd),this._input.removeListener("error",this._streamError)},this)}function y(t){var e,n,r,i=/^\\s*-?(\\d*\\.?\\d+|\\d+\\.?\\d*)(e[-+]?\\d+)?\\s*$/i,o=/(\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d\\.\\d+([+-][0-2]\\d:[0-5]\\d|Z))|(\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d([+-][0-2]\\d:[0-5]\\d|Z))|(\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d([+-][0-2]\\d:[0-5]\\d|Z))/,a=this,s=0,c=0,f=!1,l=!1,h=[],p={data:[],errors:[],meta:{}};if(O(t.step)){var d=t.step;t.step=function(e){if(p=e,_())m();else{if(m(),0===p.data.length)return;s+=e.data.length,t.preview&&s>t.preview?n.abort():d(p,a)}}}function y(e){return"greedy"===t.skipEmptyLines?""===e.join("").trim():1===e.length&&0===e[0].length}function m(){if(p&&r&&(S("Delimiter","UndetectableDelimiter","Unable to auto-detect delimiting character; defaulted to \'"+u.DefaultDelimiter+"\'"),r=!1),t.skipEmptyLines)for(var e=0;e<p.data.length;e++)y(p.data[e])&&p.data.splice(e--,1);return _()&&function(){if(p){for(var e=0;_()&&e<p.data.length;e++)for(var n=0;n<p.data[e].length;n++){var r=p.data[e][n];t.trimHeaders&&(r=r.trim()),h.push(r)}p.data.splice(0,1)}}(),function(){if(!p||!t.header&&!t.dynamicTyping&&!t.transform)return p;for(var e=0;e<p.data.length;e++){var n,r=t.header?{}:[];for(n=0;n<p.data[e].length;n++){var i=n,o=p.data[e][n];t.header&&(i=n>=h.length?"__parsed_extra":h[n]),t.transform&&(o=t.transform(o,i)),o=g(i,o),"__parsed_extra"===i?(r[i]=r[i]||[],r[i].push(o)):r[i]=o}p.data[e]=r,t.header&&(n>h.length?S("FieldMismatch","TooManyFields","Too many fields: expected "+h.length+" fields but parsed "+n,c+e):n<h.length&&S("FieldMismatch","TooFewFields","Too few fields: expected "+h.length+" fields but parsed "+n,c+e))}return t.header&&p.meta&&(p.meta.fields=h),c+=p.data.length,p}()}function _(){return t.header&&0===h.length}function g(e,n){return r=e,t.dynamicTypingFunction&&void 0===t.dynamicTyping[r]&&(t.dynamicTyping[r]=t.dynamicTypingFunction(r)),!0===(t.dynamicTyping[r]||t.dynamicTyping)?"true"===n||"TRUE"===n||"false"!==n&&"FALSE"!==n&&(i.test(n)?parseFloat(n):o.test(n)?new Date(n):""===n?null:n):n;var r}function S(t,e,n,r){p.errors.push({type:t,code:e,message:n,row:r})}this.parse=function(i,o,a){var s=t.quoteChar||\'"\';if(t.newline||(t.newline=function(t,e){t=t.substr(0,1048576);var n=new RegExp(b(e)+"([^]*?)"+b(e),"gm"),r=(t=t.replace(n,"")).split("\\r"),i=t.split("\\n"),o=1<i.length&&i[0].length<r[0].length;if(1===r.length||o)return"\\n";for(var a=0,s=0;s<r.length;s++)"\\n"===r[s][0]&&a++;return a>=r.length/2?"\\r\\n":"\\r"}(i,s)),r=!1,t.delimiter)O(t.delimiter)&&(t.delimiter=t.delimiter(i),p.meta.delimiter=t.delimiter);else{var c=function(e,n,r,i){for(var o,a,s,c=[",","\\t","|",";",u.RECORD_SEP,u.UNIT_SEP],f=0;f<c.length;f++){var l=c[f],h=0,p=0,d=0;s=void 0;for(var b=new v({comments:i,delimiter:l,newline:n,preview:10}).parse(e),m=0;m<b.data.length;m++)if(r&&y(b.data[m]))d++;else{var _=b.data[m].length;p+=_,void 0!==s?1<_&&(h+=Math.abs(_-s),s=_):s=0}0<b.data.length&&(p/=b.data.length-d),(void 0===a||a<h)&&1.99<p&&(a=h,o=l)}return{successful:!!(t.delimiter=o),bestDelimiter:o}}(i,t.newline,t.skipEmptyLines,t.comments);c.successful?t.delimiter=c.bestDelimiter:(r=!0,t.delimiter=u.DefaultDelimiter),p.meta.delimiter=t.delimiter}var l=w(t);return t.preview&&t.header&&l.preview++,e=i,n=new v(l),p=n.parse(e,o,a),m(),f?{meta:{paused:!0}}:p||{meta:{paused:!1}}},this.paused=function(){return f},this.pause=function(){f=!0,n.abort(),e=e.substr(n.getCharIndex())},this.resume=function(){f=!1,a.streamer.parseChunk(e,!0)},this.aborted=function(){return l},this.abort=function(){l=!0,n.abort(),p.meta.aborted=!0,O(t.complete)&&t.complete(p),e=""}}function b(t){return t.replace(/[.*+?^${}()|[\\]\\\\]/g,"\\\\$&")}function v(t){var e,n=(t=t||{}).delimiter,r=t.newline,i=t.comments,o=t.step,a=t.preview,s=t.fastMode,c=e=void 0===t.quoteChar?\'"\':t.quoteChar;if(void 0!==t.escapeChar&&(c=t.escapeChar),("string"!=typeof n||-1<u.BAD_DELIMITERS.indexOf(n))&&(n=","),i===n)throw"Comment character same as delimiter";!0===i?i="#":("string"!=typeof i||-1<u.BAD_DELIMITERS.indexOf(i))&&(i=!1),"\\n"!==r&&"\\r"!==r&&"\\r\\n"!==r&&(r="\\n");var f=0,l=!1;this.parse=function(t,u,h){if("string"!=typeof t)throw"Input must be a string";var p=t.length,d=n.length,y=r.length,v=i.length,m=O(o),_=[],g=[],w=[],S=f=0;if(!t)return L();if(s||!1!==s&&-1===t.indexOf(e)){for(var T=t.split(r),I=0;I<T.length;I++){if(w=T[I],f+=w.length,I!==T.length-1)f+=r.length;else if(h)return L();if(!i||w.substr(0,v)!==i){if(m){if(_=[],E(w.split(n)),U(),l)return L()}else E(w.split(n));if(a&&a<=I)return _=_.slice(0,a),L(!0)}}return L()}for(var D,A=t.indexOf(n,f),M=t.indexOf(r,f),k=new RegExp(b(c)+b(e),"g");;)if(t[f]!==e)if(i&&0===w.length&&t.substr(f,v)===i){if(-1===M)return L();f=M+y,M=t.indexOf(r,f),A=t.indexOf(n,f)}else if(-1!==A&&(A<M||-1===M))w.push(t.substring(f,A)),f=A+d,A=t.indexOf(n,f);else{if(-1===M)break;if(w.push(t.substring(f,M)),P(M+y),m&&(U(),l))return L();if(a&&_.length>=a)return L(!0)}else for(D=f,f++;;){if(-1===(D=t.indexOf(e,D+1)))return h||g.push({type:"Quotes",code:"MissingQuotes",message:"Quoted field unterminated",row:_.length,index:f}),F();if(D===p-1)return F(t.substring(f,D).replace(k,e));if(e!==c||t[D+1]!==c){if(e===c||0===D||t[D-1]!==c){var j=B(-1===M?A:Math.min(A,M));if(t[D+1+j]===n){w.push(t.substring(f,D).replace(k,e)),f=D+1+j+d,A=t.indexOf(n,f),M=t.indexOf(r,f);break}var x=B(M);if(t.substr(D+1+x,y)===r){if(w.push(t.substring(f,D).replace(k,e)),P(D+1+x+y),A=t.indexOf(n,f),m&&(U(),l))return L();if(a&&_.length>=a)return L(!0);break}g.push({type:"Quotes",code:"InvalidQuotes",message:"Trailing quote on quoted field is malformed",row:_.length,index:f}),D++}}else D++}return F();function E(t){_.push(t),S=f}function B(e){var n=0;if(-1!==e){var r=t.substring(D+1,e);r&&""===r.trim()&&(n=r.length)}return n}function F(e){return h||(void 0===e&&(e=t.substr(f)),w.push(e),f=p,E(w),m&&U()),L()}function P(e){f=e,E(w),w=[],M=t.indexOf(r,f)}function L(t){return{data:_,errors:g,meta:{delimiter:n,linebreak:r,aborted:l,truncated:!!t,cursor:S+(u||0)}}}function U(){o(L()),_=[],g=[]}},this.abort=function(){l=!0},this.getCharIndex=function(){return f}}function m(t){var e=t.data,n=a[e.workerId],r=!1;if(e.error)n.userError(e.error,e.file);else if(e.results&&e.results.data){var i={abort:function(){r=!0,_(e.workerId,{data:[],errors:[],meta:{aborted:!0}})},pause:g,resume:g};if(O(n.userStep)){for(var o=0;o<e.results.data.length&&(n.userStep({data:[e.results.data[o]],errors:e.results.errors,meta:e.results.meta},i),!r);o++);delete e.results}else O(n.userChunk)&&(n.userChunk(e.results,i,e.file),delete e.results)}e.finished&&!r&&_(e.workerId,e.results)}function _(t,e){var n=a[t];O(n.userComplete)&&n.userComplete(e),n.terminate(),delete a[t]}function g(){throw"Not implemented."}function w(t){if("object"!=typeof t||null===t)return t;var e=Array.isArray(t)?[]:{};for(var n in t)e[n]=w(t[n]);return e}function S(t,e){return function(){t.apply(e,arguments)}}function O(t){return"function"==typeof t}return i?n.onmessage=function(t){var e=t.data;if(void 0===u.WORKER_ID&&e&&(u.WORKER_ID=e.workerId),"string"==typeof e.input)n.postMessage({workerId:u.WORKER_ID,results:u.parse(e.input,e.config),finished:!0});else if(n.File&&e.input instanceof File||e.input instanceof Object){var r=u.parse(e.input,e.config);r&&n.postMessage({workerId:u.WORKER_ID,results:r,finished:!0})}}:u.WORKERS_SUPPORTED&&(e=document.getElementsByTagName("script"),t=e.length?e[e.length-1].src:"",document.body?document.addEventListener("DOMContentLoaded",function(){o=!0},!0):o=!0),(l.prototype=Object.create(f.prototype)).constructor=l,(h.prototype=Object.create(f.prototype)).constructor=h,(p.prototype=Object.create(p.prototype)).constructor=p,(d.prototype=Object.create(f.prototype)).constructor=d,u})?r.apply(e,[]):r)||(t.exports=i)},function(t,e){t.exports=function(t){return"object"==typeof t?null!==t:"function"==typeof t}},function(t,e){t.exports=function(t){if(null==t)throw TypeError("Can\'t call method on  "+t);return t}},function(t,e,n){var r=n(28),i=Math.min;t.exports=function(t){return t>0?i(r(t),9007199254740991):0}},function(t,e){var n=Math.ceil,r=Math.floor;t.exports=function(t){return isNaN(t=+t)?0:(t>0?r:n)(t)}},function(t,e,n){t.exports=!n(22)(function(){return 7!=Object.defineProperty({},"a",{get:function(){return 7}}).a})},function(t,e,n){var r=n(18),i=n(19),o=n(31),a=n(42)("src"),s=n(81),u=(""+s).split("toString");n(41).inspectSource=function(t){return s.call(t)},(t.exports=function(t,e,n,s){var c="function"==typeof n;c&&(o(n,"name")||i(n,"name",e)),t[e]!==n&&(c&&(o(n,a)||i(n,a,t[e]?""+t[e]:u.join(String(e)))),t===r?t[e]=n:s?t[e]?t[e]=n:i(t,e,n):(delete t[e],i(t,e,n)))})(Function.prototype,"toString",function(){return"function"==typeof this&&this[a]||s.call(this)})},function(t,e){var n={}.hasOwnProperty;t.exports=function(t,e){return n.call(t,e)}},function(t,e,n){for(var r=n(83),i=n(61),o=n(30),a=n(18),s=n(19),u=n(48),c=n(15),f=c("iterator"),l=c("toStringTag"),h=u.Array,p={CSSRuleList:!0,CSSStyleDeclaration:!1,CSSValueList:!1,ClientRectList:!1,DOMRectList:!1,DOMStringList:!1,DOMTokenList:!0,DataTransferItemList:!1,FileList:!1,HTMLAllCollection:!1,HTMLCollection:!1,HTMLFormElement:!1,HTMLSelectElement:!1,MediaList:!0,MimeTypeArray:!1,NamedNodeMap:!1,NodeList:!0,PaintRequestList:!1,Plugin:!1,PluginArray:!1,SVGLengthList:!1,SVGNumberList:!1,SVGPathSegList:!1,SVGPointList:!1,SVGStringList:!1,SVGTransformList:!1,SourceBufferList:!1,StyleSheetList:!0,TextTrackCueList:!1,TextTrackList:!1,TouchList:!1},d=i(p),y=0;y<d.length;y++){var b,v=d[y],m=p[v],_=a[v],g=_&&_.prototype;if(g&&(g[f]||s(g,f,h),g[l]||s(g,l,v),u[v]=h,m))for(b in r)g[b]||o(g,b,r[b],!0)}},function(t,e,n){"use strict";n(51);var r=n(24),i=n.n(r);const o={initDataValue:()=>[],initRowValue:()=>({}),initColumnValue:(t,e,n)=>e[n]=[],setColumnValue:(t,e,n,r)=>e[n]=r,addColumnValue:(t,e,n,r)=>e[n].unshift(r),addRow:(t,e)=>t.push(e),formatData:t=>t,slice:(t,e)=>t.slice(e)},a=Object.assign({},o,{addColumnValue:(t,e,n,r)=>e[n.split("|").join(",")].unshift(r),setColumnValue:(t,e,n,r)=>e[n.split("|").join(",")]=r,formatData:(t,e)=>i.a.unparse(t,e)}),s={initDataValue:()=>new Object,initRowValue:()=>{},setColumnValue:(t,e,n,r)=>{t[n]=t[n]||[],t[n].push(r)},addColumnValue:(t,e,n,r)=>{t[n]=t[n]||[],t[n][t[n].length-1].unshift(r)},initColumnValue:(t,e,n)=>{t[n]=t[n]||[],t[n].push([])},addRow:()=>{},formatData:t=>t,slice:(t,e)=>{let n={};for(let r in t)n[r]=t[r].slice(e);return n}};e.a={jsonFormatter:o,jsonTableFormatter:s,csvFormatter:a}},function(t,e){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(t){"object"==typeof window&&(n=window)}t.exports=n},function(t,e){var n,r,i=t.exports={};function o(){throw new Error("setTimeout has not been defined")}function a(){throw new Error("clearTimeout has not been defined")}function s(t){if(n===setTimeout)return setTimeout(t,0);if((n===o||!n)&&setTimeout)return n=setTimeout,setTimeout(t,0);try{return n(t,0)}catch(e){try{return n.call(null,t,0)}catch(e){return n.call(this,t,0)}}}!function(){try{n="function"==typeof setTimeout?setTimeout:o}catch(t){n=o}try{r="function"==typeof clearTimeout?clearTimeout:a}catch(t){r=a}}();var u,c=[],f=!1,l=-1;function h(){f&&u&&(f=!1,u.length?c=u.concat(c):l=-1,c.length&&p())}function p(){if(!f){var t=s(h);f=!0;for(var e=c.length;e;){for(u=c,c=[];++l<e;)u&&u[l].run();l=-1,e=c.length}u=null,f=!1,function(t){if(r===clearTimeout)return clearTimeout(t);if((r===a||!r)&&clearTimeout)return r=clearTimeout,clearTimeout(t);try{r(t)}catch(e){try{return r.call(null,t)}catch(e){return r.call(this,t)}}}(t)}}function d(t,e){this.fun=t,this.array=e}function y(){}i.nextTick=function(t){var e=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)e[n-1]=arguments[n];c.push(new d(t,e)),1!==c.length||f||s(p)},d.prototype.run=function(){this.fun.apply(null,this.array)},i.title="browser",i.browser=!0,i.env={},i.argv=[],i.version="",i.versions={},i.on=y,i.addListener=y,i.once=y,i.off=y,i.removeListener=y,i.removeAllListeners=y,i.emit=y,i.prependListener=y,i.prependOnceListener=y,i.listeners=function(t){return[]},i.binding=function(t){throw new Error("process.binding is not supported")},i.cwd=function(){return"/"},i.chdir=function(t){throw new Error("process.chdir is not supported")},i.umask=function(){return 0}},function(t,e,n){var r=n(26);t.exports=function(t){return Object(r(t))}},function(t,e,n){"use strict";var r=n(75)(!0);t.exports=function(t,e,n){return e+(n?r(t,e).length:1)}},function(t,e,n){"use strict";var r=n(76),i=RegExp.prototype.exec;t.exports=function(t,e){var n=t.exec;if("function"==typeof n){var o=n.call(t,e);if("object"!=typeof o)throw new TypeError("RegExp exec method returned something other than an Object or null");return o}if("RegExp"!==r(t))throw new TypeError("RegExp#exec called on incompatible receiver");return i.call(t,e)}},function(t,e){var n={}.toString;t.exports=function(t){return n.call(t).slice(8,-1)}},function(t,e,n){var r=n(41),i=n(18),o=i["__core-js_shared__"]||(i["__core-js_shared__"]={});(t.exports=function(t,e){return o[t]||(o[t]=void 0!==e?e:{})})("versions",[]).push({version:r.version,mode:n(58)?"pure":"global",copyright:"© 2019 Denis Pushkarev (zloirock.ru)"})},function(t,e){var n=t.exports={version:"2.6.8"};"number"==typeof __e&&(__e=n)},function(t,e){var n=0,r=Math.random();t.exports=function(t){return"Symbol(".concat(void 0===t?"":t,")_",(++n+r).toString(36))}},function(t,e,n){"use strict";n(77);var r=n(30),i=n(19),o=n(22),a=n(26),s=n(15),u=n(44),c=s("species"),f=!o(function(){var t=/./;return t.exec=function(){var t=[];return t.groups={a:"7"},t},"7"!=="".replace(t,"$<a>")}),l=function(){var t=/(?:)/,e=t.exec;t.exec=function(){return e.apply(this,arguments)};var n="ab".split(t);return 2===n.length&&"a"===n[0]&&"b"===n[1]}();t.exports=function(t,e,n){var h=s(t),p=!o(function(){var e={};return e[h]=function(){return 7},7!=""[t](e)}),d=p?!o(function(){var e=!1,n=/a/;return n.exec=function(){return e=!0,null},"split"===t&&(n.constructor={},n.constructor[c]=function(){return n}),n[h](""),!e}):void 0;if(!p||!d||"replace"===t&&!f||"split"===t&&!l){var y=/./[h],b=n(a,h,""[t],function(t,e,n,r,i){return e.exec===u?p&&!i?{done:!0,value:y.call(e,n,r)}:{done:!0,value:t.call(n,e,r)}:{done:!1}}),v=b[0],m=b[1];r(String.prototype,t,v),i(RegExp.prototype,h,2==e?function(t,e){return m.call(t,this,e)}:function(t){return m.call(t,this)})}}},function(t,e,n){"use strict";var r,i,o=n(78),a=RegExp.prototype.exec,s=String.prototype.replace,u=a,c=(r=/a/,i=/b*/g,a.call(r,"a"),a.call(i,"a"),0!==r.lastIndex||0!==i.lastIndex),f=void 0!==/()??/.exec("")[1];(c||f)&&(u=function(t){var e,n,r,i,u=this;return f&&(n=new RegExp("^"+u.source+"$(?!\\\\s)",o.call(u))),c&&(e=u.lastIndex),r=a.call(u,t),c&&r&&(u.lastIndex=u.global?r.index+r[0].length:e),f&&r&&r.length>1&&s.call(r[0],n,function(){for(i=1;i<arguments.length-2;i++)void 0===arguments[i]&&(r[i]=void 0)}),r}),t.exports=u},function(t,e,n){var r=n(18),i=n(41),o=n(19),a=n(30),s=n(82),u=function(t,e,n){var c,f,l,h,p=t&u.F,d=t&u.G,y=t&u.S,b=t&u.P,v=t&u.B,m=d?r:y?r[e]||(r[e]={}):(r[e]||{}).prototype,_=d?i:i[e]||(i[e]={}),g=_.prototype||(_.prototype={});for(c in d&&(n=e),n)l=((f=!p&&m&&void 0!==m[c])?m:n)[c],h=v&&f?s(l,r):b&&"function"==typeof l?s(Function.call,l):l,m&&a(m,c,l,t&u.U),_[c]!=l&&o(_,c,h),b&&g[c]!=l&&(g[c]=l)};r.core=i,u.F=1,u.G=2,u.S=4,u.P=8,u.B=16,u.W=32,u.U=64,u.R=128,t.exports=u},function(t,e,n){var r=n(16),i=n(79),o=n(80),a=Object.defineProperty;e.f=n(29)?Object.defineProperty:function(t,e,n){if(r(t),e=o(e,!0),r(n),i)try{return a(t,e,n)}catch(t){}if("get"in n||"set"in n)throw TypeError("Accessors not supported!");return"value"in n&&(t[e]=n.value),t}},function(t,e){t.exports=function(t){if("function"!=typeof t)throw TypeError(t+" is not a function!");return t}},function(t,e){t.exports={}},function(t,e,n){var r=n(86),i=n(26);t.exports=function(t){return r(i(t))}},function(t,e,n){var r=n(40)("keys"),i=n(42);t.exports=function(t){return r[t]||(r[t]=i(t))}},function(t,e,n){"use strict";var r=n(96),i=n(16),o=n(97),a=n(37),s=n(27),u=n(38),c=n(44),f=n(22),l=Math.min,h=[].push,p=!f(function(){RegExp(4294967295,"y")});n(43)("split",2,function(t,e,n,f){var d;return d="c"=="abbc".split(/(b)*/)[1]||4!="test".split(/(?:)/,-1).length||2!="ab".split(/(?:ab)*/).length||4!=".".split(/(.?)(.?)/).length||".".split(/()()/).length>1||"".split(/.?/).length?function(t,e){var i=String(this);if(void 0===t&&0===e)return[];if(!r(t))return n.call(i,t,e);for(var o,a,s,u=[],f=(t.ignoreCase?"i":"")+(t.multiline?"m":"")+(t.unicode?"u":"")+(t.sticky?"y":""),l=0,p=void 0===e?4294967295:e>>>0,d=new RegExp(t.source,f+"g");(o=c.call(d,i))&&!((a=d.lastIndex)>l&&(u.push(i.slice(l,o.index)),o.length>1&&o.index<i.length&&h.apply(u,o.slice(1)),s=o[0].length,l=a,u.length>=p));)d.lastIndex===o.index&&d.lastIndex++;return l===i.length?!s&&d.test("")||u.push(""):u.push(i.slice(l)),u.length>p?u.slice(0,p):u}:"0".split(void 0,0).length?function(t,e){return void 0===t&&0===e?[]:n.call(this,t,e)}:n,[function(n,r){var i=t(this),o=null==n?void 0:n[e];return void 0!==o?o.call(n,i,r):d.call(String(i),n,r)},function(t,e){var r=f(d,t,this,e,d!==n);if(r.done)return r.value;var c=i(t),h=String(this),y=o(c,RegExp),b=c.unicode,v=(c.ignoreCase?"i":"")+(c.multiline?"m":"")+(c.unicode?"u":"")+(p?"y":"g"),m=new y(p?c:"^(?:"+c.source+")",v),_=void 0===e?4294967295:e>>>0;if(0===_)return[];if(0===h.length)return null===u(m,h)?[h]:[];for(var g=0,w=0,S=[];w<h.length;){m.lastIndex=p?w:0;var O,T=u(m,p?h:h.slice(w));if(null===T||(O=l(s(m.lastIndex+(p?0:w)),h.length))===g)w=a(h,w,b);else{if(S.push(h.slice(g,w)),S.length===_)return S;for(var I=1;I<=T.length-1;I++)if(S.push(T[I]),S.length===_)return S;w=g=O}}return S.push(h.slice(g)),S}]})},function(t,e,n){"use strict";(function(t,r){var i,o=(i="undefined"!=typeof document&&document.currentScript?document.currentScript.src:void 0,function(e){var o;e=e||{},o||(o=void 0!==e?e:{});var a,s={};for(a in o)o.hasOwnProperty(a)&&(s[a]=o[a]);o.arguments=[],o.thisProgram="./this.program",o.quit=function(t,e){throw e},o.preRun=[],o.postRun=[];var u=!1,c=!1,f=!1,l=!1;u="object"==typeof window,c="function"==typeof importScripts,f="object"==typeof t&&!u&&!c,l=!u&&!f&&!c;var h,p,d="";f?(d=r+"/",o.read=function(t,e){return h||(h=n(69)),p||(p=n(70)),t=p.normalize(t),t=h.readFileSync(t),e?t:t.toString()},o.readBinary=function(t){return(t=o.read(t,!0)).buffer||(t=new Uint8Array(t)),t.buffer||We("Assertion failed: undefined"),t},1<t.argv.length&&(o.thisProgram=t.argv[1].replace(/\\\\/g,"/")),o.arguments=t.argv.slice(2),t.on("uncaughtException",function(t){if(!(t instanceof Ve))throw t}),t.on("unhandledRejection",We),o.quit=function(e){t.exit(e)},o.inspect=function(){return"[Emscripten Module object]"}):l?("undefined"!=typeof read&&(o.read=function(t){return read(t)}),o.readBinary=function(t){return"function"==typeof readbuffer?new Uint8Array(readbuffer(t)):("object"==typeof(t=read(t,"binary"))||We("Assertion failed: undefined"),t)},"undefined"!=typeof scriptArgs?o.arguments=scriptArgs:void 0!==arguments&&(o.arguments=arguments),"function"==typeof quit&&(o.quit=function(t){quit(t)})):(u||c)&&(c?d=self.location.href:document.currentScript&&(d=document.currentScript.src),i&&(d=i),d=0!==d.indexOf("blob:")?d.substr(0,d.lastIndexOf("/")+1):"",o.read=function(t){var e=new XMLHttpRequest;return e.open("GET",t,!1),e.send(null),e.responseText},c&&(o.readBinary=function(t){var e=new XMLHttpRequest;return e.open("GET",t,!1),e.responseType="arraybuffer",e.send(null),new Uint8Array(e.response)}),o.readAsync=function(t,e,n){var r=new XMLHttpRequest;r.open("GET",t,!0),r.responseType="arraybuffer",r.onload=function(){200==r.status||0==r.status&&r.response?e(r.response):n()},r.onerror=n,r.send(null)},o.setWindowTitle=function(t){document.title=t});var y=o.print||("undefined"!=typeof console?console.log.bind(console):"undefined"!=typeof print?print:null),b=o.printErr||("undefined"!=typeof printErr?printErr:"undefined"!=typeof console&&console.warn.bind(console)||y);for(a in s)s.hasOwnProperty(a)&&(o[a]=s[a]);function v(t){var e=I[C>>2];if((t=e+t+15&-16)<=R)I[C>>2]=t;else if(!_e(t))return 0;return e}s=void 0;var m={"f64-rem":function(t,e){return t%e},debugger:function(){}};"object"!=typeof WebAssembly&&b("no native wasm support detected");var _,g,w,S,O,T,I,D,A,M,k=!1,j="undefined"!=typeof TextDecoder?new TextDecoder("utf8"):void 0;function x(t,e,n){var r=e+n;for(n=e;t[n]&&!(n>=r);)++n;if(16<n-e&&t.subarray&&j)return j.decode(t.subarray(e,n));for(r="";e<n;){var i=t[e++];if(128&i){var o=63&t[e++];if(192==(224&i))r+=String.fromCharCode((31&i)<<6|o);else{var a=63&t[e++];65536>(i=224==(240&i)?(15&i)<<12|o<<6|a:(7&i)<<18|o<<12|a<<6|63&t[e++])?r+=String.fromCharCode(i):(i-=65536,r+=String.fromCharCode(55296|i>>10,56320|1023&i))}}else r+=String.fromCharCode(i)}return r}function E(t){return t?x(S,t,void 0):""}function B(t,e,n,r){if(0<r){r=n+r-1;for(var i=0;i<t.length;++i){var o=t.charCodeAt(i);if(55296<=o&&57343>=o&&(o=65536+((1023&o)<<10)|1023&t.charCodeAt(++i)),127>=o){if(n>=r)break;e[n++]=o}else{if(2047>=o){if(n+1>=r)break;e[n++]=192|o>>6}else{if(65535>=o){if(n+2>=r)break;e[n++]=224|o>>12}else{if(n+3>=r)break;e[n++]=240|o>>18,e[n++]=128|o>>12&63}e[n++]=128|o>>6&63}e[n++]=128|63&o}}e[n]=0}}function F(t){for(var e=0,n=0;n<t.length;++n){var r=t.charCodeAt(n);55296<=r&&57343>=r&&(r=65536+((1023&r)<<10)|1023&t.charCodeAt(++n)),127>=r?++e:e=2047>=r?e+2:65535>=r?e+3:e+4}return e}function P(t){var e=F(t)+1,n=Re(e);return B(t,w,n,e),n}function L(t){return 0<t%65536&&(t+=65536-t%65536),t}function U(){o.HEAP8=w=new Int8Array(g),o.HEAP16=O=new Int16Array(g),o.HEAP32=I=new Int32Array(g),o.HEAPU8=S=new Uint8Array(g),o.HEAPU16=T=new Uint16Array(g),o.HEAPU32=D=new Uint32Array(g),o.HEAPF32=A=new Float32Array(g),o.HEAPF64=M=new Float64Array(g)}"undefined"!=typeof TextDecoder&&new TextDecoder("utf-16le");var C=38304,R=o.TOTAL_MEMORY||16777216;function N(t){for(;0<t.length;){var e=t.shift();if("function"==typeof e)e();else{var n=e.Tb;"number"==typeof n?void 0===e.Eb?o.dynCall_v(n):o.dynCall_vi(n,e.Eb):n(void 0===e.Eb?null:e.Eb)}}}5242880>R&&b("TOTAL_MEMORY should be larger than TOTAL_STACK, was "+R+"! (TOTAL_STACK=5242880)"),o.buffer?g=o.buffer:("object"==typeof WebAssembly&&"function"==typeof WebAssembly.Memory?(_=new WebAssembly.Memory({initial:R/65536}),g=_.buffer):g=new ArrayBuffer(R),o.buffer=g),U(),I[C>>2]=5281440;var Y=[],V=[],z=[],W=[],H=!1;function G(){var t=o.preRun.shift();Y.unshift(t)}var q=0,Z=null,J=null;function K(){var t=$;return String.prototype.startsWith?t.startsWith("data:application/octet-stream;base64,"):0===t.indexOf("data:application/octet-stream;base64,")}o.preloadedImages={},o.preloadedAudios={};var $="psp.async.wasm";if(!K()){var Q=$;$=o.locateFile?o.locateFile(Q,d):d+Q}function X(){try{if(o.wasmBinary)return new Uint8Array(o.wasmBinary);if(o.readBinary)return o.readBinary($);throw"both async and sync fetching of the wasm failed"}catch(t){We(t)}}function tt(){return o.wasmBinary||!u&&!c||"function"!=typeof fetch?new Promise(function(t){t(X())}):fetch($,{credentials:"same-origin"}).then(function(t){if(!t.ok)throw"failed to load wasm binary file at \'"+$+"\'";return t.arrayBuffer()}).catch(function(){return X()})}function et(t){function e(t){o.asm=t.exports,q--,o.monitorRunDependencies&&o.monitorRunDependencies(q),0==q&&(null!==Z&&(clearInterval(Z),Z=null),J&&(t=J,J=null,t()))}function n(t){e(t.instance)}function r(t){tt().then(function(t){return WebAssembly.instantiate(t,i)}).then(t,function(t){b("failed to asynchronously prepare wasm: "+t),We(t)})}var i={env:t,global:{NaN:NaN,Infinity:1/0},"global.Math":Math,asm2wasm:m};if(q++,o.monitorRunDependencies&&o.monitorRunDependencies(q),o.instantiateWasm)try{return o.instantiateWasm(i,e)}catch(t){return b("Module.instantiateWasm callback failed with error: "+t),!1}return o.wasmBinary||"function"!=typeof WebAssembly.instantiateStreaming||K()||"function"!=typeof fetch?r(n):WebAssembly.instantiateStreaming(fetch($,{credentials:"same-origin"}),i).then(n,function(t){b("wasm streaming compile failed: "+t),b("falling back to ArrayBuffer instantiation"),r(n)}),{}}o.asm=function(t,e){return e.memory=_,e.table=new WebAssembly.Table({initial:1430,maximum:1430,element:"anyfunc"}),e.__memory_base=1024,e.__table_base=0,et(e)};var nt=[function(){if("undefined"!=typeof self)if(self.dispatchEvent&&!self.lb&&self.document){self.lb=!0;var t=self.document.createEvent("Event");t.initEvent("perspective-ready",!1,!0),self.dispatchEvent(t)}else!self.document&&self.postMessage&&self.postMessage({})}];V.push({Tb:function(){Ce()}});var rt={};function it(t){if(it.lb)var e=I[t>>2],n=I[e>>2];else it.lb=!0,rt.USER=rt.LOGNAME="web_user",rt.PATH="/",rt.PWD="/",rt.HOME="/home/web_user",rt.LANG="C.UTF-8",rt._=o.thisProgram,n=H?Pe(1024):v(1024),e=H?Pe(256):v(256),I[e>>2]=n,I[t>>2]=e;t=[];var r,i=0;for(r in rt)if("string"==typeof rt[r]){var a=r+"="+rt[r];t.push(a),i+=a.length}if(1024<i)throw Error("Environment size exceeded TOTAL_ENV_SIZE!");for(r=0;r<t.length;r++){i=a=t[r];for(var s=n,u=0;u<i.length;++u)w[s++>>0]=i.charCodeAt(u);w[s>>0]=0,I[e+4*r>>2]=n,n+=a.length+1}I[e+4*t.length>>2]=0}function ot(t){return o.___errno_location&&(I[o.___errno_location()>>2]=t),t}var at=[null,[],[]],st=0;function ut(){return I[(st+=4)-4>>2]}var ct={},ft={};function lt(t){for(;t.length;){var e=t.pop();t.pop()(e)}}function ht(t){return this.fromWireType(D[t>>2])}var pt={},dt={},yt={};function bt(t){if(void 0===t)return"_unknown";var e=(t=t.replace(/[^a-zA-Z0-9_]/g,"$")).charCodeAt(0);return 48<=e&&57>=e?"_"+t:t}function vt(t,e){return t=bt(t),new Function("body","return function "+t+\'() {\\n    "use strict";    return body.apply(this, arguments);\\n};\\n\')(e)}function mt(t){var e=Error,n=vt(t,function(e){this.name=t,this.message=e,void 0!==(e=Error(e).stack)&&(this.stack=this.toString()+"\\n"+e.replace(/^Error(:[^\\n]*)?\\n/,""))});return n.prototype=Object.create(e.prototype),n.prototype.constructor=n,n.prototype.toString=function(){return void 0===this.message?this.name:this.name+": "+this.message},n}var _t=void 0;function gt(t){throw new _t(t)}function wt(t,e,n){function r(e){(e=n(e)).length!==t.length&&gt("Mismatched type converter count");for(var r=0;r<t.length;++r)At(t[r],e[r])}t.forEach(function(t){yt[t]=e});var i=Array(e.length),o=[],a=0;e.forEach(function(t,e){dt.hasOwnProperty(t)?i[e]=dt[t]:(o.push(t),pt.hasOwnProperty(t)||(pt[t]=[]),pt[t].push(function(){i[e]=dt[t],++a===o.length&&r(i)}))}),0===o.length&&r(i)}function St(t){switch(t){case 1:return 0;case 2:return 1;case 4:return 2;case 8:return 3;default:throw new TypeError("Unknown type size: "+t)}}var Ot=void 0;function Tt(t){for(var e="";S[t];)e+=Ot[S[t++]];return e}var It=void 0;function Dt(t){throw new It(t)}function At(t,e,n){if(n=n||{},!("argPackAdvance"in e))throw new TypeError("registerType registeredInstance requires argPackAdvance");var r=e.name;if(t||Dt(\'type "\'+r+\'" must have a positive integer typeid pointer\'),dt.hasOwnProperty(t)){if(n.ac)return;Dt("Cannot register type \'"+r+"\' twice")}dt[t]=e,delete yt[t],pt.hasOwnProperty(t)&&(e=pt[t],delete pt[t],e.forEach(function(t){t()}))}function Mt(t){return{count:t.count,vb:t.vb,xb:t.xb,hb:t.hb,ib:t.ib,mb:t.mb,ob:t.ob}}function kt(t){Dt(t.fb.ib.gb.name+" instance already deleted")}var jt=void 0,xt=[];function Et(){for(;xt.length;){var t=xt.pop();t.fb.vb=!1,t.delete()}}function Bt(){}var Ft={};function Pt(t,e,n){if(void 0===t[e].kb){var r=t[e];t[e]=function(){return t[e].kb.hasOwnProperty(arguments.length)||Dt("Function \'"+n+"\' called with an invalid number of arguments ("+arguments.length+") - expects one of ("+t[e].kb+")!"),t[e].kb[arguments.length].apply(this,arguments)},t[e].kb=[],t[e].kb[r.zb]=r}}function Lt(t,e,n){o.hasOwnProperty(t)?((void 0===n||void 0!==o[t].kb&&void 0!==o[t].kb[n])&&Dt("Cannot register public name \'"+t+"\' twice"),Pt(o,t,t),o.hasOwnProperty(n)&&Dt("Cannot register multiple overloads of a function with the same number of arguments ("+n+")!"),o[t].kb[n]=e):(o[t]=e,void 0!==n&&(o[t].wc=n))}function Ut(t,e,n,r,i,o,a,s){this.name=t,this.constructor=e,this.wb=n,this.qb=r,this.pb=i,this.Ub=o,this.yb=a,this.Rb=s,this.fc=[]}function Ct(t,e,n){for(;e!==n;)e.yb||Dt("Expected null or instance of "+n.name+", got an instance of "+e.name),t=e.yb(t),e=e.pb;return t}function Rt(t,e){return null===e?(this.Fb&&Dt("null is not a valid "+this.name),0):(e.fb||Dt(\'Cannot pass "\'+se(e)+\'" as a \'+this.name),e.fb.hb||Dt("Cannot pass deleted object as a pointer of type "+this.name),Ct(e.fb.hb,e.fb.ib.gb,this.gb))}function Nt(t,e){if(null===e){if(this.Fb&&Dt("null is not a valid "+this.name),this.Bb){var n=this.Gb();return null!==t&&t.push(this.qb,n),n}return 0}if(e.fb||Dt(\'Cannot pass "\'+se(e)+\'" as a \'+this.name),e.fb.hb||Dt("Cannot pass deleted object as a pointer of type "+this.name),!this.Ab&&e.fb.ib.Ab&&Dt("Cannot convert argument of type "+(e.fb.ob?e.fb.ob.name:e.fb.ib.name)+" to parameter type "+this.name),n=Ct(e.fb.hb,e.fb.ib.gb,this.gb),this.Bb)switch(void 0===e.fb.mb&&Dt("Passing raw pointer to smart pointer is illegal"),this.lc){case 0:e.fb.ob===this?n=e.fb.mb:Dt("Cannot convert argument of type "+(e.fb.ob?e.fb.ob.name:e.fb.ib.name)+" to parameter type "+this.name);break;case 1:n=e.fb.mb;break;case 2:if(e.fb.ob===this)n=e.fb.mb;else{var r=e.clone();n=this.hc(n,ie(function(){r.delete()})),null!==t&&t.push(this.qb,n)}break;default:Dt("Unsupporting sharing policy")}return n}function Yt(t,e){return null===e?(this.Fb&&Dt("null is not a valid "+this.name),0):(e.fb||Dt(\'Cannot pass "\'+se(e)+\'" as a \'+this.name),e.fb.hb||Dt("Cannot pass deleted object as a pointer of type "+this.name),e.fb.ib.Ab&&Dt("Cannot convert argument of type "+e.fb.ib.name+" to parameter type "+this.name),Ct(e.fb.hb,e.fb.ib.gb,this.gb))}function Vt(t,e,n){return e===n?t:void 0===n.pb?null:null===(t=Vt(t,e,n.pb))?null:n.Rb(t)}var zt={};function Wt(t,e){for(void 0===e&&Dt("ptr should not be undefined");t.pb;)e=t.yb(e),t=t.pb;return zt[e]}function Ht(t,e){return e.ib&&e.hb||gt("makeClassHandle requires ptr and ptrType"),!!e.ob!=!!e.mb&&gt("Both smartPtrType and smartPtr must be specified"),e.count={value:1},Object.create(t,{fb:{value:e}})}function Gt(t,e,n,r,i,o,a,s,u,c,f){this.name=t,this.gb=e,this.Fb=n,this.Ab=r,this.Bb=i,this.ec=o,this.lc=a,this.Ob=s,this.Gb=u,this.hc=c,this.qb=f,i||void 0!==e.pb?this.toWireType=Nt:(this.toWireType=r?Rt:Yt,this.nb=null)}function qt(t,e,n){o.hasOwnProperty(t)||gt("Replacing nonexistant public symbol"),void 0!==o[t].kb&&void 0!==n?o[t].kb[n]=e:(o[t]=e,o[t].zb=n)}function Zt(t,e){if(t=Tt(t),void 0!==o["FUNCTION_TABLE_"+t])var n=o["FUNCTION_TABLE_"+t][e];else if("undefined"!=typeof FUNCTION_TABLE)n=FUNCTION_TABLE[e];else{void 0===(n=o["dynCall_"+t])&&void 0===(n=o["dynCall_"+t.replace(/f/g,"d")])&&Dt("No dynCall invoker for signature: "+t);for(var r=[],i=1;i<t.length;++i)r.push("a"+i);i="return function dynCall_"+t+"_"+e+"("+r.join(", ")+") {\\n",i+="    return dynCall(rawFunction"+(r.length?", ":"")+r.join(", ")+");\\n",n=new Function("dynCall","rawFunction",i+"};\\n")(n,e)}return"function"!=typeof n&&Dt("unknown function pointer with signature "+t+": "+e),n}var Jt=void 0;function Kt(t){var e=Tt(t=Be(t));return Fe(t),e}function $t(t,e){var n=[],r={};throw e.forEach(function t(e){r[e]||dt[e]||(yt[e]?yt[e].forEach(t):(n.push(e),r[e]=!0))}),new Jt(t+": "+n.map(Kt).join([", "]))}function Qt(t,e){for(var n=[],r=0;r<t;r++)n.push(I[(e>>2)+r]);return n}function Xt(t){var e=Function;if(!(e instanceof Function))throw new TypeError("new_ called with constructor type "+typeof e+" which is not a function");var n=vt(e.name||"unknownFunctionName",function(){});return n.prototype=e.prototype,n=new n,(t=e.apply(n,t))instanceof Object?t:n}function te(t,e,n,r,i){var o=e.length;2>o&&Dt("argTypes array size mismatch! Must at least get return value and \'this\' types!");var a=null!==e[1]&&null!==n,s=!1;for(n=1;n<e.length;++n)if(null!==e[n]&&void 0===e[n].nb){s=!0;break}var u="void"!==e[0].name,c="",f="";for(n=0;n<o-2;++n)c+=(0!==n?", ":"")+"arg"+n,f+=(0!==n?", ":"")+"arg"+n+"Wired";t="return function "+bt(t)+"("+c+") {\\nif (arguments.length !== "+(o-2)+") {\\nthrowBindingError(\'function "+t+" called with \' + arguments.length + \' arguments, expected "+(o-2)+" args!\');\\n}\\n",s&&(t+="var destructors = [];\\n");var l=s?"destructors":"null";for(c="throwBindingError invoker fn runDestructors retType classParam".split(" "),r=[Dt,r,i,lt,e[0],e[1]],a&&(t+="var thisWired = classParam.toWireType("+l+", this);\\n"),n=0;n<o-2;++n)t+="var arg"+n+"Wired = argType"+n+".toWireType("+l+", arg"+n+"); // "+e[n+2].name+"\\n",c.push("argType"+n),r.push(e[n+2]);if(a&&(f="thisWired"+(0<f.length?", ":"")+f),t+=(u?"var rv = ":"")+"invoker(fn"+(0<f.length?", ":"")+f+");\\n",s)t+="runDestructors(destructors);\\n";else for(n=a?1:2;n<e.length;++n)o=1===n?"thisWired":"arg"+(n-2)+"Wired",null!==e[n].nb&&(t+=o+"_dtor("+o+"); // "+e[n].name+"\\n",c.push(o+"_dtor"),r.push(e[n].nb));return u&&(t+="var ret = retType.fromWireType(rv);\\nreturn ret;\\n"),c.push(t+"}\\n"),Xt(c).apply(null,r)}var ee=[],ne=[{},{value:void 0},{value:null},{value:!0},{value:!1}];function re(t){4<t&&0==--ne[t].Hb&&(ne[t]=void 0,ee.push(t))}function ie(t){switch(t){case void 0:return 1;case null:return 2;case!0:return 3;case!1:return 4;default:var e=ee.length?ee.pop():ne.length;return ne[e]={Hb:1,value:t},e}}function oe(t,e,n){switch(e){case 0:return function(t){return this.fromWireType((n?w:S)[t])};case 1:return function(t){return this.fromWireType((n?O:T)[t>>1])};case 2:return function(t){return this.fromWireType((n?I:D)[t>>2])};default:throw new TypeError("Unknown integer type: "+t)}}function ae(t,e){var n=dt[t];return void 0===n&&Dt(e+" has unknown type "+Kt(t)),n}function se(t){if(null===t)return"null";var e=typeof t;return"object"===e||"array"===e||"function"===e?t.toString():""+t}function ue(t,e){switch(e){case 2:return function(t){return this.fromWireType(A[t>>2])};case 3:return function(t){return this.fromWireType(M[t>>3])};default:throw new TypeError("Unknown float type: "+t)}}function ce(t,e,n){switch(e){case 0:return n?function(t){return w[t]}:function(t){return S[t]};case 1:return n?function(t){return O[t>>1]}:function(t){return T[t>>1]};case 2:return n?function(t){return I[t>>2]}:function(t){return D[t>>2]};default:throw new TypeError("Unknown integer type: "+t)}}function fe(t){return t||Dt("Cannot use deleted val. handle = "+t),ne[t].value}function le(t,e){for(var n=Array(t),r=0;r<t;++r)n[r]=ae(I[(e>>2)+r],"parameter "+r);return n}var he={};function pe(t){var e=he[t];return void 0===e?Tt(t):e}var de=[];function ye(t){var e=de.length;return de.push(t),e}var be={};function ve(){return void 0===ve.start&&(ve.start=Date.now()),1e3*(Date.now()-ve.start)|0}function me(){We()}function _e(t){if(2147418112<t)return!1;for(var e=Math.max(R,16777216);e<t;)e=536870912>=e?L(2*e):Math.min(L((3*e+2147483648)/4),2147418112);t=L(e);var n=o.buffer.byteLength;try{var r=-1!==_.grow((t-n)/65536)?o.buffer=_.buffer:null}catch(t){r=null}return!(!r||r.byteLength!=e||(o.buffer=g=r,U(),R=e,0))}function ge(t){if(0===t)return 0;if(t=E(t),!rt.hasOwnProperty(t))return 0;ge.lb&&Fe(ge.lb);var e=F(t=rt[t])+1,n=Pe(e);return n&&B(t,w,n,e),ge.lb=n,ge.lb}function we(){return we.lb||(we.lb=[]),we.lb.push(Ye()),we.lb.length-1}function Se(t){return 0==t%4&&(0!=t%100||0==t%400)}function Oe(t,e){for(var n=0,r=0;r<=e;n+=t[r++]);return n}var Te=[31,29,31,30,31,30,31,31,30,31,30,31],Ie=[31,28,31,30,31,30,31,31,30,31,30,31];function De(t,e){for(t=new Date(t.getTime());0<e;){var n=t.getMonth(),r=(Se(t.getFullYear())?Te:Ie)[n];if(!(e>r-t.getDate())){t.setDate(t.getDate()+e);break}e-=r-t.getDate()+1,t.setDate(1),11>n?t.setMonth(n+1):(t.setMonth(0),t.setFullYear(t.getFullYear()+1))}return t}function Ae(t,e,n,r){function i(t,e,n){for(t="number"==typeof t?t.toString():t||"";t.length<e;)t=n[0]+t;return t}function o(t,e){return i(t,e,"0")}function a(t,e){function n(t){return 0>t?-1:0<t?1:0}var r;return 0===(r=n(t.getFullYear()-e.getFullYear()))&&0===(r=n(t.getMonth()-e.getMonth()))&&(r=n(t.getDate()-e.getDate())),r}function s(t){switch(t.getDay()){case 0:return new Date(t.getFullYear()-1,11,29);case 1:return t;case 2:return new Date(t.getFullYear(),0,3);case 3:return new Date(t.getFullYear(),0,2);case 4:return new Date(t.getFullYear(),0,1);case 5:return new Date(t.getFullYear()-1,11,31);case 6:return new Date(t.getFullYear()-1,11,30)}}function u(t){t=De(new Date(t.jb+1900,0,1),t.Db);var e=s(new Date(t.getFullYear()+1,0,4));return 0>=a(s(new Date(t.getFullYear(),0,4)),t)?0>=a(e,t)?t.getFullYear()+1:t.getFullYear():t.getFullYear()-1}var c=I[r+40>>2];for(var f in r={oc:I[r>>2],nc:I[r+4>>2],Cb:I[r+8>>2],ub:I[r+12>>2],sb:I[r+16>>2],jb:I[r+20>>2],Pb:I[r+24>>2],Db:I[r+28>>2],xc:I[r+32>>2],mc:I[r+36>>2],pc:c?E(c):""},n=E(n),c={"%c":"%a %b %d %H:%M:%S %Y","%D":"%m/%d/%y","%F":"%Y-%m-%d","%h":"%b","%r":"%I:%M:%S %p","%R":"%H:%M","%T":"%H:%M:%S","%x":"%m/%d/%y","%X":"%H:%M:%S"})n=n.replace(new RegExp(f,"g"),c[f]);var l="Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(" "),h="January February March April May June July August September October November December".split(" ");for(f in c={"%a":function(t){return l[t.Pb].substring(0,3)},"%A":function(t){return l[t.Pb]},"%b":function(t){return h[t.sb].substring(0,3)},"%B":function(t){return h[t.sb]},"%C":function(t){return o((t.jb+1900)/100|0,2)},"%d":function(t){return o(t.ub,2)},"%e":function(t){return i(t.ub,2," ")},"%g":function(t){return u(t).toString().substring(2)},"%G":function(t){return u(t)},"%H":function(t){return o(t.Cb,2)},"%I":function(t){return 0==(t=t.Cb)?t=12:12<t&&(t-=12),o(t,2)},"%j":function(t){return o(t.ub+Oe(Se(t.jb+1900)?Te:Ie,t.sb-1),3)},"%m":function(t){return o(t.sb+1,2)},"%M":function(t){return o(t.nc,2)},"%n":function(){return"\\n"},"%p":function(t){return 0<=t.Cb&&12>t.Cb?"AM":"PM"},"%S":function(t){return o(t.oc,2)},"%t":function(){return"\\t"},"%u":function(t){return new Date(t.jb+1900,t.sb+1,t.ub,0,0,0,0).getDay()||7},"%U":function(t){var e=new Date(t.jb+1900,0,1),n=0===e.getDay()?e:De(e,7-e.getDay());return 0>a(n,t=new Date(t.jb+1900,t.sb,t.ub))?o(Math.ceil((31-n.getDate()+(Oe(Se(t.getFullYear())?Te:Ie,t.getMonth()-1)-31)+t.getDate())/7),2):0===a(n,e)?"01":"00"},"%V":function(t){var e=s(new Date(t.jb+1900,0,4)),n=s(new Date(t.jb+1901,0,4)),r=De(new Date(t.jb+1900,0,1),t.Db);return 0>a(r,e)?"53":0>=a(n,r)?"01":o(Math.ceil((e.getFullYear()<t.jb+1900?t.Db+32-e.getDate():t.Db+1-e.getDate())/7),2)},"%w":function(t){return new Date(t.jb+1900,t.sb+1,t.ub,0,0,0,0).getDay()},"%W":function(t){var e=new Date(t.jb,0,1),n=1===e.getDay()?e:De(e,0===e.getDay()?1:7-e.getDay()+1);return 0>a(n,t=new Date(t.jb+1900,t.sb,t.ub))?o(Math.ceil((31-n.getDate()+(Oe(Se(t.getFullYear())?Te:Ie,t.getMonth()-1)-31)+t.getDate())/7),2):0===a(n,e)?"01":"00"},"%y":function(t){return(t.jb+1900).toString().substring(2)},"%Y":function(t){return t.jb+1900},"%z":function(t){var e=0<=(t=t.mc);return t=Math.abs(t)/60,(e?"+":"-")+String("0000"+(t/60*100+t%60)).slice(-4)},"%Z":function(t){return t.pc},"%%":function(){return"%"}})0<=n.indexOf(f)&&(n=n.replace(new RegExp(f,"g"),c[f](r)));return(f=je(n)).length>e?0:(w.set(f,t),f.length-1)}_t=o.InternalError=mt("InternalError");for(var Me=Array(256),ke=0;256>ke;++ke)Me[ke]=String.fromCharCode(ke);function je(t){var e=Array(F(t)+1);return B(t,e,0,e.length),e}Ot=Me,It=o.BindingError=mt("BindingError"),Bt.prototype.isAliasOf=function(t){if(!(this instanceof Bt&&t instanceof Bt))return!1;var e=this.fb.ib.gb,n=this.fb.hb,r=t.fb.ib.gb;for(t=t.fb.hb;e.pb;)n=e.yb(n),e=e.pb;for(;r.pb;)t=r.yb(t),r=r.pb;return e===r&&n===t},Bt.prototype.clone=function(){if(this.fb.hb||kt(this),this.fb.xb)return this.fb.count.value+=1,this;var t=Object.create(Object.getPrototypeOf(this),{fb:{value:Mt(this.fb)}});return t.fb.count.value+=1,t.fb.vb=!1,t},Bt.prototype.delete=function(){if(this.fb.hb||kt(this),this.fb.vb&&!this.fb.xb&&Dt("Object already scheduled for deletion"),--this.fb.count.value,0===this.fb.count.value){var t=this.fb;t.mb?t.ob.qb(t.mb):t.ib.gb.qb(t.hb)}this.fb.xb||(this.fb.mb=void 0,this.fb.hb=void 0)},Bt.prototype.isDeleted=function(){return!this.fb.hb},Bt.prototype.deleteLater=function(){return this.fb.hb||kt(this),this.fb.vb&&!this.fb.xb&&Dt("Object already scheduled for deletion"),xt.push(this),1===xt.length&&jt&&jt(Et),this.fb.vb=!0,this},Gt.prototype.Vb=function(t){return this.Ob&&(t=this.Ob(t)),t},Gt.prototype.Jb=function(t){this.qb&&this.qb(t)},Gt.prototype.argPackAdvance=8,Gt.prototype.readValueFromPointer=ht,Gt.prototype.deleteObject=function(t){null!==t&&t.delete()},Gt.prototype.fromWireType=function(t){function e(){return this.Bb?Ht(this.gb.wb,{ib:this.ec,hb:n,ob:this,mb:t}):Ht(this.gb.wb,{ib:this,hb:t})}var n=this.Vb(t);if(!n)return this.Jb(t),null;var r=Wt(this.gb,n);if(void 0!==r)return 0===r.fb.count.value?(r.fb.hb=n,r.fb.mb=t,r.clone()):(r=r.clone(),this.Jb(t),r);if(r=this.gb.Ub(n),!(r=Ft[r]))return e.call(this);r=this.Ab?r.Qb:r.pointerType;var i=Vt(n,this.gb,r.gb);return null===i?e.call(this):this.Bb?Ht(r.gb.wb,{ib:r,hb:i,ob:this,mb:t}):Ht(r.gb.wb,{ib:r,hb:i})},o.getInheritedInstanceCount=function(){return Object.keys(zt).length},o.getLiveInheritedInstances=function(){var t,e=[];for(t in zt)zt.hasOwnProperty(t)&&e.push(zt[t]);return e},o.flushPendingDeletes=Et,o.setDelayFunction=function(t){jt=t,xt.length&&jt&&jt(Et)},Jt=o.UnboundTypeError=mt("UnboundTypeError"),o.count_emval_handles=function(){for(var t=0,e=5;e<ne.length;++e)void 0!==ne[e]&&++t;return t},o.get_first_emval=function(){for(var t=5;t<ne.length;++t)if(void 0!==ne[t])return ne[t];return null},me=f?function(){var e=t.hrtime();return 1e3*e[0]+e[1]/1e6}:"undefined"!=typeof dateNow?dateNow:"object"==typeof performance&&performance&&"function"==typeof performance.now?function(){return performance.now()}:Date.now;var xe=o.asm({},{t:We,v:function(t,e,n,r){We("Assertion failed: "+E(t)+", at: "+[e?E(e):"unknown filename",n,r?E(r):"unknown function"])},ra:it,e:function(t){return Pe(t)},la:function(){throw k=!0,"Pure virtual function called!"},d:function(t){throw"uncaught_exception"in Ee?Ee.Lb++:Ee.Lb=1,t},ga:function(){return!!Ee.Lb},I:function(){},U:function(){return ot(1),-1},O:ot,ya:function(t,e){st=e;try{var n=E(ut());return(void 0).unlink(n),0}catch(t){return We(t),-t.rb}},xa:function(t,e){st=e;try{var n=ct.Mb();ut();var r=ut(),i=ut(),o=ut();return(void 0).tc(n,r,o),I[i>>2]=n.position,n.Xb&&0===r&&0===o&&(n.Xb=null),0}catch(t){return We(t),-t.rb}},wa:function(t,e){st=e;try{var n=ct.Mb(),r=ut(),i=ut();return ct.rc(n,r,i)}catch(t){return We(t),-t.rb}},T:function(t,e){st=e;try{var n=ut(),r=ut(),i=ut();for(e=t=0;e<i;e++){for(var o=I[r+8*e>>2],a=I[r+(8*e+4)>>2],s=0;s<a;s++){var u=S[o+s],c=at[n];0===u||10===u?((1===n?y:b)(x(c,0)),c.length=0):c.push(u)}t+=a}return t}catch(t){return We(t),-t.rb}},va:function(t,e){return st=e,-12},ua:function(t,e){st=e;try{var n=ut(),r=ut(),i=ut(),o=ut(),a=ut(),s=ut();if(s<<=12,t=!1,-1===a){var u=Le(16384,r);if(!u)return-12;Ue(u,0,r),t=!0}else{var c=(void 0).Wb(a);if(!c)return-9;var f=(void 0).uc(c,S,n,r,s,i,o);u=f.hb,t=f.Ib}return ct.Nb[u]={dc:u,cc:r,Ib:t,fd:a,flags:o},u}catch(t){return We(t),-t.rb}},ta:function(t,e){st=e;try{var n=ut();ut();var r=ut();return ut(),(void 0).sc(n,r),0}catch(t){return We(t),-t.rb}},sa:function(t,e){return st=e,42},H:function(t,e){return st=e,0},S:function(t,e){st=e;try{var n=E(ut()),r=ut(),i=ut();return(void 0).open(n,r,i).fd}catch(t){return We(t),-t.rb}},R:function(t,e){return st=e,0},N:function(t,e){st=e;try{var n=ct.Mb();return(void 0).close(n),0}catch(t){return We(t),-t.rb}},qa:function(t,e){st=e;try{var n=ut(),r=ut(),i=ct.Nb[n];if(!i)return 0;if(r===i.cc){var o=(void 0).Wb(i.fd);ct.qc(n,o,r,i.flags),(void 0).vc(o),ct.Nb[n]=null,i.Ib&&Fe(i.dc)}return 0}catch(t){return We(t),-t.rb}},G:function(){},M:function(t){var e=ft[t];delete ft[t];var n=e.Gb,r=e.qb,i=e.Kb;wt([t],i.map(function(t){return t.$b}).concat(i.map(function(t){return t.jc})),function(t){var o={};return i.forEach(function(e,n){var r=t[n],a=e.Yb,s=e.Zb,u=t[n+i.length],c=e.ic,f=e.kc;o[e.Sb]={read:function(t){return r.fromWireType(a(s,t))},write:function(t,e){var n=[];c(f,t,u.toWireType(n,e)),lt(n)}}}),[{name:e.name,fromWireType:function(t){var e,n={};for(e in o)n[e]=o[e].read(t);return r(t),n},toWireType:function(t,e){for(var i in o)if(!(i in e))throw new TypeError("Missing field");var a=n();for(i in o)o[i].write(a,e[i]);return null!==t&&t.push(r,a),a},argPackAdvance:8,readValueFromPointer:ht,nb:r}]})},pa:function(t,e,n,r,i){var o=St(n);At(t,{name:e=Tt(e),fromWireType:function(t){return!!t},toWireType:function(t,e){return e?r:i},argPackAdvance:8,readValueFromPointer:function(t){if(1===n)var r=w;else if(2===n)r=O;else{if(4!==n)throw new TypeError("Unknown boolean type size: "+e);r=I}return this.fromWireType(r[t>>o])},nb:null})},s:function(t,e,n,r,i,o,a,s,u,c,f,l,h){f=Tt(f),o=Zt(i,o),s&&(s=Zt(a,s)),c&&(c=Zt(u,c)),h=Zt(l,h);var p=bt(f);Lt(p,function(){$t("Cannot construct "+f+" due to unbound types",[r])}),wt([t,e,n],r?[r]:[],function(e){if(e=e[0],r)var n=e.gb,i=n.wb;else i=Bt.prototype;e=vt(p,function(){if(Object.getPrototypeOf(this)!==a)throw new It("Use \'new\' to construct "+f);if(void 0===u.tb)throw new It(f+" has no accessible constructor");var t=u.tb[arguments.length];if(void 0===t)throw new It("Tried to invoke ctor of "+f+" with invalid number of parameters ("+arguments.length+") - expected ("+Object.keys(u.tb).toString()+") parameters instead!");return t.apply(this,arguments)});var a=Object.create(i,{constructor:{value:e}});e.prototype=a;var u=new Ut(f,e,a,h,n,o,s,c);n=new Gt(f,u,!0,!1,!1),i=new Gt(f+"*",u,!1,!1,!1);var l=new Gt(f+" const*",u,!1,!0,!1);return Ft[t]={pointerType:i,Qb:l},qt(p,e),[n,i,l]})},y:function(t,e,n,r,i,o){var a=Qt(e,n);i=Zt(r,i),wt([],[t],function(t){var n="constructor "+(t=t[0]).name;if(void 0===t.gb.tb&&(t.gb.tb=[]),void 0!==t.gb.tb[e-1])throw new It("Cannot register multiple constructors with identical number of parameters ("+(e-1)+") for class \'"+t.name+"\'! Overload resolution is currently only performed using the parameter count, not actual type info!");return t.gb.tb[e-1]=function(){$t("Cannot construct "+t.name+" due to unbound types",a)},wt([],a,function(r){return t.gb.tb[e-1]=function(){arguments.length!==e-1&&Dt(n+" called with "+arguments.length+" arguments, expected "+(e-1));var t=[],a=Array(e);a[0]=o;for(var s=1;s<e;++s)a[s]=r[s].toWireType(t,arguments[s-1]);return a=i.apply(null,a),lt(t),r[0].fromWireType(a)},[]}),[]})},g:function(t,e,n,r,i,o,a,s){var u=Qt(n,r);e=Tt(e),o=Zt(i,o),wt([],[t],function(t){function r(){$t("Cannot call "+i+" due to unbound types",u)}var i=(t=t[0]).name+"."+e;s&&t.gb.fc.push(e);var c=t.gb.wb,f=c[e];return void 0===f||void 0===f.kb&&f.className!==t.name&&f.zb===n-2?(r.zb=n-2,r.className=t.name,c[e]=r):(Pt(c,e,i),c[e].kb[n-2]=r),wt([],u,function(r){return r=te(i,r,t,o,a),void 0===c[e].kb?(r.zb=n-2,c[e]=r):c[e].kb[n-2]=r,[]}),[]})},oa:function(t,e){At(t,{name:e=Tt(e),fromWireType:function(t){var e=ne[t].value;return re(t),e},toWireType:function(t,e){return ie(e)},argPackAdvance:8,readValueFromPointer:ht,nb:null})},na:function(t,e,n,r){function i(){}n=St(n),e=Tt(e),i.values={},At(t,{name:e,constructor:i,fromWireType:function(t){return this.constructor.values[t]},toWireType:function(t,e){return e.value},argPackAdvance:8,readValueFromPointer:oe(e,n,r),nb:null}),Lt(e,i)},r:function(t,e,n){var r=ae(t,"enum");e=Tt(e),t=r.constructor,r=Object.create(r.constructor.prototype,{value:{value:n},constructor:{value:vt(r.name+"_"+e,function(){})}}),t.values[n]=r,t[e]=r},Q:function(t,e,n){n=St(n),At(t,{name:e=Tt(e),fromWireType:function(t){return t},toWireType:function(t,e){if("number"!=typeof e&&"boolean"!=typeof e)throw new TypeError(\'Cannot convert "\'+se(e)+\'" to \'+this.name);return e},argPackAdvance:8,readValueFromPointer:ue(e,n),nb:null})},x:function(t,e,n,r,i,o){var a=Qt(e,n);t=Tt(t),i=Zt(r,i),Lt(t,function(){$t("Cannot call "+t+" due to unbound types",a)},e-1),wt([],a,function(n){return n=[n[0],null].concat(n.slice(1)),qt(t,te(t,n,null,i,o),e-1),[]})},B:function(t,e,n,r,i){function o(t){return t}e=Tt(e),-1===i&&(i=4294967295);var a=St(n);if(0===r){var s=32-8*n;o=function(t){return t<<s>>>s}}var u=-1!=e.indexOf("unsigned");At(t,{name:e,fromWireType:o,toWireType:function(t,n){if("number"!=typeof n&&"boolean"!=typeof n)throw new TypeError(\'Cannot convert "\'+se(n)+\'" to \'+this.name);if(n<r||n>i)throw new TypeError(\'Passing a number "\'+se(n)+\'" from JS side to C/C++ side to an argument of type "\'+e+\'", which is outside the valid range [\'+r+", "+i+"]!");return u?n>>>0:0|n},argPackAdvance:8,readValueFromPointer:ce(e,a,0!==r),nb:null})},w:function(t,e,n){function r(t){t>>=2;var e=D;return new i(e.buffer,e[t+1],e[t])}var i=[Int8Array,Uint8Array,Int16Array,Uint16Array,Int32Array,Uint32Array,Float32Array,Float64Array][e];At(t,{name:n=Tt(n),fromWireType:r,argPackAdvance:8,readValueFromPointer:r},{ac:!0})},z:function(t,e,n,r,i,o,a,s,u,c,f,l){n=Tt(n),o=Zt(i,o),s=Zt(a,s),c=Zt(u,c),l=Zt(f,l),wt([t],[e],function(t){return t=t[0],[new Gt(n,t.gb,!1,!1,!0,t,r,o,s,c,l)]})},P:function(t,e){var n="std::string"===(e=Tt(e));At(t,{name:e,fromWireType:function(t){var e=D[t>>2];if(n){var r=S[t+4+e],i=0;0!=r&&(i=r,S[t+4+e]=0);var o=t+4;for(r=0;r<=e;++r){var a=t+4+r;if(0==S[a]){if(o=E(o),void 0===s)var s=o;else s+=String.fromCharCode(0),s+=o;o=a+1}}0!=i&&(S[t+4+e]=i)}else{for(s=Array(e),r=0;r<e;++r)s[r]=String.fromCharCode(S[t+4+r]);s=s.join("")}return Fe(t),s},toWireType:function(t,e){e instanceof ArrayBuffer&&(e=new Uint8Array(e));var r="string"==typeof e;r||e instanceof Uint8Array||e instanceof Uint8ClampedArray||e instanceof Int8Array||Dt("Cannot pass non-string to std::string");var i=(n&&r?function(){return F(e)}:function(){return e.length})(),o=Pe(4+i+1);if(D[o>>2]=i,n&&r)B(e,S,o+4,i+1);else if(r)for(r=0;r<i;++r){var a=e.charCodeAt(r);255<a&&(Fe(o),Dt("String has UTF-16 code units that do not fit in 8 bits")),S[o+4+r]=a}else for(r=0;r<i;++r)S[o+4+r]=e[r];return null!==t&&t.push(Fe,o),o},argPackAdvance:8,readValueFromPointer:ht,nb:function(t){Fe(t)}})},ma:function(t,e,n){if(n=Tt(n),2===e)var r=function(){return T},i=1;else 4===e&&(r=function(){return D},i=2);At(t,{name:n,fromWireType:function(t){for(var e=r(),n=D[t>>2],o=Array(n),a=t+4>>i,s=0;s<n;++s)o[s]=String.fromCharCode(e[a+s]);return Fe(t),o.join("")},toWireType:function(t,n){var o=r(),a=n.length,s=Pe(4+a*e);D[s>>2]=a;for(var u=s+4>>i,c=0;c<a;++c)o[u+c]=n.charCodeAt(c);return null!==t&&t.push(Fe,s),s},argPackAdvance:8,readValueFromPointer:ht,nb:function(t){Fe(t)}})},L:function(t,e,n,r,i,o){ft[t]={name:Tt(e),Gb:Zt(n,r),qb:Zt(i,o),Kb:[]}},A:function(t,e,n,r,i,o,a,s,u,c){ft[t].Kb.push({Sb:Tt(e),$b:n,Yb:Zt(r,i),Zb:o,jc:a,ic:Zt(s,u),kc:c})},ka:function(t,e){At(t,{bc:!0,name:e=Tt(e),argPackAdvance:0,fromWireType:function(){},toWireType:function(){}})},k:function(t,e,n){t=fe(t),e=ae(e,"emval::as");var r=[],i=ie(r);return I[n>>2]=i,e.toWireType(r,t)},E:function(t,e,n,r){t=fe(t),n=le(e,n);for(var i=Array(e),o=0;o<e;++o){var a=n[o];i[o]=a.readValueFromPointer(r),r+=a.argPackAdvance}return ie(t=t.apply(void 0,i))},o:function(t,e,n,r,i){t=de[t],e=fe(e),n=pe(n);var o=[];return I[r>>2]=ie(o),t(e,n,o,i)},u:function(t,e,n,r){(t=de[t])(e=fe(e),n=pe(n),null,r)},c:re,p:function(t){return 0===t?ie(Function("return this")()):(t=pe(t),ie(Function("return this")()[t]))},l:function(t,e){for(var n=(e=le(t,e))[0],r=n.name+"_$"+e.slice(1).map(function(t){return t.name}).join("_")+"$",i=["retType"],o=[n],a="",s=0;s<t-1;++s)a+=(0!==s?", ":"")+"arg"+s,i.push("argType"+s),o.push(e[1+s]);r="return function "+bt("methodCaller_"+r)+"(handle, name, destructors, args) {\\n";var u=0;for(s=0;s<t-1;++s)r+="    var arg"+s+" = argType"+s+".readValueFromPointer(args"+(u?"+"+u:"")+");\\n",u+=e[s+1].argPackAdvance;for(r+="    var rv = handle[name]("+a+");\\n",s=0;s<t-1;++s)e[s+1].deleteObject&&(r+="    argType"+s+".deleteObject(arg"+s+");\\n");return n.bc||(r+="    return retType.toWireType(destructors, rv);\\n"),i.push(r+"};\\n"),ye(t=Xt(i).apply(null,o))},F:function(t){return t=pe(t),ie(o[t])},i:function(t,e){return ie((t=fe(t))[e=fe(e)])},f:function(t){4<t&&(ne[t].Hb+=1)},ja:function(t,e){return(t=fe(t))instanceof fe(e)},q:function(t,e,n,r){t=fe(t);var i=be[e];if(!i){i="";for(var a=0;a<e;++a)i+=(0!==a?", ":"")+"arg"+a;var s="return function emval_allocator_"+e+"(constructor, argTypes, args) {\\n";for(a=0;a<e;++a)s+="var argType"+a+" = requireRegisteredType(Module[\'HEAP32\'][(argTypes >> 2) + "+a+\'], "parameter \'+a+\'");\\nvar arg\'+a+" = argType"+a+".readValueFromPointer(args);\\nargs += argType"+a+"[\'argPackAdvance\'];\\n";i=new Function("requireRegisteredType","Module","__emval_register",s+"var obj = new constructor("+i+");\\nreturn __emval_register(obj);\\n}\\n")(ae,o,ie),be[e]=i}return i(t,n,r)},j:function(t){return ie(pe(t))},K:function(){return ie({})},h:function(t){lt(ne[t].value),re(t)},ia:function(t,e,n){t=fe(t),e=fe(e),n=fe(n),t[e]=n},m:function(t,e){return ie(t=(t=ae(t,"_emval_take_value")).readValueFromPointer(e))},ha:function(t){return ie(typeof(t=fe(t)))},b:function(){o.abort()},fa:ve,J:function(t,e){if(0===t)t=Date.now();else{if(1!==t||!(f||"undefined"!=typeof dateNow||"object"==typeof performance&&performance&&"function"==typeof performance.now))return ot(22),-1;t=me()}return I[e>>2]=t/1e3|0,I[e+4>>2]=t%1e3*1e6|0,0},ea:function(t){return nt[t]()},da:function(){return R},ca:function(t,e,n){S.set(S.subarray(e,e+n),t)},ba:_e,n:ge,aa:function(){return 16384},$:function(t){var e=Date.now();return I[t>>2]=e/1e3|0,I[t+4>>2]=e%1e3*1e3|0,0},D:function(t){var e=we.lb[t];we.lb.splice(t,1),Ne(e)},C:we,_:function(){We("trap!")},Z:function(t,e){var n=I[t>>2];if(t=I[t+4>>2],0!==e&&(I[e>>2]=0,I[e+4>>2]=0),e=(1e6*n+t/1e3)/1e3,(u||c)&&self.performance&&self.performance.now)for(n=self.performance.now();self.performance.now()-n<e;);else for(n=Date.now();Date.now()-n<e;);return 0},Y:function(){return 0},X:function(){return ot(38),-1},W:function(t,e,n,r){return Ae(t,e,n,r)},V:function(){We("OOM")},a:C},g);o.asm=xe;var Ee=o.__ZSt18uncaught_exceptionv=function(){return o.asm.za.apply(null,arguments)},Be=o.___getTypeName=function(){return o.asm.Aa.apply(null,arguments)},Fe=o._free=function(){return o.asm.Ba.apply(null,arguments)};o._main=function(){return o.asm.Ca.apply(null,arguments)};var Pe=o._malloc=function(){return o.asm.Da.apply(null,arguments)},Le=o._memalign=function(){return o.asm.Ea.apply(null,arguments)},Ue=o._memset=function(){return o.asm.Fa.apply(null,arguments)},Ce=o.globalCtors=function(){return o.asm.bb.apply(null,arguments)},Re=o.stackAlloc=function(){return o.asm.cb.apply(null,arguments)},Ne=o.stackRestore=function(){return o.asm.db.apply(null,arguments)},Ye=o.stackSave=function(){return o.asm.eb.apply(null,arguments)};function Ve(t){this.name="ExitStatus",this.message="Program terminated with exit("+t+")",this.status=t}function ze(t){function e(){if(!o.calledRun&&(o.calledRun=!0,!k)){if(H||(H=!0,N(V)),N(z),o.onRuntimeInitialized&&o.onRuntimeInitialized(),o._main&&He&&o.callMain(t),o.postRun)for("function"==typeof o.postRun&&(o.postRun=[o.postRun]);o.postRun.length;){var e=o.postRun.shift();W.unshift(e)}N(W)}}if(t=t||o.arguments,!(0<q)){if(o.preRun)for("function"==typeof o.preRun&&(o.preRun=[o.preRun]);o.preRun.length;)G();N(Y),0<q||o.calledRun||(o.setStatus?(o.setStatus("Running..."),setTimeout(function(){setTimeout(function(){o.setStatus("")},1),e()},1)):e())}}function We(t){throw o.onAbort&&o.onAbort(t),void 0!==t?(y(t),b(t),t=JSON.stringify(t)):t="",k=!0,"abort("+t+"). Build with -s ASSERTIONS=1 for more info."}if(o.dynCall_i=function(){return o.asm.Ga.apply(null,arguments)},o.dynCall_ii=function(){return o.asm.Ha.apply(null,arguments)},o.dynCall_iii=function(){return o.asm.Ia.apply(null,arguments)},o.dynCall_iiii=function(){return o.asm.Ja.apply(null,arguments)},o.dynCall_iiiii=function(){return o.asm.Ka.apply(null,arguments)},o.dynCall_iiiiid=function(){return o.asm.La.apply(null,arguments)},o.dynCall_iiiiii=function(){return o.asm.Ma.apply(null,arguments)},o.dynCall_iiiiiid=function(){return o.asm.Na.apply(null,arguments)},o.dynCall_iiiiiii=function(){return o.asm.Oa.apply(null,arguments)},o.dynCall_iiiiiiii=function(){return o.asm.Pa.apply(null,arguments)},o.dynCall_iiiiiiiii=function(){return o.asm.Qa.apply(null,arguments)},o.dynCall_iiiiiiiiiiii=function(){return o.asm.Ra.apply(null,arguments)},o.dynCall_iiiiij=function(){return o.asm.Sa.apply(null,arguments)},o.dynCall_v=function(){return o.asm.Ta.apply(null,arguments)},o.dynCall_vi=function(){return o.asm.Ua.apply(null,arguments)},o.dynCall_vii=function(){return o.asm.Va.apply(null,arguments)},o.dynCall_viii=function(){return o.asm.Wa.apply(null,arguments)},o.dynCall_viiii=function(){return o.asm.Xa.apply(null,arguments)},o.dynCall_viiiii=function(){return o.asm.Ya.apply(null,arguments)},o.dynCall_viiiiii=function(){return o.asm.Za.apply(null,arguments)},o.dynCall_viiiiiii=function(){return o.asm._a.apply(null,arguments)},o.dynCall_viiiiiiiiiii=function(){return o.asm.$a.apply(null,arguments)},o.dynCall_viijii=function(){return o.asm.ab.apply(null,arguments)},o.asm=xe,o.then=function(t){if(o.calledRun)t(o);else{var e=o.onRuntimeInitialized;o.onRuntimeInitialized=function(){e&&e(),t(o)}}return o},Ve.prototype=Error(),Ve.prototype.constructor=Ve,J=function t(){o.calledRun||ze(),o.calledRun||(J=t)},o.callMain=function(t){t=t||[],H||(H=!0,N(V));var e=t.length+1,n=Re(4*(e+1));I[n>>2]=P(o.thisProgram);for(var r=1;r<e;r++)I[(n>>2)+r]=P(t[r-1]);I[(n>>2)+e]=0;try{var i=o._main(e,n,0);o.noExitRuntime&&0===i||(!o.noExitRuntime&&(k=!0,o.onExit)&&o.onExit(i),o.quit(i,new Ve(i)))}catch(e){e instanceof Ve||("SimulateInfiniteLoop"==e?o.noExitRuntime=!0:((t=e)&&"object"==typeof e&&e.stack&&(t=[e,e.stack]),b("exception thrown: "+t),o.quit(1,e)))}},o.run=ze,o.abort=We,o.preInit)for("function"==typeof o.preInit&&(o.preInit=[o.preInit]);0<o.preInit.length;)o.preInit.pop()();var He=!0;return o.noInitialRun&&(He=!1),o.noExitRuntime=!0,ze(),e});e.a=o}).call(this,n(35),"/")},function(t,e,n){"use strict";(function(t,r){n(74),n(32),n(51),n(98);var i=n(13),o=n(65),a=n(23),s=n(66),u=n(20),c=n(54),f=n(1),l=n(56),h=n(9),p=n(6),d=n(4),y=n(2),b=n(33),v=n(24),m=n.n(v);void 0===t.performance&&(t.performance={now:Date.now}),"undefined"!=typeof self&&void 0===self.performance&&(self.performance={now:Date.now}),e.a=function(t){let e=t,n=new o.a;function v(t,e,n){return t+=e,n&&(t%=n),t}let _=[];function g(t){-1!==_.indexOf(t)&&(t._process(),_=_.filter(e=>e!==t))}function w(t){_=_.filter(e=>e!==t)}function S(t,n,r,i,o,a,s,u,c,f){if(f)for(let l of t)r=e.make_table(n,r,l,i,s,a||4294967295,o,u,c,f),s=v(s,l.cdata[0].length,a);else r=e.make_table(n,r,t,i,s,a||4294967295,o,u,c,f),s=v(s,t.row_count,a);return u||c?function(t){-1===_.indexOf(t)&&(_.push(t),setTimeout(()=>g(t)))}(n):n._process(),[r,s]}function O(t){let e=l.a.from([new Uint8Array(t)]),n=e.schema.fields.reduce((t,n,r)=>t.loadColumn(n,e.getColumnAt(r)),new T),r=n.cdata[0].chunks.length,i=[];for(let t=0;t<r;t++)i.push({row_count:n.cdata[0].chunks[t].length,is_arrow:!0,names:n.names,types:n.types,cdata:n.cdata.map(e=>e.chunks[t])});return i}class T extends h.a{constructor(t,e,n){super(),this.cdata=t||[],this.names=e||[],this.types=n||[]}loadColumn(t,e){return this.visit(t.type)&&(this.cdata.push(e),this.names.push(t.name)),this}visitBool(){return this.types.push(e.t_dtype.DTYPE_BOOL),!0}visitInt(t){const n=t.bitWidth;return 64===n?this.types.push(e.t_dtype.DTYPE_INT64):32===n?this.types.push(e.t_dtype.DTYPE_INT32):16===n?this.types.push(e.t_dtype.DTYPE_INT16):8===n&&this.types.push(e.t_dtype.DTYPE_INT8),!0}visitFloat(t){const n=t.precision;return n===f.f.DOUBLE?this.types.push(e.t_dtype.DTYPE_FLOAT64):n===f.f.SINGLE&&this.types.push(e.t_dtype.DTYPE_FLOAT32),!0}visitUtf8(){return this.types.push(e.t_dtype.DTYPE_STR),!0}visitBinary(){return this.types.push(e.t_dtype.DTYPE_STR),!0}visitTimestamp(){return this.types.push(e.t_dtype.DTYPE_TIME),!0}visitDictionary(t){return this.visit(t.dictionary)}}function I(t,n,r,o,s,c,f){this._View=void 0,this.date_parser=new a.a,this.config=o||{},0===n?this._View=e.make_view_zero(t,r,s,i.COLUMN_SEPARATOR_STRING,this.config,this.date_parser):1===n?this._View=e.make_view_one(t,r,s,i.COLUMN_SEPARATOR_STRING,this.config,this.date_parser):2===n&&(this._View=e.make_view_two(t,r,s,i.COLUMN_SEPARATOR_STRING,this.config,this.date_parser)),this.pool=t,this.ctx=this._View.get_context(),this.column_only=this._View.is_column_only(),this.callbacks=c,this.name=s,this.table=f,Object(u.a)(this)}function D(t){let n=[];for(let r=0;r<t.size();r++)n.push(e.scalar_vec_to_string(t,r));return t.delete(),n}I.prototype.get_config=function(){return JSON.parse(JSON.stringify(this.config))},I.prototype.delete=function(){w(this.pool),this._View.delete(),this.ctx.delete(),this.table.views.splice(this.table.views.indexOf(this),1),this.table=void 0;let t=0,e=0;for(;t<this.callbacks.length;){let n=this.callbacks[t];n.view!==this&&(this.callbacks[e++]=n),t++}this.callbacks.length=e,this._delete_callback&&this._delete_callback()},I.prototype.sides=function(){return this._View.sides()},I.prototype._num_hidden=function(){let t=0;for(const e of this.config.sort)-1===this.config.columns.indexOf(e[0])&&t++;return t};const A=function(t){let e=[];for(let n=0;n<t.size();n++){let r=t.get(n);e.push(D(r))}return t.delete(),e};I.prototype.schema=function(){return Object(s.a)(this._View.schema())},I.prototype._column_names=function(t=!1,e=0){return A(this._View.column_names(t,e)).map(t=>t.join(i.COLUMN_SEPARATOR_STRING))},I.prototype.get_data_slice=function(t,n,r,i){const o=this.sides();return e["get_data_slice_".concat(["zero","one","two"][o])](this._View,t,n,r,i)};const M=function(t,n){g(this.pool),t=t||{};const r=this._View.num_columns()+(0===this.sides()?0:1),o=this._View.num_rows(),a=this._num_hidden(),s=this.config.viewport?this.config.viewport:{},u=t.start_row||(s.top?s.top:0),c=t.end_row||(s.height?u+s.height:o),f=t.start_col||(s.left?s.left:0),l=Math.min(r,(t.end_col||(s.width?f+s.width:r))*(a+1)),h=this.sides(),p=["zero","one","two"][h],d=this.get_data_slice(u,c,f,l),y=d.get_column_names(),b=A(y).map(t=>t.join(i.COLUMN_SEPARATOR_STRING));let v=n.initDataValue();for(let t=u;t<c;t++){let r=n.initRowValue();for(let i=f;i<l;i++){const o=b[i];if(!((i-(h>0?1:0))%(this.config.columns.length+a)>=this.config.columns.length))if(i===f&&0!==h){if(!this.column_only){const i=d.get_row_path(t);n.initColumnValue(v,r,"__ROW_PATH__");for(let t=0;t<i.size();t++){const o=e.scalar_vec_to_val(i,t);n.addColumnValue(v,r,"__ROW_PATH__",o)}i.delete()}}else{const a=e["get_from_data_slice_".concat(p)](d,t,i);n.setColumnValue(v,r,o,a)}}n.addRow(v,r)}return d.delete(),n.formatData(v,t.config)};function k(t,e,n,r,i,o){this.gnode=t,this.pool=e,this.name=Math.random()+"",this.initialized=!1,this.index=n,this.pool.set_update_delegate(this),this.computed=r||[],this.callbacks=[],this.views=[],this.limit=i,this.limit_index=o,Object(u.a)(this)}let j;I.prototype.to_columns=function(t){return M.call(this,t,b.a.jsonTableFormatter)},I.prototype.to_json=function(t){return M.call(this,t,b.a.jsonFormatter)},I.prototype.to_csv=function(t){return M.call(this,t,b.a.csvFormatter)},I.prototype.col_to_js_typed_array=function(t,n={}){const r=e.col_to_js_typed_array;return function(t,e,n){const r=this.num_rows(),i=e.start_row||0,o=e.end_row||r;let a,s,u=this._column_names().indexOf(t);if(-1===u)return;this.sides()>0&&u++;const c=n(a=e.data_slice?e.data_slice.get_column_slice(u):(s=this.get_data_slice(i,o,u,u+1)).get_slice(),this._View.get_column_dtype(u),u);return a.delete(),s&&s.delete(),c}.call(this,t,n,r)},I.prototype.to_arrow=function(t={}){const e=this._column_names(),n=this.schema(),r=[],o=t.start_col||0,a=t.end_col||e.length;for(let s=o;s<a;s++){const o=e[s],a=o.split(i.COLUMN_SEPARATOR_STRING),u=n[a[a.length-1]];if("float"===u){const[e,n,i]=this.col_to_js_typed_array(o,t);r.push(d.a.new(p.a.Float(new y.n,0,e.length,n,i,e)))}else if("integer"===u){const[e,n,i]=this.col_to_js_typed_array(o,t);r.push(d.a.new(p.a.Int(new y.q,0,e.length,n,i,e)))}else if("boolean"===u){const[e,n,i]=this.col_to_js_typed_array(o,t);r.push(d.a.new(p.a.Bool(new y.b,0,e.length,n,i,e)))}else if("date"===u||"datetime"===u){const[e,n,i]=this.col_to_js_typed_array(o,t);r.push(d.a.new(p.a.Timestamp(new y.A,0,e.length,n,i,e)))}else{if("string"!==u)throw new Error("Type ".concat(u," not supported"));{const[e,n,i,a,s]=this.col_to_js_typed_array(o,t),u=d.a.new(p.a.Utf8(new y.G,0,n.length-1,0,null,n,e)),c=new y.h(u.type,new y.C,null,null,u);r.push(d.a.new(p.a.Dictionary(c,0,i.length,a,s,i)))}}}return l.a.fromVectors(r,e.slice(o,a)).serialize("binary",!1).buffer},I.prototype.num_rows=function(){return this._View.num_rows()},I.prototype.num_columns=function(){const t=this._View.num_columns(),e=this._num_hidden();return t-t/(this.config.columns.length+e)*e},I.prototype.get_row_expanded=function(t){return this._View.get_row_expanded(t)},I.prototype.expand=function(t){return this._View.expand(t,this.config.row_pivots.length)},I.prototype.collapse=function(t){return this._View.collapse(t)},I.prototype.set_depth=function(t){return this._View.set_depth(t,this.config.row_pivots.length)},I.prototype._get_step_delta=async function(){let t,e=this._View.get_step_delta(0,2147483647);if(0===e.cells.size())t=this.to_json();else{let n={};for(let t=0;t<e.cells.size();t++)n[e.cells.get(t).row]=!0;const r=(n=Object.keys(n)).map(t=>this.to_json({start_row:Number.parseInt(t),end_row:Number.parseInt(t)+1}));t=[].concat.apply([],r)}return e.cells.delete(),t},I.prototype._get_row_delta=async function(){let t=this._View.get_row_delta(),e=this.to_arrow({data_slice:t});return t.delete(),e},I.prototype.on_update=function(t,{mode:e="none"}={}){if(g(this.pool),-1===["none","cell","row"].indexOf(e))throw new Error(\'Invalid update mode "\'.concat(e,\'" - valid modes are "none", "cell" and "row".\'));"cell"!==e&&"row"!==e||this._View._get_deltas_enabled()||this._View._set_deltas_enabled(!0),this.callbacks.push({view:this,orig_callback:t,callback:async n=>{switch(e){case"cell":void 0===n.step_delta&&(n.step_delta=await this._get_step_delta()),t(n.step_delta);break;case"row":void 0===n.row_delta&&(n.row_delta=await this._get_row_delta()),t(n.row_delta);break;default:t()}}})},I.prototype.remove_update=function(t){g(this.pool);const e=this.callbacks.length;!function(t,e){let n=0,r=0;for(;n<t.length;){const i=t[n];e(i)&&(t[r++]=i),n++}t.length=r}(this.callbacks,e=>e.orig_callback!==t),console.assert(e>this.callbacks.length,\'"callback" does not match a registered updater\')},I.prototype.on_delete=function(t){this._delete_callback=t},I.prototype.remove_delete=function(){this._delete_callback=void 0},k.prototype._update_callback=function(){let t={};for(let e in this.callbacks)this.callbacks[e].callback(t)},k.prototype.clear=function(){w(this.pool),this.gnode.reset()},k.prototype.replace=function(t){w(this.pool),this.gnode.reset(),this.update(t),g(this.pool)},k.prototype.delete=function(){if(this.views.length>0)throw"Table still has contexts - refusing to delete.";w(this.pool),this.pool.unregister_gnode(this.gnode.get_id()),this.gnode.delete(),this.pool.delete(),this._delete_callback&&this._delete_callback()},k.prototype.on_delete=function(t){this._delete_callback=t},k.prototype.size=function(){return this.gnode.get_table().size()},k.prototype._schema=function(t){let e=this.gnode.get_tblschema(),n=e.columns(),r=e.types(),i={};const o=this.computed_schema();for(let e=0;e<n.size();e++){const a=n.get(e);("psp_okey"!==a||void 0!==o[a]&&!t)&&(i[a]=Object(u.b)(r.get(e).value))}return e.delete(),n.delete(),r.delete(),i},k.prototype.schema=function(t=!1){return this._schema(t)},k.prototype._computed_schema=function(){if(this.computed.length<0)return{};const t={};for(let e=0;e<this.computed.length;e++){const n=this.computed[e].column,r=this.computed[e].type,i={};i.type=r,i.input_columns=this.computed[e].inputs,i.input_type=this.computed[e].input_type,i.computation=this.computed[e].computation,t[n]=i}return t},k.prototype.computed_schema=function(){return this._computed_schema()},k.prototype._is_date_filter=function(t){return e=>"datetime"===t[e]||"date"===t[e]},k.prototype._is_valid_filter=function(t){const e=this._schema();return null!=(this._is_date_filter(e)(t[0])?(new a.a).parse(t[2]):t[2])},k.prototype.is_valid_filter=function(t){return this._is_valid_filter(t)},k.prototype.view=function(t={}){g(this.pool);let e={};for(const n of Object.keys(t))if(i.CONFIG_ALIASES[n]){if(e[i.CONFIG_ALIASES[n]])throw new Error(\'Duplicate configuration parameter "\'.concat(n,\'"\'));console.warn(\'Deprecated: "\'.concat(n,\'" config parameter, please use "\').concat(i.CONFIG_ALIASES[n],\'" instead\')),e[i.CONFIG_ALIASES[n]]=t[n]}else if("aggregate"===n){console.warn(\'Deprecated: "aggregate" config parameter has been replaced by "aggregates" and "columns"\'),e.aggregates={},e.columns=[];for(const n of t.aggregate)e.aggregates[n.column]=n.op,e.columns.push(n.column)}else{if(!(i.CONFIG_VALID_KEYS.indexOf(n)>-1))throw new Error(\'Unrecognized config parameter "\'.concat(n,\'"\'));e[n]=t[n]}e.row_pivots=e.row_pivots||[],e.column_pivots=e.column_pivots||[],e.aggregates=e.aggregates||{},e.filter=e.filter||[],e.sort=e.sort||[],void 0===e.columns&&(e.columns=this._columns(!0));let n,r=Math.random()+"";n=e.row_pivots.length>0||e.column_pivots.length>0?e.column_pivots&&e.column_pivots.length>0?2:1:0;let o=new I(this.pool,n,this.gnode,e,r,this.callbacks,this);return this.views.push(o),o},k.prototype.update=function(t){let r,i=this._columns(),o=this.gnode.get_tblschema(),a=o.types(),s=!1;if(r=n,t instanceof ArrayBuffer){if(0===this.size())throw new Error("Overriding Arrow Schema is not supported.");r=O(t),j&&j(r.map(t=>t.row_count).reduce((t,e)=>t+e)),s=!0}else"string"==typeof t?(","===t[0]&&(t="_"+t),n.init(e,m.a.parse(t.trim(),{header:!0}).data),n.names=i,n.types=n.extract_typevec(a).slice(0,i.length),j&&j(n.row_count)):(n.init(e,t),n.names=i,n.types=n.extract_typevec(a).slice(0,i.length),j&&j(n.row_count));if(0!==n.row_count)try{[,this.limit_index]=S(r,this.pool,this.gnode,this.computed,this.index||"",this.limit,this.limit_index,!0,!1,s),this.initialized=!0}catch(t){console.error("Update failed: ".concat(t))}finally{o.delete(),a.delete()}else console.warn("table.update called with no data - ignoring")},k.prototype.remove=function(t){let r,i=this._columns(),o=this.gnode.get_tblschema(),a=o.types(),s=!1;(t=t.map(t=>({[this.index]:t})))instanceof ArrayBuffer?(r=O(t,this.index),s=!0):(n.init(e,t),n.names=[this.index],n.types=[n.extract_typevec(a)[i.indexOf(this.index)]],r=n);try{[,this.limit_index]=S(r,this.pool,this.gnode,void 0,this.index||"",this.limit,this.limit_index,!1,!0,s),this.initialized=!0}catch(t){console.error("Remove failed",t)}finally{o.delete(),a.delete()}},k.prototype.add_computed=function(t){let n,r;try{return n=new e.t_pool,r=e.clone_gnode_table(n,this.gnode,t),this.computed.length>0&&(t=this.computed.concat(t)),new k(r,n,this.index,t,this.limit,this.limit_index)}catch(t){throw n&&n.delete(),r&&r.delete(),t}},k.prototype._columns=function(t=!1){let e=this.gnode.get_tblschema(),n=this._computed_schema(),r=e.columns(),i=[];for(let e=0;e<r.size();e++){let o=r.get(e);"psp_okey"===o||void 0!==n[o]&&!t||i.push(o)}return e.delete(),r.delete(),i},k.prototype.columns=function(t=!1){return this._columns(t)},k.prototype._column_metadata=function(){let t=this.gnode.get_tblschema(),e=this._computed_schema(),n=t.columns(),r=t.types(),i=[];for(let t=0;t<n.size();t++){let o=n.get(t),a={};if("psp_okey"===o)continue;a.name=o,a.type=Object(u.b)(r.get(t).value);let s=e[o];a.computed=void 0!==s?{input_columns:s.input_columns,input_type:s.input_type,computation:s.computation}:void 0,i.push(a)}return r.delete(),n.delete(),t.delete(),i},k.prototype.column_metadata=function(){return this._column_metadata()},k.prototype.execute=function(t){t(this)};const x={__module__:e,Server:c.a,worker:function(){return this},initialize_profile_thread:function(){if(void 0===j){let t=0,e=performance.now();setTimeout(function n(){let r=performance.now();console.log("".concat((1e3*t/(r-e)).toFixed(2)," msgs/sec")),t=0,e=r,setTimeout(n,5e3)},5e3),j=function(e){t+=e},console.log("Profiling initialized")}},table:function(t,i){let o;(i=i||{}).index=i.index||"";let a=!1;if(t instanceof ArrayBuffer||r&&t instanceof r?(o=O(t),a=!0):("string"==typeof t&&(","===t[0]&&(t="_"+t),t=m.a.parse(t.trim(),{dynamicTyping:!0,header:!0}).data),n.clean(),n.init(e,t),o=n),i.index&&i.limit)throw"Cannot specify both index \'".concat(i.index,"\' and limit \'").concat(i.limit,"\'.");let s,u,c=0;try{return u=new e.t_pool,[s,c]=S(o,u,void 0,void 0,i.index,i.limit,c,!1,!1,a),new k(s,u,i.index,void 0,i.limit,c)}catch(t){throw u&&u.delete(),s&&s.delete(),console.error("Table initialization failed: ".concat(t)),t}}};for(let t of Object.keys(i))x[t]=i[t];return"undefined"!=typeof self&&self.addEventListener&&new class extends c.a{constructor(t){super(t),self.addEventListener("message",t=>this.process(t.data),!1)}post(t,e){self.postMessage(t,e)}init(t){"undefined"==typeof WebAssembly?console.log("Loading asm.js"):(console.log("Loading wasm"),e=e({wasmBinary:t.buffer,wasmJSMethod:"native-wasm"}).then(()=>super.init(t)))}}(x),x}}).call(this,n(34),n(57).Buffer)},function(module,__webpack_exports__,__webpack_require__){"use strict";__webpack_require__.d(__webpack_exports__,"a",function(){return Server});var core_js_modules_web_dom_iterable__WEBPACK_IMPORTED_MODULE_0__=__webpack_require__(32),core_js_modules_web_dom_iterable__WEBPACK_IMPORTED_MODULE_0___default=__webpack_require__.n(core_js_modules_web_dom_iterable__WEBPACK_IMPORTED_MODULE_0__),_config__WEBPACK_IMPORTED_MODULE_1__=__webpack_require__(67),_config__WEBPACK_IMPORTED_MODULE_1___default=__webpack_require__.n(_config__WEBPACK_IMPORTED_MODULE_1__);function error_to_json(t){const e={};return"string"!=typeof t?Object.getOwnPropertyNames(t).forEach(n=>{e[n]=t[n]},t):e.message=t,e}class Server{constructor(t){this.perspective=t,this._tables={},this._views={},this._callback_cache=new Map}init(t){t.config&&Object(_config__WEBPACK_IMPORTED_MODULE_1__.override_config)(t.config),this.post(t)}post(){throw new Error("post() not implemented!")}clear_views(t){for(let e of Object.keys(this._views))if(this._views[e].client_id===t){try{this._views[e].delete()}catch(t){console.error(t)}delete this._views[e]}console.debug("GC ".concat(Object.keys(this._views).length," views in memory"))}process(msg,client_id){switch(msg.cmd){case"init_profile_thread":this.perspective.initialize_profile_thread();break;case"init":this.init(msg);break;case"table":if(void 0===msg.args[0])this._tables[msg.name]=[];else{const t=this._tables[msg.name];if(this._tables[msg.name]=this.perspective.table(msg.args[0],msg.options),t)for(const e of t)this.process(e)}break;case"add_computed":let table=this._tables[msg.original],computed=msg.computed;for(let i=0;i<computed.length;++i){let column=computed[i];eval("column.func = "+column.func)}this._tables[msg.name]=table.add_computed(computed);break;case"table_generate":let g;eval("g = "+msg.args),g(function(t){this._tables[msg.name]=t,this.post({id:msg.id,data:"created!"})});break;case"table_execute":let f;eval("f = "+msg.f),f(this._tables[msg.name]);break;case"table_method":case"view_method":this.process_method_call(msg);break;case"view":this._views[msg.view_name]=this._tables[msg.table_name].view(msg.config),this._views[msg.view_name].client_id=client_id}}process_error(t,e){this.post({id:t.id,error:error_to_json(e)})}process_subscribe(t,e){try{let n;"on"===t.method.slice(0,2)?(n=r=>{let i={id:t.id,data:r};try{if(t.args&&t.args[0]&&"on_update"===t.method&&"row"===t.args[0].mode)return void this.post(i,[r]);this.post(i)}catch(t){console.error("Removing callback after failed on_update() (presumably due to closed connection)"),e.remove_update(n)}},t.callback_id&&this._callback_cache.set(t.callback_id,n)):t.callback_id&&(n=this._callback_cache.get(t.callback_id),this._callback_cache.delete(t.callback_id)),n?e[t.method](n,...t.args):console.error(\'Callback not found for remote call "\'.concat(t,\'"\'))}catch(e){return void this.process_error(t,e)}}process_method_call_response(t,e){"delete"===t.method&&delete this._views[t.name],"to_arrow"===t.method?this.post({id:t.id,data:e},[e]):this.post({id:t.id,data:e})}process_method_call(t){let e,n;if((e="table_method"===t.cmd?this._tables[t.name]:this._views[t.name])||"view_method"!==t.cmd)if(e&&e.push)e.push(t);else try{if(t.subscribe)return void this.process_subscribe(t,e);(n=e[t.method].apply(e,t.args))instanceof Promise?n.then(e=>this.process_method_call_response(t,e)).catch(e=>this.process_error(t,e)):this.process_method_call_response(t,n)}catch(e){return void this.process_error(t,e)}else this.process_error(t,{message:"View is not initialized"})}}},function(t,e,n){"use strict";function r(t,e,n){return e<=t&&t<=n}function i(t){if(void 0===t)return{};if(t===Object(t))return t;throw TypeError("Could not convert argument to dictionary")}n.d(e,"b",function(){return l}),n.d(e,"a",function(){return f});var o=-1;function a(t){this.tokens=[].slice.call(t)}a.prototype={endOfStream:function(){return!this.tokens.length},read:function(){return this.tokens.length?this.tokens.shift():o},prepend:function(t){if(Array.isArray(t))for(var e=t;e.length;)this.tokens.unshift(e.pop());else this.tokens.unshift(t)},push:function(t){if(Array.isArray(t))for(var e=t;e.length;)this.tokens.push(e.shift());else this.tokens.push(t)}};var s=-1;function u(t,e){if(t)throw TypeError("Decoder error");return e||65533}var c="utf-8";function f(t,e){if(!(this instanceof f))return new f(t,e);if((t=void 0!==t?String(t).toLowerCase():c)!==c)throw new Error("Encoding not supported. Only utf-8 is supported");e=i(e),this._streaming=!1,this._BOMseen=!1,this._decoder=null,this._fatal=Boolean(e.fatal),this._ignoreBOM=Boolean(e.ignoreBOM),Object.defineProperty(this,"encoding",{value:"utf-8"}),Object.defineProperty(this,"fatal",{value:this._fatal}),Object.defineProperty(this,"ignoreBOM",{value:this._ignoreBOM})}function l(t,e){if(!(this instanceof l))return new l(t,e);if((t=void 0!==t?String(t).toLowerCase():c)!==c)throw new Error("Encoding not supported. Only utf-8 is supported");e=i(e),this._streaming=!1,this._encoder=null,this._options={fatal:Boolean(e.fatal)},Object.defineProperty(this,"encoding",{value:"utf-8"})}function h(t){var e=t.fatal,n=0,i=0,a=0,c=128,f=191;this.handler=function(t,l){if(l===o&&0!==a)return a=0,u(e);if(l===o)return s;if(0===a){if(r(l,0,127))return l;if(r(l,194,223))a=1,n=l-192;else if(r(l,224,239))224===l&&(c=160),237===l&&(f=159),a=2,n=l-224;else{if(!r(l,240,244))return u(e);240===l&&(c=144),244===l&&(f=143),a=3,n=l-240}return n<<=6*a,null}if(!r(l,c,f))return n=a=i=0,c=128,f=191,t.prepend(l),u(e);if(c=128,f=191,n+=l-128<<6*(a-(i+=1)),i!==a)return null;var h=n;return n=a=i=0,h}}function p(t){t.fatal,this.handler=function(t,e){if(e===o)return s;if(r(e,0,127))return e;var n,i;r(e,128,2047)?(n=1,i=192):r(e,2048,65535)?(n=2,i=224):r(e,65536,1114111)&&(n=3,i=240);for(var a=[(e>>6*n)+i];n>0;){var u=e>>6*(n-1);a.push(128|63&u),n-=1}return a}}f.prototype={decode:function(t,e){var n;n="object"==typeof t&&t instanceof ArrayBuffer?new Uint8Array(t):"object"==typeof t&&"buffer"in t&&t.buffer instanceof ArrayBuffer?new Uint8Array(t.buffer,t.byteOffset,t.byteLength):new Uint8Array(0),e=i(e),this._streaming||(this._decoder=new h({fatal:this._fatal}),this._BOMseen=!1),this._streaming=Boolean(e.stream);for(var r,o=new a(n),u=[];!o.endOfStream()&&(r=this._decoder.handler(o,o.read()))!==s;)null!==r&&(Array.isArray(r)?u.push.apply(u,r):u.push(r));if(!this._streaming){do{if((r=this._decoder.handler(o,o.read()))===s)break;null!==r&&(Array.isArray(r)?u.push.apply(u,r):u.push(r))}while(!o.endOfStream());this._decoder=null}return u.length&&(-1===["utf-8"].indexOf(this.encoding)||this._ignoreBOM||this._BOMseen||(65279===u[0]?(this._BOMseen=!0,u.shift()):this._BOMseen=!0)),function(t){for(var e="",n=0;n<t.length;++n){var r=t[n];r<=65535?e+=String.fromCharCode(r):(r-=65536,e+=String.fromCharCode(55296+(r>>10),56320+(1023&r)))}return e}(u)}},l.prototype={encode:function(t,e){t=t?String(t):"",e=i(e),this._streaming||(this._encoder=new p(this._options)),this._streaming=Boolean(e.stream);for(var n,r=[],o=new a(function(t){for(var e=String(t),n=e.length,r=0,i=[];r<n;){var o=e.charCodeAt(r);if(o<55296||o>57343)i.push(o);else if(56320<=o&&o<=57343)i.push(65533);else if(55296<=o&&o<=56319)if(r===n-1)i.push(65533);else{var a=t.charCodeAt(r+1);if(56320<=a&&a<=57343){var s=1023&o,u=1023&a;i.push(65536+(s<<10)+u),r+=1}else i.push(65533)}r+=1}return i}(t));!o.endOfStream()&&(n=this._encoder.handler(o,o.read()))!==s;)Array.isArray(n)?r.push.apply(r,n):r.push(n);if(!this._streaming){for(;(n=this._encoder.handler(o,o.read()))!==s;)Array.isArray(n)?r.push.apply(r,n):r.push(n);this._encoder=null}return new Uint8Array(r)}}},function(t,e,n){"use strict";var r=n(0),i=n(4),o=n(2),a=function(){function t(t,e,n,r){var i;this._fields=t||[],this._metadata=e||new Map,n&&r||(i=function t(e,n,r){for(var i=-1,a=e.length;++i<a;){var s=e[i],u=s.type;if(o.c.isDictionary(u))if(r.get(u.id)||r.set(u.id,[]),n.has(u.id)){if(n.get(u.id)!==u.dictionary)throw new Error("Cannot create Schema containing two different dictionaries with the same Id")}else n.set(u.id,u.dictionary),r.get(u.id).push(s);u.children&&t(u.children,n,r)}return{dictionaries:n,dictionaryFields:r}}(t,n||new Map,r||new Map),n=i.dictionaries,r=i.dictionaryFields),this._dictionaries=n,this._dictionaryFields=r}return t.from=function(e,n){return void 0===n&&(n=[]),new t(e.map(function(t,e){return new s(""+(n[e]||e),t.type)}))},Object.defineProperty(t.prototype,"fields",{get:function(){return this._fields},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"metadata",{get:function(){return this._metadata},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dictionaries",{get:function(){return this._dictionaries},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dictionaryFields",{get:function(){return this._dictionaryFields},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,Symbol.toStringTag,{get:function(){return"Schema"},enumerable:!0,configurable:!0}),t.prototype.toString=function(){return"Schema<{ "+this._fields.map(function(t,e){return e+": "+t}).join(", ")+" }>"},t.prototype.select=function(){for(var e=[],n=0;n<arguments.length;n++)e[n]=arguments[n];var r=e.reduce(function(t,e){return(t[e]=!0)&&t},Object.create(null));return new t(this.fields.filter(function(t){return r[t.name]}),this.metadata)},t}(),s=function(){function t(t,e,n,r){void 0===n&&(n=!1),this._name=t,this._type=e,this._nullable=n,this._metadata=r||new Map}return Object.defineProperty(t.prototype,"type",{get:function(){return this._type},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"name",{get:function(){return this._name},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"nullable",{get:function(){return this._nullable},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"metadata",{get:function(){return this._metadata},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"typeId",{get:function(){return this._type.typeId},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,Symbol.toStringTag,{get:function(){return"Field"},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"indices",{get:function(){return o.c.isDictionary(this._type)?this._type.indices:this._type},enumerable:!0,configurable:!0}),t.prototype.toString=function(){return this.name+": "+this.type},t}(),u=void 0;function c(t){return null===t?"null":t===u?"undefined":"string"==typeof t?\'"\'+t+\'"\':"function"==typeof t[Symbol.toPrimitive]?t[Symbol.toPrimitive]("string"):ArrayBuffer.isView(t)?"["+t+"]":JSON.stringify(t)}var f,l={enumerable:!0,configurable:!1,get:function(){}},h={writable:!1,enumerable:!1,configurable:!1,value:-1},p={writable:!1,enumerable:!1,configurable:!0,value:null},d={writable:!1,enumerable:!1,configurable:!1,value:null},y={parent:d,rowIndex:p},b=function(){function t(t,e){var n=this;h.value=t.length,Object.defineProperty(this,"length",h),t.forEach(function(t,r){l.get=n._bindGetter(r),l.configurable=!0,l.enumerable=e,Object.defineProperty(n,t.name,l),l.configurable=!1,l.enumerable=!e,Object.defineProperty(n,r,l),l.get=null})}return t.new=function(e,n){var r,i;return void 0===n&&(n=!1),Array.isArray(e)?i=e:(r=e,n=!0,i=Object.keys(r).map(function(t){return new s(t,r[t])})),new t(i,n)},t.prototype[Symbol.iterator]=function(){var t,e;return r.h(this,function(n){switch(n.label){case 0:t=-1,e=this.length,n.label=1;case 1:return++t<e?[4,this[t]]:[3,4];case 2:n.sent(),n.label=3;case 3:return[3,1];case 4:return[2]}})},t.prototype._bindGetter=function(t){return function(){var e=this.parent.getChildAt(t);return e?e.get(this.rowIndex):null}},t.prototype.get=function(t){return this[t]},t.prototype.bind=function(t,e){p.value=e,d.value=t;var n=Object.create(this,y);return p.value=null,d.value=null,n},t.prototype.toJSON=function(){var t=this;return o.c.isStruct(this.parent.type)?r.j(this):Object.getOwnPropertyNames(this).reduce(function(e,n){return(e[n]=t[n])&&e||e},{})},t.prototype.toString=function(){var t=this;return o.c.isStruct(this.parent.type)?r.j(this).map(function(t){return c(t)}).join(", "):Object.getOwnPropertyNames(this).reduce(function(e,n){return(e[n]=c(t[n]))&&e||e},{})},t}(),v=n(3);function m(t,e,n,r){var i=t.length,o=void 0===i?0:i,a="number"!=typeof e?0:e,s="number"!=typeof n?o:n;return a<0&&(a=(a%o+o)%o),s<0&&(s=(s%o+o)%o),s<a&&(f=a,a=s,s=f),s>o&&(s=o),r?r(t,a,s):[a,s]}function _(t){if(null==t||"object"!=typeof t)return function(e){return e===t};if(t instanceof Date){var e=t.valueOf();return function(t){return t instanceof Date&&t.valueOf()===e}}if(ArrayBuffer.isView(t))return function(e){return!!e&&Object(v.a)(t,e)};if(Array.isArray(t)){for(var n=t.length,r=[],o=-1;++o<n;)r[o]=_(t[o]);return function(t){if(!t||t.length!==n)return!1;if(t instanceof b||t instanceof i.a){for(var e=-1,o=t.length;++e<o;)if(!r[e](t.get(e)))return!1;return!0}e=-1;for(var a=t.length;++e<a;)if(!r[e](t[e]))return!1;return!0}}if(t instanceof b||t instanceof i.a){var a=t.length,s=t.constructor,u=[];for(o=-1;++o<a;)u[o]=_(t.get(o));return function(t){if(!(t instanceof s))return!1;if(t.length!==a)return!1;for(var e=-1;++e<a;)if(!u[e](t.get(e)))return!1;return!0}}var c=Object.keys(t);if(c.length>0){var f=c.length,l=[];for(o=-1;++o<f;)l[o]=_(t[c[o]]);return function(t){if(!t||"object"!=typeof t)return!1;for(var e=-1;++e<f;)if(!l[e](t[c[e]]))return!1;return!0}}return function(){return!1}}var g,w,S,O,T,I,D=function(t){function e(e,n,r){void 0===n&&(n=[]),void 0===r&&(r=function(t){for(var e=new Uint32Array((t||[]).length+1),n=e[0]=0,r=e.length,i=0;++i<r;)e[i]=n+=t[i-1].length;return e}(n));var i=t.call(this)||this;return i._nullCount=-1,i._type=e,i._chunks=n,i._chunkOffsets=r,i._length=r[r.length-1],i._numChildren=(i._type.children||[]).length,i}return r.g(e,t),e.flatten=function(){for(var t=[],n=0;n<arguments.length;n++)t[n]=arguments[n];return t.reduce(function t(n,i){return i instanceof e?i.chunks.reduce(t,n):r.j(n,[i])},[]).filter(function(t){return t instanceof i.a})},e.concat=function(){for(var t=[],n=0;n<arguments.length;n++)t[n]=arguments[n];return new e(t[0].type,e.flatten.apply(e,r.j(t)))},Object.defineProperty(e.prototype,"type",{get:function(){return this._type},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"length",{get:function(){return this._length},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"chunks",{get:function(){return this._chunks},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"typeId",{get:function(){return this._type.typeId},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"data",{get:function(){return this._chunks[0]?this._chunks[0].data:null},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"ArrayType",{get:function(){return this._type.ArrayType},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"numChildren",{get:function(){return this._numChildren},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"stride",{get:function(){return this._chunks[0]?this._chunks[0].stride:1},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"nullCount",{get:function(){var t=this._nullCount;return t<0&&(this._nullCount=t=this._chunks.reduce(function(t,e){return t+e.nullCount},0)),t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"indices",{get:function(){if(o.c.isDictionary(this._type)){if(!this._indices){var t=this._chunks;this._indices=1===t.length?t[0].indices:e.concat.apply(e,r.j(t.map(function(t){return t.indices})))}return this._indices}return null},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dictionary",{get:function(){return o.c.isDictionary(this._type)?this._type.dictionaryVector:null},enumerable:!0,configurable:!0}),e.prototype[Symbol.iterator]=function(){var t,e,n,i,o,a;return r.h(this,function(s){switch(s.label){case 0:s.trys.push([0,5,6,7]),n=r.k(this._chunks),i=n.next(),s.label=1;case 1:return i.done?[3,4]:(o=i.value,[5,r.k(o)]);case 2:s.sent(),s.label=3;case 3:return i=n.next(),[3,1];case 4:return[3,7];case 5:return a=s.sent(),t={error:a},[3,7];case 6:try{i&&!i.done&&(e=n.return)&&e.call(n)}finally{if(t)throw t.error}return[7];case 7:return[2]}})},e.prototype.clone=function(t){return void 0===t&&(t=this._chunks),new e(this._type,t)},e.prototype.concat=function(){for(var t=[],n=0;n<arguments.length;n++)t[n]=arguments[n];return this.clone(e.flatten.apply(e,r.j([this],t)))},e.prototype.slice=function(t,e){return m(this,t,e,this._sliceInternal)},e.prototype.getChildAt=function(t){if(t<0||t>=this._numChildren)return null;var n,r,i,o=this._children||(this._children=[]);return(n=o[t])?n:(r=(this._type.children||[])[t])&&(i=this._chunks.map(function(e){return e.getChildAt(t)}).filter(function(t){return null!=t})).length>0?o[t]=new e(r.type,i):null},e.prototype.search=function(t,e){var n=t,r=this._chunkOffsets,i=r.length-1;if(n<0)return null;if(n>=r[i])return null;if(i<=1)return e?e(this,0,n):[0,n];var o=0,a=0,s=0;do{if(o+1===i)return e?e(this,o,n-a):[o,n-a];n>=r[s=o+(i-o)/2|0]?o=s:i=s}while(n<r[i]&&n>=(a=r[o]));return null},e.prototype.isValid=function(t){return!!this.search(t,this.isValidInternal)},e.prototype.get=function(t){return this.search(t,this.getInternal)},e.prototype.set=function(t,e){this.search(t,function(t,n,r){return t.chunks[n].set(r,e)})},e.prototype.indexOf=function(t,e){var n=this;return e&&"number"==typeof e?this.search(e,function(e,r,i){return n.indexOfInternal(e,r,i,t)}):this.indexOfInternal(this,0,Math.max(0,e||0),t)},e.prototype.toArray=function(){var t=this.chunks,e=t.length,n=this._type.ArrayType;if(e<=0)return new n(0);if(e<=1)return t[0].toArray();for(var r=0,i=new Array(e),o=-1;++o<e;)r+=(i[o]=t[o].toArray()).length;n!==i[0].constructor&&(n=i[0].constructor);for(var a=new n(r),s=n===Array?M:A,u=(o=-1,0);++o<e;)u=s(i[o],a,u);return a},e.prototype.getInternal=function(t,e,n){return t._chunks[e].get(n)},e.prototype.isValidInternal=function(t,e,n){return t._chunks[e].isValid(n)},e.prototype.indexOfInternal=function(t,e,n,r){for(var i=t._chunks,o=e-1,a=i.length,s=n,u=0,c=-1;++o<a;){if(~(c=i[o].indexOf(r,s)))return u+c;s=0,u+=i[o].length}return-1},e.prototype._sliceInternal=function(t,e,n){for(var r=[],i=t.chunks,o=t._chunkOffsets,a=-1,s=i.length;++a<s;){var u=i[a],c=u.length,f=o[a];if(f>=n)break;if(!(e>=f+c))if(f>=e&&f+c<=n)r.push(u);else{var l=Math.max(0,e-f),h=l+Math.min(c-l,n-f);r.push(u.slice(l,h))}}return t.clone(r)},e}(i.a),A=function(t,e,n){return e.set(t,n),n+t.length},M=function(t,e,n){for(var r=n-1,i=-1,o=t.length;++i<o;)e[++r]=t[i];return r},k=function(t){function e(e,n,i){void 0===n&&(n=[]);var o=this;return n=D.flatten.apply(D,r.j(n)),(o=t.call(this,e.type,n,i)||this)._field=e,1!==n.length||o instanceof j?o:new j(e,n[0],o._chunkOffsets)}return r.g(e,t),Object.defineProperty(e.prototype,"field",{get:function(){return this._field},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"name",{get:function(){return this._field.name},enumerable:!0,configurable:!0}),e.prototype.clone=function(t){return void 0===t&&(t=this._chunks),new e(this._field,t)},e.prototype.getChildAt=function(t){if(t<0||t>=this.numChildren)return null;var n,r,i,o=this._children||(this._children=[]);return(n=o[t])?n:(r=(this.type.children||[])[t])&&(i=this._chunks.map(function(e){return e.getChildAt(t)}).filter(function(t){return null!=t})).length>0?o[t]=new e(r,i):null},e}(D),j=function(t){function e(e,n,r){var i=t.call(this,e,[n],r)||this;return i._chunk=n,i}return r.g(e,t),e.prototype.search=function(t,e){return e?e(this,0,t):[0,t]},e.prototype.isValid=function(t){return this._chunk.isValid(t)},e.prototype.get=function(t){return this._chunk.get(t)},e.prototype.set=function(t,e){this._chunk.set(t,e)},e.prototype.indexOf=function(t,e){return this._chunk.indexOf(t,e)},e}(k),x=n(7),E=n(6),B=function(t){function e(e,n){var r=t.call(this)||this;return r._children=n,r.numChildren=e.childData.length,r._bindDataAccessors(r.data=e),r}return r.g(e,t),Object.defineProperty(e.prototype,"type",{get:function(){return this.data.type},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"typeId",{get:function(){return this.data.typeId},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"length",{get:function(){return this.data.length},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"offset",{get:function(){return this.data.offset},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"stride",{get:function(){return this.data.stride},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"nullCount",{get:function(){return this.data.nullCount},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"VectorName",{get:function(){return this.constructor.name},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"ArrayType",{get:function(){return this.data.ArrayType},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"values",{get:function(){return this.data.values},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"typeIds",{get:function(){return this.data.typeIds},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"nullBitmap",{get:function(){return this.data.nullBitmap},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"valueOffsets",{get:function(){return this.data.valueOffsets},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,Symbol.toStringTag,{get:function(){return this.VectorName+"<"+this.type[Symbol.toStringTag]+">"},enumerable:!0,configurable:!0}),e.prototype.clone=function(t,e){return void 0===e&&(e=this._children),i.a.new(t,e)},e.prototype.concat=function(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return D.concat.apply(D,r.j([this],t))},e.prototype.slice=function(t,e){return m(this,t,e,this._sliceInternal)},e.prototype.isValid=function(t){if(this.nullCount>0){var e=this.offset+t;return 0!=(this.nullBitmap[e>>3]&1<<e%8)}return!0},e.prototype.getChildAt=function(t){return t<0||t>=this.numChildren?null:(this._children||(this._children=[]))[t]||(this._children[t]=i.a.new(this.data.childData[t]))},e.prototype.toJSON=function(){return r.j(this)},e.prototype._sliceInternal=function(t,e,n){return t.clone(t.data.slice(e,n-e))},e.prototype._bindDataAccessors=function(t){},e}(i.a),F=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.prototype.asMap=function(t){return void 0===t&&(t=!1),i.a.new(this.data.clone(new o.v(this.type.children,t)))},Object.defineProperty(e.prototype,"rowProxy",{get:function(){return this._rowProxy||(this._rowProxy=b.new(this.type.children||[],!1))},enumerable:!0,configurable:!0}),e}(B),P=function(t){function e(){for(var e,n=[],i=0;i<arguments.length;i++)n[i]=arguments[i];var a,s,u=this,c=n[0];if("number"==typeof n[1]){var f=c.fields,l=r.i(n,3),h=l[1],p=l[2];a=E.a.Struct(new o.x(f),0,h,0,null,p)}else a=(e=r.i(n,3))[1],s=e[2];return(u=t.call(this,a,s)||this)._schema=c,u}return r.g(e,t),e.from=function(t,n){return void 0===n&&(n=[]),new e(a.from(t,n),t.reduce(function(t,e){return Math.max(t,e.length)},0),t)},e.prototype.clone=function(t,n){return void 0===n&&(n=this._children),new e(this._schema,t,n)},e.prototype.concat=function(){for(var t=[],n=0;n<arguments.length;n++)t[n]=arguments[n];var i=this._schema,o=D.flatten.apply(D,r.j([this],t));return new ei(i,o.map(function(t){var n=t.data;return new e(i,n)}))},Object.defineProperty(e.prototype,"schema",{get:function(){return this._schema},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"numCols",{get:function(){return this._schema.fields.length},enumerable:!0,configurable:!0}),e.prototype.select=function(){for(var t,n=[],i=0;i<arguments.length;i++)n[i]=arguments[i];var a=this._schema.fields,s=(t=this._schema).select.apply(t,r.j(n)),u=n.reduce(function(t,e){return(t[e]=!0)&&t},{}),c=this.data.childData.filter(function(t,e){return u[a[e].name]}),f=E.a.Struct(new o.x(s.fields),0,this.length,0,null,c);return new e(s,f)},e}(F),L=n(1),U=n(5);w=(O=(T=(I=g||(g={})).apache||(I.apache={})).arrow||(T.arrow={})).flatbuf||(O.flatbuf={}),S=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.getRootAsFooter=function(e,n){return(n||new t).__init(e.readInt32(e.position())+e.position(),e)},t.prototype.version=function(){var t=this.bb.__offset(this.bb_pos,4);return t?this.bb.readInt16(this.bb_pos+t):U.a.apache.arrow.flatbuf.MetadataVersion.V1},t.prototype.schema=function(t){var e=this.bb.__offset(this.bb_pos,6);return e?(t||new U.a.apache.arrow.flatbuf.Schema).__init(this.bb.__indirect(this.bb_pos+e),this.bb):null},t.prototype.dictionaries=function(t,e){var n=this.bb.__offset(this.bb_pos,8);return n?(e||new I.apache.arrow.flatbuf.Block).__init(this.bb.__vector(this.bb_pos+n)+24*t,this.bb):null},t.prototype.dictionariesLength=function(){var t=this.bb.__offset(this.bb_pos,8);return t?this.bb.__vector_len(this.bb_pos+t):0},t.prototype.recordBatches=function(t,e){var n=this.bb.__offset(this.bb_pos,10);return n?(e||new I.apache.arrow.flatbuf.Block).__init(this.bb.__vector(this.bb_pos+n)+24*t,this.bb):null},t.prototype.recordBatchesLength=function(){var t=this.bb.__offset(this.bb_pos,10);return t?this.bb.__vector_len(this.bb_pos+t):0},t.startFooter=function(t){t.startObject(4)},t.addVersion=function(t,e){t.addFieldInt16(0,e,U.a.apache.arrow.flatbuf.MetadataVersion.V1)},t.addSchema=function(t,e){t.addFieldOffset(1,e,0)},t.addDictionaries=function(t,e){t.addFieldOffset(2,e,0)},t.startDictionariesVector=function(t,e){t.startVector(24,e,8)},t.addRecordBatches=function(t,e){t.addFieldOffset(3,e,0)},t.startRecordBatchesVector=function(t,e){t.startVector(24,e,8)},t.endFooter=function(t){return t.endObject()},t.finishFooterBuffer=function(t,e){t.finish(e)},t}(),w.Footer=S,function(t){!function(t){!function(t){!function(t){var e=function(){function t(){this.bb_pos=0}return t.prototype.__init=function(t,e){return this.bb_pos=t,this.bb=e,this},t.prototype.offset=function(){return this.bb.readInt64(this.bb_pos)},t.prototype.metaDataLength=function(){return this.bb.readInt32(this.bb_pos+8)},t.prototype.bodyLength=function(){return this.bb.readInt64(this.bb_pos+16)},t.createBlock=function(t,e,n,r){return t.prep(8,24),t.writeInt64(r),t.pad(4),t.writeInt32(n),t.writeInt64(e),t.offset()},t}();t.Block=e}(t.flatbuf||(t.flatbuf={}))}(t.arrow||(t.arrow={}))}(t.apache||(t.apache={}))}(g||(g={}));var C=n(12),R=C.a.Long,N=C.a.Builder,Y=C.a.ByteBuffer,V=g.apache.arrow.flatbuf.Block,z=g.apache.arrow.flatbuf.Footer,W=function(){function t(t,e,n,r){void 0===e&&(e=L.e.V4),this.schema=t,this.version=e,n&&(this._recordBatches=n),r&&(this._dictionaryBatches=r)}return t.decode=function(t){t=new Y(Object(v.e)(t));var e=z.getRootAsFooter(t),n=a.decode(e.schema());return new H(n,e)},t.encode=function(t){var e=new N,n=a.encode(e,t.schema);z.startRecordBatchesVector(e,t.numRecordBatches),r.j(t.recordBatches()).slice().reverse().forEach(function(t){return G.encode(e,t)});var i=e.endVector();z.startDictionariesVector(e,t.numDictionaries),r.j(t.dictionaryBatches()).slice().reverse().forEach(function(t){return G.encode(e,t)});var o=e.endVector();return z.startFooter(e),z.addSchema(e,n),z.addVersion(e,L.e.V4),z.addRecordBatches(e,i),z.addDictionaries(e,o),z.finishFooterBuffer(e,z.endFooter(e)),e.asUint8Array()},Object.defineProperty(t.prototype,"numRecordBatches",{get:function(){return this._recordBatches.length},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"numDictionaries",{get:function(){return this._dictionaryBatches.length},enumerable:!0,configurable:!0}),t.prototype.recordBatches=function(){var t,e,n;return r.h(this,function(r){switch(r.label){case 0:t=void 0,e=-1,n=this.numRecordBatches,r.label=1;case 1:return++e<n?(t=this.getRecordBatch(e))?[4,t]:[3,3]:[3,4];case 2:r.sent(),r.label=3;case 3:return[3,1];case 4:return[2]}})},t.prototype.dictionaryBatches=function(){var t,e,n;return r.h(this,function(r){switch(r.label){case 0:t=void 0,e=-1,n=this.numDictionaries,r.label=1;case 1:return++e<n?(t=this.getDictionaryBatch(e))?[4,t]:[3,3]:[3,4];case 2:r.sent(),r.label=3;case 3:return[3,1];case 4:return[2]}})},t.prototype.getRecordBatch=function(t){return t>=0&&t<this.numRecordBatches&&this._recordBatches[t]||null},t.prototype.getDictionaryBatch=function(t){return t>=0&&t<this.numDictionaries&&this._dictionaryBatches[t]||null},t}(),H=function(t){function e(e,n){var r=t.call(this,e,n.version())||this;return r._footer=n,r}return r.g(e,t),Object.defineProperty(e.prototype,"numRecordBatches",{get:function(){return this._footer.recordBatchesLength()},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"numDictionaries",{get:function(){return this._footer.dictionariesLength()},enumerable:!0,configurable:!0}),e.prototype.getRecordBatch=function(t){if(t>=0&&t<this.numRecordBatches){var e=this._footer.recordBatches(t);if(e)return G.decode(e)}return null},e.prototype.getDictionaryBatch=function(t){if(t>=0&&t<this.numDictionaries){var e=this._footer.dictionaries(t);if(e)return G.decode(e)}return null},e}(W),G=function(){function t(t,e,n){this.metaDataLength=t,this.offset="number"==typeof n?n:n.low,this.bodyLength="number"==typeof e?e:e.low}return t.decode=function(e){return new t(e.metaDataLength(),e.bodyLength(),e.offset())},t.encode=function(t,e){var n=e.metaDataLength,r=new R(e.offset,0),i=new R(e.bodyLength,0);return V.createBlock(t,r,n,i)},t}(),q=n(11),Z=n(14),J=n(8),K=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.prototype.write=function(e){if((e=Object(v.e)(e)).byteLength>0)return t.prototype.write.call(this,e)},e.prototype.toString=function(t){return void 0===t&&(t=!1),t?Object(Z.a)(this.toUint8Array(!0)):this.toUint8Array(!1).then(Z.a)},e.prototype.toUint8Array=function(t){return void 0===t&&(t=!1),t?Object(v.b)(this._values)[0]:r.f(this,void 0,void 0,function(){var t,e,n,i,o,a,s,u;return r.h(this,function(c){switch(c.label){case 0:n=[],i=0,c.label=1;case 1:c.trys.push([1,6,7,12]),o=r.d(this),c.label=2;case 2:return[4,o.next()];case 3:if((a=c.sent()).done)return[3,5];s=a.value,n.push(s),i+=s.byteLength,c.label=4;case 4:return[3,2];case 5:return[3,12];case 6:return u=c.sent(),t={error:u},[3,12];case 7:return c.trys.push([7,,10,11]),a&&!a.done&&(e=o.return)?[4,e.call(o)]:[3,9];case 8:c.sent(),c.label=9;case 9:return[3,11];case 10:if(t)throw t.error;return[7];case 11:return[7];case 12:return[2,Object(v.b)(n,i)[0]]}})})},e}(J.b),$=function(){function t(t){t&&(this.source=new X(q.a.fromIterable(t)))}return t.prototype[Symbol.iterator]=function(){return this},t.prototype.next=function(t){return this.source.next(t)},t.prototype.throw=function(t){return this.source.throw(t)},t.prototype.return=function(t){return this.source.return(t)},t.prototype.peek=function(t){return this.source.peek(t)},t.prototype.read=function(t){return this.source.read(t)},t}(),Q=function(){function t(e){e instanceof t?this.source=e.source:e instanceof K?this.source=new tt(q.a.fromAsyncIterable(e)):Object(x.i)(e)?this.source=new tt(q.a.fromNodeStream(e)):Object(x.c)(e)?this.source=new tt(q.a.fromDOMStream(e.body)):Object(x.e)(e)?this.source=new tt(q.a.fromIterable(e)):Object(x.g)(e)?this.source=new tt(q.a.fromAsyncIterable(e)):Object(x.b)(e)?this.source=new tt(q.a.fromAsyncIterable(e)):Object(x.h)(e)&&(this.source=new tt(q.a.fromDOMStream(e)))}return t.prototype[Symbol.asyncIterator]=function(){return this},t.prototype.next=function(t){return this.source.next(t)},t.prototype.throw=function(t){return this.source.throw(t)},t.prototype.return=function(t){return this.source.return(t)},Object.defineProperty(t.prototype,"closed",{get:function(){return this.source.closed},enumerable:!0,configurable:!0}),t.prototype.cancel=function(t){return this.source.cancel(t)},t.prototype.peek=function(t){return this.source.peek(t)},t.prototype.read=function(t){return this.source.read(t)},t}(),X=function(){function t(t){this.source=t}return t.prototype.cancel=function(t){this.return(t)},t.prototype.peek=function(t){return this.next(t,"peek").value},t.prototype.read=function(t){return this.next(t,"read").value},t.prototype.next=function(t,e){return void 0===e&&(e="read"),this.source.next({cmd:e,size:t})},t.prototype.throw=function(t){return Object.create(this.source.throw&&this.source.throw(t)||J.c)},t.prototype.return=function(t){return Object.create(this.source.return&&this.source.return(t)||J.c)},t}(),tt=function(){function t(t){var e=this;this.source=t,this._closedPromise=new Promise(function(t){return e._closedPromiseResolve=t})}return t.prototype.cancel=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,this.return(t)];case 1:return e.sent(),[2]}})})},Object.defineProperty(t.prototype,"closed",{get:function(){return this._closedPromise},enumerable:!0,configurable:!0}),t.prototype.read=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,this.next(t,"read")];case 1:return[2,e.sent().value]}})})},t.prototype.peek=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,this.next(t,"peek")];case 1:return[2,e.sent().value]}})})},t.prototype.next=function(t,e){return void 0===e&&(e="read"),r.f(this,void 0,void 0,function(){return r.h(this,function(n){switch(n.label){case 0:return[4,this.source.next({cmd:e,size:t})];case 1:return[2,n.sent()]}})})},t.prototype.throw=function(t){return r.f(this,void 0,void 0,function(){var e,n;return r.h(this,function(r){switch(r.label){case 0:return(n=this.source.throw)?[4,this.source.throw(t)]:[3,2];case 1:n=r.sent(),r.label=2;case 2:return e=n||J.c,this._closedPromiseResolve&&this._closedPromiseResolve(),this._closedPromiseResolve=void 0,[2,Object.create(e)]}})})},t.prototype.return=function(t){return r.f(this,void 0,void 0,function(){var e,n;return r.h(this,function(r){switch(r.label){case 0:return(n=this.source.return)?[4,this.source.return(t)]:[3,2];case 1:n=r.sent(),r.label=2;case 2:return e=n||J.c,this._closedPromiseResolve&&this._closedPromiseResolve(),this._closedPromiseResolve=void 0,[2,Object.create(e)]}})})},t}(),et=function(t){function e(e,n){var r=t.call(this)||this;return r.position=0,r.buffer=Object(v.e)(e),r.size=void 0===n?r.buffer.byteLength:n,r}return r.g(e,t),e.prototype.readInt32=function(t){var e=this.readAt(t,4),n=e.buffer,r=e.byteOffset;return new DataView(n,r).getInt32(0,!0)},e.prototype.seek=function(t){return this.position=Math.min(t,this.size),t<this.size},e.prototype.read=function(t){var e=this.buffer,n=this.size,r=this.position;return e&&r<n?("number"!=typeof t&&(t=1/0),this.position=Math.min(n,r+Math.min(n-r,t)),e.subarray(r,this.position)):null},e.prototype.readAt=function(t,e){var n=this.buffer,r=Math.min(this.size,t+e);return n?n.subarray(t,r):new Uint8Array(e)},e.prototype.close=function(){this.buffer&&(this.buffer=null)},e.prototype.throw=function(t){return this.close(),{done:!0,value:t}},e.prototype.return=function(t){return this.close(),{done:!0,value:t}},e}($),nt=function(t){function e(e,n){var i=t.call(this)||this;return i.position=0,i._handle=e,"number"==typeof n?i.size=n:i._pending=r.f(i,void 0,void 0,function(){var t;return r.h(this,function(n){switch(n.label){case 0:return delete this._pending,t=this,[4,e.stat()];case 1:return t.size=n.sent().size,[2]}})}),i}return r.g(e,t),e.prototype.readInt32=function(t){return r.f(this,void 0,void 0,function(){var e,n,i;return r.h(this,function(r){switch(r.label){case 0:return[4,this.readAt(t,4)];case 1:return e=r.sent(),n=e.buffer,i=e.byteOffset,[2,new DataView(n,i).getInt32(0,!0)]}})})},e.prototype.seek=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return this._pending?[4,this._pending]:[3,2];case 1:e.sent(),e.label=2;case 2:return this.position=Math.min(t,this.size),[2,t<this.size]}})})},e.prototype.read=function(t){return r.f(this,void 0,void 0,function(){var e,n,i,o,a,s,u,c,f;return r.h(this,function(r){switch(r.label){case 0:return this._pending?[4,this._pending]:[3,2];case 1:r.sent(),r.label=2;case 2:if(n=(e=this)._handle,i=e.size,o=e.position,!(n&&o<i))return[3,6];"number"!=typeof t&&(t=1/0),a=o,s=0,u=0,c=Math.min(i,a+Math.min(i-a,t)),f=new Uint8Array(Math.max(0,(this.position=c)-a)),r.label=3;case 3:return(a+=u)<c&&(s+=u)<f.byteLength?[4,n.read(f,s,f.byteLength-s,a)]:[3,5];case 4:return u=r.sent().bytesRead,[3,3];case 5:return[2,f];case 6:return[2,null]}})})},e.prototype.readAt=function(t,e){return r.f(this,void 0,void 0,function(){var n,i,o,a,s;return r.h(this,function(r){switch(r.label){case 0:return this._pending?[4,this._pending]:[3,2];case 1:r.sent(),r.label=2;case 2:return i=(n=this)._handle,o=n.size,i&&t+e<o?(a=Math.min(o,t+e),s=new Uint8Array(a-t),[4,i.read(s,0,e,t)]):[3,4];case 3:return[2,r.sent().buffer];case 4:return[2,new Uint8Array(e)]}})})},e.prototype.close=function(){return r.f(this,void 0,void 0,function(){var t;return r.h(this,function(e){switch(e.label){case 0:return t=this._handle,this._handle=null,t?[4,t.close()]:[3,2];case 1:e.sent(),e.label=2;case 2:return[2]}})})},e.prototype.throw=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,this.close()];case 1:return e.sent(),[2,{done:!0,value:t}]}})})},e.prototype.return=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,this.close()];case 1:return e.sent(),[2,{done:!0,value:t}]}})})},e}(Q),rt=n(9),it=n(10);function ot(t){return t<0&&(t=4294967295+t+1),"0x"+t.toString(16)}var at=[1,10,100,1e3,1e4,1e5,1e6,1e7,1e8],st=function(){function t(t){this.buffer=t}return t.prototype.high=function(){return this.buffer[1]},t.prototype.low=function(){return this.buffer[0]},t.prototype._times=function(t){var e=new Uint32Array([this.buffer[1]>>>16,65535&this.buffer[1],this.buffer[0]>>>16,65535&this.buffer[0]]),n=new Uint32Array([t.buffer[1]>>>16,65535&t.buffer[1],t.buffer[0]>>>16,65535&t.buffer[0]]),r=e[3]*n[3];this.buffer[0]=65535&r;var i=r>>>16;return i+=r=e[2]*n[3],i+=r=e[3]*n[2]>>>0,this.buffer[0]+=i<<16,this.buffer[1]=i>>>0<r?65536:0,this.buffer[1]+=i>>>16,this.buffer[1]+=e[1]*n[3]+e[2]*n[2]+e[3]*n[1],this.buffer[1]+=e[0]*n[3]+e[1]*n[2]+e[2]*n[1]+e[3]*n[0]<<16,this},t.prototype._plus=function(t){var e=this.buffer[0]+t.buffer[0]>>>0;this.buffer[1]+=t.buffer[1],e<this.buffer[0]>>>0&&++this.buffer[1],this.buffer[0]=e},t.prototype.lessThan=function(t){return this.buffer[1]<t.buffer[1]||this.buffer[1]===t.buffer[1]&&this.buffer[0]<t.buffer[0]},t.prototype.equals=function(t){return this.buffer[1]===t.buffer[1]&&this.buffer[0]==t.buffer[0]},t.prototype.greaterThan=function(t){return t.lessThan(this)},t.prototype.hex=function(){return ot(this.buffer[1])+" "+ot(this.buffer[0])},t}(),ut=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.prototype.times=function(t){return this._times(t),this},e.prototype.plus=function(t){return this._plus(t),this},e.from=function(t,n){return void 0===n&&(n=new Uint32Array(2)),e.fromString("string"==typeof t?t:t.toString(),n)},e.fromNumber=function(t,n){return void 0===n&&(n=new Uint32Array(2)),e.fromString(t.toString(),n)},e.fromString=function(t,n){void 0===n&&(n=new Uint32Array(2));for(var r=t.length,i=new e(n),o=0;o<r;){var a=8<r-o?8:r-o,s=new e(new Uint32Array([parseInt(t.substr(o,a),10),0])),u=new e(new Uint32Array([at[a],0]));i.times(u),i.plus(s),o+=a}return i},e.convertArray=function(t){for(var n=new Uint32Array(2*t.length),r=-1,i=t.length;++r<i;)e.from(t[r],new Uint32Array(n.buffer,n.byteOffset+2*r*4,2));return n},e.multiply=function(t,n){return new e(new Uint32Array(t.buffer)).times(n)},e.add=function(t,n){return new e(new Uint32Array(t.buffer)).plus(n)},e}(st),ct=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.prototype.negate=function(){return this.buffer[0]=1+~this.buffer[0],this.buffer[1]=~this.buffer[1],0==this.buffer[0]&&++this.buffer[1],this},e.prototype.times=function(t){return this._times(t),this},e.prototype.plus=function(t){return this._plus(t),this},e.prototype.lessThan=function(t){var e=this.buffer[1]<<0,n=t.buffer[1]<<0;return e<n||e===n&&this.buffer[0]<t.buffer[0]},e.from=function(t,n){return void 0===n&&(n=new Uint32Array(2)),e.fromString("string"==typeof t?t:t.toString(),n)},e.fromNumber=function(t,n){return void 0===n&&(n=new Uint32Array(2)),e.fromString(t.toString(),n)},e.fromString=function(t,n){void 0===n&&(n=new Uint32Array(2));for(var r=t.startsWith("-"),i=t.length,o=new e(n),a=r?1:0;a<i;){var s=8<i-a?8:i-a,u=new e(new Uint32Array([parseInt(t.substr(a,s),10),0])),c=new e(new Uint32Array([at[s],0]));o.times(c),o.plus(u),a+=s}return r?o.negate():o},e.convertArray=function(t){for(var n=new Uint32Array(2*t.length),r=-1,i=t.length;++r<i;)e.from(t[r],new Uint32Array(n.buffer,n.byteOffset+2*r*4,2));return n},e.multiply=function(t,n){return new e(new Uint32Array(t.buffer)).times(n)},e.add=function(t,n){return new e(new Uint32Array(t.buffer)).plus(n)},e}(st),ft=function(){function t(t){this.buffer=t}return t.prototype.high=function(){return new ct(new Uint32Array(this.buffer.buffer,this.buffer.byteOffset+8,2))},t.prototype.low=function(){return new ct(new Uint32Array(this.buffer.buffer,this.buffer.byteOffset,2))},t.prototype.negate=function(){return this.buffer[0]=1+~this.buffer[0],this.buffer[1]=~this.buffer[1],this.buffer[2]=~this.buffer[2],this.buffer[3]=~this.buffer[3],0==this.buffer[0]&&++this.buffer[1],0==this.buffer[1]&&++this.buffer[2],0==this.buffer[2]&&++this.buffer[3],this},t.prototype.times=function(t){var e=new ut(new Uint32Array([this.buffer[3],0])),n=new ut(new Uint32Array([this.buffer[2],0])),r=new ut(new Uint32Array([this.buffer[1],0])),i=new ut(new Uint32Array([this.buffer[0],0])),o=new ut(new Uint32Array([t.buffer[3],0])),a=new ut(new Uint32Array([t.buffer[2],0])),s=new ut(new Uint32Array([t.buffer[1],0])),u=new ut(new Uint32Array([t.buffer[0],0])),c=ut.multiply(i,u);this.buffer[0]=c.low();var f=new ut(new Uint32Array([c.high(),0]));return c=ut.multiply(r,u),f.plus(c),c=ut.multiply(i,s),f.plus(c),this.buffer[1]=f.low(),this.buffer[3]=f.lessThan(c)?1:0,this.buffer[2]=f.high(),new ut(new Uint32Array(this.buffer.buffer,this.buffer.byteOffset+8,2)).plus(ut.multiply(n,u)).plus(ut.multiply(r,s)).plus(ut.multiply(i,a)),this.buffer[3]+=ut.multiply(e,u).plus(ut.multiply(n,s)).plus(ut.multiply(r,a)).plus(ut.multiply(i,o)).low(),this},t.prototype.plus=function(t){var e=new Uint32Array(4);return e[3]=this.buffer[3]+t.buffer[3]>>>0,e[2]=this.buffer[2]+t.buffer[2]>>>0,e[1]=this.buffer[1]+t.buffer[1]>>>0,e[0]=this.buffer[0]+t.buffer[0]>>>0,e[0]<this.buffer[0]>>>0&&++e[1],e[1]<this.buffer[1]>>>0&&++e[2],e[2]<this.buffer[2]>>>0&&++e[3],this.buffer[3]=e[3],this.buffer[2]=e[2],this.buffer[1]=e[1],this.buffer[0]=e[0],this},t.prototype.hex=function(){return ot(this.buffer[3])+" "+ot(this.buffer[2])+" "+ot(this.buffer[1])+" "+ot(this.buffer[0])},t.multiply=function(e,n){return new t(new Uint32Array(e.buffer)).times(n)},t.add=function(e,n){return new t(new Uint32Array(e.buffer)).plus(n)},t.from=function(e,n){return void 0===n&&(n=new Uint32Array(4)),t.fromString("string"==typeof e?e:e.toString(),n)},t.fromNumber=function(e,n){return void 0===n&&(n=new Uint32Array(4)),t.fromString(e.toString(),n)},t.fromString=function(e,n){void 0===n&&(n=new Uint32Array(4));for(var r=e.startsWith("-"),i=e.length,o=new t(n),a=r?1:0;a<i;){var s=8<i-a?8:i-a,u=new t(new Uint32Array([parseInt(e.substr(a,s),10),0,0,0])),c=new t(new Uint32Array([at[s],0,0,0]));o.times(c),o.plus(u),a+=s}return r?o.negate():o},t.convertArray=function(e){for(var n=new Uint32Array(4*e.length),r=-1,i=e.length;++r<i;)t.from(e[r],new Uint32Array(n.buffer,n.byteOffset+16*r,4));return n},t}(),lt=function(t){function e(e,n,r){var i=t.call(this)||this;return i.nodesIndex=-1,i.buffersIndex=-1,i.bytes=e,i.nodes=n,i.buffers=r,i}return r.g(e,t),e.prototype.visit=function(e){return t.prototype.visit.call(this,e instanceof s?e.type:e)},e.prototype.visitNull=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Null(t,0,r,i,this.readNullBitmap(t,i))},e.prototype.visitBool=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Bool(t,0,r,i,this.readNullBitmap(t,i),this.readData(t))},e.prototype.visitInt=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Int(t,0,r,i,this.readNullBitmap(t,i),this.readData(t))},e.prototype.visitFloat=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Float(t,0,r,i,this.readNullBitmap(t,i),this.readData(t))},e.prototype.visitUtf8=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Utf8(t,0,r,i,this.readNullBitmap(t,i),this.readOffsets(t),this.readData(t))},e.prototype.visitBinary=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Binary(t,0,r,i,this.readNullBitmap(t,i),this.readOffsets(t),this.readData(t))},e.prototype.visitFixedSizeBinary=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.FixedSizeBinary(t,0,r,i,this.readNullBitmap(t,i),this.readData(t))},e.prototype.visitDate=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Date(t,0,r,i,this.readNullBitmap(t,i),this.readData(t))},e.prototype.visitTimestamp=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Timestamp(t,0,r,i,this.readNullBitmap(t,i),this.readData(t))},e.prototype.visitTime=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Time(t,0,r,i,this.readNullBitmap(t,i),this.readData(t))},e.prototype.visitDecimal=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Decimal(t,0,r,i,this.readNullBitmap(t,i),this.readData(t))},e.prototype.visitList=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.List(t,0,r,i,this.readNullBitmap(t,i),this.readOffsets(t),this.visit(t.children[0]))},e.prototype.visitStruct=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Struct(t,0,r,i,this.readNullBitmap(t,i),this.visitMany(t.children))},e.prototype.visitUnion=function(t){return t.mode===L.i.Sparse?this.visitSparseUnion(t):this.visitDenseUnion(t)},e.prototype.visitDenseUnion=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Union(t,0,r,i,this.readNullBitmap(t,i),this.readTypeIds(t),this.readOffsets(t),this.visitMany(t.children))},e.prototype.visitSparseUnion=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Union(t,0,r,i,this.readNullBitmap(t,i),this.readTypeIds(t),this.visitMany(t.children))},e.prototype.visitDictionary=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Dictionary(t,0,r,i,this.readNullBitmap(t,i),this.readData(t.indices))},e.prototype.visitInterval=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Interval(t,0,r,i,this.readNullBitmap(t,i),this.readData(t))},e.prototype.visitFixedSizeList=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.FixedSizeList(t,0,r,i,this.readNullBitmap(t,i),this.visit(t.children[0]))},e.prototype.visitMap=function(t,e){var n=void 0===e?this.nextFieldNode():e,r=n.length,i=n.nullCount;return E.a.Map(t,0,r,i,this.readNullBitmap(t,i),this.visitMany(t.children))},e.prototype.nextFieldNode=function(){return this.nodes[++this.nodesIndex]},e.prototype.nextBufferRange=function(){return this.buffers[++this.buffersIndex]},e.prototype.readNullBitmap=function(t,e,n){return void 0===n&&(n=this.nextBufferRange()),e>0&&this.readData(t,n)||new Uint8Array(0)},e.prototype.readOffsets=function(t,e){return this.readData(t,e)},e.prototype.readTypeIds=function(t,e){return this.readData(t,e)},e.prototype.readData=function(t,e){var n=void 0===e?this.nextBufferRange():e,r=n.length,i=n.offset;return this.bytes.subarray(i,i+r)},e}(rt.a),ht=function(t){function e(e,n,r){var i=t.call(this,new Uint8Array(0),n,r)||this;return i.sources=e,i}return r.g(e,t),e.prototype.readNullBitmap=function(t,e,n){var r=(void 0===n?this.nextBufferRange():n).offset;return e<=0?new Uint8Array(0):Object(it.d)(this.sources[r])},e.prototype.readOffsets=function(t,e){var n=(void 0===e?this.nextBufferRange():e).offset;return Object(v.d)(Uint8Array,Object(v.d)(Int32Array,this.sources[n]))},e.prototype.readTypeIds=function(t,e){var n=(void 0===e?this.nextBufferRange():e).offset;return Object(v.d)(Uint8Array,Object(v.d)(t.ArrayType,this.sources[n]))},e.prototype.readData=function(t,e){var n=(void 0===e?this.nextBufferRange():e).offset,r=this.sources;return o.c.isTimestamp(t)?Object(v.d)(Uint8Array,ct.convertArray(r[n])):(o.c.isInt(t)||o.c.isTime(t))&&64===t.bitWidth?Object(v.d)(Uint8Array,ct.convertArray(r[n])):o.c.isDate(t)&&t.unit===L.b.MILLISECOND?Object(v.d)(Uint8Array,ct.convertArray(r[n])):o.c.isDecimal(t)?Object(v.d)(Uint8Array,ft.convertArray(r[n])):o.c.isBinary(t)||o.c.isFixedSizeBinary(t)?function(t){for(var e=r[n].join(""),i=new Uint8Array(e.length/2),o=0;o<e.length;o+=2)i[o>>1]=parseInt(e.substr(o,2),16);return i}():o.c.isBool(t)?Object(it.d)(r[n]):o.c.isUtf8(t)?Object(Z.b)(r[n].join("")):Object(v.d)(Uint8Array,Object(v.d)(t.ArrayType,r[n].map(function(t){return+t})))},e}(lt),pt=n(21),dt=C.a.Long,yt=U.a.apache.arrow.flatbuf.Null,bt=U.a.apache.arrow.flatbuf.Int,vt=U.a.apache.arrow.flatbuf.FloatingPoint,mt=U.a.apache.arrow.flatbuf.Binary,_t=U.a.apache.arrow.flatbuf.Bool,gt=U.a.apache.arrow.flatbuf.Utf8,wt=U.a.apache.arrow.flatbuf.Decimal,St=U.a.apache.arrow.flatbuf.Date,Ot=U.a.apache.arrow.flatbuf.Time,Tt=U.a.apache.arrow.flatbuf.Timestamp,It=U.a.apache.arrow.flatbuf.Interval,Dt=U.a.apache.arrow.flatbuf.List,At=U.a.apache.arrow.flatbuf.Struct_,Mt=U.a.apache.arrow.flatbuf.Union,kt=U.a.apache.arrow.flatbuf.DictionaryEncoding,jt=U.a.apache.arrow.flatbuf.FixedSizeBinary,xt=U.a.apache.arrow.flatbuf.FixedSizeList,Et=U.a.apache.arrow.flatbuf.Map,Bt=new(function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.prototype.visit=function(e,n){return null==e||null==n?void 0:t.prototype.visit.call(this,e,n)},e.prototype.visitNull=function(t,e){return yt.startNull(e),yt.endNull(e)},e.prototype.visitInt=function(t,e){return bt.startInt(e),bt.addBitWidth(e,t.bitWidth),bt.addIsSigned(e,t.isSigned),bt.endInt(e)},e.prototype.visitFloat=function(t,e){return vt.startFloatingPoint(e),vt.addPrecision(e,t.precision),vt.endFloatingPoint(e)},e.prototype.visitBinary=function(t,e){return mt.startBinary(e),mt.endBinary(e)},e.prototype.visitBool=function(t,e){return _t.startBool(e),_t.endBool(e)},e.prototype.visitUtf8=function(t,e){return gt.startUtf8(e),gt.endUtf8(e)},e.prototype.visitDecimal=function(t,e){return wt.startDecimal(e),wt.addScale(e,t.scale),wt.addPrecision(e,t.precision),wt.endDecimal(e)},e.prototype.visitDate=function(t,e){return St.startDate(e),St.addUnit(e,t.unit),St.endDate(e)},e.prototype.visitTime=function(t,e){return Ot.startTime(e),Ot.addUnit(e,t.unit),Ot.addBitWidth(e,t.bitWidth),Ot.endTime(e)},e.prototype.visitTimestamp=function(t,e){var n=t.timezone&&e.createString(t.timezone)||void 0;return Tt.startTimestamp(e),Tt.addUnit(e,t.unit),void 0!==n&&Tt.addTimezone(e,n),Tt.endTimestamp(e)},e.prototype.visitInterval=function(t,e){return It.startInterval(e),It.addUnit(e,t.unit),It.endInterval(e)},e.prototype.visitList=function(t,e){return Dt.startList(e),Dt.endList(e)},e.prototype.visitStruct=function(t,e){return At.startStruct_(e),At.endStruct_(e)},e.prototype.visitUnion=function(t,e){Mt.startTypeIdsVector(e,t.typeIds.length);var n=Mt.createTypeIdsVector(e,t.typeIds);return Mt.startUnion(e),Mt.addMode(e,t.mode),Mt.addTypeIds(e,n),Mt.endUnion(e)},e.prototype.visitDictionary=function(t,e){var n=this.visit(t.indices,e);return kt.startDictionaryEncoding(e),kt.addId(e,new dt(t.id,0)),kt.addIsOrdered(e,t.isOrdered),void 0!==n&&kt.addIndexType(e,n),kt.endDictionaryEncoding(e)},e.prototype.visitFixedSizeBinary=function(t,e){return jt.startFixedSizeBinary(e),jt.addByteWidth(e,t.byteWidth),jt.endFixedSizeBinary(e)},e.prototype.visitFixedSizeList=function(t,e){return xt.startFixedSizeList(e),xt.addListSize(e,t.listSize),xt.endFixedSizeList(e)},e.prototype.visitMap=function(t,e){return Et.startMap(e),Et.addKeysSorted(e,t.keysSorted),Et.endMap(e)},e}(rt.a));function Ft(t){return new Xt(t.count,function t(e){return(e||[]).reduce(function(e,n){return r.j(e,[new ne(n.count,(i=n.VALIDITY,(i||[]).reduce(function(t,e){return t+ +(0===e)},0)))],t(n.children));var i},[])}(t.columns),function t(e,n){void 0===n&&(n=[]);for(var r=-1,i=(e||[]).length;++r<i;){var o=e[r];o.VALIDITY&&n.push(new ee(n.length,o.VALIDITY.length)),o.TYPE&&n.push(new ee(n.length,o.TYPE.length)),o.OFFSET&&n.push(new ee(n.length,o.OFFSET.length)),o.DATA&&n.push(new ee(n.length,o.DATA.length)),n=t(o.children,n)}return n}(t.columns))}function Pt(t,e,n){return(t.children||[]).filter(Boolean).map(function(t){return s.fromJSON(t,e,n)})}function Lt(t){return new Map(Object.entries(t||{}))}function Ut(t){return new o.o(t.isSigned,t.bitWidth)}function Ct(t,e){var n=t.type.name;switch(n){case"NONE":return new o.c;case"null":return new o.w;case"binary":return new o.a;case"utf8":return new o.G;case"bool":return new o.b;case"list":return new o.u((e||[])[0]);case"struct":case"struct_":return new o.x(e||[])}switch(n){case"int":var r=t.type;return new o.o(r.isSigned,r.bitWidth);case"floatingpoint":return r=t.type,new o.k(L.f[r.precision]);case"decimal":return r=t.type,new o.g(r.scale,r.precision);case"date":return r=t.type,new o.f(L.b[r.unit]);case"time":return r=t.type,new o.y(L.g[r.unit],r.bitWidth);case"timestamp":return r=t.type,new o.z(L.g[r.unit],r.timezone);case"interval":return r=t.type,new o.t(L.c[r.unit]);case"union":return r=t.type,new o.F(L.i[r.mode],r.typeIds||[],e||[]);case"fixedsizebinary":return r=t.type,new o.i(r.byteWidth);case"fixedsizelist":return r=t.type,new o.j(r.listSize,(e||[])[0]);case"map":return r=t.type,new o.v(e||[],r.keysSorted)}throw new Error(\'Unrecognized type: "\'+n+\'"\')}var Rt=C.a.Long,Nt=C.a.Builder,Yt=C.a.ByteBuffer,Vt=U.a.apache.arrow.flatbuf.Type,zt=U.a.apache.arrow.flatbuf.Field,Wt=U.a.apache.arrow.flatbuf.Schema,Ht=U.a.apache.arrow.flatbuf.Buffer,Gt=pt.a.apache.arrow.flatbuf.Message,qt=U.a.apache.arrow.flatbuf.KeyValue,Zt=pt.a.apache.arrow.flatbuf.FieldNode,Jt=U.a.apache.arrow.flatbuf.Endianness,Kt=pt.a.apache.arrow.flatbuf.RecordBatch,$t=pt.a.apache.arrow.flatbuf.DictionaryBatch,Qt=function(){function t(t,e,n,r){this._version=e,this._headerType=n,this.body=new Uint8Array(0),r&&(this._createHeader=function(){return r}),this._bodyLength="number"==typeof t?t:t.low}return t.fromJSON=function(e,n){var r=new t(0,L.e.V4,n);return r._createHeader=function(t,e){return function(){switch(e){case L.d.Schema:return a.fromJSON(t);case L.d.RecordBatch:return Xt.fromJSON(t);case L.d.DictionaryBatch:return te.fromJSON(t)}throw new Error("Unrecognized Message type: { name: "+L.d[e]+", type: "+e+" }")}}(e,n),r},t.decode=function(e){e=new Yt(Object(v.e)(e));var n=Gt.getRootAsMessage(e),r=n.bodyLength(),i=n.version(),o=n.headerType(),s=new t(r,i,o);return s._createHeader=function(t,e){return function(){switch(e){case L.d.Schema:return a.decode(t.header(new Wt));case L.d.RecordBatch:return Xt.decode(t.header(new Kt),t.version());case L.d.DictionaryBatch:return te.decode(t.header(new $t),t.version())}throw new Error("Unrecognized Message type: { name: "+L.d[e]+", type: "+e+" }")}}(n,o),s},t.encode=function(t){var e=new Nt,n=-1;return t.isSchema()?n=a.encode(e,t.header()):t.isRecordBatch()?n=Xt.encode(e,t.header()):t.isDictionaryBatch()&&(n=te.encode(e,t.header())),Gt.startMessage(e),Gt.addVersion(e,L.e.V4),Gt.addHeader(e,n),Gt.addHeaderType(e,t.headerType),Gt.addBodyLength(e,new Rt(t.bodyLength,0)),Gt.finishMessageBuffer(e,Gt.endMessage(e)),e.asUint8Array()},t.from=function(e,n){if(void 0===n&&(n=0),e instanceof a)return new t(0,L.e.V4,L.d.Schema,e);if(e instanceof Xt)return new t(n,L.e.V4,L.d.RecordBatch,e);if(e instanceof te)return new t(n,L.e.V4,L.d.DictionaryBatch,e);throw new Error("Unrecognized Message header: "+e)},Object.defineProperty(t.prototype,"type",{get:function(){return this.headerType},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"version",{get:function(){return this._version},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"headerType",{get:function(){return this._headerType},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"bodyLength",{get:function(){return this._bodyLength},enumerable:!0,configurable:!0}),t.prototype.header=function(){return this._createHeader()},t.prototype.isSchema=function(){return this.headerType===L.d.Schema},t.prototype.isRecordBatch=function(){return this.headerType===L.d.RecordBatch},t.prototype.isDictionaryBatch=function(){return this.headerType===L.d.DictionaryBatch},t}(),Xt=function(){function t(t,e,n){this._nodes=e,this._buffers=n,this._length="number"==typeof t?t:t.low}return Object.defineProperty(t.prototype,"nodes",{get:function(){return this._nodes},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"length",{get:function(){return this._length},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"buffers",{get:function(){return this._buffers},enumerable:!0,configurable:!0}),t}(),te=function(){function t(t,e,n){void 0===n&&(n=!1),this._data=t,this._isDelta=n,this._id="number"==typeof e?e:e.low}return Object.defineProperty(t.prototype,"id",{get:function(){return this._id},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"data",{get:function(){return this._data},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isDelta",{get:function(){return this._isDelta},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"length",{get:function(){return this.data.length},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"nodes",{get:function(){return this.data.nodes},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"buffers",{get:function(){return this.data.buffers},enumerable:!0,configurable:!0}),t}(),ee=function(t,e){this.offset="number"==typeof t?t:t.low,this.length="number"==typeof e?e:e.low},ne=function(t,e){this.length="number"==typeof t?t:t.low,this.nullCount="number"==typeof e?e:e.low};function re(t,e,n){for(var r=[],i=void 0,o=-1,a=-1,u=t.childrenLength();++o<u;)(i=t.children(o))&&(r[++a]=s.decode(i,e,n));return r}function ie(t){var e=new Map;if(t)for(var n=void 0,r=void 0,i=-1,o=0|t.customMetadataLength();++i<o;)(n=t.customMetadata(i))&&null!=(r=n.key())&&e.set(r,n.value());return e}function oe(t){return new o.o(t.isSigned(),t.bitWidth())}function ae(t,e){var n=t.typeType();switch(n){case Vt.NONE:return new o.c;case Vt.Null:return new o.w;case Vt.Binary:return new o.a;case Vt.Utf8:return new o.G;case Vt.Bool:return new o.b;case Vt.List:return new o.u((e||[])[0]);case Vt.Struct_:return new o.x(e||[])}switch(n){case Vt.Int:var r=t.type(new U.a.apache.arrow.flatbuf.Int);return new o.o(r.isSigned(),r.bitWidth());case Vt.FloatingPoint:return r=t.type(new U.a.apache.arrow.flatbuf.FloatingPoint),new o.k(r.precision());case Vt.Decimal:return r=t.type(new U.a.apache.arrow.flatbuf.Decimal),new o.g(r.scale(),r.precision());case Vt.Date:return r=t.type(new U.a.apache.arrow.flatbuf.Date),new o.f(r.unit());case Vt.Time:return r=t.type(new U.a.apache.arrow.flatbuf.Time),new o.y(r.unit(),r.bitWidth());case Vt.Timestamp:return r=t.type(new U.a.apache.arrow.flatbuf.Timestamp),new o.z(r.unit(),r.timezone());case Vt.Interval:return r=t.type(new U.a.apache.arrow.flatbuf.Interval),new o.t(r.unit());case Vt.Union:return r=t.type(new U.a.apache.arrow.flatbuf.Union),new o.F(r.mode(),r.typeIdsArray()||[],e||[]);case Vt.FixedSizeBinary:return r=t.type(new U.a.apache.arrow.flatbuf.FixedSizeBinary),new o.i(r.byteWidth());case Vt.FixedSizeList:return r=t.type(new U.a.apache.arrow.flatbuf.FixedSizeList),new o.j(r.listSize(),(e||[])[0]);case Vt.Map:return r=t.type(new U.a.apache.arrow.flatbuf.Map),new o.v(e||[],r.keysSorted())}throw new Error(\'Unrecognized type: "\'+Vt[n]+\'" (\'+n+")")}s.encode=function(t,e){var n=-1,i=-1,a=-1,u=e.type,c=e.typeId;o.c.isDictionary(u)?(c=u.dictionary.typeId,a=Bt.visit(u,t),i=Bt.visit(u.dictionary,t)):i=Bt.visit(u,t);var f=(u.children||[]).map(function(e){return s.encode(t,e)}),l=zt.createChildrenVector(t,f),h=e.metadata&&e.metadata.size>0?zt.createCustomMetadataVector(t,r.j(e.metadata).map(function(e){var n=r.i(e,2),i=n[0],o=n[1],a=t.createString(""+i),s=t.createString(""+o);return qt.startKeyValue(t),qt.addKey(t,a),qt.addValue(t,s),qt.endKeyValue(t)})):-1;return e.name&&(n=t.createString(e.name)),zt.startField(t),zt.addType(t,i),zt.addTypeType(t,c),zt.addChildren(t,l),zt.addNullable(t,!!e.nullable),-1!==n&&zt.addName(t,n),-1!==a&&zt.addDictionary(t,a),-1!==h&&zt.addCustomMetadata(t,h),zt.endField(t)},s.decode=function(t,e,n){var r,i,a,u,c,f,l;return e&&n&&(f=t.dictionary())?e.has(r=f.id().low)?(u=(u=f.indexType())?oe(u):new o.q,c=new o.h(e.get(r),u,r,f.isOrdered()),l=new s(t.name(),c,t.nullable(),ie(t)),n.get(r).push(i=l)):(u=(u=f.indexType())?oe(u):new o.q,e.set(r,a=ae(t,re(t))),c=new o.h(a,u,r,f.isOrdered()),l=new s(t.name(),c,t.nullable(),ie(t)),n.set(r,[i=l])):(a=ae(t,re(t,e,n)),i=new s(t.name(),a,t.nullable(),ie(t))),i||null},s.fromJSON=function(t,e,n){var r,i,a,u,c,f,l;return e&&n&&(u=t.dictionary)?e.has(r=u.id)?(i=(i=u.indexType)?Ut(i):new o.q,f=new o.h(e.get(r),i,r,u.isOrdered),l=new s(t.name,f,t.nullable,Lt(t.customMetadata)),n.get(r).push(a=l)):(i=(i=u.indexType)?Ut(i):new o.q,e.set(r,c=Ct(t,Pt(t))),f=new o.h(c,i,r,u.isOrdered),l=new s(t.name,f,t.nullable,Lt(t.customMetadata)),n.set(r,[a=l])):(c=Ct(t,Pt(t,e,n)),a=new s(t.name,c,t.nullable,Lt(t.customMetadata))),a||null},a.encode=function(t,e){var n=e.fields.map(function(e){return s.encode(t,e)});Wt.startFieldsVector(t,n.length);var i=Wt.createFieldsVector(t,n),o=e.metadata&&e.metadata.size>0?Wt.createCustomMetadataVector(t,r.j(e.metadata).map(function(e){var n=r.i(e,2),i=n[0],o=n[1],a=t.createString(""+i),s=t.createString(""+o);return qt.startKeyValue(t),qt.addKey(t,a),qt.addValue(t,s),qt.endKeyValue(t)})):-1;return Wt.startSchema(t),Wt.addFields(t,i),Wt.addEndianness(t,ue?Jt.Little:Jt.Big),-1!==o&&Wt.addCustomMetadata(t,o),Wt.endSchema(t)},a.decode=function(t,e,n){void 0===e&&(e=new Map),void 0===n&&(n=new Map);var r=function(t,e,n){for(var r=[],i=void 0,o=-1,a=-1,u=t.fieldsLength();++o<u;)(i=t.fields(o))&&(r[++a]=s.decode(i,e,n));return r}(t,e,n);return new a(r,ie(t),e,n)},a.fromJSON=function(t,e,n){return void 0===e&&(e=new Map),void 0===n&&(n=new Map),new a(function(t,e,n){return(t.fields||[]).filter(Boolean).map(function(t){return s.fromJSON(t,e,n)})}(t,e,n),Lt(t.customMetadata),e,n)},Xt.encode=function(t,e){var n=e.nodes||[],r=e.buffers||[];Kt.startNodesVector(t,n.length),n.slice().reverse().forEach(function(e){return ne.encode(t,e)});var i=t.endVector();Kt.startBuffersVector(t,r.length),r.slice().reverse().forEach(function(e){return ee.encode(t,e)});var o=t.endVector();return Kt.startRecordBatch(t),Kt.addLength(t,new Rt(e.length,0)),Kt.addNodes(t,i),Kt.addBuffers(t,o),Kt.endRecordBatch(t)},Xt.decode=function(t,e){return void 0===e&&(e=L.e.V4),new Xt(t.length(),function(t){for(var e=[],n=void 0,r=-1,i=-1,o=t.nodesLength();++r<o;)(n=t.nodes(r))&&(e[++i]=ne.decode(n));return e}(t),function(t,e){for(var n=[],r=void 0,i=-1,o=-1,a=t.buffersLength();++i<a;)(r=t.buffers(i))&&(e<L.e.V4&&(r.bb_pos+=8*(i+1)),n[++o]=ee.decode(r));return n}(t,e))},Xt.fromJSON=Ft,te.encode=function(t,e){var n=Xt.encode(t,e.data);return $t.startDictionaryBatch(t),$t.addId(t,new Rt(e.id,0)),$t.addIsDelta(t,e.isDelta),$t.addData(t,n),$t.endDictionaryBatch(t)},te.decode=function(t,e){return void 0===e&&(e=L.e.V4),new te(Xt.decode(t.data(),e),t.id(),t.isDelta())},te.fromJSON=function(t){return new te(Ft(t.data),t.id,t.isDelta)},ne.encode=function(t,e){return Zt.createFieldNode(t,new Rt(e.length,0),new Rt(e.nullCount,0))},ne.decode=function(t){return new ne(t.length(),t.nullCount())},ee.encode=function(t,e){return Ht.createBuffer(t,new Rt(e.offset,0),new Rt(e.length,0))},ee.decode=function(t){return new ee(t.offset(),t.length())};for(var se,ue=(se=new ArrayBuffer(2),new DataView(se).setInt16(0,256,!0),256===new Int16Array(se)[0]),ce=C.a.ByteBuffer,fe=function(t){return"Expected "+L.d[t]+" Message in stream, but was null or length 0."},le=function(t){return"Header pointer of flatbuffer-encoded "+L.d[t]+" Message is null or length 0."},he=function(t,e){return"Expected to read "+t+" metadata bytes, but only read "+e+"."},pe=function(t,e){return"Expected to read "+t+" bytes for message body, but only read "+e+"."},de=function(){function t(t){this.source=t instanceof $?t:new $(t)}return t.prototype[Symbol.iterator]=function(){return this},t.prototype.next=function(){var t;return(t=this.readMetadataLength()).done?J.c:(t=this.readMetadata(t.value)).done?J.c:t},t.prototype.throw=function(t){return this.source.throw(t)},t.prototype.return=function(t){return this.source.return(t)},t.prototype.readMessage=function(t){var e;if((e=this.next()).done)return null;if(null!=t&&e.value.headerType!==t)throw new Error(fe(t));return e.value},t.prototype.readMessageBody=function(t){if(t<=0)return new Uint8Array(0);var e=Object(v.e)(this.source.read(t));if(e.byteLength<t)throw new Error(pe(t,e.byteLength));return e.byteOffset%8==0&&e.byteOffset+e.byteLength<=e.buffer.byteLength?e:e.slice()},t.prototype.readSchema=function(t){void 0===t&&(t=!1);var e=L.d.Schema,n=this.readMessage(e),r=n&&n.header();if(t&&!r)throw new Error(le(e));return r},t.prototype.readMetadataLength=function(){var t=this.source.read(ve),e=t&&new ce(t),n=+(e&&e.readInt32(0));return{done:n<=0,value:n}},t.prototype.readMetadata=function(t){var e=this.source.read(t);if(!e)return J.c;if(e.byteLength<t)throw new Error(he(t,e.byteLength));return{done:!1,value:Qt.decode(e)}},t}(),ye=function(){function t(t,e){this.source=t instanceof Q?t:Object(x.d)(t)?new nt(t,e):new Q(t)}return t.prototype[Symbol.asyncIterator]=function(){return this},t.prototype.next=function(){return r.f(this,void 0,void 0,function(){var t;return r.h(this,function(e){switch(e.label){case 0:return[4,this.readMetadataLength()];case 1:return(t=e.sent()).done?[2,J.c]:[4,this.readMetadata(t.value)];case 2:return(t=e.sent()).done?[2,J.c]:[2,t]}})})},t.prototype.throw=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,this.source.throw(t)];case 1:return[2,e.sent()]}})})},t.prototype.return=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,this.source.return(t)];case 1:return[2,e.sent()]}})})},t.prototype.readMessage=function(t){return r.f(this,void 0,void 0,function(){var e;return r.h(this,function(n){switch(n.label){case 0:return[4,this.next()];case 1:if((e=n.sent()).done)return[2,null];if(null!=t&&e.value.headerType!==t)throw new Error(fe(t));return[2,e.value]}})})},t.prototype.readMessageBody=function(t){return r.f(this,void 0,void 0,function(){var e,n;return r.h(this,function(r){switch(r.label){case 0:return t<=0?[2,new Uint8Array(0)]:(n=v.e,[4,this.source.read(t)]);case 1:if((e=n.apply(void 0,[r.sent()])).byteLength<t)throw new Error(pe(t,e.byteLength));return[2,e.byteOffset%8==0&&e.byteOffset+e.byteLength<=e.buffer.byteLength?e:e.slice()]}})})},t.prototype.readSchema=function(t){return void 0===t&&(t=!1),r.f(this,void 0,void 0,function(){var e,n,i;return r.h(this,function(r){switch(r.label){case 0:return e=L.d.Schema,[4,this.readMessage(e)];case 1:if(n=r.sent(),i=n&&n.header(),t&&!i)throw new Error(le(e));return[2,i]}})})},t.prototype.readMetadataLength=function(){return r.f(this,void 0,void 0,function(){var t,e,n;return r.h(this,function(r){switch(r.label){case 0:return[4,this.source.read(ve)];case 1:return t=r.sent(),e=t&&new ce(t),[2,{done:(n=+(e&&e.readInt32(0)))<=0,value:n}]}})})},t.prototype.readMetadata=function(t){return r.f(this,void 0,void 0,function(){var e;return r.h(this,function(n){switch(n.label){case 0:return[4,this.source.read(t)];case 1:if(!(e=n.sent()))return[2,J.c];if(e.byteLength<t)throw new Error(he(t,e.byteLength));return[2,{done:!1,value:Qt.decode(e)}]}})})},t}(),be=function(t){function e(e){var n=t.call(this,new Uint8Array(0))||this;return n._schema=!1,n._body=[],n._batchIndex=0,n._dictionaryIndex=0,n._json=e instanceof J.a?e:new J.a(e),n}return r.g(e,t),e.prototype.next=function(){var t=this._json,e=this._batchIndex,n=this._dictionaryIndex,r=t.batches.length,i=t.dictionaries.length;if(!this._schema)return this._schema=!0,{value:Qt.fromJSON(t.schema,L.d.Schema),done:e>=r&&n>=i};if(n<i){var o=t.dictionaries[this._dictionaryIndex++];return this._body=o.data.columns,{done:!1,value:Qt.fromJSON(o,L.d.DictionaryBatch)}}return e<r?(o=t.batches[this._batchIndex++],this._body=o.columns,{done:!1,value:Qt.fromJSON(o,L.d.RecordBatch)}):(this._body=[],J.c)},e.prototype.readMessageBody=function(t){return function t(e){return(e||[]).reduce(function(e,n){return r.j(e,n.VALIDITY&&[n.VALIDITY]||[],n.TYPE&&[n.TYPE]||[],n.OFFSET&&[n.OFFSET]||[],n.DATA&&[n.DATA]||[],t(n.children))},[])}(this._body)},e.prototype.readMessage=function(t){var e;if((e=this.next()).done)return null;if(null!=t&&e.value.headerType!==t)throw new Error(fe(t));return e.value},e.prototype.readSchema=function(){var t=L.d.Schema,e=this.readMessage(t),n=e&&e.header();if(!e||!n)throw new Error(le(t));return n},e}(de),ve=4,me=new Uint8Array("ARROW1".length),_e=0;_e<"ARROW1".length;_e+=1)me[_e]="ARROW1".charCodeAt(_e);function ge(t,e){void 0===e&&(e=0);for(var n=-1,r=me.length;++n<r;)if(me[n]!==t[e+n])return!1;return!0}var we=me.length,Se=we+ve,Oe=2*we+ve,Te=function(t){function e(e){var n=t.call(this)||this;return n._impl=e,n}return r.g(e,t),Object.defineProperty(e.prototype,"closed",{get:function(){return this._impl.closed},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"schema",{get:function(){return this._impl.schema},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"autoDestroy",{get:function(){return this._impl.autoDestroy},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dictionaries",{get:function(){return this._impl.dictionaries},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"numDictionaries",{get:function(){return this._impl.numDictionaries},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"numRecordBatches",{get:function(){return this._impl.numRecordBatches},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"footer",{get:function(){return this._impl.isFile()?this._impl.footer:null},enumerable:!0,configurable:!0}),e.prototype.isSync=function(){return this._impl.isSync()},e.prototype.isAsync=function(){return this._impl.isAsync()},e.prototype.isFile=function(){return this._impl.isFile()},e.prototype.isStream=function(){return this._impl.isStream()},e.prototype.next=function(){return this._impl.next()},e.prototype.throw=function(t){return this._impl.throw(t)},e.prototype.return=function(t){return this._impl.return(t)},e.prototype.cancel=function(){return this._impl.cancel()},e.prototype.reset=function(t){return this._impl.reset(t),this},e.prototype.open=function(t){var e=this,n=this._impl.open(t);return Object(x.g)(n)?n.then(function(){return e}):this},e.prototype.readRecordBatch=function(t){return this._impl.isFile()?this._impl.readRecordBatch(t):null},e.prototype[Symbol.iterator]=function(){return this._impl[Symbol.iterator]()},e.prototype[Symbol.asyncIterator]=function(){return this._impl[Symbol.asyncIterator]()},e.prototype.toDOMStream=function(){var t,e,n=this;return q.a.toDOMStream(this.isSync()?((t={})[Symbol.iterator]=function(){return n},t):((e={})[Symbol.asyncIterator]=function(){return n},e))},e.prototype.toNodeStream=function(){var t,e,n=this;return q.a.toNodeStream(this.isSync()?((t={})[Symbol.iterator]=function(){return n},t):((e={})[Symbol.asyncIterator]=function(){return n},e),{objectMode:!0})},e.throughNode=function(t){throw new Error(\'"throughNode" not available in this environment\')},e.throughDOM=function(t,e){throw new Error(\'"throughDOM" not available in this environment\')},e.from=function(t){return t instanceof e?t:Object(x.a)(t)?function(t){return new Ie(new Fe(t))}(t):Object(x.d)(t)?function(t){return r.f(this,void 0,void 0,function(){var e,n,i;return r.h(this,function(r){switch(r.label){case 0:return[4,t.stat()];case 1:return e=r.sent().size,n=new nt(t,e),e>=Oe?(i=ge,[4,n.readAt(0,we+7&-8)]):[3,3];case 2:if(i.apply(void 0,[r.sent()]))return[2,new Me(new Be(n))];r.label=3;case 3:return[2,new De(new xe(n))]}})})}(t):Object(x.g)(t)?r.f(this,void 0,void 0,function(){var n,i;return r.h(this,function(r){switch(r.label){case 0:return i=(n=e).from,[4,t];case 1:return[4,i.apply(n,[r.sent()])];case 2:return[2,r.sent()]}})}):Object(x.c)(t)||Object(x.h)(t)||Object(x.i)(t)||Object(x.b)(t)?function(t){return r.f(this,void 0,void 0,function(){var e,n,i,o,a;return r.h(this,function(s){switch(s.label){case 0:return[4,t.peek(we+7&-8)];case 1:return(e=s.sent())&&e.byteLength>=4?ge(e)?[3,2]:(i=new De(new xe(t)),[3,4]):[3,5];case 2:return o=Ae.bind,a=Ee.bind,[4,t.read()];case 3:i=new(o.apply(Ae,[void 0,new(a.apply(Ee,[void 0,s.sent()]))])),s.label=4;case 4:return n=i,[3,6];case 5:n=new De(new xe(function(){return r.c(this,arguments,function(){return r.h(this,function(t){return[2]})})}())),s.label=6;case 6:return[2,n]}})})}(new Q(t)):function(t){var e=t.peek(we+7&-8);return e&&e.byteLength>=4?ge(e)?new Ae(new Ee(t.read())):new Ie(new je(t)):new Ie(new je(function(){return r.h(this,function(t){return[2]})}()))}(new $(t))},e.readAll=function(t){return t instanceof e?t.isSync()?Le(t):Ue(t):Object(x.a)(t)||ArrayBuffer.isView(t)||Object(x.e)(t)||Object(x.f)(t)?Le(t):Ue(t)},e}(J.d),Ie=function(t){function e(e){var n=t.call(this,e)||this;return n._impl=e,n}return r.g(e,t),e.prototype[Symbol.iterator]=function(){return this._impl[Symbol.iterator]()},e.prototype[Symbol.asyncIterator]=function(){return r.c(this,arguments,function(){return r.h(this,function(t){switch(t.label){case 0:return[5,r.k(r.b(r.d(this[Symbol.iterator]())))];case 1:return[4,r.e.apply(void 0,[t.sent()])];case 2:return t.sent(),[2]}})})},e}(Te),De=function(t){function e(e){var n=t.call(this,e)||this;return n._impl=e,n}return r.g(e,t),e.prototype[Symbol.iterator]=function(){throw new Error("AsyncRecordBatchStreamReader is not Iterable")},e.prototype[Symbol.asyncIterator]=function(){return this._impl[Symbol.asyncIterator]()},e}(Te),Ae=function(t){function e(e){var n=t.call(this,e)||this;return n._impl=e,n}return r.g(e,t),e}(Ie),Me=function(t){function e(e){var n=t.call(this,e)||this;return n._impl=e,n}return r.g(e,t),e}(De),ke=function(){function t(t){void 0===t&&(t=new Map),this.closed=!1,this.autoDestroy=!0,this._dictionaryIndex=0,this._recordBatchIndex=0,this.dictionaries=t}return Object.defineProperty(t.prototype,"numDictionaries",{get:function(){return this._dictionaryIndex},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"numRecordBatches",{get:function(){return this._recordBatchIndex},enumerable:!0,configurable:!0}),t.prototype.isSync=function(){return!1},t.prototype.isAsync=function(){return!1},t.prototype.isFile=function(){return!1},t.prototype.isStream=function(){return!1},t.prototype.reset=function(t){return this._dictionaryIndex=0,this._recordBatchIndex=0,this.schema=t,this.dictionaries=new Map,this},t.prototype._loadRecordBatch=function(t,e){return new P(this.schema,t.length,this._loadVectors(t,e,this.schema.fields))},t.prototype._loadDictionaryBatch=function(t,e){var n=t.id,r=t.isDelta,o=t.data,a=this.dictionaries,s=this.schema;if(r||!a.get(n)){var u=s.dictionaries.get(n),c=r?a.get(n).concat(i.a.new(this._loadVectors(o,e,[u])[0])):i.a.new(this._loadVectors(o,e,[u])[0]);return(s.dictionaryFields.get(n)||[]).forEach(function(t){return t.type.dictionaryVector=c}),c}return a.get(n)},t.prototype._loadVectors=function(t,e,n){return new lt(e,t.nodes,t.buffers).visitMany(n)},t}(),je=function(t){function e(e,n){var r=t.call(this,n)||this;return r._reader=Object(x.a)(e)?new be(r._handle=e):new de(r._handle=e),r}return r.g(e,t),e.prototype.isSync=function(){return!0},e.prototype.isStream=function(){return!0},e.prototype[Symbol.iterator]=function(){return this},e.prototype.cancel=function(){!this.closed&&(this.closed=!0)&&(this.reset()._reader.return(),this._reader=null,this.dictionaries=null)},e.prototype.open=function(t){return this.closed||(this.autoDestroy=Pe(this,t),this.schema||(this.schema=this._reader.readSchema())||this.cancel()),this},e.prototype.throw=function(t){return!this.closed&&this.autoDestroy&&(this.closed=!0)?this.reset()._reader.throw(t):J.c},e.prototype.return=function(t){return!this.closed&&this.autoDestroy&&(this.closed=!0)?this.reset()._reader.return(t):J.c},e.prototype.next=function(){if(this.closed)return J.c;for(var t,e=this._reader;t=this._readNextMessageAndValidate();)if(t.isSchema())this.reset(t.header());else{if(t.isRecordBatch()){this._recordBatchIndex++;var n=t.header(),r=e.readMessageBody(t.bodyLength);return{done:!1,value:this._loadRecordBatch(n,r)}}if(t.isDictionaryBatch()){this._dictionaryIndex++,n=t.header(),r=e.readMessageBody(t.bodyLength);var i=this._loadDictionaryBatch(n,r);this.dictionaries.set(n.id,i)}}return this.return()},e.prototype._readNextMessageAndValidate=function(t){return this._reader.readMessage(t)},e}(ke),xe=function(t){function e(e,n){var r=t.call(this,n)||this;return r._reader=new ye(r._handle=e),r}return r.g(e,t),e.prototype.isAsync=function(){return!0},e.prototype.isStream=function(){return!0},e.prototype[Symbol.asyncIterator]=function(){return this},e.prototype.cancel=function(){return r.f(this,void 0,void 0,function(){return r.h(this,function(t){switch(t.label){case 0:return this.closed||!(this.closed=!0)?[3,2]:[4,this.reset()._reader.return()];case 1:t.sent(),this._reader=null,this.dictionaries=null,t.label=2;case 2:return[2]}})})},e.prototype.open=function(t){return r.f(this,void 0,void 0,function(){var e,n;return r.h(this,function(r){switch(r.label){case 0:return this.closed?[3,4]:(this.autoDestroy=Pe(this,t),(e=this.schema)?[3,2]:(n=this,[4,this._reader.readSchema()]));case 1:e=n.schema=r.sent(),r.label=2;case 2:return e?[3,4]:[4,this.cancel()];case 3:r.sent(),r.label=4;case 4:return[2,this]}})})},e.prototype.throw=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return!this.closed&&this.autoDestroy&&(this.closed=!0)?[4,this.reset()._reader.throw(t)]:[3,2];case 1:return[2,e.sent()];case 2:return[2,J.c]}})})},e.prototype.return=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return!this.closed&&this.autoDestroy&&(this.closed=!0)?[4,this.reset()._reader.return(t)]:[3,2];case 1:return[2,e.sent()];case 2:return[2,J.c]}})})},e.prototype.next=function(){return r.f(this,void 0,void 0,function(){var t,e,n,i,o;return r.h(this,function(r){switch(r.label){case 0:if(this.closed)return[2,J.c];e=this._reader,r.label=1;case 1:return[4,this._readNextMessageAndValidate()];case 2:return(t=r.sent())?t.isSchema()?[4,this.reset(t.header())]:[3,4]:[3,9];case 3:return r.sent(),[3,8];case 4:return t.isRecordBatch()?(this._recordBatchIndex++,n=t.header(),[4,e.readMessageBody(t.bodyLength)]):[3,6];case 5:return i=r.sent(),[2,{done:!1,value:this._loadRecordBatch(n,i)}];case 6:return t.isDictionaryBatch()?(this._dictionaryIndex++,n=t.header(),[4,e.readMessageBody(t.bodyLength)]):[3,8];case 7:i=r.sent(),o=this._loadDictionaryBatch(n,i),this.dictionaries.set(n.id,o),r.label=8;case 8:return[3,1];case 9:return[4,this.return()];case 10:return[2,r.sent()]}})})},e.prototype._readNextMessageAndValidate=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(e){switch(e.label){case 0:return[4,this._reader.readMessage(t)];case 1:return[2,e.sent()]}})})},e}(ke),Ee=function(t){function e(e,n){return t.call(this,e instanceof et?e:new et(e),n)||this}return r.g(e,t),Object.defineProperty(e.prototype,"footer",{get:function(){return this._footer},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"numDictionaries",{get:function(){return this._footer?this._footer.numDictionaries:0},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"numRecordBatches",{get:function(){return this._footer?this._footer.numRecordBatches:0},enumerable:!0,configurable:!0}),e.prototype.isSync=function(){return!0},e.prototype.isFile=function(){return!0},e.prototype.open=function(e){var n,i;if(!this.closed&&!this._footer){this.schema=(this._footer=this._readFooter()).schema;try{for(var o=r.k(this._footer.dictionaryBatches()),a=o.next();!a.done;a=o.next())a.value&&this._readDictionaryBatch(this._dictionaryIndex++)}catch(t){n={error:t}}finally{try{a&&!a.done&&(i=o.return)&&i.call(o)}finally{if(n)throw n.error}}}return t.prototype.open.call(this,e)},e.prototype.readRecordBatch=function(t){if(this.closed)return null;this._footer||this.open();var e=this._footer&&this._footer.getRecordBatch(t);if(e&&this._handle.seek(e.offset)){var n=this._reader.readMessage(L.d.RecordBatch);if(n&&n.isRecordBatch()){var r=n.header(),i=this._reader.readMessageBody(n.bodyLength);return this._loadRecordBatch(r,i)}}return null},e.prototype._readDictionaryBatch=function(t){var e=this._footer&&this._footer.getDictionaryBatch(t);if(e&&this._handle.seek(e.offset)){var n=this._reader.readMessage(L.d.DictionaryBatch);if(n&&n.isDictionaryBatch()){var r=n.header(),i=this._reader.readMessageBody(n.bodyLength),o=this._loadDictionaryBatch(r,i);this.dictionaries.set(r.id,o)}}},e.prototype._readFooter=function(){var t=this._handle,e=t.size-Se,n=t.readInt32(e),r=t.readAt(e-n,n);return W.decode(r)},e.prototype._readNextMessageAndValidate=function(t){if(this._footer||this.open(),this._footer&&this._recordBatchIndex<this.numRecordBatches){var e=this._footer&&this._footer.getRecordBatch(this._recordBatchIndex);if(e&&this._handle.seek(e.offset))return this._reader.readMessage(t)}return null},e}(je),Be=function(t){function e(e){for(var n=[],r=1;r<arguments.length;r++)n[r-1]=arguments[r];var i="number"!=typeof n[0]?n.shift():void 0,o=n[0]instanceof Map?n.shift():void 0;return t.call(this,e instanceof nt?e:new nt(e,i),o)||this}return r.g(e,t),Object.defineProperty(e.prototype,"footer",{get:function(){return this._footer},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"numDictionaries",{get:function(){return this._footer?this._footer.numDictionaries:0},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"numRecordBatches",{get:function(){return this._footer?this._footer.numRecordBatches:0},enumerable:!0,configurable:!0}),e.prototype.isFile=function(){return!0},e.prototype.isAsync=function(){return!0},e.prototype.open=function(e){return r.f(this,void 0,void 0,function(){var n,i,o,a,s,u,c;return r.h(this,function(f){switch(f.label){case 0:return this.closed||this._footer?[3,10]:(o=this,a=this,[4,this._readFooter()]);case 1:o.schema=(a._footer=f.sent()).schema,f.label=2;case 2:f.trys.push([2,8,9,10]),s=r.k(this._footer.dictionaryBatches()),u=s.next(),f.label=3;case 3:return u.done?[3,7]:u.value?[4,this._readDictionaryBatch(this._dictionaryIndex++)]:[3,5];case 4:f.sent(),f.label=5;case 5:f.label=6;case 6:return u=s.next(),[3,3];case 7:return[3,10];case 8:return c=f.sent(),n={error:c},[3,10];case 9:try{u&&!u.done&&(i=s.return)&&i.call(s)}finally{if(n)throw n.error}return[7];case 10:return[4,t.prototype.open.call(this,e)];case 11:return[2,f.sent()]}})})},e.prototype.readRecordBatch=function(t){return r.f(this,void 0,void 0,function(){var e,n,i,o,a;return r.h(this,function(r){switch(r.label){case 0:return this.closed?[2,null]:this._footer?[3,2]:[4,this.open()];case 1:r.sent(),r.label=2;case 2:return e=this._footer&&this._footer.getRecordBatch(t),(n=e)?[4,this._handle.seek(e.offset)]:[3,4];case 3:n=r.sent(),r.label=4;case 4:return n?[4,this._reader.readMessage(L.d.RecordBatch)]:[3,7];case 5:return(i=r.sent())&&i.isRecordBatch()?(o=i.header(),[4,this._reader.readMessageBody(i.bodyLength)]):[3,7];case 6:return a=r.sent(),[2,this._loadRecordBatch(o,a)];case 7:return[2,null]}})})},e.prototype._readDictionaryBatch=function(t){return r.f(this,void 0,void 0,function(){var e,n,i,o,a,s;return r.h(this,function(r){switch(r.label){case 0:return e=this._footer&&this._footer.getDictionaryBatch(t),(n=e)?[4,this._handle.seek(e.offset)]:[3,2];case 1:n=r.sent(),r.label=2;case 2:return n?[4,this._reader.readMessage(L.d.DictionaryBatch)]:[3,5];case 3:return(i=r.sent())&&i.isDictionaryBatch()?(o=i.header(),[4,this._reader.readMessageBody(i.bodyLength)]):[3,5];case 4:a=r.sent(),s=this._loadDictionaryBatch(o,a),this.dictionaries.set(o.id,s),r.label=5;case 5:return[2]}})})},e.prototype._readFooter=function(){return r.f(this,void 0,void 0,function(){var t,e,n,i;return r.h(this,function(r){switch(r.label){case 0:return(t=this._handle)._pending?[4,t._pending]:[3,2];case 1:r.sent(),r.label=2;case 2:return e=t.size-Se,[4,t.readInt32(e)];case 3:return n=r.sent(),[4,t.readAt(e-n,n)];case 4:return i=r.sent(),[2,W.decode(i)]}})})},e.prototype._readNextMessageAndValidate=function(t){return r.f(this,void 0,void 0,function(){var e,n;return r.h(this,function(r){switch(r.label){case 0:return this._footer?[3,2]:[4,this.open()];case 1:r.sent(),r.label=2;case 2:return this._footer&&this._recordBatchIndex<this.numRecordBatches?(e=this._footer.getRecordBatch(this._recordBatchIndex),(n=e)?[4,this._handle.seek(e.offset)]:[3,4]):[3,6];case 3:n=r.sent(),r.label=4;case 4:return n?[4,this._reader.readMessage(t)]:[3,6];case 5:return[2,r.sent()];case 6:return[2,null]}})})},e}(xe),Fe=function(t){function e(e,n){return t.call(this,e,n)||this}return r.g(e,t),e.prototype._loadVectors=function(t,e,n){return new ht(e,t.nodes,t.buffers).visitMany(n)},e}(je);function Pe(t,e){return e&&"boolean"==typeof e.autoDestroy?e.autoDestroy:t.autoDestroy}function Le(t){var e;return r.h(this,function(n){switch(n.label){case 0:e=Te.from(t),n.label=1;case 1:if(n.trys.push([1,,6,7]),e.open({autoDestroy:!1}).closed)return[3,5];n.label=2;case 2:return[4,e];case 3:n.sent(),n.label=4;case 4:if(!e.reset().open().closed)return[3,2];n.label=5;case 5:return[3,7];case 6:return e.cancel(),[7];case 7:return[2]}})}function Ue(t){return r.c(this,arguments,function(){var e;return r.h(this,function(n){switch(n.label){case 0:return[4,r.e(Te.from(t))];case 1:e=n.sent(),n.label=2;case 2:return n.trys.push([2,,10,12]),[4,r.e(e.open({autoDestroy:!1}))];case 3:if(n.sent().closed)return[3,9];n.label=4;case 4:return[4,r.e(e)];case 5:return[4,n.sent()];case 6:n.sent(),n.label=7;case 7:return[4,r.e(e.reset().open())];case 8:if(!n.sent().closed)return[3,4];n.label=9;case 9:return[3,12];case 10:return[4,r.e(e.cancel())];case 11:return n.sent(),[7];case 12:return[2]}})})}var Ce,Re=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.prototype.asUtf8=function(){return i.a.new(this.data.clone(new o.G))},e}(B),Ne=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.from=function(t){var e=0,n=Object(it.d)(function(){var n,i,o,a,s,u;return r.h(this,function(c){switch(c.label){case 0:c.trys.push([0,5,6,7]),o=r.k(t),a=o.next(),c.label=1;case 1:return a.done?[3,4]:(s=a.value,e++,[4,s]);case 2:c.sent(),c.label=3;case 3:return a=o.next(),[3,1];case 4:return[3,7];case 5:return u=c.sent(),n={error:u},[3,7];case 6:try{a&&!a.done&&(i=o.return)&&i.call(o)}finally{if(n)throw n.error}return[7];case 7:return[2]}})}());return i.a.new(E.a.Bool(new o.b,0,e,0,null,n))},e}(B),Ye=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.from=function(t,e){switch(void 0===e&&(e=L.b.MILLISECOND),e){case L.b.DAY:var n=Int32Array.from(t.map(function(t){return t.valueOf()/864e5}));return i.a.new(E.a.Date(new o.d,0,t.length,0,null,n));case L.b.MILLISECOND:return n=ct.convertArray(t.map(function(t){return t.valueOf()})),i.a.new(E.a.Date(new o.e,0,t.length,0,null,n))}throw new TypeError(\'Unrecognized date unit "\'+L.b[e]+\'"\')},e}(B),Ve=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(Ye),ze=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(Ye),We=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(B),He=function(t){function e(e){var n=t.call(this,e)||this;return n._indices=i.a.new(e.clone(n.type.indices)),n}return r.g(e,t),e.from=function(t,e,n){var r=new o.h(t.type,e,null,null,t);return i.a.new(E.a.Dictionary(r,0,n.length,0,null,n))},Object.defineProperty(e.prototype,"indices",{get:function(){return this._indices},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dictionary",{get:function(){return this.data.type.dictionaryVector},enumerable:!0,configurable:!0}),e.prototype.isValid=function(t){return this._indices.isValid(t)},e.prototype.reverseLookup=function(t){return this.dictionary.indexOf(t)},e.prototype.getKey=function(t){return this._indices.get(t)},e.prototype.getValue=function(t){return this.dictionary.get(t)},e.prototype.setKey=function(t,e){return this._indices.set(t,e)},e.prototype.setValue=function(t,e){return this.dictionary.set(t,e)},e}(B),Ge=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(B),qe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(B),Ze=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.from=function(t){switch(t.constructor){case Uint16Array:return i.a.new(E.a.Float(new o.l,0,t.length,0,null,t));case Float32Array:return i.a.new(E.a.Float(new o.m,0,t.length,0,null,t));case Float64Array:return i.a.new(E.a.Float(new o.n,0,t.length,0,null,t))}throw new TypeError("Unrecognized Float data")},e}(B),Je=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(Ze),Ke=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(Ze),$e=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(Ze),Qe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(B),Xe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(Qe),tn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(Qe),en=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.from=function(t,e){if(!0===e)return t instanceof Int32Array?i.a.new(E.a.Int(new o.r,0,t.length,0,null,t)):i.a.new(E.a.Int(new o.D,0,t.length,0,null,t));switch(t.constructor){case Int8Array:return i.a.new(E.a.Int(new o.s,0,t.length,0,null,t));case Int16Array:return i.a.new(E.a.Int(new o.p,0,t.length,0,null,t));case Int32Array:return i.a.new(E.a.Int(new o.q,0,t.length,0,null,t));case Uint8Array:return i.a.new(E.a.Int(new o.E,0,t.length,0,null,t));case Uint16Array:return i.a.new(E.a.Int(new o.B,0,t.length,0,null,t));case Uint32Array:return i.a.new(E.a.Int(new o.C,0,t.length,0,null,t))}throw new TypeError("Unrecognized Int data")},e}(B),nn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(en),rn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(en),on=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(en),an=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(en),sn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(en),un=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(en),cn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(en),fn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(en),ln=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(B),hn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.prototype.asStruct=function(){return i.a.new(this.data.clone(new o.x(this.type.children)))},Object.defineProperty(e.prototype,"rowProxy",{get:function(){return this._rowProxy||(this._rowProxy=b.new(this.type.children||[],!0))},enumerable:!0,configurable:!0}),e}(B),pn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(B),dn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(B),yn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(dn),bn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(dn),vn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(dn),mn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(dn),_n=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(B),gn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(_n),wn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(_n),Sn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(_n),On=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(_n),Tn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),Object.defineProperty(e.prototype,"typeIdToChildIndex",{get:function(){return this.type.typeIdToChildIndex},enumerable:!0,configurable:!0}),e}(B),In=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),Object.defineProperty(e.prototype,"valueOffsets",{get:function(){return this.data.valueOffsets},enumerable:!0,configurable:!0}),e}(Tn),Dn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(Tn),An=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.from=function(t){var e=t.length,n=Object(Z.b)(t.join("")),r=t.reduce(function(t,e,n){return t[n+1]=t[n]+e.length,t},new Uint32Array(t.length+1));return i.a.new(E.a.Utf8(new o.G,0,e,0,null,r,n))},e.prototype.asBinary=function(){return i.a.new(this.data.clone(new o.a))},e}(B),Mn=((Ce={toJSON:function(){return\'"\'+Bn(this)+\'"\'},valueOf:function(){return En(this)},toString:function(){return Bn(this)}})[Symbol.toPrimitive]=function(t){return"number"===t?En(this):"string"===t||"function"!=typeof BigInt?Bn(this):BigInt(Bn(this))},Ce),kn=Object.assign({},Mn,{signed:!0,constructor:void 0}),jn=Object.assign({},Mn,{signed:!1,constructor:void 0}),xn=function(){function t(e,n){return void 0===n&&(n=e instanceof Int32Array),t.new(e,n)}return t.new=function(e,n){return void 0===n&&(n=e instanceof Int8Array||e instanceof Int16Array||e instanceof Int32Array),!0===n?t.signed(e):t.unsigned(e)},t.signed=function(t){var e=ArrayBuffer.isView(t)?t.constructor:Int32Array,n=Object(v.d)(e,t),r=new e(n.buffer,n.byteOffset,n.length);return Object.assign(r,kn)},t.unsigned=function(t){var e=ArrayBuffer.isView(t)?t.constructor:Uint32Array,n=Object(v.d)(e,t),r=new e(n.buffer,n.byteOffset,n.length);return Object.assign(r,jn)},t}();function En(t){for(var e=t.buffer,n=t.byteOffset,r=t.length,i=0,o=new Uint32Array(e,n,r),a=0,s=o.length;a<s;)i+=o[a++]+o[a++]*Math.pow(a,32);return i}function Bn(t){var e=t.buffer,n=t.byteOffset,r=t.length,i="",o=-1,a=new Uint32Array(2),s=new Uint16Array(e,n,2*r),u=new Uint32Array((s=new Uint16Array(s).reverse()).buffer),c=s.length-1;do{for(a[0]=s[o=0];o<c;)s[o++]=a[1]=a[0]/10,a[0]=(a[0]-10*a[1]<<16)+s[o];s[o]=a[1]=a[0]/10,a[0]=a[0]-10*a[1],i=""+a[0]+i}while(u[0]||u[1]||u[2]||u[3]);return i||"0"}var Fn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(rt.a),Pn=function(t,e){return 4294967296*t[e+1]+(t[e]>>>0)},Ln=function(t){return new Date(t)},Un=function(t,e,n){var r=e[n],i=e[n+1];return null!=r&&null!=i?t.subarray(r,i):null},Cn=function(t,e){return function(t,e){return Ln(function(t,e){return 864e5*t[e]}(t,e))}(t.values,e)},Rn=function(t,e){return function(t,e){return Ln(Pn(t,e))}(t.values,2*e)},Nn=function(t,e){var n=t.stride;return t.values[n*e]},Yn=function(t,e){var n=t.stride;return(t.values[n*e]-32767)/32767},Vn=function(t,e){var n=t.stride,r=t.values,i=t.type;return xn.new(r.subarray(n*e,n*(e+1)),i.isSigned)},zn=function(t,e){var n=t.values;return 1e3*Pn(n,2*e)},Wn=function(t,e){var n=t.values;return Pn(n,2*e)},Hn=function(t,e){return function(t,e){return t[e+1]/1e3*4294967296+(t[e]>>>0)/1e3}(t.values,2*e)},Gn=function(t,e){return function(t,e){return t[e+1]/1e6*4294967296+(t[e]>>>0)/1e6}(t.values,2*e)},qn=function(t,e){return t.values[t.stride*e]},Zn=function(t,e){return t.values[t.stride*e]},Jn=function(t,e){var n=t.values;return xn.new(n.subarray(2*e,2*(e+1)),!0)},Kn=function(t,e){var n=t.values;return xn.new(n.subarray(2*e,2*(e+1)),!0)},$n=function(t,e){return t.rowProxy.bind(t,e)},Qn=function(t,e){var n=t.typeIds,r=t.type.typeIdToChildIndex,i=t.getChildAt(r[n[e]]);return i?i.get(t.valueOffsets[e]):null},Xn=function(t,e){var n=t.typeIds,r=t.type.typeIdToChildIndex,i=t.getChildAt(r[n[e]]);return i?i.get(e):null},tr=function(t,e){return t.values.subarray(2*e,2*(e+1))},er=function(t,e){var n=t.values[e],r=new Int32Array(2);return r[0]=n/12|0,r[1]=n%12|0,r};Fn.prototype.visitNull=function(t,e){return null},Fn.prototype.visitBool=function(t,e){var n=t.offset+e;return 0!=(t.values[n>>3]&1<<n%8)},Fn.prototype.visitInt=function(t,e){return t.type.bitWidth<64?Nn(t,e):Vn(t,e)},Fn.prototype.visitInt8=Nn,Fn.prototype.visitInt16=Nn,Fn.prototype.visitInt32=Nn,Fn.prototype.visitInt64=Vn,Fn.prototype.visitUint8=Nn,Fn.prototype.visitUint16=Nn,Fn.prototype.visitUint32=Nn,Fn.prototype.visitUint64=Vn,Fn.prototype.visitFloat=function(t,e){return t.type.precision!==L.f.HALF?Nn(t,e):Yn(t,e)},Fn.prototype.visitFloat16=Yn,Fn.prototype.visitFloat32=Nn,Fn.prototype.visitFloat64=Nn,Fn.prototype.visitUtf8=function(t,e){var n=t.values,r=t.valueOffsets,i=Un(n,r,e);return null!==i?Object(Z.a)(i):null},Fn.prototype.visitBinary=function(t,e){var n=t.values,r=t.valueOffsets;return Un(n,r,e)},Fn.prototype.visitFixedSizeBinary=function(t,e){var n=t.stride;return t.values.subarray(n*e,n*(e+1))},Fn.prototype.visitDate=function(t,e){return t.type.unit===L.b.DAY?Cn(t,e):Rn(t,e)},Fn.prototype.visitDateDay=Cn,Fn.prototype.visitDateMillisecond=Rn,Fn.prototype.visitTimestamp=function(t,e){switch(t.type.unit){case L.g.SECOND:return zn(t,e);case L.g.MILLISECOND:return Wn(t,e);case L.g.MICROSECOND:return Hn(t,e);case L.g.NANOSECOND:return Gn(t,e)}},Fn.prototype.visitTimestampSecond=zn,Fn.prototype.visitTimestampMillisecond=Wn,Fn.prototype.visitTimestampMicrosecond=Hn,Fn.prototype.visitTimestampNanosecond=Gn,Fn.prototype.visitTime=function(t,e){switch(t.type.unit){case L.g.SECOND:return qn(t,e);case L.g.MILLISECOND:return Zn(t,e);case L.g.MICROSECOND:return Jn(t,e);case L.g.NANOSECOND:return Kn(t,e)}},Fn.prototype.visitTimeSecond=qn,Fn.prototype.visitTimeMillisecond=Zn,Fn.prototype.visitTimeMicrosecond=Jn,Fn.prototype.visitTimeNanosecond=Kn,Fn.prototype.visitDecimal=function(t,e){var n=t.values;return xn.new(n.subarray(4*e,4*(e+1)),!1)},Fn.prototype.visitList=function(t,e){var n=t.getChildAt(0),r=t.valueOffsets,i=t.stride;return n.slice(r[e*i],r[e*i+1])},Fn.prototype.visitStruct=$n,Fn.prototype.visitUnion=function(t,e){return t.type.mode===L.i.Dense?Qn(t,e):Xn(t,e)},Fn.prototype.visitDenseUnion=Qn,Fn.prototype.visitSparseUnion=Xn,Fn.prototype.visitDictionary=function(t,e){return t.getValue(t.getKey(e))},Fn.prototype.visitInterval=function(t,e){return t.type.unit===L.c.DAY_TIME?tr(t,e):er(t,e)},Fn.prototype.visitIntervalDayTime=tr,Fn.prototype.visitIntervalYearMonth=er,Fn.prototype.visitFixedSizeList=function(t,e){var n=t.getChildAt(0),r=t.stride;return n.slice(e*r,(e+1)*r)},Fn.prototype.visitMap=$n;var nr=new Fn,rr=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(rt.a),ir=function(t,e,n){t[e]=n%4294967296|0,t[e+1]=n/4294967296|0},or=function(t,e,n,r){var i=e[n],o=e[n+1];null!=i&&null!=o&&t.set(r.subarray(0,o-i),i)},ar=function(t,e,n){!function(t,e,n){t[e]=n/864e5|0}(t.values,e,n.valueOf())},sr=function(t,e,n){var r=t.values;ir(r,2*e,n.valueOf())},ur=function(t,e,n){var r=t.stride;t.values[r*e]=n},cr=function(t,e,n){var r=t.stride;t.values[r*e]=32767*n+32767},fr=function(t,e,n){var r=t.stride;t.values.set(n.subarray(0,r),r*e)},lr=function(t,e,n){var r=t.values;return ir(r,2*e,n/1e3)},hr=function(t,e,n){var r=t.values;return ir(r,2*e,n)},pr=function(t,e,n){return function(t,e,n){t[e]=1e3*n%4294967296|0,t[e+1]=1e3*n/4294967296|0}(t.values,2*e,n)},dr=function(t,e,n){return function(t,e,n){t[e]=1e6*n%4294967296|0,t[e+1]=1e6*n/4294967296|0}(t.values,2*e,n)},yr=function(t,e,n){t.values[t.stride*e]=n},br=function(t,e,n){t.values[t.stride*e]=n},vr=function(t,e,n){t.values.set(n.subarray(0,2),2*e)},mr=function(t,e,n){t.values.set(n.subarray(0,2),2*e)},_r=function(t,e,n){t.type.children.forEach(function(r,i){var o=r.name,a=t.getChildAt(i);a&&a.set(e,n[o])})},gr=function(t,e,n){var r=t.typeIds,i=t.type.typeIdToChildIndex,o=t.getChildAt(i[r[e]]);o&&o.set(t.valueOffsets[e],n)},wr=function(t,e,n){var r=t.typeIds,i=t.type.typeIdToChildIndex,o=t.getChildAt(i[r[e]]);o&&o.set(e,n)},Sr=function(t,e,n){t.values.set(n.subarray(0,2),2*e)},Or=function(t,e,n){t.values[e]=12*n[0]+n[1]%12};rr.prototype.visitBool=function(t,e,n){var r=t.offset,i=t.values,o=r+e;n?i[o>>3]|=1<<o%8:i[o>>3]&=~(1<<o%8)},rr.prototype.visitInt=function(t,e,n){t.type.bitWidth<64?ur(t,e,n):fr(t,e,n)},rr.prototype.visitInt8=ur,rr.prototype.visitInt16=ur,rr.prototype.visitInt32=ur,rr.prototype.visitInt64=fr,rr.prototype.visitUint8=ur,rr.prototype.visitUint16=ur,rr.prototype.visitUint32=ur,rr.prototype.visitUint64=fr,rr.prototype.visitFloat=function(t,e,n){t.type.precision!==L.f.HALF?ur(t,e,n):cr(t,e,n)},rr.prototype.visitFloat16=cr,rr.prototype.visitFloat32=ur,rr.prototype.visitFloat64=ur,rr.prototype.visitUtf8=function(t,e,n){var r=t.values,i=t.valueOffsets;or(r,i,e,Object(Z.b)(n))},rr.prototype.visitBinary=function(t,e,n){var r=t.values,i=t.valueOffsets;return or(r,i,e,n)},rr.prototype.visitFixedSizeBinary=function(t,e,n){var r=t.stride;t.values.set(n.subarray(0,r),r*e)},rr.prototype.visitDate=function(t,e,n){t.type.unit===L.b.DAY?ar(t,e,n):sr(t,e,n)},rr.prototype.visitDateDay=ar,rr.prototype.visitDateMillisecond=sr,rr.prototype.visitTimestamp=function(t,e,n){switch(t.type.unit){case L.g.SECOND:return lr(t,e,n);case L.g.MILLISECOND:return hr(t,e,n);case L.g.MICROSECOND:return pr(t,e,n);case L.g.NANOSECOND:return dr(t,e,n)}},rr.prototype.visitTimestampSecond=lr,rr.prototype.visitTimestampMillisecond=hr,rr.prototype.visitTimestampMicrosecond=pr,rr.prototype.visitTimestampNanosecond=dr,rr.prototype.visitTime=function(t,e,n){switch(t.type.unit){case L.g.SECOND:return yr(t,e,n);case L.g.MILLISECOND:return br(t,e,n);case L.g.MICROSECOND:return vr(t,e,n);case L.g.NANOSECOND:return mr(t,e,n)}},rr.prototype.visitTimeSecond=yr,rr.prototype.visitTimeMillisecond=br,rr.prototype.visitTimeMicrosecond=vr,rr.prototype.visitTimeNanosecond=mr,rr.prototype.visitDecimal=function(t,e,n){t.values.set(n.subarray(0,4),4*e)},rr.prototype.visitList=function(t,e,n){for(var r=t.getChildAt(0),i=t.valueOffsets,o=t.stride,a=-1,s=i[e*o],u=Math.min(n.length,i[e*o+1]-s);s<u;)r.set(s++,n.get(++a))},rr.prototype.visitStruct=_r,rr.prototype.visitUnion=function(t,e,n){t.type.mode===L.i.Dense?gr(t,e,n):wr(t,e,n)},rr.prototype.visitDenseUnion=gr,rr.prototype.visitSparseUnion=wr,rr.prototype.visitDictionary=function(t,e,n){var r=t.getKey(e);null!==r&&t.setValue(r,n)},rr.prototype.visitInterval=function(t,e,n){t.type.unit===L.c.DAY_TIME?Sr(t,e,n):Or(t,e,n)},rr.prototype.visitIntervalDayTime=Sr,rr.prototype.visitIntervalYearMonth=Or,rr.prototype.visitFixedSizeList=function(t,e,n){for(var r=t.getChildAt(0),i=t.stride,o=-1,a=e*i;++o<i;)r.set(a+o,n.get(o))},rr.prototype.visitMap=_r;var Tr=new rr,Ir=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(rt.a);function Dr(t,e,n){if(void 0===e)return-1;if(null===e)return function(t,e){var n,i,o=t.nullBitmap;if(!o||t.nullCount<=0)return-1;var a=0;try{for(var s=r.k(Object(it.c)(o,t.data.offset+(e||0),t.length,o,it.b)),u=s.next();!u.done;u=s.next()){if(!u.value)return a;++a}}catch(t){n={error:t}}finally{try{u&&!u.done&&(i=s.return)&&i.call(s)}finally{if(n)throw n.error}}return-1}(t,n);for(var i=_(e),o=(n||0)-1,a=t.length;++o<a;)if(i(t.get(o)))return o;return-1}function Ar(t,e,n){for(var r=_(e),i=(n||0)-1,o=t.length;++i<o;)if(r(t.get(i)))return i;return-1}Ir.prototype.visitNull=function(t,e){return null===e&&t.length>0?0:-1},Ir.prototype.visitBool=Dr,Ir.prototype.visitInt=Dr,Ir.prototype.visitInt8=Dr,Ir.prototype.visitInt16=Dr,Ir.prototype.visitInt32=Dr,Ir.prototype.visitInt64=Dr,Ir.prototype.visitUint8=Dr,Ir.prototype.visitUint16=Dr,Ir.prototype.visitUint32=Dr,Ir.prototype.visitUint64=Dr,Ir.prototype.visitFloat=Dr,Ir.prototype.visitFloat16=Dr,Ir.prototype.visitFloat32=Dr,Ir.prototype.visitFloat64=Dr,Ir.prototype.visitUtf8=Dr,Ir.prototype.visitBinary=Dr,Ir.prototype.visitFixedSizeBinary=Dr,Ir.prototype.visitDate=Dr,Ir.prototype.visitDateDay=Dr,Ir.prototype.visitDateMillisecond=Dr,Ir.prototype.visitTimestamp=Dr,Ir.prototype.visitTimestampSecond=Dr,Ir.prototype.visitTimestampMillisecond=Dr,Ir.prototype.visitTimestampMicrosecond=Dr,Ir.prototype.visitTimestampNanosecond=Dr,Ir.prototype.visitTime=Dr,Ir.prototype.visitTimeSecond=Dr,Ir.prototype.visitTimeMillisecond=Dr,Ir.prototype.visitTimeMicrosecond=Dr,Ir.prototype.visitTimeNanosecond=Dr,Ir.prototype.visitDecimal=Dr,Ir.prototype.visitList=Dr,Ir.prototype.visitStruct=Dr,Ir.prototype.visitUnion=Dr,Ir.prototype.visitDenseUnion=Ar,Ir.prototype.visitSparseUnion=Ar,Ir.prototype.visitDictionary=Dr,Ir.prototype.visitInterval=Dr,Ir.prototype.visitIntervalDayTime=Dr,Ir.prototype.visitIntervalYearMonth=Dr,Ir.prototype.visitFixedSizeList=Dr,Ir.prototype.visitMap=Dr;var Mr=new Ir,kr=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(rt.a);function jr(t){if(t.nullCount>0)return function(t){var e=nr.getVisitFn(t);return Object(it.c)(t.nullBitmap,t.offset,t.length,t,function(t,n,r,i){return 0!=(r&1<<i)?e(t,n):null})}(t);var e=t.type,n=t.typeId,i=t.length;return 1===t.stride&&(n===L.h.Timestamp||n===L.h.Int&&64!==e.bitWidth||n===L.h.Time&&64!==e.bitWidth||n===L.h.Float&&e.precision>0)?t.values.subarray(0,i)[Symbol.iterator]():function(e){var n;return r.h(this,function(r){switch(r.label){case 0:n=-1,r.label=1;case 1:return++n<i?[4,e(t,n)]:[3,4];case 2:r.sent(),r.label=3;case 3:return[3,1];case 4:return[2]}})}(nr.getVisitFn(t))}kr.prototype.visitNull=jr,kr.prototype.visitBool=jr,kr.prototype.visitInt=jr,kr.prototype.visitInt8=jr,kr.prototype.visitInt16=jr,kr.prototype.visitInt32=jr,kr.prototype.visitInt64=jr,kr.prototype.visitUint8=jr,kr.prototype.visitUint16=jr,kr.prototype.visitUint32=jr,kr.prototype.visitUint64=jr,kr.prototype.visitFloat=jr,kr.prototype.visitFloat16=jr,kr.prototype.visitFloat32=jr,kr.prototype.visitFloat64=jr,kr.prototype.visitUtf8=jr,kr.prototype.visitBinary=jr,kr.prototype.visitFixedSizeBinary=jr,kr.prototype.visitDate=jr,kr.prototype.visitDateDay=jr,kr.prototype.visitDateMillisecond=jr,kr.prototype.visitTimestamp=jr,kr.prototype.visitTimestampSecond=jr,kr.prototype.visitTimestampMillisecond=jr,kr.prototype.visitTimestampMicrosecond=jr,kr.prototype.visitTimestampNanosecond=jr,kr.prototype.visitTime=jr,kr.prototype.visitTimeSecond=jr,kr.prototype.visitTimeMillisecond=jr,kr.prototype.visitTimeMicrosecond=jr,kr.prototype.visitTimeNanosecond=jr,kr.prototype.visitDecimal=jr,kr.prototype.visitList=jr,kr.prototype.visitStruct=jr,kr.prototype.visitUnion=jr,kr.prototype.visitDenseUnion=jr,kr.prototype.visitSparseUnion=jr,kr.prototype.visitDictionary=jr,kr.prototype.visitInterval=jr,kr.prototype.visitIntervalDayTime=jr,kr.prototype.visitIntervalYearMonth=jr,kr.prototype.visitFixedSizeList=jr,kr.prototype.visitMap=jr;var xr=new kr,Er=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e}(rt.a);function Br(t){var e=t.type,n=t.length,i=t.stride;switch(e.typeId){case L.h.Int:case L.h.Decimal:case L.h.Time:case L.h.Timestamp:return t.values.subarray(0,n*i);case L.h.Float:return e.precision===L.f.HALF?new Float32Array(t[Symbol.iterator]()):t.values.subarray(0,n*i)}return r.j(xr.visit(t))}Er.prototype.visitNull=Br,Er.prototype.visitBool=Br,Er.prototype.visitInt=Br,Er.prototype.visitInt8=Br,Er.prototype.visitInt16=Br,Er.prototype.visitInt32=Br,Er.prototype.visitInt64=Br,Er.prototype.visitUint8=Br,Er.prototype.visitUint16=Br,Er.prototype.visitUint32=Br,Er.prototype.visitUint64=Br,Er.prototype.visitFloat=Br,Er.prototype.visitFloat16=Br,Er.prototype.visitFloat32=Br,Er.prototype.visitFloat64=Br,Er.prototype.visitUtf8=Br,Er.prototype.visitBinary=Br,Er.prototype.visitFixedSizeBinary=Br,Er.prototype.visitDate=Br,Er.prototype.visitDateDay=Br,Er.prototype.visitDateMillisecond=Br,Er.prototype.visitTimestamp=Br,Er.prototype.visitTimestampSecond=Br,Er.prototype.visitTimestampMillisecond=Br,Er.prototype.visitTimestampMicrosecond=Br,Er.prototype.visitTimestampNanosecond=Br,Er.prototype.visitTime=Br,Er.prototype.visitTimeSecond=Br,Er.prototype.visitTimeMillisecond=Br,Er.prototype.visitTimeMicrosecond=Br,Er.prototype.visitTimeNanosecond=Br,Er.prototype.visitDecimal=Br,Er.prototype.visitList=Br,Er.prototype.visitStruct=Br,Er.prototype.visitUnion=Br,Er.prototype.visitDenseUnion=Br,Er.prototype.visitSparseUnion=Br,Er.prototype.visitDictionary=Br,Er.prototype.visitInterval=Br,Er.prototype.visitIntervalDayTime=Br,Er.prototype.visitIntervalYearMonth=Br,Er.prototype.visitFixedSizeList=Br,Er.prototype.visitMap=Br;var Fr=new Er,Pr=function(t,e){return t+e},Lr=function(t){return"Cannot compute the byte width of variable-width column "+t},Ur=new(function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.prototype.visitNull=function(t){return 0},e.prototype.visitInt=function(t){return t.bitWidth/8},e.prototype.visitFloat=function(t){return t.ArrayType.BYTES_PER_ELEMENT},e.prototype.visitBinary=function(t){throw new Error(Lr(t))},e.prototype.visitUtf8=function(t){throw new Error(Lr(t))},e.prototype.visitBool=function(t){return 1/8},e.prototype.visitDecimal=function(t){return 16},e.prototype.visitDate=function(t){return 4*(t.unit+1)},e.prototype.visitTime=function(t){return t.bitWidth/8},e.prototype.visitTimestamp=function(t){return t.unit===L.g.SECOND?4:8},e.prototype.visitInterval=function(t){return 4*(t.unit+1)},e.prototype.visitList=function(t){throw new Error(Lr(t))},e.prototype.visitStruct=function(t){return this.visitFields(t.children).reduce(Pr,0)},e.prototype.visitUnion=function(t){return this.visitFields(t.children).reduce(Pr,0)},e.prototype.visitFixedSizeBinary=function(t){return t.byteWidth},e.prototype.visitFixedSizeList=function(t){return t.listSize*this.visitFields(t.children).reduce(Pr,0)},e.prototype.visitMap=function(t){return this.visitFields(t.children).reduce(Pr,0)},e.prototype.visitDictionary=function(t){return this.visit(t.indices)},e.prototype.visitFields=function(t){var e=this;return(t||[]).map(function(t){return e.visit(t.type)})},e.prototype.visitSchema=function(t){return this.visitFields(t.fields).reduce(Pr,0)},e}(rt.a)),Cr=new(function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.prototype.visitNull=function(){return pn},e.prototype.visitBool=function(){return Ne},e.prototype.visitInt=function(){return en},e.prototype.visitInt8=function(){return nn},e.prototype.visitInt16=function(){return rn},e.prototype.visitInt32=function(){return on},e.prototype.visitInt64=function(){return an},e.prototype.visitUint8=function(){return sn},e.prototype.visitUint16=function(){return un},e.prototype.visitUint32=function(){return cn},e.prototype.visitUint64=function(){return fn},e.prototype.visitFloat=function(){return Ze},e.prototype.visitFloat16=function(){return Je},e.prototype.visitFloat32=function(){return Ke},e.prototype.visitFloat64=function(){return $e},e.prototype.visitUtf8=function(){return An},e.prototype.visitBinary=function(){return Re},e.prototype.visitFixedSizeBinary=function(){return Ge},e.prototype.visitDate=function(){return Ye},e.prototype.visitDateDay=function(){return Ve},e.prototype.visitDateMillisecond=function(){return ze},e.prototype.visitTimestamp=function(){return dn},e.prototype.visitTimestampSecond=function(){return yn},e.prototype.visitTimestampMillisecond=function(){return bn},e.prototype.visitTimestampMicrosecond=function(){return vn},e.prototype.visitTimestampNanosecond=function(){return mn},e.prototype.visitTime=function(){return _n},e.prototype.visitTimeSecond=function(){return gn},e.prototype.visitTimeMillisecond=function(){return wn},e.prototype.visitTimeMicrosecond=function(){return Sn},e.prototype.visitTimeNanosecond=function(){return On},e.prototype.visitDecimal=function(){return We},e.prototype.visitList=function(){return ln},e.prototype.visitStruct=function(){return F},e.prototype.visitUnion=function(){return Tn},e.prototype.visitDenseUnion=function(){return In},e.prototype.visitSparseUnion=function(){return Dn},e.prototype.visitDictionary=function(){return He},e.prototype.visitInterval=function(){return Qe},e.prototype.visitIntervalDayTime=function(){return Xe},e.prototype.visitIntervalYearMonth=function(){return tn},e.prototype.visitFixedSizeList=function(){return qe},e.prototype.visitMap=function(){return hn},e}(rt.a));function Rr(t){return function(){return t(this)}}function Nr(t){return function(e,n){return t(this,e,n)}}i.a.new=function(t){for(var e,n=[],i=1;i<arguments.length;i++)n[i-1]=arguments[i];return new((e=Cr.getVisitFn(t.type)()).bind.apply(e,r.j([void 0,t],n)))},B.prototype.get=function(t){return nr.visit(this,t)},B.prototype.set=function(t,e){return Tr.visit(this,t,e)},B.prototype.indexOf=function(t,e){return Mr.visit(this,t,e)},B.prototype.toArray=function(){return Fr.visit(this)},B.prototype.getByteWidth=function(){return Ur.visit(this.type)},B.prototype[Symbol.iterator]=function(){return xr.visit(this)},B.prototype._bindDataAccessors=function(){var t,e=this.nullBitmap;e&&e.byteLength>0&&(this.get=(t=this.get,function(e){return this.isValid(e)?t.call(this,e):null}),this.set=function(t){return function(e,n){Object(it.f)(this.nullBitmap,this.offset+e,null!=n)&&t.call(this,e,n)}}(this.set))},Object.keys(L.h).filter(function(t){return t!==L.h.NONE&&t!==L.h[L.h.NONE]}).map(function(t){return L.h[t]}).filter(function(t){return"number"==typeof t}).forEach(function(t){var e;switch(t){case L.h.Int:e=[L.h.Int8,L.h.Int16,L.h.Int32,L.h.Int64,L.h.Uint8,L.h.Uint16,L.h.Uint32,L.h.Uint64];break;case L.h.Float:e=[L.h.Float16,L.h.Float32,L.h.Float64];break;case L.h.Date:e=[L.h.DateDay,L.h.DateMillisecond];break;case L.h.Time:e=[L.h.TimeSecond,L.h.TimeMillisecond,L.h.TimeMicrosecond,L.h.TimeNanosecond];break;case L.h.Timestamp:e=[L.h.TimestampSecond,L.h.TimestampMillisecond,L.h.TimestampMicrosecond,L.h.TimestampNanosecond];break;case L.h.Interval:e=[L.h.IntervalDayTime,L.h.IntervalYearMonth];break;case L.h.Union:e=[L.h.DenseUnion,L.h.SparseUnion];break;default:e=[t]}e.forEach(function(t){var e,n=Cr.visit(t);n.prototype.get=(e=nr.getVisitFn(t),function(t){return e(this,t)}),n.prototype.set=Nr(Tr.getVisitFn(t)),n.prototype.indexOf=Nr(Mr.getVisitFn(t)),n.prototype.toArray=Rr(Fr.getVisitFn(t)),n.prototype.getByteWidth=function(t){return function(){return t(this.type)}}(Ur.getVisitFn(t)),n.prototype[Symbol.iterator]=Rr(xr.getVisitFn(t))})});var Yr=function(t){function e(){var e=t.call(this)||this;return e._byteLength=0,e._nodes=[],e._buffers=[],e._bufferRegions=[],e}return r.g(e,t),e.assemble=function(){for(var t=[],n=0;n<arguments.length;n++)t[n]=arguments[n];var o=t.reduce(function t(e,n){return Array.isArray(n)?n.reduce(t,e):n instanceof P?r.j(e,n.schema.fields.map(function(t,e){return n.getChildAt(e)})):r.j(e,[n])},[]).filter(function(t){return t instanceof i.a});return(new e).visitMany(o)[0]},e.prototype.visit=function(e){if(!o.c.isDictionary(e.type)){var n=e.data,r=e.length,i=e.nullCount;if(r>2147483647)throw new RangeError("Cannot write arrays larger than 2^31 - 1 in length");Vr.call(this,i<=0?new Uint8Array(0):Object(it.g)(n.offset,r,n.nullBitmap)).nodes.push(new ne(r,i))}return t.prototype.visit.call(this,e)},e.prototype.visitNull=function(t){return this},e.prototype.visitDictionary=function(t){return this.visit(t.indices)},Object.defineProperty(e.prototype,"nodes",{get:function(){return this._nodes},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"buffers",{get:function(){return this._buffers},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"byteLength",{get:function(){return this._byteLength},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"bufferRegions",{get:function(){return this._bufferRegions},enumerable:!0,configurable:!0}),e}(rt.a);function Vr(t){var e=t.byteLength+7&-8;return this.buffers.push(t),this.bufferRegions.push(new ee(this._byteLength,e)),this._byteLength+=e,this}function zr(t){return Vr.call(this,t.values.subarray(0,t.length*t.stride))}function Wr(t){var e=t.length,n=t.values,r=t.valueOffsets,i=r[0],o=r[e],a=Math.min(o-i,n.byteLength-i);return Vr.call(this,Object(v.c)(-r[0],e,r)),Vr.call(this,n.subarray(i,i+a)),this}function Hr(t){var e=t.length,n=t.valueOffsets;return n&&Vr.call(this,Object(v.c)(n[0],e,n)),this.visit(t.getChildAt(0))}function Gr(t){return this.visitMany(t.type.children.map(function(e,n){return t.getChildAt(n)}).filter(Boolean))[0]}Yr.prototype.visitBool=function(t){var e;return t.nullCount>=t.length?Vr.call(this,new Uint8Array(0)):(e=t.values)instanceof Uint8Array?Vr.call(this,Object(it.g)(t.offset,t.length,e)):Vr.call(this,Object(it.d)(t))},Yr.prototype.visitInt=zr,Yr.prototype.visitFloat=zr,Yr.prototype.visitUtf8=Wr,Yr.prototype.visitBinary=Wr,Yr.prototype.visitFixedSizeBinary=zr,Yr.prototype.visitDate=zr,Yr.prototype.visitTimestamp=zr,Yr.prototype.visitTime=zr,Yr.prototype.visitDecimal=zr,Yr.prototype.visitList=Hr,Yr.prototype.visitStruct=Gr,Yr.prototype.visitUnion=function(t){var e=t.type,n=t.length,r=t.typeIds,i=t.valueOffsets;if(Vr.call(this,r),e.mode===L.i.Sparse)return Gr.call(this,t);if(e.mode===L.i.Dense){if(t.offset<=0)return Vr.call(this,i),Gr.call(this,t);for(var o=r.reduce(function(t,e){return Math.max(t,e)},r[0]),a=new Int32Array(o+1),s=new Int32Array(o+1).fill(-1),u=new Int32Array(n),c=Object(v.c)(-i[0],n,i),f=void 0,l=void 0,h=-1;++h<n;)-1===(l=s[f=r[h]])&&(l=s[f]=c[f]),u[h]=c[h]-l,++a[f];Vr.call(this,u);for(var p=void 0,d=-1,y=e.children.length;++d<y;)if(p=t.getChildAt(d)){f=e.typeIds[d];var b=Math.min(n,a[f]);this.visit(p.slice(s[f],b))}}return this},Yr.prototype.visitInterval=zr,Yr.prototype.visitFixedSizeList=Hr,Yr.prototype.visitMap=Gr;var qr=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.prototype.visit=function(e){return null==e?void 0:t.prototype.visit.call(this,e)},e.prototype.visitNull=function(t){var e=t.typeId;return{name:L.a[e].toLowerCase()}},e.prototype.visitInt=function(t){var e=t.typeId,n=t.bitWidth,r=t.isSigned;return{name:L.a[e].toLowerCase(),bitWidth:n,isSigned:r}},e.prototype.visitFloat=function(t){var e=t.typeId,n=t.precision;return{name:L.a[e].toLowerCase(),precision:L.f[n]}},e.prototype.visitBinary=function(t){var e=t.typeId;return{name:L.a[e].toLowerCase()}},e.prototype.visitBool=function(t){var e=t.typeId;return{name:L.a[e].toLowerCase()}},e.prototype.visitUtf8=function(t){var e=t.typeId;return{name:L.a[e].toLowerCase()}},e.prototype.visitDecimal=function(t){var e=t.typeId,n=t.scale,r=t.precision;return{name:L.a[e].toLowerCase(),scale:n,precision:r}},e.prototype.visitDate=function(t){var e=t.typeId,n=t.unit;return{name:L.a[e].toLowerCase(),unit:L.b[n]}},e.prototype.visitTime=function(t){var e=t.typeId,n=t.unit,r=t.bitWidth;return{name:L.a[e].toLowerCase(),unit:L.g[n],bitWidth:r}},e.prototype.visitTimestamp=function(t){var e=t.typeId,n=t.timezone,r=t.unit;return{name:L.a[e].toLowerCase(),unit:L.g[r],timezone:n}},e.prototype.visitInterval=function(t){var e=t.typeId,n=t.unit;return{name:L.a[e].toLowerCase(),unit:L.c[n]}},e.prototype.visitList=function(t){var e=t.typeId;return{name:L.a[e].toLowerCase()}},e.prototype.visitStruct=function(t){var e=t.typeId;return{name:L.a[e].toLowerCase()}},e.prototype.visitUnion=function(t){var e=t.typeId,n=t.mode,i=t.typeIds;return{name:L.a[e].toLowerCase(),mode:L.i[n],typeIds:r.j(i)}},e.prototype.visitDictionary=function(t){return this.visit(t.dictionary)},e.prototype.visitFixedSizeBinary=function(t){var e=t.typeId,n=t.byteWidth;return{name:L.a[e].toLowerCase(),byteWidth:n}},e.prototype.visitFixedSizeList=function(t){var e=t.typeId,n=t.listSize;return{name:L.a[e].toLowerCase(),listSize:n}},e.prototype.visitMap=function(t){var e=t.typeId,n=t.keysSorted;return{name:L.a[e].toLowerCase(),keysSorted:n}},e}(rt.a),Zr=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.assemble=function(){for(var t=[],n=0;n<arguments.length;n++)t[n]=arguments[n];var i=t.reduce(function t(e,n){return Array.isArray(n)?n.reduce(t,e):n instanceof P?e.concat(n.schema.fields.map(function(t,e){return new k(t,[n.getChildAt(e)])})):r.j(e,[n])},[]).filter(function(t){return t instanceof k});return(new e).visitMany(i)},e.prototype.visit=function(e){var n,a=e.data,s=e.name,u=e.length,c=a.offset,f=a.nullCount,l=a.nullBitmap,h=o.c.isDictionary(e.type)?e.type.indices:e.type,p=Object.assign([],a.buffers,((n={})[L.j.VALIDITY]=void 0,n));return r.a({name:s,count:u,VALIDITY:f<=0?Array.from({length:u},function(){return 1}):r.j(Object(it.c)(l,c,u,null,it.a))},t.prototype.visit.call(this,i.a.new(a.clone(h,c,u,0,p))))},e.prototype.visitNull=function(){return{}},e.prototype.visitBool=function(t){var e=t.values,n=t.offset,i=t.length;return{DATA:r.j(Object(it.c)(e,n,i,null,it.b))}},e.prototype.visitInt=function(t){return{DATA:t.type.bitWidth<64?r.j(t.values):r.j(Kr(t.values,2))}},e.prototype.visitFloat=function(t){return{DATA:r.j(t.values)}},e.prototype.visitUtf8=function(t){return{DATA:r.j(t),OFFSET:r.j(t.valueOffsets)}},e.prototype.visitBinary=function(t){return{DATA:r.j(Jr(t)),OFFSET:r.j(t.valueOffsets)}},e.prototype.visitFixedSizeBinary=function(t){return{DATA:r.j(Jr(t))}},e.prototype.visitDate=function(t){return{DATA:t.type.unit===L.b.DAY?r.j(t.values):r.j(Kr(t.values,2))}},e.prototype.visitTimestamp=function(t){return{DATA:r.j(Kr(t.values,2))}},e.prototype.visitTime=function(t){return{DATA:t.type.unit<L.g.MICROSECOND?r.j(t.values):r.j(Kr(t.values,2))}},e.prototype.visitDecimal=function(t){return{DATA:r.j(Kr(t.values,4))}},e.prototype.visitList=function(t){var e=this;return{OFFSET:r.j(t.valueOffsets),children:t.type.children.map(function(n,r){return e.visit(new k(n,[t.getChildAt(r)]))})}},e.prototype.visitStruct=function(t){var e=this;return{children:t.type.children.map(function(n,r){return e.visit(new k(n,[t.getChildAt(r)]))})}},e.prototype.visitUnion=function(t){var e=this;return{TYPE:r.j(t.typeIds),OFFSET:t.type.mode===L.i.Dense?r.j(t.valueOffsets):void 0,children:t.type.children.map(function(n,r){return e.visit(new k(n,[t.getChildAt(r)]))})}},e.prototype.visitInterval=function(t){return{DATA:r.j(t.values)}},e.prototype.visitFixedSizeList=function(t){var e=this;return{children:t.type.children.map(function(n,r){return e.visit(new k(n,[t.getChildAt(r)]))})}},e.prototype.visitMap=function(t){var e=this;return{children:t.type.children.map(function(n,r){return e.visit(new k(n,[t.getChildAt(r)]))})}},e}(rt.a);function Jr(t){var e,n,i,o,a;return r.h(this,function(s){switch(s.label){case 0:s.trys.push([0,5,6,7]),i=r.k(t),o=i.next(),s.label=1;case 1:return o.done?[3,4]:[4,o.value.reduce(function(t,e){return""+t+("0"+(255&e).toString(16)).slice(-2)},"").toUpperCase()];case 2:s.sent(),s.label=3;case 3:return o=i.next(),[3,1];case 4:return[3,7];case 5:return a=s.sent(),e={error:a},[3,7];case 6:try{o&&!o.done&&(n=i.return)&&n.call(i)}finally{if(e)throw e.error}return[7];case 7:return[2]}})}function Kr(t,e){var n,i;return r.h(this,function(r){switch(r.label){case 0:n=-1,i=t.length/e,r.label=1;case 1:return++n<i?[4,""+xn.new(t.subarray((n+0)*e,(n+1)*e))]:[3,4];case 2:r.sent(),r.label=3;case 3:return[3,1];case 4:return[2]}})}var $r=function(t){function e(e){var n=t.call(this)||this;return n._position=0,n._started=!1,n._sink=new K,n._schema=null,n._dictionaryBlocks=[],n._recordBatchBlocks=[],n._autoDestroy=!e||"boolean"!=typeof e.autoDestroy||e.autoDestroy,n}return r.g(e,t),e.throughNode=function(t){throw new Error(\'"throughNode" not available in this environment\')},e.throughDOM=function(t,e){throw new Error(\'"throughDOM" not available in this environment\')},e.prototype.toString=function(t){return void 0===t&&(t=!1),this._sink.toString(t)},e.prototype.toUint8Array=function(t){return void 0===t&&(t=!1),this._sink.toUint8Array(t)},e.prototype.writeAll=function(t){var e=this;return Object(x.g)(t)?t.then(function(t){return e.writeAll(t)}):Object(x.b)(t)?function(t,e){var n,i;return r.f(this,void 0,void 0,function(){var o,a,s,u;return r.h(this,function(c){switch(c.label){case 0:c.trys.push([0,5,6,11]),n=r.d(e),c.label=1;case 1:return[4,n.next()];case 2:if((i=c.sent()).done)return[3,4];s=i.value,t.write(s),c.label=3;case 3:return[3,1];case 4:return[3,11];case 5:return u=c.sent(),o={error:u},[3,11];case 6:return c.trys.push([6,,9,10]),i&&!i.done&&(a=n.return)?[4,a.call(n)]:[3,8];case 7:c.sent(),c.label=8;case 8:return[3,10];case 9:if(o)throw o.error;return[7];case 10:return[7];case 11:return[2,t.finish()]}})})}(this,t):function(t,e){var n,i,o=e instanceof ei?e.chunks:e;try{for(var a=r.k(o),s=a.next();!s.done;s=a.next()){var u=s.value;t.write(u)}}catch(t){n={error:t}}finally{try{s&&!s.done&&(i=a.return)&&i.call(a)}finally{if(n)throw n.error}}return t.finish()}(this,t)},Object.defineProperty(e.prototype,"closed",{get:function(){return this._sink.closed},enumerable:!0,configurable:!0}),e.prototype[Symbol.asyncIterator]=function(){return this._sink[Symbol.asyncIterator]()},e.prototype.toDOMStream=function(t){return this._sink.toDOMStream(t)},e.prototype.toNodeStream=function(t){return this._sink.toNodeStream(t)},e.prototype.close=function(){return this.reset()._sink.close()},e.prototype.abort=function(t){return this.reset()._sink.abort(t)},e.prototype.finish=function(){return this._autoDestroy?this.close():this.reset(this._sink,this._schema),this},e.prototype.reset=function(t,e){return void 0===t&&(t=this._sink),void 0===e&&(e=null),t===this._sink||t instanceof K?this._sink=t:(this._sink=new K,t&&Object(x.j)(t)?this.toDOMStream({type:"bytes"}).pipeTo(t):t&&Object(x.k)(t)&&this.toNodeStream({objectMode:!1}).pipe(t)),this._started&&this._schema&&this._writeFooter(),this._started=!1,this._dictionaryBlocks=[],this._recordBatchBlocks=[],e&&e===this._schema||(null===e?(this._position=0,this._schema=null):(this._started=!0,this._schema=e,this._writeSchema(e))),this},e.prototype.write=function(t){var e;if(!this._sink)throw new Error("RecordBatchWriter is closed");if(!t||!(e=t.schema))return this.finish()&&void 0;if(e!==this._schema){if(this._started&&this._autoDestroy)return this.close();this.reset(this._sink,e)}t instanceof ei?this.writeAll(t.chunks):this._writeRecordBatch(t)},e.prototype._writeMessage=function(t,e){void 0===e&&(e=8);var n=e-1,r=Qt.encode(t),i=r.byteLength,o=i+4+n&~n,a=o-i-4;return t.headerType===L.d.RecordBatch?this._recordBatchBlocks.push(new G(o,t.bodyLength,this._position)):t.headerType===L.d.DictionaryBatch&&this._dictionaryBlocks.push(new G(o,t.bodyLength,this._position)),this._write(Int32Array.of(o-4)),i>0&&this._write(r),this._writePadding(a)},e.prototype._write=function(t){if(this._started){var e=Object(v.e)(t);e&&e.byteLength>0&&(this._sink.write(e),this._position+=e.byteLength)}return this},e.prototype._writeSchema=function(t){return this._writeMessage(Qt.from(t))._writeDictionaries(t.dictionaryFields)},e.prototype._writeFooter=function(){return this._writePadding(4)},e.prototype._writeMagic=function(){return this._write(me)},e.prototype._writePadding=function(t){return t>0?this._write(new Uint8Array(t)):this},e.prototype._writeRecordBatch=function(t){var e=Yr.assemble(t),n=e.byteLength,r=e.nodes,i=e.bufferRegions,o=e.buffers,a=new Xt(t.length,r,i),s=Qt.from(a,n);return this._writeMessage(s)._writeBodyBuffers(o)},e.prototype._writeDictionaryBatch=function(t,e,n){void 0===n&&(n=!1);var r=Yr.assemble(t),i=r.byteLength,o=r.nodes,a=r.bufferRegions,s=r.buffers,u=new Xt(t.length,o,a),c=new te(u,e,n),f=Qt.from(c,i);return this._writeMessage(f)._writeBodyBuffers(s)},e.prototype._writeBodyBuffers=function(t){for(var e,n,r,i=-1,o=t.length;++i<o;)(e=t[i])&&(n=e.byteLength)>0&&(this._write(e),(r=(n+7&-8)-n)>0&&this._writePadding(r));return this},e.prototype._writeDictionaries=function(t){var e,n;try{for(var i=r.k(t),o=i.next();!o.done;o=i.next()){var a=r.i(o.value,2),s=a[0],u=a[1][0].type.dictionaryVector;if(u instanceof D)for(var c=u.chunks,f=-1,l=c.length;++f<l;)this._writeDictionaryBatch(c[f],s,f>0);else this._writeDictionaryBatch(u,s,!1)}}catch(t){e={error:t}}finally{try{o&&!o.done&&(n=i.return)&&n.call(i)}finally{if(e)throw e.error}}return this},e}(J.d),Qr=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return r.g(e,t),e.writeAll=function(t,n){return new e(n).writeAll(t)},e}($r),Xr=function(t){function e(){var e=t.call(this)||this;return e._autoDestroy=!0,e}return r.g(e,t),e.writeAll=function(t){return(new e).writeAll(t)},e.prototype._writeSchema=function(t){return this._writeMagic()._writePadding(2)._writeDictionaries(t.dictionaryFields)},e.prototype._writeFooter=function(){var t=W.encode(new W(this._schema,L.e.V4,this._recordBatchBlocks,this._dictionaryBlocks));return this._write(t)._write(Int32Array.of(t.byteLength))._writeMagic()},e}($r);function ti(t){var e=t.name,n=t.type,r=t.nullable,i=new qr;return{name:e,nullable:r,type:i.visit(n),children:(n.children||[]).map(ti),dictionary:o.c.isDictionary(n)?{id:n.id,isOrdered:n.isOrdered,indexType:i.visit(n.indices)}:void 0}}!function(t){function e(){var e=t.call(this)||this;return e._autoDestroy=!0,e}r.g(e,t),e.writeAll=function(t){return(new e).writeAll(t)},e.prototype._writeMessage=function(){return this},e.prototype._writeSchema=function(t){return this._write(\'{\\n  "schema": \'+JSON.stringify({fields:t.fields.map(ti)},null,2))._writeDictionaries(t.dictionaryFields)},e.prototype._writeDictionaries=function(e){return this._write(\',\\n  "dictionaries": [\\n\'),t.prototype._writeDictionaries.call(this,e),this._write("\\n  ]")},e.prototype._writeDictionaryBatch=function(t,e,n){return void 0===n&&(n=!1),this._write(0===this._dictionaryBlocks.length?"    ":",\\n    "),this._write(""+function(t,e,n,r){void 0===r&&(r=!1);var i=t.dictionaryFields.get(n)[0],o=new s(i.name,i.type.dictionary,i.nullable,i.metadata),a=Zr.assemble(new k(o,[e]));return JSON.stringify({id:n,isDelta:r,data:{count:e.length,columns:a}},null,2)}(this._schema,t,e,n)),this._dictionaryBlocks.push(new G(0,0,0)),this},e.prototype._writeRecordBatch=function(t){return this._write(0===this._recordBatchBlocks.length?\',\\n  "batches": [\\n    \':",\\n    "),this._write(""+function(t){return JSON.stringify({count:t.length,columns:Zr.assemble(t)},null,2)}(t)),this._recordBatchBlocks.push(new G(0,0,0)),this},e.prototype.close=function(){return this._recordBatchBlocks.length>0&&this._write("\\n  ]"),this._schema&&this._write("\\n}"),t.prototype.close.call(this)}}($r),n.d(e,"a",function(){return ei});var ei=function(t){function e(){for(var e=[],n=0;n<arguments.length;n++)e[n]=arguments[n];var i=this,o=null;e[0]instanceof a&&(o=e.shift());var s=e.reduce(function t(e,n){return Array.isArray(n)?n.reduce(t,e):r.j(e,[n])},[]).filter(function(t){return t instanceof P});if(!o&&!(o=s[0]&&s[0].schema))throw new TypeError("Table must be initialized with a Schema or at least one RecordBatch");return s[0]||(s[0]=new P(o,0,[])),(i=t.call(this,s[0].type,s)||this)._schema=o,i._chunks=s,i}return r.g(e,t),e.empty=function(){return new e(new a([]),[])},e.from=function(t){if(!t)return e.empty();var n,i=Te.from(t);return Object(x.g)(i)?r.f(this,void 0,void 0,function(){var t,n;return r.h(this,function(r){switch(r.label){case 0:return n=(t=e).from,[4,i];case 1:return[4,n.apply(t,[r.sent()])];case 2:return[2,r.sent()]}})}):i.isSync()&&(i=i.open())?i.schema?new e(i.schema,r.j(i)):e.empty():(n=i.open(),r.f(this,void 0,void 0,function(){var t,i,o,a,s,u,c,f,l;return r.h(this,function(h){switch(h.label){case 0:return[4,n];case 1:if(o=h.sent(),a=o.schema,s=[],!a)return[3,14];h.label=2;case 2:h.trys.push([2,7,8,13]),u=r.d(o),h.label=3;case 3:return[4,u.next()];case 4:if((c=h.sent()).done)return[3,6];f=c.value,s.push(f),h.label=5;case 5:return[3,3];case 6:return[3,13];case 7:return l=h.sent(),t={error:l},[3,13];case 8:return h.trys.push([8,,11,12]),c&&!c.done&&(i=u.return)?[4,i.call(u)]:[3,10];case 9:h.sent(),h.label=10;case 10:return[3,12];case 11:if(t)throw t.error;return[7];case 12:return[7];case 13:return[2,new e(a,s)];case 14:return[2,e.empty()]}})}))},e.fromAsync=function(t){return r.f(this,void 0,void 0,function(){return r.h(this,function(n){switch(n.label){case 0:return[4,e.from(t)];case 1:return[2,n.sent()]}})})},e.fromVectors=function(t,n){return new e(P.from(t,n))},e.fromStruct=function(t){var n=new a(t.type.children),r=t instanceof D?t.chunks:[t];return new e(n,r.map(function(t){return new P(n,t.data)}))},Object.defineProperty(e.prototype,"schema",{get:function(){return this._schema},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"length",{get:function(){return this._length},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"chunks",{get:function(){return this._chunks},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"numCols",{get:function(){return this._numChildren},enumerable:!0,configurable:!0}),e.prototype.clone=function(t){return void 0===t&&(t=this._chunks),new e(this._schema,t)},e.prototype.getColumnAt=function(t){return this.getChildAt(t)},e.prototype.getColumn=function(t){return this.getColumnAt(this.getColumnIndex(t))},e.prototype.getColumnIndex=function(t){return this._schema.fields.findIndex(function(e){return e.name===t})},e.prototype.getChildAt=function(t){if(t<0||t>=this.numChildren)return null;var e,n,r,i=this._schema,o=this._children||(this._children=[]);return(e=o[t])?e:(n=(i.fields||[])[t])&&(r=this._chunks.map(function(e){return e.getChildAt(t)}).filter(function(t){return null!=t})).length>0?o[t]=new k(n,r):null},e.prototype.serialize=function(t,e){return void 0===t&&(t="binary"),void 0===e&&(e=!0),(e?Qr:Xr).writeAll(this._chunks).toUint8Array(!0)},e.prototype.count=function(){return this._length},e.prototype.select=function(){for(var t=[],n=0;n<arguments.length;n++)t[n]=arguments[n];return new e(this._chunks.map(function(e){return e.select.apply(e,r.j(t))}))},e}(D)},function(t,e,n){"use strict";(function(t){\n/*!\n * The buffer module from node.js, for the browser.\n *\n * @author   Feross Aboukhadijeh <feross@feross.org> <http://feross.org>\n * @license  MIT\n */\nvar r=n(71),i=n(72),o=n(73);function a(){return u.TYPED_ARRAY_SUPPORT?2147483647:1073741823}function s(t,e){if(a()<e)throw new RangeError("Invalid typed array length");return u.TYPED_ARRAY_SUPPORT?(t=new Uint8Array(e)).__proto__=u.prototype:(null===t&&(t=new u(e)),t.length=e),t}function u(t,e,n){if(!(u.TYPED_ARRAY_SUPPORT||this instanceof u))return new u(t,e,n);if("number"==typeof t){if("string"==typeof e)throw new Error("If encoding is specified then the first argument must be a string");return l(this,t)}return c(this,t,e,n)}function c(t,e,n,r){if("number"==typeof e)throw new TypeError(\'"value" argument must not be a number\');return"undefined"!=typeof ArrayBuffer&&e instanceof ArrayBuffer?function(t,e,n,r){if(e.byteLength,n<0||e.byteLength<n)throw new RangeError("\'offset\' is out of bounds");if(e.byteLength<n+(r||0))throw new RangeError("\'length\' is out of bounds");return e=void 0===n&&void 0===r?new Uint8Array(e):void 0===r?new Uint8Array(e,n):new Uint8Array(e,n,r),u.TYPED_ARRAY_SUPPORT?(t=e).__proto__=u.prototype:t=h(t,e),t}(t,e,n,r):"string"==typeof e?function(t,e,n){if("string"==typeof n&&""!==n||(n="utf8"),!u.isEncoding(n))throw new TypeError(\'"encoding" must be a valid string encoding\');var r=0|d(e,n),i=(t=s(t,r)).write(e,n);return i!==r&&(t=t.slice(0,i)),t}(t,e,n):function(t,e){if(u.isBuffer(e)){var n=0|p(e.length);return 0===(t=s(t,n)).length?t:(e.copy(t,0,0,n),t)}if(e){if("undefined"!=typeof ArrayBuffer&&e.buffer instanceof ArrayBuffer||"length"in e)return"number"!=typeof e.length||(r=e.length)!=r?s(t,0):h(t,e);if("Buffer"===e.type&&o(e.data))return h(t,e.data)}var r;throw new TypeError("First argument must be a string, Buffer, ArrayBuffer, Array, or array-like object.")}(t,e)}function f(t){if("number"!=typeof t)throw new TypeError(\'"size" argument must be a number\');if(t<0)throw new RangeError(\'"size" argument must not be negative\')}function l(t,e){if(f(e),t=s(t,e<0?0:0|p(e)),!u.TYPED_ARRAY_SUPPORT)for(var n=0;n<e;++n)t[n]=0;return t}function h(t,e){var n=e.length<0?0:0|p(e.length);t=s(t,n);for(var r=0;r<n;r+=1)t[r]=255&e[r];return t}function p(t){if(t>=a())throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x"+a().toString(16)+" bytes");return 0|t}function d(t,e){if(u.isBuffer(t))return t.length;if("undefined"!=typeof ArrayBuffer&&"function"==typeof ArrayBuffer.isView&&(ArrayBuffer.isView(t)||t instanceof ArrayBuffer))return t.byteLength;"string"!=typeof t&&(t=""+t);var n=t.length;if(0===n)return 0;for(var r=!1;;)switch(e){case"ascii":case"latin1":case"binary":return n;case"utf8":case"utf-8":case void 0:return N(t).length;case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return 2*n;case"hex":return n>>>1;case"base64":return Y(t).length;default:if(r)return N(t).length;e=(""+e).toLowerCase(),r=!0}}function y(t,e,n){var r=!1;if((void 0===e||e<0)&&(e=0),e>this.length)return"";if((void 0===n||n>this.length)&&(n=this.length),n<=0)return"";if((n>>>=0)<=(e>>>=0))return"";for(t||(t="utf8");;)switch(t){case"hex":return j(this,e,n);case"utf8":case"utf-8":return D(this,e,n);case"ascii":return M(this,e,n);case"latin1":case"binary":return k(this,e,n);case"base64":return I(this,e,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return x(this,e,n);default:if(r)throw new TypeError("Unknown encoding: "+t);t=(t+"").toLowerCase(),r=!0}}function b(t,e,n){var r=t[e];t[e]=t[n],t[n]=r}function v(t,e,n,r,i){if(0===t.length)return-1;if("string"==typeof n?(r=n,n=0):n>2147483647?n=2147483647:n<-2147483648&&(n=-2147483648),n=+n,isNaN(n)&&(n=i?0:t.length-1),n<0&&(n=t.length+n),n>=t.length){if(i)return-1;n=t.length-1}else if(n<0){if(!i)return-1;n=0}if("string"==typeof e&&(e=u.from(e,r)),u.isBuffer(e))return 0===e.length?-1:m(t,e,n,r,i);if("number"==typeof e)return e&=255,u.TYPED_ARRAY_SUPPORT&&"function"==typeof Uint8Array.prototype.indexOf?i?Uint8Array.prototype.indexOf.call(t,e,n):Uint8Array.prototype.lastIndexOf.call(t,e,n):m(t,[e],n,r,i);throw new TypeError("val must be string, number or Buffer")}function m(t,e,n,r,i){var o,a=1,s=t.length,u=e.length;if(void 0!==r&&("ucs2"===(r=String(r).toLowerCase())||"ucs-2"===r||"utf16le"===r||"utf-16le"===r)){if(t.length<2||e.length<2)return-1;a=2,s/=2,u/=2,n/=2}function c(t,e){return 1===a?t[e]:t.readUInt16BE(e*a)}if(i){var f=-1;for(o=n;o<s;o++)if(c(t,o)===c(e,-1===f?0:o-f)){if(-1===f&&(f=o),o-f+1===u)return f*a}else-1!==f&&(o-=o-f),f=-1}else for(n+u>s&&(n=s-u),o=n;o>=0;o--){for(var l=!0,h=0;h<u;h++)if(c(t,o+h)!==c(e,h)){l=!1;break}if(l)return o}return-1}function _(t,e,n,r){n=Number(n)||0;var i=t.length-n;r?(r=Number(r))>i&&(r=i):r=i;var o=e.length;if(o%2!=0)throw new TypeError("Invalid hex string");r>o/2&&(r=o/2);for(var a=0;a<r;++a){var s=parseInt(e.substr(2*a,2),16);if(isNaN(s))return a;t[n+a]=s}return a}function g(t,e,n,r){return V(N(e,t.length-n),t,n,r)}function w(t,e,n,r){return V(function(t){for(var e=[],n=0;n<t.length;++n)e.push(255&t.charCodeAt(n));return e}(e),t,n,r)}function S(t,e,n,r){return w(t,e,n,r)}function O(t,e,n,r){return V(Y(e),t,n,r)}function T(t,e,n,r){return V(function(t,e){for(var n,r,i,o=[],a=0;a<t.length&&!((e-=2)<0);++a)r=(n=t.charCodeAt(a))>>8,i=n%256,o.push(i),o.push(r);return o}(e,t.length-n),t,n,r)}function I(t,e,n){return 0===e&&n===t.length?r.fromByteArray(t):r.fromByteArray(t.slice(e,n))}function D(t,e,n){n=Math.min(t.length,n);for(var r=[],i=e;i<n;){var o,a,s,u,c=t[i],f=null,l=c>239?4:c>223?3:c>191?2:1;if(i+l<=n)switch(l){case 1:c<128&&(f=c);break;case 2:128==(192&(o=t[i+1]))&&(u=(31&c)<<6|63&o)>127&&(f=u);break;case 3:o=t[i+1],a=t[i+2],128==(192&o)&&128==(192&a)&&(u=(15&c)<<12|(63&o)<<6|63&a)>2047&&(u<55296||u>57343)&&(f=u);break;case 4:o=t[i+1],a=t[i+2],s=t[i+3],128==(192&o)&&128==(192&a)&&128==(192&s)&&(u=(15&c)<<18|(63&o)<<12|(63&a)<<6|63&s)>65535&&u<1114112&&(f=u)}null===f?(f=65533,l=1):f>65535&&(f-=65536,r.push(f>>>10&1023|55296),f=56320|1023&f),r.push(f),i+=l}return function(t){var e=t.length;if(e<=A)return String.fromCharCode.apply(String,t);for(var n="",r=0;r<e;)n+=String.fromCharCode.apply(String,t.slice(r,r+=A));return n}(r)}e.Buffer=u,e.SlowBuffer=function(t){return+t!=t&&(t=0),u.alloc(+t)},e.INSPECT_MAX_BYTES=50,u.TYPED_ARRAY_SUPPORT=void 0!==t.TYPED_ARRAY_SUPPORT?t.TYPED_ARRAY_SUPPORT:function(){try{var t=new Uint8Array(1);return t.__proto__={__proto__:Uint8Array.prototype,foo:function(){return 42}},42===t.foo()&&"function"==typeof t.subarray&&0===t.subarray(1,1).byteLength}catch(t){return!1}}(),e.kMaxLength=a(),u.poolSize=8192,u._augment=function(t){return t.__proto__=u.prototype,t},u.from=function(t,e,n){return c(null,t,e,n)},u.TYPED_ARRAY_SUPPORT&&(u.prototype.__proto__=Uint8Array.prototype,u.__proto__=Uint8Array,"undefined"!=typeof Symbol&&Symbol.species&&u[Symbol.species]===u&&Object.defineProperty(u,Symbol.species,{value:null,configurable:!0})),u.alloc=function(t,e,n){return function(t,e,n,r){return f(e),e<=0?s(null,e):void 0!==n?"string"==typeof r?s(null,e).fill(n,r):s(null,e).fill(n):s(null,e)}(0,t,e,n)},u.allocUnsafe=function(t){return l(null,t)},u.allocUnsafeSlow=function(t){return l(null,t)},u.isBuffer=function(t){return!(null==t||!t._isBuffer)},u.compare=function(t,e){if(!u.isBuffer(t)||!u.isBuffer(e))throw new TypeError("Arguments must be Buffers");if(t===e)return 0;for(var n=t.length,r=e.length,i=0,o=Math.min(n,r);i<o;++i)if(t[i]!==e[i]){n=t[i],r=e[i];break}return n<r?-1:r<n?1:0},u.isEncoding=function(t){switch(String(t).toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"latin1":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return!0;default:return!1}},u.concat=function(t,e){if(!o(t))throw new TypeError(\'"list" argument must be an Array of Buffers\');if(0===t.length)return u.alloc(0);var n;if(void 0===e)for(e=0,n=0;n<t.length;++n)e+=t[n].length;var r=u.allocUnsafe(e),i=0;for(n=0;n<t.length;++n){var a=t[n];if(!u.isBuffer(a))throw new TypeError(\'"list" argument must be an Array of Buffers\');a.copy(r,i),i+=a.length}return r},u.byteLength=d,u.prototype._isBuffer=!0,u.prototype.swap16=function(){var t=this.length;if(t%2!=0)throw new RangeError("Buffer size must be a multiple of 16-bits");for(var e=0;e<t;e+=2)b(this,e,e+1);return this},u.prototype.swap32=function(){var t=this.length;if(t%4!=0)throw new RangeError("Buffer size must be a multiple of 32-bits");for(var e=0;e<t;e+=4)b(this,e,e+3),b(this,e+1,e+2);return this},u.prototype.swap64=function(){var t=this.length;if(t%8!=0)throw new RangeError("Buffer size must be a multiple of 64-bits");for(var e=0;e<t;e+=8)b(this,e,e+7),b(this,e+1,e+6),b(this,e+2,e+5),b(this,e+3,e+4);return this},u.prototype.toString=function(){var t=0|this.length;return 0===t?"":0===arguments.length?D(this,0,t):y.apply(this,arguments)},u.prototype.equals=function(t){if(!u.isBuffer(t))throw new TypeError("Argument must be a Buffer");return this===t||0===u.compare(this,t)},u.prototype.inspect=function(){var t="",n=e.INSPECT_MAX_BYTES;return this.length>0&&(t=this.toString("hex",0,n).match(/.{2}/g).join(" "),this.length>n&&(t+=" ... ")),"<Buffer "+t+">"},u.prototype.compare=function(t,e,n,r,i){if(!u.isBuffer(t))throw new TypeError("Argument must be a Buffer");if(void 0===e&&(e=0),void 0===n&&(n=t?t.length:0),void 0===r&&(r=0),void 0===i&&(i=this.length),e<0||n>t.length||r<0||i>this.length)throw new RangeError("out of range index");if(r>=i&&e>=n)return 0;if(r>=i)return-1;if(e>=n)return 1;if(this===t)return 0;for(var o=(i>>>=0)-(r>>>=0),a=(n>>>=0)-(e>>>=0),s=Math.min(o,a),c=this.slice(r,i),f=t.slice(e,n),l=0;l<s;++l)if(c[l]!==f[l]){o=c[l],a=f[l];break}return o<a?-1:a<o?1:0},u.prototype.includes=function(t,e,n){return-1!==this.indexOf(t,e,n)},u.prototype.indexOf=function(t,e,n){return v(this,t,e,n,!0)},u.prototype.lastIndexOf=function(t,e,n){return v(this,t,e,n,!1)},u.prototype.write=function(t,e,n,r){if(void 0===e)r="utf8",n=this.length,e=0;else if(void 0===n&&"string"==typeof e)r=e,n=this.length,e=0;else{if(!isFinite(e))throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");e|=0,isFinite(n)?(n|=0,void 0===r&&(r="utf8")):(r=n,n=void 0)}var i=this.length-e;if((void 0===n||n>i)&&(n=i),t.length>0&&(n<0||e<0)||e>this.length)throw new RangeError("Attempt to write outside buffer bounds");r||(r="utf8");for(var o=!1;;)switch(r){case"hex":return _(this,t,e,n);case"utf8":case"utf-8":return g(this,t,e,n);case"ascii":return w(this,t,e,n);case"latin1":case"binary":return S(this,t,e,n);case"base64":return O(this,t,e,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return T(this,t,e,n);default:if(o)throw new TypeError("Unknown encoding: "+r);r=(""+r).toLowerCase(),o=!0}},u.prototype.toJSON=function(){return{type:"Buffer",data:Array.prototype.slice.call(this._arr||this,0)}};var A=4096;function M(t,e,n){var r="";n=Math.min(t.length,n);for(var i=e;i<n;++i)r+=String.fromCharCode(127&t[i]);return r}function k(t,e,n){var r="";n=Math.min(t.length,n);for(var i=e;i<n;++i)r+=String.fromCharCode(t[i]);return r}function j(t,e,n){var r,i=t.length;(!e||e<0)&&(e=0),(!n||n<0||n>i)&&(n=i);for(var o="",a=e;a<n;++a)o+=(r=t[a])<16?"0"+r.toString(16):r.toString(16);return o}function x(t,e,n){for(var r=t.slice(e,n),i="",o=0;o<r.length;o+=2)i+=String.fromCharCode(r[o]+256*r[o+1]);return i}function E(t,e,n){if(t%1!=0||t<0)throw new RangeError("offset is not uint");if(t+e>n)throw new RangeError("Trying to access beyond buffer length")}function B(t,e,n,r,i,o){if(!u.isBuffer(t))throw new TypeError(\'"buffer" argument must be a Buffer instance\');if(e>i||e<o)throw new RangeError(\'"value" argument is out of bounds\');if(n+r>t.length)throw new RangeError("Index out of range")}function F(t,e,n,r){e<0&&(e=65535+e+1);for(var i=0,o=Math.min(t.length-n,2);i<o;++i)t[n+i]=(e&255<<8*(r?i:1-i))>>>8*(r?i:1-i)}function P(t,e,n,r){e<0&&(e=4294967295+e+1);for(var i=0,o=Math.min(t.length-n,4);i<o;++i)t[n+i]=e>>>8*(r?i:3-i)&255}function L(t,e,n,r,i,o){if(n+r>t.length)throw new RangeError("Index out of range");if(n<0)throw new RangeError("Index out of range")}function U(t,e,n,r,o){return o||L(t,0,n,4),i.write(t,e,n,r,23,4),n+4}function C(t,e,n,r,o){return o||L(t,0,n,8),i.write(t,e,n,r,52,8),n+8}u.prototype.slice=function(t,e){var n,r=this.length;if((t=~~t)<0?(t+=r)<0&&(t=0):t>r&&(t=r),(e=void 0===e?r:~~e)<0?(e+=r)<0&&(e=0):e>r&&(e=r),e<t&&(e=t),u.TYPED_ARRAY_SUPPORT)(n=this.subarray(t,e)).__proto__=u.prototype;else{var i=e-t;n=new u(i,void 0);for(var o=0;o<i;++o)n[o]=this[o+t]}return n},u.prototype.readUIntLE=function(t,e,n){t|=0,e|=0,n||E(t,e,this.length);for(var r=this[t],i=1,o=0;++o<e&&(i*=256);)r+=this[t+o]*i;return r},u.prototype.readUIntBE=function(t,e,n){t|=0,e|=0,n||E(t,e,this.length);for(var r=this[t+--e],i=1;e>0&&(i*=256);)r+=this[t+--e]*i;return r},u.prototype.readUInt8=function(t,e){return e||E(t,1,this.length),this[t]},u.prototype.readUInt16LE=function(t,e){return e||E(t,2,this.length),this[t]|this[t+1]<<8},u.prototype.readUInt16BE=function(t,e){return e||E(t,2,this.length),this[t]<<8|this[t+1]},u.prototype.readUInt32LE=function(t,e){return e||E(t,4,this.length),(this[t]|this[t+1]<<8|this[t+2]<<16)+16777216*this[t+3]},u.prototype.readUInt32BE=function(t,e){return e||E(t,4,this.length),16777216*this[t]+(this[t+1]<<16|this[t+2]<<8|this[t+3])},u.prototype.readIntLE=function(t,e,n){t|=0,e|=0,n||E(t,e,this.length);for(var r=this[t],i=1,o=0;++o<e&&(i*=256);)r+=this[t+o]*i;return r>=(i*=128)&&(r-=Math.pow(2,8*e)),r},u.prototype.readIntBE=function(t,e,n){t|=0,e|=0,n||E(t,e,this.length);for(var r=e,i=1,o=this[t+--r];r>0&&(i*=256);)o+=this[t+--r]*i;return o>=(i*=128)&&(o-=Math.pow(2,8*e)),o},u.prototype.readInt8=function(t,e){return e||E(t,1,this.length),128&this[t]?-1*(255-this[t]+1):this[t]},u.prototype.readInt16LE=function(t,e){e||E(t,2,this.length);var n=this[t]|this[t+1]<<8;return 32768&n?4294901760|n:n},u.prototype.readInt16BE=function(t,e){e||E(t,2,this.length);var n=this[t+1]|this[t]<<8;return 32768&n?4294901760|n:n},u.prototype.readInt32LE=function(t,e){return e||E(t,4,this.length),this[t]|this[t+1]<<8|this[t+2]<<16|this[t+3]<<24},u.prototype.readInt32BE=function(t,e){return e||E(t,4,this.length),this[t]<<24|this[t+1]<<16|this[t+2]<<8|this[t+3]},u.prototype.readFloatLE=function(t,e){return e||E(t,4,this.length),i.read(this,t,!0,23,4)},u.prototype.readFloatBE=function(t,e){return e||E(t,4,this.length),i.read(this,t,!1,23,4)},u.prototype.readDoubleLE=function(t,e){return e||E(t,8,this.length),i.read(this,t,!0,52,8)},u.prototype.readDoubleBE=function(t,e){return e||E(t,8,this.length),i.read(this,t,!1,52,8)},u.prototype.writeUIntLE=function(t,e,n,r){t=+t,e|=0,n|=0,r||B(this,t,e,n,Math.pow(2,8*n)-1,0);var i=1,o=0;for(this[e]=255&t;++o<n&&(i*=256);)this[e+o]=t/i&255;return e+n},u.prototype.writeUIntBE=function(t,e,n,r){t=+t,e|=0,n|=0,r||B(this,t,e,n,Math.pow(2,8*n)-1,0);var i=n-1,o=1;for(this[e+i]=255&t;--i>=0&&(o*=256);)this[e+i]=t/o&255;return e+n},u.prototype.writeUInt8=function(t,e,n){return t=+t,e|=0,n||B(this,t,e,1,255,0),u.TYPED_ARRAY_SUPPORT||(t=Math.floor(t)),this[e]=255&t,e+1},u.prototype.writeUInt16LE=function(t,e,n){return t=+t,e|=0,n||B(this,t,e,2,65535,0),u.TYPED_ARRAY_SUPPORT?(this[e]=255&t,this[e+1]=t>>>8):F(this,t,e,!0),e+2},u.prototype.writeUInt16BE=function(t,e,n){return t=+t,e|=0,n||B(this,t,e,2,65535,0),u.TYPED_ARRAY_SUPPORT?(this[e]=t>>>8,this[e+1]=255&t):F(this,t,e,!1),e+2},u.prototype.writeUInt32LE=function(t,e,n){return t=+t,e|=0,n||B(this,t,e,4,4294967295,0),u.TYPED_ARRAY_SUPPORT?(this[e+3]=t>>>24,this[e+2]=t>>>16,this[e+1]=t>>>8,this[e]=255&t):P(this,t,e,!0),e+4},u.prototype.writeUInt32BE=function(t,e,n){return t=+t,e|=0,n||B(this,t,e,4,4294967295,0),u.TYPED_ARRAY_SUPPORT?(this[e]=t>>>24,this[e+1]=t>>>16,this[e+2]=t>>>8,this[e+3]=255&t):P(this,t,e,!1),e+4},u.prototype.writeIntLE=function(t,e,n,r){if(t=+t,e|=0,!r){var i=Math.pow(2,8*n-1);B(this,t,e,n,i-1,-i)}var o=0,a=1,s=0;for(this[e]=255&t;++o<n&&(a*=256);)t<0&&0===s&&0!==this[e+o-1]&&(s=1),this[e+o]=(t/a>>0)-s&255;return e+n},u.prototype.writeIntBE=function(t,e,n,r){if(t=+t,e|=0,!r){var i=Math.pow(2,8*n-1);B(this,t,e,n,i-1,-i)}var o=n-1,a=1,s=0;for(this[e+o]=255&t;--o>=0&&(a*=256);)t<0&&0===s&&0!==this[e+o+1]&&(s=1),this[e+o]=(t/a>>0)-s&255;return e+n},u.prototype.writeInt8=function(t,e,n){return t=+t,e|=0,n||B(this,t,e,1,127,-128),u.TYPED_ARRAY_SUPPORT||(t=Math.floor(t)),t<0&&(t=255+t+1),this[e]=255&t,e+1},u.prototype.writeInt16LE=function(t,e,n){return t=+t,e|=0,n||B(this,t,e,2,32767,-32768),u.TYPED_ARRAY_SUPPORT?(this[e]=255&t,this[e+1]=t>>>8):F(this,t,e,!0),e+2},u.prototype.writeInt16BE=function(t,e,n){return t=+t,e|=0,n||B(this,t,e,2,32767,-32768),u.TYPED_ARRAY_SUPPORT?(this[e]=t>>>8,this[e+1]=255&t):F(this,t,e,!1),e+2},u.prototype.writeInt32LE=function(t,e,n){return t=+t,e|=0,n||B(this,t,e,4,2147483647,-2147483648),u.TYPED_ARRAY_SUPPORT?(this[e]=255&t,this[e+1]=t>>>8,this[e+2]=t>>>16,this[e+3]=t>>>24):P(this,t,e,!0),e+4},u.prototype.writeInt32BE=function(t,e,n){return t=+t,e|=0,n||B(this,t,e,4,2147483647,-2147483648),t<0&&(t=4294967295+t+1),u.TYPED_ARRAY_SUPPORT?(this[e]=t>>>24,this[e+1]=t>>>16,this[e+2]=t>>>8,this[e+3]=255&t):P(this,t,e,!1),e+4},u.prototype.writeFloatLE=function(t,e,n){return U(this,t,e,!0,n)},u.prototype.writeFloatBE=function(t,e,n){return U(this,t,e,!1,n)},u.prototype.writeDoubleLE=function(t,e,n){return C(this,t,e,!0,n)},u.prototype.writeDoubleBE=function(t,e,n){return C(this,t,e,!1,n)},u.prototype.copy=function(t,e,n,r){if(n||(n=0),r||0===r||(r=this.length),e>=t.length&&(e=t.length),e||(e=0),r>0&&r<n&&(r=n),r===n)return 0;if(0===t.length||0===this.length)return 0;if(e<0)throw new RangeError("targetStart out of bounds");if(n<0||n>=this.length)throw new RangeError("sourceStart out of bounds");if(r<0)throw new RangeError("sourceEnd out of bounds");r>this.length&&(r=this.length),t.length-e<r-n&&(r=t.length-e+n);var i,o=r-n;if(this===t&&n<e&&e<r)for(i=o-1;i>=0;--i)t[i+e]=this[i+n];else if(o<1e3||!u.TYPED_ARRAY_SUPPORT)for(i=0;i<o;++i)t[i+e]=this[i+n];else Uint8Array.prototype.set.call(t,this.subarray(n,n+o),e);return o},u.prototype.fill=function(t,e,n,r){if("string"==typeof t){if("string"==typeof e?(r=e,e=0,n=this.length):"string"==typeof n&&(r=n,n=this.length),1===t.length){var i=t.charCodeAt(0);i<256&&(t=i)}if(void 0!==r&&"string"!=typeof r)throw new TypeError("encoding must be a string");if("string"==typeof r&&!u.isEncoding(r))throw new TypeError("Unknown encoding: "+r)}else"number"==typeof t&&(t&=255);if(e<0||this.length<e||this.length<n)throw new RangeError("Out of range index");if(n<=e)return this;var o;if(e>>>=0,n=void 0===n?this.length:n>>>0,t||(t=0),"number"==typeof t)for(o=e;o<n;++o)this[o]=t;else{var a=u.isBuffer(t)?t:N(new u(t,r).toString()),s=a.length;for(o=0;o<n-e;++o)this[o+e]=a[o%s]}return this};var R=/[^+\\/0-9A-Za-z-_]/g;function N(t,e){var n;e=e||1/0;for(var r=t.length,i=null,o=[],a=0;a<r;++a){if((n=t.charCodeAt(a))>55295&&n<57344){if(!i){if(n>56319){(e-=3)>-1&&o.push(239,191,189);continue}if(a+1===r){(e-=3)>-1&&o.push(239,191,189);continue}i=n;continue}if(n<56320){(e-=3)>-1&&o.push(239,191,189),i=n;continue}n=65536+(i-55296<<10|n-56320)}else i&&(e-=3)>-1&&o.push(239,191,189);if(i=null,n<128){if((e-=1)<0)break;o.push(n)}else if(n<2048){if((e-=2)<0)break;o.push(n>>6|192,63&n|128)}else if(n<65536){if((e-=3)<0)break;o.push(n>>12|224,n>>6&63|128,63&n|128)}else{if(!(n<1114112))throw new Error("Invalid code point");if((e-=4)<0)break;o.push(n>>18|240,n>>12&63|128,n>>6&63|128,63&n|128)}}return o}function Y(t){return r.toByteArray(function(t){if((t=function(t){return t.trim?t.trim():t.replace(/^\\s+|\\s+$/g,"")}(t).replace(R,"")).length<2)return"";for(;t.length%4!=0;)t+="=";return t}(t))}function V(t,e,n,r){for(var i=0;i<r&&!(i+n>=e.length||i>=t.length);++i)e[i+n]=t[i];return i}}).call(this,n(34))},function(t,e){t.exports=!1},function(t,e,n){var r=n(25),i=n(18).document,o=r(i)&&r(i.createElement);t.exports=function(t){return o?i.createElement(t):{}}},function(t,e){t.exports=function(t,e){return{enumerable:!(1&t),configurable:!(2&t),writable:!(4&t),value:e}}},function(t,e,n){var r=n(91),i=n(62);t.exports=Object.keys||function(t){return r(t,i)}},function(t,e){t.exports="constructor,hasOwnProperty,isPrototypeOf,propertyIsEnumerable,toLocaleString,toString,valueOf".split(",")},function(t,e,n){var r=n(46).f,i=n(31),o=n(15)("toStringTag");t.exports=function(t,e,n){t&&!i(t=n?t:t.prototype,o)&&r(t,o,{configurable:!0,value:e})}},function(t,e){t.exports=function(t){return t.webpackPolyfill||(t.deprecate=function(){},t.paths=[],t.children||(t.children=[]),Object.defineProperty(t,"loaded",{enumerable:!0,get:function(){return t.l}}),Object.defineProperty(t,"id",{enumerable:!0,get:function(){return t.i}}),t.webpackPolyfill=1),t}},function(t,e,n){"use strict";n.d(e,"a",function(){return o}),n(32);var r=n(23),i=n(20);class o{constructor(){this.data_formats={row:0,column:1,schema:2},this.format=void 0,this.data=void 0,this.names=void 0,this.types=void 0,this.row_count=void 0,this.date_parsers={},this.date_validator=t=>Object(r.b)(t)}extract_typevec(t){let e=[];for(let n=0;n<t.size()-1;n++)e.push(t.get(n));return e}is_format(t){if(Array.isArray(t))return this.data_formats.row;if(Array.isArray(t[Object.keys(t)[0]]))return this.data_formats.column;if("string"==typeof t[Object.keys(t)[0]]||"function"==typeof t[Object.keys(t)[0]])return this.data_formats.schema;throw"Unknown data format!"}get_row_count(t){return this.format===this.data_formats.row?t.length:this.format===this.data_formats.column?t[Object.keys(t)[0]].length:0}get(t,e){let n=void 0;if(this.format===this.data_formats.row){let r=this.data[e];r.hasOwnProperty(t)&&(n=r[t])}else if(this.format===this.data_formats.column)this.data.hasOwnProperty(t)&&(n=this.data[t][e]);else{if(this.format!==this.data_formats.schema)throw"Unknown data format!";n=void 0}return n}marshal(t,e,n){const o=this.names[t];let a=null===(s=this.get(o,e))||"null"===s?null:void 0===s||"undefined"===s?void 0:s;var s;let u;if(null===a)return null;if(void 0!==a){switch(void 0===this.date_parsers[o]&&(this.date_parsers[o]=new r.a),u=this.date_parsers[o],Object(i.b)(n.value)){case"float":case"integer":a=Number(a);break;case"boolean":a="string"==typeof a?"true"===a.toLowerCase():!!a;break;case"datetime":case"date":a=u.parse(a);break;default:a+=""}return a}}clean(){this.date_parsers={},this.names=void 0,this.types=void 0}init(t,e){if(this.data=e,this.format=this.is_format(this.data),this.row_count=this.get_row_count(this.data),this.format===this.data_formats.row)e.length>0?this.names=Object.keys(e[0]):this.clean.names=[];else{if(this.format!==this.data_formats.column&&this.format!==this.data_formats.schema)throw"Unknown data format!";this.names=Object.keys(e)}}}},function(t,e,n){"use strict";n.d(e,"a",function(){return r}),n(32);const r=function(t){let e={},n=t.keys();for(let r=0;r<n.size();r++){let i=n.get(r);e[i]=t.get(i)}return t.delete(),n.delete(),e}},function(t,e,n){(function(t,e){n(51);const r=n(105).default,i=["perspective.config.js","perspective.config.json","package.json"];function o(t){return t&&"object"==typeof t&&!Array.isArray(t)}function a(t,...e){if(!e.length)return t;const n=e.shift();if(o(t)&&o(n))for(const e in n)o(n[e])?(t[e]||Object.assign(t,{[e]:{}}),a(t[e],n[e])):Object.assign(t,{[e]:n[e]});return a(t,...e)}let s;t.exports.get_type_config=function(e){return t.exports.get_config().types[e]||{}},t.exports.override_config=function(t){s&&console.warn("Config already initialized!"),s=a(r,t)},t.exports.get_config=function(){return s||(s=a(r,"undefined"==typeof window?function(){void 0!==require||t.require;const r=n(0),o=n(0),[a,...s]=e.cwd().split(r.sep);for(;s.length>0;){for(const t of i){const e="".concat(a).concat(r.sep).concat(r.join(...s,t));if(o.existsSync(e)){if(!t.endsWith("json")){const t=n(2)(e);return t.default||t}{const n=JSON.parse(o.readFileSync(e));if("package.json"!==t)return n;if(n.perspective)return n.perspective}}}s.pop()}}():{types:{float:{filter_operator:"==",aggregate:"sum",format:{style:"decimal",minimumFractionDigits:2,maximumFractionDigits:2}},string:{filter_operator:"==",aggregate:"count"},integer:{filter_operator:"==",aggregate:"sum",format:{}},boolean:{filter_operator:"==",aggregate:"count"},datetime:{filter_operator:"==",aggregate:"count",format:{week:"numeric",year:"numeric",month:"numeric",day:"numeric",hour:"numeric",minute:"numeric",second:"numeric"}},date:{filter_operator:"==",aggregate:"count",format:{week:"numeric",year:"numeric",month:"numeric",day:"numeric"}}}}||{})),s}}).call(this,n(64)(t),n(35))},function(t,e,n){"use strict";n.r(e),function(t){var r=n(52),i=n(53);let o;o=void 0!==t.document&&"undefined"!=typeof WebAssembly?t.perspective=Object(i.a)(Object(r.a)({wasmJSMethod:"native-wasm",printErr:t=>console.error(t),print:t=>console.log(t)})):t.perspective=Object(i.a)(r.a),e.default=o}.call(this,n(34))},function(t,e){},function(t,e,n){(function(t){function n(t,e){for(var n=0,r=t.length-1;r>=0;r--){var i=t[r];"."===i?t.splice(r,1):".."===i?(t.splice(r,1),n++):n&&(t.splice(r,1),n--)}if(e)for(;n--;n)t.unshift("..");return t}var r=/^(\\/?|)([\\s\\S]*?)((?:\\.{1,2}|[^\\/]+?|)(\\.[^.\\/]*|))(?:[\\/]*)$/,i=function(t){return r.exec(t).slice(1)};function o(t,e){if(t.filter)return t.filter(e);for(var n=[],r=0;r<t.length;r++)e(t[r],r,t)&&n.push(t[r]);return n}e.resolve=function(){for(var e="",r=!1,i=arguments.length-1;i>=-1&&!r;i--){var a=i>=0?arguments[i]:t.cwd();if("string"!=typeof a)throw new TypeError("Arguments to path.resolve must be strings");a&&(e=a+"/"+e,r="/"===a.charAt(0))}return(r?"/":"")+(e=n(o(e.split("/"),function(t){return!!t}),!r).join("/"))||"."},e.normalize=function(t){var r=e.isAbsolute(t),i="/"===a(t,-1);return(t=n(o(t.split("/"),function(t){return!!t}),!r).join("/"))||r||(t="."),t&&i&&(t+="/"),(r?"/":"")+t},e.isAbsolute=function(t){return"/"===t.charAt(0)},e.join=function(){var t=Array.prototype.slice.call(arguments,0);return e.normalize(o(t,function(t,e){if("string"!=typeof t)throw new TypeError("Arguments to path.join must be strings");return t}).join("/"))},e.relative=function(t,n){function r(t){for(var e=0;e<t.length&&""===t[e];e++);for(var n=t.length-1;n>=0&&""===t[n];n--);return e>n?[]:t.slice(e,n-e+1)}t=e.resolve(t).substr(1),n=e.resolve(n).substr(1);for(var i=r(t.split("/")),o=r(n.split("/")),a=Math.min(i.length,o.length),s=a,u=0;u<a;u++)if(i[u]!==o[u]){s=u;break}var c=[];for(u=s;u<i.length;u++)c.push("..");return(c=c.concat(o.slice(s))).join("/")},e.sep="/",e.delimiter=":",e.dirname=function(t){var e=i(t),n=e[0],r=e[1];return n||r?(r&&(r=r.substr(0,r.length-1)),n+r):"."},e.basename=function(t,e){var n=i(t)[2];return e&&n.substr(-1*e.length)===e&&(n=n.substr(0,n.length-e.length)),n},e.extname=function(t){return i(t)[3]};var a="b"==="ab".substr(-1)?function(t,e,n){return t.substr(e,n)}:function(t,e,n){return e<0&&(e=t.length+e),t.substr(e,n)}}).call(this,n(35))},function(t,e,n){"use strict";e.byteLength=function(t){var e=c(t),n=e[0],r=e[1];return 3*(n+r)/4-r},e.toByteArray=function(t){for(var e,n=c(t),r=n[0],a=n[1],s=new o(function(t,e,n){return 3*(e+n)/4-n}(0,r,a)),u=0,f=a>0?r-4:r,l=0;l<f;l+=4)e=i[t.charCodeAt(l)]<<18|i[t.charCodeAt(l+1)]<<12|i[t.charCodeAt(l+2)]<<6|i[t.charCodeAt(l+3)],s[u++]=e>>16&255,s[u++]=e>>8&255,s[u++]=255&e;return 2===a&&(e=i[t.charCodeAt(l)]<<2|i[t.charCodeAt(l+1)]>>4,s[u++]=255&e),1===a&&(e=i[t.charCodeAt(l)]<<10|i[t.charCodeAt(l+1)]<<4|i[t.charCodeAt(l+2)]>>2,s[u++]=e>>8&255,s[u++]=255&e),s},e.fromByteArray=function(t){for(var e,n=t.length,i=n%3,o=[],a=0,s=n-i;a<s;a+=16383)o.push(f(t,a,a+16383>s?s:a+16383));return 1===i?(e=t[n-1],o.push(r[e>>2]+r[e<<4&63]+"==")):2===i&&(e=(t[n-2]<<8)+t[n-1],o.push(r[e>>10]+r[e>>4&63]+r[e<<2&63]+"=")),o.join("")};for(var r=[],i=[],o="undefined"!=typeof Uint8Array?Uint8Array:Array,a="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",s=0,u=a.length;s<u;++s)r[s]=a[s],i[a.charCodeAt(s)]=s;function c(t){var e=t.length;if(e%4>0)throw new Error("Invalid string. Length must be a multiple of 4");var n=t.indexOf("=");return-1===n&&(n=e),[n,n===e?0:4-n%4]}function f(t,e,n){for(var i,o,a=[],s=e;s<n;s+=3)i=(t[s]<<16&16711680)+(t[s+1]<<8&65280)+(255&t[s+2]),a.push(r[(o=i)>>18&63]+r[o>>12&63]+r[o>>6&63]+r[63&o]);return a.join("")}i["-".charCodeAt(0)]=62,i["_".charCodeAt(0)]=63},function(t,e){e.read=function(t,e,n,r,i){var o,a,s=8*i-r-1,u=(1<<s)-1,c=u>>1,f=-7,l=n?i-1:0,h=n?-1:1,p=t[e+l];for(l+=h,o=p&(1<<-f)-1,p>>=-f,f+=s;f>0;o=256*o+t[e+l],l+=h,f-=8);for(a=o&(1<<-f)-1,o>>=-f,f+=r;f>0;a=256*a+t[e+l],l+=h,f-=8);if(0===o)o=1-c;else{if(o===u)return a?NaN:1/0*(p?-1:1);a+=Math.pow(2,r),o-=c}return(p?-1:1)*a*Math.pow(2,o-r)},e.write=function(t,e,n,r,i,o){var a,s,u,c=8*o-i-1,f=(1<<c)-1,l=f>>1,h=23===i?Math.pow(2,-24)-Math.pow(2,-77):0,p=r?0:o-1,d=r?1:-1,y=e<0||0===e&&1/e<0?1:0;for(e=Math.abs(e),isNaN(e)||e===1/0?(s=isNaN(e)?1:0,a=f):(a=Math.floor(Math.log(e)/Math.LN2),e*(u=Math.pow(2,-a))<1&&(a--,u*=2),(e+=a+l>=1?h/u:h*Math.pow(2,1-l))*u>=2&&(a++,u/=2),a+l>=f?(s=0,a=f):a+l>=1?(s=(e*u-1)*Math.pow(2,i),a+=l):(s=e*Math.pow(2,l-1)*Math.pow(2,i),a=0));i>=8;t[n+p]=255&s,p+=d,s/=256,i-=8);for(a=a<<i|s,c+=i;c>0;t[n+p]=255&a,p+=d,a/=256,c-=8);t[n+p-d]|=128*y}},function(t,e){var n={}.toString;t.exports=Array.isArray||function(t){return"[object Array]"==n.call(t)}},function(t,e,n){"use strict";var r=n(16),i=n(36),o=n(27),a=n(28),s=n(37),u=n(38),c=Math.max,f=Math.min,l=Math.floor,h=/\\$([$&`\']|\\d\\d?|<[^>]*>)/g,p=/\\$([$&`\']|\\d\\d?)/g;n(43)("replace",2,function(t,e,n,d){return[function(r,i){var o=t(this),a=null==r?void 0:r[e];return void 0!==a?a.call(r,o,i):n.call(String(o),r,i)},function(t,e){var i=d(n,t,this,e);if(i.done)return i.value;var l=r(t),h=String(this),p="function"==typeof e;p||(e=String(e));var b=l.global;if(b){var v=l.unicode;l.lastIndex=0}for(var m=[];;){var _=u(l,h);if(null===_)break;if(m.push(_),!b)break;""===String(_[0])&&(l.lastIndex=s(h,o(l.lastIndex),v))}for(var g,w="",S=0,O=0;O<m.length;O++){_=m[O];for(var T=String(_[0]),I=c(f(a(_.index),h.length),0),D=[],A=1;A<_.length;A++)D.push(void 0===(g=_[A])?g:String(g));var M=_.groups;if(p){var k=[T].concat(D,I,h);void 0!==M&&k.push(M);var j=String(e.apply(void 0,k))}else j=y(T,h,I,D,M,e);I>=S&&(w+=h.slice(S,I)+j,S=I+T.length)}return w+h.slice(S)}];function y(t,e,r,o,a,s){var u=r+t.length,c=o.length,f=p;return void 0!==a&&(a=i(a),f=h),n.call(s,f,function(n,i){var s;switch(i.charAt(0)){case"$":return"$";case"&":return t;case"`":return e.slice(0,r);case"\'":return e.slice(u);case"<":s=a[i.slice(1,-1)];break;default:var f=+i;if(0===f)return n;if(f>c){var h=l(f/10);return 0===h?n:h<=c?void 0===o[h-1]?i.charAt(1):o[h-1]+i.charAt(1):n}s=o[f-1]}return void 0===s?"":s})}})},function(t,e,n){var r=n(28),i=n(26);t.exports=function(t){return function(e,n){var o,a,s=String(i(e)),u=r(n),c=s.length;return u<0||u>=c?t?"":void 0:(o=s.charCodeAt(u))<55296||o>56319||u+1===c||(a=s.charCodeAt(u+1))<56320||a>57343?t?s.charAt(u):o:t?s.slice(u,u+2):a-56320+(o-55296<<10)+65536}}},function(t,e,n){var r=n(39),i=n(15)("toStringTag"),o="Arguments"==r(function(){return arguments}());t.exports=function(t){var e,n,a;return void 0===t?"Undefined":null===t?"Null":"string"==typeof(n=function(t,e){try{return t[e]}catch(t){}}(e=Object(t),i))?n:o?r(e):"Object"==(a=r(e))&&"function"==typeof e.callee?"Arguments":a}},function(t,e,n){"use strict";var r=n(44);n(45)({target:"RegExp",proto:!0,forced:r!==/./.exec},{exec:r})},function(t,e,n){"use strict";var r=n(16);t.exports=function(){var t=r(this),e="";return t.global&&(e+="g"),t.ignoreCase&&(e+="i"),t.multiline&&(e+="m"),t.unicode&&(e+="u"),t.sticky&&(e+="y"),e}},function(t,e,n){t.exports=!n(29)&&!n(22)(function(){return 7!=Object.defineProperty(n(59)("div"),"a",{get:function(){return 7}}).a})},function(t,e,n){var r=n(25);t.exports=function(t,e){if(!r(t))return t;var n,i;if(e&&"function"==typeof(n=t.toString)&&!r(i=n.call(t)))return i;if("function"==typeof(n=t.valueOf)&&!r(i=n.call(t)))return i;if(!e&&"function"==typeof(n=t.toString)&&!r(i=n.call(t)))return i;throw TypeError("Can\'t convert object to primitive value")}},function(t,e,n){t.exports=n(40)("native-function-to-string",Function.toString)},function(t,e,n){var r=n(47);t.exports=function(t,e,n){if(r(t),void 0===e)return t;switch(n){case 1:return function(n){return t.call(e,n)};case 2:return function(n,r){return t.call(e,n,r)};case 3:return function(n,r,i){return t.call(e,n,r,i)}}return function(){return t.apply(e,arguments)}}},function(t,e,n){"use strict";var r=n(84),i=n(85),o=n(48),a=n(49);t.exports=n(87)(Array,"Array",function(t,e){this._t=a(t),this._i=0,this._k=e},function(){var t=this._t,e=this._k,n=this._i++;return!t||n>=t.length?(this._t=void 0,i(1)):i(0,"keys"==e?n:"values"==e?t[n]:[n,t[n]])},"values"),o.Arguments=o.Array,r("keys"),r("values"),r("entries")},function(t,e,n){var r=n(15)("unscopables"),i=Array.prototype;null==i[r]&&n(19)(i,r,{}),t.exports=function(t){i[r][t]=!0}},function(t,e){t.exports=function(t,e){return{value:e,done:!!t}}},function(t,e,n){var r=n(39);t.exports=Object("z").propertyIsEnumerable(0)?Object:function(t){return"String"==r(t)?t.split(""):Object(t)}},function(t,e,n){"use strict";var r=n(58),i=n(45),o=n(30),a=n(19),s=n(48),u=n(88),c=n(63),f=n(95),l=n(15)("iterator"),h=!([].keys&&"next"in[].keys()),p=function(){return this};t.exports=function(t,e,n,d,y,b,v){u(n,e,d);var m,_,g,w=function(t){if(!h&&t in I)return I[t];switch(t){case"keys":case"values":return function(){return new n(this,t)}}return function(){return new n(this,t)}},S=e+" Iterator",O="values"==y,T=!1,I=t.prototype,D=I[l]||I["@@iterator"]||y&&I[y],A=D||w(y),M=y?O?w("entries"):A:void 0,k="Array"==e&&I.entries||D;if(k&&(g=f(k.call(new t)))!==Object.prototype&&g.next&&(c(g,S,!0),r||"function"==typeof g[l]||a(g,l,p)),O&&D&&"values"!==D.name&&(T=!0,A=function(){return D.call(this)}),r&&!v||!h&&!T&&I[l]||a(I,l,A),s[e]=A,s[S]=p,y)if(m={values:O?A:w("values"),keys:b?A:w("keys"),entries:M},v)for(_ in m)_ in I||o(I,_,m[_]);else i(i.P+i.F*(h||T),e,m);return m}},function(t,e,n){"use strict";var r=n(89),i=n(60),o=n(63),a={};n(19)(a,n(15)("iterator"),function(){return this}),t.exports=function(t,e,n){t.prototype=r(a,{next:i(1,n)}),o(t,e+" Iterator")}},function(t,e,n){var r=n(16),i=n(90),o=n(62),a=n(50)("IE_PROTO"),s=function(){},u=function(){var t,e=n(59)("iframe"),r=o.length;for(e.style.display="none",n(94).appendChild(e),e.src="javascript:",(t=e.contentWindow.document).open(),t.write("<script>document.F=Object<\\/script>"),t.close(),u=t.F;r--;)delete u.prototype[o[r]];return u()};t.exports=Object.create||function(t,e){var n;return null!==t?(s.prototype=r(t),n=new s,s.prototype=null,n[a]=t):n=u(),void 0===e?n:i(n,e)}},function(t,e,n){var r=n(46),i=n(16),o=n(61);t.exports=n(29)?Object.defineProperties:function(t,e){i(t);for(var n,a=o(e),s=a.length,u=0;s>u;)r.f(t,n=a[u++],e[n]);return t}},function(t,e,n){var r=n(31),i=n(49),o=n(92)(!1),a=n(50)("IE_PROTO");t.exports=function(t,e){var n,s=i(t),u=0,c=[];for(n in s)n!=a&&r(s,n)&&c.push(n);for(;e.length>u;)r(s,n=e[u++])&&(~o(c,n)||c.push(n));return c}},function(t,e,n){var r=n(49),i=n(27),o=n(93);t.exports=function(t){return function(e,n,a){var s,u=r(e),c=i(u.length),f=o(a,c);if(t&&n!=n){for(;c>f;)if((s=u[f++])!=s)return!0}else for(;c>f;f++)if((t||f in u)&&u[f]===n)return t||f||0;return!t&&-1}}},function(t,e,n){var r=n(28),i=Math.max,o=Math.min;t.exports=function(t,e){return(t=r(t))<0?i(t+e,0):o(t,e)}},function(t,e,n){var r=n(18).document;t.exports=r&&r.documentElement},function(t,e,n){var r=n(31),i=n(36),o=n(50)("IE_PROTO"),a=Object.prototype;t.exports=Object.getPrototypeOf||function(t){return t=i(t),r(t,o)?t[o]:"function"==typeof t.constructor&&t instanceof t.constructor?t.constructor.prototype:t instanceof Object?a:null}},function(t,e,n){var r=n(25),i=n(39),o=n(15)("match");t.exports=function(t){var e;return r(t)&&(void 0!==(e=t[o])?!!e:"RegExp"==i(t))}},function(t,e,n){var r=n(16),i=n(47),o=n(15)("species");t.exports=function(t,e){var n,a=r(t).constructor;return void 0===a||null==(n=r(a)[o])?e:i(n)}},function(t,e,n){"use strict";var r=n(45),i=n(47),o=n(36),a=n(22),s=[].sort,u=[1,2,3];r(r.P+r.F*(a(function(){u.sort(void 0)})||!a(function(){u.sort(null)})||!n(99)(s)),"Array",{sort:function(t){return void 0===t?s.call(o(this)):s.call(o(this),i(t))}})},function(t,e,n){"use strict";var r=n(22);t.exports=function(t,e){return!!t&&r(function(){e?t.call(null,function(){},1):t.call(null)})}},function(t,e,n){var r={"./es":101,"./fr":102};function i(t){var e=o(t);return n(e)}function o(t){if(!n.o(r,t)){var e=new Error("Cannot find module \'"+t+"\'");throw e.code="MODULE_NOT_FOUND",e}return r[t]}i.keys=function(){return Object.keys(r)},i.resolve=o,t.exports=i,i.id=100},function(t,e,n){!function(t){"use strict";var e="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),n="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),r=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],i=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\\.?|feb\\.?|mar\\.?|abr\\.?|may\\.?|jun\\.?|jul\\.?|ago\\.?|sep\\.?|oct\\.?|nov\\.?|dic\\.?)/i;t.defineLocale("es",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(t,r){return t?/-MMM-/.test(r)?n[t.month()]:e[t.month()]:e},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\\.?|feb\\.?|mar\\.?|abr\\.?|may\\.?|jun\\.?|jul\\.?|ago\\.?|sep\\.?|oct\\.?|nov\\.?|dic\\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(17))},function(t,e,n){!function(t){"use strict";n(17).defineLocale("fr",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\\d{1,2}(er|)/,ordinal:function(t,e){switch(e){case"D":return t+(1===t?"er":"");default:case"M":case"Q":case"DDD":case"d":return t+(1===t?"er":"e");case"w":case"W":return t+(1===t?"re":"e")}},week:{dow:1,doy:4}})}()},function(t,e,n){"use strict";var r=n(16),i=n(27),o=n(37),a=n(38);n(43)("match",1,function(t,e,n,s){return[function(n){var r=t(this),i=null==n?void 0:n[e];return void 0!==i?i.call(n,r):new RegExp(n)[e](String(r))},function(t){var e=s(n,t,this);if(e.done)return e.value;var u=r(t),c=String(this);if(!u.global)return a(u,c);var f=u.unicode;u.lastIndex=0;for(var l,h=[],p=0;null!==(l=a(u,c));){var d=String(l[0]);h[p]=d,""===d&&(u.lastIndex=o(c,i(u.lastIndex),f)),p++}return 0===p?null:h}]})},function(t,e){t.exports=function(){var t=window.navigator.userAgent,e=t.indexOf("MSIE ");if(e>0)return parseInt(t.substring(e+5,t.indexOf(".",e)),10);if(t.indexOf("Trident/")>0){var n=t.indexOf("rv:");return parseInt(t.substring(n+3,t.indexOf(".",n)),10)}var r=t.indexOf("Edge/");return r>0&&parseInt(t.substring(r+5,t.indexOf(".",r)),10)}},function(t,e){t.exports.default={types:{float:{filter_operator:"==",aggregate:"sum",format:{style:"decimal",minimumFractionDigits:2,maximumFractionDigits:2}},string:{filter_operator:"==",aggregate:"count"},integer:{filter_operator:"==",aggregate:"sum",format:{}},boolean:{filter_operator:"==",aggregate:"count"},datetime:{filter_operator:"==",aggregate:"count",format:{week:"numeric",year:"numeric",month:"numeric",day:"numeric",hour:"numeric",minute:"numeric",second:"numeric"}},date:{filter_operator:"==",aggregate:"count",format:{week:"numeric",year:"numeric",month:"numeric",day:"numeric"}}}}}])},function(t,e){function n(t){var e=new Error("Cannot find module \'"+t+"\'");throw e.code="MODULE_NOT_FOUND",e}n.keys=function(){return[]},n.resolve=n,t.exports=n,n.id=2}]);',n.p+"9d49ef44aa557c63fe69.worker.js")}},function(t,e,n){"use strict";var r=window.URL||window.webkitURL;t.exports=function(t,e){try{try{var n;try{(n=new(window.BlobBuilder||window.WebKitBlobBuilder||window.MozBlobBuilder||window.MSBlobBuilder)).append(t),n=n.getBlob()}catch(e){n=new Blob([t])}return new Worker(r.createObjectURL(n))}catch(e){return new Worker("data:application/javascript,"+encodeURIComponent(t))}}catch(t){if(!e)throw Error("Inline worker is not supported");return new Worker(e)}}},function(t,e,n){var r,i,o,a,s,A,l,c,u,g,h,d,f,p,I,C,m,y;window,t.exports=(r=n(126),i=n(60),o=n(29),a=n(77),s=n(78),A=n(188),l=n(44),c=n(109),u=n(76),g=n(194),h=n(195),d=n(196),f=n(213),p=n(264),I=n(75),C=n(101),m=n(119),y=n(120),function(t){var e={};function n(r){if(e[r])return e[r].exports;var i=e[r]={i:r,l:!1,exports:{}};return t[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=t,n.c=e,n.d=function(t,e,r){n.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:r})},n.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},n.t=function(t,e){if(1&e&&(t=n(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var i in t)n.d(r,i,function(e){return t[e]}.bind(null,i));return r},n.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(e,"a",e),e},n.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},n.p="",n(n.s=29)}([function(t,e){t.exports=r},function(t,e){t.exports=i},function(t,e,n){"use strict";(function(t){n.d(e,"b",function(){return i}),n.d(e,"a",function(){return o}),n(9);const r={},i=new class{registerPlugin(t,e){if(r[t])throw new Error('A perspective-viewer plugin "'.concat(t,'" has already been registered'));for(const n in r){const i=r[n];i&&i.name===e.name&&(console.warn('Conflicting plugin name "'.concat(e.name,'", qualifying with id')),i.name="".concat(i.name," [").concat(n,"]"),e.name="".concat(e.name," [").concat(t,"]"))}r[t]=e}getPlugin(t){return r[t]}getInstance(){return r}};function o(){t.registerPlugin("debug",{name:"Debug",create:function(t){var e,n;return regeneratorRuntime.async(function(r){for(;;)switch(r.prev=r.next){case 0:return r.next=2,regeneratorRuntime.awrap(this._view.to_csv({config:{delimiter:"|"}}));case 2:e=r.sent,n=this._render_time(),t.innerHTML='<pre style="margin:0;overflow:scroll;position:absolute;width:100%;height:100%">'.concat(e,"</pre>"),n();case 6:case"end":return r.stop()}},null,this)},selectMode:"toggle",resize:function(){},delete:function(){}})}t.registerPlugin=i.registerPlugin,t.getPlugin=i.getPlugin}).call(this,n(24))},function(t,e){t.exports=o},function(t,e){t.exports=a},function(t,e){t.exports=s},function(t,e){t.exports=A},function(t,e){t.exports=function(t){var e=[];return e.toString=function(){return this.map(function(e){var n=function(t,e){var n,r=t[1]||"",i=t[3];if(!i)return r;if(e&&"function"==typeof btoa){var o=(n=i,"/*# sourceMappingURL=data:application/json;charset=utf-8;base64,"+btoa(unescape(encodeURIComponent(JSON.stringify(n))))+" */"),a=i.sources.map(function(t){return"/*# sourceURL="+i.sourceRoot+t+" */"});return[r].concat(a).concat([o]).join("\n")}return[r].join("\n")}(e,t);return e[2]?"@media "+e[2]+"{"+n+"}":n}).join("")},e.i=function(t,n){"string"==typeof t&&(t=[[null,t,""]]);for(var r={},i=0;i<this.length;i++){var o=this[i][0];"number"==typeof o&&(r[o]=!0)}for(i=0;i<t.length;i++){var a=t[i];"number"==typeof a[0]&&r[a[0]]||(n&&!a[2]?a[2]=n:n&&(a[2]="("+a[2]+") and ("+n+")"),e.push(a))}},e}},function(t,e){t.exports=l},function(t,e){t.exports=c},function(t,e){t.exports=u},function(t,e){t.exports=g},function(t,e){t.exports=h},function(t,e){t.exports=d},function(t,e){t.exports='\x3c!--\n\nCopyright (c) 2017, the Perspective Authors.\n\nThis file is part of the Perspective library, distributed under the terms of\nthe Apache License 2.0.  The full license can be found in the LICENSE file.\n\n--\x3e\n\n<template id="perspective-computed-column">\n    <div id="psp-cc__body">\n        <span id="psp-cc__close" aria-label="Close Computed Column">&times;</span>\n        <div class="psp-cc__container" style="display:none">\n            <span id="psp-cc__mode"></span>\n        </div>\n        <div class="psp-cc__container" style="margin-top:-7px;">\n            <div class="psp-cc__content">\n                <div id="psp-cc-computation__type"></div>\n                <span contentEditable=true type="text" required maxlength="25" size="10" autocomplete="off" id="psp-cc-name"></span>\n            </div>\n            <div class="psp-cc__content psp-cc__content--nomargin">\n                <span class="psp-cc__label psp-cc__error" id="psp-cc__error--name"></span>\n            </div>\n        </div>\n        <div class="psp-cc__container">\n            <div class="psp-cc__content">\n                <select id="psp-cc-computation__select">\n                </select>\n            </div>\n        </div>\n        <div class="psp-cc__container" style="margin-top:-12px;">\n            <div id="psp-cc-computation-inputs">\n                \x3c!--<div class="psp-cc-computation__input-column" drop-target></div>--\x3e\n            </div>\n        </div>\n    </div>\n    <div id="psp-cc__actions">\n        <button id="psp-cc-button-delete" class="psp-cc__button">Delete</button>\n        <button id="psp-cc-button-save" class="psp-cc__button" disabled>Save</button>\n    </div>\n</template>'},function(t,e,n){(t.exports=n(7)(!1)).push([t.i,':host{display:none;font-family:var(--interface--font-family,Arial,sans-serif);flex-direction:column;max-height:35%;margin-top:34px;margin-bottom:24px;flex:0 0 auto!important}:host *{box-sizing:border-box}:host select{-webkit-appearance:none;-moz-appearance:none;appearance:none;background:url(data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPCEtLSBHZW5lcmF0b3I6IEFkb2JlIElsbHVzdHJhdG9yIDIwLjAuMCwgU1ZHIEV4cG9ydCBQbHVnLUluIC4gU1ZHIFZlcnNpb246IDYuMDAgQnVpbGQgMCkgIC0tPgo8c3ZnIHZlcnNpb249IjEuMSIgaWQ9IkxheWVyXzEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgeG1sbnM6eGxpbms9Imh0dHA6Ly93d3cudzMub3JnLzE5OTkveGxpbmsiIHg9IjBweCIgeT0iMHB4IgoJIHZpZXdCb3g9IjAgMCA0LjkgMTAiIHN0eWxlPSJlbmFibGUtYmFja2dyb3VuZDpuZXcgMCAwIDQuOSAxMDsiIHhtbDpzcGFjZT0icHJlc2VydmUiPgo8c3R5bGUgdHlwZT0idGV4dC9jc3MiPgoJLnN0MHtmaWxsOiM0NDQ0NDQ7fQo8L3N0eWxlPgo8dGl0bGU+YXJyb3dzPC90aXRsZT4KPHBvbHlnb24gY2xhc3M9InN0MCIgcG9pbnRzPSIxLjQsNC43IDIuNSwzLjIgMy41LDQuNyAiLz4KPHBvbHlnb24gY2xhc3M9InN0MCIgcG9pbnRzPSIzLjUsNS4zIDIuNSw2LjggMS40LDUuMyAiLz4KPC9zdmc+Cg==) 95% 50% no-repeat #fff;border:none;height:24px;padding:0 0 0 6px;font-size:12px}:host.edit #psp-cc-button-delete{display:block!important}:host .psp-cc__label{color:#666;font-size:10px;margin-left:8px}:host .psp-cc__label.psp-cc__error{color:#cb4549!important;display:none}:host #psp-cc__actions{display:flex;justify-content:flex-start;margin-right:5px;margin-left:20px;margin-top:12px}:host #psp-cc__body,:host .psp-cc__container{display:flex;flex-direction:column}:host #psp-cc__body.row,:host .psp-cc__container.row{align-items:center;flex-direction:row}:host span#psp-cc__close{color:#666;position:absolute}:host span#psp-cc__close:hover{cursor:pointer}:host .psp-cc__container{margin:2px 0 10px 26px}:host .psp-cc__container:first-of-type{margin-bottom:0}:host .psp-cc__container span#psp-cc__mode{color:#000;font-size:12px}:host .psp-cc__container input{-webkit-appearance:none;-moz-appearance:none;appearance:none;background:#fff;border-bottom:1px solid #ccc;font-size:12px;padding:3px 4px;flex-basis:500px}:host #psp-cc-name{background-color:#eee;border-bottom:1px solid #ccc;color:#333;cursor:default!important;font-size:12px;padding-left:2px;width:100%}:host #psp-cc-name:empty:before{content:"New Column";color:#ccc}:host #psp-cc-name *{display:inline}:host #psp-cc-name br{display:none}:host .psp-label__placeholder{color:#ccc;display:inline-block;font-size:12px;margin-left:4px}:host .psp-label__requiredType{margin-left:5px;margin-top:-1px;display:inline-block}:host .psp-label__requiredType.float::before,:host .psp-label__requiredType.integer::before{font-family:monospace;display:flex;content:"123";color:#016bc6}:host .psp-label__requiredType.string::before{font-family:monospace;display:flex;content:"abc";color:#fe9292}:host .psp-label__requiredType.boolean::before{font-family:monospace;display:flex;content:"t/f";color:#999}:host .psp-label__requiredType.datetime::before{font-family:monospace;display:flex;content:"mdy";color:#999}:host .psp-label__requiredType.date::before{font-family:monospace;display:flex;content:"mdy";color:#999}:host .psp-cc-computation__input-column{box-sizing:border-box;display:flex;align-items:center;border-bottom:1px solid #ccc;margin-top:5px;height:23px;width:100%}:host .psp-cc-computation__input-column perspective-row{margin-left:-26px;margin-right:-5px;width:calc(100% + 26px)}:host .psp-cc-computation__input-column.dropping{background-color:#f0f0ff!important;border:1px solid #1078d1!important;cursor:copy!important}:host .psp-cc-computation__input-column.dropping perspective-row{display:none!important;pointer-events:none}:host .psp-cc-computation__input-column.dropped{border:0;background:0 0}:host perspective-row{margin-bottom:2px;--row_draggable-overflow:hidden;--name-before-display:inline-block;--is_visible-display:inline-block;--row_draggable-padding:2px 10px 2px 2px;--row_close-display:none;--psp_row-margin:0 5px 0 0;--column_name-max-width:180px}:host perspective-row[drop-target]{--row_draggable_children-opacity:0}:host .psp-cc__content{align-items:center;display:flex;margin-top:5px}:host .psp-cc__content.psp-cc__content--nomargin{margin-top:0}:host #psp-cc-computation__type{box-sizing:border-box;display:flex;font-family:monospace;flex-direction:column;font-size:12px;padding-right:3px;padding-left:6px;background:eee;height:24px;text-align:center;padding-top:4px}:host #psp-cc-computation__type .boolean,:host #psp-cc-computation__type .date,:host #psp-cc-computation__type .datetime{color:#999}:host #psp-cc-computation__type .float,:host #psp-cc-computation__type .integer{color:#016bc6}:host #psp-cc-computation__type .string{color:#fe9292}:host #psp-cc-computation__select{margin-left:auto;width:100%;background-color:#eee;font-size:10px;border-bottom:none;color:#666}:host #psp-cc-computation__drop-target-hover{display:none}:host .psp-cc__button{border-radius:0;border-bottom:1px solid #ccc;border-left:0;border-right:0;border-top:0;font-size:12px;margin-left:6px;padding:2px 5px;width:45px}:host .psp-cc__button:hover{cursor:pointer}:host .psp-cc__button[disabled]{background:#666!important;color:#a0a0a0!important;cursor:not-allowed;border-bottom-color:#000!important;opacity:.5}:host .psp-cc__button#psp-cc-button-delete{background:#e76056;border-bottom-color:#bf4d45;color:#fff;display:none}:host .psp-cc__button#psp-cc-button-delete:active{background:#bf4d45;border-bottom-color:#9d3f39}:host .psp-cc__button#psp-cc-button-save{background:#0171d2;border-bottom-color:#0160b1;color:#fff;height:22px}:host .psp-cc__button#psp-cc-button-save:active{background:#0160b1;border-bottom-color:#014986}',""])},function(t,e,n){(t.exports=n(7)(!1)).push([t.i,'.awesomplete [hidden] {\n    display: none;\n}\n\n.awesomplete .visually-hidden {\n    position: absolute;\n    clip: rect(0, 0, 0, 0);\n}\n\n.awesomplete {\n    display: inline-block;\n    position: relative;\n}\n\n.awesomplete > input {\n    display: block;\n}\n\n.awesomplete > ul {\n    position: absolute;\n    left: 0;\n    z-index: 1;\n    min-width: 100%;\n    box-sizing: border-box;\n    list-style: none;\n    padding: 0;\n    margin: 0;\n    background: #fff;\n}\n\n.awesomplete > ul:empty {\n    display: none;\n}\n\n.awesomplete > ul {\n\tborder-radius: .3em;\n\tmargin: .2em 0 0;\n\tbackground: hsla(0,0%,100%,.9);\n\tbackground: linear-gradient(to bottom right, white, hsla(0,0%,100%,.8));\n\tborder: 1px solid rgba(0,0,0,.3);\n\tbox-shadow: .05em .2em .6em rgba(0,0,0,.2);\n\ttext-shadow: none;\n}\n\n@supports (transform: scale(0)) {\n\t.awesomplete > ul {\n\t\ttransition: .3s cubic-bezier(.4,.2,.5,1.4);\n\t\ttransform-origin: 1.43em -.43em;\n\t}\n\t\n\t.awesomplete > ul[hidden],\n\t.awesomplete > ul:empty {\n\t\topacity: 0;\n\t\ttransform: scale(0);\n\t\tdisplay: block;\n\t\ttransition-timing-function: ease;\n\t}\n}\n\n\t/* Pointer */\n\t.awesomplete > ul:before {\n\t\tcontent: "";\n\t\tposition: absolute;\n\t\ttop: -.43em;\n\t\tleft: 1em;\n\t\twidth: 0; height: 0;\n\t\tpadding: .4em;\n\t\tbackground: white;\n\t\tborder: inherit;\n\t\tborder-right: 0;\n\t\tborder-bottom: 0;\n\t\t-webkit-transform: rotate(45deg);\n\t\ttransform: rotate(45deg);\n\t}\n\n\t.awesomplete > ul > li {\n\t\tposition: relative;\n\t\tpadding: .2em .5em;\n\t\tcursor: pointer;\n\t}\n\t\n\t.awesomplete > ul > li:hover {\n\t\tbackground: hsl(200, 40%, 80%);\n\t\tcolor: black;\n\t}\n\t\n\t.awesomplete > ul > li[aria-selected="true"] {\n\t\tbackground: hsl(205, 40%, 40%);\n\t\tcolor: white;\n\t}\n\t\n\t\t.awesomplete mark {\n\t\t\tbackground: hsl(65, 100%, 50%);\n\t\t}\n\t\t\n\t\t.awesomplete li:hover mark {\n\t\t\tbackground: hsl(68, 100%, 41%);\n\t\t}\n\t\t\n\t\t.awesomplete li[aria-selected="true"] mark {\n\t\t\tbackground: hsl(86, 100%, 21%);\n\t\t\tcolor: inherit;\n\t\t}\n',""])},function(t,e){t.exports='\x3c!--\r\n   \r\n   Copyright (c) 2017, the Perspective Authors.\r\n   \r\n   This file is part of the Perspective library, distributed under the terms of\r\n   the Apache License 2.0.  The full license can be found in the LICENSE file.\r\n\r\n--\x3e\r\n\r\n<template id="perspective-row">\r\n    <div id="psp_row">\r\n        <span class=\'is_visible\'></span>\r\n        <div draggable=\'true\' class="row_draggable">\r\n            <span id=\'name\' class="column_name"></span>\r\n            <div class="row_computed">\r\n                <span id="row_edit"></span>\r\n            </div>\r\n            <span id="sort_order"></span>\r\n            <select id="column_aggregate" class="string"></select>\r\n            <select id="filter_operator"></select>\r\n            <input id="filter_operand" placeholder="Value" />\r\n            <span id="row_exclamation" hidden>&#x26A0;</span>\r\n            <span id=\'row_close\'>&#x2715;</span>\r\n        </div>\r\n    </div>\r\n</template>'},function(t,e,n){(t.exports=n(7)(!1)).push([t.i,':host{display:block;border:0 solid transparent}:host #row_close{min-width:5px;max-width:5px;margin-right:3px;flex-grow:0;margin-left:3px}:host .row_computed{display:none;flex-direction:row}.dropping *{pointer-events:none}:host #psp_row{display:var(--psp_row-display,flex);margin:var(--psp_row-margin,0);overflow:visible;align-items:var(--column_selector--align-items,center);width:auto!important;height:auto!important;float:none!important}:host .is_visible{width:var(--column_selector--width,20px);min-width:var(--column_selector--width,20px);height:var(--is_visible--height,auto);cursor:pointer;font-size:var(--column_selector--font-size,14px);margin-top:-2px;display:var(--is_visible-display,none);color:var(--column-selector--color,#999);font-family:var(--column-selector--font-family,Arial)!important}:host #sort_order{display:none;font-family:var(--button--font-family,inherit);margin-right:4px;cursor:pointer;width:10px;display:var(--sort_order-display,none);padding:var(--sort_order-padding,0)}:host #sort_order:before{content:var(--row-sort-order--before, "-")}:host #sort_order:hover{color:var(--button-hover--color,inherit)}:host #filter_operand{display:var(--filter_operand-display,none);margin:0;padding:0;height:19px;color:inherit;outline:0;border-color:var(--column--border-color,#ccc)}:host #filter_operator{margin:0 0 0 15px;display:var(--filter_operator-display,none);height:17px}:host #filter_operator:hover{color:var(--button-hover--color,inherit)}@keyframes expand2{from{transform:translateX(10px)}}@keyframes expand3{from{transform:scale(.9)}}:host(:hover) .is_visible{color:var(--button-hover--color,--column-selector--color)}:host #row_close{display:var(--row_close-display,inline);cursor:pointer;color:#999;font-family:Arial;font-size:10px}:host .is_visible:before{content:var(--is_visible--before,none)}:host #row_exclamation{color:red}:host #row_close:hover{color:var(--button-hover--color,inherit)}:host .row_draggable select{border:none}:host .row_draggable select:focus{outline:0}:host select::-ms-expand{display:none}:host select{margin-left:15px;padding-right:15px;cursor:pointer;-webkit-appearance:none;-moz-appearance:none;-ms-appearance:none;appearance:none;background:url(data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPCEtLSBHZW5lcmF0b3I6IEFkb2JlIElsbHVzdHJhdG9yIDIwLjAuMCwgU1ZHIEV4cG9ydCBQbHVnLUluIC4gU1ZHIFZlcnNpb246IDYuMDAgQnVpbGQgMCkgIC0tPgo8c3ZnIHZlcnNpb249IjEuMSIgaWQ9IkxheWVyXzEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgeG1sbnM6eGxpbms9Imh0dHA6Ly93d3cudzMub3JnLzE5OTkveGxpbmsiIHg9IjBweCIgeT0iMHB4IgoJIHZpZXdCb3g9IjAgMCA0LjkgMTAiIHN0eWxlPSJlbmFibGUtYmFja2dyb3VuZDpuZXcgMCAwIDQuOSAxMDsiIHhtbDpzcGFjZT0icHJlc2VydmUiPgo8c3R5bGUgdHlwZT0idGV4dC9jc3MiPgoJLnN0MHtmaWxsOiM0NDQ0NDQ7fQo8L3N0eWxlPgo8dGl0bGU+YXJyb3dzPC90aXRsZT4KPHBvbHlnb24gY2xhc3M9InN0MCIgcG9pbnRzPSIxLjQsNC43IDIuNSwzLjIgMy41LDQuNyAiLz4KPHBvbHlnb24gY2xhc3M9InN0MCIgcG9pbnRzPSIzLjUsNS4zIDIuNSw2LjggMS40LDUuMyAiLz4KPC9zdmc+Cg==) 95% 50% no-repeat;background-color:var(--select--background-color,#fff);color:inherit;border-radius:5px;height:24px;padding:var(--column_type--padding,0 0 0 5px);font-size:12px}:host .awesomplete>ul{border-radius:0;margin:0;border:none;transition:none!important;background:var(--plugin--background,#fff)!important}:host .awesomplete>ul:before{content:none}:host .awesomplete>input,:host input{border-color:#ccc;border-width:0 0 1px;box-sizing:border-box;background-color:transparent}:host .row_draggable{animation:var(--row_draggable--animation,expand2 .5s ease-out);cursor:move;border-style:solid;background-color:var(--row_draggable-background-color,none);overflow:var(--row_draggable-overflow,visible);border-color:var(--row_draggable-border-color,none);border-width:var(--row_draggable-border-width,0);padding:var(--row_draggable-padding,0);font-size:12px;flex-grow:1;color:inherit;transition:height .2s;box-sizing:border-box;height:var(--row_draggable-height,21px);min-height:var(--row_draggable-height,21px);display:var(--row_draggable-display,flex);flex-direction:column;justify-content:var(--row_draggable--justify-content,center);margin:var(--row_draggable--margin,0);transition:var(--row_draggable--transition,none)}:host .row_draggable *{opacity:var(--row_draggable_children-opacity,1)}:host .row_draggable select{margin:0 0 0 27px}:host ul{transition:background-color .2s,border-color .2s}:host([drop-target]) .row_draggable{background-color:var(--column-drop-target--background,#f0f0ff)!important;animation:var(--drop-target--animation,var(--row_draggable--animation,expand3 .5s ease-out));border:1px solid #1078d1!important}:host .float:before,:host .integer:before{content:"123";color:var(--column_type-float--color,#016bc6)}:host .string:before{content:"abc";color:var(--column_type-string--color,#fe9292)}:host .boolean:before{content:"t/f";color:var(--column_type-boolean--color,#999)}:host .date:before{content:"mdy";color:var(--column_type-date--color,#999)}:host .datetime:before{content:"mdy";color:var(--column_type-datetime--color,#999)}:host .column_name{overflow:hidden;max-width:var(--column_name-max-width,none);white-space:nowrap;text-overflow:ellipsis}:host span#name:before{font-family:var(--interface-monospace--font-family,monospace);position:relative;display:inline-block;padding:var(--column_type--padding,0 5px);min-width:18px;display:var(--name-before-display,none);width:var(--column_type--width,auto)}:host #column_aggregate{display:var(--column_aggregate-display,none);font-size:10px;font-family:Arial,sans-serif;width:88%;height:20px}:host span#name{width:100%;white-space:nowrap;margin:var(--column_name-margin,0)}',""])},function(t,e){t.exports='\x3c!--\r\n   \r\n   Copyright (c) 2017, the Perspective Authors.\r\n   \r\n   This file is part of the Perspective library, distributed under the terms of\r\n   the Apache License 2.0.  The full license can be found in the LICENSE file.\r\n\r\n--\x3e\r\n\r\n<template id="perspective-viewer">\r\n\r\n    <div id="app">\r\n        <div id="menubar">\r\n            <div id="copy_button" class="noselect button" title="Copy CSV"></div>\r\n            <div id="download_button" class="noselect button" title="Download CSV"></div>\r\n            <div id="reset_button" class="noselect button" title="Reset Viewer"></div>\r\n        </div>\r\n        <div id=\'side_panel\' class="column fixed noselect">\r\n            <div id="vis_selector_container">\r\n                <select id="vis_selector">\r\n                </select>\r\n            </div>\r\n            <div id="columns_container" style="visibility: hidden">\r\n                <ul id=\'active_columns\'></ul>\r\n                <div id=\'divider\'></div>\r\n                <div id="sub_columns">\r\n                    <ul id=\'inactive_columns\'></ul>\r\n                    <div id="side_panel__actions" style="visibility: hidden">\r\n                        <div id="add-computed-column" class="side_panel-action"><span class="psp-icon psp-icon__add"></span><span class="psp-title__columnName">New Column</span></div>\r\n                    </div>\r\n                </div>\r\n                <perspective-computed-column></perspective-computed-column>\r\n            </div>\r\n            <div id="resize_bar"></div>\r\n        </div>\r\n        <div class="column noselect">\r\n            <div id=\'top_panel\'>\r\n                <div class="rrow">\r\n                    <div id="row_pivots">\r\n                        <div class="psp-text-field">\r\n                            <ul class="psp-text-field__input" for="row-pivots"></ul>\r\n                            <label for="row_pivots"></label>\r\n                        </div>\r\n                    </div>\r\n                </div>\r\n                <span id="transpose_button" class="rrow centered" title="Transpose Pivots">&#x21C4;</span>\r\n                <div class="rrow">\r\n                    <div id="column_pivots">\r\n                        <div class="psp-text-field">\r\n                            <ul class="psp-text-field__input" for="column-pivots"></ul>\r\n                            <label for="column_pivots"></label>\r\n                        </div>\r\n                    </div>\r\n                </div>\r\n                <div class="rrow">\r\n                    <div id="sort">\r\n                        <div class="psp-text-field">\r\n                            <ul class="psp-text-field__input" for="sort"></ul>\r\n                            <label for="sort"></label>\r\n                        </div>\r\n                    </div>\r\n                </div>\r\n                <div class="rrow" style=\'flex-grow:3;height:auto;flex:1 1 400px;max-width:none\'>\r\n                    <div id="filters">\r\n                        <div class="psp-text-field">\r\n                            <ul class="psp-text-field__input" for=\'filters\'></ul>\r\n                            <label for="filters"></label>\r\n                        </div>\r\n                    </div>\r\n                </div>\r\n            </div>\r\n            <div id="pivot_chart_container">\r\n                <div class="plugin_information plugin_information--warning hidden" id="plugin_information--size">\r\n                    <span class="plugin_information__text" id="plugin_information_count">Estimated rendering</span>\r\n                    <span class="plugin_information__action">Continue to Render</span>\r\n                    <span class="plugin_information__action plugin_information__action--dismiss">Don\'t Show Again</span>\r\n                </div>\r\n                <div id="pivot_chart"></div>\r\n            </div>\r\n        </div>\r\n\r\n        <div id="config_button" class="noselect button" title="Right click for additional options"></div>\r\n\r\n        <div id=\'drop_target\'>\r\n            <div id=\'drop_target_inner\'>\r\n\r\n            </div>\r\n        </div>\r\n\r\n        <slot></slot>\r\n    </div>\r\n\r\n</template>'},function(t,e,n){(t.exports=n(7)(!1)).push([t.i,':host #app #top_panel #row_pivots label:before{content:var(--row_pivots--before, "Row Pivots")}:host #app #top_panel #column_pivots label:before{content:var(--column_pivots--before, "Column Pivots")}:host #app #top_panel #sort label:before{content:"Sort"}:host #app #top_panel #filters label:before{content:"Filter"}:host #app #side_panel #active_columns perspective-row:before{font-size:10px;margin-left:var(--column_selector--width,20px);left:1px;padding:var(--column_type--padding,0 0 0 5px);color:#666;position:absolute;margin-top:-15px}:host:hover #config_button{opacity:1}#config_button{background:0 0;opacity:1!important}.button{padding:var(--button--padding,12px 14px 24px 8px)}[settings=true] #config_button{opacity:1!important}@keyframes expand{from{opacity:0}}:host{position:relative;display:inline-block;overflow:hidden;word-wrap:normal;--highcharts-heatmap-gradient-full:linear-gradient(#4d342f 0%, #e4521b 22.5%, #feeb65 42.5%, #f0f0f0 50%, #dcedc8 57.5%, #42b3d5 67.5%, #1a237e 100%);--highcharts-heatmap-gradient-positive:linear-gradient(#dcedc8 0%, #42b3d5 35%, #1a237e 100%);--highcharts-heatmap-gradient-negative:linear-gradient(#feeb65 100%, #e4521b 70%, #4d342f 0%);--hypergrid-tree-header--background:#fff;--hypergrid-tree-header--color:#666;--hypergrid-header--background:#fff;--hypergrid-separator--color:#666;--hypergrid-header--color:#666;--hypergrid-positive--color:#1078d1;--hypergrid-negative--color:#ff8888;--hypergrid-positive--background:#fff;--hypergrid-negative--background:#fff;--hypergrid--font-size:12px;--hypergrid-header--font-size:12px;--hypergrid--font-family:Helvetica,Arial,sans-serif;--hypergrid-header--font-family:Helvetica,Arial,sans-serif;--hypergrid-row-hover--background:#f6f6f6;--hypergrid-row-hover--color:#666;--hypergrid-cell-hover--background:#eeeeee;--hypergrid-cell-hover--color:#666;--d3fc-series:rgba(31, 119, 180, 0.5);--d3fc-series-1:#0366d6;--d3fc-series-2:#ff7f0e;--d3fc-series-3:#2ca02c;--d3fc-series-4:#d62728;--d3fc-series-5:#9467bd;--d3fc-series-6:#8c564b;--d3fc-series-7:#e377c2;--d3fc-series-8:#7f7f7f;--d3fc-series-9:#bcbd22;--d3fc-series-10:#17becf;--d3fc-gradient-full:linear-gradient(#4d342f 0%, #e4521b 22.5%, #feeb65 42.5%, #f0f0f0 50%, #dcedc8 57.5%, #42b3d5 67.5%, #1a237e 100%);--d3fc-gradient-positive:linear-gradient(#dcedc8 0%, #42b3d5 35%, #1a237e 100%);--d3fc-gradient-negative:linear-gradient(#feeb65 100%, #e4521b 70%, #4d342f 0%)}:host #pivot_chart{position:absolute;width:100%;height:100%;display:grid;justify-content:stretch}:host .hidden{display:none}:host .psp-icon{color:#999}:host .psp-icon:hover{cursor:pointer}:host .psp-icon__add{display:flex;width:20px;min-width:20px;margin-top:-2px;height:0;font-family:var(--column-add--font-family,inherit)}:host .psp-icon__add:before{font-feature-settings:"liga";content:var(--column-add--before, "+")}:host .psp-title__columnName{display:flex;margin-left:34px;margin-top:3px;font-size:12px;color:#666}:host .side_panel-action:hover{cursor:pointer}:host *{box-sizing:border-box;line-height:1.42857143}:host #app{display:flex;position:relative;width:100%;height:100%}:host #app.hide_message #drop_target{display:none}:host .chart{padding:15px}:host #pivot_chart_container{flex-grow:1;position:relative;border:var(--plugin--border,none)}:host .config{display:flex}:host #column_pivots #psp_row,:host #filters #psp_row,:host #row_pivots #psp_row,:host #sort #psp_row{display:inline}:host ul{min-height:20px;padding:0;margin:0;font-size:12px}:host #active_columns perspective-row{animation:.3s ease-out expand;--column-selector--color:var(--active-column-selector--color, #999);--is_visible--before:var(--active-column-selector--content, "\\25CF")}:host #inactive_columns perspective-row{animation:.3s ease-out expand;--column-selector--color:var(--inactive-column-selector--color, #999);--is_visible--before:var(--inactive-column-selector--content, "\\25CB");--row_draggable--animation:none;opacity:var(--inactive-column--opacity,1);transition:opacity .3s}:host #inactive_columns perspective-row:hover{transition:none;opacity:1}:host #top_panel perspective-row{animation:.3s ease-out expand;margin-bottom:-1px;display:inline-block;--psp_row-display:inline-block}:host #active_columns,:host #inactive_columns{list-style:none;padding:0;width:100%;margin:var(--column-container--margin,8px 0 0 0);overflow-y:auto;overflow-x:hidden;position:relative}:host #inactive_columns perspective-row.active{--row_draggable--margin:0px 0px 0px 10px;opacity:0;height:0!important;overflow:hidden}:host .columns_horizontal #sub_columns.collapse #side_panel__actions{margin-top:0}:host #sub_columns{min-height:33.3%;flex-shrink:100000;display:flex;flex-direction:column;overflow-x:hidden;width:100%}:host #sub_columns.collapse{flex-shrink:0;min-height:64px}:host #sub_columns.collapse #inactive_columns{display:none}:host #divider{min-height:2px;height:2px;margin:15px 13px 5px}:host .column{display:flex;list-style:none;flex-direction:column;flex:1}:host .rrow{display:flex;flex-grow:1;min-height:24px;align-items:center;max-width:400px;list-style:none;min-width:140px;margin:var(--column-drop-container--margin,5px 10px 0 0)}:host .rrow>div{display:flex;min-height:24px;flex-direction:column;justify-content:flex-end}:host #top_panel>.rrow>*{width:100%;padding:12px 0;margin:-12px 0}:host .rrow #psp_row{white-space:nowrap}:host #sort perspective-row{--sort_order-display:inline;--sort_order-padding:0 0 0 10px}:host #sort perspective-row[sort-order=asc]{--row-sort-order--before:var(--sort-order-asc--before, "\\2191")}:host #sort perspective-row[sort-order=desc]{--row-sort-order--before:var(--sort-order-desc--before, "\\2193")}:host #sort perspective-row[sort-order=none]{--row-sort-order--before:var(--sort-order-none--before, "-")}:host #sort perspective-row[sort-order="asc abs"]{--row-sort-order--before:var(--sort-order-asc-abs--before, "\\21E7")}:host #sort perspective-row[sort-order="desc abs"]{--row-sort-order--before:var(--sort-order-desc-abs--before, "\\21E9")}:host #sort perspective-row[sort-order="col asc"]{--row-sort-order--before:var(--sort-order-col-asc--before, "\\2192")}:host #sort perspective-row[sort-order="col desc"]{--row-sort-order--before:var(--sort-order-col-desc--before, "\\2190")}:host #sort perspective-row[sort-order="col asc abs"]{--row-sort-order--before:var(--sort-order-col-asc-abs--before, "\\21E8")}:host #sort perspective-row[sort-order="col desc abs"]{--row-sort-order--before:var(--sort-order-col-desc-abs--before, "\\21E6")}:host #top_panel perspective-row+perspective-row:before{content:",";padding:0 10px 0 0}:host label{white-space:nowrap;display:inline-block;vertical-align:middle;margin-bottom:0}:host #drop_target{position:absolute;left:0;top:0;right:0;bottom:0;padding:30px}:host #drop_target_inner{width:100%;height:100%;display:flex;align-items:center;justify-content:center}:host #drop_target_inner h3{font-weight:300}:host #config_button{position:absolute;top:0;left:0;opacity:0;display:flex;align-items:center;justify-content:center}:host #config_button:hover{color:var(--button-hover--color,inherit)}:host .button{cursor:pointer;font-size:16px;font-weight:400}:host select::-ms-expand{display:none}:host select{-webkit-appearance:none;-moz-appearance:none;-ms-appearance:none;appearance:none;background:url(data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPCEtLSBHZW5lcmF0b3I6IEFkb2JlIElsbHVzdHJhdG9yIDIwLjAuMCwgU1ZHIEV4cG9ydCBQbHVnLUluIC4gU1ZHIFZlcnNpb246IDYuMDAgQnVpbGQgMCkgIC0tPgo8c3ZnIHZlcnNpb249IjEuMSIgaWQ9IkxheWVyXzEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgeG1sbnM6eGxpbms9Imh0dHA6Ly93d3cudzMub3JnLzE5OTkveGxpbmsiIHg9IjBweCIgeT0iMHB4IgoJIHZpZXdCb3g9IjAgMCA0LjkgMTAiIHN0eWxlPSJlbmFibGUtYmFja2dyb3VuZDpuZXcgMCAwIDQuOSAxMDsiIHhtbDpzcGFjZT0icHJlc2VydmUiPgo8c3R5bGUgdHlwZT0idGV4dC9jc3MiPgoJLnN0MHtmaWxsOiM0NDQ0NDQ7fQo8L3N0eWxlPgo8dGl0bGU+YXJyb3dzPC90aXRsZT4KPHBvbHlnb24gY2xhc3M9InN0MCIgcG9pbnRzPSIxLjQsNC43IDIuNSwzLjIgMy41LDQuNyAiLz4KPHBvbHlnb24gY2xhc3M9InN0MCIgcG9pbnRzPSIzLjUsNS4zIDIuNSw2LjggMS40LDUuMyAiLz4KPC9zdmc+Cg==) 95% 50% no-repeat #fff;color:inherit;border-radius:5px;height:24px;padding:var(--select--padding,0 0 0 6px);font-size:12px}:host input{padding:0 10px}:host .centered{display:flex;align-items:center;justify-content:center}:host #top_panel{display:flex;flex-wrap:wrap;justify-content:flex-start;padding:var(--top_panel--padding,5px 0 10px 0);max-width:1000px;z-index:1}:host #side_panel{position:relative;flex:0 0 auto;padding:var(--side_panel--padding,10px 10px 0 11px)}:host #side_panel>div{display:flex}:host #vis_selector_container{min-height:29px;margin-left:var(--column_selector--width,20px);overflow:hidden}:host #vis_selector{flex-grow:1}:host #transpose_button{cursor:pointer;min-width:5px;max-width:5px;flex-grow:0;color:#999;font-family:Arial;font-size:12px}:host #transpose_button:hover{color:var(--button-hover--color,inherit)}:host #resize_bar{position:absolute;top:0;right:0;bottom:0;width:8px;cursor:col-resize}',""])},function(t,e,n){(t.exports=n(7)(!1)).push([t.i,'.psp-text-field{position:relative;background-color:var(--column-drop-container--background,#fff);border-width:0 0 1px;border-style:solid;border-color:var(--column--border-color,#ccc);display:flex;flex-direction:column;justify-content:flex-end;min-height:24px;transition:border-color .3s ease-out}:host(.dragging) .psp-text-field.dropping{border-bottom-width:2px;border-style:solid;border-color:var(--psp-text-field--border-color,#1a7da1)}:host(.dragging) .psp-text-field{border-bottom-width:1px;border-style:solid;border-color:var(--psp-text-field--border-color,#1a7da1)}.psp-text-field>.psp-text-field__input{border:0;line-height:20px;vertical-align:bottom;padding-bottom:0;display:none}.psp-text-field__input+label{height:var(--column--height,auto);line-height:17px;box-sizing:border-box;color:var(--column-drop-label--color,#ccc);font-weight:400;white-space:nowrap;padding:var(--column-drop-container--padding,0 0 2px 10px);font-size:12px;display:inline-block;transition:font-size .3s,margin .3s}.dropping .psp-text-field__input,.psp-text-field__input:not(:empty){display:block!important}.dropping .psp-text-field__input::after{content:"|";width:0;overflow:visible;margin-top:-1px;position:absolute}.dropping *{pointer-events:none}.dropping .psp-text-field label,.psp-text-field__input:not(:empty)+label{position:absolute;top:0;margin:var(--column-drop-label--margin,0);font-size:var(--column-drop-label--font-size,12px);display:var(--column-drop-label--display,none)}.dropping perspective-row.inserting{border-top:25px solid transparent}.dropping perspective-row.postserting{border-bottom:25px solid transparent}:host perspective-row{height:var(--column--height,21px);--row_draggable-height:var(--column--height, 21px)}:host(:not([row-pivots="[]"])) #active_columns perspective-row{height:var(--active-column--height,41px);min-height:var(--active-column--height,41px);--row_draggable-height:var(--active-column--height, 41px);--column_aggregate-display:inline-block;--is_visible--height:var(--active-column-selector--height, auto)}:host{font-family:var(--interface--font-family,Arial,sans-serif);background-color:#eee;color:#333}:host #app.columns_horizontal #columns_container{flex-direction:row-reverse}:host #app.columns_horizontal #active_columns,:host #app.columns_horizontal #inactive_columns{display:flex;flex-direction:column;flex:0 1 auto;width:100%}:host #app.columns_horizontal #inactive_columns{flex:1 1 auto}:host #app.columns_horizontal #sub_columns{flex-shrink:1}:host #app.columns_horizontal #sub_columns perspective-row{--column_name-margin:0 25px 0 0}:host #app.columns_horizontal #side_panel #divider{display:none}:host #app.columns_horizontal perspective-computed-column{max-height:none;margin-right:24px}:host perspective-computed-column{--is_visible--before:var(--active-column-selector--content, "\\25CF");--row_draggable-background-color:var(--active-column--background, white);--row_draggable-border-color:var(--column--border-color, #ccc);--row_draggable-border-width:0 0 1px 0;--row_draggable-overflow:hidden;--column-selector--color:var(--active-column-selector--color, #999)}:host #side_panel perspective-row{--name-before-display:inline-block;--is_visible-display:inline-block;--row_draggable-padding:0px 10px 0px 0px;--row_close-display:none;--column_name-max-width:180px}:host #top_panel perspective-row{--row_draggable-display:inline-block}:host #columns_container{flex-direction:column;height:calc(100% - 29px)}:host #columns_container #active_columns{flex:0 1 auto}:host #columns_container #active_columns perspective-row{margin-bottom:2px;--row_draggable-background-color:var(--active-column--background, white);--row_draggable-border-color:var(--column--border-color, #ccc);--row_draggable-border-width:0 0 1px 0;--row_draggable-overflow:hidden;--row_draggable--justify-content:space-evenly}:host #columns_container #active_columns perspective-row:last-child{--row_draggable-border-width:var(--row_draggable-border-last-child-width, 0 0 1px 0)}:host #columns_container #active_columns perspective-row[drop-target]{--row_draggable_children-opacity:0}:host #columns_container #active_columns,:host #columns_container #inactive_columns{border:none!important}:host ul{transition:background-color .2s,border-color .2s;outline:0;border-style:solid;border-color:var(--column--border-color,#ccc);font-size:12px;padding:var(--column-drop-container--padding,0 10px 0 10px);border-width:0 0 1px}:host #side_panel #side_panel__actions{display:flex;flex-direction:column;flex-shrink:1000;justify-content:flex-start;position:relative;overflow-y:hidden;width:100%;padding-top:20px;margin:var(--column-container--margin,8px);min-height:36px}:host #side_panel #side_panel__actions button.side_panel-action{align-items:center;align-content:center;background:0 0;border:none;display:flex;flex-direction:row;font-family:var(--interface--font-family,Arial,sans-serif)!important;font-size:12px;padding-right:10px;-webkit-appearance:none;-moz-appearance:none;appearance:none}:host #side_panel #side_panel__actions button.side_panel-action span{margin-left:5px;font-size:15px}:host #side_panel #side_panel__actions button.side_panel-action:hover{cursor:pointer}:host .noselect{-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}:host svg text{font-family:var(--interface--font-family,Arial,sans-serif)!important}:host svg .highcharts-axis-title{font-family:var(--interface--font-family,Arial,sans-serif)!important}:host svg .highcharts-legend-box{fill:none}:host #pivot_chart{background-color:var(--plugin--background,#fff)}:host #drop_target{border:var(--plugin--border,none);background:var(--plugin--background,#fff)}:host #pivot_chart_container{box-shadow:var(--plugin--box-shadow,none)}:host #filters perspective-row{margin:0 24px 0 0;--filter_operator-display:inline-block;--filter_operand-display:inline-block}:host .plugin_information{color:var(--warning--color,inherit);display:flex;position:absolute;flex-direction:row;justify-content:flex-start;padding:.5rem;width:100%;z-index:10000;transition:opacity .3s ease-out;opacity:1}:host .plugin_information.hidden{display:none;opacity:0}:host .plugin_information.plugin_information--warning{background:#ffca28}:host .plugin_information .plugin_information__text{font-size:12px;margin-right:.25rem}:host .plugin_information .plugin_information__action{font-size:12px;text-decoration:underline;cursor:pointer}:host .plugin_information .plugin_information__action--dismiss{margin-left:auto}:host #divider{min-height:2px;height:2px;margin:15px 13px 5px}:host #drop_target_inner h3{color:#aaa;font-weight:300}:host #config_button,:host #copy_button,:host #download_button,:host #reset_button{font-weight:400;font-family:var(--button--font-family,Arial);color:var(--button--color,inherit);font-size:var(--button--font-size,16px);color:#999;transition:opacity .3s;overflow:hidden}:host #config_button:hover,:host #copy_button:hover,:host #download_button:hover,:host #reset_button:hover{color:var(--button-hover--color,inherit)}:host #config_button:before{font-feature-settings:"liga";content:var(--config_button--before, "\\1F527")}:host #reset_button:before{font-feature-settings:"liga";content:var(--reset_button--before, "Reset")}:host #download_button:before{font-feature-settings:"liga";content:var(--download_button--before, "Save")}:host #copy_button:before{font-feature-settings:"liga";content:var(--copy_button--before, "Copy")}:host #menubar{display:none;flex-direction:column;padding-top:55px}:host #app.show_menu #menubar{display:flex}:host select{border-radius:0;background-color:var(--select--background-color,#fff);border-style:solid;border-color:var(--column--border-color,#ccc);border-width:0 0 1px}:host select:focus{outline:0}:host input{border-style:solid;border-color:var(--column--border-color,#ccc);border-width:0 0 1px}:host input::placeholder{color:var(--column-drop-label--color,#ccc)}:host option[value=""]{color:#fff!important}:host #app{background-color:inherit}:host perspective-hypergrid{color:#666}:host ::-webkit-scrollbar,:host ::shadow ::-webkit-scrollbar{width:12px}:host ::-webkit-scrollbar-thumb,:host ::shadow ::-webkit-scrollbar-thumb{background:#ccc}:host ::-webkit-scrollbar-track,:host ::shadow ::-webkit-scrollbar-track{background:0 0}',""])},function(t,e){t.exports=f},function(t,e){t.exports=p},function(t,e){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(t){"object"==typeof window&&(n=window)}t.exports=n},function(t,e){t.exports=I},function(t,e){t.exports=C},function(t,e){t.exports=m},function(t,e){t.exports=y},function(t,e,n){"use strict";n.r(e),n(8),n(4),n(5),n(9),n(3);var r=n(6),i=n.n(r),o=(n(23),n(12));function a(t){for(var e=arguments.length,n=new Array(e>1?e-1:0),r=1;r<e;r++)n[r-1]=arguments[r];const i=n.map(t=>t.toString()).join("\n");return function(e){return function(t,e,n){const r=function(t){const e=document.createElement("div");return e.innerHTML=t,Array.prototype.slice.call(e.children)[0]}(t);!function(t){if(t.content)return;let e;for(t.content=document.createDocumentFragment();e=t.firstChild;)Node.prototype.appendChild.call(t.content,e)}(r),e&&(r.innerHTML="<style>".concat(e.toString(),"</style>")+r.innerHTML),r.innerHTML='<style id="psp_styles" scope="'.concat(r.getAttribute("id"),'">test{}</style>')+r.innerHTML;var i=!0,o=!1,a=void 0;try{for(var s,A=Object.getOwnPropertyNames(n.prototype)[Symbol.iterator]();!(i=(s=A.next()).done);i=!0){let t=s.value,e=Object.getOwnPropertyDescriptor(n.prototype,t);if(e&&e.set){let r=e.set;e.set=function(e){this.getAttribute(t)===e?(this._initializing||this._initialized)&&r.call(this,e):this.setAttribute(t,e)},Object.defineProperty(n.prototype,t,e)}}}catch(t){o=!0,a=t}finally{try{i||null==A.return||A.return()}finally{if(o)throw a}}let l=r.getAttribute("id");console.log("Registered ".concat(l)),window.customElements.define(l,class extends n{attributeChangedCallback(t,e,n){"_"!==t[0]&&e!=n&&(this[t]=n)}connectedCallback(){if(!this._initialized){for(this._initializing=!0,this._old_children=[];this.hasChildNodes();)1===this.lastChild.nodeType&&this._old_children.push(this.lastChild),this.removeChild(this.lastChild);this._old_children=this._old_children.reverse();var t=document.importNode(r.content,!0);this.attachShadow({mode:"open"}),this.shadowRoot.appendChild(t),super.connectedCallback&&super.connectedCallback();var e=!0,i=!1,o=void 0;try{for(var a,s=Object.getOwnPropertyNames(n.prototype)[Symbol.iterator]();!(e=(a=s.next()).done);e=!0){let t=a.value;"connectedCallback"!==t&&this.hasAttribute(t)&&"_"!==t[0]&&(this[t]=this.getAttribute(t))}}catch(t){i=!0,o=t}finally{try{e||null==s.return||s.return()}finally{if(i)throw o}}this._initializing=!1,this._initialized=!0}}static get observedAttributes(){return Object.getOwnPropertyNames(n.prototype)}})}(t,{toString:()=>i},e)}}function s(t){return function(e,n,r){const i=r.value;return r.set=function(e){let r=this.getAttribute(n);try{if(null==e&&(e=t()),"string"!=typeof e&&(e=JSON.stringify(e)),e!==r)return this.setAttribute(n,e),void(r=e);r=JSON.parse(r)}catch(i){console.error('Invalid value for attribute "'.concat(n,'": ').concat(e)),r=t()}i.call(this,r)},r.get=function(){if(this.hasAttribute(n))return JSON.parse(this.getAttribute(n))},delete r.value,delete r.writable,r}}n(10);const A=s(()=>({})),l=s(()=>[]);var c=n(2),u=n(13),g=n.n(u);class h{constructor(){this.edit=!1,this.column_name=void 0,this.computation=void 0,this.input_columns=[],this.swap_target=!1,this.name_edited=!1}is_valid(){const t=g()(this);return!t.includes(null)&&!t.includes(void 0)&&!t.includes("")&&this.input_columns.length===this.computation.num_params}}class d{constructor(t,e,n,r){let i=arguments.length>4&&void 0!==arguments[4]?arguments[4]:1;this.name=t,this.input_type=e,this.return_type=n,this.func=r.toString(),this.num_params=i}}var f=n(14),p=n.n(f),I=n(15),C=n.n(I);function m(){var t=!0,e=!1,n=void 0;try{for(var r,i=this._active_columns.children[Symbol.iterator]();!(t=(r=i.next()).done);t=!0)r.value.className=""}catch(t){e=!0,n=t}finally{try{t||null==i.return||i.return()}finally{if(e)throw n}}}function y(t){let e=t.target.getRootNode().host,n=e;n="PERSPECTIVE-VIEWER"===n.tagName?t.target.parentElement:e.parentElement;let r=Array.prototype.slice.call(n.children).indexOf("PERSPECTIVE-ROW"===e.tagName?e:t.target),i=n.getAttribute("for"),o=JSON.parse(this.getAttribute(i));o.splice(r,1),this.setAttribute(i,JSON.stringify(o))}function E(t){t.preventDefault(),t.currentTarget.classList.remove("dropping"),this._drop_target_hover&&this._drop_target_hover.removeAttribute("drop-target");let e=t.dataTransfer.getData("text");if(!e)return;e=JSON.parse(e);let n=t.currentTarget.querySelector("ul").getAttribute("for")||t.currentTarget.getAttribute("id").replace("_","-"),r=JSON.parse(this.getAttribute(n)||"[]"),i=r.indexOf(e[0]);-1!==i&&r.splice(i,1);const o=n.indexOf("filter")>-1;if(o?this.setAttribute(n,JSON.stringify(r.concat([e]))):n.indexOf("sort")>-1?this.setAttribute(n,JSON.stringify(r.concat([[e[0]]]))):this.setAttribute(n,JSON.stringify(r.concat([e[0]]))),"pivots"===this._plugin.deselectMode&&this._get_visible_column_count()>1&&"sort"!==n&&!o){var a=!0,s=!1,A=void 0;try{for(var l,c=this.shadowRoot.querySelectorAll("#active_columns perspective-row")[Symbol.iterator]();!(a=(l=c.next()).done);a=!0){let t=l.value;if(t.getAttribute("name")===e[0]){this._active_columns.removeChild(t);break}}}catch(t){s=!0,A=t}finally{try{a||null==c.return||c.return()}finally{if(s)throw A}}this._update_column_view()}this._debounce_update()}function v(t){t.stopPropagation(),t.preventDefault(),t.currentTarget.classList.add("dropping")}function B(t){t.stopPropagation(),t.preventDefault(),t.currentTarget.classList.add("dropping"),t.dataTransfer.dropEffect="move"}function b(t){t.currentTarget==t.target&&(t.stopPropagation(),t.preventDefault(),t.currentTarget.classList.remove("dropping"))}function Q(){return Reflect.construct(HTMLElement,[],this.__proto__.constructor)}Object.setPrototypeOf(Q.prototype,HTMLElement.prototype),Object.setPrototypeOf(Q,HTMLElement);const w={hour_of_day:new d("hour_of_day","datetime","integer",function(t){return new Date(t).getHours()}),day_of_week:new d("day_of_week","datetime","string",function(t){return["1 Sunday","2 Monday","3 Tuesday","4 Wednesday","5 Thursday","6 Friday","7 Saturday"][new Date(t).getDay()]}),month_of_year:new d("month_of_year","datetime","string",function(t){return["01 January","02 February","03 March","04 April","05 May","06 June","07 July","08 August","09 September","10 October","11 November","12 December"][new Date(t).getMonth()]}),hour_bucket:new d("hour_bucket","datetime","datetime",function(t){let e=new Date(t);return e.setMinutes(0),e.setSeconds(0),e}),day_bucket:new d("day_bucket","datetime","date",function(t){let e=new Date(t);return e.setHours(0),e.setMinutes(0),e.setSeconds(0),e}),week_bucket:new d("week_bucket","datetime","date",function(t){let e=new Date(t),n=e.getDay(),r=e.getDate()-n+(0==n?-6:1);return e.setHours(0),e.setMinutes(0),e.setSeconds(0),e.setDate(r),e}),month_bucket:new d("month_bucket","datetime","date",function(t){let e=new Date(t);return e.setHours(0),e.setMinutes(0),e.setSeconds(0),e.setDate(1),e}),uppercase:new d("uppercase","string","string",t=>t.toUpperCase()),lowercase:new d("lowercase","string","string",t=>t.toLowerCase()),length:new d("length","string","integer",t=>t.length),add:new d("add","float","float",(t,e)=>t+e,2),subtract:new d("subtract","float","float",(t,e)=>t-e,2),multiply:new d("multiply","float","float",(t,e)=>t*e,2),divide:new d("divide","float","float",(t,e)=>t/e,2),percent_a_of_b:new d("percent_a_of_b","float","float",(t,e)=>t/e*100,2),concat_space:new d("concat_space","string","string",(t,e)=>t+" "+e,2),concat_comma:new d("concat_comma","string","string",(t,e)=>t+", "+e,2)};a(p.a,C.a)(class extends Q{constructor(){super(),this.state=new h,this.column_names=[],this.type_markers={float:"123",integer:"123",string:"abc",boolean:"t/f",datetime:"mdy",date:"mdy"}}get computations(){return w}connectedCallback(){this._register_ids(),this._register_computations(),this._register_callbacks(),this._update_computation(null),this._register_inputs()}_register_computations(){this._computation_selector.innerHTML="";let t=!0;for(var e=0,n=Object.keys(w);e<n.length;e++){let r=n[e];this._computation_selector.innerHTML+='<option value="'.concat(r,'"').concat(t?' selected="selected"':"",">").concat(r.replace(/_/g," "),"</option>"),t=!1}}_register_inputs(){this._clear_error_messages(),this._disable_save_button(),this._input_columns.innerHTML="";const t=this.state.computation,e=t.input_type;this.state.input_columns=[],this.state.swap_target=!1;for(let n=0;n<t.num_params;n++)this._input_columns.innerHTML+='<div class="psp-cc-computation__input-column" \n                      data-index="'.concat(n,'" \n                      drop-target>\n                      <span class="psp-label__requiredType ').concat(e,'"></span>\n                      <span class="psp-label__placeholder">Param ').concat(n+1,'</span>\n                      <div class="psp-cc-computation__drop-target-hover"></div>\n                </div>');var n=!0,r=!1,i=void 0;try{for(var o,a=this._input_columns.children[Symbol.iterator]();!(n=(o=a.next()).done);n=!0){let t=o.value;t.addEventListener("drop",this._drop_column.bind(this)),t.addEventListener("dragstart",this._drag_column.bind(this)),t.addEventListener("dragend",this._remove_column.bind(this)),t.addEventListener("dragover",this._hover_column.bind(this)),t.addEventListener("dragleave",this._pass_column.bind(this))}}catch(t){r=!0,i=t}finally{try{n||null==a.return||a.return()}finally{if(r)throw i}}this._clear_column_name()}_parse_data_transfer(t){const e=JSON.parse(t);if(e)return{column_name:e[0],column_type:e[3]}}_drag_column(t){this.state.computation.num_params>1&&(this.state.swap_target=t.currentTarget)}_hover_column(t){t.preventDefault(),t.dataTransfer.dropEffect="move";const e=t.currentTarget,n=e.querySelector(".psp-cc-computation__drop-target-hover");"dropping"!==e.className&&e.classList.add("dropping"),n&&!n.hasAttribute("drop-target")&&n.setAttribute("drop-target","true"),2===e.children.length&&e.parentNode.insertBefore(n,e.nextSibling)}_drop_column(t){const e=t.currentTarget;t.preventDefault(),e.classList.remove("dropping");const n=void 0!==this.state.swap_target&&e.innerHTML.indexOf("perspective-row")>-1,r=this._parse_data_transfer(t.dataTransfer.getData("text"));if(r){if(n){const n=e.children[0],r=n.getAttribute("name"),i=n.getAttribute("type");t.swapTarget=this.state.swap_target,this._set_input_column(t,r,i),this.state.swap_target=!1,delete t.swapTarget}this._set_input_column(t,r.column_name,r.column_type)}}deselect_column(t){this.state.input_columns=this.state.input_columns.map(e=>e&&e.name===t?void 0:e),this._apply_state(this.state.input_columns,this.state.computation)}_remove_column(t){t.currentTarget.classList.remove("dropping")}_pass_column(t){const e=t.currentTarget,n=e.getBoundingClientRect(),r=t.pageX>=n.left&&t.pageX<=n.right-2,i=t.pageY>=n.top&&t.pageY<=n.bottom-2;if(!(r&&i||null===e||"SPAN"===e.nodeName)){e.classList.remove("dropping");const t=e.querySelector(".psp-cc-computation__drop-target-hover");t&&t.removeAttribute("drop-target")}}_apply_state(t,e,n){this._update_computation(null,e.name),this.state.input_columns=t;const r=this._input_columns.children;for(let t=0;t<this.state.input_columns.length;t++)void 0!==this.state.input_columns[t]&&this._set_input_column({currentTarget:r[t]},this.state.input_columns[t].name,this.state.input_columns[t].type);this._column_name_input.innerText=n||"",this._set_column_name(),this.state.name_edited=void 0!==n,this.state.is_valid()&&this._enable_save_button()}_set_column_name(){let t=this._column_name_input.innerText;if(0==t.length)return this.state.column_name=void 0,void this._disable_save_button();this.state.column_name=t,this.state.is_valid()&&this._enable_save_button()}_auto_column_name(){if(!this.state.name_edited){if(this.state.input_columns.length>0){let o=[];var t=!0,e=!1,n=void 0;try{for(var r,i=this.state.input_columns[Symbol.iterator]();!(t=(r=i.next()).done);t=!0){let t=r.value;o.push(t.name)}}catch(t){e=!0,n=t}finally{try{t||null==i.return||i.return()}finally{if(e)throw n}}this._column_name_input.innerText="".concat(this.state.computation.name,"(").concat(o.join(", "),")")}else this._column_name_input.innerText="";this._set_column_name()}}_clear_column_name(){this._column_name_input.innerText="",this.state.name_edited=!1,this._set_column_name()}_set_input_column(t,e,n){const r=this.state.computation,i=r.input_type,o=this.state.input_columns;let a;a=t.swapTarget?t.swapTarget:t.currentTarget;const s=Number.parseInt(a.getAttribute("data-index"));if("float"!==i&&"datetime"!==i&&n!==i||"float"===i&&"float"!==n&&"integer"!==n||"datetime"===i&&"datetime"!==n&&"date"!==n)return this._register_inputs(),void a.classList.remove("dropped");a.classList.add("dropped");const A=a.querySelector(".psp-cc-computation__drop-target-hover");A&&A.removeAttribute("drop-target"),a.innerHTML="";const l={name:e,type:n};o[s]=l,this.state.input_columns=o,o.filter(t=>t).length===r.num_params&&this._auto_column_name(),this.dispatchEvent(new CustomEvent("perspective-computed-column-update",{detail:{target:a,column:l}})),this.state.is_valid()&&this._enable_save_button()}_update_computation(t,e){const n=this._computation_selector;e?null==t&&(n.value=e):e=n[n.selectedIndex].value;const r=Object.assign({},w[e]);if(void 0===r)throw"Undefined computation could not be set.";const i=r.num_params,o=r.input_type,a=r.return_type;let s=!0;this.state.computation&&(s=o!==this.state.computation.input_type||i!==this.state.computation.num_params),this._computation_type.innerHTML='<span class="'.concat(a,'">').concat(this.type_markers[a],"</span>"),this.state.computation=r,s||null===t?(this._register_inputs(),this._clear_column_name()):this._auto_column_name()}_set_error_message(t,e){e&&(e.innerText=t,e.style.display="block")}_clear_error_messages(){this._column_name_error.innerText="",this._column_name_error.style.display="none"}_disable_save_button(){this._save_button.setAttribute("disabled","true")}_enable_save_button(){this._save_button.removeAttribute("disabled")}_save_computed_column(){if(this.state.is_valid()){const t=this.state;if(this.column_names.includes(this.state.column_name))return void this._set_error_message("Column names must be unique.",this._column_name_error);this._clear_error_messages();const e=new CustomEvent("perspective-computed-column-save",{detail:{name:t.column_name,inputs:t.input_columns.map(t=>t.name),func:t.computation.name}});this.dispatchEvent(e),this.column_names.push(t.column_name)}}_close_computed_column(){this.style.display="none",this._side_panel_actions.style.display="flex",this.classList.remove("edit"),this._column_name_input.innerText="",this._input_columns.innerHTML="";var t=!0,e=!1,n=void 0;try{for(var r,i=this._input_columns.children[Symbol.iterator]();!(t=(r=i.next()).done);t=!0)r.value.classList.remove("dropped")}catch(t){e=!0,n=t}finally{try{t||null==i.return||i.return()}finally{if(e)throw n}}this.state=new h,this._update_computation()}_register_ids(){this._side_panel_actions=this.parentElement.querySelector("#side_panel__actions"),this._close_button=this.shadowRoot.querySelector("#psp-cc__close"),this._column_name_input=this.shadowRoot.querySelector("#psp-cc-name"),this._column_name_error=this.shadowRoot.querySelector("#psp-cc__error--name"),this._computation_selector=this.shadowRoot.querySelector("#psp-cc-computation__select"),this._computation_type=this.shadowRoot.querySelector("#psp-cc-computation__type"),this._input_columns=this.shadowRoot.querySelector("#psp-cc-computation-inputs"),this._save_button=this.shadowRoot.querySelector("#psp-cc-button-save")}_register_callbacks(){this._close_button.addEventListener("click",this._close_computed_column.bind(this)),this._computation_selector.addEventListener("change",this._update_computation.bind(this)),this._column_name_input.addEventListener("dragover",b.bind(this)),this._column_name_input.addEventListener("keyup",t=>{this.state.name_edited=this._column_name_input.innerText&&this._column_name_input.innerText.length>0,this._set_column_name(t)}),this._save_button.addEventListener("click",this._save_computed_column.bind(this))}}),n(25),n(26);var x=n(11),_=n.n(x),D=n(16),M=n.n(D),k=n(0),S=n.n(k),L=n(1),R=n(17),N=n.n(R),F=n(18),T=n.n(F);function U(){return Reflect.construct(HTMLElement,[],this.__proto__.constructor)}function O(t){let e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,n=document.createElement("span");n.style.visibility="hidden",n.style.fontFamily="monospace",n.style.fontSize="12px",n.style.position="absolute",n.innerHTML=t,document.body.appendChild(n);let r="".concat(Math.max(e,n.offsetWidth)+20,"px");return document.body.removeChild(n),r}Object.setPrototypeOf(U.prototype,HTMLElement.prototype),Object.setPrototypeOf(U,HTMLElement),a(N.a,{toString:()=>T.a+"\n"+M.a})(class extends U{set name(t){this.shadowRoot.querySelector("#name").innerHTML=this.getAttribute("name")}set type(t){let e=this.shadowRoot.querySelector("#name"),n=this.getAttribute("type");if(!n)return;e.classList.add(n);let r=this.shadowRoot.querySelector("#column_aggregate"),i=this.shadowRoot.querySelector("#filter_operator");switch(n){case"float":case"integer":r.innerHTML=S.a.TYPE_AGGREGATES.float.map(t=>'<option value="'.concat(t,'">').concat(t,"</option>")).join(""),i.innerHTML=S.a.TYPE_FILTERS.float.map(t=>'<option value="'.concat(t,'">').concat(t,"</option>")).join("");break;case"boolean":r.innerHTML=S.a.TYPE_AGGREGATES.boolean.map(t=>'<option value="'.concat(t,'">').concat(t,"</option>")).join(""),i.innerHTML=S.a.TYPE_FILTERS.boolean.map(t=>'<option value="'.concat(t,'">').concat(t,"</option>")).join("");break;case"date":case"datetime":r.innerHTML=S.a.TYPE_AGGREGATES.datetime.map(t=>'<option value="'.concat(t,'">').concat(t,"</option>")).join(""),i.innerHTML=S.a.TYPE_FILTERS.datetime.map(t=>'<option value="'.concat(t,'">').concat(t,"</option>")).join("");break;case"string":r.innerHTML=S.a.TYPE_AGGREGATES.string.map(t=>'<option value="'.concat(t,'">').concat(t,"</option>")).join(""),i.innerHTML=S.a.TYPE_FILTERS.string.map(t=>'<option value="'.concat(t,'">').concat(t,"</option>")).join("")}this.hasAttribute("aggregate")||this.setAttribute("aggregate",Object(L.get_type_config)(n).aggregate);let o=this.shadowRoot.querySelector("#filter_operand");this._callback=t=>this._update_filter(t),o.addEventListener("keyup",this._callback.bind(this))}choices(t){let e=this.shadowRoot.querySelector("#filter_operand"),n=this.shadowRoot.querySelector("#filter_operator");const r=new _.a(e,{label:this.getAttribute("name"),list:t,minChars:0,autoFirst:!0,filter:function(t,e){return _.a.FILTER_CONTAINS(t,e.match(/[^,]*$/)[0])},item:function(t,e){return _.a.ITEM(t,e.match(/[^,]*$/)[0])},replace:function(t){let e=this.input.value.match(/^.+,\s*|/)[0];"in"===n.value||"not in"===n.value?this.input.value=e+t+", ":this.input.value=e+t}});""===e.value&&r.evaluate(),e.focus(),this._filter_operand.addEventListener("focus",()=>{0===e.value.trim().length&&r.evaluate()}),e.addEventListener("awesomplete-selectcomplete",this._callback)}set filter(t){const e=this.shadowRoot.querySelector("#filter_operator"),n=JSON.parse(this.getAttribute("filter"));e.value!==n.operator&&(e.value=n.operator||Object(L.get_type_config)(this.getAttribute("type")).filter_operator),e.style.width=O(e.value);const r=this.shadowRoot.querySelector("#filter_operand"),i=n.operand?n.operand.toString():"";this._initialized||(r.value=i),r.style.width=O(i,30)}set aggregate(t){let e=this.shadowRoot.querySelector("#column_aggregate"),n=this.getAttribute("aggregate");if(e.value!==n&&this.hasAttribute("type")){let t=this.getAttribute("type");e.value=n||Object(L.get_type_config)(t).aggregate}}set computed_column(t){}_get_computed_data(){const t=JSON.parse(this.getAttribute("computed_column"));return{column_name:t.column_name,input_columns:t.input_columns,input_type:t.input_type,computation:t.computation,type:t.type}}_update_filter(t){let e=this.shadowRoot.querySelector("#filter_operand"),n=this.shadowRoot.querySelector("#filter_operator"),r=e.value;switch(this.getAttribute("type")){case"float":r=parseFloat(r);break;case"integer":r=parseInt(r);break;case"boolean":r=r.toLowerCase().indexOf("true")>-1}"in"!==n.value&&"not in"!==n.value||(r=r.split(",").map(t=>t.trim())),this.setAttribute("filter",JSON.stringify({operator:n.value,operand:r})),this.dispatchEvent(new CustomEvent("filter-selected",{detail:t}))}_set_data_transfer(t){if(this.hasAttribute("filter")){let e=JSON.parse(this.getAttribute("filter")),n=e.operator,r=e.operand;t.dataTransfer.setData("text",JSON.stringify([this.getAttribute("name"),n,r,this.getAttribute("type"),this.getAttribute("aggregate")]))}else t.dataTransfer.setData("text",JSON.stringify([this.getAttribute("name"),Object(L.get_type_config)(this.getAttribute("type")).filter_operator,void 0,this.getAttribute("type"),this.getAttribute("aggregate")]));this.dispatchEvent(new CustomEvent("row-drag"))}_register_ids(){this._li=this.shadowRoot.querySelector(".row_draggable"),this._visible=this.shadowRoot.querySelector(".is_visible"),this._row_close=this.shadowRoot.querySelector("#row_close"),this._agg_dropdown=this.shadowRoot.querySelector("#column_aggregate"),this._sort_order=this.shadowRoot.querySelector("#sort_order"),this._filter_operand=this.shadowRoot.querySelector("#filter_operand"),this._filter_operator=this.shadowRoot.querySelector("#filter_operator"),this._edit_computed_column_button=this.shadowRoot.querySelector("#row_edit")}_register_callbacks(){this._li.addEventListener("dragstart",this._set_data_transfer.bind(this)),this._li.addEventListener("dragend",()=>{this.dispatchEvent(new CustomEvent("row-dragend"))}),this._visible.addEventListener("mousedown",t=>this.dispatchEvent(new CustomEvent("visibility-clicked",{detail:t}))),this._row_close.addEventListener("mousedown",t=>this.dispatchEvent(new CustomEvent("close-clicked",{detail:t}))),this._agg_dropdown.addEventListener("change",t=>{this.setAttribute("aggregate",this._agg_dropdown.value),this.dispatchEvent(new CustomEvent("aggregate-selected",{detail:t}))}),this._sort_order.addEventListener("click",t=>{this.dispatchEvent(new CustomEvent("sort-order",{detail:t}))});const t=i()(t=>this._update_filter(t),50);this._filter_operator.addEventListener("change",()=>{this._filter_operand.focus(),this._filter_operator.style.width=O(this._filter_operator.value),this.shadowRoot.querySelector("#filter_operand").style.width=O(""+this._filter_operand.value,30),t()}),this._edit_computed_column_button.addEventListener("click",()=>{this.dispatchEvent(new CustomEvent("perspective-computed-column-edit",{bubbles:!0,detail:this._get_computed_data()}))})}connectedCallback(){this._register_ids(),this._register_callbacks()}});var G=n(19),H=n.n(G),j=n(20),Y=n.n(j),P=n(21),K=n.n(P),z=(n(27),n(28),n(22)),J=n.n(z);class q{constructor(t){let e=arguments.length>1&&void 0!==arguments[1]&&arguments[1];this._on_cancel=t,this._cancelled=!1,this.initial=e}cancel(){!this._cancelled&&this._on_cancel&&(this._on_cancel(),this._on_cancel=void 0),this._cancelled=!0}get cancelled(){return this._cancelled}}function V(){return Reflect.construct(HTMLElement,[],this.__proto__.constructor)}Object.setPrototypeOf(V.prototype,HTMLElement.prototype),Object.setPrototypeOf(V,HTMLElement);class W extends V{get _plugin(){let t=c.b.getInstance(),e=this.getAttribute("plugin");return e||(e=Object.keys(t)[0]),this.setAttribute("plugin",e),t[e]||t[Object.keys(t)[0]]}_get_view_dom_columns(t,e){t=t||"#active_columns perspective-row";let n=Array.prototype.slice.call(this.shadowRoot.querySelectorAll(t));return e?n.map(e):n}_get_view_columns(){let t,e=(arguments.length>0&&void 0!==arguments[0]?arguments[0]:{}).active;return t=void 0===e||e?"#active_columns perspective-row":"#inactive_columns perspective-row",this._get_view_dom_columns(t,t=>t.getAttribute("name"))}_get_view_aggregates(t){return t=t||"#active_columns perspective-row",this._get_view_dom_columns(t,t=>({op:t.getAttribute("aggregate"),column:t.getAttribute("name")}))}_get_view_row_pivots(){return this._get_view_dom_columns("#row_pivots perspective-row",t=>t.getAttribute("name"))}_get_view_column_pivots(){return this._get_view_dom_columns("#column_pivots perspective-row",t=>t.getAttribute("name"))}_get_view_filter_nodes(){return this._get_view_dom_columns("#filters perspective-row")}_get_view_filters(){return this._get_view_dom_columns("#filters perspective-row",t=>{let e=JSON.parse(t.getAttribute("filter")),n=e.operator,r=e.operand;return[t.getAttribute("name"),n,r]})}_get_view_sorts(){return this._get_view_dom_columns("#sort perspective-row",t=>{let e=t.getAttribute("sort-order")||"asc";return[t.getAttribute("name"),e]})}_get_visible_column_count(){return this._get_view_dom_columns().length}get_aggregate_attribute(){const t=JSON.parse(this.getAttribute("aggregates"))||{};return Object.keys(t).map(e=>({column:e,op:t[e]}))}}function X(t,e){return function(t){if(Array.isArray(t))return t}(t)||function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!e||n.length!==e);r=!0);}catch(t){i=!0,o=t}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(t,e)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}function Z(t){return t.toString().replace(/\B(?=(\d{3})+(?!\d))/g,",")}let $={integer:2,string:0,float:3,boolean:4,datetime:1,date:1};const tt=t=>(e,n)=>{const r=$[t[e]],i=$[t[n]];return r==i?e.toLowerCase()<n.toLowerCase()?-1:1:r<i?-1:1};function et(t,e,n){const r=new Set,i=[];var o=!0,a=!1,s=void 0;try{for(var A,l=t[Symbol.iterator]();!(o=(A=l.next()).done);o=!0){const t=A.value,n=e[t.column];r.add(t.column),n?(""!==t.op&&-1!==S.a.TYPE_AGGREGATES[n].indexOf(t.op)||(t.op=Object(L.get_type_config)(n).aggregate),i.push(t)):console.warn('No column "'.concat(t.column,'" found (specified in aggregates attribute).'))}}catch(t){a=!0,s=t}finally{try{o||null==l.return||l.return()}finally{if(a)throw s}}var c=!0,u=!1,g=void 0;try{for(var h,d=n[Symbol.iterator]();!(c=(h=d.next()).done);c=!0){const t=h.value;r.has(t)||i.push({column:t,op:Object(L.get_type_config)(e[t]).aggregate})}}catch(t){u=!0,g=t}finally{try{c||null==d.return||d.return()}finally{if(u)throw g}}return i}class nt extends W{_check_recreate_computed_columns(){var t,e,n,r,i,o,a,s=this;return regeneratorRuntime.async(function(A){for(;;)switch(A.prev=A.next){case 0:if(!((t=JSON.parse(s.getAttribute("computed-columns"))).length>0)){A.next=30;break}e=!0,n=!1,r=void 0,A.prev=5,i=t[Symbol.iterator]();case 7:if(e=(o=i.next()).done){A.next=14;break}return a=o.value,A.next=11,regeneratorRuntime.awrap(s._create_computed_column({detail:{column_name:a.name,input_columns:a.inputs.map(t=>({name:t})),computation:w[a.func]}}));case 11:e=!0,A.next=7;break;case 14:A.next=20;break;case 16:A.prev=16,A.t0=A.catch(5),n=!0,r=A.t0;case 20:A.prev=20,A.prev=21,e||null==i.return||i.return();case 23:if(A.prev=23,!n){A.next=26;break}throw r;case 26:return A.finish(23);case 27:return A.finish(20);case 28:return s._debounce_update({ignore_size_check:!1}),A.abrupt("return",!0);case 30:return A.abrupt("return",!1);case 31:case"end":return A.stop()}},null,null,[[5,16,20,28],[21,,23,27]])}_load_table(t){var e,n,r,i,o,a,s,A,l,c,u,g,h,d,f,p,I,C,m,y,E,v,B,b,Q,w,x=this,_=arguments;return regeneratorRuntime.async(function(D){for(;;)switch(D.prev=D.next){case 0:if(e=_.length>1&&void 0!==_[1]&&_[1],x.shadowRoot.querySelector("#app").classList.add("hide_message"),n=x._set_updating(),x._table&&!e&&x.removeAttribute("computed-columns"),x._clear_state(),x._table=t,!x.hasAttribute("computed-columns")||e){D.next=11;break}return D.next=9,regeneratorRuntime.awrap(x._check_recreate_computed_columns());case 9:if(!D.sent){D.next=11;break}return D.abrupt("return");case 11:return D.next=13,regeneratorRuntime.awrap(Promise.all([t.columns(),t.schema(!0),t.computed_schema()]));case 13:for(r=D.sent,i=X(r,3),o=i[0],a=i[1],s=i[2],x._clear_columns(),x._initial_col_order=o.slice(),x.hasAttribute("columns")||x.setAttribute("columns",JSON.stringify(x._initial_col_order)),o.sort(tt(a)),A=Object.entries(s).map(t=>{let e=X(t,2);return{column:e[0],op:e[1]}}),l=o.concat(Object.keys(s)),c=et(x.get_aggregate_attribute().concat(A),a,l),0===(u=JSON.parse(x.getAttribute("columns")).filter(t=>l.indexOf(t)>-1)).length&&(u=x._initial_col_order),x.set_aggregate_attribute(c),g=!0,h=!1,d=void 0,D.prev=31,f=l[Symbol.iterator]();!(g=(p=f.next()).done);g=!0)I=p.value,C=c.find(t=>t.column===I).op,m=x._new_row(I,a[I],C,null,null,s[I]),x._inactive_columns.appendChild(m),u.includes(I)&&m.classList.add("active");D.next=39;break;case 35:D.prev=35,D.t0=D.catch(31),h=!0,d=D.t0;case 39:D.prev=39,D.prev=40,g||null==f.return||f.return();case 42:if(D.prev=42,!h){D.next=45;break}throw d;case 45:return D.finish(42);case 46:return D.finish(39);case 47:for(y=!0,E=!1,v=void 0,D.prev=50,B=u[Symbol.iterator]();!(y=(b=B.next()).done);y=!0)Q=b.value,w=x._new_row(Q,a[Q]),x._active_columns.appendChild(w);D.next=58;break;case 54:D.prev=54,D.t1=D.catch(50),E=!0,v=D.t1;case 58:D.prev=58,D.prev=59,y||null==B.return||B.return();case 61:if(D.prev=61,!E){D.next=64;break}throw v;case 64:return D.finish(61);case 65:return D.finish(58);case 66:return l.length===u.length?x._inactive_columns.parentElement.classList.add("collapse"):x._inactive_columns.parentElement.classList.remove("collapse"),x._show_column_selectors(),x.filters=x.getAttribute("filters"),D.next=71,regeneratorRuntime.awrap(x._debounce_update({force_update:!0}));case 71:n();case 72:case"end":return D.stop()}},null,null,[[31,35,39,47],[40,,42,46],[50,54,58,66],[59,,61,65]])}_warn_render_size_exceeded(){var t,e,n,r,i,o=this;return regeneratorRuntime.async(function(a){for(;;)switch(a.prev=a.next){case 0:if(!o._show_warnings||void 0===o._plugin.max_size){a.next=18;break}return a.next=3,regeneratorRuntime.awrap(o._view.num_columns());case 3:return t=a.sent,a.next=6,regeneratorRuntime.awrap(o._view.num_rows());case 6:if(e=a.sent,!((n=t*e)>=o._plugin.max_size)){a.next=17;break}return o._plugin_information.classList.remove("hidden"),r=Math.floor(n/o._plugin.max_size*100)-100,i="Rendering estimated ".concat(Z(n)," (+").concat(Z(r),"%) points.  "),o._plugin_information_message.innerText=i,o.removeAttribute("updating"),a.abrupt("return",!0);case 17:o._plugin_information.classList.add("hidden");case 18:return a.abrupt("return",!1);case 19:case"end":return a.stop()}})}_view_on_update(){this._debounced||(this._debounced=setTimeout(()=>{var t,e,n;return regeneratorRuntime.async(function(r){for(;;)switch(r.prev=r.next){case 0:return this._debounced=void 0,t=this._render_time(),this._task&&!this._task.initial&&this._task.cancel(),e=this._task=new q,n=this._plugin.update||this._plugin.create,r.prev=5,r.next=8,regeneratorRuntime.awrap(n.call(this,this._datavis,this._view,e));case 8:t(),e.cancel(),r.next=15;break;case 12:r.prev=12,r.t0=r.catch(5),console.error("Error rendering plugin.",r.t0);case 15:return r.prev=15,this.dispatchEvent(new Event("perspective-view-update")),r.finish(15);case 18:case"end":return r.stop()}},null,this,[[5,12,15,18]])},function(t){const e=t*document.getElementsByTagName("perspective-viewer").length*2;return Math.min(1e4,Math.max(0,e))}(this.getAttribute("render_time"))))}_validate_filters(){var t,e,n,r,i,o,a,s,A,l,c,u,g,h=this;return regeneratorRuntime.async(function(d){for(;;)switch(d.prev=d.next){case 0:t=[],e=!0,n=!1,r=void 0,d.prev=4,i=h._get_view_filter_nodes()[Symbol.iterator]();case 6:if(e=(o=i.next()).done){d.next=30;break}return a=o.value,s=a.shadowRoot.getElementById("filter_operand"),A=a.shadowRoot.getElementById("row_exclamation"),l=JSON.parse(a.getAttribute("filter")),c=l.operator,u=l.operand,g=[a.getAttribute("name"),c,u],d.next=14,regeneratorRuntime.awrap(h._table.is_valid_filter(g));case 14:if(d.t0=d.sent,!d.t0){d.next=17;break}d.t0=""!==u;case 17:if(!d.t0){d.next=24;break}t.push(g),a.title="",s.style.borderColor="",A.hidden=!0,d.next=27;break;case 24:a.title="Invalid Filter",s.style.borderColor="red",A.hidden=!1;case 27:e=!0,d.next=6;break;case 30:d.next=36;break;case 32:d.prev=32,d.t1=d.catch(4),n=!0,r=d.t1;case 36:d.prev=36,d.prev=37,e||null==i.return||i.return();case 39:if(d.prev=39,!n){d.next=42;break}throw r;case 42:return d.finish(39);case 43:return d.finish(36);case 44:return d.abrupt("return",t);case 45:case"end":return d.stop()}},null,null,[[4,32,36,44],[37,,39,43]])}_is_config_changed(t){const e=this.getAttribute("plugin");return!(J()(t,this._previous_config)&&e===this._previous_plugin_name||(this._previous_config=t,this._previous_plugin_name=e,0))}_new_view(){var t,e,n,r,i,o,a,s,A,l,c,u,g,h,d,f,p,I,C,m,y,E,v,B,b,Q,w,x,_,D,M,k,S=this,L=arguments;return regeneratorRuntime.async(function(R){for(;;)switch(R.prev=R.next){case 0:if(t=L.length>0&&void 0!==L[0]?L[0]:{},e=t.ignore_size_check,n=void 0!==e&&e,r=t.force_update,i=void 0!==r&&r,S._table){R.next=3;break}return R.abrupt("return");case 3:return S._check_responsive_layout(),o=S._get_view_row_pivots(),a=S._get_view_column_pivots(),R.next=8,regeneratorRuntime.awrap(S._validate_filters());case 8:if(s=R.sent,0!==(A=S._get_view_aggregates()).length){R.next=12;break}return R.abrupt("return");case 12:for(l=S._get_view_sorts(),c=A.map(t=>t.column),u={},g=!0,h=!1,d=void 0,R.prev=18,f=A[Symbol.iterator]();!(g=(p=f.next()).done);g=!0)I=p.value,u[I.column]=I.op;R.next=26;break;case 22:R.prev=22,R.t0=R.catch(18),h=!0,d=R.t0;case 26:R.prev=26,R.prev=27,g||null==f.return||f.return();case 29:if(R.prev=29,!h){R.next=32;break}throw d;case 32:return R.finish(29);case 33:return R.finish(26);case 34:for(C=!0,m=!1,y=void 0,R.prev=37,E=l[Symbol.iterator]();!(C=(v=E.next()).done);C=!0)B=v.value,b=B[0],-1!==c.indexOf(b)||a.indexOf(B)>-1||o.indexOf(B)>-1||(Q=S.get_aggregate_attribute(),w=Q.reduce((t,e)=>e.column===b?e:t),x=w.column,_=w.op,u[x]=_);R.next=45;break;case 41:R.prev=41,R.t1=R.catch(37),m=!0,y=R.t1;case 45:R.prev=45,R.prev=46,C||null==E.return||E.return();case 48:if(R.prev=48,!m){R.next=51;break}throw y;case 51:return R.finish(48);case 52:return R.finish(45);case 53:if(D={filter:s,row_pivots:o,column_pivots:a,aggregates:u,columns:c,sort:l},S._is_config_changed(D)||n||i){R.next=57;break}return S.removeAttribute("updating"),R.abrupt("return");case 57:if(S._view&&(S._view.delete(),S._view.remove_update(S._view_updater),S._view.remove_delete(),S._view=void 0),S._view=S._table.view(D),n){R.next=64;break}return R.next=62,regeneratorRuntime.awrap(S._warn_render_size_exceeded());case 62:if(!R.sent){R.next=64;break}return R.abrupt("return");case 64:return S._view_updater=()=>S._view_on_update(),S._view.on_update(S._view_updater),M=S._render_time(),S._render_count=(S._render_count||0)+1,S._task&&S._task.cancel(),k=S._task=new q(()=>S._render_count--,!0),R.prev=70,R.next=73,regeneratorRuntime.awrap(S._plugin.create.call(S,S._datavis,S._view,k));case 73:R.next=78;break;case 75:R.prev=75,R.t2=R.catch(70),console.warn(R.t2);case 78:return R.prev=78,S.hasAttribute("render_time")||S.dispatchEvent(new Event("perspective-view-update")),M(),k.cancel(),0===S._render_count&&(S.removeAttribute("updating"),S.dispatchEvent(new Event("perspective-update-complete"))),R.finish(78);case 84:case"end":return R.stop()}},null,null,[[18,22,26,34],[27,,29,33],[37,41,45,53],[46,,48,52],[70,75,78,84]])}_render_time(){const t=performance.now();return()=>this.setAttribute("render_time",performance.now()-t)}_restyle_plugin(){if(this._plugin.styleElement){const t=this._task=new q;this._plugin.styleElement.call(this,this._datavis,this._view,t)}}_clear_state(){let t=!(arguments.length>0&&void 0!==arguments[0])||arguments[0];this._task&&this._task.cancel();const e=[];if(this._view){const t=this._view;this._view=void 0,e.push(t.delete()),t.remove_update(this._view_updater),t.remove_delete()}if(this._table&&t){const t=this._table;this._table=void 0,t._owner_viewer&&t._owner_viewer===this&&e.push(t.delete())}return Promise.all(e)}_set_updating(){let t;return this.setAttribute("updating",!0),this._updating_promise=new Promise(e=>{t=e}),t}_register_debounce_instance(){var t=this;const e=i()((t,e,n)=>{this._new_view({ignore_size_check:e,force_update:n}).then(t)},0);this._debounce_update=function(){var n,r,i,o,a,s,A=arguments;return regeneratorRuntime.async(function(l){for(;;)switch(l.prev=l.next){case 0:if(n=A.length>0&&void 0!==A[0]?A[0]:{},r=n.ignore_size_check,i=void 0!==r&&r,o=n.force_update,a=void 0!==o&&o,!t._table){l.next=6;break}return s=t._set_updating(),l.next=5,regeneratorRuntime.awrap(new Promise(t=>e(t,i,a)));case 5:s();case 6:case"end":return l.stop()}})}}_get_worker(){return this._table?this._table._worker:S.a.shared_worker()}}class rt extends nt{_clear_columns(){this._inactive_columns.innerHTML="",this._active_columns.innerHTML=""}set_aggregate_attribute(t){this.setAttribute("aggregates",JSON.stringify(t.reduce((t,e)=>(t[e.column]=e.op,t),{})))}_new_row(t,e,n,r,i,o){let a=document.createElement("perspective-row");if(!e){let n=this._get_view_dom_columns("#inactive_columns perspective-row");e=n.length>0?(e=n.find(e=>e.getAttribute("name")===t))?e.getAttribute("type"):"integer":""}if(!n){let r=this.get_aggregate_attribute();n=r&&(n=r.find(e=>e.column===t))?n.op:Object(L.get_type_config)(e).aggregate}if(r&&(a.setAttribute("filter",r),"string"===e)){const e=this._table.view({row_pivots:[t],aggregates:{}});e.to_json().then(t=>{a.choices(t.slice(1,t.length).map(t=>t.__ROW_PATH__))}),e.delete()}return i?a.setAttribute("sort-order",i):this._get_view_column_pivots().indexOf(t)>-1?a.setAttribute("sort-order","col asc"):a.setAttribute("sort-order","asc"),a.setAttribute("type",e),a.setAttribute("name",t),a.setAttribute("aggregate",n),a.addEventListener("visibility-clicked",this._column_visibility_clicked.bind(this)),a.addEventListener("aggregate-selected",this._column_aggregate_clicked.bind(this)),a.addEventListener("filter-selected",this._column_filter_clicked.bind(this)),a.addEventListener("close-clicked",t=>y.call(this,t.detail)),a.addEventListener("sort-order",this._sort_order_clicked.bind(this)),a.addEventListener("row-drag",()=>{this.classList.add("dragging"),this._original_index=Array.prototype.slice.call(this._active_columns.children).findIndex(e=>e.getAttribute("name")===t),-1!==this._original_index?(this._drop_target_hover=this._active_columns.children[this._original_index],setTimeout(()=>a.setAttribute("drop-target",!0))):this._drop_target_hover=this._new_row(t,e,n)}),a.addEventListener("row-dragend",()=>this.classList.remove("dragging")),o&&(a.setAttribute("computed_column",JSON.stringify(o)),a.classList.add("computed")),a}_update_column_view(t){let e=arguments.length>1&&void 0!==arguments[1]&&arguments[1];t||(t=this._get_view_columns()),this.setAttribute("columns",JSON.stringify(t));const n=this._get_view_dom_columns("#inactive_columns perspective-row");t.length===n.length?this._inactive_columns.parentElement.classList.add("collapse"):this._inactive_columns.parentElement.classList.remove("collapse"),n.forEach(e=>{-1===t.indexOf(e.getAttribute("name"))?e.classList.remove("active"):e.classList.add("active")}),e&&this._update_column_list(t,this._active_columns,t=>{const e=n.find(e=>e.getAttribute("name")===t);if(e)return this._new_row(e.getAttribute("name"),e.getAttribute("type"))})}_update_column_list(t,e,n,r){r=r||((t,e)=>e.getAttribute("name")===t);const i=Array.prototype.slice.call(e.children);for(let o=0,a=0;o<i.length||a<t.length;o++,a++){const s=t[a],A=i[o],l=i[o+1];if(A){if(void 0===s)e.removeChild(A);else if(!r(s,A))if(l&&r(s,l))e.removeChild(A),o++;else{const t=n(s);t&&(e.insertBefore(t,A),o--)}}else{const t=n(s);t&&e.appendChild(t)}}}_set_row_styles(){let t="";if(this._plugin.initial&&this._plugin.initial.names)for(const e in this._plugin.initial.names){const n=this._plugin.initial.names[e];t+="#active_columns perspective-row:nth-child(".concat(parseInt(e)+1,"){margin-top:23px;}"),t+="#active_columns perspective-row:nth-child(".concat(parseInt(e)+1,'):before{content:"').concat(n,'";}')}this.shadowRoot.querySelector("#psp_styles").innerHTML=t}_show_column_selectors(){this.shadowRoot.querySelector("#columns_container").style.visibility="visible",this.shadowRoot.querySelector("#side_panel__actions").style.visibility="visible"}_set_column_defaults(){let t=this._get_view_dom_columns("#inactive_columns perspective-row"),e=this._get_view_dom_columns();if(t.length>0)if(this._plugin.initial){let n=[],r=this._plugin.initial.count||2;e.length===r?n=e.map(t=>t.getAttribute("name")):e.length<r?(n=e.map(t=>t.getAttribute("name")),this._fill_numeric(t,n),n.length<r&&this._fill_numeric(t,n,!0)):"number"===this._plugin.initial.type&&(this._fill_numeric(e,n),n.length<r&&this._fill_numeric(t,n),n.length<r&&this._fill_numeric(t,n,!0)),this.setAttribute("columns",JSON.stringify(n.slice(0,r)))}else"select"===this._plugin.selectMode&&this.setAttribute("columns",JSON.stringify([t[0].getAttribute("name")]))}_fill_numeric(t,e){let n=arguments.length>2&&void 0!==arguments[2]&&arguments[2];var r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done);r=!0){let t=a.value,r=t.getAttribute("type"),i=t.getAttribute("name");(n||["float","integer"].indexOf(r)>-1&&-1===e.indexOf(i))&&e.push(i)}}catch(t){i=!0,o=t}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}}_check_responsive_layout(){this.clientHeight<500&&this.clientWidth>600&&this._get_view_columns({active:!1}).length>this._get_view_columns().length?this.shadowRoot.querySelector("#app").classList.add("columns_horizontal"):this.shadowRoot.querySelector("#app").classList.remove("columns_horizontal")}_register_ids(){this._aggregate_selector=this.shadowRoot.querySelector("#aggregate_selector"),this._vis_selector=this.shadowRoot.querySelector("#vis_selector"),this._filters=this.shadowRoot.querySelector("#filters"),this._row_pivots=this.shadowRoot.querySelector("#row_pivots"),this._column_pivots=this.shadowRoot.querySelector("#column_pivots"),this._datavis=this.shadowRoot.querySelector("#pivot_chart"),this._active_columns=this.shadowRoot.querySelector("#active_columns"),this._inactive_columns=this.shadowRoot.querySelector("#inactive_columns"),this._side_panel_actions=this.shadowRoot.querySelector("#side_panel__actions"),this._add_computed_column=this.shadowRoot.querySelector("#add-computed-column"),this._computed_column=this.shadowRoot.querySelector("perspective-computed-column"),this._computed_column_inputs=this._computed_column.querySelector("#psp-cc-computation-inputs"),this._inner_drop_target=this.shadowRoot.querySelector("#drop_target_inner"),this._drop_target=this.shadowRoot.querySelector("#drop_target"),this._config_button=this.shadowRoot.querySelector("#config_button"),this._reset_button=this.shadowRoot.querySelector("#reset_button"),this._download_button=this.shadowRoot.querySelector("#download_button"),this._copy_button=this.shadowRoot.querySelector("#copy_button"),this._side_panel=this.shadowRoot.querySelector("#side_panel"),this._top_panel=this.shadowRoot.querySelector("#top_panel"),this._sort=this.shadowRoot.querySelector("#sort"),this._transpose_button=this.shadowRoot.querySelector("#transpose_button"),this._plugin_information=this.shadowRoot.querySelector(".plugin_information"),this._plugin_information_action=this.shadowRoot.querySelector(".plugin_information__action"),this._plugin_information_dismiss=this.shadowRoot.querySelector(".plugin_information__action--dismiss"),this._plugin_information_message=this.shadowRoot.querySelector("#plugin_information_count"),this._resize_bar=this.shadowRoot.querySelector("#resize_bar")}_register_view_options(){let t=c.b.getInstance();for(let e in t){const n=t[e].name||e,r='<option value="'.concat(e,'">').concat(n,"</option>");this._vis_selector.innerHTML+=r}}_register_data_attribute(){if(this.getAttribute("data")){let t=this.getAttribute("data");try{t=JSON.parse(t)}catch(t){}this.load(t)}}}class it extends rt{_show_context_menu(t){return this.shadowRoot.querySelector("#app").classList.toggle("show_menu"),t.stopPropagation(),t.preventDefault(),!1}_hide_context_menu(){this.shadowRoot.querySelector("#app").classList.remove("show_menu")}_toggle_config(t){t&&2===t.button||(this._show_config?(this._side_panel.style.display="none",this._top_panel.style.display="none",this.removeAttribute("settings")):(this._side_panel.style.display="flex",this._top_panel.style.display="flex",this.setAttribute("settings",!0)),this._show_config=!this._show_config,this._plugin.resize.call(this,!0),this._hide_context_menu(),this.dispatchEvent(new CustomEvent("perspective-toggle-settings",{detail:this._show_config})))}_open_computed_column(t){t.stopImmediatePropagation(),this._computed_column.style.display="flex",this._side_panel_actions.style.display="none"}_set_computed_column_input(t){t.detail.target.appendChild(this._new_row(t.detail.column.name,t.detail.column.type)),this._update_column_view()}_validate_computed_column(t){const e=t.detail;let n=JSON.parse(this.getAttribute("computed-columns"));null===n&&(n=[]);var r=!0,i=!1,o=void 0;try{for(var a,s=n[Symbol.iterator]();!(r=(a=s.next()).done);r=!0){let t=a.value;if(e.name===t.name)return}}catch(t){i=!0,o=t}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}n.push(e),this.setAttribute("computed-columns",JSON.stringify(n))}_create_computed_column(t){var e,n,r,i,o=this;return regeneratorRuntime.async(function(a){for(;;)switch(a.prev=a.next){case 0:return e=t.detail,n=e.column_name,a.next=4,regeneratorRuntime.awrap(o._table.columns());case 4:return a.sent.includes(n)&&(console.log(n),n+=" ".concat(Math.round(100*Math.random()))),r=[{computation:e.computation,column:n,func:e.computation.func,inputs:e.input_columns.map(t=>t.name),input_type:e.computation.input_type,type:e.computation.return_type}],i=o._table.add_computed(r),a.next=10,regeneratorRuntime.awrap(o._load_table(i,!0));case 10:o._update_column_view();case 11:case"end":return a.stop()}})}_column_visibility_clicked(t){let e=t.currentTarget;if("active_columns"===e.parentElement.getAttribute("id")){if(1===this._get_visible_column_count())return;if(t.detail.shiftKey){var n=!0,r=!1,i=void 0;try{for(var o,a=Array.prototype.slice.call(this._active_columns.children)[Symbol.iterator]();!(n=(o=a.next()).done);n=!0){let t=o.value;t!==e&&this._active_columns.removeChild(t)}}catch(t){r=!0,i=t}finally{try{n||null==a.return||a.return()}finally{if(r)throw i}}}else this._active_columns.removeChild(e)}else{if(t.path&&t.path[1].classList.contains("psp-cc-computation__input-column"))return this._computed_column.deselect_column(t.currentTarget.getAttribute("name")),void this._update_column_view();if(t.detail.shiftKey&&"toggle"===this._plugin.selectMode||!t.detail.shiftKey&&"select"===this._plugin.selectMode){var s=!0,A=!1,l=void 0;try{for(var c,u=Array.prototype.slice.call(this._active_columns.children)[Symbol.iterator]();!(s=(c=u.next()).done);s=!0){let t=c.value;this._active_columns.removeChild(t)}}catch(t){A=!0,l=t}finally{try{s||null==u.return||u.return()}finally{if(A)throw l}}}let n=this._new_row(e.getAttribute("name"),e.getAttribute("type"));this._active_columns.appendChild(n)}this._check_responsive_layout();let g=this._get_view_columns();this._update_column_view(g)}_column_aggregate_clicked(){let t=this.get_aggregate_attribute(),e=this._get_view_aggregates();var n=!0,r=!1,i=void 0;try{for(var o,a=t[Symbol.iterator]();!(n=(o=a.next()).done);n=!0){let t=o.value,n=e.find(e=>e.column===t.column);n&&(t.op=n.op)}}catch(t){r=!0,i=t}finally{try{n||null==a.return||a.return()}finally{if(r)throw i}}this.set_aggregate_attribute(t),this._update_column_view(),this._debounce_update()}_column_filter_clicked(){let t=this._get_view_filters();this._updating_filter=!0,this.setAttribute("filters",JSON.stringify(t)),this._updating_filter=!1,this._debounce_update()}_increment_sort(t,e,n){let r=["asc","desc"];return e&&r.push("col asc","col desc"),n&&(r=r.map(t=>"".concat(t," abs"))),r.push("none"),r[(r.indexOf(t)+1)%r.length]}_sort_order_clicked(t){const e=t.target,n=t.detail.shiftKey&&"string"!==e.getAttribute("type"),r=this._increment_sort(e.getAttribute("sort-order"),this._get_view_column_pivots().length>0,n);e.setAttribute("sort-order",r);let i=JSON.parse(this.getAttribute("sort")),o=this._get_view_sorts();var a=!0,s=!1,A=void 0;try{for(var l,c=i[Symbol.iterator]();!(a=(l=c.next()).done);a=!0){let t=l.value,e=o.find(e=>e[0]===t[0]);e&&(t[1]=e[1])}}catch(t){s=!0,A=t}finally{try{a||null==c.return||c.return()}finally{if(s)throw A}}this.setAttribute("sort",JSON.stringify(i))}_transpose(){let t=this.getAttribute("row-pivots");this.setAttribute("row-pivots",this.getAttribute("column-pivots")),this.setAttribute("column-pivots",t)}_resize_sidepanel(t){const e=document.body.style.cursor;document.body.style.cursor="col-resize";const n=t.clientX,r=this._side_panel.offsetWidth,i=t=>{const e=Math.max(0,Math.min(r+(t.clientX-n),this.offsetWidth-10));this._side_panel.style.width="".concat(e,"px")},o=()=>{document.body.style.cursor=e,document.removeEventListener("mousemove",i),document.removeEventListener("mouseup",o)};document.addEventListener("mousemove",i),document.addEventListener("mouseup",o)}_register_callbacks(){this._sort.addEventListener("drop",E.bind(this)),this._sort.addEventListener("dragend",y.bind(this)),this._sort.addEventListener("dragenter",v.bind(this)),this._sort.addEventListener("dragover",B.bind(this)),this._sort.addEventListener("dragleave",b.bind(this)),this._row_pivots.addEventListener("drop",E.bind(this)),this._row_pivots.addEventListener("dragend",y.bind(this)),this._row_pivots.addEventListener("dragenter",v.bind(this)),this._row_pivots.addEventListener("dragover",B.bind(this)),this._row_pivots.addEventListener("dragleave",b.bind(this)),this._column_pivots.addEventListener("drop",E.bind(this)),this._column_pivots.addEventListener("dragend",y.bind(this)),this._column_pivots.addEventListener("dragenter",v.bind(this)),this._column_pivots.addEventListener("dragover",B.bind(this)),this._column_pivots.addEventListener("dragleave",b.bind(this)),this._filters.addEventListener("drop",E.bind(this)),this._filters.addEventListener("dragend",y.bind(this)),this._filters.addEventListener("dragenter",v.bind(this)),this._filters.addEventListener("dragover",B.bind(this)),this._filters.addEventListener("dragleave",b.bind(this)),this._active_columns.addEventListener("drop",function(t){t.preventDefault(),t.currentTarget.classList.remove("dropping"),this._drop_target_hover.parentElement===this._active_columns&&this._drop_target_hover.removeAttribute("drop-target"),m.bind(this)(),t.dataTransfer.getData("text")&&this._update_column_view()}.bind(this)),this._active_columns.addEventListener("dragenter",v.bind(this)),this._active_columns.addEventListener("dragend",function(t){let e=t.target.parentElement.parentElement;m.bind(this)(),this._get_visible_column_count()>1&&"move"!==t.dataTransfer.dropEffect&&(this._active_columns.removeChild(e),this._update_column_view()),this._active_columns.classList.remove("dropping")}.bind(this)),this._active_columns.addEventListener("dragover",function(t){t.preventDefault(),t.dataTransfer.dropEffect="move","dropping"!==t.currentTarget.className&&t.currentTarget.classList.add("dropping"),this._drop_target_hover.hasAttribute("drop-target")||this._drop_target_hover.setAttribute("drop-target",!0);let e=function(t){if(0==this._active_columns.children.length)return 0;for(let e in this._active_columns.children){let n=this._active_columns.children[e];if(n.offsetTop+n.offsetHeight>t.offsetY+this._active_columns.scrollTop)return parseInt(e)}return this._active_columns.children.length}.call(this,t);Array.prototype.slice.call(this._active_columns.children).indexOf(this._drop_target_hover)<e&&(e+=1),e<this._active_columns.children.length?this._active_columns.children[e].hasAttribute("drop-target")||this._active_columns.insertBefore(this._drop_target_hover,this._active_columns.children[e]):this._active_columns.children[this._active_columns.children.length-1].hasAttribute("drop-target")||this._active_columns.appendChild(this._drop_target_hover)}.bind(this)),this._active_columns.addEventListener("dragleave",function(t){let e=t.relatedTarget;for(;e&&e!==this._active_columns;)e=e.parentElement;null===e&&(this._active_columns.classList.remove("dropping"),this._drop_target_hover.parentElement===this._active_columns&&this._active_columns.removeChild(this._drop_target_hover),-1!==this._original_index&&this._active_columns.insertBefore(this._drop_target_hover,this._active_columns.children[this._original_index]),this._drop_target_hover.removeAttribute("drop-target"))}.bind(this)),this._add_computed_column.addEventListener("click",this._open_computed_column.bind(this)),this._computed_column.addEventListener("perspective-computed-column-save",this._validate_computed_column.bind(this)),this._computed_column.addEventListener("perspective-computed-column-update",this._set_computed_column_input.bind(this)),this._config_button.addEventListener("mousedown",this._toggle_config.bind(this)),this._config_button.addEventListener("contextmenu",this._show_context_menu.bind(this)),this._reset_button.addEventListener("click",this.reset.bind(this)),this._copy_button.addEventListener("click",t=>this.copy(t.shiftKey)),this._download_button.addEventListener("click",t=>this.download(t.shiftKey)),this._transpose_button.addEventListener("click",this._transpose.bind(this)),this._drop_target.addEventListener("dragover",B.bind(this)),this._resize_bar.addEventListener("mousedown",this._resize_sidepanel.bind(this)),this._vis_selector.addEventListener("change",()=>{this.setAttribute("plugin",this._vis_selector.value),this._debounce_update()}),this._plugin_information_action.addEventListener("click",()=>{this._debounce_update({ignore_size_check:!0}),this._plugin_information.classList.add("hidden")}),this._plugin_information_dismiss.addEventListener("click",()=>{this._debounce_update({ignore_size_check:!0}),this._plugin_information.classList.add("hidden"),this._show_warnings=!1})}}var ot;function at(t,e,n,r,i){var o={};return Object.keys(r).forEach(function(t){o[t]=r[t]}),o.enumerable=!!o.enumerable,o.configurable=!!o.configurable,("value"in o||o.initializer)&&(o.writable=!0),o=n.slice().reverse().reduce(function(n,r){return r(t,e,n)||n},o),i&&void 0!==o.initializer&&(o.value=o.initializer?o.initializer.call(i):void 0,o.initializer=void 0),void 0===o.initializer&&(Object.defineProperty(t,e,o),o=null),o}Object(o.polyfill)({}),a(H.a,Y.a,K.a)((at((ot=class extends it{constructor(){super(),this._register_debounce_instance(),this._show_config=!0,this._show_warnings=!0,this._resize_handler=i()(this.notifyResize,250).bind(this),window.addEventListener("load",this._resize_handler),window.addEventListener("resize",this._resize_handler)}connectedCallback(){0===Object.keys(c.b.getInstance()).length&&Object(c.a)(),this.setAttribute("settings",!0),this._register_ids(),this._register_callbacks(),this._register_view_options(),this._register_data_attribute(),this.toggleConfig();for(var t=0,e=["row-pivots","column-pivots","filters","sort"];t<e.length;t++){let n=e[t];this.hasAttribute(n)||this.setAttribute(n,"[]")}}sort(t){var e=this._sort.querySelector("ul");this._update_column_list(t,e,t=>{let e="asc";return Array.isArray(t)&&(e=t[1],t=t[0]),this._new_row(t,!1,!1,!1,e)},(t,e)=>Array.isArray(t)?e.getAttribute("name")===t[0]&&e.getAttribute("sort-order")===t[1]:e.getAttribute("name")===t),this.dispatchEvent(new Event("perspective-config-update")),this._debounce_update()}columns(t){this._update_column_view(t,!0),this.dispatchEvent(new Event("perspective-config-update")),this._debounce_update()}"computed-columns"(t){const e=this._set_updating();this._computed_column._close_computed_column(),(()=>{var n,r,i,o,a,s;regeneratorRuntime.async(function(A){for(;;)switch(A.prev=A.next){case 0:if(!this._table){A.next=30;break}n=!0,r=!1,i=void 0,A.prev=4,o=t[Symbol.iterator]();case 6:if(n=(a=o.next()).done){A.next=13;break}return s=a.value,A.next=10,regeneratorRuntime.awrap(this._create_computed_column({detail:{column_name:s.name,input_columns:s.inputs.map(t=>({name:t})),computation:w[s.func]}}));case 10:n=!0,A.next=6;break;case 13:A.next=19;break;case 15:A.prev=15,A.t0=A.catch(4),r=!0,i=A.t0;case 19:A.prev=19,A.prev=20,n||null==o.return||o.return();case 22:if(A.prev=22,!r){A.next=25;break}throw i;case 25:return A.finish(22);case 26:return A.finish(19);case 27:return A.next=29,regeneratorRuntime.awrap(this._debounce_update());case 29:e();case 30:this.dispatchEvent(new Event("perspective-config-update")),this.dispatchEvent(new Event("perspective-computed-column-update"));case 32:case"end":return A.stop()}},null,this,[[4,15,19,27],[20,,22,26]])})()}aggregates(t){this._get_view_dom_columns().map(e=>{let n=t[e.getAttribute("name")];n&&e.setAttribute("aggregate",n)}),this.dispatchEvent(new Event("perspective-config-update")),this._debounce_update()}filters(t){if(!this._updating_filter&&void 0!==this._table){var e=this._filters.querySelector("ul");this._update_column_list(t,e,t=>{const e=JSON.stringify({operator:t[1],operand:t[2]});return this._new_row(t[0],void 0,void 0,e)},(t,e)=>e.getAttribute("name")===t[0]&&e.getAttribute("filter")===JSON.stringify({operator:t[1],operand:t[2]}))}this.dispatchEvent(new Event("perspective-config-update")),this._debounce_update()}set plugin(t){const e=Object.keys(c.b.getInstance());let n=this.getAttribute("plugin");if(-1===e.indexOf(n)){const t=e.find(t=>t.indexOf(n)>-1);t?(console.warn('Unknown plugin "'.concat(n,'", using "').concat(t,'"')),this.setAttribute("plugin",t)):console.error('Unknown plugin "'.concat(n,'"'))}else this._vis_selector.value=n,this._set_row_styles(),this._set_column_defaults(),this.dispatchEvent(new Event("perspective-config-update"))}"column-pivots"(t){var e=this._column_pivots.querySelector("ul");this._update_column_list(t,e,t=>this._new_row(t)),this.dispatchEvent(new Event("perspective-config-update")),this._debounce_update()}"row-pivots"(t){var e=this._row_pivots.querySelector("ul");this._update_column_list(t,e,t=>this._new_row(t)),this.dispatchEvent(new Event("perspective-config-update")),this._debounce_update()}get worker(){return this._get_worker()}get table(){return this._table}get view(){return this._view}load(t,e){try{t=t.trim()}catch(t){}let n;return t.hasOwnProperty("_name")&&"table"===t.type?n=t:(n=this.worker.table(t,e))._owner_viewer=this,this._load_table(n)}update(t){void 0===this._table?this.load(t):this._table.update(t)}notifyResize(){this._check_responsive_layout(),!document.hidden&&this.offsetParent&&this._plugin.resize.call(this)}clone(t){t.hasAttribute("index")&&this.setAttribute("index",t.getAttribute("index")),this._inner_drop_target&&(this._inner_drop_target.innerHTML=t._inner_drop_target.innerHTML),this._load_table(t.table),this.restore(t.save())}delete(){let t=!(arguments.length>0&&void 0!==arguments[0])||arguments[0],e=this._clear_state(t);return this._plugin.delete&&this._plugin.delete.call(this),window.removeEventListener("load",this._resize_handler),window.removeEventListener("resize",this._resize_handler),e}restyleElement(){this._restyle_plugin()}save(){let t={};for(let e=0;e<this.attributes.length;e++){let n=this.attributes[e];-1===["id"].indexOf(n.name)&&(t[n.name]=n.value)}return this._plugin.save&&(t.plugin_config=this._plugin.save.call(this)),t}restore(t){var e,n,r=this;return regeneratorRuntime.async(function(i){for(;;)switch(i.prev=i.next){case 0:for(e in"string"==typeof t&&(t=JSON.parse(t)),t)"string"!=typeof(n=t[e])&&(n=JSON.stringify(n)),r.setAttribute(e,n);return r._plugin.restore&&t.plugin_config&&r._plugin.restore.call(r,t.plugin_config),i.next=5,regeneratorRuntime.awrap(r._debounce_update());case 5:case"end":return i.stop()}})}flush(){var t=this;return regeneratorRuntime.async(function(e){for(;;)switch(e.prev=e.next){case 0:return e.next=2,regeneratorRuntime.awrap(new Promise(setTimeout));case 2:if(!t.hasAttribute("updating")){e.next=7;break}return e.next=5,regeneratorRuntime.awrap(t._updating_promise);case 5:e.next=2;break;case 7:case"end":return e.stop()}})}clear(){this._table.clear()}replace(t){this._table.replace(t)}reset(){this.setAttribute("row-pivots",JSON.stringify([])),this.setAttribute("column-pivots",JSON.stringify([])),this.setAttribute("filters",JSON.stringify([])),this.setAttribute("sort",JSON.stringify([])),this.removeAttribute("index"),this._initial_col_order?this.setAttribute("columns",JSON.stringify(this._initial_col_order||[])):this.removeAttribute("columns"),this.setAttribute("plugin",Object.keys(c.b.getInstance())[0]),this.dispatchEvent(new Event("perspective-config-update")),this._hide_context_menu()}download(){var t,e,n,r,i,o,a,s,A,l=this,c=arguments;return regeneratorRuntime.async(function(u){for(;;)switch(u.prev=u.next){case 0:return t=c.length>0&&void 0!==c[0]&&c[0],e=t?l._table.view():l._view,u.next=4,regeneratorRuntime.awrap(e.to_csv());case 4:for(n=u.sent,r=document.createElement("a"),o=(i=n).length,a=new Uint8Array(o),s=0;s<o;s++)a[s]=i.charCodeAt(s);A=new Blob([a]),r.setAttribute("href",URL.createObjectURL(A)),r.setAttribute("download","perspective.csv"),r.style.display="none",document.body.appendChild(r),r.click(),document.body.removeChild(r),l._hide_context_menu();case 18:case"end":return u.stop()}})}copy(){let t;(arguments.length>0&&void 0!==arguments[0]&&arguments[0]?this._table.view():this._view).to_csv().then(e=>{t=e}).catch(e=>{console.error(e),t=""});let e=0,n=()=>{void 0!==t?function(t){let e=document.createElement("textarea");document.body.appendChild(e),e.value=t,e.select(),document.execCommand("copy"),document.body.removeChild(e)}(t):e<200?(e++,setTimeout(n,50)):console.warn("Timeout expired - copy to clipboard cancelled.")};n(),this._hide_context_menu()}toggleConfig(){this._toggle_config()}}).prototype,"sort",[l],Object.getOwnPropertyDescriptor(ot.prototype,"sort"),ot.prototype),at(ot.prototype,"columns",[l],Object.getOwnPropertyDescriptor(ot.prototype,"columns"),ot.prototype),at(ot.prototype,"computed-columns",[l],Object.getOwnPropertyDescriptor(ot.prototype,"computed-columns"),ot.prototype),at(ot.prototype,"aggregates",[A],Object.getOwnPropertyDescriptor(ot.prototype,"aggregates"),ot.prototype),at(ot.prototype,"filters",[l],Object.getOwnPropertyDescriptor(ot.prototype,"filters"),ot.prototype),at(ot.prototype,"column-pivots",[l],Object.getOwnPropertyDescriptor(ot.prototype,"column-pivots"),ot.prototype),at(ot.prototype,"row-pivots",[l],Object.getOwnPropertyDescriptor(ot.prototype,"row-pivots"),ot.prototype),ot))}]))},function(t,e,n){var r=n(53),i=n(105),o=n(80);t.exports=function(t){var e=r(t),n=i.f;if(n)for(var a,s=n(t),A=o.f,l=0;s.length>l;)A.call(t,a=s[l++])&&e.push(a);return e}},function(t,e,n){var r=n(48);t.exports=Array.isArray||function(t){return"Array"==r(t)}},function(t,e,n){var r=n(30),i=n(106).f,o={}.toString,a="object"==typeof window&&window&&Object.getOwnPropertyNames?Object.getOwnPropertyNames(window):[];t.exports.f=function(t){return a&&"[object Window]"==o.call(t)?function(t){try{return i(t)}catch(t){return a.slice()}}(t):i(r(t))}},function(t,e,n){var r=n(55),i=n(189),o=n(190),a=Math.max,s=Math.min;t.exports=function(t,e,n){var A,l,c,u,g,h,d=0,f=!1,p=!1,I=!0;if("function"!=typeof t)throw new TypeError("Expected a function");function C(e){var n=A,r=l;return A=l=void 0,d=e,u=t.apply(r,n)}function m(t){var n=t-h;return void 0===h||n>=e||n<0||p&&t-d>=c}function y(){var t=i();if(m(t))return E(t);g=setTimeout(y,function(t){var n=e-(t-h);return p?s(n,c-(t-d)):n}(t))}function E(t){return g=void 0,I&&A?C(t):(A=l=void 0,u)}function v(){var t=i(),n=m(t);if(A=arguments,l=this,h=t,n){if(void 0===g)return function(t){return d=t,g=setTimeout(y,e),f?C(t):u}(h);if(p)return g=setTimeout(y,e),C(h)}return void 0===g&&(g=setTimeout(y,e)),u}return e=o(e)||0,r(n)&&(f=!!n.leading,c=(p="maxWait"in n)?a(o(n.maxWait)||0,e):c,I="trailing"in n?!!n.trailing:I),v.cancel=function(){void 0!==g&&clearTimeout(g),d=0,A=h=l=g=void 0},v.flush=function(){return void 0===g?u:E(i())},v}},function(t,e,n){var r=n(18);t.exports=function(){return r.Date.now()}},function(t,e,n){var r=n(55),i=n(191),o=/^\s+|\s+$/g,a=/^[-+]0x[0-9a-f]+$/i,s=/^0b[01]+$/i,A=/^0o[0-7]+$/i,l=parseInt;t.exports=function(t){if("number"==typeof t)return t;if(i(t))return NaN;if(r(t)){var e="function"==typeof t.valueOf?t.valueOf():t;t=r(e)?e+"":e}if("string"!=typeof t)return 0===t?t:+t;t=t.replace(o,"");var n=s.test(t);return n||A.test(t)?l(t.slice(2),n?2:8):a.test(t)?NaN:+t}},function(t,e,n){var r=n(39),i=n(40);t.exports=function(t){return"symbol"==typeof t||i(t)&&"[object Symbol]"==r(t)}},function(t,e,n){var r=n(81),i=Object.prototype,o=i.hasOwnProperty,a=i.toString,s=r?r.toStringTag:void 0;t.exports=function(t){var e=o.call(t,s),n=t[s];try{t[s]=void 0;var r=!0}catch(t){}var i=a.call(t);return r&&(e?t[s]=n:delete t[s]),i}},function(t,e){var n=Object.prototype.toString;t.exports=function(t){return n.call(t)}},function(t,e,n){!function(){var e=function(t,n){var r=this;e.count=(e.count||0)+1,this.count=e.count,this.isOpened=!1,this.input=i(t),this.input.setAttribute("autocomplete","off"),this.input.setAttribute("aria-owns","awesomplete_list_"+this.count),this.input.setAttribute("role","combobox"),this.options=n=n||{},function(t,e,n){for(var r in e){var i=e[r],o=t.input.getAttribute("data-"+r.toLowerCase());"number"==typeof i?t[r]=parseInt(o):!1===i?t[r]=null!==o:i instanceof Function?t[r]=null:t[r]=o,t[r]||0===t[r]||(t[r]=r in n?n[r]:i)}}(this,{minChars:2,maxItems:10,autoFirst:!1,data:e.DATA,filter:e.FILTER_CONTAINS,sort:!1!==n.sort&&e.SORT_BYLENGTH,container:e.CONTAINER,item:e.ITEM,replace:e.REPLACE,tabSelect:!1},n),this.index=-1,this.container=this.container(t),this.ul=i.create("ul",{hidden:"hidden",role:"listbox",id:"awesomplete_list_"+this.count,inside:this.container}),this.status=i.create("span",{className:"visually-hidden",role:"status","aria-live":"assertive","aria-atomic":!0,inside:this.container,textContent:0!=this.minChars?"Type "+this.minChars+" or more characters for results.":"Begin typing for results."}),this._events={input:{input:this.evaluate.bind(this),blur:this.close.bind(this,{reason:"blur"}),keydown:function(t){var e=t.keyCode;r.opened&&(13===e&&r.selected?(t.preventDefault(),r.select()):9===e&&r.selected&&r.tabSelect?r.select():27===e?r.close({reason:"esc"}):38!==e&&40!==e||(t.preventDefault(),r[38===e?"previous":"next"]()))}},form:{submit:this.close.bind(this,{reason:"submit"})},ul:{mousedown:function(t){t.preventDefault()},click:function(t){var e=t.target;if(e!==this){for(;e&&!/li/i.test(e.nodeName);)e=e.parentNode;e&&0===t.button&&(t.preventDefault(),r.select(e,t.target))}}}},i.bind(this.input,this._events.input),i.bind(this.input.form,this._events.form),i.bind(this.ul,this._events.ul),this.input.hasAttribute("list")?(this.list="#"+this.input.getAttribute("list"),this.input.removeAttribute("list")):this.list=this.input.getAttribute("data-list")||n.list||[],e.all.push(this)};function n(t){var e=Array.isArray(t)?{label:t[0],value:t[1]}:"object"==typeof t&&"label"in t&&"value"in t?t:{label:t,value:t};this.label=e.label||e.value,this.value=e.value}e.prototype={set list(t){if(Array.isArray(t))this._list=t;else if("string"==typeof t&&t.indexOf(",")>-1)this._list=t.split(/\s*,\s*/);else if((t=i(t))&&t.children){var e=[];r.apply(t.children).forEach(function(t){if(!t.disabled){var n=t.textContent.trim(),r=t.value||n,i=t.label||n;""!==r&&e.push({label:i,value:r})}}),this._list=e}document.activeElement===this.input&&this.evaluate()},get selected(){return this.index>-1},get opened(){return this.isOpened},close:function(t){this.opened&&(this.ul.setAttribute("hidden",""),this.isOpened=!1,this.index=-1,this.status.setAttribute("hidden",""),i.fire(this.input,"awesomplete-close",t||{}))},open:function(){this.ul.removeAttribute("hidden"),this.isOpened=!0,this.status.removeAttribute("hidden"),this.autoFirst&&-1===this.index&&this.goto(0),i.fire(this.input,"awesomplete-open")},destroy:function(){if(i.unbind(this.input,this._events.input),i.unbind(this.input.form,this._events.form),!this.options.container){var t=this.container.parentNode;t.insertBefore(this.input,this.container),t.removeChild(this.container)}this.input.removeAttribute("autocomplete"),this.input.removeAttribute("aria-autocomplete");var n=e.all.indexOf(this);-1!==n&&e.all.splice(n,1)},next:function(){var t=this.ul.children.length;this.goto(this.index<t-1?this.index+1:t?0:-1)},previous:function(){var t=this.ul.children.length,e=this.index-1;this.goto(this.selected&&-1!==e?e:t-1)},goto:function(t){var e=this.ul.children;this.selected&&e[this.index].setAttribute("aria-selected","false"),this.index=t,t>-1&&e.length>0&&(e[t].setAttribute("aria-selected","true"),this.status.textContent=e[t].textContent+", list item "+(t+1)+" of "+e.length,this.input.setAttribute("aria-activedescendant",this.ul.id+"_item_"+this.index),this.ul.scrollTop=e[t].offsetTop-this.ul.clientHeight+e[t].clientHeight,i.fire(this.input,"awesomplete-highlight",{text:this.suggestions[this.index]}))},select:function(t,e){if(t?this.index=i.siblingIndex(t):t=this.ul.children[this.index],t){var n=this.suggestions[this.index];i.fire(this.input,"awesomplete-select",{text:n,origin:e||t})&&(this.replace(n),this.close({reason:"select"}),i.fire(this.input,"awesomplete-selectcomplete",{text:n}))}},evaluate:function(){var t=this,e=this.input.value;e.length>=this.minChars&&this._list&&this._list.length>0?(this.index=-1,this.ul.innerHTML="",this.suggestions=this._list.map(function(r){return new n(t.data(r,e))}).filter(function(n){return t.filter(n,e)}),!1!==this.sort&&(this.suggestions=this.suggestions.sort(this.sort)),this.suggestions=this.suggestions.slice(0,this.maxItems),this.suggestions.forEach(function(n,r){t.ul.appendChild(t.item(n,e,r))}),0===this.ul.children.length?(this.status.textContent="No results found",this.close({reason:"nomatches"})):(this.open(),this.status.textContent=this.ul.children.length+" results found")):(this.close({reason:"nomatches"}),this.status.textContent="No results found")}},e.all=[],e.FILTER_CONTAINS=function(t,e){return RegExp(i.regExpEscape(e.trim()),"i").test(t)},e.FILTER_STARTSWITH=function(t,e){return RegExp("^"+i.regExpEscape(e.trim()),"i").test(t)},e.SORT_BYLENGTH=function(t,e){return t.length!==e.length?t.length-e.length:t<e?-1:1},e.CONTAINER=function(t){return i.create("div",{className:"awesomplete",around:t})},e.ITEM=function(t,e,n){var r=""===e.trim()?t:t.replace(RegExp(i.regExpEscape(e.trim()),"gi"),"<mark>$&</mark>");return i.create("li",{innerHTML:r,"aria-selected":"false",id:"awesomplete_list_"+this.count+"_item_"+n})},e.REPLACE=function(t){this.input.value=t.value},e.DATA=function(t){return t},Object.defineProperty(n.prototype=Object.create(String.prototype),"length",{get:function(){return this.label.length}}),n.prototype.toString=n.prototype.valueOf=function(){return""+this.label};var r=Array.prototype.slice;function i(t,e){return"string"==typeof t?(e||document).querySelector(t):t||null}function o(t,e){return r.call((e||document).querySelectorAll(t))}function a(){o("input.awesomplete").forEach(function(t){new e(t)})}i.create=function(t,e){var n=document.createElement(t);for(var r in e){var o=e[r];if("inside"===r)i(o).appendChild(n);else if("around"===r){var a=i(o);a.parentNode.insertBefore(n,a),n.appendChild(a),null!=a.getAttribute("autofocus")&&a.focus()}else r in n?n[r]=o:n.setAttribute(r,o)}return n},i.bind=function(t,e){if(t)for(var n in e){var r=e[n];n.split(/\s+/).forEach(function(e){t.addEventListener(e,r)})}},i.unbind=function(t,e){if(t)for(var n in e){var r=e[n];n.split(/\s+/).forEach(function(e){t.removeEventListener(e,r)})}},i.fire=function(t,e,n){var r=document.createEvent("HTMLEvents");for(var i in r.initEvent(e,!0,!0),n)r[i]=n[i];return t.dispatchEvent(r)},i.regExpEscape=function(t){return t.replace(/[-\\^$*+?.()|[\]{}]/g,"\\$&")},i.siblingIndex=function(t){for(var e=0;t=t.previousElementSibling;e++);return e},"undefined"!=typeof self&&(self.Awesomplete=e),"undefined"!=typeof Document&&("loading"!==document.readyState?a():document.addEventListener("DOMContentLoaded",a)),e.$=i,e.$$=o,t.exports&&(t.exports=e)}()},function(t,e,n){
/*! mobile-drag-drop 2.1.0 | Copyright (c) 2017 Tim Ruffles | MIT License */
!function(t){"use strict";function e(t){if(!p){var e=function(t){var e=t.target;do{if(!1!==e.draggable&&e.getAttribute&&"true"===e.getAttribute("draggable"))return e}while((e=e.parentNode)&&e!==document.body)}(t);if(e)try{p=new w(t,C,e,n)}catch(e){throw n(C,t,3),e}}}function n(t,e,n){if(0===n&&t.defaultActionOverride)try{t.defaultActionOverride(e),e.defaultPrevented}catch(t){}p=null}function r(t,e,n){void 0===n&&(n=!0),document.addEventListener(t,e,!!f&&{passive:n})}function i(t,e){document.removeEventListener(t,e)}function o(t){return 0===t.length?0:t.reduce(function(t,e){return e+t},0)/t.length}function a(t){return t&&t.tagName}function s(t,e){for(var n=0;n<t.changedTouches.length;n++){if(t.changedTouches[n].identifier===e)return!0}return!1}function A(t,e,n){for(var r=[],i=[],a=0;a<e.touches.length;a++){var s=e.touches[a];r.push(s[t+"X"]),i.push(s[t+"Y"])}n.x=o(r),n.y=o(i)}function l(t,e,n,r,i){void 0===i&&(i=!0);var o=e.x,a=e.y;r&&(o+=r.x,a+=r.y),i&&(o-=parseInt(t.offsetWidth,10)/2,a-=parseInt(t.offsetHeight,10)/2);for(var s="translate3d("+o+"px,"+a+"px, 0)",A=0;A<E.length;A++){var l=E[A]+"transform";t.style[l]=s+" "+n[A]}}function c(t,e){return t?t===m[0]?y[0]:0===t.indexOf(m[1])||t===m[7]?y[1]:0===t.indexOf(m[4])?y[3]:t===m[6]?y[2]:y[1]:3===e.nodeType&&"A"===e.tagName?y[3]:y[1]}function u(t,e,n,r,i,o,a){if(void 0===o&&(o=!0),void 0===a&&(a=null),I){var s=v+"debug",A=v+"event-target",l=v+"event-related-target";e.classList.add(s),e.classList.add(A),a&&(a.classList.add(s),a.classList.add(l))}var c=function(t,e,n,r,i,o,a){void 0===a&&(a=null);var s=e.changedTouches[0],A=new Event(n,{bubbles:!0,cancelable:r});A.dataTransfer=o,A.relatedTarget=a,A.screenX=s.screenX,A.screenY=s.screenY,A.clientX=s.clientX,A.clientY=s.clientY,A.pageX=s.pageX,A.pageY=s.pageY;var l=t.getBoundingClientRect();return A.offsetX=A.clientX-l.left,A.offsetY=A.clientY-l.top,A}(e,n,t,o,document.defaultView,i,a),u=!e.dispatchEvent(c);return r.g=0,I&&(e.classList.remove(A),a&&a.classList.remove(l)),u}function g(t,e){if(!t||t===m[7])return e;if(e===y[1]){if(0===t.indexOf(y[1]))return y[1]}else if(e===y[3]){if(0===t.indexOf(y[3])||t.indexOf("Link")>-1)return y[3]}else if(e===y[2]&&(0===t.indexOf(y[2])||t.indexOf("Move")>-1))return y[2];return y[0]}function h(t){var n=t.target,r=function(){o.off(),a.off(),s.off(),clearTimeout(i)},i=setTimeout(function(){o.off(),a.off(),s.off(),e(t)},C.holdToDrag),o=d(n,"touchend",r,this),a=d(n,"touchcancel",r,this),s=d(window,"scroll",r,this)}function d(t,e,n,r){return r&&(n=n.bind(r)),t.addEventListener(e,n),{off:function(){return t.removeEventListener(e,n)}}}var f,p,I=!1,C={iterationInterval:150},m=["none","copy","copyLink","copyMove","link","linkMove","move","all"],y=["none","copy","move","link"],E=["","-webkit-"],v="dnd-poly-",B=v+"drag-image",b=v+"snapback",Q=v+"icon",w=function(){function t(t,e,n,i){this.h=t,this.i=e,this.j=n,this.k=i,this.l=0,this.m=null,this.o=null,this.p=t,this.q=t.changedTouches[0],this.s=this.t.bind(this),this.u=this.v.bind(this),r("touchmove",this.s,!1),r("touchend",this.u,!1),r("touchcancel",this.u,!1)}return t.prototype.A=function(){var t=this;this.l=1,this.B=y[0],this.C={D:{},F:void 0,g:3,G:[]},this.H={x:null,y:null},this.I={x:null,y:null};var e=this.j;if(this.J=new x(this.C,function(n,r,i){e=n,"number"!=typeof r&&"number"!=typeof i||(t.K={x:r||0,y:i||0})}),this.C.g=2,this.J.dropEffect=y[0],u("dragstart",this.j,this.p,this.C,this.J))return this.l=3,this.L(),!1;if(A("page",this.p,this.I),this.M=function(t){var e=t.cloneNode(!0);return function t(e,n){if(1===e.nodeType){for(var r=getComputedStyle(e),i=0;i<r.length;i++){var o=r[i];n.style.setProperty(o,r.getPropertyValue(o),r.getPropertyPriority(o))}n.style.pointerEvents="none",n.removeAttribute("id"),n.removeAttribute("class"),n.removeAttribute("draggable")}if(e.hasChildNodes())for(i=0;i<e.childNodes.length;i++)t(e.childNodes[i],n.childNodes[i])}(t,e),e.style.position="absolute",e.style.left="0px",e.style.top="0px",e.style.zIndex="999999",e.classList.add(B),e.classList.add(Q),e}(e),this.N=function(t){return E.map(function(e){var n=t.style[e+"transform"];return n&&"none"!==n?n.replace(/translate\(\D*\d+[^,]*,\D*\d+[^,]*\)\s*/g,""):""})}(this.M),!this.K)if(this.i.dragImageOffset)this.K={x:this.i.dragImageOffset.x,y:this.i.dragImageOffset.y};else if(this.i.dragImageCenterOnTouch){var n=getComputedStyle(e);this.K={x:0-parseInt(n.marginLeft,10),y:0-parseInt(n.marginTop,10)}}else{var r=e.getBoundingClientRect();n=getComputedStyle(e);this.K={x:r.left-this.q.clientX-parseInt(n.marginLeft,10)+r.width/2,y:r.top-this.q.clientY-parseInt(n.marginTop,10)+r.height/2}}return l(this.M,this.I,this.N,this.K,this.i.dragImageCenterOnTouch),document.body.appendChild(this.M),this.O=setInterval(function(){t.P||(t.P=!0,t.R(),t.P=!1)},this.i.iterationInterval),!0},t.prototype.L=function(){this.O&&(clearInterval(this.O),this.O=null),i("touchmove",this.s),i("touchend",this.u),i("touchcancel",this.u),this.M&&(this.M.parentNode.removeChild(this.M),this.M=null),this.k(this.i,this.p,this.l)},t.prototype.t=function(t){var e=this;if(!1!==s(t,this.q.identifier)){if(this.p=t,0===this.l){var n=void 0;if(this.i.dragStartConditionOverride)try{n=this.i.dragStartConditionOverride(t)}catch(t){n=!1}else n=1===t.touches.length;return n?void(!0===this.A()&&(this.h.preventDefault(),t.preventDefault())):void this.L()}if(t.preventDefault(),A("client",t,this.H),A("page",t,this.I),this.i.dragImageTranslateOverride)try{var r=!1;if(this.i.dragImageTranslateOverride(t,{x:this.H.x,y:this.H.y},this.m,function(t,n){e.M&&(r=!0,e.H.x+=t,e.H.y+=n,e.I.x+=t,e.I.y+=n,l(e.M,e.I,e.N,e.K,e.i.dragImageCenterOnTouch))}),r)return}catch(t){}l(this.M,this.I,this.N,this.K,this.i.dragImageCenterOnTouch)}},t.prototype.v=function(t){if(!1!==s(t,this.q.identifier)){if(this.i.dragImageTranslateOverride)try{this.i.dragImageTranslateOverride(void 0,void 0,void 0,function(){})}catch(t){}if(0===this.l)return void this.L();t.preventDefault(),this.l="touchcancel"===t.type?3:2}},t.prototype.R=function(){var t=this;if(I)var e=v+"debug",n=v+"immediate-user-selection",r=v+"current-drop-target";var i=this.B;this.C.g=3,this.J.dropEffect=y[0];var o=u("drag",this.j,this.p,this.C,this.J);if(o&&(this.B=y[0]),o||2===this.l||3===this.l)return this.S(this.l)?void function(t,e,n,r){var i=getComputedStyle(t);if("hidden"!==i.visibility&&"none"!==i.display){e.classList.add(b);var o=getComputedStyle(e),a=parseFloat(o.transitionDuration);if(isNaN(a)||0===a)r();else{var s=t.getBoundingClientRect(),A={x:s.left,y:s.top};A.x+=document.body.scrollLeft||document.documentElement.scrollLeft,A.y+=document.body.scrollTop||document.documentElement.scrollTop,A.x-=parseInt(i.marginLeft,10),A.y-=parseInt(i.marginTop,10);var c=parseFloat(o.transitionDelay),u=Math.round(1e3*(a+c));l(e,A,n,void 0,!1),setTimeout(r,u)}}else r()}(this.j,this.M,this.N,function(){t.T()}):void this.T();var s=document.elementFromPoint(this.H.x,this.H.y),A=this.o;s!==this.m&&s!==this.o&&(I&&(this.m&&this.m.classList.remove(n),s&&(s.classList.add(e),s.classList.add(n))),this.m=s,null!==this.o&&(this.C.g=3,this.J.dropEffect=y[0],u("dragexit",this.o,this.p,this.C,this.J,!1)),null===this.m?this.o=this.m:(this.C.g=3,this.J.dropEffect=c(this.C.F,this.j),u("dragenter",this.m,this.p,this.C,this.J)?(this.o=this.m,this.B=g(this.J.effectAllowed,this.J.dropEffect)):this.m!==document.body&&(this.o=document.body))),A!==this.o&&a(A)&&(I&&A.classList.remove(r),this.C.g=3,this.J.dropEffect=y[0],u("dragleave",A,this.p,this.C,this.J,!1,this.o)),a(this.o)&&(I&&(this.o.classList.add(e),this.o.classList.add(r)),this.C.g=3,this.J.dropEffect=c(this.C.F,this.j),!1===u("dragover",this.o,this.p,this.C,this.J)?this.B=y[0]:this.B=g(this.J.effectAllowed,this.J.dropEffect)),i!==this.B&&this.M.classList.remove(v+i);var h=v+this.B;!1===this.M.classList.contains(h)&&this.M.classList.add(h)},t.prototype.S=function(t){if(I){var e=v+"immediate-user-selection",n=v+"current-drop-target";this.o&&this.o.classList.remove(n),this.m&&this.m.classList.remove(e)}var r=this.B===y[0]||null===this.o||3===t;return r?a(this.o)&&(this.C.g=3,this.J.dropEffect=y[0],u("dragleave",this.o,this.p,this.C,this.J,!1)):a(this.o)&&(this.C.g=1,this.J.dropEffect=this.B,!0===u("drop",this.o,this.p,this.C,this.J)?this.B=this.J.dropEffect:this.B=y[0]),r},t.prototype.T=function(){this.C.g=3,this.J.dropEffect=this.B,u("dragend",this.j,this.p,this.C,this.J,!1),this.l=2,this.L()},t}(),x=function(){function t(t,e){this.U=t,this.V=e,this.W=y[0]}return Object.defineProperty(t.prototype,"types",{get:function(){if(0!==this.U.g)return Object.freeze(this.U.G)},enumerable:!0,configurable:!0}),t.prototype.setData=function(t,e){if(2===this.U.g){if(t.indexOf(" ")>-1)throw new Error("illegal arg: type contains space");this.U.D[t]=e,-1===this.U.G.indexOf(t)&&this.U.G.push(t)}},t.prototype.getData=function(t){if(1===this.U.g||2===this.U.g)return this.U.D[t]||""},t.prototype.clearData=function(t){if(2===this.U.g){if(t&&this.U.D[t]){delete this.U.D[t];var e=this.U.G.indexOf(t);return void(e>-1&&this.U.G.splice(e,1))}this.U.D={},this.U.G=[]}},t.prototype.setDragImage=function(t,e,n){2===this.U.g&&this.V(t,e,n)},Object.defineProperty(t.prototype,"effectAllowed",{get:function(){return this.U.F},set:function(t){2===this.U.g&&m.indexOf(t)>-1&&(this.U.F=t)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dropEffect",{get:function(){return this.W},set:function(t){0!==this.U.g&&m.indexOf(t)>-1&&(this.W=t)},enumerable:!0,configurable:!0}),t}();t.polyfill=function(t){if(t&&Object.keys(t).forEach(function(e){C[e]=t[e]}),!C.forceApply){var n=function(){var t={dragEvents:"ondragstart"in document.documentElement,draggable:"draggable"in document.documentElement,touchEvents:"ontouchstart"in document.documentElement,userAgentSupportingNativeDnD:void 0},e=!!window.chrome||/chrome/i.test(navigator.userAgent);return t.userAgentSupportingNativeDnD=!(/iPad|iPhone|iPod|Android/.test(navigator.userAgent)||e&&t.touchEvents),I&&Object.keys(t).forEach(function(t){}),t}();if(n.userAgentSupportingNativeDnD&&n.draggable&&n.dragEvents)return!1}return f=function(){var t=!1;try{var e=Object.defineProperty({},"passive",{get:function(){t=!0}});window.addEventListener("test",null,e)}catch(t){}return t}(),C.holdToDrag?r("touchstart",h,!1):r("touchstart",e,!1),!0},Object.defineProperty(t,"__esModule",{value:!0})}(e)},function(t,e,n){var r=n(197),i=n(110);t.exports=function(t){return null==t?[]:r(t,i(t))}},function(t,e,n){var r=n(198);t.exports=function(t,e){return r(e,function(e){return t[e]})}},function(t,e){t.exports=function(t,e){for(var n=-1,r=null==t?0:t.length,i=Array(r);++n<r;)i[n]=e(t[n],n,t);return i}},function(t,e,n){var r=n(200),i=n(201),o=n(82),a=n(111),s=n(204),A=n(112),l=Object.prototype.hasOwnProperty;t.exports=function(t,e){var n=o(t),c=!n&&i(t),u=!n&&!c&&a(t),g=!n&&!c&&!u&&A(t),h=n||c||u||g,d=h?r(t.length,String):[],f=d.length;for(var p in t)!e&&!l.call(t,p)||h&&("length"==p||u&&("offset"==p||"parent"==p)||g&&("buffer"==p||"byteLength"==p||"byteOffset"==p)||s(p,f))||d.push(p);return d}},function(t,e){t.exports=function(t,e){for(var n=-1,r=Array(t);++n<t;)r[n]=e(n);return r}},function(t,e,n){var r=n(202),i=n(40),o=Object.prototype,a=o.hasOwnProperty,s=o.propertyIsEnumerable,A=r(function(){return arguments}())?r:function(t){return i(t)&&a.call(t,"callee")&&!s.call(t,"callee")};t.exports=A},function(t,e,n){var r=n(39),i=n(40);t.exports=function(t){return i(t)&&"[object Arguments]"==r(t)}},function(t,e){t.exports=function(){return!1}},function(t,e){var n=/^(?:0|[1-9]\d*)$/;t.exports=function(t,e){var r=typeof t;return!!(e=null==e?9007199254740991:e)&&("number"==r||"symbol"!=r&&n.test(t))&&t>-1&&t%1==0&&t<e}},function(t,e,n){var r=n(39),i=n(113),o=n(40),a={};a["[object Float32Array]"]=a["[object Float64Array]"]=a["[object Int8Array]"]=a["[object Int16Array]"]=a["[object Int32Array]"]=a["[object Uint8Array]"]=a["[object Uint8ClampedArray]"]=a["[object Uint16Array]"]=a["[object Uint32Array]"]=!0,a["[object Arguments]"]=a["[object Array]"]=a["[object ArrayBuffer]"]=a["[object Boolean]"]=a["[object DataView]"]=a["[object Date]"]=a["[object Error]"]=a["[object Function]"]=a["[object Map]"]=a["[object Number]"]=a["[object Object]"]=a["[object RegExp]"]=a["[object Set]"]=a["[object String]"]=a["[object WeakMap]"]=!1,t.exports=function(t){return o(t)&&i(t.length)&&!!a[r(t)]}},function(t,e){t.exports=function(t){return function(e){return t(e)}}},function(t,e,n){(function(t){var r=n(108),i=e&&!e.nodeType&&e,o=i&&"object"==typeof t&&t&&!t.nodeType&&t,a=o&&o.exports===i&&r.process,s=function(){try{return o&&o.require&&o.require("util").types||a&&a.binding&&a.binding("util")}catch(t){}}();t.exports=s}).call(this,n(43)(t))},function(t,e,n){var r=n(209),i=n(210),o=Object.prototype.hasOwnProperty;t.exports=function(t){if(!r(t))return i(t);var e=[];for(var n in Object(t))o.call(t,n)&&"constructor"!=n&&e.push(n);return e}},function(t,e){var n=Object.prototype;t.exports=function(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||n)}},function(t,e,n){var r=n(211)(Object.keys,Object);t.exports=r},function(t,e){t.exports=function(t,e){return function(n){return t(e(n))}}},function(t,e,n){var r=n(114),i=n(113);t.exports=function(t){return null!=t&&i(t.length)&&!r(t)}},function(t,e,n){var r=n(214);t.exports=function(t,e){return r(t,e)}},function(t,e,n){var r=n(215),i=n(40);t.exports=function t(e,n,o,a,s){return e===n||(null==e||null==n||!i(e)&&!i(n)?e!=e&&n!=n:r(e,n,o,a,t,s))}},function(t,e,n){var r=n(216),i=n(118),o=n(248),a=n(252),s=n(259),A=n(82),l=n(111),c=n(112),u="[object Arguments]",g="[object Array]",h="[object Object]",d=Object.prototype.hasOwnProperty;t.exports=function(t,e,n,f,p,I){var C=A(t),m=A(e),y=C?g:s(t),E=m?g:s(e),v=(y=y==u?h:y)==h,B=(E=E==u?h:E)==h,b=y==E;if(b&&l(t)){if(!l(e))return!1;C=!0,v=!1}if(b&&!v)return I||(I=new r),C||c(t)?i(t,e,n,f,p,I):o(t,e,y,n,f,p,I);if(!(1&n)){var Q=v&&d.call(t,"__wrapped__"),w=B&&d.call(e,"__wrapped__");if(Q||w){var x=Q?t.value():t,_=w?e.value():e;return I||(I=new r),p(x,_,n,f,I)}}return!!b&&(I||(I=new r),a(t,e,n,f,p,I))}},function(t,e,n){var r=n(56),i=n(222),o=n(223),a=n(224),s=n(225),A=n(226);function l(t){var e=this.__data__=new r(t);this.size=e.size}l.prototype.clear=i,l.prototype.delete=o,l.prototype.get=a,l.prototype.has=s,l.prototype.set=A,t.exports=l},function(t,e){t.exports=function(){this.__data__=[],this.size=0}},function(t,e,n){var r=n(57),i=Array.prototype.splice;t.exports=function(t){var e=this.__data__,n=r(e,t);return!(n<0||(n==e.length-1?e.pop():i.call(e,n,1),--this.size,0))}},function(t,e,n){var r=n(57);t.exports=function(t){var e=this.__data__,n=r(e,t);return n<0?void 0:e[n][1]}},function(t,e,n){var r=n(57);t.exports=function(t){return r(this.__data__,t)>-1}},function(t,e,n){var r=n(57);t.exports=function(t,e){var n=this.__data__,i=r(n,t);return i<0?(++this.size,n.push([t,e])):n[i][1]=e,this}},function(t,e,n){var r=n(56);t.exports=function(){this.__data__=new r,this.size=0}},function(t,e){t.exports=function(t){var e=this.__data__,n=e.delete(t);return this.size=e.size,n}},function(t,e){t.exports=function(t){return this.__data__.get(t)}},function(t,e){t.exports=function(t){return this.__data__.has(t)}},function(t,e,n){var r=n(56),i=n(83),o=n(117);t.exports=function(t,e){var n=this.__data__;if(n instanceof r){var a=n.__data__;if(!i||a.length<199)return a.push([t,e]),this.size=++n.size,this;n=this.__data__=new o(a)}return n.set(t,e),this.size=n.size,this}},function(t,e,n){var r=n(114),i=n(228),o=n(55),a=n(116),s=/^\[object .+?Constructor\]$/,A=Function.prototype,l=Object.prototype,c=A.toString,u=l.hasOwnProperty,g=RegExp("^"+c.call(u).replace(/[\\^$.*+?()[\]{}|]/g,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");t.exports=function(t){return!(!o(t)||i(t))&&(r(t)?g:s).test(a(t))}},function(t,e,n){var r,i=n(229),o=(r=/[^.]+$/.exec(i&&i.keys&&i.keys.IE_PROTO||""))?"Symbol(src)_1."+r:"";t.exports=function(t){return!!o&&o in t}},function(t,e,n){var r=n(18)["__core-js_shared__"];t.exports=r},function(t,e){t.exports=function(t,e){return null==t?void 0:t[e]}},function(t,e,n){var r=n(232),i=n(56),o=n(83);t.exports=function(){this.size=0,this.__data__={hash:new r,map:new(o||i),string:new r}}},function(t,e,n){var r=n(233),i=n(234),o=n(235),a=n(236),s=n(237);function A(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}A.prototype.clear=r,A.prototype.delete=i,A.prototype.get=o,A.prototype.has=a,A.prototype.set=s,t.exports=A},function(t,e,n){var r=n(58);t.exports=function(){this.__data__=r?r(null):{},this.size=0}},function(t,e){t.exports=function(t){var e=this.has(t)&&delete this.__data__[t];return this.size-=e?1:0,e}},function(t,e,n){var r=n(58),i=Object.prototype.hasOwnProperty;t.exports=function(t){var e=this.__data__;if(r){var n=e[t];return"__lodash_hash_undefined__"===n?void 0:n}return i.call(e,t)?e[t]:void 0}},function(t,e,n){var r=n(58),i=Object.prototype.hasOwnProperty;t.exports=function(t){var e=this.__data__;return r?void 0!==e[t]:i.call(e,t)}},function(t,e,n){var r=n(58);t.exports=function(t,e){var n=this.__data__;return this.size+=this.has(t)?0:1,n[t]=r&&void 0===e?"__lodash_hash_undefined__":e,this}},function(t,e,n){var r=n(59);t.exports=function(t){var e=r(this,t).delete(t);return this.size-=e?1:0,e}},function(t,e){t.exports=function(t){var e=typeof t;return"string"==e||"number"==e||"symbol"==e||"boolean"==e?"__proto__"!==t:null===t}},function(t,e,n){var r=n(59);t.exports=function(t){return r(this,t).get(t)}},function(t,e,n){var r=n(59);t.exports=function(t){return r(this,t).has(t)}},function(t,e,n){var r=n(59);t.exports=function(t,e){var n=r(this,t),i=n.size;return n.set(t,e),this.size+=n.size==i?0:1,this}},function(t,e,n){var r=n(117),i=n(244),o=n(245);function a(t){var e=-1,n=null==t?0:t.length;for(this.__data__=new r;++e<n;)this.add(t[e])}a.prototype.add=a.prototype.push=i,a.prototype.has=o,t.exports=a},function(t,e){t.exports=function(t){return this.__data__.set(t,"__lodash_hash_undefined__"),this}},function(t,e){t.exports=function(t){return this.__data__.has(t)}},function(t,e){t.exports=function(t,e){for(var n=-1,r=null==t?0:t.length;++n<r;)if(e(t[n],n,t))return!0;return!1}},function(t,e){t.exports=function(t,e){return t.has(e)}},function(t,e,n){var r=n(81),i=n(249),o=n(115),a=n(118),s=n(250),A=n(251),l=r?r.prototype:void 0,c=l?l.valueOf:void 0;t.exports=function(t,e,n,r,l,u,g){switch(n){case"[object DataView]":if(t.byteLength!=e.byteLength||t.byteOffset!=e.byteOffset)return!1;t=t.buffer,e=e.buffer;case"[object ArrayBuffer]":return!(t.byteLength!=e.byteLength||!u(new i(t),new i(e)));case"[object Boolean]":case"[object Date]":case"[object Number]":return o(+t,+e);case"[object Error]":return t.name==e.name&&t.message==e.message;case"[object RegExp]":case"[object String]":return t==e+"";case"[object Map]":var h=s;case"[object Set]":var d=1&r;if(h||(h=A),t.size!=e.size&&!d)return!1;var f=g.get(t);if(f)return f==e;r|=2,g.set(t,e);var p=a(h(t),h(e),r,l,u,g);return g.delete(t),p;case"[object Symbol]":if(c)return c.call(t)==c.call(e)}return!1}},function(t,e,n){var r=n(18).Uint8Array;t.exports=r},function(t,e){t.exports=function(t){var e=-1,n=Array(t.size);return t.forEach(function(t,r){n[++e]=[r,t]}),n}},function(t,e){t.exports=function(t){var e=-1,n=Array(t.size);return t.forEach(function(t){n[++e]=t}),n}},function(t,e,n){var r=n(253),i=Object.prototype.hasOwnProperty;t.exports=function(t,e,n,o,a,s){var A=1&n,l=r(t),c=l.length;if(c!=r(e).length&&!A)return!1;for(var u=c;u--;){var g=l[u];if(!(A?g in e:i.call(e,g)))return!1}var h=s.get(t);if(h&&s.get(e))return h==e;var d=!0;s.set(t,e),s.set(e,t);for(var f=A;++u<c;){var p=t[g=l[u]],I=e[g];if(o)var C=A?o(I,p,g,e,t,s):o(p,I,g,t,e,s);if(!(void 0===C?p===I||a(p,I,n,o,s):C)){d=!1;break}f||(f="constructor"==g)}if(d&&!f){var m=t.constructor,y=e.constructor;m!=y&&"constructor"in t&&"constructor"in e&&!("function"==typeof m&&m instanceof m&&"function"==typeof y&&y instanceof y)&&(d=!1)}return s.delete(t),s.delete(e),d}},function(t,e,n){var r=n(254),i=n(256),o=n(110);t.exports=function(t){return r(t,o,i)}},function(t,e,n){var r=n(255),i=n(82);t.exports=function(t,e,n){var o=e(t);return i(t)?o:r(o,n(t))}},function(t,e){t.exports=function(t,e){for(var n=-1,r=e.length,i=t.length;++n<r;)t[i+n]=e[n];return t}},function(t,e,n){var r=n(257),i=n(258),o=Object.prototype.propertyIsEnumerable,a=Object.getOwnPropertySymbols,s=a?function(t){return null==t?[]:(t=Object(t),r(a(t),function(e){return o.call(t,e)}))}:i;t.exports=s},function(t,e){t.exports=function(t,e){for(var n=-1,r=null==t?0:t.length,i=0,o=[];++n<r;){var a=t[n];e(a,n,t)&&(o[i++]=a)}return o}},function(t,e){t.exports=function(){return[]}},function(t,e,n){var r=n(260),i=n(83),o=n(261),a=n(262),s=n(263),A=n(39),l=n(116),c=l(r),u=l(i),g=l(o),h=l(a),d=l(s),f=A;(r&&"[object DataView]"!=f(new r(new ArrayBuffer(1)))||i&&"[object Map]"!=f(new i)||o&&"[object Promise]"!=f(o.resolve())||a&&"[object Set]"!=f(new a)||s&&"[object WeakMap]"!=f(new s))&&(f=function(t){var e=A(t),n="[object Object]"==e?t.constructor:void 0,r=n?l(n):"";if(r)switch(r){case c:return"[object DataView]";case u:return"[object Map]";case g:return"[object Promise]";case h:return"[object Set]";case d:return"[object WeakMap]"}return e}),t.exports=f},function(t,e,n){var r=n(31)(n(18),"DataView");t.exports=r},function(t,e,n){var r=n(31)(n(18),"Promise");t.exports=r},function(t,e,n){var r=n(31)(n(18),"Set");t.exports=r},function(t,e,n){var r=n(31)(n(18),"WeakMap");t.exports=r},function(t,e,n){(function(t,e){
/**
@license @nocompile
Copyright (c) 2018 The Polymer Project Authors. All rights reserved.
This code may only be used under the BSD style license found at http://polymer.github.io/LICENSE.txt
The complete set of authors may be found at http://polymer.github.io/AUTHORS.txt
The complete set of contributors may be found at http://polymer.github.io/CONTRIBUTORS.txt
Code distributed by Google as part of the polymer project is also
subject to an additional IP rights grant found at http://polymer.github.io/PATENTS.txt
*/
(function(){"use strict";var n,r="function"==typeof Object.defineProperties?Object.defineProperty:function(t,e,n){t!=Array.prototype&&t!=Object.prototype&&(t[e]=n.value)},i="undefined"!=typeof window&&window===this?this:void 0!==t&&null!=t?t:this;function o(){o=function(){},i.Symbol||(i.Symbol=A)}var a,s,A=(a=0,function(t){return"jscomp_symbol_"+(t||"")+a++});function l(){o();var t=i.Symbol.iterator;t||(t=i.Symbol.iterator=i.Symbol("iterator")),"function"!=typeof Array.prototype[t]&&r(Array.prototype,t,{configurable:!0,writable:!0,value:function(){return c(this)}}),l=function(){}}function c(t){var e=0;return function(t){return l(),(t={next:t})[i.Symbol.iterator]=function(){return this},t}(function(){return e<t.length?{done:!1,value:t[e++]}:{done:!0}})}function u(t){l();var e=t[Symbol.iterator];return e?e.call(t):c(t)}if("function"==typeof Object.setPrototypeOf)s=Object.setPrototypeOf;else{var g;t:{var h={};try{h.__proto__={Pa:!0},g=h.Pa;break t}catch(a){}g=!1}s=g?function(t,e){if(t.__proto__=e,t.__proto__!==e)throw new TypeError(t+" is not extensible");return t}:null}var d=s;function f(){this.f=!1,this.b=null,this.fa=void 0,this.a=1,this.G=0,this.c=null}function p(t){if(t.f)throw new TypeError("Generator is already running");t.f=!0}function I(t,e){t.c={Ra:e,Va:!0},t.a=t.G}function C(t,e){return t.a=3,{value:e}}function m(t){this.a=new f,this.b=t}function y(t,e,n,r){try{var i=e.call(t.a.b,n);if(!(i instanceof Object))throw new TypeError("Iterator result "+i+" is not an object");if(!i.done)return t.a.f=!1,i;var o=i.value}catch(e){return t.a.b=null,I(t.a,e),E(t)}return t.a.b=null,r.call(t.a,o),E(t)}function E(t){for(;t.a.a;)try{var e=t.b(t.a);if(e)return t.a.f=!1,{value:e.value,done:!1}}catch(e){t.a.fa=void 0,I(t.a,e)}if(t.a.f=!1,t.a.c){if(e=t.a.c,t.a.c=null,e.Va)throw e.Ra;return{value:e.return,done:!0}}return{value:void 0,done:!0}}function v(t){this.next=function(e){return p(t.a),t.a.b?e=y(t,t.a.b.next,e,t.a.m):(t.a.m(e),e=E(t)),e},this.throw=function(e){return p(t.a),t.a.b?e=y(t,t.a.b.throw,e,t.a.m):(I(t.a,e),e=E(t)),e},this.return=function(e){return function(t,e){p(t.a);var n=t.a.b;return n?y(t,"return"in n?n.return:function(t){return{value:t,done:!0}},e,t.a.return):(t.a.return(e),E(t))}(t,e)},l(),this[Symbol.iterator]=function(){return this}}function B(t,e){return e=new v(new m(e)),d&&d(e,t.prototype),e}f.prototype.m=function(t){this.fa=t},f.prototype.return=function(t){this.c={return:t},this.a=this.G},function(){if(!function(){var t=document.createEvent("Event");return t.initEvent("foo",!0,!0),t.preventDefault(),t.defaultPrevented}()){var t=Event.prototype.preventDefault;Event.prototype.preventDefault=function(){this.cancelable&&(t.call(this),Object.defineProperty(this,"defaultPrevented",{get:function(){return!0},configurable:!0}))}}var e=/Trident/.test(navigator.userAgent);if((!window.CustomEvent||e&&"function"!=typeof window.CustomEvent)&&(window.CustomEvent=function(t,e){e=e||{};var n=document.createEvent("CustomEvent");return n.initCustomEvent(t,!!e.bubbles,!!e.cancelable,e.detail),n},window.CustomEvent.prototype=window.Event.prototype),!window.Event||e&&"function"!=typeof window.Event){var n=window.Event;if(window.Event=function(t,e){e=e||{};var n=document.createEvent("Event");return n.initEvent(t,!!e.bubbles,!!e.cancelable),n},n)for(var r in n)window.Event[r]=n[r];window.Event.prototype=n.prototype}if(!window.MouseEvent||e&&"function"!=typeof window.MouseEvent){if(e=window.MouseEvent,window.MouseEvent=function(t,e){e=e||{};var n=document.createEvent("MouseEvent");return n.initMouseEvent(t,!!e.bubbles,!!e.cancelable,e.view||window,e.detail,e.screenX,e.screenY,e.clientX,e.clientY,e.ctrlKey,e.altKey,e.shiftKey,e.metaKey,e.button,e.relatedTarget),n},e)for(r in e)window.MouseEvent[r]=e[r];window.MouseEvent.prototype=e.prototype}Array.from||(Array.from=function(t){return[].slice.call(t)}),Object.assign||(Object.assign=function(t,e){for(var n,r=[].slice.call(arguments,1),i=0;i<r.length;i++)if(n=r[i])for(var o=t,a=n,s=Object.getOwnPropertyNames(a),A=0;A<s.length;A++)o[n=s[A]]=a[n];return t})}(window.WebComponents),function(){function t(){}function e(t,e){if(!t.childNodes.length)return[];switch(t.nodeType){case Node.DOCUMENT_NODE:return f.call(t,e);case Node.DOCUMENT_FRAGMENT_NODE:return p.call(t,e);default:return d.call(t,e)}}var n="undefined"==typeof HTMLTemplateElement,r=!(document.createDocumentFragment().cloneNode()instanceof DocumentFragment),i=!1;/Trident/.test(navigator.userAgent)&&function(){function t(t,e){if(t instanceof DocumentFragment)for(var r;r=t.firstChild;)n.call(this,r,e);else n.call(this,t,e);return t}i=!0;var e=Node.prototype.cloneNode;Node.prototype.cloneNode=function(t){return t=e.call(this,t),this instanceof DocumentFragment&&(t.__proto__=DocumentFragment.prototype),t},DocumentFragment.prototype.querySelectorAll=HTMLElement.prototype.querySelectorAll,DocumentFragment.prototype.querySelector=HTMLElement.prototype.querySelector,Object.defineProperties(DocumentFragment.prototype,{nodeType:{get:function(){return Node.DOCUMENT_FRAGMENT_NODE},configurable:!0},localName:{get:function(){},configurable:!0},nodeName:{get:function(){return"#document-fragment"},configurable:!0}});var n=Node.prototype.insertBefore;Node.prototype.insertBefore=t;var r=Node.prototype.appendChild;Node.prototype.appendChild=function(e){return e instanceof DocumentFragment?t.call(this,e,null):r.call(this,e),e};var o=Node.prototype.removeChild,a=Node.prototype.replaceChild;Node.prototype.replaceChild=function(e,n){return e instanceof DocumentFragment?(t.call(this,e,n),o.call(this,n)):a.call(this,e,n),n},Document.prototype.createDocumentFragment=function(){var t=this.createElement("df");return t.__proto__=DocumentFragment.prototype,t};var s=Document.prototype.importNode;Document.prototype.importNode=function(t,e){return e=s.call(this,t,e||!1),t instanceof DocumentFragment&&(e.__proto__=DocumentFragment.prototype),e}}();var o=Node.prototype.cloneNode,a=Document.prototype.createElement,s=Document.prototype.importNode,A=Node.prototype.removeChild,l=Node.prototype.appendChild,c=Node.prototype.replaceChild,u=DOMParser.prototype.parseFromString,g=Object.getOwnPropertyDescriptor(window.HTMLElement.prototype,"innerHTML"),h=Object.getOwnPropertyDescriptor(window.Node.prototype,"childNodes"),d=Element.prototype.querySelectorAll,f=Document.prototype.querySelectorAll,p=DocumentFragment.prototype.querySelectorAll,I=function(){if(!n){var t=document.createElement("template"),e=document.createElement("template");return e.content.appendChild(document.createElement("div")),t.content.appendChild(e),0===(t=t.cloneNode(!0)).content.childNodes.length||0===t.content.firstChild.content.childNodes.length||r}}();if(n){var C=document.implementation.createHTMLDocument("template"),m=!0,y=document.createElement("style");y.textContent="template{display:none;}";var E=document.head;E.insertBefore(y,E.firstElementChild),t.prototype=Object.create(HTMLElement.prototype);var v=!document.createElement("div").hasOwnProperty("innerHTML");t.R=function(e){if(!e.content&&e.namespaceURI===document.documentElement.namespaceURI){e.content=C.createDocumentFragment();for(var n;n=e.firstChild;)l.call(e.content,n);if(v)e.__proto__=t.prototype;else if(e.cloneNode=function(e){return t.b(this,e)},m)try{b(e),Q(e)}catch(t){m=!1}t.a(e.content)}};var B={option:["select"],thead:["table"],col:["colgroup","table"],tr:["tbody","table"],th:["tr","tbody","table"],td:["tr","tbody","table"]},b=function(e){Object.defineProperty(e,"innerHTML",{get:function(){return k(this)},set:function(e){var n=B[(/<([a-z][^\/\0>\x20\t\r\n\f]+)/i.exec(e)||["",""])[1].toLowerCase()];if(n)for(var r=0;r<n.length;r++)e="<"+n[r]+">"+e+"</"+n[r]+">";for(C.body.innerHTML=e,t.a(C);this.content.firstChild;)A.call(this.content,this.content.firstChild);if(e=C.body,n)for(r=0;r<n.length;r++)e=e.lastChild;for(;e.firstChild;)l.call(this.content,e.firstChild)},configurable:!0})},Q=function(t){Object.defineProperty(t,"outerHTML",{get:function(){return"<template>"+this.innerHTML+"</template>"},set:function(t){if(!this.parentNode)throw Error("Failed to set the 'outerHTML' property on 'Element': This element has no parent node.");for(C.body.innerHTML=t,t=this.ownerDocument.createDocumentFragment();C.body.firstChild;)l.call(t,C.body.firstChild);c.call(this.parentNode,t,this)},configurable:!0})};b(t.prototype),Q(t.prototype),t.a=function(n){for(var r,i=0,o=(n=e(n,"template")).length;i<o&&(r=n[i]);i++)t.R(r)},document.addEventListener("DOMContentLoaded",function(){t.a(document)}),Document.prototype.createElement=function(){var e=a.apply(this,arguments);return"template"===e.localName&&t.R(e),e},DOMParser.prototype.parseFromString=function(){var e=u.apply(this,arguments);return t.a(e),e},Object.defineProperty(HTMLElement.prototype,"innerHTML",{get:function(){return k(this)},set:function(e){g.set.call(this,e),t.a(this)},configurable:!0,enumerable:!0});var w=/[&\u00A0"]/g,x=/[&\u00A0<>]/g,_=function(t){switch(t){case"&":return"&amp;";case"<":return"&lt;";case">":return"&gt;";case'"':return"&quot;";case" ":return"&nbsp;"}},D=(y=function(t){for(var e={},n=0;n<t.length;n++)e[t[n]]=!0;return e})("area base br col command embed hr img input keygen link meta param source track wbr".split(" ")),M=y("style script xmp iframe noembed noframes plaintext noscript".split(" ")),k=function(t,e){"template"===t.localName&&(t=t.content);for(var n,r="",i=e?e(t):h.get.call(t),o=0,a=i.length;o<a&&(n=i[o]);o++){t:{var s=n,A=t,l=e;switch(s.nodeType){case Node.ELEMENT_NODE:for(var c=s.localName,u="<"+c,g=s.attributes,d=0;A=g[d];d++)u+=" "+A.name+'="'+A.value.replace(w,_)+'"';u+=">",s=D[c]?u:u+k(s,l)+"</"+c+">";break t;case Node.TEXT_NODE:s=s.data,s=A&&M[A.localName]?s:s.replace(x,_);break t;case Node.COMMENT_NODE:s="\x3c!--"+s.data+"--\x3e";break t;default:throw window.console.error(s),Error("not implemented")}}r+=s}return r}}if(n||I){t.b=function(t,e){var n=o.call(t,!1);return this.R&&this.R(n),e&&(l.call(n.content,o.call(t.content,!0)),S(n.content,t.content)),n};var S=function(n,r){if(r.querySelectorAll&&0!==(r=e(r,"template")).length)for(var i,o,a=0,s=(n=e(n,"template")).length;a<s;a++)o=r[a],i=n[a],t&&t.R&&t.R(o),c.call(i.parentNode,L.call(o,!0),i)},L=Node.prototype.cloneNode=function(e){if(!i&&r&&this instanceof DocumentFragment){if(!e)return this.ownerDocument.createDocumentFragment();var n=R.call(this.ownerDocument,this,!0)}else n=this.nodeType===Node.ELEMENT_NODE&&"template"===this.localName&&this.namespaceURI==document.documentElement.namespaceURI?t.b(this,e):o.call(this,e);return e&&S(n,this),n},R=Document.prototype.importNode=function(n,r){if(r=r||!1,"template"===n.localName)return t.b(n,r);var i=s.call(this,n,r);if(r){S(i,n),n=e(i,'script:not([type]),script[type="application/javascript"],script[type="text/javascript"]');for(var o,A=0;A<n.length;A++){o=n[A],(r=a.call(document,"script")).textContent=o.textContent;for(var l,u=o.attributes,g=0;g<u.length;g++)l=u[g],r.setAttribute(l.name,l.value);c.call(o.parentNode,r,o)}}return i}}n&&(window.HTMLTemplateElement=t)}();var b=setTimeout;function Q(){}function w(t){if(!(this instanceof w))throw new TypeError("Promises must be constructed via new");if("function"!=typeof t)throw new TypeError("not a function");this.J=0,this.wa=!1,this.A=void 0,this.U=[],S(t,this)}function x(t,e){for(;3===t.J;)t=t.A;0===t.J?t.U.push(e):(t.wa=!0,w.a(function(){var n=1===t.J?e.Xa:e.Ya;if(null===n)(1===t.J?_:D)(e.qa,t.A);else{try{var r=n(t.A)}catch(t){return void D(e.qa,t)}_(e.qa,r)}}))}function _(t,e){try{if(e===t)throw new TypeError("A promise cannot be resolved with itself.");if(e&&("object"==typeof e||"function"==typeof e)){var n=e.then;if(e instanceof w)return t.J=3,t.A=e,void M(t);if("function"==typeof n)return void S(function(t,e){return function(){t.apply(e,arguments)}}(n,e),t)}t.J=1,t.A=e,M(t)}catch(e){D(t,e)}}function D(t,e){t.J=2,t.A=e,M(t)}function M(t){2===t.J&&0===t.U.length&&w.a(function(){t.wa||w.b(t.A)});for(var e=0,n=t.U.length;e<n;e++)x(t,t.U[e]);t.U=null}function k(t,e,n){this.Xa="function"==typeof t?t:null,this.Ya="function"==typeof e?e:null,this.qa=n}function S(t,e){var n=!1;try{t(function(t){n||(n=!0,_(e,t))},function(t){n||(n=!0,D(e,t))})}catch(t){n||(n=!0,D(e,t))}}if(w.prototype.catch=function(t){return this.then(null,t)},w.prototype.then=function(t,e){var n=new this.constructor(Q);return x(this,new k(t,e,n)),n},w.prototype.finally=function(t){var e=this.constructor;return this.then(function(n){return e.resolve(t()).then(function(){return n})},function(n){return e.resolve(t()).then(function(){return e.reject(n)})})},w.c=function(t){return new w(function(e,n){function r(t,a){try{if(a&&("object"==typeof a||"function"==typeof a)){var s=a.then;if("function"==typeof s)return void s.call(a,function(e){r(t,e)},n)}i[t]=a,0==--o&&e(i)}catch(t){n(t)}}if(!t||void 0===t.length)throw new TypeError("Promise.all accepts an array");var i=Array.prototype.slice.call(t);if(0===i.length)return e([]);for(var o=i.length,a=0;a<i.length;a++)r(a,i[a])})},w.resolve=function(t){return t&&"object"==typeof t&&t.constructor===w?t:new w(function(e){e(t)})},w.reject=function(t){return new w(function(e,n){n(t)})},w.f=function(t){return new w(function(e,n){for(var r=0,i=t.length;r<i;r++)t[r].then(e,n)})},w.a="function"==typeof e&&function(t){e(t)}||function(t){b(t,0)},w.b=function(t){"undefined"!=typeof console&&console&&console.warn("Possible Unhandled Promise Rejection:",t)},!window.Promise&&(window.Promise=w,w.prototype.then=w.prototype.then,w.all=w.c,w.race=w.f,w.resolve=w.resolve,w.reject=w.reject,!window.setImmediate)){var L=document.createTextNode(""),R=[];new MutationObserver(function(){for(var t=R.length,e=0;e<t;e++)R[e]();R.splice(0,t)}).observe(L,{characterData:!0}),w.a=function(t){R.push(t),L.textContent=0<L.textContent.length?"":"a"}}var N={},F=Object.create,T=Object.defineProperties,U=Object.defineProperty;function O(t,e){return{value:t,configurable:!!(e=void 0===e?{}:e).xa,writable:!!e.eb,enumerable:!!e.e}}var G=void 0;try{G=1===U({},"y",{get:function(){return 1}}).y}catch(a){G=!1}var H={},j=F(null);function Y(t){if(this instanceof Y)throw new TypeError("Symbol is not a constructor");var e=function(t){t=String(t);for(var e="",n=0;H[t+e];)e=n+=1;H[t+e]=1;var r="Symbol("+t+e+")";return G&&U(Object.prototype,r,{get:void 0,set:function(t){U(this,r,O(t,{xa:!0,eb:!0}))},configurable:!0,enumerable:!1}),r}(t=void 0===t?"":String(t));return G?F(j,{ta:O(t),Ja:O(e)}):e}T(Y,{for:O(function(t){if(t=String(t),N[t])return N[t];var e=Y(t);return N[t]=e}),keyFor:O(function(t){if(G&&(!t||"Symbol"!==t[Y.toStringTag]))throw new TypeError(t+" is not a symbol");for(var e in N)if(N[e]===t)return G?N[e].ta:N[e].substr(7,N[e].length-8)})}),T(Y,{vb:O(Y("hasInstance")),wb:O(Y("isConcatSpreadable")),iterator:O(Y("iterator")),match:O(Y("match")),replace:O(Y("replace")),search:O(Y("search")),yb:O(Y("species")),split:O(Y("split")),zb:O(Y("toPrimitive")),toStringTag:O(Y("toStringTag")),unscopables:O(Y("unscopables"))}),T(j,{constructor:O(Y),toString:O(function(){return this.Ja}),valueOf:O(function(){return"Symbol("+this.ta+")"})}),G&&U(j,Y.toStringTag,O("Symbol",{xa:!0}));var P="function"==typeof Symbol?Symbol:Y;if(!window.Symbol){window.Symbol=P;var K=window.Symbol.iterator;Array.prototype[K]=function t(){var e,n=this;return B(t,function(t){if(1==t.a&&(e=0),3!=t.a)return e<n.length?t=C(t,n[e]):(t.a=0,t=void 0),t;e++,t.a=2})},Set.prototype[K]=function t(){var e,n,r=this;return B(t,function(t){if(1==t.a&&(e=[],r.forEach(function(t){e.push(t)}),n=0),3!=t.a)return n<e.length?t=C(t,e[n]):(t.a=0,t=void 0),t;n++,t.a=2})},Map.prototype[K]=function t(){var e,n,r=this;return B(t,function(t){if(1==t.a&&(e=[],r.forEach(function(t,n){e.push([n,t])}),n=0),3!=t.a)return n<e.length?t=C(t,e[n]):(t.a=0,t=void 0),t;n++,t.a=2})},String.prototype[K]=function t(){var e,n=this;return B(t,function(t){if(1==t.a&&(e=0),3!=t.a)return e<n.length?t=C(t,n[e]):(t.a=0,t=void 0),t;e++,t.a=2})}}window.WebComponents=window.WebComponents||{flags:{}};var z=document.querySelector('script[src*="webcomponents-bundle"]'),J=/wc-(.+)/,q={};if(!q.noOpts){if(location.search.slice(1).split("&").forEach(function(t){var e;(t=t.split("="))[0]&&(e=t[0].match(J))&&(q[e[1]]=t[1]||!0)}),z)for(var V,W=0;V=z.attributes[W];W++)"src"!==V.name&&(q[V.name]=V.value||!0);if(q.log&&q.log.split){var X=q.log.split(",");q.log={},X.forEach(function(t){q.log[t]=!0})}else q.log={}}window.WebComponents.flags=q;var Z=q.shadydom;Z&&(window.ShadyDOM=window.ShadyDOM||{},window.ShadyDOM.force=Z);var $=q.register||q.ce;function tt(){this.Aa=this.root=null,this.da=!1,this.N=this.$=this.ma=this.assignedSlot=this.assignedNodes=this.S=null,this.childNodes=this.nextSibling=this.previousSibling=this.lastChild=this.firstChild=this.parentNode=this.V=void 0,this.Fa=this.ua=!1,this.Z={}}function et(t){return t.ka||(t.ka=new tt),t.ka}function nt(t){return t&&t.ka}$&&window.customElements&&(window.customElements.forcePolyfill=$),tt.prototype.toJSON=function(){return{}};var rt=window.ShadyDOM||{};rt.Ta=!(!Element.prototype.attachShadow||!Node.prototype.getRootNode);var it=Object.getOwnPropertyDescriptor(Node.prototype,"firstChild");rt.K=!!(it&&it.configurable&&it.get),rt.oa=rt.force||!rt.Ta;var ot=navigator.userAgent.match("Trident"),at=navigator.userAgent.match("Edge");function st(t){return(t=nt(t))&&void 0!==t.firstChild}function At(t){return"ShadyRoot"===t.La}function lt(t){if(At(t=t.getRootNode()))return t}void 0===rt.Ca&&(rt.Ca=rt.K&&(ot||at));var ct=Element.prototype,ut=ct.matches||ct.matchesSelector||ct.mozMatchesSelector||ct.msMatchesSelector||ct.oMatchesSelector||ct.webkitMatchesSelector;function gt(t,e){if(t&&e)for(var n,r=Object.getOwnPropertyNames(e),i=0;i<r.length&&(n=r[i]);i++){var o=n,a=t,s=Object.getOwnPropertyDescriptor(e,o);s&&Object.defineProperty(a,o,s)}}function ht(t,e){for(var n=[],r=1;r<arguments.length;++r)n[r-1]=arguments[r];for(r=0;r<n.length;r++)gt(t,n[r]);return t}var dt=document.createTextNode(""),ft=0,pt=[];function It(t){pt.push(t),dt.textContent=ft++}new MutationObserver(function(){for(;pt.length;)try{pt.shift()()}catch(t){throw dt.textContent=ft++,t}}).observe(dt,{characterData:!0});var Ct=!!document.contains;function mt(t,e){for(;e;){if(e==t)return!0;e=e.parentNode}return!1}function yt(t){for(var e=t.length-1;0<=e;e--){var n=t[e],r=n.getAttribute("id")||n.getAttribute("name");r&&"length"!==r&&isNaN(r)&&(t[r]=n)}return t.item=function(e){return t[e]},t.namedItem=function(e){if("length"!==e&&isNaN(e)&&t[e])return t[e];for(var n=u(t),r=n.next();!r.done;r=n.next())if(((r=r.value).getAttribute("id")||r.getAttribute("name"))==e)return r;return null},t}var Et,vt=[];function Bt(t){Et||(Et=!0,It(bt)),vt.push(t)}function bt(){Et=!1;for(var t=!!vt.length;vt.length;)vt.shift()();return t}function Qt(){this.a=!1,this.addedNodes=[],this.removedNodes=[],this.ca=new Set}bt.list=vt,Qt.prototype.flush=function(){if(this.a){this.a=!1;var t=this.takeRecords();t.length&&this.ca.forEach(function(e){e(t)})}},Qt.prototype.takeRecords=function(){if(this.addedNodes.length||this.removedNodes.length){var t=[{addedNodes:this.addedNodes,removedNodes:this.removedNodes}];return this.addedNodes=[],this.removedNodes=[],t}return[]};var wt=Element.prototype.insertBefore,xt=Element.prototype.replaceChild,_t=Element.prototype.removeChild,Dt=Element.prototype.setAttribute,Mt=Element.prototype.removeAttribute,kt=Element.prototype.cloneNode,St=Document.prototype.importNode,Lt=Element.prototype.addEventListener,Rt=Element.prototype.removeEventListener,Nt=Window.prototype.addEventListener,Ft=Window.prototype.removeEventListener,Tt=Element.prototype.dispatchEvent,Ut=Node.prototype.contains||HTMLElement.prototype.contains,Ot=Document.prototype.getElementById,Gt=Element.prototype.querySelector,Ht=DocumentFragment.prototype.querySelector,jt=Document.prototype.querySelector,Yt=Element.prototype.querySelectorAll,Pt=DocumentFragment.prototype.querySelectorAll,Kt=Document.prototype.querySelectorAll,zt={};zt.appendChild=Element.prototype.appendChild,zt.insertBefore=wt,zt.replaceChild=xt,zt.removeChild=_t,zt.setAttribute=Dt,zt.removeAttribute=Mt,zt.cloneNode=kt,zt.importNode=St,zt.addEventListener=Lt,zt.removeEventListener=Rt,zt.fb=Nt,zt.gb=Ft,zt.dispatchEvent=Tt,zt.contains=Ut,zt.getElementById=Ot,zt.pb=Gt,zt.tb=Ht,zt.nb=jt,zt.querySelector=function(t){switch(this.nodeType){case Node.ELEMENT_NODE:return Gt.call(this,t);case Node.DOCUMENT_NODE:return jt.call(this,t);default:return Ht.call(this,t)}},zt.qb=Yt,zt.ub=Pt,zt.ob=Kt,zt.querySelectorAll=function(t){switch(this.nodeType){case Node.ELEMENT_NODE:return Yt.call(this,t);case Node.DOCUMENT_NODE:return Kt.call(this,t);default:return Pt.call(this,t)}};var Jt=/[&\u00A0"]/g,qt=/[&\u00A0<>]/g;function Vt(t){switch(t){case"&":return"&amp;";case"<":return"&lt;";case">":return"&gt;";case'"':return"&quot;";case" ":return"&nbsp;"}}function Wt(t){for(var e={},n=0;n<t.length;n++)e[t[n]]=!0;return e}var Xt=Wt("area base br col command embed hr img input keygen link meta param source track wbr".split(" ")),Zt=Wt("style script xmp iframe noembed noframes plaintext noscript".split(" "));function $t(t,e){"template"===t.localName&&(t=t.content);for(var n,r="",i=e?e(t):t.childNodes,o=0,a=i.length;o<a&&(n=i[o]);o++){t:{var s=n,A=t,l=e;switch(s.nodeType){case Node.ELEMENT_NODE:for(var c=s.localName,u="<"+c,g=s.attributes,h=0;A=g[h];h++)u+=" "+A.name+'="'+A.value.replace(Jt,Vt)+'"';u+=">",s=Xt[c]?u:u+$t(s,l)+"</"+c+">";break t;case Node.TEXT_NODE:s=s.data,s=A&&Zt[A.localName]?s:s.replace(qt,Vt);break t;case Node.COMMENT_NODE:s="\x3c!--"+s.data+"--\x3e";break t;default:throw window.console.error(s),Error("not implemented")}}r+=s}return r}var te=document.createTreeWalker(document,NodeFilter.SHOW_ALL,null,!1),ee=document.createTreeWalker(document,NodeFilter.SHOW_ELEMENT,null,!1);function ne(t){var e=[];for(te.currentNode=t,t=te.firstChild();t;)e.push(t),t=te.nextSibling();return e}var re={parentNode:function(t){return te.currentNode=t,te.parentNode()},firstChild:function(t){return te.currentNode=t,te.firstChild()},lastChild:function(t){return te.currentNode=t,te.lastChild()},previousSibling:function(t){return te.currentNode=t,te.previousSibling()},nextSibling:function(t){return te.currentNode=t,te.nextSibling()}};re.childNodes=ne,re.parentElement=function(t){return ee.currentNode=t,ee.parentNode()},re.firstElementChild=function(t){return ee.currentNode=t,ee.firstChild()},re.lastElementChild=function(t){return ee.currentNode=t,ee.lastChild()},re.previousElementSibling=function(t){return ee.currentNode=t,ee.previousSibling()},re.nextElementSibling=function(t){return ee.currentNode=t,ee.nextSibling()},re.children=function(t){var e=[];for(ee.currentNode=t,t=ee.firstChild();t;)e.push(t),t=ee.nextSibling();return yt(e)},re.innerHTML=function(t){return $t(t,function(t){return ne(t)})},re.textContent=function(t){switch(t.nodeType){case Node.ELEMENT_NODE:case Node.DOCUMENT_FRAGMENT_NODE:t=document.createTreeWalker(t,NodeFilter.SHOW_TEXT,null,!1);for(var e,n="";e=t.nextNode();)n+=e.nodeValue;return n;default:return t.nodeValue}};var ie=rt.K,oe=[Node.prototype,Element.prototype,HTMLElement.prototype];function ae(t){var e;t:{for(e=0;e<oe.length;e++){var n=oe[e];if(n.hasOwnProperty(t)){e=n;break t}}e=void 0}if(!e)throw Error("Could not find descriptor for "+t);return Object.getOwnPropertyDescriptor(e,t)}var se=ie?{parentNode:ae("parentNode"),firstChild:ae("firstChild"),lastChild:ae("lastChild"),previousSibling:ae("previousSibling"),nextSibling:ae("nextSibling"),childNodes:ae("childNodes"),parentElement:ae("parentElement"),previousElementSibling:ae("previousElementSibling"),nextElementSibling:ae("nextElementSibling"),innerHTML:ae("innerHTML"),textContent:ae("textContent"),firstElementChild:ae("firstElementChild"),lastElementChild:ae("lastElementChild"),children:ae("children")}:{},Ae=ie?{firstElementChild:Object.getOwnPropertyDescriptor(DocumentFragment.prototype,"firstElementChild"),lastElementChild:Object.getOwnPropertyDescriptor(DocumentFragment.prototype,"lastElementChild"),children:Object.getOwnPropertyDescriptor(DocumentFragment.prototype,"children")}:{},le=ie?{firstElementChild:Object.getOwnPropertyDescriptor(Document.prototype,"firstElementChild"),lastElementChild:Object.getOwnPropertyDescriptor(Document.prototype,"lastElementChild"),children:Object.getOwnPropertyDescriptor(Document.prototype,"children")}:{},ce={za:se,sb:Ae,mb:le,parentNode:function(t){return se.parentNode.get.call(t)},firstChild:function(t){return se.firstChild.get.call(t)},lastChild:function(t){return se.lastChild.get.call(t)},previousSibling:function(t){return se.previousSibling.get.call(t)},nextSibling:function(t){return se.nextSibling.get.call(t)},childNodes:function(t){return Array.prototype.slice.call(se.childNodes.get.call(t))},parentElement:function(t){return se.parentElement.get.call(t)},previousElementSibling:function(t){return se.previousElementSibling.get.call(t)},nextElementSibling:function(t){return se.nextElementSibling.get.call(t)},innerHTML:function(t){return se.innerHTML.get.call(t)},textContent:function(t){return se.textContent.get.call(t)},children:function(t){switch(t.nodeType){case Node.DOCUMENT_FRAGMENT_NODE:return Ae.children.get.call(t);case Node.DOCUMENT_NODE:return le.children.get.call(t);default:return se.children.get.call(t)}},firstElementChild:function(t){switch(t.nodeType){case Node.DOCUMENT_FRAGMENT_NODE:return Ae.firstElementChild.get.call(t);case Node.DOCUMENT_NODE:return le.firstElementChild.get.call(t);default:return se.firstElementChild.get.call(t)}},lastElementChild:function(t){switch(t.nodeType){case Node.DOCUMENT_FRAGMENT_NODE:return Ae.lastElementChild.get.call(t);case Node.DOCUMENT_NODE:return le.lastElementChild.get.call(t);default:return se.lastElementChild.get.call(t)}}},ue=rt.Ca?ce:re;function ge(t){for(;t.firstChild;)t.removeChild(t.firstChild)}var he=rt.K,de=document.implementation.createHTMLDocument("inert"),fe=Object.getOwnPropertyDescriptor(Node.prototype,"isConnected"),pe=fe&&fe.get,Ie=Object.getOwnPropertyDescriptor(Document.prototype,"activeElement"),Ce={parentElement:{get:function(){var t=nt(this);return(t=t&&t.parentNode)&&t.nodeType!==Node.ELEMENT_NODE&&(t=null),void 0!==t?t:ue.parentElement(this)},configurable:!0},parentNode:{get:function(){var t=nt(this);return void 0!==(t=t&&t.parentNode)?t:ue.parentNode(this)},configurable:!0},nextSibling:{get:function(){var t=nt(this);return void 0!==(t=t&&t.nextSibling)?t:ue.nextSibling(this)},configurable:!0},previousSibling:{get:function(){var t=nt(this);return void 0!==(t=t&&t.previousSibling)?t:ue.previousSibling(this)},configurable:!0},nextElementSibling:{get:function(){var t=nt(this);if(t&&void 0!==t.nextSibling){for(t=this.nextSibling;t&&t.nodeType!==Node.ELEMENT_NODE;)t=t.nextSibling;return t}return ue.nextElementSibling(this)},configurable:!0},previousElementSibling:{get:function(){var t=nt(this);if(t&&void 0!==t.previousSibling){for(t=this.previousSibling;t&&t.nodeType!==Node.ELEMENT_NODE;)t=t.previousSibling;return t}return ue.previousElementSibling(this)},configurable:!0}},me={className:{get:function(){return this.getAttribute("class")||""},set:function(t){this.setAttribute("class",t)},configurable:!0}},ye={childNodes:{get:function(){if(st(this)){var t=nt(this);if(!t.childNodes){t.childNodes=[];for(var e=this.firstChild;e;e=e.nextSibling)t.childNodes.push(e)}var n=t.childNodes}else n=ue.childNodes(this);return n.item=function(t){return n[t]},n},configurable:!0},childElementCount:{get:function(){return this.children.length},configurable:!0},firstChild:{get:function(){var t=nt(this);return void 0!==(t=t&&t.firstChild)?t:ue.firstChild(this)},configurable:!0},lastChild:{get:function(){var t=nt(this);return void 0!==(t=t&&t.lastChild)?t:ue.lastChild(this)},configurable:!0},textContent:{get:function(){if(st(this)){for(var t,e=[],n=0,r=this.childNodes;t=r[n];n++)t.nodeType!==Node.COMMENT_NODE&&e.push(t.textContent);return e.join("")}return ue.textContent(this)},set:function(t){switch(null==t&&(t=""),this.nodeType){case Node.ELEMENT_NODE:case Node.DOCUMENT_FRAGMENT_NODE:if(!st(this)&&he){var e=this.firstChild;(e!=this.lastChild||e&&e.nodeType!=Node.TEXT_NODE)&&ge(this),ce.za.textContent.set.call(this,t)}else ge(this),(0<t.length||this.nodeType===Node.ELEMENT_NODE)&&this.appendChild(document.createTextNode(t));break;default:this.nodeValue=t}},configurable:!0},firstElementChild:{get:function(){var t=nt(this);if(t&&void 0!==t.firstChild){for(t=this.firstChild;t&&t.nodeType!==Node.ELEMENT_NODE;)t=t.nextSibling;return t}return ue.firstElementChild(this)},configurable:!0},lastElementChild:{get:function(){var t=nt(this);if(t&&void 0!==t.lastChild){for(t=this.lastChild;t&&t.nodeType!==Node.ELEMENT_NODE;)t=t.previousSibling;return t}return ue.lastElementChild(this)},configurable:!0},children:{get:function(){return st(this)?yt(Array.prototype.filter.call(this.childNodes,function(t){return t.nodeType===Node.ELEMENT_NODE})):ue.children(this)},configurable:!0},innerHTML:{get:function(){return st(this)?$t("template"===this.localName?this.content:this):ue.innerHTML(this)},set:function(t){var e="template"===this.localName?this.content:this;ge(e);var n=this.localName||"div";for(n=this.namespaceURI&&this.namespaceURI!==de.namespaceURI?de.createElementNS(this.namespaceURI,n):de.createElement(n),he?ce.za.innerHTML.set.call(n,t):n.innerHTML=t,t="template"===this.localName?n.content:n;t.firstChild;)e.appendChild(t.firstChild)},configurable:!0}},Ee={shadowRoot:{get:function(){var t=nt(this);return t&&t.Aa||null},configurable:!0}},ve={activeElement:{get:function(){var t=Ie&&Ie.get?Ie.get.call(document):rt.K?void 0:document.activeElement;if(t&&t.nodeType){var e=!!At(this);if(this===document||e&&this.host!==t&&zt.contains.call(this.host,t)){for(e=lt(t);e&&e!==this;)e=lt(t=e.host);t=this===document?e?null:t:e===this?t:null}else t=null}else t=null;return t},set:function(){},configurable:!0}};function Be(t,e,n){for(var r in e){var i=Object.getOwnPropertyDescriptor(t,r);i&&i.configurable||!i&&n?Object.defineProperty(t,r,e[r]):n&&console.warn("Could not define",r,"on",t)}}function be(t){Be(t,Ce),Be(t,me),Be(t,ye),Be(t,ve)}var Qe=rt.K?function(){}:function(t){var e=et(t);e.ua||(e.ua=!0,Be(t,Ce,!0),Be(t,me,!0))},we=rt.K?function(){}:function(t){et(t).Fa||(Be(t,ye,!0),Be(t,Ee,!0))},xe=ue.childNodes;function _e(t,e,n){Qe(t),n=n||null;var r=et(t),i=et(e),o=n?et(n):null;r.previousSibling=n?o.previousSibling:e.lastChild,(o=nt(r.previousSibling))&&(o.nextSibling=t),(o=nt(r.nextSibling=n))&&(o.previousSibling=t),r.parentNode=e,n?n===i.firstChild&&(i.firstChild=t):(i.lastChild=t,i.firstChild||(i.firstChild=t)),i.childNodes=null}function De(t){var e=et(t);if(void 0===e.firstChild){e.childNodes=null;var n=xe(t);for(e.firstChild=n[0]||null,e.lastChild=n[n.length-1]||null,we(t),e=0;e<n.length;e++){var r=n[e],i=et(r);i.parentNode=t,i.nextSibling=n[e+1]||null,i.previousSibling=n[e-1]||null,Qe(r)}}}var Me=ue.parentNode;function ke(t,e,n){if(e===t)throw Error("Failed to execute 'appendChild' on 'Node': The new child element contains the parent.");if(n){var r=nt(n);if(void 0!==(r=r&&r.parentNode)&&r!==t||void 0===r&&Me(n)!==t)throw Error("Failed to execute 'insertBefore' on 'Node': The node before which the new node is to be inserted is not a child of this node.")}if(n===e)return e;var i,o,a;if((e.parentNode&&Se(e.parentNode,e),r=!0,!e.__noInsertionPoint)&&((o=i=lt(t))&&("slot"===e.localName?a=[e]:e.querySelectorAll&&(a=e.querySelectorAll("slot")),o=a&&a.length?a:void 0),o)){var s=o;(a=i).a=a.a||[],a.o=a.o||[],a.w=a.w||{},a.a.push.apply(a.a,s instanceof Array?s:function(t){for(var e,n=[];!(e=t.next()).done;)n.push(e.value);return n}(u(s)))}if(("slot"===t.localName||o)&&(i=i||lt(t))&&fn(i),st(t)){if(i=n,we(t),void 0!==(o=et(t)).firstChild&&(o.childNodes=null),e.nodeType===Node.DOCUMENT_FRAGMENT_NODE){for(o=e.childNodes,a=0;a<o.length;a++)_e(o[a],t,i);o=void 0!==(i=et(e)).firstChild?null:void 0,i.firstChild=i.lastChild=o,i.childNodes=o}else _e(e,t,i);i=nt(t),Le(t)?(fn(i.root),r=!1):i.root&&(r=!1)}return r?(r=At(t)?t.host:t,n?(n=function t(e){var n=e;return e&&"slot"===e.localName&&(n=(n=(n=nt(e))&&n.N)&&n.length?n[0]:t(e.nextSibling)),n}(n),zt.insertBefore.call(r,e,n)):zt.appendChild.call(r,e)):e.ownerDocument!==t.ownerDocument&&t.ownerDocument.adoptNode(e),Ne(t,e),e}function Se(t,e){if(e.parentNode!==t)throw Error("The node to be removed is not a child of this node: "+e);var n=lt(e),r=nt(t);if(st(t)){var i=et(e),o=et(t);e===o.firstChild&&(o.firstChild=i.nextSibling),e===o.lastChild&&(o.lastChild=i.previousSibling);var a=i.previousSibling,s=i.nextSibling;if(a&&(et(a).nextSibling=s),s&&(et(s).previousSibling=a),i.parentNode=i.previousSibling=i.nextSibling=void 0,void 0!==o.childNodes&&(o.childNodes=null),Le(t)){fn(r.root);var A=!0}}if(function t(e){var n=nt(e);if(n&&void 0!==n.V)for(var r,i=0,o=(n=e.childNodes).length;i<o&&(r=n[i]);i++)t(r);(e=nt(e))&&(e.V=void 0)}(e),n){if((i=t&&"slot"===t.localName)&&(A=!0),n.o){for(d in yn(n),o=n.w)for(a=o[d],s=0;s<a.length;s++){var l=a[s];if(mt(e,l)){a.splice(s,1);var c=n.o.indexOf(l);if(0<=c&&n.o.splice(c,1),s--,l=(c=nt(l)).N)for(var u=0;u<l.length;u++){var g=l[u],h=ln(g);h&&zt.removeChild.call(h,g)}c.N=[],c.assignedNodes=[],c=!0}}var d=c}else d=void 0;(d||i)&&fn(n)}return A||(A=At(t)?t.host:t,(!r.root&&"slot"!==e.localName||A===Me(e))&&zt.removeChild.call(A,e)),Ne(t,null,e),e}function Le(t){return(t=(t=nt(t))&&t.root)&&Bn(t)}function Re(t,e){if("slot"===e)Le(t=t.parentNode)&&fn(nt(t).root);else if("slot"===t.localName&&"name"===e&&(e=lt(t))){if(e.o){yn(e);var n=t.Ia,r=En(t);if(r!==n){var i=(n=e.w[n]).indexOf(t);0<=i&&n.splice(i,1),(n=e.w[r]||(e.w[r]=[])).push(t),1<n.length&&(e.w[r]=vn(n))}}fn(e)}}function Ne(t,e,n){(t=(t=nt(t))&&t.S)&&(e&&t.addedNodes.push(e),n&&t.removedNodes.push(n),function(t){t.a||(t.a=!0,It(function(){t.flush()}))}(t))}function Fe(t,e,n){var r=[];return function t(e,n,r,i){for(var o,a=0,s=e.length;a<s&&(o=e[a]);a++){var A;if(A=o.nodeType===Node.ELEMENT_NODE){var l=n,c=r,u=i,g=l(A=o);g&&u.push(A),c&&c(g)?A=g:(t(A.childNodes,l,c,u),A=void 0)}if(A)break}}(t.childNodes,e,n,r),r}var Te=null;function Ue(t,e,n){Te||(Te=window.ShadyCSS&&window.ShadyCSS.ScopingShim),Te&&"class"===e?Te.setElementClass(t,n):(zt.setAttribute.call(t,e,n),Re(t,e))}var Oe="__eventWrappers"+Date.now(),Ge=function(){var t=Object.getOwnPropertyDescriptor(Event.prototype,"composed");return t?function(e){return t.get.call(e)}:null}(),He={blur:!0,focus:!0,focusin:!0,focusout:!0,click:!0,dblclick:!0,mousedown:!0,mouseenter:!0,mouseleave:!0,mousemove:!0,mouseout:!0,mouseover:!0,mouseup:!0,wheel:!0,beforeinput:!0,input:!0,keydown:!0,keyup:!0,compositionstart:!0,compositionupdate:!0,compositionend:!0,touchstart:!0,touchend:!0,touchmove:!0,touchcancel:!0,pointerover:!0,pointerenter:!0,pointerdown:!0,pointermove:!0,pointerup:!0,pointercancel:!0,pointerout:!0,pointerleave:!0,gotpointercapture:!0,lostpointercapture:!0,dragstart:!0,drag:!0,dragenter:!0,dragleave:!0,dragover:!0,drop:!0,dragend:!0,DOMActivate:!0,DOMFocusIn:!0,DOMFocusOut:!0,keypress:!0},je={DOMAttrModified:!0,DOMAttributeNameChanged:!0,DOMCharacterDataModified:!0,DOMElementNameChanged:!0,DOMNodeInserted:!0,DOMNodeInsertedIntoDocument:!0,DOMNodeRemoved:!0,DOMNodeRemovedFromDocument:!0,DOMSubtreeModified:!0};function Ye(t,e){var n=[],r=t;for(t=t===window?window:t.getRootNode();r;)n.push(r),r=r.assignedSlot?r.assignedSlot:r.nodeType===Node.DOCUMENT_FRAGMENT_NODE&&r.host&&(e||r!==t)?r.host:r.parentNode;return n[n.length-1]===document&&n.push(window),n}function Pe(t,e){if(!At)return t;t=Ye(t,!0);for(var n,r,i,o,a=0;a<e.length;a++)if((i=(n=e[a])===window?window:n.getRootNode())!==r&&(o=t.indexOf(i),r=i),!At(i)||-1<o)return n}var Ke={get composed(){return void 0===this.Y&&(Ge?this.Y=Ge(this):!1!==this.isTrusted&&(this.Y=He[this.type])),this.Y||!1},composedPath:function(){return this.sa||(this.sa=Ye(this.__target,this.composed)),this.sa},get target(){return Pe(this.currentTarget||this.__previousCurrentTarget,this.composedPath())},get relatedTarget(){return this.ja?(this.va||(this.va=Ye(this.ja,!0)),Pe(this.currentTarget||this.__previousCurrentTarget,this.va)):null},stopPropagation:function(){Event.prototype.stopPropagation.call(this),this.ia=!0},stopImmediatePropagation:function(){Event.prototype.stopImmediatePropagation.call(this),this.ia=this.Ea=!0}};function ze(t){function e(e,n){return(e=new t(e,n)).Y=n&&!!n.composed,e}return function(t,e){for(var n in e)t[n]=e[n]}(e,t),e.prototype=t.prototype,e}var Je={focus:!0,blur:!0};function qe(t){return t.__target!==t.target||t.ja!==t.relatedTarget}function Ve(t,e,n){if(n=e.__handlers&&e.__handlers[t.type]&&e.__handlers[t.type][n])for(var r,i=0;(r=n[i])&&(!qe(t)||t.target!==t.relatedTarget)&&(r.call(e,t),!t.Ea);i++);}function We(t){var e,n=t.composedPath();Object.defineProperty(t,"currentTarget",{get:function(){return i},configurable:!0});for(var r=n.length-1;0<=r;r--){var i=n[r];if(Ve(t,i,"capture"),t.ia)return}for(Object.defineProperty(t,"eventPhase",{get:function(){return Event.AT_TARGET}}),r=0;r<n.length;r++){var o=nt(i=n[r]);if(o=o&&o.root,(0===r||o&&o===e)&&(Ve(t,i,"bubble"),i!==window&&(e=i.getRootNode()),t.ia))break}}function Xe(t,e,n,r,i,o){for(var a=0;a<t.length;a++){var s=t[a],A=s.type,l=s.capture,c=s.once,u=s.passive;if(e===s.node&&n===A&&r===l&&i===c&&o===u)return a}return-1}function Ze(t,e,n){if(e){var r=typeof e;if(("function"===r||"object"===r)&&("object"!==r||e.handleEvent&&"function"==typeof e.handleEvent)){var i=this instanceof Window?zt.fb:zt.addEventListener;if(je[t])return i.call(this,t,e,n);if(n&&"object"==typeof n)var o=!!n.capture,a=!!n.once,s=!!n.passive;else o=!!n,s=a=!1;var A=n&&n.la||this,l=e[Oe];if(l){if(-1<Xe(l,A,t,o,a,s))return}else e[Oe]=[];l=function(i){if(a&&this.removeEventListener(t,e,n),i.__target||tn(i),A!==this){var o=Object.getOwnPropertyDescriptor(i,"currentTarget");Object.defineProperty(i,"currentTarget",{get:function(){return A},configurable:!0})}if(i.__previousCurrentTarget=i.currentTarget,(!At(A)||-1!=i.composedPath().indexOf(A))&&(i.composed||-1<i.composedPath().indexOf(A)))if(qe(i)&&i.target===i.relatedTarget)i.eventPhase===Event.BUBBLING_PHASE&&i.stopImmediatePropagation();else if(i.eventPhase===Event.CAPTURING_PHASE||i.bubbles||i.target===A||A instanceof Window){var s="function"===r?e.call(A,i):e.handleEvent&&e.handleEvent(i);return A!==this&&(o?(Object.defineProperty(i,"currentTarget",o),o=null):delete i.currentTarget),s}},e[Oe].push({node:A,type:t,capture:o,once:a,passive:s,hb:l}),Je[t]?(this.__handlers=this.__handlers||{},this.__handlers[t]=this.__handlers[t]||{capture:[],bubble:[]},this.__handlers[t][o?"capture":"bubble"].push(l)):i.call(this,t,l,n)}}}function $e(t,e,n){if(e){var r=this instanceof Window?zt.gb:zt.removeEventListener;if(je[t])return r.call(this,t,e,n);if(n&&"object"==typeof n)var i=!!n.capture,o=!!n.once,a=!!n.passive;else i=!!n,a=o=!1;var s=n&&n.la||this,A=void 0,l=null;try{l=e[Oe]}catch(t){}l&&-1<(o=Xe(l,s,t,i,o,a))&&(A=l.splice(o,1)[0].hb,l.length||(e[Oe]=void 0)),r.call(this,t,A||e,n),A&&Je[t]&&this.__handlers&&this.__handlers[t]&&-1<(A=(t=this.__handlers[t][i?"capture":"bubble"]).indexOf(A))&&t.splice(A,1)}}function tn(t){if(t.__target=t.target,t.ja=t.relatedTarget,rt.K){var e=Object.getPrototypeOf(t);if(!e.hasOwnProperty("__patchProto")){var n=Object.create(e);n.jb=e,gt(n,Ke),e.__patchProto=n}t.__proto__=e.__patchProto}else gt(t,Ke)}var en=ze(window.Event),nn=ze(window.CustomEvent),rn=ze(window.MouseEvent);function on(t,e){return{index:t,W:[],ba:e}}function an(t,e,n,r){var i=0,o=0,a=0,s=0,A=Math.min(e-i,r-o);if(0==i&&0==o)t:{for(a=0;a<A;a++)if(t[a]!==n[a])break t;a=A}if(e==t.length&&r==n.length){s=t.length;for(var l=n.length,c=0;c<A-a&&sn(t[--s],n[--l]);)c++;s=c}if(o+=a,r-=s,0==(e-=s)-(i+=a)&&0==r-o)return[];if(i==e){for(e=on(i,0);o<r;)e.W.push(n[o++]);return[e]}if(o==r)return[on(i,e-i)];for(r=r-(a=o)+1,s=e-(A=i)+1,e=Array(r),l=0;l<r;l++)e[l]=Array(s),e[l][0]=l;for(l=0;l<s;l++)e[0][l]=l;for(l=1;l<r;l++)for(c=1;c<s;c++)if(t[A+c-1]===n[a+l-1])e[l][c]=e[l-1][c-1];else{var u=e[l-1][c]+1,g=e[l][c-1]+1;e[l][c]=u<g?u:g}for(A=e.length-1,a=e[0].length-1,r=e[A][a],t=[];0<A||0<a;)0==A?(t.push(2),a--):0==a?(t.push(3),A--):(s=e[A-1][a-1],(u=(l=e[A-1][a])<(c=e[A][a-1])?l<s?l:s:c<s?c:s)==s?(s==r?t.push(0):(t.push(1),r=s),A--,a--):u==l?(t.push(3),A--,r=l):(t.push(2),a--,r=c));for(t.reverse(),e=void 0,A=[],a=0;a<t.length;a++)switch(t[a]){case 0:e&&(A.push(e),e=void 0),i++,o++;break;case 1:e||(e=on(i,0)),e.ba++,i++,e.W.push(n[o]),o++;break;case 2:e||(e=on(i,0)),e.ba++,i++;break;case 3:e||(e=on(i,0)),e.W.push(n[o]),o++}return e&&A.push(e),A}function sn(t,e){return t===e}var An,ln=ue.parentNode,cn=ue.childNodes,un={},gn=rt.deferConnectionCallbacks&&"loading"===document.readyState;function hn(t){var e=[];do{e.unshift(t)}while(t=t.parentNode);return e}function dn(t,e,n){if(t!==un)throw new TypeError("Illegal constructor");this.La="ShadyRoot",this.host=e,this.c=n&&n.mode,De(e),(t=et(e)).root=this,t.Aa="closed"!==this.c?this:null,(t=et(this)).firstChild=t.lastChild=t.parentNode=t.nextSibling=t.previousSibling=null,t.childNodes=[],this.b=this.aa=!1,this.a=this.w=this.o=null,fn(this)}function fn(t){t.aa||(t.aa=!0,Bt(function(){return pn(t)}))}function pn(t){for(var e;t;){t.aa&&(e=t);t:{var n=t;if(At(t=n.host.getRootNode()))for(var r=n.host.childNodes,i=0;i<r.length;i++)if("slot"==(n=r[i]).localName)break t;t=void 0}}e&&e._renderRoot()}function In(t,e,n){var r=et(e),i=r.$;r.$=null,n||(n=(t=t.w[e.slot||"__catchall"])&&t[0]),n?(et(n).assignedNodes.push(e),r.assignedSlot=n):r.assignedSlot=void 0,i!==r.assignedSlot&&r.assignedSlot&&(et(r.assignedSlot).da=!0)}function Cn(t,e,n){for(var r,i=0;i<n.length&&(r=n[i]);i++)if("slot"==r.localName){var o=nt(r).assignedNodes;o&&o.length&&Cn(t,e,o)}else e.push(n[i])}function mn(t,e){zt.dispatchEvent.call(e,new Event("slotchange")),(e=nt(e)).assignedSlot&&mn(t,e.assignedSlot)}function yn(t){if(t.a&&t.a.length){for(var e,n=t.a,r=0;r<n.length;r++){var i=n[r];De(i),De(i.parentNode);var o=En(i);t.w[o]?((e=e||{})[o]=!0,t.w[o].push(i)):t.w[o]=[i],t.o.push(i)}if(e)for(var a in e)t.w[a]=vn(t.w[a]);t.a=[]}}function En(t){var e=t.name||t.getAttribute("name")||"__catchall";return t.Ia=e}function vn(t){return t.sort(function(t,e){t=hn(t);for(var n=hn(e),r=0;r<t.length;r++){e=t[r];var i=n[r];if(e!==i)return(t=Array.from(e.parentNode.childNodes)).indexOf(e)-t.indexOf(i)}})}function Bn(t){return yn(t),!(!t.o||!t.o.length)}if(dn.prototype._renderRoot=function(){var t=gn;if(gn=!0,this.aa=!1,this.o){yn(this);for(var e,n=0;n<this.o.length;n++){var r=nt(e=this.o[n]),i=r.assignedNodes;if(r.assignedNodes=[],r.N=[],r.ma=i)for(r=0;r<i.length;r++){var o=nt(i[r]);o.$=o.assignedSlot,o.assignedSlot===e&&(o.assignedSlot=null)}}for(e=this.host.firstChild;e;e=e.nextSibling)In(this,e);for(n=0;n<this.o.length;n++){if(!(i=nt(e=this.o[n])).assignedNodes.length)for(r=e.firstChild;r;r=r.nextSibling)In(this,r,e);if((r=(r=nt(e.parentNode))&&r.root)&&Bn(r)&&r._renderRoot(),Cn(this,i.N,i.assignedNodes),r=i.ma){for(o=0;o<r.length;o++)nt(r[o]).$=null;i.ma=null,r.length>i.assignedNodes.length&&(i.da=!0)}i.da&&(i.da=!1,mn(this,e))}for(n=this.o,e=[],i=0;i<n.length;i++)(o=nt(r=n[i].parentNode))&&o.root||!(0>e.indexOf(r))||e.push(r);for(n=0;n<e.length;n++){r=(i=e[n])===this?this.host:i,o=[],i=i.childNodes;for(var a=0;a<i.length;a++){var s=i[a];if("slot"==s.localName){s=nt(s).N;for(var A=0;A<s.length;A++)o.push(s[A])}else o.push(s)}i=void 0,a=cn(r),s=an(o,o.length,a,a.length);for(var l=A=0;A<s.length&&(i=s[A]);A++){for(var c,u=0;u<i.W.length&&(c=i.W[u]);u++)ln(c)===r&&zt.removeChild.call(r,c),a.splice(i.index+l,1);l-=i.ba}for(l=0;l<s.length&&(i=s[l]);l++)for(A=a[i.index],u=i.index;u<i.index+i.ba;u++)c=o[u],zt.insertBefore.call(r,c,A),a.splice(u,0,c)}}if(!this.b)for(e=0,n=(c=this.host.childNodes).length;e<n;e++)r=nt(i=c[e]),ln(i)!==this.host||"slot"!==i.localName&&r.assignedSlot||zt.removeChild.call(this.host,i);this.b=!0,gn=t,An&&An()},window.customElements&&rt.oa){var bn=new Map;An=function(){var t=Array.from(bn);bn.clear();for(var e=(t=u(t)).next();!e.done;e=t.next()){var n=(e=u(e.value)).next().value;e.next().value?n.Ga():n.Ha()}},gn&&document.addEventListener("readystatechange",function(){gn=!1,An()},{once:!0});var Qn=window.customElements.define;Object.defineProperty(window.CustomElementRegistry.prototype,"define",{value:function(t,e){var n=e.prototype.connectedCallback,r=e.prototype.disconnectedCallback;Qn.call(window.customElements,t,function(t,e,n){var r=0,i="__isConnected"+r++;return(e||n)&&(t.prototype.connectedCallback=t.prototype.Ga=function(){gn?bn.set(this,!0):this[i]||(this[i]=!0,e&&e.call(this))},t.prototype.disconnectedCallback=t.prototype.Ha=function(){gn?this.isConnected||bn.set(this,!1):this[i]&&(this[i]=!1,n&&n.call(this))}),t}(e,n,r)),e.prototype.connectedCallback=n,e.prototype.disconnectedCallback=r}})}function wn(t){var e=t.getRootNode();return At(e)&&pn(e),(t=nt(t))&&t.assignedSlot||null}var xn={addEventListener:Ze.bind(window),removeEventListener:$e.bind(window)},_n={addEventListener:Ze,removeEventListener:$e,appendChild:function(t){return ke(this,t)},insertBefore:function(t,e){return ke(this,t,e)},removeChild:function(t){return Se(this,t)},replaceChild:function(t,e){return ke(this,t,e),Se(this,e),t},cloneNode:function(t){if("template"==this.localName)var e=zt.cloneNode.call(this,t);else if(e=zt.cloneNode.call(this,!1),t&&e.nodeType!==Node.ATTRIBUTE_NODE){t=this.childNodes;for(var n,r=0;r<t.length;r++)n=t[r].cloneNode(!0),e.appendChild(n)}return e},getRootNode:function(){return function t(e){if(e&&e.nodeType){var n=et(e),r=n.V;return void 0===r&&(At(e)?(r=e,n.V=r):(r=(r=e.parentNode)?t(r):e,zt.contains.call(document.documentElement,e)&&(n.V=r))),r}}(this)},contains:function(t){return mt(this,t)},dispatchEvent:function(t){return bt(),zt.dispatchEvent.call(this,t)}};Object.defineProperties(_n,{isConnected:{get:function(){if(pe&&pe.call(this))return!0;if(this.nodeType==Node.DOCUMENT_FRAGMENT_NODE)return!1;var t=this.ownerDocument;if(Ct){if(zt.contains.call(t,this))return!0}else if(t.documentElement&&zt.contains.call(t.documentElement,this))return!0;for(t=this;t&&!(t instanceof Document);)t=t.parentNode||(At(t)?t.host:void 0);return!!(t&&t instanceof Document)},configurable:!0}});var Dn={get assignedSlot(){return wn(this)}},Mn={querySelector:function(t){return Fe(this,function(e){return ut.call(e,t)},function(t){return!!t})[0]||null},querySelectorAll:function(t,e){if(e){e=Array.prototype.slice.call(zt.querySelectorAll.call(this,t));var n=this.getRootNode();return e.filter(function(t){return t.getRootNode()==n})}return Fe(this,function(e){return ut.call(e,t)})}},kn={assignedNodes:function(t){if("slot"===this.localName){var e=this.getRootNode();return At(e)&&pn(e),(e=nt(this))&&(t&&t.flatten?e.N:e.assignedNodes)||[]}}},Sn=ht({setAttribute:function(t,e){Ue(this,t,e)},removeAttribute:function(t){zt.removeAttribute.call(this,t),Re(this,t)},attachShadow:function(t){if(!this)throw"Must provide a host.";if(!t)throw"Not enough arguments.";return new dn(un,this,t)},get slot(){return this.getAttribute("slot")},set slot(t){Ue(this,"slot",t)},get assignedSlot(){return wn(this)}},Mn,kn);Object.defineProperties(Sn,Ee);var Ln=ht({importNode:function(t,e){return function t(e,n){if(e.ownerDocument!==document||"template"===e.localName)return zt.importNode.call(document,e,n);var r=zt.importNode.call(document,e,!1);if(n){e=e.childNodes,n=0;for(var i;n<e.length;n++)i=t(e[n],!0),r.appendChild(i)}return r}(t,e)},getElementById:function(t){return Fe(this,function(e){return e.id==t},function(t){return!!t})[0]||null}},Mn);Object.defineProperties(Ln,{_activeElement:ve.activeElement});for(var Rn=HTMLElement.prototype.blur,Nn={blur:function(){var t=nt(this);(t=(t=t&&t.root)&&t.activeElement)?t.blur():Rn.call(this)}},Fn={},Tn=u(Object.getOwnPropertyNames(Document.prototype)),Un=Tn.next();!Un.done;Fn={H:Fn.H},Un=Tn.next())Fn.H=Un.value,"on"===Fn.H.substring(0,2)&&Object.defineProperty(Nn,Fn.H,{set:function(t){return function(e){var n=et(this),r=t.H.substring(2);n.Z[t.H]&&this.removeEventListener(r,n.Z[t.H]),this.addEventListener(r,e,{}),n.Z[t.H]=e}}(Fn),get:function(t){return function(){var e=nt(this);return e&&e.Z[t.H]}}(Fn),configurable:!0});var On={addEventListener:function(t,e,n){"object"!=typeof n&&(n={capture:!!n}),n.la=this,this.host.addEventListener(t,e,n)},removeEventListener:function(t,e,n){"object"!=typeof n&&(n={capture:!!n}),n.la=this,this.host.removeEventListener(t,e,n)},getElementById:function(t){return Fe(this,function(e){return e.id==t},function(t){return!!t})[0]||null}};function Gn(t,e){for(var n=Object.getOwnPropertyNames(e),r=0;r<n.length;r++){var i=n[r],o=Object.getOwnPropertyDescriptor(e,i);o.value?t[i]=o.value:Object.defineProperty(t,i,o)}}if(rt.oa){var Hn={inUse:rt.oa,patch:function(t){return we(t),Qe(t),t},isShadyRoot:At,enqueue:Bt,flush:bt,settings:rt,filterMutations:function(t,e){var n=e.getRootNode();return t.map(function(t){var e=n===t.target.getRootNode();if(e&&t.addedNodes){if((e=Array.from(t.addedNodes).filter(function(t){return n===t.getRootNode()})).length)return t=Object.create(t),Object.defineProperty(t,"addedNodes",{value:e,configurable:!0}),t}else if(e)return t}).filter(function(t){return t})},observeChildren:function(t,e){var n=et(t);n.S||(n.S=new Qt),n.S.ca.add(e);var r=n.S;return{Ka:e,P:r,Ma:t,takeRecords:function(){return r.takeRecords()}}},unobserveChildren:function(t){var e=t&&t.P;e&&(e.ca.delete(t.Ka),e.ca.size||(et(t.Ma).S=null))},nativeMethods:zt,nativeTree:ue,deferConnectionCallbacks:rt.deferConnectionCallbacks};window.ShadyDOM=Hn,function(){if(window.Event=en,window.CustomEvent=nn,window.MouseEvent=rn,function(){for(var t in Je)window.addEventListener(t,function(t){t.__target||(tn(t),We(t))},!0)}(),!Ge&&Object.getOwnPropertyDescriptor(Event.prototype,"isTrusted")){var t=function(){var t=new MouseEvent("click",{bubbles:!0,cancelable:!0,composed:!0});this.dispatchEvent(t)};Element.prototype.click?Element.prototype.click=t:HTMLElement.prototype.click&&(HTMLElement.prototype.click=t)}}();var jn=window.customElements&&window.customElements.nativeHTMLElement||HTMLElement;Gn(dn.prototype,On),Gn(window.Node.prototype,_n),Gn(window.Window.prototype,xn),Gn(window.Text.prototype,Dn),Gn(window.DocumentFragment.prototype,Mn),Gn(window.Element.prototype,Sn),Gn(window.Document.prototype,Ln),window.HTMLSlotElement&&Gn(window.HTMLSlotElement.prototype,kn),Gn(jn.prototype,Nn),rt.K&&(be(window.Node.prototype),be(window.Text.prototype),be(window.DocumentFragment.prototype),be(window.Element.prototype),be(jn.prototype),be(window.Document.prototype),window.HTMLSlotElement&&be(window.HTMLSlotElement.prototype)),function(){var t=dn.prototype;t.__proto__=DocumentFragment.prototype,Be(t,Ce,!0),Be(t,ye,!0),Be(t,ve,!0),Object.defineProperties(t,{nodeType:{value:Node.DOCUMENT_FRAGMENT_NODE,configurable:!0},nodeName:{value:"#document-fragment",configurable:!0},nodeValue:{value:null,configurable:!0}}),["localName","namespaceURI","prefix"].forEach(function(e){Object.defineProperty(t,e,{value:void 0,configurable:!0})}),["ownerDocument","baseURI","isConnected"].forEach(function(e){Object.defineProperty(t,e,{get:function(){return this.host[e]},configurable:!0})})}(),window.ShadowRoot=dn}var Yn=new Set("annotation-xml color-profile font-face font-face-src font-face-uri font-face-format font-face-name missing-glyph".split(" "));function Pn(t){var e=Yn.has(t);return t=/^[a-z][.0-9_a-z]*-[\-.0-9_a-z]*$/.test(t),!e&&t}function Kn(t){var e=t.isConnected;if(void 0!==e)return e;for(;t&&!(t.__CE_isImportDocument||t instanceof Document);)t=t.parentNode||(window.ShadowRoot&&t instanceof ShadowRoot?t.host:void 0);return!(!t||!(t.__CE_isImportDocument||t instanceof Document))}function zn(t,e){for(;e&&e!==t&&!e.nextSibling;)e=e.parentNode;return e&&e!==t?e.nextSibling:null}function Jn(t,e,n){n=void 0===n?new Set:n;for(var r=t;r;){if(r.nodeType===Node.ELEMENT_NODE){var i=r;e(i);var o=i.localName;if("link"===o&&"import"===i.getAttribute("rel")){if((r=i.import)instanceof Node&&!n.has(r))for(n.add(r),r=r.firstChild;r;r=r.nextSibling)Jn(r,e,n);r=zn(t,i);continue}if("template"===o){r=zn(t,i);continue}if(i=i.__CE_shadowRoot)for(i=i.firstChild;i;i=i.nextSibling)Jn(i,e,n)}r=r.firstChild?r.firstChild:zn(t,r)}}function qn(t,e,n){t[e]=n}function Vn(){this.a=new Map,this.m=new Map,this.f=[],this.c=!1}function Wn(t,e){t.c=!0,t.f.push(e)}function Xn(t,e){t.c&&Jn(e,function(e){return t.b(e)})}function Zn(t,e){var n=[];for(Jn(e,function(t){return n.push(t)}),e=0;e<n.length;e++){var r=n[e];1===r.__CE_state?t.connectedCallback(r):er(t,r)}}function $n(t,e){var n=[];for(Jn(e,function(t){return n.push(t)}),e=0;e<n.length;e++){var r=n[e];1===r.__CE_state&&t.disconnectedCallback(r)}}function tr(t,e,n){var r=(n=void 0===n?{}:n).cb||new Set,i=n.ha||function(e){return er(t,e)},o=[];if(Jn(e,function(e){if("link"===e.localName&&"import"===e.getAttribute("rel")){var n=e.import;n instanceof Node&&(n.__CE_isImportDocument=!0,n.__CE_hasRegistry=!0),n&&"complete"===n.readyState?n.__CE_documentLoadHandled=!0:e.addEventListener("load",function(){var n=e.import;if(!n.__CE_documentLoadHandled){n.__CE_documentLoadHandled=!0;var o=new Set(r);o.delete(n),tr(t,n,{cb:o,ha:i})}})}else o.push(e)},r),t.c)for(e=0;e<o.length;e++)t.b(o[e]);for(e=0;e<o.length;e++)i(o[e])}function er(t,e){if(void 0===e.__CE_state){var n=e.ownerDocument;if((n.defaultView||n.__CE_isImportDocument&&n.__CE_hasRegistry)&&(n=t.a.get(e.localName))){n.constructionStack.push(e);var r=n.constructor;try{try{if(new r!==e)throw Error("The custom element constructor did not produce the element being upgraded.")}finally{n.constructionStack.pop()}}catch(t){throw e.__CE_state=2,t}if(e.__CE_state=1,e.__CE_definition=n,n.attributeChangedCallback)for(n=n.observedAttributes,r=0;r<n.length;r++){var i=n[r],o=e.getAttribute(i);null!==o&&t.attributeChangedCallback(e,i,null,o,null)}Kn(e)&&t.connectedCallback(e)}}}function nr(t){var e=document;this.b=t,this.a=e,this.P=void 0,tr(this.b,this.a),"loading"===this.a.readyState&&(this.P=new MutationObserver(this.c.bind(this)),this.P.observe(this.a,{childList:!0,subtree:!0}))}function rr(t){t.P&&t.P.disconnect()}function ir(){var t=this;this.a=this.A=void 0,this.b=new Promise(function(e){t.a=e,t.A&&e(t.A)})}function or(t){this.c=!1,this.a=t,this.G=new Map,this.f=function(t){return t()},this.b=!1,this.m=[],this.fa=new nr(t)}Vn.prototype.b=function(t){if(this.c&&!t.__CE_patched){t.__CE_patched=!0;for(var e=0;e<this.f.length;e++)this.f[e](t)}},Vn.prototype.connectedCallback=function(t){var e=t.__CE_definition;e.connectedCallback&&e.connectedCallback.call(t)},Vn.prototype.disconnectedCallback=function(t){var e=t.__CE_definition;e.disconnectedCallback&&e.disconnectedCallback.call(t)},Vn.prototype.attributeChangedCallback=function(t,e,n,r,i){var o=t.__CE_definition;o.attributeChangedCallback&&-1<o.observedAttributes.indexOf(e)&&o.attributeChangedCallback.call(t,e,n,r,i)},nr.prototype.c=function(t){var e=this.a.readyState;for("interactive"!==e&&"complete"!==e||rr(this),e=0;e<t.length;e++)for(var n=t[e].addedNodes,r=0;r<n.length;r++)tr(this.b,n[r])},ir.prototype.resolve=function(t){if(this.A)throw Error("Already resolved.");this.A=t,this.a&&this.a(t)},(n=or.prototype).define=function(t,e){var n=this;if(!(e instanceof Function))throw new TypeError("Custom element constructors must be functions.");if(!Pn(t))throw new SyntaxError("The element name '"+t+"' is not valid.");if(this.a.a.get(t))throw Error("A custom element with name '"+t+"' has already been defined.");if(this.c)throw Error("A custom element is already being defined.");this.c=!0;try{var r=function(t){var e=i[t];if(void 0!==e&&!(e instanceof Function))throw Error("The '"+t+"' callback must be a function.");return e},i=e.prototype;if(!(i instanceof Object))throw new TypeError("The custom element constructor's prototype is not an object.");var o=r("connectedCallback"),a=r("disconnectedCallback"),s=r("adoptedCallback"),A=r("attributeChangedCallback"),l=e.observedAttributes||[]}catch(t){return}finally{this.c=!1}e={localName:t,constructor:e,connectedCallback:o,disconnectedCallback:a,adoptedCallback:s,attributeChangedCallback:A,observedAttributes:l,constructionStack:[]},function(t,e,n){t.a.set(e,n),t.m.set(n.constructor,n)}(this.a,t,e),this.m.push(e),this.b||(this.b=!0,this.f(function(){return function(t){if(!1!==t.b){t.b=!1;for(var e=t.m,n=[],r=new Map,i=0;i<e.length;i++)r.set(e[i].localName,[]);for(tr(t.a,document,{ha:function(e){if(void 0===e.__CE_state){var i=e.localName,o=r.get(i);o?o.push(e):t.a.a.get(i)&&n.push(e)}}}),i=0;i<n.length;i++)er(t.a,n[i]);for(;0<e.length;){var o=e.shift();i=o.localName,o=r.get(o.localName);for(var a=0;a<o.length;a++)er(t.a,o[a]);(i=t.G.get(i))&&i.resolve(void 0)}}}(n)}))},n.ha=function(t){tr(this.a,t)},n.get=function(t){if(t=this.a.a.get(t))return t.constructor},n.Da=function(t){if(!Pn(t))return Promise.reject(new SyntaxError("'"+t+"' is not a valid custom element name."));var e=this.G.get(t);return e?e.b:(e=new ir,this.G.set(t,e),this.a.a.get(t)&&!this.m.some(function(e){return e.localName===t})&&e.resolve(void 0),e.b)},n.Za=function(t){rr(this.fa);var e=this.f;this.f=function(n){return t(function(){return e(n)})}},window.CustomElementRegistry=or,or.prototype.define=or.prototype.define,or.prototype.upgrade=or.prototype.ha,or.prototype.get=or.prototype.get,or.prototype.whenDefined=or.prototype.Da,or.prototype.polyfillWrapFlushCallback=or.prototype.Za;var ar=window.Document.prototype.createElement,sr=window.Document.prototype.createElementNS,Ar=window.Document.prototype.importNode,lr=window.Document.prototype.prepend,cr=window.Document.prototype.append,ur=window.DocumentFragment.prototype.prepend,gr=window.DocumentFragment.prototype.append,hr=window.Node.prototype.cloneNode,dr=window.Node.prototype.appendChild,fr=window.Node.prototype.insertBefore,pr=window.Node.prototype.removeChild,Ir=window.Node.prototype.replaceChild,Cr=Object.getOwnPropertyDescriptor(window.Node.prototype,"textContent"),mr=window.Element.prototype.attachShadow,yr=Object.getOwnPropertyDescriptor(window.Element.prototype,"innerHTML"),Er=window.Element.prototype.getAttribute,vr=window.Element.prototype.setAttribute,Br=window.Element.prototype.removeAttribute,br=window.Element.prototype.getAttributeNS,Qr=window.Element.prototype.setAttributeNS,wr=window.Element.prototype.removeAttributeNS,xr=window.Element.prototype.insertAdjacentElement,_r=window.Element.prototype.insertAdjacentHTML,Dr=window.Element.prototype.prepend,Mr=window.Element.prototype.append,kr=window.Element.prototype.before,Sr=window.Element.prototype.after,Lr=window.Element.prototype.replaceWith,Rr=window.Element.prototype.remove,Nr=window.HTMLElement,Fr=Object.getOwnPropertyDescriptor(window.HTMLElement.prototype,"innerHTML"),Tr=window.HTMLElement.prototype.insertAdjacentElement,Ur=window.HTMLElement.prototype.insertAdjacentHTML,Or=new function(){};function Gr(t,e,n){function r(e){return function(n){for(var r=[],i=0;i<arguments.length;++i)r[i]=arguments[i];i=[];for(var o=[],a=0;a<r.length;a++){var s=r[a];if(s instanceof Element&&Kn(s)&&o.push(s),s instanceof DocumentFragment)for(s=s.firstChild;s;s=s.nextSibling)i.push(s);else i.push(s)}for(e.apply(this,r),r=0;r<o.length;r++)$n(t,o[r]);if(Kn(this))for(r=0;r<i.length;r++)(o=i[r])instanceof Element&&Zn(t,o)}}void 0!==n.ga&&(e.prepend=r(n.ga)),void 0!==n.append&&(e.append=r(n.append))}var Hr=window.customElements;if(!Hr||Hr.forcePolyfill||"function"!=typeof Hr.define||"function"!=typeof Hr.get){var jr=new Vn;!function(){var t=jr;window.HTMLElement=function(){function e(){var e=this.constructor,n=t.m.get(e);if(!n)throw Error("The custom element being constructed was not registered with `customElements`.");var r=n.constructionStack;if(0===r.length)return r=ar.call(document,n.localName),Object.setPrototypeOf(r,e.prototype),r.__CE_state=1,r.__CE_definition=n,t.b(r),r;var i=r[n=r.length-1];if(i===Or)throw Error("The HTMLElement constructor was either called reentrantly for this constructor or called multiple times.");return r[n]=Or,Object.setPrototypeOf(i,e.prototype),t.b(i),i}return e.prototype=Nr.prototype,Object.defineProperty(e.prototype,"constructor",{writable:!0,configurable:!0,enumerable:!1,value:e}),e}()}(),function(){var t=jr;qn(Document.prototype,"createElement",function(e){if(this.__CE_hasRegistry){var n=t.a.get(e);if(n)return new n.constructor}return e=ar.call(this,e),t.b(e),e}),qn(Document.prototype,"importNode",function(e,n){return e=Ar.call(this,e,n),this.__CE_hasRegistry?tr(t,e):Xn(t,e),e}),qn(Document.prototype,"createElementNS",function(e,n){if(this.__CE_hasRegistry&&(null===e||"http://www.w3.org/1999/xhtml"===e)){var r=t.a.get(n);if(r)return new r.constructor}return e=sr.call(this,e,n),t.b(e),e}),Gr(t,Document.prototype,{ga:lr,append:cr})}(),Gr(jr,DocumentFragment.prototype,{ga:ur,append:gr}),function(){function t(t,n){Object.defineProperty(t,"textContent",{enumerable:n.enumerable,configurable:!0,get:n.get,set:function(t){if(this.nodeType===Node.TEXT_NODE)n.set.call(this,t);else{var r=void 0;if(this.firstChild){var i=this.childNodes,o=i.length;if(0<o&&Kn(this)){r=Array(o);for(var a=0;a<o;a++)r[a]=i[a]}}if(n.set.call(this,t),r)for(t=0;t<r.length;t++)$n(e,r[t])}}})}var e=jr;qn(Node.prototype,"insertBefore",function(t,n){if(t instanceof DocumentFragment){var r=Array.prototype.slice.apply(t.childNodes);if(t=fr.call(this,t,n),Kn(this))for(n=0;n<r.length;n++)Zn(e,r[n]);return t}return r=Kn(t),n=fr.call(this,t,n),r&&$n(e,t),Kn(this)&&Zn(e,t),n}),qn(Node.prototype,"appendChild",function(t){if(t instanceof DocumentFragment){var n=Array.prototype.slice.apply(t.childNodes);if(t=dr.call(this,t),Kn(this))for(var r=0;r<n.length;r++)Zn(e,n[r]);return t}return n=Kn(t),r=dr.call(this,t),n&&$n(e,t),Kn(this)&&Zn(e,t),r}),qn(Node.prototype,"cloneNode",function(t){return t=hr.call(this,t),this.ownerDocument.__CE_hasRegistry?tr(e,t):Xn(e,t),t}),qn(Node.prototype,"removeChild",function(t){var n=Kn(t),r=pr.call(this,t);return n&&$n(e,t),r}),qn(Node.prototype,"replaceChild",function(t,n){if(t instanceof DocumentFragment){var r=Array.prototype.slice.apply(t.childNodes);if(t=Ir.call(this,t,n),Kn(this))for($n(e,n),n=0;n<r.length;n++)Zn(e,r[n]);return t}r=Kn(t);var i=Ir.call(this,t,n),o=Kn(this);return o&&$n(e,n),r&&$n(e,t),o&&Zn(e,t),i}),Cr&&Cr.get?t(Node.prototype,Cr):Wn(e,function(e){t(e,{enumerable:!0,configurable:!0,get:function(){for(var t=[],e=0;e<this.childNodes.length;e++)t.push(this.childNodes[e].textContent);return t.join("")},set:function(t){for(;this.firstChild;)pr.call(this,this.firstChild);dr.call(this,document.createTextNode(t))}})})}(),function(){function t(t,e){Object.defineProperty(t,"innerHTML",{enumerable:e.enumerable,configurable:!0,get:e.get,set:function(t){var n=this,i=void 0;if(Kn(this)&&(i=[],Jn(this,function(t){t!==n&&i.push(t)})),e.set.call(this,t),i)for(var o=0;o<i.length;o++){var a=i[o];1===a.__CE_state&&r.disconnectedCallback(a)}return this.ownerDocument.__CE_hasRegistry?tr(r,this):Xn(r,this),t}})}function e(t,e){qn(t,"insertAdjacentElement",function(t,n){var i=Kn(n);return t=e.call(this,t,n),i&&$n(r,n),Kn(t)&&Zn(r,n),t})}function n(t,e){function n(t,e){for(var n=[];t!==e;t=t.nextSibling)n.push(t);for(e=0;e<n.length;e++)tr(r,n[e])}qn(t,"insertAdjacentHTML",function(t,r){if("beforebegin"===(t=t.toLowerCase())){var i=this.previousSibling;e.call(this,t,r),n(i||this.parentNode.firstChild,this)}else if("afterbegin"===t)i=this.firstChild,e.call(this,t,r),n(this.firstChild,i);else if("beforeend"===t)i=this.lastChild,e.call(this,t,r),n(i||this.firstChild,null);else{if("afterend"!==t)throw new SyntaxError("The value provided ("+String(t)+") is not one of 'beforebegin', 'afterbegin', 'beforeend', or 'afterend'.");i=this.nextSibling,e.call(this,t,r),n(this.nextSibling,i)}})}var r=jr;mr&&qn(Element.prototype,"attachShadow",function(t){return this.__CE_shadowRoot=mr.call(this,t)}),yr&&yr.get?t(Element.prototype,yr):Fr&&Fr.get?t(HTMLElement.prototype,Fr):Wn(r,function(e){t(e,{enumerable:!0,configurable:!0,get:function(){return hr.call(this,!0).innerHTML},set:function(t){var e="template"===this.localName,n=e?this.content:this,r=sr.call(document,this.namespaceURI,this.localName);for(r.innerHTML=t;0<n.childNodes.length;)pr.call(n,n.childNodes[0]);for(t=e?r.content:r;0<t.childNodes.length;)dr.call(n,t.childNodes[0])}})}),qn(Element.prototype,"setAttribute",function(t,e){if(1!==this.__CE_state)return vr.call(this,t,e);var n=Er.call(this,t);vr.call(this,t,e),e=Er.call(this,t),r.attributeChangedCallback(this,t,n,e,null)}),qn(Element.prototype,"setAttributeNS",function(t,e,n){if(1!==this.__CE_state)return Qr.call(this,t,e,n);var i=br.call(this,t,e);Qr.call(this,t,e,n),n=br.call(this,t,e),r.attributeChangedCallback(this,e,i,n,t)}),qn(Element.prototype,"removeAttribute",function(t){if(1!==this.__CE_state)return Br.call(this,t);var e=Er.call(this,t);Br.call(this,t),null!==e&&r.attributeChangedCallback(this,t,e,null,null)}),qn(Element.prototype,"removeAttributeNS",function(t,e){if(1!==this.__CE_state)return wr.call(this,t,e);var n=br.call(this,t,e);wr.call(this,t,e);var i=br.call(this,t,e);n!==i&&r.attributeChangedCallback(this,e,n,i,t)}),Tr?e(HTMLElement.prototype,Tr):xr?e(Element.prototype,xr):console.warn("Custom Elements: `Element#insertAdjacentElement` was not patched."),Ur?n(HTMLElement.prototype,Ur):_r?n(Element.prototype,_r):console.warn("Custom Elements: `Element#insertAdjacentHTML` was not patched."),Gr(r,Element.prototype,{ga:Dr,append:Mr}),function(t){function e(e){return function(n){for(var r=[],i=0;i<arguments.length;++i)r[i]=arguments[i];i=[];for(var o=[],a=0;a<r.length;a++){var s=r[a];if(s instanceof Element&&Kn(s)&&o.push(s),s instanceof DocumentFragment)for(s=s.firstChild;s;s=s.nextSibling)i.push(s);else i.push(s)}for(e.apply(this,r),r=0;r<o.length;r++)$n(t,o[r]);if(Kn(this))for(r=0;r<i.length;r++)(o=i[r])instanceof Element&&Zn(t,o)}}var n=Element.prototype;void 0!==kr&&(n.before=e(kr)),void 0!==kr&&(n.after=e(Sr)),void 0!==Lr&&qn(n,"replaceWith",function(e){for(var n=[],r=0;r<arguments.length;++r)n[r]=arguments[r];r=[];for(var i=[],o=0;o<n.length;o++){var a=n[o];if(a instanceof Element&&Kn(a)&&i.push(a),a instanceof DocumentFragment)for(a=a.firstChild;a;a=a.nextSibling)r.push(a);else r.push(a)}for(o=Kn(this),Lr.apply(this,n),n=0;n<i.length;n++)$n(t,i[n]);if(o)for($n(t,this),n=0;n<r.length;n++)(i=r[n])instanceof Element&&Zn(t,i)}),void 0!==Rr&&qn(n,"remove",function(){var e=Kn(this);Rr.call(this),e&&$n(t,this)})}(r)}(),document.__CE_hasRegistry=!0;var Yr=new or(jr);Object.defineProperty(window,"customElements",{configurable:!0,enumerable:!0,value:Yr})}function Pr(){this.end=this.start=0,this.rules=this.parent=this.previous=null,this.cssText=this.parsedCssText="",this.atRule=!1,this.type=0,this.parsedSelector=this.selector=this.keyframesName=""}function Kr(t){var e=zr,n=t=t.replace(Zr,"").replace($r,""),r=new Pr;r.start=0,r.end=n.length;for(var i=r,o=0,a=n.length;o<a;o++)if("{"===n[o]){i.rules||(i.rules=[]);var s=i,A=s.rules[s.rules.length-1]||null;(i=new Pr).start=o+1,i.parent=s,i.previous=A,s.rules.push(i)}else"}"===n[o]&&(i.end=o+1,i=i.parent||r);return e(r,t)}function zr(t,e){var n=e.substring(t.start,t.end-1);if(t.parsedCssText=t.cssText=n.trim(),t.parent&&(n=(n=(n=function(t){return t.replace(/\\([0-9a-f]{1,6})\s/gi,function(t,e){for(e=6-(t=e).length;e--;)t="0"+t;return"\\"+t})}(n=e.substring(t.previous?t.previous.end:t.parent.start,t.start-1))).replace(oi," ")).substring(n.lastIndexOf(";")+1),n=t.parsedSelector=t.selector=n.trim(),t.atRule=0===n.indexOf("@"),t.atRule?0===n.indexOf("@media")?t.type=Wr:n.match(ii)&&(t.type=Vr,t.keyframesName=t.selector.split(oi).pop()):t.type=0===n.indexOf("--")?Xr:qr),n=t.rules)for(var r,i=0,o=n.length;i<o&&(r=n[i]);i++)zr(r,e);return t}var Jr,qr=1,Vr=7,Wr=4,Xr=1e3,Zr=/\/\*[^*]*\*+([^\/*][^*]*\*+)*\//gim,$r=/@import[^;]*;/gim,ti=/(?:^[^;\-\s}]+)?--[^;{}]*?:[^{};]*?(?:[;\n]|$)/gim,ei=/(?:^[^;\-\s}]+)?--[^;{}]*?:[^{};]*?{[^}]*?}(?:[;\n]|$)?/gim,ni=/@apply\s*\(?[^);]*\)?\s*(?:[;\n]|$)?/gim,ri=/[^;:]*?:[^;]*?var\([^;]*\)(?:[;\n]|$)?/gim,ii=/^@[^\s]*keyframes/,oi=/\s+/g,ai=!(window.ShadyDOM&&window.ShadyDOM.inUse);function si(t){Jr=(!t||!t.shimcssproperties)&&(ai||!(navigator.userAgent.match(/AppleWebKit\/601|Edge\/15/)||!window.CSS||!CSS.supports||!CSS.supports("box-shadow","0 0 0 var(--foo)")))}window.ShadyCSS&&void 0!==window.ShadyCSS.nativeCss?Jr=window.ShadyCSS.nativeCss:window.ShadyCSS?(si(window.ShadyCSS),window.ShadyCSS=void 0):si(window.WebComponents&&window.WebComponents.flags);var Ai=Jr,li=/(?:^|[;\s{]\s*)(--[\w-]*?)\s*:\s*(?:((?:'(?:\\'|.)*?'|"(?:\\"|.)*?"|\([^)]*?\)|[^};{])+)|\{([^}]*)\}(?:(?=[;\s}])|$))/gi,ci=/(?:^|\W+)@apply\s*\(?([^);\n]*)\)?/gi,ui=/(--[\w-]+)\s*([:,;)]|$)/gi,gi=/(animation\s*:)|(animation-name\s*:)/,hi=/@media\s(.*)/,di=/\{[^}]*\}/g,fi=new Set;function pi(t,e){return t?("string"==typeof t&&(t=Kr(t)),e&&mi(t,e),function t(e,n,r){r=void 0===r?"":r;var i="";if(e.cssText||e.rules){var o,a=e.rules;if((o=a)&&(o=!((o=a[0])&&o.selector&&0===o.selector.indexOf("--"))),o){o=0;for(var s,A=a.length;o<A&&(s=a[o]);o++)i=t(s,n,i)}else(i=(n=n?e.cssText:(n=(n=e.cssText).replace(ti,"").replace(ei,"")).replace(ni,"").replace(ri,"")).trim())&&(i="  "+i+"\n")}return i&&(e.selector&&(r+=e.selector+" {\n"),r+=i,e.selector&&(r+="}\n\n")),r}(t,Ai)):""}function Ii(t){return!t.__cssRules&&t.textContent&&(t.__cssRules=Kr(t.textContent)),t.__cssRules||null}function Ci(t){return!!t.parent&&t.parent.type===Vr}function mi(t,e,n,r){if(t){var i=!1,o=t.type;if(r&&o===Wr){var a=t.selector.match(hi);a&&(window.matchMedia(a[1]).matches||(i=!0))}if(o===qr?e(t):n&&o===Vr?n(t):o===Xr&&(i=!0),(t=t.rules)&&!i){i=0,o=t.length;for(var s;i<o&&(s=t[i]);i++)mi(s,e,n,r)}}}function yi(t,e,n,r){var i=document.createElement("style");return e&&i.setAttribute("scope",e),i.textContent=t,Bi(i,n,r),i}var Ei=null;function vi(t){t=document.createComment(" Shady DOM styles for "+t+" ");var e=document.head;return e.insertBefore(t,(Ei?Ei.nextSibling:null)||e.firstChild),Ei=t}function Bi(t,e,n){(e=e||document.head).insertBefore(t,n&&n.nextSibling||e.firstChild),Ei?t.compareDocumentPosition(Ei)===Node.DOCUMENT_POSITION_PRECEDING&&(Ei=t):Ei=t}function bi(t,e){for(var n=0,r=t.length;e<r;e++)if("("===t[e])n++;else if(")"===t[e]&&0==--n)return e;return-1}function Qi(t,e){ai?t.setAttribute("class",e):window.ShadyDOM.nativeMethods.setAttribute.call(t,"class",e)}function wi(t){var e=t.localName,n="";return e?-1<e.indexOf("-")||(n=e,e=t.getAttribute&&t.getAttribute("is")||""):(e=t.is,n=t.extends),{is:e,X:n}}function xi(t){for(var e=[],n="",r=0;0<=r&&r<t.length;r++)if("("===t[r]){var i=bi(t,r);n+=t.slice(r,i+1),r=i}else","===t[r]?(e.push(n),n=""):n+=t[r];return n&&e.push(n),e}function _i(){}function Di(t,e,n){if(e.nodeType===Node.ELEMENT_NODE&&n(e),e="template"===e.localName?(e.content||e.kb||e).childNodes:e.children||e.childNodes)for(var r=0;r<e.length;r++)Di(t,e[r],n)}function Mi(t,e,n){if(e)if(t.classList)n?(t.classList.remove("style-scope"),t.classList.remove(e)):(t.classList.add("style-scope"),t.classList.add(e));else if(t.getAttribute){var r=t.getAttribute($i);n?r&&Qi(t,e=r.replace("style-scope","").replace(e,"")):Qi(t,(r?r+" ":"")+"style-scope "+e)}}function ki(t,e,n){var r=no;t.__styleScoped?t.__styleScoped=null:Di(r,t,function(t){Mi(t,e,!0),Mi(t,n)})}function Si(t,e){var n=no;t.__styleScoped?t.__styleScoped=null:Di(n,t,function(t){Mi(t,e||"",!0)})}function Li(t,e,n){var r=no,i=t.__cssBuild;return(e=ai||"shady"===i?pi(e,n):function(t,e,n,r,i){var o=Ri(n,r);return n=n?Xi+n:"",pi(e,function(e){e.c||(e.selector=e.F=Ni(t,e,t.b,n,o),e.c=!0),i&&i(e,n,o)})}(r,e,(t=wi(t)).is,t.X,n)+"\n\n").trim()}function Ri(t,e){return e?"[is="+t+"]":t}function Ni(t,e,n,r,i){var o=xi(e.selector);if(!Ci(e)){e=0;for(var a,s=o.length;e<s&&(a=o[e]);e++)o[e]=n.call(t,a,r,i)}return o.filter(function(t){return!!t}).join(Hi)}function Fi(t){return t.replace(Oi,function(t,e,n){return-1<n.indexOf("+")?n=n.replace(/\+/g,"___"):-1<n.indexOf("___")&&(n=n.replace(/___/g,"+")),":"+e+"("+n+")"})}function Ti(t,e){return(t=t.split(Zi))[0]+=e,t.join(Zi)}function Ui(t){t.selector===Ki&&(t.selector="html")}_i.prototype.b=function(t,e,n){var r=!1;t=t.trim();var i=Oi.test(t);i&&(t=Fi(t=t.replace(Oi,function(t,e,n){return":"+e+"("+n.replace(/\s/g,"")+")"})));var o=eo.test(t);if(o){var a=function(t){for(var e,n=[];e=t.match(eo);){var r=e.index,i=bi(t,r);if(-1===i)throw Error(e.input+" selector missing ')'");e=t.slice(r,i+1),t=t.replace(e,""),n.push(e)}return{ra:t,matches:n}}(t);t=a.ra,a=a.matches}return t=(t=t.replace(Ji,Pi+" $1")).replace(ji,function(t,i,o){return r||(t=function(t,e,n,r){var i=t.indexOf(zi);if(0<=t.indexOf(Pi)?t=function(t,e){var n=t.match(qi);return(n=n&&n[2].trim()||"")?n[0].match(Yi)?t.replace(qi,function(t,n,r){return e+r}):n.split(Yi)[0]===e?n:to:t.replace(Pi,e)}(t,r):0!==i&&(t=n?Ti(t,n):t),n=!1,0<=i&&(e="",n=!0),n){var o=!0;n&&(t=t.replace(Vi,function(t,e){return" > "+e}))}return{value:t=t.replace(Wi,function(t,e,n){return'[dir="'+n+'"] '+e+", "+e+'[dir="'+n+'"]'}),Qa:e,stop:o}}(o,i,e,n),r=r||t.stop,i=t.Qa,o=t.value),i+o}),o&&(t=function(t,e){var n=t.split("");return e.reduce(function(t,e,r){return t+e+n[r+1]},n[0])}(t,a)),i&&(t=Fi(t)),t},_i.prototype.c=function(t){return t.match(Pi)?"":t.match(zi)?this.b(t,Gi):Ti(t.trim(),Gi)},i.Object.defineProperties(_i.prototype,{a:{configurable:!0,enumerable:!0,get:function(){return"style-scope"}}});var Oi=/:(nth[-\w]+)\(([^)]+)\)/,Gi=":not(.style-scope)",Hi=",",ji=/(^|[\s>+~]+)((?:\[.+?\]|[^\s>+~=[])+)/g,Yi=/[[.:#*]/,Pi=":host",Ki=":root",zi="::slotted",Ji=new RegExp("^("+zi+")"),qi=/(:host)(?:\(((?:\([^)(]*\)|[^)(]*)+?)\))/,Vi=/(?:::slotted)(?:\(((?:\([^)(]*\)|[^)(]*)+?)\))/,Wi=/(.*):dir\((?:(ltr|rtl))\)/,Xi=".",Zi=":",$i="class",to="should_not_match",eo=/:(?:matches|any|-(?:webkit|moz)-any)/,no=new _i;function ro(t,e,n,r){this.M=t||null,this.b=e||null,this.pa=n||[],this.T=null,this.X=r||"",this.a=this.I=this.O=null}function io(t){return t?t.__styleInfo:null}function oo(t,e){return t.__styleInfo=e}function ao(t){var e=this.matches||this.matchesSelector||this.mozMatchesSelector||this.msMatchesSelector||this.oMatchesSelector||this.webkitMatchesSelector;return e&&e.call(this,t)}ro.prototype.c=function(){return this.M},ro.prototype._getStyleRules=ro.prototype.c;var so=navigator.userAgent.match("Trident");function Ao(){}function lo(t){if(!t.B){var e={},n={};co(t,n)&&(e.L=n,t.rules=null),e.cssText=t.parsedCssText.replace(di,"").replace(li,""),t.B=e}}function co(t,e){var n=t.B;if(!n){n=t.parsedCssText;for(var r;t=li.exec(n);)"inherit"===(r=(t[2]||t[3]).trim())&&"unset"===r||(e[t[1].trim()]=r),r=!0;return r}if(n.L)return Object.assign(e,n.L),!0}function uo(t,e,n){return e&&(e=0<=e.indexOf(";")?go(t,e,n):function t(e,n){var r=e.indexOf("var(");if(-1===r)return n(e,"","","");var i=bi(e,r+3),o=e.substring(r+4,i);return r=e.substring(0,r),e=t(e.substring(i+1),n),-1===(i=o.indexOf(","))?n(r,o.trim(),"",e):n(r,o.substring(0,i).trim(),o.substring(i+1).trim(),e)}(e,function(e,r,i,o){return r?((r=uo(t,n[r],n))&&"initial"!==r?"apply-shim-inherit"===r&&(r="inherit"):r=uo(t,n[i]||i,n)||i,e+(r||"")+o):e+o})),e&&e.trim()||""}function go(t,e,n){e=e.split(";");for(var r,i,o=0;o<e.length;o++)if(r=e[o]){if(ci.lastIndex=0,i=ci.exec(r))r=uo(t,n[i[1]],n);else if(-1!==(i=r.indexOf(":"))){var a=r.substring(i);a=uo(t,a=a.trim(),n)||a,r=r.substring(0,i)+a}e[o]=r&&r.lastIndexOf(";")===r.length-1?r.slice(0,-1):r||""}return e.join(";")}function ho(t){return function(e){return e.replace(t.f,t.a)}}function fo(t,e){var n=po,r=Ii(t);t.textContent=pi(r,function(t){var r=t.cssText=t.parsedCssText;t.B&&t.B.cssText&&(r=r.replace(ti,"").replace(ei,""),t.cssText=go(n,r,e))})}i.Object.defineProperties(Ao.prototype,{a:{configurable:!0,enumerable:!0,get:function(){return"x-scope"}}});var po=new Ao,Io={},Co=window.customElements;if(Co&&!ai){var mo=Co.define;Co.define=function(t,e,n){Io[t]||(Io[t]=vi(t)),mo.call(Co,t,e,n)}}function yo(){this.cache={}}function Eo(){}function vo(t){for(var e=0;e<t.length;e++){var n=t[e];if(n.target!==document.documentElement&&n.target!==document.head)for(var r=0;r<n.addedNodes.length;r++){var i=n.addedNodes[r];if(i.nodeType===Node.ELEMENT_NODE){var o=i.getRootNode(),a=i,s=[];if(a.classList?s=Array.from(a.classList):a instanceof window.SVGElement&&a.hasAttribute("class")&&(s=a.getAttribute("class").split(/\s+/)),s=(a=s).indexOf(no.a),(a=-1<s?a[s+1]:"")&&o===i.ownerDocument)Si(i,a);else if(o.nodeType===Node.DOCUMENT_FRAGMENT_NODE&&(o=o.host))for((o=wi(o).is)!==a&&ki(i,a,o),i=window.ShadyDOM.nativeMethods.querySelectorAll.call(i,":not(."+no.a+")"),o=0;o<i.length;o++)(s=(a=i[o]).getRootNode().host)&&Mi(a,s=wi(s).is)}}}}if(yo.prototype.store=function(t,e,n,r){var i=this.cache[t]||[];i.push({L:e,styleElement:n,I:r}),100<i.length&&i.shift(),this.cache[t]=i},yo.prototype.fetch=function(t,e,n){if(t=this.cache[t])for(var r=t.length-1;0<=r;r--){var i,o=t[r];t:{for(i=0;i<n.length;i++){var a=n[i];if(o.L[a]!==e[a]){i=!1;break t}}i=!0}if(i)return o}},!ai){var Bo=new MutationObserver(vo),bo=function(t){Bo.observe(t,{childList:!0,subtree:!0})};if(window.customElements&&!window.customElements.polyfillWrapFlushCallback)bo(document);else{var Qo=function(){bo(document.body)};window.HTMLImports?window.HTMLImports.whenReady(Qo):requestAnimationFrame(function(){if("loading"===document.readyState){var t=function(){Qo(),document.removeEventListener("readystatechange",t)};document.addEventListener("readystatechange",t)}else Qo()})}Eo=function(){vo(Bo.takeRecords())}}var wo=Eo,xo={},_o=Promise.resolve();function Do(t){(t=xo[t])&&(t._applyShimCurrentVersion=t._applyShimCurrentVersion||0,t._applyShimValidatingVersion=t._applyShimValidatingVersion||0,t._applyShimNextVersion=(t._applyShimNextVersion||0)+1)}function Mo(t){return t._applyShimCurrentVersion===t._applyShimNextVersion}var ko=new yo;function So(){this.G={},this.c=document.documentElement;var t=new Pr;t.rules=[],this.f=oo(this.c,new ro(t)),this.m=!1,this.b=this.a=null}function Lo(t){!t.a&&window.ShadyCSS&&window.ShadyCSS.ApplyShim&&(t.a=window.ShadyCSS.ApplyShim,t.a.invalidCallback=Do),function(t){!t.b&&window.ShadyCSS&&window.ShadyCSS.CustomStyleInterface&&(t.b=window.ShadyCSS.CustomStyleInterface,t.b.transformCallback=function(e){t.Ba(e)},t.b.validateCallback=function(){requestAnimationFrame(function(){(t.b.enqueued||t.m)&&t.flushCustomStyles()})})}(t)}function Ro(t,e){return(e=e.getRootNode().host)?io(e)?e:Ro(t,e):t.c}function No(t,e,n){var r=io(t=Ro(t,e));t=Object.create(r.O||null);var i=function(t,e){var n={},r={},i=e&&e.__cssBuild;return mi(e,function(e){!function(t,e,n,r){if(e.B||lo(e),e.B.L){var i=wi(t);t=i.is,i=i.X,i=t?Ri(t,i):"html";var o=e.parsedSelector,a=":host > *"===o||"html"===o,s=0===o.indexOf(":host")&&!a;"shady"===n&&(s=!(a=o===i+" > *."+i||-1!==o.indexOf("html"))&&0===o.indexOf(i)),"shadow"===n&&(a=":host > *"===o||"html"===o,s=s&&!a),(a||s)&&(n=i,s&&(e.F||(e.F=Ni(no,e,no.b,t?Xi+t:"",i)),n=e.F||i),r({ra:n,Wa:s,xb:a}))}}(t,e,i,function(i){ao.call(t.lb||t,i.ra)&&(i.Wa?co(e,n):co(e,r))})},null,!0),{$a:r,Ua:n}}(e,n.M);for(var o in e=function(t,e){var n={},r=[];return mi(t,function(t){t.B||lo(t);var i=t.F||t.parsedSelector;e&&t.B.L&&i&&ao.call(e,i)&&(co(t,n),t=t.index,i=parseInt(t/32,10),r[i]=(r[i]||0)|1<<t%32)},null,!0),{L:n,key:r}}(r.M,e).L,Object.assign(t,i.Ua,e,i.$a),e=n.T)((i=e[o])||0===i)&&(t[o]=i);for(o=po,e=Object.getOwnPropertyNames(t),i=0;i<e.length;i++)t[r=e[i]]=uo(o,t[r],t);n.O=t}(n=So.prototype).flush=function(){wo()},n.Sa=function(t){return Ii(t)},n.bb=function(t){return pi(t)},n.prepareTemplate=function(t,e,n){this.prepareTemplateDom(t,e),this.prepareTemplateStyles(t,e,n)},n.prepareTemplateStyles=function(t,e,n){if(!t.m){ai||Io[e]||(Io[e]=vi(e)),t.m=!0,t.name=e,t.extends=n,xo[e]=t;for(var r=(r=t.content.querySelector("style"))&&r.getAttribute("css-build")||"",i=[],o=t.content.querySelectorAll("style"),a=0;a<o.length;a++){var s=o[a];if(s.hasAttribute("shady-unscoped")){if(!ai){var A=s.textContent;fi.has(A)||(fi.add(A),A=s.cloneNode(!0),document.head.appendChild(A)),s.parentNode.removeChild(s)}}else i.push(s.textContent),s.parentNode.removeChild(s)}i=i.join("").trim(),n={is:e,extends:n,ib:r},Lo(this),o=ci.test(i)||li.test(i),ci.lastIndex=0,li.lastIndex=0,i=Kr(i),o&&Ai&&this.a&&this.a.transformRules(i,e),t._styleAst=i,t.G=r,r=[],Ai||(r=function(t){var e={},n=[],r=0;for(var i in mi(t,function(t){lo(t),t.index=r++,t=t.B.cssText;for(var n;n=ui.exec(t);){var i=n[1];":"!==n[2]&&(e[i]=!0)}},function(t){n.push(t)}),t.b=n,t=[],e)t.push(i);return t}(t._styleAst)),r.length&&!Ai||(i=ai?t.content:null,e=Io[e]||null,e=(o=Li(n,t._styleAst)).length?yi(o,n.is,i,e):void 0,t.a=e),t.f=r}},n.prepareTemplateDom=function(t,e){ai||t.c||(t.c=!0,function(t,e){var n=no;t.__styleScoped?t.__styleScoped=null:Di(n,t,function(t){Mi(t,e||"")})}(t.content,e))},n.flushCustomStyles=function(){if(Lo(this),this.b){var t=this.b.processStyles();if(this.b.enqueued){if(Ai)for(var e=0;e<t.length;e++){var n=this.b.getStyleForCustomStyle(t[e]);if(n&&Ai&&this.a){var r=Ii(n);Lo(this),this.a.transformRules(r),n.textContent=pi(r)}}else for(No(this,this.c,this.f),e=0;e<t.length;e++)(n=this.b.getStyleForCustomStyle(t[e]))&&fo(n,this.f.O);this.b.enqueued=!1,this.m&&!Ai&&this.styleDocument()}}},n.styleElement=function(t,e){var n=wi(t).is,r=io(t);if(!r){var i=wi(t);r=i.is,i=i.X;var o=Io[r]||null;if(r=xo[r])var a=r._styleAst,s=r.f;a=new ro(a,o,s,i),r&&oo(t,a),r=a}if(t!==this.c&&(this.m=!0),e&&(r.T=r.T||{},Object.assign(r.T,e)),Ai){if(r.T)for(var A in e=r.T)null===A?t.style.removeProperty(A):t.style.setProperty(A,e[A]);((A=xo[n])||t===this.c)&&A&&A.a&&!Mo(A)&&((Mo(A)||A._applyShimValidatingVersion!==A._applyShimNextVersion)&&(Lo(this),this.a&&this.a.transformRules(A._styleAst,n),A.a.textContent=Li(t,r.M),function(t){t._applyShimValidatingVersion=t._applyShimNextVersion,t.b||(t.b=!0,_o.then(function(){t._applyShimCurrentVersion=t._applyShimNextVersion,t.b=!1}))}(A)),ai&&(n=t.shadowRoot)&&(n.querySelector("style").textContent=Li(t,r.M)),r.M=A._styleAst)}else if(this.flush(),No(this,t,r),r.pa&&r.pa.length){n=r,A=wi(t).is,r=(e=ko.fetch(A,n.O,n.pa))?e.styleElement:null,a=n.I,(s=e&&e.I)||(s=A+"-"+(s=this.G[A]=(this.G[A]||0)+1)),n.I=s,s=n.I,i=po,i=r?r.textContent||"":function(t,e,n,r){var i=wi(e),o=Ri(i.is,i.X),a=new RegExp("(?:^|[^.#[:])"+(e.extends?"\\"+o.slice(0,-1)+"\\]":o)+"($|[.:[\\s>+~])"),s=function(t,e){t=t.b;var n={};if(!ai&&t)for(var r=0,i=t[r];r<t.length;i=t[++r]){var o=i,a=e;o.f=new RegExp("\\b"+o.keyframesName+"(?!\\B|-)","g"),o.a=o.keyframesName+"-"+a,o.F=o.F||o.selector,o.selector=o.F.replace(o.keyframesName,o.a),n[i.keyframesName]=ho(i)}return n}(i=io(e).M,r);return Li(e,i,function(e){var i="";if(e.B||lo(e),e.B.cssText&&(i=go(t,e.B.cssText,n)),e.cssText=i,!ai&&!Ci(e)&&e.cssText){var A=i=e.cssText;if(null==e.ya&&(e.ya=gi.test(i)),e.ya)if(null==e.ea)for(var l in e.ea=[],s)i!==(A=(A=s[l])(i))&&(i=A,e.ea.push(l));else{for(l=0;l<e.ea.length;++l)i=(A=s[e.ea[l]])(i);A=i}e.cssText=A,e.F=e.F||e.selector,i="."+r,A=0;for(var c,u=(l=xi(e.F)).length;A<u&&(c=l[A]);A++)l[A]=c.match(a)?c.replace(o,i):i+" "+c;e.selector=l.join(",")}})}(i,t,n.O,s);var l=(o=io(t)).a;l&&!ai&&l!==r&&(l._useCount--,0>=l._useCount&&l.parentNode&&l.parentNode.removeChild(l)),ai?o.a?(o.a.textContent=i,r=o.a):i&&(r=yi(i,s,t.shadowRoot,o.b)):r?r.parentNode||(so&&-1<i.indexOf("@media")&&(r.textContent=i),Bi(r,null,o.b)):i&&(r=yi(i,s,null,o.b)),r&&(r._useCount=r._useCount||0,o.a!=r&&r._useCount++,o.a=r),s=r,ai||(r=n.I,o=i=t.getAttribute("class")||"",a&&(o=i.replace(new RegExp("\\s*x-scope\\s*"+a+"\\s*","g")," ")),i!==(o+=(o?" ":"")+"x-scope "+r)&&Qi(t,o)),e||ko.store(A,n.O,s,n.I)}},n.styleDocument=function(t){this.styleSubtree(this.c,t)},n.styleSubtree=function(t,e){var n=t.shadowRoot;if((n||t===this.c)&&this.styleElement(t,e),e=n&&(n.children||n.childNodes))for(t=0;t<e.length;t++)this.styleSubtree(e[t]);else if(t=t.children||t.childNodes)for(e=0;e<t.length;e++)this.styleSubtree(t[e])},n.Ba=function(t){var e=this,n=Ii(t);mi(n,function(t){if(ai)Ui(t);else{var n=no;t.selector=t.parsedSelector,Ui(t),t.selector=t.F=Ni(n,t,n.c,void 0,void 0)}Ai&&(Lo(e),e.a&&e.a.transformRule(t))}),Ai?t.textContent=pi(n):this.f.M.rules.push(n)},n.getComputedStyleValue=function(t,e){var n;return Ai||(n=(io(t)||io(Ro(this,t))).O[e]),(n=n||window.getComputedStyle(t).getPropertyValue(e))?n.trim():""},n.ab=function(t,e){var n=t.getRootNode();if(e=e?e.split(/\s/):[],!(n=n.host&&n.host.localName)){var r=t.getAttribute("class");if(r){r=r.split(/\s/);for(var i=0;i<r.length;i++)if(r[i]===no.a){n=r[i+1];break}}}n&&e.push(no.a,n),Ai||(n=io(t))&&n.I&&e.push(po.a,n.I),Qi(t,e.join(" "))},n.Na=function(t){return io(t)},So.prototype.flush=So.prototype.flush,So.prototype.prepareTemplate=So.prototype.prepareTemplate,So.prototype.styleElement=So.prototype.styleElement,So.prototype.styleDocument=So.prototype.styleDocument,So.prototype.styleSubtree=So.prototype.styleSubtree,So.prototype.getComputedStyleValue=So.prototype.getComputedStyleValue,So.prototype.setElementClass=So.prototype.ab,So.prototype._styleInfoForNode=So.prototype.Na,So.prototype.transformCustomStyleForDocument=So.prototype.Ba,So.prototype.getStyleAst=So.prototype.Sa,So.prototype.styleAstToString=So.prototype.bb,So.prototype.flushCustomStyles=So.prototype.flushCustomStyles,Object.defineProperties(So.prototype,{nativeShadow:{get:function(){return ai}},nativeCss:{get:function(){return Ai}}});var Fo,To,Uo=new So;window.ShadyCSS&&(Fo=window.ShadyCSS.ApplyShim,To=window.ShadyCSS.CustomStyleInterface),window.ShadyCSS={ScopingShim:Uo,prepareTemplate:function(t,e,n){Uo.flushCustomStyles(),Uo.prepareTemplate(t,e,n)},prepareTemplateDom:function(t,e){Uo.prepareTemplateDom(t,e)},prepareTemplateStyles:function(t,e,n){Uo.flushCustomStyles(),Uo.prepareTemplateStyles(t,e,n)},styleSubtree:function(t,e){Uo.flushCustomStyles(),Uo.styleSubtree(t,e)},styleElement:function(t){Uo.flushCustomStyles(),Uo.styleElement(t)},styleDocument:function(t){Uo.flushCustomStyles(),Uo.styleDocument(t)},flushCustomStyles:function(){Uo.flushCustomStyles()},getComputedStyleValue:function(t,e){return Uo.getComputedStyleValue(t,e)},nativeCss:Ai,nativeShadow:ai},Fo&&(window.ShadyCSS.ApplyShim=Fo),To&&(window.ShadyCSS.CustomStyleInterface=To),function(t){function e(t){return""==t&&(o.call(this),this.i=!0),t.toLowerCase()}function n(t){var e=t.charCodeAt(0);return 32<e&&127>e&&-1==[34,35,60,62,63,96].indexOf(e)?t:encodeURIComponent(t)}function r(t){var e=t.charCodeAt(0);return 32<e&&127>e&&-1==[34,35,60,62,96].indexOf(e)?t:encodeURIComponent(t)}function i(t,i,a){function s(t){I.push(t)}var A=i||"scheme start",h=0,d="",f=!1,p=!1,I=[];t:for(;(null!=t[h-1]||0==h)&&!this.i;){var C=t[h];switch(A){case"scheme start":if(!C||!u.test(C)){if(i){s("Invalid scheme.");break t}d="",A="no scheme";continue}d+=C.toLowerCase(),A="scheme";break;case"scheme":if(C&&g.test(C))d+=C.toLowerCase();else{if(":"!=C){if(i){null!=C&&s("Code point not allowed in scheme: "+C);break t}d="",h=0,A="no scheme";continue}if(this.h=d,d="",i)break t;void 0!==l[this.h]&&(this.D=!0),A="file"==this.h?"relative":this.D&&a&&a.h==this.h?"relative or authority":this.D?"authority first slash":"scheme data"}break;case"scheme data":"?"==C?(this.u="?",A="query"):"#"==C?(this.C="#",A="fragment"):null!=C&&"\t"!=C&&"\n"!=C&&"\r"!=C&&(this.na+=n(C));break;case"no scheme":if(a&&void 0!==l[a.h]){A="relative";continue}s("Missing scheme."),o.call(this),this.i=!0;break;case"relative or authority":if("/"!=C||"/"!=t[h+1]){s("Expected /, got: "+C),A="relative";continue}A="authority ignore slashes";break;case"relative":if(this.D=!0,"file"!=this.h&&(this.h=a.h),null==C){this.j=a.j,this.s=a.s,this.l=a.l.slice(),this.u=a.u,this.v=a.v,this.g=a.g;break t}if("/"==C||"\\"==C)"\\"==C&&s("\\ is an invalid code point."),A="relative slash";else if("?"==C)this.j=a.j,this.s=a.s,this.l=a.l.slice(),this.u="?",this.v=a.v,this.g=a.g,A="query";else{if("#"!=C){A=t[h+1];var m=t[h+2];("file"!=this.h||!u.test(C)||":"!=A&&"|"!=A||null!=m&&"/"!=m&&"\\"!=m&&"?"!=m&&"#"!=m)&&(this.j=a.j,this.s=a.s,this.v=a.v,this.g=a.g,this.l=a.l.slice(),this.l.pop()),A="relative path";continue}this.j=a.j,this.s=a.s,this.l=a.l.slice(),this.u=a.u,this.C="#",this.v=a.v,this.g=a.g,A="fragment"}break;case"relative slash":if("/"!=C&&"\\"!=C){"file"!=this.h&&(this.j=a.j,this.s=a.s,this.v=a.v,this.g=a.g),A="relative path";continue}"\\"==C&&s("\\ is an invalid code point."),A="file"==this.h?"file host":"authority ignore slashes";break;case"authority first slash":if("/"!=C){s("Expected '/', got: "+C),A="authority ignore slashes";continue}A="authority second slash";break;case"authority second slash":if(A="authority ignore slashes","/"!=C){s("Expected '/', got: "+C);continue}break;case"authority ignore slashes":if("/"!=C&&"\\"!=C){A="authority";continue}s("Expected authority, got: "+C);break;case"authority":if("@"==C){for(f&&(s("@ already seen."),d+="%40"),f=!0,C=0;C<d.length;C++)"\t"==(m=d[C])||"\n"==m||"\r"==m?s("Invalid whitespace in authority."):":"==m&&null===this.g?this.g="":(m=n(m),null!==this.g?this.g+=m:this.v+=m);d=""}else{if(null==C||"/"==C||"\\"==C||"?"==C||"#"==C){h-=d.length,d="",A="host";continue}d+=C}break;case"file host":if(null==C||"/"==C||"\\"==C||"?"==C||"#"==C){2!=d.length||!u.test(d[0])||":"!=d[1]&&"|"!=d[1]?(0!=d.length&&(this.j=e.call(this,d),d=""),A="relative path start"):A="relative path";continue}"\t"==C||"\n"==C||"\r"==C?s("Invalid whitespace in file host."):d+=C;break;case"host":case"hostname":if(":"!=C||p){if(null==C||"/"==C||"\\"==C||"?"==C||"#"==C){if(this.j=e.call(this,d),d="",A="relative path start",i)break t;continue}"\t"!=C&&"\n"!=C&&"\r"!=C?("["==C?p=!0:"]"==C&&(p=!1),d+=C):s("Invalid code point in host/hostname: "+C)}else if(this.j=e.call(this,d),d="",A="port","hostname"==i)break t;break;case"port":if(/[0-9]/.test(C))d+=C;else{if(null==C||"/"==C||"\\"==C||"?"==C||"#"==C||i){if(""!=d&&((d=parseInt(d,10))!=l[this.h]&&(this.s=d+""),d=""),i)break t;A="relative path start";continue}"\t"==C||"\n"==C||"\r"==C?s("Invalid code point in port: "+C):(o.call(this),this.i=!0)}break;case"relative path start":if("\\"==C&&s("'\\' not allowed in path."),A="relative path","/"!=C&&"\\"!=C)continue;break;case"relative path":null!=C&&"/"!=C&&"\\"!=C&&(i||"?"!=C&&"#"!=C)?"\t"!=C&&"\n"!=C&&"\r"!=C&&(d+=n(C)):("\\"==C&&s("\\ not allowed in relative path."),(m=c[d.toLowerCase()])&&(d=m),".."==d?(this.l.pop(),"/"!=C&&"\\"!=C&&this.l.push("")):"."==d&&"/"!=C&&"\\"!=C?this.l.push(""):"."!=d&&("file"==this.h&&0==this.l.length&&2==d.length&&u.test(d[0])&&"|"==d[1]&&(d=d[0]+":"),this.l.push(d)),d="","?"==C?(this.u="?",A="query"):"#"==C&&(this.C="#",A="fragment"));break;case"query":i||"#"!=C?null!=C&&"\t"!=C&&"\n"!=C&&"\r"!=C&&(this.u+=r(C)):(this.C="#",A="fragment");break;case"fragment":null!=C&&"\t"!=C&&"\n"!=C&&"\r"!=C&&(this.C+=C)}h++}}function o(){this.v=this.na=this.h="",this.g=null,this.s=this.j="",this.l=[],this.C=this.u="",this.D=this.i=!1}function a(t,e){void 0===e||e instanceof a||(e=new a(String(e))),this.Oa=t,o.call(this),t=t.replace(/^[ \t\r\n\f]+|[ \t\r\n\f]+$/g,""),i.call(this,t,null,e)}var s=!1;if(!t.rb)try{var A=new URL("b","http://a");A.pathname="c%20d",s="http://a/c%20d"===A.href}catch(t){}if(!s){var l=Object.create(null);l.ftp=21,l.file=0,l.gopher=70,l.http=80,l.https=443,l.ws=80,l.wss=443;var c=Object.create(null);c["%2e"]=".",c[".%2e"]="..",c["%2e."]="..",c["%2e%2e"]="..";var u=/[a-zA-Z]/,g=/[a-zA-Z0-9\+\-\.]/;a.prototype={toString:function(){return this.href},get href(){if(this.i)return this.Oa;var t="";return""==this.v&&null==this.g||(t=this.v+(null!=this.g?":"+this.g:"")+"@"),this.protocol+(this.D?"//"+t+this.host:"")+this.pathname+this.u+this.C},set href(t){o.call(this),i.call(this,t)},get protocol(){return this.h+":"},set protocol(t){this.i||i.call(this,t+":","scheme start")},get host(){return this.i?"":this.s?this.j+":"+this.s:this.j},set host(t){!this.i&&this.D&&i.call(this,t,"host")},get hostname(){return this.j},set hostname(t){!this.i&&this.D&&i.call(this,t,"hostname")},get port(){return this.s},set port(t){!this.i&&this.D&&i.call(this,t,"port")},get pathname(){return this.i?"":this.D?"/"+this.l.join("/"):this.na},set pathname(t){!this.i&&this.D&&(this.l=[],i.call(this,t,"relative path start"))},get search(){return this.i||!this.u||"?"==this.u?"":this.u},set search(t){!this.i&&this.D&&(this.u="?","?"==t[0]&&(t=t.slice(1)),i.call(this,t,"query"))},get hash(){return this.i||!this.C||"#"==this.C?"":this.C},set hash(t){this.i||(this.C="#","#"==t[0]&&(t=t.slice(1)),i.call(this,t,"fragment"))},get origin(){var t;if(this.i||!this.h)return"";switch(this.h){case"data":case"file":case"javascript":case"mailto":return"null"}return(t=this.host)?this.h+"://"+t:""}};var h=t.URL;h&&(a.createObjectURL=function(t){return h.createObjectURL.apply(h,arguments)},a.revokeObjectURL=function(t){h.revokeObjectURL(t)}),t.URL=a}}(window),Object.getOwnPropertyDescriptor(Node.prototype,"baseURI")||Object.defineProperty(Node.prototype,"baseURI",{get:function(){var t=(this.ownerDocument||this).querySelector("base[href]");return t&&t.href||window.location.href},configurable:!0,enumerable:!0});var Oo=document.createElement("style");Oo.textContent="body {transition: opacity ease-in 0.2s; } \nbody[unresolved] {opacity: 0; display: block; overflow: hidden; position: relative; } \n";var Go=document.querySelector("head");Go.insertBefore(Oo,Go.firstChild);var Ho=window.customElements,jo=!1,Yo=null;function Po(){window.HTMLTemplateElement.bootstrap&&window.HTMLTemplateElement.bootstrap(window.document),Yo&&Yo(),jo=!0,window.WebComponents.ready=!0,document.dispatchEvent(new CustomEvent("WebComponentsReady",{bubbles:!0}))}Ho.polyfillWrapFlushCallback&&Ho.polyfillWrapFlushCallback(function(t){Yo=t,jo&&t()}),"complete"!==document.readyState?(window.addEventListener("load",Po),window.addEventListener("DOMContentLoaded",function(){window.removeEventListener("load",Po),Po()})):Po()}).call(this)}).call(this,n(21),n(265).setImmediate)},function(t,e,n){(function(t){var r=void 0!==t&&t||"undefined"!=typeof self&&self||window,i=Function.prototype.apply;function o(t,e){this._id=t,this._clearFn=e}e.setTimeout=function(){return new o(i.call(setTimeout,r,arguments),clearTimeout)},e.setInterval=function(){return new o(i.call(setInterval,r,arguments),clearInterval)},e.clearTimeout=e.clearInterval=function(t){t&&t.close()},o.prototype.unref=o.prototype.ref=function(){},o.prototype.close=function(){this._clearFn.call(r,this._id)},e.enroll=function(t,e){clearTimeout(t._idleTimeoutId),t._idleTimeout=e},e.unenroll=function(t){clearTimeout(t._idleTimeoutId),t._idleTimeout=-1},e._unrefActive=e.active=function(t){clearTimeout(t._idleTimeoutId);var e=t._idleTimeout;e>=0&&(t._idleTimeoutId=setTimeout(function(){t._onTimeout&&t._onTimeout()},e))},n(266),e.setImmediate="undefined"!=typeof self&&self.setImmediate||void 0!==t&&t.setImmediate||this&&this.setImmediate,e.clearImmediate="undefined"!=typeof self&&self.clearImmediate||void 0!==t&&t.clearImmediate||this&&this.clearImmediate}).call(this,n(21))},function(t,e,n){(function(t,e){!function(t,n){"use strict";if(!t.setImmediate){var r,i,o,a,s,A=1,l={},c=!1,u=t.document,g=Object.getPrototypeOf&&Object.getPrototypeOf(t);g=g&&g.setTimeout?g:t,"[object process]"==={}.toString.call(t.process)?r=function(t){e.nextTick(function(){d(t)})}:function(){if(t.postMessage&&!t.importScripts){var e=!0,n=t.onmessage;return t.onmessage=function(){e=!1},t.postMessage("","*"),t.onmessage=n,e}}()?(a="setImmediate$"+Math.random()+"$",s=function(e){e.source===t&&"string"==typeof e.data&&0===e.data.indexOf(a)&&d(+e.data.slice(a.length))},t.addEventListener?t.addEventListener("message",s,!1):t.attachEvent("onmessage",s),r=function(e){t.postMessage(a+e,"*")}):t.MessageChannel?((o=new MessageChannel).port1.onmessage=function(t){d(t.data)},r=function(t){o.port2.postMessage(t)}):u&&"onreadystatechange"in u.createElement("script")?(i=u.documentElement,r=function(t){var e=u.createElement("script");e.onreadystatechange=function(){d(t),e.onreadystatechange=null,i.removeChild(e),e=null},i.appendChild(e)}):r=function(t){setTimeout(d,0,t)},g.setImmediate=function(t){"function"!=typeof t&&(t=new Function(""+t));for(var e=new Array(arguments.length-1),n=0;n<e.length;n++)e[n]=arguments[n+1];var i={callback:t,args:e};return l[A]=i,r(A),A++},g.clearImmediate=h}function h(t){delete l[t]}function d(t){if(c)setTimeout(d,0,t);else{var e=l[t];if(e){c=!0;try{!function(t){var e=t.callback,r=t.args;switch(r.length){case 0:e();break;case 1:e(r[0]);break;case 2:e(r[0],r[1]);break;case 3:e(r[0],r[1],r[2]);break;default:e.apply(n,r)}}(e)}finally{h(t),c=!1}}}}}("undefined"==typeof self?void 0===t?this:t:self)}).call(this,n(21),n(100))},function(t,e,n){"use strict";var r=n(17);t.exports=function(t,e){return!!t&&r(function(){e?t.call(null,function(){},1):t.call(null)})}},function(t,e,n){"use strict";var r=n(19).f,i=n(72),o=n(121),a=n(52),s=n(122),A=n(123),l=n(98),c=n(97),u=n(272),g=n(16),h=n(79).fastKey,d=n(124),f=g?"_s":"size",p=function(t,e){var n,r=h(e);if("F"!==r)return t._i[r];for(n=t._f;n;n=n.n)if(n.k==e)return n};t.exports={getConstructor:function(t,e,n,l){var c=t(function(t,r){s(t,c,e,"_i"),t._t=e,t._i=i(null),t._f=void 0,t._l=void 0,t[f]=0,null!=r&&A(r,n,t[l],t)});return o(c.prototype,{clear:function(){for(var t=d(this,e),n=t._i,r=t._f;r;r=r.n)r.r=!0,r.p&&(r.p=r.p.n=void 0),delete n[r.i];t._f=t._l=void 0,t[f]=0},delete:function(t){var n=d(this,e),r=p(n,t);if(r){var i=r.n,o=r.p;delete n._i[r.i],r.r=!0,o&&(o.n=i),i&&(i.p=o),n._f==r&&(n._f=i),n._l==r&&(n._l=o),n[f]--}return!!r},forEach:function(t){d(this,e);for(var n,r=a(t,arguments.length>1?arguments[1]:void 0,3);n=n?n.n:this._f;)for(r(n.v,n.k,this);n&&n.r;)n=n.p},has:function(t){return!!p(d(this,e),t)}}),g&&r(c.prototype,"size",{get:function(){return d(this,e)[f]}}),c},def:function(t,e,n){var r,i,o=p(t,e);return o?o.v=n:(t._l=o={i:i=h(e,!0),k:e,v:n,p:r=t._l,n:void 0,r:!1},t._f||(t._f=o),r&&(r.n=o),t[f]++,"F"!==i&&(t._i[i]=o)),t},getEntry:p,setStrong:function(t,e,n){l(t,e,function(t,n){this._t=d(t,e),this._k=n,this._l=void 0},function(){for(var t=this._k,e=this._l;e&&e.r;)e=e.p;return this._t&&(this._l=e=e?e.n:this._t._f)?c(0,"keys"==t?e.k:"values"==t?e.v:[e.k,e.v]):(this._t=void 0,c(1))},n?"entries":"values",!n,!0),u(e)}}},function(t,e,n){var r=n(11);t.exports=function(t,e,n,i){try{return i?e(r(n)[0],n[1]):e(n)}catch(e){var o=t.return;throw void 0!==o&&r(o.call(t)),e}}},function(t,e,n){var r=n(38),i=n(7)("iterator"),o=Array.prototype;t.exports=function(t){return void 0!==t&&(r.Array===t||o[i]===t)}},function(t,e,n){var r=n(94),i=n(7)("iterator"),o=n(38);t.exports=n(35).getIteratorMethod=function(t){if(null!=t)return t[i]||t["@@iterator"]||o[r(t)]}},function(t,e,n){"use strict";var r=n(13),i=n(19),o=n(16),a=n(7)("species");t.exports=function(t){var e=r[t];o&&e&&!e[a]&&i.f(e,a,{configurable:!0,get:function(){return this}})}},function(t,e,n){"use strict";var r=n(13),i=n(37),o=n(23),a=n(121),s=n(79),A=n(123),l=n(122),c=n(15),u=n(17),g=n(274),h=n(54),d=n(275);t.exports=function(t,e,n,f,p,I){var C=r[t],m=C,y=p?"set":"add",E=m&&m.prototype,v={},B=function(t){var e=E[t];o(E,t,"delete"==t?function(t){return!(I&&!c(t))&&e.call(this,0===t?0:t)}:"has"==t?function(t){return!(I&&!c(t))&&e.call(this,0===t?0:t)}:"get"==t?function(t){return I&&!c(t)?void 0:e.call(this,0===t?0:t)}:"add"==t?function(t){return e.call(this,0===t?0:t),this}:function(t,n){return e.call(this,0===t?0:t,n),this})};if("function"==typeof m&&(I||E.forEach&&!u(function(){(new m).entries().next()}))){var b=new m,Q=b[y](I?{}:-0,1)!=b,w=u(function(){b.has(1)}),x=g(function(t){new m(t)}),_=!I&&u(function(){for(var t=new m,e=5;e--;)t[y](e,e);return!t.has(-0)});x||((m=e(function(e,n){l(e,m,t);var r=d(new C,e,m);return null!=n&&A(n,p,r[y],r),r})).prototype=E,E.constructor=m),(w||_)&&(B("delete"),B("has"),p&&B("get")),(_||Q)&&B(y),I&&E.clear&&delete E.clear}else m=f.getConstructor(e,t,p,y),a(m.prototype,n),s.NEED=!0;return h(m,t),v[t]=m,i(i.G+i.W+i.F*(m!=C),v),I||f.setStrong(m,t,p),m}},function(t,e,n){var r=n(7)("iterator"),i=!1;try{var o=[7][r]();o.return=function(){i=!0},Array.from(o,function(){throw 2})}catch(t){}t.exports=function(t,e){if(!e&&!i)return!1;var n=!1;try{var o=[7],a=o[r]();a.next=function(){return{done:n=!0}},o[r]=function(){return a},t(o)}catch(t){}return n}},function(t,e,n){var r=n(15),i=n(276).set;t.exports=function(t,e,n){var o,a=e.constructor;return a!==n&&"function"==typeof a&&(o=a.prototype)!==n.prototype&&r(o)&&i&&i(t,o),t}},function(t,e,n){var r=n(15),i=n(11),o=function(t,e){if(i(t),!r(e)&&null!==e)throw TypeError(e+": can't set as prototype!")};t.exports={set:Object.setPrototypeOf||("__proto__"in{}?function(t,e,r){try{(r=n(52)(Function.call,n(107).f(Object.prototype,"__proto__").set,2))(t,[]),e=!(t instanceof Array)}catch(t){e=!0}return function(t,n){return o(t,n),e?t.__proto__=n:r(t,n),t}}({},!1):void 0),check:o}},function(t,e,n){window,t.exports=function(t){var e={};function n(r){if(e[r])return e[r].exports;var i=e[r]={i:r,l:!1,exports:{}};return t[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=t,n.c=e,n.d=function(t,e,r){n.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:r})},n.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},n.t=function(t,e){if(1&e&&(t=n(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var i in t)n.d(r,i,function(e){return t[e]}.bind(null,i));return r},n.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(e,"a",e),e},n.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},n.p="",n(n.s=81)}([function(t,e,n){"use strict";var r=n(3).extend("Feature",{next:null,detached:null,cursor:null,currentHoverCell:null,setNext:function(t){this.next?this.next.setNext(t):(this.next=t,this.detached=t)},detachChain:function(){this.next=null},attachChain:function(){this.next=this.detached},handleMouseMove:function(t,e){this.next&&this.next.handleMouseMove(t,e)},handleMouseExit:function(t,e){this.next&&this.next.handleMouseExit(t,e)},handleMouseEnter:function(t,e){this.next&&this.next.handleMouseEnter(t,e)},handleMouseDown:function(t,e){this.next&&this.next.handleMouseDown(t,e)},handleMouseUp:function(t,e){this.next&&this.next.handleMouseUp(t,e)},handleKeyDown:function(t,e){if(!this.next)return!0;this.next.handleKeyDown(t,e)},handleKeyUp:function(t,e){this.next&&this.next.handleKeyUp(t,e)},handleWheelMoved:function(t,e){this.next&&this.next.handleWheelMoved(t,e)},handleDoubleClick:function(t,e){this.next&&this.next.handleDoubleClick(t,e)},handleClick:function(t,e){this.next&&this.next.handleClick(t,e)},handleMouseDrag:function(t,e){this.next&&this.next.handleMouseDrag(t,e)},handleContextMenu:function(t,e){this.next&&this.next.handleContextMenu(t,e)},moveSingleSelect:function(t,e,n){this.next&&this.next.moveSingleSelect(t,e,n)},isFirstFixedRow:function(t,e){return e.gridCell.y<1},isFirstFixedColumn:function(t,e){return 0===e.gridCell.x},setCursor:function(t){this.next&&this.next.setCursor(t),this.cursor&&t.beCursor(this.cursor)},initializeOn:function(t){this.next&&this.next.initializeOn(t)}});t.exports=r},function(t,e,n){var r=n(20)("wks"),i=n(16),o=n(5).Symbol,a="function"==typeof o;(t.exports=function(t){return r[t]||(r[t]=a&&o[t]||(a?o:i)("Symbol."+t))}).store=r},function(t,e,n){"use strict";function r(t,e){Object.defineProperty(this,t,{value:e,writable:!1,enumerable:!0,configurable:!1})}function i(t,e){r.call(this,"x",Number(t)||0),r.call(this,"y",Number(e)||0)}function o(t,e,n,o){t=Number(t)||0,e=Number(e)||0,(n=Number(n)||0)<0&&(t+=n,n=-n),(o=Number(o)||0)<0&&(e+=o,o=-o),r.call(this,"origin",new i(t,e)),r.call(this,"extent",new i(n,o)),r.call(this,"corner",new i(t+n,e+o)),r.call(this,"center",new i(t+n/2,e+o/2))}i.prototype={plus:function(t){return new i(this.x+t.x,this.y+t.y)},plusXY:function(t,e){return new i(this.x+(t||0),this.y+(e||0))},minus:function(t){return new i(this.x-t.x,this.y-t.y)},min:function(t){return new i(Math.min(this.x,t.x),Math.min(this.y,t.y))},max:function(t){return new i(Math.max(this.x,t.x),Math.max(this.y,t.y))},distance:function(t){var e=t.x-this.x,n=t.y-this.y;return Math.sqrt(e*e+n*n)},equals:function(t){var e=!1;return t&&(e=this.x===t.x&&this.y===t.y),e},greaterThan:function(t){return this.x>t.x&&this.y>t.y},lessThan:function(t){return this.x<t.x&&this.y<t.y},greaterThanOrEqualTo:function(t){return this.x>=t.x&&this.y>=t.y},lessThanOrEqualTo:function(t){return this.x<=t.x&&this.y<=t.y},within:function(t){var e=t.origin.x,n=e+t.extent.x,r=t.origin.y,i=r+t.extent.y;return t.extent.x<0&&(e=n,n=t.origin.x),t.extent.y<0&&(r=i,i=t.origin.y),e<=this.x&&this.x<n&&r<=this.y&&this.y<i}},i.prototype.EQ=i.prototype.equals,i.prototype.GT=i.prototype.greaterThan,i.prototype.LT=i.prototype.lessThan,i.prototype.GE=i.prototype.greaterThanOrEqualTo,i.prototype.LE=i.prototype.lessThanOrEqualTo,o.prototype={get top(){return this.origin.y},get left(){return this.origin.x},get bottom(){return this.corner.y},get right(){return this.corner.x},get width(){return this.extent.x},get height(){return this.extent.y},get area(){return this.width*this.height},flattenXAt:function(t){return new o(t,this.origin.y,0,this.extent.y)},flattenYAt:function(t){return new o(this.origin.x,t,this.extent.x,0)},contains:function(t){return t.within(this)},within:function(t){return t.origin.lessThanOrEqualTo(this.origin)&&t.corner.greaterThanOrEqualTo(this.corner)},growBy:function(t){return new o(this.origin.x+t,this.origin.y+t,this.extent.x-t-t,this.extent.y-t-t)},shrinkBy:function(t){return this.growBy(-t)},union:function(t){var e=this.origin.min(t.origin),n=this.corner.max(t.corner).minus(e);return new o(e.x,e.y,n.x,n.y)},forEach:function(t,e){e=e||this;for(var n=this.origin.x,r=this.corner.x;n<r;n++)for(var i=this.origin.y,o=this.corner.y;i<o;i++)t.call(e,n,i)},intersect:function(t,e,n){var r=null,i=this.origin.max(t.origin),a=this.corner.min(t.corner).minus(i);return a.x>0&&a.y>0?r=new o(i.x,i.y,a.x,a.y):"function"==typeof e&&(r=e.call(n||this,t)),r},intersects:function(t){return t.corner.x>this.origin.x&&t.corner.y>this.origin.y&&t.origin.x<this.corner.x&&t.origin.y<this.corner.y}},e.Point=i,e.Rectangle=o},function(t,e,n){"use strict";var r=n(45).Base;Object.defineProperty(r.prototype,"version",{enumerable:!0,writable:!1,configurable:!1,value:n(66).version}),r.prototype.versionAtLeast=function(t){var e=t.split(".").map(Number),n=this.version.split(".").map(function(t,n){return Number(t)-e[n]});return n[0]>0||0===n[0]&&(n[1]>0||0===n[1]&&n[2]>=0)},r.prototype.deprecated=n(114),r.prototype.HypergridError=n(17),r.prototype.notify=function(t,e){switch(e){case"warn":console.warn(t);break;case"alert":alert(t);break;default:throw new this.HypergridError(t)}},r.prototype.unwrap=function(t){return"f"===(typeof t)[0]&&(t=t()),t},r.prototype.mixIn=n(27).mixIn,r.prototype.createApply=function(t,e,n){var r=Array.prototype.slice.call(arguments,2),i=[null].concat(r).concat(e),o=t.bind.apply(t,i);return new o},t.exports=r},function(t,e,n){"use strict";var r=n(3).extend("CellRenderer",{paint:function(t,e){},roundRect:function(t,e,n,r,i,o,a,s){s||(s=!0),o||(o=5),t.beginPath(),t.moveTo(e+o,n),t.lineTo(e+r-o,n),t.quadraticCurveTo(e+r,n,e+r,n+o),t.lineTo(e+r,n+i-o),t.quadraticCurveTo(e+r,n+i,e+r-o,n+i),t.lineTo(e+o,n+i),t.quadraticCurveTo(e,n+i,e,n+i-o),t.lineTo(e,n+o),t.quadraticCurveTo(e,n,e+o,n),t.closePath(),s&&t.stroke(),a&&t.fill(),t.closePath()}});t.exports=r},function(t,e){var n=t.exports="undefined"!=typeof window&&window.Math==Math?window:"undefined"!=typeof self&&self.Math==Math?self:Function("return this")();"number"==typeof __g&&(__g=n)},function(t,e){t.exports=function(t){try{return!!t()}catch(t){return!0}}},function(t,e,n){var r=n(13);t.exports=function(t){if(!r(t))throw TypeError(t+" is not an object!");return t}},function(t,e,n){var r=n(10),i=n(25);t.exports=n(9)?function(t,e,n){return r.f(t,e,i(1,n))}:function(t,e,n){return t[e]=n,t}},function(t,e,n){t.exports=!n(6)(function(){return 7!=Object.defineProperty({},"a",{get:function(){return 7}}).a})},function(t,e,n){var r=n(7),i=n(55),o=n(37),a=Object.defineProperty;e.f=n(9)?Object.defineProperty:function(t,e,n){if(r(t),e=o(e,!0),r(n),i)try{return a(t,e,n)}catch(t){}if("get"in n||"set"in n)throw TypeError("Accessors not supported!");return"value"in n&&(t[e]=n.value),t}},function(t,e){var n={}.hasOwnProperty;t.exports=function(t,e){return n.call(t,e)}},function(t,e,n){"use strict";var r=n(3),i=n(126),o=n(48),a=r.extend("CellEditor",{initialize:function(t,e){for(var n in e)this[n]=e[n];this.event=e;var r=this.event.value;r instanceof Array&&(r=r[1]),this.grid=t,this.grid.cellEditor=this,this.locale=t.localization.locale;var i=this.grid.localization.get(e.format);i!==o.prototype.string&&"string"!==e.format&&(this.localizer=i),this.initialValue=r;var a=document.createElement("DIV");a.innerHTML=this.grid.modules.templater.render(this.template,this),this.el=a.firstChild,this.input=this.el,this.errors=0;var s=this;this.el.addEventListener("keyup",this.keyup.bind(this)),this.el.addEventListener("keydown",function(e){9===e.keyCode&&e.preventDefault(),t.fireSyntheticEditorKeyDownEvent(s,e)}),this.el.addEventListener("keypress",function(e){t.fireSyntheticEditorKeyPressEvent(s,e)}),this.el.addEventListener("mousedown",function(t){s.onmousedown(t)})},onmousedown:function(t){t.stopPropagation()},localizer:o.prototype.null,specialKeyups:{9:"stopEditing",13:"stopEditing",27:"cancelEditing"},keyup:function(t){var e,n,r=this.grid,i=this.event.properties,o=i.feedbackCount,a=r.canvas.getKeyChar(t);if((e=this.specialKeyups[t.keyCode])&&(n=this[e](o))&&r.repaint(),i.mappedNavKey(a,t.ctrlKey)&&(!e&&(n=this.stopEditing(o))&&r.repaint(),n)){var s=r.canvas.newEvent(t,"fin-editor-keydown",{grid:r,alt:t.altKey,ctrl:t.ctrlKey,char:a,legacyChar:t.legacyKey,code:t.charCode,key:t.keyCode,meta:t.metaKey,shift:t.shiftKey,identifier:t.key,editor:this});r.delegateKeyDown(s)}return this.grid.fireSyntheticEditorKeyUpEvent(this,t),n},checkEditorPositionFlag:!1,gridRenderedNotification:function(){this.checkEditor()},scrollValueChangedNotification:function(){this.checkEditorPositionFlag=!0},moveEditor:function(){this.setBounds(this.event.bounds)},beginEditing:function(){this.grid.fireRequestCellEdit(this.event,this.initialValue)&&(this.checkEditorPositionFlag=!0,this.checkEditor())},setEditorValue:function(t){this.input.value=this.localizer.format(t)},showEditor:function(){this.el.style.display="inline"},hideEditor:function(){this.el.style.display="none"},stopEditing:function(t){var e=this.input.value;try{var n=this.validateEditorValue(e);if(!n)var r=this.getEditorValue(e)}catch(t){n=t}if(!n&&this.grid.fireSyntheticEditorDataChangeEvent(this,this.initialValue,r))try{this.saveEditorValue(r)}catch(t){n=t}return n?t>=0?this.errorEffectBegin(++this.errors%t==0&&n):this.cancelEditing():(this.hideEditor(),this.grid.cellEditor=null,this.el.remove()),!n},cancelEditing:function(){return this.setEditorValue(this.initialValue),this.hideEditor(),this.grid.cellEditor=null,this.el.remove(),this.grid.takeFocus(),!0},errorEffectBegin:function(t){if(!this.effecting){var e=this.grid.properties.feedbackEffect,n=i[e.name||e];if(n){var r=Object.assign({},e.options);r.callback=this.errorEffectEnd.bind(this,t),this.effecting=!0,n.call(this,r)}}},errorEffectEnd:function(t,e){if(t){var n='Invalid value. To resolve, do one of the following:\n\n   * Correct the error and try again.\n         - or -\n   * Cancel editing by pressing the "esc" (escape) key.';"string"!=typeof(t=t.message||t)&&(t=""),this.localizer.expectation&&(t=t?t+"\n"+this.localizer.expectation:this.localizer.expectation),t&&(/[\n\r]/.test(t)&&(t=(t="\n"+t).replace(/[\n\r]+/g,"\n\n   * ")),n+="\n\nAdditional information about this error: "+t),setTimeout(function(){alert(n)})}this.effecting=!1},saveEditorValue:function(t){var e=!(t&&t===this.initialValue)&&this.grid.fireBeforeCellEdit(this.event.gridCell,this.initialValue,t,this);return e&&(this.event.value=t,this.grid.fireAfterCellEdit(this.event.gridCell,this.initialValue,t,this)),e},getEditorValue:function(t){return this.localizer.parse(t||this.input.value)},validateEditorValue:function(t){return this.localizer.invalid&&this.localizer.invalid(t||this.input.value)},takeFocus:function(){var t=this.el,e=t.style.left,n=t.style.top;t.style.left=t.style.top=0;var r=window.scrollX,i=window.scrollY;this.input.focus(),window.scrollTo(r,i),this.selectAll(),t.style.left=e,t.style.top=n},selectAll:function(){},setBounds:function(t){var e=this.el.style;e.left=s(t.x),e.top=s(t.y),e.width=s(t.width),e.height=s(t.height)},checkEditor:function(){this.checkEditorPositionFlag&&(this.checkEditorPositionFlag=!1,this.event.isCellVisible?(this.setEditorValue(this.initialValue),this.attachEditor(),this.moveEditor(),this.showEditor(),this.takeFocus()):this.hideEditor())},attachEditor:function(){this.grid.div.appendChild(this.el)},template:""});function s(t){return t+"px"}t.exports=a},function(t,e){t.exports=function(t){return"object"==typeof t?null!==t:"function"==typeof t}},function(t,e,n){var r=n(5),i=n(8),o=n(11),a=n(16)("src"),s=n(92),A=(""+s).split("toString");n(21).inspectSource=function(t){return s.call(t)},(t.exports=function(t,e,n,s){var l="function"==typeof n;l&&(o(n,"name")||i(n,"name",e)),t[e]!==n&&(l&&(o(n,a)||i(n,a,t[e]?""+t[e]:A.join(String(e)))),t===r?t[e]=n:s?t[e]?t[e]=n:i(t,e,n):(delete t[e],i(t,e,n)))})(Function.prototype,"toString",function(){return"function"==typeof this&&this[a]||s.call(this)})},function(t,e,n){var r=n(98),i=n(23);t.exports=function(t){return r(i(t))}},function(t,e){var n=0,r=Math.random();t.exports=function(t){return"Symbol(".concat(void 0===t?"":t,")_",(++n+r).toString(36))}},function(t,e,n){"use strict";function r(t){this.message=t}r.prototype=Object.create(Error.prototype),r.prototype.name="HypergridError",t.exports=r},function(t,e,n){"use strict";var r=n(28),i=n(117),o=n(118);r(o).each(function(t,e){var n=new Image;n.src="data:"+t.type+";base64,"+t.data,o[e]=n}),o["checkbox-on"]=o.checked,o["checkbox-off"]=o.unchecked,Object.defineProperties(o,{add:{value:function(t,e,n,r,i){return/^data:image\/svg\+xml|\.svg/.test(e.src)&&(e.themeable=!!n,"object"==typeof r&&(i=r,r=void 0),r&&(e.setSvgProps=r),i&&(e.themeableRules=function(t,e,n,r){var i=document.querySelector("style#injected-stylesheet-themeables");i||((i=document.createElement("style")).id="injected-stylesheet-themeables",document.head.appendChild(i));var o=i.sheet;return(r.length?r:["background-image"]).reduce(function(r,i){var a=".hypergrid-"+i+"-"+t,s=Array.prototype.findIndex.call(o.cssRules,function(t){return t.selectorText===a});-1!==s&&o.deleteRule(s);var A={};A[i]="url("+e.src+")",e.width&&(A.width=e.width+"px"),e.height&&(A.height=e.height+"px");var l="{"+Object.keys(A).map(function(t){return t+":"+A[t]}).join(";")+"}";o.insertRule(a+l);var c={rule:o.cssRules[0]};return n&&(c.setSvgProps=n),r.push(c),r},[])}(t,e,r,i))),o[t]=e}},setTheme:{value:function(t){Object.keys(o).forEach(function(e){var n=o[e];n.themeable&&i.setImgSvgProps.call(n,t,n.setSvgProps),n.themeableRules&&n.themeableRules.forEach(function(e){var r=e.rule.selectorText,o=new RegExp("^.hypergrid-("+i.cssImagePropertyNames.join("|")+")-.*$"),a=r.replace(o,"$1");i.setRuleSvgProps.call(e.rule,t,n.setSvgProps,a)})})}},checkbox:{value:function(t){return o[t?"checked":"unchecked"]}},filter:{value:function(t){return o[t?"filter-on":"filter-off"]}}}),t.exports=o},function(t,e){var n={}.toString;t.exports=function(t){return n.call(t).slice(8,-1)}},function(t,e,n){var r=n(21),i=n(5),o=i["__core-js_shared__"]||(i["__core-js_shared__"]={});(t.exports=function(t,e){return o[t]||(o[t]=void 0!==e?e:{})})("versions",[]).push({version:r.version,mode:n(22)?"pure":"global",copyright:"© 2019 Denis Pushkarev (zloirock.ru)"})},function(t,e){var n=t.exports={version:"2.6.8"};"number"==typeof __e&&(__e=n)},function(t,e){t.exports=!1},function(t,e){t.exports=function(t){if(null==t)throw TypeError("Can't call method on  "+t);return t}},function(t,e,n){var r=n(5),i=n(21),o=n(8),a=n(14),s=n(93),A=function(t,e,n){var l,c,u,g,h=t&A.F,d=t&A.G,f=t&A.S,p=t&A.P,I=t&A.B,C=d?r:f?r[e]||(r[e]={}):(r[e]||{}).prototype,m=d?i:i[e]||(i[e]={}),y=m.prototype||(m.prototype={});for(l in d&&(n=e),n)u=((c=!h&&C&&void 0!==C[l])?C:n)[l],g=I&&c?s(u,r):p&&"function"==typeof u?s(Function.call,u):u,C&&a(C,l,u,t&A.U),m[l]!=u&&o(m,l,g),p&&y[l]!=u&&(y[l]=u)};r.core=i,A.F=1,A.G=2,A.S=4,A.P=8,A.B=16,A.W=32,A.U=64,A.R=128,t.exports=A},function(t,e){t.exports=function(t,e){return{enumerable:!(1&t),configurable:!(2&t),writable:!(4&t),value:e}}},function(t,e,n){var r=n(59),i=n(40);t.exports=Object.keys||function(t){return r(t,i)}},function(t,e,n){"use strict";function r(t,e){for(var n=1;n<arguments.length;++n)i.call(t,arguments[n]);return t}function i(t){var e;for(var n in t)(e=Object.getOwnPropertyDescriptor(t,n))&&Object.defineProperty(this,n,e);return this}r.mixInTo=function(t){var e;for(var n in this)(e=Object.getOwnPropertyDescriptor(this,n))&&Object.defineProperty(t,n,e);return t},r.mixIn=i,t.exports=r},function(t,e,n){"use strict";function r(t){return t instanceof r?t:this instanceof r?(this.originalValue=t,void(this.o=t||{})):new r(t)}r.chain=function(t){var e=r(t);return e.chaining=!0,e},r.prototype={value:function(){return this.originalValue},each:function(t,e){var n=this.o;return Object.keys(n).forEach(function(e){t.call(this,n[e],e,n)},e||n),this},find:function(t,e){var n,r=this.o;return r&&void 0!==(n=Object.keys(r).find(function(e){return t.call(this,r[e],e,r)},e||r))&&(n=r[n]),n},filter:function(t,e){var n=this.o,r=[];return n&&Object.keys(n).forEach(function(e){t.call(this,n[e],e,n)&&r.push(n[e])},e||n),r},map:function(t,e){var n=this.o,r=[];return n&&Object.keys(n).forEach(function(e){r.push(t.call(this,n[e],e,n))},e||n),r},reduce:function(t,e,n){var r=this.o;return r&&Object.keys(r).forEach(function(n,i){e=i||void 0!==e?t(e,r[n],n,r):r[n]},n||r),e},extend:function(t){var e=this.o;return Array.prototype.slice.call(arguments).forEach(function(t){if(t)for(var n in t)e[n]=t[n]}),this.chaining?this:e},extendOwn:function(t){var e=this.o;return Array.prototype.slice.call(arguments).forEach(function(t){r(t).each(function(t,n){e[n]=t})}),this.chaining?this:e}},Array.prototype.find||(Array.prototype.find=function(t){if(null===this)throw new TypeError("Array.prototype.find called on null or undefined");if("function"!=typeof t)throw new TypeError("predicate must be a function");for(var e,n=Object(this),r=n.length>>>0,i=arguments[1],o=0;o<r;o++)if(e=n[o],t.call(i,e,o,n))return e}),t.exports=r},function(t,e,n){"use strict";var r=n(47),i=/^\d+$/,o=["transformations","propPath","dictPath","force"];function a(t){t&&o.forEach(function(e){t[e]&&(this[e]=t[e])},this)}function s(t,e){return e.reduce(function(t,e){return t[e]||(t[e]=Object.create(null))},t)}function A(t){return(t=t?Array.isArray(t)?t.slice():(t+"").split("."):[]).toString=l,t}function l(){return this.join(".")}function c(t,e,n){var i=r[n],o=this.transformations[n],a=Array.isArray(o)?o.slice():o.split("."),A=a.splice(a.length-1,1)[0],l=s(t,a);!this.force&&A in l||(l[A]=i(e))}a.prototype={constructor:a,transformations:["verbatim","toCamelCase"],set propPath(t){this._propPath=A(t)},get propPath(){return this._propPath},_propPath:["name"],set dictPath(t){this._dictPath=A(t)},get dictPath(){return this._dictPath},_dictPath:[],getSynonyms:function(t,e){var n=[];return"string"==typeof t&&t&&(e=e||this.transformations,Array.isArray(e)||(e=Object.keys(e)),e.forEach(function(e){if("function"!=typeof r[e])throw new ReferenceError('Unknown transformer "'+e+'"');var i=r[e](t);""===i||i in n||n.push(i)})),n},decorate:function(t,e,n){var r=s(t,this.dictPath),o=r===t,a=this.force;return e.forEach(function(t){o&&i.test(t)||!a&&t in r||(r[t]=n)}),t},decorateList:function(t,e,n){var r;return"number"==typeof t?r=[e[t]]:(e=r=arguments[0],n=arguments[1]),n=n?A(n):this.propPath,r.forEach(function(t){var r=void 0!==n&&"object"==typeof t?s(t,n):t;if(Array.isArray(this.transformations)){var i=this.getSynonyms(r);this.decorate(e,i,t)}else Object.keys(this.transformations).forEach(c.bind(this,t,r))},this),e}},a.prototype.decorateObject=a.prototype.decorate,a.prototype.decorateArray=a.prototype.decorateList,t.exports=a},function(t,e,n){"use strict";var r=n(3).extend("Registry",{initialize:function(){this.items=Object.create(null)},add:function(t,e){if(1===arguments.length&&(e=t,t=void 0),e){if(!(t=t||e.getClassName&&e.getClassName()))throw new this.HypergridError("Cannot register "+this.friendlyName()+" without a name.");return this.items[t]=e,t=t.toLowerCase(),Object.keys(this.items).filter(function(e){return e.toLowerCase()===t}).forEach(function(t){this.items[t]=e},this),e}},addSynonym:function(t,e){return this.items[t]=this.items[e]},make:function(t,e){var n=arguments.length-1;arguments[n]=this.BaseClass.extend(arguments[n]),this.add.apply(this,arguments)},get:function(t){if(t){var e=this.items[t];if(!e){var n=t.toLowerCase(),r=Object.keys(this.items).find(function(t){return n===t.toLowerCase()});if(!(e=this.items[r]))throw new this.HypergridError('Expected "'+t+'" to be a case-insensitive match for a registered '+this.friendlyName()+".");this.addSynonym(t,r)}return e}},friendlyName:function(){if(this.BaseClass){var t=this.BaseClass.getClassName();t=t&&t.replace(/([A-Z])/g," $1").trim().toLowerCase()}else t=function(t){return i.find(function(e){if(e.plural.test(t))return t=t.replace(e.plural,e.singular),!0}),t}(this.getClassName()).toLowerCase();return function(t){return/^[aeiou]/.test(t)?"an":"a"}(t=t||"item")+" "+t}}),i=[{plural:/ies$/,singular:"y"},{plural:/s$/,singular:""}];t.exports=r},function(t,e,n){"use strict";var r=n(45).Base.extend("DatasaurBase",{isNullObject:!0,drillDownCharMap:{true:"▼",false:"▶",undefined:"",null:"   "},DataError:o,initialize:function(t,e){if(t)for(this.handlers=t.handlers,this.next=t;t;)this.source=t,t=t.next;else this.handlers=[],this.source=this;this.install(Object.getPrototypeOf(this))},install:function(t,e){e=e||{};var n=this,o=function(t){var e=t.hasOwnProperty("!!keys")&&t["!!keys"],n=i.concat(t.hasOwnProperty("!keys")&&t["!keys"]||[]);return(Array.isArray(t)?t:Object.keys(t).filter(function(e){return"function"==typeof Object.getOwnPropertyDescriptor(t,e).value})).filter(function(t){return!(e&&e.indexOf(t)<0||n.indexOf(t)>=0)})}(t),a=e.inject&&!Array.isArray(t);o.forEach(function(i){a&&(function(t,e,n){do{if(t[e])if(n)for(var i=t;i&&i!==Object.prototype;i=Object.getPrototypeOf(i))delete i[e];else if(t[e]!==r.prototype[e])return t[e];t=t.next}while(t)}(n,i,e.force)||(this.source[i]=t[i])),r.prototype[i]||(r.prototype[i]=function(){if(this.next)return this.next[i].apply(this.next,arguments)})},this)},dispatchEvent:function(t){this.handlers.forEach(function(e){e.call(this,t)},this)},addListener:function(t){this.handlers.indexOf(t)<0&&this.handlers.push(t)},removeListener:function(t){var e=this.handlers.indexOf(t);e>=0&&delete this.handlers[e]},removeAllListeners:function(){this.handlers.length=0},dump:function(t){t=Math.min(this.getRowCount(),t||Math.max(100,this.getRowCount()));for(var e=[],n=this.getSchema(),r=n?n.map(function(t){return t.name}):this.getHeaders(),i=this.getColumnCount(),o=this.viewMakesSense,a=0;a<t;a++){for(var s={},A=0;A<i;A++){var l=this.getValue(A,a);0===A&&o&&(l=this.fixIndentForTableDisplay(l)),s[r[A]]=l}e[a]=s}console.table(e)}}),i=["constructor","initialize","!keys","!!keys"];function o(t){this.message=t}o.prototype=Object.create(Error.prototype),o.prototype.name="DataError",t.exports=r},function(t,e){t.exports=function(t){return t.webpackPolyfill||(t.deprecate=function(){},t.paths=[],t.children||(t.children=[]),Object.defineProperty(t,"loaded",{enumerable:!0,get:function(){return t.l}}),Object.defineProperty(t,"id",{enumerable:!0,get:function(){return t.i}}),t.webpackPolyfill=1),t}},function(t,e){t.exports=function(t){if("function"!=typeof t)throw TypeError(t+" is not a function!");return t}},function(t,e){var n=Math.ceil,r=Math.floor;t.exports=function(t){return isNaN(t=+t)?0:(t>0?r:n)(t)}},function(t,e,n){"use strict";var r,i,o=n(36),a=RegExp.prototype.exec,s=String.prototype.replace,A=a,l=(r=/a/,i=/b*/g,a.call(r,"a"),a.call(i,"a"),0!==r.lastIndex||0!==i.lastIndex),c=void 0!==/()??/.exec("")[1];(l||c)&&(A=function(t){var e,n,r,i,A=this;return c&&(n=new RegExp("^"+A.source+"$(?!\\s)",o.call(A))),l&&(e=A.lastIndex),r=a.call(A,t),l&&r&&(A.lastIndex=A.global?r.index+r[0].length:e),c&&r&&r.length>1&&s.call(r[0],n,function(){for(i=1;i<arguments.length-2;i++)void 0===arguments[i]&&(r[i]=void 0)}),r}),t.exports=A},function(t,e,n){"use strict";var r=n(7);t.exports=function(){var t=r(this),e="";return t.global&&(e+="g"),t.ignoreCase&&(e+="i"),t.multiline&&(e+="m"),t.unicode&&(e+="u"),t.sticky&&(e+="y"),e}},function(t,e,n){var r=n(13);t.exports=function(t,e){if(!r(t))return t;var n,i;if(e&&"function"==typeof(n=t.toString)&&!r(i=n.call(t)))return i;if("function"==typeof(n=t.valueOf)&&!r(i=n.call(t)))return i;if(!e&&"function"==typeof(n=t.toString)&&!r(i=n.call(t)))return i;throw TypeError("Can't convert object to primitive value")}},function(t,e){t.exports={}},function(t,e,n){var r=n(20)("keys"),i=n(16);t.exports=function(t){return r[t]||(r[t]=i(t))}},function(t,e){t.exports="constructor,hasOwnProperty,isPrototypeOf,propertyIsEnumerable,toLocaleString,toString,valueOf".split(",")},function(t,e,n){var r=n(10).f,i=n(11),o=n(1)("toStringTag");t.exports=function(t,e,n){t&&!i(t=n?t:t.prototype,o)&&r(t,o,{configurable:!0,value:e})}},function(t,e,n){var r=n(23);t.exports=function(t){return Object(r(t))}},function(t,e){e.f={}.propertyIsEnumerable},function(t,e,n){"use strict";var r=n(2).Point,i=n(3),o=n(115),a=n(121),s=n(122),A=n(123),l=n(29),c=n(46),u=n(49),g=["columnHeader","columnHeaderColumnSelection","filterProperties","rowHeader","rowHeaderRowSelection","rowNumbersProperties","treeColumnProperties","treeColumnPropertiesColumnSelection"],h=i.extend("Behavior",{initialize:function(t,e){this.grid=t,this.initializeFeatureChain(),this.grid.behavior=this,this.reset(e)},initializeFeatureChain:function(t){var e;if(this.featureChain=void 0,this.featureMap={},this.featureRegistry=this.featureRegistry||A,this.grid.properties.features){var n=this.featureRegistry.get.bind(this.featureRegistry);e=this.grid.properties.features.map(n)}else this.features&&(e=this.features,function(){var t=[],e=[],n=0;if(this.features.forEach(function(r){var i=r.prototype.$$CLASS_NAME||r.name,o=i||"feature"+n++;if(t.push(o),!this.featureRegistry.get(o,!0)){var a=r.name||r.prototype.$$CLASS_NAME||"FeatureConstructor"+n,s=[];i||s.push("'"+o+"'"),s.push(a),e.push(s.join(", "))}},this),t.length){var r="Hypergrid.defaults.features = [\n"+d("\t'",t,"',\n")+"];";e.length&&(r+="\n\nThe following custom features are unregistered and will need to be registered prior to behavior instantiation:\n\n"+d("Features.add(",e,");\n")),n&&(r+="\n\n(You should provide meaningful names for your custom features rather than the generated names above.)"),console.warn("`grid.behavior.features` (array of feature constructors) has been deprecated as of version 2.1.0 in favor of `grid.properties.features` (array of feature names). Remove `features` array from your behavior and add `features` property to your grid state object (or Hypergrid.defaults), e.g.:\n\n"+r)}}.call(this));e.forEach(function(t,e){var n=new t;this.featureMap[n.$$CLASS_NAME]=n,e?this.featureChain.setNext(n):this.featureChain=n},this),this.featureChain&&this.featureChain.initializeOn(this.grid)},features:[],reset:function(t){var e=this.resetDataModel(t);e&&(this.CellEvent=a(this.grid)),this.scrollPositionX=this.scrollPositionY=0,this.rowPropertiesPrototype=Object.create(this.grid.properties,n(67).rowPropertiesPrototypeDescriptors),this.clearColumns(),this.createColumns(),this.subgrids=t&&t.subgrids||!e&&this.subgrids||this.grid.properties.subgrids,this.setData(t)},setHeaders:function(t){if(t instanceof Array){var e=this.allColumns;t.forEach(function(t,n){e[n].header=t})}else"object"==typeof t&&this.allColumns.forEach(function(e){t[e.name]&&(e.header=t[e.name])})},setData:function(t,e){if(Array.isArray(t)||"function"==typeof t||(t=(e=t)&&e.data),void 0!==(t=this.unwrap(t))){if(!Array.isArray(t))throw"Expected data to be an array (of data row objects).";e=e||{};var n=this.unwrap(e.schema);this.subgrids.forEach(function(e){e.setData(t,n)}),this.grid.cellEditor&&this.grid.cellEditor.cancelEditing(),(e.apply||void 0===e.apply)&&this.reindex(),this.grid.allowEvents(this.getRowCount())}},get renderedColumnCount(){return this.grid.renderer.visibleColumns.length},get renderedRowCount(){return this.grid.renderer.visibleRows.length},get leftMostColIndex(){return this.grid.properties.showRowNumbers?this.rowColumnIndex:this.hasTreeColumn()?this.treeColumnIndex:0},clearColumns:function(){var t=this.schema,e=this.treeColumnIndex,n=this.rowColumnIndex;this.columnsCreated=!1,t[e]=t[e]||{},t[e].index=e,void 0===t[e].name&&(t[e].name=this.treeColumnNameDefault),void 0===t[e].header&&(t[e].header=this.treeColumnHeaderDefault),t[n]=t[n]||{},t[n].index=n,void 0===t[n].name&&(t[n].name=this.rowColumnNameDefault),void 0===t[n].header&&(t[n].header=""),this.columns=this.grid.decorateColumnArray(),this.allColumns=this.grid.decorateColumnArray(),this.allColumns[e]=this.columns[e]=this.newColumn(t[e]),this.allColumns[n]=this.columns[n]=this.newColumn(t[n]),this.columns[e].properties.propClassLayers=this.columns[n].properties.propClassLayers=this.grid.properties.propClassLayersMap.NO_ROWS,this.grid.renderer.resetRowHeaderColumnWidth()},getActiveColumn:function(t){return this.columns[t]},getActiveColumnIndex:function(t){var e=t instanceof o?t.index:t,n="number"==typeof index?"index":"name";return this.columns.findIndex(function(t){return t[n]===e})},getColumn:function(t){return this.allColumns[t]},newColumn:function(t){return new o(this,t)},addColumn:function(t){var e=this.newColumn(t),n=new l,r=n.getSynonyms(e.name);return this.columns.push(e),n.decorateObject(this.columns,r,e),this.allColumns.push(e),n.decorateObject(this.allColumns,r,e),e},createColumns:function(t){var e=this.dataModel.getSchema();s.normalizeSchema(e),s.decorateSchema(e),s.decorateColumnSchema(e,this.grid.properties.headerify),this.createDataRowProxy(),this.clearColumns(),t&&t.call(this),this.columnsCreated=!0,this.changed(),u.call(this.grid,"fin-hypergrid-columns-created")},createDataRowProxy:function(){},getColumnWidth:function(t){var e=(t!==this.treeColumnIndex||this.hasTreeColumn())&&this.getActiveColumn(t);return e?e.getWidth():0},setColumnWidth:function(t,e){(t>=-2?this.getActiveColumn(t):t).setWidth(e),this.stateChanged()},reindex:function(){this.dataModel.reindex()},clearObjectProperties:function(t,e){for(var n in t)t.hasOwnProperty(n)&&(void 0===e||!e&&g.indexOf(n)>=0||e&&g.indexOf(n)<0)&&delete t[n]},getState:function(){var t=JSON.parse(JSON.stringify(this.grid.properties));return this.clearObjectProperties(t.columnProperties,!1),t},clearState:function(){this.grid.clearState(),this.createColumns()},setState:function(t){this.addState(t,!0)},addState:function(t,e){e&&this.clearState();var n=this.grid.properties;n.settingState=e,c(n,t),delete n.settingState,this.reindex()},setAllColumnProperties:function(t){this.addAllColumnProperties(t,!0)},addAllColumnProperties:function(t,e){if(t){var n=this.grid.behavior.getColumns();Object.keys(t).forEach(function(r){var i=n[r];i&&i.addProperties(t[r],e)})}},setColumnOrder:function(t){if(Array.isArray(t)){var e=this.columns,n=this.allColumns,r=new l;e.length=0;var i=this.treeColumnIndex.toString(),o=this.rowColumnIndex.toString();Object.keys(e).forEach(function(t){switch(t){case i:case o:break;default:delete e[t]}}),t.forEach(function(t){e.push(n[t])}),r.decorateArray(e)}},setColumnOrderByName:function(t){if(Array.isArray(t)){var e=this.allColumns;this.setColumnOrder(t.map(function(t){return e[t].index}))}},setColumnIndexes:function(t,e){this.grid.properties.columnIndexes=t,e||this.grid.fireSyntheticOnColumnsChangedEvent()},showColumns:function(t,e,n,r){("number"==typeof t||Array.isArray(t))&&(r=n,n=e,e=t,t=!1);var i=this.columns,o=t?i:this.allColumns;"number"==typeof e&&(e=[e]);var a=e.map(function(t){return o[t]}).filter(function(t){return t});"number"!=typeof n&&(r=n,n=i.length),r||a.forEach(function(t){var e=i.indexOf(t);e>=0&&(i.splice(e,1),n>e&&--n)}),n>=0&&i.splice.apply(i,[n,0].concat(a)),this.grid.properties.columnIndexes=i.map(function(t){return t.index})},hideColumns:function(t,e){var n=Array.prototype.slice.call(arguments);n.push(-1),this.showColumns.apply(this,n)},resolveProperty:function(t){return this.grid.resolveProperty(t)},lookupFeature:function(t){return this.featureMap[t]},getFixedColumnsWidth:function(){for(var t,e=this.getFixedColumnCount(),n=0,r=this.leftMostColIndex,i=this.grid.properties,o="border-box"!==i.boxSizing,a=i.gridLinesVWidth;r<e;r++)(t=this.getColumnWidth(r))&&(n+=t,o&&(n+=a));return i.fixedLinesVWidth&&(n+=i.fixedLinesHWidth-a),n},getFixedColumnsMaxWidth:function(){return this.getFixedColumnsWidth()},setCursor:function(t){t.updateCursor(),this.featureChain.setCursor(t)},onMouseMove:function(t,e){this.featureChain&&(this.featureChain.handleMouseMove(t,e),this.setCursor(t))},onClick:function(t,e){this.featureChain&&(this.featureChain.handleClick(t,e),this.setCursor(t))},onContextMenu:function(t,e){this.featureChain&&(this.featureChain.handleContextMenu(t,e),this.setCursor(t))},onWheelMoved:function(t,e){this.featureChain&&(this.featureChain.handleWheelMoved(t,e),this.setCursor(t))},onMouseUp:function(t,e){this.featureChain&&(this.featureChain.handleMouseUp(t,e),this.setCursor(t))},onMouseDrag:function(t,e){this.featureChain&&(this.featureChain.handleMouseDrag(t,e),this.setCursor(t))},onKeyDown:function(t,e){this.featureChain&&(this.featureChain.handleKeyDown(t,e),this.setCursor(t))},onKeyUp:function(t,e){this.featureChain&&(this.featureChain.handleKeyUp(t,e),this.setCursor(t))},onDoubleClick:function(t,e){this.featureChain&&(this.featureChain.handleDoubleClick(t,e),this.setCursor(t))},handleMouseDown:function(t,e){this.featureChain&&(this.featureChain.handleMouseDown(t,e),this.setCursor(t))},handleMouseExit:function(t,e){this.featureChain&&(this.featureChain.handleMouseExit(t,e),this.setCursor(t))},changed:function(){this.grid.behaviorChanged()},shapeChanged:function(){this.grid.behaviorShapeChanged()},stateChanged:function(){this.grid.behaviorStateChanged()},isColumnReorderable:function(){return this.deprecated("isColumnReorderable()","grid.properties.columnsReorderable","2.1.3")},getColumnProperties:function(t){var e=(t!==this.treeColumnIndex||this.hasTreeColumn())&&this.getColumn(t);return e&&e.properties},setColumnProperties:function(t,e){var n=this.getColumn(t);if(!n)throw"Expected column.";var r=Object.assign(n.properties,e);return this.changed(),r},clearAllCellProperties:function(t){var e;for(void 0===t?(t=0,e=this.columns.length):e=t+1;t<e;t++){var n=this.getColumn(t);n&&n.clearAllCellProperties()}},getHiddenColumnDescriptors:function(){for(var t=this.grid.properties.columnIndexes,e=[],n=this.getActiveColumnCount(),r=0;r<n;r++)if(-1===t.indexOf(r)){var i=this.getActiveColumn(r);e.push({id:r,header:i.header,field:i.name})}return e},getFixedColumnCount:function(){return this.grid.properties.fixedColumnCount},setFixedColumnCount:function(t){this.grid.properties.fixedColumnCount=t},getFixedRowCount:function(){return this.getHeaderRowCount()+this.grid.properties.fixedRowCount},setFixedRowCount:function(t){this.grid.properties.fixedRowCount=t},endDragColumnNotification:function(){},getCursorAt:function(t,e){return null},getActiveColumnCount:function(){return this.columns.length},setScrollPositionX:function(t){this.scrollPositionX=t},getScrollPositionX:function(){return this.scrollPositionX},setScrollPositionY:function(t){this.scrollPositionY=t},getScrollPositionY:function(){return this.scrollPositionY},getCellEditorAt:function(t){return t.isDataColumn&&t.column.getCellEditorAt(t)},highlightCellOnHover:function(t,e){return t&&e},set cellPropertiesPrePaintNotification(t){throw new this.HypergridError("cellPropertiesPrePaintNotification has been deprecated as of v3.0.0. Code to inspect or mutate the render config object should be moved to the getCell hook.")},swapColumns:function(t,e){var n=this.columns,r=n[t];n[t]=n[e],n[e]=r,this.changed()},convertViewPointToDataPoint:function(t){return new r(this.getActiveColumn(t.x).index,t.y)},hasTreeColumn:function(t){return!1},getSelectionMatrixFunction:function(t){return function(){return null}},getRowHeaderColumn:function(){return this.allColumns[this.rowColumnIndex]},getTreeColumn:function(){return this.allColumns[this.treeColumnIndex]},autosizeAllColumns:function(){this.checkColumnAutosizing(!0),this.changed()},checkColumnAutosizing:function(t){var e=this.autoSizeRowNumberColumn(t)||this.autoSizeTreeColumn(t);return this.allColumns.findWithNeg(function(n){e=n.checkColumnAutosizing(t)||e}),e},autoSizeRowNumberColumn:function(t){if(this.grid.properties.showRowNumbers&&this.grid.properties.rowNumberAutosizing)return this.getRowHeaderColumn().checkColumnAutosizing(t)},autoSizeTreeColumn:function(t){if(this.grid.properties.showTreeColumn&&this.grid.properties.treeColumnAutosizing)return this.getTreeColumn().checkColumnAutosizing(t)},getColumns:function(){return this.allColumns},getActiveColumns:function(){return this.columns},getHiddenColumns:function(){for(var t=this.columns,e=this.allColumns,n=[],r=0;r<e.length;r++)-1===t.indexOf(e[r])&&n.push(e[r]);return n.sort(function(t,e){return t.header<e.header}),n},getSelectedRows:function(){return this.grid.selectionModel.getSelectedRows()},getSelectedColumns:function(){return this.grid.selectionModel.getSelectedColumns()},getSelections:function(){return this.grid.selectionModel.getSelections()},getIndexedData:function(){return this.deprecated("getIndexedData()","getData()","3.0.0")}});function d(t,e,n){return t+e.join(n+t)+n}Object.defineProperties(h.prototype,{treeColumnIndex:{value:-1},treeColumnNameDefault:{value:"Tree",writable:!0},treeColumnHeaderDefault:{value:"Tree",writable:!0},rowColumnIndex:{value:-2},rowColumnNameDefault:{value:"RowHeader",writable:!0}}),h.prototype.applyAnalytics=h.prototype.reindex,h.prototype.mixIn(n(67).mixin),h.prototype.mixIn(n(137).mixin),h.prototype.mixIn(n(138).mixin),h.prototype.mixIn(n(139).mixin),t.exports=h},function(t,e,n){"use strict";function r(t,e){switch(arguments.length){case 0:e={};break;case 1:switch(typeof t){case"object":e=t,t=void 0;break;case"string":e={};break;default:throw"Single-parameter overload must be either string or object."}break;case 2:if("string"!=typeof t||"object"!=typeof e)throw"Two-parameter overload must be string, object.";break;default:throw"Too many parameters"}function n(){this.preInitialize&&this.preInitialize.apply(this,arguments),function(){var t=this,e=arguments;!function n(r){var i=Object.getPrototypeOf(r);i.constructor!==Object&&(n(i),i.hasOwnProperty("initialize")&&i.initialize.apply(t,e))}(t)}.apply(this,arguments),this.postInitialize&&this.postInitialize.apply(this,arguments)}n.extend=r,n.getClassName=o,n.parent=a;var i,s,A=n.prototype=Object.create(this.prototype);for(i in A.constructor=n,(t=t||A.$$CLASS_NAME||A.name)&&(Object.defineProperty(n,"name",{value:t,configurable:!0}),A.$$CLASS_NAME=t),e)(s=Object.getOwnPropertyDescriptor(e,i))&&Object.defineProperty(A,i,s);return"function"==typeof this.postExtend&&this.postExtend(A),n}function i(){}function o(){return this.prototype.$$CLASS_NAME||this.prototype.name||this.name}function a(t){var e=this.prototype;if(e)do{e=Object.getPrototypeOf(e)}while(t&&e&&e.constructor.name!==t);return e&&e.constructor}i.prototype={constructor:i.prototype.constructor,getClassName:function(){return this.$$CLASS_NAME||this.name||this.constructor.name},get super(){return Object.getPrototypeOf(Object.getPrototypeOf(this))},superMember:function(t){var e=this.super;do{e=Object.getPrototypeOf(e)}while(!e.hasOwnProperty(t));return e&&e[t]},superMethod:function(t){var e=this.superMember(t);if("function"!=typeof e)throw new TypeError("this."+t+" is not a function");return e},callSuperMethod:function(t){return this.superMethod(t).apply(this,Array.prototype.slice.call(arguments,1))}},i.extend=r,r.Base=i,t.exports=r},function(t,e,n){"use strict";t.exports=function(t,e){Object.keys(e).forEach(function(n){for(var r,i=t;i&&!(r=Object.getOwnPropertyDescriptor(i,n));i=Object.getPrototypeOf(i));void 0!==e[n]?(!r||r.writable||r.set)&&(t[n]=e[n]):r&&(!r.configurable||r.set||r.get?(r.writable||r.set)&&(t[n]=void 0):delete t[n])})}},function(t,e,n){"use strict";var r=/([^_A-Z])([A-Z]+)/g,i=/[^a-z0-9]+/gi,o=/[^a-z0-9]+([a-z0-9])?/gi;function a(t,e){return void 0===e?"":e.toUpperCase()}var s=/^(\d)/,A=/^([A-Z])/;function l(t,e){return e.toLowerCase()}var c=/[a-z]/,u=/[\s\-_]*([^\s\-_])([^\s\-_]+)/g,g=function(t,e,n){return e.toUpperCase()+n},h=/[A-Z]+/g,d=/([A-Z]+)([A-Z][a-z])/g;t.exports={verbatim:function(t){return t+""},toCamelCase:function(t){return t.replace(o,a).replace(s,"$$$1").replace(A,l)},toAllCaps:function(t){return t.replace(i,"_").replace(r,"$1_$2").replace(s,"$$$1").toUpperCase()},toTitle:function(t){return(c.test(t)?t:t.toLowerCase()).replace(u,g).replace(h," $&").replace(d,"$1 $2").trim()}}},function(t,e,n){"use strict";var r=n(3).extend({initialize:function(t,e,n){"object"==typeof e&&(n=e,e=t),this.locale=e,n&&("function"==typeof n.invalid&&(this.invalid=n.invalid),n.expectation&&(this.expectation=n.expectation))}});window.Intl||(window.Intl={NumberFormat:function(t,e){this.format=function(t){var n=t.toString();if(!e||void 0===e.useGrouping||e.useGrouping){var r=n.indexOf(".");for(r<0&&(r=n.length);(r-=3)>0&&"0123456789".indexOf(n[r-1])>=0;)n=n.substr(0,r)+","+n.substr(r)}return n}},DateTimeFormat:function(t,e){this.format=function(t){return null!=t?("object"!=typeof t&&(t=new Date(t)),t=t.getMonth()+1+"-"+t.getDate()+"-"+t.getFullYear()):t=null,t}}});var i=r.extend("NumberFormatter",{initialize:function(t,e,n){"object"==typeof e&&(n=e),n=n||{},this.format=new Intl.NumberFormat(this.locale,n).format;var r=new Intl.NumberFormat(this.locale,{useGrouping:!1}).format;this.demapper=function(t){var e=this.map.indexOf(t)%11;return e<0?"":e<10?e:"."}.bind(this),this.map=r(10123456789.5).substr(1,11),n.acceptStandardDigits&&"0123456789."!==this.map&&(this.map+="0123456789."),this.invalids=new RegExp("[^"+this.format(11111).replace(this.map[1],"")+this.map+"]")},invalid:function(t){return this.invalids.test(t)},expectation:"Expected a number with optional commas (thousands grouping separator), optional decimal point, and an optional fractional part.\nComma separators are part of the format and will always be displayed for values >= 1000.\nEdited values are always saved in their entirety even though the formatted value is rounded to the specified number of decimal places.",parse:function(t){var e=Number(t.split("").map(this.demapper).join(""));if(isNaN(e))throw"Invalid Number";return e}}),o=r.extend("DateFormatter",{initialize:function(t,e,n){"object"==typeof e&&(n=e),n=n||{},this.format=new Intl.DateTimeFormat(this.locale,n).format;var r=new Intl.NumberFormat(this.locale,{useGrouping:!1,style:"decimal"}).format,i=this.localizedDigits=r(10123456789).substr(1,10);this.digitFormatter=function(t){return this.localizedDigits[t]}.bind(this),this.digitParser=function(t){var e=this.localizedDigits.indexOf(t);return e<0&&(e=""),e}.bind(this);var o=this.transformNumber(this.digitFormatter,1987),a=this.transformNumber(this.digitFormatter,12),s=this.transformNumber(this.digitFormatter,30),A=new Date(1987,11,30),l=(0,new Intl.DateTimeFormat(this.locale).format)(A),c=new Intl.NumberFormat(this.locale).format(456),u=this.localizedNumberPattern=new RegExp("["+i+"]+","g"),g=l.match(u);this.partsMap={yy:g.indexOf(o),mm:g.indexOf(a),dd:g.indexOf(s)},n.acceptStandardDigits&&(c+="1234567890"),this.invalids=new RegExp("[^"+l.replace(/-/g,"\\-")+c+"]")},invalid:function(t){return this.invalids.test(t)},parse:function(t){var e=t.match(this.localizedNumberPattern);if(!e||3!==e.length)throw"Invalid Date";var n=this.transformNumber(this.digitParser,e[this.partsMap.yy]),r=this.transformNumber(this.digitParser,e[this.partsMap.mm])-1,i=this.transformNumber(this.digitParser,e[this.partsMap.dd]);return new Date(n,r,i)},transformNumber:function(t,e){return e.toString().split("").map(t).join("")}});function a(t,e,n){this.locale=t,this.int=this.float=this.construct("number",i,e),this.construct("date",o,n)}a.prototype={constructor:a.prototype.constructor,$$CLASS_NAME:"Localization",construct:function(t,e,n){var r=t[0].toUpperCase()+t.substr(1).toLowerCase()+"Formatter",i=e.bind(null,this.locale),o=new i(n);return this[r]=i,this.add(t,o)},add:function(t,e){if("object"==typeof t&&(e=t,t=void 0),"object"!=typeof e||"function"!=typeof e.format||"function"!=typeof e.parse||e.invalid&&"function"!=typeof e.invalid||e.expectation&&"string"!=typeof e.expectation)throw"Expected localizer object to conform to interface.";return this[t=(t=t||e.name)&&t.toLowerCase()]=e,e},get:function(t){return this[t&&t.toLowerCase()]||this.string},chromeDate:{format:function(t){if(null!=t){"object"!=typeof t&&(t=new Date(t));var e=t.getFullYear(),n=t.getMonth()+1,r=n<10?"0"+n:n,i=t.getDate();t=e+"-"+r+"-"+(i<10?"0"+i:i)}else t=null;return t},parse:function(t){var e=t.split("-");return e&&3===e.length?new Date(e[0],e[1]-1,e[2]):null}},null:{format:function(t){return t},parse:function(t){return t}},string:{format:function(t){return t+""},parse:function(t){return t+""}}},t.exports=a},function(t,e,n){"use strict";var r=["gridCell","dataCell","mousePoint","gridPoint","clientPoint","pagePoint","keys","row"];t.exports=function(t,e,n,i){var o;if(this.canvas)return"boolean"!=typeof e&&(i=n,n=e,e=!1),n?n instanceof CustomEvent&&(n=Object({},n)):n={},n.type||(n.type=t),n.detail||(n={detail:n}),(o=n.detail).grid||(o.grid=this),o.time=Date.now(),i&&(o.primitiveEvent||(o.primitiveEvent=i),r.forEach(function(t){t in i&&!(t in o)&&(o[t]=i[t])}),"dataRow"in i&&Object.defineProperty(o,"row",{get:function(){return i.dataRow}})),n.cancelable=e,this.canvas.dispatchEvent(new CustomEvent(t,n))}},function(t,e,n){"use strict";var r=n(66).version,i=n(17),o={COLUMNS:1,STRIPES:2,ROWS:3,CELLS:4},a={DEFAULT:[o.COLUMNS,o.STRIPES,o.ROWS,o.CELLS],NO_ROWS:[o.COLUMNS,o.CELLS]},s={themeName:"default",noDataMessage:"",wheelHFactor:.01,wheelVFactor:.05,subgrids:["HeaderSubgrid","data"],font:"13px Tahoma, Geneva, sans-serif",color:"rgb(25, 25, 25)",backgroundColor:"rgb(241, 241, 241)",foregroundSelectionFont:"bold 13px Tahoma, Geneva, sans-serif",foregroundSelectionColor:"rgb(0, 0, 128)",backgroundSelectionColor:"rgba(147, 185, 255, 0.625)",columnHeaderFont:"12px Tahoma, Geneva, sans-serif",columnHeaderColor:"rgb(25, 25, 25)",columnHeaderForegroundSelectionFont:"bold 12px Tahoma, Geneva, sans-serif",columnHeaderBackgroundColor:"rgb(223, 227, 232)",columnHeaderForegroundSelectionColor:"rgb(80, 80, 80)",columnHeaderBackgroundSelectionColor:"rgba(255, 220, 97, 0.45)",columnHeaderHalign:"center",columnHeaderRenderer:"SimpleCell",columnHeaderFormat:"header",rowHeaderFont:"12px Tahoma, Geneva, sans-serif",rowHeaderColor:"rgb(25, 25, 25)",rowHeaderBackgroundColor:"rgb(223, 227, 232)",rowHeaderForegroundSelectionColor:"rgb(80, 80, 80)",rowHeaderForegroundSelectionFont:"bold 12px Tahoma, Geneva, sans-serif",rowHeaderBackgroundSelectionColor:"rgba(255, 220, 97, 0.45)",backgroundColor2:"rgb(201, 201, 201)",treeHeaderFont:"12px Tahoma, Geneva, sans-serif",treeHeaderColor:"rgb(25, 25, 25)",treeHeaderBackgroundColor:"rgb(223, 227, 232)",treeHeaderForegroundSelectionColor:"rgb(80, 80, 80)",treeHeaderForegroundSelectionFont:"bold 12px Tahoma, Geneva, sans-serif",treeHeaderBackgroundSelectionColor:"rgba(255, 220, 97, 0.45)",filterFont:"12px Tahoma, Geneva, sans-serif",filterColor:"rgb(25, 25, 25)",filterBackgroundColor:"white",filterForegroundSelectionColor:"rgb(25, 25, 25)",filterBackgroundSelectionColor:"rgb(255, 220, 97)",filterHalign:"center",filterRenderer:"SimpleCell",filterEditor:"TextField",filterable:!0,showFilterRow:!1,voffset:0,scrollbarHoverOver:"visible",scrollbarHoverOff:"hidden",scrollingEnabled:!0,vScrollbarClassPrefix:"",hScrollbarClassPrefix:"",halign:"center",cellPadding:5,iconPadding:3,leftIcon:void 0,centerIcon:void 0,rightIcon:void 0,renderFalsy:!1,headerify:"toTitle",gridLinesH:!0,gridLinesHWidth:1,gridLinesHColor:"rgb(199, 199, 199)",gridLinesV:!0,gridLinesVWidth:1,gridLinesVColor:"rgb(199, 199, 199)",gridLinesColumnHeader:!0,gridLinesRowHeader:!0,gridLinesUserDataArea:!0,gridBorder:!1,gridBorderLeft:!1,gridBorderRight:!1,gridBorderTop:!1,gridBorderBottom:!1,fixedLinesHWidth:2,fixedLinesHEdge:void 0,fixedLinesHColor:"rgb(164,164,164)",fixedLinesVWidth:2,fixedLinesVEdge:void 0,fixedLinesVColor:"rgb(164,164,164)",boxSizing:r>2?"content-box":"border-box",defaultRowHeight:r>2?14:15,defaultColumnWidth:100,minimumColumnWidth:5,maximumColumnWidth:void 0,resizeColumnInPlace:!1,repaintIntervalRate:60,repaintImmediately:!1,useBitBlit:!1,useHiDPI:!0,navKeyMap:{RETURN:"DOWN",RETURNSHIFT:"UP",TAB:"RIGHT",TABSHIFT:"LEFT"},feedbackCount:3,feedbackEffect:"shaker",readOnly:!1,fixedColumnCount:0,fixedRowCount:0,rowHeaderNumbers:!0,rowHeaderCheckboxes:!0,showTreeColumn:!0,treeRenderer:"SimpleCell",showHeaderRow:!0,cellSelection:!0,columnSelection:!0,rowSelection:!0,singleRowSelectionMode:!0,selectionRegionOverlayColor:"transparent",selectionRegionOutlineColor:"rgb(69, 69, 69)",columnAutosizing:!0,rowNumberAutosizing:!0,treeColumnAutosizing:!0,columnAutosizingMax:400,treeColumnAutosizingMax:400,headerTextWrapping:!1,rowResize:!1,editable:!0,editOnDoubleClick:!0,editOnKeydown:!0,editOnNextCell:!1,unsortable:!1,sortOnDoubleClick:!0,maxSortColumns:3,sortOnHiddenColumns:!0,checkboxOnlyRowSelections:!1,autoSelectRows:!1,autoSelectColumns:!1,collapseCellSelections:!1,format:void 0,editor:void 0,renderer:"SimpleCell",gridRenderer:"by-columns-and-rows",hoverCellHighlight:{enabled:!0,backgroundColor:"rgba(160, 160, 40, 0.45)"},hoverRowHighlight:{enabled:!0,backgroundColor:"rgba(100, 100, 25, 0.30)"},hoverColumnHighlight:{enabled:!0,backgroundColor:"rgba(60, 60, 15, 0.15)"},link:!1,linkTarget:"_blank",linkOnHover:!1,linkColor:"blue",linkVisitedColor:"purple",linkColorOnHover:!1,strikeThrough:!1,multipleSelections:!1,enableContinuousRepaint:!1,columnsReorderable:!0,columnGrabMargin:5,columnClip:!0,rowStripes:void 0,propClassLayers:a.DEFAULT,features:["filters","cellselection","keypaging","columnresizing","rowselection","columnselection","columnmoving","columnsorting","cellclick","cellediting","onhover"],restoreRowSelections:!0,restoreColumnSelections:!0,truncateTextWithEllipsis:!0},A={};function l(){A.rowProperties||(A.rowProperties=!0,console.warn("The `rowProperties` property has been deprecated as of v2.1.0 in favor of `rowStripes`. (Will be removed in a future release.)"))}function c(){throw new i("Attempt to set/get column-only property on a non-column properties object.")}Object.defineProperties(s,{rowProperties:{get:function(){return l(),this.rowStripes},set:function(t){l(),this.rowStripes=t}}}),["name","type","header","calculator"].forEach(function(t){Object.defineProperty(s,t,{set:c})}),Object.defineProperties(s,{mixIn:{value:n(27).mixIn},delete:{value:function(t){var e=Object.getOwnPropertyDescriptor(this,t);return!!e&&(e.get?!e.get.toString().indexOf(".var.")||(this.var[t]=Object.getPrototypeOf(this)[t],this.grid.repaint(),!1):delete this[t])}},propClassEnum:{value:o},propClassLayersMap:{value:a},navKey:{value:function(t,e){var n;return(t.length>1||!this.editOnKeydown||e)&&(n=t),n}},mappedNavKey:{value:function(t,e){return(t=this.navKeyMap[t])&&this.navKey(t)}},reapplyCellProperties:{set:function(t){A.reapplyCellProperties||(console.warn("The `.reapplyCellProperties` property has been deprecated as of v2.1.3 in favor of using the new `.propClassLayers` property. (May be removed in a future release.) This property is now a setter which sets `.propClassLayers` to `.propClassLayersMap.DEFAULT` (grid ← columns ← stripes ← rows ← cells) on truthy or `propClassLayersMap.NO_ROWS` (grid ← columns ← cells) on falsy, which is what you will see on properties stringification. This will give the same effect in most cases as the former property implementation, but not in all cases due to it no longer being applied dynamically. Developers should discontinue use of this property and start specifying `.propClassLayers` instead."),A.reapplyCellProperties=!0),this.propClassLayers=t?a.NO_ROWS:a.DEFAULT}},exec:{value:function(t){if(this.dataRow){var e="f"===(typeof t)[0]&&t||this.calculator;e&&(t=e(this.dataRow,this.name,this.subrow))}return t}}}),t.exports=s},function(t,e,n){"use strict";t.exports=function(t){var e,n,r,i=this.grid.properties,o=this.visibleRows,a=o.length;if(t){r=this.cellEventPool;var s=0;this.visibleColumns.forEachWithNeg(function(t){for(n=0;n<a;n++,s++)e=o[n],r[s].reset(t,e)})}var A,l,c=[],u=i.backgroundColor;this.visibleColumns.forEachWithNeg(function(t){l=t.column.properties.backgroundColor,A&&A.backgroundColor===l?A.right=t.right:l===u?A=void 0:(A={backgroundColor:l,left:t.left,right:t.right},c.push(A))}),this.columnBundles=c}},function(t,e,n){"use strict";var r=n(31).extend("DatasaurLocal",{initialize:function(t,e){this.reset()},reset:function(){this.schema=[],this.data=[]},setData:function(t,e){this.data=t||[],e?this.setSchema(e):this.data.length&&!this.schema.length&&this.setSchema([]),this.dispatchEvent("fin-hypergrid-data-loaded")},getSchema:function(){return this.schema},setSchema:function(t){if(!t.length){var e=this.getFirstRow();e&&(t=Object.keys(e))}this.schema=t,this.dispatchEvent("fin-hypergrid-schema-loaded")},getFirstRow:function(){for(var t in this.data)if(this.data[t])return this.data[t]},getRow:function(t){return this.data[t]},setRow:function(t,e){this.data[t]=e||void 0},getRowMetadata:function(t,e){var n=this.data[t];return n&&(n.__META||void 0!==e&&(n.__META=Object.create(e)))},setRowMetadata:function(t,e){var n=this.data[t];return n&&(e?n.__META=e:delete n.__META),!!n},addRow:function(t,e){1===arguments.length&&(e=arguments[0],t=void 0),void 0===t||t>=this.getRowCount()?this.data.push(e):this.data.splice(t,0,e),this.dispatchEvent("fin-hypergrid-data-shape-changed")},delRow:function(t,e){var n=this.data.splice(t,void 0===e?1:e);return n.length&&this.dispatchEvent("fin-hypergrid-data-shape-changed"),n},getValue:function(t,e){var n=this.data[e];return n?n[this.schema[t].name]:null},setValue:function(t,e,n){this.data[e][this.schema[t].name]=n},getRowCount:function(){return this.data.length},getColumnCount:function(){return this.schema.length}});t.exports=r},function(t,e,n){"use strict";var r=n(84),i=n(7),o=n(85),a=n(86),s=n(54),A=n(88),l=n(35),c=n(6),u=Math.min,g=[].push,h=!c(function(){RegExp(4294967295,"y")});n(90)("split",2,function(t,e,n,c){var d;return d="c"=="abbc".split(/(b)*/)[1]||4!="test".split(/(?:)/,-1).length||2!="ab".split(/(?:ab)*/).length||4!=".".split(/(.?)(.?)/).length||".".split(/()()/).length>1||"".split(/.?/).length?function(t,e){var i=String(this);if(void 0===t&&0===e)return[];if(!r(t))return n.call(i,t,e);for(var o,a,s,A=[],c=(t.ignoreCase?"i":"")+(t.multiline?"m":"")+(t.unicode?"u":"")+(t.sticky?"y":""),u=0,h=void 0===e?4294967295:e>>>0,d=new RegExp(t.source,c+"g");(o=l.call(d,i))&&!((a=d.lastIndex)>u&&(A.push(i.slice(u,o.index)),o.length>1&&o.index<i.length&&g.apply(A,o.slice(1)),s=o[0].length,u=a,A.length>=h));)d.lastIndex===o.index&&d.lastIndex++;return u===i.length?!s&&d.test("")||A.push(""):A.push(i.slice(u)),A.length>h?A.slice(0,h):A}:"0".split(void 0,0).length?function(t,e){return void 0===t&&0===e?[]:n.call(this,t,e)}:n,[function(n,r){var i=t(this),o=null==n?void 0:n[e];return void 0!==o?o.call(n,i,r):d.call(String(i),n,r)},function(t,e){var r=c(d,t,this,e,d!==n);if(r.done)return r.value;var l=i(t),g=String(this),f=o(l,RegExp),p=l.unicode,I=(l.ignoreCase?"i":"")+(l.multiline?"m":"")+(l.unicode?"u":"")+(h?"y":"g"),C=new f(h?l:"^(?:"+l.source+")",I),m=void 0===e?4294967295:e>>>0;if(0===m)return[];if(0===g.length)return null===A(C,g)?[g]:[];for(var y=0,E=0,v=[];E<g.length;){C.lastIndex=h?E:0;var B,b=A(C,h?g:g.slice(E));if(null===b||(B=u(s(C.lastIndex+(h?0:E)),g.length))===y)E=a(g,E,p);else{if(v.push(g.slice(y,E)),v.length===m)return v;for(var Q=1;Q<=b.length-1;Q++)if(v.push(b[Q]),v.length===m)return v;E=y=B}}return v.push(g.slice(y)),v}]})},function(t,e,n){var r=n(34),i=Math.min;t.exports=function(t){return t>0?i(r(t),9007199254740991):0}},function(t,e,n){t.exports=!n(9)&&!n(6)(function(){return 7!=Object.defineProperty(n(56)("div"),"a",{get:function(){return 7}}).a})},function(t,e,n){var r=n(13),i=n(5).document,o=r(i)&&r(i.createElement);t.exports=function(t){return o?i.createElement(t):{}}},function(t,e,n){for(var r=n(95),i=n(26),o=n(14),a=n(5),s=n(8),A=n(38),l=n(1),c=l("iterator"),u=l("toStringTag"),g=A.Array,h={CSSRuleList:!0,CSSStyleDeclaration:!1,CSSValueList:!1,ClientRectList:!1,DOMRectList:!1,DOMStringList:!1,DOMTokenList:!0,DataTransferItemList:!1,FileList:!1,HTMLAllCollection:!1,HTMLCollection:!1,HTMLFormElement:!1,HTMLSelectElement:!1,MediaList:!0,MimeTypeArray:!1,NamedNodeMap:!1,NodeList:!0,PaintRequestList:!1,Plugin:!1,PluginArray:!1,SVGLengthList:!1,SVGNumberList:!1,SVGPathSegList:!1,SVGPointList:!1,SVGStringList:!1,SVGTransformList:!1,SourceBufferList:!1,StyleSheetList:!0,TextTrackCueList:!1,TextTrackList:!1,TouchList:!1},d=i(h),f=0;f<d.length;f++){var p,I=d[f],C=h[I],m=a[I],y=m&&m.prototype;if(y&&(y[c]||s(y,c,g),y[u]||s(y,u,I),A[I]=g,C))for(p in r)y[p]||o(y,p,r[p],!0)}},function(t,e,n){var r=n(7),i=n(101),o=n(40),a=n(39)("IE_PROTO"),s=function(){},A=function(){var t,e=n(56)("iframe"),r=o.length;for(e.style.display="none",n(104).appendChild(e),e.src="javascript:",(t=e.contentWindow.document).open(),t.write("<script>document.F=Object<\/script>"),t.close(),A=t.F;r--;)delete A.prototype[o[r]];return A()};t.exports=Object.create||function(t,e){var n;return null!==t?(s.prototype=r(t),n=new s,s.prototype=null,n[a]=t):n=A(),void 0===e?n:i(n,e)}},function(t,e,n){var r=n(11),i=n(15),o=n(102)(!1),a=n(39)("IE_PROTO");t.exports=function(t,e){var n,s=i(t),A=0,l=[];for(n in s)n!=a&&r(s,n)&&l.push(n);for(;e.length>A;)r(s,n=e[A++])&&(~o(l,n)||l.push(n));return l}},function(t,e,n){n(61)("asyncIterator")},function(t,e,n){var r=n(5),i=n(21),o=n(22),a=n(62),s=n(10).f;t.exports=function(t){var e=i.Symbol||(i.Symbol=o?{}:r.Symbol||{});"_"==t.charAt(0)||t in e||s(e,t,{value:a.f(t)})}},function(t,e,n){e.f=n(1)},function(t,e,n){"use strict";var r=n(5),i=n(11),o=n(9),a=n(24),s=n(14),A=n(107).KEY,l=n(6),c=n(20),u=n(41),g=n(16),h=n(1),d=n(62),f=n(61),p=n(108),I=n(109),C=n(7),m=n(13),y=n(42),E=n(15),v=n(37),B=n(25),b=n(58),Q=n(110),w=n(111),x=n(64),_=n(10),D=n(26),M=w.f,k=_.f,S=Q.f,L=r.Symbol,R=r.JSON,N=R&&R.stringify,F=h("_hidden"),T=h("toPrimitive"),U={}.propertyIsEnumerable,O=c("symbol-registry"),G=c("symbols"),H=c("op-symbols"),j=Object.prototype,Y="function"==typeof L&&!!x.f,P=r.QObject,K=!P||!P.prototype||!P.prototype.findChild,z=o&&l(function(){return 7!=b(k({},"a",{get:function(){return k(this,"a",{value:7}).a}})).a})?function(t,e,n){var r=M(j,e);r&&delete j[e],k(t,e,n),r&&t!==j&&k(j,e,r)}:k,J=function(t){var e=G[t]=b(L.prototype);return e._k=t,e},q=Y&&"symbol"==typeof L.iterator?function(t){return"symbol"==typeof t}:function(t){return t instanceof L},V=function(t,e,n){return t===j&&V(H,e,n),C(t),e=v(e,!0),C(n),i(G,e)?(n.enumerable?(i(t,F)&&t[F][e]&&(t[F][e]=!1),n=b(n,{enumerable:B(0,!1)})):(i(t,F)||k(t,F,B(1,{})),t[F][e]=!0),z(t,e,n)):k(t,e,n)},W=function(t,e){C(t);for(var n,r=p(e=E(e)),i=0,o=r.length;o>i;)V(t,n=r[i++],e[n]);return t},X=function(t){var e=U.call(this,t=v(t,!0));return!(this===j&&i(G,t)&&!i(H,t))&&(!(e||!i(this,t)||!i(G,t)||i(this,F)&&this[F][t])||e)},Z=function(t,e){if(t=E(t),e=v(e,!0),t!==j||!i(G,e)||i(H,e)){var n=M(t,e);return!n||!i(G,e)||i(t,F)&&t[F][e]||(n.enumerable=!0),n}},$=function(t){for(var e,n=S(E(t)),r=[],o=0;n.length>o;)i(G,e=n[o++])||e==F||e==A||r.push(e);return r},tt=function(t){for(var e,n=t===j,r=S(n?H:E(t)),o=[],a=0;r.length>a;)!i(G,e=r[a++])||n&&!i(j,e)||o.push(G[e]);return o};Y||(s((L=function(){if(this instanceof L)throw TypeError("Symbol is not a constructor!");var t=g(arguments.length>0?arguments[0]:void 0),e=function(n){this===j&&e.call(H,n),i(this,F)&&i(this[F],t)&&(this[F][t]=!1),z(this,t,B(1,n))};return o&&K&&z(j,t,{configurable:!0,set:e}),J(t)}).prototype,"toString",function(){return this._k}),w.f=Z,_.f=V,n(65).f=Q.f=$,n(43).f=X,x.f=tt,o&&!n(22)&&s(j,"propertyIsEnumerable",X,!0),d.f=function(t){return J(h(t))}),a(a.G+a.W+a.F*!Y,{Symbol:L});for(var et="hasInstance,isConcatSpreadable,iterator,match,replace,search,species,split,toPrimitive,toStringTag,unscopables".split(","),nt=0;et.length>nt;)h(et[nt++]);for(var rt=D(h.store),it=0;rt.length>it;)f(rt[it++]);a(a.S+a.F*!Y,"Symbol",{for:function(t){return i(O,t+="")?O[t]:O[t]=L(t)},keyFor:function(t){if(!q(t))throw TypeError(t+" is not a symbol!");for(var e in O)if(O[e]===t)return e},useSetter:function(){K=!0},useSimple:function(){K=!1}}),a(a.S+a.F*!Y,"Object",{create:function(t,e){return void 0===e?b(t):W(b(t),e)},defineProperty:V,defineProperties:W,getOwnPropertyDescriptor:Z,getOwnPropertyNames:$,getOwnPropertySymbols:tt});var ot=l(function(){x.f(1)});a(a.S+a.F*ot,"Object",{getOwnPropertySymbols:function(t){return x.f(y(t))}}),R&&a(a.S+a.F*(!Y||l(function(){var t=L();return"[null]"!=N([t])||"{}"!=N({a:t})||"{}"!=N(Object(t))})),"JSON",{stringify:function(t){for(var e,n,r=[t],i=1;arguments.length>i;)r.push(arguments[i++]);if(n=e=r[1],(m(e)||void 0!==t)&&!q(t))return I(e)||(e=function(t,e){if("function"==typeof n&&(e=n.call(this,t,e)),!q(e))return e}),r[1]=e,N.apply(R,r)}}),L.prototype[T]||n(8)(L.prototype,T,L.prototype.valueOf),u(L,"Symbol"),u(Math,"Math",!0),u(r.JSON,"JSON",!0)},function(t,e){e.f=Object.getOwnPropertySymbols},function(t,e,n){var r=n(59),i=n(40).concat("length","prototype");e.f=Object.getOwnPropertyNames||function(t){return r(t,i)}},function(t){t.exports=JSON.parse('{"name":"fin-hypergrid","version":"3.2.0","description":"Canvas-based high-performance grid","main":"src/Hypergrid","repository":{"type":"git","url":"git://github.com/fin-hypergrid/core.git"},"author":"Steve Wirts (https://github.com/stevewirts)","contributors":["Jonathan Eiten (https://github.com/joneit)","Dwayne Jones (https://github.com/dwaynekj)","Naveen Michaud-Agrawal (https://github.com/nmichaud)","Hugo Nogueira (https://github.com/hugoeanogueira)"],"license":"MIT","readmeFilename":"README.md","gitHead":"","keywords":["spreadsheet","grid"],"dependencies":{"datasaur-base":"^3.0.0","datasaur-local":"^3.0.0","extend-me":"^2.7.0","finbars":"^2.0.0","inject-stylesheet-template":"^1.0.1","mustache":"^2.3.0","object-iterators":"1.3.0","overrider":"^0","rectangular":"1.0.1","sparse-boolean-array":"1.0.1","svg-themer":"^1.1.2","synonomous":"^2.1.2"},"devDependencies":{"gulp":"^3.9.0","gulp-concat":"^2.6.0","gulp-each":"^0.1.1","gulp-eslint":"^4.0.2","gulp-footer":"^1.1.1","gulp-header":"^1.8.2","gulp-imagine-64":"^2.0.1","gulp-load-plugins":"^1.1.0","gulp-mocha":"^6.0.0","run-sequence":"^1.1.4"}}')},function(t,e,n){"use strict";e.mixin={getFixedRowsHeight:function(){for(var t,e,n,r,i=this.subgrids,o=0,a=this.grid.properties,s="border-box"!==a.boxSizing,A=a.gridLinesHWidth,l=0;l<i.length&&!e;++l){for(r=(e=(t=i[l]).isData)?a.fixedRowCount:t.getRowCount(),n=0;n<r;++n)o+=this.getRowHeight(n,t),s&&(o+=A);e&&a.fixedLinesHWidth&&(o+=a.fixedLinesHWidth-A)}return o},getRowProperties:function(t,e,n){"object"==typeof t&&(n=t.subgrid,t=t.dataCell.y);var r=(n||this.dataModel).getRowMetadata(t,void 0===e?void 0:null);return r&&(r.__ROW||void 0!==e&&(r.__ROW=Object.create(e)))},setRowProperties:function(t,e,n){if(e){"object"==typeof t&&(n=t.subgrid,t=t.dataCell.y);var r=(n||this.dataModel).getRowMetadata(t,null);r&&(r.__ROW=Object.create(this.rowPropertiesPrototype),this.addRowProperties(t,e,n,r.__ROW),this.stateChanged())}},setRowProperty:function(t,e,n,r){var i,o="height"===e;void 0!==n?(i=this.getRowProperties(t,this.rowPropertiesPrototype,r))[e]=n:(i=this.getRowProperties(t,void 0,r))&&delete i[o?"_height":e],o?this.shapeChanged():this.stateChanged()},addRowProperties:function(t,e,n,r){var i,o;e&&(r=r||this.getRowProperties(t,this.rowPropertiesPrototype,n))&&(Object.keys(e).forEach(function(t){var n=e[t];void 0!==n?r[t]=n:(delete r[(i="height"===t)?"_height":t],o=o||i)}),o?this.shapeChanged():this.stateChanged())},getRowHeight:function(t,e){var n=this.getRowProperties(t,void 0,e);return n&&n.height||this.grid.properties.defaultRowHeight},setRowHeight:function(t,e,n){this.setRowProperty(t,"height",e,n)}},e.rowPropertiesPrototypeDescriptors={height:{enumerable:!0,get:function(){return this._height||this.defaultRowHeight},set:function(t){t=Math.max(5,Math.ceil(t)),isNaN(t)&&(t=void 0),t!==this._height&&(t?Object.defineProperty(this,"_height",{value:t,configurable:!0}):delete this._height,this.grid.behaviorStateChanged())}}}},function(t,e,n){"use strict";n(143);var r=n(7),i=n(36),o=n(9),a=/./.toString,s=function(t){n(14)(RegExp.prototype,"toString",t,!0)};n(6)(function(){return"/a/b"!=a.call({source:"a",flags:"b"})})?s(function(){var t=r(this);return"/".concat(t.source,"/","flags"in t?t.flags:!o&&t instanceof RegExp?i.call(t):void 0)}):"toString"!=a.name&&s(function(){return a.call(this)})},function(t,e,n){"use strict";var r,i={theme:{enumerable:!0,get:function(){return this.grid.getTheme()},set:function(t){this.grid.applyTheme(t)}},wheelHFactor:{enumerable:!0,get:function(){return this.grid.sbHScroller.deltaXFactor},set:function(t){this.grid.sbHScroller.deltaXFactor=t}},wheelVFactor:{enumerable:!0,get:function(){return this.grid.sbVScroller.deltaYFactor},set:function(t){this.grid.sbVScroller.deltaYFactor=t}},subgrids:{enumerable:!0,get:function(){return this.var.subgrids},set:function(t){this.var.subgrids=t.slice(),this.grid.behavior&&(this.grid.behavior.subgrids=t)}},features:{enumerable:!0,get:function(){return this.var.features},set:function(t){this.var.features=t.slice(),this.grid.behavior&&(this.grid.behavior.initializeFeatureChain(t),this.grid.allowEvents(this.grid.getRowCount()))}},gridRenderer:{enumerable:!0,get:function(){return this.var.gridRenderer},set:function(t){this.var.gridRenderer=t,this.grid.renderer.setGridRenderer(t)}},columnIndexes:{enumerable:!0,get:function(){return this.grid.behavior.getActiveColumns().map(function(t){return t.index})},set:function(t){this.grid.behavior.setColumnOrder(t),this.grid.behavior.changed()}},columnNames:{enumerable:!0,get:function(){return this.grid.behavior.getActiveColumns().map(function(t){return t.name})},set:function(t){this.grid.behavior.setColumnOrderByName(t),this.grid.behavior.changed()}},rows:{enumerable:!0,get:function(){var t={},e=this.grid.behavior,n=this.grid.properties.defaultRowHeight;return e.subgrids.forEach(function(r){for(var i=r.name||r.type,o=0,a=r.getRowCount();o<a;++o){var s=e.getRowProperties(o,void 0,r);if(s){var A={height:s.height};A.height===n&&(A=void 0),s=Object.assign({},s,A),Object.getOwnPropertyNames(s).find(l)&&((t[i]||(t[i]={}))[o]=s)}}function l(t){return void 0!==s[t]}}),t},set:function(t){t&&(function(t){var e=this.grid.behavior,n=this.settingState?"setRowProperties":"addRowProperties";Object.keys(t).forEach(function(r){var i=e.subgrids.lookup[r];if(i){var o=t[r];Object.keys(o).forEach(function(t){var r=o[t];e[n](t,r,i)})}})}.call(this,t),this.grid.behavior.changed())}},columns:{enumerable:!0,get:function(){var t=this.grid.behavior.getColumns(),e=this.grid.headerify;return t.reduce(function(t,n){var r=Object.keys(n.properties).reduce(function(t,r){switch(r){case"preferredWidth":break;case"header":if(e&&n.properties.header===e(n.properties.name))break;default:t[r]=n.properties[r]}return t},{});return Object.keys(r).length&&(t[n.name]=r),t},{})},set:function(t){t&&(function(t){this.grid.behavior.addAllColumnProperties(t,this.settingState)}.call(this,t),this.grid.behavior.changed())}},cells:{enumerable:!0,get:function(){var t=this.grid.behavior,e=t.getColumns(),n={};return t.subgrids.forEach(function(r){for(var i=r.name||r.type,o=0,a=r.getRowCount();o<a;++o)e.forEach(s);function s(e){var a=t.getCellOwnProperties(e.index,o,r);if(a){var s=n[i]=n[i]||{};(s[o]=s[o]={})[e.name]=Object.assign({},a)}}}),n},set:function(t){t&&(function(t){var e=this.grid.behavior.subgrids,n=this.grid.behavior.getColumns(),r=this.settingState?"setCellProperties":"addCellProperties";Object.keys(t).forEach(function(i){var o=e.lookup[i];if(o){var a=t[i];Object.keys(a).forEach(function(t){var e=a[t];Object.keys(e).forEach(function(i){var a=e[i];if(a){var s=n.find(function(t){return t.name===i});s&&s[r](t,a,o)}})})}})}.call(this,t),this.grid.behavior.changed())}},rowHeaderCheckboxes:{enumerable:!0,get:function(){return this.var.rowHeaderCheckboxes},set:function(t){this.var.rowHeaderCheckboxes=t,this.grid.renderer.resetRowHeaderColumnWidth()}},rowHeaderNumbers:{enumerable:!0,get:function(){return this.var.rowHeaderNumbers},set:function(t){this.var.rowHeaderNumbers=t,this.grid.renderer.resetRowHeaderColumnWidth()}},showRowNumbers:{enumerable:!1,get:function(){return this.rowHeaderCheckboxes||this.rowHeaderNumbers},set:function(t){this.rowHeaderCheckboxes=this.rowHeaderNumbers=t}},doubleClickDelay:{enumerable:!0,get:function(){return this.var.doubleClickDelay},set:function(t){r||(r=!0,console.warn("The doubleClickDelay property has been deprecated as of v2.1.0. Setting this property no longer has any effect. Set double-click speed in your system's mouse preferences. (This warning will be removed in a future release.)")),this.var.doubleClickDelay=t}},lineColor:{get:function(){return this.gridLinesHColor},set:function(t){this.gridLinesHColor=this.gridLinesVColor=t}},lineWidth:{get:function(){return this.gridLinesHWidth},set:function(t){this.gridLinesHWidth=this.gridLinesVWidth=t}},gridBorder:o(),gridBorderLeft:o("Left"),gridBorderRight:o("Right"),gridBorderTop:o("Top"),gridBorderBottom:o("Bottom")};function o(t){var e="gridBorder"+(t||"");return{enumerable:!0,get:function(){return this.var[e]},set:function(n){this.var[e]=n,t||(this.var.gridBorderLeft=this.var.gridBorderRight=this.var.gridBorderTop=this.var.gridBorderBottom=n),this.grid.resetGridBorder(t)}}}i.columnProperties=i.columns,t.exports=i},function(t,e,n){"use strict";var r=n(2).Rectangle;function i(t,e,n,i){r.call(this,t,e,n-1,i-1)}i.prototype=Object.create(r.prototype,{width:{get:function(){return this.extent.x+1}},height:{get:function(){return this.extent.y+1}}}),t.exports=i},function(t,e,n){"use strict";var r=n(51),i=n(72);function o(t){var e,n,o,a,s,A,l,c,u,g,h,d=this.grid.properties.backgroundColor,f=this.visibleColumns,p=this.visibleRows,I=f.length,C=I-1,m=p.length,y=this.cellEventPool,E=I?f[I-1].right:0,v=m?p[m-1].bottom:0;if(t.clearRect(0,0,this.bounds.width,this.bounds.height),I&&m){if(t.alpha(d)>0&&(t.cache.fillStyle=d,t.fillRect(0,0,E,v)),this.gridRenderer.reset?(this.resetAllGridRenderers(),this.gridRenderer.reset=!1,i.call(this,!1),r.call(this,!0)):this.gridRenderer.rebundle&&(this.gridRenderer.rebundle=!1,r.call(this)),(s=this.rowBundles).length)for(n=this.rowPrefillColors,u=s.length;u--;)a=s[u],t.clearFill(0,a.top,E,a.bottom-a.top,a.backgroundColor);else for(c=(l=this.columnBundles).length;c--;)A=l[c],t.clearFill(A.left,0,A.right-A.left,v,A.backgroundColor);var B=0;this.visibleColumns.forEachWithNeg(function(r,i){for(r=(o=y[B]).visibleColumn,n||(e=o.column.properties.backgroundColor),h=r.column.properties.columnClip,t.clipSave(h||null===h&&i===C,0,0,r.right,v),g=u=0;u<m;u++,B++){n&&(e=n[u]);try{g=Math.max(g,this._paintCell(t,y[B],e))}catch(e){this.renderErrorCell(e,t,r,y[B].visibleRow)}}t.clipRestore(h),o.column.properties.preferredWidth=Math.round(g)},this),this.paintGridlines(t)}}o.key="by-columns-and-rows",o.rebundle=!0,t.exports=o},function(t,e,n){"use strict";t.exports=function(t){var e,n,r,i,o=this.grid.properties,a=this.visibleRows,s=a.length;if(t)for(i=this.cellEventPool,r=0,n=0;n<s;n++)e=a[n],this.visibleColumns.forEachWithNeg(function(t){i[++r].reset(t,e)});var A,l,c,u=[],g=o.backgroundColor,h=o.rowStripes,d=Array(s);for(n=0;n<s;n++)l=(e=a[n]).subgrid.isData&&h&&h[e.rowIndex%h.length],c=d[n]=l&&l.backgroundColor||g,A&&A.backgroundColor===c?A.bottom=e.bottom:c===g?A=void 0:(A={backgroundColor:c,top:e.top,bottom:e.bottom},u.push(A));this.rowBundles=u,this.rowPrefillColors=d}},function(t,e,n){"use strict";(function(t){!function(t){function e(){this.selection=[],this.states=[],this.storeState=function(){for(var t,e=this.selection,n=[],r=0;r<e.length;r++)t=[].concat(e[r]),n.push(t);this.states.push(n)}}function n(t,e){return t instanceof Array?n.apply(this,t):void 0===e?[t,t]:t<=e?[t,e]:[e,t]}function r(t,e){return t[0]<=e[0]&&e[0]<=t[1]||t[0]<=e[1]&&e[1]<=t[1]||e[0]<t[0]&&t[1]<e[1]}e.prototype={select:function(t,e){this.storeState();var i=n(t,e),o=[0,1];return this.selection.forEach(function(t){var e,n;r(t,i)||(n=i,(e=t)[1]===n[0]-1||n[1]===e[0]-1)?i=function(t,e){return[Math.min(Math.min.apply(Math,t),Math.min.apply(Math,e)),Math.max(Math.max.apply(Math,t),Math.max.apply(Math,e))]}(t,i):o.push(t)}),o.push(i),o[1]=this.selection.length,this.selection.splice.apply(this.selection,o),this},deselect:function(t,e){var i=n(t,e),o=[0,0];return this.selection.forEach(function(t){if(r(t,i)){var e=function(t,e){var n=t[0],r=t[1],i=e[0],o=e[1],a=[];return i<=n&&o<r?a.push([o+1,r]):i>n&&o>=r?a.push([n,i-1]):n<i&&o<r?(a.push([n,i-1]),a.push([o+1,r])):(o<n||i>r)&&a.push(t),a}(t,i);o=o.concat(e)}else o.push(t)}),o[1]=this.selection.length,this.selection.splice.apply(this.selection,o),this},clear:function(){return this.states.length=0,this.selection.length=0,this},clearMostRecentSelection:function(){0!==this.states.length&&(this.selection=this.states.pop())},isSelected:function(t){return this.selection.some(function(e){return e[0]<=t&&t<=e[1]})},isEmpty:function(){return 0===this.selection.length},getSelections:function(){var t=[];return this.selection.forEach(function(e){for(var n=e[0];n<=e[1];n++)t.push(n)}),t.sort(function(t,e){return t-e}),t}},t.exports=e}(t||(window.RangeSelectionModel={}),t.exports||(window.RangeSelectionModel.exports={}))}).call(this,n(32)(t))},function(t,e,n){"use strict";t.exports={"fin-hypergrid-schema-loaded":function(t){this.behavior.createColumns()},"fin-hypergrid-data-loaded":function(t){this.repaint()},"fin-hypergrid-data-shape-changed":function(t){this.behaviorShapeChanged()},"fin-hypergrid-data-prereindex":function(t){this.stashRowSelections(),this.stashColumnSelections()},"fin-hypergrid-data-postreindex":function(t){this.selectionModel.reset(),this.unstashRowSelections(),this.unstashColumnSelections(),this.behaviorShapeChanged()}}},function(t,e,n){"use strict";var r=n(29),i=n(47),o={};function a(t,e){o[t]||(console.warn("."+t+" has been deprecated as of v3.0.0. (Will be removed in a future release.) "+(e||"")),o[t]=!0)}e.mixin={columnEnumSynchronize:function(){this._columnEnumKey=this._columnEnumKey||"toAllCaps";var t=this._columnEnum||(this._columnEnum={}),e=this.allColumns,n=new r({transformations:[this._columnEnumKey]}).decorateArray(e.slice());n.length=0,Object.keys(n).reduce(function(t,e){return t[e]=n[e].index,t},t),Object.keys(t).forEach(function(e){e in n||delete t[e]})},get columnEnum(){return o.columnEnum||(console.warn(".columnEnum[propName] has been deprecated as of v3.0.0 in favor of either .getColumns()[propName].index or .schema[propName].index. (Will be removed in a future release.)"),o.columnEnum=!0),this._columnEnum},get columnEnumKey(){return a("columnEnumKey"),"verbatim"===this._columnEnumKey?"passThrough":this._columnEnumKey},set columnEnumKey(t){a("columnEnumKey");var e=typeof t,n=Object.keys(i);switch(e){case"string":if("passThrough"===t)t="verbatim";else if(!(t in i))throw new this.HypergridError("Expected registered transformer for .columnEnumKey value from: "+n);this._columnEnumKey=t;break;case"function":if(this._columnEnumKey=n.find(function(e){return t===i[e]}),!this._columnEnumKey)throw new this.HypergridError('.columnEnumKey has been deprecated as of v3.0.0 and now accepts a function reference (or string key) from require("synonmous/transformers"): '+n);break;default:throw new this.HypergridError("Expected string or function for .columnEnumKey assignment but received "+e+".")}}},e.mixInShared={get columnEnumDecorators(){return a("columnEnumDecorators"),i}}},function(t,e,n){"use strict";var r=n(4).extend("Button",{paint:function(t,e){var n=e.value,r=e.bounds,i=r.x+1,o=r.y+1,a=r.width-2,s=r.height-2,A=s/2,l=t.createLinearGradient(i,o,i,o+s);"border-box"===e.boxSizing&&(a-=e.gridLinesVWidth,s-=e.gridLinesHWidth),e.mouseDown?(l.addColorStop(0,"#B5CBED"),l.addColorStop(1,"#4d74ea")):(l.addColorStop(0,"#ffffff"),l.addColorStop(1,"#aaaaaa")),t.cache.fillStyle=e.backgroundColor,t.fillRect(r.x,r.y,r.width,r.height),t.cache.fillStyle=l,t.cache.strokeStyle="#000000",this.roundRect(t,i,o,a,s,A,l,!0);var c=(a-t.getTextWidth(n))/2,u=(s-t.getTextHeight(t.cache.font).descent)/2;t.cache.textBaseline="middle",t.cache.fillStyle="#333333",t.cache.font=s-2+"px sans-serif",e.backgroundColor="rgba(0,0,0,0)",t.fillText(n,i+c,o+u)}});t.exports=r},function(t,e,n){"use strict";var r=n(12),i=n(48),o=r.extend("Textfield",{template:'<input type="text" lang="{{locale}}" class="hypergrid-textfield" style="{{style}}">',initialize:function(){this.input.style.textAlign=this.event.properties.halign,this.input.style.font=this.event.properties.font},localizer:i.prototype.string,selectAll:function(){this.input.setSelectionRange(0,this.input.value.length)}});t.exports=o},function(t,e,n){"use strict";t.exports={Scrollbar:n(79),templater:n(180)},Object.defineProperties(t.exports,{"datasaur-base":{value:n(31)},"datasaur-local":{value:n(52)},"extend-me":{value:n(45)},finbars:{value:n(79)},"object-iterators":{value:n(28)},overrider:{value:n(27)},rectangular:{value:n(2)},"sparse-boolean-array":{value:n(73)},synonomous:{value:n(29)}})},function(t,e,n){"use strict";var r=n(179),i="position: absolute;",o="position: absolute;";function a(t){var e=this._bound={};Object.keys(A).forEach(function(t){e[t]=A[t].bind(this)},this);var n=this.thumb=document.createElement("div");n.classList.add("thumb"),n.setAttribute("style",o),n.onclick=e.shortStop,n.onmouseover=e.onmouseover,n.onmouseout=this._bound.onmouseout;var c,u,g,h,d=this.bar=document.createElement("div");d.classList.add("finbar-vertical"),d.setAttribute("style",i),d.onmousedown=this._bound.onmousedown,this.paging&&(d.onclick=e.onclick),d.appendChild(n),t=t||{},this.orientation="vertical",this._min=this._index=0,this._max=100,this.normal=(u=(c=window.navigator).userAgent,g=c.platform.substr(0,3).toLowerCase(),h=/Edge/.test(u)?"edge":/Opera|OPR|Chrome|Safari/.test(u)?"webkit":/Firefox/.test(u)?"moz":document.documentMode?"ms":void 0,(a.normals[g]||{})[h]||1),Object.keys(t).forEach(function(e){var n=t[e];if(void 0!==n)switch(e){case"index":this._index=n;break;case"range":s(n),this._min=n.min,this._max=n.max,this.contentSize=n.max-n.min+1;break;default:"_"!==e.charAt(0)&&"function"!=typeof a.prototype[e]&&(this[e]=n)}},this),r(l,"finbar-base",t.cssStylesheetReferenceElement)}function s(t){2===Object.keys(t).length&&"number"==typeof t.min&&"number"==typeof t.max&&t.min<=t.max||g("Invalid .range object.")}a.prototype={set orientation(t){t!==this._orientation&&(this._orientation=t,this.oh=c[this._orientation],this.oh||g("Invalid value for `options._orientation."),this.deltaProp=this.oh.delta,this.bar.className=this.bar.className.replace(/(vertical|horizontal)/g,t),this.bar.style.cssText===i&&this.thumb.style.cssText===o||(this.bar.setAttribute("style",i),this.thumb.setAttribute("style",o),this.resize()))},get orientation(){return this._orientation},onchange:null,set classPrefix(t){this._classPrefix&&this.bar.classList.remove(this._classPrefix+this.orientation),this._classPrefix=t,t&&this.bar.classList.add(t+"-"+this.orientation)},get classPrefix(){return this._classPrefix},increment:1,deltaXFactor:1,deltaYFactor:1,deltaZFactor:1,barStyles:null,set style(t){var e=Object.keys(t=function(t){for(var e=1;e<arguments.length;++e){var n=arguments[e];if(n)for(var r in n)t[r]=n[r]}return t}({},t,this._auxStyles));if(e.length){var n=this.bar,r=n.getBoundingClientRect(),o=(this.container||n.parentElement).getBoundingClientRect(),a=this.oh;n.setAttribute("style",i),e.forEach(function(e){var i=t[e];if(e in a&&(e=a[e]),isNaN(Number(i))){if(/%$/.test(i)){var s=u[e],A=r[s.marginLeading]+r[s.marginTrailing];A&&(i=parseInt(i,10)/100*o[s.size]-A+"px")}}else i=(i||0)+"px";n.style[e]=i})}},paging:!0,set range(t){s(t),this._min=t.min,this._max=t.max,this.contentSize=t.max-t.min+1,this.index=this.index},get range(){return{min:this._min,max:this._max}},set index(t){t=Math.min(this._max,Math.max(this._min,t)),this._setScroll(t)},get index(){return this._index},_setScroll:function(t,e){this._index=t,this.testPanelItem&&this.testPanelItem.index instanceof Element&&(this.testPanelItem.index.innerHTML=Math.round(t)),this.onchange&&this.onchange.call(this,Math.round(t)),void 0===e&&(e=(t-this._min)/(this._max-this._min)*this._thumbMax),this.thumb.style[this.oh.leading]=e+"px"},scrollRealContent:function(t){var e=this.content.parentElement.getBoundingClientRect(),n=this.oh.size,r=Math.max(0,this.content[n]-e[n]),i=(t-this._min)/(this._max-this._min)*r;this.content.style[this.oh.leading]=-i+"px"},resize:function(t,e){var n=this.bar;if(n.parentNode){var r=this.container||n.parentElement,i=r.getBoundingClientRect();"object"==typeof t&&(e=t,t=void 0),this.style=this.barStyles=e||this.barStyles,this.content&&(this.onchange||(this.onchange=this.scrollRealContent,this.contentSize=this.content[this.oh.size],this._min=0,this._max=this.contentSize-1)),this.onchange===this.scrollRealContent?(this.containerSize=i[this.oh.size],this.increment=this.containerSize/(this.contentSize-this.containerSize)*(this._max-this._min)):(this.containerSize=1,this.increment=t||this.increment);var o=this.index;return this.testPanelItem=this.testPanelItem||this._addTestPanelItem(),this._setThumbSize(),this.index=o,null!==this.deltaProp&&r.addEventListener("wheel",this._bound.onwheel),this}},shortenBy:function(t){return this.shortenEndBy("trailing",t)},foreshortenBy:function(t){return this.shortenEndBy("leading",t)},shortenEndBy:function(t,e){if(e){if(e instanceof a&&e.orientation!==this.orientation){var n=window.getComputedStyle(e.bar),r=c[e.orientation];this._auxStyles={},this._auxStyles[t]=n[r.thickness]}}else delete this._auxStyles;return this},_setThumbSize:function(){var t=this.oh,e=window.getComputedStyle(this.thumb),n=parseInt(e[t.marginLeading]),r=n+parseInt(e[t.marginTrailing]),i=this.bar.getBoundingClientRect()[t.size],o=Math.max(20,i*this.containerSize/this.contentSize);this.containerSize<this.contentSize?(this.bar.style.visibility="visible",this.thumb.style[t.size]=o+"px"):this.bar.style.visibility="hidden",this._thumbMax=i-o-r,this._thumbMarginLeading=n},remove:function(){this.bar.onmousedown=null,this._removeEvt("mousemove"),this._removeEvt("mouseup"),(this.container||this.bar.parentElement)._removeEvt("wheel"),this.bar.onclick=this.thumb.onclick=this.thumb.onmouseover=this.thumb.transitionend=this.thumb.onmouseout=null,this.bar.remove()},_addTestPanelItem:function(){var t,e=document.querySelector("."+this._classPrefix+".test-panel")||document.querySelector(".test-panel");if(e){var n=["mousedown","mousemove","mouseup","index"],r=document.createElement("li");n.forEach(function(t){r.innerHTML+='<span class="'+t+'">'+t.replace("mouse","")+"</span>"}),e.appendChild(r),t={},n.forEach(function(e){t[e]=r.getElementsByClassName(e)[0]})}return t},_addEvt:function(t){var e=this.testPanelItem&&this.testPanelItem[t];e&&e.classList.add("listening"),window.addEventListener(t,this._bound["on"+t])},_removeEvt:function(t){var e=this.testPanelItem&&this.testPanelItem[t];e&&e.classList.remove("listening"),window.removeEventListener(t,this._bound["on"+t])}};var A={shortStop:function(t){t.stopPropagation()},onwheel:function(t){this.index+=t[this.deltaProp]*this[this.deltaProp+"Factor"]*this.normal,t.stopPropagation(),t.preventDefault()},onclick:function(t){var e=this.thumb.getBoundingClientRect(),n=t[this.oh.coordinate]<e[this.oh.leading];"object"==typeof this.paging?this.index=this.paging[n?"up":"down"](Math.round(this.index)):this.index+=n?-this.increment:this.increment,this.thumb.classList.add("hover");var r=this;this.thumb.addEventListener("transitionend",function e(){this.removeEventListener("transitionend",e),r._bound.onmouseup(t)}),t.stopPropagation()},onmouseover:function(){this.thumb.classList.add("hover")},onmouseout:function(){this.dragging||this.thumb.classList.remove("hover")},onmousedown:function(t){var e=this.thumb.getBoundingClientRect();this.pinOffset=t[this.oh.axis]-e[this.oh.leading]+this.bar.getBoundingClientRect()[this.oh.leading]+this._thumbMarginLeading,document.documentElement.style.cursor="default",this.dragging=!0,this._addEvt("mousemove"),this._addEvt("mouseup"),t.stopPropagation(),t.preventDefault()},onmousemove:function(t){if(1&t.buttons){var e=Math.min(this._thumbMax,Math.max(0,t[this.oh.axis]-this.pinOffset)),n=e/this._thumbMax*(this._max-this._min)+this._min;this._setScroll(n,e),t.stopPropagation(),t.preventDefault()}else window.dispatchEvent(new MouseEvent("mouseup",t))},onmouseup:function(t){this._removeEvt("mousemove"),this._removeEvt("mouseup"),this.dragging=!1,document.documentElement.style.cursor="auto";var e=this.thumb.getBoundingClientRect();e.left<=t.clientX&&t.clientX<=e.right&&e.top<=t.clientY&&t.clientY<=e.bottom?this._bound.onmouseover(t):this._bound.onmouseout(t),t.stopPropagation(),t.preventDefault()}};a.normals={mac:{webkit:1,moz:35},win:{webkit:2.6,moz:85,ms:2.9,edge:2}};var l,c={vertical:{coordinate:"clientY",axis:"pageY",size:"height",outside:"right",inside:"left",leading:"top",trailing:"bottom",marginLeading:"marginTop",marginTrailing:"marginBottom",thickness:"width",delta:"deltaY"},horizontal:{coordinate:"clientX",axis:"pageX",size:"width",outside:"bottom",inside:"top",leading:"left",trailing:"right",marginLeading:"marginLeft",marginTrailing:"marginRight",thickness:"height",delta:"deltaX"}},u={top:"vertical",bottom:"vertical",height:"vertical",left:"horizontal",right:"horizontal",width:"horizontal"};function g(t){throw"finbars: "+t}l="div.finbar-horizontal,div.finbar-vertical{margin:3px}div.finbar-horizontal>.thumb,div.finbar-vertical>.thumb{background-color:#d3d3d3;-webkit-box-shadow:0 0 1px #000;-moz-box-shadow:0 0 1px #000;box-shadow:0 0 1px #000;border-radius:4px;margin:2px;opacity:.4;transition:opacity .5s}div.finbar-horizontal>.thumb.hover,div.finbar-vertical>.thumb.hover{opacity:1;transition:opacity .5s}div.finbar-vertical{top:0;bottom:0;right:0;width:11px}div.finbar-vertical>.thumb{top:0;right:0;width:7px}div.finbar-horizontal{left:0;right:0;bottom:0;height:11px}div.finbar-horizontal>.thumb{left:0;bottom:0;height:7px}",t.exports=a},function(t,e,n){"use strict";var r=n(28),i=n(50),o=n(69),a=n(17),s=n(18),A=["BackgroundColor","Color","Font"],l=A.concat(["Halign"]),c=[{prefix:"",props:l.concat(["cellPadding","iconPadding"])},{prefix:"foregroundSelection",props:A},{prefix:"columnHeader",props:l},{prefix:"columnHeaderForegroundSelection",props:A},{prefix:"rowHeader",props:A},{prefix:"rowHeaderForegroundSelection",props:A}],u={rowHeaderCheckboxes:i.rowHeaderCheckboxes,rowHeaderNumbers:i.rowHeaderNumbers,gridBorder:i.gridBorder,gridBorderTop:i.gridBorderTop,gridBorderRight:i.gridBorderRight,gridBorderBottom:i.gridBorderBottom,gridBorderLeft:i.gridBorderLeft,gridRenderer:i.gridRenderer},g=Object.assign({},u,{themeName:i.themeName});c.reduce(function(t,e){return e.props.reduce(function(t,n){return t[n=(n=(n=e.prefix+n).replace("ForegroundSelectionBackground","BackgroundSelection"))[0].toLowerCase()+n.substr(1)]=i[n],t},t)},g);var h=Object.create(null,{default:{value:g}});function d(t){var e,n,r,s;if(t&&"object"==typeof t&&!Object.getOwnPropertyNames(t).length&&(t=null),this._theme?(n=[this],e=this._theme,r=this.properties,t||(Object.keys(e).forEach(function(t){t in o||delete r[t]}),Object.keys(u).forEach(function(t){r.var[t]=i[t]})),Object.keys(e).forEach(function(t){delete e[t]})):(n=this.grids,e=i,t=t||"default"),"object"==typeof t)s=t;else{if(!h[t])throw new a('Unknown theme "'+t+'"');s=h[t]}return s&&(s.themeName||(s.themeName=void 0),Object.keys(s).forEach(function(t){if(t in o)if(t in u)n.forEach(function(e){e.properties[t]=s[t]});else switch(t){case"lineColor":s.gridLinesHColor=s.gridLinesVColor=s[t];break;default:console.warn("Ignoring unexpected dynamic property "+t+" from theme object.")}}),Object.defineProperties(e,Object.getOwnPropertyDescriptors(s))),n.forEach(function(t){t.repaint()}),s}var f={initThemeLayer:function(){return this._theme=Object.create(i),Object.create(this._theme,o)},applyTheme:d,getTheme:function(){var t=this._theme,e=t.themeName;return"default"!==e&&Object.getOwnPropertyNames(t).length?e in h?e:t:void 0}};Object.defineProperty(f,"theme",{enumerable:!0,set:d,get:f.getTheme});var p={registerTheme:function(t,e){if("object"==typeof t&&(t=(e=t).themeName),!t)throw new a("Cannot register an anonymous theme.");if("default"===t)throw new a('Cannot register or unregister the "default" theme.');e?(e.themeName=t,h[t]=e):delete h[t]},registerThemes:function(t){t?r(t).each(function(t,e){this.registerTheme(e,t)},this):Object.keys(h).forEach(function(t){this.registerTheme(t)},this)},applyTheme:function(t){var e=d.call(this,t);s.setTheme(e)}};Object.defineProperty(p,"theme",{enumerable:!0,set:p.applyTheme,get:function(){return i}}),t.exports={mixin:f,sharedMixin:p}},function(t,e,n){var r,i,o,a,s,A,l,c,u,g,h,d,f,p,I,C,m;window,t.exports=(r=n(82),i=n(57),o=n(106),a=n(60),s=n(63),A=n(53),l=n(112),c=n(44),u=n(142),g=n(144),h=n(3),d=n(186),f=n(187),p=n(68),I=n(2),C=n(189),m=n(52),function(t){var e={};function n(r){if(e[r])return e[r].exports;var i=e[r]={i:r,l:!1,exports:{}};return t[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=t,n.c=e,n.d=function(t,e,r){n.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:r})},n.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},n.t=function(t,e){if(1&e&&(t=n(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var i in t)n.d(r,i,function(e){return t[e]}.bind(null,i));return r},n.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(e,"a",e),e},n.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},n.p="",n(n.s=10)}([function(t,e){t.exports=r},function(t,e){t.exports=i},function(t,e){t.exports=o},function(t,e){t.exports=a},function(t,e){t.exports=s},function(t,e){t.exports=A},function(t,e){t.exports=l},function(t,e){t.exports=c},function(t,e){t.exports=u},function(t,e,n){(t.exports=n(25)(!1)).push([t.i,':host{position:relative;color:#666;width:100%;height:100%}:host table{font:12px "Open Sans",helvetica,sans-serif}:host td.hover{background-color:#eee}:host tr.hover{background-color:#f6f6f6}:host td.positive{color:#1078d1}:host td.negative{color:#de3838}:host #gridWrapper{box-sizing:border-box;width:100%;height:100%;margin:0;padding:var(--hypergrid--padding,0)}:host #mainGrid{position:relative;width:100%;height:100%;margin:0;padding:0;overflow:hidden}:host #mainGrid>div{position:absolute;width:100%;height:100%}:host #mainGrid input{margin:0}:host div.finbar-vertical{margin:0!important;width:15px!important;box-shadow:0 0 0 #000,0 0 0 #000,0 0 0 #000}:host div.finbar-vertical div.thumb{width:var(--hypergrid-scrollbar--width,13px);right:-1px;background-color:#9e9e9e;box-shadow:0 0 0 #000,0 0 0 #000,0 0 0 #000;margin:0!important;border-radius:var(--hypergrid-scrollbar--border-radius,0)}:host div.finbar-horizontal{margin:0!important;height:15px!important;box-shadow:0 0 0 #000,0 0 0 #000,0 0 0 #000}:host div.finbar-horizontal div.thumb{margin:-3px 2px;height:var(--hypergrid-scrollbar--width,13px);bottom:4px;background-color:#9e9e9e;box-shadow:0 0 0 #000,0 0 0 #000,0 0 0 #000;border-radius:var(--hypergrid-scrollbar--border-radius,0)}',""])},function(t,e,n){"use strict";n.r(e),function(t){n(1),n(2),n(3),n(4);var e=n(8),r=n(9),i=n.n(r),o=n(0);function a(t,e){return function(t){if(Array.isArray(t))return t}(t)||function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!e||n.length!==e);r=!0);}catch(t){i=!0,o=t}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(t,e)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}function s(){return Reflect.construct(HTMLElement,[],this.__proto__.constructor)}Object.setPrototypeOf(s.prototype,HTMLElement.prototype),Object.setPrototypeOf(s,HTMLElement);const A=n(12),l=n(13),c=n(14),u=n(15),g=n(20),h=n(22).treeLineRendererPaint,d=n(23),f=d.psp2hypergrid,p=d.page2hypergrid,I=n(6).cloneDeep,C=n(24),m={autoSelectRows:!1,cellPadding:5,cellSelection:!1,columnSelection:!1,rowSelection:!1,checkboxOnlyRowSelections:!1,columnClip:!0,columnHeaderFont:"12px Helvetica, sans-serif",columnHeaderForegroundSelectionFont:'12px "Arial", Helvetica, sans-serif',columnsReorderable:!1,defaultRowHeight:24,editable:!1,editOnKeydown:!0,editor:"textfield",editorActivationKeys:["alt","esc"],enableContinuousRepaint:!1,fixedColumnCount:0,fixedRowCount:0,fixedLinesHWidth:1,fixedLinesVWidth:1,font:'12px "Arial", Helvetica, sans-serif',foregroundSelectionFont:'12px "Arial", Helvetica, sans-serif',gridLinesH:!1,gridLinesV:!0,gridLinesUserDataArea:!1,halign:"left",headerTextWrapping:!1,hoverColumnHighlight:{enabled:!1},hoverRowHighlight:{enabled:!0,backgroundColor:"#555"},hoverCellHighlight:{enabled:!0,backgroundColor:"#333"},noDataMessage:"",minimumColumnWidth:50,multipleSelections:!1,renderFalsy:!1,rowHeaderFont:"12px Arial, Helvetica, sans-serif",rowHeaderForegroundSelectionFont:'12px "Arial", Helvetica, sans-serif',rowResize:!0,scrollbarHoverOff:"visible",rowHeaderCheckboxes:!1,rowHeaderNumbers:!1,showFilterRow:!0,showHeaderRow:!0,showTreeColumn:!1,singleRowSelectionMode:!1,sortColumns:[],sortOnDoubleClick:!0,treeRenderer:"TreeCell",treeHeaderFont:"12px Arial, Helvetica, sans-serif",treeHeaderForegroundSelectionFont:'12px "Arial", Helvetica, sans-serif',useBitBlit:!1,vScrollbarClassPrefix:"",voffset:0},y={backgroundColor:"#ffffff",color:"#666",lineColor:"#AAA",font:'12px "Open Sans", Helvetica, sans-serif',foregroundSelectionFont:"12px amplitude-regular, Helvetica, sans-serif",foregroundSelectionColor:"#666",backgroundSelectionColor:"rgba(162, 183, 206, 0.3)",selectionRegionOutlineColor:"rgb(45, 64, 85)",columnHeaderColor:"#666",columnHeaderHalign:"left",columnHeaderBackgroundColor:"#fff",columnHeaderForegroundSelectionColor:"#333",columnHeaderBackgroundSelectionColor:"#40536d",rowHeaderForegroundSelectionFont:"12px Arial, Helvetica, sans-serif",treeHeaderColor:"#666",treeHeaderBackgroundColor:"#fff",treeHeaderForegroundSelectionColor:"#333",treeHeaderBackgroundSelectionColor:"#40536d",hoverCellHighlight:{enabled:!0,backgroundColor:"#eeeeee"},hoverRowHighlight:{enabled:!0,backgroundColor:"#f6f6f6"}};function E(t){let e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"",n=t.format.bind(t);return t.format=t=>{if("string"==typeof t)return t;if(e===t)return"-";let r=n(t);return""===r?"-":r},t}Object(e.bindTemplate)(C,i.a)(class extends s{set_data(t,e,n,r,i){const o=f(t,e,n,r,i);this.grid?this.grid.behavior.setPSP(o):this._hg_data=o}get_style(t){return window.ShadyCSS?window.ShadyCSS.getComputedStyleValue(this,t):getComputedStyle(this).getPropertyValue(t)}apply_styles(){const t="".concat(this.get_style("--hypergrid--font-size")," ").concat(this.get_style("--hypergrid--font-family")),e="".concat(this.get_style("--hypergrid-header--font-size")," ").concat(this.get_style("--hypergrid-header--font-family")),n={enabled:!0,backgroundColor:this.get_style("--hypergrid-row-hover--background"),color:this.get_style("--hypergrid-row-hover--color")},r={enabled:!0,backgroundColor:this.get_style("--hypergrid-cell-hover--background"),color:this.get_style("--hypergrid-cell-hover--color")},i={treeHeaderBackgroundColor:this.get_style("--hypergrid-tree-header--background"),backgroundColor:this.get_style("--hypergrid-tree-header--background"),treeHeaderColor:this.get_style("--hypergrid-tree-header--color"),color:this.get_style("--hypergrid-tree-header--color"),columnHeaderBackgroundColor:this.get_style("--hypergrid-header--background"),columnHeaderSeparatorColor:this.get_style("--hypergrid-separator--color"),columnHeaderColor:this.get_style("--hypergrid-header--color"),columnColorNumberPositive:this.get_style("--hypergrid-positive--color"),columnColorNumberNegative:this.get_style("--hypergrid-negative--color"),columnBackgroundColorNumberPositive:this.get_style("--hypergrid-positive--background"),columnBackgroundColorNumberNegative:this.get_style("--hypergrid-negative--background"),columnHeaderFont:e,font:t,rowHeaderFont:t,treeHeaderFont:t,hoverRowHighlight:n,hoverCellHighlight:r};this.grid&&this.grid.addProperties(i)}connectedCallback(){if(!this.grid){const e=this.shadowRoot.querySelector("#mainGrid");e.setAttribute("hidden",!0),this.grid=new A(e,{DataModel:g}),this.grid.canvas.stopResizeLoop(),e.removeAttribute("hidden"),this.grid.installPlugins([u,[c,{paintBackground:null,columnHeaderLines:!1,groupConfig:[{halign:"center",font:"12px Open Sans, sans-serif"}]}]]);let n=this.grid.cellRenderers.items.GroupedHeader.paint;this.grid.cellRenderers.items.GroupedHeader.paint=function(t,e){return this.visibleColumns=e.grid.renderer.visibleColumns,n.call(this,t,e)};const r=(t=y,Object.assign({},I(m),I(t)));r.showRowNumbers=r.showCheckboxes||r.showRowNumbers,this.grid.addProperties(r),this.apply_styles(),this.grid.localization.header={format:t=>this.grid.behavior.formatColumnHeader(t)},this.grid.cellRenderers.add("TreeCell",l.extend({paint:h}));const i=new this.grid.localization.NumberFormatter("en-US",Object(o.get_type_config)("float").format),a=new this.grid.localization.NumberFormatter("en-us",Object(o.get_type_config)("integer").format);this.grid.localization.add("FinanceFloat",E(i)),this.grid.localization.add("FinanceInteger",E(a));const s=new this.grid.localization.DateFormatter("en-us",Object(o.get_type_config)("datetime").format),d=new this.grid.localization.DateFormatter("en-us",Object(o.get_type_config)("date").format);this.grid.localization.add("FinanceDatetime",E(s,-1)),this.grid.localization.add("FinanceDate",E(d,-1)),this.grid.localization.add("FinanceTree",{format:function(t,e){const n={date:d,datetime:s,integer:a,float:i}[e];return n?n.format(t):t},parse:t=>t}),this._hg_data&&(this.grid.behavior.setPSP(this._hg_data),delete this._hgdata)}var t}});const v=Symbol("Hypergrid private");function B(t){var e;return regeneratorRuntime.async(function(n){for(;;)switch(n.prev=n.next){case 0:if(this.hypergrid?e=this[v].grid:(e=this[v].grid=document.createElement("perspective-hypergrid"),Object.defineProperty(this,"hypergrid",{configurable:!0,get:()=>this[v].grid?this[v].grid.grid:void 0})),e.isConnected){n.next=7;break}return t.innerHTML="",t.appendChild(e),n.next=6,regeneratorRuntime.awrap(new Promise(t=>setTimeout(t)));case 6:e.grid.canvas.resize(!1,!0);case 7:return n.abrupt("return",e);case 8:case"end":return n.stop()}},null,this)}t.registerPlugin("hypergrid",{name:"Grid",create:function(t,e,n){var r,i,o,s,A,l,c,u,g,h,d,f,I,C;return regeneratorRuntime.async(function(m){for(;;)switch(m.prev=m.next){case 0:return this[v]=this[v]||{},(r=this.hypergrid)&&(r.behavior.dataModel._view=void 0),m.next=5,regeneratorRuntime.awrap(e.get_config());case 5:if(i=m.sent,!n.cancelled){m.next=8;break}return m.abrupt("return");case 8:return o=i.column_pivots,s=i.row_pivots,A={start_row:0,end_row:Math.max(o.length+1,s.length+1)},m.next=13,regeneratorRuntime.awrap(Promise.all([e.num_rows(),e.to_columns(A),e.schema(),this._table.schema()]));case 13:if(l=m.sent,c=a(l,4),u=c[0],g=c[1],h=c[2],d=c[3],!n.cancelled){m.next=21;break}return m.abrupt("return");case 21:return m.next=23,regeneratorRuntime.awrap(B.call(this,t));case 23:if(f=m.sent,!n.cancelled){m.next=26;break}return m.abrupt("return");case 26:return I=this.hypergrid.behavior.dataModel,C=Object.keys(g),I.setIsTree(s.length>0),I.setDirty(u),I._view=e,I._config=i,I._viewer=this,I.pspFetch=t=>{var e,n,r,i;return regeneratorRuntime.async(function(o){for(;;)switch(o.prev=o.next){case 0:return t.end_row+=this.hasAttribute("settings")?8:2,t.end_col+=s&&s.length>0?1:0,o.next=4,regeneratorRuntime.awrap(I._view.to_columns(t));case 4:e=o.sent,I.data=[],n=p(e,s,C),r=I.data,i=t.start_row,n.forEach((t,e)=>r[i+e]=t);case 10:case"end":return o.stop()}},null,this)},f.set_data(g,h,d,s,C),this.hypergrid.renderer.computeCellsBounds(!0),m.next=38,regeneratorRuntime.awrap(this.hypergrid.canvas.resize(!0));case 38:this.hypergrid.canvas.paintNow(),this.hypergrid.canvas.paintNow();case 40:case"end":return m.stop()}},null,this)},selectMode:"toggle",update:function(t,e,n){var r,i;return regeneratorRuntime.async(function(t){for(;;)switch(t.prev=t.next){case 0:return t.next=2,regeneratorRuntime.awrap(e.num_rows());case 2:if(r=t.sent,!n.cancelled){t.next=5;break}return t.abrupt("return");case 5:(i=this.hypergrid.behavior.dataModel).setDirty(r),i._view=e,this.hypergrid.canvas.paintNow();case 9:case"end":return t.stop()}},null,this)},deselectMode:"pivots",styleElement:function(){const t=this[v].grid;t.apply_styles(),t.grid.canvas.paintNow()},resize:function(){var t;return regeneratorRuntime.async(function(e){for(;;)switch(e.prev=e.next){case 0:if(!this.hypergrid){e.next=8;break}return this.hypergrid.canvas.checksize(),this.hypergrid.canvas.paintNow(),e.next=5,regeneratorRuntime.awrap(this._view.num_rows());case 5:t=e.sent,this.hypergrid.behavior.dataModel.setDirty(t),this.hypergrid.canvas.paintNow();case 8:case"end":return e.stop()}},null,this)},delete:function(){this.hypergrid&&(this.hypergrid.terminate(),this.hypergrid.div=void 0,this.hypergrid.canvas.div=void 0,this.hypergrid.canvas.canvas=void 0,this.hypergrid.sbVScroller=void 0,this.hypergrid.sbHScroller=void 0,delete this[v].grid)}})}.call(this,n(11))},function(t,e){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(t){"object"==typeof window&&(n=window)}t.exports=n},function(t,e){t.exports=g},function(t,e){t.exports=h},function(t,e){t.exports=d},function(t,e,n){function r(t,e){return function(t){if(Array.isArray(t))return t}(t)||function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!e||n.length!==e);r=!0);}catch(t){i=!0,o=t}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(t,e)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}n(3),n(4),n(2),n(1),n(5),n(16),n(17);const i=n(18),o=n(19),a=n(6);function s(t){const e=[];t.isTree&&(e[this.treeColumnIndex]={name:this.treeColumnIndex.toString(),header:" "}),t.columnPaths.forEach(function(n,r){const i=n.join("|"),o=t.configuration.columnAliases,a=o&&o[i]||i,s=r.toString(),A=t.columnTypes[r];t.isTree&&0===r?e[-1]={name:s,header:a,type:A}:e.push({name:s,header:a,type:A,index:r-(t.isTree?1:0)})}),this.grid.properties.showTreeColumn=t.isTree,this.createColumns=A,this._memoized_schema&&a.isEqual(this._memoized_schema.slice(0,this._memoized_schema.length),e.slice(0,e.length))&&a.isEqual(t.rowPivots,this._memoized_pivots)?(this.grid.sbVScroller.index=0,this.grid.behavior.dataModel.data=t.rows):(this.grid.sbVScroller.index=0,this.grid.sbHScroller.index=0,this.grid.setData({data:t.rows,schema:e})),this._memoized_schema=e,this._memoized_pivots=t.rowPivots}function A(){Object.getPrototypeOf(this).createColumns.call(this),this.getActiveColumns().forEach(function(t){l.call(this,t)},this);let t=this.getTreeColumn();t&&l.call(this,t),this.stashedWidths=void 0,this.setHeaders(),this.schema_loaded=!0}function l(t){var e=t.properties;switch(t.type){case"number":case"float":e.halign="right",e.columnHeaderHalign="right",e.format="FinanceFloat";break;case"integer":e.halign="right",e.columnHeaderHalign="right",e.format="FinanceInteger";break;case"date":e.format="FinanceDate";break;case"datetime":e.format="FinanceDatetime";break;default:t.index===this.treeColumnIndex&&(e.format="FinanceTree")}}function c(t){const e=this.dataModel.getConfig(),n=e.sort.findIndex(e=>e[0]===t.trim());if(n>-1){const r=e.sort[n][1];r in this.charMap&&(t="".concat(t," ").concat(this.charMap[r]).concat(e.sort.length>1?o(n+1):""))}return t}const u=t=>{const e=t.detail.primitiveEvent,n=new MouseEvent(e.type,e);t.target.parentElement.parentElement.parentElement.dispatchEvent(n)};e.install=function(t){var e;e=t.behavior.charMap,Object.assign(e,{asc:"↑",desc:"↓","asc abs":"⇧","desc abs":"⇩","col asc":"→","col desc":"←","col asc abs":"⇨","col desc abs":"⇦"}),Object.getPrototypeOf(t.behavior).setPSP=s,Object.getPrototypeOf(t.behavior).formatColumnHeader=c,t.addEventListener("fin-column-sort",function(t){t.preventDefault(),t.handled=!0;const e=this.behavior.dataModel.getConfig(),n=this.behavior.getColumn(t.detail.column);let r,i;e.column_pivots.length>0?(r=!0,i=n.header.split("|")[e.column_pivots.length]):(r=!1,i=n.header);const o=this.behavior.dataModel._viewer,a=e.sort.findIndex(t=>t[0]===i.trim()),s=a>-1,A=t.detail.keys&&(t.detail.keys.indexOf("ALTSHIFT")>-1||t.detail.keys.indexOf("ALT")>-1)&&"string"!==n.type,l=t.detail.keys&&(t.detail.keys.indexOf("ALTSHIFT")>-1||t.detail.keys.indexOf("SHIFT")>-1);let c;if(s){const t=e.sort[a],n=t[1];c=o._increment_sort(n,r,A),t[1]=c}else c=o._increment_sort("none",r,A);l&&s?("none"===c&&e.sort.splice(a,1),o.sort=JSON.stringify(e.sort)):l?(e.sort.push([i,c]),o.sort=JSON.stringify(e.sort)):o.sort=JSON.stringify([[i,c]])}.bind(t)),t.addEventListener("fin-canvas-context-menu",u),Object.getPrototypeOf(t.behavior).cellClicked=function(t){var e,n,i,o,a,s,A,l,c,u,g,h,d,f,p,I,C,m;return regeneratorRuntime.async(function(y){for(;;)switch(y.prev=y.next){case 0:return t.primitiveEvent.preventDefault(),t.handled=!0,e=t.dataCell,n=e.x,i=e.y,o=this.dataModel.getConfig(),a=o.row_pivots,s=o.column_pivots,l=(A=i>=0?i:0)+1,y.next=10,regeneratorRuntime.awrap(this.dataModel._view.to_json({start_row:A,end_row:l}));case 10:return c=y.sent,u=c.map(t=>t.__ROW_PATH__),g=u[0]||[],h=a.map((t,e)=>{const n=g[e];return n?[t,"==",n]:void 0}).filter(t=>t),d=a.length>0?n+1:n,f=Object.keys(c[0])[d],p=[],f&&(C=f.split("|"),I=[C[C.length-1]],p=s.map((t,e)=>{const n=C[e];return n?[t,"==",n]:void 0}).filter(t=>t).filter(t=>"__ROW_PATH__"!==r(t,3)[2])),m=o.filter.concat(h).concat(p),this.grid.canvas.dispatchEvent(new CustomEvent("perspective-click",{bubbles:!0,composed:!0,detail:{config:{filters:m},column_names:I,row:c[0]}})),y.abrupt("return",this.dataModel.toggleRow(t.dataCell.y,t.dataCell.x,t));case 21:case"end":return y.stop()}},null,this)},t.canvas.resize=function(e,n){var r,o,a,s,A,l,c;return regeneratorRuntime.async(function(u){for(;;)switch(u.prev=u.next){case 0:if(r=this.width=Math.floor(this.div.clientWidth),o=this.height=Math.floor(this.div.clientHeight),a=1,(s=window.devicePixelRatio&&this.component.properties.useHiDPI)&&(A=window.devicePixelRatio||1,l=this.gc.webkitBackingStorePixelRatio||this.gc.mozBackingStorePixelRatio||this.gc.msBackingStorePixelRatio||this.gc.oBackingStorePixelRatio||this.gc.backingStorePixelRatio||1,a=A/l),this.bounds=new i.Rectangle(0,0,r,o),this.component.setBounds(this.bounds),this.resizeNotification(),c=!1,n||!(o*a>this.canvas.height||e)){u.next=13;break}return u.next=12,regeneratorRuntime.awrap(new Promise(t=>this.component.grid.behavior.dataModel.fetchData(void 0,t)));case 12:c=u.sent;case 13:c&&!n||(this.bounds=new i.Rectangle(0,0,r,o),this.component.setBounds(this.bounds),this.buffer.width=this.canvas.width=r*a,this.buffer.height=this.canvas.height=o*a,this.canvas.style.width=this.buffer.style.width=r+"px",this.canvas.style.height=this.buffer.style.height=o+"px",this.bc.scale(a,a),s&&!this.component.properties.useBitBlit&&this.gc.scale(a,a),t.canvas.paintNow());case 14:case"end":return u.stop()}},null,this)}}},function(t,e){t.exports=f},function(t,e){t.exports=p},function(t,e){t.exports=I},function(t,e){t.exports=C},function(t,e,n){n(1),n(2);const r=n(7).prototype.treeColumnIndex;function i(t){if(!this.dataWindow)return[];var e=this.dataWindow;return[this.grid.newRectangle(e.left,e.top,e.width,t?Math.min(t-e.top,e.height):e.height)]}function o(t){let e=this.data[t.origin.y],n=this.data[Math.min(t.corner.y,this.getRowCount()-1)];return!(e&&void 0!==e[t.origin.x]&&n&&void 0!==n[t.corner.x-1])}t.exports=n(21).extend("PerspectiveDataModel",{isTreeCol:function(t){return t===r&&this.isTree()},getValue:function(t,e){var n=this.data[e];return n?n[t]:null},getRowCount:function(){return this._nrows||0},getConfig:function(){return this._config},setRowCount:function(t){this._nrows=t||0},isTree:function(){return this._isTree},setIsTree:function(t){this._isTree=t},isCached:function(t){return!t||!t.find(o,this)},setDirty:function(t){t!==this._nrows&&this.grid.renderer.computeCellsBounds(),this._dirty=!0,this._nrows=t,this.grid.behaviorChanged()},toggleRow:function(t,e,n){var r,i,o;return regeneratorRuntime.async(function(a){for(;;)switch(a.prev=a.next){case 0:if(!this.isTreeCol(e)){a.next=12;break}return r=!1,n.primitiveEvent.detail.primitiveEvent&&(r=!!n.primitiveEvent.detail.primitiveEvent.shiftKey),a.next=5,regeneratorRuntime.awrap(this._view.get_row_expanded(t));case 5:return i=a.sent,r?i?1===this.data[t][e].rowPath.length?this._view.collapse(t):this._view.set_depth(this.data[t][e].rowPath.length-2):this._view.set_depth(this.data[t][e].rowPath.length-1):i?this._view.collapse(t):this._view.expand(t),a.next=9,regeneratorRuntime.awrap(this._view.num_rows());case 9:o=a.sent,this.setDirty(o),this.grid.canvas.paintNow();case 12:case"end":return a.stop()}},null,this)},fetchData:function(t,e){var n,r;return regeneratorRuntime.async(function(a){for(;;)switch(a.prev=a.next){case 0:if(t=i.call(this.grid.renderer),void 0!==this._view){a.next=4;break}return e(!0),a.abrupt("return");case 4:if(this._dirty||t.find(o,this)){a.next=7;break}return e(!1),a.abrupt("return");case 7:if(!this._outstanding_requested_rects||!t[0].within(this._outstanding_requested_rects[0])){a.next=10;break}return e(!0),a.abrupt("return");case 10:return this._dirty=!1,this._outstanding_requested_rects=t,n=t.map(t=>this.pspFetch({start_row:t.origin.y,end_row:t.corner.y,start_col:t.origin.x,end_col:t.corner.x+1})),a.prev=13,a.next=16,regeneratorRuntime.awrap(Promise.all(n));case 16:r=i.call(this.grid.renderer),e(!!r.find(o,this)),a.next=23;break;case 20:a.prev=20,a.t0=a.catch(13),e(!0);case 23:return a.prev=23,this._outstanding_requested_rects=void 0,a.finish(23);case 26:case"end":return a.stop()}},null,this,[[13,20,23,26]])},getCell:function(t,e){var n,i;return t.isUserDataArea?function(t){if(null===t.value||void 0===t.value)t.value="-";else{const e=this.schema[t.dataCell.x].type;["number","float","integer"].indexOf(e)>-1?0===t.value?t.value="float"===e?"0.00":"0":isNaN(t.value)?t.value="-":t.value>0?(t.color=t.columnColorNumberPositive||"rgb(160,207,255)",t.backgroundColor=t.columnBackgroundColorNumberPositive?t.columnBackgroundColorNumberPositive:t.backgroundColor):(t.color=t.columnColorNumberNegative||"rgb(255,136,136)",t.backgroundColor=t.columnBackgroundColorNumberNegative?t.columnBackgroundColorNumberNegative:t.backgroundColor):"boolean"===e&&(t.value=String(t.value))}}.call(this,t,e):t.dataCell.x===r&&t.value&&(i=(n=this.getRow(t.dataCell.y+1))?t.value.rowPath.length-n[r].rowPath.length:1,t.last=0!==i,t.expanded=i<0,t._type=this.schema[-1].type[t.value.rowPath.length-2]),t.grid.cellRenderers.get(e)},pspFetch:function(){return regeneratorRuntime.async(function(t){for(;;)switch(t.prev=t.next){case 0:case"end":return t.stop()}})}})},function(t,e){t.exports=m},function(t,e,n){"use strict";function r(t,e){var n=e.bounds.x,r=e.bounds.y,i=e.bounds.width,o=e.bounds.height;if(e.minWidth=1,null===e.value)return;var a=e.value.rollup,s=e.value.isLeaf,A=e.value.rowPath.length-1,l=e.expanded,c=e.last,u=e.backgroundColor;e.isSelected&&(u=e.backgroundSelectionColor),e.isRowHovered&&e.hoverRowHighlight.enabled&&!e.isCellHovered?u=e.hoverRowHighlight.backgroundColor:e.isCellHovered&&e.hoverCellHighlight.enabled&&(u=e.hoverCellHighlight.backgroundColor),t.save(),t.cache.fillStyle=u,t.rect(n,r,i,o),t.fillRect(n,r,i,o);var g=e.isSelected?e.foregroundSelectionColor:e.color;t.cache.strokeStyle=g,t.cache.fillStyle=g;var h=n;t.globalAlpha=.3,t.strokeStyle=g,t.beginPath();for(var d=1;d<=A;d++){h+=16;var f=c&&!l?o/2:o;t.moveTo(h,r),t.lineTo(h,r),t.lineTo(h,r+f),d===A?(t.moveTo(h,r+o/2),t.lineTo(h,r+o/2),t.lineTo(h+16-3,r+o/2)):t.lineTo(h,r+o)}s?t.lineTo(h+16+3,r+o/2):(t.moveTo(h+16+3,r+o/2),t.arc(h+16,r+o/2,3,0,2*Math.PI),e.isCellHovered&&(t.globalAlpha=.45,t.fill(),t.globalAlpha=.3)),l&&!s&&(t.globalAlpha=.8,t.fill(),t.moveTo(h+16,r+o/2+3),t.lineTo(h+16,r+o),t.globalAlpha=.3),t.stroke(),t.closePath(),t.globalAlpha=1,t.fillStyle=e.isSelected?e.foregroundSelectionColor:e.color,t.textAlign="start",t.textBaseline="middle",t.font=e.isSelected?e.foregroundSelectionFont:e.treeHeaderFont;var p=h+16+6+3;let I=e.formatValue(a,e._type);e.minWidth=p+t.getTextWidth(I)+15;var C=t.getTextWidthTruncated(I,i-p+(n-3),!0),m=r+o/2;t.fillText(C.string?C.string:I,p,m),t.restore()}n.r(e),n.d(e,"treeLineRendererPaint",function(){return r})},function(t,e,n){n(5),n(1);const r=n(7).prototype.treeColumnIndex;function i(t,e,n){const i=Object.keys(t),o=i.length>0?i[0]:void 0;if(0===n.length||0===t[o].length)return[];const a=!!e.length,s=e.length?n.slice(1):n,A=i.map(t=>s.indexOf(t)),l=[];for(let n=0;n<t[o].length;n++){const o={};for(const e in i){const r=i[e];o[A[e]]=t[r][n]}if(a){void 0===t.__ROW_PATH__[n]&&(t.__ROW_PATH__[n]=[]);let i=t.__ROW_PATH__[n][t.__ROW_PATH__[n].length-1];void 0===i&&0===n&&(i="TOTAL"),o[r]={rollup:i,rowPath:["ROOT"].concat(t.__ROW_PATH__[n]),isLeaf:t.__ROW_PATH__[n].length>=e.length}}l.push(o)}return l}t.exports={psp2hypergrid:function(t,e,n,r,o){const a=Object.keys(t).length>0?Object.keys(t)[0]:void 0;if(0===o.length||0===t[a].length){const t=Object.keys(e);return{rows:[],isTree:!1,configuration:{},columnPaths:t.map(t=>[t]),columnTypes:t.map(t=>e[t])}}const s=(r.length?o.slice(1):o).map(t=>t.split("|")),A=!!r.length;return{rows:i(t,r,o),isTree:A,configuration:{},rowPivots:r,columnPaths:(A?[[" "]]:[]).concat(s),columnTypes:(A?[r.map(t=>n[t])]:[]).concat(s.map(t=>e[t[t.length-1]]))}},page2hypergrid:i}},function(t,e){t.exports="\x3c!--\n   \n   Copyright (c) 2017, the Perspective Authors.\n   \n   This file is part of the Perspective library, distributed under the terms of\n   the Apache License 2.0.  The full license can be found in the LICENSE file.\n\n--\x3e\n\n<template id='perspective-hypergrid'>\n    <div id='gridWrapper'>\n        <div id='mainGrid'></div>\n    </div>\n</template>"},function(t,e){t.exports=function(t){var e=[];return e.toString=function(){return this.map(function(e){var n=function(t,e){var n,r=t[1]||"",i=t[3];if(!i)return r;if(e&&"function"==typeof btoa){var o=(n=i,"/*# sourceMappingURL=data:application/json;charset=utf-8;base64,"+btoa(unescape(encodeURIComponent(JSON.stringify(n))))+" */"),a=i.sources.map(function(t){return"/*# sourceURL="+i.sourceRoot+t+" */"});return[r].concat(a).concat([o]).join("\n")}return[r].join("\n")}(e,t);return e[2]?"@media "+e[2]+"{"+n+"}":n}).join("")},e.i=function(t,n){"string"==typeof t&&(t=[[null,t,""]]);for(var r={},i=0;i<this.length;i++){var o=this[i][0];"number"==typeof o&&(r[o]=!0)}for(i=0;i<t.length;i++){var a=t[i];"number"==typeof a[0]&&r[a[0]]||(n&&!a[2]?a[2]=n:n&&(a[2]="("+a[2]+") and ("+n+")"),e.push(a))}},e}}]))},function(t,e,r){(function(t,e){r(53);const i=r(94).default,o=["perspective.config.js","perspective.config.json","package.json"];function a(t){return t&&"object"==typeof t&&!Array.isArray(t)}function s(t,...e){if(!e.length)return t;const n=e.shift();if(a(t)&&a(n))for(const e in n)a(n[e])?(t[e]||Object.assign(t,{[e]:{}}),s(t[e],n[e])):Object.assign(t,{[e]:n[e]});return s(t,...e)}let A;t.exports.get_type_config=function(e){return t.exports.get_config().types[e]||{}},t.exports.override_config=function(t){A&&console.warn("Config already initialized!"),A=s(i,t)},t.exports.get_config=function(){return A||(A=s(i,"undefined"==typeof window?function(){t.require;const r=n(125),i=n(125),[a,...s]=e.cwd().split(r.sep);for(;s.length>0;){for(const t of o){const e="".concat(a).concat(r.sep).concat(r.join(...s,t));if(i.existsSync(e)){if(!t.endsWith("json")){const t=n(278)(e);return t.default||t}{const n=JSON.parse(i.readFileSync(e));if("package.json"!==t)return n;if(n.perspective)return n.perspective}}}s.pop()}}():{types:{float:{filter_operator:"==",aggregate:"sum",format:{style:"decimal",minimumFractionDigits:2,maximumFractionDigits:2}},string:{filter_operator:"==",aggregate:"count"},integer:{filter_operator:"==",aggregate:"sum",format:{}},boolean:{filter_operator:"==",aggregate:"count"},datetime:{filter_operator:"==",aggregate:"count",format:{week:"numeric",year:"numeric",month:"numeric",day:"numeric",hour:"numeric",minute:"numeric",second:"numeric"}},date:{filter_operator:"==",aggregate:"count",format:{week:"numeric",year:"numeric",month:"numeric",day:"numeric"}}}}||{})),A}}).call(this,r(32)(t),r(83))},function(t,e){var n,r,i=t.exports={};function o(){throw new Error("setTimeout has not been defined")}function a(){throw new Error("clearTimeout has not been defined")}function s(t){if(n===setTimeout)return setTimeout(t,0);if((n===o||!n)&&setTimeout)return n=setTimeout,setTimeout(t,0);try{return n(t,0)}catch(e){try{return n.call(null,t,0)}catch(e){return n.call(this,t,0)}}}!function(){try{n="function"==typeof setTimeout?setTimeout:o}catch(t){n=o}try{r="function"==typeof clearTimeout?clearTimeout:a}catch(t){r=a}}();var A,l=[],c=!1,u=-1;function g(){c&&A&&(c=!1,A.length?l=A.concat(l):u=-1,l.length&&h())}function h(){if(!c){var t=s(g);c=!0;for(var e=l.length;e;){for(A=l,l=[];++u<e;)A&&A[u].run();u=-1,e=l.length}A=null,c=!1,function(t){if(r===clearTimeout)return clearTimeout(t);if((r===a||!r)&&clearTimeout)return r=clearTimeout,clearTimeout(t);try{r(t)}catch(e){try{return r.call(null,t)}catch(e){return r.call(this,t)}}}(t)}}function d(t,e){this.fun=t,this.array=e}function f(){}i.nextTick=function(t){var e=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)e[n-1]=arguments[n];l.push(new d(t,e)),1!==l.length||c||s(h)},d.prototype.run=function(){this.fun.apply(null,this.array)},i.title="browser",i.browser=!0,i.env={},i.argv=[],i.version="",i.versions={},i.on=f,i.addListener=f,i.once=f,i.off=f,i.removeListener=f,i.removeAllListeners=f,i.emit=f,i.prependListener=f,i.prependOnceListener=f,i.listeners=function(t){return[]},i.binding=function(t){throw new Error("process.binding is not supported")},i.cwd=function(){return"/"},i.chdir=function(t){throw new Error("process.chdir is not supported")},i.umask=function(){return 0}},function(t,e,n){var r=n(13),i=n(19),o=n(1)("match");t.exports=function(t){var e;return r(t)&&(void 0!==(e=t[o])?!!e:"RegExp"==i(t))}},function(t,e,n){var r=n(7),i=n(33),o=n(1)("species");t.exports=function(t,e){var n,a=r(t).constructor;return void 0===a||null==(n=r(a)[o])?e:i(n)}},function(t,e,n){"use strict";var r=n(87)(!0);t.exports=function(t,e,n){return e+(n?r(t,e).length:1)}},function(t,e,n){var r=n(34),i=n(23);t.exports=function(t){return function(e,n){var o,a,s=String(i(e)),A=r(n),l=s.length;return A<0||A>=l?t?"":void 0:(o=s.charCodeAt(A))<55296||o>56319||A+1===l||(a=s.charCodeAt(A+1))<56320||a>57343?t?s.charAt(A):o:t?s.slice(A,A+2):a-56320+(o-55296<<10)+65536}}},function(t,e,n){"use strict";var r=n(89),i=RegExp.prototype.exec;t.exports=function(t,e){var n=t.exec;if("function"==typeof n){var o=n.call(t,e);if("object"!=typeof o)throw new TypeError("RegExp exec method returned something other than an Object or null");return o}if("RegExp"!==r(t))throw new TypeError("RegExp#exec called on incompatible receiver");return i.call(t,e)}},function(t,e,n){var r=n(19),i=n(1)("toStringTag"),o="Arguments"==r(function(){return arguments}());t.exports=function(t){var e,n,a;return void 0===t?"Undefined":null===t?"Null":"string"==typeof(n=function(t,e){try{return t[e]}catch(t){}}(e=Object(t),i))?n:o?r(e):"Object"==(a=r(e))&&"function"==typeof e.callee?"Arguments":a}},function(t,e,n){"use strict";n(91);var r=n(14),i=n(8),o=n(6),a=n(23),s=n(1),A=n(35),l=s("species"),c=!o(function(){var t=/./;return t.exec=function(){var t=[];return t.groups={a:"7"},t},"7"!=="".replace(t,"$<a>")}),u=function(){var t=/(?:)/,e=t.exec;t.exec=function(){return e.apply(this,arguments)};var n="ab".split(t);return 2===n.length&&"a"===n[0]&&"b"===n[1]}();t.exports=function(t,e,n){var g=s(t),h=!o(function(){var e={};return e[g]=function(){return 7},7!=""[t](e)}),d=h?!o(function(){var e=!1,n=/a/;return n.exec=function(){return e=!0,null},"split"===t&&(n.constructor={},n.constructor[l]=function(){return n}),n[g](""),!e}):void 0;if(!h||!d||"replace"===t&&!c||"split"===t&&!u){var f=/./[g],p=n(a,g,""[t],function(t,e,n,r,i){return e.exec===A?h&&!i?{done:!0,value:f.call(e,n,r)}:{done:!0,value:t.call(n,e,r)}:{done:!1}}),I=p[0],C=p[1];r(String.prototype,t,I),i(RegExp.prototype,g,2==e?function(t,e){return C.call(t,this,e)}:function(t){return C.call(t,this)})}}},function(t,e,n){"use strict";var r=n(35);n(24)({target:"RegExp",proto:!0,forced:r!==/./.exec},{exec:r})},function(t,e,n){t.exports=n(20)("native-function-to-string",Function.toString)},function(t,e,n){var r=n(33);t.exports=function(t,e,n){if(r(t),void 0===e)return t;switch(n){case 1:return function(n){return t.call(e,n)};case 2:return function(n,r){return t.call(e,n,r)};case 3:return function(n,r,i){return t.call(e,n,r,i)}}return function(){return t.apply(e,arguments)}}},function(t,e){t.exports.default={types:{float:{filter_operator:"==",aggregate:"sum",format:{style:"decimal",minimumFractionDigits:2,maximumFractionDigits:2}},string:{filter_operator:"==",aggregate:"count"},integer:{filter_operator:"==",aggregate:"sum",format:{}},boolean:{filter_operator:"==",aggregate:"count"},datetime:{filter_operator:"==",aggregate:"count",format:{week:"numeric",year:"numeric",month:"numeric",day:"numeric",hour:"numeric",minute:"numeric",second:"numeric"}},date:{filter_operator:"==",aggregate:"count",format:{week:"numeric",year:"numeric",month:"numeric",day:"numeric"}}}}},function(t,e,n){"use strict";var r=n(96),i=n(97),o=n(38),a=n(15);t.exports=n(99)(Array,"Array",function(t,e){this._t=a(t),this._i=0,this._k=e},function(){var t=this._t,e=this._k,n=this._i++;return!t||n>=t.length?(this._t=void 0,i(1)):i(0,"keys"==e?n:"values"==e?t[n]:[n,t[n]])},"values"),o.Arguments=o.Array,r("keys"),r("values"),r("entries")},function(t,e,n){var r=n(1)("unscopables"),i=Array.prototype;null==i[r]&&n(8)(i,r,{}),t.exports=function(t){i[r][t]=!0}},function(t,e){t.exports=function(t,e){return{value:e,done:!!t}}},function(t,e,n){var r=n(19);t.exports=Object("z").propertyIsEnumerable(0)?Object:function(t){return"String"==r(t)?t.split(""):Object(t)}},function(t,e,n){"use strict";var r=n(22),i=n(24),o=n(14),a=n(8),s=n(38),A=n(100),l=n(41),c=n(105),u=n(1)("iterator"),g=!([].keys&&"next"in[].keys()),h=function(){return this};t.exports=function(t,e,n,d,f,p,I){A(n,e,d);var C,m,y,E=function(t){if(!g&&t in Q)return Q[t];switch(t){case"keys":case"values":return function(){return new n(this,t)}}return function(){return new n(this,t)}},v=e+" Iterator",B="values"==f,b=!1,Q=t.prototype,w=Q[u]||Q["@@iterator"]||f&&Q[f],x=w||E(f),_=f?B?E("entries"):x:void 0,D="Array"==e&&Q.entries||w;if(D&&(y=c(D.call(new t)))!==Object.prototype&&y.next&&(l(y,v,!0),r||"function"==typeof y[u]||a(y,u,h)),B&&w&&"values"!==w.name&&(b=!0,x=function(){return w.call(this)}),r&&!I||!g&&!b&&Q[u]||a(Q,u,x),s[e]=x,s[v]=h,f)if(C={values:B?x:E("values"),keys:p?x:E("keys"),entries:_},I)for(m in C)m in Q||o(Q,m,C[m]);else i(i.P+i.F*(g||b),e,C);return C}},function(t,e,n){"use strict";var r=n(58),i=n(25),o=n(41),a={};n(8)(a,n(1)("iterator"),function(){return this}),t.exports=function(t,e,n){t.prototype=r(a,{next:i(1,n)}),o(t,e+" Iterator")}},function(t,e,n){var r=n(10),i=n(7),o=n(26);t.exports=n(9)?Object.defineProperties:function(t,e){i(t);for(var n,a=o(e),s=a.length,A=0;s>A;)r.f(t,n=a[A++],e[n]);return t}},function(t,e,n){var r=n(15),i=n(54),o=n(103);t.exports=function(t){return function(e,n,a){var s,A=r(e),l=i(A.length),c=o(a,l);if(t&&n!=n){for(;l>c;)if((s=A[c++])!=s)return!0}else for(;l>c;c++)if((t||c in A)&&A[c]===n)return t||c||0;return!t&&-1}}},function(t,e,n){var r=n(34),i=Math.max,o=Math.min;t.exports=function(t,e){return(t=r(t))<0?i(t+e,0):o(t,e)}},function(t,e,n){var r=n(5).document;t.exports=r&&r.documentElement},function(t,e,n){var r=n(11),i=n(42),o=n(39)("IE_PROTO"),a=Object.prototype;t.exports=Object.getPrototypeOf||function(t){return t=i(t),r(t,o)?t[o]:"function"==typeof t.constructor&&t instanceof t.constructor?t.constructor.prototype:t instanceof Object?a:null}},function(t,e,n){var r=function(t){"use strict";var e,n=Object.prototype,r=n.hasOwnProperty,i="function"==typeof Symbol?Symbol:{},o=i.iterator||"@@iterator",a=i.asyncIterator||"@@asyncIterator",s=i.toStringTag||"@@toStringTag";function A(t,e,n,r){var i=e&&e.prototype instanceof f?e:f,o=Object.create(i.prototype),a=new x(r||[]);return o._invoke=function(t,e,n){var r=c;return function(i,o){if(r===g)throw new Error("Generator is already running");if(r===h){if("throw"===i)throw o;return D()}for(n.method=i,n.arg=o;;){var a=n.delegate;if(a){var s=b(a,n);if(s){if(s===d)continue;return s}}if("next"===n.method)n.sent=n._sent=n.arg;else if("throw"===n.method){if(r===c)throw r=h,n.arg;n.dispatchException(n.arg)}else"return"===n.method&&n.abrupt("return",n.arg);r=g;var A=l(t,e,n);if("normal"===A.type){if(r=n.done?h:u,A.arg===d)continue;return{value:A.arg,done:n.done}}"throw"===A.type&&(r=h,n.method="throw",n.arg=A.arg)}}}(t,n,a),o}function l(t,e,n){try{return{type:"normal",arg:t.call(e,n)}}catch(t){return{type:"throw",arg:t}}}t.wrap=A;var c="suspendedStart",u="suspendedYield",g="executing",h="completed",d={};function f(){}function p(){}function I(){}var C={};C[o]=function(){return this};var m=Object.getPrototypeOf,y=m&&m(m(_([])));y&&y!==n&&r.call(y,o)&&(C=y);var E=I.prototype=f.prototype=Object.create(C);function v(t){["next","throw","return"].forEach(function(e){t[e]=function(t){return this._invoke(e,t)}})}function B(t){var e;this._invoke=function(n,i){function o(){return new Promise(function(e,o){!function e(n,i,o,a){var s=l(t[n],t,i);if("throw"!==s.type){var A=s.arg,c=A.value;return c&&"object"==typeof c&&r.call(c,"__await")?Promise.resolve(c.__await).then(function(t){e("next",t,o,a)},function(t){e("throw",t,o,a)}):Promise.resolve(c).then(function(t){A.value=t,o(A)},function(t){return e("throw",t,o,a)})}a(s.arg)}(n,i,e,o)})}return e=e?e.then(o,o):o()}}function b(t,n){var r=t.iterator[n.method];if(r===e){if(n.delegate=null,"throw"===n.method){if(t.iterator.return&&(n.method="return",n.arg=e,b(t,n),"throw"===n.method))return d;n.method="throw",n.arg=new TypeError("The iterator does not provide a 'throw' method")}return d}var i=l(r,t.iterator,n.arg);if("throw"===i.type)return n.method="throw",n.arg=i.arg,n.delegate=null,d;var o=i.arg;return o?o.done?(n[t.resultName]=o.value,n.next=t.nextLoc,"return"!==n.method&&(n.method="next",n.arg=e),n.delegate=null,d):o:(n.method="throw",n.arg=new TypeError("iterator result is not an object"),n.delegate=null,d)}function Q(t){var e={tryLoc:t[0]};1 in t&&(e.catchLoc=t[1]),2 in t&&(e.finallyLoc=t[2],e.afterLoc=t[3]),this.tryEntries.push(e)}function w(t){var e=t.completion||{};e.type="normal",delete e.arg,t.completion=e}function x(t){this.tryEntries=[{tryLoc:"root"}],t.forEach(Q,this),this.reset(!0)}function _(t){if(t){var n=t[o];if(n)return n.call(t);if("function"==typeof t.next)return t;if(!isNaN(t.length)){var i=-1,a=function n(){for(;++i<t.length;)if(r.call(t,i))return n.value=t[i],n.done=!1,n;return n.value=e,n.done=!0,n};return a.next=a}}return{next:D}}function D(){return{value:e,done:!0}}return p.prototype=E.constructor=I,I.constructor=p,I[s]=p.displayName="GeneratorFunction",t.isGeneratorFunction=function(t){var e="function"==typeof t&&t.constructor;return!!e&&(e===p||"GeneratorFunction"===(e.displayName||e.name))},t.mark=function(t){return Object.setPrototypeOf?Object.setPrototypeOf(t,I):(t.__proto__=I,s in t||(t[s]="GeneratorFunction")),t.prototype=Object.create(E),t},t.awrap=function(t){return{__await:t}},v(B.prototype),B.prototype[a]=function(){return this},t.AsyncIterator=B,t.async=function(e,n,r,i){var o=new B(A(e,n,r,i));return t.isGeneratorFunction(n)?o:o.next().then(function(t){return t.done?t.value:o.next()})},v(E),E[s]="Generator",E[o]=function(){return this},E.toString=function(){return"[object Generator]"},t.keys=function(t){var e=[];for(var n in t)e.push(n);return e.reverse(),function n(){for(;e.length;){var r=e.pop();if(r in t)return n.value=r,n.done=!1,n}return n.done=!0,n}},t.values=_,x.prototype={constructor:x,reset:function(t){if(this.prev=0,this.next=0,this.sent=this._sent=e,this.done=!1,this.delegate=null,this.method="next",this.arg=e,this.tryEntries.forEach(w),!t)for(var n in this)"t"===n.charAt(0)&&r.call(this,n)&&!isNaN(+n.slice(1))&&(this[n]=e)},stop:function(){this.done=!0;var t=this.tryEntries[0].completion;if("throw"===t.type)throw t.arg;return this.rval},dispatchException:function(t){if(this.done)throw t;var n=this;function i(r,i){return s.type="throw",s.arg=t,n.next=r,i&&(n.method="next",n.arg=e),!!i}for(var o=this.tryEntries.length-1;o>=0;--o){var a=this.tryEntries[o],s=a.completion;if("root"===a.tryLoc)return i("end");if(a.tryLoc<=this.prev){var A=r.call(a,"catchLoc"),l=r.call(a,"finallyLoc");if(A&&l){if(this.prev<a.catchLoc)return i(a.catchLoc,!0);if(this.prev<a.finallyLoc)return i(a.finallyLoc)}else if(A){if(this.prev<a.catchLoc)return i(a.catchLoc,!0)}else{if(!l)throw new Error("try statement without catch or finally");if(this.prev<a.finallyLoc)return i(a.finallyLoc)}}}},abrupt:function(t,e){for(var n=this.tryEntries.length-1;n>=0;--n){var i=this.tryEntries[n];if(i.tryLoc<=this.prev&&r.call(i,"finallyLoc")&&this.prev<i.finallyLoc){var o=i;break}}o&&("break"===t||"continue"===t)&&o.tryLoc<=e&&e<=o.finallyLoc&&(o=null);var a=o?o.completion:{};return a.type=t,a.arg=e,o?(this.method="next",this.next=o.finallyLoc,d):this.complete(a)},complete:function(t,e){if("throw"===t.type)throw t.arg;return"break"===t.type||"continue"===t.type?this.next=t.arg:"return"===t.type?(this.rval=this.arg=t.arg,this.method="return",this.next="end"):"normal"===t.type&&e&&(this.next=e),d},finish:function(t){for(var e=this.tryEntries.length-1;e>=0;--e){var n=this.tryEntries[e];if(n.finallyLoc===t)return this.complete(n.completion,n.afterLoc),w(n),d}},catch:function(t){for(var e=this.tryEntries.length-1;e>=0;--e){var n=this.tryEntries[e];if(n.tryLoc===t){var r=n.completion;if("throw"===r.type){var i=r.arg;w(n)}return i}}throw new Error("illegal catch attempt")},delegateYield:function(t,n,r){return this.delegate={iterator:_(t),resultName:n,nextLoc:r},"next"===this.method&&(this.arg=e),d}},t}(t.exports);try{regeneratorRuntime=r}catch(t){Function("r","regeneratorRuntime = r")(r)}},function(t,e,n){var r=n(16)("meta"),i=n(13),o=n(11),a=n(10).f,s=0,A=Object.isExtensible||function(){return!0},l=!n(6)(function(){return A(Object.preventExtensions({}))}),c=function(t){a(t,r,{value:{i:"O"+ ++s,w:{}}})},u=t.exports={KEY:r,NEED:!1,fastKey:function(t,e){if(!i(t))return"symbol"==typeof t?t:("string"==typeof t?"S":"P")+t;if(!o(t,r)){if(!A(t))return"F";if(!e)return"E";c(t)}return t[r].i},getWeak:function(t,e){if(!o(t,r)){if(!A(t))return!0;if(!e)return!1;c(t)}return t[r].w},onFreeze:function(t){return l&&u.NEED&&A(t)&&!o(t,r)&&c(t),t}}},function(t,e,n){var r=n(26),i=n(64),o=n(43);t.exports=function(t){var e=r(t),n=i.f;if(n)for(var a,s=n(t),A=o.f,l=0;s.length>l;)A.call(t,a=s[l++])&&e.push(a);return e}},function(t,e,n){var r=n(19);t.exports=Array.isArray||function(t){return"Array"==r(t)}},function(t,e,n){var r=n(15),i=n(65).f,o={}.toString,a="object"==typeof window&&window&&Object.getOwnPropertyNames?Object.getOwnPropertyNames(window):[];t.exports.f=function(t){return a&&"[object Window]"==o.call(t)?function(t){try{return i(t)}catch(t){return a.slice()}}(t):i(r(t))}},function(t,e,n){var r=n(43),i=n(25),o=n(15),a=n(37),s=n(11),A=n(55),l=Object.getOwnPropertyDescriptor;e.f=n(9)?l:function(t,e){if(t=o(t),e=a(e,!0),A)try{return l(t,e)}catch(t){}if(s(t,e))return i(!r.f.call(t,e),t[e])}},function(t,e,n){(function(t,r){var i;(function(){var o,a=200,s="Expected a function",A="__lodash_hash_undefined__",l="__lodash_placeholder__",c=1,u=2,g=4,h=1,d=2,f=1,p=2,I=4,C=8,m=16,y=32,E=64,v=128,B=256,b=512,Q=800,w=16,x=1/0,_=9007199254740991,D=17976931348623157e292,M=NaN,k=4294967295,S=k-1,L=k>>>1,R=[["ary",v],["bind",f],["bindKey",p],["curry",C],["curryRight",m],["flip",b],["partial",y],["partialRight",E],["rearg",B]],N="[object Arguments]",F="[object Array]",T="[object AsyncFunction]",U="[object Boolean]",O="[object Date]",G="[object DOMException]",H="[object Error]",j="[object Function]",Y="[object GeneratorFunction]",P="[object Map]",K="[object Number]",z="[object Null]",J="[object Object]",q="[object Proxy]",V="[object RegExp]",W="[object Set]",X="[object String]",Z="[object Symbol]",$="[object Undefined]",tt="[object WeakMap]",et="[object ArrayBuffer]",nt="[object DataView]",rt="[object Float32Array]",it="[object Float64Array]",ot="[object Int8Array]",at="[object Int16Array]",st="[object Int32Array]",At="[object Uint8Array]",lt="[object Uint8ClampedArray]",ct="[object Uint16Array]",ut="[object Uint32Array]",gt=/\b__p \+= '';/g,ht=/\b(__p \+=) '' \+/g,dt=/(__e\(.*?\)|\b__t\)) \+\n'';/g,ft=/&(?:amp|lt|gt|quot|#39);/g,pt=/[&<>"']/g,It=RegExp(ft.source),Ct=RegExp(pt.source),mt=/<%-([\s\S]+?)%>/g,yt=/<%([\s\S]+?)%>/g,Et=/<%=([\s\S]+?)%>/g,vt=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,Bt=/^\w*$/,bt=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,Qt=/[\\^$.*+?()[\]{}|]/g,wt=RegExp(Qt.source),xt=/^\s+|\s+$/g,_t=/^\s+/,Dt=/\s+$/,Mt=/\{(?:\n\/\* \[wrapped with .+\] \*\/)?\n?/,kt=/\{\n\/\* \[wrapped with (.+)\] \*/,St=/,? & /,Lt=/[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g,Rt=/\\(\\)?/g,Nt=/\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g,Ft=/\w*$/,Tt=/^[-+]0x[0-9a-f]+$/i,Ut=/^0b[01]+$/i,Ot=/^\[object .+?Constructor\]$/,Gt=/^0o[0-7]+$/i,Ht=/^(?:0|[1-9]\d*)$/,jt=/[\xc0-\xd6\xd8-\xf6\xf8-\xff\u0100-\u017f]/g,Yt=/($^)/,Pt=/['\n\r\u2028\u2029\\]/g,Kt="\\u0300-\\u036f\\ufe20-\\ufe2f\\u20d0-\\u20ff",zt="\\xac\\xb1\\xd7\\xf7\\x00-\\x2f\\x3a-\\x40\\x5b-\\x60\\x7b-\\xbf\\u2000-\\u206f \\t\\x0b\\f\\xa0\\ufeff\\n\\r\\u2028\\u2029\\u1680\\u180e\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200a\\u202f\\u205f\\u3000",Jt="["+zt+"]",qt="["+Kt+"]",Vt="\\d+",Wt="[a-z\\xdf-\\xf6\\xf8-\\xff]",Xt="[^\\ud800-\\udfff"+zt+Vt+"\\u2700-\\u27bfa-z\\xdf-\\xf6\\xf8-\\xffA-Z\\xc0-\\xd6\\xd8-\\xde]",Zt="\\ud83c[\\udffb-\\udfff]",$t="[^\\ud800-\\udfff]",te="(?:\\ud83c[\\udde6-\\uddff]){2}",ee="[\\ud800-\\udbff][\\udc00-\\udfff]",ne="[A-Z\\xc0-\\xd6\\xd8-\\xde]",re="(?:"+Wt+"|"+Xt+")",ie="(?:"+ne+"|"+Xt+")",oe="(?:"+qt+"|"+Zt+")?",ae="[\\ufe0e\\ufe0f]?"+oe+"(?:\\u200d(?:"+[$t,te,ee].join("|")+")[\\ufe0e\\ufe0f]?"+oe+")*",se="(?:"+["[\\u2700-\\u27bf]",te,ee].join("|")+")"+ae,Ae="(?:"+[$t+qt+"?",qt,te,ee,"[\\ud800-\\udfff]"].join("|")+")",le=RegExp("['’]","g"),ce=RegExp(qt,"g"),ue=RegExp(Zt+"(?="+Zt+")|"+Ae+ae,"g"),ge=RegExp([ne+"?"+Wt+"+(?:['’](?:d|ll|m|re|s|t|ve))?(?="+[Jt,ne,"$"].join("|")+")",ie+"+(?:['’](?:D|LL|M|RE|S|T|VE))?(?="+[Jt,ne+re,"$"].join("|")+")",ne+"?"+re+"+(?:['’](?:d|ll|m|re|s|t|ve))?",ne+"+(?:['’](?:D|LL|M|RE|S|T|VE))?","\\d*(?:1ST|2ND|3RD|(?![123])\\dTH)(?=\\b|[a-z_])","\\d*(?:1st|2nd|3rd|(?![123])\\dth)(?=\\b|[A-Z_])",Vt,se].join("|"),"g"),he=RegExp("[\\u200d\\ud800-\\udfff"+Kt+"\\ufe0e\\ufe0f]"),de=/[a-z][A-Z]|[A-Z]{2}[a-z]|[0-9][a-zA-Z]|[a-zA-Z][0-9]|[^a-zA-Z0-9 ]/,fe=["Array","Buffer","DataView","Date","Error","Float32Array","Float64Array","Function","Int8Array","Int16Array","Int32Array","Map","Math","Object","Promise","RegExp","Set","String","Symbol","TypeError","Uint8Array","Uint8ClampedArray","Uint16Array","Uint32Array","WeakMap","_","clearTimeout","isFinite","parseInt","setTimeout"],pe=-1,Ie={};Ie[rt]=Ie[it]=Ie[ot]=Ie[at]=Ie[st]=Ie[At]=Ie[lt]=Ie[ct]=Ie[ut]=!0,Ie[N]=Ie[F]=Ie[et]=Ie[U]=Ie[nt]=Ie[O]=Ie[H]=Ie[j]=Ie[P]=Ie[K]=Ie[J]=Ie[V]=Ie[W]=Ie[X]=Ie[tt]=!1;var Ce={};Ce[N]=Ce[F]=Ce[et]=Ce[nt]=Ce[U]=Ce[O]=Ce[rt]=Ce[it]=Ce[ot]=Ce[at]=Ce[st]=Ce[P]=Ce[K]=Ce[J]=Ce[V]=Ce[W]=Ce[X]=Ce[Z]=Ce[At]=Ce[lt]=Ce[ct]=Ce[ut]=!0,Ce[H]=Ce[j]=Ce[tt]=!1;var me={"\\":"\\","'":"'","\n":"n","\r":"r","\u2028":"u2028","\u2029":"u2029"},ye=parseFloat,Ee=parseInt,ve="object"==typeof t&&t&&t.Object===Object&&t,Be="object"==typeof self&&self&&self.Object===Object&&self,be=ve||Be||Function("return this")(),Qe=e&&!e.nodeType&&e,we=Qe&&"object"==typeof r&&r&&!r.nodeType&&r,xe=we&&we.exports===Qe,_e=xe&&ve.process,De=function(){try{return we&&we.require&&we.require("util").types||_e&&_e.binding&&_e.binding("util")}catch(t){}}(),Me=De&&De.isArrayBuffer,ke=De&&De.isDate,Se=De&&De.isMap,Le=De&&De.isRegExp,Re=De&&De.isSet,Ne=De&&De.isTypedArray;function Fe(t,e,n){switch(n.length){case 0:return t.call(e);case 1:return t.call(e,n[0]);case 2:return t.call(e,n[0],n[1]);case 3:return t.call(e,n[0],n[1],n[2])}return t.apply(e,n)}function Te(t,e,n,r){for(var i=-1,o=null==t?0:t.length;++i<o;){var a=t[i];e(r,a,n(a),t)}return r}function Ue(t,e){for(var n=-1,r=null==t?0:t.length;++n<r&&!1!==e(t[n],n,t););return t}function Oe(t,e){for(var n=-1,r=null==t?0:t.length;++n<r;)if(!e(t[n],n,t))return!1;return!0}function Ge(t,e){for(var n=-1,r=null==t?0:t.length,i=0,o=[];++n<r;){var a=t[n];e(a,n,t)&&(o[i++]=a)}return o}function He(t,e){return!(null==t||!t.length)&&Xe(t,e,0)>-1}function je(t,e,n){for(var r=-1,i=null==t?0:t.length;++r<i;)if(n(e,t[r]))return!0;return!1}function Ye(t,e){for(var n=-1,r=null==t?0:t.length,i=Array(r);++n<r;)i[n]=e(t[n],n,t);return i}function Pe(t,e){for(var n=-1,r=e.length,i=t.length;++n<r;)t[i+n]=e[n];return t}function Ke(t,e,n,r){var i=-1,o=null==t?0:t.length;for(r&&o&&(n=t[++i]);++i<o;)n=e(n,t[i],i,t);return n}function ze(t,e,n,r){var i=null==t?0:t.length;for(r&&i&&(n=t[--i]);i--;)n=e(n,t[i],i,t);return n}function Je(t,e){for(var n=-1,r=null==t?0:t.length;++n<r;)if(e(t[n],n,t))return!0;return!1}var qe=en("length");function Ve(t,e,n){var r;return n(t,function(t,n,i){if(e(t,n,i))return r=n,!1}),r}function We(t,e,n,r){for(var i=t.length,o=n+(r?1:-1);r?o--:++o<i;)if(e(t[o],o,t))return o;return-1}function Xe(t,e,n){return e==e?function(t,e,n){for(var r=n-1,i=t.length;++r<i;)if(t[r]===e)return r;return-1}(t,e,n):We(t,$e,n)}function Ze(t,e,n,r){for(var i=n-1,o=t.length;++i<o;)if(r(t[i],e))return i;return-1}function $e(t){return t!=t}function tn(t,e){var n=null==t?0:t.length;return n?on(t,e)/n:M}function en(t){return function(e){return null==e?o:e[t]}}function nn(t){return function(e){return null==t?o:t[e]}}function rn(t,e,n,r,i){return i(t,function(t,i,o){n=r?(r=!1,t):e(n,t,i,o)}),n}function on(t,e){for(var n,r=-1,i=t.length;++r<i;){var a=e(t[r]);a!==o&&(n=n===o?a:n+a)}return n}function an(t,e){for(var n=-1,r=Array(t);++n<t;)r[n]=e(n);return r}function sn(t){return function(e){return t(e)}}function An(t,e){return Ye(e,function(e){return t[e]})}function ln(t,e){return t.has(e)}function cn(t,e){for(var n=-1,r=t.length;++n<r&&Xe(e,t[n],0)>-1;);return n}function un(t,e){for(var n=t.length;n--&&Xe(e,t[n],0)>-1;);return n}var gn=nn({"À":"A","Á":"A","Â":"A","Ã":"A","Ä":"A","Å":"A","à":"a","á":"a","â":"a","ã":"a","ä":"a","å":"a","Ç":"C","ç":"c","Ð":"D","ð":"d","È":"E","É":"E","Ê":"E","Ë":"E","è":"e","é":"e","ê":"e","ë":"e","Ì":"I","Í":"I","Î":"I","Ï":"I","ì":"i","í":"i","î":"i","ï":"i","Ñ":"N","ñ":"n","Ò":"O","Ó":"O","Ô":"O","Õ":"O","Ö":"O","Ø":"O","ò":"o","ó":"o","ô":"o","õ":"o","ö":"o","ø":"o","Ù":"U","Ú":"U","Û":"U","Ü":"U","ù":"u","ú":"u","û":"u","ü":"u","Ý":"Y","ý":"y","ÿ":"y","Æ":"Ae","æ":"ae","Þ":"Th","þ":"th","ß":"ss","Ā":"A","Ă":"A","Ą":"A","ā":"a","ă":"a","ą":"a","Ć":"C","Ĉ":"C","Ċ":"C","Č":"C","ć":"c","ĉ":"c","ċ":"c","č":"c","Ď":"D","Đ":"D","ď":"d","đ":"d","Ē":"E","Ĕ":"E","Ė":"E","Ę":"E","Ě":"E","ē":"e","ĕ":"e","ė":"e","ę":"e","ě":"e","Ĝ":"G","Ğ":"G","Ġ":"G","Ģ":"G","ĝ":"g","ğ":"g","ġ":"g","ģ":"g","Ĥ":"H","Ħ":"H","ĥ":"h","ħ":"h","Ĩ":"I","Ī":"I","Ĭ":"I","Į":"I","İ":"I","ĩ":"i","ī":"i","ĭ":"i","į":"i","ı":"i","Ĵ":"J","ĵ":"j","Ķ":"K","ķ":"k","ĸ":"k","Ĺ":"L","Ļ":"L","Ľ":"L","Ŀ":"L","Ł":"L","ĺ":"l","ļ":"l","ľ":"l","ŀ":"l","ł":"l","Ń":"N","Ņ":"N","Ň":"N","Ŋ":"N","ń":"n","ņ":"n","ň":"n","ŋ":"n","Ō":"O","Ŏ":"O","Ő":"O","ō":"o","ŏ":"o","ő":"o","Ŕ":"R","Ŗ":"R","Ř":"R","ŕ":"r","ŗ":"r","ř":"r","Ś":"S","Ŝ":"S","Ş":"S","Š":"S","ś":"s","ŝ":"s","ş":"s","š":"s","Ţ":"T","Ť":"T","Ŧ":"T","ţ":"t","ť":"t","ŧ":"t","Ũ":"U","Ū":"U","Ŭ":"U","Ů":"U","Ű":"U","Ų":"U","ũ":"u","ū":"u","ŭ":"u","ů":"u","ű":"u","ų":"u","Ŵ":"W","ŵ":"w","Ŷ":"Y","ŷ":"y","Ÿ":"Y","Ź":"Z","Ż":"Z","Ž":"Z","ź":"z","ż":"z","ž":"z","Ĳ":"IJ","ĳ":"ij","Œ":"Oe","œ":"oe","ŉ":"'n","ſ":"s"}),hn=nn({"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"});function dn(t){return"\\"+me[t]}function fn(t){return he.test(t)}function pn(t){var e=-1,n=Array(t.size);return t.forEach(function(t,r){n[++e]=[r,t]}),n}function In(t,e){return function(n){return t(e(n))}}function Cn(t,e){for(var n=-1,r=t.length,i=0,o=[];++n<r;){var a=t[n];a!==e&&a!==l||(t[n]=l,o[i++]=n)}return o}function mn(t){var e=-1,n=Array(t.size);return t.forEach(function(t){n[++e]=t}),n}function yn(t){return fn(t)?function(t){for(var e=ue.lastIndex=0;ue.test(t);)++e;return e}(t):qe(t)}function En(t){return fn(t)?function(t){return t.match(ue)||[]}(t):function(t){return t.split("")}(t)}var vn=nn({"&amp;":"&","&lt;":"<","&gt;":">","&quot;":'"',"&#39;":"'"}),Bn=function t(e){var n,r=(e=null==e?be:Bn.defaults(be.Object(),e,Bn.pick(be,fe))).Array,i=e.Date,Kt=e.Error,zt=e.Function,Jt=e.Math,qt=e.Object,Vt=e.RegExp,Wt=e.String,Xt=e.TypeError,Zt=r.prototype,$t=zt.prototype,te=qt.prototype,ee=e["__core-js_shared__"],ne=$t.toString,re=te.hasOwnProperty,ie=0,oe=(n=/[^.]+$/.exec(ee&&ee.keys&&ee.keys.IE_PROTO||""))?"Symbol(src)_1."+n:"",ae=te.toString,se=ne.call(qt),Ae=be._,ue=Vt("^"+ne.call(re).replace(Qt,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),he=xe?e.Buffer:o,me=e.Symbol,ve=e.Uint8Array,Be=he?he.allocUnsafe:o,Qe=In(qt.getPrototypeOf,qt),we=qt.create,_e=te.propertyIsEnumerable,De=Zt.splice,qe=me?me.isConcatSpreadable:o,nn=me?me.iterator:o,bn=me?me.toStringTag:o,Qn=function(){try{var t=bo(qt,"defineProperty");return t({},"",{}),t}catch(t){}}(),wn=e.clearTimeout!==be.clearTimeout&&e.clearTimeout,xn=i&&i.now!==be.Date.now&&i.now,_n=e.setTimeout!==be.setTimeout&&e.setTimeout,Dn=Jt.ceil,Mn=Jt.floor,kn=qt.getOwnPropertySymbols,Sn=he?he.isBuffer:o,Ln=e.isFinite,Rn=Zt.join,Nn=In(qt.keys,qt),Fn=Jt.max,Tn=Jt.min,Un=i.now,On=e.parseInt,Gn=Jt.random,Hn=Zt.reverse,jn=bo(e,"DataView"),Yn=bo(e,"Map"),Pn=bo(e,"Promise"),Kn=bo(e,"Set"),zn=bo(e,"WeakMap"),Jn=bo(qt,"create"),qn=zn&&new zn,Vn={},Wn=Wo(jn),Xn=Wo(Yn),Zn=Wo(Pn),$n=Wo(Kn),tr=Wo(zn),er=me?me.prototype:o,nr=er?er.valueOf:o,rr=er?er.toString:o;function ir(t){if(hs(t)&&!ns(t)&&!(t instanceof Ar)){if(t instanceof sr)return t;if(re.call(t,"__wrapped__"))return Xo(t)}return new sr(t)}var or=function(){function t(){}return function(e){if(!gs(e))return{};if(we)return we(e);t.prototype=e;var n=new t;return t.prototype=o,n}}();function ar(){}function sr(t,e){this.__wrapped__=t,this.__actions__=[],this.__chain__=!!e,this.__index__=0,this.__values__=o}function Ar(t){this.__wrapped__=t,this.__actions__=[],this.__dir__=1,this.__filtered__=!1,this.__iteratees__=[],this.__takeCount__=k,this.__views__=[]}function lr(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function cr(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function ur(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}function gr(t){var e=-1,n=null==t?0:t.length;for(this.__data__=new ur;++e<n;)this.add(t[e])}function hr(t){var e=this.__data__=new cr(t);this.size=e.size}function dr(t,e){var n=ns(t),r=!n&&es(t),i=!n&&!r&&as(t),o=!n&&!r&&!i&&Es(t),a=n||r||i||o,s=a?an(t.length,Wt):[],A=s.length;for(var l in t)!e&&!re.call(t,l)||a&&("length"==l||i&&("offset"==l||"parent"==l)||o&&("buffer"==l||"byteLength"==l||"byteOffset"==l)||ko(l,A))||s.push(l);return s}function fr(t){var e=t.length;return e?t[li(0,e-1)]:o}function pr(t,e,n){(n===o||Za(t[e],n))&&(n!==o||e in t)||Er(t,e,n)}function Ir(t,e,n){var r=t[e];re.call(t,e)&&Za(r,n)&&(n!==o||e in t)||Er(t,e,n)}function Cr(t,e){for(var n=t.length;n--;)if(Za(t[n][0],e))return n;return-1}function mr(t,e,n,r){return _r(t,function(t,i,o){e(r,t,n(t),o)}),r}function yr(t,e){return t&&ji(e,Ys(e),t)}function Er(t,e,n){"__proto__"==e&&Qn?Qn(t,e,{configurable:!0,enumerable:!0,value:n,writable:!0}):t[e]=n}function vr(t,e){for(var n=-1,i=e.length,a=r(i),s=null==t;++n<i;)a[n]=s?o:Us(t,e[n]);return a}function Br(t,e,n){return t==t&&(n!==o&&(t=t<=n?t:n),e!==o&&(t=t>=e?t:e)),t}function br(t,e,n,r,i,a){var s,A=e&c,l=e&u,h=e&g;if(n&&(s=i?n(t,r,i,a):n(t)),s!==o)return s;if(!gs(t))return t;var d=ns(t);if(d){if(s=function(t){var e=t.length,n=new t.constructor(e);return e&&"string"==typeof t[0]&&re.call(t,"index")&&(n.index=t.index,n.input=t.input),n}(t),!A)return Hi(t,s)}else{var f=xo(t),p=f==j||f==Y;if(as(t))return Ni(t,A);if(f==J||f==N||p&&!i){if(s=l||p?{}:Do(t),!A)return l?function(t,e){return ji(t,wo(t),e)}(t,function(t,e){return t&&ji(e,Ps(e),t)}(s,t)):function(t,e){return ji(t,Qo(t),e)}(t,yr(s,t))}else{if(!Ce[f])return i?t:{};s=function(t,e,n){var r,i=t.constructor;switch(e){case et:return Fi(t);case U:case O:return new i(+t);case nt:return function(t,e){var n=e?Fi(t.buffer):t.buffer;return new t.constructor(n,t.byteOffset,t.byteLength)}(t,n);case rt:case it:case ot:case at:case st:case At:case lt:case ct:case ut:return Ti(t,n);case P:return new i;case K:case X:return new i(t);case V:return function(t){var e=new t.constructor(t.source,Ft.exec(t));return e.lastIndex=t.lastIndex,e}(t);case W:return new i;case Z:return r=t,nr?qt(nr.call(r)):{}}}(t,f,A)}}a||(a=new hr);var I=a.get(t);if(I)return I;a.set(t,s),Cs(t)?t.forEach(function(r){s.add(br(r,e,n,r,t,a))}):ds(t)&&t.forEach(function(r,i){s.set(i,br(r,e,n,i,t,a))});var C=d?o:(h?l?Io:po:l?Ps:Ys)(t);return Ue(C||t,function(r,i){C&&(r=t[i=r]),Ir(s,i,br(r,e,n,i,t,a))}),s}function Qr(t,e,n){var r=n.length;if(null==t)return!r;for(t=qt(t);r--;){var i=n[r],a=e[i],s=t[i];if(s===o&&!(i in t)||!a(s))return!1}return!0}function wr(t,e,n){if("function"!=typeof t)throw new Xt(s);return Yo(function(){t.apply(o,n)},e)}function xr(t,e,n,r){var i=-1,o=He,s=!0,A=t.length,l=[],c=e.length;if(!A)return l;n&&(e=Ye(e,sn(n))),r?(o=je,s=!1):e.length>=a&&(o=ln,s=!1,e=new gr(e));t:for(;++i<A;){var u=t[i],g=null==n?u:n(u);if(u=r||0!==u?u:0,s&&g==g){for(var h=c;h--;)if(e[h]===g)continue t;l.push(u)}else o(e,g,r)||l.push(u)}return l}ir.templateSettings={escape:mt,evaluate:yt,interpolate:Et,variable:"",imports:{_:ir}},ir.prototype=ar.prototype,ir.prototype.constructor=ir,sr.prototype=or(ar.prototype),sr.prototype.constructor=sr,Ar.prototype=or(ar.prototype),Ar.prototype.constructor=Ar,lr.prototype.clear=function(){this.__data__=Jn?Jn(null):{},this.size=0},lr.prototype.delete=function(t){var e=this.has(t)&&delete this.__data__[t];return this.size-=e?1:0,e},lr.prototype.get=function(t){var e=this.__data__;if(Jn){var n=e[t];return n===A?o:n}return re.call(e,t)?e[t]:o},lr.prototype.has=function(t){var e=this.__data__;return Jn?e[t]!==o:re.call(e,t)},lr.prototype.set=function(t,e){var n=this.__data__;return this.size+=this.has(t)?0:1,n[t]=Jn&&e===o?A:e,this},cr.prototype.clear=function(){this.__data__=[],this.size=0},cr.prototype.delete=function(t){var e=this.__data__,n=Cr(e,t);return!(n<0||(n==e.length-1?e.pop():De.call(e,n,1),--this.size,0))},cr.prototype.get=function(t){var e=this.__data__,n=Cr(e,t);return n<0?o:e[n][1]},cr.prototype.has=function(t){return Cr(this.__data__,t)>-1},cr.prototype.set=function(t,e){var n=this.__data__,r=Cr(n,t);return r<0?(++this.size,n.push([t,e])):n[r][1]=e,this},ur.prototype.clear=function(){this.size=0,this.__data__={hash:new lr,map:new(Yn||cr),string:new lr}},ur.prototype.delete=function(t){var e=vo(this,t).delete(t);return this.size-=e?1:0,e},ur.prototype.get=function(t){return vo(this,t).get(t)},ur.prototype.has=function(t){return vo(this,t).has(t)},ur.prototype.set=function(t,e){var n=vo(this,t),r=n.size;return n.set(t,e),this.size+=n.size==r?0:1,this},gr.prototype.add=gr.prototype.push=function(t){return this.__data__.set(t,A),this},gr.prototype.has=function(t){return this.__data__.has(t)},hr.prototype.clear=function(){this.__data__=new cr,this.size=0},hr.prototype.delete=function(t){var e=this.__data__,n=e.delete(t);return this.size=e.size,n},hr.prototype.get=function(t){return this.__data__.get(t)},hr.prototype.has=function(t){return this.__data__.has(t)},hr.prototype.set=function(t,e){var n=this.__data__;if(n instanceof cr){var r=n.__data__;if(!Yn||r.length<a-1)return r.push([t,e]),this.size=++n.size,this;n=this.__data__=new ur(r)}return n.set(t,e),this.size=n.size,this};var _r=Ki(Fr),Dr=Ki(Tr,!0);function Mr(t,e){var n=!0;return _r(t,function(t,r,i){return n=!!e(t,r,i)}),n}function kr(t,e,n){for(var r=-1,i=t.length;++r<i;){var a=t[r],s=e(a);if(null!=s&&(A===o?s==s&&!ys(s):n(s,A)))var A=s,l=a}return l}function Sr(t,e){var n=[];return _r(t,function(t,r,i){e(t,r,i)&&n.push(t)}),n}function Lr(t,e,n,r,i){var o=-1,a=t.length;for(n||(n=Mo),i||(i=[]);++o<a;){var s=t[o];e>0&&n(s)?e>1?Lr(s,e-1,n,r,i):Pe(i,s):r||(i[i.length]=s)}return i}var Rr=zi(),Nr=zi(!0);function Fr(t,e){return t&&Rr(t,e,Ys)}function Tr(t,e){return t&&Nr(t,e,Ys)}function Ur(t,e){return Ge(e,function(e){return ls(t[e])})}function Or(t,e){for(var n=0,r=(e=ki(e,t)).length;null!=t&&n<r;)t=t[Vo(e[n++])];return n&&n==r?t:o}function Gr(t,e,n){var r=e(t);return ns(t)?r:Pe(r,n(t))}function Hr(t){return null==t?t===o?$:z:bn&&bn in qt(t)?function(t){var e=re.call(t,bn),n=t[bn];try{t[bn]=o;var r=!0}catch(t){}var i=ae.call(t);return r&&(e?t[bn]=n:delete t[bn]),i}(t):function(t){return ae.call(t)}(t)}function jr(t,e){return t>e}function Yr(t,e){return null!=t&&re.call(t,e)}function Pr(t,e){return null!=t&&e in qt(t)}function Kr(t,e,n){for(var i=n?je:He,a=t[0].length,s=t.length,A=s,l=r(s),c=1/0,u=[];A--;){var g=t[A];A&&e&&(g=Ye(g,sn(e))),c=Tn(g.length,c),l[A]=!n&&(e||a>=120&&g.length>=120)?new gr(A&&g):o}g=t[0];var h=-1,d=l[0];t:for(;++h<a&&u.length<c;){var f=g[h],p=e?e(f):f;if(f=n||0!==f?f:0,!(d?ln(d,p):i(u,p,n))){for(A=s;--A;){var I=l[A];if(!(I?ln(I,p):i(t[A],p,n)))continue t}d&&d.push(p),u.push(f)}}return u}function zr(t,e,n){var r=null==(t=Go(t,e=ki(e,t)))?t:t[Vo(Aa(e))];return null==r?o:Fe(r,t,n)}function Jr(t){return hs(t)&&Hr(t)==N}function qr(t,e,n,r,i){return t===e||(null==t||null==e||!hs(t)&&!hs(e)?t!=t&&e!=e:function(t,e,n,r,i,a){var s=ns(t),A=ns(e),l=s?F:xo(t),c=A?F:xo(e),u=(l=l==N?J:l)==J,g=(c=c==N?J:c)==J,f=l==c;if(f&&as(t)){if(!as(e))return!1;s=!0,u=!1}if(f&&!u)return a||(a=new hr),s||Es(t)?ho(t,e,n,r,i,a):function(t,e,n,r,i,o,a){switch(n){case nt:if(t.byteLength!=e.byteLength||t.byteOffset!=e.byteOffset)return!1;t=t.buffer,e=e.buffer;case et:return!(t.byteLength!=e.byteLength||!o(new ve(t),new ve(e)));case U:case O:case K:return Za(+t,+e);case H:return t.name==e.name&&t.message==e.message;case V:case X:return t==e+"";case P:var s=pn;case W:var A=r&h;if(s||(s=mn),t.size!=e.size&&!A)return!1;var l=a.get(t);if(l)return l==e;r|=d,a.set(t,e);var c=ho(s(t),s(e),r,i,o,a);return a.delete(t),c;case Z:if(nr)return nr.call(t)==nr.call(e)}return!1}(t,e,l,n,r,i,a);if(!(n&h)){var p=u&&re.call(t,"__wrapped__"),I=g&&re.call(e,"__wrapped__");if(p||I){var C=p?t.value():t,m=I?e.value():e;return a||(a=new hr),i(C,m,n,r,a)}}return!!f&&(a||(a=new hr),function(t,e,n,r,i,a){var s=n&h,A=po(t),l=A.length;if(l!=po(e).length&&!s)return!1;for(var c=l;c--;){var u=A[c];if(!(s?u in e:re.call(e,u)))return!1}var g=a.get(t);if(g&&a.get(e))return g==e;var d=!0;a.set(t,e),a.set(e,t);for(var f=s;++c<l;){var p=t[u=A[c]],I=e[u];if(r)var C=s?r(I,p,u,e,t,a):r(p,I,u,t,e,a);if(!(C===o?p===I||i(p,I,n,r,a):C)){d=!1;break}f||(f="constructor"==u)}if(d&&!f){var m=t.constructor,y=e.constructor;m!=y&&"constructor"in t&&"constructor"in e&&!("function"==typeof m&&m instanceof m&&"function"==typeof y&&y instanceof y)&&(d=!1)}return a.delete(t),a.delete(e),d}(t,e,n,r,i,a))}(t,e,n,r,qr,i))}function Vr(t,e,n,r){var i=n.length,a=i,s=!r;if(null==t)return!a;for(t=qt(t);i--;){var A=n[i];if(s&&A[2]?A[1]!==t[A[0]]:!(A[0]in t))return!1}for(;++i<a;){var l=(A=n[i])[0],c=t[l],u=A[1];if(s&&A[2]){if(c===o&&!(l in t))return!1}else{var g=new hr;if(r)var f=r(c,u,l,t,e,g);if(!(f===o?qr(u,c,h|d,r,g):f))return!1}}return!0}function Wr(t){return!(!gs(t)||(e=t,oe&&oe in e))&&(ls(t)?ue:Ot).test(Wo(t));var e}function Xr(t){return"function"==typeof t?t:null==t?fA:"object"==typeof t?ns(t)?ni(t[0],t[1]):ei(t):bA(t)}function Zr(t){if(!Fo(t))return Nn(t);var e=[];for(var n in qt(t))re.call(t,n)&&"constructor"!=n&&e.push(n);return e}function $r(t,e){return t<e}function ti(t,e){var n=-1,i=is(t)?r(t.length):[];return _r(t,function(t,r,o){i[++n]=e(t,r,o)}),i}function ei(t){var e=Bo(t);return 1==e.length&&e[0][2]?Uo(e[0][0],e[0][1]):function(n){return n===t||Vr(n,t,e)}}function ni(t,e){return Lo(t)&&To(e)?Uo(Vo(t),e):function(n){var r=Us(n,t);return r===o&&r===e?Os(n,t):qr(e,r,h|d)}}function ri(t,e,n,r,i){t!==e&&Rr(e,function(a,s){if(i||(i=new hr),gs(a))!function(t,e,n,r,i,a,s){var A=Ho(t,n),l=Ho(e,n),c=s.get(l);if(c)pr(t,n,c);else{var u=a?a(A,l,n+"",t,e,s):o,g=u===o;if(g){var h=ns(l),d=!h&&as(l),f=!h&&!d&&Es(l);u=l,h||d||f?ns(A)?u=A:os(A)?u=Hi(A):d?(g=!1,u=Ni(l,!0)):f?(g=!1,u=Ti(l,!0)):u=[]:ps(l)||es(l)?(u=A,es(A)?u=Ds(A):gs(A)&&!ls(A)||(u=Do(l))):g=!1}g&&(s.set(l,u),i(u,l,r,a,s),s.delete(l)),pr(t,n,u)}}(t,e,s,n,ri,r,i);else{var A=r?r(Ho(t,s),a,s+"",t,e,i):o;A===o&&(A=a),pr(t,s,A)}},Ps)}function ii(t,e){var n=t.length;if(n)return ko(e+=e<0?n:0,n)?t[e]:o}function oi(t,e,n){var r=-1;return e=Ye(e.length?e:[fA],sn(Eo())),function(t,e){var n=t.length;for(t.sort(e);n--;)t[n]=t[n].value;return t}(ti(t,function(t,n,i){return{criteria:Ye(e,function(e){return e(t)}),index:++r,value:t}}),function(t,e){return function(t,e,n){for(var r=-1,i=t.criteria,o=e.criteria,a=i.length,s=n.length;++r<a;){var A=Ui(i[r],o[r]);if(A)return r>=s?A:A*("desc"==n[r]?-1:1)}return t.index-e.index}(t,e,n)})}function ai(t,e,n){for(var r=-1,i=e.length,o={};++r<i;){var a=e[r],s=Or(t,a);n(s,a)&&gi(o,ki(a,t),s)}return o}function si(t,e,n,r){var i=r?Ze:Xe,o=-1,a=e.length,s=t;for(t===e&&(e=Hi(e)),n&&(s=Ye(t,sn(n)));++o<a;)for(var A=0,l=e[o],c=n?n(l):l;(A=i(s,c,A,r))>-1;)s!==t&&De.call(s,A,1),De.call(t,A,1);return t}function Ai(t,e){for(var n=t?e.length:0,r=n-1;n--;){var i=e[n];if(n==r||i!==o){var o=i;ko(i)?De.call(t,i,1):Bi(t,i)}}return t}function li(t,e){return t+Mn(Gn()*(e-t+1))}function ci(t,e){var n="";if(!t||e<1||e>_)return n;do{e%2&&(n+=t),(e=Mn(e/2))&&(t+=t)}while(e);return n}function ui(t,e){return Po(Oo(t,e,fA),t+"")}function gi(t,e,n,r){if(!gs(t))return t;for(var i=-1,a=(e=ki(e,t)).length,s=a-1,A=t;null!=A&&++i<a;){var l=Vo(e[i]),c=n;if(i!=s){var u=A[l];(c=r?r(u,l,A):o)===o&&(c=gs(u)?u:ko(e[i+1])?[]:{})}Ir(A,l,c),A=A[l]}return t}var hi=qn?function(t,e){return qn.set(t,e),t}:fA,di=Qn?function(t,e){return Qn(t,"toString",{configurable:!0,enumerable:!1,value:gA(e),writable:!0})}:fA;function fi(t,e,n){var i=-1,o=t.length;e<0&&(e=-e>o?0:o+e),(n=n>o?o:n)<0&&(n+=o),o=e>n?0:n-e>>>0,e>>>=0;for(var a=r(o);++i<o;)a[i]=t[i+e];return a}function pi(t,e){var n;return _r(t,function(t,r,i){return!(n=e(t,r,i))}),!!n}function Ii(t,e,n){var r=0,i=null==t?r:t.length;if("number"==typeof e&&e==e&&i<=L){for(;r<i;){var o=r+i>>>1,a=t[o];null!==a&&!ys(a)&&(n?a<=e:a<e)?r=o+1:i=o}return i}return Ci(t,e,fA,n)}function Ci(t,e,n,r){e=n(e);for(var i=0,a=null==t?0:t.length,s=e!=e,A=null===e,l=ys(e),c=e===o;i<a;){var u=Mn((i+a)/2),g=n(t[u]),h=g!==o,d=null===g,f=g==g,p=ys(g);if(s)var I=r||f;else I=c?f&&(r||h):A?f&&h&&(r||!d):l?f&&h&&!d&&(r||!p):!d&&!p&&(r?g<=e:g<e);I?i=u+1:a=u}return Tn(a,S)}function mi(t,e){for(var n=-1,r=t.length,i=0,o=[];++n<r;){var a=t[n],s=e?e(a):a;if(!n||!Za(s,A)){var A=s;o[i++]=0===a?0:a}}return o}function yi(t){return"number"==typeof t?t:ys(t)?M:+t}function Ei(t){if("string"==typeof t)return t;if(ns(t))return Ye(t,Ei)+"";if(ys(t))return rr?rr.call(t):"";var e=t+"";return"0"==e&&1/t==-x?"-0":e}function vi(t,e,n){var r=-1,i=He,o=t.length,s=!0,A=[],l=A;if(n)s=!1,i=je;else if(o>=a){var c=e?null:so(t);if(c)return mn(c);s=!1,i=ln,l=new gr}else l=e?[]:A;t:for(;++r<o;){var u=t[r],g=e?e(u):u;if(u=n||0!==u?u:0,s&&g==g){for(var h=l.length;h--;)if(l[h]===g)continue t;e&&l.push(g),A.push(u)}else i(l,g,n)||(l!==A&&l.push(g),A.push(u))}return A}function Bi(t,e){return null==(t=Go(t,e=ki(e,t)))||delete t[Vo(Aa(e))]}function bi(t,e,n,r){return gi(t,e,n(Or(t,e)),r)}function Qi(t,e,n,r){for(var i=t.length,o=r?i:-1;(r?o--:++o<i)&&e(t[o],o,t););return n?fi(t,r?0:o,r?o+1:i):fi(t,r?o+1:0,r?i:o)}function wi(t,e){var n=t;return n instanceof Ar&&(n=n.value()),Ke(e,function(t,e){return e.func.apply(e.thisArg,Pe([t],e.args))},n)}function xi(t,e,n){var i=t.length;if(i<2)return i?vi(t[0]):[];for(var o=-1,a=r(i);++o<i;)for(var s=t[o],A=-1;++A<i;)A!=o&&(a[o]=xr(a[o]||s,t[A],e,n));return vi(Lr(a,1),e,n)}function _i(t,e,n){for(var r=-1,i=t.length,a=e.length,s={};++r<i;){var A=r<a?e[r]:o;n(s,t[r],A)}return s}function Di(t){return os(t)?t:[]}function Mi(t){return"function"==typeof t?t:fA}function ki(t,e){return ns(t)?t:Lo(t,e)?[t]:qo(Ms(t))}var Si=ui;function Li(t,e,n){var r=t.length;return n=n===o?r:n,!e&&n>=r?t:fi(t,e,n)}var Ri=wn||function(t){return be.clearTimeout(t)};function Ni(t,e){if(e)return t.slice();var n=t.length,r=Be?Be(n):new t.constructor(n);return t.copy(r),r}function Fi(t){var e=new t.constructor(t.byteLength);return new ve(e).set(new ve(t)),e}function Ti(t,e){var n=e?Fi(t.buffer):t.buffer;return new t.constructor(n,t.byteOffset,t.length)}function Ui(t,e){if(t!==e){var n=t!==o,r=null===t,i=t==t,a=ys(t),s=e!==o,A=null===e,l=e==e,c=ys(e);if(!A&&!c&&!a&&t>e||a&&s&&l&&!A&&!c||r&&s&&l||!n&&l||!i)return 1;if(!r&&!a&&!c&&t<e||c&&n&&i&&!r&&!a||A&&n&&i||!s&&i||!l)return-1}return 0}function Oi(t,e,n,i){for(var o=-1,a=t.length,s=n.length,A=-1,l=e.length,c=Fn(a-s,0),u=r(l+c),g=!i;++A<l;)u[A]=e[A];for(;++o<s;)(g||o<a)&&(u[n[o]]=t[o]);for(;c--;)u[A++]=t[o++];return u}function Gi(t,e,n,i){for(var o=-1,a=t.length,s=-1,A=n.length,l=-1,c=e.length,u=Fn(a-A,0),g=r(u+c),h=!i;++o<u;)g[o]=t[o];for(var d=o;++l<c;)g[d+l]=e[l];for(;++s<A;)(h||o<a)&&(g[d+n[s]]=t[o++]);return g}function Hi(t,e){var n=-1,i=t.length;for(e||(e=r(i));++n<i;)e[n]=t[n];return e}function ji(t,e,n,r){var i=!n;n||(n={});for(var a=-1,s=e.length;++a<s;){var A=e[a],l=r?r(n[A],t[A],A,n,t):o;l===o&&(l=t[A]),i?Er(n,A,l):Ir(n,A,l)}return n}function Yi(t,e){return function(n,r){var i=ns(n)?Te:mr,o=e?e():{};return i(n,t,Eo(r,2),o)}}function Pi(t){return ui(function(e,n){var r=-1,i=n.length,a=i>1?n[i-1]:o,s=i>2?n[2]:o;for(a=t.length>3&&"function"==typeof a?(i--,a):o,s&&So(n[0],n[1],s)&&(a=i<3?o:a,i=1),e=qt(e);++r<i;){var A=n[r];A&&t(e,A,r,a)}return e})}function Ki(t,e){return function(n,r){if(null==n)return n;if(!is(n))return t(n,r);for(var i=n.length,o=e?i:-1,a=qt(n);(e?o--:++o<i)&&!1!==r(a[o],o,a););return n}}function zi(t){return function(e,n,r){for(var i=-1,o=qt(e),a=r(e),s=a.length;s--;){var A=a[t?s:++i];if(!1===n(o[A],A,o))break}return e}}function Ji(t){return function(e){var n=fn(e=Ms(e))?En(e):o,r=n?n[0]:e.charAt(0),i=n?Li(n,1).join(""):e.slice(1);return r[t]()+i}}function qi(t){return function(e){return Ke(lA(eA(e).replace(le,"")),t,"")}}function Vi(t){return function(){var e=arguments;switch(e.length){case 0:return new t;case 1:return new t(e[0]);case 2:return new t(e[0],e[1]);case 3:return new t(e[0],e[1],e[2]);case 4:return new t(e[0],e[1],e[2],e[3]);case 5:return new t(e[0],e[1],e[2],e[3],e[4]);case 6:return new t(e[0],e[1],e[2],e[3],e[4],e[5]);case 7:return new t(e[0],e[1],e[2],e[3],e[4],e[5],e[6])}var n=or(t.prototype),r=t.apply(n,e);return gs(r)?r:n}}function Wi(t){return function(e,n,r){var i=qt(e);if(!is(e)){var a=Eo(n,3);e=Ys(e),n=function(t){return a(i[t],t,i)}}var s=t(e,n,r);return s>-1?i[a?e[s]:s]:o}}function Xi(t){return fo(function(e){var n=e.length,r=n,i=sr.prototype.thru;for(t&&e.reverse();r--;){var a=e[r];if("function"!=typeof a)throw new Xt(s);if(i&&!A&&"wrapper"==mo(a))var A=new sr([],!0)}for(r=A?r:n;++r<n;){var l=mo(a=e[r]),c="wrapper"==l?Co(a):o;A=c&&Ro(c[0])&&c[1]==(v|C|y|B)&&!c[4].length&&1==c[9]?A[mo(c[0])].apply(A,c[3]):1==a.length&&Ro(a)?A[l]():A.thru(a)}return function(){var t=arguments,r=t[0];if(A&&1==t.length&&ns(r))return A.plant(r).value();for(var i=0,o=n?e[i].apply(this,t):r;++i<n;)o=e[i].call(this,o);return o}})}function Zi(t,e,n,i,a,s,A,l,c,u){var g=e&v,h=e&f,d=e&p,I=e&(C|m),y=e&b,E=d?o:Vi(t);return function f(){for(var p=arguments.length,C=r(p),m=p;m--;)C[m]=arguments[m];if(I)var v=yo(f),B=function(t,e){for(var n=t.length,r=0;n--;)t[n]===e&&++r;return r}(C,v);if(i&&(C=Oi(C,i,a,I)),s&&(C=Gi(C,s,A,I)),p-=B,I&&p<u){var b=Cn(C,v);return oo(t,e,Zi,f.placeholder,n,C,b,l,c,u-p)}var Q=h?n:this,w=d?Q[t]:t;return p=C.length,l?C=function(t,e){for(var n=t.length,r=Tn(e.length,n),i=Hi(t);r--;){var a=e[r];t[r]=ko(a,n)?i[a]:o}return t}(C,l):y&&p>1&&C.reverse(),g&&c<p&&(C.length=c),this&&this!==be&&this instanceof f&&(w=E||Vi(w)),w.apply(Q,C)}}function $i(t,e){return function(n,r){return function(t,e,n,r){return Fr(t,function(t,i,o){e(r,n(t),i,o)}),r}(n,t,e(r),{})}}function to(t,e){return function(n,r){var i;if(n===o&&r===o)return e;if(n!==o&&(i=n),r!==o){if(i===o)return r;"string"==typeof n||"string"==typeof r?(n=Ei(n),r=Ei(r)):(n=yi(n),r=yi(r)),i=t(n,r)}return i}}function eo(t){return fo(function(e){return e=Ye(e,sn(Eo())),ui(function(n){var r=this;return t(e,function(t){return Fe(t,r,n)})})})}function no(t,e){var n=(e=e===o?" ":Ei(e)).length;if(n<2)return n?ci(e,t):e;var r=ci(e,Dn(t/yn(e)));return fn(e)?Li(En(r),0,t).join(""):r.slice(0,t)}function ro(t){return function(e,n,i){return i&&"number"!=typeof i&&So(e,n,i)&&(n=i=o),e=Qs(e),n===o?(n=e,e=0):n=Qs(n),function(t,e,n,i){for(var o=-1,a=Fn(Dn((e-t)/(n||1)),0),s=r(a);a--;)s[i?a:++o]=t,t+=n;return s}(e,n,i=i===o?e<n?1:-1:Qs(i),t)}}function io(t){return function(e,n){return"string"==typeof e&&"string"==typeof n||(e=_s(e),n=_s(n)),t(e,n)}}function oo(t,e,n,r,i,a,s,A,l,c){var u=e&C;e|=u?y:E,(e&=~(u?E:y))&I||(e&=~(f|p));var g=[t,e,i,u?a:o,u?s:o,u?o:a,u?o:s,A,l,c],h=n.apply(o,g);return Ro(t)&&jo(h,g),h.placeholder=r,Ko(h,t,e)}function ao(t){var e=Jt[t];return function(t,n){if(t=_s(t),(n=null==n?0:Tn(ws(n),292))&&Ln(t)){var r=(Ms(t)+"e").split("e");return+((r=(Ms(e(r[0]+"e"+(+r[1]+n)))+"e").split("e"))[0]+"e"+(+r[1]-n))}return e(t)}}var so=Kn&&1/mn(new Kn([,-0]))[1]==x?function(t){return new Kn(t)}:yA;function Ao(t){return function(e){var n=xo(e);return n==P?pn(e):n==W?function(t){var e=-1,n=Array(t.size);return t.forEach(function(t){n[++e]=[t,t]}),n}(e):function(t,e){return Ye(e,function(e){return[e,t[e]]})}(e,t(e))}}function lo(t,e,n,i,a,A,c,u){var g=e&p;if(!g&&"function"!=typeof t)throw new Xt(s);var h=i?i.length:0;if(h||(e&=~(y|E),i=a=o),c=c===o?c:Fn(ws(c),0),u=u===o?u:ws(u),h-=a?a.length:0,e&E){var d=i,b=a;i=a=o}var Q=g?o:Co(t),w=[t,e,n,i,a,d,b,A,c,u];if(Q&&function(t,e){var n=t[1],r=e[1],i=n|r,o=i<(f|p|v),a=r==v&&n==C||r==v&&n==B&&t[7].length<=e[8]||r==(v|B)&&e[7].length<=e[8]&&n==C;if(!o&&!a)return t;r&f&&(t[2]=e[2],i|=n&f?0:I);var s=e[3];if(s){var A=t[3];t[3]=A?Oi(A,s,e[4]):s,t[4]=A?Cn(t[3],l):e[4]}(s=e[5])&&(A=t[5],t[5]=A?Gi(A,s,e[6]):s,t[6]=A?Cn(t[5],l):e[6]),(s=e[7])&&(t[7]=s),r&v&&(t[8]=null==t[8]?e[8]:Tn(t[8],e[8])),null==t[9]&&(t[9]=e[9]),t[0]=e[0],t[1]=i}(w,Q),t=w[0],e=w[1],n=w[2],i=w[3],a=w[4],!(u=w[9]=w[9]===o?g?0:t.length:Fn(w[9]-h,0))&&e&(C|m)&&(e&=~(C|m)),e&&e!=f)x=e==C||e==m?function(t,e,n){var i=Vi(t);return function a(){for(var s=arguments.length,A=r(s),l=s,c=yo(a);l--;)A[l]=arguments[l];var u=s<3&&A[0]!==c&&A[s-1]!==c?[]:Cn(A,c);return(s-=u.length)<n?oo(t,e,Zi,a.placeholder,o,A,u,o,o,n-s):Fe(this&&this!==be&&this instanceof a?i:t,this,A)}}(t,e,u):e!=y&&e!=(f|y)||a.length?Zi.apply(o,w):function(t,e,n,i){var o=e&f,a=Vi(t);return function e(){for(var s=-1,A=arguments.length,l=-1,c=i.length,u=r(c+A),g=this&&this!==be&&this instanceof e?a:t;++l<c;)u[l]=i[l];for(;A--;)u[l++]=arguments[++s];return Fe(g,o?n:this,u)}}(t,e,n,i);else var x=function(t,e,n){var r=e&f,i=Vi(t);return function e(){return(this&&this!==be&&this instanceof e?i:t).apply(r?n:this,arguments)}}(t,e,n);return Ko((Q?hi:jo)(x,w),t,e)}function co(t,e,n,r){return t===o||Za(t,te[n])&&!re.call(r,n)?e:t}function uo(t,e,n,r,i,a){return gs(t)&&gs(e)&&(a.set(e,t),ri(t,e,o,uo,a),a.delete(e)),t}function go(t){return ps(t)?o:t}function ho(t,e,n,r,i,a){var s=n&h,A=t.length,l=e.length;if(A!=l&&!(s&&l>A))return!1;var c=a.get(t);if(c&&a.get(e))return c==e;var u=-1,g=!0,f=n&d?new gr:o;for(a.set(t,e),a.set(e,t);++u<A;){var p=t[u],I=e[u];if(r)var C=s?r(I,p,u,e,t,a):r(p,I,u,t,e,a);if(C!==o){if(C)continue;g=!1;break}if(f){if(!Je(e,function(t,e){if(!ln(f,e)&&(p===t||i(p,t,n,r,a)))return f.push(e)})){g=!1;break}}else if(p!==I&&!i(p,I,n,r,a)){g=!1;break}}return a.delete(t),a.delete(e),g}function fo(t){return Po(Oo(t,o,ra),t+"")}function po(t){return Gr(t,Ys,Qo)}function Io(t){return Gr(t,Ps,wo)}var Co=qn?function(t){return qn.get(t)}:yA;function mo(t){for(var e=t.name+"",n=Vn[e],r=re.call(Vn,e)?n.length:0;r--;){var i=n[r],o=i.func;if(null==o||o==t)return i.name}return e}function yo(t){return(re.call(ir,"placeholder")?ir:t).placeholder}function Eo(){var t=ir.iteratee||pA;return t=t===pA?Xr:t,arguments.length?t(arguments[0],arguments[1]):t}function vo(t,e){var n,r,i=t.__data__;return("string"==(r=typeof(n=e))||"number"==r||"symbol"==r||"boolean"==r?"__proto__"!==n:null===n)?i["string"==typeof e?"string":"hash"]:i.map}function Bo(t){for(var e=Ys(t),n=e.length;n--;){var r=e[n],i=t[r];e[n]=[r,i,To(i)]}return e}function bo(t,e){var n=function(t,e){return null==t?o:t[e]}(t,e);return Wr(n)?n:o}var Qo=kn?function(t){return null==t?[]:(t=qt(t),Ge(kn(t),function(e){return _e.call(t,e)}))}:xA,wo=kn?function(t){for(var e=[];t;)Pe(e,Qo(t)),t=Qe(t);return e}:xA,xo=Hr;function _o(t,e,n){for(var r=-1,i=(e=ki(e,t)).length,o=!1;++r<i;){var a=Vo(e[r]);if(!(o=null!=t&&n(t,a)))break;t=t[a]}return o||++r!=i?o:!!(i=null==t?0:t.length)&&us(i)&&ko(a,i)&&(ns(t)||es(t))}function Do(t){return"function"!=typeof t.constructor||Fo(t)?{}:or(Qe(t))}function Mo(t){return ns(t)||es(t)||!!(qe&&t&&t[qe])}function ko(t,e){var n=typeof t;return!!(e=null==e?_:e)&&("number"==n||"symbol"!=n&&Ht.test(t))&&t>-1&&t%1==0&&t<e}function So(t,e,n){if(!gs(n))return!1;var r=typeof e;return!!("number"==r?is(n)&&ko(e,n.length):"string"==r&&e in n)&&Za(n[e],t)}function Lo(t,e){if(ns(t))return!1;var n=typeof t;return!("number"!=n&&"symbol"!=n&&"boolean"!=n&&null!=t&&!ys(t))||Bt.test(t)||!vt.test(t)||null!=e&&t in qt(e)}function Ro(t){var e=mo(t),n=ir[e];if("function"!=typeof n||!(e in Ar.prototype))return!1;if(t===n)return!0;var r=Co(n);return!!r&&t===r[0]}(jn&&xo(new jn(new ArrayBuffer(1)))!=nt||Yn&&xo(new Yn)!=P||Pn&&"[object Promise]"!=xo(Pn.resolve())||Kn&&xo(new Kn)!=W||zn&&xo(new zn)!=tt)&&(xo=function(t){var e=Hr(t),n=e==J?t.constructor:o,r=n?Wo(n):"";if(r)switch(r){case Wn:return nt;case Xn:return P;case Zn:return"[object Promise]";case $n:return W;case tr:return tt}return e});var No=ee?ls:_A;function Fo(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||te)}function To(t){return t==t&&!gs(t)}function Uo(t,e){return function(n){return null!=n&&n[t]===e&&(e!==o||t in qt(n))}}function Oo(t,e,n){return e=Fn(e===o?t.length-1:e,0),function(){for(var i=arguments,o=-1,a=Fn(i.length-e,0),s=r(a);++o<a;)s[o]=i[e+o];o=-1;for(var A=r(e+1);++o<e;)A[o]=i[o];return A[e]=n(s),Fe(t,this,A)}}function Go(t,e){return e.length<2?t:Or(t,fi(e,0,-1))}function Ho(t,e){if(("constructor"!==e||"function"!=typeof t[e])&&"__proto__"!=e)return t[e]}var jo=zo(hi),Yo=_n||function(t,e){return be.setTimeout(t,e)},Po=zo(di);function Ko(t,e,n){var r=e+"";return Po(t,function(t,e){var n=e.length;if(!n)return t;var r=n-1;return e[r]=(n>1?"& ":"")+e[r],e=e.join(n>2?", ":" "),t.replace(Mt,"{\n/* [wrapped with "+e+"] */\n")}(r,function(t,e){return Ue(R,function(n){var r="_."+n[0];e&n[1]&&!He(t,r)&&t.push(r)}),t.sort()}(function(t){var e=r.match(kt);return e?e[1].split(St):[]}(),n)))}function zo(t){var e=0,n=0;return function(){var r=Un(),i=w-(r-n);if(n=r,i>0){if(++e>=Q)return arguments[0]}else e=0;return t.apply(o,arguments)}}function Jo(t,e){var n=-1,r=t.length,i=r-1;for(e=e===o?r:e;++n<e;){var a=li(n,i),s=t[a];t[a]=t[n],t[n]=s}return t.length=e,t}var qo=function(t){var e=za(function(t){var e=[];return 46===t.charCodeAt(0)&&e.push(""),t.replace(bt,function(t,n,r,i){e.push(r?i.replace(Rt,"$1"):n||t)}),e},function(t){return 500===n.size&&n.clear(),t}),n=e.cache;return e}();function Vo(t){if("string"==typeof t||ys(t))return t;var e=t+"";return"0"==e&&1/t==-x?"-0":e}function Wo(t){if(null!=t){try{return ne.call(t)}catch(t){}try{return t+""}catch(t){}}return""}function Xo(t){if(t instanceof Ar)return t.clone();var e=new sr(t.__wrapped__,t.__chain__);return e.__actions__=Hi(t.__actions__),e.__index__=t.__index__,e.__values__=t.__values__,e}var Zo=ui(function(t,e){return os(t)?xr(t,Lr(e,1,os,!0)):[]}),$o=ui(function(t,e){var n=Aa(e);return os(n)&&(n=o),os(t)?xr(t,Lr(e,1,os,!0),Eo(n,2)):[]}),ta=ui(function(t,e){var n=Aa(e);return os(n)&&(n=o),os(t)?xr(t,Lr(e,1,os,!0),o,n):[]});function ea(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=null==n?0:ws(n);return i<0&&(i=Fn(r+i,0)),We(t,Eo(e,3),i)}function na(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=r-1;return n!==o&&(i=ws(n),i=n<0?Fn(r+i,0):Tn(i,r-1)),We(t,Eo(e,3),i,!0)}function ra(t){return null!=t&&t.length?Lr(t,1):[]}function ia(t){return t&&t.length?t[0]:o}var oa=ui(function(t){var e=Ye(t,Di);return e.length&&e[0]===t[0]?Kr(e):[]}),aa=ui(function(t){var e=Aa(t),n=Ye(t,Di);return e===Aa(n)?e=o:n.pop(),n.length&&n[0]===t[0]?Kr(n,Eo(e,2)):[]}),sa=ui(function(t){var e=Aa(t),n=Ye(t,Di);return(e="function"==typeof e?e:o)&&n.pop(),n.length&&n[0]===t[0]?Kr(n,o,e):[]});function Aa(t){var e=null==t?0:t.length;return e?t[e-1]:o}var la=ui(ca);function ca(t,e){return t&&t.length&&e&&e.length?si(t,e):t}var ua=fo(function(t,e){var n=null==t?0:t.length,r=vr(t,e);return Ai(t,Ye(e,function(t){return ko(t,n)?+t:t}).sort(Ui)),r});function ga(t){return null==t?t:Hn.call(t)}var ha=ui(function(t){return vi(Lr(t,1,os,!0))}),da=ui(function(t){var e=Aa(t);return os(e)&&(e=o),vi(Lr(t,1,os,!0),Eo(e,2))}),fa=ui(function(t){var e=Aa(t);return e="function"==typeof e?e:o,vi(Lr(t,1,os,!0),o,e)});function pa(t){if(!t||!t.length)return[];var e=0;return t=Ge(t,function(t){if(os(t))return e=Fn(t.length,e),!0}),an(e,function(e){return Ye(t,en(e))})}function Ia(t,e){if(!t||!t.length)return[];var n=pa(t);return null==e?n:Ye(n,function(t){return Fe(e,o,t)})}var Ca=ui(function(t,e){return os(t)?xr(t,e):[]}),ma=ui(function(t){return xi(Ge(t,os))}),ya=ui(function(t){var e=Aa(t);return os(e)&&(e=o),xi(Ge(t,os),Eo(e,2))}),Ea=ui(function(t){var e=Aa(t);return e="function"==typeof e?e:o,xi(Ge(t,os),o,e)}),va=ui(pa),Ba=ui(function(t){var e=t.length,n=e>1?t[e-1]:o;return n="function"==typeof n?(t.pop(),n):o,Ia(t,n)});function ba(t){var e=ir(t);return e.__chain__=!0,e}function Qa(t,e){return e(t)}var wa=fo(function(t){var e=t.length,n=e?t[0]:0,r=this.__wrapped__,i=function(e){return vr(e,t)};return!(e>1||this.__actions__.length)&&r instanceof Ar&&ko(n)?((r=r.slice(n,+n+(e?1:0))).__actions__.push({func:Qa,args:[i],thisArg:o}),new sr(r,this.__chain__).thru(function(t){return e&&!t.length&&t.push(o),t})):this.thru(i)}),xa=Yi(function(t,e,n){re.call(t,n)?++t[n]:Er(t,n,1)}),_a=Wi(ea),Da=Wi(na);function Ma(t,e){return(ns(t)?Ue:_r)(t,Eo(e,3))}function ka(t,e){return(ns(t)?function(t,e){for(var n=null==t?0:t.length;n--&&!1!==e(t[n],n,t););return t}:Dr)(t,Eo(e,3))}var Sa=Yi(function(t,e,n){re.call(t,n)?t[n].push(e):Er(t,n,[e])}),La=ui(function(t,e,n){var i=-1,o="function"==typeof e,a=is(t)?r(t.length):[];return _r(t,function(t){a[++i]=o?Fe(e,t,n):zr(t,e,n)}),a}),Ra=Yi(function(t,e,n){Er(t,n,e)});function Na(t,e){return(ns(t)?Ye:ti)(t,Eo(e,3))}var Fa=Yi(function(t,e,n){t[n?0:1].push(e)},function(){return[[],[]]}),Ta=ui(function(t,e){if(null==t)return[];var n=e.length;return n>1&&So(t,e[0],e[1])?e=[]:n>2&&So(e[0],e[1],e[2])&&(e=[e[0]]),oi(t,Lr(e,1),[])}),Ua=xn||function(){return be.Date.now()};function Oa(t,e,n){return e=n?o:e,e=t&&null==e?t.length:e,lo(t,v,o,o,o,o,e)}function Ga(t,e){var n;if("function"!=typeof e)throw new Xt(s);return t=ws(t),function(){return--t>0&&(n=e.apply(this,arguments)),t<=1&&(e=o),n}}var Ha=ui(function(t,e,n){var r=f;if(n.length){var i=Cn(n,yo(Ha));r|=y}return lo(t,r,e,n,i)}),ja=ui(function(t,e,n){var r=f|p;if(n.length){var i=Cn(n,yo(ja));r|=y}return lo(e,r,t,n,i)});function Ya(t,e,n){var r,i,a,A,l,c,u=0,g=!1,h=!1,d=!0;if("function"!=typeof t)throw new Xt(s);function f(e){var n=r,a=i;return r=i=o,u=e,A=t.apply(a,n)}function p(t){var n=t-c;return c===o||n>=e||n<0||h&&t-u>=a}function I(){var t=Ua();if(p(t))return C(t);l=Yo(I,function(t){var n=e-(t-c);return h?Tn(n,a-(t-u)):n}(t))}function C(t){return l=o,d&&r?f(t):(r=i=o,A)}function m(){var t=Ua(),n=p(t);if(r=arguments,i=this,c=t,n){if(l===o)return function(t){return u=t,l=Yo(I,e),g?f(t):A}(c);if(h)return Ri(l),l=Yo(I,e),f(c)}return l===o&&(l=Yo(I,e)),A}return e=_s(e)||0,gs(n)&&(g=!!n.leading,a=(h="maxWait"in n)?Fn(_s(n.maxWait)||0,e):a,d="trailing"in n?!!n.trailing:d),m.cancel=function(){l!==o&&Ri(l),u=0,r=c=i=l=o},m.flush=function(){return l===o?A:C(Ua())},m}var Pa=ui(function(t,e){return wr(t,1,e)}),Ka=ui(function(t,e,n){return wr(t,_s(e)||0,n)});function za(t,e){if("function"!=typeof t||null!=e&&"function"!=typeof e)throw new Xt(s);var n=function(){var r=arguments,i=e?e.apply(this,r):r[0],o=n.cache;if(o.has(i))return o.get(i);var a=t.apply(this,r);return n.cache=o.set(i,a)||o,a};return n.cache=new(za.Cache||ur),n}function Ja(t){if("function"!=typeof t)throw new Xt(s);return function(){var e=arguments;switch(e.length){case 0:return!t.call(this);case 1:return!t.call(this,e[0]);case 2:return!t.call(this,e[0],e[1]);case 3:return!t.call(this,e[0],e[1],e[2])}return!t.apply(this,e)}}za.Cache=ur;var qa=Si(function(t,e){var n=(e=1==e.length&&ns(e[0])?Ye(e[0],sn(Eo())):Ye(Lr(e,1),sn(Eo()))).length;return ui(function(r){for(var i=-1,o=Tn(r.length,n);++i<o;)r[i]=e[i].call(this,r[i]);return Fe(t,this,r)})}),Va=ui(function(t,e){var n=Cn(e,yo(Va));return lo(t,y,o,e,n)}),Wa=ui(function(t,e){var n=Cn(e,yo(Wa));return lo(t,E,o,e,n)}),Xa=fo(function(t,e){return lo(t,B,o,o,o,e)});function Za(t,e){return t===e||t!=t&&e!=e}var $a=io(jr),ts=io(function(t,e){return t>=e}),es=Jr(function(){return arguments}())?Jr:function(t){return hs(t)&&re.call(t,"callee")&&!_e.call(t,"callee")},ns=r.isArray,rs=Me?sn(Me):function(t){return hs(t)&&Hr(t)==et};function is(t){return null!=t&&us(t.length)&&!ls(t)}function os(t){return hs(t)&&is(t)}var as=Sn||_A,ss=ke?sn(ke):function(t){return hs(t)&&Hr(t)==O};function As(t){if(!hs(t))return!1;var e=Hr(t);return e==H||e==G||"string"==typeof t.message&&"string"==typeof t.name&&!ps(t)}function ls(t){if(!gs(t))return!1;var e=Hr(t);return e==j||e==Y||e==T||e==q}function cs(t){return"number"==typeof t&&t==ws(t)}function us(t){return"number"==typeof t&&t>-1&&t%1==0&&t<=_}function gs(t){var e=typeof t;return null!=t&&("object"==e||"function"==e)}function hs(t){return null!=t&&"object"==typeof t}var ds=Se?sn(Se):function(t){return hs(t)&&xo(t)==P};function fs(t){return"number"==typeof t||hs(t)&&Hr(t)==K}function ps(t){if(!hs(t)||Hr(t)!=J)return!1;var e=Qe(t);if(null===e)return!0;var n=re.call(e,"constructor")&&e.constructor;return"function"==typeof n&&n instanceof n&&ne.call(n)==se}var Is=Le?sn(Le):function(t){return hs(t)&&Hr(t)==V},Cs=Re?sn(Re):function(t){return hs(t)&&xo(t)==W};function ms(t){return"string"==typeof t||!ns(t)&&hs(t)&&Hr(t)==X}function ys(t){return"symbol"==typeof t||hs(t)&&Hr(t)==Z}var Es=Ne?sn(Ne):function(t){return hs(t)&&us(t.length)&&!!Ie[Hr(t)]},vs=io($r),Bs=io(function(t,e){return t<=e});function bs(t){if(!t)return[];if(is(t))return ms(t)?En(t):Hi(t);if(nn&&t[nn])return function(t){for(var e,n=[];!(e=t.next()).done;)n.push(e.value);return n}(t[nn]());var e=xo(t);return(e==P?pn:e==W?mn:Zs)(t)}function Qs(t){return t?(t=_s(t))===x||t===-x?(t<0?-1:1)*D:t==t?t:0:0===t?t:0}function ws(t){var e=Qs(t),n=e%1;return e==e?n?e-n:e:0}function xs(t){return t?Br(ws(t),0,k):0}function _s(t){if("number"==typeof t)return t;if(ys(t))return M;if(gs(t)){var e="function"==typeof t.valueOf?t.valueOf():t;t=gs(e)?e+"":e}if("string"!=typeof t)return 0===t?t:+t;t=t.replace(xt,"");var n=Ut.test(t);return n||Gt.test(t)?Ee(t.slice(2),n?2:8):Tt.test(t)?M:+t}function Ds(t){return ji(t,Ps(t))}function Ms(t){return null==t?"":Ei(t)}var ks=Pi(function(t,e){if(Fo(e)||is(e))ji(e,Ys(e),t);else for(var n in e)re.call(e,n)&&Ir(t,n,e[n])}),Ss=Pi(function(t,e){ji(e,Ps(e),t)}),Ls=Pi(function(t,e,n,r){ji(e,Ps(e),t,r)}),Rs=Pi(function(t,e,n,r){ji(e,Ys(e),t,r)}),Ns=fo(vr),Fs=ui(function(t,e){t=qt(t);var n=-1,r=e.length,i=r>2?e[2]:o;for(i&&So(e[0],e[1],i)&&(r=1);++n<r;)for(var a=e[n],s=Ps(a),A=-1,l=s.length;++A<l;){var c=s[A],u=t[c];(u===o||Za(u,te[c])&&!re.call(t,c))&&(t[c]=a[c])}return t}),Ts=ui(function(t){return t.push(o,uo),Fe(zs,o,t)});function Us(t,e,n){var r=null==t?o:Or(t,e);return r===o?n:r}function Os(t,e){return null!=t&&_o(t,e,Pr)}var Gs=$i(function(t,e,n){null!=e&&"function"!=typeof e.toString&&(e=ae.call(e)),t[e]=n},gA(fA)),Hs=$i(function(t,e,n){null!=e&&"function"!=typeof e.toString&&(e=ae.call(e)),re.call(t,e)?t[e].push(n):t[e]=[n]},Eo),js=ui(zr);function Ys(t){return is(t)?dr(t):Zr(t)}function Ps(t){return is(t)?dr(t,!0):function(t){if(!gs(t))return function(t){var e=[];if(null!=t)for(var n in qt(t))e.push(n);return e}(t);var e=Fo(t),n=[];for(var r in t)("constructor"!=r||!e&&re.call(t,r))&&n.push(r);return n}(t)}var Ks=Pi(function(t,e,n){ri(t,e,n)}),zs=Pi(function(t,e,n,r){ri(t,e,n,r)}),Js=fo(function(t,e){var n={};if(null==t)return n;var r=!1;e=Ye(e,function(e){return e=ki(e,t),r||(r=e.length>1),e}),ji(t,Io(t),n),r&&(n=br(n,c|u|g,go));for(var i=e.length;i--;)Bi(n,e[i]);return n}),qs=fo(function(t,e){return null==t?{}:function(t,e){return ai(t,e,function(e,n){return Os(t,n)})}(t,e)});function Vs(t,e){if(null==t)return{};var n=Ye(Io(t),function(t){return[t]});return e=Eo(e),ai(t,n,function(t,n){return e(t,n[0])})}var Ws=Ao(Ys),Xs=Ao(Ps);function Zs(t){return null==t?[]:An(t,Ys(t))}var $s=qi(function(t,e,n){return e=e.toLowerCase(),t+(n?tA(e):e)});function tA(t){return AA(Ms(t).toLowerCase())}function eA(t){return(t=Ms(t))&&t.replace(jt,gn).replace(ce,"")}var nA=qi(function(t,e,n){return t+(n?"-":"")+e.toLowerCase()}),rA=qi(function(t,e,n){return t+(n?" ":"")+e.toLowerCase()}),iA=Ji("toLowerCase"),oA=qi(function(t,e,n){return t+(n?"_":"")+e.toLowerCase()}),aA=qi(function(t,e,n){return t+(n?" ":"")+AA(e)}),sA=qi(function(t,e,n){return t+(n?" ":"")+e.toUpperCase()}),AA=Ji("toUpperCase");function lA(t,e,n){return t=Ms(t),(e=n?o:e)===o?function(t){return de.test(t)}(t)?function(t){return t.match(ge)||[]}(t):function(t){return t.match(Lt)||[]}(t):t.match(e)||[]}var cA=ui(function(t,e){try{return Fe(t,o,e)}catch(t){return As(t)?t:new Kt(t)}}),uA=fo(function(t,e){return Ue(e,function(e){e=Vo(e),Er(t,e,Ha(t[e],t))}),t});function gA(t){return function(){return t}}var hA=Xi(),dA=Xi(!0);function fA(t){return t}function pA(t){return Xr("function"==typeof t?t:br(t,c))}var IA=ui(function(t,e){return function(n){return zr(n,t,e)}}),CA=ui(function(t,e){return function(n){return zr(t,n,e)}});function mA(t,e,n){var r=Ys(e),i=Ur(e,r);null!=n||gs(e)&&(i.length||!r.length)||(n=e,e=t,t=this,i=Ur(e,Ys(e)));var o=!(gs(n)&&"chain"in n&&!n.chain),a=ls(t);return Ue(i,function(n){var r=e[n];t[n]=r,a&&(t.prototype[n]=function(){var e=this.__chain__;if(o||e){var n=t(this.__wrapped__),i=n.__actions__=Hi(this.__actions__);return i.push({func:r,args:arguments,thisArg:t}),n.__chain__=e,n}return r.apply(t,Pe([this.value()],arguments))})}),t}function yA(){}var EA=eo(Ye),vA=eo(Oe),BA=eo(Je);function bA(t){return Lo(t)?en(Vo(t)):function(t){return function(e){return Or(e,t)}}(t)}var QA=ro(),wA=ro(!0);function xA(){return[]}function _A(){return!1}var DA,MA=to(function(t,e){return t+e},0),kA=ao("ceil"),SA=to(function(t,e){return t/e},1),LA=ao("floor"),RA=to(function(t,e){return t*e},1),NA=ao("round"),FA=to(function(t,e){return t-e},0);return ir.after=function(t,e){if("function"!=typeof e)throw new Xt(s);return t=ws(t),function(){if(--t<1)return e.apply(this,arguments)}},ir.ary=Oa,ir.assign=ks,ir.assignIn=Ss,ir.assignInWith=Ls,ir.assignWith=Rs,ir.at=Ns,ir.before=Ga,ir.bind=Ha,ir.bindAll=uA,ir.bindKey=ja,ir.castArray=function(){if(!arguments.length)return[];var t=arguments[0];return ns(t)?t:[t]},ir.chain=ba,ir.chunk=function(t,e,n){e=(n?So(t,e,n):e===o)?1:Fn(ws(e),0);var i=null==t?0:t.length;if(!i||e<1)return[];for(var a=0,s=0,A=r(Dn(i/e));a<i;)A[s++]=fi(t,a,a+=e);return A},ir.compact=function(t){for(var e=-1,n=null==t?0:t.length,r=0,i=[];++e<n;){var o=t[e];o&&(i[r++]=o)}return i},ir.concat=function(){var t=arguments.length;if(!t)return[];for(var e=r(t-1),n=arguments[0],i=t;i--;)e[i-1]=arguments[i];return Pe(ns(n)?Hi(n):[n],Lr(e,1))},ir.cond=function(t){var e=null==t?0:t.length,n=Eo();return t=e?Ye(t,function(t){if("function"!=typeof t[1])throw new Xt(s);return[n(t[0]),t[1]]}):[],ui(function(n){for(var r=-1;++r<e;){var i=t[r];if(Fe(i[0],this,n))return Fe(i[1],this,n)}})},ir.conforms=function(t){return function(t){var e=Ys(t);return function(n){return Qr(n,t,e)}}(br(t,c))},ir.constant=gA,ir.countBy=xa,ir.create=function(t,e){var n=or(t);return null==e?n:yr(n,e)},ir.curry=function t(e,n,r){var i=lo(e,C,o,o,o,o,o,n=r?o:n);return i.placeholder=t.placeholder,i},ir.curryRight=function t(e,n,r){var i=lo(e,m,o,o,o,o,o,n=r?o:n);return i.placeholder=t.placeholder,i},ir.debounce=Ya,ir.defaults=Fs,ir.defaultsDeep=Ts,ir.defer=Pa,ir.delay=Ka,ir.difference=Zo,ir.differenceBy=$o,ir.differenceWith=ta,ir.drop=function(t,e,n){var r=null==t?0:t.length;return r?fi(t,(e=n||e===o?1:ws(e))<0?0:e,r):[]},ir.dropRight=function(t,e,n){var r=null==t?0:t.length;return r?fi(t,0,(e=r-(e=n||e===o?1:ws(e)))<0?0:e):[]},ir.dropRightWhile=function(t,e){return t&&t.length?Qi(t,Eo(e,3),!0,!0):[]},ir.dropWhile=function(t,e){return t&&t.length?Qi(t,Eo(e,3),!0):[]},ir.fill=function(t,e,n,r){var i=null==t?0:t.length;return i?(n&&"number"!=typeof n&&So(t,e,n)&&(n=0,r=i),function(t,e,n,r){var i=t.length;for((n=ws(n))<0&&(n=-n>i?0:i+n),(r=r===o||r>i?i:ws(r))<0&&(r+=i),r=n>r?0:xs(r);n<r;)t[n++]=e;return t}(t,e,n,r)):[]},ir.filter=function(t,e){return(ns(t)?Ge:Sr)(t,Eo(e,3))},ir.flatMap=function(t,e){return Lr(Na(t,e),1)},ir.flatMapDeep=function(t,e){return Lr(Na(t,e),x)},ir.flatMapDepth=function(t,e,n){return n=n===o?1:ws(n),Lr(Na(t,e),n)},ir.flatten=ra,ir.flattenDeep=function(t){return null!=t&&t.length?Lr(t,x):[]},ir.flattenDepth=function(t,e){return null!=t&&t.length?Lr(t,e=e===o?1:ws(e)):[]},ir.flip=function(t){return lo(t,b)},ir.flow=hA,ir.flowRight=dA,ir.fromPairs=function(t){for(var e=-1,n=null==t?0:t.length,r={};++e<n;){var i=t[e];r[i[0]]=i[1]}return r},ir.functions=function(t){return null==t?[]:Ur(t,Ys(t))},ir.functionsIn=function(t){return null==t?[]:Ur(t,Ps(t))},ir.groupBy=Sa,ir.initial=function(t){return null!=t&&t.length?fi(t,0,-1):[]},ir.intersection=oa,ir.intersectionBy=aa,ir.intersectionWith=sa,ir.invert=Gs,ir.invertBy=Hs,ir.invokeMap=La,ir.iteratee=pA,ir.keyBy=Ra,ir.keys=Ys,ir.keysIn=Ps,ir.map=Na,ir.mapKeys=function(t,e){var n={};return e=Eo(e,3),Fr(t,function(t,r,i){Er(n,e(t,r,i),t)}),n},ir.mapValues=function(t,e){var n={};return e=Eo(e,3),Fr(t,function(t,r,i){Er(n,r,e(t,r,i))}),n},ir.matches=function(t){return ei(br(t,c))},ir.matchesProperty=function(t,e){return ni(t,br(e,c))},ir.memoize=za,ir.merge=Ks,ir.mergeWith=zs,ir.method=IA,ir.methodOf=CA,ir.mixin=mA,ir.negate=Ja,ir.nthArg=function(t){return t=ws(t),ui(function(e){return ii(e,t)})},ir.omit=Js,ir.omitBy=function(t,e){return Vs(t,Ja(Eo(e)))},ir.once=function(t){return Ga(2,t)},ir.orderBy=function(t,e,n,r){return null==t?[]:(ns(e)||(e=null==e?[]:[e]),ns(n=r?o:n)||(n=null==n?[]:[n]),oi(t,e,n))},ir.over=EA,ir.overArgs=qa,ir.overEvery=vA,ir.overSome=BA,ir.partial=Va,ir.partialRight=Wa,ir.partition=Fa,ir.pick=qs,ir.pickBy=Vs,ir.property=bA,ir.propertyOf=function(t){return function(e){return null==t?o:Or(t,e)}},ir.pull=la,ir.pullAll=ca,ir.pullAllBy=function(t,e,n){return t&&t.length&&e&&e.length?si(t,e,Eo(n,2)):t},ir.pullAllWith=function(t,e,n){return t&&t.length&&e&&e.length?si(t,e,o,n):t},ir.pullAt=ua,ir.range=QA,ir.rangeRight=wA,ir.rearg=Xa,ir.reject=function(t,e){return(ns(t)?Ge:Sr)(t,Ja(Eo(e,3)))},ir.remove=function(t,e){var n=[];if(!t||!t.length)return n;var r=-1,i=[],o=t.length;for(e=Eo(e,3);++r<o;){var a=t[r];e(a,r,t)&&(n.push(a),i.push(r))}return Ai(t,i),n},ir.rest=function(t,e){if("function"!=typeof t)throw new Xt(s);return ui(t,e=e===o?e:ws(e))},ir.reverse=ga,ir.sampleSize=function(t,e,n){return e=(n?So(t,e,n):e===o)?1:ws(e),(ns(t)?function(t,e){return Jo(Hi(t),Br(e,0,t.length))}:function(t,e){var n=Zs(t);return Jo(n,Br(e,0,n.length))})(t,e)},ir.set=function(t,e,n){return null==t?t:gi(t,e,n)},ir.setWith=function(t,e,n,r){return r="function"==typeof r?r:o,null==t?t:gi(t,e,n,r)},ir.shuffle=function(t){return(ns(t)?function(t){return Jo(Hi(t))}:function(t){return Jo(Zs(t))})(t)},ir.slice=function(t,e,n){var r=null==t?0:t.length;return r?(n&&"number"!=typeof n&&So(t,e,n)?(e=0,n=r):(e=null==e?0:ws(e),n=n===o?r:ws(n)),fi(t,e,n)):[]},ir.sortBy=Ta,ir.sortedUniq=function(t){return t&&t.length?mi(t):[]},ir.sortedUniqBy=function(t,e){return t&&t.length?mi(t,Eo(e,2)):[]},ir.split=function(t,e,n){return n&&"number"!=typeof n&&So(t,e,n)&&(e=n=o),(n=n===o?k:n>>>0)?(t=Ms(t))&&("string"==typeof e||null!=e&&!Is(e))&&!(e=Ei(e))&&fn(t)?Li(En(t),0,n):t.split(e,n):[]},ir.spread=function(t,e){if("function"!=typeof t)throw new Xt(s);return e=null==e?0:Fn(ws(e),0),ui(function(n){var r=n[e],i=Li(n,0,e);return r&&Pe(i,r),Fe(t,this,i)})},ir.tail=function(t){var e=null==t?0:t.length;return e?fi(t,1,e):[]},ir.take=function(t,e,n){return t&&t.length?fi(t,0,(e=n||e===o?1:ws(e))<0?0:e):[]},ir.takeRight=function(t,e,n){var r=null==t?0:t.length;return r?fi(t,(e=r-(e=n||e===o?1:ws(e)))<0?0:e,r):[]},ir.takeRightWhile=function(t,e){return t&&t.length?Qi(t,Eo(e,3),!1,!0):[]},ir.takeWhile=function(t,e){return t&&t.length?Qi(t,Eo(e,3)):[]},ir.tap=function(t,e){return e(t),t},ir.throttle=function(t,e,n){var r=!0,i=!0;if("function"!=typeof t)throw new Xt(s);return gs(n)&&(r="leading"in n?!!n.leading:r,i="trailing"in n?!!n.trailing:i),Ya(t,e,{leading:r,maxWait:e,trailing:i})},ir.thru=Qa,ir.toArray=bs,ir.toPairs=Ws,ir.toPairsIn=Xs,ir.toPath=function(t){return ns(t)?Ye(t,Vo):ys(t)?[t]:Hi(qo(Ms(t)))},ir.toPlainObject=Ds,ir.transform=function(t,e,n){var r=ns(t),i=r||as(t)||Es(t);if(e=Eo(e,4),null==n){var o=t&&t.constructor;n=i?r?new o:[]:gs(t)&&ls(o)?or(Qe(t)):{}}return(i?Ue:Fr)(t,function(t,r,i){return e(n,t,r,i)}),n},ir.unary=function(t){return Oa(t,1)},ir.union=ha,ir.unionBy=da,ir.unionWith=fa,ir.uniq=function(t){return t&&t.length?vi(t):[]},ir.uniqBy=function(t,e){return t&&t.length?vi(t,Eo(e,2)):[]},ir.uniqWith=function(t,e){return e="function"==typeof e?e:o,t&&t.length?vi(t,o,e):[]},ir.unset=function(t,e){return null==t||Bi(t,e)},ir.unzip=pa,ir.unzipWith=Ia,ir.update=function(t,e,n){return null==t?t:bi(t,e,Mi(n))},ir.updateWith=function(t,e,n,r){return r="function"==typeof r?r:o,null==t?t:bi(t,e,Mi(n),r)},ir.values=Zs,ir.valuesIn=function(t){return null==t?[]:An(t,Ps(t))},ir.without=Ca,ir.words=lA,ir.wrap=function(t,e){return Va(Mi(e),t)},ir.xor=ma,ir.xorBy=ya,ir.xorWith=Ea,ir.zip=va,ir.zipObject=function(t,e){return _i(t||[],e||[],Ir)},ir.zipObjectDeep=function(t,e){return _i(t||[],e||[],gi)},ir.zipWith=Ba,ir.entries=Ws,ir.entriesIn=Xs,ir.extend=Ss,ir.extendWith=Ls,mA(ir,ir),ir.add=MA,ir.attempt=cA,ir.camelCase=$s,ir.capitalize=tA,ir.ceil=kA,ir.clamp=function(t,e,n){return n===o&&(n=e,e=o),n!==o&&(n=(n=_s(n))==n?n:0),e!==o&&(e=(e=_s(e))==e?e:0),Br(_s(t),e,n)},ir.clone=function(t){return br(t,g)},ir.cloneDeep=function(t){return br(t,c|g)},ir.cloneDeepWith=function(t,e){return br(t,c|g,e="function"==typeof e?e:o)},ir.cloneWith=function(t,e){return br(t,g,e="function"==typeof e?e:o)},ir.conformsTo=function(t,e){return null==e||Qr(t,e,Ys(e))},ir.deburr=eA,ir.defaultTo=function(t,e){return null==t||t!=t?e:t},ir.divide=SA,ir.endsWith=function(t,e,n){t=Ms(t),e=Ei(e);var r=t.length,i=n=n===o?r:Br(ws(n),0,r);return(n-=e.length)>=0&&t.slice(n,i)==e},ir.eq=Za,ir.escape=function(t){return(t=Ms(t))&&Ct.test(t)?t.replace(pt,hn):t},ir.escapeRegExp=function(t){return(t=Ms(t))&&wt.test(t)?t.replace(Qt,"\\$&"):t},ir.every=function(t,e,n){var r=ns(t)?Oe:Mr;return n&&So(t,e,n)&&(e=o),r(t,Eo(e,3))},ir.find=_a,ir.findIndex=ea,ir.findKey=function(t,e){return Ve(t,Eo(e,3),Fr)},ir.findLast=Da,ir.findLastIndex=na,ir.findLastKey=function(t,e){return Ve(t,Eo(e,3),Tr)},ir.floor=LA,ir.forEach=Ma,ir.forEachRight=ka,ir.forIn=function(t,e){return null==t?t:Rr(t,Eo(e,3),Ps)},ir.forInRight=function(t,e){return null==t?t:Nr(t,Eo(e,3),Ps)},ir.forOwn=function(t,e){return t&&Fr(t,Eo(e,3))},ir.forOwnRight=function(t,e){return t&&Tr(t,Eo(e,3))},ir.get=Us,ir.gt=$a,ir.gte=ts,ir.has=function(t,e){return null!=t&&_o(t,e,Yr)},ir.hasIn=Os,ir.head=ia,ir.identity=fA,ir.includes=function(t,e,n,r){t=is(t)?t:Zs(t),n=n&&!r?ws(n):0;var i=t.length;return n<0&&(n=Fn(i+n,0)),ms(t)?n<=i&&t.indexOf(e,n)>-1:!!i&&Xe(t,e,n)>-1},ir.indexOf=function(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=null==n?0:ws(n);return i<0&&(i=Fn(r+i,0)),Xe(t,e,i)},ir.inRange=function(t,e,n){return e=Qs(e),n===o?(n=e,e=0):n=Qs(n),function(t,e,n){return t>=Tn(e,n)&&t<Fn(e,n)}(t=_s(t),e,n)},ir.invoke=js,ir.isArguments=es,ir.isArray=ns,ir.isArrayBuffer=rs,ir.isArrayLike=is,ir.isArrayLikeObject=os,ir.isBoolean=function(t){return!0===t||!1===t||hs(t)&&Hr(t)==U},ir.isBuffer=as,ir.isDate=ss,ir.isElement=function(t){return hs(t)&&1===t.nodeType&&!ps(t)},ir.isEmpty=function(t){if(null==t)return!0;if(is(t)&&(ns(t)||"string"==typeof t||"function"==typeof t.splice||as(t)||Es(t)||es(t)))return!t.length;var e=xo(t);if(e==P||e==W)return!t.size;if(Fo(t))return!Zr(t).length;for(var n in t)if(re.call(t,n))return!1;return!0},ir.isEqual=function(t,e){return qr(t,e)},ir.isEqualWith=function(t,e,n){var r=(n="function"==typeof n?n:o)?n(t,e):o;return r===o?qr(t,e,o,n):!!r},ir.isError=As,ir.isFinite=function(t){return"number"==typeof t&&Ln(t)},ir.isFunction=ls,ir.isInteger=cs,ir.isLength=us,ir.isMap=ds,ir.isMatch=function(t,e){return t===e||Vr(t,e,Bo(e))},ir.isMatchWith=function(t,e,n){return n="function"==typeof n?n:o,Vr(t,e,Bo(e),n)},ir.isNaN=function(t){return fs(t)&&t!=+t},ir.isNative=function(t){if(No(t))throw new Kt("Unsupported core-js use. Try https://npms.io/search?q=ponyfill.");return Wr(t)},ir.isNil=function(t){return null==t},ir.isNull=function(t){return null===t},ir.isNumber=fs,ir.isObject=gs,ir.isObjectLike=hs,ir.isPlainObject=ps,ir.isRegExp=Is,ir.isSafeInteger=function(t){return cs(t)&&t>=-_&&t<=_},ir.isSet=Cs,ir.isString=ms,ir.isSymbol=ys,ir.isTypedArray=Es,ir.isUndefined=function(t){return t===o},ir.isWeakMap=function(t){return hs(t)&&xo(t)==tt},ir.isWeakSet=function(t){return hs(t)&&"[object WeakSet]"==Hr(t)},ir.join=function(t,e){return null==t?"":Rn.call(t,e)},ir.kebabCase=nA,ir.last=Aa,ir.lastIndexOf=function(t,e,n){var r=null==t?0:t.length;if(!r)return-1;var i=r;return n!==o&&(i=(i=ws(n))<0?Fn(r+i,0):Tn(i,r-1)),e==e?function(t,e,n){for(var r=NaN;r--;)if(t[r]===e)return r;return r}(t,e):We(t,$e,i,!0)},ir.lowerCase=rA,ir.lowerFirst=iA,ir.lt=vs,ir.lte=Bs,ir.max=function(t){return t&&t.length?kr(t,fA,jr):o},ir.maxBy=function(t,e){return t&&t.length?kr(t,Eo(e,2),jr):o},ir.mean=function(t){return tn(t,fA)},ir.meanBy=function(t,e){return tn(t,Eo(e,2))},ir.min=function(t){return t&&t.length?kr(t,fA,$r):o},ir.minBy=function(t,e){return t&&t.length?kr(t,Eo(e,2),$r):o},ir.stubArray=xA,ir.stubFalse=_A,ir.stubObject=function(){return{}},ir.stubString=function(){return""},ir.stubTrue=function(){return!0},ir.multiply=RA,ir.nth=function(t,e){return t&&t.length?ii(t,ws(e)):o},ir.noConflict=function(){return be._===this&&(be._=Ae),this},ir.noop=yA,ir.now=Ua,ir.pad=function(t,e,n){t=Ms(t);var r=(e=ws(e))?yn(t):0;if(!e||r>=e)return t;var i=(e-r)/2;return no(Mn(i),n)+t+no(Dn(i),n)},ir.padEnd=function(t,e,n){t=Ms(t);var r=(e=ws(e))?yn(t):0;return e&&r<e?t+no(e-r,n):t},ir.padStart=function(t,e,n){t=Ms(t);var r=(e=ws(e))?yn(t):0;return e&&r<e?no(e-r,n)+t:t},ir.parseInt=function(t,e,n){return n||null==e?e=0:e&&(e=+e),On(Ms(t).replace(_t,""),e||0)},ir.random=function(t,e,n){if(n&&"boolean"!=typeof n&&So(t,e,n)&&(e=n=o),n===o&&("boolean"==typeof e?(n=e,e=o):"boolean"==typeof t&&(n=t,t=o)),t===o&&e===o?(t=0,e=1):(t=Qs(t),e===o?(e=t,t=0):e=Qs(e)),t>e){var r=t;t=e,e=r}if(n||t%1||e%1){var i=Gn();return Tn(t+i*(e-t+ye("1e-"+((i+"").length-1))),e)}return li(t,e)},ir.reduce=function(t,e,n){var r=ns(t)?Ke:rn,i=arguments.length<3;return r(t,Eo(e,4),n,i,_r)},ir.reduceRight=function(t,e,n){var r=ns(t)?ze:rn,i=arguments.length<3;return r(t,Eo(e,4),n,i,Dr)},ir.repeat=function(t,e,n){return e=(n?So(t,e,n):e===o)?1:ws(e),ci(Ms(t),e)},ir.replace=function(){var t=arguments,e=Ms(t[0]);return t.length<3?e:e.replace(t[1],t[2])},ir.result=function(t,e,n){var r=-1,i=(e=ki(e,t)).length;for(i||(i=1,t=o);++r<i;){var a=null==t?o:t[Vo(e[r])];a===o&&(r=i,a=n),t=ls(a)?a.call(t):a}return t},ir.round=NA,ir.runInContext=t,ir.sample=function(t){return(ns(t)?fr:function(t){return fr(Zs(t))})(t)},ir.size=function(t){if(null==t)return 0;if(is(t))return ms(t)?yn(t):t.length;var e=xo(t);return e==P||e==W?t.size:Zr(t).length},ir.snakeCase=oA,ir.some=function(t,e,n){var r=ns(t)?Je:pi;return n&&So(t,e,n)&&(e=o),r(t,Eo(e,3))},ir.sortedIndex=function(t,e){return Ii(t,e)},ir.sortedIndexBy=function(t,e,n){return Ci(t,e,Eo(n,2))},ir.sortedIndexOf=function(t,e){var n=null==t?0:t.length;if(n){var r=Ii(t,e);if(r<n&&Za(t[r],e))return r}return-1},ir.sortedLastIndex=function(t,e){return Ii(t,e,!0)},ir.sortedLastIndexBy=function(t,e,n){return Ci(t,e,Eo(n,2),!0)},ir.sortedLastIndexOf=function(t,e){if(null!=t&&t.length){var n=Ii(t,e,!0)-1;if(Za(t[n],e))return n}return-1},ir.startCase=aA,ir.startsWith=function(t,e,n){return t=Ms(t),n=null==n?0:Br(ws(n),0,t.length),e=Ei(e),t.slice(n,n+e.length)==e},ir.subtract=FA,ir.sum=function(t){return t&&t.length?on(t,fA):0},ir.sumBy=function(t,e){return t&&t.length?on(t,Eo(e,2)):0},ir.template=function(t,e,n){var r=ir.templateSettings;n&&So(t,e,n)&&(e=o),t=Ms(t),e=Ls({},e,r,co);var i,a,s=Ls({},e.imports,r.imports,co),A=Ys(s),l=An(s,A),c=0,u=e.interpolate||Yt,g="__p += '",h=Vt((e.escape||Yt).source+"|"+u.source+"|"+(u===Et?Nt:Yt).source+"|"+(e.evaluate||Yt).source+"|$","g"),d="//# sourceURL="+(re.call(e,"sourceURL")?(e.sourceURL+"").replace(/[\r\n]/g," "):"lodash.templateSources["+ ++pe+"]")+"\n";t.replace(h,function(e,n,r,o,s,A){return r||(r=o),g+=t.slice(c,A).replace(Pt,dn),n&&(i=!0,g+="' +\n__e("+n+") +\n'"),s&&(a=!0,g+="';\n"+s+";\n__p += '"),r&&(g+="' +\n((__t = ("+r+")) == null ? '' : __t) +\n'"),c=A+e.length,e}),g+="';\n";var f=re.call(e,"variable")&&e.variable;f||(g="with (obj) {\n"+g+"\n}\n"),g=(a?g.replace(gt,""):g).replace(ht,"$1").replace(dt,"$1;"),g="function("+(f||"obj")+") {\n"+(f?"":"obj || (obj = {});\n")+"var __t, __p = ''"+(i?", __e = _.escape":"")+(a?", __j = Array.prototype.join;\nfunction print() { __p += __j.call(arguments, '') }\n":";\n")+g+"return __p\n}";var p=cA(function(){return zt(A,d+"return "+g).apply(o,l)});if(p.source=g,As(p))throw p;return p},ir.times=function(t,e){if((t=ws(t))<1||t>_)return[];var n=k,r=Tn(t,k);e=Eo(e),t-=k;for(var i=an(r,e);++n<t;)e(n);return i},ir.toFinite=Qs,ir.toInteger=ws,ir.toLength=xs,ir.toLower=function(t){return Ms(t).toLowerCase()},ir.toNumber=_s,ir.toSafeInteger=function(t){return t?Br(ws(t),-_,_):0===t?t:0},ir.toString=Ms,ir.toUpper=function(t){return Ms(t).toUpperCase()},ir.trim=function(t,e,n){if((t=Ms(t))&&(n||e===o))return t.replace(xt,"");if(!t||!(e=Ei(e)))return t;var r=En(t),i=En(e);return Li(r,cn(r,i),un(r,i)+1).join("")},ir.trimEnd=function(t,e,n){if((t=Ms(t))&&(n||e===o))return t.replace(Dt,"");if(!t||!(e=Ei(e)))return t;var r=En(t);return Li(r,0,un(r,En(e))+1).join("")},ir.trimStart=function(t,e,n){if((t=Ms(t))&&(n||e===o))return t.replace(_t,"");if(!t||!(e=Ei(e)))return t;var r=En(t);return Li(r,cn(r,En(e))).join("")},ir.truncate=function(t,e){var n=30,r="...";if(gs(e)){var i="separator"in e?e.separator:i;n="length"in e?ws(e.length):n,r="omission"in e?Ei(e.omission):r}var a=(t=Ms(t)).length;if(fn(t)){var s=En(t);a=s.length}if(n>=a)return t;var A=n-yn(r);if(A<1)return r;var l=s?Li(s,0,A).join(""):t.slice(0,A);if(i===o)return l+r;if(s&&(A+=l.length-A),Is(i)){if(t.slice(A).search(i)){var c,u=l;for(i.global||(i=Vt(i.source,Ms(Ft.exec(i))+"g")),i.lastIndex=0;c=i.exec(u);)var g=c.index;l=l.slice(0,g===o?A:g)}}else if(t.indexOf(Ei(i),A)!=A){var h=l.lastIndexOf(i);h>-1&&(l=l.slice(0,h))}return l+r},ir.unescape=function(t){return(t=Ms(t))&&It.test(t)?t.replace(ft,vn):t},ir.uniqueId=function(t){var e=++ie;return Ms(t)+e},ir.upperCase=sA,ir.upperFirst=AA,ir.each=Ma,ir.eachRight=ka,ir.first=ia,mA(ir,(DA={},Fr(ir,function(t,e){re.call(ir.prototype,e)||(DA[e]=t)}),DA),{chain:!1}),ir.VERSION="4.17.13",Ue(["bind","bindKey","curry","curryRight","partial","partialRight"],function(t){ir[t].placeholder=ir}),Ue(["drop","take"],function(t,e){Ar.prototype[t]=function(n){n=n===o?1:Fn(ws(n),0);var r=this.__filtered__&&!e?new Ar(this):this.clone();return r.__filtered__?r.__takeCount__=Tn(n,r.__takeCount__):r.__views__.push({size:Tn(n,k),type:t+(r.__dir__<0?"Right":"")}),r},Ar.prototype[t+"Right"]=function(e){return this.reverse()[t](e).reverse()}}),Ue(["filter","map","takeWhile"],function(t,e){var n=e+1,r=1==n||3==n;Ar.prototype[t]=function(t){var e=this.clone();return e.__iteratees__.push({iteratee:Eo(t,3),type:n}),e.__filtered__=e.__filtered__||r,e}}),Ue(["head","last"],function(t,e){var n="take"+(e?"Right":"");Ar.prototype[t]=function(){return this[n](1).value()[0]}}),Ue(["initial","tail"],function(t,e){var n="drop"+(e?"":"Right");Ar.prototype[t]=function(){return this.__filtered__?new Ar(this):this[n](1)}}),Ar.prototype.compact=function(){return this.filter(fA)},Ar.prototype.find=function(t){return this.filter(t).head()},Ar.prototype.findLast=function(t){return this.reverse().find(t)},Ar.prototype.invokeMap=ui(function(t,e){return"function"==typeof t?new Ar(this):this.map(function(n){return zr(n,t,e)})}),Ar.prototype.reject=function(t){return this.filter(Ja(Eo(t)))},Ar.prototype.slice=function(t,e){t=ws(t);var n=this;return n.__filtered__&&(t>0||e<0)?new Ar(n):(t<0?n=n.takeRight(-t):t&&(n=n.drop(t)),e!==o&&(n=(e=ws(e))<0?n.dropRight(-e):n.take(e-t)),n)},Ar.prototype.takeRightWhile=function(t){return this.reverse().takeWhile(t).reverse()},Ar.prototype.toArray=function(){return this.take(k)},Fr(Ar.prototype,function(t,e){var n=/^(?:filter|find|map|reject)|While$/.test(e),r=/^(?:head|last)$/.test(e),i=ir[r?"take"+("last"==e?"Right":""):e],a=r||/^find/.test(e);i&&(ir.prototype[e]=function(){var e=this.__wrapped__,s=r?[1]:arguments,A=e instanceof Ar,l=s[0],c=A||ns(e),u=function(t){var e=i.apply(ir,Pe([t],s));return r&&g?e[0]:e};c&&n&&"function"==typeof l&&1!=l.length&&(A=c=!1);var g=this.__chain__,h=!!this.__actions__.length,d=a&&!g,f=A&&!h;if(!a&&c){e=f?e:new Ar(this);var p=t.apply(e,s);return p.__actions__.push({func:Qa,args:[u],thisArg:o}),new sr(p,g)}return d&&f?t.apply(this,s):(p=this.thru(u),d?r?p.value()[0]:p.value():p)})}),Ue(["pop","push","shift","sort","splice","unshift"],function(t){var e=Zt[t],n=/^(?:push|sort|unshift)$/.test(t)?"tap":"thru",r=/^(?:pop|shift)$/.test(t);ir.prototype[t]=function(){var t=arguments;if(r&&!this.__chain__){var i=this.value();return e.apply(ns(i)?i:[],t)}return this[n](function(n){return e.apply(ns(n)?n:[],t)})}}),Fr(Ar.prototype,function(t,e){var n=ir[e];if(n){var r=n.name+"";re.call(Vn,r)||(Vn[r]=[]),Vn[r].push({name:e,func:n})}}),Vn[Zi(o,p).name]=[{name:"wrapper",func:o}],Ar.prototype.clone=function(){var t=new Ar(this.__wrapped__);return t.__actions__=Hi(this.__actions__),t.__dir__=this.__dir__,t.__filtered__=this.__filtered__,t.__iteratees__=Hi(this.__iteratees__),t.__takeCount__=this.__takeCount__,t.__views__=Hi(this.__views__),t},Ar.prototype.reverse=function(){if(this.__filtered__){var t=new Ar(this);t.__dir__=-1,t.__filtered__=!0}else(t=this.clone()).__dir__*=-1;return t},Ar.prototype.value=function(){var t=this.__wrapped__.value(),e=this.__dir__,n=ns(t),r=e<0,i=n?t.length:0,o=function(t,e,n){for(var r=-1,i=n.length;++r<i;){var o=n[r],a=o.size;switch(o.type){case"drop":t+=a;break;case"dropRight":e-=a;break;case"take":e=Tn(e,t+a);break;case"takeRight":t=Fn(t,e-a)}}return{start:t,end:e}}(0,i,this.__views__),a=o.start,s=o.end,A=s-a,l=r?s:a-1,c=this.__iteratees__,u=c.length,g=0,h=Tn(A,this.__takeCount__);if(!n||!r&&i==A&&h==A)return wi(t,this.__actions__);var d=[];t:for(;A--&&g<h;){for(var f=-1,p=t[l+=e];++f<u;){var I=c[f],C=I.iteratee,m=I.type,y=C(p);if(2==m)p=y;else if(!y){if(1==m)continue t;break t}}d[g++]=p}return d},ir.prototype.at=wa,ir.prototype.chain=function(){return ba(this)},ir.prototype.commit=function(){return new sr(this.value(),this.__chain__)},ir.prototype.next=function(){this.__values__===o&&(this.__values__=bs(this.value()));var t=this.__index__>=this.__values__.length;return{done:t,value:t?o:this.__values__[this.__index__++]}},ir.prototype.plant=function(t){for(var e,n=this;n instanceof ar;){var r=Xo(n);r.__index__=0,r.__values__=o,e?i.__wrapped__=r:e=r;var i=r;n=n.__wrapped__}return i.__wrapped__=t,e},ir.prototype.reverse=function(){var t=this.__wrapped__;if(t instanceof Ar){var e=t;return this.__actions__.length&&(e=new Ar(this)),(e=e.reverse()).__actions__.push({func:Qa,args:[ga],thisArg:o}),new sr(e,this.__chain__)}return this.thru(ga)},ir.prototype.toJSON=ir.prototype.valueOf=ir.prototype.value=function(){return wi(this.__wrapped__,this.__actions__)},ir.prototype.first=ir.prototype.head,nn&&(ir.prototype[nn]=function(){return this}),ir}();be._=Bn,(i=function(){return Bn}.call(e,n,e,r))===o||(r.exports=i)}).call(this)}).call(this,n(113),n(32)(t))},function(t,e){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(t){"object"==typeof window&&(n=window)}t.exports=n},function(t,e,n){"use strict";console.warn||(console.warn=function(){console.log.apply(console,["WARNING:"].concat(Array.prototype.slice.call(arguments)))});var r=/^[\w\.]+\(.*\)$/,i=function(t,e,n,a,s){"string"==typeof a&&(s=a,a=void 0);var A,l,c=e.split("."),u=this.$$DEPRECATION_WARNED=this.$$DEPRECATION_WARNED||{},g=this,h=e.indexOf(" ")>=0,d=r.test(t);if(t in u||(u[t]=i.warnings),!d){if(h)return void(u[t]&&(--u[t],console.warn(e)));throw"Expected method name with parentheses in 1st parameter OR simple warning (containing one or more spaces) in 2nd parameter of deprecated() call."}if(h)throw"Expected replacement method or property in 2nd parameter of deprecated() call.";u[t]&&(--u[t],A=r.test(e)?"method":"property",l="The ."+t+" method has been deprecated as of v"+n+" in favor of the ."+c.join(".")+" "+A+". (Will be removed in a future release.)",s&&(l+=" "+s),console.warn(l));var f=o(t);function p(t){var e=f.indexOf(t);if(-1===e)throw'Actual arg "'+t+'" not found in formal arg list '+f;return a[e]}for(var I=0,C=c.length-1;I<=C;++I){var m=c[I],y=m.match(/\w+/)[0],E=r.test(m),v=E?o(m):void 0,B=[];v?(B=v.map(p),g=g[y].apply(g,B)):g=E?g[y]():g[y]}return g};function o(t){return t.match(/^\w+\((.*)\)$/)[1].match(/(\w+)/g)}i.warnings=1,t.exports=i},function(t,e,n){"use strict";var r=n(27),i=n(116),o=n(46),a=n(17),s=n(18),A={};function l(t,e){switch(typeof e){case"number":A.number||(console.warn("Column(behavior: object, index: number) overload has been deprecated as of v2.1.6 in favor of Column(behavior: object, columnSchema: object) overload with defined columnSchema.index. (Will be removed in a future release.)"),A.number=!0),e={index:e};break;case"string":var n,r;if(A.string||(console.warn("Column(behavior:object, name: string) overload (where name is sought in schema) has been deprecated as of v2.1.6 in favor of Column(behavior: object, columnSchema: object) overload with defined columnSchema.index. (Will be removed in a future release.)"),A.string=!0),n=e,(r=t.dataModel.schema.findIndex(function(t){return t.name===n}))<0)throw new this.HypergridError('Column named "'+n+'" not found in schema.');e={name:n,index:r};break;case"object":if(void 0===e.index){if(A.object||(console.warn("Column(behavior:object, columnSchema: object) overload (where columnSchema.index is undefined but columnSchema.name is sought in schema) has been deprecated as of v2.1.6 in favor of defined columnSchema.index. (Will be removed in a future release.)"),A.object=!0),n=e.name,(r=t.dataModel.schema.findIndex(function(t){return t.name===n}))<0)throw new this.HypergridError('Column named "'+n+'" not found in schema.');e.index=r}}if(void 0===e.index)throw new a("Column index required.");switch(this.behavior=t,this.dataModel=t.dataModel,Object.defineProperties(this,{index:{value:e.index},name:{enumerable:!0,value:e.name||e.index.toString()}}),this.properties=this.schema=e,e.index){case this.behavior.treeColumnIndex:var i=s[Object.create(this.properties.treeHeader,{isDataRow:{value:!0}}).leftIcon];this.properties.minimumColumnWidth=i?i.width+3:0;break;case this.behavior.rowColumnIndex:break;default:if(e.index<0)throw new this.HypergridError("New column index "+e.index+" out of range.")}}l.prototype={constructor:l.prototype.constructor,$$CLASS_NAME:"Column",HypergridError:a,mixIn:r.mixIn,set header(t){this.schema.header=t,this.behavior.grid.repaint()},get header(){return this.schema.header},set calculator(t){(t=function(t){if(t){var e=' (for column "'+this.name+'").';if("function"==typeof t)t=t.toString();else if("string"!=typeof t)throw new a("Expected calculator function OR string containing calculator function OR calculator name"+e);var n,r,o=this.behavior.grid.properties.calculators||(this.behavior.grid.properties.calculators={});if(/^\w+$/.test(t)){if(!o[r=t])throw new a('Unknown calculator name "'+r+e)}else if(n=t.match(u))r=n[1];else if(t.test(g))r=t;else if(c.test(t))throw new a("Arrow function not permitted as column calculator "+e);return o[r]||(o[r]=t),o[r]=i(o[r]),o[r]}}.call(this,t))!==this.schema.calculator&&(this.schema.calculator=t,this.behavior.grid.reindex())},get calculator(){return this.schema.calculator},set type(t){this.schema.type=t,this.behavior.reindex()},get type(){return this.schema.type},getValue:function(t,e){return this.dataModel.getValue(this.index,t,e)},setValue:function(t,e,n){return this.dataModel.setValue(this.index,t,e,n)},getWidth:function(){return this.properties.width||this.behavior.grid.properties.defaultColumnWidth},setWidth:function(t){(t=Math.min(Math.max(this.properties.minimumColumnWidth,t),this.properties.maximumColumnWidth||1/0))!==this.properties.width&&(this.properties.width=t,this.properties.columnAutosizing=!1)},checkColumnAutosizing:function(t){var e,n,r,i=this.properties;return i.columnAutosizing&&(e=i.width,n=i.preferredWidth||e,t=t||!i.columnAutosized,(e!==n||t&&void 0!==n)&&(i.width=t?n:Math.max(e,n),i.columnAutosizingMax&&i.width>i.columnAutosizingMax&&(i.width=i.columnAutosizingMax),i.columnAutosized=!isNaN(i.width),r=i.width!==e)),r},getCellType:function(t){var e=this.getValue(t);return this.typeOf(e)},getType:function(){var t=this.properties,e=t.type;return e||"unknown"!==(e=this.computeColumnType())&&(t.type=e),e},computeColumnType:function(){var t=this.behavior.getHeaderRowCount(),e=this.behavior.getRowCount(),n=this.getValue(t),r=this.typeOf(n);if(!r)return"unknown";for(var i=this.typeOf(n),o=t;o<e;o++)n=this.getValue(o),r=this.typeOf(n);return i},typeOf:function(t){if(null==t)return null;var e=typeof t;switch(e){case"object":return t.constructor.name.toLowerCase();case"number":return parseInt(t)===t?"int":"float";default:return e}},get properties(){return this._properties},set properties(t){this.addProperties(t,!0)},addProperties:function(t,e){t&&(!e&&this._properties||(this._properties=this.createColumnProperties()),o(this._properties,t))},setBackgroundColor:function(t){this.properties.backgroundColor!==t&&(this.properties.backgroundColor=t,this.behavior.grid.renderer.rebundleGridRenderers())},getCellEditorAt:function(t){var e=this.index,n=t.gridCell.y,r=t.properties.editor,i=Object.create(t,{format:{writable:!0,enumerable:!0,value:t.properties.format}}),o=t.subgrid.getCellEditorAt(e,n,r,i);return o&&!o.grid&&(o=void 0),o},getFormatter:function(){var t=this.properties.format;return this.behavior.grid.localization.get(t).format}};var c=/^(\(.*\)|\w+)\s*=>/,u=/^function\s+(\w+)\(/,g=/^function\s*\(/;l.prototype.mixIn(n(119).mixin),l.prototype.mixIn(n(120).mixin),t.exports=l},function(t,e,n){"use strict";var r=n(17);t.exports=function(t){switch(typeof t){case"undefined":case"function":return t;case"string":break;default:throw new r("Expected string, function, or undefined.")}var e=t.match(/^function\s*\w*\s*\(([^]*?)\)/);if(!e)throw new r("Expected function keyword with formal parameter list.");e=e[1].split(",").map(function(t,e){if(!(t=t.match(/\s*(\w*)\s*/))&&e)throw new r("Expected formal parameter.");return t[1]});var n=t.match(/{\s*([^]*?)\s*}\s*$/);if(!n)throw new r("Expected function body.");return n=n[1],1!==e.length||e[0]?e=e.concat(n):e[0]=n,Function.apply(null,e)}},function(t,e,n){"use strict";var r,i="((data:image/svg\\+xml)(;\\w+)?(,)(.*)|(.*\\.svg))",o=new RegExp("^()"+i+"()$"),a=new RegExp('^(url\\(")'+i+'("\\))$'),s=0,A=2,l=3,c=4,u=5,g=6,h=7;function d(t,e,n,r){var i=this;t[h]?function(t,e){var n=new XMLHttpRequest;n.open("GET",t,!0),n.onreadystatechange=function(){n.readyState===f&&n.status===p&&e(n.responseText)},n.send(null)}(t[h],function(o){t[l]="data:image/svg+xml",t[c]=";base64",t[u]=",",t[g]=o,t[h]="",I.call(i,t,e,n,r)}):I.call(this,t,e,n,r)}(r=e).cssImagePropertyNames=["background-image","list-style-image","border-image","content"],r.setSvgProps=function(t){t.color&&(this.style.stroke=this.style.color=t.color),t.backgroundColor&&(this.style.fill=t.backgroundColor)},r.setImgSvgProps=function(t,e){if("IMG"===this.tagName){var n=this.src.match(o);n&&d.call(this,n,t,e,function(t){this.src=t})}return this},r.setRuleSvgProps=function(t,e,n){var i,o=this.style;function s(t){return i=o[t].match(a)}if("string"==typeof e&&(n=e,e=void 0),n)s(n);else for(var A=0;A<r.cssImagePropertyNames.length&&!s(n=r.cssImagePropertyNames[A]);++A);return i&&d.call(this,i,t,e,function(t){o[n]=t}),this};var f=4,p=200;function I(t,e,n,i){var o=t[c],a=t[g],l=document.createElement("div");switch(n=n||this.setSvgProps||r.setSvgProps,o){case void 0:l.innerHTML=a.indexOf("%3C")>=0?decodeURIComponent(a):a,n.call(l.firstElementChild,e);var u=(new XMLSerializer).serializeToString(l.firstElementChild);t[g]=encodeURIComponent(u);break;case";base64":l.innerHTML=a.indexOf("<")<0?atob(a):a,n.call(l.firstElementChild,e),u=(new XMLSerializer).serializeToString(l.firstElementChild),t[g]=btoa(u);break;default:throw new TypeError('Unexpected encoding "'+o+'"')}t[s]=t[A]=void 0,i.call(this,t.join(""))}r.mixin=function(t,e){return"svg"===t.tagName?e?t.setTheme=e:Object.defineProperty(t,"setTheme",{get:function(){return r.setSvgProps}}):"IMG"===t.tagName?(e&&(t.setSvgProps=e),t.setTheme=r.setImgSvgProps):"[object CSSStyleRule]"==t&&(e&&(t.setSvgProps=e),t.setTheme=r.setRuleSvgProps),t}},function(t,e){t.exports={calendar:{type:"image/png",data:"iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAc0lEQVR4nIXQwQkCMRSE4U9ZLMCT9Xjaq2AfNhfYU5oQLMAOtoN48EWei5iBIRPe/yYQ3qrhf1lFG7iKcEaJxSfukUvMWgdHavt0uWHtg2QwxXnAnJZ2uOLyVZtybzzhgWNmfoFl0/YB87NbzR1cjP9xeQHSDC6mcL1xFQAAAABJRU5ErkJggg=="},checked:{type:"image/png",data:"iVBORw0KGgoAAAANSUhEUgAAAAsAAAALCAMAAACecocUAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAA4xpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADw/eHBhY2tldCBiZWdpbj0i77u/IiBpZD0iVzVNME1wQ2VoaUh6cmVTek5UY3prYzlkIj8+IDx4OnhtcG1ldGEgeG1sbnM6eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IkFkb2JlIFhNUCBDb3JlIDUuNi1jMTQwIDc5LjE2MDMwMiwgMjAxNy8wMy8wMi0xNjo1OTozOCAgICAgICAgIj4gPHJkZjpSREYgeG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4gPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIgeG1sbnM6eG1wTU09Imh0dHA6Ly9ucy5hZG9iZS5jb20veGFwLzEuMC9tbS8iIHhtbG5zOnN0UmVmPSJodHRwOi8vbnMuYWRvYmUuY29tL3hhcC8xLjAvc1R5cGUvUmVzb3VyY2VSZWYjIiB4bWxuczp4bXA9Imh0dHA6Ly9ucy5hZG9iZS5jb20veGFwLzEuMC8iIHhtcE1NOk9yaWdpbmFsRG9jdW1lbnRJRD0ieG1wLmRpZDoxZmM0YjdjYS01N2U3LTRjZTUtYWE3Zi1kYmFjZTU0ODJmZjciIHhtcE1NOkRvY3VtZW50SUQ9InhtcC5kaWQ6MkIzRUM3OEQzNkMyMTFFODhEOUJCMzRBRUM0QUQ5QUMiIHhtcE1NOkluc3RhbmNlSUQ9InhtcC5paWQ6MkIzRUM3OEMzNkMyMTFFODhEOUJCMzRBRUM0QUQ5QUMiIHhtcDpDcmVhdG9yVG9vbD0iQWRvYmUgUGhvdG9zaG9wIEVsZW1lbnRzIDE2LjAgKE1hY2ludG9zaCkiPiA8eG1wTU06RGVyaXZlZEZyb20gc3RSZWY6aW5zdGFuY2VJRD0ieG1wLmlpZDphZjdjMDA1NS1mMzJlLTQ1NjAtOGQxOS1lYTU1NGVhNDkzMmEiIHN0UmVmOmRvY3VtZW50SUQ9ImFkb2JlOmRvY2lkOnBob3Rvc2hvcDo0YmZiODQwOS03ZWY0LTExN2ItYjI4MS05MDYxODEwMDAwMDAiLz4gPC9yZGY6RGVzY3JpcHRpb24+IDwvcmRmOlJERj4gPC94OnhtcG1ldGE+IDw/eHBhY2tldCBlbmQ9InIiPz5PI8IaAAAAkFBMVEVOTk5NTU29vb1paWnPz8/AwMC7u7u5ublDQ0NSUlJFRUVfX1/IyMixsbHi4uKFhYXT09PDw8NXV1fKysrMzMxYWFhzc3N7e3ve3t6ioqK+vr7Ly8uhoaHa2tqysrKurq7s7Oyvr6/Jycm2trbt7e3X19fh4eG1tbW/v7+3t7fo6OhERES6urrW1tbd3d0AAADC/hfQAAAAMHRSTlP//////////////////////////////////////////////////////////////wBipdB4AAAAcElEQVR42hTIRRbDMAwAUYVTZoa4ZleyfP/btZrVfwOlq4l66utlge5lrbIqq0cL9JX07Onv8PZr791pvm8H+DBrbhBiTn9f4IzjjcW6Qhw3zNnAxLHChln+iq9xsRUPQNFlyYUjlF0KwZiQDuUnwACjfw8FagWYLwAAAABJRU5ErkJggg=="},"down-rectangle":{type:"image/png",data:"iVBORw0KGgoAAAANSUhEUgAAAAkAAAAECAYAAABcDxXOAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAADsMAAA7DAcdvqGQAAAAadEVYdFNvZnR3YXJlAFBhaW50Lk5FVCB2My41LjExR/NCNwAAABpJREFUGFdjgIL/eDAKIKgABggqgAE0BQwMAPTlD/Fpi0JfAAAAAElFTkSuQmCC"},"filter-off":{type:"image/png",data:"iVBORw0KGgoAAAANSUhEUgAAAA4AAAAMCAYAAABSgIzaAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAADsMAAA7DAcdvqGQAAAAYdEVYdFNvZnR3YXJlAHBhaW50Lm5ldCA0LjAuNWWFMmUAAAChSURBVChTzZHBCoUgFET9TqEiskgyWoutQvRLRIr+cR7XQAjiJW/1BgZmMUevXsY5xy9OoDEGMcYiUzeB67qibVuwQjVNA6311V+WBeM4vsLDMEApde/1fY9pmtI453neHEKAlBJd1z0fXtc16PbjODK07zvmeUZVVd8nooc75zJIOX3Gm6i0bVsGKf8xKIRIuyJTLgJJ3nvQzsjW2geIsQ/pr9hMVrSncAAAAABJRU5ErkJggg=="},"filter-on":{type:"image/png",data:"iVBORw0KGgoAAAANSUhEUgAAAA4AAAAMCAYAAABSgIzaAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAADsMAAA7DAcdvqGQAAAAYdEVYdFNvZnR3YXJlAHBhaW50Lm5ldCA0LjAuNWWFMmUAAACoSURBVChTY3BqfP2fHAzWmDbj7f8p294RhVOBasEa02e+/e/VBmQQCTxaX/9PnvYGoj5ywpv/Qd2ENft3vv4f1gfVBAP+nW/+h/a+ATtn1q73KHjytvdgg3070DTBgHvL6/8g22fsQGiaDmSHA21xaybgIpDHixa8hWssnA8NDEIApCh3LkIjiD2INYJCL2X6W3B8gdhEaQQBUOCA4gyE8+e9xaKJgQEA/74BNE3cElkAAAAASUVORK5CYII="},unchecked:{type:"image/png",data:"iVBORw0KGgoAAAANSUhEUgAAAAsAAAALCAMAAACecocUAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAA4xpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADw/eHBhY2tldCBiZWdpbj0i77u/IiBpZD0iVzVNME1wQ2VoaUh6cmVTek5UY3prYzlkIj8+IDx4OnhtcG1ldGEgeG1sbnM6eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IkFkb2JlIFhNUCBDb3JlIDUuNi1jMTQwIDc5LjE2MDMwMiwgMjAxNy8wMy8wMi0xNjo1OTozOCAgICAgICAgIj4gPHJkZjpSREYgeG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4gPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIgeG1sbnM6eG1wTU09Imh0dHA6Ly9ucy5hZG9iZS5jb20veGFwLzEuMC9tbS8iIHhtbG5zOnN0UmVmPSJodHRwOi8vbnMuYWRvYmUuY29tL3hhcC8xLjAvc1R5cGUvUmVzb3VyY2VSZWYjIiB4bWxuczp4bXA9Imh0dHA6Ly9ucy5hZG9iZS5jb20veGFwLzEuMC8iIHhtcE1NOk9yaWdpbmFsRG9jdW1lbnRJRD0ieG1wLmRpZDphNDQ3MTQ0Zi0wYmFmLTQ3YzMtYjBmNi1kODRlNDI0MjU4ZDIiIHhtcE1NOkRvY3VtZW50SUQ9InhtcC5kaWQ6MkIzRUM3OTEzNkMyMTFFODhEOUJCMzRBRUM0QUQ5QUMiIHhtcE1NOkluc3RhbmNlSUQ9InhtcC5paWQ6MkIzRUM3OTAzNkMyMTFFODhEOUJCMzRBRUM0QUQ5QUMiIHhtcDpDcmVhdG9yVG9vbD0iQWRvYmUgUGhvdG9zaG9wIEVsZW1lbnRzIDE2LjAgKE1hY2ludG9zaCkiPiA8eG1wTU06RGVyaXZlZEZyb20gc3RSZWY6aW5zdGFuY2VJRD0ieG1wLmlpZDo1OWY2YWVhMC0xZjJkLTRjM2UtYTRiZi02MWYyMDNlMmI4YjciIHN0UmVmOmRvY3VtZW50SUQ9ImFkb2JlOmRvY2lkOnBob3Rvc2hvcDo2Nzk3NWVjNC03ZWY0LTExN2ItYjI4MS05MDYxODEwMDAwMDAiLz4gPC9yZGY6RGVzY3JpcHRpb24+IDwvcmRmOlJERj4gPC94OnhtcG1ldGE+IDw/eHBhY2tldCBlbmQ9InIiPz6ywX99AAAAYFBMVEW5ubm/v7/X19fLy8vp6ene3t69vb2xsbG6urq+vr6urq7a2tq4uLji4uLIyMjBwcHDw8PJycnPz8/V1dW1tbWvr6/t7e23t7e7u7vs7Oy2trbo6Ojh4eHW1tbd3d0AAAAm0pqPAAAAIHRSTlP/////////////////////////////////////////AFxcG+0AAABZSURBVHjaTIqHDoAgEEOLC9wL5RTP+/+/VIIxdCTNSyFjZQwR1YNAtftxhOQLUHjvQ68K+orqW2C7P7FL9okmrjLl5cv1/+lAKuOVmTObQyZ32tdulkeAAQCpggrEM7zMmAAAAABJRU5ErkJggg=="},"up-down-spin":{type:"image/png",data:"iVBORw0KGgoAAAANSUhEUgAAAA4AAAAPCAYAAADUFP50AAAABGdBTUEAALGPC/xhBQAAAAlwSFlzAAAOwQAADsEBuJFr7QAAABh0RVh0U29mdHdhcmUAcGFpbnQubmV0IDQuMC41ZYUyZQAAAGJJREFUOE+lkwEKACEIBH2Zb/PnHsoGeaVJDUjGOgRRpKpkiIj+y4MME3eDR7kaKOVNsJyMNjIHzGy9YnW6J7qIcrriQimeCqORNABd0fpRTkt8uVUj7EsxC6vs/q3e/Q6iD2bwnByjPXHNAAAAAElFTkSuQmCC"},"up-down":{type:"image/png",data:"iVBORw0KGgoAAAANSUhEUgAAAA4AAAAPCAYAAADUFP50AAAABGdBTUEAALGPC/xhBQAAAAlwSFlzAAAOwQAADsEBuJFr7QAAABh0RVh0U29mdHdhcmUAcGFpbnQubmV0IDQuMC41ZYUyZQAAAGFJREFUOE+lkkEKQCEIRD2ZJ3Ph3iN4WD9GflpYhj0YYowpGgJmbikd3gjMDFokwbuT1iAiurG5nomgqo5QaPo9ERQRI6Jf7sfGjudy2je23+i0Wl2oQ85TOdlfrJQOazF8br+rqTXQKn0AAAAASUVORK5CYII="}}},function(t,e,n){"use strict";var r=n(46);function i(t,e){return this.getCellOwnProperties(t,e)||o.call(this,t,e)}function o(t,e){var n=(e||this.dataModel).getRowMetadata(t,null),r=this.properties;switch(this.index){case this.behavior.treeColumnIndex:r=r.treeHeader;break;case this.behavior.rowColumnIndex:r=r.rowHeader;break;default:e&&"filter"===e.type&&(r=this.properties.filterProperties)}return n[this.name]=Object.create(r)}e.mixin={getCellProperties:function(t,e){return this.getCellOwnProperties(t,e)||this.properties},setCellProperties:function(t,e,n){if(e)return Object.assign(o.call(this,t,n),e)},addCellProperties:function(t,e,n){if(e)return r(i.call(this,t,n),e)},getCellOwnProperties:function(t,e){var n;return(n=(e||this.dataModel).getRowMetadata(t))&&n&&n[this.name]||null},deleteCellOwnProperties:function(t,e){var n=(e=e||this.dataModel).getRowMetadata(t);n&&(delete n[this.name],0===Object.keys(n).length&&e.setRowMetadata(t))},getCellProperty:function(t,e,n){return this.getCellProperties(t,n)[e]},setCellProperty:function(t,e,n,r){var o=i.call(this,t,r);return o[e]=n,o},deleteCellProperty:function(t,e,n){var r=this.getCellOwnProperties(t,n);r&&delete r[e]},clearAllCellProperties:function(){this.behavior.subgrids.forEach(function(t){for(var e=t.getRowCount();e--;)this.deleteCellOwnProperties(e,t)},this)}}},function(t,e,n){"use strict";function r(){var t,e=this,n=e.behavior.grid.properties;switch(t=Object.create(n,{index:{get:function(){return e.index}},name:{get:function(){return e.name}},field:{get:function(){return e.name}},columnName:{get:function(){return e.name}},header:{get:function(){return e.header},set:function(t){this!==e.properties&&(n.header=t),e.header=t}},type:{get:function(){return e.type},set:function(t){this!==e.properties&&(n.type=t),e.type=t}},calculator:{get:function(){return e.calculator},set:function(t){this!==e.properties&&(n.calculator=t),e.calculator=t}},toJSON:{value:function(){return Object.assign({header:this.header,type:this.type,calculator:this.calculator},this)}}}),Object.defineProperties(t,{rowHeader:{value:Object.create(t,r.rowHeaderDescriptors)},treeHeader:{value:Object.create(t,r.treeHeaderDescriptors)},columnHeader:{value:Object.create(t,r.columnHeaderDescriptors)},filterProperties:{value:Object.create(t,r.filterDescriptors)}}),e.index){case e.behavior.treeColumnIndex:t=t.treeHeader;break;case e.behavior.rowColumnIndex:t=t.rowHeader}return t}r.treeHeaderDescriptors={font:{configurable:!0,enumerable:!0,get:function(){return this.treeHeaderFont},set:function(t){this.treeHeaderFont=t}},color:{configurable:!0,enumerable:!0,get:function(){return this.treeHeaderColor},set:function(t){this.treeHeaderColor=t}},backgroundColor:{configurable:!0,enumerable:!0,get:function(){return this.treeHeaderBackgroundColor},set:function(t){this.treeHeaderBackgroundColor=t}},foregroundSelectionFont:{configurable:!0,enumerable:!0,get:function(){return this.treeHeaderForegroundSelectionFont},set:function(t){this.treeHeaderForegroundSelectionFont=t}},foregroundSelectionColor:{configurable:!0,enumerable:!0,get:function(){return this.treeHeaderForegroundSelectionColor},set:function(t){this.treeHeaderForegroundSelectionColor=t}},renderer:{configurable:!0,enumerable:!0,get:function(){return this.isDataRow?this.treeRenderer:this.grid.properties.renderer},set:function(t){this.treeRenderer=t}},backgroundSelectionColor:{configurable:!0,enumerable:!0,get:function(){return this.treeHeaderBackgroundSelectionColor},set:function(t){this.treeHeaderBackgroundSelectionColor=t}},columnAutosizing:{configurable:!0,enumerable:!0,get:function(){return this.treeColumnAutosizing},set:function(t){this.treeColumnAutosizing=t}},columnAutosizingMax:{configurable:!0,enumerable:!0,get:function(){return this.treeColumnAutosizingMax},set:function(t){this.treeColumnAutosizingMax=t}}},r.rowHeaderDescriptors={font:{configurable:!0,enumerable:!0,get:function(){return this.rowHeaderFont},set:function(t){this.rowHeaderFont=t}},color:{configurable:!0,enumerable:!0,get:function(){return this.rowHeaderColor},set:function(t){this.rowHeaderColor=t}},backgroundColor:{configurable:!0,enumerable:!0,get:function(){return this.rowHeaderBackgroundColor},set:function(t){this.rowHeaderBackgroundColor=t}},foregroundSelectionFont:{configurable:!0,enumerable:!0,get:function(){return this.rowHeaderForegroundSelectionFont},set:function(t){this.rowHeaderForegroundSelectionFont=t}},foregroundSelectionColor:{configurable:!0,enumerable:!0,get:function(){return this.rowHeaderForegroundSelectionColor},set:function(t){this.rowHeaderForegroundSelectionColor=t}},backgroundSelectionColor:{configurable:!0,enumerable:!0,get:function(){return this.rowHeaderBackgroundSelectionColor},set:function(t){this.rowHeaderBackgroundSelectionColor=t}},leftIcon:{configurable:!0,enumerable:!0,get:function(){var t;if(this.grid.properties.rowHeaderCheckboxes)return this.isDataRow?t=this.isRowSelected?"checked":"unchecked":this.isHeaderRow?t=this.allRowsSelected?"checked":"unchecked":this.isFilterRow&&(t="filter-off"),t},set:function(t){Object.defineProperty(this,"leftIcon",{configurable:!0,enumerable:!0,writable:!0,value:t})}},columnAutosizing:{configurable:!0,enumerable:!0,get:function(){return this.rowNumberAutosizing},set:function(t){this.rowNumberAutosizing=t}}},r.filterDescriptors={font:{configurable:!0,enumerable:!0,get:function(){return this.filterFont},set:function(t){this.filterFont=t}},color:{configurable:!0,enumerable:!0,get:function(){return this.filterColor},set:function(t){this.filterColor=t}},backgroundColor:{configurable:!0,enumerable:!0,get:function(){return this.filterBackgroundColor},set:function(t){this.filterBackgroundColor=t}},foregroundSelectionColor:{configurable:!0,enumerable:!0,get:function(){return this.filterForegroundSelectionColor},set:function(t){this.filterForegroundSelectionColor=t}},backgroundSelectionColor:{configurable:!0,enumerable:!0,get:function(){return this.filterBackgroundSelectionColor},set:function(t){this.filterBackgroundSelectionColor=t}},halign:{configurable:!0,enumerable:!0,get:function(){return this.filterHalign},set:function(t){this.filterHalign=t}},renderer:{configurable:!0,enumerable:!0,get:function(){return this.filterRenderer},set:function(t){this.filterRenderer=t}},editor:{configurable:!0,enumerable:!0,get:function(){return this.filterEditor},set:function(t){this.filterEditor=t}},rightIcon:{configurable:!0,enumerable:!0,get:function(){var t;return this.filterable&&(t=this.filter?"filter-on":"filter-off"),t},set:function(t){Object.defineProperty(this,"rightIcon",{configurable:!0,enumerable:!0,writable:!0,value:t})}}},r.columnHeaderDescriptors={font:{configurable:!0,enumerable:!0,get:function(){return this.columnHeaderFont},set:function(t){this.columnHeaderFont=t}},color:{configurable:!0,enumerable:!0,get:function(){return this.columnHeaderColor},set:function(t){this.columnHeaderColor=t}},backgroundColor:{configurable:!0,enumerable:!0,get:function(){return this.columnHeaderBackgroundColor},set:function(t){this.columnHeaderBackgroundColor=t}},foregroundSelectionFont:{configurable:!0,enumerable:!0,get:function(){return this.columnHeaderForegroundSelectionFont},set:function(t){this.columnHeaderForegroundSelectionFont=t}},foregroundSelectionColor:{configurable:!0,enumerable:!0,get:function(){return this.columnHeaderForegroundSelectionColor},set:function(t){this.columnHeaderForegroundSelectionColor=t}},backgroundSelectionColor:{configurable:!0,enumerable:!0,get:function(){return this.columnHeaderBackgroundSelectionColor},set:function(t){this.columnHeaderBackgroundSelectionColor=t}},halign:{configurable:!0,enumerable:!0,get:function(){return this.columnHeaderHalign},set:function(t){this.columnHeaderHalign=t}},format:{configurable:!0,enumerable:!0,get:function(){return this.columnHeaderFormat},set:function(t){this.columnHeaderFormat=t}},renderer:{configurable:!0,enumerable:!0,get:function(){return this.columnHeaderRenderer},set:function(t){this.columnHeaderRenderer=t}},leftIcon:{writable:!0,value:void 0},centerIcon:{writable:!0,value:void 0},rightIcon:{writable:!0,value:void 0}},e.mixin={createColumnProperties:r}},function(t,e,n){"use strict";var r=Object.defineProperties({},{value:{get:function(){return this.subgrid.getValue(this.dataCell.x,this.dataCell.y)},set:function(t){this.subgrid.setValue(this.dataCell.x,this.dataCell.y,t)}},dataRow:{get:function(){return this.subgrid.getRow(this.dataCell.y)}},formattedValue:{get:function(){return this.grid.formatValue(this.properties.format,this.value)}},bounds:{get:function(){return this._bounds||(this._bounds={x:this.visibleColumn.left,y:this.visibleRow.top,width:this.visibleColumn.width,height:this.visibleRow.height})}},columnProperties:{get:function(){var t=this._columnProperties;return t||(t=this.column.properties,this.isDataColumn&&(this.isDataRow||(t=this.isFilterRow?t.filterProperties:t.columnHeader)),this._columnProperties=t),t}},cellOwnProperties:{get:function(){return void 0===this._cellOwnProperties&&(this._cellOwnProperties=this.column.getCellOwnProperties(this.dataCell.y,this.subgrid)),this._cellOwnProperties}},properties:{get:function(){return this.cellOwnProperties||this.columnProperties}},getCellProperty:{value:function(t){return this.properties[t]}},setCellProperty:{value:function(t,e){this._cellOwnProperties=this.column.setCellProperty(this.dataCell.y,t,e,this.subgrid)}},rowOwnProperties:{get:function(){return this.behavior.getRowProperties(this,void 0,this.subgrid)}},rowProperties:{get:function(){return this.behavior.getRowProperties(this,null,this.subgrid)},set:function(t){this.behavior.setRowProperties(this,t,this.subgrid)}},getRowProperty:{value:function(t){var e=this.rowOwnProperties;return e&&e[t]}},setRowProperty:{value:function(t,e){this.rowProperties[t]=e}},reset:{value:function(t,e){this._columnProperties=void 0,this._cellOwnProperties=void 0,this._bounds=void 0,this.snapshot=[],this.minWidth=void 0,this.visibleColumn=t,this.visibleRow=e,this.subgrid=e.subgrid,this.column=t.column,this.gridCell.x=t.columnIndex,this.gridCell.y=e.index,this.dataCell.x=this.column&&this.column.index,this.dataCell.y=e.rowIndex}},resetGridCY:{value:function(t,e){var n,r,i=(r=this.renderer.getVisibleColumn(t))&&(n=this.renderer.getVisibleRow(e));return i&&this.reset(r,n),i}},resetGridXY:{value:function(t,e){var n,r,i=(r=this.renderer.visibleColumns[t])&&(n=this.renderer.getVisibleRow(e));return i&&this.reset(r,n),i}},resetDataXY:{value:function(t,e,n){var r,i,o=(i=this.renderer.getVisibleDataColumn(t))&&(r=this.renderer.getVisibleDataRow(e,n));return o&&this.reset(i,r),o}},resetGridXDataY:{value:function(t,e,n,r){var i,o,a;return r?(o={column:this.behavior.getColumn(t),columnIndex:t},a={subgrid:n||this.behavior.dataModel,rowIndex:e},i=!0):i=(o=this.renderer.getVisibleColumn(t))&&(a=this.renderer.getVisibleDataRow(e,n)),i&&this.reset(o,a),i&&this}},clone:{value:function(t){var e=new this.constructor;return e.resetGridXY(this.visibleColumn.index,this.visibleRow.index),t&&Object.assign(e,this),e}},editPoint:{get:function(){throw"The `.editPoint` property is no longer available as of v1.2.10. Use the following coordinates instead:\n`.gridCell.x` - The active column index. (Adjusted for column scrolling after fixed columns.)\n`.gridCell.y` - The vertical grid coordinate. (Unaffected by row scrolling.)\n`.dataCell.x` - The data model's column index. (Unaffected by column scrolling.)\n`.dataCell.y` - The data model's row index. (Adjusted for data row scrolling after fixed rows.)\n"}},mousePointInClickRect:{get:function(){var t="clickRect"in this?this.clickRect:this.properties.clickRect;return!t||("function"==typeof t.contains?t.contains(this.mousePoint):t.x<=this.mousePoint.x&&this.mousePoint.x<t.x+t.width&&t.y<=this.mousePoint.y&&this.mousePoint.y<t.y+t.height)}},isRowVisible:{get:function(){return!!this.visibleRow}},isColumnVisible:{get:function(){return!!this.visibleColumn}},isCellVisible:{get:function(){return this.isRowVisible&&this.isColumnVisible}},isDataRow:{get:function(){return this.subgrid.isData}},isDataColumn:{get:function(){return this.gridCell.x>=0}},isDataCell:{get:function(){return this.isDataRow&&this.isDataColumn}},isRowSelected:{get:function(){return this.isDataRow&&this.selectionModel.isRowSelected(this.dataCell.y)}},isColumnSelected:{get:function(){return this.isDataColumn&&this.selectionModel.isColumnSelected(this.gridCell.x)}},isCellSelected:{get:function(){return this.selectionModel.isCellSelected(this.gridCell.x,this.dataCell.y)}},isRowHovered:{get:function(){return this.grid.canvas.hasMouse&&this.isDataRow&&this.grid.hoverCell&&this.grid.hoverCell.y===this.gridCell.y}},isColumnHovered:{get:function(){return this.grid.canvas.hasMouse&&this.isDataColumn&&this.grid.hoverCell&&this.grid.hoverCell.x===this.gridCell.x}},isCellHovered:{get:function(){return this.isRowHovered&&this.isColumnHovered}},isRowFixed:{get:function(){return this.isDataRow&&this.dataCell.y<this.grid.properties.fixedRowCount}},isColumnFixed:{get:function(){return this.isDataColumn&&this.gridCell.x<this.grid.properties.fixedColumnCount}},isCellFixed:{get:function(){return this.isRowFixed&&this.isColumnFixed}},isHandleColumn:{get:function(){return this.gridCell.x===this.behavior.rowColumnIndex&&this.grid.properties.showRowNumbers}},isHandleCell:{get:function(){return this.isHandleColumn&&this.isDataRow}},isTreeColumn:{get:function(){return this.gridCell.x===this.behavior.treeColumnIndex}},isHeaderRow:{get:function(){return this.subgrid.isHeader}},isHeaderHandle:{get:function(){return this.isHeaderRow&&this.isHandleColumn}},isHeaderCell:{get:function(){return this.isHeaderRow&&this.isDataColumn}},isFilterRow:{get:function(){return this.subgrid.isFilter}},isFilterHandle:{get:function(){return this.isFilterRow&&this.isHandleColumn}},isFilterCell:{get:function(){return this.isFilterRow&&this.isDataColumn}},isSummaryRow:{get:function(){return this.subgrid.isSummary}},isSummaryHandle:{get:function(){return this.isSummaryRow&&this.isHandleColumn}},isSummaryCell:{get:function(){return this.isSummaryRow&&this.isDataColumn}},isTopTotalsRow:{get:function(){return this.subgrid===this.behavior.subgrids.lookup.topTotals}},isTopTotalsHandle:{get:function(){return this.isTopTotalsRow&&this.isHandleColumn}},isTopTotalsCell:{get:function(){return this.isTopTotalsRow&&this.isDataColumn}},isBottomTotalsRow:{get:function(){return this.subgrid===this.behavior.subgrids.lookup.bottomTotals}},isBottomTotalsHandle:{get:function(){return this.isBottomTotalsRow&&this.isHandleColumn}},isBottomTotalsCell:{get:function(){return this.isBottomTotalsRow&&this.isDataColumn}},$$CLASS_NAME:{value:"CellEvent"}}),i=n(2).Point;function o(t,e){}o.prototype=i.prototype;var a={writable:!0},s={writable:!0,enumerable:!0};t.exports=function(t){function e(t,e){Object.defineProperties(this,{visibleColumn:a,visibleRow:a,subgrid:a,gridCell:{value:new o},dataCell:{value:new o},column:s,_columnProperties:a,_cellOwnProperties:a,_bounds:a,snapshot:a,minWidth:a,disabled:a}),arguments.length&&this.resetGridCY(t,e)}return e.prototype=Object.create(r,{constructor:{value:e},grid:{value:t},renderer:{value:t.renderer},selectionModel:{value:t.selectionModel},behavior:{value:t.behavior},dataModel:{value:t.behavior.dataModel}}),e}},function(t,e,n){"use strict";var r=n(29),i=/^__/,o={decorateSchema:function(t){return(new r).decorateArray(t),t},decorateColumnSchema:function(t,e){var n=new r;return n.transformations={},n.transformations[e]="header",n.decorateArray(t),t},normalizeSchema:function(t){t.forEach(function(e,n){"string"==typeof e&&(t[n]={name:e})});for(var e=t.length;e--;)i.test(t[e].name)&&t.splice(e,1);return t.forEach(function(t,e){t.index=e}),t},getFieldNames:function(t){return Object.keys(t||{}).filter(function(t){return!i.test(t)})},titleize:n(47).toTitle,getSchema:function(t){var e=t&&t[0]||{},n=o.getFieldNames(e);return o.normalizeSchema(n),o.decorateSchema(n),o.decorateColumnSchema(n,"toTitle"),n}};t.exports=o},function(t,e,n){"use strict";var r=n(30).extend("Features",{BaseClass:n(0),initialize:function(){this.add(r.CellClick),this.add(r.CellEditing),this.add(r.CellSelection),this.add(r.ColumnMoving),this.add(r.ColumnResizing),this.add(r.ColumnSelection),this.add(r.ColumnSorting),this.add(r.Filters),this.add(r.KeyPaging),this.add(r.OnHover),this.add(r.RowSelection),this.add(r.ThumbwheelScrolling)}});r.BaseClass=n(0),r.CellClick=n(124),r.CellEditing=n(125),r.CellSelection=n(127),r.ColumnMoving=n(128),r.ColumnResizing=n(129),r.ColumnSelection=n(130),r.ColumnSorting=n(131),r.Filters=n(132),r.KeyPaging=n(133),r.OnHover=n(134),r.RowSelection=n(135),r.ThumbwheelScrolling=n(136),t.exports=new r},function(t,e,n){"use strict";var r=n(0).extend("CellClick",{handleMouseMove:function(t,e){var n=e.properties.link,r=n&&"boolean"!=typeof n;this.cursor=r?"pointer":null,this.next&&this.next.handleMouseMove(t,e)},handleClick:function(t,e){(!e.isDataCell&&!e.isTreeColumn||void 0===this.openLink(t,e)&&!t.cellClicked(e))&&this.next&&this.next.handleClick(t,e)},openLink:function(t,e){var n,r,i=e.dataRow,o=Object.create(e.properties,{dataRow:{value:i}}),a=o.exec(e.value),s=e.properties.link,A=s instanceof Array,l=A?s[0]:s;switch(typeof l){case"string":"*"===l?r=a:/^\w+$/.test(l)&&(r=i[l]);break;case"function":r=l(e)}return r&&(r=r.toString().replace(/%name/g,o.name).replace(/%value/g,a),A?((s=s.slice())[0]=r,n=t.windowOpen.apply(t,s)):n=t.windowOpen(r,e.properties.linkTarget)),n&&(e.setCellProperty("linkColor",t.properties.linkVisitedColor),t.renderer.resetCellPropertiesCache(e),t.repaint()),n}});t.exports=r},function(t,e,n){"use strict";var r=n(0),i=n(12),o=r.extend("CellEditing",{handleClick:function(t,e){a.call(this,t,e)},handleDoubleClick:function(t,e){a.call(this,t,e,!0)},handleKeyDown:function(t,e){var n,r,o,a,s;(s=t.getGridCellFromLastSelection())&&s.properties.editOnKeydown&&!t.cellEditor&&("F2"===(n=e.detail.char)||(r=1===n.length&&!(e.detail.meta||e.detail.ctrl))||(o="DELETE"===n||"BACKSPACE"===n))?(a=t.onEditorActivate(s))instanceof i&&(r?a.input.value=n:o&&a.setEditorValue(""),e.detail.primitiveEvent.preventDefault()):this.next&&this.next.handleKeyDown(t,e)}});function a(t,e,n){!e.isDataCell||e.getCellProperty("editOnDoubleClick")^n||t.onEditorActivate(e),this.next&&this.next[n?"handleDoubleClick":"handleClick"](t,e)}t.exports=o},function(t,e,n){"use strict";e.shaker=function(t){var e=this,n=(t=t||{}).el||e.el,r=t.duration||"0.065s",i=window.getComputedStyle(n),o=i.transition.split(","),a=i.position,s=parseInt(i.left),A=-3,l=6;function c(r){r&&"left"!==r.propertyName||(n.style.left=s+A+"px",l--||(n.removeEventListener("transitionend",c),o.pop(),n.style.transition=o.join(","),n.style.position=a,t.callback&&t.callback.call(e,t)),A=l?-A:0)}o.push("left "+r),n.style.transition=o.join(","),n.addEventListener("transitionend",c),c()};var r={"background-color":"yellow","box-shadow":"0 0 10px red"};e.glower=function(t){var e=this,n=(t=t||{}).el||e.el,i=t.duration||"0.25s",o=t.styles||r,a=o.length,s=window.getComputedStyle(n),A={},l=s.transition,c=l.split(",");Object.keys(o).forEach(function(t){A[t]={style:s[t],undo:!0},c.push(t+" "+i)}),n.style.transition=c.join(","),n.addEventListener("transitionend",function r(i){var o=A[i.propertyName];o.undo?(n.style[i.propertyName]=o.style,o.undo=!1):--a||(n.removeEventListener("transitionend",r),n.style.transition=l,t.callback&&t.callback.call(e,t))}),Object.keys(o).forEach(function(t){n.style[t]=o[t]})}},function(t,e,n){"use strict";var r=n(0).extend("CellSelection",{currentDrag:null,lastDragCell:null,sbLastAuto:0,sbAutoStart:0,handleMouseUp:function(t,e){this.dragging&&(this.dragging=!1),this.next&&this.next.handleMouseUp(t,e)},handleMouseDown:function(t,e){var n=e.gridCell.x,r=e.dataCell.y;if(t.behavior.getCellProperty(e.dataCell.x,e.gridCell.y,"cellSelection")&&e.isDataCell&&!e.primitiveEvent.detail.isRightClick){var i=t.newPoint(n,r),o=e.primitiveEvent.detail.keys;this.dragging=!0,this.extendSelection(t,i,o)}else this.next&&this.next.handleMouseDown(t,e)},handleMouseDrag:function(t,e){this.dragging&&t.properties.cellSelection&&!e.primitiveEvent.detail.isRightClick?(this.currentDrag=e.primitiveEvent.detail.mouse,this.lastDragCell=t.newPoint(e.gridCell.x,e.dataCell.y),this.checkDragScroll(t,this.currentDrag),this.handleMouseDragCellSelection(t,this.lastDragCell,e.primitiveEvent.detail.keys)):this.next&&this.next.handleMouseDrag(t,e)},handleKeyDown:function(t,e){var n=e.detail,r=t.getGridCellFromLastSelection(!0),i=this["handle"+(r&&(r.properties.mappedNavKey(n.char,n.ctrl)||r.properties.navKey(n.char,n.ctrl)))];i?(i.call(this,t,n),r.properties.editOnNextCell&&(t.renderer.computeCellsBounds(!0),r=t.getGridCellFromLastSelection(),t.editAt(r)),t.cellEditor||t.takeFocus()):this.next&&this.next.handleKeyDown(t,e)},handleMouseDragCellSelection:function(t,e,n){var r=Math.max(0,e.x),i=Math.max(0,e.y),o=t.getDragExtent(),a=t.getMouseDown(),s=r-a.x,A=i-a.y;o.x===s&&o.y===A||(t.clearMostRecentSelection(),t.select(a.x,a.y,s,A),t.setDragExtent(t.newPoint(s,A)),t.repaint())},checkDragScroll:function(t,e){t.properties.scrollingEnabled&&(t.getDataBounds().contains(e)?t.isScrollingNow()&&t.setScrollingNow(!1):t.isScrollingNow()||(t.setScrollingNow(!0),this.scrollDrag(t)))},scrollDrag:function(t){if(t.isScrollingNow()){var e=t.isMouseDownInHeaderArea(),n=this.lastDragCell,r=t.getDataBounds(),i=0,o=0,a=t.getFixedColumnCount(),s=t.getFixedRowCount(),A=n.x<a,l=n.y<s;e||(this.currentDrag.x<r.origin.x&&(i=-1),this.currentDrag.y<r.origin.y&&(o=-1)),this.currentDrag.x>r.origin.x+r.extent.x&&(i=1),this.currentDrag.y>r.origin.y+r.extent.y&&(o=1);var c=i,u=o;A&&(c=0),l&&(u=0),this.lastDragCell=n.plusXY(c,u),t.scrollBy(i,o),this.handleMouseDragCellSelection(t,n,[]),t.repaint(),setTimeout(this.scrollDrag.bind(this,t),25)}},extendSelection:function(t,e,n){var r=n.indexOf("CTRL")>=0,i=n.indexOf("SHIFT")>=0,o=t.getMouseDown(),a=e.x,s=e.y;if(!(a<0||s<0)){if(r&&a===o.x&&s===o.y)return t.clearMostRecentSelection(),t.popMouseDown(),void t.repaint();r||i||t.clearSelections(),i?(t.clearMostRecentSelection(),t.select(o.x,o.y,a-o.x,s-o.y),t.setDragExtent(t.newPoint(a-o.x,s-o.y))):(t.select(a,s,0,0),t.setMouseDown(t.newPoint(a,s)),t.setDragExtent(t.newPoint(0,0))),t.repaint()}},handleDOWNSHIFT:function(t){this.moveShiftSelect(t,0,1)},handleUPSHIFT:function(t){this.moveShiftSelect(t,0,-1)},handleLEFTSHIFT:function(t){this.moveShiftSelect(t,-1,0)},handleRIGHTSHIFT:function(t){this.moveShiftSelect(t,1,0)},handleDOWN:function(t,e){e.primitiveEvent.preventDefault();var n=this.getAutoScrollAcceleration();t.moveSingleSelect(0,n)},handleUP:function(t,e){e.primitiveEvent.preventDefault();var n=this.getAutoScrollAcceleration();t.moveSingleSelect(0,-n)},handleLEFT:function(t){t.moveSingleSelect(-1,0)},handleRIGHT:function(t){t.moveSingleSelect(1,0)},getAutoScrollAcceleration:function(){var t=this.getAutoScrollDuration()/2e3;return Math.max(1,Math.floor(t*t*t*t))},setAutoScrollStartTime:function(){this.sbAutoStart=Date.now()},pingAutoScroll:function(){Date.now()-this.sbLastAuto>500&&this.setAutoScrollStartTime(),this.sbLastAuto=Date.now()},getAutoScrollDuration:function(){return Date.now()-this.sbLastAuto>500?0:Date.now()-this.sbAutoStart},moveShiftSelect:function(t,e,n){t.extendSelect(e,n)&&this.pingAutoScroll()}});t.exports=r},function(t,e,n){"use strict";var r,i,o,a,s=n(0),A=["grab","-moz-grab","-webkit-grab"],l=["grabbing","-moz-grabbing","-webkit-grabbing"],c=function(t){this.cursor=t},u=s.extend("ColumnMoving",{floaterAnimationQueue:[],columnDragAutoScrollingRight:!1,columnDragAutoScrollingLeft:!1,dragArmed:!1,dragging:!1,dragCol:-1,dragOffset:0,initializeOn:function(t){this.isFloatingNow=!1,this.initializeAnimationSupport(t),this.next&&this.next.initializeOn(t)},initializeAnimationSupport:function(t){r||((r=document.createElement("canvas")).setAttribute("width","0px"),r.setAttribute("height","0px"),r.style.position="fixed",document.body.appendChild(r),i=r.getContext("2d",{alpha:!1})),o||((o=document.createElement("canvas")).setAttribute("width","0px"),o.setAttribute("height","0px"),o.style.position="fixed",document.body.appendChild(o),a=o.getContext("2d",{alpha:!1}))},handleMouseDrag:function(t,e){var n,r=e.gridCell;Math.abs(e.primitiveEvent.detail.dragstart.x-e.primitiveEvent.detail.mouse.x)<10||e.isColumnFixed?this.next&&this.next.handleMouseDrag(t,e):(e.isHeaderCell&&this.dragArmed&&!this.dragging?(this.dragging=!0,this.dragCol=r.x,this.dragOffset=e.mousePoint.x,this.detachChain(),n=e.primitiveEvent.detail.mouse.x-this.dragOffset,this.createDragColumn(t,n,this.dragCol)):this.next&&this.next.handleMouseDrag(t,e),this.dragging&&(n=e.primitiveEvent.detail.mouse.x-this.dragOffset,this.dragColumn(t,n)))},handleMouseDown:function(t,e){t.properties.columnsReorderable&&!e.primitiveEvent.detail.isRightClick&&!e.isColumnFixed&&e.isHeaderCell?(this.dragArmed=!0,this.cursor=l,t.clearSelections()):this.next&&this.next.handleMouseDown(t,e)},handleMouseUp:function(t,e){if(this.dragging){this.cursor=null;var n=this;this.endDragColumn(t),setTimeout(function(){n.attachChain()},200)}this.dragCol=-1,this.dragging=!1,this.dragArmed=!1,this.cursor=null,t.repaint(),this.next&&this.next.handleMouseUp(t,e)},handleMouseMove:function(t,e){t.properties.columnsReorderable&&!e.isColumnFixed&&!this.dragging&&e.isHeaderCell&&e.mousePoint.y<t.properties.columnGrabMargin?this.cursor=A:this.cursor=null,this.next&&this.next.handleMouseMove(t,e),e.isHeaderCell&&this.dragging&&(this.cursor=l)},floatColumnTo:function(t,e){this.floatingNow=!0;var n,r,i=t.renderer.visibleColumns,o=t.getHScrollValue(),a=t.renderOverridesCache.floater.columnIndex,s=t.renderOverridesCache.dragger.columnIndex,A=t.renderOverridesCache.dragger.hdpiratio,l=t.getFixedColumnCount(),c=t.getColumnWidth(s),u=t.getColumnWidth(a),g=t.getVisibleColumnsCount(),h=0,d=0;s>=l&&(h=o),a>=l&&(d=o),e?(n=i[Math.min(g,s-h)].left,r=i[Math.min(g,a-d)].left,t.renderOverridesCache.dragger.startX=(n+u)*A,t.renderOverridesCache.floater.startX=n*A):(n=(r=i[Math.min(g,a-d)].left)+c,t.renderOverridesCache.dragger.startX=r*A,t.renderOverridesCache.floater.startX=n*A),t.swapColumns(s,a),t.renderOverridesCache.dragger.columnIndex=a,t.renderOverridesCache.floater.columnIndex=s,this.floaterAnimationQueue.unshift(this.doColumnMoveAnimation(t,r,n)),this.doFloaterAnimation(t)},doColumnMoveAnimation:function(t,e,n){var r=this;return function(){var i=o;i.style.display="inline",r.setCrossBrowserProperty(i,"transform","translate("+e+"px, 0px)"),requestAnimationFrame(function(){r.setCrossBrowserProperty(i,"transition",(r.isWebkit?"-webkit-":"")+"transform 150ms ease"),r.setCrossBrowserProperty(i,"transform","translate("+n+"px, "+-2+"px)")}),t.repaint(),setTimeout(function(){r.setCrossBrowserProperty(i,"transition",""),t.renderOverridesCache.floater=null,t.repaint(),r.doFloaterAnimation(t),requestAnimationFrame(function(){i.style.display="none",r.isFloatingNow=!1})},200)}},doFloaterAnimation:function(t){if(0===this.floaterAnimationQueue.length)return this.floatingNow=!1,void t.repaint();this.floaterAnimationQueue.pop()()},createFloatColumn:function(t,e){var n=t.getFixedColumnCount(),r=t.getHScrollValue();e<n&&(r=0);var i=t.getColumnWidth(e),s=t.div.clientHeight,A=o,u=A.style,g=t.div.getBoundingClientRect();u.top=g.top-2+"px",u.left=g.left+"px";var h=t.getHiDPI(a);A.setAttribute("width",Math.round(i*h)+"px"),A.setAttribute("height",Math.round(s*h)+"px"),u.boxShadow="0 10px 20px rgba(0,0,0,0.19), 0 6px 6px rgba(0,0,0,0.23)",u.width=i+"px",u.height=s+"px",u.borderTop="1px solid "+t.properties.lineColor,u.backgroundColor=t.properties.backgroundColor;var d=t.renderer.visibleColumns[e-r].left*h;a.scale(h,h),t.renderOverridesCache.floater={columnIndex:e,ctx:a,startX:d,width:i,height:s,hdpiratio:h},u.zIndex="4",this.setCrossBrowserProperty(A,"transform","translate("+d+"px, "+-2+"px)"),l.forEach(c,u),t.repaint()},setCrossBrowserProperty:function(t,e,n){var r=e[0].toUpperCase()+e.substr(1);this.setProp(t,"webkit"+r,n),this.setProp(t,"Moz"+r,n),this.setProp(t,"ms"+r,n),this.setProp(t,"O"+r,n),this.setProp(t,e,n)},setProp:function(t,e,n){e in t.style&&(t.style[e]=n)},createDragColumn:function(t,e,n){var o=t.getFixedColumnCount(),a=t.getHScrollValue();n<o&&(a=0);var s=t.getHiDPI(i),A=t.getColumnWidth(n),u=t.div.clientHeight,g=r,h=t.div.getBoundingClientRect(),d=g.style;d.top=h.top+"px",d.left=h.left+"px",d.opacity=.85,d.boxShadow="0 19px 38px rgba(0,0,0,0.30), 0 15px 12px rgba(0,0,0,0.22)",d.borderTop="1px solid "+t.properties.lineColor,d.backgroundColor=t.properties.backgroundColor,g.setAttribute("width",Math.round(A*s)+"px"),g.setAttribute("height",Math.round(u*s)+"px"),d.width=A+"px",d.height=u+"px";var f=t.renderer.visibleColumns[n-a].left*s;i.scale(s,s),t.renderOverridesCache.dragger={columnIndex:n,startIndex:n,ctx:i,startX:f,width:A,height:u,hdpiratio:s},this.setCrossBrowserProperty(g,"transform","translate("+e+"px, -5px)"),d.zIndex="5",l.forEach(c,d),t.repaint()},dragColumn:function(t,e){var n=this.columnDragAutoScrollingRight||this.columnDragAutoScrollingLeft,o=t.getHiDPI(i),a=t.renderOverridesCache.dragger.columnIndex,s=t.renderer.getFinalVisibleColumnBoundary();e=Math.min(e,s+15);var A=(e=Math.max(-15,e))<0&&0!==a,l=e>s,c=r;this.setCrossBrowserProperty(c,"transition",(this.isWebkit?"-webkit-":"")+"transform 0ms ease, box-shadow 150ms ease"),this.setCrossBrowserProperty(c,"transform","translate("+e+"px, "+-10+"px)"),requestAnimationFrame(function(){c.style.display="inline"});var u=t.renderer.getColumnFromPixelX(e+c.width/2/o);A&&(u=0),l&&(u=t.getColumnCount()-1);var g=a>u;if((g=g||u-a>=1)&&!n){var h=a<u;if(this.isFloatingNow)return;this.isFloatingNow=!0,this.createFloatColumn(t,u),this.floatColumnTo(t,h)}else{if(e<-10&&this.checkAutoScrollToLeft(t,e),e>-10&&(this.columnDragAutoScrollingLeft=!1),l||e>s+10)return void this.checkAutoScrollToRight(t,e);e<s+10&&(this.columnDragAutoScrollingRight=!1)}},checkAutoScrollToRight:function(t,e){this.columnDragAutoScrollingRight||(this.columnDragAutoScrollingRight=!0,this._checkAutoScrollToRight(t,e))},_checkAutoScrollToRight:function(t,e){if(this.columnDragAutoScrollingRight){var n=t.getHScrollValue();if(t.dragging&&!(n>t.sbHScroller.range.max-2)){var r=t.renderOverridesCache.dragger.columnIndex;t.scrollBy(1,0);var i=r+1;t.swapColumns(i,r),t.renderOverridesCache.dragger.columnIndex=i,setTimeout(this._checkAutoScrollToRight.bind(this,t,e),250)}}},checkAutoScrollToLeft:function(t,e){this.columnDragAutoScrollingLeft||(this.columnDragAutoScrollingLeft=!0,this._checkAutoScrollToLeft(t,e))},_checkAutoScrollToLeft:function(t,e){if(this.columnDragAutoScrollingLeft){var n=t.getHScrollValue();if(t.dragging&&!(n<1)){var r=t.renderOverridesCache.dragger.columnIndex;t.swapColumns(r+n,r+n-1),t.scrollBy(-1,0),setTimeout(this._checkAutoScrollToLeft.bind(this,t,e),250)}}},endDragColumn:function(t){var e=t.getFixedColumnCount(),n=t.getHScrollValue(),i=t.renderOverridesCache.dragger.columnIndex;i<e&&(n=0);var o=t.renderer.visibleColumns[i-n].left,a=r,s=t.renderOverridesCache.dragger.startIndex!==t.renderOverridesCache.dragger.columnIndex;this.setCrossBrowserProperty(a,"transition",(this.isWebkit?"-webkit-":"")+"transform 150ms ease, box-shadow 150ms ease"),this.setCrossBrowserProperty(a,"transform","translate("+o+"px, "+-1+"px)"),a.style.boxShadow="0px 0px 0px #888888",setTimeout(function(){t.renderOverridesCache.dragger=null,t.repaint(),requestAnimationFrame(function(){a.style.display="none",t.endDragColumnNotification(),s&&t.fireSyntheticOnColumnsChangedEvent()})},200)}});t.exports=u},function(t,e,n){"use strict";var r=n(0).extend("ColumnResizing",{dragStart:-1,dragStartWidth:-1,getMouseValue:function(t){return t.primitiveEvent.detail.mouse.x},overAreaDivider:function(t,e){var n=t.behavior.leftMostColIndex;return e.gridCell.x!==n&&e.mousePoint.x<=3||e.mousePoint.x>=e.bounds.width-3},getCursorName:function(){return"col-resize"},handleMouseDrag:function(t,e){if(this.dragColumn){var n=this.getMouseValue(e)-this.dragStart,r=this.dragStartWidth+n,i=this.nextStartWidth-n;if(this.nextColumn){var o=this.nextColumn.properties,a=this.dragColumn.properties;(0<n&&n<=this.nextStartWidth-o.minimumColumnWidth&&(!a.maximumColumnWidth||r<=a.maximumColumnWidth)||0>n&&n>=-(this.dragStartWidth-a.minimumColumnWidth)&&(!o.maximumColumnWidth||i<o.maximumColumnWidth))&&(t.behavior.setColumnWidth(this.dragColumn,r),t.behavior.setColumnWidth(this.nextColumn,i))}else t.behavior.setColumnWidth(this.dragColumn,r)}else this.next&&this.next.handleMouseDrag(t,e)},handleMouseDown:function(t,e){if(e.isHeaderRow&&this.overAreaDivider(t,e)){var n=e.gridCell.x;if(e.mousePoint.x<=3){n-=1;var r=t.renderer.visibleColumns[n]||t.renderer.visibleColumns[n-1];if(!r)return;this.dragColumn=r.column,this.dragStartWidth=r.width}else this.dragColumn=e.column,this.dragStartWidth=e.bounds.width;this.dragStart=this.getMouseValue(e),this.dragColumn.properties.resizeColumnInPlace?(n+=1,(r=t.renderer.visibleColumns[n]||t.renderer.visibleColumns[n+1])?(this.nextColumn=r.column,this.nextStartWidth=this.nextColumn.getWidth()):this.nextColumn=void 0):this.nextColumn=void 0}else this.next&&this.next.handleMouseDown(t,e)},handleMouseUp:function(t,e){this.dragColumn?(this.cursor=null,this.dragColumn=!1,e.primitiveEvent.stopPropagation(),t.behaviorShapeChanged()):this.next&&this.next.handleMouseUp(t,e)},handleMouseMove:function(t,e){this.dragColumn||(this.cursor=null,this.next&&this.next.handleMouseMove(t,e),this.cursor=e.isHeaderRow&&this.overAreaDivider(t,e)?this.getCursorName():null)},handleDoubleClick:function(t,e){if(e.isHeaderRow&&this.overAreaDivider(t,e)){var n=e.mousePoint.x<=3?t.behavior.getActiveColumn(e.gridCell.x-1):e.column;n.addProperties({columnAutosizing:!0,columnAutosized:!1}),setTimeout(function(){t.autosizeColumn(n)})}else this.next&&this.next.handleDoubleClick(t,e)}});t.exports=r},function(t,e,n){"use strict";var r=n(0).extend("ColumnSelection",{currentDrag:null,lastDragColumn:null,sbLastAuto:0,sbAutoStart:0,handleMouseUp:function(t,e){this.dragging&&(this.dragging=!1),this.next&&this.next.handleMouseUp(t,e)},handleDoubleClick:function(t,e){this.doubleClickTimer&&(clearTimeout(this.doubleClickTimer),this.doubleClickTimer=void 0),this.next&&this.next.handleDoubleClick(t,e)},handleMouseDown:function(t,e){this.doubleClickTimer||(t.properties.columnSelection&&!e.primitiveEvent.detail.isRightClick&&(t.properties.autoSelectColumns||e.isHeaderCell&&e.mousePoint.y>=5)?this.doubleClickTimer=setTimeout(function(t,e){this.doubleClickTimer=void 0,this.dragging=!0,this.extendSelection(t,e.gridCell.x,e.primitiveEvent.detail.keys)}.bind(this,t,e),function(t,e){var n;return e.isHeaderCell&&!(n=e.columnProperties).unsortable&&n.sortOnDoubleClick&&300}.call(this,t,e)):this.next&&this.next.handleMouseDown(t,e))},handleMouseDrag:function(t,e){t.properties.columnSelection&&!this.isColumnDragging(t)&&!e.primitiveEvent.detail.isRightClick&&this.dragging?(this.lastDragColumn=e.gridCell.x,this.currentDrag=e.primitiveEvent.detail.mouse,this.checkDragScroll(t,this.currentDrag),this.handleMouseDragCellSelection(t,this.lastDragColumn,e.primitiveEvent.detail.keys)):this.next&&this.next.handleMouseDrag(t,e)},handleKeyDown:function(t,e){var n=e.detail,r="column"===t.getLastSelectionType()&&this["handle"+n.char];r?r.call(this,t,n):this.next&&this.next.handleKeyDown(t,e)},handleMouseDragCellSelection:function(t,e,n){var r=t.getMouseDown().x;t.clearMostRecentColumnSelection(),t.selectColumn(r,e),t.setDragExtent(t.newPoint(e-r,0)),t.repaint()},checkDragScroll:function(t,e){t.properties.scrollingEnabled&&t.getDataBounds().contains(e)?t.isScrollingNow()&&t.setScrollingNow(!1):t.isScrollingNow()||(t.setScrollingNow(!0),this.scrollDrag(t))},scrollDrag:function(t){if(t.isScrollingNow()){var e,n=t.getDataBounds();this.currentDrag.x<n.origin.x?e=-1:this.currentDrag.x>n.origin.x+n.extent.x&&(e=1),e&&(this.lastDragColumn>=t.getFixedColumnCount()&&(this.lastDragColumn+=e),t.scrollBy(e,0)),this.handleMouseDragCellSelection(t,this.lastDragColumn,[]),t.repaint(),setTimeout(this.scrollDrag.bind(this,t),25)}},extendSelection:function(t,e,n){if(t.abortEditing()){var r=t.getMouseDown().x,i=n.indexOf("SHIFT")>0;e<0||(i?(t.clearMostRecentColumnSelection(),t.selectColumn(e,r),t.setDragExtent(t.newPoint(e-r,0))):(t.toggleSelectColumn(e,n),t.setMouseDown(t.newPoint(e,0)),t.setDragExtent(t.newPoint(0,0))),t.repaint())}},handleDOWNSHIFT:function(t){},handleUPSHIFT:function(t){},handleLEFTSHIFT:function(t){this.moveShiftSelect(t,-1)},handleRIGHTSHIFT:function(t){this.moveShiftSelect(t,1)},handleDOWN:function(t){},handleUP:function(t){},handleLEFT:function(t){this.moveSingleSelect(t,-1)},handleRIGHT:function(t){this.moveSingleSelect(t,1)},getAutoScrollAcceleration:function(){var t=this.getAutoScrollDuration()/2e3;return Math.max(1,Math.floor(t*t*t*t))},setAutoScrollStartTime:function(){this.sbAutoStart=Date.now()},pingAutoScroll:function(){Date.now()-this.sbLastAuto>500&&this.setAutoScrollStartTime(),this.sbLastAuto=Date.now()},getAutoScrollDuration:function(){return Date.now()-this.sbLastAuto>500?0:Date.now()-this.sbAutoStart},moveShiftSelect:function(t,e){var n=t.getMouseDown(),r=t.getDragExtent().x+e,i=t.renderer.visibleColumns.length-1,o=t.getColumnCount()-1;t.properties.scrollingEnabled||(o=Math.min(o,i)),r=Math.min(o-n.x,Math.max(-n.x,r)),t.clearMostRecentColumnSelection(),t.selectColumn(n.x,n.x+r),t.setDragExtent(t.newPoint(r,0)),t.insureModelColIsVisible(r+n.x,e)&&this.pingAutoScroll(),t.repaint()},moveSingleSelect:function(t,e){var n=t.getDragExtent(),r=t.getMouseDown().plus(n).x+e,i=t.getColumnCount()-1,o=t.getVisibleColumnsCount()-1;t.properties.scrollingEnabled||(i=Math.min(i,o)),r=Math.min(i,Math.max(0,r)),t.clearSelections(),t.selectColumn(r),t.setMouseDown(t.newPoint(r,0)),t.setDragExtent(t.newPoint(0,0)),t.insureModelColIsVisible(r,e)&&this.pingAutoScroll(),t.repaint()},isColumnDragging:function(t){var e=t.lookupFeature("ColumnMoving");return e&&(e.dragging||e.dragArmed)}});t.exports=r},function(t,e,n){"use strict";var r=n(0).extend("ColumnSorting",{handleClick:function(t,e){i.call(this,t,e)},handleDoubleClick:function(t,e){i.call(this,t,e,!0)},handleMouseMove:function(t,e){var n;e.isRowFixed&&e.isHeaderCell&&(n=t.behavior.getColumnProperties(e.gridCell.x))&&!n.unsortable?this.cursor="pointer":this.cursor=null,this.next&&this.next.handleMouseMove(t,e)}});function i(t,e,n){var r;!e.isHeaderCell||(r=e.columnProperties).unsortable||r.sortOnDoubleClick^n||t.fireSyntheticColumnSortEvent(e.gridCell.x,e.primitiveEvent.detail.keys),this.next&&this.next[n?"handleDoubleClick":"handleClick"](t,e)}t.exports=r},function(t,e,n){"use strict";var r=n(0).extend("Filters",{handleKeyDown:function(t,e){var n,r,i=e.detail;i.editor&&(n=i.editor.event).isFilterCell&&(r=this["handle"+n.properties.mappedNavKey(i.char)]),r?r.call(this,t,i):this.next&&this.next.handleKeyDown(t,e)},handleLEFT:function(t,e){i(t,e,-1)},handleRIGHT:function(t,e){i(t,e,1)},handleUP:o,handleDOWN:o,handleDoubleClick:function(t,e){e.isFilterCell?t.onEditorActivate(e):this.next&&this.next.handleDoubleClick(t,e)},handleClick:function(t,e){e.isFilterCell?t.onEditorActivate(e):this.next&&this.next.handleClick(t,e)}});function i(t,e,n){var r=e.editor.event,i=r.visibleColumn.index,a=r.visibleRow.index,s=i,A=t.renderer.visibleColumns.length;for(r=new t.behavior.CellEvent;(i=(i+n+A)%A)!==s&&r.resetGridXY(i,a);)if(r.properties.filterable)return void t.editAt(r);o(t,r)}function o(t,e){var n=e.editor.event.visibleColumn.index;t.selectViewportCell(n,0),t.takeFocus()}t.exports=r},function(t,e,n){"use strict";var r=n(0),i={PAGEDOWN:function(t){t.pageDown()},PAGEUP:function(t){t.pageUp()},PAGELEFT:function(t){t.pageLeft()},PAGERIGHT:function(t){t.pageRight()}},o=r.extend("KeyPaging",{handleKeyDown:function(t,e){var n=i[e.detail.char];n?n(t):this.next&&this.next.handleKeyDown(t,e)}});t.exports=o},function(t,e,n){"use strict";var r=n(0).extend("OnHover",{handleMouseMove:function(t,e){var n=t.hoverCell;e.gridCell.equals(n)?this.next&&this.next.handleMouseMove(t,e):(n&&this.handleMouseExit(t,n),this.handleMouseEnter(t,e),t.setHoverCell(e))}});t.exports=r},function(t,e,n){"use strict";var r=n(0).extend("RowSelection",{currentDrag:null,lastDragCell:null,sbLastAuto:0,sbAutoStart:0,dragArmed:!1,handleMouseUp:function(t,e){this.dragArmed?(this.dragArmed=!1,i(t),t.fireSyntheticRowSelectionChangedEvent()):this.dragging?(this.dragging=!1,i(t),t.fireSyntheticRowSelectionChangedEvent()):this.next&&this.next.handleMouseUp(t,e)},handleMouseDown:function(t,e){var n=!e.primitiveEvent.detail.isRightClick,r=n&&t.properties.showRowNumbers&&e.isHandleColumn&&e.mousePointInClickRect;if(!r||t.fireSyntheticRowHeaderClickedEvent(e)){var i=t.properties.rowSelection&&(n&&t.properties.autoSelectRows||r);i&&e.isHeaderHandle?t.toggleSelectAllRows():i&&e.isDataRow?(this.dragArmed=!0,this.extendSelection(t,e.dataCell.y,e.primitiveEvent.detail.keys)):this.next&&this.next.handleMouseDown(t,e)}},handleMouseDrag:function(t,e){this.dragArmed&&t.properties.rowSelection&&!e.primitiveEvent.detail.isRightClick?(this.lastDragRow=e.dataCell.y,this.dragging=!0,this.currentDrag=e.primitiveEvent.detail.mouse,this.checkDragScroll(t,this.currentDrag),this.handleMouseDragCellSelection(t,this.lastDragRow,e.primitiveEvent.detail.keys)):this.next&&this.next.handleMouseDrag(t,e)},handleKeyDown:function(t,e){var n;"row"===t.getLastSelectionType()&&(n=this["handle"+e.detail.char])?n.call(this,t,e.detail):this.next&&this.next.handleKeyDown(t,e)},handleMouseDragCellSelection:function(t,e,n){var r=t.getMouseDown().y;t.clearMostRecentRowSelection(),t.selectRow(r,e),t.setDragExtent(t.newPoint(0,e-r)),t.repaint()},checkDragScroll:function(t,e){t.properties.scrollingEnabled&&t.getDataBounds().contains(e)?t.isScrollingNow()&&t.setScrollingNow(!1):t.isScrollingNow()||(t.setScrollingNow(!0),this.scrollDrag(t))},scrollDrag:function(t){if(t.isScrollingNow()){var e,n=t.getDataBounds();this.currentDrag.y<n.origin.y?e=-1:this.currentDrag.y>n.origin.y+n.extent.y&&(e=1),e&&(this.lastDragRow>=t.getFixedRowCount()&&(this.lastDragRow+=e),t.scrollBy(0,e)),this.handleMouseDragCellSelection(t,this.lastDragRow,[]),t.repaint(),setTimeout(this.scrollDrag.bind(this,t),25)}},extendSelection:function(t,e,n){if(t.abortEditing()){var r=t.getMouseDown().y,i=-1!==n.indexOf("SHIFT");e<0||(i?(t.clearMostRecentRowSelection(),t.selectRow(e,r),t.setDragExtent(t.newPoint(0,e-r))):(t.toggleSelectRow(e,n),t.setMouseDown(t.newPoint(0,e)),t.setDragExtent(t.newPoint(0,0))),t.repaint())}},handleDOWNSHIFT:function(t){this.moveShiftSelect(t,1)},handleUPSHIFT:function(t){this.moveShiftSelect(t,-1)},handleLEFTSHIFT:function(t){},handleRIGHTSHIFT:function(t){},handleDOWN:function(t){this.moveSingleSelect(t,1)},handleUP:function(t){this.moveSingleSelect(t,-1)},handleLEFT:function(t){},handleRIGHT:function(t){var e=t.getMouseDown().plus(t.getDragExtent()),n=t.getColumnCount()-1,r=t.getHScrollValue(),i=e.y;r=Math.min(n,r),t.clearSelections(),t.select(r,i,0,0),t.setMouseDown(t.newPoint(r,i)),t.setDragExtent(t.newPoint(0,0)),t.repaint()},getAutoScrollAcceleration:function(){var t=this.getAutoScrollDuration()/2e3;return Math.max(1,Math.floor(t*t*t*t))},setAutoScrollStartTime:function(){this.sbAutoStart=Date.now()},pingAutoScroll:function(){Date.now()-this.sbLastAuto>500&&this.setAutoScrollStartTime(),this.sbLastAuto=Date.now()},getAutoScrollDuration:function(){return Date.now()-this.sbLastAuto>500?0:Date.now()-this.sbAutoStart},moveShiftSelect:function(t,e){this.moveSingleSelect(t,e,!0)},moveSingleSelect:function(t,e,n){var r=t.selectionModel.rowSelectionModel.selection,o=r[r.length-1],a=o[0],s=o[1];if(n){var A=o.offsetY=o.offsetY||e;o.offsetY<0?a+=e:s+=e}else a+=e,s+=e;a<0||s>=t.getRowCount()||(r.length-=1,r.length&&delete(o=r[r.length-1]).offsetY,t.selectRow(a,s),n&&a!==s&&((o=r[r.length-1]).offsetY=A),t.setMouseDown(t.newPoint(0,a)),t.setDragExtent(t.newPoint(0,s-a)),t.scrollToMakeVisible(t.properties.fixedColumnCount,e<0?a:s+1),i(t),t.fireSyntheticRowSelectionChangedEvent(),t.repaint())},isSingleRowSelection:function(){return!0}});function i(t){var e;if(t.properties.collapseCellSelections&&t.properties.singleRowSelectionMode&&!t.properties.multipleSelections&&(e=t.getSelectedRows()).length&&t.selectionModel.getSelections().length){var n=t.selectionModel.getLastSelection(),r=n.left,i=e[0],o=n.right-r;t.selectionModel.select(r,i,o,0,!1),t.repaint()}}t.exports=r},function(t,e,n){"use strict";var r=n(0).extend("ThumbwheelScrolling",{handleWheelMoved:function(t,e){if(t.properties.scrollingEnabled){var n=e.primitiveEvent,r=Math.sign(n.wheelDeltaX||-n.deltaX),i=Math.sign(n.wheelDeltaY||-n.deltaY);(r||i)&&t.scrollBy(-r||0,-i||0)}}});t.exports=r},function(t,e,n){"use strict";e.mixin={getCellOwnProperties:function(t,e,n){return 1===arguments.length?t.column.getCellOwnProperties(t.dataCell.y,t.subgrid):this.getColumn(t).getCellOwnProperties(e,n)},getCellProperties:function(t,e,n){return 1===arguments.length?t.properties:this.getColumn(t).getCellProperties(e,n)},getCellProperty:function(t,e,n,r){return"object"==typeof t?(n=e,t.properties[n]):this.getColumn(t).getCellProperty(e,n,r)},setCellProperties:function(t,e,n,r){return"object"==typeof t?(n=e,t.column.setCellProperties(t.dataCell.y,n,t.subgrid)):this.getColumn(t).setCellProperties(e,n,r)},addCellProperties:function(t,e,n,r){return"object"==typeof t?(n=e,t.column.addCellProperties(t.dataCell.y,n,t.subgrid)):this.getColumn(t).addCellProperties(e,n,r)},setCellProperty:function(t,e,n,r,i){var o;return"object"==typeof t?(r=n,n=e,o=t.setCellProperty(n,r)):(o=this.getColumn(t).setCellProperty(e,n,r,i),this.grid.renderer.resetCellPropertiesCache(t,e,i)),o}}},function(t,e,n){"use strict";e.mixin={getSchema:function(){return this.dataModel.getSchema()},setSchema:function(t){this.dataModel.setSchema(t)},getRowCount:function(){return this.dataModel.getRowCount()},getValue:function(t,e,n){if(n)return n.getValue(t,e);var r=new this.CellEvent;return r.resetDataXY(t,e,n)?r.value:void 0},setValue:function(t,e,n,r){if(r)r.setValue(t,e,n);else{var i=new this.CellEvent;i.resetDataXY(t,e,r)&&(i.value=n)}},reindex:function(){this.dataModel.apply()},getRow:function(t){return this.dataModel.getRow(t)},getData:function(){return this.dataModel.getData()}}},function(t,e,n){"use strict";var r=n(140);function i(t){var e;switch(typeof t){case"string":e=r.get(t);break;case"function":e=t;break;default:throw new this.HypergridError("Expected subgrid ref to be registered name or constructor, but found "+typeof t+".")}return e}e.mixin={set subgrids(t){var e=this._subgrids=[];e.lookup={},t.forEach(function(t){t&&e.push(this.createSubgrid(t))},this),this.shapeChanged()},get subgrids(){return this._subgrids},createSubgrid:function(t,e){var n,r,o;"data"===t?n=this.dataModel:t instanceof Array&&t.length?(r=i.call(this,t[0]),o=t.slice(1),n=this.createApply(r,o,void 0,{grid:this.grid}),n=this.decorateDataModel(n)):"object"==typeof t?n=t:(r=i.call(this,t),o=Array.prototype.slice.call(arguments,1),n=this.createApply(r,o,void 0,{grid:this.grid}),n=this.decorateDataModel(n)),n.type||(n.type="data");var a="data"===n.type&&n.type||n.name||n.type;return this._subgrids.lookup[a]=this._subgrids.lookup[a]||n,n["is"+n.type[0].toUpperCase()+n.type.substr(1)]=!0,n},getHeaderRowCount:function(){var t=0;return this.subgrids.find(function(e){if(e.isData)return!0;t+=e.getRowCount()}),t},getFooterRowCount:function(){var t;return this.subgrids.reduce(function(e,n){return t&&!n.isData?e+=n.getRowCount():t=n.isData,e},0)},getLogicalRowCount:function(){return this.subgrids.reduce(function(t,e){return t+e.getRowCount()},0)}}},function(t,e,n){"use strict";var r=n(30).extend("DataModels",{BaseClass:n(31),initialize:function(){this.add(n(141))}});t.exports=new r},function(t,e,n){"use strict";var r=n(31).extend("HeaderSubgrid",{type:"header",initialize:function(t,e){this.grid=e.grid},getRowCount:function(){return this.grid.properties.showHeaderRow?1:0},getValue:function(t,e){var n=this.grid.behavior.getColumn(t);return n.header||n.name},setValue:function(t,e,n){e<this.getRowCount()&&(this.grid.behavior.getColumn(t).header=n)},getRow:function(t){return this.dataRow}});t.exports=r},function(t,e,n){"use strict";function r(t){const e=document.createElement("div");return e.innerHTML=t,Array.prototype.slice.call(e.children)[0]}function i(t,e,n){const i=r(t);!function(t){if(t.content)return;let e;for(t.content=document.createDocumentFragment();e=t.firstChild;)Node.prototype.appendChild.call(t.content,e)}(i),e&&(i.innerHTML="<style>".concat(e.toString(),"</style>")+i.innerHTML),i.innerHTML='<style id="psp_styles" scope="'.concat(i.getAttribute("id"),'">test{}</style>')+i.innerHTML;var o=!0,a=!1,s=void 0;try{for(var A,l=Object.getOwnPropertyNames(n.prototype)[Symbol.iterator]();!(o=(A=l.next()).done);o=!0){let t=A.value,e=Object.getOwnPropertyDescriptor(n.prototype,t);if(e&&e.set){let r=e.set;e.set=function(e){this.getAttribute(t)===e?(this._initializing||this._initialized)&&r.call(this,e):this.setAttribute(t,e)},Object.defineProperty(n.prototype,t,e)}}}catch(t){a=!0,s=t}finally{try{o||null==l.return||l.return()}finally{if(a)throw s}}let c=i.getAttribute("id");console.log("Registered ".concat(c)),window.customElements.define(c,class extends n{attributeChangedCallback(t,e,n){"_"!==t[0]&&e!=n&&(this[t]=n)}connectedCallback(){if(!this._initialized){for(this._initializing=!0,this._old_children=[];this.hasChildNodes();)1===this.lastChild.nodeType&&this._old_children.push(this.lastChild),this.removeChild(this.lastChild);this._old_children=this._old_children.reverse();var t=document.importNode(i.content,!0);this.attachShadow({mode:"open"}),this.shadowRoot.appendChild(t),super.connectedCallback&&super.connectedCallback();var e=!0,r=!1,o=void 0;try{for(var a,s=Object.getOwnPropertyNames(n.prototype)[Symbol.iterator]();!(e=(a=s.next()).done);e=!0){let t=a.value;"connectedCallback"!==t&&this.hasAttribute(t)&&"_"!==t[0]&&(this[t]=this.getAttribute(t))}}catch(t){r=!0,o=t}finally{try{e||null==s.return||s.return()}finally{if(r)throw o}}this._initializing=!1,this._initialized=!0}}static get observedAttributes(){return Object.getOwnPropertyNames(n.prototype)}})}function o(t){for(var e=arguments.length,n=new Array(e>1?e-1:0),r=1;r<e;r++)n[r-1]=arguments[r];const o=n.map(t=>t.toString()).join("\n");return function(e){return i(t,{toString:()=>o},e)}}function a(t){return function(e,n,r){const i=r.value;return r.set=function(e){let r=this.getAttribute(n);try{if(null==e&&(e=t()),"string"!=typeof e&&(e=JSON.stringify(e)),e!==r)return this.setAttribute(n,e),void(r=e);r=JSON.parse(r)}catch(i){console.error('Invalid value for attribute "'.concat(n,'": ').concat(e)),r=t()}i.call(this,r)},r.get=function(){if(this.hasAttribute(n))return JSON.parse(this.getAttribute(n))},delete r.value,delete r.writable,r}}function s(t){let e=document.createElement("textarea");document.body.appendChild(e),e.value=t,e.select(),document.execCommand("copy"),document.body.removeChild(e)}n.r(e),n.d(e,"importTemplate",function(){return r}),n.d(e,"registerElement",function(){return i}),n.d(e,"bindTemplate",function(){return o}),n.d(e,"copy_to_clipboard",function(){return s}),n.d(e,"json_attribute",function(){return A}),n.d(e,"array_attribute",function(){return l}),n(60),n(63),n(57),n(68);const A=a(()=>({})),l=a(()=>[])},function(t,e,n){n(9)&&"g"!=/./g.flags&&n(10).f(RegExp.prototype,"flags",{configurable:!0,get:n(36)})},function(t,e,n){"use strict";n(145);var r=n(2).Point,i=n(2).Rectangle,o=n(28),a=n(146).bind(n(148)),s=n(3),A=n(50),l=n(69),c=n(149),u=n(151),g=n(157),h=n(48),d=n(44),f=n(158),p=n(164),I=n(173),C=n(78),m=["top","bottom","left","right"],y=m.concat(["width","height","position"]),E=s.extend("Hypergrid",{initialize:function(t,e){this.selectionInitialize(),"string"==typeof t||t instanceof HTMLElement||(e=t,t=null),e=e||{},this.clearState(),this.setContainer(t||e.container||function(t){var e=document.getElementById("hypergrid");return e&&!e.firstElementChild||(b(e=document.createElement("div"),t,y),document.body.appendChild(e)),e}(e.boundingRect)),E.prototype.installPlugins(e.plugins),this.lastEdgeSelection=[0,0],this.isWebkit=navigator.userAgent.toLowerCase().indexOf("webkit")>-1,this.selectionModel=new g(this),this.renderOverridesCache={},this.allowEventHandlers=!0,this.dragExtent=new r(0,0),this.numRows=0,this.numColumns=0,this.clearMouseDown(),this.setFormatter(e.localization),this.listeners={},this.cellRenderers=p,this.cellEditors=Object.create(I),Object.defineProperty(this.cellEditors,"create",{value:w.bind(this)}),this.initCanvas(e),e.data?this.setData(e.data,e):(e.Behavior||e.dataModel||e.DataModel)&&this.setBehavior(e),e.state&&this.loadState(e.state),this.plugins={},this.installPlugins(e.plugins),document.addEventListener("mousedown",this.mouseCatcher=function(){this.abortEditing()}.bind(this)),setTimeout(this.repaint.bind(this)),E.grids.push(this),this.resetGridBorder("Top"),this.resetGridBorder("Right"),this.resetGridBorder("Bottom"),this.resetGridBorder("Left")},terminate:function(){document.removeEventListener("mousedown",this.mouseCatcher),this.behavior.dataModel.removeListener(this.behavior.boundDispatchEvent),this.removeAllEventListeners(!0),this.canvas.stop();for(var t=this.div;t.hasChildNodes();)t.removeChild(t.firstChild);E.grids.splice(E.grids.indexOf(this),1)},resetGridBorder:function(t){var e="gridBorder"+(t=t||""),n="border"+t,r=this.properties,i=r[e];switch(i){case!0:i=r.lineWidth+"px solid "+r.lineColor;break;case!1:i=null}this.canvas.canvas.style[n]=i},modules:C,behavior:null,isWebkit:!0,mouseDown:[],dragExtent:null,selectionModel:null,cellEditor:null,renderOverridesCache:{},hoverCell:null,lastEdgeSelection:null,setAttribute:function(t,e){this.div.setAttribute(t,e)},clearState:function(){this.properties=Object.defineProperties(this.initThemeLayer(),{grid:{value:this},var:{value:new B}}),Object.keys(A).forEach(function(t){var e=A[t];"object"==typeof e&&(l[t]?this[t]=e:this[t]=function t(e){var n,r=(n=e,Array.isArray(n)?n.slice():"object"==typeof n?Object.defineProperties({},Object.getOwnPropertyDescriptors(n)):n);return Object.keys(r).forEach(function(e){var n=Object.getOwnPropertyDescriptor(r,e);"object"==typeof n.value&&(r[e]=t(n.value))}),r}(e))},this.properties)},reset:function(t){this.clearState(),this.removeAllEventListeners(),this.lastEdgeSelection=[0,0],this.selectionModel.reset(),this.renderOverridesCache={},this.clearMouseDown(),this.dragExtent=new r(0,0),this.numRows=0,this.numColumns=0,this.vScrollValue=0,this.hScrollValue=0,this.cancelEditing(),this.sbPrevVScrollValue=null,this.sbPrevHScrollValue=null,this.hoverCell=null,this.scrollingNow=!1,this.lastEdgeSelection=[0,0],this.behavior.reset({subgrids:t&&t.subgrids}),this.renderer.reset(),this.canvas.resize(),this.behaviorChanged(),this.refreshProperties()},installPlugins:function(t){var e=this===E.prototype;t&&(Array.isArray(t)||(t=[t]),t.forEach(function(t){var n,r,i;if(t&&(r=[this],e&&r.push(d.prototype),Array.isArray(t)&&(t.length?"string"!=typeof t[0]?(r=r.concat(t.slice(1)),t=t[0]):t.length>=2?(r=r.concat(t.slice(2)),n=t[0],t=t[1]):t=void 0:t=void 0),t)){if((n=n||t.name||t.$$CLASS_NAME)&&(n=n.substr(0,1).toLowerCase()+n.substr(1)),e)i=this.constructor.plugins,t.preinstall&&!i[n]&&t.preinstall.apply(t,r);else if(i=this.plugins,"function"==typeof t)t=this.createApply(t,r);else if(t.install)t.install.apply(t,r);else if(!t.preinstall)throw new s.prototype.HypergridError("Expected plugin (a constructor; or an API with a `preinstall` method and/or an `install` method).");n&&(i[n]=t)}},this))},uninstallPlugins:function(t){t?Array.isArray(t)||(t=[t]):t=[],o(this.plugins).each(function(e,n,r){r.hasOwnProperty(n)&&t.indexOf(n)>=0&&e.uninstall&&e.uninstall(this,n,r)},this)},computeCellsBounds:function(){this.renderer.computeCellsBounds()},setFormatter:function(t){t=t||{},this.localization=new h(t.locale||E.localization.locale,t.numberOptions||E.localization.numberOptions,t.dateOptions||E.localization.dateOptions)},getFormatter:function(t){return this.localization.get(t).format},formatValue:function(t,e){return this.getFormatter(t)(e)},setHoverCell:function(t){var e=this.hoverCell;e&&e.equals(t.gridCell)||(this.hoverCell=t.gridCell,e&&this.fireSyntheticOnCellExitEvent(t),this.fireSyntheticOnCellEnterEvent(t),this.repaint())},addProperties:function(t){Object.assign(this.properties,t),this.refreshProperties()},refreshProperties:function(){this.behaviorShapeChanged(),this.behavior.defaultRowHeight=null,this.behavior.autosizeAllColumns()},setState:function(t){this.addState(t,!0)},addState:function(t,e){this.behavior.addState(t,e),this.refreshProperties(),this.behaviorChanged()},getState:function(){return this.behavior.getState()},loadState:function(t){this.behavior.setState(t)},saveState:function(t){var e=void 0===(t=t||{}).space?"\t":t.space,n=this.properties,r=n.calculators;if((t.blacklist=t.blacklist||[]).push("columnProperties"),r){if(t.compact){var i=this.behavior.getColumns();Object.keys(r).forEach(function(t){i.find(function(e){return e.properties.calculator===r[t]})||delete r[t]})}r.toJSON=Q}this.headerify=t.headerify;var o=JSON.stringify(n,function(e,i){return this===n&&t.blacklist.indexOf(e)>=0?i=void 0:"calculator"===e&&(i=r?Object.keys(r).find(function(t){return r[t]===i}):i.toString()),i},e);return delete this.headerify,o},getMouseDown:function(){if(this.mouseDown.length)return this.mouseDown[this.mouseDown.length-1]},popMouseDown:function(){var t;return this.mouseDown.length&&(t=this.mouseDown.pop()),t},clearMouseDown:function(){this.mouseDown=[new r(-1,-1)],this.dragExtent=null},setMouseDown:function(t){this.mouseDown.push(t)},getDragExtent:function(){return this.dragExtent},setDragExtent:function(t){this.dragExtent=t},gridRenderedNotification:function(){this.cellEditor&&this.cellEditor.gridRenderedNotification(),this.checkColumnAutosizing()&&this.paintNow(),this.fireSyntheticGridRenderedEvent()},tickNotification:function(){this.fireSyntheticTickEvent()},checkColumnAutosizing:function(){var t=this.behavior.checkColumnAutosizing(!1);return t&&this.behaviorShapeChanged(),t},checkClipboardCopy:function(t){if(this.hasFocus()){t.preventDefault();var e=this.getSelectionAsTSV();t.clipboardData.setData("text/plain",e)}},hasFocus:function(){return this.canvas.hasFocus()},setBehavior:function(t){var e=t&&t.Behavior||f;this.behavior=new e(this,t),this.initScrollbars(),this.refreshProperties(),this.behavior.reindex()},getColumnCount:function(){return this.behavior.getActiveColumnCount()},getRowCount:function(){return this.behavior.getRowCount()},getRow:function(t){return this.behavior.getRow(t)},getValue:function(t,e,n){return this.behavior.getValue(t,e,n)},setValue:function(t,e,n,r){this.behavior.setValue(t,e,n,r)},setData:function(t,e){this.behavior||this.setBehavior(e),this.behavior.setData(t,e),this.setInfo(t.length?"":this.properties.noDataMessage),this.behavior.shapeChanged()},setInfo:function(t){this.renderer.setInfo(t)},reindex:function(){v.call(this)?this.needsReindex=!0:this.behavior.reindex(),this.behaviorShapeChanged()},behaviorChanged:function(){this.divCanvas&&(this.numColumns!==this.getColumnCount()||this.numRows!==this.getRowCount()?(this.numColumns=this.getColumnCount(),this.numRows=this.getRowCount(),this.behaviorShapeChanged()):this.behaviorStateChanged())},behaviorShapeChanged:function(){v.call(this)?(this.needsShapeChanged=!0,this.canvas.requestRepaint()):this.divCanvas&&(this.synchronizeScrollingBoundaries(),this.repaint())},behaviorStateChanged:function(){v.call(this)?(this.needsStateChanged=!0,this.canvas.requestRepaint()):this.divCanvas&&(this.computeCellsBounds(),this.repaint())},deferredBehaviorChange:function(){this.needsReindex&&(this.behavior.reindex(),this.needsReindex=!1),this.needsShapeChanged?this.divCanvas&&this.synchronizeScrollingBoundaries():this.needsStateChanged&&this.divCanvas&&this.computeCellsBounds(),this.needsShapeChanged=this.needsStateChanged=!1},getBounds:function(){return this.renderer.getBounds()},repaint:function(){var t=this.canvas;t&&(this.properties.repaintImmediately?this.paintNow():t.repaint())},paintNow:function(){this.behavior.columnsCreated&&this.canvas.paintNow()},setContainer:function(t){this.initContainer(t),this.initRenderer()},initContainer:function(t){"string"==typeof t&&(t=document.querySelector(t)),t.style.position||(t.style.position=null),t.clientHeight<1&&(t.style.height=null),a("grid"),t.oncontextmenu=function(t){return t.stopPropagation(),t.preventDefault(),!1},t.removeAttribute("tabindex"),t.classList.add("hypergrid-container"),t.id=t.id||"hypergrid"+(document.querySelectorAll(".hypergrid-container").length-1||""),this.div=t},initCanvas:function(t){if(!this.divCanvas){var e=document.createElement("div");b(e,t&&t.margin,m),this.div.appendChild(e);var n=t&&(t.contextAttributes||t.canvasContextAttributes),r=new c(e,this.renderer,n);r.canvas.classList.add("hypergrid"),this.divCanvas=e,this.canvas=r,this.delegateCanvasEvents()}},convertViewPointToDataPoint:function(t){return this.behavior.convertViewPointToDataPoint(t)},convertDataPointToViewPoint:function(t){return this.behavior.convertDataPointToViewPoint(t)},beCursor:function(t){t?Array.isArray(t)||(t=[t]):t=["default"],t.forEach(function(t){this.cursor=t},this.div.style)},stopEditing:function(){return!this.cellEditor||this.cellEditor.stopEditing()},cancelEditing:function(){return!this.cellEditor||this.cellEditor.cancelEditing()},abortEditing:function(){return!this.cellEditor||(this.cellEditor.abortEditing?this.cellEditor.abortEditing():this.cellEditor.stopEditing())},getDataBounds:function(){var t=this.canvas.bounds;return new i(0,0,t.origin.x+t.extent.x,t.origin.y+t.extent.y)},editAt:function(t){var e;return this.abortEditing(),t.isDataColumn&&t.properties[t.isDataRow?"editable":"filterable"]&&(e=this.getCellEditorAt(t))&&e.beginEditing(),e},isColumnVisible:function(t){return this.renderer.isColumnVisible(t)},isDataRowVisible:function(t){return this.renderer.isDataRowVisible(t)},isDataVisible:function(t,e){return this.isDataRowVisible(e)&&this.isColumnVisible(t)},insureModelColIsVisible:function(t,e){var n=this.getColumnCount()-1,r=t+Math.sign(e),i=!this.isColumnVisible(r)||t===n;return i&&this.scrollBy(e,0),i},insureModelRowIsVisible:function(t,e){var n=this.getRowCount()-1,r=t+(e>-1?1:0),i=!this.isDataRowVisible(r)||t===n;return i&&this.scrollBy(0,e),i},getGridCellFromMousePoint:function(t){return this.renderer.getGridCellFromMousePoint(t)},getBoundsOfCell:function(t){var e=this.renderer.getBoundsOfCell(t.x,t.y);return new i(e.x,e.y,e.width,e.height)},resized:function(){this.behaviorShapeChanged()},cellClicked:function(t){return this.behavior.cellClicked(t)},windowOpen:function(t,e,n,r){return window.open.apply(window,arguments)},getColumns:function(t,e){var n=this.behavior.getColumns();return n.slice.apply(n,arguments)},getActiveColumns:function(t,e){var n=this.behavior.getActiveColumns();return n.slice.apply(n,arguments)},getHiddenColumns:function(){return this.behavior.getHiddenColumns()},takeFocus:function(){var t=this.cellEditor;this.stopEditing(),t||this.canvas.takeFocus()},editorTakeFocus:function(){if(this.cellEditor)return this.cellEditor.takeFocus()},getVisibleRows:function(){return this.renderer.getVisibleRows()},getVisibleColumns:function(){return this.renderer.getVisibleColumns()},initRenderer:function(){this.renderer=this.renderer||new u(this)},getColumnWidth:function(t){return this.behavior.getColumnWidth(t)},setColumnWidth:function(t,e){this.abortEditing()&&this.behavior.setColumnWidth(t,e)},getFixedColumnsWidth:function(){return this.behavior.getFixedColumnsWidth()},getRowHeight:function(t,e){return this.behavior.getRowHeight(t,e)},setRowHeight:function(t,e,n){this.abortEditing()&&this.behavior.setRowHeight(t,e,n)},getFixedRowsHeight:function(){return this.behavior.getFixedRowsHeight()},getFixedColumnCount:function(){return this.behavior.getFixedColumnCount()},getFixedRowCount:function(){return this.behavior.getFixedRowCount()},topLeftClicked:function(t){this.behavior.topLeftClicked(this,t)},rowHeaderClicked:function(t){this.behavior.rowHeaderClicked(this,t)},columnHeaderClicked:function(t){this.behavior.columnHeaderClicked(this,t)},onEditorActivate:function(t){return this.editAt(t)},getCellEditorAt:function(t){return this.behavior.getCellEditorAt(t)},toggleHiDPI:function(){this.properties.useHiDPI?this.removeAttribute("hidpi"):this.setAttribute("hidpi",null),this.canvas.resize()},getHiDPI:function(t){if(window.devicePixelRatio&&this.properties.useHiDPI)var e=(window.devicePixelRatio||1)/(t.webkitBackingStorePixelRatio||t.mozBackingStorePixelRatio||t.msBackingStorePixelRatio||t.oBackingStorePixelRatio||t.backingStorePixelRatio||1);else e=1;return e},getRenderedWidth:function(t){return this.renderer.getRenderedWidth(t)},getRenderedHeight:function(t){return this.renderer.getRenderedHeight(t)},updateCursor:function(){var t=this.behavior.getCursorAt(-1,-1),e=this.hoverCell;if(e&&e.x>-1&&e.y>-1){var n=e.x+this.getHScrollValue();t=this.behavior.getCursorAt(n,e.y+this.getVScrollValue())}this.beCursor(t)},repaintCell:function(t,e){this.renderer.repaintCell(t,e)},isDraggingColumn:function(){return!!this.renderOverridesCache.dragger},getRenderedData:function(){return this.renderer.getVisibleCellMatrix()},autosizeColumn:function(t){(t>=-2?this.behavior.getActiveColumn(t):t).checkColumnAutosizing(!0),this.computeCellsBounds()},setFocusable:function(t){this.canvas.setFocusable(t)},getVisibleColumnsCount:function(){return this.renderer.getVisibleColumnsCount()},getVisibleRowsCount:function(){return this.renderer.getVisibleRowsCount()},updateSize:function(){this.canvas.checksize()},stopPaintThread:function(){this.canvas.stopPaintThread()},stopResizeThread:function(){this.canvas.stopResizeThread()},restartResizeThread:function(){this.canvas.restartResizeThread()},restartPaintThread:function(){this.canvas.restartPaintThread()},swapColumns:function(t,e){this.behavior.swapColumns(t,e)},endDragColumnNotification:function(){this.behavior.endDragColumnNotification()},getFixedColumnsMaxWidth:function(){return this.behavior.getFixedColumnsMaxWidth()},isMouseDownInHeaderArea:function(){var t=this.getHeaderRowCount(),e=this.getMouseDown();return e.x<0||e.y<t},getColumnProperties:function(t){return this.behavior.getColumnProperties(t)},setColumnProperties:function(t,e){this.behavior.setColumnProperties(t,e)},clearAllCellProperties:function(t){this.behavior.clearAllCellProperties(t),this.renderer.resetAllCellPropertiesCaches()},isGridRow:function(t){return new this.behavior.CellEvent(0,t).isDataRow},getHeaderRowCount:function(){return this.behavior.getHeaderRowCount()},getFooterRowCount:function(){return this.behavior.getFooterRowCount()},getLogicalRowCount:function(){return this.behavior.getLogicalRowCount()},hasTreeColumn:function(){return this.behavior.hasTreeColumn()},lookupFeature:function(t){return this.behavior.lookupFeature(t)},newPoint:function(t,e){return new r(t,e)},newRectangle:function(t,e,n,r){return new i(t,e,n,r)},get charMap(){return this.behavior.charMap},decorateColumnArray:function(t){if(!this.columnArrayDecorations){var e=this;this.columnArrayDecorations={findWithNeg:{value:function(t,n){for(var r=e.behavior.leftMostColIndex;r<0;r++)if(this[r]&&t.call(n,this[r],r,this))return this[r];return this.find(t,n)}},forEachWithNeg:{value:function(t,n){for(var r=e.behavior.leftMostColIndex;r<0;r++)this[r]&&t.call(n,this[r],r,this);return this.forEach(t,n)}}}}return Object.defineProperties(t||[],this.columnArrayDecorations)}});function v(){return!this.properties.repaintImmediately&&this.canvas.paintLoopRunning()}function B(){this.gridRenderer=A.gridRenderer,this.rowHeaderCheckboxes=A.rowHeaderCheckboxes,this.rowHeaderNumbers=A.rowHeaderNumbers,this.gridBorder=A.gridBorder,this.gridBorderTop=A.gridBorderTop,this.gridBorderRight=A.gridBorderRight,this.gridBorderBottom=A.gridBorderBottom,this.gridBorderLeft=A.gridBorderLeft}function b(t,e,n){if(e){var r=t.style;n.forEach(function(t){void 0!==e[t]&&(r[t]=e[t])})}}function Q(){var t=this;return Object.keys(this).reduce(function(e,n){return"toJSON"!==n&&(e[n]=/^function /.test(n)?null:t[n].toString().replace(/^function anonymous\(/,"function(").replace("\n/*``*/)",")")),e},{})}function w(t,e){var n=I.get(t);if(n)return new n(this,e)}function x(t,e){if(!x.warned[t]){var n=t.match(/\w+$/)[0];console.warn("Reference to "+n+" external module using `Hypergrid."+n+".` has been deprecated as of v3.0.0 in favor of `require('"+t+"')` from within a Hypergrid Client Module (otherwise use `Hypergrid.require(...)`) and will be removed in a future release. See https://github.com/fin-hypergrid/core/wiki/Client-Modules#internal-modules."),x.warned[t]=!0}return e}E.plugins={},E.localization={locale:"en-US",numberOptions:{maximumFractionDigits:0}},E.mixIn=E.prototype.mixIn,E.mixIn(n(80).sharedMixin),E.prototype.mixIn(n(80).mixin),E.prototype.mixIn(n(181).mixin),E.prototype.mixIn(n(182).mixin),E.prototype.mixIn(n(184).mixin),E.prototype.mixIn(n(185).mixin),x.warned={},Object.defineProperties(E,{Base:{get:function(){return x("fin-hypergrid/src/Base",n(3))}},images:{get:function(){return x("fin-hypergrid/images",n(18))}}}),E.grids=[],E.defaults=E.properties=A,E.modules=C,t.exports=E},function(t,e,n){"use strict";Math.sign=Math.sign=function(t){return 0==(t=+t)||isNaN(t)?t:t>0?1:-1},"function"!=typeof Array.prototype.findIndex&&(Array.prototype.findIndex=function(t){for(var e=arguments[1],n=0,r=this.length;n<r;n++)if(t.call(e,this[n],n,this))return n;return-1}),"function"!=typeof Array.prototype.fill&&(Array.prototype.fill=function(t,e,n){e=void 0===e?0:e<0?this.length+e:e,n=void 0===n?this.length:n<0?this.length+n:n;for(var r=e||0;r<n;++r)this[r]=t;return this}),"function"!=typeof Object.assign&&(Object.assign=function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];if(null!=n)for(var r in n)n.hasOwnProperty(r)&&(t[r]=n[r])}return t}),"function"!=typeof Object.getOwnPropertyDescriptors&&(Object.getOwnPropertyDescriptors=function(t){return Object.getOwnPropertyNames(t).reduce(function(e,n){return e[n]=Object.getOwnPropertyDescriptor(t,n),e},{})})},function(t,e,n){"use strict";var r=n(147);function i(t,e){var n,o,a,s,A,l,c=i.prefix;if("boolean"==typeof t?n=2:(e=t,t=i.before,n=1),!(o=document.getElementById(c+e))){if(a=document.querySelector("head"),t&&(s=Array.prototype.slice.call(a.children).find(function(t){var e=t.getAttribute("id");return"STYLE"===t.tagName&&(!e||e.indexOf(c)!==c)||"LINK"===t.tagName&&"stylesheet"===t.getAttribute("rel")})),!(A=this[e]))throw"Expected to find member `"+e+"` in calling context.";l=["<style>\n"+A+"\n</style>\n",a,s||null],arguments.length>1&&(l=l.concat(Array.prototype.slice.call(arguments,n))),(o=r.append.apply(null,l)[0]).id=c+e}return o}i.before=!0,i.prefix="injected-stylesheet-",t.exports=i},function(t,e,n){"use strict";function r(t,e){var n=arguments.length>1;if("function"==typeof t&&(t=t.call(this)),n){var i=arguments;t=(t=t.replace(r.replacersRegex,function(t,e){return e-=-1,i.length>e?i[e]:""})).replace(r.encodersRegex,function(t,e){if(e-=-1,i.length>e){var n=document.createElement("DIV");return n.textContent=i[e],n.innerHTML}return""})}return t}function i(t,e,n){var i="object"!=typeof e,o=Array.prototype.slice.call(arguments,1);return i?(e=document.createElement("DIV"),o.unshift(t)):o[0]=t,e.innerHTML=r.apply(null,o),e}r.encodersRegex=/%\{(\d+)\}/g,r.replacersRegex=/\$\{(.*?)\}/g,r.format=r,r.replace=i,r.append=function(t,e,n,r){var o=3;"object"!=typeof n&&(n=null,o=2),r=Array.prototype.slice.call(arguments,o);for(var a=[],s=i.apply(null,[t].concat(r));s.childNodes.length;)a.push(s.firstChild),e.insertBefore(s.firstChild,n);return a},r.firstChild=function(t,e){return i.apply(null,arguments).firstChild},r.firstElement=function(t,e){return i.apply(null,arguments).firstElementChild},t.exports=r},function(t,e,n){"use strict";e.grid=[".hypergrid-container {","\tposition: relative;","\theight: 500px;","}",".hypergrid-container > div:first-child {","\tposition: absolute;","\tleft: 0;","\ttop: 0;","\tright: 0;","\tbottom: 0;","}",".hypergrid-container > div:first-child > div.info {","\tposition: absolute;","\tdisplay: none; /* initially hidden */","\tmargin-top: 150px; /* to place below headers */","\tcolor: #eee;","\ttext-shadow: 1px 1px #ccc;","\tfont-size: 36pt;","\tfont-weight: bold;","\ttext-align: center;","\ttop: 0; right: 0; bottom: 0; left: 0;","}",".hypergrid-textfield {","\tposition: absolute;","\tfont-size: 12px;","\tcolor: black;","\tbackground-color: ivory;","\tbox-sizing: border-box;","\tmargin: 0;","\tpadding: 0 5px;","\tborder: 0; /*border: 1px solid #777;*/","\toutline: 0;","}","",""].join("\n")},function(t,e,n){"use strict";"function"!=typeof window.CustomEvent&&(window.CustomEvent=function(t,e){e=e||{bubbles:!1,cancelable:!1,detail:void 0};var n=document.createEvent("CustomEvent");return n.initCustomEvent(t,e.bubbles,e.cancelable,e.detail),n},window.CustomEvent.prototype=window.Event.prototype);var r,i,o=n(2),a=200,s=[],A=[],l=function(){for(var t=[],e=["",""],n=0;n<256;n++)t[n]=e;return t[27]=["ESC","ESCSHIFT"],t[192]=["`","~"],t[49]=["1","!"],t[50]=["2","@"],t[51]=["3","#"],t[52]=["4","$"],t[53]=["5","%"],t[54]=["6","^"],t[55]=["7","&"],t[56]=["8","*"],t[57]=["9","("],t[48]=["0",")"],t[189]=["-","_"],t[187]=["=","+"],t[8]=["BACKSPACE","BACKSPACESHIFT"],t[46]=["DELETE","DELETESHIFT"],t[9]=["TAB","TABSHIFT"],t[81]=["q","Q"],t[87]=["w","W"],t[69]=["e","E"],t[82]=["r","R"],t[84]=["t","T"],t[89]=["y","Y"],t[85]=["u","U"],t[73]=["i","I"],t[79]=["o","O"],t[80]=["p","P"],t[219]=["[","{"],t[221]=["]","}"],t[220]=["\\","|"],t[220]=["CAPSLOCK","CAPSLOCKSHIFT"],t[65]=["a","A"],t[83]=["s","S"],t[68]=["d","D"],t[70]=["f","F"],t[71]=["g","G"],t[72]=["h","H"],t[74]=["j","J"],t[75]=["k","K"],t[76]=["l","L"],t[186]=[";",":"],t[222]=["'","|"],t[13]=["RETURN","RETURNSHIFT"],t[16]=["SHIFT","SHIFT"],t[90]=["z","Z"],t[88]=["x","X"],t[67]=["c","C"],t[86]=["v","V"],t[66]=["b","B"],t[78]=["n","N"],t[77]=["m","M"],t[188]=[",","<"],t[190]=[".",">"],t[191]=["/","?"],t[16]=["SHIFT","SHIFT"],t[17]=["CTRL","CTRLSHIFT"],t[18]=["ALT","ALTSHIFT"],t[91]=["COMMANDLEFT","COMMANDLEFTSHIFT"],t[32]=["SPACE","SPACESHIFT"],t[93]=["COMMANDRIGHT","COMMANDRIGHTSHIFT"],t[18]=["ALT","ALTSHIFT"],t[38]=["UP","UPSHIFT"],t[37]=["LEFT","LEFTSHIFT"],t[40]=["DOWN","DOWNSHIFT"],t[39]=["RIGHT","RIGHTSHIFT"],t[33]=["PAGEUP","PAGEUPSHIFT"],t[34]=["PAGEDOWN","PAGEDOWNSHIFT"],t[35]=["PAGERIGHT","PAGERIGHTSHIFT"],t[36]=["PAGELEFT","PAGELEFTSHIFT"],t[112]=["F1","F1SHIFT"],t[113]=["F2","F2SHIFT"],t[114]=["F3","F3SHIFT"],t[115]=["F4","F4SHIFT"],t[116]=["F5","F5SHIFT"],t[117]=["F6","F6SHIFT"],t[118]=["F7","F7SHIFT"],t[119]=["F8","F8SHIFT"],t[120]=["F9","F9SHIFT"],t[121]=["F10","F10SHIFT"],t[122]=["F11","F11SHIFT"],t[123]=["F12","F12SHIFT"],t}();function c(t,e,n){var r=this;this.div=t,this.component=e,this.dragEndtime=Date.now(),this.infoDiv=document.createElement("div"),this.infoDiv.className="info",this.div.appendChild(this.infoDiv),this.gc=C(this.canvas=document.createElement("canvas"),n),this.bc=C(this.buffer=document.createElement("canvas"),n),this.div.appendChild(this.canvas),this.canvas.style.outline="none",this.mouseLocation=new o.Point(-1,-1),this.dragstart=new o.Point(-1,-1),this.bounds=new o.Rectangle(0,0,0,0),this.hasMouse=!1,document.addEventListener("mousemove",function(t){(r.hasMouse||r.isDragging())&&r.finmousemove(t)}),document.addEventListener("mouseup",function(t){r.finmouseup(t)}),document.addEventListener("wheel",function(t){r.finwheelmoved(t)}),document.addEventListener("keydown",function(t){r.finkeydown(t)}),document.addEventListener("keyup",function(t){r.finkeyup(t)}),this.canvas.onmouseover=function(){r.hasMouse=!0},this.addEventListener("focus",function(t){r.finfocusgained(t)}),this.addEventListener("blur",function(t){r.finfocuslost(t)}),this.addEventListener("mousedown",function(t){r.finmousedown(t)}),this.addEventListener("mouseout",function(t){r.hasMouse=!1,r.finmouseout(t)}),this.addEventListener("click",function(t){r.finclick(t)}),this.addEventListener("dblclick",function(t){r.findblclick(t)}),this.addEventListener("contextmenu",function(t){return r.fincontextmenu(t),t.preventDefault(),!1}),this.canvas.setAttribute("tabindex",0),this.resize(),this.beginResizing(),this.beginPainting()}function u(t){r&&(s.forEach(function(e){try{e.tickPainter(t)}catch(t){console.error(t)}e.component.tickNotification&&e.component.tickNotification()}),r=requestAnimationFrame(u))}function g(){r=r||requestAnimationFrame(u)}function h(t){if(i)for(var e=0;e<A.length;e++)try{A[e].tickResizer(t)}catch(t){console.error(t)}}function d(){i=i||setInterval(h,a)}function f(t,e){var n=this.getKeyChar(t);switch(n){case"TAB":case"TABSHIFT":case"Tab":t.preventDefault()}return p.call(this,n,e),n}function p(t,e){var n=this.currentKeys.indexOf(t);!e&&n>=0&&this.currentKeys.splice(n,1),"SHIFT"===t&&this.currentKeys.forEach(function(t,n,r){var i=l.find(function(n){return n[e?0:1]===t});i&&(r[n]=i[e?1:0])}),e&&n<0&&this.currentKeys.push(t)}function I(t,e,n){var r=this;return Object.defineProperty(n,e,{configurable:!0,ennumerable:!0,get:function(){var e;"shiftKey"in t?p.call(r,"SHIFT",e=t.shiftKey):e=r.currentKeys.indexOf("SHIFT")>=0;var n=e?"SHIFT":"";return"ctrlKey"in t&&p.call(r,"CTRL"+n,t.ctrlKey),"altKey"in t&&p.call(r,"ALT"+n,t.altKey),r.currentKeys.slice()}}),n}function C(t,e){var r=t.getContext("2d",e),i={},o={};function a(t){t in i||/^(webkit|moz|ms|o)[A-Z]/.test(t)||"function"==typeof r[t]||Object.defineProperty(i,t,{get:function(){return o[t]=o[t]||r[t]},set:function(e){e!==o[t]&&(r[t]=o[t]=e)}})}return Object.keys(Object.getPrototypeOf(r)).forEach(a),Object.keys(r).forEach(a),r.cache=i,r.cache.save=function(){r.save(),o=Object.create(o)},r.cache.restore=function(){r.restore(),o=Object.getPrototypeOf(o)},r.conditionalsStack=[],Object.getOwnPropertyNames(c.graphicsContextAliases).forEach(function(t){r[t]=r[c.graphicsContextAliases[t]]}),Object.assign(r,n(150))}c.prototype={constructor:c.prototype.constructor,div:null,component:null,canvas:null,focuser:null,buffer:null,ctx:null,mouseLocation:null,dragstart:null,origin:null,bounds:null,dirty:!1,size:null,mousedown:!1,dragging:!1,repeatKeyCount:0,repeatKey:null,repeatKeyStartTime:0,currentKeys:[],hasMouse:!1,dragEndTime:0,lastRepaintTime:0,currentPaintCount:0,currentFPS:0,lastFPSComputeTime:0,addEventListener:function(t,e){this.canvas.addEventListener(t,e)},removeEventListener:function(t,e){this.canvas.removeEventListener(t,e)},stopPaintLoop:function(){r&&(cancelAnimationFrame(r),r=void 0)},restartPaintLoop:g,stopResizeLoop:function(){i&&(clearInterval(i),i=void 0)},restartResizeLoop:d,detached:function(){this.stopPainting(),this.stopResizing()},getCurrentFPS:function(){return this.currentFPS},tickPaint:function(t){var e=this.component.properties.enableContinuousRepaint,n=this.component.properties.repaintIntervalRate;if(0!==n){var r=1e3/n;t-this.lastRepaintTime>r&&(e||this.dirty)&&(this.paintNow(),this.lastRepaintTime=t,e&&(this.currentPaintCount++,t-this.lastFPSComputeTime>=1e3&&(this.currentFPS=1e3*this.currentPaintCount/(t-this.lastFPSComputeTime),this.currentPaintCount=0,this.lastFPSComputeTime=t)))}},beginPainting:function(){var t=this;this.requestRepaint(),this.tickPainter=function(e){t.tickPaint(e)},s.push(this)},stopPainting:function(){s.splice(s.indexOf(this),1)},beginResizing:function(){var t=this;this.tickResizer=function(){t.checksize()},A.push(this)},stopResizing:function(){A.splice(A.indexOf(this),1)},start:function(){this.beginPainting(),this.beginResizing()},stop:function(){this.stopPainting(),this.stopResizing()},getDivBoundingClientRect:function(){var t=this.div.getBoundingClientRect(),e=Math.floor(t.top),n=Math.floor(t.left),r=Math.ceil(t.width),i=Math.ceil(t.height);return{top:e,right:n+r,bottom:e+i,left:n,width:r,height:i,x:t.x,y:t.y}},checksize:function(){var t=this.getDivBoundingClientRect();t.width===this.size.width&&t.height===this.size.height||this.resize()},resize:function(){var t=this.size=this.getDivBoundingClientRect();this.width=t.width,this.height=t.height;var e=1,n=window.devicePixelRatio&&this.component.properties.useHiDPI;n&&(e=(window.devicePixelRatio||1)/(this.gc.webkitBackingStorePixelRatio||this.gc.mozBackingStorePixelRatio||this.gc.msBackingStorePixelRatio||this.gc.oBackingStorePixelRatio||this.gc.backingStorePixelRatio||1)),this.buffer.width=this.canvas.width=this.width*e,this.buffer.height=this.canvas.height=this.height*e,this.canvas.style.width=this.buffer.style.width=this.width+"px",this.canvas.style.height=this.buffer.style.height=this.height+"px",this.bc.scale(e,e),n&&!this.component.properties.useBitBlit&&this.gc.scale(e,e),this.bounds=new o.Rectangle(0,0,this.width,this.height),this.component.setBounds(this.bounds),this.resizeNotification(),this.paintNow()},resizeNotification:function(){this.dispatchNewEvent(void 0,"fin-canvas-resized",{width:this.width,height:this.height})},getBounds:function(){return this.bounds},paintNow:function(){var t=this.component.properties.useBitBlit,e=t?this.bc:this.gc;try{e.cache.save(),this.dirty=!1,this.component.paint(e)}catch(t){console.error(t)}finally{e.cache.restore()}t&&this.flushBuffer()},flushBuffer:function(){this.buffer.width>0&&this.buffer.height>0&&this.gc.drawImage(this.buffer,0,0)},newEvent:function(t,e,n){var r={detail:n||{}};return t&&(r.detail.primitiveEvent=t),new CustomEvent(e,r)},dispatchNewEvent:function(t,e,n){return this.canvas.dispatchEvent(this.newEvent(t,e,n))},dispatchNewMouseKeysEvent:function(t,e,n){return(n=n||{}).mouse=this.mouseLocation,I.call(this,t,"keys",n),this.dispatchNewEvent(t,e,n)},finmousemove:function(t){!this.isDragging()&&this.mousedown&&(this.beDragging(),this.dispatchNewMouseKeysEvent(t,"fin-canvas-dragstart",{isRightClick:this.isRightClick(t),dragstart:this.dragstart}),this.dragstart=new o.Point(this.mouseLocation.x,this.mouseLocation.y)),this.mouseLocation=this.getLocal(t),this.isDragging()&&this.dispatchNewMouseKeysEvent(t,"fin-canvas-drag",{dragstart:this.dragstart,isRightClick:this.isRightClick(t)}),this.bounds.contains(this.mouseLocation)&&this.dispatchNewMouseKeysEvent(t,"fin-canvas-mousemove")},finmousedown:function(t){this.mouseLocation=this.mouseDownLocation=this.getLocal(t),this.mousedown=!0,this.dispatchNewMouseKeysEvent(t,"fin-canvas-mousedown",{isRightClick:this.isRightClick(t)}),this.takeFocus()},finmouseup:function(t){this.mousedown&&(this.isDragging()&&(this.dispatchNewMouseKeysEvent(t,"fin-canvas-dragend",{dragstart:this.dragstart,isRightClick:this.isRightClick(t)}),this.beNotDragging(),this.dragEndtime=Date.now()),this.mousedown=!1,this.dispatchNewMouseKeysEvent(t,"fin-canvas-mouseup",{dragstart:this.dragstart,isRightClick:this.isRightClick(t)}))},finmouseout:function(t){this.mousedown||(this.mouseLocation=new o.Point(-1,-1)),this.repaint(),this.dispatchNewMouseKeysEvent(t,"fin-canvas-mouseout",{dragstart:this.dragstart})},finwheelmoved:function(t){!this.isDragging()&&this.hasFocus()&&(t.preventDefault(),this.dispatchNewMouseKeysEvent(t,"fin-canvas-wheelmoved",{isRightClick:this.isRightClick(t)}))},finclick:function(t){this.mouseLocation=this.getLocal(t),this.dispatchNewMouseKeysEvent(t,"fin-canvas-click",{isRightClick:this.isRightClick(t)})},findblclick:function(t){this.mouseLocation=this.getLocal(t),this.dispatchNewMouseKeysEvent(t,"fin-canvas-dblclick",{isRightClick:this.isRightClick(t)})},getCharMap:function(){return l},getKeyChar:function(t){var e=t.keyCode||t.detail.key,n=t.shiftKey||t.detail.shift,r=t.key;return t.legacyKey=l[e]&&l[e][n?1:0],"string"==typeof r&&1===r.length?r:t.legacyKey||r},finkeydown:function(t){if(this.hasFocus()){var e=f.call(this,t,!0);t.repeat?this.repeatKey===e?this.repeatKeyCount++:(this.repeatKey=e,this.repeatKeyStartTime=Date.now()):(this.repeatKey=null,this.repeatKeyCount=0,this.repeatKeyStartTime=0),this.dispatchNewEvent(t,"fin-canvas-keydown",I.call(this,t,"currentKeys",{alt:t.altKey,ctrl:t.ctrlKey,char:e,legacyChar:t.legacyKey,code:t.charCode,key:t.keyCode,meta:t.metaKey,repeatCount:this.repeatKeyCount,repeatStartTime:this.repeatKeyStartTime,shift:t.shiftKey,identifier:t.key}))}},finkeyup:function(t){if(this.hasFocus()){var e=f.call(this,t,!1);this.repeatKeyCount=0,this.repeatKey=null,this.repeatKeyStartTime=0,this.dispatchNewEvent(t,"fin-canvas-keyup",I.call(this,t,"currentKeys",{alt:t.altKey,ctrl:t.ctrlKey,char:e,legacyChar:t.legacyKey,code:t.charCode,key:t.keyCode,meta:t.metaKey,repeat:t.repeat,shift:t.shiftKey,identifier:t.key,currentKeys:this.currentKeys.slice(0)}))}},finfocusgained:function(t){this.dispatchNewEvent(t,"fin-canvas-focus-gained")},finfocuslost:function(t){this.dispatchNewEvent(t,"fin-canvas-focus-lost")},fincontextmenu:function(t){t.ctrlKey&&-1===this.currentKeys.indexOf("CTRL")&&this.currentKeys.push("CTRL"),this.dispatchNewMouseKeysEvent(t,"fin-canvas-context-menu",{isRightClick:this.isRightClick(t)})},paintLoopRunning:function(){return!!r},requestRepaint:function(){this.dirty=!0},repaint:function(){this.requestRepaint(),r&&0!==this.component.properties.repaintIntervalRate||this.paintNow()},getMouseLocation:function(){return this.mouseLocation},getOrigin:function(){var t=this.canvas.getBoundingClientRect();return new o.Point(t.left,t.top)},getLocal:function(t){var e=this.canvas.getBoundingClientRect();return new o.Point(t.clientX-e.left,t.clientY-e.top)},hasFocus:function(){return document.activeElement===this.canvas},takeFocus:function(){var t=this;this.hasFocus()||setTimeout(function(){t.canvas.focus()},10)},beDragging:function(){this.dragging=!0,this.disableDocumentElementSelection()},beNotDragging:function(){this.dragging=!1,this.enableDocumentElementSelection()},isDragging:function(){return this.dragging},disableDocumentElementSelection:function(){var t=document.body.style;t.cssText=t.cssText+"-webkit-user-select: none"},enableDocumentElementSelection:function(){var t=document.body.style;t.cssText=t.cssText.replace("-webkit-user-select: none","")},setFocusable:function(t){this.focuser.style.display=t?"":"none"},isRightClick:function(t){var e;return"which"in(t=t||window.event)?e=3===t.which:"button"in t&&(e=2===t.button),e},dispatchEvent:function(t){return this.canvas.dispatchEvent(t)},setInfo:function(t,e){t&&(void 0!==e&&(e&&!isNaN(Number(e))&&(e+="px"),this.infoDiv.style.width=e),t.indexOf("<")?this.infoDiv.innerHTML=t:this.infoDiv.innerText=t),this.infoDiv.style.display=t?"block":"none"}},g(),d(),c.graphicsContextAliases={simpleText:"fillText"},t.exports=c},function(t,e,n){"use strict";var r,i=/^(transparent|((RGB|HSL)A\(.*,\s*([\d\.]+)\)))$/i;function o(t){var e;return t?null===(e=t.match(i))?1:void 0===e[4]?0:Number(e[4]):0}var a={},s="…",A={};r={clearFill:function(t,e,n,r,i){var a=o(i);a<1&&this.clearRect(t,e,n,r),a>0&&(this.cache.fillStyle=i,this.fillRect(t,e,n,r))},alpha:o,getTextWidth:function(t){for(var e=a[this.cache.font]=a[this.cache.font]||{},n=0,r=0,i=(t+="").length;n<i;++n){var o=t[n];r+=e[o]=e[o]||this.measureText(o).width}return r},getTextWidthTruncated:function(t,e,n,r){var i,o,A,l=a[this.cache.font],c=void 0!==n;l||((l=a[this.cache.font]={})[s]=this.measureText(s).width),e+=!1===n?2:-1;for(var u=0,g=0,h=(t+="").length;u<h;++u){var d=t[u],f=l[d]=l[d]||this.measureText(d).width;if(g+=f,!i&&c&&g>e){switch(A=u,n){case!0:for(o=g-f+l[s];A&&o>e;)o-=l[t[--A]];i=o>e?"":i=t.substr(0,A)+s;break;case!1:i=t.substr(0,A);break;default:++A<t.length&&(i=t.substr(0,A))}if(r)break}}return{string:i,width:g}},getTextHeight:function(t){var e=A[t];if(!e){e={};var n=document.createElement("span");n.textContent="Hg",n.style.font=t;var r=document.createElement("div");r.style.display="inline-block",r.style.width="1px",r.style.height="0px";var i=document.createElement("div");i.appendChild(n),i.appendChild(r),i.style.position="absolute",document.body.appendChild(i);try{r.style.verticalAlign="baseline";var o=r.getBoundingClientRect(),a=n.getBoundingClientRect();e.ascent=o.top-a.top,r.style.verticalAlign="bottom",e.height=o.top-a.top,e.descent=e.height-e.ascent}finally{document.body.removeChild(i)}0!==e.height&&(A[t]=e)}return e},clipSave:function(t,e,n,r,i){this.conditionalsStack.push(t),t&&(this.cache.save(),this.beginPath(),this.rect(e,n,r,i),this.clip())},clipRestore:function(t){this.conditionalsStack.pop()&&this.cache.restore()},truncateTextWithEllipsis:!0},t.exports=r},function(t,e,n){"use strict";var r=n(3),i=n(18),o=n(152),a=n(70),s=[],A=r.extend("Renderer",{cell:{x:0,y:0,width:0,height:0},scrollHeight:0,reset:function(){this.bounds={width:0,height:0},this.visibleColumns=this.grid.decorateColumnArray(),this.visibleRows=[],this.insertionBounds=[],this.cellEventPool=[]},initialize:function(t){this.grid=t,this.gridRenderers={},s.forEach(function(t){this.registerGridRenderer(t)},this),this.setGridRenderer(this.properties.gridRenderer||"by-columns-and-rows"),this.reset()},registerGridRenderer:function(t){this.gridRenderers[t.key]={paintCells:t}},setGridRenderer:function(t){var e=this.gridRenderers[t];if(!e)throw new this.HypergridError('Unregistered grid renderer "'+t+'"');e!==this.gridRenderer&&(this.gridRenderer=e,this.gridRenderer.reset=!0)},resetAllGridRenderers:function(t){Object.keys(this.gridRenderers).forEach(function(e){this.gridRenderers[e].reset=!t||t.indexOf(e)<0},this)},rebundleGridRenderers:function(){Object.keys(this.gridRenderers).forEach(function(t){this.gridRenderers[t].paintCells.rebundle&&(this.gridRenderers[t].rebundle=!0)},this)},resetRowHeaderColumnWidth:function(){this.lastKnowRowCount=void 0},computeCellsBounds:function(t){t?(l.call(this),this.needsComputeCellsBounds=!1):this.needsComputeCellsBounds=!0},get properties(){return this.grid.properties},paint:function(t){this.grid.canvas&&(this.renderGrid(t),this.grid.gridRenderedNotification())},tickNotification:function(){this.grid.tickNotification()},getVisibleRowsCount:function(){return this.visibleRows.length-1},getVisibleScrollHeight:function(){var t=this.grid.properties.defaultRowHeight*this.grid.behavior.getFooterRowCount();return this.getBounds().height-t-this.grid.getFixedRowsHeight()},getVisibleColumnsCount:function(){return this.visibleColumns.length-1},getBoundsOfCell:function(t,e){var n=this.visibleColumns[t],r=this.visibleRows[e];return{x:n.left,y:r.top,width:n.width,height:r.height}},getColumnFromPixelX:function(t){for(var e=this.grid.getFixedColumnCount(),n=this.grid.getHScrollValue(),r=this.visibleColumns,i=1;i<r.length-1;i++)if(t<r[i].left-(r[i].left-r[i-1].left)/2)return i>e&&(i+=n),i-1;return i>e&&(i+=n),i-1},getGridCellFromMousePoint:function(t){var e=t.x,n=t.y,r=!1,i=!1,o=this.visibleRows,a=this.visibleColumns,s=a[this.grid.behavior.leftMostColIndex],A=s&&e<s.right?s:a.findWithNeg(function(t){return e<t.right}),l=o.find(function(t){return n<t.bottom}),c={fake:!1};l?r=!1:(l=o[o.length-1],r=!0),A?i=!1:(A=a[a.length-1],i=!0);var u=new this.grid.behavior.CellEvent(A.columnIndex,l.index),g=this.findCell(u);return c.cellEvent=g?Object.create(g):u,c.cellEvent.mousePoint=this.grid.newPoint(e-A.left,n-l.top),(i||r)&&(c.fake=!0,this.grid.beCursor(null)),c},getVisibleCellMatrix:function(){for(var t=Array(this.visibleRows.length),e=this.grid.behavior.hasTreeColumn()?1:0,n=0;n<t.length;++n)t[n]=Array(this.visibleColumns.length);return this.cellEventPool.forEach(function(n){var r=n.gridCell.x+e;r>=0&&(t[n.gridCell.y][r]=n.value)}),t},isColumnVisible:function(t){return!!this.getVisibleColumn(t)},getVisibleColumn:function(t){return this.visibleColumns.findWithNeg(function(e){return e.columnIndex===t})},getMinimumLeftPositionToShowColumn:function(t){var e,n,r,i,o=this.grid.getFixedColumnCount(),a=0,s=0,A=0,l=[],c=null,u=0,g=0,h=0;for(u=0;u<=t;u++)g=h,h+=Math.ceil(this.grid.getColumnWidth(u)),l.push({left:g,right:h});r=l[l.length-1].right,i=l[o-1],this.properties.showRowNumbers&&(a=this.grid.getColumnWidth(this.grid.behavior.rowColumnIndex)),this.grid.hasTreeColumn()&&(s=this.grid.getColumnWidth(this.grid.behavior.treeColumnIndex)),e=i?i.right:0,n=this.getBounds().width-e-a-s,u=t;do{A=u,c=l[u],u--}while(c.left+n>r&&u>=0);return A},isDataColumnVisible:function(t){return!!this.getVisibleDataColumn(t)},getVisibleDataColumn:function(t){return this.visibleColumns.findWithNeg(function(e){return e.column.index===t})},getFinalVisibleColumnBoundary:function(){var t=this.isLastColumnVisible()?2:1,e=this.visibleColumns[this.visibleColumns.length-t].right;return Math.min(e,this.getBounds().width)},isRowVisible:function(t){return!!this.visibleRows[t]},getVisibleRow:function(t){return this.visibleRows[t]},isDataRowVisible:function(t,e){return!!this.getVisibleDataRow(t,e)},getVisibleDataRow:function(t,e){return e=e||this.grid.behavior.dataModel,this.visibleRows.find(function(n){return n.subgrid===e&&n.rowIndex===t})},isSelected:function(t,e){return this.grid.isSelected(t,e)},renderGrid:function(t){var e=this.grid;e.deferredBehaviorChange();var n=e.getRowCount();if(n!==this.lastKnowRowCount){var r=function(t,e){var n=this.grid.behavior.getColumnProperties(this.grid.behavior.rowColumnIndex),r=this.grid.properties,o=2*n.cellPadding;if(r.rowHeaderCheckboxes&&i.checked&&(o+=i.checked.width),r.rowHeaderNumbers){var a=n.rowHeader;t.cache.font=a.foregroundSelectionFont.indexOf("bold ")>=0?a.foregroundSelectionFont:a.font,o+=t.getTextWidth(e)}n.preferredWidth=n.width=o}.call(this,t,n);r!==this.handleColumnWidth&&(this.needsComputeCellsBounds=!0,this.handleColumnWidth=r),this.lastKnowRowCount=n}this.needsComputeCellsBounds&&(l.call(this),this.needsComputeCellsBounds=!1,e.behavior.dataModel.fetchData)?e.behavior.dataModel.fetchData(function(){var t=this.dataWindow;if(!this.grid.properties.fetchSubregions)return[this.grid.newRectangle(t.left,t.top,t.width,t.height)];var e=this.visibleColumns.map(function(t){return t.column.index}).sort(c),n=e[0],r=e[e.length-1]-n+1;return[this.grid.newRectangle(n,t.top,r,t.height)]}.call(this),function(t,e){e||(this.gridRenderer.paintCells.call(this,t),this.grid.repaint())}.bind(this,t)):(this.gridRenderer.paintCells.call(this,t),this.renderOverrides(t),this.renderLastSelection(t))},renderLastSelection:function(t){var e,n,r,i,o,s=this.grid,A=s.properties,l=s.selectionModel;switch(l.getLastSelectionType()){case"column":var c=l.columnSelectionModel.selection,u=c[c.length-1];n=u[0],r=0,i=u[1]-n+1,o=s.getRowCount(),e=new a(n,r,i,o);break;case"row":if(!A.collapseCellSelections||"cell"!==l.getLastSelectionType(1)){var g=l.rowSelectionModel.selection,h=g[g.length-1];n=0,r=h[0],i=s.behavior.getActiveColumns().length,o=h[1]-r+1,e=new a(n,r,i,o);break}case"cell":e=l.getLastSelection()}if(e&&-1!==e.origin.x){var d,f,p=this.visibleColumnsByIndex,I=this.visibleRowsByDataRowIndex,C=this.visibleColumns[this.visibleColumns.length-1],m=this.visibleRows[this.visibleRows.length-1],y=p[this.dataWindow.origin.x],E=I[this.dataWindow.origin.y],v=A.fixedColumnCount,B=A.fixedRowCount,b=s.getHeaderRowCount();if(!((d=this.visibleColumns[0])&&e.corner.x<d.columnIndex||v&&y&&(d=this.visibleColumns[v-1])&&e.origin.x>d.columnIndex&&e.corner.x<y.columnIndex||C&&e.origin.x>C.columnIndex||(f=this.visibleRows[b])&&e.corner.y<f.rowIndex||B&&E&&(f=this.visibleRows[b+B-1])&&e.origin.y>f.rowIndex&&e.corner.y<E.rowIndex||m&&e.origin.y>m.rowIndex)){var Q=p[e.origin.x]||y,w=I[e.origin.y]||E,x=p[e.corner.x]||(e.corner.x>C.columnIndex?C:p[v-1]),_=I[e.corner.y]||(e.corner.y>m.rowIndex?m:I[B-1]);if(Q&&w&&x&&_){var D={bounds:{x:Q.left,y:w.top,width:x.right-Q.left,height:_.bottom-w.top},selectionRegionOverlayColor:this.gridRenderer.paintCells.partial?"transparent":A.selectionRegionOverlayColor,selectionRegionOutlineColor:A.selectionRegionOutlineColor};s.cellRenderers.get("lastselection").paint(t,D),"by-cells"===this.gridRenderer.paintCells.key&&(this.gridRenderer.reset=!0)}}}},renderOverrides:function(t){var e=this.grid.renderOverridesCache;for(var n in e)if(e.hasOwnProperty(n)){var r=e[n];r&&this.renderOverride(t,r)}},renderOverride:function(t,e){var n=e.hdpiratio,r=e.startX,i=e.width+1,o=e.height,a=e.ctx,s=t.getImageData(r,0,Math.round(i*n),Math.round(o*n));a.putImageData(s,0,0),t.cache.fillStyle=this.properties.backgroundColor2,t.fillRect(Math.round(r/n),0,i,o)},getScrollTop:function(){return this.grid.getVScrollValue()},getScrollLeft:function(){return this.grid.getHScrollValue()},isLastColumnVisible:function(){var t=this.grid.getColumnCount()-1;return!!this.visibleColumns.findWithNeg(function(e){return e.columnIndex===t})},getRenderedWidth:function(t){var e=this.visibleColumns;return t>=e.length?e[e.length-1].right:e[t].left},getRenderedHeight:function(t){var e=this.visibleRows;return t>=e.length?e[e.length-1].bottom:e[t].top},isDraggingColumn:function(){return this.grid.isDraggingColumn()},getPageUpRow:function(){for(var t=this.grid,e=this.getVisibleScrollHeight(),n=this.dataWindow.origin.y-this.properties.fixedRowCount-1,r=0;r<e&&n>=0;)r+=t.getRowHeight(n),n--;return n+1},getPageDownRow:function(){return this.dataWindow.corner.y-this.properties.fixedRowCount},renderErrorCell:function(t,e,n,r){var i=t&&(t.message||t)||"Unknown error.",o={x:n.left,y:r.top,width:n.width,height:r.height},a={bounds:o};console.error(i),e.cache.save(),e.beginPath(),e.rect(o.x,o.y,o.width,o.height),e.clip(),this.grid.cellRenderers.get("errorcell").paint(e,a,i),e.cache.restore()},paintGridlines:function(t){var e=this.visibleColumns,n=e.length,r=this.visibleRows,i=r.length;if(n&&i){var o,a,s,A=this.properties,l=n-1,c=i-1,u=e[l].right,g=r[c].bottom,h=A.gridLinesVColor,d=A.gridLinesHColor,f="border-box"===A.boxSizing;if(A.gridLinesV&&(A.gridLinesUserDataArea||A.gridLinesColumnHeader)){var p=A.gridLinesVWidth,I=this.grid.getHeaderRowCount(),C=r[I-1],m=r[I],y=m&&m.top,E=A.gridLinesColumnHeader?0:y,v=A.gridLinesUserDataArea?g:C&&C.bottom;void 0!==E&&void 0!==v&&(t.cache.fillStyle=h,e.forEachWithNeg(function(e,n){if(e&&n<l){var r=e.right,i=Math.max(E,e.top||0),o=Math.min(v,e.bottom||1/0)-i;f&&(r-=p),t.fillRect(r,i,p,o),A.gridLinesUserDataArea&&e.bottom<y&&t.fillRect(r,y,p,v-y)}}))}if(A.gridLinesH&&(A.gridLinesUserDataArea||(o=A.gridLinesRowHeader&&(e[-1]||e[-2])))){var B=A.gridLinesHWidth,b=A.gridLinesRowHeader?0:e[0].left,Q=A.gridLinesUserDataArea?u:o.right;t.cache.fillStyle=d,r.forEach(function(e,n){if(n<c){var r=e.bottom;f&&(r-=B),t.fillRect(b,r,Q-b,B)}})}void 0!==A.fixedLinesHWidth&&(s=r.gap)&&(t.cache.fillStyle=A.fixedLinesHColor||d,(a=A.fixedLinesHEdge)?(t.fillRect(0,s.top,u,a),t.fillRect(0,s.bottom-a,u,a)):t.fillRect(0,s.top,u,s.bottom-s.top)),void 0!==A.fixedLinesVWidth&&(s=e.gap)&&(t.cache.fillStyle=A.fixedLinesVColor||h,(a=A.fixedLinesVEdge)?(t.fillRect(s.left,0,a,g),t.fillRect(s.right-a,0,a,g)):t.fillRect(s.left,0,s.right-s.left,g))}},_paintCell:function(t,e,n){var r,i,o=this.grid,a=o.selectionModel,s=e.isHandleColumn,A=e.isTreeColumn,l=e.isColumnSelected,c=e.isDataRow,u=e.isRowSelected,g=e.isCellSelected,h=e.isHeaderRow,d=e.isFilterRow,f=s||A,p=!f&&c,I=this.assignProps(e),C=(I.gridCell=e.gridCell).x,m=(I.dataCell=e.dataCell).y;s?(i=u||a.isCellSelectedInRow(m),I.halign="right"):A?(i=u||a.isCellSelectedInRow(m),I.halign="left"):i=c?g||u||l:!d&&(!!l||a.isCellSelectedInColumn(C)),s?c?I.rowHeaderNumbers&&(r=m+1):h&&(I.allRowsSelected=a.areAllRowsSelected()):(I.dataRow=e.dataRow,r=e.value),I.isSelected=i,I.isDataColumn=!f,I.isHandleColumn=s,I.isTreeColumn=A,I.isDataRow=c,I.isHeaderRow=h,I.isFilterRow=d,I.isUserDataArea=p,I.isColumnHovered=e.isColumnHovered,I.isRowHovered=e.isRowHovered,I.isCellHovered=e.isCellHovered,I.bounds=e.bounds,I.isCellSelected=g,I.isRowSelected=u,I.isColumnSelected=l,I.isInCurrentSelectionRectangle=a.isInCurrentSelectionRectangle(C,m),I.prefillColor=n,o.mouseDownState&&(I.mouseDown=o.mouseDownState.gridCell.equals(e.gridCell)),I.subrow=0;var y=p&&r&&r.constructor===Array,E=y&&r.subrows&&r.length;if(E){var v=I.bounds=Object.assign({},I.bounds);v.height/=E,I.subrows=E,I.value=I.exec(r[0])}else E=1,I.value=!y&&p?I.exec(r):r;for(;;){var B=e.subgrid.getCell(I,I.renderer);if(I.formatValue=o.getFormatter(I.format),I.snapshot=e.snapshot[I.subrow],I.minWidth=e.minWidth,B.forEach?B.forEach(function(e){e.paint(t,I)}):B.paint(t,I),e.snapshot[I.subrow]=I.snapshot,(void 0===e.minWidth||I.minWidth>e.minWidth)&&(e.minWidth=I.minWidth),++I.subrow===E)break;v.y+=v.height,I.value=I.exec(r[I.subrow])}return e.clickRect=I.clickRect,e.cellRenderer=B,I.minWidth},assignProps:o,findCell:function(t,e,n){var r,i,o=this.cellEventPool;"object"==typeof t?(n=t.subgrid,e=t.dataCell.y,r=t.dataCell.x):r=t,n=n||this.grid.behavior.dataModel;var a=this.visibleColumns.length;this.grid.properties.showRowNumbers&&a++,this.grid.behavior.hasTreeColumn()&&a++,a*=this.visibleRows.length;for(var s=0;s<a;++s)if((i=o[s]).subgrid===n&&i.dataCell.x===r&&i.dataCell.y===e)return i},resetCellPropertiesCache:function(t,e,n){var r=this.findCell.apply(this,arguments);return r&&(r._cellOwnProperties=void 0),r},resetAllCellPropertiesCaches:function(){this.cellEventPool.forEach(function(t){t._cellOwnProperties=void 0})},getBounds:function(){return this.bounds},setBounds:function(t){return this.bounds=t},setInfo:function(t){var e;this.visibleColumns.length&&(e=this.visibleColumns[this.visibleColumns.length-1].right),this.grid.canvas.setInfo(t,e)}});function l(){var t,e,n,r,i,o,s,A,l,c,u,g,h,d,f,p,I,C,m,y,E,v,B,b,Q,w,x,_,D,M,k,S,L,R,N,F,T,U,O,G,H=this.getScrollTop(),j=this.getScrollLeft(),Y=this.getBounds(),P=this.grid,K=P.behavior,z=K.hasTreeColumn(),J=K.treeColumnIndex,q=P.cellEditor&&P.cellEditor.event,V=0,W=0,X=P.properties,Z="border-box"===X.boxSizing,$=X.gridLinesVWidth,tt=Z?0:$,et=X.gridLinesHWidth,nt=Z?0:et,rt=this.grid.getFixedColumnCount(),it=this.grid.getFixedRowCount(),ot=K.leftMostColIndex,at=0,st=-3,At=-1,lt=K.subgrids;for(q&&(e=q.gridCell.x,r=q.dataCell.y,i=q.subgrid),X.showRowNumbers&&(st=z?J:0,at+=1),z&&(st=0,at+=1),rt&&(st=rt),it&&(At=it),void 0===X.fixedLinesVWidth?st=-3:(m=Math.max(X.fixedLinesVWidth||$,$),E=(y=Z?m-$:m)-m),void 0===X.fixedLinesHWidth?At=-1:(v=Math.max(X.fixedLinesHWidth||et,et),b=(B=Z?v-et:v)-v),this.scrollHeight=0,this.visibleColumns.length=0,this.visibleColumns.gap=this.visibleColumns[-1]=this.visibleColumns[-2]=void 0,this.visibleRows.length=0,this.visibleRows.gap=void 0,this.visibleColumnsByIndex=[],this.visibleRowsByDataRowIndex={},this.insertionBounds=[],s=0,u=ot,g=P.getColumnCount(),A=Y.width||P.canvas.width;u<g&&s<=A;u++)if(z||u!==J){if((G=u===st)?(this.visibleColumns.gap={left:S.right+E,right:void 0},s+=y):s&&(s+=tt),D=u,u>=rt&&(F=D+=j,void 0===N&&(N=F)),D>=g)break;L=Math.ceil(K.getColumnWidth(D)),this.visibleColumns[u]=this.visibleColumnsByIndex[D]=S={index:u,columnIndex:D,column:K.getActiveColumn(D),left:s,width:L,right:s+L},G&&(this.visibleColumns.gap.right=S.left),e===D&&(t=S),s+=L,V+=Math.round(L/2)+W,this.insertionBounds.push(V),W=Math.round(L/2)}else this.visibleColumns[u]=void 0;for(_=X.defaultRowHeight*K.getFooterRowCount(),C=f=h=l=0,d=lt.length,c=Y.height-_;h<d;h++,C+=I){for(I=(Q=lt[h]).getRowCount(),x=Q.isData,o=i===Q,O=f,p=f+I;f<p&&l<c&&((G=x&&f===At)?(this.visibleRows.gap={top:k.bottom+b,bottom:void 0},l+=B):l&&(l+=nt),M=f,!(x&&f>=it&&(U=(M+=H)-C,void 0===T&&(T=U),M>=p)));f++)w=M-C,R=K.getRowHeight(w,Q),this.visibleRows[f]=k={index:f,subgrid:Q,rowIndex:w,top:l,height:R,bottom:l+R},G&&(this.visibleRows.gap.bottom=k.top),x&&(this.visibleRowsByDataRowIndex[M-C]=k),o&&r===w&&(n=k),l+=R;x&&(I=f-O)}q&&(q.visibleColumn=t,q.visibleRow=n,q.gridCell.y=n&&n.index,q._bounds=null),this.dataWindow=new a(N,T,Math.min(F-N+1,this.visibleColumns.length),Math.min(U-T+1,this.visibleRows.length));var ct=this.cellEventPool,ut=ct.length,gt=(this.visibleColumns.length+at)*this.visibleRows.length;gt>ut&&(ct.length=gt);for(var ht=ut;ht<gt;ht++)ct[ht]=new K.CellEvent;this.resetAllGridRenderers()}function c(t,e){return t-e}function u(t){s.indexOf(t)<0&&s.push(t)}u(n(153)),u(n(154)),u(n(155)),u(n(71)),u(n(156)),A.registerGridRenderer=u,t.exports=A},function(t,e,n){"use strict";var r=n(50),i=r.propClassEnum.COLUMNS,o=r.propClassEnum.CELLS,a=r.propClassEnum.ROWS,s=r.propClassEnum.STRIPES,A=[];A[i]=function(t){return t.columnProperties},A[o]=function(t){return t.cellOwnProperties},A[s]=function(t){var e=t.isDataRow&&t.columnProperties.rowStripes;return e&&e[t.dataCell.y%e.length]},A[a]=function(t){return t.rowOwnProperties},t.exports=function(t){var e,n,r,l=t.properties.propClassLayers;switch(l[0]){case a:e=1,n=t.rowOwnProperties||this.grid.properties;break;case i:l[1]===o?(e=2,n=t.properties):(e=1,n=t.columnProperties);break;case o:l[1]===i?(n=t.cellOwnProperties)?e=1:(e=2,n=t.columnProperties):l[2]===i||l[3]===i?(e=1,n=t.properties):(e=0,n=this.grid.properties);break;case s:e=0,n=this.grid.properties}for(r=[Object.create(n)];e<l.length;++e)r.push(A[l[e]](t));return Object.assign.apply(Object,r)}},function(t,e,n){"use strict";var r=n(71);function i(t){var e,n,i,o,a=this.visibleColumns,s=this.visibleRows,A=a.length,l=A-1,c=s.length,u=0,g=this.cellEventPool,h=c?s[c-1].bottom:0;A&&c&&(this.gridRenderer.reset&&(this.resetAllGridRenderers(),r.call(this,t),this.gridRenderer.reset=!1),this.visibleColumns.forEachWithNeg(function(r,a){for(r=(e=g[u]).visibleColumn,o=r.column.properties.columnClip,t.clipSave(o||null===o&&a===l,0,0,r.right,h),i=n=0;n<c;n++,u++){e=g[u];try{i=Math.max(i,this._paintCell(t,g[u]))}catch(e){this.renderErrorCell(e,t,r,g[u].visibleRow)}}t.clipRestore(o),e.column.properties.preferredWidth=Math.round(i)},this),"border-box"===this.grid.properties.boxSizing&&this.paintGridlines(t))}i.key="by-cells",i.partial=!0,t.exports=i},function(t,e,n){"use strict";var r=n(51);function i(t){var e,n,i,o,a,s,A,l,c=this.grid.properties.backgroundColor,u=this.visibleColumns,g=this.visibleRows,h=u.length,d=h-1,f=g.length,p=this.cellEventPool,I=h?u[d].right:0,C=f?g[f-1].bottom:0;if(t.clearRect(0,0,this.bounds.width,this.bounds.height),h&&f){for(t.alpha(c)>0&&(t.cache.fillStyle=c,t.fillRect(0,0,I,C)),this.gridRenderer.reset?(this.resetAllGridRenderers(["by-columns-discrete"]),this.gridRenderer.reset=!1,r.call(this,!0)):this.gridRenderer.rebundle&&(this.gridRenderer.rebundle=!1,r.call(this)),a=(o=this.columnBundles).length;a--;)i=o[a],t.clearFill(i.left,0,i.right-i.left,C,i.backgroundColor);var m=0;this.visibleColumns.forEachWithNeg(function(r,i){for(r=(n=p[m]).visibleColumn,e=n.column.properties.backgroundColor,l=r.column.properties.columnClip,t.clipSave(l||null===l&&i===d,0,0,r.right,C),A=s=0;s<f;s++,m++){n=p[m];try{A=Math.max(A,this._paintCell(t,n,e))}catch(e){this.renderErrorCell(e,t,r,n.visibleRow)}}t.clipRestore(l),n.column.properties.preferredWidth=Math.round(A)},this),this.paintGridlines(t)}}i.key="by-columns",i.rebundle=!0,t.exports=i},function(t,e,n){"use strict";var r=n(51);function i(t){var e,n,i,o,a,s=this.visibleColumns,A=this.visibleRows,l=s.length,c=l-1,u=A.length,g=this.cellEventPool,h=u?A[u-1].bottom:0;if(t.clearRect(0,0,this.bounds.width,this.bounds.height),l&&u){this.gridRenderer.reset&&(this.resetAllGridRenderers(["by-columns"]),this.gridRenderer.reset=!1,r.call(this,!0));var d=0;this.visibleColumns.forEachWithNeg(function(r,s){for(r=(n=g[d]).visibleColumn,e=n.column.properties.backgroundColor,t.clearFill(r.left,0,r.width,h,e),a=r.column.properties.columnClip,t.clipSave(a||null===a&&s===c,0,0,r.right,h),o=i=0;i<u;i++,d++){n=g[d];try{o=Math.max(o,this._paintCell(t,n,e))}catch(e){this.renderErrorCell(e,t,r,n.visibleRow)}}t.clipRestore(a),n.column.properties.preferredWidth=Math.round(o)},this),this.paintGridlines(t)}}i.key="by-columns-discrete",t.exports=i},function(t,e,n){"use strict";var r=n(72);function i(t){var e,n,i,o,a,s,A,l,c=this.grid.properties,u=c.backgroundColor,g=this.rowBundles,h=this.visibleColumns,d=this.visibleRows,f=h.length,p=f-1,I=d.length,C=this.cellEventPool,m=Array(f-0).fill(0),y=f?h[f-1].right:0,E=I?d[I-1].bottom:0,v=c.gridLinesH,B=c.gridLinesHWidth,b=c.gridLinesHColor;if(t.clearRect(0,0,this.bounds.width,this.bounds.height),f&&I){for(t.alpha(u)>0&&(t.cache.fillStyle=u,t.fillRect(0,0,y,E)),this.gridRenderer.reset&&(this.resetAllGridRenderers(),this.gridRenderer.reset=!1,r.call(this,!0)),n=this.rowPrefillColors,s=g.length;s--;)o=g[s],t.clearFill(0,o.top,y,o.bottom-o.top,o.backgroundColor);for(A=0,s=0;s<I;s++)e=n[s],v&&(t.cache.fillStyle=b,t.fillRect(0,C[A].visibleRow.bottom,y,B)),this.visibleColumns.forEachWithNeg(function(n){n=(i=C[++A]).visibleColumn,l=n.column.properties.columnClip,t.clipSave(l||null===l&&a===p,0,0,n.right,E);try{m[a]=Math.max(m[a],this._paintCell(t,i,e))}catch(e){this.renderErrorCell(e,t,n,void 0)}t.clipRestore(l)},this);this.paintGridlines(t),this.visibleColumns.forEachWithNeg(function(t,e){t.column.properties.preferredWidth=Math.round(m[e])})}}i.key="by-rows",t.exports=i},function(t,e,n){"use strict";var r=n(73),i=n(70);function o(t){this.grid=t,this.reset()}o.prototype={constructor:o.prototype.constructor,allRowsSelected:!1,reset:function(){this.selections=[],this.flattenedX=[],this.flattenedY=[],this.rowSelectionModel=new r,this.columnSelectionModel=new r,this.lastSelectionType=[]},getLastSelection:function(){var t=this.selections;return t[t.length-1]},getLastSelectionType:function(t){return this.lastSelectionType[t||0]||""},setLastSelectionType:function(t,e){var n=this.lastSelectionType.indexOf(t);(0!==n||e)&&(n>=0&&this.lastSelectionType.splice(n,1),e||this.lastSelectionType.unshift(t))},select:function(t,e,n,r,o){var a=new i(t,e,n+1,r+1);a.firstSelectedCell=this.grid.newPoint(t,e),a.lastSelectedCell=a.firstSelectedCell.x===a.origin.x&&a.firstSelectedCell.y===a.origin.y?a.corner:a.origin,this.grid.properties.multipleSelections?(this.selections.push(a),this.flattenedX.push(a.flattenXAt(0)),this.flattenedY.push(a.flattenYAt(0))):(this.selections[0]=a,this.flattenedX[0]=a.flattenXAt(0),this.flattenedY[0]=a.flattenYAt(0)),this.setLastSelectionType("cell"),this.grid.selectionChanged(o)},toggleSelect:function(t,e,n,r){var i=this.selections.findIndex(function(i){return i.origin.x===t&&i.origin.y===e&&i.extent.x===n&&i.extent.y===r});i>=0?(this.selections.splice(i,1),this.flattenedX.splice(i,1),this.flattenedY.splice(i,1),this.setLastSelectionType("cell",!this.selections.length),this.grid.selectionChanged()):this.select(t,e,n,r)},clearMostRecentSelection:function(t){t||this.setAllRowsSelected(!1),this.selections.length&&--this.selections.length,this.flattenedX.length&&--this.flattenedX.length,this.flattenedY.length&&--this.flattenedY.length,this.setLastSelectionType("cell",!this.selections.length)},clearMostRecentColumnSelection:function(){this.columnSelectionModel.clearMostRecentSelection(),this.setLastSelectionType("column",!this.columnSelectionModel.selection.length)},clearMostRecentRowSelection:function(){this.rowSelectionModel.clearMostRecentSelection(),this.setLastSelectionType("row",!this.rowSelectionModel.selection.length)},clearRowSelection:function(){this.rowSelectionModel.clear(),this.setLastSelectionType("row",!this.rowSelectionModel.selection.length)},getSelections:function(){return this.selections},hasSelections:function(){return 0!==this.selections.length},hasRowSelections:function(){return!this.rowSelectionModel.isEmpty()},hasColumnSelections:function(){return!this.columnSelectionModel.isEmpty()},isCellSelectedInRow:function(t){return this._isCellSelected(this.flattenedX,0,t)},isCellSelectedInColumn:function(t){return this._isCellSelected(this.flattenedY,t,0)},isSelected:function(t,e){return this.isColumnSelected(t)||this.isRowSelected(e)||this._isCellSelected(this.selections,t,e)},isCellSelected:function(t,e){return this._isCellSelected(this.selections,t,e)},_isCellSelected:function(t,e,n){var r=this;return!!t.find(function(t){return r.rectangleContains(t,e,n)})},clear:function(t){this.selections.length=0,this.flattenedX.length=0,this.flattenedY.length=0,this.columnSelectionModel.clear(),t?this.lastSelectionType.indexOf("row")>=0?this.lastSelectionType=["row"]:this.lastSelectionType.length=0:(this.lastSelectionType.length=0,this.setAllRowsSelected(!1),this.rowSelectionModel.clear())},isRectangleSelected:function(t,e,n,r){return!!this.selections.find(function(i){return i.origin.x===t&&i.origin.y===e&&i.extent.x===n&&i.extent.y===r})},isColumnSelected:function(t){return this.columnSelectionModel.isSelected(t)},isRowSelected:function(t){return this.allRowsSelected||this.rowSelectionModel.isSelected(t)},selectColumn:function(t,e){this.columnSelectionModel.select(t,e),this.setLastSelectionType("column",!this.columnSelectionModel.selection.length)},selectAllRows:function(){this.clear(),this.setAllRowsSelected(!0)},setAllRowsSelected:function(t){this.allRowsSelected=t},areAllRowsSelected:function(){return this.allRowsSelected},selectRow:function(t,e){this.rowSelectionModel.select(t,e),this.setLastSelectionType("row",!this.rowSelectionModel.selection.length)},deselectColumn:function(t,e){this.columnSelectionModel.deselect(t,e),this.setLastSelectionType("column",!this.columnSelectionModel.selection.length)},deselectRow:function(t,e){this.areAllRowsSelected()&&(this.setAllRowsSelected(!1),this.rowSelectionModel.select(0,this.grid.getRowCount()-1)),this.rowSelectionModel.deselect(t,e),this.setLastSelectionType("row",!this.rowSelectionModel.selection.length)},getSelectedRows:function(){if(this.areAllRowsSelected()){for(var t=this.grid.getRowCount(),e=new Array(t),n=0;n<t;n++)e[n]=n;return e}return this.rowSelectionModel.getSelections()},getSelectedColumns:function(){return this.columnSelectionModel.getSelections()},isColumnOrRowSelected:function(){return!this.columnSelectionModel.isEmpty()||!this.rowSelectionModel.isEmpty()},getFlattenedYs:function(){var t=[],e={};return this.selections.forEach(function(n){for(var r=n.origin.y,i=n.height,o=0;o<i;o++){var a=o+r;e[a]||(t.push(a),e[a]=!0)}}),t.sort(function(t,e){return t-e}),t},selectRowsFromCells:function(t,e){t=t||0;var n=this.rowSelectionModel;e||(this.setAllRowsSelected(!1),n.clear()),this.selections.forEach(function(e){var r=e.origin.y,i=e.extent.y;r+=t,n.select(r,r+i)})},selectColumnsFromCells:function(t){t=t||0;var e=this.columnSelectionModel;e.clear(),this.selections.forEach(function(n){var r=n.origin.x,i=n.extent.x;r+=t,e.select(r,r+i)})},isInCurrentSelectionRectangle:function(t,e){var n=this.getLastSelection();return n&&this.rectangleContains(n,t,e)},rectangleContains:function(t,e,n){var r=t.origin.x,i=t.origin.y,o=r+t.extent.x,a=i+t.extent.y;return t.extent.x<0&&(r=o,o=t.origin.x),t.extent.y<0&&(i=a,a=t.origin.y),e>=r&&n>=i&&e<=o&&n<=a}},t.exports=o},function(t,e,n){"use strict";var r=n(44),i=n(52),o=n(159),a=n(163),s=r.extend("Local",{initialize:function(t,e){this.setData(e)},get schema(){return this.dataModel.getSchema()},set schema(t){this.dataModel.setSchema(t)},dataModelEventHandlers:n(74).dataModelEventHandlers,createColumns:function(){this.super.createColumns.call(this,A)},createDataRowProxy:function(){var t=this.dataModel,e={};Object.defineProperty(e,"$y$",{enumerable:!1,writable:!0}),this.schema.forEach(function(n,r){Object.defineProperty(e,n.name,{enumerable:!0,get:function(){return t.getValue(r,this.$y$)},set:function(e){return t.setValue(r,this.$y$,e)}})}),t.$rowProxy$=e},getNewDataModel:function(t){var e;return(t=t||{}).dataModel?e=t.dataModel:[].concat(t.DataModel||i).forEach(function(t){e=new t(e)}),e},resetDataModel:function(t){var e=this.getNewDataModel(t),n=e&&e!==this.dataModel;return n&&(this.dataModel=this.decorateDataModel(e,t),o.addDeprecationWarnings.call(this),o.addFriendlierDrillDownMapKeys.call(this)),n},decorateDataModel:function(t,e){return o.injectPolyfills(t),o.injectCode(t),o.injectDefaulthooks(t),t.setMetadataStore(e&&e.metadata),this.boundDispatchEvent=this.boundDispatchEvent||a.bind(this.grid),t.addListener(this.boundDispatchEvent),t},get charMap(){return this.dataModel.drillDownCharMap},cellClicked:function(t){return this.dataModel.toggleRow(t.dataCell.y,t.dataCell.x)},hasTreeColumn:function(){return this.dataModel.isTree()&&this.grid.properties.showTreeColumn}});function A(){this.schema.forEach(function(t){this.addColumn(t)},this),this.columnEnumSynchronize()}s.prototype.mixIn(n(75).mixin),s.prototype.mixIn.call(s,n(75).mixInShared),t.exports=s},function(t,e,n){"use strict";var r=n(160),i=n(161),o=n(162),a={},s={true:"OPEN",false:"CLOSE",null:"INDENT"};t.exports={injectPolyfills:function(t){Object.keys(o).forEach(function(e){t[e]||(t[e]=o[e])})},injectCode:function(t){t.install(i,{inject:!0})},addDeprecationWarnings:function(){var t=this.grid;Object.defineProperties(this.dataModel,{grid:{configurable:!0,enumerable:!1,get:function(){return a.grid||(console.warn("dataModel.grid has been deprecated as of v3.0.0. (Will be removed in a future release.) Data models should have no direct knowledge of or access to the grid. (If your data model needs to call grid methods, add a data event to your grid with `grid.addEventListener('fin-hypergrid-data-my-event', myHandler)` and trigger it from your data model with `this.dispatchEvent('fin-hypergrid-data-my-event')` or `this.dispatchEvent({ type: 'fin-hypergrid-data-my-event' })`. If you need access to the grid object from within a `getCell` or `getCellEditAt` override, define `grid` in the same closure as the override.)"),a.grid=!0),t}},dataSource:{configurable:!0,enumerable:!1,get:function(){return a.dataSource||(console.warn("dataModel.dataSource has been deprecated as of 3.0.0 in favor of `dataModel`. (Will be removed in a future release.) The _external_ data model, formerly `grid.behavior.dataModel.dataSource`, is now `grid.behavior.dataModel`."),a.dataSource=!0),this.dataModel}}})},addFriendlierDrillDownMapKeys:function(){var t=this.dataModel.drillDownCharMap;t&&Object.keys(s).forEach(function(e){if(e in t){var n=s[e];n in t||Object.defineProperty(t,n,{get:function(){return this[e]},set:function(t){this[e]=t}})}})},injectDefaulthooks:function(t){t.install(r,{inject:!0})}}},function(t,e,n){"use strict";e.getCell=function(t,e){return t.grid.cellRenderers.get(e)},e.getCellEditorAt=function(t,e,n,r){return r.grid.cellEditors.create(n,r)}},function(t,e,n){"use strict";t.exports={apply:function(){},isTree:function(){return!1},isTreeCol:function(t){return!1},toggleRow:function(t,e,n){},getColumnCount:function(){return this.getSchema().length},getRow:function(t){return this.$rowProxy$.$y$=t,this.$rowProxy$},getData:function(t){var e,n,r,i=this.getRowCount(),o=new Array(i);for(e=0;e<i;e++)(n=this.getRow(e))&&(o[e]=Object.assign({},n),t&&(r=this.getRowMetadata(e))&&(o[e][t]=r));return o},setData:function(t){},setValue:function(t,e,n){console.warn("dataModel.setValue("+t+", "+e+', "'+n+'") called but no implementation. Data not saved.')},setSchema:function(t){console.warn("dataModel.setSchema(schema) called but no implementation. Schema not updated.")},getRowIndex:function(t){return t},getRowMetadata:function(t,e){return this.metadata[t]||void 0!==e&&(this.metadata[t]=Object.create(e))},getMetadataStore:function(){return this.metadata},setRowMetadata:function(t,e){return e?this.metadata[t]=e:delete this.metadata[t],e},setMetadataStore:function(t){this.metadata=t||[]}}},function(t,e,n){"use strict";function r(){}t.exports={install:function(t){if(t){var e=Array.isArray(t);(e?t:Object.keys(t).filter(function(e){return"function"==typeof t[e]&&"constructor"!==e&&"initialize"!==e})).forEach(function(n){this[n]||(this[n]=e?r:t[n])},this)}},dispatchEvent:function(t){this.handlers&&this.handlers.forEach(function(e){e.call(this,t)},this)},addListener:function(t){this.handlers?this.handlers.indexOf(t)<0&&this.handlers.push(t):this.handlers=[t]},removeListener:function(t){if(this.handlers&&this.handlers.length){var e=this.handlers.indexOf(t);e>=0&&delete this.handlers[e]}},removeAllListeners:function(){this.handlers&&(this.handlers.length=0)}}},function(t,e,n){"use strict";var r=n(74),i=n(49),o=/^fin-hypergrid-(data|schema)(-[a-z]+)+$/;t.exports=function(t){var e;switch(typeof t){case"string":t={type:e=t};break;case"object":if("type"in t){e=t.type;break}default:throw new this.HypergridError("Expected data model event to be: (string | {type:string})")}if(!o.test(e))throw new this.HypergridError('Expected data model event type "'+e+'" to match '+o+".");var n=r[t.type];if(n)var a=n.call(this,t);return void 0!==a?a:i.call(this,t.type,t)}},function(t,e,n){"use strict";var r,i=n(30),o=i.extend("CellRenderers",{BaseClass:n(4),initialize:function(){this.add(n(76)),this.add(n(165)),this.add(n(166)),this.add(n(167)),this.add(n(168)),this.add(n(169)),this.add(n(170)),this.add(n(171)),this.add(n(172)),this.add("emptycell",this.BaseClass)},add:function(t,e){return 1===arguments.length?(e=t,i.prototype.add.call(this,new e)):i.prototype.add.call(this,t,new e)},get:function(t){if(t.map)return t.map(function(t){return i.prototype.get.call(this,t)},this);var e=i.prototype.get.call(this,t);return e===this.items.emptycell&&(r||(console.warn('grid.cellRenderers.get("'+t+'").constructor has been deprecated as of v2.1.0 in favor of grid.cellRenderers.BaseClass property. (Will be removed in a future release.)'),r=!0),this.BaseClass.constructor=this.BaseClass),e}});t.exports=new o},function(t,e,n){"use strict";var r=n(4),i=n(2).Rectangle,o=n(18),a=/\s\s+/g,s=r.extend("SimpleCell",{paint:function(t,e){var n,r,s,c,u,g,h,d,f,p,I,C,m,y,E,v,B=e.value,b=e.bounds,Q=b.x,w=b.y,x=b.width,_=b.height,D=e.iconPadding,M=void 0===e.prefillColor,k=e.snapshot,S=k&&M,L=0;B&&B.constructor===Array?(u=B[0],g=B[2],(B=e.exec(B[1]))&&void 0!==B.naturalWidth&&(h=B,B=null)):(u=o[e.leftIcon],h=o[e.centerIcon],g=o[e.rightIcon]);var R=B||e.renderFalsy&&0==B;if(R?(B=e.formatValue(B,e),r=e.isSelected?e.foregroundSelectionFont:e.font,n=t.cache.strokeStyle=e.isSelected?e.foregroundSelectionColor:e.color):B="",S=S&&B===k.value&&r===k.textFont&&n===k.textColor,v=[],E=0,e.isCellHovered&&e.hoverCellHighlight.enabled?I=e.hoverCellHighlight.backgroundColor:e.isRowHovered&&(p=e.hoverRowHighlight).enabled?I=e.isDataColumn||!p.header||void 0===p.header.backgroundColor?p.backgroundColor:p.header.backgroundColor:e.isColumnHovered&&(p=e.hoverColumnHighlight).enabled&&(I=e.isDataRow||!p.header||void 0===p.header.backgroundColor?p.backgroundColor:p.header.backgroundColor),t.alpha(I)<1&&(e.isSelected&&(C=e.backgroundSelectionColor),t.alpha(C)<1&&((y=e.backgroundColor===e.prefillColor)||(m=!0,v.push(e.backgroundColor),S=S&&m===k.foundationColor&&e.backgroundColor===k.colors[E++])),void 0!==C&&(v.push(C),S=S&&C===k.colors[E++])),void 0!==I&&(v.push(I),S=S&&I===k.colors[E++]),!S||E!==k.colors.length){if(e.snapshot={value:B,textColor:n,textFont:r,foundationColor:m,colors:v},l(t,v,Q,w,x,_,m),d=u?D+u.width+D:e.cellPadding,f=g?D+g.width+D:e.cellPadding,R&&(t.cache.fillStyle=n,t.cache.font=r,L=e.isHeaderRow&&e.headerTextWrapping?function(t,e,n,r,i){var o=e.bounds.x,s=e.bounds.y,l=e.bounds.width,c=e.bounds.height,u=(n+"").trim().replace(a," "),g=function t(e,n,r,i){if(1===r.length)return r;for(var o,a=[r.shift()];(o=e.getTextWidth(a.join(" "))<i)&&r.length;)a.push(r.shift());return!o&&a.length>1&&r.unshift(a.pop()),a=[a.join(" ")],r.length&&(a=a.concat(t(e,n,r,i))),a}(t,e,u.split(" "),l);if(1===g.length)return A(t,e,u,r,i);var h=r,d=e.voffset,f=e.halign,p=t.getTextHeight(e.font).height;switch(f){case"right":h=l-i;break;case"center":h=l/2}var I=Math.ceil(p/2);d+=Math.ceil((c-(g.length-1)*p)/2),h=Math.max(0,h),d=Math.max(I,d),t.cache.save(),t.beginPath(),t.rect(o,s,l,c),t.clip(),t.cache.textAlign=f,t.cache.textBaseline="middle";for(var C=0;C<g.length;C++)t.simpleText(g[C],o+h,s+d+C*p);return t.cache.restore(),l}(t,e,B,d,f):A(t,e,B,d,f)),h&&(c=Math.round((_-h.height)/2),s=x-Math.round((x-h.width)/2)-h.width,t.drawImage(h,Q+s,w+c,h.width,h.height),L=D+h.width+D,"center"===e.hotIcon&&(e.clickRect=new i(s,c,h.width,h.height))),u&&(c=Math.round((_-u.height)/2),t.drawImage(u,Q+D,w+c,u.width,u.height),"left"===e.hotIcon&&(e.clickRect=new i(D,c,u.width,u.height))),g){var N=Q+(s=x-(g.width+D));y&&(m=!0,v.unshift(e.backgroundColor)),l(t,v,N,w,f,_,m),c=Math.round((_-g.height)/2),t.drawImage(g,N,w+c,g.width,g.height),"right"===e.hotIcon&&(e.clickRect=new i(s,c,g.width,g.height))}e.cellBorderThickness&&(t.beginPath(),t.rect(Q,w,x,_),t.cache.lineWidth=e.cellBorderThickness,t.cache.strokeStyle=e.cellBorderStyle,t.stroke(),t.closePath()),e.minWidth=d+L+f}}});function A(t,e,n,r,i){var o,a,s=e.bounds.x,A=e.bounds.y,l=e.bounds.width,c=r,u=e.halign;if(e.columnAutosizing)switch(o=(a=t.getTextWidthTruncated(n,l-r,e.truncateTextWithEllipsis)).width,n=a.string||n,u){case"right":c=l-i-a.width;break;case"center":c=(l-a.width)/2}else if(o=0,void 0!==(a=t.getTextWidthTruncated(n,l-r,e.truncateTextWithEllipsis,!0)).string)n=a.string;else switch(u){case"right":c=l-i-a.width;break;case"center":c=(l-a.width)/2}return null!==n&&(s+=Math.max(r,c),A+=Math.floor(e.bounds.height/2),e.isUserDataArea&&(e.link&&(!e.isCellHovered&&e.linkOnHover||(e.linkColor&&(t.cache.strokeStyle=e.linkColor),t.beginPath(),function(t,e,n,r,i,o){var a=e.getTextHeight(t.font).height,s=e.getTextWidth(n);switch(e.cache.textAlign){case"center":r-=s/2;break;case"right":r-=s}i=Math.ceil(i)+Math.round(a/2)-.5,e.cache.lineWidth=1,e.moveTo(r,i),e.lineTo(r+s,i)}(e,t,n,s,A),t.stroke(),t.closePath()),!e.linkColor||!e.isCellHovered&&e.linkColorOnHover||(t.cache.fillStyle=e.linkColor)),!0===e.strikeThrough&&(t.beginPath(),function(t,e,n,r,i,o){var a=e.getTextWidth(n);switch(e.cache.textAlign){case"center":r-=a/2;break;case"right":r-=a}i=Math.round(i)+.5,e.cache.lineWidth=1,e.moveTo(r-1,i),e.lineTo(r+a+1,i)}(0,t,n,s,A),t.stroke(),t.closePath())),t.cache.textAlign="left",t.cache.textBaseline="middle",t.simpleText(n,s,A)),o}function l(t,e,n,r,i,o,a){for(var s=0;s<e.length;s++)a&&!s?t.clearFill(n,r,i,o,e[s]):(t.cache.fillStyle=e[s],t.fillRect(n,r,i,o))}t.exports=s},function(t,e,n){"use strict";var r=n(4).extend("Slider",{paint:function(t,e){var n=e.bounds.x,r=e.bounds.y,i=e.bounds.width,o=e.bounds.height;t.cache.strokeStyle="white";var a=e.value,s=o/2,A=i*a,l=e.isSelected?e.backgroundColor:"#333333",c=t.createLinearGradient(n,r,n,r+o);c.addColorStop(0,l),c.addColorStop(1,"#666666");var u=t.createLinearGradient(n,r,n,r+o);u.addColorStop(0,"#aaaaaa"),u.addColorStop(1,"#777777"),t.cache.fillStyle=c,this.roundRect(t,n,r,i,o,s,c),t.cache.fillStyle=a<1?u:"#eeeeee",t.beginPath(),t.arc(n+Math.max(A-s,s),r+s,s,0,2*Math.PI),t.fill(),t.closePath(),e.minWidth=100}});t.exports=r},function(t,e,n){"use strict";var r=n(4).extend("SparkBar",{paint:function(t,e){var n=e.bounds.x,r=e.bounds.y,i=e.bounds.width,o=e.bounds.height;t.beginPath();var a=e.value;if(a&&a.length){var s=a.length,A=i/s,l=e.isSelected?e.foregroundSelectionColor:e.color;(e.backgroundColor||e.isSelected)&&(t.cache.fillStyle=e.isSelected?"blue":e.backgroundColor,t.fillRect(n,r,i,o)),t.cache.fillStyle=l;for(var c=0;c<a.length;c++){var u=a[c]/110*o;t.fillRect(n+5,r+o-u,.6666*A,u),n+=A}t.closePath(),e.minWidth=10*s}}});t.exports=r},function(t,e,n){"use strict";var r=n(4).extend("LastSelection",{paint:function(t,e){var n=t.alpha(e.selectionRegionOverlayColor)>0,r=t.alpha(e.selectionRegionOutlineColor)>0;if(n||r){var i=e.bounds.x,o=e.bounds.y,a=e.bounds.width,s=e.bounds.height;t.beginPath(),t.rect(i,o,a,s),n&&(t.cache.fillStyle=e.selectionRegionOverlayColor,t.fill()),r&&(t.cache.lineWidth=1,t.cache.strokeStyle=e.selectionRegionOutlineColor,t.stroke()),t.closePath()}}});t.exports=r},function(t,e,n){"use strict";var r=n(4).extend("SparkLine",{paint:function(t,e){var n=e.bounds.x,r=e.bounds.y,i=e.bounds.width,o=e.bounds.height;t.beginPath();var a=e.value;if(a&&a.length){var s,A=a.length,l=i/A,c=e.isSelected?e.foregroundSelectionColor:e.color;(e.backgroundColor||e.isSelected)&&(t.cache.fillStyle=e.isSelected?e.backgroundSelectionColor:e.backgroundColor,t.fillRect(n,r,i,o)),t.cache.strokeStyle=c,t.cache.fillStyle=c,t.beginPath();for(var u=0;u<a.length;u++){var g=a[u]/110*o;s||(s=g),t.lineTo(n+5,r+o-g),t.arc(n+5,r+o-g,1,0,2*Math.PI,!1),n+=l}e.minWidth=10*A,t.stroke(),t.closePath()}}});t.exports=r},function(t,e,n){"use strict";var r=n(4).extend("ErrorCell",{paint:function(t,e,n){var r=e.bounds.x,i=e.bounds.y,o=e.bounds.height;t.cache.fillStyle="#FFD500",t.fill(),t.cache.fillStyle="#A00",t.cache.textAlign="start",t.cache.textBaseline="middle",t.cache.font='bold 6pt "arial narrow", verdana, geneva',t.fillText(n,r+4,i+o/2+.5)}});t.exports=r},function(t,e,n){"use strict";var r=n(4).extend("Tag",{paint:function(t,e){if(e.tagbands){var n=e.tagbands.find(function(t){return e.value>=t.floor}),r=n&&n.fillStyle;if(r){var i=e.bounds,o=i.x+i.width-1,a=i.y;t.beginPath(),t.moveTo(o,a),t.lineTo(o,a+8),t.lineTo(o-8,a),t.closePath(),t.cache.fillStyle=r,t.fill()}}}});t.exports=r},function(t,e,n){"use strict";var r=n(4).extend("TreeCell",{paint:function(t,e){var n=e.bounds.x,r=e.bounds.y,i=e.value.data,o=e.value.indent,a=e.value.icon;if((e.backgroundColor||e.isSelected)&&(t.cache.fillStyle=(e.isSelected,e.backgroundColor),t.fillRect(n,r,e.bounds.width,e.bounds.height)),i&&i.length){t.cache.fillStyle=(e.isSelected,e.backgroundColor);var s=Math.ceil(e.bounds.height/2);t.fillText(a+i,n+o,r+s),e.minWidth=n+o+t.getTextWidth(a+i)+10}}});t.exports=r},function(t,e,n){"use strict";var r,i=n(30),o=i.extend("CellEditors",{BaseClass:n(12),initialize:function(){this.add(n(174)),this.add(n(175)),this.add(n(176)),this.add(n(177)),this.add(n(178)),this.add(n(77))},get:function(t){if(t&&"celleditor"===t.toLowerCase())return r||(console.warn('grid.cellEditors.get("'+t+'") method call has been deprecated as of v2.1.0 in favor of grid.cellEditors.BaseClass property. (Will be removed in a future release.)'),r=!0),this.BaseClass;try{var e=i.prototype.get.call(this,t)}catch(t){}return e}});t.exports=new o},function(t,e,n){"use strict";var r=n(12).extend("Color",{template:'<input type="color" lang="{{locale}}" style="{{style}}">'});t.exports=r},function(t,e,n){"use strict";var r=n(12),i=window.chrome,o=window.navigator,a=o.vendor,s=o.userAgent.indexOf("OPR")>-1,A=o.userAgent.indexOf("Edge")>-1,l=!o.userAgent.match("CriOS")&&null!=i&&"Google Inc."===a&&0==s&&0==A,c=r.extend("Date",{initialize:function(t){var e;l?(e="chromeDate",this.template='<input type="date">'):(e="date",this.template='<input type="text" lang="{{locale}}">',this.selectAll=function(){var t=this.getEditorValue().length;this.input.setSelectionRange(0,t)}),this.localizer=t.localization.get(e)}});t.exports=c},function(t,e,n){"use strict";var r=n(77).extend("Number",{initialize:function(t){this.localizer=t.localization.get("number")}});t.exports=r},function(t,e,n){"use strict";var r=n(12).extend("Slider",{template:'<input type="range" lang="{{locale}}" style="{{style}}">'});t.exports=r},function(t,e,n){"use strict";var r=n(12).extend("Spinner",{template:'<input type="number" lang="{{locale}}" style="{{style}}">'});t.exports=r},function(t,e,n){"use strict";function r(t,e,n){if("string"==typeof n){if(!(n=document.querySelector(n)))throw"Cannot find reference element for CSS injection."}else if(n&&!(n instanceof Element))throw"Given value not a reference element.";var i=n&&n.parentNode||document.head||document.getElementsByTagName("head")[0];if(!e||(e=r.idPrefix+e,!i.querySelector("#"+e))){var o=document.createElement("style");return o.type="text/css",e&&(o.id=e),t instanceof Array&&(t=t.join("\n")),t="\n"+t+"\n",o.styleSheet?o.styleSheet.cssText=t:o.appendChild(document.createTextNode(t)),void 0===n&&(n=i.firstChild),i.insertBefore(o,n),o}}r.idPrefix="injected-stylesheet-",t.exports=r},function(t,e,n){var r,i,o,a;a=function(t){var e=Object.prototype.toString,n=Array.isArray||function(t){return"[object Array]"===e.call(t)};function r(t){return"function"==typeof t}function i(t){return t.replace(/[\-\[\]{}()*+?.,\\\^$|#\s]/g,"\\$&")}function o(t,e){return null!=t&&"object"==typeof t&&e in t}var a=RegExp.prototype.test,s=/\S/;function A(t){return!function(t,e){return a.call(t,e)}(s,t)}var l={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;","/":"&#x2F;","`":"&#x60;","=":"&#x3D;"},c=/\s*/,u=/\s+/,g=/\s*=/,h=/\s*\}/,d=/#|\^|\/|>|\{|&|=|!/;function f(t){this.string=t,this.tail=t,this.pos=0}function p(t,e){this.view=t,this.cache={".":this.view},this.parent=e}function I(){this.cache={}}f.prototype.eos=function(){return""===this.tail},f.prototype.scan=function(t){var e=this.tail.match(t);if(!e||0!==e.index)return"";var n=e[0];return this.tail=this.tail.substring(n.length),this.pos+=n.length,n},f.prototype.scanUntil=function(t){var e,n=this.tail.search(t);switch(n){case-1:e=this.tail,this.tail="";break;case 0:e="";break;default:e=this.tail.substring(0,n),this.tail=this.tail.substring(n)}return this.pos+=e.length,e},p.prototype.push=function(t){return new p(t,this)},p.prototype.lookup=function(t){var e,n=this.cache;if(n.hasOwnProperty(t))e=n[t];else{for(var i,a,s=this,A=!1;s;){if(t.indexOf(".")>0)for(e=s.view,i=t.split("."),a=0;null!=e&&a<i.length;)a===i.length-1&&(A=o(e,i[a])),e=e[i[a++]];else e=s.view[t],A=o(s.view,t);if(A)break;s=s.parent}n[t]=e}return r(e)&&(e=e.call(this.view)),e},I.prototype.clearCache=function(){this.cache={}},I.prototype.parse=function(e,r){var o=this.cache,a=o[e];return null==a&&(a=o[e]=function(e,r){if(!e)return[];var o,a,s,l=[],p=[],I=[],C=!1,m=!1;function y(){if(C&&!m)for(;I.length;)delete p[I.pop()];else I=[];C=!1,m=!1}function E(t){if("string"==typeof t&&(t=t.split(u,2)),!n(t)||2!==t.length)throw new Error("Invalid tags: "+t);o=new RegExp(i(t[0])+"\\s*"),a=new RegExp("\\s*"+i(t[1])),s=new RegExp("\\s*"+i("}"+t[1]))}E(r||t.tags);for(var v,B,b,Q,w,x,_=new f(e);!_.eos();){if(v=_.pos,b=_.scanUntil(o))for(var D=0,M=b.length;D<M;++D)A(Q=b.charAt(D))?I.push(p.length):m=!0,p.push(["text",Q,v,v+1]),v+=1,"\n"===Q&&y();if(!_.scan(o))break;if(C=!0,B=_.scan(d)||"name",_.scan(c),"="===B?(b=_.scanUntil(g),_.scan(g),_.scanUntil(a)):"{"===B?(b=_.scanUntil(s),_.scan(h),_.scanUntil(a),B="&"):b=_.scanUntil(a),!_.scan(a))throw new Error("Unclosed tag at "+_.pos);if(w=[B,b,v,_.pos],p.push(w),"#"===B||"^"===B)l.push(w);else if("/"===B){if(!(x=l.pop()))throw new Error('Unopened section "'+b+'" at '+v);if(x[1]!==b)throw new Error('Unclosed section "'+x[1]+'" at '+v)}else"name"===B||"{"===B||"&"===B?m=!0:"="===B&&E(b)}if(x=l.pop())throw new Error('Unclosed section "'+x[1]+'" at '+_.pos);return function(t){for(var e,n=[],r=n,i=[],o=0,a=t.length;o<a;++o)switch((e=t[o])[0]){case"#":case"^":r.push(e),i.push(e),r=e[4]=[];break;case"/":i.pop()[5]=e[2],r=i.length>0?i[i.length-1][4]:n;break;default:r.push(e)}return n}(function(t){for(var e,n,r=[],i=0,o=t.length;i<o;++i)(e=t[i])&&("text"===e[0]&&n&&"text"===n[0]?(n[1]+=e[1],n[3]=e[3]):(r.push(e),n=e));return r}(p))}(e,r)),a},I.prototype.render=function(t,e,n){var r=this.parse(t),i=e instanceof p?e:new p(e);return this.renderTokens(r,i,n,t)},I.prototype.renderTokens=function(t,e,n,r){for(var i,o,a,s="",A=0,l=t.length;A<l;++A)a=void 0,"#"===(o=(i=t[A])[0])?a=this.renderSection(i,e,n,r):"^"===o?a=this.renderInverted(i,e,n,r):">"===o?a=this.renderPartial(i,e,n,r):"&"===o?a=this.unescapedValue(i,e):"name"===o?a=this.escapedValue(i,e):"text"===o&&(a=this.rawValue(i)),void 0!==a&&(s+=a);return s},I.prototype.renderSection=function(t,e,i,o){var a=this,s="",A=e.lookup(t[1]);if(A){if(n(A))for(var l=0,c=A.length;l<c;++l)s+=this.renderTokens(t[4],e.push(A[l]),i,o);else if("object"==typeof A||"string"==typeof A||"number"==typeof A)s+=this.renderTokens(t[4],e.push(A),i,o);else if(r(A)){if("string"!=typeof o)throw new Error("Cannot use higher-order sections without the original template");null!=(A=A.call(e.view,o.slice(t[3],t[5]),function(t){return a.render(t,e,i)}))&&(s+=A)}else s+=this.renderTokens(t[4],e,i,o);return s}},I.prototype.renderInverted=function(t,e,r,i){var o=e.lookup(t[1]);if(!o||n(o)&&0===o.length)return this.renderTokens(t[4],e,r,i)},I.prototype.renderPartial=function(t,e,n){if(n){var i=r(n)?n(t[1]):n[t[1]];return null!=i?this.renderTokens(this.parse(i),e,n,i):void 0}},I.prototype.unescapedValue=function(t,e){var n=e.lookup(t[1]);if(null!=n)return n},I.prototype.escapedValue=function(e,n){var r=n.lookup(e[1]);if(null!=r)return t.escape(r)},I.prototype.rawValue=function(t){return t[1]},t.name="mustache.js",t.version="2.3.2",t.tags=["{{","}}"];var C=new I;return t.clearCache=function(){return C.clearCache()},t.parse=function(t,e){return C.parse(t,e)},t.render=function(t,e,r){if("string"!=typeof t)throw new TypeError('Invalid template! Template should be a "string" but "'+(n(i=t)?"array":typeof i)+'" was given as the first argument for mustache#render(template, view, partials)');var i;return C.render(t,e,r)},t.to_html=function(e,n,i,o){var a=t.render(e,n,i);if(!r(o))return a;o(a)},t.escape=function(t){return String(t).replace(/[&<>"'`=\/]/g,function(t){return l[t]})},t.Scanner=f,t.Context=p,t.Writer=I,t},e&&"string"!=typeof e.nodeName?a(e):(i=[e],void 0===(o="function"==typeof(r=a)?r.apply(e,i):r)||(t.exports=o))},function(t,e,n){"use strict";var r=n(49),i=n(76);e.mixin={addEventListener:function(t,e,n){var r=this,i=this.listeners[t]=this.listeners[t]||[];if(!i.find(function(t){return t.listener===e})){var o={internal:n,listener:e,decorator:function(t){r.allowEventHandlers&&e(t)}};i.push(o),this.canvas.addEventListener(t,o.decorator)}},addInternalEventListener:function(t,e){this.addEventListener(t,e,!0)},removeEventListener:function(t,e){var n=this.listeners[t];n&&n.find(function(r,i){if(r.listener===e)return 1===n.length?delete this.listeners[t]:n.splice(i,1),this.canvas.removeEventListener(t,r.decorator),!0},this)},removeAllEventListeners:function(t){Object.keys(this.listeners).forEach(function(e){this.listeners[e].slice().forEach(function(n){!t&&n.internal||this.removeEventListener(e,n.listener)},this)},this)},allowEvents:function(t){this.allowEventHandlers=!!t,this.behavior.featureChain&&(t?this.behavior.featureChain.attachChain():this.behavior.featureChain.detachChain()),this.behavior.changed()},fireSyntheticColumnSortEvent:function(t,e){return r.call(this,"fin-column-sort",{column:t,keys:e})},fireSyntheticEditorKeyUpEvent:function(t,e){return r.call(this,"fin-editor-keyup",{input:t,keyEvent:e,char:this.canvas.getKeyChar(e),legacyChar:e.legacyKey})},fireSyntheticEditorKeyDownEvent:function(t,e){return r.call(this,"fin-editor-keydown",{input:t,keyEvent:e,char:this.canvas.getKeyChar(e),legacyChar:e.legacyKey})},fireSyntheticEditorKeyPressEvent:function(t,e){return r.call(this,"fin-editor-keypress",{input:t,keyEvent:e,char:this.canvas.getKeyChar(e),legacyChar:e.legacyKey})},fireSyntheticEditorDataChangeEvent:function(t,e,n){return r.call(this,"fin-editor-data-change",!0,{input:t,oldValue:e,newValue:n})},fireSyntheticRowHeaderClickedEvent:function(t){return r.call(this,"fin-row-header-clicked",!0,{},t)},fireSyntheticRowSelectionChangedEvent:function(){return r.call(this,"fin-row-selection-changed",this.selectionDetailGetters)},fireSyntheticColumnSelectionChangedEvent:function(){return r.call(this,"fin-column-selection-changed",this.selectionDetailGetters)},fireSyntheticContextMenuEvent:function(t){return Object.defineProperties(t,this.selectionDetailGetterDescriptors),r.call(this,"fin-context-menu",{},t)},fireSyntheticMouseUpEvent:function(t){return Object.defineProperties(t,this.selectionDetailGetterDescriptors),r.call(this,"fin-mouseup",{},t)},fireSyntheticMouseDownEvent:function(t){return Object.defineProperties(t,this.selectionDetailGetterDescriptors),r.call(this,"fin-mousedown",{},t)},fireSyntheticMouseMoveEvent:function(t){return r.call(this,"fin-mousemove",{},t)},fireSyntheticButtonPressedEvent:function(t){if(t.cellRenderer instanceof i){if(t.value&&t.value.subrows){var e=t.primitiveEvent.detail.mouse.y-t.bounds.y,n=t.bounds.height/t.value.subrows;t.subrow=Math.floor(e/n)}return r.call(this,"fin-button-pressed",{},t)}},fireSyntheticOnColumnsChangedEvent:function(){return r.call(this,"fin-column-changed-event",{})},fireSyntheticKeydownEvent:function(t){return r.call(this,"fin-keydown",t.detail)},fireSyntheticKeyupEvent:function(t){return r.call(this,"fin-keyup",t.detail)},fireSyntheticFilterAppliedEvent:function(){return r.call(this,"fin-filter-applied",{})},fireSyntheticOnCellEnterEvent:function(t){return r.call(this,"fin-cell-enter",t)},fireSyntheticOnCellExitEvent:function(t){return r.call(this,"fin-cell-exit",t)},fireSyntheticClickEvent:function(t){return r.call(this,"fin-click",{},t)},fireSyntheticDoubleClickEvent:function(t){if(this.abortEditing())return r.call(this,"fin-double-click",{},t)},fireSyntheticGridRenderedEvent:function(){return r.call(this,"fin-grid-rendered",{source:this})},fireSyntheticTickEvent:function(){return r.call(this,"fin-tick",{source:this})},fireSyntheticGridResizedEvent:function(t){return r.call(this,"fin-grid-resized",t)},fireScrollEvent:function(t,e,n){return r.call(this,t,{oldValue:e,value:n})},fireRequestCellEdit:function(t,e){return r.call(this,"fin-request-cell-edit",!0,{value:e},t)},fireBeforeCellEdit:function(t,e,n,i){return r.call(this,"fin-before-cell-edit",!0,{oldValue:e,newValue:n,input:i},t)},fireAfterCellEdit:function(t,e,n,i){return r.call(this,"fin-after-cell-edit",{newValue:n,oldValue:e,input:i},t)},delegateCanvasEvents:function(){var t=this;function e(e,n){if(0!==t.getLogicalRowCount()){var r,i,o,a=t.getGridCellFromMousePoint(e.detail.mouse);a.fake&&!e.detail.dragstart||(r=a.cellEvent),r&&(i=Object.defineProperty(r,"primitiveEvent",{value:e,enumerable:!1,configurable:!0,writable:!0}),(o=r.primitiveEvent&&r.primitiveEvent.detail)&&(i.gridPoint=o.mouse,(o=o.primitiveEvent)&&(i.clientPoint={x:o.clientX,y:o.clientY},i.pagePoint={x:o.clientX+window.scrollX,y:o.clientY+window.scrollY})),n.call(t,i))}}this.addInternalEventListener("fin-canvas-resized",function(e){t.resized(),t.fireSyntheticGridResizedEvent(e)}),this.addInternalEventListener("fin-canvas-mousemove",function(n){t.properties.readOnly||e(n,function(t){this.delegateMouseMove(t),this.fireSyntheticMouseMoveEvent(t)})}),this.addInternalEventListener("fin-canvas-mousedown",function(n){t.properties.readOnly||(t.abortEditing()?e(n,function(t){t.keys=n.detail.keys,this.mouseDownState=t,this.delegateMouseDown(t),this.fireSyntheticMouseDownEvent(t),this.repaint()}):n.stopPropagation())}),this.addInternalEventListener("fin-canvas-click",function(n){t.properties.readOnly||e(n,function(t){this.mouseDownState&&this.mouseDownState.gridCell.equals(t.gridCell)&&t.mousePointInClickRect&&(t.keys=n.detail.keys,this.mouseDownState&&this.fireSyntheticButtonPressedEvent(this.mouseDownState),this.fireSyntheticClickEvent(t),this.delegateClick(t)),this.mouseDownState=null})}),this.addInternalEventListener("fin-canvas-mouseup",function(n){t.properties.readOnly||(t.dragging=!1,t.isScrollingNow()&&t.setScrollingNow(!1),t.columnDragAutoScrolling&&(t.columnDragAutoScrolling=!1),e(n,function(t){this.delegateMouseUp(t),this.fireSyntheticMouseUpEvent(t)}))}),this.addInternalEventListener("fin-canvas-dblclick",function(n){t.properties.readOnly||e(n,function(t){this.fireSyntheticDoubleClickEvent(t,n),this.delegateDoubleClick(t)})}),this.addInternalEventListener("fin-canvas-drag",function(n){t.properties.readOnly||(t.dragging=!0,e(n,t.delegateMouseDrag))}),this.addInternalEventListener("fin-canvas-keydown",function(e){t.properties.readOnly||(t.fireSyntheticKeydownEvent(e),t.delegateKeyDown(e))}),this.addInternalEventListener("fin-canvas-keyup",function(e){t.properties.readOnly||(t.fireSyntheticKeyupEvent(e),t.delegateKeyUp(e))}),this.addInternalEventListener("fin-canvas-wheelmoved",function(n){e(n,t.delegateWheelMoved)}),this.addInternalEventListener("fin-canvas-mouseout",function(n){t.properties.readOnly||e(n,t.delegateMouseExit)}),this.addInternalEventListener("fin-canvas-context-menu",function(n){e(n,function(e){t.delegateContextMenu(e),t.fireSyntheticContextMenuEvent(e)})}),document.body.addEventListener("copy",function(e){t.checkClipboardCopy(e)})},delegateWheelMoved:function(t){this.behavior.onWheelMoved(this,t)},delegateMouseExit:function(t){this.behavior.handleMouseExit(this,t)},delegateContextMenu:function(t){this.behavior.onContextMenu(this,t)},delegateMouseMove:function(t){this.behavior.onMouseMove(this,t)},delegateMouseDown:function(t){this.behavior.handleMouseDown(this,t)},delegateMouseUp:function(t){this.behavior.onMouseUp(this,t)},delegateClick:function(t){this.behavior.onClick(this,t)},delegateMouseDrag:function(t){this.behavior.onMouseDrag(this,t)},delegateDoubleClick:function(t){this.behavior.onDoubleClick(this,t)},delegateKeyDown:function(t){this.behavior.onKeyDown(this,t)},delegateKeyUp:function(t){this.behavior.onKeyUp(this,t)}}},function(t,e,n){"use strict";var r=n(183);function i(t){var e,n=this.behavior.getColumns(),r=this.behavior.getActiveColumns();return Array.isArray(t)?(e=[],t.forEach(function(t){var i="number"==typeof t?"index":"name",o=n.find(function(e){return e[i]===t});r.indexOf(o)<0&&e.push(o)}),e=e.concat(r)):e=t?n:r,e}function o(t){var e=t.origin,n=t.corner,i=Math.min(e.x,n.x),o=Math.min(e.y,n.y),a=Math.max(e.x,n.x),s=Math.max(e.y,n.y);return new r(i,o,a-i,s-o)}function a(t,e){var n,r;return t&&(r="f"===(typeof(n=t[e.name]))[0]&&n||e.calculator)&&(n=r(t,e.name)),n||0===n||!1===n?n:""}e.mixin={selectionInitialize:function(){var t=this;this.selectionDetailGetters={get rows(){return t.getSelectedRows()},get columns(){return t.getSelectedColumns()},get selections(){return t.selectionModel.getSelections()}},this.selectionDetailGetterDescriptors=Object.getOwnPropertyDescriptors(this.selectionDetailGetters)},hasSelections:function(){if(this.getSelectionModel)return this.selectionModel.hasSelections()},getSelectionAsTSV:function(){switch(this.selectionModel.getLastSelectionType()){case"cell":var t=this.getSelectionMatrix();return t=t[t.length-1],this.getMatrixSelectionAsTSV(t);case"row":return this.getMatrixSelectionAsTSV(this.getRowSelectionMatrix());case"column":return this.getMatrixSelectionAsTSV(this.getColumnSelectionMatrix())}},getMatrixSelectionAsTSV:function(t){var e="";if(t.length){var n=t.length,r=t[0].length,i=n-1,o=r>1?"\n":"";if(n*r>2e4)return alert("selection size is too big to copy to the paste buffer"),"";for(var a=0;a<r;a++)for(var s=0;s<n;s++)e+=t[s][a]+(s<i?"\t":o)}return e},clearSelections:function(){var t=this.properties.checkboxOnlyRowSelections;this.selectionModel.clear(t),this.clearMouseDown()},clearMostRecentSelection:function(){var t=this.properties.checkboxOnlyRowSelections;this.selectionModel.clearMostRecentSelection(t)},clearMostRecentColumnSelection:function(){this.selectionModel.clearMostRecentColumnSelection()},clearMostRecentRowSelection:function(){},clearRowSelection:function(){this.selectionModel.clearRowSelection()},select:function(t,e,n,r){t<0||e<0||this.selectionModel.select(t,e,n,r)},isSelected:function(t,e){return this.selectionModel.isSelected(t,e)},isCellSelectedInRow:function(t){return this.selectionModel.isCellSelectedInRow(t)},isCellSelectedInColumn:function(t){return this.selectionModel.isCellSelectedInColumn(t)},getRowSelection:function(t){for(var e=this.behavior.dataModel,n=this.selectionModel.getSelectedRows(),r=i.call(this,t),o={},s=0,A=r.length;s<A;s++){var l=r[s],c=o[l.name]=new Array(n.length);n.forEach(u)}function u(t,n){var r=e.getRow(t);c[n]=a(r,l)}return o},getRowSelectionMatrix:function(t){for(var e=this.behavior.dataModel,n=this.selectionModel.getSelectedRows(),r=i.call(this,t),o=new Array(r.length),s=0,A=r.length;s<A;s++){var l=r[s];o[s]=new Array(n.length),n.forEach(c)}function c(t,n){var r=e.getRow(t);o[s][n]=a(r,l)}return o},getColumnSelectionMatrix:function(){var t=this.behavior,e=t.dataModel,n=this.getHeaderRowCount(),r=this.getSelectedColumns(),i=this.getRowCount(),o=new Array(r.length);return r.forEach(function(r,s){for(var A=t.getActiveColumn(r),l=o[s]=new Array(i),c=n;c<i;c++){var u=e.getRow(c);l[c]=a(u,A)}}),o},getColumnSelection:function(){var t=this.behavior,e=t.dataModel,n=this.getHeaderRowCount(),r=this.getSelectedColumns(),i={},o=this.getRowCount();return r.forEach(function(r){for(var s=t.getActiveColumn(r),A=i[s.name]=new Array(o),l=n;l<o;l++){var c=e.getRow(l);A[l]=a(c,s)}}),i},getSelection:function(){var t=this.behavior,e=t.dataModel,n=this.getSelections(),r=new Array(n.length);return n.forEach(function(n,i){for(var s=o(n),A=s.width,l=s.height,c={},u=0,g=s.origin.x;u<A;u++,g++)for(var h=t.getActiveColumn(g),d=c[h.name]=new Array(l),f=0,p=s.origin.y;f<l;f++,p++){var I=e.getRow(p);d[f]=a(I,h)}r[i]=c}),r},getSelectionMatrix:function(){var t=this.behavior,e=t.dataModel,n=this.getSelections(),r=new Array(n.length);return n.forEach(function(n,i){for(var s=o(n),A=s.width,l=s.height,c=[],u=0,g=s.origin.x;u<A;u++,g++)for(var h=c[u]=new Array(l),d=t.getActiveColumn(g),f=0,p=s.origin.y;f<l;f++,p++){var I=e.getRow(p);h[f]=a(I,d)}r[i]=c}),r},selectCell:function(t,e,n){var r=this.properties.checkboxOnlyRowSelections;this.selectionModel.clear(r),this.selectionModel.select(t,e,0,0,n)},toggleSelectColumn:function(t,e){e=e||[];var n=this.selectionModel,r=n.isColumnSelected(t),i=e.indexOf("CTRL")>-1,o=e.indexOf("SHIFT")>-1;i||o?(i&&(r?n.deselectColumn(t):n.selectColumn(t)),o&&(n.clear(),n.selectColumn(this.lastEdgeSelection[0],t))):(n.clear(),r||n.selectColumn(t)),r||o||(this.lastEdgeSelection[0]=t),this.repaint(),this.fireSyntheticColumnSelectionChangedEvent()},toggleSelectRow:function(t,e){e=e||[];var n=this.selectionModel,r=n.isRowSelected(t),i=e.indexOf("SHIFT")>=0;r?n.deselectRow(t):(this.singleSelect(),n.selectRow(t)),i&&(n.clear(),n.selectRow(this.lastEdgeSelection[1],t)),r||i||(this.lastEdgeSelection[1]=t),this.repaint()},singleSelect:function(){var t=this.properties.singleRowSelectionMode;return t&&this.selectionModel.clearRowSelection(),t},selectViewportCell:function(t,e){var n,r;this.getRowCount()&&(n=this.renderer.visibleColumns[t])&&(r=this.renderer.visibleRows[e+this.getHeaderRowCount()])&&(t=n.columnIndex,e=r.rowIndex,this.clearSelections(),this.select(t,e,0,0),this.setMouseDown(this.newPoint(t,e)),this.setDragExtent(this.newPoint(0,0)),this.repaint())},selectToViewportCell:function(t,e){var n,r,i;if((n=this.getSelections())&&n.length&&(r=this.renderer.visibleColumns[t])&&(i=this.renderer.visibleRows[e+this.getHeaderRowCount()])){var o=n[0].origin;t=r.columnIndex,e=i.rowIndex,this.setDragExtent(this.newPoint(t-o.x,e-o.y)),this.select(o.x,o.y,t-o.x,e-o.y),this.repaint()}},selectToFinalCellOfCurrentRow:function(){this.selectFinalCellOfCurrentRow(!0)},selectFinalCellOfCurrentRow:function(t){if(this.getRowCount()){var e=this.getSelections();if(e&&e.length){var n=e[0],r=n.origin,i=n.extent,o=this.getColumnCount();this.scrollBy(o,0),this.clearSelections(),t?this.select(r.x,r.y,o-r.x-1,i.y):this.select(o-1,r.y,0,0),this.repaint()}}},selectToFirstCellOfCurrentRow:function(){this.selectFirstCellOfCurrentRow(!0)},selectFirstCellOfCurrentRow:function(t){if(this.getRowCount()){var e=this.getSelections();if(e&&e.length){var n=e[0],r=n.origin,i=n.extent;this.clearSelections(),t?this.select(r.x,r.y,-r.x,i.y):this.select(0,r.y,0,0),this.setHScrollValue(0),this.repaint()}}},selectFinalCell:function(){this.getRowCount()&&(this.selectCellAndScrollToMakeVisible(this.getColumnCount()-1,this.getRowCount()-1),this.repaint())},selectToFinalCell:function(){if(this.getRowCount()){var t=this.getSelections();if(t&&t.length){var e=t[0].origin,n=this.getColumnCount(),r=this.getRowCount();this.clearSelections(),this.select(e.x,e.y,n-e.x-1,r-e.y-1),this.repaint()}}},getSelectedRow:function(){var t=this.selectionModel.getSelections();if(t.length){for(var e=this.behavior,n=this.getColumnCount(),r=t[0].origin.y,i={},o=0;o<n;o++)i[e.getActiveColumn(o).header]=e.getValue(o,r);return i}},selectionChanged:function(t){this.selectRowsFromCells(),this.selectColumnsFromCells(),t||this.canvas.dispatchEvent(new CustomEvent("fin-selection-changed",{detail:this.selectionDetailGetters}))},isColumnOrRowSelected:function(){return this.selectionModel.isColumnOrRowSelected()},selectColumn:function(t,e){this.selectionModel.selectColumn(t,e)},selectRow:function(t,e){var n=this.selectionModel;this.singleSelect()?t=e:void 0===e&&(e=t),n.selectRow(Math.min(t,e),Math.max(t,e))},selectRowsFromCells:function(){var t;!this.properties.checkboxOnlyRowSelections&&this.properties.autoSelectRows&&(this.properties.singleRowSelectionMode?(t=this.selectionModel.getLastSelection())?this.selectRow(null,t.corner.y):this.clearRowSelection():this.selectionModel.selectRowsFromCells(0,!0),this.fireSyntheticRowSelectionChangedEvent())},selectColumnsFromCells:function(){this.properties.autoSelectColumns&&this.selectionModel.selectColumnsFromCells()},getSelectedRows:function(){return this.behavior.getSelectedRows()},getSelectedColumns:function(){return this.behavior.getSelectedColumns()},getSelections:function(){return this.behavior.getSelections()},getLastSelectionType:function(t){return this.selectionModel.getLastSelectionType(t)},isInCurrentSelectionRectangle:function(t,e){return this.selectionModel.isInCurrentSelectionRectangle(t,e)},selectAllRows:function(){this.selectionModel.selectAllRows()},areAllRowsSelected:function(){return this.selectionModel.areAllRowsSelected()},toggleSelectAllRows:function(){this.areAllRowsSelected()?this.selectionModel.clear():this.selectAllRows(),this.repaint()},moveSingleSelect:function(t,e){var n=this.getMouseDown().plus(this.getDragExtent());this.moveToSingleSelect(n.x+t,n.y+e)},moveToSingleSelect:function(t,e){var n=this.getColumnCount()-1,r=this.getRowCount()-1,i=this.getVisibleColumnsCount()-1,o=this.getVisibleRowsCount()-1;this.properties.scrollingEnabled||(n=Math.min(n,i),r=Math.min(r,o)),t=Math.min(n,Math.max(0,t)),e=Math.min(r,Math.max(0,e)),this.clearSelections(),this.select(t,e,0,0),this.setMouseDown(this.newPoint(t,e)),this.setDragExtent(this.newPoint(0,0)),this.selectCellAndScrollToMakeVisible(t,e),this.repaint()},extendSelect:function(t,e){var n=this.getColumnCount()-1,r=this.getRowCount()-1,i=this.renderer.visibleColumns.length-1,o=this.renderer.visibleRows.length-1,a=this.getMouseDown(),s=this.getDragExtent(),A=s.x+t,l=s.y+e;this.properties.scrollingEnabled||(n=Math.min(n,i),r=Math.min(r,o)),A=Math.min(n-a.x,Math.max(-a.x,A)),l=Math.min(r-a.y,Math.max(-a.y,l)),this.clearMostRecentSelection(),this.select(a.x,a.y,A,l),this.setDragExtent(this.newPoint(A,l));var c=this.insureModelColIsVisible(A+a.x,t),u=this.insureModelRowIsVisible(l+a.y,e);return this.repaint(),c||u},getGridCellFromLastSelection:function(t){var e=this.selectionModel.getLastSelection();return e&&(new this.behavior.CellEvent).resetGridXDataY(e.origin.x,e.origin.y,null,t)}}},function(t,e,n){"use strict";var r=n(2).Rectangle;function i(){r.apply(this,arguments)}i.prototype=Object.create(r.prototype,{width:{get:function(){return this.extent.x+1}},height:{get:function(){return this.extent.y+1}}}),t.exports=i},function(t,e,n){"use strict";e.mixin={stashRowSelections:function(){if(this.properties.restoreRowSelections){var t=this.behavior.dataModel;return this.selectedDataRowIndexes=this.getSelectedRows().map(function(e){return t.getRowIndex(e)}),this.selectedDataRowIndexes.length}},unstashRowSelections:function(){var t=this.selectedDataRowIndexes;if(t){delete this.selectedDataRowIndexes;var e,n,r=this.behavior.dataModel,i=this.getRowCount(),o=t.length,a=[],s=this.selectionModel;for(n=0;o&&n<i;++n)(e=t.indexOf(r.getRowIndex(n)))>=0&&(a.push(n),delete t[e],o--);return a.forEach(function(t){s.selectRow(t)}),a.length}},stashColumnSelections:function(){if(this.properties.restoreColumnSelections){var t=this.behavior;return this.selectedColumnNames=this.getSelectedColumns().map(function(e){return t.getActiveColumn(e).name}),this.selectedColumnNames.length}},unstashColumnSelections:function(t){var e=this.selectedColumnNames;if(e){delete this.selectedColumnNames;var n=this.behavior,r=this.selectionModel;return e.reduce(function(t,e){var i=n.getActiveColumnIndex(e);return i&&(r.selectColumn(i),t++),t},0)}}}},function(t,e,n){"use strict";var r=n(78).Scrollbar;e.mixin={vScrollValue:0,hScrollValue:0,sbVScroller:null,sbHScroller:null,sbPrevVScrollValue:null,sbPrevHScrollValue:null,scrollingNow:!1,setScrollingNow:function(t){this.scrollingNow=t},isScrollingNow:function(){return this.scrollingNow},scrollBy:function(t,e){this.scrollHBy(t),this.scrollVBy(e)},scrollVBy:function(t){var e=this.sbVScroller.range.max,n=this.getVScrollValue(),r=Math.min(e,Math.max(0,n+t));r!==n&&this.setVScrollValue(r)},scrollHBy:function(t){var e=this.sbHScroller.range.max,n=this.getHScrollValue(),r=Math.min(e,Math.max(0,n+t));r!==n&&this.setHScrollValue(r)},scrollToMakeVisible:function(t,e){var n,r=this.renderer.dataWindow,i=this.properties.fixedColumnCount,o=this.properties.fixedRowCount;t>=i&&((n=t-r.origin.x)<0?this.sbHScroller.index+=n:t-r.corner.x>0&&(this.sbHScroller.index=this.renderer.getMinimumLeftPositionToShowColumn(t))),e>=o&&((n=e-r.origin.y-1)<0||(n=e-r.corner.y)>0)&&(this.sbVScroller.index+=n)},selectCellAndScrollToMakeVisible:function(t,e){this.scrollToMakeVisible(t,e),this.selectCell(t,e,!0)},setVScrollValue:function(t){var e=this;if((t=Math.min(this.sbVScroller.range.max,Math.max(0,Math.round(t))))!==this.vScrollValue){this.behavior.setScrollPositionY(t),this.behavior.changed();var n=this.vScrollValue;this.vScrollValue=t,this.scrollValueChangedNotification(),setTimeout(function(){e.fireScrollEvent("fin-scroll-y",n,t)})}},getVScrollValue:function(){return this.vScrollValue},setHScrollValue:function(t){var e=this;if((t=Math.min(this.sbHScroller.range.max,Math.max(0,Math.round(t))))!==this.hScrollValue){this.behavior.setScrollPositionX(t),this.behavior.changed();var n=this.hScrollValue;this.hScrollValue=t,this.scrollValueChangedNotification(),setTimeout(function(){e.fireScrollEvent("fin-scroll-x",n,t)})}},getHScrollValue:function(){return this.hScrollValue},initScrollbars:function(){if(!this.sbHScroller||!this.sbVScroller){var t=new r({orientation:"horizontal",deltaXFactor:this.constructor.defaults.wheelHFactor,onchange:this.setHScrollValue.bind(this),cssStylesheetReferenceElement:this.div}),e=new r({orientation:"vertical",deltaYFactor:this.constructor.defaults.wheelVFactor,onchange:this.setVScrollValue.bind(this),cssStylesheetReferenceElement:this.div,paging:{up:this.pageUp.bind(this),down:this.pageDown.bind(this)}});this.sbHScroller=t,this.sbVScroller=e;var n=this.properties.hScrollbarClassPrefix,i=this.properties.vScrollbarClassPrefix;n&&""!==n&&(this.sbHScroller.classPrefix=n),i&&""!==i&&(this.sbVScroller.classPrefix=i),this.div.appendChild(t.bar),this.div.appendChild(e.bar),this.resizeScrollbars()}},resizeScrollbars:function(){this.sbHScroller.shortenBy(this.sbVScroller).resize(),this.sbVScroller.resize()},setVScrollbarValues:function(t){this.sbVScroller.range={min:0,max:t}},setHScrollbarValues:function(t){this.sbHScroller.range={min:0,max:t}},scrollValueChangedNotification:function(){this.hScrollValue===this.sbPrevHScrollValue&&this.vScrollValue===this.sbPrevVScrollValue||(this.sbPrevHScrollValue=this.hScrollValue,this.sbPrevVScrollValue=this.vScrollValue,this.cellEditor&&this.cellEditor.scrollValueChangedNotification(),this.computeCellsBounds())},synchronizeScrollingBoundaries:function(){var t=this.getBounds();if(t){for(var e=this.getFixedColumnCount(),n=this.getColumnCount(),r=this.getRowCount(),i=t.width-this.behavior.getFixedColumnsMaxWidth(),o=this.properties,a="border-box"===o.boxSizing,s=a?0:o.gridLinesVWidth,A=0,l=0;l<n&&A<i;l++)A+=this.getColumnWidth(n-l-1)+s;A>i&&l--;var c=this.renderer.getVisibleScrollHeight();s=a?0:o.gridLinesHWidth;for(var u=0,g=0;g<r&&u<c;g++)u+=this.getRowHeight(r-g-1)+s;if(u>c&&g--,this.sbHScroller){var h=Math.max(0,n-e-l);this.setHScrollbarValues(h),this.setHScrollValue(Math.min(this.getHScrollValue(),h))}if(this.sbVScroller){var d=Math.max(0,r-o.fixedRowCount-g);this.setVScrollbarValues(d),this.setVScrollValue(Math.min(this.getVScrollValue(),d))}this.computeCellsBounds(),setTimeout(this.resizeScrollbars.bind(this))}},pageUp:function(){var t=this.renderer.getPageUpRow();return this.setVScrollValue(t),t},pageDown:function(){var t=this.renderer.getPageDownRow();return this.setVScrollValue(t),t},pageLeft:function(){throw"page left not yet implemented"},pageRight:function(){throw"page right not yet implemented"}}},function(t,e,n){"use strict";var r,i="GroupedHeader",o={paint:function(t,e){var n=this.super.paint,i=e.gridCell.x,o=e.value.split(this.delimiter),a=o.length,s=e.bounds,A=s.y+s.height,l=Object.assign({},s),c=this.visibleColumns.find(function(t){return t.columnIndex===i-1});e.prefillColor=null,s.height/=a;for(var g,h=0,d=s.y,f=this.tree;h<a;h++,d+=s.height,g=f.children){var p=o[h],I=!1;0===h?0===i||1===a||this.groupCount!==a||this.tree.value!==p?(i>0&&this.tree&&this.autosizeGroups&&u(e.grid.behavior,this.tree),f=this.tree={value:p,children:[],left:l.x,width:l.width},c&&!this.columnHeaderLines&&1===a&&(c.top=A)):I=!0:h<a-1&&(f=g[g.length-1])&&f.value===p?I=!0:(f={value:p,children:[],left:l.x,width:l.width},g.push(f)),I&&(f.width+=e.gridLinesVWidth+l.width,c&&(c.top=this.columnHeaderLines||h<a-2?d+s.height:A)),h<a-1&&(e.prefillColor=null)}for(h=0,f=this.tree;h<a;h++,s.y+=s.height,f=f.children[f.children.length-1]){s.x=f.left,s.width=f.width;var C={isColumnHovered:e.isColumnHovered,isSelected:e.isSelected},m=this.groupConfig,y=m&&m.length,E=h<a-1;if(E){e.isColumnHovered=e.isSelected=!1,y&&(m=m[h%y],Object.keys(m).forEach(B,this));var v=e.paintBackground||this.paintBackground;if(v){if("function"!=typeof v&&(v=r[v]),"function"!=typeof v)throw"Expected background paint function or name of registered background paint function.";v.overlay||(e.prefillColor=e.backgroundColor,v.call(this,t,e))}}e.value=f.value,n.call(this,t,e),f.minWidth=e.minWidth,f.columnIndex=i,E&&(v&&v.overlay&&v.call(this,t,e),Object.assign(e,C))}function B(n){C[n]=e[n];var r=m[n];"paintBackground"!==n&&"function"==typeof r&&(r=r.call(this,t,e)),e[n]=r}this.groupCount=a,Object.assign(s,l)},delimiter:"|",paintBackground:h,flatHeight:"250%",nestedHeight:"160%",columnHeaderLines:!0,autosizeGroups:!0,groupConfig:[{halign:"center",thickness:1,gradientStops:function(t,e){return this.backgroundColor||e.backgroundColor,[[0,"white"],[1,this.backgroundColor||e.backgroundColor]]}}]};function a(t){var e=Object.getPrototypeOf(this).setHeaders,n=this.grid.cellRenderers.get(i),r=n.delimiter,o=this.columns.reduce(function(t,e){return Math.max(e.header.split(r).length,t)},0),a=function(t,e,n){var r=e.columnHeaderFont.match(s),i=r&&r[1]||.8*e.defaultRowHeight;return 1===n?l(t.flatHeight,i):l(t.nestedHeight,i)*n}(n,this.grid.properties,o),A=this.grid.behavior.subgrids.lookup.header;e.call(this,t),this.grid.setRowHeight(0,a,A)}var s=/\b(\d+(\.\d+)?)px\b/,A=/^(\d+(\.\d+)?)%$/;function l(t,e){var n=t.match(A);return n?n[1]/100*e:t}function c(){var t=Object.getPrototypeOf(this).isColumnReorderable.call(this),e=this.grid.cellRenderers.get(i).delimiter;return t&&!this.columns.find(function(t){return t.getCellProperty(0,"renderer")===i&&-1!==t.header.indexOf(e)})}function u(t,e){e.children.length&&(e.minWidth=Math.max(e.minWidth,function t(e,n){return n.children.length?n.children.reduce(function(n,r){return n+(r.minWidth=Math.max(r.minWidth,t(e,r)))},0):n.minWidth=Math.max(n.minWidth,e.getActiveColumn(n.columnIndex).properties.preferredWidth)}(t,e)),function t(e,n){var r=n.children.reduce(function(t,e){return t+e.minWidth},0),i=(n.minWidth-r)/n.children.length;n.children.forEach(function(n){(n.minWidth+=i,n.children.length)?t(e,n):e.getActiveColumn(n.columnIndex).properties.preferredWidth=n.minWidth})}(t,e))}var g=/^rgba|^transparent$/;function h(t,e){var n=e.bounds,r=e.thickness||this.groupCount-this.groupIndex;t.cache.fillStyle=e.gridLinesVColor,t.fillRect(n.x,n.y+n.height-r,n.width,r)}h.overlay=!0,t.exports=r={name:"groupedHeader",install:function(t,e){var n=(e=e||{}).CellRenderer||"SimpleCell";if("string"==typeof n&&(n=t.cellRenderers.get(n).constructor),"function"!=typeof n)throw new t.HypergridError("Expected `options.CellRenderer` to be a constructor or a registered cell redenderer.");var r=n.extend(i,o),s=t.cellRenderers.add(r);t.addEventListener("fin-grid-rendered",function(){s.tree&&(u(t.behavior,s.tree),t.checkColumnAutosizing())}),Object.assign(s,e),delete s.CellRenderer,s.visibleColumns=t.renderer.visibleColumns,t.properties.columnHeaderRenderer=i,t.behavior.setHeaders=a,t.behavior.isColumnReorderable=c,t.behavior.dataModel.groupHeaderDelimiter=s.delimiter},decorateBackgroundWithBottomBorder:h,decorateBackgroundWithLinearGradient:function(t,e){var n,r=e.bounds,i=t.createLinearGradient(r.x,r.y,r.x,r.y+r.height);(e.gradientStops||this.gradientStops).forEach(function(t){n=n||g.test(t),i.addColorStop.apply(i,t)}),n&&t.clearRect(r.x,r.y,r.width,r.height),t.cache.fillStyle=i,t.fillRect(r.x,r.y,r.width,r.height)}}},function(t,e,n){"use strict";var r=n(24),i=n(33),o=n(42),a=n(6),s=[].sort,A=[1,2,3];r(r.P+r.F*(a(function(){A.sort(void 0)})||!a(function(){A.sort(null)})||!n(188)(s)),"Array",{sort:function(t){return void 0===t?s.call(o(this)):s.call(o(this),i(t))}})},function(t,e,n){"use strict";var r=n(6);t.exports=function(t,e){return!!t&&r(function(){e?t.call(null,function(){},1):t.call(null)})}},function(t,e){t.exports=function(t,e){var r=t.toString(e);return"NaN"===r?"ᴺᵃᴺ":"Infinity"===r?"⁺ᴵⁿᶠ":"-Infinity"===r?"⁻ᴵⁿᶠ":r.split("").map(function(t){return n[t]||""}).join("")};var n={0:"⁰",1:"¹",2:"²",3:"³",4:"⁴",5:"⁵",6:"⁶",7:"⁷",8:"⁸",9:"⁹","+":"⁺","-":"⁻",a:"ᵃ",b:"ᵇ",c:"ᶜ",d:"ᵈ",e:"ᵉ",f:"ᶠ",g:"ᵍ",h:"ʰ",i:"ⁱ",j:"ʲ",k:"ᵏ",l:"ˡ",m:"ᵐ",n:"ⁿ",o:"ᵒ",p:"ᵖ",r:"ʳ",s:"ˢ",t:"ᵗ",u:"ᵘ",v:"ᵛ",w:"ʷ",x:"ˣ",y:"ʸ",z:"ᶻ"}}])},function(t,e){function n(t){var e=new Error("Cannot find module '"+t+"'");throw e.code="MODULE_NOT_FOUND",e}n.keys=function(){return[]},n.resolve=n,t.exports=n,n.id=278},function(t,e,n){var r,i,o,a,s,A,l,c,u,g,h,d,f,p,I;window,t.exports=(r=n(286),i=n(280),o=n(29),a=n(75),s=n(78),A=n(77),l=n(2),c=n(119),u=n(287),g=n(120),h=n(60),d=n(109),f=n(281),p=n(282),I=n(44),function(t){var e={};function n(r){if(e[r])return e[r].exports;var i=e[r]={i:r,l:!1,exports:{}};return t[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=t,n.c=e,n.d=function(t,e,r){n.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:r})},n.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},n.t=function(t,e){if(1&e&&(t=n(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var i in t)n.d(r,i,function(e){return t[e]}.bind(null,i));return r},n.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(e,"a",e),e},n.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},n.p="",n(n.s=23)}([function(t,e){t.exports=r},function(t,e){t.exports=i},function(t,e,n){"use strict";function r(t,e){return JSON.stringify(t)===JSON.stringify(e)}function i(t,e,n){let r=t.select(e);return r.size()>0?r:n()}function o(t,e){let n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"right";if("right"===n||"bottom"===n)return t[n]<e[n];if("left"===n||"top"===n)return t[n]>e[n];throw"Direction being checked for overflow is invalid: ".concat(n)}function a(t,e,n){let r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:0;const i="x"===t?"width":"height",o=e[t],a=e[t]+e[i],s=n[t],A=n[t]+n[i];return s+r>o&&s-r<a||A+r>o&&A-r<a||s+r<o&&A-r>a}n.d(e,"a",function(){return r}),n.d(e,"b",function(){return i}),n.d(e,"c",function(){return o}),n.d(e,"d",function(){return a})},function(t,e){t.exports=o},function(t,e){t.exports=a},function(t,e){t.exports=s},function(t,e){t.exports=A},function(t,e){t.exports=l},function(t,e){t.exports=c},function(t,e){t.exports=u},function(t,e){t.exports=g},function(t,e){t.exports=h},function(t,e,n){"use strict";var r={};n.r(r),n.d(r,"scale",function(){return d}),n.d(r,"domain",function(){return f}),n.d(r,"labelFunction",function(){return p});var i={};n.r(i),n.d(i,"scale",function(){return E}),n.d(i,"domain",function(){return v}),n.d(i,"labelFunction",function(){return B}),n.d(i,"tickFormatFunction",function(){return b});var o={};n.r(o),n.d(o,"scale",function(){return Q}),n.d(o,"domain",function(){return w}),n.d(o,"labelFunction",function(){return x});var a={};n.r(a),n.d(a,"scale",function(){return N}),n.d(a,"domain",function(){return F}),n.d(a,"labelFunction",function(){return T}),n.d(a,"component",function(){return U});var s=n(1);const A={none:"none",ordinal:"ordinal",time:"time",linear:"linear"},l=t=>{let e="crossValues",n=null,r=null;const i=()=>{const i=r=>{const i=n?t[e].filter(t=>n==t.name):t[e];return!("crossValues"==e&&i.length>1)&&i.some(t=>r.includes(t.type))};return 0===t[e].length?A.none:r!=A.time&&i(["datetime"])?A.time:r!=A.linear&&i(["integer","float"])?A.linear:r==A.ordinal?A.linear:A.ordinal};return i.settingName=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],i):e},i.settingValue=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},i.excludeType=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],i):r},i};n(3),n(10);var c=n(0);const u=t=>{const e=t=>(e,n)=>void 0===e?n:void 0===n?e:t(e,n);return t.reduce((t,n)=>[e(Math.min)(t[0],n[0]),e(Math.max)(t[1],n[1])],[void 0,void 0])},g=t=>Array.isArray(t)?[].concat(...t.map(g)):[t];var h=t=>{const e=e=>t(e);return Object(s.rebindAll)(e,t),e.bandwidth=function(){return arguments.length?(t.bandwidth(...arguments),e):Math.max(t.bandwidth(),1)},e};const d=()=>(t=>{const e=e=>t(e);return Object(s.rebindAll)(e,t),e.ticks=function(){return[]},e})(h(c.scaleBand())),f=()=>{let t=["crossValue"],e="horizontal";const n=e=>{const n=g(e);return r([...new Set(n.map(e=>e[t[0]]))])},r=t=>"vertical"==e?t.reverse():t;return n.valueName=function(){return arguments.length?(t=[arguments.length<=0?void 0:arguments[0]],n):t[0]},n.valueNames=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],n):t},n.orient=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],n):e},n},p=t=>e=>e[t].join("|");n(8),n(6),n(5);var I=n(7);const C=()=>{let t=[0,0],e="percent";const n=n=>{switch(e){case"domain":n[0]-=t[0],n[1]+=t[1];break;case"percent":{let e=n[1]-n[0];n[0]-=t[0]*e,n[1]+=t[1]*e;break}default:throw new Error("Unknown padUnit: "+e)}return n};return n.pad=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],n):t},n.padUnit=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],n):e},n};let m=function(t){if(Array.isArray(t)){let e=Array(t.length);for(let n=0;n<t.length;n++)e[n]=t[n];return e}return Array.from(t)};var y=t=>Math.abs(t)>=1e7?c.format(".3s")(t):Number.isInteger(t)?c.format(",.0f")(t):c.format(",.2f")(t);const E=()=>c.scaleLinear(),v=()=>{const t=function(){let t=[function(t){return t}],e=null,n=[],r=C();const i=function(i){let o=new Array(i.length),a=!0,s=!1,A=void 0,l=t[Symbol.iterator]();try{for(let t;!(a=(t=l.next()).done);a=!0){let e=t.value;for(let t=0;t<i.length;t++){let n=e(i[t],t);Array.isArray(n)?o.push.apply(o,m(n)):o.push(n)}}}catch(t){s=!0,A=t}finally{try{!a&&l.return&&l.return()}finally{if(s)throw A}}let c=[I.min(o),I.max(o)];if(c[0]=null==c[0]?I.min(n):I.min([c[0]].concat(m(n))),c[1]=null==c[1]?I.max(n):I.max([c[1]].concat(m(n))),null!=e){let t=Math.max(Math.abs(c[1]-e),Math.abs(c[0]-e));c[0]=e-t,c[1]=e+t}return r(c)};return i.accessors=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],i):t},i.pad=function(){return arguments.length?(r.pad(arguments.length<=0?void 0:arguments[0]),i):r.pad},i.padUnit=function(){return arguments.length?(r.padUnit(arguments.length<=0?void 0:arguments[0]),i):r.padUnit},i.include=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},i.symmetricalAbout=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],i):e},i.paddingStrategy=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],i):r},i}().pad([0,.1]).padUnit("percent");let e=["crossValue"];const n=n=>(t.accessors(e.map(t=>e=>parseFloat(e[t]))),r(g(n)));s.rebindAll(n,t);const r=n=>{if("domain"==t.padUnit()){const r=(t=>{const n=e.map(e=>t.map(t=>t[e]).sort((t,e)=>t-e).filter((t,e,n)=>0===e||t!==n[e-1]).reduce((t,e,n,r)=>0===n||t<=e-r[n-1]?t:Math.abs(e-r[n-1])));return Math.min(...n)})(n);return t.pad([r/2,r/2])(n)}return t(n)};return n.valueName=function(){return arguments.length?(e=[arguments.length<=0?void 0:arguments[0]],n):e[0]},n.valueNames=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],n):e},n},B=t=>e=>e[t][0],b=y,Q=()=>c.scaleTime(),w=()=>{const t=s.extentTime();let e=["crossValue"];const n=n=>(t.accessors(e.map(t=>e=>new Date(e[t]))),r(g(n)));s.rebindAll(n,t,s.exclude("include","paddingStrategy"));const r=n=>{const r=(t=>{const n=e.map(e=>t.map(t=>new Date(t[e]).getTime()).sort((t,e)=>t-e).filter((t,e,n)=>0===e||t!==n[e-1]).reduce((t,e,n,r)=>0===n||t<=e-r[n-1]?t:Math.abs(e-r[n-1])));return Math.min(...n)})(n);return t.padUnit("domain").pad([r/2,r/2])(n)};return n.valueName=function(){return arguments.length?(e=[arguments.length<=0?void 0:arguments[0]],n):e[0]},n.valueNames=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],n):e},n},x=t=>e=>new Date(e[t][0]);n(4);const _=(t,e,n)=>{let r=6,i=6,o=function(){const t={},e=e=>{for(var n=0,r=Object.keys(t);n<r.length;n++){const i=r[n];e[i](t[i])}return e};for(var n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];for(var o=0,a=r;o<a.length;o++){const n=a[o];e[n]=function(){return arguments.length?(t[n]=arguments.length<=0?void 0:arguments[0],e):t[n]}}return e}("tickFormat","ticks","tickArguments","tickValues","tickPadding"),a=()=>{},A=null;const l=Object(s.dataJoin)("g","group"),u=Object(s.dataJoin)("path","domain"),g=(t,e)=>h()?"translate(".concat(e,", ").concat(t,")"):"translate(".concat(t,", ").concat(e,")"),h=()=>"left"===t||"right"===t,d=s=>{A?(s.selection&&(l.transition(s),u.transition(s)),s.each((s,d,p)=>{const I=p[d],C=Object(c.select)(I),m="bottom"===t||"right"===t?1:-1,y=n.range(),E=(t=>h()?t.map(t=>[t[1],t[0]]):t)([[y[0],m*r],[y[0],0],[y[1],0],[y[1],m*r]]);u(C,[s]).attr("d",Object(c.line)()(E)).attr("stroke","#000").attr("fill","none");const v=l(C,A),B=t=>Array.isArray(i)?i[t]:i,b=t=>{let e=0;for(let n=0;n<t;n++)e+=B(n);return e};v.attr("transform",(t,e)=>g(0,m*b(e))).each((t,r,i)=>{const s=Object(c.select)(i[r]),A=f(n,t);o(e(A)).decorate((t,e)=>a(t,e,r)).tickSizeInner(B(r)).tickOffset(t=>A.step(t)/2)(s),s.select("path.domain").attr("visibility","hidden")}),v.exit().attr("transform",(t,e)=>g(0,m*b(e)))})):o(e(n).decorate(a))(s)},f=(t,e)=>{function n(e){const n=e.domain;return n.reduce((e,n)=>e+t(n),0)/n.length}return n.ticks=()=>e,n.tickFormat=()=>t=>t.text,n.copy=()=>f(t,e),n.step=e=>e.domain.length*t.step(),Object(s.rebindAll)(n,t,Object(s.exclude)("ticks","step","copy")),n};return d.tickSize=function(){return arguments.length?(i=r=Number(arguments.length<=0?void 0:arguments[0]),d):i},d.tickSizeInner=function(){return arguments.length?(i=Array.isArray(arguments.length<=0?void 0:arguments[0])?arguments.length<=0?void 0:arguments[0]:Number(arguments.length<=0?void 0:arguments[0]),d):i},d.tickSizeOuter=function(){return arguments.length?(r=Number(arguments.length<=0?void 0:arguments[0]),d):r},d.decorate=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],d):a},d.groups=function(){return arguments.length?(A=arguments.length<=0?void 0:arguments[0],d):A},Object(s.rebindAll)(d,o),d},D=t=>_("top",s.axisOrdinalTop,t),M=t=>_("bottom",s.axisOrdinalBottom,t),k=t=>_("left",s.axisOrdinalLeft,t),S=t=>_("right",s.axisOrdinalRight,t);function L(t){return t.getRootNode().host}function R(t){return t.closest("#container.chart")}const N=()=>h(c.scaleBand()).padding(.5),F=()=>{let t=["crossValue"],e="horizontal";const n=e=>{const n=g(e);return r([...new Set(n.map(e=>e[t[0]]))])},r=t=>"vertical"==e?t.reverse():t;return n.valueName=function(){return arguments.length?(t=[arguments.length<=0?void 0:arguments[0]],n):t[0]},n.valueNames=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],n):t},n.orient=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],n):e},n},T=t=>e=>e[t].join("|"),U=t=>{let e="horizontal",n="crossValues",r=null;const i=()=>{const i=t[n].length>1,s=a(r),c=s.map(A),u=i?c.map(t=>t.size):c[0].size,h=c.reduce((t,e)=>t+e.size,0),d=t=>n=>{const r=t(n);return i&&r.groups(s).tickSizeInner(u).tickSizeOuter(h),"horizontal"!==e&&r.tickPadding(10),r},f=o(i);return{bottom:d(f.bottom),left:d(f.left),right:d(f.right),top:d(f.top),size:"".concat(h+10,"px"),decorate:(t,n,r)=>{const i=c[r].rotation;"horizontal"===e&&g(t,i),l(t,i)}}},o=t=>t?{bottom:M,left:k,top:D,right:S}:{bottom:s.axisOrdinalBottom,left:s.axisOrdinalLeft,top:s.axisOrdinalTop,right:s.axisOrdinalRight},a=t=>{const e=[];return t.forEach(t=>{(t.split?t.split("|"):[t]).forEach((n,r)=>{for(;e.length<=r;)e.push([]);const i=e[r];i.length>0&&i[i.length-1].text===n?i[i.length-1].domain.push(t):i.push({text:n,domain:[t]})})}),e.reverse()},A=n=>{const r=t.size.width,i=Math.max(...n.map(t=>t.text.length));return"horizontal"===e?16*n.length>r-100?{size:5*i+10,rotation:90}:n.length*(6*i+10)>r-100?{size:3*i+20,rotation:45}:{size:25,rotation:0}:{size:5*i+10,rotation:0}},l=(t,n)=>{const r=n?(t,e)=>t.x+t.width+14>e.x+e.width:(t,e)=>t.x<=e.x+e.width&&e.x<=t.x+t.width&&t.y<=e.y+e.height&&e.y<=t.y+t.height,i="horizontal"==e?u(t):null,o=[];t.each((t,e,n)=>{const a=c.select(n[e]),s=(t=>{const e=-1!==t.indexOf(",")?",":" ",n=t.substring(t.indexOf("(")+1,t.indexOf(")")).split(e).map(t=>parseInt(t));for(;n.length<2;)n.push(0);return n})(a.attr("transform")),A=a.node().getBBox(),l={x:A.x+s[0],y:A.y+s[1],width:A.width,height:A.height},u=o.some(t=>r(t,l))||i&&!((t,e)=>t.x>=e.x&&t.x+t.width<=e.x+e.width&&t.y>=e.y&&t.y+t.height<=e.y+e.height)(l,i);a.attr("visibility",u?"hidden":""),u||o.push(l)})},u=t=>{const e=R(t.node()).querySelector(".cartesian-chart"),n=e.querySelector(".x-axis"),r=e.getBoundingClientRect(),i=n.getBoundingClientRect();return{x:r.left-i.left,width:r.width,y:r.top-i.top,height:r.height}},g=(t,e)=>{const n=(t=>t?"rotate(-".concat(t,t<60?" 5 5)":" 3 7)"):"translate(0, 8)")(e),r=e?"end":"";t.each((t,e,i)=>{c.select(i[e]).select("text").attr("transform",n).style("text-anchor",r)})};return i.orient=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],i):e},i.settingName=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},i.domain=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],i):r},i},O={none:r,ordinal:a,time:o,linear:i},G=t=>{let e=null,n="horizontal",r="crossValues",i=null,o=["crossValue"];const a=["include","paddingStrategy","pad"],A={},c=s=>{const c=l(t).excludeType(e).settingName(r).settingValue(i)(),h=O[c],d=h.domain().valueNames(o);a.forEach(t=>{A[t]&&d[t]&&d[t](A[t])}),d.orient&&d.orient(n);const f=d(s),p=h.component?u(h,f,s):g();return{scale:h.scale(),domain:f,domainFunction:d,labelFunction:h.labelFunction,component:{bottom:p.bottom,left:p.left,top:p.top,right:p.right},size:p.size,decorate:p.decorate,label:t[r].map(t=>t.name).join(", "),tickFormatFunction:h.tickFormatFunction}},u=(e,i,o)=>e.component(t).orient(n).settingName(r).domain(i)(o),g=()=>({bottom:s.axisBottom,left:s.axisLeft,top:s.axisTop,right:s.axisRight,decorate:()=>{}});return c.excludeType=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],c):e},c.orient=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],c):n},c.settingName=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],c):r},c.settingValue=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],c):i},c.valueName=function(){return arguments.length?(o=[arguments.length<=0?void 0:arguments[0]],c):o[0]},c.valueNames=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],c):o},a.forEach(t=>{c[t]=function(){return arguments.length?(A[t]=arguments.length<=0?void 0:arguments[0],c):A[t]}}),c},H=(t,e)=>Y(t,e,s.chartSvgCartesian,!1),j=(t,e)=>Y(t,e,s.chartCanvasCartesian,!0),Y=(t,e,n,r)=>{let i=null,o=null;const a=n({xScale:t.scale,yScale:e.scale,xAxis:t.component,yAxis:e.component}).xDomain(t.domain).xLabel(t.label).xAxisHeight(t.size).xDecorate(t.decorate).xTickFormat(t.tickFormatFunction).yDomain(e.domain).yLabel(e.label).yAxisWidth(e.size).yDecorate(e.decorate).yOrient("left").yTickFormat(e.tickFormatFunction);t.decorate&&a.xDecorate(t.decorate),e.decorate&&a.yDecorate(e.decorate),a.xPaddingInner&&a.xPaddingInner(1),a.xPaddingOuter&&a.xPaddingOuter(.5),a.yPaddingInner&&a.yPaddingInner(1),a.yPaddingOuter&&a.yPaddingOuter(.5),a.axisSplitter=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],a):i},a.altAxis=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],a):o};const A=a.decorate();return a.decorate((n,l)=>{if(A(n,l),i){if(i.haveSplit()){const A=i.altData(),u=s.dataJoin("d3fc-svg","y2-axis").key(t=>t),g=s.dataJoin("g","y-series").key(t=>t);n.enter().append("div").attr("class","y2-label-container").style("grid-column",5).style("-ms-grid-column",5).style("grid-row",3).style("-ms-grid-row",3).style("width",o.size||"1em").style("display","flex").style("align-items","center").style("justify-content","center").append("div").attr("class","y-label").style("transform","rotate(-90deg)");const h=o.scale.domain(o.domain),d=o.component.right(h);if(d.tickFormat(o.tickFormatFunction),o.decorate&&d.decorate(o.decorate),u(n,["right"]).attr("class",t=>"y-axis ".concat(t,"-axis")).on("measure",(t,e,n)=>{const r=c.event.detail,i=r.width,o=r.height;"left"===t&&c.select(n[e]).select("svg").attr("viewBox","".concat(-i," 0 ").concat(i," ").concat(o)),h.range([o,0])}).on("draw",(t,e,n)=>{c.select(n[e]).select("svg").call(d)}),r){const r=n=>{const r=a.plotArea();r.context(n.node().getContext("2d")).xScale(t.scale);const i=[e.scale,h];[l,A].forEach((t,e)=>{r.yScale(i[e]),r(t)})};n.select("d3fc-canvas.plot-area").on("draw",(t,e,n)=>{r(c.select(n[e]).select("canvas"))})}else{const r=n=>{const r=a.plotArea();r.xScale(t.scale);const i=[e.scale,h];g(n,[l,A]).each((t,e,n)=>{r.yScale(i[e]),c.select(n[e]).datum(t).call(r)})};n.select("d3fc-svg.plot-area").on("draw",(t,e,n)=>{r(c.select(n[e]).select("svg"))})}}i(n)}}),a};var P=n(2),K=n(15),z=n.n(K);function J(t,e){switch(t){case"datetime":return e instanceof Date?e:new Date(parseInt(e));case"integer":return parseInt(e,10);case"float":return parseFloat(e)}return e}function q(t,e){return 0===e.crossValues.length?[]:((t.crossValue.split?t.crossValue.split("|"):[t.crossValue])||[t.key]).map((t,n)=>({name:e.crossValues[n].name,value:J(e.crossValues[n].type,t)}))}function V(t,e){if(0===e.splitValues.length)return[];const n=t.key?t.key.split("|"):t.mainValue.split?t.mainValue.split("|"):[t.mainValue];return e.splitValues.map((t,e)=>({name:t.name,value:J(t.type,n[e])}))}function W(t,e){return e.mainValues.length>1?t.mainValue?[{name:t.key,value:t.mainValue-(t.baseValue||0)}]:e.mainValues.map((e,n)=>({name:e.name,value:J(e.type,t.mainValues[n])})):[{name:e.mainValues[0].name,value:J(e.mainValues[0].type,t.colorValue||t.mainValue-t.baseValue||t.mainValue||t.mainValues)}]}var X=n(11);const Z=t=>[t.name,"==",t.value],$=(t,e,n)=>{const r=W(e,n).map(t=>t.name),i=q(e,n).map(Z),o=V(e,n).map(Z),a=n.filter.concat(i).concat(o);t.dispatchEvent(new CustomEvent("perspective-click",{bubbles:!0,composed:!0,detail:{column_names:r,config:{filters:a},row:e.row}}))},tt=()=>{let t=!1,e=null,n=null,r=!1;const i=i=>{const o=i.node();if(!o||!o.isConnected)return void et(e);const a=Object(c.select)(R(o));e=function(t){return Object(P.b)(t,"div.tooltip",()=>t.append("div").attr("class","tooltip").style("z-index",3).style("opacity",0).html(z.a))}(a);const s=(t,i,o)=>{(function(t,e,n){!function(t,e){t.select("#tooltip-values").selectAll("li").data(e).join("li").each(function(t){Object(c.select)(this).text("".concat(t.name,": ")).append("b").text((t=>t.toLocaleString(void 0,{style:"decimal",minimumFractionDigits:Object(X.get_type_config)("float").precision,maximumFractionDigits:Object(X.get_type_config)("float").precision}))(t.value))})}(t,q(e,n).concat(V(e,n)).concat(W(e,n)))})(e,t,n),function(t,e,n,r){const i=t.getBoundingClientRect(),o=e.getBoundingClientRect();let a=o.left+o.width/2-i.left,s=o.top-i.top+t.scrollTop;if(r&&(s=o.top+o.height/2-i.top+t.scrollTop),n.style("left","".concat(a,"px")).style("top","".concat(s,"px")).transition().duration(200).style("opacity",.9),r){var A=function(t,e){return function(t){if(Array.isArray(t))return t}(t)||function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),2!==n.length);r=!0);}catch(t){i=!0,o=t}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(t)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}(function(t,e){const n=t.node().getBoundingClientRect(),r=n.width/2,i=n.left-r-e.left;t.style("left","".concat(i,"px"));const o=n.height/2,a=n.top-o-e.top;return t.style("top","".concat(a,"px")),[i,a]}(n,i));a=A[0],s=A[1]}!function(t,e,n,r){let i=arguments.length>4&&void 0!==arguments[4]&&arguments[4];const o=t.node().getBoundingClientRect();if(Object(P.c)(e,o)){const r=o.right-e.right;t.style("left","".concat(n-r,"px"))}if(Object(P.c)(e,o,"bottom")){const n=o.bottom-e.bottom;t.style("top","".concat(r-n,"px"))}if(i){if(Object(P.c)(e,o,"left")){const r=o.left-e.left;t.style("left","".concat(n-r,"px"))}if(Object(P.c)(e,o,"top")){const n=o.top-e.top;t.style("top","".concat(r-n,"px"))}}}(n,i,a,s,r)}(a.node(),o[i],e,r),Object(c.select)(o[i]).style("opacity","0.7")};t?i.each(s):(i.on("mouseover",s).on("mouseout",(t,n,r)=>{et(e),r&&Object(c.select)(r[n]).style("opacity","1")}),(()=>{let t=null;const e=e=>{const n=e.node();e.on("click",e=>$(n,e,t))};return e.settings=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],e):t},e})().settings(n)(i))};return i.alwaysShow=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],i):t},i.centered=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],i):r},i.settings=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},i};function et(t){t&&t.transition().duration(500).style("opacity",0)}function nt(t,e){let n=t.mainValues.length>1?s.seriesSvgGrouped(s.seriesSvgBar()):s.seriesSvgBar();return n=n.decorate(n=>{tt().settings(t)(n),n.style("fill",t=>e(t.key))}),s.autoBandwidth(rt(n)).crossValue(t=>t.crossValue).mainValue(t=>t.mainValue).baseValue(t=>t.baseValue)}const rt=t=>{const e=e=>t(e);return s.rebindAll(e,t),e.bandwidth=function(){return arguments.length?(t.bandwidth(Math.max(arguments.length<=0?void 0:arguments[0],1)),e):t.bandwidth()},e};function it(t){return t.split("|").slice(0,-1).join("|")}function ot(t){const e=t.data&&t.data.length>0?t.data[0]:{},n=Object.keys(e).filter(t=>"__ROW_PATH__"!==t);return at().settings(t).domain(n)()}function at(){let t=null,e=null,n={},r=t=>At(t,n.colorStyles&&n.colorStyles.opacity);const i=()=>{const i=n.colorStyles,o=e||[i.series];if(o||t.length>1){const e=t.length>1?i.scheme:o;return c.scaleOrdinal(e.map(r)).domain(t)}return null};return i.domain=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],i):t},i.defaultColors=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],i):e},i.mapFunction=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],i):r},i.settings=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},i}function st(t){return lt(1)(t)}function At(t){return lt(arguments.length>1&&void 0!==arguments[1]?arguments[1]:.5)(t)}function lt(t){return e=>{const n=c.color(e);return n.opacity=t,n+""}}const ct={none:p,ordinal:T,time:x,linear:B},ut=t=>{const e=l(t);let n="__ROW_PATH__";const r=(t,r)=>ct[e()](n)(t,r);return Object(s.rebindAll)(r,e),r.valueName=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],r):n},r};function gt(t,e){let n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=n.stack,i=void 0!==r&&r,o=n.excludeEmpty,a=void 0!==o&&o;const s=e||t.data;return t.splitValues.length>0?function(t,e,n){let r=n.stack,i=void 0!==r&&r,o=n.excludeEmpty,a=void 0!==o&&o;const s={};return e.forEach(t=>{const e={},n={};Object.keys(t).filter(t=>"__ROW_PATH__"!==t).filter(e=>!a||null!=t[e]&&null!=t[e]).forEach(r=>{const o=r.split("|"),a=o[o.length-1],s=t[r]||0,A="".concat(a).concat(s>=0?"+ve":"-ve"),l=o.slice(0,o.length-1).join("|"),c=n[l]=n[l]||{__ROW_PATH__:t.__ROW_PATH__},u=e[A]||0;c.__KEY__=l,i?(c[a]=u+s,c["__BASE_VALUE__".concat(a)]=u,e[A]=c[a]):c[a]=s,c.row=t}),Object.keys(n).forEach(t=>{(s[t]=s[t]||[]).push(n[t])})}),Object.keys(s).map(t=>{const e=s[t];return e.key=t,e})}(0,s,{stack:i,excludeEmpty:a}):[s]}function ht(t,e){const n={stack:!0};return gt(t,e,n).map(e=>dt(t,e,n))}function dt(t,e,n){let r=n.stack;const i=function(t,e,n){let r=n.stack,i=void 0!==r&&r;const o=ut(t);return t=>{const n=e=>i&&e["__BASE_VALUE__".concat(t.name)]||0,r=e.map((e,r)=>({crossValue:o(e,r),mainValue:e[t.name]?e[t.name]:null,baseValue:n(e),key:e.__KEY__?"".concat(e.__KEY__,"|").concat(t.name):t.name,row:e.row||e}));return r.key=r[0].key,r}}(t,e,{stack:void 0!==r&&r});return t.mainValues.length>1?t.mainValues.map(i):i(t.mainValues[0])}var ft=n(9),pt=n(16),It=n.n(pt);const Ct=10;function mt(t,e,n){const r=c.select(L(t).getContainer()).node().getBoundingClientRect(),i=t.getBoundingClientRect(),o={top:i.top+n-Ct,right:i.right+e+Ct,bottom:i.bottom+n+Ct,left:i.left+e-Ct},a={x:e,y:n};return[{edge:"right",dimension:"x"},{edge:"left",dimension:"x"},{edge:"top",dimension:"y"},{edge:"bottom",dimension:"y"}].forEach(t=>{if(Object(P.c)(r,o,t.edge)){const e=o[t.edge]-r[t.edge];a[t.dimension]=a[t.dimension]-e}}),a}function yt(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t}const Et="resize.for-dragging";function vt(){let t=!0,e=null;const n=n=>{const r=n.node();r.style.cursor="move",e.legend&&(r.style.left=e.legend.left,r.style.top=e.legend.top);const i=c.drag().on("drag",function(){const n=mt(this,c.event.dx,c.event.dy);this.style.left="".concat(this.offsetLeft+n.x,"px"),this.style.top="".concat(this.offsetTop+n.y,"px");const i={left:this.style.left,top:this.style.top};e.legend=function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter(function(t){return Object.getOwnPropertyDescriptor(n,t).enumerable}))),r.forEach(function(e){yt(t,e,n[e])})}return t}({},e.legend,i),t=function(t){const e=t.getBoundingClientRect(),n=c.select(R(t)).node().getBoundingClientRect();return e.right+Ct+5>=n.right&&e.top-Ct-5<=n.top}(r)?function(t){return c.select(window).on(Et,null),t.style.left="auto",!0}(r):function(t,e){return!1!==e&&c.select(window).on(Et,function(){const e=mt(t,0,0);t.style.left="".concat(t.offsetLeft+e.x,"px"),t.style.top="".concat(t.offsetTop+e.y,"px")}),!1}(r,t)});n.call(i)};return n.settings=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],n):e},n}function Bt(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t}function bt(t,e){return function(t){if(Array.isArray(t))return t}(t)||function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!e||n.length!==e);r=!0);}catch(t){i=!0,o=t}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(t,e)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}const Qt="horizontal-drag-handle",wt="vertical-drag-handle",xt="corner-drag-handle",_t="dragHandles",Dt=0,Mt="resize";function kt(){let t=9,e=3,n=null;const r={height:100,width:100},i={height:null,width:null},o=[],a=(t,e)=>o.filter(e=>e.event===t).forEach(t=>t.execute(e)),s=r=>{if(Lt(r))return;const o={left:()=>a(Mt,{horizontal:D(c.event),vertical:!1}),top:()=>a(Mt,{horizontal:!1,vertical:k(c.event)}),right:()=>a(Mt,{horizontal:M(c.event),vertical:!1}),bottom:()=>a(Mt,{horizontal:!1,vertical:S(c.event)}),topleft:()=>a(Mt,{horizontal:D(c.event),vertical:k(c.event)}),topright:()=>a(Mt,{horizontal:M(c.event),vertical:k(c.event)}),bottomright:()=>a(Mt,{horizontal:M(c.event),vertical:S(c.event)}),bottomleft:()=>a(Mt,{horizontal:D(c.event),vertical:S(c.event)})},s=r.node();n.legend&&(s.style.height=n.legend.height,s.style.width=n.legend.width);const u=s.getBoundingClientRect(),g=r.append("svg").attr("id",_t).attr("width",u.width).attr("height",u.height),h=g.append("g"),d=t=>"left"===t||"right"===t,f={left:0,top:t,right:u.width-t,bottom:t},p={left:t,top:0,right:t,bottom:u.height-t},I=bt(["left","top","right","bottom"].map(n=>h.append("rect").attr("id","drag".concat(n)).attr("class",d(n)?wt:Qt).attr("y",p[n]).attr("x",f[n]).attr("height",d(n)?u.height-2*t:t).attr("width",d(n)?t:u.width-2*t).attr("fill",d(n)?"lightgreen":"lightblue").attr("fill-opacity",Dt).style("z-index",e).attr("cursor",d(n)?"ew-resize":"ns-resize").call(c.drag().on("drag",o[n]))),4),C=I[0],m=I[1],y=I[2],E=I[3],v=t=>"".concat(t[0]).concat(t[1]),B={topleft:"nwse",topright:"nesw",bottomright:"nwse",bottomleft:"nesw"},b=bt([["top","left"],["top","right"],["bottom","right"],["bottom","left"]].map(n=>h.append("rect").attr("id","drag".concat(v(n))).attr("class","".concat(xt," ").concat(n[0]," ").concat(n[1])).attr("height",t).attr("width",t).attr("fill","red").attr("fill-opacity",Dt).style("z-index",e).attr("cursor","".concat(B[v(n)],"-resize")).call(c.drag().on("drag",o[v(n)]))),4),Q=b[0],w=b[1],x=b[2],_=b[3];function D(t){const e=l(mt(C.node(),t.x,0).x,g,"width",(t,e)=>t-e);return s.style.left="".concat(s.offsetLeft+e,"px"),s.style.width="".concat(s.offsetWidth-e,"px"),L(),R(y,e,"width","x")}function M(t){const e=-l(mt(y.node(),t.dx,0).x,g,"width",(t,e)=>t+e);return!A(e,"x",y,t)&&(s.style.width="".concat(s.offsetWidth-e,"px"),L(),R(y,e,"width","x"))}function k(t){const e=l(mt(m.node(),0,t.y).y,g,"height",(t,e)=>t-e);return s.style.top="".concat(s.offsetTop+e,"px"),s.style.height="".concat(s.offsetHeight-e,"px"),L(),R(E,e,"height","y")}function S(t){const e=-l(mt(E.node(),0,t.dy).y,g,"height",(t,e)=>t+e);return!A(e,"y",E,t)&&(s.style.height="".concat(s.offsetHeight-e,"px"),L(),R(E,e,"height","y"))}function L(){const t={top:s.style.top,left:s.style.left,height:s.style.height,width:s.style.width};n.legend=function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter(function(t){return Object.getOwnPropertyDescriptor(n,t).enumerable}))),r.forEach(function(e){Bt(t,e,n[e])})}return t}({},n.legend,t)}function R(t,e,n,r){return Nt(g,n,e),Rt(t,r,e),St(g,e,n,"height"===n?wt:Qt),N(g),0!=e}function N(e){Q.attr("y",0,"x",0),w.attr("y",0).attr("x",e.attr("width")-t),x.attr("y",e.attr("height")-t).attr("x",e.attr("width")-t),_.attr("y",e.attr("height")-t).attr("x",0)}function F(t,e,n){i[t]&&i[t]<u[t]&&(s.style[t]="".concat(i[t],"px"),R(n,u[t]-i[t],t,e))}F("height","y",E),F("width","x",y),N(g)};function A(t,e,n,r){return((t,e,n,r)=>t<0&&r[e]<Number(n.attr(e)))(t,e,n,r)||((t,e,n,r)=>t>0&&r[e]>Number(n.attr(e)))(t,e,n,r)}function l(t,e,n,o){const a=o(Number(e.attr(n)),t);return a<r[n]?o(t,r[n]-a):i[n]&&a>i[n]?o(t,i[n]-a):t}return s.on=(t,e)=>(o.push({event:t,execute:e}),s),s.zIndex=t=>(e=t,s),s.settings=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],s):n},s.minWidth=t=>(r.width=t,i.width&&(i.width=Math.max(r.width,i.width)),s),s.minHeight=t=>(r.height=t,i.height&&(i.height=Math.max(r.height,i.height)),s),s.handleWidth=e=>(t=e,s),s.maxWidth=t=>(i.width=t,r.width=Math.min(r.width,i.width),s),s.maxHeight=t=>(i.height=t,r.height=Math.min(r.height,i.height),s),s}const St=(t,e,n,r)=>{t.selectAll(".".concat(r)).each((t,r,i)=>{const o=i[r];c.select(o).attr(n,o.getBoundingClientRect()[n]-e)})},Lt=t=>t.select("#".concat(_t)).size()>0,Rt=(t,e,n)=>t.attr(e,Number(t.attr(e))-n),Nt=(t,e,n)=>t.attr(e,t.node().getBoundingClientRect()[e]-n);function Ft(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t}var Tt=(t,e)=>{const n=t||ft.legendColor();let r,i,o=[],a=1,A=e.legend&&e.legend.pageIndex?e.legend.pageIndex:0,l=()=>{},c=vt().settings(e);const u=t=>{o=n.scale().domain(),(i=kt().settings(e).maxHeight(16*o.length+20).on("resize",()=>g(t)))(t),c(t),g(t)},g=t=>{I(t),h(t),d(t),function(t){const e=t.select("g.legendCells"),n=t.node().getBoundingClientRect();if(!Object(P.c)(n,e.node().getBoundingClientRect()))return;const r=t.select(".legend");e.selectAll(".label").text((t,e,i)=>{const o=i[e];if(Object(P.c)(n,o.getBoundingClientRect())){const e=function(t,e,n){const r=t.getBoundingClientRect(),i=e.node().createSVGPoint();return i.x=n.right-r.left-3,i.y=0,t.getCharNumAtPosition(i)}(o,r,n);return"".concat(t.substring(0,e-3),"...")}return t})}(t)},h=t=>{const e=m(t);e.style("display",a<=1?"none":"block"),e.select("#page-text").text("".concat(A+1,"/").concat(a)),e.select("#up-arrow").attr("class",0===A?"disabled":"").on("click",()=>{A>0&&(f(A-1),g(t))}),e.select("#down-arrow").attr("class",A>=a-1?"disabled":"").on("click",()=>{A<a-1&&(f(A+1),g(t))})},d=t=>{a>1&&n.cellFilter(p()),t.select("g.legendCells").remove();const e=y(t);e.call(n);const r=t.select("g.legendCells").node().getBBox();e.attr("height",r.height+20),l(t)},f=t=>{A=t,e.legend=function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter(function(t){return Object.getOwnPropertyDescriptor(n,t).enumerable}))),r.forEach(function(e){Ft(t,e,n[e])})}return t}({},e.legend,{pageIndex:A})},p=()=>(t,e)=>e>=r*A&&e<r*A+r,I=t=>{const e=t.node().getBoundingClientRect();let n=Math.floor(e.height/16)-1;r=n<o.length?n-1:n,a=C(n),A=Math.min(A,a-1)},C=t=>Math.ceil(o.length/t),m=t=>Object(P.b)(t,".legend-controls",()=>t.append("g").attr("class","legend-controls").html(It.a)),y=t=>Object(P.b)(t,".legend",()=>t.append("svg").attr("class","legend"));return u.decorate=function(){return arguments.length?(l=arguments.length<=0?void 0:arguments[0],u):l},Object(s.rebindAll)(u,n),u};const Ut=t=>Tt(ft.legendColor().shape("circle").shapeRadius(6),t),Ot=t=>Tt(ft.legendSymbol().shapePadding(1).labelOffset(3),t),Gt=()=>Yt(Ut),Ht=()=>Yt(Ot,jt);function jt(t){if(!t)return null;const e=t.domain(),n=t.range().map(t=>c.symbol().type(t)());return c.scaleOrdinal().domain(e).range(n)}function Yt(t,e){let n={},r=null,i=null;function o(e){if(r&&r.range().length>1){const o=t(n);o.scale(r).orient("vertical").on("cellclick",function(t){n.hideKeys=n.hideKeys||[],n.hideKeys.includes(t)?n.hideKeys=n.hideKeys.filter(e=>e!==t):n.hideKeys.push(t),L(this).draw()}),o.labels(t=>{const e=t.domain[t.i].split("|");return n.mainValues.length<=1&&e.length>1?e.slice(0,e.length-1).join("|"):t.domain[t.i]});const a=Object(P.b)(e,"div.legend-container",()=>e.append("div"));o.decorate(t=>{const e=t=>n.hideKeys&&n.hideKeys.includes(t),r=t.select("g.legendCells").attr("transform","translate(20,20)").selectAll("g.cell");r.classed("hidden",e),r.append("title").html(t=>t),i&&r.select("path").style("fill",t=>e(t)?null:i(t)).style("stroke",t=>e(t)?null:st(i(t)))}),a.attr("class","legend-container").attr("borderbox-on-hover",!0).style("z-index","2").call(o)}}return o.settings=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],o):n},o.scale=function(){return arguments.length?(r=e?e(arguments.length<=0?void 0:arguments[0]):arguments.length<=0?void 0:arguments[0],o):r},o.color=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],o):i},o}function Pt(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t}function Kt(t,e){const n=e||t.data;return t.hideKeys&&t.hideKeys.length>0?n.map(e=>{const n=function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter(function(t){return Object.getOwnPropertyDescriptor(n,t).enumerable}))),r.forEach(function(e){Pt(t,e,n[e])})}return t}({},e);return t.hideKeys.forEach(t=>{delete n[t]}),n}):n}function zt(t,e){const n=e||t.data;return t.hideKeys&&t.hideKeys.length>0?n.map(e=>{const n={};return Object.keys(e).map(r=>{t.hideKeys.includes(it(r))||(n[r]=e[r])}),n}):n}const Jt=t=>{t.globalAlpha=.3,t.lineWidth=1},qt=t=>t.style("display","none"),Vt=t=>{t.globalAlpha=0};var Wt=(t,e)=>{let n="both",r=!1,i=null,o=null,a=null,A=s.seriesSvgMulti(),l=s.annotationSvgGridline(),c=(t=>e=>e.style("opacity","0.3").style("stroke-width","1.0").style("stroke",t?t.colorStyles.grid.gridLineColor:"#bbb"))(e),u=qt;const g=function(){r&&(A=s.seriesCanvasMulti().context(a),l=s.annotationCanvasGridline(),c=Jt,u=Vt);const e=A.xScale(i).yScale(o),g="vertical"===n?u:c,h="horizontal"===n?u:c,d=l.xDecorate(g).yDecorate(h);return e.series([d,t])(...arguments)};return g.orient=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],g):n},g.canvas=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],g):r},g.xScale=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],g):i},g.yScale=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],g):o},g.context=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],g):a},g};const Xt=()=>{const t=C(),e=e=>{let n=t.pad(),r=t.padUnit(),i=1;switch(r){case"domain":break;case"percent":i=e[1]-e[0];break;default:throw new Error("Unknown padUnit: "+r)}let o=e[0]-n[0]*i,a=e[1]+n[1]*i;return e[0]=e[0]>=0&&o<0?0:o,e[1]=e[1]<=0&&a>0?0:a,e};return s.rebindAll(e,t),e};var Zt=n(17),$t=n.n(Zt);function te(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t}var ee=()=>{let t=null,e=null,n=null,r=null,i=null,o=null,a=!1,s=!1,A=()=>{};function l(A){const l="d3fc-".concat(s?"canvas":"svg",".plot-area");if(n||i){const s=r&&r.domain()[0]instanceof Date,h=c.zoom().on("zoom",()=>{const a=c.event.transform;e.zoom={k:a.k,x:a.x,y:a.y},u(a),A.call(t);const d=1===a.k&&0===a.x&&0===a.y,f=g(A).style("display",d?"none":"");f.select("#zoom-reset").on("click",()=>A.select(l).call(h.transform,c.zoomIdentity));const p=f.select("#one-year").style("display",s?"":"none"),I=f.select("#six-months").style("display",s?"":"none"),C=f.select("#one-month").style("display",s?"":"none");if(s){const t=t=>()=>{const e=new Date(n.domain()[0]),a=new Date(e);t(e,a);const s=r.range(),u=(s[1]-s[0])/(r(a)-r(e)),g=-r(e)*u;let d=0;if(i){const t=i.domain().reduce((t,e)=>t+e)/2;d=-o(t)*u+i(t)}A.select(l).call(h.transform,c.zoomIdentity.translate(g,d).scale(u))};p.on("click",t((t,e)=>e.setYear(t.getFullYear()+1))),I.on("click",t((t,e)=>e.setMonth(t.getMonth()+6))),C.on("click",t((t,e)=>e.setMonth(t.getMonth()+1)))}}),d=t.decorate();t.decorate((t,n)=>{if(d(t,n),!a){a=!0;const n=t.select(l);n.on("measure.zoom-range",()=>{if(r&&r.range([0,c.event.detail.width]),o&&o.range([0,c.event.detail.height]),e.zoom){const t=c.zoomIdentity.translate(e.zoom.x,e.zoom.y).scale(e.zoom.k);n.call(h.transform,t)}}).call(h)}})}A.call(t)}l.chart=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],l):t},l.settings=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],l):e},l.xScale=function(){return arguments.length?(n=h(arguments.length<=0?void 0:arguments[0]),r=n?n.copy():null,l):n},l.yScale=function(){if(!arguments.length)return i;if(i=h(arguments.length<=0?void 0:arguments[0]),o=i?i.copy():null){const t=o.domain();o.domain([t[1],t[0]])}return l},l.canvas=function(){return arguments.length?(s=arguments.length<=0?void 0:arguments[0],l):s},l.onChange=function(){return arguments.length?(A=arguments.length<=0?void 0:arguments[0],l):A};const u=t=>{const e=function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter(function(t){return Object.getOwnPropertyDescriptor(n,t).enumerable}))),r.forEach(function(e){te(t,e,n[e])})}return t}({},t);if(n&&(n.domain(t.rescaleX(r).domain()),e.xDomain=n.domain()),i){const n=t.rescaleY(o).domain();i.domain([n[1],n[0]]),e.yDomain=i.domain()}A(e)},g=t=>Object(P.b)(t,".zoom-controls",()=>t.append("div").attr("class","zoom-controls").style("display","none").html($t.a)),h=t=>t&&t.nice?t:null;return l};function ne(t,e){const n=ht(e,Kt(e)),r=ot(e),i=Gt().settings(e).scale(r),o=nt(e,r).orient("horizontal"),a=s.seriesSvgMulti().mapping((t,e)=>t[e]).series(n.map(()=>o)),l=G(e).settingName("mainValues").valueName("mainValue").excludeType(A.ordinal).include([0]).paddingStrategy(Xt())(n),c=G(e).excludeType(A.linear).settingName("crossValues").valueName("crossValue").orient("vertical")(n),u=H(l,c).plotArea(Wt(a,e).orient("horizontal"));u.yPaddingInner&&(u.yPaddingInner(.5),u.yPaddingOuter(.25),o.align("left")),u.xNice&&u.xNice();const g=ee().chart(u).settings(e).yScale(c.scale);t.datum(n).call(g),t.call(i)}ne.plugin={type:"d3_x_bar",name:"X Bar Chart",max_size:25e3};var re=ne,ie=(t,e)=>{if(!oe(t)||!oe(e))return;const n=ae(t),r=ae(e);n>r?e[0]=se(e,n):t[0]=se(t,r)};const oe=t=>2===t.length&&!isNaN(t[0])&&!isNaN(t[1])&&t[0]!==t[1],ae=t=>(0-t[0])/(t[1]-t[0]),se=(t,e)=>e*t[1]/(e-1),Ae=t=>{let e,n=[],r=!1;const i=i=>{i.text("");const a=s.dataJoin("span","splitter-label").key(t=>t),A=!r&&1===n.length,l=e&&0===t.splitValues.length;a(i,n).classed("disabled",A).text(t=>t.name).style("color",t=>l?st(e(t.name)):void 0).on("click",e=>{A||(t.splitMainValues=r?t.splitMainValues.filter(t=>t!=e.name):[e.name].concat(t.splitMainValues||[]),o(i))})},o=t=>{const e=L(t.node());e.remove(),e.draw()};return i.labels=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},i.alt=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],i):r},i.color=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],i):e},i},le=function(t,e){let n,r,i,o=arguments.length>2&&void 0!==arguments[2]?arguments[2]:ce;const a=t.splitMainValues||[],s=t=>{const e=t.split("|");return a.includes(e[e.length-1])},A=t.mainValues.some(t=>s(t.name));r=A?o(e,t=>!s(t)):e,i=A?o(e,s):null;const l=e=>{if(1===t.mainValues.length)return;const r=t.mainValues.map((t,e)=>({index:e,name:t.name})),i=r.filter(t=>!s(t.name)),o=r.filter(t=>s(t.name)),a=()=>Ae(t).color(n);e.select(".y-label-container>.y-label").call(a().labels(i)),e.select(".y2-label-container>.y-label").call(a().labels(o).alt(!0))};return l.color=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],l):n},l.haveSplit=()=>A,l.data=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],l):r},l.altData=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],l):i},l},ce=(t,e)=>t.map(t=>t.filter(t=>e(t.key))),ue=(t,e)=>t.map(t=>e(t.key)?t:t.map(t=>Object.assign({},t,{mainValue:null}))),ge=(t,e)=>t.map(t=>ue(t,e));function he(t,e){const n=ht(e,Kt(e)),r=ot(e),i=Gt().settings(e).scale(r),o=nt(e,r).orient("vertical"),a=s.seriesSvgMulti().mapping((t,e)=>t[e]).series(n.map(()=>o)),l=G(e).excludeType(A.linear).settingName("crossValues").valueName("crossValue")(n),c=G(e).settingName("mainValues").valueName("mainValue").excludeType(A.ordinal).orient("vertical").include([0]).paddingStrategy(Xt()),u=e.mainValues.length>1?ge:ue,g=le(e,n,u).color(r),h=c(g.data()),d=g.haveSplit()?a:Wt(a,e).orient("vertical"),f=H(l,h).axisSplitter(g).plotArea(d);f.xPaddingInner&&(f.xPaddingInner(.5),f.xPaddingOuter(.25),o.align("left")),f.yNice&&f.yNice();const p=ee().chart(f).settings(e).xScale(l.scale);if(g.haveSplit()){const t=c(g.altData());ie(h.domain,t.domain),f.yDomain(h.domain).altAxis(t)}t.datum(g.data()).call(p),t.call(i)}he.plugin={type:"d3_y_bar",name:"Y Bar Chart",max_size:25e3};var de=he;const fe=function(t){const e=[];var n=!0,r=!1,i=void 0;try{for(var o,a=t[Symbol.iterator]();!(n=(o=a.next()).done);n=!0){const t=o.value;for(let n=0;n<t.length;n++)null!==t[n].mainValue&&(e[n]=e[n]||[],e[n].push(t[n]))}}catch(t){r=!0,i=t}finally{try{n||null==a.return||a.return()}finally{if(r)throw i}}return e},pe=function(t,e){let n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:Math.min;const r=t=>t.reduce((t,r)=>{const o=i(r,e);return!o||t&&n(t.value,o.value)!==o.value?t:o},null),i=t=>{if(Array.isArray(t))return r(t,e);const n=e(t);return null!==n?{item:t,value:n}:null},o=r(t,e);return o?o.item:null};var Ie=()=>{const t=tt().alwaysShow(!0);let e=null,n=null,r=null,i=null,o=!1,a=null,A="crossValue",l="mainValue",c=null;function u(r){const i="d3fc-".concat(o?"canvas":"svg",".plot-area");if(e||n){let e=null;const o=s.pointer().on("point",t=>{const i=t.length?h(t[0]):null;e=i?[i.data]:[];const o=i?i.scale:n;g(r,e,o)});r.select(i).on("measure.nearbyTip",()=>g(r,[])).on("click",()=>{e.length&&$(r.node(),e[0],t.settings())}).call(o)}}const g=function(o,a){let s=arguments.length>2&&void 0!==arguments[2]?arguments[2]:n;const c=o.select("d3fc-svg.plot-area svg").selectAll("circle.nearbyTip").data(a);c.exit().remove(),c.enter().append("circle").attr("class","nearbyTip").merge(c).attr("r",t=>i?Math.sqrt(i(t.size)):10).attr("transform",t=>"translate(".concat(e(t[A]),",").concat(s(t[l]),")")).style("stroke","none").style("fill",t=>r&&At(r(t.key))),t(c)},h=t=>{const r=n=>r=>void 0===r[l]||null===r[l]||void 0===r[A]||null===r[A]?null:Math.sqrt(Math.pow(e(r[A])-t.x,2)+Math.pow(n(r[l])-t.y,2)),i=r(n),o=pe(a,i,Math.min);if(c){const t=r(c.yScale),e=pe(c.data,t,Math.min);return i(o)<=t(e)?{data:o,scale:n}:{data:e,scale:c.yScale}}return{data:o,scale:n}};return u.xScale=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],u):e},u.yScale=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],u):n},u.color=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],u):r},u.size=function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];return e.length?(i=e[0]?e[0].copy().range([40,4e3]):null,u):i},u.canvas=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],u):o},u.data=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],u):a},u.xValueName=function(){return arguments.length?(A=arguments.length<=0?void 0:arguments[0],u):A},u.yValueName=function(){return arguments.length?(l=arguments.length<=0?void 0:arguments[0],u):l},u.altDataWithScale=function(){return arguments.length?(c=arguments.length<=0?void 0:arguments[0],u):c},s.rebindAll(u,t),u};function Ce(t,e){const n=function(t,e){const n=ut(t);return e.map((t,e)=>Object.keys(t).filter(t=>"__ROW_PATH__"!==t).map(r=>({key:r,crossValue:n(t,e),mainValue:t[r],row:t})))}(e,Kt(e)),r=ot(e),i=Gt().settings(e).scale(r),o=s.seriesSvgRepeat().series(function(t,e){let n=s.seriesSvgLine();return(n=n.decorate(t=>{t.style("stroke",t=>st(e(t[0]&&t[0].key)))})).crossValue(t=>t.crossValue).mainValue(t=>t.mainValue)}(0,r)).orient("horizontal"),a=Xt().pad([.1,.1]).padUnit("percent"),l=G(e).excludeType(A.linear).settingName("crossValues").valueName("crossValue")(n),c=G(e).settingName("mainValues").valueName("mainValue").orient("vertical").paddingStrategy(a),u=le(e,fe(n)).color(r),g=c(u.data()),h=u.haveSplit()?o:Wt(o,e).orient("vertical"),d=H(l,g).axisSplitter(u).plotArea(h);d.yNice&&d.yNice();const f=ee().chart(d).settings(e).xScale(l.scale),p=Ie().settings(e).xScale(l.scale).yScale(g.scale).color(r).data(n);if(u.haveSplit()){const t=c(u.altData());d.altAxis(t),p.data(u.data()).altDataWithScale({yScale:t.scale,data:u.altData()})}const I=u.data();t.datum(I).call(f),t.call(p),t.call(i)}Ce.plugin={type:"d3_y_line",name:"Y Line Chart",max_size:25e3};var me=Ce;function ye(t,e){const n=function(t,e){const n=ut(t);return e.map((t,e)=>{const r={};return Object.keys(t).filter(t=>"__ROW_PATH__"!==t).map(i=>{const o=i.split("|"),a=o[o.length-1],s=r[a]||0,A=s+t[i];return r[a]=A,{key:i,crossValue:n(t,e),mainValue:A,baseValue:s,row:t}})})}(e,Kt(e)),r=ot(e),i=Gt().settings(e).scale(r),o=s.seriesSvgRepeat().series(function(t,e){let n=s.seriesSvgArea();return(n=n.decorate(t=>{t.style("fill",t=>e(t[0].key))})).crossValue(t=>t.crossValue).mainValue(t=>t.mainValue).baseValue(t=>t.baseValue)}(0,r).orient("vertical")),a=G(e).excludeType(A.linear).settingName("crossValues").valueName("crossValue")(n),l=G(e).settingName("mainValues").valueName("mainValue").excludeType(A.ordinal).orient("vertical").include([0]).paddingStrategy(Xt()),c=le(e,n).color(r),u=l(c.data()),g=c.haveSplit()?o:Wt(o,e).orient("vertical"),h=H(a,u).axisSplitter(c).plotArea(g);h.yNice&&h.yNice();const d=ee().chart(h).settings(e).xScale(a.scale),f=Ie().settings(e).xScale(a.scale).yScale(u.scale).color(r).data(n);if(c.haveSplit()){const t=l(c.altData());h.altAxis(t),f.data(c.data()).altDataWithScale({yScale:t.scale,data:c.altData()})}t.datum(c.data()).call(d),t.call(f),t.call(i)}ye.plugin={type:"d3_y_area",name:"Y Area Chart",max_size:25e3};var Ee=ye;const ve=[c.symbolCircle,c.symbolCross,c.symbolDiamond,c.symbolSquare,c.symbolStar,c.symbolTriangle,c.symbolWye];function Be(t){return t.length>1?c.scaleOrdinal().domain(t).range(ve):null}function be(t,e){const n=function(t,e){const n={stack:!1},r=gt(t,e,n).map(e=>dt(t,e,n));return t.mainValues.length>1?r.reduce((t,e)=>t.concat(e)):r}(e,Kt(e)),r=function(t){const e=t.data&&t.data.length>0?t.data[0]:{};return Be(Object.keys(e).filter(t=>"__ROW_PATH__"!==t))}(e),i=ot(e),o=Ht().settings(e).scale(r).color(i),a=s.seriesSvgMulti().mapping((t,e)=>t[e]).series(n.map(t=>(function(t,e,n,r){let i=s.seriesSvgPoint().size(100);const o=t.colorStyles&&t.colorStyles.opacity;return r&&i.type(r(e)),i.decorate(t=>{t.style("stroke",t=>st(n(t.colorValue||e))).style("fill",t=>At(n(t.colorValue||e),o))}),i.crossValue(t=>t.crossValue).mainValue(t=>t.mainValue)})(e,t.key,i,r))),l=Xt().pad([.05,.05]).padUnit("percent"),c=G(e).excludeType(A.linear).settingName("crossValues").valueName("crossValue")(n),u=G(e).settingName("mainValues").valueName("mainValue").orient("vertical").paddingStrategy(l),g=le(e,n).color(i),h=u(g.data()),d=g.haveSplit()?a:Wt(a,e).orient("vertical"),f=H(c,h).axisSplitter(g).plotArea(d);f.yNice&&f.yNice();const p=ee().chart(f).settings(e).xScale(c.scale),I=Ie().settings(e).xScale(c.scale).yScale(h.scale).color(i).data(n);if(g.haveSplit()){const t=u(g.altData());f.altAxis(t),I.data(g.data()).altDataWithScale({yScale:t.scale,data:g.altData()})}t.datum(g.data()).call(p),t.call(I),o&&t.call(o)}be.plugin={type:"d3_y_scatter",name:"Y Scatter Chart",max_size:25e3};var Qe=be;function we(t,e){return function(t){if(Array.isArray(t))return t}(t)||function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!e||n.length!==e);r=!0);}catch(t){i=!0,o=t}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(t,e)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}function xe(t,e,n,r){let i=_e(e,n,r),o=t.colorStyles.gradient.full;if(i[0]>=0)o=t.colorStyles.gradient.positive;else if(i[1]<=0)o=t.colorStyles.gradient.negative;else{const t=Math.max(-i[0],i[1]);i=[-t,t]}const a=De(o);return c.scaleSequential(a).domain(i)}const _e=(t,e,n)=>n||v().valueName(e).pad([0,0])(t),De=t=>{const e=t.slice(1).map((e,n)=>c.interpolate(t[n][1],e[1]));return n=>{const r=t.findIndex((e,r)=>r<t.length-1&&n<=t[r+1][0]&&n>e[0]);if(-1===r)return n<=t[0][0]?t[0][1]:t[t.length-1][1];const i=e[r],o=we(t[r],1)[0],a=we(t[r+1],1)[0];return i((n-o)/(a-o))}};function Me(){let t=null;function e(e){const n=Object(P.b)(e,"div.legend-container",()=>e.append("div").attr("class","legend-container legend-color").style("z-index","2")),r=n.node().getBoundingClientRect(),i=r.width,o=r.height,a=c.scaleBand().domain([0,1]).range([0,i]),A=t.copy().nice().domain(),l=s.extentLinear().pad([.1,.1]).padUnit("percent")(A),u=function(t,e){return function(t){if(Array.isArray(t))return t}(t)||function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),2!==n.length);r=!0);}catch(t){i=!0,o=t}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(t)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}(l),g=u[0],h=u[1],d=c.range(g,h,(h-g)/o),f=c.scaleLinear().domain(l).range([o,0]),p=s.autoBandwidth(s.seriesSvgBar()).xScale(a).yScale(f).crossValue(0).baseValue((t,e)=>e>0?d[e-1]:0).mainValue(t=>t).decorate(e=>{e.selectAll("path").style("fill",e=>t(e))}),I=A[0]<0&&A[1]>0?0:Math.round((A[1]+A[0])/2),C=[...A,I],m=s.axisRight(f).tickValues(C).tickSizeOuter(0).tickFormat(t=>y(t)),E=Object(P.b)(n,"svg",()=>n.append("svg")).style("width",i).style("height",o),v=Object(P.b)(E,"g",()=>E.append("g")).datum(d).call(p),B=Math.abs(v.node().getBBox().x);Object(P.b)(E,"#legend-axis",()=>E.append("g").attr("id","legend-axis")).attr("transform","translate(".concat(B,")")).datum(d).call(m).select(".domain").attr("visibility","hidden")}return e.scale=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],e):t},e}function ke(t,e){const n=function(t,e){return gt(t,e,{excludeEmpty:!0}).map(e=>(function(t,e){const n=ut(t),r=e.map((r,i)=>({crossValue:n(r,i),mainValues:t.mainValues.map(t=>r[t.name]),x:r[t.mainValues[0].name],y:r[t.mainValues[1].name],colorValue:t.mainValues.length>2?r[t.mainValues[2].name]:void 0,size:t.mainValues.length>3?r[t.mainValues[3].name]:void 0,key:e.key,row:r}));return r.key=e.key,r})(t,e))}(e,zt(e)),r=function(t){const e=t.data&&t.data.length>0?t.data[0]:{},n=[];return Object.keys(e).forEach(t=>{if("__ROW_PATH__"!==t){const e=it(t);n.includes(e)||n.push(e)}}),Be(n)}(e),i=e.mainValues.length<=2;let o=null,a=null;i?(o=function(t){const e=t.data&&t.data.length>0?t.data[0]:{},n=[];return Object.keys(e).forEach(t=>{if("__ROW_PATH__"!==t){const e=it(t);n.includes(e)||n.push(e)}}),at().settings(t).domain(n)()}(e),a=Ht().settings(e).scale(r).color(i?o:null)):(o=xe(e,n,"colorValue"),a=Me().scale(o));const A=e.mainValues.length>3?function(t,e,n,r){return c.scaleLinear().domain(_e(e,"size",void 0))}(0,n).range([10,1e4]):null,l=s.seriesCanvasMulti().mapping((t,e)=>t[e]).series(n.map(t=>(function(t,e,n,r,i){let o=Object(s.seriesCanvasPoint)().crossValue(t=>t.x).mainValue(t=>t.y);return n&&o.size(t=>n(t.size)),i&&o.type(i(e)),o.decorate((n,i)=>{const o=r(void 0!==i.colorValue?i.colorValue:e),a=t.colorStyles&&t.colorStyles.opacity;n.strokeStyle=st(o),n.fillStyle=At(o,a)}),o})(e,t.key,A,o,r))),u=()=>G(e).settingName("mainValues").paddingStrategy(Xt()).pad([.1,.1]),g=u().settingValue(e.mainValues[0].name).valueName("x")(n),h=u().orient("vertical").settingValue(e.mainValues[1].name).valueName("y")(n),d=j(g,h).xLabel(e.mainValues[0].name).yLabel(e.mainValues[1].name).plotArea(Wt(l,e).canvas(!0));d.xNice&&d.xNice(),d.yNice&&d.yNice();const f=ee().chart(d).settings(e).xScale(g.scale).yScale(h.scale).canvas(!0),p=Ie().settings(e).canvas(!0).xScale(g.scale).xValueName("x").yValueName("y").yScale(h.scale).color(i&&o).size(A).data(n);t.datum(n).call(f),t.call(p),a&&t.call(a)}ke.plugin={type:"d3_xy_scatter",name:"X/Y Scatter Chart",max_size:25e3,initial:{type:"number",count:2,names:["X Axis","Y Axis","Color","Size"]}};var Se=ke;function Le(t,e){const n=function(t,e){const n=ut(t),r=l(t).excludeType(A.linear).settingName("splitValues")(),i=[];return e.forEach((t,e)=>{const o=n(t,e);Object.keys(t).filter(t=>"__ROW_PATH__"!==t).forEach(e=>{const n=function(t){const e=t.split("|");return e.pop(),e.join("|")}(e);i.push({crossValue:o,mainValue:r===A.time?new Date(n):n,colorValue:t[e],row:t})})}),i}(e,Kt(e)),r=xe(e,n,"colorValue"),i=function(t,e){let n=s.seriesSvgHeatmap();return n.decorate(n=>{tt().settings(t)(n),n.select("path").attr("fill",t=>e(t.colorValue))}),s.autoBandwidth(n).xValue(t=>t.crossValue).yValue(t=>t.mainValue).colorValue(t=>t.colorValue).colorInterpolate(e.interpolator()).widthFraction(1)}(e,r),o=Me().scale(r),a=G(e).excludeType(A.linear).settingName("crossValues").valueName("crossValue")(n),c=G(e).excludeType(A.linear).settingName("splitValues").valueName("mainValue").orient("vertical")(n),u=H(a,c).plotArea(Wt(i,e));u.xPaddingInner&&(u.xPaddingInner(0),u.xPaddingOuter(0),i.xAlign("right")),u.yPaddingInner&&(u.yPaddingInner(0),u.yPaddingOuter(0),i.yAlign("top"));const g=ee().chart(u).settings(e).xScale(a.scale).yScale(c.scale);t.datum(n).call(g),t.call(o)}Le.plugin={type:"d3_heatmap",name:"Heatmap",max_size:25e3};var Re=Le,Ne=function(t){return function(e,n){const r=function(t,e){return gt(t,e,{excludeEmpty:!0}).map(e=>(function(t,e){const n=ut(t),r=e.map((r,i)=>{const o=t.mainValues.length>=1?r[t.mainValues[0].name]:void 0,a=t.mainValues.length>=2?r[t.mainValues[1].name]:(n=>e[n<e.length-1?n+1:n][t.mainValues[0].name])(i);return{crossValue:n(r,i),mainValues:t.mainValues.map(t=>r[t.name]),openValue:o,closeValue:a,highValue:t.mainValues.length>=3?r[t.mainValues[2].name]:Math.max(o,a),lowValue:t.mainValues.length>=4?r[t.mainValues[3].name]:Math.min(o,a),key:e.key,row:r}});return r.key=e.key,r})(t,e))}(n,zt(n)),i=s.indicatorBollingerBands().value(t=>t.openValue),o=r.map(t=>{const e=i(t);return t.map((t,n)=>Object.assign({bollinger:e[n]},t))}),a=r.map(t=>t.key).concat(n.hideKeys?n.hideKeys:[]).sort(),A=at().domain(a).settings(n).mapFunction(lt(1))(),l=Gt().settings(n).scale(a.length>1?A:null),c=function(t,e,n){const r=n.domain(),i=at().domain(r).settings(t).defaultColors([t.colorStyles["series-2"]]).mapFunction(lt(.5))(),o=at().settings(t).domain(r)(),a=e().crossValue(t=>t.crossValue).openValue(t=>t.openValue).highValue(t=>t.highValue).lowValue(t=>t.lowValue).closeValue(t=>t.closeValue).decorate((t,e)=>{const r=(t=>t.closeValue>=t.openValue)(e)?n(e.key):i(e.key);t.fillStyle=r,t.strokeStyle=r}),A=s.seriesCanvasLine().mainValue(t=>t.bollinger.average).crossValue(t=>t.crossValue).decorate((t,e)=>{t.strokeStyle=o(e[0].key)}),l=s.seriesCanvasArea().mainValue(t=>t.bollinger.upper).baseValue(t=>t.bollinger.lower).crossValue(t=>t.crossValue).decorate((t,e)=>{t.fillStyle=lt(.25)(o(e[0].key))});return s.seriesCanvasMulti().series([l,a,A])}(n,t,A),u=s.seriesCanvasMulti().mapping((t,e)=>t[e]).series(o.map(()=>c)),g=Xt().pad([.1,.1]).padUnit("percent"),h=G(n).settingName("crossValues").valueName("crossValue")(o),d=G(n).settingName("mainValues").valueNames(["lowValue","highValue"]).orient("vertical").paddingStrategy(g)(o),f=j(h,d).plotArea(Wt(u,n).orient("vertical").canvas(!0));f.yNice&&f.yNice();const p=ee().chart(f).settings(n).xScale(h.scale).onChange(t=>{const e=o.map(e=>e.filter(e=>e.crossValue>=t.xDomain[0]&&e.crossValue<=t.xDomain[1]));f.yDomain(d.domainFunction(e))}).canvas(!0),I=Ie().settings(n).xScale(h.scale).yScale(d.scale).yValueName("closeValue").color(A).data(o).canvas(!0);e.datum(o).call(p),e.call(I),e.call(l)}};const Fe=Ne(s.seriesCanvasOhlc);Fe.plugin={type:"d3_ohlc",name:"OHLC Chart",max_size:25e3,initial:{type:"number",count:4,names:["Open","Close","High","Low"]}};var Te=Fe;const Ue=Ne(s.seriesCanvasCandlestick);Ue.plugin={type:"d3_candlestick",name:"Candlestick Chart",max_size:25e3,initial:{type:"number",count:4,names:["Open","Close","High","Low"]}};var Oe=Ue;function Ge(t){const e={};return t.data.forEach(n=>{const r=n.__ROW_PATH__;(function(t){const e=[];return Object.keys(t).forEach(t=>{if("__ROW_PATH__"!==t){const n=t.split("|").slice(0,-1).join("|");e.includes(n)||e.push(n)}}),e})(n).forEach(i=>{let o;e[i]||(e[i]=[]),o=e[i],r.forEach((e,a)=>{let s=o.find(t=>t.name===e);if(s||(s={name:e,children:[]},o.push(s)),t.mainValues.length>1){const e=He(n,t.mainValues[1],i);s.color=s.color?s.color+e:e}if(a===r.length-1&&(s.name=r.slice(-1)[0],r.length===t.crossValues.length)){const e=He(n,t.mainValues[0],i);s.size=e>0?e:0}o=s.children})})}),Object.entries(e).map(e=>{const n={name:"root",children:e[1]},r=c.hierarchy(n).sum(t=>t.size),i=c.partition().size([2*Math.PI,r.height+1])(r);return i.each(n=>{n.current=n,n.mainValues=1===t.mainValues.length?n.value:[n.value,n.data.color],n.crossValue=n.ancestors().slice(0,-1).reverse().map(t=>t.data.name).join("|"),n.key=e[0]}),{split:e[0],data:i,extents:je(t,e)}})}const He=(t,e,n)=>n.length?t["".concat(n,"|").concat(e.name)]:t[e.name];function je(t,e){let n=function(t,e){return function(t){if(Array.isArray(t))return t}(t)||function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),2!==n.length);r=!0);}catch(t){i=!0,o=t}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(t)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}(e),r=n[0],i=n[1];if(t.mainValues.length>1)return[Math.min(...t.data.map(e=>He(e,t.mainValues[1],r))),Math.max(...i.map(t=>t.color))]}const Ye=t=>Object(c.arc)().startAngle(t=>t.x0).endAngle(t=>t.x1).padAngle(t=>Math.min((t.x1-t.x0)/2,.005)).padRadius(t).innerRadius(e=>e.y0*t).outerRadius(e=>Math.max(e.y0*t,e.y1*t-1)),Pe=t=>t.y1<=3&&t.y0>=1&&t.x1>t.x0;n(25);const Ke=t=>t.y1<=3&&t.y0>=1&&(t.y1-t.y0)*(t.x1-t.x0)>.06;function ze(t,e){const n=(t.x0+t.x1)/2*180/Math.PI,r=(t.y0+t.y1)/2*e;return"rotate(".concat(n-90,") translate(").concat(r,",0) rotate(").concat(n<180?0:180,")")}const Je=(t,e,n,r,i,o,a,s,A)=>(l,u)=>{A.sunburstLevel[s]=l.data.name,l.parent?(n.datum(l.parent),n.style("cursor","pointer"),r.html("&#8682; ".concat(l.data.name))):(n.datum(t),n.style("cursor","default"),r.html("")),t.each(t=>t.target={x0:2*Math.max(0,Math.min(1,(t.x0-l.x0)/(l.x1-l.x0)))*Math.PI,x1:2*Math.max(0,Math.min(1,(t.x1-l.x0)/(l.x1-l.x0)))*Math.PI,y0:Math.max(0,t.y0-l.depth),y1:Math.max(0,t.y1-l.depth)});const g=e.transition().duration(u?0:750);i.transition(g).tween("data",t=>{const e=Object(c.interpolate)(t.current,t.target);return n=>t.current=e(n)}).filter(function(t){return+this.getAttribute("fill-opacity")||Pe(t.target)}).attr("fill-opacity",t=>Pe(t.target)?1:0).attr("user-select",t=>Pe(t.target)?"initial":"none").attr("pointer-events",t=>Pe(t.target)?"initial":"none").attrTween("d",t=>()=>Ye(a)(t.current)),o.filter(function(t){return+this.getAttribute("fill-opacity")||Ke(t.target)}).transition(g).attr("fill-opacity",t=>+Ke(t.target)).attrTween("transform",t=>()=>ze(t.current,a))};function qe(){let t="element-prefix-unset",e=null,n=null,r=null,i=null,o=null;const a=a=>{const s=Object(P.b)(a,"div.inner-container",()=>a.append("div").attr("class","inner-container")),A=s.node().getBoundingClientRect(),l=A.height,c=A.width-0,u=a.datum(),g=Math.min(u.length,Math.floor(c/500)),h=Math.ceil(u.length/g);l/h>.75*(o={width:c/g,height:Math.min(l,Math.max(l/h,c/g))}).height&&(o.height=l/h),s.style("grid-template-columns","repeat(".concat(g,", ").concat(o.width,"px)")),s.style("grid-template-rows","repeat(".concat(h,", ").concat(o.height,"px)")),(r=s.selectAll("div.".concat(t,"-container")).data(u,t=>t.split)).exit().remove(),n=r.enter().append("div").attr("class","".concat(t,"-container")),i=n.append("div").attr("class","title-container").style("text-align","center").attr("display","inline-block").append("text").attr("class","title").style("text-align","left"),e=n.append("svg").append("g").attr("class",t)};return a.elementsPrefix=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],a):t},a.chartContainer=()=>e,a.chartEnter=()=>n,a.chartDiv=()=>r,a.chartTitle=()=>i,a.containerSize=()=>o,a}function Ve(t,e){if(0===e.crossValues.length)return void console.warn("Unable to render a chart in the absence of any groups.");const n=Ge(e),r=function(t,e){if(t.mainValues.length>1)return xe(t,null,null,u(e))}(e,n.map(t=>t.extents)),i=qe().elementsPrefix("sunburst");if(t.datum(n).call(i),r){const e=Me().scale(r);t.call(e)}const o=i.chartContainer(),a=i.chartEnter(),s=i.chartDiv(),A=i.chartTitle(),l=i.containerSize();A.each((t,e,n)=>Object(c.select)(n[e]).text(t.split)),o.append("circle").attr("fill","none").attr("pointer-events","all"),o.append("text").attr("class","parent"),a.merge(s).select("svg").select("g.sunburst").attr("transform","translate(".concat(l.width/2,", ").concat(l.height/2,")")).each(function(t){let n=t.split,i=t.data;const o=Object(c.select)(this),a=this.parentNode.getBoundingClientRect(),s=a.width,A=a.height,l=(Math.min(s,A)-120)/6;(function(){let t=null,e=null,n=null,r=null,i=null;const o=o=>{const a=o.selectAll("g.segment").data(n.descendants().slice(1)),s=a.enter().append("g").attr("class","segment");s.append("path"),s.append("text").attr("class","segment").attr("dy","0.35em");const A=s.merge(a),l=A.select("path").attr("fill-opacity",t=>Pe(t.current)?1:0).attr("user-select",t=>Pe(t.current)?"initial":"none").attr("pointer-events",t=>Pe(t.current)?"initial":"none").attr("d",t=>Ye(i)(t.current));r&&l.style("fill",t=>r(t.data.color));const u=A.select("text").attr("fill-opacity",t=>+Ke(t.current)).attr("transform",t=>ze(t.current,i)).text(t=>t.data.name).each(function(t){(function(t,e){let n=this.getBBox().width;if(n>e){let r=t.data.name;const i=Object(c.select)(this);for(;n>e;)r=r.substring(0,r.length-1),i.text(()=>r),n=this.getBBox().width;i.text(()=>"".concat(r.substring(0,r.length-3).replace(/\s+$/,""),"..."))}}).call(this,t,i)}),g=o.select("text.parent"),h=o.select("circle").attr("r",i).datum(n),d=Je(n,o,h,g,l,u,i,e,t);if(t.sunburstLevel){const r=n.descendants().find(n=>n.data.name===t.sunburstLevel[e]);r&&d(r,!0)}else t.sunburstLevel={};h.on("click",t=>d(t,!1)),l.filter(t=>t.children).style("cursor","pointer").on("click",t=>d(t,!1))};return o.settings=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],o):t},o.split=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],o):e},o.data=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],o):n},o.color=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],o):r},o.radius=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],o):i},o})().settings(e).split(n).data(i).color(r).radius(l)(o),tt().settings(e)(o.selectAll("g.segment"))})}Ve.plugin={type:"d3_sunburst",name:"Sunburst",max_size:25e3,initial:{type:"number",count:2,names:["Size","Color"]}};var We=Ve;const Xe=t=>!(!t.target||!t.target.textAttributes),Ze=(t,e,n)=>{t.selectAll("text").style("font-size",null).attr("class",t=>un(t,e,n));const r=on(t);sn(r),tn(r)},$e=t=>{t.each((t,e,n)=>{Object(c.select)(n[e]).selectAll("text").attr("dx",t.target.textAttributes.dx).attr("dy",t.target.textAttributes.dy).attr("class",t.target.textAttributes.class).style("font-size",t.target.textAttributes["font-size"])})},tn=t=>{const e=t=>t.getBoundingClientRect(),n=[];t.selectAll("text").filter((t,e,n)=>Object(c.select)(n[e]).attr("class")===gn.high).each((t,e,r)=>n.push(r[e])),t.selectAll("text").filter((t,e,n)=>Object(c.select)(n[e]).attr("class")===gn.low).each((t,r,i)=>{const o=i[r];n.filter(t=>Object(P.d)("x",e(t),e(o))&&Object(P.d)("y",e(t),e(o),-2)).forEach(()=>Object(c.select)(o).attr("dy",Number(Object(c.select)(o).attr("dy"))+19))})},en=t=>Object(c.select)(t).style("opacity",rn[Object(c.select)(t).attr("class")]),nn=t=>Object(c.select)(t).style("opacity",null),rn={top:1,mid:.7,lower:0},on=t=>t.filter((t,e,n)=>Object(c.select)(n[e]).selectAll("text").attr("class")!==gn.zero),an=t=>t.selectAll("text").each((t,e,n)=>ln(n[e])),sn=t=>t.selectAll("text").each((t,e,n)=>An(n[e])),An=t=>{const e=Object(c.select)(t),n=t.getBoundingClientRect();e.attr("dx",0-n.width/2).attr("dy",0+n.height/4)},ln=t=>{const e=t.parentNode.childNodes[0],n=t.getBoundingClientRect(),r=e.getBoundingClientRect();cn(t,r,n,"left")||cn(t,r,n,"bottom")||Object(c.select)(t).attr("class",Object(c.select)(t).attr("class"))},cn=(t,e,n,r)=>{if(Object(P.c)(e,n,r)){const e=parseInt(Object(c.select)(t).style("font-size"));return e>7?(Object(c.select)(t).style("font-size","".concat(e-1,"px")),An(t),ln(t)):(Object(c.select)(t).style("font-size",null),Object(c.select)(t).attr("class",gn.zero)),!0}return!1},un=(t,e,n)=>{if(!n.filter(t=>""!==t).every(e=>t.crossValue.split("|").includes(e)))return gn.zero;switch(t.depth){case e+1:return gn.high;case e+2:return gn.low;default:return gn.zero}},gn={high:"top",low:"mid",zero:"lower"};var hn=(t,e)=>{const n=c.treemap().size([t,e]).paddingInner(t=>1+2*t.height);return n.tile(c.treemapBinary),n};n(13);const dn=(t,e)=>e.every(e=>t.crossValue.split("|").includes(e));function fn(t,e,n,r,i,o){!function(t,e,n,r,i){const o=t.x0,a=t.y0,s=t.x1-t.x0,A=t.y1-t.y0,l=i.x1-i.x0,c=i.y1-i.y0,u=l/s,g=c/A;n.each(t=>{const n=(t.x0-o)*u,i=(t.y0-a)*g,s=wn(t)*u,A=xn(t)*g,l=dn(t,e)&&t.data.name!=e[r-1];t.mapLevel[r]={x0:n,x1:s+n,y0:i,y1:A+i,visible:l,opacity:l?1:0}}),t.mapLevel[r].levelRoot=!0}(t,e,n,r,i),t.mapLevel[r].levelRoot=!0,function(t,e,n,r){const i=hn(e,n)(Object(c.hierarchy)(t.data).sum(t=>t.size)).descendants(),o=t.descendants();i.forEach((t,e)=>{o[e].mapLevel[r].x0=i[e].x0,o[e].mapLevel[r].x1=i[e].x1,o[e].mapLevel[r].y0=i[e].y0,o[e].mapLevel[r].y1=i[e].y1})}(t,o.node().getBoundingClientRect().width,o.node().getBoundingClientRect().height,r),In(n,r)}const pn=(t,e)=>{t.each((t,n,r)=>{const i=Object(c.select)(r[n]).selectAll("text");t.mapLevel[e].textAttributes={dx:i.attr("dx"),dy:i.attr("dy"),class:i.attr("class"),"font-size":i.style("font-size")}})};function In(t,e){t.selectAll("text").each((t,n,r)=>{const i=Object(c.select)(r[n]),o=Object(c.select)(r[n]).datum(),a=i.attr("class");o.mapLevel[e].textLockedAt={opacity:rn[a]}})}function Cn(t,e,n,r,i,o,a,s,A){if(!t.children)return;i.treemapLevel<t.depth?i.treemapRoute.push(t.crossValue):i.treemapRoute.pop(),i.treemapLevel=t.depth;const l=t.crossValue.split("|");t.mapLevel[i.treemapLevel]&&t.mapLevel[i.treemapLevel].levelRoot||fn(t,l,n,i.treemapLevel,s,o),mn(t,e,n,r,i,o,a,s,i.treemapLevel,l,A)}function mn(t,e,n,r,i,o,a,s,A,l,u){let g=arguments.length>11&&void 0!==arguments[11]?arguments[11]:500,h=!(arguments.length>12&&void 0!==arguments[12])||arguments[12];const d=t.parent,f=a.transition("main transition").duration(g).ease(c.easeCubicOut);n.each(t=>t.target=t.mapLevel[A]),Xe(t)||vn(n,u),n.transition(f).tween("data",t=>{const e=c.interpolate(t.current,t.target);return n=>t.current=e(n)}).styleTween("opacity",t=>()=>t.current.opacity).attrTween("pointer-events",t=>()=>t.target.visible?"all":"none"),e.transition(f).filter(t=>t.target.visible).styleTween("x",t=>()=>"".concat(t.current.x0,"px")).styleTween("y",t=>()=>"".concat(t.current.y0,"px")).styleTween("width",t=>()=>"".concat(t.current.x1-t.current.x0,"px")).styleTween("height",t=>()=>"".concat(t.current.y1-t.current.y0,"px")),r.transition(f).filter(t=>t.target.visible).attrTween("x",t=>()=>t.current.x0+wn(t.current)/2).attrTween("y",t=>()=>t.current.y0+xn(t.current)/2).end().catch(()=>Bn(n)).then(()=>{Xe(t)||(tn(p),an(p),function(t,e){var n,r,i=arguments;regeneratorRuntime.async(function(o){for(;;)switch(o.prev=o.next){case 0:return n=i.length>2&&void 0!==i[2]?i[2]:400,r=e.transition("text fade transition").duration(n).ease(c.easeCubicOut),o.next=4,regeneratorRuntime.awrap(t.transition(r).filter(t=>t.target.visible).tween("data",(t,e,n)=>{const r=n[e],i=c.interpolate(yn(t),En(r));return e=>t.current.opacity=i(e)}).styleTween("opacity",t=>()=>t.current.opacity).end().catch(t=>console.error("Exception in text fade transition",t)).then(()=>t.each((t,e,n)=>nn(n[e]))));case 4:case"end":return o.stop()}})}(r,a,g),h&&pn(n,A),Bn(n,u,d))}).catch(t=>{console.error("Exception completing promises after main transition",t),Bn(n,u,d)}),Xe(t)?$e(n):(r.each((t,e,n)=>en(n[e])),Ze(n,A,l));const p=on(n);d?u.hide(!1).text(t.data.name).onClick(()=>Cn(d,e,n,r,i,o,a,s,u))():u.hide(!0)()}const yn=t=>t.target.textLockedAt.opacity,En=t=>rn[c.select(t).attr("class")],vn=(t,e)=>{e.deactivate(!0),t.each((t,e,n)=>{c.select(n[e]).selectAll("rect").attr("pointer-events","none")})},Bn=(t,e)=>{e&&e.deactivate(!1),t.each((t,e,n)=>{c.select(n[e]).selectAll("rect").attr("pointer-events",null)})};var bn=n(18),Qn=n.n(bn);const wn=t=>t.x1-t.x0,xn=t=>t.y1-t.y0,_n=(t,e)=>0===e.depth?"rootnode":((t,e)=>e.depth===t)(t,e)?"leafnode":"branchnode";function Dn(t,e){if(0===e.crossValues.length)return void console.warn("Unable to render a chart in the absence of any groups.");e.treemaps||(e.treemaps={});const n=Ge(e),r=function(t,e){if(t.mainValues.length<=1)return;const n=e.filter(t=>t.height>0).map(t=>(function t(e){let n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:[];return e.children&&e.children.length>0?e.children.forEach(e=>n.concat(t(e,n))):n.push(e.data.color),n})(t)).reduce((t,e)=>t.concat(e));return xe(t,null,null,[Math.min(...n),Math.max(...n)])}(e,n.map(t=>t.data)),i=qe().elementsPrefix("treemap");if(t.datum(n).call(i),r){const e=Me().scale(r);t.call(e)}const o=i.chartEnter(),a=i.chartDiv();i.chartTitle().each((t,e,n)=>c.select(n[e]).text(t.split)),o.merge(a).select("svg").select("g.treemap").each(function(t){let n=t.split,i=t.data;const o=c.select(this);e.treemaps[n]||(e.treemaps[n]={}),function(){let t=null,e=null,n=null,r=null,i=null;const o=o=>{(i=function(t){let e=null,n=null,r=!0,i=!1;const o=Object(P.b)(t,".parent-controls",()=>t.append("div").attr("class","parent-controls").style("display",r?"none":"").html(Qn.a)),a=()=>{o.style("display",r?"none":"").select("#goto-parent").style("pointer-events",i?"none":null).html("⇪ ".concat(n)).on("click",()=>e())};return a.deactivate=function(){if(!arguments.length)return i;i=arguments.length<=0?void 0:arguments[0];const t=o.select("#goto-parent");return i?t.style("pointer-events","none"):t.style("pointer-events",null),a},a.hide=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],a):r},a.text=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],a):n},a.onClick=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],a):e},a}(r))();const a=e.height;t.treemapLevel||(t.treemapLevel=0),t.treemapRoute||(t.treemapRoute=[]),hn(r.node().getBoundingClientRect().width,r.node().getBoundingClientRect().height)(e);const s=o.selectAll("g").data(e.descendants()),A=s.enter().append("g");A.append("rect"),A.append("text");const l=A.merge(s).sort((t,e)=>e.depth-t.depth),c=l.select("rect").attr("class",t=>"treerect ".concat(_n(a,t))).style("x",t=>t.x0).style("y",t=>t.y0).style("width",t=>wn(t)).style("height",t=>xn(t));n&&c.style("fill",t=>n(t.data.color));const u=l.filter(t=>0!==t.value).select("text").attr("x",t=>t.x0+wn(t)/2).attr("y",t=>t.y0+xn(t)/2).text(t=>t.data.name),g=c.filter(t=>""===t.crossValue).datum();!function(t,e){t.each(t=>{t.mapLevel=[],t.mapLevel[0]={x0:t.x0,x1:wn(t)+t.x0,y0:t.y0,y1:xn(t)+t.y0,visible:!0,opacity:1}}),e.mapLevel[0].levelRoot=!0,In(t,0)}(l,g),Ze(l,0,[]),an(on(l)),pn(l,0),0===t.treemapRoute.length&&t.treemapRoute.push(g.crossValue),c.filter(t=>t.children).on("click",e=>Cn(e,c,l,u,t,r,o,g,i)),function(t,e,n,r,i,o,a,s){if(r.treemapLevel>0){const A=a.crossValue.split("|");mn(a,t,e,n,r,i,o,a,0,A,s,1,!1),r.treemapRoute.slice(1,r.treemapRoute.length).forEach(A=>{const l=e.filter(t=>t.crossValue===A).datum(),c=l.crossValue.split("|");fn(l,c,e,l.depth,a,i),mn(l,t,e,n,r,i,o,a,l.depth,c,s,1,!1)})}}(c,l,u,t,r,o,g,i)};return o.settings=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],o):t},o.data=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],o):e},o.color=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],o):n},o.container=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],o):r},o}().settings(e.treemaps[n]).data(i).container(c.select(c.select(this.parentNode).node().parentNode)).color(r)(o),tt().settings(e).centered(!0)(o.selectAll("g"))})}Dn.plugin={type:"d3_treemap",name:"Treemap",max_size:25e3,initial:{type:"number",count:2}};const Mn=[re,de,me,Ee,Qe,Se,Re,Te,Oe,We,Dn];e.a=Mn},function(t,e){t.exports=d},function(t,e,n){"use strict";(function(t){n.d(e,"a",function(){return l}),n(13),n(3),n(10),n(6),n(5);var r=n(12);function i(t,e){return function(t){if(Array.isArray(t))return t}(t)||function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!e||n.length!==e);r=!0);}catch(t){i=!0,o=t}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(t,e)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}function o(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter(function(t){return Object.getOwnPropertyDescriptor(n,t).enumerable}))),r.forEach(function(e){a(t,e,n[e])})}return t}function a(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t}n(26),n(30);const s=Symbol("D3FC chart"),A={initial:{type:"number",count:1},selectMode:"select"};function l(){for(var e=arguments.length,n=new Array(e),i=0;i<e;i++)n[i]=arguments[i];n=new Set(n.length>0?n:r.a.map(t=>t.plugin.type)),r.a.forEach(e=>{n.has(e.plugin.type)&&t.registerPlugin(e.plugin.type,o({},A,e.plugin,{create:c(e),resize:h,delete:d,save:f,restore:p}))})}function c(t){return function(e,n,r){var a,s,A,l,c,u,h,d,f,p,I,C,m;return regeneratorRuntime.async(function(y){for(;;)switch(y.prev=y.next){case 0:return y.next=2,regeneratorRuntime.awrap(Promise.all([this._table.schema(),n.schema(),n.to_json(),n.get_config()]));case 2:if(a=y.sent,s=i(a,4),A=s[0],l=s[1],c=s[2],u=s[3],!r.cancelled){y.next=10;break}return y.abrupt("return");case 10:h=u.columns,d=u.row_pivots,f=u.column_pivots,p=u.filter,I=d.length>0?c.filter(t=>t.__ROW_PATH__&&t.__ROW_PATH__.length==d.length):c,C=(t,e)=>d.length?t:o({},t,{__ROW_PATH__:[e]}),m={crossValues:d.map(t=>({name:t,type:A[t]})),mainValues:h.map(t=>({name:t,type:l[t]})),splitValues:f.map(t=>({name:t,type:A[t]})),filter:p,data:I.map(C)},g.call(this,e,t,m);case 15:case"end":return y.stop()}},null,this)}}function u(){return this[s]=this[s]||{},this[s].chart?this[s].chart:this[s].chart=document.createElement("perspective-d3fc-chart")}function g(t,e,n){const r=u.call(this);document.body.contains(r)||(t.innerHTML="",t.appendChild(r)),r.render(e,n)}function h(){this[s]&&this[s].chart&&this[s].chart.resize()}function d(){this[s]&&this[s].chart&&this[s].chart.delete()}function f(){if(this[s]&&this[s].chart)return this[s].chart.getSettings()}function p(t){u.call(this).setSettings(t)}Element.prototype.matches||(Element.prototype.matches=Element.prototype.msMatchesSelector)}).call(this,n(24))},function(t,e){t.exports='<ul id="tooltip-values"></ul>'},function(t,e){t.exports='<span id="up-arrow">&#9650;</span>\n<span id="page-text"></span>\n<span id="down-arrow">&#9660;</span>'},function(t,e){t.exports='<button id="zoom-reset">Reset zoom</button>\n<button id="one-year">1 Year</button>\n<button id="six-months">6 Months</button>\n<button id="one-month">1 Month</button>'},function(t,e){t.exports='<button id="goto-parent">Goto parent</button>'},function(t,e,n){(t.exports=n(29)(!1)).push([t.i,':host .chart{position:absolute;box-sizing:border-box;left:0;top:0;width:100%;height:100%;padding:15px;overflow:hidden}:host .chart.d3_heatmap d3fc-group.cartesian-chart,:host .chart.d3_treemap d3fc-group.cartesian-chart{grid-template-columns:minmax(0,max-content) auto 1fr auto minmax(0,max-content);grid-template-rows:minmax(0,max-content) auto 1fr auto minmax(0,max-content)}:host .chart .chart-label{display:none}:host .chart.d3_heatmap{padding-right:120px}:host .chart.d3_sunburst{padding:0}:host .chart.d3_sunburst .inner-container{width:100%;height:100%;display:inline-grid;padding:0;margin:0;overflow-x:hidden;overflow-y:auto}:host .chart.d3_sunburst .inner-container div{overflow:hidden}:host .chart.d3_sunburst .inner-container svg{width:100%;height:100%;transform:translate(0,-27px)}:host .chart.d3_sunburst .inner-container path{fill:var(--d3fc-series,rgba(31,119,180,.5))}:host .chart.d3_sunburst .inner-container text.parent,:host .chart.d3_sunburst .inner-container text.segment{text-anchor:middle;user-select:none;pointer-events:none;fill:var(--d3fc-treedata--labels,#333)}:host .chart.d3_sunburst .inner-container div.title-container{margin-top:5px}:host .chart.d3_treemap{padding-right:120px}:host .chart.d3_treemap .treemap-container{position:relative}:host .chart.d3_treemap .inner-container{width:100%;height:100%;display:inline-grid;padding:0;margin:0;overflow-x:hidden;overflow-y:auto}:host .chart.d3_treemap .inner-container div{overflow:hidden}:host .chart.d3_treemap .inner-container svg{width:100%;height:100%}:host .chart.d3_treemap .inner-container .treerect{stroke:var(--d3fc-treedata-axis--lines,var(--d3fc-axis--lines,#fff));fill:var(--d3fc-series,rgba(31,119,180,.5))}:host .chart.d3_treemap .inner-container .treerect:hover{cursor:pointer;stroke:var(--d3fc-treedata--hover-highlight,#000);stroke-opacity:1}:host .chart.d3_treemap .inner-container .rootnode{opacity:0;pointer-events:none;z-index:0}:host .chart.d3_treemap .inner-container .branchnode{opacity:0}:host .chart.d3_treemap .inner-container .branchnode:hover{fill-opacity:.1;opacity:1;fill:var(--d3fc-treedata--hover-highlight,#000)}:host .chart.d3_treemap .inner-container .leafnode{fill-opacity:.8}:host .chart.d3_treemap .inner-container .leafnode:hover{fill-opacity:.5}:host .chart.d3_treemap .inner-container #hidden{opacity:0;pointer-events:none;z-index:0}:host .chart.d3_treemap .inner-container .top{font-size:19px;font-weight:700;z-index:5;pointer-events:none}:host .chart.d3_treemap .inner-container .mid{font-size:10px;font-weight:700;opacity:.7;z-index:4}:host .chart.d3_treemap .inner-container .lower{font-size:0;font-weight:700;opacity:0;z-index:4}:host .chart.d3_treemap .inner-container text{fill:var(--d3fc-treedata--labels,#333)}:host .chart.d3_treemap .inner-container text.title{dominant-baseline:hanging}:host .chart.d3_treemap .inner-container .parent-controls{position:absolute;top:15px;right:auto;left:30px;width:auto;z-index:4;transition:box-shadow 1s}:host .chart.d3_treemap .inner-container .parent-controls:hover{box-shadow:2px 2px 6px #000;transition:box-shadow .2s}:host .chart .x-label{height:1.2em!important}:host .chart d3fc-canvas.plot-area{display:none}:host .chart.d3_candlestick d3fc-canvas.plot-area,:host .chart.d3_ohlc d3fc-canvas.plot-area,:host .chart.d3_xy_scatter d3fc-canvas.plot-area{display:block}:host .chart .x-axis text,:host .chart .y-axis text{fill:var(--d3fc-axis--ticks,#505050);font-size:11px}:host .chart .x-axis path,:host .chart .y-axis path{stroke:var(--d3fc-axis--lines,#b4d2e1)}:host .chart .splitter-label{display:inline-block;margin:0 10px;cursor:pointer}:host .chart .splitter-label::after{margin-left:5px;color:var(--d3fc-axis--lines,#b4d2e1)}:host .chart .splitter-label:hover::after{color:var(--d3fc-axis--ticks,#505050)}:host .chart .splitter-label.disabled{cursor:default}:host .chart .splitter-label.disabled::after{color:rgba(80,80,80,.3)}:host .chart .y-label-container .splitter-label::after{content:"\\1F847"}:host .chart .y2-label-container .splitter-label::after{content:"\\1F845"}:host .chart.d3_x_bar .x-axis path,:host .chart.d3_xy_scatter .x-axis path,:host .chart.d3_xy_scatter .y-axis path,:host .chart.d3_y_area .y-axis path,:host .chart.d3_y_bar .y-axis path,:host .chart.d3_y_line .y-axis path,:host .chart.d3_y_scatter .y-axis path{visibility:hidden}:host .chart .y-label{white-space:nowrap}:host .chart .nearbyTip{fill:var(--d3fc-series,rgba(31,119,180,.5))}:host .chart .series{fill:var(--d3fc-series,rgba(31,119,180,.5))}:host .chart .series-1{fill:var(--d3fc-series-1,#0366d6)}:host .chart .series-2{fill:var(--d3fc-series-2,#ff7f0e)}:host .chart .series-3{fill:var(--d3fc-series-3,#2ca02c)}:host .chart .series-4{fill:var(--d3fc-series-4,#d62728)}:host .chart .series-5{fill:var(--d3fc-series-5,#9467bd)}:host .chart .series-6{fill:var(--d3fc-series-6,#8c564b)}:host .chart .series-7{fill:var(--d3fc-series-7,#e377c2)}:host .chart .series-8{fill:var(--d3fc-series-8,#7f7f7f)}:host .chart .series-9{fill:var(--d3fc-series-9,#bcbd22)}:host .chart .series-10{fill:var(--d3fc-series-10,#17becf)}:host .disabled{filter:grayscale(100%)}:host #dragHandles{position:absolute;top:0;right:0;left:auto;pointer-events:none}:host #dragHandles rect{pointer-events:all}:host .label rect{fill:none}:host .legend-container{position:absolute;top:15px;right:15px;left:auto;width:150px;height:280px}:host .legend-container.legend-color{width:90px;height:150px}:host .legend-container[borderbox-on-hover=true]{transition:box-shadow 1s}:host .legend-container[borderbox-on-hover=true]:hover{box-shadow:2px 2px 6px #000;transition:box-shadow .2s}:host .legend-container .legend{font-size:8pt;width:100%}:host .legend-container .legend .cell{cursor:pointer}:host .legend-container .legend .cell path{fill:var(--d3fc-series,rgba(31,119,180,.5));stroke:var(--d3fc-series,#1f77b4)}:host .legend-container .legend .cell.hidden path{fill:rgba(204,204,204,.5);stroke:#ccc}:host .legend-container .legend .cell.hidden circle{fill:#ccc!important}:host .legend-container text{fill:var(--d3fc-legend--text,#333)}:host .legend-container .label:hover{fill:#22a0ce}:host .legend-container #legend-axis path{stroke:var(--d3fc-legend--text,#333)}:host .legend-container .legend-controls{margin-left:12px}:host .legend-container .legend-controls #down-arrow,:host .legend-container .legend-controls #up-arrow{cursor:pointer;color:#1f77b4}:host .legend-container .legend-controls #down-arrow.disabled,:host .legend-container .legend-controls #up-arrow.disabled{cursor:default;color:#333}:host .d3_heatmap .legend-container.legend-color{height:200px}:host .d3_treemap .legend-container.legend-color{height:200px}:host .zoom-controls{position:absolute;top:10px;left:0;width:100%;text-align:center}:host .zoom-controls button{-webkit-appearance:none;background:#f7f7f7;border:1px solid #ccc;padding:10px;opacity:.5;cursor:pointer}:host .zoom-controls button:hover{background:#e6e6e6}:host .parent-controls{position:absolute;top:30px;right:145px;width:100%;text-align:right}:host .parent-controls button{-webkit-appearance:none;background:#f7f7f7;border:1px solid #ccc;padding:10px;opacity:.5;cursor:pointer}:host .parent-controls button:hover{background:#e6e6e6}:host div.tooltip{position:absolute;text-align:left;font-size:10pt;white-space:nowrap;padding:.5em;background:var(--d3fc-tooltip--background,#fff);color:var(--d3fc-tooltip--color,#000);border:1px solid #777;border-radius:2px;pointer-events:none}:host div.tooltip ul{margin:0;padding:0;list-style-type:none}',""])},function(t,e){t.exports='<template id="perspective-d3fc-chart">\r\n    <div id="container" class="chart">\r\n    </div>\r\n</template>'},function(t,e){t.exports=f},function(t,e){t.exports=p},function(t,e,n){"use strict";n.r(e);var r=n(14);Object(r.a)()},function(t,e){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(t){"object"==typeof window&&(n=window)}t.exports=n},function(t,e){t.exports=I},function(t,e,n){"use strict";n(27),n(28)},function(t,e){Element.prototype.matches||(Element.prototype.matches=Element.prototype.msMatchesSelector||Element.prototype.webkitMatchesSelector)},function(t,e){Element.prototype.matches||(Element.prototype.matches=Element.prototype.msMatchesSelector||Element.prototype.webkitMatchesSelector),Element.prototype.closest||(Element.prototype.closest=function(t){var e=this;do{if(e.matches(t))return e;e=e.parentElement||e.parentNode}while(null!==e&&1===e.nodeType);return null})},function(t,e){t.exports=function(t){var e=[];return e.toString=function(){return this.map(function(e){var n=function(t,e){var n,r=t[1]||"",i=t[3];if(!i)return r;if(e&&"function"==typeof btoa){var o=(n=i,"/*# sourceMappingURL=data:application/json;charset=utf-8;base64,"+btoa(unescape(encodeURIComponent(JSON.stringify(n))))+" */"),a=i.sources.map(function(t){return"/*# sourceURL="+i.sourceRoot+t+" */"});return[r].concat(a).concat([o]).join("\n")}return[r].join("\n")}(e,t);return e[2]?"@media "+e[2]+"{"+n+"}":n}).join("")},e.i=function(t,n){"string"==typeof t&&(t=[[null,t,""]]);for(var r={},i=0;i<this.length;i++){var o=this[i][0];"number"==typeof o&&(r[o]=!0)}for(i=0;i<t.length;i++){var a=t[i];"number"==typeof a[0]&&r[a[0]]||(n&&!a[2]?a[2]=n:n&&(a[2]="("+a[2]+") and ("+n+")"),e.push(a))}},e}},function(t,e,n){"use strict";n(5),n(3);var r=n(0),i=n(19),o=n.n(i),a=n(20),s=n.n(a),A=n(2),l=(n(8),n(21));function c(){const t=function(t,e){return e||(e=t.slice(0)),Object.freeze(Object.defineProperties(t,{raw:{value:Object.freeze(e)}}))}(["--d3fc-gridline--color"]);return c=function(){return t},t}const u=t=>r.color(t).opacity,g=(t,e)=>{const n=r.color("#".concat(t));return n.opacity=e,n+""};var h=n(22);function d(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter(function(t){return Object.getOwnPropertyDescriptor(n,t).enumerable}))),r.forEach(function(e){f(t,e,n[e])})}return t}function f(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t}function p(){return Reflect.construct(HTMLElement,[],this.__proto__.constructor)}Object.setPrototypeOf(p.prototype,HTMLElement.prototype),Object.setPrototypeOf(p,HTMLElement);const I="".concat(o.a).concat(function(){const t=document.querySelector("head").querySelectorAll("style"),e=[];return t.forEach(t=>{-1!==t.innerText.indexOf("d3fc-")&&e.push(t.innerText)}),e.join("")}());Object(h.bindTemplate)(s.a,I)(class extends p{constructor(){super(),this._chart=null,this._settings=null}connectedCallback(){console.log("connected callback"),this._container=this.shadowRoot.querySelector(".chart")}render(t,e){this._chart=t,this._settings=this._configureSettings(this._settings,e),((t,e)=>{if(!e.colorStyles){const n=["series"];for(let t=1;t<=10;t++)n.push("series-".concat(t));const r={scheme:[],gradient:{},interpolator:{},grid:{}},i=(t=>{if(window.ShadyCSS)return e=>window.ShadyCSS.getComputedStyleValue(t,e);{const e=getComputedStyle(t);return t=>e.getPropertyValue(t)}})(t);n.forEach((t,e)=>{r[t]=i("--d3fc-".concat(t)),e>0&&r.scheme.push(r[t])}),r.opacity=u(r.series),r.grid.gridLineColor=i(c()),["full","positive","negative"].forEach(t=>{const e=i("--d3fc-gradient-".concat(t));r.gradient[t]=((t,e)=>{const n=l.parse(t)[0].colorStops;return n.map((t,r)=>[t.length?t.length.value/100:r/(n.length-1),g(t.value,e)]).sort((t,e)=>t[0]-e[0])})(e,r.opacity)}),e.colorStyles=r}})(this._container,this._settings),(this._settings.data&&this._settings.data.length>0||t.plugin.type!==this._chart.plugin.type)&&this.remove(),this.draw(),window.navigator.userAgent.indexOf("Edge")>-1&&setTimeout(()=>this.draw())}draw(){if(this._settings.data){const t=r.select(this._container),e="chart ".concat(this._chart.plugin.type);this._settings.size=this._container.getBoundingClientRect(),this._settings.data.length>0?this._chart(t.attr("class",e),this._settings):t.attr("class","".concat(e," disabled"))}}resize(){this.draw()}remove(){this._container.innerHTML=""}delete(){this.remove()}getContainer(){return this._container}getSettings(){const t=d({},this._settings);return["crossValues","mainValues","splitValues","filter","data","size","colorStyles"].forEach(e=>{delete t[e]}),t}setSettings(t){this._settings=d({},this._settings,t),this.draw()}_configureSettings(t,e){if(t){if(!t.data)return d({},t,e);const n=[t.crossValues,t.mainValues,t.splitValues],r=[e.crossValues,e.mainValues,e.splitValues];if(Object(A.a)(n,r))return d({},t,{data:e.data,colorStyles:null})}return this.remove(),e}})}]))},function(t,e,n){(function(t){!function(e,n,r,i,o,a,s,A,l,c,u){"use strict";var g=function(t,e,n){var r=e[n];if("function"!=typeof r)throw new Error("Attempt to rebind "+n+" which isn't a function on the source object");return function(){for(var n=arguments.length,i=Array(n),o=0;o<n;o++)i[o]=arguments[o];var a=r.apply(e,i);return a===e?t:a}},h=function(t,e){for(var n=arguments.length,r=Array(n>2?n-2:0),i=2;i<n;i++)r[i-2]=arguments[i];var o=!0,a=!1,s=void 0;try{for(var A,l=r[Symbol.iterator]();!(o=(A=l.next()).done);o=!0){var c=A.value;t[c]=g(t,e,c)}}catch(t){a=!0,s=t}finally{try{!o&&l.return&&l.return()}finally{if(a)throw s}}return t},d=function(t){return function(e){return t.reduce(function(t,e){return t&&e(t)},e)}},f=function(t,e){for(var n=arguments.length,r=Array(n>2?n-2:0),i=2;i<n;i++)r[i-2]=arguments[i];var o=d(r),a=!0,s=!1,A=void 0;try{for(var l,c=Object.keys(e)[Symbol.iterator]();!(a=(l=c.next()).done);a=!0){var u=l.value,h=o(u);h&&(t[h]=g(t,e,u))}}catch(t){s=!0,A=t}finally{try{!a&&c.return&&c.return()}finally{if(s)throw A}}return t},p=function(t){return t.map(function(t){return"string"==typeof t?new RegExp("^"+t+"$"):t})},I=function(){for(var t=arguments.length,e=Array(t),n=0;n<t;n++)e[n]=arguments[n];return e=p(e),function(t){return e.every(function(e){return!e.test(t)})&&t}},C=function(){for(var t=arguments.length,e=Array(t),n=0;n<t;n++)e[n]=arguments[n];return e=p(e),function(t){return e.some(function(e){return e.test(t)})&&t}},m=function(t){return function(e){return t[e]}},y=function(t){return function(e){return t+((n=e)[0].toUpperCase()+n.slice(1));var n}};function E(t){return t}function v(t){}function B(t){return"function"==typeof t?t:function(){return t}}function b(t){return"number"==typeof t&&isNaN(t)?void 0:t}var Q=function(){var t=function(){return 10},e=v,n=E,r=function(t){return null!=t},i=function(i){var o=t.apply(this,arguments),a=i.slice(0,o).map(n);return i.map(function(t,i){return i>=o&&(a.shift(),a.push(n(t,i))),i<o-1||a.some(function(t){return!r(t)})?e(void 0,i):e(a,i)})};return i.period=function(){return arguments.length?(t=B(arguments.length<=0?void 0:arguments[0]),i):t},i.accumulator=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],i):e},i.defined=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],i):r},i.value=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},i},w=function(){var t=E,e=function(){return 9},r=function(t){var e=[];return function(r){var i=void 0;return e.length<t&&(null!=r?e.push(r):e=[]),e.length>=t&&(i=n.mean(e)),i}},i=function(n){var i=e.apply(this,arguments),o=2/(i+1),a=r(i),s=void 0;return n.map(function(e,n){var r=t(e,n);return b(s=void 0===s?a(r):r*o+(1-o)*s)})};return i.period=function(){return arguments.length?(e=B(arguments.length<=0?void 0:arguments[0]),i):e},i.value=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],i):t},i},x="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},_=function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")},D=function(){function t(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}return function(e,n,r){return n&&t(e.prototype,n),r&&t(e,r),e}}(),M=function(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function, not "+typeof e);t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,enumerable:!1,writable:!0,configurable:!0}}),e&&(Object.setPrototypeOf?Object.setPrototypeOf(t,e):t.__proto__=e)},k=function(t,e){if(!t)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return!e||"object"!=typeof e&&"function"!=typeof e?t:e},S=function(t,e){if(Array.isArray(t))return t;if(Symbol.iterator in Object(t))return function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!e||n.length!==e);r=!0);}catch(t){i=!0,o=t}finally{try{!r&&s.return&&s.return()}finally{if(i)throw o}}return n}(t,e);throw new TypeError("Invalid attempt to destructure non-iterable instance")},L=function(t){if(Array.isArray(t)){for(var e=0,n=Array(t.length);e<t.length;e++)n[e]=t[e];return n}return Array.from(t)},R=function(){var t=Q().accumulator(function(t){return t&&n.mean(t)}),e=function(e){return t(e)};return h(e,t,"period","value"),e},N=function(){var t={distance:function(t,e){return e-t},offset:function(t,e){return t instanceof Date?new Date(t.getTime()+e):t+e},clampUp:function(t){return t},clampDown:function(t){return t},copy:function(){return t}};return t};function F(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done);r=!0){var A=a.value,l=e.clampUp(A);l===e.clampDown(A)&&n.push(l)}}catch(t){i=!0,o=t}finally{try{!r&&s.return&&s.return()}finally{if(i)throw o}}return n}var T=function(){var t=[function(t){return t}],e=[0,0],r="percent",i=null,o=[],a=function(a){var s=new Array(a.length),A=!0,l=!1,c=void 0;try{for(var u,g=t[Symbol.iterator]();!(A=(u=g.next()).done);A=!0)for(var h=u.value,d=0;d<a.length;d++){var f=h(a[d],d);Array.isArray(f)?s.push.apply(s,L(f)):s.push(f)}}catch(t){l=!0,c=t}finally{try{!A&&g.return&&g.return()}finally{if(l)throw c}}var p=[n.min(s),n.max(s)];if(p[0]=null==p[0]?n.min(o):n.min([p[0]].concat(L(o))),p[1]=null==p[1]?n.max(o):n.max([p[1]].concat(L(o))),null!=i){var I=Math.max(Math.abs(p[1]-i),Math.abs(p[0]-i));p[0]=i-I,p[1]=i+I}switch(r){case"domain":p[0]-=e[0],p[1]+=e[1];break;case"percent":var C=p[1]-p[0];p[0]-=e[0]*C,p[1]+=e[1]*C;break;default:throw new Error("Unknown padUnit: "+r)}return p};return a.accessors=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],a):t},a.pad=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],a):e},a.padUnit=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],a):r},a.include=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],a):o},a.symmetricalAbout=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],a):i},a},U=function(){var t=[],e=[0,0],n="percent",r=null,i=[],o=T(),a=function(t){return null!=t?t.valueOf():null},s=function(s){var A=t.map(function(t){return function(){var e=t.apply(void 0,arguments);return Array.isArray(e)?e.map(a):a(e)}});return o.accessors(A).pad(e).padUnit(n).symmetricalAbout(null!=r?r.valueOf():null).include(i.map(function(t){return t.valueOf()})),o(s).map(function(t){return new Date(t)})};return s.accessors=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],s):t},s.pad=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],s):e},s.padUnit=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],s):n},s.include=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],s):i},s.symmetricalAbout=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],s):r},s},O=function(){var t=1,e=20,n=.1,r=.1,i=o.randomNormal(),a=function(){for(var o=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0,a=t/e,s=[],A=0;A<e+1;A++){s.push(o);var l=i()*Math.sqrt(a)*r+(n-r*r/2)*a;o*=Math.exp(l)}return s};return a.period=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],a):t},a.steps=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],a):e},a.mu=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],a):n},a.sigma=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],a):r},a.random=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],a):i},a};function G(t){return"function"==typeof t?t:function(){return t}}var H=function(){var t=10,e=function(e){return t<=1?e.map(function(t){return[t]}):n.range(0,Math.ceil(e.length/t)).map(function(n){return e.slice(n*t,(n+1)*t)})};return e.bucketSize=function(n){return arguments.length?(t=n,e):t},e},j=function(t){return"function"==typeof t?t:function(){return t}},Y=function(){var t=null,e=function(t){return t.date},n=function(t){return t.open},r=function(t){return t.high},i=function(t){return t.low},o=function(t){return t.close},a="vertical",A=j(3),l=function(l){var c=t||s.path();return l.forEach(function(t,s){var l=e(t,s),u=n(t,s),g=r(t,s),h=i(t,s),d=o(t,s),f=A(t,s)/2;"vertical"===a?(c.moveTo(l,h),c.lineTo(l,g),c.moveTo(l,u),c.lineTo(l-f,u),c.moveTo(l,d),c.lineTo(l+f,d)):(c.moveTo(h,l),c.lineTo(g,l),c.moveTo(u,l),c.lineTo(u,l+f),c.moveTo(d,l),c.lineTo(d,l-f))}),t?null:c.toString()};return l.context=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],l):t},l.x=function(){return arguments.length?(e=j(arguments.length<=0?void 0:arguments[0]),l):e},l.open=function(){return arguments.length?(n=j(arguments.length<=0?void 0:arguments[0]),l):n},l.high=function(){return arguments.length?(r=j(arguments.length<=0?void 0:arguments[0]),l):r},l.low=function(){return arguments.length?(i=j(arguments.length<=0?void 0:arguments[0]),l):i},l.close=function(){return arguments.length?(o=j(arguments.length<=0?void 0:arguments[0]),l):o},l.width=function(){return arguments.length?(A=j(arguments.length<=0?void 0:arguments[0]),l):A},l.orient=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],l):a},l},P=function(){var t=null,e=function(t){return t.x},n=function(t){return t.y},r="center",i="center",o=function(t){return t.height},a=j(3),A=function(A,l){var c=t||s.path();return A.forEach(function(t,s){var A=e.call(this,t,l||s),u=n.call(this,t,l||s),g=o.call(this,t,l||s),h=a.call(this,t,l||s),d=void 0;switch(r){case"left":d=h;break;case"right":d=0;break;case"center":d=h/2;break;default:throw new Error("Invalid horizontal alignment "+r)}var f=void 0;switch(i){case"bottom":f=-g;break;case"top":f=0;break;case"center":f=g/2;break;default:throw new Error("Invalid vertical alignment "+i)}c.rect(A-d,u-f,h,g)},this),t?null:c.toString()};return A.context=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],A):t},A.x=function(){return arguments.length?(e=j(arguments.length<=0?void 0:arguments[0]),A):e},A.y=function(){return arguments.length?(n=j(arguments.length<=0?void 0:arguments[0]),A):n},A.width=function(){return arguments.length?(a=j(arguments.length<=0?void 0:arguments[0]),A):a},A.horizontalAlign=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],A):r},A.height=function(){return arguments.length?(o=j(arguments.length<=0?void 0:arguments[0]),A):o},A.verticalAlign=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],A):i},A},K=function(){var t=null,e=function(t){return t.date},n=function(t){return t.open},r=function(t){return t.high},i=function(t){return t.low},o=function(t){return t.close},a=j(3),A=function(A){var l=t||s.path();return A.forEach(function(t,s){var A=e(t,s),c=n(t,s),u=r(t,s),g=i(t,s),h=o(t,s),d=a(t,s),f=d/2;l.rect(A-f,c,d,h-c),l.moveTo(A,Math.min(h,c)),l.lineTo(A,u),l.moveTo(A,Math.max(h,c)),l.lineTo(A,g)}),t?null:l.toString()};return A.context=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],A):t},A.x=function(){return arguments.length?(e=j(arguments.length<=0?void 0:arguments[0]),A):e},A.open=function(){return arguments.length?(n=j(arguments.length<=0?void 0:arguments[0]),A):n},A.high=function(){return arguments.length?(r=j(arguments.length<=0?void 0:arguments[0]),A):r},A.low=function(){return arguments.length?(i=j(arguments.length<=0?void 0:arguments[0]),A):i},A.close=function(){return arguments.length?(o=j(arguments.length<=0?void 0:arguments[0]),A):o},A.width=function(){return arguments.length?(a=j(arguments.length<=0?void 0:arguments[0]),A):a},A},z=function(){var t=null,e=function(t){return t.value},n=function(t){return t.median},r=function(t){return t.upperQuartile},i=function(t){return t.lowerQuartile},o=function(t){return t.high},a=function(t){return t.low},A="vertical",l=j(5),c=j(.5),u=function(u){var g=t||s.path();return u.forEach(function(t,s){var u=e(t,s),h=l(t,s),d=h/2,f=h*c(t,s)/2,p=o(t,s),I=r(t,s),C=n(t,s),m=i(t,s),y=a(t,s),E=m-I;"vertical"===A?(g.moveTo(u-f,p),g.lineTo(u+f,p),g.moveTo(u,p),g.lineTo(u,I),g.rect(u-d,I,h,E),g.moveTo(u-d,C),g.lineTo(u+d,C),g.moveTo(u,m),g.lineTo(u,y),g.moveTo(u-f,y),g.lineTo(u+f,y)):(g.moveTo(y,u-f),g.lineTo(y,u+f),g.moveTo(y,u),g.lineTo(m,u),g.rect(m,u-d,-E,h),g.moveTo(C,u-d),g.lineTo(C,u+d),g.moveTo(I,u),g.lineTo(p,u),g.moveTo(p,u-f),g.lineTo(p,u+f))}),t?null:g.toString()};return u.context=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],u):t},u.value=function(){return arguments.length?(e=j(arguments.length<=0?void 0:arguments[0]),u):e},u.median=function(){return arguments.length?(n=j(arguments.length<=0?void 0:arguments[0]),u):n},u.upperQuartile=function(){return arguments.length?(r=j(arguments.length<=0?void 0:arguments[0]),u):r},u.lowerQuartile=function(){return arguments.length?(i=j(arguments.length<=0?void 0:arguments[0]),u):i},u.high=function(){return arguments.length?(o=j(arguments.length<=0?void 0:arguments[0]),u):o},u.low=function(){return arguments.length?(a=j(arguments.length<=0?void 0:arguments[0]),u):a},u.width=function(){return arguments.length?(l=j(arguments.length<=0?void 0:arguments[0]),u):l},u.orient=function(){return arguments.length?(A=arguments.length<=0?void 0:arguments[0],u):A},u.cap=function(){return arguments.length?(c=j(arguments.length<=0?void 0:arguments[0]),u):c},u},J=function(){var t=null,e=function(t){return t.x},n=function(t){return t.high},r=function(t){return t.low},i="vertical",o=j(5),a=function(a){var A=t||s.path();return a.forEach(function(t,a){var s=e(t,a),l=o(t,a)/2,c=n(t,a),u=r(t,a);"vertical"===i?(A.moveTo(s-l,c),A.lineTo(s+l,c),A.moveTo(s,c),A.lineTo(s,u),A.moveTo(s-l,u),A.lineTo(s+l,u)):(A.moveTo(u,s-l),A.lineTo(u,s+l),A.moveTo(u,s),A.lineTo(c,s),A.moveTo(c,s-l),A.lineTo(c,s+l))}),t?null:A.toString()};return a.context=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],a):t},a.value=function(){return arguments.length?(e=j(arguments.length<=0?void 0:arguments[0]),a):e},a.high=function(){return arguments.length?(n=j(arguments.length<=0?void 0:arguments[0]),a):n},a.low=function(){return arguments.length?(r=j(arguments.length<=0?void 0:arguments[0]),a):r},a.width=function(){return arguments.length?(o=j(arguments.length<=0?void 0:arguments[0]),a):o},a.orient=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],a):i},a},q=function(t){return"function"==typeof t?t:function(){return t}},V=function(t,e){t=t||"g";var n=function(t,e){return e},r=null,i=function(i,o){o=o||function(t){return t};var a=i.selection?i:null;a&&(i=i.selection());var s=i.selectAll(function(t,e,n){return Array.from(n[e].childNodes).filter(function(t){return 1===t.nodeType})}).filter(null==e?t:t+"."+e).data(o,n),A=s.enter().append(t).attr("class",e),l=s.exit();s=s.merge(A);var c=a||r;return c&&(s=s.transition(c).style("opacity",1),A.style("opacity",1e-6),l=l.transition(c).style("opacity",1e-6)),l.remove(),s.enter=function(){return A},s.exit=function(){return l},s};return i.element=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],i):t},i.className=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],i):e},i.key=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},i.transition=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],i):r},i},W=function(t,e){if(function(t,e){return!(t.x>=e.x+e.width||t.x+t.width<=e.x||t.y>=e.y+e.height||t.y+t.height<=e.y)}(t,e)){var n=Math.max(t.x,e.x),r=Math.min(t.x+t.width,e.x+e.width),i=Math.max(t.y,e.y);return(r-n)*(Math.min(t.y+t.height,e.y+e.height)-i)}return 0},X=function(t,e){return n.sum(t.map(function(n,r){return e===r?0:W(t[e],n)}))},Z=function(t,e,n,r,i){return{x:t,y:e,width:n,height:r,location:i}},$=function(t){var e=t.x,n=t.y,r=t.width,i=t.height;return[Z(e,n,r,i,"bottom-right"),Z(e-r,n,r,i,"bottom-left"),Z(e-r,n-i,r,i,"top-left"),Z(e,n-i,r,i,"top-right"),Z(e,n-i/2,r,i,"middle-right"),Z(e-r/2,n,r,i,"bottom-center"),Z(e-r,n-i/2,r,i,"middle-left"),Z(e-r/2,n-i,r,i,"top-center")]},tt=function(t,e,n){return[].concat(L(t.slice(0,e)),[n],L(t.slice(e+1)))},et=function(t,e){return t<e},nt=function t(){var e=null,r=et,i=function(){return 0},o=void 0,a=function a(s,A){e||(e=n.sum(o.map(function(t,e){return i(t,e,o)})));var l=function(t,n){return e-i(o[n],n,o)+i(t,n,tt(o,n,t))}(s,A);return r(l,e)?t().locationScore(i).winningScore(r).score(l).rectangles(tt(o,A,s)):a};return a.rectangles=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],a):o},a.score=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],a):e},a.winningScore=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],a):r},a.locationScore=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],a):i},a},rt=function(t){return Math.floor(Math.random()*t.length)},it=function(t){return"function"==typeof t?t:function(){return t}};function ot(){var t=arguments;return function(e,n){for(var r=0,i=t.length;r<i;r++)if(null==t[r](e,n))return!1;return!0}}var at=function(t,e){switch(t){case"left":return e/2;case"right":return-e/2;default:return 0}},st=function(t){var e=Object.assign({},t),n=function(){};return Object.keys(e).forEach(function(t){n[t]=function(){return arguments.length?(e[t]=arguments.length<=0?void 0:arguments[0],n):e[t]}}),n},At=function(){var t=function(){return 0},e=function(t){return t.x},n=function(t){return t.y},i="center",o=function(){return 5},a="vertical",s=st({decorate:function(){},defined:function(r,i){return ot(t,e,n)(r,i)},xScale:r.scaleIdentity(),yScale:r.scaleIdentity()});return s.values=function(r,A){var l=o(r,A),c=at(i,l),u=s.xScale(),g=s.yScale();if("vertical"===a){var h=g(n(r,A),A),d=g(t(r,A),A),f=u(e(r,A),A)+c;return{d:r,x:f,y:h,y0:d,width:l,height:h-d,origin:[f,h],baseOrigin:[f,d],transposedX:f,transposedY:h}}var p=u(n(r,A),A),I=u(t(r,A),A),C=g(e(r,A),A)+c;return{d:r,x:C,y:p,y0:I,width:l,height:p-I,origin:[p,C],baseOrigin:[I,C],transposedX:p,transposedY:C}},s.baseValue=function(){return arguments.length?(t=it(arguments.length<=0?void 0:arguments[0]),s):t},s.crossValue=function(){return arguments.length?(e=it(arguments.length<=0?void 0:arguments[0]),s):e},s.mainValue=function(){return arguments.length?(n=it(arguments.length<=0?void 0:arguments[0]),s):n},s.bandwidth=function(){return arguments.length?(o=it(arguments.length<=0?void 0:arguments[0]),s):o},s.align=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],s):i},s.orient=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],s):a},s},lt="#000",ct="#ddd",ut=function(){var t=At(),e=l.line().x(function(e,n){return t.values(e,n).transposedX}).y(function(e,n){return t.values(e,n).transposedY}),n=V("path","line"),r=function(r){r.selection&&n.transition(r),e.defined(t.defined()),r.each(function(r,i,o){var a=n(A.select(o[i]),[r]);a.enter().attr("fill","none").attr("stroke",lt),a.attr("d",e),t.decorate()(a,r,i)})};return f(r,t,I("baseValue","bandwidth","align")),h(r,n,"key"),h(r,e,"curve"),r},gt=function(){var t=At(),e=l.line().x(function(e,n){return t.values(e,n).transposedX}).y(function(e,n){return t.values(e,n).transposedY}),n=function(n){var r=e.context();r.beginPath(),e.defined(t.defined())(n),r.strokeStyle=lt,r.fillStyle="transparent",t.decorate()(r,n),r.fill(),r.stroke(),r.closePath()};return f(n,t,I("baseValue","bandwidth","align")),h(n,e,"curve","context"),n},ht=function(){var t=l.symbol(),e=At(),n=V("g","point"),r=function(t){return"translate("+t[0]+", "+t[1]+")"},i=function(i){i.selection&&n.transition(i),i.each(function(i,o,a){var s=i.filter(e.defined()),l=n(A.select(a[o]),s);l.enter().attr("transform",function(t,n){return r(e.values(t,n).origin)}).attr("fill",ct).attr("stroke",lt).append("path"),l.attr("transform",function(t,n){return r(e.values(t,n).origin)}).select("path").attr("d",t),e.decorate()(l,i,o)})};return f(i,e,I("baseValue","bandwidth","align")),h(i,n,"key"),h(i,t,"type","size"),i},dt=function(){var t=l.symbol(),e=At(),n=function(n){var r=n.filter(e.defined()),i=t.context();r.forEach(function(n,r){i.save();var o=e.values(n,r);i.translate(o.origin[0],o.origin[1]),i.beginPath(),t(n,r),i.strokeStyle=lt,i.fillStyle=ct,e.decorate()(i,n,r),i.fill(),i.stroke(),i.closePath(),i.restore()})};return f(n,e,I("baseValue","bandwidth","align")),h(n,t,"size","type","context"),n},ft=function(){var t=function(t){return t.high},e=function(t){return t.low},n=function(t){return t.cross},i="vertical",o="center",a=function(){return 5},s=st({decorate:function(){},defined:function(r,i){return ot(e,t,n)(r,i)},xScale:r.scaleIdentity(),yScale:r.scaleIdentity()});return s.values=function(r,A){var l=a(r,A),c=at(o,l),u=s.xScale(),g=s.yScale();if("vertical"===i){var h=g(t(r,A));return{origin:[u(n(r,A))+c,h],high:0,low:g(e(r,A))-h,width:l}}var d=u(e(r,A));return{origin:[d,g(n(r,A))+c],high:u(t(r,A))-d,low:0,width:l}},s.orient=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],s):i},s.lowValue=function(){return arguments.length?(e=it(arguments.length<=0?void 0:arguments[0]),s):e},s.highValue=function(){return arguments.length?(t=it(arguments.length<=0?void 0:arguments[0]),s):t},s.crossValue=function(){return arguments.length?(n=it(arguments.length<=0?void 0:arguments[0]),s):n},s.bandwidth=function(){return arguments.length?(a=it(arguments.length<=0?void 0:arguments[0]),s):a},s.align=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],s):o},s},pt=function(){var t=void 0,e=function(t){return t.date},n=function(t){return t.open},i=function(t){return t.high},o=function(t){return t.low},a=function(t){return t.close},s=function(){return 5},A="center",l=function(n,r){return t.xScale()(e(n,r))};return(t=st({decorate:function(){},defined:function(t,r){return ot(e,n,o,i,a)(t,r)},xScale:r.scaleIdentity(),yScale:r.scaleIdentity()})).values=function(e,r){var c=a(e,r),u=n(e,r),g=s(e,r),h=at(A,g),d="";return c>u?d="up":c<u&&(d="down"),{cross:l(e,r)+h,open:t.yScale()(u),high:t.yScale()(i(e,r)),low:t.yScale()(o(e,r)),close:t.yScale()(c),width:g,direction:d}},t.crossValue=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],t):e},t.openValue=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],t):n},t.highValue=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],t):i},t.lowValue=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],t):o},t.yValue=t.closeValue=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],t):a},t.bandwidth=function(){return arguments.length?(s=it(arguments.length<=0?void 0:arguments[0]),t):s},t.align=function(){return arguments.length?(A=arguments.length<=0?void 0:arguments[0],t):A},t},It=function(t,e){var n=pt(),r=V("g",e),i=function(t){return"translate("+t.cross+", "+t.high+")"},o=function(o){o.selection&&r.transition(o);var a,s=(a=o,function(t){return a.selection?t.transition(a):t});o.each(function(o,a,l){var c=o.filter(n.defined()),u=r(A.select(l[a]),c);u.enter().attr("transform",function(t,e){return i(n.values(t,e))+" scale(1e-6, 1)"}).append("path"),u.each(function(r,o,a){var l=n.values(r,o),c="up"===l.direction?"#6c0":"#c60",u=s(A.select(a[o])).attr("class",e+" "+l.direction).attr("stroke",c).attr("fill",c).attr("transform",function(){return i(l)+" scale(1)"});t.x(0).width(l.width).open(function(){return l.open-l.high}).high(0).low(function(){return l.low-l.high}).close(function(){return l.close-l.high}),u.select("path").attr("d",t([r]))}),n.decorate()(u,o,a)})};return h(o,r,"key"),f(o,n),o},Ct=function(t){var e=pt(),n=function(n){var r=n.filter(e.defined()),i=t.context();r.forEach(function(n,r){i.save();var o=e.values(n,r);i.translate(o.cross,o.high),i.beginPath(),t.x(0).open(function(){return o.open-o.high}).width(o.width).high(0).low(function(){return o.low-o.high}).close(function(){return o.close-o.high})([n]);var a="up"===o.direction?"#6c0":"#c60";i.strokeStyle=a,i.fillStyle=a,e.decorate()(i,n,r),i.fill(),i.stroke(),i.closePath(),i.restore()})};return h(n,t,"context"),f(n,e),n},mt=function(){var t=function(t){return t.upperQuartile},e=function(t){return t.lowerQuartile},n=function(t){return t.high},i=function(t){return t.low},o=function(t){return t.value},a=function(t){return t.median},s="vertical",A="center",l=function(){return 5},c=st({decorate:function(){},defined:function(r,s){return ot(i,n,e,t,o,a)(r,s)},xScale:r.scaleIdentity(),yScale:r.scaleIdentity()});return c.values=function(r,u){var g=l(r,u),h=at(A,g),d=c.xScale(),f=c.yScale();if("vertical"===s){var p=f(n(r,u));return{origin:[d(o(r,u))+h,p],high:0,upperQuartile:f(t(r,u))-p,median:f(a(r,u))-p,lowerQuartile:f(e(r,u))-p,low:f(i(r,u))-p,width:g}}var I=d(i(r,u));return{origin:[I,f(o(r,u))+h],high:d(n(r,u))-I,upperQuartile:d(t(r,u))-I,median:d(a(r,u))-I,lowerQuartile:d(e(r,u))-I,low:0,width:g}},c.orient=function(){return arguments.length?(s=arguments.length<=0?void 0:arguments[0],c):s},c.lowerQuartileValue=function(){return arguments.length?(e=it(arguments.length<=0?void 0:arguments[0]),c):e},c.upperQuartileValue=function(){return arguments.length?(t=it(arguments.length<=0?void 0:arguments[0]),c):t},c.lowValue=function(){return arguments.length?(i=it(arguments.length<=0?void 0:arguments[0]),c):i},c.highValue=function(){return arguments.length?(n=it(arguments.length<=0?void 0:arguments[0]),c):n},c.crossValue=function(){return arguments.length?(o=it(arguments.length<=0?void 0:arguments[0]),c):o},c.medianValue=function(){return arguments.length?(a=it(arguments.length<=0?void 0:arguments[0]),c):a},c.bandwidth=function(){return arguments.length?(l=it(arguments.length<=0?void 0:arguments[0]),c):l},c.align=function(){return arguments.length?(A=arguments.length<=0?void 0:arguments[0],c):A},c},yt=function(){var t=[],e=function(t){return t},n=function(t,e){return e},i=st({decorate:function(){},xScale:r.scaleIdentity(),yScale:r.scaleIdentity()});return i.mapping=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],i):e},i.key=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},i.series=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],i):t},i},Et=function(){var t=yt(),e=V("g"),n=V("g","multi"),r=function(r){r.selection&&(n.transition(r),e.transition(r));var i=t.mapping(),o=t.series(),a=t.xScale(),s=t.yScale();r.each(function(r,l,c){var u=n(A.select(c[l]),o);u.each(function(t,n,l){t.xScale(a).yScale(s);var c=i(r,n,o);e(A.select(l[n]),[c]).call(t)}),(u.selection?u.selection():u).order(),t.decorate()(u,r,l)})};return f(r,t),h(r,n,"key"),r},vt=function(){var t=null,e=yt(),n=function(n){var r=e.mapping(),i=e.series(),o=e.xScale(),a=e.yScale();i.forEach(function(s,A){var l=r(n,A,i);s.context(t).xScale(o).yScale(a);var c=void 0;s.decorate?(c=s.decorate(),s.decorate(function(t,r,i){e.decorate()(t,n,A),c(t,r,i)})):e.decorate()(t,n,A),s(l),c&&s.decorate(c)})};return n.context=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],n):t},f(n,e),n},Bt=function(t){var e=function(){return 50},i="center",o=r.scaleBand(),a=st({decorate:function(){},xScale:r.scaleLinear(),yScale:r.scaleLinear()});return a.offsetScaleForDatum=function(t,r,a){var s=e(r,a),A=at(i,s),l=s/2;return o.domain(n.range(0,t.length)).range([-l+A,l+A])},a.bandwidth=function(){return arguments.length?(e=it(arguments.length<=0?void 0:arguments[0]),a):e},a.align=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],a):i},f(a,o,m({paddingInner:"paddingOuter"})),a},bt=function(){var t=function(t){return t.x},e=function(t){return t.y},i=function(t){return t.color},o=function(){return 5},a=function(){return 5},s=r.interpolateViridis,A=st({decorate:function(){},defined:function(n,r){return ot(t,e,i)(n,r)},xScale:r.scaleIdentity(),yScale:r.scaleIdentity()});return A.pathGenerator=P().x(0).y(0),A.colorScale=function(t){var e=t.map(i);return r.scaleLinear().domain([n.min(e),n.max(e)])},A.values=function(n,r){return{x:A.xScale()(t(n,r)),y:A.yScale()(e(n,r)),colorValue:i(n,r),width:a(n,r),height:o(n,r)}},A.xValue=function(){return arguments.length?(t=it(arguments.length<=0?void 0:arguments[0]),A):t},A.yValue=function(){return arguments.length?(e=it(arguments.length<=0?void 0:arguments[0]),A):e},A.colorValue=function(){return arguments.length?(i=it(arguments.length<=0?void 0:arguments[0]),A):i},A.colorInterpolate=function(){return arguments.length?(s=arguments.length<=0?void 0:arguments[0],A):s},A.xBandwidth=function(){return arguments.length?(a=it(arguments.length<=0?void 0:arguments[0]),A):a},A.yBandwidth=function(){return arguments.length?(o=it(arguments.length<=0?void 0:arguments[0]),A):o},f(A,A.pathGenerator,m({horizontalAlign:"xAlign",verticalAlign:"yAlign"})),A},Qt=function(t){return"function"==typeof t?t:function(){return t}},wt=function(){var t=r.scaleIdentity(),e=r.scaleIdentity(),n=function(t){return t},i=n,o=function(){},a="horizontal",s=V("g","annotation-line"),l=function(r){if(r.selection&&s.transition(r),"horizontal"!==a&&"vertical"!==a)throw new Error("Invalid orientation");var l="horizontal"===a,c=l?function(t,e){return"translate("+t+", "+e+")"}:function(t,e){return"translate("+e+", "+t+")"},u=l?"x2":"y2",g=l?e:t,h=l?"left-handle":"bottom-handle",d=l?"right-handle":"top-handle",f=l?"9":"0",p=l?"0":"9",I=l?"0.32em":"0.71em",C=l?"start":"middle",m=(l?t:e).range(),y=function(){return c(m[0],g(n.apply(void 0,arguments)))},E=m[1]-m[0];r.each(function(t,e,n){var r=s(A.select(n[e]),t),l=r.enter().attr("transform",y).style("stroke","#bbb");l.append("line").attr(u,E),l.append("g").classed(h,!0).style("stroke","none"),l.append("g").classed(d,!0).style("stroke","none").attr("transform",c(E,0)).append("text").attr("text-anchor",C).attr("x",f).attr("y",p).attr("dy",I),r.attr("class","annotation-line "+a),r.attr("transform",y),r.select("line").attr(u,E),r.select("g."+d).attr("transform",c(E,0)),r.select("text").text(i),o(r,t,e)})};return l.xScale=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],l):t},l.yScale=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],l):e},l.value=function(){return arguments.length?(n=Qt(arguments.length<=0?void 0:arguments[0]),l):n},l.label=function(){return arguments.length?(i=Qt(arguments.length<=0?void 0:arguments[0]),l):i},l.decorate=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],l):o},l.orient=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],l):a},l},xt=function(){var t=r.scaleIdentity(),e=r.scaleIdentity(),n=function(t){return t},i=n,o=function(){},a="horizontal",s=l.line(),A=function(r){if("horizontal"!==a&&"vertical"!==a)throw new Error("Invalid orientation");var A="horizontal"===a,l=s.context(),c=A?t:e,u=A?e:t,g=c.domain(),h=A?9:0,d=A?0:9,f=A?"left":"center",p=A?"middle":"hanging";r.forEach(function(t,e){l.save(),l.beginPath(),l.strokeStyle="#bbb",l.fillStyle="#000",l.textAlign=f,l.textBaseline=p,o(l,t,e),s.context(l)(g.map(function(e){var r=[c(e),u(n(t))];return A?r:r.reverse()}));var r=A?c(g[1]):u(n(t)),a=A?u(n(t)):c(g[1]);l.fillText(i(t),r+h,a+d),l.fill(),l.stroke(),l.closePath(),l.restore()})};return A.xScale=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],A):t},A.yScale=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],A):e},A.value=function(){return arguments.length?(n=Qt(arguments.length<=0?void 0:arguments[0]),A):n},A.label=function(){return arguments.length?(i=Qt(arguments.length<=0?void 0:arguments[0]),A):i},A.decorate=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],A):o},A.orient=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],A):a},h(A,s,"context"),A},_t=function(){var t=r.scaleIdentity(),e=[10],n=null,i=function(){var r;return null!=n?n:t.ticks?(r=t).ticks.apply(r,L(e)):t.domain()};return i.scale=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],i):t},i.ticks=function(){for(var t=arguments.length,n=Array(t),r=0;r<t;r++)n[r]=arguments[r];return e=n,i},i.tickArguments=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],i):e},i.tickValues=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},i},Dt=function(t){return t},Mt=function(t){return t},kt=function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=[10],i=null,o=function(){},a=null,s=6,c=6,u=3,g=l.line(),h=V("g","tick").key(Mt),d=V("path","domain"),f=function(){return{offset:[0,c+u]}},p=function(){return{path:[[0,0],[0,c]]}},I=n.labelOffset||f,C=n.tickPath||p,m=function(t,e){var n=0;return t.bandwidth&&(n=t.bandwidth()/2,t.round()&&(n=Math.round(n))),function(r){return e(t(r)+n,0)}},y=function(t,e){return v()?"translate("+e+", "+t+")":"translate("+t+", "+e+")"},E=function(t){return v()?t.map(function(t){return[t[1],t[0]]}):t},v=function(){return"left"===t||"right"===t},B=function(t,n,r){return e[t]?e[t].apply(e,n):r},b=function(n){n.selection&&(h.transition(n),d.transition(n)),n.each(function(n,l,c){var u=c[l],f=A.select(u);u.__scale__||f.attr("fill","none").attr("font-size",10).attr("font-family","sans-serif").attr("text-anchor","right"===t?"start":"left"===t?"end":"middle");var p=u.__scale__||e;u.__scale__=e.copy();var b=null==i?B("ticks",r,e.domain()):i,Q=null==a?B("tickFormat",r,Mt):a,w="bottom"===t||"right"===t?1:-1,x=function(t){var e=S(t,2),n=e[0],r=e[1];return[n,w*r]},_=e.range(),D=E([[_[0],w*s],[_[0],0],[_[1],0],[_[1],w*s]]);d(f,[n]).attr("d",g(D)).attr("stroke","#000");var M=h(f,b),k=b.map(function(t,e){return I(t,e,b)}),R=b.map(function(t,e){return C(t,e,b)});M.enter().attr("transform",m(p,y)).append("path").attr("stroke","#000"),M.enter().append("text").attr("transform",function(t,e){return y.apply(void 0,L(x(k[e].offset)))}).attr("fill","#000"),M.exit().attr("transform",m(e,y)),M.select("path").attr("visibility",function(t,e){return R[e].hidden&&"hidden"}).attr("d",function(t,e){return g(E(R[e].path.map(x)))}),M.select("text").attr("visibility",function(t,e){return k[e].hidden&&"hidden"}).attr("transform",function(t,e){return y.apply(void 0,L(x(k[e].offset)))}).attr("dy",function(){var e="0em";return v()?e="0.32em":"bottom"===t&&(e="0.71em"),e}).text(Q),M.attr("transform",m(e,y)),o(M,n,l)})};return b.tickFormat=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],b):a},b.tickSize=function(){return arguments.length?(c=s=Number(arguments.length<=0?void 0:arguments[0]),b):c},b.tickSizeInner=function(){return arguments.length?(c=Number(arguments.length<=0?void 0:arguments[0]),b):c},b.tickSizeOuter=function(){return arguments.length?(s=Number(arguments.length<=0?void 0:arguments[0]),b):s},b.tickPadding=function(){return arguments.length?(u=arguments.length<=0?void 0:arguments[0],b):u},b.decorate=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],b):o},b.scale=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],b):e},b.ticks=function(){for(var t=arguments.length,e=Array(t),n=0;n<t;n++)e[n]=arguments[n];return r=[].concat(e),b},b.tickArguments=function(){return arguments.length?(r=null==(arguments.length<=0?void 0:arguments[0])?[]:[].concat(L(arguments.length<=0?void 0:arguments[0])),b):r.slice()},b.tickValues=function(){return arguments.length?(i=null==(arguments.length<=0?void 0:arguments[0])?[]:[].concat(L(arguments.length<=0?void 0:arguments[0])),b):i.slice()},b.orient=function(){return t},b},St=function(t,e){var n=!1,r=kt(t,e,{labelOffset:function(t,i,o){var a=0,s=r.tickSizeInner()+r.tickPadding(),A=!1;if(n){var l=e(t),c=i<o.length-1?e(o[i+1]):e.range()[1];a=(c-l)/2,s=r.tickPadding(),A=i===o.length-1&&l===c}return{offset:[a,s],hidden:A}}}),i=function(t){return r(t)};return i.tickCenterLabel=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},f(i,r),i},Lt=function(t){return St("top",t)},Rt=function(t){return St("bottom",t)},Nt=function(t){return St("left",t)},Ft=function(t){return St("right",t)},Tt=function(t,e){var n=null,r=kt(t,e,{labelOffset:function(){return{offset:[0,r.tickPadding()]}},tickPath:function(t,i,o){var a;return{path:[[a=n?n(t,i):function(t,n,r){if(e.step)return e.step();var i=e(t);return n<r.length-1?e(r[n+1])/i:2*(e.range()[1]-i)}(t,i,o)/2,0],[a,r.tickSizeInner()]],hidden:i===o.length-1}}}),i=function(t){r(t)};return i.tickOffset=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},f(i,r),i},Ut=function(t){return function(e){var n=t.ticks?t.ticks():t.domain(),r=e.append("text"),i=n.map(function(t){return r.text(t).node().getBBox()}),o=Math.max.apply(Math,L(i.map(function(t){return t.height}))),a=Math.max.apply(Math,L(i.map(function(t){return t.width})));return r.remove(),{maxHeight:o,maxWidth:a,labelCount:n.length}}},Ot="__d3fc-elements__",Gt=function(t){return t[Ot]||{}},Ht=function(t,e){t[Ot]=e},jt=function(t){return"D3FC-GROUP"===t.tagName?[t].concat(L(t.querySelectorAll("d3fc-canvas, d3fc-group, d3fc-svg"))):[t]},Yt=function(e){var n=Gt(e),r=n.width,i=n.height,o=e.useDevicePixelRatio&&null!=t.devicePixelRatio?t.devicePixelRatio:1,a=e.clientWidth*o,s=e.clientHeight*o;Ht(e,{pixelRatio:o,width:a,height:s,resized:a!==r||s!==i})};if("function"!=typeof CustomEvent)throw new Error("d3fc-element depends on CustomEvent. Make sure that you load a polyfill in older browsers. See README.");var Pt=function(t){var e=Gt(t),n=new CustomEvent("measure",{detail:e});t.dispatchEvent(n)},Kt=function(t){var e=Gt(t),n=new CustomEvent("draw",{detail:e});t.dispatchEvent(n)},zt=function(t){return Gt(t.ownerDocument).queue||[]},Jt=function(t){return function(t){return delete t[Ot]}(t.ownerDocument)},qt=function(t,e){var n=t;do{if(n.parentNode===e)return!0}while(n=n.parentNode);return!1},Vt=function(t){var e=zt(t);if(!(e.indexOf(t)>-1||e.some(function(e){return qt(t,e)}))){var n=e.filter(function(e){return!qt(e,t)});n.push(t),function(t,e){var n=Gt(t.ownerDocument).requestId;null==n&&(n=requestAnimationFrame(function(){var e;(e=zt(t).map(jt).reduce(function(t,e){return t.concat(e)})).forEach(Yt),e.forEach(Pt),e.forEach(Kt),Jt(t)})),Ht(t.ownerDocument,{queue:e,requestId:n})}(t,n)}};function Wt(){return Reflect.construct(HTMLElement,[],this.__proto__.constructor)}if(Object.setPrototypeOf(Wt.prototype,HTMLElement.prototype),Object.setPrototypeOf(Wt,HTMLElement),"function"!=typeof HTMLElement)throw new Error("d3fc-element depends on Custom Elements (v1). Make sure that you load a polyfill in older browsers. See README.");var Xt=function(t,e){return function(n){function r(){return _(this,r),k(this,(r.__proto__||Object.getPrototypeOf(r)).apply(this,arguments))}return M(r,Wt),D(r,[{key:"attributeChangedCallback",value:function(t){switch(t){case"use-device-pixel-ratio":this.requestRedraw()}}},{key:"connectedCallback",value:function(){0===this.childNodes.length&&this.appendChild(t()),function(t){null==t.__measureListener__&&(t.__measureListener__=function(e){return t.setMeasurements(e.detail)},t.addEventListener("measure",t.__measureListener__))}(this)}},{key:"disconnectedCallback",value:function(){!function(t){null!=t.__measureListener__&&(t.removeEventListener("measure",t.__measureListener__),t.__measureListener__=null)}(this)}},{key:"setMeasurements",value:function(t){var n,r=t.width,i=t.height,o=(n=this.childNodes,Array.isArray(n)?n:Array.from(n)),a=o[0];if(o.slice(1).length>0)throw new Error("A d3fc-svg/canvas element must only contain a single svg/canvas element.");e(a,{width:r,height:i})}},{key:"requestRedraw",value:function(){Vt(this)}},{key:"useDevicePixelRatio",get:function(){return this.hasAttribute("use-device-pixel-ratio")&&"false"!==this.getAttribute("use-device-pixel-ratio")},set:function(t){t&&!this.useDevicePixelRatio?this.setAttribute("use-device-pixel-ratio",""):!t&&this.useDevicePixelRatio&&this.removeAttribute("use-device-pixel-ratio"),this.requestRedraw()}}],[{key:"observedAttributes",get:function(){return["use-device-pixel-ratio"]}}]),r}()},Zt=Xt(function(){return document.createElement("canvas")},function(t,e){var n=e.width,r=e.height;t.setAttribute("width",n),t.setAttribute("height",r)});function $t(){return Reflect.construct(HTMLElement,[],this.__proto__.constructor)}Object.setPrototypeOf($t.prototype,HTMLElement.prototype),Object.setPrototypeOf($t,HTMLElement);var te=function(t){t.autoResize?ee(t):ne(t)},ee=function(t){null==t.__autoResizeListener__&&(t.__autoResizeListener__=function(){return Vt(t)},addEventListener("resize",t.__autoResizeListener__))},ne=function(t){null!=t.__autoResizeListener__&&(removeEventListener("resize",t.__autoResizeListener__),t.__autoResizeListener__=null)},re=function(t){function e(){return _(this,e),k(this,(e.__proto__||Object.getPrototypeOf(e)).apply(this,arguments))}return M(e,$t),D(e,[{key:"connectedCallback",value:function(){te(this)}},{key:"disconnectedCallback",value:function(){ne(this)}},{key:"requestRedraw",value:function(){Vt(this)}},{key:"attributeChangedCallback",value:function(t){switch(t){case"auto-resize":te(this)}}},{key:"autoResize",get:function(){return this.hasAttribute("auto-resize")&&"false"!==this.getAttribute("auto-resize")},set:function(t){t&&!this.autoResize?this.setAttribute("auto-resize",""):!t&&this.autoResize&&this.removeAttribute("auto-resize"),te(this)}}],[{key:"observedAttributes",get:function(){return["auto-resize"]}}]),e}(),ie=Xt(function(){return document.createElementNS("http://www.w3.org/2000/svg","svg")},function(t,e){var n=e.width,r=e.height;t.setAttribute("viewBox","0 0 "+n+" "+r)}),oe="d3fc-canvas,d3fc-svg{position:relative;display:block}d3fc-canvas>canvas,d3fc-svg>svg{position:absolute;height:100%;width:100%}d3fc-svg>svg{overflow:visible}",ae=document.createElement("style");if(ae.setAttribute("type","text/css"),document.querySelector("head").appendChild(ae),ae.styleSheet?ae.styleSheet.cssText+=oe:ae.textContent+=oe,"object"!==("undefined"==typeof customElements?"undefined":x(customElements))||"function"!=typeof customElements.define)throw new Error("d3fc-element depends on Custom Elements (v1). Make sure that you load a polyfill in older browsers. See README.");customElements.define("d3fc-canvas",Zt),customElements.define("d3fc-group",re),customElements.define("d3fc-svg",ie);var se=function(){for(var t=arguments.length,e=Array(t),n=0;n<t;n++)e[n]=arguments[n];var r={},i=function(t){var e=!0,n=!1,i=void 0;try{for(var o,a=Object.keys(r)[Symbol.iterator]();!(e=(o=a.next()).done);e=!0){var s=o.value;t[s](r[s])}}catch(t){n=!0,i=t}finally{try{!e&&a.return&&a.return()}finally{if(n)throw i}}return t},o=function(t){i[t]=function(){return arguments.length?(r[t]=arguments.length<=0?void 0:arguments[0],i):r[t]}},a=!0,s=!1,A=void 0;try{for(var l,c=e[Symbol.iterator]();!(a=(l=c.next()).done);a=!0){var u=l.value;o(u)}}catch(t){s=!0,A=t}finally{try{!a&&c.return&&c.return()}finally{if(s)throw A}}return i},Ae="d3fc-group.cartesian-chart{width:100%;height:100%;overflow:hidden;display:grid;display:-ms-grid;grid-template-columns:minmax(1em,max-content) auto 1fr auto minmax(1em,max-content);-ms-grid-columns:minmax(1em,max-content) auto 1fr auto minmax(1em,max-content);grid-template-rows:minmax(1em,max-content) auto 1fr auto minmax(1em,max-content);-ms-grid-rows:minmax(1em,max-content) auto 1fr auto minmax(1em,max-content);}\nd3fc-group.cartesian-chart>.top-label{align-self:center;-ms-grid-column-align:center;justify-self:center;-ms-grid-row-align:center;grid-column:3;-ms-grid-column:3;-ms-grid-row:1;-ms-grid-row:1;}\nd3fc-group.cartesian-chart>.top-axis{height:2em;grid-column:3;-ms-grid-column:3;grid-row:2;-ms-grid-row:2;}\nd3fc-group.cartesian-chart>.left-label{align-self:center;-ms-grid-column-align:center;justify-self:center;-ms-grid-row-align:center;grid-column:1;-ms-grid-column:1;grid-row:3;-ms-grid-row:3;}\nd3fc-group.cartesian-chart>.left-axis{width:3em;grid-column:2;-ms-grid-column:2;grid-row:3;-ms-grid-row:3;}\nd3fc-group.cartesian-chart>.plot-area{overflow:hidden;grid-column:3;-ms-grid-column:3;grid-row:3;-ms-grid-row:3;}\nd3fc-group.cartesian-chart>.right-axis{width:3em;grid-column:4;-ms-grid-column:4;grid-row:3;-ms-grid-row:3;}\nd3fc-group.cartesian-chart>.right-label{align-self:center;-ms-grid-column-align:center;justify-self:center;-ms-grid-row-align:center;grid-column:5;-ms-grid-column:5;grid-row:3;-ms-grid-row:3;}\nd3fc-group.cartesian-chart>.bottom-axis{height:2em;grid-column:3;-ms-grid-column:3;grid-row:4;-ms-grid-row:4;}\nd3fc-group.cartesian-chart>.bottom-label{align-self:center;-ms-grid-column-align:center;justify-self:center;-ms-grid-row-align:center;grid-column:3;-ms-grid-column:3;grid-row:5;-ms-grid-row:5;}",le=document.createElement("style");le.setAttribute("type","text/css"),document.querySelector("head").appendChild(le),le.styleSheet?le.styleSheet.cssText+=Ae:le.textContent+=Ae;var ce=function(t){return"function"==typeof t?t:function(){return t}},ue=function(){for(var t=arguments.length,e=Array(t),n=0;n<t;n++)e[n]=arguments[n];var r=ge.apply(void 0,e),i=r.xScale,o=r.yScale,a=r.xAxis,s=r.yAxis,l=ce(""),c=ce(""),u=ce(null),g=ce(null),h=ce("right"),d=ce("bottom"),p=vt(),C=Et(),m=se("tickFormat","ticks","tickArguments","tickSize","tickSizeInner","tickSizeOuter","tickValues","tickPadding","tickCenterLabel"),E=function(){},v=se("tickFormat","ticks","tickArguments","tickSize","tickSizeInner","tickSizeOuter","tickValues","tickPadding","tickCenterLabel"),B=function(){},b=function(){},Q=V("d3fc-group","cartesian-chart"),w=V("d3fc-svg","x-axis").key(function(t){return t}),x=V("d3fc-svg","y-axis").key(function(t){return t}),_=V("div","x-label").key(function(t){return t}),D=V("div","y-label").key(function(t){return t}),M=function(t){return function(e){return t.selection?e.transition(t):e}},k=function(t){var e=M(t);t.each(function(t,n,r){var f=Q(A.select(r[n]),[t]);f.enter().attr("auto-resize","").html('<d3fc-svg class="plot-area"></d3fc-svg><d3fc-canvas class="plot-area"></d3fc-canvas>'),_(f,[d(t)]).attr("class",function(t){return"x-label "+t+"-label"}).text(l(t)),D(f,[h(t)]).attr("class",function(t){return"y-label "+t+"-label"}).text(c(t)),w(f,[d(t)]).attr("class",function(t){return"x-axis "+t+"-axis"}).style("height",u(t)).on("measure",function(t,e,n){var r=A.event.detail,o=r.width,a=r.height;"top"===t&&A.select(n[e]).select("svg").attr("viewBox","0 "+-a+" "+o+" "+a),i.range([0,o])}).on("draw",function(t,n,r){var o="top"===t?a.top(i):a.bottom(i);o.decorate(E),e(A.select(r[n])).select("svg").call(m(o))}),x(f,[h(t)]).attr("class",function(t){return"y-axis "+t+"-axis"}).style("width",g(t)).on("measure",function(t,e,n){var r=A.event.detail,i=r.width,a=r.height;"left"===t&&A.select(n[e]).select("svg").attr("viewBox",-i+" 0 "+i+" "+a),o.range([a,0])}).on("draw",function(t,n,r){var i="left"===t?s.left(o):s.right(o);i.decorate(B),e(A.select(r[n])).select("svg").call(v(i))}),f.select("d3fc-canvas.plot-area").on("draw",function(t,e,n){var r=A.select(n[e]).select("canvas").node();p.context(r.getContext("2d")).xScale(i).yScale(o),p(t)}),f.select("d3fc-svg.plot-area").on("draw",function(t,n,r){C.xScale(i).yScale(o),e(A.select(r[n])).select("svg").call(C)}),f.each(function(t,e,n){return n[e].requestRedraw()}),b(f,t,n)})},S=I(/range\w*/,/tickFormat/);return f(k,i,S,y("x")),f(k,o,S,y("y")),f(k,m,y("x")),f(k,v,y("y")),k.xOrient=function(){return arguments.length?(d=ce(arguments.length<=0?void 0:arguments[0]),k):d},k.yOrient=function(){return arguments.length?(h=ce(arguments.length<=0?void 0:arguments[0]),k):h},k.xDecorate=function(){return arguments.length?(E=arguments.length<=0?void 0:arguments[0],k):E},k.yDecorate=function(){return arguments.length?(B=arguments.length<=0?void 0:arguments[0],k):B},k.xLabel=function(){return arguments.length?(l=ce(arguments.length<=0?void 0:arguments[0]),k):l},k.yLabel=function(){return arguments.length?(c=ce(arguments.length<=0?void 0:arguments[0]),k):c},k.xAxisHeight=function(){return arguments.length?(u=ce(arguments.length<=0?void 0:arguments[0]),k):u},k.yAxisWidth=function(){return arguments.length?(g=ce(arguments.length<=0?void 0:arguments[0]),k):g},k.canvasPlotArea=function(){return arguments.length?(p=arguments.length<=0?void 0:arguments[0],k):p},k.svgPlotArea=function(){return arguments.length?(C=arguments.length<=0?void 0:arguments[0],k):C},k.decorate=function(){return arguments.length?(b=arguments.length<=0?void 0:arguments[0],k):b},k},ge=function(){for(var t=arguments.length,e=Array(t),n=0;n<t;n++)e[n]=arguments[n];var i={xScale:r.scaleIdentity(),yScale:r.scaleIdentity(),xAxis:{bottom:Rt,top:Lt},yAxis:{right:Ft,left:Nt}};return 1!==e.length||e[0].domain||e[0].range?Object.assign(i,{xScale:e[0]||i.xScale,yScale:e[1]||i.yScale}):Object.assign(i,e[0])},he=function(t){return"function"==typeof t?t:function(){return t}},de=function(t,e){return function(){for(var n=arguments.length,r=Array(n),i=0;i<n;i++)r[i]=arguments[i];var o=he(""),a=he(""),s=e,A=function(){},l=ue.apply(void 0,r),c=function(e){t(l,s),l.decorate(function(t,e,n){t.enter().select(".x-label").style("height","1em").style("line-height","1em");var r=l.yOrient()(e);t.enter().append("div").attr("class","y-label-container").style("grid-column","left"===r?1:5).style("-ms-grid-column","left"===r?1:5).style("grid-row",3).style("-ms-grid-row",3).style("width","1em").style("display","flex").style("align-items","center").style("justify-content","center").append("div").attr("class","y-label").style("transform","rotate(-90deg)"),t.select(".y-label-container>.y-label").text(a),t.select(".top-label").style("margin-top","2em"),t.enter().append("div").attr("class","chart-label").style("grid-column",3).style("-ms-grid-column",3).style("grid-row",1).style("-ms-grid-row",1).style("height","2em").style("line-height","2em").style("text-align","center"),t.select(".chart-label").text(o(e)),A(t,e,n)}),e.call(l)};return f(c,l,C(/^x/,/^y/)),c.chartLabel=function(){return arguments.length?(o=he(arguments.length<=0?void 0:arguments[0]),c):o},c.yLabel=function(){return arguments.length?(a=he(arguments.length<=0?void 0:arguments[0]),c):a},c.plotArea=function(){return arguments.length?(s=arguments.length<=0?void 0:arguments[0],c):s},c.decorate=function(){return arguments.length?(A=arguments.length<=0?void 0:arguments[0],c):A},c}},fe=de(function(t,e){return t.svgPlotArea(e)},ut),pe=de(function(t,e){return t.canvasPlotArea(e)},gt),Ie=function(t){return[t[1],t[0]]},Ce=function(t){var e=function(t){switch(t){case"x":return u.brushX();case"y":return u.brushY();case"xy":return u.brush()}}(t),n=c.dispatch("brush","start","end"),i=r.scaleIdentity(),o=r.scaleIdentity(),a=V("g","brush"),s=function(e,n,r){switch(t){case"x":return e.map(n);case"y":return e.map(r);case"xy":return[[n(e[0][0]),r(e[0][1])],[n(e[1][0]),r(e[1][1])]]}},l=function(e){var n=r.scaleLinear().domain(i.domain());return"x"===t?e.map(n.invert):"xy"===t?[n.invert(e[0][0]),n.invert(e[1][0])]:void 0},g=function(e){var n=r.scaleLinear().domain(Ie(o.domain()));return"y"===t?[e[1],e[0]].map(n.invert):"xy"===t?[n.invert(e[1][1]),n.invert(e[0][1])]:void 0},d=function(t){var e;if(!t.sourceEvent||"draw"!==t.sourceEvent.type)if(t.selection){var a=(e=t.selection,s(e,r.scaleLinear().domain(i.range()),r.scaleLinear().domain(Ie(o.range()))));n.call(t.type,{},{selection:a,xDomain:l(a),yDomain:g(a)})}else n.call(t.type,{},{})},f=function(t){t.each(function(t,n,l){e.extent([[i.range()[0],o.range()[1]],[i.range()[1],o.range()[0]]]),e.on("end",function(){return d(A.event)}).on("brush",function(){return d(A.event)}).on("start",function(){return d(A.event)}),a(A.select(l[n]),[t]).call(e).call(e.move,t?s(t,r.scaleLinear().domain(i.range()).invert,r.scaleLinear().domain(Ie(o.range())).invert):null)})};return f.xScale=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],f):i},f.yScale=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],f):o},h(f,n,"on"),h(f,e,"filter","handleSize"),f};e.indicatorBollingerBands=function(){var t=2,e=Q().accumulator(function(e){var r=e&&n.deviation(e),i=e&&n.mean(e);return{average:i,upper:b(i+t*r),lower:b(i-t*r)}}),r=function(t){return e(t)};return r.multiplier=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],r):t},h(r,e,"period","value"),r},e.indicatorExponentialMovingAverage=w,e.indicatorMacd=function(){var t=E,e=w().period(12),r=w().period(26),i=w().period(9),o=function(o){e.value(t),r.value(t);var a=n.zip(e(o),r(o)).map(function(t){return void 0!==t[0]&&void 0!==t[1]?t[0]-t[1]:void 0}),s=i(a);return n.zip(a,s).map(function(t){return{macd:t[0],signal:t[1],divergence:void 0!==t[0]&&void 0!==t[1]?t[0]-t[1]:void 0}})};return o.value=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],o):t},f(o,e,m({period:"fastPeriod"})),f(o,r,m({period:"slowPeriod"})),f(o,i,m({period:"signalPeriod"})),o},e.indicatorRelativeStrengthIndex=function(){var t=Q().period(14),e=function(t){var e=S(t,2),n=e[0],r=e[1];return n<r?0:n-r},r=function(t){var e=S(t,2),n=e[0],r=e[1];return n>r?0:r-n},i=function(t,e){return void 0!==e?(i=e)+((r=t)[r.length-1]-i)/r.length:n.mean(t);var r,i},o=function(o){var a,s,A,l=(a=void 0,s=void 0,A=void 0,function(t){if(t){if(void 0!==a){var o=n.pairs([a].concat(L(t)));return s=i(o.map(e),s),A=i(o.map(r),A),b(100-100/(1+(isNaN(a)?NaN:A/s)))}a=t[0]}else void 0!==a&&(a=NaN)});return t.accumulator(l),t(o)};return h(o,t,"period","value"),o},e.indicatorStochasticOscillator=function(){var t=function(t,e){return t.close},e=function(t,e){return t.high},r=function(t,e){return t.low},i=Q().period(5).defined(function(n){return null!=t(n)&&null!=e(n)&&null!=r(n)}).accumulator(function(i){var o=i&&n.max(i,e),a=i&&n.min(i,r);return b(i&&100*(t(i[i.length-1])-a)/(o-a))}),o=R().period(3),a=function(t){var e=i(t),n=o(e);return e.map(function(t,e){return{k:t,d:n[e]}})};return a.closeValue=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],a):t},a.highValue=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],a):e},a.lowValue=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],a):r},f(a,i,m({period:"kPeriod"})),f(a,o,m({period:"dPeriod"})),a},e.indicatorForceIndex=function(){var t=function(t,e){return t.volume},e=function(t,e){return t.close},n=w().period(13),r=Q().period(2).defined(function(n){return null!=e(n)&&null!=t(n)}).accumulator(function(n){return n&&b((e(n[1])-e(n[0]))*t(n[1]))}),i=function(t){var e=r(t);return n(e)};return i.volumeValue=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],i):t},i.closeValue=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],i):e},h(i,n,"period"),i},e.indicatorEnvelope=function(){var t=.1,e=E,n=function(n){return n.map(function(n){return{lower:b(e(n)*(1-t)),upper:b(e(n)*(1+t))}})};return n.factor=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],n):t},n.value=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],n):e},n},e.indicatorElderRay=function(){var t=function(t,e){return t.close},e=function(t,e){return t.high},r=function(t,e){return t.low},i=w().period(13),o=function(o){return i.value(t),n.zip(o,i(o)).map(function(t){return{bullPower:b(e(t[0])-t[1]),bearPower:b(r(t[0])-t[1])}})};return o.closeValue=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],o):t},o.highValue=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],o):e},o.lowValue=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],o):r},h(o,i,"period"),o},e.indicatorMovingAverage=R,e.scaleDiscontinuous=function t(e){var n=this;arguments.length||(e=r.scaleIdentity());var i=N(),o=function(t){var n=e.domain(),r=e.range(),o=i.distance(n[0],n[1]);return i.distance(n[0],t)/o*(r[1]-r[0])+r[0]};return o.invert=function(t){var n=e.domain(),r=e.range(),o=(t-r[0])/(r[1]-r[0])*i.distance(n[0],n[1]);return i.offset(n[0],o)},o.domain=function(){if(!arguments.length)return e.domain();var t=arguments.length<=0?void 0:arguments[0],n=i.clampUp(t[0]),r=i.clampDown(t[1]);return e.domain([n,r]),o},o.nice=function(){e.nice();var t=e.domain(),n=i.clampUp(t[0]),r=i.clampDown(t[1]);return e.domain([n,r]),o},o.ticks=function(){for(var t=arguments.length,r=Array(t),o=0;o<t;o++)r[o]=arguments[o];var a=e.ticks.apply(n,r);return F(a,i)},o.copy=function(){return t(e.copy()).discontinuityProvider(i.copy())},o.discontinuityProvider=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],o):i},f(o,e,C("range","rangeRound","interpolate","clamp","tickFormat")),o},e.discontinuitySkipWeekends=function(){var t={},e=function(t){return 0===t.getDay()||6===t.getDay()};return t.clampDown=function(t){if(t&&e(t)){var n=i.timeDay.ceil(t);return 0===n.getDay()?i.timeDay.offset(n,-1):1===n.getDay()?i.timeDay.offset(n,-2):n}return t},t.clampUp=function(t){if(t&&e(t)){var n=i.timeDay.floor(t);return 6===n.getDay()?i.timeDay.offset(n,2):0===n.getDay()?i.timeDay.offset(n,1):n}return t},t.distance=function(e,n){e=t.clampUp(e),n=t.clampDown(n);var r=i.timeSaturday.ceil(e);if(n<r)return n.getTime()-e.getTime();var o=r.getTime()-e.getTime(),a=i.timeSaturday.ceil(n),s=a.getTime()-n.getTime();return 432e6*Math.round((a.getTime()-r.getTime())/6048e5)+o-s},t.offset=function(n,r){var o=e(n)?t.clampUp(n):n;if(0===r)return o;var a=r<0,s=r>0,A=r,l=a?i.timeMonday.floor(o):i.timeSaturday.ceil(o);if(A-=l.getTime()-o.getTime(),a&&A>0||s&&A<0)return new Date(o.getTime()+r);o=a?l:i.timeDay.offset(l,2);var c=Math.floor(A/432e6);return o=i.timeDay.offset(o,7*c),A-=432e6*c,new Date(o.getTime()+A)},t.copy=function(){return t},t},e.discontinuityIdentity=N,e.discontinuityRange=function(){for(var t=arguments.length,e=Array(t),n=0;n<t;n++)e[n]=arguments[n];var r=function(t,e){return t>e[0]&&t<e[1]},i=function(t,e){return t[0]>=e[0]&&t[1]<=e[1]},o={distance:function(t,n){t=o.clampUp(t),n=o.clampDown(n);var r=e.filter(function(e){return i(e,[t,n])}).map(function(t){return t[1]-t[0]});return n-t-r.reduce(function(t,e){return t+e},0)}},a=function(t,e){return t instanceof Date?new Date(t.getTime()+e):t+e};return o.offset=function(t,n){if(n>0){var r=function(){for(var r=o.clampUp(t),i=n;i>0;){var s=e.filter(function(t){return t[0]>r}).sort(function(t,e){return t[0]-e[0]});if(s.length){var A=s[0],l=A[0]-r;l>i?(r=a(r,i),i=0):(r=A[1],i-=l)}else r=a(r,i),i=0}return{v:r}}();if("object"===(void 0===r?"undefined":x(r)))return r.v}else{var i=function(){for(var r=o.clampDown(t),i=n;i<0;){var s=e.filter(function(t){return t[1]<r}).sort(function(t,e){return e[0]-t[0]});if(s.length){var A=s[0],l=A[1]-r;l<i?(r=a(r,i),i=0):(r=A[0],i-=l)}else r=a(r,i),i=0}return{v:r}}();if("object"===(void 0===i?"undefined":x(i)))return i.v}},o.clampUp=function(t){return e.reduce(function(t,e){return r(t,e)?e[1]:t},t)},o.clampDown=function(t){return e.reduce(function(t,e){return r(t,e)?e[0]:t},t)},o.copy=function(){return o},o},e.extentLinear=T,e.extentTime=U,e.extentDate=U,e.randomFinancial=function(){var t=new Date,e=100,n=i.timeDay,r=1,a=i.timeYear,s=1,A=null,l=function(){var t=o.randomNormal(1,.1);return Math.ceil(1e3*t())},c=O(),u=function(t,e){var i,o,A=(i=t,o=a.offset(i,s)-i,(n.offset(i,r)-i)/o),u=c.period(A)(e),g={date:t,open:u[0],high:Math.max.apply(Math,u),low:Math.min.apply(Math,u),close:u[c.steps()]};return g.volume=l(g),g},g=function(i){var o=void 0,a=void 0,s=void 0;do{o=i?n.offset(i.date,r):new Date(t.getTime()),a=i?i.close:e,i=u(o,a),s=A&&!A(i)}while(s);return i},h=function(){var t=void 0,e={next:function(){var e=g(t);return t=e,e},take:function(t){return e.until(function(e,n){return!t||t<0||n===t})},until:function(e){for(var n=[],r=0,i=g(t),o=e&&!e(i,r);o;)n.push(i),i=g(t=i),r+=1,o=e&&!e(i,r);return n}};return e},d=function(t){return h().take(t)};if(d.stream=h,"function"!=typeof Symbol||"symbol"!==x(Symbol.iterator))throw new Error("d3fc-random-data depends on Symbol. Make sure that you load a polyfill in older browsers. See README.");return d[Symbol.iterator]=function(){var t=h();return{next:function(){return{value:t.next(),done:!1}}}},d.startDate=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],d):t},d.startPrice=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],d):e},d.interval=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],d):n},d.intervalStep=function(){return arguments.length?(r=arguments.length<=0?void 0:arguments[0],d):r},d.unitInterval=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],d):a},d.unitIntervalStep=function(){return arguments.length?(s=arguments.length<=0?void 0:arguments[0],d):s},d.filter=function(){return arguments.length?(A=arguments.length<=0?void 0:arguments[0],d):A},d.volume=function(){return arguments.length?(l=G(arguments.length<=0?void 0:arguments[0]),d):l},f(d,c),d},e.randomGeometricBrownianMotion=O,e.randomSkipWeekends=function(t){var e=t.date.getDay();return!(0===e||6===e)},e.feedGdax=function(){var t="BTC-USD",e=null,n=null,r=null,i=function(){var i=[];null!=e&&i.push("start="+e.toISOString()),null!=n&&i.push("end="+n.toISOString()),null!=r&&i.push("granularity="+r);var o="https://api.gdax.com/products/"+t+"/candles?"+i.join("&");return a.json(o).then(function(t){return t.map(function(t){return{date:new Date(1e3*t[0]),open:t[3],high:t[2],low:t[1],close:t[4],volume:t[5]}})})};return i.product=function(e){return arguments.length?(t=e,i):t},i.start=function(t){return arguments.length?(e=t,i):e},i.end=function(t){return arguments.length?(n=t,i):n},i.granularity=function(t){return arguments.length?(r=t,i):r},i},e.bucket=H,e.largestTriangleOneBucket=function(){var t=H(),e=function(t){return t},r=function(t){return t},i=function(i){if(t.bucketSize()>=i.length)return i;var o=function(t){var i=t.map(function(t){return[e(t),r(t)]});return n.range(1,i.length-1).map(function(t){var e=i[t-1],n=i[t],r=i[t+1];return.5*Math.abs((e[0]-r[0])*(n[1]-e[1])-(e[0]-n[0])*(r[1]-e[1]))})}(i),a=t(o),s=t(i.slice(1,i.length-1)).map(function(t,e){var r=a[e],i=n.max(r);return t[r.indexOf(i)]});return[].concat([i[0]],s,[i[i.length-1]])};return h(i,t,"bucketSize"),i.x=function(t){return arguments.length?(e=t,i):e},i.y=function(t){return arguments.length?(r=t,i):r},i},e.largestTriangleThreeBucket=function(){var t=function(t){return t},e=function(t){return t},r=H(),i=function(i){if(r.bucketSize()>=i.length)return i;var o=r(i.slice(1,i.length-1)),a=i[0],s=i[i.length-1],A=[].concat([a],o,[s]),l=t(a),c=e(a),u=o.map(function(r,i){var o=n.mean(A[i+1],t),a=n.mean(A[i+1],e),s=r.map(function(n){return[t(n),e(n)]}),u=s.map(function(t){return.5*Math.abs((l-o)*(t[1]-c)-(l-t[0])*(a-c))}),g=u.indexOf(n.max(u)),h=s[g];return l=h[0],c=h[1],r[g]});return[].concat([i[0]],u,[i[i.length-1]])};return h(i,r,"bucketSize"),i.x=function(e){return arguments.length?(t=e,i):t},i.y=function(t){return arguments.length?(e=t,i):e},i},e.modeMedian=function(){var t=H(),e=function(t){return t},r=function(r){if(t.bucketSize()>r.length)return r;var i=n.extent(r,e),o=t(r.slice(1,r.length-1)).map(function(t,n){var r,o,a={},s=!0,A=t.map(e),l=A.filter(function(t){return t===i[0]||t===i[1]}).map(function(t){return A.indexOf(t)})[0];return void 0!==l?t[l]:(A.forEach(function(t,e){void 0===a[t]&&(a[t]=0),a[t]++,a[t]>a[r]||void 0===r?(r=t,o=e,s=!0):a[t]===a[r]&&(s=!1)}),s?t[o]:t[Math.floor(t.length/2)])});return[].concat([r[0]],o,[r[r.length-1]])};return h(r,t,"bucketSize"),r.value=function(t){return arguments.length?(e=t,r):e},r},e.rebind=h,e.rebindAll=f,e.exclude=I,e.include=C,e.includeMap=m,e.prefix=y,e.shapeOhlc=Y,e.shapeBar=P,e.shapeCandlestick=K,e.shapeBoxPlot=z,e.shapeErrorBar=J,e.layoutLabel=function(t){var e=function(){},n=function(){return[0,0]},i=function(t,e){return[t.x,t.y]},o=t||function(t){return t},a=function(){},s=r.scaleIdentity(),l=r.scaleIdentity(),c=V("g","label"),u=function(t){t.each(function(t,r,u){var g=c(A.select(u[r]),t).call(a),h=g.nodes(),d=h.map(function(t,e){var r=A.select(t).datum(),o=i(r,e,h),a=[s(o[0]),l(o[1])],c=n(r,e,h);return{hidden:!1,x:a[0],y:a[1],width:c[0],height:c[1]}}),f=o(d);g.attr("style",function(t,e){return"display:"+(f[e].hidden?"none":"inherit")}).attr("transform",function(t,e){return"translate("+f[e].x+", "+f[e].y+")"}).attr("layout-width",function(t,e){return f[e].width}).attr("layout-height",function(t,e){return f[e].height}).attr("anchor-x",function(t,e,n){return d[e].x-f[e].x}).attr("anchor-y",function(t,e,n){return d[e].y-f[e].y}),g.call(a),e(g,t,r)})};return f(u,c,C("key")),f(u,o),u.size=function(){return arguments.length?(n=q(arguments.length<=0?void 0:arguments[0]),u):n},u.position=function(){return arguments.length?(i=q(arguments.length<=0?void 0:arguments[0]),u):i},u.component=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],u):a},u.decorate=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],u):e},u.xScale=function(){return arguments.length?(s=arguments.length<=0?void 0:arguments[0],u):s},u.yScale=function(){return arguments.length?(l=arguments.length<=0?void 0:arguments[0],u):l},u},e.layoutTextLabel=function(t){var e=2,n=function(t){return t},r=V("text"),i=V("rect"),o=V("circle"),a=function(t){t.each(function(t,a,s){var l=s[a],c=A.select(l),u=Number(l.getAttribute("layout-width")),g=Number(l.getAttribute("layout-height"));i(c,[t]).attr("width",u).attr("height",g);var h=Number(l.getAttribute("anchor-x")),d=Number(l.getAttribute("anchor-y"));o(c,[t]).attr("r",2).attr("cx",h).attr("cy",d);var f=r(c,[t]);f.enter().attr("dy","0.9em").attr("transform","translate("+e+", "+e+")"),f.text(n)})};return a.padding=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],a):e},a.value=function(){return arguments.length?(n=q(arguments.length<=0?void 0:arguments[0]),a):n},a},e.layoutGreedy=function(){var t=void 0,e=function(e,n,r){return X(r,n)+function(e){return t?e.width*e.height-W(e,t):0}(e)},n=function(t){var n=nt().locationScore(e).rectangles(t);return t.forEach(function(t,e){$(t).forEach(function(t,r){n=n(t,e)})}),n.rectangles()};return n.bounds=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],n):t},n},e.layoutAnnealing=function(){var t=1e3,e=1,n=void 0,r=function(t,e,r){return X(r,e)+function(t){return n?t.width*t.height-W(t,n):0}(t)+function(t){switch(t.location){case"bottom-right":return 0;case"middle-right":case"bottom-center":return t.width*t.height/8}return t.width*t.height/4}(t)},i=function(n){for(var i,o=t,a=nt().locationScore(r).winningScore(function(t,e){return Math.exp((e-t)/o)>Math.random()}).rectangles(n);o>0;){var s=rt(n);a=a((i=$(n[s]))[rt(i)],s),o-=e}return a.rectangles()};return i.temperature=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],i):t},i.cooling=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],i):e},i.bounds=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],i):n},i},e.layoutRemoveOverlaps=function(t){var e=function(e){e=t(e);for(var r=function(){var t,r,i=e.filter(function(t){return!t.hidden}),o=(r=function(t,e){return e[1]-t[1]},(t=i.map(function(t,e){return[t,X(i,e)]}))[n.scan(t,r)]);if(!(o[1]>0))return"break";o[0].hidden=!0};"break"!==r(););return e};return f(e,t=t||function(t){return t}),e},e.layoutBoundingBox=function(){var t=[0,0],e=function(e){return e.map(function(e,n){var r=e.x,i=e.y;return r+e.width>t[0]&&(r-=e.width),i+e.height>t[1]&&(i-=e.height),{height:e.height,width:e.width,x:r,y:i}})};return e.bounds=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],e):t},e},e.dataJoin=V,e.effectivelyZero=1e-6,e.seriesSvgLine=ut,e.seriesCanvasLine=gt,e.seriesSvgPoint=ht,e.seriesCanvasPoint=dt,e.seriesSvgBar=function(){var t=P().x(0).y(0),e=At(),n=V("g","bar"),r=function(t){return"vertical"===e.orient()?t.width:t.height},i=function(t){return"translate("+t[0]+", "+t[1]+")"},o=function(o){o.selection&&n.transition(o),o.each(function(o,a,s){var l=e.orient();if("vertical"!==l&&"horizontal"!==l)throw new Error("The bar series does not support an orientation of "+l);var c=o.filter(e.defined()),u=c.map(e.values);t.width(0).height(0),"vertical"===e.orient()?(t.verticalAlign("top"),t.horizontalAlign("center")):(t.horizontalAlign("right"),t.verticalAlign("center"));var g=n(A.select(s[a]),c);g.enter().attr("transform",function(t,e){return i(u[e].baseOrigin)}).attr("class","bar "+e.orient()).attr("fill","#999").append("path").attr("d",function(e,n){return r(t)(u[n].width),t([e])}),g.attr("transform",function(t,e){return i(u[e].origin)}).select("path").attr("d",function(n,i){var o;return r(t)(u[i].width),(o=t,"vertical"===e.orient()?o.height:o.width)(-u[i].height),t([n])}),e.decorate()(g,c,a)})};return f(o,e),h(o,n,"key"),o},e.seriesCanvasBar=function(){var t=At(),e=P().x(0).y(0),n=function(n){var r=e.context(),i=n.filter(t.defined()).map(t.values);"vertical"===t.orient()?(e.verticalAlign("top"),e.horizontalAlign("center")):(e.horizontalAlign("right"),e.verticalAlign("center")),i.forEach(function(n,i){var o;r.save(),r.beginPath(),r.translate(n.origin[0],n.origin[1]),(o=e,"vertical"===t.orient()?o.height:o.width)(-n.height),function(e){return"vertical"===t.orient()?e.width:e.height}(e)(n.width),e([n]),r.fillStyle="#999",r.strokeStyle="transparent",t.decorate()(r,n.d,i),r.fill(),r.stroke(),r.closePath(),r.restore()})};return f(n,t),h(n,e,"context"),n},e.seriesSvgErrorBar=function(){var t=ft(),e=V("g","error-bar"),n=J().value(0),r=function(t){return"translate("+t.origin[0]+", "+t.origin[1]+")"},i=function(i){i.selection&&e.transition(i);var o,a=(o=i,function(t){return o.selection?t.transition(o):t});i.each(function(i,o,s){var l=i.filter(t.defined()),c=l.map(t.values),u=e(A.select(s[o]),l);u.enter().attr("stroke",lt).attr("fill",ct).attr("transform",function(e,n){return r(t.values(e,n))+" scale(1e-6, 1)"}).append("path"),n.orient(t.orient()),u.each(function(t,e,i){var o=c[e];n.high(o.high).low(o.low).width(o.width),a(A.select(i[e])).attr("transform",r(o)+" scale(1)").select("path").attr("d",n([t]))}),t.decorate()(u,i,o)})};return f(i,t),h(i,e,"key"),i},e.seriesCanvasErrorBar=function(){var t=ft(),e=J().value(0),n=function(n){var r=n.filter(t.defined()),i=e.context();e.orient(t.orient()),r.forEach(function(n,r){i.save();var o=t.values(n,r);i.translate(o.origin[0],o.origin[1]),i.beginPath(),e.high(o.high).width(o.width).low(o.low)([n]),i.strokeStyle=lt,i.fillStyle=ct,t.decorate()(i,n,r),i.fill(),i.stroke(),i.closePath(),i.restore()})};return f(n,t),h(n,e,"context"),n},e.seriesSvgArea=function(){var t=At(),e=l.area(),n=V("path","area"),r=function(r){r.selection&&n.transition(r),e.defined(t.defined()),r.each(function(r,i,o){var a=r.map(t.values);e.x(function(t,e){return a[e].transposedX}).y(function(t,e){return a[e].transposedY});var s="vertical"===t.orient()?"y":"x";e[s+"0"](function(t,e){return a[e].y0}),e[s+"1"](function(t,e){return a[e].y});var l=n(A.select(o[i]),[r]);l.enter().attr("fill",ct),l.attr("d",e),t.decorate()(l,r,i)})};return f(r,t,I("bandwidth","align")),h(r,n,"key"),h(r,e,"curve"),r},e.seriesCanvasArea=function(){var t=At(),e=l.area(),n=function(n){var r=e.context();e.defined(t.defined());var i=n.map(t.values);e.x(function(t,e){return i[e].transposedX}).y(function(t,e){return i[e].transposedY});var o="vertical"===t.orient()?"y":"x";e[o+"0"](function(t,e){return i[e].y0}),e[o+"1"](function(t,e){return i[e].y}),r.beginPath(),e(n),r.fillStyle=ct,r.strokeStyle="transparent",t.decorate()(r,n),r.fill(),r.stroke(),r.closePath()};return f(n,t,I("bandwidth","align")),h(n,e,"curve","context"),n},e.seriesSvgCandlestick=function(){return It(K(),"candlestick")},e.seriesCanvasCandlestick=function(){return Ct(K())},e.seriesSvgBoxPlot=function(){var t=mt(),e=V("g","box-plot"),n=z().value(0),r=function(t){return"translate("+t.origin[0]+", "+t.origin[1]+")"},i=function(i){i.selection&&e.transition(i);var o,a=(o=i,function(t){return o.selection?t.transition(o):t});i.each(function(i,o,s){var l=i.filter(t.defined()),c=e(A.select(s[o]),l);c.enter().attr("stroke",lt).attr("fill",ct).attr("transform",function(e,n){return r(t.values(e,n))+" scale(1e-6, 1)"}).append("path"),n.orient(t.orient()),c.each(function(e,i,o){var s=t.values(e,i);n.median(s.median).upperQuartile(s.upperQuartile).lowerQuartile(s.lowerQuartile).width(s.width).high(s.high).low(s.low),a(A.select(o[i])).attr("transform",r(s)).select("path").attr("d",n([e]))}),t.decorate()(c,i,o)})};return f(i,t),h(i,e,"key"),h(i,n,"cap"),i},e.seriesCanvasBoxPlot=function(){var t=mt(),e=z().value(0),n=function(n){var r=n.filter(t.defined()),i=e.context();e.orient(t.orient()),r.forEach(function(n,r){i.save();var o=t.values(n,r);i.translate(o.origin[0],o.origin[1]),i.beginPath(),e.median(o.median).upperQuartile(o.upperQuartile).lowerQuartile(o.lowerQuartile).high(o.high).width(o.width).low(o.low)([n]),i.fillStyle=ct,i.strokeStyle=lt,t.decorate()(i,n,r),i.fill(),i.stroke(),i.closePath(),i.restore()})};return f(n,t),h(n,e,"cap","context"),n},e.seriesSvgOhlc=function(){return It(Y(),"ohlc")},e.seriesCanvasOhlc=function(){return Ct(Y())},e.seriesSvgMulti=Et,e.seriesCanvasMulti=vt,e.seriesSvgGrouped=function(t){var e=Bt(),n=V("g","grouped"),r=function(r){r.selection&&n.transition(r),r.each(function(r,i,o){var a=n(A.select(o[i]),r);a.enter().append("g"),a.select("g").each(function(n,i,o){var a=A.select(o[i]),s="horizontal"!==t.orient(),l=function(t,n){var o=e.offsetScaleForDatum(r,t,n);return(s?e.xScale():e.yScale())(t)+o(i)+o.bandwidth()/2};s?(t.xScale(l),t.yScale(e.yScale())):(t.yScale(l),t.xScale(e.xScale())),t.bandwidth&&t.bandwidth(function(t,n){return e.offsetScaleForDatum(r,t,n).bandwidth()}),t.decorate(function(t,n){return e.decorate()(t,n,i)}),a.call(t)})})};return f(r,t,I("decorate","xScale","yScale")),f(r,e,I("offsetScaleForDatum")),r},e.seriesCanvasGrouped=function(t){var e=Bt(),n=function(n){n.forEach(function(r,i){var o="horizontal"!==t.orient(),a=function(t,r){var a=e.offsetScaleForDatum(n,t,r);return(o?e.xScale():e.yScale())(t)+a(i)+a.bandwidth()/2};o?(t.xScale(a),t.yScale(e.yScale())):(t.yScale(a),t.xScale(e.xScale())),t.bandwidth&&t.bandwidth(function(t,r){return e.offsetScaleForDatum(n,t,r).bandwidth()}),t.decorate(function(t,n){return e.decorate()(t,n,i)}),t(r)})};return f(n,t,I("decorate","xScale","yScale")),f(n,e,I("offsetScaleForDatum")),n},e.seriesSvgRepeat=function(){var t="vertical",e=ut(),n=Et(),r=function(r){return r.each(function(r,i,o){"vertical"===t?n.series(r[0].map(function(t){return e})).mapping(function(t,e){return t.map(function(t){return t[e]})}):n.series(r.map(function(t){return e})).mapping(function(t,e){return t[e]}),A.select(o[i]).call(n)})};return r.series=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],r):e},r.orient=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],r):t},f(r,n,I("series","mapping")),r},e.seriesCanvasRepeat=function(){var t="vertical",e=gt(),n=vt(),r=function(r){"vertical"===t?n.series(r[0].map(function(t){return e})).mapping(function(t,e){return t.map(function(t){return t[e]})}):n.series(r.map(function(t){return e})).mapping(function(t,e){return t[e]}),n(r)};return r.series=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],r):e},r.orient=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],r):t},f(r,n,I("series","mapping")),r},e.autoBandwidth=function(t){var e=.75,r=function(t){if(t.length<=1)return 10;t=t.sort(n.ascending).filter(function(t,e,n){return-1===n.indexOf(t,e+1)});var r=n.pairs(t).map(function(t){return Math.abs(t[0]-t[1])}),i=n.min(r);return e*i},i=function(e,n,i){if(e.bandwidth)return e.bandwidth();var o,a=(Array.isArray(n)?(o=[]).concat.apply(o,L(n)):n).filter(t.defined()).map(i()).map(e);return r(a)},o=function(e){var n=function(e){if(t.xBandwidth&&t.yBandwidth)t.xBandwidth(i(t.xScale(),e,t.xValue)),t.yBandwidth(i(t.yScale(),e,t.yValue));else{var n=t.orient&&"horizontal"===t.orient()?t.yScale():t.xScale();t.bandwidth(i(n,e,t.crossValue))}};e instanceof A.selection?e.each(function(e,r,i){n(e),t(A.select(i[r]))}):(n(e),t(e))};return f(o,t),o.widthFraction=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],o):e},o},e.seriesSvgHeatmap=function(){var t=bt(),e=V("g","box"),n=function(n){n.each(function(n,r,i){var o=n.filter(t.defined()),a=t.colorValue(),s=t.colorInterpolate(),l=t.colorScale(o),c=e(A.select(i[r]),o);c.enter().append("path").attr("stroke","transparent"),c.attr("transform",function(e,n){return"translate("+(r=t.values(e,n)).x+", "+r.y+")";var r}).select("path").attr("d",function(e,n){return t.pathGenerator.width(t.values(e,n).width).height(t.values(e,n).height)([e])}).attr("fill",function(t,e){return s(l(a(t,e)))}),t.decorate()(c,n,r)})};return f(n,t),n},e.seriesCanvasHeatmap=function(){var t=bt(),e=function(e){var n=e.filter(t.defined()),r=(t.colorValue(),t.colorInterpolate()),i=t.colorScale(n),o=t.pathGenerator.context();n.forEach(function(e,n){o.save(),o.beginPath();var a=t.values(e,n);o.translate(a.x,a.y),o.fillStyle=r(i(a.colorValue)),o.strokeStyle="transparent",t.pathGenerator.height(a.height).width(a.width)([e]),t.decorate()(o,e,n),o.fill(),o.stroke(),o.closePath(),o.restore()})};return h(e,t.pathGenerator,"context"),f(e,t),e},e.annotationSvgBand=function(){var t=r.scaleIdentity(),e=r.scaleIdentity(),n="horizontal",i=function(t){return t.from},o=function(t){return t.to},a=function(){},s=V("g","annotation-band"),l=P().horizontalAlign("center").verticalAlign("center").x(0).y(0),c=function(r){if(r.selection&&s.transition(r),"horizontal"!==n&&"vertical"!==n)throw new Error("Invalid orientation");var c="horizontal"===n,u=c?function(t,e){return"translate("+t+", "+e+")"}:function(t,e){return"translate("+e+", "+t+")"},g=c?e:t,h=(c?t:e).range(),d=h[1]-h[0],f=c?"height":"width",p=function(){return u((h[1]+h[0])/2,(g(o.apply(void 0,arguments))+g(i.apply(void 0,arguments)))/2)};l[c?"width":"height"](d),l[f](function(){return g(o.apply(void 0,arguments))-g(i.apply(void 0,arguments))}),r.each(function(t,e,r){var i=s(A.select(r[e]),t);i.enter().attr("transform",p).append("path").classed("band",!0),i.attr("class","annotation-band "+n).attr("transform",p).select("path").attr("d",function(t,e){return l([t],e)}),a(i,t,e)})};return c.xScale=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],c):t},c.yScale=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],c):e},c.orient=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],c):n},c.decorate=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],c):a},c.fromValue=function(){return arguments.length?(i=Qt(arguments.length<=0?void 0:arguments[0]),c):i},c.toValue=function(){return arguments.length?(o=Qt(arguments.length<=0?void 0:arguments[0]),c):o},c},e.annotationCanvasBand=function(){var t=r.scaleIdentity(),e=r.scaleIdentity(),n="horizontal",i=function(t){return t.from},o=function(t){return t.to},a=function(){},s=P().horizontalAlign("right").verticalAlign("top"),A=function(r){if("horizontal"!==n&&"vertical"!==n)throw new Error("Invalid orientation");var A=s.context(),l="horizontal"===n,c=l?e:t,u=(l?t:e).range(),g=u[1]-u[0],h=l?"x":"y",d=l?"y":"x",f=l?"height":"width",p=l?"width":"height";r.forEach(function(t,e){A.save(),A.beginPath(),A.strokeStyle="transparent",s[d](c(i(t))),s[h](u[0]),s[p](g),s[f](c(o(t))-c(i(t))),a(A,t,e),s.context(A)([t],e),A.fill(),A.stroke(),A.closePath(),A.restore()})};return A.xScale=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],A):t},A.yScale=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],A):e},A.orient=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],A):n},A.decorate=function(){return arguments.length?(a=arguments.length<=0?void 0:arguments[0],A):a},A.fromValue=function(){return arguments.length?(i=Qt(arguments.length<=0?void 0:arguments[0]),A):i},A.toValue=function(){return arguments.length?(o=Qt(arguments.length<=0?void 0:arguments[0]),A):o},h(A,s,"context"),A},e.annotationSvgCrosshair=function(){var t=function(t){return t.x},e=function(t){return t.y},n=r.scaleIdentity(),i=r.scaleIdentity(),o=function(){},a=V("g","annotation-crosshair"),s=ht(),l=wt(),c=wt().orient("vertical"),u=r.scaleIdentity(),g=r.scaleIdentity(),h=Et().series([l,c,s]).xScale(u).yScale(g).mapping(function(t){return[t]}),d=function(r){r.selection&&a.transition(r),r.each(function(r,d,f){var p=a(A.select(f[d]),r);p.enter().style("pointer-events","none"),u.range(n.range()),g.range(i.range()),s.crossValue(t).mainValue(e),l.value(e),c.value(t),p.call(h),o(p,r,d)})};d.x=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],d):t},d.y=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],d):e},d.xScale=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],d):n},d.yScale=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],d):i},d.decorate=function(){return arguments.length?(o=arguments.length<=0?void 0:arguments[0],d):o};var p=C("label");return f(d,l,p,y("y")),f(d,c,p,y("x")),d},e.annotationCanvasCrosshair=function(){var t=function(t){return t.x},e=function(t){return t.y},n=r.scaleIdentity(),i=r.scaleIdentity(),o=dt(),a=xt(),s=xt().orient("vertical"),A=r.scaleIdentity(),l=r.scaleIdentity(),c=vt().series([a,s,o]).xScale(A).yScale(l).mapping(function(t){return[t]}),u=function(r){r.forEach(function(r){A.range(n.range()),l.range(i.range()),o.crossValue(t).mainValue(e),a.value(e),s.value(t),c(r)})};u.x=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],u):t},u.y=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],u):e},u.xScale=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],u):n},u.yScale=function(){return arguments.length?(i=arguments.length<=0?void 0:arguments[0],u):i};var g=C("label","decorate");return f(u,a,g,y("y")),f(u,s,g,y("x")),h(u,o,"decorate"),h(u,c,"context"),u},e.annotationSvgLine=wt,e.annotationCanvasLine=xt,e.annotationSvgGridline=function(){var t=function(){},e=function(){},n=_t(),r=_t(),i=V("line","gridline-y").key(Dt),o=V("line","gridline-x").key(Dt),a=function(a){a.selection&&(i.transition(a),o.transition(a)),a.each(function(a,s,l){var c=l[s],u=A.select(l[s]),g=n.scale(),h=r.scale(),d=c.__x_scale__||g;c.__x_scale__=g.copy();var f=n(),p=i(u,f);p.enter().attr("x1",d).attr("x2",d).attr("y1",h.range()[0]).attr("y2",h.range()[1]),p.attr("x1",g).attr("x2",g).attr("y1",h.range()[0]).attr("y2",h.range()[1]).attr("stroke","#bbb"),p.exit().attr("x1",g).attr("x2",g),t(p,f,s);var I=c.__y_scale__||h;c.__y_scale__=h.copy();var C=r(),m=o(u,C);m.enter().attr("y1",I).attr("y2",I).attr("x1",g.range()[0]).attr("x2",g.range()[1]),m.attr("y1",h).attr("y2",h).attr("x1",g.range()[0]).attr("x2",g.range()[1]).attr("stroke","#bbb"),m.exit().attr("y1",h).attr("y2",h),e(m,C,s)})};return a.yDecorate=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],a):e},a.xDecorate=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],a):t},f(a,i,m({key:"xKey"})),f(a,o,m({key:"yKey"})),f(a,n,y("x")),f(a,r,y("y")),a},e.annotationCanvasGridline=function(){var t=function(){},e=function(){},n=_t(),r=_t(),i=l.line(),o=function(){var o=i.context(),a=n.scale(),s=r.scale();n().forEach(function(e,n){o.save(),o.beginPath(),o.strokeStyle="#bbb",o.fillStyle="transparent",t(o,e,n),i.context(o)(s.domain().map(function(t){return[a(e),s(t)]})),o.fill(),o.stroke(),o.closePath(),o.restore()}),r().forEach(function(t,n){o.save(),o.beginPath(),o.strokeStyle="#bbb",o.fillStyle="transparent",e(o,t,n),i.context(o)(a.domain().map(function(e){return[a(e),s(t)]})),o.fill(),o.stroke(),o.closePath(),o.restore()})};return o.yDecorate=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],o):e},o.xDecorate=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],o):t},f(o,n,y("x")),f(o,r,y("y")),h(o,i,"context"),o},e.axisLabelRotate=function(t){var e="auto",n=function(){},r=function(){return"left"===t.orient()||"right"===t.orient()},i=function(){return"top"===t.orient()||"left"===t.orient()?-1:1},o=function(){switch(t.orient()){case"top":case"right":return"start";default:return"end"}},a=function(n){var a=function(n){var a=Ut(t.scale())(n),s=a.maxHeight,A=a.maxWidth,l=a.labelCount*A,c=void 0;if("auto"===e){var u=t.scale().range()[1];c=u<l?90*Math.min(1,(l/u-.8)/2):0}else c=e;return{rotate:r()?Math.floor(i()*(90-c)):Math.floor(-c),maxHeight:s,maxWidth:A,anchor:c?o():"middle"}}(n),s=a.rotate,A=a.maxHeight,l=a.anchor,c=n.select("text"),u=c.attr("transform"),g=i()*Math.floor(A/2),h=r()?"translate("+g+", 0)":"translate(0, "+g+")";c.style("text-anchor",l).attr("transform",u+" "+h+" rotate("+s+" 0 0)")},s=function(e){t(e)};return t.decorate(function(t){a(t),n(t)}),s.decorate=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],s):n},s.labelRotate=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],s):e},f(s,t,I("decorate")),s},e.axisLabelOffset=function(t){var e="auto",n=function(){},r=function(){return"left"===t.orient()||"right"===t.orient()},i=function(){return"top"===t.orient()||"left"===t.orient()?-1:1},o=function(n){var o=Ut(t.scale())(n),a=o.maxHeight,s=o.maxWidth,A=o.labelCount,l=t.scale().range()[1],c="auto"===e?Math.floor((r()?a:s)*A/l)+1:e,u=n.select("text"),g=u.attr("transform");u.attr("transform",function(t,e){return g+" "+function(t){return r()?"translate("+t%c*s*i()+", 0)":"translate(0, "+t%c*a*i()+")"}(e)})},a=function(e){return t(e)};return t.decorate(function(t){o(t),n(t)}),a.decorate=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],a):n},a.labelOffsetDepth=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],a):e},f(a,t,I("decorate")),a},e.axisTop=Lt,e.axisBottom=Rt,e.axisLeft=Nt,e.axisRight=Ft,e.axisOrdinalTop=function(t){return Tt("top",t)},e.axisOrdinalBottom=function(t){return Tt("bottom",t)},e.axisOrdinalLeft=function(t){return Tt("left",t)},e.axisOrdinalRight=function(t){return Tt("right",t)},e.pointer=function(){var t=c.dispatch("point");function e(){var e=A.mouse(this);t.call("point",this,[{x:e[0],y:e[1]}])}function n(){t.call("point",this,[])}var r=function(t){t.on("mouseenter.pointer",e).on("mousemove.pointer",e).on("mouseleave.pointer",n)};return h(r,t,"on"),r},e.group=function(){var t="",e="vertical",n=function(t,e){return Number(t[e])},r=function(r){return"vertical"===e?function(e){return Object.keys(e[0]).filter(function(e){return e!==t}).map(function(r){var i=e.filter(function(t){return t[r]}).map(function(e){var i=[e[t],n(e,r)];return i.data=e,i});return i.key=r,i})}(r):r.map(function(e){var r=Object.keys(e).filter(function(e){return e!==t}).map(function(t){var r=[t,n(e,t)];return r.data=e,r});return r.key=e[t],r})};return r.key=function(){return arguments.length?(t=arguments.length<=0?void 0:arguments[0],r):t},r.value=function(){return arguments.length?(n=arguments.length<=0?void 0:arguments[0],r):n},r.orient=function(){return arguments.length?(e=arguments.length<=0?void 0:arguments[0],r):e},r},e.chartSvgCartesian=fe,e.chartCanvasCartesian=pe,e.chartCartesian=ue,e.brushX=function(){return Ce("x")},e.brushY=function(){return Ce("y")},e.brush=function(){return Ce("xy")},Object.defineProperty(e,"__esModule",{value:!0})}(e,n(2),n(288),n(1),n(85),n(86),n(8),n(0),n(84),n(12),n(87))}).call(this,n(21))},function(t,e){var n={};n.parse=function(){var t={linearGradient:/^(\-(webkit|o|ms|moz)\-)?(linear\-gradient)/i,repeatingLinearGradient:/^(\-(webkit|o|ms|moz)\-)?(repeating\-linear\-gradient)/i,radialGradient:/^(\-(webkit|o|ms|moz)\-)?(radial\-gradient)/i,repeatingRadialGradient:/^(\-(webkit|o|ms|moz)\-)?(repeating\-radial\-gradient)/i,sideOrCorner:/^to (left (top|bottom)|right (top|bottom)|left|right|top|bottom)/i,extentKeywords:/^(closest\-side|closest\-corner|farthest\-side|farthest\-corner|contain|cover)/,positionKeywords:/^(left|center|right|top|bottom)/i,pixelValue:/^(-?(([0-9]*\.[0-9]+)|([0-9]+\.?)))px/,percentageValue:/^(-?(([0-9]*\.[0-9]+)|([0-9]+\.?)))\%/,emValue:/^(-?(([0-9]*\.[0-9]+)|([0-9]+\.?)))em/,angleValue:/^(-?(([0-9]*\.[0-9]+)|([0-9]+\.?)))deg/,startCall:/^\(/,endCall:/^\)/,comma:/^,/,hexColor:/^\#([0-9a-fA-F]+)/,literalColor:/^([a-zA-Z]+)/,rgbColor:/^rgb/i,rgbaColor:/^rgba/i,number:/^(([0-9]*\.[0-9]+)|([0-9]+\.?))/},e="";function n(t){var n=new Error(e+": "+t);throw n.source=e,n}function r(){return i("linear-gradient",t.linearGradient,a)||i("repeating-linear-gradient",t.repeatingLinearGradient,a)||i("radial-gradient",t.radialGradient,s)||i("repeating-radial-gradient",t.repeatingRadialGradient,s)}function i(e,r,i){return o(r,function(r){var o=i();return o&&(I(t.comma)||n("Missing comma before color stops")),{type:e,orientation:o,colorStops:u(g)}})}function o(e,r){var i=I(e);if(i)return I(t.startCall)||n("Missing ("),result=r(i),I(t.endCall)||n("Missing )"),result}function a(){return p("directional",t.sideOrCorner,1)||p("angular",t.angleValue,1)}function s(){var n,r,i=A();return i&&((n=[]).push(i),r=e,I(t.comma)&&((i=A())?n.push(i):e=r)),n}function A(){var t=function(){var t=p("shape",/^(circle)/i,0);return t&&(t.style=f()||l()),t}()||function(){var t=p("shape",/^(ellipse)/i,0);return t&&(t.style=d()||l()),t}();if(t)t.at=function(){if(p("position",/^at/,0)){var t=c();return t||n("Missing positioning value"),t}}();else{var e=c();e&&(t={type:"default-radial",at:e})}return t}function l(){return p("extent-keyword",t.extentKeywords,1)}function c(){var t={x:d(),y:d()};if(t.x||t.y)return{type:"position",value:t}}function u(e){var r=e(),i=[];if(r)for(i.push(r);I(t.comma);)(r=e())?i.push(r):n("One extra comma");return i}function g(){var e=p("hex",t.hexColor,1)||o(t.rgbaColor,function(){return{type:"rgba",value:u(h)}})||o(t.rgbColor,function(){return{type:"rgb",value:u(h)}})||p("literal",t.literalColor,0);return e||n("Expected color definition"),e.length=d(),e}function h(){return I(t.number)[1]}function d(){return p("%",t.percentageValue,1)||p("position-keyword",t.positionKeywords,1)||f()}function f(){return p("px",t.pixelValue,1)||p("em",t.emValue,1)}function p(t,e,n){var r=I(e);if(r)return{type:t,value:r[n]}}function I(t){var n,r;return(r=/^[\n\r\t\s]+/.exec(e))&&C(r[0].length),(n=t.exec(e))&&C(n[0].length),n}function C(t){e=e.substr(t)}return function(t){return e=t.toString(),function(){var t=u(r);return e.length>0&&n("Invalid input not EOF"),t}()}}(),e.parse=(n||{}).parse},function(t,e,n){"use strict";function r(t){const e=document.createElement("div");return e.innerHTML=t,Array.prototype.slice.call(e.children)[0]}function i(t,e,n){const i=r(t);!function(t){if(t.content)return;let e;for(t.content=document.createDocumentFragment();e=t.firstChild;)Node.prototype.appendChild.call(t.content,e)}(i),e&&(i.innerHTML="<style>".concat(e.toString(),"</style>")+i.innerHTML),i.innerHTML='<style id="psp_styles" scope="'.concat(i.getAttribute("id"),'">test{}</style>')+i.innerHTML;var o=!0,a=!1,s=void 0;try{for(var A,l=Object.getOwnPropertyNames(n.prototype)[Symbol.iterator]();!(o=(A=l.next()).done);o=!0){let t=A.value,e=Object.getOwnPropertyDescriptor(n.prototype,t);if(e&&e.set){let r=e.set;e.set=function(e){this.getAttribute(t)===e?(this._initializing||this._initialized)&&r.call(this,e):this.setAttribute(t,e)},Object.defineProperty(n.prototype,t,e)}}}catch(t){a=!0,s=t}finally{try{o||null==l.return||l.return()}finally{if(a)throw s}}let c=i.getAttribute("id");console.log("Registered ".concat(c)),window.customElements.define(c,class extends n{attributeChangedCallback(t,e,n){"_"!==t[0]&&e!=n&&(this[t]=n)}connectedCallback(){if(!this._initialized){for(this._initializing=!0,this._old_children=[];this.hasChildNodes();)1===this.lastChild.nodeType&&this._old_children.push(this.lastChild),this.removeChild(this.lastChild);this._old_children=this._old_children.reverse();var t=document.importNode(i.content,!0);this.attachShadow({mode:"open"}),this.shadowRoot.appendChild(t),super.connectedCallback&&super.connectedCallback();var e=!0,r=!1,o=void 0;try{for(var a,s=Object.getOwnPropertyNames(n.prototype)[Symbol.iterator]();!(e=(a=s.next()).done);e=!0){let t=a.value;"connectedCallback"!==t&&this.hasAttribute(t)&&"_"!==t[0]&&(this[t]=this.getAttribute(t))}}catch(t){r=!0,o=t}finally{try{e||null==s.return||s.return()}finally{if(r)throw o}}this._initializing=!1,this._initialized=!0}}static get observedAttributes(){return Object.getOwnPropertyNames(n.prototype)}})}function o(t){for(var e=arguments.length,n=new Array(e>1?e-1:0),r=1;r<e;r++)n[r-1]=arguments[r];const o=n.map(t=>t.toString()).join("\n");return function(e){return i(t,{toString:()=>o},e)}}function a(t){return function(e,n,r){const i=r.value;return r.set=function(e){let r=this.getAttribute(n);try{if(null==e&&(e=t()),"string"!=typeof e&&(e=JSON.stringify(e)),e!==r)return this.setAttribute(n,e),void(r=e);r=JSON.parse(r)}catch(i){console.error('Invalid value for attribute "'.concat(n,'": ').concat(e)),r=t()}i.call(this,r)},r.get=function(){if(this.hasAttribute(n))return JSON.parse(this.getAttribute(n))},delete r.value,delete r.writable,r}}function s(t){let e=document.createElement("textarea");document.body.appendChild(e),e.value=t,e.select(),document.execCommand("copy"),document.body.removeChild(e)}n.r(e),n.d(e,"importTemplate",function(){return r}),n.d(e,"registerElement",function(){return i}),n.d(e,"bindTemplate",function(){return o}),n.d(e,"copy_to_clipboard",function(){return s}),n.d(e,"json_attribute",function(){return A}),n.d(e,"array_attribute",function(){return l}),n(77),n(78),n(29),n(76);const A=a(()=>({})),l=a(()=>[])},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(88),i=n(33),o=n(64),a=n(28),s=n(93),A=n(92),l=n(26),c=n(284),u=n(285);e.RegisterGridCommands=function(t,e,n,a){const{commands:l,contextMenu:c}=t;if(null==l)return;const u={EditColumnProperties:n+"mavenworks:edit-column-properties",CopyColumnProperties:n+"mavenworks:copy-column-properties",PasteColumnProperties:n+"mavenworks:paste-column-properties",AddNewColumn:n+"mavenworks:table-editor:add-new",EditColumn:n+"mavenworks:table-editor:edit-name",DeleteColumn:n+"mavenworks:table-editor:delete-column",DeleteRow:n+"mavenworks:table-editor:delete-row",EditTableAsCsv:n+"mavenworks:table-editor:edit-as-csv"};l.addCommand(u.EditColumnProperties,{label:"Edit Column Properties",isEnabled:()=>{const t=e();return null!=t&&t instanceof r.SlickGridPart&&null!=t.lastColumn},execute:async()=>{const t=e();if(null==t||!(t instanceof r.SlickGridPart)||null==t.lastColumn)return;const n=t.lastColumn;if(!n)return;const a=t.getColumnFormatting(n),A=i.getFormattingMetadata();let l=i.getFormattingWithDefaults(a);const c=new class extends o.ReactWrapperWidget{render(){return s.createElement(o.PropertiesEditor,{properties:A,renderEditor:this.renderEditor.bind(this)})}renderEditor(t,e){return s.createElement(o.TypeEditor,{key:t,value:l[t],type:e.type,schema:e.schema,onValueChanged:e=>{l={...l,[t]:e},this.update()}})}};c.node.style.overflowY="auto",await o.HoverManager.GetManager().launchEditorDialog(c,t,400,800,"Column Properties",()=>{const e=i.stripDefaultsFromFormatting(l);t.setColumnFormatting(n,e)})}}),c.addItem({command:u.EditColumnProperties,selector:a+" .m-SlickGridPart:not(.m-TableEditorPart) .slick-header-column"}),l.addCommand(u.CopyColumnProperties,{label:"Copy Column Properties",isEnabled:()=>{const t=e();return null!=t&&t instanceof r.SlickGridPart&&null!=t.lastColumn},execute:async()=>{const t=e();if(null==t||!(t instanceof r.SlickGridPart)||null==t.lastColumn)return;const n=t.lastColumn;if(!n)return;const o=t.getColumnFormatting(n);localStorage.setItem("mavenworks:column-properties",i.serializeFormatting({buffer:o}))}}),c.addItem({command:u.CopyColumnProperties,selector:a+" .m-SlickGridPart:not(.m-TableEditorPart) .slick-header-column"}),l.addCommand(u.PasteColumnProperties,{label:"Paste Column Properties",isEnabled:()=>{const t=e();return null!=t&&t instanceof r.SlickGridPart&&null!=t.lastColumn},execute:async()=>{const t=e();if(null==t||!(t instanceof r.SlickGridPart)||null==t.lastColumn)return;const n=t.lastColumn;if(!n)return;const o=localStorage.getItem("mavenworks:column-properties");if(null==o)return;const a=i.parseFormatting(o).buffer;t.setColumnFormatting(n,a)}}),c.addItem({command:u.PasteColumnProperties,selector:a+" .m-SlickGridPart:not(.m-TableEditorPart) .slick-header-column"}),l.addCommand(u.AddNewColumn,{label:"Add New Column",isEnabled:()=>{const t=e();return null!=t&&t instanceof A.TableEditorPart},execute:async()=>{const t=e();if(null==t||!(t instanceof A.TableEditorPart))return;const n=new g("New column","Any");await o.HoverManager.GetManager().launchEditorDialog(n,t,500,500,"Add New Column",e=>{t.addNewColumn(e.name,e.type)})}}),c.addItem({command:u.AddNewColumn,selector:a+" .m-TableEditorPart .slick-header"}),l.addCommand(u.EditColumn,{label:"Edit Column",isEnabled:()=>{const t=e();return null!=t&&t instanceof A.TableEditorPart&&null!=t.activeColumn},execute:async()=>{const t=e();if(null==t||!(t instanceof A.TableEditorPart)||null==t.activeColumn)return;const n=t.activeColumn;if(!n)return;const r=new g(n,t.getTypeForCol(n).serializableName);await o.HoverManager.GetManager().launchEditorDialog(r,t,500,500,"Edit Column",e=>t.changeColumnName(n,e.name,e.type))}}),c.addItem({command:u.EditColumn,selector:a+" .m-TableEditorPart .slick-header-column"}),l.addCommand(u.DeleteColumn,{label:"Delete Column",isEnabled:()=>{const t=e();return null!=t&&t instanceof A.TableEditorPart&&null!=t.activeColumn},execute:async()=>{const t=e();if(null==t||!(t instanceof A.TableEditorPart)||null==t.activeColumn)return;const n=t.activeColumn;n&&t.deleteColumn(n)}}),c.addItem({command:u.DeleteColumn,selector:a+" .m-TableEditorPart .slick-header-column"}),c.addItem({type:"separator",selector:a+" .m-TableEditorPart .slick-pane-header"}),l.addCommand(u.DeleteRow,{label:"Delete Row",isEnabled:()=>{const t=e();return null!=t&&t instanceof A.TableEditorPart&&null!=t.activeCell},execute:async()=>{const t=e();if(null==t||!(t instanceof A.TableEditorPart)||null==t.activeCell)return;const n=t.activeCell;n&&t.deleteRow(n.row)}}),c.addItem({command:u.DeleteRow,selector:a+" .m-TableEditorPart .slick-row"}),l.addCommand(u.EditTableAsCsv,{label:"Edit Table as CSV",isEnabled:()=>{const t=e();return null!=t&&t instanceof A.TableEditorPart},execute:async()=>{const t=e();if(null==t||!(t instanceof A.TableEditorPart))return;const n=t.toCsv(),r=new h(n);await o.HoverManager.GetManager().launchEditorDialog(r,t,600,600,"Edit CSV",e=>t.fromCsv(""+e))}}),c.addItem({command:u.EditTableAsCsv,selector:a+" .m-TableEditorPart"})};class g extends a.Widget{constructor(t,e){super(),this.node.innerHTML=`\n        <span>\n            <label for="colName">Column Name</label>\n            <input type="text" value="${t}" name="colName" />\n        </span>\n        <span>\n            <label for="colType">Column Type</label>\n            <select value="${e}" name="colType">\n                ${l.Types.registered.map(t=>`<option ${t.serializableName===e?"selected":""}>\n                        ${t.serializableName}\n                    </option>`).join("\n")}\n            </select>\n        </span>`}getValue(){const t=this.node.querySelector("input[name='colName']"),e=this.node.querySelector("select[name='colType']");return{name:t.value,type:e.value}}}class h extends a.Widget{constructor(t){super(),this.editorFactory=new u.CodeMirrorEditorFactory,this.editorModel=new c.CodeEditor.Model({mimeType:"text/csv",value:t}),this.node.style.width="800px",this.node.style.height="600px";const e=this.layout=new a.BoxLayout;this.editor=new c.CodeEditorWrapper({factory:this.editorFactory.newDocumentEditor,model:this.editorModel}),e.addWidget(this.editor)}dispose(){this.isDisposed||(this.editorModel.dispose(),super.dispose())}getValue(){return this.editorModel.value.text}}},function(t,e){t.exports=__WEBPACK_EXTERNAL_MODULE__284__},function(t,e){t.exports=__WEBPACK_EXTERNAL_MODULE__285__},function(t,e,n){"use strict";n.r(e);var r=n(2),i=Array.prototype.slice,o=function(t){return t},a=1,s=2,A=3,l=4,c=1e-6;function u(t){return"translate("+(t+.5)+",0)"}function g(t){return"translate(0,"+(t+.5)+")"}function h(){return!this.__axis}function d(t,e){var n=[],r=null,d=null,f=6,p=6,I=3,C=t===a||t===l?-1:1,m=t===l||t===s?"x":"y",y=t===a||t===A?u:g;function E(i){var u=null==r?e.ticks?e.ticks.apply(e,n):e.domain():r,g=null==d?e.tickFormat?e.tickFormat.apply(e,n):o:d,E=Math.max(f,0)+I,v=e.range(),B=+v[0]+.5,b=+v[v.length-1]+.5,Q=(e.bandwidth?function(t){var e=Math.max(0,t.bandwidth()-1)/2;return t.round()&&(e=Math.round(e)),function(n){return+t(n)+e}}:function(t){return function(e){return+t(e)}})(e.copy()),w=i.selection?i.selection():i,x=w.selectAll(".domain").data([null]),_=w.selectAll(".tick").data(u,e).order(),D=_.exit(),M=_.enter().append("g").attr("class","tick"),k=_.select("line"),S=_.select("text");x=x.merge(x.enter().insert("path",".tick").attr("class","domain").attr("stroke","currentColor")),_=_.merge(M),k=k.merge(M.append("line").attr("stroke","currentColor").attr(m+"2",C*f)),S=S.merge(M.append("text").attr("fill","currentColor").attr(m,C*E).attr("dy",t===a?"0em":t===A?"0.71em":"0.32em")),i!==w&&(x=x.transition(i),_=_.transition(i),k=k.transition(i),S=S.transition(i),D=D.transition(i).attr("opacity",c).attr("transform",function(t){return isFinite(t=Q(t))?y(t):this.getAttribute("transform")}),M.attr("opacity",c).attr("transform",function(t){var e=this.parentNode.__axis;return y(e&&isFinite(e=e(t))?e:Q(t))})),D.remove(),x.attr("d",t===l||t==s?p?"M"+C*p+","+B+"H0.5V"+b+"H"+C*p:"M0.5,"+B+"V"+b:p?"M"+B+","+C*p+"V0.5H"+b+"V"+C*p:"M"+B+",0.5H"+b),_.attr("opacity",1).attr("transform",function(t){return y(Q(t))}),k.attr(m+"2",C*f),S.attr(m,C*E).text(g),w.filter(h).attr("fill","none").attr("font-size",10).attr("font-family","sans-serif").attr("text-anchor",t===s?"start":t===l?"end":"middle"),w.each(function(){this.__axis=Q})}return E.scale=function(t){return arguments.length?(e=t,E):e},E.ticks=function(){return n=i.call(arguments),E},E.tickArguments=function(t){return arguments.length?(n=null==t?[]:i.call(t),E):n.slice()},E.tickValues=function(t){return arguments.length?(r=null==t?null:i.call(t),E):r&&r.slice()},E.tickFormat=function(t){return arguments.length?(d=t,E):d},E.tickSize=function(t){return arguments.length?(f=p=+t,E):f},E.tickSizeInner=function(t){return arguments.length?(f=+t,E):f},E.tickSizeOuter=function(t){return arguments.length?(p=+t,E):p},E.tickPadding=function(t){return arguments.length?(I=+t,E):I},E}function f(t){return d(a,t)}function p(t){return d(s,t)}function I(t){return d(A,t)}function C(t){return d(l,t)}var m=n(87),y=Math.cos,E=Math.sin,v=Math.PI,B=v/2,b=2*v,Q=Math.max;function w(t){return function(e,n){return t(e.source.value+e.target.value,n.source.value+n.target.value)}}var x=function(){var t=0,e=null,n=null,i=null;function o(o){var a,s,A,l,c,u,g=o.length,h=[],d=Object(r.range)(g),f=[],p=[],I=p.groups=new Array(g),C=new Array(g*g);for(a=0,c=-1;++c<g;){for(s=0,u=-1;++u<g;)s+=o[c][u];h.push(s),f.push(Object(r.range)(g)),a+=s}for(e&&d.sort(function(t,n){return e(h[t],h[n])}),n&&f.forEach(function(t,e){t.sort(function(t,r){return n(o[e][t],o[e][r])})}),l=(a=Q(0,b-t*g)/a)?t:b/g,s=0,c=-1;++c<g;){for(A=s,u=-1;++u<g;){var m=d[c],y=f[m][u],E=o[m][y],v=s,B=s+=E*a;C[y*g+m]={index:m,subindex:y,startAngle:v,endAngle:B,value:E}}I[m]={index:m,startAngle:A,endAngle:s,value:h[m]},s+=l}for(c=-1;++c<g;)for(u=c-1;++u<g;){var w=C[u*g+c],x=C[c*g+u];(w.value||x.value)&&p.push(w.value<x.value?{source:x,target:w}:{source:w,target:x})}return i?p.sort(i):p}return o.padAngle=function(e){return arguments.length?(t=Q(0,e),o):t},o.sortGroups=function(t){return arguments.length?(e=t,o):e},o.sortSubgroups=function(t){return arguments.length?(n=t,o):n},o.sortChords=function(t){return arguments.length?(null==t?i=null:(i=w(t))._=t,o):i&&i._},o},_=Array.prototype.slice,D=function(t){return function(){return t}},M=n(8);function k(t){return t.source}function S(t){return t.target}function L(t){return t.radius}function R(t){return t.startAngle}function N(t){return t.endAngle}var F=function(){var t=k,e=S,n=L,r=R,i=N,o=null;function a(){var a,s=_.call(arguments),A=t.apply(this,s),l=e.apply(this,s),c=+n.apply(this,(s[0]=A,s)),u=r.apply(this,s)-B,g=i.apply(this,s)-B,h=c*y(u),d=c*E(u),f=+n.apply(this,(s[0]=l,s)),p=r.apply(this,s)-B,I=i.apply(this,s)-B;if(o||(o=a=Object(M.path)()),o.moveTo(h,d),o.arc(0,0,c,u,g),u===p&&g===I||(o.quadraticCurveTo(0,0,f*y(p),f*E(p)),o.arc(0,0,f,p,I)),o.quadraticCurveTo(0,0,h,d),o.closePath(),a)return o=null,a+""||null}return a.radius=function(t){return arguments.length?(n="function"==typeof t?t:D(+t),a):n},a.startAngle=function(t){return arguments.length?(r="function"==typeof t?t:D(+t),a):r},a.endAngle=function(t){return arguments.length?(i="function"==typeof t?t:D(+t),a):i},a.source=function(e){return arguments.length?(t=e,a):t},a.target=function(t){return arguments.length?(e=t,a):e},a.context=function(t){return arguments.length?(o=null==t?null:t,a):o},a},T=n(9),U=n(3),O=Array.prototype.slice,G=function(t,e){return t-e},H=function(t){for(var e=0,n=t.length,r=t[n-1][1]*t[0][0]-t[n-1][0]*t[0][1];++e<n;)r+=t[e-1][1]*t[e][0]-t[e-1][0]*t[e][1];return r},j=function(t){return function(){return t}},Y=function(t,e){for(var n,r=-1,i=e.length;++r<i;)if(n=P(t,e[r]))return n;return 0};function P(t,e){for(var n=e[0],r=e[1],i=-1,o=0,a=t.length,s=a-1;o<a;s=o++){var A=t[o],l=A[0],c=A[1],u=t[s],g=u[0],h=u[1];if(K(A,u,e))return 0;c>r!=h>r&&n<(g-l)*(r-c)/(h-c)+l&&(i=-i)}return i}function K(t,e,n){var r,i,o,a;return function(t,e,n){return(e[0]-t[0])*(n[1]-t[1])==(n[0]-t[0])*(e[1]-t[1])}(t,e,n)&&(i=t[r=+(t[0]===e[0])],o=n[r],a=e[r],i<=o&&o<=a||a<=o&&o<=i)}var z=function(){},J=[[],[[[1,1.5],[.5,1]]],[[[1.5,1],[1,1.5]]],[[[1.5,1],[.5,1]]],[[[1,.5],[1.5,1]]],[[[1,1.5],[.5,1]],[[1,.5],[1.5,1]]],[[[1,.5],[1,1.5]]],[[[1,.5],[.5,1]]],[[[.5,1],[1,.5]]],[[[1,1.5],[1,.5]]],[[[.5,1],[1,.5]],[[1.5,1],[1,1.5]]],[[[1.5,1],[1,.5]]],[[[.5,1],[1.5,1]]],[[[1,1.5],[1.5,1]]],[[[.5,1],[1,1.5]]],[]],q=function(){var t=1,e=1,n=r.thresholdSturges,i=A;function o(t){var e=n(t);if(Array.isArray(e))e=e.slice().sort(G);else{var i=Object(r.extent)(t),o=i[0],s=i[1];e=Object(r.tickStep)(o,s,e),e=Object(r.range)(Math.floor(o/e)*e,Math.floor(s/e)*e,e)}return e.map(function(e){return a(t,e)})}function a(n,r){var o=[],a=[];return function(n,r,i){var o,a,A,l,c,u,g=new Array,h=new Array;for(o=a=-1,l=n[0]>=r,J[l<<1].forEach(d);++o<t-1;)A=l,l=n[o+1]>=r,J[A|l<<1].forEach(d);for(J[l<<0].forEach(d);++a<e-1;){for(o=-1,l=n[a*t+t]>=r,c=n[a*t]>=r,J[l<<1|c<<2].forEach(d);++o<t-1;)A=l,l=n[a*t+t+o+1]>=r,u=c,c=n[a*t+o+1]>=r,J[A|l<<1|c<<2|u<<3].forEach(d);J[l|c<<3].forEach(d)}for(o=-1,c=n[a*t]>=r,J[c<<2].forEach(d);++o<t-1;)u=c,c=n[a*t+o+1]>=r,J[c<<2|u<<3].forEach(d);function d(t){var e,n,r=[t[0][0]+o,t[0][1]+a],A=[t[1][0]+o,t[1][1]+a],l=s(r),c=s(A);(e=h[l])?(n=g[c])?(delete h[e.end],delete g[n.start],e===n?(e.ring.push(A),i(e.ring)):g[e.start]=h[n.end]={start:e.start,end:n.end,ring:e.ring.concat(n.ring)}):(delete h[e.end],e.ring.push(A),h[e.end=c]=e):(e=g[c])?(n=h[l])?(delete g[e.start],delete h[n.end],e===n?(e.ring.push(A),i(e.ring)):g[n.start]=h[e.end]={start:n.start,end:e.end,ring:n.ring.concat(e.ring)}):(delete g[e.start],e.ring.unshift(r),g[e.start=l]=e):g[l]=h[c]={start:l,end:c,ring:[r,A]}}J[c<<3].forEach(d)}(n,r,function(t){i(t,n,r),H(t)>0?o.push([t]):a.push(t)}),a.forEach(function(t){for(var e,n=0,r=o.length;n<r;++n)if(-1!==Y((e=o[n])[0],t))return void e.push(t)}),{type:"MultiPolygon",value:r,coordinates:o}}function s(e){return 2*e[0]+e[1]*(t+1)*4}function A(n,r,i){n.forEach(function(n){var o,a=n[0],s=n[1],A=0|a,l=0|s,c=r[l*t+A];a>0&&a<t&&A===a&&(o=r[l*t+A-1],n[0]=a+(i-o)/(c-o)-.5),s>0&&s<e&&l===s&&(o=r[(l-1)*t+A],n[1]=s+(i-o)/(c-o)-.5)})}return o.contour=a,o.size=function(n){if(!arguments.length)return[t,e];var r=Math.ceil(n[0]),i=Math.ceil(n[1]);if(!(r>0&&i>0))throw new Error("invalid size");return t=r,e=i,o},o.thresholds=function(t){return arguments.length?(n="function"==typeof t?t:Array.isArray(t)?j(O.call(t)):j(t),o):n},o.smooth=function(t){return arguments.length?(i=t?A:z,o):i===A},o};function V(t,e,n){for(var r=t.width,i=t.height,o=1+(n<<1),a=0;a<i;++a)for(var s=0,A=0;s<r+n;++s)s<r&&(A+=t.data[s+a*r]),s>=n&&(s>=o&&(A-=t.data[s-o+a*r]),e.data[s-n+a*r]=A/Math.min(s+1,r-1+o-s,o))}function W(t,e,n){for(var r=t.width,i=t.height,o=1+(n<<1),a=0;a<r;++a)for(var s=0,A=0;s<i+n;++s)s<i&&(A+=t.data[a+s*r]),s>=n&&(s>=o&&(A-=t.data[a+(s-o)*r]),e.data[a+(s-n)*r]=A/Math.min(s+1,i-1+o-s,o))}function X(t){return t[0]}function Z(t){return t[1]}function $(){return 1}var tt=function(){var t=X,e=Z,n=$,i=960,o=500,a=20,s=2,A=3*a,l=i+2*A>>s,c=o+2*A>>s,u=j(20);function g(i){var o=new Float32Array(l*c),g=new Float32Array(l*c);i.forEach(function(r,i,a){var u=+t(r,i,a)+A>>s,g=+e(r,i,a)+A>>s,h=+n(r,i,a);u>=0&&u<l&&g>=0&&g<c&&(o[u+g*l]+=h)}),V({width:l,height:c,data:o},{width:l,height:c,data:g},a>>s),W({width:l,height:c,data:g},{width:l,height:c,data:o},a>>s),V({width:l,height:c,data:o},{width:l,height:c,data:g},a>>s),W({width:l,height:c,data:g},{width:l,height:c,data:o},a>>s),V({width:l,height:c,data:o},{width:l,height:c,data:g},a>>s),W({width:l,height:c,data:g},{width:l,height:c,data:o},a>>s);var d=u(o);if(!Array.isArray(d)){var f=Object(r.max)(o);d=Object(r.tickStep)(0,f,d),(d=Object(r.range)(0,Math.floor(f/d)*d,d)).shift()}return q().thresholds(d).size([l,c])(o).map(h)}function h(t){return t.value*=Math.pow(2,-2*s),t.coordinates.forEach(d),t}function d(t){t.forEach(f)}function f(t){t.forEach(p)}function p(t){t[0]=t[0]*Math.pow(2,s)-A,t[1]=t[1]*Math.pow(2,s)-A}function I(){return l=i+2*(A=3*a)>>s,c=o+2*A>>s,g}return g.x=function(e){return arguments.length?(t="function"==typeof e?e:j(+e),g):t},g.y=function(t){return arguments.length?(e="function"==typeof t?t:j(+t),g):e},g.weight=function(t){return arguments.length?(n="function"==typeof t?t:j(+t),g):n},g.size=function(t){if(!arguments.length)return[i,o];var e=Math.ceil(t[0]),n=Math.ceil(t[1]);if(!(e>=0||e>=0))throw new Error("invalid size");return i=e,o=n,I()},g.cellSize=function(t){if(!arguments.length)return 1<<s;if(!((t=+t)>=1))throw new Error("invalid cell size");return s=Math.floor(Math.log(t)/Math.LN2),I()},g.thresholds=function(t){return arguments.length?(u="function"==typeof t?t:Array.isArray(t)?j(O.call(t)):j(t),g):u},g.bandwidth=function(t){if(!arguments.length)return Math.sqrt(a*(a+1));if(!((t=+t)>=0))throw new Error("invalid bandwidth");return a=Math.round((Math.sqrt(4*t*t+1)-1)/2),I()},g},et=n(12),nt=n(24),rt=n(32),it=n(61),ot=n(86),at=function(t,e){var n;function r(){var r,i,o=n.length,a=0,s=0;for(r=0;r<o;++r)a+=(i=n[r]).x,s+=i.y;for(a=a/o-t,s=s/o-e,r=0;r<o;++r)(i=n[r]).x-=a,i.y-=s}return null==t&&(t=0),null==e&&(e=0),r.initialize=function(t){n=t},r.x=function(e){return arguments.length?(t=+e,r):t},r.y=function(t){return arguments.length?(e=+t,r):e},r},st=function(t){return function(){return t}},At=function(){return 1e-6*(Math.random()-.5)};function lt(t,e,n,r){if(isNaN(e)||isNaN(n))return t;var i,o,a,s,A,l,c,u,g,h=t._root,d={data:r},f=t._x0,p=t._y0,I=t._x1,C=t._y1;if(!h)return t._root=d,t;for(;h.length;)if((l=e>=(o=(f+I)/2))?f=o:I=o,(c=n>=(a=(p+C)/2))?p=a:C=a,i=h,!(h=h[u=c<<1|l]))return i[u]=d,t;if(s=+t._x.call(null,h.data),A=+t._y.call(null,h.data),e===s&&n===A)return d.next=h,i?i[u]=d:t._root=d,t;do{i=i?i[u]=new Array(4):t._root=new Array(4),(l=e>=(o=(f+I)/2))?f=o:I=o,(c=n>=(a=(p+C)/2))?p=a:C=a}while((u=c<<1|l)==(g=(A>=a)<<1|s>=o));return i[g]=h,i[u]=d,t}var ct=function(t,e,n,r,i){this.node=t,this.x0=e,this.y0=n,this.x1=r,this.y1=i};function ut(t){return t[0]}function gt(t){return t[1]}function ht(t,e,n){var r=new dt(null==e?ut:e,null==n?gt:n,NaN,NaN,NaN,NaN);return null==t?r:r.addAll(t)}function dt(t,e,n,r,i,o){this._x=t,this._y=e,this._x0=n,this._y0=r,this._x1=i,this._y1=o,this._root=void 0}function ft(t){for(var e={data:t.data},n=e;t=t.next;)n=n.next={data:t.data};return e}var pt=ht.prototype=dt.prototype;function It(t){return t.x+t.vx}function Ct(t){return t.y+t.vy}pt.copy=function(){var t,e,n=new dt(this._x,this._y,this._x0,this._y0,this._x1,this._y1),r=this._root;if(!r)return n;if(!r.length)return n._root=ft(r),n;for(t=[{source:r,target:n._root=new Array(4)}];r=t.pop();)for(var i=0;i<4;++i)(e=r.source[i])&&(e.length?t.push({source:e,target:r.target[i]=new Array(4)}):r.target[i]=ft(e));return n},pt.add=function(t){var e=+this._x.call(null,t),n=+this._y.call(null,t);return lt(this.cover(e,n),e,n,t)},pt.addAll=function(t){var e,n,r,i,o=t.length,a=new Array(o),s=new Array(o),A=1/0,l=1/0,c=-1/0,u=-1/0;for(n=0;n<o;++n)isNaN(r=+this._x.call(null,e=t[n]))||isNaN(i=+this._y.call(null,e))||(a[n]=r,s[n]=i,r<A&&(A=r),r>c&&(c=r),i<l&&(l=i),i>u&&(u=i));if(A>c||l>u)return this;for(this.cover(A,l).cover(c,u),n=0;n<o;++n)lt(this,a[n],s[n],t[n]);return this},pt.cover=function(t,e){if(isNaN(t=+t)||isNaN(e=+e))return this;var n=this._x0,r=this._y0,i=this._x1,o=this._y1;if(isNaN(n))i=(n=Math.floor(t))+1,o=(r=Math.floor(e))+1;else{for(var a,s,A=i-n,l=this._root;n>t||t>=i||r>e||e>=o;)switch(s=(e<r)<<1|t<n,(a=new Array(4))[s]=l,l=a,A*=2,s){case 0:i=n+A,o=r+A;break;case 1:n=i-A,o=r+A;break;case 2:i=n+A,r=o-A;break;case 3:n=i-A,r=o-A}this._root&&this._root.length&&(this._root=l)}return this._x0=n,this._y0=r,this._x1=i,this._y1=o,this},pt.data=function(){var t=[];return this.visit(function(e){if(!e.length)do{t.push(e.data)}while(e=e.next)}),t},pt.extent=function(t){return arguments.length?this.cover(+t[0][0],+t[0][1]).cover(+t[1][0],+t[1][1]):isNaN(this._x0)?void 0:[[this._x0,this._y0],[this._x1,this._y1]]},pt.find=function(t,e,n){var r,i,o,a,s,A,l,c=this._x0,u=this._y0,g=this._x1,h=this._y1,d=[],f=this._root;for(f&&d.push(new ct(f,c,u,g,h)),null==n?n=1/0:(c=t-n,u=e-n,g=t+n,h=e+n,n*=n);A=d.pop();)if(!(!(f=A.node)||(i=A.x0)>g||(o=A.y0)>h||(a=A.x1)<c||(s=A.y1)<u))if(f.length){var p=(i+a)/2,I=(o+s)/2;d.push(new ct(f[3],p,I,a,s),new ct(f[2],i,I,p,s),new ct(f[1],p,o,a,I),new ct(f[0],i,o,p,I)),(l=(e>=I)<<1|t>=p)&&(A=d[d.length-1],d[d.length-1]=d[d.length-1-l],d[d.length-1-l]=A)}else{var C=t-+this._x.call(null,f.data),m=e-+this._y.call(null,f.data),y=C*C+m*m;if(y<n){var E=Math.sqrt(n=y);c=t-E,u=e-E,g=t+E,h=e+E,r=f.data}}return r},pt.remove=function(t){if(isNaN(o=+this._x.call(null,t))||isNaN(a=+this._y.call(null,t)))return this;var e,n,r,i,o,a,s,A,l,c,u,g,h=this._root,d=this._x0,f=this._y0,p=this._x1,I=this._y1;if(!h)return this;if(h.length)for(;;){if((l=o>=(s=(d+p)/2))?d=s:p=s,(c=a>=(A=(f+I)/2))?f=A:I=A,e=h,!(h=h[u=c<<1|l]))return this;if(!h.length)break;(e[u+1&3]||e[u+2&3]||e[u+3&3])&&(n=e,g=u)}for(;h.data!==t;)if(r=h,!(h=h.next))return this;return(i=h.next)&&delete h.next,r?(i?r.next=i:delete r.next,this):e?(i?e[u]=i:delete e[u],(h=e[0]||e[1]||e[2]||e[3])&&h===(e[3]||e[2]||e[1]||e[0])&&!h.length&&(n?n[g]=h:this._root=h),this):(this._root=i,this)},pt.removeAll=function(t){for(var e=0,n=t.length;e<n;++e)this.remove(t[e]);return this},pt.root=function(){return this._root},pt.size=function(){var t=0;return this.visit(function(e){if(!e.length)do{++t}while(e=e.next)}),t},pt.visit=function(t){var e,n,r,i,o,a,s=[],A=this._root;for(A&&s.push(new ct(A,this._x0,this._y0,this._x1,this._y1));e=s.pop();)if(!t(A=e.node,r=e.x0,i=e.y0,o=e.x1,a=e.y1)&&A.length){var l=(r+o)/2,c=(i+a)/2;(n=A[3])&&s.push(new ct(n,l,c,o,a)),(n=A[2])&&s.push(new ct(n,r,c,l,a)),(n=A[1])&&s.push(new ct(n,l,i,o,c)),(n=A[0])&&s.push(new ct(n,r,i,l,c))}return this},pt.visitAfter=function(t){var e,n=[],r=[];for(this._root&&n.push(new ct(this._root,this._x0,this._y0,this._x1,this._y1));e=n.pop();){var i=e.node;if(i.length){var o,a=e.x0,s=e.y0,A=e.x1,l=e.y1,c=(a+A)/2,u=(s+l)/2;(o=i[0])&&n.push(new ct(o,a,s,c,u)),(o=i[1])&&n.push(new ct(o,c,s,A,u)),(o=i[2])&&n.push(new ct(o,a,u,c,l)),(o=i[3])&&n.push(new ct(o,c,u,A,l))}r.push(e)}for(;e=r.pop();)t(e.node,e.x0,e.y0,e.x1,e.y1);return this},pt.x=function(t){return arguments.length?(this._x=t,this):this._x},pt.y=function(t){return arguments.length?(this._y=t,this):this._y};var mt=function(t){var e,n,r=1,i=1;function o(){for(var t,o,s,A,l,c,u,g=e.length,h=0;h<i;++h)for(o=ht(e,It,Ct).visitAfter(a),t=0;t<g;++t)s=e[t],c=n[s.index],u=c*c,A=s.x+s.vx,l=s.y+s.vy,o.visit(d);function d(t,e,n,i,o){var a=t.data,g=t.r,h=c+g;if(!a)return e>A+h||i<A-h||n>l+h||o<l-h;if(a.index>s.index){var d=A-a.x-a.vx,f=l-a.y-a.vy,p=d*d+f*f;p<h*h&&(0===d&&(p+=(d=At())*d),0===f&&(p+=(f=At())*f),p=(h-(p=Math.sqrt(p)))/p*r,s.vx+=(d*=p)*(h=(g*=g)/(u+g)),s.vy+=(f*=p)*h,a.vx-=d*(h=1-h),a.vy-=f*h)}}}function a(t){if(t.data)return t.r=n[t.data.index];for(var e=t.r=0;e<4;++e)t[e]&&t[e].r>t.r&&(t.r=t[e].r)}function s(){if(e){var r,i,o=e.length;for(n=new Array(o),r=0;r<o;++r)i=e[r],n[i.index]=+t(i,r,e)}}return"function"!=typeof t&&(t=st(null==t?1:+t)),o.initialize=function(t){e=t,s()},o.iterations=function(t){return arguments.length?(i=+t,o):i},o.strength=function(t){return arguments.length?(r=+t,o):r},o.radius=function(e){return arguments.length?(t="function"==typeof e?e:st(+e),s(),o):t},o};function yt(t){return t.index}function Et(t,e){var n=t.get(e);if(!n)throw new Error("missing: "+e);return n}var vt=function(t){var e,n,r,i,o,a=yt,s=function(t){return 1/Math.min(i[t.source.index],i[t.target.index])},A=st(30),l=1;function c(r){for(var i=0,a=t.length;i<l;++i)for(var s,A,c,u,g,h,d,f=0;f<a;++f)A=(s=t[f]).source,u=(c=s.target).x+c.vx-A.x-A.vx||At(),g=c.y+c.vy-A.y-A.vy||At(),u*=h=((h=Math.sqrt(u*u+g*g))-n[f])/h*r*e[f],g*=h,c.vx-=u*(d=o[f]),c.vy-=g*d,A.vx+=u*(d=1-d),A.vy+=g*d}function u(){if(r){var s,A,l=r.length,c=t.length,u=Object(T.c)(r,a);for(s=0,i=new Array(l);s<c;++s)(A=t[s]).index=s,"object"!=typeof A.source&&(A.source=Et(u,A.source)),"object"!=typeof A.target&&(A.target=Et(u,A.target)),i[A.source.index]=(i[A.source.index]||0)+1,i[A.target.index]=(i[A.target.index]||0)+1;for(s=0,o=new Array(c);s<c;++s)A=t[s],o[s]=i[A.source.index]/(i[A.source.index]+i[A.target.index]);e=new Array(c),g(),n=new Array(c),h()}}function g(){if(r)for(var n=0,i=t.length;n<i;++n)e[n]=+s(t[n],n,t)}function h(){if(r)for(var e=0,i=t.length;e<i;++e)n[e]=+A(t[e],e,t)}return null==t&&(t=[]),c.initialize=function(t){r=t,u()},c.links=function(e){return arguments.length?(t=e,u(),c):t},c.id=function(t){return arguments.length?(a=t,c):a},c.iterations=function(t){return arguments.length?(l=+t,c):l},c.strength=function(t){return arguments.length?(s="function"==typeof t?t:st(+t),g(),c):s},c.distance=function(t){return arguments.length?(A="function"==typeof t?t:st(+t),h(),c):A},c},Bt=n(14);function bt(t){return t.x}function Qt(t){return t.y}var wt=10,xt=Math.PI*(3-Math.sqrt(5)),_t=function(t){var e,n=1,r=.001,i=1-Math.pow(r,1/300),o=0,a=.6,s=Object(T.c)(),A=Object(Bt.d)(c),l=Object(et.dispatch)("tick","end");function c(){u(),l.call("tick",e),n<r&&(A.stop(),l.call("end",e))}function u(r){var A,l,c=t.length;void 0===r&&(r=1);for(var u=0;u<r;++u)for(n+=(o-n)*i,s.each(function(t){t(n)}),A=0;A<c;++A)null==(l=t[A]).fx?l.x+=l.vx*=a:(l.x=l.fx,l.vx=0),null==l.fy?l.y+=l.vy*=a:(l.y=l.fy,l.vy=0);return e}function g(){for(var e,n=0,r=t.length;n<r;++n){if((e=t[n]).index=n,null!=e.fx&&(e.x=e.fx),null!=e.fy&&(e.y=e.fy),isNaN(e.x)||isNaN(e.y)){var i=wt*Math.sqrt(n),o=n*xt;e.x=i*Math.cos(o),e.y=i*Math.sin(o)}(isNaN(e.vx)||isNaN(e.vy))&&(e.vx=e.vy=0)}}function h(e){return e.initialize&&e.initialize(t),e}return null==t&&(t=[]),g(),e={tick:u,restart:function(){return A.restart(c),e},stop:function(){return A.stop(),e},nodes:function(n){return arguments.length?(t=n,g(),s.each(h),e):t},alpha:function(t){return arguments.length?(n=+t,e):n},alphaMin:function(t){return arguments.length?(r=+t,e):r},alphaDecay:function(t){return arguments.length?(i=+t,e):+i},alphaTarget:function(t){return arguments.length?(o=+t,e):o},velocityDecay:function(t){return arguments.length?(a=1-t,e):1-a},force:function(t,n){return arguments.length>1?(null==n?s.remove(t):s.set(t,h(n)),e):s.get(t)},find:function(e,n,r){var i,o,a,s,A,l=0,c=t.length;for(null==r?r=1/0:r*=r,l=0;l<c;++l)(a=(i=e-(s=t[l]).x)*i+(o=n-s.y)*o)<r&&(A=s,r=a);return A},on:function(t,n){return arguments.length>1?(l.on(t,n),e):l.on(t)}}},Dt=function(){var t,e,n,r,i=st(-30),o=1,a=1/0,s=.81;function A(r){var i,o=t.length,a=ht(t,bt,Qt).visitAfter(c);for(n=r,i=0;i<o;++i)e=t[i],a.visit(u)}function l(){if(t){var e,n,o=t.length;for(r=new Array(o),e=0;e<o;++e)n=t[e],r[n.index]=+i(n,e,t)}}function c(t){var e,n,i,o,a,s=0,A=0;if(t.length){for(i=o=a=0;a<4;++a)(e=t[a])&&(n=Math.abs(e.value))&&(s+=e.value,A+=n,i+=n*e.x,o+=n*e.y);t.x=i/A,t.y=o/A}else{(e=t).x=e.data.x,e.y=e.data.y;do{s+=r[e.data.index]}while(e=e.next)}t.value=s}function u(t,i,A,l){if(!t.value)return!0;var c=t.x-e.x,u=t.y-e.y,g=l-i,h=c*c+u*u;if(g*g/s<h)return h<a&&(0===c&&(h+=(c=At())*c),0===u&&(h+=(u=At())*u),h<o&&(h=Math.sqrt(o*h)),e.vx+=c*t.value*n/h,e.vy+=u*t.value*n/h),!0;if(!(t.length||h>=a)){(t.data!==e||t.next)&&(0===c&&(h+=(c=At())*c),0===u&&(h+=(u=At())*u),h<o&&(h=Math.sqrt(o*h)));do{t.data!==e&&(g=r[t.data.index]*n/h,e.vx+=c*g,e.vy+=u*g)}while(t=t.next)}}return A.initialize=function(e){t=e,l()},A.strength=function(t){return arguments.length?(i="function"==typeof t?t:st(+t),l(),A):i},A.distanceMin=function(t){return arguments.length?(o=t*t,A):Math.sqrt(o)},A.distanceMax=function(t){return arguments.length?(a=t*t,A):Math.sqrt(a)},A.theta=function(t){return arguments.length?(s=t*t,A):Math.sqrt(s)},A},Mt=function(t,e,n){var r,i,o,a=st(.1);function s(t){for(var a=0,s=r.length;a<s;++a){var A=r[a],l=A.x-e||1e-6,c=A.y-n||1e-6,u=Math.sqrt(l*l+c*c),g=(o[a]-u)*i[a]*t/u;A.vx+=l*g,A.vy+=c*g}}function A(){if(r){var e,n=r.length;for(i=new Array(n),o=new Array(n),e=0;e<n;++e)o[e]=+t(r[e],e,r),i[e]=isNaN(o[e])?0:+a(r[e],e,r)}}return"function"!=typeof t&&(t=st(+t)),null==e&&(e=0),null==n&&(n=0),s.initialize=function(t){r=t,A()},s.strength=function(t){return arguments.length?(a="function"==typeof t?t:st(+t),A(),s):a},s.radius=function(e){return arguments.length?(t="function"==typeof e?e:st(+e),A(),s):t},s.x=function(t){return arguments.length?(e=+t,s):e},s.y=function(t){return arguments.length?(n=+t,s):n},s},kt=function(t){var e,n,r,i=st(.1);function o(t){for(var i,o=0,a=e.length;o<a;++o)(i=e[o]).vx+=(r[o]-i.x)*n[o]*t}function a(){if(e){var o,a=e.length;for(n=new Array(a),r=new Array(a),o=0;o<a;++o)n[o]=isNaN(r[o]=+t(e[o],o,e))?0:+i(e[o],o,e)}}return"function"!=typeof t&&(t=st(null==t?0:+t)),o.initialize=function(t){e=t,a()},o.strength=function(t){return arguments.length?(i="function"==typeof t?t:st(+t),a(),o):i},o.x=function(e){return arguments.length?(t="function"==typeof e?e:st(+e),a(),o):t},o},St=function(t){var e,n,r,i=st(.1);function o(t){for(var i,o=0,a=e.length;o<a;++o)(i=e[o]).vy+=(r[o]-i.y)*n[o]*t}function a(){if(e){var o,a=e.length;for(n=new Array(a),r=new Array(a),o=0;o<a;++o)n[o]=isNaN(r[o]=+t(e[o],o,e))?0:+i(e[o],o,e)}}return"function"!=typeof t&&(t=st(null==t?0:+t)),o.initialize=function(t){e=t,a()},o.strength=function(t){return arguments.length?(i="function"==typeof t?t:st(+t),a(),o):i},o.y=function(e){return arguments.length?(t="function"==typeof e?e:st(+e),a(),o):t},o},Lt=n(6),Rt=function(){return new Nt};function Nt(){this.reset()}Nt.prototype={constructor:Nt,reset:function(){this.s=this.t=0},add:function(t){Tt(Ft,t,this.t),Tt(this,Ft.s,this.s),this.s?this.t+=Ft.t:this.s=Ft.t},valueOf:function(){return this.s}};var Ft=new Nt;function Tt(t,e,n){var r=t.s=e+n,i=r-e,o=r-i;t.t=e-o+(n-i)}var Ut=1e-6,Ot=1e-12,Gt=Math.PI,Ht=Gt/2,jt=Gt/4,Yt=2*Gt,Pt=180/Gt,Kt=Gt/180,zt=Math.abs,Jt=Math.atan,qt=Math.atan2,Vt=Math.cos,Wt=Math.ceil,Xt=Math.exp,Zt=(Math.floor,Math.log),$t=Math.pow,te=Math.sin,ee=Math.sign||function(t){return t>0?1:t<0?-1:0},ne=Math.sqrt,re=Math.tan;function ie(t){return t>1?0:t<-1?Gt:Math.acos(t)}function oe(t){return t>1?Ht:t<-1?-Ht:Math.asin(t)}function ae(t){return(t=te(t/2))*t}function se(){}function Ae(t,e){t&&ce.hasOwnProperty(t.type)&&ce[t.type](t,e)}var le={Feature:function(t,e){Ae(t.geometry,e)},FeatureCollection:function(t,e){for(var n=t.features,r=-1,i=n.length;++r<i;)Ae(n[r].geometry,e)}},ce={Sphere:function(t,e){e.sphere()},Point:function(t,e){t=t.coordinates,e.point(t[0],t[1],t[2])},MultiPoint:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)t=n[r],e.point(t[0],t[1],t[2])},LineString:function(t,e){ue(t.coordinates,e,0)},MultiLineString:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)ue(n[r],e,0)},Polygon:function(t,e){ge(t.coordinates,e)},MultiPolygon:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)ge(n[r],e)},GeometryCollection:function(t,e){for(var n=t.geometries,r=-1,i=n.length;++r<i;)Ae(n[r],e)}};function ue(t,e,n){var r,i=-1,o=t.length-n;for(e.lineStart();++i<o;)r=t[i],e.point(r[0],r[1],r[2]);e.lineEnd()}function ge(t,e){var n=-1,r=t.length;for(e.polygonStart();++n<r;)ue(t[n],e,1);e.polygonEnd()}var he,de,fe,pe,Ie,Ce=function(t,e){t&&le.hasOwnProperty(t.type)?le[t.type](t,e):Ae(t,e)},me=Rt(),ye=Rt(),Ee={point:se,lineStart:se,lineEnd:se,polygonStart:function(){me.reset(),Ee.lineStart=ve,Ee.lineEnd=Be},polygonEnd:function(){var t=+me;ye.add(t<0?Yt+t:t),this.lineStart=this.lineEnd=this.point=se},sphere:function(){ye.add(Yt)}};function ve(){Ee.point=be}function Be(){Qe(he,de)}function be(t,e){Ee.point=Qe,he=t,de=e,fe=t*=Kt,pe=Vt(e=(e*=Kt)/2+jt),Ie=te(e)}function Qe(t,e){var n=(t*=Kt)-fe,r=n>=0?1:-1,i=r*n,o=Vt(e=(e*=Kt)/2+jt),a=te(e),s=Ie*a,A=pe*o+s*Vt(i),l=s*r*te(i);me.add(qt(l,A)),fe=t,pe=o,Ie=a}var we=function(t){return ye.reset(),Ce(t,Ee),2*ye};function xe(t){return[qt(t[1],t[0]),oe(t[2])]}function _e(t){var e=t[0],n=t[1],r=Vt(n);return[r*Vt(e),r*te(e),te(n)]}function De(t,e){return t[0]*e[0]+t[1]*e[1]+t[2]*e[2]}function Me(t,e){return[t[1]*e[2]-t[2]*e[1],t[2]*e[0]-t[0]*e[2],t[0]*e[1]-t[1]*e[0]]}function ke(t,e){t[0]+=e[0],t[1]+=e[1],t[2]+=e[2]}function Se(t,e){return[t[0]*e,t[1]*e,t[2]*e]}function Le(t){var e=ne(t[0]*t[0]+t[1]*t[1]+t[2]*t[2]);t[0]/=e,t[1]/=e,t[2]/=e}var Re,Ne,Fe,Te,Ue,Oe,Ge,He,je,Ye,Pe=Rt(),Ke={point:ze,lineStart:qe,lineEnd:Ve,polygonStart:function(){Ke.point=We,Ke.lineStart=Xe,Ke.lineEnd=Ze,Pe.reset(),Ee.polygonStart()},polygonEnd:function(){Ee.polygonEnd(),Ke.point=ze,Ke.lineStart=qe,Ke.lineEnd=Ve,me<0?(Re=-(Fe=180),Ne=-(Te=90)):Pe>Ut?Te=90:Pe<-Ut&&(Ne=-90),Ye[0]=Re,Ye[1]=Fe},sphere:function(){Re=-(Fe=180),Ne=-(Te=90)}};function ze(t,e){je.push(Ye=[Re=t,Fe=t]),e<Ne&&(Ne=e),e>Te&&(Te=e)}function Je(t,e){var n=_e([t*Kt,e*Kt]);if(He){var r=Me(He,n),i=Me([r[1],-r[0],0],r);Le(i),i=xe(i);var o,a=t-Ue,s=a>0?1:-1,A=i[0]*Pt*s,l=zt(a)>180;l^(s*Ue<A&&A<s*t)?(o=i[1]*Pt)>Te&&(Te=o):l^(s*Ue<(A=(A+360)%360-180)&&A<s*t)?(o=-i[1]*Pt)<Ne&&(Ne=o):(e<Ne&&(Ne=e),e>Te&&(Te=e)),l?t<Ue?$e(Re,t)>$e(Re,Fe)&&(Fe=t):$e(t,Fe)>$e(Re,Fe)&&(Re=t):Fe>=Re?(t<Re&&(Re=t),t>Fe&&(Fe=t)):t>Ue?$e(Re,t)>$e(Re,Fe)&&(Fe=t):$e(t,Fe)>$e(Re,Fe)&&(Re=t)}else je.push(Ye=[Re=t,Fe=t]);e<Ne&&(Ne=e),e>Te&&(Te=e),He=n,Ue=t}function qe(){Ke.point=Je}function Ve(){Ye[0]=Re,Ye[1]=Fe,Ke.point=ze,He=null}function We(t,e){if(He){var n=t-Ue;Pe.add(zt(n)>180?n+(n>0?360:-360):n)}else Oe=t,Ge=e;Ee.point(t,e),Je(t,e)}function Xe(){Ee.lineStart()}function Ze(){We(Oe,Ge),Ee.lineEnd(),zt(Pe)>Ut&&(Re=-(Fe=180)),Ye[0]=Re,Ye[1]=Fe,He=null}function $e(t,e){return(e-=t)<0?e+360:e}function tn(t,e){return t[0]-e[0]}function en(t,e){return t[0]<=t[1]?t[0]<=e&&e<=t[1]:e<t[0]||t[1]<e}var nn,rn,on,an,sn,An,ln,cn,un,gn,hn,dn,fn,pn,In,Cn,mn=function(t){var e,n,r,i,o,a,s;if(Te=Fe=-(Re=Ne=1/0),je=[],Ce(t,Ke),n=je.length){for(je.sort(tn),e=1,o=[r=je[0]];e<n;++e)en(r,(i=je[e])[0])||en(r,i[1])?($e(r[0],i[1])>$e(r[0],r[1])&&(r[1]=i[1]),$e(i[0],r[1])>$e(r[0],r[1])&&(r[0]=i[0])):o.push(r=i);for(a=-1/0,e=0,r=o[n=o.length-1];e<=n;r=i,++e)i=o[e],(s=$e(r[1],i[0]))>a&&(a=s,Re=i[0],Fe=r[1])}return je=Ye=null,Re===1/0||Ne===1/0?[[NaN,NaN],[NaN,NaN]]:[[Re,Ne],[Fe,Te]]},yn={sphere:se,point:En,lineStart:Bn,lineEnd:wn,polygonStart:function(){yn.lineStart=xn,yn.lineEnd=_n},polygonEnd:function(){yn.lineStart=Bn,yn.lineEnd=wn}};function En(t,e){t*=Kt;var n=Vt(e*=Kt);vn(n*Vt(t),n*te(t),te(e))}function vn(t,e,n){on+=(t-on)/++nn,an+=(e-an)/nn,sn+=(n-sn)/nn}function Bn(){yn.point=bn}function bn(t,e){t*=Kt;var n=Vt(e*=Kt);pn=n*Vt(t),In=n*te(t),Cn=te(e),yn.point=Qn,vn(pn,In,Cn)}function Qn(t,e){t*=Kt;var n=Vt(e*=Kt),r=n*Vt(t),i=n*te(t),o=te(e),a=qt(ne((a=In*o-Cn*i)*a+(a=Cn*r-pn*o)*a+(a=pn*i-In*r)*a),pn*r+In*i+Cn*o);rn+=a,An+=a*(pn+(pn=r)),ln+=a*(In+(In=i)),cn+=a*(Cn+(Cn=o)),vn(pn,In,Cn)}function wn(){yn.point=En}function xn(){yn.point=Dn}function _n(){Mn(dn,fn),yn.point=En}function Dn(t,e){dn=t,fn=e,t*=Kt,e*=Kt,yn.point=Mn;var n=Vt(e);pn=n*Vt(t),In=n*te(t),Cn=te(e),vn(pn,In,Cn)}function Mn(t,e){t*=Kt;var n=Vt(e*=Kt),r=n*Vt(t),i=n*te(t),o=te(e),a=In*o-Cn*i,s=Cn*r-pn*o,A=pn*i-In*r,l=ne(a*a+s*s+A*A),c=oe(l),u=l&&-c/l;un+=u*a,gn+=u*s,hn+=u*A,rn+=c,An+=c*(pn+(pn=r)),ln+=c*(In+(In=i)),cn+=c*(Cn+(Cn=o)),vn(pn,In,Cn)}var kn=function(t){nn=rn=on=an=sn=An=ln=cn=un=gn=hn=0,Ce(t,yn);var e=un,n=gn,r=hn,i=e*e+n*n+r*r;return i<Ot&&(e=An,n=ln,r=cn,rn<Ut&&(e=on,n=an,r=sn),(i=e*e+n*n+r*r)<Ot)?[NaN,NaN]:[qt(n,e)*Pt,oe(r/ne(i))*Pt]},Sn=function(t){return function(){return t}},Ln=function(t,e){function n(n,r){return n=t(n,r),e(n[0],n[1])}return t.invert&&e.invert&&(n.invert=function(n,r){return(n=e.invert(n,r))&&t.invert(n[0],n[1])}),n};function Rn(t,e){return[zt(t)>Gt?t+Math.round(-t/Yt)*Yt:t,e]}function Nn(t,e,n){return(t%=Yt)?e||n?Ln(Tn(t),Un(e,n)):Tn(t):e||n?Un(e,n):Rn}function Fn(t){return function(e,n){return[(e+=t)>Gt?e-Yt:e<-Gt?e+Yt:e,n]}}function Tn(t){var e=Fn(t);return e.invert=Fn(-t),e}function Un(t,e){var n=Vt(t),r=te(t),i=Vt(e),o=te(e);function a(t,e){var a=Vt(e),s=Vt(t)*a,A=te(t)*a,l=te(e),c=l*n+s*r;return[qt(A*i-c*o,s*n-l*r),oe(c*i+A*o)]}return a.invert=function(t,e){var a=Vt(e),s=Vt(t)*a,A=te(t)*a,l=te(e),c=l*i-A*o;return[qt(A*i+l*o,s*n+c*r),oe(c*n-s*r)]},a}Rn.invert=Rn;var On=function(t){function e(e){return(e=t(e[0]*Kt,e[1]*Kt))[0]*=Pt,e[1]*=Pt,e}return t=Nn(t[0]*Kt,t[1]*Kt,t.length>2?t[2]*Kt:0),e.invert=function(e){return(e=t.invert(e[0]*Kt,e[1]*Kt))[0]*=Pt,e[1]*=Pt,e},e};function Gn(t,e,n,r,i,o){if(n){var a=Vt(e),s=te(e),A=r*n;null==i?(i=e+r*Yt,o=e-A/2):(i=Hn(a,i),o=Hn(a,o),(r>0?i<o:i>o)&&(i+=r*Yt));for(var l,c=i;r>0?c>o:c<o;c-=A)l=xe([a,-s*Vt(c),-s*te(c)]),t.point(l[0],l[1])}}function Hn(t,e){(e=_e(e))[0]-=t,Le(e);var n=ie(-e[1]);return((-e[2]<0?-n:n)+Yt-Ut)%Yt}var jn=function(){var t,e,n=Sn([0,0]),r=Sn(90),i=Sn(6),o={point:function(n,r){t.push(n=e(n,r)),n[0]*=Pt,n[1]*=Pt}};function a(){var a=n.apply(this,arguments),s=r.apply(this,arguments)*Kt,A=i.apply(this,arguments)*Kt;return t=[],e=Nn(-a[0]*Kt,-a[1]*Kt,0).invert,Gn(o,s,A,1),a={type:"Polygon",coordinates:[t]},t=e=null,a}return a.center=function(t){return arguments.length?(n="function"==typeof t?t:Sn([+t[0],+t[1]]),a):n},a.radius=function(t){return arguments.length?(r="function"==typeof t?t:Sn(+t),a):r},a.precision=function(t){return arguments.length?(i="function"==typeof t?t:Sn(+t),a):i},a},Yn=function(){var t,e=[];return{point:function(e,n){t.push([e,n])},lineStart:function(){e.push(t=[])},lineEnd:se,rejoin:function(){e.length>1&&e.push(e.pop().concat(e.shift()))},result:function(){var n=e;return e=[],t=null,n}}},Pn=function(t,e){return zt(t[0]-e[0])<Ut&&zt(t[1]-e[1])<Ut};function Kn(t,e,n,r){this.x=t,this.z=e,this.o=n,this.e=r,this.v=!1,this.n=this.p=null}var zn=function(t,e,n,r,i){var o,a,s=[],A=[];if(t.forEach(function(t){if(!((e=t.length-1)<=0)){var e,n,r=t[0],a=t[e];if(Pn(r,a)){for(i.lineStart(),o=0;o<e;++o)i.point((r=t[o])[0],r[1]);i.lineEnd()}else s.push(n=new Kn(r,t,null,!0)),A.push(n.o=new Kn(r,null,n,!1)),s.push(n=new Kn(a,t,null,!1)),A.push(n.o=new Kn(a,null,n,!0))}}),s.length){for(A.sort(e),Jn(s),Jn(A),o=0,a=A.length;o<a;++o)A[o].e=n=!n;for(var l,c,u=s[0];;){for(var g=u,h=!0;g.v;)if((g=g.n)===u)return;l=g.z,i.lineStart();do{if(g.v=g.o.v=!0,g.e){if(h)for(o=0,a=l.length;o<a;++o)i.point((c=l[o])[0],c[1]);else r(g.x,g.n.x,1,i);g=g.n}else{if(h)for(l=g.p.z,o=l.length-1;o>=0;--o)i.point((c=l[o])[0],c[1]);else r(g.x,g.p.x,-1,i);g=g.p}l=(g=g.o).z,h=!h}while(!g.v);i.lineEnd()}}};function Jn(t){if(e=t.length){for(var e,n,r=0,i=t[0];++r<e;)i.n=n=t[r],n.p=i,i=n;i.n=n=t[0],n.p=i}}var qn=Rt();function Vn(t){return zt(t[0])<=Gt?t[0]:ee(t[0])*((zt(t[0])+Gt)%Yt-Gt)}var Wn=function(t,e){var n=Vn(e),r=e[1],i=te(r),o=[te(n),-Vt(n),0],a=0,s=0;qn.reset(),1===i?r=Ht+Ut:-1===i&&(r=-Ht-Ut);for(var A=0,l=t.length;A<l;++A)if(u=(c=t[A]).length)for(var c,u,g=c[u-1],h=Vn(g),d=g[1]/2+jt,f=te(d),p=Vt(d),I=0;I<u;++I,h=m,f=E,p=v,g=C){var C=c[I],m=Vn(C),y=C[1]/2+jt,E=te(y),v=Vt(y),B=m-h,b=B>=0?1:-1,Q=b*B,w=Q>Gt,x=f*E;if(qn.add(qt(x*b*te(Q),p*v+x*Vt(Q))),a+=w?B+b*Yt:B,w^h>=n^m>=n){var _=Me(_e(g),_e(C));Le(_);var D=Me(o,_);Le(D);var M=(w^B>=0?-1:1)*oe(D[2]);(r>M||r===M&&(_[0]||_[1]))&&(s+=w^B>=0?1:-1)}}return(a<-Ut||a<Ut&&qn<-Ut)^1&s},Xn=function(t,e,n,i){return function(o){var a,s,A,l=e(o),c=Yn(),u=e(c),g=!1,h={point:d,lineStart:p,lineEnd:I,polygonStart:function(){h.point=C,h.lineStart=m,h.lineEnd=y,s=[],a=[]},polygonEnd:function(){h.point=d,h.lineStart=p,h.lineEnd=I,s=Object(r.merge)(s);var t=Wn(a,i);s.length?(g||(o.polygonStart(),g=!0),zn(s,$n,t,n,o)):t&&(g||(o.polygonStart(),g=!0),o.lineStart(),n(null,null,1,o),o.lineEnd()),g&&(o.polygonEnd(),g=!1),s=a=null},sphere:function(){o.polygonStart(),o.lineStart(),n(null,null,1,o),o.lineEnd(),o.polygonEnd()}};function d(e,n){t(e,n)&&o.point(e,n)}function f(t,e){l.point(t,e)}function p(){h.point=f,l.lineStart()}function I(){h.point=d,l.lineEnd()}function C(t,e){A.push([t,e]),u.point(t,e)}function m(){u.lineStart(),A=[]}function y(){C(A[0][0],A[0][1]),u.lineEnd();var t,e,n,r,i=u.clean(),l=c.result(),h=l.length;if(A.pop(),a.push(A),A=null,h)if(1&i){if((e=(n=l[0]).length-1)>0){for(g||(o.polygonStart(),g=!0),o.lineStart(),t=0;t<e;++t)o.point((r=n[t])[0],r[1]);o.lineEnd()}}else h>1&&2&i&&l.push(l.pop().concat(l.shift())),s.push(l.filter(Zn))}return h}};function Zn(t){return t.length>1}function $n(t,e){return((t=t.x)[0]<0?t[1]-Ht-Ut:Ht-t[1])-((e=e.x)[0]<0?e[1]-Ht-Ut:Ht-e[1])}var tr=Xn(function(){return!0},function(t){var e,n=NaN,r=NaN,i=NaN;return{lineStart:function(){t.lineStart(),e=1},point:function(o,a){var s=o>0?Gt:-Gt,A=zt(o-n);zt(A-Gt)<Ut?(t.point(n,r=(r+a)/2>0?Ht:-Ht),t.point(i,r),t.lineEnd(),t.lineStart(),t.point(s,r),t.point(o,r),e=0):i!==s&&A>=Gt&&(zt(n-i)<Ut&&(n-=i*Ut),zt(o-s)<Ut&&(o-=s*Ut),r=function(t,e,n,r){var i,o,a=te(t-n);return zt(a)>Ut?Jt((te(e)*(o=Vt(r))*te(n)-te(r)*(i=Vt(e))*te(t))/(i*o*a)):(e+r)/2}(n,r,o,a),t.point(i,r),t.lineEnd(),t.lineStart(),t.point(s,r),e=0),t.point(n=o,r=a),i=s},lineEnd:function(){t.lineEnd(),n=r=NaN},clean:function(){return 2-e}}},function(t,e,n,r){var i;if(null==t)i=n*Ht,r.point(-Gt,i),r.point(0,i),r.point(Gt,i),r.point(Gt,0),r.point(Gt,-i),r.point(0,-i),r.point(-Gt,-i),r.point(-Gt,0),r.point(-Gt,i);else if(zt(t[0]-e[0])>Ut){var o=t[0]<e[0]?Gt:-Gt;i=n*o/2,r.point(-o,i),r.point(0,i),r.point(o,i)}else r.point(e[0],e[1])},[-Gt,-Ht]),er=function(t){var e=Vt(t),n=6*Kt,r=e>0,i=zt(e)>Ut;function o(t,n){return Vt(t)*Vt(n)>e}function a(t,n,r){var i=[1,0,0],o=Me(_e(t),_e(n)),a=De(o,o),s=o[0],A=a-s*s;if(!A)return!r&&t;var l=e*a/A,c=-e*s/A,u=Me(i,o),g=Se(i,l);ke(g,Se(o,c));var h=u,d=De(g,h),f=De(h,h),p=d*d-f*(De(g,g)-1);if(!(p<0)){var I=ne(p),C=Se(h,(-d-I)/f);if(ke(C,g),C=xe(C),!r)return C;var m,y=t[0],E=n[0],v=t[1],B=n[1];E<y&&(m=y,y=E,E=m);var b=E-y,Q=zt(b-Gt)<Ut;if(!Q&&B<v&&(m=v,v=B,B=m),Q||b<Ut?Q?v+B>0^C[1]<(zt(C[0]-y)<Ut?v:B):v<=C[1]&&C[1]<=B:b>Gt^(y<=C[0]&&C[0]<=E)){var w=Se(h,(-d+I)/f);return ke(w,g),[C,xe(w)]}}}function s(e,n){var i=r?t:Gt-t,o=0;return e<-i?o|=1:e>i&&(o|=2),n<-i?o|=4:n>i&&(o|=8),o}return Xn(o,function(t){var e,n,A,l,c;return{lineStart:function(){l=A=!1,c=1},point:function(u,g){var h,d=[u,g],f=o(u,g),p=r?f?0:s(u,g):f?s(u+(u<0?Gt:-Gt),g):0;if(!e&&(l=A=f)&&t.lineStart(),f!==A&&(!(h=a(e,d))||Pn(e,h)||Pn(d,h))&&(d[0]+=Ut,d[1]+=Ut,f=o(d[0],d[1])),f!==A)c=0,f?(t.lineStart(),h=a(d,e),t.point(h[0],h[1])):(h=a(e,d),t.point(h[0],h[1]),t.lineEnd()),e=h;else if(i&&e&&r^f){var I;p&n||!(I=a(d,e,!0))||(c=0,r?(t.lineStart(),t.point(I[0][0],I[0][1]),t.point(I[1][0],I[1][1]),t.lineEnd()):(t.point(I[1][0],I[1][1]),t.lineEnd(),t.lineStart(),t.point(I[0][0],I[0][1])))}!f||e&&Pn(e,d)||t.point(d[0],d[1]),e=d,A=f,n=p},lineEnd:function(){A&&t.lineEnd(),e=null},clean:function(){return c|(l&&A)<<1}}},function(e,r,i,o){Gn(o,t,n,i,e,r)},r?[0,-t]:[-Gt,t-Gt])},nr=function(t,e,n,r,i,o){var a,s=t[0],A=t[1],l=0,c=1,u=e[0]-s,g=e[1]-A;if(a=n-s,u||!(a>0)){if(a/=u,u<0){if(a<l)return;a<c&&(c=a)}else if(u>0){if(a>c)return;a>l&&(l=a)}if(a=i-s,u||!(a<0)){if(a/=u,u<0){if(a>c)return;a>l&&(l=a)}else if(u>0){if(a<l)return;a<c&&(c=a)}if(a=r-A,g||!(a>0)){if(a/=g,g<0){if(a<l)return;a<c&&(c=a)}else if(g>0){if(a>c)return;a>l&&(l=a)}if(a=o-A,g||!(a<0)){if(a/=g,g<0){if(a>c)return;a>l&&(l=a)}else if(g>0){if(a<l)return;a<c&&(c=a)}return l>0&&(t[0]=s+l*u,t[1]=A+l*g),c<1&&(e[0]=s+c*u,e[1]=A+c*g),!0}}}}},rr=1e9,ir=-rr;function or(t,e,n,i){function o(r,o){return t<=r&&r<=n&&e<=o&&o<=i}function a(r,o,a,A){var c=0,u=0;if(null==r||(c=s(r,a))!==(u=s(o,a))||l(r,o)<0^a>0)do{A.point(0===c||3===c?t:n,c>1?i:e)}while((c=(c+a+4)%4)!==u);else A.point(o[0],o[1])}function s(r,i){return zt(r[0]-t)<Ut?i>0?0:3:zt(r[0]-n)<Ut?i>0?2:1:zt(r[1]-e)<Ut?i>0?1:0:i>0?3:2}function A(t,e){return l(t.x,e.x)}function l(t,e){var n=s(t,1),r=s(e,1);return n!==r?n-r:0===n?e[1]-t[1]:1===n?t[0]-e[0]:2===n?t[1]-e[1]:e[0]-t[0]}return function(s){var l,c,u,g,h,d,f,p,I,C,m,y=s,E=Yn(),v={point:B,lineStart:function(){v.point=b,c&&c.push(u=[]),C=!0,I=!1,f=p=NaN},lineEnd:function(){l&&(b(g,h),d&&I&&E.rejoin(),l.push(E.result())),v.point=B,I&&y.lineEnd()},polygonStart:function(){y=E,l=[],c=[],m=!0},polygonEnd:function(){var e=function(){for(var e=0,n=0,r=c.length;n<r;++n)for(var o,a,s=c[n],A=1,l=s.length,u=s[0],g=u[0],h=u[1];A<l;++A)o=g,a=h,g=(u=s[A])[0],h=u[1],a<=i?h>i&&(g-o)*(i-a)>(h-a)*(t-o)&&++e:h<=i&&(g-o)*(i-a)<(h-a)*(t-o)&&--e;return e}(),n=m&&e,o=(l=Object(r.merge)(l)).length;(n||o)&&(s.polygonStart(),n&&(s.lineStart(),a(null,null,1,s),s.lineEnd()),o&&zn(l,A,e,a,s),s.polygonEnd()),y=s,l=c=u=null}};function B(t,e){o(t,e)&&y.point(t,e)}function b(r,a){var s=o(r,a);if(c&&u.push([r,a]),C)g=r,h=a,d=s,C=!1,s&&(y.lineStart(),y.point(r,a));else if(s&&I)y.point(r,a);else{var A=[f=Math.max(ir,Math.min(rr,f)),p=Math.max(ir,Math.min(rr,p))],l=[r=Math.max(ir,Math.min(rr,r)),a=Math.max(ir,Math.min(rr,a))];nr(A,l,t,e,n,i)?(I||(y.lineStart(),y.point(A[0],A[1])),y.point(l[0],l[1]),s||y.lineEnd(),m=!1):s&&(y.lineStart(),y.point(r,a),m=!1)}f=r,p=a,I=s}return v}}var ar,sr,Ar,lr=function(){var t,e,n,r=0,i=0,o=960,a=500;return n={stream:function(n){return t&&e===n?t:t=or(r,i,o,a)(e=n)},extent:function(s){return arguments.length?(r=+s[0][0],i=+s[0][1],o=+s[1][0],a=+s[1][1],t=e=null,n):[[r,i],[o,a]]}}},cr=Rt(),ur={sphere:se,point:se,lineStart:function(){ur.point=hr,ur.lineEnd=gr},lineEnd:se,polygonStart:se,polygonEnd:se};function gr(){ur.point=ur.lineEnd=se}function hr(t,e){ar=t*=Kt,sr=te(e*=Kt),Ar=Vt(e),ur.point=dr}function dr(t,e){t*=Kt;var n=te(e*=Kt),r=Vt(e),i=zt(t-ar),o=Vt(i),a=r*te(i),s=Ar*n-sr*r*o,A=sr*n+Ar*r*o;cr.add(qt(ne(a*a+s*s),A)),ar=t,sr=n,Ar=r}var fr=function(t){return cr.reset(),Ce(t,ur),+cr},pr=[null,null],Ir={type:"LineString",coordinates:pr},Cr=function(t,e){return pr[0]=t,pr[1]=e,fr(Ir)},mr={Feature:function(t,e){return Er(t.geometry,e)},FeatureCollection:function(t,e){for(var n=t.features,r=-1,i=n.length;++r<i;)if(Er(n[r].geometry,e))return!0;return!1}},yr={Sphere:function(){return!0},Point:function(t,e){return vr(t.coordinates,e)},MultiPoint:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)if(vr(n[r],e))return!0;return!1},LineString:function(t,e){return Br(t.coordinates,e)},MultiLineString:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)if(Br(n[r],e))return!0;return!1},Polygon:function(t,e){return br(t.coordinates,e)},MultiPolygon:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)if(br(n[r],e))return!0;return!1},GeometryCollection:function(t,e){for(var n=t.geometries,r=-1,i=n.length;++r<i;)if(Er(n[r],e))return!0;return!1}};function Er(t,e){return!(!t||!yr.hasOwnProperty(t.type))&&yr[t.type](t,e)}function vr(t,e){return 0===Cr(t,e)}function Br(t,e){for(var n,r,i,o=0,a=t.length;o<a;o++){if(0===(r=Cr(t[o],e)))return!0;if(o>0&&(i=Cr(t[o],t[o-1]))>0&&n<=i&&r<=i&&(n+r-i)*(1-Math.pow((n-r)/i,2))<Ot*i)return!0;n=r}return!1}function br(t,e){return!!Wn(t.map(Qr),wr(e))}function Qr(t){return(t=t.map(wr)).pop(),t}function wr(t){return[t[0]*Kt,t[1]*Kt]}var xr=function(t,e){return(t&&mr.hasOwnProperty(t.type)?mr[t.type]:Er)(t,e)};function _r(t,e,n){var i=Object(r.range)(t,e-Ut,n).concat(e);return function(t){return i.map(function(e){return[t,e]})}}function Dr(t,e,n){var i=Object(r.range)(t,e-Ut,n).concat(e);return function(t){return i.map(function(e){return[e,t]})}}function Mr(){var t,e,n,i,o,a,s,A,l,c,u,g,h=10,d=h,f=90,p=360,I=2.5;function C(){return{type:"MultiLineString",coordinates:m()}}function m(){return Object(r.range)(Wt(i/f)*f,n,f).map(u).concat(Object(r.range)(Wt(A/p)*p,s,p).map(g)).concat(Object(r.range)(Wt(e/h)*h,t,h).filter(function(t){return zt(t%f)>Ut}).map(l)).concat(Object(r.range)(Wt(a/d)*d,o,d).filter(function(t){return zt(t%p)>Ut}).map(c))}return C.lines=function(){return m().map(function(t){return{type:"LineString",coordinates:t}})},C.outline=function(){return{type:"Polygon",coordinates:[u(i).concat(g(s).slice(1),u(n).reverse().slice(1),g(A).reverse().slice(1))]}},C.extent=function(t){return arguments.length?C.extentMajor(t).extentMinor(t):C.extentMinor()},C.extentMajor=function(t){return arguments.length?(i=+t[0][0],n=+t[1][0],A=+t[0][1],s=+t[1][1],i>n&&(t=i,i=n,n=t),A>s&&(t=A,A=s,s=t),C.precision(I)):[[i,A],[n,s]]},C.extentMinor=function(n){return arguments.length?(e=+n[0][0],t=+n[1][0],a=+n[0][1],o=+n[1][1],e>t&&(n=e,e=t,t=n),a>o&&(n=a,a=o,o=n),C.precision(I)):[[e,a],[t,o]]},C.step=function(t){return arguments.length?C.stepMajor(t).stepMinor(t):C.stepMinor()},C.stepMajor=function(t){return arguments.length?(f=+t[0],p=+t[1],C):[f,p]},C.stepMinor=function(t){return arguments.length?(h=+t[0],d=+t[1],C):[h,d]},C.precision=function(r){return arguments.length?(I=+r,l=_r(a,o,90),c=Dr(e,t,I),u=_r(A,s,90),g=Dr(i,n,I),C):I},C.extentMajor([[-180,-90+Ut],[180,90-Ut]]).extentMinor([[-180,-80-Ut],[180,80+Ut]])}function kr(){return Mr()()}var Sr,Lr,Rr,Nr,Fr=function(t,e){var n=t[0]*Kt,r=t[1]*Kt,i=e[0]*Kt,o=e[1]*Kt,a=Vt(r),s=te(r),A=Vt(o),l=te(o),c=a*Vt(n),u=a*te(n),g=A*Vt(i),h=A*te(i),d=2*oe(ne(ae(o-r)+a*A*ae(i-n))),f=te(d),p=d?function(t){var e=te(t*=d)/f,n=te(d-t)/f,r=n*c+e*g,i=n*u+e*h,o=n*s+e*l;return[qt(i,r)*Pt,qt(o,ne(r*r+i*i))*Pt]}:function(){return[n*Pt,r*Pt]};return p.distance=d,p},Tr=function(t){return t},Ur=Rt(),Or=Rt(),Gr={point:se,lineStart:se,lineEnd:se,polygonStart:function(){Gr.lineStart=Hr,Gr.lineEnd=Pr},polygonEnd:function(){Gr.lineStart=Gr.lineEnd=Gr.point=se,Ur.add(zt(Or)),Or.reset()},result:function(){var t=Ur/2;return Ur.reset(),t}};function Hr(){Gr.point=jr}function jr(t,e){Gr.point=Yr,Sr=Rr=t,Lr=Nr=e}function Yr(t,e){Or.add(Nr*t-Rr*e),Rr=t,Nr=e}function Pr(){Yr(Sr,Lr)}var Kr,zr,Jr,qr,Vr=Gr,Wr=1/0,Xr=Wr,Zr=-Wr,$r=Zr,ti={point:function(t,e){t<Wr&&(Wr=t),t>Zr&&(Zr=t),e<Xr&&(Xr=e),e>$r&&($r=e)},lineStart:se,lineEnd:se,polygonStart:se,polygonEnd:se,result:function(){var t=[[Wr,Xr],[Zr,$r]];return Zr=$r=-(Xr=Wr=1/0),t}},ei=0,ni=0,ri=0,ii=0,oi=0,ai=0,si=0,Ai=0,li=0,ci={point:ui,lineStart:gi,lineEnd:fi,polygonStart:function(){ci.lineStart=pi,ci.lineEnd=Ii},polygonEnd:function(){ci.point=ui,ci.lineStart=gi,ci.lineEnd=fi},result:function(){var t=li?[si/li,Ai/li]:ai?[ii/ai,oi/ai]:ri?[ei/ri,ni/ri]:[NaN,NaN];return ei=ni=ri=ii=oi=ai=si=Ai=li=0,t}};function ui(t,e){ei+=t,ni+=e,++ri}function gi(){ci.point=hi}function hi(t,e){ci.point=di,ui(Jr=t,qr=e)}function di(t,e){var n=t-Jr,r=e-qr,i=ne(n*n+r*r);ii+=i*(Jr+t)/2,oi+=i*(qr+e)/2,ai+=i,ui(Jr=t,qr=e)}function fi(){ci.point=ui}function pi(){ci.point=Ci}function Ii(){mi(Kr,zr)}function Ci(t,e){ci.point=mi,ui(Kr=Jr=t,zr=qr=e)}function mi(t,e){var n=t-Jr,r=e-qr,i=ne(n*n+r*r);ii+=i*(Jr+t)/2,oi+=i*(qr+e)/2,ai+=i,si+=(i=qr*t-Jr*e)*(Jr+t),Ai+=i*(qr+e),li+=3*i,ui(Jr=t,qr=e)}var yi=ci;function Ei(t){this._context=t}Ei.prototype={_radius:4.5,pointRadius:function(t){return this._radius=t,this},polygonStart:function(){this._line=0},polygonEnd:function(){this._line=NaN},lineStart:function(){this._point=0},lineEnd:function(){0===this._line&&this._context.closePath(),this._point=NaN},point:function(t,e){switch(this._point){case 0:this._context.moveTo(t,e),this._point=1;break;case 1:this._context.lineTo(t,e);break;default:this._context.moveTo(t+this._radius,e),this._context.arc(t,e,this._radius,0,Yt)}},result:se};var vi,Bi,bi,Qi,wi,xi=Rt(),_i={point:se,lineStart:function(){_i.point=Di},lineEnd:function(){vi&&Mi(Bi,bi),_i.point=se},polygonStart:function(){vi=!0},polygonEnd:function(){vi=null},result:function(){var t=+xi;return xi.reset(),t}};function Di(t,e){_i.point=Mi,Bi=Qi=t,bi=wi=e}function Mi(t,e){Qi-=t,wi-=e,xi.add(ne(Qi*Qi+wi*wi)),Qi=t,wi=e}var ki=_i;function Si(){this._string=[]}function Li(t){return"m0,"+t+"a"+t+","+t+" 0 1,1 0,"+-2*t+"a"+t+","+t+" 0 1,1 0,"+2*t+"z"}Si.prototype={_radius:4.5,_circle:Li(4.5),pointRadius:function(t){return(t=+t)!==this._radius&&(this._radius=t,this._circle=null),this},polygonStart:function(){this._line=0},polygonEnd:function(){this._line=NaN},lineStart:function(){this._point=0},lineEnd:function(){0===this._line&&this._string.push("Z"),this._point=NaN},point:function(t,e){switch(this._point){case 0:this._string.push("M",t,",",e),this._point=1;break;case 1:this._string.push("L",t,",",e);break;default:null==this._circle&&(this._circle=Li(this._radius)),this._string.push("M",t,",",e,this._circle)}},result:function(){if(this._string.length){var t=this._string.join("");return this._string=[],t}return null}};var Ri=function(t,e){var n,r,i=4.5;function o(t){return t&&("function"==typeof i&&r.pointRadius(+i.apply(this,arguments)),Ce(t,n(r))),r.result()}return o.area=function(t){return Ce(t,n(Vr)),Vr.result()},o.measure=function(t){return Ce(t,n(ki)),ki.result()},o.bounds=function(t){return Ce(t,n(ti)),ti.result()},o.centroid=function(t){return Ce(t,n(yi)),yi.result()},o.projection=function(e){return arguments.length?(n=null==e?(t=null,Tr):(t=e).stream,o):t},o.context=function(t){return arguments.length?(r=null==t?(e=null,new Si):new Ei(e=t),"function"!=typeof i&&r.pointRadius(i),o):e},o.pointRadius=function(t){return arguments.length?(i="function"==typeof t?t:(r.pointRadius(+t),+t),o):i},o.projection(t).context(e)},Ni=function(t){return{stream:Fi(t)}};function Fi(t){return function(e){var n=new Ti;for(var r in t)n[r]=t[r];return n.stream=e,n}}function Ti(){}function Ui(t,e,n){var r=t.clipExtent&&t.clipExtent();return t.scale(150).translate([0,0]),null!=r&&t.clipExtent(null),Ce(n,t.stream(ti)),e(ti.result()),null!=r&&t.clipExtent(r),t}function Oi(t,e,n){return Ui(t,function(n){var r=e[1][0]-e[0][0],i=e[1][1]-e[0][1],o=Math.min(r/(n[1][0]-n[0][0]),i/(n[1][1]-n[0][1])),a=+e[0][0]+(r-o*(n[1][0]+n[0][0]))/2,s=+e[0][1]+(i-o*(n[1][1]+n[0][1]))/2;t.scale(150*o).translate([a,s])},n)}function Gi(t,e,n){return Oi(t,[[0,0],e],n)}function Hi(t,e,n){return Ui(t,function(n){var r=+e,i=r/(n[1][0]-n[0][0]),o=(r-i*(n[1][0]+n[0][0]))/2,a=-i*n[0][1];t.scale(150*i).translate([o,a])},n)}function ji(t,e,n){return Ui(t,function(n){var r=+e,i=r/(n[1][1]-n[0][1]),o=-i*n[0][0],a=(r-i*(n[1][1]+n[0][1]))/2;t.scale(150*i).translate([o,a])},n)}Ti.prototype={constructor:Ti,point:function(t,e){this.stream.point(t,e)},sphere:function(){this.stream.sphere()},lineStart:function(){this.stream.lineStart()},lineEnd:function(){this.stream.lineEnd()},polygonStart:function(){this.stream.polygonStart()},polygonEnd:function(){this.stream.polygonEnd()}};var Yi=16,Pi=Vt(30*Kt),Ki=function(t,e){return+e?function(t,e){function n(r,i,o,a,s,A,l,c,u,g,h,d,f,p){var I=l-r,C=c-i,m=I*I+C*C;if(m>4*e&&f--){var y=a+g,E=s+h,v=A+d,B=ne(y*y+E*E+v*v),b=oe(v/=B),Q=zt(zt(v)-1)<Ut||zt(o-u)<Ut?(o+u)/2:qt(E,y),w=t(Q,b),x=w[0],_=w[1],D=x-r,M=_-i,k=C*D-I*M;(k*k/m>e||zt((I*D+C*M)/m-.5)>.3||a*g+s*h+A*d<Pi)&&(n(r,i,o,a,s,A,x,_,Q,y/=B,E/=B,v,f,p),p.point(x,_),n(x,_,Q,y,E,v,l,c,u,g,h,d,f,p))}}return function(e){var r,i,o,a,s,A,l,c,u,g,h,d,f={point:p,lineStart:I,lineEnd:m,polygonStart:function(){e.polygonStart(),f.lineStart=y},polygonEnd:function(){e.polygonEnd(),f.lineStart=I}};function p(n,r){n=t(n,r),e.point(n[0],n[1])}function I(){c=NaN,f.point=C,e.lineStart()}function C(r,i){var o=_e([r,i]),a=t(r,i);n(c,u,l,g,h,d,c=a[0],u=a[1],l=r,g=o[0],h=o[1],d=o[2],Yi,e),e.point(c,u)}function m(){f.point=p,e.lineEnd()}function y(){I(),f.point=E,f.lineEnd=v}function E(t,e){C(r=t,e),i=c,o=u,a=g,s=h,A=d,f.point=C}function v(){n(c,u,l,g,h,d,i,o,r,a,s,A,Yi,e),f.lineEnd=m,m()}return f}}(t,e):function(t){return Fi({point:function(e,n){e=t(e,n),this.stream.point(e[0],e[1])}})}(t)},zi=Fi({point:function(t,e){this.stream.point(t*Kt,e*Kt)}});function Ji(t,e,n,r){var i=Vt(r),o=te(r),a=i*t,s=o*t,A=i/t,l=o/t,c=(o*n-i*e)/t,u=(o*e+i*n)/t;function g(t,r){return[a*t-s*r+e,n-s*t-a*r]}return g.invert=function(t,e){return[A*t-l*e+c,u-l*t-A*e]},g}function qi(t){return Vi(function(){return t})()}function Vi(t){var e,n,r,i,o,a,s,A,l,c,u=150,g=480,h=250,d=0,f=0,p=0,I=0,C=0,m=0,y=null,E=tr,v=null,B=Tr,b=.5;function Q(t){return A(t[0]*Kt,t[1]*Kt)}function w(t){return(t=A.invert(t[0],t[1]))&&[t[0]*Pt,t[1]*Pt]}function x(){var t=Ji(u,0,0,m).apply(null,e(d,f)),r=(m?Ji:function(t,e,n){function r(r,i){return[e+t*r,n-t*i]}return r.invert=function(r,i){return[(r-e)/t,(n-i)/t]},r})(u,g-t[0],h-t[1],m);return n=Nn(p,I,C),s=Ln(e,r),A=Ln(n,s),a=Ki(s,b),_()}function _(){return l=c=null,Q}return Q.stream=function(t){return l&&c===t?l:l=zi(function(t){return Fi({point:function(e,n){var r=t(e,n);return this.stream.point(r[0],r[1])}})}(n)(E(a(B(c=t)))))},Q.preclip=function(t){return arguments.length?(E=t,y=void 0,_()):E},Q.postclip=function(t){return arguments.length?(B=t,v=r=i=o=null,_()):B},Q.clipAngle=function(t){return arguments.length?(E=+t?er(y=t*Kt):(y=null,tr),_()):y*Pt},Q.clipExtent=function(t){return arguments.length?(B=null==t?(v=r=i=o=null,Tr):or(v=+t[0][0],r=+t[0][1],i=+t[1][0],o=+t[1][1]),_()):null==v?null:[[v,r],[i,o]]},Q.scale=function(t){return arguments.length?(u=+t,x()):u},Q.translate=function(t){return arguments.length?(g=+t[0],h=+t[1],x()):[g,h]},Q.center=function(t){return arguments.length?(d=t[0]%360*Kt,f=t[1]%360*Kt,x()):[d*Pt,f*Pt]},Q.rotate=function(t){return arguments.length?(p=t[0]%360*Kt,I=t[1]%360*Kt,C=t.length>2?t[2]%360*Kt:0,x()):[p*Pt,I*Pt,C*Pt]},Q.angle=function(t){return arguments.length?(m=t%360*Kt,x()):m*Pt},Q.precision=function(t){return arguments.length?(a=Ki(s,b=t*t),_()):ne(b)},Q.fitExtent=function(t,e){return Oi(Q,t,e)},Q.fitSize=function(t,e){return Gi(Q,t,e)},Q.fitWidth=function(t,e){return Hi(Q,t,e)},Q.fitHeight=function(t,e){return ji(Q,t,e)},function(){return e=t.apply(this,arguments),Q.invert=e.invert&&w,x()}}function Wi(t){var e=0,n=Gt/3,r=Vi(t),i=r(e,n);return i.parallels=function(t){return arguments.length?r(e=t[0]*Kt,n=t[1]*Kt):[e*Pt,n*Pt]},i}function Xi(t,e){var n=te(t),r=(n+te(e))/2;if(zt(r)<Ut)return function(t){var e=Vt(t);function n(t,n){return[t*e,te(n)/e]}return n.invert=function(t,n){return[t/e,oe(n*e)]},n}(t);var i=1+n*(2*r-n),o=ne(i)/r;function a(t,e){var n=ne(i-2*r*te(e))/r;return[n*te(t*=r),o-n*Vt(t)]}return a.invert=function(t,e){var n=o-e;return[qt(t,zt(n))/r*ee(n),oe((i-(t*t+n*n)*r*r)/(2*r))]},a}var Zi=function(){return Wi(Xi).scale(155.424).center([0,33.6442])},$i=function(){return Zi().parallels([29.5,45.5]).scale(1070).translate([480,250]).rotate([96,0]).center([-.6,38.7])},to=function(){var t,e,n,r,i,o,a=$i(),s=Zi().rotate([154,0]).center([-2,58.5]).parallels([55,65]),A=Zi().rotate([157,0]).center([-3,19.9]).parallels([8,18]),l={point:function(t,e){o=[t,e]}};function c(t){var e=t[0],a=t[1];return o=null,n.point(e,a),o||(r.point(e,a),o)||(i.point(e,a),o)}function u(){return t=e=null,c}return c.invert=function(t){var e=a.scale(),n=a.translate(),r=(t[0]-n[0])/e,i=(t[1]-n[1])/e;return(i>=.12&&i<.234&&r>=-.425&&r<-.214?s:i>=.166&&i<.234&&r>=-.214&&r<-.115?A:a).invert(t)},c.stream=function(n){return t&&e===n?t:(r=[a.stream(e=n),s.stream(n),A.stream(n)],i=r.length,t={point:function(t,e){for(var n=-1;++n<i;)r[n].point(t,e)},sphere:function(){for(var t=-1;++t<i;)r[t].sphere()},lineStart:function(){for(var t=-1;++t<i;)r[t].lineStart()},lineEnd:function(){for(var t=-1;++t<i;)r[t].lineEnd()},polygonStart:function(){for(var t=-1;++t<i;)r[t].polygonStart()},polygonEnd:function(){for(var t=-1;++t<i;)r[t].polygonEnd()}});var r,i},c.precision=function(t){return arguments.length?(a.precision(t),s.precision(t),A.precision(t),u()):a.precision()},c.scale=function(t){return arguments.length?(a.scale(t),s.scale(.35*t),A.scale(t),c.translate(a.translate())):a.scale()},c.translate=function(t){if(!arguments.length)return a.translate();var e=a.scale(),o=+t[0],c=+t[1];return n=a.translate(t).clipExtent([[o-.455*e,c-.238*e],[o+.455*e,c+.238*e]]).stream(l),r=s.translate([o-.307*e,c+.201*e]).clipExtent([[o-.425*e+Ut,c+.12*e+Ut],[o-.214*e-Ut,c+.234*e-Ut]]).stream(l),i=A.translate([o-.205*e,c+.212*e]).clipExtent([[o-.214*e+Ut,c+.166*e+Ut],[o-.115*e-Ut,c+.234*e-Ut]]).stream(l),u()},c.fitExtent=function(t,e){return Oi(c,t,e)},c.fitSize=function(t,e){return Gi(c,t,e)},c.fitWidth=function(t,e){return Hi(c,t,e)},c.fitHeight=function(t,e){return ji(c,t,e)},c.scale(1070)};function eo(t){return function(e,n){var r=Vt(e),i=Vt(n),o=t(r*i);return[o*i*te(e),o*te(n)]}}function no(t){return function(e,n){var r=ne(e*e+n*n),i=t(r),o=te(i),a=Vt(i);return[qt(e*o,r*a),oe(r&&n*o/r)]}}var ro=eo(function(t){return ne(2/(1+t))});ro.invert=no(function(t){return 2*oe(t/2)});var io=function(){return qi(ro).scale(124.75).clipAngle(179.999)},oo=eo(function(t){return(t=ie(t))&&t/te(t)});oo.invert=no(function(t){return t});var ao=function(){return qi(oo).scale(79.4188).clipAngle(179.999)};function so(t,e){return[t,Zt(re((Ht+e)/2))]}so.invert=function(t,e){return[t,2*Jt(Xt(e))-Ht]};var Ao=function(){return lo(so).scale(961/Yt)};function lo(t){var e,n,r,i=qi(t),o=i.center,a=i.scale,s=i.translate,A=i.clipExtent,l=null;function c(){var o=Gt*a(),s=i(On(i.rotate()).invert([0,0]));return A(null==l?[[s[0]-o,s[1]-o],[s[0]+o,s[1]+o]]:t===so?[[Math.max(s[0]-o,l),e],[Math.min(s[0]+o,n),r]]:[[l,Math.max(s[1]-o,e)],[n,Math.min(s[1]+o,r)]])}return i.scale=function(t){return arguments.length?(a(t),c()):a()},i.translate=function(t){return arguments.length?(s(t),c()):s()},i.center=function(t){return arguments.length?(o(t),c()):o()},i.clipExtent=function(t){return arguments.length?(null==t?l=e=n=r=null:(l=+t[0][0],e=+t[0][1],n=+t[1][0],r=+t[1][1]),c()):null==l?null:[[l,e],[n,r]]},c()}function co(t){return re((Ht+t)/2)}function uo(t,e){var n=Vt(t),r=t===e?te(t):Zt(n/Vt(e))/Zt(co(e)/co(t)),i=n*$t(co(t),r)/r;if(!r)return so;function o(t,e){i>0?e<-Ht+Ut&&(e=-Ht+Ut):e>Ht-Ut&&(e=Ht-Ut);var n=i/$t(co(e),r);return[n*te(r*t),i-n*Vt(r*t)]}return o.invert=function(t,e){var n=i-e,o=ee(r)*ne(t*t+n*n);return[qt(t,zt(n))/r*ee(n),2*Jt($t(i/o,1/r))-Ht]},o}var go=function(){return Wi(uo).scale(109.5).parallels([30,30])};function ho(t,e){return[t,e]}ho.invert=ho;var fo=function(){return qi(ho).scale(152.63)};function po(t,e){var n=Vt(t),r=t===e?te(t):(n-Vt(e))/(e-t),i=n/r+t;if(zt(r)<Ut)return ho;function o(t,e){var n=i-e,o=r*t;return[n*te(o),i-n*Vt(o)]}return o.invert=function(t,e){var n=i-e;return[qt(t,zt(n))/r*ee(n),i-ee(r)*ne(t*t+n*n)]},o}var Io=function(){return Wi(po).scale(131.154).center([0,13.9389])},Co=1.340264,mo=-.081106,yo=893e-6,Eo=.003796,vo=ne(3)/2;function Bo(t,e){var n=oe(vo*te(e)),r=n*n,i=r*r*r;return[t*Vt(n)/(vo*(Co+3*mo*r+i*(7*yo+9*Eo*r))),n*(Co+mo*r+i*(yo+Eo*r))]}Bo.invert=function(t,e){for(var n,r=e,i=r*r,o=i*i*i,a=0;a<12&&(o=(i=(r-=n=(r*(Co+mo*i+o*(yo+Eo*i))-e)/(Co+3*mo*i+o*(7*yo+9*Eo*i)))*r)*i*i,!(zt(n)<Ot));++a);return[vo*t*(Co+3*mo*i+o*(7*yo+9*Eo*i))/Vt(r),oe(te(r)/vo)]};var bo=function(){return qi(Bo).scale(177.158)};function Qo(t,e){var n=Vt(e),r=Vt(t)*n;return[n*te(t)/r,te(e)/r]}Qo.invert=no(Jt);var wo=function(){return qi(Qo).scale(144.049).clipAngle(60)};function xo(t,e,n,r){return 1===t&&1===e&&0===n&&0===r?Tr:Fi({point:function(i,o){this.stream.point(i*t+n,o*e+r)}})}var _o=function(){var t,e,n,r,i,o,a=1,s=0,A=0,l=1,c=1,u=Tr,g=null,h=Tr;function d(){return r=i=null,o}return o={stream:function(t){return r&&i===t?r:r=u(h(i=t))},postclip:function(r){return arguments.length?(h=r,g=t=e=n=null,d()):h},clipExtent:function(r){return arguments.length?(h=null==r?(g=t=e=n=null,Tr):or(g=+r[0][0],t=+r[0][1],e=+r[1][0],n=+r[1][1]),d()):null==g?null:[[g,t],[e,n]]},scale:function(t){return arguments.length?(u=xo((a=+t)*l,a*c,s,A),d()):a},translate:function(t){return arguments.length?(u=xo(a*l,a*c,s=+t[0],A=+t[1]),d()):[s,A]},reflectX:function(t){return arguments.length?(u=xo(a*(l=t?-1:1),a*c,s,A),d()):l<0},reflectY:function(t){return arguments.length?(u=xo(a*l,a*(c=t?-1:1),s,A),d()):c<0},fitExtent:function(t,e){return Oi(o,t,e)},fitSize:function(t,e){return Gi(o,t,e)},fitWidth:function(t,e){return Hi(o,t,e)},fitHeight:function(t,e){return ji(o,t,e)}}};function Do(t,e){var n=e*e,r=n*n;return[t*(.8707-.131979*n+r*(r*(.003971*n-.001529*r)-.013791)),e*(1.007226+n*(.015085+r*(.028874*n-.044475-.005916*r)))]}Do.invert=function(t,e){var n,r=e,i=25;do{var o=r*r,a=o*o;r-=n=(r*(1.007226+o*(.015085+a*(.028874*o-.044475-.005916*a)))-e)/(1.007226+o*(.045255+a*(.259866*o-.311325-.005916*11*a)))}while(zt(n)>Ut&&--i>0);return[t/(.8707+(o=r*r)*(o*(o*o*o*(.003971-.001529*o)-.013791)-.131979)),r]};var Mo=function(){return qi(Do).scale(175.295)};function ko(t,e){return[Vt(e)*te(t),te(e)]}ko.invert=no(oe);var So=function(){return qi(ko).scale(249.5).clipAngle(90+Ut)};function Lo(t,e){var n=Vt(e),r=1+Vt(t)*n;return[n*te(t)/r,te(e)/r]}Lo.invert=no(function(t){return 2*Jt(t)});var Ro=function(){return qi(Lo).scale(250).clipAngle(142)};function No(t,e){return[Zt(re((Ht+e)/2)),-t]}No.invert=function(t,e){return[-e,2*Jt(Xt(t))-Ht]};var Fo=function(){var t=lo(No),e=t.center,n=t.rotate;return t.center=function(t){return arguments.length?e([-t[1],t[0]]):[(t=e())[1],-t[0]]},t.rotate=function(t){return arguments.length?n([t[0],t[1],t.length>2?t[2]+90:90]):[(t=n())[0],t[1],t[2]-90]},n([0,0,90]).scale(159.155)};function To(t,e){return t.parent===e.parent?1:2}function Uo(t,e){return t+e.x}function Oo(t,e){return Math.max(t,e.y)}var Go=function(){var t=To,e=1,n=1,r=!1;function i(i){var o,a=0;i.eachAfter(function(e){var n=e.children;n?(e.x=function(t){return t.reduce(Uo,0)/t.length}(n),e.y=1+n.reduce(Oo,0)):(e.x=o?a+=t(e,o):0,e.y=0,o=e)});var s=function(t){for(var e;e=t.children;)t=e[0];return t}(i),A=function(t){for(var e;e=t.children;)t=e[e.length-1];return t}(i),l=s.x-t(s,A)/2,c=A.x+t(A,s)/2;return i.eachAfter(r?function(t){t.x=(t.x-i.x)*e,t.y=(i.y-t.y)*n}:function(t){t.x=(t.x-l)/(c-l)*e,t.y=(1-(i.y?t.y/i.y:1))*n})}return i.separation=function(e){return arguments.length?(t=e,i):t},i.size=function(t){return arguments.length?(r=!1,e=+t[0],n=+t[1],i):r?null:[e,n]},i.nodeSize=function(t){return arguments.length?(r=!0,e=+t[0],n=+t[1],i):r?[e,n]:null},i};function Ho(t){var e=0,n=t.children,r=n&&n.length;if(r)for(;--r>=0;)e+=n[r].value;else e=1;t.value=e}function jo(t,e){var n,r,i,o,a,s=new zo(t),A=+t.value&&(s.value=t.value),l=[s];for(null==e&&(e=Yo);n=l.pop();)if(A&&(n.value=+n.data.value),(i=e(n.data))&&(a=i.length))for(n.children=new Array(a),o=a-1;o>=0;--o)l.push(r=n.children[o]=new zo(i[o])),r.parent=n,r.depth=n.depth+1;return s.eachBefore(Ko)}function Yo(t){return t.children}function Po(t){t.data=t.data.data}function Ko(t){var e=0;do{t.height=e}while((t=t.parent)&&t.height<++e)}function zo(t){this.data=t,this.depth=this.height=0,this.parent=null}zo.prototype=jo.prototype={constructor:zo,count:function(){return this.eachAfter(Ho)},each:function(t){var e,n,r,i,o=this,a=[o];do{for(e=a.reverse(),a=[];o=e.pop();)if(t(o),n=o.children)for(r=0,i=n.length;r<i;++r)a.push(n[r])}while(a.length);return this},eachAfter:function(t){for(var e,n,r,i=this,o=[i],a=[];i=o.pop();)if(a.push(i),e=i.children)for(n=0,r=e.length;n<r;++n)o.push(e[n]);for(;i=a.pop();)t(i);return this},eachBefore:function(t){for(var e,n,r=this,i=[r];r=i.pop();)if(t(r),e=r.children)for(n=e.length-1;n>=0;--n)i.push(e[n]);return this},sum:function(t){return this.eachAfter(function(e){for(var n=+t(e.data)||0,r=e.children,i=r&&r.length;--i>=0;)n+=r[i].value;e.value=n})},sort:function(t){return this.eachBefore(function(e){e.children&&e.children.sort(t)})},path:function(t){for(var e=this,n=function(t,e){if(t===e)return t;var n=t.ancestors(),r=e.ancestors(),i=null;for(t=n.pop(),e=r.pop();t===e;)i=t,t=n.pop(),e=r.pop();return i}(e,t),r=[e];e!==n;)e=e.parent,r.push(e);for(var i=r.length;t!==n;)r.splice(i,0,t),t=t.parent;return r},ancestors:function(){for(var t=this,e=[t];t=t.parent;)e.push(t);return e},descendants:function(){var t=[];return this.each(function(e){t.push(e)}),t},leaves:function(){var t=[];return this.eachBefore(function(e){e.children||t.push(e)}),t},links:function(){var t=this,e=[];return t.each(function(n){n!==t&&e.push({source:n.parent,target:n})}),e},copy:function(){return jo(this).eachBefore(Po)}};var Jo=Array.prototype.slice,qo=function(t){for(var e,n,r=0,i=(t=function(t){for(var e,n,r=t.length;r;)n=Math.random()*r--|0,e=t[r],t[r]=t[n],t[n]=e;return t}(Jo.call(t))).length,o=[];r<i;)e=t[r],n&&Xo(n,e)?++r:(n=$o(o=Vo(o,e)),r=0);return n};function Vo(t,e){var n,r;if(Zo(e,t))return[e];for(n=0;n<t.length;++n)if(Wo(e,t[n])&&Zo(ta(t[n],e),t))return[t[n],e];for(n=0;n<t.length-1;++n)for(r=n+1;r<t.length;++r)if(Wo(ta(t[n],t[r]),e)&&Wo(ta(t[n],e),t[r])&&Wo(ta(t[r],e),t[n])&&Zo(ea(t[n],t[r],e),t))return[t[n],t[r],e];throw new Error}function Wo(t,e){var n=t.r-e.r,r=e.x-t.x,i=e.y-t.y;return n<0||n*n<r*r+i*i}function Xo(t,e){var n=t.r-e.r+1e-6,r=e.x-t.x,i=e.y-t.y;return n>0&&n*n>r*r+i*i}function Zo(t,e){for(var n=0;n<e.length;++n)if(!Xo(t,e[n]))return!1;return!0}function $o(t){switch(t.length){case 1:return{x:(e=t[0]).x,y:e.y,r:e.r};case 2:return ta(t[0],t[1]);case 3:return ea(t[0],t[1],t[2])}var e}function ta(t,e){var n=t.x,r=t.y,i=t.r,o=e.x,a=e.y,s=e.r,A=o-n,l=a-r,c=s-i,u=Math.sqrt(A*A+l*l);return{x:(n+o+A/u*c)/2,y:(r+a+l/u*c)/2,r:(u+i+s)/2}}function ea(t,e,n){var r=t.x,i=t.y,o=t.r,a=e.x,s=e.y,A=e.r,l=n.x,c=n.y,u=n.r,g=r-a,h=r-l,d=i-s,f=i-c,p=A-o,I=u-o,C=r*r+i*i-o*o,m=C-a*a-s*s+A*A,y=C-l*l-c*c+u*u,E=h*d-g*f,v=(d*y-f*m)/(2*E)-r,B=(f*p-d*I)/E,b=(h*m-g*y)/(2*E)-i,Q=(g*I-h*p)/E,w=B*B+Q*Q-1,x=2*(o+v*B+b*Q),_=v*v+b*b-o*o,D=-(w?(x+Math.sqrt(x*x-4*w*_))/(2*w):_/x);return{x:r+v+B*D,y:i+b+Q*D,r:D}}function na(t,e,n){var r,i,o,a,s=t.x-e.x,A=t.y-e.y,l=s*s+A*A;l?(i=e.r+n.r,i*=i,a=t.r+n.r,i>(a*=a)?(r=(l+a-i)/(2*l),o=Math.sqrt(Math.max(0,a/l-r*r)),n.x=t.x-r*s-o*A,n.y=t.y-r*A+o*s):(r=(l+i-a)/(2*l),o=Math.sqrt(Math.max(0,i/l-r*r)),n.x=e.x+r*s-o*A,n.y=e.y+r*A+o*s)):(n.x=e.x+n.r,n.y=e.y)}function ra(t,e){var n=t.r+e.r-1e-6,r=e.x-t.x,i=e.y-t.y;return n>0&&n*n>r*r+i*i}function ia(t){var e=t._,n=t.next._,r=e.r+n.r,i=(e.x*n.r+n.x*e.r)/r,o=(e.y*n.r+n.y*e.r)/r;return i*i+o*o}function oa(t){this._=t,this.next=null,this.previous=null}function aa(t){if(!(i=t.length))return 0;var e,n,r,i,o,a,s,A,l,c,u;if((e=t[0]).x=0,e.y=0,!(i>1))return e.r;if(n=t[1],e.x=-n.r,n.x=e.r,n.y=0,!(i>2))return e.r+n.r;na(n,e,r=t[2]),e=new oa(e),n=new oa(n),r=new oa(r),e.next=r.previous=n,n.next=e.previous=r,r.next=n.previous=e;t:for(s=3;s<i;++s){na(e._,n._,r=t[s]),r=new oa(r),A=n.next,l=e.previous,c=n._.r,u=e._.r;do{if(c<=u){if(ra(A._,r._)){n=A,e.next=n,n.previous=e,--s;continue t}c+=A._.r,A=A.next}else{if(ra(l._,r._)){(e=l).next=n,n.previous=e,--s;continue t}u+=l._.r,l=l.previous}}while(A!==l.next);for(r.previous=e,r.next=n,e.next=n.previous=n=r,o=ia(e);(r=r.next)!==n;)(a=ia(r))<o&&(e=r,o=a);n=e.next}for(e=[n._],r=n;(r=r.next)!==n;)e.push(r._);for(r=qo(e),s=0;s<i;++s)(e=t[s]).x-=r.x,e.y-=r.y;return r.r}var sa=function(t){return aa(t),t};function Aa(t){return null==t?null:la(t)}function la(t){if("function"!=typeof t)throw new Error;return t}function ca(){return 0}var ua=function(t){return function(){return t}};function ga(t){return Math.sqrt(t.value)}var ha=function(){var t=null,e=1,n=1,r=ca;function i(i){return i.x=e/2,i.y=n/2,t?i.eachBefore(da(t)).eachAfter(fa(r,.5)).eachBefore(pa(1)):i.eachBefore(da(ga)).eachAfter(fa(ca,1)).eachAfter(fa(r,i.r/Math.min(e,n))).eachBefore(pa(Math.min(e,n)/(2*i.r))),i}return i.radius=function(e){return arguments.length?(t=Aa(e),i):t},i.size=function(t){return arguments.length?(e=+t[0],n=+t[1],i):[e,n]},i.padding=function(t){return arguments.length?(r="function"==typeof t?t:ua(+t),i):r},i};function da(t){return function(e){e.children||(e.r=Math.max(0,+t(e)||0))}}function fa(t,e){return function(n){if(r=n.children){var r,i,o,a=r.length,s=t(n)*e||0;if(s)for(i=0;i<a;++i)r[i].r+=s;if(o=aa(r),s)for(i=0;i<a;++i)r[i].r-=s;n.r=o+s}}}function pa(t){return function(e){var n=e.parent;e.r*=t,n&&(e.x=n.x+t*e.x,e.y=n.y+t*e.y)}}var Ia=function(t){t.x0=Math.round(t.x0),t.y0=Math.round(t.y0),t.x1=Math.round(t.x1),t.y1=Math.round(t.y1)},Ca=function(t,e,n,r,i){for(var o,a=t.children,s=-1,A=a.length,l=t.value&&(r-e)/t.value;++s<A;)(o=a[s]).y0=n,o.y1=i,o.x0=e,o.x1=e+=o.value*l},ma=function(){var t=1,e=1,n=0,r=!1;function i(i){var o=i.height+1;return i.x0=i.y0=n,i.x1=t,i.y1=e/o,i.eachBefore(function(t,e){return function(r){r.children&&Ca(r,r.x0,t*(r.depth+1)/e,r.x1,t*(r.depth+2)/e);var i=r.x0,o=r.y0,a=r.x1-n,s=r.y1-n;a<i&&(i=a=(i+a)/2),s<o&&(o=s=(o+s)/2),r.x0=i,r.y0=o,r.x1=a,r.y1=s}}(e,o)),r&&i.eachBefore(Ia),i}return i.round=function(t){return arguments.length?(r=!!t,i):r},i.size=function(n){return arguments.length?(t=+n[0],e=+n[1],i):[t,e]},i.padding=function(t){return arguments.length?(n=+t,i):n},i},ya="$",Ea={depth:-1},va={};function Ba(t){return t.id}function ba(t){return t.parentId}var Qa=function(){var t=Ba,e=ba;function n(n){var r,i,o,a,s,A,l,c=n.length,u=new Array(c),g={};for(i=0;i<c;++i)r=n[i],s=u[i]=new zo(r),null!=(A=t(r,i,n))&&(A+="")&&(g[l=ya+(s.id=A)]=l in g?va:s);for(i=0;i<c;++i)if(s=u[i],null!=(A=e(n[i],i,n))&&(A+="")){if(!(a=g[ya+A]))throw new Error("missing: "+A);if(a===va)throw new Error("ambiguous: "+A);a.children?a.children.push(s):a.children=[s],s.parent=a}else{if(o)throw new Error("multiple roots");o=s}if(!o)throw new Error("no root");if(o.parent=Ea,o.eachBefore(function(t){t.depth=t.parent.depth+1,--c}).eachBefore(Ko),o.parent=null,c>0)throw new Error("cycle");return o}return n.id=function(e){return arguments.length?(t=la(e),n):t},n.parentId=function(t){return arguments.length?(e=la(t),n):e},n};function wa(t,e){return t.parent===e.parent?1:2}function xa(t){var e=t.children;return e?e[0]:t.t}function _a(t){var e=t.children;return e?e[e.length-1]:t.t}function Da(t,e,n){var r=n/(e.i-t.i);e.c-=r,e.s+=n,t.c+=r,e.z+=n,e.m+=n}function Ma(t,e,n){return t.a.parent===e.parent?t.a:n}function ka(t,e){this._=t,this.parent=null,this.children=null,this.A=null,this.a=this,this.z=0,this.m=0,this.c=0,this.s=0,this.t=null,this.i=e}ka.prototype=Object.create(zo.prototype);var Sa=function(){var t=wa,e=1,n=1,r=null;function i(i){var A=function(t){for(var e,n,r,i,o,a=new ka(t,0),s=[a];e=s.pop();)if(r=e._.children)for(e.children=new Array(o=r.length),i=o-1;i>=0;--i)s.push(n=e.children[i]=new ka(r[i],i)),n.parent=e;return(a.parent=new ka(null,0)).children=[a],a}(i);if(A.eachAfter(o),A.parent.m=-A.z,A.eachBefore(a),r)i.eachBefore(s);else{var l=i,c=i,u=i;i.eachBefore(function(t){t.x<l.x&&(l=t),t.x>c.x&&(c=t),t.depth>u.depth&&(u=t)});var g=l===c?1:t(l,c)/2,h=g-l.x,d=e/(c.x+g+h),f=n/(u.depth||1);i.eachBefore(function(t){t.x=(t.x+h)*d,t.y=t.depth*f})}return i}function o(e){var n=e.children,r=e.parent.children,i=e.i?r[e.i-1]:null;if(n){!function(t){for(var e,n=0,r=0,i=t.children,o=i.length;--o>=0;)(e=i[o]).z+=n,e.m+=n,n+=e.s+(r+=e.c)}(e);var o=(n[0].z+n[n.length-1].z)/2;i?(e.z=i.z+t(e._,i._),e.m=e.z-o):e.z=o}else i&&(e.z=i.z+t(e._,i._));e.parent.A=function(e,n,r){if(n){for(var i,o=e,a=e,s=n,A=o.parent.children[0],l=o.m,c=a.m,u=s.m,g=A.m;s=_a(s),o=xa(o),s&&o;)A=xa(A),(a=_a(a)).a=e,(i=s.z+u-o.z-l+t(s._,o._))>0&&(Da(Ma(s,e,r),e,i),l+=i,c+=i),u+=s.m,l+=o.m,g+=A.m,c+=a.m;s&&!_a(a)&&(a.t=s,a.m+=u-c),o&&!xa(A)&&(A.t=o,A.m+=l-g,r=e)}return r}(e,i,e.parent.A||r[0])}function a(t){t._.x=t.z+t.parent.m,t.m+=t.parent.m}function s(t){t.x*=e,t.y=t.depth*n}return i.separation=function(e){return arguments.length?(t=e,i):t},i.size=function(t){return arguments.length?(r=!1,e=+t[0],n=+t[1],i):r?null:[e,n]},i.nodeSize=function(t){return arguments.length?(r=!0,e=+t[0],n=+t[1],i):r?[e,n]:null},i},La=function(t,e,n,r,i){for(var o,a=t.children,s=-1,A=a.length,l=t.value&&(i-n)/t.value;++s<A;)(o=a[s]).x0=e,o.x1=r,o.y0=n,o.y1=n+=o.value*l},Ra=(1+Math.sqrt(5))/2;function Na(t,e,n,r,i,o){for(var a,s,A,l,c,u,g,h,d,f,p,I=[],C=e.children,m=0,y=0,E=C.length,v=e.value;m<E;){A=i-n,l=o-r;do{c=C[y++].value}while(!c&&y<E);for(u=g=c,p=c*c*(f=Math.max(l/A,A/l)/(v*t)),d=Math.max(g/p,p/u);y<E;++y){if(c+=s=C[y].value,s<u&&(u=s),s>g&&(g=s),p=c*c*f,(h=Math.max(g/p,p/u))>d){c-=s;break}d=h}I.push(a={value:c,dice:A<l,children:C.slice(m,y)}),a.dice?Ca(a,n,r,i,v?r+=l*c/v:o):La(a,n,r,v?n+=A*c/v:i,o),v-=c,m=y}return I}var Fa=function t(e){function n(t,n,r,i,o){Na(e,t,n,r,i,o)}return n.ratio=function(e){return t((e=+e)>1?e:1)},n}(Ra),Ta=function(){var t=Fa,e=!1,n=1,r=1,i=[0],o=ca,a=ca,s=ca,A=ca,l=ca;function c(t){return t.x0=t.y0=0,t.x1=n,t.y1=r,t.eachBefore(u),i=[0],e&&t.eachBefore(Ia),t}function u(e){var n=i[e.depth],r=e.x0+n,c=e.y0+n,u=e.x1-n,g=e.y1-n;u<r&&(r=u=(r+u)/2),g<c&&(c=g=(c+g)/2),e.x0=r,e.y0=c,e.x1=u,e.y1=g,e.children&&(n=i[e.depth+1]=o(e)/2,r+=l(e)-n,c+=a(e)-n,(u-=s(e)-n)<r&&(r=u=(r+u)/2),(g-=A(e)-n)<c&&(c=g=(c+g)/2),t(e,r,c,u,g))}return c.round=function(t){return arguments.length?(e=!!t,c):e},c.size=function(t){return arguments.length?(n=+t[0],r=+t[1],c):[n,r]},c.tile=function(e){return arguments.length?(t=la(e),c):t},c.padding=function(t){return arguments.length?c.paddingInner(t).paddingOuter(t):c.paddingInner()},c.paddingInner=function(t){return arguments.length?(o="function"==typeof t?t:ua(+t),c):o},c.paddingOuter=function(t){return arguments.length?c.paddingTop(t).paddingRight(t).paddingBottom(t).paddingLeft(t):c.paddingTop()},c.paddingTop=function(t){return arguments.length?(a="function"==typeof t?t:ua(+t),c):a},c.paddingRight=function(t){return arguments.length?(s="function"==typeof t?t:ua(+t),c):s},c.paddingBottom=function(t){return arguments.length?(A="function"==typeof t?t:ua(+t),c):A},c.paddingLeft=function(t){return arguments.length?(l="function"==typeof t?t:ua(+t),c):l},c},Ua=function(t,e,n,r,i){var o,a,s=t.children,A=s.length,l=new Array(A+1);for(l[0]=a=o=0;o<A;++o)l[o+1]=a+=s[o].value;!function t(e,n,r,i,o,a,A){if(e>=n-1){var c=s[e];return c.x0=i,c.y0=o,c.x1=a,void(c.y1=A)}for(var u=l[e],g=r/2+u,h=e+1,d=n-1;h<d;){var f=h+d>>>1;l[f]<g?h=f+1:d=f}g-l[h-1]<l[h]-g&&e+1<h&&--h;var p=l[h]-u,I=r-p;if(a-i>A-o){var C=(i*I+a*p)/r;t(e,h,p,i,o,C,A),t(h,n,I,C,o,a,A)}else{var m=(o*I+A*p)/r;t(e,h,p,i,o,a,m),t(h,n,I,i,m,a,A)}}(0,A,t.value,e,n,r,i)},Oa=function(t,e,n,r,i){(1&t.depth?La:Ca)(t,e,n,r,i)},Ga=function t(e){function n(t,n,r,i,o){if((a=t._squarify)&&a.ratio===e)for(var a,s,A,l,c,u=-1,g=a.length,h=t.value;++u<g;){for(A=(s=a[u]).children,l=s.value=0,c=A.length;l<c;++l)s.value+=A[l].value;s.dice?Ca(s,n,r,i,r+=(o-r)*s.value/h):La(s,n,r,n+=(i-n)*s.value/h,o),h-=s.value}else t._squarify=a=Na(e,t,n,r,i,o),a.ratio=e}return n.ratio=function(e){return t((e=+e)>1?e:1)},n}(Ra),Ha=n(4),ja=function(t){for(var e,n=-1,r=t.length,i=t[r-1],o=0;++n<r;)e=i,i=t[n],o+=e[1]*i[0]-e[0]*i[1];return o/2},Ya=function(t){for(var e,n,r=-1,i=t.length,o=0,a=0,s=t[i-1],A=0;++r<i;)e=s,s=t[r],A+=n=e[0]*s[1]-s[0]*e[1],o+=(e[0]+s[0])*n,a+=(e[1]+s[1])*n;return[o/(A*=3),a/A]},Pa=function(t,e,n){return(e[0]-t[0])*(n[1]-t[1])-(e[1]-t[1])*(n[0]-t[0])};function Ka(t,e){return t[0]-e[0]||t[1]-e[1]}function za(t){for(var e=t.length,n=[0,1],r=2,i=2;i<e;++i){for(;r>1&&Pa(t[n[r-2]],t[n[r-1]],t[i])<=0;)--r;n[r++]=i}return n.slice(0,r)}var Ja=function(t){if((n=t.length)<3)return null;var e,n,r=new Array(n),i=new Array(n);for(e=0;e<n;++e)r[e]=[+t[e][0],+t[e][1],e];for(r.sort(Ka),e=0;e<n;++e)i[e]=[r[e][0],-r[e][1]];var o=za(r),a=za(i),s=a[0]===o[0],A=a[a.length-1]===o[o.length-1],l=[];for(e=o.length-1;e>=0;--e)l.push(t[r[o[e]][2]]);for(e=+s;e<a.length-A;++e)l.push(t[r[a[e]][2]]);return l},qa=function(t,e){for(var n,r,i=t.length,o=t[i-1],a=e[0],s=e[1],A=o[0],l=o[1],c=!1,u=0;u<i;++u)n=(o=t[u])[0],(r=o[1])>s!=l>s&&a<(A-n)*(s-r)/(l-r)+n&&(c=!c),A=n,l=r;return c},Va=function(t){for(var e,n,r=-1,i=t.length,o=t[i-1],a=o[0],s=o[1],A=0;++r<i;)e=a,n=s,e-=a=(o=t[r])[0],n-=s=o[1],A+=Math.sqrt(e*e+n*n);return A},Wa=n(85);function Xa(t,e){switch(arguments.length){case 0:break;case 1:this.range(t);break;default:this.range(e).domain(t)}return this}function Za(t,e){switch(arguments.length){case 0:break;case 1:this.interpolator(t);break;default:this.interpolator(e).domain(t)}return this}var $a=Array.prototype,ts=$a.map,es=$a.slice,ns={name:"implicit"};function rs(){var t=Object(T.c)(),e=[],n=[],r=ns;function i(i){var o=i+"",a=t.get(o);if(!a){if(r!==ns)return r;t.set(o,a=e.push(i))}return n[(a-1)%n.length]}return i.domain=function(n){if(!arguments.length)return e.slice();e=[],t=Object(T.c)();for(var r,o,a=-1,s=n.length;++a<s;)t.has(o=(r=n[a])+"")||t.set(o,e.push(r));return i},i.range=function(t){return arguments.length?(n=es.call(t),i):n.slice()},i.unknown=function(t){return arguments.length?(r=t,i):r},i.copy=function(){return rs(e,n).unknown(r)},Xa.apply(i,arguments),i}function is(){var t,e,n=rs().unknown(void 0),i=n.domain,o=n.range,a=[0,1],s=!1,A=0,l=0,c=.5;function u(){var n=i().length,u=a[1]<a[0],g=a[u-0],h=a[1-u];t=(h-g)/Math.max(1,n-A+2*l),s&&(t=Math.floor(t)),g+=(h-g-t*(n-A))*c,e=t*(1-A),s&&(g=Math.round(g),e=Math.round(e));var d=Object(r.range)(n).map(function(e){return g+t*e});return o(u?d.reverse():d)}return delete n.unknown,n.domain=function(t){return arguments.length?(i(t),u()):i()},n.range=function(t){return arguments.length?(a=[+t[0],+t[1]],u()):a.slice()},n.rangeRound=function(t){return a=[+t[0],+t[1]],s=!0,u()},n.bandwidth=function(){return e},n.step=function(){return t},n.round=function(t){return arguments.length?(s=!!t,u()):s},n.padding=function(t){return arguments.length?(A=Math.min(1,l=+t),u()):A},n.paddingInner=function(t){return arguments.length?(A=Math.min(1,t),u()):A},n.paddingOuter=function(t){return arguments.length?(l=+t,u()):l},n.align=function(t){return arguments.length?(c=Math.max(0,Math.min(1,t)),u()):c},n.copy=function(){return is(i(),a).round(s).paddingInner(A).paddingOuter(l).align(c)},Xa.apply(u(),arguments)}function os(){return function t(e){var n=e.copy;return e.padding=e.paddingOuter,delete e.paddingInner,delete e.paddingOuter,e.copy=function(){return t(n())},e}(is.apply(null,arguments).paddingInner(1))}var as=function(t){return function(){return t}},ss=function(t){return+t},As=[0,1];function ls(t){return t}function cs(t,e){return(e-=t=+t)?function(n){return(n-t)/e}:as(isNaN(e)?NaN:.5)}function us(t){var e,n=t[0],r=t[t.length-1];return n>r&&(e=n,n=r,r=e),function(t){return Math.max(n,Math.min(r,t))}}function gs(t,e,n){var r=t[0],i=t[1],o=e[0],a=e[1];return i<r?(r=cs(i,r),o=n(a,o)):(r=cs(r,i),o=n(o,a)),function(t){return o(r(t))}}function hs(t,e,n){var i=Math.min(t.length,e.length)-1,o=new Array(i),a=new Array(i),s=-1;for(t[i]<t[0]&&(t=t.slice().reverse(),e=e.slice().reverse());++s<i;)o[s]=cs(t[s],t[s+1]),a[s]=n(e[s],e[s+1]);return function(e){var n=Object(r.bisect)(t,e,1,i)-1;return a[n](o[n](e))}}function ds(t,e){return e.domain(t.domain()).range(t.range()).interpolate(t.interpolate()).clamp(t.clamp()).unknown(t.unknown())}function fs(){var t,e,n,r,i,o,a=As,s=As,A=Ha.a,l=ls;function c(){return r=Math.min(a.length,s.length)>2?hs:gs,i=o=null,u}function u(e){return isNaN(e=+e)?n:(i||(i=r(a.map(t),s,A)))(t(l(e)))}return u.invert=function(n){return l(e((o||(o=r(s,a.map(t),Ha.o)))(n)))},u.domain=function(t){return arguments.length?(a=ts.call(t,ss),l===ls||(l=us(a)),c()):a.slice()},u.range=function(t){return arguments.length?(s=es.call(t),c()):s.slice()},u.rangeRound=function(t){return s=es.call(t),A=Ha.t,c()},u.clamp=function(t){return arguments.length?(l=t?us(a):ls,u):l!==ls},u.interpolate=function(t){return arguments.length?(A=t,c()):A},u.unknown=function(t){return arguments.length?(n=t,u):n},function(n,r){return t=n,e=r,c()}}function ps(t,e){return fs()(t,e)}var Is=function(t,e,n,i){var o,a=Object(r.tickStep)(t,e,n);switch((i=Object(Lt.e)(null==i?",f":i)).type){case"s":var s=Math.max(Math.abs(t),Math.abs(e));return null!=i.precision||isNaN(o=Object(Lt.g)(a,s))||(i.precision=o),Object(Lt.d)(i,s);case"":case"e":case"g":case"p":case"r":null!=i.precision||isNaN(o=Object(Lt.h)(a,Math.max(Math.abs(t),Math.abs(e))))||(i.precision=o-("e"===i.type));break;case"f":case"%":null!=i.precision||isNaN(o=Object(Lt.f)(a))||(i.precision=o-2*("%"===i.type))}return Object(Lt.a)(i)};function Cs(t){var e=t.domain;return t.ticks=function(t){var n=e();return Object(r.ticks)(n[0],n[n.length-1],null==t?10:t)},t.tickFormat=function(t,n){var r=e();return Is(r[0],r[r.length-1],null==t?10:t,n)},t.nice=function(n){null==n&&(n=10);var i,o=e(),a=0,s=o.length-1,A=o[a],l=o[s];return l<A&&(i=A,A=l,l=i,i=a,a=s,s=i),(i=Object(r.tickIncrement)(A,l,n))>0?(A=Math.floor(A/i)*i,l=Math.ceil(l/i)*i,i=Object(r.tickIncrement)(A,l,n)):i<0&&(A=Math.ceil(A*i)/i,l=Math.floor(l*i)/i,i=Object(r.tickIncrement)(A,l,n)),i>0?(o[a]=Math.floor(A/i)*i,o[s]=Math.ceil(l/i)*i,e(o)):i<0&&(o[a]=Math.ceil(A*i)/i,o[s]=Math.floor(l*i)/i,e(o)),t},t}function ms(){var t=ps(ls,ls);return t.copy=function(){return ds(t,ms())},Xa.apply(t,arguments),Cs(t)}function ys(t){var e;function n(t){return isNaN(t=+t)?e:t}return n.invert=n,n.domain=n.range=function(e){return arguments.length?(t=ts.call(e,ss),n):t.slice()},n.unknown=function(t){return arguments.length?(e=t,n):e},n.copy=function(){return ys(t).unknown(e)},t=arguments.length?ts.call(t,ss):[0,1],Cs(n)}var Es=function(t,e){var n,r=0,i=(t=t.slice()).length-1,o=t[r],a=t[i];return a<o&&(n=r,r=i,i=n,n=o,o=a,a=n),t[r]=e.floor(o),t[i]=e.ceil(a),t};function vs(t){return Math.log(t)}function Bs(t){return Math.exp(t)}function bs(t){return-Math.log(-t)}function Qs(t){return-Math.exp(-t)}function ws(t){return isFinite(t)?+("1e"+t):t<0?0:t}function xs(t){return function(e){return-t(-e)}}function _s(t){var e,n,i=t(vs,Bs),o=i.domain,a=10;function s(){return e=function(t){return t===Math.E?Math.log:10===t&&Math.log10||2===t&&Math.log2||(t=Math.log(t),function(e){return Math.log(e)/t})}(a),n=function(t){return 10===t?ws:t===Math.E?Math.exp:function(e){return Math.pow(t,e)}}(a),o()[0]<0?(e=xs(e),n=xs(n),t(bs,Qs)):t(vs,Bs),i}return i.base=function(t){return arguments.length?(a=+t,s()):a},i.domain=function(t){return arguments.length?(o(t),s()):o()},i.ticks=function(t){var i,s=o(),A=s[0],l=s[s.length-1];(i=l<A)&&(h=A,A=l,l=h);var c,u,g,h=e(A),d=e(l),f=null==t?10:+t,p=[];if(!(a%1)&&d-h<f){if(h=Math.round(h)-1,d=Math.round(d)+1,A>0){for(;h<d;++h)for(u=1,c=n(h);u<a;++u)if(!((g=c*u)<A)){if(g>l)break;p.push(g)}}else for(;h<d;++h)for(u=a-1,c=n(h);u>=1;--u)if(!((g=c*u)<A)){if(g>l)break;p.push(g)}}else p=Object(r.ticks)(h,d,Math.min(d-h,f)).map(n);return i?p.reverse():p},i.tickFormat=function(t,r){if(null==r&&(r=10===a?".0e":","),"function"!=typeof r&&(r=Object(Lt.a)(r)),t===1/0)return r;null==t&&(t=10);var o=Math.max(1,a*t/i.ticks().length);return function(t){var i=t/n(Math.round(e(t)));return i*a<a-.5&&(i*=a),i<=o?r(t):""}},i.nice=function(){return o(Es(o(),{floor:function(t){return n(Math.floor(e(t)))},ceil:function(t){return n(Math.ceil(e(t)))}}))},i}function Ds(){var t=_s(fs()).domain([1,10]);return t.copy=function(){return ds(t,Ds()).base(t.base())},Xa.apply(t,arguments),t}function Ms(t){return function(e){return Math.sign(e)*Math.log1p(Math.abs(e/t))}}function ks(t){return function(e){return Math.sign(e)*Math.expm1(Math.abs(e))*t}}function Ss(t){var e=1,n=t(Ms(e),ks(e));return n.constant=function(n){return arguments.length?t(Ms(e=+n),ks(e)):e},Cs(n)}function Ls(){var t=Ss(fs());return t.copy=function(){return ds(t,Ls()).constant(t.constant())},Xa.apply(t,arguments)}function Rs(t){return function(e){return e<0?-Math.pow(-e,t):Math.pow(e,t)}}function Ns(t){return t<0?-Math.sqrt(-t):Math.sqrt(t)}function Fs(t){return t<0?-t*t:t*t}function Ts(t){var e=t(ls,ls),n=1;function r(){return 1===n?t(ls,ls):.5===n?t(Ns,Fs):t(Rs(n),Rs(1/n))}return e.exponent=function(t){return arguments.length?(n=+t,r()):n},Cs(e)}function Us(){var t=Ts(fs());return t.copy=function(){return ds(t,Us()).exponent(t.exponent())},Xa.apply(t,arguments),t}function Os(){return Us.apply(null,arguments).exponent(.5)}function Gs(){var t,e=[],n=[],i=[];function o(){var t=0,o=Math.max(1,n.length);for(i=new Array(o-1);++t<o;)i[t-1]=Object(r.quantile)(e,t/o);return a}function a(e){return isNaN(e=+e)?t:n[Object(r.bisect)(i,e)]}return a.invertExtent=function(t){var r=n.indexOf(t);return r<0?[NaN,NaN]:[r>0?i[r-1]:e[0],r<i.length?i[r]:e[e.length-1]]},a.domain=function(t){if(!arguments.length)return e.slice();e=[];for(var n,i=0,a=t.length;i<a;++i)null==(n=t[i])||isNaN(n=+n)||e.push(n);return e.sort(r.ascending),o()},a.range=function(t){return arguments.length?(n=es.call(t),o()):n.slice()},a.unknown=function(e){return arguments.length?(t=e,a):t},a.quantiles=function(){return i.slice()},a.copy=function(){return Gs().domain(e).range(n).unknown(t)},Xa.apply(a,arguments)}function Hs(){var t,e=0,n=1,i=1,o=[.5],a=[0,1];function s(e){return e<=e?a[Object(r.bisect)(o,e,0,i)]:t}function A(){var t=-1;for(o=new Array(i);++t<i;)o[t]=((t+1)*n-(t-i)*e)/(i+1);return s}return s.domain=function(t){return arguments.length?(e=+t[0],n=+t[1],A()):[e,n]},s.range=function(t){return arguments.length?(i=(a=es.call(t)).length-1,A()):a.slice()},s.invertExtent=function(t){var r=a.indexOf(t);return r<0?[NaN,NaN]:r<1?[e,o[0]]:r>=i?[o[i-1],n]:[o[r-1],o[r]]},s.unknown=function(e){return arguments.length?(t=e,s):s},s.thresholds=function(){return o.slice()},s.copy=function(){return Hs().domain([e,n]).range(a).unknown(t)},Xa.apply(Cs(s),arguments)}function js(){var t,e=[.5],n=[0,1],i=1;function o(o){return o<=o?n[Object(r.bisect)(e,o,0,i)]:t}return o.domain=function(t){return arguments.length?(e=es.call(t),i=Math.min(e.length,n.length-1),o):e.slice()},o.range=function(t){return arguments.length?(n=es.call(t),i=Math.min(e.length,n.length-1),o):n.slice()},o.invertExtent=function(t){var r=n.indexOf(t);return[e[r-1],e[r]]},o.unknown=function(e){return arguments.length?(t=e,o):t},o.copy=function(){return js().domain(e).range(n).unknown(t)},Xa.apply(o,arguments)}var Ys=n(1),Ps=n(10),Ks=1e3,zs=60*Ks,Js=60*zs,qs=24*Js,Vs=7*qs,Ws=30*qs,Xs=365*qs;function Zs(t){return new Date(t)}function $s(t){return t instanceof Date?+t:+new Date(+t)}function tA(t,e,n,i,o,a,s,A,l){var c=ps(ls,ls),u=c.invert,g=c.domain,h=l(".%L"),d=l(":%S"),f=l("%I:%M"),p=l("%I %p"),I=l("%a %d"),C=l("%b %d"),m=l("%B"),y=l("%Y"),E=[[s,1,Ks],[s,5,5*Ks],[s,15,15*Ks],[s,30,30*Ks],[a,1,zs],[a,5,5*zs],[a,15,15*zs],[a,30,30*zs],[o,1,Js],[o,3,3*Js],[o,6,6*Js],[o,12,12*Js],[i,1,qs],[i,2,2*qs],[n,1,Vs],[e,1,Ws],[e,3,3*Ws],[t,1,Xs]];function v(r){return(s(r)<r?h:a(r)<r?d:o(r)<r?f:i(r)<r?p:e(r)<r?n(r)<r?I:C:t(r)<r?m:y)(r)}function B(e,n,i,o){if(null==e&&(e=10),"number"==typeof e){var a=Math.abs(i-n)/e,s=Object(r.bisector)(function(t){return t[2]}).right(E,a);s===E.length?(o=Object(r.tickStep)(n/Xs,i/Xs,e),e=t):s?(o=(s=E[a/E[s-1][2]<E[s][2]/a?s-1:s])[1],e=s[0]):(o=Math.max(Object(r.tickStep)(n,i,e),1),e=A)}return null==o?e:e.every(o)}return c.invert=function(t){return new Date(u(t))},c.domain=function(t){return arguments.length?g(ts.call(t,$s)):g().map(Zs)},c.ticks=function(t,e){var n,r=g(),i=r[0],o=r[r.length-1],a=o<i;return a&&(n=i,i=o,o=n),n=(n=B(t,i,o,e))?n.range(i,o+1):[],a?n.reverse():n},c.tickFormat=function(t,e){return null==e?v:l(e)},c.nice=function(t,e){var n=g();return(t=B(t,n[0],n[n.length-1],e))?g(Es(n,t)):c},c.copy=function(){return ds(c,tA(t,e,n,i,o,a,s,A,l))},c}var eA=function(){return Xa.apply(tA(Ys.timeYear,Ys.timeMonth,Ys.timeWeek,Ys.timeDay,Ys.timeHour,Ys.timeMinute,Ys.timeSecond,Ys.timeMillisecond,Ps.c).domain([new Date(2e3,0,1),new Date(2e3,0,2)]),arguments)},nA=function(){return Xa.apply(tA(Ys.utcYear,Ys.utcMonth,Ys.utcWeek,Ys.utcDay,Ys.utcHour,Ys.utcMinute,Ys.utcSecond,Ys.utcMillisecond,Ps.g).domain([Date.UTC(2e3,0,1),Date.UTC(2e3,0,2)]),arguments)};function rA(){var t,e,n,r,i,o=0,a=1,s=ls,A=!1;function l(e){return isNaN(e=+e)?i:s(0===n?.5:(e=(r(e)-t)*n,A?Math.max(0,Math.min(1,e)):e))}return l.domain=function(i){return arguments.length?(t=r(o=+i[0]),e=r(a=+i[1]),n=t===e?0:1/(e-t),l):[o,a]},l.clamp=function(t){return arguments.length?(A=!!t,l):A},l.interpolator=function(t){return arguments.length?(s=t,l):s},l.unknown=function(t){return arguments.length?(i=t,l):i},function(i){return r=i,t=i(o),e=i(a),n=t===e?0:1/(e-t),l}}function iA(t,e){return e.domain(t.domain()).interpolator(t.interpolator()).clamp(t.clamp()).unknown(t.unknown())}function oA(){var t=Cs(rA()(ls));return t.copy=function(){return iA(t,oA())},Za.apply(t,arguments)}function aA(){var t=_s(rA()).domain([1,10]);return t.copy=function(){return iA(t,aA()).base(t.base())},Za.apply(t,arguments)}function sA(){var t=Ss(rA());return t.copy=function(){return iA(t,sA()).constant(t.constant())},Za.apply(t,arguments)}function AA(){var t=Ts(rA());return t.copy=function(){return iA(t,AA()).exponent(t.exponent())},Za.apply(t,arguments)}function lA(){return AA.apply(null,arguments).exponent(.5)}function cA(){var t=[],e=ls;function n(n){if(!isNaN(n=+n))return e((Object(r.bisect)(t,n)-1)/(t.length-1))}return n.domain=function(e){if(!arguments.length)return t.slice();t=[];for(var i,o=0,a=e.length;o<a;++o)null==(i=e[o])||isNaN(i=+i)||t.push(i);return t.sort(r.ascending),n},n.interpolator=function(t){return arguments.length?(e=t,n):e},n.copy=function(){return cA(e).domain(t)},Za.apply(n,arguments)}function uA(){var t,e,n,r,i,o,a,s=0,A=.5,l=1,c=ls,u=!1;function g(t){return isNaN(t=+t)?a:(t=.5+((t=+o(t))-e)*(t<e?r:i),c(u?Math.max(0,Math.min(1,t)):t))}return g.domain=function(a){return arguments.length?(t=o(s=+a[0]),e=o(A=+a[1]),n=o(l=+a[2]),r=t===e?0:.5/(e-t),i=e===n?0:.5/(n-e),g):[s,A,l]},g.clamp=function(t){return arguments.length?(u=!!t,g):u},g.interpolator=function(t){return arguments.length?(c=t,g):c},g.unknown=function(t){return arguments.length?(a=t,g):a},function(a){return o=a,t=a(s),e=a(A),n=a(l),r=t===e?0:.5/(e-t),i=e===n?0:.5/(n-e),g}}function gA(){var t=Cs(uA()(ls));return t.copy=function(){return iA(t,gA())},Za.apply(t,arguments)}function hA(){var t=_s(uA()).domain([.1,1,10]);return t.copy=function(){return iA(t,hA()).base(t.base())},Za.apply(t,arguments)}function dA(){var t=Ss(uA());return t.copy=function(){return iA(t,dA()).constant(t.constant())},Za.apply(t,arguments)}function fA(){var t=Ts(uA());return t.copy=function(){return iA(t,fA()).exponent(t.exponent())},Za.apply(t,arguments)}function pA(){return fA.apply(null,arguments).exponent(.5)}var IA=function(t){for(var e=t.length/6|0,n=new Array(e),r=0;r<e;)n[r]="#"+t.slice(6*r,6*++r);return n},CA=IA("1f77b4ff7f0e2ca02cd627289467bd8c564be377c27f7f7fbcbd2217becf"),mA=IA("7fc97fbeaed4fdc086ffff99386cb0f0027fbf5b17666666"),yA=IA("1b9e77d95f027570b3e7298a66a61ee6ab02a6761d666666"),EA=IA("a6cee31f78b4b2df8a33a02cfb9a99e31a1cfdbf6fff7f00cab2d66a3d9affff99b15928"),vA=IA("fbb4aeb3cde3ccebc5decbe4fed9a6ffffcce5d8bdfddaecf2f2f2"),BA=IA("b3e2cdfdcdaccbd5e8f4cae4e6f5c9fff2aef1e2cccccccc"),bA=IA("e41a1c377eb84daf4a984ea3ff7f00ffff33a65628f781bf999999"),QA=IA("66c2a5fc8d628da0cbe78ac3a6d854ffd92fe5c494b3b3b3"),wA=IA("8dd3c7ffffb3bebadafb807280b1d3fdb462b3de69fccde5d9d9d9bc80bdccebc5ffed6f"),xA=function(t){return Object(Ha.r)(t[t.length-1])},_A=new Array(3).concat("d8b365f5f5f55ab4ac","a6611adfc27d80cdc1018571","a6611adfc27df5f5f580cdc1018571","8c510ad8b365f6e8c3c7eae55ab4ac01665e","8c510ad8b365f6e8c3f5f5f5c7eae55ab4ac01665e","8c510abf812ddfc27df6e8c3c7eae580cdc135978f01665e","8c510abf812ddfc27df6e8c3f5f5f5c7eae580cdc135978f01665e","5430058c510abf812ddfc27df6e8c3c7eae580cdc135978f01665e003c30","5430058c510abf812ddfc27df6e8c3f5f5f5c7eae580cdc135978f01665e003c30").map(IA),DA=xA(_A),MA=new Array(3).concat("af8dc3f7f7f77fbf7b","7b3294c2a5cfa6dba0008837","7b3294c2a5cff7f7f7a6dba0008837","762a83af8dc3e7d4e8d9f0d37fbf7b1b7837","762a83af8dc3e7d4e8f7f7f7d9f0d37fbf7b1b7837","762a839970abc2a5cfe7d4e8d9f0d3a6dba05aae611b7837","762a839970abc2a5cfe7d4e8f7f7f7d9f0d3a6dba05aae611b7837","40004b762a839970abc2a5cfe7d4e8d9f0d3a6dba05aae611b783700441b","40004b762a839970abc2a5cfe7d4e8f7f7f7d9f0d3a6dba05aae611b783700441b").map(IA),kA=xA(MA),SA=new Array(3).concat("e9a3c9f7f7f7a1d76a","d01c8bf1b6dab8e1864dac26","d01c8bf1b6daf7f7f7b8e1864dac26","c51b7de9a3c9fde0efe6f5d0a1d76a4d9221","c51b7de9a3c9fde0eff7f7f7e6f5d0a1d76a4d9221","c51b7dde77aef1b6dafde0efe6f5d0b8e1867fbc414d9221","c51b7dde77aef1b6dafde0eff7f7f7e6f5d0b8e1867fbc414d9221","8e0152c51b7dde77aef1b6dafde0efe6f5d0b8e1867fbc414d9221276419","8e0152c51b7dde77aef1b6dafde0eff7f7f7e6f5d0b8e1867fbc414d9221276419").map(IA),LA=xA(SA),RA=new Array(3).concat("998ec3f7f7f7f1a340","5e3c99b2abd2fdb863e66101","5e3c99b2abd2f7f7f7fdb863e66101","542788998ec3d8daebfee0b6f1a340b35806","542788998ec3d8daebf7f7f7fee0b6f1a340b35806","5427888073acb2abd2d8daebfee0b6fdb863e08214b35806","5427888073acb2abd2d8daebf7f7f7fee0b6fdb863e08214b35806","2d004b5427888073acb2abd2d8daebfee0b6fdb863e08214b358067f3b08","2d004b5427888073acb2abd2d8daebf7f7f7fee0b6fdb863e08214b358067f3b08").map(IA),NA=xA(RA),FA=new Array(3).concat("ef8a62f7f7f767a9cf","ca0020f4a58292c5de0571b0","ca0020f4a582f7f7f792c5de0571b0","b2182bef8a62fddbc7d1e5f067a9cf2166ac","b2182bef8a62fddbc7f7f7f7d1e5f067a9cf2166ac","b2182bd6604df4a582fddbc7d1e5f092c5de4393c32166ac","b2182bd6604df4a582fddbc7f7f7f7d1e5f092c5de4393c32166ac","67001fb2182bd6604df4a582fddbc7d1e5f092c5de4393c32166ac053061","67001fb2182bd6604df4a582fddbc7f7f7f7d1e5f092c5de4393c32166ac053061").map(IA),TA=xA(FA),UA=new Array(3).concat("ef8a62ffffff999999","ca0020f4a582bababa404040","ca0020f4a582ffffffbababa404040","b2182bef8a62fddbc7e0e0e09999994d4d4d","b2182bef8a62fddbc7ffffffe0e0e09999994d4d4d","b2182bd6604df4a582fddbc7e0e0e0bababa8787874d4d4d","b2182bd6604df4a582fddbc7ffffffe0e0e0bababa8787874d4d4d","67001fb2182bd6604df4a582fddbc7e0e0e0bababa8787874d4d4d1a1a1a","67001fb2182bd6604df4a582fddbc7ffffffe0e0e0bababa8787874d4d4d1a1a1a").map(IA),OA=xA(UA),GA=new Array(3).concat("fc8d59ffffbf91bfdb","d7191cfdae61abd9e92c7bb6","d7191cfdae61ffffbfabd9e92c7bb6","d73027fc8d59fee090e0f3f891bfdb4575b4","d73027fc8d59fee090ffffbfe0f3f891bfdb4575b4","d73027f46d43fdae61fee090e0f3f8abd9e974add14575b4","d73027f46d43fdae61fee090ffffbfe0f3f8abd9e974add14575b4","a50026d73027f46d43fdae61fee090e0f3f8abd9e974add14575b4313695","a50026d73027f46d43fdae61fee090ffffbfe0f3f8abd9e974add14575b4313695").map(IA),HA=xA(GA),jA=new Array(3).concat("fc8d59ffffbf91cf60","d7191cfdae61a6d96a1a9641","d7191cfdae61ffffbfa6d96a1a9641","d73027fc8d59fee08bd9ef8b91cf601a9850","d73027fc8d59fee08bffffbfd9ef8b91cf601a9850","d73027f46d43fdae61fee08bd9ef8ba6d96a66bd631a9850","d73027f46d43fdae61fee08bffffbfd9ef8ba6d96a66bd631a9850","a50026d73027f46d43fdae61fee08bd9ef8ba6d96a66bd631a9850006837","a50026d73027f46d43fdae61fee08bffffbfd9ef8ba6d96a66bd631a9850006837").map(IA),YA=xA(jA),PA=new Array(3).concat("fc8d59ffffbf99d594","d7191cfdae61abdda42b83ba","d7191cfdae61ffffbfabdda42b83ba","d53e4ffc8d59fee08be6f59899d5943288bd","d53e4ffc8d59fee08bffffbfe6f59899d5943288bd","d53e4ff46d43fdae61fee08be6f598abdda466c2a53288bd","d53e4ff46d43fdae61fee08bffffbfe6f598abdda466c2a53288bd","9e0142d53e4ff46d43fdae61fee08be6f598abdda466c2a53288bd5e4fa2","9e0142d53e4ff46d43fdae61fee08bffffbfe6f598abdda466c2a53288bd5e4fa2").map(IA),KA=xA(PA),zA=new Array(3).concat("e5f5f999d8c92ca25f","edf8fbb2e2e266c2a4238b45","edf8fbb2e2e266c2a42ca25f006d2c","edf8fbccece699d8c966c2a42ca25f006d2c","edf8fbccece699d8c966c2a441ae76238b45005824","f7fcfde5f5f9ccece699d8c966c2a441ae76238b45005824","f7fcfde5f5f9ccece699d8c966c2a441ae76238b45006d2c00441b").map(IA),JA=xA(zA),qA=new Array(3).concat("e0ecf49ebcda8856a7","edf8fbb3cde38c96c688419d","edf8fbb3cde38c96c68856a7810f7c","edf8fbbfd3e69ebcda8c96c68856a7810f7c","edf8fbbfd3e69ebcda8c96c68c6bb188419d6e016b","f7fcfde0ecf4bfd3e69ebcda8c96c68c6bb188419d6e016b","f7fcfde0ecf4bfd3e69ebcda8c96c68c6bb188419d810f7c4d004b").map(IA),VA=xA(qA),WA=new Array(3).concat("e0f3dba8ddb543a2ca","f0f9e8bae4bc7bccc42b8cbe","f0f9e8bae4bc7bccc443a2ca0868ac","f0f9e8ccebc5a8ddb57bccc443a2ca0868ac","f0f9e8ccebc5a8ddb57bccc44eb3d32b8cbe08589e","f7fcf0e0f3dbccebc5a8ddb57bccc44eb3d32b8cbe08589e","f7fcf0e0f3dbccebc5a8ddb57bccc44eb3d32b8cbe0868ac084081").map(IA),XA=xA(WA),ZA=new Array(3).concat("fee8c8fdbb84e34a33","fef0d9fdcc8afc8d59d7301f","fef0d9fdcc8afc8d59e34a33b30000","fef0d9fdd49efdbb84fc8d59e34a33b30000","fef0d9fdd49efdbb84fc8d59ef6548d7301f990000","fff7ecfee8c8fdd49efdbb84fc8d59ef6548d7301f990000","fff7ecfee8c8fdd49efdbb84fc8d59ef6548d7301fb300007f0000").map(IA),$A=xA(ZA),tl=new Array(3).concat("ece2f0a6bddb1c9099","f6eff7bdc9e167a9cf02818a","f6eff7bdc9e167a9cf1c9099016c59","f6eff7d0d1e6a6bddb67a9cf1c9099016c59","f6eff7d0d1e6a6bddb67a9cf3690c002818a016450","fff7fbece2f0d0d1e6a6bddb67a9cf3690c002818a016450","fff7fbece2f0d0d1e6a6bddb67a9cf3690c002818a016c59014636").map(IA),el=xA(tl),nl=new Array(3).concat("ece7f2a6bddb2b8cbe","f1eef6bdc9e174a9cf0570b0","f1eef6bdc9e174a9cf2b8cbe045a8d","f1eef6d0d1e6a6bddb74a9cf2b8cbe045a8d","f1eef6d0d1e6a6bddb74a9cf3690c00570b0034e7b","fff7fbece7f2d0d1e6a6bddb74a9cf3690c00570b0034e7b","fff7fbece7f2d0d1e6a6bddb74a9cf3690c00570b0045a8d023858").map(IA),rl=xA(nl),il=new Array(3).concat("e7e1efc994c7dd1c77","f1eef6d7b5d8df65b0ce1256","f1eef6d7b5d8df65b0dd1c77980043","f1eef6d4b9dac994c7df65b0dd1c77980043","f1eef6d4b9dac994c7df65b0e7298ace125691003f","f7f4f9e7e1efd4b9dac994c7df65b0e7298ace125691003f","f7f4f9e7e1efd4b9dac994c7df65b0e7298ace125698004367001f").map(IA),ol=xA(il),al=new Array(3).concat("fde0ddfa9fb5c51b8a","feebe2fbb4b9f768a1ae017e","feebe2fbb4b9f768a1c51b8a7a0177","feebe2fcc5c0fa9fb5f768a1c51b8a7a0177","feebe2fcc5c0fa9fb5f768a1dd3497ae017e7a0177","fff7f3fde0ddfcc5c0fa9fb5f768a1dd3497ae017e7a0177","fff7f3fde0ddfcc5c0fa9fb5f768a1dd3497ae017e7a017749006a").map(IA),sl=xA(al),Al=new Array(3).concat("edf8b17fcdbb2c7fb8","ffffcca1dab441b6c4225ea8","ffffcca1dab441b6c42c7fb8253494","ffffccc7e9b47fcdbb41b6c42c7fb8253494","ffffccc7e9b47fcdbb41b6c41d91c0225ea80c2c84","ffffd9edf8b1c7e9b47fcdbb41b6c41d91c0225ea80c2c84","ffffd9edf8b1c7e9b47fcdbb41b6c41d91c0225ea8253494081d58").map(IA),ll=xA(Al),cl=new Array(3).concat("f7fcb9addd8e31a354","ffffccc2e69978c679238443","ffffccc2e69978c67931a354006837","ffffccd9f0a3addd8e78c67931a354006837","ffffccd9f0a3addd8e78c67941ab5d238443005a32","ffffe5f7fcb9d9f0a3addd8e78c67941ab5d238443005a32","ffffe5f7fcb9d9f0a3addd8e78c67941ab5d238443006837004529").map(IA),ul=xA(cl),gl=new Array(3).concat("fff7bcfec44fd95f0e","ffffd4fed98efe9929cc4c02","ffffd4fed98efe9929d95f0e993404","ffffd4fee391fec44ffe9929d95f0e993404","ffffd4fee391fec44ffe9929ec7014cc4c028c2d04","ffffe5fff7bcfee391fec44ffe9929ec7014cc4c028c2d04","ffffe5fff7bcfee391fec44ffe9929ec7014cc4c02993404662506").map(IA),hl=xA(gl),dl=new Array(3).concat("ffeda0feb24cf03b20","ffffb2fecc5cfd8d3ce31a1c","ffffb2fecc5cfd8d3cf03b20bd0026","ffffb2fed976feb24cfd8d3cf03b20bd0026","ffffb2fed976feb24cfd8d3cfc4e2ae31a1cb10026","ffffccffeda0fed976feb24cfd8d3cfc4e2ae31a1cb10026","ffffccffeda0fed976feb24cfd8d3cfc4e2ae31a1cbd0026800026").map(IA),fl=xA(dl),pl=new Array(3).concat("deebf79ecae13182bd","eff3ffbdd7e76baed62171b5","eff3ffbdd7e76baed63182bd08519c","eff3ffc6dbef9ecae16baed63182bd08519c","eff3ffc6dbef9ecae16baed64292c62171b5084594","f7fbffdeebf7c6dbef9ecae16baed64292c62171b5084594","f7fbffdeebf7c6dbef9ecae16baed64292c62171b508519c08306b").map(IA),Il=xA(pl),Cl=new Array(3).concat("e5f5e0a1d99b31a354","edf8e9bae4b374c476238b45","edf8e9bae4b374c47631a354006d2c","edf8e9c7e9c0a1d99b74c47631a354006d2c","edf8e9c7e9c0a1d99b74c47641ab5d238b45005a32","f7fcf5e5f5e0c7e9c0a1d99b74c47641ab5d238b45005a32","f7fcf5e5f5e0c7e9c0a1d99b74c47641ab5d238b45006d2c00441b").map(IA),ml=xA(Cl),yl=new Array(3).concat("f0f0f0bdbdbd636363","f7f7f7cccccc969696525252","f7f7f7cccccc969696636363252525","f7f7f7d9d9d9bdbdbd969696636363252525","f7f7f7d9d9d9bdbdbd969696737373525252252525","fffffff0f0f0d9d9d9bdbdbd969696737373525252252525","fffffff0f0f0d9d9d9bdbdbd969696737373525252252525000000").map(IA),El=xA(yl),vl=new Array(3).concat("efedf5bcbddc756bb1","f2f0f7cbc9e29e9ac86a51a3","f2f0f7cbc9e29e9ac8756bb154278f","f2f0f7dadaebbcbddc9e9ac8756bb154278f","f2f0f7dadaebbcbddc9e9ac8807dba6a51a34a1486","fcfbfdefedf5dadaebbcbddc9e9ac8807dba6a51a34a1486","fcfbfdefedf5dadaebbcbddc9e9ac8807dba6a51a354278f3f007d").map(IA),Bl=xA(vl),bl=new Array(3).concat("fee0d2fc9272de2d26","fee5d9fcae91fb6a4acb181d","fee5d9fcae91fb6a4ade2d26a50f15","fee5d9fcbba1fc9272fb6a4ade2d26a50f15","fee5d9fcbba1fc9272fb6a4aef3b2ccb181d99000d","fff5f0fee0d2fcbba1fc9272fb6a4aef3b2ccb181d99000d","fff5f0fee0d2fcbba1fc9272fb6a4aef3b2ccb181da50f1567000d").map(IA),Ql=xA(bl),wl=new Array(3).concat("fee6cefdae6be6550d","feeddefdbe85fd8d3cd94701","feeddefdbe85fd8d3ce6550da63603","feeddefdd0a2fdae6bfd8d3ce6550da63603","feeddefdd0a2fdae6bfd8d3cf16913d948018c2d04","fff5ebfee6cefdd0a2fdae6bfd8d3cf16913d948018c2d04","fff5ebfee6cefdd0a2fdae6bfd8d3cf16913d94801a636037f2704").map(IA),xl=xA(wl),_l=Object(Ha.f)(Object(U.b)(300,.5,0),Object(U.b)(-240,.5,1)),Dl=Object(Ha.f)(Object(U.b)(-100,.75,.35),Object(U.b)(80,1.5,.8)),Ml=Object(Ha.f)(Object(U.b)(260,.75,.35),Object(U.b)(80,1.5,.8)),kl=Object(U.b)(),Sl=function(t){(t<0||t>1)&&(t-=Math.floor(t));var e=Math.abs(t-.5);return kl.h=360*t-100,kl.s=1.5-1.5*e,kl.l=.8-.9*e,kl+""},Ll=Object(U.h)(),Rl=Math.PI/3,Nl=2*Math.PI/3,Fl=function(t){var e;return t=(.5-t)*Math.PI,Ll.r=255*(e=Math.sin(t))*e,Ll.g=255*(e=Math.sin(t+Rl))*e,Ll.b=255*(e=Math.sin(t+Nl))*e,Ll+""};function Tl(t){var e=t.length;return function(n){return t[Math.max(0,Math.min(e-1,Math.floor(n*e)))]}}var Ul=Tl(IA("44015444025645045745055946075a46085c460a5d460b5e470d60470e6147106347116447136548146748166848176948186a481a6c481b6d481c6e481d6f481f70482071482173482374482475482576482677482878482979472a7a472c7a472d7b472e7c472f7d46307e46327e46337f463480453581453781453882443983443a83443b84433d84433e85423f854240864241864142874144874045884046883f47883f48893e49893e4a893e4c8a3d4d8a3d4e8a3c4f8a3c508b3b518b3b528b3a538b3a548c39558c39568c38588c38598c375a8c375b8d365c8d365d8d355e8d355f8d34608d34618d33628d33638d32648e32658e31668e31678e31688e30698e306a8e2f6b8e2f6c8e2e6d8e2e6e8e2e6f8e2d708e2d718e2c718e2c728e2c738e2b748e2b758e2a768e2a778e2a788e29798e297a8e297b8e287c8e287d8e277e8e277f8e27808e26818e26828e26828e25838e25848e25858e24868e24878e23888e23898e238a8d228b8d228c8d228d8d218e8d218f8d21908d21918c20928c20928c20938c1f948c1f958b1f968b1f978b1f988b1f998a1f9a8a1e9b8a1e9c891e9d891f9e891f9f881fa0881fa1881fa1871fa28720a38620a48621a58521a68522a78522a88423a98324aa8325ab8225ac8226ad8127ad8128ae8029af7f2ab07f2cb17e2db27d2eb37c2fb47c31b57b32b67a34b67935b77937b87838b9773aba763bbb753dbc743fbc7340bd7242be7144bf7046c06f48c16e4ac16d4cc26c4ec36b50c46a52c56954c56856c66758c7655ac8645cc8635ec96260ca6063cb5f65cb5e67cc5c69cd5b6ccd5a6ece5870cf5773d05675d05477d1537ad1517cd2507fd34e81d34d84d44b86d54989d5488bd6468ed64590d74393d74195d84098d83e9bd93c9dd93ba0da39a2da37a5db36a8db34aadc32addc30b0dd2fb2dd2db5de2bb8de29bade28bddf26c0df25c2df23c5e021c8e020cae11fcde11dd0e11cd2e21bd5e21ad8e219dae319dde318dfe318e2e418e5e419e7e419eae51aece51befe51cf1e51df4e61ef6e620f8e621fbe723fde725")),Ol=Tl(IA("00000401000501010601010802010902020b02020d03030f03031204041405041606051806051a07061c08071e0907200a08220b09240c09260d0a290e0b2b100b2d110c2f120d31130d34140e36150e38160f3b180f3d19103f1a10421c10441d11471e114920114b21114e22115024125325125527125829115a2a115c2c115f2d11612f116331116533106734106936106b38106c390f6e3b0f703d0f713f0f72400f74420f75440f764510774710784910784a10794c117a4e117b4f127b51127c52137c54137d56147d57157e59157e5a167e5c167f5d177f5f187f601880621980641a80651a80671b80681c816a1c816b1d816d1d816e1e81701f81721f817320817521817621817822817922827b23827c23827e24828025828125818326818426818627818827818928818b29818c29818e2a81902a81912b81932b80942c80962c80982d80992d809b2e7f9c2e7f9e2f7fa02f7fa1307ea3307ea5317ea6317da8327daa337dab337cad347cae347bb0357bb2357bb3367ab5367ab73779b83779ba3878bc3978bd3977bf3a77c03a76c23b75c43c75c53c74c73d73c83e73ca3e72cc3f71cd4071cf4070d0416fd2426fd3436ed5446dd6456cd8456cd9466bdb476adc4869de4968df4a68e04c67e24d66e34e65e44f64e55064e75263e85362e95462ea5661eb5760ec5860ed5a5fee5b5eef5d5ef05f5ef1605df2625df2645cf3655cf4675cf4695cf56b5cf66c5cf66e5cf7705cf7725cf8745cf8765cf9785df9795df97b5dfa7d5efa7f5efa815ffb835ffb8560fb8761fc8961fc8a62fc8c63fc8e64fc9065fd9266fd9467fd9668fd9869fd9a6afd9b6bfe9d6cfe9f6dfea16efea36ffea571fea772fea973feaa74feac76feae77feb078feb27afeb47bfeb67cfeb77efeb97ffebb81febd82febf84fec185fec287fec488fec68afec88cfeca8dfecc8ffecd90fecf92fed194fed395fed597fed799fed89afdda9cfddc9efddea0fde0a1fde2a3fde3a5fde5a7fde7a9fde9aafdebacfcecaefceeb0fcf0b2fcf2b4fcf4b6fcf6b8fcf7b9fcf9bbfcfbbdfcfdbf")),Gl=Tl(IA("00000401000501010601010802010a02020c02020e03021004031204031405041706041907051b08051d09061f0a07220b07240c08260d08290e092b10092d110a30120a32140b34150b37160b39180c3c190c3e1b0c411c0c431e0c451f0c48210c4a230c4c240c4f260c51280b53290b552b0b572d0b592f0a5b310a5c320a5e340a5f3609613809623909633b09643d09653e0966400a67420a68440a68450a69470b6a490b6a4a0c6b4c0c6b4d0d6c4f0d6c510e6c520e6d540f6d550f6d57106e59106e5a116e5c126e5d126e5f136e61136e62146e64156e65156e67166e69166e6a176e6c186e6d186e6f196e71196e721a6e741a6e751b6e771c6d781c6d7a1d6d7c1d6d7d1e6d7f1e6c801f6c82206c84206b85216b87216b88226a8a226a8c23698d23698f24699025689225689326679526679727669827669a28659b29649d29649f2a63a02a63a22b62a32c61a52c60a62d60a82e5fa92e5eab2f5ead305dae305cb0315bb1325ab3325ab43359b63458b73557b93556ba3655bc3754bd3853bf3952c03a51c13a50c33b4fc43c4ec63d4dc73e4cc83f4bca404acb4149cc4248ce4347cf4446d04545d24644d34743d44842d54a41d74b3fd84c3ed94d3dda4e3cdb503bdd513ade5238df5337e05536e15635e25734e35933e45a31e55c30e65d2fe75e2ee8602de9612bea632aeb6429eb6628ec6726ed6925ee6a24ef6c23ef6e21f06f20f1711ff1731df2741cf3761bf37819f47918f57b17f57d15f67e14f68013f78212f78410f8850ff8870ef8890cf98b0bf98c0af98e09fa9008fa9207fa9407fb9606fb9706fb9906fb9b06fb9d07fc9f07fca108fca309fca50afca60cfca80dfcaa0ffcac11fcae12fcb014fcb216fcb418fbb61afbb81dfbba1ffbbc21fbbe23fac026fac228fac42afac62df9c72ff9c932f9cb35f8cd37f8cf3af7d13df7d340f6d543f6d746f5d949f5db4cf4dd4ff4df53f4e156f3e35af3e55df2e661f2e865f2ea69f1ec6df1ed71f1ef75f1f179f2f27df2f482f3f586f3f68af4f88ef5f992f6fa96f8fb9af9fc9dfafda1fcffa4")),Hl=Tl(IA("0d088710078813078916078a19068c1b068d1d068e20068f2206902406912605912805922a05932c05942e05952f059631059733059735049837049938049a3a049a3c049b3e049c3f049c41049d43039e44039e46039f48039f4903a04b03a14c02a14e02a25002a25102a35302a35502a45601a45801a45901a55b01a55c01a65e01a66001a66100a76300a76400a76600a76700a86900a86a00a86c00a86e00a86f00a87100a87201a87401a87501a87701a87801a87a02a87b02a87d03a87e03a88004a88104a78305a78405a78606a68707a68808a68a09a58b0aa58d0ba58e0ca48f0da4910ea3920fa39410a29511a19613a19814a099159f9a169f9c179e9d189d9e199da01a9ca11b9ba21d9aa31e9aa51f99a62098a72197a82296aa2395ab2494ac2694ad2793ae2892b02991b12a90b22b8fb32c8eb42e8db52f8cb6308bb7318ab83289ba3388bb3488bc3587bd3786be3885bf3984c03a83c13b82c23c81c33d80c43e7fc5407ec6417dc7427cc8437bc9447aca457acb4679cc4778cc4977cd4a76ce4b75cf4c74d04d73d14e72d24f71d35171d45270d5536fd5546ed6556dd7566cd8576bd9586ada5a6ada5b69db5c68dc5d67dd5e66de5f65de6164df6263e06363e16462e26561e26660e3685fe4695ee56a5de56b5de66c5ce76e5be76f5ae87059e97158e97257ea7457eb7556eb7655ec7754ed7953ed7a52ee7b51ef7c51ef7e50f07f4ff0804ef1814df1834cf2844bf3854bf3874af48849f48948f58b47f58c46f68d45f68f44f79044f79143f79342f89441f89540f9973ff9983ef99a3efa9b3dfa9c3cfa9e3bfb9f3afba139fba238fca338fca537fca636fca835fca934fdab33fdac33fdae32fdaf31fdb130fdb22ffdb42ffdb52efeb72dfeb82cfeba2cfebb2bfebd2afebe2afec029fdc229fdc328fdc527fdc627fdc827fdca26fdcb26fccd25fcce25fcd025fcd225fbd324fbd524fbd724fad824fada24f9dc24f9dd25f8df25f8e125f7e225f7e425f6e626f6e826f5e926f5eb27f4ed27f3ee27f3f027f2f227f1f426f1f525f0f724f0f921")),jl=n(0),Yl=n(84),Pl=n(20),Kl=function(t){return function(){return t}};function zl(t){return t[0]}function Jl(t){return t[1]}function ql(){this._=null}function Vl(t){t.U=t.C=t.L=t.R=t.P=t.N=null}function Wl(t,e){var n=e,r=e.R,i=n.U;i?i.L===n?i.L=r:i.R=r:t._=r,r.U=i,n.U=r,n.R=r.L,n.R&&(n.R.U=n),r.L=n}function Xl(t,e){var n=e,r=e.L,i=n.U;i?i.L===n?i.L=r:i.R=r:t._=r,r.U=i,n.U=r,n.L=r.R,n.L&&(n.L.U=n),r.R=n}function Zl(t){for(;t.L;)t=t.L;return t}ql.prototype={constructor:ql,insert:function(t,e){var n,r,i;if(t){if(e.P=t,e.N=t.N,t.N&&(t.N.P=e),t.N=e,t.R){for(t=t.R;t.L;)t=t.L;t.L=e}else t.R=e;n=t}else this._?(t=Zl(this._),e.P=null,e.N=t,t.P=t.L=e,n=t):(e.P=e.N=null,this._=e,n=null);for(e.L=e.R=null,e.U=n,e.C=!0,t=e;n&&n.C;)n===(r=n.U).L?(i=r.R)&&i.C?(n.C=i.C=!1,r.C=!0,t=r):(t===n.R&&(Wl(this,n),n=(t=n).U),n.C=!1,r.C=!0,Xl(this,r)):(i=r.L)&&i.C?(n.C=i.C=!1,r.C=!0,t=r):(t===n.L&&(Xl(this,n),n=(t=n).U),n.C=!1,r.C=!0,Wl(this,r)),n=t.U;this._.C=!1},remove:function(t){t.N&&(t.N.P=t.P),t.P&&(t.P.N=t.N),t.N=t.P=null;var e,n,r,i=t.U,o=t.L,a=t.R;if(n=o?a?Zl(a):o:a,i?i.L===t?i.L=n:i.R=n:this._=n,o&&a?(r=n.C,n.C=t.C,n.L=o,o.U=n,n!==a?(i=n.U,n.U=t.U,t=n.R,i.L=t,n.R=a,a.U=n):(n.U=i,i=n,t=n.R)):(r=t.C,t=n),t&&(t.U=i),!r)if(t&&t.C)t.C=!1;else{do{if(t===this._)break;if(t===i.L){if((e=i.R).C&&(e.C=!1,i.C=!0,Wl(this,i),e=i.R),e.L&&e.L.C||e.R&&e.R.C){e.R&&e.R.C||(e.L.C=!1,e.C=!0,Xl(this,e),e=i.R),e.C=i.C,i.C=e.R.C=!1,Wl(this,i),t=this._;break}}else if((e=i.L).C&&(e.C=!1,i.C=!0,Xl(this,i),e=i.L),e.L&&e.L.C||e.R&&e.R.C){e.L&&e.L.C||(e.R.C=!1,e.C=!0,Wl(this,e),e=i.L),e.C=i.C,i.C=e.L.C=!1,Xl(this,i),t=this._;break}e.C=!0,t=i,i=i.U}while(!t.C);t&&(t.C=!1)}}};var $l=ql;function tc(t,e,n,r){var i=[null,null],o=bc.push(i)-1;return i.left=t,i.right=e,n&&nc(i,t,e,n),r&&nc(i,e,t,r),vc[t.index].halfedges.push(o),vc[e.index].halfedges.push(o),i}function ec(t,e,n){var r=[e,n];return r.left=t,r}function nc(t,e,n,r){t[0]||t[1]?t.left===n?t[1]=r:t[0]=r:(t[0]=r,t.left=e,t.right=n)}function rc(t,e,n,r,i){var o,a=t[0],s=t[1],A=a[0],l=a[1],c=0,u=1,g=s[0]-A,h=s[1]-l;if(o=e-A,g||!(o>0)){if(o/=g,g<0){if(o<c)return;o<u&&(u=o)}else if(g>0){if(o>u)return;o>c&&(c=o)}if(o=r-A,g||!(o<0)){if(o/=g,g<0){if(o>u)return;o>c&&(c=o)}else if(g>0){if(o<c)return;o<u&&(u=o)}if(o=n-l,h||!(o>0)){if(o/=h,h<0){if(o<c)return;o<u&&(u=o)}else if(h>0){if(o>u)return;o>c&&(c=o)}if(o=i-l,h||!(o<0)){if(o/=h,h<0){if(o>u)return;o>c&&(c=o)}else if(h>0){if(o<c)return;o<u&&(u=o)}return!(c>0||u<1)||(c>0&&(t[0]=[A+c*g,l+c*h]),u<1&&(t[1]=[A+u*g,l+u*h]),!0)}}}}}function ic(t,e,n,r,i){var o=t[1];if(o)return!0;var a,s,A=t[0],l=t.left,c=t.right,u=l[0],g=l[1],h=c[0],d=c[1],f=(u+h)/2,p=(g+d)/2;if(d===g){if(f<e||f>=r)return;if(u>h){if(A){if(A[1]>=i)return}else A=[f,n];o=[f,i]}else{if(A){if(A[1]<n)return}else A=[f,i];o=[f,n]}}else if(s=p-(a=(u-h)/(d-g))*f,a<-1||a>1)if(u>h){if(A){if(A[1]>=i)return}else A=[(n-s)/a,n];o=[(i-s)/a,i]}else{if(A){if(A[1]<n)return}else A=[(i-s)/a,i];o=[(n-s)/a,n]}else if(g<d){if(A){if(A[0]>=r)return}else A=[e,a*e+s];o=[r,a*r+s]}else{if(A){if(A[0]<e)return}else A=[r,a*r+s];o=[e,a*e+s]}return t[0]=A,t[1]=o,!0}function oc(t,e){var n=t.site,r=e.left,i=e.right;return n===i&&(i=r,r=n),i?Math.atan2(i[1]-r[1],i[0]-r[0]):(n===r?(r=e[1],i=e[0]):(r=e[0],i=e[1]),Math.atan2(r[0]-i[0],i[1]-r[1]))}function ac(t,e){return e[+(e.left!==t.site)]}function sc(t,e){return e[+(e.left===t.site)]}var Ac,lc=[];function cc(){Vl(this),this.x=this.y=this.arc=this.site=this.cy=null}function uc(t){var e=t.P,n=t.N;if(e&&n){var r=e.site,i=t.site,o=n.site;if(r!==o){var a=i[0],s=i[1],A=r[0]-a,l=r[1]-s,c=o[0]-a,u=o[1]-s,g=2*(A*u-l*c);if(!(g>=-wc)){var h=A*A+l*l,d=c*c+u*u,f=(u*h-l*d)/g,p=(A*d-c*h)/g,I=lc.pop()||new cc;I.arc=t,I.site=i,I.x=f+a,I.y=(I.cy=p+s)+Math.sqrt(f*f+p*p),t.circle=I;for(var C=null,m=Bc._;m;)if(I.y<m.y||I.y===m.y&&I.x<=m.x){if(!m.L){C=m.P;break}m=m.L}else{if(!m.R){C=m;break}m=m.R}Bc.insert(C,I),C||(Ac=I)}}}}function gc(t){var e=t.circle;e&&(e.P||(Ac=e.N),Bc.remove(e),lc.push(e),Vl(e),t.circle=null)}var hc=[];function dc(){Vl(this),this.edge=this.site=this.circle=null}function fc(t){var e=hc.pop()||new dc;return e.site=t,e}function pc(t){gc(t),Ec.remove(t),hc.push(t),Vl(t)}function Ic(t){var e=t.circle,n=e.x,r=e.cy,i=[n,r],o=t.P,a=t.N,s=[t];pc(t);for(var A=o;A.circle&&Math.abs(n-A.circle.x)<Qc&&Math.abs(r-A.circle.cy)<Qc;)o=A.P,s.unshift(A),pc(A),A=o;s.unshift(A),gc(A);for(var l=a;l.circle&&Math.abs(n-l.circle.x)<Qc&&Math.abs(r-l.circle.cy)<Qc;)a=l.N,s.push(l),pc(l),l=a;s.push(l),gc(l);var c,u=s.length;for(c=1;c<u;++c)l=s[c],A=s[c-1],nc(l.edge,A.site,l.site,i);A=s[0],(l=s[u-1]).edge=tc(A.site,l.site,null,i),uc(A),uc(l)}function Cc(t){for(var e,n,r,i,o=t[0],a=t[1],s=Ec._;s;)if((r=mc(s,a)-o)>Qc)s=s.L;else{if(!((i=o-yc(s,a))>Qc)){r>-Qc?(e=s.P,n=s):i>-Qc?(e=s,n=s.N):e=n=s;break}if(!s.R){e=s;break}s=s.R}!function(t){vc[t.index]={site:t,halfedges:[]}}(t);var A=fc(t);if(Ec.insert(e,A),e||n){if(e===n)return gc(e),n=fc(e.site),Ec.insert(A,n),A.edge=n.edge=tc(e.site,A.site),uc(e),void uc(n);if(n){gc(e),gc(n);var l=e.site,c=l[0],u=l[1],g=t[0]-c,h=t[1]-u,d=n.site,f=d[0]-c,p=d[1]-u,I=2*(g*p-h*f),C=g*g+h*h,m=f*f+p*p,y=[(p*C-h*m)/I+c,(g*m-f*C)/I+u];nc(n.edge,l,d,y),A.edge=tc(l,t,null,y),n.edge=tc(t,d,null,y),uc(e),uc(n)}else A.edge=tc(e.site,A.site)}}function mc(t,e){var n=t.site,r=n[0],i=n[1],o=i-e;if(!o)return r;var a=t.P;if(!a)return-1/0;var s=(n=a.site)[0],A=n[1],l=A-e;if(!l)return s;var c=s-r,u=1/o-1/l,g=c/l;return u?(-g+Math.sqrt(g*g-2*u*(c*c/(-2*l)-A+l/2+i-o/2)))/u+r:(r+s)/2}function yc(t,e){var n=t.N;if(n)return mc(n,e);var r=t.site;return r[1]===e?r[0]:1/0}var Ec,vc,Bc,bc,Qc=1e-6,wc=1e-12;function xc(t,e){return e[1]-t[1]||e[0]-t[0]}function _c(t,e){var n,r,i,o=t.sort(xc).pop();for(bc=[],vc=new Array(t.length),Ec=new $l,Bc=new $l;;)if(i=Ac,o&&(!i||o[1]<i.y||o[1]===i.y&&o[0]<i.x))o[0]===n&&o[1]===r||(Cc(o),n=o[0],r=o[1]),o=t.pop();else{if(!i)break;Ic(i.arc)}if(function(){for(var t,e,n,r,i=0,o=vc.length;i<o;++i)if((t=vc[i])&&(r=(e=t.halfedges).length)){var a=new Array(r),s=new Array(r);for(n=0;n<r;++n)a[n]=n,s[n]=oc(t,bc[e[n]]);for(a.sort(function(t,e){return s[e]-s[t]}),n=0;n<r;++n)s[n]=e[a[n]];for(n=0;n<r;++n)e[n]=s[n]}}(),e){var a=+e[0][0],s=+e[0][1],A=+e[1][0],l=+e[1][1];!function(t,e,n,r){for(var i,o=bc.length;o--;)ic(i=bc[o],t,e,n,r)&&rc(i,t,e,n,r)&&(Math.abs(i[0][0]-i[1][0])>Qc||Math.abs(i[0][1]-i[1][1])>Qc)||delete bc[o]}(a,s,A,l),function(t,e,n,r){var i,o,a,s,A,l,c,u,g,h,d,f,p=vc.length,I=!0;for(i=0;i<p;++i)if(o=vc[i]){for(a=o.site,s=(A=o.halfedges).length;s--;)bc[A[s]]||A.splice(s,1);for(s=0,l=A.length;s<l;)d=(h=sc(o,bc[A[s]]))[0],f=h[1],u=(c=ac(o,bc[A[++s%l]]))[0],g=c[1],(Math.abs(d-u)>Qc||Math.abs(f-g)>Qc)&&(A.splice(s,0,bc.push(ec(a,h,Math.abs(d-t)<Qc&&r-f>Qc?[t,Math.abs(u-t)<Qc?g:r]:Math.abs(f-r)<Qc&&n-d>Qc?[Math.abs(g-r)<Qc?u:n,r]:Math.abs(d-n)<Qc&&f-e>Qc?[n,Math.abs(u-n)<Qc?g:e]:Math.abs(f-e)<Qc&&d-t>Qc?[Math.abs(g-e)<Qc?u:t,e]:null))-1),++l);l&&(I=!1)}if(I){var C,m,y,E=1/0;for(i=0,I=null;i<p;++i)(o=vc[i])&&(y=(C=(a=o.site)[0]-t)*C+(m=a[1]-e)*m)<E&&(E=y,I=o);if(I){var v=[t,e],B=[t,r],b=[n,r],Q=[n,e];I.halfedges.push(bc.push(ec(a=I.site,v,B))-1,bc.push(ec(a,B,b))-1,bc.push(ec(a,b,Q))-1,bc.push(ec(a,Q,v))-1)}}for(i=0;i<p;++i)(o=vc[i])&&(o.halfedges.length||delete vc[i])}(a,s,A,l)}this.edges=bc,this.cells=vc,Ec=Bc=bc=vc=null}_c.prototype={constructor:_c,polygons:function(){var t=this.edges;return this.cells.map(function(e){var n=e.halfedges.map(function(n){return ac(e,t[n])});return n.data=e.site.data,n})},triangles:function(){var t=[],e=this.edges;return this.cells.forEach(function(n,r){if(o=(i=n.halfedges).length)for(var i,o,a,s,A,l,c=n.site,u=-1,g=e[i[o-1]],h=g.left===c?g.right:g.left;++u<o;)a=h,h=(g=e[i[u]]).left===c?g.right:g.left,a&&h&&r<a.index&&r<h.index&&(A=a,l=h,((s=c)[0]-l[0])*(A[1]-s[1])-(s[0]-A[0])*(l[1]-s[1])<0)&&t.push([c.data,a.data,h.data])}),t},links:function(){return this.edges.filter(function(t){return t.right}).map(function(t){return{source:t.left.data,target:t.right.data}})},find:function(t,e,n){for(var r,i,o=this,a=o._found||0,s=o.cells.length;!(i=o.cells[a]);)if(++a>=s)return null;var A=t-i.site[0],l=e-i.site[1],c=A*A+l*l;do{i=o.cells[r=a],a=null,i.halfedges.forEach(function(n){var r=o.edges[n],s=r.left;if(s!==i.site&&s||(s=r.right)){var A=t-s[0],l=e-s[1],u=A*A+l*l;u<c&&(c=u,a=s.index)}})}while(null!==a);return o._found=r,null==n||c<=n*n?i.site:null}};var Dc=function(){var t=zl,e=Jl,n=null;function r(r){return new _c(r.map(function(n,i){var o=[Math.round(t(n,i,r)/Qc)*Qc,Math.round(e(n,i,r)/Qc)*Qc];return o.index=i,o.data=n,o}),n)}return r.polygons=function(t){return r(t).polygons()},r.links=function(t){return r(t).links()},r.triangles=function(t){return r(t).triangles()},r.x=function(e){return arguments.length?(t="function"==typeof e?e:Kl(+e),r):t},r.y=function(t){return arguments.length?(e="function"==typeof t?t:Kl(+t),r):e},r.extent=function(t){return arguments.length?(n=null==t?null:[[+t[0][0],+t[0][1]],[+t[1][0],+t[1][1]]],r):n&&[[n[0][0],n[0][1]],[n[1][0],n[1][1]]]},r.size=function(t){return arguments.length?(n=null==t?null:[[0,0],[+t[0],+t[1]]],r):n&&[n[1][0]-n[0][0],n[1][1]-n[0][1]]},r},Mc=function(t){return function(){return t}};function kc(t,e,n){this.target=t,this.type=e,this.transform=n}function Sc(t,e,n){this.k=t,this.x=e,this.y=n}Sc.prototype={constructor:Sc,scale:function(t){return 1===t?this:new Sc(this.k*t,this.x,this.y)},translate:function(t,e){return 0===t&0===e?this:new Sc(this.k,this.x+this.k*t,this.y+this.k*e)},apply:function(t){return[t[0]*this.k+this.x,t[1]*this.k+this.y]},applyX:function(t){return t*this.k+this.x},applyY:function(t){return t*this.k+this.y},invert:function(t){return[(t[0]-this.x)/this.k,(t[1]-this.y)/this.k]},invertX:function(t){return(t-this.x)/this.k},invertY:function(t){return(t-this.y)/this.k},rescaleX:function(t){return t.copy().domain(t.range().map(this.invertX,this).map(t.invert,t))},rescaleY:function(t){return t.copy().domain(t.range().map(this.invertY,this).map(t.invert,t))},toString:function(){return"translate("+this.x+","+this.y+") scale("+this.k+")"}};var Lc=new Sc(1,0,0);function Rc(t){return t.__zoom||Lc}function Nc(){jl.event.stopImmediatePropagation()}Rc.prototype=Sc.prototype;var Fc=function(){jl.event.preventDefault(),jl.event.stopImmediatePropagation()};function Tc(){return!jl.event.button}function Uc(){var t,e,n=this;return n instanceof SVGElement?(t=(n=n.ownerSVGElement||n).width.baseVal.value,e=n.height.baseVal.value):(t=n.clientWidth,e=n.clientHeight),[[0,0],[t,e]]}function Oc(){return this.__zoom||Lc}function Gc(){return-jl.event.deltaY*(jl.event.deltaMode?120:1)/500}function Hc(){return"ontouchstart"in this}function jc(t,e,n){var r=t.invertX(e[0][0])-n[0][0],i=t.invertX(e[1][0])-n[1][0],o=t.invertY(e[0][1])-n[0][1],a=t.invertY(e[1][1])-n[1][1];return t.translate(i>r?(r+i)/2:Math.min(0,r)||Math.max(0,i),a>o?(o+a)/2:Math.min(0,o)||Math.max(0,a))}var Yc=function(){var t,e,n=Tc,r=Uc,i=jc,o=Gc,a=Hc,s=[0,1/0],A=[[-1/0,-1/0],[1/0,1/0]],l=250,c=Ha.x,u=[],g=Object(et.dispatch)("start","zoom","end"),h=500,d=150,f=0;function p(t){t.property("__zoom",Oc).on("wheel.zoom",B).on("mousedown.zoom",b).on("dblclick.zoom",Q).filter(a).on("touchstart.zoom",w).on("touchmove.zoom",x).on("touchend.zoom touchcancel.zoom",_).style("touch-action","none").style("-webkit-tap-highlight-color","rgba(0,0,0,0)")}function I(t,e){return(e=Math.max(s[0],Math.min(s[1],e)))===t.k?t:new Sc(e,t.x,t.y)}function C(t,e,n){var r=e[0]-n[0]*t.k,i=e[1]-n[1]*t.k;return r===t.x&&i===t.y?t:new Sc(t.k,r,i)}function m(t){return[(+t[0][0]+ +t[1][0])/2,(+t[0][1]+ +t[1][1])/2]}function y(t,e,n){t.on("start.zoom",function(){E(this,arguments).start()}).on("interrupt.zoom end.zoom",function(){E(this,arguments).end()}).tween("zoom",function(){var t=this,i=arguments,o=E(t,i),a=r.apply(t,i),s=n||m(a),A=Math.max(a[1][0]-a[0][0],a[1][1]-a[0][1]),l=t.__zoom,u="function"==typeof e?e.apply(t,i):e,g=c(l.invert(s).concat(A/l.k),u.invert(s).concat(A/u.k));return function(t){if(1===t)t=u;else{var e=g(t),n=A/e[2];t=new Sc(n,s[0]-e[0]*n,s[1]-e[1]*n)}o.zoom(null,t)}})}function E(t,e){for(var n,r=0,i=u.length;r<i;++r)if((n=u[r]).that===t)return n;return new v(t,e)}function v(t,e){this.that=t,this.args=e,this.index=-1,this.active=0,this.extent=r.apply(t,e)}function B(){if(n.apply(this,arguments)){var t=E(this,arguments),e=this.__zoom,r=Math.max(s[0],Math.min(s[1],e.k*Math.pow(2,o.apply(this,arguments)))),a=Object(jl.mouse)(this);if(t.wheel)t.mouse[0][0]===a[0]&&t.mouse[0][1]===a[1]||(t.mouse[1]=e.invert(t.mouse[0]=a)),clearTimeout(t.wheel);else{if(e.k===r)return;t.mouse=[a,e.invert(a)],Object(Pl.b)(this),t.start()}Fc(),t.wheel=setTimeout(function(){t.wheel=null,t.end()},d),t.zoom("mouse",i(C(I(e,r),t.mouse[0],t.mouse[1]),t.extent,A))}}function b(){if(!e&&n.apply(this,arguments)){var t=E(this,arguments),r=Object(jl.select)(jl.event.view).on("mousemove.zoom",function(){if(Fc(),!t.moved){var e=jl.event.clientX-a,n=jl.event.clientY-s;t.moved=e*e+n*n>f}t.zoom("mouse",i(C(t.that.__zoom,t.mouse[0]=Object(jl.mouse)(t.that),t.mouse[1]),t.extent,A))},!0).on("mouseup.zoom",function(){r.on("mousemove.zoom mouseup.zoom",null),Object(nt.c)(jl.event.view,t.moved),Fc(),t.end()},!0),o=Object(jl.mouse)(this),a=jl.event.clientX,s=jl.event.clientY;Object(nt.b)(jl.event.view),Nc(),t.mouse=[o,this.__zoom.invert(o)],Object(Pl.b)(this),t.start()}}function Q(){if(n.apply(this,arguments)){var t=this.__zoom,e=Object(jl.mouse)(this),o=t.invert(e),a=t.k*(jl.event.shiftKey?.5:2),s=i(C(I(t,a),e,o),r.apply(this,arguments),A);Fc(),l>0?Object(jl.select)(this).transition().duration(l).call(y,s,e):Object(jl.select)(this).call(p.transform,s)}}function w(){if(n.apply(this,arguments)){var e,r,i,o,a=E(this,arguments),s=jl.event.changedTouches,A=s.length;for(Nc(),r=0;r<A;++r)i=s[r],o=[o=Object(jl.touch)(this,s,i.identifier),this.__zoom.invert(o),i.identifier],a.touch0?a.touch1||(a.touch1=o):(a.touch0=o,e=!0);if(t&&(t=clearTimeout(t),!a.touch1))return a.end(),void((o=Object(jl.select)(this).on("dblclick.zoom"))&&o.apply(this,arguments));e&&(t=setTimeout(function(){t=null},h),Object(Pl.b)(this),a.start())}}function x(){var e,n,r,o,a=E(this,arguments),s=jl.event.changedTouches,l=s.length;for(Fc(),t&&(t=clearTimeout(t)),e=0;e<l;++e)n=s[e],r=Object(jl.touch)(this,s,n.identifier),a.touch0&&a.touch0[2]===n.identifier?a.touch0[0]=r:a.touch1&&a.touch1[2]===n.identifier&&(a.touch1[0]=r);if(n=a.that.__zoom,a.touch1){var c=a.touch0[0],u=a.touch0[1],g=a.touch1[0],h=a.touch1[1],d=(d=g[0]-c[0])*d+(d=g[1]-c[1])*d,f=(f=h[0]-u[0])*f+(f=h[1]-u[1])*f;n=I(n,Math.sqrt(d/f)),r=[(c[0]+g[0])/2,(c[1]+g[1])/2],o=[(u[0]+h[0])/2,(u[1]+h[1])/2]}else{if(!a.touch0)return;r=a.touch0[0],o=a.touch0[1]}a.zoom("touch",i(C(n,r,o),a.extent,A))}function _(){var t,n,r=E(this,arguments),i=jl.event.changedTouches,o=i.length;for(Nc(),e&&clearTimeout(e),e=setTimeout(function(){e=null},h),t=0;t<o;++t)n=i[t],r.touch0&&r.touch0[2]===n.identifier?delete r.touch0:r.touch1&&r.touch1[2]===n.identifier&&delete r.touch1;r.touch1&&!r.touch0&&(r.touch0=r.touch1,delete r.touch1),r.touch0?r.touch0[1]=this.__zoom.invert(r.touch0[0]):r.end()}return p.transform=function(t,e){var n=t.selection?t.selection():t;n.property("__zoom",Oc),t!==n?y(t,e):n.interrupt().each(function(){E(this,arguments).start().zoom(null,"function"==typeof e?e.apply(this,arguments):e).end()})},p.scaleBy=function(t,e){p.scaleTo(t,function(){var t=this.__zoom.k,n="function"==typeof e?e.apply(this,arguments):e;return t*n})},p.scaleTo=function(t,e){p.transform(t,function(){var t=r.apply(this,arguments),n=this.__zoom,o=m(t),a=n.invert(o),s="function"==typeof e?e.apply(this,arguments):e;return i(C(I(n,s),o,a),t,A)})},p.translateBy=function(t,e,n){p.transform(t,function(){return i(this.__zoom.translate("function"==typeof e?e.apply(this,arguments):e,"function"==typeof n?n.apply(this,arguments):n),r.apply(this,arguments),A)})},p.translateTo=function(t,e,n){p.transform(t,function(){var t=r.apply(this,arguments),o=this.__zoom,a=m(t);return i(Lc.translate(a[0],a[1]).scale(o.k).translate("function"==typeof e?-e.apply(this,arguments):-e,"function"==typeof n?-n.apply(this,arguments):-n),t,A)})},v.prototype={start:function(){return 1==++this.active&&(this.index=u.push(this)-1,this.emit("start")),this},zoom:function(t,e){return this.mouse&&"mouse"!==t&&(this.mouse[1]=e.invert(this.mouse[0])),this.touch0&&"touch"!==t&&(this.touch0[1]=e.invert(this.touch0[0])),this.touch1&&"touch"!==t&&(this.touch1[1]=e.invert(this.touch1[0])),this.that.__zoom=e,this.emit("zoom"),this},end:function(){return 0==--this.active&&(u.splice(this.index,1),this.index=-1,this.emit("end")),this},emit:function(t){Object(jl.customEvent)(new kc(p,t,this.that.__zoom),g.apply,g,[t,this.that,this.args])}},p.wheelDelta=function(t){return arguments.length?(o="function"==typeof t?t:Mc(+t),p):o},p.filter=function(t){return arguments.length?(n="function"==typeof t?t:Mc(!!t),p):n},p.touchable=function(t){return arguments.length?(a="function"==typeof t?t:Mc(!!t),p):a},p.extent=function(t){return arguments.length?(r="function"==typeof t?t:Mc([[+t[0][0],+t[0][1]],[+t[1][0],+t[1][1]]]),p):r},p.scaleExtent=function(t){return arguments.length?(s[0]=+t[0],s[1]=+t[1],p):[s[0],s[1]]},p.translateExtent=function(t){return arguments.length?(A[0][0]=+t[0][0],A[1][0]=+t[1][0],A[0][1]=+t[0][1],A[1][1]=+t[1][1],p):[[A[0][0],A[0][1]],[A[1][0],A[1][1]]]},p.constrain=function(t){return arguments.length?(i=t,p):i},p.duration=function(t){return arguments.length?(l=+t,p):l},p.interpolate=function(t){return arguments.length?(c=t,p):c},p.on=function(){var t=g.on.apply(g,arguments);return t===g?p:t},p.clickDistance=function(t){return arguments.length?(f=(t=+t)*t,p):Math.sqrt(f)},p};n.d(e,"version",function(){return"5.9.7"}),n.d(e,"bisect",function(){return r.bisect}),n.d(e,"bisectRight",function(){return r.bisectRight}),n.d(e,"bisectLeft",function(){return r.bisectLeft}),n.d(e,"ascending",function(){return r.ascending}),n.d(e,"bisector",function(){return r.bisector}),n.d(e,"cross",function(){return r.cross}),n.d(e,"descending",function(){return r.descending}),n.d(e,"deviation",function(){return r.deviation}),n.d(e,"extent",function(){return r.extent}),n.d(e,"histogram",function(){return r.histogram}),n.d(e,"thresholdFreedmanDiaconis",function(){return r.thresholdFreedmanDiaconis}),n.d(e,"thresholdScott",function(){return r.thresholdScott}),n.d(e,"thresholdSturges",function(){return r.thresholdSturges}),n.d(e,"max",function(){return r.max}),n.d(e,"mean",function(){return r.mean}),n.d(e,"median",function(){return r.median}),n.d(e,"merge",function(){return r.merge}),n.d(e,"min",function(){return r.min}),n.d(e,"pairs",function(){return r.pairs}),n.d(e,"permute",function(){return r.permute}),n.d(e,"quantile",function(){return r.quantile}),n.d(e,"range",function(){return r.range}),n.d(e,"scan",function(){return r.scan}),n.d(e,"shuffle",function(){return r.shuffle}),n.d(e,"sum",function(){return r.sum}),n.d(e,"ticks",function(){return r.ticks}),n.d(e,"tickIncrement",function(){return r.tickIncrement}),n.d(e,"tickStep",function(){return r.tickStep}),n.d(e,"transpose",function(){return r.transpose}),n.d(e,"variance",function(){return r.variance}),n.d(e,"zip",function(){return r.zip}),n.d(e,"axisTop",function(){return f}),n.d(e,"axisRight",function(){return p}),n.d(e,"axisBottom",function(){return I}),n.d(e,"axisLeft",function(){return C}),n.d(e,"brush",function(){return m.brush}),n.d(e,"brushX",function(){return m.brushX}),n.d(e,"brushY",function(){return m.brushY}),n.d(e,"brushSelection",function(){return m.brushSelection}),n.d(e,"chord",function(){return x}),n.d(e,"ribbon",function(){return F}),n.d(e,"nest",function(){return T.d}),n.d(e,"set",function(){return T.e}),n.d(e,"map",function(){return T.c}),n.d(e,"keys",function(){return T.b}),n.d(e,"values",function(){return T.f}),n.d(e,"entries",function(){return T.a}),n.d(e,"color",function(){return U.a}),n.d(e,"rgb",function(){return U.h}),n.d(e,"hsl",function(){return U.e}),n.d(e,"lab",function(){return U.f}),n.d(e,"hcl",function(){return U.d}),n.d(e,"lch",function(){return U.g}),n.d(e,"gray",function(){return U.c}),n.d(e,"cubehelix",function(){return U.b}),n.d(e,"contours",function(){return q}),n.d(e,"contourDensity",function(){return tt}),n.d(e,"dispatch",function(){return et.dispatch}),n.d(e,"drag",function(){return nt.a}),n.d(e,"dragDisable",function(){return nt.b}),n.d(e,"dragEnable",function(){return nt.c}),n.d(e,"dsvFormat",function(){return rt.g}),n.d(e,"csvParse",function(){return rt.e}),n.d(e,"csvParseRows",function(){return rt.f}),n.d(e,"csvFormat",function(){return rt.b}),n.d(e,"csvFormatBody",function(){return rt.c}),n.d(e,"csvFormatRows",function(){return rt.d}),n.d(e,"tsvParse",function(){return rt.k}),n.d(e,"tsvParseRows",function(){return rt.l}),n.d(e,"tsvFormat",function(){return rt.h}),n.d(e,"tsvFormatBody",function(){return rt.i}),n.d(e,"tsvFormatRows",function(){return rt.j}),n.d(e,"autoType",function(){return rt.a}),n.d(e,"easeLinear",function(){return it.y}),n.d(e,"easeQuad",function(){return it.D}),n.d(e,"easeQuadIn",function(){return it.E}),n.d(e,"easeQuadOut",function(){return it.G}),n.d(e,"easeQuadInOut",function(){return it.F}),n.d(e,"easeCubic",function(){return it.m}),n.d(e,"easeCubicIn",function(){return it.n}),n.d(e,"easeCubicOut",function(){return it.p}),n.d(e,"easeCubicInOut",function(){return it.o}),n.d(e,"easePoly",function(){return it.z}),n.d(e,"easePolyIn",function(){return it.A}),n.d(e,"easePolyOut",function(){return it.C}),n.d(e,"easePolyInOut",function(){return it.B}),n.d(e,"easeSin",function(){return it.H}),n.d(e,"easeSinIn",function(){return it.I}),n.d(e,"easeSinOut",function(){return it.K}),n.d(e,"easeSinInOut",function(){return it.J}),n.d(e,"easeExp",function(){return it.u}),n.d(e,"easeExpIn",function(){return it.v}),n.d(e,"easeExpOut",function(){return it.x}),n.d(e,"easeExpInOut",function(){return it.w}),n.d(e,"easeCircle",function(){return it.i}),n.d(e,"easeCircleIn",function(){return it.j}),n.d(e,"easeCircleOut",function(){return it.l}),n.d(e,"easeCircleInOut",function(){return it.k}),n.d(e,"easeBounce",function(){return it.e}),n.d(e,"easeBounceIn",function(){return it.f}),n.d(e,"easeBounceOut",function(){return it.h}),n.d(e,"easeBounceInOut",function(){return it.g}),n.d(e,"easeBack",function(){return it.a}),n.d(e,"easeBackIn",function(){return it.b}),n.d(e,"easeBackOut",function(){return it.d}),n.d(e,"easeBackInOut",function(){return it.c}),n.d(e,"easeElastic",function(){return it.q}),n.d(e,"easeElasticIn",function(){return it.r}),n.d(e,"easeElasticOut",function(){return it.t}),n.d(e,"easeElasticInOut",function(){return it.s}),n.d(e,"blob",function(){return ot.blob}),n.d(e,"buffer",function(){return ot.buffer}),n.d(e,"dsv",function(){return ot.dsv}),n.d(e,"csv",function(){return ot.csv}),n.d(e,"tsv",function(){return ot.tsv}),n.d(e,"image",function(){return ot.image}),n.d(e,"json",function(){return ot.json}),n.d(e,"text",function(){return ot.text}),n.d(e,"xml",function(){return ot.xml}),n.d(e,"html",function(){return ot.html}),n.d(e,"svg",function(){return ot.svg}),n.d(e,"forceCenter",function(){return at}),n.d(e,"forceCollide",function(){return mt}),n.d(e,"forceLink",function(){return vt}),n.d(e,"forceManyBody",function(){return Dt}),n.d(e,"forceRadial",function(){return Mt}),n.d(e,"forceSimulation",function(){return _t}),n.d(e,"forceX",function(){return kt}),n.d(e,"forceY",function(){return St}),n.d(e,"formatDefaultLocale",function(){return Lt.b}),n.d(e,"format",function(){return Lt.a}),n.d(e,"formatPrefix",function(){return Lt.d}),n.d(e,"formatLocale",function(){return Lt.c}),n.d(e,"formatSpecifier",function(){return Lt.e}),n.d(e,"precisionFixed",function(){return Lt.f}),n.d(e,"precisionPrefix",function(){return Lt.g}),n.d(e,"precisionRound",function(){return Lt.h}),n.d(e,"geoArea",function(){return we}),n.d(e,"geoBounds",function(){return mn}),n.d(e,"geoCentroid",function(){return kn}),n.d(e,"geoCircle",function(){return jn}),n.d(e,"geoClipAntimeridian",function(){return tr}),n.d(e,"geoClipCircle",function(){return er}),n.d(e,"geoClipExtent",function(){return lr}),n.d(e,"geoClipRectangle",function(){return or}),n.d(e,"geoContains",function(){return xr}),n.d(e,"geoDistance",function(){return Cr}),n.d(e,"geoGraticule",function(){return Mr}),n.d(e,"geoGraticule10",function(){return kr}),n.d(e,"geoInterpolate",function(){return Fr}),n.d(e,"geoLength",function(){return fr}),n.d(e,"geoPath",function(){return Ri}),n.d(e,"geoAlbers",function(){return $i}),n.d(e,"geoAlbersUsa",function(){return to}),n.d(e,"geoAzimuthalEqualArea",function(){return io}),n.d(e,"geoAzimuthalEqualAreaRaw",function(){return ro}),n.d(e,"geoAzimuthalEquidistant",function(){return ao}),n.d(e,"geoAzimuthalEquidistantRaw",function(){return oo}),n.d(e,"geoConicConformal",function(){return go}),n.d(e,"geoConicConformalRaw",function(){return uo}),n.d(e,"geoConicEqualArea",function(){return Zi}),n.d(e,"geoConicEqualAreaRaw",function(){return Xi}),n.d(e,"geoConicEquidistant",function(){return Io}),n.d(e,"geoConicEquidistantRaw",function(){return po}),n.d(e,"geoEqualEarth",function(){return bo}),n.d(e,"geoEqualEarthRaw",function(){return Bo}),n.d(e,"geoEquirectangular",function(){return fo}),n.d(e,"geoEquirectangularRaw",function(){return ho}),n.d(e,"geoGnomonic",function(){return wo}),n.d(e,"geoGnomonicRaw",function(){return Qo}),n.d(e,"geoIdentity",function(){return _o}),n.d(e,"geoProjection",function(){return qi}),n.d(e,"geoProjectionMutator",function(){return Vi}),n.d(e,"geoMercator",function(){return Ao}),n.d(e,"geoMercatorRaw",function(){return so}),n.d(e,"geoNaturalEarth1",function(){return Mo}),n.d(e,"geoNaturalEarth1Raw",function(){return Do}),n.d(e,"geoOrthographic",function(){return So}),n.d(e,"geoOrthographicRaw",function(){return ko}),n.d(e,"geoStereographic",function(){return Ro}),n.d(e,"geoStereographicRaw",function(){return Lo}),n.d(e,"geoTransverseMercator",function(){return Fo}),n.d(e,"geoTransverseMercatorRaw",function(){return No}),n.d(e,"geoRotation",function(){return On}),n.d(e,"geoStream",function(){return Ce}),n.d(e,"geoTransform",function(){return Ni}),n.d(e,"cluster",function(){return Go}),n.d(e,"hierarchy",function(){return jo}),n.d(e,"pack",function(){return ha}),n.d(e,"packSiblings",function(){return sa}),n.d(e,"packEnclose",function(){return qo}),n.d(e,"partition",function(){return ma}),n.d(e,"stratify",function(){return Qa}),n.d(e,"tree",function(){return Sa}),n.d(e,"treemap",function(){return Ta}),n.d(e,"treemapBinary",function(){return Ua}),n.d(e,"treemapDice",function(){return Ca}),n.d(e,"treemapSlice",function(){return La}),n.d(e,"treemapSliceDice",function(){return Oa}),n.d(e,"treemapSquarify",function(){return Fa}),n.d(e,"treemapResquarify",function(){return Ga}),n.d(e,"interpolate",function(){return Ha.a}),n.d(e,"interpolateArray",function(){return Ha.b}),n.d(e,"interpolateBasis",function(){return Ha.c}),n.d(e,"interpolateBasisClosed",function(){return Ha.d}),n.d(e,"interpolateDate",function(){return Ha.g}),n.d(e,"interpolateDiscrete",function(){return Ha.h}),n.d(e,"interpolateHue",function(){return Ha.m}),n.d(e,"interpolateNumber",function(){return Ha.o}),n.d(e,"interpolateObject",function(){return Ha.p}),n.d(e,"interpolateRound",function(){return Ha.t}),n.d(e,"interpolateString",function(){return Ha.u}),n.d(e,"interpolateTransformCss",function(){return Ha.v}),n.d(e,"interpolateTransformSvg",function(){return Ha.w}),n.d(e,"interpolateZoom",function(){return Ha.x}),n.d(e,"interpolateRgb",function(){return Ha.q}),n.d(e,"interpolateRgbBasis",function(){return Ha.r}),n.d(e,"interpolateRgbBasisClosed",function(){return Ha.s}),n.d(e,"interpolateHsl",function(){return Ha.k}),n.d(e,"interpolateHslLong",function(){return Ha.l}),n.d(e,"interpolateLab",function(){return Ha.n}),n.d(e,"interpolateHcl",function(){return Ha.i}),n.d(e,"interpolateHclLong",function(){return Ha.j}),n.d(e,"interpolateCubehelix",function(){return Ha.e}),n.d(e,"interpolateCubehelixLong",function(){return Ha.f}),n.d(e,"piecewise",function(){return Ha.y}),n.d(e,"quantize",function(){return Ha.z}),n.d(e,"path",function(){return M.path}),n.d(e,"polygonArea",function(){return ja}),n.d(e,"polygonCentroid",function(){return Ya}),n.d(e,"polygonHull",function(){return Ja}),n.d(e,"polygonContains",function(){return qa}),n.d(e,"polygonLength",function(){return Va}),n.d(e,"quadtree",function(){return ht}),n.d(e,"randomUniform",function(){return Wa.randomUniform}),n.d(e,"randomNormal",function(){return Wa.randomNormal}),n.d(e,"randomLogNormal",function(){return Wa.randomLogNormal}),n.d(e,"randomBates",function(){return Wa.randomBates}),n.d(e,"randomIrwinHall",function(){return Wa.randomIrwinHall}),n.d(e,"randomExponential",function(){return Wa.randomExponential}),n.d(e,"scaleBand",function(){return is}),n.d(e,"scalePoint",function(){return os}),n.d(e,"scaleIdentity",function(){return ys}),n.d(e,"scaleLinear",function(){return ms}),n.d(e,"scaleLog",function(){return Ds}),n.d(e,"scaleSymlog",function(){return Ls}),n.d(e,"scaleOrdinal",function(){return rs}),n.d(e,"scaleImplicit",function(){return ns}),n.d(e,"scalePow",function(){return Us}),n.d(e,"scaleSqrt",function(){return Os}),n.d(e,"scaleQuantile",function(){return Gs}),n.d(e,"scaleQuantize",function(){return Hs}),n.d(e,"scaleThreshold",function(){return js}),n.d(e,"scaleTime",function(){return eA}),n.d(e,"scaleUtc",function(){return nA}),n.d(e,"scaleSequential",function(){return oA}),n.d(e,"scaleSequentialLog",function(){return aA}),n.d(e,"scaleSequentialPow",function(){return AA}),n.d(e,"scaleSequentialSqrt",function(){return lA}),n.d(e,"scaleSequentialSymlog",function(){return sA}),n.d(e,"scaleSequentialQuantile",function(){return cA}),n.d(e,"scaleDiverging",function(){return gA}),n.d(e,"scaleDivergingLog",function(){return hA}),n.d(e,"scaleDivergingPow",function(){return fA}),n.d(e,"scaleDivergingSqrt",function(){return pA}),n.d(e,"scaleDivergingSymlog",function(){return dA}),n.d(e,"tickFormat",function(){return Is}),n.d(e,"schemeCategory10",function(){return CA}),n.d(e,"schemeAccent",function(){return mA}),n.d(e,"schemeDark2",function(){return yA}),n.d(e,"schemePaired",function(){return EA}),n.d(e,"schemePastel1",function(){return vA}),n.d(e,"schemePastel2",function(){return BA}),n.d(e,"schemeSet1",function(){return bA}),n.d(e,"schemeSet2",function(){return QA}),n.d(e,"schemeSet3",function(){return wA}),n.d(e,"interpolateBrBG",function(){return DA}),n.d(e,"schemeBrBG",function(){return _A}),n.d(e,"interpolatePRGn",function(){return kA}),n.d(e,"schemePRGn",function(){return MA}),n.d(e,"interpolatePiYG",function(){return LA}),n.d(e,"schemePiYG",function(){return SA}),n.d(e,"interpolatePuOr",function(){return NA}),n.d(e,"schemePuOr",function(){return RA}),n.d(e,"interpolateRdBu",function(){return TA}),n.d(e,"schemeRdBu",function(){return FA}),n.d(e,"interpolateRdGy",function(){return OA}),n.d(e,"schemeRdGy",function(){return UA}),n.d(e,"interpolateRdYlBu",function(){return HA}),n.d(e,"schemeRdYlBu",function(){return GA}),n.d(e,"interpolateRdYlGn",function(){return YA}),n.d(e,"schemeRdYlGn",function(){return jA}),n.d(e,"interpolateSpectral",function(){return KA}),n.d(e,"schemeSpectral",function(){return PA}),n.d(e,"interpolateBuGn",function(){return JA}),n.d(e,"schemeBuGn",function(){return zA}),n.d(e,"interpolateBuPu",function(){return VA}),n.d(e,"schemeBuPu",function(){return qA}),n.d(e,"interpolateGnBu",function(){return XA}),n.d(e,"schemeGnBu",function(){return WA}),n.d(e,"interpolateOrRd",function(){return $A}),n.d(e,"schemeOrRd",function(){return ZA}),n.d(e,"interpolatePuBuGn",function(){return el}),n.d(e,"schemePuBuGn",function(){return tl}),n.d(e,"interpolatePuBu",function(){return rl}),n.d(e,"schemePuBu",function(){return nl}),n.d(e,"interpolatePuRd",function(){return ol}),n.d(e,"schemePuRd",function(){return il}),n.d(e,"interpolateRdPu",function(){return sl}),n.d(e,"schemeRdPu",function(){return al}),n.d(e,"interpolateYlGnBu",function(){return ll}),n.d(e,"schemeYlGnBu",function(){return Al}),n.d(e,"interpolateYlGn",function(){return ul}),n.d(e,"schemeYlGn",function(){return cl}),n.d(e,"interpolateYlOrBr",function(){return hl}),n.d(e,"schemeYlOrBr",function(){return gl}),n.d(e,"interpolateYlOrRd",function(){return fl}),n.d(e,"schemeYlOrRd",function(){return dl}),n.d(e,"interpolateBlues",function(){return Il}),n.d(e,"schemeBlues",function(){return pl}),n.d(e,"interpolateGreens",function(){return ml}),n.d(e,"schemeGreens",function(){return Cl}),n.d(e,"interpolateGreys",function(){return El}),n.d(e,"schemeGreys",function(){return yl}),n.d(e,"interpolatePurples",function(){return Bl}),n.d(e,"schemePurples",function(){return vl}),n.d(e,"interpolateReds",function(){return Ql}),n.d(e,"schemeReds",function(){return bl}),n.d(e,"interpolateOranges",function(){return xl}),n.d(e,"schemeOranges",function(){return wl}),n.d(e,"interpolateCubehelixDefault",function(){return _l}),n.d(e,"interpolateRainbow",function(){return Sl}),n.d(e,"interpolateWarm",function(){return Dl}),n.d(e,"interpolateCool",function(){return Ml}),n.d(e,"interpolateSinebow",function(){return Fl}),n.d(e,"interpolateViridis",function(){return Ul}),n.d(e,"interpolateMagma",function(){return Ol}),n.d(e,"interpolateInferno",function(){return Gl}),n.d(e,"interpolatePlasma",function(){return Hl}),n.d(e,"create",function(){return jl.create}),n.d(e,"creator",function(){return jl.creator}),n.d(e,"local",function(){return jl.local}),n.d(e,"matcher",function(){return jl.matcher}),n.d(e,"mouse",function(){return jl.mouse}),n.d(e,"namespace",function(){return jl.namespace}),n.d(e,"namespaces",function(){return jl.namespaces}),n.d(e,"clientPoint",function(){return jl.clientPoint}),n.d(e,"select",function(){return jl.select}),n.d(e,"selectAll",function(){return jl.selectAll}),n.d(e,"selection",function(){return jl.selection}),n.d(e,"selector",function(){return jl.selector}),n.d(e,"selectorAll",function(){return jl.selectorAll}),n.d(e,"style",function(){return jl.style}),n.d(e,"touch",function(){return jl.touch}),n.d(e,"touches",function(){return jl.touches}),n.d(e,"window",function(){return jl.window}),n.d(e,"event",function(){return jl.event}),n.d(e,"customEvent",function(){return jl.customEvent}),n.d(e,"arc",function(){return Yl.arc}),n.d(e,"area",function(){return Yl.area}),n.d(e,"line",function(){return Yl.line}),n.d(e,"pie",function(){return Yl.pie}),n.d(e,"areaRadial",function(){return Yl.areaRadial}),n.d(e,"radialArea",function(){return Yl.radialArea}),n.d(e,"lineRadial",function(){return Yl.lineRadial}),n.d(e,"radialLine",function(){return Yl.radialLine}),n.d(e,"pointRadial",function(){return Yl.pointRadial}),n.d(e,"linkHorizontal",function(){return Yl.linkHorizontal}),n.d(e,"linkVertical",function(){return Yl.linkVertical}),n.d(e,"linkRadial",function(){return Yl.linkRadial}),n.d(e,"symbol",function(){return Yl.symbol}),n.d(e,"symbols",function(){return Yl.symbols}),n.d(e,"symbolCircle",function(){return Yl.symbolCircle}),n.d(e,"symbolCross",function(){return Yl.symbolCross}),n.d(e,"symbolDiamond",function(){return Yl.symbolDiamond}),n.d(e,"symbolSquare",function(){return Yl.symbolSquare}),n.d(e,"symbolStar",function(){return Yl.symbolStar}),n.d(e,"symbolTriangle",function(){return Yl.symbolTriangle}),n.d(e,"symbolWye",function(){return Yl.symbolWye}),n.d(e,"curveBasisClosed",function(){return Yl.curveBasisClosed}),n.d(e,"curveBasisOpen",function(){return Yl.curveBasisOpen}),n.d(e,"curveBasis",function(){return Yl.curveBasis}),n.d(e,"curveBundle",function(){return Yl.curveBundle}),n.d(e,"curveCardinalClosed",function(){return Yl.curveCardinalClosed}),n.d(e,"curveCardinalOpen",function(){return Yl.curveCardinalOpen}),n.d(e,"curveCardinal",function(){return Yl.curveCardinal}),n.d(e,"curveCatmullRomClosed",function(){return Yl.curveCatmullRomClosed}),n.d(e,"curveCatmullRomOpen",function(){return Yl.curveCatmullRomOpen}),n.d(e,"curveCatmullRom",function(){return Yl.curveCatmullRom}),n.d(e,"curveLinearClosed",function(){return Yl.curveLinearClosed}),n.d(e,"curveLinear",function(){return Yl.curveLinear}),n.d(e,"curveMonotoneX",function(){return Yl.curveMonotoneX}),n.d(e,"curveMonotoneY",function(){return Yl.curveMonotoneY}),n.d(e,"curveNatural",function(){return Yl.curveNatural}),n.d(e,"curveStep",function(){return Yl.curveStep}),n.d(e,"curveStepAfter",function(){return Yl.curveStepAfter}),n.d(e,"curveStepBefore",function(){return Yl.curveStepBefore}),n.d(e,"stack",function(){return Yl.stack}),n.d(e,"stackOffsetExpand",function(){return Yl.stackOffsetExpand}),n.d(e,"stackOffsetDiverging",function(){return Yl.stackOffsetDiverging}),n.d(e,"stackOffsetNone",function(){return Yl.stackOffsetNone}),n.d(e,"stackOffsetSilhouette",function(){return Yl.stackOffsetSilhouette}),n.d(e,"stackOffsetWiggle",function(){return Yl.stackOffsetWiggle}),n.d(e,"stackOrderAppearance",function(){return Yl.stackOrderAppearance}),n.d(e,"stackOrderAscending",function(){return Yl.stackOrderAscending}),n.d(e,"stackOrderDescending",function(){return Yl.stackOrderDescending}),n.d(e,"stackOrderInsideOut",function(){return Yl.stackOrderInsideOut}),n.d(e,"stackOrderNone",function(){return Yl.stackOrderNone}),n.d(e,"stackOrderReverse",function(){return Yl.stackOrderReverse}),n.d(e,"timeInterval",function(){return Ys.timeInterval}),n.d(e,"timeMillisecond",function(){return Ys.timeMillisecond}),n.d(e,"timeMilliseconds",function(){return Ys.timeMilliseconds}),n.d(e,"utcMillisecond",function(){return Ys.utcMillisecond}),n.d(e,"utcMilliseconds",function(){return Ys.utcMilliseconds}),n.d(e,"timeSecond",function(){return Ys.timeSecond}),n.d(e,"timeSeconds",function(){return Ys.timeSeconds}),n.d(e,"utcSecond",function(){return Ys.utcSecond}),n.d(e,"utcSeconds",function(){return Ys.utcSeconds}),n.d(e,"timeMinute",function(){return Ys.timeMinute}),n.d(e,"timeMinutes",function(){return Ys.timeMinutes}),n.d(e,"timeHour",function(){return Ys.timeHour}),n.d(e,"timeHours",function(){return Ys.timeHours}),n.d(e,"timeDay",function(){return Ys.timeDay}),n.d(e,"timeDays",function(){return Ys.timeDays}),n.d(e,"timeWeek",function(){return Ys.timeWeek}),n.d(e,"timeWeeks",function(){return Ys.timeWeeks}),n.d(e,"timeSunday",function(){return Ys.timeSunday}),n.d(e,"timeSundays",function(){return Ys.timeSundays}),n.d(e,"timeMonday",function(){return Ys.timeMonday}),n.d(e,"timeMondays",function(){return Ys.timeMondays}),n.d(e,"timeTuesday",function(){return Ys.timeTuesday}),n.d(e,"timeTuesdays",function(){return Ys.timeTuesdays}),n.d(e,"timeWednesday",function(){return Ys.timeWednesday}),n.d(e,"timeWednesdays",function(){return Ys.timeWednesdays}),n.d(e,"timeThursday",function(){return Ys.timeThursday}),n.d(e,"timeThursdays",function(){return Ys.timeThursdays}),n.d(e,"timeFriday",function(){return Ys.timeFriday}),n.d(e,"timeFridays",function(){return Ys.timeFridays}),n.d(e,"timeSaturday",function(){return Ys.timeSaturday}),n.d(e,"timeSaturdays",function(){return Ys.timeSaturdays}),n.d(e,"timeMonth",function(){return Ys.timeMonth}),n.d(e,"timeMonths",function(){return Ys.timeMonths}),n.d(e,"timeYear",function(){return Ys.timeYear}),n.d(e,"timeYears",function(){return Ys.timeYears}),n.d(e,"utcMinute",function(){return Ys.utcMinute}),n.d(e,"utcMinutes",function(){return Ys.utcMinutes}),n.d(e,"utcHour",function(){return Ys.utcHour}),n.d(e,"utcHours",function(){return Ys.utcHours}),n.d(e,"utcDay",function(){return Ys.utcDay}),n.d(e,"utcDays",function(){return Ys.utcDays}),n.d(e,"utcWeek",function(){return Ys.utcWeek}),n.d(e,"utcWeeks",function(){return Ys.utcWeeks}),n.d(e,"utcSunday",function(){return Ys.utcSunday}),n.d(e,"utcSundays",function(){return Ys.utcSundays}),n.d(e,"utcMonday",function(){return Ys.utcMonday}),n.d(e,"utcMondays",function(){return Ys.utcMondays}),n.d(e,"utcTuesday",function(){return Ys.utcTuesday}),n.d(e,"utcTuesdays",function(){return Ys.utcTuesdays}),n.d(e,"utcWednesday",function(){return Ys.utcWednesday}),n.d(e,"utcWednesdays",function(){return Ys.utcWednesdays}),n.d(e,"utcThursday",function(){return Ys.utcThursday}),n.d(e,"utcThursdays",function(){return Ys.utcThursdays}),n.d(e,"utcFriday",function(){return Ys.utcFriday}),n.d(e,"utcFridays",function(){return Ys.utcFridays}),n.d(e,"utcSaturday",function(){return Ys.utcSaturday}),n.d(e,"utcSaturdays",function(){return Ys.utcSaturdays}),n.d(e,"utcMonth",function(){return Ys.utcMonth}),n.d(e,"utcMonths",function(){return Ys.utcMonths}),n.d(e,"utcYear",function(){return Ys.utcYear}),n.d(e,"utcYears",function(){return Ys.utcYears}),n.d(e,"timeFormatDefaultLocale",function(){return Ps.d}),n.d(e,"timeFormat",function(){return Ps.c}),n.d(e,"timeParse",function(){return Ps.f}),n.d(e,"utcFormat",function(){return Ps.g}),n.d(e,"utcParse",function(){return Ps.h}),n.d(e,"timeFormatLocale",function(){return Ps.e}),n.d(e,"isoFormat",function(){return Ps.a}),n.d(e,"isoParse",function(){return Ps.b}),n.d(e,"now",function(){return Bt.b}),n.d(e,"timer",function(){return Bt.d}),n.d(e,"timerFlush",function(){return Bt.e}),n.d(e,"timeout",function(){return Bt.c}),n.d(e,"interval",function(){return Bt.a}),n.d(e,"transition",function(){return Pl.c}),n.d(e,"active",function(){return Pl.a}),n.d(e,"interrupt",function(){return Pl.b}),n.d(e,"voronoi",function(){return Dc}),n.d(e,"zoom",function(){return Yc}),n.d(e,"zoomTransform",function(){return Rc}),n.d(e,"zoomIdentity",function(){return Lc})},function(t,e,n){"use strict";n.r(e);var r="http://www.w3.org/1999/xhtml",i={svg:"http://www.w3.org/2000/svg",xhtml:r,xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"},o=function(t){var e=t+="",n=e.indexOf(":");return n>=0&&"xmlns"!==(e=t.slice(0,n))&&(t=t.slice(n+1)),i.hasOwnProperty(e)?{space:i[e],local:t}:t},a=function(t){var e=o(t);return(e.local?function(t){return function(){return this.ownerDocument.createElementNS(t.space,t.local)}}:function(t){return function(){var e=this.ownerDocument,n=this.namespaceURI;return n===r&&e.documentElement.namespaceURI===r?e.createElement(t):e.createElementNS(n,t)}})(e)},s=0;function A(){this._="@"+(++s).toString(36)}A.prototype=function(){return new A}.prototype={constructor:A,get:function(t){for(var e=this._;!(e in t);)if(!(t=t.parentNode))return;return t[e]},set:function(t,e){return t[this._]=e},remove:function(t){return this._ in t&&delete t[this._]},toString:function(){return this._}};var l=function(t){return function(){return this.matches(t)}};if("undefined"!=typeof document){var c=document.documentElement;if(!c.matches){var u=c.webkitMatchesSelector||c.msMatchesSelector||c.mozMatchesSelector||c.oMatchesSelector;l=function(t){return function(){return u.call(this,t)}}}}var g=l,h={},d=null;function f(t,e,n){return t=p(t,e,n),function(e){var n=e.relatedTarget;n&&(n===this||8&n.compareDocumentPosition(this))||t.call(this,e)}}function p(t,e,n){return function(r){var i=d;d=r;try{t.call(this,this.__data__,e,n)}finally{d=i}}}function I(t){return t.trim().split(/^|\s+/).map(function(t){var e="",n=t.indexOf(".");return n>=0&&(e=t.slice(n+1),t=t.slice(0,n)),{type:t,name:e}})}function C(t){return function(){var e=this.__on;if(e){for(var n,r=0,i=-1,o=e.length;r<o;++r)n=e[r],t.type&&n.type!==t.type||n.name!==t.name?e[++i]=n:this.removeEventListener(n.type,n.listener,n.capture);++i?e.length=i:delete this.__on}}}function m(t,e,n){var r=h.hasOwnProperty(t.type)?f:p;return function(i,o,a){var s,A=this.__on,l=r(e,o,a);if(A)for(var c=0,u=A.length;c<u;++c)if((s=A[c]).type===t.type&&s.name===t.name)return this.removeEventListener(s.type,s.listener,s.capture),this.addEventListener(s.type,s.listener=l,s.capture=n),void(s.value=e);this.addEventListener(t.type,l,n),s={type:t.type,name:t.name,value:e,listener:l,capture:n},A?A.push(s):this.__on=[s]}}function y(){}"undefined"!=typeof document&&("onmouseenter"in document.documentElement||(h={mouseenter:"mouseover",mouseleave:"mouseout"}));var E=function(t){return null==t?y:function(){return this.querySelector(t)}};function v(){return[]}var B=function(t){return new Array(t.length)};function b(t,e){this.ownerDocument=t.ownerDocument,this.namespaceURI=t.namespaceURI,this._next=null,this._parent=t,this.__data__=e}b.prototype={constructor:b,appendChild:function(t){return this._parent.insertBefore(t,this._next)},insertBefore:function(t,e){return this._parent.insertBefore(t,e)},querySelector:function(t){return this._parent.querySelector(t)},querySelectorAll:function(t){return this._parent.querySelectorAll(t)}};var Q="$";function w(t,e,n,r,i,o){for(var a,s=0,A=e.length,l=o.length;s<l;++s)(a=e[s])?(a.__data__=o[s],r[s]=a):n[s]=new b(t,o[s]);for(;s<A;++s)(a=e[s])&&(i[s]=a)}function x(t,e,n,r,i,o,a){var s,A,l,c={},u=e.length,g=o.length,h=new Array(u);for(s=0;s<u;++s)(A=e[s])&&(h[s]=l=Q+a.call(A,A.__data__,s,e),l in c?i[s]=A:c[l]=A);for(s=0;s<g;++s)(A=c[l=Q+a.call(t,o[s],s,o)])?(r[s]=A,A.__data__=o[s],c[l]=null):n[s]=new b(t,o[s]);for(s=0;s<u;++s)(A=e[s])&&c[h[s]]===A&&(i[s]=A)}function _(t,e){return t<e?-1:t>e?1:t>=e?0:NaN}function D(t){return function(){this.removeAttribute(t)}}function M(t){return function(){this.removeAttributeNS(t.space,t.local)}}function k(t,e){return function(){this.setAttribute(t,e)}}function S(t,e){return function(){this.setAttributeNS(t.space,t.local,e)}}function L(t,e){return function(){var n=e.apply(this,arguments);null==n?this.removeAttribute(t):this.setAttribute(t,n)}}function R(t,e){return function(){var n=e.apply(this,arguments);null==n?this.removeAttributeNS(t.space,t.local):this.setAttributeNS(t.space,t.local,n)}}var N=function(t){return t.ownerDocument&&t.ownerDocument.defaultView||t.document&&t||t.defaultView};function F(t){return function(){this.style.removeProperty(t)}}function T(t,e,n){return function(){this.style.setProperty(t,e,n)}}function U(t,e,n){return function(){var r=e.apply(this,arguments);null==r?this.style.removeProperty(t):this.style.setProperty(t,r,n)}}function O(t){return function(){delete this[t]}}function G(t,e){return function(){this[t]=e}}function H(t,e){return function(){var n=e.apply(this,arguments);null==n?delete this[t]:this[t]=n}}function j(t){return t.trim().split(/^|\s+/)}function Y(t){return t.classList||new P(t)}function P(t){this._node=t,this._names=j(t.getAttribute("class")||"")}function K(t,e){for(var n=Y(t),r=-1,i=e.length;++r<i;)n.add(e[r])}function z(t,e){for(var n=Y(t),r=-1,i=e.length;++r<i;)n.remove(e[r])}function J(t){return function(){K(this,t)}}function q(t){return function(){z(this,t)}}function V(t,e){return function(){(e.apply(this,arguments)?K:z)(this,t)}}function W(){this.textContent=""}function X(t){return function(){this.textContent=t}}function Z(t){return function(){var e=t.apply(this,arguments);this.textContent=null==e?"":e}}function $(){this.innerHTML=""}function tt(t){return function(){this.innerHTML=t}}function et(t){return function(){var e=t.apply(this,arguments);this.innerHTML=null==e?"":e}}function nt(){this.nextSibling&&this.parentNode.appendChild(this)}function rt(){this.previousSibling&&this.parentNode.insertBefore(this,this.parentNode.firstChild)}function it(){return null}function ot(){var t=this.parentNode;t&&t.removeChild(this)}function at(t,e,n){var r=N(t),i=r.CustomEvent;i?i=new i(e,n):(i=r.document.createEvent("Event"),n?(i.initEvent(e,n.bubbles,n.cancelable),i.detail=n.detail):i.initEvent(e,!1,!1)),t.dispatchEvent(i)}P.prototype={add:function(t){this._names.indexOf(t)<0&&(this._names.push(t),this._node.setAttribute("class",this._names.join(" ")))},remove:function(t){var e=this._names.indexOf(t);e>=0&&(this._names.splice(e,1),this._node.setAttribute("class",this._names.join(" ")))},contains:function(t){return this._names.indexOf(t)>=0}};var st=[null];function At(t,e){this._groups=t,this._parents=e}At.prototype=function(){return new At([[document.documentElement]],st)}.prototype={constructor:At,select:function(t){"function"!=typeof t&&(t=E(t));for(var e=this._groups,n=e.length,r=new Array(n),i=0;i<n;++i)for(var o,a,s=e[i],A=s.length,l=r[i]=new Array(A),c=0;c<A;++c)(o=s[c])&&(a=t.call(o,o.__data__,c,s))&&("__data__"in o&&(a.__data__=o.__data__),l[c]=a);return new At(r,this._parents)},selectAll:function(t){var e;"function"!=typeof t&&(t=null==(e=t)?v:function(){return this.querySelectorAll(e)});for(var n=this._groups,r=n.length,i=[],o=[],a=0;a<r;++a)for(var s,A=n[a],l=A.length,c=0;c<l;++c)(s=A[c])&&(i.push(t.call(s,s.__data__,c,A)),o.push(s));return new At(i,o)},filter:function(t){"function"!=typeof t&&(t=g(t));for(var e=this._groups,n=e.length,r=new Array(n),i=0;i<n;++i)for(var o,a=e[i],s=a.length,A=r[i]=[],l=0;l<s;++l)(o=a[l])&&t.call(o,o.__data__,l,a)&&A.push(o);return new At(r,this._parents)},data:function(t,e){if(!t)return d=new Array(this.size()),c=-1,this.each(function(t){d[++c]=t}),d;var n,r=e?x:w,i=this._parents,o=this._groups;"function"!=typeof t&&(n=t,t=function(){return n});for(var a=o.length,s=new Array(a),A=new Array(a),l=new Array(a),c=0;c<a;++c){var u=i[c],g=o[c],h=g.length,d=t.call(u,u&&u.__data__,c,i),f=d.length,p=A[c]=new Array(f),I=s[c]=new Array(f);r(u,g,p,I,l[c]=new Array(h),d,e);for(var C,m,y=0,E=0;y<f;++y)if(C=p[y]){for(y>=E&&(E=y+1);!(m=I[E])&&++E<f;);C._next=m||null}}return(s=new At(s,i))._enter=A,s._exit=l,s},enter:function(){return new At(this._enter||this._groups.map(B),this._parents)},exit:function(){return new At(this._exit||this._groups.map(B),this._parents)},merge:function(t){for(var e=this._groups,n=t._groups,r=e.length,i=n.length,o=Math.min(r,i),a=new Array(r),s=0;s<o;++s)for(var A,l=e[s],c=n[s],u=l.length,g=a[s]=new Array(u),h=0;h<u;++h)(A=l[h]||c[h])&&(g[h]=A);for(;s<r;++s)a[s]=e[s];return new At(a,this._parents)},order:function(){for(var t=this._groups,e=-1,n=t.length;++e<n;)for(var r,i=t[e],o=i.length-1,a=i[o];--o>=0;)(r=i[o])&&(a&&a!==r.nextSibling&&a.parentNode.insertBefore(r,a),a=r);return this},sort:function(t){function e(e,n){return e&&n?t(e.__data__,n.__data__):!e-!n}t||(t=_);for(var n=this._groups,r=n.length,i=new Array(r),o=0;o<r;++o){for(var a,s=n[o],A=s.length,l=i[o]=new Array(A),c=0;c<A;++c)(a=s[c])&&(l[c]=a);l.sort(e)}return new At(i,this._parents).order()},call:function(){var t=arguments[0];return arguments[0]=this,t.apply(null,arguments),this},nodes:function(){var t=new Array(this.size()),e=-1;return this.each(function(){t[++e]=this}),t},node:function(){for(var t=this._groups,e=0,n=t.length;e<n;++e)for(var r=t[e],i=0,o=r.length;i<o;++i){var a=r[i];if(a)return a}return null},size:function(){var t=0;return this.each(function(){++t}),t},empty:function(){return!this.node()},each:function(t){for(var e=this._groups,n=0,r=e.length;n<r;++n)for(var i,o=e[n],a=0,s=o.length;a<s;++a)(i=o[a])&&t.call(i,i.__data__,a,o);return this},attr:function(t,e){var n=o(t);if(arguments.length<2){var r=this.node();return n.local?r.getAttributeNS(n.space,n.local):r.getAttribute(n)}return this.each((null==e?n.local?M:D:"function"==typeof e?n.local?R:L:n.local?S:k)(n,e))},style:function(t,e,n){var r;return arguments.length>1?this.each((null==e?F:"function"==typeof e?U:T)(t,e,null==n?"":n)):N(r=this.node()).getComputedStyle(r,null).getPropertyValue(t)},property:function(t,e){return arguments.length>1?this.each((null==e?O:"function"==typeof e?H:G)(t,e)):this.node()[t]},classed:function(t,e){var n=j(t+"");if(arguments.length<2){for(var r=Y(this.node()),i=-1,o=n.length;++i<o;)if(!r.contains(n[i]))return!1;return!0}return this.each(("function"==typeof e?V:e?J:q)(n,e))},text:function(t){return arguments.length?this.each(null==t?W:("function"==typeof t?Z:X)(t)):this.node().textContent},html:function(t){return arguments.length?this.each(null==t?$:("function"==typeof t?et:tt)(t)):this.node().innerHTML},raise:function(){return this.each(nt)},lower:function(){return this.each(rt)},append:function(t){var e="function"==typeof t?t:a(t);return this.select(function(){return this.appendChild(e.apply(this,arguments))})},insert:function(t,e){var n="function"==typeof t?t:a(t),r=null==e?it:"function"==typeof e?e:E(e);return this.select(function(){return this.insertBefore(n.apply(this,arguments),r.apply(this,arguments)||null)})},remove:function(){return this.each(ot)},datum:function(t){return arguments.length?this.property("__data__",t):this.node().__data__},on:function(t,e,n){var r,i,o=I(t+""),a=o.length;if(!(arguments.length<2)){for(s=e?m:C,null==n&&(n=!1),r=0;r<a;++r)this.each(s(o[r],e,n));return this}var s=this.node().__on;if(s)for(var A,l=0,c=s.length;l<c;++l)for(r=0,A=s[l];r<a;++r)if((i=o[r]).type===A.type&&i.name===A.name)return A.value},dispatch:function(t,e){return this.each(("function"==typeof e?function(t,e){return function(){return at(this,t,e.apply(this,arguments))}}:function(t,e){return function(){return at(this,t,e)}})(t,e))}};var lt,ct=function(t,e){if((n=(t=e?t.toExponential(e-1):t.toExponential()).indexOf("e"))<0)return null;var n,r=t.slice(0,n);return[r.length>1?r[0]+r.slice(2):r,+t.slice(n+1)]},ut=function(t,e){var n=ct(t,e);if(!n)return t+"";var r=n[0],i=n[1];return i<0?"0."+new Array(-i).join("0")+r:r.length>i+1?r.slice(0,i+1)+"."+r.slice(i+1):r+new Array(i-r.length+2).join("0")},gt={"":function(t,e){t:for(var n,r=(t=t.toPrecision(e)).length,i=1,o=-1;i<r;++i)switch(t[i]){case".":o=n=i;break;case"0":0===o&&(o=i),n=i;break;case"e":break t;default:o>0&&(o=0)}return o>0?t.slice(0,o)+t.slice(n+1):t},"%":function(t,e){return(100*t).toFixed(e)},b:function(t){return Math.round(t).toString(2)},c:function(t){return t+""},d:function(t){return Math.round(t).toString(10)},e:function(t,e){return t.toExponential(e)},f:function(t,e){return t.toFixed(e)},g:function(t,e){return t.toPrecision(e)},o:function(t){return Math.round(t).toString(8)},p:function(t,e){return ut(100*t,e)},r:ut,s:function(t,e){var n=ct(t,e);if(!n)return t+"";var r=n[0],i=n[1],o=i-(lt=3*Math.max(-8,Math.min(8,Math.floor(i/3))))+1,a=r.length;return o===a?r:o>a?r+new Array(o-a+1).join("0"):o>0?r.slice(0,o)+"."+r.slice(o):"0."+new Array(1-o).join("0")+ct(t,Math.max(0,e+o-1))[0]},X:function(t){return Math.round(t).toString(16).toUpperCase()},x:function(t){return Math.round(t).toString(16)}},ht=/^(?:(.)?([<>=^]))?([+\-\( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?([a-z%])?$/i,dt=function(t){return new ft(t)};function ft(t){if(!(e=ht.exec(t)))throw new Error("invalid format: "+t);var e,n=e[1]||" ",r=e[2]||">",i=e[3]||"-",o=e[4]||"",a=!!e[5],s=e[6]&&+e[6],A=!!e[7],l=e[8]&&+e[8].slice(1),c=e[9]||"";"n"===c?(A=!0,c="g"):gt[c]||(c=""),(a||"0"===n&&"="===r)&&(a=!0,n="0",r="="),this.fill=n,this.align=r,this.sign=i,this.symbol=o,this.zero=a,this.width=s,this.comma=A,this.precision=l,this.type=c}ft.prototype.toString=function(){return this.fill+this.align+this.sign+this.symbol+(this.zero?"0":"")+(null==this.width?"":Math.max(1,0|this.width))+(this.comma?",":"")+(null==this.precision?"":"."+Math.max(0,0|this.precision))+this.type};var pt=["y","z","a","f","p","n","µ","m","","k","M","G","T","P","E","Z","Y"];function It(t){return t}var Ct,mt,yt,Et=function(t){var e,n,r=t.grouping&&t.thousands?(e=t.grouping,n=t.thousands,function(t,r){for(var i=t.length,o=[],a=0,s=e[0],A=0;i>0&&s>0&&(A+s+1>r&&(s=Math.max(1,r-A)),o.push(t.substring(i-=s,i+s)),!((A+=s+1)>r));)s=e[a=(a+1)%e.length];return o.reverse().join(n)}):It,i=t.currency,o=t.decimal;function a(t){var e=(t=dt(t)).fill,n=t.align,a=t.sign,s=t.symbol,A=t.zero,l=t.width,c=t.comma,u=t.precision,g=t.type,h="$"===s?i[0]:"#"===s&&/[boxX]/.test(g)?"0"+g.toLowerCase():"",d="$"===s?i[1]:/[%p]/.test(g)?"%":"",f=gt[g],p=!g||/[defgprs%]/.test(g);function I(t){var i,s,I,C=h,m=d;if("c"===g)m=f(t)+m,t="";else{var y=((t=+t)<0||1/t<0)&&(t*=-1,!0);if(t=f(t,u),y)for(i=-1,s=t.length,y=!1;++i<s;)if(48<(I=t.charCodeAt(i))&&I<58||"x"===g&&96<I&&I<103||"X"===g&&64<I&&I<71){y=!0;break}if(C=(y?"("===a?a:"-":"-"===a||"("===a?"":a)+C,m=m+("s"===g?pt[8+lt/3]:"")+(y&&"("===a?")":""),p)for(i=-1,s=t.length;++i<s;)if(48>(I=t.charCodeAt(i))||I>57){m=(46===I?o+t.slice(i+1):t.slice(i))+m,t=t.slice(0,i);break}}c&&!A&&(t=r(t,1/0));var E=C.length+t.length+m.length,v=E<l?new Array(l-E+1).join(e):"";switch(c&&A&&(t=r(v+t,v.length?l-m.length:1/0),v=""),n){case"<":return C+t+m+v;case"=":return C+v+t+m;case"^":return v.slice(0,E=v.length>>1)+C+t+m+v.slice(E)}return v+C+t+m}return u=null==u?g?6:12:/[gprs]/.test(g)?Math.max(1,Math.min(21,u)):Math.max(0,Math.min(20,u)),I.toString=function(){return t+""},I}return{format:a,formatPrefix:function(t,e){var n=a(((t=dt(t)).type="f",t)),r=3*Math.max(-8,Math.min(8,Math.floor(function(t){return(t=ct(Math.abs(t)))?t[1]:NaN}(e)/3))),i=Math.pow(10,-r),o=pt[8+r/3];return function(t){return n(i*t)+o}}}};mt=(Ct=Et({decimal:".",thousands:",",grouping:[3],currency:["$",""]})).format,yt=Ct.formatPrefix;var vt={value:function(){}};function Bt(){for(var t,e=0,n=arguments.length,r={};e<n;++e){if(!(t=arguments[e]+"")||t in r)throw new Error("illegal type: "+t);r[t]=[]}return new bt(r)}function bt(t){this._=t}function Qt(t,e){return t.trim().split(/^|\s+/).map(function(t){var n="",r=t.indexOf(".");if(r>=0&&(n=t.slice(r+1),t=t.slice(0,r)),t&&!e.hasOwnProperty(t))throw new Error("unknown type: "+t);return{type:t,name:n}})}function wt(t,e){for(var n,r=0,i=t.length;r<i;++r)if((n=t[r]).name===e)return n.value}function xt(t,e,n){for(var r=0,i=t.length;r<i;++r)if(t[r].name===e){t[r]=vt,t=t.slice(0,r).concat(t.slice(r+1));break}return null!=n&&t.push({name:e,value:n}),t}bt.prototype=Bt.prototype={constructor:bt,on:function(t,e){var n,r=this._,i=Qt(t+"",r),o=-1,a=i.length;if(!(arguments.length<2)){if(null!=e&&"function"!=typeof e)throw new Error("invalid callback: "+e);for(;++o<a;)if(n=(t=i[o]).type)r[n]=xt(r[n],t.name,e);else if(null==e)for(n in r)r[n]=xt(r[n],t.name,null);return this}for(;++o<a;)if((n=(t=i[o]).type)&&(n=wt(r[n],t.name)))return n},copy:function(){var t={},e=this._;for(var n in e)t[n]=e[n].slice();return new bt(t)},call:function(t,e){if((n=arguments.length-2)>0)for(var n,r,i=new Array(n),o=0;o<n;++o)i[o]=arguments[o+2];if(!this._.hasOwnProperty(t))throw new Error("unknown type: "+t);for(o=0,n=(r=this._[t]).length;o<n;++o)r[o].value.apply(e,i)},apply:function(t,e,n){if(!this._.hasOwnProperty(t))throw new Error("unknown type: "+t);for(var r=this._[t],i=0,o=r.length;i<o;++i)r[i].value.apply(e,n)}};var _t=Bt,Dt=function(t,e){return t<e?-1:t>e?1:t>=e?0:NaN},Mt=function(t){var e;return 1===t.length&&(e=t,t=function(t,n){return Dt(e(t),n)}),{left:function(e,n,r,i){for(null==r&&(r=0),null==i&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)<0?r=o+1:i=o}return r},right:function(e,n,r,i){for(null==r&&(r=0),null==i&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)>0?i=o:r=o+1}return r}}}(Dt),kt=Mt.right,St=(Mt.left,kt),Lt=Array.prototype,Rt=(Lt.slice,Lt.map,Math.sqrt(50)),Nt=Math.sqrt(10),Ft=Math.sqrt(2),Tt=function(t,e,n){var r,i,o,a,s=-1;if(n=+n,(t=+t)==(e=+e)&&n>0)return[t];if((r=e<t)&&(i=t,t=e,e=i),0===(a=function(t,e,n){var r=(e-t)/Math.max(0,n),i=Math.floor(Math.log(r)/Math.LN10),o=r/Math.pow(10,i);return i>=0?(o>=Rt?10:o>=Nt?5:o>=Ft?2:1)*Math.pow(10,i):-Math.pow(10,-i)/(o>=Rt?10:o>=Nt?5:o>=Ft?2:1)}(t,e,n))||!isFinite(a))return[];if(a>0)for(t=Math.ceil(t/a),e=Math.floor(e/a),o=new Array(i=Math.ceil(e-t+1));++s<i;)o[s]=(t+s)*a;else for(t=Math.floor(t*a),e=Math.ceil(e*a),o=new Array(i=Math.ceil(t-e+1));++s<i;)o[s]=(t-s)/a;return r&&o.reverse(),o};function Ut(t,e,n){var r=Math.abs(e-t)/Math.max(0,n),i=Math.pow(10,Math.floor(Math.log(r)/Math.LN10)),o=r/i;return o>=Rt?i*=10:o>=Nt?i*=5:o>=Ft&&(i*=2),e<t?-i:i}n(9);var Ot=Array.prototype,Gt=Ot.map,Ht=Ot.slice,jt=n(4),Yt=function(t){return function(){return t}},Pt=function(t){return+t},Kt=[0,1];function zt(t,e){return(e-=t=+t)?function(n){return(n-t)/e}:Yt(e)}function Jt(t,e,n,r){var i=t[0],o=t[1],a=e[0],s=e[1];return o<i?(i=n(o,i),a=r(s,a)):(i=n(i,o),a=r(a,s)),function(t){return a(i(t))}}function qt(t,e,n,r){var i=Math.min(t.length,e.length)-1,o=new Array(i),a=new Array(i),s=-1;for(t[i]<t[0]&&(t=t.slice().reverse(),e=e.slice().reverse());++s<i;)o[s]=n(t[s],t[s+1]),a[s]=r(e[s],e[s+1]);return function(e){var n=St(t,e,1,i)-1;return a[n](o[n](e))}}var Vt=function(t,e){if((n=(t=e?t.toExponential(e-1):t.toExponential()).indexOf("e"))<0)return null;var n,r=t.slice(0,n);return[r.length>1?r[0]+r.slice(2):r,+t.slice(n+1)]},Wt=function(t){return(t=Vt(Math.abs(t)))?t[1]:NaN},Xt=/^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i;function Zt(t){return new $t(t)}function $t(t){if(!(e=Xt.exec(t)))throw new Error("invalid format: "+t);var e;this.fill=e[1]||" ",this.align=e[2]||">",this.sign=e[3]||"-",this.symbol=e[4]||"",this.zero=!!e[5],this.width=e[6]&&+e[6],this.comma=!!e[7],this.precision=e[8]&&+e[8].slice(1),this.trim=!!e[9],this.type=e[10]||""}Zt.prototype=$t.prototype,$t.prototype.toString=function(){return this.fill+this.align+this.sign+this.symbol+(this.zero?"0":"")+(null==this.width?"":Math.max(1,0|this.width))+(this.comma?",":"")+(null==this.precision?"":"."+Math.max(0,0|this.precision))+(this.trim?"~":"")+this.type};var te,ee,ne,re,ie=function(t){t:for(var e,n=t.length,r=1,i=-1;r<n;++r)switch(t[r]){case".":i=e=r;break;case"0":0===i&&(i=r),e=r;break;default:if(i>0){if(!+t[r])break t;i=0}}return i>0?t.slice(0,i)+t.slice(e+1):t},oe=function(t,e){var n=Vt(t,e);if(!n)return t+"";var r=n[0],i=n[1];return i<0?"0."+new Array(-i).join("0")+r:r.length>i+1?r.slice(0,i+1)+"."+r.slice(i+1):r+new Array(i-r.length+2).join("0")},ae={"%":function(t,e){return(100*t).toFixed(e)},b:function(t){return Math.round(t).toString(2)},c:function(t){return t+""},d:function(t){return Math.round(t).toString(10)},e:function(t,e){return t.toExponential(e)},f:function(t,e){return t.toFixed(e)},g:function(t,e){return t.toPrecision(e)},o:function(t){return Math.round(t).toString(8)},p:function(t,e){return oe(100*t,e)},r:oe,s:function(t,e){var n=Vt(t,e);if(!n)return t+"";var r=n[0],i=n[1],o=i-(te=3*Math.max(-8,Math.min(8,Math.floor(i/3))))+1,a=r.length;return o===a?r:o>a?r+new Array(o-a+1).join("0"):o>0?r.slice(0,o)+"."+r.slice(o):"0."+new Array(1-o).join("0")+Vt(t,Math.max(0,e+o-1))[0]},X:function(t){return Math.round(t).toString(16).toUpperCase()},x:function(t){return Math.round(t).toString(16)}},se=function(t){return t},Ae=["y","z","a","f","p","n","µ","m","","k","M","G","T","P","E","Z","Y"];ee=function(t){var e,n,r=t.grouping&&t.thousands?(e=t.grouping,n=t.thousands,function(t,r){for(var i=t.length,o=[],a=0,s=e[0],A=0;i>0&&s>0&&(A+s+1>r&&(s=Math.max(1,r-A)),o.push(t.substring(i-=s,i+s)),!((A+=s+1)>r));)s=e[a=(a+1)%e.length];return o.reverse().join(n)}):se,i=t.currency,o=t.decimal,a=t.numerals?function(t){return function(e){return e.replace(/[0-9]/g,function(e){return t[+e]})}}(t.numerals):se,s=t.percent||"%";function A(t){var e=(t=Zt(t)).fill,n=t.align,A=t.sign,l=t.symbol,c=t.zero,u=t.width,g=t.comma,h=t.precision,d=t.trim,f=t.type;"n"===f?(g=!0,f="g"):ae[f]||(null==h&&(h=12),d=!0,f="g"),(c||"0"===e&&"="===n)&&(c=!0,e="0",n="=");var p="$"===l?i[0]:"#"===l&&/[boxX]/.test(f)?"0"+f.toLowerCase():"",I="$"===l?i[1]:/[%p]/.test(f)?s:"",C=ae[f],m=/[defgprs%]/.test(f);function y(t){var i,s,l,y=p,E=I;if("c"===f)E=C(t)+E,t="";else{var v=(t=+t)<0;if(t=C(Math.abs(t),h),d&&(t=ie(t)),v&&0==+t&&(v=!1),y=(v?"("===A?A:"-":"-"===A||"("===A?"":A)+y,E=("s"===f?Ae[8+te/3]:"")+E+(v&&"("===A?")":""),m)for(i=-1,s=t.length;++i<s;)if(48>(l=t.charCodeAt(i))||l>57){E=(46===l?o+t.slice(i+1):t.slice(i))+E,t=t.slice(0,i);break}}g&&!c&&(t=r(t,1/0));var B=y.length+t.length+E.length,b=B<u?new Array(u-B+1).join(e):"";switch(g&&c&&(t=r(b+t,b.length?u-E.length:1/0),b=""),n){case"<":t=y+t+E+b;break;case"=":t=y+b+t+E;break;case"^":t=b.slice(0,B=b.length>>1)+y+t+E+b.slice(B);break;default:t=b+y+t+E}return a(t)}return h=null==h?6:/[gprs]/.test(f)?Math.max(1,Math.min(21,h)):Math.max(0,Math.min(20,h)),y.toString=function(){return t+""},y}return{format:A,formatPrefix:function(t,e){var n=A(((t=Zt(t)).type="f",t)),r=3*Math.max(-8,Math.min(8,Math.floor(Wt(e)/3))),i=Math.pow(10,-r),o=Ae[8+r/3];return function(t){return n(i*t)+o}}}}({decimal:".",thousands:",",grouping:[3],currency:["$",""]}),ne=ee.format,re=ee.formatPrefix;var le=function(t,e,n){var r,i=t[0],o=t[t.length-1],a=Ut(i,o,null==e?10:e);switch((n=Zt(null==n?",f":n)).type){case"s":var s=Math.max(Math.abs(i),Math.abs(o));return null!=n.precision||isNaN(r=function(t,e){return Math.max(0,3*Math.max(-8,Math.min(8,Math.floor(Wt(e)/3)))-Wt(Math.abs(t)))}(a,s))||(n.precision=r),re(n,s);case"":case"e":case"g":case"p":case"r":null!=n.precision||isNaN(r=function(t,e){return t=Math.abs(t),e=Math.abs(e)-t,Math.max(0,Wt(e)-Wt(t))+1}(a,Math.max(Math.abs(i),Math.abs(o))))||(n.precision=r-("e"===n.type));break;case"f":case"%":null!=n.precision||isNaN(r=function(t){return Math.max(0,-Wt(Math.abs(t)))}(a))||(n.precision=r-2*("%"===n.type))}return ne(n)};function ce(){var t=function(t,e){var n,r,i,o=Kt,a=Kt,s=jt.a,A=!1;function l(){return n=Math.min(o.length,a.length)>2?qt:Jt,r=i=null,c}function c(e){return(r||(r=n(o,a,A?function(t){return function(e,n){var r=t(e=+e,n=+n);return function(t){return t<=e?0:t>=n?1:r(t)}}}(t):t,s)))(+e)}return c.invert=function(t){return(i||(i=n(a,o,zt,A?function(t){return function(e,n){var r=t(e=+e,n=+n);return function(t){return t<=0?e:t>=1?n:r(t)}}}(e):e)))(+t)},c.domain=function(t){return arguments.length?(o=Gt.call(t,Pt),l()):o.slice()},c.range=function(t){return arguments.length?(a=Ht.call(t),l()):a.slice()},c.rangeRound=function(t){return a=Ht.call(t),s=jt.t,l()},c.clamp=function(t){return arguments.length?(A=!!t,l()):A},c.interpolate=function(t){return arguments.length?(s=t,l()):s},l()}(zt,jt.o);return t.copy=function(){return function(t,e){return e.domain(t.domain()).range(t.range()).interpolate(t.interpolate()).clamp(t.clamp())}(t,ce())},function(t){var e=t.domain;return t.ticks=function(t){var n=e();return Tt(n[0],n[n.length-1],null==t?10:t)},t.tickFormat=function(t,n){return le(e(),t,n)},t.nice=function(n){var r=e(),i=r.length-1,o=null==n?10:n,a=r[0],s=r[i],A=Ut(a,s,o);return A&&(A=Ut(Math.floor(a/A)*A,Math.ceil(s/A)*A,o),r[0]=Math.floor(a/A)*A,r[i]=Math.ceil(s/A)*A,e(r)),t},t}(t)}n(1),n(10);var ue=function(t){return t.match(/.{6}/g).map(function(t){return"#"+t})},ge=(ue("1f77b4ff7f0e2ca02cd627289467bd8c564be377c27f7f7fbcbd2217becf"),ue("393b795254a36b6ecf9c9ede6379398ca252b5cf6bcedb9c8c6d31bd9e39e7ba52e7cb94843c39ad494ad6616be7969c7b4173a55194ce6dbdde9ed6"),ue("3182bd6baed69ecae1c6dbefe6550dfd8d3cfdae6bfdd0a231a35474c476a1d99bc7e9c0756bb19e9ac8bcbddcdadaeb636363969696bdbdbdd9d9d9"),ue("1f77b4aec7e8ff7f0effbb782ca02c98df8ad62728ff98969467bdc5b0d58c564bc49c94e377c2f7b6d27f7f7fc7c7c7bcbd22dbdb8d17becf9edae5"),n(3));function he(t){var e=t.length;return function(n){return t[Math.max(0,Math.min(e-1,Math.floor(n*e)))]}}Object(jt.f)(Object(ge.b)(300,.5,0),Object(ge.b)(-240,.5,1)),Object(jt.f)(Object(ge.b)(-100,.75,.35),Object(ge.b)(80,1.5,.8)),Object(jt.f)(Object(ge.b)(260,.75,.35),Object(ge.b)(80,1.5,.8)),Object(ge.b)(),he(ue("44015444025645045745055946075a46085c460a5d460b5e470d60470e6147106347116447136548146748166848176948186a481a6c481b6d481c6e481d6f481f70482071482173482374482475482576482677482878482979472a7a472c7a472d7b472e7c472f7d46307e46327e46337f463480453581453781453882443983443a83443b84433d84433e85423f854240864241864142874144874045884046883f47883f48893e49893e4a893e4c8a3d4d8a3d4e8a3c4f8a3c508b3b518b3b528b3a538b3a548c39558c39568c38588c38598c375a8c375b8d365c8d365d8d355e8d355f8d34608d34618d33628d33638d32648e32658e31668e31678e31688e30698e306a8e2f6b8e2f6c8e2e6d8e2e6e8e2e6f8e2d708e2d718e2c718e2c728e2c738e2b748e2b758e2a768e2a778e2a788e29798e297a8e297b8e287c8e287d8e277e8e277f8e27808e26818e26828e26828e25838e25848e25858e24868e24878e23888e23898e238a8d228b8d228c8d228d8d218e8d218f8d21908d21918c20928c20928c20938c1f948c1f958b1f968b1f978b1f988b1f998a1f9a8a1e9b8a1e9c891e9d891f9e891f9f881fa0881fa1881fa1871fa28720a38620a48621a58521a68522a78522a88423a98324aa8325ab8225ac8226ad8127ad8128ae8029af7f2ab07f2cb17e2db27d2eb37c2fb47c31b57b32b67a34b67935b77937b87838b9773aba763bbb753dbc743fbc7340bd7242be7144bf7046c06f48c16e4ac16d4cc26c4ec36b50c46a52c56954c56856c66758c7655ac8645cc8635ec96260ca6063cb5f65cb5e67cc5c69cd5b6ccd5a6ece5870cf5773d05675d05477d1537ad1517cd2507fd34e81d34d84d44b86d54989d5488bd6468ed64590d74393d74195d84098d83e9bd93c9dd93ba0da39a2da37a5db36a8db34aadc32addc30b0dd2fb2dd2db5de2bb8de29bade28bddf26c0df25c2df23c5e021c8e020cae11fcde11dd0e11cd2e21bd5e21ad8e219dae319dde318dfe318e2e418e5e419e7e419eae51aece51befe51cf1e51df4e61ef6e620f8e621fbe723fde725")),he(ue("00000401000501010601010802010902020b02020d03030f03031204041405041606051806051a07061c08071e0907200a08220b09240c09260d0a290e0b2b100b2d110c2f120d31130d34140e36150e38160f3b180f3d19103f1a10421c10441d11471e114920114b21114e22115024125325125527125829115a2a115c2c115f2d11612f116331116533106734106936106b38106c390f6e3b0f703d0f713f0f72400f74420f75440f764510774710784910784a10794c117a4e117b4f127b51127c52137c54137d56147d57157e59157e5a167e5c167f5d177f5f187f601880621980641a80651a80671b80681c816a1c816b1d816d1d816e1e81701f81721f817320817521817621817822817922827b23827c23827e24828025828125818326818426818627818827818928818b29818c29818e2a81902a81912b81932b80942c80962c80982d80992d809b2e7f9c2e7f9e2f7fa02f7fa1307ea3307ea5317ea6317da8327daa337dab337cad347cae347bb0357bb2357bb3367ab5367ab73779b83779ba3878bc3978bd3977bf3a77c03a76c23b75c43c75c53c74c73d73c83e73ca3e72cc3f71cd4071cf4070d0416fd2426fd3436ed5446dd6456cd8456cd9466bdb476adc4869de4968df4a68e04c67e24d66e34e65e44f64e55064e75263e85362e95462ea5661eb5760ec5860ed5a5fee5b5eef5d5ef05f5ef1605df2625df2645cf3655cf4675cf4695cf56b5cf66c5cf66e5cf7705cf7725cf8745cf8765cf9785df9795df97b5dfa7d5efa7f5efa815ffb835ffb8560fb8761fc8961fc8a62fc8c63fc8e64fc9065fd9266fd9467fd9668fd9869fd9a6afd9b6bfe9d6cfe9f6dfea16efea36ffea571fea772fea973feaa74feac76feae77feb078feb27afeb47bfeb67cfeb77efeb97ffebb81febd82febf84fec185fec287fec488fec68afec88cfeca8dfecc8ffecd90fecf92fed194fed395fed597fed799fed89afdda9cfddc9efddea0fde0a1fde2a3fde3a5fde5a7fde7a9fde9aafdebacfcecaefceeb0fcf0b2fcf2b4fcf4b6fcf6b8fcf7b9fcf9bbfcfbbdfcfdbf")),he(ue("00000401000501010601010802010a02020c02020e03021004031204031405041706041907051b08051d09061f0a07220b07240c08260d08290e092b10092d110a30120a32140b34150b37160b39180c3c190c3e1b0c411c0c431e0c451f0c48210c4a230c4c240c4f260c51280b53290b552b0b572d0b592f0a5b310a5c320a5e340a5f3609613809623909633b09643d09653e0966400a67420a68440a68450a69470b6a490b6a4a0c6b4c0c6b4d0d6c4f0d6c510e6c520e6d540f6d550f6d57106e59106e5a116e5c126e5d126e5f136e61136e62146e64156e65156e67166e69166e6a176e6c186e6d186e6f196e71196e721a6e741a6e751b6e771c6d781c6d7a1d6d7c1d6d7d1e6d7f1e6c801f6c82206c84206b85216b87216b88226a8a226a8c23698d23698f24699025689225689326679526679727669827669a28659b29649d29649f2a63a02a63a22b62a32c61a52c60a62d60a82e5fa92e5eab2f5ead305dae305cb0315bb1325ab3325ab43359b63458b73557b93556ba3655bc3754bd3853bf3952c03a51c13a50c33b4fc43c4ec63d4dc73e4cc83f4bca404acb4149cc4248ce4347cf4446d04545d24644d34743d44842d54a41d74b3fd84c3ed94d3dda4e3cdb503bdd513ade5238df5337e05536e15635e25734e35933e45a31e55c30e65d2fe75e2ee8602de9612bea632aeb6429eb6628ec6726ed6925ee6a24ef6c23ef6e21f06f20f1711ff1731df2741cf3761bf37819f47918f57b17f57d15f67e14f68013f78212f78410f8850ff8870ef8890cf98b0bf98c0af98e09fa9008fa9207fa9407fb9606fb9706fb9906fb9b06fb9d07fc9f07fca108fca309fca50afca60cfca80dfcaa0ffcac11fcae12fcb014fcb216fcb418fbb61afbb81dfbba1ffbbc21fbbe23fac026fac228fac42afac62df9c72ff9c932f9cb35f8cd37f8cf3af7d13df7d340f6d543f6d746f5d949f5db4cf4dd4ff4df53f4e156f3e35af3e55df2e661f2e865f2ea69f1ec6df1ed71f1ef75f1f179f2f27df2f482f3f586f3f68af4f88ef5f992f6fa96f8fb9af9fc9dfafda1fcffa4")),he(ue("0d088710078813078916078a19068c1b068d1d068e20068f2206902406912605912805922a05932c05942e05952f059631059733059735049837049938049a3a049a3c049b3e049c3f049c41049d43039e44039e46039f48039f4903a04b03a14c02a14e02a25002a25102a35302a35502a45601a45801a45901a55b01a55c01a65e01a66001a66100a76300a76400a76600a76700a86900a86a00a86c00a86e00a86f00a87100a87201a87401a87501a87701a87801a87a02a87b02a87d03a87e03a88004a88104a78305a78405a78606a68707a68808a68a09a58b0aa58d0ba58e0ca48f0da4910ea3920fa39410a29511a19613a19814a099159f9a169f9c179e9d189d9e199da01a9ca11b9ba21d9aa31e9aa51f99a62098a72197a82296aa2395ab2494ac2694ad2793ae2892b02991b12a90b22b8fb32c8eb42e8db52f8cb6308bb7318ab83289ba3388bb3488bc3587bd3786be3885bf3984c03a83c13b82c23c81c33d80c43e7fc5407ec6417dc7427cc8437bc9447aca457acb4679cc4778cc4977cd4a76ce4b75cf4c74d04d73d14e72d24f71d35171d45270d5536fd5546ed6556dd7566cd8576bd9586ada5a6ada5b69db5c68dc5d67dd5e66de5f65de6164df6263e06363e16462e26561e26660e3685fe4695ee56a5de56b5de66c5ce76e5be76f5ae87059e97158e97257ea7457eb7556eb7655ec7754ed7953ed7a52ee7b51ef7c51ef7e50f07f4ff0804ef1814df1834cf2844bf3854bf3874af48849f48948f58b47f58c46f68d45f68f44f79044f79143f79342f89441f89540f9973ff9983ef99a3efa9b3dfa9c3cfa9e3bfb9f3afba139fba238fca338fca537fca636fca835fca934fdab33fdac33fdae32fdaf31fdb130fdb22ffdb42ffdb52efeb72dfeb82cfeba2cfebb2bfebd2afebe2afec029fdc229fdc328fdc527fdc627fdc827fdca26fdcb26fccd25fcce25fcd025fcd225fbd324fbd524fbd724fad824fada24f9dc24f9dd25f8df25f8e125f7e225f7e425f6e626f6e826f5e926f5eb27f4ed27f3ee27f3f027f2f227f1f426f1f525f0f724f0f921"));var de=function(t,e){return t<e?-1:t>e?1:t>=e?0:NaN};1===(fe=de).length&&(pe=fe,fe=function(t,e){return de(pe(t),e)});var fe,pe,Ie=Array.prototype;Ie.slice,Ie.map,Math.sqrt(50),Math.sqrt(10),Math.sqrt(2);var Ce=function(t,e){var n,r,i=-1,o=t.length;if(null==e){for(;++i<o;)if(null!=(r=t[i])&&r>=r){n=r;break}for(;++i<o;)null!=(r=t[i])&&r>n&&(n=r)}else{for(;++i<o;)if(null!=(r=e(t[i],i,t))&&r>=r){n=r;break}for(;++i<o;)null!=(r=e(t[i],i,t))&&r>n&&(n=r)}return n},me=function(t,e){var n,r=0,i=t.length,o=-1;if(null==e)for(;++o<i;)(n=+t[o])&&(r+=n);else for(;++o<i;)(n=+e(t[o],o,t))&&(r+=n);return r};n.d(e,"legendColor",function(){return Qe}),n.d(e,"legendSize",function(){return we}),n.d(e,"legendSymbol",function(){return xe}),n.d(e,"legendHelpers",function(){return _e});var ye="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Ee=function(t){return t},ve=function(t){for(var e=[],n=0,r=t.length;n<r;n++)e[n]=t[r-n-1];return e},Be=function(t,e){t.each(function(){for(var t,n="string"==typeof this?new At([[document.querySelector(this)]],[document.documentElement]):new At([[this]],st),r=n.text().split(/\s+/).reverse(),i=[],o=(n.attr("y"),parseFloat(n.attr("dy"))||0),a=n.text(null).append("tspan").attr("x",0).attr("dy",o+"em");t=r.pop();)i.push(t),a.text(i.join(" ")),a.node().getComputedTextLength()>e&&i.length>1&&(i.pop(),a.text(i.join(" ")),i=[t],a=n.append("tspan").attr("x",0).attr("dy",1.2+o+"em").text(t))})},be={d3_drawShapes:function(t,e,n,r,i,o){"rect"===t?e.attr("height",n).attr("width",r):"circle"===t?e.attr("r",i):"line"===t?e.attr("x1",0).attr("x2",r).attr("y1",0).attr("y2",0):"path"===t&&e.attr("d",o)},d3_addText:function(t,e,n,r,i){e.append("text").attr("class",r+"label");var o=t.selectAll("g."+r+"cell text."+r+"label").data(n).text(Ee);return i&&t.selectAll("g."+r+"cell text."+r+"label").call(Be,i),o},d3_calcType:function(t,e,n,r,i,o){var a=t.invertExtent?function(t,e,n){var r=t.range().map(function(r){var i=t.invertExtent(r);return e(i[0])+" "+n+" "+e(i[1])});return{data:t.range(),labels:r,feature:Ee}}(t,i,o):t.ticks?function(t,e,n){var r=[];if(e.length>1)r=e;else for(var i=t.domain(),o=(i[i.length-1]-i[0])/(e-1),a=0;a<e;a++)r.push(i[0]+a*o);var s=r.map(n);return{data:r,labels:s,feature:function(e){return t(e)}}}(t,n,i):function(t){return{data:t.domain(),labels:t.domain(),feature:function(e){return t(e)}}}(t),s=t.range&&t.range()||t.domain();return a.labels=function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:[],e=arguments[1],n=arguments[2],r=arguments[3],i=arguments[4];if("object"===(void 0===e?"undefined":ye(e))){if(0===e.length)return t;for(var o=e.length;o<t.length;o++)e.push(t[o]);return e}if("function"==typeof e){for(var a=[],s=t.length,A=0;A<s;A++)a.push(e({i:A,genLength:s,generatedLabels:t,domain:n,range:r,labelDelimiter:i}));return a}return t}(a.labels,r,t.domain(),s,o),e&&(a.labels=ve(a.labels),a.data=ve(a.data)),a},d3_filterCells:function(t,e){var n=t.data.map(function(e,n){return{data:e,label:t.labels[n]}}).filter(e),r=n.map(function(t){return t.data}),i=n.map(function(t){return t.label});return t.data=t.data.filter(function(t){return-1!==r.indexOf(t)}),t.labels=t.labels.filter(function(t){return-1!==i.indexOf(t)}),t},d3_placement:function(t,e,n,r,i,o){e.attr("transform",n),r.attr("transform",i),"horizontal"===t&&r.style("text-anchor",o)},d3_addEvents:function(t,e){t.on("mouseover.legend",function(t){!function(t,e,n){t.call("cellover",n,e)}(e,t,this)}).on("mouseout.legend",function(t){!function(t,e,n){t.call("cellout",n,e)}(e,t,this)}).on("click.legend",function(t){!function(t,e,n){t.call("cellclick",n,e)}(e,t,this)})},d3_title:function(t,e,n,r){if(""!==e){t.selectAll("text."+n+"legendTitle").data([e]).enter().append("text").attr("class",n+"legendTitle"),t.selectAll("text."+n+"legendTitle").text(e),r&&t.selectAll("text."+n+"legendTitle").call(Be,r);var i=t.select("."+n+"legendCells"),o=t.select("."+n+"legendTitle").nodes().map(function(t){return t.getBBox().height})[0],a=-i.nodes().map(function(t){return t.getBBox().x})[0];i.attr("transform","translate("+a+","+o+")")}},d3_defaultLocale:{format:mt,formatPrefix:yt},d3_defaultFormatSpecifier:".01f",d3_defaultDelimiter:"to"};function Qe(){var t=ce(),e="rect",n=15,r=15,i=10,o=2,a=[5],s=void 0,A=[],l="",c=!1,u="",g=be.d3_defaultLocale,h=be.d3_defaultFormatSpecifier,d=10,f="middle",p=be.d3_defaultDelimiter,I=void 0,C="vertical",m=!1,y=void 0,E=void 0,v=_t("cellover","cellout","cellclick");function B(B){var b=be.d3_calcType(t,m,a,A,g.format(h),p);B.selectAll("g").data([t]).enter().append("g").attr("class",l+"legendCells"),s&&be.d3_filterCells(b,s);var Q=B.select("."+l+"legendCells").selectAll("."+l+"cell").data(b.data),w=Q.enter().append("g").attr("class",l+"cell");w.append(e).attr("class",l+"swatch");var x=B.selectAll("g."+l+"cell "+e+"."+l+"swatch").data(b.data);be.d3_addEvents(w,v),Q.exit().transition().style("opacity",0).remove(),x.exit().transition().style("opacity",0).remove(),x=x.merge(x),be.d3_drawShapes(e,x,r,n,i,y);var _=be.d3_addText(B,w,b.labels,l,I);Q=w.merge(Q);var D=_.nodes().map(function(t){return t.getBBox()}),M=x.nodes().map(function(t){return t.getBBox()});c?x.attr("class",function(t){return l+"swatch "+b.feature(t)}):"line"==e?x.style("stroke",b.feature):x.style("fill",b.feature);var k,S=void 0,L=void 0,R="start"==f?0:"middle"==f?.5:1;"vertical"===C?(k=D.map(function(t,e){return Math.max(t.height,M[e].height)}),S=function(t,e){return"translate(0, "+(me(k.slice(0,e))+e*o)+")"},L=function(t,e){return"translate( "+(M[e].width+M[e].x+d)+", "+(M[e].y+M[e].height/2+5)+")"}):"horizontal"===C&&(S=function(t,e){return"translate("+e*(M[e].width+o)+",0)"},L=function(t,e){return"translate("+(M[e].width*R+M[e].x)+",\n          "+(M[e].height+M[e].y+d+8)+")"}),be.d3_placement(C,Q,S,_,L,f),be.d3_title(B,u,l,E),Q.transition().style("opacity",1)}return B.scale=function(e){return arguments.length?(t=e,B):t},B.cells=function(t){return arguments.length?((t.length>1||t>=2)&&(a=t),B):a},B.cellFilter=function(t){return arguments.length?(s=t,B):s},B.shape=function(t,n){return arguments.length?(("rect"==t||"circle"==t||"line"==t||"path"==t&&"string"==typeof n)&&(e=t,y=n),B):e},B.shapeWidth=function(t){return arguments.length?(n=+t,B):n},B.shapeHeight=function(t){return arguments.length?(r=+t,B):r},B.shapeRadius=function(t){return arguments.length?(i=+t,B):i},B.shapePadding=function(t){return arguments.length?(o=+t,B):o},B.labels=function(t){return arguments.length?(A=t,B):A},B.labelAlign=function(t){return arguments.length?("start"!=t&&"end"!=t&&"middle"!=t||(f=t),B):f},B.locale=function(t){return arguments.length?(g=Et(t),B):g},B.labelFormat=function(t){return arguments.length?(h=dt(t),B):B.locale().format(h)},B.labelOffset=function(t){return arguments.length?(d=+t,B):d},B.labelDelimiter=function(t){return arguments.length?(p=t,B):p},B.labelWrap=function(t){return arguments.length?(I=t,B):I},B.useClass=function(t){return arguments.length?(!0!==t&&!1!==t||(c=t),B):c},B.orient=function(t){return arguments.length?("horizontal"!=(t=t.toLowerCase())&&"vertical"!=t||(C=t),B):C},B.ascending=function(t){return arguments.length?(m=!!t,B):m},B.classPrefix=function(t){return arguments.length?(l=t,B):l},B.title=function(t){return arguments.length?(u=t,B):u},B.titleWidth=function(t){return arguments.length?(E=t,B):E},B.textWrap=function(t){return arguments.length?(textWrap=t,B):textWrap},B.on=function(){var t=v.on.apply(v,arguments);return t===v?B:t},B}function we(){var t=ce(),e="rect",n=15,r=2,i=[5],o=void 0,a=[],s="",A="",l=be.d3_defaultLocale,c=be.d3_defaultFormatSpecifier,u=10,g="middle",h=be.d3_defaultDelimiter,d=void 0,f="vertical",p=!1,I=void 0,C=void 0,m=_t("cellover","cellout","cellclick");function y(y){var E=be.d3_calcType(t,p,i,a,l.format(c),h),v=y.selectAll("g").data([t]);o&&be.d3_filterCells(E,o),v.enter().append("g").attr("class",s+"legendCells");var B=y.select("."+s+"legendCells").selectAll("."+s+"cell").data(E.data),b=B.enter().append("g").attr("class",s+"cell");b.append(e).attr("class",s+"swatch");var Q=y.selectAll("g."+s+"cell "+e+"."+s+"swatch");be.d3_addEvents(b,m),B.exit().transition().style("opacity",0).remove(),Q.exit().transition().style("opacity",0).remove(),Q=Q.merge(Q),"line"===e?(be.d3_drawShapes(e,Q,0,n),Q.attr("stroke-width",E.feature)):be.d3_drawShapes(e,Q,E.feature,E.feature,E.feature,I);var w=be.d3_addText(y,b,E.labels,s,d);B=b.merge(B);var x,_,D=w.nodes().map(function(t){return t.getBBox()}),M=Q.nodes().map(function(n,r){var i=n.getBBox(),o=t(E.data[r]);return"line"===e&&"horizontal"===f?i.height=i.height+o:"line"===e&&"vertical"===f&&(i.width=i.width),i}),k=Ce(M,function(t){return t.height+t.y}),S=Ce(M,function(t){return t.width+t.x}),L=void 0,R=void 0,N="start"==g?0:"middle"==g?.5:1;"vertical"===f?(x=D.map(function(t,e){return Math.max(t.height,M[e].height)}),_="circle"==e||"line"==e?M[0].height/2:0,L=function(t,e){var n=me(x.slice(0,e));return"translate(0, "+(_+n+e*r)+")"},R=function(t,e){return"translate( "+(S+u)+",\n          "+(M[e].y+M[e].height/2+5)+")"}):"horizontal"===f&&function(){L=function(t,n){return"translate("+(me(M.slice(0,n),function(t){return t.width})+n*r)+", "+("circle"==e||"line"==e?k/2:0)+")"};var t="line"==e?k/2:k;R=function(e,n){return"translate( "+(M[n].width*N+M[n].x)+",\n              "+(t+u)+")"}}(),be.d3_placement(f,B,L,w,R,g),be.d3_title(y,A,s,C),B.transition().style("opacity",1)}return y.scale=function(e){return arguments.length?(t=e,y):t},y.cells=function(t){return arguments.length?((t.length>1||t>=2)&&(i=t),y):i},y.cellFilter=function(t){return arguments.length?(o=t,y):o},y.shape=function(t,n){return arguments.length?("rect"!=t&&"circle"!=t&&"line"!=t||(e=t,I=n),y):e},y.shapeWidth=function(t){return arguments.length?(n=+t,y):n},y.shapePadding=function(t){return arguments.length?(r=+t,y):r},y.labels=function(t){return arguments.length?(a=t,y):a},y.labelAlign=function(t){return arguments.length?("start"!=t&&"end"!=t&&"middle"!=t||(g=t),y):g},y.locale=function(t){return arguments.length?(l=Et(t),y):l},y.labelFormat=function(t){return arguments.length?(c=dt(t),y):y.locale().format(c)},y.labelOffset=function(t){return arguments.length?(u=+t,y):u},y.labelDelimiter=function(t){return arguments.length?(h=t,y):h},y.labelWrap=function(t){return arguments.length?(d=t,y):d},y.orient=function(t){return arguments.length?("horizontal"!=(t=t.toLowerCase())&&"vertical"!=t||(f=t),y):f},y.ascending=function(t){return arguments.length?(p=!!t,y):p},y.classPrefix=function(t){return arguments.length?(s=t,y):s},y.title=function(t){return arguments.length?(A=t,y):A},y.titleWidth=function(t){return arguments.length?(C=t,y):C},y.on=function(){var t=m.on.apply(m,arguments);return t===m?y:t},y}function xe(){var t=ce(),e="path",n=15,r=15,i=10,o=5,a=[5],s=void 0,A=[],l="",c="",u=be.d3_defaultLocale,g=be.d3_defaultFormatSpecifier,h="middle",d=10,f=be.d3_defaultDelimiter,p=void 0,I="vertical",C=!1,m=void 0,y=_t("cellover","cellout","cellclick");function E(E){var v=be.d3_calcType(t,C,a,A,u.format(g),f),B=E.selectAll("g").data([t]);s&&be.d3_filterCells(v,s),B.enter().append("g").attr("class",l+"legendCells");var b=E.select("."+l+"legendCells").selectAll("."+l+"cell").data(v.data),Q=b.enter().append("g").attr("class",l+"cell");Q.append(e).attr("class",l+"swatch");var w=E.selectAll("g."+l+"cell "+e+"."+l+"swatch");be.d3_addEvents(Q,y),b.exit().transition().style("opacity",0).remove(),w.exit().transition().style("opacity",0).remove(),w=w.merge(w),be.d3_drawShapes(e,w,r,n,i,v.feature);var x=be.d3_addText(E,Q,v.labels,l,p);b=Q.merge(b);var _,D=x.nodes().map(function(t){return t.getBBox()}),M=w.nodes().map(function(t){return t.getBBox()}),k=Ce(M,function(t){return t.height}),S=Ce(M,function(t){return t.width}),L=void 0,R=void 0,N="start"==h?0:"middle"==h?.5:1;"vertical"===I?(_=D.map(function(t,e){return Math.max(k,t.height)}),L=function(t,e){return"translate(0, "+(me(_.slice(0,e))+e*o)+" )"},R=function(t,e){return"translate( "+(S+d)+",\n              "+(M[e].y+M[e].height/2+5)+")"}):"horizontal"===I&&(L=function(t,e){return"translate( "+e*(S+o)+",0)"},R=function(t,e){return"translate( "+(M[e].width*N+M[e].x)+",\n              "+(k+d)+")"}),be.d3_placement(I,b,L,x,R,h),be.d3_title(E,c,l,m),b.transition().style("opacity",1)}return E.scale=function(e){return arguments.length?(t=e,E):t},E.cells=function(t){return arguments.length?((t.length>1||t>=2)&&(a=t),E):a},E.cellFilter=function(t){return arguments.length?(s=t,E):s},E.shapePadding=function(t){return arguments.length?(o=+t,E):o},E.labels=function(t){return arguments.length?(A=t,E):A},E.labelAlign=function(t){return arguments.length?("start"!=t&&"end"!=t&&"middle"!=t||(h=t),E):h},E.locale=function(t){return arguments.length?(u=Et(t),E):u},E.labelFormat=function(t){return arguments.length?(g=dt(t),E):E.locale().format(g)},E.labelOffset=function(t){return arguments.length?(d=+t,E):d},E.labelDelimiter=function(t){return arguments.length?(f=t,E):f},E.labelWrap=function(t){return arguments.length?(p=t,E):p},E.orient=function(t){return arguments.length?("horizontal"!=(t=t.toLowerCase())&&"vertical"!=t||(I=t),E):I},E.ascending=function(t){return arguments.length?(C=!!t,E):C},E.classPrefix=function(t){return arguments.length?(l=t,E):l},E.title=function(t){return arguments.length?(c=t,E):c},E.titleWidth=function(t){return arguments.length?(m=t,E):m},E.on=function(){var t=y.on.apply(y,arguments);return t===y?E:t},E}var _e={thresholdLabels:function(t){var e=t.i,n=t.genLength,r=t.generatedLabels,i=t.labelDelimiter;return 0===e?"Less than "+r[e].split(" "+i+" ")[1]:e===n-1?r[e].split(" "+i+" ")[0]+" or more":r[e]}},De={legendColor:Qe,legendSize:we,legendSymbol:xe,legendHelpers:_e};e.default=De},function(t,e,n){"use strict";n.r(e);var r=n(2),i=n(9),o=Array.prototype,a=o.map,s=o.slice,A={name:"implicit"};function l(t){var e=Object(i.c)(),n=[],r=A;function o(i){var o=i+"",a=e.get(o);if(!a){if(r!==A)return r;e.set(o,a=n.push(i))}return t[(a-1)%t.length]}return t=null==t?[]:s.call(t),o.domain=function(t){if(!arguments.length)return n.slice();n=[],e=Object(i.c)();for(var r,a,s=-1,A=t.length;++s<A;)e.has(a=(r=t[s])+"")||e.set(a,n.push(r));return o},o.range=function(e){return arguments.length?(t=s.call(e),o):t.slice()},o.unknown=function(t){return arguments.length?(r=t,o):r},o.copy=function(){return l().domain(n).range(t).unknown(r)},o}function c(){var t,e,n=l().unknown(void 0),i=n.domain,o=n.range,a=[0,1],s=!1,A=0,u=0,g=.5;function h(){var n=i().length,l=a[1]<a[0],c=a[l-0],h=a[1-l];t=(h-c)/Math.max(1,n-A+2*u),s&&(t=Math.floor(t)),c+=(h-c-t*(n-A))*g,e=t*(1-A),s&&(c=Math.round(c),e=Math.round(e));var d=Object(r.range)(n).map(function(e){return c+t*e});return o(l?d.reverse():d)}return delete n.unknown,n.domain=function(t){return arguments.length?(i(t),h()):i()},n.range=function(t){return arguments.length?(a=[+t[0],+t[1]],h()):a.slice()},n.rangeRound=function(t){return a=[+t[0],+t[1]],s=!0,h()},n.bandwidth=function(){return e},n.step=function(){return t},n.round=function(t){return arguments.length?(s=!!t,h()):s},n.padding=function(t){return arguments.length?(A=u=Math.max(0,Math.min(1,t)),h()):A},n.paddingInner=function(t){return arguments.length?(A=Math.max(0,Math.min(1,t)),h()):A},n.paddingOuter=function(t){return arguments.length?(u=Math.max(0,Math.min(1,t)),h()):u},n.align=function(t){return arguments.length?(g=Math.max(0,Math.min(1,t)),h()):g},n.copy=function(){return c().domain(i()).range(a).round(s).paddingInner(A).paddingOuter(u).align(g)},h()}function u(){return function t(e){var n=e.copy;return e.padding=e.paddingOuter,delete e.paddingInner,delete e.paddingOuter,e.copy=function(){return t(n())},e}(c().paddingInner(1))}var g=n(4),h=function(t){return function(){return t}},d=function(t){return+t},f=[0,1];function p(t,e){return(e-=t=+t)?function(n){return(n-t)/e}:h(e)}function I(t,e,n,r){var i=t[0],o=t[1],a=e[0],s=e[1];return o<i?(i=n(o,i),a=r(s,a)):(i=n(i,o),a=r(a,s)),function(t){return a(i(t))}}function C(t,e,n,i){var o=Math.min(t.length,e.length)-1,a=new Array(o),s=new Array(o),A=-1;for(t[o]<t[0]&&(t=t.slice().reverse(),e=e.slice().reverse());++A<o;)a[A]=n(t[A],t[A+1]),s[A]=i(e[A],e[A+1]);return function(e){var n=Object(r.bisect)(t,e,1,o)-1;return s[n](a[n](e))}}function m(t,e){return e.domain(t.domain()).range(t.range()).interpolate(t.interpolate()).clamp(t.clamp())}function y(t,e){var n,r,i,o=f,A=f,l=g.a,c=!1;function u(){return n=Math.min(o.length,A.length)>2?C:I,r=i=null,h}function h(e){return(r||(r=n(o,A,c?function(t){return function(e,n){var r=t(e=+e,n=+n);return function(t){return t<=e?0:t>=n?1:r(t)}}}(t):t,l)))(+e)}return h.invert=function(t){return(i||(i=n(A,o,p,c?function(t){return function(e,n){var r=t(e=+e,n=+n);return function(t){return t<=0?e:t>=1?n:r(t)}}}(e):e)))(+t)},h.domain=function(t){return arguments.length?(o=a.call(t,d),u()):o.slice()},h.range=function(t){return arguments.length?(A=s.call(t),u()):A.slice()},h.rangeRound=function(t){return A=s.call(t),l=g.t,u()},h.clamp=function(t){return arguments.length?(c=!!t,u()):c},h.interpolate=function(t){return arguments.length?(l=t,u()):l},u()}var E=n(6),v=function(t,e,n){var i,o=t[0],a=t[t.length-1],s=Object(r.tickStep)(o,a,null==e?10:e);switch((n=Object(E.e)(null==n?",f":n)).type){case"s":var A=Math.max(Math.abs(o),Math.abs(a));return null!=n.precision||isNaN(i=Object(E.g)(s,A))||(n.precision=i),Object(E.d)(n,A);case"":case"e":case"g":case"p":case"r":null!=n.precision||isNaN(i=Object(E.h)(s,Math.max(Math.abs(o),Math.abs(a))))||(n.precision=i-("e"===n.type));break;case"f":case"%":null!=n.precision||isNaN(i=Object(E.f)(s))||(n.precision=i-2*("%"===n.type))}return Object(E.a)(n)};function B(t){var e=t.domain;return t.ticks=function(t){var n=e();return Object(r.ticks)(n[0],n[n.length-1],null==t?10:t)},t.tickFormat=function(t,n){return v(e(),t,n)},t.nice=function(n){null==n&&(n=10);var i,o=e(),a=0,s=o.length-1,A=o[a],l=o[s];return l<A&&(i=A,A=l,l=i,i=a,a=s,s=i),(i=Object(r.tickIncrement)(A,l,n))>0?(A=Math.floor(A/i)*i,l=Math.ceil(l/i)*i,i=Object(r.tickIncrement)(A,l,n)):i<0&&(A=Math.ceil(A*i)/i,l=Math.floor(l*i)/i,i=Object(r.tickIncrement)(A,l,n)),i>0?(o[a]=Math.floor(A/i)*i,o[s]=Math.ceil(l/i)*i,e(o)):i<0&&(o[a]=Math.ceil(A*i)/i,o[s]=Math.floor(l*i)/i,e(o)),t},t}function b(){var t=y(p,g.o);return t.copy=function(){return m(t,b())},B(t)}function Q(){var t=[0,1];function e(t){return+t}return e.invert=e,e.domain=e.range=function(n){return arguments.length?(t=a.call(n,d),e):t.slice()},e.copy=function(){return Q().domain(t)},B(e)}var w=function(t,e){var n,r=0,i=(t=t.slice()).length-1,o=t[r],a=t[i];return a<o&&(n=r,r=i,i=n,n=o,o=a,a=n),t[r]=e.floor(o),t[i]=e.ceil(a),t};function x(t,e){return(e=Math.log(e/t))?function(n){return Math.log(n/t)/e}:h(e)}function _(t,e){return t<0?function(n){return-Math.pow(-e,n)*Math.pow(-t,1-n)}:function(n){return Math.pow(e,n)*Math.pow(t,1-n)}}function D(t){return isFinite(t)?+("1e"+t):t<0?0:t}function M(t){return 10===t?D:t===Math.E?Math.exp:function(e){return Math.pow(t,e)}}function k(t){return t===Math.E?Math.log:10===t&&Math.log10||2===t&&Math.log2||(t=Math.log(t),function(e){return Math.log(e)/t})}function S(t){return function(e){return-t(-e)}}function L(){var t=y(x,_).domain([1,10]),e=t.domain,n=10,i=k(10),o=M(10);function a(){return i=k(n),o=M(n),e()[0]<0&&(i=S(i),o=S(o)),t}return t.base=function(t){return arguments.length?(n=+t,a()):n},t.domain=function(t){return arguments.length?(e(t),a()):e()},t.ticks=function(t){var a,s=e(),A=s[0],l=s[s.length-1];(a=l<A)&&(h=A,A=l,l=h);var c,u,g,h=i(A),d=i(l),f=null==t?10:+t,p=[];if(!(n%1)&&d-h<f){if(h=Math.round(h)-1,d=Math.round(d)+1,A>0){for(;h<d;++h)for(u=1,c=o(h);u<n;++u)if(!((g=c*u)<A)){if(g>l)break;p.push(g)}}else for(;h<d;++h)for(u=n-1,c=o(h);u>=1;--u)if(!((g=c*u)<A)){if(g>l)break;p.push(g)}}else p=Object(r.ticks)(h,d,Math.min(d-h,f)).map(o);return a?p.reverse():p},t.tickFormat=function(e,r){if(null==r&&(r=10===n?".0e":","),"function"!=typeof r&&(r=Object(E.a)(r)),e===1/0)return r;null==e&&(e=10);var a=Math.max(1,n*e/t.ticks().length);return function(t){var e=t/o(Math.round(i(t)));return e*n<n-.5&&(e*=n),e<=a?r(t):""}},t.nice=function(){return e(w(e(),{floor:function(t){return o(Math.floor(i(t)))},ceil:function(t){return o(Math.ceil(i(t)))}}))},t.copy=function(){return m(t,L().base(n))},t}function R(t,e){return t<0?-Math.pow(-t,e):Math.pow(t,e)}function N(){var t=1,e=y(function(e,n){return(n=R(n,t)-(e=R(e,t)))?function(r){return(R(r,t)-e)/n}:h(n)},function(e,n){return n=R(n,t)-(e=R(e,t)),function(r){return R(e+n*r,1/t)}}),n=e.domain;return e.exponent=function(e){return arguments.length?(t=+e,n(n())):t},e.copy=function(){return m(e,N().exponent(t))},B(e)}function F(){return N().exponent(.5)}function T(){var t=[],e=[],n=[];function i(){var i=0,a=Math.max(1,e.length);for(n=new Array(a-1);++i<a;)n[i-1]=Object(r.quantile)(t,i/a);return o}function o(t){if(!isNaN(t=+t))return e[Object(r.bisect)(n,t)]}return o.invertExtent=function(r){var i=e.indexOf(r);return i<0?[NaN,NaN]:[i>0?n[i-1]:t[0],i<n.length?n[i]:t[t.length-1]]},o.domain=function(e){if(!arguments.length)return t.slice();t=[];for(var n,o=0,a=e.length;o<a;++o)null==(n=e[o])||isNaN(n=+n)||t.push(n);return t.sort(r.ascending),i()},o.range=function(t){return arguments.length?(e=s.call(t),i()):e.slice()},o.quantiles=function(){return n.slice()},o.copy=function(){return T().domain(t).range(e)},o}function U(){var t=0,e=1,n=1,i=[.5],o=[0,1];function a(t){if(t<=t)return o[Object(r.bisect)(i,t,0,n)]}function A(){var r=-1;for(i=new Array(n);++r<n;)i[r]=((r+1)*e-(r-n)*t)/(n+1);return a}return a.domain=function(n){return arguments.length?(t=+n[0],e=+n[1],A()):[t,e]},a.range=function(t){return arguments.length?(n=(o=s.call(t)).length-1,A()):o.slice()},a.invertExtent=function(r){var a=o.indexOf(r);return a<0?[NaN,NaN]:a<1?[t,i[0]]:a>=n?[i[n-1],e]:[i[a-1],i[a]]},a.copy=function(){return U().domain([t,e]).range(o)},B(a)}function O(){var t=[.5],e=[0,1],n=1;function i(i){if(i<=i)return e[Object(r.bisect)(t,i,0,n)]}return i.domain=function(r){return arguments.length?(t=s.call(r),n=Math.min(t.length,e.length-1),i):t.slice()},i.range=function(r){return arguments.length?(e=s.call(r),n=Math.min(t.length,e.length-1),i):e.slice()},i.invertExtent=function(n){var r=e.indexOf(n);return[t[r-1],t[r]]},i.copy=function(){return O().domain(t).range(e)},i}var G=n(1),H=n(10),j=1e3,Y=60*j,P=60*Y,K=24*P,z=7*K,J=30*K,q=365*K;function V(t){return new Date(t)}function W(t){return t instanceof Date?+t:+new Date(+t)}function X(t,e,n,i,o,s,A,l,c){var u=y(p,g.o),h=u.invert,d=u.domain,f=c(".%L"),I=c(":%S"),C=c("%I:%M"),E=c("%I %p"),v=c("%a %d"),B=c("%b %d"),b=c("%B"),Q=c("%Y"),x=[[A,1,j],[A,5,5*j],[A,15,15*j],[A,30,30*j],[s,1,Y],[s,5,5*Y],[s,15,15*Y],[s,30,30*Y],[o,1,P],[o,3,3*P],[o,6,6*P],[o,12,12*P],[i,1,K],[i,2,2*K],[n,1,z],[e,1,J],[e,3,3*J],[t,1,q]];function _(r){return(A(r)<r?f:s(r)<r?I:o(r)<r?C:i(r)<r?E:e(r)<r?n(r)<r?v:B:t(r)<r?b:Q)(r)}function D(e,n,i,o){if(null==e&&(e=10),"number"==typeof e){var a=Math.abs(i-n)/e,s=Object(r.bisector)(function(t){return t[2]}).right(x,a);s===x.length?(o=Object(r.tickStep)(n/q,i/q,e),e=t):s?(o=(s=x[a/x[s-1][2]<x[s][2]/a?s-1:s])[1],e=s[0]):(o=Math.max(Object(r.tickStep)(n,i,e),1),e=l)}return null==o?e:e.every(o)}return u.invert=function(t){return new Date(h(t))},u.domain=function(t){return arguments.length?d(a.call(t,W)):d().map(V)},u.ticks=function(t,e){var n,r=d(),i=r[0],o=r[r.length-1],a=o<i;return a&&(n=i,i=o,o=n),n=(n=D(t,i,o,e))?n.range(i,o+1):[],a?n.reverse():n},u.tickFormat=function(t,e){return null==e?_:c(e)},u.nice=function(t,e){var n=d();return(t=D(t,n[0],n[n.length-1],e))?d(w(n,t)):u},u.copy=function(){return m(u,X(t,e,n,i,o,s,A,l,c))},u}var Z=function(){return X(G.timeYear,G.timeMonth,G.timeWeek,G.timeDay,G.timeHour,G.timeMinute,G.timeSecond,G.timeMillisecond,H.c).domain([new Date(2e3,0,1),new Date(2e3,0,2)])},$=function(){return X(G.utcYear,G.utcMonth,G.utcWeek,G.utcDay,G.utcHour,G.utcMinute,G.utcSecond,G.utcMillisecond,H.g).domain([Date.UTC(2e3,0,1),Date.UTC(2e3,0,2)])},tt=function(t){return t.match(/.{6}/g).map(function(t){return"#"+t})},et=tt("1f77b4ff7f0e2ca02cd627289467bd8c564be377c27f7f7fbcbd2217becf"),nt=tt("393b795254a36b6ecf9c9ede6379398ca252b5cf6bcedb9c8c6d31bd9e39e7ba52e7cb94843c39ad494ad6616be7969c7b4173a55194ce6dbdde9ed6"),rt=tt("3182bd6baed69ecae1c6dbefe6550dfd8d3cfdae6bfdd0a231a35474c476a1d99bc7e9c0756bb19e9ac8bcbddcdadaeb636363969696bdbdbdd9d9d9"),it=tt("1f77b4aec7e8ff7f0effbb782ca02c98df8ad62728ff98969467bdc5b0d58c564bc49c94e377c2f7b6d27f7f7fc7c7c7bcbd22dbdb8d17becf9edae5"),ot=n(3),at=Object(g.f)(Object(ot.b)(300,.5,0),Object(ot.b)(-240,.5,1)),st=Object(g.f)(Object(ot.b)(-100,.75,.35),Object(ot.b)(80,1.5,.8)),At=Object(g.f)(Object(ot.b)(260,.75,.35),Object(ot.b)(80,1.5,.8)),lt=Object(ot.b)(),ct=function(t){(t<0||t>1)&&(t-=Math.floor(t));var e=Math.abs(t-.5);return lt.h=360*t-100,lt.s=1.5-1.5*e,lt.l=.8-.9*e,lt+""};function ut(t){var e=t.length;return function(n){return t[Math.max(0,Math.min(e-1,Math.floor(n*e)))]}}var gt=ut(tt("44015444025645045745055946075a46085c460a5d460b5e470d60470e6147106347116447136548146748166848176948186a481a6c481b6d481c6e481d6f481f70482071482173482374482475482576482677482878482979472a7a472c7a472d7b472e7c472f7d46307e46327e46337f463480453581453781453882443983443a83443b84433d84433e85423f854240864241864142874144874045884046883f47883f48893e49893e4a893e4c8a3d4d8a3d4e8a3c4f8a3c508b3b518b3b528b3a538b3a548c39558c39568c38588c38598c375a8c375b8d365c8d365d8d355e8d355f8d34608d34618d33628d33638d32648e32658e31668e31678e31688e30698e306a8e2f6b8e2f6c8e2e6d8e2e6e8e2e6f8e2d708e2d718e2c718e2c728e2c738e2b748e2b758e2a768e2a778e2a788e29798e297a8e297b8e287c8e287d8e277e8e277f8e27808e26818e26828e26828e25838e25848e25858e24868e24878e23888e23898e238a8d228b8d228c8d228d8d218e8d218f8d21908d21918c20928c20928c20938c1f948c1f958b1f968b1f978b1f988b1f998a1f9a8a1e9b8a1e9c891e9d891f9e891f9f881fa0881fa1881fa1871fa28720a38620a48621a58521a68522a78522a88423a98324aa8325ab8225ac8226ad8127ad8128ae8029af7f2ab07f2cb17e2db27d2eb37c2fb47c31b57b32b67a34b67935b77937b87838b9773aba763bbb753dbc743fbc7340bd7242be7144bf7046c06f48c16e4ac16d4cc26c4ec36b50c46a52c56954c56856c66758c7655ac8645cc8635ec96260ca6063cb5f65cb5e67cc5c69cd5b6ccd5a6ece5870cf5773d05675d05477d1537ad1517cd2507fd34e81d34d84d44b86d54989d5488bd6468ed64590d74393d74195d84098d83e9bd93c9dd93ba0da39a2da37a5db36a8db34aadc32addc30b0dd2fb2dd2db5de2bb8de29bade28bddf26c0df25c2df23c5e021c8e020cae11fcde11dd0e11cd2e21bd5e21ad8e219dae319dde318dfe318e2e418e5e419e7e419eae51aece51befe51cf1e51df4e61ef6e620f8e621fbe723fde725")),ht=ut(tt("00000401000501010601010802010902020b02020d03030f03031204041405041606051806051a07061c08071e0907200a08220b09240c09260d0a290e0b2b100b2d110c2f120d31130d34140e36150e38160f3b180f3d19103f1a10421c10441d11471e114920114b21114e22115024125325125527125829115a2a115c2c115f2d11612f116331116533106734106936106b38106c390f6e3b0f703d0f713f0f72400f74420f75440f764510774710784910784a10794c117a4e117b4f127b51127c52137c54137d56147d57157e59157e5a167e5c167f5d177f5f187f601880621980641a80651a80671b80681c816a1c816b1d816d1d816e1e81701f81721f817320817521817621817822817922827b23827c23827e24828025828125818326818426818627818827818928818b29818c29818e2a81902a81912b81932b80942c80962c80982d80992d809b2e7f9c2e7f9e2f7fa02f7fa1307ea3307ea5317ea6317da8327daa337dab337cad347cae347bb0357bb2357bb3367ab5367ab73779b83779ba3878bc3978bd3977bf3a77c03a76c23b75c43c75c53c74c73d73c83e73ca3e72cc3f71cd4071cf4070d0416fd2426fd3436ed5446dd6456cd8456cd9466bdb476adc4869de4968df4a68e04c67e24d66e34e65e44f64e55064e75263e85362e95462ea5661eb5760ec5860ed5a5fee5b5eef5d5ef05f5ef1605df2625df2645cf3655cf4675cf4695cf56b5cf66c5cf66e5cf7705cf7725cf8745cf8765cf9785df9795df97b5dfa7d5efa7f5efa815ffb835ffb8560fb8761fc8961fc8a62fc8c63fc8e64fc9065fd9266fd9467fd9668fd9869fd9a6afd9b6bfe9d6cfe9f6dfea16efea36ffea571fea772fea973feaa74feac76feae77feb078feb27afeb47bfeb67cfeb77efeb97ffebb81febd82febf84fec185fec287fec488fec68afec88cfeca8dfecc8ffecd90fecf92fed194fed395fed597fed799fed89afdda9cfddc9efddea0fde0a1fde2a3fde3a5fde5a7fde7a9fde9aafdebacfcecaefceeb0fcf0b2fcf2b4fcf4b6fcf6b8fcf7b9fcf9bbfcfbbdfcfdbf")),dt=ut(tt("00000401000501010601010802010a02020c02020e03021004031204031405041706041907051b08051d09061f0a07220b07240c08260d08290e092b10092d110a30120a32140b34150b37160b39180c3c190c3e1b0c411c0c431e0c451f0c48210c4a230c4c240c4f260c51280b53290b552b0b572d0b592f0a5b310a5c320a5e340a5f3609613809623909633b09643d09653e0966400a67420a68440a68450a69470b6a490b6a4a0c6b4c0c6b4d0d6c4f0d6c510e6c520e6d540f6d550f6d57106e59106e5a116e5c126e5d126e5f136e61136e62146e64156e65156e67166e69166e6a176e6c186e6d186e6f196e71196e721a6e741a6e751b6e771c6d781c6d7a1d6d7c1d6d7d1e6d7f1e6c801f6c82206c84206b85216b87216b88226a8a226a8c23698d23698f24699025689225689326679526679727669827669a28659b29649d29649f2a63a02a63a22b62a32c61a52c60a62d60a82e5fa92e5eab2f5ead305dae305cb0315bb1325ab3325ab43359b63458b73557b93556ba3655bc3754bd3853bf3952c03a51c13a50c33b4fc43c4ec63d4dc73e4cc83f4bca404acb4149cc4248ce4347cf4446d04545d24644d34743d44842d54a41d74b3fd84c3ed94d3dda4e3cdb503bdd513ade5238df5337e05536e15635e25734e35933e45a31e55c30e65d2fe75e2ee8602de9612bea632aeb6429eb6628ec6726ed6925ee6a24ef6c23ef6e21f06f20f1711ff1731df2741cf3761bf37819f47918f57b17f57d15f67e14f68013f78212f78410f8850ff8870ef8890cf98b0bf98c0af98e09fa9008fa9207fa9407fb9606fb9706fb9906fb9b06fb9d07fc9f07fca108fca309fca50afca60cfca80dfcaa0ffcac11fcae12fcb014fcb216fcb418fbb61afbb81dfbba1ffbbc21fbbe23fac026fac228fac42afac62df9c72ff9c932f9cb35f8cd37f8cf3af7d13df7d340f6d543f6d746f5d949f5db4cf4dd4ff4df53f4e156f3e35af3e55df2e661f2e865f2ea69f1ec6df1ed71f1ef75f1f179f2f27df2f482f3f586f3f68af4f88ef5f992f6fa96f8fb9af9fc9dfafda1fcffa4")),ft=ut(tt("0d088710078813078916078a19068c1b068d1d068e20068f2206902406912605912805922a05932c05942e05952f059631059733059735049837049938049a3a049a3c049b3e049c3f049c41049d43039e44039e46039f48039f4903a04b03a14c02a14e02a25002a25102a35302a35502a45601a45801a45901a55b01a55c01a65e01a66001a66100a76300a76400a76600a76700a86900a86a00a86c00a86e00a86f00a87100a87201a87401a87501a87701a87801a87a02a87b02a87d03a87e03a88004a88104a78305a78405a78606a68707a68808a68a09a58b0aa58d0ba58e0ca48f0da4910ea3920fa39410a29511a19613a19814a099159f9a169f9c179e9d189d9e199da01a9ca11b9ba21d9aa31e9aa51f99a62098a72197a82296aa2395ab2494ac2694ad2793ae2892b02991b12a90b22b8fb32c8eb42e8db52f8cb6308bb7318ab83289ba3388bb3488bc3587bd3786be3885bf3984c03a83c13b82c23c81c33d80c43e7fc5407ec6417dc7427cc8437bc9447aca457acb4679cc4778cc4977cd4a76ce4b75cf4c74d04d73d14e72d24f71d35171d45270d5536fd5546ed6556dd7566cd8576bd9586ada5a6ada5b69db5c68dc5d67dd5e66de5f65de6164df6263e06363e16462e26561e26660e3685fe4695ee56a5de56b5de66c5ce76e5be76f5ae87059e97158e97257ea7457eb7556eb7655ec7754ed7953ed7a52ee7b51ef7c51ef7e50f07f4ff0804ef1814df1834cf2844bf3854bf3874af48849f48948f58b47f58c46f68d45f68f44f79044f79143f79342f89441f89540f9973ff9983ef99a3efa9b3dfa9c3cfa9e3bfb9f3afba139fba238fca338fca537fca636fca835fca934fdab33fdac33fdae32fdaf31fdb130fdb22ffdb42ffdb52efeb72dfeb82cfeba2cfebb2bfebd2afebe2afec029fdc229fdc328fdc527fdc627fdc827fdca26fdcb26fccd25fcce25fcd025fcd225fbd324fbd524fbd724fad824fada24f9dc24f9dd25f8df25f8e125f7e225f7e425f6e626f6e826f5e926f5eb27f4ed27f3ee27f3f027f2f227f1f426f1f525f0f724f0f921"));function pt(t){var e=0,n=1,r=!1;function i(i){var o=(i-e)/(n-e);return t(r?Math.max(0,Math.min(1,o)):o)}return i.domain=function(t){return arguments.length?(e=+t[0],n=+t[1],i):[e,n]},i.clamp=function(t){return arguments.length?(r=!!t,i):r},i.interpolator=function(e){return arguments.length?(t=e,i):t},i.copy=function(){return pt(t).domain([e,n]).clamp(r)},B(i)}n.d(e,"scaleBand",function(){return c}),n.d(e,"scalePoint",function(){return u}),n.d(e,"scaleIdentity",function(){return Q}),n.d(e,"scaleLinear",function(){return b}),n.d(e,"scaleLog",function(){return L}),n.d(e,"scaleOrdinal",function(){return l}),n.d(e,"scaleImplicit",function(){return A}),n.d(e,"scalePow",function(){return N}),n.d(e,"scaleSqrt",function(){return F}),n.d(e,"scaleQuantile",function(){return T}),n.d(e,"scaleQuantize",function(){return U}),n.d(e,"scaleThreshold",function(){return O}),n.d(e,"scaleTime",function(){return Z}),n.d(e,"scaleUtc",function(){return $}),n.d(e,"schemeCategory10",function(){return et}),n.d(e,"schemeCategory20b",function(){return nt}),n.d(e,"schemeCategory20c",function(){return rt}),n.d(e,"schemeCategory20",function(){return it}),n.d(e,"interpolateCubehelixDefault",function(){return at}),n.d(e,"interpolateRainbow",function(){return ct}),n.d(e,"interpolateWarm",function(){return st}),n.d(e,"interpolateCool",function(){return At}),n.d(e,"interpolateViridis",function(){return gt}),n.d(e,"interpolateMagma",function(){return ht}),n.d(e,"interpolateInferno",function(){return dt}),n.d(e,"interpolatePlasma",function(){return ft}),n.d(e,"scaleSequential",function(){return pt})}])},module.exports=t(__webpack_require__(234),__webpack_require__(28),__webpack_require__(6),__webpack_require__(48),__webpack_require__(66),__webpack_require__(9),__webpack_require__(0),__webpack_require__(10),__webpack_require__(16),__webpack_require__(83),__webpack_require__(72))},function(t,e){},function(t,e){function n(t){var e=new Error("Cannot find module '"+t+"'");throw e.code="MODULE_NOT_FOUND",e}n.keys=function(){return[]},n.resolve=n,t.exports=n,n.id=1328},function(t,e,n){"use strict";function r(t){for(var n in t)e.hasOwnProperty(n)||(e[n]=t[n])}Object.defineProperty(e,"__esModule",{value:!0}),r(n(688)),r(n(235)),r(n(1333));var i=n(1336);e.RenderedDashboard=i.RenderedDashboard},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(60),i=n(43),o=n(64),a=n(689);class s{constructor({session:t}){this._isDisposed=!1,this.idToName=new Map,this.onHandleUpdatedSrc$=new o.Subject,this.session=t,t.kernel&&this.setupKernelHooks(t.kernel),t.kernelChanged.connect(this.onKernelChanged,this),this._onHandleUpdated=this.onHandleUpdatedSrc$.asObservable()}static GetManager(t){let e=this.Outputs.get(t);return null==e&&(e=new s({session:t}),this.Outputs.set(t,e)),e}get isDisposed(){return this._isDisposed}get onHandleUpdated(){return this._onHandleUpdated}dispose(){this._isDisposed||(this.session.kernelChanged.disconnect(this.onKernelChanged,this),this.session.kernel&&this.destroyKernelHooks(this.session.kernel),this.onHandleUpdatedSrc$.complete())}registerIdForName(t,e){this.idToName.set(t,e)}getAllNamedHandles(){return this.idToName.values()}createHandle(t,e,n){return new a.DisplayHandle({name:t,registry:e,sanitizer:n,onUpdated:this.onHandleUpdated})}onKernelChanged(t,{oldValue:e,newValue:n}){null!=e&&this.destroyKernelHooks(e),null!=n&&this.setupKernelHooks(n)}setupKernelHooks(t){t.iopubMessage.connect(this.onKernelMsg,this)}destroyKernelHooks(t){t.iopubMessage.disconnect(this.onKernelMsg,this),this.idToName.clear()}onKernelMsg(t,e){if(!r.KernelMessage.isUpdateDisplayDataMsg(e))return;const n=e.content.transient.display_id,i=this.idToName.get(n);if(null==i)return;const{data:o,metadata:a}=e.content;this.onHandleUpdatedSrc$.next({data:o,metadata:a,name:i})}}s.Outputs=new i.AttachedProperty({name:"DisplayOutputs",create:()=>null,changed:(t,e)=>{e&&!e.isDisposed&&e.dispose()}}),e.DisplayHandleManager=s},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(43),o=n(64),a=n(113);class s{constructor({session:t,commName:e}){this._msgRecievedSrc$=new o.Subject,this._commClosedSrc$=new o.Subject,this._commOpenedSrc$=new o.Subject,this.comm=null,this._isDisposed=!1,this.awaitingSetup=!1,this._commName=e,this.session=t,this.connectionLock=new r.AsyncTools.Mutex,this._msgRecieved=this._msgRecievedSrc$.asObservable(),this._commClosed=this._commClosedSrc$.asObservable(),this._commOpened=this._commOpenedSrc$.asObservable(),this.session.statusChanged.connect(this.onKernelStatusChanged,this),this.session.kernelChanged.connect(this.onKernelChanged,this),this.session.iopubMessage.connect(this.onIopubMsg,this)}static Create({session:t,commName:e}){const n=A.InstanceMap.get(t);let r=n.find(t=>t.commName===e);return null!=r?r:(r=new s({session:t,commName:e}),n.push(r),r)}dispose(){if(this._isDisposed)return;this._msgRecievedSrc$.complete(),this._commClosedSrc$.complete(),this._commOpenedSrc$.complete(),this.session.iopubMessage.disconnect(this.onIopubMsg,this),this.session.statusChanged.disconnect(this.onKernelStatusChanged,this),this.session.kernelChanged.disconnect(this.onKernelChanged,this);const t=A.InstanceMap.get(this.session);A.InstanceMap.set(this.session,t.filter(t=>t.commName!==this.commName)),this._isDisposed=!0}get commName(){return this._commName}get isDisposed(){return this._isDisposed}get msgRecieved(){return this._msgRecieved}get commClosed(){return this._commClosed}get commOpened(){return this._commOpened}get isOpen(){return!this.awaitingSetup&&null!=this.comm&&!this.comm.isDisposed&&this.connectionLock.isFree}async send(t){await this.connectToComm();const e=this.comm.send(t),n=await e.done;return e.dispose(),n}async sendAndAwaitResponse(t,e,n=2e4){await this.connectToComm();const r=o.race(this.msgRecieved.pipe(a.first(e),a.timeout(n)),this.commClosed).toPromise();await this.send(t);try{return await r}catch(t){if(console.log("[CommManager]","Comm response error"),console.log(t),t instanceof o.TimeoutError)throw new Error("Timeout: expected a response from the kernel but recieved none");throw new Error("Comm channel closed, expected a response from the kernel")}}async connectToComm(t=!1){if(this.isOpen)return;if(this.awaitingSetup)throw Error(A.AWAITING_SETUP_ERR);if(!t&&!this.connectionLock.isFree){if(await this.connectionLock.lock,!this.isOpen)throw Error("Failed to connect to kernel");return}if(null==this.session.kernel)throw Error("No kernel connected");await this.session.ready,this.comm=this.session.kernel.connectToComm(this._commName),this.comm.onMsg=t=>{this._msgRecievedSrc$.next(t.content.data)};const e=this.comm.open();if(await e.done,e.dispose(),null!=this.comm&&!this.comm.isDisposed)return this.comm.onClose=()=>{console.log("Comm closed"),this._commClosedSrc$.error(new Error("Comm closed"))},this.connectionLock.release(),void this._commOpenedSrc$.next();if(this.connectionLock.release(),t)throw Error("Could not connect to kernel. Check your MavenWorks install");throw this.awaitingSetup=!0,Error(A.AWAITING_SETUP_ERR)}async onIopubMsg(t,{content:e,header:n}){if("display_data"!==n.msg_type)return;if(this.isOpen)return;const{transient:r}=e;r&&r.hasOwnProperty("mavenomics_state_ok")&&(await this.connectionLock.lock,this.awaitingSetup=!1,this.isOpen||this.connectToComm(!0))}onKernelChanged(){null!=this.comm&&(console.log("Comm closed: Kernel Changing"),this._commClosedSrc$.error(new Error("Comm closed")))}onKernelStatusChanged(t,e){null!=this.comm&&"restarting"===e&&(console.log("Comm closed: Kernel Restarting"),this._commClosedSrc$.error(new Error("Comm closed")))}}var A;e.CommManager=s,function(t){t.InstanceMap=new i.AttachedProperty({create:()=>[],name:"InstanceMap"}),t.AWAITING_SETUP_ERR="Kernel not initialized!\n\nIf you are running Python, you may need to import the library to use this feature. Try running:\n\n```\nimport mavenworks\n```\n\nto clear this error."}(A||(A={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(49),i=n(126);class o extends Error{static async Create(t,e){Array.isArray(t)&&(t=t.join("\n"));const n=new o(t,e);return await n._onRendered,n}constructor(t,e){super("Error in "+e+" code");const n=document.createElement("p");this.traceback=t,this.prettyTraceback="",this._onRendered=i.renderText({source:t,host:n,sanitizer:r.defaultSanitizer}).then(()=>{this.prettyTraceback=n.innerHTML})}}e.KernelError=o},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var r=n(690);e.KernelExpressionEvaluator=r.KernelExpressionEvaluator;var i=n(1334);e.registerUDPs=i.registerUDPs;var o=n(691);e.KernelProxyPart=o.KernelProxyPart;var a=n(1335);e.SyncMetadata=a.SyncMetadata},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(48),i=n(17);e.registerUDPs=async function(t,e){const n=i.URLExt.join(i.PageConfig.getBaseUrl(),"/serverconfig/");let o="";null!=e&&(o=i.URLExt.objectToQueryString({path:i.PathExt.dirname(e)}));try{const e=await fetch(n+o).then(t=>t.json());for(const n of e.objs){const e=r.JavascriptEvalPart.generateWrapper(n,n.path);t.registerPart(e.name,e)}}catch(t){throw console.error("Failed to connect to config, UDPs weren't registered"),console.error(t),t}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(28),i=n(9),o=n(691),a=n(235);e.SyncMetadata=class{constructor(t,e,n){this._isDisposed=!1,this.isSetup=!1,this.kernelParts=new Set,this.comm=a.CommManager.Create({session:t,commName:"maven_metadata"}),this.session=t,this.factory=e,this.handleManager=n,this.session.ready.then(()=>this.hookUpdater()),this.comm.msgRecieved.subscribe(t=>{switch(t.msg_type){case"new_part":this.registerPart(t.payload);break;case"named_display_handle":this.handleManager.registerIdForName(t.handle_id,t.handle_name)}}),this.comm.commClosed.subscribe(null,()=>this.unregisterKernelParts()),this.comm.commOpened.subscribe(()=>this.hookUpdater())}get isDisposed(){return this._isDisposed}get ready(){return this._ready?this._ready.promise:Promise.resolve()}dispose(){this._isDisposed||(this.comm.dispose(),this.unregisterKernelParts(),this._isDisposed=!0)}unregisterKernelParts(){this.isSetup=!1;for(const t of this.kernelParts)this.factory.unregisterPart(t);this.kernelParts.clear()}async hookUpdater(){if(!this.isSetup){this._ready=new i.PromiseDelegate;try{await this.comm.sendAndAwaitResponse({msg_type:"send_parts"},t=>"kernel_parts_sent"===t.msg_type,1e4),this.isSetup=!0}catch(t){console.warn("Sync Error",t)}this._ready.resolve()}}registerPart(t){this.kernelParts.add(t.name),this.factory.registerPart(t.name,class extends o.KernelProxyPart{constructor(e){super(e),this.type=t.name,this.title.label=t.name}static GetMetadata(){const e=o.KernelProxyPart.GetMetadata();for(const n in t.options){const i=t.options[n],o=r.Types.findType(i.type)||r.Types.Any,a=r.Converters.deserialize(i.value);e.addOption(i.name,o,a)}return e}})}}},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(189),i=n(17),o=n(310),a=n(6),s=n(688),A=n(690);class l extends a.Widget{static isEditableCell(t){const e=t.outputArea.model;for(let t=0;t<e.length;t++){const n=e.get(t);if(o.DashboardSerializer.MAVEN_LAYOUT_MIME_TYPE in n.data){return!!n.data[o.DashboardSerializer.MAVEN_LAYOUT_MIME_TYPE].visual}}return!1}static getCellDashboard(t){const e=t.outputArea.model;for(let t=0;t<e.length;t++){const n=e.get(t);if(o.DashboardSerializer.MAVEN_LAYOUT_MIME_TYPE in n.data)return n.data[o.DashboardSerializer.MAVEN_LAYOUT_MIME_TYPE]}return null}static getDashboardFromCell(t){let e=t.outputArea.widgets[0].widgets[1].dashboard;return null!=e&&e instanceof o.Dashboard?e:null}static getCellLayoutGuid(t){if(t.outputArea.model.length<=0)return null;const e=this.getDashboardFromCell(t);return null==e?null:e.uuid}static getPythonCode(t){return`# Auto-generated code, do not edit!\n_json = __import__("json")\ndisplay(_json.loads(${JSON.stringify(`{"${o.DashboardSerializer.MAVEN_LAYOUT_MIME_TYPE}": ${JSON.stringify(t)}}`)}), raw=True)\ndel _json`}constructor({rendermime:t,session:e,factory:n,ready:r}){super(),this.mimeTypeRegistry=t,this.ready=r,this.addClass("m-RenderedLayout"),this.layout=new a.BoxLayout,this.layout.fitPolicy="set-no-constraint";let c=new A.KernelExpressionEvaluator({session:e}),u=s.DisplayHandleManager.GetManager(e);this.dashboard=new o.Dashboard({factory:n,rendermime:t,session:e,evaluator:c,externalPartRenderer:new l.ExternalPartRenderer(t,u),baseUrl:i.URLExt.join(i.PageConfig.getBaseUrl(),"/files"),baseViewUrl:i.URLExt.join(i.PageConfig.getBaseUrl(),"/view")}),this.node.style.height="500px",c.globals=this.dashboard.globals,this.dashboard.OnDirty.subscribe(()=>{this.saveLayout(),this.dashboard.setClean()}),this.layout.addWidget(this.dashboard)}async renderModel(t){await this.ready;const e=t.data[o.DashboardSerializer.MAVEN_LAYOUT_MIME_TYPE];this.node.style.height=`${(e.dims||{height:500}).height}px`,this.dashboard.fit(),await this.dashboard.loadFromModel(e)}saveLayout(){let t=this.parent;for(;null!=t;){if(t instanceof r.CodeCell&&l.isEditableCell(t)){let e=o.DashboardSerializer.toJson(this.dashboard);e.visual=!0;let n=l.getCellLayoutGuid(t);if(e&&n===this.dashboard.uuid)return t.model.value.text=l.getPythonCode(e),void(t.inputHidden=!0)}t=t.parent}}dispose(){this.isDisposed||(this.dashboard.dispose(),delete this.mimeTypeRegistry,super.dispose())}}e.RenderedDashboard=l,function(t){t.ExternalPartRenderer=class extends o.Dashboard.DefaultExternalPartRenderer{constructor(t,e){super(t),this.handles=new Map,this.manager=e}renderModel(t,e,n){if(null!=t[s.DisplayHandle.MIME_TYPE]){const e=t[s.DisplayHandle.MIME_TYPE],r=this.manager.createHandle(""+e.name,this.rendermime,this.rendermime.sanitizer);return this.handles.set(n,r),Promise.resolve()}return super.renderModel(t,e,n)}getPartById(t){return this.handles.get(t)||super.getPartById(t)}getMetadata(){const t=[];for(let e of this.manager.getAllNamedHandles())t.push({name:e,model:{[s.DisplayHandle.MIME_TYPE]:{name:e}}});return t}*getAllPartIds(){yield*super.getAllPartIds(),yield*this.handles.keys()}serializePart(t){const e=this.handles.get(t);return null!=e?[{[s.DisplayHandle.MIME_TYPE]:{name:e.name}},null]:super.serializePart(t)}}}(l=e.RenderedDashboard||(e.RenderedDashboard={}))},function(t,e,n){"use strict";Object.defineProperty(e,"__esModule",{value:!0});const r=n(1451),i=n(20),o=n(1450),a=n(203),s=n(1415);e.registerWidgets=function(t,e,n){const A=new r.WidgetManager(e,n,{saveState:!1});A.register({name:"@jupyter-widgets/base",version:i.JUPYTER_WIDGETS_VERSION,exports:i}),A.register({name:"@jupyter-widgets/controls",version:o.JUPYTER_CONTROLS_VERSION,exports:o}),A.register({name:"@jupyter-widgets/output",version:a.OUTPUT_WIDGET_VERSION,exports:a}),A.register({name:"plotlywidget",version:s.version,exports:s}),t.content.rendermime.addFactory({safe:!1,mimeTypes:["application/vnd.jupyter.widget-view+json"],createRenderer:t=>new r.WidgetRenderer(t,A)})}},function(t,e,n){var r=n(1339),i=n(183);t.exports=function t(e,n,o,a,s){return e===n||(null==e||null==n||!i(e)&&!i(n)?e!=e&&n!=n:r(e,n,o,a,t,s))}},function(t,e,n){var r=n(1340),i=n(698),o=n(1374),a=n(1378),s=n(1399),A=n(345),l=n(699),c=n(700),u=1,g="[object Arguments]",h="[object Array]",d="[object Object]",f=Object.prototype.hasOwnProperty;t.exports=function(t,e,n,p,I,C){var m=A(t),y=A(e),E=m?h:s(t),v=y?h:s(e),B=(E=E==g?d:E)==d,b=(v=v==g?d:v)==d,Q=E==v;if(Q&&l(t)){if(!l(e))return!1;m=!0,B=!1}if(Q&&!B)return C||(C=new r),m||c(t)?i(t,e,n,p,I,C):o(t,e,E,n,p,I,C);if(!(n&u)){var w=B&&f.call(t,"__wrapped__"),x=b&&f.call(e,"__wrapped__");if(w||x){var _=w?t.value():t,D=x?e.value():e;return C||(C=new r),I(_,D,n,p,C)}}return!!Q&&(C||(C=new r),a(t,e,n,p,I,C))}},function(t,e,n){var r=n(236),i=n(1346),o=n(1347),a=n(1348),s=n(1349),A=n(1350);function l(t){var e=this.__data__=new r(t);this.size=e.size}l.prototype.clear=i,l.prototype.delete=o,l.prototype.get=a,l.prototype.has=s,l.prototype.set=A,t.exports=l},function(t,e){t.exports=function(){this.__data__=[],this.size=0}},function(t,e,n){var r=n(237),i=Array.prototype.splice;t.exports=function(t){var e=this.__data__,n=r(e,t);return!(n<0||(n==e.length-1?e.pop():i.call(e,n,1),--this.size,0))}},function(t,e,n){var r=n(237);t.exports=function(t){var e=this.__data__,n=r(e,t);return n<0?void 0:e[n][1]}},function(t,e,n){var r=n(237);t.exports=function(t){return r(this.__data__,t)>-1}},function(t,e,n){var r=n(237);t.exports=function(t,e){var n=this.__data__,i=r(n,t);return i<0?(++this.size,n.push([t,e])):n[i][1]=e,this}},function(t,e,n){var r=n(236);t.exports=function(){this.__data__=new r,this.size=0}},function(t,e){t.exports=function(t){var e=this.__data__,n=e.delete(t);return this.size=e.size,n}},function(t,e){t.exports=function(t){return this.__data__.get(t)}},function(t,e){t.exports=function(t){return this.__data__.has(t)}},function(t,e,n){var r=n(236),i=n(343),o=n(697),a=200;t.exports=function(t,e){var n=this.__data__;if(n instanceof r){var s=n.__data__;if(!i||s.length<a-1)return s.push([t,e]),this.size=++n.size,this;n=this.__data__=new o(s)}return n.set(t,e),this.size=n.size,this}},function(t,e,n){var r=n(693),i=n(1354),o=n(695),a=n(696),s=/^\[object .+?Constructor\]$/,A=Function.prototype,l=Object.prototype,c=A.toString,u=l.hasOwnProperty,g=RegExp("^"+c.call(u).replace(/[\\^$.*+?()[\]{}|]/g,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");t.exports=function(t){return!(!o(t)||i(t))&&(r(t)?g:s).test(a(t))}},function(t,e,n){var r=n(344),i=Object.prototype,o=i.hasOwnProperty,a=i.toString,s=r?r.toStringTag:void 0;t.exports=function(t){var e=o.call(t,s),n=t[s];try{t[s]=void 0;var r=!0}catch(t){}var i=a.call(t);return r&&(e?t[s]=n:delete t[s]),i}},function(t,e){var n=Object.prototype.toString;t.exports=function(t){return n.call(t)}},function(t,e,n){var r,i=n(1355),o=(r=/[^.]+$/.exec(i&&i.keys&&i.keys.IE_PROTO||""))?"Symbol(src)_1."+r:"";t.exports=function(t){return!!o&&o in t}},function(t,e,n){var r=n(99)["__core-js_shared__"];t.exports=r},function(t,e){t.exports=function(t,e){return null==t?void 0:t[e]}},function(t,e,n){var r=n(1358),i=n(236),o=n(343);t.exports=function(){this.size=0,this.__data__={hash:new r,map:new(o||i),string:new r}}},function(t,e,n){var r=n(1359),i=n(1360),o=n(1361),a=n(1362),s=n(1363);function A(t){var e=-1,n=null==t?0:t.length;for(this.clear();++e<n;){var r=t[e];this.set(r[0],r[1])}}A.prototype.clear=r,A.prototype.delete=i,A.prototype.get=o,A.prototype.has=a,A.prototype.set=s,t.exports=A},function(t,e,n){var r=n(238);t.exports=function(){this.__data__=r?r(null):{},this.size=0}},function(t,e){t.exports=function(t){var e=this.has(t)&&delete this.__data__[t];return this.size-=e?1:0,e}},function(t,e,n){var r=n(238),i="__lodash_hash_undefined__",o=Object.prototype.hasOwnProperty;t.exports=function(t){var e=this.__data__;if(r){var n=e[t];return n===i?void 0:n}return o.call(e,t)?e[t]:void 0}},function(t,e,n){var r=n(238),i=Object.prototype.hasOwnProperty;t.exports=function(t){var e=this.__data__;return r?void 0!==e[t]:i.call(e,t)}},function(t,e,n){var r=n(238),i="__lodash_hash_undefined__";t.exports=function(t,e){var n=this.__data__;return this.size+=this.has(t)?0:1,n[t]=r&&void 0===e?i:e,this}},function(t,e,n){var r=n(239);t.exports=function(t){var e=r(this,t).delete(t);return this.size-=e?1:0,e}},function(t,e){t.exports=function(t){var e=typeof t;return"string"==e||"number"==e||"symbol"==e||"boolean"==e?"__proto__"!==t:null===t}},function(t,e,n){var r=n(239);t.exports=function(t){return r(this,t).get(t)}},function(t,e,n){var r=n(239);t.exports=function(t){return r(this,t).has(t)}},function(t,e,n){var r=n(239);t.exports=function(t,e){var n=r(this,t),i=n.size;return n.set(t,e),this.size+=n.size==i?0:1,this}},function(t,e,n){var r=n(697),i=n(1370),o=n(1371);function a(t){var e=-1,n=null==t?0:t.length;for(this.__data__=new r;++e<n;)this.add(t[e])}a.prototype.add=a.prototype.push=i,a.prototype.has=o,t.exports=a},function(t,e){var n="__lodash_hash_undefined__";t.exports=function(t){return this.__data__.set(t,n),this}},function(t,e){t.exports=function(t){return this.__data__.has(t)}},function(t,e){t.exports=function(t,e){for(var n=-1,r=null==t?0:t.length;++n<r;)if(e(t[n],n,t))return!0;return!1}},function(t,e){t.exports=function(t,e){return t.has(e)}},function(t,e,n){var r=n(344),i=n(1375),o=n(692),a=n(698),s=n(1376),A=n(1377),l=1,c=2,u="[object Boolean]",g="[object Date]",h="[object Error]",d="[object Map]",f="[object Number]",p="[object RegExp]",I="[object Set]",C="[object String]",m="[object Symbol]",y="[object ArrayBuffer]",E="[object DataView]",v=r?r.prototype:void 0,B=v?v.valueOf:void 0;t.exports=function(t,e,n,r,v,b,Q){switch(n){case E:if(t.byteLength!=e.byteLength||t.byteOffset!=e.byteOffset)return!1;t=t.buffer,e=e.buffer;case y:return!(t.byteLength!=e.byteLength||!b(new i(t),new i(e)));case u:case g:case f:return o(+t,+e);case h:return t.name==e.name&&t.message==e.message;case p:case C:return t==e+"";case d:var w=s;case I:var x=r&l;if(w||(w=A),t.size!=e.size&&!x)return!1;var _=Q.get(t);if(_)return _==e;r|=c,Q.set(t,e);var D=a(w(t),w(e),r,v,b,Q);return Q.delete(t),D;case m:if(B)return B.call(t)==B.call(e)}return!1}},function(t,e,n){var r=n(99).Uint8Array;t.exports=r},function(t,e){t.exports=function(t){var e=-1,n=Array(t.size);return t.forEach(function(t,r){n[++e]=[r,t]}),n}},function(t,e){t.exports=function(t){var e=-1,n=Array(t.size);return t.forEach(function(t){n[++e]=t}),n}},function(t,e,n){var r=n(1379),i=1,o=Object.prototype.hasOwnProperty;t.exports=function(t,e,n,a,s,A){var l=n&i,c=r(t),u=c.length;if(u!=r(e).length&&!l)return!1;for(var g=u;g--;){var h=c[g];if(!(l?h in e:o.call(e,h)))return!1}var d=A.get(t);if(d&&A.get(e))return d==e;var f=!0;A.set(t,e),A.set(e,t);for(var p=l;++g<u;){var I=t[h=c[g]],C=e[h];if(a)var m=l?a(C,I,h,e,t,A):a(I,C,h,t,e,A);if(!(void 0===m?I===C||s(I,C,n,a,A):m)){f=!1;break}p||(p="constructor"==h)}if(f&&!p){var y=t.constructor,E=e.constructor;y!=E&&"constructor"in t&&"constructor"in e&&!("function"==typeof y&&y instanceof y&&"function"==typeof E&&E instanceof E)&&(f=!1)}return A.delete(t),A.delete(e),f}},function(t,e,n){var r=n(1380),i=n(1382),o=n(1385);t.exports=function(t){return r(t,o,i)}},function(t,e,n){var r=n(1381),i=n(345);t.exports=function(t,e,n){var o=e(t);return i(t)?o:r(o,n(t))}},function(t,e){t.exports=function(t,e){for(var n=-1,r=e.length,i=t.length;++n<r;)t[i+n]=e[n];return t}},function(t,e,n){var r=n(1383),i=n(1384),o=Object.prototype.propertyIsEnumerable,a=Object.getOwnPropertySymbols,s=a?function(t){return null==t?[]:(t=Object(t),r(a(t),function(e){return o.call(t,e)}))}:i;t.exports=s},function(t,e){t.exports=function(t,e){for(var n=-1,r=null==t?0:t.length,i=0,o=[];++n<r;){var a=t[n];e(a,n,t)&&(o[i++]=a)}return o}},function(t,e){t.exports=function(){return[]}},function(t,e,n){var r=n(1386),i=n(1395),o=n(1398);t.exports=function(t){return o(t)?r(t):i(t)}},function(t,e,n){var r=n(1387),i=n(1388),o=n(345),a=n(699),s=n(1391),A=n(700),l=Object.prototype.hasOwnProperty;t.exports=function(t,e){var n=o(t),c=!n&&i(t),u=!n&&!c&&a(t),g=!n&&!c&&!u&&A(t),h=n||c||u||g,d=h?r(t.length,String):[],f=d.length;for(var p in t)!e&&!l.call(t,p)||h&&("length"==p||u&&("offset"==p||"parent"==p)||g&&("buffer"==p||"byteLength"==p||"byteOffset"==p)||s(p,f))||d.push(p);return d}},function(t,e){t.exports=function(t,e){for(var n=-1,r=Array(t);++n<t;)r[n]=e(n);return r}},function(t,e,n){var r=n(1389),i=n(183),o=Object.prototype,a=o.hasOwnProperty,s=o.propertyIsEnumerable,A=r(function(){return arguments}())?r:function(t){return i(t)&&a.call(t,"callee")&&!s.call(t,"callee")};t.exports=A},function(t,e,n){var r=n(182),i=n(183),o="[object Arguments]";t.exports=function(t){return i(t)&&r(t)==o}},function(t,e){t.exports=function(){return!1}},function(t,e){var n=9007199254740991,r=/^(?:0|[1-9]\d*)$/;t.exports=function(t,e){var i=typeof t;return!!(e=null==e?n:e)&&("number"==i||"symbol"!=i&&r.test(t))&&t>-1&&t%1==0&&t<e}},function(t,e,n){var r=n(182),i=n(701),o=n(183),a={};a["[object Float32Array]"]=a["[object Float64Array]"]=a["[object Int8Array]"]=a["[object Int16Array]"]=a["[object Int32Array]"]=a["[object Uint8Array]"]=a["[object Uint8ClampedArray]"]=a["[object Uint16Array]"]=a["[object Uint32Array]"]=!0,a["[object Arguments]"]=a["[object Array]"]=a["[object ArrayBuffer]"]=a["[object Boolean]"]=a["[object DataView]"]=a["[object Date]"]=a["[object Error]"]=a["[object Function]"]=a["[object Map]"]=a["[object Number]"]=a["[object Object]"]=a["[object RegExp]"]=a["[object Set]"]=a["[object String]"]=a["[object WeakMap]"]=!1,t.exports=function(t){return o(t)&&i(t.length)&&!!a[r(t)]}},function(t,e){t.exports=function(t){return function(e){return t(e)}}},function(t,e,n){(function(t){var r=n(694),i=e&&!e.nodeType&&e,o=i&&"object"==typeof t&&t&&!t.nodeType&&t,a=o&&o.exports===i&&r.process,s=function(){try{var t=o&&o.require&&o.require("util").types;return t||a&&a.binding&&a.binding("util")}catch(t){}}();t.exports=s}).call(this,n(98)(t))},function(t,e,n){var r=n(1396),i=n(1397),o=Object.prototype.hasOwnProperty;t.exports=function(t){if(!r(t))return i(t);var e=[];for(var n in Object(t))o.call(t,n)&&"constructor"!=n&&e.push(n);return e}},function(t,e){var n=Object.prototype;t.exports=function(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||n)}},function(t,e,n){var r=n(702)(Object.keys,Object);t.exports=r},function(t,e,n){var r=n(693),i=n(701);t.exports=function(t){return null!=t&&i(t.length)&&!r(t)}},function(t,e,n){var r=n(1400),i=n(343),o=n(1401),a=n(1402),s=n(1403),A=n(182),l=n(696),c=l(r),u=l(i),g=l(o),h=l(a),d=l(s),f=A;(r&&"[object DataView]"!=f(new r(new ArrayBuffer(1)))||i&&"[object Map]"!=f(new i)||o&&"[object Promise]"!=f(o.resolve())||a&&"[object Set]"!=f(new a)||s&&"[object WeakMap]"!=f(new s))&&(f=function(t){var e=A(t),n="[object Object]"==e?t.constructor:void 0,r=n?l(n):"";if(r)switch(r){case c:return"[object DataView]";case u:return"[object Map]";case g:return"[object Promise]";case h:return"[object Set]";case d:return"[object WeakMap]"}return e}),t.exports=f},function(t,e,n){var r=n(147)(n(99),"DataView");t.exports=r},function(t,e,n){var r=n(147)(n(99),"Promise");t.exports=r},function(t,e,n){var r=n(147)(n(99),"Set");t.exports=r},function(t,e,n){var r=n(147)(n(99),"WeakMap");t.exports=r},function(t,e,n){var r=n(702)(Object.getPrototypeOf,Object);t.exports=r},function(t,e,n){var r=n(1406);"string"==typeof r&&(r=[[t.i,r,""]]);var i={hmr:!0,transform:void 0,insertInto:void 0};n(287)(r,i);r.locals&&(t.exports=r.locals)},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,'/* Copyright (c) Jupyter Development Team.\n * Distributed under the terms of the Modified BSD License.\n */\n\n .jupyter-widgets-disconnected::before {\n    content: "\\F127"; /* chain-broken */\n    display: inline-block;\n    font: normal normal normal 14px/1 FontAwesome;\n    font-size: inherit;\n    text-rendering: auto;\n    -webkit-font-smoothing: antialiased;\n    -moz-osx-font-smoothing: grayscale;\n    color: #d9534f;\n    padding: 3px;\n    align-self: flex-start;\n}\n',""])},function(t,e,n){var r=n(1408);"string"==typeof r&&(r=[[t.i,r,""]]);var i={hmr:!0,transform:void 0,insertInto:void 0};n(287)(r,i);r.locals&&(t.exports=r.locals)},function(t,e,n){(e=t.exports=n(12)(!1)).i(n(1409),""),e.push([t.i,'/* Copyright (c) Jupyter Development Team.\n * Distributed under the terms of the Modified BSD License.\n */\n\n/*\n * We assume that the CSS variables in\n * https://github.com/jupyterlab/jupyterlab/blob/master/src/default-theme/variables.css\n * have been defined.\n */\n\n:root {\n    --jp-widgets-color: var(--jp-content-font-color1);\n    --jp-widgets-label-color: var(--jp-widgets-color);\n    --jp-widgets-readout-color: var(--jp-widgets-color);\n    --jp-widgets-font-size: var(--jp-ui-font-size1);\n    --jp-widgets-margin: 2px;\n    --jp-widgets-inline-height: 28px;\n    --jp-widgets-inline-width: 300px;\n    --jp-widgets-inline-width-short: calc(var(--jp-widgets-inline-width) / 2 - var(--jp-widgets-margin));\n    --jp-widgets-inline-width-tiny: calc(var(--jp-widgets-inline-width-short) / 2 - var(--jp-widgets-margin));\n    --jp-widgets-inline-margin: 4px; /* margin between inline elements */\n    --jp-widgets-inline-label-width: 80px;\n    --jp-widgets-border-width: var(--jp-border-width);\n    --jp-widgets-vertical-height: 200px;\n    --jp-widgets-horizontal-tab-height: 24px;\n    --jp-widgets-horizontal-tab-width: 144px;\n    --jp-widgets-horizontal-tab-top-border: 2px;\n    --jp-widgets-progress-thickness: 20px;\n    --jp-widgets-container-padding: 15px;\n    --jp-widgets-input-padding: 4px;\n    --jp-widgets-radio-item-height-adjustment: 8px;\n    --jp-widgets-radio-item-height: calc(var(--jp-widgets-inline-height) - var(--jp-widgets-radio-item-height-adjustment));\n    --jp-widgets-slider-track-thickness: 4px;\n    --jp-widgets-slider-border-width: var(--jp-widgets-border-width);\n    --jp-widgets-slider-handle-size: 16px;\n    --jp-widgets-slider-handle-border-color: var(--jp-border-color1);\n    --jp-widgets-slider-handle-background-color: var(--jp-layout-color1);\n    --jp-widgets-slider-active-handle-color: var(--jp-brand-color1);\n    --jp-widgets-menu-item-height: 24px;\n    --jp-widgets-dropdown-arrow: url("data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPCEtLSBHZW5lcmF0b3I6IEFkb2JlIElsbHVzdHJhdG9yIDE5LjIuMSwgU1ZHIEV4cG9ydCBQbHVnLUluIC4gU1ZHIFZlcnNpb246IDYuMDAgQnVpbGQgMCkgIC0tPgo8c3ZnIHZlcnNpb249IjEuMSIgaWQ9IkxheWVyXzEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgeG1sbnM6eGxpbms9Imh0dHA6Ly93d3cudzMub3JnLzE5OTkveGxpbmsiIHg9IjBweCIgeT0iMHB4IgoJIHZpZXdCb3g9IjAgMCAxOCAxOCIgc3R5bGU9ImVuYWJsZS1iYWNrZ3JvdW5kOm5ldyAwIDAgMTggMTg7IiB4bWw6c3BhY2U9InByZXNlcnZlIj4KPHN0eWxlIHR5cGU9InRleHQvY3NzIj4KCS5zdDB7ZmlsbDpub25lO30KPC9zdHlsZT4KPHBhdGggZD0iTTUuMiw1LjlMOSw5LjdsMy44LTMuOGwxLjIsMS4ybC00LjksNWwtNC45LTVMNS4yLDUuOXoiLz4KPHBhdGggY2xhc3M9InN0MCIgZD0iTTAtMC42aDE4djE4SDBWLTAuNnoiLz4KPC9zdmc+Cg");\n    --jp-widgets-input-color: var(--jp-ui-font-color1);\n    --jp-widgets-input-background-color: var(--jp-layout-color1);\n    --jp-widgets-input-border-color: var(--jp-border-color1);\n    --jp-widgets-input-focus-border-color: var(--jp-brand-color2);\n    --jp-widgets-input-border-width: var(--jp-widgets-border-width);\n    --jp-widgets-disabled-opacity: 0.6;\n\n    /* From Material Design Lite */\n    --md-shadow-key-umbra-opacity: 0.2;\n    --md-shadow-key-penumbra-opacity: 0.14;\n    --md-shadow-ambient-shadow-opacity: 0.12;\n}\n\n.jupyter-widgets {\n    margin: var(--jp-widgets-margin);\n    box-sizing: border-box;\n    color: var(--jp-widgets-color);\n    overflow: visible;\n}\n\n.jupyter-widgets.jupyter-widgets-disconnected::before {\n    line-height: var(--jp-widgets-inline-height);\n    height: var(--jp-widgets-inline-height);\n}\n\n.jp-Output-result > .jupyter-widgets {\n    margin-left: 0;\n    margin-right: 0;\n}\n\n/* vbox and hbox */\n\n.widget-inline-hbox {\n    /* Horizontal widgets */\n    box-sizing: border-box;\n    display: flex;\n    flex-direction: row;\n    align-items: baseline;\n}\n\n.widget-inline-vbox {\n    /* Vertical Widgets */\n    box-sizing: border-box;\n    display: flex;\n    flex-direction: column;\n    align-items: center;\n}\n\n.widget-box {\n    box-sizing: border-box;\n    display: flex;\n    margin: 0;\n    overflow: auto;\n}\n\n.widget-gridbox {\n    box-sizing: border-box;\n    display: grid;\n    margin: 0;\n    overflow: auto;\n}\n\n.widget-hbox {\n    flex-direction: row;\n}\n\n.widget-vbox {\n    flex-direction: column;\n}\n\n/* General Button Styling */\n\n.jupyter-button {\n    padding-left: 10px;\n    padding-right: 10px;\n    padding-top: 0px;\n    padding-bottom: 0px;\n    display: inline-block;\n    white-space: nowrap;\n    overflow: hidden;\n    text-overflow: ellipsis;\n    text-align: center;\n    font-size: var(--jp-widgets-font-size);\n    cursor: pointer;\n\n    height: var(--jp-widgets-inline-height);\n    border: 0px solid;\n    line-height: var(--jp-widgets-inline-height);\n    box-shadow: none;\n\n    color: var(--jp-ui-font-color1);\n    background-color: var(--jp-layout-color2);\n    border-color: var(--jp-border-color2);\n    border: none;\n    user-select: none;\n}\n\n.jupyter-button i.fa {\n    margin-right: var(--jp-widgets-inline-margin);\n    pointer-events: none;\n}\n\n.jupyter-button:empty:before {\n    content: "\\200B"; /* zero-width space */\n}\n\n.jupyter-widgets.jupyter-button:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n.jupyter-button i.fa.center {\n    margin-right: 0;\n}\n\n.jupyter-button:hover:enabled, .jupyter-button:focus:enabled {\n    /* MD Lite 2dp shadow */\n    box-shadow: 0 2px 2px 0 rgba(0, 0, 0, var(--md-shadow-key-penumbra-opacity)),\n                0 3px 1px -2px rgba(0, 0, 0, var(--md-shadow-key-umbra-opacity)),\n                0 1px 5px 0 rgba(0, 0, 0, var(--md-shadow-ambient-shadow-opacity));\n}\n\n.jupyter-button:active, .jupyter-button.mod-active {\n    /* MD Lite 4dp shadow */\n    box-shadow: 0 4px 5px 0 rgba(0, 0, 0, var(--md-shadow-key-penumbra-opacity)),\n                0 1px 10px 0 rgba(0, 0, 0, var(--md-shadow-ambient-shadow-opacity)),\n                0 2px 4px -1px rgba(0, 0, 0, var(--md-shadow-key-umbra-opacity));\n    color: var(--jp-ui-font-color1);\n    background-color: var(--jp-layout-color3);\n}\n\n.jupyter-button:focus:enabled {\n    outline: 1px solid var(--jp-widgets-input-focus-border-color);\n}\n\n/* Button "Primary" Styling */\n\n.jupyter-button.mod-primary {\n    color: var(--jp-inverse-ui-font-color1);\n    background-color: var(--jp-brand-color1);\n}\n\n.jupyter-button.mod-primary.mod-active {\n    color: var(--jp-inverse-ui-font-color0);\n    background-color: var(--jp-brand-color0);\n}\n\n.jupyter-button.mod-primary:active {\n    color: var(--jp-inverse-ui-font-color0);\n    background-color: var(--jp-brand-color0);\n}\n\n/* Button "Success" Styling */\n\n.jupyter-button.mod-success {\n    color: var(--jp-inverse-ui-font-color1);\n    background-color: var(--jp-success-color1);\n}\n\n.jupyter-button.mod-success.mod-active {\n    color: var(--jp-inverse-ui-font-color0);\n    background-color: var(--jp-success-color0);\n }\n\n.jupyter-button.mod-success:active {\n    color: var(--jp-inverse-ui-font-color0);\n    background-color: var(--jp-success-color0);\n }\n\n /* Button "Info" Styling */\n\n.jupyter-button.mod-info {\n    color: var(--jp-inverse-ui-font-color1);\n    background-color: var(--jp-info-color1);\n}\n\n.jupyter-button.mod-info.mod-active {\n    color: var(--jp-inverse-ui-font-color0);\n    background-color: var(--jp-info-color0);\n}\n\n.jupyter-button.mod-info:active {\n    color: var(--jp-inverse-ui-font-color0);\n    background-color: var(--jp-info-color0);\n}\n\n/* Button "Warning" Styling */\n\n.jupyter-button.mod-warning {\n    color: var(--jp-inverse-ui-font-color1);\n    background-color: var(--jp-warn-color1);\n}\n\n.jupyter-button.mod-warning.mod-active {\n    color: var(--jp-inverse-ui-font-color0);\n    background-color: var(--jp-warn-color0);\n}\n\n.jupyter-button.mod-warning:active {\n    color: var(--jp-inverse-ui-font-color0);\n    background-color: var(--jp-warn-color0);\n}\n\n/* Button "Danger" Styling */\n\n.jupyter-button.mod-danger {\n    color: var(--jp-inverse-ui-font-color1);\n    background-color: var(--jp-error-color1);\n}\n\n.jupyter-button.mod-danger.mod-active {\n    color: var(--jp-inverse-ui-font-color0);\n    background-color: var(--jp-error-color0);\n}\n\n.jupyter-button.mod-danger:active {\n    color: var(--jp-inverse-ui-font-color0);\n    background-color: var(--jp-error-color0);\n}\n\n/* Widget Button, Widget Toggle Button, Widget Upload */\n\n.widget-button, .widget-toggle-button, .widget-upload {\n    width: var(--jp-widgets-inline-width-short);\n}\n\n/* Widget Label Styling */\n\n/* Override Bootstrap label css */\n.jupyter-widgets label {\n    margin-bottom: initial;\n}\n\n.widget-label-basic {\n    /* Basic Label */\n    color: var(--jp-widgets-label-color);\n    font-size: var(--jp-widgets-font-size);\n    overflow: hidden;\n    text-overflow: ellipsis;\n    white-space: nowrap;\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-label {\n    /* Label */\n    color: var(--jp-widgets-label-color);\n    font-size: var(--jp-widgets-font-size);\n    overflow: hidden;\n    text-overflow: ellipsis;\n    white-space: nowrap;\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-inline-hbox .widget-label {\n    /* Horizontal Widget Label */\n    color: var(--jp-widgets-label-color);\n    text-align: right;\n    margin-right: calc( var(--jp-widgets-inline-margin) * 2 );\n    width: var(--jp-widgets-inline-label-width);\n    flex-shrink: 0;\n}\n\n.widget-inline-vbox .widget-label {\n    /* Vertical Widget Label */\n    color: var(--jp-widgets-label-color);\n    text-align: center;\n    line-height: var(--jp-widgets-inline-height);\n}\n\n/* Widget Readout Styling */\n\n.widget-readout {\n    color: var(--jp-widgets-readout-color);\n    font-size: var(--jp-widgets-font-size);\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n    overflow: hidden;\n    white-space: nowrap;\n    text-align: center;\n}\n\n.widget-readout.overflow {\n    /* Overflowing Readout */\n\n    /* From Material Design Lite\n        shadow-key-umbra-opacity: 0.2;\n        shadow-key-penumbra-opacity: 0.14;\n        shadow-ambient-shadow-opacity: 0.12;\n     */\n    -webkit-box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.2),\n                        0 3px 1px -2px rgba(0, 0, 0, 0.14),\n                        0 1px 5px 0 rgba(0, 0, 0, 0.12);\n\n    -moz-box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.2),\n                     0 3px 1px -2px rgba(0, 0, 0, 0.14),\n                     0 1px 5px 0 rgba(0, 0, 0, 0.12);\n\n    box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.2),\n                0 3px 1px -2px rgba(0, 0, 0, 0.14),\n                0 1px 5px 0 rgba(0, 0, 0, 0.12);\n}\n\n.widget-inline-hbox .widget-readout {\n    /* Horizontal Readout */\n    text-align: center;\n    max-width: var(--jp-widgets-inline-width-short);\n    min-width: var(--jp-widgets-inline-width-tiny);\n    margin-left: var(--jp-widgets-inline-margin);\n}\n\n.widget-inline-vbox .widget-readout {\n    /* Vertical Readout */\n    margin-top: var(--jp-widgets-inline-margin);\n    /* as wide as the widget */\n    width: inherit;\n}\n\n/* Widget Checkbox Styling */\n\n.widget-checkbox {\n    width: var(--jp-widgets-inline-width);\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-checkbox input[type="checkbox"] {\n    margin: 0px calc( var(--jp-widgets-inline-margin) * 2 ) 0px 0px;\n    line-height: var(--jp-widgets-inline-height);\n    font-size: large;\n    flex-grow: 1;\n    flex-shrink: 0;\n    align-self: center;\n}\n\n/* Widget Valid Styling */\n\n.widget-valid {\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n    width: var(--jp-widgets-inline-width-short);\n    font-size: var(--jp-widgets-font-size);\n}\n\n.widget-valid i:before {\n    line-height: var(--jp-widgets-inline-height);\n    margin-right: var(--jp-widgets-inline-margin);\n    margin-left: var(--jp-widgets-inline-margin);\n\n    /* from the fa class in FontAwesome: https://github.com/FortAwesome/Font-Awesome/blob/49100c7c3a7b58d50baa71efef11af41a66b03d3/css/font-awesome.css#L14 */\n    display: inline-block;\n    font: normal normal normal 14px/1 FontAwesome;\n    font-size: inherit;\n    text-rendering: auto;\n    -webkit-font-smoothing: antialiased;\n    -moz-osx-font-smoothing: grayscale;\n}\n\n.widget-valid.mod-valid i:before {\n    content: "\\F00C";\n    color: green;\n}\n\n.widget-valid.mod-invalid i:before {\n    content: "\\F00D";\n    color: red;\n}\n\n.widget-valid.mod-valid .widget-valid-readout {\n    display: none;\n}\n\n/* Widget Text and TextArea Stying */\n\n.widget-textarea, .widget-text {\n    width: var(--jp-widgets-inline-width);\n}\n\n.widget-text input[type="text"], .widget-text input[type="number"]{\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-text input[type="text"]:disabled, .widget-text input[type="number"]:disabled, .widget-textarea textarea:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n.widget-text input[type="text"], .widget-text input[type="number"], .widget-textarea textarea {\n    box-sizing: border-box;\n    border: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n    background-color: var(--jp-widgets-input-background-color);\n    color: var(--jp-widgets-input-color);\n    font-size: var(--jp-widgets-font-size);\n    flex-grow: 1;\n    min-width: 0; /* This makes it possible for the flexbox to shrink this input */\n    flex-shrink: 1;\n    outline: none !important;\n}\n    \n.widget-text input[type="text"], .widget-textarea textarea {\n    padding: var(--jp-widgets-input-padding) calc( var(--jp-widgets-input-padding) *  2);\n}\n\n.widget-text input[type="number"] {\n    padding: var(--jp-widgets-input-padding) 0 var(--jp-widgets-input-padding) calc(var(--jp-widgets-input-padding) *  2);\n}\n\n.widget-textarea textarea {\n    height: inherit;\n    width: inherit;\n}\n\n.widget-text input:focus, .widget-textarea textarea:focus {\n    border-color: var(--jp-widgets-input-focus-border-color);\n}\n\n/* Widget Slider */\n\n.widget-slider .ui-slider {\n    /* Slider Track */\n    border: var(--jp-widgets-slider-border-width) solid var(--jp-layout-color3);\n    background: var(--jp-layout-color3);\n    box-sizing: border-box;\n    position: relative;\n    border-radius: 0px;\n}\n\n.widget-slider .ui-slider .ui-slider-handle {\n    /* Slider Handle */\n    outline: none !important; /* focused slider handles are colored - see below */\n    position: absolute;\n    background-color: var(--jp-widgets-slider-handle-background-color);\n    border: var(--jp-widgets-slider-border-width) solid var(--jp-widgets-slider-handle-border-color);\n    box-sizing: border-box;\n    z-index: 1;\n    background-image: none; /* Override jquery-ui */\n}\n\n/* Override jquery-ui */\n.widget-slider .ui-slider .ui-slider-handle:hover, .widget-slider .ui-slider .ui-slider-handle:focus {\n    background-color: var(--jp-widgets-slider-active-handle-color);\n    border: var(--jp-widgets-slider-border-width) solid var(--jp-widgets-slider-active-handle-color);\n}\n\n.widget-slider .ui-slider .ui-slider-handle:active {\n    background-color: var(--jp-widgets-slider-active-handle-color);\n    border-color: var(--jp-widgets-slider-active-handle-color);\n    z-index: 2;\n    transform: scale(1.2);\n}\n\n.widget-slider  .ui-slider .ui-slider-range {\n    /* Interval between the two specified value of a double slider */\n    position: absolute;\n    background: var(--jp-widgets-slider-active-handle-color);\n    z-index: 0;\n}\n\n/* Shapes of Slider Handles */\n\n.widget-hslider .ui-slider .ui-slider-handle {\n    width: var(--jp-widgets-slider-handle-size);\n    height: var(--jp-widgets-slider-handle-size);\n    margin-top: calc((var(--jp-widgets-slider-track-thickness) - var(--jp-widgets-slider-handle-size)) / 2 - var(--jp-widgets-slider-border-width));\n    margin-left: calc(var(--jp-widgets-slider-handle-size) / -2 + var(--jp-widgets-slider-border-width));\n    border-radius: 50%;\n    top: 0;\n}\n\n.widget-vslider .ui-slider .ui-slider-handle {\n    width: var(--jp-widgets-slider-handle-size);\n    height: var(--jp-widgets-slider-handle-size);\n    margin-bottom: calc(var(--jp-widgets-slider-handle-size) / -2 + var(--jp-widgets-slider-border-width));\n    margin-left: calc((var(--jp-widgets-slider-track-thickness) - var(--jp-widgets-slider-handle-size)) / 2 - var(--jp-widgets-slider-border-width));\n    border-radius: 50%;\n    left: 0;\n}\n\n.widget-hslider .ui-slider .ui-slider-range {\n    height: calc( var(--jp-widgets-slider-track-thickness) * 2 );\n    margin-top: calc((var(--jp-widgets-slider-track-thickness) - var(--jp-widgets-slider-track-thickness) * 2 ) / 2 - var(--jp-widgets-slider-border-width));\n}\n\n.widget-vslider .ui-slider .ui-slider-range {\n    width: calc( var(--jp-widgets-slider-track-thickness) * 2 );\n    margin-left: calc((var(--jp-widgets-slider-track-thickness) - var(--jp-widgets-slider-track-thickness) * 2 ) / 2 - var(--jp-widgets-slider-border-width));\n}\n\n/* Horizontal Slider */\n\n.widget-hslider {\n    width: var(--jp-widgets-inline-width);\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n\n    /* Override the align-items baseline. This way, the description and readout\n    still seem to align their baseline properly, and we don\'t have to have\n    align-self: stretch in the .slider-container. */\n    align-items: center;\n}\n\n.widgets-slider .slider-container {\n    overflow: visible;\n}\n\n.widget-hslider .slider-container {\n    height: var(--jp-widgets-inline-height);\n    margin-left: calc(var(--jp-widgets-slider-handle-size) / 2 - 2 * var(--jp-widgets-slider-border-width));\n    margin-right: calc(var(--jp-widgets-slider-handle-size) / 2 - 2 * var(--jp-widgets-slider-border-width));\n    flex: 1 1 var(--jp-widgets-inline-width-short);\n}\n\n.widget-hslider .ui-slider {\n    /* Inner, invisible slide div */\n    height: var(--jp-widgets-slider-track-thickness);\n    margin-top: calc((var(--jp-widgets-inline-height) - var(--jp-widgets-slider-track-thickness)) / 2);\n    width: 100%;\n}\n\n/* Vertical Slider */\n\n.widget-vbox .widget-label {\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-vslider {\n    /* Vertical Slider */\n    height: var(--jp-widgets-vertical-height);\n    width: var(--jp-widgets-inline-width-tiny);\n}\n\n.widget-vslider .slider-container {\n    flex: 1 1 var(--jp-widgets-inline-width-short);\n    margin-left: auto;\n    margin-right: auto;\n    margin-bottom: calc(var(--jp-widgets-slider-handle-size) / 2 - 2 * var(--jp-widgets-slider-border-width));\n    margin-top: calc(var(--jp-widgets-slider-handle-size) / 2 - 2 * var(--jp-widgets-slider-border-width));\n    display: flex;\n    flex-direction: column;\n}\n\n.widget-vslider .ui-slider-vertical {\n    /* Inner, invisible slide div */\n    width: var(--jp-widgets-slider-track-thickness);\n    flex-grow: 1;\n    margin-left: auto;\n    margin-right: auto;\n}\n\n/* Widget Progress Styling */\n\n.progress-bar {\n    -webkit-transition: none;\n    -moz-transition: none;\n    -ms-transition: none;\n    -o-transition: none;\n    transition: none;\n}\n\n.progress-bar {\n    height: var(--jp-widgets-inline-height);\n}\n\n.progress-bar {\n    background-color: var(--jp-brand-color1);\n}\n\n.progress-bar-success {\n    background-color: var(--jp-success-color1);\n}\n\n.progress-bar-info {\n    background-color: var(--jp-info-color1);\n}\n\n.progress-bar-warning {\n    background-color: var(--jp-warn-color1);\n}\n\n.progress-bar-danger {\n    background-color: var(--jp-error-color1);\n}\n\n.progress {\n    background-color: var(--jp-layout-color2);\n    border: none;\n    box-shadow: none;\n}\n\n/* Horisontal Progress */\n\n.widget-hprogress {\n    /* Progress Bar */\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n    width: var(--jp-widgets-inline-width);\n    align-items: center;\n\n}\n\n.widget-hprogress .progress {\n    flex-grow: 1;\n    margin-top: var(--jp-widgets-input-padding);\n    margin-bottom: var(--jp-widgets-input-padding);\n    align-self: stretch;\n    /* Override bootstrap style */\n    height: initial;\n}\n\n/* Vertical Progress */\n\n.widget-vprogress {\n    height: var(--jp-widgets-vertical-height);\n    width: var(--jp-widgets-inline-width-tiny);\n}\n\n.widget-vprogress .progress {\n    flex-grow: 1;\n    width: var(--jp-widgets-progress-thickness);\n    margin-left: auto;\n    margin-right: auto;\n    margin-bottom: 0;\n}\n\n/* Select Widget Styling */\n\n.widget-dropdown {\n    height: var(--jp-widgets-inline-height);\n    width: var(--jp-widgets-inline-width);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-dropdown > select {\n    padding-right: 20px;\n    border: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n    border-radius: 0;\n    height: inherit;\n    flex: 1 1 var(--jp-widgets-inline-width-short);\n    min-width: 0; /* This makes it possible for the flexbox to shrink this input */\n    box-sizing: border-box;\n    outline: none !important;\n    box-shadow: none;\n    background-color: var(--jp-widgets-input-background-color);\n    color: var(--jp-widgets-input-color);\n    font-size: var(--jp-widgets-font-size);\n    vertical-align: top;\n    padding-left: calc( var(--jp-widgets-input-padding) * 2);\n\tappearance: none;\n\t-webkit-appearance: none;\n\t-moz-appearance: none;\n    background-repeat: no-repeat;\n\tbackground-size: 20px;\n\tbackground-position: right center;\n    background-image: var(--jp-widgets-dropdown-arrow);\n}\n.widget-dropdown > select:focus {\n    border-color: var(--jp-widgets-input-focus-border-color);\n}\n\n.widget-dropdown > select:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n/* To disable the dotted border in Firefox around select controls.\n   See http://stackoverflow.com/a/18853002 */\n.widget-dropdown > select:-moz-focusring {\n    color: transparent;\n    text-shadow: 0 0 0 #000;\n}\n\n/* Select and SelectMultiple */\n\n.widget-select {\n    width: var(--jp-widgets-inline-width);\n    line-height: var(--jp-widgets-inline-height);\n\n    /* Because Firefox defines the baseline of a select as the bottom of the\n    control, we align the entire control to the top and add padding to the\n    select to get an approximate first line baseline alignment. */\n    align-items: flex-start;\n}\n\n.widget-select > select {\n    border: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n    background-color: var(--jp-widgets-input-background-color);\n    color: var(--jp-widgets-input-color);\n    font-size: var(--jp-widgets-font-size);\n    flex: 1 1 var(--jp-widgets-inline-width-short);\n    outline: none !important;\n    overflow: auto;\n    height: inherit;\n\n    /* Because Firefox defines the baseline of a select as the bottom of the\n    control, we align the entire control to the top and add padding to the\n    select to get an approximate first line baseline alignment. */\n    padding-top: 5px;\n}\n\n.widget-select > select:focus {\n    border-color: var(--jp-widgets-input-focus-border-color);\n}\n\n.wiget-select > select > option {\n    padding-left: var(--jp-widgets-input-padding);\n    line-height: var(--jp-widgets-inline-height);\n    /* line-height doesn\'t work on some browsers for select options */\n    padding-top: calc(var(--jp-widgets-inline-height)-var(--jp-widgets-font-size)/2);\n    padding-bottom: calc(var(--jp-widgets-inline-height)-var(--jp-widgets-font-size)/2);\n}\n\n\n\n/* Toggle Buttons Styling */\n\n.widget-toggle-buttons {\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-toggle-buttons .widget-toggle-button {\n    margin-left: var(--jp-widgets-margin);\n    margin-right: var(--jp-widgets-margin);\n}\n\n.widget-toggle-buttons .jupyter-button:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n/* Radio Buttons Styling */\n\n.widget-radio {\n    width: var(--jp-widgets-inline-width);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-radio-box {\n    display: flex;\n    flex-direction: column;\n    align-items: stretch;\n    box-sizing: border-box;\n    flex-grow: 1;\n    margin-bottom: var(--jp-widgets-radio-item-height-adjustment);\n}\n\n.widget-radio-box label {\n    height: var(--jp-widgets-radio-item-height);\n    line-height: var(--jp-widgets-radio-item-height);\n    font-size: var(--jp-widgets-font-size);\n}\n\n.widget-radio-box input {\n    height: var(--jp-widgets-radio-item-height);\n    line-height: var(--jp-widgets-radio-item-height);\n    margin: 0 calc( var(--jp-widgets-input-padding) * 2 ) 0 1px;\n    float: left;\n}\n\n/* Color Picker Styling */\n\n.widget-colorpicker {\n    width: var(--jp-widgets-inline-width);\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-colorpicker > .widget-colorpicker-input {\n    flex-grow: 1;\n    flex-shrink: 1;\n    min-width: var(--jp-widgets-inline-width-tiny);\n}\n\n.widget-colorpicker input[type="color"] {\n    width: var(--jp-widgets-inline-height);\n    height: var(--jp-widgets-inline-height);\n    padding: 0 2px; /* make the color square actually square on Chrome on OS X */\n    background: var(--jp-widgets-input-background-color);\n    color: var(--jp-widgets-input-color);\n    border: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n    border-left: none;\n    flex-grow: 0;\n    flex-shrink: 0;\n    box-sizing: border-box;\n    align-self: stretch;\n    outline: none !important;\n}\n\n.widget-colorpicker.concise input[type="color"] {\n    border-left: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n}\n\n.widget-colorpicker input[type="color"]:focus, .widget-colorpicker input[type="text"]:focus {\n    border-color: var(--jp-widgets-input-focus-border-color);\n}\n\n.widget-colorpicker input[type="text"] {\n    flex-grow: 1;\n    outline: none !important;\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n    background: var(--jp-widgets-input-background-color);\n    color: var(--jp-widgets-input-color);\n    border: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n    font-size: var(--jp-widgets-font-size);\n    padding: var(--jp-widgets-input-padding) calc( var(--jp-widgets-input-padding) *  2 );\n    min-width: 0; /* This makes it possible for the flexbox to shrink this input */\n    flex-shrink: 1;\n    box-sizing: border-box;\n}\n\n.widget-colorpicker input[type="text"]:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n/* Date Picker Styling */\n\n.widget-datepicker {\n    width: var(--jp-widgets-inline-width);\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-datepicker input[type="date"] {\n    flex-grow: 1;\n    flex-shrink: 1;\n    min-width: 0; /* This makes it possible for the flexbox to shrink this input */\n    outline: none !important;\n    height: var(--jp-widgets-inline-height);\n    border: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n    background-color: var(--jp-widgets-input-background-color);\n    color: var(--jp-widgets-input-color);\n    font-size: var(--jp-widgets-font-size);\n    padding: var(--jp-widgets-input-padding) calc( var(--jp-widgets-input-padding) *  2 );\n    box-sizing: border-box;\n}\n\n.widget-datepicker input[type="date"]:focus {\n    border-color: var(--jp-widgets-input-focus-border-color);\n}\n\n.widget-datepicker input[type="date"]:invalid {\n    border-color: var(--jp-warn-color1);\n}\n\n.widget-datepicker input[type="date"]:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n/* Play Widget */\n\n.widget-play {\n    width: var(--jp-widgets-inline-width-short);\n    display: flex;\n    align-items: stretch;\n}\n\n.widget-play .jupyter-button {\n    flex-grow: 1;\n    height: auto;\n}\n\n.widget-play .jupyter-button:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n/* Tab Widget */\n\n.jupyter-widgets.widget-tab {\n    display: flex;\n    flex-direction: column;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar {\n    /* Necessary so that a tab can be shifted down to overlay the border of the box below. */\n    overflow-x: visible;\n    overflow-y: visible;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar > .p-TabBar-content {\n    /* Make sure that the tab grows from bottom up */\n    align-items: flex-end;\n    min-width: 0;\n    min-height: 0;\n}\n\n.jupyter-widgets.widget-tab > .widget-tab-contents {\n    width: 100%;\n    box-sizing: border-box;\n    margin: 0;\n    background: var(--jp-layout-color1);\n    color: var(--jp-ui-font-color1);\n    border: var(--jp-border-width) solid var(--jp-border-color1);\n    padding: var(--jp-widgets-container-padding);\n    flex-grow: 1;\n    overflow: auto;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar {\n    font: var(--jp-widgets-font-size) Helvetica, Arial, sans-serif;\n    min-height: calc(var(--jp-widgets-horizontal-tab-height) + var(--jp-border-width));\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab {\n    flex: 0 1 var(--jp-widgets-horizontal-tab-width);\n    min-width: 35px;\n    min-height: calc(var(--jp-widgets-horizontal-tab-height) + var(--jp-border-width));\n    line-height: var(--jp-widgets-horizontal-tab-height);\n    margin-left: calc(-1 * var(--jp-border-width));\n    padding: 0px 10px;\n    background: var(--jp-layout-color2);\n    color: var(--jp-ui-font-color2);\n    border: var(--jp-border-width) solid var(--jp-border-color1);\n    border-bottom: none;\n    position: relative;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab.p-mod-current {\n    color: var(--jp-ui-font-color0);\n    /* We want the background to match the tab content background */\n    background: var(--jp-layout-color1);\n    min-height: calc(var(--jp-widgets-horizontal-tab-height) + 2 * var(--jp-border-width));\n    transform: translateY(var(--jp-border-width));\n    overflow: visible;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab.p-mod-current:before {\n    position: absolute;\n    top: calc(-1 * var(--jp-border-width));\n    left: calc(-1 * var(--jp-border-width));\n    content: \'\';\n    height: var(--jp-widgets-horizontal-tab-top-border);\n    width: calc(100% + 2 * var(--jp-border-width));\n    background: var(--jp-brand-color1);\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab:first-child {\n    margin-left: 0;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab:hover:not(.p-mod-current) {\n    background: var(--jp-layout-color1);\n    color: var(--jp-ui-font-color1);\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-mod-closable > .p-TabBar-tabCloseIcon {\n    margin-left: 4px;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-mod-closable > .p-TabBar-tabCloseIcon:before {\n    font-family: FontAwesome;\n    content: \'\\F00D\'; /* close */\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tabIcon,\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tabLabel,\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tabCloseIcon {\n    line-height: var(--jp-widgets-horizontal-tab-height);\n}\n\n/* Accordion Widget */\n\n.p-Collapse {\n    display: flex;\n    flex-direction: column;\n    align-items: stretch;\n}\n\n.p-Collapse-header {\n    padding: var(--jp-widgets-input-padding);\n    cursor: pointer;\n    color: var(--jp-ui-font-color2);\n    background-color: var(--jp-layout-color2);\n    border: var(--jp-widgets-border-width) solid var(--jp-border-color1);\n    padding: calc(var(--jp-widgets-container-padding) * 2 / 3) var(--jp-widgets-container-padding);\n    font-weight: bold;\n}\n\n.p-Collapse-header:hover {\n    background-color: var(--jp-layout-color1);\n    color: var(--jp-ui-font-color1);\n}\n\n.p-Collapse-open > .p-Collapse-header {\n    background-color: var(--jp-layout-color1);\n    color: var(--jp-ui-font-color0);\n    cursor: default;\n    border-bottom: none;\n}\n\n.p-Collapse .p-Collapse-header::before {\n    content: \'\\F0DA\\A0\';  /* caret-right, non-breaking space */\n    display: inline-block;\n    font: normal normal normal 14px/1 FontAwesome;\n    font-size: inherit;\n    text-rendering: auto;\n    -webkit-font-smoothing: antialiased;\n    -moz-osx-font-smoothing: grayscale;\n}\n\n.p-Collapse-open > .p-Collapse-header::before {\n    content: \'\\F0D7\\A0\'; /* caret-down, non-breaking space */\n}\n\n.p-Collapse-contents {\n    padding: var(--jp-widgets-container-padding);\n    background-color: var(--jp-layout-color1);\n    color: var(--jp-ui-font-color1);\n    border-left: var(--jp-widgets-border-width) solid var(--jp-border-color1);\n    border-right: var(--jp-widgets-border-width) solid var(--jp-border-color1);\n    border-bottom: var(--jp-widgets-border-width) solid var(--jp-border-color1);\n    overflow: auto;\n}\n\n.p-Accordion {\n    display: flex;\n    flex-direction: column;\n    align-items: stretch;\n}\n\n.p-Accordion .p-Collapse {\n    margin-bottom: 0;\n}\n\n.p-Accordion .p-Collapse + .p-Collapse {\n    margin-top: 4px;\n}\n\n\n\n/* HTML widget */\n\n.widget-html, .widget-htmlmath {\n    font-size: var(--jp-widgets-font-size);\n}\n\n.widget-html > .widget-html-content, .widget-htmlmath > .widget-html-content {\n    /* Fill out the area in the HTML widget */\n    align-self: stretch;\n    flex-grow: 1;\n    flex-shrink: 1;\n    /* Makes sure the baseline is still aligned with other elements */\n    line-height: var(--jp-widgets-inline-height);\n    /* Make it possible to have absolutely-positioned elements in the html */\n    position: relative;\n}\n\n\n/* Image widget  */\n\n.widget-image {\n    max-width: 100%;\n    height: auto;\n}\n',""])},function(t,e,n){(t.exports=n(12)(!1)).push([t.i,"/* This file has code derived from PhosphorJS CSS files, as noted below. The license for this PhosphorJS code is:\n\nCopyright (c) 2014-2017, PhosphorJS Contributors\nAll rights reserved.\n\nRedistribution and use in source and binary forms, with or without\nmodification, are permitted provided that the following conditions are met:\n\n* Redistributions of source code must retain the above copyright notice, this\n  list of conditions and the following disclaimer.\n\n* Redistributions in binary form must reproduce the above copyright notice,\n  this list of conditions and the following disclaimer in the documentation\n  and/or other materials provided with the distribution.\n\n* Neither the name of the copyright holder nor the names of its\n  contributors may be used to endorse or promote products derived from\n  this software without specific prior written permission.\n\nTHIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS \"AS IS\"\nAND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE\nIMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE\nDISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE\nFOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL\nDAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR\nSERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER\nCAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,\nOR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE\nOF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.\n\n*/\n\n/*\n * The following section is derived from https://github.com/phosphorjs/phosphor/blob/23b9d075ebc5b73ab148b6ebfc20af97f85714c4/packages/widgets/style/tabbar.css \n * We've scoped the rules so that they are consistent with exactly our code.\n */\n\n.jupyter-widgets.widget-tab > .p-TabBar {\n  display: flex;\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar[data-orientation='horizontal'] {\n  flex-direction: row;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar[data-orientation='vertical'] {\n  flex-direction: column;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar > .p-TabBar-content {\n  margin: 0;\n  padding: 0;\n  display: flex;\n  flex: 1 1 auto;\n  list-style-type: none;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar[data-orientation='horizontal'] > .p-TabBar-content {\n  flex-direction: row;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar[data-orientation='vertical'] > .p-TabBar-content {\n  flex-direction: column;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab {\n  display: flex;\n  flex-direction: row;\n  box-sizing: border-box;\n  overflow: hidden;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tabIcon,\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tabCloseIcon {\n  flex: 0 0 auto;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tabLabel {\n  flex: 1 1 auto;\n  overflow: hidden;\n  white-space: nowrap;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab.p-mod-hidden {\n  display: none !important;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar.p-mod-dragging .p-TabBar-tab {\n  position: relative;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar.p-mod-dragging[data-orientation='horizontal'] .p-TabBar-tab {\n  left: 0;\n  transition: left 150ms ease;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar.p-mod-dragging[data-orientation='vertical'] .p-TabBar-tab {\n  top: 0;\n  transition: top 150ms ease;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar.p-mod-dragging .p-TabBar-tab.p-mod-dragging {\n  transition: none;\n}\n\n/* End tabbar.css */\n",""])},function(t,e,n){var r,i,o;
/*!
 * jQuery UI Slider 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */i=[n(44),n(1411),n(1413),n(184),n(703)],void 0===(o="function"==typeof(r=function(t){return t.widget("ui.slider",t.ui.mouse,{version:"1.12.1",widgetEventPrefix:"slide",options:{animate:!1,classes:{"ui-slider":"ui-corner-all","ui-slider-handle":"ui-corner-all","ui-slider-range":"ui-corner-all ui-widget-header"},distance:0,max:100,min:0,orientation:"horizontal",range:!1,step:1,value:0,values:null,change:null,slide:null,start:null,stop:null},numPages:5,_create:function(){this._keySliding=!1,this._mouseSliding=!1,this._animateOff=!0,this._handleIndex=null,this._detectOrientation(),this._mouseInit(),this._calculateNewMax(),this._addClass("ui-slider ui-slider-"+this.orientation,"ui-widget ui-widget-content"),this._refresh(),this._animateOff=!1},_refresh:function(){this._createRange(),this._createHandles(),this._setupEvents(),this._refreshValue()},_createHandles:function(){var e,n,r=this.options,i=this.element.find(".ui-slider-handle"),o=[];for(n=r.values&&r.values.length||1,i.length>n&&(i.slice(n).remove(),i=i.slice(0,n)),e=i.length;e<n;e++)o.push("<span tabindex='0'></span>");this.handles=i.add(t(o.join("")).appendTo(this.element)),this._addClass(this.handles,"ui-slider-handle","ui-state-default"),this.handle=this.handles.eq(0),this.handles.each(function(e){t(this).data("ui-slider-handle-index",e).attr("tabIndex",0)})},_createRange:function(){var e=this.options;e.range?(!0===e.range&&(e.values?e.values.length&&2!==e.values.length?e.values=[e.values[0],e.values[0]]:t.isArray(e.values)&&(e.values=e.values.slice(0)):e.values=[this._valueMin(),this._valueMin()]),this.range&&this.range.length?(this._removeClass(this.range,"ui-slider-range-min ui-slider-range-max"),this.range.css({left:"",bottom:""})):(this.range=t("<div>").appendTo(this.element),this._addClass(this.range,"ui-slider-range")),"min"!==e.range&&"max"!==e.range||this._addClass(this.range,"ui-slider-range-"+e.range)):(this.range&&this.range.remove(),this.range=null)},_setupEvents:function(){this._off(this.handles),this._on(this.handles,this._handleEvents),this._hoverable(this.handles),this._focusable(this.handles)},_destroy:function(){this.handles.remove(),this.range&&this.range.remove(),this._mouseDestroy()},_mouseCapture:function(e){var n,r,i,o,a,s,A,l=this,c=this.options;return!c.disabled&&(this.elementSize={width:this.element.outerWidth(),height:this.element.outerHeight()},this.elementOffset=this.element.offset(),n={x:e.pageX,y:e.pageY},r=this._normValueFromMouse(n),i=this._valueMax()-this._valueMin()+1,this.handles.each(function(e){var n=Math.abs(r-l.values(e));(i>n||i===n&&(e===l._lastChangedValue||l.values(e)===c.min))&&(i=n,o=t(this),a=e)}),!1!==this._start(e,a)&&(this._mouseSliding=!0,this._handleIndex=a,this._addClass(o,null,"ui-state-active"),o.trigger("focus"),s=o.offset(),A=!t(e.target).parents().addBack().is(".ui-slider-handle"),this._clickOffset=A?{left:0,top:0}:{left:e.pageX-s.left-o.width()/2,top:e.pageY-s.top-o.height()/2-(parseInt(o.css("borderTopWidth"),10)||0)-(parseInt(o.css("borderBottomWidth"),10)||0)+(parseInt(o.css("marginTop"),10)||0)},this.handles.hasClass("ui-state-hover")||this._slide(e,a,r),this._animateOff=!0,!0))},_mouseStart:function(){return!0},_mouseDrag:function(t){var e={x:t.pageX,y:t.pageY},n=this._normValueFromMouse(e);return this._slide(t,this._handleIndex,n),!1},_mouseStop:function(t){return this._removeClass(this.handles,null,"ui-state-active"),this._mouseSliding=!1,this._stop(t,this._handleIndex),this._change(t,this._handleIndex),this._handleIndex=null,this._clickOffset=null,this._animateOff=!1,!1},_detectOrientation:function(){this.orientation="vertical"===this.options.orientation?"vertical":"horizontal"},_normValueFromMouse:function(t){var e,n,r,i,o;return"horizontal"===this.orientation?(e=this.elementSize.width,n=t.x-this.elementOffset.left-(this._clickOffset?this._clickOffset.left:0)):(e=this.elementSize.height,n=t.y-this.elementOffset.top-(this._clickOffset?this._clickOffset.top:0)),(r=n/e)>1&&(r=1),r<0&&(r=0),"vertical"===this.orientation&&(r=1-r),i=this._valueMax()-this._valueMin(),o=this._valueMin()+r*i,this._trimAlignValue(o)},_uiHash:function(t,e,n){var r={handle:this.handles[t],handleIndex:t,value:void 0!==e?e:this.value()};return this._hasMultipleValues()&&(r.value=void 0!==e?e:this.values(t),r.values=n||this.values()),r},_hasMultipleValues:function(){return this.options.values&&this.options.values.length},_start:function(t,e){return this._trigger("start",t,this._uiHash(e))},_slide:function(t,e,n){var r,i=this.value(),o=this.values();this._hasMultipleValues()&&(r=this.values(e?0:1),i=this.values(e),2===this.options.values.length&&!0===this.options.range&&(n=0===e?Math.min(r,n):Math.max(r,n)),o[e]=n),n!==i&&!1!==this._trigger("slide",t,this._uiHash(e,n,o))&&(this._hasMultipleValues()?this.values(e,n):this.value(n))},_stop:function(t,e){this._trigger("stop",t,this._uiHash(e))},_change:function(t,e){this._keySliding||this._mouseSliding||(this._lastChangedValue=e,this._trigger("change",t,this._uiHash(e)))},value:function(t){return arguments.length?(this.options.value=this._trimAlignValue(t),this._refreshValue(),void this._change(null,0)):this._value()},values:function(e,n){var r,i,o;if(arguments.length>1)return this.options.values[e]=this._trimAlignValue(n),this._refreshValue(),void this._change(null,e);if(!arguments.length)return this._values();if(!t.isArray(arguments[0]))return this._hasMultipleValues()?this._values(e):this.value();for(r=this.options.values,i=arguments[0],o=0;o<r.length;o+=1)r[o]=this._trimAlignValue(i[o]),this._change(null,o);this._refreshValue()},_setOption:function(e,n){var r,i=0;switch("range"===e&&!0===this.options.range&&("min"===n?(this.options.value=this._values(0),this.options.values=null):"max"===n&&(this.options.value=this._values(this.options.values.length-1),this.options.values=null)),t.isArray(this.options.values)&&(i=this.options.values.length),this._super(e,n),e){case"orientation":this._detectOrientation(),this._removeClass("ui-slider-horizontal ui-slider-vertical")._addClass("ui-slider-"+this.orientation),this._refreshValue(),this.options.range&&this._refreshRange(n),this.handles.css("horizontal"===n?"bottom":"left","");break;case"value":this._animateOff=!0,this._refreshValue(),this._change(null,0),this._animateOff=!1;break;case"values":for(this._animateOff=!0,this._refreshValue(),r=i-1;r>=0;r--)this._change(null,r);this._animateOff=!1;break;case"step":case"min":case"max":this._animateOff=!0,this._calculateNewMax(),this._refreshValue(),this._animateOff=!1;break;case"range":this._animateOff=!0,this._refresh(),this._animateOff=!1}},_setOptionDisabled:function(t){this._super(t),this._toggleClass(null,"ui-state-disabled",!!t)},_value:function(){var t=this.options.value;return t=this._trimAlignValue(t)},_values:function(t){var e,n,r;if(arguments.length)return e=this.options.values[t],e=this._trimAlignValue(e);if(this._hasMultipleValues()){for(n=this.options.values.slice(),r=0;r<n.length;r+=1)n[r]=this._trimAlignValue(n[r]);return n}return[]},_trimAlignValue:function(t){if(t<=this._valueMin())return this._valueMin();if(t>=this._valueMax())return this._valueMax();var e=this.options.step>0?this.options.step:1,n=(t-this._valueMin())%e,r=t-n;return 2*Math.abs(n)>=e&&(r+=n>0?e:-e),parseFloat(r.toFixed(5))},_calculateNewMax:function(){var t=this.options.max,e=this._valueMin(),n=this.options.step,r=Math.round((t-e)/n)*n;(t=r+e)>this.options.max&&(t-=n),this.max=parseFloat(t.toFixed(this._precision()))},_precision:function(){var t=this._precisionOf(this.options.step);return null!==this.options.min&&(t=Math.max(t,this._precisionOf(this.options.min))),t},_precisionOf:function(t){var e=t.toString(),n=e.indexOf(".");return-1===n?0:e.length-n-1},_valueMin:function(){return this.options.min},_valueMax:function(){return this.max},_refreshRange:function(t){"vertical"===t&&this.range.css({width:"",left:""}),"horizontal"===t&&this.range.css({height:"",bottom:""})},_refreshValue:function(){var e,n,r,i,o,a=this.options.range,s=this.options,A=this,l=!this._animateOff&&s.animate,c={};this._hasMultipleValues()?this.handles.each(function(r){n=(A.values(r)-A._valueMin())/(A._valueMax()-A._valueMin())*100,c["horizontal"===A.orientation?"left":"bottom"]=n+"%",t(this).stop(1,1)[l?"animate":"css"](c,s.animate),!0===A.options.range&&("horizontal"===A.orientation?(0===r&&A.range.stop(1,1)[l?"animate":"css"]({left:n+"%"},s.animate),1===r&&A.range[l?"animate":"css"]({width:n-e+"%"},{queue:!1,duration:s.animate})):(0===r&&A.range.stop(1,1)[l?"animate":"css"]({bottom:n+"%"},s.animate),1===r&&A.range[l?"animate":"css"]({height:n-e+"%"},{queue:!1,duration:s.animate}))),e=n}):(r=this.value(),i=this._valueMin(),o=this._valueMax(),n=o!==i?(r-i)/(o-i)*100:0,c["horizontal"===this.orientation?"left":"bottom"]=n+"%",this.handle.stop(1,1)[l?"animate":"css"](c,s.animate),"min"===a&&"horizontal"===this.orientation&&this.range.stop(1,1)[l?"animate":"css"]({width:n+"%"},s.animate),"max"===a&&"horizontal"===this.orientation&&this.range.stop(1,1)[l?"animate":"css"]({width:100-n+"%"},s.animate),"min"===a&&"vertical"===this.orientation&&this.range.stop(1,1)[l?"animate":"css"]({height:n+"%"},s.animate),"max"===a&&"vertical"===this.orientation&&this.range.stop(1,1)[l?"animate":"css"]({height:100-n+"%"},s.animate))},_handleEvents:{keydown:function(e){var n,r,i,o=t(e.target).data("ui-slider-handle-index");switch(e.keyCode){case t.ui.keyCode.HOME:case t.ui.keyCode.END:case t.ui.keyCode.PAGE_UP:case t.ui.keyCode.PAGE_DOWN:case t.ui.keyCode.UP:case t.ui.keyCode.RIGHT:case t.ui.keyCode.DOWN:case t.ui.keyCode.LEFT:if(e.preventDefault(),!this._keySliding&&(this._keySliding=!0,this._addClass(t(e.target),null,"ui-state-active"),!1===this._start(e,o)))return}switch(i=this.options.step,n=r=this._hasMultipleValues()?this.values(o):this.value(),e.keyCode){case t.ui.keyCode.HOME:r=this._valueMin();break;case t.ui.keyCode.END:r=this._valueMax();break;case t.ui.keyCode.PAGE_UP:r=this._trimAlignValue(n+(this._valueMax()-this._valueMin())/this.numPages);break;case t.ui.keyCode.PAGE_DOWN:r=this._trimAlignValue(n-(this._valueMax()-this._valueMin())/this.numPages);break;case t.ui.keyCode.UP:case t.ui.keyCode.RIGHT:if(n===this._valueMax())return;r=this._trimAlignValue(n+i);break;case t.ui.keyCode.DOWN:case t.ui.keyCode.LEFT:if(n===this._valueMin())return;r=this._trimAlignValue(n-i)}this._slide(e,o,r)},keyup:function(e){var n=t(e.target).data("ui-slider-handle-index");this._keySliding&&(this._keySliding=!1,this._stop(e,n),this._change(e,n),this._removeClass(t(e.target),null,"ui-state-active"))}}})})?r.apply(e,i):r)||(t.exports=o)},function(t,e,n){var r,i,o;
/*!
 * jQuery UI Mouse 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */i=[n(44),n(1412),n(184),n(703)],void 0===(o="function"==typeof(r=function(t){var e=!1;return t(document).on("mouseup",function(){e=!1}),t.widget("ui.mouse",{version:"1.12.1",options:{cancel:"input, textarea, button, select, option",distance:1,delay:0},_mouseInit:function(){var e=this;this.element.on("mousedown."+this.widgetName,function(t){return e._mouseDown(t)}).on("click."+this.widgetName,function(n){if(!0===t.data(n.target,e.widgetName+".preventClickEvent"))return t.removeData(n.target,e.widgetName+".preventClickEvent"),n.stopImmediatePropagation(),!1}),this.started=!1},_mouseDestroy:function(){this.element.off("."+this.widgetName),this._mouseMoveDelegate&&this.document.off("mousemove."+this.widgetName,this._mouseMoveDelegate).off("mouseup."+this.widgetName,this._mouseUpDelegate)},_mouseDown:function(n){if(!e){this._mouseMoved=!1,this._mouseStarted&&this._mouseUp(n),this._mouseDownEvent=n;var r=this,i=1===n.which,o=!("string"!=typeof this.options.cancel||!n.target.nodeName)&&t(n.target).closest(this.options.cancel).length;return!(i&&!o&&this._mouseCapture(n)&&(this.mouseDelayMet=!this.options.delay,this.mouseDelayMet||(this._mouseDelayTimer=setTimeout(function(){r.mouseDelayMet=!0},this.options.delay)),this._mouseDistanceMet(n)&&this._mouseDelayMet(n)&&(this._mouseStarted=!1!==this._mouseStart(n),!this._mouseStarted)?(n.preventDefault(),0):(!0===t.data(n.target,this.widgetName+".preventClickEvent")&&t.removeData(n.target,this.widgetName+".preventClickEvent"),this._mouseMoveDelegate=function(t){return r._mouseMove(t)},this._mouseUpDelegate=function(t){return r._mouseUp(t)},this.document.on("mousemove."+this.widgetName,this._mouseMoveDelegate).on("mouseup."+this.widgetName,this._mouseUpDelegate),n.preventDefault(),e=!0,0)))}},_mouseMove:function(e){if(this._mouseMoved){if(t.ui.ie&&(!document.documentMode||document.documentMode<9)&&!e.button)return this._mouseUp(e);if(!e.which)if(e.originalEvent.altKey||e.originalEvent.ctrlKey||e.originalEvent.metaKey||e.originalEvent.shiftKey)this.ignoreMissingWhich=!0;else if(!this.ignoreMissingWhich)return this._mouseUp(e)}return(e.which||e.button)&&(this._mouseMoved=!0),this._mouseStarted?(this._mouseDrag(e),e.preventDefault()):(this._mouseDistanceMet(e)&&this._mouseDelayMet(e)&&(this._mouseStarted=!1!==this._mouseStart(this._mouseDownEvent,e),this._mouseStarted?this._mouseDrag(e):this._mouseUp(e)),!this._mouseStarted)},_mouseUp:function(n){this.document.off("mousemove."+this.widgetName,this._mouseMoveDelegate).off("mouseup."+this.widgetName,this._mouseUpDelegate),this._mouseStarted&&(this._mouseStarted=!1,n.target===this._mouseDownEvent.target&&t.data(n.target,this.widgetName+".preventClickEvent",!0),this._mouseStop(n)),this._mouseDelayTimer&&(clearTimeout(this._mouseDelayTimer),delete this._mouseDelayTimer),this.ignoreMissingWhich=!1,e=!1,n.preventDefault()},_mouseDistanceMet:function(t){return Math.max(Math.abs(this._mouseDownEvent.pageX-t.pageX),Math.abs(this._mouseDownEvent.pageY-t.pageY))>=this.options.distance},_mouseDelayMet:function(){return this.mouseDelayMet},_mouseStart:function(){},_mouseDrag:function(){},_mouseStop:function(){},_mouseCapture:function(){return!0}})})?r.apply(e,i):r)||(t.exports=o)},function(t,e,n){var r,i,o;i=[n(44),n(184)],void 0===(o="function"==typeof(r=function(t){return t.ui.ie=!!/msie [\w.]+/.exec(navigator.userAgent.toLowerCase())})?r.apply(e,i):r)||(t.exports=o)},function(t,e,n){var r,i,o;
/*!
 * jQuery UI Keycode 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */i=[n(44),n(184)],void 0===(o="function"==typeof(r=function(t){return t.ui.keyCode={BACKSPACE:8,COMMA:188,DELETE:46,DOWN:40,END:35,ENTER:13,ESCAPE:27,HOME:36,LEFT:37,PAGE_DOWN:34,PAGE_UP:33,PERIOD:190,RIGHT:39,SPACE:32,TAB:9,UP:38}})?r.apply(e,i):r)||(t.exports=o)},function(t){t.exports=JSON.parse('{"name":"@jupyter-widgets/controls","version":"1.5.2","description":"Jupyter interactive widgets","repository":{"type":"git","url":"https://github.com/jupyter-widgets/ipywidgets.git"},"license":"BSD-3-Clause","author":"Project Jupyter","files":["lib/**/*.d.ts","lib/**/*.js","css/*.css","dist/"],"main":"lib/index.js","typings":"lib/index.d.ts","scripts":{"build":"npm run build:src && npm run build:css","build:css":"postcss --use postcss-import --use postcss-cssnext -o css/widgets.built.css css/widgets.css","build:src":"tsc","build:test":"tsc --project test && webpack --config test/webpack.conf.js","clean":"npm run clean:src","clean:src":"rimraf lib && rimraf tsconfig.tsbuildinfo","lint":"tslint --project tslint.json --format stylish","prepublish":"npm run clean && npm run build","test":"npm run test:unit","test:coverage":"npm run build:test && webpack --config test/webpack-cov.conf.js && karma start test/karma-cov.conf.js","test:unit":"npm run test:unit:firefox && npm run test:unit:chrome","test:unit:chrome":"npm run test:unit:default -- --browsers=Chrome","test:unit:default":"npm run build:test && karma start test/karma.conf.js --log-level debug","test:unit:firefox":"npm run test:unit:default -- --browsers=Firefox","test:unit:ie":"npm run test:unit:default -- --browsers=IE"},"dependencies":{"@jupyter-widgets/base":"^1.0.0 || ^2.0.0","@phosphor/algorithm":"^1.1.0","@phosphor/domutils":"^1.1.0","@phosphor/messaging":"^1.2.1","@phosphor/signaling":"^1.2.0","@phosphor/widgets":"^1.3.0","d3-format":"^1.3.0","jquery":"^3.1.1","jquery-ui":"^1.12.1","underscore":"^1.8.3"},"devDependencies":{"@jupyterlab/services":"^2.0.0 || ^3.0.0 || ^4.0.0","@types/d3-format":"^1.3.1","@types/expect.js":"^0.3.29","@types/mathjax":"^0.0.35","@types/mocha":"^5.2.7","@types/node":"^12.0.10","chai":"^4.0.0","css-loader":"^3.0.0","expect.js":"^0.3.1","file-loader":"^4.0.0","istanbul-instrumenter-loader":"^3.0.1","json-loader":"^0.5.7","karma":"^4.1.0","karma-chrome-launcher":"^2.2.0","karma-coverage":"^1.1.2","karma-firefox-launcher":"^1.1.0","karma-ie-launcher":"^1.0.0","karma-mocha":"^1.3.0","karma-mocha-reporter":"^2.2.5","karma-webpack":"^4.0.2","mocha":"^6.1.4","npm-run-all":"^4.1.5","postcss-cli":"^6.1.2","postcss-cssnext":"^3.1.0","postcss-import":"^12.0.1","postcss-loader":"^3.0.0","rimraf":"^2.6.1","sinon":"^7.3.2","sinon-chai":"^3.3.0","style-loader":"^0.23.1","tslint":"^5.18.0","typescript":"~3.5.2","url-loader":"^2.0.0","webpack":"^4.35.0"},"gitHead":"4bbe07f28042b218f8566dd0df60e92e2b38b29b"}')},function(t,e,n){t.exports=n(1416),t.exports.version=n(712).version},function(t,e,n){var r=n(20),i=n(108);window.PlotlyConfig={MathJaxConfig:"local"};var o=n(1417),a=n(1418),s="^"+n(712).version,A=r.DOMWidgetModel.extend({defaults:i.extend(r.DOMWidgetModel.prototype.defaults(),{_model_name:"FigureModel",_view_name:"FigureView",_model_module:"plotlywidget",_view_module:"plotlywidget",_view_module_version:s,_model_module_version:s,_data:[],_layout:{},_config:{},_py2js_addTraces:null,_py2js_deleteTraces:null,_py2js_moveTraces:null,_py2js_restyle:null,_py2js_relayout:null,_py2js_update:null,_py2js_animate:null,_py2js_removeLayoutProps:null,_py2js_removeTraceProps:null,_js2py_restyle:null,_js2py_relayout:null,_js2py_update:null,_js2py_layoutDelta:null,_js2py_traceDeltas:null,_js2py_pointsCallback:null,_last_layout_edit_id:0,_last_trace_edit_id:0}),initialize:function(){A.__super__.initialize.apply(this,arguments),this.on("change:_data",this.do_data,this),this.on("change:_layout",this.do_layout,this),this.on("change:_py2js_addTraces",this.do_addTraces,this),this.on("change:_py2js_deleteTraces",this.do_deleteTraces,this),this.on("change:_py2js_moveTraces",this.do_moveTraces,this),this.on("change:_py2js_restyle",this.do_restyle,this),this.on("change:_py2js_relayout",this.do_relayout,this),this.on("change:_py2js_update",this.do_update,this),this.on("change:_py2js_animate",this.do_animate,this),this.on("change:_py2js_removeLayoutProps",this.do_removeLayoutProps,this),this.on("change:_py2js_removeTraceProps",this.do_removeTraceProps,this)},_normalize_trace_indexes:function(t){if(null==t){var e=this.get("_data").length;t=i.range(e)}return Array.isArray(t)||(t=[t]),t},do_data:function(){},do_layout:function(){},do_addTraces:function(){var t=this.get("_py2js_addTraces");if(null!==t){var e=this.get("_data"),n=t.trace_data;i.forEach(n,function(t){e.push(t)})}},do_deleteTraces:function(){var t=this.get("_py2js_deleteTraces");if(null!==t){var e=t.delete_inds,n=this.get("_data");e.slice().reverse().forEach(function(t){n.splice(t,1)})}},do_moveTraces:function(){var t=this.get("_py2js_moveTraces");null!==t&&function(t,e,n){for(var r=[],o=e.length-1;o>=0;o--)r.splice(0,0,t[e[o]]),t.splice(e[o],1);var a=i(n).zip(r).sortBy(0).unzip().value();n=a[0],r=a[1];for(var s=0;s<n.length;s++)t.splice(n[s],0,r[s])}(this.get("_data"),t.current_trace_inds,t.new_trace_inds)},do_restyle:function(){var t=this.get("_py2js_restyle");if(null!==t){var e=t.restyle_data,n=this._normalize_trace_indexes(t.restyle_traces);f(this.get("_data"),e,n)}},do_relayout:function(){var t=this.get("_py2js_relayout");null!==t&&d(this.get("_layout"),t.relayout_data)},do_update:function(){var t=this.get("_py2js_update");if(null!==t){var e=t.style_data,n=t.layout_data,r=this._normalize_trace_indexes(t.style_traces);f(this.get("_data"),e,r),d(this.get("_layout"),n)}},do_animate:function(){var t=this.get("_py2js_animate");if(null!==t){for(var e=t.style_data,n=t.layout_data,r=this._normalize_trace_indexes(t.style_traces),i=0;i<e.length;i++){var o=e[i],a=r[i];d(this.get("_data")[a],o)}d(this.get("_layout"),n)}},do_removeLayoutProps:function(){var t=this.get("_py2js_removeLayoutProps");if(null!==t){var e=t.remove_props;p(this.get("_layout"),e)}},do_removeTraceProps:function(){var t=this.get("_py2js_removeTraceProps");if(null!==t){var e=t.remove_props,n=t.remove_trace;p(this.get("_data")[n],e)}}},{serializers:i.extend({_data:{deserialize:g,serialize:u},_layout:{deserialize:g,serialize:u},_py2js_addTraces:{deserialize:g,serialize:u},_py2js_deleteTraces:{deserialize:g,serialize:u},_py2js_moveTraces:{deserialize:g,serialize:u},_py2js_restyle:{deserialize:g,serialize:u},_py2js_relayout:{deserialize:g,serialize:u},_py2js_update:{deserialize:g,serialize:u},_py2js_animate:{deserialize:g,serialize:u},_py2js_removeLayoutProps:{deserialize:g,serialize:u},_py2js_removeTraceProps:{deserialize:g,serialize:u},_js2py_restyle:{deserialize:g,serialize:u},_js2py_relayout:{deserialize:g,serialize:u},_js2py_update:{deserialize:g,serialize:u},_js2py_layoutDelta:{deserialize:g,serialize:u},_js2py_traceDeltas:{deserialize:g,serialize:u},_js2py_pointsCallback:{deserialize:g,serialize:u}},r.DOMWidgetModel.serializers)}),l=r.DOMWidgetView.extend({perform_render:function(){var t=this;this.model.on("change:_py2js_addTraces",this.do_addTraces,this),this.model.on("change:_py2js_deleteTraces",this.do_deleteTraces,this),this.model.on("change:_py2js_moveTraces",this.do_moveTraces,this),this.model.on("change:_py2js_restyle",this.do_restyle,this),this.model.on("change:_py2js_relayout",this.do_relayout,this),this.model.on("change:_py2js_update",this.do_update,this),this.model.on("change:_py2js_animate",this.do_animate,this),window.MathJax&&MathJax.Hub.Config({SVG:{font:"STIX-Web"}});var e=this.model.get("_last_layout_edit_id"),n=this.model.get("_last_trace_edit_id");this.viewID=a.randstr();var r=i.cloneDeep(this.model.get("_data")),s=i.cloneDeep(this.model.get("_layout")),A=this.model.get("_config");o.newPlot(t.el,r,s,A).then(function(){t._sendTraceDeltas(n),t._sendLayoutDelta(e),t.el.on("plotly_restyle",function(e){t.handle_plotly_restyle(e)}),t.el.on("plotly_relayout",function(e){t.handle_plotly_relayout(e)}),t.el.on("plotly_update",function(e){t.handle_plotly_update(e)}),t.el.on("plotly_click",function(e){t.handle_plotly_click(e)}),t.el.on("plotly_hover",function(e){t.handle_plotly_hover(e)}),t.el.on("plotly_unhover",function(e){t.handle_plotly_unhover(e)}),t.el.on("plotly_selected",function(e){t.handle_plotly_selected(e)}),t.el.on("plotly_deselect",function(e){t.handle_plotly_deselect(e)}),t.el.on("plotly_doubleclick",function(e){t.handle_plotly_doubleclick(e)});var r=new CustomEvent("plotlywidget-after-render",{detail:{element:t.el,viewID:t.viewID}});document.dispatchEvent(r)})},processPhosphorMessage:function(t){l.__super__.processPhosphorMessage.apply(this,arguments);var e=this;switch(t.type){case"before-attach":var n={showgrid:!1,showline:!1,tickvals:[]};o.newPlot(e.el,[],{xaxis:n,yaxis:n}),window.addEventListener("resize",function(){e.autosizeFigure()});break;case"after-attach":this.perform_render();break;case"resize":this.autosizeFigure()}},autosizeFigure:function(){var t=this,e=t.model.get("_layout");(i.isNil(e)||i.isNil(e.width))&&o.Plots.resize(t.el).then(function(){var e=t.model.get("_last_layout_edit_id");t._sendLayoutDelta(e)})},destroy:function(){o.purge(this.el)},getFullData:function(){return i.mergeWith({},this.el._fullData,this.el.data,h)},getFullLayout:function(){return i.mergeWith({},this.el._fullLayout,this.el.layout,h)},buildPointsObject:function(t){var e;if(t.hasOwnProperty("points")){var n=t.points,r=n.length;e={trace_indexes:new Array(r),point_indexes:new Array(r),xs:new Array(r),ys:new Array(r)};for(var i=0;i<r;i++)e.trace_indexes[i]=n[i].curveNumber,e.point_indexes[i]=n[i].pointNumber,e.xs[i]=n[i].x,e.ys[i]=n[i].y;if(void 0!==n[0]&&n[0].hasOwnProperty("z"))for(e.zs=new Array(r),i=0;i<r;i++)e.zs[i]=n[i].z;return e}return null},buildInputDeviceStateObject:function(t){var e=t.event;return void 0===e?null:{alt:e.altKey,ctrl:e.ctrlKey,meta:e.metaKey,shift:e.shiftKey,button:e.button,buttons:e.buttons}},buildSelectorObject:function(t){return t.hasOwnProperty("range")?{type:"box",selector_state:{xrange:t.range.x,yrange:t.range.y}}:t.hasOwnProperty("lassoPoints")?{type:"lasso",selector_state:{xs:t.lassoPoints.x,ys:t.lassoPoints.y}}:null},handle_plotly_restyle:function(t){if(!(null==t||t[0]&&t[0].hasOwnProperty("_doNotReportToPy"))){var e={style_data:t[0],style_traces:t[1],source_view_id:this.viewID};this.model.set("_js2py_restyle",e),this.touch()}},handle_plotly_relayout:function(t){if(null!=t&&!t.hasOwnProperty("_doNotReportToPy")){var e={relayout_data:t,source_view_id:this.viewID};this.model.set("_js2py_relayout",e),this.touch()}},handle_plotly_update:function(t){if(!(null==t||t.data&&t.data[0].hasOwnProperty("_doNotReportToPy"))){var e={style_data:t.data[0],style_traces:t.data[1],layout_data:t.layout,source_view_id:this.viewID};this.model.set("_js2py_update",e),this.touch()}},handle_plotly_click:function(t){this._send_points_callback_message(t,"plotly_click")},handle_plotly_hover:function(t){this._send_points_callback_message(t,"plotly_hover")},handle_plotly_unhover:function(t){this._send_points_callback_message(t,"plotly_unhover")},handle_plotly_selected:function(t){this._send_points_callback_message(t,"plotly_selected")},handle_plotly_deselect:function(t){t={points:[]},this._send_points_callback_message(t,"plotly_deselect")},_send_points_callback_message:function(t,e){if(null!=t){var n={event_type:e,points:this.buildPointsObject(t),device_state:this.buildInputDeviceStateObject(t),selector:this.buildSelectorObject(t)};null!=n.points&&(this.model.set("_js2py_pointsCallback",n),this.touch())}},handle_plotly_doubleclick:function(t){},do_addTraces:function(){var t=this.model.get("_py2js_addTraces");if(null!==t){this.el.data.length;var e=this;o.addTraces(this.el,t.trace_data).then(function(){e._sendTraceDeltas(t.trace_edit_id);var n=t.layout_edit_id;e._sendLayoutDelta(n)})}},do_deleteTraces:function(){var t=this.model.get("_py2js_deleteTraces");if(null!==t){var e=t.delete_inds,n=this;o.deleteTraces(this.el,e).then(function(){var e=t.trace_edit_id;n._sendTraceDeltas(e);var r=t.layout_edit_id;n._sendLayoutDelta(r)})}},do_moveTraces:function(){var t=this.model.get("_py2js_moveTraces");if(null!==t){var e=t.current_trace_inds,n=t.new_trace_inds;i.isEqual(e,n)||o.moveTraces(this.el,e,n)}},do_restyle:function(){var t=this.model.get("_py2js_restyle");if(null!==t){var e=t.restyle_data,n=this.model._normalize_trace_indexes(t.restyle_traces);e._doNotReportToPy=!0,o.restyle(this.el,e,n),this._sendTraceDeltas(t.trace_edit_id);var r=t.layout_edit_id;this._sendLayoutDelta(r)}},do_relayout:function(){var t=this.model.get("_py2js_relayout");if(null!==t){if(t.source_view_id!==this.viewID)t.relayout_data._doNotReportToPy=!0,o.relayout(this.el,t.relayout_data);var e=t.layout_edit_id;this._sendLayoutDelta(e)}},do_update:function(){var t=this.model.get("_py2js_update");if(null!==t){var e=t.style_data||{},n=t.layout_data||{},r=this.model._normalize_trace_indexes(t.style_traces);e._doNotReportToPy=!0,o.update(this.el,e,n,r),this._sendTraceDeltas(t.trace_edit_id);var i=t.layout_edit_id;this._sendLayoutDelta(i)}},do_animate:function(){var t=this.model.get("_py2js_animate");if(null!==t){var e=t.animation_opts,n={data:t.style_data,layout:t.layout_data,traces:this.model._normalize_trace_indexes(t.style_traces),_doNotReportToPy:!0},r=this;o.animate(this.el,n,e).then(function(){r._sendTraceDeltas(t.trace_edit_id);var e=t.layout_edit_id;r._sendLayoutDelta(e)})}},_sendLayoutDelta:function(t){var e={layout_delta:I(this.getFullLayout(),this.model.get("_layout")),layout_edit_id:t};this.model.set("_js2py_layoutDelta",e),this.touch()},_sendTraceDeltas:function(t){for(var e=this.model.get("_data"),n=i.range(e.length),r=new Array(n.length),o=this.getFullData(),a=0;a<n.length;a++){var s=n[a];r[a]=I(o[s],e[s])}var A={trace_deltas:r,trace_edit_id:t};this.model.set("_js2py_traceDeltas",A),this.touch()}}),c={int8:Int8Array,int16:Int16Array,int32:Int32Array,uint8:Uint8Array,uint16:Uint16Array,uint32:Uint32Array,float32:Float32Array,float64:Float64Array};function u(t,e){var n;if(i.isTypedArray(t))n=function(t){var e;if(t instanceof Int8Array)e="int8";else if(t instanceof Int16Array)e="int16";else if(t instanceof Int32Array)e="int32";else if(t instanceof Uint8Array)e="uint8";else if(t instanceof Uint16Array)e="uint16";else if(t instanceof Uint32Array)e="uint32";else if(t instanceof Float32Array)e="float32";else{if(!(t instanceof Float64Array))return t;e="float64"}return{dtype:e,shape:[t.length],value:t.buffer}}(t);else if(Array.isArray(t)){n=new Array(t.length);for(var r=0;r<t.length;r++)n[r]=u(t[r])}else if(i.isPlainObject(t))for(var o in n={},t)t.hasOwnProperty(o)&&(n[o]=u(t[o]));else n=void 0===t?"_undefined_":t;return n}function g(t,e){var n;if(Array.isArray(t)){n=new Array(t.length);for(var r=0;r<t.length;r++)n[r]=g(t[r])}else if(i.isPlainObject(t)){if((i.has(t,"value")||i.has(t,"buffer"))&&i.has(t,"dtype")&&i.has(t,"shape"))n=new(0,c[t.dtype])(i.has(t,"value")?t.value.buffer:t.buffer.buffer);else for(var o in n={},t)t.hasOwnProperty(o)&&(n[o]=g(t[o]))}else n="_undefined_"===t?void 0:t;return n}function h(t,e,n){return"_"===n[0]?null:(r=e,!ArrayBuffer.isView(r)||r instanceof DataView?void 0:e);var r}function d(t,e){for(var n in e)if(e.hasOwnProperty(n)){var r=e[n];null===r?i.unset(t,n):i.set(t,n,r)}}function f(t,e,n){for(var r in e)if(e.hasOwnProperty(r)){var o=e[r];Array.isArray(o)||(o=[o]);for(var a=0;a<n.length;a++){var s=t[n[a]],A=o[a%o.length];null===A?i.unset(s,r):void 0!==A&&i.set(s,r,A)}}}function p(t,e){for(var n=0;n<e.length;n++){var r=e[n];i.unset(t,r)}}function I(t,e){var n;for(var r in n=Array.isArray(t)?new Array(t.length):{},null==e&&(e={}),t){if("_"!==r[0]&&t.hasOwnProperty(r)&&null!==t[r])if(!i.isEqual(t[r],e[r])||"uid"===r){var o=t[r];if(e.hasOwnProperty(r)&&"object"==typeof o)if(Array.isArray(o))if(o.length>0&&"object"==typeof o[0]){n[r]=new Array(o.length);for(var a=0;a<o.length;a++)!Array.isArray(e[r])||e[r].length<=a?n[r][a]=o[a]:n[r][a]=I(o[a],e[r][a])}else n[r]=o;else{var s=I(o,e[r]);Object.keys(s).length>0&&(n[r]=s)}else"object"!=typeof o||Array.isArray(o)?void 0!==o&&"function"!=typeof o&&(n[r]=o):n[r]=I(o,{})}}return n}t.exports={FigureView:l,FigureModel:A}},function(t,e,n){(function(e){var n;t.exports=function t(e,r,i){function o(s,A){if(!r[s]){if(!e[s]){var l="function"==typeof n&&n;if(!A&&l)return n(s,!0);if(a)return a(s,!0);var c=new Error("Cannot find module '"+s+"'");throw c.code="MODULE_NOT_FOUND",c}var u=r[s]={exports:{}};e[s][0].call(u.exports,function(t){return o(e[s][1][t]||t)},u,u.exports,t,e,r,i)}return r[s].exports}for(var a="function"==typeof n&&n,s=0;s<i.length;s++)o(i[s]);return o}({1:[function(t,e,n){"use strict";var r=t("../src/lib"),i={"X,X div":"direction:ltr;font-family:'Open Sans', verdana, arial, sans-serif;margin:0;padding:0;","X input,X button":"font-family:'Open Sans', verdana, arial, sans-serif;","X input:focus,X button:focus":"outline:none;","X a":"text-decoration:none;","X a:hover":"text-decoration:none;","X .crisp":"shape-rendering:crispEdges;","X .user-select-none":"-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;-o-user-select:none;user-select:none;","X svg":"overflow:hidden;","X svg a":"fill:#447adb;","X svg a:hover":"fill:#3c6dc5;","X .main-svg":"position:absolute;top:0;left:0;pointer-events:none;","X .main-svg .draglayer":"pointer-events:all;","X .cursor-default":"cursor:default;","X .cursor-pointer":"cursor:pointer;","X .cursor-crosshair":"cursor:crosshair;","X .cursor-move":"cursor:move;","X .cursor-col-resize":"cursor:col-resize;","X .cursor-row-resize":"cursor:row-resize;","X .cursor-ns-resize":"cursor:ns-resize;","X .cursor-ew-resize":"cursor:ew-resize;","X .cursor-sw-resize":"cursor:sw-resize;","X .cursor-s-resize":"cursor:s-resize;","X .cursor-se-resize":"cursor:se-resize;","X .cursor-w-resize":"cursor:w-resize;","X .cursor-e-resize":"cursor:e-resize;","X .cursor-nw-resize":"cursor:nw-resize;","X .cursor-n-resize":"cursor:n-resize;","X .cursor-ne-resize":"cursor:ne-resize;","X .cursor-grab":"cursor:-webkit-grab;cursor:grab;","X .modebar":"position:absolute;top:2px;right:2px;","X .ease-bg":"-webkit-transition:background-color 0.3s ease 0s;-moz-transition:background-color 0.3s ease 0s;-ms-transition:background-color 0.3s ease 0s;-o-transition:background-color 0.3s ease 0s;transition:background-color 0.3s ease 0s;","X .modebar--hover>:not(.watermark)":"opacity:0;-webkit-transition:opacity 0.3s ease 0s;-moz-transition:opacity 0.3s ease 0s;-ms-transition:opacity 0.3s ease 0s;-o-transition:opacity 0.3s ease 0s;transition:opacity 0.3s ease 0s;","X:hover .modebar--hover .modebar-group":"opacity:1;","X .modebar-group":"float:left;display:inline-block;box-sizing:border-box;padding-left:8px;position:relative;vertical-align:middle;white-space:nowrap;","X .modebar-btn":"position:relative;font-size:16px;padding:3px 4px;height:22px;cursor:pointer;line-height:normal;box-sizing:border-box;","X .modebar-btn svg":"position:relative;top:2px;","X .modebar.vertical":"display:flex;flex-direction:column;flex-wrap:wrap;align-content:flex-end;max-height:100%;","X .modebar.vertical svg":"top:-1px;","X .modebar.vertical .modebar-group":"display:block;float:none;padding-left:0px;padding-bottom:8px;","X .modebar.vertical .modebar-group .modebar-btn":"display:block;text-align:center;","X [data-title]:before,X [data-title]:after":"position:absolute;-webkit-transform:translate3d(0, 0, 0);-moz-transform:translate3d(0, 0, 0);-ms-transform:translate3d(0, 0, 0);-o-transform:translate3d(0, 0, 0);transform:translate3d(0, 0, 0);display:none;opacity:0;z-index:1001;pointer-events:none;top:110%;right:50%;","X [data-title]:hover:before,X [data-title]:hover:after":"display:block;opacity:1;","X [data-title]:before":"content:'';position:absolute;background:transparent;border:6px solid transparent;z-index:1002;margin-top:-12px;border-bottom-color:#69738a;margin-right:-6px;","X [data-title]:after":"content:attr(data-title);background:#69738a;color:white;padding:8px 10px;font-size:12px;line-height:12px;white-space:nowrap;margin-right:-18px;border-radius:2px;","X .vertical [data-title]:before,X .vertical [data-title]:after":"top:0%;right:200%;","X .vertical [data-title]:before":"border:6px solid transparent;border-left-color:#69738a;margin-top:8px;margin-right:-30px;","X .select-outline":"fill:none;stroke-width:1;shape-rendering:crispEdges;","X .select-outline-1":"stroke:white;","X .select-outline-2":"stroke:black;stroke-dasharray:2px 2px;",Y:"font-family:'Open Sans';position:fixed;top:50px;right:20px;z-index:10000;font-size:10pt;max-width:180px;","Y p":"margin:0;","Y .notifier-note":"min-width:180px;max-width:250px;border:1px solid #fff;z-index:3000;margin:0;background-color:#8c97af;background-color:rgba(140,151,175,0.9);color:#fff;padding:10px;overflow-wrap:break-word;word-wrap:break-word;-ms-hyphens:auto;-webkit-hyphens:auto;hyphens:auto;","Y .notifier-close":"color:#fff;opacity:0.8;float:right;padding:0 5px;background:none;border:none;font-size:20px;font-weight:bold;line-height:20px;","Y .notifier-close:hover":"color:#444;text-decoration:none;cursor:pointer;"};for(var o in i){var a=o.replace(/^,/," ,").replace(/X/g,".js-plotly-plot .plotly").replace(/Y/g,".plotly-notifier");r.addStyleRule(a,i[o])}},{"../src/lib":719}],2:[function(t,e,n){"use strict";e.exports=t("../src/transforms/aggregate")},{"../src/transforms/aggregate":1265}],3:[function(t,e,n){"use strict";e.exports=t("../src/traces/bar")},{"../src/traces/bar":864}],4:[function(t,e,n){"use strict";e.exports=t("../src/traces/barpolar")},{"../src/traces/barpolar":876}],5:[function(t,e,n){"use strict";e.exports=t("../src/traces/box")},{"../src/traces/box":886}],6:[function(t,e,n){"use strict";e.exports=t("../src/components/calendars")},{"../src/components/calendars":591}],7:[function(t,e,n){"use strict";e.exports=t("../src/traces/candlestick")},{"../src/traces/candlestick":895}],8:[function(t,e,n){"use strict";e.exports=t("../src/traces/carpet")},{"../src/traces/carpet":914}],9:[function(t,e,n){"use strict";e.exports=t("../src/traces/choropleth")},{"../src/traces/choropleth":928}],10:[function(t,e,n){"use strict";e.exports=t("../src/traces/choroplethmapbox")},{"../src/traces/choroplethmapbox":935}],11:[function(t,e,n){"use strict";e.exports=t("../src/traces/cone")},{"../src/traces/cone":941}],12:[function(t,e,n){"use strict";e.exports=t("../src/traces/contour")},{"../src/traces/contour":956}],13:[function(t,e,n){"use strict";e.exports=t("../src/traces/contourcarpet")},{"../src/traces/contourcarpet":967}],14:[function(t,e,n){"use strict";e.exports=t("../src/core")},{"../src/core":697}],15:[function(t,e,n){"use strict";e.exports=t("../src/traces/densitymapbox")},{"../src/traces/densitymapbox":977}],16:[function(t,e,n){"use strict";e.exports=t("../src/transforms/filter")},{"../src/transforms/filter":1266}],17:[function(t,e,n){"use strict";e.exports=t("../src/traces/funnel")},{"../src/traces/funnel":987}],18:[function(t,e,n){"use strict";e.exports=t("../src/traces/funnelarea")},{"../src/traces/funnelarea":996}],19:[function(t,e,n){"use strict";e.exports=t("../src/transforms/groupby")},{"../src/transforms/groupby":1267}],20:[function(t,e,n){"use strict";e.exports=t("../src/traces/heatmap")},{"../src/traces/heatmap":1009}],21:[function(t,e,n){"use strict";e.exports=t("../src/traces/heatmapgl")},{"../src/traces/heatmapgl":1018}],22:[function(t,e,n){"use strict";e.exports=t("../src/traces/histogram")},{"../src/traces/histogram":1030}],23:[function(t,e,n){"use strict";e.exports=t("../src/traces/histogram2d")},{"../src/traces/histogram2d":1036}],24:[function(t,e,n){"use strict";e.exports=t("../src/traces/histogram2dcontour")},{"../src/traces/histogram2dcontour":1040}],25:[function(t,e,n){"use strict";var r=t("./core");r.register([t("./bar"),t("./box"),t("./heatmap"),t("./histogram"),t("./histogram2d"),t("./histogram2dcontour"),t("./contour"),t("./scatterternary"),t("./violin"),t("./funnel"),t("./waterfall"),t("./pie"),t("./sunburst"),t("./funnelarea"),t("./scatter3d"),t("./surface"),t("./isosurface"),t("./volume"),t("./mesh3d"),t("./cone"),t("./streamtube"),t("./scattergeo"),t("./choropleth"),t("./scattergl"),t("./splom"),t("./pointcloud"),t("./heatmapgl"),t("./parcoords"),t("./parcats"),t("./scattermapbox"),t("./choroplethmapbox"),t("./densitymapbox"),t("./sankey"),t("./indicator"),t("./table"),t("./carpet"),t("./scattercarpet"),t("./contourcarpet"),t("./ohlc"),t("./candlestick"),t("./scatterpolar"),t("./scatterpolargl"),t("./barpolar")]),r.register([t("./aggregate"),t("./filter"),t("./groupby"),t("./sort")]),r.register([t("./calendars")]),e.exports=r},{"./aggregate":2,"./bar":3,"./barpolar":4,"./box":5,"./calendars":6,"./candlestick":7,"./carpet":8,"./choropleth":9,"./choroplethmapbox":10,"./cone":11,"./contour":12,"./contourcarpet":13,"./core":14,"./densitymapbox":15,"./filter":16,"./funnel":17,"./funnelarea":18,"./groupby":19,"./heatmap":20,"./heatmapgl":21,"./histogram":22,"./histogram2d":23,"./histogram2dcontour":24,"./indicator":26,"./isosurface":27,"./mesh3d":28,"./ohlc":29,"./parcats":30,"./parcoords":31,"./pie":32,"./pointcloud":33,"./sankey":34,"./scatter3d":35,"./scattercarpet":36,"./scattergeo":37,"./scattergl":38,"./scattermapbox":39,"./scatterpolar":40,"./scatterpolargl":41,"./scatterternary":42,"./sort":43,"./splom":44,"./streamtube":45,"./sunburst":46,"./surface":47,"./table":48,"./violin":49,"./volume":50,"./waterfall":51}],26:[function(t,e,n){"use strict";e.exports=t("../src/traces/indicator")},{"../src/traces/indicator":1046}],27:[function(t,e,n){"use strict";e.exports=t("../src/traces/isosurface")},{"../src/traces/isosurface":1052}],28:[function(t,e,n){"use strict";e.exports=t("../src/traces/mesh3d")},{"../src/traces/mesh3d":1057}],29:[function(t,e,n){"use strict";e.exports=t("../src/traces/ohlc")},{"../src/traces/ohlc":1062}],30:[function(t,e,n){"use strict";e.exports=t("../src/traces/parcats")},{"../src/traces/parcats":1071}],31:[function(t,e,n){"use strict";e.exports=t("../src/traces/parcoords")},{"../src/traces/parcoords":1081}],32:[function(t,e,n){"use strict";e.exports=t("../src/traces/pie")},{"../src/traces/pie":1092}],33:[function(t,e,n){"use strict";e.exports=t("../src/traces/pointcloud")},{"../src/traces/pointcloud":1101}],34:[function(t,e,n){"use strict";e.exports=t("../src/traces/sankey")},{"../src/traces/sankey":1107}],35:[function(t,e,n){"use strict";e.exports=t("../src/traces/scatter3d")},{"../src/traces/scatter3d":1143}],36:[function(t,e,n){"use strict";e.exports=t("../src/traces/scattercarpet")},{"../src/traces/scattercarpet":1149}],37:[function(t,e,n){"use strict";e.exports=t("../src/traces/scattergeo")},{"../src/traces/scattergeo":1156}],38:[function(t,e,n){"use strict";e.exports=t("../src/traces/scattergl")},{"../src/traces/scattergl":1167}],39:[function(t,e,n){"use strict";e.exports=t("../src/traces/scattermapbox")},{"../src/traces/scattermapbox":1176}],40:[function(t,e,n){"use strict";e.exports=t("../src/traces/scatterpolar")},{"../src/traces/scatterpolar":1183}],41:[function(t,e,n){"use strict";e.exports=t("../src/traces/scatterpolargl")},{"../src/traces/scatterpolargl":1189}],42:[function(t,e,n){"use strict";e.exports=t("../src/traces/scatterternary")},{"../src/traces/scatterternary":1196}],43:[function(t,e,n){"use strict";e.exports=t("../src/transforms/sort")},{"../src/transforms/sort":1269}],44:[function(t,e,n){"use strict";e.exports=t("../src/traces/splom")},{"../src/traces/splom":1205}],45:[function(t,e,n){"use strict";e.exports=t("../src/traces/streamtube")},{"../src/traces/streamtube":1213}],46:[function(t,e,n){"use strict";e.exports=t("../src/traces/sunburst")},{"../src/traces/sunburst":1219}],47:[function(t,e,n){"use strict";e.exports=t("../src/traces/surface")},{"../src/traces/surface":1228}],48:[function(t,e,n){"use strict";e.exports=t("../src/traces/table")},{"../src/traces/table":1236}],49:[function(t,e,n){"use strict";e.exports=t("../src/traces/violin")},{"../src/traces/violin":1244}],50:[function(t,e,n){"use strict";e.exports=t("../src/traces/volume")},{"../src/traces/volume":1252}],51:[function(t,e,n){"use strict";e.exports=t("../src/traces/waterfall")},{"../src/traces/waterfall":1260}],52:[function(t,e,n){"use strict";e.exports=function(t){var e=(t=t||{}).eye||[0,0,1],n=t.center||[0,0,0],s=t.up||[0,1,0],A=t.distanceLimits||[0,1/0],l=t.mode||"turntable",c=r(),u=i(),g=o();return c.setDistanceLimits(A[0],A[1]),c.lookAt(0,e,n,s),u.setDistanceLimits(A[0],A[1]),u.lookAt(0,e,n,s),g.setDistanceLimits(A[0],A[1]),g.lookAt(0,e,n,s),new a({turntable:c,orbit:u,matrix:g},l)};var r=t("turntable-camera-controller"),i=t("orbit-camera-controller"),o=t("matrix-camera-controller");function a(t,e){this._controllerNames=Object.keys(t),this._controllerList=this._controllerNames.map(function(e){return t[e]}),this._mode=e,this._active=t[e],this._active||(this._mode="turntable",this._active=t.turntable),this.modes=this._controllerNames,this.computedMatrix=this._active.computedMatrix,this.computedEye=this._active.computedEye,this.computedUp=this._active.computedUp,this.computedCenter=this._active.computedCenter,this.computedRadius=this._active.computedRadius}var s=a.prototype;[["flush",1],["idle",1],["lookAt",4],["rotate",4],["pan",4],["translate",4],["setMatrix",2],["setDistanceLimits",2],["setDistance",2]].forEach(function(t){for(var e=t[0],n=[],r=0;r<t[1];++r)n.push("a"+r);var i="var cc=this._controllerList;for(var i=0;i<cc.length;++i){cc[i]."+t[0]+"("+n.join()+")}";s[e]=Function.apply(null,n.concat(i))}),s.recalcMatrix=function(t){this._active.recalcMatrix(t)},s.getDistance=function(t){return this._active.getDistance(t)},s.getDistanceLimits=function(t){return this._active.getDistanceLimits(t)},s.lastT=function(){return this._active.lastT()},s.setMode=function(t){if(t!==this._mode){var e=this._controllerNames.indexOf(t);if(!(e<0)){var n=this._active,r=this._controllerList[e],i=Math.max(n.lastT(),r.lastT());n.recalcMatrix(i),r.setMatrix(i,n.computedMatrix),this._active=r,this._mode=t,this.computedMatrix=this._active.computedMatrix,this.computedEye=this._active.computedEye,this.computedUp=this._active.computedUp,this.computedCenter=this._active.computedCenter,this.computedRadius=this._active.computedRadius}}},s.getMode=function(){return this._mode}},{"matrix-camera-controller":433,"orbit-camera-controller":456,"turntable-camera-controller":542}],53:[function(t,e,n){var r,i;r=this,i=function(t,e,n,r,i){"use strict";function o(t){return t.target.depth}function a(t,e){return t.sourceLinks.length?t.depth:e-1}function s(t){return function(){return t}}i=i&&i.hasOwnProperty("default")?i.default:i;var A="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t};function l(t,e){return u(t.source,e.source)||t.index-e.index}function c(t,e){return u(t.target,e.target)||t.index-e.index}function u(t,e){return t.partOfCycle===e.partOfCycle?t.y0-e.y0:"top"===t.circularLinkType||"bottom"===e.circularLinkType?-1:1}function g(t){return t.value}function h(t){return(t.y0+t.y1)/2}function d(t){return h(t.source)}function f(t){return h(t.target)}function p(t){return t.index}function I(t){return t.nodes}function C(t){return t.links}function m(t,e){var n=t.get(e);if(!n)throw new Error("missing: "+e);return n}function y(t,e){return e(t)}var E=25,v=10,B=.3;function b(t,e){var n=0,r=0;t.links.forEach(function(i){i.circular&&(i.source.circularLinkType||i.target.circularLinkType?i.circularLinkType=i.source.circularLinkType?i.source.circularLinkType:i.target.circularLinkType:i.circularLinkType=n<r?"top":"bottom","top"==i.circularLinkType?n+=1:r+=1,t.nodes.forEach(function(t){y(t,e)!=y(i.source,e)&&y(t,e)!=y(i.target,e)||(t.circularLinkType=i.circularLinkType)}))}),t.links.forEach(function(t){t.circular&&(t.source.circularLinkType==t.target.circularLinkType&&(t.circularLinkType=t.source.circularLinkType),z(t,e)&&(t.circularLinkType=t.source.circularLinkType))})}function Q(t){var e=Math.abs(t.y1-t.y0),n=Math.abs(t.target.x0-t.source.x1);return Math.atan(n/e)}function w(t,e){var n=0;t.sourceLinks.forEach(function(t){n=t.circular&&!z(t,e)?n+1:n});var r=0;return t.targetLinks.forEach(function(t){r=t.circular&&!z(t,e)?r+1:r}),n+r}function x(t){var e=t.source.sourceLinks,n=0;e.forEach(function(t){n=t.circular?n+1:n});var r=t.target.targetLinks,i=0;return r.forEach(function(t){i=t.circular?i+1:i}),!(n>1||i>1)}function _(t,e,n){return t.sort(M),t.forEach(function(r,i){var o,a,s=0;if(z(r,n)&&x(r))r.circularPathData.verticalBuffer=s+r.width/2;else{for(var A=0;A<i;A++)if(o=t[i],a=t[A],!(o.source.column<a.target.column||o.target.column>a.source.column)){var l=t[A].circularPathData.verticalBuffer+t[A].width/2+e;s=l>s?l:s}r.circularPathData.verticalBuffer=s+r.width/2}}),t}function D(t,n,i,o){var a=e.min(t.links,function(t){return t.source.y0});t.links.forEach(function(t){t.circular&&(t.circularPathData={})}),_(t.links.filter(function(t){return"top"==t.circularLinkType}),n,o),_(t.links.filter(function(t){return"bottom"==t.circularLinkType}),n,o),t.links.forEach(function(e){if(e.circular){if(e.circularPathData.arcRadius=e.width+v,e.circularPathData.leftNodeBuffer=5,e.circularPathData.rightNodeBuffer=5,e.circularPathData.sourceWidth=e.source.x1-e.source.x0,e.circularPathData.sourceX=e.source.x0+e.circularPathData.sourceWidth,e.circularPathData.targetX=e.target.x0,e.circularPathData.sourceY=e.y0,e.circularPathData.targetY=e.y1,z(e,o)&&x(e))e.circularPathData.leftSmallArcRadius=v+e.width/2,e.circularPathData.leftLargeArcRadius=v+e.width/2,e.circularPathData.rightSmallArcRadius=v+e.width/2,e.circularPathData.rightLargeArcRadius=v+e.width/2,"bottom"==e.circularLinkType?(e.circularPathData.verticalFullExtent=e.source.y1+E+e.circularPathData.verticalBuffer,e.circularPathData.verticalLeftInnerExtent=e.circularPathData.verticalFullExtent-e.circularPathData.leftLargeArcRadius,e.circularPathData.verticalRightInnerExtent=e.circularPathData.verticalFullExtent-e.circularPathData.rightLargeArcRadius):(e.circularPathData.verticalFullExtent=e.source.y0-E-e.circularPathData.verticalBuffer,e.circularPathData.verticalLeftInnerExtent=e.circularPathData.verticalFullExtent+e.circularPathData.leftLargeArcRadius,e.circularPathData.verticalRightInnerExtent=e.circularPathData.verticalFullExtent+e.circularPathData.rightLargeArcRadius);else{var s=e.source.column,A=e.circularLinkType,l=t.links.filter(function(t){return t.source.column==s&&t.circularLinkType==A});"bottom"==e.circularLinkType?l.sort(S):l.sort(k);var c=0;l.forEach(function(t,r){t.circularLinkID==e.circularLinkID&&(e.circularPathData.leftSmallArcRadius=v+e.width/2+c,e.circularPathData.leftLargeArcRadius=v+e.width/2+r*n+c),c+=t.width}),s=e.target.column,l=t.links.filter(function(t){return t.target.column==s&&t.circularLinkType==A}),"bottom"==e.circularLinkType?l.sort(R):l.sort(L),c=0,l.forEach(function(t,r){t.circularLinkID==e.circularLinkID&&(e.circularPathData.rightSmallArcRadius=v+e.width/2+c,e.circularPathData.rightLargeArcRadius=v+e.width/2+r*n+c),c+=t.width}),"bottom"==e.circularLinkType?(e.circularPathData.verticalFullExtent=Math.max(i,e.source.y1,e.target.y1)+E+e.circularPathData.verticalBuffer,e.circularPathData.verticalLeftInnerExtent=e.circularPathData.verticalFullExtent-e.circularPathData.leftLargeArcRadius,e.circularPathData.verticalRightInnerExtent=e.circularPathData.verticalFullExtent-e.circularPathData.rightLargeArcRadius):(e.circularPathData.verticalFullExtent=a-E-e.circularPathData.verticalBuffer,e.circularPathData.verticalLeftInnerExtent=e.circularPathData.verticalFullExtent+e.circularPathData.leftLargeArcRadius,e.circularPathData.verticalRightInnerExtent=e.circularPathData.verticalFullExtent+e.circularPathData.rightLargeArcRadius)}e.circularPathData.leftInnerExtent=e.circularPathData.sourceX+e.circularPathData.leftNodeBuffer,e.circularPathData.rightInnerExtent=e.circularPathData.targetX-e.circularPathData.rightNodeBuffer,e.circularPathData.leftFullExtent=e.circularPathData.sourceX+e.circularPathData.leftLargeArcRadius+e.circularPathData.leftNodeBuffer,e.circularPathData.rightFullExtent=e.circularPathData.targetX-e.circularPathData.rightLargeArcRadius-e.circularPathData.rightNodeBuffer}if(e.circular)e.path=function(t){return"top"==t.circularLinkType?"M"+t.circularPathData.sourceX+" "+t.circularPathData.sourceY+" L"+t.circularPathData.leftInnerExtent+" "+t.circularPathData.sourceY+" A"+t.circularPathData.leftLargeArcRadius+" "+t.circularPathData.leftSmallArcRadius+" 0 0 0 "+t.circularPathData.leftFullExtent+" "+(t.circularPathData.sourceY-t.circularPathData.leftSmallArcRadius)+" L"+t.circularPathData.leftFullExtent+" "+t.circularPathData.verticalLeftInnerExtent+" A"+t.circularPathData.leftLargeArcRadius+" "+t.circularPathData.leftLargeArcRadius+" 0 0 0 "+t.circularPathData.leftInnerExtent+" "+t.circularPathData.verticalFullExtent+" L"+t.circularPathData.rightInnerExtent+" "+t.circularPathData.verticalFullExtent+" A"+t.circularPathData.rightLargeArcRadius+" "+t.circularPathData.rightLargeArcRadius+" 0 0 0 "+t.circularPathData.rightFullExtent+" "+t.circularPathData.verticalRightInnerExtent+" L"+t.circularPathData.rightFullExtent+" "+(t.circularPathData.targetY-t.circularPathData.rightSmallArcRadius)+" A"+t.circularPathData.rightLargeArcRadius+" "+t.circularPathData.rightSmallArcRadius+" 0 0 0 "+t.circularPathData.rightInnerExtent+" "+t.circularPathData.targetY+" L"+t.circularPathData.targetX+" "+t.circularPathData.targetY:"M"+t.circularPathData.sourceX+" "+t.circularPathData.sourceY+" L"+t.circularPathData.leftInnerExtent+" "+t.circularPathData.sourceY+" A"+t.circularPathData.leftLargeArcRadius+" "+t.circularPathData.leftSmallArcRadius+" 0 0 1 "+t.circularPathData.leftFullExtent+" "+(t.circularPathData.sourceY+t.circularPathData.leftSmallArcRadius)+" L"+t.circularPathData.leftFullExtent+" "+t.circularPathData.verticalLeftInnerExtent+" A"+t.circularPathData.leftLargeArcRadius+" "+t.circularPathData.leftLargeArcRadius+" 0 0 1 "+t.circularPathData.leftInnerExtent+" "+t.circularPathData.verticalFullExtent+" L"+t.circularPathData.rightInnerExtent+" "+t.circularPathData.verticalFullExtent+" A"+t.circularPathData.rightLargeArcRadius+" "+t.circularPathData.rightLargeArcRadius+" 0 0 1 "+t.circularPathData.rightFullExtent+" "+t.circularPathData.verticalRightInnerExtent+" L"+t.circularPathData.rightFullExtent+" "+(t.circularPathData.targetY+t.circularPathData.rightSmallArcRadius)+" A"+t.circularPathData.rightLargeArcRadius+" "+t.circularPathData.rightSmallArcRadius+" 0 0 1 "+t.circularPathData.rightInnerExtent+" "+t.circularPathData.targetY+" L"+t.circularPathData.targetX+" "+t.circularPathData.targetY}(e);else{var u=r.linkHorizontal().source(function(t){return[t.source.x0+(t.source.x1-t.source.x0),t.y0]}).target(function(t){return[t.target.x0,t.y1]});e.path=u(e)}})}function M(t,e){return N(t)==N(e)?"bottom"==t.circularLinkType?S(t,e):k(t,e):N(e)-N(t)}function k(t,e){return t.y0-e.y0}function S(t,e){return e.y0-t.y0}function L(t,e){return t.y1-e.y1}function R(t,e){return e.y1-t.y1}function N(t){return t.target.column-t.source.column}function F(t){return t.target.x0-t.source.x1}function T(t,e){var n=Q(t),r=F(e)/Math.tan(n);return"up"==K(t)?t.y1+r:t.y1-r}function U(t,e){var n=Q(t),r=F(e)/Math.tan(n);return"up"==K(t)?t.y1-r:t.y1+r}function O(t,e,n,r){t.links.forEach(function(i){if(!i.circular&&i.target.column-i.source.column>1){var o=i.source.column+1,a=i.target.column-1,s=1,A=a-o+1;for(s=1;o<=a;o++,s++)t.nodes.forEach(function(a){if(a.column==o){var l,c=s/(A+1),u=Math.pow(1-c,3),g=3*c*Math.pow(1-c,2),h=3*Math.pow(c,2)*(1-c),d=Math.pow(c,3),f=u*i.y0+g*i.y0+h*i.y1+d*i.y1,p=f-i.width/2,I=f+i.width/2;p>a.y0&&p<a.y1?(l=a.y1-p+10,l="bottom"==a.circularLinkType?l:-l,a=H(a,l,e,n),t.nodes.forEach(function(t){y(t,r)!=y(a,r)&&t.column==a.column&&G(a,t)&&H(t,l,e,n)})):I>a.y0&&I<a.y1?(l=I-a.y0+10,a=H(a,l,e,n),t.nodes.forEach(function(t){y(t,r)!=y(a,r)&&t.column==a.column&&t.y0<a.y1&&t.y1>a.y1&&H(t,l,e,n)})):p<a.y0&&I>a.y1&&(l=I-a.y0+10,a=H(a,l,e,n),t.nodes.forEach(function(t){y(t,r)!=y(a,r)&&t.column==a.column&&t.y0<a.y1&&t.y1>a.y1&&H(t,l,e,n)}))}})}})}function G(t,e){return t.y0>e.y0&&t.y0<e.y1||t.y1>e.y0&&t.y1<e.y1||t.y0<e.y0&&t.y1>e.y1}function H(t,e,n,r){return t.y0+e>=n&&t.y1+e<=r&&(t.y0=t.y0+e,t.y1=t.y1+e,t.targetLinks.forEach(function(t){t.y1=t.y1+e}),t.sourceLinks.forEach(function(t){t.y0=t.y0+e})),t}function j(t,e,n,r){t.nodes.forEach(function(i){r&&i.y+(i.y1-i.y0)>e&&(i.y=i.y-(i.y+(i.y1-i.y0)-e));var o=t.links.filter(function(t){return y(t.source,n)==y(i,n)}),a=o.length;a>1&&o.sort(function(t,e){if(!t.circular&&!e.circular){if(t.target.column==e.target.column)return t.y1-e.y1;if(!P(t,e))return t.y1-e.y1;if(t.target.column>e.target.column){var n=U(e,t);return t.y1-n}if(e.target.column>t.target.column)return U(t,e)-e.y1}return t.circular&&!e.circular?"top"==t.circularLinkType?-1:1:e.circular&&!t.circular?"top"==e.circularLinkType?1:-1:t.circular&&e.circular?t.circularLinkType===e.circularLinkType&&"top"==t.circularLinkType?t.target.column===e.target.column?t.target.y1-e.target.y1:e.target.column-t.target.column:t.circularLinkType===e.circularLinkType&&"bottom"==t.circularLinkType?t.target.column===e.target.column?e.target.y1-t.target.y1:t.target.column-e.target.column:"top"==t.circularLinkType?-1:1:void 0});var s=i.y0;o.forEach(function(t){t.y0=s+t.width/2,s+=t.width}),o.forEach(function(t,e){if("bottom"==t.circularLinkType){for(var n=e+1,r=0;n<a;n++)r+=o[n].width;t.y0=i.y1-r-t.width/2}})})}function Y(t,e,n){t.nodes.forEach(function(e){var r=t.links.filter(function(t){return y(t.target,n)==y(e,n)}),i=r.length;i>1&&r.sort(function(t,e){if(!t.circular&&!e.circular){if(t.source.column==e.source.column)return t.y0-e.y0;if(!P(t,e))return t.y0-e.y0;if(e.source.column<t.source.column){var n=T(e,t);return t.y0-n}if(t.source.column<e.source.column)return T(t,e)-e.y0}return t.circular&&!e.circular?"top"==t.circularLinkType?-1:1:e.circular&&!t.circular?"top"==e.circularLinkType?1:-1:t.circular&&e.circular?t.circularLinkType===e.circularLinkType&&"top"==t.circularLinkType?t.source.column===e.source.column?t.source.y1-e.source.y1:t.source.column-e.source.column:t.circularLinkType===e.circularLinkType&&"bottom"==t.circularLinkType?t.source.column===e.source.column?t.source.y1-e.source.y1:e.source.column-t.source.column:"top"==t.circularLinkType?-1:1:void 0});var o=e.y0;r.forEach(function(t){t.y1=o+t.width/2,o+=t.width}),r.forEach(function(t,n){if("bottom"==t.circularLinkType){for(var o=n+1,a=0;o<i;o++)a+=r[o].width;t.y1=e.y1-a-t.width/2}})})}function P(t,e){return K(t)==K(e)}function K(t){return t.y0-t.y1>0?"up":"down"}function z(t,e){return y(t.source,e)==y(t.target,e)}t.sankeyCircular=function(){var t,r,o=0,y=0,Q=1,x=1,_=24,M=p,k=a,S=I,L=C,R=32,N=2,F=null;function T(){var a={nodes:S.apply(null,arguments),links:L.apply(null,arguments)};!function(t){t.nodes.forEach(function(t,e){t.index=e,t.sourceLinks=[],t.targetLinks=[]});var e=n.map(t.nodes,M);t.links.forEach(function(t,n){t.index=n;var r=t.source,i=t.target;"object"!==(void 0===r?"undefined":A(r))&&(r=t.source=m(e,r)),"object"!==(void 0===i?"undefined":A(i))&&(i=t.target=m(e,i)),r.sourceLinks.push(t),i.targetLinks.push(t)})}(a),function(t,e,n){var r=0;if(null===n){for(var o=[],a=0;a<t.links.length;a++){var s=t.links[a],A=s.source.index,l=s.target.index;o[A]||(o[A]=[]),o[l]||(o[l]=[]),-1===o[A].indexOf(l)&&o[A].push(l)}var c=i(o);c.sort(function(t,e){return t.length-e.length});var u={};for(a=0;a<c.length;a++){var g=c[a],h=g.slice(-2);u[h[0]]||(u[h[0]]={}),u[h[0]][h[1]]=!0}t.links.forEach(function(t){var e=t.target.index,n=t.source.index;e===n||u[n]&&u[n][e]?(t.circular=!0,t.circularLinkID=r,r+=1):t.circular=!1})}else t.links.forEach(function(t){t.source[n]<t.target[n]?t.circular=!1:(t.circular=!0,t.circularLinkID=r,r+=1)})}(a,0,F),function(t){t.nodes.forEach(function(t){t.partOfCycle=!1,t.value=Math.max(e.sum(t.sourceLinks,g),e.sum(t.targetLinks,g)),t.sourceLinks.forEach(function(e){e.circular&&(t.partOfCycle=!0,t.circularLinkType=e.circularLinkType)}),t.targetLinks.forEach(function(e){e.circular&&(t.partOfCycle=!0,t.circularLinkType=e.circularLinkType)})})}(a),function(t){var e,n,r;for(e=t.nodes,n=[],r=0;e.length;++r,e=n,n=[])e.forEach(function(t){t.depth=r,t.sourceLinks.forEach(function(t){n.indexOf(t.target)<0&&!t.circular&&n.push(t.target)})});for(e=t.nodes,n=[],r=0;e.length;++r,e=n,n=[])e.forEach(function(t){t.height=r,t.targetLinks.forEach(function(t){n.indexOf(t.source)<0&&!t.circular&&n.push(t.source)})});t.nodes.forEach(function(t){t.column=Math.floor(k.call(null,t,r))})}(a),b(a,M),function(i,a,s){var A=n.nest().key(function(t){return t.column}).sortKeys(e.ascending).entries(i.nodes).map(function(t){return t.values});(function(n){if(r){var a=1/0;A.forEach(function(t){var e=x*r/(t.length+1);a=e<a?e:a}),t=a}var s=e.min(A,function(n){return(x-y-(n.length-1)*t)/e.sum(n,g)});s*=B,i.links.forEach(function(t){t.width=t.value*s});var l=function(t){var n=0,r=0,i=0,o=0,a=e.max(t.nodes,function(t){return t.column});return t.links.forEach(function(t){t.circular&&("top"==t.circularLinkType?n+=t.width:r+=t.width,0==t.target.column&&(o+=t.width),t.source.column==a&&(i+=t.width))}),{top:n=n>0?n+E+v:n,bottom:r=r>0?r+E+v:r,left:o=o>0?o+E+v:o,right:i=i>0?i+E+v:i}}(i),c=function(t,n){var r=e.max(t.nodes,function(t){return t.column}),i=Q-o,a=x-y,s=i+n.right+n.left,A=a+n.top+n.bottom,l=i/s,c=a/A;return o=o*l+n.left,Q=0==n.right?Q:Q*l,y=y*c+n.top,x*=c,t.nodes.forEach(function(t){t.x0=o+t.column*((Q-o-_)/r),t.x1=t.x0+_}),c}(i,l);s*=c,i.links.forEach(function(t){t.width=t.value*s}),A.forEach(function(t){var e=t.length;t.forEach(function(t,r){t.depth==A.length-1&&1==e?(t.y0=x/2-t.value*s,t.y1=t.y0+t.value*s):0==t.depth&&1==e?(t.y0=x/2-t.value*s,t.y1=t.y0+t.value*s):t.partOfCycle?0==w(t,n)?(t.y0=x/2+r,t.y1=t.y0+t.value*s):"top"==t.circularLinkType?(t.y0=y+r,t.y1=t.y0+t.value*s):(t.y0=x-t.value*s-r,t.y1=t.y0+t.value*s):0==l.top||0==l.bottom?(t.y0=(x-y)/e*r,t.y1=t.y0+t.value*s):(t.y0=(x-y)/2-e/2+r,t.y1=t.y0+t.value*s)})})})(s),I();for(var l=1,c=a;c>0;--c)p(l*=.99,s),I();function p(t,n){var r=A.length;A.forEach(function(i){var o=i.length,a=i[0].depth;i.forEach(function(i){var s;if(i.sourceLinks.length||i.targetLinks.length)if(i.partOfCycle&&w(i,n)>0);else if(0==a&&1==o)s=i.y1-i.y0,i.y0=x/2-s/2,i.y1=x/2+s/2;else if(a==r-1&&1==o)s=i.y1-i.y0,i.y0=x/2-s/2,i.y1=x/2+s/2;else{var A=e.mean(i.sourceLinks,f),l=e.mean(i.targetLinks,d),c=((A&&l?(A+l)/2:A||l)-h(i))*t;i.y0+=c,i.y1+=c}})})}function I(){A.forEach(function(e){var n,r,i,o=y,a=e.length;for(e.sort(u),i=0;i<a;++i)n=e[i],(r=o-n.y0)>0&&(n.y0+=r,n.y1+=r),o=n.y1+t;if((r=o-t-x)>0)for(o=n.y0-=r,n.y1-=r,i=a-2;i>=0;--i)n=e[i],(r=n.y1+t-o)>0&&(n.y0-=r,n.y1-=r),o=n.y0})}}(a,R,M),U(a);for(var s=0;s<4;s++)j(a,x,M),Y(a,0,M),O(a,y,x,M),j(a,x,M),Y(a,0,M);return function(t,n,r){var i=t.nodes,o=t.links,a=!1,s=!1;if(o.forEach(function(t){"top"==t.circularLinkType?a=!0:"bottom"==t.circularLinkType&&(s=!0)}),0==a||0==s){var A=e.min(i,function(t){return t.y0}),l=e.max(i,function(t){return t.y1}),c=l-A,u=r-n,g=u/c;i.forEach(function(t){var e=(t.y1-t.y0)*g;t.y0=(t.y0-A)*g,t.y1=t.y0+e}),o.forEach(function(t){t.y0=(t.y0-A)*g,t.y1=(t.y1-A)*g,t.width=t.width*g})}}(a,y,x),D(a,N,x,M),a}function U(t){t.nodes.forEach(function(t){t.sourceLinks.sort(c),t.targetLinks.sort(l)}),t.nodes.forEach(function(t){var e=t.y0,n=e,r=t.y1,i=r;t.sourceLinks.forEach(function(t){t.circular?(t.y0=r-t.width/2,r-=t.width):(t.y0=e+t.width/2,e+=t.width)}),t.targetLinks.forEach(function(t){t.circular?(t.y1=i-t.width/2,i-=t.width):(t.y1=n+t.width/2,n+=t.width)})})}return T.nodeId=function(t){return arguments.length?(M="function"==typeof t?t:s(t),T):M},T.nodeAlign=function(t){return arguments.length?(k="function"==typeof t?t:s(t),T):k},T.nodeWidth=function(t){return arguments.length?(_=+t,T):_},T.nodePadding=function(e){return arguments.length?(t=+e,T):t},T.nodes=function(t){return arguments.length?(S="function"==typeof t?t:s(t),T):S},T.links=function(t){return arguments.length?(L="function"==typeof t?t:s(t),T):L},T.size=function(t){return arguments.length?(o=y=0,Q=+t[0],x=+t[1],T):[Q-o,x-y]},T.extent=function(t){return arguments.length?(o=+t[0][0],Q=+t[1][0],y=+t[0][1],x=+t[1][1],T):[[o,y],[Q,x]]},T.iterations=function(t){return arguments.length?(R=+t,T):R},T.circularLinkGap=function(t){return arguments.length?(N=+t,T):N},T.nodePaddingRatio=function(t){return arguments.length?(r=+t,T):r},T.sortNodes=function(t){return arguments.length?(F=t,T):F},T.update=function(t){return b(t,M),U(t),t.links.forEach(function(t){t.circular&&(t.circularLinkType=t.y0+t.y1<x?"top":"bottom",t.source.circularLinkType=t.circularLinkType,t.target.circularLinkType=t.circularLinkType)}),j(t,x,M,!1),Y(t,0,M),D(t,N,x,M),t},T},t.sankeyCenter=function(t){return t.targetLinks.length?t.depth:t.sourceLinks.length?e.min(t.sourceLinks,o)-1:0},t.sankeyLeft=function(t){return t.depth},t.sankeyRight=function(t,e){return e-1-t.height},t.sankeyJustify=a,Object.defineProperty(t,"__esModule",{value:!0})},"object"==typeof n&&void 0!==e?i(n,t("d3-array"),t("d3-collection"),t("d3-shape"),t("elementary-circuits-directed-graph")):i(r.d3=r.d3||{},r.d3,r.d3,r.d3,null)},{"d3-array":152,"d3-collection":153,"d3-shape":161,"elementary-circuits-directed-graph":172}],54:[function(t,e,n){var r,i;r=this,i=function(t,e,n,r){"use strict";function i(t){return t.target.depth}function o(t,e){return t.sourceLinks.length?t.depth:e-1}function a(t){return function(){return t}}function s(t,e){return l(t.source,e.source)||t.index-e.index}function A(t,e){return l(t.target,e.target)||t.index-e.index}function l(t,e){return t.y0-e.y0}function c(t){return t.value}function u(t){return(t.y0+t.y1)/2}function g(t){return u(t.source)*t.value}function h(t){return u(t.target)*t.value}function d(t){return t.index}function f(t){return t.nodes}function p(t){return t.links}function I(t,e){var n=t.get(e);if(!n)throw new Error("missing: "+e);return n}function C(t){return[t.source.x1,t.y0]}function m(t){return[t.target.x0,t.y1]}t.sankey=function(){var t=0,r=0,i=1,C=1,m=24,y=8,E=d,v=o,B=f,b=p,Q=32,w=2/3;function x(){var o={nodes:B.apply(null,arguments),links:b.apply(null,arguments)};return function(t){t.nodes.forEach(function(t,e){t.index=e,t.sourceLinks=[],t.targetLinks=[]});var e=n.map(t.nodes,E);t.links.forEach(function(t,n){t.index=n;var r=t.source,i=t.target;"object"!=typeof r&&(r=t.source=I(e,r)),"object"!=typeof i&&(i=t.target=I(e,i)),r.sourceLinks.push(t),i.targetLinks.push(t)})}(o),function(t){t.nodes.forEach(function(t){t.value=Math.max(e.sum(t.sourceLinks,c),e.sum(t.targetLinks,c))})}(o),function(e){var n,r,o;for(n=e.nodes,r=[],o=0;n.length;++o,n=r,r=[])n.forEach(function(t){t.depth=o,t.sourceLinks.forEach(function(t){r.indexOf(t.target)<0&&r.push(t.target)})});for(n=e.nodes,r=[],o=0;n.length;++o,n=r,r=[])n.forEach(function(t){t.height=o,t.targetLinks.forEach(function(t){r.indexOf(t.source)<0&&r.push(t.source)})});var a=(i-t-m)/(o-1);e.nodes.forEach(function(e){e.x1=(e.x0=t+Math.max(0,Math.min(o-1,Math.floor(v.call(null,e,o))))*a)+m})}(o),function(t){var i=n.nest().key(function(t){return t.x0}).sortKeys(e.ascending).entries(t.nodes).map(function(t){return t.values});(function(){var n=e.max(i,function(t){return t.length}),o=w*(C-r)/(n-1);y>o&&(y=o);var a=e.min(i,function(t){return(C-r-(t.length-1)*y)/e.sum(t,c)});i.forEach(function(t){t.forEach(function(t,e){t.y1=(t.y0=e)+t.value*a})}),t.links.forEach(function(t){t.width=t.value*a})})(),d();for(var o=1,a=Q;a>0;--a)A(o*=.99),d(),s(o),d();function s(t){i.forEach(function(n){n.forEach(function(n){if(n.targetLinks.length){var r=(e.sum(n.targetLinks,g)/e.sum(n.targetLinks,c)-u(n))*t;n.y0+=r,n.y1+=r}})})}function A(t){i.slice().reverse().forEach(function(n){n.forEach(function(n){if(n.sourceLinks.length){var r=(e.sum(n.sourceLinks,h)/e.sum(n.sourceLinks,c)-u(n))*t;n.y0+=r,n.y1+=r}})})}function d(){i.forEach(function(t){var e,n,i,o=r,a=t.length;for(t.sort(l),i=0;i<a;++i)e=t[i],(n=o-e.y0)>0&&(e.y0+=n,e.y1+=n),o=e.y1+y;if((n=o-y-C)>0)for(o=e.y0-=n,e.y1-=n,i=a-2;i>=0;--i)e=t[i],(n=e.y1+y-o)>0&&(e.y0-=n,e.y1-=n),o=e.y0})}}(o),_(o),o}function _(t){t.nodes.forEach(function(t){t.sourceLinks.sort(A),t.targetLinks.sort(s)}),t.nodes.forEach(function(t){var e=t.y0,n=e;t.sourceLinks.forEach(function(t){t.y0=e+t.width/2,e+=t.width}),t.targetLinks.forEach(function(t){t.y1=n+t.width/2,n+=t.width})})}return x.update=function(t){return _(t),t},x.nodeId=function(t){return arguments.length?(E="function"==typeof t?t:a(t),x):E},x.nodeAlign=function(t){return arguments.length?(v="function"==typeof t?t:a(t),x):v},x.nodeWidth=function(t){return arguments.length?(m=+t,x):m},x.nodePadding=function(t){return arguments.length?(y=+t,x):y},x.nodes=function(t){return arguments.length?(B="function"==typeof t?t:a(t),x):B},x.links=function(t){return arguments.length?(b="function"==typeof t?t:a(t),x):b},x.size=function(e){return arguments.length?(t=r=0,i=+e[0],C=+e[1],x):[i-t,C-r]},x.extent=function(e){return arguments.length?(t=+e[0][0],i=+e[1][0],r=+e[0][1],C=+e[1][1],x):[[t,r],[i,C]]},x.iterations=function(t){return arguments.length?(Q=+t,x):Q},x},t.sankeyCenter=function(t){return t.targetLinks.length?t.depth:t.sourceLinks.length?e.min(t.sourceLinks,i)-1:0},t.sankeyLeft=function(t){return t.depth},t.sankeyRight=function(t,e){return e-1-t.height},t.sankeyJustify=o,t.sankeyLinkHorizontal=function(){return r.linkHorizontal().source(C).target(m)},Object.defineProperty(t,"__esModule",{value:!0})},"object"==typeof n&&void 0!==e?i(n,t("d3-array"),t("d3-collection"),t("d3-shape")):i(r.d3=r.d3||{},r.d3,r.d3,r.d3)},{"d3-array":152,"d3-collection":153,"d3-shape":161}],55:[function(t,e,n){"use strict";Object.defineProperty(n,"__esModule",{value:!0});var r=t("@turf/meta"),i=6378137;function o(t){var e=0;if(t&&t.length>0){e+=Math.abs(a(t[0]));for(var n=1;n<t.length;n++)e-=Math.abs(a(t[n]))}return e}function a(t){var e,n,r,o,a,A,l=0,c=t.length;if(c>2){for(A=0;A<c;A++)A===c-2?(r=c-2,o=c-1,a=0):A===c-1?(r=c-1,o=0,a=1):(r=A,o=A+1,a=A+2),e=t[r],n=t[o],l+=(s(t[a][0])-s(e[0]))*Math.sin(s(n[1]));l=l*i*i/2}return l}function s(t){return t*Math.PI/180}n.default=function(t){return r.geomReduce(t,function(t,e){return t+function(t){var e,n=0;switch(t.type){case"Polygon":return o(t.coordinates);case"MultiPolygon":for(e=0;e<t.coordinates.length;e++)n+=o(t.coordinates[e]);return n;case"Point":case"MultiPoint":case"LineString":case"MultiLineString":return 0}return 0}(e)},0)}},{"@turf/meta":58}],56:[function(t,e,n){"use strict";Object.defineProperty(n,"__esModule",{value:!0});var r=t("@turf/meta"),i=t("@turf/helpers");n.default=function(t,e){void 0===e&&(e={});var n=0,o=0,a=0;return r.coordEach(t,function(t){n+=t[0],o+=t[1],a++}),i.point([n/a,o/a],e.properties)}},{"@turf/helpers":57,"@turf/meta":58}],57:[function(t,e,n){"use strict";function r(t,e,n){void 0===n&&(n={});var r={type:"Feature"};return(0===n.id||n.id)&&(r.id=n.id),n.bbox&&(r.bbox=n.bbox),r.properties=e||{},r.geometry=t,r}function i(t,e,n){return void 0===n&&(n={}),r({type:"Point",coordinates:t},e,n)}function o(t,e,n){void 0===n&&(n={});for(var i=0,o=t;i<o.length;i++){var a=o[i];if(a.length<4)throw new Error("Each LinearRing of a Polygon must have 4 or more Positions.");for(var s=0;s<a[a.length-1].length;s++)if(a[a.length-1][s]!==a[0][s])throw new Error("First and last Position are not equivalent.")}return r({type:"Polygon",coordinates:t},e,n)}function a(t,e,n){if(void 0===n&&(n={}),t.length<2)throw new Error("coordinates must be an array of two or more positions");return r({type:"LineString",coordinates:t},e,n)}function s(t,e){void 0===e&&(e={});var n={type:"FeatureCollection"};return e.id&&(n.id=e.id),e.bbox&&(n.bbox=e.bbox),n.features=t,n}function A(t,e,n){return void 0===n&&(n={}),r({type:"MultiLineString",coordinates:t},e,n)}function l(t,e,n){return void 0===n&&(n={}),r({type:"MultiPoint",coordinates:t},e,n)}function c(t,e,n){return void 0===n&&(n={}),r({type:"MultiPolygon",coordinates:t},e,n)}function u(t,e){void 0===e&&(e="kilometers");var r=n.factors[e];if(!r)throw new Error(e+" units is invalid");return t*r}function g(t,e){void 0===e&&(e="kilometers");var r=n.factors[e];if(!r)throw new Error(e+" units is invalid");return t/r}function h(t){return t%(2*Math.PI)*180/Math.PI}function d(t){return!isNaN(t)&&null!==t&&!Array.isArray(t)&&!/^\s*$/.test(t)}Object.defineProperty(n,"__esModule",{value:!0}),n.earthRadius=6371008.8,n.factors={centimeters:100*n.earthRadius,centimetres:100*n.earthRadius,degrees:n.earthRadius/111325,feet:3.28084*n.earthRadius,inches:39.37*n.earthRadius,kilometers:n.earthRadius/1e3,kilometres:n.earthRadius/1e3,meters:n.earthRadius,metres:n.earthRadius,miles:n.earthRadius/1609.344,millimeters:1e3*n.earthRadius,millimetres:1e3*n.earthRadius,nauticalmiles:n.earthRadius/1852,radians:1,yards:n.earthRadius/1.0936},n.unitsFactors={centimeters:100,centimetres:100,degrees:1/111325,feet:3.28084,inches:39.37,kilometers:.001,kilometres:.001,meters:1,metres:1,miles:1/1609.344,millimeters:1e3,millimetres:1e3,nauticalmiles:1/1852,radians:1/n.earthRadius,yards:1/1.0936},n.areaFactors={acres:247105e-9,centimeters:1e4,centimetres:1e4,feet:10.763910417,inches:1550.003100006,kilometers:1e-6,kilometres:1e-6,meters:1,metres:1,miles:386e-9,millimeters:1e6,millimetres:1e6,yards:1.195990046},n.feature=r,n.geometry=function(t,e,n){switch(void 0===n&&(n={}),t){case"Point":return i(e).geometry;case"LineString":return a(e).geometry;case"Polygon":return o(e).geometry;case"MultiPoint":return l(e).geometry;case"MultiLineString":return A(e).geometry;case"MultiPolygon":return c(e).geometry;default:throw new Error(t+" is invalid")}},n.point=i,n.points=function(t,e,n){return void 0===n&&(n={}),s(t.map(function(t){return i(t,e)}),n)},n.polygon=o,n.polygons=function(t,e,n){return void 0===n&&(n={}),s(t.map(function(t){return o(t,e)}),n)},n.lineString=a,n.lineStrings=function(t,e,n){return void 0===n&&(n={}),s(t.map(function(t){return a(t,e)}),n)},n.featureCollection=s,n.multiLineString=A,n.multiPoint=l,n.multiPolygon=c,n.geometryCollection=function(t,e,n){return void 0===n&&(n={}),r({type:"GeometryCollection",geometries:t},e,n)},n.round=function(t,e){if(void 0===e&&(e=0),e&&!(e>=0))throw new Error("precision must be a positive number");var n=Math.pow(10,e||0);return Math.round(t*n)/n},n.radiansToLength=u,n.lengthToRadians=g,n.lengthToDegrees=function(t,e){return h(g(t,e))},n.bearingToAzimuth=function(t){var e=t%360;return e<0&&(e+=360),e},n.radiansToDegrees=h,n.degreesToRadians=function(t){return t%360*Math.PI/180},n.convertLength=function(t,e,n){if(void 0===e&&(e="kilometers"),void 0===n&&(n="kilometers"),!(t>=0))throw new Error("length must be a positive number");return u(g(t,e),n)},n.convertArea=function(t,e,r){if(void 0===e&&(e="meters"),void 0===r&&(r="kilometers"),!(t>=0))throw new Error("area must be a positive number");var i=n.areaFactors[e];if(!i)throw new Error("invalid original units");var o=n.areaFactors[r];if(!o)throw new Error("invalid final units");return t/i*o},n.isNumber=d,n.isObject=function(t){return!!t&&t.constructor===Object},n.validateBBox=function(t){if(!t)throw new Error("bbox is required");if(!Array.isArray(t))throw new Error("bbox must be an Array");if(4!==t.length&&6!==t.length)throw new Error("bbox must be an Array of 4 or 6 numbers");t.forEach(function(t){if(!d(t))throw new Error("bbox must only contain numbers")})},n.validateId=function(t){if(!t)throw new Error("id is required");if(-1===["string","number"].indexOf(typeof t))throw new Error("id must be a number or a string")},n.radians2degrees=function(){throw new Error("method has been renamed to `radiansToDegrees`")},n.degrees2radians=function(){throw new Error("method has been renamed to `degreesToRadians`")},n.distanceToDegrees=function(){throw new Error("method has been renamed to `lengthToDegrees`")},n.distanceToRadians=function(){throw new Error("method has been renamed to `lengthToRadians`")},n.radiansToDistance=function(){throw new Error("method has been renamed to `radiansToLength`")},n.bearingToAngle=function(){throw new Error("method has been renamed to `bearingToAzimuth`")},n.convertDistance=function(){throw new Error("method has been renamed to `convertLength`")}},{}],58:[function(t,e,n){"use strict";Object.defineProperty(n,"__esModule",{value:!0});var r=t("@turf/helpers");function i(t,e,n){if(null!==t)for(var r,o,a,s,A,l,c,u,g=0,h=0,d=t.type,f="FeatureCollection"===d,p="Feature"===d,I=f?t.features.length:1,C=0;C<I;C++){A=(u=!!(c=f?t.features[C].geometry:p?t.geometry:t)&&"GeometryCollection"===c.type)?c.geometries.length:1;for(var m=0;m<A;m++){var y=0,E=0;if(null!==(s=u?c.geometries[m]:c)){l=s.coordinates;var v=s.type;switch(g=!n||"Polygon"!==v&&"MultiPolygon"!==v?0:1,v){case null:break;case"Point":if(!1===e(l,h,C,y,E))return!1;h++,y++;break;case"LineString":case"MultiPoint":for(r=0;r<l.length;r++){if(!1===e(l[r],h,C,y,E))return!1;h++,"MultiPoint"===v&&y++}"LineString"===v&&y++;break;case"Polygon":case"MultiLineString":for(r=0;r<l.length;r++){for(o=0;o<l[r].length-g;o++){if(!1===e(l[r][o],h,C,y,E))return!1;h++}"MultiLineString"===v&&y++,"Polygon"===v&&E++}"Polygon"===v&&y++;break;case"MultiPolygon":for(r=0;r<l.length;r++){for(E=0,o=0;o<l[r].length;o++){for(a=0;a<l[r][o].length-g;a++){if(!1===e(l[r][o][a],h,C,y,E))return!1;h++}E++}y++}break;case"GeometryCollection":for(r=0;r<s.geometries.length;r++)if(!1===i(s.geometries[r],e,n))return!1;break;default:throw new Error("Unknown Geometry Type")}}}}}function o(t,e){var n;switch(t.type){case"FeatureCollection":for(n=0;n<t.features.length&&!1!==e(t.features[n].properties,n);n++);break;case"Feature":e(t.properties,0)}}function a(t,e){if("Feature"===t.type)e(t,0);else if("FeatureCollection"===t.type)for(var n=0;n<t.features.length&&!1!==e(t.features[n],n);n++);}function s(t,e){var n,r,i,o,a,s,A,l,c,u,g=0,h="FeatureCollection"===t.type,d="Feature"===t.type,f=h?t.features.length:1;for(n=0;n<f;n++){for(s=h?t.features[n].geometry:d?t.geometry:t,l=h?t.features[n].properties:d?t.properties:{},c=h?t.features[n].bbox:d?t.bbox:void 0,u=h?t.features[n].id:d?t.id:void 0,a=(A=!!s&&"GeometryCollection"===s.type)?s.geometries.length:1,i=0;i<a;i++)if(null!==(o=A?s.geometries[i]:s))switch(o.type){case"Point":case"LineString":case"MultiPoint":case"Polygon":case"MultiLineString":case"MultiPolygon":if(!1===e(o,g,l,c,u))return!1;break;case"GeometryCollection":for(r=0;r<o.geometries.length;r++)if(!1===e(o.geometries[r],g,l,c,u))return!1;break;default:throw new Error("Unknown Geometry Type")}else if(!1===e(null,g,l,c,u))return!1;g++}}function A(t,e){s(t,function(t,n,i,o,a){var s,A=null===t?null:t.type;switch(A){case null:case"Point":case"LineString":case"Polygon":return!1!==e(r.feature(t,i,{bbox:o,id:a}),n,0)&&void 0}switch(A){case"MultiPoint":s="Point";break;case"MultiLineString":s="LineString";break;case"MultiPolygon":s="Polygon"}for(var l=0;l<t.coordinates.length;l++){var c={type:s,coordinates:t.coordinates[l]};if(!1===e(r.feature(c,i),n,l))return!1}})}function l(t,e){A(t,function(t,n,o){var a=0;if(t.geometry){var s=t.geometry.type;if("Point"!==s&&"MultiPoint"!==s){var A,l=0,c=0,u=0;return!1!==i(t,function(i,s,g,h,d){if(void 0===A||n>l||h>c||d>u)return A=i,l=n,c=h,u=d,void(a=0);var f=r.lineString([A,i],t.properties);if(!1===e(f,n,o,d,a))return!1;a++,A=i})&&void 0}}})}function c(t,e){if(!t)throw new Error("geojson is required");A(t,function(t,n,i){if(null!==t.geometry){var o=t.geometry.type,a=t.geometry.coordinates;switch(o){case"LineString":if(!1===e(t,n,i,0,0))return!1;break;case"Polygon":for(var s=0;s<a.length;s++)if(!1===e(r.lineString(a[s],t.properties),n,i,s))return!1}}})}n.coordEach=i,n.coordReduce=function(t,e,n,r){var o=n;return i(t,function(t,r,i,a,s){o=0===r&&void 0===n?t:e(o,t,r,i,a,s)},r),o},n.propEach=o,n.propReduce=function(t,e,n){var r=n;return o(t,function(t,i){r=0===i&&void 0===n?t:e(r,t,i)}),r},n.featureEach=a,n.featureReduce=function(t,e,n){var r=n;return a(t,function(t,i){r=0===i&&void 0===n?t:e(r,t,i)}),r},n.coordAll=function(t){var e=[];return i(t,function(t){e.push(t)}),e},n.geomEach=s,n.geomReduce=function(t,e,n){var r=n;return s(t,function(t,i,o,a,s){r=0===i&&void 0===n?t:e(r,t,i,o,a,s)}),r},n.flattenEach=A,n.flattenReduce=function(t,e,n){var r=n;return A(t,function(t,i,o){r=0===i&&0===o&&void 0===n?t:e(r,t,i,o)}),r},n.segmentEach=l,n.segmentReduce=function(t,e,n){var r=n,i=!1;return l(t,function(t,o,a,s,A){r=!1===i&&void 0===n?t:e(r,t,o,a,s,A),i=!0}),r},n.lineEach=c,n.lineReduce=function(t,e,n){var r=n;return c(t,function(t,i,o,a){r=0===i&&void 0===n?t:e(r,t,i,o,a)}),r},n.findSegment=function(t,e){if(e=e||{},!r.isObject(e))throw new Error("options is invalid");var n,i=e.featureIndex||0,o=e.multiFeatureIndex||0,a=e.geometryIndex||0,s=e.segmentIndex||0,A=e.properties;switch(t.type){case"FeatureCollection":i<0&&(i=t.features.length+i),A=A||t.features[i].properties,n=t.features[i].geometry;break;case"Feature":A=A||t.properties,n=t.geometry;break;case"Point":case"MultiPoint":return null;case"LineString":case"Polygon":case"MultiLineString":case"MultiPolygon":n=t;break;default:throw new Error("geojson is invalid")}if(null===n)return null;var l=n.coordinates;switch(n.type){case"Point":case"MultiPoint":return null;case"LineString":return s<0&&(s=l.length+s-1),r.lineString([l[s],l[s+1]],A,e);case"Polygon":return a<0&&(a=l.length+a),s<0&&(s=l[a].length+s-1),r.lineString([l[a][s],l[a][s+1]],A,e);case"MultiLineString":return o<0&&(o=l.length+o),s<0&&(s=l[o].length+s-1),r.lineString([l[o][s],l[o][s+1]],A,e);case"MultiPolygon":return o<0&&(o=l.length+o),a<0&&(a=l[o].length+a),s<0&&(s=l[o][a].length-s-1),r.lineString([l[o][a][s],l[o][a][s+1]],A,e)}throw new Error("geojson is invalid")},n.findPoint=function(t,e){if(e=e||{},!r.isObject(e))throw new Error("options is invalid");var n,i=e.featureIndex||0,o=e.multiFeatureIndex||0,a=e.geometryIndex||0,s=e.coordIndex||0,A=e.properties;switch(t.type){case"FeatureCollection":i<0&&(i=t.features.length+i),A=A||t.features[i].properties,n=t.features[i].geometry;break;case"Feature":A=A||t.properties,n=t.geometry;break;case"Point":case"MultiPoint":return null;case"LineString":case"Polygon":case"MultiLineString":case"MultiPolygon":n=t;break;default:throw new Error("geojson is invalid")}if(null===n)return null;var l=n.coordinates;switch(n.type){case"Point":return r.point(l,A,e);case"MultiPoint":return o<0&&(o=l.length+o),r.point(l[o],A,e);case"LineString":return s<0&&(s=l.length+s),r.point(l[s],A,e);case"Polygon":return a<0&&(a=l.length+a),s<0&&(s=l[a].length+s),r.point(l[a][s],A,e);case"MultiLineString":return o<0&&(o=l.length+o),s<0&&(s=l[o].length+s),r.point(l[o][s],A,e);case"MultiPolygon":return o<0&&(o=l.length+o),a<0&&(a=l[o].length+a),s<0&&(s=l[o][a].length-s),r.point(l[o][a][s],A,e)}throw new Error("geojson is invalid")}},{"@turf/helpers":57}],59:[function(t,e,n){"use strict";var r="undefined"==typeof WeakMap?t("weak-map"):WeakMap,i=t("gl-buffer"),o=t("gl-vao"),a=new r;e.exports=function(t){var e=a.get(t),n=e&&(e._triangleBuffer.handle||e._triangleBuffer.buffer);if(!n||!t.isBuffer(n)){var r=i(t,new Float32Array([-1,-1,-1,4,4,-1]));(e=o(t,[{buffer:r,type:t.FLOAT,size:2}]))._triangleBuffer=r,a.set(t,e)}e.bind(),t.drawArrays(t.TRIANGLES,0,3),e.unbind()}},{"gl-buffer":241,"gl-vao":327,"weak-map":552}],60:[function(t,e,n){e.exports=function(t){var e=0,n=0,r=0,i=0;return t.map(function(t){var o=(t=t.slice())[0],a=o.toUpperCase();if(o!=a)switch(t[0]=a,o){case"a":t[6]+=r,t[7]+=i;break;case"v":t[1]+=i;break;case"h":t[1]+=r;break;default:for(var s=1;s<t.length;)t[s++]+=r,t[s++]+=i}switch(a){case"Z":r=e,i=n;break;case"H":r=t[1];break;case"V":i=t[1];break;case"M":r=e=t[1],i=n=t[2];break;default:r=t[t.length-2],i=t[t.length-1]}return t})}},{}],61:[function(t,e,n){var r=t("pad-left");e.exports=function(t,e,n){e="number"==typeof e?e:1,n=n||": ";var i=t.split(/\r?\n/),o=String(i.length+e-1).length;return i.map(function(t,i){var a=i+e,s=String(a).length,A=r(a,o-s);return A+n+t}).join("\n")}},{"pad-left":457}],62:[function(t,e,n){"use strict";e.exports=function(t){var e=t.length;if(0===e)return[];if(1===e)return[0];for(var n=t[0].length,r=[t[0]],o=[0],a=1;a<e;++a)if(r.push(t[a]),i(r,n)){if(o.push(a),o.length===n+1)return o}else r.pop();return o};var r=t("robust-orientation");function i(t,e){for(var n=new Array(e+1),i=0;i<t.length;++i)n[i]=t[i];for(i=0;i<=t.length;++i){for(var o=t.length;o<=e;++o){for(var a=new Array(e),s=0;s<e;++s)a[s]=Math.pow(o+1-i,s);n[o]=a}if(r.apply(void 0,n))return!0}return!1}},{"robust-orientation":510}],63:[function(t,e,n){"use strict";e.exports=function(t,e){return r(e).filter(function(n){for(var r=new Array(n.length),o=0;o<n.length;++o)r[o]=e[n[o]];return i(r)*t<1})};var r=t("delaunay-triangulate"),i=t("circumradius")},{circumradius:114,"delaunay-triangulate":165}],64:[function(t,e,n){e.exports=function(t,e){return i(r(t,e))};var r=t("alpha-complex"),i=t("simplicial-complex-boundary")},{"alpha-complex":63,"simplicial-complex-boundary":517}],65:[function(t,e,n){"use strict";e.exports=function(t,e){if(!t||null==t.length)throw Error("Argument should be an array");e=null==e?1:Math.floor(e);for(var n=Array(2*e),r=0;r<e;r++){for(var i=-1/0,o=1/0,a=r,s=t.length;a<s;a+=e)t[a]>i&&(i=t[a]),t[a]<o&&(o=t[a]);n[r]=o,n[e+r]=i}return n}},{}],66:[function(t,e,n){"use strict";var r=t("array-bounds");e.exports=function(t,e,n){if(!t||null==t.length)throw Error("Argument should be an array");null==e&&(e=1),null==n&&(n=r(t,e));for(var i=0;i<e;i++){var o=n[e+i],a=n[i],s=i,A=t.length;if(o===1/0&&a===-1/0)for(s=i;s<A;s+=e)t[s]=t[s]===o?1:t[s]===a?0:.5;else if(o===1/0)for(s=i;s<A;s+=e)t[s]=t[s]===o?1:0;else if(a===-1/0)for(s=i;s<A;s+=e)t[s]=t[s]===a?0:1;else{var l=o-a;for(s=i;s<A;s+=e)t[s]=0===l?.5:(t[s]-a)/l}}return t}},{"array-bounds":65}],67:[function(t,e,n){e.exports=function(t,e){var n="number"==typeof t,r="number"==typeof e;n&&!r?(e=t,t=0):n||r||(t=0,e=0);var i=(e|=0)-(t|=0);if(i<0)throw new Error("array length must be positive");for(var o=new Array(i),a=0,s=t;a<i;a++,s++)o[a]=s;return o}},{}],68:[function(t,n,r){(function(e){"use strict";var r=t("object-assign");function i(t,e){if(t===e)return 0;for(var n=t.length,r=e.length,i=0,o=Math.min(n,r);i<o;++i)if(t[i]!==e[i]){n=t[i],r=e[i];break}return n<r?-1:r<n?1:0}function o(t){return e.Buffer&&"function"==typeof e.Buffer.isBuffer?e.Buffer.isBuffer(t):!(null==t||!t._isBuffer)}var a=t("util/"),s=Object.prototype.hasOwnProperty,A=Array.prototype.slice,l="foo"===function(){}.name;function c(t){return Object.prototype.toString.call(t)}function u(t){return!o(t)&&"function"==typeof e.ArrayBuffer&&("function"==typeof ArrayBuffer.isView?ArrayBuffer.isView(t):!!t&&(t instanceof DataView||!!(t.buffer&&t.buffer instanceof ArrayBuffer)))}var g=n.exports=C,h=/\s*function\s+([^\(\s]*)\s*/;function d(t){if(a.isFunction(t)){if(l)return t.name;var e=t.toString().match(h);return e&&e[1]}}function f(t,e){return"string"==typeof t?t.length<e?t:t.slice(0,e):t}function p(t){if(l||!a.isFunction(t))return a.inspect(t);var e=d(t);return"[Function"+(e?": "+e:"")+"]"}function I(t,e,n,r,i){throw new g.AssertionError({message:n,actual:t,expected:e,operator:r,stackStartFunction:i})}function C(t,e){t||I(t,!0,e,"==",g.ok)}function m(t,e,n,r){if(t===e)return!0;if(o(t)&&o(e))return 0===i(t,e);if(a.isDate(t)&&a.isDate(e))return t.getTime()===e.getTime();if(a.isRegExp(t)&&a.isRegExp(e))return t.source===e.source&&t.global===e.global&&t.multiline===e.multiline&&t.lastIndex===e.lastIndex&&t.ignoreCase===e.ignoreCase;if(null!==t&&"object"==typeof t||null!==e&&"object"==typeof e){if(u(t)&&u(e)&&c(t)===c(e)&&!(t instanceof Float32Array||t instanceof Float64Array))return 0===i(new Uint8Array(t.buffer),new Uint8Array(e.buffer));if(o(t)!==o(e))return!1;var s=(r=r||{actual:[],expected:[]}).actual.indexOf(t);return-1!==s&&s===r.expected.indexOf(e)||(r.actual.push(t),r.expected.push(e),function(t,e,n,r){if(null==t||null==e)return!1;if(a.isPrimitive(t)||a.isPrimitive(e))return t===e;if(n&&Object.getPrototypeOf(t)!==Object.getPrototypeOf(e))return!1;var i=y(t),o=y(e);if(i&&!o||!i&&o)return!1;if(i)return t=A.call(t),e=A.call(e),m(t,e,n);var s,l,c=B(t),u=B(e);if(c.length!==u.length)return!1;for(c.sort(),u.sort(),l=c.length-1;l>=0;l--)if(c[l]!==u[l])return!1;for(l=c.length-1;l>=0;l--)if(s=c[l],!m(t[s],e[s],n,r))return!1;return!0}(t,e,n,r))}return n?t===e:t==e}function y(t){return"[object Arguments]"==Object.prototype.toString.call(t)}function E(t,e){if(!t||!e)return!1;if("[object RegExp]"==Object.prototype.toString.call(e))return e.test(t);try{if(t instanceof e)return!0}catch(t){}return!Error.isPrototypeOf(e)&&!0===e.call({},t)}function v(t,e,n,r){var i;if("function"!=typeof e)throw new TypeError('"block" argument must be a function');"string"==typeof n&&(r=n,n=null),i=function(t){var e;try{t()}catch(t){e=t}return e}(e),r=(n&&n.name?" ("+n.name+").":".")+(r?" "+r:"."),t&&!i&&I(i,n,"Missing expected exception"+r);var o="string"==typeof r,s=!t&&i&&!n;if((!t&&a.isError(i)&&o&&E(i,n)||s)&&I(i,n,"Got unwanted exception"+r),t&&i&&n&&!E(i,n)||!t&&i)throw i}g.AssertionError=function(t){this.name="AssertionError",this.actual=t.actual,this.expected=t.expected,this.operator=t.operator,t.message?(this.message=t.message,this.generatedMessage=!1):(this.message=f(p(this.actual),128)+" "+this.operator+" "+f(p(this.expected),128),this.generatedMessage=!0);var e=t.stackStartFunction||I;if(Error.captureStackTrace)Error.captureStackTrace(this,e);else{var n=new Error;if(n.stack){var r=n.stack,i=d(e),o=r.indexOf("\n"+i);if(o>=0){var a=r.indexOf("\n",o+1);r=r.substring(a+1)}this.stack=r}}},a.inherits(g.AssertionError,Error),g.fail=I,g.ok=C,g.equal=function(t,e,n){t!=e&&I(t,e,n,"==",g.equal)},g.notEqual=function(t,e,n){t==e&&I(t,e,n,"!=",g.notEqual)},g.deepEqual=function(t,e,n){m(t,e,!1)||I(t,e,n,"deepEqual",g.deepEqual)},g.deepStrictEqual=function(t,e,n){m(t,e,!0)||I(t,e,n,"deepStrictEqual",g.deepStrictEqual)},g.notDeepEqual=function(t,e,n){m(t,e,!1)&&I(t,e,n,"notDeepEqual",g.notDeepEqual)},g.notDeepStrictEqual=function t(e,n,r){m(e,n,!0)&&I(e,n,r,"notDeepStrictEqual",t)},g.strictEqual=function(t,e,n){t!==e&&I(t,e,n,"===",g.strictEqual)},g.notStrictEqual=function(t,e,n){t===e&&I(t,e,n,"!==",g.notStrictEqual)},g.throws=function(t,e,n){v(!0,t,e,n)},g.doesNotThrow=function(t,e,n){v(!1,t,e,n)},g.ifError=function(t){if(t)throw t},g.strict=r(function t(e,n){e||I(e,!0,n,"==",t)},g,{equal:g.strictEqual,deepEqual:g.deepStrictEqual,notEqual:g.notStrictEqual,notDeepEqual:g.notDeepStrictEqual}),g.strict.strict=g.strict;var B=Object.keys||function(t){var e=[];for(var n in t)s.call(t,n)&&e.push(n);return e}}).call(this,void 0!==e?e:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{"object-assign":454,"util/":71}],69:[function(t,e,n){"function"==typeof Object.create?e.exports=function(t,e){t.super_=e,t.prototype=Object.create(e.prototype,{constructor:{value:t,enumerable:!1,writable:!0,configurable:!0}})}:e.exports=function(t,e){t.super_=e;var n=function(){};n.prototype=e.prototype,t.prototype=new n,t.prototype.constructor=t}},{}],70:[function(t,e,n){e.exports=function(t){return t&&"object"==typeof t&&"function"==typeof t.copy&&"function"==typeof t.fill&&"function"==typeof t.readUInt8}},{}],71:[function(t,n,r){(function(e,n){var i=/%[sdj%]/g;r.format=function(t){if(!I(t)){for(var e=[],n=0;n<arguments.length;n++)e.push(s(arguments[n]));return e.join(" ")}n=1;for(var r=arguments,o=r.length,a=String(t).replace(i,function(t){if("%%"===t)return"%";if(n>=o)return t;switch(t){case"%s":return String(r[n++]);case"%d":return Number(r[n++]);case"%j":try{return JSON.stringify(r[n++])}catch(t){return"[Circular]"}default:return t}}),A=r[n];n<o;A=r[++n])f(A)||!y(A)?a+=" "+A:a+=" "+s(A);return a},r.deprecate=function(t,i){if(C(n.process))return function(){return r.deprecate(t,i).apply(this,arguments)};if(!0===e.noDeprecation)return t;var o=!1;return function(){if(!o){if(e.throwDeprecation)throw new Error(i);e.traceDeprecation?console.trace(i):console.error(i),o=!0}return t.apply(this,arguments)}};var o,a={};function s(t,e){var n={seen:[],stylize:l};return arguments.length>=3&&(n.depth=arguments[2]),arguments.length>=4&&(n.colors=arguments[3]),d(e)?n.showHidden=e:e&&r._extend(n,e),C(n.showHidden)&&(n.showHidden=!1),C(n.depth)&&(n.depth=2),C(n.colors)&&(n.colors=!1),C(n.customInspect)&&(n.customInspect=!0),n.colors&&(n.stylize=A),c(n,t,n.depth)}function A(t,e){var n=s.styles[e];return n?"["+s.colors[n][0]+"m"+t+"["+s.colors[n][1]+"m":t}function l(t,e){return t}function c(t,e,n){if(t.customInspect&&e&&B(e.inspect)&&e.inspect!==r.inspect&&(!e.constructor||e.constructor.prototype!==e)){var i=e.inspect(n,t);return I(i)||(i=c(t,i,n)),i}var o=function(t,e){if(C(e))return t.stylize("undefined","undefined");if(I(e)){var n="'"+JSON.stringify(e).replace(/^"|"$/g,"").replace(/'/g,"\\'").replace(/\\"/g,'"')+"'";return t.stylize(n,"string")}return p(e)?t.stylize(""+e,"number"):d(e)?t.stylize(""+e,"boolean"):f(e)?t.stylize("null","null"):void 0}(t,e);if(o)return o;var a=Object.keys(e),s=function(t){var e={};return t.forEach(function(t,n){e[t]=!0}),e}(a);if(t.showHidden&&(a=Object.getOwnPropertyNames(e)),v(e)&&(a.indexOf("message")>=0||a.indexOf("description")>=0))return u(e);if(0===a.length){if(B(e)){var A=e.name?": "+e.name:"";return t.stylize("[Function"+A+"]","special")}if(m(e))return t.stylize(RegExp.prototype.toString.call(e),"regexp");if(E(e))return t.stylize(Date.prototype.toString.call(e),"date");if(v(e))return u(e)}var l,y="",b=!1,Q=["{","}"];return h(e)&&(b=!0,Q=["[","]"]),B(e)&&(y=" [Function"+(e.name?": "+e.name:"")+"]"),m(e)&&(y=" "+RegExp.prototype.toString.call(e)),E(e)&&(y=" "+Date.prototype.toUTCString.call(e)),v(e)&&(y=" "+u(e)),0!==a.length||b&&0!=e.length?n<0?m(e)?t.stylize(RegExp.prototype.toString.call(e),"regexp"):t.stylize("[Object]","special"):(t.seen.push(e),l=b?function(t,e,n,r,i){for(var o=[],a=0,s=e.length;a<s;++a)x(e,String(a))?o.push(g(t,e,n,r,String(a),!0)):o.push("");return i.forEach(function(i){i.match(/^\d+$/)||o.push(g(t,e,n,r,i,!0))}),o}(t,e,n,s,a):a.map(function(r){return g(t,e,n,s,r,b)}),t.seen.pop(),function(t,e,n){return t.reduce(function(t,e){return e.indexOf("\n"),t+e.replace(/\u001b\[\d\d?m/g,"").length+1},0)>60?n[0]+(""===e?"":e+"\n ")+" "+t.join(",\n  ")+" "+n[1]:n[0]+e+" "+t.join(", ")+" "+n[1]}(l,y,Q)):Q[0]+y+Q[1]}function u(t){return"["+Error.prototype.toString.call(t)+"]"}function g(t,e,n,r,i,o){var a,s,A;if((A=Object.getOwnPropertyDescriptor(e,i)||{value:e[i]}).get?s=A.set?t.stylize("[Getter/Setter]","special"):t.stylize("[Getter]","special"):A.set&&(s=t.stylize("[Setter]","special")),x(r,i)||(a="["+i+"]"),s||(t.seen.indexOf(A.value)<0?(s=f(n)?c(t,A.value,null):c(t,A.value,n-1)).indexOf("\n")>-1&&(s=o?s.split("\n").map(function(t){return"  "+t}).join("\n").substr(2):"\n"+s.split("\n").map(function(t){return"   "+t}).join("\n")):s=t.stylize("[Circular]","special")),C(a)){if(o&&i.match(/^\d+$/))return s;(a=JSON.stringify(""+i)).match(/^"([a-zA-Z_][a-zA-Z_0-9]*)"$/)?(a=a.substr(1,a.length-2),a=t.stylize(a,"name")):(a=a.replace(/'/g,"\\'").replace(/\\"/g,'"').replace(/(^"|"$)/g,"'"),a=t.stylize(a,"string"))}return a+": "+s}function h(t){return Array.isArray(t)}function d(t){return"boolean"==typeof t}function f(t){return null===t}function p(t){return"number"==typeof t}function I(t){return"string"==typeof t}function C(t){return void 0===t}function m(t){return y(t)&&"[object RegExp]"===b(t)}function y(t){return"object"==typeof t&&null!==t}function E(t){return y(t)&&"[object Date]"===b(t)}function v(t){return y(t)&&("[object Error]"===b(t)||t instanceof Error)}function B(t){return"function"==typeof t}function b(t){return Object.prototype.toString.call(t)}function Q(t){return t<10?"0"+t.toString(10):t.toString(10)}r.debuglog=function(t){if(C(o)&&(o=e.env.NODE_DEBUG||""),t=t.toUpperCase(),!a[t])if(new RegExp("\\b"+t+"\\b","i").test(o)){var n=e.pid;a[t]=function(){var e=r.format.apply(r,arguments);console.error("%s %d: %s",t,n,e)}}else a[t]=function(){};return a[t]},r.inspect=s,s.colors={bold:[1,22],italic:[3,23],underline:[4,24],inverse:[7,27],white:[37,39],grey:[90,39],black:[30,39],blue:[34,39],cyan:[36,39],green:[32,39],magenta:[35,39],red:[31,39],yellow:[33,39]},s.styles={special:"cyan",number:"yellow",boolean:"yellow",undefined:"grey",null:"bold",string:"green",date:"magenta",regexp:"red"},r.isArray=h,r.isBoolean=d,r.isNull=f,r.isNullOrUndefined=function(t){return null==t},r.isNumber=p,r.isString=I,r.isSymbol=function(t){return"symbol"==typeof t},r.isUndefined=C,r.isRegExp=m,r.isObject=y,r.isDate=E,r.isError=v,r.isFunction=B,r.isPrimitive=function(t){return null===t||"boolean"==typeof t||"number"==typeof t||"string"==typeof t||"symbol"==typeof t||void 0===t},r.isBuffer=t("./support/isBuffer");var w=["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"];function x(t,e){return Object.prototype.hasOwnProperty.call(t,e)}r.log=function(){var t,e;console.log("%s - %s",(t=new Date,e=[Q(t.getHours()),Q(t.getMinutes()),Q(t.getSeconds())].join(":"),[t.getDate(),w[t.getMonth()],e].join(" ")),r.format.apply(r,arguments))},r.inherits=t("inherits"),r._extend=function(t,e){if(!e||!y(e))return t;for(var n=Object.keys(e),r=n.length;r--;)t[n[r]]=e[n[r]];return t}}).call(this,t("_process"),void 0!==e?e:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{"./support/isBuffer":70,_process:482,inherits:69}],72:[function(t,e,n){e.exports=function(t){return atob(t)}},{}],73:[function(t,e,n){"use strict";e.exports=function(t,e){for(var n=e.length,o=new Array(n+1),a=0;a<n;++a){for(var s=new Array(n+1),A=0;A<=n;++A)s[A]=t[A][a];o[a]=s}o[n]=new Array(n+1);for(var a=0;a<=n;++a)o[n][a]=1;for(var l=new Array(n+1),a=0;a<n;++a)l[a]=e[a];l[n]=1;var c=r(o,l),u=i(c[n+1]);0===u&&(u=1);for(var g=new Array(n+1),a=0;a<=n;++a)g[a]=i(c[a])/u;return g};var r=t("robust-linear-solve");function i(t){for(var e=0,n=0;n<t.length;++n)e+=t[n];return e}},{"robust-linear-solve":509}],74:[function(t,e,n){"use strict";n.byteLength=function(t){var e=l(t),n=e[0],r=e[1];return 3*(n+r)/4-r},n.toByteArray=function(t){for(var e,n=l(t),r=n[0],a=n[1],s=new o(NaN),A=0,c=a>0?r-4:r,u=0;u<c;u+=4)e=i[t.charCodeAt(u)]<<18|i[t.charCodeAt(u+1)]<<12|i[t.charCodeAt(u+2)]<<6|i[t.charCodeAt(u+3)],s[A++]=e>>16&255,s[A++]=e>>8&255,s[A++]=255&e;return 2===a&&(e=i[t.charCodeAt(u)]<<2|i[t.charCodeAt(u+1)]>>4,s[A++]=255&e),1===a&&(e=i[t.charCodeAt(u)]<<10|i[t.charCodeAt(u+1)]<<4|i[t.charCodeAt(u+2)]>>2,s[A++]=e>>8&255,s[A++]=255&e),s},n.fromByteArray=function(t){for(var e,n=t.length,i=n%3,o=[],a=0,s=n-i;a<s;a+=16383)o.push(c(t,a,a+16383>s?s:a+16383));return 1===i?(e=t[n-1],o.push(r[e>>2]+r[e<<4&63]+"==")):2===i&&(e=(t[n-2]<<8)+t[n-1],o.push(r[e>>10]+r[e>>4&63]+r[e<<2&63]+"=")),o.join("")};for(var r=[],i=[],o="undefined"!=typeof Uint8Array?Uint8Array:Array,a="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",s=0,A=a.length;s<A;++s)r[s]=a[s],i[a.charCodeAt(s)]=s;function l(t){var e=t.length;if(e%4>0)throw new Error("Invalid string. Length must be a multiple of 4");var n=t.indexOf("=");return-1===n&&(n=e),[n,n===e?0:4-n%4]}function c(t,e,n){for(var i,o,a=[],s=e;s<n;s+=3)i=(t[s]<<16&16711680)+(t[s+1]<<8&65280)+(255&t[s+2]),a.push(r[(o=i)>>18&63]+r[o>>12&63]+r[o>>6&63]+r[63&o]);return a.join("")}i["-".charCodeAt(0)]=62,i["_".charCodeAt(0)]=63},{}],75:[function(t,e,n){"use strict";var r=t("./lib/rationalize");e.exports=function(t,e){return r(t[0].mul(e[1]).add(e[0].mul(t[1])),t[1].mul(e[1]))}},{"./lib/rationalize":85}],76:[function(t,e,n){"use strict";e.exports=function(t,e){return t[0].mul(e[1]).cmp(e[0].mul(t[1]))}},{}],77:[function(t,e,n){"use strict";var r=t("./lib/rationalize");e.exports=function(t,e){return r(t[0].mul(e[1]),t[1].mul(e[0]))}},{"./lib/rationalize":85}],78:[function(t,e,n){"use strict";var r=t("./is-rat"),i=t("./lib/is-bn"),o=t("./lib/num-to-bn"),a=t("./lib/str-to-bn"),s=t("./lib/rationalize"),A=t("./div");e.exports=function t(e,n){if(r(e))return n?A(e,t(n)):[e[0].clone(),e[1].clone()];var l,c,u=0;if(i(e))l=e.clone();else if("string"==typeof e)l=a(e);else{if(0===e)return[o(0),o(1)];if(e===Math.floor(e))l=o(e);else{for(;e!==Math.floor(e);)e*=Math.pow(2,256),u-=256;l=o(e)}}if(r(n))l.mul(n[1]),c=n[0].clone();else if(i(n))c=n.clone();else if("string"==typeof n)c=a(n);else if(n)if(n===Math.floor(n))c=o(n);else{for(;n!==Math.floor(n);)n*=Math.pow(2,256),u+=256;c=o(n)}else c=o(1);return u>0?l=l.ushln(u):u<0&&(c=c.ushln(-u)),s(l,c)}},{"./div":77,"./is-rat":79,"./lib/is-bn":83,"./lib/num-to-bn":84,"./lib/rationalize":85,"./lib/str-to-bn":86}],79:[function(t,e,n){"use strict";var r=t("./lib/is-bn");e.exports=function(t){return Array.isArray(t)&&2===t.length&&r(t[0])&&r(t[1])}},{"./lib/is-bn":83}],80:[function(t,e,n){"use strict";var r=t("bn.js");e.exports=function(t){return t.cmp(new r(0))}},{"bn.js":94}],81:[function(t,e,n){"use strict";var r=t("./bn-sign");e.exports=function(t){var e=t.length,n=t.words,i=0;if(1===e)i=n[0];else if(2===e)i=n[0]+67108864*n[1];else for(var o=0;o<e;o++){var a=n[o];i+=a*Math.pow(67108864,o)}return r(t)*i}},{"./bn-sign":80}],82:[function(t,e,n){"use strict";var r=t("double-bits"),i=t("bit-twiddle").countTrailingZeros;e.exports=function(t){var e=i(r.lo(t));if(e<32)return e;var n=i(r.hi(t));return n>20?52:n+32}},{"bit-twiddle":92,"double-bits":167}],83:[function(t,e,n){"use strict";t("bn.js"),e.exports=function(t){return t&&"object"==typeof t&&Boolean(t.words)}},{"bn.js":94}],84:[function(t,e,n){"use strict";var r=t("bn.js"),i=t("double-bits");e.exports=function(t){var e=i.exponent(t);return e<52?new r(t):new r(t*Math.pow(2,52-e)).ushln(e-52)}},{"bn.js":94,"double-bits":167}],85:[function(t,e,n){"use strict";var r=t("./num-to-bn"),i=t("./bn-sign");e.exports=function(t,e){var n=i(t),o=i(e);if(0===n)return[r(0),r(1)];if(0===o)return[r(0),r(0)];o<0&&(t=t.neg(),e=e.neg());var a=t.gcd(e);return a.cmpn(1)?[t.div(a),e.div(a)]:[t,e]}},{"./bn-sign":80,"./num-to-bn":84}],86:[function(t,e,n){"use strict";var r=t("bn.js");e.exports=function(t){return new r(t)}},{"bn.js":94}],87:[function(t,e,n){"use strict";var r=t("./lib/rationalize");e.exports=function(t,e){return r(t[0].mul(e[0]),t[1].mul(e[1]))}},{"./lib/rationalize":85}],88:[function(t,e,n){"use strict";var r=t("./lib/bn-sign");e.exports=function(t){return r(t[0])*r(t[1])}},{"./lib/bn-sign":80}],89:[function(t,e,n){"use strict";var r=t("./lib/rationalize");e.exports=function(t,e){return r(t[0].mul(e[1]).sub(t[1].mul(e[0])),t[1].mul(e[1]))}},{"./lib/rationalize":85}],90:[function(t,e,n){"use strict";var r=t("./lib/bn-to-num"),i=t("./lib/ctz");e.exports=function(t){var e=t[0],n=t[1];if(0===e.cmpn(0))return 0;var o=e.abs().divmod(n.abs()),a=o.div,s=r(a),A=o.mod,l=e.negative!==n.negative?-1:1;if(0===A.cmpn(0))return l*s;if(s){var c=i(s)+4,u=r(A.ushln(c).divRound(n));return l*(s+u*Math.pow(2,-c))}var g=n.bitLength()-A.bitLength()+53,u=r(A.ushln(g).divRound(n));return g<1023?l*u*Math.pow(2,-g):(u*=Math.pow(2,-1023),l*u*Math.pow(2,1023-g))}},{"./lib/bn-to-num":81,"./lib/ctz":82}],91:[function(t,e,n){"use strict";function r(t,e,n,r,i,o){var a=["function ",t,"(a,l,h,",r.join(","),"){",o?"":"var i=",n?"l-1":"h+1",";while(l<=h){var m=(l+h)>>>1,x=a",i?".get(m)":"[m]"];return o?e.indexOf("c")<0?a.push(";if(x===y){return m}else if(x<=y){"):a.push(";var p=c(x,y);if(p===0){return m}else if(p<=0){"):a.push(";if(",e,"){i=m;"),n?a.push("l=m+1}else{h=m-1}"):a.push("h=m-1}else{l=m+1}"),a.push("}"),o?a.push("return -1};"):a.push("return i};"),a.join("")}function i(t,e,n,i){return new Function([r("A","x"+t+"y",e,["y"],!1,i),r("B","x"+t+"y",e,["y"],!0,i),r("P","c(x,y)"+t+"0",e,["y","c"],!1,i),r("Q","c(x,y)"+t+"0",e,["y","c"],!0,i),"function dispatchBsearch",n,"(a,y,c,l,h){if(a.shape){if(typeof(c)==='function'){return Q(a,(l===undefined)?0:l|0,(h===undefined)?a.shape[0]-1:h|0,y,c)}else{return B(a,(c===undefined)?0:c|0,(l===undefined)?a.shape[0]-1:l|0,y)}}else{if(typeof(c)==='function'){return P(a,(l===undefined)?0:l|0,(h===undefined)?a.length-1:h|0,y,c)}else{return A(a,(c===undefined)?0:c|0,(l===undefined)?a.length-1:l|0,y)}}}return dispatchBsearch",n].join(""))()}e.exports={ge:i(">=",!1,"GE"),gt:i(">",!1,"GT"),lt:i("<",!0,"LT"),le:i("<=",!0,"LE"),eq:i("-",!0,"EQ",!0)}},{}],92:[function(t,e,n){"use strict";function r(t){var e=32;return(t&=-t)&&e--,65535&t&&(e-=16),16711935&t&&(e-=8),252645135&t&&(e-=4),858993459&t&&(e-=2),1431655765&t&&(e-=1),e}n.INT_BITS=32,n.INT_MAX=2147483647,n.INT_MIN=-1<<31,n.sign=function(t){return(t>0)-(t<0)},n.abs=function(t){var e=t>>31;return(t^e)-e},n.min=function(t,e){return e^(t^e)&-(t<e)},n.max=function(t,e){return t^(t^e)&-(t<e)},n.isPow2=function(t){return!(t&t-1||!t)},n.log2=function(t){var e,n;return e=(t>65535)<<4,e|=n=((t>>>=e)>255)<<3,e|=n=((t>>>=n)>15)<<2,(e|=n=((t>>>=n)>3)<<1)|(t>>>=n)>>1},n.log10=function(t){return t>=1e9?9:t>=1e8?8:t>=1e7?7:t>=1e6?6:t>=1e5?5:t>=1e4?4:t>=1e3?3:t>=100?2:t>=10?1:0},n.popCount=function(t){return 16843009*((t=(858993459&(t-=t>>>1&1431655765))+(t>>>2&858993459))+(t>>>4)&252645135)>>>24},n.countTrailingZeros=r,n.nextPow2=function(t){return t+=0===t,--t,t|=t>>>1,t|=t>>>2,t|=t>>>4,t|=t>>>8,1+(t|=t>>>16)},n.prevPow2=function(t){return t|=t>>>1,t|=t>>>2,t|=t>>>4,t|=t>>>8,(t|=t>>>16)-(t>>>1)},n.parity=function(t){return t^=t>>>16,t^=t>>>8,t^=t>>>4,27030>>>(t&=15)&1};var i=new Array(256);!function(t){for(var e=0;e<256;++e){var n=e,r=e,i=7;for(n>>>=1;n;n>>>=1)r<<=1,r|=1&n,--i;t[e]=r<<i&255}}(i),n.reverse=function(t){return i[255&t]<<24|i[t>>>8&255]<<16|i[t>>>16&255]<<8|i[t>>>24&255]},n.interleave2=function(t,e){return(t=1431655765&((t=858993459&((t=252645135&((t=16711935&((t&=65535)|t<<8))|t<<4))|t<<2))|t<<1))|(e=1431655765&((e=858993459&((e=252645135&((e=16711935&((e&=65535)|e<<8))|e<<4))|e<<2))|e<<1))<<1},n.deinterleave2=function(t,e){return(t=65535&((t=16711935&((t=252645135&((t=858993459&((t=t>>>e&1431655765)|t>>>1))|t>>>2))|t>>>4))|t>>>16))<<16>>16},n.interleave3=function(t,e,n){return t=1227133513&((t=3272356035&((t=251719695&((t=4278190335&((t&=1023)|t<<16))|t<<8))|t<<4))|t<<2),(t|=(e=1227133513&((e=3272356035&((e=251719695&((e=4278190335&((e&=1023)|e<<16))|e<<8))|e<<4))|e<<2))<<1)|(n=1227133513&((n=3272356035&((n=251719695&((n=4278190335&((n&=1023)|n<<16))|n<<8))|n<<4))|n<<2))<<2},n.deinterleave3=function(t,e){return(t=1023&((t=4278190335&((t=251719695&((t=3272356035&((t=t>>>e&1227133513)|t>>>2))|t>>>4))|t>>>8))|t>>>16))<<22>>22},n.nextCombination=function(t){var e=t|t-1;return e+1|(~e&-~e)-1>>>r(t)+1}},{}],93:[function(t,e,n){"use strict";var r=t("clamp");e.exports=function(t,e){e||(e={});var n,a,s,A,l,c,u,g,h,d,f,p=null==e.cutoff?.25:e.cutoff,I=null==e.radius?8:e.radius,C=e.channel||0;if(ArrayBuffer.isView(t)||Array.isArray(t)){if(!e.width||!e.height)throw Error("For raw data width and height should be provided by options");n=e.width,a=e.height,A=t,c=e.stride?e.stride:Math.floor(t.length/n/a)}else window.HTMLCanvasElement&&t instanceof window.HTMLCanvasElement?(u=(g=t).getContext("2d"),n=g.width,a=g.height,h=u.getImageData(0,0,n,a),A=h.data,c=4):window.CanvasRenderingContext2D&&t instanceof window.CanvasRenderingContext2D?(g=t.canvas,u=t,n=g.width,a=g.height,h=u.getImageData(0,0,n,a),A=h.data,c=4):window.ImageData&&t instanceof window.ImageData&&(h=t,n=t.width,a=t.height,A=h.data,c=4);if(s=Math.max(n,a),window.Uint8ClampedArray&&A instanceof window.Uint8ClampedArray||window.Uint8Array&&A instanceof window.Uint8Array)for(l=A,A=Array(n*a),d=0,f=l.length;d<f;d++)A[d]=l[d*c+C]/255;else if(1!==c)throw Error("Raw data can have only 1 value per pixel");var m=Array(n*a),y=Array(n*a),E=Array(s),v=Array(s),B=Array(s+1),b=Array(s);for(d=0,f=n*a;d<f;d++){var Q=A[d];m[d]=1===Q?0:0===Q?i:Math.pow(Math.max(0,.5-Q),2),y[d]=1===Q?i:0===Q?0:Math.pow(Math.max(0,Q-.5),2)}o(m,n,a,E,v,b,B),o(y,n,a,E,v,b,B);var w=window.Float32Array?new Float32Array(n*a):new Array(n*a);for(d=0,f=n*a;d<f;d++)w[d]=r(1-((m[d]-y[d])/I+p),0,1);return w};var i=1e20;function o(t,e,n,r,i,o,s){for(var A=0;A<e;A++){for(var l=0;l<n;l++)r[l]=t[l*e+A];for(a(r,i,o,s,n),l=0;l<n;l++)t[l*e+A]=i[l]}for(l=0;l<n;l++){for(A=0;A<e;A++)r[A]=t[l*e+A];for(a(r,i,o,s,e),A=0;A<e;A++)t[l*e+A]=Math.sqrt(i[A])}}function a(t,e,n,r,o){n[0]=0,r[0]=-i,r[1]=+i;for(var a=1,s=0;a<o;a++){for(var A=(t[a]+a*a-(t[n[s]]+n[s]*n[s]))/(2*a-2*n[s]);A<=r[s];)s--,A=(t[a]+a*a-(t[n[s]]+n[s]*n[s]))/(2*a-2*n[s]);n[++s]=a,r[s]=A,r[s+1]=+i}for(a=0,s=0;a<o;a++){for(;r[s+1]<a;)s++;e[a]=(a-n[s])*(a-n[s])+t[n[s]]}}},{clamp:115}],94:[function(t,e,n){!function(e,n){"use strict";function r(t,e){if(!t)throw new Error(e||"Assertion failed")}function i(t,e){t.super_=e;var n=function(){};n.prototype=e.prototype,t.prototype=new n,t.prototype.constructor=t}function o(t,e,n){if(o.isBN(t))return t;this.negative=0,this.words=null,this.length=0,this.red=null,null!==t&&("le"!==e&&"be"!==e||(n=e,e=10),this._init(t||0,e||10,n||"be"))}var a;"object"==typeof e?e.exports=o:n.BN=o,o.BN=o,o.wordSize=26;try{a=t("buffer").Buffer}catch(t){}function s(t,e,n){for(var r=0,i=Math.min(t.length,n),o=e;o<i;o++){var a=t.charCodeAt(o)-48;r<<=4,r|=a>=49&&a<=54?a-49+10:a>=17&&a<=22?a-17+10:15&a}return r}function A(t,e,n,r){for(var i=0,o=Math.min(t.length,n),a=e;a<o;a++){var s=t.charCodeAt(a)-48;i*=r,i+=s>=49?s-49+10:s>=17?s-17+10:s}return i}o.isBN=function(t){return t instanceof o||null!==t&&"object"==typeof t&&t.constructor.wordSize===o.wordSize&&Array.isArray(t.words)},o.max=function(t,e){return t.cmp(e)>0?t:e},o.min=function(t,e){return t.cmp(e)<0?t:e},o.prototype._init=function(t,e,n){if("number"==typeof t)return this._initNumber(t,e,n);if("object"==typeof t)return this._initArray(t,e,n);"hex"===e&&(e=16),r(e===(0|e)&&e>=2&&e<=36);var i=0;"-"===(t=t.toString().replace(/\s+/g,""))[0]&&i++,16===e?this._parseHex(t,i):this._parseBase(t,e,i),"-"===t[0]&&(this.negative=1),this.strip(),"le"===n&&this._initArray(this.toArray(),e,n)},o.prototype._initNumber=function(t,e,n){t<0&&(this.negative=1,t=-t),t<67108864?(this.words=[67108863&t],this.length=1):t<4503599627370496?(this.words=[67108863&t,t/67108864&67108863],this.length=2):(r(t<9007199254740992),this.words=[67108863&t,t/67108864&67108863,1],this.length=3),"le"===n&&this._initArray(this.toArray(),e,n)},o.prototype._initArray=function(t,e,n){if(r("number"==typeof t.length),t.length<=0)return this.words=[0],this.length=1,this;this.length=Math.ceil(t.length/3),this.words=new Array(this.length);for(var i=0;i<this.length;i++)this.words[i]=0;var o,a,s=0;if("be"===n)for(i=t.length-1,o=0;i>=0;i-=3)a=t[i]|t[i-1]<<8|t[i-2]<<16,this.words[o]|=a<<s&67108863,this.words[o+1]=a>>>26-s&67108863,(s+=24)>=26&&(s-=26,o++);else if("le"===n)for(i=0,o=0;i<t.length;i+=3)a=t[i]|t[i+1]<<8|t[i+2]<<16,this.words[o]|=a<<s&67108863,this.words[o+1]=a>>>26-s&67108863,(s+=24)>=26&&(s-=26,o++);return this.strip()},o.prototype._parseHex=function(t,e){this.length=Math.ceil((t.length-e)/6),this.words=new Array(this.length);for(var n=0;n<this.length;n++)this.words[n]=0;var r,i,o=0;for(n=t.length-6,r=0;n>=e;n-=6)i=s(t,n,n+6),this.words[r]|=i<<o&67108863,this.words[r+1]|=i>>>26-o&4194303,(o+=24)>=26&&(o-=26,r++);n+6!==e&&(i=s(t,e,n+6),this.words[r]|=i<<o&67108863,this.words[r+1]|=i>>>26-o&4194303),this.strip()},o.prototype._parseBase=function(t,e,n){this.words=[0],this.length=1;for(var r=0,i=1;i<=67108863;i*=e)r++;r--,i=i/e|0;for(var o=t.length-n,a=o%r,s=Math.min(o,o-a)+n,l=0,c=n;c<s;c+=r)l=A(t,c,c+r,e),this.imuln(i),this.words[0]+l<67108864?this.words[0]+=l:this._iaddn(l);if(0!==a){var u=1;for(l=A(t,c,t.length,e),c=0;c<a;c++)u*=e;this.imuln(u),this.words[0]+l<67108864?this.words[0]+=l:this._iaddn(l)}},o.prototype.copy=function(t){t.words=new Array(this.length);for(var e=0;e<this.length;e++)t.words[e]=this.words[e];t.length=this.length,t.negative=this.negative,t.red=this.red},o.prototype.clone=function(){var t=new o(null);return this.copy(t),t},o.prototype._expand=function(t){for(;this.length<t;)this.words[this.length++]=0;return this},o.prototype.strip=function(){for(;this.length>1&&0===this.words[this.length-1];)this.length--;return this._normSign()},o.prototype._normSign=function(){return 1===this.length&&0===this.words[0]&&(this.negative=0),this},o.prototype.inspect=function(){return(this.red?"<BN-R: ":"<BN: ")+this.toString(16)+">"};var l=["","0","00","000","0000","00000","000000","0000000","00000000","000000000","0000000000","00000000000","000000000000","0000000000000","00000000000000","000000000000000","0000000000000000","00000000000000000","000000000000000000","0000000000000000000","00000000000000000000","000000000000000000000","0000000000000000000000","00000000000000000000000","000000000000000000000000","0000000000000000000000000"],c=[0,0,25,16,12,11,10,9,8,8,7,7,7,7,6,6,6,6,6,6,6,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5],u=[0,0,33554432,43046721,16777216,48828125,60466176,40353607,16777216,43046721,1e7,19487171,35831808,62748517,7529536,11390625,16777216,24137569,34012224,47045881,64e6,4084101,5153632,6436343,7962624,9765625,11881376,14348907,17210368,20511149,243e5,28629151,33554432,39135393,45435424,52521875,60466176];function g(t,e,n){n.negative=e.negative^t.negative;var r=t.length+e.length|0;n.length=r,r=r-1|0;var i=0|t.words[0],o=0|e.words[0],a=i*o,s=67108863&a,A=a/67108864|0;n.words[0]=s;for(var l=1;l<r;l++){for(var c=A>>>26,u=67108863&A,g=Math.min(l,e.length-1),h=Math.max(0,l-t.length+1);h<=g;h++){var d=l-h|0;c+=(a=(i=0|t.words[d])*(o=0|e.words[h])+u)/67108864|0,u=67108863&a}n.words[l]=0|u,A=0|c}return 0!==A?n.words[l]=0|A:n.length--,n.strip()}o.prototype.toString=function(t,e){var n;if(e=0|e||1,16===(t=t||10)||"hex"===t){n="";for(var i=0,o=0,a=0;a<this.length;a++){var s=this.words[a],A=(16777215&(s<<i|o)).toString(16);n=0!=(o=s>>>24-i&16777215)||a!==this.length-1?l[6-A.length]+A+n:A+n,(i+=2)>=26&&(i-=26,a--)}for(0!==o&&(n=o.toString(16)+n);n.length%e!=0;)n="0"+n;return 0!==this.negative&&(n="-"+n),n}if(t===(0|t)&&t>=2&&t<=36){var g=c[t],h=u[t];n="";var d=this.clone();for(d.negative=0;!d.isZero();){var f=d.modn(h).toString(t);n=(d=d.idivn(h)).isZero()?f+n:l[g-f.length]+f+n}for(this.isZero()&&(n="0"+n);n.length%e!=0;)n="0"+n;return 0!==this.negative&&(n="-"+n),n}r(!1,"Base should be between 2 and 36")},o.prototype.toNumber=function(){var t=this.words[0];return 2===this.length?t+=67108864*this.words[1]:3===this.length&&1===this.words[2]?t+=4503599627370496+67108864*this.words[1]:this.length>2&&r(!1,"Number can only safely store up to 53 bits"),0!==this.negative?-t:t},o.prototype.toJSON=function(){return this.toString(16)},o.prototype.toBuffer=function(t,e){return r(void 0!==a),this.toArrayLike(a,t,e)},o.prototype.toArray=function(t,e){return this.toArrayLike(Array,t,e)},o.prototype.toArrayLike=function(t,e,n){var i=this.byteLength(),o=n||Math.max(1,i);r(i<=o,"byte array longer than desired length"),r(o>0,"Requested array length <= 0"),this.strip();var a,s,A="le"===e,l=new t(o),c=this.clone();if(A){for(s=0;!c.isZero();s++)a=c.andln(255),c.iushrn(8),l[s]=a;for(;s<o;s++)l[s]=0}else{for(s=0;s<o-i;s++)l[s]=0;for(s=0;!c.isZero();s++)a=c.andln(255),c.iushrn(8),l[o-s-1]=a}return l},Math.clz32?o.prototype._countBits=function(t){return 32-Math.clz32(t)}:o.prototype._countBits=function(t){var e=t,n=0;return e>=4096&&(n+=13,e>>>=13),e>=64&&(n+=7,e>>>=7),e>=8&&(n+=4,e>>>=4),e>=2&&(n+=2,e>>>=2),n+e},o.prototype._zeroBits=function(t){if(0===t)return 26;var e=t,n=0;return 0==(8191&e)&&(n+=13,e>>>=13),0==(127&e)&&(n+=7,e>>>=7),0==(15&e)&&(n+=4,e>>>=4),0==(3&e)&&(n+=2,e>>>=2),0==(1&e)&&n++,n},o.prototype.bitLength=function(){var t=this.words[this.length-1],e=this._countBits(t);return 26*(this.length-1)+e},o.prototype.zeroBits=function(){if(this.isZero())return 0;for(var t=0,e=0;e<this.length;e++){var n=this._zeroBits(this.words[e]);if(t+=n,26!==n)break}return t},o.prototype.byteLength=function(){return Math.ceil(this.bitLength()/8)},o.prototype.toTwos=function(t){return 0!==this.negative?this.abs().inotn(t).iaddn(1):this.clone()},o.prototype.fromTwos=function(t){return this.testn(t-1)?this.notn(t).iaddn(1).ineg():this.clone()},o.prototype.isNeg=function(){return 0!==this.negative},o.prototype.neg=function(){return this.clone().ineg()},o.prototype.ineg=function(){return this.isZero()||(this.negative^=1),this},o.prototype.iuor=function(t){for(;this.length<t.length;)this.words[this.length++]=0;for(var e=0;e<t.length;e++)this.words[e]=this.words[e]|t.words[e];return this.strip()},o.prototype.ior=function(t){return r(0==(this.negative|t.negative)),this.iuor(t)},o.prototype.or=function(t){return this.length>t.length?this.clone().ior(t):t.clone().ior(this)},o.prototype.uor=function(t){return this.length>t.length?this.clone().iuor(t):t.clone().iuor(this)},o.prototype.iuand=function(t){var e;e=this.length>t.length?t:this;for(var n=0;n<e.length;n++)this.words[n]=this.words[n]&t.words[n];return this.length=e.length,this.strip()},o.prototype.iand=function(t){return r(0==(this.negative|t.negative)),this.iuand(t)},o.prototype.and=function(t){return this.length>t.length?this.clone().iand(t):t.clone().iand(this)},o.prototype.uand=function(t){return this.length>t.length?this.clone().iuand(t):t.clone().iuand(this)},o.prototype.iuxor=function(t){var e,n;this.length>t.length?(e=this,n=t):(e=t,n=this);for(var r=0;r<n.length;r++)this.words[r]=e.words[r]^n.words[r];if(this!==e)for(;r<e.length;r++)this.words[r]=e.words[r];return this.length=e.length,this.strip()},o.prototype.ixor=function(t){return r(0==(this.negative|t.negative)),this.iuxor(t)},o.prototype.xor=function(t){return this.length>t.length?this.clone().ixor(t):t.clone().ixor(this)},o.prototype.uxor=function(t){return this.length>t.length?this.clone().iuxor(t):t.clone().iuxor(this)},o.prototype.inotn=function(t){r("number"==typeof t&&t>=0);var e=0|Math.ceil(t/26),n=t%26;this._expand(e),n>0&&e--;for(var i=0;i<e;i++)this.words[i]=67108863&~this.words[i];return n>0&&(this.words[i]=~this.words[i]&67108863>>26-n),this.strip()},o.prototype.notn=function(t){return this.clone().inotn(t)},o.prototype.setn=function(t,e){r("number"==typeof t&&t>=0);var n=t/26|0,i=t%26;return this._expand(n+1),this.words[n]=e?this.words[n]|1<<i:this.words[n]&~(1<<i),this.strip()},o.prototype.iadd=function(t){var e,n,r;if(0!==this.negative&&0===t.negative)return this.negative=0,e=this.isub(t),this.negative^=1,this._normSign();if(0===this.negative&&0!==t.negative)return t.negative=0,e=this.isub(t),t.negative=1,e._normSign();this.length>t.length?(n=this,r=t):(n=t,r=this);for(var i=0,o=0;o<r.length;o++)e=(0|n.words[o])+(0|r.words[o])+i,this.words[o]=67108863&e,i=e>>>26;for(;0!==i&&o<n.length;o++)e=(0|n.words[o])+i,this.words[o]=67108863&e,i=e>>>26;if(this.length=n.length,0!==i)this.words[this.length]=i,this.length++;else if(n!==this)for(;o<n.length;o++)this.words[o]=n.words[o];return this},o.prototype.add=function(t){var e;return 0!==t.negative&&0===this.negative?(t.negative=0,e=this.sub(t),t.negative^=1,e):0===t.negative&&0!==this.negative?(this.negative=0,e=t.sub(this),this.negative=1,e):this.length>t.length?this.clone().iadd(t):t.clone().iadd(this)},o.prototype.isub=function(t){if(0!==t.negative){t.negative=0;var e=this.iadd(t);return t.negative=1,e._normSign()}if(0!==this.negative)return this.negative=0,this.iadd(t),this.negative=1,this._normSign();var n,r,i=this.cmp(t);if(0===i)return this.negative=0,this.length=1,this.words[0]=0,this;i>0?(n=this,r=t):(n=t,r=this);for(var o=0,a=0;a<r.length;a++)o=(e=(0|n.words[a])-(0|r.words[a])+o)>>26,this.words[a]=67108863&e;for(;0!==o&&a<n.length;a++)o=(e=(0|n.words[a])+o)>>26,this.words[a]=67108863&e;if(0===o&&a<n.length&&n!==this)for(;a<n.length;a++)this.words[a]=n.words[a];return this.length=Math.max(this.length,a),n!==this&&(this.negative=1),this.strip()},o.prototype.sub=function(t){return this.clone().isub(t)};var h=function(t,e,n){var r,i,o,a=t.words,s=e.words,A=n.words,l=0,c=0|a[0],u=8191&c,g=c>>>13,h=0|a[1],d=8191&h,f=h>>>13,p=0|a[2],I=8191&p,C=p>>>13,m=0|a[3],y=8191&m,E=m>>>13,v=0|a[4],B=8191&v,b=v>>>13,Q=0|a[5],w=8191&Q,x=Q>>>13,_=0|a[6],D=8191&_,M=_>>>13,k=0|a[7],S=8191&k,L=k>>>13,R=0|a[8],N=8191&R,F=R>>>13,T=0|a[9],U=8191&T,O=T>>>13,G=0|s[0],H=8191&G,j=G>>>13,Y=0|s[1],P=8191&Y,K=Y>>>13,z=0|s[2],J=8191&z,q=z>>>13,V=0|s[3],W=8191&V,X=V>>>13,Z=0|s[4],$=8191&Z,tt=Z>>>13,et=0|s[5],nt=8191&et,rt=et>>>13,it=0|s[6],ot=8191&it,at=it>>>13,st=0|s[7],At=8191&st,lt=st>>>13,ct=0|s[8],ut=8191&ct,gt=ct>>>13,ht=0|s[9],dt=8191&ht,ft=ht>>>13;n.negative=t.negative^e.negative,n.length=19;var pt=(l+(r=Math.imul(u,H))|0)+((8191&(i=(i=Math.imul(u,j))+Math.imul(g,H)|0))<<13)|0;l=((o=Math.imul(g,j))+(i>>>13)|0)+(pt>>>26)|0,pt&=67108863,r=Math.imul(d,H),i=(i=Math.imul(d,j))+Math.imul(f,H)|0,o=Math.imul(f,j);var It=(l+(r=r+Math.imul(u,P)|0)|0)+((8191&(i=(i=i+Math.imul(u,K)|0)+Math.imul(g,P)|0))<<13)|0;l=((o=o+Math.imul(g,K)|0)+(i>>>13)|0)+(It>>>26)|0,It&=67108863,r=Math.imul(I,H),i=(i=Math.imul(I,j))+Math.imul(C,H)|0,o=Math.imul(C,j),r=r+Math.imul(d,P)|0,i=(i=i+Math.imul(d,K)|0)+Math.imul(f,P)|0,o=o+Math.imul(f,K)|0;var Ct=(l+(r=r+Math.imul(u,J)|0)|0)+((8191&(i=(i=i+Math.imul(u,q)|0)+Math.imul(g,J)|0))<<13)|0;l=((o=o+Math.imul(g,q)|0)+(i>>>13)|0)+(Ct>>>26)|0,Ct&=67108863,r=Math.imul(y,H),i=(i=Math.imul(y,j))+Math.imul(E,H)|0,o=Math.imul(E,j),r=r+Math.imul(I,P)|0,i=(i=i+Math.imul(I,K)|0)+Math.imul(C,P)|0,o=o+Math.imul(C,K)|0,r=r+Math.imul(d,J)|0,i=(i=i+Math.imul(d,q)|0)+Math.imul(f,J)|0,o=o+Math.imul(f,q)|0;var mt=(l+(r=r+Math.imul(u,W)|0)|0)+((8191&(i=(i=i+Math.imul(u,X)|0)+Math.imul(g,W)|0))<<13)|0;l=((o=o+Math.imul(g,X)|0)+(i>>>13)|0)+(mt>>>26)|0,mt&=67108863,r=Math.imul(B,H),i=(i=Math.imul(B,j))+Math.imul(b,H)|0,o=Math.imul(b,j),r=r+Math.imul(y,P)|0,i=(i=i+Math.imul(y,K)|0)+Math.imul(E,P)|0,o=o+Math.imul(E,K)|0,r=r+Math.imul(I,J)|0,i=(i=i+Math.imul(I,q)|0)+Math.imul(C,J)|0,o=o+Math.imul(C,q)|0,r=r+Math.imul(d,W)|0,i=(i=i+Math.imul(d,X)|0)+Math.imul(f,W)|0,o=o+Math.imul(f,X)|0;var yt=(l+(r=r+Math.imul(u,$)|0)|0)+((8191&(i=(i=i+Math.imul(u,tt)|0)+Math.imul(g,$)|0))<<13)|0;l=((o=o+Math.imul(g,tt)|0)+(i>>>13)|0)+(yt>>>26)|0,yt&=67108863,r=Math.imul(w,H),i=(i=Math.imul(w,j))+Math.imul(x,H)|0,o=Math.imul(x,j),r=r+Math.imul(B,P)|0,i=(i=i+Math.imul(B,K)|0)+Math.imul(b,P)|0,o=o+Math.imul(b,K)|0,r=r+Math.imul(y,J)|0,i=(i=i+Math.imul(y,q)|0)+Math.imul(E,J)|0,o=o+Math.imul(E,q)|0,r=r+Math.imul(I,W)|0,i=(i=i+Math.imul(I,X)|0)+Math.imul(C,W)|0,o=o+Math.imul(C,X)|0,r=r+Math.imul(d,$)|0,i=(i=i+Math.imul(d,tt)|0)+Math.imul(f,$)|0,o=o+Math.imul(f,tt)|0;var Et=(l+(r=r+Math.imul(u,nt)|0)|0)+((8191&(i=(i=i+Math.imul(u,rt)|0)+Math.imul(g,nt)|0))<<13)|0;l=((o=o+Math.imul(g,rt)|0)+(i>>>13)|0)+(Et>>>26)|0,Et&=67108863,r=Math.imul(D,H),i=(i=Math.imul(D,j))+Math.imul(M,H)|0,o=Math.imul(M,j),r=r+Math.imul(w,P)|0,i=(i=i+Math.imul(w,K)|0)+Math.imul(x,P)|0,o=o+Math.imul(x,K)|0,r=r+Math.imul(B,J)|0,i=(i=i+Math.imul(B,q)|0)+Math.imul(b,J)|0,o=o+Math.imul(b,q)|0,r=r+Math.imul(y,W)|0,i=(i=i+Math.imul(y,X)|0)+Math.imul(E,W)|0,o=o+Math.imul(E,X)|0,r=r+Math.imul(I,$)|0,i=(i=i+Math.imul(I,tt)|0)+Math.imul(C,$)|0,o=o+Math.imul(C,tt)|0,r=r+Math.imul(d,nt)|0,i=(i=i+Math.imul(d,rt)|0)+Math.imul(f,nt)|0,o=o+Math.imul(f,rt)|0;var vt=(l+(r=r+Math.imul(u,ot)|0)|0)+((8191&(i=(i=i+Math.imul(u,at)|0)+Math.imul(g,ot)|0))<<13)|0;l=((o=o+Math.imul(g,at)|0)+(i>>>13)|0)+(vt>>>26)|0,vt&=67108863,r=Math.imul(S,H),i=(i=Math.imul(S,j))+Math.imul(L,H)|0,o=Math.imul(L,j),r=r+Math.imul(D,P)|0,i=(i=i+Math.imul(D,K)|0)+Math.imul(M,P)|0,o=o+Math.imul(M,K)|0,r=r+Math.imul(w,J)|0,i=(i=i+Math.imul(w,q)|0)+Math.imul(x,J)|0,o=o+Math.imul(x,q)|0,r=r+Math.imul(B,W)|0,i=(i=i+Math.imul(B,X)|0)+Math.imul(b,W)|0,o=o+Math.imul(b,X)|0,r=r+Math.imul(y,$)|0,i=(i=i+Math.imul(y,tt)|0)+Math.imul(E,$)|0,o=o+Math.imul(E,tt)|0,r=r+Math.imul(I,nt)|0,i=(i=i+Math.imul(I,rt)|0)+Math.imul(C,nt)|0,o=o+Math.imul(C,rt)|0,r=r+Math.imul(d,ot)|0,i=(i=i+Math.imul(d,at)|0)+Math.imul(f,ot)|0,o=o+Math.imul(f,at)|0;var Bt=(l+(r=r+Math.imul(u,At)|0)|0)+((8191&(i=(i=i+Math.imul(u,lt)|0)+Math.imul(g,At)|0))<<13)|0;l=((o=o+Math.imul(g,lt)|0)+(i>>>13)|0)+(Bt>>>26)|0,Bt&=67108863,r=Math.imul(N,H),i=(i=Math.imul(N,j))+Math.imul(F,H)|0,o=Math.imul(F,j),r=r+Math.imul(S,P)|0,i=(i=i+Math.imul(S,K)|0)+Math.imul(L,P)|0,o=o+Math.imul(L,K)|0,r=r+Math.imul(D,J)|0,i=(i=i+Math.imul(D,q)|0)+Math.imul(M,J)|0,o=o+Math.imul(M,q)|0,r=r+Math.imul(w,W)|0,i=(i=i+Math.imul(w,X)|0)+Math.imul(x,W)|0,o=o+Math.imul(x,X)|0,r=r+Math.imul(B,$)|0,i=(i=i+Math.imul(B,tt)|0)+Math.imul(b,$)|0,o=o+Math.imul(b,tt)|0,r=r+Math.imul(y,nt)|0,i=(i=i+Math.imul(y,rt)|0)+Math.imul(E,nt)|0,o=o+Math.imul(E,rt)|0,r=r+Math.imul(I,ot)|0,i=(i=i+Math.imul(I,at)|0)+Math.imul(C,ot)|0,o=o+Math.imul(C,at)|0,r=r+Math.imul(d,At)|0,i=(i=i+Math.imul(d,lt)|0)+Math.imul(f,At)|0,o=o+Math.imul(f,lt)|0;var bt=(l+(r=r+Math.imul(u,ut)|0)|0)+((8191&(i=(i=i+Math.imul(u,gt)|0)+Math.imul(g,ut)|0))<<13)|0;l=((o=o+Math.imul(g,gt)|0)+(i>>>13)|0)+(bt>>>26)|0,bt&=67108863,r=Math.imul(U,H),i=(i=Math.imul(U,j))+Math.imul(O,H)|0,o=Math.imul(O,j),r=r+Math.imul(N,P)|0,i=(i=i+Math.imul(N,K)|0)+Math.imul(F,P)|0,o=o+Math.imul(F,K)|0,r=r+Math.imul(S,J)|0,i=(i=i+Math.imul(S,q)|0)+Math.imul(L,J)|0,o=o+Math.imul(L,q)|0,r=r+Math.imul(D,W)|0,i=(i=i+Math.imul(D,X)|0)+Math.imul(M,W)|0,o=o+Math.imul(M,X)|0,r=r+Math.imul(w,$)|0,i=(i=i+Math.imul(w,tt)|0)+Math.imul(x,$)|0,o=o+Math.imul(x,tt)|0,r=r+Math.imul(B,nt)|0,i=(i=i+Math.imul(B,rt)|0)+Math.imul(b,nt)|0,o=o+Math.imul(b,rt)|0,r=r+Math.imul(y,ot)|0,i=(i=i+Math.imul(y,at)|0)+Math.imul(E,ot)|0,o=o+Math.imul(E,at)|0,r=r+Math.imul(I,At)|0,i=(i=i+Math.imul(I,lt)|0)+Math.imul(C,At)|0,o=o+Math.imul(C,lt)|0,r=r+Math.imul(d,ut)|0,i=(i=i+Math.imul(d,gt)|0)+Math.imul(f,ut)|0,o=o+Math.imul(f,gt)|0;var Qt=(l+(r=r+Math.imul(u,dt)|0)|0)+((8191&(i=(i=i+Math.imul(u,ft)|0)+Math.imul(g,dt)|0))<<13)|0;l=((o=o+Math.imul(g,ft)|0)+(i>>>13)|0)+(Qt>>>26)|0,Qt&=67108863,r=Math.imul(U,P),i=(i=Math.imul(U,K))+Math.imul(O,P)|0,o=Math.imul(O,K),r=r+Math.imul(N,J)|0,i=(i=i+Math.imul(N,q)|0)+Math.imul(F,J)|0,o=o+Math.imul(F,q)|0,r=r+Math.imul(S,W)|0,i=(i=i+Math.imul(S,X)|0)+Math.imul(L,W)|0,o=o+Math.imul(L,X)|0,r=r+Math.imul(D,$)|0,i=(i=i+Math.imul(D,tt)|0)+Math.imul(M,$)|0,o=o+Math.imul(M,tt)|0,r=r+Math.imul(w,nt)|0,i=(i=i+Math.imul(w,rt)|0)+Math.imul(x,nt)|0,o=o+Math.imul(x,rt)|0,r=r+Math.imul(B,ot)|0,i=(i=i+Math.imul(B,at)|0)+Math.imul(b,ot)|0,o=o+Math.imul(b,at)|0,r=r+Math.imul(y,At)|0,i=(i=i+Math.imul(y,lt)|0)+Math.imul(E,At)|0,o=o+Math.imul(E,lt)|0,r=r+Math.imul(I,ut)|0,i=(i=i+Math.imul(I,gt)|0)+Math.imul(C,ut)|0,o=o+Math.imul(C,gt)|0;var wt=(l+(r=r+Math.imul(d,dt)|0)|0)+((8191&(i=(i=i+Math.imul(d,ft)|0)+Math.imul(f,dt)|0))<<13)|0;l=((o=o+Math.imul(f,ft)|0)+(i>>>13)|0)+(wt>>>26)|0,wt&=67108863,r=Math.imul(U,J),i=(i=Math.imul(U,q))+Math.imul(O,J)|0,o=Math.imul(O,q),r=r+Math.imul(N,W)|0,i=(i=i+Math.imul(N,X)|0)+Math.imul(F,W)|0,o=o+Math.imul(F,X)|0,r=r+Math.imul(S,$)|0,i=(i=i+Math.imul(S,tt)|0)+Math.imul(L,$)|0,o=o+Math.imul(L,tt)|0,r=r+Math.imul(D,nt)|0,i=(i=i+Math.imul(D,rt)|0)+Math.imul(M,nt)|0,o=o+Math.imul(M,rt)|0,r=r+Math.imul(w,ot)|0,i=(i=i+Math.imul(w,at)|0)+Math.imul(x,ot)|0,o=o+Math.imul(x,at)|0,r=r+Math.imul(B,At)|0,i=(i=i+Math.imul(B,lt)|0)+Math.imul(b,At)|0,o=o+Math.imul(b,lt)|0,r=r+Math.imul(y,ut)|0,i=(i=i+Math.imul(y,gt)|0)+Math.imul(E,ut)|0,o=o+Math.imul(E,gt)|0;var xt=(l+(r=r+Math.imul(I,dt)|0)|0)+((8191&(i=(i=i+Math.imul(I,ft)|0)+Math.imul(C,dt)|0))<<13)|0;l=((o=o+Math.imul(C,ft)|0)+(i>>>13)|0)+(xt>>>26)|0,xt&=67108863,r=Math.imul(U,W),i=(i=Math.imul(U,X))+Math.imul(O,W)|0,o=Math.imul(O,X),r=r+Math.imul(N,$)|0,i=(i=i+Math.imul(N,tt)|0)+Math.imul(F,$)|0,o=o+Math.imul(F,tt)|0,r=r+Math.imul(S,nt)|0,i=(i=i+Math.imul(S,rt)|0)+Math.imul(L,nt)|0,o=o+Math.imul(L,rt)|0,r=r+Math.imul(D,ot)|0,i=(i=i+Math.imul(D,at)|0)+Math.imul(M,ot)|0,o=o+Math.imul(M,at)|0,r=r+Math.imul(w,At)|0,i=(i=i+Math.imul(w,lt)|0)+Math.imul(x,At)|0,o=o+Math.imul(x,lt)|0,r=r+Math.imul(B,ut)|0,i=(i=i+Math.imul(B,gt)|0)+Math.imul(b,ut)|0,o=o+Math.imul(b,gt)|0;var _t=(l+(r=r+Math.imul(y,dt)|0)|0)+((8191&(i=(i=i+Math.imul(y,ft)|0)+Math.imul(E,dt)|0))<<13)|0;l=((o=o+Math.imul(E,ft)|0)+(i>>>13)|0)+(_t>>>26)|0,_t&=67108863,r=Math.imul(U,$),i=(i=Math.imul(U,tt))+Math.imul(O,$)|0,o=Math.imul(O,tt),r=r+Math.imul(N,nt)|0,i=(i=i+Math.imul(N,rt)|0)+Math.imul(F,nt)|0,o=o+Math.imul(F,rt)|0,r=r+Math.imul(S,ot)|0,i=(i=i+Math.imul(S,at)|0)+Math.imul(L,ot)|0,o=o+Math.imul(L,at)|0,r=r+Math.imul(D,At)|0,i=(i=i+Math.imul(D,lt)|0)+Math.imul(M,At)|0,o=o+Math.imul(M,lt)|0,r=r+Math.imul(w,ut)|0,i=(i=i+Math.imul(w,gt)|0)+Math.imul(x,ut)|0,o=o+Math.imul(x,gt)|0;var Dt=(l+(r=r+Math.imul(B,dt)|0)|0)+((8191&(i=(i=i+Math.imul(B,ft)|0)+Math.imul(b,dt)|0))<<13)|0;l=((o=o+Math.imul(b,ft)|0)+(i>>>13)|0)+(Dt>>>26)|0,Dt&=67108863,r=Math.imul(U,nt),i=(i=Math.imul(U,rt))+Math.imul(O,nt)|0,o=Math.imul(O,rt),r=r+Math.imul(N,ot)|0,i=(i=i+Math.imul(N,at)|0)+Math.imul(F,ot)|0,o=o+Math.imul(F,at)|0,r=r+Math.imul(S,At)|0,i=(i=i+Math.imul(S,lt)|0)+Math.imul(L,At)|0,o=o+Math.imul(L,lt)|0,r=r+Math.imul(D,ut)|0,i=(i=i+Math.imul(D,gt)|0)+Math.imul(M,ut)|0,o=o+Math.imul(M,gt)|0;var Mt=(l+(r=r+Math.imul(w,dt)|0)|0)+((8191&(i=(i=i+Math.imul(w,ft)|0)+Math.imul(x,dt)|0))<<13)|0;l=((o=o+Math.imul(x,ft)|0)+(i>>>13)|0)+(Mt>>>26)|0,Mt&=67108863,r=Math.imul(U,ot),i=(i=Math.imul(U,at))+Math.imul(O,ot)|0,o=Math.imul(O,at),r=r+Math.imul(N,At)|0,i=(i=i+Math.imul(N,lt)|0)+Math.imul(F,At)|0,o=o+Math.imul(F,lt)|0,r=r+Math.imul(S,ut)|0,i=(i=i+Math.imul(S,gt)|0)+Math.imul(L,ut)|0,o=o+Math.imul(L,gt)|0;var kt=(l+(r=r+Math.imul(D,dt)|0)|0)+((8191&(i=(i=i+Math.imul(D,ft)|0)+Math.imul(M,dt)|0))<<13)|0;l=((o=o+Math.imul(M,ft)|0)+(i>>>13)|0)+(kt>>>26)|0,kt&=67108863,r=Math.imul(U,At),i=(i=Math.imul(U,lt))+Math.imul(O,At)|0,o=Math.imul(O,lt),r=r+Math.imul(N,ut)|0,i=(i=i+Math.imul(N,gt)|0)+Math.imul(F,ut)|0,o=o+Math.imul(F,gt)|0;var St=(l+(r=r+Math.imul(S,dt)|0)|0)+((8191&(i=(i=i+Math.imul(S,ft)|0)+Math.imul(L,dt)|0))<<13)|0;l=((o=o+Math.imul(L,ft)|0)+(i>>>13)|0)+(St>>>26)|0,St&=67108863,r=Math.imul(U,ut),i=(i=Math.imul(U,gt))+Math.imul(O,ut)|0,o=Math.imul(O,gt);var Lt=(l+(r=r+Math.imul(N,dt)|0)|0)+((8191&(i=(i=i+Math.imul(N,ft)|0)+Math.imul(F,dt)|0))<<13)|0;l=((o=o+Math.imul(F,ft)|0)+(i>>>13)|0)+(Lt>>>26)|0,Lt&=67108863;var Rt=(l+(r=Math.imul(U,dt))|0)+((8191&(i=(i=Math.imul(U,ft))+Math.imul(O,dt)|0))<<13)|0;return l=((o=Math.imul(O,ft))+(i>>>13)|0)+(Rt>>>26)|0,Rt&=67108863,A[0]=pt,A[1]=It,A[2]=Ct,A[3]=mt,A[4]=yt,A[5]=Et,A[6]=vt,A[7]=Bt,A[8]=bt,A[9]=Qt,A[10]=wt,A[11]=xt,A[12]=_t,A[13]=Dt,A[14]=Mt,A[15]=kt,A[16]=St,A[17]=Lt,A[18]=Rt,0!==l&&(A[19]=l,n.length++),n};function d(t,e,n){return(new f).mulp(t,e,n)}function f(t,e){this.x=t,this.y=e}Math.imul||(h=g),o.prototype.mulTo=function(t,e){var n=this.length+t.length;return 10===this.length&&10===t.length?h(this,t,e):n<63?g(this,t,e):n<1024?function(t,e,n){n.negative=e.negative^t.negative,n.length=t.length+e.length;for(var r=0,i=0,o=0;o<n.length-1;o++){var a=i;i=0;for(var s=67108863&r,A=Math.min(o,e.length-1),l=Math.max(0,o-t.length+1);l<=A;l++){var c=o-l,u=(0|t.words[c])*(0|e.words[l]),g=67108863&u;s=67108863&(g=g+s|0),i+=(a=(a=a+(u/67108864|0)|0)+(g>>>26)|0)>>>26,a&=67108863}n.words[o]=s,r=a,a=i}return 0!==r?n.words[o]=r:n.length--,n.strip()}(this,t,e):d(this,t,e)},f.prototype.makeRBT=function(t){for(var e=new Array(t),n=o.prototype._countBits(t)-1,r=0;r<t;r++)e[r]=this.revBin(r,n,t);return e},f.prototype.revBin=function(t,e,n){if(0===t||t===n-1)return t;for(var r=0,i=0;i<e;i++)r|=(1&t)<<e-i-1,t>>=1;return r},f.prototype.permute=function(t,e,n,r,i,o){for(var a=0;a<o;a++)r[a]=e[t[a]],i[a]=n[t[a]]},f.prototype.transform=function(t,e,n,r,i,o){this.permute(o,t,e,n,r,i);for(var a=1;a<i;a<<=1)for(var s=a<<1,A=Math.cos(2*Math.PI/s),l=Math.sin(2*Math.PI/s),c=0;c<i;c+=s)for(var u=A,g=l,h=0;h<a;h++){var d=n[c+h],f=r[c+h],p=n[c+h+a],I=r[c+h+a],C=u*p-g*I;I=u*I+g*p,p=C,n[c+h]=d+p,r[c+h]=f+I,n[c+h+a]=d-p,r[c+h+a]=f-I,h!==s&&(C=A*u-l*g,g=A*g+l*u,u=C)}},f.prototype.guessLen13b=function(t,e){var n=1|Math.max(e,t),r=1&n,i=0;for(n=n/2|0;n;n>>>=1)i++;return 1<<i+1+r},f.prototype.conjugate=function(t,e,n){if(!(n<=1))for(var r=0;r<n/2;r++){var i=t[r];t[r]=t[n-r-1],t[n-r-1]=i,i=e[r],e[r]=-e[n-r-1],e[n-r-1]=-i}},f.prototype.normalize13b=function(t,e){for(var n=0,r=0;r<e/2;r++){var i=8192*Math.round(t[2*r+1]/e)+Math.round(t[2*r]/e)+n;t[r]=67108863&i,n=i<67108864?0:i/67108864|0}return t},f.prototype.convert13b=function(t,e,n,i){for(var o=0,a=0;a<e;a++)o+=0|t[a],n[2*a]=8191&o,o>>>=13,n[2*a+1]=8191&o,o>>>=13;for(a=2*e;a<i;++a)n[a]=0;r(0===o),r(0==(-8192&o))},f.prototype.stub=function(t){for(var e=new Array(t),n=0;n<t;n++)e[n]=0;return e},f.prototype.mulp=function(t,e,n){var r=2*this.guessLen13b(t.length,e.length),i=this.makeRBT(r),o=this.stub(r),a=new Array(r),s=new Array(r),A=new Array(r),l=new Array(r),c=new Array(r),u=new Array(r),g=n.words;g.length=r,this.convert13b(t.words,t.length,a,r),this.convert13b(e.words,e.length,l,r),this.transform(a,o,s,A,r,i),this.transform(l,o,c,u,r,i);for(var h=0;h<r;h++){var d=s[h]*c[h]-A[h]*u[h];A[h]=s[h]*u[h]+A[h]*c[h],s[h]=d}return this.conjugate(s,A,r),this.transform(s,A,g,o,r,i),this.conjugate(g,o,r),this.normalize13b(g,r),n.negative=t.negative^e.negative,n.length=t.length+e.length,n.strip()},o.prototype.mul=function(t){var e=new o(null);return e.words=new Array(this.length+t.length),this.mulTo(t,e)},o.prototype.mulf=function(t){var e=new o(null);return e.words=new Array(this.length+t.length),d(this,t,e)},o.prototype.imul=function(t){return this.clone().mulTo(t,this)},o.prototype.imuln=function(t){r("number"==typeof t),r(t<67108864);for(var e=0,n=0;n<this.length;n++){var i=(0|this.words[n])*t,o=(67108863&i)+(67108863&e);e>>=26,e+=i/67108864|0,e+=o>>>26,this.words[n]=67108863&o}return 0!==e&&(this.words[n]=e,this.length++),this},o.prototype.muln=function(t){return this.clone().imuln(t)},o.prototype.sqr=function(){return this.mul(this)},o.prototype.isqr=function(){return this.imul(this.clone())},o.prototype.pow=function(t){var e=function(t){for(var e=new Array(t.bitLength()),n=0;n<e.length;n++){var r=n/26|0,i=n%26;e[n]=(t.words[r]&1<<i)>>>i}return e}(t);if(0===e.length)return new o(1);for(var n=this,r=0;r<e.length&&0===e[r];r++,n=n.sqr());if(++r<e.length)for(var i=n.sqr();r<e.length;r++,i=i.sqr())0!==e[r]&&(n=n.mul(i));return n},o.prototype.iushln=function(t){r("number"==typeof t&&t>=0);var e,n=t%26,i=(t-n)/26,o=67108863>>>26-n<<26-n;if(0!==n){var a=0;for(e=0;e<this.length;e++){var s=this.words[e]&o,A=(0|this.words[e])-s<<n;this.words[e]=A|a,a=s>>>26-n}a&&(this.words[e]=a,this.length++)}if(0!==i){for(e=this.length-1;e>=0;e--)this.words[e+i]=this.words[e];for(e=0;e<i;e++)this.words[e]=0;this.length+=i}return this.strip()},o.prototype.ishln=function(t){return r(0===this.negative),this.iushln(t)},o.prototype.iushrn=function(t,e,n){var i;r("number"==typeof t&&t>=0),i=e?(e-e%26)/26:0;var o=t%26,a=Math.min((t-o)/26,this.length),s=67108863^67108863>>>o<<o,A=n;if(i-=a,i=Math.max(0,i),A){for(var l=0;l<a;l++)A.words[l]=this.words[l];A.length=a}if(0===a);else if(this.length>a)for(this.length-=a,l=0;l<this.length;l++)this.words[l]=this.words[l+a];else this.words[0]=0,this.length=1;var c=0;for(l=this.length-1;l>=0&&(0!==c||l>=i);l--){var u=0|this.words[l];this.words[l]=c<<26-o|u>>>o,c=u&s}return A&&0!==c&&(A.words[A.length++]=c),0===this.length&&(this.words[0]=0,this.length=1),this.strip()},o.prototype.ishrn=function(t,e,n){return r(0===this.negative),this.iushrn(t,e,n)},o.prototype.shln=function(t){return this.clone().ishln(t)},o.prototype.ushln=function(t){return this.clone().iushln(t)},o.prototype.shrn=function(t){return this.clone().ishrn(t)},o.prototype.ushrn=function(t){return this.clone().iushrn(t)},o.prototype.testn=function(t){r("number"==typeof t&&t>=0);var e=t%26,n=(t-e)/26,i=1<<e;return!(this.length<=n||!(this.words[n]&i))},o.prototype.imaskn=function(t){r("number"==typeof t&&t>=0);var e=t%26,n=(t-e)/26;if(r(0===this.negative,"imaskn works only with positive numbers"),this.length<=n)return this;if(0!==e&&n++,this.length=Math.min(n,this.length),0!==e){var i=67108863^67108863>>>e<<e;this.words[this.length-1]&=i}return this.strip()},o.prototype.maskn=function(t){return this.clone().imaskn(t)},o.prototype.iaddn=function(t){return r("number"==typeof t),r(t<67108864),t<0?this.isubn(-t):0!==this.negative?1===this.length&&(0|this.words[0])<t?(this.words[0]=t-(0|this.words[0]),this.negative=0,this):(this.negative=0,this.isubn(t),this.negative=1,this):this._iaddn(t)},o.prototype._iaddn=function(t){this.words[0]+=t;for(var e=0;e<this.length&&this.words[e]>=67108864;e++)this.words[e]-=67108864,e===this.length-1?this.words[e+1]=1:this.words[e+1]++;return this.length=Math.max(this.length,e+1),this},o.prototype.isubn=function(t){if(r("number"==typeof t),r(t<67108864),t<0)return this.iaddn(-t);if(0!==this.negative)return this.negative=0,this.iaddn(t),this.negative=1,this;if(this.words[0]-=t,1===this.length&&this.words[0]<0)this.words[0]=-this.words[0],this.negative=1;else for(var e=0;e<this.length&&this.words[e]<0;e++)this.words[e]+=67108864,this.words[e+1]-=1;return this.strip()},o.prototype.addn=function(t){return this.clone().iaddn(t)},o.prototype.subn=function(t){return this.clone().isubn(t)},o.prototype.iabs=function(){return this.negative=0,this},o.prototype.abs=function(){return this.clone().iabs()},o.prototype._ishlnsubmul=function(t,e,n){var i,o,a=t.length+n;this._expand(a);var s=0;for(i=0;i<t.length;i++){o=(0|this.words[i+n])+s;var A=(0|t.words[i])*e;s=((o-=67108863&A)>>26)-(A/67108864|0),this.words[i+n]=67108863&o}for(;i<this.length-n;i++)s=(o=(0|this.words[i+n])+s)>>26,this.words[i+n]=67108863&o;if(0===s)return this.strip();for(r(-1===s),s=0,i=0;i<this.length;i++)s=(o=-(0|this.words[i])+s)>>26,this.words[i]=67108863&o;return this.negative=1,this.strip()},o.prototype._wordDiv=function(t,e){var n=(this.length,t.length),r=this.clone(),i=t,a=0|i.words[i.length-1];0!=(n=26-this._countBits(a))&&(i=i.ushln(n),r.iushln(n),a=0|i.words[i.length-1]);var s,A=r.length-i.length;if("mod"!==e){(s=new o(null)).length=A+1,s.words=new Array(s.length);for(var l=0;l<s.length;l++)s.words[l]=0}var c=r.clone()._ishlnsubmul(i,1,A);0===c.negative&&(r=c,s&&(s.words[A]=1));for(var u=A-1;u>=0;u--){var g=67108864*(0|r.words[i.length+u])+(0|r.words[i.length+u-1]);for(g=Math.min(g/a|0,67108863),r._ishlnsubmul(i,g,u);0!==r.negative;)g--,r.negative=0,r._ishlnsubmul(i,1,u),r.isZero()||(r.negative^=1);s&&(s.words[u]=g)}return s&&s.strip(),r.strip(),"div"!==e&&0!==n&&r.iushrn(n),{div:s||null,mod:r}},o.prototype.divmod=function(t,e,n){return r(!t.isZero()),this.isZero()?{div:new o(0),mod:new o(0)}:0!==this.negative&&0===t.negative?(s=this.neg().divmod(t,e),"mod"!==e&&(i=s.div.neg()),"div"!==e&&(a=s.mod.neg(),n&&0!==a.negative&&a.iadd(t)),{div:i,mod:a}):0===this.negative&&0!==t.negative?(s=this.divmod(t.neg(),e),"mod"!==e&&(i=s.div.neg()),{div:i,mod:s.mod}):0!=(this.negative&t.negative)?(s=this.neg().divmod(t.neg(),e),"div"!==e&&(a=s.mod.neg(),n&&0!==a.negative&&a.isub(t)),{div:s.div,mod:a}):t.length>this.length||this.cmp(t)<0?{div:new o(0),mod:this}:1===t.length?"div"===e?{div:this.divn(t.words[0]),mod:null}:"mod"===e?{div:null,mod:new o(this.modn(t.words[0]))}:{div:this.divn(t.words[0]),mod:new o(this.modn(t.words[0]))}:this._wordDiv(t,e);var i,a,s},o.prototype.div=function(t){return this.divmod(t,"div",!1).div},o.prototype.mod=function(t){return this.divmod(t,"mod",!1).mod},o.prototype.umod=function(t){return this.divmod(t,"mod",!0).mod},o.prototype.divRound=function(t){var e=this.divmod(t);if(e.mod.isZero())return e.div;var n=0!==e.div.negative?e.mod.isub(t):e.mod,r=t.ushrn(1),i=t.andln(1),o=n.cmp(r);return o<0||1===i&&0===o?e.div:0!==e.div.negative?e.div.isubn(1):e.div.iaddn(1)},o.prototype.modn=function(t){r(t<=67108863);for(var e=(1<<26)%t,n=0,i=this.length-1;i>=0;i--)n=(e*n+(0|this.words[i]))%t;return n},o.prototype.idivn=function(t){r(t<=67108863);for(var e=0,n=this.length-1;n>=0;n--){var i=(0|this.words[n])+67108864*e;this.words[n]=i/t|0,e=i%t}return this.strip()},o.prototype.divn=function(t){return this.clone().idivn(t)},o.prototype.egcd=function(t){r(0===t.negative),r(!t.isZero());var e=this,n=t.clone();e=0!==e.negative?e.umod(t):e.clone();for(var i=new o(1),a=new o(0),s=new o(0),A=new o(1),l=0;e.isEven()&&n.isEven();)e.iushrn(1),n.iushrn(1),++l;for(var c=n.clone(),u=e.clone();!e.isZero();){for(var g=0,h=1;0==(e.words[0]&h)&&g<26;++g,h<<=1);if(g>0)for(e.iushrn(g);g-- >0;)(i.isOdd()||a.isOdd())&&(i.iadd(c),a.isub(u)),i.iushrn(1),a.iushrn(1);for(var d=0,f=1;0==(n.words[0]&f)&&d<26;++d,f<<=1);if(d>0)for(n.iushrn(d);d-- >0;)(s.isOdd()||A.isOdd())&&(s.iadd(c),A.isub(u)),s.iushrn(1),A.iushrn(1);e.cmp(n)>=0?(e.isub(n),i.isub(s),a.isub(A)):(n.isub(e),s.isub(i),A.isub(a))}return{a:s,b:A,gcd:n.iushln(l)}},o.prototype._invmp=function(t){r(0===t.negative),r(!t.isZero());var e=this,n=t.clone();e=0!==e.negative?e.umod(t):e.clone();for(var i,a=new o(1),s=new o(0),A=n.clone();e.cmpn(1)>0&&n.cmpn(1)>0;){for(var l=0,c=1;0==(e.words[0]&c)&&l<26;++l,c<<=1);if(l>0)for(e.iushrn(l);l-- >0;)a.isOdd()&&a.iadd(A),a.iushrn(1);for(var u=0,g=1;0==(n.words[0]&g)&&u<26;++u,g<<=1);if(u>0)for(n.iushrn(u);u-- >0;)s.isOdd()&&s.iadd(A),s.iushrn(1);e.cmp(n)>=0?(e.isub(n),a.isub(s)):(n.isub(e),s.isub(a))}return(i=0===e.cmpn(1)?a:s).cmpn(0)<0&&i.iadd(t),i},o.prototype.gcd=function(t){if(this.isZero())return t.abs();if(t.isZero())return this.abs();var e=this.clone(),n=t.clone();e.negative=0,n.negative=0;for(var r=0;e.isEven()&&n.isEven();r++)e.iushrn(1),n.iushrn(1);for(;;){for(;e.isEven();)e.iushrn(1);for(;n.isEven();)n.iushrn(1);var i=e.cmp(n);if(i<0){var o=e;e=n,n=o}else if(0===i||0===n.cmpn(1))break;e.isub(n)}return n.iushln(r)},o.prototype.invm=function(t){return this.egcd(t).a.umod(t)},o.prototype.isEven=function(){return 0==(1&this.words[0])},o.prototype.isOdd=function(){return 1==(1&this.words[0])},o.prototype.andln=function(t){return this.words[0]&t},o.prototype.bincn=function(t){r("number"==typeof t);var e=t%26,n=(t-e)/26,i=1<<e;if(this.length<=n)return this._expand(n+1),this.words[n]|=i,this;for(var o=i,a=n;0!==o&&a<this.length;a++){var s=0|this.words[a];o=(s+=o)>>>26,s&=67108863,this.words[a]=s}return 0!==o&&(this.words[a]=o,this.length++),this},o.prototype.isZero=function(){return 1===this.length&&0===this.words[0]},o.prototype.cmpn=function(t){var e,n=t<0;if(0!==this.negative&&!n)return-1;if(0===this.negative&&n)return 1;if(this.strip(),this.length>1)e=1;else{n&&(t=-t),r(t<=67108863,"Number is too big");var i=0|this.words[0];e=i===t?0:i<t?-1:1}return 0!==this.negative?0|-e:e},o.prototype.cmp=function(t){if(0!==this.negative&&0===t.negative)return-1;if(0===this.negative&&0!==t.negative)return 1;var e=this.ucmp(t);return 0!==this.negative?0|-e:e},o.prototype.ucmp=function(t){if(this.length>t.length)return 1;if(this.length<t.length)return-1;for(var e=0,n=this.length-1;n>=0;n--){var r=0|this.words[n],i=0|t.words[n];if(r!==i){r<i?e=-1:r>i&&(e=1);break}}return e},o.prototype.gtn=function(t){return 1===this.cmpn(t)},o.prototype.gt=function(t){return 1===this.cmp(t)},o.prototype.gten=function(t){return this.cmpn(t)>=0},o.prototype.gte=function(t){return this.cmp(t)>=0},o.prototype.ltn=function(t){return-1===this.cmpn(t)},o.prototype.lt=function(t){return-1===this.cmp(t)},o.prototype.lten=function(t){return this.cmpn(t)<=0},o.prototype.lte=function(t){return this.cmp(t)<=0},o.prototype.eqn=function(t){return 0===this.cmpn(t)},o.prototype.eq=function(t){return 0===this.cmp(t)},o.red=function(t){return new v(t)},o.prototype.toRed=function(t){return r(!this.red,"Already a number in reduction context"),r(0===this.negative,"red works only with positives"),t.convertTo(this)._forceRed(t)},o.prototype.fromRed=function(){return r(this.red,"fromRed works only with numbers in reduction context"),this.red.convertFrom(this)},o.prototype._forceRed=function(t){return this.red=t,this},o.prototype.forceRed=function(t){return r(!this.red,"Already a number in reduction context"),this._forceRed(t)},o.prototype.redAdd=function(t){return r(this.red,"redAdd works only with red numbers"),this.red.add(this,t)},o.prototype.redIAdd=function(t){return r(this.red,"redIAdd works only with red numbers"),this.red.iadd(this,t)},o.prototype.redSub=function(t){return r(this.red,"redSub works only with red numbers"),this.red.sub(this,t)},o.prototype.redISub=function(t){return r(this.red,"redISub works only with red numbers"),this.red.isub(this,t)},o.prototype.redShl=function(t){return r(this.red,"redShl works only with red numbers"),this.red.shl(this,t)},o.prototype.redMul=function(t){return r(this.red,"redMul works only with red numbers"),this.red._verify2(this,t),this.red.mul(this,t)},o.prototype.redIMul=function(t){return r(this.red,"redMul works only with red numbers"),this.red._verify2(this,t),this.red.imul(this,t)},o.prototype.redSqr=function(){return r(this.red,"redSqr works only with red numbers"),this.red._verify1(this),this.red.sqr(this)},o.prototype.redISqr=function(){return r(this.red,"redISqr works only with red numbers"),this.red._verify1(this),this.red.isqr(this)},o.prototype.redSqrt=function(){return r(this.red,"redSqrt works only with red numbers"),this.red._verify1(this),this.red.sqrt(this)},o.prototype.redInvm=function(){return r(this.red,"redInvm works only with red numbers"),this.red._verify1(this),this.red.invm(this)},o.prototype.redNeg=function(){return r(this.red,"redNeg works only with red numbers"),this.red._verify1(this),this.red.neg(this)},o.prototype.redPow=function(t){return r(this.red&&!t.red,"redPow(normalNum)"),this.red._verify1(this),this.red.pow(this,t)};var p={k256:null,p224:null,p192:null,p25519:null};function I(t,e){this.name=t,this.p=new o(e,16),this.n=this.p.bitLength(),this.k=new o(1).iushln(this.n).isub(this.p),this.tmp=this._tmp()}function C(){I.call(this,"k256","ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff fffffffe fffffc2f")}function m(){I.call(this,"p224","ffffffff ffffffff ffffffff ffffffff 00000000 00000000 00000001")}function y(){I.call(this,"p192","ffffffff ffffffff ffffffff fffffffe ffffffff ffffffff")}function E(){I.call(this,"25519","7fffffffffffffff ffffffffffffffff ffffffffffffffff ffffffffffffffed")}function v(t){if("string"==typeof t){var e=o._prime(t);this.m=e.p,this.prime=e}else r(t.gtn(1),"modulus must be greater than 1"),this.m=t,this.prime=null}function B(t){v.call(this,t),this.shift=this.m.bitLength(),this.shift%26!=0&&(this.shift+=26-this.shift%26),this.r=new o(1).iushln(this.shift),this.r2=this.imod(this.r.sqr()),this.rinv=this.r._invmp(this.m),this.minv=this.rinv.mul(this.r).isubn(1).div(this.m),this.minv=this.minv.umod(this.r),this.minv=this.r.sub(this.minv)}I.prototype._tmp=function(){var t=new o(null);return t.words=new Array(Math.ceil(this.n/13)),t},I.prototype.ireduce=function(t){var e,n=t;do{this.split(n,this.tmp),e=(n=(n=this.imulK(n)).iadd(this.tmp)).bitLength()}while(e>this.n);var r=e<this.n?-1:n.ucmp(this.p);return 0===r?(n.words[0]=0,n.length=1):r>0?n.isub(this.p):n.strip(),n},I.prototype.split=function(t,e){t.iushrn(this.n,0,e)},I.prototype.imulK=function(t){return t.imul(this.k)},i(C,I),C.prototype.split=function(t,e){for(var n=Math.min(t.length,9),r=0;r<n;r++)e.words[r]=t.words[r];if(e.length=n,t.length<=9)return t.words[0]=0,void(t.length=1);var i=t.words[9];for(e.words[e.length++]=4194303&i,r=10;r<t.length;r++){var o=0|t.words[r];t.words[r-10]=(4194303&o)<<4|i>>>22,i=o}i>>>=22,t.words[r-10]=i,0===i&&t.length>10?t.length-=10:t.length-=9},C.prototype.imulK=function(t){t.words[t.length]=0,t.words[t.length+1]=0,t.length+=2;for(var e=0,n=0;n<t.length;n++){var r=0|t.words[n];e+=977*r,t.words[n]=67108863&e,e=64*r+(e/67108864|0)}return 0===t.words[t.length-1]&&(t.length--,0===t.words[t.length-1]&&t.length--),t},i(m,I),i(y,I),i(E,I),E.prototype.imulK=function(t){for(var e=0,n=0;n<t.length;n++){var r=19*(0|t.words[n])+e,i=67108863&r;r>>>=26,t.words[n]=i,e=r}return 0!==e&&(t.words[t.length++]=e),t},o._prime=function(t){if(p[t])return p[t];var e;if("k256"===t)e=new C;else if("p224"===t)e=new m;else if("p192"===t)e=new y;else{if("p25519"!==t)throw new Error("Unknown prime "+t);e=new E}return p[t]=e,e},v.prototype._verify1=function(t){r(0===t.negative,"red works only with positives"),r(t.red,"red works only with red numbers")},v.prototype._verify2=function(t,e){r(0==(t.negative|e.negative),"red works only with positives"),r(t.red&&t.red===e.red,"red works only with red numbers")},v.prototype.imod=function(t){return this.prime?this.prime.ireduce(t)._forceRed(this):t.umod(this.m)._forceRed(this)},v.prototype.neg=function(t){return t.isZero()?t.clone():this.m.sub(t)._forceRed(this)},v.prototype.add=function(t,e){this._verify2(t,e);var n=t.add(e);return n.cmp(this.m)>=0&&n.isub(this.m),n._forceRed(this)},v.prototype.iadd=function(t,e){this._verify2(t,e);var n=t.iadd(e);return n.cmp(this.m)>=0&&n.isub(this.m),n},v.prototype.sub=function(t,e){this._verify2(t,e);var n=t.sub(e);return n.cmpn(0)<0&&n.iadd(this.m),n._forceRed(this)},v.prototype.isub=function(t,e){this._verify2(t,e);var n=t.isub(e);return n.cmpn(0)<0&&n.iadd(this.m),n},v.prototype.shl=function(t,e){return this._verify1(t),this.imod(t.ushln(e))},v.prototype.imul=function(t,e){return this._verify2(t,e),this.imod(t.imul(e))},v.prototype.mul=function(t,e){return this._verify2(t,e),this.imod(t.mul(e))},v.prototype.isqr=function(t){return this.imul(t,t.clone())},v.prototype.sqr=function(t){return this.mul(t,t)},v.prototype.sqrt=function(t){if(t.isZero())return t.clone();var e=this.m.andln(3);if(r(e%2==1),3===e){var n=this.m.add(new o(1)).iushrn(2);return this.pow(t,n)}for(var i=this.m.subn(1),a=0;!i.isZero()&&0===i.andln(1);)a++,i.iushrn(1);r(!i.isZero());var s=new o(1).toRed(this),A=s.redNeg(),l=this.m.subn(1).iushrn(1),c=this.m.bitLength();for(c=new o(2*c*c).toRed(this);0!==this.pow(c,l).cmp(A);)c.redIAdd(A);for(var u=this.pow(c,i),g=this.pow(t,i.addn(1).iushrn(1)),h=this.pow(t,i),d=a;0!==h.cmp(s);){for(var f=h,p=0;0!==f.cmp(s);p++)f=f.redSqr();r(p<d);var I=this.pow(u,new o(1).iushln(d-p-1));g=g.redMul(I),u=I.redSqr(),h=h.redMul(u),d=p}return g},v.prototype.invm=function(t){var e=t._invmp(this.m);return 0!==e.negative?(e.negative=0,this.imod(e).redNeg()):this.imod(e)},v.prototype.pow=function(t,e){if(e.isZero())return new o(1).toRed(this);if(0===e.cmpn(1))return t.clone();var n=new Array(16);n[0]=new o(1).toRed(this),n[1]=t;for(var r=2;r<n.length;r++)n[r]=this.mul(n[r-1],t);var i=n[0],a=0,s=0,A=e.bitLength()%26;for(0===A&&(A=26),r=e.length-1;r>=0;r--){for(var l=e.words[r],c=A-1;c>=0;c--){var u=l>>c&1;i!==n[0]&&(i=this.sqr(i)),0!==u||0!==a?(a<<=1,a|=u,(4==++s||0===r&&0===c)&&(i=this.mul(i,n[a]),s=0,a=0)):s=0}A=26}return i},v.prototype.convertTo=function(t){var e=t.umod(this.m);return e===t?e.clone():e},v.prototype.convertFrom=function(t){var e=t.clone();return e.red=null,e},o.mont=function(t){return new B(t)},i(B,v),B.prototype.convertTo=function(t){return this.imod(t.ushln(this.shift))},B.prototype.convertFrom=function(t){var e=this.imod(t.mul(this.rinv));return e.red=null,e},B.prototype.imul=function(t,e){if(t.isZero()||e.isZero())return t.words[0]=0,t.length=1,t;var n=t.imul(e),r=n.maskn(this.shift).mul(this.minv).imaskn(this.shift).mul(this.m),i=n.isub(r).iushrn(this.shift),o=i;return i.cmp(this.m)>=0?o=i.isub(this.m):i.cmpn(0)<0&&(o=i.iadd(this.m)),o._forceRed(this)},B.prototype.mul=function(t,e){if(t.isZero()||e.isZero())return new o(0)._forceRed(this);var n=t.mul(e),r=n.maskn(this.shift).mul(this.minv).imaskn(this.shift).mul(this.m),i=n.isub(r).iushrn(this.shift),a=i;return i.cmp(this.m)>=0?a=i.isub(this.m):i.cmpn(0)<0&&(a=i.iadd(this.m)),a._forceRed(this)},B.prototype.invm=function(t){return this.imod(t._invmp(this.m).mul(this.r2))._forceRed(this)}}(void 0===e||e,this)},{buffer:103}],95:[function(t,e,n){"use strict";e.exports=function(t){var e,n,r,i=t.length,o=0;for(e=0;e<i;++e)o+=t[e].length;var a=new Array(o),s=0;for(e=0;e<i;++e){var A=t[e],l=A.length;for(n=0;n<l;++n){var c=a[s++]=new Array(l-1),u=0;for(r=0;r<l;++r)r!==n&&(c[u++]=A[r]);if(1&n){var g=c[1];c[1]=c[0],c[0]=g}}}return a}},{}],96:[function(t,e,n){"use strict";e.exports=function(t,e,n){switch(arguments.length){case 1:return r=[],l(i=t,i,c,!0),r;case 2:return"function"==typeof e?l(t,t,e,!0):function(t,e){return r=[],l(t,e,c,!1),r}(t,e);case 3:return l(t,e,n,!1);default:throw new Error("box-intersect: Invalid arguments")}var i};var r,i=t("typedarray-pool"),o=t("./lib/sweep"),a=t("./lib/intersect");function s(t,e){for(var n=0;n<t;++n)if(!(e[n]<=e[n+t]))return!0;return!1}function A(t,e,n,r){for(var i=0,o=0,a=0,A=t.length;a<A;++a){var l=t[a];if(!s(e,l)){for(var c=0;c<2*e;++c)n[i++]=l[c];r[o++]=a}}return o}function l(t,e,n,r){var s=t.length,l=e.length;if(!(s<=0||l<=0)){var c=t[0].length>>>1;if(!(c<=0)){var u,g=i.mallocDouble(2*c*s),h=i.mallocInt32(s);if((s=A(t,c,g,h))>0){if(1===c&&r)o.init(s),u=o.sweepComplete(c,n,0,s,g,h,0,s,g,h);else{var d=i.mallocDouble(2*c*l),f=i.mallocInt32(l);(l=A(e,c,d,f))>0&&(o.init(s+l),u=1===c?o.sweepBipartite(c,n,0,s,g,h,0,l,d,f):a(c,n,r,s,g,h,l,d,f),i.free(d),i.free(f))}i.free(g),i.free(h)}return u}}}function c(t,e){r.push([t,e])}},{"./lib/intersect":98,"./lib/sweep":102,"typedarray-pool":545}],97:[function(t,e,n){"use strict";var r="d",i="ax",o="vv",a="fp",s="es",A="rs",l="re",c="rb",u="ri",g="rp",h="bs",d="be",f="bb",p="bi",I="bp",C="rv",m="Q",y=[r,i,o,A,l,c,u,h,d,f,p];function E(t){var e="bruteForce"+(t?"Full":"Partial"),n=[],E=y.slice();t||E.splice(3,0,a);var v=["function "+e+"("+E.join()+"){"];function B(e,a){var E=function(t,e,n){var a="bruteForce"+(t?"Red":"Blue")+(e?"Flip":"")+(n?"Full":""),E=["function ",a,"(",y.join(),"){","var ",s,"=2*",r,";"],v="for(var i="+A+","+g+"="+s+"*"+A+";i<"+l+";++i,"+g+"+="+s+"){var x0="+c+"["+i+"+"+g+"],x1="+c+"["+i+"+"+g+"+"+r+"],xi="+u+"[i];",B="for(var j="+h+","+I+"="+s+"*"+h+";j<"+d+";++j,"+I+"+="+s+"){var y0="+f+"["+i+"+"+I+"],"+(n?"y1="+f+"["+i+"+"+I+"+"+r+"],":"")+"yi="+p+"[j];";return t?E.push(v,m,":",B):E.push(B,m,":",v),n?E.push("if(y1<x0||x1<y0)continue;"):e?E.push("if(y0<=x0||x1<y0)continue;"):E.push("if(y0<x0||x1<y0)continue;"),E.push("for(var k="+i+"+1;k<"+r+";++k){var r0="+c+"[k+"+g+"],r1="+c+"[k+"+r+"+"+g+"],b0="+f+"[k+"+I+"],b1="+f+"[k+"+r+"+"+I+"];if(r1<b0||b1<r0)continue "+m+";}var "+C+"="+o+"("),e?E.push("yi,xi"):E.push("xi,yi"),E.push(");if("+C+"!==void 0)return "+C+";}}}"),{name:a,code:E.join("")}}(e,a,t);n.push(E.code),v.push("return "+E.name+"("+y.join()+");")}v.push("if("+l+"-"+A+">"+d+"-"+h+"){"),t?(B(!0,!1),v.push("}else{"),B(!1,!1)):(v.push("if("+a+"){"),B(!0,!0),v.push("}else{"),B(!0,!1),v.push("}}else{if("+a+"){"),B(!1,!0),v.push("}else{"),B(!1,!1),v.push("}")),v.push("}}return "+e);var b=n.join("")+v.join("");return new Function(b)()}n.partial=E(!1),n.full=E(!0)},{}],98:[function(t,e,n){"use strict";e.exports=function(t,e,n,o,c,x,_,D,M){!function(t,e){var n=8*i.log2(e+1)*(t+1)|0,o=i.nextPow2(y*n);v.length<o&&(r.free(v),v=r.mallocInt32(o));var a=i.nextPow2(E*n);B<a&&(r.free(B),B=r.mallocDouble(a))}(t,o+_);var k,S=0,L=2*t;for(b(S++,0,0,o,0,_,n?16:0,-1/0,1/0),n||b(S++,0,0,_,0,o,1,-1/0,1/0);S>0;){var R=(S-=1)*y,N=v[R],F=v[R+1],T=v[R+2],U=v[R+3],O=v[R+4],G=v[R+5],H=S*E,j=B[H],Y=B[H+1],P=1&G,K=!!(16&G),z=c,J=x,q=D,V=M;if(P&&(z=D,J=M,q=c,V=x),!(2&G&&(T=p(t,N,F,T,z,J,Y),F>=T)||4&G&&(F=I(t,N,F,T,z,J,j))>=T)){var W=T-F,X=O-U;if(K){if(t*W*(W+X)<h){if(void 0!==(k=A.scanComplete(t,N,e,F,T,z,J,U,O,q,V)))return k;continue}}else{if(t*Math.min(W,X)<u){if(void 0!==(k=a(t,N,e,P,F,T,z,J,U,O,q,V)))return k;continue}if(t*W*X<g){if(void 0!==(k=A.scanBipartite(t,N,e,P,F,T,z,J,U,O,q,V)))return k;continue}}var Z=d(t,N,F,T,z,J,j,Y);if(F<Z)if(t*(Z-F)<u){if(void 0!==(k=s(t,N+1,e,F,Z,z,J,U,O,q,V)))return k}else if(N===t-2){if(void 0!==(k=P?A.sweepBipartite(t,e,U,O,q,V,F,Z,z,J):A.sweepBipartite(t,e,F,Z,z,J,U,O,q,V)))return k}else b(S++,N+1,F,Z,U,O,P,-1/0,1/0),b(S++,N+1,U,O,F,Z,1^P,-1/0,1/0);if(Z<T){var $=l(t,N,U,O,q,V),tt=q[L*$+N],et=f(t,N,$,O,q,V,tt);if(et<O&&b(S++,N,Z,T,et,O,(4|P)+(K?16:0),tt,Y),U<$&&b(S++,N,Z,T,U,$,(2|P)+(K?16:0),j,tt),$+1===et){if(void 0!==(k=K?w(t,N,e,Z,T,z,J,$,q,V[$]):Q(t,N,e,P,Z,T,z,J,$,q,V[$])))return k}else if($<et){var nt;if(K){if(nt=C(t,N,Z,T,z,J,tt),Z<nt){var rt=f(t,N,Z,nt,z,J,tt);if(N===t-2){if(Z<rt&&void 0!==(k=A.sweepComplete(t,e,Z,rt,z,J,$,et,q,V)))return k;if(rt<nt&&void 0!==(k=A.sweepBipartite(t,e,rt,nt,z,J,$,et,q,V)))return k}else Z<rt&&b(S++,N+1,Z,rt,$,et,16,-1/0,1/0),rt<nt&&(b(S++,N+1,rt,nt,$,et,0,-1/0,1/0),b(S++,N+1,$,et,rt,nt,1,-1/0,1/0))}}else nt=P?m(t,N,Z,T,z,J,tt):C(t,N,Z,T,z,J,tt),Z<nt&&(N===t-2?k=P?A.sweepBipartite(t,e,$,et,q,V,Z,nt,z,J):A.sweepBipartite(t,e,Z,nt,z,J,$,et,q,V):(b(S++,N+1,Z,nt,$,et,P,-1/0,1/0),b(S++,N+1,$,et,Z,nt,1^P,-1/0,1/0)))}}}}};var r=t("typedarray-pool"),i=t("bit-twiddle"),o=t("./brute"),a=o.partial,s=o.full,A=t("./sweep"),l=t("./median"),c=t("./partition"),u=128,g=1<<22,h=1<<22,d=c("!(lo>=p0)&&!(p1>=hi)",["p0","p1"]),f=c("lo===p0",["p0"]),p=c("lo<p0",["p0"]),I=c("hi<=p0",["p0"]),C=c("lo<=p0&&p0<=hi",["p0"]),m=c("lo<p0&&p0<=hi",["p0"]),y=6,E=2,v=r.mallocInt32(1024),B=r.mallocDouble(1024);function b(t,e,n,r,i,o,a,s,A){var l=y*t;v[l]=e,v[l+1]=n,v[l+2]=r,v[l+3]=i,v[l+4]=o,v[l+5]=a;var c=E*t;B[c]=s,B[c+1]=A}function Q(t,e,n,r,i,o,a,s,A,l,c){var u=2*t,g=A*u,h=l[g+e];t:for(var d=i,f=i*u;d<o;++d,f+=u){var p=a[f+e],I=a[f+e+t];if(!(h<p||I<h||r&&h===p)){for(var C,m=s[d],y=e+1;y<t;++y){p=a[f+y],I=a[f+y+t];var E=l[g+y],v=l[g+y+t];if(I<E||v<p)continue t}if(void 0!==(C=r?n(c,m):n(m,c)))return C}}}function w(t,e,n,r,i,o,a,s,A,l){var c=2*t,u=s*c,g=A[u+e];t:for(var h=r,d=r*c;h<i;++h,d+=c){var f=a[h];if(f!==l){var p=o[d+e],I=o[d+e+t];if(!(g<p||I<g)){for(var C=e+1;C<t;++C){p=o[d+C],I=o[d+C+t];var m=A[u+C],y=A[u+C+t];if(I<m||y<p)continue t}var E=n(f,l);if(void 0!==E)return E}}}}},{"./brute":97,"./median":99,"./partition":100,"./sweep":102,"bit-twiddle":92,"typedarray-pool":545}],99:[function(t,e,n){"use strict";e.exports=function(t,e,n,a,s,A){if(a<=n+1)return n;for(var l=n,c=a,u=a+n>>>1,g=2*t,h=u,d=s[g*u+e];l<c;){if(c-l<i){o(t,e,l,c,s,A),d=s[g*u+e];break}var f=c-l,p=Math.random()*f+l|0,I=s[g*p+e],C=Math.random()*f+l|0,m=s[g*C+e],y=Math.random()*f+l|0,E=s[g*y+e];I<=m?E>=m?(h=C,d=m):I>=E?(h=p,d=I):(h=y,d=E):m>=E?(h=C,d=m):E>=I?(h=p,d=I):(h=y,d=E);for(var v=g*(c-1),B=g*h,b=0;b<g;++b,++v,++B){var Q=s[v];s[v]=s[B],s[B]=Q}var w=A[c-1];A[c-1]=A[h],A[h]=w,h=r(t,e,l,c-1,s,A,d);for(var v=g*(c-1),B=g*h,b=0;b<g;++b,++v,++B){var Q=s[v];s[v]=s[B],s[B]=Q}var w=A[c-1];if(A[c-1]=A[h],A[h]=w,u<h){for(c=h-1;l<c&&s[g*(c-1)+e]===d;)c-=1;c+=1}else{if(!(h<u))break;for(l=h+1;l<c&&s[g*l+e]===d;)l+=1}}return r(t,e,n,u,s,A,s[g*u+e])};var r=t("./partition")("lo<p0",["p0"]),i=8;function o(t,e,n,r,i,o){for(var a=2*t,s=a*(n+1)+e,A=n+1;A<r;++A,s+=a)for(var l=i[s],c=A,u=a*(A-1);c>n&&i[u+e]>l;--c,u-=a){for(var g=u,h=u+a,d=0;d<a;++d,++g,++h){var f=i[g];i[g]=i[h],i[h]=f}var p=o[c];o[c]=o[c-1],o[c-1]=p}}},{"./partition":100}],100:[function(t,e,n){"use strict";e.exports=function(t,e){var n="abcdef".split("").concat(e),i=[];return t.indexOf("lo")>=0&&i.push("lo=e[k+n]"),t.indexOf("hi")>=0&&i.push("hi=e[k+o]"),n.push(r.replace("_",i.join()).replace("$",t)),Function.apply(void 0,n)};var r="for(var j=2*a,k=j*c,l=k,m=c,n=b,o=a+b,p=c;d>p;++p,k+=j){var _;if($)if(m===p)m+=1,l+=j;else{for(var s=0;j>s;++s){var t=e[k+s];e[k+s]=e[l],e[l++]=t}var u=f[p];f[p]=f[m],f[m++]=u}}return m"},{}],101:[function(t,e,n){"use strict";e.exports=function(t,e){e<=4*r?i(0,e-1,t):function t(e,n,u){var g=(n-e+1)/6|0,h=e+g,d=n-g,f=e+n>>1,p=f-g,I=f+g,C=h,m=p,y=f,E=I,v=d,B=e+1,b=n-1,Q=0;l(C,m,u)&&(Q=C,C=m,m=Q),l(E,v,u)&&(Q=E,E=v,v=Q),l(C,y,u)&&(Q=C,C=y,y=Q),l(m,y,u)&&(Q=m,m=y,y=Q),l(C,E,u)&&(Q=C,C=E,E=Q),l(y,E,u)&&(Q=y,y=E,E=Q),l(m,v,u)&&(Q=m,m=v,v=Q),l(m,y,u)&&(Q=m,m=y,y=Q),l(E,v,u)&&(Q=E,E=v,v=Q);for(var w=u[2*m],x=u[2*m+1],_=u[2*E],D=u[2*E+1],M=2*C,k=2*y,S=2*v,L=2*h,R=2*f,N=2*d,F=0;F<2;++F){var T=u[M+F],U=u[k+F],O=u[S+F];u[L+F]=T,u[R+F]=U,u[N+F]=O}a(p,e,u),a(I,n,u);for(var G=B;G<=b;++G)if(c(G,w,x,u))G!==B&&o(G,B,u),++B;else if(!c(G,_,D,u))for(;;){if(c(b,_,D,u)){c(b,w,x,u)?(s(G,B,b,u),++B,--b):(o(G,b,u),--b);break}if(--b<G)break}A(e,B-1,w,x,u),A(n,b+1,_,D,u),B-2-e<=r?i(e,B-2,u):t(e,B-2,u),n-(b+2)<=r?i(b+2,n,u):t(b+2,n,u),b-B<=r?i(B,b,u):t(B,b,u)}(0,e-1,t)};var r=32;function i(t,e,n){for(var r=2*(t+1),i=t+1;i<=e;++i){for(var o=n[r++],a=n[r++],s=i,A=r-2;s-- >t;){var l=n[A-2],c=n[A-1];if(l<o)break;if(l===o&&c<a)break;n[A]=l,n[A+1]=c,A-=2}n[A]=o,n[A+1]=a}}function o(t,e,n){e*=2;var r=n[t*=2],i=n[t+1];n[t]=n[e],n[t+1]=n[e+1],n[e]=r,n[e+1]=i}function a(t,e,n){e*=2,n[t*=2]=n[e],n[t+1]=n[e+1]}function s(t,e,n,r){e*=2,n*=2;var i=r[t*=2],o=r[t+1];r[t]=r[e],r[t+1]=r[e+1],r[e]=r[n],r[e+1]=r[n+1],r[n]=i,r[n+1]=o}function A(t,e,n,r,i){e*=2,i[t*=2]=i[e],i[e]=n,i[t+1]=i[e+1],i[e+1]=r}function l(t,e,n){e*=2;var r=n[t*=2],i=n[e];return!(r<i)&&(r!==i||n[t+1]>n[e+1])}function c(t,e,n,r){var i=r[t*=2];return i<e||i===e&&r[t+1]<n}},{}],102:[function(t,e,n){"use strict";e.exports={init:function(t){var e=i.nextPow2(t);s.length<e&&(r.free(s),s=r.mallocInt32(e)),A.length<e&&(r.free(A),A=r.mallocInt32(e)),l.length<e&&(r.free(l),l=r.mallocInt32(e)),c.length<e&&(r.free(c),c=r.mallocInt32(e)),u.length<e&&(r.free(u),u=r.mallocInt32(e)),g.length<e&&(r.free(g),g=r.mallocInt32(e));var n=8*e;h.length<n&&(r.free(h),h=r.mallocDouble(n))},sweepBipartite:function(t,e,n,r,i,u,g,p,I,C){for(var m=0,y=2*t,E=t-1,v=y-1,B=n;B<r;++B){var b=u[B],Q=y*B;h[m++]=i[Q+E],h[m++]=-(b+1),h[m++]=i[Q+v],h[m++]=b}for(var B=g;B<p;++B){var b=C[B]+a,w=y*B;h[m++]=I[w+E],h[m++]=-b,h[m++]=I[w+v],h[m++]=b}var x=m>>>1;o(h,x);for(var _=0,D=0,B=0;B<x;++B){var M=0|h[2*B+1];if(M>=a)d(l,c,D--,M=M-a|0);else if(M>=0)d(s,A,_--,M);else if(M<=-a){M=-M-a|0;for(var k=0;k<_;++k){var S=e(s[k],M);if(void 0!==S)return S}f(l,c,D++,M)}else{M=-M-1|0;for(var k=0;k<D;++k){var S=e(M,l[k]);if(void 0!==S)return S}f(s,A,_++,M)}}},sweepComplete:function(t,e,n,r,i,a,p,I,C,m){for(var y=0,E=2*t,v=t-1,B=E-1,b=n;b<r;++b){var Q=a[b]+1<<1,w=E*b;h[y++]=i[w+v],h[y++]=-Q,h[y++]=i[w+B],h[y++]=Q}for(var b=p;b<I;++b){var Q=m[b]+1<<1,x=E*b;h[y++]=C[x+v],h[y++]=1|-Q,h[y++]=C[x+B],h[y++]=1|Q}var _=y>>>1;o(h,_);for(var D=0,M=0,k=0,b=0;b<_;++b){var S=0|h[2*b+1],L=1&S;if(b<_-1&&S>>1==h[2*b+3]>>1&&(L=2,b+=1),S<0){for(var R=-(S>>1)-1,N=0;N<k;++N){var F=e(u[N],R);if(void 0!==F)return F}if(0!==L)for(var N=0;N<D;++N){var F=e(s[N],R);if(void 0!==F)return F}if(1!==L)for(var N=0;N<M;++N){var F=e(l[N],R);if(void 0!==F)return F}0===L?f(s,A,D++,R):1===L?f(l,c,M++,R):2===L&&f(u,g,k++,R)}else{var R=(S>>1)-1;0===L?d(s,A,D--,R):1===L?d(l,c,M--,R):2===L&&d(u,g,k--,R)}}},scanBipartite:function(t,e,n,r,i,l,c,u,g,p,I,C){var m=0,y=2*t,E=e,v=e+t,B=1,b=1;r?b=a:B=a;for(var Q=i;Q<l;++Q){var w=Q+B,x=y*Q;h[m++]=c[x+E],h[m++]=-w,h[m++]=c[x+v],h[m++]=w}for(var Q=g;Q<p;++Q){var w=Q+b,_=y*Q;h[m++]=I[_+E],h[m++]=-w}var D=m>>>1;o(h,D);for(var M=0,Q=0;Q<D;++Q){var k=0|h[2*Q+1];if(k<0){var w=-k,S=!1;if(w>=a?(S=!r,w-=a):(S=!!r,w-=1),S)f(s,A,M++,w);else{var L=C[w],R=y*w,N=I[R+e+1],F=I[R+e+1+t];t:for(var T=0;T<M;++T){var U=s[T],O=y*U;if(!(F<c[O+e+1]||c[O+e+1+t]<N)){for(var G=e+2;G<t;++G)if(I[R+G+t]<c[O+G]||c[O+G+t]<I[R+G])continue t;var H,j=u[U];if(void 0!==(H=r?n(L,j):n(j,L)))return H}}}}else d(s,A,M--,k-B)}},scanComplete:function(t,e,n,r,i,A,l,c,u,g,d){for(var f=0,p=2*t,I=e,C=e+t,m=r;m<i;++m){var y=m+a,E=p*m;h[f++]=A[E+I],h[f++]=-y,h[f++]=A[E+C],h[f++]=y}for(var m=c;m<u;++m){var y=m+1,v=p*m;h[f++]=g[v+I],h[f++]=-y}var B=f>>>1;o(h,B);for(var b=0,m=0;m<B;++m){var Q=0|h[2*m+1];if(Q<0){var y=-Q;if(y>=a)s[b++]=y-a;else{var w=d[y-=1],x=p*y,_=g[x+e+1],D=g[x+e+1+t];t:for(var M=0;M<b;++M){var k=s[M],S=l[k];if(S===w)break;var L=p*k;if(!(D<A[L+e+1]||A[L+e+1+t]<_)){for(var R=e+2;R<t;++R)if(g[x+R+t]<A[L+R]||A[L+R+t]<g[x+R])continue t;var N=n(S,w);if(void 0!==N)return N}}}}else{for(var y=Q-a,M=b-1;M>=0;--M)if(s[M]===y){for(var R=M+1;R<b;++R)s[R-1]=s[R];break}--b}}}};var r=t("typedarray-pool"),i=t("bit-twiddle"),o=t("./sort"),a=1<<28,s=r.mallocInt32(1024),A=r.mallocInt32(1024),l=r.mallocInt32(1024),c=r.mallocInt32(1024),u=r.mallocInt32(1024),g=r.mallocInt32(1024),h=r.mallocDouble(8192);function d(t,e,n,r){var i=e[r],o=t[n-1];t[i]=o,e[o]=i}function f(t,e,n,r){t[n]=r,e[r]=n}},{"./sort":101,"bit-twiddle":92,"typedarray-pool":545}],103:[function(t,e,n){},{}],104:[function(t,e,n){var r=Object.create||function(t){var e=function(){};return e.prototype=t,new e},i=Object.keys||function(t){var e=[];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&e.push(n);return n},o=Function.prototype.bind||function(t){var e=this;return function(){return e.apply(t,arguments)}};function a(){this._events&&Object.prototype.hasOwnProperty.call(this,"_events")||(this._events=r(null),this._eventsCount=0),this._maxListeners=this._maxListeners||void 0}e.exports=a,a.EventEmitter=a,a.prototype._events=void 0,a.prototype._maxListeners=void 0;var s,A=10;try{var l={};Object.defineProperty&&Object.defineProperty(l,"x",{value:0}),s=0===l.x}catch(t){s=!1}function c(t){return void 0===t._maxListeners?a.defaultMaxListeners:t._maxListeners}function u(t,e,n,i){var o,a,s;if("function"!=typeof n)throw new TypeError('"listener" argument must be a function');if((a=t._events)?(a.newListener&&(t.emit("newListener",e,n.listener?n.listener:n),a=t._events),s=a[e]):(a=t._events=r(null),t._eventsCount=0),s){if("function"==typeof s?s=a[e]=i?[n,s]:[s,n]:i?s.unshift(n):s.push(n),!s.warned&&(o=c(t))&&o>0&&s.length>o){s.warned=!0;var A=new Error("Possible EventEmitter memory leak detected. "+s.length+' "'+String(e)+'" listeners added. Use emitter.setMaxListeners() to increase limit.');A.name="MaxListenersExceededWarning",A.emitter=t,A.type=e,A.count=s.length,"object"==typeof console&&console.warn&&console.warn("%s: %s",A.name,A.message)}}else s=a[e]=n,++t._eventsCount;return t}function g(){if(!this.fired)switch(this.target.removeListener(this.type,this.wrapFn),this.fired=!0,arguments.length){case 0:return this.listener.call(this.target);case 1:return this.listener.call(this.target,arguments[0]);case 2:return this.listener.call(this.target,arguments[0],arguments[1]);case 3:return this.listener.call(this.target,arguments[0],arguments[1],arguments[2]);default:for(var t=new Array(arguments.length),e=0;e<t.length;++e)t[e]=arguments[e];this.listener.apply(this.target,t)}}function h(t,e,n){var r={fired:!1,wrapFn:void 0,target:t,type:e,listener:n},i=o.call(g,r);return i.listener=n,r.wrapFn=i,i}function d(t,e,n){var r=t._events;if(!r)return[];var i=r[e];return i?"function"==typeof i?n?[i.listener||i]:[i]:n?function(t){for(var e=new Array(t.length),n=0;n<e.length;++n)e[n]=t[n].listener||t[n];return e}(i):p(i,i.length):[]}function f(t){var e=this._events;if(e){var n=e[t];if("function"==typeof n)return 1;if(n)return n.length}return 0}function p(t,e){for(var n=new Array(e),r=0;r<e;++r)n[r]=t[r];return n}s?Object.defineProperty(a,"defaultMaxListeners",{enumerable:!0,get:function(){return A},set:function(t){if("number"!=typeof t||t<0||t!=t)throw new TypeError('"defaultMaxListeners" must be a positive number');A=t}}):a.defaultMaxListeners=A,a.prototype.setMaxListeners=function(t){if("number"!=typeof t||t<0||isNaN(t))throw new TypeError('"n" argument must be a positive number');return this._maxListeners=t,this},a.prototype.getMaxListeners=function(){return c(this)},a.prototype.emit=function(t){var e,n,r,i,o,a,s="error"===t;if(a=this._events)s=s&&null==a.error;else if(!s)return!1;if(s){if(arguments.length>1&&(e=arguments[1]),e instanceof Error)throw e;var A=new Error('Unhandled "error" event. ('+e+")");throw A.context=e,A}if(!(n=a[t]))return!1;var l="function"==typeof n;switch(r=arguments.length){case 1:!function(t,e,n){for(var r=t.length,i=p(t,r),o=0;o<r;++o)i[o].call(n)}(n,0,this);break;case 2:!function(t,e,n,r){if(e)t.call(n,r);else for(var i=t.length,o=p(t,i),a=0;a<i;++a)o[a].call(n,r)}(n,l,this,arguments[1]);break;case 3:!function(t,e,n,r,i){if(e)t.call(n,r,i);else for(var o=t.length,a=p(t,o),s=0;s<o;++s)a[s].call(n,r,i)}(n,l,this,arguments[1],arguments[2]);break;case 4:!function(t,e,n,r,i,o){if(e)t.call(n,r,i,o);else for(var a=t.length,s=p(t,a),A=0;A<a;++A)s[A].call(n,r,i,o)}(n,l,this,arguments[1],arguments[2],arguments[3]);break;default:for(i=new Array(r-1),o=1;o<r;o++)i[o-1]=arguments[o];!function(t,e,n,r){for(var i=t.length,o=p(t,i),a=0;a<i;++a)o[a].apply(n,r)}(n,0,this,i)}return!0},a.prototype.addListener=function(t,e){return u(this,t,e,!1)},a.prototype.on=a.prototype.addListener,a.prototype.prependListener=function(t,e){return u(this,t,e,!0)},a.prototype.once=function(t,e){if("function"!=typeof e)throw new TypeError('"listener" argument must be a function');return this.on(t,h(this,t,e)),this},a.prototype.prependOnceListener=function(t,e){if("function"!=typeof e)throw new TypeError('"listener" argument must be a function');return this.prependListener(t,h(this,t,e)),this},a.prototype.removeListener=function(t,e){var n,i,o,a,s;if("function"!=typeof e)throw new TypeError('"listener" argument must be a function');if(!(i=this._events))return this;if(!(n=i[t]))return this;if(n===e||n.listener===e)0==--this._eventsCount?this._events=r(null):(delete i[t],i.removeListener&&this.emit("removeListener",t,n.listener||e));else if("function"!=typeof n){for(o=-1,a=n.length-1;a>=0;a--)if(n[a]===e||n[a].listener===e){s=n[a].listener,o=a;break}if(o<0)return this;0===o?n.shift():function(t,e){for(var n=e,r=n+1,i=t.length;r<i;n+=1,r+=1)t[n]=t[r];t.pop()}(n,o),1===n.length&&(i[t]=n[0]),i.removeListener&&this.emit("removeListener",t,s||e)}return this},a.prototype.removeAllListeners=function(t){var e,n,o;if(!(n=this._events))return this;if(!n.removeListener)return 0===arguments.length?(this._events=r(null),this._eventsCount=0):n[t]&&(0==--this._eventsCount?this._events=r(null):delete n[t]),this;if(0===arguments.length){var a,s=i(n);for(o=0;o<s.length;++o)"removeListener"!==(a=s[o])&&this.removeAllListeners(a);return this.removeAllListeners("removeListener"),this._events=r(null),this._eventsCount=0,this}if("function"==typeof(e=n[t]))this.removeListener(t,e);else if(e)for(o=e.length-1;o>=0;o--)this.removeListener(t,e[o]);return this},a.prototype.listeners=function(t){return d(this,t,!0)},a.prototype.rawListeners=function(t){return d(this,t,!1)},a.listenerCount=function(t,e){return"function"==typeof t.listenerCount?t.listenerCount(e):f.call(t,e)},a.prototype.listenerCount=f,a.prototype.eventNames=function(){return this._eventsCount>0?Reflect.ownKeys(this._events):[]}},{}],105:[function(t,e,n){(function(e){"use strict";var r=t("base64-js"),i=t("ieee754");n.Buffer=e,n.SlowBuffer=function(t){return+t!=t&&(t=0),e.alloc(+t)},n.INSPECT_MAX_BYTES=50;var o=2147483647;function a(t){if(t>o)throw new RangeError('The value "'+t+'" is invalid for option "size"');var n=new Uint8Array(t);return n.__proto__=e.prototype,n}function e(t,e,n){if("number"==typeof t){if("string"==typeof e)throw new TypeError('The "string" argument must be of type string. Received type number');return l(t)}return s(t,e,n)}function s(t,n,r){if("string"==typeof t)return function(t,n){if("string"==typeof n&&""!==n||(n="utf8"),!e.isEncoding(n))throw new TypeError("Unknown encoding: "+n);var r=0|g(t,n),i=a(r),o=i.write(t,n);return o!==r&&(i=i.slice(0,o)),i}(t,n);if(ArrayBuffer.isView(t))return c(t);if(null==t)throw TypeError("The first argument must be one of type string, Buffer, ArrayBuffer, Array, or Array-like Object. Received type "+typeof t);if(O(t,ArrayBuffer)||t&&O(t.buffer,ArrayBuffer))return function(t,n,r){if(n<0||t.byteLength<n)throw new RangeError('"offset" is outside of buffer bounds');if(t.byteLength<n+(r||0))throw new RangeError('"length" is outside of buffer bounds');var i;return(i=void 0===n&&void 0===r?new Uint8Array(t):void 0===r?new Uint8Array(t,n):new Uint8Array(t,n,r)).__proto__=e.prototype,i}(t,n,r);if("number"==typeof t)throw new TypeError('The "value" argument must not be of type number. Received type number');var i=t.valueOf&&t.valueOf();if(null!=i&&i!==t)return e.from(i,n,r);var o=function(t){if(e.isBuffer(t)){var n=0|u(t.length),r=a(n);return 0===r.length?r:(t.copy(r,0,0,n),r)}return void 0!==t.length?"number"!=typeof t.length||G(t.length)?a(0):c(t):"Buffer"===t.type&&Array.isArray(t.data)?c(t.data):void 0}(t);if(o)return o;if("undefined"!=typeof Symbol&&null!=Symbol.toPrimitive&&"function"==typeof t[Symbol.toPrimitive])return e.from(t[Symbol.toPrimitive]("string"),n,r);throw new TypeError("The first argument must be one of type string, Buffer, ArrayBuffer, Array, or Array-like Object. Received type "+typeof t)}function A(t){if("number"!=typeof t)throw new TypeError('"size" argument must be of type number');if(t<0)throw new RangeError('The value "'+t+'" is invalid for option "size"')}function l(t){return A(t),a(t<0?0:0|u(t))}function c(t){for(var e=t.length<0?0:0|u(t.length),n=a(e),r=0;r<e;r+=1)n[r]=255&t[r];return n}function u(t){if(t>=o)throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x"+o.toString(16)+" bytes");return 0|t}function g(t,n){if(e.isBuffer(t))return t.length;if(ArrayBuffer.isView(t)||O(t,ArrayBuffer))return t.byteLength;if("string"!=typeof t)throw new TypeError('The "string" argument must be one of type string, Buffer, or ArrayBuffer. Received type '+typeof t);var r=t.length,i=arguments.length>2&&!0===arguments[2];if(!i&&0===r)return 0;for(var o=!1;;)switch(n){case"ascii":case"latin1":case"binary":return r;case"utf8":case"utf-8":return F(t).length;case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return 2*r;case"hex":return r>>>1;case"base64":return T(t).length;default:if(o)return i?-1:F(t).length;n=(""+n).toLowerCase(),o=!0}}function h(t,e,n){var r=t[e];t[e]=t[n],t[n]=r}function d(t,n,r,i,o){if(0===t.length)return-1;if("string"==typeof r?(i=r,r=0):r>2147483647?r=2147483647:r<-2147483648&&(r=-2147483648),G(r=+r)&&(r=o?0:t.length-1),r<0&&(r=t.length+r),r>=t.length){if(o)return-1;r=t.length-1}else if(r<0){if(!o)return-1;r=0}if("string"==typeof n&&(n=e.from(n,i)),e.isBuffer(n))return 0===n.length?-1:f(t,n,r,i,o);if("number"==typeof n)return n&=255,"function"==typeof Uint8Array.prototype.indexOf?o?Uint8Array.prototype.indexOf.call(t,n,r):Uint8Array.prototype.lastIndexOf.call(t,n,r):f(t,[n],r,i,o);throw new TypeError("val must be string, number or Buffer")}function f(t,e,n,r,i){var o,a=1,s=t.length,A=e.length;if(void 0!==r&&("ucs2"===(r=String(r).toLowerCase())||"ucs-2"===r||"utf16le"===r||"utf-16le"===r)){if(t.length<2||e.length<2)return-1;a=2,s/=2,A/=2,n/=2}function l(t,e){return 1===a?t[e]:t.readUInt16BE(e*a)}if(i){var c=-1;for(o=n;o<s;o++)if(l(t,o)===l(e,-1===c?0:o-c)){if(-1===c&&(c=o),o-c+1===A)return c*a}else-1!==c&&(o-=o-c),c=-1}else for(n+A>s&&(n=s-A),o=n;o>=0;o--){for(var u=!0,g=0;g<A;g++)if(l(t,o+g)!==l(e,g)){u=!1;break}if(u)return o}return-1}function p(t,e,n,r){n=Number(n)||0;var i=t.length-n;r?(r=Number(r))>i&&(r=i):r=i;var o=e.length;r>o/2&&(r=o/2);for(var a=0;a<r;++a){var s=parseInt(e.substr(2*a,2),16);if(G(s))return a;t[n+a]=s}return a}function I(t,e,n,r){return U(F(e,t.length-n),t,n,r)}function C(t,e,n,r){return U(function(t){for(var e=[],n=0;n<t.length;++n)e.push(255&t.charCodeAt(n));return e}(e),t,n,r)}function m(t,e,n,r){return C(t,e,n,r)}function y(t,e,n,r){return U(T(e),t,n,r)}function E(t,e,n,r){return U(function(t,e){for(var n,r,i,o=[],a=0;a<t.length&&!((e-=2)<0);++a)n=t.charCodeAt(a),r=n>>8,i=n%256,o.push(i),o.push(r);return o}(e,t.length-n),t,n,r)}function v(t,e,n){return 0===e&&n===t.length?r.fromByteArray(t):r.fromByteArray(t.slice(e,n))}function B(t,e,n){n=Math.min(t.length,n);for(var r=[],i=e;i<n;){var o,a,s,A,l=t[i],c=null,u=l>239?4:l>223?3:l>191?2:1;if(i+u<=n)switch(u){case 1:l<128&&(c=l);break;case 2:128==(192&(o=t[i+1]))&&(A=(31&l)<<6|63&o)>127&&(c=A);break;case 3:o=t[i+1],a=t[i+2],128==(192&o)&&128==(192&a)&&(A=(15&l)<<12|(63&o)<<6|63&a)>2047&&(A<55296||A>57343)&&(c=A);break;case 4:o=t[i+1],a=t[i+2],s=t[i+3],128==(192&o)&&128==(192&a)&&128==(192&s)&&(A=(15&l)<<18|(63&o)<<12|(63&a)<<6|63&s)>65535&&A<1114112&&(c=A)}null===c?(c=65533,u=1):c>65535&&(c-=65536,r.push(c>>>10&1023|55296),c=56320|1023&c),r.push(c),i+=u}return function(t){var e=t.length;if(e<=b)return String.fromCharCode.apply(String,t);for(var n="",r=0;r<e;)n+=String.fromCharCode.apply(String,t.slice(r,r+=b));return n}(r)}n.kMaxLength=o,e.TYPED_ARRAY_SUPPORT=function(){try{var t=new Uint8Array(1);return t.__proto__={__proto__:Uint8Array.prototype,foo:function(){return 42}},42===t.foo()}catch(t){return!1}}(),e.TYPED_ARRAY_SUPPORT||"undefined"==typeof console||"function"!=typeof console.error||console.error("This browser lacks typed array (Uint8Array) support which is required by `buffer` v5.x. Use `buffer` v4.x if you require old browser support."),Object.defineProperty(e.prototype,"parent",{enumerable:!0,get:function(){if(e.isBuffer(this))return this.buffer}}),Object.defineProperty(e.prototype,"offset",{enumerable:!0,get:function(){if(e.isBuffer(this))return this.byteOffset}}),"undefined"!=typeof Symbol&&null!=Symbol.species&&e[Symbol.species]===e&&Object.defineProperty(e,Symbol.species,{value:null,configurable:!0,enumerable:!1,writable:!1}),e.poolSize=8192,e.from=function(t,e,n){return s(t,e,n)},e.prototype.__proto__=Uint8Array.prototype,e.__proto__=Uint8Array,e.alloc=function(t,e,n){return function(t,e,n){return A(t),t<=0?a(t):void 0!==e?"string"==typeof n?a(t).fill(e,n):a(t).fill(e):a(t)}(t,e,n)},e.allocUnsafe=function(t){return l(t)},e.allocUnsafeSlow=function(t){return l(t)},e.isBuffer=function(t){return null!=t&&!0===t._isBuffer&&t!==e.prototype},e.compare=function(t,n){if(O(t,Uint8Array)&&(t=e.from(t,t.offset,t.byteLength)),O(n,Uint8Array)&&(n=e.from(n,n.offset,n.byteLength)),!e.isBuffer(t)||!e.isBuffer(n))throw new TypeError('The "buf1", "buf2" arguments must be one of type Buffer or Uint8Array');if(t===n)return 0;for(var r=t.length,i=n.length,o=0,a=Math.min(r,i);o<a;++o)if(t[o]!==n[o]){r=t[o],i=n[o];break}return r<i?-1:i<r?1:0},e.isEncoding=function(t){switch(String(t).toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"latin1":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return!0;default:return!1}},e.concat=function(t,n){if(!Array.isArray(t))throw new TypeError('"list" argument must be an Array of Buffers');if(0===t.length)return e.alloc(0);var r;if(void 0===n)for(n=0,r=0;r<t.length;++r)n+=t[r].length;var i=e.allocUnsafe(n),o=0;for(r=0;r<t.length;++r){var a=t[r];if(O(a,Uint8Array)&&(a=e.from(a)),!e.isBuffer(a))throw new TypeError('"list" argument must be an Array of Buffers');a.copy(i,o),o+=a.length}return i},e.byteLength=g,e.prototype._isBuffer=!0,e.prototype.swap16=function(){var t=this.length;if(t%2!=0)throw new RangeError("Buffer size must be a multiple of 16-bits");for(var e=0;e<t;e+=2)h(this,e,e+1);return this},e.prototype.swap32=function(){var t=this.length;if(t%4!=0)throw new RangeError("Buffer size must be a multiple of 32-bits");for(var e=0;e<t;e+=4)h(this,e,e+3),h(this,e+1,e+2);return this},e.prototype.swap64=function(){var t=this.length;if(t%8!=0)throw new RangeError("Buffer size must be a multiple of 64-bits");for(var e=0;e<t;e+=8)h(this,e,e+7),h(this,e+1,e+6),h(this,e+2,e+5),h(this,e+3,e+4);return this},e.prototype.toString=function(){var t=this.length;return 0===t?"":0===arguments.length?B(this,0,t):function(t,e,n){var r=!1;if((void 0===e||e<0)&&(e=0),e>this.length)return"";if((void 0===n||n>this.length)&&(n=this.length),n<=0)return"";if((n>>>=0)<=(e>>>=0))return"";for(t||(t="utf8");;)switch(t){case"hex":return x(this,e,n);case"utf8":case"utf-8":return B(this,e,n);case"ascii":return Q(this,e,n);case"latin1":case"binary":return w(this,e,n);case"base64":return v(this,e,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return _(this,e,n);default:if(r)throw new TypeError("Unknown encoding: "+t);t=(t+"").toLowerCase(),r=!0}}.apply(this,arguments)},e.prototype.toLocaleString=e.prototype.toString,e.prototype.equals=function(t){if(!e.isBuffer(t))throw new TypeError("Argument must be a Buffer");return this===t||0===e.compare(this,t)},e.prototype.inspect=function(){var t="",e=n.INSPECT_MAX_BYTES;return t=this.toString("hex",0,e).replace(/(.{2})/g,"$1 ").trim(),this.length>e&&(t+=" ... "),"<Buffer "+t+">"},e.prototype.compare=function(t,n,r,i,o){if(O(t,Uint8Array)&&(t=e.from(t,t.offset,t.byteLength)),!e.isBuffer(t))throw new TypeError('The "target" argument must be one of type Buffer or Uint8Array. Received type '+typeof t);if(void 0===n&&(n=0),void 0===r&&(r=t?t.length:0),void 0===i&&(i=0),void 0===o&&(o=this.length),n<0||r>t.length||i<0||o>this.length)throw new RangeError("out of range index");if(i>=o&&n>=r)return 0;if(i>=o)return-1;if(n>=r)return 1;if(this===t)return 0;for(var a=(o>>>=0)-(i>>>=0),s=(r>>>=0)-(n>>>=0),A=Math.min(a,s),l=this.slice(i,o),c=t.slice(n,r),u=0;u<A;++u)if(l[u]!==c[u]){a=l[u],s=c[u];break}return a<s?-1:s<a?1:0},e.prototype.includes=function(t,e,n){return-1!==this.indexOf(t,e,n)},e.prototype.indexOf=function(t,e,n){return d(this,t,e,n,!0)},e.prototype.lastIndexOf=function(t,e,n){return d(this,t,e,n,!1)},e.prototype.write=function(t,e,n,r){if(void 0===e)r="utf8",n=this.length,e=0;else if(void 0===n&&"string"==typeof e)r=e,n=this.length,e=0;else{if(!isFinite(e))throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");e>>>=0,isFinite(n)?(n>>>=0,void 0===r&&(r="utf8")):(r=n,n=void 0)}var i=this.length-e;if((void 0===n||n>i)&&(n=i),t.length>0&&(n<0||e<0)||e>this.length)throw new RangeError("Attempt to write outside buffer bounds");r||(r="utf8");for(var o=!1;;)switch(r){case"hex":return p(this,t,e,n);case"utf8":case"utf-8":return I(this,t,e,n);case"ascii":return C(this,t,e,n);case"latin1":case"binary":return m(this,t,e,n);case"base64":return y(this,t,e,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return E(this,t,e,n);default:if(o)throw new TypeError("Unknown encoding: "+r);r=(""+r).toLowerCase(),o=!0}},e.prototype.toJSON=function(){return{type:"Buffer",data:Array.prototype.slice.call(this._arr||this,0)}};var b=4096;function Q(t,e,n){var r="";n=Math.min(t.length,n);for(var i=e;i<n;++i)r+=String.fromCharCode(127&t[i]);return r}function w(t,e,n){var r="";n=Math.min(t.length,n);for(var i=e;i<n;++i)r+=String.fromCharCode(t[i]);return r}function x(t,e,n){var r=t.length;(!e||e<0)&&(e=0),(!n||n<0||n>r)&&(n=r);for(var i="",o=e;o<n;++o)i+=N(t[o]);return i}function _(t,e,n){for(var r=t.slice(e,n),i="",o=0;o<r.length;o+=2)i+=String.fromCharCode(r[o]+256*r[o+1]);return i}function D(t,e,n){if(t%1!=0||t<0)throw new RangeError("offset is not uint");if(t+e>n)throw new RangeError("Trying to access beyond buffer length")}function M(t,n,r,i,o,a){if(!e.isBuffer(t))throw new TypeError('"buffer" argument must be a Buffer instance');if(n>o||n<a)throw new RangeError('"value" argument is out of bounds');if(r+i>t.length)throw new RangeError("Index out of range")}function k(t,e,n,r,i,o){if(n+r>t.length)throw new RangeError("Index out of range");if(n<0)throw new RangeError("Index out of range")}function S(t,e,n,r,o){return e=+e,n>>>=0,o||k(t,0,n,4),i.write(t,e,n,r,23,4),n+4}function L(t,e,n,r,o){return e=+e,n>>>=0,o||k(t,0,n,8),i.write(t,e,n,r,52,8),n+8}e.prototype.slice=function(t,n){var r=this.length;(t=~~t)<0?(t+=r)<0&&(t=0):t>r&&(t=r),(n=void 0===n?r:~~n)<0?(n+=r)<0&&(n=0):n>r&&(n=r),n<t&&(n=t);var i=this.subarray(t,n);return i.__proto__=e.prototype,i},e.prototype.readUIntLE=function(t,e,n){t>>>=0,e>>>=0,n||D(t,e,this.length);for(var r=this[t],i=1,o=0;++o<e&&(i*=256);)r+=this[t+o]*i;return r},e.prototype.readUIntBE=function(t,e,n){t>>>=0,e>>>=0,n||D(t,e,this.length);for(var r=this[t+--e],i=1;e>0&&(i*=256);)r+=this[t+--e]*i;return r},e.prototype.readUInt8=function(t,e){return t>>>=0,e||D(t,1,this.length),this[t]},e.prototype.readUInt16LE=function(t,e){return t>>>=0,e||D(t,2,this.length),this[t]|this[t+1]<<8},e.prototype.readUInt16BE=function(t,e){return t>>>=0,e||D(t,2,this.length),this[t]<<8|this[t+1]},e.prototype.readUInt32LE=function(t,e){return t>>>=0,e||D(t,4,this.length),(this[t]|this[t+1]<<8|this[t+2]<<16)+16777216*this[t+3]},e.prototype.readUInt32BE=function(t,e){return t>>>=0,e||D(t,4,this.length),16777216*this[t]+(this[t+1]<<16|this[t+2]<<8|this[t+3])},e.prototype.readIntLE=function(t,e,n){t>>>=0,e>>>=0,n||D(t,e,this.length);for(var r=this[t],i=1,o=0;++o<e&&(i*=256);)r+=this[t+o]*i;return r>=(i*=128)&&(r-=Math.pow(2,8*e)),r},e.prototype.readIntBE=function(t,e,n){t>>>=0,e>>>=0,n||D(t,e,this.length);for(var r=e,i=1,o=this[t+--r];r>0&&(i*=256);)o+=this[t+--r]*i;return o>=(i*=128)&&(o-=Math.pow(2,8*e)),o},e.prototype.readInt8=function(t,e){return t>>>=0,e||D(t,1,this.length),128&this[t]?-1*(255-this[t]+1):this[t]},e.prototype.readInt16LE=function(t,e){t>>>=0,e||D(t,2,this.length);var n=this[t]|this[t+1]<<8;return 32768&n?4294901760|n:n},e.prototype.readInt16BE=function(t,e){t>>>=0,e||D(t,2,this.length);var n=this[t+1]|this[t]<<8;return 32768&n?4294901760|n:n},e.prototype.readInt32LE=function(t,e){return t>>>=0,e||D(t,4,this.length),this[t]|this[t+1]<<8|this[t+2]<<16|this[t+3]<<24},e.prototype.readInt32BE=function(t,e){return t>>>=0,e||D(t,4,this.length),this[t]<<24|this[t+1]<<16|this[t+2]<<8|this[t+3]},e.prototype.readFloatLE=function(t,e){return t>>>=0,e||D(t,4,this.length),i.read(this,t,!0,23,4)},e.prototype.readFloatBE=function(t,e){return t>>>=0,e||D(t,4,this.length),i.read(this,t,!1,23,4)},e.prototype.readDoubleLE=function(t,e){return t>>>=0,e||D(t,8,this.length),i.read(this,t,!0,52,8)},e.prototype.readDoubleBE=function(t,e){return t>>>=0,e||D(t,8,this.length),i.read(this,t,!1,52,8)},e.prototype.writeUIntLE=function(t,e,n,r){t=+t,e>>>=0,n>>>=0,r||M(this,t,e,n,Math.pow(2,8*n)-1,0);var i=1,o=0;for(this[e]=255&t;++o<n&&(i*=256);)this[e+o]=t/i&255;return e+n},e.prototype.writeUIntBE=function(t,e,n,r){t=+t,e>>>=0,n>>>=0,r||M(this,t,e,n,Math.pow(2,8*n)-1,0);var i=n-1,o=1;for(this[e+i]=255&t;--i>=0&&(o*=256);)this[e+i]=t/o&255;return e+n},e.prototype.writeUInt8=function(t,e,n){return t=+t,e>>>=0,n||M(this,t,e,1,255,0),this[e]=255&t,e+1},e.prototype.writeUInt16LE=function(t,e,n){return t=+t,e>>>=0,n||M(this,t,e,2,65535,0),this[e]=255&t,this[e+1]=t>>>8,e+2},e.prototype.writeUInt16BE=function(t,e,n){return t=+t,e>>>=0,n||M(this,t,e,2,65535,0),this[e]=t>>>8,this[e+1]=255&t,e+2},e.prototype.writeUInt32LE=function(t,e,n){return t=+t,e>>>=0,n||M(this,t,e,4,4294967295,0),this[e+3]=t>>>24,this[e+2]=t>>>16,this[e+1]=t>>>8,this[e]=255&t,e+4},e.prototype.writeUInt32BE=function(t,e,n){return t=+t,e>>>=0,n||M(this,t,e,4,4294967295,0),this[e]=t>>>24,this[e+1]=t>>>16,this[e+2]=t>>>8,this[e+3]=255&t,e+4},e.prototype.writeIntLE=function(t,e,n,r){if(t=+t,e>>>=0,!r){var i=Math.pow(2,8*n-1);M(this,t,e,n,i-1,-i)}var o=0,a=1,s=0;for(this[e]=255&t;++o<n&&(a*=256);)t<0&&0===s&&0!==this[e+o-1]&&(s=1),this[e+o]=(t/a>>0)-s&255;return e+n},e.prototype.writeIntBE=function(t,e,n,r){if(t=+t,e>>>=0,!r){var i=Math.pow(2,8*n-1);M(this,t,e,n,i-1,-i)}var o=n-1,a=1,s=0;for(this[e+o]=255&t;--o>=0&&(a*=256);)t<0&&0===s&&0!==this[e+o+1]&&(s=1),this[e+o]=(t/a>>0)-s&255;return e+n},e.prototype.writeInt8=function(t,e,n){return t=+t,e>>>=0,n||M(this,t,e,1,127,-128),t<0&&(t=255+t+1),this[e]=255&t,e+1},e.prototype.writeInt16LE=function(t,e,n){return t=+t,e>>>=0,n||M(this,t,e,2,32767,-32768),this[e]=255&t,this[e+1]=t>>>8,e+2},e.prototype.writeInt16BE=function(t,e,n){return t=+t,e>>>=0,n||M(this,t,e,2,32767,-32768),this[e]=t>>>8,this[e+1]=255&t,e+2},e.prototype.writeInt32LE=function(t,e,n){return t=+t,e>>>=0,n||M(this,t,e,4,2147483647,-2147483648),this[e]=255&t,this[e+1]=t>>>8,this[e+2]=t>>>16,this[e+3]=t>>>24,e+4},e.prototype.writeInt32BE=function(t,e,n){return t=+t,e>>>=0,n||M(this,t,e,4,2147483647,-2147483648),t<0&&(t=4294967295+t+1),this[e]=t>>>24,this[e+1]=t>>>16,this[e+2]=t>>>8,this[e+3]=255&t,e+4},e.prototype.writeFloatLE=function(t,e,n){return S(this,t,e,!0,n)},e.prototype.writeFloatBE=function(t,e,n){return S(this,t,e,!1,n)},e.prototype.writeDoubleLE=function(t,e,n){return L(this,t,e,!0,n)},e.prototype.writeDoubleBE=function(t,e,n){return L(this,t,e,!1,n)},e.prototype.copy=function(t,n,r,i){if(!e.isBuffer(t))throw new TypeError("argument should be a Buffer");if(r||(r=0),i||0===i||(i=this.length),n>=t.length&&(n=t.length),n||(n=0),i>0&&i<r&&(i=r),i===r)return 0;if(0===t.length||0===this.length)return 0;if(n<0)throw new RangeError("targetStart out of bounds");if(r<0||r>=this.length)throw new RangeError("Index out of range");if(i<0)throw new RangeError("sourceEnd out of bounds");i>this.length&&(i=this.length),t.length-n<i-r&&(i=t.length-n+r);var o=i-r;if(this===t&&"function"==typeof Uint8Array.prototype.copyWithin)this.copyWithin(n,r,i);else if(this===t&&r<n&&n<i)for(var a=o-1;a>=0;--a)t[a+n]=this[a+r];else Uint8Array.prototype.set.call(t,this.subarray(r,i),n);return o},e.prototype.fill=function(t,n,r,i){if("string"==typeof t){if("string"==typeof n?(i=n,n=0,r=this.length):"string"==typeof r&&(i=r,r=this.length),void 0!==i&&"string"!=typeof i)throw new TypeError("encoding must be a string");if("string"==typeof i&&!e.isEncoding(i))throw new TypeError("Unknown encoding: "+i);if(1===t.length){var o=t.charCodeAt(0);("utf8"===i&&o<128||"latin1"===i)&&(t=o)}}else"number"==typeof t&&(t&=255);if(n<0||this.length<n||this.length<r)throw new RangeError("Out of range index");if(r<=n)return this;var a;if(n>>>=0,r=void 0===r?this.length:r>>>0,t||(t=0),"number"==typeof t)for(a=n;a<r;++a)this[a]=t;else{var s=e.isBuffer(t)?t:e.from(t,i),A=s.length;if(0===A)throw new TypeError('The value "'+t+'" is invalid for argument "value"');for(a=0;a<r-n;++a)this[a+n]=s[a%A]}return this};var R=/[^+\/0-9A-Za-z-_]/g;function N(t){return t<16?"0"+t.toString(16):t.toString(16)}function F(t,e){var n;e=e||1/0;for(var r=t.length,i=null,o=[],a=0;a<r;++a){if((n=t.charCodeAt(a))>55295&&n<57344){if(!i){if(n>56319){(e-=3)>-1&&o.push(239,191,189);continue}if(a+1===r){(e-=3)>-1&&o.push(239,191,189);continue}i=n;continue}if(n<56320){(e-=3)>-1&&o.push(239,191,189),i=n;continue}n=65536+(i-55296<<10|n-56320)}else i&&(e-=3)>-1&&o.push(239,191,189);if(i=null,n<128){if((e-=1)<0)break;o.push(n)}else if(n<2048){if((e-=2)<0)break;o.push(n>>6|192,63&n|128)}else if(n<65536){if((e-=3)<0)break;o.push(n>>12|224,n>>6&63|128,63&n|128)}else{if(!(n<1114112))throw new Error("Invalid code point");if((e-=4)<0)break;o.push(n>>18|240,n>>12&63|128,n>>6&63|128,63&n|128)}}return o}function T(t){return r.toByteArray(function(t){if((t=(t=t.split("=")[0]).trim().replace(R,"")).length<2)return"";for(;t.length%4!=0;)t+="=";return t}(t))}function U(t,e,n,r){for(var i=0;i<r&&!(i+n>=e.length||i>=t.length);++i)e[i+n]=t[i];return i}function O(t,e){return t instanceof e||null!=t&&null!=t.constructor&&null!=t.constructor.name&&t.constructor.name===e.name}function G(t){return t!=t}}).call(this,t("buffer").Buffer)},{"base64-js":74,buffer:105,ieee754:412}],106:[function(t,e,n){"use strict";var r=t("./lib/monotone"),i=t("./lib/triangulation"),o=t("./lib/delaunay"),a=t("./lib/filter");function s(t){return[Math.min(t[0],t[1]),Math.max(t[0],t[1])]}function A(t,e){return t[0]-e[0]||t[1]-e[1]}function l(t,e,n){return e in t?t[e]:n}e.exports=function(t,e,n){Array.isArray(e)?(n=n||{},e=e||[]):(n=e||{},e=[]);var c=!!l(n,"delaunay",!0),u=!!l(n,"interior",!0),g=!!l(n,"exterior",!0),h=!!l(n,"infinity",!1);if(!u&&!g||0===t.length)return[];var d=r(t,e);if(c||u!==g||h){for(var f=i(t.length,function(t){return t.map(s).sort(A)}(e)),p=0;p<d.length;++p){var I=d[p];f.addTriangle(I[0],I[1],I[2])}return c&&o(t,f),g?u?h?a(f,0,h):f.cells():a(f,1,h):a(f,-1)}return d}},{"./lib/delaunay":107,"./lib/filter":108,"./lib/monotone":109,"./lib/triangulation":110}],107:[function(t,e,n){"use strict";var r=t("robust-in-sphere")[4];function i(t,e,n,i,o,a){var s=e.opposite(i,o);if(!(s<0)){if(o<i){var A=i;i=o,o=A,A=a,a=s,s=A}e.isConstraint(i,o)||r(t[i],t[o],t[a],t[s])<0&&n.push(i,o)}}t("binary-search-bounds"),e.exports=function(t,e){for(var n=[],o=t.length,a=e.stars,s=0;s<o;++s)for(var A=a[s],l=1;l<A.length;l+=2){var c=A[l];if(!(c<s||e.isConstraint(s,c))){for(var u=A[l-1],g=-1,h=1;h<A.length;h+=2)if(A[h-1]===c){g=A[h];break}g<0||r(t[s],t[c],t[u],t[g])<0&&n.push(s,c)}}for(;n.length>0;){for(var c=n.pop(),s=n.pop(),u=-1,g=-1,A=a[s],d=1;d<A.length;d+=2){var f=A[d-1],p=A[d];f===c?g=p:p===c&&(u=f)}u<0||g<0||r(t[s],t[c],t[u],t[g])>=0||(e.flip(s,c),i(t,e,n,u,s,g),i(t,e,n,s,g,u),i(t,e,n,g,c,u),i(t,e,n,c,u,g))}}},{"binary-search-bounds":111,"robust-in-sphere":508}],108:[function(t,e,n){"use strict";var r,i=t("binary-search-bounds");function o(t,e,n,r,i,o,a){this.cells=t,this.neighbor=e,this.flags=r,this.constraint=n,this.active=i,this.next=o,this.boundary=a}function a(t,e){return t[0]-e[0]||t[1]-e[1]||t[2]-e[2]}e.exports=function(t,e,n){var r=function(t,e){for(var n=t.cells(),r=n.length,i=0;i<r;++i){var s=n[i],A=s[0],l=s[1],c=s[2];l<c?l<A&&(s[0]=l,s[1]=c,s[2]=A):c<A&&(s[0]=c,s[1]=A,s[2]=l)}n.sort(a);for(var u=new Array(r),i=0;i<u.length;++i)u[i]=0;var g=[],h=[],d=new Array(3*r),f=new Array(3*r),p=null;e&&(p=[]);for(var I=new o(n,d,f,u,g,h,p),i=0;i<r;++i)for(var s=n[i],C=0;C<3;++C){var A=s[C],l=s[(C+1)%3],m=d[3*i+C]=I.locate(l,A,t.opposite(l,A)),y=f[3*i+C]=t.isConstraint(A,l);m<0&&(y?h.push(i):(g.push(i),u[i]=1),e&&p.push([l,A,-1]))}return I}(t,n);if(0===e)return n?r.cells.concat(r.boundary):r.cells;for(var i=1,s=r.active,A=r.next,l=r.flags,c=r.cells,u=r.constraint,g=r.neighbor;s.length>0||A.length>0;){for(;s.length>0;){var h=s.pop();if(l[h]!==-i){l[h]=i,c[h];for(var d=0;d<3;++d){var f=g[3*h+d];f>=0&&0===l[f]&&(u[3*h+d]?A.push(f):(s.push(f),l[f]=i))}}}var p=A;A=s,s=p,A.length=0,i=-i}var I=function(t,e,n){for(var r=0,i=0;i<t.length;++i)e[i]===n&&(t[r++]=t[i]);return t.length=r,t}(c,l,e);return n?I.concat(r.boundary):I},o.prototype.locate=(r=[0,0,0],function(t,e,n){var o=t,s=e,A=n;return e<n?e<t&&(o=e,s=n,A=t):n<t&&(o=n,s=t,A=e),o<0?-1:(r[0]=o,r[1]=s,r[2]=A,i.eq(this.cells,r,a))})},{"binary-search-bounds":111}],109:[function(t,e,n){"use strict";var r=t("binary-search-bounds"),i=t("robust-orientation")[3],o=0;function a(t,e,n,r,i){this.a=t,this.b=e,this.idx=n,this.lowerIds=r,this.upperIds=i}function s(t,e,n,r){this.a=t,this.b=e,this.type=n,this.idx=r}function A(t,e){var n=t.a[0]-e.a[0]||t.a[1]-e.a[1]||t.type-e.type;return n||(t.type!==o&&(n=i(t.a,t.b,e.b))?n:t.idx-e.idx)}function l(t,e){return i(t.a,t.b,e)}function c(t,e,n,o,a){for(var s=r.lt(e,o,l),A=r.gt(e,o,l),c=s;c<A;++c){for(var u=e[c],g=u.lowerIds,h=g.length;h>1&&i(n[g[h-2]],n[g[h-1]],o)>0;)t.push([g[h-1],g[h-2],a]),h-=1;g.length=h,g.push(a);var d=u.upperIds;for(h=d.length;h>1&&i(n[d[h-2]],n[d[h-1]],o)<0;)t.push([d[h-2],d[h-1],a]),h-=1;d.length=h,d.push(a)}}function u(t,e){var n;return(n=t.a[0]<e.a[0]?i(t.a,t.b,e.a):i(e.b,e.a,t.a))?n:(n=e.b[0]<t.b[0]?i(t.a,t.b,e.b):i(e.b,e.a,t.b))||t.idx-e.idx}function g(t,e,n){var i=r.le(t,n,u),o=t[i],s=o.upperIds,A=s[s.length-1];o.upperIds=[A],t.splice(i+1,0,new a(n.a,n.b,n.idx,[A],s))}function h(t,e,n){var i=n.a;n.a=n.b,n.b=i;var o=r.eq(t,n,u),a=t[o];t[o-1].upperIds=a.upperIds,t.splice(o,1)}e.exports=function(t,e){for(var n=t.length,r=e.length,i=[],l=0;l<n;++l)i.push(new s(t[l],null,o,l));for(var l=0;l<r;++l){var u=e[l],d=t[u[0]],f=t[u[1]];d[0]<f[0]?i.push(new s(d,f,2,l),new s(f,d,1,l)):d[0]>f[0]&&i.push(new s(f,d,2,l),new s(d,f,1,l))}i.sort(A);for(var p=i[0].a[0]-(1+Math.abs(i[0].a[0]))*Math.pow(2,-52),I=[new a([p,1],[p,0],-1,[],[],[],[])],C=[],l=0,m=i.length;l<m;++l){var y=i[l],E=y.type;E===o?c(C,I,t,y.a,y.idx):2===E?g(I,0,y):h(I,0,y)}return C}},{"binary-search-bounds":111,"robust-orientation":510}],110:[function(t,e,n){"use strict";var r=t("binary-search-bounds");function i(t,e){this.stars=t,this.edges=e}e.exports=function(t,e){for(var n=new Array(t),r=0;r<t;++r)n[r]=[];return new i(n,e)};var o=i.prototype;function a(t,e,n){for(var r=1,i=t.length;r<i;r+=2)if(t[r-1]===e&&t[r]===n)return t[r-1]=t[i-2],t[r]=t[i-1],void(t.length=i-2)}o.isConstraint=function(){var t=[0,0];function e(t,e){return t[0]-e[0]||t[1]-e[1]}return function(n,i){return t[0]=Math.min(n,i),t[1]=Math.max(n,i),r.eq(this.edges,t,e)>=0}}(),o.removeTriangle=function(t,e,n){var r=this.stars;a(r[t],e,n),a(r[e],n,t),a(r[n],t,e)},o.addTriangle=function(t,e,n){var r=this.stars;r[t].push(e,n),r[e].push(n,t),r[n].push(t,e)},o.opposite=function(t,e){for(var n=this.stars[e],r=1,i=n.length;r<i;r+=2)if(n[r]===t)return n[r-1];return-1},o.flip=function(t,e){var n=this.opposite(t,e),r=this.opposite(e,t);this.removeTriangle(t,e,n),this.removeTriangle(e,t,r),this.addTriangle(t,r,n),this.addTriangle(e,n,r)},o.edges=function(){for(var t=this.stars,e=[],n=0,r=t.length;n<r;++n)for(var i=t[n],o=0,a=i.length;o<a;o+=2)e.push([i[o],i[o+1]]);return e},o.cells=function(){for(var t=this.stars,e=[],n=0,r=t.length;n<r;++n)for(var i=t[n],o=0,a=i.length;o<a;o+=2){var s=i[o],A=i[o+1];n<Math.min(s,A)&&e.push([n,s,A])}return e}},{"binary-search-bounds":111}],111:[function(t,e,n){"use strict";function r(t,e,n,r,i){var o=["function ",t,"(a,l,h,",r.join(","),"){",i?"":"var i=",n?"l-1":"h+1",";while(l<=h){var m=(l+h)>>>1,x=a[m]"];return i?e.indexOf("c")<0?o.push(";if(x===y){return m}else if(x<=y){"):o.push(";var p=c(x,y);if(p===0){return m}else if(p<=0){"):o.push(";if(",e,"){i=m;"),n?o.push("l=m+1}else{h=m-1}"):o.push("h=m-1}else{l=m+1}"),o.push("}"),i?o.push("return -1};"):o.push("return i};"),o.join("")}function i(t,e,n,i){return new Function([r("A","x"+t+"y",e,["y"],i),r("P","c(x,y)"+t+"0",e,["y","c"],i),"function dispatchBsearch",n,"(a,y,c,l,h){if(typeof(c)==='function'){return P(a,(l===void 0)?0:l|0,(h===void 0)?a.length-1:h|0,y,c)}else{return A(a,(c===void 0)?0:c|0,(l===void 0)?a.length-1:l|0,y)}}return dispatchBsearch",n].join(""))()}e.exports={ge:i(">=",!1,"GE"),gt:i(">",!1,"GT"),lt:i("<",!0,"LT"),le:i("<=",!0,"LE"),eq:i("-",!0,"EQ",!0)}},{}],112:[function(t,e,n){"use strict";e.exports=function(t){for(var e=1,n=1;n<t.length;++n)for(var r=0;r<n;++r)if(t[n]<t[r])e=-e;else if(t[r]===t[n])return 0;return e}},{}],113:[function(t,e,n){"use strict";var r=t("dup"),i=t("robust-linear-solve");function o(t,e){for(var n=0,r=t.length,i=0;i<r;++i)n+=t[i]*e[i];return n}function a(t){var e=t.length;if(0===e)return[];t[0].length;var n=r([t.length+1,t.length+1],1),a=r([t.length+1],1);n[e][e]=0;for(var s=0;s<e;++s){for(var A=0;A<=s;++A)n[A][s]=n[s][A]=2*o(t[s],t[A]);a[s]=o(t[s],t[s])}var l=i(n,a),c=0,u=l[e+1];for(s=0;s<u.length;++s)c+=u[s];var g=new Array(e);for(s=0;s<e;++s){u=l[s];var h=0;for(A=0;A<u.length;++A)h+=u[A];g[s]=h/c}return g}function s(t){if(0===t.length)return[];for(var e=t[0].length,n=r([e]),i=a(t),o=0;o<t.length;++o)for(var s=0;s<e;++s)n[s]+=t[o][s]*i[o];return n}s.barycenetric=a,e.exports=s},{dup:170,"robust-linear-solve":509}],114:[function(t,e,n){e.exports=function(t){for(var e=r(t),n=0,i=0;i<t.length;++i)for(var o=t[i],a=0;a<e.length;++a)n+=Math.pow(o[a]-e[a],2);return Math.sqrt(n/t.length)};var r=t("circumcenter")},{circumcenter:113}],115:[function(t,e,n){e.exports=function(t,e,n){return e<n?t<e?e:t>n?n:t:t<n?n:t>e?e:t}},{}],116:[function(t,e,n){"use strict";e.exports=function(t,e,n){var r;if(n){r=e;for(var i=new Array(e.length),o=0;o<e.length;++o){var a=e[o];i[o]=[a[0],a[1],n[o]]}e=i}for(var s=function(t,e,n){var r=d(t,[],h(t));return I(e,r,n),!!r}(t,e,!!n);C(t,e,!!n);)s=!0;if(n&&s){r.length=0,n.length=0;for(var o=0;o<e.length;++o){var a=e[o];r.push([a[0],a[1]]),n.push(a[2])}}return s};var r=t("union-find"),i=t("box-intersect"),o=t("robust-segment-intersect"),a=t("big-rat"),s=t("big-rat/cmp"),A=t("big-rat/to-float"),l=t("rat-vec"),c=t("nextafter"),u=t("./lib/rat-seg-intersect");function g(t){var e=A(t);return[c(e,-1/0),c(e,1/0)]}function h(t){for(var e=new Array(t.length),n=0;n<t.length;++n){var r=t[n];e[n]=[c(r[0],-1/0),c(r[1],-1/0),c(r[0],1/0),c(r[1],1/0)]}return e}function d(t,e,n){for(var o=e.length,a=new r(o),s=[],A=0;A<e.length;++A){var l=e[A],u=g(l[0]),h=g(l[1]);s.push([c(u[0],-1/0),c(h[0],-1/0),c(u[1],1/0),c(h[1],1/0)])}i(s,function(t,e){a.link(t,e)});var d=!0,f=new Array(o);for(A=0;A<o;++A)(I=a.find(A))!==A&&(d=!1,t[I]=[Math.min(t[A][0],t[I][0]),Math.min(t[A][1],t[I][1])]);if(d)return null;var p=0;for(A=0;A<o;++A){var I;(I=a.find(A))===A?(f[A]=p,t[p++]=t[A]):f[A]=-1}for(t.length=p,A=0;A<o;++A)f[A]<0&&(f[A]=f[a.find(A)]);return f}function f(t,e){return t[0]-e[0]||t[1]-e[1]}function p(t,e){var n=t[0]-e[0]||t[1]-e[1];return n||(t[2]<e[2]?-1:t[2]>e[2]?1:0)}function I(t,e,n){if(0!==t.length){if(e)for(var r=0;r<t.length;++r){var i=e[(a=t[r])[0]],o=e[a[1]];a[0]=Math.min(i,o),a[1]=Math.max(i,o)}else for(r=0;r<t.length;++r){var a;i=(a=t[r])[0],o=a[1],a[0]=Math.min(i,o),a[1]=Math.max(i,o)}n?t.sort(p):t.sort(f);var s=1;for(r=1;r<t.length;++r){var A=t[r-1],l=t[r];(l[0]!==A[0]||l[1]!==A[1]||n&&l[2]!==A[2])&&(t[s++]=l)}t.length=s}}function C(t,e,n){var r=function(t,e){for(var n=new Array(e.length),r=0;r<e.length;++r){var i=e[r],o=t[i[0]],a=t[i[1]];n[r]=[c(Math.min(o[0],a[0]),-1/0),c(Math.min(o[1],a[1]),-1/0),c(Math.max(o[0],a[0]),1/0),c(Math.max(o[1],a[1]),1/0)]}return n}(t,e),g=function(t,e,n){var r=[];return i(n,function(n,i){var a=e[n],s=e[i];if(a[0]!==s[0]&&a[0]!==s[1]&&a[1]!==s[0]&&a[1]!==s[1]){var A=t[a[0]],l=t[a[1]],c=t[s[0]],u=t[s[1]];o(A,l,c,u)&&r.push([n,i])}}),r}(t,e,r),f=h(t),p=function(t,e,n,r){var a=[];return i(n,r,function(n,r){var i=e[n];if(i[0]!==r&&i[1]!==r){var s=t[r],A=t[i[0]],l=t[i[1]];o(A,l,s,s)&&a.push([n,r])}}),a}(t,e,r,f),C=d(t,function(t,e,n,r,i){var o,c,g=t.map(function(t){return[a(t[0]),a(t[1])]});for(o=0;o<n.length;++o){var h=n[o];c=h[0];var d=h[1],f=e[c],p=e[d],I=u(l(t[f[0]]),l(t[f[1]]),l(t[p[0]]),l(t[p[1]]));if(I){var C=t.length;t.push([A(I[0]),A(I[1])]),g.push(I),r.push([c,C],[d,C])}}for(r.sort(function(t,e){if(t[0]!==e[0])return t[0]-e[0];var n=g[t[1]],r=g[e[1]];return s(n[0],r[0])||s(n[1],r[1])}),o=r.length-1;o>=0;--o){var m=e[c=(x=r[o])[0]],y=m[0],E=m[1],v=t[y],B=t[E];if((v[0]-B[0]||v[1]-B[1])<0){var b=y;y=E,E=b}m[0]=y;var Q,w=m[1]=x[1];for(i&&(Q=m[2]);o>0&&r[o-1][0]===c;){var x,_=(x=r[--o])[1];i?e.push([w,_,Q]):e.push([w,_]),w=_}i?e.push([w,E,Q]):e.push([w,E])}return g}(t,e,g,p,n));return I(e,C,n),!!C||g.length>0||p.length>0}},{"./lib/rat-seg-intersect":117,"big-rat":78,"big-rat/cmp":76,"big-rat/to-float":90,"box-intersect":96,nextafter:451,"rat-vec":486,"robust-segment-intersect":513,"union-find":546}],117:[function(t,e,n){"use strict";e.exports=function(t,e,n,r){var o=s(e,t),u=s(r,n),g=c(o,u);if(0===a(g))return null;var h=s(t,n),d=c(u,h),f=i(d,g),p=l(o,f);return A(t,p)};var r=t("big-rat/mul"),i=t("big-rat/div"),o=t("big-rat/sub"),a=t("big-rat/sign"),s=t("rat-vec/sub"),A=t("rat-vec/add"),l=t("rat-vec/muls");function c(t,e){return o(r(t[0],e[1]),r(t[1],e[0]))}},{"big-rat/div":77,"big-rat/mul":87,"big-rat/sign":88,"big-rat/sub":89,"rat-vec/add":485,"rat-vec/muls":487,"rat-vec/sub":488}],118:[function(t,e,n){"use strict";var r=t("clamp");function i(t,e){null==e&&(e=!0);var n=t[0],i=t[1],o=t[2],a=t[3];return null==a&&(a=e?1:255),e&&(n*=255,i*=255,o*=255,a*=255),16777216*(n=255&r(n,0,255))+((i=255&r(i,0,255))<<16)+((o=255&r(o,0,255))<<8)+(a=255&r(a,0,255))}e.exports=i,e.exports.to=i,e.exports.from=function(t,e){var n=(t=+t)>>>24,r=(16711680&t)>>>16,i=(65280&t)>>>8,o=255&t;return!1===e?[n,r,i,o]:[n/255,r/255,i/255,o/255]}},{clamp:115}],119:[function(t,e,n){"use strict";e.exports={aliceblue:[240,248,255],antiquewhite:[250,235,215],aqua:[0,255,255],aquamarine:[127,255,212],azure:[240,255,255],beige:[245,245,220],bisque:[255,228,196],black:[0,0,0],blanchedalmond:[255,235,205],blue:[0,0,255],blueviolet:[138,43,226],brown:[165,42,42],burlywood:[222,184,135],cadetblue:[95,158,160],chartreuse:[127,255,0],chocolate:[210,105,30],coral:[255,127,80],cornflowerblue:[100,149,237],cornsilk:[255,248,220],crimson:[220,20,60],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgoldenrod:[184,134,11],darkgray:[169,169,169],darkgreen:[0,100,0],darkgrey:[169,169,169],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkseagreen:[143,188,143],darkslateblue:[72,61,139],darkslategray:[47,79,79],darkslategrey:[47,79,79],darkturquoise:[0,206,209],darkviolet:[148,0,211],deeppink:[255,20,147],deepskyblue:[0,191,255],dimgray:[105,105,105],dimgrey:[105,105,105],dodgerblue:[30,144,255],firebrick:[178,34,34],floralwhite:[255,250,240],forestgreen:[34,139,34],fuchsia:[255,0,255],gainsboro:[220,220,220],ghostwhite:[248,248,255],gold:[255,215,0],goldenrod:[218,165,32],gray:[128,128,128],green:[0,128,0],greenyellow:[173,255,47],grey:[128,128,128],honeydew:[240,255,240],hotpink:[255,105,180],indianred:[205,92,92],indigo:[75,0,130],ivory:[255,255,240],khaki:[240,230,140],lavender:[230,230,250],lavenderblush:[255,240,245],lawngreen:[124,252,0],lemonchiffon:[255,250,205],lightblue:[173,216,230],lightcoral:[240,128,128],lightcyan:[224,255,255],lightgoldenrodyellow:[250,250,210],lightgray:[211,211,211],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightsalmon:[255,160,122],lightseagreen:[32,178,170],lightskyblue:[135,206,250],lightslategray:[119,136,153],lightslategrey:[119,136,153],lightsteelblue:[176,196,222],lightyellow:[255,255,224],lime:[0,255,0],limegreen:[50,205,50],linen:[250,240,230],magenta:[255,0,255],maroon:[128,0,0],mediumaquamarine:[102,205,170],mediumblue:[0,0,205],mediumorchid:[186,85,211],mediumpurple:[147,112,219],mediumseagreen:[60,179,113],mediumslateblue:[123,104,238],mediumspringgreen:[0,250,154],mediumturquoise:[72,209,204],mediumvioletred:[199,21,133],midnightblue:[25,25,112],mintcream:[245,255,250],mistyrose:[255,228,225],moccasin:[255,228,181],navajowhite:[255,222,173],navy:[0,0,128],oldlace:[253,245,230],olive:[128,128,0],olivedrab:[107,142,35],orange:[255,165,0],orangered:[255,69,0],orchid:[218,112,214],palegoldenrod:[238,232,170],palegreen:[152,251,152],paleturquoise:[175,238,238],palevioletred:[219,112,147],papayawhip:[255,239,213],peachpuff:[255,218,185],peru:[205,133,63],pink:[255,192,203],plum:[221,160,221],powderblue:[176,224,230],purple:[128,0,128],rebeccapurple:[102,51,153],red:[255,0,0],rosybrown:[188,143,143],royalblue:[65,105,225],saddlebrown:[139,69,19],salmon:[250,128,114],sandybrown:[244,164,96],seagreen:[46,139,87],seashell:[255,245,238],sienna:[160,82,45],silver:[192,192,192],skyblue:[135,206,235],slateblue:[106,90,205],slategray:[112,128,144],slategrey:[112,128,144],snow:[255,250,250],springgreen:[0,255,127],steelblue:[70,130,180],tan:[210,180,140],teal:[0,128,128],thistle:[216,191,216],tomato:[255,99,71],turquoise:[64,224,208],violet:[238,130,238],wheat:[245,222,179],white:[255,255,255],whitesmoke:[245,245,245],yellow:[255,255,0],yellowgreen:[154,205,50]}},{}],120:[function(t,e,n){"use strict";var r=t("color-rgba"),i=t("clamp"),o=t("dtype");e.exports=function(t,e){"float"!==e&&e||(e="array"),"uint"===e&&(e="uint8"),"uint_clamped"===e&&(e="uint8_clamped");var n=new(o(e))(4),a="uint8"!==e&&"uint8_clamped"!==e;return t.length&&"string"!=typeof t||((t=r(t))[0]/=255,t[1]/=255,t[2]/=255),function(t){return t instanceof Uint8Array||t instanceof Uint8ClampedArray||!!(Array.isArray(t)&&(t[0]>1||0===t[0])&&(t[1]>1||0===t[1])&&(t[2]>1||0===t[2])&&(!t[3]||t[3]>1))}(t)?(n[0]=t[0],n[1]=t[1],n[2]=t[2],n[3]=null!=t[3]?t[3]:255,a&&(n[0]/=255,n[1]/=255,n[2]/=255,n[3]/=255),n):(a?(n[0]=t[0],n[1]=t[1],n[2]=t[2],n[3]=null!=t[3]?t[3]:1):(n[0]=i(Math.floor(255*t[0]),0,255),n[1]=i(Math.floor(255*t[1]),0,255),n[2]=i(Math.floor(255*t[2]),0,255),n[3]=null==t[3]?255:i(Math.floor(255*t[3]),0,255)),n)}},{clamp:115,"color-rgba":122,dtype:169}],121:[function(t,n,r){(function(e){"use strict";var r=t("color-name"),i=t("is-plain-obj"),o=t("defined");n.exports=function(t){var n,s,A=[],l=1;if("string"==typeof t)if(r[t])A=r[t].slice(),s="rgb";else if("transparent"===t)l=0,s="rgb",A=[0,0,0];else if(/^#[A-Fa-f0-9]+$/.test(t)){var c=t.slice(1),u=c.length,g=u<=4;l=1,g?(A=[parseInt(c[0]+c[0],16),parseInt(c[1]+c[1],16),parseInt(c[2]+c[2],16)],4===u&&(l=parseInt(c[3]+c[3],16)/255)):(A=[parseInt(c[0]+c[1],16),parseInt(c[2]+c[3],16),parseInt(c[4]+c[5],16)],8===u&&(l=parseInt(c[6]+c[7],16)/255)),A[0]||(A[0]=0),A[1]||(A[1]=0),A[2]||(A[2]=0),s="rgb"}else if(n=/^((?:rgb|hs[lvb]|hwb|cmyk?|xy[zy]|gray|lab|lchu?v?|[ly]uv|lms)a?)\s*\(([^\)]*)\)/.exec(t)){var h=n[1],d="rgb"===h,c=h.replace(/a$/,"");s=c;var u="cmyk"===c?4:"gray"===c?1:3;A=n[2].trim().split(/\s*,\s*/).map(function(t,e){if(/%$/.test(t))return e===u?parseFloat(t)/100:"rgb"===c?255*parseFloat(t)/100:parseFloat(t);if("h"===c[e]){if(/deg$/.test(t))return parseFloat(t);if(void 0!==a[t])return a[t]}return parseFloat(t)}),h===c&&A.push(1),l=d?1:void 0===A[u]?1:A[u],A=A.slice(0,u)}else t.length>10&&/[0-9](?:\s|\/)/.test(t)&&(A=t.match(/([0-9]+)/g).map(function(t){return parseFloat(t)}),s=t.match(/([a-z])/gi).join("").toLowerCase());else if(isNaN(t))if(i(t)){var f=o(t.r,t.red,t.R,null);null!==f?(s="rgb",A=[f,o(t.g,t.green,t.G),o(t.b,t.blue,t.B)]):(s="hsl",A=[o(t.h,t.hue,t.H),o(t.s,t.saturation,t.S),o(t.l,t.lightness,t.L,t.b,t.brightness)]),l=o(t.a,t.alpha,t.opacity,1),null!=t.opacity&&(l/=100)}else(Array.isArray(t)||e.ArrayBuffer&&ArrayBuffer.isView&&ArrayBuffer.isView(t))&&(A=[t[0],t[1],t[2]],s="rgb",l=4===t.length?t[3]:1);else s="rgb",A=[t>>>16,(65280&t)>>>8,255&t];return{space:s,values:A,alpha:l}};var a={red:0,orange:60,yellow:120,green:180,blue:240,purple:300}}).call(this,void 0!==e?e:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{"color-name":119,defined:164,"is-plain-obj":422}],122:[function(t,e,n){"use strict";var r=t("color-parse"),i=t("color-space/hsl"),o=t("clamp");e.exports=function(t){var e,n=r(t);return n.space?((e=Array(3))[0]=o(n.values[0],0,255),e[1]=o(n.values[1],0,255),e[2]=o(n.values[2],0,255),"h"===n.space[0]&&(e=i.rgb(e)),e.push(o(n.alpha,0,1)),e):[]}},{clamp:115,"color-parse":121,"color-space/hsl":123}],123:[function(t,e,n){"use strict";var r=t("./rgb");e.exports={name:"hsl",min:[0,0,0],max:[360,100,100],channel:["hue","saturation","lightness"],alias:["HSL"],rgb:function(t){var e,n,r,i,o,a=t[0]/360,s=t[1]/100,A=t[2]/100;if(0===s)return[o=255*A,o,o];e=2*A-(n=A<.5?A*(1+s):A+s-A*s),i=[0,0,0];for(var l=0;l<3;l++)(r=a+1/3*-(l-1))<0?r++:r>1&&r--,o=6*r<1?e+6*(n-e)*r:2*r<1?n:3*r<2?e+(n-e)*(2/3-r)*6:e,i[l]=255*o;return i}},r.hsl=function(t){var e,n,r=t[0]/255,i=t[1]/255,o=t[2]/255,a=Math.min(r,i,o),s=Math.max(r,i,o),A=s-a;return s===a?e=0:r===s?e=(i-o)/A:i===s?e=2+(o-r)/A:o===s&&(e=4+(r-i)/A),(e=Math.min(60*e,360))<0&&(e+=360),n=(a+s)/2,[e,100*(s===a?0:n<=.5?A/(s+a):A/(2-s-a)),100*n]}},{"./rgb":124}],124:[function(t,e,n){"use strict";e.exports={name:"rgb",min:[0,0,0],max:[255,255,255],channel:["red","green","blue"],alias:["RGB"]}},{}],125:[function(t,e,n){e.exports={jet:[{index:0,rgb:[0,0,131]},{index:.125,rgb:[0,60,170]},{index:.375,rgb:[5,255,255]},{index:.625,rgb:[255,255,0]},{index:.875,rgb:[250,0,0]},{index:1,rgb:[128,0,0]}],hsv:[{index:0,rgb:[255,0,0]},{index:.169,rgb:[253,255,2]},{index:.173,rgb:[247,255,2]},{index:.337,rgb:[0,252,4]},{index:.341,rgb:[0,252,10]},{index:.506,rgb:[1,249,255]},{index:.671,rgb:[2,0,253]},{index:.675,rgb:[8,0,253]},{index:.839,rgb:[255,0,251]},{index:.843,rgb:[255,0,245]},{index:1,rgb:[255,0,6]}],hot:[{index:0,rgb:[0,0,0]},{index:.3,rgb:[230,0,0]},{index:.6,rgb:[255,210,0]},{index:1,rgb:[255,255,255]}],cool:[{index:0,rgb:[0,255,255]},{index:1,rgb:[255,0,255]}],spring:[{index:0,rgb:[255,0,255]},{index:1,rgb:[255,255,0]}],summer:[{index:0,rgb:[0,128,102]},{index:1,rgb:[255,255,102]}],autumn:[{index:0,rgb:[255,0,0]},{index:1,rgb:[255,255,0]}],winter:[{index:0,rgb:[0,0,255]},{index:1,rgb:[0,255,128]}],bone:[{index:0,rgb:[0,0,0]},{index:.376,rgb:[84,84,116]},{index:.753,rgb:[169,200,200]},{index:1,rgb:[255,255,255]}],copper:[{index:0,rgb:[0,0,0]},{index:.804,rgb:[255,160,102]},{index:1,rgb:[255,199,127]}],greys:[{index:0,rgb:[0,0,0]},{index:1,rgb:[255,255,255]}],yignbu:[{index:0,rgb:[8,29,88]},{index:.125,rgb:[37,52,148]},{index:.25,rgb:[34,94,168]},{index:.375,rgb:[29,145,192]},{index:.5,rgb:[65,182,196]},{index:.625,rgb:[127,205,187]},{index:.75,rgb:[199,233,180]},{index:.875,rgb:[237,248,217]},{index:1,rgb:[255,255,217]}],greens:[{index:0,rgb:[0,68,27]},{index:.125,rgb:[0,109,44]},{index:.25,rgb:[35,139,69]},{index:.375,rgb:[65,171,93]},{index:.5,rgb:[116,196,118]},{index:.625,rgb:[161,217,155]},{index:.75,rgb:[199,233,192]},{index:.875,rgb:[229,245,224]},{index:1,rgb:[247,252,245]}],yiorrd:[{index:0,rgb:[128,0,38]},{index:.125,rgb:[189,0,38]},{index:.25,rgb:[227,26,28]},{index:.375,rgb:[252,78,42]},{index:.5,rgb:[253,141,60]},{index:.625,rgb:[254,178,76]},{index:.75,rgb:[254,217,118]},{index:.875,rgb:[255,237,160]},{index:1,rgb:[255,255,204]}],bluered:[{index:0,rgb:[0,0,255]},{index:1,rgb:[255,0,0]}],rdbu:[{index:0,rgb:[5,10,172]},{index:.35,rgb:[106,137,247]},{index:.5,rgb:[190,190,190]},{index:.6,rgb:[220,170,132]},{index:.7,rgb:[230,145,90]},{index:1,rgb:[178,10,28]}],picnic:[{index:0,rgb:[0,0,255]},{index:.1,rgb:[51,153,255]},{index:.2,rgb:[102,204,255]},{index:.3,rgb:[153,204,255]},{index:.4,rgb:[204,204,255]},{index:.5,rgb:[255,255,255]},{index:.6,rgb:[255,204,255]},{index:.7,rgb:[255,153,255]},{index:.8,rgb:[255,102,204]},{index:.9,rgb:[255,102,102]},{index:1,rgb:[255,0,0]}],rainbow:[{index:0,rgb:[150,0,90]},{index:.125,rgb:[0,0,200]},{index:.25,rgb:[0,25,255]},{index:.375,rgb:[0,152,255]},{index:.5,rgb:[44,255,150]},{index:.625,rgb:[151,255,0]},{index:.75,rgb:[255,234,0]},{index:.875,rgb:[255,111,0]},{index:1,rgb:[255,0,0]}],portland:[{index:0,rgb:[12,51,131]},{index:.25,rgb:[10,136,186]},{index:.5,rgb:[242,211,56]},{index:.75,rgb:[242,143,56]},{index:1,rgb:[217,30,30]}],blackbody:[{index:0,rgb:[0,0,0]},{index:.2,rgb:[230,0,0]},{index:.4,rgb:[230,210,0]},{index:.7,rgb:[255,255,255]},{index:1,rgb:[160,200,255]}],earth:[{index:0,rgb:[0,0,130]},{index:.1,rgb:[0,180,180]},{index:.2,rgb:[40,210,40]},{index:.4,rgb:[230,230,50]},{index:.6,rgb:[120,70,20]},{index:1,rgb:[255,255,255]}],electric:[{index:0,rgb:[0,0,0]},{index:.15,rgb:[30,0,100]},{index:.4,rgb:[120,0,100]},{index:.6,rgb:[160,90,0]},{index:.8,rgb:[230,200,0]},{index:1,rgb:[255,250,220]}],alpha:[{index:0,rgb:[255,255,255,0]},{index:1,rgb:[255,255,255,1]}],viridis:[{index:0,rgb:[68,1,84]},{index:.13,rgb:[71,44,122]},{index:.25,rgb:[59,81,139]},{index:.38,rgb:[44,113,142]},{index:.5,rgb:[33,144,141]},{index:.63,rgb:[39,173,129]},{index:.75,rgb:[92,200,99]},{index:.88,rgb:[170,220,50]},{index:1,rgb:[253,231,37]}],inferno:[{index:0,rgb:[0,0,4]},{index:.13,rgb:[31,12,72]},{index:.25,rgb:[85,15,109]},{index:.38,rgb:[136,34,106]},{index:.5,rgb:[186,54,85]},{index:.63,rgb:[227,89,51]},{index:.75,rgb:[249,140,10]},{index:.88,rgb:[249,201,50]},{index:1,rgb:[252,255,164]}],magma:[{index:0,rgb:[0,0,4]},{index:.13,rgb:[28,16,68]},{index:.25,rgb:[79,18,123]},{index:.38,rgb:[129,37,129]},{index:.5,rgb:[181,54,122]},{index:.63,rgb:[229,80,100]},{index:.75,rgb:[251,135,97]},{index:.88,rgb:[254,194,135]},{index:1,rgb:[252,253,191]}],plasma:[{index:0,rgb:[13,8,135]},{index:.13,rgb:[75,3,161]},{index:.25,rgb:[125,3,168]},{index:.38,rgb:[168,34,150]},{index:.5,rgb:[203,70,121]},{index:.63,rgb:[229,107,93]},{index:.75,rgb:[248,148,65]},{index:.88,rgb:[253,195,40]},{index:1,rgb:[240,249,33]}],warm:[{index:0,rgb:[125,0,179]},{index:.13,rgb:[172,0,187]},{index:.25,rgb:[219,0,170]},{index:.38,rgb:[255,0,130]},{index:.5,rgb:[255,63,74]},{index:.63,rgb:[255,123,0]},{index:.75,rgb:[234,176,0]},{index:.88,rgb:[190,228,0]},{index:1,rgb:[147,255,0]}],cool:[{index:0,rgb:[125,0,179]},{index:.13,rgb:[116,0,218]},{index:.25,rgb:[98,74,237]},{index:.38,rgb:[68,146,231]},{index:.5,rgb:[0,204,197]},{index:.63,rgb:[0,247,146]},{index:.75,rgb:[0,255,88]},{index:.88,rgb:[40,255,8]},{index:1,rgb:[147,255,0]}],"rainbow-soft":[{index:0,rgb:[125,0,179]},{index:.1,rgb:[199,0,180]},{index:.2,rgb:[255,0,121]},{index:.3,rgb:[255,108,0]},{index:.4,rgb:[222,194,0]},{index:.5,rgb:[150,255,0]},{index:.6,rgb:[0,255,55]},{index:.7,rgb:[0,246,150]},{index:.8,rgb:[50,167,222]},{index:.9,rgb:[103,51,235]},{index:1,rgb:[124,0,186]}],bathymetry:[{index:0,rgb:[40,26,44]},{index:.13,rgb:[59,49,90]},{index:.25,rgb:[64,76,139]},{index:.38,rgb:[63,110,151]},{index:.5,rgb:[72,142,158]},{index:.63,rgb:[85,174,163]},{index:.75,rgb:[120,206,163]},{index:.88,rgb:[187,230,172]},{index:1,rgb:[253,254,204]}],cdom:[{index:0,rgb:[47,15,62]},{index:.13,rgb:[87,23,86]},{index:.25,rgb:[130,28,99]},{index:.38,rgb:[171,41,96]},{index:.5,rgb:[206,67,86]},{index:.63,rgb:[230,106,84]},{index:.75,rgb:[242,149,103]},{index:.88,rgb:[249,193,135]},{index:1,rgb:[254,237,176]}],chlorophyll:[{index:0,rgb:[18,36,20]},{index:.13,rgb:[25,63,41]},{index:.25,rgb:[24,91,59]},{index:.38,rgb:[13,119,72]},{index:.5,rgb:[18,148,80]},{index:.63,rgb:[80,173,89]},{index:.75,rgb:[132,196,122]},{index:.88,rgb:[175,221,162]},{index:1,rgb:[215,249,208]}],density:[{index:0,rgb:[54,14,36]},{index:.13,rgb:[89,23,80]},{index:.25,rgb:[110,45,132]},{index:.38,rgb:[120,77,178]},{index:.5,rgb:[120,113,213]},{index:.63,rgb:[115,151,228]},{index:.75,rgb:[134,185,227]},{index:.88,rgb:[177,214,227]},{index:1,rgb:[230,241,241]}],"freesurface-blue":[{index:0,rgb:[30,4,110]},{index:.13,rgb:[47,14,176]},{index:.25,rgb:[41,45,236]},{index:.38,rgb:[25,99,212]},{index:.5,rgb:[68,131,200]},{index:.63,rgb:[114,156,197]},{index:.75,rgb:[157,181,203]},{index:.88,rgb:[200,208,216]},{index:1,rgb:[241,237,236]}],"freesurface-red":[{index:0,rgb:[60,9,18]},{index:.13,rgb:[100,17,27]},{index:.25,rgb:[142,20,29]},{index:.38,rgb:[177,43,27]},{index:.5,rgb:[192,87,63]},{index:.63,rgb:[205,125,105]},{index:.75,rgb:[216,162,148]},{index:.88,rgb:[227,199,193]},{index:1,rgb:[241,237,236]}],oxygen:[{index:0,rgb:[64,5,5]},{index:.13,rgb:[106,6,15]},{index:.25,rgb:[144,26,7]},{index:.38,rgb:[168,64,3]},{index:.5,rgb:[188,100,4]},{index:.63,rgb:[206,136,11]},{index:.75,rgb:[220,174,25]},{index:.88,rgb:[231,215,44]},{index:1,rgb:[248,254,105]}],par:[{index:0,rgb:[51,20,24]},{index:.13,rgb:[90,32,35]},{index:.25,rgb:[129,44,34]},{index:.38,rgb:[159,68,25]},{index:.5,rgb:[182,99,19]},{index:.63,rgb:[199,134,22]},{index:.75,rgb:[212,171,35]},{index:.88,rgb:[221,210,54]},{index:1,rgb:[225,253,75]}],phase:[{index:0,rgb:[145,105,18]},{index:.13,rgb:[184,71,38]},{index:.25,rgb:[186,58,115]},{index:.38,rgb:[160,71,185]},{index:.5,rgb:[110,97,218]},{index:.63,rgb:[50,123,164]},{index:.75,rgb:[31,131,110]},{index:.88,rgb:[77,129,34]},{index:1,rgb:[145,105,18]}],salinity:[{index:0,rgb:[42,24,108]},{index:.13,rgb:[33,50,162]},{index:.25,rgb:[15,90,145]},{index:.38,rgb:[40,118,137]},{index:.5,rgb:[59,146,135]},{index:.63,rgb:[79,175,126]},{index:.75,rgb:[120,203,104]},{index:.88,rgb:[193,221,100]},{index:1,rgb:[253,239,154]}],temperature:[{index:0,rgb:[4,35,51]},{index:.13,rgb:[23,51,122]},{index:.25,rgb:[85,59,157]},{index:.38,rgb:[129,79,143]},{index:.5,rgb:[175,95,130]},{index:.63,rgb:[222,112,101]},{index:.75,rgb:[249,146,66]},{index:.88,rgb:[249,196,65]},{index:1,rgb:[232,250,91]}],turbidity:[{index:0,rgb:[34,31,27]},{index:.13,rgb:[65,50,41]},{index:.25,rgb:[98,69,52]},{index:.38,rgb:[131,89,57]},{index:.5,rgb:[161,112,59]},{index:.63,rgb:[185,140,66]},{index:.75,rgb:[202,174,88]},{index:.88,rgb:[216,209,126]},{index:1,rgb:[233,246,171]}],"velocity-blue":[{index:0,rgb:[17,32,64]},{index:.13,rgb:[35,52,116]},{index:.25,rgb:[29,81,156]},{index:.38,rgb:[31,113,162]},{index:.5,rgb:[50,144,169]},{index:.63,rgb:[87,173,176]},{index:.75,rgb:[149,196,189]},{index:.88,rgb:[203,221,211]},{index:1,rgb:[254,251,230]}],"velocity-green":[{index:0,rgb:[23,35,19]},{index:.13,rgb:[24,64,38]},{index:.25,rgb:[11,95,45]},{index:.38,rgb:[39,123,35]},{index:.5,rgb:[95,146,12]},{index:.63,rgb:[152,165,18]},{index:.75,rgb:[201,186,69]},{index:.88,rgb:[233,216,137]},{index:1,rgb:[255,253,205]}],cubehelix:[{index:0,rgb:[0,0,0]},{index:.07,rgb:[22,5,59]},{index:.13,rgb:[60,4,105]},{index:.2,rgb:[109,1,135]},{index:.27,rgb:[161,0,147]},{index:.33,rgb:[210,2,142]},{index:.4,rgb:[251,11,123]},{index:.47,rgb:[255,29,97]},{index:.53,rgb:[255,54,69]},{index:.6,rgb:[255,85,46]},{index:.67,rgb:[255,120,34]},{index:.73,rgb:[255,157,37]},{index:.8,rgb:[241,191,57]},{index:.87,rgb:[224,220,93]},{index:.93,rgb:[218,241,142]},{index:1,rgb:[227,253,198]}]}},{}],126:[function(t,e,n){"use strict";var r=t("./colorScale"),i=t("lerp");function o(t){return[t[0]/255,t[1]/255,t[2]/255,t[3]]}function a(t){for(var e,n="#",r=0;r<3;++r)n+=("00"+(e=(e=t[r]).toString(16))).substr(e.length);return n}function s(t){return"rgba("+t.join(",")+")"}e.exports=function(t){var e,n,A,l,c,u,g,h,d,f;if(t||(t={}),h=(t.nshades||72)-1,g=t.format||"hex",(u=t.colormap)||(u="jet"),"string"==typeof u){if(u=u.toLowerCase(),!r[u])throw Error(u+" not a supported colorscale");c=r[u]}else{if(!Array.isArray(u))throw Error("unsupported colormap option",u);c=u.slice()}if(c.length>h+1)throw new Error(u+" map requires nshades to be at least size "+c.length);d=Array.isArray(t.alpha)?2!==t.alpha.length?[1,1]:t.alpha.slice():"number"==typeof t.alpha?[t.alpha,t.alpha]:[1,1],e=c.map(function(t){return Math.round(t.index*h)}),d[0]=Math.min(Math.max(d[0],0),1),d[1]=Math.min(Math.max(d[1],0),1);var p=c.map(function(t,e){var n=c[e].index,r=c[e].rgb.slice();return 4===r.length&&r[3]>=0&&r[3]<=1?r:(r[3]=d[0]+(d[1]-d[0])*n,r)}),I=[];for(f=0;f<e.length-1;++f){l=e[f+1]-e[f],n=p[f],A=p[f+1];for(var C=0;C<l;C++){var m=C/l;I.push([Math.round(i(n[0],A[0],m)),Math.round(i(n[1],A[1],m)),Math.round(i(n[2],A[2],m)),i(n[3],A[3],m)])}}return I.push(c[c.length-1].rgb.concat(d[1])),"hex"===g?I=I.map(a):"rgbaString"===g?I=I.map(s):"float"===g&&(I=I.map(o)),I}},{"./colorScale":125,lerp:425}],127:[function(t,e,n){"use strict";e.exports=function(t,e,n,o){var a=r(e,n,o);if(0===a){var s=i(r(t,e,n)),l=i(r(t,e,o));if(s===l){if(0===s){var c=A(t,e,n),u=A(t,e,o);return c===u?0:c?1:-1}return 0}return 0===l?s>0?-1:A(t,e,o)?-1:1:0===s?l>0?1:A(t,e,n)?1:-1:i(l-s)}var g=r(t,e,n);if(g>0)return a>0&&r(t,e,o)>0?1:-1;if(g<0)return a>0||r(t,e,o)>0?1:-1;var h=r(t,e,o);return h>0?1:A(t,e,n)?1:-1};var r=t("robust-orientation"),i=t("signum"),o=t("two-sum"),a=t("robust-product"),s=t("robust-sum");function A(t,e,n){var r=o(t[0],-e[0]),i=o(t[1],-e[1]),A=o(n[0],-e[0]),l=o(n[1],-e[1]),c=s(a(r,A),a(i,l));return c[c.length-1]>=0}},{"robust-orientation":510,"robust-product":511,"robust-sum":515,signum:516,"two-sum":544}],128:[function(t,e,n){e.exports=function(t,e){var n=t.length,o=t.length-e.length;if(o)return o;switch(n){case 0:return 0;case 1:return t[0]-e[0];case 2:return t[0]+t[1]-e[0]-e[1]||r(t[0],t[1])-r(e[0],e[1]);case 3:var a=t[0]+t[1],s=e[0]+e[1];if(o=a+t[2]-(s+e[2]))return o;var A=r(t[0],t[1]),l=r(e[0],e[1]);return r(A,t[2])-r(l,e[2])||r(A+t[2],a)-r(l+e[2],s);case 4:var c=t[0],u=t[1],g=t[2],h=t[3],d=e[0],f=e[1],p=e[2],I=e[3];return c+u+g+h-(d+f+p+I)||r(c,u,g,h)-r(d,f,p,I,d)||r(c+u,c+g,c+h,u+g,u+h,g+h)-r(d+f,d+p,d+I,f+p,f+I,p+I)||r(c+u+g,c+u+h,c+g+h,u+g+h)-r(d+f+p,d+f+I,d+p+I,f+p+I);default:for(var C=t.slice().sort(i),m=e.slice().sort(i),y=0;y<n;++y)if(o=C[y]-m[y])return o;return 0}};var r=Math.min;function i(t,e){return t-e}},{}],129:[function(t,e,n){"use strict";var r=t("compare-cell"),i=t("cell-orientation");e.exports=function(t,e){return r(t,e)||i(t)-i(e)}},{"cell-orientation":112,"compare-cell":128}],130:[function(t,e,n){"use strict";var r=t("./lib/ch1d"),i=t("./lib/ch2d"),o=t("./lib/chnd");e.exports=function(t){var e=t.length;if(0===e)return[];if(1===e)return[[0]];var n=t[0].length;return 0===n?[]:1===n?r(t):2===n?i(t):o(t,n)}},{"./lib/ch1d":131,"./lib/ch2d":132,"./lib/chnd":133}],131:[function(t,e,n){"use strict";e.exports=function(t){for(var e=0,n=0,r=1;r<t.length;++r)t[r][0]<t[e][0]&&(e=r),t[r][0]>t[n][0]&&(n=r);return e<n?[[e],[n]]:e>n?[[n],[e]]:[[e]]}},{}],132:[function(t,e,n){"use strict";e.exports=function(t){var e=r(t),n=e.length;if(n<=2)return[];for(var i=new Array(n),o=e[n-1],a=0;a<n;++a){var s=e[a];i[a]=[o,s],o=s}return i};var r=t("monotone-convex-hull-2d")},{"monotone-convex-hull-2d":434}],133:[function(t,e,n){"use strict";e.exports=function(t,e){try{return r(t,!0)}catch(s){var n=i(t);if(n.length<=e)return[];var o=function(t,e){for(var n=t.length,r=new Array(n),i=0;i<e.length;++i)r[i]=t[e[i]];for(var o=e.length,i=0;i<n;++i)e.indexOf(i)<0&&(r[o++]=t[i]);return r}(t,n),a=r(o,!0);return function(t,e){for(var n=t.length,r=e.length,i=0;i<n;++i)for(var o=t[i],a=0;a<o.length;++a){var s=o[a];if(s<r)o[a]=e[s];else{s-=r;for(var A=0;A<r;++A)s>=e[A]&&(s+=1);o[a]=s}}return t}(a,n)}};var r=t("incremental-convex-hull"),i=t("affine-hull")},{"affine-hull":62,"incremental-convex-hull":413}],134:[function(t,e,n){e.exports={AFG:"afghan",ALA:"\\b\\wland",ALB:"albania",DZA:"algeria",ASM:"^(?=.*americ).*samoa",AND:"andorra",AGO:"angola",AIA:"anguill?a",ATA:"antarctica",ATG:"antigua",ARG:"argentin",ARM:"armenia",ABW:"^(?!.*bonaire).*\\baruba",AUS:"australia",AUT:"^(?!.*hungary).*austria|\\baustri.*\\bemp",AZE:"azerbaijan",BHS:"bahamas",BHR:"bahrain",BGD:"bangladesh|^(?=.*east).*paki?stan",BRB:"barbados",BLR:"belarus|byelo",BEL:"^(?!.*luxem).*belgium",BLZ:"belize|^(?=.*british).*honduras",BEN:"benin|dahome",BMU:"bermuda",BTN:"bhutan",BOL:"bolivia",BES:"^(?=.*bonaire).*eustatius|^(?=.*carib).*netherlands|\\bbes.?islands",BIH:"herzegovina|bosnia",BWA:"botswana|bechuana",BVT:"bouvet",BRA:"brazil",IOT:"british.?indian.?ocean",BRN:"brunei",BGR:"bulgaria",BFA:"burkina|\\bfaso|upper.?volta",BDI:"burundi",CPV:"verde",KHM:"cambodia|kampuchea|khmer",CMR:"cameroon",CAN:"canada",CYM:"cayman",CAF:"\\bcentral.african.republic",TCD:"\\bchad",CHL:"\\bchile",CHN:"^(?!.*\\bmac)(?!.*\\bhong)(?!.*\\btai)(?!.*\\brep).*china|^(?=.*peo)(?=.*rep).*china",CXR:"christmas",CCK:"\\bcocos|keeling",COL:"colombia",COM:"comoro",COG:"^(?!.*\\bdem)(?!.*\\bd[\\.]?r)(?!.*kinshasa)(?!.*zaire)(?!.*belg)(?!.*l.opoldville)(?!.*free).*\\bcongo",COK:"\\bcook",CRI:"costa.?rica",CIV:"ivoire|ivory",HRV:"croatia",CUB:"\\bcuba",CUW:"^(?!.*bonaire).*\\bcura(c|ç)ao",CYP:"cyprus",CSK:"czechoslovakia",CZE:"^(?=.*rep).*czech|czechia|bohemia",COD:"\\bdem.*congo|congo.*\\bdem|congo.*\\bd[\\.]?r|\\bd[\\.]?r.*congo|belgian.?congo|congo.?free.?state|kinshasa|zaire|l.opoldville|drc|droc|rdc",DNK:"denmark",DJI:"djibouti",DMA:"dominica(?!n)",DOM:"dominican.rep",ECU:"ecuador",EGY:"egypt",SLV:"el.?salvador",GNQ:"guine.*eq|eq.*guine|^(?=.*span).*guinea",ERI:"eritrea",EST:"estonia",ETH:"ethiopia|abyssinia",FLK:"falkland|malvinas",FRO:"faroe|faeroe",FJI:"fiji",FIN:"finland",FRA:"^(?!.*\\bdep)(?!.*martinique).*france|french.?republic|\\bgaul",GUF:"^(?=.*french).*guiana",PYF:"french.?polynesia|tahiti",ATF:"french.?southern",GAB:"gabon",GMB:"gambia",GEO:"^(?!.*south).*georgia",DDR:"german.?democratic.?republic|democratic.?republic.*germany|east.germany",DEU:"^(?!.*east).*germany|^(?=.*\\bfed.*\\brep).*german",GHA:"ghana|gold.?coast",GIB:"gibraltar",GRC:"greece|hellenic|hellas",GRL:"greenland",GRD:"grenada",GLP:"guadeloupe",GUM:"\\bguam",GTM:"guatemala",GGY:"guernsey",GIN:"^(?!.*eq)(?!.*span)(?!.*bissau)(?!.*portu)(?!.*new).*guinea",GNB:"bissau|^(?=.*portu).*guinea",GUY:"guyana|british.?guiana",HTI:"haiti",HMD:"heard.*mcdonald",VAT:"holy.?see|vatican|papal.?st",HND:"^(?!.*brit).*honduras",HKG:"hong.?kong",HUN:"^(?!.*austr).*hungary",ISL:"iceland",IND:"india(?!.*ocea)",IDN:"indonesia",IRN:"\\biran|persia",IRQ:"\\biraq|mesopotamia",IRL:"(^ireland)|(^republic.*ireland)",IMN:"^(?=.*isle).*\\bman",ISR:"israel",ITA:"italy",JAM:"jamaica",JPN:"japan",JEY:"jersey",JOR:"jordan",KAZ:"kazak",KEN:"kenya|british.?east.?africa|east.?africa.?prot",KIR:"kiribati",PRK:"^(?=.*democrat|people|north|d.*p.*.r).*\\bkorea|dprk|korea.*(d.*p.*r)",KWT:"kuwait",KGZ:"kyrgyz|kirghiz",LAO:"\\blaos?\\b",LVA:"latvia",LBN:"lebanon",LSO:"lesotho|basuto",LBR:"liberia",LBY:"libya",LIE:"liechtenstein",LTU:"lithuania",LUX:"^(?!.*belg).*luxem",MAC:"maca(o|u)",MDG:"madagascar|malagasy",MWI:"malawi|nyasa",MYS:"malaysia",MDV:"maldive",MLI:"\\bmali\\b",MLT:"\\bmalta",MHL:"marshall",MTQ:"martinique",MRT:"mauritania",MUS:"mauritius",MYT:"\\bmayotte",MEX:"\\bmexic",FSM:"fed.*micronesia|micronesia.*fed",MCO:"monaco",MNG:"mongolia",MNE:"^(?!.*serbia).*montenegro",MSR:"montserrat",MAR:"morocco|\\bmaroc",MOZ:"mozambique",MMR:"myanmar|burma",NAM:"namibia",NRU:"nauru",NPL:"nepal",NLD:"^(?!.*\\bant)(?!.*\\bcarib).*netherlands",ANT:"^(?=.*\\bant).*(nether|dutch)",NCL:"new.?caledonia",NZL:"new.?zealand",NIC:"nicaragua",NER:"\\bniger(?!ia)",NGA:"nigeria",NIU:"niue",NFK:"norfolk",MNP:"mariana",NOR:"norway",OMN:"\\boman|trucial",PAK:"^(?!.*east).*paki?stan",PLW:"palau",PSE:"palestin|\\bgaza|west.?bank",PAN:"panama",PNG:"papua|new.?guinea",PRY:"paraguay",PER:"peru",PHL:"philippines",PCN:"pitcairn",POL:"poland",PRT:"portugal",PRI:"puerto.?rico",QAT:"qatar",KOR:"^(?!.*d.*p.*r)(?!.*democrat)(?!.*people)(?!.*north).*\\bkorea(?!.*d.*p.*r)",MDA:"moldov|b(a|e)ssarabia",REU:"r(e|é)union",ROU:"r(o|u|ou)mania",RUS:"\\brussia|soviet.?union|u\\.?s\\.?s\\.?r|socialist.?republics",RWA:"rwanda",BLM:"barth(e|é)lemy",SHN:"helena",KNA:"kitts|\\bnevis",LCA:"\\blucia",MAF:"^(?=.*collectivity).*martin|^(?=.*france).*martin(?!ique)|^(?=.*french).*martin(?!ique)",SPM:"miquelon",VCT:"vincent",WSM:"^(?!.*amer).*samoa",SMR:"san.?marino",STP:"\\bs(a|ã)o.?tom(e|é)",SAU:"\\bsa\\w*.?arabia",SEN:"senegal",SRB:"^(?!.*monte).*serbia",SYC:"seychell",SLE:"sierra",SGP:"singapore",SXM:"^(?!.*martin)(?!.*saba).*maarten",SVK:"^(?!.*cze).*slovak",SVN:"slovenia",SLB:"solomon",SOM:"somali",ZAF:"south.africa|s\\\\..?africa",SGS:"south.?georgia|sandwich",SSD:"\\bs\\w*.?sudan",ESP:"spain",LKA:"sri.?lanka|ceylon",SDN:"^(?!.*\\bs(?!u)).*sudan",SUR:"surinam|dutch.?guiana",SJM:"svalbard",SWZ:"swaziland",SWE:"sweden",CHE:"switz|swiss",SYR:"syria",TWN:"taiwan|taipei|formosa|^(?!.*peo)(?=.*rep).*china",TJK:"tajik",THA:"thailand|\\bsiam",MKD:"macedonia|fyrom",TLS:"^(?=.*leste).*timor|^(?=.*east).*timor",TGO:"togo",TKL:"tokelau",TON:"tonga",TTO:"trinidad|tobago",TUN:"tunisia",TUR:"turkey",TKM:"turkmen",TCA:"turks",TUV:"tuvalu",UGA:"uganda",UKR:"ukrain",ARE:"emirates|^u\\.?a\\.?e\\.?$|united.?arab.?em",GBR:"united.?kingdom|britain|^u\\.?k\\.?$",TZA:"tanzania",USA:"united.?states\\b(?!.*islands)|\\bu\\.?s\\.?a\\.?\\b|^\\s*u\\.?s\\.?\\b(?!.*islands)",UMI:"minor.?outlying.?is",URY:"uruguay",UZB:"uzbek",VUT:"vanuatu|new.?hebrides",VEN:"venezuela",VNM:"^(?!.*republic).*viet.?nam|^(?=.*socialist).*viet.?nam",VGB:"^(?=.*\\bu\\.?\\s?k).*virgin|^(?=.*brit).*virgin|^(?=.*kingdom).*virgin",VIR:"^(?=.*\\bu\\.?\\s?s).*virgin|^(?=.*states).*virgin",WLF:"futuna|wallis",ESH:"western.sahara",YEM:"^(?!.*arab)(?!.*north)(?!.*sana)(?!.*peo)(?!.*dem)(?!.*south)(?!.*aden)(?!.*\\bp\\.?d\\.?r).*yemen",YMD:"^(?=.*peo).*yemen|^(?!.*rep)(?=.*dem).*yemen|^(?=.*south).*yemen|^(?=.*aden).*yemen|^(?=.*\\bp\\.?d\\.?r).*yemen",YUG:"yugoslavia",ZMB:"zambia|northern.?rhodesia",EAZ:"zanzibar",ZWE:"zimbabwe|^(?!.*northern).*rhodesia"}},{}],135:[function(t,e,n){e.exports=["xx-small","x-small","small","medium","large","x-large","xx-large","larger","smaller"]},{}],136:[function(t,e,n){e.exports=["normal","condensed","semi-condensed","extra-condensed","ultra-condensed","expanded","semi-expanded","extra-expanded","ultra-expanded"]},{}],137:[function(t,e,n){e.exports=["normal","italic","oblique"]},{}],138:[function(t,e,n){e.exports=["normal","bold","bolder","lighter","100","200","300","400","500","600","700","800","900"]},{}],139:[function(t,e,n){"use strict";e.exports={parse:t("./parse"),stringify:t("./stringify")}},{"./parse":141,"./stringify":142}],140:[function(t,e,n){"use strict";var r=t("css-font-size-keywords");e.exports={isSize:function(t){return/^[\d\.]/.test(t)||-1!==t.indexOf("/")||-1!==r.indexOf(t)}}},{"css-font-size-keywords":135}],141:[function(t,e,n){"use strict";var r=t("unquote"),i=t("css-global-keywords"),o=t("css-system-font-keywords"),a=t("css-font-weight-keywords"),s=t("css-font-style-keywords"),A=t("css-font-stretch-keywords"),l=t("string-split-by"),c=t("./lib/util").isSize;e.exports=g;var u=g.cache={};function g(t){if("string"!=typeof t)throw new Error("Font argument must be a string.");if(u[t])return u[t];if(""===t)throw new Error("Cannot parse an empty string.");if(-1!==o.indexOf(t))return u[t]={system:t};for(var e,n={style:"normal",variant:"normal",weight:"normal",stretch:"normal",lineHeight:"normal",size:"1rem",family:["serif"]},g=l(t,/\s+/);e=g.shift();){if(-1!==i.indexOf(e))return["style","variant","weight","stretch"].forEach(function(t){n[t]=e}),u[t]=n;if(-1===s.indexOf(e))if("normal"!==e&&"small-caps"!==e)if(-1===A.indexOf(e)){if(-1===a.indexOf(e)){if(c(e)){var d=l(e,"/");if(n.size=d[0],null!=d[1]?n.lineHeight=h(d[1]):"/"===g[0]&&(g.shift(),n.lineHeight=h(g.shift())),!g.length)throw new Error("Missing required font-family.");return n.family=l(g.join(" "),/\s*,\s*/).map(r),u[t]=n}throw new Error("Unknown or unsupported font token: "+e)}n.weight=e}else n.stretch=e;else n.variant=e;else n.style=e}throw new Error("Missing required font-size.")}function h(t){var e=parseFloat(t);return e.toString()===t?e:t}},{"./lib/util":140,"css-font-stretch-keywords":136,"css-font-style-keywords":137,"css-font-weight-keywords":138,"css-global-keywords":143,"css-system-font-keywords":144,"string-split-by":529,unquote:548}],142:[function(t,e,n){"use strict";var r=t("pick-by-alias"),i=t("./lib/util").isSize,o=d(t("css-global-keywords")),a=d(t("css-system-font-keywords")),s=d(t("css-font-weight-keywords")),A=d(t("css-font-style-keywords")),l=d(t("css-font-stretch-keywords")),c={normal:1,"small-caps":1},u={serif:1,"sans-serif":1,monospace:1,cursive:1,fantasy:1,"system-ui":1},g="serif";function h(t,e){if(t&&!e[t]&&!o[t])throw Error("Unknown keyword `"+t+"`");return t}function d(t){for(var e={},n=0;n<t.length;n++)e[t[n]]=1;return e}e.exports=function(t){if((t=r(t,{style:"style fontstyle fontStyle font-style slope distinction",variant:"variant font-variant fontVariant fontvariant var capitalization",weight:"weight w font-weight fontWeight fontweight",stretch:"stretch font-stretch fontStretch fontstretch width",size:"size s font-size fontSize fontsize height em emSize",lineHeight:"lh line-height lineHeight lineheight leading",family:"font family fontFamily font-family fontfamily type typeface face",system:"system reserved default global"})).system)return t.system&&h(t.system,a),t.system;if(h(t.style,A),h(t.variant,c),h(t.weight,s),h(t.stretch,l),null==t.size&&(t.size="1rem"),"number"==typeof t.size&&(t.size+="px"),!i)throw Error("Bad size value `"+t.size+"`");t.family||(t.family=g),Array.isArray(t.family)&&(t.family.length||(t.family=[g]),t.family=t.family.map(function(t){return u[t]?t:'"'+t+'"'}).join(", "));var e=[];return e.push(t.style),t.variant!==t.style&&e.push(t.variant),t.weight!==t.variant&&t.weight!==t.style&&e.push(t.weight),t.stretch!==t.weight&&t.stretch!==t.variant&&t.stretch!==t.style&&e.push(t.stretch),e.push(t.size+(null==t.lineHeight||"normal"===t.lineHeight||t.lineHeight+""=="1"?"":"/"+t.lineHeight)),e.push(t.family),e.filter(Boolean).join(" ")}},{"./lib/util":140,"css-font-stretch-keywords":136,"css-font-style-keywords":137,"css-font-weight-keywords":138,"css-global-keywords":143,"css-system-font-keywords":144,"pick-by-alias":465}],143:[function(t,e,n){e.exports=["inherit","initial","unset"]},{}],144:[function(t,e,n){e.exports=["caption","icon","menu","message-box","small-caption","status-bar"]},{}],145:[function(t,e,n){"use strict";e.exports=function(t,e,n,r,i,o){var a=i-1,s=i*i,A=a*a,l=(1+2*i)*A,c=i*A,u=s*(3-2*i),g=s*a;if(t.length){o||(o=new Array(t.length));for(var h=t.length-1;h>=0;--h)o[h]=l*t[h]+c*e[h]+u*n[h]+g*r[h];return o}return l*t+c*e+u*n+g*r},e.exports.derivative=function(t,e,n,r,i,o){var a=6*i*i-6*i,s=3*i*i-4*i+1,A=-6*i*i+6*i,l=3*i*i-2*i;if(t.length){o||(o=new Array(t.length));for(var c=t.length-1;c>=0;--c)o[c]=a*t[c]+s*e[c]+A*n[c]+l*r[c];return o}return a*t+s*e+A*n[c]+l*r}},{}],146:[function(t,e,n){"use strict";var r=t("./lib/thunk.js");function i(){this.argTypes=[],this.shimArgs=[],this.arrayArgs=[],this.arrayBlockIndices=[],this.scalarArgs=[],this.offsetArgs=[],this.offsetArgIndex=[],this.indexArgs=[],this.shapeArgs=[],this.funcName="",this.pre=null,this.body=null,this.post=null,this.debug=!1}e.exports=function(t){var e=new i;e.pre=t.pre,e.body=t.body,e.post=t.post;var n=t.args.slice(0);e.argTypes=n;for(var o=0;o<n.length;++o){var a=n[o];if("array"===a||"object"==typeof a&&a.blockIndices){if(e.argTypes[o]="array",e.arrayArgs.push(o),e.arrayBlockIndices.push(a.blockIndices?a.blockIndices:0),e.shimArgs.push("array"+o),o<e.pre.args.length&&e.pre.args[o].count>0)throw new Error("cwise: pre() block may not reference array args");if(o<e.post.args.length&&e.post.args[o].count>0)throw new Error("cwise: post() block may not reference array args")}else if("scalar"===a)e.scalarArgs.push(o),e.shimArgs.push("scalar"+o);else if("index"===a){if(e.indexArgs.push(o),o<e.pre.args.length&&e.pre.args[o].count>0)throw new Error("cwise: pre() block may not reference array index");if(o<e.body.args.length&&e.body.args[o].lvalue)throw new Error("cwise: body() block may not write to array index");if(o<e.post.args.length&&e.post.args[o].count>0)throw new Error("cwise: post() block may not reference array index")}else if("shape"===a){if(e.shapeArgs.push(o),o<e.pre.args.length&&e.pre.args[o].lvalue)throw new Error("cwise: pre() block may not write to array shape");if(o<e.body.args.length&&e.body.args[o].lvalue)throw new Error("cwise: body() block may not write to array shape");if(o<e.post.args.length&&e.post.args[o].lvalue)throw new Error("cwise: post() block may not write to array shape")}else{if("object"!=typeof a||!a.offset)throw new Error("cwise: Unknown argument type "+n[o]);e.argTypes[o]="offset",e.offsetArgs.push({array:a.array,offset:a.offset}),e.offsetArgIndex.push(o)}}if(e.arrayArgs.length<=0)throw new Error("cwise: No array arguments specified");if(e.pre.args.length>n.length)throw new Error("cwise: Too many arguments in pre() block");if(e.body.args.length>n.length)throw new Error("cwise: Too many arguments in body() block");if(e.post.args.length>n.length)throw new Error("cwise: Too many arguments in post() block");return e.debug=!!t.printCode||!!t.debug,e.funcName=t.funcName||"cwise",e.blockSize=t.blockSize||64,r(e)}},{"./lib/thunk.js":148}],147:[function(t,e,n){"use strict";var r=t("uniq");function i(t,e,n){var r,i,o=t.length,a=e.arrayArgs.length,s=e.indexArgs.length>0,A=[],l=[],c=0,u=0;for(r=0;r<o;++r)l.push(["i",r,"=0"].join(""));for(i=0;i<a;++i)for(r=0;r<o;++r)u=c,c=t[r],0===r?l.push(["d",i,"s",r,"=t",i,"p",c].join("")):l.push(["d",i,"s",r,"=(t",i,"p",c,"-s",u,"*t",i,"p",u,")"].join(""));for(l.length>0&&A.push("var "+l.join(",")),r=o-1;r>=0;--r)c=t[r],A.push(["for(i",r,"=0;i",r,"<s",c,";++i",r,"){"].join(""));for(A.push(n),r=0;r<o;++r){for(u=c,c=t[r],i=0;i<a;++i)A.push(["p",i,"+=d",i,"s",r].join(""));s&&(r>0&&A.push(["index[",u,"]-=s",u].join("")),A.push(["++index[",c,"]"].join(""))),A.push("}")}return A.join("\n")}function o(t,e,n){for(var r=t.body,i=[],o=[],a=0;a<t.args.length;++a){var s=t.args[a];if(!(s.count<=0)){var A=new RegExp(s.name,"g"),l="",c=e.arrayArgs.indexOf(a);switch(e.argTypes[a]){case"offset":var u=e.offsetArgIndex.indexOf(a);c=e.offsetArgs[u].array,l="+q"+u;case"array":l="p"+c+l;var g="l"+a,h="a"+c;if(0===e.arrayBlockIndices[c])1===s.count?"generic"===n[c]?s.lvalue?(i.push(["var ",g,"=",h,".get(",l,")"].join("")),r=r.replace(A,g),o.push([h,".set(",l,",",g,")"].join(""))):r=r.replace(A,[h,".get(",l,")"].join("")):r=r.replace(A,[h,"[",l,"]"].join("")):"generic"===n[c]?(i.push(["var ",g,"=",h,".get(",l,")"].join("")),r=r.replace(A,g),s.lvalue&&o.push([h,".set(",l,",",g,")"].join(""))):(i.push(["var ",g,"=",h,"[",l,"]"].join("")),r=r.replace(A,g),s.lvalue&&o.push([h,"[",l,"]=",g].join("")));else{for(var d=[s.name],f=[l],p=0;p<Math.abs(e.arrayBlockIndices[c]);p++)d.push("\\s*\\[([^\\]]+)\\]"),f.push("$"+(p+1)+"*t"+c+"b"+p);if(A=new RegExp(d.join(""),"g"),l=f.join("+"),"generic"===n[c])throw new Error("cwise: Generic arrays not supported in combination with blocks!");r=r.replace(A,[h,"[",l,"]"].join(""))}break;case"scalar":r=r.replace(A,"Y"+e.scalarArgs.indexOf(a));break;case"index":r=r.replace(A,"index");break;case"shape":r=r.replace(A,"shape")}}}return[i.join("\n"),r,o.join("\n")].join("\n").trim()}e.exports=function(t,e){for(var n=e[1].length-Math.abs(t.arrayBlockIndices[0])|0,a=new Array(t.arrayArgs.length),s=new Array(t.arrayArgs.length),A=0;A<t.arrayArgs.length;++A)s[A]=e[2*A],a[A]=e[2*A+1];var l=[],c=[],u=[],g=[],h=[];for(A=0;A<t.arrayArgs.length;++A){t.arrayBlockIndices[A]<0?(u.push(0),g.push(n),l.push(n),c.push(n+t.arrayBlockIndices[A])):(u.push(t.arrayBlockIndices[A]),g.push(t.arrayBlockIndices[A]+n),l.push(0),c.push(t.arrayBlockIndices[A]));for(var d=[],f=0;f<a[A].length;f++)u[A]<=a[A][f]&&a[A][f]<g[A]&&d.push(a[A][f]-u[A]);h.push(d)}var p=["SS"],I=["'use strict'"],C=[];for(f=0;f<n;++f)C.push(["s",f,"=SS[",f,"]"].join(""));for(A=0;A<t.arrayArgs.length;++A){for(p.push("a"+A),p.push("t"+A),p.push("p"+A),f=0;f<n;++f)C.push(["t",A,"p",f,"=t",A,"[",u[A]+f,"]"].join(""));for(f=0;f<Math.abs(t.arrayBlockIndices[A]);++f)C.push(["t",A,"b",f,"=t",A,"[",l[A]+f,"]"].join(""))}for(A=0;A<t.scalarArgs.length;++A)p.push("Y"+A);if(t.shapeArgs.length>0&&C.push("shape=SS.slice(0)"),t.indexArgs.length>0){var m=new Array(n);for(A=0;A<n;++A)m[A]="0";C.push(["index=[",m.join(","),"]"].join(""))}for(A=0;A<t.offsetArgs.length;++A){var y=t.offsetArgs[A],E=[];for(f=0;f<y.offset.length;++f)0!==y.offset[f]&&(1===y.offset[f]?E.push(["t",y.array,"p",f].join("")):E.push([y.offset[f],"*t",y.array,"p",f].join("")));0===E.length?C.push("q"+A+"=0"):C.push(["q",A,"=",E.join("+")].join(""))}var v=r([].concat(t.pre.thisVars).concat(t.body.thisVars).concat(t.post.thisVars));for((C=C.concat(v)).length>0&&I.push("var "+C.join(",")),A=0;A<t.arrayArgs.length;++A)I.push("p"+A+"|=0");t.pre.body.length>3&&I.push(o(t.pre,t,s));var B=o(t.body,t,s),b=function(t){for(var e=0,n=t[0].length;e<n;){for(var r=1;r<t.length;++r)if(t[r][e]!==t[0][e])return e;++e}return e}(h);b<n?I.push(function(t,e,n,r){for(var o=e.length,a=n.arrayArgs.length,s=n.blockSize,A=n.indexArgs.length>0,l=[],c=0;c<a;++c)l.push(["var offset",c,"=p",c].join(""));for(c=t;c<o;++c)l.push(["for(var j"+c+"=SS[",e[c],"]|0;j",c,">0;){"].join("")),l.push(["if(j",c,"<",s,"){"].join("")),l.push(["s",e[c],"=j",c].join("")),l.push(["j",c,"=0"].join("")),l.push(["}else{s",e[c],"=",s].join("")),l.push(["j",c,"-=",s,"}"].join("")),A&&l.push(["index[",e[c],"]=j",c].join(""));for(c=0;c<a;++c){for(var u=["offset"+c],g=t;g<o;++g)u.push(["j",g,"*t",c,"p",e[g]].join(""));l.push(["p",c,"=(",u.join("+"),")"].join(""))}for(l.push(i(e,n,r)),c=t;c<o;++c)l.push("}");return l.join("\n")}(b,h[0],t,B)):I.push(i(h[0],t,B)),t.post.body.length>3&&I.push(o(t.post,t,s)),t.debug&&console.log("-----Generated cwise routine for ",e,":\n"+I.join("\n")+"\n----------");var Q=[t.funcName||"unnamed","_cwise_loop_",a[0].join("s"),"m",b,function(t){for(var e=new Array(t.length),n=!0,r=0;r<t.length;++r){var i=t[r],o=i.match(/\d+/);o=o?o[0]:"",0===i.charAt(0)?e[r]="u"+i.charAt(1)+o:e[r]=i.charAt(0)+o,r>0&&(n=n&&e[r]===e[r-1])}return n?e[0]:e.join("")}(s)].join("");return new Function(["function ",Q,"(",p.join(","),"){",I.join("\n"),"} return ",Q].join(""))()}},{uniq:547}],148:[function(t,e,n){"use strict";var r=t("./compile.js");e.exports=function(t){var e=["'use strict'","var CACHED={}"],n=[],i=t.funcName+"_cwise_thunk";e.push(["return function ",i,"(",t.shimArgs.join(","),"){"].join(""));for(var o=[],a=[],s=[["array",t.arrayArgs[0],".shape.slice(",Math.max(0,t.arrayBlockIndices[0]),t.arrayBlockIndices[0]<0?","+t.arrayBlockIndices[0]+")":")"].join("")],A=[],l=[],c=0;c<t.arrayArgs.length;++c){var u=t.arrayArgs[c];n.push(["t",u,"=array",u,".dtype,","r",u,"=array",u,".order"].join("")),o.push("t"+u),o.push("r"+u),a.push("t"+u),a.push("r"+u+".join()"),s.push("array"+u+".data"),s.push("array"+u+".stride"),s.push("array"+u+".offset|0"),c>0&&(A.push("array"+t.arrayArgs[0]+".shape.length===array"+u+".shape.length+"+(Math.abs(t.arrayBlockIndices[0])-Math.abs(t.arrayBlockIndices[c]))),l.push("array"+t.arrayArgs[0]+".shape[shapeIndex+"+Math.max(0,t.arrayBlockIndices[0])+"]===array"+u+".shape[shapeIndex+"+Math.max(0,t.arrayBlockIndices[c])+"]"))}for(t.arrayArgs.length>1&&(e.push("if (!("+A.join(" && ")+")) throw new Error('cwise: Arrays do not all have the same dimensionality!')"),e.push("for(var shapeIndex=array"+t.arrayArgs[0]+".shape.length-"+Math.abs(t.arrayBlockIndices[0])+"; shapeIndex--\x3e0;) {"),e.push("if (!("+l.join(" && ")+")) throw new Error('cwise: Arrays do not all have the same shape!')"),e.push("}")),c=0;c<t.scalarArgs.length;++c)s.push("scalar"+t.scalarArgs[c]);return n.push(["type=[",a.join(","),"].join()"].join("")),n.push("proc=CACHED[type]"),e.push("var "+n.join(",")),e.push(["if(!proc){","CACHED[type]=proc=compile([",o.join(","),"])}","return proc(",s.join(","),")}"].join("")),t.debug&&console.log("-----Generated thunk:\n"+e.join("\n")+"\n----------"),new Function("compile",e.join("\n"))(r.bind(void 0,t))}},{"./compile.js":147}],149:[function(t,e,n){e.exports=t("cwise-compiler")},{"cwise-compiler":146}],150:[function(t,e,n){"use strict";var r,i=t("es5-ext/object/copy"),o=t("es5-ext/object/normalize-options"),a=t("es5-ext/object/valid-callable"),s=t("es5-ext/object/map"),A=t("es5-ext/object/valid-callable"),l=t("es5-ext/object/valid-value"),c=Function.prototype.bind,u=Object.defineProperty,g=Object.prototype.hasOwnProperty;r=function(t,e,n){var r,o=l(e)&&A(e.value);return delete(r=i(e)).writable,delete r.value,r.get=function(){return!n.overwriteDefinition&&g.call(this,t)?o:(e.value=c.call(o,n.resolveContext?n.resolveContext(this):this),u(this,t,e),this[t])},r},e.exports=function(t){var e=o(arguments[1]);return null!=e.resolveContext&&a(e.resolveContext),s(t,function(t,n){return r(n,t,e)})}},{"es5-ext/object/copy":189,"es5-ext/object/map":198,"es5-ext/object/normalize-options":199,"es5-ext/object/valid-callable":203,"es5-ext/object/valid-value":205}],151:[function(t,e,n){"use strict";var r=t("es5-ext/object/assign"),i=t("es5-ext/object/normalize-options"),o=t("es5-ext/object/is-callable"),a=t("es5-ext/string/#/contains");(e.exports=function(t,e){var n,o,s,A,l;return arguments.length<2||"string"!=typeof t?(A=e,e=t,t=null):A=arguments[2],null==t?(n=s=!0,o=!1):(n=a.call(t,"c"),o=a.call(t,"e"),s=a.call(t,"w")),l={value:e,configurable:n,enumerable:o,writable:s},A?r(i(A),l):l}).gs=function(t,e,n){var s,A,l,c;return"string"!=typeof t?(l=n,n=e,e=t,t=null):l=arguments[3],null==e?e=void 0:o(e)?null==n?n=void 0:o(n)||(l=n,n=void 0):(l=e,e=n=void 0),null==t?(s=!0,A=!1):(s=a.call(t,"c"),A=a.call(t,"e")),c={get:e,set:n,configurable:s,enumerable:A},l?r(i(l),c):c}},{"es5-ext/object/assign":186,"es5-ext/object/is-callable":192,"es5-ext/object/normalize-options":199,"es5-ext/string/#/contains":206}],152:[function(t,e,n){!function(t){"use strict";function e(t,e){return t<e?-1:t>e?1:t>=e?0:NaN}function n(t){var n;return 1===t.length&&(n=t,t=function(t,r){return e(n(t),r)}),{left:function(e,n,r,i){for(null==r&&(r=0),null==i&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)<0?r=o+1:i=o}return r},right:function(e,n,r,i){for(null==r&&(r=0),null==i&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)>0?i=o:r=o+1}return r}}}var r=n(e),i=r.right,o=r.left;function a(t,e){return[t,e]}function s(t){return null===t?NaN:+t}function A(t,e){var n,r,i=t.length,o=0,a=-1,A=0,l=0;if(null==e)for(;++a<i;)isNaN(n=s(t[a]))||(l+=(r=n-A)*(n-(A+=r/++o)));else for(;++a<i;)isNaN(n=s(e(t[a],a,t)))||(l+=(r=n-A)*(n-(A+=r/++o)));if(o>1)return l/(o-1)}function l(t,e){var n=A(t,e);return n?Math.sqrt(n):n}function c(t,e){var n,r,i,o=t.length,a=-1;if(null==e){for(;++a<o;)if(null!=(n=t[a])&&n>=n)for(r=i=n;++a<o;)null!=(n=t[a])&&(r>n&&(r=n),i<n&&(i=n))}else for(;++a<o;)if(null!=(n=e(t[a],a,t))&&n>=n)for(r=i=n;++a<o;)null!=(n=e(t[a],a,t))&&(r>n&&(r=n),i<n&&(i=n));return[r,i]}var u=Array.prototype,g=u.slice,h=u.map;function d(t){return function(){return t}}function f(t){return t}function p(t,e,n){t=+t,e=+e,n=(i=arguments.length)<2?(e=t,t=0,1):i<3?1:+n;for(var r=-1,i=0|Math.max(0,Math.ceil((e-t)/n)),o=new Array(i);++r<i;)o[r]=t+r*n;return o}var I=Math.sqrt(50),C=Math.sqrt(10),m=Math.sqrt(2);function y(t,e,n){var r=(e-t)/Math.max(0,n),i=Math.floor(Math.log(r)/Math.LN10),o=r/Math.pow(10,i);return i>=0?(o>=I?10:o>=C?5:o>=m?2:1)*Math.pow(10,i):-Math.pow(10,-i)/(o>=I?10:o>=C?5:o>=m?2:1)}function E(t,e,n){var r=Math.abs(e-t)/Math.max(0,n),i=Math.pow(10,Math.floor(Math.log(r)/Math.LN10)),o=r/i;return o>=I?i*=10:o>=C?i*=5:o>=m&&(i*=2),e<t?-i:i}function v(t){return Math.ceil(Math.log(t.length)/Math.LN2)+1}function B(t,e,n){if(null==n&&(n=s),r=t.length){if((e=+e)<=0||r<2)return+n(t[0],0,t);if(e>=1)return+n(t[r-1],r-1,t);var r,i=(r-1)*e,o=Math.floor(i),a=+n(t[o],o,t);return a+(+n(t[o+1],o+1,t)-a)*(i-o)}}function b(t,e){var n,r,i=t.length,o=-1;if(null==e){for(;++o<i;)if(null!=(n=t[o])&&n>=n)for(r=n;++o<i;)null!=(n=t[o])&&r>n&&(r=n)}else for(;++o<i;)if(null!=(n=e(t[o],o,t))&&n>=n)for(r=n;++o<i;)null!=(n=e(t[o],o,t))&&r>n&&(r=n);return r}function Q(t){if(!(i=t.length))return[];for(var e=-1,n=b(t,w),r=new Array(n);++e<n;)for(var i,o=-1,a=r[e]=new Array(i);++o<i;)a[o]=t[o][e];return r}function w(t){return t.length}t.bisect=i,t.bisectRight=i,t.bisectLeft=o,t.ascending=e,t.bisector=n,t.cross=function(t,e,n){var r,i,o,s,A=t.length,l=e.length,c=new Array(A*l);for(null==n&&(n=a),r=o=0;r<A;++r)for(s=t[r],i=0;i<l;++i,++o)c[o]=n(s,e[i]);return c},t.descending=function(t,e){return e<t?-1:e>t?1:e>=t?0:NaN},t.deviation=l,t.extent=c,t.histogram=function(){var t=f,e=c,n=v;function r(r){var o,a,s=r.length,A=new Array(s);for(o=0;o<s;++o)A[o]=t(r[o],o,r);var l=e(A),c=l[0],u=l[1],g=n(A,c,u);Array.isArray(g)||(g=E(c,u,g),g=p(Math.ceil(c/g)*g,u,g));for(var h=g.length;g[0]<=c;)g.shift(),--h;for(;g[h-1]>u;)g.pop(),--h;var d,f=new Array(h+1);for(o=0;o<=h;++o)(d=f[o]=[]).x0=o>0?g[o-1]:c,d.x1=o<h?g[o]:u;for(o=0;o<s;++o)c<=(a=A[o])&&a<=u&&f[i(g,a,0,h)].push(r[o]);return f}return r.value=function(e){return arguments.length?(t="function"==typeof e?e:d(e),r):t},r.domain=function(t){return arguments.length?(e="function"==typeof t?t:d([t[0],t[1]]),r):e},r.thresholds=function(t){return arguments.length?(n="function"==typeof t?t:Array.isArray(t)?d(g.call(t)):d(t),r):n},r},t.thresholdFreedmanDiaconis=function(t,n,r){return t=h.call(t,s).sort(e),Math.ceil((r-n)/(2*(B(t,.75)-B(t,.25))*Math.pow(t.length,-1/3)))},t.thresholdScott=function(t,e,n){return Math.ceil((n-e)/(3.5*l(t)*Math.pow(t.length,-1/3)))},t.thresholdSturges=v,t.max=function(t,e){var n,r,i=t.length,o=-1;if(null==e){for(;++o<i;)if(null!=(n=t[o])&&n>=n)for(r=n;++o<i;)null!=(n=t[o])&&n>r&&(r=n)}else for(;++o<i;)if(null!=(n=e(t[o],o,t))&&n>=n)for(r=n;++o<i;)null!=(n=e(t[o],o,t))&&n>r&&(r=n);return r},t.mean=function(t,e){var n,r=t.length,i=r,o=-1,a=0;if(null==e)for(;++o<r;)isNaN(n=s(t[o]))?--i:a+=n;else for(;++o<r;)isNaN(n=s(e(t[o],o,t)))?--i:a+=n;if(i)return a/i},t.median=function(t,n){var r,i=t.length,o=-1,a=[];if(null==n)for(;++o<i;)isNaN(r=s(t[o]))||a.push(r);else for(;++o<i;)isNaN(r=s(n(t[o],o,t)))||a.push(r);return B(a.sort(e),.5)},t.merge=function(t){for(var e,n,r,i=t.length,o=-1,a=0;++o<i;)a+=t[o].length;for(n=new Array(a);--i>=0;)for(e=(r=t[i]).length;--e>=0;)n[--a]=r[e];return n},t.min=b,t.pairs=function(t,e){null==e&&(e=a);for(var n=0,r=t.length-1,i=t[0],o=new Array(r<0?0:r);n<r;)o[n]=e(i,i=t[++n]);return o},t.permute=function(t,e){for(var n=e.length,r=new Array(n);n--;)r[n]=t[e[n]];return r},t.quantile=B,t.range=p,t.scan=function(t,n){if(r=t.length){var r,i,o=0,a=0,s=t[a];for(null==n&&(n=e);++o<r;)(n(i=t[o],s)<0||0!==n(s,s))&&(s=i,a=o);return 0===n(s,s)?a:void 0}},t.shuffle=function(t,e,n){for(var r,i,o=(null==n?t.length:n)-(e=null==e?0:+e);o;)i=Math.random()*o--|0,r=t[o+e],t[o+e]=t[i+e],t[i+e]=r;return t},t.sum=function(t,e){var n,r=t.length,i=-1,o=0;if(null==e)for(;++i<r;)(n=+t[i])&&(o+=n);else for(;++i<r;)(n=+e(t[i],i,t))&&(o+=n);return o},t.ticks=function(t,e,n){var r,i,o,a,s=-1;if(n=+n,(t=+t)==(e=+e)&&n>0)return[t];if((r=e<t)&&(i=t,t=e,e=i),0===(a=y(t,e,n))||!isFinite(a))return[];if(a>0)for(t=Math.ceil(t/a),e=Math.floor(e/a),o=new Array(i=Math.ceil(e-t+1));++s<i;)o[s]=(t+s)*a;else for(t=Math.floor(t*a),e=Math.ceil(e*a),o=new Array(i=Math.ceil(t-e+1));++s<i;)o[s]=(t-s)/a;return r&&o.reverse(),o},t.tickIncrement=y,t.tickStep=E,t.transpose=Q,t.variance=A,t.zip=function(){return Q(arguments)},Object.defineProperty(t,"__esModule",{value:!0})}("object"==typeof n&&void 0!==e?n:this.d3=this.d3||{})},{}],153:[function(t,e,n){!function(t){"use strict";function e(){}function n(t,n){var r=new e;if(t instanceof e)t.each(function(t,e){r.set(e,t)});else if(Array.isArray(t)){var i,o=-1,a=t.length;if(null==n)for(;++o<a;)r.set(o,t[o]);else for(;++o<a;)r.set(n(i=t[o],o,t),i)}else if(t)for(var s in t)r.set(s,t[s]);return r}function r(){return{}}function i(t,e,n){t[e]=n}function o(){return n()}function a(t,e,n){t.set(e,n)}function s(){}e.prototype=n.prototype={constructor:e,has:function(t){return"$"+t in this},get:function(t){return this["$"+t]},set:function(t,e){return this["$"+t]=e,this},remove:function(t){var e="$"+t;return e in this&&delete this[e]},clear:function(){for(var t in this)"$"===t[0]&&delete this[t]},keys:function(){var t=[];for(var e in this)"$"===e[0]&&t.push(e.slice(1));return t},values:function(){var t=[];for(var e in this)"$"===e[0]&&t.push(this[e]);return t},entries:function(){var t=[];for(var e in this)"$"===e[0]&&t.push({key:e.slice(1),value:this[e]});return t},size:function(){var t=0;for(var e in this)"$"===e[0]&&++t;return t},empty:function(){for(var t in this)if("$"===t[0])return!1;return!0},each:function(t){for(var e in this)"$"===e[0]&&t(this[e],e.slice(1),this)}};var A=n.prototype;function l(t,e){var n=new s;if(t instanceof s)t.each(function(t){n.add(t)});else if(t){var r=-1,i=t.length;if(null==e)for(;++r<i;)n.add(t[r]);else for(;++r<i;)n.add(e(t[r],r,t))}return n}s.prototype=l.prototype={constructor:s,has:A.has,add:function(t){return this["$"+(t+="")]=t,this},remove:A.remove,clear:A.clear,values:A.keys,size:A.size,empty:A.empty,each:A.each},t.nest=function(){var t,e,s,A=[],l=[];function c(r,i,o,a){if(i>=A.length)return null!=t&&r.sort(t),null!=e?e(r):r;for(var s,l,u,g=-1,h=r.length,d=A[i++],f=n(),p=o();++g<h;)(u=f.get(s=d(l=r[g])+""))?u.push(l):f.set(s,[l]);return f.each(function(t,e){a(p,e,c(t,i,o,a))}),p}return s={object:function(t){return c(t,0,r,i)},map:function(t){return c(t,0,o,a)},entries:function(t){return function t(n,r){if(++r>A.length)return n;var i,o=l[r-1];return null!=e&&r>=A.length?i=n.entries():(i=[],n.each(function(e,n){i.push({key:n,values:t(e,r)})})),null!=o?i.sort(function(t,e){return o(t.key,e.key)}):i}(c(t,0,o,a),0)},key:function(t){return A.push(t),s},sortKeys:function(t){return l[A.length-1]=t,s},sortValues:function(e){return t=e,s},rollup:function(t){return e=t,s}}},t.set=l,t.map=n,t.keys=function(t){var e=[];for(var n in t)e.push(n);return e},t.values=function(t){var e=[];for(var n in t)e.push(t[n]);return e},t.entries=function(t){var e=[];for(var n in t)e.push({key:n,value:t[n]});return e},Object.defineProperty(t,"__esModule",{value:!0})}("object"==typeof n&&void 0!==e?n:this.d3=this.d3||{})},{}],154:[function(t,e,n){!function(t){"use strict";function e(t,e,n){t.prototype=e.prototype=n,n.constructor=t}function n(t,e){var n=Object.create(t.prototype);for(var r in e)n[r]=e[r];return n}function r(){}var i="\\s*([+-]?\\d+)\\s*",o="\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*",a="\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*",s=/^#([0-9a-f]{3})$/,A=/^#([0-9a-f]{6})$/,l=new RegExp("^rgb\\("+[i,i,i]+"\\)$"),c=new RegExp("^rgb\\("+[a,a,a]+"\\)$"),u=new RegExp("^rgba\\("+[i,i,i,o]+"\\)$"),g=new RegExp("^rgba\\("+[a,a,a,o]+"\\)$"),h=new RegExp("^hsl\\("+[o,a,a]+"\\)$"),d=new RegExp("^hsla\\("+[o,a,a,o]+"\\)$"),f={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074};function p(t){var e;return t=(t+"").trim().toLowerCase(),(e=s.exec(t))?new E((e=parseInt(e[1],16))>>8&15|e>>4&240,e>>4&15|240&e,(15&e)<<4|15&e,1):(e=A.exec(t))?I(parseInt(e[1],16)):(e=l.exec(t))?new E(e[1],e[2],e[3],1):(e=c.exec(t))?new E(255*e[1]/100,255*e[2]/100,255*e[3]/100,1):(e=u.exec(t))?C(e[1],e[2],e[3],e[4]):(e=g.exec(t))?C(255*e[1]/100,255*e[2]/100,255*e[3]/100,e[4]):(e=h.exec(t))?B(e[1],e[2]/100,e[3]/100,1):(e=d.exec(t))?B(e[1],e[2]/100,e[3]/100,e[4]):f.hasOwnProperty(t)?I(f[t]):"transparent"===t?new E(NaN,NaN,NaN,0):null}function I(t){return new E(t>>16&255,t>>8&255,255&t,1)}function C(t,e,n,r){return r<=0&&(t=e=n=NaN),new E(t,e,n,r)}function m(t){return t instanceof r||(t=p(t)),t?new E((t=t.rgb()).r,t.g,t.b,t.opacity):new E}function y(t,e,n,r){return 1===arguments.length?m(t):new E(t,e,n,null==r?1:r)}function E(t,e,n,r){this.r=+t,this.g=+e,this.b=+n,this.opacity=+r}function v(t){return((t=Math.max(0,Math.min(255,Math.round(t)||0)))<16?"0":"")+t.toString(16)}function B(t,e,n,r){return r<=0?t=e=n=NaN:n<=0||n>=1?t=e=NaN:e<=0&&(t=NaN),new Q(t,e,n,r)}function b(t,e,n,i){return 1===arguments.length?function(t){if(t instanceof Q)return new Q(t.h,t.s,t.l,t.opacity);if(t instanceof r||(t=p(t)),!t)return new Q;if(t instanceof Q)return t;var e=(t=t.rgb()).r/255,n=t.g/255,i=t.b/255,o=Math.min(e,n,i),a=Math.max(e,n,i),s=NaN,A=a-o,l=(a+o)/2;return A?(s=e===a?(n-i)/A+6*(n<i):n===a?(i-e)/A+2:(e-n)/A+4,A/=l<.5?a+o:2-a-o,s*=60):A=l>0&&l<1?0:s,new Q(s,A,l,t.opacity)}(t):new Q(t,e,n,null==i?1:i)}function Q(t,e,n,r){this.h=+t,this.s=+e,this.l=+n,this.opacity=+r}function w(t,e,n){return 255*(t<60?e+(n-e)*t/60:t<180?n:t<240?e+(n-e)*(240-t)/60:e)}e(r,p,{displayable:function(){return this.rgb().displayable()},hex:function(){return this.rgb().hex()},toString:function(){return this.rgb()+""}}),e(E,y,n(r,{brighter:function(t){return t=null==t?1/.7:Math.pow(1/.7,t),new E(this.r*t,this.g*t,this.b*t,this.opacity)},darker:function(t){return t=null==t?.7:Math.pow(.7,t),new E(this.r*t,this.g*t,this.b*t,this.opacity)},rgb:function(){return this},displayable:function(){return 0<=this.r&&this.r<=255&&0<=this.g&&this.g<=255&&0<=this.b&&this.b<=255&&0<=this.opacity&&this.opacity<=1},hex:function(){return"#"+v(this.r)+v(this.g)+v(this.b)},toString:function(){var t=this.opacity;return(1===(t=isNaN(t)?1:Math.max(0,Math.min(1,t)))?"rgb(":"rgba(")+Math.max(0,Math.min(255,Math.round(this.r)||0))+", "+Math.max(0,Math.min(255,Math.round(this.g)||0))+", "+Math.max(0,Math.min(255,Math.round(this.b)||0))+(1===t?")":", "+t+")")}})),e(Q,b,n(r,{brighter:function(t){return t=null==t?1/.7:Math.pow(1/.7,t),new Q(this.h,this.s,this.l*t,this.opacity)},darker:function(t){return t=null==t?.7:Math.pow(.7,t),new Q(this.h,this.s,this.l*t,this.opacity)},rgb:function(){var t=this.h%360+360*(this.h<0),e=isNaN(t)||isNaN(this.s)?0:this.s,n=this.l,r=n+(n<.5?n:1-n)*e,i=2*n-r;return new E(w(t>=240?t-240:t+120,i,r),w(t,i,r),w(t<120?t+240:t-120,i,r),this.opacity)},displayable:function(){return(0<=this.s&&this.s<=1||isNaN(this.s))&&0<=this.l&&this.l<=1&&0<=this.opacity&&this.opacity<=1}}));var x=Math.PI/180,_=180/Math.PI,D=.96422,M=1,k=.82521,S=4/29,L=6/29,R=3*L*L,N=L*L*L;function F(t){if(t instanceof U)return new U(t.l,t.a,t.b,t.opacity);if(t instanceof K){if(isNaN(t.h))return new U(t.l,0,0,t.opacity);var e=t.h*x;return new U(t.l,Math.cos(e)*t.c,Math.sin(e)*t.c,t.opacity)}t instanceof E||(t=m(t));var n,r,i=j(t.r),o=j(t.g),a=j(t.b),s=O((.2225045*i+.7168786*o+.0606169*a)/M);return i===o&&o===a?n=r=s:(n=O((.4360747*i+.3850649*o+.1430804*a)/D),r=O((.0139322*i+.0971045*o+.7141733*a)/k)),new U(116*s-16,500*(n-s),200*(s-r),t.opacity)}function T(t,e,n,r){return 1===arguments.length?F(t):new U(t,e,n,null==r?1:r)}function U(t,e,n,r){this.l=+t,this.a=+e,this.b=+n,this.opacity=+r}function O(t){return t>N?Math.pow(t,1/3):t/R+S}function G(t){return t>L?t*t*t:R*(t-S)}function H(t){return 255*(t<=.0031308?12.92*t:1.055*Math.pow(t,1/2.4)-.055)}function j(t){return(t/=255)<=.04045?t/12.92:Math.pow((t+.055)/1.055,2.4)}function Y(t){if(t instanceof K)return new K(t.h,t.c,t.l,t.opacity);if(t instanceof U||(t=F(t)),0===t.a&&0===t.b)return new K(NaN,0,t.l,t.opacity);var e=Math.atan2(t.b,t.a)*_;return new K(e<0?e+360:e,Math.sqrt(t.a*t.a+t.b*t.b),t.l,t.opacity)}function P(t,e,n,r){return 1===arguments.length?Y(t):new K(t,e,n,null==r?1:r)}function K(t,e,n,r){this.h=+t,this.c=+e,this.l=+n,this.opacity=+r}e(U,T,n(r,{brighter:function(t){return new U(this.l+18*(null==t?1:t),this.a,this.b,this.opacity)},darker:function(t){return new U(this.l-18*(null==t?1:t),this.a,this.b,this.opacity)},rgb:function(){var t=(this.l+16)/116,e=isNaN(this.a)?t:t+this.a/500,n=isNaN(this.b)?t:t-this.b/200;return new E(H(3.1338561*(e=D*G(e))-1.6168667*(t=M*G(t))-.4906146*(n=k*G(n))),H(-.9787684*e+1.9161415*t+.033454*n),H(.0719453*e-.2289914*t+1.4052427*n),this.opacity)}})),e(K,P,n(r,{brighter:function(t){return new K(this.h,this.c,this.l+18*(null==t?1:t),this.opacity)},darker:function(t){return new K(this.h,this.c,this.l-18*(null==t?1:t),this.opacity)},rgb:function(){return F(this).rgb()}}));var z=-.14861,J=1.78277,q=-.29227,V=-.90649,W=1.97294,X=W*V,Z=W*J,$=J*q-V*z;function tt(t,e,n,r){return 1===arguments.length?function(t){if(t instanceof et)return new et(t.h,t.s,t.l,t.opacity);t instanceof E||(t=m(t));var e=t.r/255,n=t.g/255,r=t.b/255,i=($*r+X*e-Z*n)/($+X-Z),o=r-i,a=(W*(n-i)-q*o)/V,s=Math.sqrt(a*a+o*o)/(W*i*(1-i)),A=s?Math.atan2(a,o)*_-120:NaN;return new et(A<0?A+360:A,s,i,t.opacity)}(t):new et(t,e,n,null==r?1:r)}function et(t,e,n,r){this.h=+t,this.s=+e,this.l=+n,this.opacity=+r}e(et,tt,n(r,{brighter:function(t){return t=null==t?1/.7:Math.pow(1/.7,t),new et(this.h,this.s,this.l*t,this.opacity)},darker:function(t){return t=null==t?.7:Math.pow(.7,t),new et(this.h,this.s,this.l*t,this.opacity)},rgb:function(){var t=isNaN(this.h)?0:(this.h+120)*x,e=+this.l,n=isNaN(this.s)?0:this.s*e*(1-e),r=Math.cos(t),i=Math.sin(t);return new E(255*(e+n*(z*r+J*i)),255*(e+n*(q*r+V*i)),255*(e+n*(W*r)),this.opacity)}})),t.color=p,t.rgb=y,t.hsl=b,t.lab=T,t.hcl=P,t.lch=function(t,e,n,r){return 1===arguments.length?Y(t):new K(n,e,t,null==r?1:r)},t.gray=function(t,e){return new U(t,0,0,null==e?1:e)},t.cubehelix=tt,Object.defineProperty(t,"__esModule",{value:!0})}("object"==typeof n&&void 0!==e?n:this.d3=this.d3||{})},{}],155:[function(t,e,n){!function(t){"use strict";var e={value:function(){}};function n(){for(var t,e=0,n=arguments.length,i={};e<n;++e){if(!(t=arguments[e]+"")||t in i)throw new Error("illegal type: "+t);i[t]=[]}return new r(i)}function r(t){this._=t}function i(t,e){for(var n,r=0,i=t.length;r<i;++r)if((n=t[r]).name===e)return n.value}function o(t,n,r){for(var i=0,o=t.length;i<o;++i)if(t[i].name===n){t[i]=e,t=t.slice(0,i).concat(t.slice(i+1));break}return null!=r&&t.push({name:n,value:r}),t}r.prototype=n.prototype={constructor:r,on:function(t,e){var n,r,a=this._,s=(r=a,(t+"").trim().split(/^|\s+/).map(function(t){var e="",n=t.indexOf(".");if(n>=0&&(e=t.slice(n+1),t=t.slice(0,n)),t&&!r.hasOwnProperty(t))throw new Error("unknown type: "+t);return{type:t,name:e}})),A=-1,l=s.length;if(!(arguments.length<2)){if(null!=e&&"function"!=typeof e)throw new Error("invalid callback: "+e);for(;++A<l;)if(n=(t=s[A]).type)a[n]=o(a[n],t.name,e);else if(null==e)for(n in a)a[n]=o(a[n],t.name,null);return this}for(;++A<l;)if((n=(t=s[A]).type)&&(n=i(a[n],t.name)))return n},copy:function(){var t={},e=this._;for(var n in e)t[n]=e[n].slice();return new r(t)},call:function(t,e){if((n=arguments.length-2)>0)for(var n,r,i=new Array(n),o=0;o<n;++o)i[o]=arguments[o+2];if(!this._.hasOwnProperty(t))throw new Error("unknown type: "+t);for(o=0,n=(r=this._[t]).length;o<n;++o)r[o].value.apply(e,i)},apply:function(t,e,n){if(!this._.hasOwnProperty(t))throw new Error("unknown type: "+t);for(var r=this._[t],i=0,o=r.length;i<o;++i)r[i].value.apply(e,n)}},t.dispatch=n,Object.defineProperty(t,"__esModule",{value:!0})}("object"==typeof n&&void 0!==e?n:this.d3=this.d3||{})},{}],156:[function(t,e,n){var r,i;r=this,i=function(t,e,n,r,i){"use strict";var o=function(t){return function(){return t}},a=function(){return 1e-6*(Math.random()-.5)};function s(t){return t.x+t.vx}function A(t){return t.y+t.vy}function l(t){return t.index}function c(t,e){var n=t.get(e);if(!n)throw new Error("missing: "+e);return n}function u(t){return t.x}function g(t){return t.y}var h=10,d=Math.PI*(3-Math.sqrt(5));t.forceCenter=function(t,e){var n;function r(){var r,i,o=n.length,a=0,s=0;for(r=0;r<o;++r)a+=(i=n[r]).x,s+=i.y;for(a=a/o-t,s=s/o-e,r=0;r<o;++r)(i=n[r]).x-=a,i.y-=s}return null==t&&(t=0),null==e&&(e=0),r.initialize=function(t){n=t},r.x=function(e){return arguments.length?(t=+e,r):t},r.y=function(t){return arguments.length?(e=+t,r):e},r},t.forceCollide=function(t){var n,r,i=1,l=1;function c(){for(var t,o,c,g,h,d,f,p=n.length,I=0;I<l;++I)for(o=e.quadtree(n,s,A).visitAfter(u),t=0;t<p;++t)c=n[t],d=r[c.index],f=d*d,g=c.x+c.vx,h=c.y+c.vy,o.visit(C);function C(t,e,n,r,o){var s=t.data,A=t.r,l=d+A;if(!s)return e>g+l||r<g-l||n>h+l||o<h-l;if(s.index>c.index){var u=g-s.x-s.vx,p=h-s.y-s.vy,I=u*u+p*p;I<l*l&&(0===u&&(I+=(u=a())*u),0===p&&(I+=(p=a())*p),I=(l-(I=Math.sqrt(I)))/I*i,c.vx+=(u*=I)*(l=(A*=A)/(f+A)),c.vy+=(p*=I)*l,s.vx-=u*(l=1-l),s.vy-=p*l)}}}function u(t){if(t.data)return t.r=r[t.data.index];for(var e=t.r=0;e<4;++e)t[e]&&t[e].r>t.r&&(t.r=t[e].r)}function g(){if(n){var e,i,o=n.length;for(r=new Array(o),e=0;e<o;++e)i=n[e],r[i.index]=+t(i,e,n)}}return"function"!=typeof t&&(t=o(null==t?1:+t)),c.initialize=function(t){n=t,g()},c.iterations=function(t){return arguments.length?(l=+t,c):l},c.strength=function(t){return arguments.length?(i=+t,c):i},c.radius=function(e){return arguments.length?(t="function"==typeof e?e:o(+e),g(),c):t},c},t.forceLink=function(t){var e,r,i,s,A,u=l,g=function(t){return 1/Math.min(s[t.source.index],s[t.target.index])},h=o(30),d=1;function f(n){for(var i=0,o=t.length;i<d;++i)for(var s,l,c,u,g,h,f,p=0;p<o;++p)l=(s=t[p]).source,u=(c=s.target).x+c.vx-l.x-l.vx||a(),g=c.y+c.vy-l.y-l.vy||a(),u*=h=((h=Math.sqrt(u*u+g*g))-r[p])/h*n*e[p],g*=h,c.vx-=u*(f=A[p]),c.vy-=g*f,l.vx+=u*(f=1-f),l.vy+=g*f}function p(){if(i){var o,a,l=i.length,g=t.length,h=n.map(i,u);for(o=0,s=new Array(l);o<g;++o)(a=t[o]).index=o,"object"!=typeof a.source&&(a.source=c(h,a.source)),"object"!=typeof a.target&&(a.target=c(h,a.target)),s[a.source.index]=(s[a.source.index]||0)+1,s[a.target.index]=(s[a.target.index]||0)+1;for(o=0,A=new Array(g);o<g;++o)a=t[o],A[o]=s[a.source.index]/(s[a.source.index]+s[a.target.index]);e=new Array(g),I(),r=new Array(g),C()}}function I(){if(i)for(var n=0,r=t.length;n<r;++n)e[n]=+g(t[n],n,t)}function C(){if(i)for(var e=0,n=t.length;e<n;++e)r[e]=+h(t[e],e,t)}return null==t&&(t=[]),f.initialize=function(t){i=t,p()},f.links=function(e){return arguments.length?(t=e,p(),f):t},f.id=function(t){return arguments.length?(u=t,f):u},f.iterations=function(t){return arguments.length?(d=+t,f):d},f.strength=function(t){return arguments.length?(g="function"==typeof t?t:o(+t),I(),f):g},f.distance=function(t){return arguments.length?(h="function"==typeof t?t:o(+t),C(),f):h},f},t.forceManyBody=function(){var t,n,r,i,s=o(-30),A=1,l=1/0,c=.81;function h(i){var o,a=t.length,s=e.quadtree(t,u,g).visitAfter(f);for(r=i,o=0;o<a;++o)n=t[o],s.visit(p)}function d(){if(t){var e,n,r=t.length;for(i=new Array(r),e=0;e<r;++e)n=t[e],i[n.index]=+s(n,e,t)}}function f(t){var e,n,r,o,a,s=0,A=0;if(t.length){for(r=o=a=0;a<4;++a)(e=t[a])&&(n=Math.abs(e.value))&&(s+=e.value,A+=n,r+=n*e.x,o+=n*e.y);t.x=r/A,t.y=o/A}else{(e=t).x=e.data.x,e.y=e.data.y;do{s+=i[e.data.index]}while(e=e.next)}t.value=s}function p(t,e,o,s){if(!t.value)return!0;var u=t.x-n.x,g=t.y-n.y,h=s-e,d=u*u+g*g;if(h*h/c<d)return d<l&&(0===u&&(d+=(u=a())*u),0===g&&(d+=(g=a())*g),d<A&&(d=Math.sqrt(A*d)),n.vx+=u*t.value*r/d,n.vy+=g*t.value*r/d),!0;if(!(t.length||d>=l)){(t.data!==n||t.next)&&(0===u&&(d+=(u=a())*u),0===g&&(d+=(g=a())*g),d<A&&(d=Math.sqrt(A*d)));do{t.data!==n&&(h=i[t.data.index]*r/d,n.vx+=u*h,n.vy+=g*h)}while(t=t.next)}}return h.initialize=function(e){t=e,d()},h.strength=function(t){return arguments.length?(s="function"==typeof t?t:o(+t),d(),h):s},h.distanceMin=function(t){return arguments.length?(A=t*t,h):Math.sqrt(A)},h.distanceMax=function(t){return arguments.length?(l=t*t,h):Math.sqrt(l)},h.theta=function(t){return arguments.length?(c=t*t,h):Math.sqrt(c)},h},t.forceRadial=function(t,e,n){var r,i,a,s=o(.1);function A(t){for(var o=0,s=r.length;o<s;++o){var A=r[o],l=A.x-e||1e-6,c=A.y-n||1e-6,u=Math.sqrt(l*l+c*c),g=(a[o]-u)*i[o]*t/u;A.vx+=l*g,A.vy+=c*g}}function l(){if(r){var e,n=r.length;for(i=new Array(n),a=new Array(n),e=0;e<n;++e)a[e]=+t(r[e],e,r),i[e]=isNaN(a[e])?0:+s(r[e],e,r)}}return"function"!=typeof t&&(t=o(+t)),null==e&&(e=0),null==n&&(n=0),A.initialize=function(t){r=t,l()},A.strength=function(t){return arguments.length?(s="function"==typeof t?t:o(+t),l(),A):s},A.radius=function(e){return arguments.length?(t="function"==typeof e?e:o(+e),l(),A):t},A.x=function(t){return arguments.length?(e=+t,A):e},A.y=function(t){return arguments.length?(n=+t,A):n},A},t.forceSimulation=function(t){var e,o=1,a=.001,s=1-Math.pow(a,1/300),A=0,l=.6,c=n.map(),u=i.timer(f),g=r.dispatch("tick","end");function f(){p(),g.call("tick",e),o<a&&(u.stop(),g.call("end",e))}function p(){var e,n,r=t.length;for(o+=(A-o)*s,c.each(function(t){t(o)}),e=0;e<r;++e)null==(n=t[e]).fx?n.x+=n.vx*=l:(n.x=n.fx,n.vx=0),null==n.fy?n.y+=n.vy*=l:(n.y=n.fy,n.vy=0)}function I(){for(var e,n=0,r=t.length;n<r;++n){if((e=t[n]).index=n,isNaN(e.x)||isNaN(e.y)){var i=h*Math.sqrt(n),o=n*d;e.x=i*Math.cos(o),e.y=i*Math.sin(o)}(isNaN(e.vx)||isNaN(e.vy))&&(e.vx=e.vy=0)}}function C(e){return e.initialize&&e.initialize(t),e}return null==t&&(t=[]),I(),e={tick:p,restart:function(){return u.restart(f),e},stop:function(){return u.stop(),e},nodes:function(n){return arguments.length?(t=n,I(),c.each(C),e):t},alpha:function(t){return arguments.length?(o=+t,e):o},alphaMin:function(t){return arguments.length?(a=+t,e):a},alphaDecay:function(t){return arguments.length?(s=+t,e):+s},alphaTarget:function(t){return arguments.length?(A=+t,e):A},velocityDecay:function(t){return arguments.length?(l=1-t,e):1-l},force:function(t,n){return arguments.length>1?(null==n?c.remove(t):c.set(t,C(n)),e):c.get(t)},find:function(e,n,r){var i,o,a,s,A,l=0,c=t.length;for(null==r?r=1/0:r*=r,l=0;l<c;++l)(a=(i=e-(s=t[l]).x)*i+(o=n-s.y)*o)<r&&(A=s,r=a);return A},on:function(t,n){return arguments.length>1?(g.on(t,n),e):g.on(t)}}},t.forceX=function(t){var e,n,r,i=o(.1);function a(t){for(var i,o=0,a=e.length;o<a;++o)(i=e[o]).vx+=(r[o]-i.x)*n[o]*t}function s(){if(e){var o,a=e.length;for(n=new Array(a),r=new Array(a),o=0;o<a;++o)n[o]=isNaN(r[o]=+t(e[o],o,e))?0:+i(e[o],o,e)}}return"function"!=typeof t&&(t=o(null==t?0:+t)),a.initialize=function(t){e=t,s()},a.strength=function(t){return arguments.length?(i="function"==typeof t?t:o(+t),s(),a):i},a.x=function(e){return arguments.length?(t="function"==typeof e?e:o(+e),s(),a):t},a},t.forceY=function(t){var e,n,r,i=o(.1);function a(t){for(var i,o=0,a=e.length;o<a;++o)(i=e[o]).vy+=(r[o]-i.y)*n[o]*t}function s(){if(e){var o,a=e.length;for(n=new Array(a),r=new Array(a),o=0;o<a;++o)n[o]=isNaN(r[o]=+t(e[o],o,e))?0:+i(e[o],o,e)}}return"function"!=typeof t&&(t=o(null==t?0:+t)),a.initialize=function(t){e=t,s()},a.strength=function(t){return arguments.length?(i="function"==typeof t?t:o(+t),s(),a):i},a.y=function(e){return arguments.length?(t="function"==typeof e?e:o(+e),s(),a):t},a},Object.defineProperty(t,"__esModule",{value:!0})},"object"==typeof n&&void 0!==e?i(n,t("d3-quadtree"),t("d3-collection"),t("d3-dispatch"),t("d3-timer")):i(r.d3=r.d3||{},r.d3,r.d3,r.d3,r.d3)},{"d3-collection":153,"d3-dispatch":155,"d3-quadtree":160,"d3-timer":162}],157:[function(t,e,n){!function(t){"use strict";function e(t,e){return t.parent===e.parent?1:2}function n(t,e){return t+e.x}function r(t,e){return Math.max(t,e.y)}function i(t){var e=0,n=t.children,r=n&&n.length;if(r)for(;--r>=0;)e+=n[r].value;else e=1;t.value=e}function o(t,e){var n,r,i,o,s,c=new l(t),u=+t.value&&(c.value=t.value),g=[c];for(null==e&&(e=a);n=g.pop();)if(u&&(n.value=+n.data.value),(i=e(n.data))&&(s=i.length))for(n.children=new Array(s),o=s-1;o>=0;--o)g.push(r=n.children[o]=new l(i[o])),r.parent=n,r.depth=n.depth+1;return c.eachBefore(A)}function a(t){return t.children}function s(t){t.data=t.data.data}function A(t){var e=0;do{t.height=e}while((t=t.parent)&&t.height<++e)}function l(t){this.data=t,this.depth=this.height=0,this.parent=null}l.prototype=o.prototype={constructor:l,count:function(){return this.eachAfter(i)},each:function(t){var e,n,r,i,o=this,a=[o];do{for(e=a.reverse(),a=[];o=e.pop();)if(t(o),n=o.children)for(r=0,i=n.length;r<i;++r)a.push(n[r])}while(a.length);return this},eachAfter:function(t){for(var e,n,r,i=this,o=[i],a=[];i=o.pop();)if(a.push(i),e=i.children)for(n=0,r=e.length;n<r;++n)o.push(e[n]);for(;i=a.pop();)t(i);return this},eachBefore:function(t){for(var e,n,r=this,i=[r];r=i.pop();)if(t(r),e=r.children)for(n=e.length-1;n>=0;--n)i.push(e[n]);return this},sum:function(t){return this.eachAfter(function(e){for(var n=+t(e.data)||0,r=e.children,i=r&&r.length;--i>=0;)n+=r[i].value;e.value=n})},sort:function(t){return this.eachBefore(function(e){e.children&&e.children.sort(t)})},path:function(t){for(var e=this,n=function(t,e){if(t===e)return t;var n=t.ancestors(),r=e.ancestors(),i=null;for(t=n.pop(),e=r.pop();t===e;)i=t,t=n.pop(),e=r.pop();return i}(e,t),r=[e];e!==n;)e=e.parent,r.push(e);for(var i=r.length;t!==n;)r.splice(i,0,t),t=t.parent;return r},ancestors:function(){for(var t=this,e=[t];t=t.parent;)e.push(t);return e},descendants:function(){var t=[];return this.each(function(e){t.push(e)}),t},leaves:function(){var t=[];return this.eachBefore(function(e){e.children||t.push(e)}),t},links:function(){var t=this,e=[];return t.each(function(n){n!==t&&e.push({source:n.parent,target:n})}),e},copy:function(){return o(this).eachBefore(s)}};var c=Array.prototype.slice;function u(t){for(var e,n,r=0,i=(t=function(t){for(var e,n,r=t.length;r;)n=Math.random()*r--|0,e=t[r],t[r]=t[n],t[n]=e;return t}(c.call(t))).length,o=[];r<i;)e=t[r],n&&d(n,e)?++r:(n=p(o=g(o,e)),r=0);return n}function g(t,e){var n,r;if(f(e,t))return[e];for(n=0;n<t.length;++n)if(h(e,t[n])&&f(I(t[n],e),t))return[t[n],e];for(n=0;n<t.length-1;++n)for(r=n+1;r<t.length;++r)if(h(I(t[n],t[r]),e)&&h(I(t[n],e),t[r])&&h(I(t[r],e),t[n])&&f(C(t[n],t[r],e),t))return[t[n],t[r],e];throw new Error}function h(t,e){var n=t.r-e.r,r=e.x-t.x,i=e.y-t.y;return n<0||n*n<r*r+i*i}function d(t,e){var n=t.r-e.r+1e-6,r=e.x-t.x,i=e.y-t.y;return n>0&&n*n>r*r+i*i}function f(t,e){for(var n=0;n<e.length;++n)if(!d(t,e[n]))return!1;return!0}function p(t){switch(t.length){case 1:return{x:(e=t[0]).x,y:e.y,r:e.r};case 2:return I(t[0],t[1]);case 3:return C(t[0],t[1],t[2])}var e}function I(t,e){var n=t.x,r=t.y,i=t.r,o=e.x,a=e.y,s=e.r,A=o-n,l=a-r,c=s-i,u=Math.sqrt(A*A+l*l);return{x:(n+o+A/u*c)/2,y:(r+a+l/u*c)/2,r:(u+i+s)/2}}function C(t,e,n){var r=t.x,i=t.y,o=t.r,a=e.x,s=e.y,A=e.r,l=n.x,c=n.y,u=n.r,g=r-a,h=r-l,d=i-s,f=i-c,p=A-o,I=u-o,C=r*r+i*i-o*o,m=C-a*a-s*s+A*A,y=C-l*l-c*c+u*u,E=h*d-g*f,v=(d*y-f*m)/(2*E)-r,B=(f*p-d*I)/E,b=(h*m-g*y)/(2*E)-i,Q=(g*I-h*p)/E,w=B*B+Q*Q-1,x=2*(o+v*B+b*Q),_=v*v+b*b-o*o,D=-(w?(x+Math.sqrt(x*x-4*w*_))/(2*w):_/x);return{x:r+v+B*D,y:i+b+Q*D,r:D}}function m(t,e,n){var r,i,o,a,s=t.x-e.x,A=t.y-e.y,l=s*s+A*A;l?(i=e.r+n.r,i*=i,a=t.r+n.r,i>(a*=a)?(r=(l+a-i)/(2*l),o=Math.sqrt(Math.max(0,a/l-r*r)),n.x=t.x-r*s-o*A,n.y=t.y-r*A+o*s):(r=(l+i-a)/(2*l),o=Math.sqrt(Math.max(0,i/l-r*r)),n.x=e.x+r*s-o*A,n.y=e.y+r*A+o*s)):(n.x=e.x+n.r,n.y=e.y)}function y(t,e){var n=t.r+e.r-1e-6,r=e.x-t.x,i=e.y-t.y;return n>0&&n*n>r*r+i*i}function E(t){var e=t._,n=t.next._,r=e.r+n.r,i=(e.x*n.r+n.x*e.r)/r,o=(e.y*n.r+n.y*e.r)/r;return i*i+o*o}function v(t){this._=t,this.next=null,this.previous=null}function B(t){if(!(i=t.length))return 0;var e,n,r,i,o,a,s,A,l,c,g;if((e=t[0]).x=0,e.y=0,!(i>1))return e.r;if(n=t[1],e.x=-n.r,n.x=e.r,n.y=0,!(i>2))return e.r+n.r;m(n,e,r=t[2]),e=new v(e),n=new v(n),r=new v(r),e.next=r.previous=n,n.next=e.previous=r,r.next=n.previous=e;t:for(s=3;s<i;++s){m(e._,n._,r=t[s]),r=new v(r),A=n.next,l=e.previous,c=n._.r,g=e._.r;do{if(c<=g){if(y(A._,r._)){n=A,e.next=n,n.previous=e,--s;continue t}c+=A._.r,A=A.next}else{if(y(l._,r._)){(e=l).next=n,n.previous=e,--s;continue t}g+=l._.r,l=l.previous}}while(A!==l.next);for(r.previous=e,r.next=n,e.next=n.previous=n=r,o=E(e);(r=r.next)!==n;)(a=E(r))<o&&(e=r,o=a);n=e.next}for(e=[n._],r=n;(r=r.next)!==n;)e.push(r._);for(r=u(e),s=0;s<i;++s)(e=t[s]).x-=r.x,e.y-=r.y;return r.r}function b(t){if("function"!=typeof t)throw new Error;return t}function Q(){return 0}function w(t){return function(){return t}}function x(t){return Math.sqrt(t.value)}function _(t){return function(e){e.children||(e.r=Math.max(0,+t(e)||0))}}function D(t,e){return function(n){if(r=n.children){var r,i,o,a=r.length,s=t(n)*e||0;if(s)for(i=0;i<a;++i)r[i].r+=s;if(o=B(r),s)for(i=0;i<a;++i)r[i].r-=s;n.r=o+s}}}function M(t){return function(e){var n=e.parent;e.r*=t,n&&(e.x=n.x+t*e.x,e.y=n.y+t*e.y)}}function k(t){t.x0=Math.round(t.x0),t.y0=Math.round(t.y0),t.x1=Math.round(t.x1),t.y1=Math.round(t.y1)}function S(t,e,n,r,i){for(var o,a=t.children,s=-1,A=a.length,l=t.value&&(r-e)/t.value;++s<A;)(o=a[s]).y0=n,o.y1=i,o.x0=e,o.x1=e+=o.value*l}var L="$",R={depth:-1},N={};function F(t){return t.id}function T(t){return t.parentId}function U(t,e){return t.parent===e.parent?1:2}function O(t){var e=t.children;return e?e[0]:t.t}function G(t){var e=t.children;return e?e[e.length-1]:t.t}function H(t,e,n){var r=n/(e.i-t.i);e.c-=r,e.s+=n,t.c+=r,e.z+=n,e.m+=n}function j(t,e,n){return t.a.parent===e.parent?t.a:n}function Y(t,e){this._=t,this.parent=null,this.children=null,this.A=null,this.a=this,this.z=0,this.m=0,this.c=0,this.s=0,this.t=null,this.i=e}function P(t,e,n,r,i){for(var o,a=t.children,s=-1,A=a.length,l=t.value&&(i-n)/t.value;++s<A;)(o=a[s]).x0=e,o.x1=r,o.y0=n,o.y1=n+=o.value*l}Y.prototype=Object.create(l.prototype);var K=(1+Math.sqrt(5))/2;function z(t,e,n,r,i,o){for(var a,s,A,l,c,u,g,h,d,f,p,I=[],C=e.children,m=0,y=0,E=C.length,v=e.value;m<E;){A=i-n,l=o-r;do{c=C[y++].value}while(!c&&y<E);for(u=g=c,p=c*c*(f=Math.max(l/A,A/l)/(v*t)),d=Math.max(g/p,p/u);y<E;++y){if(c+=s=C[y].value,s<u&&(u=s),s>g&&(g=s),p=c*c*f,(h=Math.max(g/p,p/u))>d){c-=s;break}d=h}I.push(a={value:c,dice:A<l,children:C.slice(m,y)}),a.dice?S(a,n,r,i,v?r+=l*c/v:o):P(a,n,r,v?n+=A*c/v:i,o),v-=c,m=y}return I}var J=function t(e){function n(t,n,r,i,o){z(e,t,n,r,i,o)}return n.ratio=function(e){return t((e=+e)>1?e:1)},n}(K),q=function t(e){function n(t,n,r,i,o){if((a=t._squarify)&&a.ratio===e)for(var a,s,A,l,c,u=-1,g=a.length,h=t.value;++u<g;){for(A=(s=a[u]).children,l=s.value=0,c=A.length;l<c;++l)s.value+=A[l].value;s.dice?S(s,n,r,i,r+=(o-r)*s.value/h):P(s,n,r,n+=(i-n)*s.value/h,o),h-=s.value}else t._squarify=a=z(e,t,n,r,i,o),a.ratio=e}return n.ratio=function(e){return t((e=+e)>1?e:1)},n}(K);t.cluster=function(){var t=e,i=1,o=1,a=!1;function s(e){var s,A=0;e.eachAfter(function(e){var i=e.children;i?(e.x=function(t){return t.reduce(n,0)/t.length}(i),e.y=function(t){return 1+t.reduce(r,0)}(i)):(e.x=s?A+=t(e,s):0,e.y=0,s=e)});var l=function(t){for(var e;e=t.children;)t=e[0];return t}(e),c=function(t){for(var e;e=t.children;)t=e[e.length-1];return t}(e),u=l.x-t(l,c)/2,g=c.x+t(c,l)/2;return e.eachAfter(a?function(t){t.x=(t.x-e.x)*i,t.y=(e.y-t.y)*o}:function(t){t.x=(t.x-u)/(g-u)*i,t.y=(1-(e.y?t.y/e.y:1))*o})}return s.separation=function(e){return arguments.length?(t=e,s):t},s.size=function(t){return arguments.length?(a=!1,i=+t[0],o=+t[1],s):a?null:[i,o]},s.nodeSize=function(t){return arguments.length?(a=!0,i=+t[0],o=+t[1],s):a?[i,o]:null},s},t.hierarchy=o,t.pack=function(){var t=null,e=1,n=1,r=Q;function i(i){return i.x=e/2,i.y=n/2,t?i.eachBefore(_(t)).eachAfter(D(r,.5)).eachBefore(M(1)):i.eachBefore(_(x)).eachAfter(D(Q,1)).eachAfter(D(r,i.r/Math.min(e,n))).eachBefore(M(Math.min(e,n)/(2*i.r))),i}return i.radius=function(e){return arguments.length?(t=null==(n=e)?null:b(n),i):t;var n},i.size=function(t){return arguments.length?(e=+t[0],n=+t[1],i):[e,n]},i.padding=function(t){return arguments.length?(r="function"==typeof t?t:w(+t),i):r},i},t.packSiblings=function(t){return B(t),t},t.packEnclose=u,t.partition=function(){var t=1,e=1,n=0,r=!1;function i(i){var o=i.height+1;return i.x0=i.y0=n,i.x1=t,i.y1=e/o,i.eachBefore(function(t,e){return function(r){r.children&&S(r,r.x0,t*(r.depth+1)/e,r.x1,t*(r.depth+2)/e);var i=r.x0,o=r.y0,a=r.x1-n,s=r.y1-n;a<i&&(i=a=(i+a)/2),s<o&&(o=s=(o+s)/2),r.x0=i,r.y0=o,r.x1=a,r.y1=s}}(e,o)),r&&i.eachBefore(k),i}return i.round=function(t){return arguments.length?(r=!!t,i):r},i.size=function(n){return arguments.length?(t=+n[0],e=+n[1],i):[t,e]},i.padding=function(t){return arguments.length?(n=+t,i):n},i},t.stratify=function(){var t=F,e=T;function n(n){var r,i,o,a,s,c,u,g=n.length,h=new Array(g),d={};for(i=0;i<g;++i)r=n[i],s=h[i]=new l(r),null!=(c=t(r,i,n))&&(c+="")&&(d[u=L+(s.id=c)]=u in d?N:s);for(i=0;i<g;++i)if(s=h[i],null!=(c=e(n[i],i,n))&&(c+="")){if(!(a=d[L+c]))throw new Error("missing: "+c);if(a===N)throw new Error("ambiguous: "+c);a.children?a.children.push(s):a.children=[s],s.parent=a}else{if(o)throw new Error("multiple roots");o=s}if(!o)throw new Error("no root");if(o.parent=R,o.eachBefore(function(t){t.depth=t.parent.depth+1,--g}).eachBefore(A),o.parent=null,g>0)throw new Error("cycle");return o}return n.id=function(e){return arguments.length?(t=b(e),n):t},n.parentId=function(t){return arguments.length?(e=b(t),n):e},n},t.tree=function(){var t=U,e=1,n=1,r=null;function i(i){var A=function(t){for(var e,n,r,i,o,a=new Y(t,0),s=[a];e=s.pop();)if(r=e._.children)for(e.children=new Array(o=r.length),i=o-1;i>=0;--i)s.push(n=e.children[i]=new Y(r[i],i)),n.parent=e;return(a.parent=new Y(null,0)).children=[a],a}(i);if(A.eachAfter(o),A.parent.m=-A.z,A.eachBefore(a),r)i.eachBefore(s);else{var l=i,c=i,u=i;i.eachBefore(function(t){t.x<l.x&&(l=t),t.x>c.x&&(c=t),t.depth>u.depth&&(u=t)});var g=l===c?1:t(l,c)/2,h=g-l.x,d=e/(c.x+g+h),f=n/(u.depth||1);i.eachBefore(function(t){t.x=(t.x+h)*d,t.y=t.depth*f})}return i}function o(e){var n=e.children,r=e.parent.children,i=e.i?r[e.i-1]:null;if(n){!function(t){for(var e,n=0,r=0,i=t.children,o=i.length;--o>=0;)(e=i[o]).z+=n,e.m+=n,n+=e.s+(r+=e.c)}(e);var o=(n[0].z+n[n.length-1].z)/2;i?(e.z=i.z+t(e._,i._),e.m=e.z-o):e.z=o}else i&&(e.z=i.z+t(e._,i._));e.parent.A=function(e,n,r){if(n){for(var i,o=e,a=e,s=n,A=o.parent.children[0],l=o.m,c=a.m,u=s.m,g=A.m;s=G(s),o=O(o),s&&o;)A=O(A),(a=G(a)).a=e,(i=s.z+u-o.z-l+t(s._,o._))>0&&(H(j(s,e,r),e,i),l+=i,c+=i),u+=s.m,l+=o.m,g+=A.m,c+=a.m;s&&!G(a)&&(a.t=s,a.m+=u-c),o&&!O(A)&&(A.t=o,A.m+=l-g,r=e)}return r}(e,i,e.parent.A||r[0])}function a(t){t._.x=t.z+t.parent.m,t.m+=t.parent.m}function s(t){t.x*=e,t.y=t.depth*n}return i.separation=function(e){return arguments.length?(t=e,i):t},i.size=function(t){return arguments.length?(r=!1,e=+t[0],n=+t[1],i):r?null:[e,n]},i.nodeSize=function(t){return arguments.length?(r=!0,e=+t[0],n=+t[1],i):r?[e,n]:null},i},t.treemap=function(){var t=J,e=!1,n=1,r=1,i=[0],o=Q,a=Q,s=Q,A=Q,l=Q;function c(t){return t.x0=t.y0=0,t.x1=n,t.y1=r,t.eachBefore(u),i=[0],e&&t.eachBefore(k),t}function u(e){var n=i[e.depth],r=e.x0+n,c=e.y0+n,u=e.x1-n,g=e.y1-n;u<r&&(r=u=(r+u)/2),g<c&&(c=g=(c+g)/2),e.x0=r,e.y0=c,e.x1=u,e.y1=g,e.children&&(n=i[e.depth+1]=o(e)/2,r+=l(e)-n,c+=a(e)-n,(u-=s(e)-n)<r&&(r=u=(r+u)/2),(g-=A(e)-n)<c&&(c=g=(c+g)/2),t(e,r,c,u,g))}return c.round=function(t){return arguments.length?(e=!!t,c):e},c.size=function(t){return arguments.length?(n=+t[0],r=+t[1],c):[n,r]},c.tile=function(e){return arguments.length?(t=b(e),c):t},c.padding=function(t){return arguments.length?c.paddingInner(t).paddingOuter(t):c.paddingInner()},c.paddingInner=function(t){return arguments.length?(o="function"==typeof t?t:w(+t),c):o},c.paddingOuter=function(t){return arguments.length?c.paddingTop(t).paddingRight(t).paddingBottom(t).paddingLeft(t):c.paddingTop()},c.paddingTop=function(t){return arguments.length?(a="function"==typeof t?t:w(+t),c):a},c.paddingRight=function(t){return arguments.length?(s="function"==typeof t?t:w(+t),c):s},c.paddingBottom=function(t){return arguments.length?(A="function"==typeof t?t:w(+t),c):A},c.paddingLeft=function(t){return arguments.length?(l="function"==typeof t?t:w(+t),c):l},c},t.treemapBinary=function(t,e,n,r,i){var o,a,s=t.children,A=s.length,l=new Array(A+1);for(l[0]=a=o=0;o<A;++o)l[o+1]=a+=s[o].value;!function t(e,n,r,i,o,a,A){if(e>=n-1){var c=s[e];return c.x0=i,c.y0=o,c.x1=a,void(c.y1=A)}for(var u=l[e],g=r/2+u,h=e+1,d=n-1;h<d;){var f=h+d>>>1;l[f]<g?h=f+1:d=f}g-l[h-1]<l[h]-g&&e+1<h&&--h;var p=l[h]-u,I=r-p;if(a-i>A-o){var C=(i*I+a*p)/r;t(e,h,p,i,o,C,A),t(h,n,I,C,o,a,A)}else{var m=(o*I+A*p)/r;t(e,h,p,i,o,a,m),t(h,n,I,i,m,a,A)}}(0,A,t.value,e,n,r,i)},t.treemapDice=S,t.treemapSlice=P,t.treemapSliceDice=function(t,e,n,r,i){(1&t.depth?P:S)(t,e,n,r,i)},t.treemapSquarify=J,t.treemapResquarify=q,Object.defineProperty(t,"__esModule",{value:!0})}("object"==typeof n&&void 0!==e?n:this.d3=this.d3||{})},{}],158:[function(t,e,n){var r,i;r=this,i=function(t,e){"use strict";function n(t,e,n,r,i){var o=t*t,a=o*t;return((1-3*t+3*o-a)*e+(4-6*o+3*a)*n+(1+3*t+3*o-3*a)*r+a*i)/6}function r(t){var e=t.length-1;return function(r){var i=r<=0?r=0:r>=1?(r=1,e-1):Math.floor(r*e),o=t[i],a=t[i+1],s=i>0?t[i-1]:2*o-a,A=i<e-1?t[i+2]:2*a-o;return n((r-i/e)*e,s,o,a,A)}}function i(t){var e=t.length;return function(r){var i=Math.floor(((r%=1)<0?++r:r)*e),o=t[(i+e-1)%e],a=t[i%e],s=t[(i+1)%e],A=t[(i+2)%e];return n((r-i/e)*e,o,a,s,A)}}function o(t){return function(){return t}}function a(t,e){return function(n){return t+n*e}}function s(t,e){var n=e-t;return n?a(t,n>180||n<-180?n-360*Math.round(n/360):n):o(isNaN(t)?e:t)}function A(t,e){var n=e-t;return n?a(t,n):o(isNaN(t)?e:t)}var l=function t(n){var r=function(t){return 1==(t=+t)?A:function(e,n){return n-e?function(t,e,n){return t=Math.pow(t,n),e=Math.pow(e,n)-t,n=1/n,function(r){return Math.pow(t+r*e,n)}}(e,n,t):o(isNaN(e)?n:e)}}(n);function i(t,n){var i=r((t=e.rgb(t)).r,(n=e.rgb(n)).r),o=r(t.g,n.g),a=r(t.b,n.b),s=A(t.opacity,n.opacity);return function(e){return t.r=i(e),t.g=o(e),t.b=a(e),t.opacity=s(e),t+""}}return i.gamma=t,i}(1);function c(t){return function(n){var r,i,o=n.length,a=new Array(o),s=new Array(o),A=new Array(o);for(r=0;r<o;++r)i=e.rgb(n[r]),a[r]=i.r||0,s[r]=i.g||0,A[r]=i.b||0;return a=t(a),s=t(s),A=t(A),i.opacity=1,function(t){return i.r=a(t),i.g=s(t),i.b=A(t),i+""}}}var u=c(r),g=c(i);function h(t,e){var n,r=e?e.length:0,i=t?Math.min(r,t.length):0,o=new Array(i),a=new Array(r);for(n=0;n<i;++n)o[n]=y(t[n],e[n]);for(;n<r;++n)a[n]=e[n];return function(t){for(n=0;n<i;++n)a[n]=o[n](t);return a}}function d(t,e){var n=new Date;return e-=t=+t,function(r){return n.setTime(t+e*r),n}}function f(t,e){return e-=t=+t,function(n){return t+e*n}}function p(t,e){var n,r={},i={};for(n in null!==t&&"object"==typeof t||(t={}),null!==e&&"object"==typeof e||(e={}),e)n in t?r[n]=y(t[n],e[n]):i[n]=e[n];return function(t){for(n in r)i[n]=r[n](t);return i}}var I=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,C=new RegExp(I.source,"g");function m(t,e){var n,r,i,o=I.lastIndex=C.lastIndex=0,a=-1,s=[],A=[];for(t+="",e+="";(n=I.exec(t))&&(r=C.exec(e));)(i=r.index)>o&&(i=e.slice(o,i),s[a]?s[a]+=i:s[++a]=i),(n=n[0])===(r=r[0])?s[a]?s[a]+=r:s[++a]=r:(s[++a]=null,A.push({i:a,x:f(n,r)})),o=C.lastIndex;return o<e.length&&(i=e.slice(o),s[a]?s[a]+=i:s[++a]=i),s.length<2?A[0]?function(t){return function(e){return t(e)+""}}(A[0].x):function(t){return function(){return t}}(e):(e=A.length,function(t){for(var n,r=0;r<e;++r)s[(n=A[r]).i]=n.x(t);return s.join("")})}function y(t,n){var r,i=typeof n;return null==n||"boolean"===i?o(n):("number"===i?f:"string"===i?(r=e.color(n))?(n=r,l):m:n instanceof e.color?l:n instanceof Date?d:Array.isArray(n)?h:"function"!=typeof n.valueOf&&"function"!=typeof n.toString||isNaN(n)?p:f)(t,n)}var E,v,B,b,Q=180/Math.PI,w={translateX:0,translateY:0,rotate:0,skewX:0,scaleX:1,scaleY:1};function x(t,e,n,r,i,o){var a,s,A;return(a=Math.sqrt(t*t+e*e))&&(t/=a,e/=a),(A=t*n+e*r)&&(n-=t*A,r-=e*A),(s=Math.sqrt(n*n+r*r))&&(n/=s,r/=s,A/=s),t*r<e*n&&(t=-t,e=-e,A=-A,a=-a),{translateX:i,translateY:o,rotate:Math.atan2(e,t)*Q,skewX:Math.atan(A)*Q,scaleX:a,scaleY:s}}function _(t,e,n,r){function i(t){return t.length?t.pop()+" ":""}return function(o,a){var s=[],A=[];return o=t(o),a=t(a),function(t,r,i,o,a,s){if(t!==i||r!==o){var A=a.push("translate(",null,e,null,n);s.push({i:A-4,x:f(t,i)},{i:A-2,x:f(r,o)})}else(i||o)&&a.push("translate("+i+e+o+n)}(o.translateX,o.translateY,a.translateX,a.translateY,s,A),function(t,e,n,o){t!==e?(t-e>180?e+=360:e-t>180&&(t+=360),o.push({i:n.push(i(n)+"rotate(",null,r)-2,x:f(t,e)})):e&&n.push(i(n)+"rotate("+e+r)}(o.rotate,a.rotate,s,A),function(t,e,n,o){t!==e?o.push({i:n.push(i(n)+"skewX(",null,r)-2,x:f(t,e)}):e&&n.push(i(n)+"skewX("+e+r)}(o.skewX,a.skewX,s,A),function(t,e,n,r,o,a){if(t!==n||e!==r){var s=o.push(i(o)+"scale(",null,",",null,")");a.push({i:s-4,x:f(t,n)},{i:s-2,x:f(e,r)})}else 1===n&&1===r||o.push(i(o)+"scale("+n+","+r+")")}(o.scaleX,o.scaleY,a.scaleX,a.scaleY,s,A),o=a=null,function(t){for(var e,n=-1,r=A.length;++n<r;)s[(e=A[n]).i]=e.x(t);return s.join("")}}}var D=_(function(t){return"none"===t?w:(E||(E=document.createElement("DIV"),v=document.documentElement,B=document.defaultView),E.style.transform=t,t=B.getComputedStyle(v.appendChild(E),null).getPropertyValue("transform"),v.removeChild(E),x(+(t=t.slice(7,-1).split(","))[0],+t[1],+t[2],+t[3],+t[4],+t[5]))},"px, ","px)","deg)"),M=_(function(t){return null==t?w:(b||(b=document.createElementNS("http://www.w3.org/2000/svg","g")),b.setAttribute("transform",t),(t=b.transform.baseVal.consolidate())?x((t=t.matrix).a,t.b,t.c,t.d,t.e,t.f):w)},", ",")",")"),k=Math.SQRT2;function S(t){return((t=Math.exp(t))+1/t)/2}function L(t){return function(n,r){var i=t((n=e.hsl(n)).h,(r=e.hsl(r)).h),o=A(n.s,r.s),a=A(n.l,r.l),s=A(n.opacity,r.opacity);return function(t){return n.h=i(t),n.s=o(t),n.l=a(t),n.opacity=s(t),n+""}}}var R=L(s),N=L(A);function F(t){return function(n,r){var i=t((n=e.hcl(n)).h,(r=e.hcl(r)).h),o=A(n.c,r.c),a=A(n.l,r.l),s=A(n.opacity,r.opacity);return function(t){return n.h=i(t),n.c=o(t),n.l=a(t),n.opacity=s(t),n+""}}}var T=F(s),U=F(A);function O(t){return function n(r){function i(n,i){var o=t((n=e.cubehelix(n)).h,(i=e.cubehelix(i)).h),a=A(n.s,i.s),s=A(n.l,i.l),l=A(n.opacity,i.opacity);return function(t){return n.h=o(t),n.s=a(t),n.l=s(Math.pow(t,r)),n.opacity=l(t),n+""}}return r=+r,i.gamma=n,i}(1)}var G=O(s),H=O(A);t.interpolate=y,t.interpolateArray=h,t.interpolateBasis=r,t.interpolateBasisClosed=i,t.interpolateDate=d,t.interpolateDiscrete=function(t){var e=t.length;return function(n){return t[Math.max(0,Math.min(e-1,Math.floor(n*e)))]}},t.interpolateHue=function(t,e){var n=s(+t,+e);return function(t){var e=n(t);return e-360*Math.floor(e/360)}},t.interpolateNumber=f,t.interpolateObject=p,t.interpolateRound=function(t,e){return e-=t=+t,function(n){return Math.round(t+e*n)}},t.interpolateString=m,t.interpolateTransformCss=D,t.interpolateTransformSvg=M,t.interpolateZoom=function(t,e){var n,r,i=t[0],o=t[1],a=t[2],s=e[0],A=e[1],l=e[2],c=s-i,u=A-o,g=c*c+u*u;if(g<1e-12)r=Math.log(l/a)/k,n=function(t){return[i+t*c,o+t*u,a*Math.exp(k*t*r)]};else{var h=Math.sqrt(g),d=(l*l-a*a+4*g)/(2*a*2*h),f=(l*l-a*a-4*g)/(2*l*2*h),p=Math.log(Math.sqrt(d*d+1)-d),I=Math.log(Math.sqrt(f*f+1)-f);r=(I-p)/k,n=function(t){var e,n=t*r,s=S(p),A=a/(2*h)*(s*(e=k*n+p,((e=Math.exp(2*e))-1)/(e+1))-function(t){return((t=Math.exp(t))-1/t)/2}(p));return[i+A*c,o+A*u,a*s/S(k*n+p)]}}return n.duration=1e3*r,n},t.interpolateRgb=l,t.interpolateRgbBasis=u,t.interpolateRgbBasisClosed=g,t.interpolateHsl=R,t.interpolateHslLong=N,t.interpolateLab=function(t,n){var r=A((t=e.lab(t)).l,(n=e.lab(n)).l),i=A(t.a,n.a),o=A(t.b,n.b),a=A(t.opacity,n.opacity);return function(e){return t.l=r(e),t.a=i(e),t.b=o(e),t.opacity=a(e),t+""}},t.interpolateHcl=T,t.interpolateHclLong=U,t.interpolateCubehelix=G,t.interpolateCubehelixLong=H,t.piecewise=function(t,e){for(var n=0,r=e.length-1,i=e[0],o=new Array(r<0?0:r);n<r;)o[n]=t(i,i=e[++n]);return function(t){var e=Math.max(0,Math.min(r-1,Math.floor(t*=r)));return o[e](t-e)}},t.quantize=function(t,e){for(var n=new Array(e),r=0;r<e;++r)n[r]=t(r/(e-1));return n},Object.defineProperty(t,"__esModule",{value:!0})},"object"==typeof n&&void 0!==e?i(n,t("d3-color")):i(r.d3=r.d3||{},r.d3)},{"d3-color":154}],159:[function(t,e,n){!function(t){"use strict";var e=Math.PI,n=2*e,r=n-1e-6;function i(){this._x0=this._y0=this._x1=this._y1=null,this._=""}function o(){return new i}i.prototype=o.prototype={constructor:i,moveTo:function(t,e){this._+="M"+(this._x0=this._x1=+t)+","+(this._y0=this._y1=+e)},closePath:function(){null!==this._x1&&(this._x1=this._x0,this._y1=this._y0,this._+="Z")},lineTo:function(t,e){this._+="L"+(this._x1=+t)+","+(this._y1=+e)},quadraticCurveTo:function(t,e,n,r){this._+="Q"+ +t+","+ +e+","+(this._x1=+n)+","+(this._y1=+r)},bezierCurveTo:function(t,e,n,r,i,o){this._+="C"+ +t+","+ +e+","+ +n+","+ +r+","+(this._x1=+i)+","+(this._y1=+o)},arcTo:function(t,n,r,i,o){t=+t,n=+n,r=+r,i=+i,o=+o;var a=this._x1,s=this._y1,A=r-t,l=i-n,c=a-t,u=s-n,g=c*c+u*u;if(o<0)throw new Error("negative radius: "+o);if(null===this._x1)this._+="M"+(this._x1=t)+","+(this._y1=n);else if(g>1e-6)if(Math.abs(u*A-l*c)>1e-6&&o){var h=r-a,d=i-s,f=A*A+l*l,p=h*h+d*d,I=Math.sqrt(f),C=Math.sqrt(g),m=o*Math.tan((e-Math.acos((f+g-p)/(2*I*C)))/2),y=m/C,E=m/I;Math.abs(y-1)>1e-6&&(this._+="L"+(t+y*c)+","+(n+y*u)),this._+="A"+o+","+o+",0,0,"+ +(u*h>c*d)+","+(this._x1=t+E*A)+","+(this._y1=n+E*l)}else this._+="L"+(this._x1=t)+","+(this._y1=n)},arc:function(t,i,o,a,s,A){t=+t,i=+i;var l=(o=+o)*Math.cos(a),c=o*Math.sin(a),u=t+l,g=i+c,h=1^A,d=A?a-s:s-a;if(o<0)throw new Error("negative radius: "+o);null===this._x1?this._+="M"+u+","+g:(Math.abs(this._x1-u)>1e-6||Math.abs(this._y1-g)>1e-6)&&(this._+="L"+u+","+g),o&&(d<0&&(d=d%n+n),d>r?this._+="A"+o+","+o+",0,1,"+h+","+(t-l)+","+(i-c)+"A"+o+","+o+",0,1,"+h+","+(this._x1=u)+","+(this._y1=g):d>1e-6&&(this._+="A"+o+","+o+",0,"+ +(d>=e)+","+h+","+(this._x1=t+o*Math.cos(s))+","+(this._y1=i+o*Math.sin(s))))},rect:function(t,e,n,r){this._+="M"+(this._x0=this._x1=+t)+","+(this._y0=this._y1=+e)+"h"+ +n+"v"+ +r+"h"+-n+"Z"},toString:function(){return this._}},t.path=o,Object.defineProperty(t,"__esModule",{value:!0})}("object"==typeof n&&void 0!==e?n:this.d3=this.d3||{})},{}],160:[function(t,e,n){!function(t){"use strict";function e(t,e,n,r){if(isNaN(e)||isNaN(n))return t;var i,o,a,s,A,l,c,u,g,h=t._root,d={data:r},f=t._x0,p=t._y0,I=t._x1,C=t._y1;if(!h)return t._root=d,t;for(;h.length;)if((l=e>=(o=(f+I)/2))?f=o:I=o,(c=n>=(a=(p+C)/2))?p=a:C=a,i=h,!(h=h[u=c<<1|l]))return i[u]=d,t;if(s=+t._x.call(null,h.data),A=+t._y.call(null,h.data),e===s&&n===A)return d.next=h,i?i[u]=d:t._root=d,t;do{i=i?i[u]=new Array(4):t._root=new Array(4),(l=e>=(o=(f+I)/2))?f=o:I=o,(c=n>=(a=(p+C)/2))?p=a:C=a}while((u=c<<1|l)==(g=(A>=a)<<1|s>=o));return i[g]=h,i[u]=d,t}var n=function(t,e,n,r,i){this.node=t,this.x0=e,this.y0=n,this.x1=r,this.y1=i};function r(t){return t[0]}function i(t){return t[1]}function o(t,e,n){var o=new a(null==e?r:e,null==n?i:n,NaN,NaN,NaN,NaN);return null==t?o:o.addAll(t)}function a(t,e,n,r,i,o){this._x=t,this._y=e,this._x0=n,this._y0=r,this._x1=i,this._y1=o,this._root=void 0}function s(t){for(var e={data:t.data},n=e;t=t.next;)n=n.next={data:t.data};return e}var A=o.prototype=a.prototype;A.copy=function(){var t,e,n=new a(this._x,this._y,this._x0,this._y0,this._x1,this._y1),r=this._root;if(!r)return n;if(!r.length)return n._root=s(r),n;for(t=[{source:r,target:n._root=new Array(4)}];r=t.pop();)for(var i=0;i<4;++i)(e=r.source[i])&&(e.length?t.push({source:e,target:r.target[i]=new Array(4)}):r.target[i]=s(e));return n},A.add=function(t){var n=+this._x.call(null,t),r=+this._y.call(null,t);return e(this.cover(n,r),n,r,t)},A.addAll=function(t){var n,r,i,o,a=t.length,s=new Array(a),A=new Array(a),l=1/0,c=1/0,u=-1/0,g=-1/0;for(r=0;r<a;++r)isNaN(i=+this._x.call(null,n=t[r]))||isNaN(o=+this._y.call(null,n))||(s[r]=i,A[r]=o,i<l&&(l=i),i>u&&(u=i),o<c&&(c=o),o>g&&(g=o));for(u<l&&(l=this._x0,u=this._x1),g<c&&(c=this._y0,g=this._y1),this.cover(l,c).cover(u,g),r=0;r<a;++r)e(this,s[r],A[r],t[r]);return this},A.cover=function(t,e){if(isNaN(t=+t)||isNaN(e=+e))return this;var n=this._x0,r=this._y0,i=this._x1,o=this._y1;if(isNaN(n))i=(n=Math.floor(t))+1,o=(r=Math.floor(e))+1;else{if(!(n>t||t>i||r>e||e>o))return this;var a,s,A=i-n,l=this._root;switch(s=(e<(r+o)/2)<<1|t<(n+i)/2){case 0:do{(a=new Array(4))[s]=l,l=a}while(o=r+(A*=2),t>(i=n+A)||e>o);break;case 1:do{(a=new Array(4))[s]=l,l=a}while(o=r+(A*=2),(n=i-A)>t||e>o);break;case 2:do{(a=new Array(4))[s]=l,l=a}while(r=o-(A*=2),t>(i=n+A)||r>e);break;case 3:do{(a=new Array(4))[s]=l,l=a}while(r=o-(A*=2),(n=i-A)>t||r>e)}this._root&&this._root.length&&(this._root=l)}return this._x0=n,this._y0=r,this._x1=i,this._y1=o,this},A.data=function(){var t=[];return this.visit(function(e){if(!e.length)do{t.push(e.data)}while(e=e.next)}),t},A.extent=function(t){return arguments.length?this.cover(+t[0][0],+t[0][1]).cover(+t[1][0],+t[1][1]):isNaN(this._x0)?void 0:[[this._x0,this._y0],[this._x1,this._y1]]},A.find=function(t,e,r){var i,o,a,s,A,l,c,u=this._x0,g=this._y0,h=this._x1,d=this._y1,f=[],p=this._root;for(p&&f.push(new n(p,u,g,h,d)),null==r?r=1/0:(u=t-r,g=e-r,h=t+r,d=e+r,r*=r);l=f.pop();)if(!(!(p=l.node)||(o=l.x0)>h||(a=l.y0)>d||(s=l.x1)<u||(A=l.y1)<g))if(p.length){var I=(o+s)/2,C=(a+A)/2;f.push(new n(p[3],I,C,s,A),new n(p[2],o,C,I,A),new n(p[1],I,a,s,C),new n(p[0],o,a,I,C)),(c=(e>=C)<<1|t>=I)&&(l=f[f.length-1],f[f.length-1]=f[f.length-1-c],f[f.length-1-c]=l)}else{var m=t-+this._x.call(null,p.data),y=e-+this._y.call(null,p.data),E=m*m+y*y;if(E<r){var v=Math.sqrt(r=E);u=t-v,g=e-v,h=t+v,d=e+v,i=p.data}}return i},A.remove=function(t){if(isNaN(o=+this._x.call(null,t))||isNaN(a=+this._y.call(null,t)))return this;var e,n,r,i,o,a,s,A,l,c,u,g,h=this._root,d=this._x0,f=this._y0,p=this._x1,I=this._y1;if(!h)return this;if(h.length)for(;;){if((l=o>=(s=(d+p)/2))?d=s:p=s,(c=a>=(A=(f+I)/2))?f=A:I=A,e=h,!(h=h[u=c<<1|l]))return this;if(!h.length)break;(e[u+1&3]||e[u+2&3]||e[u+3&3])&&(n=e,g=u)}for(;h.data!==t;)if(r=h,!(h=h.next))return this;return(i=h.next)&&delete h.next,r?(i?r.next=i:delete r.next,this):e?(i?e[u]=i:delete e[u],(h=e[0]||e[1]||e[2]||e[3])&&h===(e[3]||e[2]||e[1]||e[0])&&!h.length&&(n?n[g]=h:this._root=h),this):(this._root=i,this)},A.removeAll=function(t){for(var e=0,n=t.length;e<n;++e)this.remove(t[e]);return this},A.root=function(){return this._root},A.size=function(){var t=0;return this.visit(function(e){if(!e.length)do{++t}while(e=e.next)}),t},A.visit=function(t){var e,r,i,o,a,s,A=[],l=this._root;for(l&&A.push(new n(l,this._x0,this._y0,this._x1,this._y1));e=A.pop();)if(!t(l=e.node,i=e.x0,o=e.y0,a=e.x1,s=e.y1)&&l.length){var c=(i+a)/2,u=(o+s)/2;(r=l[3])&&A.push(new n(r,c,u,a,s)),(r=l[2])&&A.push(new n(r,i,u,c,s)),(r=l[1])&&A.push(new n(r,c,o,a,u)),(r=l[0])&&A.push(new n(r,i,o,c,u))}return this},A.visitAfter=function(t){var e,r=[],i=[];for(this._root&&r.push(new n(this._root,this._x0,this._y0,this._x1,this._y1));e=r.pop();){var o=e.node;if(o.length){var a,s=e.x0,A=e.y0,l=e.x1,c=e.y1,u=(s+l)/2,g=(A+c)/2;(a=o[0])&&r.push(new n(a,s,A,u,g)),(a=o[1])&&r.push(new n(a,u,A,l,g)),(a=o[2])&&r.push(new n(a,s,g,u,c)),(a=o[3])&&r.push(new n(a,u,g,l,c))}i.push(e)}for(;e=i.pop();)t(e.node,e.x0,e.y0,e.x1,e.y1);return this},A.x=function(t){return arguments.length?(this._x=t,this):this._x},A.y=function(t){return arguments.length?(this._y=t,this):this._y},t.quadtree=o,Object.defineProperty(t,"__esModule",{value:!0})}("object"==typeof n&&void 0!==e?n:this.d3=this.d3||{})},{}],161:[function(t,e,n){var r,i;r=this,i=function(t,e){"use strict";function n(t){return function(){return t}}var r=Math.abs,i=Math.atan2,o=Math.cos,a=Math.max,s=Math.min,A=Math.sin,l=Math.sqrt,c=1e-12,u=Math.PI,g=u/2,h=2*u;function d(t){return t>=1?g:t<=-1?-g:Math.asin(t)}function f(t){return t.innerRadius}function p(t){return t.outerRadius}function I(t){return t.startAngle}function C(t){return t.endAngle}function m(t){return t&&t.padAngle}function y(t,e,n,r,i,o,s){var A=t-n,c=e-r,u=(s?o:-o)/l(A*A+c*c),g=u*c,h=-u*A,d=t+g,f=e+h,p=n+g,I=r+h,C=(d+p)/2,m=(f+I)/2,y=p-d,E=I-f,v=y*y+E*E,B=i-o,b=d*I-p*f,Q=(E<0?-1:1)*l(a(0,B*B*v-b*b)),w=(b*E-y*Q)/v,x=(-b*y-E*Q)/v,_=(b*E+y*Q)/v,D=(-b*y+E*Q)/v,M=w-C,k=x-m,S=_-C,L=D-m;return M*M+k*k>S*S+L*L&&(w=_,x=D),{cx:w,cy:x,x01:-g,y01:-h,x11:w*(i/B-1),y11:x*(i/B-1)}}function E(t){this._context=t}function v(t){return new E(t)}function B(t){return t[0]}function b(t){return t[1]}function Q(){var t=B,r=b,i=n(!0),o=null,a=v,s=null;function A(n){var A,l,c,u=n.length,g=!1;for(null==o&&(s=a(c=e.path())),A=0;A<=u;++A)!(A<u&&i(l=n[A],A,n))===g&&((g=!g)?s.lineStart():s.lineEnd()),g&&s.point(+t(l,A,n),+r(l,A,n));if(c)return s=null,c+""||null}return A.x=function(e){return arguments.length?(t="function"==typeof e?e:n(+e),A):t},A.y=function(t){return arguments.length?(r="function"==typeof t?t:n(+t),A):r},A.defined=function(t){return arguments.length?(i="function"==typeof t?t:n(!!t),A):i},A.curve=function(t){return arguments.length?(a=t,null!=o&&(s=a(o)),A):a},A.context=function(t){return arguments.length?(null==t?o=s=null:s=a(o=t),A):o},A}function w(){var t=B,r=null,i=n(0),o=b,a=n(!0),s=null,A=v,l=null;function c(n){var c,u,g,h,d,f=n.length,p=!1,I=new Array(f),C=new Array(f);for(null==s&&(l=A(d=e.path())),c=0;c<=f;++c){if(!(c<f&&a(h=n[c],c,n))===p)if(p=!p)u=c,l.areaStart(),l.lineStart();else{for(l.lineEnd(),l.lineStart(),g=c-1;g>=u;--g)l.point(I[g],C[g]);l.lineEnd(),l.areaEnd()}p&&(I[c]=+t(h,c,n),C[c]=+i(h,c,n),l.point(r?+r(h,c,n):I[c],o?+o(h,c,n):C[c]))}if(d)return l=null,d+""||null}function u(){return Q().defined(a).curve(A).context(s)}return c.x=function(e){return arguments.length?(t="function"==typeof e?e:n(+e),r=null,c):t},c.x0=function(e){return arguments.length?(t="function"==typeof e?e:n(+e),c):t},c.x1=function(t){return arguments.length?(r=null==t?null:"function"==typeof t?t:n(+t),c):r},c.y=function(t){return arguments.length?(i="function"==typeof t?t:n(+t),o=null,c):i},c.y0=function(t){return arguments.length?(i="function"==typeof t?t:n(+t),c):i},c.y1=function(t){return arguments.length?(o=null==t?null:"function"==typeof t?t:n(+t),c):o},c.lineX0=c.lineY0=function(){return u().x(t).y(i)},c.lineY1=function(){return u().x(t).y(o)},c.lineX1=function(){return u().x(r).y(i)},c.defined=function(t){return arguments.length?(a="function"==typeof t?t:n(!!t),c):a},c.curve=function(t){return arguments.length?(A=t,null!=s&&(l=A(s)),c):A},c.context=function(t){return arguments.length?(null==t?s=l=null:l=A(s=t),c):s},c}function x(t,e){return e<t?-1:e>t?1:e>=t?0:NaN}function _(t){return t}E.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._point=0},lineEnd:function(){(this._line||0!==this._line&&1===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1,this._line?this._context.lineTo(t,e):this._context.moveTo(t,e);break;case 1:this._point=2;default:this._context.lineTo(t,e)}}};var D=k(v);function M(t){this._curve=t}function k(t){function e(e){return new M(t(e))}return e._curve=t,e}function S(t){var e=t.curve;return t.angle=t.x,delete t.x,t.radius=t.y,delete t.y,t.curve=function(t){return arguments.length?e(k(t)):e()._curve},t}function L(){return S(Q().curve(D))}function R(){var t=w().curve(D),e=t.curve,n=t.lineX0,r=t.lineX1,i=t.lineY0,o=t.lineY1;return t.angle=t.x,delete t.x,t.startAngle=t.x0,delete t.x0,t.endAngle=t.x1,delete t.x1,t.radius=t.y,delete t.y,t.innerRadius=t.y0,delete t.y0,t.outerRadius=t.y1,delete t.y1,t.lineStartAngle=function(){return S(n())},delete t.lineX0,t.lineEndAngle=function(){return S(r())},delete t.lineX1,t.lineInnerRadius=function(){return S(i())},delete t.lineY0,t.lineOuterRadius=function(){return S(o())},delete t.lineY1,t.curve=function(t){return arguments.length?e(k(t)):e()._curve},t}function N(t,e){return[(e=+e)*Math.cos(t-=Math.PI/2),e*Math.sin(t)]}M.prototype={areaStart:function(){this._curve.areaStart()},areaEnd:function(){this._curve.areaEnd()},lineStart:function(){this._curve.lineStart()},lineEnd:function(){this._curve.lineEnd()},point:function(t,e){this._curve.point(e*Math.sin(t),e*-Math.cos(t))}};var F=Array.prototype.slice;function T(t){return t.source}function U(t){return t.target}function O(t){var r=T,i=U,o=B,a=b,s=null;function A(){var n,A=F.call(arguments),l=r.apply(this,A),c=i.apply(this,A);if(s||(s=n=e.path()),t(s,+o.apply(this,(A[0]=l,A)),+a.apply(this,A),+o.apply(this,(A[0]=c,A)),+a.apply(this,A)),n)return s=null,n+""||null}return A.source=function(t){return arguments.length?(r=t,A):r},A.target=function(t){return arguments.length?(i=t,A):i},A.x=function(t){return arguments.length?(o="function"==typeof t?t:n(+t),A):o},A.y=function(t){return arguments.length?(a="function"==typeof t?t:n(+t),A):a},A.context=function(t){return arguments.length?(s=null==t?null:t,A):s},A}function G(t,e,n,r,i){t.moveTo(e,n),t.bezierCurveTo(e=(e+r)/2,n,e,i,r,i)}function H(t,e,n,r,i){t.moveTo(e,n),t.bezierCurveTo(e,n=(n+i)/2,r,n,r,i)}function j(t,e,n,r,i){var o=N(e,n),a=N(e,n=(n+i)/2),s=N(r,n),A=N(r,i);t.moveTo(o[0],o[1]),t.bezierCurveTo(a[0],a[1],s[0],s[1],A[0],A[1])}var Y={draw:function(t,e){var n=Math.sqrt(e/u);t.moveTo(n,0),t.arc(0,0,n,0,h)}},P={draw:function(t,e){var n=Math.sqrt(e/5)/2;t.moveTo(-3*n,-n),t.lineTo(-n,-n),t.lineTo(-n,-3*n),t.lineTo(n,-3*n),t.lineTo(n,-n),t.lineTo(3*n,-n),t.lineTo(3*n,n),t.lineTo(n,n),t.lineTo(n,3*n),t.lineTo(-n,3*n),t.lineTo(-n,n),t.lineTo(-3*n,n),t.closePath()}},K=Math.sqrt(1/3),z=2*K,J={draw:function(t,e){var n=Math.sqrt(e/z),r=n*K;t.moveTo(0,-n),t.lineTo(r,0),t.lineTo(0,n),t.lineTo(-r,0),t.closePath()}},q=Math.sin(u/10)/Math.sin(7*u/10),V=Math.sin(h/10)*q,W=-Math.cos(h/10)*q,X={draw:function(t,e){var n=Math.sqrt(.8908130915292852*e),r=V*n,i=W*n;t.moveTo(0,-n),t.lineTo(r,i);for(var o=1;o<5;++o){var a=h*o/5,s=Math.cos(a),A=Math.sin(a);t.lineTo(A*n,-s*n),t.lineTo(s*r-A*i,A*r+s*i)}t.closePath()}},Z={draw:function(t,e){var n=Math.sqrt(e),r=-n/2;t.rect(r,r,n,n)}},$=Math.sqrt(3),tt={draw:function(t,e){var n=-Math.sqrt(e/(3*$));t.moveTo(0,2*n),t.lineTo(-$*n,-n),t.lineTo($*n,-n),t.closePath()}},et=-.5,nt=Math.sqrt(3)/2,rt=1/Math.sqrt(12),it=3*(rt/2+1),ot={draw:function(t,e){var n=Math.sqrt(e/it),r=n/2,i=n*rt,o=r,a=n*rt+n,s=-o,A=a;t.moveTo(r,i),t.lineTo(o,a),t.lineTo(s,A),t.lineTo(et*r-nt*i,nt*r+et*i),t.lineTo(et*o-nt*a,nt*o+et*a),t.lineTo(et*s-nt*A,nt*s+et*A),t.lineTo(et*r+nt*i,et*i-nt*r),t.lineTo(et*o+nt*a,et*a-nt*o),t.lineTo(et*s+nt*A,et*A-nt*s),t.closePath()}},at=[Y,P,J,Z,X,tt,ot];function st(){}function At(t,e,n){t._context.bezierCurveTo((2*t._x0+t._x1)/3,(2*t._y0+t._y1)/3,(t._x0+2*t._x1)/3,(t._y0+2*t._y1)/3,(t._x0+4*t._x1+e)/6,(t._y0+4*t._y1+n)/6)}function lt(t){this._context=t}function ct(t){this._context=t}function ut(t){this._context=t}function gt(t,e){this._basis=new lt(t),this._beta=e}lt.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=NaN,this._point=0},lineEnd:function(){switch(this._point){case 3:At(this,this._x1,this._y1);case 2:this._context.lineTo(this._x1,this._y1)}(this._line||0!==this._line&&1===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1,this._line?this._context.lineTo(t,e):this._context.moveTo(t,e);break;case 1:this._point=2;break;case 2:this._point=3,this._context.lineTo((5*this._x0+this._x1)/6,(5*this._y0+this._y1)/6);default:At(this,t,e)}this._x0=this._x1,this._x1=t,this._y0=this._y1,this._y1=e}},ct.prototype={areaStart:st,areaEnd:st,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._y0=this._y1=this._y2=this._y3=this._y4=NaN,this._point=0},lineEnd:function(){switch(this._point){case 1:this._context.moveTo(this._x2,this._y2),this._context.closePath();break;case 2:this._context.moveTo((this._x2+2*this._x3)/3,(this._y2+2*this._y3)/3),this._context.lineTo((this._x3+2*this._x2)/3,(this._y3+2*this._y2)/3),this._context.closePath();break;case 3:this.point(this._x2,this._y2),this.point(this._x3,this._y3),this.point(this._x4,this._y4)}},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1,this._x2=t,this._y2=e;break;case 1:this._point=2,this._x3=t,this._y3=e;break;case 2:this._point=3,this._x4=t,this._y4=e,this._context.moveTo((this._x0+4*this._x1+t)/6,(this._y0+4*this._y1+e)/6);break;default:At(this,t,e)}this._x0=this._x1,this._x1=t,this._y0=this._y1,this._y1=e}},ut.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=NaN,this._point=0},lineEnd:function(){(this._line||0!==this._line&&3===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3;var n=(this._x0+4*this._x1+t)/6,r=(this._y0+4*this._y1+e)/6;this._line?this._context.lineTo(n,r):this._context.moveTo(n,r);break;case 3:this._point=4;default:At(this,t,e)}this._x0=this._x1,this._x1=t,this._y0=this._y1,this._y1=e}},gt.prototype={lineStart:function(){this._x=[],this._y=[],this._basis.lineStart()},lineEnd:function(){var t=this._x,e=this._y,n=t.length-1;if(n>0)for(var r,i=t[0],o=e[0],a=t[n]-i,s=e[n]-o,A=-1;++A<=n;)r=A/n,this._basis.point(this._beta*t[A]+(1-this._beta)*(i+r*a),this._beta*e[A]+(1-this._beta)*(o+r*s));this._x=this._y=null,this._basis.lineEnd()},point:function(t,e){this._x.push(+t),this._y.push(+e)}};var ht=function t(e){function n(t){return 1===e?new lt(t):new gt(t,e)}return n.beta=function(e){return t(+e)},n}(.85);function dt(t,e,n){t._context.bezierCurveTo(t._x1+t._k*(t._x2-t._x0),t._y1+t._k*(t._y2-t._y0),t._x2+t._k*(t._x1-e),t._y2+t._k*(t._y1-n),t._x2,t._y2)}function ft(t,e){this._context=t,this._k=(1-e)/6}ft.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x2,this._y2);break;case 3:dt(this,this._x1,this._y1)}(this._line||0!==this._line&&1===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1,this._line?this._context.lineTo(t,e):this._context.moveTo(t,e);break;case 1:this._point=2,this._x1=t,this._y1=e;break;case 2:this._point=3;default:dt(this,t,e)}this._x0=this._x1,this._x1=this._x2,this._x2=t,this._y0=this._y1,this._y1=this._y2,this._y2=e}};var pt=function t(e){function n(t){return new ft(t,e)}return n.tension=function(e){return t(+e)},n}(0);function It(t,e){this._context=t,this._k=(1-e)/6}It.prototype={areaStart:st,areaEnd:st,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._x5=this._y0=this._y1=this._y2=this._y3=this._y4=this._y5=NaN,this._point=0},lineEnd:function(){switch(this._point){case 1:this._context.moveTo(this._x3,this._y3),this._context.closePath();break;case 2:this._context.lineTo(this._x3,this._y3),this._context.closePath();break;case 3:this.point(this._x3,this._y3),this.point(this._x4,this._y4),this.point(this._x5,this._y5)}},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1,this._x3=t,this._y3=e;break;case 1:this._point=2,this._context.moveTo(this._x4=t,this._y4=e);break;case 2:this._point=3,this._x5=t,this._y5=e;break;default:dt(this,t,e)}this._x0=this._x1,this._x1=this._x2,this._x2=t,this._y0=this._y1,this._y1=this._y2,this._y2=e}};var Ct=function t(e){function n(t){return new It(t,e)}return n.tension=function(e){return t(+e)},n}(0);function mt(t,e){this._context=t,this._k=(1-e)/6}mt.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._point=0},lineEnd:function(){(this._line||0!==this._line&&3===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3,this._line?this._context.lineTo(this._x2,this._y2):this._context.moveTo(this._x2,this._y2);break;case 3:this._point=4;default:dt(this,t,e)}this._x0=this._x1,this._x1=this._x2,this._x2=t,this._y0=this._y1,this._y1=this._y2,this._y2=e}};var yt=function t(e){function n(t){return new mt(t,e)}return n.tension=function(e){return t(+e)},n}(0);function Et(t,e,n){var r=t._x1,i=t._y1,o=t._x2,a=t._y2;if(t._l01_a>c){var s=2*t._l01_2a+3*t._l01_a*t._l12_a+t._l12_2a,A=3*t._l01_a*(t._l01_a+t._l12_a);r=(r*s-t._x0*t._l12_2a+t._x2*t._l01_2a)/A,i=(i*s-t._y0*t._l12_2a+t._y2*t._l01_2a)/A}if(t._l23_a>c){var l=2*t._l23_2a+3*t._l23_a*t._l12_a+t._l12_2a,u=3*t._l23_a*(t._l23_a+t._l12_a);o=(o*l+t._x1*t._l23_2a-e*t._l12_2a)/u,a=(a*l+t._y1*t._l23_2a-n*t._l12_2a)/u}t._context.bezierCurveTo(r,i,o,a,t._x2,t._y2)}function vt(t,e){this._context=t,this._alpha=e}vt.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._l01_a=this._l12_a=this._l23_a=this._l01_2a=this._l12_2a=this._l23_2a=this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x2,this._y2);break;case 3:this.point(this._x2,this._y2)}(this._line||0!==this._line&&1===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){if(t=+t,e=+e,this._point){var n=this._x2-t,r=this._y2-e;this._l23_a=Math.sqrt(this._l23_2a=Math.pow(n*n+r*r,this._alpha))}switch(this._point){case 0:this._point=1,this._line?this._context.lineTo(t,e):this._context.moveTo(t,e);break;case 1:this._point=2;break;case 2:this._point=3;default:Et(this,t,e)}this._l01_a=this._l12_a,this._l12_a=this._l23_a,this._l01_2a=this._l12_2a,this._l12_2a=this._l23_2a,this._x0=this._x1,this._x1=this._x2,this._x2=t,this._y0=this._y1,this._y1=this._y2,this._y2=e}};var Bt=function t(e){function n(t){return e?new vt(t,e):new ft(t,0)}return n.alpha=function(e){return t(+e)},n}(.5);function bt(t,e){this._context=t,this._alpha=e}bt.prototype={areaStart:st,areaEnd:st,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._x5=this._y0=this._y1=this._y2=this._y3=this._y4=this._y5=NaN,this._l01_a=this._l12_a=this._l23_a=this._l01_2a=this._l12_2a=this._l23_2a=this._point=0},lineEnd:function(){switch(this._point){case 1:this._context.moveTo(this._x3,this._y3),this._context.closePath();break;case 2:this._context.lineTo(this._x3,this._y3),this._context.closePath();break;case 3:this.point(this._x3,this._y3),this.point(this._x4,this._y4),this.point(this._x5,this._y5)}},point:function(t,e){if(t=+t,e=+e,this._point){var n=this._x2-t,r=this._y2-e;this._l23_a=Math.sqrt(this._l23_2a=Math.pow(n*n+r*r,this._alpha))}switch(this._point){case 0:this._point=1,this._x3=t,this._y3=e;break;case 1:this._point=2,this._context.moveTo(this._x4=t,this._y4=e);break;case 2:this._point=3,this._x5=t,this._y5=e;break;default:Et(this,t,e)}this._l01_a=this._l12_a,this._l12_a=this._l23_a,this._l01_2a=this._l12_2a,this._l12_2a=this._l23_2a,this._x0=this._x1,this._x1=this._x2,this._x2=t,this._y0=this._y1,this._y1=this._y2,this._y2=e}};var Qt=function t(e){function n(t){return e?new bt(t,e):new It(t,0)}return n.alpha=function(e){return t(+e)},n}(.5);function wt(t,e){this._context=t,this._alpha=e}wt.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._l01_a=this._l12_a=this._l23_a=this._l01_2a=this._l12_2a=this._l23_2a=this._point=0},lineEnd:function(){(this._line||0!==this._line&&3===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){if(t=+t,e=+e,this._point){var n=this._x2-t,r=this._y2-e;this._l23_a=Math.sqrt(this._l23_2a=Math.pow(n*n+r*r,this._alpha))}switch(this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3,this._line?this._context.lineTo(this._x2,this._y2):this._context.moveTo(this._x2,this._y2);break;case 3:this._point=4;default:Et(this,t,e)}this._l01_a=this._l12_a,this._l12_a=this._l23_a,this._l01_2a=this._l12_2a,this._l12_2a=this._l23_2a,this._x0=this._x1,this._x1=this._x2,this._x2=t,this._y0=this._y1,this._y1=this._y2,this._y2=e}};var xt=function t(e){function n(t){return e?new wt(t,e):new mt(t,0)}return n.alpha=function(e){return t(+e)},n}(.5);function _t(t){this._context=t}function Dt(t){return t<0?-1:1}function Mt(t,e,n){var r=t._x1-t._x0,i=e-t._x1,o=(t._y1-t._y0)/(r||i<0&&-0),a=(n-t._y1)/(i||r<0&&-0),s=(o*i+a*r)/(r+i);return(Dt(o)+Dt(a))*Math.min(Math.abs(o),Math.abs(a),.5*Math.abs(s))||0}function kt(t,e){var n=t._x1-t._x0;return n?(3*(t._y1-t._y0)/n-e)/2:e}function St(t,e,n){var r=t._x0,i=t._y0,o=t._x1,a=t._y1,s=(o-r)/3;t._context.bezierCurveTo(r+s,i+s*e,o-s,a-s*n,o,a)}function Lt(t){this._context=t}function Rt(t){this._context=new Nt(t)}function Nt(t){this._context=t}function Ft(t){this._context=t}function Tt(t){var e,n,r=t.length-1,i=new Array(r),o=new Array(r),a=new Array(r);for(i[0]=0,o[0]=2,a[0]=t[0]+2*t[1],e=1;e<r-1;++e)i[e]=1,o[e]=4,a[e]=4*t[e]+2*t[e+1];for(i[r-1]=2,o[r-1]=7,a[r-1]=8*t[r-1]+t[r],e=1;e<r;++e)n=i[e]/o[e-1],o[e]-=n,a[e]-=n*a[e-1];for(i[r-1]=a[r-1]/o[r-1],e=r-2;e>=0;--e)i[e]=(a[e]-i[e+1])/o[e];for(o[r-1]=(t[r]+i[r-1])/2,e=0;e<r-1;++e)o[e]=2*t[e+1]-i[e+1];return[i,o]}function Ut(t,e){this._context=t,this._t=e}function Ot(t,e){if((i=t.length)>1)for(var n,r,i,o=1,a=t[e[0]],s=a.length;o<i;++o)for(r=a,a=t[e[o]],n=0;n<s;++n)a[n][1]+=a[n][0]=isNaN(r[n][1])?r[n][0]:r[n][1]}function Gt(t){for(var e=t.length,n=new Array(e);--e>=0;)n[e]=e;return n}function Ht(t,e){return t[e]}function jt(t){var e=t.map(Yt);return Gt(t).sort(function(t,n){return e[t]-e[n]})}function Yt(t){for(var e,n=-1,r=0,i=t.length,o=-1/0;++n<i;)(e=+t[n][1])>o&&(o=e,r=n);return r}function Pt(t){var e=t.map(Kt);return Gt(t).sort(function(t,n){return e[t]-e[n]})}function Kt(t){for(var e,n=0,r=-1,i=t.length;++r<i;)(e=+t[r][1])&&(n+=e);return n}_t.prototype={areaStart:st,areaEnd:st,lineStart:function(){this._point=0},lineEnd:function(){this._point&&this._context.closePath()},point:function(t,e){t=+t,e=+e,this._point?this._context.lineTo(t,e):(this._point=1,this._context.moveTo(t,e))}},Lt.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=this._t0=NaN,this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x1,this._y1);break;case 3:St(this,this._t0,kt(this,this._t0))}(this._line||0!==this._line&&1===this._point)&&this._context.closePath(),this._line=1-this._line},point:function(t,e){var n=NaN;if(e=+e,(t=+t)!==this._x1||e!==this._y1){switch(this._point){case 0:this._point=1,this._line?this._context.lineTo(t,e):this._context.moveTo(t,e);break;case 1:this._point=2;break;case 2:this._point=3,St(this,kt(this,n=Mt(this,t,e)),n);break;default:St(this,this._t0,n=Mt(this,t,e))}this._x0=this._x1,this._x1=t,this._y0=this._y1,this._y1=e,this._t0=n}}},(Rt.prototype=Object.create(Lt.prototype)).point=function(t,e){Lt.prototype.point.call(this,e,t)},Nt.prototype={moveTo:function(t,e){this._context.moveTo(e,t)},closePath:function(){this._context.closePath()},lineTo:function(t,e){this._context.lineTo(e,t)},bezierCurveTo:function(t,e,n,r,i,o){this._context.bezierCurveTo(e,t,r,n,o,i)}},Ft.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x=[],this._y=[]},lineEnd:function(){var t=this._x,e=this._y,n=t.length;if(n)if(this._line?this._context.lineTo(t[0],e[0]):this._context.moveTo(t[0],e[0]),2===n)this._context.lineTo(t[1],e[1]);else for(var r=Tt(t),i=Tt(e),o=0,a=1;a<n;++o,++a)this._context.bezierCurveTo(r[0][o],i[0][o],r[1][o],i[1][o],t[a],e[a]);(this._line||0!==this._line&&1===n)&&this._context.closePath(),this._line=1-this._line,this._x=this._y=null},point:function(t,e){this._x.push(+t),this._y.push(+e)}},Ut.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x=this._y=NaN,this._point=0},lineEnd:function(){0<this._t&&this._t<1&&2===this._point&&this._context.lineTo(this._x,this._y),(this._line||0!==this._line&&1===this._point)&&this._context.closePath(),this._line>=0&&(this._t=1-this._t,this._line=1-this._line)},point:function(t,e){switch(t=+t,e=+e,this._point){case 0:this._point=1,this._line?this._context.lineTo(t,e):this._context.moveTo(t,e);break;case 1:this._point=2;default:if(this._t<=0)this._context.lineTo(this._x,e),this._context.lineTo(t,e);else{var n=this._x*(1-this._t)+t*this._t;this._context.lineTo(n,this._y),this._context.lineTo(n,e)}}this._x=t,this._y=e}},t.arc=function(){var t=f,a=p,E=n(0),v=null,B=I,b=C,Q=m,w=null;function x(){var n,f,p,I=+t.apply(this,arguments),C=+a.apply(this,arguments),m=B.apply(this,arguments)-g,x=b.apply(this,arguments)-g,_=r(x-m),D=x>m;if(w||(w=n=e.path()),C<I&&(f=C,C=I,I=f),C>c)if(_>h-c)w.moveTo(C*o(m),C*A(m)),w.arc(0,0,C,m,x,!D),I>c&&(w.moveTo(I*o(x),I*A(x)),w.arc(0,0,I,x,m,D));else{var M,k,S=m,L=x,R=m,N=x,F=_,T=_,U=Q.apply(this,arguments)/2,O=U>c&&(v?+v.apply(this,arguments):l(I*I+C*C)),G=s(r(C-I)/2,+E.apply(this,arguments)),H=G,j=G;if(O>c){var Y=d(O/I*A(U)),P=d(O/C*A(U));(F-=2*Y)>c?(R+=Y*=D?1:-1,N-=Y):(F=0,R=N=(m+x)/2),(T-=2*P)>c?(S+=P*=D?1:-1,L-=P):(T=0,S=L=(m+x)/2)}var K=C*o(S),z=C*A(S),J=I*o(N),q=I*A(N);if(G>c){var V,W=C*o(L),X=C*A(L),Z=I*o(R),$=I*A(R);if(_<u&&(V=function(t,e,n,r,i,o,a,s){var A=NaN,l=NaN,u=NaN,g=NaN,h=g*A-u*l;if(!(h*h<c))return[t+(h=(NaN*u-NaN*g)/h)*A,e+h*l]}(K,z))){var tt=K-V[0],et=z-V[1],nt=W-V[0],rt=X-V[1],it=1/A(((p=(tt*nt+et*rt)/(l(tt*tt+et*et)*l(nt*nt+rt*rt)))>1?0:p<-1?u:Math.acos(p))/2),ot=l(V[0]*V[0]+V[1]*V[1]);H=s(G,(I-ot)/(it-1)),j=s(G,(C-ot)/(it+1))}}T>c?j>c?(M=y(Z,$,K,z,C,j,D),k=y(W,X,J,q,C,j,D),w.moveTo(M.cx+M.x01,M.cy+M.y01),j<G?w.arc(M.cx,M.cy,j,i(M.y01,M.x01),i(k.y01,k.x01),!D):(w.arc(M.cx,M.cy,j,i(M.y01,M.x01),i(M.y11,M.x11),!D),w.arc(0,0,C,i(M.cy+M.y11,M.cx+M.x11),i(k.cy+k.y11,k.cx+k.x11),!D),w.arc(k.cx,k.cy,j,i(k.y11,k.x11),i(k.y01,k.x01),!D))):(w.moveTo(K,z),w.arc(0,0,C,S,L,!D)):w.moveTo(K,z),I>c&&F>c?H>c?(M=y(J,q,W,X,I,-H,D),k=y(K,z,Z,$,I,-H,D),w.lineTo(M.cx+M.x01,M.cy+M.y01),H<G?w.arc(M.cx,M.cy,H,i(M.y01,M.x01),i(k.y01,k.x01),!D):(w.arc(M.cx,M.cy,H,i(M.y01,M.x01),i(M.y11,M.x11),!D),w.arc(0,0,I,i(M.cy+M.y11,M.cx+M.x11),i(k.cy+k.y11,k.cx+k.x11),D),w.arc(k.cx,k.cy,H,i(k.y11,k.x11),i(k.y01,k.x01),!D))):w.arc(0,0,I,N,R,D):w.lineTo(J,q)}else w.moveTo(0,0);if(w.closePath(),n)return w=null,n+""||null}return x.centroid=function(){var e=(+t.apply(this,arguments)+ +a.apply(this,arguments))/2,n=(+B.apply(this,arguments)+ +b.apply(this,arguments))/2-u/2;return[o(n)*e,A(n)*e]},x.innerRadius=function(e){return arguments.length?(t="function"==typeof e?e:n(+e),x):t},x.outerRadius=function(t){return arguments.length?(a="function"==typeof t?t:n(+t),x):a},x.cornerRadius=function(t){return arguments.length?(E="function"==typeof t?t:n(+t),x):E},x.padRadius=function(t){return arguments.length?(v=null==t?null:"function"==typeof t?t:n(+t),x):v},x.startAngle=function(t){return arguments.length?(B="function"==typeof t?t:n(+t),x):B},x.endAngle=function(t){return arguments.length?(b="function"==typeof t?t:n(+t),x):b},x.padAngle=function(t){return arguments.length?(Q="function"==typeof t?t:n(+t),x):Q},x.context=function(t){return arguments.length?(w=null==t?null:t,x):w},x},t.area=w,t.line=Q,t.pie=function(){var t=_,e=x,r=null,i=n(0),o=n(h),a=n(0);function s(n){var s,A,l,c,u,g=n.length,d=0,f=new Array(g),p=new Array(g),I=+i.apply(this,arguments),C=Math.min(h,Math.max(-h,o.apply(this,arguments)-I)),m=Math.min(Math.abs(C)/g,a.apply(this,arguments)),y=m*(C<0?-1:1);for(s=0;s<g;++s)(u=p[f[s]=s]=+t(n[s],s,n))>0&&(d+=u);for(null!=e?f.sort(function(t,n){return e(p[t],p[n])}):null!=r&&f.sort(function(t,e){return r(n[t],n[e])}),s=0,l=d?(C-g*y)/d:0;s<g;++s,I=c)A=f[s],c=I+((u=p[A])>0?u*l:0)+y,p[A]={data:n[A],index:s,value:u,startAngle:I,endAngle:c,padAngle:m};return p}return s.value=function(e){return arguments.length?(t="function"==typeof e?e:n(+e),s):t},s.sortValues=function(t){return arguments.length?(e=t,r=null,s):e},s.sort=function(t){return arguments.length?(r=t,e=null,s):r},s.startAngle=function(t){return arguments.length?(i="function"==typeof t?t:n(+t),s):i},s.endAngle=function(t){return arguments.length?(o="function"==typeof t?t:n(+t),s):o},s.padAngle=function(t){return arguments.length?(a="function"==typeof t?t:n(+t),s):a},s},t.areaRadial=R,t.radialArea=R,t.lineRadial=L,t.radialLine=L,t.pointRadial=N,t.linkHorizontal=function(){return O(G)},t.linkVertical=function(){return O(H)},t.linkRadial=function(){var t=O(j);return t.angle=t.x,delete t.x,t.radius=t.y,delete t.y,t},t.symbol=function(){var t=n(Y),r=n(64),i=null;function o(){var n;if(i||(i=n=e.path()),t.apply(this,arguments).draw(i,+r.apply(this,arguments)),n)return i=null,n+""||null}return o.type=function(e){return arguments.length?(t="function"==typeof e?e:n(e),o):t},o.size=function(t){return arguments.length?(r="function"==typeof t?t:n(+t),o):r},o.context=function(t){return arguments.length?(i=null==t?null:t,o):i},o},t.symbols=at,t.symbolCircle=Y,t.symbolCross=P,t.symbolDiamond=J,t.symbolSquare=Z,t.symbolStar=X,t.symbolTriangle=tt,t.symbolWye=ot,t.curveBasisClosed=function(t){return new ct(t)},t.curveBasisOpen=function(t){return new ut(t)},t.curveBasis=function(t){return new lt(t)},t.curveBundle=ht,t.curveCardinalClosed=Ct,t.curveCardinalOpen=yt,t.curveCardinal=pt,t.curveCatmullRomClosed=Qt,t.curveCatmullRomOpen=xt,t.curveCatmullRom=Bt,t.curveLinearClosed=function(t){return new _t(t)},t.curveLinear=v,t.curveMonotoneX=function(t){return new Lt(t)},t.curveMonotoneY=function(t){return new Rt(t)},t.curveNatural=function(t){return new Ft(t)},t.curveStep=function(t){return new Ut(t,.5)},t.curveStepAfter=function(t){return new Ut(t,1)},t.curveStepBefore=function(t){return new Ut(t,0)},t.stack=function(){var t=n([]),e=Gt,r=Ot,i=Ht;function o(n){var o,a,s=t.apply(this,arguments),A=n.length,l=s.length,c=new Array(l);for(o=0;o<l;++o){for(var u,g=s[o],h=c[o]=new Array(A),d=0;d<A;++d)h[d]=u=[0,+i(n[d],g,d,n)],u.data=n[d];h.key=g}for(o=0,a=e(c);o<l;++o)c[a[o]].index=o;return r(c,a),c}return o.keys=function(e){return arguments.length?(t="function"==typeof e?e:n(F.call(e)),o):t},o.value=function(t){return arguments.length?(i="function"==typeof t?t:n(+t),o):i},o.order=function(t){return arguments.length?(e=null==t?Gt:"function"==typeof t?t:n(F.call(t)),o):e},o.offset=function(t){return arguments.length?(r=null==t?Ot:t,o):r},o},t.stackOffsetExpand=function(t,e){if((r=t.length)>0){for(var n,r,i,o=0,a=t[0].length;o<a;++o){for(i=n=0;n<r;++n)i+=t[n][o][1]||0;if(i)for(n=0;n<r;++n)t[n][o][1]/=i}Ot(t,e)}},t.stackOffsetDiverging=function(t,e){if((s=t.length)>1)for(var n,r,i,o,a,s,A=0,l=t[e[0]].length;A<l;++A)for(o=a=0,n=0;n<s;++n)(i=(r=t[e[n]][A])[1]-r[0])>=0?(r[0]=o,r[1]=o+=i):i<0?(r[1]=a,r[0]=a+=i):r[0]=o},t.stackOffsetNone=Ot,t.stackOffsetSilhouette=function(t,e){if((n=t.length)>0){for(var n,r=0,i=t[e[0]],o=i.length;r<o;++r){for(var a=0,s=0;a<n;++a)s+=t[a][r][1]||0;i[r][1]+=i[r][0]=-s/2}Ot(t,e)}},t.stackOffsetWiggle=function(t,e){if((i=t.length)>0&&(r=(n=t[e[0]]).length)>0){for(var n,r,i,o=0,a=1;a<r;++a){for(var s=0,A=0,l=0;s<i;++s){for(var c=t[e[s]],u=c[a][1]||0,g=(u-(c[a-1][1]||0))/2,h=0;h<s;++h){var d=t[e[h]];g+=(d[a][1]||0)-(d[a-1][1]||0)}A+=u,l+=g*u}n[a-1][1]+=n[a-1][0]=o,A&&(o-=l/A)}n[a-1][1]+=n[a-1][0]=o,Ot(t,e)}},t.stackOrderAppearance=jt,t.stackOrderAscending=Pt,t.stackOrderDescending=function(t){return Pt(t).reverse()},t.stackOrderInsideOut=function(t){var e,n,r=t.length,i=t.map(Kt),o=jt(t),a=0,s=0,A=[],l=[];for(e=0;e<r;++e)n=o[e],a<s?(a+=i[n],A.push(n)):(s+=i[n],l.push(n));return l.reverse().concat(A)},t.stackOrderNone=Gt,t.stackOrderReverse=function(t){return Gt(t).reverse()},Object.defineProperty(t,"__esModule",{value:!0})},"object"==typeof n&&void 0!==e?i(n,t("d3-path")):i(r.d3=r.d3||{},r.d3)},{"d3-path":159}],162:[function(t,e,n){!function(t){"use strict";var e,n,r=0,i=0,o=0,a=1e3,s=0,A=0,l=0,c="object"==typeof performance&&performance.now?performance:Date,u="object"==typeof window&&window.requestAnimationFrame?window.requestAnimationFrame.bind(window):function(t){setTimeout(t,17)};function g(){return A||(u(h),A=c.now()+l)}function h(){A=0}function d(){this._call=this._time=this._next=null}function f(t,e,n){var r=new d;return r.restart(t,e,n),r}function p(){g(),++r;for(var t,n=e;n;)(t=A-n._time)>=0&&n._call.call(null,t),n=n._next;--r}function I(){A=(s=c.now())+l,r=i=0;try{p()}finally{r=0,function(){for(var t,r,i=e,o=1/0;i;)i._call?(o>i._time&&(o=i._time),t=i,i=i._next):(r=i._next,i._next=null,i=t?t._next=r:e=r);n=t,m(o)}(),A=0}}function C(){var t=c.now(),e=t-s;e>a&&(l-=e,s=t)}function m(t){r||(i&&(i=clearTimeout(i)),t-A>24?(t<1/0&&(i=setTimeout(I,t-c.now()-l)),o&&(o=clearInterval(o))):(o||(s=c.now(),o=setInterval(C,a)),r=1,u(I)))}d.prototype=f.prototype={constructor:d,restart:function(t,r,i){if("function"!=typeof t)throw new TypeError("callback is not a function");i=(null==i?g():+i)+(null==r?0:+r),this._next||n===this||(n?n._next=this:e=this,n=this),this._call=t,this._time=i,m()},stop:function(){this._call&&(this._call=null,this._time=1/0,m())}},t.now=g,t.timer=f,t.timerFlush=p,t.timeout=function(t,e,n){var r=new d;return e=null==e?0:+e,r.restart(function(n){r.stop(),t(n+e)},e,n),r},t.interval=function(t,e,n){var r=new d,i=e;return null==e?(r.restart(t,e,n),r):(e=+e,n=null==n?g():+n,r.restart(function o(a){a+=i,r.restart(o,i+=e,n),t(a)},e,n),r)},Object.defineProperty(t,"__esModule",{value:!0})}("object"==typeof n&&void 0!==e?n:this.d3=this.d3||{})},{}],163:[function(t,e,n){!function(){var t={version:"3.5.17"},n=[].slice,r=function(t){return n.call(t)},i=this.document;function o(t){return t&&(t.ownerDocument||t.document||t).documentElement}function a(t){return t&&(t.ownerDocument&&t.ownerDocument.defaultView||t.document&&t||t.defaultView)}if(i)try{r(i.documentElement.childNodes)[0].nodeType}catch(t){r=function(t){for(var e=t.length,n=new Array(e);e--;)n[e]=t[e];return n}}if(Date.now||(Date.now=function(){return+new Date}),i)try{i.createElement("DIV").style.setProperty("opacity",0,"")}catch(t){var s=this.Element.prototype,A=s.setAttribute,l=s.setAttributeNS,c=this.CSSStyleDeclaration.prototype,u=c.setProperty;s.setAttribute=function(t,e){A.call(this,t,e+"")},s.setAttributeNS=function(t,e,n){l.call(this,t,e,n+"")},c.setProperty=function(t,e,n){u.call(this,t,e+"",n)}}function g(t,e){return t<e?-1:t>e?1:t>=e?0:NaN}function h(t){return null===t?NaN:+t}function d(t){return!isNaN(t)}function f(t){return{left:function(e,n,r,i){for(arguments.length<3&&(r=0),arguments.length<4&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)<0?r=o+1:i=o}return r},right:function(e,n,r,i){for(arguments.length<3&&(r=0),arguments.length<4&&(i=e.length);r<i;){var o=r+i>>>1;t(e[o],n)>0?i=o:r=o+1}return r}}}t.ascending=g,t.descending=function(t,e){return e<t?-1:e>t?1:e>=t?0:NaN},t.min=function(t,e){var n,r,i=-1,o=t.length;if(1===arguments.length){for(;++i<o;)if(null!=(r=t[i])&&r>=r){n=r;break}for(;++i<o;)null!=(r=t[i])&&n>r&&(n=r)}else{for(;++i<o;)if(null!=(r=e.call(t,t[i],i))&&r>=r){n=r;break}for(;++i<o;)null!=(r=e.call(t,t[i],i))&&n>r&&(n=r)}return n},t.max=function(t,e){var n,r,i=-1,o=t.length;if(1===arguments.length){for(;++i<o;)if(null!=(r=t[i])&&r>=r){n=r;break}for(;++i<o;)null!=(r=t[i])&&r>n&&(n=r)}else{for(;++i<o;)if(null!=(r=e.call(t,t[i],i))&&r>=r){n=r;break}for(;++i<o;)null!=(r=e.call(t,t[i],i))&&r>n&&(n=r)}return n},t.extent=function(t,e){var n,r,i,o=-1,a=t.length;if(1===arguments.length){for(;++o<a;)if(null!=(r=t[o])&&r>=r){n=i=r;break}for(;++o<a;)null!=(r=t[o])&&(n>r&&(n=r),i<r&&(i=r))}else{for(;++o<a;)if(null!=(r=e.call(t,t[o],o))&&r>=r){n=i=r;break}for(;++o<a;)null!=(r=e.call(t,t[o],o))&&(n>r&&(n=r),i<r&&(i=r))}return[n,i]},t.sum=function(t,e){var n,r=0,i=t.length,o=-1;if(1===arguments.length)for(;++o<i;)d(n=+t[o])&&(r+=n);else for(;++o<i;)d(n=+e.call(t,t[o],o))&&(r+=n);return r},t.mean=function(t,e){var n,r=0,i=t.length,o=-1,a=i;if(1===arguments.length)for(;++o<i;)d(n=h(t[o]))?r+=n:--a;else for(;++o<i;)d(n=h(e.call(t,t[o],o)))?r+=n:--a;if(a)return r/a},t.quantile=function(t,e){var n=(t.length-1)*e+1,r=Math.floor(n),i=+t[r-1],o=n-r;return o?i+o*(t[r]-i):i},t.median=function(e,n){var r,i=[],o=e.length,a=-1;if(1===arguments.length)for(;++a<o;)d(r=h(e[a]))&&i.push(r);else for(;++a<o;)d(r=h(n.call(e,e[a],a)))&&i.push(r);if(i.length)return t.quantile(i.sort(g),.5)},t.variance=function(t,e){var n,r,i=t.length,o=0,a=0,s=-1,A=0;if(1===arguments.length)for(;++s<i;)d(n=h(t[s]))&&(a+=(r=n-o)*(n-(o+=r/++A)));else for(;++s<i;)d(n=h(e.call(t,t[s],s)))&&(a+=(r=n-o)*(n-(o+=r/++A)));if(A>1)return a/(A-1)},t.deviation=function(){var e=t.variance.apply(this,arguments);return e?Math.sqrt(e):e};var p=f(g);function I(t){return t.length}t.bisectLeft=p.left,t.bisect=t.bisectRight=p.right,t.bisector=function(t){return f(1===t.length?function(e,n){return g(t(e),n)}:t)},t.shuffle=function(t,e,n){(o=arguments.length)<3&&(n=t.length,o<2&&(e=0));for(var r,i,o=n-e;o;)i=Math.random()*o--|0,r=t[o+e],t[o+e]=t[i+e],t[i+e]=r;return t},t.permute=function(t,e){for(var n=e.length,r=new Array(n);n--;)r[n]=t[e[n]];return r},t.pairs=function(t){for(var e=0,n=t.length-1,r=t[0],i=new Array(n<0?0:n);e<n;)i[e]=[r,r=t[++e]];return i},t.transpose=function(e){if(!(o=e.length))return[];for(var n=-1,r=t.min(e,I),i=new Array(r);++n<r;)for(var o,a=-1,s=i[n]=new Array(o);++a<o;)s[a]=e[a][n];return i},t.zip=function(){return t.transpose(arguments)},t.keys=function(t){var e=[];for(var n in t)e.push(n);return e},t.values=function(t){var e=[];for(var n in t)e.push(t[n]);return e},t.entries=function(t){var e=[];for(var n in t)e.push({key:n,value:t[n]});return e},t.merge=function(t){for(var e,n,r,i=t.length,o=-1,a=0;++o<i;)a+=t[o].length;for(n=new Array(a);--i>=0;)for(e=(r=t[i]).length;--e>=0;)n[--a]=r[e];return n};var C=Math.abs;function m(t,e){for(var n in e)Object.defineProperty(t.prototype,n,{value:e[n],enumerable:!1})}function y(){this._=Object.create(null)}t.range=function(t,e,n){if(arguments.length<3&&(n=1,arguments.length<2&&(e=t,t=0)),(e-t)/n==1/0)throw new Error("infinite range");var r,i=[],o=function(t){for(var e=1;t*e%1;)e*=10;return e}(C(n)),a=-1;if(t*=o,e*=o,(n*=o)<0)for(;(r=t+n*++a)>e;)i.push(r/o);else for(;(r=t+n*++a)<e;)i.push(r/o);return i},t.map=function(t,e){var n=new y;if(t instanceof y)t.forEach(function(t,e){n.set(t,e)});else if(Array.isArray(t)){var r,i=-1,o=t.length;if(1===arguments.length)for(;++i<o;)n.set(i,t[i]);else for(;++i<o;)n.set(e.call(t,r=t[i],i),r)}else for(var a in t)n.set(a,t[a]);return n};var E="__proto__",v="\0";function B(t){return(t+="")===E||t[0]===v?v+t:t}function b(t){return(t+="")[0]===v?t.slice(1):t}function Q(t){return B(t)in this._}function w(t){return(t=B(t))in this._&&delete this._[t]}function x(){var t=[];for(var e in this._)t.push(b(e));return t}function _(){var t=0;for(var e in this._)++t;return t}function D(){for(var t in this._)return!1;return!0}function M(){this._=Object.create(null)}function k(t){return t}function S(t,e,n){return function(){var r=n.apply(e,arguments);return r===e?t:r}}function L(t,e){if(e in t)return e;e=e.charAt(0).toUpperCase()+e.slice(1);for(var n=0,r=R.length;n<r;++n){var i=R[n]+e;if(i in t)return i}}m(y,{has:Q,get:function(t){return this._[B(t)]},set:function(t,e){return this._[B(t)]=e},remove:w,keys:x,values:function(){var t=[];for(var e in this._)t.push(this._[e]);return t},entries:function(){var t=[];for(var e in this._)t.push({key:b(e),value:this._[e]});return t},size:_,empty:D,forEach:function(t){for(var e in this._)t.call(this,b(e),this._[e])}}),t.nest=function(){var e,n,r={},i=[],o=[];function a(t,o,s){if(s>=i.length)return n?n.call(r,o):e?o.sort(e):o;for(var A,l,c,u,g=-1,h=o.length,d=i[s++],f=new y;++g<h;)(u=f.get(A=d(l=o[g])))?u.push(l):f.set(A,[l]);return t?(l=t(),c=function(e,n){l.set(e,a(t,n,s))}):(l={},c=function(e,n){l[e]=a(t,n,s)}),f.forEach(c),l}return r.map=function(t,e){return a(e,t,0)},r.entries=function(e){return function t(e,n){if(n>=i.length)return e;var r=[],a=o[n++];return e.forEach(function(e,i){r.push({key:e,values:t(i,n)})}),a?r.sort(function(t,e){return a(t.key,e.key)}):r}(a(t.map,e,0),0)},r.key=function(t){return i.push(t),r},r.sortKeys=function(t){return o[i.length-1]=t,r},r.sortValues=function(t){return e=t,r},r.rollup=function(t){return n=t,r},r},t.set=function(t){var e=new M;if(t)for(var n=0,r=t.length;n<r;++n)e.add(t[n]);return e},m(M,{has:Q,add:function(t){return this._[B(t+="")]=!0,t},remove:w,values:x,size:_,empty:D,forEach:function(t){for(var e in this._)t.call(this,b(e))}}),t.behavior={},t.rebind=function(t,e){for(var n,r=1,i=arguments.length;++r<i;)t[n=arguments[r]]=S(t,e,e[n]);return t};var R=["webkit","ms","moz","Moz","o","O"];function N(){}function F(){}function T(t){var e=[],n=new y;function r(){for(var n,r=e,i=-1,o=r.length;++i<o;)(n=r[i].on)&&n.apply(this,arguments);return t}return r.on=function(r,i){var o,a=n.get(r);return arguments.length<2?a&&a.on:(a&&(a.on=null,e=e.slice(0,o=e.indexOf(a)).concat(e.slice(o+1)),n.remove(r)),i&&e.push(n.set(r,{on:i})),t)},r}function U(){t.event.preventDefault()}function O(){for(var e,n=t.event;e=n.sourceEvent;)n=e;return n}function G(e){for(var n=new F,r=0,i=arguments.length;++r<i;)n[arguments[r]]=T(n);return n.of=function(r,i){return function(o){try{var a=o.sourceEvent=t.event;o.target=e,t.event=o,n[o.type].apply(r,i)}finally{t.event=a}}},n}t.dispatch=function(){for(var t=new F,e=-1,n=arguments.length;++e<n;)t[arguments[e]]=T(t);return t},F.prototype.on=function(t,e){var n=t.indexOf("."),r="";if(n>=0&&(r=t.slice(n+1),t=t.slice(0,n)),t)return arguments.length<2?this[t].on(r):this[t].on(r,e);if(2===arguments.length){if(null==e)for(t in this)this.hasOwnProperty(t)&&this[t].on(r,null);return this}},t.event=null,t.requote=function(t){return t.replace(H,"\\$&")};var H=/[\\\^\$\*\+\?\|\[\]\(\)\.\{\}]/g,j={}.__proto__?function(t,e){t.__proto__=e}:function(t,e){for(var n in e)t[n]=e[n]};function Y(t){return j(t,J),t}var P=function(t,e){return e.querySelector(t)},K=function(t,e){return e.querySelectorAll(t)},z=function(t,e){var n=t.matches||t[L(t,"matchesSelector")];return(z=function(t,e){return n.call(t,e)})(t,e)};"function"==typeof Sizzle&&(P=function(t,e){return Sizzle(t,e)[0]||null},K=Sizzle,z=Sizzle.matchesSelector),t.selection=function(){return t.select(i.documentElement)};var J=t.selection.prototype=[];function q(t){return"function"==typeof t?t:function(){return P(t,this)}}function V(t){return"function"==typeof t?t:function(){return K(t,this)}}J.select=function(t){var e,n,r,i,o=[];t=q(t);for(var a=-1,s=this.length;++a<s;){o.push(e=[]),e.parentNode=(r=this[a]).parentNode;for(var A=-1,l=r.length;++A<l;)(i=r[A])?(e.push(n=t.call(i,i.__data__,A,a)),n&&"__data__"in i&&(n.__data__=i.__data__)):e.push(null)}return Y(o)},J.selectAll=function(t){var e,n,i=[];t=V(t);for(var o=-1,a=this.length;++o<a;)for(var s=this[o],A=-1,l=s.length;++A<l;)(n=s[A])&&(i.push(e=r(t.call(n,n.__data__,A,o))),e.parentNode=n);return Y(i)};var W="http://www.w3.org/1999/xhtml",X={svg:"http://www.w3.org/2000/svg",xhtml:W,xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"};function Z(e,n){return e=t.ns.qualify(e),null==n?e.local?function(){this.removeAttributeNS(e.space,e.local)}:function(){this.removeAttribute(e)}:"function"==typeof n?e.local?function(){var t=n.apply(this,arguments);null==t?this.removeAttributeNS(e.space,e.local):this.setAttributeNS(e.space,e.local,t)}:function(){var t=n.apply(this,arguments);null==t?this.removeAttribute(e):this.setAttribute(e,t)}:e.local?function(){this.setAttributeNS(e.space,e.local,n)}:function(){this.setAttribute(e,n)}}function $(t){return t.trim().replace(/\s+/g," ")}function tt(e){return new RegExp("(?:^|\\s+)"+t.requote(e)+"(?:\\s+|$)","g")}function et(t){return(t+"").trim().split(/^|\s+/)}function nt(t,e){var n=(t=et(t).map(rt)).length;return"function"==typeof e?function(){for(var r=-1,i=e.apply(this,arguments);++r<n;)t[r](this,i)}:function(){for(var r=-1;++r<n;)t[r](this,e)}}function rt(t){var e=tt(t);return function(n,r){if(i=n.classList)return r?i.add(t):i.remove(t);var i=n.getAttribute("class")||"";r?(e.lastIndex=0,e.test(i)||n.setAttribute("class",$(i+" "+t))):n.setAttribute("class",$(i.replace(e," ")))}}function it(t,e,n){return null==e?function(){this.style.removeProperty(t)}:"function"==typeof e?function(){var r=e.apply(this,arguments);null==r?this.style.removeProperty(t):this.style.setProperty(t,r,n)}:function(){this.style.setProperty(t,e,n)}}function ot(t,e){return null==e?function(){delete this[t]}:"function"==typeof e?function(){var n=e.apply(this,arguments);null==n?delete this[t]:this[t]=n}:function(){this[t]=e}}function at(e){return"function"==typeof e?e:(e=t.ns.qualify(e)).local?function(){return this.ownerDocument.createElementNS(e.space,e.local)}:function(){var t=this.ownerDocument,n=this.namespaceURI;return n===W&&t.documentElement.namespaceURI===W?t.createElement(e):t.createElementNS(n,e)}}function st(){var t=this.parentNode;t&&t.removeChild(this)}function At(t){return{__data__:t}}function lt(t){return function(){return z(this,t)}}function ct(t,e){for(var n=0,r=t.length;n<r;n++)for(var i,o=t[n],a=0,s=o.length;a<s;a++)(i=o[a])&&e(i,a,n);return t}function ut(t){return j(t,gt),t}t.ns={prefix:X,qualify:function(t){var e=t.indexOf(":"),n=t;return e>=0&&"xmlns"!==(n=t.slice(0,e))&&(t=t.slice(e+1)),X.hasOwnProperty(n)?{space:X[n],local:t}:t}},J.attr=function(e,n){if(arguments.length<2){if("string"==typeof e){var r=this.node();return(e=t.ns.qualify(e)).local?r.getAttributeNS(e.space,e.local):r.getAttribute(e)}for(n in e)this.each(Z(n,e[n]));return this}return this.each(Z(e,n))},J.classed=function(t,e){if(arguments.length<2){if("string"==typeof t){var n=this.node(),r=(t=et(t)).length,i=-1;if(e=n.classList){for(;++i<r;)if(!e.contains(t[i]))return!1}else for(e=n.getAttribute("class");++i<r;)if(!tt(t[i]).test(e))return!1;return!0}for(e in t)this.each(nt(e,t[e]));return this}return this.each(nt(t,e))},J.style=function(t,e,n){var r=arguments.length;if(r<3){if("string"!=typeof t){for(n in r<2&&(e=""),t)this.each(it(n,t[n],e));return this}if(r<2){var i=this.node();return a(i).getComputedStyle(i,null).getPropertyValue(t)}n=""}return this.each(it(t,e,n))},J.property=function(t,e){if(arguments.length<2){if("string"==typeof t)return this.node()[t];for(e in t)this.each(ot(e,t[e]));return this}return this.each(ot(t,e))},J.text=function(t){return arguments.length?this.each("function"==typeof t?function(){var e=t.apply(this,arguments);this.textContent=null==e?"":e}:null==t?function(){this.textContent=""}:function(){this.textContent=t}):this.node().textContent},J.html=function(t){return arguments.length?this.each("function"==typeof t?function(){var e=t.apply(this,arguments);this.innerHTML=null==e?"":e}:null==t?function(){this.innerHTML=""}:function(){this.innerHTML=t}):this.node().innerHTML},J.append=function(t){return t=at(t),this.select(function(){return this.appendChild(t.apply(this,arguments))})},J.insert=function(t,e){return t=at(t),e=q(e),this.select(function(){return this.insertBefore(t.apply(this,arguments),e.apply(this,arguments)||null)})},J.remove=function(){return this.each(st)},J.data=function(t,e){var n,r,i=-1,o=this.length;if(!arguments.length){for(t=new Array(o=(n=this[0]).length);++i<o;)(r=n[i])&&(t[i]=r.__data__);return t}function a(t,n){var r,i,o,a=t.length,c=n.length,u=Math.min(a,c),g=new Array(c),h=new Array(c),d=new Array(a);if(e){var f,p=new y,I=new Array(a);for(r=-1;++r<a;)(i=t[r])&&(p.has(f=e.call(i,i.__data__,r))?d[r]=i:p.set(f,i),I[r]=f);for(r=-1;++r<c;)(i=p.get(f=e.call(n,o=n[r],r)))?!0!==i&&(g[r]=i,i.__data__=o):h[r]=At(o),p.set(f,!0);for(r=-1;++r<a;)r in I&&!0!==p.get(I[r])&&(d[r]=t[r])}else{for(r=-1;++r<u;)i=t[r],o=n[r],i?(i.__data__=o,g[r]=i):h[r]=At(o);for(;r<c;++r)h[r]=At(n[r]);for(;r<a;++r)d[r]=t[r]}h.update=g,h.parentNode=g.parentNode=d.parentNode=t.parentNode,s.push(h),A.push(g),l.push(d)}var s=ut([]),A=Y([]),l=Y([]);if("function"==typeof t)for(;++i<o;)a(n=this[i],t.call(n,n.parentNode.__data__,i));else for(;++i<o;)a(n=this[i],t);return A.enter=function(){return s},A.exit=function(){return l},A},J.datum=function(t){return arguments.length?this.property("__data__",t):this.property("__data__")},J.filter=function(t){var e,n,r,i=[];"function"!=typeof t&&(t=lt(t));for(var o=0,a=this.length;o<a;o++){i.push(e=[]),e.parentNode=(n=this[o]).parentNode;for(var s=0,A=n.length;s<A;s++)(r=n[s])&&t.call(r,r.__data__,s,o)&&e.push(r)}return Y(i)},J.order=function(){for(var t=-1,e=this.length;++t<e;)for(var n,r=this[t],i=r.length-1,o=r[i];--i>=0;)(n=r[i])&&(o&&o!==n.nextSibling&&o.parentNode.insertBefore(n,o),o=n);return this},J.sort=function(t){t=function(t){return arguments.length||(t=g),function(e,n){return e&&n?t(e.__data__,n.__data__):!e-!n}}.apply(this,arguments);for(var e=-1,n=this.length;++e<n;)this[e].sort(t);return this.order()},J.each=function(t){return ct(this,function(e,n,r){t.call(e,e.__data__,n,r)})},J.call=function(t){var e=r(arguments);return t.apply(e[0]=this,e),this},J.empty=function(){return!this.node()},J.node=function(){for(var t=0,e=this.length;t<e;t++)for(var n=this[t],r=0,i=n.length;r<i;r++){var o=n[r];if(o)return o}return null},J.size=function(){var t=0;return ct(this,function(){++t}),t};var gt=[];function ht(e,n,i){var o="__on"+e,a=e.indexOf("."),s=ft;a>0&&(e=e.slice(0,a));var A=dt.get(e);function l(){var t=this[o];t&&(this.removeEventListener(e,t,t.$),delete this[o])}return A&&(e=A,s=pt),a?n?function(){var t=s(n,r(arguments));l.call(this),this.addEventListener(e,this[o]=t,t.$=i),t._=n}:l:n?N:function(){var n,r=new RegExp("^__on([^.]+)"+t.requote(e)+"$");for(var i in this)if(n=i.match(r)){var o=this[i];this.removeEventListener(n[1],o,o.$),delete this[i]}}}t.selection.enter=ut,t.selection.enter.prototype=gt,gt.append=J.append,gt.empty=J.empty,gt.node=J.node,gt.call=J.call,gt.size=J.size,gt.select=function(t){for(var e,n,r,i,o,a=[],s=-1,A=this.length;++s<A;){r=(i=this[s]).update,a.push(e=[]),e.parentNode=i.parentNode;for(var l=-1,c=i.length;++l<c;)(o=i[l])?(e.push(r[l]=n=t.call(i.parentNode,o.__data__,l,s)),n.__data__=o.__data__):e.push(null)}return Y(a)},gt.insert=function(t,e){var n,r,i;return arguments.length<2&&(n=this,e=function(t,e,o){var a,s=n[o].update,A=s.length;for(o!=i&&(i=o,r=0),e>=r&&(r=e+1);!(a=s[r])&&++r<A;);return a}),J.insert.call(this,t,e)},t.select=function(t){var e;return"string"==typeof t?(e=[P(t,i)]).parentNode=i.documentElement:(e=[t]).parentNode=o(t),Y([e])},t.selectAll=function(t){var e;return"string"==typeof t?(e=r(K(t,i))).parentNode=i.documentElement:(e=r(t)).parentNode=null,Y([e])},J.on=function(t,e,n){var r=arguments.length;if(r<3){if("string"!=typeof t){for(n in r<2&&(e=!1),t)this.each(ht(n,t[n],e));return this}if(r<2)return(r=this.node()["__on"+t])&&r._;n=!1}return this.each(ht(t,e,n))};var dt=t.map({mouseenter:"mouseover",mouseleave:"mouseout"});function ft(e,n){return function(r){var i=t.event;t.event=r,n[0]=this.__data__;try{e.apply(this,n)}finally{t.event=i}}}function pt(t,e){var n=ft(t,e);return function(t){var e=t.relatedTarget;e&&(e===this||8&e.compareDocumentPosition(this))||n.call(this,t)}}i&&dt.forEach(function(t){"on"+t in i&&dt.remove(t)});var It,Ct=0;function mt(e){var n=".dragsuppress-"+ ++Ct,r="click"+n,i=t.select(a(e)).on("touchmove"+n,U).on("dragstart"+n,U).on("selectstart"+n,U);if(null==It&&(It=!("onselectstart"in e)&&L(e.style,"userSelect")),It){var s=o(e).style,A=s[It];s[It]="none"}return function(t){if(i.on(n,null),It&&(s[It]=A),t){var e=function(){i.on(r,null)};i.on(r,function(){U(),e()},!0),setTimeout(e,0)}}}t.mouse=function(t){return Et(t,O())};var yt=this.navigator&&/WebKit/.test(this.navigator.userAgent)?-1:0;function Et(e,n){n.changedTouches&&(n=n.changedTouches[0]);var r=e.ownerSVGElement||e;if(r.createSVGPoint){var i=r.createSVGPoint();if(yt<0){var o=a(e);if(o.scrollX||o.scrollY){var s=(r=t.select("body").append("svg").style({position:"absolute",top:0,left:0,margin:0,padding:0,border:"none"},"important"))[0][0].getScreenCTM();yt=!(s.f||s.e),r.remove()}}return yt?(i.x=n.pageX,i.y=n.pageY):(i.x=n.clientX,i.y=n.clientY),[(i=i.matrixTransform(e.getScreenCTM().inverse())).x,i.y]}var A=e.getBoundingClientRect();return[n.clientX-A.left-e.clientLeft,n.clientY-A.top-e.clientTop]}function vt(){return t.event.changedTouches[0].identifier}t.touch=function(t,e,n){if(arguments.length<3&&(n=e,e=O().changedTouches),e)for(var r,i=0,o=e.length;i<o;++i)if((r=e[i]).identifier===n)return Et(t,r)},t.behavior.drag=function(){var e=G(o,"drag","dragstart","dragend"),n=null,r=s(N,t.mouse,a,"mousemove","mouseup"),i=s(vt,t.touch,k,"touchmove","touchend");function o(){this.on("mousedown.drag",r).on("touchstart.drag",i)}function s(r,i,o,a,s){return function(){var A,l=t.event.target.correspondingElement||t.event.target,c=this.parentNode,u=e.of(this,arguments),g=0,h=r(),d=".drag"+(null==h?"":"-"+h),f=t.select(o(l)).on(a+d,function(){var t,e,n=i(c,h);n&&(t=n[0]-I[0],e=n[1]-I[1],g|=t|e,I=n,u({type:"drag",x:n[0]+A[0],y:n[1]+A[1],dx:t,dy:e}))}).on(s+d,function(){i(c,h)&&(f.on(a+d,null).on(s+d,null),p(g),u({type:"dragend"}))}),p=mt(l),I=i(c,h);A=n?[(A=n.apply(this,arguments)).x-I[0],A.y-I[1]]:[0,0],u({type:"dragstart"})}}return o.origin=function(t){return arguments.length?(n=t,o):n},t.rebind(o,e,"on")},t.touches=function(t,e){return arguments.length<2&&(e=O().touches),e?r(e).map(function(e){var n=Et(t,e);return n.identifier=e.identifier,n}):[]};var Bt=1e-6,bt=Bt*Bt,Qt=Math.PI,wt=2*Qt,xt=wt-Bt,_t=Qt/2,Dt=Qt/180,Mt=180/Qt;function kt(t){return t>0?1:t<0?-1:0}function St(t,e,n){return(e[0]-t[0])*(n[1]-t[1])-(e[1]-t[1])*(n[0]-t[0])}function Lt(t){return t>1?0:t<-1?Qt:Math.acos(t)}function Rt(t){return t>1?_t:t<-1?-_t:Math.asin(t)}function Nt(t){return((t=Math.exp(t))+1/t)/2}function Ft(t){return(t=Math.sin(t/2))*t}var Tt=Math.SQRT2;t.interpolateZoom=function(t,e){var n,r,i=t[0],o=t[1],a=t[2],s=e[0],A=e[1],l=e[2],c=s-i,u=A-o,g=c*c+u*u;if(g<bt)r=Math.log(l/a)/Tt,n=function(t){return[i+t*c,o+t*u,a*Math.exp(Tt*t*r)]};else{var h=Math.sqrt(g),d=(l*l-a*a+4*g)/(2*a*2*h),f=(l*l-a*a-4*g)/(2*l*2*h),p=Math.log(Math.sqrt(d*d+1)-d),I=Math.log(Math.sqrt(f*f+1)-f);r=(I-p)/Tt,n=function(t){var e,n=t*r,s=Nt(p),A=a/(2*h)*(s*(e=Tt*n+p,((e=Math.exp(2*e))-1)/(e+1))-function(t){return((t=Math.exp(t))-1/t)/2}(p));return[i+A*c,o+A*u,a*s/Nt(Tt*n+p)]}}return n.duration=1e3*r,n},t.behavior.zoom=function(){var e,n,r,o,s,A,l,c,u,g={x:0,y:0,k:1},h=[960,500],d=Gt,f=250,p=0,I="mousedown.zoom",C="mousemove.zoom",m="mouseup.zoom",y="touchstart.zoom",E=G(v,"zoomstart","zoom","zoomend");function v(t){t.on(I,k).on(Ot+".zoom",L).on("dblclick.zoom",R).on(y,S)}function B(t){return[(t[0]-g.x)/g.k,(t[1]-g.y)/g.k]}function b(t){g.k=Math.max(d[0],Math.min(d[1],t))}function Q(t,e){e=function(t){return[t[0]*g.k+g.x,t[1]*g.k+g.y]}(e),g.x+=t[0]-e[0],g.y+=t[1]-e[1]}function w(e,r,i,o){e.__chart__={x:g.x,y:g.y,k:g.k},b(Math.pow(2,o)),Q(n=r,i),e=t.select(e),f>0&&(e=e.transition().duration(f)),e.call(v.event)}function x(){l&&l.domain(A.range().map(function(t){return(t-g.x)/g.k}).map(A.invert)),u&&u.domain(c.range().map(function(t){return(t-g.y)/g.k}).map(c.invert))}function _(t){p++||t({type:"zoomstart"})}function D(t){x(),t({type:"zoom",scale:g.k,translate:[g.x,g.y]})}function M(t){--p||(t({type:"zoomend"}),n=null)}function k(){var e=this,n=E.of(e,arguments),r=0,i=t.select(a(e)).on(C,function(){r=1,Q(t.mouse(e),o),D(n)}).on(m,function(){i.on(C,null).on(m,null),s(r),M(n)}),o=B(t.mouse(e)),s=mt(e);cs.call(e),_(n)}function S(){var e,n=this,r=E.of(n,arguments),i={},o=0,a=".zoom-"+t.event.changedTouches[0].identifier,A="touchmove"+a,l="touchend"+a,c=[],u=t.select(n),h=mt(n);function d(){var r=t.touches(n);return e=g.k,r.forEach(function(t){t.identifier in i&&(i[t.identifier]=B(t))}),r}function f(){var e=t.event.target;t.select(e).on(A,p).on(l,C),c.push(e);for(var r=t.event.changedTouches,a=0,u=r.length;a<u;++a)i[r[a].identifier]=null;var h=d(),f=Date.now();if(1===h.length){if(f-s<500){var I=h[0];w(n,I,i[I.identifier],Math.floor(Math.log(g.k)/Math.LN2)+1),U()}s=f}else if(h.length>1){I=h[0];var m=h[1],y=I[0]-m[0],E=I[1]-m[1];o=y*y+E*E}}function p(){var a,A,l,c,u=t.touches(n);cs.call(n);for(var g=0,h=u.length;g<h;++g,c=null)if(l=u[g],c=i[l.identifier]){if(A)break;a=l,A=c}if(c){var d=(d=l[0]-a[0])*d+(d=l[1]-a[1])*d,f=o&&Math.sqrt(d/o);a=[(a[0]+l[0])/2,(a[1]+l[1])/2],A=[(A[0]+c[0])/2,(A[1]+c[1])/2],b(f*e)}s=null,Q(a,A),D(r)}function C(){if(t.event.touches.length){for(var e=t.event.changedTouches,n=0,o=e.length;n<o;++n)delete i[e[n].identifier];for(var s in i)return void d()}t.selectAll(c).on(a,null),u.on(I,k).on(y,S),h(),M(r)}f(),_(r),u.on(I,null).on(y,f)}function L(){var i=E.of(this,arguments);o?clearTimeout(o):(cs.call(this),e=B(n=r||t.mouse(this)),_(i)),o=setTimeout(function(){o=null,M(i)},50),U(),b(Math.pow(2,.002*Ut())*g.k),Q(n,e),D(i)}function R(){var e=t.mouse(this),n=Math.log(g.k)/Math.LN2;w(this,e,B(e),t.event.shiftKey?Math.ceil(n)-1:Math.floor(n)+1)}return Ot||(Ot="onwheel"in i?(Ut=function(){return-t.event.deltaY*(t.event.deltaMode?120:1)},"wheel"):"onmousewheel"in i?(Ut=function(){return t.event.wheelDelta},"mousewheel"):(Ut=function(){return-t.event.detail},"MozMousePixelScroll")),v.event=function(e){e.each(function(){var e=E.of(this,arguments),r=g;hs?t.select(this).transition().each("start.zoom",function(){g=this.__chart__||{x:0,y:0,k:1},_(e)}).tween("zoom:zoom",function(){var i=h[0],o=h[1],a=n?n[0]:i/2,s=n?n[1]:o/2,A=t.interpolateZoom([(a-g.x)/g.k,(s-g.y)/g.k,i/g.k],[(a-r.x)/r.k,(s-r.y)/r.k,i/r.k]);return function(t){var n=A(t),r=i/n[2];this.__chart__=g={x:a-n[0]*r,y:s-n[1]*r,k:r},D(e)}}).each("interrupt.zoom",function(){M(e)}).each("end.zoom",function(){M(e)}):(this.__chart__=g,_(e),D(e),M(e))})},v.translate=function(t){return arguments.length?(g={x:+t[0],y:+t[1],k:g.k},x(),v):[g.x,g.y]},v.scale=function(t){return arguments.length?(g={x:g.x,y:g.y,k:null},b(+t),x(),v):g.k},v.scaleExtent=function(t){return arguments.length?(d=null==t?Gt:[+t[0],+t[1]],v):d},v.center=function(t){return arguments.length?(r=t&&[+t[0],+t[1]],v):r},v.size=function(t){return arguments.length?(h=t&&[+t[0],+t[1]],v):h},v.duration=function(t){return arguments.length?(f=+t,v):f},v.x=function(t){return arguments.length?(l=t,A=t.copy(),g={x:0,y:0,k:1},v):l},v.y=function(t){return arguments.length?(u=t,c=t.copy(),g={x:0,y:0,k:1},v):u},t.rebind(v,E,"on")};var Ut,Ot,Gt=[0,1/0];function Ht(){}function jt(t,e,n){return this instanceof jt?(this.h=+t,this.s=+e,void(this.l=+n)):arguments.length<2?t instanceof jt?new jt(t.h,t.s,t.l):le(""+t,ce,jt):new jt(t,e,n)}t.color=Ht,Ht.prototype.toString=function(){return this.rgb()+""},t.hsl=jt;var Yt=jt.prototype=new Ht;function Pt(t,e,n){var r,i;function o(t){return Math.round(255*function(t){return t>360?t-=360:t<0&&(t+=360),t<60?r+(i-r)*t/60:t<180?i:t<240?r+(i-r)*(240-t)/60:r}(t))}return t=isNaN(t)?0:(t%=360)<0?t+360:t,e=isNaN(e)?0:e<0?0:e>1?1:e,r=2*(n=n<0?0:n>1?1:n)-(i=n<=.5?n*(1+e):n+e-n*e),new ie(o(t+120),o(t),o(t-120))}function Kt(e,n,r){return this instanceof Kt?(this.h=+e,this.c=+n,void(this.l=+r)):arguments.length<2?e instanceof Kt?new Kt(e.h,e.c,e.l):function(t,e,n){return t>0?new Kt(Math.atan2(n,e)*Mt,Math.sqrt(e*e+n*n),t):new Kt(NaN,NaN,t)}(e instanceof qt?e.l:(e=ue((e=t.rgb(e)).r,e.g,e.b)).l,e.a,e.b):new Kt(e,n,r)}Yt.brighter=function(t){return t=Math.pow(.7,arguments.length?t:1),new jt(this.h,this.s,this.l/t)},Yt.darker=function(t){return t=Math.pow(.7,arguments.length?t:1),new jt(this.h,this.s,t*this.l)},Yt.rgb=function(){return Pt(this.h,this.s,this.l)},t.hcl=Kt;var zt=Kt.prototype=new Ht;function Jt(t,e,n){return isNaN(t)&&(t=0),isNaN(e)&&(e=0),new qt(n,Math.cos(t*=Dt)*e,Math.sin(t)*e)}function qt(t,e,n){return this instanceof qt?(this.l=+t,this.a=+e,void(this.b=+n)):arguments.length<2?t instanceof qt?new qt(t.l,t.a,t.b):t instanceof Kt?Jt(t.h,t.c,t.l):ue((t=ie(t)).r,t.g,t.b):new qt(t,e,n)}zt.brighter=function(t){return new Kt(this.h,this.c,Math.min(100,this.l+Vt*(arguments.length?t:1)))},zt.darker=function(t){return new Kt(this.h,this.c,Math.max(0,this.l-Vt*(arguments.length?t:1)))},zt.rgb=function(){return Jt(this.h,this.c,this.l).rgb()},t.lab=qt;var Vt=18,Wt=.95047,Xt=1,Zt=1.08883,$t=qt.prototype=new Ht;function te(t,e,n){var r=(t+16)/116,i=r+e/500,o=r-n/200;return new ie(re(3.2404542*(i=ee(i)*Wt)-1.5371385*(r=ee(r)*Xt)-.4985314*(o=ee(o)*Zt)),re(-.969266*i+1.8760108*r+.041556*o),re(.0556434*i-.2040259*r+1.0572252*o))}function ee(t){return t>.206893034?t*t*t:(t-4/29)/7.787037}function ne(t){return t>.008856?Math.pow(t,1/3):7.787037*t+4/29}function re(t){return Math.round(255*(t<=.00304?12.92*t:1.055*Math.pow(t,1/2.4)-.055))}function ie(t,e,n){return this instanceof ie?(this.r=~~t,this.g=~~e,void(this.b=~~n)):arguments.length<2?t instanceof ie?new ie(t.r,t.g,t.b):le(""+t,ie,Pt):new ie(t,e,n)}function oe(t){return new ie(t>>16,t>>8&255,255&t)}function ae(t){return oe(t)+""}$t.brighter=function(t){return new qt(Math.min(100,this.l+Vt*(arguments.length?t:1)),this.a,this.b)},$t.darker=function(t){return new qt(Math.max(0,this.l-Vt*(arguments.length?t:1)),this.a,this.b)},$t.rgb=function(){return te(this.l,this.a,this.b)},t.rgb=ie;var se=ie.prototype=new Ht;function Ae(t){return t<16?"0"+Math.max(0,t).toString(16):Math.min(255,t).toString(16)}function le(t,e,n){var r,i,o,a=0,s=0,A=0;if(r=/([a-z]+)\((.*)\)/.exec(t=t.toLowerCase()))switch(i=r[2].split(","),r[1]){case"hsl":return n(parseFloat(i[0]),parseFloat(i[1])/100,parseFloat(i[2])/100);case"rgb":return e(he(i[0]),he(i[1]),he(i[2]))}return(o=de.get(t))?e(o.r,o.g,o.b):(null==t||"#"!==t.charAt(0)||isNaN(o=parseInt(t.slice(1),16))||(4===t.length?(a=(3840&o)>>4,a|=a>>4,s=240&o,s|=s>>4,A=15&o,A|=A<<4):7===t.length&&(a=(16711680&o)>>16,s=(65280&o)>>8,A=255&o)),e(a,s,A))}function ce(t,e,n){var r,i,o=Math.min(t/=255,e/=255,n/=255),a=Math.max(t,e,n),s=a-o,A=(a+o)/2;return s?(i=A<.5?s/(a+o):s/(2-a-o),r=t==a?(e-n)/s+(e<n?6:0):e==a?(n-t)/s+2:(t-e)/s+4,r*=60):(r=NaN,i=A>0&&A<1?0:r),new jt(r,i,A)}function ue(t,e,n){var r=ne((.4124564*(t=ge(t))+.3575761*(e=ge(e))+.1804375*(n=ge(n)))/Wt),i=ne((.2126729*t+.7151522*e+.072175*n)/Xt);return qt(116*i-16,500*(r-i),200*(i-ne((.0193339*t+.119192*e+.9503041*n)/Zt)))}function ge(t){return(t/=255)<=.04045?t/12.92:Math.pow((t+.055)/1.055,2.4)}function he(t){var e=parseFloat(t);return"%"===t.charAt(t.length-1)?Math.round(2.55*e):e}se.brighter=function(t){t=Math.pow(.7,arguments.length?t:1);var e=this.r,n=this.g,r=this.b,i=30;return e||n||r?(e&&e<i&&(e=i),n&&n<i&&(n=i),r&&r<i&&(r=i),new ie(Math.min(255,e/t),Math.min(255,n/t),Math.min(255,r/t))):new ie(i,i,i)},se.darker=function(t){return new ie((t=Math.pow(.7,arguments.length?t:1))*this.r,t*this.g,t*this.b)},se.hsl=function(){return ce(this.r,this.g,this.b)},se.toString=function(){return"#"+Ae(this.r)+Ae(this.g)+Ae(this.b)};var de=t.map({aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074});function fe(t){return"function"==typeof t?t:function(){return t}}function pe(t){return function(e,n,r){return 2===arguments.length&&"function"==typeof n&&(r=n,n=null),Ie(e,n,t,r)}}function Ie(e,n,i,o){var a={},s=t.dispatch("beforesend","progress","load","error"),A={},l=new XMLHttpRequest,c=null;function u(){var t,e=l.status;if(!e&&function(t){var e=t.responseType;return e&&"text"!==e?t.response:t.responseText}(l)||e>=200&&e<300||304===e){try{t=i.call(a,l)}catch(t){return void s.error.call(a,t)}s.load.call(a,t)}else s.error.call(a,l)}return!this.XDomainRequest||"withCredentials"in l||!/^(http(s)?:)?\/\//.test(e)||(l=new XDomainRequest),"onload"in l?l.onload=l.onerror=u:l.onreadystatechange=function(){l.readyState>3&&u()},l.onprogress=function(e){var n=t.event;t.event=e;try{s.progress.call(a,l)}finally{t.event=n}},a.header=function(t,e){return t=(t+"").toLowerCase(),arguments.length<2?A[t]:(null==e?delete A[t]:A[t]=e+"",a)},a.mimeType=function(t){return arguments.length?(n=null==t?null:t+"",a):n},a.responseType=function(t){return arguments.length?(c=t,a):c},a.response=function(t){return i=t,a},["get","post"].forEach(function(t){a[t]=function(){return a.send.apply(a,[t].concat(r(arguments)))}}),a.send=function(t,r,i){if(2===arguments.length&&"function"==typeof r&&(i=r,r=null),l.open(t,e,!0),null==n||"accept"in A||(A.accept=n+",*/*"),l.setRequestHeader)for(var o in A)l.setRequestHeader(o,A[o]);return null!=n&&l.overrideMimeType&&l.overrideMimeType(n),null!=c&&(l.responseType=c),null!=i&&a.on("error",i).on("load",function(t){i(null,t)}),s.beforesend.call(a,l),l.send(null==r?null:r),a},a.abort=function(){return l.abort(),a},t.rebind(a,s,"on"),null==o?a:a.get(function(t){return 1===t.length?function(e,n){t(null==e?n:null)}:t}(o))}de.forEach(function(t,e){de.set(t,oe(e))}),t.functor=fe,t.xhr=pe(k),t.dsv=function(t,e){var n=new RegExp('["'+t+"\n]"),r=t.charCodeAt(0);function i(t,n,r){arguments.length<3&&(r=n,n=null);var i=Ie(t,e,null==n?o:a(n),r);return i.row=function(t){return arguments.length?i.response(null==(n=t)?o:a(t)):n},i}function o(t){return i.parse(t.responseText)}function a(t){return function(e){return i.parse(e.responseText,t)}}function s(e){return e.map(A).join(t)}function A(t){return n.test(t)?'"'+t.replace(/\"/g,'""')+'"':t}return i.parse=function(t,e){var n;return i.parseRows(t,function(t,r){if(n)return n(t,r-1);var i=new Function("d","return {"+t.map(function(t,e){return JSON.stringify(t)+": d["+e+"]"}).join(",")+"}");n=e?function(t,n){return e(i(t),n)}:i})},i.parseRows=function(t,e){var n,i,o={},a={},s=[],A=t.length,l=0,c=0;function u(){if(l>=A)return a;if(i)return i=!1,o;var e=l;if(34===t.charCodeAt(e)){for(var n=e;n++<A;)if(34===t.charCodeAt(n)){if(34!==t.charCodeAt(n+1))break;++n}return l=n+2,13===(s=t.charCodeAt(n+1))?(i=!0,10===t.charCodeAt(n+2)&&++l):10===s&&(i=!0),t.slice(e+1,n).replace(/""/g,'"')}for(;l<A;){var s,c=1;if(10===(s=t.charCodeAt(l++)))i=!0;else if(13===s)i=!0,10===t.charCodeAt(l)&&(++l,++c);else if(s!==r)continue;return t.slice(e,l-c)}return t.slice(e)}for(;(n=u())!==a;){for(var g=[];n!==o&&n!==a;)g.push(n),n=u();e&&null==(g=e(g,c++))||s.push(g)}return s},i.format=function(e){if(Array.isArray(e[0]))return i.formatRows(e);var n=new M,r=[];return e.forEach(function(t){for(var e in t)n.has(e)||r.push(n.add(e))}),[r.map(A).join(t)].concat(e.map(function(e){return r.map(function(t){return A(e[t])}).join(t)})).join("\n")},i.formatRows=function(t){return t.map(s).join("\n")},i},t.csv=t.dsv(",","text/csv"),t.tsv=t.dsv("\t","text/tab-separated-values");var Ce,me,ye,Ee,ve=this[L(this,"requestAnimationFrame")]||function(t){setTimeout(t,17)};function Be(t,e,n){var r=arguments.length;r<2&&(e=0),r<3&&(n=Date.now());var i={c:t,t:n+e,n:null};return me?me.n=i:Ce=i,me=i,ye||(Ee=clearTimeout(Ee),ye=1,ve(be)),i}function be(){var t=Qe(),e=we()-t;e>24?(isFinite(e)&&(clearTimeout(Ee),Ee=setTimeout(be,e)),ye=0):(ye=1,ve(be))}function Qe(){for(var t=Date.now(),e=Ce;e;)t>=e.t&&e.c(t-e.t)&&(e.c=null),e=e.n;return t}function we(){for(var t,e=Ce,n=1/0;e;)e.c?(e.t<n&&(n=e.t),e=(t=e).n):e=t?t.n=e.n:Ce=e.n;return me=t,n}function xe(t,e){return e-(t?Math.ceil(Math.log(t)/Math.LN10):1)}t.timer=function(){Be.apply(this,arguments)},t.timer.flush=function(){Qe(),we()},t.round=function(t,e){return e?Math.round(t*(e=Math.pow(10,e)))/e:Math.round(t)};var _e=["y","z","a","f","p","n","µ","m","","k","M","G","T","P","E","Z","Y"].map(function(t,e){var n=Math.pow(10,3*C(8-e));return{scale:e>8?function(t){return t/n}:function(t){return t*n},symbol:t}});t.formatPrefix=function(e,n){var r=0;return(e=+e)&&(e<0&&(e*=-1),n&&(e=t.round(e,xe(e,n))),r=1+Math.floor(1e-12+Math.log(e)/Math.LN10),r=Math.max(-24,Math.min(24,3*Math.floor((r-1)/3)))),_e[8+r/3]};var De=/(?:([^{])?([<>=^]))?([+\- ])?([$#])?(0)?(\d+)?(,)?(\.-?\d+)?([a-z%])?/i,Me=t.map({b:function(t){return t.toString(2)},c:function(t){return String.fromCharCode(t)},o:function(t){return t.toString(8)},x:function(t){return t.toString(16)},X:function(t){return t.toString(16).toUpperCase()},g:function(t,e){return t.toPrecision(e)},e:function(t,e){return t.toExponential(e)},f:function(t,e){return t.toFixed(e)},r:function(e,n){return(e=t.round(e,xe(e,n))).toFixed(Math.max(0,Math.min(20,xe(e*(1+1e-15),n))))}});function ke(t){return t+""}var Se=t.time={},Le=Date;function Re(){this._=new Date(arguments.length>1?Date.UTC.apply(this,arguments):arguments[0])}Re.prototype={getDate:function(){return this._.getUTCDate()},getDay:function(){return this._.getUTCDay()},getFullYear:function(){return this._.getUTCFullYear()},getHours:function(){return this._.getUTCHours()},getMilliseconds:function(){return this._.getUTCMilliseconds()},getMinutes:function(){return this._.getUTCMinutes()},getMonth:function(){return this._.getUTCMonth()},getSeconds:function(){return this._.getUTCSeconds()},getTime:function(){return this._.getTime()},getTimezoneOffset:function(){return 0},valueOf:function(){return this._.valueOf()},setDate:function(){Ne.setUTCDate.apply(this._,arguments)},setDay:function(){Ne.setUTCDay.apply(this._,arguments)},setFullYear:function(){Ne.setUTCFullYear.apply(this._,arguments)},setHours:function(){Ne.setUTCHours.apply(this._,arguments)},setMilliseconds:function(){Ne.setUTCMilliseconds.apply(this._,arguments)},setMinutes:function(){Ne.setUTCMinutes.apply(this._,arguments)},setMonth:function(){Ne.setUTCMonth.apply(this._,arguments)},setSeconds:function(){Ne.setUTCSeconds.apply(this._,arguments)},setTime:function(){Ne.setTime.apply(this._,arguments)}};var Ne=Date.prototype;function Fe(t,e,n){function r(e){var n=t(e),r=o(n,1);return e-n<r-e?n:r}function i(n){return e(n=t(new Le(n-1)),1),n}function o(t,n){return e(t=new Le(+t),n),t}function a(t,r,o){var a=i(t),s=[];if(o>1)for(;a<r;)n(a)%o||s.push(new Date(+a)),e(a,1);else for(;a<r;)s.push(new Date(+a)),e(a,1);return s}t.floor=t,t.round=r,t.ceil=i,t.offset=o,t.range=a;var s=t.utc=Te(t);return s.floor=s,s.round=Te(r),s.ceil=Te(i),s.offset=Te(o),s.range=function(t,e,n){try{Le=Re;var r=new Re;return r._=t,a(r,e,n)}finally{Le=Date}},t}function Te(t){return function(e,n){try{Le=Re;var r=new Re;return r._=e,t(r,n)._}finally{Le=Date}}}Se.year=Fe(function(t){return(t=Se.day(t)).setMonth(0,1),t},function(t,e){t.setFullYear(t.getFullYear()+e)},function(t){return t.getFullYear()}),Se.years=Se.year.range,Se.years.utc=Se.year.utc.range,Se.day=Fe(function(t){var e=new Le(2e3,0);return e.setFullYear(t.getFullYear(),t.getMonth(),t.getDate()),e},function(t,e){t.setDate(t.getDate()+e)},function(t){return t.getDate()-1}),Se.days=Se.day.range,Se.days.utc=Se.day.utc.range,Se.dayOfYear=function(t){var e=Se.year(t);return Math.floor((t-e-6e4*(t.getTimezoneOffset()-e.getTimezoneOffset()))/864e5)},["sunday","monday","tuesday","wednesday","thursday","friday","saturday"].forEach(function(t,e){e=7-e;var n=Se[t]=Fe(function(t){return(t=Se.day(t)).setDate(t.getDate()-(t.getDay()+e)%7),t},function(t,e){t.setDate(t.getDate()+7*Math.floor(e))},function(t){var n=Se.year(t).getDay();return Math.floor((Se.dayOfYear(t)+(n+e)%7)/7)-(n!==e)});Se[t+"s"]=n.range,Se[t+"s"].utc=n.utc.range,Se[t+"OfYear"]=function(t){var n=Se.year(t).getDay();return Math.floor((Se.dayOfYear(t)+(n+e)%7)/7)}}),Se.week=Se.sunday,Se.weeks=Se.sunday.range,Se.weeks.utc=Se.sunday.utc.range,Se.weekOfYear=Se.sundayOfYear;var Ue={"-":"",_:" ",0:"0"},Oe=/^\s*\d+/,Ge=/^%/;function He(t,e,n){var r=t<0?"-":"",i=(r?-t:t)+"",o=i.length;return r+(o<n?new Array(n-o+1).join(e)+i:i)}function je(e){return new RegExp("^(?:"+e.map(t.requote).join("|")+")","i")}function Ye(t){for(var e=new y,n=-1,r=t.length;++n<r;)e.set(t[n].toLowerCase(),n);return e}function Pe(t,e,n){Oe.lastIndex=0;var r=Oe.exec(e.slice(n,n+1));return r?(t.w=+r[0],n+r[0].length):-1}function Ke(t,e,n){Oe.lastIndex=0;var r=Oe.exec(e.slice(n));return r?(t.U=+r[0],n+r[0].length):-1}function ze(t,e,n){Oe.lastIndex=0;var r=Oe.exec(e.slice(n));return r?(t.W=+r[0],n+r[0].length):-1}function Je(t,e,n){Oe.lastIndex=0;var r=Oe.exec(e.slice(n,n+4));return r?(t.y=+r[0],n+r[0].length):-1}function qe(t,e,n){Oe.lastIndex=0;var r,i=Oe.exec(e.slice(n,n+2));return i?(t.y=(r=+i[0])+(r>68?1900:2e3),n+i[0].length):-1}function Ve(t,e,n){return/^[+-]\d{4}$/.test(e=e.slice(n,n+5))?(t.Z=-e,n+5):-1}function We(t,e,n){Oe.lastIndex=0;var r=Oe.exec(e.slice(n,n+2));return r?(t.m=r[0]-1,n+r[0].length):-1}function Xe(t,e,n){Oe.lastIndex=0;var r=Oe.exec(e.slice(n,n+2));return r?(t.d=+r[0],n+r[0].length):-1}function Ze(t,e,n){Oe.lastIndex=0;var r=Oe.exec(e.slice(n,n+3));return r?(t.j=+r[0],n+r[0].length):-1}function $e(t,e,n){Oe.lastIndex=0;var r=Oe.exec(e.slice(n,n+2));return r?(t.H=+r[0],n+r[0].length):-1}function tn(t,e,n){Oe.lastIndex=0;var r=Oe.exec(e.slice(n,n+2));return r?(t.M=+r[0],n+r[0].length):-1}function en(t,e,n){Oe.lastIndex=0;var r=Oe.exec(e.slice(n,n+2));return r?(t.S=+r[0],n+r[0].length):-1}function nn(t,e,n){Oe.lastIndex=0;var r=Oe.exec(e.slice(n,n+3));return r?(t.L=+r[0],n+r[0].length):-1}function rn(t){var e=t.getTimezoneOffset(),n=e>0?"-":"+",r=C(e)/60|0,i=C(e)%60;return n+He(r,"0",2)+He(i,"0",2)}function on(t,e,n){Ge.lastIndex=0;var r=Ge.exec(e.slice(n,n+1));return r?n+r[0].length:-1}function an(t){for(var e=t.length,n=-1;++n<e;)t[n][0]=this(t[n][0]);return function(e){for(var n=0,r=t[n];!r[1](e);)r=t[++n];return r[0](e)}}t.locale=function(e){return{numberFormat:function(e){var n=e.decimal,r=e.thousands,i=e.grouping,o=e.currency,a=i&&r?function(t,e){for(var n=t.length,o=[],a=0,s=i[0],A=0;n>0&&s>0&&(A+s+1>e&&(s=Math.max(1,e-A)),o.push(t.substring(n-=s,n+s)),!((A+=s+1)>e));)s=i[a=(a+1)%i.length];return o.reverse().join(r)}:k;return function(e){var r=De.exec(e),i=r[1]||" ",s=r[2]||">",A=r[3]||"-",l=r[4]||"",c=r[5],u=+r[6],g=r[7],h=r[8],d=r[9],f=1,p="",I="",C=!1,m=!0;switch(h&&(h=+h.substring(1)),(c||"0"===i&&"="===s)&&(c=i="0",s="="),d){case"n":g=!0,d="g";break;case"%":f=100,I="%",d="f";break;case"p":f=100,I="%",d="r";break;case"b":case"o":case"x":case"X":"#"===l&&(p="0"+d.toLowerCase());case"c":m=!1;case"d":C=!0,h=0;break;case"s":f=-1,d="r"}"$"===l&&(p=o[0],I=o[1]),"r"!=d||h||(d="g"),null!=h&&("g"==d?h=Math.max(1,Math.min(21,h)):"e"!=d&&"f"!=d||(h=Math.max(0,Math.min(20,h)))),d=Me.get(d)||ke;var y=c&&g;return function(e){var r=I;if(C&&e%1)return"";var o=e<0||0===e&&1/e<0?(e=-e,"-"):"-"===A?"":A;if(f<0){var l=t.formatPrefix(e,h);e=l.scale(e),r=l.symbol+I}else e*=f;var E,v,B=(e=d(e,h)).lastIndexOf(".");if(B<0){var b=m?e.lastIndexOf("e"):-1;b<0?(E=e,v=""):(E=e.substring(0,b),v=e.substring(b))}else E=e.substring(0,B),v=n+e.substring(B+1);!c&&g&&(E=a(E,1/0));var Q=p.length+E.length+v.length+(y?0:o.length),w=Q<u?new Array(Q=u-Q+1).join(i):"";return y&&(E=a(w+E,w.length?u-v.length:1/0)),o+=p,e=E+v,("<"===s?o+e+w:">"===s?w+o+e:"^"===s?w.substring(0,Q>>=1)+o+e+w.substring(Q):o+(y?e:w+e))+r}}}(e),timeFormat:function(e){var n=e.dateTime,r=e.date,i=e.time,o=e.periods,a=e.days,s=e.shortDays,A=e.months,l=e.shortMonths;function c(t){var e=t.length;function n(n){for(var r,i,o,a=[],s=-1,A=0;++s<e;)37===t.charCodeAt(s)&&(a.push(t.slice(A,s)),null!=(i=Ue[r=t.charAt(++s)])&&(r=t.charAt(++s)),(o=E[r])&&(r=o(n,null==i?"e"===r?" ":"0":i)),a.push(r),A=s+1);return a.push(t.slice(A,s)),a.join("")}return n.parse=function(e){var n={y:1900,m:0,d:1,H:0,M:0,S:0,L:0,Z:null};if(u(n,t,e,0)!=e.length)return null;"p"in n&&(n.H=n.H%12+12*n.p);var r=null!=n.Z&&Le!==Re,i=new(r?Re:Le);return"j"in n?i.setFullYear(n.y,0,n.j):"W"in n||"U"in n?("w"in n||(n.w="W"in n?1:0),i.setFullYear(n.y,0,1),i.setFullYear(n.y,0,"W"in n?(n.w+6)%7+7*n.W-(i.getDay()+5)%7:n.w+7*n.U-(i.getDay()+6)%7)):i.setFullYear(n.y,n.m,n.d),i.setHours(n.H+(n.Z/100|0),n.M+n.Z%100,n.S,n.L),r?i._:i},n.toString=function(){return t},n}function u(t,e,n,r){for(var i,o,a,s=0,A=e.length,l=n.length;s<A;){if(r>=l)return-1;if(37===(i=e.charCodeAt(s++))){if(a=e.charAt(s++),!(o=v[a in Ue?e.charAt(s++):a])||(r=o(t,n,r))<0)return-1}else if(i!=n.charCodeAt(r++))return-1}return r}c.utc=function(t){var e=c(t);function n(t){try{var n=new(Le=Re);return n._=t,e(n)}finally{Le=Date}}return n.parse=function(t){try{Le=Re;var n=e.parse(t);return n&&n._}finally{Le=Date}},n.toString=e.toString,n},c.multi=c.utc.multi=an;var g=t.map(),h=je(a),d=Ye(a),f=je(s),p=Ye(s),I=je(A),C=Ye(A),m=je(l),y=Ye(l);o.forEach(function(t,e){g.set(t.toLowerCase(),e)});var E={a:function(t){return s[t.getDay()]},A:function(t){return a[t.getDay()]},b:function(t){return l[t.getMonth()]},B:function(t){return A[t.getMonth()]},c:c(n),d:function(t,e){return He(t.getDate(),e,2)},e:function(t,e){return He(t.getDate(),e,2)},H:function(t,e){return He(t.getHours(),e,2)},I:function(t,e){return He(t.getHours()%12||12,e,2)},j:function(t,e){return He(1+Se.dayOfYear(t),e,3)},L:function(t,e){return He(t.getMilliseconds(),e,3)},m:function(t,e){return He(t.getMonth()+1,e,2)},M:function(t,e){return He(t.getMinutes(),e,2)},p:function(t){return o[+(t.getHours()>=12)]},S:function(t,e){return He(t.getSeconds(),e,2)},U:function(t,e){return He(Se.sundayOfYear(t),e,2)},w:function(t){return t.getDay()},W:function(t,e){return He(Se.mondayOfYear(t),e,2)},x:c(r),X:c(i),y:function(t,e){return He(t.getFullYear()%100,e,2)},Y:function(t,e){return He(t.getFullYear()%1e4,e,4)},Z:rn,"%":function(){return"%"}},v={a:function(t,e,n){f.lastIndex=0;var r=f.exec(e.slice(n));return r?(t.w=p.get(r[0].toLowerCase()),n+r[0].length):-1},A:function(t,e,n){h.lastIndex=0;var r=h.exec(e.slice(n));return r?(t.w=d.get(r[0].toLowerCase()),n+r[0].length):-1},b:function(t,e,n){m.lastIndex=0;var r=m.exec(e.slice(n));return r?(t.m=y.get(r[0].toLowerCase()),n+r[0].length):-1},B:function(t,e,n){I.lastIndex=0;var r=I.exec(e.slice(n));return r?(t.m=C.get(r[0].toLowerCase()),n+r[0].length):-1},c:function(t,e,n){return u(t,E.c.toString(),e,n)},d:Xe,e:Xe,H:$e,I:$e,j:Ze,L:nn,m:We,M:tn,p:function(t,e,n){var r=g.get(e.slice(n,n+=2).toLowerCase());return null==r?-1:(t.p=r,n)},S:en,U:Ke,w:Pe,W:ze,x:function(t,e,n){return u(t,E.x.toString(),e,n)},X:function(t,e,n){return u(t,E.X.toString(),e,n)},y:qe,Y:Je,Z:Ve,"%":on};return c}(e)}};var sn=t.locale({decimal:".",thousands:",",grouping:[3],currency:["$",""],dateTime:"%a %b %e %X %Y",date:"%m/%d/%Y",time:"%H:%M:%S",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]});function An(){}t.format=sn.numberFormat,t.geo={},An.prototype={s:0,t:0,add:function(t){cn(t,this.t,ln),cn(ln.s,this.s,this),this.s?this.t+=ln.t:this.s=ln.t},reset:function(){this.s=this.t=0},valueOf:function(){return this.s}};var ln=new An;function cn(t,e,n){var r=n.s=t+e,i=r-t,o=r-i;n.t=t-o+(e-i)}function un(t,e){t&&hn.hasOwnProperty(t.type)&&hn[t.type](t,e)}t.geo.stream=function(t,e){t&&gn.hasOwnProperty(t.type)?gn[t.type](t,e):un(t,e)};var gn={Feature:function(t,e){un(t.geometry,e)},FeatureCollection:function(t,e){for(var n=t.features,r=-1,i=n.length;++r<i;)un(n[r].geometry,e)}},hn={Sphere:function(t,e){e.sphere()},Point:function(t,e){t=t.coordinates,e.point(t[0],t[1],t[2])},MultiPoint:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)t=n[r],e.point(t[0],t[1],t[2])},LineString:function(t,e){dn(t.coordinates,e,0)},MultiLineString:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)dn(n[r],e,0)},Polygon:function(t,e){fn(t.coordinates,e)},MultiPolygon:function(t,e){for(var n=t.coordinates,r=-1,i=n.length;++r<i;)fn(n[r],e)},GeometryCollection:function(t,e){for(var n=t.geometries,r=-1,i=n.length;++r<i;)un(n[r],e)}};function dn(t,e,n){var r,i=-1,o=t.length-n;for(e.lineStart();++i<o;)r=t[i],e.point(r[0],r[1],r[2]);e.lineEnd()}function fn(t,e){var n=-1,r=t.length;for(e.polygonStart();++n<r;)dn(t[n],e,1);e.polygonEnd()}t.geo.area=function(e){return pn=0,t.geo.stream(e,Dn),pn};var pn,In,Cn,mn,yn,En,vn,Bn,bn,Qn,wn,xn,_n=new An,Dn={sphere:function(){pn+=4*Qt},point:N,lineStart:N,lineEnd:N,polygonStart:function(){_n.reset(),Dn.lineStart=Mn},polygonEnd:function(){var t=2*_n;pn+=t<0?4*Qt+t:t,Dn.lineStart=Dn.lineEnd=Dn.point=N}};function Mn(){var t,e,n,r,i;function o(t,e){e=e*Dt/2+Qt/4;var o=(t*=Dt)-n,a=o>=0?1:-1,s=a*o,A=Math.cos(e),l=Math.sin(e),c=i*l,u=r*A+c*Math.cos(s),g=c*a*Math.sin(s);_n.add(Math.atan2(g,u)),n=t,r=A,i=l}Dn.point=function(a,s){Dn.point=o,n=(t=a)*Dt,r=Math.cos(s=(e=s)*Dt/2+Qt/4),i=Math.sin(s)},Dn.lineEnd=function(){o(t,e)}}function kn(t){var e=t[0],n=t[1],r=Math.cos(n);return[r*Math.cos(e),r*Math.sin(e),Math.sin(n)]}function Sn(t,e){return t[0]*e[0]+t[1]*e[1]+t[2]*e[2]}function Ln(t,e){return[t[1]*e[2]-t[2]*e[1],t[2]*e[0]-t[0]*e[2],t[0]*e[1]-t[1]*e[0]]}function Rn(t,e){t[0]+=e[0],t[1]+=e[1],t[2]+=e[2]}function Nn(t,e){return[t[0]*e,t[1]*e,t[2]*e]}function Fn(t){var e=Math.sqrt(t[0]*t[0]+t[1]*t[1]+t[2]*t[2]);t[0]/=e,t[1]/=e,t[2]/=e}function Tn(t){return[Math.atan2(t[1],t[0]),Rt(t[2])]}function Un(t,e){return C(t[0]-e[0])<Bt&&C(t[1]-e[1])<Bt}t.geo.bounds=function(){var e,n,r,i,o,a,s,A,l,c,u,g={point:h,lineStart:f,lineEnd:p,polygonStart:function(){g.point=I,g.lineStart=m,g.lineEnd=y,l=0,Dn.polygonStart()},polygonEnd:function(){Dn.polygonEnd(),g.point=h,g.lineStart=f,g.lineEnd=p,_n<0?(e=-(r=180),n=-(i=90)):l>Bt?i=90:l<-Bt&&(n=-90),u[0]=e,u[1]=r}};function h(t,o){c.push(u=[e=t,r=t]),o<n&&(n=o),o>i&&(i=o)}function d(t,a){var s=kn([t*Dt,a*Dt]);if(A){var l=Ln(A,s),c=Ln([l[1],-l[0],0],l);Fn(c),c=Tn(c);var u=t-o,g=u>0?1:-1,d=c[0]*Mt*g,f=C(u)>180;if(f^(g*o<d&&d<g*t))(p=c[1]*Mt)>i&&(i=p);else if(f^(g*o<(d=(d+360)%360-180)&&d<g*t)){var p;(p=-c[1]*Mt)<n&&(n=p)}else a<n&&(n=a),a>i&&(i=a);f?t<o?E(e,t)>E(e,r)&&(r=t):E(t,r)>E(e,r)&&(e=t):r>=e?(t<e&&(e=t),t>r&&(r=t)):t>o?E(e,t)>E(e,r)&&(r=t):E(t,r)>E(e,r)&&(e=t)}else h(t,a);A=s,o=t}function f(){g.point=d}function p(){u[0]=e,u[1]=r,g.point=h,A=null}function I(t,e){if(A){var n=t-o;l+=C(n)>180?n+(n>0?360:-360):n}else a=t,s=e;Dn.point(t,e),d(t,e)}function m(){Dn.lineStart()}function y(){I(a,s),Dn.lineEnd(),C(l)>Bt&&(e=-(r=180)),u[0]=e,u[1]=r,A=null}function E(t,e){return(e-=t)<0?e+360:e}function v(t,e){return t[0]-e[0]}function B(t,e){return e[0]<=e[1]?e[0]<=t&&t<=e[1]:t<e[0]||e[1]<t}return function(o){if(i=r=-(e=n=1/0),c=[],t.geo.stream(o,g),l=c.length){c.sort(v);for(var a=1,s=[f=c[0]];a<l;++a)B((h=c[a])[0],f)||B(h[1],f)?(E(f[0],h[1])>E(f[0],f[1])&&(f[1]=h[1]),E(h[0],f[1])>E(f[0],f[1])&&(f[0]=h[0])):s.push(f=h);for(var A,l,h,d=-1/0,f=(a=0,s[l=s.length-1]);a<=l;f=h,++a)h=s[a],(A=E(f[1],h[0]))>d&&(d=A,e=h[0],r=f[1])}return c=u=null,e===1/0||n===1/0?[[NaN,NaN],[NaN,NaN]]:[[e,n],[r,i]]}}(),t.geo.centroid=function(e){In=Cn=mn=yn=En=vn=Bn=bn=Qn=wn=xn=0,t.geo.stream(e,On);var n=Qn,r=wn,i=xn,o=n*n+r*r+i*i;return o<bt&&(n=vn,r=Bn,i=bn,Cn<Bt&&(n=mn,r=yn,i=En),(o=n*n+r*r+i*i)<bt)?[NaN,NaN]:[Math.atan2(r,n)*Mt,Rt(i/Math.sqrt(o))*Mt]};var On={sphere:N,point:Gn,lineStart:jn,lineEnd:Yn,polygonStart:function(){On.lineStart=Pn},polygonEnd:function(){On.lineStart=jn}};function Gn(t,e){t*=Dt;var n=Math.cos(e*=Dt);Hn(n*Math.cos(t),n*Math.sin(t),Math.sin(e))}function Hn(t,e,n){mn+=(t-mn)/++In,yn+=(e-yn)/In,En+=(n-En)/In}function jn(){var t,e,n;function r(r,i){r*=Dt;var o=Math.cos(i*=Dt),a=o*Math.cos(r),s=o*Math.sin(r),A=Math.sin(i),l=Math.atan2(Math.sqrt((l=e*A-n*s)*l+(l=n*a-t*A)*l+(l=t*s-e*a)*l),t*a+e*s+n*A);Cn+=l,vn+=l*(t+(t=a)),Bn+=l*(e+(e=s)),bn+=l*(n+(n=A)),Hn(t,e,n)}On.point=function(i,o){i*=Dt;var a=Math.cos(o*=Dt);t=a*Math.cos(i),e=a*Math.sin(i),n=Math.sin(o),On.point=r,Hn(t,e,n)}}function Yn(){On.point=Gn}function Pn(){var t,e,n,r,i;function o(t,e){t*=Dt;var o=Math.cos(e*=Dt),a=o*Math.cos(t),s=o*Math.sin(t),A=Math.sin(e),l=r*A-i*s,c=i*a-n*A,u=n*s-r*a,g=Math.sqrt(l*l+c*c+u*u),h=n*a+r*s+i*A,d=g&&-Lt(h)/g,f=Math.atan2(g,h);Qn+=d*l,wn+=d*c,xn+=d*u,Cn+=f,vn+=f*(n+(n=a)),Bn+=f*(r+(r=s)),bn+=f*(i+(i=A)),Hn(n,r,i)}On.point=function(a,s){t=a,e=s,On.point=o,a*=Dt;var A=Math.cos(s*=Dt);n=A*Math.cos(a),r=A*Math.sin(a),i=Math.sin(s),Hn(n,r,i)},On.lineEnd=function(){o(t,e),On.lineEnd=Yn,On.point=Gn}}function Kn(t,e){function n(n,r){return n=t(n,r),e(n[0],n[1])}return t.invert&&e.invert&&(n.invert=function(n,r){return(n=e.invert(n,r))&&t.invert(n[0],n[1])}),n}function zn(){return!0}function Jn(t,e,n,r,i){var o=[],a=[];if(t.forEach(function(t){if(!((e=t.length-1)<=0)){var e,n=t[0],r=t[e];if(Un(n,r)){i.lineStart();for(var s=0;s<e;++s)i.point((n=t[s])[0],n[1]);i.lineEnd()}else{var A=new Vn(n,t,null,!0),l=new Vn(n,null,A,!1);A.o=l,o.push(A),a.push(l),A=new Vn(r,t,null,!1),l=new Vn(r,null,A,!0),A.o=l,o.push(A),a.push(l)}}}),a.sort(e),qn(o),qn(a),o.length){for(var s=0,A=n,l=a.length;s<l;++s)a[s].e=A=!A;for(var c,u,g=o[0];;){for(var h=g,d=!0;h.v;)if((h=h.n)===g)return;c=h.z,i.lineStart();do{if(h.v=h.o.v=!0,h.e){if(d)for(s=0,l=c.length;s<l;++s)i.point((u=c[s])[0],u[1]);else r(h.x,h.n.x,1,i);h=h.n}else{if(d)for(s=(c=h.p.z).length-1;s>=0;--s)i.point((u=c[s])[0],u[1]);else r(h.x,h.p.x,-1,i);h=h.p}c=(h=h.o).z,d=!d}while(!h.v);i.lineEnd()}}}function qn(t){if(e=t.length){for(var e,n,r=0,i=t[0];++r<e;)i.n=n=t[r],n.p=i,i=n;i.n=n=t[0],n.p=i}}function Vn(t,e,n,r){this.x=t,this.z=e,this.o=n,this.e=r,this.v=!1,this.n=this.p=null}function Wn(e,n,r,i){return function(o,a){var s,A=n(a),l=o.invert(i[0],i[1]),c={point:u,lineStart:h,lineEnd:d,polygonStart:function(){c.point=y,c.lineStart=E,c.lineEnd=v,s=[],f=[]},polygonEnd:function(){c.point=u,c.lineStart=h,c.lineEnd=d,s=t.merge(s);var e=function(t,e){var n=t[0],r=t[1],i=[Math.sin(n),-Math.cos(n),0],o=0,a=0;_n.reset();for(var s=0,A=e.length;s<A;++s){var l=e[s],c=l.length;if(c)for(var u=l[0],g=u[0],h=u[1]/2+Qt/4,d=Math.sin(h),f=Math.cos(h),p=1;;){p===c&&(p=0);var I=(t=l[p])[0],C=t[1]/2+Qt/4,m=Math.sin(C),y=Math.cos(C),E=I-g,v=E>=0?1:-1,B=v*E,b=B>Qt,Q=d*m;if(_n.add(Math.atan2(Q*v*Math.sin(B),f*y+Q*Math.cos(B))),o+=b?E+v*wt:E,b^g>=n^I>=n){var w=Ln(kn(u),kn(t));Fn(w);var x=Ln(i,w);Fn(x);var _=(b^E>=0?-1:1)*Rt(x[2]);(r>_||r===_&&(w[0]||w[1]))&&(a+=b^E>=0?1:-1)}if(!p++)break;g=I,d=m,f=y,u=t}}return(o<-Bt||o<Bt&&_n<-Bt)^1&a}(l,f);s.length?(m||(a.polygonStart(),m=!0),Jn(s,$n,e,r,a)):e&&(m||(a.polygonStart(),m=!0),a.lineStart(),r(null,null,1,a),a.lineEnd()),m&&(a.polygonEnd(),m=!1),s=f=null},sphere:function(){a.polygonStart(),a.lineStart(),r(null,null,1,a),a.lineEnd(),a.polygonEnd()}};function u(t,n){var r=o(t,n);e(t=r[0],n=r[1])&&a.point(t,n)}function g(t,e){var n=o(t,e);A.point(n[0],n[1])}function h(){c.point=g,A.lineStart()}function d(){c.point=u,A.lineEnd()}var f,p,I=Zn(),C=n(I),m=!1;function y(t,e){p.push([t,e]);var n=o(t,e);C.point(n[0],n[1])}function E(){C.lineStart(),p=[]}function v(){y(p[0][0],p[0][1]),C.lineEnd();var t,e=C.clean(),n=I.buffer(),r=n.length;if(p.pop(),f.push(p),p=null,r)if(1&e){var i,o=-1;if((r=(t=n[0]).length-1)>0){for(m||(a.polygonStart(),m=!0),a.lineStart();++o<r;)a.point((i=t[o])[0],i[1]);a.lineEnd()}}else r>1&&2&e&&n.push(n.pop().concat(n.shift())),s.push(n.filter(Xn))}return c}}function Xn(t){return t.length>1}function Zn(){var t,e=[];return{lineStart:function(){e.push(t=[])},point:function(e,n){t.push([e,n])},lineEnd:N,buffer:function(){var n=e;return e=[],t=null,n},rejoin:function(){e.length>1&&e.push(e.pop().concat(e.shift()))}}}function $n(t,e){return((t=t.x)[0]<0?t[1]-_t-Bt:_t-t[1])-((e=e.x)[0]<0?e[1]-_t-Bt:_t-e[1])}var tr=Wn(zn,function(t){var e,n=NaN,r=NaN,i=NaN;return{lineStart:function(){t.lineStart(),e=1},point:function(o,a){var s=o>0?Qt:-Qt,A=C(o-n);C(A-Qt)<Bt?(t.point(n,r=(r+a)/2>0?_t:-_t),t.point(i,r),t.lineEnd(),t.lineStart(),t.point(s,r),t.point(o,r),e=0):i!==s&&A>=Qt&&(C(n-i)<Bt&&(n-=i*Bt),C(o-s)<Bt&&(o-=s*Bt),r=function(t,e,n,r){var i,o,a=Math.sin(t-n);return C(a)>Bt?Math.atan((Math.sin(e)*(o=Math.cos(r))*Math.sin(n)-Math.sin(r)*(i=Math.cos(e))*Math.sin(t))/(i*o*a)):(e+r)/2}(n,r,o,a),t.point(i,r),t.lineEnd(),t.lineStart(),t.point(s,r),e=0),t.point(n=o,r=a),i=s},lineEnd:function(){t.lineEnd(),n=r=NaN},clean:function(){return 2-e}}},function(t,e,n,r){var i;if(null==t)i=n*_t,r.point(-Qt,i),r.point(0,i),r.point(Qt,i),r.point(Qt,0),r.point(Qt,-i),r.point(0,-i),r.point(-Qt,-i),r.point(-Qt,0),r.point(-Qt,i);else if(C(t[0]-e[0])>Bt){var o=t[0]<e[0]?Qt:-Qt;i=n*o/2,r.point(-o,i),r.point(0,i),r.point(o,i)}else r.point(e[0],e[1])},[-Qt,-Qt/2]);function er(t,e,n,r){return function(i){var o,a=i.a,s=i.b,A=a.x,l=a.y,c=0,u=1,g=s.x-A,h=s.y-l;if(o=t-A,g||!(o>0)){if(o/=g,g<0){if(o<c)return;o<u&&(u=o)}else if(g>0){if(o>u)return;o>c&&(c=o)}if(o=n-A,g||!(o<0)){if(o/=g,g<0){if(o>u)return;o>c&&(c=o)}else if(g>0){if(o<c)return;o<u&&(u=o)}if(o=e-l,h||!(o>0)){if(o/=h,h<0){if(o<c)return;o<u&&(u=o)}else if(h>0){if(o>u)return;o>c&&(c=o)}if(o=r-l,h||!(o<0)){if(o/=h,h<0){if(o>u)return;o>c&&(c=o)}else if(h>0){if(o<c)return;o<u&&(u=o)}return c>0&&(i.a={x:A+c*g,y:l+c*h}),u<1&&(i.b={x:A+u*g,y:l+u*h}),i}}}}}}var nr=1e9;function rr(e,n,r,i){return function(A){var l,c,u,g,h,d,f,p,I,C,m,y=A,E=Zn(),v=er(e,n,r,i),B={point:w,lineStart:function(){B.point=x,c&&c.push(u=[]),C=!0,I=!1,f=p=NaN},lineEnd:function(){l&&(x(g,h),d&&I&&E.rejoin(),l.push(E.buffer())),B.point=w,I&&A.lineEnd()},polygonStart:function(){A=E,l=[],c=[],m=!0},polygonEnd:function(){A=y,l=t.merge(l);var n=function(t){for(var e=0,n=c.length,r=t[1],i=0;i<n;++i)for(var o,a=1,s=c[i],A=s.length,l=s[0];a<A;++a)o=s[a],l[1]<=r?o[1]>r&&St(l,o,t)>0&&++e:o[1]<=r&&St(l,o,t)<0&&--e,l=o;return 0!==e}([e,i]),r=m&&n,o=l.length;(r||o)&&(A.polygonStart(),r&&(A.lineStart(),b(null,null,1,A),A.lineEnd()),o&&Jn(l,a,n,b,A),A.polygonEnd()),l=c=u=null}};function b(t,a,A,l){var c=0,u=0;if(null==t||(c=o(t,A))!==(u=o(a,A))||s(t,a)<0^A>0)do{l.point(0===c||3===c?e:r,c>1?i:n)}while((c=(c+A+4)%4)!==u);else l.point(a[0],a[1])}function Q(t,o){return e<=t&&t<=r&&n<=o&&o<=i}function w(t,e){Q(t,e)&&A.point(t,e)}function x(t,e){var n=Q(t=Math.max(-nr,Math.min(nr,t)),e=Math.max(-nr,Math.min(nr,e)));if(c&&u.push([t,e]),C)g=t,h=e,d=n,C=!1,n&&(A.lineStart(),A.point(t,e));else if(n&&I)A.point(t,e);else{var r={a:{x:f,y:p},b:{x:t,y:e}};v(r)?(I||(A.lineStart(),A.point(r.a.x,r.a.y)),A.point(r.b.x,r.b.y),n||A.lineEnd(),m=!1):n&&(A.lineStart(),A.point(t,e),m=!1)}f=t,p=e,I=n}return B};function o(t,i){return C(t[0]-e)<Bt?i>0?0:3:C(t[0]-r)<Bt?i>0?2:1:C(t[1]-n)<Bt?i>0?1:0:i>0?3:2}function a(t,e){return s(t.x,e.x)}function s(t,e){var n=o(t,1),r=o(e,1);return n!==r?n-r:0===n?e[1]-t[1]:1===n?t[0]-e[0]:2===n?t[1]-e[1]:e[0]-t[0]}}function ir(t){var e=0,n=Qt/3,r=_r(t),i=r(e,n);return i.parallels=function(t){return arguments.length?r(e=t[0]*Qt/180,n=t[1]*Qt/180):[e/Qt*180,n/Qt*180]},i}function or(t,e){var n=Math.sin(t),r=(n+Math.sin(e))/2,i=1+n*(2*r-n),o=Math.sqrt(i)/r;function a(t,e){var n=Math.sqrt(i-2*r*Math.sin(e))/r;return[n*Math.sin(t*=r),o-n*Math.cos(t)]}return a.invert=function(t,e){var n=o-e;return[Math.atan2(t,n)/r,Rt((i-(t*t+n*n)*r*r)/(2*r))]},a}t.geo.clipExtent=function(){var t,e,n,r,i,o,a={stream:function(t){return i&&(i.valid=!1),(i=o(t)).valid=!0,i},extent:function(s){return arguments.length?(o=rr(t=+s[0][0],e=+s[0][1],n=+s[1][0],r=+s[1][1]),i&&(i.valid=!1,i=null),a):[[t,e],[n,r]]}};return a.extent([[0,0],[960,500]])},(t.geo.conicEqualArea=function(){return ir(or)}).raw=or,t.geo.albers=function(){return t.geo.conicEqualArea().rotate([96,0]).center([-.6,38.7]).parallels([29.5,45.5]).scale(1070)},t.geo.albersUsa=function(){var e,n,r,i,o=t.geo.albers(),a=t.geo.conicEqualArea().rotate([154,0]).center([-2,58.5]).parallels([55,65]),s=t.geo.conicEqualArea().rotate([157,0]).center([-3,19.9]).parallels([8,18]),A={point:function(t,n){e=[t,n]}};function l(t){var o=t[0],a=t[1];return e=null,n(o,a),e||(r(o,a),e)||i(o,a),e}return l.invert=function(t){var e=o.scale(),n=o.translate(),r=(t[0]-n[0])/e,i=(t[1]-n[1])/e;return(i>=.12&&i<.234&&r>=-.425&&r<-.214?a:i>=.166&&i<.234&&r>=-.214&&r<-.115?s:o).invert(t)},l.stream=function(t){var e=o.stream(t),n=a.stream(t),r=s.stream(t);return{point:function(t,i){e.point(t,i),n.point(t,i),r.point(t,i)},sphere:function(){e.sphere(),n.sphere(),r.sphere()},lineStart:function(){e.lineStart(),n.lineStart(),r.lineStart()},lineEnd:function(){e.lineEnd(),n.lineEnd(),r.lineEnd()},polygonStart:function(){e.polygonStart(),n.polygonStart(),r.polygonStart()},polygonEnd:function(){e.polygonEnd(),n.polygonEnd(),r.polygonEnd()}}},l.precision=function(t){return arguments.length?(o.precision(t),a.precision(t),s.precision(t),l):o.precision()},l.scale=function(t){return arguments.length?(o.scale(t),a.scale(.35*t),s.scale(t),l.translate(o.translate())):o.scale()},l.translate=function(t){if(!arguments.length)return o.translate();var e=o.scale(),c=+t[0],u=+t[1];return n=o.translate(t).clipExtent([[c-.455*e,u-.238*e],[c+.455*e,u+.238*e]]).stream(A).point,r=a.translate([c-.307*e,u+.201*e]).clipExtent([[c-.425*e+Bt,u+.12*e+Bt],[c-.214*e-Bt,u+.234*e-Bt]]).stream(A).point,i=s.translate([c-.205*e,u+.212*e]).clipExtent([[c-.214*e+Bt,u+.166*e+Bt],[c-.115*e-Bt,u+.234*e-Bt]]).stream(A).point,l},l.scale(1070)};var ar,sr,Ar,lr,cr,ur,gr={point:N,lineStart:N,lineEnd:N,polygonStart:function(){sr=0,gr.lineStart=hr},polygonEnd:function(){gr.lineStart=gr.lineEnd=gr.point=N,ar+=C(sr/2)}};function hr(){var t,e,n,r;function i(t,e){sr+=r*t-n*e,n=t,r=e}gr.point=function(o,a){gr.point=i,t=n=o,e=r=a},gr.lineEnd=function(){i(t,e)}}var dr={point:function(t,e){t<Ar&&(Ar=t),t>cr&&(cr=t),e<lr&&(lr=e),e>ur&&(ur=e)},lineStart:N,lineEnd:N,polygonStart:N,polygonEnd:N};function fr(){var t=pr(4.5),e=[],n={point:r,lineStart:function(){n.point=i},lineEnd:a,polygonStart:function(){n.lineEnd=s},polygonEnd:function(){n.lineEnd=a,n.point=r},pointRadius:function(e){return t=pr(e),n},result:function(){if(e.length){var t=e.join("");return e=[],t}}};function r(n,r){e.push("M",n,",",r,t)}function i(t,r){e.push("M",t,",",r),n.point=o}function o(t,n){e.push("L",t,",",n)}function a(){n.point=r}function s(){e.push("Z")}return n}function pr(t){return"m0,"+t+"a"+t+","+t+" 0 1,1 0,"+-2*t+"a"+t+","+t+" 0 1,1 0,"+2*t+"z"}var Ir,Cr={point:mr,lineStart:yr,lineEnd:Er,polygonStart:function(){Cr.lineStart=vr},polygonEnd:function(){Cr.point=mr,Cr.lineStart=yr,Cr.lineEnd=Er}};function mr(t,e){mn+=t,yn+=e,++En}function yr(){var t,e;function n(n,r){var i=n-t,o=r-e,a=Math.sqrt(i*i+o*o);vn+=a*(t+n)/2,Bn+=a*(e+r)/2,bn+=a,mr(t=n,e=r)}Cr.point=function(r,i){Cr.point=n,mr(t=r,e=i)}}function Er(){Cr.point=mr}function vr(){var t,e,n,r;function i(t,e){var i=t-n,o=e-r,a=Math.sqrt(i*i+o*o);vn+=a*(n+t)/2,Bn+=a*(r+e)/2,bn+=a,Qn+=(a=r*t-n*e)*(n+t),wn+=a*(r+e),xn+=3*a,mr(n=t,r=e)}Cr.point=function(o,a){Cr.point=i,mr(t=n=o,e=r=a)},Cr.lineEnd=function(){i(t,e)}}function Br(t){var e=4.5,n={point:r,lineStart:function(){n.point=i},lineEnd:a,polygonStart:function(){n.lineEnd=s},polygonEnd:function(){n.lineEnd=a,n.point=r},pointRadius:function(t){return e=t,n},result:N};function r(n,r){t.moveTo(n+e,r),t.arc(n,r,e,0,wt)}function i(e,r){t.moveTo(e,r),n.point=o}function o(e,n){t.lineTo(e,n)}function a(){n.point=r}function s(){t.closePath()}return n}function br(t){var e=.5,n=Math.cos(30*Dt),r=16;function i(e){return(r?function(e){var n,i,a,s,A,l,c,u,g,h,d,f,p={point:I,lineStart:C,lineEnd:y,polygonStart:function(){e.polygonStart(),p.lineStart=E},polygonEnd:function(){e.polygonEnd(),p.lineStart=C}};function I(n,r){n=t(n,r),e.point(n[0],n[1])}function C(){u=NaN,p.point=m,e.lineStart()}function m(n,i){var a=kn([n,i]),s=t(n,i);o(u,g,c,h,d,f,u=s[0],g=s[1],c=n,h=a[0],d=a[1],f=a[2],r,e),e.point(u,g)}function y(){p.point=I,e.lineEnd()}function E(){C(),p.point=v,p.lineEnd=B}function v(t,e){m(n=t,e),i=u,a=g,s=h,A=d,l=f,p.point=m}function B(){o(u,g,c,h,d,f,i,a,n,s,A,l,r,e),p.lineEnd=y,y()}return p}:function(e){return wr(e,function(n,r){n=t(n,r),e.point(n[0],n[1])})})(e)}function o(r,i,a,s,A,l,c,u,g,h,d,f,p,I){var m=c-r,y=u-i,E=m*m+y*y;if(E>4*e&&p--){var v=s+h,B=A+d,b=l+f,Q=Math.sqrt(v*v+B*B+b*b),w=Math.asin(b/=Q),x=C(C(b)-1)<Bt||C(a-g)<Bt?(a+g)/2:Math.atan2(B,v),_=t(x,w),D=_[0],M=_[1],k=D-r,S=M-i,L=y*k-m*S;(L*L/E>e||C((m*k+y*S)/E-.5)>.3||s*h+A*d+l*f<n)&&(o(r,i,a,s,A,l,D,M,x,v/=Q,B/=Q,b,p,I),I.point(D,M),o(D,M,x,v,B,b,c,u,g,h,d,f,p,I))}}return i.precision=function(t){return arguments.length?(r=(e=t*t)>0&&16,i):Math.sqrt(e)},i}function Qr(t){this.stream=t}function wr(t,e){return{point:e,sphere:function(){t.sphere()},lineStart:function(){t.lineStart()},lineEnd:function(){t.lineEnd()},polygonStart:function(){t.polygonStart()},polygonEnd:function(){t.polygonEnd()}}}function xr(t){return _r(function(){return t})()}function _r(e){var n,r,i,o,a,s,A=br(function(t,e){return[(t=n(t,e))[0]*l+o,a-t[1]*l]}),l=150,c=480,u=250,g=0,h=0,d=0,f=0,p=0,I=tr,m=k,y=null,E=null;function v(t){return[(t=i(t[0]*Dt,t[1]*Dt))[0]*l+o,a-t[1]*l]}function B(t){return(t=i.invert((t[0]-o)/l,(a-t[1])/l))&&[t[0]*Mt,t[1]*Mt]}function b(){i=Kn(r=Sr(d,f,p),n);var t=n(g,h);return o=c-t[0]*l,a=u+t[1]*l,Q()}function Q(){return s&&(s.valid=!1,s=null),v}return v.stream=function(t){return s&&(s.valid=!1),(s=Dr(I(r,A(m(t))))).valid=!0,s},v.clipAngle=function(t){return arguments.length?(I=null==t?(y=t,tr):function(t){var e=Math.cos(t),n=e>0,r=C(e)>Bt;return Wn(i,function(t){var e,s,A,l,c;return{lineStart:function(){l=A=!1,c=1},point:function(u,g){var h,d=[u,g],f=i(u,g),p=n?f?0:a(u,g):f?a(u+(u<0?Qt:-Qt),g):0;if(!e&&(l=A=f)&&t.lineStart(),f!==A&&(h=o(e,d),(Un(e,h)||Un(d,h))&&(d[0]+=Bt,d[1]+=Bt,f=i(d[0],d[1]))),f!==A)c=0,f?(t.lineStart(),h=o(d,e),t.point(h[0],h[1])):(h=o(e,d),t.point(h[0],h[1]),t.lineEnd()),e=h;else if(r&&e&&n^f){var I;p&s||!(I=o(d,e,!0))||(c=0,n?(t.lineStart(),t.point(I[0][0],I[0][1]),t.point(I[1][0],I[1][1]),t.lineEnd()):(t.point(I[1][0],I[1][1]),t.lineEnd(),t.lineStart(),t.point(I[0][0],I[0][1])))}!f||e&&Un(e,d)||t.point(d[0],d[1]),e=d,A=f,s=p},lineEnd:function(){A&&t.lineEnd(),e=null},clean:function(){return c|(l&&A)<<1}}},Fr(t,6*Dt),n?[0,-t]:[-Qt,t-Qt]);function i(t,n){return Math.cos(t)*Math.cos(n)>e}function o(t,n,r){var i=[1,0,0],o=Ln(kn(t),kn(n)),a=Sn(o,o),s=o[0],A=a-s*s;if(!A)return!r&&t;var l=e*a/A,c=-e*s/A,u=Ln(i,o),g=Nn(i,l);Rn(g,Nn(o,c));var h=u,d=Sn(g,h),f=Sn(h,h),p=d*d-f*(Sn(g,g)-1);if(!(p<0)){var I=Math.sqrt(p),m=Nn(h,(-d-I)/f);if(Rn(m,g),m=Tn(m),!r)return m;var y,E=t[0],v=n[0],B=t[1],b=n[1];v<E&&(y=E,E=v,v=y);var Q=v-E,w=C(Q-Qt)<Bt;if(!w&&b<B&&(y=B,B=b,b=y),w||Q<Bt?w?B+b>0^m[1]<(C(m[0]-E)<Bt?B:b):B<=m[1]&&m[1]<=b:Q>Qt^(E<=m[0]&&m[0]<=v)){var x=Nn(h,(-d+I)/f);return Rn(x,g),[m,Tn(x)]}}}function a(e,r){var i=n?t:Qt-t,o=0;return e<-i?o|=1:e>i&&(o|=2),r<-i?o|=4:r>i&&(o|=8),o}}((y=+t)*Dt),Q()):y},v.clipExtent=function(t){return arguments.length?(E=t,m=t?rr(t[0][0],t[0][1],t[1][0],t[1][1]):k,Q()):E},v.scale=function(t){return arguments.length?(l=+t,b()):l},v.translate=function(t){return arguments.length?(c=+t[0],u=+t[1],b()):[c,u]},v.center=function(t){return arguments.length?(g=t[0]%360*Dt,h=t[1]%360*Dt,b()):[g*Mt,h*Mt]},v.rotate=function(t){return arguments.length?(d=t[0]%360*Dt,f=t[1]%360*Dt,p=t.length>2?t[2]%360*Dt:0,b()):[d*Mt,f*Mt,p*Mt]},t.rebind(v,A,"precision"),function(){return n=e.apply(this,arguments),v.invert=n.invert&&B,b()}}function Dr(t){return wr(t,function(e,n){t.point(e*Dt,n*Dt)})}function Mr(t,e){return[t,e]}function kr(t,e){return[t>Qt?t-wt:t<-Qt?t+wt:t,e]}function Sr(t,e,n){return t?e||n?Kn(Rr(t),Nr(e,n)):Rr(t):e||n?Nr(e,n):kr}function Lr(t){return function(e,n){return[(e+=t)>Qt?e-wt:e<-Qt?e+wt:e,n]}}function Rr(t){var e=Lr(t);return e.invert=Lr(-t),e}function Nr(t,e){var n=Math.cos(t),r=Math.sin(t),i=Math.cos(e),o=Math.sin(e);function a(t,e){var a=Math.cos(e),s=Math.cos(t)*a,A=Math.sin(t)*a,l=Math.sin(e),c=l*n+s*r;return[Math.atan2(A*i-c*o,s*n-l*r),Rt(c*i+A*o)]}return a.invert=function(t,e){var a=Math.cos(e),s=Math.cos(t)*a,A=Math.sin(t)*a,l=Math.sin(e),c=l*i-A*o;return[Math.atan2(A*i+l*o,s*n+c*r),Rt(c*n-s*r)]},a}function Fr(t,e){var n=Math.cos(t),r=Math.sin(t);return function(i,o,a,s){var A=a*e;null!=i?(i=Tr(n,i),o=Tr(n,o),(a>0?i<o:i>o)&&(i+=a*wt)):(i=t+a*wt,o=t-.5*A);for(var l,c=i;a>0?c>o:c<o;c-=A)s.point((l=Tn([n,-r*Math.cos(c),-r*Math.sin(c)]))[0],l[1])}}function Tr(t,e){var n=kn(e);n[0]-=t,Fn(n);var r=Lt(-n[1]);return((-n[2]<0?-r:r)+2*Math.PI-Bt)%(2*Math.PI)}function Ur(e,n,r){var i=t.range(e,n-Bt,r).concat(n);return function(t){return i.map(function(e){return[t,e]})}}function Or(e,n,r){var i=t.range(e,n-Bt,r).concat(n);return function(t){return i.map(function(e){return[e,t]})}}function Gr(t){return t.source}function Hr(t){return t.target}t.geo.path=function(){var e,n,r,i,o,a=4.5;function s(e){return e&&("function"==typeof a&&i.pointRadius(+a.apply(this,arguments)),o&&o.valid||(o=r(i)),t.geo.stream(e,o)),i.result()}function A(){return o=null,s}return s.area=function(e){return ar=0,t.geo.stream(e,r(gr)),ar},s.centroid=function(e){return mn=yn=En=vn=Bn=bn=Qn=wn=xn=0,t.geo.stream(e,r(Cr)),xn?[Qn/xn,wn/xn]:bn?[vn/bn,Bn/bn]:En?[mn/En,yn/En]:[NaN,NaN]},s.bounds=function(e){return cr=ur=-(Ar=lr=1/0),t.geo.stream(e,r(dr)),[[Ar,lr],[cr,ur]]},s.projection=function(t){return arguments.length?(r=(e=t)?t.stream||(n=t,i=br(function(t,e){return n([t*Mt,e*Mt])}),function(t){return Dr(i(t))}):k,A()):e;var n,i},s.context=function(t){return arguments.length?(i=null==(n=t)?new fr:new Br(t),"function"!=typeof a&&i.pointRadius(a),A()):n},s.pointRadius=function(t){return arguments.length?(a="function"==typeof t?t:(i.pointRadius(+t),+t),s):a},s.projection(t.geo.albersUsa()).context(null)},t.geo.transform=function(t){return{stream:function(e){var n=new Qr(e);for(var r in t)n[r]=t[r];return n}}},Qr.prototype={point:function(t,e){this.stream.point(t,e)},sphere:function(){this.stream.sphere()},lineStart:function(){this.stream.lineStart()},lineEnd:function(){this.stream.lineEnd()},polygonStart:function(){this.stream.polygonStart()},polygonEnd:function(){this.stream.polygonEnd()}},t.geo.projection=xr,t.geo.projectionMutator=_r,(t.geo.equirectangular=function(){return xr(Mr)}).raw=Mr.invert=Mr,t.geo.rotation=function(t){function e(e){return(e=t(e[0]*Dt,e[1]*Dt))[0]*=Mt,e[1]*=Mt,e}return t=Sr(t[0]%360*Dt,t[1]*Dt,t.length>2?t[2]*Dt:0),e.invert=function(e){return(e=t.invert(e[0]*Dt,e[1]*Dt))[0]*=Mt,e[1]*=Mt,e},e},kr.invert=Mr,t.geo.circle=function(){var t,e,n=[0,0],r=6;function i(){var t="function"==typeof n?n.apply(this,arguments):n,r=Sr(-t[0]*Dt,-t[1]*Dt,0).invert,i=[];return e(null,null,1,{point:function(t,e){i.push(t=r(t,e)),t[0]*=Mt,t[1]*=Mt}}),{type:"Polygon",coordinates:[i]}}return i.origin=function(t){return arguments.length?(n=t,i):n},i.angle=function(n){return arguments.length?(e=Fr((t=+n)*Dt,r*Dt),i):t},i.precision=function(n){return arguments.length?(e=Fr(t*Dt,(r=+n)*Dt),i):r},i.angle(90)},t.geo.distance=function(t,e){var n,r=(e[0]-t[0])*Dt,i=t[1]*Dt,o=e[1]*Dt,a=Math.sin(r),s=Math.cos(r),A=Math.sin(i),l=Math.cos(i),c=Math.sin(o),u=Math.cos(o);return Math.atan2(Math.sqrt((n=u*a)*n+(n=l*c-A*u*s)*n),A*c+l*u*s)},t.geo.graticule=function(){var e,n,r,i,o,a,s,A,l,c,u,g,h=10,d=h,f=90,p=360,I=2.5;function m(){return{type:"MultiLineString",coordinates:y()}}function y(){return t.range(Math.ceil(i/f)*f,r,f).map(u).concat(t.range(Math.ceil(A/p)*p,s,p).map(g)).concat(t.range(Math.ceil(n/h)*h,e,h).filter(function(t){return C(t%f)>Bt}).map(l)).concat(t.range(Math.ceil(a/d)*d,o,d).filter(function(t){return C(t%p)>Bt}).map(c))}return m.lines=function(){return y().map(function(t){return{type:"LineString",coordinates:t}})},m.outline=function(){return{type:"Polygon",coordinates:[u(i).concat(g(s).slice(1),u(r).reverse().slice(1),g(A).reverse().slice(1))]}},m.extent=function(t){return arguments.length?m.majorExtent(t).minorExtent(t):m.minorExtent()},m.majorExtent=function(t){return arguments.length?(i=+t[0][0],r=+t[1][0],A=+t[0][1],s=+t[1][1],i>r&&(t=i,i=r,r=t),A>s&&(t=A,A=s,s=t),m.precision(I)):[[i,A],[r,s]]},m.minorExtent=function(t){return arguments.length?(n=+t[0][0],e=+t[1][0],a=+t[0][1],o=+t[1][1],n>e&&(t=n,n=e,e=t),a>o&&(t=a,a=o,o=t),m.precision(I)):[[n,a],[e,o]]},m.step=function(t){return arguments.length?m.majorStep(t).minorStep(t):m.minorStep()},m.majorStep=function(t){return arguments.length?(f=+t[0],p=+t[1],m):[f,p]},m.minorStep=function(t){return arguments.length?(h=+t[0],d=+t[1],m):[h,d]},m.precision=function(t){return arguments.length?(I=+t,l=Ur(a,o,90),c=Or(n,e,I),u=Ur(A,s,90),g=Or(i,r,I),m):I},m.majorExtent([[-180,-90+Bt],[180,90-Bt]]).minorExtent([[-180,-80-Bt],[180,80+Bt]])},t.geo.greatArc=function(){var e,n,r=Gr,i=Hr;function o(){return{type:"LineString",coordinates:[e||r.apply(this,arguments),n||i.apply(this,arguments)]}}return o.distance=function(){return t.geo.distance(e||r.apply(this,arguments),n||i.apply(this,arguments))},o.source=function(t){return arguments.length?(r=t,e="function"==typeof t?null:t,o):r},o.target=function(t){return arguments.length?(i=t,n="function"==typeof t?null:t,o):i},o.precision=function(){return arguments.length?o:0},o},t.geo.interpolate=function(t,e){return n=t[0]*Dt,r=t[1]*Dt,i=e[0]*Dt,o=e[1]*Dt,a=Math.cos(r),s=Math.sin(r),A=Math.cos(o),l=Math.sin(o),c=a*Math.cos(n),u=a*Math.sin(n),g=A*Math.cos(i),h=A*Math.sin(i),d=2*Math.asin(Math.sqrt(Ft(o-r)+a*A*Ft(i-n))),f=1/Math.sin(d),(p=d?function(t){var e=Math.sin(t*=d)*f,n=Math.sin(d-t)*f,r=n*c+e*g,i=n*u+e*h,o=n*s+e*l;return[Math.atan2(i,r)*Mt,Math.atan2(o,Math.sqrt(r*r+i*i))*Mt]}:function(){return[n*Mt,r*Mt]}).distance=d,p;var n,r,i,o,a,s,A,l,c,u,g,h,d,f,p},t.geo.length=function(e){return Ir=0,t.geo.stream(e,jr),Ir};var jr={sphere:N,point:N,lineStart:function(){var t,e,n;function r(r,i){var o=Math.sin(i*=Dt),a=Math.cos(i),s=C((r*=Dt)-t),A=Math.cos(s);Ir+=Math.atan2(Math.sqrt((s=a*Math.sin(s))*s+(s=n*o-e*a*A)*s),e*o+n*a*A),t=r,e=o,n=a}jr.point=function(i,o){t=i*Dt,e=Math.sin(o*=Dt),n=Math.cos(o),jr.point=r},jr.lineEnd=function(){jr.point=jr.lineEnd=N}},lineEnd:N,polygonStart:N,polygonEnd:N};function Yr(t,e){function n(e,n){var r=Math.cos(e),i=Math.cos(n),o=t(r*i);return[o*i*Math.sin(e),o*Math.sin(n)]}return n.invert=function(t,n){var r=Math.sqrt(t*t+n*n),i=e(r),o=Math.sin(i),a=Math.cos(i);return[Math.atan2(t*o,r*a),Math.asin(r&&n*o/r)]},n}var Pr=Yr(function(t){return Math.sqrt(2/(1+t))},function(t){return 2*Math.asin(t/2)});(t.geo.azimuthalEqualArea=function(){return xr(Pr)}).raw=Pr;var Kr=Yr(function(t){var e=Math.acos(t);return e&&e/Math.sin(e)},k);function zr(t,e){var n=Math.cos(t),r=function(t){return Math.tan(Qt/4+t/2)},i=t===e?Math.sin(t):Math.log(n/Math.cos(e))/Math.log(r(e)/r(t)),o=n*Math.pow(r(t),i)/i;if(!i)return Vr;function a(t,e){o>0?e<-_t+Bt&&(e=-_t+Bt):e>_t-Bt&&(e=_t-Bt);var n=o/Math.pow(r(e),i);return[n*Math.sin(i*t),o-n*Math.cos(i*t)]}return a.invert=function(t,e){var n=o-e,r=kt(i)*Math.sqrt(t*t+n*n);return[Math.atan2(t,n)/i,2*Math.atan(Math.pow(o/r,1/i))-_t]},a}function Jr(t,e){var n=Math.cos(t),r=t===e?Math.sin(t):(n-Math.cos(e))/(e-t),i=n/r+t;if(C(r)<Bt)return Mr;function o(t,e){var n=i-e;return[n*Math.sin(r*t),i-n*Math.cos(r*t)]}return o.invert=function(t,e){var n=i-e;return[Math.atan2(t,n)/r,i-kt(r)*Math.sqrt(t*t+n*n)]},o}(t.geo.azimuthalEquidistant=function(){return xr(Kr)}).raw=Kr,(t.geo.conicConformal=function(){return ir(zr)}).raw=zr,(t.geo.conicEquidistant=function(){return ir(Jr)}).raw=Jr;var qr=Yr(function(t){return 1/t},Math.atan);function Vr(t,e){return[t,Math.log(Math.tan(Qt/4+e/2))]}function Wr(t){var e,n=xr(t),r=n.scale,i=n.translate,o=n.clipExtent;return n.scale=function(){var t=r.apply(n,arguments);return t===n?e?n.clipExtent(null):n:t},n.translate=function(){var t=i.apply(n,arguments);return t===n?e?n.clipExtent(null):n:t},n.clipExtent=function(t){var a=o.apply(n,arguments);if(a===n){if(e=null==t){var s=Qt*r(),A=i();o([[A[0]-s,A[1]-s],[A[0]+s,A[1]+s]])}}else e&&(a=null);return a},n.clipExtent(null)}(t.geo.gnomonic=function(){return xr(qr)}).raw=qr,Vr.invert=function(t,e){return[t,2*Math.atan(Math.exp(e))-_t]},(t.geo.mercator=function(){return Wr(Vr)}).raw=Vr;var Xr=Yr(function(){return 1},Math.asin);(t.geo.orthographic=function(){return xr(Xr)}).raw=Xr;var Zr=Yr(function(t){return 1/(1+t)},function(t){return 2*Math.atan(t)});function $r(t,e){return[Math.log(Math.tan(Qt/4+e/2)),-t]}function ti(t){return t[0]}function ei(t){return t[1]}function ni(t){for(var e=t.length,n=[0,1],r=2,i=2;i<e;i++){for(;r>1&&St(t[n[r-2]],t[n[r-1]],t[i])<=0;)--r;n[r++]=i}return n.slice(0,r)}function ri(t,e){return t[0]-e[0]||t[1]-e[1]}(t.geo.stereographic=function(){return xr(Zr)}).raw=Zr,$r.invert=function(t,e){return[-e,2*Math.atan(Math.exp(t))-_t]},(t.geo.transverseMercator=function(){var t=Wr($r),e=t.center,n=t.rotate;return t.center=function(t){return t?e([-t[1],t[0]]):[(t=e())[1],-t[0]]},t.rotate=function(t){return t?n([t[0],t[1],t.length>2?t[2]+90:90]):[(t=n())[0],t[1],t[2]-90]},n([0,0,90])}).raw=$r,t.geom={},t.geom.hull=function(t){var e=ti,n=ei;if(arguments.length)return r(t);function r(t){if(t.length<3)return[];var r,i=fe(e),o=fe(n),a=t.length,s=[],A=[];for(r=0;r<a;r++)s.push([+i.call(this,t[r],r),+o.call(this,t[r],r),r]);for(s.sort(ri),r=0;r<a;r++)A.push([s[r][0],-s[r][1]]);var l=ni(s),c=ni(A),u=c[0]===l[0],g=c[c.length-1]===l[l.length-1],h=[];for(r=l.length-1;r>=0;--r)h.push(t[s[l[r]][2]]);for(r=+u;r<c.length-g;++r)h.push(t[s[c[r]][2]]);return h}return r.x=function(t){return arguments.length?(e=t,r):e},r.y=function(t){return arguments.length?(n=t,r):n},r},t.geom.polygon=function(t){return j(t,ii),t};var ii=t.geom.polygon.prototype=[];function oi(t,e,n){return(n[0]-e[0])*(t[1]-e[1])<(n[1]-e[1])*(t[0]-e[0])}function ai(t,e,n,r){var i=t[0],o=n[0],a=e[0]-i,s=r[0]-o,A=t[1],l=n[1],c=e[1]-A,u=r[1]-l,g=(s*(A-l)-u*(i-o))/(u*a-s*c);return[i+g*a,A+g*c]}function si(t){var e=t[0],n=t[t.length-1];return!(e[0]-n[0]||e[1]-n[1])}ii.area=function(){for(var t,e=-1,n=this.length,r=this[n-1],i=0;++e<n;)t=r,r=this[e],i+=t[1]*r[0]-t[0]*r[1];return.5*i},ii.centroid=function(t){var e,n,r=-1,i=this.length,o=0,a=0,s=this[i-1];for(arguments.length||(t=-1/(6*this.area()));++r<i;)e=s,s=this[r],n=e[0]*s[1]-s[0]*e[1],o+=(e[0]+s[0])*n,a+=(e[1]+s[1])*n;return[o*t,a*t]},ii.clip=function(t){for(var e,n,r,i,o,a,s=si(t),A=-1,l=this.length-si(this),c=this[l-1];++A<l;){for(e=t.slice(),t.length=0,i=this[A],o=e[(r=e.length-s)-1],n=-1;++n<r;)oi(a=e[n],c,i)?(oi(o,c,i)||t.push(ai(o,a,c,i)),t.push(a)):oi(o,c,i)&&t.push(ai(o,a,c,i)),o=a;s&&t.push(t[0]),c=i}return t};var Ai,li,ci,ui,gi,hi=[],di=[];function fi(){Li(this),this.edge=this.site=this.circle=null}function pi(t){var e=hi.pop()||new fi;return e.site=t,e}function Ii(t){wi(t),ci.remove(t),hi.push(t),Li(t)}function Ci(t){var e=t.circle,n=e.x,r=e.cy,i={x:n,y:r},o=t.P,a=t.N,s=[t];Ii(t);for(var A=o;A.circle&&C(n-A.circle.x)<Bt&&C(r-A.circle.cy)<Bt;)o=A.P,s.unshift(A),Ii(A),A=o;s.unshift(A),wi(A);for(var l=a;l.circle&&C(n-l.circle.x)<Bt&&C(r-l.circle.cy)<Bt;)a=l.N,s.push(l),Ii(l),l=a;s.push(l),wi(l);var c,u=s.length;for(c=1;c<u;++c)l=s[c],A=s[c-1],Mi(l.edge,A.site,l.site,i);A=s[0],(l=s[u-1]).edge=Di(A.site,l.site,null,i),Qi(A),Qi(l)}function mi(t){for(var e,n,r,i,o=t.x,a=t.y,s=ci._;s;)if((r=yi(s,a)-o)>Bt)s=s.L;else{if(!((i=o-Ei(s,a))>Bt)){r>-Bt?(e=s.P,n=s):i>-Bt?(e=s,n=s.N):e=n=s;break}if(!s.R){e=s;break}s=s.R}var A=pi(t);if(ci.insert(e,A),e||n){if(e===n)return wi(e),n=pi(e.site),ci.insert(A,n),A.edge=n.edge=Di(e.site,A.site),Qi(e),void Qi(n);if(n){wi(e),wi(n);var l=e.site,c=l.x,u=l.y,g=t.x-c,h=t.y-u,d=n.site,f=d.x-c,p=d.y-u,I=2*(g*p-h*f),C=g*g+h*h,m=f*f+p*p,y={x:(p*C-h*m)/I+c,y:(g*m-f*C)/I+u};Mi(n.edge,l,d,y),A.edge=Di(l,t,null,y),n.edge=Di(t,d,null,y),Qi(e),Qi(n)}else A.edge=Di(e.site,A.site)}}function yi(t,e){var n=t.site,r=n.x,i=n.y,o=i-e;if(!o)return r;var a=t.P;if(!a)return-1/0;var s=(n=a.site).x,A=n.y,l=A-e;if(!l)return s;var c=s-r,u=1/o-1/l,g=c/l;return u?(-g+Math.sqrt(g*g-2*u*(c*c/(-2*l)-A+l/2+i-o/2)))/u+r:(r+s)/2}function Ei(t,e){var n=t.N;if(n)return yi(n,e);var r=t.site;return r.y===e?r.x:1/0}function vi(t){this.site=t,this.edges=[]}function Bi(t,e){return e.angle-t.angle}function bi(){Li(this),this.x=this.y=this.arc=this.site=this.cy=null}function Qi(t){var e=t.P,n=t.N;if(e&&n){var r=e.site,i=t.site,o=n.site;if(r!==o){var a=i.x,s=i.y,A=r.x-a,l=r.y-s,c=o.x-a,u=2*(A*(p=o.y-s)-l*c);if(!(u>=-bt)){var g=A*A+l*l,h=c*c+p*p,d=(p*g-l*h)/u,f=(A*h-c*g)/u,p=f+s,I=di.pop()||new bi;I.arc=t,I.site=i,I.x=d+a,I.y=p+Math.sqrt(d*d+f*f),I.cy=p,t.circle=I;for(var C=null,m=gi._;m;)if(I.y<m.y||I.y===m.y&&I.x<=m.x){if(!m.L){C=m.P;break}m=m.L}else{if(!m.R){C=m;break}m=m.R}gi.insert(C,I),C||(ui=I)}}}}function wi(t){var e=t.circle;e&&(e.P||(ui=e.N),gi.remove(e),di.push(e),Li(e),t.circle=null)}function xi(t,e){var n=t.b;if(n)return!0;var r,i,o=t.a,a=e[0][0],s=e[1][0],A=e[0][1],l=e[1][1],c=t.l,u=t.r,g=c.x,h=c.y,d=u.x,f=u.y,p=(g+d)/2,I=(h+f)/2;if(f===h){if(p<a||p>=s)return;if(g>d){if(o){if(o.y>=l)return}else o={x:p,y:A};n={x:p,y:l}}else{if(o){if(o.y<A)return}else o={x:p,y:l};n={x:p,y:A}}}else if(i=I-(r=(g-d)/(f-h))*p,r<-1||r>1)if(g>d){if(o){if(o.y>=l)return}else o={x:(A-i)/r,y:A};n={x:(l-i)/r,y:l}}else{if(o){if(o.y<A)return}else o={x:(l-i)/r,y:l};n={x:(A-i)/r,y:A}}else if(h<f){if(o){if(o.x>=s)return}else o={x:a,y:r*a+i};n={x:s,y:r*s+i}}else{if(o){if(o.x<a)return}else o={x:s,y:r*s+i};n={x:a,y:r*a+i}}return t.a=o,t.b=n,!0}function _i(t,e){this.l=t,this.r=e,this.a=this.b=null}function Di(t,e,n,r){var i=new _i(t,e);return Ai.push(i),n&&Mi(i,t,e,n),r&&Mi(i,e,t,r),li[t.i].edges.push(new ki(i,t,e)),li[e.i].edges.push(new ki(i,e,t)),i}function Mi(t,e,n,r){t.a||t.b?t.l===n?t.b=r:t.a=r:(t.a=r,t.l=e,t.r=n)}function ki(t,e,n){var r=t.a,i=t.b;this.edge=t,this.site=e,this.angle=n?Math.atan2(n.y-e.y,n.x-e.x):t.l===e?Math.atan2(i.x-r.x,r.y-i.y):Math.atan2(r.x-i.x,i.y-r.y)}function Si(){this._=null}function Li(t){t.U=t.C=t.L=t.R=t.P=t.N=null}function Ri(t,e){var n=e,r=e.R,i=n.U;i?i.L===n?i.L=r:i.R=r:t._=r,r.U=i,n.U=r,n.R=r.L,n.R&&(n.R.U=n),r.L=n}function Ni(t,e){var n=e,r=e.L,i=n.U;i?i.L===n?i.L=r:i.R=r:t._=r,r.U=i,n.U=r,n.L=r.R,n.L&&(n.L.U=n),r.R=n}function Fi(t){for(;t.L;)t=t.L;return t}function Ti(t,e){var n,r,i,o=t.sort(Ui).pop();for(Ai=[],li=new Array(t.length),ci=new Si,gi=new Si;;)if(i=ui,o&&(!i||o.y<i.y||o.y===i.y&&o.x<i.x))o.x===n&&o.y===r||(li[o.i]=new vi(o),mi(o),n=o.x,r=o.y),o=t.pop();else{if(!i)break;Ci(i.arc)}e&&(function(t){for(var e,n=Ai,r=er(t[0][0],t[0][1],t[1][0],t[1][1]),i=n.length;i--;)(!xi(e=n[i],t)||!r(e)||C(e.a.x-e.b.x)<Bt&&C(e.a.y-e.b.y)<Bt)&&(e.a=e.b=null,n.splice(i,1))}(e),function(t){for(var e,n,r,i,o,a,s,A,l,c,u=t[0][0],g=t[1][0],h=t[0][1],d=t[1][1],f=li,p=f.length;p--;)if((o=f[p])&&o.prepare())for(A=(s=o.edges).length,a=0;a<A;)r=(c=s[a].end()).x,i=c.y,e=(l=s[++a%A].start()).x,n=l.y,(C(r-e)>Bt||C(i-n)>Bt)&&(s.splice(a,0,new ki((I=o.site,m=c,y=C(r-u)<Bt&&d-i>Bt?{x:u,y:C(e-u)<Bt?n:d}:C(i-d)<Bt&&g-r>Bt?{x:C(n-d)<Bt?e:g,y:d}:C(r-g)<Bt&&i-h>Bt?{x:g,y:C(e-g)<Bt?n:h}:C(i-h)<Bt&&r-u>Bt?{x:C(n-h)<Bt?e:u,y:h}:null,E=void 0,(E=new _i(I,null)).a=m,E.b=y,Ai.push(E),E),o.site,null)),++A);var I,m,y,E}(e));var a={cells:li,edges:Ai};return ci=gi=Ai=li=null,a}function Ui(t,e){return e.y-t.y||e.x-t.x}vi.prototype.prepare=function(){for(var t,e=this.edges,n=e.length;n--;)(t=e[n].edge).b&&t.a||e.splice(n,1);return e.sort(Bi),e.length},ki.prototype={start:function(){return this.edge.l===this.site?this.edge.a:this.edge.b},end:function(){return this.edge.l===this.site?this.edge.b:this.edge.a}},Si.prototype={insert:function(t,e){var n,r,i;if(t){if(e.P=t,e.N=t.N,t.N&&(t.N.P=e),t.N=e,t.R){for(t=t.R;t.L;)t=t.L;t.L=e}else t.R=e;n=t}else this._?(t=Fi(this._),e.P=null,e.N=t,t.P=t.L=e,n=t):(e.P=e.N=null,this._=e,n=null);for(e.L=e.R=null,e.U=n,e.C=!0,t=e;n&&n.C;)n===(r=n.U).L?(i=r.R)&&i.C?(n.C=i.C=!1,r.C=!0,t=r):(t===n.R&&(Ri(this,n),n=(t=n).U),n.C=!1,r.C=!0,Ni(this,r)):(i=r.L)&&i.C?(n.C=i.C=!1,r.C=!0,t=r):(t===n.L&&(Ni(this,n),n=(t=n).U),n.C=!1,r.C=!0,Ri(this,r)),n=t.U;this._.C=!1},remove:function(t){t.N&&(t.N.P=t.P),t.P&&(t.P.N=t.N),t.N=t.P=null;var e,n,r,i=t.U,o=t.L,a=t.R;if(n=o?a?Fi(a):o:a,i?i.L===t?i.L=n:i.R=n:this._=n,o&&a?(r=n.C,n.C=t.C,n.L=o,o.U=n,n!==a?(i=n.U,n.U=t.U,t=n.R,i.L=t,n.R=a,a.U=n):(n.U=i,i=n,t=n.R)):(r=t.C,t=n),t&&(t.U=i),!r)if(t&&t.C)t.C=!1;else{do{if(t===this._)break;if(t===i.L){if((e=i.R).C&&(e.C=!1,i.C=!0,Ri(this,i),e=i.R),e.L&&e.L.C||e.R&&e.R.C){e.R&&e.R.C||(e.L.C=!1,e.C=!0,Ni(this,e),e=i.R),e.C=i.C,i.C=e.R.C=!1,Ri(this,i),t=this._;break}}else if((e=i.L).C&&(e.C=!1,i.C=!0,Ni(this,i),e=i.L),e.L&&e.L.C||e.R&&e.R.C){e.L&&e.L.C||(e.R.C=!1,e.C=!0,Ri(this,e),e=i.L),e.C=i.C,i.C=e.L.C=!1,Ni(this,i),t=this._;break}e.C=!0,t=i,i=i.U}while(!t.C);t&&(t.C=!1)}}},t.geom.voronoi=function(t){var e=ti,n=ei,r=e,i=n,o=Oi;if(t)return a(t);function a(t){var e=new Array(t.length),n=o[0][0],r=o[0][1],i=o[1][0],a=o[1][1];return Ti(s(t),o).cells.forEach(function(o,s){var A=o.edges,l=o.site;(e[s]=A.length?A.map(function(t){var e=t.start();return[e.x,e.y]}):l.x>=n&&l.x<=i&&l.y>=r&&l.y<=a?[[n,a],[i,a],[i,r],[n,r]]:[]).point=t[s]}),e}function s(t){return t.map(function(t,e){return{x:Math.round(r(t,e)/Bt)*Bt,y:Math.round(i(t,e)/Bt)*Bt,i:e}})}return a.links=function(t){return Ti(s(t)).edges.filter(function(t){return t.l&&t.r}).map(function(e){return{source:t[e.l.i],target:t[e.r.i]}})},a.triangles=function(t){var e=[];return Ti(s(t)).cells.forEach(function(n,r){for(var i,o,a,s,A=n.site,l=n.edges.sort(Bi),c=-1,u=l.length,g=l[u-1].edge,h=g.l===A?g.r:g.l;++c<u;)i=h,h=(g=l[c].edge).l===A?g.r:g.l,r<i.i&&r<h.i&&(a=i,s=h,((o=A).x-s.x)*(a.y-o.y)-(o.x-a.x)*(s.y-o.y)<0)&&e.push([t[r],t[i.i],t[h.i]])}),e},a.x=function(t){return arguments.length?(r=fe(e=t),a):e},a.y=function(t){return arguments.length?(i=fe(n=t),a):n},a.clipExtent=function(t){return arguments.length?(o=null==t?Oi:t,a):o===Oi?null:o},a.size=function(t){return arguments.length?a.clipExtent(t&&[[0,0],t]):o===Oi?null:o&&o[1]},a};var Oi=[[-1e6,-1e6],[1e6,1e6]];function Gi(t){return t.x}function Hi(t){return t.y}function ji(e,n){e=t.rgb(e),n=t.rgb(n);var r=e.r,i=e.g,o=e.b,a=n.r-r,s=n.g-i,A=n.b-o;return function(t){return"#"+Ae(Math.round(r+a*t))+Ae(Math.round(i+s*t))+Ae(Math.round(o+A*t))}}function Yi(t,e){var n,r={},i={};for(n in t)n in e?r[n]=qi(t[n],e[n]):i[n]=t[n];for(n in e)n in t||(i[n]=e[n]);return function(t){for(n in r)i[n]=r[n](t);return i}}function Pi(t,e){return t=+t,e=+e,function(n){return t*(1-n)+e*n}}function Ki(t,e){var n,r,i,o=zi.lastIndex=Ji.lastIndex=0,a=-1,s=[],A=[];for(t+="",e+="";(n=zi.exec(t))&&(r=Ji.exec(e));)(i=r.index)>o&&(i=e.slice(o,i),s[a]?s[a]+=i:s[++a]=i),(n=n[0])===(r=r[0])?s[a]?s[a]+=r:s[++a]=r:(s[++a]=null,A.push({i:a,x:Pi(n,r)})),o=Ji.lastIndex;return o<e.length&&(i=e.slice(o),s[a]?s[a]+=i:s[++a]=i),s.length<2?A[0]?(e=A[0].x,function(t){return e(t)+""}):function(){return e}:(e=A.length,function(t){for(var n,r=0;r<e;++r)s[(n=A[r]).i]=n.x(t);return s.join("")})}t.geom.delaunay=function(e){return t.geom.voronoi().triangles(e)},t.geom.quadtree=function(t,e,n,r,i){var o,a=ti,s=ei;if(o=arguments.length)return a=Gi,s=Hi,3===o&&(i=n,r=e,n=e=0),A(t);function A(t){var A,l,c,u,g,h,d,f,p,I=fe(a),m=fe(s);if(null!=e)h=e,d=n,f=r,p=i;else if(f=p=-(h=d=1/0),l=[],c=[],g=t.length,o)for(u=0;u<g;++u)(A=t[u]).x<h&&(h=A.x),A.y<d&&(d=A.y),A.x>f&&(f=A.x),A.y>p&&(p=A.y),l.push(A.x),c.push(A.y);else for(u=0;u<g;++u){var y=+I(A=t[u],u),E=+m(A,u);y<h&&(h=y),E<d&&(d=E),y>f&&(f=y),E>p&&(p=E),l.push(y),c.push(E)}var v=f-h,B=p-d;function b(t,e,n,r,i,o,a,s){if(!isNaN(n)&&!isNaN(r))if(t.leaf){var A=t.x,l=t.y;if(null!=A)if(C(A-n)+C(l-r)<.01)Q(t,e,n,r,i,o,a,s);else{var c=t.point;t.x=t.y=t.point=null,Q(t,c,A,l,i,o,a,s),Q(t,e,n,r,i,o,a,s)}else t.x=n,t.y=r,t.point=e}else Q(t,e,n,r,i,o,a,s)}function Q(t,e,n,r,i,o,a,s){var A=.5*(i+a),l=.5*(o+s),c=n>=A,u=r>=l,g=u<<1|c;t.leaf=!1,c?i=A:a=A,u?o=l:s=l,b(t=t.nodes[g]||(t.nodes[g]={leaf:!0,nodes:[],point:null,x:null,y:null,add:function(t){b(w,t,+I(t,++u),+m(t,u),h,d,f,p)}}),e,n,r,i,o,a,s)}v>B?p=d+v:f=h+B;var w={leaf:!0,nodes:[],point:null,x:null,y:null,add:function(t){b(w,t,+I(t,++u),+m(t,u),h,d,f,p)},visit:function(t){!function t(e,n,r,i,o,a){if(!e(n,r,i,o,a)){var s=.5*(r+o),A=.5*(i+a),l=n.nodes;l[0]&&t(e,l[0],r,i,s,A),l[1]&&t(e,l[1],s,i,o,A),l[2]&&t(e,l[2],r,A,s,a),l[3]&&t(e,l[3],s,A,o,a)}}(t,w,h,d,f,p)},find:function(t){return function(t,e,n,r,i,o,a){var s,A=1/0;return function t(l,c,u,g,h){if(!(c>o||u>a||g<r||h<i)){if(d=l.point){var d,f=e-l.x,p=n-l.y,I=f*f+p*p;if(I<A){var C=Math.sqrt(A=I);r=e-C,i=n-C,o=e+C,a=n+C,s=d}}for(var m=l.nodes,y=.5*(c+g),E=.5*(u+h),v=(n>=E)<<1|e>=y,B=v+4;v<B;++v)if(l=m[3&v])switch(3&v){case 0:t(l,c,u,y,E);break;case 1:t(l,y,u,g,E);break;case 2:t(l,c,E,y,h);break;case 3:t(l,y,E,g,h)}}}(t,r,i,o,a),s}(w,t[0],t[1],h,d,f,p)}};if(u=-1,null==e){for(;++u<g;)b(w,t[u],l[u],c[u],h,d,f,p);--u}else t.forEach(w.add);return l=c=t=A=null,w}return A.x=function(t){return arguments.length?(a=t,A):a},A.y=function(t){return arguments.length?(s=t,A):s},A.extent=function(t){return arguments.length?(null==t?e=n=r=i=null:(e=+t[0][0],n=+t[0][1],r=+t[1][0],i=+t[1][1]),A):null==e?null:[[e,n],[r,i]]},A.size=function(t){return arguments.length?(null==t?e=n=r=i=null:(e=n=0,r=+t[0],i=+t[1]),A):null==e?null:[r-e,i-n]},A},t.interpolateRgb=ji,t.interpolateObject=Yi,t.interpolateNumber=Pi,t.interpolateString=Ki;var zi=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,Ji=new RegExp(zi.source,"g");function qi(e,n){for(var r,i=t.interpolators.length;--i>=0&&!(r=t.interpolators[i](e,n)););return r}function Vi(t,e){var n,r=[],i=[],o=t.length,a=e.length,s=Math.min(t.length,e.length);for(n=0;n<s;++n)r.push(qi(t[n],e[n]));for(;n<o;++n)i[n]=t[n];for(;n<a;++n)i[n]=e[n];return function(t){for(n=0;n<s;++n)i[n]=r[n](t);return i}}t.interpolate=qi,t.interpolators=[function(t,e){var n=typeof e;return("string"===n?de.has(e.toLowerCase())||/^(#|rgb\(|hsl\()/i.test(e)?ji:Ki:e instanceof Ht?ji:Array.isArray(e)?Vi:"object"===n&&isNaN(e)?Yi:Pi)(t,e)}],t.interpolateArray=Vi;var Wi=function(){return k},Xi=t.map({linear:Wi,poly:function(t){return function(e){return Math.pow(e,t)}},quad:function(){return eo},cubic:function(){return no},sin:function(){return io},exp:function(){return oo},circle:function(){return ao},elastic:function(t,e){var n;return arguments.length<2&&(e=.45),arguments.length?n=e/wt*Math.asin(1/t):(t=1,n=e/4),function(r){return 1+t*Math.pow(2,-10*r)*Math.sin((r-n)*wt/e)}},back:function(t){return t||(t=1.70158),function(e){return e*e*((t+1)*e-t)}},bounce:function(){return so}}),Zi=t.map({in:k,out:$i,"in-out":to,"out-in":function(t){return to($i(t))}});function $i(t){return function(e){return 1-t(1-e)}}function to(t){return function(e){return.5*(e<.5?t(2*e):2-t(2-2*e))}}function eo(t){return t*t}function no(t){return t*t*t}function ro(t){if(t<=0)return 0;if(t>=1)return 1;var e=t*t,n=e*t;return 4*(t<.5?n:3*(t-e)+n-.75)}function io(t){return 1-Math.cos(t*_t)}function oo(t){return Math.pow(2,10*(t-1))}function ao(t){return 1-Math.sqrt(1-t*t)}function so(t){return t<1/2.75?7.5625*t*t:t<2/2.75?7.5625*(t-=1.5/2.75)*t+.75:t<2.5/2.75?7.5625*(t-=2.25/2.75)*t+.9375:7.5625*(t-=2.625/2.75)*t+.984375}function Ao(t,e){return e-=t,function(n){return Math.round(t+e*n)}}function lo(t){var e,n,r,i=[t.a,t.b],o=[t.c,t.d],a=uo(i),s=co(i,o),A=uo(((e=o)[0]+=(r=-s)*(n=i)[0],e[1]+=r*n[1],e))||0;i[0]*o[1]<o[0]*i[1]&&(i[0]*=-1,i[1]*=-1,a*=-1,s*=-1),this.rotate=(a?Math.atan2(i[1],i[0]):Math.atan2(-o[0],o[1]))*Mt,this.translate=[t.e,t.f],this.scale=[a,A],this.skew=A?Math.atan2(s,A)*Mt:0}function co(t,e){return t[0]*e[0]+t[1]*e[1]}function uo(t){var e=Math.sqrt(co(t,t));return e&&(t[0]/=e,t[1]/=e),e}t.ease=function(t){var e,r=t.indexOf("-"),i=r>=0?t.slice(0,r):t,o=r>=0?t.slice(r+1):"in";return i=Xi.get(i)||Wi,o=Zi.get(o)||k,e=o(i.apply(null,n.call(arguments,1))),function(t){return t<=0?0:t>=1?1:e(t)}},t.interpolateHcl=function(e,n){e=t.hcl(e),n=t.hcl(n);var r=e.h,i=e.c,o=e.l,a=n.h-r,s=n.c-i,A=n.l-o;return isNaN(s)&&(s=0,i=isNaN(i)?n.c:i),isNaN(a)?(a=0,r=isNaN(r)?n.h:r):a>180?a-=360:a<-180&&(a+=360),function(t){return Jt(r+a*t,i+s*t,o+A*t)+""}},t.interpolateHsl=function(e,n){e=t.hsl(e),n=t.hsl(n);var r=e.h,i=e.s,o=e.l,a=n.h-r,s=n.s-i,A=n.l-o;return isNaN(s)&&(s=0,i=isNaN(i)?n.s:i),isNaN(a)?(a=0,r=isNaN(r)?n.h:r):a>180?a-=360:a<-180&&(a+=360),function(t){return Pt(r+a*t,i+s*t,o+A*t)+""}},t.interpolateLab=function(e,n){e=t.lab(e),n=t.lab(n);var r=e.l,i=e.a,o=e.b,a=n.l-r,s=n.a-i,A=n.b-o;return function(t){return te(r+a*t,i+s*t,o+A*t)+""}},t.interpolateRound=Ao,t.transform=function(e){var n=i.createElementNS(t.ns.prefix.svg,"g");return(t.transform=function(t){if(null!=t){n.setAttribute("transform",t);var e=n.transform.baseVal.consolidate()}return new lo(e?e.matrix:go)})(e)},lo.prototype.toString=function(){return"translate("+this.translate+")rotate("+this.rotate+")skewX("+this.skew+")scale("+this.scale+")"};var go={a:1,b:0,c:0,d:1,e:0,f:0};function ho(t){return t.length?t.pop()+",":""}function fo(e,n){var r=[],i=[];return e=t.transform(e),n=t.transform(n),function(t,e,n,r){if(t[0]!==e[0]||t[1]!==e[1]){var i=n.push("translate(",null,",",null,")");r.push({i:i-4,x:Pi(t[0],e[0])},{i:i-2,x:Pi(t[1],e[1])})}else(e[0]||e[1])&&n.push("translate("+e+")")}(e.translate,n.translate,r,i),function(t,e,n,r){t!==e?(t-e>180?e+=360:e-t>180&&(t+=360),r.push({i:n.push(ho(n)+"rotate(",null,")")-2,x:Pi(t,e)})):e&&n.push(ho(n)+"rotate("+e+")")}(e.rotate,n.rotate,r,i),function(t,e,n,r){t!==e?r.push({i:n.push(ho(n)+"skewX(",null,")")-2,x:Pi(t,e)}):e&&n.push(ho(n)+"skewX("+e+")")}(e.skew,n.skew,r,i),function(t,e,n,r){if(t[0]!==e[0]||t[1]!==e[1]){var i=n.push(ho(n)+"scale(",null,",",null,")");r.push({i:i-4,x:Pi(t[0],e[0])},{i:i-2,x:Pi(t[1],e[1])})}else 1===e[0]&&1===e[1]||n.push(ho(n)+"scale("+e+")")}(e.scale,n.scale,r,i),e=n=null,function(t){for(var e,n=-1,o=i.length;++n<o;)r[(e=i[n]).i]=e.x(t);return r.join("")}}function po(t,e){return e=(e-=t=+t)||1/e,function(n){return(n-t)/e}}function Io(t,e){return e=(e-=t=+t)||1/e,function(n){return Math.max(0,Math.min(1,(n-t)/e))}}function Co(t){for(var e=t.source,n=t.target,r=function(t,e){if(t===e)return t;for(var n=mo(t),r=mo(e),i=n.pop(),o=r.pop(),a=null;i===o;)a=i,i=n.pop(),o=r.pop();return a}(e,n),i=[e];e!==r;)e=e.parent,i.push(e);for(var o=i.length;n!==r;)i.splice(o,0,n),n=n.parent;return i}function mo(t){for(var e=[],n=t.parent;null!=n;)e.push(t),t=n,n=n.parent;return e.push(t),e}function yo(t){t.fixed|=2}function Eo(t){t.fixed&=-7}function vo(t){t.fixed|=4,t.px=t.x,t.py=t.y}function Bo(t){t.fixed&=-5}t.interpolateTransform=fo,t.layout={},t.layout.bundle=function(){return function(t){for(var e=[],n=-1,r=t.length;++n<r;)e.push(Co(t[n]));return e}},t.layout.chord=function(){var e,n,r,i,o,a,s,A={},l=0;function c(){var A,c,g,h,d,f={},p=[],I=t.range(i),C=[];for(e=[],n=[],A=0,h=-1;++h<i;){for(c=0,d=-1;++d<i;)c+=r[h][d];p.push(c),C.push(t.range(i)),A+=c}for(o&&I.sort(function(t,e){return o(p[t],p[e])}),a&&C.forEach(function(t,e){t.sort(function(t,n){return a(r[e][t],r[e][n])})}),A=(wt-l*i)/A,c=0,h=-1;++h<i;){for(g=c,d=-1;++d<i;){var m=I[h],y=C[m][d],E=r[m][y],v=c,B=c+=E*A;f[m+"-"+y]={index:m,subindex:y,startAngle:v,endAngle:B,value:E}}n[m]={index:m,startAngle:g,endAngle:c,value:p[m]},c+=l}for(h=-1;++h<i;)for(d=h-1;++d<i;){var b=f[h+"-"+d],Q=f[d+"-"+h];(b.value||Q.value)&&e.push(b.value<Q.value?{source:Q,target:b}:{source:b,target:Q})}s&&u()}function u(){e.sort(function(t,e){return s((t.source.value+t.target.value)/2,(e.source.value+e.target.value)/2)})}return A.matrix=function(t){return arguments.length?(i=(r=t)&&r.length,e=n=null,A):r},A.padding=function(t){return arguments.length?(l=t,e=n=null,A):l},A.sortGroups=function(t){return arguments.length?(o=t,e=n=null,A):o},A.sortSubgroups=function(t){return arguments.length?(a=t,e=null,A):a},A.sortChords=function(t){return arguments.length?(s=t,e&&u(),A):s},A.chords=function(){return e||c(),e},A.groups=function(){return n||c(),n},A},t.layout.force=function(){var e,n,r,i,o,a,s={},A=t.dispatch("start","tick","end"),l=[1,1],c=.9,u=bo,g=Qo,h=-30,d=wo,f=.1,p=.64,I=[],C=[];function m(t){return function(e,n,r,i){if(e.point!==t){var o=e.cx-t.x,a=e.cy-t.y,s=i-n,A=o*o+a*a;if(s*s/p<A){if(A<d){var l=e.charge/A;t.px-=o*l,t.py-=a*l}return!0}e.point&&A&&A<d&&(l=e.pointCharge/A,t.px-=o*l,t.py-=a*l)}return!e.charge}}function y(e){e.px=t.event.x,e.py=t.event.y,s.resume()}return s.tick=function(){if((r*=.99)<.005)return e=null,A.end({type:"end",alpha:r=0}),!0;var n,s,u,g,d,p,y,E,v,B=I.length,b=C.length;for(s=0;s<b;++s)g=(u=C[s]).source,(p=(E=(d=u.target).x-g.x)*E+(v=d.y-g.y)*v)&&(E*=p=r*o[s]*((p=Math.sqrt(p))-i[s])/p,v*=p,d.x-=E*(y=g.weight+d.weight?g.weight/(g.weight+d.weight):.5),d.y-=v*y,g.x+=E*(y=1-y),g.y+=v*y);if((y=r*f)&&(E=l[0]/2,v=l[1]/2,s=-1,y))for(;++s<B;)(u=I[s]).x+=(E-u.x)*y,u.y+=(v-u.y)*y;if(h)for(function t(e,n,r){var i=0,o=0;if(e.charge=0,!e.leaf)for(var a,s=e.nodes,A=s.length,l=-1;++l<A;)null!=(a=s[l])&&(t(a,n,r),e.charge+=a.charge,i+=a.charge*a.cx,o+=a.charge*a.cy);if(e.point){e.leaf||(e.point.x+=Math.random()-.5,e.point.y+=Math.random()-.5);var c=n*r[e.point.index];e.charge+=e.pointCharge=c,i+=c*e.point.x,o+=c*e.point.y}e.cx=i/e.charge,e.cy=o/e.charge}(n=t.geom.quadtree(I),r,a),s=-1;++s<B;)(u=I[s]).fixed||n.visit(m(u));for(s=-1;++s<B;)(u=I[s]).fixed?(u.x=u.px,u.y=u.py):(u.x-=(u.px-(u.px=u.x))*c,u.y-=(u.py-(u.py=u.y))*c);A.tick({type:"tick",alpha:r})},s.nodes=function(t){return arguments.length?(I=t,s):I},s.links=function(t){return arguments.length?(C=t,s):C},s.size=function(t){return arguments.length?(l=t,s):l},s.linkDistance=function(t){return arguments.length?(u="function"==typeof t?t:+t,s):u},s.distance=s.linkDistance,s.linkStrength=function(t){return arguments.length?(g="function"==typeof t?t:+t,s):g},s.friction=function(t){return arguments.length?(c=+t,s):c},s.charge=function(t){return arguments.length?(h="function"==typeof t?t:+t,s):h},s.chargeDistance=function(t){return arguments.length?(d=t*t,s):Math.sqrt(d)},s.gravity=function(t){return arguments.length?(f=+t,s):f},s.theta=function(t){return arguments.length?(p=t*t,s):Math.sqrt(p)},s.alpha=function(t){return arguments.length?(t=+t,r?t>0?r=t:(e.c=null,e.t=NaN,e=null,A.end({type:"end",alpha:r=0})):t>0&&(A.start({type:"start",alpha:r=t}),e=Be(s.tick)),s):r},s.start=function(){var t,e,n,r=I.length,A=C.length,c=l[0],d=l[1];for(t=0;t<r;++t)(n=I[t]).index=t,n.weight=0;for(t=0;t<A;++t)"number"==typeof(n=C[t]).source&&(n.source=I[n.source]),"number"==typeof n.target&&(n.target=I[n.target]),++n.source.weight,++n.target.weight;for(t=0;t<r;++t)n=I[t],isNaN(n.x)&&(n.x=f("x",c)),isNaN(n.y)&&(n.y=f("y",d)),isNaN(n.px)&&(n.px=n.x),isNaN(n.py)&&(n.py=n.y);if(i=[],"function"==typeof u)for(t=0;t<A;++t)i[t]=+u.call(this,C[t],t);else for(t=0;t<A;++t)i[t]=u;if(o=[],"function"==typeof g)for(t=0;t<A;++t)o[t]=+g.call(this,C[t],t);else for(t=0;t<A;++t)o[t]=g;if(a=[],"function"==typeof h)for(t=0;t<r;++t)a[t]=+h.call(this,I[t],t);else for(t=0;t<r;++t)a[t]=h;function f(n,i){if(!e){for(e=new Array(r),l=0;l<r;++l)e[l]=[];for(l=0;l<A;++l){var o=C[l];e[o.source.index].push(o.target),e[o.target.index].push(o.source)}}for(var a,s=e[t],l=-1,c=s.length;++l<c;)if(!isNaN(a=s[l][n]))return a;return Math.random()*i}return s.resume()},s.resume=function(){return s.alpha(.1)},s.stop=function(){return s.alpha(0)},s.drag=function(){if(n||(n=t.behavior.drag().origin(k).on("dragstart.force",yo).on("drag.force",y).on("dragend.force",Eo)),!arguments.length)return n;this.on("mouseover.force",vo).on("mouseout.force",Bo).call(n)},t.rebind(s,A,"on")};var bo=20,Qo=1,wo=1/0;function xo(e,n){return t.rebind(e,n,"sort","children","value"),e.nodes=e,e.links=Lo,e}function _o(t,e){for(var n=[t];null!=(t=n.pop());)if(e(t),(i=t.children)&&(r=i.length))for(var r,i;--r>=0;)n.push(i[r])}function Do(t,e){for(var n=[t],r=[];null!=(t=n.pop());)if(r.push(t),(o=t.children)&&(i=o.length))for(var i,o,a=-1;++a<i;)n.push(o[a]);for(;null!=(t=r.pop());)e(t)}function Mo(t){return t.children}function ko(t){return t.value}function So(t,e){return e.value-t.value}function Lo(e){return t.merge(e.map(function(t){return(t.children||[]).map(function(e){return{source:t,target:e}})}))}t.layout.hierarchy=function(){var t=So,e=Mo,n=ko;function r(i){var o,a=[i],s=[];for(i.depth=0;null!=(o=a.pop());)if(s.push(o),(l=e.call(r,o,o.depth))&&(A=l.length)){for(var A,l,c;--A>=0;)a.push(c=l[A]),c.parent=o,c.depth=o.depth+1;n&&(o.value=0),o.children=l}else n&&(o.value=+n.call(r,o,o.depth)||0),delete o.children;return Do(i,function(e){var r,i;t&&(r=e.children)&&r.sort(t),n&&(i=e.parent)&&(i.value+=e.value)}),s}return r.sort=function(e){return arguments.length?(t=e,r):t},r.children=function(t){return arguments.length?(e=t,r):e},r.value=function(t){return arguments.length?(n=t,r):n},r.revalue=function(t){return n&&(_o(t,function(t){t.children&&(t.value=0)}),Do(t,function(t){var e;t.children||(t.value=+n.call(r,t,t.depth)||0),(e=t.parent)&&(e.value+=t.value)})),t},r},t.layout.partition=function(){var e=t.layout.hierarchy(),n=[1,1];function r(t,r){var i=e.call(this,t,r);return function t(e,n,r,i){var o=e.children;if(e.x=n,e.y=e.depth*i,e.dx=r,e.dy=i,o&&(a=o.length)){var a,s,A,l=-1;for(r=e.value?r/e.value:0;++l<a;)t(s=o[l],n,A=s.value*r,i),n+=A}}(i[0],0,n[0],n[1]/function t(e){var n=e.children,r=0;if(n&&(i=n.length))for(var i,o=-1;++o<i;)r=Math.max(r,t(n[o]));return 1+r}(i[0])),i}return r.size=function(t){return arguments.length?(n=t,r):n},xo(r,e)},t.layout.pie=function(){var e=Number,n=Ro,r=0,i=wt,o=0;function a(s){var A,l=s.length,c=s.map(function(t,n){return+e.call(a,t,n)}),u=+("function"==typeof r?r.apply(this,arguments):r),g=("function"==typeof i?i.apply(this,arguments):i)-u,h=Math.min(Math.abs(g)/l,+("function"==typeof o?o.apply(this,arguments):o)),d=h*(g<0?-1:1),f=t.sum(c),p=f?(g-l*d)/f:0,I=t.range(l),C=[];return null!=n&&I.sort(n===Ro?function(t,e){return c[e]-c[t]}:function(t,e){return n(s[t],s[e])}),I.forEach(function(t){C[t]={data:s[t],value:A=c[t],startAngle:u,endAngle:u+=A*p+d,padAngle:h}}),C}return a.value=function(t){return arguments.length?(e=t,a):e},a.sort=function(t){return arguments.length?(n=t,a):n},a.startAngle=function(t){return arguments.length?(r=t,a):r},a.endAngle=function(t){return arguments.length?(i=t,a):i},a.padAngle=function(t){return arguments.length?(o=t,a):o},a};var Ro={};function No(t){return t.x}function Fo(t){return t.y}function To(t,e,n){t.y0=e,t.y=n}t.layout.stack=function(){var e=k,n=Go,r=Ho,i=To,o=No,a=Fo;function s(A,l){if(!(h=A.length))return A;var c=A.map(function(t,n){return e.call(s,t,n)}),u=c.map(function(t){return t.map(function(t,e){return[o.call(s,t,e),a.call(s,t,e)]})}),g=n.call(s,u,l);c=t.permute(c,g),u=t.permute(u,g);var h,d,f,p,I=r.call(s,u,l),C=c[0].length;for(f=0;f<C;++f)for(i.call(s,c[0][f],p=I[f],u[0][f][1]),d=1;d<h;++d)i.call(s,c[d][f],p+=u[d-1][f][1],u[d][f][1]);return A}return s.values=function(t){return arguments.length?(e=t,s):e},s.order=function(t){return arguments.length?(n="function"==typeof t?t:Uo.get(t)||Go,s):n},s.offset=function(t){return arguments.length?(r="function"==typeof t?t:Oo.get(t)||Ho,s):r},s.x=function(t){return arguments.length?(o=t,s):o},s.y=function(t){return arguments.length?(a=t,s):a},s.out=function(t){return arguments.length?(i=t,s):i},s};var Uo=t.map({"inside-out":function(e){var n,r,i=e.length,o=e.map(jo),a=e.map(Yo),s=t.range(i).sort(function(t,e){return o[t]-o[e]}),A=0,l=0,c=[],u=[];for(n=0;n<i;++n)r=s[n],A<l?(A+=a[r],c.push(r)):(l+=a[r],u.push(r));return u.reverse().concat(c)},reverse:function(e){return t.range(e.length).reverse()},default:Go}),Oo=t.map({silhouette:function(t){var e,n,r,i=t.length,o=t[0].length,a=[],s=0,A=[];for(n=0;n<o;++n){for(e=0,r=0;e<i;e++)r+=t[e][n][1];r>s&&(s=r),a.push(r)}for(n=0;n<o;++n)A[n]=(s-a[n])/2;return A},wiggle:function(t){var e,n,r,i,o,a,s,A,l,c=t.length,u=t[0],g=u.length,h=[];for(h[0]=A=l=0,n=1;n<g;++n){for(e=0,i=0;e<c;++e)i+=t[e][n][1];for(e=0,o=0,s=u[n][0]-u[n-1][0];e<c;++e){for(r=0,a=(t[e][n][1]-t[e][n-1][1])/(2*s);r<e;++r)a+=(t[r][n][1]-t[r][n-1][1])/s;o+=a*t[e][n][1]}h[n]=A-=i?o/i*s:0,A<l&&(l=A)}for(n=0;n<g;++n)h[n]-=l;return h},expand:function(t){var e,n,r,i=t.length,o=t[0].length,a=1/i,s=[];for(n=0;n<o;++n){for(e=0,r=0;e<i;e++)r+=t[e][n][1];if(r)for(e=0;e<i;e++)t[e][n][1]/=r;else for(e=0;e<i;e++)t[e][n][1]=a}for(n=0;n<o;++n)s[n]=0;return s},zero:Ho});function Go(e){return t.range(e.length)}function Ho(t){for(var e=-1,n=t[0].length,r=[];++e<n;)r[e]=0;return r}function jo(t){for(var e,n=1,r=0,i=t[0][1],o=t.length;n<o;++n)(e=t[n][1])>i&&(r=n,i=e);return r}function Yo(t){return t.reduce(Po,0)}function Po(t,e){return t+e[1]}function Ko(t,e){return zo(t,Math.ceil(Math.log(e.length)/Math.LN2+1))}function zo(t,e){for(var n=-1,r=+t[0],i=(t[1]-r)/e,o=[];++n<=e;)o[n]=i*n+r;return o}function Jo(e){return[t.min(e),t.max(e)]}function qo(t,e){return t.value-e.value}function Vo(t,e){var n=t._pack_next;t._pack_next=e,e._pack_prev=t,e._pack_next=n,n._pack_prev=e}function Wo(t,e){t._pack_next=e,e._pack_prev=t}function Xo(t,e){var n=e.x-t.x,r=e.y-t.y,i=t.r+e.r;return.999*i*i>n*n+r*r}function Zo(t){if((e=t.children)&&(A=e.length)){var e,n,r,i,o,a,s,A,l=1/0,c=-1/0,u=1/0,g=-1/0;if(e.forEach($o),(n=e[0]).x=-n.r,n.y=0,m(n),A>1&&((r=e[1]).x=r.r,r.y=0,m(r),A>2))for(ea(n,r,i=e[2]),m(i),Vo(n,i),n._pack_prev=i,Vo(i,r),r=n._pack_next,o=3;o<A;o++){ea(n,r,i=e[o]);var h=0,d=1,f=1;for(a=r._pack_next;a!==r;a=a._pack_next,d++)if(Xo(a,i)){h=1;break}if(1==h)for(s=n._pack_prev;s!==a._pack_prev&&!Xo(s,i);s=s._pack_prev,f++);h?(d<f||d==f&&r.r<n.r?Wo(n,r=a):Wo(n=s,r),o--):(Vo(n,i),r=i,m(i))}var p=(l+c)/2,I=(u+g)/2,C=0;for(o=0;o<A;o++)(i=e[o]).x-=p,i.y-=I,C=Math.max(C,i.r+Math.sqrt(i.x*i.x+i.y*i.y));t.r=C,e.forEach(ta)}function m(t){l=Math.min(t.x-t.r,l),c=Math.max(t.x+t.r,c),u=Math.min(t.y-t.r,u),g=Math.max(t.y+t.r,g)}}function $o(t){t._pack_next=t._pack_prev=t}function ta(t){delete t._pack_next,delete t._pack_prev}function ea(t,e,n){var r=t.r+n.r,i=e.x-t.x,o=e.y-t.y;if(r&&(i||o)){var a=e.r+n.r,s=i*i+o*o,A=.5+((r*=r)-(a*=a))/(2*s),l=Math.sqrt(Math.max(0,2*a*(r+s)-(r-=s)*r-a*a))/(2*s);n.x=t.x+A*i+l*o,n.y=t.y+A*o-l*i}else n.x=t.x+r,n.y=t.y}function na(t,e){return t.parent==e.parent?1:2}function ra(t){var e=t.children;return e.length?e[0]:t.t}function ia(t){var e,n=t.children;return(e=n.length)?n[e-1]:t.t}function oa(t,e,n){var r=n/(e.i-t.i);e.c-=r,e.s+=n,t.c+=r,e.z+=n,e.m+=n}function aa(t,e,n){return t.a.parent===e.parent?t.a:n}function sa(t){return{x:t.x,y:t.y,dx:t.dx,dy:t.dy}}function Aa(t,e){var n=t.x+e[3],r=t.y+e[0],i=t.dx-e[1]-e[3],o=t.dy-e[0]-e[2];return i<0&&(n+=i/2,i=0),o<0&&(r+=o/2,o=0),{x:n,y:r,dx:i,dy:o}}function la(t){var e=t[0],n=t[t.length-1];return e<n?[e,n]:[n,e]}function ca(t){return t.rangeExtent?t.rangeExtent():la(t.range())}function ua(t,e,n,r){var i=n(t[0],t[1]),o=r(e[0],e[1]);return function(t){return o(i(t))}}function ga(t,e){var n,r=0,i=t.length-1,o=t[r],a=t[i];return a<o&&(n=r,r=i,i=n,n=o,o=a,a=n),t[r]=e.floor(o),t[i]=e.ceil(a),t}function ha(t){return t?{floor:function(e){return Math.floor(e/t)*t},ceil:function(e){return Math.ceil(e/t)*t}}:da}t.layout.histogram=function(){var e=!0,n=Number,r=Jo,i=Ko;function o(o,a){for(var s,A,l=[],c=o.map(n,this),u=r.call(this,c,a),g=i.call(this,u,c,a),h=(a=-1,c.length),d=g.length-1,f=e?1:1/h;++a<d;)(s=l[a]=[]).dx=g[a+1]-(s.x=g[a]),s.y=0;if(d>0)for(a=-1;++a<h;)(A=c[a])>=u[0]&&A<=u[1]&&((s=l[t.bisect(g,A,1,d)-1]).y+=f,s.push(o[a]));return l}return o.value=function(t){return arguments.length?(n=t,o):n},o.range=function(t){return arguments.length?(r=fe(t),o):r},o.bins=function(t){return arguments.length?(i="number"==typeof t?function(e){return zo(e,t)}:fe(t),o):i},o.frequency=function(t){return arguments.length?(e=!!t,o):e},o},t.layout.pack=function(){var e,n=t.layout.hierarchy().sort(qo),r=0,i=[1,1];function o(t,o){var a=n.call(this,t,o),s=a[0],A=i[0],l=i[1],c=null==e?Math.sqrt:"function"==typeof e?e:function(){return e};if(s.x=s.y=0,Do(s,function(t){t.r=+c(t.value)}),Do(s,Zo),r){var u=r*(e?1:Math.max(2*s.r/A,2*s.r/l))/2;Do(s,function(t){t.r+=u}),Do(s,Zo),Do(s,function(t){t.r-=u})}return function t(e,n,r,i){var o=e.children;if(e.x=n+=i*e.x,e.y=r+=i*e.y,e.r*=i,o)for(var a=-1,s=o.length;++a<s;)t(o[a],n,r,i)}(s,A/2,l/2,e?1:1/Math.max(2*s.r/A,2*s.r/l)),a}return o.size=function(t){return arguments.length?(i=t,o):i},o.radius=function(t){return arguments.length?(e=null==t||"function"==typeof t?t:+t,o):e},o.padding=function(t){return arguments.length?(r=+t,o):r},xo(o,n)},t.layout.tree=function(){var e=t.layout.hierarchy().sort(null).value(null),n=na,r=[1,1],i=null;function o(t,o){var l=e.call(this,t,o),c=l[0],u=function(t){for(var e,n={A:null,children:[t]},r=[n];null!=(e=r.pop());)for(var i,o=e.children,a=0,s=o.length;a<s;++a)r.push((o[a]=i={_:o[a],parent:e,children:(i=o[a].children)&&i.slice()||[],A:null,a:null,z:0,m:0,c:0,s:0,t:null,i:a}).a=i);return n.children[0]}(c);if(Do(u,a),u.parent.m=-u.z,_o(u,s),i)_o(c,A);else{var g=c,h=c,d=c;_o(c,function(t){t.x<g.x&&(g=t),t.x>h.x&&(h=t),t.depth>d.depth&&(d=t)});var f=n(g,h)/2-g.x,p=r[0]/(h.x+n(h,g)/2+f),I=r[1]/(d.depth||1);_o(c,function(t){t.x=(t.x+f)*p,t.y=t.depth*I})}return l}function a(t){var e=t.children,r=t.parent.children,i=t.i?r[t.i-1]:null;if(e.length){!function(t){for(var e,n=0,r=0,i=t.children,o=i.length;--o>=0;)(e=i[o]).z+=n,e.m+=n,n+=e.s+(r+=e.c)}(t);var o=(e[0].z+e[e.length-1].z)/2;i?(t.z=i.z+n(t._,i._),t.m=t.z-o):t.z=o}else i&&(t.z=i.z+n(t._,i._));t.parent.A=function(t,e,r){if(e){for(var i,o=t,a=t,s=e,A=o.parent.children[0],l=o.m,c=a.m,u=s.m,g=A.m;s=ia(s),o=ra(o),s&&o;)A=ra(A),(a=ia(a)).a=t,(i=s.z+u-o.z-l+n(s._,o._))>0&&(oa(aa(s,t,r),t,i),l+=i,c+=i),u+=s.m,l+=o.m,g+=A.m,c+=a.m;s&&!ia(a)&&(a.t=s,a.m+=u-c),o&&!ra(A)&&(A.t=o,A.m+=l-g,r=t)}return r}(t,i,t.parent.A||r[0])}function s(t){t._.x=t.z+t.parent.m,t.m+=t.parent.m}function A(t){t.x*=r[0],t.y=t.depth*r[1]}return o.separation=function(t){return arguments.length?(n=t,o):n},o.size=function(t){return arguments.length?(i=null==(r=t)?A:null,o):i?null:r},o.nodeSize=function(t){return arguments.length?(i=null==(r=t)?null:A,o):i?r:null},xo(o,e)},t.layout.cluster=function(){var e=t.layout.hierarchy().sort(null).value(null),n=na,r=[1,1],i=!1;function o(o,a){var s,A=e.call(this,o,a),l=A[0],c=0;Do(l,function(e){var r=e.children;r&&r.length?(e.x=function(t){return t.reduce(function(t,e){return t+e.x},0)/t.length}(r),e.y=function(e){return 1+t.max(e,function(t){return t.y})}(r)):(e.x=s?c+=n(e,s):0,e.y=0,s=e)});var u=function t(e){var n=e.children;return n&&n.length?t(n[0]):e}(l),g=function t(e){var n,r=e.children;return r&&(n=r.length)?t(r[n-1]):e}(l),h=u.x-n(u,g)/2,d=g.x+n(g,u)/2;return Do(l,i?function(t){t.x=(t.x-l.x)*r[0],t.y=(l.y-t.y)*r[1]}:function(t){t.x=(t.x-h)/(d-h)*r[0],t.y=(1-(l.y?t.y/l.y:1))*r[1]}),A}return o.separation=function(t){return arguments.length?(n=t,o):n},o.size=function(t){return arguments.length?(i=null==(r=t),o):i?null:r},o.nodeSize=function(t){return arguments.length?(i=null!=(r=t),o):i?r:null},xo(o,e)},t.layout.treemap=function(){var e,n=t.layout.hierarchy(),r=Math.round,i=[1,1],o=null,a=sa,s=!1,A="squarify",l=.5*(1+Math.sqrt(5));function c(t,e){for(var n,r,i=-1,o=t.length;++i<o;)r=(n=t[i]).value*(e<0?0:e),n.area=isNaN(r)||r<=0?0:r}function u(t){var e=t.children;if(e&&e.length){var n,r,i,o=a(t),s=[],l=e.slice(),g=1/0,f="slice"===A?o.dx:"dice"===A?o.dy:"slice-dice"===A?1&t.depth?o.dy:o.dx:Math.min(o.dx,o.dy);for(c(l,o.dx*o.dy/t.value),s.area=0;(i=l.length)>0;)s.push(n=l[i-1]),s.area+=n.area,"squarify"!==A||(r=h(s,f))<=g?(l.pop(),g=r):(s.area-=s.pop().area,d(s,f,o,!1),f=Math.min(o.dx,o.dy),s.length=s.area=0,g=1/0);s.length&&(d(s,f,o,!0),s.length=s.area=0),e.forEach(u)}}function g(t){var e=t.children;if(e&&e.length){var n,r=a(t),i=e.slice(),o=[];for(c(i,r.dx*r.dy/t.value),o.area=0;n=i.pop();)o.push(n),o.area+=n.area,null!=n.z&&(d(o,n.z?r.dx:r.dy,r,!i.length),o.length=o.area=0);e.forEach(g)}}function h(t,e){for(var n,r=t.area,i=0,o=1/0,a=-1,s=t.length;++a<s;)(n=t[a].area)&&(n<o&&(o=n),n>i&&(i=n));return e*=e,(r*=r)?Math.max(e*i*l/r,r/(e*o*l)):1/0}function d(t,e,n,i){var o,a=-1,s=t.length,A=n.x,l=n.y,c=e?r(t.area/e):0;if(e==n.dx){for((i||c>n.dy)&&(c=n.dy);++a<s;)(o=t[a]).x=A,o.y=l,o.dy=c,A+=o.dx=Math.min(n.x+n.dx-A,c?r(o.area/c):0);o.z=!0,o.dx+=n.x+n.dx-A,n.y+=c,n.dy-=c}else{for((i||c>n.dx)&&(c=n.dx);++a<s;)(o=t[a]).x=A,o.y=l,o.dx=c,l+=o.dy=Math.min(n.y+n.dy-l,c?r(o.area/c):0);o.z=!1,o.dy+=n.y+n.dy-l,n.x+=c,n.dx-=c}}function f(t){var r=e||n(t),o=r[0];return o.x=o.y=0,o.value?(o.dx=i[0],o.dy=i[1]):o.dx=o.dy=0,e&&n.revalue(o),c([o],o.dx*o.dy/o.value),(e?g:u)(o),s&&(e=r),r}return f.size=function(t){return arguments.length?(i=t,f):i},f.padding=function(t){if(!arguments.length)return o;function e(e){return Aa(e,t)}var n;return a=null==(o=t)?sa:"function"==(n=typeof t)?function(e){var n=t.call(f,e,e.depth);return null==n?sa(e):Aa(e,"number"==typeof n?[n,n,n,n]:n)}:"number"===n?(t=[t,t,t,t],e):e,f},f.round=function(t){return arguments.length?(r=t?Math.round:Number,f):r!=Number},f.sticky=function(t){return arguments.length?(s=t,e=null,f):s},f.ratio=function(t){return arguments.length?(l=t,f):l},f.mode=function(t){return arguments.length?(A=t+"",f):A},xo(f,n)},t.random={normal:function(t,e){var n=arguments.length;return n<2&&(e=1),n<1&&(t=0),function(){var n,r,i;do{i=(n=2*Math.random()-1)*n+(r=2*Math.random()-1)*r}while(!i||i>1);return t+e*n*Math.sqrt(-2*Math.log(i)/i)}},logNormal:function(){var e=t.random.normal.apply(t,arguments);return function(){return Math.exp(e())}},bates:function(e){var n=t.random.irwinHall(e);return function(){return n()/e}},irwinHall:function(t){return function(){for(var e=0,n=0;n<t;n++)e+=Math.random();return e}}},t.scale={};var da={floor:k,ceil:k};function fa(e,n,r,i){var o=[],a=[],s=0,A=Math.min(e.length,n.length)-1;for(e[A]<e[0]&&(e=e.slice().reverse(),n=n.slice().reverse());++s<=A;)o.push(r(e[s-1],e[s])),a.push(i(n[s-1],n[s]));return function(n){var r=t.bisect(e,n,1,A)-1;return a[r](o[r](n))}}function pa(e,n){return t.rebind(e,n,"range","rangeRound","interpolate","clamp")}function Ia(t,e){return ga(t,ha(Ca(t,e)[2])),ga(t,ha(Ca(t,e)[2])),t}function Ca(t,e){null==e&&(e=10);var n=la(t),r=n[1]-n[0],i=Math.pow(10,Math.floor(Math.log(r/e)/Math.LN10)),o=e/r*i;return o<=.15?i*=10:o<=.35?i*=5:o<=.75&&(i*=2),n[0]=Math.ceil(n[0]/i)*i,n[1]=Math.floor(n[1]/i)*i+.5*i,n[2]=i,n}function ma(e,n){return t.range.apply(t,Ca(e,n))}function ya(e,n,r){var i=Ca(e,n);if(r){var o=De.exec(r);if(o.shift(),"s"===o[8]){var a=t.formatPrefix(Math.max(C(i[0]),C(i[1])));return o[7]||(o[7]="."+va(a.scale(i[2]))),o[8]="f",r=t.format(o.join("")),function(t){return r(a.scale(t))+a.symbol}}o[7]||(o[7]="."+function(t,e){var n=va(e[2]);return t in Ea?Math.abs(n-va(Math.max(C(e[0]),C(e[1]))))+ +("e"!==t):n-2*("%"===t)}(o[8],i)),r=o.join("")}else r=",."+va(i[2])+"f";return t.format(r)}t.scale.linear=function(){return function t(e,n,r,i){var o,a;function s(){var t=Math.min(e.length,n.length)>2?fa:ua,s=i?Io:po;return o=t(e,n,s,r),a=t(n,e,s,qi),A}function A(t){return o(t)}return A.invert=function(t){return a(t)},A.domain=function(t){return arguments.length?(e=t.map(Number),s()):e},A.range=function(t){return arguments.length?(n=t,s()):n},A.rangeRound=function(t){return A.range(t).interpolate(Ao)},A.clamp=function(t){return arguments.length?(i=t,s()):i},A.interpolate=function(t){return arguments.length?(r=t,s()):r},A.ticks=function(t){return ma(e,t)},A.tickFormat=function(t,n){return ya(e,t,n)},A.nice=function(t){return Ia(e,t),s()},A.copy=function(){return t(e,n,r,i)},s()}([0,1],[0,1],qi,!1)};var Ea={s:1,g:1,p:1,r:1,e:1};function va(t){return-Math.floor(Math.log(t)/Math.LN10+.01)}t.scale.log=function(){return function e(n,r,i,o){function a(t){return(i?Math.log(t<0?0:t):-Math.log(t>0?0:-t))/Math.log(r)}function s(t){return i?Math.pow(r,t):-Math.pow(r,-t)}function A(t){return n(a(t))}return A.invert=function(t){return s(n.invert(t))},A.domain=function(t){return arguments.length?(i=t[0]>=0,n.domain((o=t.map(Number)).map(a)),A):o},A.base=function(t){return arguments.length?(r=+t,n.domain(o.map(a)),A):r},A.nice=function(){var t=ga(o.map(a),i?Math:ba);return n.domain(t),o=t.map(s),A},A.ticks=function(){var t=la(o),e=[],n=t[0],A=t[1],l=Math.floor(a(n)),c=Math.ceil(a(A)),u=r%1?2:r;if(isFinite(c-l)){if(i){for(;l<c;l++)for(var g=1;g<u;g++)e.push(s(l)*g);e.push(s(l))}else for(e.push(s(l));l++<c;)for(var g=u-1;g>0;g--)e.push(s(l)*g);for(l=0;e[l]<n;l++);for(c=e.length;e[c-1]>A;c--);e=e.slice(l,c)}return e},A.tickFormat=function(e,n){if(!arguments.length)return Ba;arguments.length<2?n=Ba:"function"!=typeof n&&(n=t.format(n));var i=Math.max(1,r*e/A.ticks().length);return function(t){var e=t/s(Math.round(a(t)));return e*r<r-.5&&(e*=r),e<=i?n(t):""}},A.copy=function(){return e(n.copy(),r,i,o)},pa(A,n)}(t.scale.linear().domain([0,1]),10,!0,[1,10])};var Ba=t.format(".0e"),ba={floor:function(t){return-Math.ceil(-t)},ceil:function(t){return-Math.floor(-t)}};function Qa(t){return function(e){return e<0?-Math.pow(-e,t):Math.pow(e,t)}}t.scale.pow=function(){return function t(e,n,r){var i=Qa(n),o=Qa(1/n);function a(t){return e(i(t))}return a.invert=function(t){return o(e.invert(t))},a.domain=function(t){return arguments.length?(e.domain((r=t.map(Number)).map(i)),a):r},a.ticks=function(t){return ma(r,t)},a.tickFormat=function(t,e){return ya(r,t,e)},a.nice=function(t){return a.domain(Ia(r,t))},a.exponent=function(t){return arguments.length?(i=Qa(n=t),o=Qa(1/n),e.domain(r.map(i)),a):n},a.copy=function(){return t(e.copy(),n,r)},pa(a,e)}(t.scale.linear(),1,[0,1])},t.scale.sqrt=function(){return t.scale.pow().exponent(.5)},t.scale.ordinal=function(){return function e(n,r){var i,o,a;function s(t){return o[((i.get(t)||("range"===r.t?i.set(t,n.push(t)):NaN))-1)%o.length]}function A(e,r){return t.range(n.length).map(function(t){return e+r*t})}return s.domain=function(t){if(!arguments.length)return n;n=[],i=new y;for(var e,o=-1,a=t.length;++o<a;)i.has(e=t[o])||i.set(e,n.push(e));return s[r.t].apply(s,r.a)},s.range=function(t){return arguments.length?(o=t,a=0,r={t:"range",a:arguments},s):o},s.rangePoints=function(t,e){arguments.length<2&&(e=0);var i=t[0],l=t[1],c=n.length<2?(i=(i+l)/2,0):(l-i)/(n.length-1+e);return o=A(i+c*e/2,c),a=0,r={t:"rangePoints",a:arguments},s},s.rangeRoundPoints=function(t,e){arguments.length<2&&(e=0);var i=t[0],l=t[1],c=n.length<2?(i=l=Math.round((i+l)/2),0):(l-i)/(n.length-1+e)|0;return o=A(i+Math.round(c*e/2+(l-i-(n.length-1+e)*c)/2),c),a=0,r={t:"rangeRoundPoints",a:arguments},s},s.rangeBands=function(t,e,i){arguments.length<2&&(e=0),arguments.length<3&&(i=e);var l=t[1]<t[0],c=t[l-0],u=t[1-l],g=(u-c)/(n.length-e+2*i);return o=A(c+g*i,g),l&&o.reverse(),a=g*(1-e),r={t:"rangeBands",a:arguments},s},s.rangeRoundBands=function(t,e,i){arguments.length<2&&(e=0),arguments.length<3&&(i=e);var l=t[1]<t[0],c=t[l-0],u=t[1-l],g=Math.floor((u-c)/(n.length-e+2*i));return o=A(c+Math.round((u-c-(n.length-e)*g)/2),g),l&&o.reverse(),a=Math.round(g*(1-e)),r={t:"rangeRoundBands",a:arguments},s},s.rangeBand=function(){return a},s.rangeExtent=function(){return la(r.a[0])},s.copy=function(){return e(n,r)},s.domain(n)}([],{t:"range",a:[[]]})},t.scale.category10=function(){return t.scale.ordinal().range(wa)},t.scale.category20=function(){return t.scale.ordinal().range(xa)},t.scale.category20b=function(){return t.scale.ordinal().range(_a)},t.scale.category20c=function(){return t.scale.ordinal().range(Da)};var wa=[2062260,16744206,2924588,14034728,9725885,9197131,14907330,8355711,12369186,1556175].map(ae),xa=[2062260,11454440,16744206,16759672,2924588,10018698,14034728,16750742,9725885,12955861,9197131,12885140,14907330,16234194,8355711,13092807,12369186,14408589,1556175,10410725].map(ae),_a=[3750777,5395619,7040719,10264286,6519097,9216594,11915115,13556636,9202993,12426809,15186514,15190932,8666169,11356490,14049643,15177372,8077683,10834324,13528509,14589654].map(ae),Da=[3244733,7057110,10406625,13032431,15095053,16616764,16625259,16634018,3253076,7652470,10607003,13101504,7695281,10394312,12369372,14342891,6513507,9868950,12434877,14277081].map(ae);function Ma(){return 0}t.scale.quantile=function(){return function e(n,r){var i;function o(){var e=0,o=r.length;for(i=[];++e<o;)i[e-1]=t.quantile(n,e/o);return a}function a(e){if(!isNaN(e=+e))return r[t.bisect(i,e)]}return a.domain=function(t){return arguments.length?(n=t.map(h).filter(d).sort(g),o()):n},a.range=function(t){return arguments.length?(r=t,o()):r},a.quantiles=function(){return i},a.invertExtent=function(t){return(t=r.indexOf(t))<0?[NaN,NaN]:[t>0?i[t-1]:n[0],t<i.length?i[t]:n[n.length-1]]},a.copy=function(){return e(n,r)},o()}([],[])},t.scale.quantize=function(){return function t(e,n,r){var i,o;function a(t){return r[Math.max(0,Math.min(o,Math.floor(i*(t-e))))]}function s(){return i=r.length/(n-e),o=r.length-1,a}return a.domain=function(t){return arguments.length?(e=+t[0],n=+t[t.length-1],s()):[e,n]},a.range=function(t){return arguments.length?(r=t,s()):r},a.invertExtent=function(t){return[t=(t=r.indexOf(t))<0?NaN:t/i+e,t+1/i]},a.copy=function(){return t(e,n,r)},s()}(0,1,[0,1])},t.scale.threshold=function(){return function e(n,r){function i(e){if(e<=e)return r[t.bisect(n,e)]}return i.domain=function(t){return arguments.length?(n=t,i):n},i.range=function(t){return arguments.length?(r=t,i):r},i.invertExtent=function(t){return t=r.indexOf(t),[n[t-1],n[t]]},i.copy=function(){return e(n,r)},i}([.5],[0,1])},t.scale.identity=function(){return function t(e){function n(t){return+t}return n.invert=n,n.domain=n.range=function(t){return arguments.length?(e=t.map(n),n):e},n.ticks=function(t){return ma(e,t)},n.tickFormat=function(t,n){return ya(e,t,n)},n.copy=function(){return t(e)},n}([0,1])},t.svg={},t.svg.arc=function(){var t=Sa,e=La,n=Ma,r=ka,i=Ra,o=Na,a=Fa;function s(){var s=Math.max(0,+t.apply(this,arguments)),l=Math.max(0,+e.apply(this,arguments)),c=i.apply(this,arguments)-_t,u=o.apply(this,arguments)-_t,g=Math.abs(u-c),h=c>u?0:1;if(l<s&&(d=l,l=s,s=d),g>=xt)return A(l,h)+(s?A(s,1-h):"")+"Z";var d,f,p,I,C,m,y,E,v,B,b,Q,w=0,x=0,_=[];if((I=(+a.apply(this,arguments)||0)/2)&&(p=r===ka?Math.sqrt(s*s+l*l):+r.apply(this,arguments),h||(x*=-1),l&&(x=Rt(p/l*Math.sin(I))),s&&(w=Rt(p/s*Math.sin(I)))),l){C=l*Math.cos(c+x),m=l*Math.sin(c+x),y=l*Math.cos(u-x),E=l*Math.sin(u-x);var D=Math.abs(u-c-2*x)<=Qt?0:1;if(x&&Ta(C,m,y,E)===h^D){var M=(c+u)/2;C=l*Math.cos(M),m=l*Math.sin(M),y=E=null}}else C=m=0;if(s){v=s*Math.cos(u-w),B=s*Math.sin(u-w),b=s*Math.cos(c+w),Q=s*Math.sin(c+w);var k=Math.abs(c-u+2*w)<=Qt?0:1;if(w&&Ta(v,B,b,Q)===1-h^k){var S=(c+u)/2;v=s*Math.cos(S),B=s*Math.sin(S),b=Q=null}}else v=B=0;if(g>Bt&&(d=Math.min(Math.abs(l-s)/2,+n.apply(this,arguments)))>.001){f=s<l^h?0:1;var L=d,R=d;if(g<Qt){var N=null==b?[v,B]:null==y?[C,m]:ai([C,m],[b,Q],[y,E],[v,B]),F=C-N[0],T=m-N[1],U=y-N[0],O=E-N[1],G=1/Math.sin(Math.acos((F*U+T*O)/(Math.sqrt(F*F+T*T)*Math.sqrt(U*U+O*O)))/2),H=Math.sqrt(N[0]*N[0]+N[1]*N[1]);R=Math.min(d,(s-H)/(G-1)),L=Math.min(d,(l-H)/(G+1))}if(null!=y){var j=Ua(null==b?[v,B]:[b,Q],[C,m],l,L,h),Y=Ua([y,E],[v,B],l,L,h);d===L?_.push("M",j[0],"A",L,",",L," 0 0,",f," ",j[1],"A",l,",",l," 0 ",1-h^Ta(j[1][0],j[1][1],Y[1][0],Y[1][1]),",",h," ",Y[1],"A",L,",",L," 0 0,",f," ",Y[0]):_.push("M",j[0],"A",L,",",L," 0 1,",f," ",Y[0])}else _.push("M",C,",",m);if(null!=b){var P=Ua([C,m],[b,Q],s,-R,h),K=Ua([v,B],null==y?[C,m]:[y,E],s,-R,h);d===R?_.push("L",K[0],"A",R,",",R," 0 0,",f," ",K[1],"A",s,",",s," 0 ",h^Ta(K[1][0],K[1][1],P[1][0],P[1][1]),",",1-h," ",P[1],"A",R,",",R," 0 0,",f," ",P[0]):_.push("L",K[0],"A",R,",",R," 0 0,",f," ",P[0])}else _.push("L",v,",",B)}else _.push("M",C,",",m),null!=y&&_.push("A",l,",",l," 0 ",D,",",h," ",y,",",E),_.push("L",v,",",B),null!=b&&_.push("A",s,",",s," 0 ",k,",",1-h," ",b,",",Q);return _.push("Z"),_.join("")}function A(t,e){return"M0,"+t+"A"+t+","+t+" 0 1,"+e+" 0,"+-t+"A"+t+","+t+" 0 1,"+e+" 0,"+t}return s.innerRadius=function(e){return arguments.length?(t=fe(e),s):t},s.outerRadius=function(t){return arguments.length?(e=fe(t),s):e},s.cornerRadius=function(t){return arguments.length?(n=fe(t),s):n},s.padRadius=function(t){return arguments.length?(r=t==ka?ka:fe(t),s):r},s.startAngle=function(t){return arguments.length?(i=fe(t),s):i},s.endAngle=function(t){return arguments.length?(o=fe(t),s):o},s.padAngle=function(t){return arguments.length?(a=fe(t),s):a},s.centroid=function(){var n=(+t.apply(this,arguments)+ +e.apply(this,arguments))/2,r=(+i.apply(this,arguments)+ +o.apply(this,arguments))/2-_t;return[Math.cos(r)*n,Math.sin(r)*n]},s};var ka="auto";function Sa(t){return t.innerRadius}function La(t){return t.outerRadius}function Ra(t){return t.startAngle}function Na(t){return t.endAngle}function Fa(t){return t&&t.padAngle}function Ta(t,e,n,r){return(t-n)*e-(e-r)*t>0?0:1}function Ua(t,e,n,r,i){var o=t[0]-e[0],a=t[1]-e[1],s=(i?r:-r)/Math.sqrt(o*o+a*a),A=s*a,l=-s*o,c=t[0]+A,u=t[1]+l,g=e[0]+A,h=e[1]+l,d=(c+g)/2,f=(u+h)/2,p=g-c,I=h-u,C=p*p+I*I,m=n-r,y=c*h-g*u,E=(I<0?-1:1)*Math.sqrt(Math.max(0,m*m*C-y*y)),v=(y*I-p*E)/C,B=(-y*p-I*E)/C,b=(y*I+p*E)/C,Q=(-y*p+I*E)/C,w=v-d,x=B-f,_=b-d,D=Q-f;return w*w+x*x>_*_+D*D&&(v=b,B=Q),[[v-A,B-l],[v*n/m,B*n/m]]}function Oa(t){var e=ti,n=ei,r=zn,i=Ha,o=i.key,a=.7;function s(o){var s,A=[],l=[],c=-1,u=o.length,g=fe(e),h=fe(n);function d(){A.push("M",i(t(l),a))}for(;++c<u;)r.call(this,s=o[c],c)?l.push([+g.call(this,s,c),+h.call(this,s,c)]):l.length&&(d(),l=[]);return l.length&&d(),A.length?A.join(""):null}return s.x=function(t){return arguments.length?(e=t,s):e},s.y=function(t){return arguments.length?(n=t,s):n},s.defined=function(t){return arguments.length?(r=t,s):r},s.interpolate=function(t){return arguments.length?(o="function"==typeof t?i=t:(i=Ga.get(t)||Ha).key,s):o},s.tension=function(t){return arguments.length?(a=t,s):a},s}t.svg.line=function(){return Oa(k)};var Ga=t.map({linear:Ha,"linear-closed":ja,step:function(t){for(var e=0,n=t.length,r=t[0],i=[r[0],",",r[1]];++e<n;)i.push("H",(r[0]+(r=t[e])[0])/2,"V",r[1]);return n>1&&i.push("H",r[0]),i.join("")},"step-before":Ya,"step-after":Pa,basis:Ja,"basis-open":function(t){if(t.length<4)return Ha(t);for(var e,n=[],r=-1,i=t.length,o=[0],a=[0];++r<3;)e=t[r],o.push(e[0]),a.push(e[1]);for(n.push(qa(Xa,o)+","+qa(Xa,a)),--r;++r<i;)e=t[r],o.shift(),o.push(e[0]),a.shift(),a.push(e[1]),Za(n,o,a);return n.join("")},"basis-closed":function(t){for(var e,n,r=-1,i=t.length,o=i+4,a=[],s=[];++r<4;)n=t[r%i],a.push(n[0]),s.push(n[1]);for(e=[qa(Xa,a),",",qa(Xa,s)],--r;++r<o;)n=t[r%i],a.shift(),a.push(n[0]),s.shift(),s.push(n[1]),Za(e,a,s);return e.join("")},bundle:function(t,e){var n=t.length-1;if(n)for(var r,i,o=t[0][0],a=t[0][1],s=t[n][0]-o,A=t[n][1]-a,l=-1;++l<=n;)r=t[l],i=l/n,r[0]=e*r[0]+(1-e)*(o+i*s),r[1]=e*r[1]+(1-e)*(a+i*A);return Ja(t)},cardinal:function(t,e){return t.length<3?Ha(t):t[0]+Ka(t,za(t,e))},"cardinal-open":function(t,e){return t.length<4?Ha(t):t[1]+Ka(t.slice(1,-1),za(t,e))},"cardinal-closed":function(t,e){return t.length<3?ja(t):t[0]+Ka((t.push(t[0]),t),za([t[t.length-2]].concat(t,[t[1]]),e))},monotone:function(t){return t.length<3?Ha(t):t[0]+Ka(t,function(t){for(var e,n,r,i,o=[],a=function(t){for(var e=0,n=t.length-1,r=[],i=t[0],o=t[1],a=r[0]=$a(i,o);++e<n;)r[e]=(a+(a=$a(i=o,o=t[e+1])))/2;return r[e]=a,r}(t),s=-1,A=t.length-1;++s<A;)e=$a(t[s],t[s+1]),C(e)<Bt?a[s]=a[s+1]=0:(n=a[s]/e,r=a[s+1]/e,(i=n*n+r*r)>9&&(i=3*e/Math.sqrt(i),a[s]=i*n,a[s+1]=i*r));for(s=-1;++s<=A;)i=(t[Math.min(A,s+1)][0]-t[Math.max(0,s-1)][0])/(6*(1+a[s]*a[s])),o.push([i||0,a[s]*i||0]);return o}(t))}});function Ha(t){return t.length>1?t.join("L"):t+"Z"}function ja(t){return t.join("L")+"Z"}function Ya(t){for(var e=0,n=t.length,r=t[0],i=[r[0],",",r[1]];++e<n;)i.push("V",(r=t[e])[1],"H",r[0]);return i.join("")}function Pa(t){for(var e=0,n=t.length,r=t[0],i=[r[0],",",r[1]];++e<n;)i.push("H",(r=t[e])[0],"V",r[1]);return i.join("")}function Ka(t,e){if(e.length<1||t.length!=e.length&&t.length!=e.length+2)return Ha(t);var n=t.length!=e.length,r="",i=t[0],o=t[1],a=e[0],s=a,A=1;if(n&&(r+="Q"+(o[0]-2*a[0]/3)+","+(o[1]-2*a[1]/3)+","+o[0]+","+o[1],i=t[1],A=2),e.length>1){s=e[1],o=t[A],A++,r+="C"+(i[0]+a[0])+","+(i[1]+a[1])+","+(o[0]-s[0])+","+(o[1]-s[1])+","+o[0]+","+o[1];for(var l=2;l<e.length;l++,A++)o=t[A],s=e[l],r+="S"+(o[0]-s[0])+","+(o[1]-s[1])+","+o[0]+","+o[1]}if(n){var c=t[A];r+="Q"+(o[0]+2*s[0]/3)+","+(o[1]+2*s[1]/3)+","+c[0]+","+c[1]}return r}function za(t,e){for(var n,r=[],i=(1-e)/2,o=t[0],a=t[1],s=1,A=t.length;++s<A;)n=o,o=a,a=t[s],r.push([i*(a[0]-n[0]),i*(a[1]-n[1])]);return r}function Ja(t){if(t.length<3)return Ha(t);var e=1,n=t.length,r=t[0],i=r[0],o=r[1],a=[i,i,i,(r=t[1])[0]],s=[o,o,o,r[1]],A=[i,",",o,"L",qa(Xa,a),",",qa(Xa,s)];for(t.push(t[n-1]);++e<=n;)r=t[e],a.shift(),a.push(r[0]),s.shift(),s.push(r[1]),Za(A,a,s);return t.pop(),A.push("L",r),A.join("")}function qa(t,e){return t[0]*e[0]+t[1]*e[1]+t[2]*e[2]+t[3]*e[3]}Ga.forEach(function(t,e){e.key=t,e.closed=/-closed$/.test(t)});var Va=[0,2/3,1/3,0],Wa=[0,1/3,2/3,0],Xa=[0,1/6,2/3,1/6];function Za(t,e,n){t.push("C",qa(Va,e),",",qa(Va,n),",",qa(Wa,e),",",qa(Wa,n),",",qa(Xa,e),",",qa(Xa,n))}function $a(t,e){return(e[1]-t[1])/(e[0]-t[0])}function ts(t){for(var e,n,r,i=-1,o=t.length;++i<o;)n=(e=t[i])[0],r=e[1]-_t,e[0]=n*Math.cos(r),e[1]=n*Math.sin(r);return t}function es(t){var e=ti,n=ti,r=0,i=ei,o=zn,a=Ha,s=a.key,A=a,l="L",c=.7;function u(s){var u,g,h,d=[],f=[],p=[],I=-1,C=s.length,m=fe(e),y=fe(r),E=e===n?function(){return g}:fe(n),v=r===i?function(){return h}:fe(i);function B(){d.push("M",a(t(p),c),l,A(t(f.reverse()),c),"Z")}for(;++I<C;)o.call(this,u=s[I],I)?(f.push([g=+m.call(this,u,I),h=+y.call(this,u,I)]),p.push([+E.call(this,u,I),+v.call(this,u,I)])):f.length&&(B(),f=[],p=[]);return f.length&&B(),d.length?d.join(""):null}return u.x=function(t){return arguments.length?(e=n=t,u):n},u.x0=function(t){return arguments.length?(e=t,u):e},u.x1=function(t){return arguments.length?(n=t,u):n},u.y=function(t){return arguments.length?(r=i=t,u):i},u.y0=function(t){return arguments.length?(r=t,u):r},u.y1=function(t){return arguments.length?(i=t,u):i},u.defined=function(t){return arguments.length?(o=t,u):o},u.interpolate=function(t){return arguments.length?(s="function"==typeof t?a=t:(a=Ga.get(t)||Ha).key,A=a.reverse||a,l=a.closed?"M":"L",u):s},u.tension=function(t){return arguments.length?(c=t,u):c},u}function ns(t){return t.radius}function rs(t){return[t.x,t.y]}function is(){return 64}function os(){return"circle"}function as(t){var e=Math.sqrt(t/Qt);return"M0,"+e+"A"+e+","+e+" 0 1,1 0,"+-e+"A"+e+","+e+" 0 1,1 0,"+e+"Z"}t.svg.line.radial=function(){var t=Oa(ts);return t.radius=t.x,delete t.x,t.angle=t.y,delete t.y,t},Ya.reverse=Pa,Pa.reverse=Ya,t.svg.area=function(){return es(k)},t.svg.area.radial=function(){var t=es(ts);return t.radius=t.x,delete t.x,t.innerRadius=t.x0,delete t.x0,t.outerRadius=t.x1,delete t.x1,t.angle=t.y,delete t.y,t.startAngle=t.y0,delete t.y0,t.endAngle=t.y1,delete t.y1,t},t.svg.chord=function(){var t=Gr,e=Hr,n=ns,r=Ra,i=Na;function o(n,r){var i,o,l=a(this,t,n,r),c=a(this,e,n,r);return"M"+l.p0+s(l.r,l.p1,l.a1-l.a0)+(o=c,(i=l).a0==o.a0&&i.a1==o.a1?A(l.r,l.p1,l.r,l.p0):A(l.r,l.p1,c.r,c.p0)+s(c.r,c.p1,c.a1-c.a0)+A(c.r,c.p1,l.r,l.p0))+"Z"}function a(t,e,o,a){var s=e.call(t,o,a),A=n.call(t,s,a),l=r.call(t,s,a)-_t,c=i.call(t,s,a)-_t;return{r:A,a0:l,a1:c,p0:[A*Math.cos(l),A*Math.sin(l)],p1:[A*Math.cos(c),A*Math.sin(c)]}}function s(t,e,n){return"A"+t+","+t+" 0 "+ +(n>Qt)+",1 "+e}function A(t,e,n,r){return"Q 0,0 "+r}return o.radius=function(t){return arguments.length?(n=fe(t),o):n},o.source=function(e){return arguments.length?(t=fe(e),o):t},o.target=function(t){return arguments.length?(e=fe(t),o):e},o.startAngle=function(t){return arguments.length?(r=fe(t),o):r},o.endAngle=function(t){return arguments.length?(i=fe(t),o):i},o},t.svg.diagonal=function(){var t=Gr,e=Hr,n=rs;function r(r,i){var o=t.call(this,r,i),a=e.call(this,r,i),s=(o.y+a.y)/2,A=[o,{x:o.x,y:s},{x:a.x,y:s},a];return"M"+(A=A.map(n))[0]+"C"+A[1]+" "+A[2]+" "+A[3]}return r.source=function(e){return arguments.length?(t=fe(e),r):t},r.target=function(t){return arguments.length?(e=fe(t),r):e},r.projection=function(t){return arguments.length?(n=t,r):n},r},t.svg.diagonal.radial=function(){var e=t.svg.diagonal(),n=rs,r=e.projection;return e.projection=function(t){return arguments.length?r(function(t){return function(){var e=t.apply(this,arguments),n=e[0],r=e[1]-_t;return[n*Math.cos(r),n*Math.sin(r)]}}(n=t)):n},e},t.svg.symbol=function(){var t=os,e=is;function n(n,r){return(ss.get(t.call(this,n,r))||as)(e.call(this,n,r))}return n.type=function(e){return arguments.length?(t=fe(e),n):t},n.size=function(t){return arguments.length?(e=fe(t),n):e},n};var ss=t.map({circle:as,cross:function(t){var e=Math.sqrt(t/5)/2;return"M"+-3*e+","+-e+"H"+-e+"V"+-3*e+"H"+e+"V"+-e+"H"+3*e+"V"+e+"H"+e+"V"+3*e+"H"+-e+"V"+e+"H"+-3*e+"Z"},diamond:function(t){var e=Math.sqrt(t/(2*ls)),n=e*ls;return"M0,"+-e+"L"+n+",0 0,"+e+" "+-n+",0Z"},square:function(t){var e=Math.sqrt(t)/2;return"M"+-e+","+-e+"L"+e+","+-e+" "+e+","+e+" "+-e+","+e+"Z"},"triangle-down":function(t){var e=Math.sqrt(t/As),n=e*As/2;return"M0,"+n+"L"+e+","+-n+" "+-e+","+-n+"Z"},"triangle-up":function(t){var e=Math.sqrt(t/As),n=e*As/2;return"M0,"+-n+"L"+e+","+n+" "+-e+","+n+"Z"}});t.svg.symbolTypes=ss.keys();var As=Math.sqrt(3),ls=Math.tan(30*Dt);J.transition=function(t){for(var e,n,r=hs||++ps,i=ms(t),o=[],a=ds||{time:Date.now(),ease:ro,delay:0,duration:250},s=-1,A=this.length;++s<A;){o.push(e=[]);for(var l=this[s],c=-1,u=l.length;++c<u;)(n=l[c])&&ys(n,c,i,r,a),e.push(n)}return gs(o,i,r)},J.interrupt=function(t){return this.each(null==t?cs:us(ms(t)))};var cs=us(ms());function us(t){return function(){var e,n,r;(e=this[t])&&(r=e[n=e.active])&&(r.timer.c=null,r.timer.t=NaN,--e.count?delete e[n]:delete this[t],e.active+=.5,r.event&&r.event.interrupt.call(this,this.__data__,r.index))}}function gs(t,e,n){return j(t,fs),t.namespace=e,t.id=n,t}var hs,ds,fs=[],ps=0;function Is(t,e,n,r){var i=t.id,o=t.namespace;return ct(t,"function"==typeof n?function(t,a,s){t[o][i].tween.set(e,r(n.call(t,t.__data__,a,s)))}:(n=r(n),function(t){t[o][i].tween.set(e,n)}))}function Cs(t){return null==t&&(t=""),function(){this.textContent=t}}function ms(t){return null==t?"__transition__":"__transition_"+t+"__"}function ys(t,e,n,r,i){var o,a,s,A,l,c=t[n]||(t[n]={active:0,count:0}),u=c[r];function g(n){var i=c.active,g=c[i];for(var d in g&&(g.timer.c=null,g.timer.t=NaN,--c.count,delete c[i],g.event&&g.event.interrupt.call(t,t.__data__,g.index)),c)if(+d<r){var f=c[d];f.timer.c=null,f.timer.t=NaN,--c.count,delete c[d]}a.c=h,Be(function(){return a.c&&h(n||1)&&(a.c=null,a.t=NaN),1},0,o),c.active=r,u.event&&u.event.start.call(t,t.__data__,e),l=[],u.tween.forEach(function(n,r){(r=r.call(t,t.__data__,e))&&l.push(r)}),A=u.ease,s=u.duration}function h(i){for(var o=i/s,a=A(o),g=l.length;g>0;)l[--g].call(t,a);if(o>=1)return u.event&&u.event.end.call(t,t.__data__,e),--c.count?delete c[r]:delete t[n],1}u||(o=i.time,a=Be(function(t){var e=u.delay;if(a.t=e+o,e<=t)return g(t-e);a.c=g},0,o),u=c[r]={tween:new y,time:o,timer:a,delay:i.delay,duration:i.duration,ease:i.ease,index:e},i=null,++c.count)}fs.call=J.call,fs.empty=J.empty,fs.node=J.node,fs.size=J.size,t.transition=function(e,n){return e&&e.transition?hs?e.transition(n):e:t.selection().transition(e)},t.transition.prototype=fs,fs.select=function(t){var e,n,r,i=this.id,o=this.namespace,a=[];t=q(t);for(var s=-1,A=this.length;++s<A;){a.push(e=[]);for(var l=this[s],c=-1,u=l.length;++c<u;)(r=l[c])&&(n=t.call(r,r.__data__,c,s))?("__data__"in r&&(n.__data__=r.__data__),ys(n,c,o,i,r[o][i]),e.push(n)):e.push(null)}return gs(a,o,i)},fs.selectAll=function(t){var e,n,r,i,o,a=this.id,s=this.namespace,A=[];t=V(t);for(var l=-1,c=this.length;++l<c;)for(var u=this[l],g=-1,h=u.length;++g<h;)if(r=u[g]){o=r[s][a],n=t.call(r,r.__data__,g,l),A.push(e=[]);for(var d=-1,f=n.length;++d<f;)(i=n[d])&&ys(i,d,s,a,o),e.push(i)}return gs(A,s,a)},fs.filter=function(t){var e,n,r=[];"function"!=typeof t&&(t=lt(t));for(var i=0,o=this.length;i<o;i++){r.push(e=[]);for(var a,s=0,A=(a=this[i]).length;s<A;s++)(n=a[s])&&t.call(n,n.__data__,s,i)&&e.push(n)}return gs(r,this.namespace,this.id)},fs.tween=function(t,e){var n=this.id,r=this.namespace;return arguments.length<2?this.node()[r][n].tween.get(t):ct(this,null==e?function(e){e[r][n].tween.remove(t)}:function(i){i[r][n].tween.set(t,e)})},fs.attr=function(e,n){if(arguments.length<2){for(n in e)this.attr(n,e[n]);return this}var r="transform"==e?fo:qi,i=t.ns.qualify(e);function o(){this.removeAttribute(i)}function a(){this.removeAttributeNS(i.space,i.local)}return Is(this,"attr."+e,n,i.local?function(t){return null==t?a:(t+="",function(){var e,n=this.getAttributeNS(i.space,i.local);return n!==t&&(e=r(n,t),function(t){this.setAttributeNS(i.space,i.local,e(t))})})}:function(t){return null==t?o:(t+="",function(){var e,n=this.getAttribute(i);return n!==t&&(e=r(n,t),function(t){this.setAttribute(i,e(t))})})})},fs.attrTween=function(e,n){var r=t.ns.qualify(e);return this.tween("attr."+e,r.local?function(t,e){var i=n.call(this,t,e,this.getAttributeNS(r.space,r.local));return i&&function(t){this.setAttributeNS(r.space,r.local,i(t))}}:function(t,e){var i=n.call(this,t,e,this.getAttribute(r));return i&&function(t){this.setAttribute(r,i(t))}})},fs.style=function(t,e,n){var r=arguments.length;if(r<3){if("string"!=typeof t){for(n in r<2&&(e=""),t)this.style(n,t[n],e);return this}n=""}function i(){this.style.removeProperty(t)}return Is(this,"style."+t,e,function(e){return null==e?i:(e+="",function(){var r,i=a(this).getComputedStyle(this,null).getPropertyValue(t);return i!==e&&(r=qi(i,e),function(e){this.style.setProperty(t,r(e),n)})})})},fs.styleTween=function(t,e,n){return arguments.length<3&&(n=""),this.tween("style."+t,function(r,i){var o=e.call(this,r,i,a(this).getComputedStyle(this,null).getPropertyValue(t));return o&&function(e){this.style.setProperty(t,o(e),n)}})},fs.text=function(t){return Is(this,"text",t,Cs)},fs.remove=function(){var t=this.namespace;return this.each("end.transition",function(){var e;this[t].count<2&&(e=this.parentNode)&&e.removeChild(this)})},fs.ease=function(e){var n=this.id,r=this.namespace;return arguments.length<1?this.node()[r][n].ease:("function"!=typeof e&&(e=t.ease.apply(t,arguments)),ct(this,function(t){t[r][n].ease=e}))},fs.delay=function(t){var e=this.id,n=this.namespace;return arguments.length<1?this.node()[n][e].delay:ct(this,"function"==typeof t?function(r,i,o){r[n][e].delay=+t.call(r,r.__data__,i,o)}:(t=+t,function(r){r[n][e].delay=t}))},fs.duration=function(t){var e=this.id,n=this.namespace;return arguments.length<1?this.node()[n][e].duration:ct(this,"function"==typeof t?function(r,i,o){r[n][e].duration=Math.max(1,t.call(r,r.__data__,i,o))}:(t=Math.max(1,t),function(r){r[n][e].duration=t}))},fs.each=function(e,n){var r=this.id,i=this.namespace;if(arguments.length<2){var o=ds,a=hs;try{hs=r,ct(this,function(t,n,o){ds=t[i][r],e.call(t,t.__data__,n,o)})}finally{ds=o,hs=a}}else ct(this,function(o){var a=o[i][r];(a.event||(a.event=t.dispatch("start","end","interrupt"))).on(e,n)});return this},fs.transition=function(){for(var t,e,n,r=this.id,i=++ps,o=this.namespace,a=[],s=0,A=this.length;s<A;s++){a.push(t=[]);for(var l,c=0,u=(l=this[s]).length;c<u;c++)(e=l[c])&&ys(e,c,o,i,{time:(n=e[o][r]).time,ease:n.ease,delay:n.delay+n.duration,duration:n.duration}),t.push(e)}return gs(a,o,i)},t.svg.axis=function(){var e,n=t.scale.linear(),i=Es,o=6,a=6,s=3,A=[10],l=null;function c(r){r.each(function(){var r,c=t.select(this),u=this.__chart__||n,g=this.__chart__=n.copy(),h=null==l?g.ticks?g.ticks.apply(g,A):g.domain():l,d=null==e?g.tickFormat?g.tickFormat.apply(g,A):k:e,f=c.selectAll(".tick").data(h,g),p=f.enter().insert("g",".domain").attr("class","tick").style("opacity",Bt),I=t.transition(f.exit()).style("opacity",Bt).remove(),C=t.transition(f.order()).style("opacity",1),m=Math.max(o,0)+s,y=ca(g),E=c.selectAll(".domain").data([0]),v=(E.enter().append("path").attr("class","domain"),t.transition(E));p.append("line"),p.append("text");var B,b,Q,w,x=p.select("line"),_=C.select("line"),D=f.select("text").text(d),M=p.select("text"),S=C.select("text"),L="top"===i||"left"===i?-1:1;if("bottom"===i||"top"===i?(r=Bs,B="x",Q="y",b="x2",w="y2",D.attr("dy",L<0?"0em":".71em").style("text-anchor","middle"),v.attr("d","M"+y[0]+","+L*a+"V0H"+y[1]+"V"+L*a)):(r=bs,B="y",Q="x",b="y2",w="x2",D.attr("dy",".32em").style("text-anchor",L<0?"end":"start"),v.attr("d","M"+L*a+","+y[0]+"H0V"+y[1]+"H"+L*a)),x.attr(w,L*o),M.attr(Q,L*m),_.attr(b,0).attr(w,L*o),S.attr(B,0).attr(Q,L*m),g.rangeBand){var R=g,N=R.rangeBand()/2;u=g=function(t){return R(t)+N}}else u.rangeBand?u=g:I.call(r,g,u);p.call(r,u,g),C.call(r,g,g)})}return c.scale=function(t){return arguments.length?(n=t,c):n},c.orient=function(t){return arguments.length?(i=t in vs?t+"":Es,c):i},c.ticks=function(){return arguments.length?(A=r(arguments),c):A},c.tickValues=function(t){return arguments.length?(l=t,c):l},c.tickFormat=function(t){return arguments.length?(e=t,c):e},c.tickSize=function(t){var e=arguments.length;return e?(o=+t,a=+arguments[e-1],c):o},c.innerTickSize=function(t){return arguments.length?(o=+t,c):o},c.outerTickSize=function(t){return arguments.length?(a=+t,c):a},c.tickPadding=function(t){return arguments.length?(s=+t,c):s},c.tickSubdivide=function(){return arguments.length&&c},c};var Es="bottom",vs={top:1,right:1,bottom:1,left:1};function Bs(t,e,n){t.attr("transform",function(t){var r=e(t);return"translate("+(isFinite(r)?r:n(t))+",0)"})}function bs(t,e,n){t.attr("transform",function(t){var r=e(t);return"translate(0,"+(isFinite(r)?r:n(t))+")"})}t.svg.brush=function(){var e,n,r=G(g,"brushstart","brush","brushend"),i=null,o=null,s=[0,0],A=[0,0],l=!0,c=!0,u=ws[0];function g(e){e.each(function(){var e=t.select(this).style("pointer-events","all").style("-webkit-tap-highlight-color","rgba(0,0,0,0)").on("mousedown.brush",p).on("touchstart.brush",p),n=e.selectAll(".background").data([0]);n.enter().append("rect").attr("class","background").style("visibility","hidden").style("cursor","crosshair"),e.selectAll(".extent").data([0]).enter().append("rect").attr("class","extent").style("cursor","move");var r=e.selectAll(".resize").data(u,k);r.exit().remove(),r.enter().append("g").attr("class",function(t){return"resize "+t}).style("cursor",function(t){return Qs[t]}).append("rect").attr("x",function(t){return/[ew]$/.test(t)?-3:null}).attr("y",function(t){return/^[ns]/.test(t)?-3:null}).attr("width",6).attr("height",6).style("visibility","hidden"),r.style("display",g.empty()?"none":null);var a,s=t.transition(e),A=t.transition(n);i&&(a=ca(i),A.attr("x",a[0]).attr("width",a[1]-a[0]),d(s)),o&&(a=ca(o),A.attr("y",a[0]).attr("height",a[1]-a[0]),f(s)),h(s)})}function h(t){t.selectAll(".resize").attr("transform",function(t){return"translate("+s[+/e$/.test(t)]+","+A[+/^s/.test(t)]+")"})}function d(t){t.select(".extent").attr("x",s[0]),t.selectAll(".extent,.n>rect,.s>rect").attr("width",s[1]-s[0])}function f(t){t.select(".extent").attr("y",A[0]),t.selectAll(".extent,.e>rect,.w>rect").attr("height",A[1]-A[0])}function p(){var u,p,I=this,C=t.select(t.event.target),m=r.of(I,arguments),y=t.select(I),E=C.datum(),v=!/^(n|s)$/.test(E)&&i,B=!/^(e|w)$/.test(E)&&o,b=C.classed("extent"),Q=mt(I),w=t.mouse(I),x=t.select(a(I)).on("keydown.brush",function(){32==t.event.keyCode&&(b||(u=null,w[0]-=s[1],w[1]-=A[1],b=2),U())}).on("keyup.brush",function(){32==t.event.keyCode&&2==b&&(w[0]+=s[1],w[1]+=A[1],b=0,U())});if(t.event.changedTouches?x.on("touchmove.brush",M).on("touchend.brush",S):x.on("mousemove.brush",M).on("mouseup.brush",S),y.interrupt().selectAll("*").interrupt(),b)w[0]=s[0]-w[0],w[1]=A[0]-w[1];else if(E){var _=+/w$/.test(E),D=+/^n/.test(E);p=[s[1-_]-w[0],A[1-D]-w[1]],w[0]=s[_],w[1]=A[D]}else t.event.altKey&&(u=w.slice());function M(){var e=t.mouse(I),n=!1;p&&(e[0]+=p[0],e[1]+=p[1]),b||(t.event.altKey?(u||(u=[(s[0]+s[1])/2,(A[0]+A[1])/2]),w[0]=s[+(e[0]<u[0])],w[1]=A[+(e[1]<u[1])]):u=null),v&&k(e,i,0)&&(d(y),n=!0),B&&k(e,o,1)&&(f(y),n=!0),n&&(h(y),m({type:"brush",mode:b?"move":"resize"}))}function k(t,r,i){var o,a,g=ca(r),h=g[0],d=g[1],f=w[i],p=i?A:s,I=p[1]-p[0];if(b&&(h-=f,d-=I+f),o=(i?c:l)?Math.max(h,Math.min(d,t[i])):t[i],b?a=(o+=f)+I:(u&&(f=Math.max(h,Math.min(d,2*u[i]-o))),f<o?(a=o,o=f):a=f),p[0]!=o||p[1]!=a)return i?n=null:e=null,p[0]=o,p[1]=a,!0}function S(){M(),y.style("pointer-events","all").selectAll(".resize").style("display",g.empty()?"none":null),t.select("body").style("cursor",null),x.on("mousemove.brush",null).on("mouseup.brush",null).on("touchmove.brush",null).on("touchend.brush",null).on("keydown.brush",null).on("keyup.brush",null),Q(),m({type:"brushend"})}y.style("pointer-events","none").selectAll(".resize").style("display",null),t.select("body").style("cursor",C.style("cursor")),m({type:"brushstart"}),M()}return g.event=function(i){i.each(function(){var i=r.of(this,arguments),o={x:s,y:A,i:e,j:n},a=this.__chart__||o;this.__chart__=o,hs?t.select(this).transition().each("start.brush",function(){e=a.i,n=a.j,s=a.x,A=a.y,i({type:"brushstart"})}).tween("brush:brush",function(){var t=Vi(s,o.x),r=Vi(A,o.y);return e=n=null,function(e){s=o.x=t(e),A=o.y=r(e),i({type:"brush",mode:"resize"})}}).each("end.brush",function(){e=o.i,n=o.j,i({type:"brush",mode:"resize"}),i({type:"brushend"})}):(i({type:"brushstart"}),i({type:"brush",mode:"resize"}),i({type:"brushend"}))})},g.x=function(t){return arguments.length?(u=ws[!(i=t)<<1|!o],g):i},g.y=function(t){return arguments.length?(u=ws[!i<<1|!(o=t)],g):o},g.clamp=function(t){return arguments.length?(i&&o?(l=!!t[0],c=!!t[1]):i?l=!!t:o&&(c=!!t),g):i&&o?[l,c]:i?l:o?c:null},g.extent=function(t){var r,a,l,c,u;return arguments.length?(i&&(r=t[0],a=t[1],o&&(r=r[0],a=a[0]),e=[r,a],i.invert&&(r=i(r),a=i(a)),a<r&&(u=r,r=a,a=u),r==s[0]&&a==s[1]||(s=[r,a])),o&&(l=t[0],c=t[1],i&&(l=l[1],c=c[1]),n=[l,c],o.invert&&(l=o(l),c=o(c)),c<l&&(u=l,l=c,c=u),l==A[0]&&c==A[1]||(A=[l,c])),g):(i&&(e?(r=e[0],a=e[1]):(r=s[0],a=s[1],i.invert&&(r=i.invert(r),a=i.invert(a)),a<r&&(u=r,r=a,a=u))),o&&(n?(l=n[0],c=n[1]):(l=A[0],c=A[1],o.invert&&(l=o.invert(l),c=o.invert(c)),c<l&&(u=l,l=c,c=u))),i&&o?[[r,l],[a,c]]:i?[r,a]:o&&[l,c])},g.clear=function(){return g.empty()||(s=[0,0],A=[0,0],e=n=null),g},g.empty=function(){return!!i&&s[0]==s[1]||!!o&&A[0]==A[1]},t.rebind(g,r,"on")};var Qs={n:"ns-resize",e:"ew-resize",s:"ns-resize",w:"ew-resize",nw:"nwse-resize",ne:"nesw-resize",se:"nwse-resize",sw:"nesw-resize"},ws=[["n","e","s","w","nw","ne","se","sw"],["e","w"],["n","s"],[]],xs=Se.format=sn.timeFormat,_s=xs.utc,Ds=_s("%Y-%m-%dT%H:%M:%S.%LZ");function Ms(t){return t.toISOString()}function ks(e,n,r){function i(t){return e(t)}function o(e,r){var i=(e[1]-e[0])/r,o=t.bisect(Ls,i);return o==Ls.length?[n.year,Ca(e.map(function(t){return t/31536e6}),r)[2]]:o?n[i/Ls[o-1]<Ls[o]/i?o-1:o]:[Fs,Ca(e,r)[2]]}return i.invert=function(t){return Ss(e.invert(t))},i.domain=function(t){return arguments.length?(e.domain(t),i):e.domain().map(Ss)},i.nice=function(t,e){var n=i.domain(),r=la(n),a=null==t?o(r,10):"number"==typeof t&&o(r,t);function s(n){return!isNaN(n)&&!t.range(n,Ss(+n+1),e).length}return a&&(t=a[0],e=a[1]),i.domain(ga(n,e>1?{floor:function(e){for(;s(e=t.floor(e));)e=Ss(e-1);return e},ceil:function(e){for(;s(e=t.ceil(e));)e=Ss(+e+1);return e}}:t))},i.ticks=function(t,e){var n=la(i.domain()),r=null==t?o(n,10):"number"==typeof t?o(n,t):!t.range&&[{range:t},e];return r&&(t=r[0],e=r[1]),t.range(n[0],Ss(+n[1]+1),e<1?1:e)},i.tickFormat=function(){return r},i.copy=function(){return ks(e.copy(),n,r)},pa(i,e)}function Ss(t){return new Date(t)}xs.iso=Date.prototype.toISOString&&+new Date("2000-01-01T00:00:00.000Z")?Ms:Ds,Ms.parse=function(t){var e=new Date(t);return isNaN(e)?null:e},Ms.toString=Ds.toString,Se.second=Fe(function(t){return new Le(1e3*Math.floor(t/1e3))},function(t,e){t.setTime(t.getTime()+1e3*Math.floor(e))},function(t){return t.getSeconds()}),Se.seconds=Se.second.range,Se.seconds.utc=Se.second.utc.range,Se.minute=Fe(function(t){return new Le(6e4*Math.floor(t/6e4))},function(t,e){t.setTime(t.getTime()+6e4*Math.floor(e))},function(t){return t.getMinutes()}),Se.minutes=Se.minute.range,Se.minutes.utc=Se.minute.utc.range,Se.hour=Fe(function(t){var e=t.getTimezoneOffset()/60;return new Le(36e5*(Math.floor(t/36e5-e)+e))},function(t,e){t.setTime(t.getTime()+36e5*Math.floor(e))},function(t){return t.getHours()}),Se.hours=Se.hour.range,Se.hours.utc=Se.hour.utc.range,Se.month=Fe(function(t){return(t=Se.day(t)).setDate(1),t},function(t,e){t.setMonth(t.getMonth()+e)},function(t){return t.getMonth()}),Se.months=Se.month.range,Se.months.utc=Se.month.utc.range;var Ls=[1e3,5e3,15e3,3e4,6e4,3e5,9e5,18e5,36e5,108e5,216e5,432e5,864e5,1728e5,6048e5,2592e6,7776e6,31536e6],Rs=[[Se.second,1],[Se.second,5],[Se.second,15],[Se.second,30],[Se.minute,1],[Se.minute,5],[Se.minute,15],[Se.minute,30],[Se.hour,1],[Se.hour,3],[Se.hour,6],[Se.hour,12],[Se.day,1],[Se.day,2],[Se.week,1],[Se.month,1],[Se.month,3],[Se.year,1]],Ns=xs.multi([[".%L",function(t){return t.getMilliseconds()}],[":%S",function(t){return t.getSeconds()}],["%I:%M",function(t){return t.getMinutes()}],["%I %p",function(t){return t.getHours()}],["%a %d",function(t){return t.getDay()&&1!=t.getDate()}],["%b %d",function(t){return 1!=t.getDate()}],["%B",function(t){return t.getMonth()}],["%Y",zn]]),Fs={range:function(e,n,r){return t.range(Math.ceil(e/r)*r,+n,r).map(Ss)},floor:k,ceil:k};Rs.year=Se.year,Se.scale=function(){return ks(t.scale.linear(),Rs,Ns)};var Ts=Rs.map(function(t){return[t[0].utc,t[1]]}),Us=_s.multi([[".%L",function(t){return t.getUTCMilliseconds()}],[":%S",function(t){return t.getUTCSeconds()}],["%I:%M",function(t){return t.getUTCMinutes()}],["%I %p",function(t){return t.getUTCHours()}],["%a %d",function(t){return t.getUTCDay()&&1!=t.getUTCDate()}],["%b %d",function(t){return 1!=t.getUTCDate()}],["%B",function(t){return t.getUTCMonth()}],["%Y",zn]]);function Os(t){return JSON.parse(t.responseText)}function Gs(t){var e=i.createRange();return e.selectNode(i.body),e.createContextualFragment(t.responseText)}Ts.year=Se.year.utc,Se.scale.utc=function(){return ks(t.scale.linear(),Ts,Us)},t.text=pe(function(t){return t.responseText}),t.json=function(t,e){return Ie(t,"application/json",Os,e)},t.html=function(t,e){return Ie(t,"text/html",Gs,e)},t.xml=pe(function(t){return t.responseXML}),"object"==typeof e&&e.exports?e.exports=t:this.d3=t}()},{}],164:[function(t,e,n){e.exports=function(){for(var t=0;t<arguments.length;t++)if(void 0!==arguments[t])return arguments[t]}},{}],165:[function(t,e,n){"use strict";var r=t("incremental-convex-hull"),i=t("uniq");function o(t,e){this.point=t,this.index=e}function a(t,e){for(var n=t.point,r=e.point,i=n.length,o=0;o<i;++o){var a=r[o]-n[o];if(a)return a}return 0}e.exports=function(t,e){var n=t.length;if(0===n)return[];var s=t[0].length;if(s<1)return[];if(1===s)return function(t,e,n){if(1===t)return n?[[-1,0]]:[];var r=e.map(function(t,e){return[t[0],e]});r.sort(function(t,e){return t[0]-e[0]});for(var i=new Array(t-1),o=1;o<t;++o){var a=r[o-1],s=r[o];i[o-1]=[a[1],s[1]]}return n&&i.push([-1,i[0][1]],[i[t-1][1],-1]),i}(n,t,e);for(var A=new Array(n),l=1,c=0;c<n;++c){for(var u=t[c],g=new Array(s+1),h=0,d=0;d<s;++d){var f=u[d];g[d]=f,h+=f*f}g[s]=h,A[c]=new o(g,c),l=Math.max(h,l)}i(A,a),n=A.length;for(var p=new Array(n+s+1),I=new Array(n+s+1),C=(s+1)*(s+1)*l,m=new Array(s+1),c=0;c<=s;++c)m[c]=0;m[s]=C,p[0]=m.slice(),I[0]=-1;for(var c=0;c<=s;++c){var g=m.slice();g[c]=1,p[c+1]=g,I[c+1]=-1}for(var c=0;c<n;++c){var y=A[c];p[c+s+1]=y.point,I[c+s+1]=y.index}var E=r(p,!1);if(E=e?E.filter(function(t){for(var e=0,n=0;n<=s;++n){var r=I[t[n]];if(r<0&&++e>=2)return!1;t[n]=r}return!0}):E.filter(function(t){for(var e=0;e<=s;++e){var n=I[t[e]];if(n<0)return!1;t[e]=n}return!0}),1&s)for(var c=0;c<E.length;++c){var y=E[c],g=y[0];y[0]=y[1],y[1]=g}return E}},{"incremental-convex-hull":413,uniq:547}],166:[function(t,e,n){"use strict";e.exports=o;var r=(o.canvas=document.createElement("canvas")).getContext("2d"),i=a([32,126]);function o(t,e){Array.isArray(t)&&(t=t.join(", "));var n,o={},s=16,A=.05;e&&(2===e.length&&"number"==typeof e[0]?n=a(e):Array.isArray(e)?n=e:(e.o?n=a(e.o):e.pairs&&(n=e.pairs),e.fontSize&&(s=e.fontSize),null!=e.threshold&&(A=e.threshold))),n||(n=i),r.font=s+"px "+t;for(var l=0;l<n.length;l++){var c=n[l],u=r.measureText(c[0]).width+r.measureText(c[1]).width,g=r.measureText(c).width;if(Math.abs(u-g)>s*A){var h=(g-u)/s;o[c]=1e3*h}}return o}function a(t){for(var e=[],n=t[0];n<=t[1];n++)for(var r=String.fromCharCode(n),i=t[0];i<t[1];i++){var o=r+String.fromCharCode(i);e.push(o)}return e}o.createPairs=a,o.ascii=i},{}],167:[function(t,e,n){(function(t){var n=!1;if("undefined"!=typeof Float64Array){var r=new Float64Array(1),i=new Uint32Array(r.buffer);r[0]=1,n=!0,1072693248===i[1]?(e.exports=function(t){return r[0]=t,[i[0],i[1]]},e.exports.pack=function(t,e){return i[0]=t,i[1]=e,r[0]},e.exports.lo=function(t){return r[0]=t,i[0]},e.exports.hi=function(t){return r[0]=t,i[1]}):1072693248===i[0]?(e.exports=function(t){return r[0]=t,[i[1],i[0]]},e.exports.pack=function(t,e){return i[1]=t,i[0]=e,r[0]},e.exports.lo=function(t){return r[0]=t,i[1]},e.exports.hi=function(t){return r[0]=t,i[0]}):n=!1}if(!n){var o=new t(8);e.exports=function(t){return o.writeDoubleLE(t,0,!0),[o.readUInt32LE(0,!0),o.readUInt32LE(4,!0)]},e.exports.pack=function(t,e){return o.writeUInt32LE(t,0,!0),o.writeUInt32LE(e,4,!0),o.readDoubleLE(0,!0)},e.exports.lo=function(t){return o.writeDoubleLE(t,0,!0),o.readUInt32LE(0,!0)},e.exports.hi=function(t){return o.writeDoubleLE(t,0,!0),o.readUInt32LE(4,!0)}}e.exports.sign=function(t){return e.exports.hi(t)>>>31},e.exports.exponent=function(t){return(e.exports.hi(t)<<1>>>21)-1023},e.exports.fraction=function(t){var n=e.exports.lo(t),r=e.exports.hi(t),i=1048575&r;return 2146435072&r&&(i+=1<<20),[n,i]},e.exports.denormalized=function(t){return!(2146435072&e.exports.hi(t))}}).call(this,t("buffer").Buffer)},{buffer:105}],168:[function(t,e,n){var r=t("abs-svg-path"),i=t("normalize-svg-path"),o={M:"moveTo",C:"bezierCurveTo"};e.exports=function(t,e){t.beginPath(),i(r(e)).forEach(function(e){var n=e[0],r=e.slice(1);t[o[n]].apply(t,r)}),t.closePath()}},{"abs-svg-path":60,"normalize-svg-path":452}],169:[function(t,e,n){e.exports=function(t){switch(t){case"int8":return Int8Array;case"int16":return Int16Array;case"int32":return Int32Array;case"uint8":return Uint8Array;case"uint16":return Uint16Array;case"uint32":return Uint32Array;case"float32":return Float32Array;case"float64":return Float64Array;case"array":return Array;case"uint8_clamped":return Uint8ClampedArray}}},{}],170:[function(t,e,n){"use strict";e.exports=function(t,e){switch(void 0===e&&(e=0),typeof t){case"number":if(t>0)return function(t,e){var n,r;for(n=new Array(t),r=0;r<t;++r)n[r]=e;return n}(0|t,e);break;case"object":if("number"==typeof t.length)return function t(e,n,r){var i=0|e[r];if(i<=0)return[];var o,a=new Array(i);if(r===e.length-1)for(o=0;o<i;++o)a[o]=n;else for(o=0;o<i;++o)a[o]=t(e,n,r+1);return a}(t,e,0)}return[]}},{}],171:[function(t,e,n){"use strict";e.exports=function(t,e){var n=t.length;if("number"!=typeof e){e=0;for(var i=0;i<n;++i){var o=t[i];e=Math.max(e,o[0],o[1])}e=1+(0|e)}e|=0;for(var a=new Array(e),i=0;i<e;++i)a[i]=[];for(var i=0;i<n;++i){var o=t[i];a[o[0]].push(o[1]),a[o[1]].push(o[0])}for(var s=0;s<e;++s)r(a[s],function(t,e){return t-e});return a};var r=t("uniq")},{uniq:547}],172:[function(t,e,n){var r=t("strongly-connected-components");e.exports=function(t){var e,n=[],i=[],o=[],a={},s=[];function A(t){var n,r,c=!1;for(i.push(t),o[t]=!0,n=0;n<s[t].length;n++)(r=s[t][n])===e?(l(e,i),c=!0):o[r]||(c=A(r));if(c)!function t(e){o[e]=!1,a.hasOwnProperty(e)&&Object.keys(a[e]).forEach(function(n){delete a[e][n],o[n]&&t(n)})}(t);else for(n=0;n<s[t].length;n++){r=s[t][n];var u=a[r];u||(u={},a[r]=u),u[r]=!0}return i.pop(),c}function l(t,e){var r=[].concat(e).concat(t);n.push(r)}function c(e){!function(e){for(var n=0;n<t.length;n++)n<e&&(t[n]=[]),t[n]=t[n].filter(function(t){return t>=e})}(e);for(var n,i=r(t).components.filter(function(t){return t.length>1}),o=1/0,a=0;a<i.length;a++)for(var s=0;s<i[a].length;s++)i[a][s]<o&&(o=i[a][s],n=a);var A=i[n];return!!A&&{leastVertex:o,adjList:t.map(function(t,e){return-1===A.indexOf(e)?[]:t.filter(function(t){return-1!==A.indexOf(t)})})}}e=0;for(var u=t.length;e<u;){var g=c(e);if(e=g.leastVertex,s=g.adjList){for(var h=0;h<s.length;h++)for(var d=0;d<s[h].length;d++){var f=s[h][d];o[+f]=!1,a[f]={}}A(e),e+=1}else e=u}return n}},{"strongly-connected-components":530}],173:[function(t,e,n){"use strict";var r=t("../../object/valid-value");e.exports=function(){return r(this).length=0,this}},{"../../object/valid-value":205}],174:[function(t,e,n){"use strict";e.exports=t("./is-implemented")()?Array.from:t("./shim")},{"./is-implemented":175,"./shim":176}],175:[function(t,e,n){"use strict";e.exports=function(){var t,e,n=Array.from;return"function"==typeof n&&(e=n(t=["raz","dwa"]),Boolean(e&&e!==t&&"dwa"===e[1]))}},{}],176:[function(t,e,n){"use strict";var r=t("es6-symbol").iterator,i=t("../../function/is-arguments"),o=t("../../function/is-function"),a=t("../../number/to-pos-integer"),s=t("../../object/valid-callable"),A=t("../../object/valid-value"),l=t("../../object/is-value"),c=t("../../string/is-string"),u=Array.isArray,g=Function.prototype.call,h={configurable:!0,enumerable:!0,writable:!0,value:null},d=Object.defineProperty;e.exports=function(t){var e,n,f,p,I,C,m,y,E,v,B=arguments[1],b=arguments[2];if(t=Object(A(t)),l(B)&&s(B),this&&this!==Array&&o(this))e=this;else{if(!B){if(i(t))return 1!==(I=t.length)?Array.apply(null,t):((p=new Array(1))[0]=t[0],p);if(u(t)){for(p=new Array(I=t.length),n=0;n<I;++n)p[n]=t[n];return p}}p=[]}if(!u(t))if(void 0!==(E=t[r])){for(m=s(E).call(t),e&&(p=new e),y=m.next(),n=0;!y.done;)v=B?g.call(B,b,y.value,n):y.value,e?(h.value=v,d(p,n,h)):p[n]=v,y=m.next(),++n;I=n}else if(c(t)){for(I=t.length,e&&(p=new e),n=0,f=0;n<I;++n)v=t[n],n+1<I&&(C=v.charCodeAt(0))>=55296&&C<=56319&&(v+=t[++n]),v=B?g.call(B,b,v,f):v,e?(h.value=v,d(p,f,h)):p[f]=v,++f;I=f}if(void 0===I)for(I=a(t.length),e&&(p=new e(I)),n=0;n<I;++n)v=B?g.call(B,b,t[n],n):t[n],e?(h.value=v,d(p,n,h)):p[n]=v;return e&&(h.value=null,p.length=I),p}},{"../../function/is-arguments":177,"../../function/is-function":178,"../../number/to-pos-integer":184,"../../object/is-value":194,"../../object/valid-callable":203,"../../object/valid-value":205,"../../string/is-string":209,"es6-symbol":219}],177:[function(t,e,n){"use strict";var r=Object.prototype.toString,i=r.call(function(){return arguments}());e.exports=function(t){return r.call(t)===i}},{}],178:[function(t,e,n){"use strict";var r=Object.prototype.toString,i=r.call(t("./noop"));e.exports=function(t){return"function"==typeof t&&r.call(t)===i}},{"./noop":179}],179:[function(t,e,n){"use strict";e.exports=function(){}},{}],180:[function(t,e,n){"use strict";e.exports=t("./is-implemented")()?Math.sign:t("./shim")},{"./is-implemented":181,"./shim":182}],181:[function(t,e,n){"use strict";e.exports=function(){var t=Math.sign;return"function"==typeof t&&1===t(10)&&-1===t(-20)}},{}],182:[function(t,e,n){"use strict";e.exports=function(t){return t=Number(t),isNaN(t)||0===t?t:t>0?1:-1}},{}],183:[function(t,e,n){"use strict";var r=t("../math/sign"),i=Math.abs,o=Math.floor;e.exports=function(t){return isNaN(t)?0:0!==(t=Number(t))&&isFinite(t)?r(t)*o(i(t)):t}},{"../math/sign":180}],184:[function(t,e,n){"use strict";var r=t("./to-integer"),i=Math.max;e.exports=function(t){return i(0,r(t))}},{"./to-integer":183}],185:[function(t,e,n){"use strict";var r=t("./valid-callable"),i=t("./valid-value"),o=Function.prototype.bind,a=Function.prototype.call,s=Object.keys,A=Object.prototype.propertyIsEnumerable;e.exports=function(t,e){return function(n,l){var c,u=arguments[2],g=arguments[3];return n=Object(i(n)),r(l),c=s(n),g&&c.sort("function"==typeof g?o.call(g,n):void 0),"function"!=typeof t&&(t=c[t]),a.call(t,c,function(t,r){return A.call(n,t)?a.call(l,u,n[t],t,n,r):e})}}},{"./valid-callable":203,"./valid-value":205}],186:[function(t,e,n){"use strict";e.exports=t("./is-implemented")()?Object.assign:t("./shim")},{"./is-implemented":187,"./shim":188}],187:[function(t,e,n){"use strict";e.exports=function(){var t,e=Object.assign;return"function"==typeof e&&(e(t={foo:"raz"},{bar:"dwa"},{trzy:"trzy"}),t.foo+t.bar+t.trzy==="razdwatrzy")}},{}],188:[function(t,e,n){"use strict";var r=t("../keys"),i=t("../valid-value"),o=Math.max;e.exports=function(t,e){var n,a,s,A=o(arguments.length,2);for(t=Object(i(t)),s=function(r){try{t[r]=e[r]}catch(t){n||(n=t)}},a=1;a<A;++a)e=arguments[a],r(e).forEach(s);if(void 0!==n)throw n;return t}},{"../keys":195,"../valid-value":205}],189:[function(t,e,n){"use strict";var r=t("../array/from"),i=t("./assign"),o=t("./valid-value");e.exports=function(t){var e=Object(o(t)),n=arguments[1],a=Object(arguments[2]);if(e!==t&&!n)return e;var s={};return n?r(n,function(e){(a.ensure||e in t)&&(s[e]=t[e])}):i(s,t),s}},{"../array/from":174,"./assign":186,"./valid-value":205}],190:[function(t,e,n){"use strict";var r,i,o,a,s=Object.create;t("./set-prototype-of/is-implemented")()||(r=t("./set-prototype-of/shim")),e.exports=r?1!==r.level?s:(i={},o={},a={configurable:!1,enumerable:!1,writable:!0,value:void 0},Object.getOwnPropertyNames(Object.prototype).forEach(function(t){o[t]="__proto__"!==t?a:{configurable:!0,enumerable:!1,writable:!0,value:void 0}}),Object.defineProperties(i,o),Object.defineProperty(r,"nullPolyfill",{configurable:!1,enumerable:!1,writable:!1,value:i}),function(t,e){return s(null===t?i:t,e)}):s},{"./set-prototype-of/is-implemented":201,"./set-prototype-of/shim":202}],191:[function(t,e,n){"use strict";e.exports=t("./_iterate")("forEach")},{"./_iterate":185}],192:[function(t,e,n){"use strict";e.exports=function(t){return"function"==typeof t}},{}],193:[function(t,e,n){"use strict";var r=t("./is-value"),i={function:!0,object:!0};e.exports=function(t){return r(t)&&i[typeof t]||!1}},{"./is-value":194}],194:[function(t,e,n){"use strict";var r=t("../function/noop")();e.exports=function(t){return t!==r&&null!==t}},{"../function/noop":179}],195:[function(t,e,n){"use strict";e.exports=t("./is-implemented")()?Object.keys:t("./shim")},{"./is-implemented":196,"./shim":197}],196:[function(t,e,n){"use strict";e.exports=function(){try{return Object.keys("primitive"),!0}catch(t){return!1}}},{}],197:[function(t,e,n){"use strict";var r=t("../is-value"),i=Object.keys;e.exports=function(t){return i(r(t)?Object(t):t)}},{"../is-value":194}],198:[function(t,e,n){"use strict";var r=t("./valid-callable"),i=t("./for-each"),o=Function.prototype.call;e.exports=function(t,e){var n={},a=arguments[2];return r(e),i(t,function(t,r,i,s){n[r]=o.call(e,a,t,r,i,s)}),n}},{"./for-each":191,"./valid-callable":203}],199:[function(t,e,n){"use strict";var r=t("./is-value"),i=Array.prototype.forEach,o=Object.create;e.exports=function(t){var e=o(null);return i.call(arguments,function(t){r(t)&&function(t,e){var n;for(n in t)e[n]=t[n]}(Object(t),e)}),e}},{"./is-value":194}],200:[function(t,e,n){"use strict";e.exports=t("./is-implemented")()?Object.setPrototypeOf:t("./shim")},{"./is-implemented":201,"./shim":202}],201:[function(t,e,n){"use strict";var r=Object.create,i=Object.getPrototypeOf,o={};e.exports=function(){var t=Object.setPrototypeOf,e=arguments[0]||r;return"function"==typeof t&&i(t(e(null),o))===o}},{}],202:[function(t,e,n){"use strict";var r,i,o,a,s=t("../is-object"),A=t("../valid-value"),l=Object.prototype.isPrototypeOf,c=Object.defineProperty,u={configurable:!0,enumerable:!1,writable:!0,value:void 0};r=function(t,e){if(A(t),null===e||s(e))return t;throw new TypeError("Prototype must be null or an object")},e.exports=(i=function(){var t,e=Object.create(null),n={},r=Object.getOwnPropertyDescriptor(Object.prototype,"__proto__");if(r){try{(t=r.set).call(e,n)}catch(t){}if(Object.getPrototypeOf(e)===n)return{set:t,level:2}}return e.__proto__=n,Object.getPrototypeOf(e)===n?{level:2}:((e={}).__proto__=n,Object.getPrototypeOf(e)===n&&{level:1})}())?(2===i.level?i.set?(a=i.set,o=function(t,e){return a.call(r(t,e),e),t}):o=function(t,e){return r(t,e).__proto__=e,t}:o=function t(e,n){var i;return r(e,n),(i=l.call(t.nullPolyfill,e))&&delete t.nullPolyfill.__proto__,null===n&&(n=t.nullPolyfill),e.__proto__=n,i&&c(t.nullPolyfill,"__proto__",u),e},Object.defineProperty(o,"level",{configurable:!1,enumerable:!1,writable:!1,value:i.level})):null,t("../create")},{"../create":190,"../is-object":193,"../valid-value":205}],203:[function(t,e,n){"use strict";e.exports=function(t){if("function"!=typeof t)throw new TypeError(t+" is not a function");return t}},{}],204:[function(t,e,n){"use strict";var r=t("./is-object");e.exports=function(t){if(!r(t))throw new TypeError(t+" is not an Object");return t}},{"./is-object":193}],205:[function(t,e,n){"use strict";var r=t("./is-value");e.exports=function(t){if(!r(t))throw new TypeError("Cannot use null or undefined");return t}},{"./is-value":194}],206:[function(t,e,n){"use strict";e.exports=t("./is-implemented")()?String.prototype.contains:t("./shim")},{"./is-implemented":207,"./shim":208}],207:[function(t,e,n){"use strict";var r="razdwatrzy";e.exports=function(){return"function"==typeof r.contains&&!0===r.contains("dwa")&&!1===r.contains("foo")}},{}],208:[function(t,e,n){"use strict";var r=String.prototype.indexOf;e.exports=function(t){return r.call(this,t,arguments[1])>-1}},{}],209:[function(t,e,n){"use strict";var r=Object.prototype.toString,i=r.call("");e.exports=function(t){return"string"==typeof t||t&&"object"==typeof t&&(t instanceof String||r.call(t)===i)||!1}},{}],210:[function(t,e,n){"use strict";var r=Object.create(null),i=Math.random;e.exports=function(){var t;do{t=i().toString(36).slice(2)}while(r[t]);return t}},{}],211:[function(t,e,n){"use strict";var r,i=t("es5-ext/object/set-prototype-of"),o=t("es5-ext/string/#/contains"),a=t("d"),s=t("es6-symbol"),A=t("./"),l=Object.defineProperty;r=e.exports=function(t,e){if(!(this instanceof r))throw new TypeError("Constructor requires 'new'");A.call(this,t),e=e?o.call(e,"key+value")?"key+value":o.call(e,"key")?"key":"value":"value",l(this,"__kind__",a("",e))},i&&i(r,A),delete r.prototype.constructor,r.prototype=Object.create(A.prototype,{_resolve:a(function(t){return"value"===this.__kind__?this.__list__[t]:"key+value"===this.__kind__?[t,this.__list__[t]]:t})}),l(r.prototype,s.toStringTag,a("c","Array Iterator"))},{"./":214,d:151,"es5-ext/object/set-prototype-of":200,"es5-ext/string/#/contains":206,"es6-symbol":219}],212:[function(t,e,n){"use strict";var r=t("es5-ext/function/is-arguments"),i=t("es5-ext/object/valid-callable"),o=t("es5-ext/string/is-string"),a=t("./get"),s=Array.isArray,A=Function.prototype.call,l=Array.prototype.some;e.exports=function(t,e){var n,c,u,g,h,d,f,p,I=arguments[2];if(s(t)||r(t)?n="array":o(t)?n="string":t=a(t),i(e),u=function(){g=!0},"array"!==n)if("string"!==n)for(c=t.next();!c.done;){if(A.call(e,I,c.value,u),g)return;c=t.next()}else for(d=t.length,h=0;h<d&&(f=t[h],h+1<d&&(p=f.charCodeAt(0))>=55296&&p<=56319&&(f+=t[++h]),A.call(e,I,f,u),!g);++h);else l.call(t,function(t){return A.call(e,I,t,u),g})}},{"./get":213,"es5-ext/function/is-arguments":177,"es5-ext/object/valid-callable":203,"es5-ext/string/is-string":209}],213:[function(t,e,n){"use strict";var r=t("es5-ext/function/is-arguments"),i=t("es5-ext/string/is-string"),o=t("./array"),a=t("./string"),s=t("./valid-iterable"),A=t("es6-symbol").iterator;e.exports=function(t){return"function"==typeof s(t)[A]?t[A]():r(t)?new o(t):i(t)?new a(t):new o(t)}},{"./array":211,"./string":216,"./valid-iterable":217,"es5-ext/function/is-arguments":177,"es5-ext/string/is-string":209,"es6-symbol":219}],214:[function(t,e,n){"use strict";var r,i=t("es5-ext/array/#/clear"),o=t("es5-ext/object/assign"),a=t("es5-ext/object/valid-callable"),s=t("es5-ext/object/valid-value"),A=t("d"),l=t("d/auto-bind"),c=t("es6-symbol"),u=Object.defineProperty,g=Object.defineProperties;e.exports=r=function(t,e){if(!(this instanceof r))throw new TypeError("Constructor requires 'new'");g(this,{__list__:A("w",s(t)),__context__:A("w",e),__nextIndex__:A("w",0)}),e&&(a(e.on),e.on("_add",this._onAdd),e.on("_delete",this._onDelete),e.on("_clear",this._onClear))},delete r.prototype.constructor,g(r.prototype,o({_next:A(function(){var t;if(this.__list__)return this.__redo__&&void 0!==(t=this.__redo__.shift())?t:this.__nextIndex__<this.__list__.length?this.__nextIndex__++:void this._unBind()}),next:A(function(){return this._createResult(this._next())}),_createResult:A(function(t){return void 0===t?{done:!0,value:void 0}:{done:!1,value:this._resolve(t)}}),_resolve:A(function(t){return this.__list__[t]}),_unBind:A(function(){this.__list__=null,delete this.__redo__,this.__context__&&(this.__context__.off("_add",this._onAdd),this.__context__.off("_delete",this._onDelete),this.__context__.off("_clear",this._onClear),this.__context__=null)}),toString:A(function(){return"[object "+(this[c.toStringTag]||"Object")+"]"})},l({_onAdd:A(function(t){t>=this.__nextIndex__||(++this.__nextIndex__,this.__redo__?(this.__redo__.forEach(function(e,n){e>=t&&(this.__redo__[n]=++e)},this),this.__redo__.push(t)):u(this,"__redo__",A("c",[t])))}),_onDelete:A(function(t){var e;t>=this.__nextIndex__||(--this.__nextIndex__,this.__redo__&&(-1!==(e=this.__redo__.indexOf(t))&&this.__redo__.splice(e,1),this.__redo__.forEach(function(e,n){e>t&&(this.__redo__[n]=--e)},this)))}),_onClear:A(function(){this.__redo__&&i.call(this.__redo__),this.__nextIndex__=0})}))),u(r.prototype,c.iterator,A(function(){return this}))},{d:151,"d/auto-bind":150,"es5-ext/array/#/clear":173,"es5-ext/object/assign":186,"es5-ext/object/valid-callable":203,"es5-ext/object/valid-value":205,"es6-symbol":219}],215:[function(t,e,n){"use strict";var r=t("es5-ext/function/is-arguments"),i=t("es5-ext/object/is-value"),o=t("es5-ext/string/is-string"),a=t("es6-symbol").iterator,s=Array.isArray;e.exports=function(t){return!(!i(t)||!s(t)&&!o(t)&&!r(t)&&"function"!=typeof t[a])}},{"es5-ext/function/is-arguments":177,"es5-ext/object/is-value":194,"es5-ext/string/is-string":209,"es6-symbol":219}],216:[function(t,e,n){"use strict";var r,i=t("es5-ext/object/set-prototype-of"),o=t("d"),a=t("es6-symbol"),s=t("./"),A=Object.defineProperty;r=e.exports=function(t){if(!(this instanceof r))throw new TypeError("Constructor requires 'new'");t=String(t),s.call(this,t),A(this,"__length__",o("",t.length))},i&&i(r,s),delete r.prototype.constructor,r.prototype=Object.create(s.prototype,{_next:o(function(){if(this.__list__)return this.__nextIndex__<this.__length__?this.__nextIndex__++:void this._unBind()}),_resolve:o(function(t){var e,n=this.__list__[t];return this.__nextIndex__===this.__length__?n:(e=n.charCodeAt(0))>=55296&&e<=56319?n+this.__list__[this.__nextIndex__++]:n})}),A(r.prototype,a.toStringTag,o("c","String Iterator"))},{"./":214,d:151,"es5-ext/object/set-prototype-of":200,"es6-symbol":219}],217:[function(t,e,n){"use strict";var r=t("./is-iterable");e.exports=function(t){if(!r(t))throw new TypeError(t+" is not iterable");return t}},{"./is-iterable":215}],218:[function(t,n,r){(function(e,i){!function(t,e){"object"==typeof r&&void 0!==n?n.exports=e():t.ES6Promise=e()}(this,function(){"use strict";function n(t){return"function"==typeof t}var r=Array.isArray?Array.isArray:function(t){return"[object Array]"===Object.prototype.toString.call(t)},o=0,a=void 0,s=void 0,A=function(t,e){f[o]=t,f[o+1]=e,2===(o+=2)&&(s?s(p):E())},l="undefined"!=typeof window?window:void 0,c=l||{},u=c.MutationObserver||c.WebKitMutationObserver,g="undefined"==typeof self&&void 0!==e&&"[object process]"==={}.toString.call(e),h="undefined"!=typeof Uint8ClampedArray&&"undefined"!=typeof importScripts&&"undefined"!=typeof MessageChannel;function d(){var t=setTimeout;return function(){return t(p,1)}}var f=new Array(1e3);function p(){for(var t=0;t<o;t+=2)(0,f[t])(f[t+1]),f[t]=void 0,f[t+1]=void 0;o=0}var I,C,m,y,E=void 0;function v(t,e){var n=arguments,r=this,i=new this.constructor(Q);void 0===i[b]&&j(i);var o,a=r._state;return a?(o=n[a-1],A(function(){return G(a,i,o,r._result)})):F(r,i,t,e),i}function B(t){if(t&&"object"==typeof t&&t.constructor===this)return t;var e=new this(Q);return S(e,t),e}g?E=function(){return e.nextTick(p)}:u?(C=0,m=new u(p),y=document.createTextNode(""),m.observe(y,{characterData:!0}),E=function(){y.data=C=++C%2}):h?((I=new MessageChannel).port1.onmessage=p,E=function(){return I.port2.postMessage(0)}):E=void 0===l&&"function"==typeof t?function(){try{var e=t("vertx");return a=e.runOnLoop||e.runOnContext,function(){a(p)}}catch(t){return d()}}():d();var b=Math.random().toString(36).substring(16);function Q(){}var w=void 0,x=1,_=2,D=new U;function M(t){try{return t.then}catch(t){return D.error=t,D}}function k(t,e,r){e.constructor===t.constructor&&r===v&&e.constructor.resolve===B?function(t,e){e._state===x?R(t,e._result):e._state===_?N(t,e._result):F(e,void 0,function(e){return S(t,e)},function(e){return N(t,e)})}(t,e):r===D?N(t,D.error):void 0===r?R(t,e):n(r)?function(t,e,n){A(function(t){var r=!1,i=function(t,e,n,r){try{t.call(e,n,r)}catch(t){return t}}(n,e,function(n){r||(r=!0,e!==n?S(t,n):R(t,n))},function(e){r||(r=!0,N(t,e))},t._label);!r&&i&&(r=!0,N(t,i))},t)}(t,e,r):R(t,e)}function S(t,e){var n;t===e?N(t,new TypeError("You cannot resolve a promise with itself")):"function"==typeof(n=e)||"object"==typeof n&&null!==n?k(t,e,M(e)):R(t,e)}function L(t){t._onerror&&t._onerror(t._result),T(t)}function R(t,e){t._state===w&&(t._result=e,t._state=x,0!==t._subscribers.length&&A(T,t))}function N(t,e){t._state===w&&(t._state=_,t._result=e,A(L,t))}function F(t,e,n,r){var i=t._subscribers,o=i.length;t._onerror=null,i[o]=e,i[o+x]=n,i[o+_]=r,0===o&&t._state&&A(T,t)}function T(t){var e=t._subscribers,n=t._state;if(0!==e.length){for(var r=void 0,i=void 0,o=t._result,a=0;a<e.length;a+=3)r=e[a],i=e[a+n],r?G(n,r,i,o):i(o);t._subscribers.length=0}}function U(){this.error=null}var O=new U;function G(t,e,r,i){var o=n(r),a=void 0,s=void 0,A=void 0,l=void 0;if(o){if((a=function(t,e){try{return t(e)}catch(t){return O.error=t,O}}(r,i))===O?(l=!0,s=a.error,a=null):A=!0,e===a)return void N(e,new TypeError("A promises callback cannot return that same promise."))}else a=i,A=!0;e._state!==w||(o&&A?S(e,a):l?N(e,s):t===x?R(e,a):t===_&&N(e,a))}var H=0;function j(t){t[b]=H++,t._state=void 0,t._result=void 0,t._subscribers=[]}function Y(t,e){this._instanceConstructor=t,this.promise=new t(Q),this.promise[b]||j(this.promise),r(e)?(this._input=e,this.length=e.length,this._remaining=e.length,this._result=new Array(this.length),0===this.length?R(this.promise,this._result):(this.length=this.length||0,this._enumerate(),0===this._remaining&&R(this.promise,this._result))):N(this.promise,new Error("Array Methods must be provided an Array"))}function P(t){this[b]=H++,this._result=this._state=void 0,this._subscribers=[],Q!==t&&("function"!=typeof t&&function(){throw new TypeError("You must pass a resolver function as the first argument to the promise constructor")}(),this instanceof P?function(t,e){try{e(function(e){S(t,e)},function(e){N(t,e)})}catch(e){N(t,e)}}(this,t):function(){throw new TypeError("Failed to construct 'Promise': Please use the 'new' operator, this object constructor cannot be called as a function.")}())}function K(){var t=void 0;if(void 0!==i)t=i;else if("undefined"!=typeof self)t=self;else try{t=Function("return this")()}catch(t){throw new Error("polyfill failed because global object is unavailable in this environment")}var e=t.Promise;if(e){var n=null;try{n=Object.prototype.toString.call(e.resolve())}catch(t){}if("[object Promise]"===n&&!e.cast)return}t.Promise=P}return Y.prototype._enumerate=function(){for(var t=this.length,e=this._input,n=0;this._state===w&&n<t;n++)this._eachEntry(e[n],n)},Y.prototype._eachEntry=function(t,e){var n=this._instanceConstructor,r=n.resolve;if(r===B){var i=M(t);if(i===v&&t._state!==w)this._settledAt(t._state,e,t._result);else if("function"!=typeof i)this._remaining--,this._result[e]=t;else if(n===P){var o=new n(Q);k(o,t,i),this._willSettleAt(o,e)}else this._willSettleAt(new n(function(e){return e(t)}),e)}else this._willSettleAt(r(t),e)},Y.prototype._settledAt=function(t,e,n){var r=this.promise;r._state===w&&(this._remaining--,t===_?N(r,n):this._result[e]=n),0===this._remaining&&R(r,this._result)},Y.prototype._willSettleAt=function(t,e){var n=this;F(t,void 0,function(t){return n._settledAt(x,e,t)},function(t){return n._settledAt(_,e,t)})},P.all=function(t){return new Y(this,t).promise},P.race=function(t){var e=this;return r(t)?new e(function(n,r){for(var i=t.length,o=0;o<i;o++)e.resolve(t[o]).then(n,r)}):new e(function(t,e){return e(new TypeError("You must pass an array to race."))})},P.resolve=B,P.reject=function(t){var e=new this(Q);return N(e,t),e},P._setScheduler=function(t){s=t},P._setAsap=function(t){A=t},P._asap=A,P.prototype={constructor:P,then:v,catch:function(t){return this.then(null,t)}},K(),P.polyfill=K,P.Promise=P,P})}).call(this,t("_process"),void 0!==e?e:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{_process:482}],219:[function(t,e,n){"use strict";e.exports=t("./is-implemented")()?Symbol:t("./polyfill")},{"./is-implemented":220,"./polyfill":222}],220:[function(t,e,n){"use strict";var r={object:!0,symbol:!0};e.exports=function(){var t;if("function"!=typeof Symbol)return!1;t=Symbol("test symbol");try{String(t)}catch(t){return!1}return!!r[typeof Symbol.iterator]&&!!r[typeof Symbol.toPrimitive]&&!!r[typeof Symbol.toStringTag]}},{}],221:[function(t,e,n){"use strict";e.exports=function(t){return!!t&&("symbol"==typeof t||!!t.constructor&&"Symbol"===t.constructor.name&&"Symbol"===t[t.constructor.toStringTag])}},{}],222:[function(t,e,n){"use strict";var r,i,o,a,s=t("d"),A=t("./validate-symbol"),l=Object.create,c=Object.defineProperties,u=Object.defineProperty,g=Object.prototype,h=l(null);if("function"==typeof Symbol){r=Symbol;try{String(r()),a=!0}catch(t){}}var d,f=(d=l(null),function(t){for(var e,n,r=0;d[t+(r||"")];)++r;return d[t+=r||""]=!0,u(g,e="@@"+t,s.gs(null,function(t){n||(n=!0,u(this,e,s(t)),n=!1)})),e});o=function(t){if(this instanceof o)throw new TypeError("Symbol is not a constructor");return i(t)},e.exports=i=function t(e){var n;if(this instanceof t)throw new TypeError("Symbol is not a constructor");return a?r(e):(n=l(o.prototype),e=void 0===e?"":String(e),c(n,{__description__:s("",e),__name__:s("",f(e))}))},c(i,{for:s(function(t){return h[t]?h[t]:h[t]=i(String(t))}),keyFor:s(function(t){var e;for(e in A(t),h)if(h[e]===t)return e}),hasInstance:s("",r&&r.hasInstance||i("hasInstance")),isConcatSpreadable:s("",r&&r.isConcatSpreadable||i("isConcatSpreadable")),iterator:s("",r&&r.iterator||i("iterator")),match:s("",r&&r.match||i("match")),replace:s("",r&&r.replace||i("replace")),search:s("",r&&r.search||i("search")),species:s("",r&&r.species||i("species")),split:s("",r&&r.split||i("split")),toPrimitive:s("",r&&r.toPrimitive||i("toPrimitive")),toStringTag:s("",r&&r.toStringTag||i("toStringTag")),unscopables:s("",r&&r.unscopables||i("unscopables"))}),c(o.prototype,{constructor:s(i),toString:s("",function(){return this.__name__})}),c(i.prototype,{toString:s(function(){return"Symbol ("+A(this).__description__+")"}),valueOf:s(function(){return A(this)})}),u(i.prototype,i.toPrimitive,s("",function(){var t=A(this);return"symbol"==typeof t?t:t.toString()})),u(i.prototype,i.toStringTag,s("c","Symbol")),u(o.prototype,i.toStringTag,s("c",i.prototype[i.toStringTag])),u(o.prototype,i.toPrimitive,s("c",i.prototype[i.toPrimitive]))},{"./validate-symbol":223,d:151}],223:[function(t,e,n){"use strict";var r=t("./is-symbol");e.exports=function(t){if(!r(t))throw new TypeError(t+" is not a symbol");return t}},{"./is-symbol":221}],224:[function(t,e,n){"use strict";e.exports=function(t,e,n){var r=e||0,i=n||1;return[[t[12]+t[0],t[13]+t[1],t[14]+t[2],t[15]+t[3]],[t[12]-t[0],t[13]-t[1],t[14]-t[2],t[15]-t[3]],[t[12]+t[4],t[13]+t[5],t[14]+t[6],t[15]+t[7]],[t[12]-t[4],t[13]-t[5],t[14]-t[6],t[15]-t[7]],[r*t[12]+t[8],r*t[13]+t[9],r*t[14]+t[10],r*t[15]+t[11]],[i*t[12]-t[8],i*t[13]-t[9],i*t[14]-t[10],i*t[15]-t[11]]]}},{}],225:[function(t,e,n){"use strict";var r=t("is-string-blank");e.exports=function(t){var e=typeof t;if("string"===e){var n=t;if(0==(t=+t)&&r(n))return!1}else if("number"!==e)return!1;return t-t<1}},{"is-string-blank":423}],226:[function(t,e,n){"use strict";e.exports=function(t,e,n){switch(arguments.length){case 0:return new a([0],[0],0);case 1:if("number"==typeof t){var r=A(t);return new a(r,r,0)}return new a(t,A(t.length),0);case 2:if("number"==typeof e){var r=A(t.length);return new a(t,r,+e)}n=0;case 3:if(t.length!==e.length)throw new Error("state and velocity lengths must match");return new a(t,e,n)}};var r=t("cubic-hermite"),i=t("binary-search-bounds");function o(t,e,n){return Math.min(e,Math.max(t,n))}function a(t,e,n){this.dimension=t.length,this.bounds=[new Array(this.dimension),new Array(this.dimension)];for(var r=0;r<this.dimension;++r)this.bounds[0][r]=-1/0,this.bounds[1][r]=1/0;this._state=t.slice().reverse(),this._velocity=e.slice().reverse(),this._time=[n],this._scratch=[t.slice(),t.slice(),t.slice(),t.slice(),t.slice()]}var s=a.prototype;function A(t){for(var e=new Array(t),n=0;n<t;++n)e[n]=0;return e}s.flush=function(t){var e=i.gt(this._time,t)-1;e<=0||(this._time.splice(0,e),this._state.splice(0,e*this.dimension),this._velocity.splice(0,e*this.dimension))},s.curve=function(t){var e=this._time,n=e.length,a=i.le(e,t),s=this._scratch[0],A=this._state,l=this._velocity,c=this.dimension,u=this.bounds;if(a<0)for(var g=c-1,h=0;h<c;++h,--g)s[h]=A[g];else if(a>=n-1){g=A.length-1;var d=t-e[n-1];for(h=0;h<c;++h,--g)s[h]=A[g]+d*l[g]}else{g=c*(a+1)-1;var f=e[a],p=e[a+1]-f||1,I=this._scratch[1],C=this._scratch[2],m=this._scratch[3],y=this._scratch[4],E=!0;for(h=0;h<c;++h,--g)I[h]=A[g],m[h]=l[g]*p,C[h]=A[g+c],y[h]=l[g+c]*p,E=E&&I[h]===C[h]&&m[h]===y[h]&&0===m[h];if(E)for(h=0;h<c;++h)s[h]=I[h];else r(I,m,C,y,(t-f)/p,s)}var v=u[0],B=u[1];for(h=0;h<c;++h)s[h]=o(v[h],B[h],s[h]);return s},s.dcurve=function(t){var e=this._time,n=e.length,o=i.le(e,t),a=this._scratch[0],s=this._state,A=this._velocity,l=this.dimension;if(o>=n-1)for(var c=s.length-1,u=(e[n-1],0);u<l;++u,--c)a[u]=A[c];else{c=l*(o+1)-1;var g=e[o],h=e[o+1]-g||1,d=this._scratch[1],f=this._scratch[2],p=this._scratch[3],I=this._scratch[4],C=!0;for(u=0;u<l;++u,--c)d[u]=s[c],p[u]=A[c]*h,f[u]=s[c+l],I[u]=A[c+l]*h,C=C&&d[u]===f[u]&&p[u]===I[u]&&0===p[u];if(C)for(u=0;u<l;++u)a[u]=0;else for(r.derivative(d,p,f,I,(t-g)/h,a),u=0;u<l;++u)a[u]/=h}return a},s.lastT=function(){var t=this._time;return t[t.length-1]},s.stable=function(){for(var t=this._velocity,e=t.length,n=this.dimension-1;n>=0;--n)if(t[--e])return!1;return!0},s.jump=function(t){var e=this.lastT(),n=this.dimension;if(!(t<e||arguments.length!==n+1)){var r=this._state,i=this._velocity,a=r.length-this.dimension,s=this.bounds,A=s[0],l=s[1];this._time.push(e,t);for(var c=0;c<2;++c)for(var u=0;u<n;++u)r.push(r[a++]),i.push(0);for(this._time.push(t),u=n;u>0;--u)r.push(o(A[u-1],l[u-1],arguments[u])),i.push(0)}},s.push=function(t){var e=this.lastT(),n=this.dimension;if(!(t<e||arguments.length!==n+1)){var r=this._state,i=this._velocity,a=r.length-this.dimension,s=t-e,A=this.bounds,l=A[0],c=A[1],u=s>1e-6?1/s:0;this._time.push(t);for(var g=n;g>0;--g){var h=o(l[g-1],c[g-1],arguments[g]);r.push(h),i.push((h-r[a++])*u)}}},s.set=function(t){var e=this.dimension;if(!(t<this.lastT()||arguments.length!==e+1)){var n=this._state,r=this._velocity,i=this.bounds,a=i[0],s=i[1];this._time.push(t);for(var A=e;A>0;--A)n.push(o(a[A-1],s[A-1],arguments[A])),r.push(0)}},s.move=function(t){var e=this.lastT(),n=this.dimension;if(!(t<=e||arguments.length!==n+1)){var r=this._state,i=this._velocity,a=r.length-this.dimension,s=this.bounds,A=s[0],l=s[1],c=t-e,u=c>1e-6?1/c:0;this._time.push(t);for(var g=n;g>0;--g){var h=arguments[g];r.push(o(A[g-1],l[g-1],r[a++]+h)),i.push(h*u)}}},s.idle=function(t){var e=this.lastT();if(!(t<e)){var n=this.dimension,r=this._state,i=this._velocity,a=r.length-n,s=this.bounds,A=s[0],l=s[1],c=t-e;this._time.push(t);for(var u=n-1;u>=0;--u)r.push(o(A[u],l[u],r[a]+c*i[a])),i.push(0),a+=1}}},{"binary-search-bounds":91,"cubic-hermite":145}],227:[function(t,e,n){var r=t("dtype");e.exports=function(t,e,n){if(!t)throw new TypeError("must specify data as first parameter");if(n=0|+(n||0),Array.isArray(t)&&t[0]&&"number"==typeof t[0][0]){var i,o,a,s,A=t[0].length,l=t.length*A;e&&"string"!=typeof e||(e=new(r(e||"float32"))(l+n));var c=e.length-n;if(l!==c)throw new Error("source length "+l+" ("+A+"x"+t.length+") does not match destination length "+c);for(i=0,a=n;i<t.length;i++)for(o=0;o<A;o++)e[a++]=null===t[i][o]?NaN:t[i][o]}else if(e&&"string"!=typeof e)e.set(t,n);else{var u=r(e||"float32");if(Array.isArray(t)||"array"===e)for(e=new u(t.length+n),i=0,a=n,s=e.length;a<s;a++,i++)e[a]=null===t[i]?NaN:t[i];else 0===n?e=new u(t):(e=new u(t.length+n)).set(t,n)}return e}},{dtype:169}],228:[function(t,e,n){"use strict";var r=t("css-font/stringify"),i=[32,126];e.exports=function(t){var e=(t=t||{}).shape?t.shape:t.canvas?[t.canvas.width,t.canvas.height]:[512,512],n=t.canvas||document.createElement("canvas"),o=t.font,a="number"==typeof t.step?[t.step,t.step]:t.step||[32,32],s=t.chars||i;if(o&&"string"!=typeof o&&(o=r(o)),Array.isArray(s)){if(2===s.length&&"number"==typeof s[0]&&"number"==typeof s[1]){for(var A=[],l=s[0],c=0;l<=s[1];l++)A[c++]=String.fromCharCode(l);s=A}}else s=String(s).split("");e=e.slice(),n.width=e[0],n.height=e[1];var u=n.getContext("2d");u.fillStyle="#000",u.fillRect(0,0,n.width,n.height),u.font=o,u.textAlign="center",u.textBaseline="middle",u.fillStyle="#fff";for(var g=a[0]/2,h=a[1]/2,l=0;l<s.length;l++)u.fillText(s[l],g,h),(g+=a[0])>e[0]-a[0]/2&&(g=a[0]/2,h+=a[1]);return n}},{"css-font/stringify":142}],229:[function(t,e,n){"use strict";function r(t,e){e||(e={}),("string"==typeof t||Array.isArray(t))&&(e.family=t);var n=Array.isArray(e.family)?e.family.join(", "):e.family;if(!n)throw Error("`family` must be defined");var s=e.size||e.fontSize||e.em||48,A=e.weight||e.fontWeight||"",l=(t=[e.style||e.fontStyle||"",A,s].join(" ")+"px "+n,e.origin||"top");if(r.cache[n]&&s<=r.cache[n].em)return i(r.cache[n],l);var c=e.canvas||r.canvas,u=c.getContext("2d"),g={upper:void 0!==e.upper?e.upper:"H",lower:void 0!==e.lower?e.lower:"x",descent:void 0!==e.descent?e.descent:"p",ascent:void 0!==e.ascent?e.ascent:"h",tittle:void 0!==e.tittle?e.tittle:"i",overshoot:void 0!==e.overshoot?e.overshoot:"O"},h=Math.ceil(1.5*s);c.height=h,c.width=.5*h,u.font=t;var d={top:0};u.clearRect(0,0,h,h),u.textBaseline="top",u.fillStyle="black",u.fillText("H",0,0);var f=o(u.getImageData(0,0,h,h));u.clearRect(0,0,h,h),u.textBaseline="bottom",u.fillText("H",0,h);var p=o(u.getImageData(0,0,h,h));d.lineHeight=d.bottom=h-p+f,u.clearRect(0,0,h,h),u.textBaseline="alphabetic",u.fillText("H",0,h);var I=h-o(u.getImageData(0,0,h,h))-1+f;d.baseline=d.alphabetic=I,u.clearRect(0,0,h,h),u.textBaseline="middle",u.fillText("H",0,.5*h);var C=o(u.getImageData(0,0,h,h));d.median=d.middle=h-C-1+f-.5*h,u.clearRect(0,0,h,h),u.textBaseline="hanging",u.fillText("H",0,.5*h);var m=o(u.getImageData(0,0,h,h));d.hanging=h-m-1+f-.5*h,u.clearRect(0,0,h,h),u.textBaseline="ideographic",u.fillText("H",0,h);var y=o(u.getImageData(0,0,h,h));if(d.ideographic=h-y-1+f,g.upper&&(u.clearRect(0,0,h,h),u.textBaseline="top",u.fillText(g.upper,0,0),d.upper=o(u.getImageData(0,0,h,h)),d.capHeight=d.baseline-d.upper),g.lower&&(u.clearRect(0,0,h,h),u.textBaseline="top",u.fillText(g.lower,0,0),d.lower=o(u.getImageData(0,0,h,h)),d.xHeight=d.baseline-d.lower),g.tittle&&(u.clearRect(0,0,h,h),u.textBaseline="top",u.fillText(g.tittle,0,0),d.tittle=o(u.getImageData(0,0,h,h))),g.ascent&&(u.clearRect(0,0,h,h),u.textBaseline="top",u.fillText(g.ascent,0,0),d.ascent=o(u.getImageData(0,0,h,h))),g.descent&&(u.clearRect(0,0,h,h),u.textBaseline="top",u.fillText(g.descent,0,0),d.descent=a(u.getImageData(0,0,h,h))),g.overshoot){u.clearRect(0,0,h,h),u.textBaseline="top",u.fillText(g.overshoot,0,0);var E=a(u.getImageData(0,0,h,h));d.overshoot=E-I}for(var v in d)d[v]/=s;return d.em=s,r.cache[n]=d,i(d,l)}function i(t,e){var n={};for(var r in"string"==typeof e&&(e=t[e]),t)"em"!==r&&(n[r]=t[r]-e);return n}function o(t){for(var e=t.height,n=t.data,r=3;r<n.length;r+=4)if(0!==n[r])return Math.floor(.25*(r-3)/e)}function a(t){for(var e=t.height,n=t.data,r=n.length-1;r>0;r-=4)if(0!==n[r])return Math.floor(.25*(r-3)/e)}e.exports=r,r.canvas=document.createElement("canvas"),r.cache={}},{}],230:[function(t,e,n){"use strict";function r(t,e,n,r,i,o){this._color=t,this.key=e,this.value=n,this.left=r,this.right=i,this._count=o}function i(t){return new r(t._color,t.key,t.value,t.left,t.right,t._count)}function o(t,e){return new r(t,e.key,e.value,e.left,e.right,e._count)}function a(t){t._count=1+(t.left?t.left._count:0)+(t.right?t.right._count:0)}function s(t,e){this._compare=t,this.root=e}e.exports=function(t){return new s(t||g,null)};var A=s.prototype;function l(t,e){this.tree=t,this._stack=e}Object.defineProperty(A,"keys",{get:function(){var t=[];return this.forEach(function(e,n){t.push(e)}),t}}),Object.defineProperty(A,"values",{get:function(){var t=[];return this.forEach(function(e,n){t.push(n)}),t}}),Object.defineProperty(A,"length",{get:function(){return this.root?this.root._count:0}}),A.insert=function(t,e){for(var n=this._compare,i=this.root,A=[],l=[];i;){var c=n(t,i.key);A.push(i),l.push(c),i=c<=0?i.left:i.right}A.push(new r(0,t,e,null,null,1));for(var u=A.length-2;u>=0;--u)i=A[u],l[u]<=0?A[u]=new r(i._color,i.key,i.value,A[u+1],i.right,i._count+1):A[u]=new r(i._color,i.key,i.value,i.left,A[u+1],i._count+1);for(u=A.length-1;u>1;--u){var g=A[u-1];if(i=A[u],1===g._color||1===i._color)break;var h=A[u-2];if(h.left===g)if(g.left===i){if(!(d=h.right)||0!==d._color){h._color=0,h.left=g.right,g._color=1,g.right=h,A[u-2]=g,A[u-1]=i,a(h),a(g),u>=3&&((f=A[u-3]).left===h?f.left=g:f.right=g);break}g._color=1,h.right=o(1,d),h._color=0,u-=1}else{if(!(d=h.right)||0!==d._color){g.right=i.left,h._color=0,h.left=i.right,i._color=1,i.left=g,i.right=h,A[u-2]=i,A[u-1]=g,a(h),a(g),a(i),u>=3&&((f=A[u-3]).left===h?f.left=i:f.right=i);break}g._color=1,h.right=o(1,d),h._color=0,u-=1}else if(g.right===i){if(!(d=h.left)||0!==d._color){h._color=0,h.right=g.left,g._color=1,g.left=h,A[u-2]=g,A[u-1]=i,a(h),a(g),u>=3&&((f=A[u-3]).right===h?f.right=g:f.left=g);break}g._color=1,h.left=o(1,d),h._color=0,u-=1}else{var d;if(!(d=h.left)||0!==d._color){var f;g.left=i.right,h._color=0,h.right=i.left,i._color=1,i.right=g,i.left=h,A[u-2]=i,A[u-1]=g,a(h),a(g),a(i),u>=3&&((f=A[u-3]).right===h?f.right=i:f.left=i);break}g._color=1,h.left=o(1,d),h._color=0,u-=1}}return A[0]._color=1,new s(n,A[0])},A.forEach=function(t,e,n){if(this.root)switch(arguments.length){case 1:return function t(e,n){var r;return n.left&&(r=t(e,n.left))?r:(r=e(n.key,n.value))||(n.right?t(e,n.right):void 0)}(t,this.root);case 2:return function t(e,n,r,i){if(n(e,i.key)<=0){var o;if(i.left&&(o=t(e,n,r,i.left)))return o;if(o=r(i.key,i.value))return o}if(i.right)return t(e,n,r,i.right)}(e,this._compare,t,this.root);case 3:if(this._compare(e,n)>=0)return;return function t(e,n,r,i,o){var a,s=r(e,o.key),A=r(n,o.key);if(s<=0){if(o.left&&(a=t(e,n,r,i,o.left)))return a;if(A>0&&(a=i(o.key,o.value)))return a}if(A>0&&o.right)return t(e,n,r,i,o.right)}(e,n,this._compare,t,this.root)}},Object.defineProperty(A,"begin",{get:function(){for(var t=[],e=this.root;e;)t.push(e),e=e.left;return new l(this,t)}}),Object.defineProperty(A,"end",{get:function(){for(var t=[],e=this.root;e;)t.push(e),e=e.right;return new l(this,t)}}),A.at=function(t){if(t<0)return new l(this,[]);for(var e=this.root,n=[];;){if(n.push(e),e.left){if(t<e.left._count){e=e.left;continue}t-=e.left._count}if(!t)return new l(this,n);if(t-=1,!e.right)break;if(t>=e.right._count)break;e=e.right}return new l(this,[])},A.ge=function(t){for(var e=this._compare,n=this.root,r=[],i=0;n;){var o=e(t,n.key);r.push(n),o<=0&&(i=r.length),n=o<=0?n.left:n.right}return r.length=i,new l(this,r)},A.gt=function(t){for(var e=this._compare,n=this.root,r=[],i=0;n;){var o=e(t,n.key);r.push(n),o<0&&(i=r.length),n=o<0?n.left:n.right}return r.length=i,new l(this,r)},A.lt=function(t){for(var e=this._compare,n=this.root,r=[],i=0;n;){var o=e(t,n.key);r.push(n),o>0&&(i=r.length),n=o<=0?n.left:n.right}return r.length=i,new l(this,r)},A.le=function(t){for(var e=this._compare,n=this.root,r=[],i=0;n;){var o=e(t,n.key);r.push(n),o>=0&&(i=r.length),n=o<0?n.left:n.right}return r.length=i,new l(this,r)},A.find=function(t){for(var e=this._compare,n=this.root,r=[];n;){var i=e(t,n.key);if(r.push(n),0===i)return new l(this,r);n=i<=0?n.left:n.right}return new l(this,[])},A.remove=function(t){var e=this.find(t);return e?e.remove():this},A.get=function(t){for(var e=this._compare,n=this.root;n;){var r=e(t,n.key);if(0===r)return n.value;n=r<=0?n.left:n.right}};var c=l.prototype;function u(t,e){t.key=e.key,t.value=e.value,t.left=e.left,t.right=e.right,t._color=e._color,t._count=e._count}function g(t,e){return t<e?-1:t>e?1:0}Object.defineProperty(c,"valid",{get:function(){return this._stack.length>0}}),Object.defineProperty(c,"node",{get:function(){return this._stack.length>0?this._stack[this._stack.length-1]:null},enumerable:!0}),c.clone=function(){return new l(this.tree,this._stack.slice())},c.remove=function(){var t=this._stack;if(0===t.length)return this.tree;var e=new Array(t.length),n=t[t.length-1];e[e.length-1]=new r(n._color,n.key,n.value,n.left,n.right,n._count);for(var A=t.length-2;A>=0;--A)(n=t[A]).left===t[A+1]?e[A]=new r(n._color,n.key,n.value,e[A+1],n.right,n._count):e[A]=new r(n._color,n.key,n.value,n.left,e[A+1],n._count);if((n=e[e.length-1]).left&&n.right){var l=e.length;for(n=n.left;n.right;)e.push(n),n=n.right;var c=e[l-1];for(e.push(new r(n._color,c.key,c.value,n.left,n.right,n._count)),e[l-1].key=n.key,e[l-1].value=n.value,A=e.length-2;A>=l;--A)n=e[A],e[A]=new r(n._color,n.key,n.value,n.left,e[A+1],n._count);e[l-1].left=e[l]}if(0===(n=e[e.length-1])._color){var g=e[e.length-2];for(g.left===n?g.left=null:g.right===n&&(g.right=null),e.pop(),A=0;A<e.length;++A)e[A]._count--;return new s(this.tree._compare,e[0])}if(n.left||n.right){for(n.left?u(n,n.left):n.right&&u(n,n.right),n._color=1,A=0;A<e.length-1;++A)e[A]._count--;return new s(this.tree._compare,e[0])}if(1===e.length)return new s(this.tree._compare,null);for(A=0;A<e.length;++A)e[A]._count--;var h=e[e.length-2];return function(t){for(var e,n,r,s,A=t.length-1;A>=0;--A){if(e=t[A],0===A)return void(e._color=1);if((n=t[A-1]).left===e){if((r=n.right).right&&0===r.right._color)return s=(r=n.right=i(r)).right=i(r.right),n.right=r.left,r.left=n,r.right=s,r._color=n._color,e._color=1,n._color=1,s._color=1,a(n),a(r),A>1&&((l=t[A-2]).left===n?l.left=r:l.right=r),void(t[A-1]=r);if(r.left&&0===r.left._color)return s=(r=n.right=i(r)).left=i(r.left),n.right=s.left,r.left=s.right,s.left=n,s.right=r,s._color=n._color,n._color=1,r._color=1,e._color=1,a(n),a(r),a(s),A>1&&((l=t[A-2]).left===n?l.left=s:l.right=s),void(t[A-1]=s);if(1===r._color){if(0===n._color)return n._color=1,void(n.right=o(0,r));n.right=o(0,r);continue}r=i(r),n.right=r.left,r.left=n,r._color=n._color,n._color=0,a(n),a(r),A>1&&((l=t[A-2]).left===n?l.left=r:l.right=r),t[A-1]=r,t[A]=n,A+1<t.length?t[A+1]=e:t.push(e),A+=2}else{if((r=n.left).left&&0===r.left._color)return s=(r=n.left=i(r)).left=i(r.left),n.left=r.right,r.right=n,r.left=s,r._color=n._color,e._color=1,n._color=1,s._color=1,a(n),a(r),A>1&&((l=t[A-2]).right===n?l.right=r:l.left=r),void(t[A-1]=r);if(r.right&&0===r.right._color)return s=(r=n.left=i(r)).right=i(r.right),n.left=s.right,r.right=s.left,s.right=n,s.left=r,s._color=n._color,n._color=1,r._color=1,e._color=1,a(n),a(r),a(s),A>1&&((l=t[A-2]).right===n?l.right=s:l.left=s),void(t[A-1]=s);if(1===r._color){if(0===n._color)return n._color=1,void(n.left=o(0,r));n.left=o(0,r);continue}var l;r=i(r),n.left=r.right,r.right=n,r._color=n._color,n._color=0,a(n),a(r),A>1&&((l=t[A-2]).right===n?l.right=r:l.left=r),t[A-1]=r,t[A]=n,A+1<t.length?t[A+1]=e:t.push(e),A+=2}}}(e),h.left===n?h.left=null:h.right=null,new s(this.tree._compare,e[0])},Object.defineProperty(c,"key",{get:function(){if(this._stack.length>0)return this._stack[this._stack.length-1].key},enumerable:!0}),Object.defineProperty(c,"value",{get:function(){if(this._stack.length>0)return this._stack[this._stack.length-1].value},enumerable:!0}),Object.defineProperty(c,"index",{get:function(){var t=0,e=this._stack;if(0===e.length){var n=this.tree.root;return n?n._count:0}e[e.length-1].left&&(t=e[e.length-1].left._count);for(var r=e.length-2;r>=0;--r)e[r+1]===e[r].right&&(++t,e[r].left&&(t+=e[r].left._count));return t},enumerable:!0}),c.next=function(){var t=this._stack;if(0!==t.length){var e=t[t.length-1];if(e.right)for(e=e.right;e;)t.push(e),e=e.left;else for(t.pop();t.length>0&&t[t.length-1].right===e;)e=t[t.length-1],t.pop()}},Object.defineProperty(c,"hasNext",{get:function(){var t=this._stack;if(0===t.length)return!1;if(t[t.length-1].right)return!0;for(var e=t.length-1;e>0;--e)if(t[e-1].left===t[e])return!0;return!1}}),c.update=function(t){var e=this._stack;if(0===e.length)throw new Error("Can't update empty node!");var n=new Array(e.length),i=e[e.length-1];n[n.length-1]=new r(i._color,i.key,t,i.left,i.right,i._count);for(var o=e.length-2;o>=0;--o)(i=e[o]).left===e[o+1]?n[o]=new r(i._color,i.key,i.value,n[o+1],i.right,i._count):n[o]=new r(i._color,i.key,i.value,i.left,n[o+1],i._count);return new s(this.tree._compare,n[0])},c.prev=function(){var t=this._stack;if(0!==t.length){var e=t[t.length-1];if(e.left)for(e=e.left;e;)t.push(e),e=e.right;else for(t.pop();t.length>0&&t[t.length-1].left===e;)e=t[t.length-1],t.pop()}},Object.defineProperty(c,"hasPrev",{get:function(){var t=this._stack;if(0===t.length)return!1;if(t[t.length-1].left)return!0;for(var e=t.length-1;e>0;--e)if(t[e-1].right===t[e])return!0;return!1}})},{}],231:[function(t,e,n){var r=[.9999999999998099,676.5203681218851,-1259.1392167224028,771.3234287776531,-176.6150291621406,12.507343278686905,-.13857109526572012,9984369578019572e-21,1.5056327351493116e-7],i=607/128,o=[.9999999999999971,57.15623566586292,-59.59796035547549,14.136097974741746,-.4919138160976202,3399464998481189e-20,4652362892704858e-20,-9837447530487956e-20,.0001580887032249125,-.00021026444172410488,.00021743961811521265,-.0001643181065367639,8441822398385275e-20,-26190838401581408e-21,36899182659531625e-22];function a(t){if(t<0)return Number("0/0");for(var e=o[0],n=o.length-1;n>0;--n)e+=o[n]/(t+n);var r=t+i+.5;return.5*Math.log(2*Math.PI)+(t+.5)*Math.log(r)-r+Math.log(e)-Math.log(t)}e.exports=function t(e){if(e<.5)return Math.PI/(Math.sin(Math.PI*e)*t(1-e));if(e>100)return Math.exp(a(e));e-=1;for(var n=r[0],i=1;i<9;i++)n+=r[i]/(e+i);var o=e+7+.5;return Math.sqrt(2*Math.PI)*Math.pow(o,e+.5)*Math.exp(-o)*n},e.exports.log=a},{}],232:[function(t,e,n){e.exports=function(t,e){if("string"!=typeof t)throw new TypeError("must specify type string");if(e=e||{},"undefined"==typeof document&&!e.canvas)return null;var n=e.canvas||document.createElement("canvas");"number"==typeof e.width&&(n.width=e.width),"number"==typeof e.height&&(n.height=e.height);var r,i=e;try{var o=[t];0===t.indexOf("webgl")&&o.push("experimental-"+t);for(var a=0;a<o.length;a++)if(r=n.getContext(o[a],i))return r}catch(t){r=null}return r||null}},{}],233:[function(t,e,n){"use strict";e.exports=function(t,e){var n=new c(t);return n.update(e),n};var r=t("./lib/text.js"),i=t("./lib/lines.js"),o=t("./lib/background.js"),a=t("./lib/cube.js"),s=t("./lib/ticks.js"),A=new Float32Array([1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1]);function l(t,e){return t[0]=e[0],t[1]=e[1],t[2]=e[2],t}function c(t){this.gl=t,this.pixelRatio=1,this.bounds=[[-10,-10,-10],[10,10,10]],this.ticks=[[],[],[]],this.autoTicks=!0,this.tickSpacing=[1,1,1],this.tickEnable=[!0,!0,!0],this.tickFont=["sans-serif","sans-serif","sans-serif"],this.tickSize=[12,12,12],this.tickAngle=[0,0,0],this.tickAlign=["auto","auto","auto"],this.tickColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.tickPad=[10,10,10],this.lastCubeProps={cubeEdges:[0,0,0],axis:[0,0,0]},this.labels=["x","y","z"],this.labelEnable=[!0,!0,!0],this.labelFont="sans-serif",this.labelSize=[20,20,20],this.labelAngle=[0,0,0],this.labelAlign=["auto","auto","auto"],this.labelColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.labelPad=[10,10,10],this.lineEnable=[!0,!0,!0],this.lineMirror=[!1,!1,!1],this.lineWidth=[1,1,1],this.lineColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.lineTickEnable=[!0,!0,!0],this.lineTickMirror=[!1,!1,!1],this.lineTickLength=[0,0,0],this.lineTickWidth=[1,1,1],this.lineTickColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.gridEnable=[!0,!0,!0],this.gridWidth=[1,1,1],this.gridColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.zeroEnable=[!0,!0,!0],this.zeroLineColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.zeroLineWidth=[2,2,2],this.backgroundEnable=[!1,!1,!1],this.backgroundColor=[[.8,.8,.8,.5],[.8,.8,.8,.5],[.8,.8,.8,.5]],this._firstInit=!0,this._text=null,this._lines=null,this._background=o(t)}var u=c.prototype;function g(){this.primalOffset=[0,0,0],this.primalMinor=[0,0,0],this.mirrorOffset=[0,0,0],this.mirrorMinor=[0,0,0]}u.update=function(t){function e(e,n,r){if(r in t){var i,o=t[r],a=this[r];(e?Array.isArray(o)&&Array.isArray(o[0]):Array.isArray(o))?this[r]=i=[n(o[0]),n(o[1]),n(o[2])]:this[r]=i=[n(o),n(o),n(o)];for(var s=0;s<3;++s)if(i[s]!==a[s])return!0}return!1}t=t||{};var n,o=e.bind(this,!1,Number),a=e.bind(this,!1,Boolean),A=e.bind(this,!1,String),l=e.bind(this,!0,function(t){if(Array.isArray(t)){if(3===t.length)return[+t[0],+t[1],+t[2],1];if(4===t.length)return[+t[0],+t[1],+t[2],+t[3]]}return[0,0,0,1]}),c=!1,u=!1;if("bounds"in t)for(var g=t.bounds,h=0;h<2;++h)for(var d=0;d<3;++d)g[h][d]!==this.bounds[h][d]&&(u=!0),this.bounds[h][d]=g[h][d];if("ticks"in t)for(n=t.ticks,c=!0,this.autoTicks=!1,h=0;h<3;++h)this.tickSpacing[h]=0;else o("tickSpacing")&&(this.autoTicks=!0,u=!0);if(this._firstInit&&("ticks"in t||"tickSpacing"in t||(this.autoTicks=!0),u=!0,c=!0,this._firstInit=!1),u&&this.autoTicks&&(n=s.create(this.bounds,this.tickSpacing),c=!0),c){for(h=0;h<3;++h)n[h].sort(function(t,e){return t.x-e.x});s.equal(n,this.ticks)?c=!1:this.ticks=n}a("tickEnable"),A("tickFont")&&(c=!0),o("tickSize"),o("tickAngle"),o("tickPad"),l("tickColor");var f=A("labels");A("labelFont")&&(f=!0),a("labelEnable"),o("labelSize"),o("labelPad"),l("labelColor"),a("lineEnable"),a("lineMirror"),o("lineWidth"),l("lineColor"),a("lineTickEnable"),a("lineTickMirror"),o("lineTickLength"),o("lineTickWidth"),l("lineTickColor"),a("gridEnable"),o("gridWidth"),l("gridColor"),a("zeroEnable"),l("zeroLineColor"),o("zeroLineWidth"),a("backgroundEnable"),l("backgroundColor"),this._text?this._text&&(f||c)&&this._text.update(this.bounds,this.labels,this.labelFont,this.ticks,this.tickFont):this._text=r(this.gl,this.bounds,this.labels,this.labelFont,this.ticks,this.tickFont),this._lines&&c&&(this._lines.dispose(),this._lines=null),this._lines||(this._lines=i(this.gl,this.bounds,this.ticks))};var h=[new g,new g,new g];function d(t,e,n,r,i){for(var o=t.primalOffset,a=t.primalMinor,s=t.mirrorOffset,A=t.mirrorMinor,l=r[e],c=0;c<3;++c)if(e!==c){var u=o,g=s,h=a,d=A;l&1<<c&&(u=s,g=o,h=A,d=a),u[c]=n[0][c],g[c]=n[1][c],i[c]>0?(h[c]=-1,d[c]=0):(h[c]=0,d[c]=1)}}var f=[0,0,0],p={model:A,view:A,projection:A,_ortho:!1};u.isOpaque=function(){return!0},u.isTransparent=function(){return!1},u.drawTransparent=function(t){};var I=[0,0,0],C=[0,0,0],m=[0,0,0];u.draw=function(t){t=t||p;for(var e=this.gl,n=t.model||A,r=t.view||A,i=t.projection||A,o=this.bounds,s=t._ortho||!1,c=a(n,r,i,o,s),u=c.cubeEdges,g=c.axis,y=r[12],E=r[13],v=r[14],B=r[15],b=(s?2:1)*this.pixelRatio*(i[3]*y+i[7]*E+i[11]*v+i[15]*B)/e.drawingBufferHeight,Q=0;Q<3;++Q)this.lastCubeProps.cubeEdges[Q]=u[Q],this.lastCubeProps.axis[Q]=g[Q];var w=h;for(Q=0;Q<3;++Q)d(h[Q],Q,this.bounds,u,g);e=this.gl;var x,_,D,M=f;for(Q=0;Q<3;++Q)this.backgroundEnable[Q]?M[Q]=g[Q]:M[Q]=0;for(this._background.draw(n,r,i,o,M,this.backgroundColor),this._lines.bind(n,r,i,this),Q=0;Q<3;++Q){var k=[0,0,0];g[Q]>0?k[Q]=o[1][Q]:k[Q]=o[0][Q];for(var S=0;S<2;++S){var L=(Q+1+S)%3,R=(Q+1+(1^S))%3;this.gridEnable[L]&&this._lines.drawGrid(L,R,this.bounds,k,this.gridColor[L],this.gridWidth[L]*this.pixelRatio)}for(S=0;S<2;++S)L=(Q+1+S)%3,R=(Q+1+(1^S))%3,this.zeroEnable[R]&&Math.min(o[0][R],o[1][R])<=0&&Math.max(o[0][R],o[1][R])>=0&&this._lines.drawZero(L,R,this.bounds,k,this.zeroLineColor[R],this.zeroLineWidth[R]*this.pixelRatio)}for(Q=0;Q<3;++Q){this.lineEnable[Q]&&this._lines.drawAxisLine(Q,this.bounds,w[Q].primalOffset,this.lineColor[Q],this.lineWidth[Q]*this.pixelRatio),this.lineMirror[Q]&&this._lines.drawAxisLine(Q,this.bounds,w[Q].mirrorOffset,this.lineColor[Q],this.lineWidth[Q]*this.pixelRatio);var N=l(I,w[Q].primalMinor),F=l(C,w[Q].mirrorMinor),T=this.lineTickLength;for(S=0;S<3;++S){var U=b/n[5*S];N[S]*=T[S]*U,F[S]*=T[S]*U}this.lineTickEnable[Q]&&this._lines.drawAxisTicks(Q,w[Q].primalOffset,N,this.lineTickColor[Q],this.lineTickWidth[Q]*this.pixelRatio),this.lineTickMirror[Q]&&this._lines.drawAxisTicks(Q,w[Q].mirrorOffset,F,this.lineTickColor[Q],this.lineTickWidth[Q]*this.pixelRatio)}function O(t){(D=[0,0,0])[t]=1}function G(t,e,n){var r=(t+1)%3,i=(t+2)%3,o=e[r],a=e[i],s=n[r],A=n[i];o>0&&A>0?O(r):o>0&&A<0?O(r):o<0&&A>0?O(r):o<0&&A<0?O(r):a>0&&s>0?O(i):a>0&&s<0?O(i):a<0&&s>0?O(i):a<0&&s<0&&O(i)}for(this._lines.unbind(),this._text.bind(n,r,i,this.pixelRatio),Q=0;Q<3;++Q){var H=w[Q].primalMinor,j=w[Q].mirrorMinor,Y=l(m,w[Q].primalOffset);for(S=0;S<3;++S)this.lineTickEnable[Q]&&(Y[S]+=b*H[S]*Math.max(this.lineTickLength[S],0)/n[5*S]);var P=[0,0,0];if(P[Q]=1,this.tickEnable[Q]){for(-3600===this.tickAngle[Q]?(this.tickAngle[Q]=0,this.tickAlign[Q]="auto"):this.tickAlign[Q]=-1,_=1,"auto"===(x=[this.tickAlign[Q],.5,_])[0]?x[0]=0:x[0]=parseInt(""+x[0]),D=[0,0,0],G(Q,H,j),S=0;S<3;++S)Y[S]+=b*H[S]*this.tickPad[S]/n[5*S];this._text.drawTicks(Q,this.tickSize[Q],this.tickAngle[Q],Y,this.tickColor[Q],P,D,x)}if(this.labelEnable[Q]){for(_=0,D=[0,0,0],this.labels[Q].length>4&&(O(Q),_=1),"auto"===(x=[this.labelAlign[Q],.5,_])[0]?x[0]=0:x[0]=parseInt(""+x[0]),S=0;S<3;++S)Y[S]+=b*H[S]*this.labelPad[S]/n[5*S];Y[Q]+=.5*(o[0][Q]+o[1][Q]),this._text.drawLabel(Q,this.labelSize[Q],this.labelAngle[Q],Y,this.labelColor[Q],[0,0,0],D,x)}}this._text.unbind()},u.dispose=function(){this._text.dispose(),this._lines.dispose(),this._background.dispose(),this._lines=null,this._text=null,this._background=null,this.gl=null}},{"./lib/background.js":234,"./lib/cube.js":235,"./lib/lines.js":236,"./lib/text.js":238,"./lib/ticks.js":239}],234:[function(t,e,n){"use strict";e.exports=function(t){for(var e=[],n=[],s=0,A=0;A<3;++A)for(var l=(A+1)%3,c=(A+2)%3,u=[0,0,0],g=[0,0,0],h=-1;h<=1;h+=2){n.push(s,s+2,s+1,s+1,s+2,s+3),u[A]=h,g[A]=h;for(var d=-1;d<=1;d+=2){u[l]=d;for(var f=-1;f<=1;f+=2)u[c]=f,e.push(u[0],u[1],u[2],g[0],g[1],g[2]),s+=1}var p=l;l=c,c=p}var I=r(t,new Float32Array(e)),C=r(t,new Uint16Array(n),t.ELEMENT_ARRAY_BUFFER),m=i(t,[{buffer:I,type:t.FLOAT,size:3,offset:0,stride:24},{buffer:I,type:t.FLOAT,size:3,offset:12,stride:24}],C),y=o(t);return y.attributes.position.location=0,y.attributes.normal.location=1,new a(t,I,m,y)};var r=t("gl-buffer"),i=t("gl-vao"),o=t("./shaders").bg;function a(t,e,n,r){this.gl=t,this.buffer=e,this.vao=n,this.shader=r}var s=a.prototype;s.draw=function(t,e,n,r,i,o){for(var a=!1,s=0;s<3;++s)a=a||i[s];if(a){var A=this.gl;A.enable(A.POLYGON_OFFSET_FILL),A.polygonOffset(1,2),this.shader.bind(),this.shader.uniforms={model:t,view:e,projection:n,bounds:r,enable:i,colors:o},this.vao.bind(),this.vao.draw(this.gl.TRIANGLES,36),this.vao.unbind(),A.disable(A.POLYGON_OFFSET_FILL)}},s.dispose=function(){this.vao.dispose(),this.buffer.dispose(),this.shader.dispose()}},{"./shaders":237,"gl-buffer":241,"gl-vao":327}],235:[function(t,e,n){"use strict";e.exports=function(t,e,n,o,h){i(s,e,t),i(s,n,s);for(var C=0,m=0;m<2;++m){c[2]=o[m][2];for(var y=0;y<2;++y){c[1]=o[y][1];for(var E=0;E<2;++E)c[0]=o[E][0],g(A[C],c,s),C+=1}}for(var v=-1,m=0;m<8;++m){for(var B=A[m][3],b=0;b<3;++b)l[m][b]=A[m][b]/B;h&&(l[m][2]*=-1),B<0&&(v<0?v=m:l[m][2]<l[v][2]&&(v=m))}if(v<0){v=0;for(var Q=0;Q<3;++Q){for(var w=(Q+2)%3,x=(Q+1)%3,_=-1,D=-1,M=0;M<2;++M){var k=M<<Q,S=k+(M<<w)+(1-M<<x),L=k+(1-M<<w)+(M<<x);a(l[k],l[S],l[L],u)<0||(M?_=1:D=1)}if(_<0||D<0)D>_&&(v|=1<<Q);else{for(var M=0;M<2;++M){var k=M<<Q,S=k+(M<<w)+(1-M<<x),L=k+(1-M<<w)+(M<<x),R=d([A[k],A[S],A[L],A[k+(1<<w)+(1<<x)]]);M?_=R:D=R}D>_&&(v|=1<<Q)}}}for(var N=7^v,F=-1,m=0;m<8;++m)m!==v&&m!==N&&(F<0?F=m:l[F][1]>l[m][1]&&(F=m));for(var T=-1,m=0;m<3;++m){var U=F^1<<m;if(U!==v&&U!==N){T<0&&(T=U);var x=l[U];x[0]<l[T][0]&&(T=U)}}for(var O=-1,m=0;m<3;++m){var U=F^1<<m;if(U!==v&&U!==N&&U!==T){O<0&&(O=U);var x=l[U];x[0]>l[O][0]&&(O=U)}}var G=f;G[0]=G[1]=G[2]=0,G[r.log2(T^F)]=F&T,G[r.log2(F^O)]=F&O;var H=7^O;H===v||H===N?(H=7^T,G[r.log2(O^H)]=H&O):G[r.log2(T^H)]=H&T;for(var j=p,Y=v,Q=0;Q<3;++Q)j[Q]=Y&1<<Q?-1:1;return I};var r=t("bit-twiddle"),i=t("gl-mat4/multiply"),o=t("split-polygon"),a=t("robust-orientation"),s=new Array(16),A=new Array(8),l=new Array(8),c=new Array(3),u=[0,0,0];function g(t,e,n){for(var r=0;r<4;++r){t[r]=n[12+r];for(var i=0;i<3;++i)t[r]+=e[i]*n[4*i+r]}}!function(){for(var t=0;t<8;++t)A[t]=[1,1,1,1],l[t]=[1,1,1]}();var h=[[0,0,1,0,0],[0,0,-1,1,0],[0,-1,0,1,0],[0,1,0,1,0],[-1,0,0,1,0],[1,0,0,1,0]];function d(t){for(var e=0;e<h.length;++e)if((t=o.positive(t,h[e])).length<3)return 0;var n=t[0],r=n[0]/n[3],i=n[1]/n[3],a=0;for(e=1;e+1<t.length;++e){var s=t[e],A=t[e+1],l=s[0]/s[3]-r,c=s[1]/s[3]-i,u=A[0]/A[3]-r,g=A[1]/A[3]-i;a+=Math.abs(l*g-c*u)}return a}var f=[1,1,1],p=[0,0,0],I={cubeEdges:f,axis:p}},{"bit-twiddle":92,"gl-mat4/multiply":267,"robust-orientation":510,"split-polygon":527}],236:[function(t,e,n){"use strict";e.exports=function(t,e,n){var a=[],s=[0,0,0],A=[0,0,0],l=[0,0,0],c=[0,0,0];a.push(0,0,1,0,1,1,0,0,-1,0,0,-1,0,1,1,0,1,-1);for(var u=0;u<3;++u){for(var g=a.length/3|0,d=0;d<n[u].length;++d){var f=+n[u][d].x;a.push(f,0,1,f,1,1,f,0,-1,f,0,-1,f,1,1,f,1,-1)}var p=a.length/3|0;s[u]=g,A[u]=p-g;for(var g=a.length/3|0,I=0;I<n[u].length;++I){var f=+n[u][I].x;a.push(f,0,1,f,1,1,f,0,-1,f,0,-1,f,1,1,f,1,-1)}var p=a.length/3|0;l[u]=g,c[u]=p-g}var C=r(t,new Float32Array(a)),m=i(t,[{buffer:C,type:t.FLOAT,size:3,stride:0,offset:0}]),y=o(t);return y.attributes.position.location=0,new h(t,C,m,y,A,s,c,l)};var r=t("gl-buffer"),i=t("gl-vao"),o=t("./shaders").line,a=[0,0,0],s=[0,0,0],A=[0,0,0],l=[0,0,0],c=[1,1];function u(t){return t[0]=t[1]=t[2]=0,t}function g(t,e){return t[0]=e[0],t[1]=e[1],t[2]=e[2],t}function h(t,e,n,r,i,o,a,s){this.gl=t,this.vertBuffer=e,this.vao=n,this.shader=r,this.tickCount=i,this.tickOffset=o,this.gridCount=a,this.gridOffset=s}var d=h.prototype;d.bind=function(t,e,n){this.shader.bind(),this.shader.uniforms.model=t,this.shader.uniforms.view=e,this.shader.uniforms.projection=n,c[0]=this.gl.drawingBufferWidth,c[1]=this.gl.drawingBufferHeight,this.shader.uniforms.screenShape=c,this.vao.bind()},d.unbind=function(){this.vao.unbind()},d.drawAxisLine=function(t,e,n,r,i){var o=u(s);this.shader.uniforms.majorAxis=s,o[t]=e[1][t]-e[0][t],this.shader.uniforms.minorAxis=o;var a,c=g(l,n);c[t]+=e[0][t],this.shader.uniforms.offset=c,this.shader.uniforms.lineWidth=i,this.shader.uniforms.color=r,(a=u(A))[(t+2)%3]=1,this.shader.uniforms.screenAxis=a,this.vao.draw(this.gl.TRIANGLES,6),(a=u(A))[(t+1)%3]=1,this.shader.uniforms.screenAxis=a,this.vao.draw(this.gl.TRIANGLES,6)},d.drawAxisTicks=function(t,e,n,r,i){if(this.tickCount[t]){var o=u(a);o[t]=1,this.shader.uniforms.majorAxis=o,this.shader.uniforms.offset=e,this.shader.uniforms.minorAxis=n,this.shader.uniforms.color=r,this.shader.uniforms.lineWidth=i;var s=u(A);s[t]=1,this.shader.uniforms.screenAxis=s,this.vao.draw(this.gl.TRIANGLES,this.tickCount[t],this.tickOffset[t])}},d.drawGrid=function(t,e,n,r,i,o){if(this.gridCount[t]){var c=u(s);c[e]=n[1][e]-n[0][e],this.shader.uniforms.minorAxis=c;var h=g(l,r);h[e]+=n[0][e],this.shader.uniforms.offset=h;var d=u(a);d[t]=1,this.shader.uniforms.majorAxis=d;var f=u(A);f[t]=1,this.shader.uniforms.screenAxis=f,this.shader.uniforms.lineWidth=o,this.shader.uniforms.color=i,this.vao.draw(this.gl.TRIANGLES,this.gridCount[t],this.gridOffset[t])}},d.drawZero=function(t,e,n,r,i,o){var a=u(s);this.shader.uniforms.majorAxis=a,a[t]=n[1][t]-n[0][t],this.shader.uniforms.minorAxis=a;var c=g(l,r);c[t]+=n[0][t],this.shader.uniforms.offset=c;var h=u(A);h[e]=1,this.shader.uniforms.screenAxis=h,this.shader.uniforms.lineWidth=o,this.shader.uniforms.color=i,this.vao.draw(this.gl.TRIANGLES,6)},d.dispose=function(){this.vao.dispose(),this.vertBuffer.dispose(),this.shader.dispose()}},{"./shaders":237,"gl-buffer":241,"gl-vao":327}],237:[function(t,e,n){"use strict";var r=t("glslify"),i=t("gl-shader"),o=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position;\n\nuniform mat4 model, view, projection;\nuniform vec3 offset, majorAxis, minorAxis, screenAxis;\nuniform float lineWidth;\nuniform vec2 screenShape;\n\nvec3 project(vec3 p) {\n  vec4 pp = projection * view * model * vec4(p, 1.0);\n  return pp.xyz / max(pp.w, 0.0001);\n}\n\nvoid main() {\n  vec3 major = position.x * majorAxis;\n  vec3 minor = position.y * minorAxis;\n\n  vec3 vPosition = major + minor + offset;\n  vec3 pPosition = project(vPosition);\n  vec3 offset = project(vPosition + screenAxis * position.z);\n\n  vec2 screen = normalize((offset - pPosition).xy * screenShape) / screenShape;\n\n  gl_Position = vec4(pPosition + vec3(0.5 * screen * lineWidth, 0), 1.0);\n}\n"]),a=r(["precision highp float;\n#define GLSLIFY 1\n\nuniform vec4 color;\nvoid main() {\n  gl_FragColor = color;\n}"]);n.line=function(t){return i(t,o,a,null,[{name:"position",type:"vec3"}])};var s=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position;\n\nuniform mat4 model, view, projection;\nuniform vec3 offset, axis, alignDir, alignOpt;\nuniform float scale, angle, pixelScale;\nuniform vec2 resolution;\n\nvec3 project(vec3 p) {\n  vec4 pp = projection * view * model * vec4(p, 1.0);\n  return pp.xyz / max(pp.w, 0.0001);\n}\n\nfloat computeViewAngle(vec3 a, vec3 b) {\n  vec3 A = project(a);\n  vec3 B = project(b);\n\n  return atan(\n    (B.y - A.y) * resolution.y,\n    (B.x - A.x) * resolution.x\n  );\n}\n\nconst float PI = 3.141592;\nconst float TWO_PI = 2.0 * PI;\nconst float HALF_PI = 0.5 * PI;\nconst float ONE_AND_HALF_PI = 1.5 * PI;\n\nint option = int(floor(alignOpt.x + 0.001));\nfloat hv_ratio =       alignOpt.y;\nbool enableAlign =    (alignOpt.z != 0.0);\n\nfloat mod_angle(float a) {\n  return mod(a, PI);\n}\n\nfloat positive_angle(float a) {\n  return mod_angle((a < 0.0) ?\n    a + TWO_PI :\n    a\n  );\n}\n\nfloat look_upwards(float a) {\n  float b = positive_angle(a);\n  return ((b > HALF_PI) && (b <= ONE_AND_HALF_PI)) ?\n    b - PI :\n    b;\n}\n\nfloat look_horizontal_or_vertical(float a, float ratio) {\n  // ratio controls the ratio between being horizontal to (vertical + horizontal)\n  // if ratio is set to 0.5 then it is 50%, 50%.\n  // when using a higher ratio e.g. 0.75 the result would\n  // likely be more horizontal than vertical.\n\n  float b = positive_angle(a);\n\n  return\n    (b < (      ratio) * HALF_PI) ? 0.0 :\n    (b < (2.0 - ratio) * HALF_PI) ? -HALF_PI :\n    (b < (2.0 + ratio) * HALF_PI) ? 0.0 :\n    (b < (4.0 - ratio) * HALF_PI) ? HALF_PI :\n                                    0.0;\n}\n\nfloat roundTo(float a, float b) {\n  return float(b * floor((a + 0.5 * b) / b));\n}\n\nfloat look_round_n_directions(float a, int n) {\n  float b = positive_angle(a);\n  float div = TWO_PI / float(n);\n  float c = roundTo(b, div);\n  return look_upwards(c);\n}\n\nfloat applyAlignOption(float rawAngle, float delta) {\n  return\n    (option >  2) ? look_round_n_directions(rawAngle + delta, option) :       // option 3-n: round to n directions\n    (option == 2) ? look_horizontal_or_vertical(rawAngle + delta, hv_ratio) : // horizontal or vertical\n    (option == 1) ? rawAngle + delta :       // use free angle, and flip to align with one direction of the axis\n    (option == 0) ? look_upwards(rawAngle) : // use free angle, and stay upwards\n    (option ==-1) ? 0.0 :                    // useful for backward compatibility, all texts remains horizontal\n                    rawAngle;                // otherwise return back raw input angle\n}\n\nbool isAxisTitle = (axis.x == 0.0) &&\n                   (axis.y == 0.0) &&\n                   (axis.z == 0.0);\n\nvoid main() {\n  //Compute world offset\n  float axisDistance = position.z;\n  vec3 dataPosition = axisDistance * axis + offset;\n\n  float beta = angle; // i.e. user defined attributes for each tick\n\n  float axisAngle;\n  float clipAngle;\n  float flip;\n\n  if (enableAlign) {\n    axisAngle = (isAxisTitle) ? HALF_PI :\n                      computeViewAngle(dataPosition, dataPosition + axis);\n    clipAngle = computeViewAngle(dataPosition, dataPosition + alignDir);\n\n    axisAngle += (sin(axisAngle) < 0.0) ? PI : 0.0;\n    clipAngle += (sin(clipAngle) < 0.0) ? PI : 0.0;\n\n    flip = (dot(vec2(cos(axisAngle), sin(axisAngle)),\n                vec2(sin(clipAngle),-cos(clipAngle))) > 0.0) ? 1.0 : 0.0;\n\n    beta += applyAlignOption(clipAngle, flip * PI);\n  }\n\n  //Compute plane offset\n  vec2 planeCoord = position.xy * pixelScale;\n\n  mat2 planeXform = scale * mat2(\n     cos(beta), sin(beta),\n    -sin(beta), cos(beta)\n  );\n\n  vec2 viewOffset = 2.0 * planeXform * planeCoord / resolution;\n\n  //Compute clip position\n  vec3 clipPosition = project(dataPosition);\n\n  //Apply text offset in clip coordinates\n  clipPosition += vec3(viewOffset, 0.0);\n\n  //Done\n  gl_Position = vec4(clipPosition, 1.0);\n}"]),A=r(["precision highp float;\n#define GLSLIFY 1\n\nuniform vec4 color;\nvoid main() {\n  gl_FragColor = color;\n}"]);n.text=function(t){return i(t,s,A,null,[{name:"position",type:"vec3"}])};var l=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position;\nattribute vec3 normal;\n\nuniform mat4 model, view, projection;\nuniform vec3 enable;\nuniform vec3 bounds[2];\n\nvarying vec3 colorChannel;\n\nvoid main() {\n\n  vec3 signAxis = sign(bounds[1] - bounds[0]);\n\n  vec3 realNormal = signAxis * normal;\n\n  if(dot(realNormal, enable) > 0.0) {\n    vec3 minRange = min(bounds[0], bounds[1]);\n    vec3 maxRange = max(bounds[0], bounds[1]);\n    vec3 nPosition = mix(minRange, maxRange, 0.5 * (position + 1.0));\n    gl_Position = projection * view * model * vec4(nPosition, 1.0);\n  } else {\n    gl_Position = vec4(0,0,0,0);\n  }\n\n  colorChannel = abs(realNormal);\n}"]),c=r(["precision highp float;\n#define GLSLIFY 1\n\nuniform vec4 colors[3];\n\nvarying vec3 colorChannel;\n\nvoid main() {\n  gl_FragColor = colorChannel.x * colors[0] +\n                 colorChannel.y * colors[1] +\n                 colorChannel.z * colors[2];\n}"]);n.bg=function(t){return i(t,l,c,null,[{name:"position",type:"vec3"},{name:"normal",type:"vec3"}])}},{"gl-shader":301,glslify:409}],238:[function(t,e,n){(function(n){"use strict";e.exports=function(t,e,n,o,s,A){var c=r(t),u=i(t,[{buffer:c,size:3}]),g=a(t);g.attributes.position.location=0;var h=new l(t,g,c,u);return h.update(e,n,o,s,A),h};var r=t("gl-buffer"),i=t("gl-vao"),o=t("vectorize-text"),a=t("./shaders").text,s=window||n.global||{},A=s.__TEXT_CACHE||{};function l(t,e,n,r){this.gl=t,this.shader=e,this.buffer=n,this.vao=r,this.tickOffset=this.tickCount=this.labelOffset=this.labelCount=null}s.__TEXT_CACHE={};var c=l.prototype,u=[0,0];c.bind=function(t,e,n,r){this.vao.bind(),this.shader.bind();var i=this.shader.uniforms;i.model=t,i.view=e,i.projection=n,i.pixelScale=r,u[0]=this.gl.drawingBufferWidth,u[1]=this.gl.drawingBufferHeight,this.shader.uniforms.resolution=u},c.unbind=function(){this.vao.unbind()},c.update=function(t,e,n,r,i){var a=[];function s(t,e,n,r,i,s){var l=A[n];l||(l=A[n]={});var c=l[e];c||(c=l[e]=function(t,e){try{return o(t,e)}catch(e){return console.warn('error vectorizing text:"'+t+'" error:',e),{cells:[],positions:[]}}}(e,{triangles:!0,font:n,textAlign:"center",textBaseline:"middle",lineSpacing:i,styletags:s}));for(var u=(r||12)/12,g=c.positions,h=c.cells,d=0,f=h.length;d<f;++d)for(var p=h[d],I=2;I>=0;--I){var C=g[p[I]];a.push(u*C[0],-u*C[1],t)}}for(var l=[0,0,0],c=[0,0,0],u=[0,0,0],g=[0,0,0],h={breaklines:!0,bolds:!0,italics:!0,subscripts:!0,superscripts:!0},d=0;d<3;++d){u[d]=a.length/3|0,s(.5*(t[0][d]+t[1][d]),e[d],n[d],12,1.25,h),g[d]=(a.length/3|0)-u[d],l[d]=a.length/3|0;for(var f=0;f<r[d].length;++f)r[d][f].text&&s(r[d][f].x,r[d][f].text,r[d][f].font||i,r[d][f].fontSize||12,1.25,h);c[d]=(a.length/3|0)-l[d]}this.buffer.update(a),this.tickOffset=l,this.tickCount=c,this.labelOffset=u,this.labelCount=g},c.drawTicks=function(t,e,n,r,i,o,a,s){this.tickCount[t]&&(this.shader.uniforms.axis=o,this.shader.uniforms.color=i,this.shader.uniforms.angle=n,this.shader.uniforms.scale=e,this.shader.uniforms.offset=r,this.shader.uniforms.alignDir=a,this.shader.uniforms.alignOpt=s,this.vao.draw(this.gl.TRIANGLES,this.tickCount[t],this.tickOffset[t]))},c.drawLabel=function(t,e,n,r,i,o,a,s){this.labelCount[t]&&(this.shader.uniforms.axis=o,this.shader.uniforms.color=i,this.shader.uniforms.angle=n,this.shader.uniforms.scale=e,this.shader.uniforms.offset=r,this.shader.uniforms.alignDir=a,this.shader.uniforms.alignOpt=s,this.vao.draw(this.gl.TRIANGLES,this.labelCount[t],this.labelOffset[t]))},c.dispose=function(){this.shader.dispose(),this.vao.dispose(),this.buffer.dispose()}}).call(this,t("_process"))},{"./shaders":237,_process:482,"gl-buffer":241,"gl-vao":327,"vectorize-text":550}],239:[function(t,e,n){"use strict";function r(t,e){var n=t+"",r=n.indexOf("."),i=0;r>=0&&(i=n.length-r-1);var o=Math.pow(10,i),a=Math.round(t*e*o),s=a+"";if(s.indexOf("e")>=0)return s;var A=a/o,l=a%o;a<0?(A=0|-Math.ceil(A),l=0|-l):(A=0|Math.floor(A),l|=0);var c=""+A;if(a<0&&(c="-"+c),i){for(var u=""+l;u.length<i;)u="0"+u;return c+"."+u}return c}n.create=function(t,e){for(var n=[],i=0;i<3;++i){for(var o=[],a=(t[0][i],t[1][i],0);a*e[i]<=t[1][i];++a)o.push({x:a*e[i],text:r(e[i],a)});for(var a=-1;a*e[i]>=t[0][i];--a)o.push({x:a*e[i],text:r(e[i],a)});n.push(o)}return n},n.equal=function(t,e){for(var n=0;n<3;++n){if(t[n].length!==e[n].length)return!1;for(var r=0;r<t[n].length;++r){var i=t[n][r],o=e[n][r];if(i.x!==o.x||i.text!==o.text||i.font!==o.font||i.fontColor!==o.fontColor||i.fontSize!==o.fontSize||i.dx!==o.dx||i.dy!==o.dy)return!1}}return!0}},{}],240:[function(t,e,n){"use strict";e.exports=function(t,e,n,A,u){var g=e.model||l,h=e.view||l,I=e.projection||l,C=e._ortho||!1,m=t.bounds,y=(u=u||o(g,h,I,m,C)).axis;a(c,h,g),a(c,I,c);for(var E=f,v=0;v<3;++v)E[v].lo=1/0,E[v].hi=-1/0,E[v].pixelsPerDataUnit=1/0;var B=r(s(c,c));s(c,c);for(var b=0;b<3;++b){var Q=(b+1)%3,w=(b+2)%3,x=p;t:for(var v=0;v<2;++v){var _=[];if(y[b]<0!=!!v){x[b]=m[v][b];for(var D=0;D<2;++D){x[Q]=m[D^v][Q];for(var M=0;M<2;++M)x[w]=m[M^D^v][w],_.push(x.slice())}for(var k=C?5:4,D=k;D===k;++D){if(0===_.length)continue t;_=i.positive(_,B[D])}for(var D=0;D<_.length;++D)for(var w=_[D],S=d(p,c,w,n,A),M=0;M<3;++M)E[M].lo=Math.min(E[M].lo,w[M]),E[M].hi=Math.max(E[M].hi,w[M]),M!==b&&(E[M].pixelsPerDataUnit=Math.min(E[M].pixelsPerDataUnit,Math.abs(S[M])))}}}return E};var r=t("extract-frustum-planes"),i=t("split-polygon"),o=t("./lib/cube.js"),a=t("gl-mat4/multiply"),s=t("gl-mat4/transpose"),A=t("gl-vec4/transformMat4"),l=new Float32Array([1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1]),c=new Float32Array(16);function u(t,e,n){this.lo=t,this.hi=e,this.pixelsPerDataUnit=n}var g=[0,0,0,1],h=[0,0,0,1];function d(t,e,n,r,i){for(var o=0;o<3;++o){for(var a=g,s=h,l=0;l<3;++l)s[l]=a[l]=n[l];s[3]=a[3]=1,s[o]+=1,A(s,s,e),s[3]<0&&(t[o]=1/0),a[o]-=1,A(a,a,e),a[3]<0&&(t[o]=1/0);var c=(a[0]/a[3]-s[0]/s[3])*r,u=(a[1]/a[3]-s[1]/s[3])*i;t[o]=.25*Math.sqrt(c*c+u*u)}return t}var f=[new u(1/0,-1/0,1/0),new u(1/0,-1/0,1/0),new u(1/0,-1/0,1/0)],p=[0,0,0]},{"./lib/cube.js":235,"extract-frustum-planes":224,"gl-mat4/multiply":267,"gl-mat4/transpose":276,"gl-vec4/transformMat4":398,"split-polygon":527}],241:[function(t,e,n){"use strict";var r=t("typedarray-pool"),i=t("ndarray-ops"),o=t("ndarray"),a=["uint8","uint8_clamped","uint16","uint32","int8","int16","int32","float32"];function s(t,e,n,r,i){this.gl=t,this.type=e,this.handle=n,this.length=r,this.usage=i}var A=s.prototype;function l(t,e,n,r,i,o){var a=i.length*i.BYTES_PER_ELEMENT;if(o<0)return t.bufferData(e,i,r),a;if(a+o>n)throw new Error("gl-buffer: If resizing buffer, must not specify offset");return t.bufferSubData(e,o,i),n}function c(t,e){for(var n=r.malloc(t.length,e),i=t.length,o=0;o<i;++o)n[o]=t[o];return n}A.bind=function(){this.gl.bindBuffer(this.type,this.handle)},A.unbind=function(){this.gl.bindBuffer(this.type,null)},A.dispose=function(){this.gl.deleteBuffer(this.handle)},A.update=function(t,e){if("number"!=typeof e&&(e=-1),this.bind(),"object"==typeof t&&void 0!==t.shape){var n=t.dtype;if(a.indexOf(n)<0&&(n="float32"),this.type===this.gl.ELEMENT_ARRAY_BUFFER&&(n=gl.getExtension("OES_element_index_uint")&&"uint16"!==n?"uint32":"uint16"),n===t.dtype&&function(t,e){for(var n=1,r=e.length-1;r>=0;--r){if(e[r]!==n)return!1;n*=t[r]}return!0}(t.shape,t.stride))0===t.offset&&t.data.length===t.shape[0]?this.length=l(this.gl,this.type,this.length,this.usage,t.data,e):this.length=l(this.gl,this.type,this.length,this.usage,t.data.subarray(t.offset,t.shape[0]),e);else{var s=r.malloc(t.size,n),A=o(s,t.shape);i.assign(A,t),this.length=l(this.gl,this.type,this.length,this.usage,e<0?s:s.subarray(0,t.size),e),r.free(s)}}else if(Array.isArray(t)){var u;u=this.type===this.gl.ELEMENT_ARRAY_BUFFER?c(t,"uint16"):c(t,"float32"),this.length=l(this.gl,this.type,this.length,this.usage,e<0?u:u.subarray(0,t.length),e),r.free(u)}else if("object"==typeof t&&"number"==typeof t.length)this.length=l(this.gl,this.type,this.length,this.usage,t,e);else{if("number"!=typeof t&&void 0!==t)throw new Error("gl-buffer: Invalid data type");if(e>=0)throw new Error("gl-buffer: Cannot specify offset when resizing buffer");(t|=0)<=0&&(t=1),this.gl.bufferData(this.type,0|t,this.usage),this.length=t}},e.exports=function(t,e,n,r){if(n=n||t.ARRAY_BUFFER,r=r||t.DYNAMIC_DRAW,n!==t.ARRAY_BUFFER&&n!==t.ELEMENT_ARRAY_BUFFER)throw new Error("gl-buffer: Invalid type for webgl buffer, must be either gl.ARRAY_BUFFER or gl.ELEMENT_ARRAY_BUFFER");if(r!==t.DYNAMIC_DRAW&&r!==t.STATIC_DRAW&&r!==t.STREAM_DRAW)throw new Error("gl-buffer: Invalid usage for buffer, must be either gl.DYNAMIC_DRAW, gl.STATIC_DRAW or gl.STREAM_DRAW");var i=t.createBuffer(),o=new s(t,n,i,0,r);return o.update(e),o}},{ndarray:450,"ndarray-ops":444,"typedarray-pool":545}],242:[function(t,e,n){"use strict";var r=t("gl-vec3"),i=function(t,e){for(var n=0;n<t.length;n++)if(t[n]>=e)return n-1;return n},o=r.create(),a=r.create(),s=function(t,e,n){return t<e?e:t>n?n:t},A=function(t,e,n,A){var l=t[0],c=t[1],u=t[2],g=n[0].length,h=n[1].length,d=n[2].length,f=i(n[0],l),p=i(n[1],c),I=i(n[2],u),C=f+1,m=p+1,y=I+1;if(A&&(f=s(f,0,g-1),C=s(C,0,g-1),p=s(p,0,h-1),m=s(m,0,h-1),I=s(I,0,d-1),y=s(y,0,d-1)),f<0||p<0||I<0||C>=g||m>=h||y>=d)return r.create();var E=(l-n[0][f])/(n[0][C]-n[0][f]),v=(c-n[1][p])/(n[1][m]-n[1][p]),B=(u-n[2][I])/(n[2][y]-n[2][I]);(E<0||E>1||isNaN(E))&&(E=0),(v<0||v>1||isNaN(v))&&(v=0),(B<0||B>1||isNaN(B))&&(B=0);var b=I*g*h,Q=y*g*h,w=p*g,x=m*g,_=f,D=C,M=e[w+b+_],k=e[w+b+D],S=e[x+b+_],L=e[x+b+D],R=e[w+Q+_],N=e[w+Q+D],F=e[x+Q+_],T=e[x+Q+D],U=r.create();return r.lerp(U,M,k,E),r.lerp(o,S,L,E),r.lerp(U,U,o,v),r.lerp(o,R,N,E),r.lerp(a,F,T,E),r.lerp(o,o,a,v),r.lerp(U,U,o,B),U};e.exports=function(t,e){var n;n=t.positions?t.positions:function(t){for(var e=t[0],n=t[1],r=t[2],i=[],o=0;o<r.length;o++)for(var a=0;a<n.length;a++)for(var s=0;s<e.length;s++)i.push([r[o],n[a],e[s]]);return i}(t.meshgrid);var i=t.meshgrid,o=t.vectors,a={positions:[],vertexIntensity:[],vertexIntensityBounds:t.vertexIntensityBounds,vertexNormals:[],vectors:[],cells:[],coneOffset:t.coneOffset,colormap:t.colormap};if(0===t.positions.length)return e&&(e[0]=[0,0,0],e[1]=[0,0,0]),a;for(var s=0,l=1/0,c=-1/0,u=1/0,g=-1/0,h=1/0,d=-1/0,f=null,p=null,I=[],C=1/0,m=0;m<n.length;m++){var y,E=n[m];l=Math.min(E[0],l),c=Math.max(E[0],c),u=Math.min(E[1],u),g=Math.max(E[1],g),h=Math.min(E[2],h),d=Math.max(E[2],d),y=i?A(E,o,i,!0):o[m],r.length(y)>s&&(s=r.length(y)),m&&(C=Math.min(C,2*r.distance(f,E)/(r.length(p)+r.length(y)))),f=E,p=y,I.push(y)}var v=[l,u,h],B=[c,g,d];e&&(e[0]=v,e[1]=B),0===s&&(s=1);var b=1/s;isFinite(C)&&!isNaN(C)||(C=1),a.vectorScale=C;var Q=function(t,e,n){var i=r.create();return r.set(i,0,1,0),i}(),w=t.coneSize||.5;t.absoluteConeSize&&(w=t.absoluteConeSize*b),a.coneScale=w,m=0;for(var x=0;m<n.length;m++)for(var _=(E=n[m])[0],D=E[1],M=E[2],k=I[m],S=r.length(k)*b,L=0;L<8;L++){a.positions.push([_,D,M,x++]),a.positions.push([_,D,M,x++]),a.positions.push([_,D,M,x++]),a.positions.push([_,D,M,x++]),a.positions.push([_,D,M,x++]),a.positions.push([_,D,M,x++]),a.vectors.push(k),a.vectors.push(k),a.vectors.push(k),a.vectors.push(k),a.vectors.push(k),a.vectors.push(k),a.vertexIntensity.push(S,S,S),a.vertexIntensity.push(S,S,S),a.vertexNormals.push(Q,Q,Q),a.vertexNormals.push(Q,Q,Q);var R=a.positions.length;a.cells.push([R-6,R-5,R-4],[R-3,R-2,R-1])}return a},e.exports.createConeMesh=t("./lib/conemesh")},{"./lib/conemesh":243,"gl-vec3":346}],243:[function(t,e,n){"use strict";var r=t("gl-shader"),i=t("gl-buffer"),o=t("gl-vao"),a=t("gl-texture2d"),s=t("normals"),A=t("gl-mat4/multiply"),l=t("gl-mat4/invert"),c=t("ndarray"),u=t("colormap"),g=t("simplicial-complex-contour"),h=t("typedarray-pool"),d=t("./shaders"),f=d.meshShader,p=d.pickShader,I=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];function C(t,e,n,r,i,o,a,s,A,l,c,u,g,h,d,f,p,C,m,y,E,v,B,b){this.gl=t,this.pixelRatio=1,this.cells=[],this.positions=[],this.intensity=[],this.texture=e,this.dirty=!0,this.triShader=n,this.pickShader=r,this.trianglePositions=i,this.triangleVectors=o,this.triangleColors=s,this.triangleNormals=l,this.triangleUVs=A,this.triangleIds=a,this.triangleVAO=c,this.triangleCount=0,this.lineWidth=1,this.edgePositions=u,this.edgeColors=h,this.edgeUVs=d,this.edgeIds=g,this.edgeVAO=f,this.edgeCount=0,this.pointPositions=p,this.pointColors=m,this.pointUVs=y,this.pointSizes=E,this.pointIds=C,this.pointVAO=v,this.pointCount=0,this.contourLineWidth=1,this.contourPositions=B,this.contourVAO=b,this.contourCount=0,this.contourColor=[0,0,0],this.contourEnable=!0,this.pickId=1,this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],this.clipBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.lightPosition=[1e5,1e5,0],this.ambientLight=.8,this.diffuseLight=.8,this.specularLight=2,this.roughness=.5,this.fresnel=1.5,this.opacity=1,this.coneScale=2,this.vectorScale=1,this.coneOffset=.25,this._model=I,this._view=I,this._projection=I,this._resolution=[1,1]}var m=C.prototype;function y(t){var e=r(t,p.vertex,p.fragment,null,p.attributes);return e.attributes.position.location=0,e.attributes.id.location=1,e.attributes.vector.location=5,e}m.isOpaque=function(){return this.opacity>=1},m.isTransparent=function(){return this.opacity<1},m.pickSlots=1,m.setPickBase=function(t){this.pickId=t},m.highlight=function(t){if(t&&this.contourEnable){for(var e=g(this.cells,this.intensity,t.intensity),n=e.cells,r=e.vertexIds,i=e.vertexWeights,o=n.length,a=h.mallocFloat32(6*o),s=0,A=0;A<o;++A)for(var l=n[A],c=0;c<2;++c){var u=l[0];2===l.length&&(u=l[c]);for(var d=r[u][0],f=r[u][1],p=i[u],I=1-p,C=this.positions[d],m=this.positions[f],y=0;y<3;++y)a[s++]=p*C[y]+I*m[y]}this.contourCount=s/3|0,this.contourPositions.update(a.subarray(0,s)),h.free(a)}else this.contourCount=0},m.update=function(t){t=t||{};var e=this.gl;this.dirty=!0,"contourEnable"in t&&(this.contourEnable=t.contourEnable),"contourColor"in t&&(this.contourColor=t.contourColor),"lineWidth"in t&&(this.lineWidth=t.lineWidth),"lightPosition"in t&&(this.lightPosition=t.lightPosition),"opacity"in t&&(this.opacity=t.opacity),"ambient"in t&&(this.ambientLight=t.ambient),"diffuse"in t&&(this.diffuseLight=t.diffuse),"specular"in t&&(this.specularLight=t.specular),"roughness"in t&&(this.roughness=t.roughness),"fresnel"in t&&(this.fresnel=t.fresnel),void 0!==t.vectorScale&&(this.vectorScale=t.vectorScale),void 0!==t.coneScale&&(this.coneScale=t.coneScale),void 0!==t.coneOffset&&(this.coneOffset=t.coneOffset),t.texture?(this.texture.dispose(),this.texture=a(e,t.texture)):t.colormap&&(this.texture.shape=[256,256],this.texture.minFilter=e.LINEAR_MIPMAP_LINEAR,this.texture.magFilter=e.LINEAR,this.texture.setPixels(function(t){for(var e=u({colormap:t,nshades:256,format:"rgba"}),n=new Uint8Array(1024),r=0;r<256;++r){for(var i=e[r],o=0;o<3;++o)n[4*r+o]=i[o];n[4*r+3]=255*i[3]}return c(n,[256,256,4],[4,0,1])}(t.colormap)),this.texture.generateMipmap());var n=t.cells,r=t.positions,i=t.vectors;if(r&&n&&i){var o=[],A=[],l=[],g=[],h=[],d=[],f=[],p=[],I=[],C=[],m=[],y=[],E=[],v=[],B=[];this.cells=n,this.positions=r;var b=t.vertexNormals,Q=t.cellNormals,w=void 0===t.vertexNormalsEpsilon?1e-6:t.vertexNormalsEpsilon,x=void 0===t.faceNormalsEpsilon?1e-6:t.faceNormalsEpsilon;t.useFacetNormals&&!Q&&(Q=s.faceNormals(n,r,x)),Q||b||(b=s.vertexNormals(n,r,w));var _=t.vertexColors,D=t.cellColors,M=t.meshColor||[1,1,1,1],k=t.vertexUVs,S=t.vertexIntensity,L=t.cellUVs,R=t.cellIntensity,N=1/0,F=-1/0;if(!k&&!L)if(S)if(t.vertexIntensityBounds)N=+t.vertexIntensityBounds[0],F=+t.vertexIntensityBounds[1];else for(var T=0;T<S.length;++T){var U=S[T];N=Math.min(N,U),F=Math.max(F,U)}else if(R)for(T=0;T<R.length;++T)U=R[T],N=Math.min(N,U),F=Math.max(F,U);else for(T=0;T<r.length;++T)U=r[T][2],N=Math.min(N,U),F=Math.max(F,U);this.intensity=S||(R?function(t,e,n){for(var r=new Array(e),i=0;i<e;++i)r[i]=0;var o=t.length;for(i=0;i<o;++i)for(var a=t[i],s=0;s<a.length;++s)r[a[s]]=n[i];return r}(n,r.length,R):function(t){for(var e=t.length,n=new Array(e),r=0;r<e;++r)n[r]=t[r][2];return n}(r));var O=t.pointSizes,G=t.pointSize||1;for(this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],T=0;T<r.length;++T)for(var H=r[T],j=0;j<3;++j)!isNaN(H[j])&&isFinite(H[j])&&(this.bounds[0][j]=Math.min(this.bounds[0][j],H[j]),this.bounds[1][j]=Math.max(this.bounds[1][j],H[j]));var Y=0,P=0,K=0;t:for(T=0;T<n.length;++T){var z=n[T];switch(z.length){case 1:for(H=r[q=z[0]],j=0;j<3;++j)if(isNaN(H[j])||!isFinite(H[j]))continue t;m.push(H[0],H[1],H[2],H[3]),3===(V=_?_[q]:D?D[T]:M).length?y.push(V[0],V[1],V[2],1):y.push(V[0],V[1],V[2],V[3]),W=k?k[q]:S?[(S[q]-N)/(F-N),0]:L?L[T]:R?[(R[T]-N)/(F-N),0]:[(H[2]-N)/(F-N),0],E.push(W[0],W[1]),O?v.push(O[q]):v.push(G),B.push(T),K+=1;break;case 2:for(j=0;j<2;++j){H=r[q=z[j]];for(var J=0;J<3;++J)if(isNaN(H[J])||!isFinite(H[J]))continue t}for(j=0;j<2;++j)H=r[q=z[j]],f.push(H[0],H[1],H[2]),3===(V=_?_[q]:D?D[T]:M).length?p.push(V[0],V[1],V[2],1):p.push(V[0],V[1],V[2],V[3]),W=k?k[q]:S?[(S[q]-N)/(F-N),0]:L?L[T]:R?[(R[T]-N)/(F-N),0]:[(H[2]-N)/(F-N),0],I.push(W[0],W[1]),C.push(T);P+=1;break;case 3:for(j=0;j<3;++j)for(H=r[q=z[j]],J=0;J<3;++J)if(isNaN(H[J])||!isFinite(H[J]))continue t;for(j=0;j<3;++j){var q;H=r[q=z[2-j]],o.push(H[0],H[1],H[2],H[3]);var V,W,X,Z=i[q];A.push(Z[0],Z[1],Z[2]),3===(V=_?_[q]:D?D[T]:M).length?l.push(V[0],V[1],V[2],1):l.push(V[0],V[1],V[2],V[3]),W=k?k[q]:S?[(S[q]-N)/(F-N),0]:L?L[T]:R?[(R[T]-N)/(F-N),0]:[(H[2]-N)/(F-N),0],h.push(W[0],W[1]),X=b?b[q]:Q[T],g.push(X[0],X[1],X[2]),d.push(T)}Y+=1}}this.pointCount=K,this.edgeCount=P,this.triangleCount=Y,this.pointPositions.update(m),this.pointColors.update(y),this.pointUVs.update(E),this.pointSizes.update(v),this.pointIds.update(new Uint32Array(B)),this.edgePositions.update(f),this.edgeColors.update(p),this.edgeUVs.update(I),this.edgeIds.update(new Uint32Array(C)),this.trianglePositions.update(o),this.triangleVectors.update(A),this.triangleColors.update(l),this.triangleUVs.update(h),this.triangleNormals.update(g),this.triangleIds.update(new Uint32Array(d))}},m.drawTransparent=m.draw=function(t){t=t||{};for(var e=this.gl,n=t.model||I,r=t.view||I,i=t.projection||I,o=[[-1e6,-1e6,-1e6],[1e6,1e6,1e6]],a=0;a<3;++a)o[0][a]=Math.max(o[0][a],this.clipBounds[0][a]),o[1][a]=Math.min(o[1][a],this.clipBounds[1][a]);var s={model:n,view:r,projection:i,inverseModel:I.slice(),clipBounds:o,kambient:this.ambientLight,kdiffuse:this.diffuseLight,kspecular:this.specularLight,roughness:this.roughness,fresnel:this.fresnel,eyePosition:[0,0,0],lightPosition:[0,0,0],opacity:this.opacity,vectorScale:this.vectorScale,coneScale:this.coneScale,coneOffset:this.coneOffset,contourColor:this.contourColor,texture:0};s.inverseModel=l(s.inverseModel,s.model),e.disable(e.CULL_FACE),this.texture.bind(0);var c=new Array(16);for(A(c,s.view,s.model),A(c,s.projection,c),l(c,c),a=0;a<3;++a)s.eyePosition[a]=c[12+a]/c[15];var u=c[15];for(a=0;a<3;++a)u+=this.lightPosition[a]*c[4*a+3];for(a=0;a<3;++a){for(var g=c[12+a],h=0;h<3;++h)g+=c[4*h+a]*this.lightPosition[h];s.lightPosition[a]=g/u}if(this.triangleCount>0){var d=this.triShader;d.bind(),d.uniforms=s,this.triangleVAO.bind(),e.drawArrays(e.TRIANGLES,0,3*this.triangleCount),this.triangleVAO.unbind()}},m.drawPick=function(t){t=t||{};for(var e=this.gl,n=t.model||I,r=t.view||I,i=t.projection||I,o=[[-1e6,-1e6,-1e6],[1e6,1e6,1e6]],a=0;a<3;++a)o[0][a]=Math.max(o[0][a],this.clipBounds[0][a]),o[1][a]=Math.min(o[1][a],this.clipBounds[1][a]);this._model=[].slice.call(n),this._view=[].slice.call(r),this._projection=[].slice.call(i),this._resolution=[e.drawingBufferWidth,e.drawingBufferHeight];var s={model:n,view:r,projection:i,clipBounds:o,vectorScale:this.vectorScale,coneScale:this.coneScale,coneOffset:this.coneOffset,pickId:this.pickId/255},A=this.pickShader;A.bind(),A.uniforms=s,this.triangleCount>0&&(this.triangleVAO.bind(),e.drawArrays(e.TRIANGLES,0,3*this.triangleCount),this.triangleVAO.unbind()),this.edgeCount>0&&(this.edgeVAO.bind(),e.lineWidth(this.lineWidth*this.pixelRatio),e.drawArrays(e.LINES,0,2*this.edgeCount),this.edgeVAO.unbind())},m.pick=function(t){if(!t)return null;if(t.id!==this.pickId)return null;var e=t.value[0]+256*t.value[1]+65536*t.value[2],n=this.cells[e],r=this.positions[n[1]].slice(0,3);return{index:Math.floor(n[1]/48),position:r,dataCoordinate:r}},m.dispose=function(){this.texture.dispose(),this.triShader.dispose(),this.pickShader.dispose(),this.triangleVAO.dispose(),this.trianglePositions.dispose(),this.triangleVectors.dispose(),this.triangleColors.dispose(),this.triangleUVs.dispose(),this.triangleNormals.dispose(),this.triangleIds.dispose(),this.edgeVAO.dispose(),this.edgePositions.dispose(),this.edgeColors.dispose(),this.edgeUVs.dispose(),this.edgeIds.dispose(),this.pointVAO.dispose(),this.pointPositions.dispose(),this.pointColors.dispose(),this.pointUVs.dispose(),this.pointSizes.dispose(),this.pointIds.dispose(),this.contourVAO.dispose(),this.contourPositions.dispose()},e.exports=function(t,e){1===arguments.length&&(t=(e=t).gl);var n=e.triShader||function(t){var e=r(t,f.vertex,f.fragment,null,f.attributes);return e.attributes.position.location=0,e.attributes.color.location=2,e.attributes.uv.location=3,e.attributes.vector.location=5,e}(t),s=y(t),A=a(t,c(new Uint8Array([255,255,255,255]),[1,1,4]));A.generateMipmap(),A.minFilter=t.LINEAR_MIPMAP_LINEAR,A.magFilter=t.LINEAR;var l=i(t),u=i(t),g=i(t),h=i(t),d=i(t),p=i(t),I=o(t,[{buffer:l,type:t.FLOAT,size:4},{buffer:p,type:t.UNSIGNED_BYTE,size:4,normalized:!0},{buffer:g,type:t.FLOAT,size:4},{buffer:h,type:t.FLOAT,size:2},{buffer:d,type:t.FLOAT,size:3},{buffer:u,type:t.FLOAT,size:3}]),m=i(t),E=i(t),v=i(t),B=i(t),b=o(t,[{buffer:m,type:t.FLOAT,size:3},{buffer:B,type:t.UNSIGNED_BYTE,size:4,normalized:!0},{buffer:E,type:t.FLOAT,size:4},{buffer:v,type:t.FLOAT,size:2}]),Q=i(t),w=i(t),x=i(t),_=i(t),D=i(t),M=o(t,[{buffer:Q,type:t.FLOAT,size:3},{buffer:D,type:t.UNSIGNED_BYTE,size:4,normalized:!0},{buffer:w,type:t.FLOAT,size:4},{buffer:x,type:t.FLOAT,size:2},{buffer:_,type:t.FLOAT,size:1}]),k=i(t),S=new C(t,A,n,s,l,u,p,g,h,d,I,m,B,E,v,b,Q,D,w,x,_,M,k,o(t,[{buffer:k,type:t.FLOAT,size:3}]));return S.update(e),S}},{"./shaders":244,colormap:126,"gl-buffer":241,"gl-mat4/invert":265,"gl-mat4/multiply":267,"gl-shader":301,"gl-texture2d":322,"gl-vao":327,ndarray:450,normals:453,"simplicial-complex-contour":518,"typedarray-pool":545}],244:[function(t,e,n){var r=t("glslify"),i=r(["precision highp float;\n\nprecision highp float;\n#define GLSLIFY 1\n\nvec3 getOrthogonalVector(vec3 v) {\n  // Return up-vector for only-z vector.\n  // Return ax + by + cz = 0, a point that lies on the plane that has v as a normal and that isn't (0,0,0).\n  // From the above if-statement we have ||a|| > 0  U  ||b|| > 0.\n  // Assign z = 0, x = -b, y = a:\n  // a*-b + b*a + c*0 = -ba + ba + 0 = 0\n  if (v.x*v.x > v.z*v.z || v.y*v.y > v.z*v.z) {\n    return normalize(vec3(-v.y, v.x, 0.0));\n  } else {\n    return normalize(vec3(0.0, v.z, -v.y));\n  }\n}\n\n// Calculate the cone vertex and normal at the given index.\n//\n// The returned vertex is for a cone with its top at origin and height of 1.0,\n// pointing in the direction of the vector attribute.\n//\n// Each cone is made up of a top vertex, a center base vertex and base perimeter vertices.\n// These vertices are used to make up the triangles of the cone by the following:\n//   segment + 0 top vertex\n//   segment + 1 perimeter vertex a+1\n//   segment + 2 perimeter vertex a\n//   segment + 3 center base vertex\n//   segment + 4 perimeter vertex a\n//   segment + 5 perimeter vertex a+1\n// Where segment is the number of the radial segment * 6 and a is the angle at that radial segment.\n// To go from index to segment, floor(index / 6)\n// To go from segment to angle, 2*pi * (segment/segmentCount)\n// To go from index to segment index, index - (segment*6)\n//\nvec3 getConePosition(vec3 d, float rawIndex, float coneOffset, out vec3 normal) {\n\n  const float segmentCount = 8.0;\n\n  float index = rawIndex - floor(rawIndex /\n    (segmentCount * 6.0)) *\n    (segmentCount * 6.0);\n\n  float segment = floor(0.001 + index/6.0);\n  float segmentIndex = index - (segment*6.0);\n\n  normal = -normalize(d);\n\n  if (segmentIndex > 2.99 && segmentIndex < 3.01) {\n    return mix(vec3(0.0), -d, coneOffset);\n  }\n\n  float nextAngle = (\n    (segmentIndex > 0.99 &&  segmentIndex < 1.01) ||\n    (segmentIndex > 4.99 &&  segmentIndex < 5.01)\n  ) ? 1.0 : 0.0;\n  float angle = 2.0 * 3.14159 * ((segment + nextAngle) / segmentCount);\n\n  vec3 v1 = mix(d, vec3(0.0), coneOffset);\n  vec3 v2 = v1 - d;\n\n  vec3 u = getOrthogonalVector(d);\n  vec3 v = normalize(cross(u, d));\n\n  vec3 x = u * cos(angle) * length(d)*0.25;\n  vec3 y = v * sin(angle) * length(d)*0.25;\n  vec3 v3 = v2 + x + y;\n  if (segmentIndex < 3.0) {\n    vec3 tx = u * sin(angle);\n    vec3 ty = v * -cos(angle);\n    vec3 tangent = tx + ty;\n    normal = normalize(cross(v3 - v1, tangent));\n  }\n\n  if (segmentIndex == 0.0) {\n    return mix(d, vec3(0.0), coneOffset);\n  }\n  return v3;\n}\n\nattribute vec3 vector;\nattribute vec4 color, position;\nattribute vec2 uv;\nuniform float vectorScale;\nuniform float coneScale;\n\nuniform float coneOffset;\n\nuniform mat4 model\n           , view\n           , projection\n           , inverseModel;\nuniform vec3 eyePosition\n           , lightPosition;\n\nvarying vec3 f_normal\n           , f_lightDirection\n           , f_eyeDirection\n           , f_data\n           , f_position;\nvarying vec4 f_color;\nvarying vec2 f_uv;\n\nvoid main() {\n  // Scale the vector magnitude to stay constant with\n  // model & view changes.\n  vec3 normal;\n  vec3 XYZ = getConePosition(mat3(model) * ((vectorScale * coneScale) * vector), position.w, coneOffset, normal);\n  vec4 conePosition = model * vec4(position.xyz, 1.0) + vec4(XYZ, 0.0);\n\n  //Lighting geometry parameters\n  vec4 cameraCoordinate = view * conePosition;\n  cameraCoordinate.xyz /= cameraCoordinate.w;\n  f_lightDirection = lightPosition - cameraCoordinate.xyz;\n  f_eyeDirection   = eyePosition - cameraCoordinate.xyz;\n  f_normal = normalize((vec4(normal,0.0) * inverseModel).xyz);\n\n  // vec4 m_position  = model * vec4(conePosition, 1.0);\n  vec4 t_position  = view * conePosition;\n  gl_Position      = projection * t_position;\n\n  f_color          = color;\n  f_data           = conePosition.xyz;\n  f_position       = position.xyz;\n  f_uv             = uv;\n}\n"]),o=r(["#extension GL_OES_standard_derivatives : enable\n\nprecision highp float;\n#define GLSLIFY 1\n\nfloat beckmannDistribution(float x, float roughness) {\n  float NdotH = max(x, 0.0001);\n  float cos2Alpha = NdotH * NdotH;\n  float tan2Alpha = (cos2Alpha - 1.0) / cos2Alpha;\n  float roughness2 = roughness * roughness;\n  float denom = 3.141592653589793 * roughness2 * cos2Alpha * cos2Alpha;\n  return exp(tan2Alpha / roughness2) / denom;\n}\n\nfloat cookTorranceSpecular(\n  vec3 lightDirection,\n  vec3 viewDirection,\n  vec3 surfaceNormal,\n  float roughness,\n  float fresnel) {\n\n  float VdotN = max(dot(viewDirection, surfaceNormal), 0.0);\n  float LdotN = max(dot(lightDirection, surfaceNormal), 0.0);\n\n  //Half angle vector\n  vec3 H = normalize(lightDirection + viewDirection);\n\n  //Geometric term\n  float NdotH = max(dot(surfaceNormal, H), 0.0);\n  float VdotH = max(dot(viewDirection, H), 0.000001);\n  float LdotH = max(dot(lightDirection, H), 0.000001);\n  float G1 = (2.0 * NdotH * VdotN) / VdotH;\n  float G2 = (2.0 * NdotH * LdotN) / LdotH;\n  float G = min(1.0, min(G1, G2));\n  \n  //Distribution term\n  float D = beckmannDistribution(NdotH, roughness);\n\n  //Fresnel term\n  float F = pow(1.0 - VdotN, fresnel);\n\n  //Multiply terms and done\n  return  G * F * D / max(3.14159265 * VdotN, 0.000001);\n}\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 clipBounds[2];\nuniform float roughness\n            , fresnel\n            , kambient\n            , kdiffuse\n            , kspecular\n            , opacity;\nuniform sampler2D texture;\n\nvarying vec3 f_normal\n           , f_lightDirection\n           , f_eyeDirection\n           , f_data\n           , f_position;\nvarying vec4 f_color;\nvarying vec2 f_uv;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], f_position)) discard;\n  vec3 N = normalize(f_normal);\n  vec3 L = normalize(f_lightDirection);\n  vec3 V = normalize(f_eyeDirection);\n\n  if(gl_FrontFacing) {\n    N = -N;\n  }\n\n  float specular = min(1.0, max(0.0, cookTorranceSpecular(L, V, N, roughness, fresnel)));\n  float diffuse  = min(kambient + kdiffuse * max(dot(N, L), 0.0), 1.0);\n\n  vec4 surfaceColor = f_color * texture2D(texture, f_uv);\n  vec4 litColor = surfaceColor.a * vec4(diffuse * surfaceColor.rgb + kspecular * vec3(1,1,1) * specular,  1.0);\n\n  gl_FragColor = litColor * opacity;\n}\n"]),a=r(["precision highp float;\n\nprecision highp float;\n#define GLSLIFY 1\n\nvec3 getOrthogonalVector(vec3 v) {\n  // Return up-vector for only-z vector.\n  // Return ax + by + cz = 0, a point that lies on the plane that has v as a normal and that isn't (0,0,0).\n  // From the above if-statement we have ||a|| > 0  U  ||b|| > 0.\n  // Assign z = 0, x = -b, y = a:\n  // a*-b + b*a + c*0 = -ba + ba + 0 = 0\n  if (v.x*v.x > v.z*v.z || v.y*v.y > v.z*v.z) {\n    return normalize(vec3(-v.y, v.x, 0.0));\n  } else {\n    return normalize(vec3(0.0, v.z, -v.y));\n  }\n}\n\n// Calculate the cone vertex and normal at the given index.\n//\n// The returned vertex is for a cone with its top at origin and height of 1.0,\n// pointing in the direction of the vector attribute.\n//\n// Each cone is made up of a top vertex, a center base vertex and base perimeter vertices.\n// These vertices are used to make up the triangles of the cone by the following:\n//   segment + 0 top vertex\n//   segment + 1 perimeter vertex a+1\n//   segment + 2 perimeter vertex a\n//   segment + 3 center base vertex\n//   segment + 4 perimeter vertex a\n//   segment + 5 perimeter vertex a+1\n// Where segment is the number of the radial segment * 6 and a is the angle at that radial segment.\n// To go from index to segment, floor(index / 6)\n// To go from segment to angle, 2*pi * (segment/segmentCount)\n// To go from index to segment index, index - (segment*6)\n//\nvec3 getConePosition(vec3 d, float rawIndex, float coneOffset, out vec3 normal) {\n\n  const float segmentCount = 8.0;\n\n  float index = rawIndex - floor(rawIndex /\n    (segmentCount * 6.0)) *\n    (segmentCount * 6.0);\n\n  float segment = floor(0.001 + index/6.0);\n  float segmentIndex = index - (segment*6.0);\n\n  normal = -normalize(d);\n\n  if (segmentIndex > 2.99 && segmentIndex < 3.01) {\n    return mix(vec3(0.0), -d, coneOffset);\n  }\n\n  float nextAngle = (\n    (segmentIndex > 0.99 &&  segmentIndex < 1.01) ||\n    (segmentIndex > 4.99 &&  segmentIndex < 5.01)\n  ) ? 1.0 : 0.0;\n  float angle = 2.0 * 3.14159 * ((segment + nextAngle) / segmentCount);\n\n  vec3 v1 = mix(d, vec3(0.0), coneOffset);\n  vec3 v2 = v1 - d;\n\n  vec3 u = getOrthogonalVector(d);\n  vec3 v = normalize(cross(u, d));\n\n  vec3 x = u * cos(angle) * length(d)*0.25;\n  vec3 y = v * sin(angle) * length(d)*0.25;\n  vec3 v3 = v2 + x + y;\n  if (segmentIndex < 3.0) {\n    vec3 tx = u * sin(angle);\n    vec3 ty = v * -cos(angle);\n    vec3 tangent = tx + ty;\n    normal = normalize(cross(v3 - v1, tangent));\n  }\n\n  if (segmentIndex == 0.0) {\n    return mix(d, vec3(0.0), coneOffset);\n  }\n  return v3;\n}\n\nattribute vec3 vector;\nattribute vec4 position;\nattribute vec4 id;\n\nuniform mat4 model, view, projection;\n\nuniform float vectorScale;\nuniform float coneScale;\nuniform float coneOffset;\n\nvarying vec3 f_position;\nvarying vec4 f_id;\n\nvoid main() {\n  vec3 normal;\n  vec3 XYZ = getConePosition(mat3(model) * ((vectorScale * coneScale) * vector), position.w, coneOffset, normal);\n  vec4 conePosition = model * vec4(position.xyz, 1.0) + vec4(XYZ, 0.0);\n  gl_Position = projection * view * conePosition;\n  f_id        = id;\n  f_position  = position.xyz;\n}\n"]),s=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3  clipBounds[2];\nuniform float pickId;\n\nvarying vec3 f_position;\nvarying vec4 f_id;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], f_position)) discard;\n\n  gl_FragColor = vec4(pickId, f_id.xyz);\n}"]);n.meshShader={vertex:i,fragment:o,attributes:[{name:"position",type:"vec4"},{name:"normal",type:"vec3"},{name:"color",type:"vec4"},{name:"uv",type:"vec2"},{name:"vector",type:"vec3"}]},n.pickShader={vertex:a,fragment:s,attributes:[{name:"position",type:"vec4"},{name:"id",type:"vec4"},{name:"vector",type:"vec3"}]}},{glslify:409}],245:[function(t,e,n){e.exports={0:"NONE",1:"ONE",2:"LINE_LOOP",3:"LINE_STRIP",4:"TRIANGLES",5:"TRIANGLE_STRIP",6:"TRIANGLE_FAN",256:"DEPTH_BUFFER_BIT",512:"NEVER",513:"LESS",514:"EQUAL",515:"LEQUAL",516:"GREATER",517:"NOTEQUAL",518:"GEQUAL",519:"ALWAYS",768:"SRC_COLOR",769:"ONE_MINUS_SRC_COLOR",770:"SRC_ALPHA",771:"ONE_MINUS_SRC_ALPHA",772:"DST_ALPHA",773:"ONE_MINUS_DST_ALPHA",774:"DST_COLOR",775:"ONE_MINUS_DST_COLOR",776:"SRC_ALPHA_SATURATE",1024:"STENCIL_BUFFER_BIT",1028:"FRONT",1029:"BACK",1032:"FRONT_AND_BACK",1280:"INVALID_ENUM",1281:"INVALID_VALUE",1282:"INVALID_OPERATION",1285:"OUT_OF_MEMORY",1286:"INVALID_FRAMEBUFFER_OPERATION",2304:"CW",2305:"CCW",2849:"LINE_WIDTH",2884:"CULL_FACE",2885:"CULL_FACE_MODE",2886:"FRONT_FACE",2928:"DEPTH_RANGE",2929:"DEPTH_TEST",2930:"DEPTH_WRITEMASK",2931:"DEPTH_CLEAR_VALUE",2932:"DEPTH_FUNC",2960:"STENCIL_TEST",2961:"STENCIL_CLEAR_VALUE",2962:"STENCIL_FUNC",2963:"STENCIL_VALUE_MASK",2964:"STENCIL_FAIL",2965:"STENCIL_PASS_DEPTH_FAIL",2966:"STENCIL_PASS_DEPTH_PASS",2967:"STENCIL_REF",2968:"STENCIL_WRITEMASK",2978:"VIEWPORT",3024:"DITHER",3042:"BLEND",3088:"SCISSOR_BOX",3089:"SCISSOR_TEST",3106:"COLOR_CLEAR_VALUE",3107:"COLOR_WRITEMASK",3317:"UNPACK_ALIGNMENT",3333:"PACK_ALIGNMENT",3379:"MAX_TEXTURE_SIZE",3386:"MAX_VIEWPORT_DIMS",3408:"SUBPIXEL_BITS",3410:"RED_BITS",3411:"GREEN_BITS",3412:"BLUE_BITS",3413:"ALPHA_BITS",3414:"DEPTH_BITS",3415:"STENCIL_BITS",3553:"TEXTURE_2D",4352:"DONT_CARE",4353:"FASTEST",4354:"NICEST",5120:"BYTE",5121:"UNSIGNED_BYTE",5122:"SHORT",5123:"UNSIGNED_SHORT",5124:"INT",5125:"UNSIGNED_INT",5126:"FLOAT",5386:"INVERT",5890:"TEXTURE",6401:"STENCIL_INDEX",6402:"DEPTH_COMPONENT",6406:"ALPHA",6407:"RGB",6408:"RGBA",6409:"LUMINANCE",6410:"LUMINANCE_ALPHA",7680:"KEEP",7681:"REPLACE",7682:"INCR",7683:"DECR",7936:"VENDOR",7937:"RENDERER",7938:"VERSION",9728:"NEAREST",9729:"LINEAR",9984:"NEAREST_MIPMAP_NEAREST",9985:"LINEAR_MIPMAP_NEAREST",9986:"NEAREST_MIPMAP_LINEAR",9987:"LINEAR_MIPMAP_LINEAR",10240:"TEXTURE_MAG_FILTER",10241:"TEXTURE_MIN_FILTER",10242:"TEXTURE_WRAP_S",10243:"TEXTURE_WRAP_T",10497:"REPEAT",10752:"POLYGON_OFFSET_UNITS",16384:"COLOR_BUFFER_BIT",32769:"CONSTANT_COLOR",32770:"ONE_MINUS_CONSTANT_COLOR",32771:"CONSTANT_ALPHA",32772:"ONE_MINUS_CONSTANT_ALPHA",32773:"BLEND_COLOR",32774:"FUNC_ADD",32777:"BLEND_EQUATION_RGB",32778:"FUNC_SUBTRACT",32779:"FUNC_REVERSE_SUBTRACT",32819:"UNSIGNED_SHORT_4_4_4_4",32820:"UNSIGNED_SHORT_5_5_5_1",32823:"POLYGON_OFFSET_FILL",32824:"POLYGON_OFFSET_FACTOR",32854:"RGBA4",32855:"RGB5_A1",32873:"TEXTURE_BINDING_2D",32926:"SAMPLE_ALPHA_TO_COVERAGE",32928:"SAMPLE_COVERAGE",32936:"SAMPLE_BUFFERS",32937:"SAMPLES",32938:"SAMPLE_COVERAGE_VALUE",32939:"SAMPLE_COVERAGE_INVERT",32968:"BLEND_DST_RGB",32969:"BLEND_SRC_RGB",32970:"BLEND_DST_ALPHA",32971:"BLEND_SRC_ALPHA",33071:"CLAMP_TO_EDGE",33170:"GENERATE_MIPMAP_HINT",33189:"DEPTH_COMPONENT16",33306:"DEPTH_STENCIL_ATTACHMENT",33635:"UNSIGNED_SHORT_5_6_5",33648:"MIRRORED_REPEAT",33901:"ALIASED_POINT_SIZE_RANGE",33902:"ALIASED_LINE_WIDTH_RANGE",33984:"TEXTURE0",33985:"TEXTURE1",33986:"TEXTURE2",33987:"TEXTURE3",33988:"TEXTURE4",33989:"TEXTURE5",33990:"TEXTURE6",33991:"TEXTURE7",33992:"TEXTURE8",33993:"TEXTURE9",33994:"TEXTURE10",33995:"TEXTURE11",33996:"TEXTURE12",33997:"TEXTURE13",33998:"TEXTURE14",33999:"TEXTURE15",34e3:"TEXTURE16",34001:"TEXTURE17",34002:"TEXTURE18",34003:"TEXTURE19",34004:"TEXTURE20",34005:"TEXTURE21",34006:"TEXTURE22",34007:"TEXTURE23",34008:"TEXTURE24",34009:"TEXTURE25",34010:"TEXTURE26",34011:"TEXTURE27",34012:"TEXTURE28",34013:"TEXTURE29",34014:"TEXTURE30",34015:"TEXTURE31",34016:"ACTIVE_TEXTURE",34024:"MAX_RENDERBUFFER_SIZE",34041:"DEPTH_STENCIL",34055:"INCR_WRAP",34056:"DECR_WRAP",34067:"TEXTURE_CUBE_MAP",34068:"TEXTURE_BINDING_CUBE_MAP",34069:"TEXTURE_CUBE_MAP_POSITIVE_X",34070:"TEXTURE_CUBE_MAP_NEGATIVE_X",34071:"TEXTURE_CUBE_MAP_POSITIVE_Y",34072:"TEXTURE_CUBE_MAP_NEGATIVE_Y",34073:"TEXTURE_CUBE_MAP_POSITIVE_Z",34074:"TEXTURE_CUBE_MAP_NEGATIVE_Z",34076:"MAX_CUBE_MAP_TEXTURE_SIZE",34338:"VERTEX_ATTRIB_ARRAY_ENABLED",34339:"VERTEX_ATTRIB_ARRAY_SIZE",34340:"VERTEX_ATTRIB_ARRAY_STRIDE",34341:"VERTEX_ATTRIB_ARRAY_TYPE",34342:"CURRENT_VERTEX_ATTRIB",34373:"VERTEX_ATTRIB_ARRAY_POINTER",34466:"NUM_COMPRESSED_TEXTURE_FORMATS",34467:"COMPRESSED_TEXTURE_FORMATS",34660:"BUFFER_SIZE",34661:"BUFFER_USAGE",34816:"STENCIL_BACK_FUNC",34817:"STENCIL_BACK_FAIL",34818:"STENCIL_BACK_PASS_DEPTH_FAIL",34819:"STENCIL_BACK_PASS_DEPTH_PASS",34877:"BLEND_EQUATION_ALPHA",34921:"MAX_VERTEX_ATTRIBS",34922:"VERTEX_ATTRIB_ARRAY_NORMALIZED",34930:"MAX_TEXTURE_IMAGE_UNITS",34962:"ARRAY_BUFFER",34963:"ELEMENT_ARRAY_BUFFER",34964:"ARRAY_BUFFER_BINDING",34965:"ELEMENT_ARRAY_BUFFER_BINDING",34975:"VERTEX_ATTRIB_ARRAY_BUFFER_BINDING",35040:"STREAM_DRAW",35044:"STATIC_DRAW",35048:"DYNAMIC_DRAW",35632:"FRAGMENT_SHADER",35633:"VERTEX_SHADER",35660:"MAX_VERTEX_TEXTURE_IMAGE_UNITS",35661:"MAX_COMBINED_TEXTURE_IMAGE_UNITS",35663:"SHADER_TYPE",35664:"FLOAT_VEC2",35665:"FLOAT_VEC3",35666:"FLOAT_VEC4",35667:"INT_VEC2",35668:"INT_VEC3",35669:"INT_VEC4",35670:"BOOL",35671:"BOOL_VEC2",35672:"BOOL_VEC3",35673:"BOOL_VEC4",35674:"FLOAT_MAT2",35675:"FLOAT_MAT3",35676:"FLOAT_MAT4",35678:"SAMPLER_2D",35680:"SAMPLER_CUBE",35712:"DELETE_STATUS",35713:"COMPILE_STATUS",35714:"LINK_STATUS",35715:"VALIDATE_STATUS",35716:"INFO_LOG_LENGTH",35717:"ATTACHED_SHADERS",35718:"ACTIVE_UNIFORMS",35719:"ACTIVE_UNIFORM_MAX_LENGTH",35720:"SHADER_SOURCE_LENGTH",35721:"ACTIVE_ATTRIBUTES",35722:"ACTIVE_ATTRIBUTE_MAX_LENGTH",35724:"SHADING_LANGUAGE_VERSION",35725:"CURRENT_PROGRAM",36003:"STENCIL_BACK_REF",36004:"STENCIL_BACK_VALUE_MASK",36005:"STENCIL_BACK_WRITEMASK",36006:"FRAMEBUFFER_BINDING",36007:"RENDERBUFFER_BINDING",36048:"FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE",36049:"FRAMEBUFFER_ATTACHMENT_OBJECT_NAME",36050:"FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL",36051:"FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE",36053:"FRAMEBUFFER_COMPLETE",36054:"FRAMEBUFFER_INCOMPLETE_ATTACHMENT",36055:"FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT",36057:"FRAMEBUFFER_INCOMPLETE_DIMENSIONS",36061:"FRAMEBUFFER_UNSUPPORTED",36064:"COLOR_ATTACHMENT0",36096:"DEPTH_ATTACHMENT",36128:"STENCIL_ATTACHMENT",36160:"FRAMEBUFFER",36161:"RENDERBUFFER",36162:"RENDERBUFFER_WIDTH",36163:"RENDERBUFFER_HEIGHT",36164:"RENDERBUFFER_INTERNAL_FORMAT",36168:"STENCIL_INDEX8",36176:"RENDERBUFFER_RED_SIZE",36177:"RENDERBUFFER_GREEN_SIZE",36178:"RENDERBUFFER_BLUE_SIZE",36179:"RENDERBUFFER_ALPHA_SIZE",36180:"RENDERBUFFER_DEPTH_SIZE",36181:"RENDERBUFFER_STENCIL_SIZE",36194:"RGB565",36336:"LOW_FLOAT",36337:"MEDIUM_FLOAT",36338:"HIGH_FLOAT",36339:"LOW_INT",36340:"MEDIUM_INT",36341:"HIGH_INT",36346:"SHADER_COMPILER",36347:"MAX_VERTEX_UNIFORM_VECTORS",36348:"MAX_VARYING_VECTORS",36349:"MAX_FRAGMENT_UNIFORM_VECTORS",37440:"UNPACK_FLIP_Y_WEBGL",37441:"UNPACK_PREMULTIPLY_ALPHA_WEBGL",37442:"CONTEXT_LOST_WEBGL",37443:"UNPACK_COLORSPACE_CONVERSION_WEBGL",37444:"BROWSER_DEFAULT_WEBGL"}},{}],246:[function(t,e,n){var r=t("./1.0/numbers");e.exports=function(t){return r[t]}},{"./1.0/numbers":245}],247:[function(t,e,n){"use strict";e.exports=function(t){var e=t.gl,n=r(e),a=i(e,[{buffer:n,type:e.FLOAT,size:3,offset:0,stride:40},{buffer:n,type:e.FLOAT,size:4,offset:12,stride:40},{buffer:n,type:e.FLOAT,size:3,offset:28,stride:40}]),A=o(e);A.attributes.position.location=0,A.attributes.color.location=1,A.attributes.offset.location=2;var l=new s(e,n,a,A);return l.update(t),l};var r=t("gl-buffer"),i=t("gl-vao"),o=t("./shaders/index"),a=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];function s(t,e,n,r){this.gl=t,this.shader=r,this.buffer=e,this.vao=n,this.pixelRatio=1,this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],this.clipBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.lineWidth=[1,1,1],this.capSize=[10,10,10],this.lineCount=[0,0,0],this.lineOffset=[0,0,0],this.opacity=1,this.hasAlpha=!1}var A=s.prototype;function l(t,e){for(var n=0;n<3;++n)t[0][n]=Math.min(t[0][n],e[n]),t[1][n]=Math.max(t[1][n],e[n])}A.isOpaque=function(){return!this.hasAlpha},A.isTransparent=function(){return this.hasAlpha},A.drawTransparent=A.draw=function(t){var e=this.gl,n=this.shader.uniforms;this.shader.bind();var r=n.view=t.view||a,i=n.projection=t.projection||a;n.model=t.model||a,n.clipBounds=this.clipBounds,n.opacity=this.opacity;var o=r[12],s=r[13],A=r[14],l=r[15],c=(t._ortho?2:1)*this.pixelRatio*(i[3]*o+i[7]*s+i[11]*A+i[15]*l)/e.drawingBufferHeight;this.vao.bind();for(var u=0;u<3;++u)e.lineWidth(this.lineWidth[u]*this.pixelRatio),n.capSize=this.capSize[u]*c,this.lineCount[u]&&e.drawArrays(e.LINES,this.lineOffset[u],this.lineCount[u]);this.vao.unbind()};var c=function(){for(var t=new Array(3),e=0;e<3;++e){for(var n=[],r=1;r<=2;++r)for(var i=-1;i<=1;i+=2){var o=[0,0,0];o[(r+e)%3]=i,n.push(o)}t[e]=n}return t}();function u(t,e,n,r){for(var i=c[r],o=0;o<i.length;++o){var a=i[o];t.push(e[0],e[1],e[2],n[0],n[1],n[2],n[3],a[0],a[1],a[2])}return i.length}A.update=function(t){"lineWidth"in(t=t||{})&&(this.lineWidth=t.lineWidth,Array.isArray(this.lineWidth)||(this.lineWidth=[this.lineWidth,this.lineWidth,this.lineWidth])),"capSize"in t&&(this.capSize=t.capSize,Array.isArray(this.capSize)||(this.capSize=[this.capSize,this.capSize,this.capSize])),this.hasAlpha=!1,"opacity"in t&&(this.opacity=+t.opacity,this.opacity<1&&(this.hasAlpha=!0));var e=t.color||[[0,0,0],[0,0,0],[0,0,0]],n=t.position,r=t.error;if(Array.isArray(e[0])||(e=[e,e,e]),n&&r){var i=[],o=n.length,a=0;this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],this.lineCount=[0,0,0];for(var s=0;s<3;++s){this.lineOffset[s]=a;t:for(var A=0;A<o;++A){for(var c=n[A],g=0;g<3;++g)if(isNaN(c[g])||!isFinite(c[g]))continue t;var h,d=r[A],f=e[s];Array.isArray(f[0])&&(f=e[A]),3===f.length?f=[f[0],f[1],f[2],1]:4===f.length&&(f=[f[0],f[1],f[2],f[3]],!this.hasAlpha&&f[3]<1&&(this.hasAlpha=!0)),isNaN(d[0][s])||isNaN(d[1][s])||(d[0][s]<0&&((h=c.slice())[s]+=d[0][s],i.push(c[0],c[1],c[2],f[0],f[1],f[2],f[3],0,0,0,h[0],h[1],h[2],f[0],f[1],f[2],f[3],0,0,0),l(this.bounds,h),a+=2+u(i,h,f,s)),d[1][s]>0&&((h=c.slice())[s]+=d[1][s],i.push(c[0],c[1],c[2],f[0],f[1],f[2],f[3],0,0,0,h[0],h[1],h[2],f[0],f[1],f[2],f[3],0,0,0),l(this.bounds,h),a+=2+u(i,h,f,s)))}this.lineCount[s]=a-this.lineOffset[s]}this.buffer.update(i)}},A.dispose=function(){this.shader.dispose(),this.buffer.dispose(),this.vao.dispose()}},{"./shaders/index":248,"gl-buffer":241,"gl-vao":327}],248:[function(t,e,n){"use strict";var r=t("glslify"),i=t("gl-shader"),o=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position, offset;\nattribute vec4 color;\nuniform mat4 model, view, projection;\nuniform float capSize;\nvarying vec4 fragColor;\nvarying vec3 fragPosition;\n\nvoid main() {\n  vec4 worldPosition  = model * vec4(position, 1.0);\n  worldPosition       = (worldPosition / worldPosition.w) + vec4(capSize * offset, 0.0);\n  gl_Position         = projection * view * worldPosition;\n  fragColor           = color;\n  fragPosition        = position;\n}"]),a=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 clipBounds[2];\nuniform float opacity;\nvarying vec3 fragPosition;\nvarying vec4 fragColor;\n\nvoid main() {\n  if (\n    outOfRange(clipBounds[0], clipBounds[1], fragPosition) ||\n    fragColor.a * opacity == 0.\n  ) discard;\n\n  gl_FragColor = opacity * fragColor;\n}"]);e.exports=function(t){return i(t,o,a,null,[{name:"position",type:"vec3"},{name:"color",type:"vec4"},{name:"offset",type:"vec3"}])}},{"gl-shader":301,glslify:409}],249:[function(t,e,n){"use strict";var r=t("gl-texture2d");e.exports=function(t,e,n,r){i||(i=t.FRAMEBUFFER_UNSUPPORTED,o=t.FRAMEBUFFER_INCOMPLETE_ATTACHMENT,a=t.FRAMEBUFFER_INCOMPLETE_DIMENSIONS,s=t.FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT);var l=t.getExtension("WEBGL_draw_buffers");if(!A&&l&&function(t,e){var n=t.getParameter(e.MAX_COLOR_ATTACHMENTS_WEBGL);A=new Array(n+1);for(var r=0;r<=n;++r){for(var i=new Array(n),o=0;o<r;++o)i[o]=t.COLOR_ATTACHMENT0+o;for(var o=r;o<n;++o)i[o]=t.NONE;A[r]=i}}(t,l),Array.isArray(e)&&(r=n,n=0|e[1],e=0|e[0]),"number"!=typeof e)throw new Error("gl-fbo: Missing shape parameter");var c=t.getParameter(t.MAX_RENDERBUFFER_SIZE);if(e<0||e>c||n<0||n>c)throw new Error("gl-fbo: Parameters are too large for FBO");var u=1;if("color"in(r=r||{})){if((u=Math.max(0|r.color,0))<0)throw new Error("gl-fbo: Must specify a nonnegative number of colors");if(u>1){if(!l)throw new Error("gl-fbo: Multiple draw buffer extension not supported");if(u>t.getParameter(l.MAX_COLOR_ATTACHMENTS_WEBGL))throw new Error("gl-fbo: Context does not support "+u+" draw buffers")}}var g=t.UNSIGNED_BYTE,h=t.getExtension("OES_texture_float");if(r.float&&u>0){if(!h)throw new Error("gl-fbo: Context does not support floating point textures");g=t.FLOAT}else r.preferFloat&&u>0&&h&&(g=t.FLOAT);var f=!0;"depth"in r&&(f=!!r.depth);var p=!1;return"stencil"in r&&(p=!!r.stencil),new d(t,e,n,g,u,f,p,l)};var i,o,a,s,A=null;function l(t){return[t.getParameter(t.FRAMEBUFFER_BINDING),t.getParameter(t.RENDERBUFFER_BINDING),t.getParameter(t.TEXTURE_BINDING_2D)]}function c(t,e){t.bindFramebuffer(t.FRAMEBUFFER,e[0]),t.bindRenderbuffer(t.RENDERBUFFER,e[1]),t.bindTexture(t.TEXTURE_2D,e[2])}function u(t){switch(t){case i:throw new Error("gl-fbo: Framebuffer unsupported");case o:throw new Error("gl-fbo: Framebuffer incomplete attachment");case a:throw new Error("gl-fbo: Framebuffer incomplete dimensions");case s:throw new Error("gl-fbo: Framebuffer incomplete missing attachment");default:throw new Error("gl-fbo: Framebuffer failed for unspecified reason")}}function g(t,e,n,i,o,a){if(!i)return null;var s=r(t,e,n,o,i);return s.magFilter=t.NEAREST,s.minFilter=t.NEAREST,s.mipSamples=1,s.bind(),t.framebufferTexture2D(t.FRAMEBUFFER,a,t.TEXTURE_2D,s.handle,0),s}function h(t,e,n,r,i){var o=t.createRenderbuffer();return t.bindRenderbuffer(t.RENDERBUFFER,o),t.renderbufferStorage(t.RENDERBUFFER,r,e,n),t.framebufferRenderbuffer(t.FRAMEBUFFER,i,t.RENDERBUFFER,o),o}function d(t,e,n,r,i,o,a,s){this.gl=t,this._shape=[0|e,0|n],this._destroyed=!1,this._ext=s,this.color=new Array(i);for(var d=0;d<i;++d)this.color[d]=null;this._color_rb=null,this.depth=null,this._depth_rb=null,this._colorType=r,this._useDepth=o,this._useStencil=a;var f=this,p=[0|e,0|n];Object.defineProperties(p,{0:{get:function(){return f._shape[0]},set:function(t){return f.width=t}},1:{get:function(){return f._shape[1]},set:function(t){return f.height=t}}}),this._shapeVector=p,function(t){var e=l(t.gl),n=t.gl,r=t.handle=n.createFramebuffer(),i=t._shape[0],o=t._shape[1],a=t.color.length,s=t._ext,d=t._useStencil,f=t._useDepth,p=t._colorType;n.bindFramebuffer(n.FRAMEBUFFER,r);for(var I=0;I<a;++I)t.color[I]=g(n,i,o,p,n.RGBA,n.COLOR_ATTACHMENT0+I);0===a?(t._color_rb=h(n,i,o,n.RGBA4,n.COLOR_ATTACHMENT0),s&&s.drawBuffersWEBGL(A[0])):a>1&&s.drawBuffersWEBGL(A[a]);var C=n.getExtension("WEBGL_depth_texture");C?d?t.depth=g(n,i,o,C.UNSIGNED_INT_24_8_WEBGL,n.DEPTH_STENCIL,n.DEPTH_STENCIL_ATTACHMENT):f&&(t.depth=g(n,i,o,n.UNSIGNED_SHORT,n.DEPTH_COMPONENT,n.DEPTH_ATTACHMENT)):f&&d?t._depth_rb=h(n,i,o,n.DEPTH_STENCIL,n.DEPTH_STENCIL_ATTACHMENT):f?t._depth_rb=h(n,i,o,n.DEPTH_COMPONENT16,n.DEPTH_ATTACHMENT):d&&(t._depth_rb=h(n,i,o,n.STENCIL_INDEX,n.STENCIL_ATTACHMENT));var m=n.checkFramebufferStatus(n.FRAMEBUFFER);if(m!==n.FRAMEBUFFER_COMPLETE){for(t._destroyed=!0,n.bindFramebuffer(n.FRAMEBUFFER,null),n.deleteFramebuffer(t.handle),t.handle=null,t.depth&&(t.depth.dispose(),t.depth=null),t._depth_rb&&(n.deleteRenderbuffer(t._depth_rb),t._depth_rb=null),I=0;I<t.color.length;++I)t.color[I].dispose(),t.color[I]=null;t._color_rb&&(n.deleteRenderbuffer(t._color_rb),t._color_rb=null),c(n,e),u(m)}c(n,e)}(this)}var f=d.prototype;function p(t,e,n){if(t._destroyed)throw new Error("gl-fbo: Can't resize destroyed FBO");if(t._shape[0]!==e||t._shape[1]!==n){var r=t.gl,i=r.getParameter(r.MAX_RENDERBUFFER_SIZE);if(e<0||e>i||n<0||n>i)throw new Error("gl-fbo: Can't resize FBO, invalid dimensions");t._shape[0]=e,t._shape[1]=n;for(var o=l(r),a=0;a<t.color.length;++a)t.color[a].shape=t._shape;t._color_rb&&(r.bindRenderbuffer(r.RENDERBUFFER,t._color_rb),r.renderbufferStorage(r.RENDERBUFFER,r.RGBA4,t._shape[0],t._shape[1])),t.depth&&(t.depth.shape=t._shape),t._depth_rb&&(r.bindRenderbuffer(r.RENDERBUFFER,t._depth_rb),t._useDepth&&t._useStencil?r.renderbufferStorage(r.RENDERBUFFER,r.DEPTH_STENCIL,t._shape[0],t._shape[1]):t._useDepth?r.renderbufferStorage(r.RENDERBUFFER,r.DEPTH_COMPONENT16,t._shape[0],t._shape[1]):t._useStencil&&r.renderbufferStorage(r.RENDERBUFFER,r.STENCIL_INDEX,t._shape[0],t._shape[1])),r.bindFramebuffer(r.FRAMEBUFFER,t.handle);var s=r.checkFramebufferStatus(r.FRAMEBUFFER);s!==r.FRAMEBUFFER_COMPLETE&&(t.dispose(),c(r,o),u(s)),c(r,o)}}Object.defineProperties(f,{shape:{get:function(){return this._destroyed?[0,0]:this._shapeVector},set:function(t){if(Array.isArray(t)||(t=[0|t,0|t]),2!==t.length)throw new Error("gl-fbo: Shape vector must be length 2");var e=0|t[0],n=0|t[1];return p(this,e,n),[e,n]},enumerable:!1},width:{get:function(){return this._destroyed?0:this._shape[0]},set:function(t){return p(this,t|=0,this._shape[1]),t},enumerable:!1},height:{get:function(){return this._destroyed?0:this._shape[1]},set:function(t){return t|=0,p(this,this._shape[0],t),t},enumerable:!1}}),f.bind=function(){if(!this._destroyed){var t=this.gl;t.bindFramebuffer(t.FRAMEBUFFER,this.handle),t.viewport(0,0,this._shape[0],this._shape[1])}},f.dispose=function(){if(!this._destroyed){this._destroyed=!0;var t=this.gl;t.deleteFramebuffer(this.handle),this.handle=null,this.depth&&(this.depth.dispose(),this.depth=null),this._depth_rb&&(t.deleteRenderbuffer(this._depth_rb),this._depth_rb=null);for(var e=0;e<this.color.length;++e)this.color[e].dispose(),this.color[e]=null;this._color_rb&&(t.deleteRenderbuffer(this._color_rb),this._color_rb=null)}}},{"gl-texture2d":322}],250:[function(t,e,n){var r=t("sprintf-js").sprintf,i=t("gl-constants/lookup"),o=t("glsl-shader-name"),a=t("add-line-numbers");e.exports=function(t,e,n){"use strict";var s=o(e)||"of unknown name (see npm glsl-shader-name)",A="unknown type";void 0!==n&&(A=n===i.FRAGMENT_SHADER?"fragment":"vertex");for(var l=r("Error compiling %s shader %s:\n",A,s),c=r("%s%s",l,t),u=t.split("\n"),g={},h=0;h<u.length;h++){var d=u[h];if(""!==d&&"\0"!==d){var f=parseInt(d.split(":")[2]);if(isNaN(f))throw new Error(r("Could not parse error: %s",d));g[f]=d}}for(var p=a(e).split("\n"),h=0;h<p.length;h++)if(g[h+3]||g[h+2]||g[h+1]){var I=p[h];if(l+=I+"\n",g[h+1]){var C=g[h+1];C=C.substr(C.split(":",3).join(":").length+1).trim(),l+=r("^^^ %s\n\n",C)}}return{long:l.trim(),short:c.trim()}}},{"add-line-numbers":61,"gl-constants/lookup":246,"glsl-shader-name":401,"sprintf-js":528}],251:[function(t,e,n){"use strict";e.exports=function(t,e){var n=t.gl,r=a(n,A.vertex,A.fragment),i=a(n,A.pickVertex,A.pickFragment),o=s(n),c=s(n),u=s(n),g=s(n),h=new l(t,r,i,o,c,u,g);return h.update(e),t.addObject(h),h};var r=t("binary-search-bounds"),i=t("iota-array"),o=t("typedarray-pool"),a=t("gl-shader"),s=t("gl-buffer"),A=t("./lib/shaders");function l(t,e,n,r,i,o,a){this.plot=t,this.shader=e,this.pickShader=n,this.positionBuffer=r,this.weightBuffer=i,this.colorBuffer=o,this.idBuffer=a,this.xData=[],this.yData=[],this.shape=[0,0],this.bounds=[1/0,1/0,-1/0,-1/0],this.pickOffset=0}var c,u=l.prototype,g=[0,0,1,0,0,1,1,0,1,1,0,1];u.draw=(c=[1,0,0,0,1,0,0,0,1],function(){var t=this.plot,e=this.shader,n=this.bounds,r=this.numVertices;if(!(r<=0)){var i=t.gl,o=t.dataBox,a=n[2]-n[0],s=n[3]-n[1],A=o[2]-o[0],l=o[3]-o[1];c[0]=2*a/A,c[4]=2*s/l,c[6]=2*(n[0]-o[0])/A-1,c[7]=2*(n[1]-o[1])/l-1,e.bind();var u=e.uniforms;u.viewTransform=c,u.shape=this.shape;var g=e.attributes;this.positionBuffer.bind(),g.position.pointer(),this.weightBuffer.bind(),g.weight.pointer(i.UNSIGNED_BYTE,!1),this.colorBuffer.bind(),g.color.pointer(i.UNSIGNED_BYTE,!0),i.drawArrays(i.TRIANGLES,0,r)}}),u.drawPick=function(){var t=[1,0,0,0,1,0,0,0,1],e=[0,0,0,0];return function(n){var r=this.plot,i=this.pickShader,o=this.bounds,a=this.numVertices;if(!(a<=0)){var s=r.gl,A=r.dataBox,l=o[2]-o[0],c=o[3]-o[1],u=A[2]-A[0],g=A[3]-A[1];t[0]=2*l/u,t[4]=2*c/g,t[6]=2*(o[0]-A[0])/u-1,t[7]=2*(o[1]-A[1])/g-1;for(var h=0;h<4;++h)e[h]=n>>8*h&255;this.pickOffset=n,i.bind();var d=i.uniforms;d.viewTransform=t,d.pickOffset=e,d.shape=this.shape;var f=i.attributes;return this.positionBuffer.bind(),f.position.pointer(),this.weightBuffer.bind(),f.weight.pointer(s.UNSIGNED_BYTE,!1),this.idBuffer.bind(),f.pickId.pointer(s.UNSIGNED_BYTE,!1),s.drawArrays(s.TRIANGLES,0,a),n+this.shape[0]*this.shape[1]}}}(),u.pick=function(t,e,n){var r=this.pickOffset,i=this.shape[0]*this.shape[1];if(n<r||n>=r+i)return null;var o=n-r,a=this.xData,s=this.yData;return{object:this,pointId:o,dataCoord:[a[o%this.shape[0]],s[o/this.shape[0]|0]]}},u.update=function(t){var e=(t=t||{}).shape||[0,0],n=t.x||i(e[0]),a=t.y||i(e[1]),s=t.z||new Float32Array(e[0]*e[1]);this.xData=n,this.yData=a;var A=t.colorLevels||[0],l=t.colorValues||[0,0,0,1],c=A.length,u=this.bounds,h=u[0]=n[0],d=u[1]=a[0],f=1/((u[2]=n[n.length-1])-h),p=1/((u[3]=a[a.length-1])-d),I=e[0],C=e[1];this.shape=[I,C];var m=(I-1)*(C-1)*(g.length>>>1);this.numVertices=m;for(var y=o.mallocUint8(4*m),E=o.mallocFloat32(2*m),v=o.mallocUint8(2*m),B=o.mallocUint32(m),b=0,Q=0;Q<C-1;++Q)for(var w=p*(a[Q]-d),x=p*(a[Q+1]-d),_=0;_<I-1;++_)for(var D=f*(n[_]-h),M=f*(n[_+1]-h),k=0;k<g.length;k+=2){var S,L,R,N,F=g[k],T=g[k+1],U=s[(Q+T)*I+(_+F)],O=r.le(A,U);if(O<0)S=l[0],L=l[1],R=l[2],N=l[3];else if(O===c-1)S=l[4*c-4],L=l[4*c-3],R=l[4*c-2],N=l[4*c-1];else{var G=(U-A[O])/(A[O+1]-A[O]),H=1-G,j=4*O,Y=4*(O+1);S=H*l[j]+G*l[Y],L=H*l[j+1]+G*l[Y+1],R=H*l[j+2]+G*l[Y+2],N=H*l[j+3]+G*l[Y+3]}y[4*b]=255*S,y[4*b+1]=255*L,y[4*b+2]=255*R,y[4*b+3]=255*N,E[2*b]=.5*D+.5*M,E[2*b+1]=.5*w+.5*x,v[2*b]=F,v[2*b+1]=T,B[b]=Q*I+_,b+=1}this.positionBuffer.update(E),this.weightBuffer.update(v),this.colorBuffer.update(y),this.idBuffer.update(B),o.free(E),o.free(y),o.free(v),o.free(B)},u.dispose=function(){this.shader.dispose(),this.pickShader.dispose(),this.positionBuffer.dispose(),this.weightBuffer.dispose(),this.colorBuffer.dispose(),this.idBuffer.dispose(),this.plot.removeObject(this)}},{"./lib/shaders":252,"binary-search-bounds":253,"gl-buffer":241,"gl-shader":301,"iota-array":416,"typedarray-pool":545}],252:[function(t,e,n){"use strict";var r=t("glslify");e.exports={fragment:r(["precision lowp float;\n#define GLSLIFY 1\nvarying vec4 fragColor;\nvoid main() {\n  gl_FragColor = vec4(fragColor.rgb * fragColor.a, fragColor.a);\n}\n"]),vertex:r(["precision mediump float;\n#define GLSLIFY 1\n\nattribute vec2 position;\nattribute vec4 color;\nattribute vec2 weight;\n\nuniform vec2 shape;\nuniform mat3 viewTransform;\n\nvarying vec4 fragColor;\n\nvoid main() {\n  vec3 vPosition = viewTransform * vec3( position + (weight-.5)/(shape-1.) , 1.0);\n  fragColor = color;\n  gl_Position = vec4(vPosition.xy, 0, vPosition.z);\n}\n"]),pickFragment:r(["precision mediump float;\n#define GLSLIFY 1\n\nvarying vec4 fragId;\nvarying vec2 vWeight;\n\nuniform vec2 shape;\nuniform vec4 pickOffset;\n\nvoid main() {\n  vec2 d = step(.5, vWeight);\n  vec4 id = fragId + pickOffset;\n  id.x += d.x + d.y*shape.x;\n\n  id.y += floor(id.x / 256.0);\n  id.x -= floor(id.x / 256.0) * 256.0;\n\n  id.z += floor(id.y / 256.0);\n  id.y -= floor(id.y / 256.0) * 256.0;\n\n  id.w += floor(id.z / 256.0);\n  id.z -= floor(id.z / 256.0) * 256.0;\n\n  gl_FragColor = id/255.;\n}\n"]),pickVertex:r(["precision mediump float;\n#define GLSLIFY 1\n\nattribute vec2 position;\nattribute vec4 pickId;\nattribute vec2 weight;\n\nuniform vec2 shape;\nuniform mat3 viewTransform;\n\nvarying vec4 fragId;\nvarying vec2 vWeight;\n\nvoid main() {\n  vWeight = weight;\n\n  fragId = pickId;\n\n  vec3 vPosition = viewTransform * vec3( position + (weight-.5)/(shape-1.) , 1.0);\n  gl_Position = vec4(vPosition.xy, 0, vPosition.z);\n}\n"])}},{glslify:409}],253:[function(t,e,n){arguments[4][111][0].apply(n,arguments)},{dup:111}],254:[function(t,e,n){var r=t("glslify"),i=t("gl-shader"),o=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position, nextPosition;\nattribute float arcLength, lineWidth;\nattribute vec4 color;\n\nuniform vec2 screenShape;\nuniform float pixelRatio;\nuniform mat4 model, view, projection;\n\nvarying vec4 fragColor;\nvarying vec3 worldPosition;\nvarying float pixelArcLength;\n\nvec4 project(vec3 p) {\n  return projection * view * model * vec4(p, 1.0);\n}\n\nvoid main() {\n  vec4 startPoint = project(position);\n  vec4 endPoint   = project(nextPosition);\n\n  vec2 A = startPoint.xy / startPoint.w;\n  vec2 B =   endPoint.xy /   endPoint.w;\n\n  float clipAngle = atan(\n    (B.y - A.y) * screenShape.y,\n    (B.x - A.x) * screenShape.x\n  );\n\n  vec2 offset = 0.5 * pixelRatio * lineWidth * vec2(\n    sin(clipAngle),\n    -cos(clipAngle)\n  ) / screenShape;\n\n  gl_Position = vec4(startPoint.xy + startPoint.w * offset, startPoint.zw);\n\n  worldPosition = position;\n  pixelArcLength = arcLength;\n  fragColor = color;\n}\n"]),a=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3      clipBounds[2];\nuniform sampler2D dashTexture;\nuniform float     dashScale;\nuniform float     opacity;\n\nvarying vec3    worldPosition;\nvarying float   pixelArcLength;\nvarying vec4    fragColor;\n\nvoid main() {\n  if (\n    outOfRange(clipBounds[0], clipBounds[1], worldPosition) ||\n    fragColor.a * opacity == 0.\n  ) discard;\n\n  float dashWeight = texture2D(dashTexture, vec2(dashScale * pixelArcLength, 0)).r;\n  if(dashWeight < 0.5) {\n    discard;\n  }\n  gl_FragColor = fragColor * opacity;\n}\n"]),s=r(["precision highp float;\n#define GLSLIFY 1\n\n#define FLOAT_MAX  1.70141184e38\n#define FLOAT_MIN  1.17549435e-38\n\nlowp vec4 encode_float_1540259130(highp float v) {\n  highp float av = abs(v);\n\n  //Handle special cases\n  if(av < FLOAT_MIN) {\n    return vec4(0.0, 0.0, 0.0, 0.0);\n  } else if(v > FLOAT_MAX) {\n    return vec4(127.0, 128.0, 0.0, 0.0) / 255.0;\n  } else if(v < -FLOAT_MAX) {\n    return vec4(255.0, 128.0, 0.0, 0.0) / 255.0;\n  }\n\n  highp vec4 c = vec4(0,0,0,0);\n\n  //Compute exponent and mantissa\n  highp float e = floor(log2(av));\n  highp float m = av * pow(2.0, -e) - 1.0;\n  \n  //Unpack mantissa\n  c[1] = floor(128.0 * m);\n  m -= c[1] / 128.0;\n  c[2] = floor(32768.0 * m);\n  m -= c[2] / 32768.0;\n  c[3] = floor(8388608.0 * m);\n  \n  //Unpack exponent\n  highp float ebias = e + 127.0;\n  c[0] = floor(ebias / 2.0);\n  ebias -= c[0] * 2.0;\n  c[1] += floor(ebias) * 128.0; \n\n  //Unpack sign bit\n  c[0] += 128.0 * step(0.0, -v);\n\n  //Scale back to range\n  return c / 255.0;\n}\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform float pickId;\nuniform vec3 clipBounds[2];\n\nvarying vec3 worldPosition;\nvarying float pixelArcLength;\nvarying vec4 fragColor;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], worldPosition)) discard;\n\n  gl_FragColor = vec4(pickId/255.0, encode_float_1540259130(pixelArcLength).xyz);\n}"]),A=[{name:"position",type:"vec3"},{name:"nextPosition",type:"vec3"},{name:"arcLength",type:"float"},{name:"lineWidth",type:"float"},{name:"color",type:"vec4"}];n.createShader=function(t){return i(t,o,a,null,A)},n.createPickShader=function(t){return i(t,o,s,null,A)}},{"gl-shader":301,glslify:409}],255:[function(t,e,n){"use strict";e.exports=function(t){var e=t.gl||t.scene&&t.scene.gl,n=c(e);n.attributes.position.location=0,n.attributes.nextPosition.location=1,n.attributes.arcLength.location=2,n.attributes.lineWidth.location=3,n.attributes.color.location=4;var a=u(e);a.attributes.position.location=0,a.attributes.nextPosition.location=1,a.attributes.arcLength.location=2,a.attributes.lineWidth.location=3,a.attributes.color.location=4;for(var s=r(e),l=i(e,[{buffer:s,size:3,offset:0,stride:48},{buffer:s,size:3,offset:12,stride:48},{buffer:s,size:1,offset:24,stride:48},{buffer:s,size:1,offset:28,stride:48},{buffer:s,size:4,offset:32,stride:48}]),g=A(new Array(1024),[256,1,4]),h=0;h<1024;++h)g.data[h]=255;var d=o(e,g);d.wrap=e.REPEAT;var f=new p(e,n,a,s,l,d);return f.update(t),f};var r=t("gl-buffer"),i=t("gl-vao"),o=t("gl-texture2d"),a=t("glsl-read-float"),s=t("binary-search-bounds"),A=t("ndarray"),l=t("./lib/shaders"),c=l.createShader,u=l.createPickShader,g=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];function h(t,e){for(var n=0,r=0;r<3;++r){var i=t[r]-e[r];n+=i*i}return Math.sqrt(n)}function d(t){for(var e=[[-1e6,-1e6,-1e6],[1e6,1e6,1e6]],n=0;n<3;++n)e[0][n]=Math.max(t[0][n],e[0][n]),e[1][n]=Math.min(t[1][n],e[1][n]);return e}function f(t,e,n,r){this.arcLength=t,this.position=e,this.index=n,this.dataCoordinate=r}function p(t,e,n,r,i,o){this.gl=t,this.shader=e,this.pickShader=n,this.buffer=r,this.vao=i,this.clipBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.points=[],this.arcLength=[],this.vertexCount=0,this.bounds=[[0,0,0],[0,0,0]],this.pickId=0,this.lineWidth=1,this.texture=o,this.dashScale=1,this.opacity=1,this.hasAlpha=!1,this.dirty=!0,this.pixelRatio=1}var I=p.prototype;I.isTransparent=function(){return this.hasAlpha},I.isOpaque=function(){return!this.hasAlpha},I.pickSlots=1,I.setPickBase=function(t){this.pickId=t},I.drawTransparent=I.draw=function(t){if(this.vertexCount){var e=this.gl,n=this.shader,r=this.vao;n.bind(),n.uniforms={model:t.model||g,view:t.view||g,projection:t.projection||g,clipBounds:d(this.clipBounds),dashTexture:this.texture.bind(),dashScale:this.dashScale/this.arcLength[this.arcLength.length-1],opacity:this.opacity,screenShape:[e.drawingBufferWidth,e.drawingBufferHeight],pixelRatio:this.pixelRatio},r.bind(),r.draw(e.TRIANGLE_STRIP,this.vertexCount),r.unbind()}},I.drawPick=function(t){if(this.vertexCount){var e=this.gl,n=this.pickShader,r=this.vao;n.bind(),n.uniforms={model:t.model||g,view:t.view||g,projection:t.projection||g,pickId:this.pickId,clipBounds:d(this.clipBounds),screenShape:[e.drawingBufferWidth,e.drawingBufferHeight],pixelRatio:this.pixelRatio},r.bind(),r.draw(e.TRIANGLE_STRIP,this.vertexCount),r.unbind()}},I.update=function(t){var e,n;this.dirty=!0;var r=!!t.connectGaps;"dashScale"in t&&(this.dashScale=t.dashScale),this.hasAlpha=!1,"opacity"in t&&(this.opacity=+t.opacity,this.opacity<1&&(this.hasAlpha=!0));var i=[],o=[],a=[],l=0,c=0,u=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],g=t.position||t.positions;if(g){var d=t.color||t.colors||[0,0,0,1],f=t.lineWidth||1,p=!1;t:for(e=1;e<g.length;++e){var I,C,m,y=g[e-1],E=g[e];for(o.push(l),a.push(y.slice()),n=0;n<3;++n){if(isNaN(y[n])||isNaN(E[n])||!isFinite(y[n])||!isFinite(E[n])){if(!r&&i.length>0){for(var v=0;v<24;++v)i.push(i[i.length-12]);c+=2,p=!0}continue t}u[0][n]=Math.min(u[0][n],y[n],E[n]),u[1][n]=Math.max(u[1][n],y[n],E[n])}Array.isArray(d[0])?(I=d.length>e-1?d[e-1]:d.length>0?d[d.length-1]:[0,0,0,1],C=d.length>e?d[e]:d.length>0?d[d.length-1]:[0,0,0,1]):I=C=d,3===I.length&&(I=[I[0],I[1],I[2],1]),3===C.length&&(C=[C[0],C[1],C[2],1]),!this.hasAlpha&&I[3]<1&&(this.hasAlpha=!0),m=Array.isArray(f)?f.length>e-1?f[e-1]:f.length>0?f[f.length-1]:[0,0,0,1]:f;var B=l;if(l+=h(y,E),p){for(n=0;n<2;++n)i.push(y[0],y[1],y[2],E[0],E[1],E[2],B,m,I[0],I[1],I[2],I[3]);c+=2,p=!1}i.push(y[0],y[1],y[2],E[0],E[1],E[2],B,m,I[0],I[1],I[2],I[3],y[0],y[1],y[2],E[0],E[1],E[2],B,-m,I[0],I[1],I[2],I[3],E[0],E[1],E[2],y[0],y[1],y[2],l,-m,C[0],C[1],C[2],C[3],E[0],E[1],E[2],y[0],y[1],y[2],l,m,C[0],C[1],C[2],C[3]),c+=4}}if(this.buffer.update(i),o.push(l),a.push(g[g.length-1].slice()),this.bounds=u,this.vertexCount=c,this.points=a,this.arcLength=o,"dashes"in t){var b=t.dashes.slice();for(b.unshift(0),e=1;e<b.length;++e)b[e]=b[e-1]+b[e];var Q=A(new Array(1024),[256,1,4]);for(e=0;e<256;++e){for(n=0;n<4;++n)Q.set(e,0,n,0);1&s.le(b,b[b.length-1]*e/255)?Q.set(e,0,0,0):Q.set(e,0,0,255)}this.texture.setPixels(Q)}},I.dispose=function(){this.shader.dispose(),this.vao.dispose(),this.buffer.dispose()},I.pick=function(t){if(!t)return null;if(t.id!==this.pickId)return null;var e=a(t.value[0],t.value[1],t.value[2],0),n=s.le(this.arcLength,e);if(n<0)return null;if(n===this.arcLength.length-1)return new f(this.arcLength[this.arcLength.length-1],this.points[this.points.length-1].slice(),n);for(var r=this.points[n],i=this.points[Math.min(n+1,this.points.length-1)],o=(e-this.arcLength[n])/(this.arcLength[n+1]-this.arcLength[n]),A=1-o,l=[0,0,0],c=0;c<3;++c)l[c]=A*r[c]+o*i[c];var u=Math.min(o<.5?n:n+1,this.points.length-1);return new f(e,l,u,this.points[u])}},{"./lib/shaders":254,"binary-search-bounds":256,"gl-buffer":241,"gl-texture2d":322,"gl-vao":327,"glsl-read-float":400,ndarray:450}],256:[function(t,e,n){arguments[4][111][0].apply(n,arguments)},{dup:111}],257:[function(t,e,n){e.exports=function(t,e){var n=e[0],r=e[1],i=e[2],o=e[3],a=n*o-i*r;return a?(a=1/a,t[0]=o*a,t[1]=-r*a,t[2]=-i*a,t[3]=n*a,t):null}},{}],258:[function(t,e,n){e.exports=function(t,e){var n=e[0],r=e[1],i=e[2],o=e[3],a=e[4],s=e[5],A=e[6],l=e[7],c=e[8],u=c*a-s*l,g=-c*o+s*A,h=l*o-a*A,d=n*u+r*g+i*h;return d?(d=1/d,t[0]=u*d,t[1]=(-c*r+i*l)*d,t[2]=(s*r-i*a)*d,t[3]=g*d,t[4]=(c*n-i*A)*d,t[5]=(-s*n+i*o)*d,t[6]=h*d,t[7]=(-l*n+r*A)*d,t[8]=(a*n-r*o)*d,t):null}},{}],259:[function(t,e,n){e.exports=function(t){var e=new Float32Array(16);return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[3],e[4]=t[4],e[5]=t[5],e[6]=t[6],e[7]=t[7],e[8]=t[8],e[9]=t[9],e[10]=t[10],e[11]=t[11],e[12]=t[12],e[13]=t[13],e[14]=t[14],e[15]=t[15],e}},{}],260:[function(t,e,n){e.exports=function(){var t=new Float32Array(16);return t[0]=1,t[1]=0,t[2]=0,t[3]=0,t[4]=0,t[5]=1,t[6]=0,t[7]=0,t[8]=0,t[9]=0,t[10]=1,t[11]=0,t[12]=0,t[13]=0,t[14]=0,t[15]=1,t}},{}],261:[function(t,e,n){e.exports=function(t){var e=t[0],n=t[1],r=t[2],i=t[3],o=t[4],a=t[5],s=t[6],A=t[7],l=t[8],c=t[9],u=t[10],g=t[11],h=t[12],d=t[13],f=t[14],p=t[15];return(e*a-n*o)*(u*p-g*f)-(e*s-r*o)*(c*p-g*d)+(e*A-i*o)*(c*f-u*d)+(n*s-r*a)*(l*p-g*h)-(n*A-i*a)*(l*f-u*h)+(r*A-i*s)*(l*d-c*h)}},{}],262:[function(t,e,n){e.exports=function(t,e){var n=e[0],r=e[1],i=e[2],o=e[3],a=n+n,s=r+r,A=i+i,l=n*a,c=r*a,u=r*s,g=i*a,h=i*s,d=i*A,f=o*a,p=o*s,I=o*A;return t[0]=1-u-d,t[1]=c+I,t[2]=g-p,t[3]=0,t[4]=c-I,t[5]=1-l-d,t[6]=h+f,t[7]=0,t[8]=g+p,t[9]=h-f,t[10]=1-l-u,t[11]=0,t[12]=0,t[13]=0,t[14]=0,t[15]=1,t}},{}],263:[function(t,e,n){e.exports=function(t,e,n){var r=e[0],i=e[1],o=e[2],a=e[3],s=r+r,A=i+i,l=o+o,c=r*s,u=r*A,g=r*l,h=i*A,d=i*l,f=o*l,p=a*s,I=a*A,C=a*l;return t[0]=1-(h+f),t[1]=u+C,t[2]=g-I,t[3]=0,t[4]=u-C,t[5]=1-(c+f),t[6]=d+p,t[7]=0,t[8]=g+I,t[9]=d-p,t[10]=1-(c+h),t[11]=0,t[12]=n[0],t[13]=n[1],t[14]=n[2],t[15]=1,t}},{}],264:[function(t,e,n){e.exports=function(t){return t[0]=1,t[1]=0,t[2]=0,t[3]=0,t[4]=0,t[5]=1,t[6]=0,t[7]=0,t[8]=0,t[9]=0,t[10]=1,t[11]=0,t[12]=0,t[13]=0,t[14]=0,t[15]=1,t}},{}],265:[function(t,e,n){e.exports=function(t,e){var n=e[0],r=e[1],i=e[2],o=e[3],a=e[4],s=e[5],A=e[6],l=e[7],c=e[8],u=e[9],g=e[10],h=e[11],d=e[12],f=e[13],p=e[14],I=e[15],C=n*s-r*a,m=n*A-i*a,y=n*l-o*a,E=r*A-i*s,v=r*l-o*s,B=i*l-o*A,b=c*f-u*d,Q=c*p-g*d,w=c*I-h*d,x=u*p-g*f,_=u*I-h*f,D=g*I-h*p,M=C*D-m*_+y*x+E*w-v*Q+B*b;return M?(M=1/M,t[0]=(s*D-A*_+l*x)*M,t[1]=(i*_-r*D-o*x)*M,t[2]=(f*B-p*v+I*E)*M,t[3]=(g*v-u*B-h*E)*M,t[4]=(A*w-a*D-l*Q)*M,t[5]=(n*D-i*w+o*Q)*M,t[6]=(p*y-d*B-I*m)*M,t[7]=(c*B-g*y+h*m)*M,t[8]=(a*_-s*w+l*b)*M,t[9]=(r*w-n*_-o*b)*M,t[10]=(d*v-f*y+I*C)*M,t[11]=(u*y-c*v-h*C)*M,t[12]=(s*Q-a*x-A*b)*M,t[13]=(n*x-r*Q+i*b)*M,t[14]=(f*m-d*E-p*C)*M,t[15]=(c*E-u*m+g*C)*M,t):null}},{}],266:[function(t,e,n){var r=t("./identity");e.exports=function(t,e,n,i){var o,a,s,A,l,c,u,g,h,d,f=e[0],p=e[1],I=e[2],C=i[0],m=i[1],y=i[2],E=n[0],v=n[1],B=n[2];return Math.abs(f-E)<1e-6&&Math.abs(p-v)<1e-6&&Math.abs(I-B)<1e-6?r(t):(u=f-E,g=p-v,h=I-B,d=1/Math.sqrt(u*u+g*g+h*h),o=m*(h*=d)-y*(g*=d),a=y*(u*=d)-C*h,s=C*g-m*u,(d=Math.sqrt(o*o+a*a+s*s))?(o*=d=1/d,a*=d,s*=d):(o=0,a=0,s=0),A=g*s-h*a,l=h*o-u*s,c=u*a-g*o,(d=Math.sqrt(A*A+l*l+c*c))?(A*=d=1/d,l*=d,c*=d):(A=0,l=0,c=0),t[0]=o,t[1]=A,t[2]=u,t[3]=0,t[4]=a,t[5]=l,t[6]=g,t[7]=0,t[8]=s,t[9]=c,t[10]=h,t[11]=0,t[12]=-(o*f+a*p+s*I),t[13]=-(A*f+l*p+c*I),t[14]=-(u*f+g*p+h*I),t[15]=1,t)}},{"./identity":264}],267:[function(t,e,n){e.exports=function(t,e,n){var r=e[0],i=e[1],o=e[2],a=e[3],s=e[4],A=e[5],l=e[6],c=e[7],u=e[8],g=e[9],h=e[10],d=e[11],f=e[12],p=e[13],I=e[14],C=e[15],m=n[0],y=n[1],E=n[2],v=n[3];return t[0]=m*r+y*s+E*u+v*f,t[1]=m*i+y*A+E*g+v*p,t[2]=m*o+y*l+E*h+v*I,t[3]=m*a+y*c+E*d+v*C,m=n[4],y=n[5],E=n[6],v=n[7],t[4]=m*r+y*s+E*u+v*f,t[5]=m*i+y*A+E*g+v*p,t[6]=m*o+y*l+E*h+v*I,t[7]=m*a+y*c+E*d+v*C,m=n[8],y=n[9],E=n[10],v=n[11],t[8]=m*r+y*s+E*u+v*f,t[9]=m*i+y*A+E*g+v*p,t[10]=m*o+y*l+E*h+v*I,t[11]=m*a+y*c+E*d+v*C,m=n[12],y=n[13],E=n[14],v=n[15],t[12]=m*r+y*s+E*u+v*f,t[13]=m*i+y*A+E*g+v*p,t[14]=m*o+y*l+E*h+v*I,t[15]=m*a+y*c+E*d+v*C,t}},{}],268:[function(t,e,n){e.exports=function(t,e,n,r,i,o,a){var s=1/(e-n),A=1/(r-i),l=1/(o-a);return t[0]=-2*s,t[1]=0,t[2]=0,t[3]=0,t[4]=0,t[5]=-2*A,t[6]=0,t[7]=0,t[8]=0,t[9]=0,t[10]=2*l,t[11]=0,t[12]=(e+n)*s,t[13]=(i+r)*A,t[14]=(a+o)*l,t[15]=1,t}},{}],269:[function(t,e,n){e.exports=function(t,e,n,r,i){var o=1/Math.tan(e/2),a=1/(r-i);return t[0]=o/n,t[1]=0,t[2]=0,t[3]=0,t[4]=0,t[5]=o,t[6]=0,t[7]=0,t[8]=0,t[9]=0,t[10]=(i+r)*a,t[11]=-1,t[12]=0,t[13]=0,t[14]=2*i*r*a,t[15]=0,t}},{}],270:[function(t,e,n){e.exports=function(t,e,n,r){var i,o,a,s,A,l,c,u,g,h,d,f,p,I,C,m,y,E,v,B,b,Q,w,x,_=r[0],D=r[1],M=r[2],k=Math.sqrt(_*_+D*D+M*M);return Math.abs(k)<1e-6?null:(_*=k=1/k,D*=k,M*=k,i=Math.sin(n),o=Math.cos(n),a=1-o,s=e[0],A=e[1],l=e[2],c=e[3],u=e[4],g=e[5],h=e[6],d=e[7],f=e[8],p=e[9],I=e[10],C=e[11],m=_*_*a+o,y=D*_*a+M*i,E=M*_*a-D*i,v=_*D*a-M*i,B=D*D*a+o,b=M*D*a+_*i,Q=_*M*a+D*i,w=D*M*a-_*i,x=M*M*a+o,t[0]=s*m+u*y+f*E,t[1]=A*m+g*y+p*E,t[2]=l*m+h*y+I*E,t[3]=c*m+d*y+C*E,t[4]=s*v+u*B+f*b,t[5]=A*v+g*B+p*b,t[6]=l*v+h*B+I*b,t[7]=c*v+d*B+C*b,t[8]=s*Q+u*w+f*x,t[9]=A*Q+g*w+p*x,t[10]=l*Q+h*w+I*x,t[11]=c*Q+d*w+C*x,e!==t&&(t[12]=e[12],t[13]=e[13],t[14]=e[14],t[15]=e[15]),t)}},{}],271:[function(t,e,n){e.exports=function(t,e,n){var r=Math.sin(n),i=Math.cos(n),o=e[4],a=e[5],s=e[6],A=e[7],l=e[8],c=e[9],u=e[10],g=e[11];return e!==t&&(t[0]=e[0],t[1]=e[1],t[2]=e[2],t[3]=e[3],t[12]=e[12],t[13]=e[13],t[14]=e[14],t[15]=e[15]),t[4]=o*i+l*r,t[5]=a*i+c*r,t[6]=s*i+u*r,t[7]=A*i+g*r,t[8]=l*i-o*r,t[9]=c*i-a*r,t[10]=u*i-s*r,t[11]=g*i-A*r,t}},{}],272:[function(t,e,n){e.exports=function(t,e,n){var r=Math.sin(n),i=Math.cos(n),o=e[0],a=e[1],s=e[2],A=e[3],l=e[8],c=e[9],u=e[10],g=e[11];return e!==t&&(t[4]=e[4],t[5]=e[5],t[6]=e[6],t[7]=e[7],t[12]=e[12],t[13]=e[13],t[14]=e[14],t[15]=e[15]),t[0]=o*i-l*r,t[1]=a*i-c*r,t[2]=s*i-u*r,t[3]=A*i-g*r,t[8]=o*r+l*i,t[9]=a*r+c*i,t[10]=s*r+u*i,t[11]=A*r+g*i,t}},{}],273:[function(t,e,n){e.exports=function(t,e,n){var r=Math.sin(n),i=Math.cos(n),o=e[0],a=e[1],s=e[2],A=e[3],l=e[4],c=e[5],u=e[6],g=e[7];return e!==t&&(t[8]=e[8],t[9]=e[9],t[10]=e[10],t[11]=e[11],t[12]=e[12],t[13]=e[13],t[14]=e[14],t[15]=e[15]),t[0]=o*i+l*r,t[1]=a*i+c*r,t[2]=s*i+u*r,t[3]=A*i+g*r,t[4]=l*i-o*r,t[5]=c*i-a*r,t[6]=u*i-s*r,t[7]=g*i-A*r,t}},{}],274:[function(t,e,n){e.exports=function(t,e,n){var r=n[0],i=n[1],o=n[2];return t[0]=e[0]*r,t[1]=e[1]*r,t[2]=e[2]*r,t[3]=e[3]*r,t[4]=e[4]*i,t[5]=e[5]*i,t[6]=e[6]*i,t[7]=e[7]*i,t[8]=e[8]*o,t[9]=e[9]*o,t[10]=e[10]*o,t[11]=e[11]*o,t[12]=e[12],t[13]=e[13],t[14]=e[14],t[15]=e[15],t}},{}],275:[function(t,e,n){e.exports=function(t,e,n){var r,i,o,a,s,A,l,c,u,g,h,d,f=n[0],p=n[1],I=n[2];return e===t?(t[12]=e[0]*f+e[4]*p+e[8]*I+e[12],t[13]=e[1]*f+e[5]*p+e[9]*I+e[13],t[14]=e[2]*f+e[6]*p+e[10]*I+e[14],t[15]=e[3]*f+e[7]*p+e[11]*I+e[15]):(r=e[0],i=e[1],o=e[2],a=e[3],s=e[4],A=e[5],l=e[6],c=e[7],u=e[8],g=e[9],h=e[10],d=e[11],t[0]=r,t[1]=i,t[2]=o,t[3]=a,t[4]=s,t[5]=A,t[6]=l,t[7]=c,t[8]=u,t[9]=g,t[10]=h,t[11]=d,t[12]=r*f+s*p+u*I+e[12],t[13]=i*f+A*p+g*I+e[13],t[14]=o*f+l*p+h*I+e[14],t[15]=a*f+c*p+d*I+e[15]),t}},{}],276:[function(t,e,n){e.exports=function(t,e){if(t===e){var n=e[1],r=e[2],i=e[3],o=e[6],a=e[7],s=e[11];t[1]=e[4],t[2]=e[8],t[3]=e[12],t[4]=n,t[6]=e[9],t[7]=e[13],t[8]=r,t[9]=o,t[11]=e[14],t[12]=i,t[13]=a,t[14]=s}else t[0]=e[0],t[1]=e[4],t[2]=e[8],t[3]=e[12],t[4]=e[1],t[5]=e[5],t[6]=e[9],t[7]=e[13],t[8]=e[2],t[9]=e[6],t[10]=e[10],t[11]=e[14],t[12]=e[3],t[13]=e[7],t[14]=e[11],t[15]=e[15];return t}},{}],277:[function(t,e,n){"use strict";e.exports=function(t,e){switch(e.length){case 0:break;case 1:t[0]=1/e[0];break;case 4:r(t,e);break;case 9:i(t,e);break;case 16:o(t,e);break;default:throw new Error("currently supports matrices up to 4x4")}return t};var r=t("gl-mat2/invert"),i=t("gl-mat3/invert"),o=t("gl-mat4/invert")},{"gl-mat2/invert":257,"gl-mat3/invert":258,"gl-mat4/invert":265}],278:[function(t,e,n){"use strict";var r=t("barycentric"),i=t("polytope-closest-point/lib/closest_point_2d.js");function o(t,e){for(var n=[0,0,0,0],r=0;r<4;++r)for(var i=0;i<4;++i)n[i]+=t[4*r+i]*e[r];return n}function a(t,e,n,r,i){for(var a=o(r,o(n,o(e,[t[0],t[1],t[2],1]))),s=0;s<3;++s)a[s]/=a[3];return[.5*i[0]*(1+a[0]),.5*i[1]*(1-a[1])]}e.exports=function(t,e,n,o,s,A){if(1===t.length)return[0,t[0].slice()];for(var l=new Array(t.length),c=0;c<t.length;++c)l[c]=a(t[c],n,o,s,A);for(var u=0,g=1/0,c=0;c<l.length;++c){for(var h=0,d=0;d<2;++d)h+=Math.pow(l[c][d]-e[d],2);h<g&&(g=h,u=c)}for(var f=function(t,e){if(2===t.length){for(var n=0,o=0,a=0;a<2;++a)n+=Math.pow(e[a]-t[0][a],2),o+=Math.pow(e[a]-t[1][a],2);return n=Math.sqrt(n),o=Math.sqrt(o),n+o<1e-6?[1,0]:[o/(n+o),n/(o+n)]}if(3===t.length){var s=[0,0];return i(t[0],t[1],t[2],e,s),r(t,s)}return[]}(l,e),p=0,c=0;c<3;++c){if(f[c]<-.001||f[c]>1.0001)return null;p+=f[c]}return Math.abs(p-1)>.001?null:[u,function(t,e){for(var n=[0,0,0],r=0;r<t.length;++r)for(var i=t[r],o=e[r],a=0;a<3;++a)n[a]+=o*i[a];return n}(t,f),f]}},{barycentric:73,"polytope-closest-point/lib/closest_point_2d.js":481}],279:[function(t,e,n){var r=t("glslify"),i=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position, normal;\nattribute vec4 color;\nattribute vec2 uv;\n\nuniform mat4 model\n           , view\n           , projection\n           , inverseModel;\nuniform vec3 eyePosition\n           , lightPosition;\n\nvarying vec3 f_normal\n           , f_lightDirection\n           , f_eyeDirection\n           , f_data;\nvarying vec4 f_color;\nvarying vec2 f_uv;\n\nvec4 project(vec3 p) {\n  return projection * view * model * vec4(p, 1.0);\n}\n\nvoid main() {\n  gl_Position      = project(position);\n\n  //Lighting geometry parameters\n  vec4 cameraCoordinate = view * vec4(position , 1.0);\n  cameraCoordinate.xyz /= cameraCoordinate.w;\n  f_lightDirection = lightPosition - cameraCoordinate.xyz;\n  f_eyeDirection   = eyePosition - cameraCoordinate.xyz;\n  f_normal  = normalize((vec4(normal, 0.0) * inverseModel).xyz);\n\n  f_color          = color;\n  f_data           = position;\n  f_uv             = uv;\n}\n"]),o=r(["#extension GL_OES_standard_derivatives : enable\n\nprecision highp float;\n#define GLSLIFY 1\n\nfloat beckmannDistribution(float x, float roughness) {\n  float NdotH = max(x, 0.0001);\n  float cos2Alpha = NdotH * NdotH;\n  float tan2Alpha = (cos2Alpha - 1.0) / cos2Alpha;\n  float roughness2 = roughness * roughness;\n  float denom = 3.141592653589793 * roughness2 * cos2Alpha * cos2Alpha;\n  return exp(tan2Alpha / roughness2) / denom;\n}\n\nfloat cookTorranceSpecular(\n  vec3 lightDirection,\n  vec3 viewDirection,\n  vec3 surfaceNormal,\n  float roughness,\n  float fresnel) {\n\n  float VdotN = max(dot(viewDirection, surfaceNormal), 0.0);\n  float LdotN = max(dot(lightDirection, surfaceNormal), 0.0);\n\n  //Half angle vector\n  vec3 H = normalize(lightDirection + viewDirection);\n\n  //Geometric term\n  float NdotH = max(dot(surfaceNormal, H), 0.0);\n  float VdotH = max(dot(viewDirection, H), 0.000001);\n  float LdotH = max(dot(lightDirection, H), 0.000001);\n  float G1 = (2.0 * NdotH * VdotN) / VdotH;\n  float G2 = (2.0 * NdotH * LdotN) / LdotH;\n  float G = min(1.0, min(G1, G2));\n  \n  //Distribution term\n  float D = beckmannDistribution(NdotH, roughness);\n\n  //Fresnel term\n  float F = pow(1.0 - VdotN, fresnel);\n\n  //Multiply terms and done\n  return  G * F * D / max(3.14159265 * VdotN, 0.000001);\n}\n\n//#pragma glslify: beckmann = require(glsl-specular-beckmann) // used in gl-surface3d\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 clipBounds[2];\nuniform float roughness\n            , fresnel\n            , kambient\n            , kdiffuse\n            , kspecular;\nuniform sampler2D texture;\n\nvarying vec3 f_normal\n           , f_lightDirection\n           , f_eyeDirection\n           , f_data;\nvarying vec4 f_color;\nvarying vec2 f_uv;\n\nvoid main() {\n  if (f_color.a == 0.0 ||\n    outOfRange(clipBounds[0], clipBounds[1], f_data)\n  ) discard;\n\n  vec3 N = normalize(f_normal);\n  vec3 L = normalize(f_lightDirection);\n  vec3 V = normalize(f_eyeDirection);\n\n  if(gl_FrontFacing) {\n    N = -N;\n  }\n\n  float specular = min(1.0, max(0.0, cookTorranceSpecular(L, V, N, roughness, fresnel)));\n  //float specular = max(0.0, beckmann(L, V, N, roughness)); // used in gl-surface3d\n\n  float diffuse  = min(kambient + kdiffuse * max(dot(N, L), 0.0), 1.0);\n\n  vec4 surfaceColor = vec4(f_color.rgb, 1.0) * texture2D(texture, f_uv);\n  vec4 litColor = surfaceColor.a * vec4(diffuse * surfaceColor.rgb + kspecular * vec3(1,1,1) * specular,  1.0);\n\n  gl_FragColor = litColor * f_color.a;\n}\n"]),a=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position;\nattribute vec4 color;\nattribute vec2 uv;\n\nuniform mat4 model, view, projection;\n\nvarying vec4 f_color;\nvarying vec3 f_data;\nvarying vec2 f_uv;\n\nvoid main() {\n  gl_Position = projection * view * model * vec4(position, 1.0);\n  f_color = color;\n  f_data  = position;\n  f_uv    = uv;\n}"]),s=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 clipBounds[2];\nuniform sampler2D texture;\nuniform float opacity;\n\nvarying vec4 f_color;\nvarying vec3 f_data;\nvarying vec2 f_uv;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], f_data)) discard;\n\n  gl_FragColor = f_color * texture2D(texture, f_uv) * opacity;\n}"]),A=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nattribute vec3 position;\nattribute vec4 color;\nattribute vec2 uv;\nattribute float pointSize;\n\nuniform mat4 model, view, projection;\nuniform vec3 clipBounds[2];\n\nvarying vec4 f_color;\nvarying vec2 f_uv;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], position)) {\n\n    gl_Position = vec4(0.0, 0.0 ,0.0 ,0.0);\n  } else {\n    gl_Position = projection * view * model * vec4(position, 1.0);\n  }\n  gl_PointSize = pointSize;\n  f_color = color;\n  f_uv = uv;\n}"]),l=r(["precision highp float;\n#define GLSLIFY 1\n\nuniform sampler2D texture;\nuniform float opacity;\n\nvarying vec4 f_color;\nvarying vec2 f_uv;\n\nvoid main() {\n  vec2 pointR = gl_PointCoord.xy - vec2(0.5, 0.5);\n  if(dot(pointR, pointR) > 0.25) {\n    discard;\n  }\n  gl_FragColor = f_color * texture2D(texture, f_uv) * opacity;\n}"]),c=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position;\nattribute vec4 id;\n\nuniform mat4 model, view, projection;\n\nvarying vec3 f_position;\nvarying vec4 f_id;\n\nvoid main() {\n  gl_Position = projection * view * model * vec4(position, 1.0);\n  f_id        = id;\n  f_position  = position;\n}"]),u=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3  clipBounds[2];\nuniform float pickId;\n\nvarying vec3 f_position;\nvarying vec4 f_id;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], f_position)) discard;\n\n  gl_FragColor = vec4(pickId, f_id.xyz);\n}"]),g=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nattribute vec3  position;\nattribute float pointSize;\nattribute vec4  id;\n\nuniform mat4 model, view, projection;\nuniform vec3 clipBounds[2];\n\nvarying vec3 f_position;\nvarying vec4 f_id;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], position)) {\n\n    gl_Position = vec4(0.0, 0.0, 0.0, 0.0);\n  } else {\n    gl_Position  = projection * view * model * vec4(position, 1.0);\n    gl_PointSize = pointSize;\n  }\n  f_id         = id;\n  f_position   = position;\n}"]),h=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position;\n\nuniform mat4 model, view, projection;\n\nvoid main() {\n  gl_Position = projection * view * model * vec4(position, 1.0);\n}"]),d=r(["precision highp float;\n#define GLSLIFY 1\n\nuniform vec3 contourColor;\n\nvoid main() {\n  gl_FragColor = vec4(contourColor, 1.0);\n}\n"]);n.meshShader={vertex:i,fragment:o,attributes:[{name:"position",type:"vec3"},{name:"normal",type:"vec3"},{name:"color",type:"vec4"},{name:"uv",type:"vec2"}]},n.wireShader={vertex:a,fragment:s,attributes:[{name:"position",type:"vec3"},{name:"color",type:"vec4"},{name:"uv",type:"vec2"}]},n.pointShader={vertex:A,fragment:l,attributes:[{name:"position",type:"vec3"},{name:"color",type:"vec4"},{name:"uv",type:"vec2"},{name:"pointSize",type:"float"}]},n.pickShader={vertex:c,fragment:u,attributes:[{name:"position",type:"vec3"},{name:"id",type:"vec4"}]},n.pointPickShader={vertex:g,fragment:u,attributes:[{name:"position",type:"vec3"},{name:"pointSize",type:"float"},{name:"id",type:"vec4"}]},n.contourShader={vertex:h,fragment:d,attributes:[{name:"position",type:"vec3"}]}},{glslify:409}],280:[function(t,e,n){"use strict";var r=t("gl-shader"),i=t("gl-buffer"),o=t("gl-vao"),a=t("gl-texture2d"),s=t("normals"),A=t("gl-mat4/multiply"),l=t("gl-mat4/invert"),c=t("ndarray"),u=t("colormap"),g=t("simplicial-complex-contour"),h=t("typedarray-pool"),d=t("./lib/shaders"),f=t("./lib/closest-point"),p=d.meshShader,I=d.wireShader,C=d.pointShader,m=d.pickShader,y=d.pointPickShader,E=d.contourShader,v=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];function B(t,e,n,r,i,o,a,s,A,l,c,u,g,h,d,f,p,I,C,m,y,E,B,b,Q,w,x){this.gl=t,this.pixelRatio=1,this.cells=[],this.positions=[],this.intensity=[],this.texture=e,this.dirty=!0,this.triShader=n,this.lineShader=r,this.pointShader=i,this.pickShader=o,this.pointPickShader=a,this.contourShader=s,this.trianglePositions=A,this.triangleColors=c,this.triangleNormals=g,this.triangleUVs=u,this.triangleIds=l,this.triangleVAO=h,this.triangleCount=0,this.lineWidth=1,this.edgePositions=d,this.edgeColors=p,this.edgeUVs=I,this.edgeIds=f,this.edgeVAO=C,this.edgeCount=0,this.pointPositions=m,this.pointColors=E,this.pointUVs=B,this.pointSizes=b,this.pointIds=y,this.pointVAO=Q,this.pointCount=0,this.contourLineWidth=1,this.contourPositions=w,this.contourVAO=x,this.contourCount=0,this.contourColor=[0,0,0],this.contourEnable=!0,this.pickId=1,this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],this.clipBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.lightPosition=[1e5,1e5,0],this.ambientLight=.8,this.diffuseLight=.8,this.specularLight=2,this.roughness=.5,this.fresnel=1.5,this.opacity=1,this.hasAlpha=!1,this.opacityscale=!1,this._model=v,this._view=v,this._projection=v,this._resolution=[1,1]}var b=B.prototype;function Q(t,e){if(!e)return 1;if(!e.length)return 1;for(var n=0;n<e.length;++n){if(e.length<2)return 1;if(e[n][0]===t)return e[n][1];if(e[n][0]>t&&n>0){var r=(e[n][0]-t)/(e[n][0]-e[n-1][0]);return e[n][1]*(1-r)+r*e[n-1][1]}}return 1}function w(t){var e=r(t,C.vertex,C.fragment);return e.attributes.position.location=0,e.attributes.color.location=2,e.attributes.uv.location=3,e.attributes.pointSize.location=4,e}function x(t){var e=r(t,m.vertex,m.fragment);return e.attributes.position.location=0,e.attributes.id.location=1,e}function _(t){var e=r(t,y.vertex,y.fragment);return e.attributes.position.location=0,e.attributes.id.location=1,e.attributes.pointSize.location=4,e}function D(t){var e=r(t,E.vertex,E.fragment);return e.attributes.position.location=0,e}b.isOpaque=function(){return!this.hasAlpha},b.isTransparent=function(){return this.hasAlpha},b.pickSlots=1,b.setPickBase=function(t){this.pickId=t},b.highlight=function(t){if(t&&this.contourEnable){for(var e=g(this.cells,this.intensity,t.intensity),n=e.cells,r=e.vertexIds,i=e.vertexWeights,o=n.length,a=h.mallocFloat32(6*o),s=0,A=0;A<o;++A)for(var l=n[A],c=0;c<2;++c){var u=l[0];2===l.length&&(u=l[c]);for(var d=r[u][0],f=r[u][1],p=i[u],I=1-p,C=this.positions[d],m=this.positions[f],y=0;y<3;++y)a[s++]=p*C[y]+I*m[y]}this.contourCount=s/3|0,this.contourPositions.update(a.subarray(0,s)),h.free(a)}else this.contourCount=0},b.update=function(t){t=t||{};var e=this.gl;this.dirty=!0,"contourEnable"in t&&(this.contourEnable=t.contourEnable),"contourColor"in t&&(this.contourColor=t.contourColor),"lineWidth"in t&&(this.lineWidth=t.lineWidth),"lightPosition"in t&&(this.lightPosition=t.lightPosition),this.hasAlpha=!1,"opacity"in t&&(this.opacity=t.opacity,this.opacity<1&&(this.hasAlpha=!0)),"opacityscale"in t&&(this.opacityscale=t.opacityscale,this.hasAlpha=!0),"ambient"in t&&(this.ambientLight=t.ambient),"diffuse"in t&&(this.diffuseLight=t.diffuse),"specular"in t&&(this.specularLight=t.specular),"roughness"in t&&(this.roughness=t.roughness),"fresnel"in t&&(this.fresnel=t.fresnel),t.texture?(this.texture.dispose(),this.texture=a(e,t.texture)):t.colormap&&(this.texture.shape=[256,256],this.texture.minFilter=e.LINEAR_MIPMAP_LINEAR,this.texture.magFilter=e.LINEAR,this.texture.setPixels(function(t,e){for(var n=u({colormap:t,nshades:256,format:"rgba"}),r=new Uint8Array(1024),i=0;i<256;++i){for(var o=n[i],a=0;a<3;++a)r[4*i+a]=o[a];r[4*i+3]=e?255*Q(i/255,e):255*o[3]}return c(r,[256,256,4],[4,0,1])}(t.colormap,this.opacityscale)),this.texture.generateMipmap());var n=t.cells,r=t.positions;if(r&&n){var i=[],o=[],A=[],l=[],g=[],h=[],d=[],f=[],p=[],I=[],C=[],m=[],y=[],E=[];this.cells=n,this.positions=r;var v=t.vertexNormals,B=t.cellNormals,b=void 0===t.vertexNormalsEpsilon?1e-6:t.vertexNormalsEpsilon,w=void 0===t.faceNormalsEpsilon?1e-6:t.faceNormalsEpsilon;t.useFacetNormals&&!B&&(B=s.faceNormals(n,r,w)),B||v||(v=s.vertexNormals(n,r,b));var x=t.vertexColors,_=t.cellColors,D=t.meshColor||[1,1,1,1],M=t.vertexUVs,k=t.vertexIntensity,S=t.cellUVs,L=t.cellIntensity,R=1/0,N=-1/0;if(!M&&!S)if(k)if(t.vertexIntensityBounds)R=+t.vertexIntensityBounds[0],N=+t.vertexIntensityBounds[1];else for(var F=0;F<k.length;++F){var T=k[F];R=Math.min(R,T),N=Math.max(N,T)}else if(L)for(F=0;F<L.length;++F)T=L[F],R=Math.min(R,T),N=Math.max(N,T);else for(F=0;F<r.length;++F)T=r[F][2],R=Math.min(R,T),N=Math.max(N,T);this.intensity=k||(L?function(t,e,n){for(var r=new Array(e),i=0;i<e;++i)r[i]=0;var o=t.length;for(i=0;i<o;++i)for(var a=t[i],s=0;s<a.length;++s)r[a[s]]=n[i];return r}(n,r.length,L):function(t){for(var e=t.length,n=new Array(e),r=0;r<e;++r)n[r]=t[r][2];return n}(r));var U=t.pointSizes,O=t.pointSize||1;for(this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],F=0;F<r.length;++F)for(var G=r[F],H=0;H<3;++H)!isNaN(G[H])&&isFinite(G[H])&&(this.bounds[0][H]=Math.min(this.bounds[0][H],G[H]),this.bounds[1][H]=Math.max(this.bounds[1][H],G[H]));var j=0,Y=0,P=0;t:for(F=0;F<n.length;++F){var K=n[F];switch(K.length){case 1:for(G=r[J=K[0]],H=0;H<3;++H)if(isNaN(G[H])||!isFinite(G[H]))continue t;I.push(G[0],G[1],G[2]),q=x?x[J]:_?_[F]:D,this.opacityscale&&k?o.push(q[0],q[1],q[2],this.opacity*Q((k[J]-R)/(N-R),this.opacityscale)):3===q.length?C.push(q[0],q[1],q[2],this.opacity):(C.push(q[0],q[1],q[2],q[3]*this.opacity),!this.hasAlpha&&q[3]<1&&(this.hasAlpha=!0)),V=M?M[J]:k?[(k[J]-R)/(N-R),0]:S?S[F]:L?[(L[F]-R)/(N-R),0]:[(G[2]-R)/(N-R),0],m.push(V[0],V[1]),U?y.push(U[J]):y.push(O),E.push(F),P+=1;break;case 2:for(H=0;H<2;++H){G=r[J=K[H]];for(var z=0;z<3;++z)if(isNaN(G[z])||!isFinite(G[z]))continue t}for(H=0;H<2;++H)G=r[J=K[H]],h.push(G[0],G[1],G[2]),q=x?x[J]:_?_[F]:D,this.opacityscale&&k?o.push(q[0],q[1],q[2],this.opacity*Q((k[J]-R)/(N-R),this.opacityscale)):3===q.length?d.push(q[0],q[1],q[2],this.opacity):(d.push(q[0],q[1],q[2],q[3]*this.opacity),!this.hasAlpha&&q[3]<1&&(this.hasAlpha=!0)),V=M?M[J]:k?[(k[J]-R)/(N-R),0]:S?S[F]:L?[(L[F]-R)/(N-R),0]:[(G[2]-R)/(N-R),0],f.push(V[0],V[1]),p.push(F);Y+=1;break;case 3:for(H=0;H<3;++H)for(G=r[J=K[H]],z=0;z<3;++z)if(isNaN(G[z])||!isFinite(G[z]))continue t;for(H=0;H<3;++H){var J,q,V,W;G=r[J=K[2-H]],i.push(G[0],G[1],G[2]),q=x?x[J]:_?_[F]:D,this.opacityscale&&k?o.push(q[0],q[1],q[2],this.opacity*Q((k[J]-R)/(N-R),this.opacityscale)):3===q.length?o.push(q[0],q[1],q[2],this.opacity):(o.push(q[0],q[1],q[2],q[3]*this.opacity),!this.hasAlpha&&q[3]<1&&(this.hasAlpha=!0)),V=M?M[J]:k?[(k[J]-R)/(N-R),0]:S?S[F]:L?[(L[F]-R)/(N-R),0]:[(G[2]-R)/(N-R),0],l.push(V[0],V[1]),W=v?v[J]:B[F],A.push(W[0],W[1],W[2]),g.push(F)}j+=1}}this.pointCount=P,this.edgeCount=Y,this.triangleCount=j,this.pointPositions.update(I),this.pointColors.update(C),this.pointUVs.update(m),this.pointSizes.update(y),this.pointIds.update(new Uint32Array(E)),this.edgePositions.update(h),this.edgeColors.update(d),this.edgeUVs.update(f),this.edgeIds.update(new Uint32Array(p)),this.trianglePositions.update(i),this.triangleColors.update(o),this.triangleUVs.update(l),this.triangleNormals.update(A),this.triangleIds.update(new Uint32Array(g))}},b.drawTransparent=b.draw=function(t){t=t||{};for(var e=this.gl,n=t.model||v,r=t.view||v,i=t.projection||v,o=[[-1e6,-1e6,-1e6],[1e6,1e6,1e6]],a=0;a<3;++a)o[0][a]=Math.max(o[0][a],this.clipBounds[0][a]),o[1][a]=Math.min(o[1][a],this.clipBounds[1][a]);var s={model:n,view:r,projection:i,inverseModel:v.slice(),clipBounds:o,kambient:this.ambientLight,kdiffuse:this.diffuseLight,kspecular:this.specularLight,roughness:this.roughness,fresnel:this.fresnel,eyePosition:[0,0,0],lightPosition:[0,0,0],contourColor:this.contourColor,texture:0};s.inverseModel=l(s.inverseModel,s.model),e.disable(e.CULL_FACE),this.texture.bind(0);var c=new Array(16);for(A(c,s.view,s.model),A(c,s.projection,c),l(c,c),a=0;a<3;++a)s.eyePosition[a]=c[12+a]/c[15];var u,g=c[15];for(a=0;a<3;++a)g+=this.lightPosition[a]*c[4*a+3];for(a=0;a<3;++a){for(var h=c[12+a],d=0;d<3;++d)h+=c[4*d+a]*this.lightPosition[d];s.lightPosition[a]=h/g}this.triangleCount>0&&((u=this.triShader).bind(),u.uniforms=s,this.triangleVAO.bind(),e.drawArrays(e.TRIANGLES,0,3*this.triangleCount),this.triangleVAO.unbind()),this.edgeCount>0&&this.lineWidth>0&&((u=this.lineShader).bind(),u.uniforms=s,this.edgeVAO.bind(),e.lineWidth(this.lineWidth*this.pixelRatio),e.drawArrays(e.LINES,0,2*this.edgeCount),this.edgeVAO.unbind()),this.pointCount>0&&((u=this.pointShader).bind(),u.uniforms=s,this.pointVAO.bind(),e.drawArrays(e.POINTS,0,this.pointCount),this.pointVAO.unbind()),this.contourEnable&&this.contourCount>0&&this.contourLineWidth>0&&((u=this.contourShader).bind(),u.uniforms=s,this.contourVAO.bind(),e.drawArrays(e.LINES,0,this.contourCount),this.contourVAO.unbind())},b.drawPick=function(t){t=t||{};for(var e=this.gl,n=t.model||v,r=t.view||v,i=t.projection||v,o=[[-1e6,-1e6,-1e6],[1e6,1e6,1e6]],a=0;a<3;++a)o[0][a]=Math.max(o[0][a],this.clipBounds[0][a]),o[1][a]=Math.min(o[1][a],this.clipBounds[1][a]);this._model=[].slice.call(n),this._view=[].slice.call(r),this._projection=[].slice.call(i),this._resolution=[e.drawingBufferWidth,e.drawingBufferHeight];var s,A={model:n,view:r,projection:i,clipBounds:o,pickId:this.pickId/255};(s=this.pickShader).bind(),s.uniforms=A,this.triangleCount>0&&(this.triangleVAO.bind(),e.drawArrays(e.TRIANGLES,0,3*this.triangleCount),this.triangleVAO.unbind()),this.edgeCount>0&&(this.edgeVAO.bind(),e.lineWidth(this.lineWidth*this.pixelRatio),e.drawArrays(e.LINES,0,2*this.edgeCount),this.edgeVAO.unbind()),this.pointCount>0&&((s=this.pointPickShader).bind(),s.uniforms=A,this.pointVAO.bind(),e.drawArrays(e.POINTS,0,this.pointCount),this.pointVAO.unbind())},b.pick=function(t){if(!t)return null;if(t.id!==this.pickId)return null;for(var e=t.value[0]+256*t.value[1]+65536*t.value[2],n=this.cells[e],r=this.positions,i=new Array(n.length),o=0;o<n.length;++o)i[o]=r[n[o]];var a=f(i,[t.coord[0],this._resolution[1]-t.coord[1]],this._model,this._view,this._projection,this._resolution);if(!a)return null;var s=a[2],A=0;for(o=0;o<n.length;++o)A+=s[o]*this.intensity[n[o]];return{position:a[1],index:n[a[0]],cell:n,cellId:e,intensity:A,dataCoordinate:this.positions[n[a[0]]]}},b.dispose=function(){this.texture.dispose(),this.triShader.dispose(),this.lineShader.dispose(),this.pointShader.dispose(),this.pickShader.dispose(),this.pointPickShader.dispose(),this.triangleVAO.dispose(),this.trianglePositions.dispose(),this.triangleColors.dispose(),this.triangleUVs.dispose(),this.triangleNormals.dispose(),this.triangleIds.dispose(),this.edgeVAO.dispose(),this.edgePositions.dispose(),this.edgeColors.dispose(),this.edgeUVs.dispose(),this.edgeIds.dispose(),this.pointVAO.dispose(),this.pointPositions.dispose(),this.pointColors.dispose(),this.pointUVs.dispose(),this.pointSizes.dispose(),this.pointIds.dispose(),this.contourVAO.dispose(),this.contourPositions.dispose(),this.contourShader.dispose()},e.exports=function(t,e){if(1===arguments.length&&(t=(e=t).gl),!(t.getExtension("OES_standard_derivatives")||t.getExtension("MOZ_OES_standard_derivatives")||t.getExtension("WEBKIT_OES_standard_derivatives")))throw new Error("derivatives not supported");var n=function(t){var e=r(t,p.vertex,p.fragment);return e.attributes.position.location=0,e.attributes.color.location=2,e.attributes.uv.location=3,e.attributes.normal.location=4,e}(t),s=function(t){var e=r(t,I.vertex,I.fragment);return e.attributes.position.location=0,e.attributes.color.location=2,e.attributes.uv.location=3,e}(t),A=w(t),l=x(t),u=_(t),g=D(t),h=a(t,c(new Uint8Array([255,255,255,255]),[1,1,4]));h.generateMipmap(),h.minFilter=t.LINEAR_MIPMAP_LINEAR,h.magFilter=t.LINEAR;var d=i(t),f=i(t),C=i(t),m=i(t),y=i(t),E=o(t,[{buffer:d,type:t.FLOAT,size:3},{buffer:y,type:t.UNSIGNED_BYTE,size:4,normalized:!0},{buffer:f,type:t.FLOAT,size:4},{buffer:C,type:t.FLOAT,size:2},{buffer:m,type:t.FLOAT,size:3}]),v=i(t),b=i(t),Q=i(t),M=i(t),k=o(t,[{buffer:v,type:t.FLOAT,size:3},{buffer:M,type:t.UNSIGNED_BYTE,size:4,normalized:!0},{buffer:b,type:t.FLOAT,size:4},{buffer:Q,type:t.FLOAT,size:2}]),S=i(t),L=i(t),R=i(t),N=i(t),F=i(t),T=o(t,[{buffer:S,type:t.FLOAT,size:3},{buffer:F,type:t.UNSIGNED_BYTE,size:4,normalized:!0},{buffer:L,type:t.FLOAT,size:4},{buffer:R,type:t.FLOAT,size:2},{buffer:N,type:t.FLOAT,size:1}]),U=i(t),O=new B(t,h,n,s,A,l,u,g,d,y,f,C,m,E,v,M,b,Q,k,S,F,L,R,N,T,U,o(t,[{buffer:U,type:t.FLOAT,size:3}]));return O.update(e),O}},{"./lib/closest-point":278,"./lib/shaders":279,colormap:126,"gl-buffer":241,"gl-mat4/invert":265,"gl-mat4/multiply":267,"gl-shader":301,"gl-texture2d":322,"gl-vao":327,ndarray:450,normals:453,"simplicial-complex-contour":518,"typedarray-pool":545}],281:[function(t,e,n){"use strict";e.exports=function(t){var e=t.gl,n=r(e,[0,0,0,1,1,0,1,1]),s=i(e,o.boxVert,o.lineFrag);return new a(t,n,s)};var r=t("gl-buffer"),i=t("gl-shader"),o=t("./shaders");function a(t,e,n){this.plot=t,this.vbo=e,this.shader=n}var s,A,l=a.prototype;l.bind=function(){var t=this.shader;this.vbo.bind(),this.shader.bind(),t.attributes.coord.pointer(),t.uniforms.screenBox=this.plot.screenBox},l.drawBox=(s=[0,0],A=[0,0],function(t,e,n,r,i){var o=this.plot,a=this.shader,l=o.gl;s[0]=t,s[1]=e,A[0]=n,A[1]=r,a.uniforms.lo=s,a.uniforms.hi=A,a.uniforms.color=i,l.drawArrays(l.TRIANGLE_STRIP,0,4)}),l.dispose=function(){this.vbo.dispose(),this.shader.dispose()}},{"./shaders":284,"gl-buffer":241,"gl-shader":301}],282:[function(t,e,n){"use strict";e.exports=function(t){var e=t.gl,n=r(e),o=i(e,a.gridVert,a.gridFrag),A=i(e,a.tickVert,a.gridFrag);return new s(t,n,o,A)};var r=t("gl-buffer"),i=t("gl-shader"),o=t("binary-search-bounds"),a=t("./shaders");function s(t,e,n,r){this.plot=t,this.vbo=e,this.shader=n,this.tickShader=r,this.ticks=[[],[]]}function A(t,e){return t-e}var l,c,u,g,h,d=s.prototype;d.draw=(l=[0,0],c=[0,0],u=[0,0],function(){for(var t=this.plot,e=this.vbo,n=this.shader,r=this.ticks,i=t.gl,o=t._tickBounds,a=t.dataBox,s=t.viewBox,A=t.gridLineWidth,g=t.gridLineColor,h=t.gridLineEnable,d=t.pixelRatio,f=0;f<2;++f){var p=o[f],I=o[f+2]-p,C=.5*(a[f+2]+a[f]),m=a[f+2]-a[f];c[f]=2*I/m,l[f]=2*(p-C)/m}n.bind(),e.bind(),n.attributes.dataCoord.pointer(),n.uniforms.dataShift=l,n.uniforms.dataScale=c;var y=0;for(f=0;f<2;++f){u[0]=u[1]=0,u[f]=1,n.uniforms.dataAxis=u,n.uniforms.lineWidth=A[f]/(s[f+2]-s[f])*d,n.uniforms.color=g[f];var E=6*r[f].length;h[f]&&E&&i.drawArrays(i.TRIANGLES,y,E),y+=E}}),d.drawTickMarks=function(){var t=[0,0],e=[0,0],n=[1,0],r=[0,1],i=[0,0],a=[0,0];return function(){for(var s=this.plot,l=this.vbo,c=this.tickShader,u=this.ticks,g=s.gl,h=s._tickBounds,d=s.dataBox,f=s.viewBox,p=s.pixelRatio,I=s.screenBox,C=I[2]-I[0],m=I[3]-I[1],y=f[2]-f[0],E=f[3]-f[1],v=0;v<2;++v){var B=h[v],b=h[v+2]-B,Q=.5*(d[v+2]+d[v]),w=d[v+2]-d[v];e[v]=2*b/w,t[v]=2*(B-Q)/w}e[0]*=y/C,t[0]*=y/C,e[1]*=E/m,t[1]*=E/m,c.bind(),l.bind(),c.attributes.dataCoord.pointer();var x=c.uniforms;x.dataShift=t,x.dataScale=e;var _=s.tickMarkLength,D=s.tickMarkWidth,M=s.tickMarkColor,k=6*u[0].length,S=Math.min(o.ge(u[0],(d[0]-h[0])/(h[2]-h[0]),A),u[0].length),L=Math.min(o.gt(u[0],(d[2]-h[0])/(h[2]-h[0]),A),u[0].length),R=0+6*S,N=6*Math.max(0,L-S),F=Math.min(o.ge(u[1],(d[1]-h[1])/(h[3]-h[1]),A),u[1].length),T=Math.min(o.gt(u[1],(d[3]-h[1])/(h[3]-h[1]),A),u[1].length),U=k+6*F,O=6*Math.max(0,T-F);i[0]=2*(f[0]-_[1])/C-1,i[1]=(f[3]+f[1])/m-1,a[0]=_[1]*p/C,a[1]=D[1]*p/m,O&&(x.color=M[1],x.tickScale=a,x.dataAxis=r,x.screenOffset=i,g.drawArrays(g.TRIANGLES,U,O)),i[0]=(f[2]+f[0])/C-1,i[1]=2*(f[1]-_[0])/m-1,a[0]=D[0]*p/C,a[1]=_[0]*p/m,N&&(x.color=M[0],x.tickScale=a,x.dataAxis=n,x.screenOffset=i,g.drawArrays(g.TRIANGLES,R,N)),i[0]=2*(f[2]+_[3])/C-1,i[1]=(f[3]+f[1])/m-1,a[0]=_[3]*p/C,a[1]=D[3]*p/m,O&&(x.color=M[3],x.tickScale=a,x.dataAxis=r,x.screenOffset=i,g.drawArrays(g.TRIANGLES,U,O)),i[0]=(f[2]+f[0])/C-1,i[1]=2*(f[3]+_[2])/m-1,a[0]=D[2]*p/C,a[1]=_[2]*p/m,N&&(x.color=M[2],x.tickScale=a,x.dataAxis=n,x.screenOffset=i,g.drawArrays(g.TRIANGLES,R,N))}}(),d.update=(g=[1,1,-1,-1,1,-1],h=[1,-1,1,1,-1,-1],function(t){for(var e=t.ticks,n=t.bounds,r=new Float32Array(18*(e[0].length+e[1].length)),i=(this.plot.zeroLineEnable,0),o=[[],[]],a=0;a<2;++a)for(var s=o[a],A=e[a],l=n[a],c=n[a+2],u=0;u<A.length;++u){var d=(A[u].x-l)/(c-l);s.push(d);for(var f=0;f<6;++f)r[i++]=d,r[i++]=g[f],r[i++]=h[f]}this.ticks=o,this.vbo.update(r)}),d.dispose=function(){this.vbo.dispose(),this.shader.dispose(),this.tickShader.dispose()}},{"./shaders":284,"binary-search-bounds":286,"gl-buffer":241,"gl-shader":301}],283:[function(t,e,n){"use strict";e.exports=function(t){var e=t.gl,n=r(e,[-1,-1,-1,1,1,-1,1,1]),s=i(e,o.lineVert,o.lineFrag);return new a(t,n,s)};var r=t("gl-buffer"),i=t("gl-shader"),o=t("./shaders");function a(t,e,n){this.plot=t,this.vbo=e,this.shader=n}var s,A,l=a.prototype;l.bind=function(){var t=this.shader;this.vbo.bind(),this.shader.bind(),t.attributes.coord.pointer(),t.uniforms.screenBox=this.plot.screenBox},l.drawLine=(s=[0,0],A=[0,0],function(t,e,n,r,i,o){var a=this.plot,l=this.shader,c=a.gl;s[0]=t,s[1]=e,A[0]=n,A[1]=r,l.uniforms.start=s,l.uniforms.end=A,l.uniforms.width=i*a.pixelRatio,l.uniforms.color=o,c.drawArrays(c.TRIANGLE_STRIP,0,4)}),l.dispose=function(){this.vbo.dispose(),this.shader.dispose()}},{"./shaders":284,"gl-buffer":241,"gl-shader":301}],284:[function(t,e,n){"use strict";var r=t("glslify"),i=r(["precision lowp float;\n#define GLSLIFY 1\nuniform vec4 color;\nvoid main() {\n  gl_FragColor = vec4(color.xyz * color.w, color.w);\n}\n"]);e.exports={lineVert:r(["precision mediump float;\n#define GLSLIFY 1\n\nattribute vec2 coord;\n\nuniform vec4 screenBox;\nuniform vec2 start, end;\nuniform float width;\n\nvec2 perp(vec2 v) {\n  return vec2(v.y, -v.x);\n}\n\nvec2 screen(vec2 v) {\n  return 2.0 * (v - screenBox.xy) / (screenBox.zw - screenBox.xy) - 1.0;\n}\n\nvoid main() {\n  vec2 delta = normalize(perp(start - end));\n  vec2 offset = mix(start, end, 0.5 * (coord.y+1.0));\n  gl_Position = vec4(screen(offset + 0.5 * width * delta * coord.x), 0, 1);\n}\n"]),lineFrag:i,textVert:r(["#define GLSLIFY 1\nattribute vec3 textCoordinate;\n\nuniform vec2 dataScale, dataShift, dataAxis, screenOffset, textScale;\nuniform float angle;\n\nvoid main() {\n  float dataOffset  = textCoordinate.z;\n  vec2 glyphOffset  = textCoordinate.xy;\n  mat2 glyphMatrix = mat2(cos(angle), sin(angle), -sin(angle), cos(angle));\n  vec2 screenCoordinate = dataAxis * (dataScale * dataOffset + dataShift) +\n    glyphMatrix * glyphOffset * textScale + screenOffset;\n  gl_Position = vec4(screenCoordinate, 0, 1);\n}\n"]),textFrag:i,gridVert:r(["precision mediump float;\n#define GLSLIFY 1\n\nattribute vec3 dataCoord;\n\nuniform vec2 dataAxis, dataShift, dataScale;\nuniform float lineWidth;\n\nvoid main() {\n  vec2 pos = dataAxis * (dataScale * dataCoord.x + dataShift);\n  pos += 10.0 * dataCoord.y * vec2(dataAxis.y, -dataAxis.x) + dataCoord.z * lineWidth;\n  gl_Position = vec4(pos, 0, 1);\n}\n"]),gridFrag:i,boxVert:r(["precision mediump float;\n#define GLSLIFY 1\n\nattribute vec2 coord;\n\nuniform vec4 screenBox;\nuniform vec2 lo, hi;\n\nvec2 screen(vec2 v) {\n  return 2.0 * (v - screenBox.xy) / (screenBox.zw - screenBox.xy) - 1.0;\n}\n\nvoid main() {\n  gl_Position = vec4(screen(mix(lo, hi, coord)), 0, 1);\n}\n"]),tickVert:r(["precision mediump float;\n#define GLSLIFY 1\n\nattribute vec3 dataCoord;\n\nuniform vec2 dataAxis, dataShift, dataScale, screenOffset, tickScale;\n\nvoid main() {\n  vec2 pos = dataAxis * (dataScale * dataCoord.x + dataShift);\n  gl_Position = vec4(pos + tickScale*dataCoord.yz + screenOffset, 0, 1);\n}\n"])}},{glslify:409}],285:[function(t,e,n){"use strict";e.exports=function(t){var e=t.gl,n=r(e),o=i(e,s.textVert,s.textFrag);return new A(t,n,o)};var r=t("gl-buffer"),i=t("gl-shader"),o=t("text-cache"),a=t("binary-search-bounds"),s=t("./shaders");function A(t,e,n){this.plot=t,this.vbo=e,this.shader=n,this.tickOffset=[[],[]],this.tickX=[[],[]],this.labelOffset=[0,0],this.labelCount=[0,0]}var l,c,u,g,h,d,f=A.prototype;f.drawTicks=(l=[0,0],c=[0,0],u=[0,0],function(t){var e=this.plot,n=this.shader,r=this.tickX[t],i=this.tickOffset[t],o=e.gl,s=e.viewBox,A=e.dataBox,g=e.screenBox,h=e.pixelRatio,d=e.tickEnable,f=e.tickPad,p=e.tickColor,I=e.tickAngle,C=e.labelEnable,m=e.labelPad,y=e.labelColor,E=e.labelAngle,v=this.labelOffset[t],B=this.labelCount[t],b=a.lt(r,A[t]),Q=a.le(r,A[t+2]);l[0]=l[1]=0,l[t]=1,c[t]=(s[2+t]+s[t])/(g[2+t]-g[t])-1;var w=2/g[2+(1^t)]-g[1^t];c[1^t]=w*s[1^t]-1,d[t]&&(c[1^t]-=w*h*f[t],b<Q&&i[Q]>i[b]&&(n.uniforms.dataAxis=l,n.uniforms.screenOffset=c,n.uniforms.color=p[t],n.uniforms.angle=I[t],o.drawArrays(o.TRIANGLES,i[b],i[Q]-i[b]))),C[t]&&B&&(c[1^t]-=w*h*m[t],n.uniforms.dataAxis=u,n.uniforms.screenOffset=c,n.uniforms.color=y[t],n.uniforms.angle=E[t],o.drawArrays(o.TRIANGLES,v,B)),c[1^t]=w*s[2+(1^t)]-1,d[t+2]&&(c[1^t]+=w*h*f[t+2],b<Q&&i[Q]>i[b]&&(n.uniforms.dataAxis=l,n.uniforms.screenOffset=c,n.uniforms.color=p[t+2],n.uniforms.angle=I[t+2],o.drawArrays(o.TRIANGLES,i[b],i[Q]-i[b]))),C[t+2]&&B&&(c[1^t]+=w*h*m[t+2],n.uniforms.dataAxis=u,n.uniforms.screenOffset=c,n.uniforms.color=y[t+2],n.uniforms.angle=E[t+2],o.drawArrays(o.TRIANGLES,v,B))}),f.drawTitle=function(){var t=[0,0],e=[0,0];return function(){var n=this.plot,r=this.shader,i=n.gl,o=n.screenBox,a=n.titleCenter,s=n.titleAngle,A=n.titleColor,l=n.pixelRatio;if(this.titleCount){for(var c=0;c<2;++c)e[c]=2*(a[c]*l-o[c])/(o[2+c]-o[c])-1;r.bind(),r.uniforms.dataAxis=t,r.uniforms.screenOffset=e,r.uniforms.angle=s,r.uniforms.color=A,i.drawArrays(i.TRIANGLES,this.titleOffset,this.titleCount)}}}(),f.bind=(g=[0,0],h=[0,0],d=[0,0],function(){var t=this.plot,e=this.shader,n=t._tickBounds,r=t.dataBox,i=t.screenBox,o=t.viewBox;e.bind();for(var a=0;a<2;++a){var s=n[a],A=n[a+2]-s,l=.5*(r[a+2]+r[a]),c=r[a+2]-r[a],u=o[a],f=o[a+2]-u,p=i[a],I=i[a+2]-p;h[a]=2*A/c*f/I,g[a]=2*(s-l)/c*f/I}d[1]=2*t.pixelRatio/(i[3]-i[1]),d[0]=d[1]*(i[3]-i[1])/(i[2]-i[0]),e.uniforms.dataScale=h,e.uniforms.dataShift=g,e.uniforms.textScale=d,this.vbo.bind(),e.attributes.textCoordinate.pointer()}),f.update=function(t){var e,n,r,i,a,s=[],A=t.ticks,l=t.bounds;for(a=0;a<2;++a){var c=[Math.floor(s.length/3)],u=[-1/0],g=A[a];for(e=0;e<g.length;++e){var h=g[e],d=h.x,f=h.text,p=h.font||"sans-serif";i=h.fontSize||12;for(var I=1/(l[a+2]-l[a]),C=l[a],m=f.split("\n"),y=0;y<m.length;y++)for(r=o(p,m[y]).data,n=0;n<r.length;n+=2)s.push(r[n]*i,-r[n+1]*i-y*i*1.2,(d-C)*I);c.push(Math.floor(s.length/3)),u.push(d)}this.tickOffset[a]=c,this.tickX[a]=u}for(a=0;a<2;++a){for(this.labelOffset[a]=Math.floor(s.length/3),r=o(t.labelFont[a],t.labels[a],{textAlign:"center"}).data,i=t.labelSize[a],e=0;e<r.length;e+=2)s.push(r[e]*i,-r[e+1]*i,0);this.labelCount[a]=Math.floor(s.length/3)-this.labelOffset[a]}for(this.titleOffset=Math.floor(s.length/3),r=o(t.titleFont,t.title).data,i=t.titleSize,e=0;e<r.length;e+=2)s.push(r[e]*i,-r[e+1]*i,0);this.titleCount=Math.floor(s.length/3)-this.titleOffset,this.vbo.update(s)},f.dispose=function(){this.vbo.dispose(),this.shader.dispose()}},{"./shaders":284,"binary-search-bounds":286,"gl-buffer":241,"gl-shader":301,"text-cache":536}],286:[function(t,e,n){arguments[4][111][0].apply(n,arguments)},{dup:111}],287:[function(t,e,n){"use strict";e.exports=function(t){var e=t.gl,n=r(e,[e.drawingBufferWidth,e.drawingBufferHeight]),l=new A(e,n);return l.grid=i(l),l.text=o(l),l.line=a(l),l.box=s(l),l.update(t),l};var r=t("gl-select-static"),i=t("./lib/grid"),o=t("./lib/text"),a=t("./lib/line"),s=t("./lib/box");function A(t,e){this.gl=t,this.pickBuffer=e,this.screenBox=[0,0,t.drawingBufferWidth,t.drawingBufferHeight],this.viewBox=[0,0,0,0],this.dataBox=[-10,-10,10,10],this.gridLineEnable=[!0,!0],this.gridLineWidth=[1,1],this.gridLineColor=[[0,0,0,1],[0,0,0,1]],this.pixelRatio=1,this.tickMarkLength=[0,0,0,0],this.tickMarkWidth=[0,0,0,0],this.tickMarkColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.tickPad=[15,15,15,15],this.tickAngle=[0,0,0,0],this.tickEnable=[!0,!0,!0,!0],this.tickColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.labelPad=[15,15,15,15],this.labelAngle=[0,Math.PI/2,0,3*Math.PI/2],this.labelEnable=[!0,!0,!0,!0],this.labelColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.titleCenter=[0,0],this.titleEnable=!0,this.titleAngle=0,this.titleColor=[0,0,0,1],this.borderColor=[0,0,0,0],this.backgroundColor=[0,0,0,0],this.zeroLineEnable=[!0,!0],this.zeroLineWidth=[4,4],this.zeroLineColor=[[0,0,0,1],[0,0,0,1]],this.borderLineEnable=[!0,!0,!0,!0],this.borderLineWidth=[2,2,2,2],this.borderLineColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.grid=null,this.text=null,this.line=null,this.box=null,this.objects=[],this.overlays=[],this._tickBounds=[1/0,1/0,-1/0,-1/0],this.static=!1,this.dirty=!1,this.pickDirty=!1,this.pickDelay=120,this.pickRadius=10,this._pickTimeout=null,this._drawPick=this.drawPick.bind(this),this._depthCounter=0}var l=A.prototype;function c(t){for(var e=t.slice(),n=0;n<e.length;++n)e[n]=e[n].slice();return e}function u(t,e){return t.x-e.x}l.setDirty=function(){this.dirty=this.pickDirty=!0},l.setOverlayDirty=function(){this.dirty=!0},l.nextDepthValue=function(){return this._depthCounter++/65536},l.draw=function(){var t=this.gl,e=this.screenBox,n=this.viewBox,r=this.dataBox,i=this.pixelRatio,o=this.grid,a=this.line,s=this.text,A=this.objects;if(this._depthCounter=0,this.pickDirty&&(this._pickTimeout&&clearTimeout(this._pickTimeout),this.pickDirty=!1,this._pickTimeout=setTimeout(this._drawPick,this.pickDelay)),this.dirty){if(this.dirty=!1,t.bindFramebuffer(t.FRAMEBUFFER,null),t.enable(t.SCISSOR_TEST),t.disable(t.DEPTH_TEST),t.depthFunc(t.LESS),t.depthMask(!1),t.enable(t.BLEND),t.blendEquation(t.FUNC_ADD,t.FUNC_ADD),t.blendFunc(t.ONE,t.ONE_MINUS_SRC_ALPHA),this.borderColor){t.scissor(e[0],e[1],e[2]-e[0],e[3]-e[1]);var l=this.borderColor;t.clearColor(l[0]*l[3],l[1]*l[3],l[2]*l[3],l[3]),t.clear(t.COLOR_BUFFER_BIT|t.DEPTH_BUFFER_BIT)}t.scissor(n[0],n[1],n[2]-n[0],n[3]-n[1]),t.viewport(n[0],n[1],n[2]-n[0],n[3]-n[1]);var c=this.backgroundColor;t.clearColor(c[0]*c[3],c[1]*c[3],c[2]*c[3],c[3]),t.clear(t.COLOR_BUFFER_BIT),o.draw();var u=this.zeroLineEnable,g=this.zeroLineColor,h=this.zeroLineWidth;if(u[0]||u[1]){a.bind();for(var d=0;d<2;++d)if(u[d]&&r[d]<=0&&r[d+2]>=0){var f=e[d]-r[d]*(e[d+2]-e[d])/(r[d+2]-r[d]);0===d?a.drawLine(f,e[1],f,e[3],h[d],g[d]):a.drawLine(e[0],f,e[2],f,h[d],g[d])}}for(d=0;d<A.length;++d)A[d].draw();t.viewport(e[0],e[1],e[2]-e[0],e[3]-e[1]),t.scissor(e[0],e[1],e[2]-e[0],e[3]-e[1]),this.grid.drawTickMarks(),a.bind();var p=this.borderLineEnable,I=this.borderLineWidth,C=this.borderLineColor;for(p[1]&&a.drawLine(n[0],n[1]-.5*I[1]*i,n[0],n[3]+.5*I[3]*i,I[1],C[1]),p[0]&&a.drawLine(n[0]-.5*I[0]*i,n[1],n[2]+.5*I[2]*i,n[1],I[0],C[0]),p[3]&&a.drawLine(n[2],n[1]-.5*I[1]*i,n[2],n[3]+.5*I[3]*i,I[3],C[3]),p[2]&&a.drawLine(n[0]-.5*I[0]*i,n[3],n[2]+.5*I[2]*i,n[3],I[2],C[2]),s.bind(),d=0;d<2;++d)s.drawTicks(d);this.titleEnable&&s.drawTitle();var m=this.overlays;for(d=0;d<m.length;++d)m[d].draw();t.disable(t.SCISSOR_TEST),t.disable(t.BLEND),t.depthMask(!0)}},l.drawPick=function(){if(!this.static){var t=this.pickBuffer;this.gl,this._pickTimeout=null,t.begin();for(var e=1,n=this.objects,r=0;r<n.length;++r)e=n[r].drawPick(e);t.end()}},l.pick=function(t,e){if(!this.static){var n=this.pixelRatio,r=this.pickPixelRatio,i=this.viewBox,o=0|Math.round((t-i[0]/n)*r),a=0|Math.round((e-i[1]/n)*r),s=this.pickBuffer.query(o,a,this.pickRadius);if(!s)return null;for(var A=s.id+(s.value[0]<<8)+(s.value[1]<<16)+(s.value[2]<<24),l=this.objects,c=0;c<l.length;++c){var u=l[c].pick(o,a,A);if(u)return u}return null}},l.setScreenBox=function(t){var e=this.screenBox,n=this.pixelRatio;e[0]=0|Math.round(t[0]*n),e[1]=0|Math.round(t[1]*n),e[2]=0|Math.round(t[2]*n),e[3]=0|Math.round(t[3]*n),this.setDirty()},l.setDataBox=function(t){var e=this.dataBox;(e[0]!==t[0]||e[1]!==t[1]||e[2]!==t[2]||e[3]!==t[3])&&(e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[3],this.setDirty())},l.setViewBox=function(t){var e=this.pixelRatio,n=this.viewBox;n[0]=0|Math.round(t[0]*e),n[1]=0|Math.round(t[1]*e),n[2]=0|Math.round(t[2]*e),n[3]=0|Math.round(t[3]*e);var r=this.pickPixelRatio;this.pickBuffer.shape=[0|Math.round((t[2]-t[0])*r),0|Math.round((t[3]-t[1])*r)],this.setDirty()},l.update=function(t){t=t||{};var e=this.gl;this.pixelRatio=t.pixelRatio||1;var n=this.pixelRatio;this.pickPixelRatio=Math.max(n,1),this.setScreenBox(t.screenBox||[0,0,e.drawingBufferWidth/n,e.drawingBufferHeight/n]),this.screenBox,this.setViewBox(t.viewBox||[.125*(this.screenBox[2]-this.screenBox[0])/n,.125*(this.screenBox[3]-this.screenBox[1])/n,.875*(this.screenBox[2]-this.screenBox[0])/n,.875*(this.screenBox[3]-this.screenBox[1])/n]);var r=this.viewBox,i=(r[2]-r[0])/(r[3]-r[1]);this.setDataBox(t.dataBox||[-10,-10/i,10,10/i]),this.borderColor=!1!==t.borderColor&&(t.borderColor||[0,0,0,0]).slice(),this.backgroundColor=(t.backgroundColor||[0,0,0,0]).slice(),this.gridLineEnable=(t.gridLineEnable||[!0,!0]).slice(),this.gridLineWidth=(t.gridLineWidth||[1,1]).slice(),this.gridLineColor=c(t.gridLineColor||[[.5,.5,.5,1],[.5,.5,.5,1]]),this.zeroLineEnable=(t.zeroLineEnable||[!0,!0]).slice(),this.zeroLineWidth=(t.zeroLineWidth||[4,4]).slice(),this.zeroLineColor=c(t.zeroLineColor||[[0,0,0,1],[0,0,0,1]]),this.tickMarkLength=(t.tickMarkLength||[0,0,0,0]).slice(),this.tickMarkWidth=(t.tickMarkWidth||[0,0,0,0]).slice(),this.tickMarkColor=c(t.tickMarkColor||[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]]),this.titleCenter=(t.titleCenter||[.5*(r[0]+r[2])/n,(r[3]+120)/n]).slice(),this.titleEnable=!("titleEnable"in t&&!t.titleEnable),this.titleAngle=t.titleAngle||0,this.titleColor=(t.titleColor||[0,0,0,1]).slice(),this.labelPad=(t.labelPad||[15,15,15,15]).slice(),this.labelAngle=(t.labelAngle||[0,Math.PI/2,0,3*Math.PI/2]).slice(),this.labelEnable=(t.labelEnable||[!0,!0,!0,!0]).slice(),this.labelColor=c(t.labelColor||[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]]),this.tickPad=(t.tickPad||[15,15,15,15]).slice(),this.tickAngle=(t.tickAngle||[0,0,0,0]).slice(),this.tickEnable=(t.tickEnable||[!0,!0,!0,!0]).slice(),this.tickColor=c(t.tickColor||[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]]),this.borderLineEnable=(t.borderLineEnable||[!0,!0,!0,!0]).slice(),this.borderLineWidth=(t.borderLineWidth||[2,2,2,2]).slice(),this.borderLineColor=c(t.borderLineColor||[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]]);var o=t.ticks||[[],[]],a=this._tickBounds;a[0]=a[1]=1/0,a[2]=a[3]=-1/0;for(var s=0;s<2;++s){var A=o[s].slice(0);0!==A.length&&(A.sort(u),a[s]=Math.min(a[s],A[0].x),a[s+2]=Math.max(a[s+2],A[A.length-1].x))}this.grid.update({bounds:a,ticks:o}),this.text.update({bounds:a,ticks:o,labels:t.labels||["x","y"],labelSize:t.labelSize||[12,12],labelFont:t.labelFont||["sans-serif","sans-serif"],title:t.title||"",titleSize:t.titleSize||18,titleFont:t.titleFont||"sans-serif"}),this.static=!!t.static,this.setDirty()},l.dispose=function(){this.box.dispose(),this.grid.dispose(),this.text.dispose(),this.line.dispose();for(var t=this.objects.length-1;t>=0;--t)this.objects[t].dispose();for(this.objects.length=0,t=this.overlays.length-1;t>=0;--t)this.overlays[t].dispose();this.overlays.length=0,this.gl=null},l.addObject=function(t){this.objects.indexOf(t)<0&&(this.objects.push(t),this.setDirty())},l.removeObject=function(t){for(var e=this.objects,n=0;n<e.length;++n)if(e[n]===t){e.splice(n,1),this.setDirty();break}},l.addOverlay=function(t){this.overlays.indexOf(t)<0&&(this.overlays.push(t),this.setOverlayDirty())},l.removeOverlay=function(t){for(var e=this.overlays,n=0;n<e.length;++n)if(e[n]===t){e.splice(n,1),this.setOverlayDirty();break}}},{"./lib/box":281,"./lib/grid":282,"./lib/line":283,"./lib/text":285,"gl-select-static":300}],288:[function(t,e,n){"use strict";e.exports=function(t,e){t=t||document.body;var n=[.01,1/0];"distanceLimits"in(e=e||{})&&(n[0]=e.distanceLimits[0],n[1]=e.distanceLimits[1]),"zoomMin"in e&&(n[0]=e.zoomMin),"zoomMax"in e&&(n[1]=e.zoomMax);var l=i({center:e.center||[0,0,0],up:e.up||[0,1,0],eye:e.eye||[0,0,10],mode:e.mode||"orbit",distanceLimits:n}),c=[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],u=0,g=t.clientWidth,h=t.clientHeight,d={keyBindingMode:"rotate",enableWheel:!0,view:l,element:t,delay:e.delay||16,rotateSpeed:e.rotateSpeed||1,zoomSpeed:e.zoomSpeed||1,translateSpeed:e.translateSpeed||1,flipX:!!e.flipX,flipY:!!e.flipY,modes:l.modes,_ortho:e._ortho||e.projection&&"orthographic"===e.projection.type||!1,tick:function(){var e=r(),n=this.delay,i=e-2*n;l.idle(e-n),l.recalcMatrix(i),l.flush(e-(100+2*n));for(var o=!0,a=l.computedMatrix,s=0;s<16;++s)o=o&&c[s]===a[s],c[s]=a[s];var A=t.clientWidth===g&&t.clientHeight===h;return g=t.clientWidth,h=t.clientHeight,o?!A:(u=Math.exp(l.computedRadius[0]),!0)},lookAt:function(t,e,n){l.lookAt(l.lastT(),t,e,n)},rotate:function(t,e,n){l.rotate(l.lastT(),t,e,n)},pan:function(t,e,n){l.pan(l.lastT(),t,e,n)},translate:function(t,e,n){l.translate(l.lastT(),t,e,n)}};return Object.defineProperties(d,{matrix:{get:function(){return l.computedMatrix},set:function(t){return l.setMatrix(l.lastT(),t),l.computedMatrix},enumerable:!0},mode:{get:function(){return l.getMode()},set:function(t){var e=l.computedUp.slice(),n=l.computedEye.slice(),i=l.computedCenter.slice();if(l.setMode(t),"turntable"===t){var o=r();l._active.lookAt(o,n,i,e),l._active.lookAt(o+500,n,i,[0,0,1]),l._active.flush(o)}return l.getMode()},enumerable:!0},center:{get:function(){return l.computedCenter},set:function(t){return l.lookAt(l.lastT(),null,t),l.computedCenter},enumerable:!0},eye:{get:function(){return l.computedEye},set:function(t){return l.lookAt(l.lastT(),t),l.computedEye},enumerable:!0},up:{get:function(){return l.computedUp},set:function(t){return l.lookAt(l.lastT(),null,null,t),l.computedUp},enumerable:!0},distance:{get:function(){return u},set:function(t){return l.setDistance(l.lastT(),t),t},enumerable:!0},distanceLimits:{get:function(){return l.getDistanceLimits(n)},set:function(t){return l.setDistanceLimits(t),t},enumerable:!0}}),t.addEventListener("contextmenu",function(t){return t.preventDefault(),!1}),d._lastX=-1,d._lastY=-1,d._lastMods={shift:!1,control:!1,alt:!1,meta:!1},d.enableMouseListeners=function(){function e(e,n,i,o){var a=d.keyBindingMode;if(!1!==a){var s="rotate"===a,A="pan"===a,c="zoom"===a,g=!!o.control,h=!!o.alt,f=!!o.shift,p=!!(1&e),I=!!(2&e),C=!!(4&e),m=1/t.clientHeight,y=m*(n-d._lastX),E=m*(i-d._lastY),v=d.flipX?1:-1,B=d.flipY?1:-1,b=Math.PI*d.rotateSpeed,Q=r();if(-1!==d._lastX&&-1!==d._lastY&&((s&&p&&!g&&!h&&!f||p&&!g&&!h&&f)&&l.rotate(Q,v*b*y,-B*b*E,0),(A&&p&&!g&&!h&&!f||I||p&&g&&!h&&!f)&&l.pan(Q,-d.translateSpeed*y*u,d.translateSpeed*E*u,0),c&&p&&!g&&!h&&!f||C||p&&!g&&h&&!f)){var w=-d.zoomSpeed*E/window.innerHeight*(Q-l.lastT())*100;l.pan(Q,0,0,u*(Math.exp(w)-1))}return d._lastX=n,d._lastY=i,d._lastMods=o,!0}}d.mouseListener=o(t,e),t.addEventListener("touchstart",function(n){var r=s(n.changedTouches[0],t);e(0,r[0],r[1],d._lastMods),e(1,r[0],r[1],d._lastMods),n.preventDefault()},!!A&&{passive:!1}),t.addEventListener("touchmove",function(n){var r=s(n.changedTouches[0],t);e(1,r[0],r[1],d._lastMods),n.preventDefault()},!!A&&{passive:!1}),t.addEventListener("touchend",function(t){e(0,d._lastX,d._lastY,d._lastMods),t.preventDefault()},!!A&&{passive:!1}),d.wheelListener=a(t,function(t,e){if(!1!==d.keyBindingMode&&d.enableWheel){var n=d.flipX?1:-1,i=d.flipY?1:-1,o=r();if(Math.abs(t)>Math.abs(e))l.rotate(o,0,0,-t*n*Math.PI*d.rotateSpeed/window.innerWidth);else if(!d._ortho){var a=-d.zoomSpeed*i*e/window.innerHeight*(o-l.lastT())/20;l.pan(o,0,0,u*(Math.exp(a)-1))}}},!0)},d.enableMouseListeners(),d};var r=t("right-now"),i=t("3d-view"),o=t("mouse-change"),a=t("mouse-wheel"),s=t("mouse-event-offset"),A=t("has-passive-events")},{"3d-view":52,"has-passive-events":411,"mouse-change":435,"mouse-event-offset":436,"mouse-wheel":438,"right-now":504}],289:[function(t,e,n){var r=t("glslify"),i=t("gl-shader"),o=r(["precision mediump float;\n#define GLSLIFY 1\nattribute vec2 position;\nvarying vec2 uv;\nvoid main() {\n  uv = position;\n  gl_Position = vec4(position, 0, 1);\n}"]),a=r(["precision mediump float;\n#define GLSLIFY 1\n\nuniform sampler2D accumBuffer;\nvarying vec2 uv;\n\nvoid main() {\n  vec4 accum = texture2D(accumBuffer, 0.5 * (uv + 1.0));\n  gl_FragColor = min(vec4(1,1,1,1), accum);\n}"]);e.exports=function(t){return i(t,o,a,null,[{name:"position",type:"vec2"}])}},{"gl-shader":301,glslify:409}],290:[function(t,e,n){"use strict";var r=t("./camera.js"),i=t("gl-axes3d"),o=t("gl-axes3d/properties"),a=t("gl-spikes3d"),s=t("gl-select-static"),A=t("gl-fbo"),l=t("a-big-triangle"),c=t("mouse-change"),u=t("mouse-wheel"),g=t("gl-mat4/perspective"),h=t("gl-mat4/ortho"),d=t("./lib/shader"),f=t("is-mobile")({tablet:!0});function p(){this.mouse=[-1,-1],this.screen=null,this.distance=1/0,this.index=null,this.dataCoordinate=null,this.dataPosition=null,this.object=null,this.data=null}function I(t){var e=Math.round(Math.log(Math.abs(t))/Math.log(10));if(e<0){var n=Math.round(Math.pow(10,-e));return Math.ceil(t*n)/n}return e>0?(n=Math.round(Math.pow(10,e)),Math.ceil(t/n)*n):Math.ceil(t)}function C(t){return"boolean"!=typeof t||t}e.exports={createScene:function(t){(t=t||{}).camera=t.camera||{};var e=t.canvas;if(!e)if(e=document.createElement("canvas"),t.container){var n=t.container;n.appendChild(e)}else document.body.appendChild(e);var m=t.gl;if(m||(m=function(t,e){var n=null;try{(n=t.getContext("webgl",e))||(n=t.getContext("experimental-webgl",e))}catch(t){return null}return n}(e,t.glOptions||{premultipliedAlpha:!0,antialias:!0,preserveDrawingBuffer:f})),!m)throw new Error("webgl not supported");var y=t.bounds||[[-10,-10,-10],[10,10,10]],E=new p,v=A(m,[m.drawingBufferWidth,m.drawingBufferHeight],{preferFloat:!f}),B=d(m),b=t.cameraObject&&!0===t.cameraObject._ortho||t.camera.projection&&"orthographic"===t.camera.projection.type||!1,Q={eye:t.camera.eye||[2,0,0],center:t.camera.center||[0,0,0],up:t.camera.up||[0,1,0],zoomMin:t.camera.zoomMax||.1,zoomMax:t.camera.zoomMin||100,mode:t.camera.mode||"turntable",_ortho:b},w=t.axes||{},x=i(m,w);x.enable=!w.disable;var _=t.spikes||{},D=a(m,_),M=[],k=[],S=[],L=[],R=!0,N=!0,F=new Array(16),T=new Array(16),U={view:null,projection:F,model:T,_ortho:!1},N=!0,O=[m.drawingBufferWidth,m.drawingBufferHeight],G=t.cameraObject||r(e,Q),H={gl:m,contextLost:!1,pixelRatio:t.pixelRatio||1,canvas:e,selection:E,camera:G,axes:x,axesPixels:null,spikes:D,bounds:y,objects:M,shape:O,aspect:t.aspectRatio||[1,1,1],pickRadius:t.pickRadius||10,zNear:t.zNear||.01,zFar:t.zFar||1e3,fovy:t.fovy||Math.PI/4,clearColor:t.clearColor||[0,0,0,0],autoResize:C(t.autoResize),autoBounds:C(t.autoBounds),autoScale:!!t.autoScale,autoCenter:C(t.autoCenter),clipToBounds:C(t.clipToBounds),snapToData:!!t.snapToData,onselect:t.onselect||null,onrender:t.onrender||null,onclick:t.onclick||null,cameraParams:U,oncontextloss:null,mouseListener:null,_stopped:!1},j=[m.drawingBufferWidth/H.pixelRatio|0,m.drawingBufferHeight/H.pixelRatio|0];function Y(){if(!H._stopped&&H.autoResize){var t=e.parentNode,n=1,r=1;t&&t!==document.body?(n=t.clientWidth,r=t.clientHeight):(n=window.innerWidth,r=window.innerHeight);var i=0|Math.ceil(n*H.pixelRatio),o=0|Math.ceil(r*H.pixelRatio);if(i!==e.width||o!==e.height){e.width=i,e.height=o;var a=e.style;a.position=a.position||"absolute",a.left="0px",a.top="0px",a.width=n+"px",a.height=r+"px",R=!0}}}function P(){for(var t=M.length,e=L.length,n=0;n<e;++n)S[n]=0;t:for(var n=0;n<t;++n){var r=M[n],i=r.pickSlots;if(i){for(var o=0;o<e;++o)if(S[o]+i<255){k[n]=o,r.setPickBase(S[o]+1),S[o]+=i;continue t}var a=s(m,O);k[n]=e,L.push(a),S.push(i),r.setPickBase(1),e+=1}else k[n]=-1}for(;e>0&&0===S[e-1];)S.pop(),L.pop().dispose()}function K(){if(H.contextLost)return!0;m.isContextLost()&&(H.contextLost=!0,H.mouseListener.enabled=!1,H.selection.object=null,H.oncontextloss&&H.oncontextloss())}H.autoResize&&Y(),window.addEventListener("resize",Y),H.update=function(t){H._stopped||(t=t||{},R=!0,N=!0)},H.add=function(t){H._stopped||(t.axes=x,M.push(t),k.push(-1),R=!0,N=!0,P())},H.remove=function(t){if(!H._stopped){var e=M.indexOf(t);e<0||(M.splice(e,1),k.pop(),R=!0,N=!0,P())}},H.dispose=function(){if(!H._stopped&&(H._stopped=!0,window.removeEventListener("resize",Y),e.removeEventListener("webglcontextlost",K),H.mouseListener.enabled=!1,!H.contextLost)){x.dispose(),D.dispose();for(var t=0;t<M.length;++t)M[t].dispose();v.dispose();for(var t=0;t<L.length;++t)L[t].dispose();B.dispose(),m=null,x=null,D=null,M=[]}},H.wheelListener=u(e,function(t,e){if(!1!==G.keyBindingMode&&G.enableWheel&&G._ortho){var n=t>e?1.1:1/1.1;H.aspect[0]*=n,H.aspect[1]*=n,H.aspect[2]*=n,H.redraw()}},!0),H._mouseRotating=!1,H._prevButtons=0,H.enableMouseListeners=function(){H.mouseListener=c(e,function(t,e,n){if(!H._stopped){var r=L.length,i=M.length,o=E.object;E.distance=1/0,E.mouse[0]=e,E.mouse[1]=n,E.object=null,E.screen=null,E.dataCoordinate=E.dataPosition=null;var a=!1;if(t&&H._prevButtons)H._mouseRotating=!0;else{H._mouseRotating&&(N=!0),H._mouseRotating=!1;for(var s=0;s<r;++s){var A=L[s].query(e,j[1]-n-1,H.pickRadius);if(A){if(A.distance>E.distance)continue;for(var l=0;l<i;++l){var c=M[l];if(k[l]===s){var u=c.pick(A);u&&(E.buttons=t,E.screen=A.coord,E.distance=A.distance,E.object=c,E.index=u.distance,E.dataPosition=u.position,E.dataCoordinate=u.dataCoordinate,E.data=u,a=!0)}}}}}o&&o!==E.object&&(o.highlight&&o.highlight(null),R=!0),E.object&&(E.object.highlight&&E.object.highlight(E.data),R=!0),(a=a||E.object!==o)&&H.onselect&&H.onselect(E),1&t&&!(1&H._prevButtons)&&H.onclick&&H.onclick(E),H._prevButtons=t}})},e.addEventListener("webglcontextlost",K);var z=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],J=[z[0].slice(),z[1].slice()];function q(){if(!K()){Y();var t=H.camera.tick();U.view=H.camera.matrix,R=R||t,N=N||t,x.pixelRatio=H.pixelRatio,D.pixelRatio=H.pixelRatio;var e=M.length,n=z[0],r=z[1];n[0]=n[1]=n[2]=1/0,r[0]=r[1]=r[2]=-1/0;for(var i=0;i<e;++i){var a=M[i];a.pixelRatio=H.pixelRatio,a.axes=H.axes,R=R||!!a.dirty,N=N||!!a.dirty;var s=a.bounds;if(s)for(var A=s[0],c=s[1],u=0;u<3;++u)n[u]=Math.min(n[u],A[u]),r[u]=Math.max(r[u],c[u])}var d=H.bounds;if(H.autoBounds)for(var u=0;u<3;++u){if(r[u]<n[u])n[u]=-1,r[u]=1;else{n[u]===r[u]&&(n[u]-=1,r[u]+=1);var f=.05*(r[u]-n[u]);n[u]=n[u]-f,r[u]=r[u]+f}d[0][u]=n[u],d[1][u]=r[u]}for(var p=!1,u=0;u<3;++u)p=p||J[0][u]!==d[0][u]||J[1][u]!==d[1][u],J[0][u]=d[0][u],J[1][u]=d[1][u];if(N=N||p,R=R||p){if(p){for(var C=[0,0,0],i=0;i<3;++i)C[i]=I((d[1][i]-d[0][i])/10);x.autoTicks?x.update({bounds:d,tickSpacing:C}):x.update({bounds:d})}var y=m.drawingBufferWidth,Q=m.drawingBufferHeight;O[0]=y,O[1]=Q,j[0]=0|Math.max(y/H.pixelRatio,1),j[1]=0|Math.max(Q/H.pixelRatio,1),b?(h(F,-y/Q,y/Q,-1,1,H.zNear,H.zFar),U._ortho=!0):(g(F,H.fovy,y/Q,H.zNear,H.zFar),U._ortho=!1);for(var i=0;i<16;++i)T[i]=0;T[15]=1;for(var w=0,i=0;i<3;++i)w=Math.max(w,d[1][i]-d[0][i]);for(var i=0;i<3;++i)H.autoScale?T[5*i]=H.aspect[i]/(d[1][i]-d[0][i]):T[5*i]=1/w,H.autoCenter&&(T[12+i]=.5*-T[5*i]*(d[0][i]+d[1][i]));for(var i=0;i<e;++i){var a=M[i];a.axesBounds=d,H.clipToBounds&&(a.clipBounds=d)}E.object&&(H.snapToData?D.position=E.dataCoordinate:D.position=E.dataPosition,D.bounds=d),N&&(N=!1,function(){if(!K()){m.colorMask(!0,!0,!0,!0),m.depthMask(!0),m.disable(m.BLEND),m.enable(m.DEPTH_TEST);for(var t=M.length,e=L.length,n=0;n<e;++n){var r=L[n];r.shape=j,r.begin();for(var i=0;i<t;++i)if(k[i]===n){var o=M[i];o.drawPick&&(o.pixelRatio=1,o.drawPick(U))}r.end()}}}()),H.axesPixels=o(H.axes,U,y,Q),H.onrender&&H.onrender(),m.bindFramebuffer(m.FRAMEBUFFER,null),m.viewport(0,0,y,Q);var _=H.clearColor;m.clearColor(_[0],_[1],_[2],_[3]),m.clear(m.COLOR_BUFFER_BIT|m.DEPTH_BUFFER_BIT),m.depthMask(!0),m.colorMask(!0,!0,!0,!0),m.enable(m.DEPTH_TEST),m.depthFunc(m.LEQUAL),m.disable(m.BLEND),m.disable(m.CULL_FACE);var S=!1;x.enable&&(S=S||x.isTransparent(),x.draw(U)),D.axes=x,E.object&&D.draw(U),m.disable(m.CULL_FACE);for(var i=0;i<e;++i){var a=M[i];a.axes=x,a.pixelRatio=H.pixelRatio,a.isOpaque&&a.isOpaque()&&a.draw(U),a.isTransparent&&a.isTransparent()&&(S=!0)}if(S){v.shape=O,v.bind(),m.clear(m.DEPTH_BUFFER_BIT),m.colorMask(!1,!1,!1,!1),m.depthMask(!0),m.depthFunc(m.LESS),x.enable&&x.isTransparent()&&x.drawTransparent(U);for(var i=0;i<e;++i){var a=M[i];a.isOpaque&&a.isOpaque()&&a.draw(U)}m.enable(m.BLEND),m.blendEquation(m.FUNC_ADD),m.blendFunc(m.ONE,m.ONE_MINUS_SRC_ALPHA),m.colorMask(!0,!0,!0,!0),m.depthMask(!1),m.clearColor(0,0,0,0),m.clear(m.COLOR_BUFFER_BIT),x.isTransparent()&&x.drawTransparent(U);for(var i=0;i<e;++i){var a=M[i];a.isTransparent&&a.isTransparent()&&a.drawTransparent(U)}m.bindFramebuffer(m.FRAMEBUFFER,null),m.blendFunc(m.ONE,m.ONE_MINUS_SRC_ALPHA),m.disable(m.DEPTH_TEST),B.bind(),v.color[0].bind(0),B.uniforms.accumBuffer=0,l(m),m.disable(m.BLEND)}R=!1;for(var i=0;i<e;++i)M[i].dirty=!1}}}return H.enableMouseListeners(),function t(){H._stopped||H.contextLost||(q(),requestAnimationFrame(t))}(),H.redraw=function(){H._stopped||(R=!0,q())},H},createCamera:r}},{"./camera.js":288,"./lib/shader":289,"a-big-triangle":59,"gl-axes3d":233,"gl-axes3d/properties":240,"gl-fbo":249,"gl-mat4/ortho":268,"gl-mat4/perspective":269,"gl-select-static":300,"gl-spikes3d":310,"is-mobile":420,"mouse-change":435,"mouse-wheel":438}],291:[function(t,e,n){var r=t("glslify");n.pointVertex=r(["precision mediump float;\n#define GLSLIFY 1\n\nattribute vec2 position;\n\nuniform mat3 matrix;\nuniform float pointSize;\nuniform float pointCloud;\n\nhighp float rand(vec2 co) {\n  highp float a = 12.9898;\n  highp float b = 78.233;\n  highp float c = 43758.5453;\n  highp float d = dot(co.xy, vec2(a, b));\n  highp float e = mod(d, 3.14);\n  return fract(sin(e) * c);\n}\n\nvoid main() {\n  vec3 hgPosition = matrix * vec3(position, 1);\n  gl_Position  = vec4(hgPosition.xy, 0, hgPosition.z);\n    // if we don't jitter the point size a bit, overall point cloud\n    // saturation 'jumps' on zooming, which is disturbing and confusing\n  gl_PointSize = pointSize * ((19.5 + rand(position)) / 20.0);\n  if(pointCloud != 0.0) { // pointCloud is truthy\n    // get the same square surface as circle would be\n    gl_PointSize *= 0.886;\n  }\n}"]),n.pointFragment=r(["precision mediump float;\n#define GLSLIFY 1\n\nuniform vec4 color, borderColor;\nuniform float centerFraction;\nuniform float pointCloud;\n\nvoid main() {\n  float radius;\n  vec4 baseColor;\n  if(pointCloud != 0.0) { // pointCloud is truthy\n    if(centerFraction == 1.0) {\n      gl_FragColor = color;\n    } else {\n      gl_FragColor = mix(borderColor, color, centerFraction);\n    }\n  } else {\n    radius = length(2.0 * gl_PointCoord.xy - 1.0);\n    if(radius > 1.0) {\n      discard;\n    }\n    baseColor = mix(borderColor, color, step(radius, centerFraction));\n    gl_FragColor = vec4(baseColor.rgb * baseColor.a, baseColor.a);\n  }\n}\n"]),n.pickVertex=r(["precision mediump float;\n#define GLSLIFY 1\n\nattribute vec2 position;\nattribute vec4 pickId;\n\nuniform mat3 matrix;\nuniform float pointSize;\nuniform vec4 pickOffset;\n\nvarying vec4 fragId;\n\nvoid main() {\n  vec3 hgPosition = matrix * vec3(position, 1);\n  gl_Position  = vec4(hgPosition.xy, 0, hgPosition.z);\n  gl_PointSize = pointSize;\n\n  vec4 id = pickId + pickOffset;\n  id.y += floor(id.x / 256.0);\n  id.x -= floor(id.x / 256.0) * 256.0;\n\n  id.z += floor(id.y / 256.0);\n  id.y -= floor(id.y / 256.0) * 256.0;\n\n  id.w += floor(id.z / 256.0);\n  id.z -= floor(id.z / 256.0) * 256.0;\n\n  fragId = id;\n}\n"]),n.pickFragment=r(["precision mediump float;\n#define GLSLIFY 1\n\nvarying vec4 fragId;\n\nvoid main() {\n  float radius = length(2.0 * gl_PointCoord.xy - 1.0);\n  if(radius > 1.0) {\n    discard;\n  }\n  gl_FragColor = fragId / 255.0;\n}\n"])},{glslify:409}],292:[function(t,e,n){"use strict";var r=t("gl-shader"),i=t("gl-buffer"),o=t("typedarray-pool"),a=t("./lib/shader");function s(t,e,n,r,i){this.plot=t,this.offsetBuffer=e,this.pickBuffer=n,this.shader=r,this.pickShader=i,this.sizeMin=.5,this.sizeMinCap=2,this.sizeMax=20,this.areaRatio=1,this.pointCount=0,this.color=[1,0,0,1],this.borderColor=[0,0,0,1],this.blend=!1,this.pickOffset=0,this.points=null}e.exports=function(t,e){var n=t.gl,o=i(n),A=i(n),l=r(n,a.pointVertex,a.pointFragment),c=r(n,a.pickVertex,a.pickFragment),u=new s(t,o,A,l,c);return u.update(e),t.addObject(u),u};var A,l,c=s.prototype;c.dispose=function(){this.shader.dispose(),this.pickShader.dispose(),this.offsetBuffer.dispose(),this.pickBuffer.dispose(),this.plot.removeObject(this)},c.update=function(t){var e;function n(e,n){return e in t?t[e]:n}t=t||{},this.sizeMin=n("sizeMin",.5),this.sizeMax=n("sizeMax",20),this.color=n("color",[1,0,0,1]).slice(),this.areaRatio=n("areaRatio",1),this.borderColor=n("borderColor",[0,0,0,1]).slice(),this.blend=n("blend",!1);var r=t.positions.length>>>1,i=t.positions instanceof Float32Array,a=t.idToIndex instanceof Int32Array&&t.idToIndex.length>=r,s=t.positions,A=i?s:o.mallocFloat32(s.length),l=a?t.idToIndex:o.mallocInt32(r);if(i||A.set(s),!a)for(A.set(s),e=0;e<r;e++)l[e]=e;this.points=s,this.offsetBuffer.update(A),this.pickBuffer.update(l),i||o.free(A),a||o.free(l),this.pointCount=r,this.pickOffset=0},c.unifiedDraw=(A=[1,0,0,0,1,0,0,0,1],l=[0,0,0,0],function(t){var e=void 0!==t,n=e?this.pickShader:this.shader,r=this.plot.gl,i=this.plot.dataBox;if(0===this.pointCount)return t;var o=i[2]-i[0],a=i[3]-i[1],s=function(t,e){var n,r=0,i=t.length>>>1;for(n=0;n<i;n++){var o=t[2*n],a=t[2*n+1];o>=e[0]&&o<=e[2]&&a>=e[1]&&a<=e[3]&&r++}return r}(this.points,i),c=this.plot.pickPixelRatio*Math.max(Math.min(this.sizeMinCap,this.sizeMin),Math.min(this.sizeMax,this.sizeMax/Math.pow(s,.33333)));A[0]=2/o,A[4]=2/a,A[6]=-2*i[0]/o-1,A[7]=-2*i[1]/a-1,this.offsetBuffer.bind(),n.bind(),n.attributes.position.pointer(),n.uniforms.matrix=A,n.uniforms.color=this.color,n.uniforms.borderColor=this.borderColor,n.uniforms.pointCloud=c<5,n.uniforms.pointSize=c,n.uniforms.centerFraction=Math.min(1,Math.max(0,Math.sqrt(1-this.areaRatio))),e&&(l[0]=255&t,l[1]=t>>8&255,l[2]=t>>16&255,l[3]=t>>24&255,this.pickBuffer.bind(),n.attributes.pickId.pointer(r.UNSIGNED_BYTE),n.uniforms.pickOffset=l,this.pickOffset=t);var u=r.getParameter(r.BLEND),g=r.getParameter(r.DITHER);return u&&!this.blend&&r.disable(r.BLEND),g&&r.disable(r.DITHER),r.drawArrays(r.POINTS,0,this.pointCount),u&&!this.blend&&r.enable(r.BLEND),g&&r.enable(r.DITHER),t+this.pointCount}),c.draw=c.unifiedDraw,c.drawPick=c.unifiedDraw,c.pick=function(t,e,n){var r=this.pickOffset,i=this.pointCount;if(n<r||n>=r+i)return null;var o=n-r,a=this.points;return{object:this,pointId:o,dataCoord:[a[2*o],a[2*o+1]]}}},{"./lib/shader":291,"gl-buffer":241,"gl-shader":301,"typedarray-pool":545}],293:[function(t,e,n){e.exports=function(t,e,n,r){var i,o,a,s,A,l=e[0],c=e[1],u=e[2],g=e[3],h=n[0],d=n[1],f=n[2],p=n[3];return(o=l*h+c*d+u*f+g*p)<0&&(o=-o,h=-h,d=-d,f=-f,p=-p),1-o>1e-6?(i=Math.acos(o),a=Math.sin(i),s=Math.sin((1-r)*i)/a,A=Math.sin(r*i)/a):(s=1-r,A=r),t[0]=s*l+A*h,t[1]=s*c+A*d,t[2]=s*u+A*f,t[3]=s*g+A*p,t}},{}],294:[function(t,e,n){"use strict";e.exports=function(t){return t||0===t?t.toString():""}},{}],295:[function(t,e,n){"use strict";var r=t("vectorize-text");e.exports=function(t,e,n){var o=i[e];if(o||(o=i[e]={}),t in o)return o[t];var a={textAlign:"center",textBaseline:"middle",lineHeight:1,font:e,lineSpacing:1.25,styletags:{breaklines:!0,bolds:!0,italics:!0,subscripts:!0,superscripts:!0},triangles:!0},s=r(t,a);a.triangles=!1;var A,l,c=r(t,a);if(n&&1!==n){for(A=0;A<s.positions.length;++A)for(l=0;l<s.positions[A].length;++l)s.positions[A][l]/=n;for(A=0;A<c.positions.length;++A)for(l=0;l<c.positions[A].length;++l)c.positions[A][l]/=n}var u=[[1/0,1/0],[-1/0,-1/0]],g=c.positions.length;for(A=0;A<g;++A){var h=c.positions[A];for(l=0;l<2;++l)u[0][l]=Math.min(u[0][l],h[l]),u[1][l]=Math.max(u[1][l],h[l])}return o[t]=[s,c,u]};var i={}},{"vectorize-text":550}],296:[function(t,e,n){var r=t("gl-shader"),i=t("glslify"),o=i(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nattribute vec3 position;\nattribute vec4 color;\nattribute vec2 glyph;\nattribute vec4 id;\n\nuniform vec4 highlightId;\nuniform float highlightScale;\nuniform mat4 model, view, projection;\nuniform vec3 clipBounds[2];\n\nvarying vec4 interpColor;\nvarying vec4 pickId;\nvarying vec3 dataCoordinate;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], position)) {\n\n    gl_Position = vec4(0,0,0,0);\n  } else {\n    float scale = 1.0;\n    if(distance(highlightId, id) < 0.0001) {\n      scale = highlightScale;\n    }\n\n    vec4 worldPosition = model * vec4(position, 1);\n    vec4 viewPosition = view * worldPosition;\n    viewPosition = viewPosition / viewPosition.w;\n    vec4 clipPosition = projection * (viewPosition + scale * vec4(glyph.x, -glyph.y, 0, 0));\n\n    gl_Position = clipPosition;\n    interpColor = color;\n    pickId = id;\n    dataCoordinate = position;\n  }\n}"]),a=i(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nattribute vec3 position;\nattribute vec4 color;\nattribute vec2 glyph;\nattribute vec4 id;\n\nuniform mat4 model, view, projection;\nuniform vec2 screenSize;\nuniform vec3 clipBounds[2];\nuniform float highlightScale, pixelRatio;\nuniform vec4 highlightId;\n\nvarying vec4 interpColor;\nvarying vec4 pickId;\nvarying vec3 dataCoordinate;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], position)) {\n\n    gl_Position = vec4(0,0,0,0);\n  } else {\n    float scale = pixelRatio;\n    if(distance(highlightId.bgr, id.bgr) < 0.001) {\n      scale *= highlightScale;\n    }\n\n    vec4 worldPosition = model * vec4(position, 1.0);\n    vec4 viewPosition = view * worldPosition;\n    vec4 clipPosition = projection * viewPosition;\n    clipPosition /= clipPosition.w;\n\n    gl_Position = clipPosition + vec4(screenSize * scale * vec2(glyph.x, -glyph.y), 0.0, 0.0);\n    interpColor = color;\n    pickId = id;\n    dataCoordinate = position;\n  }\n}"]),s=i(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nattribute vec3 position;\nattribute vec4 color;\nattribute vec2 glyph;\nattribute vec4 id;\n\nuniform float highlightScale;\nuniform vec4 highlightId;\nuniform vec3 axes[2];\nuniform mat4 model, view, projection;\nuniform vec2 screenSize;\nuniform vec3 clipBounds[2];\nuniform float scale, pixelRatio;\n\nvarying vec4 interpColor;\nvarying vec4 pickId;\nvarying vec3 dataCoordinate;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], position)) {\n\n    gl_Position = vec4(0,0,0,0);\n  } else {\n    float lscale = pixelRatio * scale;\n    if(distance(highlightId, id) < 0.0001) {\n      lscale *= highlightScale;\n    }\n\n    vec4 clipCenter   = projection * view * model * vec4(position, 1);\n    vec3 dataPosition = position + 0.5*lscale*(axes[0] * glyph.x + axes[1] * glyph.y) * clipCenter.w * screenSize.y;\n    vec4 clipPosition = projection * view * model * vec4(dataPosition, 1);\n\n    gl_Position = clipPosition;\n    interpColor = color;\n    pickId = id;\n    dataCoordinate = dataPosition;\n  }\n}\n"]),A=i(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 fragClipBounds[2];\nuniform float opacity;\n\nvarying vec4 interpColor;\nvarying vec3 dataCoordinate;\n\nvoid main() {\n  if (\n    outOfRange(fragClipBounds[0], fragClipBounds[1], dataCoordinate) ||\n    interpColor.a * opacity == 0.\n  ) discard;\n  gl_FragColor = interpColor * opacity;\n}\n"]),l=i(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 fragClipBounds[2];\nuniform float pickGroup;\n\nvarying vec4 pickId;\nvarying vec3 dataCoordinate;\n\nvoid main() {\n  if (outOfRange(fragClipBounds[0], fragClipBounds[1], dataCoordinate)) discard;\n\n  gl_FragColor = vec4(pickGroup, pickId.bgr);\n}"]),c=[{name:"position",type:"vec3"},{name:"color",type:"vec4"},{name:"glyph",type:"vec2"},{name:"id",type:"vec4"}],u={vertex:o,fragment:A,attributes:c},g={vertex:a,fragment:A,attributes:c},h={vertex:s,fragment:A,attributes:c},d={vertex:o,fragment:l,attributes:c},f={vertex:a,fragment:l,attributes:c},p={vertex:s,fragment:l,attributes:c};function I(t,e){var n=r(t,e),i=n.attributes;return i.position.location=0,i.color.location=1,i.glyph.location=2,i.id.location=3,n}n.createPerspective=function(t){return I(t,u)},n.createOrtho=function(t){return I(t,g)},n.createProject=function(t){return I(t,h)},n.createPickPerspective=function(t){return I(t,d)},n.createPickOrtho=function(t){return I(t,f)},n.createPickProject=function(t){return I(t,p)}},{"gl-shader":301,glslify:409}],297:[function(t,e,n){"use strict";var r=t("is-string-blank"),i=t("gl-buffer"),o=t("gl-vao"),a=t("typedarray-pool"),s=t("gl-mat4/multiply"),A=t("./lib/shaders"),l=t("./lib/glyphs"),c=t("./lib/get-simple-string"),u=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];function g(t,e){var n=t[0],r=t[1],i=t[2],o=t[3];return t[0]=e[0]*n+e[4]*r+e[8]*i+e[12]*o,t[1]=e[1]*n+e[5]*r+e[9]*i+e[13]*o,t[2]=e[2]*n+e[6]*r+e[10]*i+e[14]*o,t[3]=e[3]*n+e[7]*r+e[11]*i+e[15]*o,t}function h(t,e,n,r){return g(r,r),g(r,r),g(r,r)}function d(t,e){this.index=t,this.dataCoordinate=this.position=e}function f(t){return!0===t?1:t>1?1:t}function p(t,e,n,r,i,o,a,s,A,l,c,u){this.gl=t,this.pixelRatio=1,this.shader=e,this.orthoShader=n,this.projectShader=r,this.pointBuffer=i,this.colorBuffer=o,this.glyphBuffer=a,this.idBuffer=s,this.vao=A,this.vertexCount=0,this.lineVertexCount=0,this.opacity=1,this.hasAlpha=!1,this.lineWidth=0,this.projectScale=[2/3,2/3,2/3],this.projectOpacity=[1,1,1],this.projectHasAlpha=!1,this.pickId=0,this.pickPerspectiveShader=l,this.pickOrthoShader=c,this.pickProjectShader=u,this.points=[],this._selectResult=new d(0,[0,0,0]),this.useOrtho=!0,this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],this.axesProject=[!0,!0,!0],this.axesBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.highlightId=[1,1,1,1],this.highlightScale=2,this.clipBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.dirty=!0}e.exports=function(t){var e=t.gl,n=A.createPerspective(e),r=A.createOrtho(e),a=A.createProject(e),s=A.createPickPerspective(e),l=A.createPickOrtho(e),c=A.createPickProject(e),u=i(e),g=i(e),h=i(e),d=i(e),f=o(e,[{buffer:u,size:3,type:e.FLOAT},{buffer:g,size:4,type:e.FLOAT},{buffer:h,size:2,type:e.FLOAT},{buffer:d,size:4,type:e.UNSIGNED_BYTE,normalized:!0}]),I=new p(e,n,r,a,u,g,h,d,f,s,l,c);return I.update(t),I};var I=p.prototype;I.pickSlots=1,I.setPickBase=function(t){this.pickId=t},I.isTransparent=function(){if(this.hasAlpha)return!0;for(var t=0;t<3;++t)if(this.axesProject[t]&&this.projectHasAlpha)return!0;return!1},I.isOpaque=function(){if(!this.hasAlpha)return!0;for(var t=0;t<3;++t)if(this.axesProject[t]&&!this.projectHasAlpha)return!0;return!1};var C=[0,0],m=[0,0,0],y=[0,0,0],E=[0,0,0,1],v=[0,0,0,1],B=u.slice(),b=[0,0,0],Q=[[0,0,0],[0,0,0]];function w(t){return t[0]=t[1]=t[2]=0,t}function x(t,e){return t[0]=e[0],t[1]=e[1],t[2]=e[2],t[3]=1,t}function _(t,e,n,r){return t[0]=e[0],t[1]=e[1],t[2]=e[2],t[n]=r,t}function D(t,e,n,r){var i,o=e.axesProject,a=e.gl,A=t.uniforms,l=n.model||u,c=n.view||u,g=n.projection||u,d=e.axesBounds,f=function(t){for(var e=Q,n=0;n<2;++n)for(var r=0;r<3;++r)e[n][r]=Math.max(Math.min(t[n][r],1e8),-1e8);return e}(e.clipBounds);i=e.axes&&e.axes.lastCubeProps?e.axes.lastCubeProps.axis:[1,1,1],C[0]=2/a.drawingBufferWidth,C[1]=2/a.drawingBufferHeight,t.bind(),A.view=c,A.projection=g,A.screenSize=C,A.highlightId=e.highlightId,A.highlightScale=e.highlightScale,A.clipBounds=f,A.pickGroup=e.pickId/255,A.pixelRatio=r;for(var p=0;p<3;++p)if(o[p]){A.scale=e.projectScale[p],A.opacity=e.projectOpacity[p];for(var I=B,D=0;D<16;++D)I[D]=0;for(D=0;D<4;++D)I[5*D]=1;I[5*p]=0,i[p]<0?I[12+p]=d[0][p]:I[12+p]=d[1][p],s(I,l,I),A.model=I;var M=(p+1)%3,k=(p+2)%3,S=w(m),L=w(y);S[M]=1,L[k]=1;var R=h(0,0,0,x(E,S)),N=h(0,0,0,x(v,L));if(Math.abs(R[1])>Math.abs(N[1])){var F=R;R=N,N=F,F=S,S=L,L=F;var T=M;M=k,k=T}R[0]<0&&(S[M]=-1),N[1]>0&&(L[k]=-1);var U=0,O=0;for(D=0;D<4;++D)U+=Math.pow(l[4*M+D],2),O+=Math.pow(l[4*k+D],2);S[M]/=Math.sqrt(U),L[k]/=Math.sqrt(O),A.axes[0]=S,A.axes[1]=L,A.fragClipBounds[0]=_(b,f[0],p,-1e8),A.fragClipBounds[1]=_(b,f[1],p,1e8),e.vao.bind(),e.vao.draw(a.TRIANGLES,e.vertexCount),e.lineWidth>0&&(a.lineWidth(e.lineWidth*r),e.vao.draw(a.LINES,e.lineVertexCount,e.vertexCount)),e.vao.unbind()}}var M=[[-1e8,-1e8,-1e8],[1e8,1e8,1e8]];function k(t,e,n,r,i,o,a){var s=n.gl;if((o===n.projectHasAlpha||a)&&D(e,n,r,i),o===n.hasAlpha||a){t.bind();var A=t.uniforms;A.model=r.model||u,A.view=r.view||u,A.projection=r.projection||u,C[0]=2/s.drawingBufferWidth,C[1]=2/s.drawingBufferHeight,A.screenSize=C,A.highlightId=n.highlightId,A.highlightScale=n.highlightScale,A.fragClipBounds=M,A.clipBounds=n.axes.bounds,A.opacity=n.opacity,A.pickGroup=n.pickId/255,A.pixelRatio=i,n.vao.bind(),n.vao.draw(s.TRIANGLES,n.vertexCount),n.lineWidth>0&&(s.lineWidth(n.lineWidth*i),n.vao.draw(s.LINES,n.lineVertexCount,n.vertexCount)),n.vao.unbind()}}function S(t,e,n,i){var o;o=Array.isArray(t)?e<t.length?t[e]:void 0:t,o=c(o);var a=!0;r(o)&&(o="▼",a=!1);var s=l(o,n,i);return{mesh:s[0],lines:s[1],bounds:s[2],visible:a}}I.draw=function(t){k(this.useOrtho?this.orthoShader:this.shader,this.projectShader,this,t,this.pixelRatio,!1,!1)},I.drawTransparent=function(t){k(this.useOrtho?this.orthoShader:this.shader,this.projectShader,this,t,this.pixelRatio,!0,!1)},I.drawPick=function(t){k(this.useOrtho?this.pickOrthoShader:this.pickPerspectiveShader,this.pickProjectShader,this,t,1,!0,!0)},I.pick=function(t){if(!t)return null;if(t.id!==this.pickId)return null;var e=t.value[2]+(t.value[1]<<8)+(t.value[0]<<16);if(e>=this.pointCount||e<0)return null;var n=this.points[e],r=this._selectResult;r.index=e;for(var i=0;i<3;++i)r.position[i]=r.dataCoordinate[i]=n[i];return r},I.highlight=function(t){if(t){var e=t.index,n=255&e,r=e>>8&255,i=e>>16&255;this.highlightId=[n/255,r/255,i/255,0]}else this.highlightId=[1,1,1,1]},I.update=function(t){if("perspective"in(t=t||{})&&(this.useOrtho=!t.perspective),"orthographic"in t&&(this.useOrtho=!!t.orthographic),"lineWidth"in t&&(this.lineWidth=t.lineWidth),"project"in t)if(Array.isArray(t.project))this.axesProject=t.project;else{var e=!!t.project;this.axesProject=[e,e,e]}if("projectScale"in t)if(Array.isArray(t.projectScale))this.projectScale=t.projectScale.slice();else{var n=+t.projectScale;this.projectScale=[n,n,n]}if(this.projectHasAlpha=!1,"projectOpacity"in t){Array.isArray(t.projectOpacity)?this.projectOpacity=t.projectOpacity.slice():(n=+t.projectOpacity,this.projectOpacity=[n,n,n]);for(var r=0;r<3;++r)this.projectOpacity[r]=f(this.projectOpacity[r]),this.projectOpacity[r]<1&&(this.projectHasAlpha=!0)}this.hasAlpha=!1,"opacity"in t&&(this.opacity=f(t.opacity),this.opacity<1&&(this.hasAlpha=!0)),this.dirty=!0;var i,o,s=t.position,A=t.font||"normal",l=t.alignment||[0,0];if(2===l.length)i=l[0],o=l[1];else for(i=[],o=[],r=0;r<l.length;++r)i[r]=l[r][0],o[r]=l[r][1];var c=[1/0,1/0,1/0],u=[-1/0,-1/0,-1/0],g=t.glyph,h=t.color,d=t.size,p=t.angle,I=t.lineColor,C=-1,m=0,y=0,E=0;if(s.length){E=s.length;t:for(r=0;r<E;++r){for(var v=s[r],B=0;B<3;++B)if(isNaN(v[B])||!isFinite(v[B]))continue t;var b=(O=S(g,r,A,this.pixelRatio)).mesh,Q=O.lines,w=O.bounds;m+=3*b.cells.length,y+=2*Q.edges.length}}var x=m+y,_=a.mallocFloat(3*x),D=a.mallocFloat(4*x),M=a.mallocFloat(2*x),k=a.mallocUint32(x);if(x>0){var L=0,R=m,N=[0,0,0,1],F=[0,0,0,1],T=Array.isArray(h)&&Array.isArray(h[0]),U=Array.isArray(I)&&Array.isArray(I[0]);t:for(r=0;r<E;++r){for(C+=1,v=s[r],B=0;B<3;++B){if(isNaN(v[B])||!isFinite(v[B]))continue t;u[B]=Math.max(u[B],v[B]),c[B]=Math.min(c[B],v[B])}b=(O=S(g,r,A,this.pixelRatio)).mesh,Q=O.lines,w=O.bounds;var O,G=O.visible;if(G)if(Array.isArray(h)){if(3===(H=T?r<h.length?h[r]:[0,0,0,0]:h).length){for(B=0;B<3;++B)N[B]=H[B];N[3]=1}else if(4===H.length){for(B=0;B<4;++B)N[B]=H[B];!this.hasAlpha&&H[3]<1&&(this.hasAlpha=!0)}}else N[0]=N[1]=N[2]=0,N[3]=1;else N=[1,1,1,0];if(G)if(Array.isArray(I)){var H;if(3===(H=U?r<I.length?I[r]:[0,0,0,0]:I).length){for(B=0;B<3;++B)F[B]=H[B];F[B]=1}else if(4===H.length){for(B=0;B<4;++B)F[B]=H[B];!this.hasAlpha&&H[3]<1&&(this.hasAlpha=!0)}}else F[0]=F[1]=F[2]=0,F[3]=1;else F=[1,1,1,0];var j=.5;G?Array.isArray(d)?j=r<d.length?+d[r]:12:d?j=+d:this.useOrtho&&(j=12):j=0;var Y=0;Array.isArray(p)?Y=r<p.length?+p[r]:0:p&&(Y=+p);var P=Math.cos(Y),K=Math.sin(Y);for(v=s[r],B=0;B<3;++B)u[B]=Math.max(u[B],v[B]),c[B]=Math.min(c[B],v[B]);var z=i,J=o;z=0,Array.isArray(i)?z=r<i.length?i[r]:0:i&&(z=i),J=0,Array.isArray(o)?J=r<o.length?o[r]:0:o&&(J=o);var q=[z*=z>0?1-w[0][0]:z<0?1+w[1][0]:1,J*=J>0?1-w[0][1]:J<0?1+w[1][1]:1],V=b.cells||[],W=b.positions||[];for(B=0;B<V.length;++B)for(var X=V[B],Z=0;Z<3;++Z){for(var $=0;$<3;++$)_[3*L+$]=v[$];for($=0;$<4;++$)D[4*L+$]=N[$];k[L]=C;var tt=W[X[Z]];M[2*L]=j*(P*tt[0]-K*tt[1]+q[0]),M[2*L+1]=j*(K*tt[0]+P*tt[1]+q[1]),L+=1}for(V=Q.edges,W=Q.positions,B=0;B<V.length;++B)for(X=V[B],Z=0;Z<2;++Z){for($=0;$<3;++$)_[3*R+$]=v[$];for($=0;$<4;++$)D[4*R+$]=F[$];k[R]=C,tt=W[X[Z]],M[2*R]=j*(P*tt[0]-K*tt[1]+q[0]),M[2*R+1]=j*(K*tt[0]+P*tt[1]+q[1]),R+=1}}}this.bounds=[c,u],this.points=s,this.pointCount=s.length,this.vertexCount=m,this.lineVertexCount=y,this.pointBuffer.update(_),this.colorBuffer.update(D),this.glyphBuffer.update(M),this.idBuffer.update(k),a.free(_),a.free(D),a.free(M),a.free(k)},I.dispose=function(){this.shader.dispose(),this.orthoShader.dispose(),this.pickPerspectiveShader.dispose(),this.pickOrthoShader.dispose(),this.vao.dispose(),this.pointBuffer.dispose(),this.colorBuffer.dispose(),this.glyphBuffer.dispose(),this.idBuffer.dispose()}},{"./lib/get-simple-string":294,"./lib/glyphs":295,"./lib/shaders":296,"gl-buffer":241,"gl-mat4/multiply":267,"gl-vao":327,"is-string-blank":423,"typedarray-pool":545}],298:[function(t,e,n){"use strict";var r=t("glslify");n.boxVertex=r(["precision mediump float;\n#define GLSLIFY 1\n\nattribute vec2 vertex;\n\nuniform vec2 cornerA, cornerB;\n\nvoid main() {\n  gl_Position = vec4(mix(cornerA, cornerB, vertex), 0, 1);\n}\n"]),n.boxFragment=r(["precision mediump float;\n#define GLSLIFY 1\n\nuniform vec4 color;\n\nvoid main() {\n  gl_FragColor = color;\n}\n"])},{glslify:409}],299:[function(t,e,n){"use strict";var r=t("gl-shader"),i=t("gl-buffer"),o=t("./lib/shaders");function a(t,e,n){this.plot=t,this.boxBuffer=e,this.boxShader=n,this.enabled=!0,this.selectBox=[1/0,1/0,-1/0,-1/0],this.borderColor=[0,0,0,1],this.innerFill=!1,this.innerColor=[0,0,0,.25],this.outerFill=!0,this.outerColor=[0,0,0,.5],this.borderWidth=10}e.exports=function(t,e){var n=t.gl,s=i(n,[0,0,0,1,1,0,1,1]),A=r(n,o.boxVertex,o.boxFragment),l=new a(t,s,A);return l.update(e),t.addOverlay(l),l};var s=a.prototype;s.draw=function(){if(this.enabled){var t=this.plot,e=this.selectBox,n=this.borderWidth,r=(this.innerFill,this.innerColor),i=(this.outerFill,this.outerColor),o=this.borderColor,a=t.box,s=t.screenBox,A=t.dataBox,l=t.viewBox,c=t.pixelRatio,u=(e[0]-A[0])*(l[2]-l[0])/(A[2]-A[0])+l[0],g=(e[1]-A[1])*(l[3]-l[1])/(A[3]-A[1])+l[1],h=(e[2]-A[0])*(l[2]-l[0])/(A[2]-A[0])+l[0],d=(e[3]-A[1])*(l[3]-l[1])/(A[3]-A[1])+l[1];if(u=Math.max(u,l[0]),g=Math.max(g,l[1]),h=Math.min(h,l[2]),d=Math.min(d,l[3]),!(h<u||d<g)){a.bind();var f=s[2]-s[0],p=s[3]-s[1];if(this.outerFill&&(a.drawBox(0,0,f,g,i),a.drawBox(0,g,u,d,i),a.drawBox(0,d,f,p,i),a.drawBox(h,g,f,d,i)),this.innerFill&&a.drawBox(u,g,h,d,r),n>0){var I=n*c;a.drawBox(u-I,g-I,h+I,g+I,o),a.drawBox(u-I,d-I,h+I,d+I,o),a.drawBox(u-I,g-I,u+I,d+I,o),a.drawBox(h-I,g-I,h+I,d+I,o)}}}},s.update=function(t){t=t||{},this.innerFill=!!t.innerFill,this.outerFill=!!t.outerFill,this.innerColor=(t.innerColor||[0,0,0,.5]).slice(),this.outerColor=(t.outerColor||[0,0,0,.5]).slice(),this.borderColor=(t.borderColor||[0,0,0,1]).slice(),this.borderWidth=t.borderWidth||0,this.selectBox=(t.selectBox||this.selectBox).slice()},s.dispose=function(){this.boxBuffer.dispose(),this.boxShader.dispose(),this.plot.removeOverlay(this)}},{"./lib/shaders":298,"gl-buffer":241,"gl-shader":301}],300:[function(t,e,n){"use strict";e.exports=function(t,e){var n=r(t,e),o=i.mallocUint8(e[0]*e[1]*4);return new l(t,n,o)};var r=t("gl-fbo"),i=t("typedarray-pool"),o=t("ndarray"),a=t("bit-twiddle").nextPow2,s=t("cwise/lib/wrapper")({args:["array",{offset:[0,0,1],array:0},{offset:[0,0,2],array:0},{offset:[0,0,3],array:0},"scalar","scalar","index"],pre:{body:"{this_closestD2=1e8,this_closestX=-1,this_closestY=-1}",args:[],thisVars:["this_closestD2","this_closestX","this_closestY"],localVars:[]},body:{body:"{if(_inline_16_arg0_<255||_inline_16_arg1_<255||_inline_16_arg2_<255||_inline_16_arg3_<255){var _inline_16_l=_inline_16_arg4_-_inline_16_arg6_[0],_inline_16_a=_inline_16_arg5_-_inline_16_arg6_[1],_inline_16_f=_inline_16_l*_inline_16_l+_inline_16_a*_inline_16_a;_inline_16_f<this_closestD2&&(this_closestD2=_inline_16_f,this_closestX=_inline_16_arg6_[0],this_closestY=_inline_16_arg6_[1])}}",args:[{name:"_inline_16_arg0_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_16_arg1_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_16_arg2_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_16_arg3_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_16_arg4_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_16_arg5_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_16_arg6_",lvalue:!1,rvalue:!0,count:4}],thisVars:["this_closestD2","this_closestX","this_closestY"],localVars:["_inline_16_a","_inline_16_f","_inline_16_l"]},post:{body:"{return[this_closestX,this_closestY,this_closestD2]}",args:[],thisVars:["this_closestD2","this_closestX","this_closestY"],localVars:[]},debug:!1,funcName:"cwise",blockSize:64});function A(t,e,n,r,i){this.coord=[t,e],this.id=n,this.value=r,this.distance=i}function l(t,e,n){this.gl=t,this.fbo=e,this.buffer=n,this._readTimeout=null;var r=this;this._readCallback=function(){r.gl&&(e.bind(),t.readPixels(0,0,e.shape[0],e.shape[1],t.RGBA,t.UNSIGNED_BYTE,r.buffer),r._readTimeout=null)}}var c=l.prototype;Object.defineProperty(c,"shape",{get:function(){return this.gl?this.fbo.shape.slice():[0,0]},set:function(t){if(this.gl){this.fbo.shape=t;var e=this.fbo.shape[0],n=this.fbo.shape[1];if(n*e*4>this.buffer.length){i.free(this.buffer);for(var r=this.buffer=i.mallocUint8(a(n*e*4)),o=0;o<n*e*4;++o)r[o]=255}return t}}}),c.begin=function(){var t=this.gl;this.shape,t&&(this.fbo.bind(),t.clearColor(1,1,1,1),t.clear(t.COLOR_BUFFER_BIT|t.DEPTH_BUFFER_BIT))},c.end=function(){var t=this.gl;t&&(t.bindFramebuffer(t.FRAMEBUFFER,null),this._readTimeout||clearTimeout(this._readTimeout),this._readTimeout=setTimeout(this._readCallback,1))},c.query=function(t,e,n){if(!this.gl)return null;var r=this.fbo.shape.slice();t|=0,e|=0,"number"!=typeof n&&(n=1);var i=0|Math.min(Math.max(t-n,0),r[0]),a=0|Math.min(Math.max(t+n,0),r[0]),l=0|Math.min(Math.max(e-n,0),r[1]),c=0|Math.min(Math.max(e+n,0),r[1]);if(a<=i||c<=l)return null;var u=[a-i,c-l],g=o(this.buffer,[u[0],u[1],4],[4,4*r[0],1],4*(i+r[0]*l)),h=s(g.hi(u[0],u[1],1),n,n),d=h[0],f=h[1];return d<0||Math.pow(this.radius,2)<h[2]?null:new A(d+i|0,f+l|0,g.get(d,f,0),[g.get(d,f,1),g.get(d,f,2),g.get(d,f,3)],Math.sqrt(h[2]))},c.dispose=function(){this.gl&&(this.fbo.dispose(),i.free(this.buffer),this.gl=null,this._readTimeout&&clearTimeout(this._readTimeout))}},{"bit-twiddle":92,"cwise/lib/wrapper":149,"gl-fbo":249,ndarray:450,"typedarray-pool":545}],301:[function(t,e,n){"use strict";var r=t("./lib/create-uniforms"),i=t("./lib/create-attributes"),o=t("./lib/reflect"),a=t("./lib/shader-cache"),s=t("./lib/runtime-reflect"),A=t("./lib/GLError");function l(t){this.gl=t,this.gl.lastAttribCount=0,this._vref=this._fref=this._relink=this.vertShader=this.fragShader=this.program=this.attributes=this.uniforms=this.types=null}var c=l.prototype;function u(t,e){return t.name<e.name?-1:1}c.bind=function(){var t;this.program||this._relink();var e=this.gl.getProgramParameter(this.program,this.gl.ACTIVE_ATTRIBUTES),n=this.gl.lastAttribCount;if(e>n)for(t=n;t<e;t++)this.gl.enableVertexAttribArray(t);else if(n>e)for(t=e;t<n;t++)this.gl.disableVertexAttribArray(t);this.gl.lastAttribCount=e,this.gl.useProgram(this.program)},c.dispose=function(){for(var t=this.gl.lastAttribCount,e=0;e<t;e++)this.gl.disableVertexAttribArray(e);this.gl.lastAttribCount=0,this._fref&&this._fref.dispose(),this._vref&&this._vref.dispose(),this.attributes=this.types=this.vertShader=this.fragShader=this.program=this._relink=this._fref=this._vref=null},c.update=function(t,e,n,l){if(!e||1===arguments.length){var c=t;t=c.vertex,e=c.fragment,n=c.uniforms,l=c.attributes}var g=this,h=g.gl,d=g._vref;g._vref=a.shader(h,h.VERTEX_SHADER,t),d&&d.dispose(),g.vertShader=g._vref.shader;var f=this._fref;if(g._fref=a.shader(h,h.FRAGMENT_SHADER,e),f&&f.dispose(),g.fragShader=g._fref.shader,!n||!l){var p=h.createProgram();if(h.attachShader(p,g.fragShader),h.attachShader(p,g.vertShader),h.linkProgram(p),!h.getProgramParameter(p,h.LINK_STATUS)){var I=h.getProgramInfoLog(p);throw new A(I,"Error linking program:"+I)}n=n||s.uniforms(h,p),l=l||s.attributes(h,p),h.deleteProgram(p)}(l=l.slice()).sort(u);var C,m=[],y=[],E=[];for(C=0;C<l.length;++C){var v=l[C];if(v.type.indexOf("mat")>=0){for(var B=0|v.type.charAt(v.type.length-1),b=new Array(B),Q=0;Q<B;++Q)b[Q]=E.length,y.push(v.name+"["+Q+"]"),"number"==typeof v.location?E.push(v.location+Q):Array.isArray(v.location)&&v.location.length===B&&"number"==typeof v.location[Q]?E.push(0|v.location[Q]):E.push(-1);m.push({name:v.name,type:v.type,locations:b})}else m.push({name:v.name,type:v.type,locations:[E.length]}),y.push(v.name),"number"==typeof v.location?E.push(0|v.location):E.push(-1)}var w=0;for(C=0;C<E.length;++C)if(E[C]<0){for(;E.indexOf(w)>=0;)w+=1;E[C]=w}var x=new Array(n.length);function _(){g.program=a.program(h,g._vref,g._fref,y,E);for(var t=0;t<n.length;++t)x[t]=h.getUniformLocation(g.program,n[t].name)}_(),g._relink=_,g.types={uniforms:o(n),attributes:o(l)},g.attributes=i(h,g,m,E),Object.defineProperty(g,"uniforms",r(h,g,n,x))},e.exports=function(t,e,n,r,i){var o=new l(t);return o.update(e,n,r,i),o}},{"./lib/GLError":302,"./lib/create-attributes":303,"./lib/create-uniforms":304,"./lib/reflect":305,"./lib/runtime-reflect":306,"./lib/shader-cache":307}],302:[function(t,e,n){function r(t,e,n){this.shortMessage=e||"",this.longMessage=n||"",this.rawError=t||"",this.message="gl-shader: "+(e||t||"")+(n?"\n"+n:""),this.stack=(new Error).stack}r.prototype=new Error,r.prototype.name="GLError",r.prototype.constructor=r,e.exports=r},{}],303:[function(t,e,n){"use strict";e.exports=function(t,e,n,i){for(var o={},A=0,l=n.length;A<l;++A){var c=n[A],u=c.name,g=c.type,h=c.locations;switch(g){case"bool":case"int":case"float":a(t,e,h[0],i,1,o,u);break;default:if(g.indexOf("vec")>=0){var d=g.charCodeAt(g.length-1)-48;if(d<2||d>4)throw new r("","Invalid data type for attribute "+u+": "+g);a(t,e,h[0],i,d,o,u)}else{if(!(g.indexOf("mat")>=0))throw new r("","Unknown data type for attribute "+u+": "+g);var d=g.charCodeAt(g.length-1)-48;if(d<2||d>4)throw new r("","Invalid data type for attribute "+u+": "+g);s(t,e,h,i,d,o,u)}}}return o};var r=t("./GLError");function i(t,e,n,r,i,o){this._gl=t,this._wrapper=e,this._index=n,this._locations=r,this._dimension=i,this._constFunc=o}var o=i.prototype;function a(t,e,n,r,o,a,s){for(var A=["gl","v"],l=[],c=0;c<o;++c)A.push("x"+c),l.push("x"+c);A.push("if(x0.length===void 0){return gl.vertexAttrib"+o+"f(v,"+l.join()+")}else{return gl.vertexAttrib"+o+"fv(v,x0)}");var u=Function.apply(null,A),g=new i(t,e,n,r,o,u);Object.defineProperty(a,s,{set:function(e){return t.disableVertexAttribArray(r[n]),u(t,r[n],e),e},get:function(){return g},enumerable:!0})}function s(t,e,n,r,i,o,s){for(var A=new Array(i),l=new Array(i),c=0;c<i;++c)a(t,e,n[c],r,i,A,c),l[c]=A[c];Object.defineProperty(A,"location",{set:function(t){if(Array.isArray(t))for(var e=0;e<i;++e)l[e].location=t[e];else for(e=0;e<i;++e)l[e].location=t+e;return t},get:function(){for(var t=new Array(i),e=0;e<i;++e)t[e]=r[n[e]];return t},enumerable:!0}),A.pointer=function(e,o,a,s){e=e||t.FLOAT,o=!!o,a=a||i*i,s=s||0;for(var A=0;A<i;++A){var l=r[n[A]];t.vertexAttribPointer(l,i,e,o,a,s+A*i),t.enableVertexAttribArray(l)}};var u=new Array(i),g=t["vertexAttrib"+i+"fv"];Object.defineProperty(o,s,{set:function(e){for(var o=0;o<i;++o){var a=r[n[o]];if(t.disableVertexAttribArray(a),Array.isArray(e[0]))g.call(t,a,e[o]);else{for(var s=0;s<i;++s)u[s]=e[i*o+s];g.call(t,a,u)}}return e},get:function(){return A},enumerable:!0})}o.pointer=function(t,e,n,r){var i=this._gl,o=this._locations[this._index];i.vertexAttribPointer(o,this._dimension,t||i.FLOAT,!!e,n||0,r||0),i.enableVertexAttribArray(o)},o.set=function(t,e,n,r){return this._constFunc(this._locations[this._index],t,e,n,r)},Object.defineProperty(o,"location",{get:function(){return this._locations[this._index]},set:function(t){return t!==this._locations[this._index]&&(this._locations[this._index]=0|t,this._wrapper.program=null),0|t}})},{"./GLError":302}],304:[function(t,e,n){"use strict";var r=t("./reflect"),i=t("./GLError");function o(t){return new Function("y","return function(){return y}")(t)}function a(t,e){for(var n=new Array(t),r=0;r<t;++r)n[r]=e;return n}e.exports=function(t,e,n,s){function A(t,e,n){switch(n){case"bool":case"int":case"sampler2D":case"samplerCube":return"gl.uniform1i(locations["+e+"],obj"+t+")";case"float":return"gl.uniform1f(locations["+e+"],obj"+t+")";default:var r=n.indexOf("vec");if(!(0<=r&&r<=1&&n.length===4+r)){if(0===n.indexOf("mat")&&4===n.length){var o=n.charCodeAt(n.length-1)-48;if(o<2||o>4)throw new i("","Invalid uniform dimension type for matrix "+name+": "+n);return"gl.uniformMatrix"+o+"fv(locations["+e+"],false,obj"+t+")"}throw new i("","Unknown uniform data type for "+name+": "+n)}var o=n.charCodeAt(n.length-1)-48;if(o<2||o>4)throw new i("","Invalid data type");switch(n.charAt(0)){case"b":case"i":return"gl.uniform"+o+"iv(locations["+e+"],obj"+t+")";case"v":return"gl.uniform"+o+"fv(locations["+e+"],obj"+t+")";default:throw new i("","Unrecognized data type for vector "+name+": "+n)}}}function l(e){for(var r=["return function updateProperty(obj){"],i=function t(e,n){if("object"!=typeof n)return[[e,n]];var r=[];for(var i in n){var o=n[i],a=e;parseInt(i)+""===i?a+="["+i+"]":a+="."+i,"object"==typeof o?r.push.apply(r,t(a,o)):r.push([a,o])}return r}("",e),o=0;o<i.length;++o){var a=i[o],l=a[0],c=a[1];s[c]&&r.push(A(l,c,n[c].type))}r.push("return obj}");var u=new Function("gl","locations",r.join("\n"));return u(t,s)}function c(r,A,c){if("object"==typeof c){var g=u(c);Object.defineProperty(r,A,{get:o(g),set:l(c),enumerable:!0,configurable:!1})}else s[c]?Object.defineProperty(r,A,{get:(h=c,new Function("gl","wrapper","locations","return function(){return gl.getUniform(wrapper.program,locations["+h+"])}")(t,e,s)),set:l(c),enumerable:!0,configurable:!1}):r[A]=function(t){switch(t){case"bool":return!1;case"int":case"sampler2D":case"samplerCube":case"float":return 0;default:var e=t.indexOf("vec");if(0<=e&&e<=1&&t.length===4+e){var n=t.charCodeAt(t.length-1)-48;if(n<2||n>4)throw new i("","Invalid data type");return"b"===t.charAt(0)?a(n,!1):a(n,0)}if(0===t.indexOf("mat")&&4===t.length){var n=t.charCodeAt(t.length-1)-48;if(n<2||n>4)throw new i("","Invalid uniform dimension type for matrix "+name+": "+t);return a(n*n,0)}throw new i("","Unknown uniform data type for "+name+": "+t)}}(n[c].type);var h}function u(t){var e;if(Array.isArray(t)){e=new Array(t.length);for(var n=0;n<t.length;++n)c(e,n,t[n])}else for(var r in e={},t)c(e,r,t[r]);return e}var g=r(n,!0);return{get:o(u(g)),set:l(g),enumerable:!0,configurable:!0}}},{"./GLError":302,"./reflect":305}],305:[function(t,e,n){"use strict";e.exports=function(t,e){for(var n={},r=0;r<t.length;++r)for(var i=t[r].name,o=i.split("."),a=n,s=0;s<o.length;++s){var A=o[s].split("[");if(A.length>1){A[0]in a||(a[A[0]]=[]),a=a[A[0]];for(var l=1;l<A.length;++l){var c=parseInt(A[l]);l<A.length-1||s<o.length-1?(c in a||(l<A.length-1?a[c]=[]:a[c]={}),a=a[c]):a[c]=e?r:t[r].type}}else s<o.length-1?(A[0]in a||(a[A[0]]={}),a=a[A[0]]):a[A[0]]=e?r:t[r].type}return n}},{}],306:[function(t,e,n){"use strict";n.uniforms=function(t,e){for(var n=t.getProgramParameter(e,t.ACTIVE_UNIFORMS),r=[],i=0;i<n;++i){var a=t.getActiveUniform(e,i);if(a){var s=o(t,a.type);if(a.size>1)for(var A=0;A<a.size;++A)r.push({name:a.name.replace("[0]","["+A+"]"),type:s});else r.push({name:a.name,type:s})}}return r},n.attributes=function(t,e){for(var n=t.getProgramParameter(e,t.ACTIVE_ATTRIBUTES),r=[],i=0;i<n;++i){var a=t.getActiveAttrib(e,i);a&&r.push({name:a.name,type:o(t,a.type)})}return r};var r={FLOAT:"float",FLOAT_VEC2:"vec2",FLOAT_VEC3:"vec3",FLOAT_VEC4:"vec4",INT:"int",INT_VEC2:"ivec2",INT_VEC3:"ivec3",INT_VEC4:"ivec4",BOOL:"bool",BOOL_VEC2:"bvec2",BOOL_VEC3:"bvec3",BOOL_VEC4:"bvec4",FLOAT_MAT2:"mat2",FLOAT_MAT3:"mat3",FLOAT_MAT4:"mat4",SAMPLER_2D:"sampler2D",SAMPLER_CUBE:"samplerCube"},i=null;function o(t,e){if(!i){var n=Object.keys(r);i={};for(var o=0;o<n.length;++o){var a=n[o];i[t[a]]=r[a]}}return i[e]}},{}],307:[function(t,e,n){"use strict";n.shader=function(t,e,n){return c(t).getShaderReference(e,n)},n.program=function(t,e,n,r,i){return c(t).getProgram(e,n,r,i)};var r=t("./GLError"),i=t("gl-format-compiler-error"),o=new("undefined"==typeof WeakMap?t("weakmap-shim"):WeakMap),a=0;function s(t,e,n,r,i,o,a){this.id=t,this.src=e,this.type=n,this.shader=r,this.count=o,this.programs=[],this.cache=a}function A(t){this.gl=t,this.shaders=[{},{}],this.programs={}}s.prototype.dispose=function(){if(0==--this.count){for(var t=this.cache,e=t.gl,n=this.programs,r=0,i=n.length;r<i;++r){var o=t.programs[n[r]];o&&(delete t.programs[r],e.deleteProgram(o))}e.deleteShader(this.shader),delete t.shaders[this.type===e.FRAGMENT_SHADER|0][this.src]}};var l=A.prototype;function c(t){var e=o.get(t);return e||(e=new A(t),o.set(t,e)),e}l.getShaderReference=function(t,e){var n=this.gl,o=this.shaders[t===n.FRAGMENT_SHADER|0],A=o[e];if(A&&n.isShader(A.shader))A.count+=1;else{var l=function(t,e,n){var o=t.createShader(e);if(t.shaderSource(o,n),t.compileShader(o),!t.getShaderParameter(o,t.COMPILE_STATUS)){var a=t.getShaderInfoLog(o);try{var s=i(a,n,e)}catch(t){throw console.warn("Failed to format compiler error: "+t),new r(a,"Error compiling shader:\n"+a)}throw new r(a,s.short,s.long)}return o}(n,t,e);A=o[e]=new s(a++,e,t,l,[],1,this)}return A},l.getProgram=function(t,e,n,i){var o=[t.id,e.id,n.join(":"),i.join(":")].join("@"),a=this.programs[o];return a&&this.gl.isProgram(a)||(this.programs[o]=a=function(t,e,n,i,o){var a=t.createProgram();t.attachShader(a,e),t.attachShader(a,n);for(var s=0;s<i.length;++s)t.bindAttribLocation(a,o[s],i[s]);if(t.linkProgram(a),!t.getProgramParameter(a,t.LINK_STATUS)){var A=t.getProgramInfoLog(a);throw new r(A,"Error linking program: "+A)}return a}(this.gl,t.shader,e.shader,n,i),t.programs.push(o),e.programs.push(o)),a}},{"./GLError":302,"gl-format-compiler-error":250,"weakmap-shim":555}],308:[function(t,e,n){"use strict";function r(t){this.plot=t,this.enable=[!0,!0,!1,!1],this.width=[1,1,1,1],this.color=[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.center=[1/0,1/0]}e.exports=function(t,e){var n=new r(t);return n.update(e),t.addOverlay(n),n};var i=r.prototype;i.update=function(t){t=t||{},this.enable=(t.enable||[!0,!0,!1,!1]).slice(),this.width=(t.width||[1,1,1,1]).slice(),this.color=(t.color||[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]]).map(function(t){return t.slice()}),this.center=(t.center||[1/0,1/0]).slice(),this.plot.setOverlayDirty()},i.draw=function(){var t=this.enable,e=this.width,n=this.color,r=this.center,i=this.plot,o=i.line,a=i.dataBox,s=i.viewBox;if(o.bind(),a[0]<=r[0]&&r[0]<=a[2]&&a[1]<=r[1]&&r[1]<=a[3]){var A=s[0]+(r[0]-a[0])/(a[2]-a[0])*(s[2]-s[0]),l=s[1]+(r[1]-a[1])/(a[3]-a[1])*(s[3]-s[1]);t[0]&&o.drawLine(A,l,s[0],l,e[0],n[0]),t[1]&&o.drawLine(A,l,A,s[1],e[1],n[1]),t[2]&&o.drawLine(A,l,s[2],l,e[2],n[2]),t[3]&&o.drawLine(A,l,A,s[3],e[3],n[3])}},i.dispose=function(){this.plot.removeOverlay(this)}},{}],309:[function(t,e,n){"use strict";var r=t("glslify"),i=t("gl-shader"),o=r(["precision mediump float;\n#define GLSLIFY 1\n\nattribute vec3 position, color;\nattribute float weight;\n\nuniform mat4 model, view, projection;\nuniform vec3 coordinates[3];\nuniform vec4 colors[3];\nuniform vec2 screenShape;\nuniform float lineWidth;\n\nvarying vec4 fragColor;\n\nvoid main() {\n  vec3 vertexPosition = mix(coordinates[0],\n    mix(coordinates[2], coordinates[1], 0.5 * (position + 1.0)), abs(position));\n\n  vec4 clipPos = projection * view * model * vec4(vertexPosition, 1.0);\n  vec2 clipOffset = (projection * view * model * vec4(color, 0.0)).xy;\n  vec2 delta = weight * clipOffset * screenShape;\n  vec2 lineOffset = normalize(vec2(delta.y, -delta.x)) / screenShape;\n\n  gl_Position   = vec4(clipPos.xy + clipPos.w * 0.5 * lineWidth * lineOffset, clipPos.z, clipPos.w);\n  fragColor     = color.x * colors[0] + color.y * colors[1] + color.z * colors[2];\n}\n"]),a=r(["precision mediump float;\n#define GLSLIFY 1\n\nvarying vec4 fragColor;\n\nvoid main() {\n  gl_FragColor = fragColor;\n}"]);e.exports=function(t){return i(t,o,a,null,[{name:"position",type:"vec3"},{name:"color",type:"vec3"},{name:"weight",type:"float"}])}},{"gl-shader":301,glslify:409}],310:[function(t,e,n){"use strict";var r=t("gl-buffer"),i=t("gl-vao"),o=t("./shaders/index");e.exports=function(t,e){var n=[];function a(t,e,r,i,o,a){var s=[t,e,r,0,0,0,1];s[i+3]=1,s[i]=o,n.push.apply(n,s),s[6]=-1,n.push.apply(n,s),s[i]=a,n.push.apply(n,s),n.push.apply(n,s),s[6]=1,n.push.apply(n,s),s[i]=o,n.push.apply(n,s)}a(0,0,0,0,0,1),a(0,0,0,1,0,1),a(0,0,0,2,0,1),a(1,0,0,1,-1,1),a(1,0,0,2,-1,1),a(0,1,0,0,-1,1),a(0,1,0,2,-1,1),a(0,0,1,0,-1,1),a(0,0,1,1,-1,1);var A=r(t,n),l=i(t,[{type:t.FLOAT,buffer:A,size:3,offset:0,stride:28},{type:t.FLOAT,buffer:A,size:3,offset:12,stride:28},{type:t.FLOAT,buffer:A,size:1,offset:24,stride:28}]),c=o(t);c.attributes.position.location=0,c.attributes.color.location=1,c.attributes.weight.location=2;var u=new s(t,A,l,c);return u.update(e),u};var a=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];function s(t,e,n,r){this.gl=t,this.buffer=e,this.vao=n,this.shader=r,this.pixelRatio=1,this.bounds=[[-1e3,-1e3,-1e3],[1e3,1e3,1e3]],this.position=[0,0,0],this.lineWidth=[2,2,2],this.colors=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.enabled=[!0,!0,!0],this.drawSides=[!0,!0,!0],this.axes=null}var A=s.prototype,l=[0,0,0],c=[0,0,0],u=[0,0];A.isTransparent=function(){return!1},A.drawTransparent=function(t){},A.draw=function(t){var e=this.gl,n=this.vao,r=this.shader;n.bind(),r.bind();var i,o=t.model||a,s=t.view||a,A=t.projection||a;this.axes&&(i=this.axes.lastCubeProps.axis);for(var g=l,h=c,d=0;d<3;++d)i&&i[d]<0?(g[d]=this.bounds[0][d],h[d]=this.bounds[1][d]):(g[d]=this.bounds[1][d],h[d]=this.bounds[0][d]);for(u[0]=e.drawingBufferWidth,u[1]=e.drawingBufferHeight,r.uniforms.model=o,r.uniforms.view=s,r.uniforms.projection=A,r.uniforms.coordinates=[this.position,g,h],r.uniforms.colors=this.colors,r.uniforms.screenShape=u,d=0;d<3;++d)r.uniforms.lineWidth=this.lineWidth[d]*this.pixelRatio,this.enabled[d]&&(n.draw(e.TRIANGLES,6,6*d),this.drawSides[d]&&n.draw(e.TRIANGLES,12,18+12*d));n.unbind()},A.update=function(t){t&&("bounds"in t&&(this.bounds=t.bounds),"position"in t&&(this.position=t.position),"lineWidth"in t&&(this.lineWidth=t.lineWidth),"colors"in t&&(this.colors=t.colors),"enabled"in t&&(this.enabled=t.enabled),"drawSides"in t&&(this.drawSides=t.drawSides))},A.dispose=function(){this.vao.dispose(),this.buffer.dispose(),this.shader.dispose()}},{"./shaders/index":309,"gl-buffer":241,"gl-vao":327}],311:[function(t,e,n){var r=t("glslify"),i=r(["precision highp float;\n\nprecision highp float;\n#define GLSLIFY 1\n\nvec3 getOrthogonalVector(vec3 v) {\n  // Return up-vector for only-z vector.\n  // Return ax + by + cz = 0, a point that lies on the plane that has v as a normal and that isn't (0,0,0).\n  // From the above if-statement we have ||a|| > 0  U  ||b|| > 0.\n  // Assign z = 0, x = -b, y = a:\n  // a*-b + b*a + c*0 = -ba + ba + 0 = 0\n  if (v.x*v.x > v.z*v.z || v.y*v.y > v.z*v.z) {\n    return normalize(vec3(-v.y, v.x, 0.0));\n  } else {\n    return normalize(vec3(0.0, v.z, -v.y));\n  }\n}\n\n// Calculate the tube vertex and normal at the given index.\n//\n// The returned vertex is for a tube ring with its center at origin, radius of length(d), pointing in the direction of d.\n//\n// Each tube segment is made up of a ring of vertices.\n// These vertices are used to make up the triangles of the tube by connecting them together in the vertex array.\n// The indexes of tube segments run from 0 to 8.\n//\nvec3 getTubePosition(vec3 d, float index, out vec3 normal) {\n  float segmentCount = 8.0;\n\n  float angle = 2.0 * 3.14159 * (index / segmentCount);\n\n  vec3 u = getOrthogonalVector(d);\n  vec3 v = normalize(cross(u, d));\n\n  vec3 x = u * cos(angle) * length(d);\n  vec3 y = v * sin(angle) * length(d);\n  vec3 v3 = x + y;\n\n  normal = normalize(v3);\n\n  return v3;\n}\n\nattribute vec4 vector;\nattribute vec4 color, position;\nattribute vec2 uv;\nuniform float vectorScale;\nuniform float tubeScale;\n\nuniform mat4 model\n           , view\n           , projection\n           , inverseModel;\nuniform vec3 eyePosition\n           , lightPosition;\n\nvarying vec3 f_normal\n           , f_lightDirection\n           , f_eyeDirection\n           , f_data\n           , f_position;\nvarying vec4 f_color;\nvarying vec2 f_uv;\n\nvoid main() {\n  // Scale the vector magnitude to stay constant with\n  // model & view changes.\n  vec3 normal;\n  vec3 XYZ = getTubePosition(mat3(model) * (tubeScale * vector.w * normalize(vector.xyz)), position.w, normal);\n  vec4 tubePosition = model * vec4(position.xyz, 1.0) + vec4(XYZ, 0.0);\n\n  //Lighting geometry parameters\n  vec4 cameraCoordinate = view * tubePosition;\n  cameraCoordinate.xyz /= cameraCoordinate.w;\n  f_lightDirection = lightPosition - cameraCoordinate.xyz;\n  f_eyeDirection   = eyePosition - cameraCoordinate.xyz;\n  f_normal = normalize((vec4(normal,0.0) * inverseModel).xyz);\n\n  // vec4 m_position  = model * vec4(tubePosition, 1.0);\n  vec4 t_position  = view * tubePosition;\n  gl_Position      = projection * t_position;\n\n  f_color          = color;\n  f_data           = tubePosition.xyz;\n  f_position       = position.xyz;\n  f_uv             = uv;\n}\n"]),o=r(["#extension GL_OES_standard_derivatives : enable\n\nprecision highp float;\n#define GLSLIFY 1\n\nfloat beckmannDistribution(float x, float roughness) {\n  float NdotH = max(x, 0.0001);\n  float cos2Alpha = NdotH * NdotH;\n  float tan2Alpha = (cos2Alpha - 1.0) / cos2Alpha;\n  float roughness2 = roughness * roughness;\n  float denom = 3.141592653589793 * roughness2 * cos2Alpha * cos2Alpha;\n  return exp(tan2Alpha / roughness2) / denom;\n}\n\nfloat cookTorranceSpecular(\n  vec3 lightDirection,\n  vec3 viewDirection,\n  vec3 surfaceNormal,\n  float roughness,\n  float fresnel) {\n\n  float VdotN = max(dot(viewDirection, surfaceNormal), 0.0);\n  float LdotN = max(dot(lightDirection, surfaceNormal), 0.0);\n\n  //Half angle vector\n  vec3 H = normalize(lightDirection + viewDirection);\n\n  //Geometric term\n  float NdotH = max(dot(surfaceNormal, H), 0.0);\n  float VdotH = max(dot(viewDirection, H), 0.000001);\n  float LdotH = max(dot(lightDirection, H), 0.000001);\n  float G1 = (2.0 * NdotH * VdotN) / VdotH;\n  float G2 = (2.0 * NdotH * LdotN) / LdotH;\n  float G = min(1.0, min(G1, G2));\n  \n  //Distribution term\n  float D = beckmannDistribution(NdotH, roughness);\n\n  //Fresnel term\n  float F = pow(1.0 - VdotN, fresnel);\n\n  //Multiply terms and done\n  return  G * F * D / max(3.14159265 * VdotN, 0.000001);\n}\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 clipBounds[2];\nuniform float roughness\n            , fresnel\n            , kambient\n            , kdiffuse\n            , kspecular\n            , opacity;\nuniform sampler2D texture;\n\nvarying vec3 f_normal\n           , f_lightDirection\n           , f_eyeDirection\n           , f_data\n           , f_position;\nvarying vec4 f_color;\nvarying vec2 f_uv;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], f_position)) discard;\n  vec3 N = normalize(f_normal);\n  vec3 L = normalize(f_lightDirection);\n  vec3 V = normalize(f_eyeDirection);\n\n  if(gl_FrontFacing) {\n    N = -N;\n  }\n\n  float specular = min(1.0, max(0.0, cookTorranceSpecular(L, V, N, roughness, fresnel)));\n  float diffuse  = min(kambient + kdiffuse * max(dot(N, L), 0.0), 1.0);\n\n  vec4 surfaceColor = f_color * texture2D(texture, f_uv);\n  vec4 litColor = surfaceColor.a * vec4(diffuse * surfaceColor.rgb + kspecular * vec3(1,1,1) * specular,  1.0);\n\n  gl_FragColor = litColor * opacity;\n}\n"]),a=r(["precision highp float;\n\nprecision highp float;\n#define GLSLIFY 1\n\nvec3 getOrthogonalVector(vec3 v) {\n  // Return up-vector for only-z vector.\n  // Return ax + by + cz = 0, a point that lies on the plane that has v as a normal and that isn't (0,0,0).\n  // From the above if-statement we have ||a|| > 0  U  ||b|| > 0.\n  // Assign z = 0, x = -b, y = a:\n  // a*-b + b*a + c*0 = -ba + ba + 0 = 0\n  if (v.x*v.x > v.z*v.z || v.y*v.y > v.z*v.z) {\n    return normalize(vec3(-v.y, v.x, 0.0));\n  } else {\n    return normalize(vec3(0.0, v.z, -v.y));\n  }\n}\n\n// Calculate the tube vertex and normal at the given index.\n//\n// The returned vertex is for a tube ring with its center at origin, radius of length(d), pointing in the direction of d.\n//\n// Each tube segment is made up of a ring of vertices.\n// These vertices are used to make up the triangles of the tube by connecting them together in the vertex array.\n// The indexes of tube segments run from 0 to 8.\n//\nvec3 getTubePosition(vec3 d, float index, out vec3 normal) {\n  float segmentCount = 8.0;\n\n  float angle = 2.0 * 3.14159 * (index / segmentCount);\n\n  vec3 u = getOrthogonalVector(d);\n  vec3 v = normalize(cross(u, d));\n\n  vec3 x = u * cos(angle) * length(d);\n  vec3 y = v * sin(angle) * length(d);\n  vec3 v3 = x + y;\n\n  normal = normalize(v3);\n\n  return v3;\n}\n\nattribute vec4 vector;\nattribute vec4 position;\nattribute vec4 id;\n\nuniform mat4 model, view, projection;\nuniform float tubeScale;\n\nvarying vec3 f_position;\nvarying vec4 f_id;\n\nvoid main() {\n  vec3 normal;\n  vec3 XYZ = getTubePosition(mat3(model) * (tubeScale * vector.w * normalize(vector.xyz)), position.w, normal);\n  vec4 tubePosition = model * vec4(position.xyz, 1.0) + vec4(XYZ, 0.0);\n\n  gl_Position = projection * view * tubePosition;\n  f_id        = id;\n  f_position  = position.xyz;\n}\n"]),s=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3  clipBounds[2];\nuniform float pickId;\n\nvarying vec3 f_position;\nvarying vec4 f_id;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], f_position)) discard;\n\n  gl_FragColor = vec4(pickId, f_id.xyz);\n}"]);n.meshShader={vertex:i,fragment:o,attributes:[{name:"position",type:"vec4"},{name:"normal",type:"vec3"},{name:"color",type:"vec4"},{name:"uv",type:"vec2"},{name:"vector",type:"vec4"}]},n.pickShader={vertex:a,fragment:s,attributes:[{name:"position",type:"vec4"},{name:"id",type:"vec4"},{name:"vector",type:"vec4"}]}},{glslify:409}],312:[function(t,e,n){"use strict";var r=t("gl-shader"),i=t("gl-buffer"),o=t("gl-vao"),a=t("gl-texture2d"),s=t("normals"),A=t("gl-mat4/multiply"),l=t("gl-mat4/invert"),c=t("ndarray"),u=t("colormap"),g=t("simplicial-complex-contour"),h=t("typedarray-pool"),d=t("./shaders"),f=d.meshShader,p=d.pickShader,I=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];function C(t,e,n,r,i,o,a,s,A,l,c,u,g,h,d,f,p,C,m,y,E,v,B,b){this.gl=t,this.cells=[],this.positions=[],this.intensity=[],this.texture=e,this.dirty=!0,this.triShader=n,this.pickShader=r,this.trianglePositions=i,this.triangleVectors=o,this.triangleColors=s,this.triangleNormals=l,this.triangleUVs=A,this.triangleIds=a,this.triangleVAO=c,this.triangleCount=0,this.lineWidth=1,this.edgePositions=u,this.edgeColors=h,this.edgeUVs=d,this.edgeIds=g,this.edgeVAO=f,this.edgeCount=0,this.pointPositions=p,this.pointColors=m,this.pointUVs=y,this.pointSizes=E,this.pointIds=C,this.pointVAO=v,this.pointCount=0,this.contourLineWidth=1,this.contourPositions=B,this.contourVAO=b,this.contourCount=0,this.contourColor=[0,0,0],this.contourEnable=!1,this.pickId=1,this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],this.clipBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.lightPosition=[1e5,1e5,0],this.ambientLight=.8,this.diffuseLight=.8,this.specularLight=2,this.roughness=.5,this.fresnel=1.5,this.opacity=1,this.tubeScale=1,this._model=I,this._view=I,this._projection=I,this._resolution=[1,1],this.pixelRatio=1}var m=C.prototype;function y(t){var e=r(t,p.vertex,p.fragment,null,p.attributes);return e.attributes.position.location=0,e.attributes.id.location=1,e.attributes.vector.location=5,e}m.isOpaque=function(){return this.opacity>=1},m.isTransparent=function(){return this.opacity<1},m.pickSlots=1,m.setPickBase=function(t){this.pickId=t},m.highlight=function(t){if(t&&this.contourEnable){for(var e=g(this.cells,this.intensity,t.intensity),n=e.cells,r=e.vertexIds,i=e.vertexWeights,o=n.length,a=h.mallocFloat32(6*o),s=0,A=0;A<o;++A)for(var l=n[A],c=0;c<2;++c){var u=l[0];2===l.length&&(u=l[c]);for(var d=r[u][0],f=r[u][1],p=i[u],I=1-p,C=this.positions[d],m=this.positions[f],y=0;y<3;++y)a[s++]=p*C[y]+I*m[y]}this.contourCount=s/3|0,this.contourPositions.update(a.subarray(0,s)),h.free(a)}else this.contourCount=0},m.update=function(t){t=t||{};var e=this.gl;this.dirty=!0,"contourEnable"in t&&(this.contourEnable=t.contourEnable),"contourColor"in t&&(this.contourColor=t.contourColor),"lineWidth"in t&&(this.lineWidth=t.lineWidth),"lightPosition"in t&&(this.lightPosition=t.lightPosition),"opacity"in t&&(this.opacity=t.opacity),"ambient"in t&&(this.ambientLight=t.ambient),"diffuse"in t&&(this.diffuseLight=t.diffuse),"specular"in t&&(this.specularLight=t.specular),"roughness"in t&&(this.roughness=t.roughness),"fresnel"in t&&(this.fresnel=t.fresnel),t.texture?(this.texture.dispose(),this.texture=a(e,t.texture)):t.colormap&&(this.texture.shape=[256,256],this.texture.minFilter=e.LINEAR_MIPMAP_LINEAR,this.texture.magFilter=e.LINEAR,this.texture.setPixels(function(t){for(var e=u({colormap:t,nshades:256,format:"rgba"}),n=new Uint8Array(1024),r=0;r<256;++r){for(var i=e[r],o=0;o<3;++o)n[4*r+o]=i[o];n[4*r+3]=255*i[3]}return c(n,[256,256,4],[4,0,1])}(t.colormap)),this.texture.generateMipmap());var n=t.cells,r=t.positions,i=t.vectors;if(r&&n&&i){void 0!==t.tubeScale&&(this.tubeScale=t.tubeScale);var o=[],A=[],l=[],g=[],h=[],d=[],f=[],p=[],I=[],C=[],m=[],y=[],E=[],v=[],B=[];this.cells=n,this.positions=r,this.vectors=i;var b=t.vertexNormals,Q=t.cellNormals,w=void 0===t.vertexNormalsEpsilon?1e-6:t.vertexNormalsEpsilon,x=void 0===t.faceNormalsEpsilon?1e-6:t.faceNormalsEpsilon;t.useFacetNormals&&!Q&&(Q=s.faceNormals(n,r,x)),Q||b||(b=s.vertexNormals(n,r,w));var _=t.vertexColors,D=t.cellColors,M=t.meshColor||[1,1,1,1],k=t.vertexUVs,S=t.vertexIntensity,L=t.cellUVs,R=t.cellIntensity,N=1/0,F=-1/0;if(!k&&!L)if(S)if(t.vertexIntensityBounds)N=+t.vertexIntensityBounds[0],F=+t.vertexIntensityBounds[1];else for(var T=0;T<S.length;++T){var U=S[T];N=Math.min(N,U),F=Math.max(F,U)}else if(R)for(T=0;T<R.length;++T)U=R[T],N=Math.min(N,U),F=Math.max(F,U);else for(T=0;T<r.length;++T)U=r[T][2],N=Math.min(N,U),F=Math.max(F,U);this.intensity=S||(R?function(t,e,n){for(var r=new Array(e),i=0;i<e;++i)r[i]=0;var o=t.length;for(i=0;i<o;++i)for(var a=t[i],s=0;s<a.length;++s)r[a[s]]=n[i];return r}(n,r.length,R):function(t){for(var e=t.length,n=new Array(e),r=0;r<e;++r)n[r]=t[r][2];return n}(r));var O=t.pointSizes,G=t.pointSize||1;for(this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],T=0;T<r.length;++T)for(var H=r[T],j=0;j<3;++j)!isNaN(H[j])&&isFinite(H[j])&&(this.bounds[0][j]=Math.min(this.bounds[0][j],H[j]),this.bounds[1][j]=Math.max(this.bounds[1][j],H[j]));var Y=0,P=0,K=0;t:for(T=0;T<n.length;++T){var z=n[T];switch(z.length){case 1:for(H=r[q=z[0]],j=0;j<3;++j)if(isNaN(H[j])||!isFinite(H[j]))continue t;m.push(H[0],H[1],H[2],H[3]),3===(V=_?_[q]:D?D[T]:M).length?y.push(V[0],V[1],V[2],1):y.push(V[0],V[1],V[2],V[3]),W=k?k[q]:S?[(S[q]-N)/(F-N),0]:L?L[T]:R?[(R[T]-N)/(F-N),0]:[(H[2]-N)/(F-N),0],E.push(W[0],W[1]),O?v.push(O[q]):v.push(G),B.push(T),K+=1;break;case 2:for(j=0;j<2;++j){H=r[q=z[j]];for(var J=0;J<3;++J)if(isNaN(H[J])||!isFinite(H[J]))continue t}for(j=0;j<2;++j)H=r[q=z[j]],f.push(H[0],H[1],H[2]),3===(V=_?_[q]:D?D[T]:M).length?p.push(V[0],V[1],V[2],1):p.push(V[0],V[1],V[2],V[3]),W=k?k[q]:S?[(S[q]-N)/(F-N),0]:L?L[T]:R?[(R[T]-N)/(F-N),0]:[(H[2]-N)/(F-N),0],I.push(W[0],W[1]),C.push(T);P+=1;break;case 3:for(j=0;j<3;++j)for(H=r[q=z[j]],J=0;J<3;++J)if(isNaN(H[J])||!isFinite(H[J]))continue t;for(j=0;j<3;++j){var q;H=r[q=z[2-j]],o.push(H[0],H[1],H[2],H[3]);var V,W,X,Z=i[q];A.push(Z[0],Z[1],Z[2],Z[3]),3===(V=_?_[q]:D?D[T]:M).length?l.push(V[0],V[1],V[2],1):l.push(V[0],V[1],V[2],V[3]),W=k?k[q]:S?[(S[q]-N)/(F-N),0]:L?L[T]:R?[(R[T]-N)/(F-N),0]:[(H[2]-N)/(F-N),0],h.push(W[0],W[1]),X=b?b[q]:Q[T],g.push(X[0],X[1],X[2]),d.push(T)}Y+=1}}this.pointCount=K,this.edgeCount=P,this.triangleCount=Y,this.pointPositions.update(m),this.pointColors.update(y),this.pointUVs.update(E),this.pointSizes.update(v),this.pointIds.update(new Uint32Array(B)),this.edgePositions.update(f),this.edgeColors.update(p),this.edgeUVs.update(I),this.edgeIds.update(new Uint32Array(C)),this.trianglePositions.update(o),this.triangleVectors.update(A),this.triangleColors.update(l),this.triangleUVs.update(h),this.triangleNormals.update(g),this.triangleIds.update(new Uint32Array(d))}},m.drawTransparent=m.draw=function(t){t=t||{};for(var e=this.gl,n=t.model||I,r=t.view||I,i=t.projection||I,o=[[-1e6,-1e6,-1e6],[1e6,1e6,1e6]],a=0;a<3;++a)o[0][a]=Math.max(o[0][a],this.clipBounds[0][a]),o[1][a]=Math.min(o[1][a],this.clipBounds[1][a]);var s={model:n,view:r,projection:i,inverseModel:I.slice(),clipBounds:o,kambient:this.ambientLight,kdiffuse:this.diffuseLight,kspecular:this.specularLight,roughness:this.roughness,fresnel:this.fresnel,eyePosition:[0,0,0],lightPosition:[0,0,0],opacity:this.opacity,tubeScale:this.tubeScale,contourColor:this.contourColor,texture:0};s.inverseModel=l(s.inverseModel,s.model),e.disable(e.CULL_FACE),this.texture.bind(0);var c=new Array(16);for(A(c,s.view,s.model),A(c,s.projection,c),l(c,c),a=0;a<3;++a)s.eyePosition[a]=c[12+a]/c[15];var u=c[15];for(a=0;a<3;++a)u+=this.lightPosition[a]*c[4*a+3];for(a=0;a<3;++a){for(var g=c[12+a],h=0;h<3;++h)g+=c[4*h+a]*this.lightPosition[h];s.lightPosition[a]=g/u}if(this.triangleCount>0){var d=this.triShader;d.bind(),d.uniforms=s,this.triangleVAO.bind(),e.drawArrays(e.TRIANGLES,0,3*this.triangleCount),this.triangleVAO.unbind()}},m.drawPick=function(t){t=t||{};for(var e=this.gl,n=t.model||I,r=t.view||I,i=t.projection||I,o=[[-1e6,-1e6,-1e6],[1e6,1e6,1e6]],a=0;a<3;++a)o[0][a]=Math.max(o[0][a],this.clipBounds[0][a]),o[1][a]=Math.min(o[1][a],this.clipBounds[1][a]);this._model=[].slice.call(n),this._view=[].slice.call(r),this._projection=[].slice.call(i),this._resolution=[e.drawingBufferWidth,e.drawingBufferHeight];var s={model:n,view:r,projection:i,clipBounds:o,tubeScale:this.tubeScale,pickId:this.pickId/255},A=this.pickShader;A.bind(),A.uniforms=s,this.triangleCount>0&&(this.triangleVAO.bind(),e.drawArrays(e.TRIANGLES,0,3*this.triangleCount),this.triangleVAO.unbind()),this.edgeCount>0&&(this.edgeVAO.bind(),e.lineWidth(this.lineWidth*this.pixelRatio),e.drawArrays(e.LINES,0,2*this.edgeCount),this.edgeVAO.unbind())},m.pick=function(t){if(!t)return null;if(t.id!==this.pickId)return null;var e=t.value[0]+256*t.value[1]+65536*t.value[2],n=this.cells[e],r=this.positions[n[1]].slice(0,3);return{index:e,position:r,intensity:this.intensity[n[1]],velocity:this.vectors[n[1]].slice(0,3),divergence:this.vectors[n[1]][3],dataCoordinate:r}},m.dispose=function(){this.texture.dispose(),this.triShader.dispose(),this.pickShader.dispose(),this.triangleVAO.dispose(),this.trianglePositions.dispose(),this.triangleVectors.dispose(),this.triangleColors.dispose(),this.triangleUVs.dispose(),this.triangleNormals.dispose(),this.triangleIds.dispose(),this.edgeVAO.dispose(),this.edgePositions.dispose(),this.edgeColors.dispose(),this.edgeUVs.dispose(),this.edgeIds.dispose(),this.pointVAO.dispose(),this.pointPositions.dispose(),this.pointColors.dispose(),this.pointUVs.dispose(),this.pointSizes.dispose(),this.pointIds.dispose(),this.contourVAO.dispose(),this.contourPositions.dispose()},e.exports=function(t,e){1===arguments.length&&(t=(e=t).gl);var n=e.triShader||function(t){var e=r(t,f.vertex,f.fragment,null,f.attributes);return e.attributes.position.location=0,e.attributes.color.location=2,e.attributes.uv.location=3,e.attributes.vector.location=5,e}(t),s=y(t),A=a(t,c(new Uint8Array([255,255,255,255]),[1,1,4]));A.generateMipmap(),A.minFilter=t.LINEAR_MIPMAP_LINEAR,A.magFilter=t.LINEAR;var l=i(t),u=i(t),g=i(t),h=i(t),d=i(t),p=i(t),I=o(t,[{buffer:l,type:t.FLOAT,size:4},{buffer:p,type:t.UNSIGNED_BYTE,size:4,normalized:!0},{buffer:g,type:t.FLOAT,size:4},{buffer:h,type:t.FLOAT,size:2},{buffer:d,type:t.FLOAT,size:3},{buffer:u,type:t.FLOAT,size:4}]),m=i(t),E=i(t),v=i(t),B=i(t),b=o(t,[{buffer:m,type:t.FLOAT,size:3},{buffer:B,type:t.UNSIGNED_BYTE,size:4,normalized:!0},{buffer:E,type:t.FLOAT,size:4},{buffer:v,type:t.FLOAT,size:2}]),Q=i(t),w=i(t),x=i(t),_=i(t),D=i(t),M=o(t,[{buffer:Q,type:t.FLOAT,size:3},{buffer:D,type:t.UNSIGNED_BYTE,size:4,normalized:!0},{buffer:w,type:t.FLOAT,size:4},{buffer:x,type:t.FLOAT,size:2},{buffer:_,type:t.FLOAT,size:1}]),k=i(t),S=new C(t,A,n,s,l,u,p,g,h,d,I,m,B,E,v,b,Q,D,w,x,_,M,k,o(t,[{buffer:k,type:t.FLOAT,size:3}]));return S.update(e),S}},{"./shaders":311,colormap:126,"gl-buffer":241,"gl-mat4/invert":265,"gl-mat4/multiply":267,"gl-shader":301,"gl-texture2d":322,"gl-vao":327,ndarray:450,normals:453,"simplicial-complex-contour":518,"typedarray-pool":545}],313:[function(t,e,n){"use strict";var r=t("gl-vec3"),i=t("gl-vec4"),o=function(t,e,n,o){for(var a=0,s=0;s<t.length;s++)for(var A=t[s].velocities,l=0;l<A.length;l++){var c=r.length(A[l]);c>a&&(a=c)}var u=t.map(function(t){return function(t,e,n,o){var a,s,A,l=t.points,c=t.velocities,u=t.divergences;r.set(r.create(),0,1,0),r.create(),r.create(),r.create();for(var g=[],h=[],d=[],f=[],p=[],I=[],C=0,m=0,y=i.create(),E=i.create(),v=0;v<l.length;v++){a=l[v],s=c[v],A=u[v],0===e&&(A=.05*n),m=r.length(s)/o,y=i.create(),r.copy(y,s),y[3]=A;for(var B=0;B<8;B++)p[B]=[a[0],a[1],a[2],B];if(f.length>0)for(B=0;B<8;B++){var b=(B+1)%8;g.push(f[B],p[B],p[b],p[b],f[b],f[B]),d.push(E,y,y,y,E,E),I.push(C,m,m,m,C,C),h.push([g.length-6,g.length-5,g.length-4],[g.length-3,g.length-2,g.length-1])}var Q=f;f=p,p=Q,Q=E,E=y,y=Q,Q=C,C=m,m=Q}return{positions:g,cells:h,vectors:d,vertexIntensity:I}}(t,n,o,a)}),g=[],h=[],d=[],f=[];for(s=0;s<u.length;s++){var p=u[s],I=g.length;for(g=g.concat(p.positions),d=d.concat(p.vectors),f=f.concat(p.vertexIntensity),l=0;l<p.cells.length;l++){var C=p.cells[l],m=[];h.push(m);for(var y=0;y<C.length;y++)m.push(C[y]+I)}}return{positions:g,cells:h,vectors:d,vertexIntensity:f,colormap:e}},a=function(t,e){var n=r.create(),i=1e-4;r.add(n,t,[i,0,0]);var o=this.getVelocity(n);r.subtract(o,o,e),r.scale(o,o,1e4),r.add(n,t,[0,i,0]);var a=this.getVelocity(n);r.subtract(a,a,e),r.scale(a,a,1e4),r.add(n,t,[0,0,i]);var s=this.getVelocity(n);return r.subtract(s,s,e),r.scale(s,s,1e4),r.add(n,o,a),r.add(n,n,s),n},s=function(t){return g(t,this.vectors,this.meshgrid,this.clampBorders)},A=function(t,e){for(var n=0;n<t.length;n++){var r=t[n];if(r===e)return n;if(r>e)return n-1}return n},l=r.create(),c=r.create(),u=function(t,e,n){return t<e?e:t>n?n:t},g=function(t,e,n,i){var o=t[0],a=t[1],s=t[2],g=n[0].length,h=n[1].length,d=n[2].length,f=A(n[0],o),p=A(n[1],a),I=A(n[2],s),C=f+1,m=p+1,y=I+1;if(n[0][f]===o&&(C=f),n[1][p]===a&&(m=p),n[2][I]===s&&(y=I),i&&(f=u(f,0,g-1),C=u(C,0,g-1),p=u(p,0,h-1),m=u(m,0,h-1),I=u(I,0,d-1),y=u(y,0,d-1)),f<0||p<0||I<0||C>=g||m>=h||y>=d)return r.create();var E=(o-n[0][f])/(n[0][C]-n[0][f]),v=(a-n[1][p])/(n[1][m]-n[1][p]),B=(s-n[2][I])/(n[2][y]-n[2][I]);(E<0||E>1||isNaN(E))&&(E=0),(v<0||v>1||isNaN(v))&&(v=0),(B<0||B>1||isNaN(B))&&(B=0);var b=I*g*h,Q=y*g*h,w=p*g,x=m*g,_=f,D=C,M=e[w+b+_],k=e[w+b+D],S=e[x+b+_],L=e[x+b+D],R=e[w+Q+_],N=e[w+Q+D],F=e[x+Q+_],T=e[x+Q+D],U=r.create();return r.lerp(U,M,k,E),r.lerp(l,S,L,E),r.lerp(U,U,l,v),r.lerp(l,R,N,E),r.lerp(c,F,T,E),r.lerp(l,l,c,v),r.lerp(U,U,l,B),U},h=function(t){var e=1/0;t.sort(function(t,e){return t-e});for(var n=1;n<t.length;n++){var r=Math.abs(t[n]-t[n-1]);r<e&&(e=r)}return e};e.exports=function(t,e){var n=t.startingPositions,i=t.maxLength||1e3,A=t.tubeSize||1,l=t.absoluteTubeSize;t.getDivergence||(t.getDivergence=a),t.getVelocity||(t.getVelocity=s),void 0===t.clampBorders&&(t.clampBorders=!0);var c=[],u=e[0][0],g=e[0][1],d=e[0][2],f=e[1][0],p=e[1][1],I=e[1][2],C=function(t,e){var n=e[0],r=e[1],i=e[2];return n>=u&&n<=f&&r>=g&&r<=p&&i>=d&&i<=I},m=10*r.distance(e[0],e[1])/i,y=m*m,E=1,v=0;r.create(),n.length>=2&&(E=function(t){for(var e=[],n=[],r=[],i={},o={},a={},s=0;s<t.length;s++){var A=t[s],l=A[0],c=A[1],u=A[2];i[l]||(e.push(l),i[l]=!0),o[c]||(n.push(c),o[c]=!0),a[u]||(r.push(u),a[u]=!0)}var g=h(e),d=h(n),f=h(r),p=Math.min(g,d,f);return isFinite(p)?p:1}(n));for(var B=0;B<n.length;B++){var b=r.create();r.copy(b,n[B]);var Q=[b],w=[],x=t.getVelocity(b),_=b;w.push(x);var D=[],M=t.getDivergence(b,x);(R=r.length(M))>v&&!isNaN(R)&&isFinite(R)&&(v=R),D.push(R),c.push({points:Q,velocities:w,divergences:D});for(var k=0;k<100*i&&Q.length<i&&C(0,b);){k++;var S=r.clone(x),L=r.squaredLength(S);if(0===L)break;L>y&&r.scale(S,S,m/Math.sqrt(L)),r.add(S,S,b),x=t.getVelocity(S),r.squaredDistance(_,S)-y>-1e-4*y&&(Q.push(S),_=S,w.push(x),M=t.getDivergence(S,x),(R=r.length(M))>v&&!isNaN(R)&&isFinite(R)&&(v=R),D.push(R)),b=S}}for(B=0;B<D.length;B++){var R=D[B];!isNaN(R)&&isFinite(R)||(D[B]=v)}var N=o(c,t.colormap,v,E);return l?N.tubeScale=l:(0===v&&(v=1),N.tubeScale=.5*A*E/v),N},e.exports.createTubeMesh=t("./lib/tubemesh")},{"./lib/tubemesh":312,"gl-vec3":346,"gl-vec4":382}],314:[function(t,e,n){var r=t("gl-shader"),i=t("glslify"),o=i(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec4 uv;\nattribute vec3 f;\nattribute vec3 normal;\n\nuniform vec3 objectOffset;\nuniform mat4 model, view, projection, inverseModel;\nuniform vec3 lightPosition, eyePosition;\nuniform sampler2D colormap;\n\nvarying float value, kill;\nvarying vec3 worldCoordinate;\nvarying vec2 planeCoordinate;\nvarying vec3 lightDirection, eyeDirection, surfaceNormal;\nvarying vec4 vColor;\n\nvoid main() {\n  vec3 localCoordinate = vec3(uv.zw, f.x);\n  worldCoordinate = objectOffset + localCoordinate;\n  vec4 worldPosition = model * vec4(worldCoordinate, 1.0);\n  vec4 clipPosition = projection * view * worldPosition;\n  gl_Position = clipPosition;\n  kill = f.y;\n  value = f.z;\n  planeCoordinate = uv.xy;\n\n  vColor = texture2D(colormap, vec2(value, value));\n\n  //Lighting geometry parameters\n  vec4 cameraCoordinate = view * worldPosition;\n  cameraCoordinate.xyz /= cameraCoordinate.w;\n  lightDirection = lightPosition - cameraCoordinate.xyz;\n  eyeDirection   = eyePosition - cameraCoordinate.xyz;\n  surfaceNormal  = normalize((vec4(normal,0) * inverseModel).xyz);\n}\n"]),a=i(["precision highp float;\n#define GLSLIFY 1\n\nfloat beckmannDistribution(float x, float roughness) {\n  float NdotH = max(x, 0.0001);\n  float cos2Alpha = NdotH * NdotH;\n  float tan2Alpha = (cos2Alpha - 1.0) / cos2Alpha;\n  float roughness2 = roughness * roughness;\n  float denom = 3.141592653589793 * roughness2 * cos2Alpha * cos2Alpha;\n  return exp(tan2Alpha / roughness2) / denom;\n}\n\nfloat beckmannSpecular(\n  vec3 lightDirection,\n  vec3 viewDirection,\n  vec3 surfaceNormal,\n  float roughness) {\n  return beckmannDistribution(dot(surfaceNormal, normalize(lightDirection + viewDirection)), roughness);\n}\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 lowerBound, upperBound;\nuniform float contourTint;\nuniform vec4 contourColor;\nuniform sampler2D colormap;\nuniform vec3 clipBounds[2];\nuniform float roughness, fresnel, kambient, kdiffuse, kspecular, opacity;\nuniform float vertexColor;\n\nvarying float value, kill;\nvarying vec3 worldCoordinate;\nvarying vec3 lightDirection, eyeDirection, surfaceNormal;\nvarying vec4 vColor;\n\nvoid main() {\n  if ((kill > 0.0) ||\n      (outOfRange(clipBounds[0], clipBounds[1], worldCoordinate))) discard;\n\n  vec3 N = normalize(surfaceNormal);\n  vec3 V = normalize(eyeDirection);\n  vec3 L = normalize(lightDirection);\n\n  if(gl_FrontFacing) {\n    N = -N;\n  }\n\n  float specular = max(beckmannSpecular(L, V, N, roughness), 0.);\n  float diffuse  = min(kambient + kdiffuse * max(dot(N, L), 0.0), 1.0);\n\n  //decide how to interpolate color — in vertex or in fragment\n  vec4 surfaceColor =\n    step(vertexColor, .5) * texture2D(colormap, vec2(value, value)) +\n    step(.5, vertexColor) * vColor;\n\n  vec4 litColor = surfaceColor.a * vec4(diffuse * surfaceColor.rgb + kspecular * vec3(1,1,1) * specular,  1.0);\n\n  gl_FragColor = mix(litColor, contourColor, contourTint) * opacity;\n}\n"]),s=i(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec4 uv;\nattribute float f;\n\nuniform vec3 objectOffset;\nuniform mat3 permutation;\nuniform mat4 model, view, projection;\nuniform float height, zOffset;\nuniform sampler2D colormap;\n\nvarying float value, kill;\nvarying vec3 worldCoordinate;\nvarying vec2 planeCoordinate;\nvarying vec3 lightDirection, eyeDirection, surfaceNormal;\nvarying vec4 vColor;\n\nvoid main() {\n  vec3 dataCoordinate = permutation * vec3(uv.xy, height);\n  worldCoordinate = objectOffset + dataCoordinate;\n  vec4 worldPosition = model * vec4(worldCoordinate, 1.0);\n\n  vec4 clipPosition = projection * view * worldPosition;\n  clipPosition.z += zOffset;\n\n  gl_Position = clipPosition;\n  value = f + objectOffset.z;\n  kill = -1.0;\n  planeCoordinate = uv.zw;\n\n  vColor = texture2D(colormap, vec2(value, value));\n\n  //Don't do lighting for contours\n  surfaceNormal   = vec3(1,0,0);\n  eyeDirection    = vec3(0,1,0);\n  lightDirection  = vec3(0,0,1);\n}\n"]),A=i(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec2 shape;\nuniform vec3 clipBounds[2];\nuniform float pickId;\n\nvarying float value, kill;\nvarying vec3 worldCoordinate;\nvarying vec2 planeCoordinate;\nvarying vec3 surfaceNormal;\n\nvec2 splitFloat(float v) {\n  float vh = 255.0 * v;\n  float upper = floor(vh);\n  float lower = fract(vh);\n  return vec2(upper / 255.0, floor(lower * 16.0) / 16.0);\n}\n\nvoid main() {\n  if ((kill > 0.0) ||\n      (outOfRange(clipBounds[0], clipBounds[1], worldCoordinate))) discard;\n\n  vec2 ux = splitFloat(planeCoordinate.x / shape.x);\n  vec2 uy = splitFloat(planeCoordinate.y / shape.y);\n  gl_FragColor = vec4(pickId, ux.x, uy.x, ux.y + (uy.y/16.0));\n}\n"]);n.createShader=function(t){var e=r(t,o,a,null,[{name:"uv",type:"vec4"},{name:"f",type:"vec3"},{name:"normal",type:"vec3"}]);return e.attributes.uv.location=0,e.attributes.f.location=1,e.attributes.normal.location=2,e},n.createPickShader=function(t){var e=r(t,o,A,null,[{name:"uv",type:"vec4"},{name:"f",type:"vec3"},{name:"normal",type:"vec3"}]);return e.attributes.uv.location=0,e.attributes.f.location=1,e.attributes.normal.location=2,e},n.createContourShader=function(t){var e=r(t,s,a,null,[{name:"uv",type:"vec4"},{name:"f",type:"float"}]);return e.attributes.uv.location=0,e.attributes.f.location=1,e},n.createPickContourShader=function(t){var e=r(t,s,A,null,[{name:"uv",type:"vec4"},{name:"f",type:"float"}]);return e.attributes.uv.location=0,e.attributes.f.location=1,e}},{"gl-shader":301,glslify:409}],315:[function(t,e,n){arguments[4][111][0].apply(n,arguments)},{dup:111}],316:[function(t,e,n){"use strict";e.exports=function(t){var e=t.gl,n=C(e),r=y(e),s=m(e),A=E(e),l=i(e),c=o(e,[{buffer:l,size:4,stride:v,offset:0},{buffer:l,size:3,stride:v,offset:16},{buffer:l,size:3,stride:v,offset:28}]),u=i(e),g=o(e,[{buffer:u,size:4,stride:20,offset:0},{buffer:u,size:1,stride:20,offset:16}]),h=i(e),d=o(e,[{buffer:h,size:2,type:e.FLOAT}]),f=a(e,1,x,e.RGBA,e.UNSIGNED_BYTE);f.minFilter=e.LINEAR,f.magFilter=e.LINEAR;var p=new _(e,[0,0],[[0,0,0],[0,0,0]],n,r,l,c,f,s,A,u,g,h,d,[0,0,0]),I={levels:[[],[],[]]};for(var B in t)I[B]=t[B];return I.colormap=I.colormap||"jet",p.update(I),p};var r=t("bit-twiddle"),i=t("gl-buffer"),o=t("gl-vao"),a=t("gl-texture2d"),s=t("typedarray-pool"),A=t("colormap"),l=t("ndarray-ops"),c=t("ndarray-pack"),u=t("ndarray"),g=t("surface-nets"),h=t("gl-mat4/multiply"),d=t("gl-mat4/invert"),f=t("binary-search-bounds"),p=t("ndarray-gradient"),I=t("./lib/shaders"),C=I.createShader,m=I.createContourShader,y=I.createPickShader,E=I.createPickContourShader,v=40,B=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1],b=[[0,0],[0,1],[1,0],[1,1],[1,0],[0,1]],Q=[[0,0,0,0,0,0,0,0,0],[0,0,0,0,0,0,0,0,0],[0,0,0,0,0,0,0,0,0]];function w(t,e,n,r,i){this.position=t,this.index=e,this.uv=n,this.level=r,this.dataCoordinate=i}!function(){for(var t=0;t<3;++t){var e=Q[t],n=(t+2)%3;e[(t+1)%3+0]=1,e[n+3]=1,e[t+6]=1}}();var x=256;function _(t,e,n,r,i,o,a,A,l,c,g,h,d,f,p){this.gl=t,this.shape=e,this.bounds=n,this.objectOffset=p,this.intensityBounds=[],this._shader=r,this._pickShader=i,this._coordinateBuffer=o,this._vao=a,this._colorMap=A,this._contourShader=l,this._contourPickShader=c,this._contourBuffer=g,this._contourVAO=h,this._contourOffsets=[[],[],[]],this._contourCounts=[[],[],[]],this._vertexCount=0,this._pickResult=new w([0,0,0],[0,0],[0,0],[0,0,0],[0,0,0]),this._dynamicBuffer=d,this._dynamicVAO=f,this._dynamicOffsets=[0,0,0],this._dynamicCounts=[0,0,0],this.contourWidth=[1,1,1],this.contourLevels=[[1],[1],[1]],this.contourTint=[0,0,0],this.contourColor=[[.5,.5,.5,1],[.5,.5,.5,1],[.5,.5,.5,1]],this.showContour=!0,this.showSurface=!0,this.enableHighlight=[!0,!0,!0],this.highlightColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.highlightTint=[1,1,1],this.highlightLevel=[-1,-1,-1],this.enableDynamic=[!0,!0,!0],this.dynamicLevel=[NaN,NaN,NaN],this.dynamicColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.dynamicTint=[1,1,1],this.dynamicWidth=[1,1,1],this.axesBounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],this.surfaceProject=[!1,!1,!1],this.contourProject=[[!1,!1,!1],[!1,!1,!1],[!1,!1,!1]],this.colorBounds=[!1,!1],this._field=[u(s.mallocFloat(1024),[0,0]),u(s.mallocFloat(1024),[0,0]),u(s.mallocFloat(1024),[0,0])],this.pickId=1,this.clipBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.snapToData=!1,this.pixelRatio=1,this.opacity=1,this.lightPosition=[10,1e4,0],this.ambientLight=.8,this.diffuseLight=.8,this.specularLight=2,this.roughness=.5,this.fresnel=1.5,this.vertexColor=0,this.dirty=!0}var D=_.prototype;D.isTransparent=function(){return this.opacity<1},D.isOpaque=function(){if(this.opacity>=1)return!0;for(var t=0;t<3;++t)if(this._contourCounts[t].length>0||this._dynamicCounts[t]>0)return!0;return!1},D.pickSlots=1,D.setPickBase=function(t){this.pickId=t};var M=[0,0,0],k={showSurface:!1,showContour:!1,projections:[B.slice(),B.slice(),B.slice()],clipBounds:[[[0,0,0],[0,0,0]],[[0,0,0],[0,0,0]],[[0,0,0],[0,0,0]]]};function S(t,e){var n,r,i,o=e.axes&&e.axes.lastCubeProps.axis||M,a=e.showSurface,s=e.showContour;for(n=0;n<3;++n)for(a=a||e.surfaceProject[n],r=0;r<3;++r)s=s||e.contourProject[n][r];for(n=0;n<3;++n){var A=k.projections[n];for(r=0;r<16;++r)A[r]=0;for(r=0;r<4;++r)A[5*r]=1;A[5*n]=0,A[12+n]=e.axesBounds[+(o[n]>0)][n],h(A,t.model,A);var l=k.clipBounds[n];for(i=0;i<2;++i)for(r=0;r<3;++r)l[i][r]=t.clipBounds[i][r];l[0][n]=-1e8,l[1][n]=1e8}return k.showSurface=a,k.showContour=s,k}var L={model:B,view:B,projection:B,inverseModel:B.slice(),lowerBound:[0,0,0],upperBound:[0,0,0],colorMap:0,clipBounds:[[0,0,0],[0,0,0]],height:0,contourTint:0,contourColor:[0,0,0,1],permutation:[1,0,0,0,1,0,0,0,1],zOffset:-1e-4,objectOffset:[0,0,0],kambient:1,kdiffuse:1,kspecular:1,lightPosition:[1e3,1e3,1e3],eyePosition:[0,0,0],roughness:1,fresnel:1,opacity:1,vertexColor:0},R=B.slice(),N=[1,0,0,0,1,0,0,0,1];function F(t,e){t=t||{};var n=this.gl;n.disable(n.CULL_FACE),this._colorMap.bind(0);var r=L;r.model=t.model||B,r.view=t.view||B,r.projection=t.projection||B,r.lowerBound=[this.bounds[0][0],this.bounds[0][1],this.colorBounds[0]||this.bounds[0][2]],r.upperBound=[this.bounds[1][0],this.bounds[1][1],this.colorBounds[1]||this.bounds[1][2]],r.objectOffset=this.objectOffset,r.contourColor=this.contourColor[0],r.inverseModel=d(r.inverseModel,r.model);for(var i=0;i<2;++i)for(var o=r.clipBounds[i],a=0;a<3;++a)o[a]=Math.min(Math.max(this.clipBounds[i][a],-1e8),1e8);r.kambient=this.ambientLight,r.kdiffuse=this.diffuseLight,r.kspecular=this.specularLight,r.roughness=this.roughness,r.fresnel=this.fresnel,r.opacity=this.opacity,r.height=0,r.permutation=N,r.vertexColor=this.vertexColor;var s=R;for(h(s,r.view,r.model),h(s,r.projection,s),d(s,s),i=0;i<3;++i)r.eyePosition[i]=s[12+i]/s[15];var A=s[15];for(i=0;i<3;++i)A+=this.lightPosition[i]*s[4*i+3];for(i=0;i<3;++i){var l=s[12+i];for(a=0;a<3;++a)l+=s[4*a+i]*this.lightPosition[a];r.lightPosition[i]=l/A}var c=S(r,this);if(c.showSurface&&e===this.opacity<1){for(this._shader.bind(),this._shader.uniforms=r,this._vao.bind(),this.showSurface&&this._vertexCount&&this._vao.draw(n.TRIANGLES,this._vertexCount),i=0;i<3;++i)this.surfaceProject[i]&&this.vertexCount&&(this._shader.uniforms.model=c.projections[i],this._shader.uniforms.clipBounds=c.clipBounds[i],this._vao.draw(n.TRIANGLES,this._vertexCount));this._vao.unbind()}if(c.showContour&&!e){var u=this._contourShader;r.kambient=1,r.kdiffuse=0,r.kspecular=0,r.opacity=1,u.bind(),u.uniforms=r;var g=this._contourVAO;for(g.bind(),i=0;i<3;++i)for(u.uniforms.permutation=Q[i],n.lineWidth(this.contourWidth[i]*this.pixelRatio),a=0;a<this.contourLevels[i].length;++a)a===this.highlightLevel[i]?(u.uniforms.contourColor=this.highlightColor[i],u.uniforms.contourTint=this.highlightTint[i]):0!==a&&a-1!==this.highlightLevel[i]||(u.uniforms.contourColor=this.contourColor[i],u.uniforms.contourTint=this.contourTint[i]),this._contourCounts[i][a]&&(u.uniforms.height=this.contourLevels[i][a],g.draw(n.LINES,this._contourCounts[i][a],this._contourOffsets[i][a]));for(i=0;i<3;++i)for(u.uniforms.model=c.projections[i],u.uniforms.clipBounds=c.clipBounds[i],a=0;a<3;++a)if(this.contourProject[i][a]){u.uniforms.permutation=Q[a],n.lineWidth(this.contourWidth[a]*this.pixelRatio);for(var f=0;f<this.contourLevels[a].length;++f)f===this.highlightLevel[a]?(u.uniforms.contourColor=this.highlightColor[a],u.uniforms.contourTint=this.highlightTint[a]):0!==f&&f-1!==this.highlightLevel[a]||(u.uniforms.contourColor=this.contourColor[a],u.uniforms.contourTint=this.contourTint[a]),this._contourCounts[a][f]&&(u.uniforms.height=this.contourLevels[a][f],g.draw(n.LINES,this._contourCounts[a][f],this._contourOffsets[a][f]))}for(g.unbind(),(g=this._dynamicVAO).bind(),i=0;i<3;++i)if(0!==this._dynamicCounts[i])for(u.uniforms.model=r.model,u.uniforms.clipBounds=r.clipBounds,u.uniforms.permutation=Q[i],n.lineWidth(this.dynamicWidth[i]*this.pixelRatio),u.uniforms.contourColor=this.dynamicColor[i],u.uniforms.contourTint=this.dynamicTint[i],u.uniforms.height=this.dynamicLevel[i],g.draw(n.LINES,this._dynamicCounts[i],this._dynamicOffsets[i]),a=0;a<3;++a)this.contourProject[a][i]&&(u.uniforms.model=c.projections[a],u.uniforms.clipBounds=c.clipBounds[a],g.draw(n.LINES,this._dynamicCounts[i],this._dynamicOffsets[i]));g.unbind()}}D.draw=function(t){return F.call(this,t,!1)},D.drawTransparent=function(t){return F.call(this,t,!0)};var T={model:B,view:B,projection:B,inverseModel:B,clipBounds:[[0,0,0],[0,0,0]],height:0,shape:[0,0],pickId:0,lowerBound:[0,0,0],upperBound:[0,0,0],zOffset:0,objectOffset:[0,0,0],permutation:[1,0,0,0,1,0,0,0,1],lightPosition:[0,0,0],eyePosition:[0,0,0]};function U(t,e){return Array.isArray(t)?[e(t[0]),e(t[1]),e(t[2])]:[e(t),e(t),e(t)]}function O(t){return Array.isArray(t)?3===t.length?[t[0],t[1],t[2],1]:[t[0],t[1],t[2],t[3]]:[0,0,0,1]}function G(t){if(Array.isArray(t)){if(Array.isArray(t))return[O(t[0]),O(t[1]),O(t[2])];var e=O(t);return[e.slice(),e.slice(),e.slice()]}}D.drawPick=function(t){t=t||{};var e=this.gl;e.disable(e.CULL_FACE);var n=T;n.model=t.model||B,n.view=t.view||B,n.projection=t.projection||B,n.shape=this._field[2].shape,n.pickId=this.pickId/255,n.lowerBound=this.bounds[0],n.upperBound=this.bounds[1],n.objectOffset=this.objectOffset,n.permutation=N;for(var r=0;r<2;++r)for(var i=n.clipBounds[r],o=0;o<3;++o)i[o]=Math.min(Math.max(this.clipBounds[r][o],-1e8),1e8);var a=S(n,this);if(a.showSurface){for(this._pickShader.bind(),this._pickShader.uniforms=n,this._vao.bind(),this._vao.draw(e.TRIANGLES,this._vertexCount),r=0;r<3;++r)this.surfaceProject[r]&&(this._pickShader.uniforms.model=a.projections[r],this._pickShader.uniforms.clipBounds=a.clipBounds[r],this._vao.draw(e.TRIANGLES,this._vertexCount));this._vao.unbind()}if(a.showContour){var s=this._contourPickShader;s.bind(),s.uniforms=n;var A=this._contourVAO;for(A.bind(),o=0;o<3;++o)for(e.lineWidth(this.contourWidth[o]*this.pixelRatio),s.uniforms.permutation=Q[o],r=0;r<this.contourLevels[o].length;++r)this._contourCounts[o][r]&&(s.uniforms.height=this.contourLevels[o][r],A.draw(e.LINES,this._contourCounts[o][r],this._contourOffsets[o][r]));for(r=0;r<3;++r)for(s.uniforms.model=a.projections[r],s.uniforms.clipBounds=a.clipBounds[r],o=0;o<3;++o)if(this.contourProject[r][o]){s.uniforms.permutation=Q[o],e.lineWidth(this.contourWidth[o]*this.pixelRatio);for(var l=0;l<this.contourLevels[o].length;++l)this._contourCounts[o][l]&&(s.uniforms.height=this.contourLevels[o][l],A.draw(e.LINES,this._contourCounts[o][l],this._contourOffsets[o][l]))}A.unbind()}},D.pick=function(t){if(!t)return null;if(t.id!==this.pickId)return null;var e=this._field[2].shape,n=this._pickResult,r=e[0]*(t.value[0]+(t.value[2]>>4)/16)/255,i=Math.floor(r),o=r-i,a=e[1]*(t.value[1]+(15&t.value[2])/16)/255,s=Math.floor(a),A=a-s;i+=1,s+=1;var l=n.position;l[0]=l[1]=l[2]=0;for(var c=0;c<2;++c)for(var u=c?o:1-o,g=0;g<2;++g)for(var h=i+c,d=s+g,p=u*(g?A:1-A),I=0;I<3;++I)l[I]+=this._field[I].get(h,d)*p;for(var C=this._pickResult.level,m=0;m<3;++m)if(C[m]=f.le(this.contourLevels[m],l[m]),C[m]<0)this.contourLevels[m].length>0&&(C[m]=0);else if(C[m]<this.contourLevels[m].length-1){var y=this.contourLevels[m][C[m]],E=this.contourLevels[m][C[m]+1];Math.abs(y-l[m])>Math.abs(E-l[m])&&(C[m]+=1)}for(n.index[0]=o<.5?i:i+1,n.index[1]=A<.5?s:s+1,n.uv[0]=r/e[0],n.uv[1]=a/e[1],I=0;I<3;++I)n.dataCoordinate[I]=this._field[I].get(n.index[0],n.index[1]);return n},D.padField=function(t,e){var n=e.shape.slice(),r=t.shape.slice();l.assign(t.lo(1,1).hi(n[0],n[1]),e),l.assign(t.lo(1).hi(n[0],1),e.hi(n[0],1)),l.assign(t.lo(1,r[1]-1).hi(n[0],1),e.lo(0,n[1]-1).hi(n[0],1)),l.assign(t.lo(0,1).hi(1,n[1]),e.hi(1)),l.assign(t.lo(r[0]-1,1).hi(1,n[1]),e.lo(n[0]-1)),t.set(0,0,e.get(0,0)),t.set(0,r[1]-1,e.get(0,n[1]-1)),t.set(r[0]-1,0,e.get(n[0]-1,0)),t.set(r[0]-1,r[1]-1,e.get(n[0]-1,n[1]-1))},D.update=function(t){t=t||{},this.objectOffset=t.objectOffset||this.objectOffset,this.dirty=!0,"contourWidth"in t&&(this.contourWidth=U(t.contourWidth,Number)),"showContour"in t&&(this.showContour=U(t.showContour,Boolean)),"showSurface"in t&&(this.showSurface=!!t.showSurface),"contourTint"in t&&(this.contourTint=U(t.contourTint,Boolean)),"contourColor"in t&&(this.contourColor=G(t.contourColor)),"contourProject"in t&&(this.contourProject=U(t.contourProject,function(t){return U(t,Boolean)})),"surfaceProject"in t&&(this.surfaceProject=t.surfaceProject),"dynamicColor"in t&&(this.dynamicColor=G(t.dynamicColor)),"dynamicTint"in t&&(this.dynamicTint=U(t.dynamicTint,Number)),"dynamicWidth"in t&&(this.dynamicWidth=U(t.dynamicWidth,Number)),"opacity"in t&&(this.opacity=t.opacity),"colorBounds"in t&&(this.colorBounds=t.colorBounds),"vertexColor"in t&&(this.vertexColor=t.vertexColor?1:0);var e=t.field||t.coords&&t.coords[2]||null,n=!1;if(e||(e=this._field[2].shape[0]||this._field[2].shape[2]?this._field[2].lo(1,1).hi(this._field[2].shape[0]-2,this._field[2].shape[1]-2):this._field[2].hi(0,0)),"field"in t||"coords"in t){var i=(e.shape[0]+2)*(e.shape[1]+2);i>this._field[2].data.length&&(s.freeFloat(this._field[2].data),this._field[2].data=s.mallocFloat(r.nextPow2(i))),this._field[2]=u(this._field[2].data,[e.shape[0]+2,e.shape[1]+2]),this.padField(this._field[2],e),this.shape=e.shape.slice();for(var o=this.shape,a=0;a<2;++a)this._field[2].size>this._field[a].data.length&&(s.freeFloat(this._field[a].data),this._field[a].data=s.mallocFloat(this._field[2].size)),this._field[a]=u(this._field[a].data,[o[0]+2,o[1]+2]);if(t.coords){var h=t.coords;if(!Array.isArray(h)||3!==h.length)throw new Error("gl-surface: invalid coordinates for x/y");for(a=0;a<2;++a){var d=h[a];for(y=0;y<2;++y)if(d.shape[y]!==o[y])throw new Error("gl-surface: coords have incorrect shape");this.padField(this._field[a],d)}}else if(t.ticks){var f=t.ticks;if(!Array.isArray(f)||2!==f.length)throw new Error("gl-surface: invalid ticks");for(a=0;a<2;++a){var I=f[a];if((Array.isArray(I)||I.length)&&(I=u(I)),I.shape[0]!==o[a])throw new Error("gl-surface: invalid tick length");var C=u(I.data,o);C.stride[a]=I.stride[0],C.stride[1^a]=0,this.padField(this._field[a],C)}}else{for(a=0;a<2;++a){var m=[0,0];m[a]=1,this._field[a]=u(this._field[a].data,[o[0]+2,o[1]+2],m,0)}this._field[0].set(0,0,0);for(var y=0;y<o[0];++y)this._field[0].set(y+1,0,y);for(this._field[0].set(o[0]+1,0,o[0]-1),this._field[1].set(0,0,0),y=0;y<o[1];++y)this._field[1].set(0,y+1,y);this._field[1].set(0,o[1]+1,o[1]-1)}var E=this._field,v=u(s.mallocFloat(3*E[2].size*2),[3,o[0]+2,o[1]+2,2]);for(a=0;a<3;++a)p(v.pick(a),E[a],"mirror");var B=u(s.mallocFloat(3*E[2].size),[o[0]+2,o[1]+2,3]);for(a=0;a<o[0]+2;++a)for(y=0;y<o[1]+2;++y){var Q=v.get(0,a,y,0),w=v.get(0,a,y,1),_=v.get(1,a,y,0),D=v.get(1,a,y,1),M=v.get(2,a,y,0),k=v.get(2,a,y,1),S=_*k-D*M,L=M*w-k*Q,R=Q*D-w*_,N=Math.sqrt(S*S+L*L+R*R);N<1e-8?(N=Math.max(Math.abs(S),Math.abs(L),Math.abs(R)))<1e-8?(R=1,L=S=0,N=1):N=1/N:N=1/Math.sqrt(N),B.set(a,y,0,S*N),B.set(a,y,1,L*N),B.set(a,y,2,R*N)}s.free(v.data);var F=[1/0,1/0,1/0],T=[-1/0,-1/0,-1/0],O=1/0,H=-1/0,j=(o[0]-1)*(o[1]-1)*6,Y=s.mallocFloat(r.nextPow2(10*j)),P=0,K=0;for(a=0;a<o[0]-1;++a)t:for(y=0;y<o[1]-1;++y){for(var z=0;z<2;++z)for(var J=0;J<2;++J)for(var q=0;q<3;++q){var V=this._field[q].get(1+a+z,1+y+J);if(isNaN(V)||!isFinite(V))continue t}for(q=0;q<6;++q){var W=a+b[q][0],X=y+b[q][1],Z=this._field[0].get(W+1,X+1),$=this._field[1].get(W+1,X+1);V=this._field[2].get(W+1,X+1),S=B.get(W+1,X+1,0),L=B.get(W+1,X+1,1),R=B.get(W+1,X+1,2),t.intensity&&(tt=t.intensity.get(W,X));var tt=t.intensity?t.intensity.get(W,X):V+this.objectOffset[2];Y[P++]=W,Y[P++]=X,Y[P++]=Z,Y[P++]=$,Y[P++]=V,Y[P++]=0,Y[P++]=tt,Y[P++]=S,Y[P++]=L,Y[P++]=R,F[0]=Math.min(F[0],Z+this.objectOffset[0]),F[1]=Math.min(F[1],$+this.objectOffset[1]),F[2]=Math.min(F[2],V+this.objectOffset[2]),O=Math.min(O,tt),T[0]=Math.max(T[0],Z+this.objectOffset[0]),T[1]=Math.max(T[1],$+this.objectOffset[1]),T[2]=Math.max(T[2],V+this.objectOffset[2]),H=Math.max(H,tt),K+=1}}for(t.intensityBounds&&(O=+t.intensityBounds[0],H=+t.intensityBounds[1]),a=6;a<P;a+=10)Y[a]=(Y[a]-O)/(H-O);this._vertexCount=K,this._coordinateBuffer.update(Y.subarray(0,P)),s.freeFloat(Y),s.free(B.data),this.bounds=[F,T],this.intensity=t.intensity||this._field[2],this.intensityBounds[0]===O&&this.intensityBounds[1]===H||(n=!0),this.intensityBounds=[O,H]}if("levels"in t){var et=t.levels;for(et=Array.isArray(et[0])?et.slice():[[],[],et],a=0;a<3;++a)et[a]=et[a].slice(),et[a].sort(function(t,e){return t-e});for(a=0;a<3;++a)for(y=0;y<et[a].length;++y)et[a][y]-=this.objectOffset[a];t:for(a=0;a<3;++a){if(et[a].length!==this.contourLevels[a].length){n=!0;break}for(y=0;y<et[a].length;++y)if(et[a][y]!==this.contourLevels[a][y]){n=!0;break t}}this.contourLevels=et}if(n){E=this._field,o=this.shape;for(var nt=[],rt=0;rt<3;++rt){var it=this.contourLevels[rt],ot=[],at=[],st=[0,0,0];for(a=0;a<it.length;++a){var At=g(this._field[rt],it[a]);ot.push(nt.length/5|0),K=0;t:for(y=0;y<At.cells.length;++y){var lt=At.cells[y];for(q=0;q<2;++q){var ct=At.positions[lt[q]],ut=ct[0],gt=0|Math.floor(ut),ht=ut-gt,dt=ct[1],ft=0|Math.floor(dt),pt=dt-ft,It=!1;e:for(var Ct=0;Ct<3;++Ct){st[Ct]=0;var mt=(rt+Ct+1)%3;for(z=0;z<2;++z){var yt=z?ht:1-ht;for(W=0|Math.min(Math.max(gt+z,0),o[0]),J=0;J<2;++J){var Et=J?pt:1-pt;if(X=0|Math.min(Math.max(ft+J,0),o[1]),V=Ct<2?this._field[mt].get(W,X):(this.intensity.get(W,X)-this.intensityBounds[0])/(this.intensityBounds[1]-this.intensityBounds[0]),!isFinite(V)||isNaN(V)){It=!0;break e}var vt=yt*Et;st[Ct]+=vt*V}}}if(It){if(q>0){for(var Bt=0;Bt<5;++Bt)nt.pop();K-=1}continue t}nt.push(st[0],st[1],ct[0],ct[1],st[2]),K+=1}}at.push(K)}this._contourOffsets[rt]=ot,this._contourCounts[rt]=at}var bt=s.mallocFloat(nt.length);for(a=0;a<nt.length;++a)bt[a]=nt[a];this._contourBuffer.update(bt),s.freeFloat(bt)}t.colormap&&this._colorMap.setPixels(function(t){var e=c([A({colormap:t,nshades:x,format:"rgba"}).map(function(t){return[t[0],t[1],t[2],255*t[3]]})]);return l.divseq(e,255),e}(t.colormap))},D.dispose=function(){this._shader.dispose(),this._vao.dispose(),this._coordinateBuffer.dispose(),this._colorMap.dispose(),this._contourBuffer.dispose(),this._contourVAO.dispose(),this._contourShader.dispose(),this._contourPickShader.dispose(),this._dynamicBuffer.dispose(),this._dynamicVAO.dispose();for(var t=0;t<3;++t)s.freeFloat(this._field[t].data)},D.highlight=function(t){var e,n;if(!t)return this._dynamicCounts=[0,0,0],this.dyanamicLevel=[NaN,NaN,NaN],void(this.highlightLevel=[-1,-1,-1]);for(e=0;e<3;++e)this.enableHighlight[e]?this.highlightLevel[e]=t.level[e]:this.highlightLevel[e]=-1;for(n=this.snapToData?t.dataCoordinate:t.position,e=0;e<3;++e)n[e]-=this.objectOffset[e];if(this.enableDynamic[0]&&n[0]!==this.dynamicLevel[0]||this.enableDynamic[1]&&n[1]!==this.dynamicLevel[1]||this.enableDynamic[2]&&n[2]!==this.dynamicLevel[2]){for(var r=0,i=this.shape,o=s.mallocFloat(12*i[0]*i[1]),a=0;a<3;++a)if(this.enableDynamic[a]){this.dynamicLevel[a]=n[a];var A=(a+1)%3,l=(a+2)%3,c=this._field[a],u=this._field[A],h=this._field[l],d=g(c,n[a]),f=d.cells,p=d.positions;for(this._dynamicOffsets[a]=r,e=0;e<f.length;++e)for(var I=f[e],C=0;C<2;++C){var m=p[I[C]],y=+m[0],E=0|y,v=0|Math.min(E+1,i[0]),B=y-E,b=1-B,Q=+m[1],w=0|Q,x=0|Math.min(w+1,i[1]),_=Q-w,D=1-_,M=b*D,k=b*_,S=B*D,L=B*_,R=M*u.get(E,w)+k*u.get(E,x)+S*u.get(v,w)+L*u.get(v,x),N=M*h.get(E,w)+k*h.get(E,x)+S*h.get(v,w)+L*h.get(v,x);if(isNaN(R)||isNaN(N)){C&&(r-=1);break}o[2*r+0]=R,o[2*r+1]=N,r+=1}this._dynamicCounts[a]=r-this._dynamicOffsets[a]}else this.dynamicLevel[a]=NaN,this._dynamicCounts[a]=0;this._dynamicBuffer.update(o.subarray(0,2*r)),s.freeFloat(o)}}},{"./lib/shaders":314,"binary-search-bounds":315,"bit-twiddle":92,colormap:126,"gl-buffer":241,"gl-mat4/invert":265,"gl-mat4/multiply":267,"gl-texture2d":322,"gl-vao":327,ndarray:450,"ndarray-gradient":441,"ndarray-ops":444,"ndarray-pack":445,"surface-nets":531,"typedarray-pool":545}],317:[function(t,e,n){"use strict";var r=t("css-font"),i=t("pick-by-alias"),o=t("regl"),a=t("gl-util/context"),s=t("es6-weak-map"),A=t("color-normalize"),l=t("font-atlas"),c=t("typedarray-pool"),u=t("parse-rect"),g=t("is-plain-obj"),h=t("parse-unit"),d=t("to-px"),f=t("detect-kerning"),p=t("object-assign"),I=t("font-measure"),C=t("flatten-vertex-data"),m=t("bit-twiddle").nextPow2,y=new s,E=!1;if(document.body){var v=document.body.appendChild(document.createElement("div"));v.style.font="italic small-caps bold condensed 16px/2 cursive",getComputedStyle(v).fontStretch&&(E=!0),document.body.removeChild(v)}var B=function(t){!function(t){return"function"==typeof t&&t._gl&&t.prop&&t.texture&&t.buffer}(t)?this.gl=a(t):(t={regl:t},this.gl=t.regl._gl),this.shader=y.get(this.gl),this.shader?this.regl=this.shader.regl:this.regl=t.regl||o({gl:this.gl}),this.charBuffer=this.regl.buffer({type:"uint8",usage:"stream"}),this.sizeBuffer=this.regl.buffer({type:"float",usage:"stream"}),this.shader||(this.shader=this.createShader(),y.set(this.gl,this.shader)),this.batch=[],this.fontSize=[],this.font=[],this.fontAtlas=[],this.draw=this.shader.draw.bind(this),this.render=function(){this.regl._refresh(),this.draw(this.batch)},this.canvas=this.gl.canvas,this.update(g(t)?t:{})};B.prototype.createShader=function(){var t=this.regl,e=t({blend:{enable:!0,color:[0,0,0,1],func:{srcRGB:"src alpha",dstRGB:"one minus src alpha",srcAlpha:"one minus dst alpha",dstAlpha:"one"}},stencil:{enable:!1},depth:{enable:!1},count:t.prop("count"),offset:t.prop("offset"),attributes:{charOffset:{offset:4,stride:8,buffer:t.this("sizeBuffer")},width:{offset:0,stride:8,buffer:t.this("sizeBuffer")},char:t.this("charBuffer"),position:t.this("position")},uniforms:{atlasSize:function(t,e){return[e.atlas.width,e.atlas.height]},atlasDim:function(t,e){return[e.atlas.cols,e.atlas.rows]},atlas:function(t,e){return e.atlas.texture},charStep:function(t,e){return e.atlas.step},em:function(t,e){return e.atlas.em},color:t.prop("color"),opacity:t.prop("opacity"),viewport:t.this("viewportArray"),scale:t.this("scale"),align:t.prop("align"),baseline:t.prop("baseline"),translate:t.this("translate"),positionOffset:t.prop("positionOffset")},primitive:"points",viewport:t.this("viewport"),vert:"\n\t\t\tprecision highp float;\n\t\t\tattribute float width, charOffset, char;\n\t\t\tattribute vec2 position;\n\t\t\tuniform float fontSize, charStep, em, align, baseline;\n\t\t\tuniform vec4 viewport;\n\t\t\tuniform vec4 color;\n\t\t\tuniform vec2 atlasSize, atlasDim, scale, translate, positionOffset;\n\t\t\tvarying vec2 charCoord, charId;\n\t\t\tvarying float charWidth;\n\t\t\tvarying vec4 fontColor;\n\t\t\tvoid main () {\n\t\t\t\t"+(B.normalViewport?"":"vec2 positionOffset = vec2(positionOffset.x,- positionOffset.y);")+"\n\n\t\t\t\tvec2 offset = floor(em * (vec2(align + charOffset, baseline)\n\t\t\t\t\t+ positionOffset))\n\t\t\t\t\t/ (viewport.zw * scale.xy);\n\n\t\t\t\tvec2 position = (position + translate) * scale;\n\t\t\t\tposition += offset * scale;\n\n\t\t\t\t"+(B.normalViewport?"position.y = 1. - position.y;":"")+"\n\n\t\t\t\tcharCoord = position * viewport.zw + viewport.xy;\n\n\t\t\t\tgl_Position = vec4(position * 2. - 1., 0, 1);\n\n\t\t\t\tgl_PointSize = charStep;\n\n\t\t\t\tcharId.x = mod(char, atlasDim.x);\n\t\t\t\tcharId.y = floor(char / atlasDim.x);\n\n\t\t\t\tcharWidth = width * em;\n\n\t\t\t\tfontColor = color / 255.;\n\t\t\t}",frag:"\n\t\t\tprecision highp float;\n\t\t\tuniform sampler2D atlas;\n\t\t\tuniform float fontSize, charStep, opacity;\n\t\t\tuniform vec2 atlasSize;\n\t\t\tuniform vec4 viewport;\n\t\t\tvarying vec4 fontColor;\n\t\t\tvarying vec2 charCoord, charId;\n\t\t\tvarying float charWidth;\n\n\t\t\tfloat lightness(vec4 color) {\n\t\t\t\treturn color.r * 0.299 + color.g * 0.587 + color.b * 0.114;\n\t\t\t}\n\n\t\t\tvoid main () {\n\t\t\t\tvec2 uv = gl_FragCoord.xy - charCoord + charStep * .5;\n\t\t\t\tfloat halfCharStep = floor(charStep * .5 + .5);\n\n\t\t\t\t// invert y and shift by 1px (FF expecially needs that)\n\t\t\t\tuv.y = charStep - uv.y;\n\n\t\t\t\t// ignore points outside of character bounding box\n\t\t\t\tfloat halfCharWidth = ceil(charWidth * .5);\n\t\t\t\tif (floor(uv.x) > halfCharStep + halfCharWidth ||\n\t\t\t\t\tfloor(uv.x) < halfCharStep - halfCharWidth) return;\n\n\t\t\t\tuv += charId * charStep;\n\t\t\t\tuv = uv / atlasSize;\n\n\t\t\t\tvec4 color = fontColor;\n\t\t\t\tvec4 mask = texture2D(atlas, uv);\n\n\t\t\t\tfloat maskY = lightness(mask);\n\t\t\t\t// float colorY = lightness(color);\n\t\t\t\tcolor.a *= maskY;\n\t\t\t\tcolor.a *= opacity;\n\n\t\t\t\t// color.a += .1;\n\n\t\t\t\t// antialiasing, see yiq color space y-channel formula\n\t\t\t\t// color.rgb += (1. - color.rgb) * (1. - mask.rgb);\n\n\t\t\t\tgl_FragColor = color;\n\t\t\t}"});return{regl:t,draw:e,atlas:{}}},B.prototype.update=function(t){var e=this;if("string"==typeof t)t={text:t};else if(!t)return;null!=(t=i(t,{position:"position positions coord coords coordinates",font:"font fontFace fontface typeface cssFont css-font family fontFamily",fontSize:"fontSize fontsize size font-size",text:"text texts chars characters value values symbols",align:"align alignment textAlign textbaseline",baseline:"baseline textBaseline textbaseline",direction:"dir direction textDirection",color:"color colour fill fill-color fillColor textColor textcolor",kerning:"kerning kern",range:"range dataBox",viewport:"vp viewport viewBox viewbox viewPort",opacity:"opacity alpha transparency visible visibility opaque",offset:"offset positionOffset padding shift indent indentation"},!0)).opacity&&(Array.isArray(t.opacity)?this.opacity=t.opacity.map(function(t){return parseFloat(t)}):this.opacity=parseFloat(t.opacity)),null!=t.viewport&&(this.viewport=u(t.viewport),B.normalViewport&&(this.viewport.y=this.canvas.height-this.viewport.y-this.viewport.height),this.viewportArray=[this.viewport.x,this.viewport.y,this.viewport.width,this.viewport.height]),null==this.viewport&&(this.viewport={x:0,y:0,width:this.gl.drawingBufferWidth,height:this.gl.drawingBufferHeight},this.viewportArray=[this.viewport.x,this.viewport.y,this.viewport.width,this.viewport.height]),null!=t.kerning&&(this.kerning=t.kerning),null!=t.offset&&("number"==typeof t.offset&&(t.offset=[t.offset,0]),this.positionOffset=C(t.offset)),t.direction&&(this.direction=t.direction),t.range&&(this.range=t.range,this.scale=[1/(t.range[2]-t.range[0]),1/(t.range[3]-t.range[1])],this.translate=[-t.range[0],-t.range[1]]),t.scale&&(this.scale=t.scale),t.translate&&(this.translate=t.translate),this.scale||(this.scale=[1/this.viewport.width,1/this.viewport.height]),this.translate||(this.translate=[0,0]),this.font.length||t.font||(t.font=B.baseFontSize+"px sans-serif");var n,o=!1,a=!1;if(t.font&&(Array.isArray(t.font)?t.font:[t.font]).forEach(function(t,n){if("string"==typeof t)try{t=r.parse(t)}catch(e){t=r.parse(B.baseFontSize+"px "+t)}else t=r.parse(r.stringify(t));var i=r.stringify({size:B.baseFontSize,family:t.family,stretch:E?t.stretch:void 0,variant:t.variant,weight:t.weight,style:t.style}),s=h(t.size),A=Math.round(s[0]*d(s[1]));if(A!==e.fontSize[n]&&(a=!0,e.fontSize[n]=A),!(e.font[n]&&i==e.font[n].baseString||(o=!0,e.font[n]=B.fonts[i],e.font[n]))){var l=t.family.join(", "),c=[t.style];t.style!=t.variant&&c.push(t.variant),t.variant!=t.weight&&c.push(t.weight),E&&t.weight!=t.stretch&&c.push(t.stretch),e.font[n]={baseString:i,family:l,weight:t.weight,stretch:t.stretch,style:t.style,variant:t.variant,width:{},kerning:{},metrics:I(l,{origin:"top",fontSize:B.baseFontSize,fontStyle:c.join(" ")})},B.fonts[i]=e.font[n]}}),(o||a)&&this.font.forEach(function(n,i){var o=r.stringify({size:e.fontSize[i],family:n.family,stretch:E?n.stretch:void 0,variant:n.variant,weight:n.weight,style:n.style});if(e.fontAtlas[i]=e.shader.atlas[o],!e.fontAtlas[i]){var a=n.metrics;e.shader.atlas[o]=e.fontAtlas[i]={fontString:o,step:2*Math.ceil(e.fontSize[i]*a.bottom*.5),em:e.fontSize[i],cols:0,rows:0,height:0,width:0,chars:[],ids:{},texture:e.regl.texture()}}null==t.text&&(t.text=e.text)}),"string"==typeof t.text&&t.position&&t.position.length>2){for(var s=Array(.5*t.position.length),g=0;g<s.length;g++)s[g]=t.text;t.text=s}if(null!=t.text||o){if(this.textOffsets=[0],Array.isArray(t.text)){this.count=t.text[0].length,this.counts=[this.count];for(var y=1;y<t.text.length;y++)this.textOffsets[y]=this.textOffsets[y-1]+t.text[y-1].length,this.count+=t.text[y].length,this.counts.push(t.text[y].length);this.text=t.text.join("")}else this.text=t.text,this.count=this.text.length,this.counts=[this.count];n=[],this.font.forEach(function(t,r){B.atlasContext.font=t.baseString;for(var i=e.fontAtlas[r],o=0;o<e.text.length;o++){var a=e.text.charAt(o);if(null==i.ids[a]&&(i.ids[a]=i.chars.length,i.chars.push(a),n.push(a)),null==t.width[a]&&(t.width[a]=B.atlasContext.measureText(a).width/B.baseFontSize,e.kerning)){var s=[];for(var A in t.width)s.push(A+a,a+A);p(t.kerning,f(t.family,{pairs:s}))}}})}if(t.position)if(t.position.length>2){for(var v=!t.position[0].length,b=c.mallocFloat(2*this.count),Q=0,w=0;Q<this.counts.length;Q++){var x=this.counts[Q];if(v)for(var _=0;_<x;_++)b[w++]=t.position[2*Q],b[w++]=t.position[2*Q+1];else for(var D=0;D<x;D++)b[w++]=t.position[Q][0],b[w++]=t.position[Q][1]}this.position.call?this.position({type:"float",data:b}):this.position=this.regl.buffer({type:"float",data:b}),c.freeFloat(b)}else this.position.destroy&&this.position.destroy(),this.position={constant:t.position};if(t.text||o){var M=c.mallocUint8(this.count),k=c.mallocFloat(2*this.count);this.textWidth=[];for(var S=0,L=0;S<this.counts.length;S++){for(var R=this.counts[S],N=this.font[S]||this.font[0],F=this.fontAtlas[S]||this.fontAtlas[0],T=0;T<R;T++){var U=this.text.charAt(L),O=this.text.charAt(L-1);if(M[L]=F.ids[U],k[2*L]=N.width[U],T){var G=k[2*L-2],H=k[2*L],j=k[2*L-1]+.5*G+.5*H;if(this.kerning){var Y=N.kerning[O+U];Y&&(j+=.001*Y)}k[2*L+1]=j}else k[2*L+1]=.5*k[2*L];L++}this.textWidth.push(k.length?.5*k[2*L-2]+k[2*L-1]:0)}t.align||(t.align=this.align),this.charBuffer({data:M,type:"uint8",usage:"stream"}),this.sizeBuffer({data:k,type:"float",usage:"stream"}),c.freeUint8(M),c.freeFloat(k),n.length&&this.font.forEach(function(t,n){var r=e.fontAtlas[n],i=r.step,o=Math.floor(B.maxAtlasSize/i),a=Math.min(o,r.chars.length),s=Math.ceil(r.chars.length/a),A=m(a*i),c=m(s*i);r.width=A,r.height=c,r.rows=s,r.cols=a,r.em&&r.texture({data:l({canvas:B.atlasCanvas,font:r.fontString,chars:r.chars,shape:[A,c],step:[i,i]})})})}if(t.align&&(this.align=t.align,this.alignOffset=this.textWidth.map(function(t,n){var r=Array.isArray(e.align)?e.align.length>1?e.align[n]:e.align[0]:e.align;if("number"==typeof r)return r;switch(r){case"right":case"end":return-t;case"center":case"centre":case"middle":return.5*-t}return 0})),null==this.baseline&&null==t.baseline&&(t.baseline=0),null!=t.baseline&&(this.baseline=t.baseline,Array.isArray(this.baseline)||(this.baseline=[this.baseline]),this.baselineOffset=this.baseline.map(function(t,n){var r=(e.font[n]||e.font[0]).metrics,i=0;return i+=.5*r.bottom,i+="number"==typeof t?t-r.baseline:-r[t],B.normalViewport||(i*=-1),i})),null!=t.color)if(t.color||(t.color="transparent"),"string"!=typeof t.color&&isNaN(t.color)){var P;if("number"==typeof t.color[0]&&t.color.length>this.counts.length){var K=t.color.length;P=c.mallocUint8(K);for(var z=(t.color.subarray||t.color.slice).bind(t.color),J=0;J<K;J+=4)P.set(A(z(J,J+4),"uint8"),J)}else{var q=t.color.length;P=c.mallocUint8(4*q);for(var V=0;V<q;V++)P.set(A(t.color[V]||0,"uint8"),4*V)}this.color=P}else this.color=A(t.color,"uint8");if(t.position||t.text||t.color||t.baseline||t.align||t.font||t.offset||t.opacity)if(this.color.length>4||this.baselineOffset.length>1||this.align&&this.align.length>1||this.fontAtlas.length>1||this.positionOffset.length>2){var W=Math.max(.5*this.position.length||0,.25*this.color.length||0,this.baselineOffset.length||0,this.alignOffset.length||0,this.font.length||0,this.opacity.length||0,.5*this.positionOffset.length||0);this.batch=Array(W);for(var X=0;X<this.batch.length;X++)this.batch[X]={count:this.counts.length>1?this.counts[X]:this.counts[0],offset:this.textOffsets.length>1?this.textOffsets[X]:this.textOffsets[0],color:this.color?this.color.length<=4?this.color:this.color.subarray(4*X,4*X+4):[0,0,0,255],opacity:Array.isArray(this.opacity)?this.opacity[X]:this.opacity,baseline:null!=this.baselineOffset[X]?this.baselineOffset[X]:this.baselineOffset[0],align:this.align?null!=this.alignOffset[X]?this.alignOffset[X]:this.alignOffset[0]:0,atlas:this.fontAtlas[X]||this.fontAtlas[0],positionOffset:this.positionOffset.length>2?this.positionOffset.subarray(2*X,2*X+2):this.positionOffset}}else this.count?this.batch=[{count:this.count,offset:0,color:this.color||[0,0,0,255],opacity:Array.isArray(this.opacity)?this.opacity[0]:this.opacity,baseline:this.baselineOffset[0],align:this.alignOffset?this.alignOffset[0]:0,atlas:this.fontAtlas[0],positionOffset:this.positionOffset}]:this.batch=[]},B.prototype.destroy=function(){},B.prototype.kerning=!0,B.prototype.position={constant:new Float32Array(2)},B.prototype.translate=null,B.prototype.scale=null,B.prototype.font=null,B.prototype.text="",B.prototype.positionOffset=[0,0],B.prototype.opacity=1,B.prototype.color=new Uint8Array([0,0,0,255]),B.prototype.alignOffset=[0,0],B.normalViewport=!1,B.maxAtlasSize=1024,B.atlasCanvas=document.createElement("canvas"),B.atlasContext=B.atlasCanvas.getContext("2d",{alpha:!1}),B.baseFontSize=64,B.fonts={},e.exports=B},{"bit-twiddle":92,"color-normalize":120,"css-font":139,"detect-kerning":166,"es6-weak-map":318,"flatten-vertex-data":227,"font-atlas":228,"font-measure":229,"gl-util/context":323,"is-plain-obj":422,"object-assign":454,"parse-rect":459,"parse-unit":461,"pick-by-alias":465,regl:502,"to-px":539,"typedarray-pool":545}],318:[function(t,e,n){"use strict";e.exports=t("./is-implemented")()?WeakMap:t("./polyfill")},{"./is-implemented":319,"./polyfill":321}],319:[function(t,e,n){"use strict";e.exports=function(){var t,e;if("function"!=typeof WeakMap)return!1;try{t=new WeakMap([[e={},"one"],[{},"two"],[{},"three"]])}catch(t){return!1}return"[object WeakMap]"===String(t)&&"function"==typeof t.set&&t.set({},1)===t&&"function"==typeof t.delete&&"function"==typeof t.has&&"one"===t.get(e)}},{}],320:[function(t,e,n){"use strict";e.exports="function"==typeof WeakMap&&"[object WeakMap]"===Object.prototype.toString.call(new WeakMap)},{}],321:[function(t,e,n){"use strict";var r,i=t("es5-ext/object/is-value"),o=t("es5-ext/object/set-prototype-of"),a=t("es5-ext/object/valid-object"),s=t("es5-ext/object/valid-value"),A=t("es5-ext/string/random-uniq"),l=t("d"),c=t("es6-iterator/get"),u=t("es6-iterator/for-of"),g=t("es6-symbol").toStringTag,h=t("./is-native-implemented"),d=Array.isArray,f=Object.defineProperty,p=Object.prototype.hasOwnProperty,I=Object.getPrototypeOf;e.exports=r=function(){var t,e=arguments[0];if(!(this instanceof r))throw new TypeError("Constructor requires 'new'");return t=h&&o&&WeakMap!==r?o(new WeakMap,I(this)):this,i(e)&&(d(e)||(e=c(e))),f(t,"__weakMapData__",l("c","$weakMap$"+A())),e?(u(e,function(e){s(e),t.set(e[0],e[1])}),t):t},h&&(o&&o(r,WeakMap),r.prototype=Object.create(WeakMap.prototype,{constructor:l(r)})),Object.defineProperties(r.prototype,{delete:l(function(t){return!!p.call(a(t),this.__weakMapData__)&&(delete t[this.__weakMapData__],!0)}),get:l(function(t){if(p.call(a(t),this.__weakMapData__))return t[this.__weakMapData__]}),has:l(function(t){return p.call(a(t),this.__weakMapData__)}),set:l(function(t,e){return f(a(t),this.__weakMapData__,l("c",e)),this}),toString:l(function(){return"[object WeakMap]"})}),f(r.prototype,g,l("c","WeakMap"))},{"./is-native-implemented":320,d:151,"es5-ext/object/is-value":194,"es5-ext/object/set-prototype-of":200,"es5-ext/object/valid-object":204,"es5-ext/object/valid-value":205,"es5-ext/string/random-uniq":210,"es6-iterator/for-of":212,"es6-iterator/get":213,"es6-symbol":219}],322:[function(t,e,n){"use strict";var r=t("ndarray"),i=t("ndarray-ops"),o=t("typedarray-pool");e.exports=function(t){if(arguments.length<=1)throw new Error("gl-texture2d: Missing arguments for texture2d constructor");if(a||function(t){a=[t.LINEAR,t.NEAREST_MIPMAP_LINEAR,t.LINEAR_MIPMAP_NEAREST,t.LINEAR_MIPMAP_NEAREST],s=[t.NEAREST,t.LINEAR,t.NEAREST_MIPMAP_NEAREST,t.NEAREST_MIPMAP_LINEAR,t.LINEAR_MIPMAP_NEAREST,t.LINEAR_MIPMAP_LINEAR],A=[t.REPEAT,t.CLAMP_TO_EDGE,t.MIRRORED_REPEAT]}(t),"number"==typeof arguments[1])return p(t,arguments[1],arguments[2],arguments[3]||t.RGBA,arguments[4]||t.UNSIGNED_BYTE);if(Array.isArray(arguments[1]))return p(t,0|arguments[1][0],0|arguments[1][1],arguments[2]||t.RGBA,arguments[3]||t.UNSIGNED_BYTE);if("object"==typeof arguments[1]){var e=arguments[1],n=l(e)?e:e.raw;if(n)return function(t,e,n,r,i,o){var a=f(t);return t.texImage2D(t.TEXTURE_2D,0,i,i,o,e),new g(t,a,n,r,i,o)}(t,n,0|e.width,0|e.height,arguments[2]||t.RGBA,arguments[3]||t.UNSIGNED_BYTE);if(e.shape&&e.data&&e.stride)return function(t,e){var n=e.dtype,a=e.shape.slice(),s=t.getParameter(t.MAX_TEXTURE_SIZE);if(a[0]<0||a[0]>s||a[1]<0||a[1]>s)throw new Error("gl-texture2d: Invalid texture size");var A=d(a,e.stride.slice()),l=0;"float32"===n?l=t.FLOAT:"float64"===n?(l=t.FLOAT,A=!1,n="float32"):"uint8"===n?l=t.UNSIGNED_BYTE:(l=t.UNSIGNED_BYTE,A=!1,n="uint8");var u,h,p=0;if(2===a.length)p=t.LUMINANCE,a=[a[0],a[1],1],e=r(e.data,a,[e.stride[0],e.stride[1],1],e.offset);else{if(3!==a.length)throw new Error("gl-texture2d: Invalid shape for texture");if(1===a[2])p=t.ALPHA;else if(2===a[2])p=t.LUMINANCE_ALPHA;else if(3===a[2])p=t.RGB;else{if(4!==a[2])throw new Error("gl-texture2d: Invalid shape for pixel coords");p=t.RGBA}}l!==t.FLOAT||t.getExtension("OES_texture_float")||(l=t.UNSIGNED_BYTE,A=!1);var I=e.size;if(A)u=0===e.offset&&e.data.length===I?e.data:e.data.subarray(e.offset,e.offset+I);else{var C=[a[2],a[2]*a[0],1];h=o.malloc(I,n);var m=r(h,a,C,0);"float32"!==n&&"float64"!==n||l!==t.UNSIGNED_BYTE?i.assign(m,e):c(m,e),u=h.subarray(0,I)}var y=f(t);return t.texImage2D(t.TEXTURE_2D,0,p,a[0],a[1],0,p,l,u),A||o.free(h),new g(t,y,a[0],a[1],p,l)}(t,e)}throw new Error("gl-texture2d: Invalid arguments for texture2d constructor")};var a=null,s=null,A=null;function l(t){return"undefined"!=typeof HTMLCanvasElement&&t instanceof HTMLCanvasElement||"undefined"!=typeof HTMLImageElement&&t instanceof HTMLImageElement||"undefined"!=typeof HTMLVideoElement&&t instanceof HTMLVideoElement||"undefined"!=typeof ImageData&&t instanceof ImageData}var c=function(t,e){i.muls(t,e,255)};function u(t,e,n){var r=t.gl,i=r.getParameter(r.MAX_TEXTURE_SIZE);if(e<0||e>i||n<0||n>i)throw new Error("gl-texture2d: Invalid texture size");return t._shape=[e,n],t.bind(),r.texImage2D(r.TEXTURE_2D,0,t.format,e,n,0,t.format,t.type,null),t._mipLevels=[0],t}function g(t,e,n,r,i,o){this.gl=t,this.handle=e,this.format=i,this.type=o,this._shape=[n,r],this._mipLevels=[0],this._magFilter=t.NEAREST,this._minFilter=t.NEAREST,this._wrapS=t.CLAMP_TO_EDGE,this._wrapT=t.CLAMP_TO_EDGE,this._anisoSamples=1;var a=this,s=[this._wrapS,this._wrapT];Object.defineProperties(s,[{get:function(){return a._wrapS},set:function(t){return a.wrapS=t}},{get:function(){return a._wrapT},set:function(t){return a.wrapT=t}}]),this._wrapVector=s;var A=[this._shape[0],this._shape[1]];Object.defineProperties(A,[{get:function(){return a._shape[0]},set:function(t){return a.width=t}},{get:function(){return a._shape[1]},set:function(t){return a.height=t}}]),this._shapeVector=A}var h=g.prototype;function d(t,e){return 3===t.length?1===e[2]&&e[1]===t[0]*t[2]&&e[0]===t[2]:1===e[0]&&e[1]===t[0]}function f(t){var e=t.createTexture();return t.bindTexture(t.TEXTURE_2D,e),t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MIN_FILTER,t.NEAREST),t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MAG_FILTER,t.NEAREST),t.texParameteri(t.TEXTURE_2D,t.TEXTURE_WRAP_S,t.CLAMP_TO_EDGE),t.texParameteri(t.TEXTURE_2D,t.TEXTURE_WRAP_T,t.CLAMP_TO_EDGE),e}function p(t,e,n,r,i){var o=t.getParameter(t.MAX_TEXTURE_SIZE);if(e<0||e>o||n<0||n>o)throw new Error("gl-texture2d: Invalid texture shape");if(i===t.FLOAT&&!t.getExtension("OES_texture_float"))throw new Error("gl-texture2d: Floating point textures not supported on this platform");var a=f(t);return t.texImage2D(t.TEXTURE_2D,0,r,e,n,0,r,i,null),new g(t,a,e,n,r,i)}Object.defineProperties(h,{minFilter:{get:function(){return this._minFilter},set:function(t){this.bind();var e=this.gl;if(this.type===e.FLOAT&&a.indexOf(t)>=0&&(e.getExtension("OES_texture_float_linear")||(t=e.NEAREST)),s.indexOf(t)<0)throw new Error("gl-texture2d: Unknown filter mode "+t);return e.texParameteri(e.TEXTURE_2D,e.TEXTURE_MIN_FILTER,t),this._minFilter=t}},magFilter:{get:function(){return this._magFilter},set:function(t){this.bind();var e=this.gl;if(this.type===e.FLOAT&&a.indexOf(t)>=0&&(e.getExtension("OES_texture_float_linear")||(t=e.NEAREST)),s.indexOf(t)<0)throw new Error("gl-texture2d: Unknown filter mode "+t);return e.texParameteri(e.TEXTURE_2D,e.TEXTURE_MAG_FILTER,t),this._magFilter=t}},mipSamples:{get:function(){return this._anisoSamples},set:function(t){var e=this._anisoSamples;if(this._anisoSamples=0|Math.max(t,1),e!==this._anisoSamples){var n=this.gl.getExtension("EXT_texture_filter_anisotropic");n&&this.gl.texParameterf(this.gl.TEXTURE_2D,n.TEXTURE_MAX_ANISOTROPY_EXT,this._anisoSamples)}return this._anisoSamples}},wrapS:{get:function(){return this._wrapS},set:function(t){if(this.bind(),A.indexOf(t)<0)throw new Error("gl-texture2d: Unknown wrap mode "+t);return this.gl.texParameteri(this.gl.TEXTURE_2D,this.gl.TEXTURE_WRAP_S,t),this._wrapS=t}},wrapT:{get:function(){return this._wrapT},set:function(t){if(this.bind(),A.indexOf(t)<0)throw new Error("gl-texture2d: Unknown wrap mode "+t);return this.gl.texParameteri(this.gl.TEXTURE_2D,this.gl.TEXTURE_WRAP_T,t),this._wrapT=t}},wrap:{get:function(){return this._wrapVector},set:function(t){if(Array.isArray(t)||(t=[t,t]),2!==t.length)throw new Error("gl-texture2d: Must specify wrap mode for rows and columns");for(var e=0;e<2;++e)if(A.indexOf(t[e])<0)throw new Error("gl-texture2d: Unknown wrap mode "+t);this._wrapS=t[0],this._wrapT=t[1];var n=this.gl;return this.bind(),n.texParameteri(n.TEXTURE_2D,n.TEXTURE_WRAP_S,this._wrapS),n.texParameteri(n.TEXTURE_2D,n.TEXTURE_WRAP_T,this._wrapT),t}},shape:{get:function(){return this._shapeVector},set:function(t){if(Array.isArray(t)){if(2!==t.length)throw new Error("gl-texture2d: Invalid texture shape")}else t=[0|t,0|t];return u(this,0|t[0],0|t[1]),[0|t[0],0|t[1]]}},width:{get:function(){return this._shape[0]},set:function(t){return u(this,t|=0,this._shape[1]),t}},height:{get:function(){return this._shape[1]},set:function(t){return t|=0,u(this,this._shape[0],t),t}}}),h.bind=function(t){var e=this.gl;return void 0!==t&&e.activeTexture(e.TEXTURE0+(0|t)),e.bindTexture(e.TEXTURE_2D,this.handle),void 0!==t?0|t:e.getParameter(e.ACTIVE_TEXTURE)-e.TEXTURE0},h.dispose=function(){this.gl.deleteTexture(this.handle)},h.generateMipmap=function(){this.bind(),this.gl.generateMipmap(this.gl.TEXTURE_2D);for(var t=Math.min(this._shape[0],this._shape[1]),e=0;t>0;++e,t>>>=1)this._mipLevels.indexOf(e)<0&&this._mipLevels.push(e)},h.setPixels=function(t,e,n,a){var s=this.gl;this.bind(),Array.isArray(e)?(a=n,n=0|e[1],e=0|e[0]):(e=e||0,n=n||0),a=a||0;var A=l(t)?t:t.raw;if(A)this._mipLevels.indexOf(a)<0?(s.texImage2D(s.TEXTURE_2D,0,this.format,this.format,this.type,A),this._mipLevels.push(a)):s.texSubImage2D(s.TEXTURE_2D,a,e,n,this.format,this.type,A);else{if(!(t.shape&&t.stride&&t.data))throw new Error("gl-texture2d: Unsupported data type");if(t.shape.length<2||e+t.shape[1]>this._shape[1]>>>a||n+t.shape[0]>this._shape[0]>>>a||e<0||n<0)throw new Error("gl-texture2d: Texture dimensions are out of bounds");!function(t,e,n,a,s,A,l,u){var g=u.dtype,h=u.shape.slice();if(h.length<2||h.length>3)throw new Error("gl-texture2d: Invalid ndarray, must be 2d or 3d");var f=0,p=0,I=d(h,u.stride.slice());if("float32"===g?f=t.FLOAT:"float64"===g?(f=t.FLOAT,I=!1,g="float32"):"uint8"===g?f=t.UNSIGNED_BYTE:(f=t.UNSIGNED_BYTE,I=!1,g="uint8"),2===h.length)p=t.LUMINANCE,h=[h[0],h[1],1],u=r(u.data,h,[u.stride[0],u.stride[1],1],u.offset);else{if(3!==h.length)throw new Error("gl-texture2d: Invalid shape for texture");if(1===h[2])p=t.ALPHA;else if(2===h[2])p=t.LUMINANCE_ALPHA;else if(3===h[2])p=t.RGB;else{if(4!==h[2])throw new Error("gl-texture2d: Invalid shape for pixel coords");p=t.RGBA}h[2]}if(p!==t.LUMINANCE&&p!==t.ALPHA||s!==t.LUMINANCE&&s!==t.ALPHA||(p=s),p!==s)throw new Error("gl-texture2d: Incompatible texture format for setPixels");var C=u.size,m=l.indexOf(a)<0;if(m&&l.push(a),f===A&&I)0===u.offset&&u.data.length===C?m?t.texImage2D(t.TEXTURE_2D,a,s,h[0],h[1],0,s,A,u.data):t.texSubImage2D(t.TEXTURE_2D,a,e,n,h[0],h[1],s,A,u.data):m?t.texImage2D(t.TEXTURE_2D,a,s,h[0],h[1],0,s,A,u.data.subarray(u.offset,u.offset+C)):t.texSubImage2D(t.TEXTURE_2D,a,e,n,h[0],h[1],s,A,u.data.subarray(u.offset,u.offset+C));else{var y;y=A===t.FLOAT?o.mallocFloat32(C):o.mallocUint8(C);var E=r(y,h,[h[2],h[2]*h[0],1]);f===t.FLOAT&&A===t.UNSIGNED_BYTE?c(E,u):i.assign(E,u),m?t.texImage2D(t.TEXTURE_2D,a,s,h[0],h[1],0,s,A,y.subarray(0,C)):t.texSubImage2D(t.TEXTURE_2D,a,e,n,h[0],h[1],s,A,y.subarray(0,C)),A===t.FLOAT?o.freeFloat32(y):o.freeUint8(y)}}(s,e,n,a,this.format,this.type,this._mipLevels,t)}}},{ndarray:450,"ndarray-ops":444,"typedarray-pool":545}],323:[function(t,n,r){(function(e){"use strict";var r=t("pick-by-alias");function i(t){if(t.container)if(t.container==document.body)document.body.style.width||(t.canvas.width=t.width||t.pixelRatio*e.innerWidth),document.body.style.height||(t.canvas.height=t.height||t.pixelRatio*e.innerHeight);else{var n=t.container.getBoundingClientRect();t.canvas.width=t.width||n.right-n.left,t.canvas.height=t.height||n.bottom-n.top}}function o(t){return"function"==typeof t.getContext&&"width"in t&&"height"in t}function a(){var t=document.createElement("canvas");return t.style.position="absolute",t.style.top=0,t.style.left=0,t}n.exports=function(t){var n;if(t?"string"==typeof t&&(t={container:t}):t={},(t=o(t)?{container:t}:"string"==typeof(n=t).nodeName&&"function"==typeof n.appendChild&&"function"==typeof n.getBoundingClientRect?{container:t}:function(t){return"function"==typeof t.drawArrays||"function"==typeof t.drawElements}(t)?{gl:t}:r(t,{container:"container target element el canvas holder parent parentNode wrapper use ref root node",gl:"gl context webgl glContext",attrs:"attributes attrs contextAttributes",pixelRatio:"pixelRatio pxRatio px ratio pxratio pixelratio",width:"w width",height:"h height"},!0)).pixelRatio||(t.pixelRatio=e.pixelRatio||1),t.gl)return t.gl;if(t.canvas&&(t.container=t.canvas.parentNode),t.container){if("string"==typeof t.container){var s=document.querySelector(t.container);if(!s)throw Error("Element "+t.container+" is not found");t.container=s}o(t.container)?(t.canvas=t.container,t.container=t.canvas.parentNode):t.canvas||(t.canvas=a(),t.container.appendChild(t.canvas),i(t))}else if(!t.canvas){if("undefined"==typeof document)throw Error("Not DOM environment. Use headless-gl.");t.container=document.body||document.documentElement,t.canvas=a(),t.container.appendChild(t.canvas),i(t)}if(!t.gl)try{t.gl=t.canvas.getContext("webgl",t.attrs)}catch(n){try{t.gl=t.canvas.getContext("experimental-webgl",t.attrs)}catch(n){t.gl=t.canvas.getContext("webgl-experimental",t.attrs)}}return t.gl}}).call(this,void 0!==e?e:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{"pick-by-alias":465}],324:[function(t,e,n){"use strict";e.exports=function(t,e,n){e?e.bind():t.bindBuffer(t.ELEMENT_ARRAY_BUFFER,null);var r=0|t.getParameter(t.MAX_VERTEX_ATTRIBS);if(n){if(n.length>r)throw new Error("gl-vao: Too many vertex attributes");for(var i=0;i<n.length;++i){var o=n[i];if(o.buffer){var a=o.buffer,s=o.size||4,A=o.type||t.FLOAT,l=!!o.normalized,c=o.stride||0,u=o.offset||0;a.bind(),t.enableVertexAttribArray(i),t.vertexAttribPointer(i,s,A,l,c,u)}else{if("number"==typeof o)t.vertexAttrib1f(i,o);else if(1===o.length)t.vertexAttrib1f(i,o[0]);else if(2===o.length)t.vertexAttrib2f(i,o[0],o[1]);else if(3===o.length)t.vertexAttrib3f(i,o[0],o[1],o[2]);else{if(4!==o.length)throw new Error("gl-vao: Invalid vertex attribute");t.vertexAttrib4f(i,o[0],o[1],o[2],o[3])}t.disableVertexAttribArray(i)}}for(;i<r;++i)t.disableVertexAttribArray(i)}else for(t.bindBuffer(t.ARRAY_BUFFER,null),i=0;i<r;++i)t.disableVertexAttribArray(i)}},{}],325:[function(t,e,n){"use strict";var r=t("./do-bind.js");function i(t){this.gl=t,this._elements=null,this._attributes=null,this._elementsType=t.UNSIGNED_SHORT}i.prototype.bind=function(){r(this.gl,this._elements,this._attributes)},i.prototype.update=function(t,e,n){this._elements=e,this._attributes=t,this._elementsType=n||this.gl.UNSIGNED_SHORT},i.prototype.dispose=function(){},i.prototype.unbind=function(){},i.prototype.draw=function(t,e,n){n=n||0;var r=this.gl;this._elements?r.drawElements(t,e,this._elementsType,n):r.drawArrays(t,n,e)},e.exports=function(t){return new i(t)}},{"./do-bind.js":324}],326:[function(t,e,n){"use strict";var r=t("./do-bind.js");function i(t,e,n,r,i,o){this.location=t,this.dimension=e,this.a=n,this.b=r,this.c=i,this.d=o}function o(t,e,n){this.gl=t,this._ext=e,this.handle=n,this._attribs=[],this._useElements=!1,this._elementsType=t.UNSIGNED_SHORT}i.prototype.bind=function(t){switch(this.dimension){case 1:t.vertexAttrib1f(this.location,this.a);break;case 2:t.vertexAttrib2f(this.location,this.a,this.b);break;case 3:t.vertexAttrib3f(this.location,this.a,this.b,this.c);break;case 4:t.vertexAttrib4f(this.location,this.a,this.b,this.c,this.d)}},o.prototype.bind=function(){this._ext.bindVertexArrayOES(this.handle);for(var t=0;t<this._attribs.length;++t)this._attribs[t].bind(this.gl)},o.prototype.unbind=function(){this._ext.bindVertexArrayOES(null)},o.prototype.dispose=function(){this._ext.deleteVertexArrayOES(this.handle)},o.prototype.update=function(t,e,n){if(this.bind(),r(this.gl,e,t),this.unbind(),this._attribs.length=0,t)for(var o=0;o<t.length;++o){var a=t[o];"number"==typeof a?this._attribs.push(new i(o,1,a)):Array.isArray(a)&&this._attribs.push(new i(o,a.length,a[0],a[1],a[2],a[3]))}this._useElements=!!e,this._elementsType=n||this.gl.UNSIGNED_SHORT},o.prototype.draw=function(t,e,n){n=n||0;var r=this.gl;this._useElements?r.drawElements(t,e,this._elementsType,n):r.drawArrays(t,n,e)},e.exports=function(t,e){return new o(t,e,e.createVertexArrayOES())}},{"./do-bind.js":324}],327:[function(t,e,n){"use strict";var r=t("./lib/vao-native.js"),i=t("./lib/vao-emulated.js");function o(t){this.bindVertexArrayOES=t.bindVertexArray.bind(t),this.createVertexArrayOES=t.createVertexArray.bind(t),this.deleteVertexArrayOES=t.deleteVertexArray.bind(t)}e.exports=function(t,e,n,a){var s,A=t.createVertexArray?new o(t):t.getExtension("OES_vertex_array_object");return(s=A?r(t,A):i(t)).update(e,n,a),s}},{"./lib/vao-emulated.js":325,"./lib/vao-native.js":326}],328:[function(t,e,n){e.exports=function(t,e,n){return t[0]=e[0]+n[0],t[1]=e[1]+n[1],t[2]=e[2]+n[2],t}},{}],329:[function(t,e,n){e.exports=function(t,e){var n=r(t[0],t[1],t[2]),a=r(e[0],e[1],e[2]);i(n,n),i(a,a);var s=o(n,a);return s>1?0:Math.acos(s)};var r=t("./fromValues"),i=t("./normalize"),o=t("./dot")},{"./dot":339,"./fromValues":345,"./normalize":356}],330:[function(t,e,n){e.exports=function(t,e){return t[0]=Math.ceil(e[0]),t[1]=Math.ceil(e[1]),t[2]=Math.ceil(e[2]),t}},{}],331:[function(t,e,n){e.exports=function(t){var e=new Float32Array(3);return e[0]=t[0],e[1]=t[1],e[2]=t[2],e}},{}],332:[function(t,e,n){e.exports=function(t,e){return t[0]=e[0],t[1]=e[1],t[2]=e[2],t}},{}],333:[function(t,e,n){e.exports=function(){var t=new Float32Array(3);return t[0]=0,t[1]=0,t[2]=0,t}},{}],334:[function(t,e,n){e.exports=function(t,e,n){var r=e[0],i=e[1],o=e[2],a=n[0],s=n[1],A=n[2];return t[0]=i*A-o*s,t[1]=o*a-r*A,t[2]=r*s-i*a,t}},{}],335:[function(t,e,n){e.exports=t("./distance")},{"./distance":336}],336:[function(t,e,n){e.exports=function(t,e){var n=e[0]-t[0],r=e[1]-t[1],i=e[2]-t[2];return Math.sqrt(n*n+r*r+i*i)}},{}],337:[function(t,e,n){e.exports=t("./divide")},{"./divide":338}],338:[function(t,e,n){e.exports=function(t,e,n){return t[0]=e[0]/n[0],t[1]=e[1]/n[1],t[2]=e[2]/n[2],t}},{}],339:[function(t,e,n){e.exports=function(t,e){return t[0]*e[0]+t[1]*e[1]+t[2]*e[2]}},{}],340:[function(t,e,n){e.exports=1e-6},{}],341:[function(t,e,n){e.exports=function(t,e){var n=t[0],i=t[1],o=t[2],a=e[0],s=e[1],A=e[2];return Math.abs(n-a)<=r*Math.max(1,Math.abs(n),Math.abs(a))&&Math.abs(i-s)<=r*Math.max(1,Math.abs(i),Math.abs(s))&&Math.abs(o-A)<=r*Math.max(1,Math.abs(o),Math.abs(A))};var r=t("./epsilon")},{"./epsilon":340}],342:[function(t,e,n){e.exports=function(t,e){return t[0]===e[0]&&t[1]===e[1]&&t[2]===e[2]}},{}],343:[function(t,e,n){e.exports=function(t,e){return t[0]=Math.floor(e[0]),t[1]=Math.floor(e[1]),t[2]=Math.floor(e[2]),t}},{}],344:[function(t,e,n){e.exports=function(t,e,n,i,o,a){var s,A;for(e||(e=3),n||(n=0),A=i?Math.min(i*e+n,t.length):t.length,s=n;s<A;s+=e)r[0]=t[s],r[1]=t[s+1],r[2]=t[s+2],o(r,r,a),t[s]=r[0],t[s+1]=r[1],t[s+2]=r[2];return t};var r=t("./create")()},{"./create":333}],345:[function(t,e,n){e.exports=function(t,e,n){var r=new Float32Array(3);return r[0]=t,r[1]=e,r[2]=n,r}},{}],346:[function(t,e,n){e.exports={EPSILON:t("./epsilon"),create:t("./create"),clone:t("./clone"),angle:t("./angle"),fromValues:t("./fromValues"),copy:t("./copy"),set:t("./set"),equals:t("./equals"),exactEquals:t("./exactEquals"),add:t("./add"),subtract:t("./subtract"),sub:t("./sub"),multiply:t("./multiply"),mul:t("./mul"),divide:t("./divide"),div:t("./div"),min:t("./min"),max:t("./max"),floor:t("./floor"),ceil:t("./ceil"),round:t("./round"),scale:t("./scale"),scaleAndAdd:t("./scaleAndAdd"),distance:t("./distance"),dist:t("./dist"),squaredDistance:t("./squaredDistance"),sqrDist:t("./sqrDist"),length:t("./length"),len:t("./len"),squaredLength:t("./squaredLength"),sqrLen:t("./sqrLen"),negate:t("./negate"),inverse:t("./inverse"),normalize:t("./normalize"),dot:t("./dot"),cross:t("./cross"),lerp:t("./lerp"),random:t("./random"),transformMat4:t("./transformMat4"),transformMat3:t("./transformMat3"),transformQuat:t("./transformQuat"),rotateX:t("./rotateX"),rotateY:t("./rotateY"),rotateZ:t("./rotateZ"),forEach:t("./forEach")}},{"./add":328,"./angle":329,"./ceil":330,"./clone":331,"./copy":332,"./create":333,"./cross":334,"./dist":335,"./distance":336,"./div":337,"./divide":338,"./dot":339,"./epsilon":340,"./equals":341,"./exactEquals":342,"./floor":343,"./forEach":344,"./fromValues":345,"./inverse":347,"./len":348,"./length":349,"./lerp":350,"./max":351,"./min":352,"./mul":353,"./multiply":354,"./negate":355,"./normalize":356,"./random":357,"./rotateX":358,"./rotateY":359,"./rotateZ":360,"./round":361,"./scale":362,"./scaleAndAdd":363,"./set":364,"./sqrDist":365,"./sqrLen":366,"./squaredDistance":367,"./squaredLength":368,"./sub":369,"./subtract":370,"./transformMat3":371,"./transformMat4":372,"./transformQuat":373}],347:[function(t,e,n){e.exports=function(t,e){return t[0]=1/e[0],t[1]=1/e[1],t[2]=1/e[2],t}},{}],348:[function(t,e,n){e.exports=t("./length")},{"./length":349}],349:[function(t,e,n){e.exports=function(t){var e=t[0],n=t[1],r=t[2];return Math.sqrt(e*e+n*n+r*r)}},{}],350:[function(t,e,n){e.exports=function(t,e,n,r){var i=e[0],o=e[1],a=e[2];return t[0]=i+r*(n[0]-i),t[1]=o+r*(n[1]-o),t[2]=a+r*(n[2]-a),t}},{}],351:[function(t,e,n){e.exports=function(t,e,n){return t[0]=Math.max(e[0],n[0]),t[1]=Math.max(e[1],n[1]),t[2]=Math.max(e[2],n[2]),t}},{}],352:[function(t,e,n){e.exports=function(t,e,n){return t[0]=Math.min(e[0],n[0]),t[1]=Math.min(e[1],n[1]),t[2]=Math.min(e[2],n[2]),t}},{}],353:[function(t,e,n){e.exports=t("./multiply")},{"./multiply":354}],354:[function(t,e,n){e.exports=function(t,e,n){return t[0]=e[0]*n[0],t[1]=e[1]*n[1],t[2]=e[2]*n[2],t}},{}],355:[function(t,e,n){e.exports=function(t,e){return t[0]=-e[0],t[1]=-e[1],t[2]=-e[2],t}},{}],356:[function(t,e,n){e.exports=function(t,e){var n=e[0],r=e[1],i=e[2],o=n*n+r*r+i*i;return o>0&&(o=1/Math.sqrt(o),t[0]=e[0]*o,t[1]=e[1]*o,t[2]=e[2]*o),t}},{}],357:[function(t,e,n){e.exports=function(t,e){e=e||1;var n=2*Math.random()*Math.PI,r=2*Math.random()-1,i=Math.sqrt(1-r*r)*e;return t[0]=Math.cos(n)*i,t[1]=Math.sin(n)*i,t[2]=r*e,t}},{}],358:[function(t,e,n){e.exports=function(t,e,n,r){var i=n[1],o=n[2],a=e[1]-i,s=e[2]-o,A=Math.sin(r),l=Math.cos(r);return t[0]=e[0],t[1]=i+a*l-s*A,t[2]=o+a*A+s*l,t}},{}],359:[function(t,e,n){e.exports=function(t,e,n,r){var i=n[0],o=n[2],a=e[0]-i,s=e[2]-o,A=Math.sin(r),l=Math.cos(r);return t[0]=i+s*A+a*l,t[1]=e[1],t[2]=o+s*l-a*A,t}},{}],360:[function(t,e,n){e.exports=function(t,e,n,r){var i=n[0],o=n[1],a=e[0]-i,s=e[1]-o,A=Math.sin(r),l=Math.cos(r);return t[0]=i+a*l-s*A,t[1]=o+a*A+s*l,t[2]=e[2],t}},{}],361:[function(t,e,n){e.exports=function(t,e){return t[0]=Math.round(e[0]),t[1]=Math.round(e[1]),t[2]=Math.round(e[2]),t}},{}],362:[function(t,e,n){e.exports=function(t,e,n){return t[0]=e[0]*n,t[1]=e[1]*n,t[2]=e[2]*n,t}},{}],363:[function(t,e,n){e.exports=function(t,e,n,r){return t[0]=e[0]+n[0]*r,t[1]=e[1]+n[1]*r,t[2]=e[2]+n[2]*r,t}},{}],364:[function(t,e,n){e.exports=function(t,e,n,r){return t[0]=e,t[1]=n,t[2]=r,t}},{}],365:[function(t,e,n){e.exports=t("./squaredDistance")},{"./squaredDistance":367}],366:[function(t,e,n){e.exports=t("./squaredLength")},{"./squaredLength":368}],367:[function(t,e,n){e.exports=function(t,e){var n=e[0]-t[0],r=e[1]-t[1],i=e[2]-t[2];return n*n+r*r+i*i}},{}],368:[function(t,e,n){e.exports=function(t){var e=t[0],n=t[1],r=t[2];return e*e+n*n+r*r}},{}],369:[function(t,e,n){e.exports=t("./subtract")},{"./subtract":370}],370:[function(t,e,n){e.exports=function(t,e,n){return t[0]=e[0]-n[0],t[1]=e[1]-n[1],t[2]=e[2]-n[2],t}},{}],371:[function(t,e,n){e.exports=function(t,e,n){var r=e[0],i=e[1],o=e[2];return t[0]=r*n[0]+i*n[3]+o*n[6],t[1]=r*n[1]+i*n[4]+o*n[7],t[2]=r*n[2]+i*n[5]+o*n[8],t}},{}],372:[function(t,e,n){e.exports=function(t,e,n){var r=e[0],i=e[1],o=e[2],a=n[3]*r+n[7]*i+n[11]*o+n[15];return a=a||1,t[0]=(n[0]*r+n[4]*i+n[8]*o+n[12])/a,t[1]=(n[1]*r+n[5]*i+n[9]*o+n[13])/a,t[2]=(n[2]*r+n[6]*i+n[10]*o+n[14])/a,t}},{}],373:[function(t,e,n){e.exports=function(t,e,n){var r=e[0],i=e[1],o=e[2],a=n[0],s=n[1],A=n[2],l=n[3],c=l*r+s*o-A*i,u=l*i+A*r-a*o,g=l*o+a*i-s*r,h=-a*r-s*i-A*o;return t[0]=c*l+h*-a+u*-A-g*-s,t[1]=u*l+h*-s+g*-a-c*-A,t[2]=g*l+h*-A+c*-s-u*-a,t}},{}],374:[function(t,e,n){e.exports=function(t,e,n){return t[0]=e[0]+n[0],t[1]=e[1]+n[1],t[2]=e[2]+n[2],t[3]=e[3]+n[3],t}},{}],375:[function(t,e,n){e.exports=function(t){var e=new Float32Array(4);return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[3],e}},{}],376:[function(t,e,n){e.exports=function(t,e){return t[0]=e[0],t[1]=e[1],t[2]=e[2],t[3]=e[3],t}},{}],377:[function(t,e,n){e.exports=function(){var t=new Float32Array(4);return t[0]=0,t[1]=0,t[2]=0,t[3]=0,t}},{}],378:[function(t,e,n){e.exports=function(t,e){var n=e[0]-t[0],r=e[1]-t[1],i=e[2]-t[2],o=e[3]-t[3];return Math.sqrt(n*n+r*r+i*i+o*o)}},{}],379:[function(t,e,n){e.exports=function(t,e,n){return t[0]=e[0]/n[0],t[1]=e[1]/n[1],t[2]=e[2]/n[2],t[3]=e[3]/n[3],t}},{}],380:[function(t,e,n){e.exports=function(t,e){return t[0]*e[0]+t[1]*e[1]+t[2]*e[2]+t[3]*e[3]}},{}],381:[function(t,e,n){e.exports=function(t,e,n,r){var i=new Float32Array(4);return i[0]=t,i[1]=e,i[2]=n,i[3]=r,i}},{}],382:[function(t,e,n){e.exports={create:t("./create"),clone:t("./clone"),fromValues:t("./fromValues"),copy:t("./copy"),set:t("./set"),add:t("./add"),subtract:t("./subtract"),multiply:t("./multiply"),divide:t("./divide"),min:t("./min"),max:t("./max"),scale:t("./scale"),scaleAndAdd:t("./scaleAndAdd"),distance:t("./distance"),squaredDistance:t("./squaredDistance"),length:t("./length"),squaredLength:t("./squaredLength"),negate:t("./negate"),inverse:t("./inverse"),normalize:t("./normalize"),dot:t("./dot"),lerp:t("./lerp"),random:t("./random"),transformMat4:t("./transformMat4"),transformQuat:t("./transformQuat")}},{"./add":374,"./clone":375,"./copy":376,"./create":377,"./distance":378,"./divide":379,"./dot":380,"./fromValues":381,"./inverse":383,"./length":384,"./lerp":385,"./max":386,"./min":387,"./multiply":388,"./negate":389,"./normalize":390,"./random":391,"./scale":392,"./scaleAndAdd":393,"./set":394,"./squaredDistance":395,"./squaredLength":396,"./subtract":397,"./transformMat4":398,"./transformQuat":399}],383:[function(t,e,n){e.exports=function(t,e){return t[0]=1/e[0],t[1]=1/e[1],t[2]=1/e[2],t[3]=1/e[3],t}},{}],384:[function(t,e,n){e.exports=function(t){var e=t[0],n=t[1],r=t[2],i=t[3];return Math.sqrt(e*e+n*n+r*r+i*i)}},{}],385:[function(t,e,n){e.exports=function(t,e,n,r){var i=e[0],o=e[1],a=e[2],s=e[3];return t[0]=i+r*(n[0]-i),t[1]=o+r*(n[1]-o),t[2]=a+r*(n[2]-a),t[3]=s+r*(n[3]-s),t}},{}],386:[function(t,e,n){e.exports=function(t,e,n){return t[0]=Math.max(e[0],n[0]),t[1]=Math.max(e[1],n[1]),t[2]=Math.max(e[2],n[2]),t[3]=Math.max(e[3],n[3]),t}},{}],387:[function(t,e,n){e.exports=function(t,e,n){return t[0]=Math.min(e[0],n[0]),t[1]=Math.min(e[1],n[1]),t[2]=Math.min(e[2],n[2]),t[3]=Math.min(e[3],n[3]),t}},{}],388:[function(t,e,n){e.exports=function(t,e,n){return t[0]=e[0]*n[0],t[1]=e[1]*n[1],t[2]=e[2]*n[2],t[3]=e[3]*n[3],t}},{}],389:[function(t,e,n){e.exports=function(t,e){return t[0]=-e[0],t[1]=-e[1],t[2]=-e[2],t[3]=-e[3],t}},{}],390:[function(t,e,n){e.exports=function(t,e){var n=e[0],r=e[1],i=e[2],o=e[3],a=n*n+r*r+i*i+o*o;return a>0&&(a=1/Math.sqrt(a),t[0]=n*a,t[1]=r*a,t[2]=i*a,t[3]=o*a),t}},{}],391:[function(t,e,n){var r=t("./normalize"),i=t("./scale");e.exports=function(t,e){return e=e||1,t[0]=Math.random(),t[1]=Math.random(),t[2]=Math.random(),t[3]=Math.random(),r(t,t),i(t,t,e),t}},{"./normalize":390,"./scale":392}],392:[function(t,e,n){e.exports=function(t,e,n){return t[0]=e[0]*n,t[1]=e[1]*n,t[2]=e[2]*n,t[3]=e[3]*n,t}},{}],393:[function(t,e,n){e.exports=function(t,e,n,r){return t[0]=e[0]+n[0]*r,t[1]=e[1]+n[1]*r,t[2]=e[2]+n[2]*r,t[3]=e[3]+n[3]*r,t}},{}],394:[function(t,e,n){e.exports=function(t,e,n,r,i){return t[0]=e,t[1]=n,t[2]=r,t[3]=i,t}},{}],395:[function(t,e,n){e.exports=function(t,e){var n=e[0]-t[0],r=e[1]-t[1],i=e[2]-t[2],o=e[3]-t[3];return n*n+r*r+i*i+o*o}},{}],396:[function(t,e,n){e.exports=function(t){var e=t[0],n=t[1],r=t[2],i=t[3];return e*e+n*n+r*r+i*i}},{}],397:[function(t,e,n){e.exports=function(t,e,n){return t[0]=e[0]-n[0],t[1]=e[1]-n[1],t[2]=e[2]-n[2],t[3]=e[3]-n[3],t}},{}],398:[function(t,e,n){e.exports=function(t,e,n){var r=e[0],i=e[1],o=e[2],a=e[3];return t[0]=n[0]*r+n[4]*i+n[8]*o+n[12]*a,t[1]=n[1]*r+n[5]*i+n[9]*o+n[13]*a,t[2]=n[2]*r+n[6]*i+n[10]*o+n[14]*a,t[3]=n[3]*r+n[7]*i+n[11]*o+n[15]*a,t}},{}],399:[function(t,e,n){e.exports=function(t,e,n){var r=e[0],i=e[1],o=e[2],a=n[0],s=n[1],A=n[2],l=n[3],c=l*r+s*o-A*i,u=l*i+A*r-a*o,g=l*o+a*i-s*r,h=-a*r-s*i-A*o;return t[0]=c*l+h*-a+u*-A-g*-s,t[1]=u*l+h*-s+g*-a-c*-A,t[2]=g*l+h*-A+c*-s-u*-a,t[3]=e[3],t}},{}],400:[function(t,e,n){e.exports=function(t,e,n,o){return r[0]=o,r[1]=n,r[2]=e,r[3]=t,i[0]};var r=new Uint8Array(4),i=new Float32Array(r.buffer)},{}],401:[function(t,e,n){var r=t("glsl-tokenizer"),i=t("atob-lite");e.exports=function(t){for(var e=Array.isArray(t)?t:r(t),n=0;n<e.length;n++){var o=e[n];if("preprocessor"===o.type){var a=o.data.match(/\#define\s+SHADER_NAME(_B64)?\s+(.+)$/);if(a&&a[2]){var s=a[1],A=a[2];return(s?i(A):A).trim()}}}}},{"atob-lite":72,"glsl-tokenizer":408}],402:[function(t,e,n){e.exports=function(t){var e,n,B,b=0,Q=0,w=A,x=[],_=[],D=1,M=0,k=0,S=!1,L=!1,R="",N=o,F=r;return"300 es"===(t=t||{}).version&&(N=s,F=a),function(t){return _=[],null!==t?function(t){var n;for(b=0,B=(R+=t).length;e=R[b],b<B;){switch(n=b,w){case c:b=H();break;case u:case g:b=G();break;case h:b=j();break;case d:b=K();break;case E:b=P();break;case f:b=z();break;case l:b=J();break;case m:b=O();break;case A:b=U()}if(n!==b)switch(R[n]){case"\n":M=0,++D;break;default:++M}}return Q+=b,R=R.slice(b),_}(t.replace?t.replace(/\r\n/g,"\n"):t):(x.length&&T(x.join("")),w=y,T("(eof)"),_)};function T(t){t.length&&_.push({type:v[w],data:t,position:k,line:D,column:M})}function U(){return x=x.length?[]:x,"/"===n&&"*"===e?(k=Q+b-1,w=c,n=e,b+1):"/"===n&&"/"===e?(k=Q+b-1,w=u,n=e,b+1):"#"===e?(w=g,k=Q+b,b):/\s/.test(e)?(w=m,k=Q+b,b):(S=/\d/.test(e),L=/[^\w_]/.test(e),k=Q+b,w=S?d:L?h:l,b)}function O(){return/[^\s]/g.test(e)?(T(x.join("")),w=A,b):(x.push(e),n=e,b+1)}function G(){return"\r"!==e&&"\n"!==e||"\\"===n?(x.push(e),n=e,b+1):(T(x.join("")),w=A,b)}function H(){return"/"===e&&"*"===n?(x.push(e),T(x.join("")),w=A,b+1):(x.push(e),n=e,b+1)}function j(){if("."===n&&/\d/.test(e))return w=f,b;if("/"===n&&"*"===e)return w=c,b;if("/"===n&&"/"===e)return w=u,b;if("."===e&&x.length){for(;Y(x););return w=f,b}if(";"===e||")"===e||"("===e){if(x.length)for(;Y(x););return T(e),w=A,b+1}var t=2===x.length&&"="!==e;if(/[\w_\d\s]/.test(e)||t){for(;Y(x););return w=A,b}return x.push(e),n=e,b+1}function Y(t){for(var e,n,r=0;;){if(e=i.indexOf(t.slice(0,t.length+r).join("")),n=i[e],-1===e){if(r--+t.length>0)continue;n=t.slice(0,1).join("")}return T(n),k+=n.length,(x=x.slice(n.length)).length}}function P(){return/[^a-fA-F0-9]/.test(e)?(T(x.join("")),w=A,b):(x.push(e),n=e,b+1)}function K(){return"."===e?(x.push(e),w=f,n=e,b+1):/[eE]/.test(e)?(x.push(e),w=f,n=e,b+1):"x"===e&&1===x.length&&"0"===x[0]?(w=E,x.push(e),n=e,b+1):/[^\d]/.test(e)?(T(x.join("")),w=A,b):(x.push(e),n=e,b+1)}function z(){return"f"===e&&(x.push(e),n=e,b+=1),/[eE]/.test(e)?(x.push(e),n=e,b+1):"-"===e&&/[eE]/.test(n)?(x.push(e),n=e,b+1):/[^\d]/.test(e)?(T(x.join("")),w=A,b):(x.push(e),n=e,b+1)}function J(){if(/[^\d\w_]/.test(e)){var t=x.join("");return w=F.indexOf(t)>-1?C:N.indexOf(t)>-1?I:p,T(x.join("")),w=A,b}return x.push(e),n=e,b+1}};var r=t("./lib/literals"),i=t("./lib/operators"),o=t("./lib/builtins"),a=t("./lib/literals-300es"),s=t("./lib/builtins-300es"),A=999,l=9999,c=0,u=1,g=2,h=3,d=4,f=5,p=6,I=7,C=8,m=9,y=10,E=11,v=["block-comment","line-comment","preprocessor","operator","integer","float","ident","builtin","keyword","whitespace","eof","integer"]},{"./lib/builtins":404,"./lib/builtins-300es":403,"./lib/literals":406,"./lib/literals-300es":405,"./lib/operators":407}],403:[function(t,e,n){var r=t("./builtins");r=r.slice().filter(function(t){return!/^(gl\_|texture)/.test(t)}),e.exports=r.concat(["gl_VertexID","gl_InstanceID","gl_Position","gl_PointSize","gl_FragCoord","gl_FrontFacing","gl_FragDepth","gl_PointCoord","gl_MaxVertexAttribs","gl_MaxVertexUniformVectors","gl_MaxVertexOutputVectors","gl_MaxFragmentInputVectors","gl_MaxVertexTextureImageUnits","gl_MaxCombinedTextureImageUnits","gl_MaxTextureImageUnits","gl_MaxFragmentUniformVectors","gl_MaxDrawBuffers","gl_MinProgramTexelOffset","gl_MaxProgramTexelOffset","gl_DepthRangeParameters","gl_DepthRange","trunc","round","roundEven","isnan","isinf","floatBitsToInt","floatBitsToUint","intBitsToFloat","uintBitsToFloat","packSnorm2x16","unpackSnorm2x16","packUnorm2x16","unpackUnorm2x16","packHalf2x16","unpackHalf2x16","outerProduct","transpose","determinant","inverse","texture","textureSize","textureProj","textureLod","textureOffset","texelFetch","texelFetchOffset","textureProjOffset","textureLodOffset","textureProjLod","textureProjLodOffset","textureGrad","textureGradOffset","textureProjGrad","textureProjGradOffset"])},{"./builtins":404}],404:[function(t,e,n){e.exports=["abs","acos","all","any","asin","atan","ceil","clamp","cos","cross","dFdx","dFdy","degrees","distance","dot","equal","exp","exp2","faceforward","floor","fract","gl_BackColor","gl_BackLightModelProduct","gl_BackLightProduct","gl_BackMaterial","gl_BackSecondaryColor","gl_ClipPlane","gl_ClipVertex","gl_Color","gl_DepthRange","gl_DepthRangeParameters","gl_EyePlaneQ","gl_EyePlaneR","gl_EyePlaneS","gl_EyePlaneT","gl_Fog","gl_FogCoord","gl_FogFragCoord","gl_FogParameters","gl_FragColor","gl_FragCoord","gl_FragData","gl_FragDepth","gl_FragDepthEXT","gl_FrontColor","gl_FrontFacing","gl_FrontLightModelProduct","gl_FrontLightProduct","gl_FrontMaterial","gl_FrontSecondaryColor","gl_LightModel","gl_LightModelParameters","gl_LightModelProducts","gl_LightProducts","gl_LightSource","gl_LightSourceParameters","gl_MaterialParameters","gl_MaxClipPlanes","gl_MaxCombinedTextureImageUnits","gl_MaxDrawBuffers","gl_MaxFragmentUniformComponents","gl_MaxLights","gl_MaxTextureCoords","gl_MaxTextureImageUnits","gl_MaxTextureUnits","gl_MaxVaryingFloats","gl_MaxVertexAttribs","gl_MaxVertexTextureImageUnits","gl_MaxVertexUniformComponents","gl_ModelViewMatrix","gl_ModelViewMatrixInverse","gl_ModelViewMatrixInverseTranspose","gl_ModelViewMatrixTranspose","gl_ModelViewProjectionMatrix","gl_ModelViewProjectionMatrixInverse","gl_ModelViewProjectionMatrixInverseTranspose","gl_ModelViewProjectionMatrixTranspose","gl_MultiTexCoord0","gl_MultiTexCoord1","gl_MultiTexCoord2","gl_MultiTexCoord3","gl_MultiTexCoord4","gl_MultiTexCoord5","gl_MultiTexCoord6","gl_MultiTexCoord7","gl_Normal","gl_NormalMatrix","gl_NormalScale","gl_ObjectPlaneQ","gl_ObjectPlaneR","gl_ObjectPlaneS","gl_ObjectPlaneT","gl_Point","gl_PointCoord","gl_PointParameters","gl_PointSize","gl_Position","gl_ProjectionMatrix","gl_ProjectionMatrixInverse","gl_ProjectionMatrixInverseTranspose","gl_ProjectionMatrixTranspose","gl_SecondaryColor","gl_TexCoord","gl_TextureEnvColor","gl_TextureMatrix","gl_TextureMatrixInverse","gl_TextureMatrixInverseTranspose","gl_TextureMatrixTranspose","gl_Vertex","greaterThan","greaterThanEqual","inversesqrt","length","lessThan","lessThanEqual","log","log2","matrixCompMult","max","min","mix","mod","normalize","not","notEqual","pow","radians","reflect","refract","sign","sin","smoothstep","sqrt","step","tan","texture2D","texture2DLod","texture2DProj","texture2DProjLod","textureCube","textureCubeLod","texture2DLodEXT","texture2DProjLodEXT","textureCubeLodEXT","texture2DGradEXT","texture2DProjGradEXT","textureCubeGradEXT"]},{}],405:[function(t,e,n){var r=t("./literals");e.exports=r.slice().concat(["layout","centroid","smooth","case","mat2x2","mat2x3","mat2x4","mat3x2","mat3x3","mat3x4","mat4x2","mat4x3","mat4x4","uint","uvec2","uvec3","uvec4","samplerCubeShadow","sampler2DArray","sampler2DArrayShadow","isampler2D","isampler3D","isamplerCube","isampler2DArray","usampler2D","usampler3D","usamplerCube","usampler2DArray","coherent","restrict","readonly","writeonly","resource","atomic_uint","noperspective","patch","sample","subroutine","common","partition","active","filter","image1D","image2D","image3D","imageCube","iimage1D","iimage2D","iimage3D","iimageCube","uimage1D","uimage2D","uimage3D","uimageCube","image1DArray","image2DArray","iimage1DArray","iimage2DArray","uimage1DArray","uimage2DArray","image1DShadow","image2DShadow","image1DArrayShadow","image2DArrayShadow","imageBuffer","iimageBuffer","uimageBuffer","sampler1DArray","sampler1DArrayShadow","isampler1D","isampler1DArray","usampler1D","usampler1DArray","isampler2DRect","usampler2DRect","samplerBuffer","isamplerBuffer","usamplerBuffer","sampler2DMS","isampler2DMS","usampler2DMS","sampler2DMSArray","isampler2DMSArray","usampler2DMSArray"])},{"./literals":406}],406:[function(t,e,n){e.exports=["precision","highp","mediump","lowp","attribute","const","uniform","varying","break","continue","do","for","while","if","else","in","out","inout","float","int","void","bool","true","false","discard","return","mat2","mat3","mat4","vec2","vec3","vec4","ivec2","ivec3","ivec4","bvec2","bvec3","bvec4","sampler1D","sampler2D","sampler3D","samplerCube","sampler1DShadow","sampler2DShadow","struct","asm","class","union","enum","typedef","template","this","packed","goto","switch","default","inline","noinline","volatile","public","static","extern","external","interface","long","short","double","half","fixed","unsigned","input","output","hvec2","hvec3","hvec4","dvec2","dvec3","dvec4","fvec2","fvec3","fvec4","sampler2DRect","sampler3DRect","sampler2DRectShadow","sizeof","cast","namespace","using"]},{}],407:[function(t,e,n){e.exports=["<<=",">>=","++","--","<<",">>","<=",">=","==","!=","&&","||","+=","-=","*=","/=","%=","&=","^^","^=","|=","(",")","[","]",".","!","~","*","/","%","+","-","<",">","&","^","|","?",":","=",",",";","{","}"]},{}],408:[function(t,e,n){var r=t("./index");e.exports=function(t,e){var n=r(e),i=[];return i=(i=i.concat(n(t))).concat(n(null))}},{"./index":402}],409:[function(t,e,n){e.exports=function(t){"string"==typeof t&&(t=[t]);for(var e=[].slice.call(arguments,1),n=[],r=0;r<t.length-1;r++)n.push(t[r],e[r]||"");return n.push(t[r]),n.join("")}},{}],410:[function(t,n,r){(function(e){"use strict";var r,i=t("is-browser");r="function"==typeof e.matchMedia?!e.matchMedia("(hover: none)").matches:i,n.exports=r}).call(this,void 0!==e?e:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{"is-browser":417}],411:[function(t,e,n){"use strict";var r=t("is-browser");e.exports=r&&function(){var t=!1;try{var e=Object.defineProperty({},"passive",{get:function(){t=!0}});window.addEventListener("test",null,e),window.removeEventListener("test",null,e)}catch(e){t=!1}return t}()},{"is-browser":417}],412:[function(t,e,n){n.read=function(t,e,n,r,i){var o,a,s=8*i-r-1,A=(1<<s)-1,l=A>>1,c=-7,u=n?i-1:0,g=n?-1:1,h=t[e+u];for(u+=g,o=h&(1<<-c)-1,h>>=-c,c+=s;c>0;o=256*o+t[e+u],u+=g,c-=8);for(a=o&(1<<-c)-1,o>>=-c,c+=r;c>0;a=256*a+t[e+u],u+=g,c-=8);if(0===o)o=1-l;else{if(o===A)return a?NaN:1/0*(h?-1:1);a+=Math.pow(2,r),o-=l}return(h?-1:1)*a*Math.pow(2,o-r)},n.write=function(t,e,n,r,i,o){var a,s,A,l=8*o-i-1,c=(1<<l)-1,u=c>>1,g=23===i?Math.pow(2,-24)-Math.pow(2,-77):0,h=r?0:o-1,d=r?1:-1,f=e<0||0===e&&1/e<0?1:0;for(e=Math.abs(e),isNaN(e)||e===1/0?(s=isNaN(e)?1:0,a=c):(a=Math.floor(Math.log(e)/Math.LN2),e*(A=Math.pow(2,-a))<1&&(a--,A*=2),(e+=a+u>=1?g/A:g*Math.pow(2,1-u))*A>=2&&(a++,A/=2),a+u>=c?(s=0,a=c):a+u>=1?(s=(e*A-1)*Math.pow(2,i),a+=u):(s=e*Math.pow(2,u-1)*Math.pow(2,i),a=0));i>=8;t[n+h]=255&s,h+=d,s/=256,i-=8);for(a=a<<i|s,l+=i;l>0;t[n+h]=255&a,h+=d,a/=256,l-=8);t[n+h-d]|=128*f}},{}],413:[function(t,e,n){"use strict";e.exports=function(t,e){var n=t.length;if(0===n)throw new Error("Must have at least d+1 points");var i=t[0].length;if(n<=i)throw new Error("Must input at least d+1 points");var a=t.slice(0,i+1),s=r.apply(void 0,a);if(0===s)throw new Error("Input not in general position");for(var A=new Array(i+1),c=0;c<=i;++c)A[c]=c;s<0&&(A[0]=1,A[1]=0);for(var u=new o(A,new Array(i+1),!1),g=u.adjacent,h=new Array(i+2),c=0;c<=i;++c){for(var d=A.slice(),f=0;f<=i;++f)f===c&&(d[f]=-1);var p=d[0];d[0]=d[1],d[1]=p;var I=new o(d,new Array(i+1),!0);g[c]=I,h[c]=I}h[i+1]=u;for(var c=0;c<=i;++c)for(var d=g[c].vertices,C=g[c].adjacent,f=0;f<=i;++f){var m=d[f];if(m<0)C[f]=u;else for(var y=0;y<=i;++y)g[y].vertices.indexOf(m)<0&&(C[f]=g[y])}for(var E=new l(i,a,h),v=!!e,c=i+1;c<n;++c)E.insert(t[c],v);return E.boundary()};var r=t("robust-orientation"),i=t("simplicial-complex").compareCells;function o(t,e,n){this.vertices=t,this.adjacent=e,this.boundary=n,this.lastVisited=-1}function a(t,e,n){this.vertices=t,this.cell=e,this.index=n}function s(t,e){return i(t.vertices,e.vertices)}o.prototype.flip=function(){var t=this.vertices[0];this.vertices[0]=this.vertices[1],this.vertices[1]=t;var e=this.adjacent[0];this.adjacent[0]=this.adjacent[1],this.adjacent[1]=e};var A=[];function l(t,e,n){this.dimension=t,this.vertices=e,this.simplices=n,this.interior=n.filter(function(t){return!t.boundary}),this.tuple=new Array(t+1);for(var i=0;i<=t;++i)this.tuple[i]=this.vertices[i];var o=A[t];o||(o=A[t]=function(t){for(var e=["function orient(){var tuple=this.tuple;return test("],n=0;n<=t;++n)n>0&&e.push(","),e.push("tuple[",n,"]");e.push(")}return orient");var i=new Function("test",e.join("")),o=r[t+1];return o||(o=r),i(o)}(t)),this.orient=o}var c=l.prototype;c.handleBoundaryDegeneracy=function(t,e){var n=this.dimension,r=this.vertices.length-1,i=this.tuple,o=this.vertices,a=[t];for(t.lastVisited=-r;a.length>0;){(t=a.pop()).vertices;for(var s=t.adjacent,A=0;A<=n;++A){var l=s[A];if(l.boundary&&!(l.lastVisited<=-r)){for(var c=l.vertices,u=0;u<=n;++u){var g=c[u];i[u]=g<0?e:o[g]}var h=this.orient();if(h>0)return l;l.lastVisited=-r,0===h&&a.push(l)}}}return null},c.walk=function(t,e){var n=this.vertices.length-1,r=this.dimension,i=this.vertices,o=this.tuple,a=e?this.interior.length*Math.random()|0:this.interior.length-1,s=this.interior[a];t:for(;!s.boundary;){for(var A=s.vertices,l=s.adjacent,c=0;c<=r;++c)o[c]=i[A[c]];for(s.lastVisited=n,c=0;c<=r;++c){var u=l[c];if(!(u.lastVisited>=n)){var g=o[c];o[c]=t;var h=this.orient();if(o[c]=g,h<0){s=u;continue t}u.boundary?u.lastVisited=-n:u.lastVisited=n}}return}return s},c.addPeaks=function(t,e){var n=this.vertices.length-1,r=this.dimension,i=this.vertices,A=this.tuple,l=this.interior,c=this.simplices,u=[e];e.lastVisited=n,e.vertices[e.vertices.indexOf(-1)]=n,e.boundary=!1,l.push(e);for(var g=[];u.length>0;){var h=(e=u.pop()).vertices,d=e.adjacent,f=h.indexOf(n);if(!(f<0))for(var p=0;p<=r;++p)if(p!==f){var I=d[p];if(I.boundary&&!(I.lastVisited>=n)){var C=I.vertices;if(I.lastVisited!==-n){for(var m=0,y=0;y<=r;++y)C[y]<0?(m=y,A[y]=t):A[y]=i[C[y]];if(this.orient()>0){C[m]=n,I.boundary=!1,l.push(I),u.push(I),I.lastVisited=n;continue}I.lastVisited=-n}var E=I.adjacent,v=h.slice(),B=d.slice(),b=new o(v,B,!0);c.push(b);var Q=E.indexOf(e);if(!(Q<0))for(E[Q]=b,B[f]=I,v[p]=-1,B[p]=e,d[p]=b,b.flip(),y=0;y<=r;++y){var w=v[y];if(!(w<0||w===n)){for(var x=new Array(r-1),_=0,D=0;D<=r;++D){var M=v[D];M<0||D===y||(x[_++]=M)}g.push(new a(x,b,y))}}}}}for(g.sort(s),p=0;p+1<g.length;p+=2){var k=g[p],S=g[p+1],L=k.index,R=S.index;L<0||R<0||(k.cell.adjacent[k.index]=S.cell,S.cell.adjacent[S.index]=k.cell)}},c.insert=function(t,e){var n=this.vertices;n.push(t);var r=this.walk(t,e);if(r){for(var i=this.dimension,o=this.tuple,a=0;a<=i;++a){var s=r.vertices[a];o[a]=s<0?t:n[s]}var A=this.orient(o);A<0||(0!==A||(r=this.handleBoundaryDegeneracy(r,t)))&&this.addPeaks(t,r)}},c.boundary=function(){for(var t=this.dimension,e=[],n=this.simplices,r=n.length,i=0;i<r;++i){var o=n[i];if(o.boundary){for(var a=new Array(t),s=o.vertices,A=0,l=0,c=0;c<=t;++c)s[c]>=0?a[A++]=s[c]:l=1&c;if(l===(1&t)){var u=a[0];a[0]=a[1],a[1]=u}e.push(a)}}return e}},{"robust-orientation":510,"simplicial-complex":520}],414:[function(t,e,n){"use strict";var r=t("binary-search-bounds"),i=0,o=1;function a(t,e,n,r,i){this.mid=t,this.left=e,this.right=n,this.leftPoints=r,this.rightPoints=i,this.count=(e?e.count:0)+(n?n.count:0)+r.length}e.exports=function(t){return t&&0!==t.length?new m(C(t)):new m(null)};var s=a.prototype;function A(t,e){t.mid=e.mid,t.left=e.left,t.right=e.right,t.leftPoints=e.leftPoints,t.rightPoints=e.rightPoints,t.count=e.count}function l(t,e){var n=C(e);t.mid=n.mid,t.left=n.left,t.right=n.right,t.leftPoints=n.leftPoints,t.rightPoints=n.rightPoints,t.count=n.count}function c(t,e){var n=t.intervals([]);n.push(e),l(t,n)}function u(t,e){var n=t.intervals([]),r=n.indexOf(e);return r<0?i:(n.splice(r,1),l(t,n),o)}function g(t,e,n){for(var r=0;r<t.length&&t[r][0]<=e;++r){var i=n(t[r]);if(i)return i}}function h(t,e,n){for(var r=t.length-1;r>=0&&t[r][1]>=e;--r){var i=n(t[r]);if(i)return i}}function d(t,e){for(var n=0;n<t.length;++n){var r=e(t[n]);if(r)return r}}function f(t,e){return t-e}function p(t,e){var n=t[0]-e[0];return n||t[1]-e[1]}function I(t,e){var n=t[1]-e[1];return n||t[0]-e[0]}function C(t){if(0===t.length)return null;for(var e=[],n=0;n<t.length;++n)e.push(t[n][0],t[n][1]);e.sort(f);var r=e[e.length>>1],i=[],o=[],s=[];for(n=0;n<t.length;++n){var A=t[n];A[1]<r?i.push(A):r<A[0]?o.push(A):s.push(A)}var l=s,c=s.slice();return l.sort(p),c.sort(I),new a(r,C(i),C(o),l,c)}function m(t){this.root=t}s.intervals=function(t){return t.push.apply(t,this.leftPoints),this.left&&this.left.intervals(t),this.right&&this.right.intervals(t),t},s.insert=function(t){var e=this.count-this.leftPoints.length;if(this.count+=1,t[1]<this.mid)this.left?4*(this.left.count+1)>3*(e+1)?c(this,t):this.left.insert(t):this.left=C([t]);else if(t[0]>this.mid)this.right?4*(this.right.count+1)>3*(e+1)?c(this,t):this.right.insert(t):this.right=C([t]);else{var n=r.ge(this.leftPoints,t,p),i=r.ge(this.rightPoints,t,I);this.leftPoints.splice(n,0,t),this.rightPoints.splice(i,0,t)}},s.remove=function(t){var e=this.count-this.leftPoints;if(t[1]<this.mid)return this.left?4*(this.right?this.right.count:0)>3*(e-1)?u(this,t):2===(l=this.left.remove(t))?(this.left=null,this.count-=1,o):(l===o&&(this.count-=1),l):i;if(t[0]>this.mid)return this.right?4*(this.left?this.left.count:0)>3*(e-1)?u(this,t):2===(l=this.right.remove(t))?(this.right=null,this.count-=1,o):(l===o&&(this.count-=1),l):i;if(1===this.count)return this.leftPoints[0]===t?2:i;if(1===this.leftPoints.length&&this.leftPoints[0]===t){if(this.left&&this.right){for(var n=this,a=this.left;a.right;)n=a,a=a.right;if(n===this)a.right=this.right;else{var s=this.left,l=this.right;n.count-=a.count,n.right=a.left,a.left=s,a.right=l}A(this,a),this.count=(this.left?this.left.count:0)+(this.right?this.right.count:0)+this.leftPoints.length}else this.left?A(this,this.left):A(this,this.right);return o}for(s=r.ge(this.leftPoints,t,p);s<this.leftPoints.length&&this.leftPoints[s][0]===t[0];++s)if(this.leftPoints[s]===t)for(this.count-=1,this.leftPoints.splice(s,1),l=r.ge(this.rightPoints,t,I);l<this.rightPoints.length&&this.rightPoints[l][1]===t[1];++l)if(this.rightPoints[l]===t)return this.rightPoints.splice(l,1),o;return i},s.queryPoint=function(t,e){return t<this.mid?this.left&&(n=this.left.queryPoint(t,e))?n:g(this.leftPoints,t,e):t>this.mid?this.right&&(n=this.right.queryPoint(t,e))?n:h(this.rightPoints,t,e):d(this.leftPoints,e);var n},s.queryInterval=function(t,e,n){var r;return t<this.mid&&this.left&&(r=this.left.queryInterval(t,e,n))?r:e>this.mid&&this.right&&(r=this.right.queryInterval(t,e,n))?r:e<this.mid?g(this.leftPoints,e,n):t>this.mid?h(this.rightPoints,t,n):d(this.leftPoints,n)};var y=m.prototype;y.insert=function(t){this.root?this.root.insert(t):this.root=new a(t[0],null,null,[t],[t])},y.remove=function(t){if(this.root){var e=this.root.remove(t);return 2===e&&(this.root=null),e!==i}return!1},y.queryPoint=function(t,e){if(this.root)return this.root.queryPoint(t,e)},y.queryInterval=function(t,e,n){if(t<=e&&this.root)return this.root.queryInterval(t,e,n)},Object.defineProperty(y,"count",{get:function(){return this.root?this.root.count:0}}),Object.defineProperty(y,"intervals",{get:function(){return this.root?this.root.intervals([]):[]}})},{"binary-search-bounds":91}],415:[function(t,e,n){"use strict";e.exports=function(t,e){e=e||new Array(t.length);for(var n=0;n<t.length;++n)e[t[n]]=n;return e}},{}],416:[function(t,e,n){"use strict";e.exports=function(t){for(var e=new Array(t),n=0;n<t;++n)e[n]=n;return e}},{}],417:[function(t,e,n){e.exports=!0},{}],418:[function(t,e,n){function r(t){return!!t.constructor&&"function"==typeof t.constructor.isBuffer&&t.constructor.isBuffer(t)}e.exports=function(t){return null!=t&&(r(t)||function(t){return"function"==typeof t.readFloatLE&&"function"==typeof t.slice&&r(t.slice(0,0))}(t)||!!t._isBuffer)}},{}],419:[function(t,e,n){"use strict";e.exports="undefined"!=typeof navigator&&(/MSIE/.test(navigator.userAgent)||/Trident\//.test(navigator.appVersion))},{}],420:[function(t,e,n){"use strict";e.exports=o,e.exports.isMobile=o;var r=/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i,i=/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino|android|ipad|playbook|silk/i;function o(t){t||(t={});var e=t.ua;return e||"undefined"==typeof navigator||(e=navigator.userAgent),e&&e.headers&&"string"==typeof e.headers["user-agent"]&&(e=e.headers["user-agent"]),"string"==typeof e&&(t.tablet?i.test(e):r.test(e))}},{}],421:[function(t,e,n){"use strict";e.exports=function(t){var e=typeof t;return null!==t&&("object"===e||"function"===e)}},{}],422:[function(t,e,n){"use strict";var r=Object.prototype.toString;e.exports=function(t){var e;return"[object Object]"===r.call(t)&&(null===(e=Object.getPrototypeOf(t))||e===Object.getPrototypeOf({}))}},{}],423:[function(t,e,n){"use strict";e.exports=function(t){for(var e,n=t.length,r=0;r<n;r++)if(((e=t.charCodeAt(r))<9||e>13)&&32!==e&&133!==e&&160!==e&&5760!==e&&6158!==e&&(e<8192||e>8205)&&8232!==e&&8233!==e&&8239!==e&&8287!==e&&8288!==e&&12288!==e&&65279!==e)return!1;return!0}},{}],424:[function(t,e,n){"use strict";e.exports=function(t){return"string"==typeof t&&(t=t.trim(),!!(/^[mzlhvcsqta]\s*[-+.0-9][^mlhvzcsqta]+/i.test(t)&&/[\dz]$/i.test(t)&&t.length>4))}},{}],425:[function(t,e,n){e.exports=function(t,e,n){return t*(1-n)+e*n}},{}],426:[function(t,e,n){var r,i;r=this,i=function(){"use strict";var t,e,n;function r(r,i){if(t)if(e){var o="var sharedChunk = {}; ("+t+")(sharedChunk); ("+e+")(sharedChunk);",a={};t(a),(n=i(a)).workerUrl=window.URL.createObjectURL(new Blob([o],{type:"text/javascript"}))}else e=i;else t=i}return r(0,function(t){function e(t,e){return t(e={exports:{}},e.exports),e.exports}var n=r;function r(t,e,n,r){this.cx=3*t,this.bx=3*(n-t)-this.cx,this.ax=1-this.cx-this.bx,this.cy=3*e,this.by=3*(r-e)-this.cy,this.ay=1-this.cy-this.by,this.p1x=t,this.p1y=r,this.p2x=n,this.p2y=r}r.prototype.sampleCurveX=function(t){return((this.ax*t+this.bx)*t+this.cx)*t},r.prototype.sampleCurveY=function(t){return((this.ay*t+this.by)*t+this.cy)*t},r.prototype.sampleCurveDerivativeX=function(t){return(3*this.ax*t+2*this.bx)*t+this.cx},r.prototype.solveCurveX=function(t,e){var n,r,i,o,a;for(void 0===e&&(e=1e-6),i=t,a=0;a<8;a++){if(o=this.sampleCurveX(i)-t,Math.abs(o)<e)return i;var s=this.sampleCurveDerivativeX(i);if(Math.abs(s)<1e-6)break;i-=o/s}if((i=t)<(n=0))return n;if(i>(r=1))return r;for(;n<r;){if(o=this.sampleCurveX(i),Math.abs(o-t)<e)return i;t>o?n=i:r=i,i=.5*(r-n)+n}return i},r.prototype.solve=function(t,e){return this.sampleCurveY(this.solveCurveX(t,e))};var i=o;function o(t,e){this.x=t,this.y=e}function a(t,e){if(Array.isArray(t)){if(!Array.isArray(e)||t.length!==e.length)return!1;for(var n=0;n<t.length;n++)if(!a(t[n],e[n]))return!1;return!0}if("object"==typeof t&&null!==t&&null!==e){if("object"!=typeof e)return!1;if(Object.keys(t).length!==Object.keys(e).length)return!1;for(var r in t)if(!a(t[r],e[r]))return!1;return!0}return t===e}function s(t,e,r,i){var o=new n(t,e,r,i);return function(t){return o.solve(t)}}o.prototype={clone:function(){return new o(this.x,this.y)},add:function(t){return this.clone()._add(t)},sub:function(t){return this.clone()._sub(t)},multByPoint:function(t){return this.clone()._multByPoint(t)},divByPoint:function(t){return this.clone()._divByPoint(t)},mult:function(t){return this.clone()._mult(t)},div:function(t){return this.clone()._div(t)},rotate:function(t){return this.clone()._rotate(t)},rotateAround:function(t,e){return this.clone()._rotateAround(t,e)},matMult:function(t){return this.clone()._matMult(t)},unit:function(){return this.clone()._unit()},perp:function(){return this.clone()._perp()},round:function(){return this.clone()._round()},mag:function(){return Math.sqrt(this.x*this.x+this.y*this.y)},equals:function(t){return this.x===t.x&&this.y===t.y},dist:function(t){return Math.sqrt(this.distSqr(t))},distSqr:function(t){var e=t.x-this.x,n=t.y-this.y;return e*e+n*n},angle:function(){return Math.atan2(this.y,this.x)},angleTo:function(t){return Math.atan2(this.y-t.y,this.x-t.x)},angleWith:function(t){return this.angleWithSep(t.x,t.y)},angleWithSep:function(t,e){return Math.atan2(this.x*e-this.y*t,this.x*t+this.y*e)},_matMult:function(t){var e=t[0]*this.x+t[1]*this.y,n=t[2]*this.x+t[3]*this.y;return this.x=e,this.y=n,this},_add:function(t){return this.x+=t.x,this.y+=t.y,this},_sub:function(t){return this.x-=t.x,this.y-=t.y,this},_mult:function(t){return this.x*=t,this.y*=t,this},_div:function(t){return this.x/=t,this.y/=t,this},_multByPoint:function(t){return this.x*=t.x,this.y*=t.y,this},_divByPoint:function(t){return this.x/=t.x,this.y/=t.y,this},_unit:function(){return this._div(this.mag()),this},_perp:function(){var t=this.y;return this.y=this.x,this.x=-t,this},_rotate:function(t){var e=Math.cos(t),n=Math.sin(t),r=e*this.x-n*this.y,i=n*this.x+e*this.y;return this.x=r,this.y=i,this},_rotateAround:function(t,e){var n=Math.cos(t),r=Math.sin(t),i=e.x+n*(this.x-e.x)-r*(this.y-e.y),o=e.y+r*(this.x-e.x)+n*(this.y-e.y);return this.x=i,this.y=o,this},_round:function(){return this.x=Math.round(this.x),this.y=Math.round(this.y),this}},o.convert=function(t){return t instanceof o?t:Array.isArray(t)?new o(t[0],t[1]):t};var A=s(.25,.1,.25,1);function l(t,e,n){return Math.min(n,Math.max(e,t))}function c(t,e,n){var r=n-e,i=((t-e)%r+r)%r+e;return i===e?n:i}function u(t){for(var e=[],n=arguments.length-1;n-- >0;)e[n]=arguments[n+1];for(var r=0,i=e;r<i.length;r+=1){var o=i[r];for(var a in o)t[a]=o[a]}return t}var g=1;function h(){return g++}function d(){return function t(e){return e?(e^16*Math.random()>>e/4).toString(16):([1e7]+-[1e3]+-4e3+-8e3+-1e11).replace(/[018]/g,t)}()}function f(t){return!!t&&/^[0-9a-f]{8}-[0-9a-f]{4}-[4][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/i.test(t)}function p(t,e){t.forEach(function(t){e[t]&&(e[t]=e[t].bind(e))})}function I(t,e){return-1!==t.indexOf(e,t.length-e.length)}function C(t,e,n){var r={};for(var i in t)r[i]=e.call(n||this,t[i],i,t);return r}function m(t,e,n){var r={};for(var i in t)e.call(n||this,t[i],i,t)&&(r[i]=t[i]);return r}function y(t){return Array.isArray(t)?t.map(y):"object"==typeof t&&t?C(t,y):t}var E={};function v(t){E[t]||("undefined"!=typeof console&&console.warn(t),E[t]=!0)}function B(t,e,n){return(n.y-t.y)*(e.x-t.x)>(e.y-t.y)*(n.x-t.x)}function b(t){for(var e=0,n=0,r=t.length,i=r-1,o=void 0,a=void 0;n<r;i=n++)o=t[n],e+=((a=t[i]).x-o.x)*(o.y+a.y);return e}function Q(t){var e={};if(t.replace(/(?:^|(?:\s*\,\s*))([^\x00-\x20\(\)<>@\,;\:\\"\/\[\]\?\=\{\}\x7F]+)(?:\=(?:([^\x00-\x20\(\)<>@\,;\:\\"\/\[\]\?\=\{\}\x7F]+)|(?:\"((?:[^"\\]|\\.)*)\")))?/g,function(t,n,r,i){var o=r||i;return e[n]=!o||o.toLowerCase(),""}),e["max-age"]){var n=parseInt(e["max-age"],10);isNaN(n)?delete e["max-age"]:e["max-age"]=n}return e}function w(t){try{var e=self[t];return e.setItem("_mapbox_test_",1),e.removeItem("_mapbox_test_"),!0}catch(t){return!1}}var x,_,D,M=self.performance&&self.performance.now?self.performance.now.bind(self.performance):Date.now.bind(Date),k=self.requestAnimationFrame||self.mozRequestAnimationFrame||self.webkitRequestAnimationFrame||self.msRequestAnimationFrame,S=self.cancelAnimationFrame||self.mozCancelAnimationFrame||self.webkitCancelAnimationFrame||self.msCancelAnimationFrame,L={now:M,frame:function(t){var e=k(t);return{cancel:function(){return S(e)}}},getImageData:function(t){var e=self.document.createElement("canvas"),n=e.getContext("2d");if(!n)throw new Error("failed to create canvas 2d context");return e.width=t.width,e.height=t.height,n.drawImage(t,0,0,t.width,t.height),n.getImageData(0,0,t.width,t.height)},resolveURL:function(t){return x||(x=self.document.createElement("a")),x.href=t,x.href},hardwareConcurrency:self.navigator.hardwareConcurrency||4,get devicePixelRatio(){return self.devicePixelRatio}},R={API_URL:"https://api.mapbox.com",get EVENTS_URL(){return this.API_URL?0===this.API_URL.indexOf("https://api.mapbox.cn")?"https://events.mapbox.cn/events/v2":0===this.API_URL.indexOf("https://api.mapbox.com")?"https://events.mapbox.com/events/v2":null:null},FEEDBACK_URL:"https://apps.mapbox.com/feedback",REQUIRE_ACCESS_TOKEN:!0,ACCESS_TOKEN:null,MAX_PARALLEL_IMAGE_REQUESTS:16},N={supported:!1,testSupport:function(t){!F&&D&&(T?U(t):_=t)}},F=!1,T=!1;function U(t){var e=t.createTexture();t.bindTexture(t.TEXTURE_2D,e);try{if(t.texImage2D(t.TEXTURE_2D,0,t.RGBA,t.RGBA,t.UNSIGNED_BYTE,D),t.isContextLost())return;N.supported=!0}catch(t){}t.deleteTexture(e),F=!0}self.document&&((D=self.document.createElement("img")).onload=function(){_&&U(_),_=null,T=!0},D.onerror=function(){F=!0,_=null},D.src="data:image/webp;base64,UklGRh4AAABXRUJQVlA4TBEAAAAvAQAAAAfQ//73v/+BiOh/AAA=");var O="01",G=function(t){this._transformRequestFn=t,this._createSkuToken()};G.prototype._createSkuToken=function(){var t=function(){for(var t="",e=0;e<10;e++)t+="0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"[Math.floor(62*Math.random())];return{token:["1",O,t].join(""),tokenExpiresAt:Date.now()+432e5}}();this._skuToken=t.token,this._skuTokenExpiresAt=t.tokenExpiresAt},G.prototype._isSkuTokenExpired=function(){return Date.now()>this._skuTokenExpiresAt},G.prototype.transformRequest=function(t,e){return this._transformRequestFn&&this._transformRequestFn(t,e)||{url:t}},G.prototype.normalizeStyleURL=function(t,e){return z(t,e)},G.prototype.normalizeGlyphsURL=function(t,e){return J(t,e)},G.prototype.normalizeSourceURL=function(t,e){return q(t,e)},G.prototype.normalizeSpriteURL=function(t,e,n,r){return V(t,e,n,r)},G.prototype.normalizeTileURL=function(t,e,n){return this._isSkuTokenExpired()&&this._createSkuToken(),X(t,e,n,this._skuToken)},G.prototype.canonicalizeTileURL=function(t){return $(t)},G.prototype.canonicalizeTileset=function(t,e){return tt(t,e)};var H="See https://www.mapbox.com/api-documentation/#access-tokens-and-token-scopes";function j(t,e){var n=nt(R.API_URL);if(t.protocol=n.protocol,t.authority=n.authority,"/"!==n.path&&(t.path=""+n.path+t.path),!R.REQUIRE_ACCESS_TOKEN)return rt(t);if(!(e=e||R.ACCESS_TOKEN))throw new Error("An API access token is required to use Mapbox GL. "+H);if("s"===e[0])throw new Error("Use a public access token (pk.*) with Mapbox GL, not a secret access token (sk.*). "+H);return t.params.push("access_token="+e),rt(t)}function Y(t){return 0===t.indexOf("mapbox:")}var P=/^((https?:)?\/\/)?([^\/]+\.)?mapbox\.c(n|om)(\/|\?|$)/i;function K(t){return P.test(t)}var z=function(t,e){if(!Y(t))return t;var n=nt(t);return n.path="/styles/v1"+n.path,j(n,e)},J=function(t,e){if(!Y(t))return t;var n=nt(t);return n.path="/fonts/v1"+n.path,j(n,e)},q=function(t,e){if(!Y(t))return t;var n=nt(t);return n.path="/v4/"+n.authority+".json",n.params.push("secure"),j(n,e)},V=function(t,e,n,r){var i=nt(t);return Y(t)?(i.path="/styles/v1"+i.path+"/sprite"+e+n,j(i,r)):(i.path+=""+e+n,rt(i))},W=/(\.(png|jpg)\d*)(?=$)/,X=function(t,e,n,r){if(!e||!Y(e))return t;var i=nt(t),o=L.devicePixelRatio>=2||512===n?"@2x":"",a=N.supported?".webp":"$1";return i.path=i.path.replace(W,""+o+a),i.path="/v4"+i.path,R.REQUIRE_ACCESS_TOKEN&&R.ACCESS_TOKEN&&r&&i.params.push("sku="+r),j(i)},Z=/\.[\w]+$/,$=function(t){var e=nt(t);if(!e.path.match(/(^\/v4\/)/)||!e.path.match(Z))return t;var n="mapbox://tiles/";n+=e.path.replace("/v4/","");var r=e.params.filter(function(t){return!t.match(/^access_token=/)});return r.length&&(n+="?"+r.join("&")),n},tt=function(t,e){if(!Y(e))return t.tiles||[];for(var n=[],r=0,i=t.tiles;r<i.length;r+=1){var o=i[r],a=$(o);n.push(a)}return n},et=/^(\w+):\/\/([^\/?]*)(\/[^?]+)?\??(.+)?/;function nt(t){var e=t.match(et);if(!e)throw new Error("Unable to parse URL object");return{protocol:e[1],authority:e[2],path:e[3]||"/",params:e[4]?e[4].split("&"):[]}}function rt(t){var e=t.params.length?"?"+t.params.join("&"):"";return t.protocol+"://"+t.authority+t.path+e}function it(t){if(!t)return null;var e,n=t.split(".");if(!n||3!==n.length)return null;try{return JSON.parse((e=n[1],decodeURIComponent(self.atob(e).split("").map(function(t){return"%"+("00"+t.charCodeAt(0).toString(16)).slice(-2)}).join(""))))}catch(t){return null}}var ot=function(t){this.type=t,this.anonId=null,this.eventData={},this.queue=[],this.pendingRequest=null};ot.prototype.getStorageKey=function(t){var e,n=it(R.ACCESS_TOKEN),r="";return n&&n.u?(e=n.u,r=self.btoa(encodeURIComponent(e).replace(/%([0-9A-F]{2})/g,function(t,e){return String.fromCharCode(Number("0x"+e))}))):r=R.ACCESS_TOKEN||"",t?"mapbox.eventData."+t+":"+r:"mapbox.eventData:"+r},ot.prototype.fetchEventData=function(){var t=w("localStorage"),e=this.getStorageKey(),n=this.getStorageKey("uuid");if(t)try{var r=self.localStorage.getItem(e);r&&(this.eventData=JSON.parse(r));var i=self.localStorage.getItem(n);i&&(this.anonId=i)}catch(t){v("Unable to read from LocalStorage")}},ot.prototype.saveEventData=function(){var t=w("localStorage"),e=this.getStorageKey(),n=this.getStorageKey("uuid");if(t)try{self.localStorage.setItem(n,this.anonId),Object.keys(this.eventData).length>=1&&self.localStorage.setItem(e,JSON.stringify(this.eventData))}catch(t){v("Unable to write to LocalStorage")}},ot.prototype.processRequests=function(){},ot.prototype.postEvent=function(t,e,n){var r=this;if(R.EVENTS_URL){var i=nt(R.EVENTS_URL);i.params.push("access_token="+(R.ACCESS_TOKEN||""));var o={event:this.type,created:new Date(t).toISOString(),sdkIdentifier:"mapbox-gl-js",sdkVersion:"1.1.1",skuId:O,userId:this.anonId},a=e?u(o,e):o,s={url:rt(i),headers:{"Content-Type":"text/plain"},body:JSON.stringify([a])};this.pendingRequest=wt(s,function(t){r.pendingRequest=null,n(t),r.saveEventData(),r.processRequests()})}},ot.prototype.queueRequest=function(t){this.queue.push(t),this.processRequests()};var at,st=function(t){function e(){t.call(this,"map.load"),this.success={},this.skuToken=""}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.postMapLoadEvent=function(t,e,n){this.skuToken=n,R.EVENTS_URL&&R.ACCESS_TOKEN&&Array.isArray(t)&&t.some(function(t){return Y(t)||K(t)})&&this.queueRequest({id:e,timestamp:Date.now()})},e.prototype.processRequests=function(){var t=this;if(!this.pendingRequest&&0!==this.queue.length){var e=this.queue.shift(),n=e.id,r=e.timestamp;n&&this.success[n]||(this.anonId||this.fetchEventData(),f(this.anonId)||(this.anonId=d()),this.postEvent(r,{skuToken:this.skuToken},function(e){e||n&&(t.success[n]=!0)}))}},e}(ot),At=new(function(t){function e(){t.call(this,"appUserTurnstile")}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.postTurnstileEvent=function(t){R.EVENTS_URL&&R.ACCESS_TOKEN&&Array.isArray(t)&&t.some(function(t){return Y(t)||K(t)})&&this.queueRequest(Date.now())},e.prototype.processRequests=function(){var t=this;if(!this.pendingRequest&&0!==this.queue.length){this.anonId&&this.eventData.lastSuccess&&this.eventData.tokenU||this.fetchEventData();var e=it(R.ACCESS_TOKEN),n=e?e.u:R.ACCESS_TOKEN,r=n!==this.eventData.tokenU;f(this.anonId)||(this.anonId=d(),r=!0);var i=this.queue.shift();if(this.eventData.lastSuccess){var o=new Date(this.eventData.lastSuccess),a=new Date(i),s=(i-this.eventData.lastSuccess)/864e5;r=r||s>=1||s<-1||o.getDate()!==a.getDate()}else r=!0;if(!r)return this.processRequests();this.postEvent(i,{"enabled.telemetry":!1},function(e){e||(t.eventData.lastSuccess=i,t.eventData.tokenU=n)})}},e}(ot)),lt=At.postTurnstileEvent.bind(At),ct=new st,ut=ct.postMapLoadEvent.bind(ct),gt="mapbox-tiles",ht=500,dt=50,ft=42e4;function pt(t){var e=t.indexOf("?");return e<0?t:t.slice(0,e)}var It=1/0,Ct={Unknown:"Unknown",Style:"Style",Source:"Source",Tile:"Tile",Glyphs:"Glyphs",SpriteImage:"SpriteImage",SpriteJSON:"SpriteJSON",Image:"Image"};"function"==typeof Object.freeze&&Object.freeze(Ct);var mt=function(t){function e(e,n,r){401===n&&K(r)&&(e+=": you may have provided an invalid Mapbox access token. See https://www.mapbox.com/api-documentation/#access-tokens-and-token-scopes"),t.call(this,e),this.status=n,this.url=r,this.name=this.constructor.name,this.message=e}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.toString=function(){return this.name+": "+this.message+" ("+this.status+"): "+this.url},e}(Error);function yt(){return"undefined"!=typeof WorkerGlobalScope&&"undefined"!=typeof self&&self instanceof WorkerGlobalScope}var Et,vt,Bt=yt()?function(){return self.worker&&self.worker.referrer}:function(){var t=self.location.origin;if(t&&"null"!==t&&"file://"!==t)return t+self.location.pathname},bt=function(t,e){if(!/^file:/.test(t.url)){if(self.fetch&&self.Request&&self.AbortController&&self.Request.prototype.hasOwnProperty("signal"))return function(t,e){var n,r=new self.AbortController,i=new self.Request(t.url,{method:t.method||"GET",body:t.body,credentials:t.credentials,headers:t.headers,referrer:Bt(),signal:r.signal}),o=!1,a=!1,s=(n=i.url).indexOf("sku=")>0&&K(n);"json"===t.type&&i.headers.set("Accept","application/json");var A=function(n,r,o){if(!a){if(n&&"SecurityError"!==n.message&&v(n),r&&o)return l(r);var A=Date.now();self.fetch(i).then(function(n){if(n.ok){var r=s?n.clone():null;return l(n,r,A)}return e(new mt(n.statusText,n.status,t.url))}).catch(function(t){20!==t.code&&e(new Error(t.message))})}},l=function(n,r,s){("arrayBuffer"===t.type?n.arrayBuffer():"json"===t.type?n.json():n.text()).then(function(t){a||(r&&s&&function(t,e,n){if(self.caches){var r={status:e.status,statusText:e.statusText,headers:new self.Headers};e.headers.forEach(function(t,e){return r.headers.set(e,t)});var i=Q(e.headers.get("Cache-Control")||"");i["no-store"]||(i["max-age"]&&r.headers.set("Expires",new Date(n+1e3*i["max-age"]).toUTCString()),new Date(r.headers.get("Expires")).getTime()-n<ft||function(t,e){if(void 0===at)try{new Response(new ReadableStream),at=!0}catch(t){at=!1}at?e(t.body):t.blob().then(e)}(e,function(e){var n=new self.Response(e,r);self.caches.open(gt).then(function(e){return e.put(pt(t.url),n)})}))}}(i,r,s),o=!0,e(null,t,n.headers.get("Cache-Control"),n.headers.get("Expires")))}).catch(function(t){return e(new Error(t.message))})};return s?function(t,e){if(!self.caches)return e(null);var n=pt(t.url);self.caches.open(gt).catch(e).then(function(t){t.match(n).catch(e).then(function(r){var i=function(t){if(!t)return!1;var e=new Date(t.headers.get("Expires")),n=Q(t.headers.get("Cache-Control")||"");return e>Date.now()&&!n["no-cache"]}(r);t.delete(n),i&&t.put(n,r.clone()),e(null,r,i)})})}(i,A):A(null,null),{cancel:function(){a=!0,o||r.abort()}}}(t,e);if(yt()&&self.worker&&self.worker.actor)return self.worker.actor.send("getResource",t,e)}return function(t,e){var n=new self.XMLHttpRequest;for(var r in n.open(t.method||"GET",t.url,!0),"arrayBuffer"===t.type&&(n.responseType="arraybuffer"),t.headers)n.setRequestHeader(r,t.headers[r]);return"json"===t.type&&n.setRequestHeader("Accept","application/json"),n.withCredentials="include"===t.credentials,n.onerror=function(){e(new Error(n.statusText))},n.onload=function(){if((n.status>=200&&n.status<300||0===n.status)&&null!==n.response){var r=n.response;if("json"===t.type)try{r=JSON.parse(n.response)}catch(t){return e(t)}e(null,r,n.getResponseHeader("Cache-Control"),n.getResponseHeader("Expires"))}else e(new mt(n.statusText,n.status,t.url))},n.send(t.body),{cancel:function(){return n.abort()}}}(t,e)},Qt=function(t,e){return bt(u(t,{type:"arrayBuffer"}),e)},wt=function(t,e){return bt(u(t,{method:"POST"}),e)};Et=[],vt=0;var xt=function(t,e){if(vt>=R.MAX_PARALLEL_IMAGE_REQUESTS){var n={requestParameters:t,callback:e,cancelled:!1,cancel:function(){this.cancelled=!0}};return Et.push(n),n}vt++;var r=!1,i=function(){if(!r)for(r=!0,vt--;Et.length&&vt<R.MAX_PARALLEL_IMAGE_REQUESTS;){var t=Et.shift(),e=t.requestParameters,n=t.callback;t.cancelled||(t.cancel=xt(e,n).cancel)}},o=Qt(t,function(t,n,r,o){if(i(),t)e(t);else if(n){var a=new self.Image,s=self.URL||self.webkitURL;a.onload=function(){e(null,a),s.revokeObjectURL(a.src)},a.onerror=function(){return e(new Error("Could not load image. Please make sure to use a supported image type such as PNG or JPEG. Note that SVGs are not supported."))};var A=new self.Blob([new Uint8Array(n)],{type:"image/png"});a.cacheControl=r,a.expires=o,a.src=n.byteLength?s.createObjectURL(A):"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAAC0lEQVQYV2NgAAIAAAUAAarVyFEAAAAASUVORK5CYII="}});return{cancel:function(){o.cancel(),i()}}};function _t(t,e,n){n[t]&&-1!==n[t].indexOf(e)||(n[t]=n[t]||[],n[t].push(e))}function Dt(t,e,n){if(n&&n[t]){var r=n[t].indexOf(e);-1!==r&&n[t].splice(r,1)}}var Mt=function(t,e){void 0===e&&(e={}),u(this,e),this.type=t},kt=function(t){function e(e,n){void 0===n&&(n={}),t.call(this,"error",u({error:e},n))}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e}(Mt),St=function(){};St.prototype.on=function(t,e){return this._listeners=this._listeners||{},_t(t,e,this._listeners),this},St.prototype.off=function(t,e){return Dt(t,e,this._listeners),Dt(t,e,this._oneTimeListeners),this},St.prototype.once=function(t,e){return this._oneTimeListeners=this._oneTimeListeners||{},_t(t,e,this._oneTimeListeners),this},St.prototype.fire=function(t,e){"string"==typeof t&&(t=new Mt(t,e||{}));var n=t.type;if(this.listens(n)){t.target=this;for(var r=0,i=this._listeners&&this._listeners[n]?this._listeners[n].slice():[];r<i.length;r+=1)i[r].call(this,t);for(var o=0,a=this._oneTimeListeners&&this._oneTimeListeners[n]?this._oneTimeListeners[n].slice():[];o<a.length;o+=1){var s=a[o];Dt(n,s,this._oneTimeListeners),s.call(this,t)}var A=this._eventedParent;A&&(u(t,"function"==typeof this._eventedParentData?this._eventedParentData():this._eventedParentData),A.fire(t))}else t instanceof kt&&console.error(t.error);return this},St.prototype.listens=function(t){return this._listeners&&this._listeners[t]&&this._listeners[t].length>0||this._oneTimeListeners&&this._oneTimeListeners[t]&&this._oneTimeListeners[t].length>0||this._eventedParent&&this._eventedParent.listens(t)},St.prototype.setEventedParent=function(t,e){return this._eventedParent=t,this._eventedParentData=e,this};var Lt={$version:8,$root:{version:{required:!0,type:"enum",values:[8]},name:{type:"string"},metadata:{type:"*"},center:{type:"array",value:"number"},zoom:{type:"number"},bearing:{type:"number",default:0,period:360,units:"degrees"},pitch:{type:"number",default:0,units:"degrees"},light:{type:"light"},sources:{required:!0,type:"sources"},sprite:{type:"string"},glyphs:{type:"string"},transition:{type:"transition"},layers:{required:!0,type:"array",value:"layer"}},sources:{"*":{type:"source"}},source:["source_vector","source_raster","source_raster_dem","source_geojson","source_video","source_image"],source_vector:{type:{required:!0,type:"enum",values:{vector:{}}},url:{type:"string"},tiles:{type:"array",value:"string"},bounds:{type:"array",value:"number",length:4,default:[-180,-85.051129,180,85.051129]},scheme:{type:"enum",values:{xyz:{},tms:{}},default:"xyz"},minzoom:{type:"number",default:0},maxzoom:{type:"number",default:22},attribution:{type:"string"},"*":{type:"*"}},source_raster:{type:{required:!0,type:"enum",values:{raster:{}}},url:{type:"string"},tiles:{type:"array",value:"string"},bounds:{type:"array",value:"number",length:4,default:[-180,-85.051129,180,85.051129]},minzoom:{type:"number",default:0},maxzoom:{type:"number",default:22},tileSize:{type:"number",default:512,units:"pixels"},scheme:{type:"enum",values:{xyz:{},tms:{}},default:"xyz"},attribution:{type:"string"},"*":{type:"*"}},source_raster_dem:{type:{required:!0,type:"enum",values:{"raster-dem":{}}},url:{type:"string"},tiles:{type:"array",value:"string"},bounds:{type:"array",value:"number",length:4,default:[-180,-85.051129,180,85.051129]},minzoom:{type:"number",default:0},maxzoom:{type:"number",default:22},tileSize:{type:"number",default:512,units:"pixels"},attribution:{type:"string"},encoding:{type:"enum",values:{terrarium:{},mapbox:{}},default:"mapbox"},"*":{type:"*"}},source_geojson:{type:{required:!0,type:"enum",values:{geojson:{}}},data:{type:"*"},maxzoom:{type:"number",default:18},attribution:{type:"string"},buffer:{type:"number",default:128,maximum:512,minimum:0},tolerance:{type:"number",default:.375},cluster:{type:"boolean",default:!1},clusterRadius:{type:"number",default:50,minimum:0},clusterMaxZoom:{type:"number"},clusterProperties:{type:"*"},lineMetrics:{type:"boolean",default:!1},generateId:{type:"boolean",default:!1}},source_video:{type:{required:!0,type:"enum",values:{video:{}}},urls:{required:!0,type:"array",value:"string"},coordinates:{required:!0,type:"array",length:4,value:{type:"array",length:2,value:"number"}}},source_image:{type:{required:!0,type:"enum",values:{image:{}}},url:{required:!0,type:"string"},coordinates:{required:!0,type:"array",length:4,value:{type:"array",length:2,value:"number"}}},layer:{id:{type:"string",required:!0},type:{type:"enum",values:{fill:{},line:{},symbol:{},circle:{},heatmap:{},"fill-extrusion":{},raster:{},hillshade:{},background:{}},required:!0},metadata:{type:"*"},source:{type:"string"},"source-layer":{type:"string"},minzoom:{type:"number",minimum:0,maximum:24},maxzoom:{type:"number",minimum:0,maximum:24},filter:{type:"filter"},layout:{type:"layout"},paint:{type:"paint"}},layout:["layout_fill","layout_line","layout_circle","layout_heatmap","layout_fill-extrusion","layout_symbol","layout_raster","layout_hillshade","layout_background"],layout_background:{visibility:{type:"enum",values:{visible:{},none:{}},default:"visible","property-type":"constant"}},layout_fill:{visibility:{type:"enum",values:{visible:{},none:{}},default:"visible","property-type":"constant"}},layout_circle:{visibility:{type:"enum",values:{visible:{},none:{}},default:"visible","property-type":"constant"}},layout_heatmap:{visibility:{type:"enum",values:{visible:{},none:{}},default:"visible","property-type":"constant"}},"layout_fill-extrusion":{visibility:{type:"enum",values:{visible:{},none:{}},default:"visible","property-type":"constant"}},layout_line:{"line-cap":{type:"enum",values:{butt:{},round:{},square:{}},default:"butt",expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"line-join":{type:"enum",values:{bevel:{},round:{},miter:{}},default:"miter",expression:{interpolated:!1,parameters:["zoom","feature"]},"property-type":"data-driven"},"line-miter-limit":{type:"number",default:2,requires:[{"line-join":"miter"}],expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"line-round-limit":{type:"number",default:1.05,requires:[{"line-join":"round"}],expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},visibility:{type:"enum",values:{visible:{},none:{}},default:"visible","property-type":"constant"}},layout_symbol:{"symbol-placement":{type:"enum",values:{point:{},line:{},"line-center":{}},default:"point",expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"symbol-spacing":{type:"number",default:250,minimum:1,units:"pixels",requires:[{"symbol-placement":"line"}],expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"symbol-avoid-edges":{type:"boolean",default:!1,expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"symbol-sort-key":{type:"number",expression:{interpolated:!1,parameters:["zoom","feature"]},"property-type":"data-driven"},"symbol-z-order":{type:"enum",values:{auto:{},"viewport-y":{},source:{}},default:"auto",expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"icon-allow-overlap":{type:"boolean",default:!1,requires:["icon-image"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"icon-ignore-placement":{type:"boolean",default:!1,requires:["icon-image"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"icon-optional":{type:"boolean",default:!1,requires:["icon-image","text-field"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"icon-rotation-alignment":{type:"enum",values:{map:{},viewport:{},auto:{}},default:"auto",requires:["icon-image"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"icon-size":{type:"number",default:1,minimum:0,units:"factor of the original icon size",requires:["icon-image"],expression:{interpolated:!0,parameters:["zoom","feature"]},"property-type":"data-driven"},"icon-text-fit":{type:"enum",values:{none:{},width:{},height:{},both:{}},default:"none",requires:["icon-image","text-field"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"icon-text-fit-padding":{type:"array",value:"number",length:4,default:[0,0,0,0],units:"pixels",requires:["icon-image","text-field",{"icon-text-fit":["both","width","height"]}],expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"icon-image":{type:"string",tokens:!0,expression:{interpolated:!1,parameters:["zoom","feature"]},"property-type":"data-driven"},"icon-rotate":{type:"number",default:0,period:360,units:"degrees",requires:["icon-image"],expression:{interpolated:!0,parameters:["zoom","feature"]},"property-type":"data-driven"},"icon-padding":{type:"number",default:2,minimum:0,units:"pixels",requires:["icon-image"],expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"icon-keep-upright":{type:"boolean",default:!1,requires:["icon-image",{"icon-rotation-alignment":"map"},{"symbol-placement":["line","line-center"]}],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"icon-offset":{type:"array",value:"number",length:2,default:[0,0],requires:["icon-image"],expression:{interpolated:!0,parameters:["zoom","feature"]},"property-type":"data-driven"},"icon-anchor":{type:"enum",values:{center:{},left:{},right:{},top:{},bottom:{},"top-left":{},"top-right":{},"bottom-left":{},"bottom-right":{}},default:"center",requires:["icon-image"],expression:{interpolated:!1,parameters:["zoom","feature"]},"property-type":"data-driven"},"icon-pitch-alignment":{type:"enum",values:{map:{},viewport:{},auto:{}},default:"auto",requires:["icon-image"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"text-pitch-alignment":{type:"enum",values:{map:{},viewport:{},auto:{}},default:"auto",requires:["text-field"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"text-rotation-alignment":{type:"enum",values:{map:{},viewport:{},auto:{}},default:"auto",requires:["text-field"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"text-field":{type:"formatted",default:"",tokens:!0,expression:{interpolated:!1,parameters:["zoom","feature"]},"property-type":"data-driven"},"text-font":{type:"array",value:"string",default:["Open Sans Regular","Arial Unicode MS Regular"],requires:["text-field"],expression:{interpolated:!1,parameters:["zoom","feature"]},"property-type":"data-driven"},"text-size":{type:"number",default:16,minimum:0,units:"pixels",requires:["text-field"],expression:{interpolated:!0,parameters:["zoom","feature"]},"property-type":"data-driven"},"text-max-width":{type:"number",default:10,minimum:0,units:"ems",requires:["text-field"],expression:{interpolated:!0,parameters:["zoom","feature"]},"property-type":"data-driven"},"text-line-height":{type:"number",default:1.2,units:"ems",requires:["text-field"],expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"text-letter-spacing":{type:"number",default:0,units:"ems",requires:["text-field"],expression:{interpolated:!0,parameters:["zoom","feature"]},"property-type":"data-driven"},"text-justify":{type:"enum",values:{auto:{},left:{},center:{},right:{}},default:"center",requires:["text-field"],expression:{interpolated:!1,parameters:["zoom","feature"]},"property-type":"data-driven"},"text-radial-offset":{type:"number",units:"ems",default:0,requires:[{"!":"text-offset"}],"property-type":"data-driven",expression:{interpolated:!0,parameters:["zoom","feature"]}},"text-variable-anchor":{type:"array",value:"enum",values:{center:{},left:{},right:{},top:{},bottom:{},"top-left":{},"top-right":{},"bottom-left":{},"bottom-right":{}},requires:[{"!":"text-anchor"},{"!":"text-offset"},{"symbol-placement":["point"]}],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"text-anchor":{type:"enum",requires:["text-field"],values:{center:{},left:{},right:{},top:{},bottom:{},"top-left":{},"top-right":{},"bottom-left":{},"bottom-right":{}},default:"center",expression:{interpolated:!1,parameters:["zoom","feature"]},"property-type":"data-driven"},"text-max-angle":{type:"number",default:45,units:"degrees",requires:["text-field",{"symbol-placement":["line","line-center"]}],expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"text-rotate":{type:"number",default:0,period:360,units:"degrees",requires:["text-field"],expression:{interpolated:!0,parameters:["zoom","feature"]},"property-type":"data-driven"},"text-padding":{type:"number",default:2,minimum:0,units:"pixels",requires:["text-field"],expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"text-keep-upright":{type:"boolean",default:!0,requires:["text-field",{"text-rotation-alignment":"map"},{"symbol-placement":["line","line-center"]}],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"text-transform":{type:"enum",values:{none:{},uppercase:{},lowercase:{}},default:"none",requires:["text-field"],expression:{interpolated:!1,parameters:["zoom","feature"]},"property-type":"data-driven"},"text-offset":{type:"array",value:"number",units:"ems",length:2,default:[0,0],requires:["text-field",{"!":"text-radial-offset"}],expression:{interpolated:!0,parameters:["zoom","feature"]},"property-type":"data-driven"},"text-allow-overlap":{type:"boolean",default:!1,requires:["text-field"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"text-ignore-placement":{type:"boolean",default:!1,requires:["text-field"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"text-optional":{type:"boolean",default:!1,requires:["text-field","icon-image"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},visibility:{type:"enum",values:{visible:{},none:{}},default:"visible","property-type":"constant"}},layout_raster:{visibility:{type:"enum",values:{visible:{},none:{}},default:"visible","property-type":"constant"}},layout_hillshade:{visibility:{type:"enum",values:{visible:{},none:{}},default:"visible","property-type":"constant"}},filter:{type:"array",value:"*"},filter_operator:{type:"enum",values:{"==":{},"!=":{},">":{},">=":{},"<":{},"<=":{},in:{},"!in":{},all:{},any:{},none:{},has:{},"!has":{}}},geometry_type:{type:"enum",values:{Point:{},LineString:{},Polygon:{}}},function:{expression:{type:"expression"},stops:{type:"array",value:"function_stop"},base:{type:"number",default:1,minimum:0},property:{type:"string",default:"$zoom"},type:{type:"enum",values:{identity:{},exponential:{},interval:{},categorical:{}},default:"exponential"},colorSpace:{type:"enum",values:{rgb:{},lab:{},hcl:{}},default:"rgb"},default:{type:"*",required:!1}},function_stop:{type:"array",minimum:0,maximum:22,value:["number","color"],length:2},expression:{type:"array",value:"*",minimum:1},expression_name:{type:"enum",values:{let:{group:"Variable binding"},var:{group:"Variable binding"},literal:{group:"Types"},array:{group:"Types"},at:{group:"Lookup"},case:{group:"Decision"},match:{group:"Decision"},coalesce:{group:"Decision"},step:{group:"Ramps, scales, curves"},interpolate:{group:"Ramps, scales, curves"},"interpolate-hcl":{group:"Ramps, scales, curves"},"interpolate-lab":{group:"Ramps, scales, curves"},ln2:{group:"Math"},pi:{group:"Math"},e:{group:"Math"},typeof:{group:"Types"},string:{group:"Types"},number:{group:"Types"},boolean:{group:"Types"},object:{group:"Types"},collator:{group:"Types"},format:{group:"Types"},"number-format":{group:"Types"},"to-string":{group:"Types"},"to-number":{group:"Types"},"to-boolean":{group:"Types"},"to-rgba":{group:"Color"},"to-color":{group:"Types"},rgb:{group:"Color"},rgba:{group:"Color"},get:{group:"Lookup"},has:{group:"Lookup"},length:{group:"Lookup"},properties:{group:"Feature data"},"feature-state":{group:"Feature data"},"geometry-type":{group:"Feature data"},id:{group:"Feature data"},zoom:{group:"Zoom"},"heatmap-density":{group:"Heatmap"},"line-progress":{group:"Feature data"},accumulated:{group:"Feature data"},"+":{group:"Math"},"*":{group:"Math"},"-":{group:"Math"},"/":{group:"Math"},"%":{group:"Math"},"^":{group:"Math"},sqrt:{group:"Math"},log10:{group:"Math"},ln:{group:"Math"},log2:{group:"Math"},sin:{group:"Math"},cos:{group:"Math"},tan:{group:"Math"},asin:{group:"Math"},acos:{group:"Math"},atan:{group:"Math"},min:{group:"Math"},max:{group:"Math"},round:{group:"Math"},abs:{group:"Math"},ceil:{group:"Math"},floor:{group:"Math"},"==":{group:"Decision"},"!=":{group:"Decision"},">":{group:"Decision"},"<":{group:"Decision"},">=":{group:"Decision"},"<=":{group:"Decision"},all:{group:"Decision"},any:{group:"Decision"},"!":{group:"Decision"},"is-supported-script":{group:"String"},upcase:{group:"String"},downcase:{group:"String"},concat:{group:"String"},"resolved-locale":{group:"String"}}},light:{anchor:{type:"enum",default:"viewport",values:{map:{},viewport:{}},"property-type":"data-constant",transition:!1,expression:{interpolated:!1,parameters:["zoom"]}},position:{type:"array",default:[1.15,210,30],length:3,value:"number","property-type":"data-constant",transition:!0,expression:{interpolated:!0,parameters:["zoom"]}},color:{type:"color","property-type":"data-constant",default:"#ffffff",expression:{interpolated:!0,parameters:["zoom"]},transition:!0},intensity:{type:"number","property-type":"data-constant",default:.5,minimum:0,maximum:1,expression:{interpolated:!0,parameters:["zoom"]},transition:!0}},paint:["paint_fill","paint_line","paint_circle","paint_heatmap","paint_fill-extrusion","paint_symbol","paint_raster","paint_hillshade","paint_background"],paint_fill:{"fill-antialias":{type:"boolean",default:!0,expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"fill-opacity":{type:"number",default:1,minimum:0,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"fill-color":{type:"color",default:"#000000",transition:!0,requires:[{"!":"fill-pattern"}],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"fill-outline-color":{type:"color",transition:!0,requires:[{"!":"fill-pattern"},{"fill-antialias":!0}],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"fill-translate":{type:"array",value:"number",length:2,default:[0,0],transition:!0,units:"pixels",expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"fill-translate-anchor":{type:"enum",values:{map:{},viewport:{}},default:"map",requires:["fill-translate"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"fill-pattern":{type:"string",transition:!0,expression:{interpolated:!1,parameters:["zoom","feature"]},"property-type":"cross-faded-data-driven"}},"paint_fill-extrusion":{"fill-extrusion-opacity":{type:"number",default:1,minimum:0,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"fill-extrusion-color":{type:"color",default:"#000000",transition:!0,requires:[{"!":"fill-extrusion-pattern"}],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"fill-extrusion-translate":{type:"array",value:"number",length:2,default:[0,0],transition:!0,units:"pixels",expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"fill-extrusion-translate-anchor":{type:"enum",values:{map:{},viewport:{}},default:"map",requires:["fill-extrusion-translate"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"fill-extrusion-pattern":{type:"string",transition:!0,expression:{interpolated:!1,parameters:["zoom","feature"]},"property-type":"cross-faded-data-driven"},"fill-extrusion-height":{type:"number",default:0,minimum:0,units:"meters",transition:!0,expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"fill-extrusion-base":{type:"number",default:0,minimum:0,units:"meters",transition:!0,requires:["fill-extrusion-height"],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"fill-extrusion-vertical-gradient":{type:"boolean",default:!0,transition:!1,expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"}},paint_line:{"line-opacity":{type:"number",default:1,minimum:0,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"line-color":{type:"color",default:"#000000",transition:!0,requires:[{"!":"line-pattern"}],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"line-translate":{type:"array",value:"number",length:2,default:[0,0],transition:!0,units:"pixels",expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"line-translate-anchor":{type:"enum",values:{map:{},viewport:{}},default:"map",requires:["line-translate"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"line-width":{type:"number",default:1,minimum:0,transition:!0,units:"pixels",expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"line-gap-width":{type:"number",default:0,minimum:0,transition:!0,units:"pixels",expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"line-offset":{type:"number",default:0,transition:!0,units:"pixels",expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"line-blur":{type:"number",default:0,minimum:0,transition:!0,units:"pixels",expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"line-dasharray":{type:"array",value:"number",minimum:0,transition:!0,units:"line widths",requires:[{"!":"line-pattern"}],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"cross-faded"},"line-pattern":{type:"string",transition:!0,expression:{interpolated:!1,parameters:["zoom","feature"]},"property-type":"cross-faded-data-driven"},"line-gradient":{type:"color",transition:!1,requires:[{"!":"line-dasharray"},{"!":"line-pattern"},{source:"geojson",has:{lineMetrics:!0}}],expression:{interpolated:!0,parameters:["line-progress"]},"property-type":"color-ramp"}},paint_circle:{"circle-radius":{type:"number",default:5,minimum:0,transition:!0,units:"pixels",expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"circle-color":{type:"color",default:"#000000",transition:!0,expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"circle-blur":{type:"number",default:0,transition:!0,expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"circle-opacity":{type:"number",default:1,minimum:0,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"circle-translate":{type:"array",value:"number",length:2,default:[0,0],transition:!0,units:"pixels",expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"circle-translate-anchor":{type:"enum",values:{map:{},viewport:{}},default:"map",requires:["circle-translate"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"circle-pitch-scale":{type:"enum",values:{map:{},viewport:{}},default:"map",expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"circle-pitch-alignment":{type:"enum",values:{map:{},viewport:{}},default:"viewport",expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"circle-stroke-width":{type:"number",default:0,minimum:0,transition:!0,units:"pixels",expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"circle-stroke-color":{type:"color",default:"#000000",transition:!0,expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"circle-stroke-opacity":{type:"number",default:1,minimum:0,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"}},paint_heatmap:{"heatmap-radius":{type:"number",default:30,minimum:1,transition:!0,units:"pixels",expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"heatmap-weight":{type:"number",default:1,minimum:0,transition:!1,expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"heatmap-intensity":{type:"number",default:1,minimum:0,transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"heatmap-color":{type:"color",default:["interpolate",["linear"],["heatmap-density"],0,"rgba(0, 0, 255, 0)",.1,"royalblue",.3,"cyan",.5,"lime",.7,"yellow",1,"red"],transition:!1,expression:{interpolated:!0,parameters:["heatmap-density"]},"property-type":"color-ramp"},"heatmap-opacity":{type:"number",default:1,minimum:0,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"}},paint_symbol:{"icon-opacity":{type:"number",default:1,minimum:0,maximum:1,transition:!0,requires:["icon-image"],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"icon-color":{type:"color",default:"#000000",transition:!0,requires:["icon-image"],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"icon-halo-color":{type:"color",default:"rgba(0, 0, 0, 0)",transition:!0,requires:["icon-image"],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"icon-halo-width":{type:"number",default:0,minimum:0,transition:!0,units:"pixels",requires:["icon-image"],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"icon-halo-blur":{type:"number",default:0,minimum:0,transition:!0,units:"pixels",requires:["icon-image"],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"icon-translate":{type:"array",value:"number",length:2,default:[0,0],transition:!0,units:"pixels",requires:["icon-image"],expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"icon-translate-anchor":{type:"enum",values:{map:{},viewport:{}},default:"map",requires:["icon-image","icon-translate"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"text-opacity":{type:"number",default:1,minimum:0,maximum:1,transition:!0,requires:["text-field"],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"text-color":{type:"color",default:"#000000",transition:!0,requires:["text-field"],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"text-halo-color":{type:"color",default:"rgba(0, 0, 0, 0)",transition:!0,requires:["text-field"],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"text-halo-width":{type:"number",default:0,minimum:0,transition:!0,units:"pixels",requires:["text-field"],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"text-halo-blur":{type:"number",default:0,minimum:0,transition:!0,units:"pixels",requires:["text-field"],expression:{interpolated:!0,parameters:["zoom","feature","feature-state"]},"property-type":"data-driven"},"text-translate":{type:"array",value:"number",length:2,default:[0,0],transition:!0,units:"pixels",requires:["text-field"],expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"text-translate-anchor":{type:"enum",values:{map:{},viewport:{}},default:"map",requires:["text-field","text-translate"],expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"}},paint_raster:{"raster-opacity":{type:"number",default:1,minimum:0,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"raster-hue-rotate":{type:"number",default:0,period:360,transition:!0,units:"degrees",expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"raster-brightness-min":{type:"number",default:0,minimum:0,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"raster-brightness-max":{type:"number",default:1,minimum:0,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"raster-saturation":{type:"number",default:0,minimum:-1,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"raster-contrast":{type:"number",default:0,minimum:-1,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"raster-resampling":{type:"enum",values:{linear:{},nearest:{}},default:"linear",expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"raster-fade-duration":{type:"number",default:300,minimum:0,transition:!1,units:"milliseconds",expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"}},paint_hillshade:{"hillshade-illumination-direction":{type:"number",default:335,minimum:0,maximum:359,transition:!1,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"hillshade-illumination-anchor":{type:"enum",values:{map:{},viewport:{}},default:"viewport",expression:{interpolated:!1,parameters:["zoom"]},"property-type":"data-constant"},"hillshade-exaggeration":{type:"number",default:.5,minimum:0,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"hillshade-shadow-color":{type:"color",default:"#000000",transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"hillshade-highlight-color":{type:"color",default:"#FFFFFF",transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"hillshade-accent-color":{type:"color",default:"#000000",transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"}},paint_background:{"background-color":{type:"color",default:"#000000",transition:!0,requires:[{"!":"background-pattern"}],expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"},"background-pattern":{type:"string",transition:!0,expression:{interpolated:!1,parameters:["zoom"]},"property-type":"cross-faded"},"background-opacity":{type:"number",default:1,minimum:0,maximum:1,transition:!0,expression:{interpolated:!0,parameters:["zoom"]},"property-type":"data-constant"}},transition:{duration:{type:"number",default:300,minimum:0,units:"milliseconds"},delay:{type:"number",default:0,minimum:0,units:"milliseconds"}},"property-type":{"data-driven":{type:"property-type"},"cross-faded":{type:"property-type"},"cross-faded-data-driven":{type:"property-type"},"color-ramp":{type:"property-type"},"data-constant":{type:"property-type"},constant:{type:"property-type"}}},Rt=function(t,e,n,r){this.message=(t?t+": ":"")+n,r&&(this.identifier=r),null!=e&&e.__line__&&(this.line=e.__line__)};function Nt(t){var e=t.key,n=t.value;return n?[new Rt(e,n,"constants have been deprecated as of v8")]:[]}function Ft(t){for(var e=[],n=arguments.length-1;n-- >0;)e[n]=arguments[n+1];for(var r=0,i=e;r<i.length;r+=1){var o=i[r];for(var a in o)t[a]=o[a]}return t}function Tt(t){return t instanceof Number||t instanceof String||t instanceof Boolean}function Ut(t){return Tt(t)?t.valueOf():t}function Ot(t){if(Array.isArray(t))return t.map(Ot);if(t instanceof Object&&!Tt(t)){var e={};for(var n in t)e[n]=Ot(t[n]);return e}return Ut(t)}var Gt=function(t){function e(e,n){t.call(this,n),this.message=n,this.key=e}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e}(Error),Ht=function(t,e){void 0===e&&(e=[]),this.parent=t,this.bindings={};for(var n=0,r=e;n<r.length;n+=1){var i=r[n],o=i[0],a=i[1];this.bindings[o]=a}};Ht.prototype.concat=function(t){return new Ht(this,t)},Ht.prototype.get=function(t){if(this.bindings[t])return this.bindings[t];if(this.parent)return this.parent.get(t);throw new Error(t+" not found in scope.")},Ht.prototype.has=function(t){return!!this.bindings[t]||!!this.parent&&this.parent.has(t)};var jt={kind:"null"},Yt={kind:"number"},Pt={kind:"string"},Kt={kind:"boolean"},zt={kind:"color"},Jt={kind:"object"},qt={kind:"value"},Vt={kind:"collator"},Wt={kind:"formatted"};function Xt(t,e){return{kind:"array",itemType:t,N:e}}function Zt(t){if("array"===t.kind){var e=Zt(t.itemType);return"number"==typeof t.N?"array<"+e+", "+t.N+">":"value"===t.itemType.kind?"array":"array<"+e+">"}return t.kind}var $t=[jt,Yt,Pt,Kt,zt,Wt,Jt,Xt(qt)];function te(t,e){if("error"===e.kind)return null;if("array"===t.kind){if("array"===e.kind&&(0===e.N&&"value"===e.itemType.kind||!te(t.itemType,e.itemType))&&("number"!=typeof t.N||t.N===e.N))return null}else{if(t.kind===e.kind)return null;if("value"===t.kind)for(var n=0,r=$t;n<r.length;n+=1)if(!te(r[n],e))return null}return"Expected "+Zt(t)+" but found "+Zt(e)+" instead."}var ee=e(function(t,e){var n={transparent:[0,0,0,0],aliceblue:[240,248,255,1],antiquewhite:[250,235,215,1],aqua:[0,255,255,1],aquamarine:[127,255,212,1],azure:[240,255,255,1],beige:[245,245,220,1],bisque:[255,228,196,1],black:[0,0,0,1],blanchedalmond:[255,235,205,1],blue:[0,0,255,1],blueviolet:[138,43,226,1],brown:[165,42,42,1],burlywood:[222,184,135,1],cadetblue:[95,158,160,1],chartreuse:[127,255,0,1],chocolate:[210,105,30,1],coral:[255,127,80,1],cornflowerblue:[100,149,237,1],cornsilk:[255,248,220,1],crimson:[220,20,60,1],cyan:[0,255,255,1],darkblue:[0,0,139,1],darkcyan:[0,139,139,1],darkgoldenrod:[184,134,11,1],darkgray:[169,169,169,1],darkgreen:[0,100,0,1],darkgrey:[169,169,169,1],darkkhaki:[189,183,107,1],darkmagenta:[139,0,139,1],darkolivegreen:[85,107,47,1],darkorange:[255,140,0,1],darkorchid:[153,50,204,1],darkred:[139,0,0,1],darksalmon:[233,150,122,1],darkseagreen:[143,188,143,1],darkslateblue:[72,61,139,1],darkslategray:[47,79,79,1],darkslategrey:[47,79,79,1],darkturquoise:[0,206,209,1],darkviolet:[148,0,211,1],deeppink:[255,20,147,1],deepskyblue:[0,191,255,1],dimgray:[105,105,105,1],dimgrey:[105,105,105,1],dodgerblue:[30,144,255,1],firebrick:[178,34,34,1],floralwhite:[255,250,240,1],forestgreen:[34,139,34,1],fuchsia:[255,0,255,1],gainsboro:[220,220,220,1],ghostwhite:[248,248,255,1],gold:[255,215,0,1],goldenrod:[218,165,32,1],gray:[128,128,128,1],green:[0,128,0,1],greenyellow:[173,255,47,1],grey:[128,128,128,1],honeydew:[240,255,240,1],hotpink:[255,105,180,1],indianred:[205,92,92,1],indigo:[75,0,130,1],ivory:[255,255,240,1],khaki:[240,230,140,1],lavender:[230,230,250,1],lavenderblush:[255,240,245,1],lawngreen:[124,252,0,1],lemonchiffon:[255,250,205,1],lightblue:[173,216,230,1],lightcoral:[240,128,128,1],lightcyan:[224,255,255,1],lightgoldenrodyellow:[250,250,210,1],lightgray:[211,211,211,1],lightgreen:[144,238,144,1],lightgrey:[211,211,211,1],lightpink:[255,182,193,1],lightsalmon:[255,160,122,1],lightseagreen:[32,178,170,1],lightskyblue:[135,206,250,1],lightslategray:[119,136,153,1],lightslategrey:[119,136,153,1],lightsteelblue:[176,196,222,1],lightyellow:[255,255,224,1],lime:[0,255,0,1],limegreen:[50,205,50,1],linen:[250,240,230,1],magenta:[255,0,255,1],maroon:[128,0,0,1],mediumaquamarine:[102,205,170,1],mediumblue:[0,0,205,1],mediumorchid:[186,85,211,1],mediumpurple:[147,112,219,1],mediumseagreen:[60,179,113,1],mediumslateblue:[123,104,238,1],mediumspringgreen:[0,250,154,1],mediumturquoise:[72,209,204,1],mediumvioletred:[199,21,133,1],midnightblue:[25,25,112,1],mintcream:[245,255,250,1],mistyrose:[255,228,225,1],moccasin:[255,228,181,1],navajowhite:[255,222,173,1],navy:[0,0,128,1],oldlace:[253,245,230,1],olive:[128,128,0,1],olivedrab:[107,142,35,1],orange:[255,165,0,1],orangered:[255,69,0,1],orchid:[218,112,214,1],palegoldenrod:[238,232,170,1],palegreen:[152,251,152,1],paleturquoise:[175,238,238,1],palevioletred:[219,112,147,1],papayawhip:[255,239,213,1],peachpuff:[255,218,185,1],peru:[205,133,63,1],pink:[255,192,203,1],plum:[221,160,221,1],powderblue:[176,224,230,1],purple:[128,0,128,1],rebeccapurple:[102,51,153,1],red:[255,0,0,1],rosybrown:[188,143,143,1],royalblue:[65,105,225,1],saddlebrown:[139,69,19,1],salmon:[250,128,114,1],sandybrown:[244,164,96,1],seagreen:[46,139,87,1],seashell:[255,245,238,1],sienna:[160,82,45,1],silver:[192,192,192,1],skyblue:[135,206,235,1],slateblue:[106,90,205,1],slategray:[112,128,144,1],slategrey:[112,128,144,1],snow:[255,250,250,1],springgreen:[0,255,127,1],steelblue:[70,130,180,1],tan:[210,180,140,1],teal:[0,128,128,1],thistle:[216,191,216,1],tomato:[255,99,71,1],turquoise:[64,224,208,1],violet:[238,130,238,1],wheat:[245,222,179,1],white:[255,255,255,1],whitesmoke:[245,245,245,1],yellow:[255,255,0,1],yellowgreen:[154,205,50,1]};function r(t){return(t=Math.round(t))<0?0:t>255?255:t}function i(t){return t<0?0:t>1?1:t}function o(t){return"%"===t[t.length-1]?r(parseFloat(t)/100*255):r(parseInt(t))}function a(t){return"%"===t[t.length-1]?i(parseFloat(t)/100):i(parseFloat(t))}function s(t,e,n){return n<0?n+=1:n>1&&(n-=1),6*n<1?t+(e-t)*n*6:2*n<1?e:3*n<2?t+(e-t)*(2/3-n)*6:t}try{e.parseCSSColor=function(t){var e,i=t.replace(/ /g,"").toLowerCase();if(i in n)return n[i].slice();if("#"===i[0])return 4===i.length?(e=parseInt(i.substr(1),16))>=0&&e<=4095?[(3840&e)>>4|(3840&e)>>8,240&e|(240&e)>>4,15&e|(15&e)<<4,1]:null:7===i.length&&(e=parseInt(i.substr(1),16))>=0&&e<=16777215?[(16711680&e)>>16,(65280&e)>>8,255&e,1]:null;var A=i.indexOf("("),l=i.indexOf(")");if(-1!==A&&l+1===i.length){var c=i.substr(0,A),u=i.substr(A+1,l-(A+1)).split(","),g=1;switch(c){case"rgba":if(4!==u.length)return null;g=a(u.pop());case"rgb":return 3!==u.length?null:[o(u[0]),o(u[1]),o(u[2]),g];case"hsla":if(4!==u.length)return null;g=a(u.pop());case"hsl":if(3!==u.length)return null;var h=(parseFloat(u[0])%360+360)%360/360,d=a(u[1]),f=a(u[2]),p=f<=.5?f*(d+1):f+d-f*d,I=2*f-p;return[r(255*s(I,p,h+1/3)),r(255*s(I,p,h)),r(255*s(I,p,h-1/3)),g];default:return null}}return null}}catch(t){}}).parseCSSColor,ne=function(t,e,n,r){void 0===r&&(r=1),this.r=t,this.g=e,this.b=n,this.a=r};ne.parse=function(t){if(t){if(t instanceof ne)return t;if("string"==typeof t){var e=ee(t);if(e)return new ne(e[0]/255*e[3],e[1]/255*e[3],e[2]/255*e[3],e[3])}}},ne.prototype.toString=function(){var t=this.toArray(),e=t[0],n=t[1],r=t[2],i=t[3];return"rgba("+Math.round(e)+","+Math.round(n)+","+Math.round(r)+","+i+")"},ne.prototype.toArray=function(){var t=this.r,e=this.g,n=this.b,r=this.a;return 0===r?[0,0,0,0]:[255*t/r,255*e/r,255*n/r,r]},ne.black=new ne(0,0,0,1),ne.white=new ne(1,1,1,1),ne.transparent=new ne(0,0,0,0),ne.red=new ne(1,0,0,1);var re=function(t,e,n){this.sensitivity=t?e?"variant":"case":e?"accent":"base",this.locale=n,this.collator=new Intl.Collator(this.locale?this.locale:[],{sensitivity:this.sensitivity,usage:"search"})};re.prototype.compare=function(t,e){return this.collator.compare(t,e)},re.prototype.resolvedLocale=function(){return new Intl.Collator(this.locale?this.locale:[]).resolvedOptions().locale};var ie=function(t,e,n){this.text=t,this.scale=e,this.fontStack=n},oe=function(t){this.sections=t};function ae(t,e,n,r){return"number"==typeof t&&t>=0&&t<=255&&"number"==typeof e&&e>=0&&e<=255&&"number"==typeof n&&n>=0&&n<=255?void 0===r||"number"==typeof r&&r>=0&&r<=1?null:"Invalid rgba value ["+[t,e,n,r].join(", ")+"]: 'a' must be between 0 and 1.":"Invalid rgba value ["+("number"==typeof r?[t,e,n,r]:[t,e,n]).join(", ")+"]: 'r', 'g', and 'b' must be between 0 and 255."}function se(t){if(null===t)return jt;if("string"==typeof t)return Pt;if("boolean"==typeof t)return Kt;if("number"==typeof t)return Yt;if(t instanceof ne)return zt;if(t instanceof re)return Vt;if(t instanceof oe)return Wt;if(Array.isArray(t)){for(var e,n=t.length,r=0,i=t;r<i.length;r+=1){var o=se(i[r]);if(e){if(e===o)continue;e=qt;break}e=o}return Xt(e||qt,n)}return Jt}function Ae(t){var e=typeof t;return null===t?"":"string"===e||"number"===e||"boolean"===e?String(t):t instanceof ne||t instanceof oe?t.toString():JSON.stringify(t)}oe.fromString=function(t){return new oe([new ie(t,null,null)])},oe.prototype.toString=function(){return this.sections.map(function(t){return t.text}).join("")},oe.prototype.serialize=function(){for(var t=["format"],e=0,n=this.sections;e<n.length;e+=1){var r=n[e];t.push(r.text);var i={};r.fontStack&&(i["text-font"]=["literal",r.fontStack.split(",")]),r.scale&&(i["font-scale"]=r.scale),t.push(i)}return t};var le=function(t,e){this.type=t,this.value=e};le.parse=function(t,e){if(2!==t.length)return e.error("'literal' expression requires exactly one argument, but found "+(t.length-1)+" instead.");if(!function t(e){if(null===e)return!0;if("string"==typeof e)return!0;if("boolean"==typeof e)return!0;if("number"==typeof e)return!0;if(e instanceof ne)return!0;if(e instanceof re)return!0;if(e instanceof oe)return!0;if(Array.isArray(e)){for(var n=0,r=e;n<r.length;n+=1)if(!t(r[n]))return!1;return!0}if("object"==typeof e){for(var i in e)if(!t(e[i]))return!1;return!0}return!1}(t[1]))return e.error("invalid value");var n=t[1],r=se(n),i=e.expectedType;return"array"!==r.kind||0!==r.N||!i||"array"!==i.kind||"number"==typeof i.N&&0!==i.N||(r=i),new le(r,n)},le.prototype.evaluate=function(){return this.value},le.prototype.eachChild=function(){},le.prototype.possibleOutputs=function(){return[this.value]},le.prototype.serialize=function(){return"array"===this.type.kind||"object"===this.type.kind?["literal",this.value]:this.value instanceof ne?["rgba"].concat(this.value.toArray()):this.value instanceof oe?this.value.serialize():this.value};var ce=function(t){this.name="ExpressionEvaluationError",this.message=t};ce.prototype.toJSON=function(){return this.message};var ue={string:Pt,number:Yt,boolean:Kt,object:Jt},ge=function(t,e){this.type=t,this.args=e};ge.parse=function(t,e){if(t.length<2)return e.error("Expected at least one argument.");var n,r=1,i=t[0];if("array"===i){var o,a;if(t.length>2){var s=t[1];if("string"!=typeof s||!(s in ue)||"object"===s)return e.error('The item type argument of "array" must be one of string, number, boolean',1);o=ue[s],r++}else o=qt;if(t.length>3){if(null!==t[2]&&("number"!=typeof t[2]||t[2]<0||t[2]!==Math.floor(t[2])))return e.error('The length argument to "array" must be a positive integer literal',2);a=t[2],r++}n=Xt(o,a)}else n=ue[i];for(var A=[];r<t.length;r++){var l=e.parse(t[r],r,qt);if(!l)return null;A.push(l)}return new ge(n,A)},ge.prototype.evaluate=function(t){for(var e=0;e<this.args.length;e++){var n=this.args[e].evaluate(t);if(!te(this.type,se(n)))return n;if(e===this.args.length-1)throw new ce("Expected value to be of type "+Zt(this.type)+", but found "+Zt(se(n))+" instead.")}return null},ge.prototype.eachChild=function(t){this.args.forEach(t)},ge.prototype.possibleOutputs=function(){var t;return(t=[]).concat.apply(t,this.args.map(function(t){return t.possibleOutputs()}))},ge.prototype.serialize=function(){var t=this.type,e=[t.kind];if("array"===t.kind){var n=t.itemType;if("string"===n.kind||"number"===n.kind||"boolean"===n.kind){e.push(n.kind);var r=t.N;("number"==typeof r||this.args.length>1)&&e.push(r)}}return e.concat(this.args.map(function(t){return t.serialize()}))};var he=function(t){this.type=Wt,this.sections=t};he.parse=function(t,e){if(t.length<3)return e.error("Expected at least two arguments.");if((t.length-1)%2!=0)return e.error("Expected an even number of arguments.");for(var n=[],r=1;r<t.length-1;r+=2){var i=e.parse(t[r],1,qt);if(!i)return null;var o=i.type.kind;if("string"!==o&&"value"!==o&&"null"!==o)return e.error("Formatted text type must be 'string', 'value', or 'null'.");var a=t[r+1];if("object"!=typeof a||Array.isArray(a))return e.error("Format options argument must be an object.");var s=null;if(a["font-scale"]&&!(s=e.parse(a["font-scale"],1,Yt)))return null;var A=null;if(a["text-font"]&&!(A=e.parse(a["text-font"],1,Xt(Pt))))return null;n.push({text:i,scale:s,font:A})}return new he(n)},he.prototype.evaluate=function(t){return new oe(this.sections.map(function(e){return new ie(Ae(e.text.evaluate(t)),e.scale?e.scale.evaluate(t):null,e.font?e.font.evaluate(t).join(","):null)}))},he.prototype.eachChild=function(t){for(var e=0,n=this.sections;e<n.length;e+=1){var r=n[e];t(r.text),r.scale&&t(r.scale),r.font&&t(r.font)}},he.prototype.possibleOutputs=function(){return[void 0]},he.prototype.serialize=function(){for(var t=["format"],e=0,n=this.sections;e<n.length;e+=1){var r=n[e];t.push(r.text.serialize());var i={};r.scale&&(i["font-scale"]=r.scale.serialize()),r.font&&(i["text-font"]=r.font.serialize()),t.push(i)}return t};var de={"to-boolean":Kt,"to-color":zt,"to-number":Yt,"to-string":Pt},fe=function(t,e){this.type=t,this.args=e};fe.parse=function(t,e){if(t.length<2)return e.error("Expected at least one argument.");var n=t[0];if(("to-boolean"===n||"to-string"===n)&&2!==t.length)return e.error("Expected one argument.");for(var r=de[n],i=[],o=1;o<t.length;o++){var a=e.parse(t[o],o,qt);if(!a)return null;i.push(a)}return new fe(r,i)},fe.prototype.evaluate=function(t){if("boolean"===this.type.kind)return Boolean(this.args[0].evaluate(t));if("color"===this.type.kind){for(var e,n,r=0,i=this.args;r<i.length;r+=1){if(n=null,(e=i[r].evaluate(t))instanceof ne)return e;if("string"==typeof e){var o=t.parseColor(e);if(o)return o}else if(Array.isArray(e)&&!(n=e.length<3||e.length>4?"Invalid rbga value "+JSON.stringify(e)+": expected an array containing either three or four numeric values.":ae(e[0],e[1],e[2],e[3])))return new ne(e[0]/255,e[1]/255,e[2]/255,e[3])}throw new ce(n||"Could not parse color from value '"+("string"==typeof e?e:String(JSON.stringify(e)))+"'")}if("number"===this.type.kind){for(var a=null,s=0,A=this.args;s<A.length;s+=1){if(null===(a=A[s].evaluate(t)))return 0;var l=Number(a);if(!isNaN(l))return l}throw new ce("Could not convert "+JSON.stringify(a)+" to number.")}return"formatted"===this.type.kind?oe.fromString(Ae(this.args[0].evaluate(t))):Ae(this.args[0].evaluate(t))},fe.prototype.eachChild=function(t){this.args.forEach(t)},fe.prototype.possibleOutputs=function(){var t;return(t=[]).concat.apply(t,this.args.map(function(t){return t.possibleOutputs()}))},fe.prototype.serialize=function(){if("formatted"===this.type.kind)return new he([{text:this.args[0],scale:null,font:null}]).serialize();var t=["to-"+this.type.kind];return this.eachChild(function(e){t.push(e.serialize())}),t};var pe=["Unknown","Point","LineString","Polygon"],Ie=function(){this.globals=null,this.feature=null,this.featureState=null,this._parseColorCache={}};Ie.prototype.id=function(){return this.feature&&"id"in this.feature?this.feature.id:null},Ie.prototype.geometryType=function(){return this.feature?"number"==typeof this.feature.type?pe[this.feature.type]:this.feature.type:null},Ie.prototype.properties=function(){return this.feature&&this.feature.properties||{}},Ie.prototype.parseColor=function(t){var e=this._parseColorCache[t];return e||(e=this._parseColorCache[t]=ne.parse(t)),e};var Ce=function(t,e,n,r){this.name=t,this.type=e,this._evaluate=n,this.args=r};Ce.prototype.evaluate=function(t){return this._evaluate(t,this.args)},Ce.prototype.eachChild=function(t){this.args.forEach(t)},Ce.prototype.possibleOutputs=function(){return[void 0]},Ce.prototype.serialize=function(){return[this.name].concat(this.args.map(function(t){return t.serialize()}))},Ce.parse=function(t,e){var n,r=t[0],i=Ce.definitions[r];if(!i)return e.error('Unknown expression "'+r+'". If you wanted a literal array, use ["literal", [...]].',0);for(var o=Array.isArray(i)?i[0]:i.type,a=Array.isArray(i)?[[i[1],i[2]]]:i.overloads,s=a.filter(function(e){var n=e[0];return!Array.isArray(n)||n.length===t.length-1}),A=null,l=0,c=s;l<c.length;l+=1){var u=c[l],g=u[0],h=u[1];A=new be(e.registry,e.path,null,e.scope);for(var d=[],f=!1,p=1;p<t.length;p++){var I=t[p],C=Array.isArray(g)?g[p-1]:g.type,m=A.parse(I,1+d.length,C);if(!m){f=!0;break}d.push(m)}if(!f)if(Array.isArray(g)&&g.length!==d.length)A.error("Expected "+g.length+" arguments, but found "+d.length+" instead.");else{for(var y=0;y<d.length;y++){var E=Array.isArray(g)?g[y]:g.type,v=d[y];A.concat(y+1).checkSubtype(E,v.type)}if(0===A.errors.length)return new Ce(r,o,h,d)}}if(1===s.length)(n=e.errors).push.apply(n,A.errors);else{for(var B=(s.length?s:a).map(function(t){var e;return e=t[0],Array.isArray(e)?"("+e.map(Zt).join(", ")+")":"("+Zt(e.type)+"...)"}).join(" | "),b=[],Q=1;Q<t.length;Q++){var w=e.parse(t[Q],1+b.length);if(!w)return null;b.push(Zt(w.type))}e.error("Expected arguments of type "+B+", but found ("+b.join(", ")+") instead.")}return null},Ce.register=function(t,e){for(var n in Ce.definitions=e,e)t[n]=Ce};var me=function(t,e,n){this.type=Vt,this.locale=n,this.caseSensitive=t,this.diacriticSensitive=e};function ye(t){if(t instanceof Ce){if("get"===t.name&&1===t.args.length)return!1;if("feature-state"===t.name)return!1;if("has"===t.name&&1===t.args.length)return!1;if("properties"===t.name||"geometry-type"===t.name||"id"===t.name)return!1;if(/^filter-/.test(t.name))return!1}var e=!0;return t.eachChild(function(t){e&&!ye(t)&&(e=!1)}),e}function Ee(t){if(t instanceof Ce&&"feature-state"===t.name)return!1;var e=!0;return t.eachChild(function(t){e&&!Ee(t)&&(e=!1)}),e}function ve(t,e){if(t instanceof Ce&&e.indexOf(t.name)>=0)return!1;var n=!0;return t.eachChild(function(t){n&&!ve(t,e)&&(n=!1)}),n}me.parse=function(t,e){if(2!==t.length)return e.error("Expected one argument.");var n=t[1];if("object"!=typeof n||Array.isArray(n))return e.error("Collator options argument must be an object.");var r=e.parse(void 0!==n["case-sensitive"]&&n["case-sensitive"],1,Kt);if(!r)return null;var i=e.parse(void 0!==n["diacritic-sensitive"]&&n["diacritic-sensitive"],1,Kt);if(!i)return null;var o=null;return n.locale&&!(o=e.parse(n.locale,1,Pt))?null:new me(r,i,o)},me.prototype.evaluate=function(t){return new re(this.caseSensitive.evaluate(t),this.diacriticSensitive.evaluate(t),this.locale?this.locale.evaluate(t):null)},me.prototype.eachChild=function(t){t(this.caseSensitive),t(this.diacriticSensitive),this.locale&&t(this.locale)},me.prototype.possibleOutputs=function(){return[void 0]},me.prototype.serialize=function(){var t={};return t["case-sensitive"]=this.caseSensitive.serialize(),t["diacritic-sensitive"]=this.diacriticSensitive.serialize(),this.locale&&(t.locale=this.locale.serialize()),["collator",t]};var Be=function(t,e){this.type=e.type,this.name=t,this.boundExpression=e};Be.parse=function(t,e){if(2!==t.length||"string"!=typeof t[1])return e.error("'var' expression requires exactly one string literal argument.");var n=t[1];return e.scope.has(n)?new Be(n,e.scope.get(n)):e.error('Unknown variable "'+n+'". Make sure "'+n+'" has been bound in an enclosing "let" expression before using it.',1)},Be.prototype.evaluate=function(t){return this.boundExpression.evaluate(t)},Be.prototype.eachChild=function(){},Be.prototype.possibleOutputs=function(){return[void 0]},Be.prototype.serialize=function(){return["var",this.name]};var be=function(t,e,n,r,i){void 0===e&&(e=[]),void 0===r&&(r=new Ht),void 0===i&&(i=[]),this.registry=t,this.path=e,this.key=e.map(function(t){return"["+t+"]"}).join(""),this.scope=r,this.errors=i,this.expectedType=n};function Qe(t,e){for(var n,r,i=t.length-1,o=0,a=i,s=0;o<=a;)if(n=t[s=Math.floor((o+a)/2)],r=t[s+1],n<=e){if(s===i||e<r)return s;o=s+1}else{if(!(n>e))throw new ce("Input is not a number.");a=s-1}return 0}be.prototype.parse=function(t,e,n,r,i){return void 0===i&&(i={}),e?this.concat(e,n,r)._parse(t,i):this._parse(t,i)},be.prototype._parse=function(t,e){function n(t,e,n){return"assert"===n?new ge(e,[t]):"coerce"===n?new fe(e,[t]):t}if(null!==t&&"string"!=typeof t&&"boolean"!=typeof t&&"number"!=typeof t||(t=["literal",t]),Array.isArray(t)){if(0===t.length)return this.error('Expected an array with at least one element. If you wanted a literal array, use ["literal", []].');var r=t[0];if("string"!=typeof r)return this.error("Expression name must be a string, but found "+typeof r+' instead. If you wanted a literal array, use ["literal", [...]].',0),null;var i=this.registry[r];if(i){var o=i.parse(t,this);if(!o)return null;if(this.expectedType){var a=this.expectedType,s=o.type;if("string"!==a.kind&&"number"!==a.kind&&"boolean"!==a.kind&&"object"!==a.kind&&"array"!==a.kind||"value"!==s.kind)if("color"!==a.kind&&"formatted"!==a.kind||"value"!==s.kind&&"string"!==s.kind){if(this.checkSubtype(a,s))return null}else o=n(o,a,e.typeAnnotation||"coerce");else o=n(o,a,e.typeAnnotation||"assert")}if(!(o instanceof le)&&function t(e){if(e instanceof Be)return t(e.boundExpression);if(e instanceof Ce&&"error"===e.name)return!1;if(e instanceof me)return!1;var n=e instanceof fe||e instanceof ge,r=!0;return e.eachChild(function(e){r=n?r&&t(e):r&&e instanceof le}),!!r&&ye(e)&&ve(e,["zoom","heatmap-density","line-progress","accumulated","is-supported-script"])}(o)){var A=new Ie;try{o=new le(o.type,o.evaluate(A))}catch(t){return this.error(t.message),null}}return o}return this.error('Unknown expression "'+r+'". If you wanted a literal array, use ["literal", [...]].',0)}return void 0===t?this.error("'undefined' value invalid. Use null instead."):"object"==typeof t?this.error('Bare objects invalid. Use ["literal", {...}] instead.'):this.error("Expected an array, but found "+typeof t+" instead.")},be.prototype.concat=function(t,e,n){var r="number"==typeof t?this.path.concat(t):this.path,i=n?this.scope.concat(n):this.scope;return new be(this.registry,r,e||null,i,this.errors)},be.prototype.error=function(t){for(var e=[],n=arguments.length-1;n-- >0;)e[n]=arguments[n+1];var r=""+this.key+e.map(function(t){return"["+t+"]"}).join("");this.errors.push(new Gt(r,t))},be.prototype.checkSubtype=function(t,e){var n=te(t,e);return n&&this.error(n),n};var we=function(t,e,n){this.type=t,this.input=e,this.labels=[],this.outputs=[];for(var r=0,i=n;r<i.length;r+=1){var o=i[r],a=o[0],s=o[1];this.labels.push(a),this.outputs.push(s)}};function xe(t,e,n){return t*(1-n)+e*n}we.parse=function(t,e){if(t.length-1<4)return e.error("Expected at least 4 arguments, but found only "+(t.length-1)+".");if((t.length-1)%2!=0)return e.error("Expected an even number of arguments.");var n=e.parse(t[1],1,Yt);if(!n)return null;var r=[],i=null;e.expectedType&&"value"!==e.expectedType.kind&&(i=e.expectedType);for(var o=1;o<t.length;o+=2){var a=1===o?-1/0:t[o],s=t[o+1],A=o,l=o+1;if("number"!=typeof a)return e.error('Input/output pairs for "step" expressions must be defined using literal numeric values (not computed expressions) for the input values.',A);if(r.length&&r[r.length-1][0]>=a)return e.error('Input/output pairs for "step" expressions must be arranged with input values in strictly ascending order.',A);var c=e.parse(s,l,i);if(!c)return null;i=i||c.type,r.push([a,c])}return new we(i,n,r)},we.prototype.evaluate=function(t){var e=this.labels,n=this.outputs;if(1===e.length)return n[0].evaluate(t);var r=this.input.evaluate(t);if(r<=e[0])return n[0].evaluate(t);var i=e.length;return r>=e[i-1]?n[i-1].evaluate(t):n[Qe(e,r)].evaluate(t)},we.prototype.eachChild=function(t){t(this.input);for(var e=0,n=this.outputs;e<n.length;e+=1)t(n[e])},we.prototype.possibleOutputs=function(){var t;return(t=[]).concat.apply(t,this.outputs.map(function(t){return t.possibleOutputs()}))},we.prototype.serialize=function(){for(var t=["step",this.input.serialize()],e=0;e<this.labels.length;e++)e>0&&t.push(this.labels[e]),t.push(this.outputs[e].serialize());return t};var _e=Object.freeze({number:xe,color:function(t,e,n){return new ne(xe(t.r,e.r,n),xe(t.g,e.g,n),xe(t.b,e.b,n),xe(t.a,e.a,n))},array:function(t,e,n){return t.map(function(t,r){return xe(t,e[r],n)})}}),De=.95047,Me=1,ke=1.08883,Se=4/29,Le=6/29,Re=3*Le*Le,Ne=Le*Le*Le,Fe=Math.PI/180,Te=180/Math.PI;function Ue(t){return t>Ne?Math.pow(t,1/3):t/Re+Se}function Oe(t){return t>Le?t*t*t:Re*(t-Se)}function Ge(t){return 255*(t<=.0031308?12.92*t:1.055*Math.pow(t,1/2.4)-.055)}function He(t){return(t/=255)<=.04045?t/12.92:Math.pow((t+.055)/1.055,2.4)}function je(t){var e=He(t.r),n=He(t.g),r=He(t.b),i=Ue((.4124564*e+.3575761*n+.1804375*r)/De),o=Ue((.2126729*e+.7151522*n+.072175*r)/Me);return{l:116*o-16,a:500*(i-o),b:200*(o-Ue((.0193339*e+.119192*n+.9503041*r)/ke)),alpha:t.a}}function Ye(t){var e=(t.l+16)/116,n=isNaN(t.a)?e:e+t.a/500,r=isNaN(t.b)?e:e-t.b/200;return e=Me*Oe(e),n=De*Oe(n),r=ke*Oe(r),new ne(Ge(3.2404542*n-1.5371385*e-.4985314*r),Ge(-.969266*n+1.8760108*e+.041556*r),Ge(.0556434*n-.2040259*e+1.0572252*r),t.alpha)}var Pe={forward:je,reverse:Ye,interpolate:function(t,e,n){return{l:xe(t.l,e.l,n),a:xe(t.a,e.a,n),b:xe(t.b,e.b,n),alpha:xe(t.alpha,e.alpha,n)}}},Ke={forward:function(t){var e=je(t),n=e.l,r=e.a,i=e.b,o=Math.atan2(i,r)*Te;return{h:o<0?o+360:o,c:Math.sqrt(r*r+i*i),l:n,alpha:t.a}},reverse:function(t){var e=t.h*Fe,n=t.c;return Ye({l:t.l,a:Math.cos(e)*n,b:Math.sin(e)*n,alpha:t.alpha})},interpolate:function(t,e,n){return{h:function(t,e,n){var r=e-t;return t+n*(r>180||r<-180?r-360*Math.round(r/360):r)}(t.h,e.h,n),c:xe(t.c,e.c,n),l:xe(t.l,e.l,n),alpha:xe(t.alpha,e.alpha,n)}}},ze=Object.freeze({lab:Pe,hcl:Ke}),Je=function(t,e,n,r,i){this.type=t,this.operator=e,this.interpolation=n,this.input=r,this.labels=[],this.outputs=[];for(var o=0,a=i;o<a.length;o+=1){var s=a[o],A=s[0],l=s[1];this.labels.push(A),this.outputs.push(l)}};function qe(t,e,n,r){var i=r-n,o=t-n;return 0===i?0:1===e?o/i:(Math.pow(e,o)-1)/(Math.pow(e,i)-1)}Je.interpolationFactor=function(t,e,r,i){var o=0;if("exponential"===t.name)o=qe(e,t.base,r,i);else if("linear"===t.name)o=qe(e,1,r,i);else if("cubic-bezier"===t.name){var a=t.controlPoints;o=new n(a[0],a[1],a[2],a[3]).solve(qe(e,1,r,i))}return o},Je.parse=function(t,e){var n=t[0],r=t[1],i=t[2],o=t.slice(3);if(!Array.isArray(r)||0===r.length)return e.error("Expected an interpolation type expression.",1);if("linear"===r[0])r={name:"linear"};else if("exponential"===r[0]){var a=r[1];if("number"!=typeof a)return e.error("Exponential interpolation requires a numeric base.",1,1);r={name:"exponential",base:a}}else{if("cubic-bezier"!==r[0])return e.error("Unknown interpolation type "+String(r[0]),1,0);var s=r.slice(1);if(4!==s.length||s.some(function(t){return"number"!=typeof t||t<0||t>1}))return e.error("Cubic bezier interpolation requires four numeric arguments with values between 0 and 1.",1);r={name:"cubic-bezier",controlPoints:s}}if(t.length-1<4)return e.error("Expected at least 4 arguments, but found only "+(t.length-1)+".");if((t.length-1)%2!=0)return e.error("Expected an even number of arguments.");if(!(i=e.parse(i,2,Yt)))return null;var A=[],l=null;"interpolate-hcl"===n||"interpolate-lab"===n?l=zt:e.expectedType&&"value"!==e.expectedType.kind&&(l=e.expectedType);for(var c=0;c<o.length;c+=2){var u=o[c],g=o[c+1],h=c+3,d=c+4;if("number"!=typeof u)return e.error('Input/output pairs for "interpolate" expressions must be defined using literal numeric values (not computed expressions) for the input values.',h);if(A.length&&A[A.length-1][0]>=u)return e.error('Input/output pairs for "interpolate" expressions must be arranged with input values in strictly ascending order.',h);var f=e.parse(g,d,l);if(!f)return null;l=l||f.type,A.push([u,f])}return"number"===l.kind||"color"===l.kind||"array"===l.kind&&"number"===l.itemType.kind&&"number"==typeof l.N?new Je(l,n,r,i,A):e.error("Type "+Zt(l)+" is not interpolatable.")},Je.prototype.evaluate=function(t){var e=this.labels,n=this.outputs;if(1===e.length)return n[0].evaluate(t);var r=this.input.evaluate(t);if(r<=e[0])return n[0].evaluate(t);var i=e.length;if(r>=e[i-1])return n[i-1].evaluate(t);var o=Qe(e,r),a=e[o],s=e[o+1],A=Je.interpolationFactor(this.interpolation,r,a,s),l=n[o].evaluate(t),c=n[o+1].evaluate(t);return"interpolate"===this.operator?_e[this.type.kind.toLowerCase()](l,c,A):"interpolate-hcl"===this.operator?Ke.reverse(Ke.interpolate(Ke.forward(l),Ke.forward(c),A)):Pe.reverse(Pe.interpolate(Pe.forward(l),Pe.forward(c),A))},Je.prototype.eachChild=function(t){t(this.input);for(var e=0,n=this.outputs;e<n.length;e+=1)t(n[e])},Je.prototype.possibleOutputs=function(){var t;return(t=[]).concat.apply(t,this.outputs.map(function(t){return t.possibleOutputs()}))},Je.prototype.serialize=function(){var t;t="linear"===this.interpolation.name?["linear"]:"exponential"===this.interpolation.name?1===this.interpolation.base?["linear"]:["exponential",this.interpolation.base]:["cubic-bezier"].concat(this.interpolation.controlPoints);for(var e=[this.operator,t,this.input.serialize()],n=0;n<this.labels.length;n++)e.push(this.labels[n],this.outputs[n].serialize());return e};var Ve=function(t,e){this.type=t,this.args=e};Ve.parse=function(t,e){if(t.length<2)return e.error("Expectected at least one argument.");var n=null,r=e.expectedType;r&&"value"!==r.kind&&(n=r);for(var i=[],o=0,a=t.slice(1);o<a.length;o+=1){var s=a[o],A=e.parse(s,1+i.length,n,void 0,{typeAnnotation:"omit"});if(!A)return null;n=n||A.type,i.push(A)}var l=r&&i.some(function(t){return te(r,t.type)});return new Ve(l?qt:n,i)},Ve.prototype.evaluate=function(t){for(var e=null,n=0,r=this.args;n<r.length&&null===(e=r[n].evaluate(t));n+=1);return e},Ve.prototype.eachChild=function(t){this.args.forEach(t)},Ve.prototype.possibleOutputs=function(){var t;return(t=[]).concat.apply(t,this.args.map(function(t){return t.possibleOutputs()}))},Ve.prototype.serialize=function(){var t=["coalesce"];return this.eachChild(function(e){t.push(e.serialize())}),t};var We=function(t,e){this.type=e.type,this.bindings=[].concat(t),this.result=e};We.prototype.evaluate=function(t){return this.result.evaluate(t)},We.prototype.eachChild=function(t){for(var e=0,n=this.bindings;e<n.length;e+=1)t(n[e][1]);t(this.result)},We.parse=function(t,e){if(t.length<4)return e.error("Expected at least 3 arguments, but found "+(t.length-1)+" instead.");for(var n=[],r=1;r<t.length-1;r+=2){var i=t[r];if("string"!=typeof i)return e.error("Expected string, but found "+typeof i+" instead.",r);if(/[^a-zA-Z0-9_]/.test(i))return e.error("Variable names must contain only alphanumeric characters or '_'.",r);var o=e.parse(t[r+1],r+1);if(!o)return null;n.push([i,o])}var a=e.parse(t[t.length-1],t.length-1,e.expectedType,n);return a?new We(n,a):null},We.prototype.possibleOutputs=function(){return this.result.possibleOutputs()},We.prototype.serialize=function(){for(var t=["let"],e=0,n=this.bindings;e<n.length;e+=1){var r=n[e],i=r[0],o=r[1];t.push(i,o.serialize())}return t.push(this.result.serialize()),t};var Xe=function(t,e,n){this.type=t,this.index=e,this.input=n};Xe.parse=function(t,e){if(3!==t.length)return e.error("Expected 2 arguments, but found "+(t.length-1)+" instead.");var n=e.parse(t[1],1,Yt),r=e.parse(t[2],2,Xt(e.expectedType||qt));if(!n||!r)return null;var i=r.type;return new Xe(i.itemType,n,r)},Xe.prototype.evaluate=function(t){var e=this.index.evaluate(t),n=this.input.evaluate(t);if(e<0)throw new ce("Array index out of bounds: "+e+" < 0.");if(e>=n.length)throw new ce("Array index out of bounds: "+e+" > "+(n.length-1)+".");if(e!==Math.floor(e))throw new ce("Array index must be an integer, but found "+e+" instead.");return n[e]},Xe.prototype.eachChild=function(t){t(this.index),t(this.input)},Xe.prototype.possibleOutputs=function(){return[void 0]},Xe.prototype.serialize=function(){return["at",this.index.serialize(),this.input.serialize()]};var Ze=function(t,e,n,r,i,o){this.inputType=t,this.type=e,this.input=n,this.cases=r,this.outputs=i,this.otherwise=o};Ze.parse=function(t,e){if(t.length<5)return e.error("Expected at least 4 arguments, but found only "+(t.length-1)+".");if(t.length%2!=1)return e.error("Expected an even number of arguments.");var n,r;e.expectedType&&"value"!==e.expectedType.kind&&(r=e.expectedType);for(var i={},o=[],a=2;a<t.length-1;a+=2){var s=t[a],A=t[a+1];Array.isArray(s)||(s=[s]);var l=e.concat(a);if(0===s.length)return l.error("Expected at least one branch label.");for(var c=0,u=s;c<u.length;c+=1){var g=u[c];if("number"!=typeof g&&"string"!=typeof g)return l.error("Branch labels must be numbers or strings.");if("number"==typeof g&&Math.abs(g)>Number.MAX_SAFE_INTEGER)return l.error("Branch labels must be integers no larger than "+Number.MAX_SAFE_INTEGER+".");if("number"==typeof g&&Math.floor(g)!==g)return l.error("Numeric branch labels must be integer values.");if(n){if(l.checkSubtype(n,se(g)))return null}else n=se(g);if(void 0!==i[String(g)])return l.error("Branch labels must be unique.");i[String(g)]=o.length}var h=e.parse(A,a,r);if(!h)return null;r=r||h.type,o.push(h)}var d=e.parse(t[1],1,qt);if(!d)return null;var f=e.parse(t[t.length-1],t.length-1,r);return f?"value"!==d.type.kind&&e.concat(1).checkSubtype(n,d.type)?null:new Ze(n,r,d,i,o,f):null},Ze.prototype.evaluate=function(t){var e=this.input.evaluate(t);return(se(e)===this.inputType&&this.outputs[this.cases[e]]||this.otherwise).evaluate(t)},Ze.prototype.eachChild=function(t){t(this.input),this.outputs.forEach(t),t(this.otherwise)},Ze.prototype.possibleOutputs=function(){var t;return(t=[]).concat.apply(t,this.outputs.map(function(t){return t.possibleOutputs()})).concat(this.otherwise.possibleOutputs())},Ze.prototype.serialize=function(){for(var t=this,e=["match",this.input.serialize()],n=[],r={},i=0,o=Object.keys(this.cases).sort();i<o.length;i+=1){var a=o[i];void 0===(u=r[this.cases[a]])?(r[this.cases[a]]=n.length,n.push([this.cases[a],[a]])):n[u][1].push(a)}for(var s=function(e){return"number"===t.inputType.kind?Number(e):e},A=0,l=n;A<l.length;A+=1){var c=l[A],u=c[0],g=c[1];1===g.length?e.push(s(g[0])):e.push(g.map(s)),e.push(this.outputs[outputIndex$1].serialize())}return e.push(this.otherwise.serialize()),e};var $e=function(t,e,n){this.type=t,this.branches=e,this.otherwise=n};function tn(t,e){return"=="===t||"!="===t?"boolean"===e.kind||"string"===e.kind||"number"===e.kind||"null"===e.kind||"value"===e.kind:"string"===e.kind||"number"===e.kind||"value"===e.kind}function en(t,e,n,r){return 0===r.compare(e,n)}function nn(t,e,n){var r="=="!==t&&"!="!==t;return function(){function i(t,e,n){this.type=Kt,this.lhs=t,this.rhs=e,this.collator=n,this.hasUntypedArgument="value"===t.type.kind||"value"===e.type.kind}return i.parse=function(t,e){if(3!==t.length&&4!==t.length)return e.error("Expected two or three arguments.");var n=t[0],o=e.parse(t[1],1,qt);if(!o)return null;if(!tn(n,o.type))return e.concat(1).error('"'+n+"\" comparisons are not supported for type '"+Zt(o.type)+"'.");var a=e.parse(t[2],2,qt);if(!a)return null;if(!tn(n,a.type))return e.concat(2).error('"'+n+"\" comparisons are not supported for type '"+Zt(a.type)+"'.");if(o.type.kind!==a.type.kind&&"value"!==o.type.kind&&"value"!==a.type.kind)return e.error("Cannot compare types '"+Zt(o.type)+"' and '"+Zt(a.type)+"'.");r&&("value"===o.type.kind&&"value"!==a.type.kind?o=new ge(a.type,[o]):"value"!==o.type.kind&&"value"===a.type.kind&&(a=new ge(o.type,[a])));var s=null;if(4===t.length){if("string"!==o.type.kind&&"string"!==a.type.kind&&"value"!==o.type.kind&&"value"!==a.type.kind)return e.error("Cannot use collator to compare non-string types.");if(!(s=e.parse(t[3],3,Vt)))return null}return new i(o,a,s)},i.prototype.evaluate=function(i){var o=this.lhs.evaluate(i),a=this.rhs.evaluate(i);if(r&&this.hasUntypedArgument){var s=se(o),A=se(a);if(s.kind!==A.kind||"string"!==s.kind&&"number"!==s.kind)throw new ce('Expected arguments for "'+t+'" to be (string, string) or (number, number), but found ('+s.kind+", "+A.kind+") instead.")}if(this.collator&&!r&&this.hasUntypedArgument){var l=se(o),c=se(a);if("string"!==l.kind||"string"!==c.kind)return e(i,o,a)}return this.collator?n(i,o,a,this.collator.evaluate(i)):e(i,o,a)},i.prototype.eachChild=function(t){t(this.lhs),t(this.rhs),this.collator&&t(this.collator)},i.prototype.possibleOutputs=function(){return[!0,!1]},i.prototype.serialize=function(){var e=[t];return this.eachChild(function(t){e.push(t.serialize())}),e},i}()}$e.parse=function(t,e){if(t.length<4)return e.error("Expected at least 3 arguments, but found only "+(t.length-1)+".");if(t.length%2!=0)return e.error("Expected an odd number of arguments.");var n;e.expectedType&&"value"!==e.expectedType.kind&&(n=e.expectedType);for(var r=[],i=1;i<t.length-1;i+=2){var o=e.parse(t[i],i,Kt);if(!o)return null;var a=e.parse(t[i+1],i+1,n);if(!a)return null;r.push([o,a]),n=n||a.type}var s=e.parse(t[t.length-1],t.length-1,n);return s?new $e(n,r,s):null},$e.prototype.evaluate=function(t){for(var e=0,n=this.branches;e<n.length;e+=1){var r=n[e],i=r[0],o=r[1];if(i.evaluate(t))return o.evaluate(t)}return this.otherwise.evaluate(t)},$e.prototype.eachChild=function(t){for(var e=0,n=this.branches;e<n.length;e+=1){var r=n[e],i=r[0],o=r[1];t(i),t(o)}t(this.otherwise)},$e.prototype.possibleOutputs=function(){var t;return(t=[]).concat.apply(t,this.branches.map(function(t){return t[0],t[1].possibleOutputs()})).concat(this.otherwise.possibleOutputs())},$e.prototype.serialize=function(){var t=["case"];return this.eachChild(function(e){t.push(e.serialize())}),t};var rn=nn("==",function(t,e,n){return e===n},en),on=nn("!=",function(t,e,n){return e!==n},function(t,e,n,r){return!en(0,e,n,r)}),an=nn("<",function(t,e,n){return e<n},function(t,e,n,r){return r.compare(e,n)<0}),sn=nn(">",function(t,e,n){return e>n},function(t,e,n,r){return r.compare(e,n)>0}),An=nn("<=",function(t,e,n){return e<=n},function(t,e,n,r){return r.compare(e,n)<=0}),ln=nn(">=",function(t,e,n){return e>=n},function(t,e,n,r){return r.compare(e,n)>=0}),cn=function(t,e,n,r,i){this.type=Pt,this.number=t,this.locale=e,this.currency=n,this.minFractionDigits=r,this.maxFractionDigits=i};cn.parse=function(t,e){if(3!==t.length)return e.error("Expected two arguments.");var n=e.parse(t[1],1,Yt);if(!n)return null;var r=t[2];if("object"!=typeof r||Array.isArray(r))return e.error("NumberFormat options argument must be an object.");var i=null;if(r.locale&&!(i=e.parse(r.locale,1,Pt)))return null;var o=null;if(r.currency&&!(o=e.parse(r.currency,1,Pt)))return null;var a=null;if(r["min-fraction-digits"]&&!(a=e.parse(r["min-fraction-digits"],1,Yt)))return null;var s=null;return r["max-fraction-digits"]&&!(s=e.parse(r["max-fraction-digits"],1,Yt))?null:new cn(n,i,o,a,s)},cn.prototype.evaluate=function(t){return new Intl.NumberFormat(this.locale?this.locale.evaluate(t):[],{style:this.currency?"currency":"decimal",currency:this.currency?this.currency.evaluate(t):void 0,minimumFractionDigits:this.minFractionDigits?this.minFractionDigits.evaluate(t):void 0,maximumFractionDigits:this.maxFractionDigits?this.maxFractionDigits.evaluate(t):void 0}).format(this.number.evaluate(t))},cn.prototype.eachChild=function(t){t(this.number),this.locale&&t(this.locale),this.currency&&t(this.currency),this.minFractionDigits&&t(this.minFractionDigits),this.maxFractionDigits&&t(this.maxFractionDigits)},cn.prototype.possibleOutputs=function(){return[void 0]},cn.prototype.serialize=function(){var t={};return this.locale&&(t.locale=this.locale.serialize()),this.currency&&(t.currency=this.currency.serialize()),this.minFractionDigits&&(t["min-fraction-digits"]=this.minFractionDigits.serialize()),this.maxFractionDigits&&(t["max-fraction-digits"]=this.maxFractionDigits.serialize()),["number-format",this.number.serialize(),t]};var un=function(t){this.type=Yt,this.input=t};un.parse=function(t,e){if(2!==t.length)return e.error("Expected 1 argument, but found "+(t.length-1)+" instead.");var n=e.parse(t[1],1);return n?"array"!==n.type.kind&&"string"!==n.type.kind&&"value"!==n.type.kind?e.error("Expected argument of type string or array, but found "+Zt(n.type)+" instead."):new un(n):null},un.prototype.evaluate=function(t){var e=this.input.evaluate(t);if("string"==typeof e)return e.length;if(Array.isArray(e))return e.length;throw new ce("Expected value to be of type string or array, but found "+Zt(se(e))+" instead.")},un.prototype.eachChild=function(t){t(this.input)},un.prototype.possibleOutputs=function(){return[void 0]},un.prototype.serialize=function(){var t=["length"];return this.eachChild(function(e){t.push(e.serialize())}),t};var gn={"==":rn,"!=":on,">":sn,"<":an,">=":ln,"<=":An,array:ge,at:Xe,boolean:ge,case:$e,coalesce:Ve,collator:me,format:he,interpolate:Je,"interpolate-hcl":Je,"interpolate-lab":Je,length:un,let:We,literal:le,match:Ze,number:ge,"number-format":cn,object:ge,step:we,string:ge,"to-boolean":fe,"to-color":fe,"to-number":fe,"to-string":fe,var:Be};function hn(t,e){var n=e[0],r=e[1],i=e[2],o=e[3];n=n.evaluate(t),r=r.evaluate(t),i=i.evaluate(t);var a=o?o.evaluate(t):1,s=ae(n,r,i,a);if(s)throw new ce(s);return new ne(n/255*a,r/255*a,i/255*a,a)}function dn(t,e){return t in e}function fn(t,e){var n=e[t];return void 0===n?null:n}function pn(t){return{type:t}}function In(t){return{result:"success",value:t}}function Cn(t){return{result:"error",value:t}}function mn(t){return"data-driven"===t["property-type"]||"cross-faded-data-driven"===t["property-type"]}function yn(t){return!!t.expression&&t.expression.parameters.indexOf("zoom")>-1}function En(t){return!!t.expression&&t.expression.interpolated}function vn(t){return t instanceof Number?"number":t instanceof String?"string":t instanceof Boolean?"boolean":Array.isArray(t)?"array":null===t?"null":typeof t}function Bn(t){return"object"==typeof t&&null!==t&&!Array.isArray(t)}function bn(t){return t}function Qn(t,e,n){return void 0!==t?t:void 0!==e?e:void 0!==n?n:void 0}function wn(t,e,n,r,i){return Qn(typeof n===i?r[n]:void 0,t.default,e.default)}function xn(t,e,n){if("number"!==vn(n))return Qn(t.default,e.default);var r=t.stops.length;if(1===r)return t.stops[0][1];if(n<=t.stops[0][0])return t.stops[0][1];if(n>=t.stops[r-1][0])return t.stops[r-1][1];var i=Qe(t.stops.map(function(t){return t[0]}),n);return t.stops[i][1]}function _n(t,e,n){var r=void 0!==t.base?t.base:1;if("number"!==vn(n))return Qn(t.default,e.default);var i=t.stops.length;if(1===i)return t.stops[0][1];if(n<=t.stops[0][0])return t.stops[0][1];if(n>=t.stops[i-1][0])return t.stops[i-1][1];var o=Qe(t.stops.map(function(t){return t[0]}),n),a=function(t,e,n,r){var i=r-n,o=t-n;return 0===i?0:1===e?o/i:(Math.pow(e,o)-1)/(Math.pow(e,i)-1)}(n,r,t.stops[o][0],t.stops[o+1][0]),s=t.stops[o][1],A=t.stops[o+1][1],l=_e[e.type]||bn;if(t.colorSpace&&"rgb"!==t.colorSpace){var c=ze[t.colorSpace];l=function(t,e){return c.reverse(c.interpolate(c.forward(t),c.forward(e),a))}}return"function"==typeof s.evaluate?{evaluate:function(){for(var t=[],e=arguments.length;e--;)t[e]=arguments[e];var n=s.evaluate.apply(void 0,t),r=A.evaluate.apply(void 0,t);if(void 0!==n&&void 0!==r)return l(n,r,a)}}:l(s,A,a)}function Dn(t,e,n){return"color"===e.type?n=ne.parse(n):"formatted"===e.type?n=oe.fromString(n.toString()):vn(n)===e.type||"enum"===e.type&&e.values[n]||(n=void 0),Qn(n,t.default,e.default)}Ce.register(gn,{error:[{kind:"error"},[Pt],function(t,e){var n=e[0];throw new ce(n.evaluate(t))}],typeof:[Pt,[qt],function(t,e){return Zt(se(e[0].evaluate(t)))}],"to-rgba":[Xt(Yt,4),[zt],function(t,e){return e[0].evaluate(t).toArray()}],rgb:[zt,[Yt,Yt,Yt],hn],rgba:[zt,[Yt,Yt,Yt,Yt],hn],has:{type:Kt,overloads:[[[Pt],function(t,e){return dn(e[0].evaluate(t),t.properties())}],[[Pt,Jt],function(t,e){var n=e[0],r=e[1];return dn(n.evaluate(t),r.evaluate(t))}]]},get:{type:qt,overloads:[[[Pt],function(t,e){return fn(e[0].evaluate(t),t.properties())}],[[Pt,Jt],function(t,e){var n=e[0],r=e[1];return fn(n.evaluate(t),r.evaluate(t))}]]},"feature-state":[qt,[Pt],function(t,e){return fn(e[0].evaluate(t),t.featureState||{})}],properties:[Jt,[],function(t){return t.properties()}],"geometry-type":[Pt,[],function(t){return t.geometryType()}],id:[qt,[],function(t){return t.id()}],zoom:[Yt,[],function(t){return t.globals.zoom}],"heatmap-density":[Yt,[],function(t){return t.globals.heatmapDensity||0}],"line-progress":[Yt,[],function(t){return t.globals.lineProgress||0}],accumulated:[qt,[],function(t){return void 0===t.globals.accumulated?null:t.globals.accumulated}],"+":[Yt,pn(Yt),function(t,e){for(var n=0,r=0,i=e;r<i.length;r+=1)n+=i[r].evaluate(t);return n}],"*":[Yt,pn(Yt),function(t,e){for(var n=1,r=0,i=e;r<i.length;r+=1)n*=i[r].evaluate(t);return n}],"-":{type:Yt,overloads:[[[Yt,Yt],function(t,e){var n=e[0],r=e[1];return n.evaluate(t)-r.evaluate(t)}],[[Yt],function(t,e){return-e[0].evaluate(t)}]]},"/":[Yt,[Yt,Yt],function(t,e){var n=e[0],r=e[1];return n.evaluate(t)/r.evaluate(t)}],"%":[Yt,[Yt,Yt],function(t,e){var n=e[0],r=e[1];return n.evaluate(t)%r.evaluate(t)}],ln2:[Yt,[],function(){return Math.LN2}],pi:[Yt,[],function(){return Math.PI}],e:[Yt,[],function(){return Math.E}],"^":[Yt,[Yt,Yt],function(t,e){var n=e[0],r=e[1];return Math.pow(n.evaluate(t),r.evaluate(t))}],sqrt:[Yt,[Yt],function(t,e){var n=e[0];return Math.sqrt(n.evaluate(t))}],log10:[Yt,[Yt],function(t,e){var n=e[0];return Math.log(n.evaluate(t))/Math.LN10}],ln:[Yt,[Yt],function(t,e){var n=e[0];return Math.log(n.evaluate(t))}],log2:[Yt,[Yt],function(t,e){var n=e[0];return Math.log(n.evaluate(t))/Math.LN2}],sin:[Yt,[Yt],function(t,e){var n=e[0];return Math.sin(n.evaluate(t))}],cos:[Yt,[Yt],function(t,e){var n=e[0];return Math.cos(n.evaluate(t))}],tan:[Yt,[Yt],function(t,e){var n=e[0];return Math.tan(n.evaluate(t))}],asin:[Yt,[Yt],function(t,e){var n=e[0];return Math.asin(n.evaluate(t))}],acos:[Yt,[Yt],function(t,e){var n=e[0];return Math.acos(n.evaluate(t))}],atan:[Yt,[Yt],function(t,e){var n=e[0];return Math.atan(n.evaluate(t))}],min:[Yt,pn(Yt),function(t,e){return Math.min.apply(Math,e.map(function(e){return e.evaluate(t)}))}],max:[Yt,pn(Yt),function(t,e){return Math.max.apply(Math,e.map(function(e){return e.evaluate(t)}))}],abs:[Yt,[Yt],function(t,e){var n=e[0];return Math.abs(n.evaluate(t))}],round:[Yt,[Yt],function(t,e){var n=e[0].evaluate(t);return n<0?-Math.round(-n):Math.round(n)}],floor:[Yt,[Yt],function(t,e){var n=e[0];return Math.floor(n.evaluate(t))}],ceil:[Yt,[Yt],function(t,e){var n=e[0];return Math.ceil(n.evaluate(t))}],"filter-==":[Kt,[Pt,qt],function(t,e){var n=e[0],r=e[1];return t.properties()[n.value]===r.value}],"filter-id-==":[Kt,[qt],function(t,e){var n=e[0];return t.id()===n.value}],"filter-type-==":[Kt,[Pt],function(t,e){var n=e[0];return t.geometryType()===n.value}],"filter-<":[Kt,[Pt,qt],function(t,e){var n=e[0],r=e[1],i=t.properties()[n.value],o=r.value;return typeof i==typeof o&&i<o}],"filter-id-<":[Kt,[qt],function(t,e){var n=e[0],r=t.id(),i=n.value;return typeof r==typeof i&&r<i}],"filter->":[Kt,[Pt,qt],function(t,e){var n=e[0],r=e[1],i=t.properties()[n.value],o=r.value;return typeof i==typeof o&&i>o}],"filter-id->":[Kt,[qt],function(t,e){var n=e[0],r=t.id(),i=n.value;return typeof r==typeof i&&r>i}],"filter-<=":[Kt,[Pt,qt],function(t,e){var n=e[0],r=e[1],i=t.properties()[n.value],o=r.value;return typeof i==typeof o&&i<=o}],"filter-id-<=":[Kt,[qt],function(t,e){var n=e[0],r=t.id(),i=n.value;return typeof r==typeof i&&r<=i}],"filter->=":[Kt,[Pt,qt],function(t,e){var n=e[0],r=e[1],i=t.properties()[n.value],o=r.value;return typeof i==typeof o&&i>=o}],"filter-id->=":[Kt,[qt],function(t,e){var n=e[0],r=t.id(),i=n.value;return typeof r==typeof i&&r>=i}],"filter-has":[Kt,[qt],function(t,e){return e[0].value in t.properties()}],"filter-has-id":[Kt,[],function(t){return null!==t.id()}],"filter-type-in":[Kt,[Xt(Pt)],function(t,e){return e[0].value.indexOf(t.geometryType())>=0}],"filter-id-in":[Kt,[Xt(qt)],function(t,e){return e[0].value.indexOf(t.id())>=0}],"filter-in-small":[Kt,[Pt,Xt(qt)],function(t,e){var n=e[0];return e[1].value.indexOf(t.properties()[n.value])>=0}],"filter-in-large":[Kt,[Pt,Xt(qt)],function(t,e){var n=e[0],r=e[1];return function(t,e,n,r){for(;n<=r;){var i=n+r>>1;if(e[i]===t)return!0;e[i]>t?r=i-1:n=i+1}return!1}(t.properties()[n.value],r.value,0,r.value.length-1)}],all:{type:Kt,overloads:[[[Kt,Kt],function(t,e){var n=e[0],r=e[1];return n.evaluate(t)&&r.evaluate(t)}],[pn(Kt),function(t,e){for(var n=0,r=e;n<r.length;n+=1)if(!r[n].evaluate(t))return!1;return!0}]]},any:{type:Kt,overloads:[[[Kt,Kt],function(t,e){var n=e[0],r=e[1];return n.evaluate(t)||r.evaluate(t)}],[pn(Kt),function(t,e){for(var n=0,r=e;n<r.length;n+=1)if(r[n].evaluate(t))return!0;return!1}]]},"!":[Kt,[Kt],function(t,e){return!e[0].evaluate(t)}],"is-supported-script":[Kt,[Pt],function(t,e){var n=e[0],r=t.globals&&t.globals.isSupportedScript;return!r||r(n.evaluate(t))}],upcase:[Pt,[Pt],function(t,e){return e[0].evaluate(t).toUpperCase()}],downcase:[Pt,[Pt],function(t,e){return e[0].evaluate(t).toLowerCase()}],concat:[Pt,pn(qt),function(t,e){return e.map(function(e){return Ae(e.evaluate(t))}).join("")}],"resolved-locale":[Pt,[Vt],function(t,e){return e[0].evaluate(t).resolvedLocale()}]});var Mn=function(t,e){var n;this.expression=t,this._warningHistory={},this._evaluator=new Ie,this._defaultValue=e?"color"===(n=e).type&&Bn(n.default)?new ne(0,0,0,0):"color"===n.type?ne.parse(n.default)||null:void 0===n.default?null:n.default:null,this._enumValues=e&&"enum"===e.type?e.values:null};function kn(t){return Array.isArray(t)&&t.length>0&&"string"==typeof t[0]&&t[0]in gn}function Sn(t,e){var n=new be(gn,[],e?function(t){var e={color:zt,string:Pt,number:Yt,enum:Pt,boolean:Kt,formatted:Wt};return"array"===t.type?Xt(e[t.value]||qt,t.length):e[t.type]}(e):void 0),r=n.parse(t,void 0,void 0,void 0,e&&"string"===e.type?{typeAnnotation:"coerce"}:void 0);return r?In(new Mn(r,e)):Cn(n.errors)}Mn.prototype.evaluateWithoutErrorHandling=function(t,e,n){return this._evaluator.globals=t,this._evaluator.feature=e,this._evaluator.featureState=n,this.expression.evaluate(this._evaluator)},Mn.prototype.evaluate=function(t,e,n){this._evaluator.globals=t,this._evaluator.feature=e||null,this._evaluator.featureState=n||null;try{var r=this.expression.evaluate(this._evaluator);if(null==r)return this._defaultValue;if(this._enumValues&&!(r in this._enumValues))throw new ce("Expected value to be one of "+Object.keys(this._enumValues).map(function(t){return JSON.stringify(t)}).join(", ")+", but found "+JSON.stringify(r)+" instead.");return r}catch(t){return this._warningHistory[t.message]||(this._warningHistory[t.message]=!0,"undefined"!=typeof console&&console.warn(t.message)),this._defaultValue}};var Ln=function(t,e){this.kind=t,this._styleExpression=e,this.isStateDependent="constant"!==t&&!Ee(e.expression)};Ln.prototype.evaluateWithoutErrorHandling=function(t,e,n){return this._styleExpression.evaluateWithoutErrorHandling(t,e,n)},Ln.prototype.evaluate=function(t,e,n){return this._styleExpression.evaluate(t,e,n)};var Rn=function(t,e,n){this.kind=t,this.zoomStops=n.labels,this._styleExpression=e,this.isStateDependent="camera"!==t&&!Ee(e.expression),n instanceof Je&&(this.interpolationType=n.interpolation)};function Nn(t,e){if("error"===(t=Sn(t,e)).result)return t;var n=t.value.expression,r=ye(n);if(!r&&!mn(e))return Cn([new Gt("","data expressions not supported")]);var i=ve(n,["zoom"]);if(!i&&!yn(e))return Cn([new Gt("","zoom expressions not supported")]);var o=function t(e){var n=null;if(e instanceof We)n=t(e.result);else if(e instanceof Ve)for(var r=0,i=e.args;r<i.length;r+=1){var o=i[r];if(n=t(o))break}else(e instanceof we||e instanceof Je)&&e.input instanceof Ce&&"zoom"===e.input.name&&(n=e);return n instanceof Gt?n:(e.eachChild(function(e){var r=t(e);r instanceof Gt?n=r:!n&&r?n=new Gt("",'"zoom" expression may only be used as input to a top-level "step" or "interpolate" expression.'):n&&r&&n!==r&&(n=new Gt("",'Only one zoom-based "step" or "interpolate" subexpression may be used in an expression.'))}),n)}(n);return o||i?o instanceof Gt?Cn([o]):o instanceof Je&&!En(e)?Cn([new Gt("",'"interpolate" expressions cannot be used with this property')]):In(o?new Rn(r?"camera":"composite",t.value,o):new Ln(r?"constant":"source",t.value)):Cn([new Gt("",'"zoom" expression may only be used as input to a top-level "step" or "interpolate" expression.')])}Rn.prototype.evaluateWithoutErrorHandling=function(t,e,n){return this._styleExpression.evaluateWithoutErrorHandling(t,e,n)},Rn.prototype.evaluate=function(t,e,n){return this._styleExpression.evaluate(t,e,n)},Rn.prototype.interpolationFactor=function(t,e,n){return this.interpolationType?Je.interpolationFactor(this.interpolationType,t,e,n):0};var Fn=function(t,e){this._parameters=t,this._specification=e,Ft(this,function t(e,n){var r,i,o,a="color"===n.type,s=e.stops&&"object"==typeof e.stops[0][0],A=s||void 0!==e.property,l=s||!A,c=e.type||(En(n)?"exponential":"interval");if(a&&((e=Ft({},e)).stops&&(e.stops=e.stops.map(function(t){return[t[0],ne.parse(t[1])]})),e.default?e.default=ne.parse(e.default):e.default=ne.parse(n.default)),e.colorSpace&&"rgb"!==e.colorSpace&&!ze[e.colorSpace])throw new Error("Unknown color space: "+e.colorSpace);if("exponential"===c)r=_n;else if("interval"===c)r=xn;else if("categorical"===c){r=wn,i=Object.create(null);for(var u=0,g=e.stops;u<g.length;u+=1){var h=g[u];i[h[0]]=h[1]}o=typeof e.stops[0][0]}else{if("identity"!==c)throw new Error('Unknown function type "'+c+'"');r=Dn}if(s){for(var d={},f=[],p=0;p<e.stops.length;p++){var I=e.stops[p],C=I[0].zoom;void 0===d[C]&&(d[C]={zoom:C,type:e.type,property:e.property,default:e.default,stops:[]},f.push(C)),d[C].stops.push([I[0].value,I[1]])}for(var m=[],y=0,E=f;y<E.length;y+=1){var v=E[y];m.push([d[v].zoom,t(d[v],n)])}var B={name:"linear"};return{kind:"composite",interpolationType:B,interpolationFactor:Je.interpolationFactor.bind(void 0,B),zoomStops:m.map(function(t){return t[0]}),evaluate:function(t,r){var i=t.zoom;return _n({stops:m,base:e.base},n,i).evaluate(i,r)}}}if(l){var b="exponential"===c?{name:"exponential",base:void 0!==e.base?e.base:1}:null;return{kind:"camera",interpolationType:b,interpolationFactor:Je.interpolationFactor.bind(void 0,b),zoomStops:e.stops.map(function(t){return t[0]}),evaluate:function(t){var a=t.zoom;return r(e,n,a,i,o)}}}return{kind:"source",evaluate:function(t,a){var s=a&&a.properties?a.properties[e.property]:void 0;return void 0===s?Qn(e.default,n.default):r(e,n,s,i,o)}}}(this._parameters,this._specification))};function Tn(t){var e=t.key,n=t.value,r=t.valueSpec||{},i=t.objectElementValidators||{},o=t.style,a=t.styleSpec,s=[],A=vn(n);if("object"!==A)return[new Rt(e,n,"object expected, "+A+" found")];for(var l in n){var c=l.split(".")[0],u=r[c]||r["*"],g=void 0;if(i[c])g=i[c];else if(r[c])g=sr;else if(i["*"])g=i["*"];else{if(!r["*"]){s.push(new Rt(e,n[l],'unknown property "'+l+'"'));continue}g=sr}s=s.concat(g({key:(e?e+".":e)+l,value:n[l],valueSpec:u,style:o,styleSpec:a,object:n,objectKey:l},n))}for(var h in r)i[h]||r[h].required&&void 0===r[h].default&&void 0===n[h]&&s.push(new Rt(e,n,'missing required property "'+h+'"'));return s}function Un(t){var e=t.value,n=t.valueSpec,r=t.style,i=t.styleSpec,o=t.key,a=t.arrayElementValidator||sr;if("array"!==vn(e))return[new Rt(o,e,"array expected, "+vn(e)+" found")];if(n.length&&e.length!==n.length)return[new Rt(o,e,"array length "+n.length+" expected, length "+e.length+" found")];if(n["min-length"]&&e.length<n["min-length"])return[new Rt(o,e,"array length at least "+n["min-length"]+" expected, length "+e.length+" found")];var s={type:n.value,values:n.values};i.$version<7&&(s.function=n.function),"object"===vn(n.value)&&(s=n.value);for(var A=[],l=0;l<e.length;l++)A=A.concat(a({array:e,arrayIndex:l,value:e[l],valueSpec:s,style:r,styleSpec:i,key:o+"["+l+"]"}));return A}function On(t){var e=t.key,n=t.value,r=t.valueSpec,i=vn(n);return"number"!==i?[new Rt(e,n,"number expected, "+i+" found")]:"minimum"in r&&n<r.minimum?[new Rt(e,n,n+" is less than the minimum value "+r.minimum)]:"maximum"in r&&n>r.maximum?[new Rt(e,n,n+" is greater than the maximum value "+r.maximum)]:[]}function Gn(t){var e,n,r,i=t.valueSpec,o=Ut(t.value.type),a={},s="categorical"!==o&&void 0===t.value.property,A=!s,l="array"===vn(t.value.stops)&&"array"===vn(t.value.stops[0])&&"object"===vn(t.value.stops[0][0]),c=Tn({key:t.key,value:t.value,valueSpec:t.styleSpec.function,style:t.style,styleSpec:t.styleSpec,objectElementValidators:{stops:function(t){if("identity"===o)return[new Rt(t.key,t.value,'identity function may not have a "stops" property')];var e=[],n=t.value;return e=e.concat(Un({key:t.key,value:n,valueSpec:t.valueSpec,style:t.style,styleSpec:t.styleSpec,arrayElementValidator:u})),"array"===vn(n)&&0===n.length&&e.push(new Rt(t.key,n,"array must have at least one stop")),e},default:function(t){return sr({key:t.key,value:t.value,valueSpec:i,style:t.style,styleSpec:t.styleSpec})}}});return"identity"===o&&s&&c.push(new Rt(t.key,t.value,'missing required property "property"')),"identity"===o||t.value.stops||c.push(new Rt(t.key,t.value,'missing required property "stops"')),"exponential"===o&&t.valueSpec.expression&&!En(t.valueSpec)&&c.push(new Rt(t.key,t.value,"exponential functions not supported")),t.styleSpec.$version>=8&&(A&&!mn(t.valueSpec)?c.push(new Rt(t.key,t.value,"property functions not supported")):s&&!yn(t.valueSpec)&&c.push(new Rt(t.key,t.value,"zoom functions not supported"))),"categorical"!==o&&!l||void 0!==t.value.property||c.push(new Rt(t.key,t.value,'"property" property is required')),c;function u(t){var e=[],o=t.value,s=t.key;if("array"!==vn(o))return[new Rt(s,o,"array expected, "+vn(o)+" found")];if(2!==o.length)return[new Rt(s,o,"array length 2 expected, length "+o.length+" found")];if(l){if("object"!==vn(o[0]))return[new Rt(s,o,"object expected, "+vn(o[0])+" found")];if(void 0===o[0].zoom)return[new Rt(s,o,"object stop key must have zoom")];if(void 0===o[0].value)return[new Rt(s,o,"object stop key must have value")];if(r&&r>Ut(o[0].zoom))return[new Rt(s,o[0].zoom,"stop zoom values must appear in ascending order")];Ut(o[0].zoom)!==r&&(r=Ut(o[0].zoom),n=void 0,a={}),e=e.concat(Tn({key:s+"[0]",value:o[0],valueSpec:{zoom:{}},style:t.style,styleSpec:t.styleSpec,objectElementValidators:{zoom:On,value:g}}))}else e=e.concat(g({key:s+"[0]",value:o[0],valueSpec:{},style:t.style,styleSpec:t.styleSpec},o));return kn(Ot(o[1]))?e.concat([new Rt(s+"[1]",o[1],"expressions are not allowed in function stops.")]):e.concat(sr({key:s+"[1]",value:o[1],valueSpec:i,style:t.style,styleSpec:t.styleSpec}))}function g(t,r){var s=vn(t.value),A=Ut(t.value),l=null!==t.value?t.value:r;if(e){if(s!==e)return[new Rt(t.key,l,s+" stop domain type must match previous stop domain type "+e)]}else e=s;if("number"!==s&&"string"!==s&&"boolean"!==s)return[new Rt(t.key,l,"stop domain value must be a number, string, or boolean")];if("number"!==s&&"categorical"!==o){var c="number expected, "+s+" found";return mn(i)&&void 0===o&&(c+='\nIf you intended to use a categorical function, specify `"type": "categorical"`.'),[new Rt(t.key,l,c)]}return"categorical"!==o||"number"!==s||isFinite(A)&&Math.floor(A)===A?"categorical"!==o&&"number"===s&&void 0!==n&&A<n?[new Rt(t.key,l,"stop domain values must appear in ascending order")]:(n=A,"categorical"===o&&A in a?[new Rt(t.key,l,"stop domain values must be unique")]:(a[A]=!0,[])):[new Rt(t.key,l,"integer expected, found "+A)]}}function Hn(t){var e=("property"===t.expressionContext?Nn:Sn)(Ot(t.value),t.valueSpec);if("error"===e.result)return e.value.map(function(e){return new Rt(""+t.key+e.key,t.value,e.message)});var n=e.value.expression||e.value._styleExpression.expression;if("property"===t.expressionContext&&"text-font"===t.propertyKey&&-1!==n.possibleOutputs().indexOf(void 0))return[new Rt(t.key,t.value,'Invalid data expression for "'+t.propertyKey+'". Output values must be contained as literals within the expression.')];if("property"===t.expressionContext&&"layout"===t.propertyType&&!Ee(n))return[new Rt(t.key,t.value,'"feature-state" data expressions are not supported with layout properties.')];if("filter"===t.expressionContext&&!Ee(n))return[new Rt(t.key,t.value,'"feature-state" data expressions are not supported with filters.')];if(t.expressionContext&&0===t.expressionContext.indexOf("cluster")){if(!ve(n,["zoom","feature-state"]))return[new Rt(t.key,t.value,'"zoom" and "feature-state" expressions are not supported with cluster properties.')];if("cluster-initial"===t.expressionContext&&!ye(n))return[new Rt(t.key,t.value,"Feature data expressions are not supported with initial expression part of cluster properties.")]}return[]}function jn(t){var e=t.key,n=t.value,r=t.valueSpec,i=[];return Array.isArray(r.values)?-1===r.values.indexOf(Ut(n))&&i.push(new Rt(e,n,"expected one of ["+r.values.join(", ")+"], "+JSON.stringify(n)+" found")):-1===Object.keys(r.values).indexOf(Ut(n))&&i.push(new Rt(e,n,"expected one of ["+Object.keys(r.values).join(", ")+"], "+JSON.stringify(n)+" found")),i}function Yn(t){if(!0===t||!1===t)return!0;if(!Array.isArray(t)||0===t.length)return!1;switch(t[0]){case"has":return t.length>=2&&"$id"!==t[1]&&"$type"!==t[1];case"in":case"!in":case"!has":case"none":return!1;case"==":case"!=":case">":case">=":case"<":case"<=":return 3!==t.length||Array.isArray(t[1])||Array.isArray(t[2]);case"any":case"all":for(var e=0,n=t.slice(1);e<n.length;e+=1){var r=n[e];if(!Yn(r)&&"boolean"!=typeof r)return!1}return!0;default:return!0}}Fn.deserialize=function(t){return new Fn(t._parameters,t._specification)},Fn.serialize=function(t){return{_parameters:t._parameters,_specification:t._specification}};var Pn={type:"boolean",default:!1,transition:!1,"property-type":"data-driven",expression:{interpolated:!1,parameters:["zoom","feature"]}};function Kn(t){if(null==t)return function(){return!0};Yn(t)||(t=Jn(t));var e=Sn(t,Pn);if("error"===e.result)throw new Error(e.value.map(function(t){return t.key+": "+t.message}).join(", "));return function(t,n){return e.value.evaluate(t,n)}}function zn(t,e){return t<e?-1:t>e?1:0}function Jn(t){if(!t)return!0;var e,n=t[0];return t.length<=1?"any"!==n:"=="===n?qn(t[1],t[2],"=="):"!="===n?Xn(qn(t[1],t[2],"==")):"<"===n||">"===n||"<="===n||">="===n?qn(t[1],t[2],n):"any"===n?(e=t.slice(1),["any"].concat(e.map(Jn))):"all"===n?["all"].concat(t.slice(1).map(Jn)):"none"===n?["all"].concat(t.slice(1).map(Jn).map(Xn)):"in"===n?Vn(t[1],t.slice(2)):"!in"===n?Xn(Vn(t[1],t.slice(2))):"has"===n?Wn(t[1]):"!has"!==n||Xn(Wn(t[1]))}function qn(t,e,n){switch(t){case"$type":return["filter-type-"+n,e];case"$id":return["filter-id-"+n,e];default:return["filter-"+n,t,e]}}function Vn(t,e){if(0===e.length)return!1;switch(t){case"$type":return["filter-type-in",["literal",e]];case"$id":return["filter-id-in",["literal",e]];default:return e.length>200&&!e.some(function(t){return typeof t!=typeof e[0]})?["filter-in-large",t,["literal",e.sort(zn)]]:["filter-in-small",t,["literal",e]]}}function Wn(t){switch(t){case"$type":return!0;case"$id":return["filter-has-id"];default:return["filter-has",t]}}function Xn(t){return["!",t]}function Zn(t){return Yn(Ot(t.value))?Hn(Ft({},t,{expressionContext:"filter",valueSpec:{value:"boolean"}})):function t(e){var n=e.value,r=e.key;if("array"!==vn(n))return[new Rt(r,n,"array expected, "+vn(n)+" found")];var i,o=e.styleSpec,a=[];if(n.length<1)return[new Rt(r,n,"filter array must have at least 1 element")];switch(a=a.concat(jn({key:r+"[0]",value:n[0],valueSpec:o.filter_operator,style:e.style,styleSpec:e.styleSpec})),Ut(n[0])){case"<":case"<=":case">":case">=":n.length>=2&&"$type"===Ut(n[1])&&a.push(new Rt(r,n,'"$type" cannot be use with operator "'+n[0]+'"'));case"==":case"!=":3!==n.length&&a.push(new Rt(r,n,'filter array for operator "'+n[0]+'" must have 3 elements'));case"in":case"!in":n.length>=2&&"string"!==(i=vn(n[1]))&&a.push(new Rt(r+"[1]",n[1],"string expected, "+i+" found"));for(var s=2;s<n.length;s++)i=vn(n[s]),"$type"===Ut(n[1])?a=a.concat(jn({key:r+"["+s+"]",value:n[s],valueSpec:o.geometry_type,style:e.style,styleSpec:e.styleSpec})):"string"!==i&&"number"!==i&&"boolean"!==i&&a.push(new Rt(r+"["+s+"]",n[s],"string, number, or boolean expected, "+i+" found"));break;case"any":case"all":case"none":for(var A=1;A<n.length;A++)a=a.concat(t({key:r+"["+A+"]",value:n[A],style:e.style,styleSpec:e.styleSpec}));break;case"has":case"!has":i=vn(n[1]),2!==n.length?a.push(new Rt(r,n,'filter array for "'+n[0]+'" operator must have 2 elements')):"string"!==i&&a.push(new Rt(r+"[1]",n[1],"string expected, "+i+" found"))}return a}(t)}function $n(t,e){var n=t.key,r=t.style,i=t.styleSpec,o=t.value,a=t.objectKey,s=i[e+"_"+t.layerType];if(!s)return[];var A=a.match(/^(.*)-transition$/);if("paint"===e&&A&&s[A[1]]&&s[A[1]].transition)return sr({key:n,value:o,valueSpec:i.transition,style:r,styleSpec:i});var l,c=t.valueSpec||s[a];if(!c)return[new Rt(n,o,'unknown property "'+a+'"')];if("string"===vn(o)&&mn(c)&&!c.tokens&&(l=/^{([^}]+)}$/.exec(o)))return[new Rt(n,o,'"'+a+'" does not support interpolation syntax\nUse an identity property function instead: `{ "type": "identity", "property": '+JSON.stringify(l[1])+" }`.")];var u=[];return"symbol"===t.layerType&&("text-field"===a&&r&&!r.glyphs&&u.push(new Rt(n,o,'use of "text-field" requires a style "glyphs" property')),"text-font"===a&&Bn(Ot(o))&&"identity"===Ut(o.type)&&u.push(new Rt(n,o,'"text-font" does not support identity functions'))),u.concat(sr({key:t.key,value:o,valueSpec:c,style:r,styleSpec:i,expressionContext:"property",propertyType:e,propertyKey:a}))}function tr(t){return $n(t,"paint")}function er(t){return $n(t,"layout")}function nr(t){var e=[],n=t.value,r=t.key,i=t.style,o=t.styleSpec;n.type||n.ref||e.push(new Rt(r,n,'either "type" or "ref" is required'));var a,s=Ut(n.type),A=Ut(n.ref);if(n.id)for(var l=Ut(n.id),c=0;c<t.arrayIndex;c++){var u=i.layers[c];Ut(u.id)===l&&e.push(new Rt(r,n.id,'duplicate layer id "'+n.id+'", previously used at line '+u.id.__line__))}if("ref"in n)["type","source","source-layer","filter","layout"].forEach(function(t){t in n&&e.push(new Rt(r,n[t],'"'+t+'" is prohibited for ref layers'))}),i.layers.forEach(function(t){Ut(t.id)===A&&(a=t)}),a?a.ref?e.push(new Rt(r,n.ref,"ref cannot reference another ref layer")):s=Ut(a.type):e.push(new Rt(r,n.ref,'ref layer "'+A+'" not found'));else if("background"!==s)if(n.source){var g=i.sources&&i.sources[n.source],h=g&&Ut(g.type);g?"vector"===h&&"raster"===s?e.push(new Rt(r,n.source,'layer "'+n.id+'" requires a raster source')):"raster"===h&&"raster"!==s?e.push(new Rt(r,n.source,'layer "'+n.id+'" requires a vector source')):"vector"!==h||n["source-layer"]?"raster-dem"===h&&"hillshade"!==s?e.push(new Rt(r,n.source,"raster-dem source can only be used with layer type 'hillshade'.")):"line"!==s||!n.paint||!n.paint["line-gradient"]||"geojson"===h&&g.lineMetrics||e.push(new Rt(r,n,'layer "'+n.id+'" specifies a line-gradient, which requires a GeoJSON source with `lineMetrics` enabled.')):e.push(new Rt(r,n,'layer "'+n.id+'" must specify a "source-layer"')):e.push(new Rt(r,n.source,'source "'+n.source+'" not found'))}else e.push(new Rt(r,n,'missing required property "source"'));return e=e.concat(Tn({key:r,value:n,valueSpec:o.layer,style:t.style,styleSpec:t.styleSpec,objectElementValidators:{"*":function(){return[]},type:function(){return sr({key:r+".type",value:n.type,valueSpec:o.layer.type,style:t.style,styleSpec:t.styleSpec,object:n,objectKey:"type"})},filter:Zn,layout:function(t){return Tn({layer:n,key:t.key,value:t.value,style:t.style,styleSpec:t.styleSpec,objectElementValidators:{"*":function(t){return er(Ft({layerType:s},t))}}})},paint:function(t){return Tn({layer:n,key:t.key,value:t.value,style:t.style,styleSpec:t.styleSpec,objectElementValidators:{"*":function(t){return tr(Ft({layerType:s},t))}}})}}}))}function rr(t){var e=t.value,n=t.key,r=t.styleSpec,i=t.style;if(!e.type)return[new Rt(n,e,'"type" is required')];var o,a=Ut(e.type);switch(a){case"vector":case"raster":case"raster-dem":if(o=Tn({key:n,value:e,valueSpec:r["source_"+a.replace("-","_")],style:t.style,styleSpec:r}),"url"in e)for(var s in e)["type","url","tileSize"].indexOf(s)<0&&o.push(new Rt(n+"."+s,e[s],'a source with a "url" property may not include a "'+s+'" property'));return o;case"geojson":if(o=Tn({key:n,value:e,valueSpec:r.source_geojson,style:i,styleSpec:r}),e.cluster)for(var A in e.clusterProperties){var l=e.clusterProperties[A],c=l[0],u=l[1],g="string"==typeof c?[c,["accumulated"],["get",A]]:c;o.push.apply(o,Hn({key:n+"."+A+".map",value:u,expressionContext:"cluster-map"})),o.push.apply(o,Hn({key:n+"."+A+".reduce",value:g,expressionContext:"cluster-reduce"}))}return o;case"video":return Tn({key:n,value:e,valueSpec:r.source_video,style:i,styleSpec:r});case"image":return Tn({key:n,value:e,valueSpec:r.source_image,style:i,styleSpec:r});case"canvas":return[new Rt(n,null,"Please use runtime APIs to add canvas sources, rather than including them in stylesheets.","source.canvas")];default:return jn({key:n+".type",value:e.type,valueSpec:{values:["vector","raster","raster-dem","geojson","video","image"]},style:i,styleSpec:r})}}function ir(t){var e=t.value,n=t.styleSpec,r=n.light,i=t.style,o=[],a=vn(e);if(void 0===e)return o;if("object"!==a)return o.concat([new Rt("light",e,"object expected, "+a+" found")]);for(var s in e){var A=s.match(/^(.*)-transition$/);o=A&&r[A[1]]&&r[A[1]].transition?o.concat(sr({key:s,value:e[s],valueSpec:n.transition,style:i,styleSpec:n})):r[s]?o.concat(sr({key:s,value:e[s],valueSpec:r[s],style:i,styleSpec:n})):o.concat([new Rt(s,e[s],'unknown property "'+s+'"')])}return o}function or(t){var e=t.value,n=t.key,r=vn(e);return"string"!==r?[new Rt(n,e,"string expected, "+r+" found")]:[]}var ar={"*":function(){return[]},array:Un,boolean:function(t){var e=t.value,n=t.key,r=vn(e);return"boolean"!==r?[new Rt(n,e,"boolean expected, "+r+" found")]:[]},number:On,color:function(t){var e=t.key,n=t.value,r=vn(n);return"string"!==r?[new Rt(e,n,"color expected, "+r+" found")]:null===ee(n)?[new Rt(e,n,'color expected, "'+n+'" found')]:[]},constants:Nt,enum:jn,filter:Zn,function:Gn,layer:nr,object:Tn,source:rr,light:ir,string:or,formatted:function(t){return 0===or(t).length?[]:Hn(t)}};function sr(t){var e=t.value,n=t.valueSpec,r=t.styleSpec;return n.expression&&Bn(Ut(e))?Gn(t):n.expression&&kn(Ot(e))?Hn(t):n.type&&ar[n.type]?ar[n.type](t):Tn(Ft({},t,{valueSpec:n.type?r[n.type]:n}))}function Ar(t){var e=t.value,n=t.key,r=or(t);return r.length?r:(-1===e.indexOf("{fontstack}")&&r.push(new Rt(n,e,'"glyphs" url must include a "{fontstack}" token')),-1===e.indexOf("{range}")&&r.push(new Rt(n,e,'"glyphs" url must include a "{range}" token')),r)}function lr(t,e){e=e||Lt;var n=[];return n=n.concat(sr({key:"",value:t,valueSpec:e.$root,styleSpec:e,style:t,objectElementValidators:{glyphs:Ar,"*":function(){return[]}}})),t.constants&&(n=n.concat(Nt({key:"constants",value:t.constants,style:t,styleSpec:e}))),cr(n)}function cr(t){return[].concat(t).sort(function(t,e){return t.line-e.line})}function ur(t){return function(){for(var e=[],n=arguments.length;n--;)e[n]=arguments[n];return cr(t.apply(this,e))}}lr.source=ur(rr),lr.light=ur(ir),lr.layer=ur(nr),lr.filter=ur(Zn),lr.paintProperty=ur(tr),lr.layoutProperty=ur(er);var gr=lr,hr=gr.light,dr=gr.paintProperty,fr=gr.layoutProperty;function pr(t,e){var n=!1;if(e&&e.length)for(var r=0,i=e;r<i.length;r+=1){var o=i[r];t.fire(new kt(new Error(o.message))),n=!0}return n}var Ir=mr,Cr=3;function mr(t,e,n){var r=this.cells=[];if(t instanceof ArrayBuffer){this.arrayBuffer=t;var i=new Int32Array(this.arrayBuffer);t=i[0],e=i[1],n=i[2],this.d=e+2*n;for(var o=0;o<this.d*this.d;o++){var a=i[Cr+o],s=i[Cr+o+1];r.push(a===s?null:i.subarray(a,s))}var A=i[Cr+r.length],l=i[Cr+r.length+1];this.keys=i.subarray(A,l),this.bboxes=i.subarray(l),this.insert=this._insertReadonly}else{this.d=e+2*n;for(var c=0;c<this.d*this.d;c++)r.push([]);this.keys=[],this.bboxes=[]}this.n=e,this.extent=t,this.padding=n,this.scale=e/t,this.uid=0;var u=n/e*t;this.min=-u,this.max=t+u}mr.prototype.insert=function(t,e,n,r,i){this._forEachCell(e,n,r,i,this._insertCell,this.uid++),this.keys.push(t),this.bboxes.push(e),this.bboxes.push(n),this.bboxes.push(r),this.bboxes.push(i)},mr.prototype._insertReadonly=function(){throw"Cannot insert into a GridIndex created from an ArrayBuffer."},mr.prototype._insertCell=function(t,e,n,r,i,o){this.cells[i].push(o)},mr.prototype.query=function(t,e,n,r,i){var o=this.min,a=this.max;if(t<=o&&e<=o&&a<=n&&a<=r&&!i)return Array.prototype.slice.call(this.keys);var s=[];return this._forEachCell(t,e,n,r,this._queryCell,s,{},i),s},mr.prototype._queryCell=function(t,e,n,r,i,o,a,s){var A=this.cells[i];if(null!==A)for(var l=this.keys,c=this.bboxes,u=0;u<A.length;u++){var g=A[u];if(void 0===a[g]){var h=4*g;(s?s(c[h+0],c[h+1],c[h+2],c[h+3]):t<=c[h+2]&&e<=c[h+3]&&n>=c[h+0]&&r>=c[h+1])?(a[g]=!0,o.push(l[g])):a[g]=!1}}},mr.prototype._forEachCell=function(t,e,n,r,i,o,a,s){for(var A=this._convertToCellCoord(t),l=this._convertToCellCoord(e),c=this._convertToCellCoord(n),u=this._convertToCellCoord(r),g=A;g<=c;g++)for(var h=l;h<=u;h++){var d=this.d*h+g;if((!s||s(this._convertFromCellCoord(g),this._convertFromCellCoord(h),this._convertFromCellCoord(g+1),this._convertFromCellCoord(h+1)))&&i.call(this,t,e,n,r,d,o,a,s))return}},mr.prototype._convertFromCellCoord=function(t){return(t-this.padding)/this.scale},mr.prototype._convertToCellCoord=function(t){return Math.max(0,Math.min(this.d-1,Math.floor(t*this.scale)+this.padding))},mr.prototype.toArrayBuffer=function(){if(this.arrayBuffer)return this.arrayBuffer;for(var t=this.cells,e=Cr+this.cells.length+1+1,n=0,r=0;r<this.cells.length;r++)n+=this.cells[r].length;var i=new Int32Array(e+n+this.keys.length+this.bboxes.length);i[0]=this.extent,i[1]=this.n,i[2]=this.padding;for(var o=e,a=0;a<t.length;a++){var s=t[a];i[Cr+a]=o,i.set(s,o),o+=s.length}return i[Cr+t.length]=o,i.set(this.keys,o),o+=this.keys.length,i[Cr+t.length+1]=o,i.set(this.bboxes,o),o+=this.bboxes.length,i.buffer};var yr=self.ImageData,Er={};function vr(t,e,n){void 0===n&&(n={}),Object.defineProperty(e,"_classRegistryKey",{value:t,writeable:!1}),Er[t]={klass:e,omit:n.omit||[],shallow:n.shallow||[]}}for(var Br in vr("Object",Object),Ir.serialize=function(t,e){var n=t.toArrayBuffer();return e&&e.push(n),{buffer:n}},Ir.deserialize=function(t){return new Ir(t.buffer)},vr("Grid",Ir),vr("Color",ne),vr("Error",Error),vr("StylePropertyFunction",Fn),vr("StyleExpression",Mn,{omit:["_evaluator"]}),vr("ZoomDependentExpression",Rn),vr("ZoomConstantExpression",Ln),vr("CompoundExpression",Ce,{omit:["_evaluate"]}),gn)gn[Br]._classRegistryKey||vr("Expression_"+Br,gn[Br]);function br(t,e){if(null==t||"boolean"==typeof t||"number"==typeof t||"string"==typeof t||t instanceof Boolean||t instanceof Number||t instanceof String||t instanceof Date||t instanceof RegExp)return t;if(t instanceof ArrayBuffer)return e&&e.push(t),t;if(ArrayBuffer.isView(t)){var n=t;return e&&e.push(n.buffer),n}if(t instanceof yr)return e&&e.push(t.data.buffer),t;if(Array.isArray(t)){for(var r=[],i=0,o=t;i<o.length;i+=1){var a=o[i];r.push(br(a,e))}return r}if("object"==typeof t){var s=t.constructor,A=s._classRegistryKey;if(!A)throw new Error("can't serialize object of unregistered class");var l=s.serialize?s.serialize(t,e):{};if(!s.serialize){for(var c in t)if(t.hasOwnProperty(c)&&!(Er[A].omit.indexOf(c)>=0)){var u=t[c];l[c]=Er[A].shallow.indexOf(c)>=0?u:br(u,e)}t instanceof Error&&(l.message=t.message)}if(l.$name)throw new Error("$name property is reserved for worker serialization logic.");return"Object"!==A&&(l.$name=A),l}throw new Error("can't serialize object of type "+typeof t)}function Qr(t){if(null==t||"boolean"==typeof t||"number"==typeof t||"string"==typeof t||t instanceof Boolean||t instanceof Number||t instanceof String||t instanceof Date||t instanceof RegExp||t instanceof ArrayBuffer||ArrayBuffer.isView(t)||t instanceof yr)return t;if(Array.isArray(t))return t.map(Qr);if("object"==typeof t){var e=t.$name||"Object",n=Er[e].klass;if(!n)throw new Error("can't deserialize unregistered class "+e);if(n.deserialize)return n.deserialize(t);for(var r=Object.create(n.prototype),i=0,o=Object.keys(t);i<o.length;i+=1){var a=o[i];if("$name"!==a){var s=t[a];r[a]=Er[e].shallow.indexOf(a)>=0?s:Qr(s)}}return r}throw new Error("can't deserialize object of type "+typeof t)}var wr=function(){this.first=!0};wr.prototype.update=function(t,e){var n=Math.floor(t);return this.first?(this.first=!1,this.lastIntegerZoom=n,this.lastIntegerZoomTime=0,this.lastZoom=t,this.lastFloorZoom=n,!0):(this.lastFloorZoom>n?(this.lastIntegerZoom=n+1,this.lastIntegerZoomTime=e):this.lastFloorZoom<n&&(this.lastIntegerZoom=n,this.lastIntegerZoomTime=e),t!==this.lastZoom&&(this.lastZoom=t,this.lastFloorZoom=n,!0))};var xr={"Latin-1 Supplement":function(t){return t>=128&&t<=255},Arabic:function(t){return t>=1536&&t<=1791},"Arabic Supplement":function(t){return t>=1872&&t<=1919},"Arabic Extended-A":function(t){return t>=2208&&t<=2303},"Hangul Jamo":function(t){return t>=4352&&t<=4607},"Unified Canadian Aboriginal Syllabics":function(t){return t>=5120&&t<=5759},Khmer:function(t){return t>=6016&&t<=6143},"Unified Canadian Aboriginal Syllabics Extended":function(t){return t>=6320&&t<=6399},"General Punctuation":function(t){return t>=8192&&t<=8303},"Letterlike Symbols":function(t){return t>=8448&&t<=8527},"Number Forms":function(t){return t>=8528&&t<=8591},"Miscellaneous Technical":function(t){return t>=8960&&t<=9215},"Control Pictures":function(t){return t>=9216&&t<=9279},"Optical Character Recognition":function(t){return t>=9280&&t<=9311},"Enclosed Alphanumerics":function(t){return t>=9312&&t<=9471},"Geometric Shapes":function(t){return t>=9632&&t<=9727},"Miscellaneous Symbols":function(t){return t>=9728&&t<=9983},"Miscellaneous Symbols and Arrows":function(t){return t>=11008&&t<=11263},"CJK Radicals Supplement":function(t){return t>=11904&&t<=12031},"Kangxi Radicals":function(t){return t>=12032&&t<=12255},"Ideographic Description Characters":function(t){return t>=12272&&t<=12287},"CJK Symbols and Punctuation":function(t){return t>=12288&&t<=12351},Hiragana:function(t){return t>=12352&&t<=12447},Katakana:function(t){return t>=12448&&t<=12543},Bopomofo:function(t){return t>=12544&&t<=12591},"Hangul Compatibility Jamo":function(t){return t>=12592&&t<=12687},Kanbun:function(t){return t>=12688&&t<=12703},"Bopomofo Extended":function(t){return t>=12704&&t<=12735},"CJK Strokes":function(t){return t>=12736&&t<=12783},"Katakana Phonetic Extensions":function(t){return t>=12784&&t<=12799},"Enclosed CJK Letters and Months":function(t){return t>=12800&&t<=13055},"CJK Compatibility":function(t){return t>=13056&&t<=13311},"CJK Unified Ideographs Extension A":function(t){return t>=13312&&t<=19903},"Yijing Hexagram Symbols":function(t){return t>=19904&&t<=19967},"CJK Unified Ideographs":function(t){return t>=19968&&t<=40959},"Yi Syllables":function(t){return t>=40960&&t<=42127},"Yi Radicals":function(t){return t>=42128&&t<=42191},"Hangul Jamo Extended-A":function(t){return t>=43360&&t<=43391},"Hangul Syllables":function(t){return t>=44032&&t<=55215},"Hangul Jamo Extended-B":function(t){return t>=55216&&t<=55295},"Private Use Area":function(t){return t>=57344&&t<=63743},"CJK Compatibility Ideographs":function(t){return t>=63744&&t<=64255},"Arabic Presentation Forms-A":function(t){return t>=64336&&t<=65023},"Vertical Forms":function(t){return t>=65040&&t<=65055},"CJK Compatibility Forms":function(t){return t>=65072&&t<=65103},"Small Form Variants":function(t){return t>=65104&&t<=65135},"Arabic Presentation Forms-B":function(t){return t>=65136&&t<=65279},"Halfwidth and Fullwidth Forms":function(t){return t>=65280&&t<=65519}};function _r(t){for(var e=0,n=t;e<n.length;e+=1)if(kr(n[e].charCodeAt(0)))return!0;return!1}function Dr(t){for(var e=0,n=t;e<n.length;e+=1)if(!Mr(n[e].charCodeAt(0)))return!1;return!0}function Mr(t){return!(xr.Arabic(t)||xr["Arabic Supplement"](t)||xr["Arabic Extended-A"](t)||xr["Arabic Presentation Forms-A"](t)||xr["Arabic Presentation Forms-B"](t))}function kr(t){return!(746!==t&&747!==t&&(t<4352||!(xr["Bopomofo Extended"](t)||xr.Bopomofo(t)||xr["CJK Compatibility Forms"](t)&&!(t>=65097&&t<=65103)||xr["CJK Compatibility Ideographs"](t)||xr["CJK Compatibility"](t)||xr["CJK Radicals Supplement"](t)||xr["CJK Strokes"](t)||!(!xr["CJK Symbols and Punctuation"](t)||t>=12296&&t<=12305||t>=12308&&t<=12319||12336===t)||xr["CJK Unified Ideographs Extension A"](t)||xr["CJK Unified Ideographs"](t)||xr["Enclosed CJK Letters and Months"](t)||xr["Hangul Compatibility Jamo"](t)||xr["Hangul Jamo Extended-A"](t)||xr["Hangul Jamo Extended-B"](t)||xr["Hangul Jamo"](t)||xr["Hangul Syllables"](t)||xr.Hiragana(t)||xr["Ideographic Description Characters"](t)||xr.Kanbun(t)||xr["Kangxi Radicals"](t)||xr["Katakana Phonetic Extensions"](t)||xr.Katakana(t)&&12540!==t||!(!xr["Halfwidth and Fullwidth Forms"](t)||65288===t||65289===t||65293===t||t>=65306&&t<=65310||65339===t||65341===t||65343===t||t>=65371&&t<=65503||65507===t||t>=65512&&t<=65519)||!(!xr["Small Form Variants"](t)||t>=65112&&t<=65118||t>=65123&&t<=65126)||xr["Unified Canadian Aboriginal Syllabics"](t)||xr["Unified Canadian Aboriginal Syllabics Extended"](t)||xr["Vertical Forms"](t)||xr["Yijing Hexagram Symbols"](t)||xr["Yi Syllables"](t)||xr["Yi Radicals"](t))))}function Sr(t){return!(kr(t)||function(t){return!!(xr["Latin-1 Supplement"](t)&&(167===t||169===t||174===t||177===t||188===t||189===t||190===t||215===t||247===t)||xr["General Punctuation"](t)&&(8214===t||8224===t||8225===t||8240===t||8241===t||8251===t||8252===t||8258===t||8263===t||8264===t||8265===t||8273===t)||xr["Letterlike Symbols"](t)||xr["Number Forms"](t)||xr["Miscellaneous Technical"](t)&&(t>=8960&&t<=8967||t>=8972&&t<=8991||t>=8996&&t<=9e3||9003===t||t>=9085&&t<=9114||t>=9150&&t<=9165||9167===t||t>=9169&&t<=9179||t>=9186&&t<=9215)||xr["Control Pictures"](t)&&9251!==t||xr["Optical Character Recognition"](t)||xr["Enclosed Alphanumerics"](t)||xr["Geometric Shapes"](t)||xr["Miscellaneous Symbols"](t)&&!(t>=9754&&t<=9759)||xr["Miscellaneous Symbols and Arrows"](t)&&(t>=11026&&t<=11055||t>=11088&&t<=11097||t>=11192&&t<=11243)||xr["CJK Symbols and Punctuation"](t)||xr.Katakana(t)||xr["Private Use Area"](t)||xr["CJK Compatibility Forms"](t)||xr["Small Form Variants"](t)||xr["Halfwidth and Fullwidth Forms"](t)||8734===t||8756===t||8757===t||t>=9984&&t<=10087||t>=10102&&t<=10131||65532===t||65533===t)}(t))}function Lr(t,e){return!(!e&&(t>=1424&&t<=2303||xr["Arabic Presentation Forms-A"](t)||xr["Arabic Presentation Forms-B"](t))||t>=2304&&t<=3583||t>=3840&&t<=4255||xr.Khmer(t))}var Rr,Nr=!1,Fr=null,Tr=!1,Ur=new St,Or={applyArabicShaping:null,processBidirectionalText:null,processStyledBidirectionalText:null,isLoaded:function(){return Tr||null!=Or.applyArabicShaping}},Gr=function(t,e){this.zoom=t,e?(this.now=e.now,this.fadeDuration=e.fadeDuration,this.zoomHistory=e.zoomHistory,this.transition=e.transition):(this.now=0,this.fadeDuration=0,this.zoomHistory=new wr,this.transition={})};Gr.prototype.isSupportedScript=function(t){return function(t,e){for(var n=0,r=t;n<r.length;n+=1)if(!Lr(r[n].charCodeAt(0),e))return!1;return!0}(t,Or.isLoaded())},Gr.prototype.crossFadingFactor=function(){return 0===this.fadeDuration?1:Math.min((this.now-this.zoomHistory.lastIntegerZoomTime)/this.fadeDuration,1)},Gr.prototype.getCrossfadeParameters=function(){var t=this.zoom,e=t-Math.floor(t),n=this.crossFadingFactor();return t>this.zoomHistory.lastIntegerZoom?{fromScale:2,toScale:1,t:e+(1-e)*n}:{fromScale:.5,toScale:1,t:1-(1-n)*e}};var Hr=function(t,e){this.property=t,this.value=e,this.expression=function(t,e){if(Bn(t))return new Fn(t,e);if(kn(t)){var n=Nn(t,e);if("error"===n.result)throw new Error(n.value.map(function(t){return t.key+": "+t.message}).join(", "));return n.value}var r=t;return"string"==typeof t&&"color"===e.type&&(r=ne.parse(t)),{kind:"constant",evaluate:function(){return r}}}(void 0===e?t.specification.default:e,t.specification)};Hr.prototype.isDataDriven=function(){return"source"===this.expression.kind||"composite"===this.expression.kind},Hr.prototype.possiblyEvaluate=function(t){return this.property.possiblyEvaluate(this,t)};var jr=function(t){this.property=t,this.value=new Hr(t,void 0)};jr.prototype.transitioned=function(t,e){return new Pr(this.property,this.value,e,u({},t.transition,this.transition),t.now)},jr.prototype.untransitioned=function(){return new Pr(this.property,this.value,null,{},0)};var Yr=function(t){this._properties=t,this._values=Object.create(t.defaultTransitionablePropertyValues)};Yr.prototype.getValue=function(t){return y(this._values[t].value.value)},Yr.prototype.setValue=function(t,e){this._values.hasOwnProperty(t)||(this._values[t]=new jr(this._values[t].property)),this._values[t].value=new Hr(this._values[t].property,null===e?void 0:y(e))},Yr.prototype.getTransition=function(t){return y(this._values[t].transition)},Yr.prototype.setTransition=function(t,e){this._values.hasOwnProperty(t)||(this._values[t]=new jr(this._values[t].property)),this._values[t].transition=y(e)||void 0},Yr.prototype.serialize=function(){for(var t={},e=0,n=Object.keys(this._values);e<n.length;e+=1){var r=n[e],i=this.getValue(r);void 0!==i&&(t[r]=i);var o=this.getTransition(r);void 0!==o&&(t[r+"-transition"]=o)}return t},Yr.prototype.transitioned=function(t,e){for(var n=new Kr(this._properties),r=0,i=Object.keys(this._values);r<i.length;r+=1){var o=i[r];n._values[o]=this._values[o].transitioned(t,e._values[o])}return n},Yr.prototype.untransitioned=function(){for(var t=new Kr(this._properties),e=0,n=Object.keys(this._values);e<n.length;e+=1){var r=n[e];t._values[r]=this._values[r].untransitioned()}return t};var Pr=function(t,e,n,r,i){this.property=t,this.value=e,this.begin=i+r.delay||0,this.end=this.begin+r.duration||0,t.specification.transition&&(r.delay||r.duration)&&(this.prior=n)};Pr.prototype.possiblyEvaluate=function(t){var e=t.now||0,n=this.value.possiblyEvaluate(t),r=this.prior;if(r){if(e>this.end)return this.prior=null,n;if(this.value.isDataDriven())return this.prior=null,n;if(e<this.begin)return r.possiblyEvaluate(t);var i=(e-this.begin)/(this.end-this.begin);return this.property.interpolate(r.possiblyEvaluate(t),n,function(t){if(i<=0)return 0;if(i>=1)return 1;var e=i*i,n=e*i;return 4*(i<.5?n:3*(i-e)+n-.75)}())}return n};var Kr=function(t){this._properties=t,this._values=Object.create(t.defaultTransitioningPropertyValues)};Kr.prototype.possiblyEvaluate=function(t){for(var e=new qr(this._properties),n=0,r=Object.keys(this._values);n<r.length;n+=1){var i=r[n];e._values[i]=this._values[i].possiblyEvaluate(t)}return e},Kr.prototype.hasTransition=function(){for(var t=0,e=Object.keys(this._values);t<e.length;t+=1){var n=e[t];if(this._values[n].prior)return!0}return!1};var zr=function(t){this._properties=t,this._values=Object.create(t.defaultPropertyValues)};zr.prototype.getValue=function(t){return y(this._values[t].value)},zr.prototype.setValue=function(t,e){this._values[t]=new Hr(this._values[t].property,null===e?void 0:y(e))},zr.prototype.serialize=function(){for(var t={},e=0,n=Object.keys(this._values);e<n.length;e+=1){var r=n[e],i=this.getValue(r);void 0!==i&&(t[r]=i)}return t},zr.prototype.possiblyEvaluate=function(t){for(var e=new qr(this._properties),n=0,r=Object.keys(this._values);n<r.length;n+=1){var i=r[n];e._values[i]=this._values[i].possiblyEvaluate(t)}return e};var Jr=function(t,e,n){this.property=t,this.value=e,this.parameters=n};Jr.prototype.isConstant=function(){return"constant"===this.value.kind},Jr.prototype.constantOr=function(t){return"constant"===this.value.kind?this.value.value:t},Jr.prototype.evaluate=function(t,e){return this.property.evaluate(this.value,this.parameters,t,e)};var qr=function(t){this._properties=t,this._values=Object.create(t.defaultPossiblyEvaluatedValues)};qr.prototype.get=function(t){return this._values[t]};var Vr=function(t){this.specification=t};Vr.prototype.possiblyEvaluate=function(t,e){return t.expression.evaluate(e)},Vr.prototype.interpolate=function(t,e,n){var r=_e[this.specification.type];return r?r(t,e,n):t};var Wr=function(t){this.specification=t};Wr.prototype.possiblyEvaluate=function(t,e){return"constant"===t.expression.kind||"camera"===t.expression.kind?new Jr(this,{kind:"constant",value:t.expression.evaluate(e)},e):new Jr(this,t.expression,e)},Wr.prototype.interpolate=function(t,e,n){if("constant"!==t.value.kind||"constant"!==e.value.kind)return t;if(void 0===t.value.value||void 0===e.value.value)return new Jr(this,{kind:"constant",value:void 0},t.parameters);var r=_e[this.specification.type];return r?new Jr(this,{kind:"constant",value:r(t.value.value,e.value.value,n)},t.parameters):t},Wr.prototype.evaluate=function(t,e,n,r){return"constant"===t.kind?t.value:t.evaluate(e,n,r)};var Xr=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.possiblyEvaluate=function(t,e){if(void 0===t.value)return new Jr(this,{kind:"constant",value:void 0},e);if("constant"===t.expression.kind){var n=t.expression.evaluate(e),r=this._calculate(n,n,n,e);return new Jr(this,{kind:"constant",value:r},e)}if("camera"===t.expression.kind){var i=this._calculate(t.expression.evaluate({zoom:e.zoom-1}),t.expression.evaluate({zoom:e.zoom}),t.expression.evaluate({zoom:e.zoom+1}),e);return new Jr(this,{kind:"constant",value:i},e)}return new Jr(this,t.expression,e)},e.prototype.evaluate=function(t,e,n,r){if("source"===t.kind){var i=t.evaluate(e,n,r);return this._calculate(i,i,i,e)}return"composite"===t.kind?this._calculate(t.evaluate({zoom:Math.floor(e.zoom)-1},n,r),t.evaluate({zoom:Math.floor(e.zoom)},n,r),t.evaluate({zoom:Math.floor(e.zoom)+1},n,r),e):t.value},e.prototype._calculate=function(t,e,n,r){return r.zoom>r.zoomHistory.lastIntegerZoom?{from:t,to:e}:{from:n,to:e}},e.prototype.interpolate=function(t){return t},e}(Wr),Zr=function(t){this.specification=t};Zr.prototype.possiblyEvaluate=function(t,e){if(void 0!==t.value){if("constant"===t.expression.kind){var n=t.expression.evaluate(e);return this._calculate(n,n,n,e)}return this._calculate(t.expression.evaluate(new Gr(Math.floor(e.zoom-1),e)),t.expression.evaluate(new Gr(Math.floor(e.zoom),e)),t.expression.evaluate(new Gr(Math.floor(e.zoom+1),e)),e)}},Zr.prototype._calculate=function(t,e,n,r){return r.zoom>r.zoomHistory.lastIntegerZoom?{from:t,to:e}:{from:n,to:e}},Zr.prototype.interpolate=function(t){return t};var $r=function(t){this.specification=t};$r.prototype.possiblyEvaluate=function(t,e){return!!t.expression.evaluate(e)},$r.prototype.interpolate=function(){return!1};var ti=function(t){for(var e in this.properties=t,this.defaultPropertyValues={},this.defaultTransitionablePropertyValues={},this.defaultTransitioningPropertyValues={},this.defaultPossiblyEvaluatedValues={},t){var n=t[e],r=this.defaultPropertyValues[e]=new Hr(n,void 0),i=this.defaultTransitionablePropertyValues[e]=new jr(n);this.defaultTransitioningPropertyValues[e]=i.untransitioned(),this.defaultPossiblyEvaluatedValues[e]=r.possiblyEvaluate({})}};vr("DataDrivenProperty",Wr),vr("DataConstantProperty",Vr),vr("CrossFadedDataDrivenProperty",Xr),vr("CrossFadedProperty",Zr),vr("ColorRampProperty",$r);var ei=function(t){function e(e,n){if(t.call(this),this.id=e.id,this.type=e.type,this._featureFilter=function(){return!0},"custom"!==e.type&&(e=e,this.metadata=e.metadata,this.minzoom=e.minzoom,this.maxzoom=e.maxzoom,"background"!==e.type&&(this.source=e.source,this.sourceLayer=e["source-layer"],this.filter=e.filter),n.layout&&(this._unevaluatedLayout=new zr(n.layout)),n.paint)){for(var r in this._transitionablePaint=new Yr(n.paint),e.paint)this.setPaintProperty(r,e.paint[r],{validate:!1});for(var i in e.layout)this.setLayoutProperty(i,e.layout[i],{validate:!1});this._transitioningPaint=this._transitionablePaint.untransitioned()}}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getCrossfadeParameters=function(){return this._crossfadeParameters},e.prototype.getLayoutProperty=function(t){return"visibility"===t?this.visibility:this._unevaluatedLayout.getValue(t)},e.prototype.setLayoutProperty=function(t,e,n){if(void 0===n&&(n={}),null!=e){var r="layers."+this.id+".layout."+t;if(this._validate(fr,r,t,e,n))return}"visibility"!==t?this._unevaluatedLayout.setValue(t,e):this.visibility=e},e.prototype.getPaintProperty=function(t){return I(t,"-transition")?this._transitionablePaint.getTransition(t.slice(0,-"-transition".length)):this._transitionablePaint.getValue(t)},e.prototype.setPaintProperty=function(t,e,n){if(void 0===n&&(n={}),null!=e){var r="layers."+this.id+".paint."+t;if(this._validate(dr,r,t,e,n))return!1}if(I(t,"-transition"))return this._transitionablePaint.setTransition(t.slice(0,-"-transition".length),e||void 0),!1;var i=this._transitionablePaint._values[t],o="cross-faded-data-driven"===i.property.specification["property-type"],a=i.value.isDataDriven();return this._transitionablePaint.setValue(t,e),this._handleSpecialPaintPropertyUpdate(t),this._transitionablePaint._values[t].value.isDataDriven()||a||o},e.prototype._handleSpecialPaintPropertyUpdate=function(t){},e.prototype.isHidden=function(t){return!!(this.minzoom&&t<this.minzoom)||!!(this.maxzoom&&t>=this.maxzoom)||"none"===this.visibility},e.prototype.updateTransitions=function(t){this._transitioningPaint=this._transitionablePaint.transitioned(t,this._transitioningPaint)},e.prototype.hasTransition=function(){return this._transitioningPaint.hasTransition()},e.prototype.recalculate=function(t){t.getCrossfadeParameters&&(this._crossfadeParameters=t.getCrossfadeParameters()),this._unevaluatedLayout&&(this.layout=this._unevaluatedLayout.possiblyEvaluate(t)),this.paint=this._transitioningPaint.possiblyEvaluate(t)},e.prototype.serialize=function(){var t={id:this.id,type:this.type,source:this.source,"source-layer":this.sourceLayer,metadata:this.metadata,minzoom:this.minzoom,maxzoom:this.maxzoom,filter:this.filter,layout:this._unevaluatedLayout&&this._unevaluatedLayout.serialize(),paint:this._transitionablePaint&&this._transitionablePaint.serialize()};return this.visibility&&(t.layout=t.layout||{},t.layout.visibility=this.visibility),m(t,function(t,e){return!(void 0===t||"layout"===e&&!Object.keys(t).length||"paint"===e&&!Object.keys(t).length)})},e.prototype._validate=function(t,e,n,r,i){return void 0===i&&(i={}),(!i||!1!==i.validate)&&pr(this,t.call(gr,{key:e,layerType:this.type,objectKey:n,value:r,styleSpec:Lt,style:{glyphs:!0,sprite:!0}}))},e.prototype.is3D=function(){return!1},e.prototype.isTileClipped=function(){return!1},e.prototype.hasOffscreenPass=function(){return!1},e.prototype.resize=function(){},e.prototype.isStateDependent=function(){for(var t in this.paint._values){var e=this.paint.get(t);if(e instanceof Jr&&mn(e.property.specification)&&("source"===e.value.kind||"composite"===e.value.kind)&&e.value.isStateDependent)return!0}return!1},e}(St),ni={Int8:Int8Array,Uint8:Uint8Array,Int16:Int16Array,Uint16:Uint16Array,Int32:Int32Array,Uint32:Uint32Array,Float32:Float32Array},ri=function(t,e){this._structArray=t,this._pos1=e*this.size,this._pos2=this._pos1/2,this._pos4=this._pos1/4,this._pos8=this._pos1/8},ii=function(){this.isTransferred=!1,this.capacity=-1,this.resize(0)};function oi(t,e){void 0===e&&(e=1);var n=0,r=0;return{members:t.map(function(t){var i,o=(i=t.type,ni[i].BYTES_PER_ELEMENT),a=n=ai(n,Math.max(e,o)),s=t.components||1;return r=Math.max(r,o),n+=o*s,{name:t.name,type:t.type,components:s,offset:a}}),size:ai(n,Math.max(r,e)),alignment:e}}function ai(t,e){return Math.ceil(t/e)*e}ii.serialize=function(t,e){return t._trim(),e&&(t.isTransferred=!0,e.push(t.arrayBuffer)),{length:t.length,arrayBuffer:t.arrayBuffer}},ii.deserialize=function(t){var e=Object.create(this.prototype);return e.arrayBuffer=t.arrayBuffer,e.length=t.length,e.capacity=t.arrayBuffer.byteLength/e.bytesPerElement,e._refreshViews(),e},ii.prototype._trim=function(){this.length!==this.capacity&&(this.capacity=this.length,this.arrayBuffer=this.arrayBuffer.slice(0,this.length*this.bytesPerElement),this._refreshViews())},ii.prototype.clear=function(){this.length=0},ii.prototype.resize=function(t){this.reserve(t),this.length=t},ii.prototype.reserve=function(t){if(t>this.capacity){this.capacity=Math.max(t,Math.floor(5*this.capacity),128),this.arrayBuffer=new ArrayBuffer(this.capacity*this.bytesPerElement);var e=this.uint8;this._refreshViews(),e&&this.uint8.set(e)}},ii.prototype._refreshViews=function(){throw new Error("_refreshViews() must be implemented by each concrete StructArray layout")};var si=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.int16=new Int16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e){var n=this.length;return this.resize(n+1),this.emplace(n,t,e)},e.prototype.emplace=function(t,e,n){var r=2*t;return this.int16[r+0]=e,this.int16[r+1]=n,t},e}(ii);si.prototype.bytesPerElement=4,vr("StructArrayLayout2i4",si);var Ai=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.int16=new Int16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n,r){var i=this.length;return this.resize(i+1),this.emplace(i,t,e,n,r)},e.prototype.emplace=function(t,e,n,r,i){var o=4*t;return this.int16[o+0]=e,this.int16[o+1]=n,this.int16[o+2]=r,this.int16[o+3]=i,t},e}(ii);Ai.prototype.bytesPerElement=8,vr("StructArrayLayout4i8",Ai);var li=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.int16=new Int16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n,r,i,o){var a=this.length;return this.resize(a+1),this.emplace(a,t,e,n,r,i,o)},e.prototype.emplace=function(t,e,n,r,i,o,a){var s=6*t;return this.int16[s+0]=e,this.int16[s+1]=n,this.int16[s+2]=r,this.int16[s+3]=i,this.int16[s+4]=o,this.int16[s+5]=a,t},e}(ii);li.prototype.bytesPerElement=12,vr("StructArrayLayout2i4i12",li);var ci=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.int16=new Int16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n,r,i,o){var a=this.length;return this.resize(a+1),this.emplace(a,t,e,n,r,i,o)},e.prototype.emplace=function(t,e,n,r,i,o,a){var s=4*t,A=8*t;return this.int16[s+0]=e,this.int16[s+1]=n,this.uint8[A+4]=r,this.uint8[A+5]=i,this.uint8[A+6]=o,this.uint8[A+7]=a,t},e}(ii);ci.prototype.bytesPerElement=8,vr("StructArrayLayout2i4ub8",ci);var ui=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.uint16=new Uint16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n,r,i,o,a,s){var A=this.length;return this.resize(A+1),this.emplace(A,t,e,n,r,i,o,a,s)},e.prototype.emplace=function(t,e,n,r,i,o,a,s,A){var l=8*t;return this.uint16[l+0]=e,this.uint16[l+1]=n,this.uint16[l+2]=r,this.uint16[l+3]=i,this.uint16[l+4]=o,this.uint16[l+5]=a,this.uint16[l+6]=s,this.uint16[l+7]=A,t},e}(ii);ui.prototype.bytesPerElement=16,vr("StructArrayLayout8ui16",ui);var gi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.int16=new Int16Array(this.arrayBuffer),this.uint16=new Uint16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n,r,i,o,a,s){var A=this.length;return this.resize(A+1),this.emplace(A,t,e,n,r,i,o,a,s)},e.prototype.emplace=function(t,e,n,r,i,o,a,s,A){var l=8*t;return this.int16[l+0]=e,this.int16[l+1]=n,this.int16[l+2]=r,this.int16[l+3]=i,this.uint16[l+4]=o,this.uint16[l+5]=a,this.uint16[l+6]=s,this.uint16[l+7]=A,t},e}(ii);gi.prototype.bytesPerElement=16,vr("StructArrayLayout4i4ui16",gi);var hi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.float32=new Float32Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n){var r=this.length;return this.resize(r+1),this.emplace(r,t,e,n)},e.prototype.emplace=function(t,e,n,r){var i=3*t;return this.float32[i+0]=e,this.float32[i+1]=n,this.float32[i+2]=r,t},e}(ii);hi.prototype.bytesPerElement=12,vr("StructArrayLayout3f12",hi);var di=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.uint32=new Uint32Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t){var e=this.length;return this.resize(e+1),this.emplace(e,t)},e.prototype.emplace=function(t,e){var n=1*t;return this.uint32[n+0]=e,t},e}(ii);di.prototype.bytesPerElement=4,vr("StructArrayLayout1ul4",di);var fi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.int16=new Int16Array(this.arrayBuffer),this.uint32=new Uint32Array(this.arrayBuffer),this.uint16=new Uint16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n,r,i,o,a,s,A,l,c){var u=this.length;return this.resize(u+1),this.emplace(u,t,e,n,r,i,o,a,s,A,l,c)},e.prototype.emplace=function(t,e,n,r,i,o,a,s,A,l,c,u){var g=12*t,h=6*t;return this.int16[g+0]=e,this.int16[g+1]=n,this.int16[g+2]=r,this.int16[g+3]=i,this.int16[g+4]=o,this.int16[g+5]=a,this.uint32[h+3]=s,this.uint16[g+8]=A,this.uint16[g+9]=l,this.int16[g+10]=c,this.int16[g+11]=u,t},e}(ii);fi.prototype.bytesPerElement=24,vr("StructArrayLayout6i1ul2ui2i24",fi);var pi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.int16=new Int16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n,r,i,o){var a=this.length;return this.resize(a+1),this.emplace(a,t,e,n,r,i,o)},e.prototype.emplace=function(t,e,n,r,i,o,a){var s=6*t;return this.int16[s+0]=e,this.int16[s+1]=n,this.int16[s+2]=r,this.int16[s+3]=i,this.int16[s+4]=o,this.int16[s+5]=a,t},e}(ii);pi.prototype.bytesPerElement=12,vr("StructArrayLayout2i2i2i12",pi);var Ii=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.float32=new Float32Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n,r){var i=this.length;return this.resize(i+1),this.emplace(i,t,e,n,r)},e.prototype.emplace=function(t,e,n,r,i){var o=12*t,a=3*t;return this.uint8[o+0]=e,this.uint8[o+1]=n,this.float32[a+1]=r,this.float32[a+2]=i,t},e}(ii);Ii.prototype.bytesPerElement=12,vr("StructArrayLayout2ub2f12",Ii);var Ci=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.int16=new Int16Array(this.arrayBuffer),this.uint16=new Uint16Array(this.arrayBuffer),this.uint32=new Uint32Array(this.arrayBuffer),this.float32=new Float32Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n,r,i,o,a,s,A,l,c,u,g,h,d){var f=this.length;return this.resize(f+1),this.emplace(f,t,e,n,r,i,o,a,s,A,l,c,u,g,h,d)},e.prototype.emplace=function(t,e,n,r,i,o,a,s,A,l,c,u,g,h,d,f){var p=22*t,I=11*t,C=44*t;return this.int16[p+0]=e,this.int16[p+1]=n,this.uint16[p+2]=r,this.uint16[p+3]=i,this.uint32[I+2]=o,this.uint32[I+3]=a,this.uint32[I+4]=s,this.uint16[p+10]=A,this.uint16[p+11]=l,this.uint16[p+12]=c,this.float32[I+7]=u,this.float32[I+8]=g,this.uint8[C+36]=h,this.uint8[C+37]=d,this.uint32[I+10]=f,t},e}(ii);Ci.prototype.bytesPerElement=44,vr("StructArrayLayout2i2ui3ul3ui2f2ub1ul44",Ci);var mi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.int16=new Int16Array(this.arrayBuffer),this.uint16=new Uint16Array(this.arrayBuffer),this.uint32=new Uint32Array(this.arrayBuffer),this.float32=new Float32Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n,r,i,o,a,s,A,l,c,u,g,h,d,f,p,I){var C=this.length;return this.resize(C+1),this.emplace(C,t,e,n,r,i,o,a,s,A,l,c,u,g,h,d,f,p,I)},e.prototype.emplace=function(t,e,n,r,i,o,a,s,A,l,c,u,g,h,d,f,p,I,C){var m=22*t,y=11*t;return this.int16[m+0]=e,this.int16[m+1]=n,this.int16[m+2]=r,this.int16[m+3]=i,this.int16[m+4]=o,this.int16[m+5]=a,this.uint16[m+6]=s,this.uint16[m+7]=A,this.uint16[m+8]=l,this.uint16[m+9]=c,this.uint16[m+10]=u,this.uint16[m+11]=g,this.uint16[m+12]=h,this.uint16[m+13]=d,this.uint16[m+14]=f,this.uint32[y+8]=p,this.float32[y+9]=I,this.float32[y+10]=C,t},e}(ii);mi.prototype.bytesPerElement=44,vr("StructArrayLayout6i9ui1ul2f44",mi);var yi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.float32=new Float32Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t){var e=this.length;return this.resize(e+1),this.emplace(e,t)},e.prototype.emplace=function(t,e){var n=1*t;return this.float32[n+0]=e,t},e}(ii);yi.prototype.bytesPerElement=4,vr("StructArrayLayout1f4",yi);var Ei=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.int16=new Int16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n){var r=this.length;return this.resize(r+1),this.emplace(r,t,e,n)},e.prototype.emplace=function(t,e,n,r){var i=3*t;return this.int16[i+0]=e,this.int16[i+1]=n,this.int16[i+2]=r,t},e}(ii);Ei.prototype.bytesPerElement=6,vr("StructArrayLayout3i6",Ei);var vi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.uint32=new Uint32Array(this.arrayBuffer),this.uint16=new Uint16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n){var r=this.length;return this.resize(r+1),this.emplace(r,t,e,n)},e.prototype.emplace=function(t,e,n,r){var i=2*t,o=4*t;return this.uint32[i+0]=e,this.uint16[o+2]=n,this.uint16[o+3]=r,t},e}(ii);vi.prototype.bytesPerElement=8,vr("StructArrayLayout1ul2ui8",vi);var Bi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.uint16=new Uint16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n){var r=this.length;return this.resize(r+1),this.emplace(r,t,e,n)},e.prototype.emplace=function(t,e,n,r){var i=3*t;return this.uint16[i+0]=e,this.uint16[i+1]=n,this.uint16[i+2]=r,t},e}(ii);Bi.prototype.bytesPerElement=6,vr("StructArrayLayout3ui6",Bi);var bi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.uint16=new Uint16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e){var n=this.length;return this.resize(n+1),this.emplace(n,t,e)},e.prototype.emplace=function(t,e,n){var r=2*t;return this.uint16[r+0]=e,this.uint16[r+1]=n,t},e}(ii);bi.prototype.bytesPerElement=4,vr("StructArrayLayout2ui4",bi);var Qi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.uint16=new Uint16Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t){var e=this.length;return this.resize(e+1),this.emplace(e,t)},e.prototype.emplace=function(t,e){var n=1*t;return this.uint16[n+0]=e,t},e}(ii);Qi.prototype.bytesPerElement=2,vr("StructArrayLayout1ui2",Qi);var wi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.float32=new Float32Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e){var n=this.length;return this.resize(n+1),this.emplace(n,t,e)},e.prototype.emplace=function(t,e,n){var r=2*t;return this.float32[r+0]=e,this.float32[r+1]=n,t},e}(ii);wi.prototype.bytesPerElement=8,vr("StructArrayLayout2f8",wi);var xi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._refreshViews=function(){this.uint8=new Uint8Array(this.arrayBuffer),this.float32=new Float32Array(this.arrayBuffer)},e.prototype.emplaceBack=function(t,e,n,r){var i=this.length;return this.resize(i+1),this.emplace(i,t,e,n,r)},e.prototype.emplace=function(t,e,n,r,i){var o=4*t;return this.float32[o+0]=e,this.float32[o+1]=n,this.float32[o+2]=r,this.float32[o+3]=i,t},e}(ii);xi.prototype.bytesPerElement=16,vr("StructArrayLayout4f16",xi);var _i=function(t){function e(){t.apply(this,arguments)}t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e;var n={anchorPointX:{configurable:!0},anchorPointY:{configurable:!0},x1:{configurable:!0},y1:{configurable:!0},x2:{configurable:!0},y2:{configurable:!0},featureIndex:{configurable:!0},sourceLayerIndex:{configurable:!0},bucketIndex:{configurable:!0},radius:{configurable:!0},signedDistanceFromAnchor:{configurable:!0},anchorPoint:{configurable:!0}};return n.anchorPointX.get=function(){return this._structArray.int16[this._pos2+0]},n.anchorPointX.set=function(t){this._structArray.int16[this._pos2+0]=t},n.anchorPointY.get=function(){return this._structArray.int16[this._pos2+1]},n.anchorPointY.set=function(t){this._structArray.int16[this._pos2+1]=t},n.x1.get=function(){return this._structArray.int16[this._pos2+2]},n.x1.set=function(t){this._structArray.int16[this._pos2+2]=t},n.y1.get=function(){return this._structArray.int16[this._pos2+3]},n.y1.set=function(t){this._structArray.int16[this._pos2+3]=t},n.x2.get=function(){return this._structArray.int16[this._pos2+4]},n.x2.set=function(t){this._structArray.int16[this._pos2+4]=t},n.y2.get=function(){return this._structArray.int16[this._pos2+5]},n.y2.set=function(t){this._structArray.int16[this._pos2+5]=t},n.featureIndex.get=function(){return this._structArray.uint32[this._pos4+3]},n.featureIndex.set=function(t){this._structArray.uint32[this._pos4+3]=t},n.sourceLayerIndex.get=function(){return this._structArray.uint16[this._pos2+8]},n.sourceLayerIndex.set=function(t){this._structArray.uint16[this._pos2+8]=t},n.bucketIndex.get=function(){return this._structArray.uint16[this._pos2+9]},n.bucketIndex.set=function(t){this._structArray.uint16[this._pos2+9]=t},n.radius.get=function(){return this._structArray.int16[this._pos2+10]},n.radius.set=function(t){this._structArray.int16[this._pos2+10]=t},n.signedDistanceFromAnchor.get=function(){return this._structArray.int16[this._pos2+11]},n.signedDistanceFromAnchor.set=function(t){this._structArray.int16[this._pos2+11]=t},n.anchorPoint.get=function(){return new i(this.anchorPointX,this.anchorPointY)},Object.defineProperties(e.prototype,n),e}(ri);_i.prototype.size=24;var Di=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.get=function(t){return new _i(this,t)},e}(fi);vr("CollisionBoxArray",Di);var Mi=function(t){function e(){t.apply(this,arguments)}t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e;var n={anchorX:{configurable:!0},anchorY:{configurable:!0},glyphStartIndex:{configurable:!0},numGlyphs:{configurable:!0},vertexStartIndex:{configurable:!0},lineStartIndex:{configurable:!0},lineLength:{configurable:!0},segment:{configurable:!0},lowerSize:{configurable:!0},upperSize:{configurable:!0},lineOffsetX:{configurable:!0},lineOffsetY:{configurable:!0},writingMode:{configurable:!0},hidden:{configurable:!0},crossTileID:{configurable:!0}};return n.anchorX.get=function(){return this._structArray.int16[this._pos2+0]},n.anchorX.set=function(t){this._structArray.int16[this._pos2+0]=t},n.anchorY.get=function(){return this._structArray.int16[this._pos2+1]},n.anchorY.set=function(t){this._structArray.int16[this._pos2+1]=t},n.glyphStartIndex.get=function(){return this._structArray.uint16[this._pos2+2]},n.glyphStartIndex.set=function(t){this._structArray.uint16[this._pos2+2]=t},n.numGlyphs.get=function(){return this._structArray.uint16[this._pos2+3]},n.numGlyphs.set=function(t){this._structArray.uint16[this._pos2+3]=t},n.vertexStartIndex.get=function(){return this._structArray.uint32[this._pos4+2]},n.vertexStartIndex.set=function(t){this._structArray.uint32[this._pos4+2]=t},n.lineStartIndex.get=function(){return this._structArray.uint32[this._pos4+3]},n.lineStartIndex.set=function(t){this._structArray.uint32[this._pos4+3]=t},n.lineLength.get=function(){return this._structArray.uint32[this._pos4+4]},n.lineLength.set=function(t){this._structArray.uint32[this._pos4+4]=t},n.segment.get=function(){return this._structArray.uint16[this._pos2+10]},n.segment.set=function(t){this._structArray.uint16[this._pos2+10]=t},n.lowerSize.get=function(){return this._structArray.uint16[this._pos2+11]},n.lowerSize.set=function(t){this._structArray.uint16[this._pos2+11]=t},n.upperSize.get=function(){return this._structArray.uint16[this._pos2+12]},n.upperSize.set=function(t){this._structArray.uint16[this._pos2+12]=t},n.lineOffsetX.get=function(){return this._structArray.float32[this._pos4+7]},n.lineOffsetX.set=function(t){this._structArray.float32[this._pos4+7]=t},n.lineOffsetY.get=function(){return this._structArray.float32[this._pos4+8]},n.lineOffsetY.set=function(t){this._structArray.float32[this._pos4+8]=t},n.writingMode.get=function(){return this._structArray.uint8[this._pos1+36]},n.writingMode.set=function(t){this._structArray.uint8[this._pos1+36]=t},n.hidden.get=function(){return this._structArray.uint8[this._pos1+37]},n.hidden.set=function(t){this._structArray.uint8[this._pos1+37]=t},n.crossTileID.get=function(){return this._structArray.uint32[this._pos4+10]},n.crossTileID.set=function(t){this._structArray.uint32[this._pos4+10]=t},Object.defineProperties(e.prototype,n),e}(ri);Mi.prototype.size=44;var ki=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.get=function(t){return new Mi(this,t)},e}(Ci);vr("PlacedSymbolArray",ki);var Si=function(t){function e(){t.apply(this,arguments)}t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e;var n={anchorX:{configurable:!0},anchorY:{configurable:!0},rightJustifiedTextSymbolIndex:{configurable:!0},centerJustifiedTextSymbolIndex:{configurable:!0},leftJustifiedTextSymbolIndex:{configurable:!0},verticalPlacedTextSymbolIndex:{configurable:!0},key:{configurable:!0},textBoxStartIndex:{configurable:!0},textBoxEndIndex:{configurable:!0},iconBoxStartIndex:{configurable:!0},iconBoxEndIndex:{configurable:!0},featureIndex:{configurable:!0},numHorizontalGlyphVertices:{configurable:!0},numVerticalGlyphVertices:{configurable:!0},numIconVertices:{configurable:!0},crossTileID:{configurable:!0},textBoxScale:{configurable:!0},radialTextOffset:{configurable:!0}};return n.anchorX.get=function(){return this._structArray.int16[this._pos2+0]},n.anchorX.set=function(t){this._structArray.int16[this._pos2+0]=t},n.anchorY.get=function(){return this._structArray.int16[this._pos2+1]},n.anchorY.set=function(t){this._structArray.int16[this._pos2+1]=t},n.rightJustifiedTextSymbolIndex.get=function(){return this._structArray.int16[this._pos2+2]},n.rightJustifiedTextSymbolIndex.set=function(t){this._structArray.int16[this._pos2+2]=t},n.centerJustifiedTextSymbolIndex.get=function(){return this._structArray.int16[this._pos2+3]},n.centerJustifiedTextSymbolIndex.set=function(t){this._structArray.int16[this._pos2+3]=t},n.leftJustifiedTextSymbolIndex.get=function(){return this._structArray.int16[this._pos2+4]},n.leftJustifiedTextSymbolIndex.set=function(t){this._structArray.int16[this._pos2+4]=t},n.verticalPlacedTextSymbolIndex.get=function(){return this._structArray.int16[this._pos2+5]},n.verticalPlacedTextSymbolIndex.set=function(t){this._structArray.int16[this._pos2+5]=t},n.key.get=function(){return this._structArray.uint16[this._pos2+6]},n.key.set=function(t){this._structArray.uint16[this._pos2+6]=t},n.textBoxStartIndex.get=function(){return this._structArray.uint16[this._pos2+7]},n.textBoxStartIndex.set=function(t){this._structArray.uint16[this._pos2+7]=t},n.textBoxEndIndex.get=function(){return this._structArray.uint16[this._pos2+8]},n.textBoxEndIndex.set=function(t){this._structArray.uint16[this._pos2+8]=t},n.iconBoxStartIndex.get=function(){return this._structArray.uint16[this._pos2+9]},n.iconBoxStartIndex.set=function(t){this._structArray.uint16[this._pos2+9]=t},n.iconBoxEndIndex.get=function(){return this._structArray.uint16[this._pos2+10]},n.iconBoxEndIndex.set=function(t){this._structArray.uint16[this._pos2+10]=t},n.featureIndex.get=function(){return this._structArray.uint16[this._pos2+11]},n.featureIndex.set=function(t){this._structArray.uint16[this._pos2+11]=t},n.numHorizontalGlyphVertices.get=function(){return this._structArray.uint16[this._pos2+12]},n.numHorizontalGlyphVertices.set=function(t){this._structArray.uint16[this._pos2+12]=t},n.numVerticalGlyphVertices.get=function(){return this._structArray.uint16[this._pos2+13]},n.numVerticalGlyphVertices.set=function(t){this._structArray.uint16[this._pos2+13]=t},n.numIconVertices.get=function(){return this._structArray.uint16[this._pos2+14]},n.numIconVertices.set=function(t){this._structArray.uint16[this._pos2+14]=t},n.crossTileID.get=function(){return this._structArray.uint32[this._pos4+8]},n.crossTileID.set=function(t){this._structArray.uint32[this._pos4+8]=t},n.textBoxScale.get=function(){return this._structArray.float32[this._pos4+9]},n.textBoxScale.set=function(t){this._structArray.float32[this._pos4+9]=t},n.radialTextOffset.get=function(){return this._structArray.float32[this._pos4+10]},n.radialTextOffset.set=function(t){this._structArray.float32[this._pos4+10]=t},Object.defineProperties(e.prototype,n),e}(ri);Si.prototype.size=44;var Li=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.get=function(t){return new Si(this,t)},e}(mi);vr("SymbolInstanceArray",Li);var Ri=function(t){function e(){t.apply(this,arguments)}t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e;var n={offsetX:{configurable:!0}};return n.offsetX.get=function(){return this._structArray.float32[this._pos4+0]},n.offsetX.set=function(t){this._structArray.float32[this._pos4+0]=t},Object.defineProperties(e.prototype,n),e}(ri);Ri.prototype.size=4;var Ni=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getoffsetX=function(t){return this.float32[1*t+0]},e.prototype.get=function(t){return new Ri(this,t)},e}(yi);vr("GlyphOffsetArray",Ni);var Fi=function(t){function e(){t.apply(this,arguments)}t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e;var n={x:{configurable:!0},y:{configurable:!0},tileUnitDistanceFromAnchor:{configurable:!0}};return n.x.get=function(){return this._structArray.int16[this._pos2+0]},n.x.set=function(t){this._structArray.int16[this._pos2+0]=t},n.y.get=function(){return this._structArray.int16[this._pos2+1]},n.y.set=function(t){this._structArray.int16[this._pos2+1]=t},n.tileUnitDistanceFromAnchor.get=function(){return this._structArray.int16[this._pos2+2]},n.tileUnitDistanceFromAnchor.set=function(t){this._structArray.int16[this._pos2+2]=t},Object.defineProperties(e.prototype,n),e}(ri);Fi.prototype.size=6;var Ti=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getx=function(t){return this.int16[3*t+0]},e.prototype.gety=function(t){return this.int16[3*t+1]},e.prototype.gettileUnitDistanceFromAnchor=function(t){return this.int16[3*t+2]},e.prototype.get=function(t){return new Fi(this,t)},e}(Ei);vr("SymbolLineVertexArray",Ti);var Ui=function(t){function e(){t.apply(this,arguments)}t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e;var n={featureIndex:{configurable:!0},sourceLayerIndex:{configurable:!0},bucketIndex:{configurable:!0}};return n.featureIndex.get=function(){return this._structArray.uint32[this._pos4+0]},n.featureIndex.set=function(t){this._structArray.uint32[this._pos4+0]=t},n.sourceLayerIndex.get=function(){return this._structArray.uint16[this._pos2+2]},n.sourceLayerIndex.set=function(t){this._structArray.uint16[this._pos2+2]=t},n.bucketIndex.get=function(){return this._structArray.uint16[this._pos2+3]},n.bucketIndex.set=function(t){this._structArray.uint16[this._pos2+3]=t},Object.defineProperties(e.prototype,n),e}(ri);Ui.prototype.size=8;var Oi=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.get=function(t){return new Ui(this,t)},e}(vi);vr("FeatureIndexArray",Oi);var Gi=oi([{name:"a_pos",components:2,type:"Int16"}],4).members,Hi=function(t){void 0===t&&(t=[]),this.segments=t};function ji(t,e){return 256*(t=l(Math.floor(t),0,255))+l(Math.floor(e),0,255)}Hi.prototype.prepareSegment=function(t,e,n,r){var i=this.segments[this.segments.length-1];return t>Hi.MAX_VERTEX_ARRAY_LENGTH&&v("Max vertices per segment is "+Hi.MAX_VERTEX_ARRAY_LENGTH+": bucket requested "+t),(!i||i.vertexLength+t>Hi.MAX_VERTEX_ARRAY_LENGTH||i.sortKey!==r)&&(i={vertexOffset:e.length,primitiveOffset:n.length,vertexLength:0,primitiveLength:0},void 0!==r&&(i.sortKey=r),this.segments.push(i)),i},Hi.prototype.get=function(){return this.segments},Hi.prototype.destroy=function(){for(var t=0,e=this.segments;t<e.length;t+=1){var n=e[t];for(var r in n.vaos)n.vaos[r].destroy()}},Hi.simpleSegment=function(t,e,n,r){return new Hi([{vertexOffset:t,primitiveOffset:e,vertexLength:n,primitiveLength:r,vaos:{},sortKey:0}])},Hi.MAX_VERTEX_ARRAY_LENGTH=Math.pow(2,16)-1,vr("SegmentVector",Hi);var Yi=function(){this.ids=[],this.positions=[],this.indexed=!1};function Pi(t,e,n){var r=t[e];t[e]=t[n],t[n]=r}Yi.prototype.add=function(t,e,n,r){this.ids.push(t),this.positions.push(e,n,r)},Yi.prototype.getPositions=function(t){for(var e=0,n=this.ids.length-1;e<n;){var r=e+n>>1;this.ids[r]>=t?n=r:e=r+1}for(var i=[];this.ids[e]===t;){var o=this.positions[3*e],a=this.positions[3*e+1],s=this.positions[3*e+2];i.push({index:o,start:a,end:s}),e++}return i},Yi.serialize=function(t,e){var n=new Float64Array(t.ids),r=new Uint32Array(t.positions);return function t(e,n,r,i){if(!(r>=i)){for(var o=e[r+i>>1],a=r-1,s=i+1;;){do{a++}while(e[a]<o);do{s--}while(e[s]>o);if(a>=s)break;Pi(e,a,s),Pi(n,3*a,3*s),Pi(n,3*a+1,3*s+1),Pi(n,3*a+2,3*s+2)}t(e,n,r,s),t(e,n,s+1,i)}}(n,r,0,n.length-1),e.push(n.buffer,r.buffer),{ids:n,positions:r}},Yi.deserialize=function(t){var e=new Yi;return e.ids=t.ids,e.positions=t.positions,e.indexed=!0,e},vr("FeaturePositionMap",Yi);var Ki=function(t,e){this.gl=t.gl,this.location=e},zi=function(t){function e(e,n){t.call(this,e,n),this.current=0}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.set=function(t){this.current!==t&&(this.current=t,this.gl.uniform1i(this.location,t))},e}(Ki),Ji=function(t){function e(e,n){t.call(this,e,n),this.current=0}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.set=function(t){this.current!==t&&(this.current=t,this.gl.uniform1f(this.location,t))},e}(Ki),qi=function(t){function e(e,n){t.call(this,e,n),this.current=[0,0]}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.set=function(t){t[0]===this.current[0]&&t[1]===this.current[1]||(this.current=t,this.gl.uniform2f(this.location,t[0],t[1]))},e}(Ki),Vi=function(t){function e(e,n){t.call(this,e,n),this.current=[0,0,0]}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.set=function(t){t[0]===this.current[0]&&t[1]===this.current[1]&&t[2]===this.current[2]||(this.current=t,this.gl.uniform3f(this.location,t[0],t[1],t[2]))},e}(Ki),Wi=function(t){function e(e,n){t.call(this,e,n),this.current=[0,0,0,0]}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.set=function(t){t[0]===this.current[0]&&t[1]===this.current[1]&&t[2]===this.current[2]&&t[3]===this.current[3]||(this.current=t,this.gl.uniform4f(this.location,t[0],t[1],t[2],t[3]))},e}(Ki),Xi=function(t){function e(e,n){t.call(this,e,n),this.current=ne.transparent}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.set=function(t){t.r===this.current.r&&t.g===this.current.g&&t.b===this.current.b&&t.a===this.current.a||(this.current=t,this.gl.uniform4f(this.location,t.r,t.g,t.b,t.a))},e}(Ki),Zi=new Float32Array(16),$i=function(t){function e(e,n){t.call(this,e,n),this.current=Zi}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.set=function(t){if(t[12]!==this.current[12]||t[0]!==this.current[0])return this.current=t,void this.gl.uniformMatrix4fv(this.location,!1,t);for(var e=1;e<16;e++)if(t[e]!==this.current[e]){this.current=t,this.gl.uniformMatrix4fv(this.location,!1,t);break}},e}(Ki);function to(t){return[ji(255*t.r,255*t.g),ji(255*t.b,255*t.a)]}var eo=function(t,e,n){this.value=t,this.names=e,this.uniformNames=this.names.map(function(t){return"u_"+t}),this.type=n,this.maxValue=-1/0};eo.prototype.defines=function(){return this.names.map(function(t){return"#define HAS_UNIFORM_u_"+t})},eo.prototype.setConstantPatternPositions=function(){},eo.prototype.populatePaintArray=function(){},eo.prototype.updatePaintArray=function(){},eo.prototype.upload=function(){},eo.prototype.destroy=function(){},eo.prototype.setUniforms=function(t,e,n,r){e.set(r.constantOr(this.value))},eo.prototype.getBinding=function(t,e){return"color"===this.type?new Xi(t,e):new Ji(t,e)},eo.serialize=function(t){var e=t.value,n=t.names,r=t.type;return{value:br(e),names:n,type:r}},eo.deserialize=function(t){var e=t.value,n=t.names,r=t.type;return new eo(Qr(e),n,r)};var no=function(t,e,n){this.value=t,this.names=e,this.uniformNames=this.names.map(function(t){return"u_"+t}),this.type=n,this.maxValue=-1/0,this.patternPositions={patternTo:null,patternFrom:null}};no.prototype.defines=function(){return this.names.map(function(t){return"#define HAS_UNIFORM_u_"+t})},no.prototype.populatePaintArray=function(){},no.prototype.updatePaintArray=function(){},no.prototype.upload=function(){},no.prototype.destroy=function(){},no.prototype.setConstantPatternPositions=function(t,e){this.patternPositions.patternTo=t.tlbr,this.patternPositions.patternFrom=e.tlbr},no.prototype.setUniforms=function(t,e,n,r,i){var o=this.patternPositions;"u_pattern_to"===i&&o.patternTo&&e.set(o.patternTo),"u_pattern_from"===i&&o.patternFrom&&e.set(o.patternFrom)},no.prototype.getBinding=function(t,e){return new Wi(t,e)};var ro=function(t,e,n,r){this.expression=t,this.names=e,this.type=n,this.uniformNames=this.names.map(function(t){return"a_"+t}),this.maxValue=-1/0,this.paintVertexAttributes=e.map(function(t){return{name:"a_"+t,type:"Float32",components:"color"===n?2:1,offset:0}}),this.paintVertexArray=new r};ro.prototype.defines=function(){return[]},ro.prototype.setConstantPatternPositions=function(){},ro.prototype.populatePaintArray=function(t,e){var n=this.paintVertexArray,r=n.length;n.reserve(t);var i=this.expression.evaluate(new Gr(0),e,{});if("color"===this.type)for(var o=to(i),a=r;a<t;a++)n.emplaceBack(o[0],o[1]);else{for(var s=r;s<t;s++)n.emplaceBack(i);this.maxValue=Math.max(this.maxValue,i)}},ro.prototype.updatePaintArray=function(t,e,n,r){var i=this.paintVertexArray,o=this.expression.evaluate({zoom:0},n,r);if("color"===this.type)for(var a=to(o),s=t;s<e;s++)i.emplace(s,a[0],a[1]);else{for(var A=t;A<e;A++)i.emplace(A,o);this.maxValue=Math.max(this.maxValue,o)}},ro.prototype.upload=function(t){this.paintVertexArray&&this.paintVertexArray.arrayBuffer&&(this.paintVertexBuffer&&this.paintVertexBuffer.buffer?this.paintVertexBuffer.updateData(this.paintVertexArray):this.paintVertexBuffer=t.createVertexBuffer(this.paintVertexArray,this.paintVertexAttributes,this.expression.isStateDependent))},ro.prototype.destroy=function(){this.paintVertexBuffer&&this.paintVertexBuffer.destroy()},ro.prototype.setUniforms=function(t,e){e.set(0)},ro.prototype.getBinding=function(t,e){return new Ji(t,e)};var io=function(t,e,n,r,i,o){this.expression=t,this.names=e,this.uniformNames=this.names.map(function(t){return"u_"+t+"_t"}),this.type=n,this.useIntegerZoom=r,this.zoom=i,this.maxValue=-1/0;var a=o;this.paintVertexAttributes=e.map(function(t){return{name:"a_"+t,type:"Float32",components:"color"===n?4:2,offset:0}}),this.paintVertexArray=new a};io.prototype.defines=function(){return[]},io.prototype.setConstantPatternPositions=function(){},io.prototype.populatePaintArray=function(t,e){var n=this.paintVertexArray,r=n.length;n.reserve(t);var i=this.expression.evaluate(new Gr(this.zoom),e,{}),o=this.expression.evaluate(new Gr(this.zoom+1),e,{});if("color"===this.type)for(var a=to(i),s=to(o),A=r;A<t;A++)n.emplaceBack(a[0],a[1],s[0],s[1]);else{for(var l=r;l<t;l++)n.emplaceBack(i,o);this.maxValue=Math.max(this.maxValue,i,o)}},io.prototype.updatePaintArray=function(t,e,n,r){var i=this.paintVertexArray,o=this.expression.evaluate({zoom:this.zoom},n,r),a=this.expression.evaluate({zoom:this.zoom+1},n,r);if("color"===this.type)for(var s=to(o),A=to(a),l=t;l<e;l++)i.emplace(l,s[0],s[1],A[0],A[1]);else{for(var c=t;c<e;c++)i.emplace(c,o,a);this.maxValue=Math.max(this.maxValue,o,a)}},io.prototype.upload=function(t){this.paintVertexArray&&this.paintVertexArray.arrayBuffer&&(this.paintVertexBuffer&&this.paintVertexBuffer.buffer?this.paintVertexBuffer.updateData(this.paintVertexArray):this.paintVertexBuffer=t.createVertexBuffer(this.paintVertexArray,this.paintVertexAttributes,this.expression.isStateDependent))},io.prototype.destroy=function(){this.paintVertexBuffer&&this.paintVertexBuffer.destroy()},io.prototype.interpolationFactor=function(t){return this.useIntegerZoom?this.expression.interpolationFactor(Math.floor(t),this.zoom,this.zoom+1):this.expression.interpolationFactor(t,this.zoom,this.zoom+1)},io.prototype.setUniforms=function(t,e,n){e.set(this.interpolationFactor(n.zoom))},io.prototype.getBinding=function(t,e){return new Ji(t,e)};var oo=function(t,e,n,r,i,o,a){this.expression=t,this.names=e,this.type=n,this.uniformNames=this.names.map(function(t){return"u_"+t+"_t"}),this.useIntegerZoom=r,this.zoom=i,this.maxValue=-1/0,this.layerId=a,this.paintVertexAttributes=e.map(function(t){return{name:"a_"+t,type:"Uint16",components:4,offset:0}}),this.zoomInPaintVertexArray=new o,this.zoomOutPaintVertexArray=new o};oo.prototype.defines=function(){return[]},oo.prototype.setConstantPatternPositions=function(){},oo.prototype.populatePaintArray=function(t,e,n){var r=this.zoomInPaintVertexArray,i=this.zoomOutPaintVertexArray,o=this.layerId,a=r.length;if(r.reserve(t),i.reserve(t),n&&e.patterns&&e.patterns[o]){var s=e.patterns[o],A=s.min,l=s.mid,c=s.max,u=n[A],g=n[l],h=n[c];if(!u||!g||!h)return;for(var d=a;d<t;d++)r.emplaceBack(g.tl[0],g.tl[1],g.br[0],g.br[1],u.tl[0],u.tl[1],u.br[0],u.br[1]),i.emplaceBack(g.tl[0],g.tl[1],g.br[0],g.br[1],h.tl[0],h.tl[1],h.br[0],h.br[1])}},oo.prototype.updatePaintArray=function(t,e,n,r,i){var o=this.zoomInPaintVertexArray,a=this.zoomOutPaintVertexArray,s=this.layerId;if(i&&n.patterns&&n.patterns[s]){var A=n.patterns[s],l=A.min,c=A.mid,u=A.max,g=i[l],h=i[c],d=i[u];if(!g||!h||!d)return;for(var f=t;f<e;f++)o.emplace(f,h.tl[0],h.tl[1],h.br[0],h.br[1],g.tl[0],g.tl[1],g.br[0],g.br[1]),a.emplace(f,h.tl[0],h.tl[1],h.br[0],h.br[1],d.tl[0],d.tl[1],d.br[0],d.br[1])}},oo.prototype.upload=function(t){this.zoomInPaintVertexArray&&this.zoomInPaintVertexArray.arrayBuffer&&this.zoomOutPaintVertexArray&&this.zoomOutPaintVertexArray.arrayBuffer&&(this.zoomInPaintVertexBuffer=t.createVertexBuffer(this.zoomInPaintVertexArray,this.paintVertexAttributes,this.expression.isStateDependent),this.zoomOutPaintVertexBuffer=t.createVertexBuffer(this.zoomOutPaintVertexArray,this.paintVertexAttributes,this.expression.isStateDependent))},oo.prototype.destroy=function(){this.zoomOutPaintVertexBuffer&&this.zoomOutPaintVertexBuffer.destroy(),this.zoomInPaintVertexBuffer&&this.zoomInPaintVertexBuffer.destroy()},oo.prototype.setUniforms=function(t,e){e.set(0)},oo.prototype.getBinding=function(t,e){return new Ji(t,e)};var ao=function(){this.binders={},this.cacheKey="",this._buffers=[],this._featureMap=new Yi,this._bufferOffset=0};ao.createDynamic=function(t,e,n){var r=new ao,i=[];for(var o in t.paint._values)if(n(o)){var a=t.paint.get(o);if(a instanceof Jr&&mn(a.property.specification)){var s=Ao(o,t.type),A=a.property.specification.type,l=a.property.useIntegerZoom;if("cross-faded"===a.property.specification["property-type"]||"cross-faded-data-driven"===a.property.specification["property-type"])if("constant"===a.value.kind)r.binders[o]=new no(a.value.value,s,A),i.push("/u_"+o);else{var c=lo(o,A,"source");r.binders[o]=new oo(a.value,s,A,l,e,c,t.id),i.push("/a_"+o)}else if("constant"===a.value.kind)r.binders[o]=new eo(a.value.value,s,A),i.push("/u_"+o);else if("source"===a.value.kind){var u=lo(o,A,"source");r.binders[o]=new ro(a.value,s,A,u),i.push("/a_"+o)}else{var g=lo(o,A,"composite");r.binders[o]=new io(a.value,s,A,l,e,g),i.push("/z_"+o)}}}return r.cacheKey=i.sort().join(""),r},ao.prototype.populatePaintArrays=function(t,e,n,r){for(var i in this.binders)this.binders[i].populatePaintArray(t,e,r);void 0!==e.id&&this._featureMap.add(+e.id,n,this._bufferOffset,t),this._bufferOffset=t},ao.prototype.setConstantPatternPositions=function(t,e){for(var n in this.binders)this.binders[n].setConstantPatternPositions(t,e)},ao.prototype.updatePaintArrays=function(t,e,n,r){var i=!1;for(var o in t)for(var a=0,s=this._featureMap.getPositions(+o);a<s.length;a+=1){var A=s[a],l=e.feature(A.index);for(var c in this.binders){var u=this.binders[c];if(!(u instanceof eo||u instanceof no)&&!0===u.expression.isStateDependent){var g=n.paint.get(c);u.expression=g.value,u.updatePaintArray(A.start,A.end,l,t[o],r),i=!0}}}return i},ao.prototype.defines=function(){var t=[];for(var e in this.binders)t.push.apply(t,this.binders[e].defines());return t},ao.prototype.getPaintVertexBuffers=function(){return this._buffers},ao.prototype.getUniforms=function(t,e){var n=[];for(var r in this.binders)for(var i=this.binders[r],o=0,a=i.uniformNames;o<a.length;o+=1){var s=a[o];if(e[s]){var A=i.getBinding(t,e[s]);n.push({name:s,property:r,binding:A})}}return n},ao.prototype.setUniforms=function(t,e,n,r){for(var i=0,o=e;i<o.length;i+=1){var a=o[i],s=a.name,A=a.property,l=a.binding;this.binders[A].setUniforms(t,l,r,n.get(A),s)}},ao.prototype.updatePatternPaintBuffers=function(t){var e=[];for(var n in this.binders){var r=this.binders[n];if(r instanceof oo){var i=2===t.fromScale?r.zoomInPaintVertexBuffer:r.zoomOutPaintVertexBuffer;i&&e.push(i)}else(r instanceof ro||r instanceof io)&&r.paintVertexBuffer&&e.push(r.paintVertexBuffer)}this._buffers=e},ao.prototype.upload=function(t){for(var e in this.binders)this.binders[e].upload(t);var n=[];for(var r in this.binders){var i=this.binders[r];(i instanceof ro||i instanceof io)&&i.paintVertexBuffer&&n.push(i.paintVertexBuffer)}this._buffers=n},ao.prototype.destroy=function(){for(var t in this.binders)this.binders[t].destroy()};var so=function(t,e,n,r){void 0===r&&(r=function(){return!0}),this.programConfigurations={};for(var i=0,o=e;i<o.length;i+=1){var a=o[i];this.programConfigurations[a.id]=ao.createDynamic(a,n,r),this.programConfigurations[a.id].layoutAttributes=t}this.needsUpload=!1};function Ao(t,e){return{"text-opacity":["opacity"],"icon-opacity":["opacity"],"text-color":["fill_color"],"icon-color":["fill_color"],"text-halo-color":["halo_color"],"icon-halo-color":["halo_color"],"text-halo-blur":["halo_blur"],"icon-halo-blur":["halo_blur"],"text-halo-width":["halo_width"],"icon-halo-width":["halo_width"],"line-gap-width":["gapwidth"],"line-pattern":["pattern_to","pattern_from"],"fill-pattern":["pattern_to","pattern_from"],"fill-extrusion-pattern":["pattern_to","pattern_from"]}[t]||[t.replace(e+"-","").replace(/-/g,"_")]}function lo(t,e,n){var r={color:{source:wi,composite:xi},number:{source:yi,composite:wi}},i=function(t){return{"line-pattern":{source:ui,composite:ui},"fill-pattern":{source:ui,composite:ui},"fill-extrusion-pattern":{source:ui,composite:ui}}[t]}(t);return i&&i[n]||r[e][n]}so.prototype.populatePaintArrays=function(t,e,n,r){for(var i in this.programConfigurations)this.programConfigurations[i].populatePaintArrays(t,e,n,r);this.needsUpload=!0},so.prototype.updatePaintArrays=function(t,e,n,r){for(var i=0,o=n;i<o.length;i+=1){var a=o[i];this.needsUpload=this.programConfigurations[a.id].updatePaintArrays(t,e,a,r)||this.needsUpload}},so.prototype.get=function(t){return this.programConfigurations[t]},so.prototype.upload=function(t){if(this.needsUpload){for(var e in this.programConfigurations)this.programConfigurations[e].upload(t);this.needsUpload=!1}},so.prototype.destroy=function(){for(var t in this.programConfigurations)this.programConfigurations[t].destroy()},vr("ConstantBinder",eo),vr("CrossFadedConstantBinder",no),vr("SourceExpressionBinder",ro),vr("CrossFadedCompositeBinder",oo),vr("CompositeExpressionBinder",io),vr("ProgramConfiguration",ao,{omit:["_buffers"]}),vr("ProgramConfigurationSet",so);var co=8192,uo={min:-1*Math.pow(2,15),max:Math.pow(2,15)-1};function go(t){for(var e=co/t.extent,n=t.loadGeometry(),r=0;r<n.length;r++)for(var i=n[r],o=0;o<i.length;o++){var a=i[o];a.x=Math.round(a.x*e),a.y=Math.round(a.y*e),(a.x<uo.min||a.x>uo.max||a.y<uo.min||a.y>uo.max)&&v("Geometry exceeds allowed extent, reduce your vector tile buffer size")}return n}function ho(t,e,n,r,i){t.emplaceBack(2*e+(r+1)/2,2*n+(i+1)/2)}var fo=function(t){this.zoom=t.zoom,this.overscaling=t.overscaling,this.layers=t.layers,this.layerIds=this.layers.map(function(t){return t.id}),this.index=t.index,this.hasPattern=!1,this.layoutVertexArray=new si,this.indexArray=new Bi,this.segments=new Hi,this.programConfigurations=new so(Gi,t.layers,t.zoom),this.stateDependentLayerIds=this.layers.filter(function(t){return t.isStateDependent()}).map(function(t){return t.id})};function po(t,e){for(var n=0;n<t.length;n++)if(Qo(e,t[n]))return!0;for(var r=0;r<e.length;r++)if(Qo(t,e[r]))return!0;return!!yo(t,e)}function Io(t,e,n){return!!Qo(t,e)||!!vo(e,t,n)}function Co(t,e){if(1===t.length)return bo(e,t[0]);for(var n=0;n<e.length;n++)for(var r=e[n],i=0;i<r.length;i++)if(Qo(t,r[i]))return!0;for(var o=0;o<t.length;o++)if(bo(e,t[o]))return!0;for(var a=0;a<e.length;a++)if(yo(t,e[a]))return!0;return!1}function mo(t,e,n){if(t.length>1){if(yo(t,e))return!0;for(var r=0;r<e.length;r++)if(vo(e[r],t,n))return!0}for(var i=0;i<t.length;i++)if(vo(t[i],e,n))return!0;return!1}function yo(t,e){if(0===t.length||0===e.length)return!1;for(var n=0;n<t.length-1;n++)for(var r=t[n],i=t[n+1],o=0;o<e.length-1;o++)if(Eo(r,i,e[o],e[o+1]))return!0;return!1}function Eo(t,e,n,r){return B(t,n,r)!==B(e,n,r)&&B(t,e,n)!==B(t,e,r)}function vo(t,e,n){var r=n*n;if(1===e.length)return t.distSqr(e[0])<r;for(var i=1;i<e.length;i++)if(Bo(t,e[i-1],e[i])<r)return!0;return!1}function Bo(t,e,n){var r=e.distSqr(n);if(0===r)return t.distSqr(e);var i=((t.x-e.x)*(n.x-e.x)+(t.y-e.y)*(n.y-e.y))/r;return i<0?t.distSqr(e):i>1?t.distSqr(n):t.distSqr(n.sub(e)._mult(i)._add(e))}function bo(t,e){for(var n,r,i,o=!1,a=0;a<t.length;a++)for(var s=0,A=(n=t[a]).length-1;s<n.length;A=s++)r=n[s],i=n[A],r.y>e.y!=i.y>e.y&&e.x<(i.x-r.x)*(e.y-r.y)/(i.y-r.y)+r.x&&(o=!o);return o}function Qo(t,e){for(var n=!1,r=0,i=t.length-1;r<t.length;i=r++){var o=t[r],a=t[i];o.y>e.y!=a.y>e.y&&e.x<(a.x-o.x)*(e.y-o.y)/(a.y-o.y)+o.x&&(n=!n)}return n}function wo(t,e,n){var r=n[0],i=n[2];if(t.x<r.x&&e.x<r.x||t.x>i.x&&e.x>i.x||t.y<r.y&&e.y<r.y||t.y>i.y&&e.y>i.y)return!1;var o=B(t,e,n[0]);return o!==B(t,e,n[1])||o!==B(t,e,n[2])||o!==B(t,e,n[3])}function xo(t,e,n){var r=e.paint.get(t).value;return"constant"===r.kind?r.value:n.programConfigurations.get(e.id).binders[t].maxValue}function _o(t){return Math.sqrt(t[0]*t[0]+t[1]*t[1])}function Do(t,e,n,r,o){if(!e[0]&&!e[1])return t;var a=i.convert(e)._mult(o);"viewport"===n&&a._rotate(-r);for(var s=[],A=0;A<t.length;A++){var l=t[A];s.push(l.sub(a))}return s}fo.prototype.populate=function(t,e){for(var n=0,r=t;n<r.length;n+=1){var i=r[n],o=i.feature,a=i.index,s=i.sourceLayerIndex;if(this.layers[0]._featureFilter(new Gr(this.zoom),o)){var A=go(o);this.addFeature(o,A,a),e.featureIndex.insert(o,A,a,s,this.index)}}},fo.prototype.update=function(t,e,n){this.stateDependentLayers.length&&this.programConfigurations.updatePaintArrays(t,e,this.stateDependentLayers,n)},fo.prototype.isEmpty=function(){return 0===this.layoutVertexArray.length},fo.prototype.uploadPending=function(){return!this.uploaded||this.programConfigurations.needsUpload},fo.prototype.upload=function(t){this.uploaded||(this.layoutVertexBuffer=t.createVertexBuffer(this.layoutVertexArray,Gi),this.indexBuffer=t.createIndexBuffer(this.indexArray)),this.programConfigurations.upload(t),this.uploaded=!0},fo.prototype.destroy=function(){this.layoutVertexBuffer&&(this.layoutVertexBuffer.destroy(),this.indexBuffer.destroy(),this.programConfigurations.destroy(),this.segments.destroy())},fo.prototype.addFeature=function(t,e,n){for(var r=0,i=e;r<i.length;r+=1)for(var o=0,a=i[r];o<a.length;o+=1){var s=a[o],A=s.x,l=s.y;if(!(A<0||A>=co||l<0||l>=co)){var c=this.segments.prepareSegment(4,this.layoutVertexArray,this.indexArray),u=c.vertexLength;ho(this.layoutVertexArray,A,l,-1,-1),ho(this.layoutVertexArray,A,l,1,-1),ho(this.layoutVertexArray,A,l,1,1),ho(this.layoutVertexArray,A,l,-1,1),this.indexArray.emplaceBack(u,u+1,u+2),this.indexArray.emplaceBack(u,u+3,u+2),c.vertexLength+=4,c.primitiveLength+=2}}this.programConfigurations.populatePaintArrays(this.layoutVertexArray.length,t,n,{})},vr("CircleBucket",fo,{omit:["layers"]});var Mo={paint:new ti({"circle-radius":new Wr(Lt.paint_circle["circle-radius"]),"circle-color":new Wr(Lt.paint_circle["circle-color"]),"circle-blur":new Wr(Lt.paint_circle["circle-blur"]),"circle-opacity":new Wr(Lt.paint_circle["circle-opacity"]),"circle-translate":new Vr(Lt.paint_circle["circle-translate"]),"circle-translate-anchor":new Vr(Lt.paint_circle["circle-translate-anchor"]),"circle-pitch-scale":new Vr(Lt.paint_circle["circle-pitch-scale"]),"circle-pitch-alignment":new Vr(Lt.paint_circle["circle-pitch-alignment"]),"circle-stroke-width":new Wr(Lt.paint_circle["circle-stroke-width"]),"circle-stroke-color":new Wr(Lt.paint_circle["circle-stroke-color"]),"circle-stroke-opacity":new Wr(Lt.paint_circle["circle-stroke-opacity"])})},ko="undefined"!=typeof Float32Array?Float32Array:Array;function So(){var t=new ko(9);return ko!=Float32Array&&(t[1]=0,t[2]=0,t[3]=0,t[5]=0,t[6]=0,t[7]=0),t[0]=1,t[4]=1,t[8]=1,t}function Lo(){var t=new ko(3);return ko!=Float32Array&&(t[0]=0,t[1]=0,t[2]=0),t}function Ro(t,e,n){var r=new ko(3);return r[0]=t,r[1]=e,r[2]=n,r}function No(t,e,n){var r=e[0],i=e[1],o=e[2],a=e[3];return t[0]=n[0]*r+n[4]*i+n[8]*o+n[12]*a,t[1]=n[1]*r+n[5]*i+n[9]*o+n[13]*a,t[2]=n[2]*r+n[6]*i+n[10]*o+n[14]*a,t[3]=n[3]*r+n[7]*i+n[11]*o+n[15]*a,t}function Fo(){var t=new ko(4);return ko!=Float32Array&&(t[0]=0,t[1]=0,t[2]=0),t[3]=1,t}Math.PI,Lo(),function(){var t;t=new ko(4),ko!=Float32Array&&(t[0]=0,t[1]=0,t[2]=0,t[3]=0)}(),Lo(),Ro(1,0,0),Ro(0,1,0),Fo(),Fo(),So(),function(){var t;t=new ko(2),ko!=Float32Array&&(t[0]=0,t[1]=0)}();var To=function(t){function e(e){t.call(this,e,Mo)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.createBucket=function(t){return new fo(t)},e.prototype.queryRadius=function(t){var e=t;return xo("circle-radius",this,e)+xo("circle-stroke-width",this,e)+_o(this.paint.get("circle-translate"))},e.prototype.queryIntersectsFeature=function(t,e,n,r,i,o,a,s){for(var A=Do(t,this.paint.get("circle-translate"),this.paint.get("circle-translate-anchor"),o.angle,a),l=this.paint.get("circle-radius").evaluate(e,n)+this.paint.get("circle-stroke-width").evaluate(e,n),c="map"===this.paint.get("circle-pitch-alignment"),u=c?A:function(t,e){return A.map(function(t){return Uo(t,e)})}(0,s),g=c?l*a:l,h=0,d=r;h<d.length;h+=1)for(var f=0,p=d[h];f<p.length;f+=1){var I=p[f],C=c?I:Uo(I,s),m=g,y=No([],[I.x,I.y,0,1],s);if("viewport"===this.paint.get("circle-pitch-scale")&&"map"===this.paint.get("circle-pitch-alignment")?m*=y[3]/o.cameraToCenterDistance:"map"===this.paint.get("circle-pitch-scale")&&"viewport"===this.paint.get("circle-pitch-alignment")&&(m*=o.cameraToCenterDistance/y[3]),Io(u,C,m))return!0}return!1},e}(ei);function Uo(t,e){var n=No([],[t.x,t.y,0,1],e);return new i(n[0]/n[3],n[1]/n[3])}var Oo=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e}(fo);function Go(t,e,n,r){var i=e.width,o=e.height;if(r){if(r instanceof Uint8ClampedArray)r=new Uint8Array(r.buffer);else if(r.length!==i*o*n)throw new RangeError("mismatched image size")}else r=new Uint8Array(i*o*n);return t.width=i,t.height=o,t.data=r,t}function Ho(t,e,n){var r=e.width,i=e.height;if(r!==t.width||i!==t.height){var o=Go({},{width:r,height:i},n);jo(t,o,{x:0,y:0},{x:0,y:0},{width:Math.min(t.width,r),height:Math.min(t.height,i)},n),t.width=r,t.height=i,t.data=o.data}}function jo(t,e,n,r,i,o){if(0===i.width||0===i.height)return e;if(i.width>t.width||i.height>t.height||n.x>t.width-i.width||n.y>t.height-i.height)throw new RangeError("out of range source coordinates for image copy");if(i.width>e.width||i.height>e.height||r.x>e.width-i.width||r.y>e.height-i.height)throw new RangeError("out of range destination coordinates for image copy");for(var a=t.data,s=e.data,A=0;A<i.height;A++)for(var l=((n.y+A)*t.width+n.x)*o,c=((r.y+A)*e.width+r.x)*o,u=0;u<i.width*o;u++)s[c+u]=a[l+u];return e}vr("HeatmapBucket",Oo,{omit:["layers"]});var Yo=function(t,e){Go(this,t,1,e)};Yo.prototype.resize=function(t){Ho(this,t,1)},Yo.prototype.clone=function(){return new Yo({width:this.width,height:this.height},new Uint8Array(this.data))},Yo.copy=function(t,e,n,r,i){jo(t,e,n,r,i,1)};var Po=function(t,e){Go(this,t,4,e)};Po.prototype.resize=function(t){Ho(this,t,4)},Po.prototype.replace=function(t,e){e?this.data.set(t):t instanceof Uint8ClampedArray?this.data=new Uint8Array(t.buffer):this.data=t},Po.prototype.clone=function(){return new Po({width:this.width,height:this.height},new Uint8Array(this.data))},Po.copy=function(t,e,n,r,i){jo(t,e,n,r,i,4)},vr("AlphaImage",Yo),vr("RGBAImage",Po);var Ko={paint:new ti({"heatmap-radius":new Wr(Lt.paint_heatmap["heatmap-radius"]),"heatmap-weight":new Wr(Lt.paint_heatmap["heatmap-weight"]),"heatmap-intensity":new Vr(Lt.paint_heatmap["heatmap-intensity"]),"heatmap-color":new $r(Lt.paint_heatmap["heatmap-color"]),"heatmap-opacity":new Vr(Lt.paint_heatmap["heatmap-opacity"])})};function zo(t,e){for(var n=new Uint8Array(1024),r={},i=0,o=0;i<256;i++,o+=4){r[e]=i/255;var a=t.evaluate(r);n[o+0]=Math.floor(255*a.r/a.a),n[o+1]=Math.floor(255*a.g/a.a),n[o+2]=Math.floor(255*a.b/a.a),n[o+3]=Math.floor(255*a.a)}return new Po({width:256,height:1},n)}var Jo=function(t){function e(e){t.call(this,e,Ko),this._updateColorRamp()}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.createBucket=function(t){return new Oo(t)},e.prototype._handleSpecialPaintPropertyUpdate=function(t){"heatmap-color"===t&&this._updateColorRamp()},e.prototype._updateColorRamp=function(){var t=this._transitionablePaint._values["heatmap-color"].value.expression;this.colorRamp=zo(t,"heatmapDensity"),this.colorRampTexture=null},e.prototype.resize=function(){this.heatmapFbo&&(this.heatmapFbo.destroy(),this.heatmapFbo=null)},e.prototype.queryRadius=function(){return 0},e.prototype.queryIntersectsFeature=function(){return!1},e.prototype.hasOffscreenPass=function(){return 0!==this.paint.get("heatmap-opacity")&&"none"!==this.visibility},e}(ei),qo={paint:new ti({"hillshade-illumination-direction":new Vr(Lt.paint_hillshade["hillshade-illumination-direction"]),"hillshade-illumination-anchor":new Vr(Lt.paint_hillshade["hillshade-illumination-anchor"]),"hillshade-exaggeration":new Vr(Lt.paint_hillshade["hillshade-exaggeration"]),"hillshade-shadow-color":new Vr(Lt.paint_hillshade["hillshade-shadow-color"]),"hillshade-highlight-color":new Vr(Lt.paint_hillshade["hillshade-highlight-color"]),"hillshade-accent-color":new Vr(Lt.paint_hillshade["hillshade-accent-color"])})},Vo=function(t){function e(e){t.call(this,e,qo)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.hasOffscreenPass=function(){return 0!==this.paint.get("hillshade-exaggeration")&&"none"!==this.visibility},e}(ei),Wo=oi([{name:"a_pos",components:2,type:"Int16"}],4).members,Xo=$o,Zo=$o;function $o(t,e,n){n=n||2;var r,i,o,a,s,A,l,c=e&&e.length,u=c?e[0]*n:t.length,g=ta(t,0,u,n,!0),h=[];if(!g||g.next===g.prev)return h;if(c&&(g=function(t,e,n,r){var i,o,a,s=[];for(i=0,o=e.length;i<o;i++)(a=ta(t,e[i]*r,i<o-1?e[i+1]*r:t.length,r,!1))===a.next&&(a.steiner=!0),s.push(ca(a));for(s.sort(sa),i=0;i<s.length;i++)Aa(s[i],n),n=ea(n,n.next);return n}(t,e,g,n)),t.length>80*n){r=o=t[0],i=a=t[1];for(var d=n;d<u;d+=n)(s=t[d])<r&&(r=s),(A=t[d+1])<i&&(i=A),s>o&&(o=s),A>a&&(a=A);l=0!==(l=Math.max(o-r,a-i))?1/l:0}return na(g,h,n,r,i,l),h}function ta(t,e,n,r,i){var o,a;if(i===Ea(t,e,n,r)>0)for(o=e;o<n;o+=r)a=Ca(o,t[o],t[o+1],a);else for(o=n-r;o>=e;o-=r)a=Ca(o,t[o],t[o+1],a);return a&&da(a,a.next)&&(ma(a),a=a.next),a}function ea(t,e){if(!t)return t;e||(e=t);var n,r=t;do{if(n=!1,r.steiner||!da(r,r.next)&&0!==ha(r.prev,r,r.next))r=r.next;else{if(ma(r),(r=e=r.prev)===r.next)break;n=!0}}while(n||r!==e);return e}function na(t,e,n,r,i,o,a){if(t){!a&&o&&function(t,e,n,r){var i=t;do{null===i.z&&(i.z=la(i.x,i.y,e,n,r)),i.prevZ=i.prev,i.nextZ=i.next,i=i.next}while(i!==t);i.prevZ.nextZ=null,i.prevZ=null,function(t){var e,n,r,i,o,a,s,A,l=1;do{for(n=t,t=null,o=null,a=0;n;){for(a++,r=n,s=0,e=0;e<l&&(s++,r=r.nextZ);e++);for(A=l;s>0||A>0&&r;)0!==s&&(0===A||!r||n.z<=r.z)?(i=n,n=n.nextZ,s--):(i=r,r=r.nextZ,A--),o?o.nextZ=i:t=i,i.prevZ=o,o=i;n=r}o.nextZ=null,l*=2}while(a>1)}(i)}(t,r,i,o);for(var s,A,l=t;t.prev!==t.next;)if(s=t.prev,A=t.next,o?ia(t,r,i,o):ra(t))e.push(s.i/n),e.push(t.i/n),e.push(A.i/n),ma(t),t=A.next,l=A.next;else if((t=A)===l){a?1===a?na(t=oa(t,e,n),e,n,r,i,o,2):2===a&&aa(t,e,n,r,i,o):na(ea(t),e,n,r,i,o,1);break}}}function ra(t){var e=t.prev,n=t,r=t.next;if(ha(e,n,r)>=0)return!1;for(var i=t.next.next;i!==t.prev;){if(ua(e.x,e.y,n.x,n.y,r.x,r.y,i.x,i.y)&&ha(i.prev,i,i.next)>=0)return!1;i=i.next}return!0}function ia(t,e,n,r){var i=t.prev,o=t,a=t.next;if(ha(i,o,a)>=0)return!1;for(var s=i.x<o.x?i.x<a.x?i.x:a.x:o.x<a.x?o.x:a.x,A=i.y<o.y?i.y<a.y?i.y:a.y:o.y<a.y?o.y:a.y,l=i.x>o.x?i.x>a.x?i.x:a.x:o.x>a.x?o.x:a.x,c=i.y>o.y?i.y>a.y?i.y:a.y:o.y>a.y?o.y:a.y,u=la(s,A,e,n,r),g=la(l,c,e,n,r),h=t.prevZ,d=t.nextZ;h&&h.z>=u&&d&&d.z<=g;){if(h!==t.prev&&h!==t.next&&ua(i.x,i.y,o.x,o.y,a.x,a.y,h.x,h.y)&&ha(h.prev,h,h.next)>=0)return!1;if(h=h.prevZ,d!==t.prev&&d!==t.next&&ua(i.x,i.y,o.x,o.y,a.x,a.y,d.x,d.y)&&ha(d.prev,d,d.next)>=0)return!1;d=d.nextZ}for(;h&&h.z>=u;){if(h!==t.prev&&h!==t.next&&ua(i.x,i.y,o.x,o.y,a.x,a.y,h.x,h.y)&&ha(h.prev,h,h.next)>=0)return!1;h=h.prevZ}for(;d&&d.z<=g;){if(d!==t.prev&&d!==t.next&&ua(i.x,i.y,o.x,o.y,a.x,a.y,d.x,d.y)&&ha(d.prev,d,d.next)>=0)return!1;d=d.nextZ}return!0}function oa(t,e,n){var r=t;do{var i=r.prev,o=r.next.next;!da(i,o)&&fa(i,r,r.next,o)&&pa(i,o)&&pa(o,i)&&(e.push(i.i/n),e.push(r.i/n),e.push(o.i/n),ma(r),ma(r.next),r=t=o),r=r.next}while(r!==t);return r}function aa(t,e,n,r,i,o){var a=t;do{for(var s=a.next.next;s!==a.prev;){if(a.i!==s.i&&ga(a,s)){var A=Ia(a,s);return a=ea(a,a.next),A=ea(A,A.next),na(a,e,n,r,i,o),void na(A,e,n,r,i,o)}s=s.next}a=a.next}while(a!==t)}function sa(t,e){return t.x-e.x}function Aa(t,e){if(e=function(t,e){var n,r=e,i=t.x,o=t.y,a=-1/0;do{if(o<=r.y&&o>=r.next.y&&r.next.y!==r.y){var s=r.x+(o-r.y)*(r.next.x-r.x)/(r.next.y-r.y);if(s<=i&&s>a){if(a=s,s===i){if(o===r.y)return r;if(o===r.next.y)return r.next}n=r.x<r.next.x?r:r.next}}r=r.next}while(r!==e);if(!n)return null;if(i===a)return n.prev;var A,l=n,c=n.x,u=n.y,g=1/0;for(r=n.next;r!==l;)i>=r.x&&r.x>=c&&i!==r.x&&ua(o<u?i:a,o,c,u,o<u?a:i,o,r.x,r.y)&&((A=Math.abs(o-r.y)/(i-r.x))<g||A===g&&r.x>n.x)&&pa(r,t)&&(n=r,g=A),r=r.next;return n}(t,e)){var n=Ia(e,t);ea(n,n.next)}}function la(t,e,n,r,i){return(t=1431655765&((t=858993459&((t=252645135&((t=16711935&((t=32767*(t-n)*i)|t<<8))|t<<4))|t<<2))|t<<1))|(e=1431655765&((e=858993459&((e=252645135&((e=16711935&((e=32767*(e-r)*i)|e<<8))|e<<4))|e<<2))|e<<1))<<1}function ca(t){var e=t,n=t;do{(e.x<n.x||e.x===n.x&&e.y<n.y)&&(n=e),e=e.next}while(e!==t);return n}function ua(t,e,n,r,i,o,a,s){return(i-a)*(e-s)-(t-a)*(o-s)>=0&&(t-a)*(r-s)-(n-a)*(e-s)>=0&&(n-a)*(o-s)-(i-a)*(r-s)>=0}function ga(t,e){return t.next.i!==e.i&&t.prev.i!==e.i&&!function(t,e){var n=t;do{if(n.i!==t.i&&n.next.i!==t.i&&n.i!==e.i&&n.next.i!==e.i&&fa(n,n.next,t,e))return!0;n=n.next}while(n!==t);return!1}(t,e)&&pa(t,e)&&pa(e,t)&&function(t,e){var n=t,r=!1,i=(t.x+e.x)/2,o=(t.y+e.y)/2;do{n.y>o!=n.next.y>o&&n.next.y!==n.y&&i<(n.next.x-n.x)*(o-n.y)/(n.next.y-n.y)+n.x&&(r=!r),n=n.next}while(n!==t);return r}(t,e)}function ha(t,e,n){return(e.y-t.y)*(n.x-e.x)-(e.x-t.x)*(n.y-e.y)}function da(t,e){return t.x===e.x&&t.y===e.y}function fa(t,e,n,r){return!!(da(t,e)&&da(n,r)||da(t,r)&&da(n,e))||ha(t,e,n)>0!=ha(t,e,r)>0&&ha(n,r,t)>0!=ha(n,r,e)>0}function pa(t,e){return ha(t.prev,t,t.next)<0?ha(t,e,t.next)>=0&&ha(t,t.prev,e)>=0:ha(t,e,t.prev)<0||ha(t,t.next,e)<0}function Ia(t,e){var n=new ya(t.i,t.x,t.y),r=new ya(e.i,e.x,e.y),i=t.next,o=e.prev;return t.next=e,e.prev=t,n.next=i,i.prev=n,r.next=n,n.prev=r,o.next=r,r.prev=o,r}function Ca(t,e,n,r){var i=new ya(t,e,n);return r?(i.next=r.next,i.prev=r,r.next.prev=i,r.next=i):(i.prev=i,i.next=i),i}function ma(t){t.next.prev=t.prev,t.prev.next=t.next,t.prevZ&&(t.prevZ.nextZ=t.nextZ),t.nextZ&&(t.nextZ.prevZ=t.prevZ)}function ya(t,e,n){this.i=t,this.x=e,this.y=n,this.prev=null,this.next=null,this.z=null,this.prevZ=null,this.nextZ=null,this.steiner=!1}function Ea(t,e,n,r){for(var i=0,o=e,a=n-r;o<n;o+=r)i+=(t[a]-t[o])*(t[o+1]+t[a+1]),a=o;return i}function va(t,e,n,r,i){!function t(e,n,r,i,o){for(;i>r;){if(i-r>600){var a=i-r+1,s=n-r+1,A=Math.log(a),l=.5*Math.exp(2*A/3),c=.5*Math.sqrt(A*l*(a-l)/a)*(s-a/2<0?-1:1);t(e,n,Math.max(r,Math.floor(n-s*l/a+c)),Math.min(i,Math.floor(n+(a-s)*l/a+c)),o)}var u=e[n],g=r,h=i;for(Ba(e,r,n),o(e[i],u)>0&&Ba(e,r,i);g<h;){for(Ba(e,g,h),g++,h--;o(e[g],u)<0;)g++;for(;o(e[h],u)>0;)h--}0===o(e[r],u)?Ba(e,r,h):Ba(e,++h,i),h<=n&&(r=h+1),n<=h&&(i=h-1)}}(t,e,n||0,r||t.length-1,i||ba)}function Ba(t,e,n){var r=t[e];t[e]=t[n],t[n]=r}function ba(t,e){return t<e?-1:t>e?1:0}function Qa(t,e){var n=t.length;if(n<=1)return[t];for(var r,i,o=[],a=0;a<n;a++){var s=b(t[a]);0!==s&&(t[a].area=Math.abs(s),void 0===i&&(i=s<0),i===s<0?(r&&o.push(r),r=[t[a]]):r.push(t[a]))}if(r&&o.push(r),e>1)for(var A=0;A<o.length;A++)o[A].length<=e||(va(o[A],e,1,o[A].length-1,wa),o[A]=o[A].slice(0,e));return o}function wa(t,e){return e.area-t.area}function xa(t,e,n){for(var r=n.patternDependencies,i=!1,o=0,a=e;o<a.length;o+=1){var s=a[o].paint.get(t+"-pattern");s.isConstant()||(i=!0);var A=s.constantOr(null);A&&(i=!0,r[A.to]=!0,r[A.from]=!0)}return i}function _a(t,e,n,r,i){for(var o=i.patternDependencies,a=0,s=e;a<s.length;a+=1){var A=s[a],l=A.paint.get(t+"-pattern").value;if("constant"!==l.kind){var c=l.evaluate({zoom:r-1},n,{}),u=l.evaluate({zoom:r},n,{}),g=l.evaluate({zoom:r+1},n,{});o[c]=!0,o[u]=!0,o[g]=!0,n.patterns[A.id]={min:c,mid:u,max:g}}}return n}$o.deviation=function(t,e,n,r){var i=e&&e.length,o=i?e[0]*n:t.length,a=Math.abs(Ea(t,0,o,n));if(i)for(var s=0,A=e.length;s<A;s++){var l=e[s]*n,c=s<A-1?e[s+1]*n:t.length;a-=Math.abs(Ea(t,l,c,n))}var u=0;for(s=0;s<r.length;s+=3){var g=r[s]*n,h=r[s+1]*n,d=r[s+2]*n;u+=Math.abs((t[g]-t[d])*(t[h+1]-t[g+1])-(t[g]-t[h])*(t[d+1]-t[g+1]))}return 0===a&&0===u?0:Math.abs((u-a)/a)},$o.flatten=function(t){for(var e=t[0][0].length,n={vertices:[],holes:[],dimensions:e},r=0,i=0;i<t.length;i++){for(var o=0;o<t[i].length;o++)for(var a=0;a<e;a++)n.vertices.push(t[i][o][a]);i>0&&(r+=t[i-1].length,n.holes.push(r))}return n},Xo.default=Zo;var Da=function(t){this.zoom=t.zoom,this.overscaling=t.overscaling,this.layers=t.layers,this.layerIds=this.layers.map(function(t){return t.id}),this.index=t.index,this.hasPattern=!1,this.layoutVertexArray=new si,this.indexArray=new Bi,this.indexArray2=new bi,this.programConfigurations=new so(Wo,t.layers,t.zoom),this.segments=new Hi,this.segments2=new Hi,this.stateDependentLayerIds=this.layers.filter(function(t){return t.isStateDependent()}).map(function(t){return t.id})};Da.prototype.populate=function(t,e){this.features=[],this.hasPattern=xa("fill",this.layers,e);for(var n=0,r=t;n<r.length;n+=1){var i=r[n],o=i.feature,a=i.index,s=i.sourceLayerIndex;if(this.layers[0]._featureFilter(new Gr(this.zoom),o)){var A=go(o),l={sourceLayerIndex:s,index:a,geometry:A,properties:o.properties,type:o.type,patterns:{}};void 0!==o.id&&(l.id=o.id),this.hasPattern?this.features.push(_a("fill",this.layers,l,this.zoom,e)):this.addFeature(l,A,a,{}),e.featureIndex.insert(o,A,a,s,this.index)}}},Da.prototype.update=function(t,e,n){this.stateDependentLayers.length&&this.programConfigurations.updatePaintArrays(t,e,this.stateDependentLayers,n)},Da.prototype.addFeatures=function(t,e){for(var n=0,r=this.features;n<r.length;n+=1){var i=r[n],o=i.geometry;this.addFeature(i,o,i.index,e)}},Da.prototype.isEmpty=function(){return 0===this.layoutVertexArray.length},Da.prototype.uploadPending=function(){return!this.uploaded||this.programConfigurations.needsUpload},Da.prototype.upload=function(t){this.uploaded||(this.layoutVertexBuffer=t.createVertexBuffer(this.layoutVertexArray,Wo),this.indexBuffer=t.createIndexBuffer(this.indexArray),this.indexBuffer2=t.createIndexBuffer(this.indexArray2)),this.programConfigurations.upload(t),this.uploaded=!0},Da.prototype.destroy=function(){this.layoutVertexBuffer&&(this.layoutVertexBuffer.destroy(),this.indexBuffer.destroy(),this.indexBuffer2.destroy(),this.programConfigurations.destroy(),this.segments.destroy(),this.segments2.destroy())},Da.prototype.addFeature=function(t,e,n,r){for(var i=0,o=Qa(e,500);i<o.length;i+=1){for(var a=o[i],s=0,A=0,l=a;A<l.length;A+=1)s+=l[A].length;for(var c=this.segments.prepareSegment(s,this.layoutVertexArray,this.indexArray),u=c.vertexLength,g=[],h=[],d=0,f=a;d<f.length;d+=1){var p=f[d];if(0!==p.length){p!==a[0]&&h.push(g.length/2);var I=this.segments2.prepareSegment(p.length,this.layoutVertexArray,this.indexArray2),C=I.vertexLength;this.layoutVertexArray.emplaceBack(p[0].x,p[0].y),this.indexArray2.emplaceBack(C+p.length-1,C),g.push(p[0].x),g.push(p[0].y);for(var m=1;m<p.length;m++)this.layoutVertexArray.emplaceBack(p[m].x,p[m].y),this.indexArray2.emplaceBack(C+m-1,C+m),g.push(p[m].x),g.push(p[m].y);I.vertexLength+=p.length,I.primitiveLength+=p.length}}for(var y=Xo(g,h),E=0;E<y.length;E+=3)this.indexArray.emplaceBack(u+y[E],u+y[E+1],u+y[E+2]);c.vertexLength+=s,c.primitiveLength+=y.length/3}this.programConfigurations.populatePaintArrays(this.layoutVertexArray.length,t,n,r)},vr("FillBucket",Da,{omit:["layers","features"]});var Ma={paint:new ti({"fill-antialias":new Vr(Lt.paint_fill["fill-antialias"]),"fill-opacity":new Wr(Lt.paint_fill["fill-opacity"]),"fill-color":new Wr(Lt.paint_fill["fill-color"]),"fill-outline-color":new Wr(Lt.paint_fill["fill-outline-color"]),"fill-translate":new Vr(Lt.paint_fill["fill-translate"]),"fill-translate-anchor":new Vr(Lt.paint_fill["fill-translate-anchor"]),"fill-pattern":new Xr(Lt.paint_fill["fill-pattern"])})},ka=function(t){function e(e){t.call(this,e,Ma)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.recalculate=function(e){t.prototype.recalculate.call(this,e);var n=this.paint._values["fill-outline-color"];"constant"===n.value.kind&&void 0===n.value.value&&(this.paint._values["fill-outline-color"]=this.paint._values["fill-color"])},e.prototype.createBucket=function(t){return new Da(t)},e.prototype.queryRadius=function(){return _o(this.paint.get("fill-translate"))},e.prototype.queryIntersectsFeature=function(t,e,n,r,i,o,a){return Co(Do(t,this.paint.get("fill-translate"),this.paint.get("fill-translate-anchor"),o.angle,a),r)},e.prototype.isTileClipped=function(){return!0},e}(ei),Sa=oi([{name:"a_pos",components:2,type:"Int16"},{name:"a_normal_ed",components:4,type:"Int16"}],4).members,La=Ra;function Ra(t,e,n,r,i){this.properties={},this.extent=n,this.type=0,this._pbf=t,this._geometry=-1,this._keys=r,this._values=i,t.readFields(Na,this,e)}function Na(t,e,n){1==t?e.id=n.readVarint():2==t?function(t,e){for(var n=t.readVarint()+t.pos;t.pos<n;){var r=e._keys[t.readVarint()],i=e._values[t.readVarint()];e.properties[r]=i}}(n,e):3==t?e.type=n.readVarint():4==t&&(e._geometry=n.pos)}function Fa(t){for(var e,n,r=0,i=0,o=t.length,a=o-1;i<o;a=i++)e=t[i],r+=((n=t[a]).x-e.x)*(e.y+n.y);return r}Ra.types=["Unknown","Point","LineString","Polygon"],Ra.prototype.loadGeometry=function(){var t=this._pbf;t.pos=this._geometry;for(var e,n=t.readVarint()+t.pos,r=1,o=0,a=0,s=0,A=[];t.pos<n;){if(o<=0){var l=t.readVarint();r=7&l,o=l>>3}if(o--,1===r||2===r)a+=t.readSVarint(),s+=t.readSVarint(),1===r&&(e&&A.push(e),e=[]),e.push(new i(a,s));else{if(7!==r)throw new Error("unknown command "+r);e&&e.push(e[0].clone())}}return e&&A.push(e),A},Ra.prototype.bbox=function(){var t=this._pbf;t.pos=this._geometry;for(var e=t.readVarint()+t.pos,n=1,r=0,i=0,o=0,a=1/0,s=-1/0,A=1/0,l=-1/0;t.pos<e;){if(r<=0){var c=t.readVarint();n=7&c,r=c>>3}if(r--,1===n||2===n)(i+=t.readSVarint())<a&&(a=i),i>s&&(s=i),(o+=t.readSVarint())<A&&(A=o),o>l&&(l=o);else if(7!==n)throw new Error("unknown command "+n)}return[a,A,s,l]},Ra.prototype.toGeoJSON=function(t,e,n){var r,i,o=this.extent*Math.pow(2,n),a=this.extent*t,s=this.extent*e,A=this.loadGeometry(),l=Ra.types[this.type];function c(t){for(var e=0;e<t.length;e++){var n=t[e],r=180-360*(n.y+s)/o;t[e]=[360*(n.x+a)/o-180,360/Math.PI*Math.atan(Math.exp(r*Math.PI/180))-90]}}switch(this.type){case 1:var u=[];for(r=0;r<A.length;r++)u[r]=A[r][0];c(A=u);break;case 2:for(r=0;r<A.length;r++)c(A[r]);break;case 3:for(A=function(t){var e=t.length;if(e<=1)return[t];for(var n,r,i=[],o=0;o<e;o++){var a=Fa(t[o]);0!==a&&(void 0===r&&(r=a<0),r===a<0?(n&&i.push(n),n=[t[o]]):n.push(t[o]))}return n&&i.push(n),i}(A),r=0;r<A.length;r++)for(i=0;i<A[r].length;i++)c(A[r][i])}1===A.length?A=A[0]:l="Multi"+l;var g={type:"Feature",geometry:{type:l,coordinates:A},properties:this.properties};return"id"in this&&(g.id=this.id),g};var Ta=Ua;function Ua(t,e){this.version=1,this.name=null,this.extent=4096,this.length=0,this._pbf=t,this._keys=[],this._values=[],this._features=[],t.readFields(Oa,this,e),this.length=this._features.length}function Oa(t,e,n){15===t?e.version=n.readVarint():1===t?e.name=n.readString():5===t?e.extent=n.readVarint():2===t?e._features.push(n.pos):3===t?e._keys.push(n.readString()):4===t&&e._values.push(function(t){for(var e=null,n=t.readVarint()+t.pos;t.pos<n;){var r=t.readVarint()>>3;e=1===r?t.readString():2===r?t.readFloat():3===r?t.readDouble():4===r?t.readVarint64():5===r?t.readVarint():6===r?t.readSVarint():7===r?t.readBoolean():null}return e}(n))}function Ga(t,e,n){if(3===t){var r=new Ta(n,n.readVarint()+n.pos);r.length&&(e[r.name]=r)}}Ua.prototype.feature=function(t){if(t<0||t>=this._features.length)throw new Error("feature index out of bounds");this._pbf.pos=this._features[t];var e=this._pbf.readVarint()+this._pbf.pos;return new La(this._pbf,e,this.extent,this._keys,this._values)};var Ha={VectorTile:function(t,e){this.layers=t.readFields(Ga,{},e)},VectorTileFeature:La,VectorTileLayer:Ta},ja=Ha.VectorTileFeature.types,Ya=Math.pow(2,13);function Pa(t,e,n,r,i,o,a,s){t.emplaceBack(e,n,2*Math.floor(r*Ya)+a,i*Ya*2,o*Ya*2,Math.round(s))}var Ka=function(t){this.zoom=t.zoom,this.overscaling=t.overscaling,this.layers=t.layers,this.layerIds=this.layers.map(function(t){return t.id}),this.index=t.index,this.hasPattern=!1,this.layoutVertexArray=new li,this.indexArray=new Bi,this.programConfigurations=new so(Sa,t.layers,t.zoom),this.segments=new Hi,this.stateDependentLayerIds=this.layers.filter(function(t){return t.isStateDependent()}).map(function(t){return t.id})};function za(t,e){return t.x===e.x&&(t.x<0||t.x>co)||t.y===e.y&&(t.y<0||t.y>co)}function Ja(t){return t.every(function(t){return t.x<0})||t.every(function(t){return t.x>co})||t.every(function(t){return t.y<0})||t.every(function(t){return t.y>co})}Ka.prototype.populate=function(t,e){this.features=[],this.hasPattern=xa("fill-extrusion",this.layers,e);for(var n=0,r=t;n<r.length;n+=1){var i=r[n],o=i.feature,a=i.index,s=i.sourceLayerIndex;if(this.layers[0]._featureFilter(new Gr(this.zoom),o)){var A=go(o),l={sourceLayerIndex:s,index:a,geometry:A,properties:o.properties,type:o.type,patterns:{}};void 0!==o.id&&(l.id=o.id),this.hasPattern?this.features.push(_a("fill-extrusion",this.layers,l,this.zoom,e)):this.addFeature(l,A,a,{}),e.featureIndex.insert(o,A,a,s,this.index,!0)}}},Ka.prototype.addFeatures=function(t,e){for(var n=0,r=this.features;n<r.length;n+=1){var i=r[n],o=i.geometry;this.addFeature(i,o,i.index,e)}},Ka.prototype.update=function(t,e,n){this.stateDependentLayers.length&&this.programConfigurations.updatePaintArrays(t,e,this.stateDependentLayers,n)},Ka.prototype.isEmpty=function(){return 0===this.layoutVertexArray.length},Ka.prototype.uploadPending=function(){return!this.uploaded||this.programConfigurations.needsUpload},Ka.prototype.upload=function(t){this.uploaded||(this.layoutVertexBuffer=t.createVertexBuffer(this.layoutVertexArray,Sa),this.indexBuffer=t.createIndexBuffer(this.indexArray)),this.programConfigurations.upload(t),this.uploaded=!0},Ka.prototype.destroy=function(){this.layoutVertexBuffer&&(this.layoutVertexBuffer.destroy(),this.indexBuffer.destroy(),this.programConfigurations.destroy(),this.segments.destroy())},Ka.prototype.addFeature=function(t,e,n,r){for(var i=0,o=Qa(e,500);i<o.length;i+=1){for(var a=o[i],s=0,A=0,l=a;A<l.length;A+=1)s+=l[A].length;for(var c=this.segments.prepareSegment(4,this.layoutVertexArray,this.indexArray),u=0,g=a;u<g.length;u+=1){var h=g[u];if(0!==h.length&&!Ja(h))for(var d=0,f=0;f<h.length;f++){var p=h[f];if(f>=1){var I=h[f-1];if(!za(p,I)){c.vertexLength+4>Hi.MAX_VERTEX_ARRAY_LENGTH&&(c=this.segments.prepareSegment(4,this.layoutVertexArray,this.indexArray));var C=p.sub(I)._perp()._unit(),m=I.dist(p);d+m>32768&&(d=0),Pa(this.layoutVertexArray,p.x,p.y,C.x,C.y,0,0,d),Pa(this.layoutVertexArray,p.x,p.y,C.x,C.y,0,1,d),d+=m,Pa(this.layoutVertexArray,I.x,I.y,C.x,C.y,0,0,d),Pa(this.layoutVertexArray,I.x,I.y,C.x,C.y,0,1,d);var y=c.vertexLength;this.indexArray.emplaceBack(y,y+2,y+1),this.indexArray.emplaceBack(y+1,y+2,y+3),c.vertexLength+=4,c.primitiveLength+=2}}}}if(c.vertexLength+s>Hi.MAX_VERTEX_ARRAY_LENGTH&&(c=this.segments.prepareSegment(s,this.layoutVertexArray,this.indexArray)),"Polygon"===ja[t.type]){for(var E=[],v=[],B=c.vertexLength,b=0,Q=a;b<Q.length;b+=1){var w=Q[b];if(0!==w.length){w!==a[0]&&v.push(E.length/2);for(var x=0;x<w.length;x++){var _=w[x];Pa(this.layoutVertexArray,_.x,_.y,0,0,1,1,0),E.push(_.x),E.push(_.y)}}}for(var D=Xo(E,v),M=0;M<D.length;M+=3)this.indexArray.emplaceBack(B+D[M],B+D[M+2],B+D[M+1]);c.primitiveLength+=D.length/3,c.vertexLength+=s}}this.programConfigurations.populatePaintArrays(this.layoutVertexArray.length,t,n,r)},vr("FillExtrusionBucket",Ka,{omit:["layers","features"]});var qa={paint:new ti({"fill-extrusion-opacity":new Vr(Lt["paint_fill-extrusion"]["fill-extrusion-opacity"]),"fill-extrusion-color":new Wr(Lt["paint_fill-extrusion"]["fill-extrusion-color"]),"fill-extrusion-translate":new Vr(Lt["paint_fill-extrusion"]["fill-extrusion-translate"]),"fill-extrusion-translate-anchor":new Vr(Lt["paint_fill-extrusion"]["fill-extrusion-translate-anchor"]),"fill-extrusion-pattern":new Xr(Lt["paint_fill-extrusion"]["fill-extrusion-pattern"]),"fill-extrusion-height":new Wr(Lt["paint_fill-extrusion"]["fill-extrusion-height"]),"fill-extrusion-base":new Wr(Lt["paint_fill-extrusion"]["fill-extrusion-base"]),"fill-extrusion-vertical-gradient":new Vr(Lt["paint_fill-extrusion"]["fill-extrusion-vertical-gradient"])})},Va=function(t){function e(e){t.call(this,e,qa)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.createBucket=function(t){return new Ka(t)},e.prototype.queryRadius=function(){return _o(this.paint.get("fill-extrusion-translate"))},e.prototype.is3D=function(){return!0},e.prototype.queryIntersectsFeature=function(t,e,n,r,o,a,s,A){var l=Do(t,this.paint.get("fill-extrusion-translate"),this.paint.get("fill-extrusion-translate-anchor"),a.angle,s),c=this.paint.get("fill-extrusion-height").evaluate(e,n),u=this.paint.get("fill-extrusion-base").evaluate(e,n),g=function(t,e,n,r){for(var o=[],a=0,s=t;a<s.length;a+=1){var A=s[a],l=[A.x,A.y,0,1];No(l,l,e),o.push(new i(l[0]/l[3],l[1]/l[3]))}return o}(l,A),h=function(t,e,n,r){for(var o=[],a=[],s=r[8]*e,A=r[9]*e,l=r[10]*e,c=r[11]*e,u=r[8]*n,g=r[9]*n,h=r[10]*n,d=r[11]*n,f=0,p=t;f<p.length;f+=1){for(var I=[],C=[],m=0,y=p[f];m<y.length;m+=1){var E=y[m],v=E.x,B=E.y,b=r[0]*v+r[4]*B+r[12],Q=r[1]*v+r[5]*B+r[13],w=r[2]*v+r[6]*B+r[14],x=r[3]*v+r[7]*B+r[15],_=w+l,D=x+c,M=b+u,k=Q+g,S=w+h,L=x+d,R=new i((b+s)/D,(Q+A)/D);R.z=_/D,I.push(R);var N=new i(M/L,k/L);N.z=S/L,C.push(N)}o.push(I),a.push(C)}return[o,a]}(r,u,c,A);return function(t,e,n){var r=1/0;Co(n,e)&&(r=Xa(n,e[0]));for(var i=0;i<e.length;i++)for(var o=e[i],a=t[i],s=0;s<o.length-1;s++){var A=o[s],l=o[s+1],c=a[s],u=[A,l,a[s+1],c,A];po(n,u)&&(r=Math.min(r,Xa(n,u)))}return r!==1/0&&r}(h[0],h[1],g)},e}(ei);function Wa(t,e){return t.x*e.x+t.y*e.y}function Xa(t,e){if(1===t.length){var n=e[0],r=e[1],i=e[3],o=t[0],a=r.sub(n),s=i.sub(n),A=o.sub(n),l=Wa(a,a),c=Wa(a,s),u=Wa(s,s),g=Wa(A,a),h=Wa(A,s),d=l*u-c*c,f=(u*g-c*h)/d,p=(l*h-c*g)/d,I=1-f-p;return n.z*I+r.z*f+i.z*p}for(var C=1/0,m=0,y=e;m<y.length;m+=1){var E=y[m];C=Math.min(C,E.z)}return C}var Za=oi([{name:"a_pos_normal",components:2,type:"Int16"},{name:"a_data",components:4,type:"Uint8"}],4).members,$a=Ha.VectorTileFeature.types,ts=63,es=Math.cos(Math.PI/180*37.5),ns=.5,rs=Math.pow(2,14)/ns;function is(t,e,n,r,i,o,a){t.emplaceBack((e.x<<1)+(r?1:0),(e.y<<1)+(i?1:0),Math.round(ts*n.x)+128,Math.round(ts*n.y)+128,1+(0===o?0:o<0?-1:1)|(a*ns&63)<<2,a*ns>>6)}var os=function(t){this.zoom=t.zoom,this.overscaling=t.overscaling,this.layers=t.layers,this.layerIds=this.layers.map(function(t){return t.id}),this.index=t.index,this.features=[],this.hasPattern=!1,this.layoutVertexArray=new ci,this.indexArray=new Bi,this.programConfigurations=new so(Za,t.layers,t.zoom),this.segments=new Hi,this.stateDependentLayerIds=this.layers.filter(function(t){return t.isStateDependent()}).map(function(t){return t.id})};function as(t,e){return(t/e.tileTotal*(e.end-e.start)+e.start)*(rs-1)}os.prototype.populate=function(t,e){this.features=[],this.hasPattern=xa("line",this.layers,e);for(var n=0,r=t;n<r.length;n+=1){var i=r[n],o=i.feature,a=i.index,s=i.sourceLayerIndex;if(this.layers[0]._featureFilter(new Gr(this.zoom),o)){var A=go(o),l={sourceLayerIndex:s,index:a,geometry:A,properties:o.properties,type:o.type,patterns:{}};void 0!==o.id&&(l.id=o.id),this.hasPattern?this.features.push(_a("line",this.layers,l,this.zoom,e)):this.addFeature(l,A,a,{}),e.featureIndex.insert(o,A,a,s,this.index)}}},os.prototype.update=function(t,e,n){this.stateDependentLayers.length&&this.programConfigurations.updatePaintArrays(t,e,this.stateDependentLayers,n)},os.prototype.addFeatures=function(t,e){for(var n=0,r=this.features;n<r.length;n+=1){var i=r[n],o=i.geometry;this.addFeature(i,o,i.index,e)}},os.prototype.isEmpty=function(){return 0===this.layoutVertexArray.length},os.prototype.uploadPending=function(){return!this.uploaded||this.programConfigurations.needsUpload},os.prototype.upload=function(t){this.uploaded||(this.layoutVertexBuffer=t.createVertexBuffer(this.layoutVertexArray,Za),this.indexBuffer=t.createIndexBuffer(this.indexArray)),this.programConfigurations.upload(t),this.uploaded=!0},os.prototype.destroy=function(){this.layoutVertexBuffer&&(this.layoutVertexBuffer.destroy(),this.indexBuffer.destroy(),this.programConfigurations.destroy(),this.segments.destroy())},os.prototype.addFeature=function(t,e,n,r){for(var i=this.layers[0].layout,o=i.get("line-join").evaluate(t,{}),a=i.get("line-cap"),s=i.get("line-miter-limit"),A=i.get("line-round-limit"),l=0,c=e;l<c.length;l+=1){var u=c[l];this.addLine(u,t,o,a,s,A,n,r)}},os.prototype.addLine=function(t,e,n,r,i,o,a,s){var A=null;e.properties&&e.properties.hasOwnProperty("mapbox_clip_start")&&e.properties.hasOwnProperty("mapbox_clip_end")&&(A={start:e.properties.mapbox_clip_start,end:e.properties.mapbox_clip_end,tileTotal:void 0});for(var l="Polygon"===$a[e.type],c=t.length;c>=2&&t[c-1].equals(t[c-2]);)c--;for(var u=0;u<c-1&&t[u].equals(t[u+1]);)u++;if(!(c<(l?3:2))){A&&(A.tileTotal=function(t,e,n){for(var r,i,o=0,a=u;a<n-1;a++)r=t[a],i=t[a+1],o+=r.dist(i);return o}(t,0,c)),"bevel"===n&&(i=1.05);var g=co/(512*this.overscaling)*15,h=t[u],d=this.segments.prepareSegment(10*c,this.layoutVertexArray,this.indexArray);this.distance=0;var f,p,I,C=r,m=l?"butt":r,y=!0,E=void 0,v=void 0,B=void 0,b=void 0;this.e1=this.e2=this.e3=-1,l&&(f=t[c-2],b=h.sub(f)._unit()._perp());for(var Q=u;Q<c;Q++)if(!(v=l&&Q===c-1?t[u+1]:t[Q+1])||!t[Q].equals(v)){b&&(B=b),f&&(E=f),f=t[Q],b=v?v.sub(f)._unit()._perp():B;var w=(B=B||b).add(b);0===w.x&&0===w.y||w._unit();var x=w.x*b.x+w.y*b.y,_=0!==x?1/x:1/0,D=x<es&&E&&v;if(D&&Q>u){var M=f.dist(E);if(M>2*g){var k=f.sub(f.sub(E)._mult(g/M)._round());this.distance+=k.dist(E),this.addCurrentVertex(k,this.distance,B.mult(1),0,0,!1,d,A),E=k}}var S=E&&v,L=S?n:v?C:m;if(S&&"round"===L&&(_<o?L="miter":_<=2&&(L="fakeround")),"miter"===L&&_>i&&(L="bevel"),"bevel"===L&&(_>2&&(L="flipbevel"),_<i&&(L="miter")),E&&(this.distance+=f.dist(E)),"miter"===L)w._mult(_),this.addCurrentVertex(f,this.distance,w,0,0,!1,d,A);else if("flipbevel"===L){if(_>100)w=b.clone().mult(-1);else{var R=B.x*b.y-B.y*b.x>0?-1:1,N=_*B.add(b).mag()/B.sub(b).mag();w._perp()._mult(N*R)}this.addCurrentVertex(f,this.distance,w,0,0,!1,d,A),this.addCurrentVertex(f,this.distance,w.mult(-1),0,0,!1,d,A)}else if("bevel"===L||"fakeround"===L){var F=B.x*b.y-B.y*b.x>0,T=-Math.sqrt(_*_-1);if(F?(I=0,p=T):(p=0,I=T),y||this.addCurrentVertex(f,this.distance,B,p,I,!1,d,A),"fakeround"===L){for(var U=Math.floor(8*(.5-(x-.5))),O=void 0,G=0;G<U;G++)O=b.mult((G+1)/(U+1))._add(B)._unit(),this.addPieSliceVertex(f,this.distance,O,F,d,A);this.addPieSliceVertex(f,this.distance,w,F,d,A);for(var H=U-1;H>=0;H--)O=B.mult((H+1)/(U+1))._add(b)._unit(),this.addPieSliceVertex(f,this.distance,O,F,d,A)}v&&this.addCurrentVertex(f,this.distance,b,-p,-I,!1,d,A)}else"butt"===L?(y||this.addCurrentVertex(f,this.distance,B,0,0,!1,d,A),v&&this.addCurrentVertex(f,this.distance,b,0,0,!1,d,A)):"square"===L?(y||(this.addCurrentVertex(f,this.distance,B,1,1,!1,d,A),this.e1=this.e2=-1),v&&this.addCurrentVertex(f,this.distance,b,-1,-1,!1,d,A)):"round"===L&&(y||(this.addCurrentVertex(f,this.distance,B,0,0,!1,d,A),this.addCurrentVertex(f,this.distance,B,1,1,!0,d,A),this.e1=this.e2=-1),v&&(this.addCurrentVertex(f,this.distance,b,-1,-1,!0,d,A),this.addCurrentVertex(f,this.distance,b,0,0,!1,d,A)));if(D&&Q<c-1){var j=f.dist(v);if(j>2*g){var Y=f.add(v.sub(f)._mult(g/j)._round());this.distance+=Y.dist(f),this.addCurrentVertex(Y,this.distance,b.mult(1),0,0,!1,d,A),f=Y}}y=!1}this.programConfigurations.populatePaintArrays(this.layoutVertexArray.length,e,a,s)}},os.prototype.addCurrentVertex=function(t,e,n,r,i,o,a,s){var A,l=this.layoutVertexArray,c=this.indexArray;s&&(e=as(e,s)),A=n.clone(),r&&A._sub(n.perp()._mult(r)),is(l,t,A,o,!1,r,e),this.e3=a.vertexLength++,this.e1>=0&&this.e2>=0&&(c.emplaceBack(this.e1,this.e2,this.e3),a.primitiveLength++),this.e1=this.e2,this.e2=this.e3,A=n.mult(-1),i&&A._sub(n.perp()._mult(i)),is(l,t,A,o,!0,-i,e),this.e3=a.vertexLength++,this.e1>=0&&this.e2>=0&&(c.emplaceBack(this.e1,this.e2,this.e3),a.primitiveLength++),this.e1=this.e2,this.e2=this.e3,e>rs/2&&!s&&(this.distance=0,this.addCurrentVertex(t,this.distance,n,r,i,o,a))},os.prototype.addPieSliceVertex=function(t,e,n,r,i,o){n=n.mult(r?-1:1);var a=this.layoutVertexArray,s=this.indexArray;o&&(e=as(e,o)),is(a,t,n,!1,r,0,e),this.e3=i.vertexLength++,this.e1>=0&&this.e2>=0&&(s.emplaceBack(this.e1,this.e2,this.e3),i.primitiveLength++),r?this.e2=this.e3:this.e1=this.e3},vr("LineBucket",os,{omit:["layers","features"]});var ss=new ti({"line-cap":new Vr(Lt.layout_line["line-cap"]),"line-join":new Wr(Lt.layout_line["line-join"]),"line-miter-limit":new Vr(Lt.layout_line["line-miter-limit"]),"line-round-limit":new Vr(Lt.layout_line["line-round-limit"])}),As={paint:new ti({"line-opacity":new Wr(Lt.paint_line["line-opacity"]),"line-color":new Wr(Lt.paint_line["line-color"]),"line-translate":new Vr(Lt.paint_line["line-translate"]),"line-translate-anchor":new Vr(Lt.paint_line["line-translate-anchor"]),"line-width":new Wr(Lt.paint_line["line-width"]),"line-gap-width":new Wr(Lt.paint_line["line-gap-width"]),"line-offset":new Wr(Lt.paint_line["line-offset"]),"line-blur":new Wr(Lt.paint_line["line-blur"]),"line-dasharray":new Zr(Lt.paint_line["line-dasharray"]),"line-pattern":new Xr(Lt.paint_line["line-pattern"]),"line-gradient":new $r(Lt.paint_line["line-gradient"])}),layout:ss},ls=new(function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.possiblyEvaluate=function(e,n){return n=new Gr(Math.floor(n.zoom),{now:n.now,fadeDuration:n.fadeDuration,zoomHistory:n.zoomHistory,transition:n.transition}),t.prototype.possiblyEvaluate.call(this,e,n)},e.prototype.evaluate=function(e,n,r,i){return n=u({},n,{zoom:Math.floor(n.zoom)}),t.prototype.evaluate.call(this,e,n,r,i)},e}(Wr))(As.paint.properties["line-width"].specification);ls.useIntegerZoom=!0;var cs=function(t){function e(e){t.call(this,e,As)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype._handleSpecialPaintPropertyUpdate=function(t){"line-gradient"===t&&this._updateGradient()},e.prototype._updateGradient=function(){var t=this._transitionablePaint._values["line-gradient"].value.expression;this.gradient=zo(t,"lineProgress"),this.gradientTexture=null},e.prototype.recalculate=function(e){t.prototype.recalculate.call(this,e),this.paint._values["line-floorwidth"]=ls.possiblyEvaluate(this._transitioningPaint._values["line-width"].value,e)},e.prototype.createBucket=function(t){return new os(t)},e.prototype.queryRadius=function(t){var e=t,n=us(xo("line-width",this,e),xo("line-gap-width",this,e)),r=xo("line-offset",this,e);return n/2+Math.abs(r)+_o(this.paint.get("line-translate"))},e.prototype.queryIntersectsFeature=function(t,e,n,r,o,a,s){var A=Do(t,this.paint.get("line-translate"),this.paint.get("line-translate-anchor"),a.angle,s),l=s/2*us(this.paint.get("line-width").evaluate(e,n),this.paint.get("line-gap-width").evaluate(e,n)),c=this.paint.get("line-offset").evaluate(e,n);return c&&(r=function(t,e){for(var n=[],r=new i(0,0),o=0;o<t.length;o++){for(var a=t[o],s=[],A=0;A<a.length;A++){var l=a[A-1],c=a[A],u=a[A+1],g=0===A?r:c.sub(l)._unit()._perp(),h=A===a.length-1?r:u.sub(c)._unit()._perp(),d=g._add(h)._unit(),f=d.x*h.x+d.y*h.y;d._mult(1/f),s.push(d._mult(e)._add(c))}n.push(s)}return n}(r,c*s)),function(t,e,n){for(var r=0;r<e.length;r++){var i=e[r];if(t.length>=3)for(var o=0;o<i.length;o++)if(Qo(t,i[o]))return!0;if(mo(t,i,n))return!0}return!1}(A,r,l)},e.prototype.isTileClipped=function(){return!0},e}(ei);function us(t,e){return e>0?e+2*t:t}var gs=oi([{name:"a_pos_offset",components:4,type:"Int16"},{name:"a_data",components:4,type:"Uint16"}]),hs=oi([{name:"a_projected_pos",components:3,type:"Float32"}],4),ds=(oi([{name:"a_fade_opacity",components:1,type:"Uint32"}],4),oi([{name:"a_placed",components:2,type:"Uint8"},{name:"a_shift",components:2,type:"Float32"}])),fs=(oi([{type:"Int16",name:"anchorPointX"},{type:"Int16",name:"anchorPointY"},{type:"Int16",name:"x1"},{type:"Int16",name:"y1"},{type:"Int16",name:"x2"},{type:"Int16",name:"y2"},{type:"Uint32",name:"featureIndex"},{type:"Uint16",name:"sourceLayerIndex"},{type:"Uint16",name:"bucketIndex"},{type:"Int16",name:"radius"},{type:"Int16",name:"signedDistanceFromAnchor"}]),oi([{name:"a_pos",components:2,type:"Int16"},{name:"a_anchor_pos",components:2,type:"Int16"},{name:"a_extrude",components:2,type:"Int16"}],4)),ps=oi([{name:"a_pos",components:2,type:"Int16"},{name:"a_anchor_pos",components:2,type:"Int16"},{name:"a_extrude",components:2,type:"Int16"}],4);function Is(t,e,n){return t.sections.forEach(function(t){t.text=function(t,e,n){var r=e.layout.get("text-transform").evaluate(n,{});return"uppercase"===r?t=t.toLocaleUpperCase():"lowercase"===r&&(t=t.toLocaleLowerCase()),Or.applyArabicShaping&&(t=Or.applyArabicShaping(t)),t}(t.text,e,n)}),t}oi([{type:"Int16",name:"anchorX"},{type:"Int16",name:"anchorY"},{type:"Uint16",name:"glyphStartIndex"},{type:"Uint16",name:"numGlyphs"},{type:"Uint32",name:"vertexStartIndex"},{type:"Uint32",name:"lineStartIndex"},{type:"Uint32",name:"lineLength"},{type:"Uint16",name:"segment"},{type:"Uint16",name:"lowerSize"},{type:"Uint16",name:"upperSize"},{type:"Float32",name:"lineOffsetX"},{type:"Float32",name:"lineOffsetY"},{type:"Uint8",name:"writingMode"},{type:"Uint8",name:"hidden"},{type:"Uint32",name:"crossTileID"}]),oi([{type:"Int16",name:"anchorX"},{type:"Int16",name:"anchorY"},{type:"Int16",name:"rightJustifiedTextSymbolIndex"},{type:"Int16",name:"centerJustifiedTextSymbolIndex"},{type:"Int16",name:"leftJustifiedTextSymbolIndex"},{type:"Int16",name:"verticalPlacedTextSymbolIndex"},{type:"Uint16",name:"key"},{type:"Uint16",name:"textBoxStartIndex"},{type:"Uint16",name:"textBoxEndIndex"},{type:"Uint16",name:"iconBoxStartIndex"},{type:"Uint16",name:"iconBoxEndIndex"},{type:"Uint16",name:"featureIndex"},{type:"Uint16",name:"numHorizontalGlyphVertices"},{type:"Uint16",name:"numVerticalGlyphVertices"},{type:"Uint16",name:"numIconVertices"},{type:"Uint32",name:"crossTileID"},{type:"Float32",name:"textBoxScale"},{type:"Float32",name:"radialTextOffset"}]),oi([{type:"Float32",name:"offsetX"}]),oi([{type:"Int16",name:"x"},{type:"Int16",name:"y"},{type:"Int16",name:"tileUnitDistanceFromAnchor"}]);var Cs={"!":"︕","#":"＃",$:"＄","%":"％","&":"＆","(":"︵",")":"︶","*":"＊","+":"＋",",":"︐","-":"︲",".":"・","/":"／",":":"︓",";":"︔","<":"︿","=":"＝",">":"﹀","?":"︖","@":"＠","[":"﹇","\\":"＼","]":"﹈","^":"＾",_:"︳","`":"｀","{":"︷","|":"―","}":"︸","~":"～","¢":"￠","£":"￡","¥":"￥","¦":"￤","¬":"￢","¯":"￣","–":"︲","—":"︱","‘":"﹃","’":"﹄","“":"﹁","”":"﹂","…":"︙","‧":"・","₩":"￦","、":"︑","。":"︒","〈":"︿","〉":"﹀","《":"︽","》":"︾","「":"﹁","」":"﹂","『":"﹃","』":"﹄","【":"︻","】":"︼","〔":"︹","〕":"︺","〖":"︗","〗":"︘","！":"︕","（":"︵","）":"︶","，":"︐","－":"︲","．":"・","：":"︓","；":"︔","＜":"︿","＞":"﹀","？":"︖","［":"﹇","］":"﹈","＿":"︳","｛":"︷","｜":"―","｝":"︸","｟":"︵","｠":"︶","｡":"︒","｢":"﹁","｣":"﹂"},ms=function(t){function e(e,n,r,i){t.call(this,e,n),this.angle=r,void 0!==i&&(this.segment=i)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.clone=function(){return new e(this.x,this.y,this.angle,this.segment)},e}(i);vr("Anchor",ms);var ys=256;function Es(t,e){var n=e.expression;if("constant"===n.kind)return{kind:"constant",layoutSize:n.evaluate(new Gr(t+1))};if("source"===n.kind)return{kind:"source"};for(var r=n.zoomStops,i=n.interpolationType,o=0;o<r.length&&r[o]<=t;)o++;for(var a=o=Math.max(0,o-1);a<r.length&&r[a]<t+1;)a++;a=Math.min(r.length-1,a);var s=r[o],A=r[a];return"composite"===n.kind?{kind:"composite",minZoom:s,maxZoom:A,interpolationType:i}:{kind:"camera",minZoom:s,maxZoom:A,minSize:n.evaluate(new Gr(s)),maxSize:n.evaluate(new Gr(A)),interpolationType:i}}function vs(t,e,n){var r=e.uSize,i=e.uSizeT,o=n.lowerSize,a=n.upperSize;return"source"===t.kind?o/ys:"composite"===t.kind?xe(o/ys,a/ys,i):r}function Bs(t,e){var n=0,r=0;if("constant"===t.kind)r=t.layoutSize;else if("source"!==t.kind){var i=t.interpolationType,o=t.minZoom,a=t.maxZoom,s=i?l(Je.interpolationFactor(i,e,o,a),0,1):0;"camera"===t.kind?r=xe(t.minSize,t.maxSize,s):n=s}return{uSizeT:n,uSize:r}}var bs=Object.freeze({getSizeData:Es,evaluateSizeForFeature:vs,evaluateSizeForZoom:Bs,SIZE_PACK_FACTOR:ys}),Qs=Ha.VectorTileFeature.types,ws=[{name:"a_fade_opacity",components:1,type:"Uint8",offset:0}];function xs(t,e,n,r,i,o,a,s){t.emplaceBack(e,n,Math.round(32*r),Math.round(32*i),o,a,s?s[0]:0,s?s[1]:0)}function _s(t,e,n){t.emplaceBack(e.x,e.y,n),t.emplaceBack(e.x,e.y,n),t.emplaceBack(e.x,e.y,n),t.emplaceBack(e.x,e.y,n)}var Ds=function(t){this.layoutVertexArray=new gi,this.indexArray=new Bi,this.programConfigurations=t,this.segments=new Hi,this.dynamicLayoutVertexArray=new hi,this.opacityVertexArray=new di,this.placedSymbolArray=new ki};Ds.prototype.upload=function(t,e,n,r){n&&(this.layoutVertexBuffer=t.createVertexBuffer(this.layoutVertexArray,gs.members),this.indexBuffer=t.createIndexBuffer(this.indexArray,e),this.dynamicLayoutVertexBuffer=t.createVertexBuffer(this.dynamicLayoutVertexArray,hs.members,!0),this.opacityVertexBuffer=t.createVertexBuffer(this.opacityVertexArray,ws,!0),this.opacityVertexBuffer.itemSize=1),(n||r)&&this.programConfigurations.upload(t)},Ds.prototype.destroy=function(){this.layoutVertexBuffer&&(this.layoutVertexBuffer.destroy(),this.indexBuffer.destroy(),this.programConfigurations.destroy(),this.segments.destroy(),this.dynamicLayoutVertexBuffer.destroy(),this.opacityVertexBuffer.destroy())},vr("SymbolBuffers",Ds);var Ms=function(t,e,n){this.layoutVertexArray=new t,this.layoutAttributes=e,this.indexArray=new n,this.segments=new Hi,this.collisionVertexArray=new Ii};Ms.prototype.upload=function(t){this.layoutVertexBuffer=t.createVertexBuffer(this.layoutVertexArray,this.layoutAttributes),this.indexBuffer=t.createIndexBuffer(this.indexArray),this.collisionVertexBuffer=t.createVertexBuffer(this.collisionVertexArray,ds.members,!0)},Ms.prototype.destroy=function(){this.layoutVertexBuffer&&(this.layoutVertexBuffer.destroy(),this.indexBuffer.destroy(),this.segments.destroy(),this.collisionVertexBuffer.destroy())},vr("CollisionBuffers",Ms);var ks=function(t){this.collisionBoxArray=t.collisionBoxArray,this.zoom=t.zoom,this.overscaling=t.overscaling,this.layers=t.layers,this.layerIds=this.layers.map(function(t){return t.id}),this.index=t.index,this.pixelRatio=t.pixelRatio,this.sourceLayerIndex=t.sourceLayerIndex,this.hasPattern=!1;var e=this.layers[0]._unevaluatedLayout._values;this.textSizeData=Es(this.zoom,e["text-size"]),this.iconSizeData=Es(this.zoom,e["icon-size"]);var n=this.layers[0].layout,r=n.get("symbol-sort-key"),i=n.get("symbol-z-order");this.sortFeaturesByKey="viewport-y"!==i&&void 0!==r.constantOr(1);var o="viewport-y"===i||"auto"===i&&!this.sortFeaturesByKey;this.sortFeaturesByY=o&&(n.get("text-allow-overlap")||n.get("icon-allow-overlap")||n.get("text-ignore-placement")||n.get("icon-ignore-placement")),this.stateDependentLayerIds=this.layers.filter(function(t){return t.isStateDependent()}).map(function(t){return t.id}),this.sourceID=t.sourceID};ks.prototype.createArrays=function(){this.text=new Ds(new so(gs.members,this.layers,this.zoom,function(t){return/^text/.test(t)})),this.icon=new Ds(new so(gs.members,this.layers,this.zoom,function(t){return/^icon/.test(t)})),this.collisionBox=new Ms(pi,fs.members,bi),this.collisionCircle=new Ms(pi,ps.members,Bi),this.glyphOffsetArray=new Ni,this.lineVertexArray=new Ti,this.symbolInstances=new Li},ks.prototype.calculateGlyphDependencies=function(t,e,n,r){for(var i=0;i<t.length;i++)if(e[t.charCodeAt(i)]=!0,n&&r){var o=Cs[t.charAt(i)];o&&(e[o.charCodeAt(0)]=!0)}},ks.prototype.populate=function(t,e){var n=this.layers[0],r=n.layout,i=r.get("text-font"),o=r.get("text-field"),a=r.get("icon-image"),s=("constant"!==o.value.kind||o.value.value.toString().length>0)&&("constant"!==i.value.kind||i.value.value.length>0),A="constant"!==a.value.kind||a.value.value&&a.value.value.length>0,l=r.get("symbol-sort-key");if(this.features=[],s||A){for(var c=e.iconDependencies,u=e.glyphDependencies,g=new Gr(this.zoom),h=0,d=t;h<d.length;h+=1){var f=d[h],p=f.feature,I=f.index,C=f.sourceLayerIndex;if(n._featureFilter(g,p)){var m=void 0;if(s){var y=n.getValueAndResolveTokens("text-field",p);m=Is(y instanceof oe?y:oe.fromString(y),n,p)}var E=void 0;if(A&&(E=n.getValueAndResolveTokens("icon-image",p)),m||E){var v=this.sortFeaturesByKey?l.evaluate(p,{}):void 0,B={text:m,icon:E,index:I,sourceLayerIndex:C,geometry:go(p),properties:p.properties,type:Qs[p.type],sortKey:v};if(void 0!==p.id&&(B.id=p.id),this.features.push(B),E&&(c[E]=!0),m)for(var b=i.evaluate(p,{}).join(","),Q="map"===r.get("text-rotation-alignment")&&"point"!==r.get("symbol-placement"),w=0,x=m.sections;w<x.length;w+=1){var _=x[w],D=_r(m.toString()),M=_.fontStack||b,k=u[M]=u[M]||{};this.calculateGlyphDependencies(_.text,k,Q,D)}}}}"line"===r.get("symbol-placement")&&(this.features=function(t){var e={},n={},r=[],i=0;function o(e){r.push(t[e]),i++}function a(t,e,i){var o=n[t];return delete n[t],n[e]=o,r[o].geometry[0].pop(),r[o].geometry[0]=r[o].geometry[0].concat(i[0]),o}function s(t,n,i){var o=e[n];return delete e[n],e[t]=o,r[o].geometry[0].shift(),r[o].geometry[0]=i[0].concat(r[o].geometry[0]),o}function A(t,e,n){var r=n?e[0][e[0].length-1]:e[0][0];return t+":"+r.x+":"+r.y}for(var l=0;l<t.length;l++){var c=t[l],u=c.geometry,g=c.text?c.text.toString():null;if(g){var h=A(g,u),d=A(g,u,!0);if(h in n&&d in e&&n[h]!==e[d]){var f=s(h,d,u),p=a(h,d,r[f].geometry);delete e[h],delete n[d],n[A(g,r[p].geometry,!0)]=p,r[f].geometry=null}else h in n?a(h,d,u):d in e?s(h,d,u):(o(l),e[h]=i-1,n[d]=i-1)}else o(l)}return r.filter(function(t){return t.geometry})}(this.features)),this.sortFeaturesByKey&&this.features.sort(function(t,e){return t.sortKey-e.sortKey})}},ks.prototype.update=function(t,e,n){this.stateDependentLayers.length&&(this.text.programConfigurations.updatePaintArrays(t,e,this.layers,n),this.icon.programConfigurations.updatePaintArrays(t,e,this.layers,n))},ks.prototype.isEmpty=function(){return 0===this.symbolInstances.length},ks.prototype.uploadPending=function(){return!this.uploaded||this.text.programConfigurations.needsUpload||this.icon.programConfigurations.needsUpload},ks.prototype.upload=function(t){this.uploaded||(this.collisionBox.upload(t),this.collisionCircle.upload(t)),this.text.upload(t,this.sortFeaturesByY,!this.uploaded,this.text.programConfigurations.needsUpload),this.icon.upload(t,this.sortFeaturesByY,!this.uploaded,this.icon.programConfigurations.needsUpload),this.uploaded=!0},ks.prototype.destroy=function(){this.text.destroy(),this.icon.destroy(),this.collisionBox.destroy(),this.collisionCircle.destroy()},ks.prototype.addToLineVertexArray=function(t,e){var n=this.lineVertexArray.length;if(void 0!==t.segment){for(var r=t.dist(e[t.segment+1]),i=t.dist(e[t.segment]),o={},a=t.segment+1;a<e.length;a++)o[a]={x:e[a].x,y:e[a].y,tileUnitDistanceFromAnchor:r},a<e.length-1&&(r+=e[a+1].dist(e[a]));for(var s=t.segment||0;s>=0;s--)o[s]={x:e[s].x,y:e[s].y,tileUnitDistanceFromAnchor:i},s>0&&(i+=e[s-1].dist(e[s]));for(var A=0;A<e.length;A++){var l=o[A];this.lineVertexArray.emplaceBack(l.x,l.y,l.tileUnitDistanceFromAnchor)}}return{lineStartIndex:n,lineLength:this.lineVertexArray.length-n}},ks.prototype.addSymbols=function(t,e,n,r,i,o,a,s,A,l){for(var c=t.indexArray,u=t.layoutVertexArray,g=t.dynamicLayoutVertexArray,h=t.segments.prepareSegment(4*e.length,t.layoutVertexArray,t.indexArray,o.sortKey),d=this.glyphOffsetArray.length,f=h.vertexLength,p=0,I=e;p<I.length;p+=1){var C=I[p],m=C.tl,y=C.tr,E=C.bl,v=C.br,B=C.tex,b=h.vertexLength,Q=C.glyphOffset[1];xs(u,s.x,s.y,m.x,Q+m.y,B.x,B.y,n),xs(u,s.x,s.y,y.x,Q+y.y,B.x+B.w,B.y,n),xs(u,s.x,s.y,E.x,Q+E.y,B.x,B.y+B.h,n),xs(u,s.x,s.y,v.x,Q+v.y,B.x+B.w,B.y+B.h,n),_s(g,s,0),c.emplaceBack(b,b+1,b+2),c.emplaceBack(b+1,b+2,b+3),h.vertexLength+=4,h.primitiveLength+=2,this.glyphOffsetArray.emplaceBack(C.glyphOffset[0])}t.placedSymbolArray.emplaceBack(s.x,s.y,d,this.glyphOffsetArray.length-d,f,A,l,s.segment,n?n[0]:0,n?n[1]:0,r[0],r[1],a,!1,0),t.programConfigurations.populatePaintArrays(t.layoutVertexArray.length,o,o.index,{})},ks.prototype._addCollisionDebugVertex=function(t,e,n,r,i,o){return e.emplaceBack(0,0),t.emplaceBack(n.x,n.y,r,i,Math.round(o.x),Math.round(o.y))},ks.prototype.addCollisionDebugVertices=function(t,e,n,r,o,a,s,A){var l=o.segments.prepareSegment(4,o.layoutVertexArray,o.indexArray),c=l.vertexLength,u=o.layoutVertexArray,g=o.collisionVertexArray,h=s.anchorX,d=s.anchorY;if(this._addCollisionDebugVertex(u,g,a,h,d,new i(t,e)),this._addCollisionDebugVertex(u,g,a,h,d,new i(n,e)),this._addCollisionDebugVertex(u,g,a,h,d,new i(n,r)),this._addCollisionDebugVertex(u,g,a,h,d,new i(t,r)),l.vertexLength+=4,A){var f=o.indexArray;f.emplaceBack(c,c+1,c+2),f.emplaceBack(c,c+2,c+3),l.primitiveLength+=2}else{var p=o.indexArray;p.emplaceBack(c,c+1),p.emplaceBack(c+1,c+2),p.emplaceBack(c+2,c+3),p.emplaceBack(c+3,c),l.primitiveLength+=4}},ks.prototype.addDebugCollisionBoxes=function(t,e,n){for(var r=t;r<e;r++){var i=this.collisionBoxArray.get(r),o=i.x1,a=i.y1,s=i.x2,A=i.y2,l=i.radius>0;this.addCollisionDebugVertices(o,a,s,A,l?this.collisionCircle:this.collisionBox,i.anchorPoint,n,l)}},ks.prototype.generateCollisionDebugBuffers=function(){for(var t=0;t<this.symbolInstances.length;t++){var e=this.symbolInstances.get(t);this.addDebugCollisionBoxes(e.textBoxStartIndex,e.textBoxEndIndex,e),this.addDebugCollisionBoxes(e.iconBoxStartIndex,e.iconBoxEndIndex,e)}},ks.prototype._deserializeCollisionBoxesForSymbol=function(t,e,n,r,i){for(var o={},a=e;a<n;a++){var s=t.get(a);if(0===s.radius){o.textBox={x1:s.x1,y1:s.y1,x2:s.x2,y2:s.y2,anchorPointX:s.anchorPointX,anchorPointY:s.anchorPointY},o.textFeatureIndex=s.featureIndex;break}o.textCircles||(o.textCircles=[],o.textFeatureIndex=s.featureIndex),o.textCircles.push(s.anchorPointX,s.anchorPointY,s.radius,s.signedDistanceFromAnchor,1)}for(var A=r;A<i;A++){var l=t.get(A);if(0===l.radius){o.iconBox={x1:l.x1,y1:l.y1,x2:l.x2,y2:l.y2,anchorPointX:l.anchorPointX,anchorPointY:l.anchorPointY},o.iconFeatureIndex=l.featureIndex;break}}return o},ks.prototype.deserializeCollisionBoxes=function(t){this.collisionArrays=[];for(var e=0;e<this.symbolInstances.length;e++){var n=this.symbolInstances.get(e);this.collisionArrays.push(this._deserializeCollisionBoxesForSymbol(t,n.textBoxStartIndex,n.textBoxEndIndex,n.iconBoxStartIndex,n.iconBoxEndIndex))}},ks.prototype.hasTextData=function(){return this.text.segments.get().length>0},ks.prototype.hasIconData=function(){return this.icon.segments.get().length>0},ks.prototype.hasCollisionBoxData=function(){return this.collisionBox.segments.get().length>0},ks.prototype.hasCollisionCircleData=function(){return this.collisionCircle.segments.get().length>0},ks.prototype.addIndicesForPlacedTextSymbol=function(t){for(var e=this.text.placedSymbolArray.get(t),n=e.vertexStartIndex+4*e.numGlyphs,r=e.vertexStartIndex;r<n;r+=4)this.text.indexArray.emplaceBack(r,r+1,r+2),this.text.indexArray.emplaceBack(r+1,r+2,r+3)},ks.prototype.getSortedSymbolIndexes=function(t){if(this.sortedAngle===t&&void 0!==this.symbolInstanceIndexes)return this.symbolInstanceIndexes;for(var e=Math.sin(t),n=Math.cos(t),r=[],i=[],o=[],a=0;a<this.symbolInstances.length;++a){o.push(a);var s=this.symbolInstances.get(a);r.push(0|Math.round(e*s.anchorX+n*s.anchorY)),i.push(s.featureIndex)}return o.sort(function(t,e){return r[t]-r[e]||i[e]-i[t]}),o},ks.prototype.sortFeatures=function(t){var e=this;if(this.sortFeaturesByY&&this.sortedAngle!==t&&!(this.text.segments.get().length>1||this.icon.segments.get().length>1)){this.symbolInstanceIndexes=this.getSortedSymbolIndexes(t),this.sortedAngle=t,this.text.indexArray.clear(),this.icon.indexArray.clear(),this.featureSortOrder=[];for(var n=0,r=this.symbolInstanceIndexes;n<r.length;n+=1){var i=r[n],o=this.symbolInstances.get(i);this.featureSortOrder.push(o.featureIndex),[o.rightJustifiedTextSymbolIndex,o.centerJustifiedTextSymbolIndex,o.leftJustifiedTextSymbolIndex].forEach(function(t,n,r){t>=0&&r.indexOf(t)===n&&e.addIndicesForPlacedTextSymbol(t)}),o.verticalPlacedTextSymbolIndex>=0&&this.addIndicesForPlacedTextSymbol(o.verticalPlacedTextSymbolIndex);var a=this.icon.placedSymbolArray.get(i);if(a.numGlyphs){var s=a.vertexStartIndex;this.icon.indexArray.emplaceBack(s,s+1,s+2),this.icon.indexArray.emplaceBack(s+1,s+2,s+3)}}this.text.indexBuffer&&this.text.indexBuffer.updateData(this.text.indexArray),this.icon.indexBuffer&&this.icon.indexBuffer.updateData(this.icon.indexArray)}},vr("SymbolBucket",ks,{omit:["layers","collisionBoxArray","features","compareText"]}),ks.MAX_GLYPHS=65535,ks.addDynamicAttributes=_s;var Ss=new ti({"symbol-placement":new Vr(Lt.layout_symbol["symbol-placement"]),"symbol-spacing":new Vr(Lt.layout_symbol["symbol-spacing"]),"symbol-avoid-edges":new Vr(Lt.layout_symbol["symbol-avoid-edges"]),"symbol-sort-key":new Wr(Lt.layout_symbol["symbol-sort-key"]),"symbol-z-order":new Vr(Lt.layout_symbol["symbol-z-order"]),"icon-allow-overlap":new Vr(Lt.layout_symbol["icon-allow-overlap"]),"icon-ignore-placement":new Vr(Lt.layout_symbol["icon-ignore-placement"]),"icon-optional":new Vr(Lt.layout_symbol["icon-optional"]),"icon-rotation-alignment":new Vr(Lt.layout_symbol["icon-rotation-alignment"]),"icon-size":new Wr(Lt.layout_symbol["icon-size"]),"icon-text-fit":new Vr(Lt.layout_symbol["icon-text-fit"]),"icon-text-fit-padding":new Vr(Lt.layout_symbol["icon-text-fit-padding"]),"icon-image":new Wr(Lt.layout_symbol["icon-image"]),"icon-rotate":new Wr(Lt.layout_symbol["icon-rotate"]),"icon-padding":new Vr(Lt.layout_symbol["icon-padding"]),"icon-keep-upright":new Vr(Lt.layout_symbol["icon-keep-upright"]),"icon-offset":new Wr(Lt.layout_symbol["icon-offset"]),"icon-anchor":new Wr(Lt.layout_symbol["icon-anchor"]),"icon-pitch-alignment":new Vr(Lt.layout_symbol["icon-pitch-alignment"]),"text-pitch-alignment":new Vr(Lt.layout_symbol["text-pitch-alignment"]),"text-rotation-alignment":new Vr(Lt.layout_symbol["text-rotation-alignment"]),"text-field":new Wr(Lt.layout_symbol["text-field"]),"text-font":new Wr(Lt.layout_symbol["text-font"]),"text-size":new Wr(Lt.layout_symbol["text-size"]),"text-max-width":new Wr(Lt.layout_symbol["text-max-width"]),"text-line-height":new Vr(Lt.layout_symbol["text-line-height"]),"text-letter-spacing":new Wr(Lt.layout_symbol["text-letter-spacing"]),"text-justify":new Wr(Lt.layout_symbol["text-justify"]),"text-radial-offset":new Wr(Lt.layout_symbol["text-radial-offset"]),"text-variable-anchor":new Vr(Lt.layout_symbol["text-variable-anchor"]),"text-anchor":new Wr(Lt.layout_symbol["text-anchor"]),"text-max-angle":new Vr(Lt.layout_symbol["text-max-angle"]),"text-rotate":new Wr(Lt.layout_symbol["text-rotate"]),"text-padding":new Vr(Lt.layout_symbol["text-padding"]),"text-keep-upright":new Vr(Lt.layout_symbol["text-keep-upright"]),"text-transform":new Wr(Lt.layout_symbol["text-transform"]),"text-offset":new Wr(Lt.layout_symbol["text-offset"]),"text-allow-overlap":new Vr(Lt.layout_symbol["text-allow-overlap"]),"text-ignore-placement":new Vr(Lt.layout_symbol["text-ignore-placement"]),"text-optional":new Vr(Lt.layout_symbol["text-optional"])}),Ls={paint:new ti({"icon-opacity":new Wr(Lt.paint_symbol["icon-opacity"]),"icon-color":new Wr(Lt.paint_symbol["icon-color"]),"icon-halo-color":new Wr(Lt.paint_symbol["icon-halo-color"]),"icon-halo-width":new Wr(Lt.paint_symbol["icon-halo-width"]),"icon-halo-blur":new Wr(Lt.paint_symbol["icon-halo-blur"]),"icon-translate":new Vr(Lt.paint_symbol["icon-translate"]),"icon-translate-anchor":new Vr(Lt.paint_symbol["icon-translate-anchor"]),"text-opacity":new Wr(Lt.paint_symbol["text-opacity"]),"text-color":new Wr(Lt.paint_symbol["text-color"]),"text-halo-color":new Wr(Lt.paint_symbol["text-halo-color"]),"text-halo-width":new Wr(Lt.paint_symbol["text-halo-width"]),"text-halo-blur":new Wr(Lt.paint_symbol["text-halo-blur"]),"text-translate":new Vr(Lt.paint_symbol["text-translate"]),"text-translate-anchor":new Vr(Lt.paint_symbol["text-translate-anchor"])}),layout:Ss},Rs=function(t){function e(e){t.call(this,e,Ls)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.recalculate=function(e){t.prototype.recalculate.call(this,e),"auto"===this.layout.get("icon-rotation-alignment")&&("point"!==this.layout.get("symbol-placement")?this.layout._values["icon-rotation-alignment"]="map":this.layout._values["icon-rotation-alignment"]="viewport"),"auto"===this.layout.get("text-rotation-alignment")&&("point"!==this.layout.get("symbol-placement")?this.layout._values["text-rotation-alignment"]="map":this.layout._values["text-rotation-alignment"]="viewport"),"auto"===this.layout.get("text-pitch-alignment")&&(this.layout._values["text-pitch-alignment"]=this.layout.get("text-rotation-alignment")),"auto"===this.layout.get("icon-pitch-alignment")&&(this.layout._values["icon-pitch-alignment"]=this.layout.get("icon-rotation-alignment"))},e.prototype.getValueAndResolveTokens=function(t,e){var n,r=this.layout.get(t).evaluate(e,{}),i=this._unevaluatedLayout._values[t];return i.isDataDriven()||kn(i.value)?r:(n=e.properties,r.replace(/{([^{}]+)}/g,function(t,e){return e in n?String(n[e]):""}))},e.prototype.createBucket=function(t){return new ks(t)},e.prototype.queryRadius=function(){return 0},e.prototype.queryIntersectsFeature=function(){return!1},e}(ei),Ns={paint:new ti({"background-color":new Vr(Lt.paint_background["background-color"]),"background-pattern":new Zr(Lt.paint_background["background-pattern"]),"background-opacity":new Vr(Lt.paint_background["background-opacity"])})},Fs=function(t){function e(e){t.call(this,e,Ns)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e}(ei),Ts={paint:new ti({"raster-opacity":new Vr(Lt.paint_raster["raster-opacity"]),"raster-hue-rotate":new Vr(Lt.paint_raster["raster-hue-rotate"]),"raster-brightness-min":new Vr(Lt.paint_raster["raster-brightness-min"]),"raster-brightness-max":new Vr(Lt.paint_raster["raster-brightness-max"]),"raster-saturation":new Vr(Lt.paint_raster["raster-saturation"]),"raster-contrast":new Vr(Lt.paint_raster["raster-contrast"]),"raster-resampling":new Vr(Lt.paint_raster["raster-resampling"]),"raster-fade-duration":new Vr(Lt.paint_raster["raster-fade-duration"])})},Us=function(t){function e(e){t.call(this,e,Ts)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e}(ei),Os=function(t){function e(e){t.call(this,e,{}),this.implementation=e}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.is3D=function(){return"3d"===this.implementation.renderingMode},e.prototype.hasOffscreenPass=function(){return void 0!==this.implementation.prerender},e.prototype.recalculate=function(){},e.prototype.updateTransitions=function(){},e.prototype.hasTransition=function(){},e.prototype.serialize=function(){},e.prototype.onAdd=function(t){this.implementation.onAdd&&this.implementation.onAdd(t,t.painter.context.gl)},e.prototype.onRemove=function(t){this.implementation.onRemove&&this.implementation.onRemove(t,t.painter.context.gl)},e}(ei),Gs={circle:To,heatmap:Jo,hillshade:Vo,fill:ka,"fill-extrusion":Va,line:cs,symbol:Rs,background:Fs,raster:Us};function Hs(t){for(var e=0,n=0,r=0,i=t;r<i.length;r+=1){var o=i[r];e+=o.w*o.h,n=Math.max(n,o.w)}t.sort(function(t,e){return e.h-t.h});for(var a=[{x:0,y:0,w:Math.max(Math.ceil(Math.sqrt(e/.95)),n),h:1/0}],s=0,A=0,l=0,c=t;l<c.length;l+=1)for(var u=c[l],g=a.length-1;g>=0;g--){var h=a[g];if(!(u.w>h.w||u.h>h.h)){if(u.x=h.x,u.y=h.y,A=Math.max(A,u.y+u.h),s=Math.max(s,u.x+u.w),u.w===h.w&&u.h===h.h){var d=a.pop();g<a.length&&(a[g]=d)}else u.h===h.h?(h.x+=u.w,h.w-=u.w):u.w===h.w?(h.y+=u.h,h.h-=u.h):(a.push({x:h.x+u.w,y:h.y,w:h.w-u.w,h:u.h}),h.y+=u.h,h.h-=u.h);break}}return{w:s,h:A,fill:e/(s*A)||0}}var js=function(t,e){var n=e.pixelRatio,r=e.version;this.paddedRect=t,this.pixelRatio=n,this.version=r},Ys={tl:{configurable:!0},br:{configurable:!0},tlbr:{configurable:!0},displaySize:{configurable:!0}};Ys.tl.get=function(){return[this.paddedRect.x+1,this.paddedRect.y+1]},Ys.br.get=function(){return[this.paddedRect.x+this.paddedRect.w-1,this.paddedRect.y+this.paddedRect.h-1]},Ys.tlbr.get=function(){return this.tl.concat(this.br)},Ys.displaySize.get=function(){return[(this.paddedRect.w-2)/this.pixelRatio,(this.paddedRect.h-2)/this.pixelRatio]},Object.defineProperties(js.prototype,Ys);var Ps=function(t,e){var n={},r={};this.haveRenderCallbacks=[];var i=[];this.addImages(t,n,i),this.addImages(e,r,i);var o=Hs(i),a=o.w,s=o.h,A=new Po({width:a||1,height:s||1});for(var l in t){var c=t[l],u=n[l].paddedRect;Po.copy(c.data,A,{x:0,y:0},{x:u.x+1,y:u.y+1},c.data)}for(var g in e){var h=e[g],d=r[g].paddedRect,f=d.x+1,p=d.y+1,I=h.data.width,C=h.data.height;Po.copy(h.data,A,{x:0,y:0},{x:f,y:p},h.data),Po.copy(h.data,A,{x:0,y:C-1},{x:f,y:p-1},{width:I,height:1}),Po.copy(h.data,A,{x:0,y:0},{x:f,y:p+C},{width:I,height:1}),Po.copy(h.data,A,{x:I-1,y:0},{x:f-1,y:p},{width:1,height:C}),Po.copy(h.data,A,{x:0,y:0},{x:f+I,y:p},{width:1,height:C})}this.image=A,this.iconPositions=n,this.patternPositions=r};Ps.prototype.addImages=function(t,e,n){for(var r in t){var i=t[r],o={x:0,y:0,w:i.data.width+2,h:i.data.height+2};n.push(o),e[r]=new js(o,i),i.hasRenderCallback&&this.haveRenderCallbacks.push(r)}},Ps.prototype.patchUpdatedImages=function(t,e){for(var n in t.dispatchRenderCallbacks(this.haveRenderCallbacks),t.updatedImages)this.patchUpdatedImage(this.iconPositions[n],t.getImage(n),e),this.patchUpdatedImage(this.patternPositions[n],t.getImage(n),e)},Ps.prototype.patchUpdatedImage=function(t,e,n){if(t&&e&&t.version!==e.version){t.version=e.version;var r=t.tl,i=r[0],o=r[1];n.update(e.data,void 0,{x:i,y:o})}},vr("ImagePosition",js),vr("ImageAtlas",Ps);var Ks=self.HTMLImageElement,zs=self.HTMLCanvasElement,Js=self.HTMLVideoElement,qs=self.ImageData,Vs=function(t,e,n,r){this.context=t,this.format=n,this.texture=t.gl.createTexture(),this.update(e,r)};Vs.prototype.update=function(t,e,n){var r=t.width,i=t.height,o=!(this.size&&this.size[0]===r&&this.size[1]===i||n),a=this.context,s=a.gl;if(this.useMipmap=Boolean(e&&e.useMipmap),s.bindTexture(s.TEXTURE_2D,this.texture),a.pixelStoreUnpackFlipY.set(!1),a.pixelStoreUnpack.set(1),a.pixelStoreUnpackPremultiplyAlpha.set(this.format===s.RGBA&&(!e||!1!==e.premultiply)),o)this.size=[r,i],t instanceof Ks||t instanceof zs||t instanceof Js||t instanceof qs?s.texImage2D(s.TEXTURE_2D,0,this.format,this.format,s.UNSIGNED_BYTE,t):s.texImage2D(s.TEXTURE_2D,0,this.format,r,i,0,this.format,s.UNSIGNED_BYTE,t.data);else{var A=n||{x:0,y:0},l=A.x,c=A.y;t instanceof Ks||t instanceof zs||t instanceof Js||t instanceof qs?s.texSubImage2D(s.TEXTURE_2D,0,l,c,s.RGBA,s.UNSIGNED_BYTE,t):s.texSubImage2D(s.TEXTURE_2D,0,l,c,r,i,s.RGBA,s.UNSIGNED_BYTE,t.data)}this.useMipmap&&this.isSizePowerOfTwo()&&s.generateMipmap(s.TEXTURE_2D)},Vs.prototype.bind=function(t,e,n){var r=this.context.gl;r.bindTexture(r.TEXTURE_2D,this.texture),n!==r.LINEAR_MIPMAP_NEAREST||this.isSizePowerOfTwo()||(n=r.LINEAR),t!==this.filter&&(r.texParameteri(r.TEXTURE_2D,r.TEXTURE_MAG_FILTER,t),r.texParameteri(r.TEXTURE_2D,r.TEXTURE_MIN_FILTER,n||t),this.filter=t),e!==this.wrap&&(r.texParameteri(r.TEXTURE_2D,r.TEXTURE_WRAP_S,e),r.texParameteri(r.TEXTURE_2D,r.TEXTURE_WRAP_T,e),this.wrap=e)},Vs.prototype.isSizePowerOfTwo=function(){return this.size[0]===this.size[1]&&Math.log(this.size[0])/Math.LN2%1==0},Vs.prototype.destroy=function(){this.context.gl.deleteTexture(this.texture),this.texture=null};var Ws=function(t,e,n,r,i){var o,a,s=8*i-r-1,A=(1<<s)-1,l=A>>1,c=-7,u=n?i-1:0,g=n?-1:1,h=t[e+u];for(u+=g,o=h&(1<<-c)-1,h>>=-c,c+=s;c>0;o=256*o+t[e+u],u+=g,c-=8);for(a=o&(1<<-c)-1,o>>=-c,c+=r;c>0;a=256*a+t[e+u],u+=g,c-=8);if(0===o)o=1-l;else{if(o===A)return a?NaN:1/0*(h?-1:1);a+=Math.pow(2,r),o-=l}return(h?-1:1)*a*Math.pow(2,o-r)},Xs=function(t,e,n,r,i,o){var a,s,A,l=8*o-i-1,c=(1<<l)-1,u=c>>1,g=23===i?Math.pow(2,-24)-Math.pow(2,-77):0,h=r?0:o-1,d=r?1:-1,f=e<0||0===e&&1/e<0?1:0;for(e=Math.abs(e),isNaN(e)||e===1/0?(s=isNaN(e)?1:0,a=c):(a=Math.floor(Math.log(e)/Math.LN2),e*(A=Math.pow(2,-a))<1&&(a--,A*=2),(e+=a+u>=1?g/A:g*Math.pow(2,1-u))*A>=2&&(a++,A/=2),a+u>=c?(s=0,a=c):a+u>=1?(s=(e*A-1)*Math.pow(2,i),a+=u):(s=e*Math.pow(2,u-1)*Math.pow(2,i),a=0));i>=8;t[n+h]=255&s,h+=d,s/=256,i-=8);for(a=a<<i|s,l+=i;l>0;t[n+h]=255&a,h+=d,a/=256,l-=8);t[n+h-d]|=128*f},Zs=$s;function $s(t){this.buf=ArrayBuffer.isView&&ArrayBuffer.isView(t)?t:new Uint8Array(t||0),this.pos=0,this.type=0,this.length=this.buf.length}function tA(t){return t.type===$s.Bytes?t.readVarint()+t.pos:t.pos+1}function eA(t,e,n){return n?4294967296*e+(t>>>0):4294967296*(e>>>0)+(t>>>0)}function nA(t,e,n){var r=e<=16383?1:e<=2097151?2:e<=268435455?3:Math.floor(Math.log(e)/(7*Math.LN2));n.realloc(r);for(var i=n.pos-1;i>=t;i--)n.buf[i+r]=n.buf[i]}function rA(t,e){for(var n=0;n<t.length;n++)e.writeVarint(t[n])}function iA(t,e){for(var n=0;n<t.length;n++)e.writeSVarint(t[n])}function oA(t,e){for(var n=0;n<t.length;n++)e.writeFloat(t[n])}function aA(t,e){for(var n=0;n<t.length;n++)e.writeDouble(t[n])}function sA(t,e){for(var n=0;n<t.length;n++)e.writeBoolean(t[n])}function AA(t,e){for(var n=0;n<t.length;n++)e.writeFixed32(t[n])}function lA(t,e){for(var n=0;n<t.length;n++)e.writeSFixed32(t[n])}function cA(t,e){for(var n=0;n<t.length;n++)e.writeFixed64(t[n])}function uA(t,e){for(var n=0;n<t.length;n++)e.writeSFixed64(t[n])}function gA(t,e){return(t[e]|t[e+1]<<8|t[e+2]<<16)+16777216*t[e+3]}function hA(t,e,n){t[n]=e,t[n+1]=e>>>8,t[n+2]=e>>>16,t[n+3]=e>>>24}function dA(t,e){return(t[e]|t[e+1]<<8|t[e+2]<<16)+(t[e+3]<<24)}$s.Varint=0,$s.Fixed64=1,$s.Bytes=2,$s.Fixed32=5,$s.prototype={destroy:function(){this.buf=null},readFields:function(t,e,n){for(n=n||this.length;this.pos<n;){var r=this.readVarint(),i=r>>3,o=this.pos;this.type=7&r,t(i,e,this),this.pos===o&&this.skip(r)}return e},readMessage:function(t,e){return this.readFields(t,e,this.readVarint()+this.pos)},readFixed32:function(){var t=gA(this.buf,this.pos);return this.pos+=4,t},readSFixed32:function(){var t=dA(this.buf,this.pos);return this.pos+=4,t},readFixed64:function(){var t=gA(this.buf,this.pos)+4294967296*gA(this.buf,this.pos+4);return this.pos+=8,t},readSFixed64:function(){var t=gA(this.buf,this.pos)+4294967296*dA(this.buf,this.pos+4);return this.pos+=8,t},readFloat:function(){var t=Ws(this.buf,this.pos,!0,23,4);return this.pos+=4,t},readDouble:function(){var t=Ws(this.buf,this.pos,!0,52,8);return this.pos+=8,t},readVarint:function(t){var e,n,r=this.buf;return e=127&(n=r[this.pos++]),n<128?e:(e|=(127&(n=r[this.pos++]))<<7,n<128?e:(e|=(127&(n=r[this.pos++]))<<14,n<128?e:(e|=(127&(n=r[this.pos++]))<<21,n<128?e:function(t,e,n){var r,i,o=n.buf;if(r=(112&(i=o[n.pos++]))>>4,i<128)return eA(t,r,e);if(r|=(127&(i=o[n.pos++]))<<3,i<128)return eA(t,r,e);if(r|=(127&(i=o[n.pos++]))<<10,i<128)return eA(t,r,e);if(r|=(127&(i=o[n.pos++]))<<17,i<128)return eA(t,r,e);if(r|=(127&(i=o[n.pos++]))<<24,i<128)return eA(t,r,e);if(r|=(1&(i=o[n.pos++]))<<31,i<128)return eA(t,r,e);throw new Error("Expected varint not more than 10 bytes")}(e|=(15&(n=r[this.pos]))<<28,t,this))))},readVarint64:function(){return this.readVarint(!0)},readSVarint:function(){var t=this.readVarint();return t%2==1?(t+1)/-2:t/2},readBoolean:function(){return Boolean(this.readVarint())},readString:function(){var t=this.readVarint()+this.pos,e=function(t,e,n){for(var r="",i=e;i<n;){var o,a,s,A=t[i],l=null,c=A>239?4:A>223?3:A>191?2:1;if(i+c>n)break;1===c?A<128&&(l=A):2===c?128==(192&(o=t[i+1]))&&(l=(31&A)<<6|63&o)<=127&&(l=null):3===c?(o=t[i+1],a=t[i+2],128==(192&o)&&128==(192&a)&&((l=(15&A)<<12|(63&o)<<6|63&a)<=2047||l>=55296&&l<=57343)&&(l=null)):4===c&&(o=t[i+1],a=t[i+2],s=t[i+3],128==(192&o)&&128==(192&a)&&128==(192&s)&&((l=(15&A)<<18|(63&o)<<12|(63&a)<<6|63&s)<=65535||l>=1114112)&&(l=null)),null===l?(l=65533,c=1):l>65535&&(l-=65536,r+=String.fromCharCode(l>>>10&1023|55296),l=56320|1023&l),r+=String.fromCharCode(l),i+=c}return r}(this.buf,this.pos,t);return this.pos=t,e},readBytes:function(){var t=this.readVarint()+this.pos,e=this.buf.subarray(this.pos,t);return this.pos=t,e},readPackedVarint:function(t,e){if(this.type!==$s.Bytes)return t.push(this.readVarint(e));var n=tA(this);for(t=t||[];this.pos<n;)t.push(this.readVarint(e));return t},readPackedSVarint:function(t){if(this.type!==$s.Bytes)return t.push(this.readSVarint());var e=tA(this);for(t=t||[];this.pos<e;)t.push(this.readSVarint());return t},readPackedBoolean:function(t){if(this.type!==$s.Bytes)return t.push(this.readBoolean());var e=tA(this);for(t=t||[];this.pos<e;)t.push(this.readBoolean());return t},readPackedFloat:function(t){if(this.type!==$s.Bytes)return t.push(this.readFloat());var e=tA(this);for(t=t||[];this.pos<e;)t.push(this.readFloat());return t},readPackedDouble:function(t){if(this.type!==$s.Bytes)return t.push(this.readDouble());var e=tA(this);for(t=t||[];this.pos<e;)t.push(this.readDouble());return t},readPackedFixed32:function(t){if(this.type!==$s.Bytes)return t.push(this.readFixed32());var e=tA(this);for(t=t||[];this.pos<e;)t.push(this.readFixed32());return t},readPackedSFixed32:function(t){if(this.type!==$s.Bytes)return t.push(this.readSFixed32());var e=tA(this);for(t=t||[];this.pos<e;)t.push(this.readSFixed32());return t},readPackedFixed64:function(t){if(this.type!==$s.Bytes)return t.push(this.readFixed64());var e=tA(this);for(t=t||[];this.pos<e;)t.push(this.readFixed64());return t},readPackedSFixed64:function(t){if(this.type!==$s.Bytes)return t.push(this.readSFixed64());var e=tA(this);for(t=t||[];this.pos<e;)t.push(this.readSFixed64());return t},skip:function(t){var e=7&t;if(e===$s.Varint)for(;this.buf[this.pos++]>127;);else if(e===$s.Bytes)this.pos=this.readVarint()+this.pos;else if(e===$s.Fixed32)this.pos+=4;else{if(e!==$s.Fixed64)throw new Error("Unimplemented type: "+e);this.pos+=8}},writeTag:function(t,e){this.writeVarint(t<<3|e)},realloc:function(t){for(var e=this.length||16;e<this.pos+t;)e*=2;if(e!==this.length){var n=new Uint8Array(e);n.set(this.buf),this.buf=n,this.length=e}},finish:function(){return this.length=this.pos,this.pos=0,this.buf.subarray(0,this.length)},writeFixed32:function(t){this.realloc(4),hA(this.buf,t,this.pos),this.pos+=4},writeSFixed32:function(t){this.realloc(4),hA(this.buf,t,this.pos),this.pos+=4},writeFixed64:function(t){this.realloc(8),hA(this.buf,-1&t,this.pos),hA(this.buf,Math.floor(t*(1/4294967296)),this.pos+4),this.pos+=8},writeSFixed64:function(t){this.realloc(8),hA(this.buf,-1&t,this.pos),hA(this.buf,Math.floor(t*(1/4294967296)),this.pos+4),this.pos+=8},writeVarint:function(t){(t=+t||0)>268435455||t<0?function(t,e){var n,r;if(t>=0?(n=t%4294967296|0,r=t/4294967296|0):(r=~(-t/4294967296),4294967295^(n=~(-t%4294967296))?n=n+1|0:(n=0,r=r+1|0)),t>=0x10000000000000000||t<-0x10000000000000000)throw new Error("Given varint doesn't fit into 10 bytes");e.realloc(10),function(t,e,n){n.buf[n.pos++]=127&t|128,t>>>=7,n.buf[n.pos++]=127&t|128,t>>>=7,n.buf[n.pos++]=127&t|128,t>>>=7,n.buf[n.pos++]=127&t|128,t>>>=7,n.buf[n.pos]=127&t}(n,0,e),function(t,e){var n=(7&t)<<4;e.buf[e.pos++]|=n|((t>>>=3)?128:0),t&&(e.buf[e.pos++]=127&t|((t>>>=7)?128:0),t&&(e.buf[e.pos++]=127&t|((t>>>=7)?128:0),t&&(e.buf[e.pos++]=127&t|((t>>>=7)?128:0),t&&(e.buf[e.pos++]=127&t|((t>>>=7)?128:0),t&&(e.buf[e.pos++]=127&t)))))}(r,e)}(t,this):(this.realloc(4),this.buf[this.pos++]=127&t|(t>127?128:0),t<=127||(this.buf[this.pos++]=127&(t>>>=7)|(t>127?128:0),t<=127||(this.buf[this.pos++]=127&(t>>>=7)|(t>127?128:0),t<=127||(this.buf[this.pos++]=t>>>7&127))))},writeSVarint:function(t){this.writeVarint(t<0?2*-t-1:2*t)},writeBoolean:function(t){this.writeVarint(Boolean(t))},writeString:function(t){t=String(t),this.realloc(4*t.length),this.pos++;var e=this.pos;this.pos=function(t,e,n){for(var r,i,o=0;o<e.length;o++){if((r=e.charCodeAt(o))>55295&&r<57344){if(!i){r>56319||o+1===e.length?(t[n++]=239,t[n++]=191,t[n++]=189):i=r;continue}if(r<56320){t[n++]=239,t[n++]=191,t[n++]=189,i=r;continue}r=i-55296<<10|r-56320|65536,i=null}else i&&(t[n++]=239,t[n++]=191,t[n++]=189,i=null);r<128?t[n++]=r:(r<2048?t[n++]=r>>6|192:(r<65536?t[n++]=r>>12|224:(t[n++]=r>>18|240,t[n++]=r>>12&63|128),t[n++]=r>>6&63|128),t[n++]=63&r|128)}return n}(this.buf,t,this.pos);var n=this.pos-e;n>=128&&nA(e,n,this),this.pos=e-1,this.writeVarint(n),this.pos+=n},writeFloat:function(t){this.realloc(4),Xs(this.buf,t,this.pos,!0,23,4),this.pos+=4},writeDouble:function(t){this.realloc(8),Xs(this.buf,t,this.pos,!0,52,8),this.pos+=8},writeBytes:function(t){var e=t.length;this.writeVarint(e),this.realloc(e);for(var n=0;n<e;n++)this.buf[this.pos++]=t[n]},writeRawMessage:function(t,e){this.pos++;var n=this.pos;t(e,this);var r=this.pos-n;r>=128&&nA(n,r,this),this.pos=n-1,this.writeVarint(r),this.pos+=r},writeMessage:function(t,e,n){this.writeTag(t,$s.Bytes),this.writeRawMessage(e,n)},writePackedVarint:function(t,e){e.length&&this.writeMessage(t,rA,e)},writePackedSVarint:function(t,e){e.length&&this.writeMessage(t,iA,e)},writePackedBoolean:function(t,e){e.length&&this.writeMessage(t,sA,e)},writePackedFloat:function(t,e){e.length&&this.writeMessage(t,oA,e)},writePackedDouble:function(t,e){e.length&&this.writeMessage(t,aA,e)},writePackedFixed32:function(t,e){e.length&&this.writeMessage(t,AA,e)},writePackedSFixed32:function(t,e){e.length&&this.writeMessage(t,lA,e)},writePackedFixed64:function(t,e){e.length&&this.writeMessage(t,cA,e)},writePackedSFixed64:function(t,e){e.length&&this.writeMessage(t,uA,e)},writeBytesField:function(t,e){this.writeTag(t,$s.Bytes),this.writeBytes(e)},writeFixed32Field:function(t,e){this.writeTag(t,$s.Fixed32),this.writeFixed32(e)},writeSFixed32Field:function(t,e){this.writeTag(t,$s.Fixed32),this.writeSFixed32(e)},writeFixed64Field:function(t,e){this.writeTag(t,$s.Fixed64),this.writeFixed64(e)},writeSFixed64Field:function(t,e){this.writeTag(t,$s.Fixed64),this.writeSFixed64(e)},writeVarintField:function(t,e){this.writeTag(t,$s.Varint),this.writeVarint(e)},writeSVarintField:function(t,e){this.writeTag(t,$s.Varint),this.writeSVarint(e)},writeStringField:function(t,e){this.writeTag(t,$s.Bytes),this.writeString(e)},writeFloatField:function(t,e){this.writeTag(t,$s.Fixed32),this.writeFloat(e)},writeDoubleField:function(t,e){this.writeTag(t,$s.Fixed64),this.writeDouble(e)},writeBooleanField:function(t,e){this.writeVarintField(t,Boolean(e))}};var fA=3;function pA(t,e,n){1===t&&n.readMessage(IA,e)}function IA(t,e,n){if(3===t){var r=n.readMessage(CA,{}),i=r.id,o=r.bitmap,a=r.width,s=r.height,A=r.left,l=r.top,c=r.advance;e.push({id:i,bitmap:new Yo({width:a+2*fA,height:s+2*fA},o),metrics:{width:a,height:s,left:A,top:l,advance:c}})}}function CA(t,e,n){1===t?e.id=n.readVarint():2===t?e.bitmap=n.readBytes():3===t?e.width=n.readVarint():4===t?e.height=n.readVarint():5===t?e.left=n.readSVarint():6===t?e.top=n.readSVarint():7===t&&(e.advance=n.readVarint())}var mA=fA,yA=function(t,e,n){this.target=t,this.parent=e,this.mapId=n,this.callbacks={},this.callbackID=0,p(["receive"],this),this.target.addEventListener("message",this.receive,!1)};function EA(t,e,n){var r=2*Math.PI*6378137/256/Math.pow(2,n);return[t*r-2*Math.PI*6378137/2,e*r-2*Math.PI*6378137/2]}yA.prototype.send=function(t,e,n,r){var i=this,o=n?this.mapId+":"+this.callbackID++:null;n&&(this.callbacks[o]=n);var a=[];if(this.target.postMessage({targetMapId:r,sourceMapId:this.mapId,type:t,id:String(o),data:br(e,a)},a),n)return{cancel:function(){i.callbacks[o]=null,i.target.postMessage({targetMapId:r,sourceMapId:i.mapId,type:"<cancel>",id:String(o)})}}},yA.prototype.receive=function(t){var e,n=this,r=t.data,i=r.id;if(!r.targetMapId||this.mapId===r.targetMapId){var o=function(t,e){delete n.callbacks[i];var r=[];n.target.postMessage({sourceMapId:n.mapId,type:"<response>",id:String(i),error:t?br(t):null,data:br(e,r)},r)};if("<response>"===r.type||"<cancel>"===r.type)e=this.callbacks[r.id],delete this.callbacks[r.id],e&&r.error?e(Qr(r.error)):e&&e(null,Qr(r.data));else if(void 0!==r.id&&this.parent[r.type]){this.callbacks[r.id]=null;var a=this.parent[r.type](r.sourceMapId,Qr(r.data),o);a&&null===this.callbacks[r.id]&&(this.callbacks[r.id]=a.cancel)}else if(void 0!==r.id&&this.parent.getWorkerSource){var s=r.type.split("."),A=Qr(r.data);this.parent.getWorkerSource(r.sourceMapId,s[0],A.source)[s[1]](A,o)}else this.parent[r.type](Qr(r.data))}},yA.prototype.remove=function(){this.target.removeEventListener("message",this.receive,!1)};var vA=function(t,e){t&&(e?this.setSouthWest(t).setNorthEast(e):4===t.length?this.setSouthWest([t[0],t[1]]).setNorthEast([t[2],t[3]]):this.setSouthWest(t[0]).setNorthEast(t[1]))};vA.prototype.setNorthEast=function(t){return this._ne=t instanceof BA?new BA(t.lng,t.lat):BA.convert(t),this},vA.prototype.setSouthWest=function(t){return this._sw=t instanceof BA?new BA(t.lng,t.lat):BA.convert(t),this},vA.prototype.extend=function(t){var e,n,r=this._sw,i=this._ne;if(t instanceof BA)e=t,n=t;else{if(!(t instanceof vA))return Array.isArray(t)?t.every(Array.isArray)?this.extend(vA.convert(t)):this.extend(BA.convert(t)):this;if(e=t._sw,n=t._ne,!e||!n)return this}return r||i?(r.lng=Math.min(e.lng,r.lng),r.lat=Math.min(e.lat,r.lat),i.lng=Math.max(n.lng,i.lng),i.lat=Math.max(n.lat,i.lat)):(this._sw=new BA(e.lng,e.lat),this._ne=new BA(n.lng,n.lat)),this},vA.prototype.getCenter=function(){return new BA((this._sw.lng+this._ne.lng)/2,(this._sw.lat+this._ne.lat)/2)},vA.prototype.getSouthWest=function(){return this._sw},vA.prototype.getNorthEast=function(){return this._ne},vA.prototype.getNorthWest=function(){return new BA(this.getWest(),this.getNorth())},vA.prototype.getSouthEast=function(){return new BA(this.getEast(),this.getSouth())},vA.prototype.getWest=function(){return this._sw.lng},vA.prototype.getSouth=function(){return this._sw.lat},vA.prototype.getEast=function(){return this._ne.lng},vA.prototype.getNorth=function(){return this._ne.lat},vA.prototype.toArray=function(){return[this._sw.toArray(),this._ne.toArray()]},vA.prototype.toString=function(){return"LngLatBounds("+this._sw.toString()+", "+this._ne.toString()+")"},vA.prototype.isEmpty=function(){return!(this._sw&&this._ne)},vA.convert=function(t){return!t||t instanceof vA?t:new vA(t)};var BA=function(t,e){if(isNaN(t)||isNaN(e))throw new Error("Invalid LngLat object: ("+t+", "+e+")");if(this.lng=+t,this.lat=+e,this.lat>90||this.lat<-90)throw new Error("Invalid LngLat latitude value: must be between -90 and 90")};function bA(t){return 2*Math.PI*6378137*Math.cos(t*Math.PI/180)}function QA(t){return(180+t)/360}function wA(t){return(180-180/Math.PI*Math.log(Math.tan(Math.PI/4+t*Math.PI/360)))/360}function xA(t,e){return t/bA(e)}function _A(t){var e=180-360*t;return 360/Math.PI*Math.atan(Math.exp(e*Math.PI/180))-90}BA.prototype.wrap=function(){return new BA(c(this.lng,-180,180),this.lat)},BA.prototype.toArray=function(){return[this.lng,this.lat]},BA.prototype.toString=function(){return"LngLat("+this.lng+", "+this.lat+")"},BA.prototype.toBounds=function(t){void 0===t&&(t=0);var e=360*t/40075017,n=e/Math.cos(Math.PI/180*this.lat);return new vA(new BA(this.lng-n,this.lat-e),new BA(this.lng+n,this.lat+e))},BA.convert=function(t){if(t instanceof BA)return t;if(Array.isArray(t)&&(2===t.length||3===t.length))return new BA(Number(t[0]),Number(t[1]));if(!Array.isArray(t)&&"object"==typeof t&&null!==t)return new BA(Number("lng"in t?t.lng:t.lon),Number(t.lat));throw new Error("`LngLatLike` argument must be specified as a LngLat instance, an object {lng: <lng>, lat: <lat>}, an object {lon: <lng>, lat: <lat>}, or an array of [<lng>, <lat>]")};var DA=function(t,e,n){void 0===n&&(n=0),this.x=+t,this.y=+e,this.z=+n};DA.fromLngLat=function(t,e){void 0===e&&(e=0);var n=BA.convert(t);return new DA(QA(n.lng),wA(n.lat),xA(e,n.lat))},DA.prototype.toLngLat=function(){return new BA(360*this.x-180,_A(this.y))},DA.prototype.toAltitude=function(){return this.z*bA(_A(this.y))};var MA=function(t,e,n){this.z=t,this.x=e,this.y=n,this.key=LA(0,t,e,n)};MA.prototype.equals=function(t){return this.z===t.z&&this.x===t.x&&this.y===t.y},MA.prototype.url=function(t,e){var n,r,i,o,a,s=(n=this.x,r=this.y,i=this.z,o=EA(256*n,256*(r=Math.pow(2,i)-r-1),i),a=EA(256*(n+1),256*(r+1),i),o[0]+","+o[1]+","+a[0]+","+a[1]),A=function(t,e,n){for(var r,i="",o=t;o>0;o--)i+=(e&(r=1<<o-1)?1:0)+(n&r?2:0);return i}(this.z,this.x,this.y);return t[(this.x+this.y)%t.length].replace("{prefix}",(this.x%16).toString(16)+(this.y%16).toString(16)).replace("{z}",String(this.z)).replace("{x}",String(this.x)).replace("{y}",String("tms"===e?Math.pow(2,this.z)-this.y-1:this.y)).replace("{quadkey}",A).replace("{bbox-epsg-3857}",s)},MA.prototype.getTilePoint=function(t){var e=Math.pow(2,this.z);return new i((t.x*e-this.x)*co,(t.y*e-this.y)*co)};var kA=function(t,e){this.wrap=t,this.canonical=e,this.key=LA(t,e.z,e.x,e.y)},SA=function(t,e,n,r,i){this.overscaledZ=t,this.wrap=e,this.canonical=new MA(n,+r,+i),this.key=LA(e,t,r,i)};function LA(t,e,n,r){(t*=2)<0&&(t=-1*t-1);var i=1<<e;return 32*(i*i*t+i*r+n)+e}SA.prototype.equals=function(t){return this.overscaledZ===t.overscaledZ&&this.wrap===t.wrap&&this.canonical.equals(t.canonical)},SA.prototype.scaledTo=function(t){var e=this.canonical.z-t;return t>this.canonical.z?new SA(t,this.wrap,this.canonical.z,this.canonical.x,this.canonical.y):new SA(t,this.wrap,t,this.canonical.x>>e,this.canonical.y>>e)},SA.prototype.isChildOf=function(t){if(t.wrap!==this.wrap)return!1;var e=this.canonical.z-t.canonical.z;return 0===t.overscaledZ||t.overscaledZ<this.overscaledZ&&t.canonical.x===this.canonical.x>>e&&t.canonical.y===this.canonical.y>>e},SA.prototype.children=function(t){if(this.overscaledZ>=t)return[new SA(this.overscaledZ+1,this.wrap,this.canonical.z,this.canonical.x,this.canonical.y)];var e=this.canonical.z+1,n=2*this.canonical.x,r=2*this.canonical.y;return[new SA(e,this.wrap,e,n,r),new SA(e,this.wrap,e,n+1,r),new SA(e,this.wrap,e,n,r+1),new SA(e,this.wrap,e,n+1,r+1)]},SA.prototype.isLessThan=function(t){return this.wrap<t.wrap||!(this.wrap>t.wrap)&&(this.overscaledZ<t.overscaledZ||!(this.overscaledZ>t.overscaledZ)&&(this.canonical.x<t.canonical.x||!(this.canonical.x>t.canonical.x)&&this.canonical.y<t.canonical.y))},SA.prototype.wrapped=function(){return new SA(this.overscaledZ,0,this.canonical.z,this.canonical.x,this.canonical.y)},SA.prototype.unwrapTo=function(t){return new SA(this.overscaledZ,t,this.canonical.z,this.canonical.x,this.canonical.y)},SA.prototype.overscaleFactor=function(){return Math.pow(2,this.overscaledZ-this.canonical.z)},SA.prototype.toUnwrapped=function(){return new kA(this.wrap,this.canonical)},SA.prototype.toString=function(){return this.overscaledZ+"/"+this.canonical.x+"/"+this.canonical.y},SA.prototype.getTilePoint=function(t){return this.canonical.getTilePoint(new DA(t.x-this.wrap,t.y))},vr("CanonicalTileID",MA),vr("OverscaledTileID",SA,{omit:["posMatrix"]});var RA=function(t,e,n){if(this.uid=t,e.height!==e.width)throw new RangeError("DEM tiles must be square");if(n&&"mapbox"!==n&&"terrarium"!==n)return v('"'+n+'" is not a valid encoding type. Valid types include "mapbox" and "terrarium".');var r=this.dim=e.height;this.stride=this.dim+2,this.data=new Int32Array(this.stride*this.stride);for(var i=e.data,o="terrarium"===n?this._unpackTerrarium:this._unpackMapbox,a=0;a<r;a++)for(var s=0;s<r;s++){var A=4*(a*r+s);this.set(s,a,o(i[A],i[A+1],i[A+2]))}for(var l=0;l<r;l++)this.set(-1,l,this.get(0,l)),this.set(r,l,this.get(r-1,l)),this.set(l,-1,this.get(l,0)),this.set(l,r,this.get(l,r-1));this.set(-1,-1,this.get(0,0)),this.set(r,-1,this.get(r-1,0)),this.set(-1,r,this.get(0,r-1)),this.set(r,r,this.get(r-1,r-1))};RA.prototype.set=function(t,e,n){this.data[this._idx(t,e)]=n+65536},RA.prototype.get=function(t,e){return this.data[this._idx(t,e)]-65536},RA.prototype._idx=function(t,e){if(t<-1||t>=this.dim+1||e<-1||e>=this.dim+1)throw new RangeError("out of range source coordinates for DEM data");return(e+1)*this.stride+(t+1)},RA.prototype._unpackMapbox=function(t,e,n){return(256*t*256+256*e+n)/10-1e4},RA.prototype._unpackTerrarium=function(t,e,n){return 256*t+e+n/256-32768},RA.prototype.getPixels=function(){return new Po({width:this.stride,height:this.stride},new Uint8Array(this.data.buffer))},RA.prototype.backfillBorder=function(t,e,n){if(this.dim!==t.dim)throw new Error("dem dimension mismatch");var r=e*this.dim,i=e*this.dim+this.dim,o=n*this.dim,a=n*this.dim+this.dim;switch(e){case-1:r=i-1;break;case 1:i=r+1}switch(n){case-1:o=a-1;break;case 1:a=o+1}for(var s=-e*this.dim,A=-n*this.dim,l=o;l<a;l++)for(var c=r;c<i;c++)this.set(c,l,t.get(c+s,l+A))},vr("DEMData",RA);var NA=oi([{name:"a_pos",type:"Int16",components:2},{name:"a_texture_pos",type:"Int16",components:2}]),FA=function(t){this._stringToNumber={},this._numberToString=[];for(var e=0;e<t.length;e++){var n=t[e];this._stringToNumber[n]=e,this._numberToString[e]=n}};FA.prototype.encode=function(t){return this._stringToNumber[t]},FA.prototype.decode=function(t){return this._numberToString[t]};var TA=function(t,e,n,r){this.type="Feature",this._vectorTileFeature=t,t._z=e,t._x=n,t._y=r,this.properties=t.properties,null!=t.id&&(this.id=t.id)},UA={geometry:{configurable:!0}};UA.geometry.get=function(){return void 0===this._geometry&&(this._geometry=this._vectorTileFeature.toGeoJSON(this._vectorTileFeature._x,this._vectorTileFeature._y,this._vectorTileFeature._z).geometry),this._geometry},UA.geometry.set=function(t){this._geometry=t},TA.prototype.toJSON=function(){var t={geometry:this.geometry};for(var e in this)"_geometry"!==e&&"_vectorTileFeature"!==e&&(t[e]=this[e]);return t},Object.defineProperties(TA.prototype,UA);var OA=function(){this.state={},this.stateChanges={},this.deletedStates={}};OA.prototype.updateState=function(t,e,n){var r=String(e);if(this.stateChanges[t]=this.stateChanges[t]||{},this.stateChanges[t][r]=this.stateChanges[t][r]||{},u(this.stateChanges[t][r],n),null===this.deletedStates[t])for(var i in this.deletedStates[t]={},this.state[t])i!==r&&(this.deletedStates[t][i]=null);else if(this.deletedStates[t]&&null===this.deletedStates[t][r])for(var o in this.deletedStates[t][r]={},this.state[t][r])n[o]||(this.deletedStates[t][r][o]=null);else for(var a in n)this.deletedStates[t]&&this.deletedStates[t][r]&&null===this.deletedStates[t][r][a]&&delete this.deletedStates[t][r][a]},OA.prototype.removeFeatureState=function(t,e,n){if(null!==this.deletedStates[t]){var r=String(e);if(this.deletedStates[t]=this.deletedStates[t]||{},n&&void 0!==e&&e>=0)null!==this.deletedStates[t][r]&&(this.deletedStates[t][r]=this.deletedStates[t][r]||{},this.deletedStates[t][r][n]=null);else if(void 0!==e&&e>=0)if(this.stateChanges[t]&&this.stateChanges[t][r])for(n in this.deletedStates[t][r]={},this.stateChanges[t][r])this.deletedStates[t][r][n]=null;else this.deletedStates[t][r]=null;else this.deletedStates[t]=null}},OA.prototype.getState=function(t,e){var n=String(e),r=this.state[t]||{},i=this.stateChanges[t]||{},o=u({},r[n],i[n]);if(null===this.deletedStates[t])return{};if(this.deletedStates[t]){var a=this.deletedStates[t][e];if(null===a)return{};for(var s in a)delete o[s]}return o},OA.prototype.initializeTileState=function(t,e){t.setFeatureState(this.state,e)},OA.prototype.coalesceChanges=function(t,e){var n={};for(var r in this.stateChanges){this.state[r]=this.state[r]||{};var i={};for(var o in this.stateChanges[r])this.state[r][o]||(this.state[r][o]={}),u(this.state[r][o],this.stateChanges[r][o]),i[o]=this.state[r][o];n[r]=i}for(var a in this.deletedStates){this.state[a]=this.state[a]||{};var s={};if(null===this.deletedStates[a])for(var A in this.state[a])s[A]={},this.state[a][A]={};else for(var l in this.deletedStates[a]){if(null===this.deletedStates[a][l])this.state[a][l]={};else for(var c=0,g=Object.keys(this.deletedStates[a][l]);c<g.length;c+=1){var h=g[c];delete this.state[a][l][h]}s[l]=this.state[a][l]}n[a]=n[a]||{},u(n[a],s)}if(this.stateChanges={},this.deletedStates={},0!==Object.keys(n).length)for(var d in t)t[d].setFeatureState(n,e)};var GA=function(t,e,n){this.tileID=t,this.x=t.canonical.x,this.y=t.canonical.y,this.z=t.canonical.z,this.grid=e||new Ir(co,16,0),this.grid3D=new Ir(co,16,0),this.featureIndexArray=n||new Oi};function HA(t){for(var e=1/0,n=1/0,r=-1/0,i=-1/0,o=0,a=t;o<a.length;o+=1){var s=a[o];e=Math.min(e,s.x),n=Math.min(n,s.y),r=Math.max(r,s.x),i=Math.max(i,s.y)}return{minX:e,minY:n,maxX:r,maxY:i}}function jA(t,e){return e-t}GA.prototype.insert=function(t,e,n,r,i,o){var a=this.featureIndexArray.length;this.featureIndexArray.emplaceBack(n,r,i);for(var s=o?this.grid3D:this.grid,A=0;A<e.length;A++){for(var l=e[A],c=[1/0,1/0,-1/0,-1/0],u=0;u<l.length;u++){var g=l[u];c[0]=Math.min(c[0],g.x),c[1]=Math.min(c[1],g.y),c[2]=Math.max(c[2],g.x),c[3]=Math.max(c[3],g.y)}c[0]<co&&c[1]<co&&c[2]>=0&&c[3]>=0&&s.insert(a,c[0],c[1],c[2],c[3])}},GA.prototype.loadVTLayers=function(){return this.vtLayers||(this.vtLayers=new Ha.VectorTile(new Zs(this.rawTileData)).layers,this.sourceLayerCoder=new FA(this.vtLayers?Object.keys(this.vtLayers).sort():["_geojsonTileLayer"])),this.vtLayers},GA.prototype.query=function(t,e,n){var r=this;this.loadVTLayers();for(var o=t.params||{},a=co/t.tileSize/t.scale,s=Kn(o.filter),A=t.queryGeometry,l=t.queryPadding*a,c=HA(A),u=this.grid.query(c.minX-l,c.minY-l,c.maxX+l,c.maxY+l),g=HA(t.cameraQueryGeometry),h=0,d=this.grid3D.query(g.minX-l,g.minY-l,g.maxX+l,g.maxY+l,function(e,n,r,o){return function(t,e,n,r,o){for(var a=0,s=t;a<s.length;a+=1){var A=s[a];if(e<=A.x&&n<=A.y&&r>=A.x&&o>=A.y)return!0}var l=[new i(e,n),new i(e,o),new i(r,o),new i(r,n)];if(t.length>2)for(var c=0,u=l;c<u.length;c+=1)if(Qo(t,u[c]))return!0;for(var g=0;g<t.length-1;g++)if(wo(t[g],t[g+1],l))return!0;return!1}(t.cameraQueryGeometry,e-l,n-l,r+l,o+l)});h<d.length;h+=1){var f=d[h];u.push(f)}u.sort(jA);for(var p,I={},C=function(i){var l=u[i];if(l!==p){p=l;var c=r.featureIndexArray.get(l),g=null;r.loadMatchingFeature(I,c.bucketIndex,c.sourceLayerIndex,c.featureIndex,s,o.layers,e,function(e,i){g||(g=go(e));var o={};return e.id&&(o=n.getState(i.sourceLayer||"_geojsonTileLayer",e.id)),i.queryIntersectsFeature(A,e,o,g,r.z,t.transform,a,t.pixelPosMatrix)})}},m=0;m<u.length;m++)C(m);return I},GA.prototype.loadMatchingFeature=function(t,e,n,r,i,o,a,s){var A=this.bucketLayerIDs[e];if(!o||function(t,e){for(var n=0;n<t.length;n++)if(e.indexOf(t[n])>=0)return!0;return!1}(o,A)){var l=this.sourceLayerCoder.decode(n),c=this.vtLayers[l].feature(r);if(i(new Gr(this.tileID.overscaledZ),c))for(var u=0;u<A.length;u++){var g=A[u];if(!(o&&o.indexOf(g)<0)){var h=a[g];if(h){var d=!s||s(c,h);if(d){var f=new TA(c,this.z,this.x,this.y);f.layer=h.serialize();var p=t[g];void 0===p&&(p=t[g]=[]),p.push({featureIndex:r,feature:f,intersectionZ:d})}}}}}},GA.prototype.lookupSymbolFeatures=function(t,e,n,r,i,o){var a={};this.loadVTLayers();for(var s=Kn(r),A=0,l=t;A<l.length;A+=1){var c=l[A];this.loadMatchingFeature(a,e,n,c,s,i,o)}return a},GA.prototype.hasLayer=function(t){for(var e=0,n=this.bucketLayerIDs;e<n.length;e+=1)for(var r=0,i=n[e];r<i.length;r+=1)if(t===i[r])return!0;return!1},vr("FeatureIndex",GA,{omit:["rawTileData","sourceLayerCoder"]});var YA=function(t,e){this.tileID=t,this.uid=h(),this.uses=0,this.tileSize=e,this.buckets={},this.expirationTime=null,this.queryPadding=0,this.hasSymbolBuckets=!1,this.expiredRequestCount=0,this.state="loading"};YA.prototype.registerFadeDuration=function(t){var e=t+this.timeAdded;e<L.now()||this.fadeEndTime&&e<this.fadeEndTime||(this.fadeEndTime=e)},YA.prototype.wasRequested=function(){return"errored"===this.state||"loaded"===this.state||"reloading"===this.state},YA.prototype.loadVectorData=function(t,e,n){if(this.hasData()&&this.unloadVectorData(),this.state="loaded",t){for(var r in t.featureIndex&&(this.latestFeatureIndex=t.featureIndex,t.rawTileData?(this.latestRawTileData=t.rawTileData,this.latestFeatureIndex.rawTileData=t.rawTileData):this.latestRawTileData&&(this.latestFeatureIndex.rawTileData=this.latestRawTileData)),this.collisionBoxArray=t.collisionBoxArray,this.buckets=function(t,e){var n={};if(!e)return n;for(var r=function(){var t=o[i],r=t.layerIds.map(function(t){return e.getLayer(t)}).filter(Boolean);if(0!==r.length){t.layers=r,t.stateDependentLayerIds&&(t.stateDependentLayers=t.stateDependentLayerIds.map(function(t){return r.filter(function(e){return e.id===t})[0]}));for(var a=0,s=r;a<s.length;a+=1){var A=s[a];n[A.id]=t}}},i=0,o=t;i<o.length;i+=1)r();return n}(t.buckets,e.style),this.hasSymbolBuckets=!1,this.buckets){var i=this.buckets[r];if(i instanceof ks){if(this.hasSymbolBuckets=!0,!n)break;i.justReloaded=!0}}for(var o in this.queryPadding=0,this.buckets){var a=this.buckets[o];this.queryPadding=Math.max(this.queryPadding,e.style.getLayer(o).queryRadius(a))}t.imageAtlas&&(this.imageAtlas=t.imageAtlas),t.glyphAtlasImage&&(this.glyphAtlasImage=t.glyphAtlasImage)}else this.collisionBoxArray=new Di},YA.prototype.unloadVectorData=function(){for(var t in this.buckets)this.buckets[t].destroy();this.buckets={},this.imageAtlasTexture&&this.imageAtlasTexture.destroy(),this.imageAtlas&&(this.imageAtlas=null),this.glyphAtlasTexture&&this.glyphAtlasTexture.destroy(),this.latestFeatureIndex=null,this.state="unloaded"},YA.prototype.unloadDEMData=function(){this.dem=null,this.neighboringTiles=null,this.state="unloaded"},YA.prototype.getBucket=function(t){return this.buckets[t.id]},YA.prototype.upload=function(t){for(var e in this.buckets){var n=this.buckets[e];n.uploadPending()&&n.upload(t)}var r=t.gl;this.imageAtlas&&!this.imageAtlas.uploaded&&(this.imageAtlasTexture=new Vs(t,this.imageAtlas.image,r.RGBA),this.imageAtlas.uploaded=!0),this.glyphAtlasImage&&(this.glyphAtlasTexture=new Vs(t,this.glyphAtlasImage,r.ALPHA),this.glyphAtlasImage=null)},YA.prototype.prepare=function(t){this.imageAtlas&&this.imageAtlas.patchUpdatedImages(t,this.imageAtlasTexture)},YA.prototype.queryRenderedFeatures=function(t,e,n,r,i,o,a,s,A){return this.latestFeatureIndex&&this.latestFeatureIndex.rawTileData?this.latestFeatureIndex.query({queryGeometry:n,cameraQueryGeometry:r,scale:i,tileSize:this.tileSize,pixelPosMatrix:A,transform:a,params:o,queryPadding:this.queryPadding*s},t,e):{}},YA.prototype.querySourceFeatures=function(t,e){if(this.latestFeatureIndex&&this.latestFeatureIndex.rawTileData){var n=this.latestFeatureIndex.loadVTLayers(),r=e?e.sourceLayer:"",i=n._geojsonTileLayer||n[r];if(i)for(var o=Kn(e&&e.filter),a=this.tileID.canonical,s=a.z,A=a.x,l=a.y,c={z:s,x:A,y:l},u=0;u<i.length;u++){var g=i.feature(u);if(o(new Gr(this.tileID.overscaledZ),g)){var h=new TA(g,s,A,l);h.tile=c,t.push(h)}}}},YA.prototype.clearMask=function(){this.segments&&(this.segments.destroy(),delete this.segments),this.maskedBoundsBuffer&&(this.maskedBoundsBuffer.destroy(),delete this.maskedBoundsBuffer),this.maskedIndexBuffer&&(this.maskedIndexBuffer.destroy(),delete this.maskedIndexBuffer)},YA.prototype.setMask=function(t,e){if(!a(this.mask,t)&&(this.mask=t,this.clearMask(),!a(t,{0:!0}))){var n=new Ai,r=new Bi;this.segments=new Hi,this.segments.prepareSegment(0,n,r);for(var o=Object.keys(t),s=0;s<o.length;s++){var A=t[+o[s]],l=co>>A.z,c=new i(A.x*l,A.y*l),u=new i(c.x+l,c.y+l),g=this.segments.prepareSegment(4,n,r);n.emplaceBack(c.x,c.y,c.x,c.y),n.emplaceBack(u.x,c.y,u.x,c.y),n.emplaceBack(c.x,u.y,c.x,u.y),n.emplaceBack(u.x,u.y,u.x,u.y);var h=g.vertexLength;r.emplaceBack(h,h+1,h+2),r.emplaceBack(h+1,h+2,h+3),g.vertexLength+=4,g.primitiveLength+=2}this.maskedBoundsBuffer=e.createVertexBuffer(n,NA.members),this.maskedIndexBuffer=e.createIndexBuffer(r)}},YA.prototype.hasData=function(){return"loaded"===this.state||"reloading"===this.state||"expired"===this.state},YA.prototype.patternsLoaded=function(){return this.imageAtlas&&!!Object.keys(this.imageAtlas.patternPositions).length},YA.prototype.setExpiryData=function(t){var e=this.expirationTime;if(t.cacheControl){var n=Q(t.cacheControl);n["max-age"]&&(this.expirationTime=Date.now()+1e3*n["max-age"])}else t.expires&&(this.expirationTime=new Date(t.expires).getTime());if(this.expirationTime){var r=Date.now(),i=!1;if(this.expirationTime>r)i=!1;else if(e)if(this.expirationTime<e)i=!0;else{var o=this.expirationTime-e;o?this.expirationTime=r+Math.max(o,3e4):i=!0}else i=!0;i?(this.expiredRequestCount++,this.state="expired"):this.expiredRequestCount=0}},YA.prototype.getExpiryTimeout=function(){if(this.expirationTime)return this.expiredRequestCount?1e3*(1<<Math.min(this.expiredRequestCount-1,31)):Math.min(this.expirationTime-(new Date).getTime(),Math.pow(2,31)-1)},YA.prototype.setFeatureState=function(t,e){if(this.latestFeatureIndex&&this.latestFeatureIndex.rawTileData&&0!==Object.keys(t).length){var n=this.latestFeatureIndex.loadVTLayers();for(var r in this.buckets){var i=this.buckets[r],o=i.layers[0].sourceLayer||"_geojsonTileLayer",a=n[o],s=t[o];a&&s&&0!==Object.keys(s).length&&(i.update(s,a,this.imageAtlas&&this.imageAtlas.patternPositions||{}),e&&e.style&&(this.queryPadding=Math.max(this.queryPadding,e.style.getLayer(r).queryRadius(i))))}}},YA.prototype.holdingForFade=function(){return void 0!==this.symbolFadeHoldUntil},YA.prototype.symbolFadeFinished=function(){return!this.symbolFadeHoldUntil||this.symbolFadeHoldUntil<L.now()},YA.prototype.clearFadeHold=function(){this.symbolFadeHoldUntil=void 0},YA.prototype.setHoldDuration=function(t){this.symbolFadeHoldUntil=L.now()+t};var PA=24,KA={horizontal:1,vertical:2,horizontalOnly:3},zA=function(){this.text="",this.sectionIndex=[],this.sections=[]};function JA(t,e,n,r,i,o,a,s,A,l){var c,u=zA.fromFeature(t,n);l===KA.vertical&&u.verticalizePunctuation();var g=Or.processBidirectionalText,h=Or.processStyledBidirectionalText;if(g&&1===u.sections.length){c=[];for(var d=0,f=g(u.toString(),$A(u,s,r,e));d<f.length;d+=1){var p=f[d],I=new zA;I.text=p,I.sections=u.sections;for(var C=0;C<p.length;C++)I.sectionIndex.push(0);c.push(I)}}else if(h){c=[];for(var m=0,y=h(u.text,u.sectionIndex,$A(u,s,r,e));m<y.length;m+=1){var E=y[m],v=new zA;v.text=E[0],v.sectionIndex=E[1],v.sections=u.sections,c.push(v)}}else c=function(t,e){for(var n=[],r=t.text,i=0,o=0,a=e;o<a.length;o+=1){var s=a[o];n.push(t.substring(i,s)),i=s}return i<r.length&&n.push(t.substring(i,r.length)),n}(u,$A(u,s,r,e));var B=[],b={positionedGlyphs:B,text:u.toString(),top:A[1],bottom:A[1],left:A[0],right:A[0],writingMode:l,lineCount:c.length};return function(t,e,n,r,i,o,a,s){for(var A=0,l=-17,c=0,u=t.positionedGlyphs,g="right"===o?1:"left"===o?0:.5,h=0,d=n;h<d.length;h+=1){var f=d[h];f.trim();var p=f.getMaxScale();if(f.length()){for(var I=u.length,C=0;C<f.length();C++){var m=f.getSection(C),y=f.getCharCode(C),E=24*(p-m.scale),v=e[m.fontStack],B=v&&v[y];B&&(kr(y)&&a!==KA.horizontal?(u.push({glyph:y,x:A,y:E,vertical:!0,scale:m.scale,fontStack:m.fontStack}),A+=PA*m.scale+s):(u.push({glyph:y,x:A,y:l+E,vertical:!1,scale:m.scale,fontStack:m.fontStack}),A+=B.metrics.advance*m.scale+s))}if(u.length!==I){var b=A-s;c=Math.max(b,c),el(u,e,I,u.length-1,g)}A=0,l+=r*p}else l+=r}var Q=tl(i),w=Q.horizontalAlign,x=Q.verticalAlign;!function(t,e,n,r,i,o,a){for(var s=(e-n)*i,A=(-r*a+.5)*o,l=0;l<t.length;l++)t[l].x+=s,t[l].y+=A}(u,g,w,x,c,r,n.length);var _=l- -17;t.top+=-x*_,t.bottom=t.top+_,t.left+=-w*c,t.right=t.left+c}(b,e,c,i,o,a,l,s),!!B.length&&b}zA.fromFeature=function(t,e){for(var n=new zA,r=0;r<t.sections.length;r++){var i=t.sections[r];n.sections.push({scale:i.scale||1,fontStack:i.fontStack||e}),n.text+=i.text;for(var o=0;o<i.text.length;o++)n.sectionIndex.push(r)}return n},zA.prototype.length=function(){return this.text.length},zA.prototype.getSection=function(t){return this.sections[this.sectionIndex[t]]},zA.prototype.getCharCode=function(t){return this.text.charCodeAt(t)},zA.prototype.verticalizePunctuation=function(){this.text=function(t){for(var e="",n=0;n<t.length;n++){var r=t.charCodeAt(n+1)||null,i=t.charCodeAt(n-1)||null;r&&Sr(r)&&!Cs[t[n+1]]||i&&Sr(i)&&!Cs[t[n-1]]||!Cs[t[n]]?e+=t[n]:e+=Cs[t[n]]}return e}(this.text)},zA.prototype.trim=function(){for(var t=0,e=0;e<this.text.length&&qA[this.text.charCodeAt(e)];e++)t++;for(var n=this.text.length,r=this.text.length-1;r>=0&&r>=t&&qA[this.text.charCodeAt(r)];r--)n--;this.text=this.text.substring(t,n),this.sectionIndex=this.sectionIndex.slice(t,n)},zA.prototype.substring=function(t,e){var n=new zA;return n.text=this.text.substring(t,e),n.sectionIndex=this.sectionIndex.slice(t,e),n.sections=this.sections,n},zA.prototype.toString=function(){return this.text},zA.prototype.getMaxScale=function(){var t=this;return this.sectionIndex.reduce(function(e,n){return Math.max(e,t.sections[n].scale)},0)};var qA={9:!0,10:!0,11:!0,12:!0,13:!0,32:!0},VA={};function WA(t,e,n,r){var i=Math.pow(t-e,2);return r?t<e?i/2:2*i:i+Math.abs(n)*n}function XA(t,e,n){var r=0;return 10===t&&(r-=1e4),n&&(r+=150),40!==t&&65288!==t||(r+=50),41!==e&&65289!==e||(r+=50),r}function ZA(t,e,n,r,i,o){for(var a=null,s=WA(e,n,i,o),A=0,l=r;A<l.length;A+=1){var c=l[A],u=WA(e-c.x,n,i,o)+c.badness;u<=s&&(a=c,s=u)}return{index:t,x:e,priorBreak:a,badness:s}}function $A(t,e,n,r){if(!n)return[];if(!t)return[];for(var i,o=[],a=function(t,e,n,r){for(var i=0,o=0;o<t.length();o++){var a=t.getSection(o),s=r[a.fontStack],A=s&&s[t.getCharCode(o)];A&&(i+=A.metrics.advance*a.scale+e)}return i/Math.max(1,Math.ceil(i/n))}(t,e,n,r),s=t.text.indexOf("​")>=0,A=0,l=0;l<t.length();l++){var c=t.getSection(l),u=t.getCharCode(l),g=r[c.fontStack],h=g&&g[u];if(h&&!qA[u]&&(A+=h.metrics.advance*c.scale+e),l<t.length()-1){var d=!((i=u)<11904||!(xr["Bopomofo Extended"](i)||xr.Bopomofo(i)||xr["CJK Compatibility Forms"](i)||xr["CJK Compatibility Ideographs"](i)||xr["CJK Compatibility"](i)||xr["CJK Radicals Supplement"](i)||xr["CJK Strokes"](i)||xr["CJK Symbols and Punctuation"](i)||xr["CJK Unified Ideographs Extension A"](i)||xr["CJK Unified Ideographs"](i)||xr["Enclosed CJK Letters and Months"](i)||xr["Halfwidth and Fullwidth Forms"](i)||xr.Hiragana(i)||xr["Ideographic Description Characters"](i)||xr["Kangxi Radicals"](i)||xr["Katakana Phonetic Extensions"](i)||xr.Katakana(i)||xr["Vertical Forms"](i)||xr["Yi Radicals"](i)||xr["Yi Syllables"](i)));(VA[u]||d)&&o.push(ZA(l+1,A,a,o,XA(u,t.getCharCode(l+1),d&&s),!1))}}return function t(e){return e?t(e.priorBreak).concat(e.index):[]}(ZA(t.length(),A,a,o,0,!0))}function tl(t){var e=.5,n=.5;switch(t){case"right":case"top-right":case"bottom-right":e=1;break;case"left":case"top-left":case"bottom-left":e=0}switch(t){case"bottom":case"bottom-right":case"bottom-left":n=1;break;case"top":case"top-right":case"top-left":n=0}return{horizontalAlign:e,verticalAlign:n}}function el(t,e,n,r,i){if(i){var o=t[r],a=e[o.fontStack],s=a&&a[o.glyph];if(s)for(var A=s.metrics.advance*o.scale,l=(t[r].x+A)*i,c=n;c<=r;c++)t[c].x-=l}}function nl(t,e,n){var r=tl(n),i=r.horizontalAlign,o=r.verticalAlign,a=e[0],s=e[1],A=a-t.displaySize[0]*i,l=A+t.displaySize[0],c=s-t.displaySize[1]*o;return{image:t,top:c,bottom:c+t.displaySize[1],left:A,right:l}}function rl(t,e,n,r,i){if(void 0===e.segment)return!0;for(var o=e,a=e.segment+1,s=0;s>-n/2;){if(--a<0)return!1;s-=t[a].dist(o),o=t[a]}s+=t[a].dist(t[a+1]),a++;for(var A=[],l=0;s<n/2;){var c=t[a-1],u=t[a],g=t[a+1];if(!g)return!1;var h=c.angleTo(u)-u.angleTo(g);for(h=Math.abs((h+3*Math.PI)%(2*Math.PI)-Math.PI),A.push({distance:s,angleDelta:h}),l+=h;s-A[0].distance>r;)l-=A.shift().angleDelta;if(l>i)return!1;a++,s+=u.dist(g)}return!0}function il(t){for(var e=0,n=0;n<t.length-1;n++)e+=t[n].dist(t[n+1]);return e}function ol(t,e,n){return t?.6*e*n:0}function al(t,e){return Math.max(t?t.right-t.left:0,e?e.right-e.left:0)}function sl(t,e,n,r,i,o){for(var a=ol(n,i,o),s=al(n,r)*o,A=0,l=il(t)/2,c=0;c<t.length-1;c++){var u=t[c],g=t[c+1],h=u.dist(g);if(A+h>l){var d=(l-A)/h,f=xe(u.x,g.x,d),p=xe(u.y,g.y,d),I=new ms(f,p,g.angleTo(u),c);return I._round(),!a||rl(t,I,s,a,e)?I:void 0}A+=h}}function Al(t,e,n,r,i,o,a,s,A){var l=ol(r,o,a),c=al(r,i),u=c*a,g=0===t[0].x||t[0].x===A||0===t[0].y||t[0].y===A;return e-u<e/4&&(e=u+e/4),function t(e,n,r,i,o,a,s,A,l){for(var c=a/2,u=il(e),g=0,h=n-r,d=[],f=0;f<e.length-1;f++){for(var p=e[f],I=e[f+1],C=p.dist(I),m=I.angleTo(p);h+r<g+C;){var y=((h+=r)-g)/C,E=xe(p.x,I.x,y),v=xe(p.y,I.y,y);if(E>=0&&E<l&&v>=0&&v<l&&h-c>=0&&h+c<=u){var B=new ms(E,v,m,f);B._round(),i&&!rl(e,B,a,i,o)||d.push(B)}}g+=C}return A||d.length||s||(d=t(e,g/2,r,i,o,a,s,!0,l)),d}(t,g?e/2*s%e:(c/2+2*o)*a*s%e,e,l,n,u,g,!1,A)}VA[10]=!0,VA[32]=!0,VA[38]=!0,VA[40]=!0,VA[41]=!0,VA[43]=!0,VA[45]=!0,VA[47]=!0,VA[173]=!0,VA[183]=!0,VA[8203]=!0,VA[8208]=!0,VA[8211]=!0,VA[8231]=!0;var ll=function(t,e,n,r,o,a,s,A,l,c,u,g){var h=s.top*A-l,d=s.bottom*A+l,f=s.left*A-l,p=s.right*A+l;if(this.boxStartIndex=t.length,c){var I=d-h,C=p-f;I>0&&(I=Math.max(10*A,I),this._addLineCollisionCircles(t,e,n,n.segment,C,I,r,o,a,u))}else{if(g){var m=new i(f,h),y=new i(p,h),E=new i(f,d),v=new i(p,d),B=g*Math.PI/180;m._rotate(B),y._rotate(B),E._rotate(B),v._rotate(B),f=Math.min(m.x,y.x,E.x,v.x),p=Math.max(m.x,y.x,E.x,v.x),h=Math.min(m.y,y.y,E.y,v.y),d=Math.max(m.y,y.y,E.y,v.y)}t.emplaceBack(n.x,n.y,f,h,p,d,r,o,a,0,0)}this.boxEndIndex=t.length};ll.prototype._addLineCollisionCircles=function(t,e,n,r,i,o,a,s,A,l){var c=o/2,u=Math.floor(i/c)||1,g=1+.4*Math.log(l)/Math.LN2,h=Math.floor(u*g/2),d=-o/2,f=n,p=r+1,I=d,C=-i/2,m=C-i/4;do{if(--p<0){if(I>C)return;p=0;break}I-=e[p].dist(f),f=e[p]}while(I>m);for(var y=e[p].dist(e[p+1]),E=-h;E<u+h;E++){var v=E*c,B=C+v;if(v<0&&(B+=v),v>i&&(B+=v-i),!(B<I)){for(;I+y<B;){if(I+=y,++p+1>=e.length)return;y=e[p].dist(e[p+1])}var b=B-I,Q=e[p],w=e[p+1].sub(Q)._unit()._mult(b)._add(Q)._round(),x=Math.abs(B-d)<c?0:.8*(B-d);t.emplaceBack(w.x,w.y,-o/2,-o/2,o/2,o/2,a,s,A,o/2,x)}}};var cl=function(t,e){if(void 0===t&&(t=[]),void 0===e&&(e=ul),this.data=t,this.length=this.data.length,this.compare=e,this.length>0)for(var n=(this.length>>1)-1;n>=0;n--)this._down(n)};function ul(t,e){return t<e?-1:t>e?1:0}function gl(t,e,n){void 0===e&&(e=1),void 0===n&&(n=!1);for(var r=1/0,o=1/0,a=-1/0,s=-1/0,A=t[0],l=0;l<A.length;l++){var c=A[l];(!l||c.x<r)&&(r=c.x),(!l||c.y<o)&&(o=c.y),(!l||c.x>a)&&(a=c.x),(!l||c.y>s)&&(s=c.y)}var u=a-r,g=s-o,h=Math.min(u,g),d=h/2,f=new cl([],hl);if(0===h)return new i(r,o);for(var p=r;p<a;p+=h)for(var I=o;I<s;I+=h)f.push(new dl(p+d,I+d,d,t));for(var C=function(t){for(var e=0,n=0,r=0,i=t[0],o=0,a=i.length,s=a-1;o<a;s=o++){var A=i[o],l=i[s],c=A.x*l.y-l.x*A.y;n+=(A.x+l.x)*c,r+=(A.y+l.y)*c,e+=3*c}return new dl(n/e,r/e,0,t)}(t),m=f.length;f.length;){var y=f.pop();(y.d>C.d||!C.d)&&(C=y,n&&console.log("found best %d after %d probes",Math.round(1e4*y.d)/1e4,m)),y.max-C.d<=e||(d=y.h/2,f.push(new dl(y.p.x-d,y.p.y-d,d,t)),f.push(new dl(y.p.x+d,y.p.y-d,d,t)),f.push(new dl(y.p.x-d,y.p.y+d,d,t)),f.push(new dl(y.p.x+d,y.p.y+d,d,t)),m+=4)}return n&&(console.log("num probes: "+m),console.log("best distance: "+C.d)),C.p}function hl(t,e){return e.max-t.max}function dl(t,e,n,r){this.p=new i(t,e),this.h=n,this.d=function(t,e){for(var n=!1,r=1/0,i=0;i<e.length;i++)for(var o=e[i],a=0,s=o.length,A=s-1;a<s;A=a++){var l=o[a],c=o[A];l.y>t.y!=c.y>t.y&&t.x<(c.x-l.x)*(t.y-l.y)/(c.y-l.y)+l.x&&(n=!n),r=Math.min(r,Bo(t,l,c))}return(n?1:-1)*Math.sqrt(r)}(this.p,r),this.max=this.d+this.h*Math.SQRT2}cl.prototype.push=function(t){this.data.push(t),this.length++,this._up(this.length-1)},cl.prototype.pop=function(){if(0!==this.length){var t=this.data[0];return this.length--,this.length>0&&(this.data[0]=this.data[this.length],this._down(0)),this.data.pop(),t}},cl.prototype.peek=function(){return this.data[0]},cl.prototype._up=function(t){for(var e=this.data,n=this.compare,r=e[t];t>0;){var i=t-1>>1,o=e[i];if(n(r,o)>=0)break;e[t]=o,t=i}e[t]=r},cl.prototype._down=function(t){for(var e=this.data,n=this.compare,r=this.length>>1,i=e[t];t<r;){var o=1+(t<<1),a=e[o],s=o+1;if(s<this.length&&n(e[s],a)<0&&(o=s,a=e[s]),n(a,i)>=0)break;e[t]=a,t=o}e[t]=i};var fl=e(function(t){t.exports=function(t,e){var n,r,i,o,a,s,A,l;for(n=3&t.length,r=t.length-n,i=e,a=3432918353,s=461845907,l=0;l<r;)A=255&t.charCodeAt(l)|(255&t.charCodeAt(++l))<<8|(255&t.charCodeAt(++l))<<16|(255&t.charCodeAt(++l))<<24,++l,i=27492+(65535&(o=5*(65535&(i=(i^=A=(65535&(A=(A=(65535&A)*a+(((A>>>16)*a&65535)<<16)&4294967295)<<15|A>>>17))*s+(((A>>>16)*s&65535)<<16)&4294967295)<<13|i>>>19))+((5*(i>>>16)&65535)<<16)&4294967295))+((58964+(o>>>16)&65535)<<16);switch(A=0,n){case 3:A^=(255&t.charCodeAt(l+2))<<16;case 2:A^=(255&t.charCodeAt(l+1))<<8;case 1:i^=A=(65535&(A=(A=(65535&(A^=255&t.charCodeAt(l)))*a+(((A>>>16)*a&65535)<<16)&4294967295)<<15|A>>>17))*s+(((A>>>16)*s&65535)<<16)&4294967295}return i^=t.length,i=2246822507*(65535&(i^=i>>>16))+((2246822507*(i>>>16)&65535)<<16)&4294967295,i=3266489909*(65535&(i^=i>>>13))+((3266489909*(i>>>16)&65535)<<16)&4294967295,(i^=i>>>16)>>>0}}),pl=e(function(t){t.exports=function(t,e){for(var n,r=t.length,i=e^r,o=0;r>=4;)n=1540483477*(65535&(n=255&t.charCodeAt(o)|(255&t.charCodeAt(++o))<<8|(255&t.charCodeAt(++o))<<16|(255&t.charCodeAt(++o))<<24))+((1540483477*(n>>>16)&65535)<<16),i=1540483477*(65535&i)+((1540483477*(i>>>16)&65535)<<16)^(n=1540483477*(65535&(n^=n>>>24))+((1540483477*(n>>>16)&65535)<<16)),r-=4,++o;switch(r){case 3:i^=(255&t.charCodeAt(o+2))<<16;case 2:i^=(255&t.charCodeAt(o+1))<<8;case 1:i=1540483477*(65535&(i^=255&t.charCodeAt(o)))+((1540483477*(i>>>16)&65535)<<16)}return i=1540483477*(65535&(i^=i>>>13))+((1540483477*(i>>>16)&65535)<<16),(i^=i>>>15)>>>0}}),Il=fl,Cl=fl,ml=pl;Il.murmur3=Cl,Il.murmur2=ml;var yl=7;function El(t,e){var n=0,r=0,i=e/Math.sqrt(2);switch(t){case"top-right":case"top-left":r=i-yl;break;case"bottom-right":case"bottom-left":r=-i+yl;break;case"bottom":r=-e+yl;break;case"top":r=e-yl}switch(t){case"top-right":case"bottom-right":n=-i;break;case"top-left":case"bottom-left":n=i;break;case"left":n=e;break;case"right":n=-e}return[n,r]}function vl(t){switch(t){case"right":case"top-right":case"bottom-right":return"right";case"left":case"top-left":case"bottom-left":return"left"}return"center"}function Bl(t,e,n,r,o,a,s){var A=a.layoutTextSize.evaluate(e,{}),l=a.layoutIconSize.evaluate(e,{}),c=a.textMaxSize.evaluate(e,{});void 0===c&&(c=A);var u=t.layers[0].layout,g=u.get("icon-offset").evaluate(e,{}),h=wl(n.horizontal),d=A/24,f=t.tilePixelRatio*d,p=t.tilePixelRatio*c/24,I=t.tilePixelRatio*l,C=t.tilePixelRatio*u.get("symbol-spacing"),m=u.get("text-padding")*t.tilePixelRatio,y=u.get("icon-padding")*t.tilePixelRatio,E=u.get("text-max-angle")/180*Math.PI,B="map"===u.get("text-rotation-alignment")&&"point"!==u.get("symbol-placement"),b="map"===u.get("icon-rotation-alignment")&&"point"!==u.get("symbol-placement"),Q=u.get("symbol-placement"),w=C/2,x=function(A,l){l.x<0||l.x>=co||l.y<0||l.y>=co||function(t,e,n,r,o,a,s,A,l,c,u,g,h,d,f,p,I,C,m,y,E){var B,b,Q=t.addToLineVertexArray(e,n),w=0,x=0,_=0,D={},M=Il(""),k=(a.layout.get("text-radial-offset").evaluate(m,{})||0)*PA;for(var S in r.horizontal){var L=r.horizontal[S];if(!B){M=Il(L.text);var R=a.layout.get("text-rotate").evaluate(m,{});B=new ll(s,n,e,A,l,c,L,u,g,h,t.overscaling,R)}var N=1===L.lineCount;if(x+=Ql(t,e,L,a,h,m,d,Q,r.vertical?KA.horizontal:KA.horizontalOnly,N?Object.keys(r.horizontal):[S],D,y,E),N)break}r.vertical&&(_+=Ql(t,e,r.vertical,a,h,m,d,Q,KA.vertical,["vertical"],D,y,E));var F=B?B.boxStartIndex:t.collisionBoxArray.length,T=B?B.boxEndIndex:t.collisionBoxArray.length;if(o){var U=function(t,e,n,r,o,a){var s,A,l,c,u=e.image,g=n.layout,h=e.top-1/u.pixelRatio,d=e.left-1/u.pixelRatio,f=e.bottom+1/u.pixelRatio,p=e.right+1/u.pixelRatio;if("none"!==g.get("icon-text-fit")&&o){var I=p-d,C=f-h,m=g.get("text-size").evaluate(a,{})/24,y=o.left*m,E=o.right*m,v=o.top*m,B=E-y,b=o.bottom*m-v,Q=g.get("icon-text-fit-padding")[0],w=g.get("icon-text-fit-padding")[1],x=g.get("icon-text-fit-padding")[2],_=g.get("icon-text-fit-padding")[3],D="width"===g.get("icon-text-fit")?.5*(b-C):0,M="height"===g.get("icon-text-fit")?.5*(B-I):0,k="width"===g.get("icon-text-fit")||"both"===g.get("icon-text-fit")?B:I,S="height"===g.get("icon-text-fit")||"both"===g.get("icon-text-fit")?b:C;s=new i(y+M-_,v+D-Q),A=new i(y+M+w+k,v+D-Q),l=new i(y+M+w+k,v+D+x+S),c=new i(y+M-_,v+D+x+S)}else s=new i(d,h),A=new i(p,h),l=new i(p,f),c=new i(d,f);var L=n.layout.get("icon-rotate").evaluate(a,{})*Math.PI/180;if(L){var R=Math.sin(L),N=Math.cos(L),F=[N,-R,R,N];s._matMult(F),A._matMult(F),c._matMult(F),l._matMult(F)}return[{tl:s,tr:A,bl:c,br:l,tex:u.paddedRect,writingMode:void 0,glyphOffset:[0,0]}]}(0,o,a,0,wl(r.horizontal),m),O=a.layout.get("icon-rotate").evaluate(m,{});b=new ll(s,n,e,A,l,c,o,f,p,!1,t.overscaling,O),w=4*U.length;var G=t.iconSizeData,H=null;"source"===G.kind?(H=[ys*a.layout.get("icon-size").evaluate(m,{})])[0]>bl&&v(t.layerIds[0]+': Value for "icon-size" is >= 256. Reduce your "icon-size".'):"composite"===G.kind&&((H=[ys*E.compositeIconSizes[0].evaluate(m,{}),ys*E.compositeIconSizes[1].evaluate(m,{})])[0]>bl||H[1]>bl)&&v(t.layerIds[0]+': Value for "icon-size" is >= 256. Reduce your "icon-size".'),t.addSymbols(t.icon,U,H,C,I,m,!1,e,Q.lineStartIndex,Q.lineLength)}var j=b?b.boxStartIndex:t.collisionBoxArray.length,Y=b?b.boxEndIndex:t.collisionBoxArray.length;t.glyphOffsetArray.length>=ks.MAX_GLYPHS&&v("Too many glyphs being rendered in a tile. See https://github.com/mapbox/mapbox-gl-js/issues/2907"),t.symbolInstances.emplaceBack(e.x,e.y,D.right>=0?D.right:-1,D.center>=0?D.center:-1,D.left>=0?D.left:-1,D.vertical||-1,M,F,T,j,Y,A,x,_,w,0,u,k)}(t,l,A,n,r,t.layers[0],t.collisionBoxArray,e.index,e.sourceLayerIndex,t.index,f,m,B,s,I,y,b,g,e,o,a)};if("line"===Q)for(var _=0,D=function(t,e,n,r,o){for(var a=[],s=0;s<t.length;s++)for(var A=t[s],l=void 0,c=0;c<A.length-1;c++){var u=A[c],g=A[c+1];u.x<0&&g.x<0||(u.x<0?u=new i(0,u.y+(g.y-u.y)*((0-u.x)/(g.x-u.x)))._round():g.x<0&&(g=new i(0,u.y+(g.y-u.y)*((0-u.x)/(g.x-u.x)))._round()),u.y<0&&g.y<0||(u.y<0?u=new i(u.x+(g.x-u.x)*((0-u.y)/(g.y-u.y)),0)._round():g.y<0&&(g=new i(u.x+(g.x-u.x)*((0-u.y)/(g.y-u.y)),0)._round()),u.x>=r&&g.x>=r||(u.x>=r?u=new i(r,u.y+(g.y-u.y)*((r-u.x)/(g.x-u.x)))._round():g.x>=r&&(g=new i(r,u.y+(g.y-u.y)*((r-u.x)/(g.x-u.x)))._round()),u.y>=o&&g.y>=o||(u.y>=o?u=new i(u.x+(g.x-u.x)*((o-u.y)/(g.y-u.y)),o)._round():g.y>=o&&(g=new i(u.x+(g.x-u.x)*((o-u.y)/(g.y-u.y)),o)._round()),l&&u.equals(l[l.length-1])||(l=[u],a.push(l)),l.push(g)))))}return a}(e.geometry,0,0,co,co);_<D.length;_+=1)for(var M=D[_],k=0,S=Al(M,C,E,n.vertical||h,r,24,p,t.overscaling,co);k<S.length;k+=1){var L=S[k];h&&xl(t,h.text,w,L)||x(M,L)}else if("line-center"===Q)for(var R=0,N=e.geometry;R<N.length;R+=1){var F=N[R];if(F.length>1){var T=sl(F,E,n.vertical||h,r,24,p);T&&x(F,T)}}else if("Polygon"===e.type)for(var U=0,O=Qa(e.geometry,0);U<O.length;U+=1){var G=O[U],H=gl(G,16);x(G[0],new ms(H.x,H.y,0))}else if("LineString"===e.type)for(var j=0,Y=e.geometry;j<Y.length;j+=1){var P=Y[j];x(P,new ms(P[0].x,P[0].y,0))}else if("Point"===e.type)for(var K=0,z=e.geometry;K<z.length;K+=1)for(var J=0,q=z[K];J<q.length;J+=1){var V=q[J];x([V],new ms(V.x,V.y,0))}}var bl=65535;function Ql(t,e,n,r,o,a,s,A,l,c,u,g,h){var d=function(t,e,n,r,o,a,s){for(var A=r.layout.get("text-rotate").evaluate(a,{})*Math.PI/180,l=e.positionedGlyphs,c=[],u=0;u<l.length;u++){var g=l[u],h=s[g.fontStack],d=h&&h[g.glyph];if(d){var f=d.rect;if(f){var p=mA+1,I=d.metrics.advance*g.scale/2,C=o?[g.x+I,g.y]:[0,0],m=o?[0,0]:[g.x+I+n[0],g.y+n[1]],y=(d.metrics.left-p)*g.scale-I+m[0],E=(-d.metrics.top-p)*g.scale+m[1],v=y+f.w*g.scale,B=E+f.h*g.scale,b=new i(y,E),Q=new i(v,E),w=new i(y,B),x=new i(v,B);if(o&&g.vertical){var _=new i(-I,I),D=-Math.PI/2,M=new i(5,0);b._rotateAround(D,_)._add(M),Q._rotateAround(D,_)._add(M),w._rotateAround(D,_)._add(M),x._rotateAround(D,_)._add(M)}if(A){var k=Math.sin(A),S=Math.cos(A),L=[S,-k,k,S];b._matMult(L),Q._matMult(L),w._matMult(L),x._matMult(L)}c.push({tl:b,tr:Q,bl:w,br:x,tex:f,writingMode:e.writingMode,glyphOffset:C})}}}return c}(0,n,s,r,o,a,g),f=t.textSizeData,p=null;"source"===f.kind?(p=[ys*r.layout.get("text-size").evaluate(a,{})])[0]>bl&&v(t.layerIds[0]+': Value for "text-size" is >= 256. Reduce your "text-size".'):"composite"===f.kind&&((p=[ys*h.compositeTextSizes[0].evaluate(a,{}),ys*h.compositeTextSizes[1].evaluate(a,{})])[0]>bl||p[1]>bl)&&v(t.layerIds[0]+': Value for "text-size" is >= 256. Reduce your "text-size".'),t.addSymbols(t.text,d,p,s,o,a,l,e,A.lineStartIndex,A.lineLength);for(var I=0,C=c;I<C.length;I+=1)u[C[I]]=t.text.placedSymbolArray.length-1;return 4*d.length}function wl(t){for(var e in t)return t[e];return null}function xl(t,e,n,r){var i=t.compareText;if(e in i){for(var o=i[e],a=o.length-1;a>=0;a--)if(r.dist(o[a])<n)return!0}else i[e]=[];return i[e].push(r),!1}t.Actor=yA,t.AlphaImage=Yo,t.CanonicalTileID=MA,t.CollisionBoxArray=Di,t.Color=ne,t.DEMData=RA,t.DataConstantProperty=Vr,t.DictionaryCoder=FA,t.EXTENT=co,t.ErrorEvent=kt,t.EvaluationParameters=Gr,t.Event=Mt,t.Evented=St,t.FeatureIndex=GA,t.FillBucket=Da,t.FillExtrusionBucket=Ka,t.ImageAtlas=Ps,t.ImagePosition=js,t.LineBucket=os,t.LngLat=BA,t.LngLatBounds=vA,t.MercatorCoordinate=DA,t.ONE_EM=PA,t.OverscaledTileID=SA,t.Point=i,t.Point$1=i,t.ProgramConfiguration=ao,t.Properties=ti,t.Protobuf=Zs,t.RGBAImage=Po,t.RequestManager=G,t.ResourceType=Ct,t.SegmentVector=Hi,t.SourceFeatureState=OA,t.StructArrayLayout1ui2=Qi,t.StructArrayLayout2i4=si,t.StructArrayLayout2ui4=bi,t.StructArrayLayout3ui6=Bi,t.StructArrayLayout4i8=Ai,t.SymbolBucket=ks,t.Texture=Vs,t.Tile=YA,t.Transitionable=Yr,t.Uniform1f=Ji,t.Uniform1i=zi,t.Uniform2f=qi,t.Uniform3f=Vi,t.Uniform4f=Wi,t.UniformColor=Xi,t.UniformMatrix4f=$i,t.UnwrappedTileID=kA,t.ValidationError=Rt,t.WritingMode=KA,t.ZoomHistory=wr,t.addDynamicAttributes=_s,t.asyncAll=function(t,e,n){if(!t.length)return n(null,[]);var r=t.length,i=new Array(t.length),o=null;t.forEach(function(t,a){e(t,function(t,e){t&&(o=t),i[a]=e,0==--r&&n(o,i)})})},t.bezier=s,t.bindAll=p,t.browser=L,t.cacheEntryPossiblyAdded=function(t){++It>dt&&(t.send("enforceCacheSizeLimit",ht),It=0)},t.clamp=l,t.clearTileCache=function(t){var e=self.caches.delete(gt);t&&e.catch(t).then(function(){return t()})},t.clone=function(t){var e=new ko(16);return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[3],e[4]=t[4],e[5]=t[5],e[6]=t[6],e[7]=t[7],e[8]=t[8],e[9]=t[9],e[10]=t[10],e[11]=t[11],e[12]=t[12],e[13]=t[13],e[14]=t[14],e[15]=t[15],e},t.clone$1=y,t.config=R,t.create=function(){var t=new ko(16);return ko!=Float32Array&&(t[1]=0,t[2]=0,t[3]=0,t[4]=0,t[6]=0,t[7]=0,t[8]=0,t[9]=0,t[11]=0,t[12]=0,t[13]=0,t[14]=0),t[0]=1,t[5]=1,t[10]=1,t[15]=1,t},t.create$1=So,t.create$2=function(){var t=new ko(4);return ko!=Float32Array&&(t[1]=0,t[2]=0),t[0]=1,t[3]=1,t},t.createCommonjsModule=e,t.createExpression=Sn,t.createLayout=oi,t.createStyleLayer=function(t){return"custom"===t.type?new Os(t):new Gs[t.type](t)},t.deepEqual=a,t.ease=A,t.emitValidationErrors=pr,t.endsWith=I,t.enforceCacheSizeLimit=function(t){self.caches&&self.caches.open(gt).then(function(e){e.keys().then(function(n){for(var r=0;r<n.length-t;r++)e.delete(n[r])})})},t.evaluateRadialOffset=El,t.evaluateSizeForFeature=vs,t.evaluateSizeForZoom=Bs,t.evented=Ur,t.extend=u,t.featureFilter=Kn,t.filterObject=m,t.fromRotation=function(t,e){var n=Math.sin(e),r=Math.cos(e);return t[0]=r,t[1]=n,t[2]=0,t[3]=-n,t[4]=r,t[5]=0,t[6]=0,t[7]=0,t[8]=1,t},t.getAnchorAlignment=tl,t.getAnchorJustification=vl,t.getArrayBuffer=Qt,t.getImage=xt,t.getJSON=function(t,e){return bt(u(t,{type:"json"}),e)},t.getReferrer=Bt,t.getVideo=function(t,e){var n,r,i=self.document.createElement("video");i.muted=!0,i.onloadstart=function(){e(null,i)};for(var o=0;o<t.length;o++){var a=self.document.createElement("source");n=t[o],r=void 0,(r=self.document.createElement("a")).href=n,(r.protocol!==self.document.location.protocol||r.host!==self.document.location.host)&&(i.crossOrigin="Anonymous"),a.src=t[o],i.appendChild(a)}return{cancel:function(){}}},t.identity=function(t){return t[0]=1,t[1]=0,t[2]=0,t[3]=0,t[4]=0,t[5]=1,t[6]=0,t[7]=0,t[8]=0,t[9]=0,t[10]=1,t[11]=0,t[12]=0,t[13]=0,t[14]=0,t[15]=1,t},t.invert=function(t,e){var n=e[0],r=e[1],i=e[2],o=e[3],a=e[4],s=e[5],A=e[6],l=e[7],c=e[8],u=e[9],g=e[10],h=e[11],d=e[12],f=e[13],p=e[14],I=e[15],C=n*s-r*a,m=n*A-i*a,y=n*l-o*a,E=r*A-i*s,v=r*l-o*s,B=i*l-o*A,b=c*f-u*d,Q=c*p-g*d,w=c*I-h*d,x=u*p-g*f,_=u*I-h*f,D=g*I-h*p,M=C*D-m*_+y*x+E*w-v*Q+B*b;return M?(M=1/M,t[0]=(s*D-A*_+l*x)*M,t[1]=(i*_-r*D-o*x)*M,t[2]=(f*B-p*v+I*E)*M,t[3]=(g*v-u*B-h*E)*M,t[4]=(A*w-a*D-l*Q)*M,t[5]=(n*D-i*w+o*Q)*M,t[6]=(p*y-d*B-I*m)*M,t[7]=(c*B-g*y+h*m)*M,t[8]=(a*_-s*w+l*b)*M,t[9]=(r*w-n*_-o*b)*M,t[10]=(d*v-f*y+I*C)*M,t[11]=(u*y-c*v-h*C)*M,t[12]=(s*Q-a*x-A*b)*M,t[13]=(n*x-r*Q+i*b)*M,t[14]=(f*m-d*E-p*C)*M,t[15]=(c*E-u*m+g*C)*M,t):null},t.isChar=xr,t.isMapboxURL=Y,t.keysDifference=function(t,e){var n=[];for(var r in t)r in e||n.push(r);return n},t.makeRequest=bt,t.mapObject=C,t.mercatorXfromLng=QA,t.mercatorYfromLat=wA,t.mercatorZfromAltitude=xA,t.multiply=function(t,e,n){var r=e[0],i=e[1],o=e[2],a=e[3],s=e[4],A=e[5],l=e[6],c=e[7],u=e[8],g=e[9],h=e[10],d=e[11],f=e[12],p=e[13],I=e[14],C=e[15],m=n[0],y=n[1],E=n[2],v=n[3];return t[0]=m*r+y*s+E*u+v*f,t[1]=m*i+y*A+E*g+v*p,t[2]=m*o+y*l+E*h+v*I,t[3]=m*a+y*c+E*d+v*C,m=n[4],y=n[5],E=n[6],v=n[7],t[4]=m*r+y*s+E*u+v*f,t[5]=m*i+y*A+E*g+v*p,t[6]=m*o+y*l+E*h+v*I,t[7]=m*a+y*c+E*d+v*C,m=n[8],y=n[9],E=n[10],v=n[11],t[8]=m*r+y*s+E*u+v*f,t[9]=m*i+y*A+E*g+v*p,t[10]=m*o+y*l+E*h+v*I,t[11]=m*a+y*c+E*d+v*C,m=n[12],y=n[13],E=n[14],v=n[15],t[12]=m*r+y*s+E*u+v*f,t[13]=m*i+y*A+E*g+v*p,t[14]=m*o+y*l+E*h+v*I,t[15]=m*a+y*c+E*d+v*C,t},t.mvt=Ha,t.number=xe,t.ortho=function(t,e,n,r,i,o,a){var s=1/(e-n),A=1/(r-i),l=1/(o-a);return t[0]=-2*s,t[1]=0,t[2]=0,t[3]=0,t[4]=0,t[5]=-2*A,t[6]=0,t[7]=0,t[8]=0,t[9]=0,t[10]=2*l,t[11]=0,t[12]=(e+n)*s,t[13]=(i+r)*A,t[14]=(a+o)*l,t[15]=1,t},t.parseGlyphPBF=function(t){return new Zs(t).readFields(pA,[])},t.pbf=Zs,t.performSymbolLayout=function(t,e,n,r,i,o){t.createArrays();var a=512*t.overscaling;t.tilePixelRatio=co/a,t.compareText={},t.iconsNeedLinear=!1;var s=t.layers[0].layout,A=t.layers[0]._unevaluatedLayout._values,l={};if("composite"===t.textSizeData.kind){var c=t.textSizeData,u=c.minZoom,g=c.maxZoom;l.compositeTextSizes=[A["text-size"].possiblyEvaluate(new Gr(u)),A["text-size"].possiblyEvaluate(new Gr(g))]}if("composite"===t.iconSizeData.kind){var h=t.iconSizeData,d=h.minZoom,f=h.maxZoom;l.compositeIconSizes=[A["icon-size"].possiblyEvaluate(new Gr(d)),A["icon-size"].possiblyEvaluate(new Gr(f))]}l.layoutTextSize=A["text-size"].possiblyEvaluate(new Gr(t.zoom+1)),l.layoutIconSize=A["icon-size"].possiblyEvaluate(new Gr(t.zoom+1)),l.textMaxSize=A["text-size"].possiblyEvaluate(new Gr(18));for(var p=s.get("text-line-height")*PA,I="map"===s.get("text-rotation-alignment")&&"point"!==s.get("symbol-placement"),C=s.get("text-keep-upright"),m=0,y=t.features;m<y.length;m+=1){var E=y[m],B=s.get("text-font").evaluate(E,{}).join(","),b=n,Q={horizontal:{},vertical:void 0},w=E.text,x=[0,0];if(w){var _=w.toString(),D=s.get("text-letter-spacing").evaluate(E,{})*PA,M=Dr(_)?D:0,k=s.get("text-anchor").evaluate(E,{}),S=s.get("text-variable-anchor"),L=s.get("text-radial-offset").evaluate(E,{});S||(x=L?El(k,L*PA):s.get("text-offset").evaluate(E,{}).map(function(t){return t*PA}));var R=I?"center":s.get("text-justify").evaluate(E,{}),N="point"===s.get("symbol-placement")?s.get("text-max-width").evaluate(E,{})*PA:0;if(!I&&S)for(var F="auto"===R?S.map(function(t){return vl(t)}):[R],T=!1,U=0;U<F.length;U++){var O=F[U];if(!Q.horizontal[O])if(T)Q.horizontal[O]=Q.horizontal[0];else{var G=JA(w,e,B,N,p,"center",O,M,x,KA.horizontal);G&&(Q.horizontal[O]=G,T=1===G.lineCount)}}else{"auto"===R&&(R=vl(k));var H=JA(w,e,B,N,p,k,R,M,x,KA.horizontal);H&&(Q.horizontal[R]=H),_r(_)&&I&&C&&(Q.vertical=JA(w,e,B,N,p,k,R,M,x,KA.vertical))}}var j=void 0;if(E.icon){var Y=r[E.icon];Y&&(j=nl(i[E.icon],s.get("icon-offset").evaluate(E,{}),s.get("icon-anchor").evaluate(E,{})),void 0===t.sdfIcons?t.sdfIcons=Y.sdf:t.sdfIcons!==Y.sdf&&v("Style sheet warning: Cannot mix SDF and non-SDF icons in one buffer"),Y.pixelRatio!==t.pixelRatio?t.iconsNeedLinear=!0:0!==s.get("icon-rotate").constantOr(1)&&(t.iconsNeedLinear=!0))}(Object.keys(Q.horizontal).length||j)&&Bl(t,E,Q,j,b,l,x)}o&&t.generateCollisionDebugBuffers()},t.perspective=function(t,e,n,r,i){var o,a=1/Math.tan(e/2);return t[0]=a/n,t[1]=0,t[2]=0,t[3]=0,t[4]=0,t[5]=a,t[6]=0,t[7]=0,t[8]=0,t[9]=0,t[11]=-1,t[12]=0,t[13]=0,t[15]=0,null!=i&&i!==1/0?(o=1/(r-i),t[10]=(i+r)*o,t[14]=2*i*r*o):(t[10]=-1,t[14]=-2*r),t},t.pick=function(t,e){for(var n={},r=0;r<e.length;r++){var i=e[r];i in t&&(n[i]=t[i])}return n},t.plugin=Or,t.polygonIntersectsPolygon=po,t.postMapLoadEvent=ut,t.postTurnstileEvent=lt,t.potpack=Hs,t.rasterBoundsAttributes=NA,t.refProperties=["type","source","source-layer","minzoom","maxzoom","filter","layout"],t.register=vr,t.registerForPluginAvailability=function(t){return Fr?t({pluginURL:Fr,completionCallback:Rr}):Ur.once("pluginAvailable",t),t},t.rotate=function(t,e,n){var r=e[0],i=e[1],o=e[2],a=e[3],s=Math.sin(n),A=Math.cos(n);return t[0]=r*A+o*s,t[1]=i*A+a*s,t[2]=r*-s+o*A,t[3]=i*-s+a*A,t},t.rotateX=function(t,e,n){var r=Math.sin(n),i=Math.cos(n),o=e[4],a=e[5],s=e[6],A=e[7],l=e[8],c=e[9],u=e[10],g=e[11];return e!==t&&(t[0]=e[0],t[1]=e[1],t[2]=e[2],t[3]=e[3],t[12]=e[12],t[13]=e[13],t[14]=e[14],t[15]=e[15]),t[4]=o*i+l*r,t[5]=a*i+c*r,t[6]=s*i+u*r,t[7]=A*i+g*r,t[8]=l*i-o*r,t[9]=c*i-a*r,t[10]=u*i-s*r,t[11]=g*i-A*r,t},t.rotateZ=function(t,e,n){var r=Math.sin(n),i=Math.cos(n),o=e[0],a=e[1],s=e[2],A=e[3],l=e[4],c=e[5],u=e[6],g=e[7];return e!==t&&(t[8]=e[8],t[9]=e[9],t[10]=e[10],t[11]=e[11],t[12]=e[12],t[13]=e[13],t[14]=e[14],t[15]=e[15]),t[0]=o*i+l*r,t[1]=a*i+c*r,t[2]=s*i+u*r,t[3]=A*i+g*r,t[4]=l*i-o*r,t[5]=c*i-a*r,t[6]=u*i-s*r,t[7]=g*i-A*r,t},t.scale=function(t,e,n){var r=n[0],i=n[1],o=n[2];return t[0]=e[0]*r,t[1]=e[1]*r,t[2]=e[2]*r,t[3]=e[3]*r,t[4]=e[4]*i,t[5]=e[5]*i,t[6]=e[6]*i,t[7]=e[7]*i,t[8]=e[8]*o,t[9]=e[9]*o,t[10]=e[10]*o,t[11]=e[11]*o,t[12]=e[12],t[13]=e[13],t[14]=e[14],t[15]=e[15],t},t.setCacheLimits=function(t,e){ht=t,dt=e},t.setRTLTextPlugin=function(t,e){if(Nr)throw new Error("setRTLTextPlugin cannot be called multiple times.");Nr=!0,Fr=L.resolveURL(t),Rr=function(t){t?(Nr=!1,Fr=null,e&&e(t)):Tr=!0},Ur.fire(new Mt("pluginAvailable",{pluginURL:Fr,completionCallback:Rr}))},t.sphericalToCartesian=function(t){var e=t[0],n=t[1],r=t[2];return n+=90,n*=Math.PI/180,r*=Math.PI/180,{x:e*Math.cos(n)*Math.sin(r),y:e*Math.sin(n)*Math.sin(r),z:e*Math.cos(r)}},t.styleSpec=Lt,t.symbolSize=bs,t.transformMat3=function(t,e,n){var r=e[0],i=e[1],o=e[2];return t[0]=r*n[0]+i*n[3]+o*n[6],t[1]=r*n[1]+i*n[4]+o*n[7],t[2]=r*n[2]+i*n[5]+o*n[8],t},t.transformMat4=No,t.translate=function(t,e,n){var r,i,o,a,s,A,l,c,u,g,h,d,f=n[0],p=n[1],I=n[2];return e===t?(t[12]=e[0]*f+e[4]*p+e[8]*I+e[12],t[13]=e[1]*f+e[5]*p+e[9]*I+e[13],t[14]=e[2]*f+e[6]*p+e[10]*I+e[14],t[15]=e[3]*f+e[7]*p+e[11]*I+e[15]):(r=e[0],i=e[1],o=e[2],a=e[3],s=e[4],A=e[5],l=e[6],c=e[7],u=e[8],g=e[9],h=e[10],d=e[11],t[0]=r,t[1]=i,t[2]=o,t[3]=a,t[4]=s,t[5]=A,t[6]=l,t[7]=c,t[8]=u,t[9]=g,t[10]=h,t[11]=d,t[12]=r*f+s*p+u*I+e[12],t[13]=i*f+A*p+g*I+e[13],t[14]=o*f+l*p+h*I+e[14],t[15]=a*f+c*p+d*I+e[15]),t},t.uniqueId=h,t.validateCustomStyleLayer=function(t){var e=[],n=t.id;return void 0===n&&e.push({message:"layers."+n+': missing required property "id"'}),void 0===t.render&&e.push({message:"layers."+n+': missing required method "render"'}),t.renderingMode&&"2d"!==t.renderingMode&&"3d"!==t.renderingMode&&e.push({message:"layers."+n+': property "renderingMode" must be either "2d" or "3d"'}),e},t.validateLight=hr,t.validateStyle=gr,t.values=function(t){var e=[];for(var n in t)e.push(t[n]);return e},t.vectorTile=Ha,t.version="1.1.1",t.warnOnce=v,t.webpSupported=N,t.window=self,t.wrap=c}),r(0,function(t){function e(t){var n=typeof t;if("number"===n||"boolean"===n||"string"===n||null==t)return JSON.stringify(t);if(Array.isArray(t)){for(var r="[",i=0,o=t;i<o.length;i+=1)r+=e(o[i])+",";return r+"]"}for(var a=Object.keys(t).sort(),s="{",A=0;A<a.length;A++)s+=JSON.stringify(a[A])+":"+e(t[a[A]])+",";return s+"}"}function n(n){for(var r="",i=0,o=t.refProperties;i<o.length;i+=1)r+="/"+e(n[o[i]]);return r}var r=function(t){this.keyCache={},t&&this.replace(t)};r.prototype.replace=function(t){this._layerConfigs={},this._layers={},this.update(t,[])},r.prototype.update=function(e,r){for(var i=this,o=0,a=e;o<a.length;o+=1){var s=a[o];this._layerConfigs[s.id]=s;var A=this._layers[s.id]=t.createStyleLayer(s);A._featureFilter=t.featureFilter(A.filter),this.keyCache[s.id]&&delete this.keyCache[s.id]}for(var l=0,c=r;l<c.length;l+=1){var u=c[l];delete this.keyCache[u],delete this._layerConfigs[u],delete this._layers[u]}this.familiesBySource={};for(var g=0,h=function(t,e){for(var r={},i=0;i<t.length;i++){var o=e&&e[t[i].id]||n(t[i]);e&&(e[t[i].id]=o);var a=r[o];a||(a=r[o]=[]),a.push(t[i])}var s=[];for(var A in r)s.push(r[A]);return s}(t.values(this._layerConfigs),this.keyCache);g<h.length;g+=1){var d=h[g].map(function(t){return i._layers[t.id]}),f=d[0];if("none"!==f.visibility){var p=f.source||"",I=this.familiesBySource[p];I||(I=this.familiesBySource[p]={});var C=f.sourceLayer||"_geojsonTileLayer",m=I[C];m||(m=I[C]=[]),m.push(d)}}};var i=function(e){var n={},r=[];for(var i in e){var o=e[i],a=n[i]={};for(var s in o){var A=o[+s];if(A&&0!==A.bitmap.width&&0!==A.bitmap.height){var l={x:0,y:0,w:A.bitmap.width+2,h:A.bitmap.height+2};r.push(l),a[s]={rect:l,metrics:A.metrics}}}}var c=t.potpack(r),u=c.w,g=c.h,h=new t.AlphaImage({width:u||1,height:g||1});for(var d in e){var f=e[d];for(var p in f){var I=f[+p];if(I&&0!==I.bitmap.width&&0!==I.bitmap.height){var C=n[d][p].rect;t.AlphaImage.copy(I.bitmap,h,{x:0,y:0},{x:C.x+1,y:C.y+1},I.bitmap)}}}this.image=h,this.positions=n};t.register("GlyphAtlas",i);var o=function(e){this.tileID=new t.OverscaledTileID(e.tileID.overscaledZ,e.tileID.wrap,e.tileID.canonical.z,e.tileID.canonical.x,e.tileID.canonical.y),this.uid=e.uid,this.zoom=e.zoom,this.pixelRatio=e.pixelRatio,this.tileSize=e.tileSize,this.source=e.source,this.overscaling=this.tileID.overscaleFactor(),this.showCollisionBoxes=e.showCollisionBoxes,this.collectResourceTiming=!!e.collectResourceTiming,this.returnDependencies=!!e.returnDependencies};function a(e,n){for(var r=new t.EvaluationParameters(n),i=0,o=e;i<o.length;i+=1)o[i].recalculate(r)}o.prototype.parse=function(e,n,r,o){var s=this;this.status="parsing",this.data=e,this.collisionBoxArray=new t.CollisionBoxArray;var A=new t.DictionaryCoder(Object.keys(e.layers).sort()),l=new t.FeatureIndex(this.tileID);l.bucketLayerIDs=[];var c,u,g,h,d={},f={featureIndex:l,iconDependencies:{},patternDependencies:{},glyphDependencies:{}},p=n.familiesBySource[this.source];for(var I in p){var C=e.layers[I];if(C){1===C.version&&t.warnOnce('Vector tile source "'+this.source+'" layer "'+I+'" does not use vector tile spec v2 and therefore may have some rendering errors.');for(var m=A.encode(I),y=[],E=0;E<C.length;E++){var v=C.feature(E);y.push({feature:v,index:E,sourceLayerIndex:m})}for(var B=0,b=p[I];B<b.length;B+=1){var Q=b[B],w=Q[0];w.minzoom&&this.zoom<Math.floor(w.minzoom)||w.maxzoom&&this.zoom>=w.maxzoom||"none"!==w.visibility&&(a(Q,this.zoom),(d[w.id]=w.createBucket({index:l.bucketLayerIDs.length,layers:Q,zoom:this.zoom,pixelRatio:this.pixelRatio,overscaling:this.overscaling,collisionBoxArray:this.collisionBoxArray,sourceLayerIndex:m,sourceID:this.source})).populate(y,f),l.bucketLayerIDs.push(Q.map(function(t){return t.id})))}}}var x=t.mapObject(f.glyphDependencies,function(t){return Object.keys(t).map(Number)});Object.keys(x).length?r.send("getGlyphs",{uid:this.uid,stacks:x},function(t,e){c||(c=t,u=e,M.call(s))}):u={};var _=Object.keys(f.iconDependencies);_.length?r.send("getImages",{icons:_},function(t,e){c||(c=t,g=e,M.call(s))}):g={};var D=Object.keys(f.patternDependencies);function M(){if(c)return o(c);if(u&&g&&h){var e=new i(u),n=new t.ImageAtlas(g,h);for(var r in d){var s=d[r];s instanceof t.SymbolBucket?(a(s.layers,this.zoom),t.performSymbolLayout(s,u,e.positions,g,n.iconPositions,this.showCollisionBoxes)):s.hasPattern&&(s instanceof t.LineBucket||s instanceof t.FillBucket||s instanceof t.FillExtrusionBucket)&&(a(s.layers,this.zoom),s.addFeatures(f,n.patternPositions))}this.status="done",o(null,{buckets:t.values(d).filter(function(t){return!t.isEmpty()}),featureIndex:l,collisionBoxArray:this.collisionBoxArray,glyphAtlasImage:e.image,imageAtlas:n,glyphMap:this.returnDependencies?u:null,iconMap:this.returnDependencies?g:null,glyphPositions:this.returnDependencies?e.positions:null})}}D.length?r.send("getImages",{icons:D},function(t,e){c||(c=t,h=e,M.call(s))}):h={},M.call(this)};var s="undefined"!=typeof performance,A={getEntriesByName:function(t){return!!(s&&performance&&performance.getEntriesByName)&&performance.getEntriesByName(t)},mark:function(t){return!!(s&&performance&&performance.mark)&&performance.mark(t)},measure:function(t,e,n){return!!(s&&performance&&performance.measure)&&performance.measure(t,e,n)},clearMarks:function(t){return!!(s&&performance&&performance.clearMarks)&&performance.clearMarks(t)},clearMeasures:function(t){return!!(s&&performance&&performance.clearMeasures)&&performance.clearMeasures(t)}},l=function(t){this._marks={start:[t.url,"start"].join("#"),end:[t.url,"end"].join("#"),measure:t.url.toString()},A.mark(this._marks.start)};function c(e,n){var r=t.getArrayBuffer(e.request,function(e,r,i,o){e?n(e):r&&n(null,{vectorTile:new t.vectorTile.VectorTile(new t.pbf(r)),rawData:r,cacheControl:i,expires:o})});return function(){r.cancel(),n()}}l.prototype.finish=function(){A.mark(this._marks.end);var t=A.getEntriesByName(this._marks.measure);return 0===t.length&&(A.measure(this._marks.measure,this._marks.start,this._marks.end),t=A.getEntriesByName(this._marks.measure),A.clearMarks(this._marks.start),A.clearMarks(this._marks.end),A.clearMeasures(this._marks.measure)),t},A.Performance=l;var u=function(t,e,n){this.actor=t,this.layerIndex=e,this.loadVectorData=n||c,this.loading={},this.loaded={}};u.prototype.loadTile=function(e,n){var r=this,i=e.uid;this.loading||(this.loading={});var a=!!(e&&e.request&&e.request.collectResourceTiming)&&new A.Performance(e.request),s=this.loading[i]=new o(e);s.abort=this.loadVectorData(e,function(e,o){if(delete r.loading[i],e||!o)return s.status="done",r.loaded[i]=s,n(e);var A=o.rawData,l={};o.expires&&(l.expires=o.expires),o.cacheControl&&(l.cacheControl=o.cacheControl);var c={};if(a){var u=a.finish();u&&(c.resourceTiming=JSON.parse(JSON.stringify(u)))}s.vectorTile=o.vectorTile,s.parse(o.vectorTile,r.layerIndex,r.actor,function(e,r){if(e||!r)return n(e);n(null,t.extend({rawTileData:A.slice(0)},r,l,c))}),r.loaded=r.loaded||{},r.loaded[i]=s})},u.prototype.reloadTile=function(t,e){var n=this.loaded,r=t.uid,i=this;if(n&&n[r]){var o=n[r];o.showCollisionBoxes=t.showCollisionBoxes;var a=function(t,n){var r=o.reloadCallback;r&&(delete o.reloadCallback,o.parse(o.vectorTile,i.layerIndex,i.actor,r)),e(t,n)};"parsing"===o.status?o.reloadCallback=a:"done"===o.status&&(o.vectorTile?o.parse(o.vectorTile,this.layerIndex,this.actor,a):a())}},u.prototype.abortTile=function(t,e){var n=this.loading,r=t.uid;n&&n[r]&&n[r].abort&&(n[r].abort(),delete n[r]),e()},u.prototype.removeTile=function(t,e){var n=this.loaded,r=t.uid;n&&n[r]&&delete n[r],e()};var g=function(){this.loaded={}};g.prototype.loadTile=function(e,n){var r=e.uid,i=e.encoding,o=e.rawImageData,a=new t.DEMData(r,o,i);this.loaded=this.loaded||{},this.loaded[r]=a,n(null,a)},g.prototype.removeTile=function(t){var e=this.loaded,n=t.uid;e&&e[n]&&delete e[n]};var h={RADIUS:6378137,FLATTENING:1/298.257223563,POLAR_RADIUS:6356752.3142};function d(t){var e=0;if(t&&t.length>0){e+=Math.abs(f(t[0]));for(var n=1;n<t.length;n++)e-=Math.abs(f(t[n]))}return e}function f(t){var e,n,r,i,o,a,s=0,A=t.length;if(A>2){for(a=0;a<A;a++)a===A-2?(r=A-2,i=A-1,o=0):a===A-1?(r=A-1,i=0,o=1):(r=a,i=a+1,o=a+2),e=t[r],n=t[i],s+=(p(t[o][0])-p(e[0]))*Math.sin(p(n[1]));s=s*h.RADIUS*h.RADIUS/2}return s}function p(t){return t*Math.PI/180}var I={geometry:function t(e){var n,r=0;switch(e.type){case"Polygon":return d(e.coordinates);case"MultiPolygon":for(n=0;n<e.coordinates.length;n++)r+=d(e.coordinates[n]);return r;case"Point":case"MultiPoint":case"LineString":case"MultiLineString":return 0;case"GeometryCollection":for(n=0;n<e.geometries.length;n++)r+=t(e.geometries[n]);return r}},ring:f};function C(t,e){return function(n){return t(n,e)}}function m(t,e){e=!!e,t[0]=y(t[0],e);for(var n=1;n<t.length;n++)t[n]=y(t[n],!e);return t}function y(t,e){return function(t){return I.ring(t)>=0}(t)===e?t:t.reverse()}var E=t.vectorTile.VectorTileFeature.prototype.toGeoJSON,v=function(e){this._feature=e,this.extent=t.EXTENT,this.type=e.type,this.properties=e.tags,"id"in e&&!isNaN(e.id)&&(this.id=parseInt(e.id,10))};v.prototype.loadGeometry=function(){if(1===this._feature.type){for(var e=[],n=0,r=this._feature.geometry;n<r.length;n+=1){var i=r[n];e.push([new t.Point$1(i[0],i[1])])}return e}for(var o=[],a=0,s=this._feature.geometry;a<s.length;a+=1){for(var A=[],l=0,c=s[a];l<c.length;l+=1){var u=c[l];A.push(new t.Point$1(u[0],u[1]))}o.push(A)}return o},v.prototype.toGeoJSON=function(t,e,n){return E.call(this,t,e,n)};var B=function(e){this.layers={_geojsonTileLayer:this},this.name="_geojsonTileLayer",this.extent=t.EXTENT,this.length=e.length,this._features=e};B.prototype.feature=function(t){return new v(this._features[t])};var b=t.vectorTile.VectorTileFeature,Q=w;function w(t,e){this.options=e||{},this.features=t,this.length=t.length}function x(t,e){this.id="number"==typeof t.id?t.id:void 0,this.type=t.type,this.rawGeometry=1===t.type?[t.geometry]:t.geometry,this.properties=t.tags,this.extent=e||4096}w.prototype.feature=function(t){return new x(this.features[t],this.options.extent)},x.prototype.loadGeometry=function(){var e=this.rawGeometry;this.geometry=[];for(var n=0;n<e.length;n++){for(var r=e[n],i=[],o=0;o<r.length;o++)i.push(new t.Point$1(r[o][0],r[o][1]));this.geometry.push(i)}return this.geometry},x.prototype.bbox=function(){this.geometry||this.loadGeometry();for(var t=this.geometry,e=1/0,n=-1/0,r=1/0,i=-1/0,o=0;o<t.length;o++)for(var a=t[o],s=0;s<a.length;s++){var A=a[s];e=Math.min(e,A.x),n=Math.max(n,A.x),r=Math.min(r,A.y),i=Math.max(i,A.y)}return[e,r,n,i]},x.prototype.toGeoJSON=b.prototype.toGeoJSON;var _=k,D=k,M=Q;function k(e){var n=new t.pbf;return function(t,e){for(var n in t.layers)e.writeMessage(3,S,t.layers[n])}(e,n),n.finish()}function S(t,e){var n;e.writeVarintField(15,t.version||1),e.writeStringField(1,t.name||""),e.writeVarintField(5,t.extent||4096);var r={keys:[],values:[],keycache:{},valuecache:{}};for(n=0;n<t.length;n++)r.feature=t.feature(n),e.writeMessage(2,L,r);var i=r.keys;for(n=0;n<i.length;n++)e.writeStringField(3,i[n]);var o=r.values;for(n=0;n<o.length;n++)e.writeMessage(4,U,o[n])}function L(t,e){var n=t.feature;void 0!==n.id&&e.writeVarintField(1,n.id),e.writeMessage(2,R,t),e.writeVarintField(3,n.type),e.writeMessage(4,T,n)}function R(t,e){var n=t.feature,r=t.keys,i=t.values,o=t.keycache,a=t.valuecache;for(var s in n.properties){var A=o[s];void 0===A&&(r.push(s),A=r.length-1,o[s]=A),e.writeVarint(A);var l=n.properties[s],c=typeof l;"string"!==c&&"boolean"!==c&&"number"!==c&&(l=JSON.stringify(l));var u=c+":"+l,g=a[u];void 0===g&&(i.push(l),g=i.length-1,a[u]=g),e.writeVarint(g)}}function N(t,e){return(e<<3)+(7&t)}function F(t){return t<<1^t>>31}function T(t,e){for(var n=t.loadGeometry(),r=t.type,i=0,o=0,a=n.length,s=0;s<a;s++){var A=n[s],l=1;1===r&&(l=A.length),e.writeVarint(N(1,l));for(var c=3===r?A.length-1:A.length,u=0;u<c;u++){1===u&&1!==r&&e.writeVarint(N(2,c-1));var g=A[u].x-i,h=A[u].y-o;e.writeVarint(F(g)),e.writeVarint(F(h)),i+=g,o+=h}3===r&&e.writeVarint(N(7,1))}}function U(t,e){var n=typeof t;"string"===n?e.writeStringField(1,t):"boolean"===n?e.writeBooleanField(7,t):"number"===n&&(t%1!=0?e.writeDoubleField(3,t):t<0?e.writeSVarintField(6,t):e.writeVarintField(5,t))}function O(t,e,n,r){G(t,n,r),G(e,2*n,2*r),G(e,2*n+1,2*r+1)}function G(t,e,n){var r=t[e];t[e]=t[n],t[n]=r}function H(t,e,n,r){var i=t-n,o=e-r;return i*i+o*o}_.fromVectorTileJs=D,_.fromGeojsonVt=function(t,e){e=e||{};var n={};for(var r in t)n[r]=new Q(t[r].features,e),n[r].name=r,n[r].version=e.version,n[r].extent=e.extent;return k({layers:n})},_.GeoJSONWrapper=M;var j=function(t){return t[0]},Y=function(t){return t[1]},P=function(t,e,n,r,i){void 0===e&&(e=j),void 0===n&&(n=Y),void 0===r&&(r=64),void 0===i&&(i=Float64Array),this.nodeSize=r,this.points=t;for(var o=t.length<65536?Uint16Array:Uint32Array,a=this.ids=new o(t.length),s=this.coords=new i(2*t.length),A=0;A<t.length;A++)a[A]=A,s[2*A]=e(t[A]),s[2*A+1]=n(t[A]);!function t(e,n,r,i,o,a){if(!(o-i<=r)){var s=i+o>>1;!function t(e,n,r,i,o,a){for(;o>i;){if(o-i>600){var s=o-i+1,A=r-i+1,l=Math.log(s),c=.5*Math.exp(2*l/3),u=.5*Math.sqrt(l*c*(s-c)/s)*(A-s/2<0?-1:1);t(e,n,r,Math.max(i,Math.floor(r-A*c/s+u)),Math.min(o,Math.floor(r+(s-A)*c/s+u)),a)}var g=n[2*r+a],h=i,d=o;for(O(e,n,i,r),n[2*o+a]>g&&O(e,n,i,o);h<d;){for(O(e,n,h,d),h++,d--;n[2*h+a]<g;)h++;for(;n[2*d+a]>g;)d--}n[2*i+a]===g?O(e,n,i,d):O(e,n,++d,o),d<=r&&(i=d+1),r<=d&&(o=d-1)}}(e,n,s,i,o,a%2),t(e,n,r,i,s-1,a+1),t(e,n,r,s+1,o,a+1)}}(a,s,r,0,a.length-1,0)};P.prototype.range=function(t,e,n,r){return function(t,e,n,r,i,o,a){for(var s,A,l=[0,t.length-1,0],c=[];l.length;){var u=l.pop(),g=l.pop(),h=l.pop();if(g-h<=a)for(var d=h;d<=g;d++)s=e[2*d],A=e[2*d+1],s>=n&&s<=i&&A>=r&&A<=o&&c.push(t[d]);else{var f=Math.floor((h+g)/2);s=e[2*f],A=e[2*f+1],s>=n&&s<=i&&A>=r&&A<=o&&c.push(t[f]);var p=(u+1)%2;(0===u?n<=s:r<=A)&&(l.push(h),l.push(f-1),l.push(p)),(0===u?i>=s:o>=A)&&(l.push(f+1),l.push(g),l.push(p))}}return c}(this.ids,this.coords,t,e,n,r,this.nodeSize)},P.prototype.within=function(t,e,n){return function(t,e,n,r,i,o){for(var a=[0,t.length-1,0],s=[],A=i*i;a.length;){var l=a.pop(),c=a.pop(),u=a.pop();if(c-u<=o)for(var g=u;g<=c;g++)H(e[2*g],e[2*g+1],n,r)<=A&&s.push(t[g]);else{var h=Math.floor((u+c)/2),d=e[2*h],f=e[2*h+1];H(d,f,n,r)<=A&&s.push(t[h]);var p=(l+1)%2;(0===l?n-i<=d:r-i<=f)&&(a.push(u),a.push(h-1),a.push(p)),(0===l?n+i>=d:r+i>=f)&&(a.push(h+1),a.push(c),a.push(p))}}return s}(this.ids,this.coords,t,e,n,this.nodeSize)};var K={minZoom:0,maxZoom:16,radius:40,extent:512,nodeSize:64,log:!1,reduce:null,map:function(t){return t}},z=function(t){this.options=$(Object.create(K),t),this.trees=new Array(this.options.maxZoom+1)};function J(t,e,n,r,i){return{x:t,y:e,zoom:1/0,id:n,parentId:-1,numPoints:r,properties:i}}function q(t,e){var n=t.geometry.coordinates,r=n[0],i=n[1];return{x:X(r),y:Z(i),zoom:1/0,index:e,parentId:-1}}function V(t){return{type:"Feature",id:t.id,properties:W(t),geometry:{type:"Point",coordinates:[(r=t.x,360*(r-.5)),(e=t.y,n=(180-360*e)*Math.PI/180,360*Math.atan(Math.exp(n))/Math.PI-90)]}};var e,n,r}function W(t){var e=t.numPoints,n=e>=1e4?Math.round(e/1e3)+"k":e>=1e3?Math.round(e/100)/10+"k":e;return $($({},t.properties),{cluster:!0,cluster_id:t.id,point_count:e,point_count_abbreviated:n})}function X(t){return t/360+.5}function Z(t){var e=Math.sin(t*Math.PI/180),n=.5-.25*Math.log((1+e)/(1-e))/Math.PI;return n<0?0:n>1?1:n}function $(t,e){for(var n in e)t[n]=e[n];return t}function tt(t){return t.x}function et(t){return t.y}function nt(t,e,n,r,i,o){var a=i-n,s=o-r;if(0!==a||0!==s){var A=((t-n)*a+(e-r)*s)/(a*a+s*s);A>1?(n=i,r=o):A>0&&(n+=a*A,r+=s*A)}return(a=t-n)*a+(s=e-r)*s}function rt(t,e,n,r){var i={id:void 0===t?null:t,type:e,geometry:n,tags:r,minX:1/0,minY:1/0,maxX:-1/0,maxY:-1/0};return function(t){var e=t.geometry,n=t.type;if("Point"===n||"MultiPoint"===n||"LineString"===n)it(t,e);else if("Polygon"===n||"MultiLineString"===n)for(var r=0;r<e.length;r++)it(t,e[r]);else if("MultiPolygon"===n)for(r=0;r<e.length;r++)for(var i=0;i<e[r].length;i++)it(t,e[r][i])}(i),i}function it(t,e){for(var n=0;n<e.length;n+=3)t.minX=Math.min(t.minX,e[n]),t.minY=Math.min(t.minY,e[n+1]),t.maxX=Math.max(t.maxX,e[n]),t.maxY=Math.max(t.maxY,e[n+1])}function ot(t,e,n,r){if(e.geometry){var i=e.geometry.coordinates,o=e.geometry.type,a=Math.pow(n.tolerance/((1<<n.maxZoom)*n.extent),2),s=[],A=e.id;if(n.promoteId?A=e.properties[n.promoteId]:n.generateId&&(A=r||0),"Point"===o)at(i,s);else if("MultiPoint"===o)for(var l=0;l<i.length;l++)at(i[l],s);else if("LineString"===o)st(i,s,a,!1);else if("MultiLineString"===o){if(n.lineMetrics){for(l=0;l<i.length;l++)s=[],st(i[l],s,a,!1),t.push(rt(A,"LineString",s,e.properties));return}At(i,s,a,!1)}else if("Polygon"===o)At(i,s,a,!0);else{if("MultiPolygon"!==o){if("GeometryCollection"===o){for(l=0;l<e.geometry.geometries.length;l++)ot(t,{id:A,geometry:e.geometry.geometries[l],properties:e.properties},n,r);return}throw new Error("Input data is not a valid GeoJSON object.")}for(l=0;l<i.length;l++){var c=[];At(i[l],c,a,!0),s.push(c)}}t.push(rt(A,o,s,e.properties))}}function at(t,e){e.push(lt(t[0])),e.push(ct(t[1])),e.push(0)}function st(t,e,n,r){for(var i,o,a=0,s=0;s<t.length;s++){var A=lt(t[s][0]),l=ct(t[s][1]);e.push(A),e.push(l),e.push(0),s>0&&(a+=r?(i*l-A*o)/2:Math.sqrt(Math.pow(A-i,2)+Math.pow(l-o,2))),i=A,o=l}var c=e.length-3;e[2]=1,function t(e,n,r,i){for(var o,a=i,s=r-n>>1,A=r-n,l=e[n],c=e[n+1],u=e[r],g=e[r+1],h=n+3;h<r;h+=3){var d=nt(e[h],e[h+1],l,c,u,g);if(d>a)o=h,a=d;else if(d===a){var f=Math.abs(h-s);f<A&&(o=h,A=f)}}a>i&&(o-n>3&&t(e,n,o,i),e[o+2]=a,r-o>3&&t(e,o,r,i))}(e,0,c,n),e[c+2]=1,e.size=Math.abs(a),e.start=0,e.end=e.size}function At(t,e,n,r){for(var i=0;i<t.length;i++){var o=[];st(t[i],o,n,r),e.push(o)}}function lt(t){return t/360+.5}function ct(t){var e=Math.sin(t*Math.PI/180),n=.5-.25*Math.log((1+e)/(1-e))/Math.PI;return n<0?0:n>1?1:n}function ut(t,e,n,r,i,o,a,s){if(r/=e,o>=(n/=e)&&a<r)return t;if(a<n||o>=r)return null;for(var A=[],l=0;l<t.length;l++){var c=t[l],u=c.geometry,g=c.type,h=0===i?c.minX:c.minY,d=0===i?c.maxX:c.maxY;if(h>=n&&d<r)A.push(c);else if(!(d<n||h>=r)){var f=[];if("Point"===g||"MultiPoint"===g)gt(u,f,n,r,i);else if("LineString"===g)ht(u,f,n,r,i,!1,s.lineMetrics);else if("MultiLineString"===g)ft(u,f,n,r,i,!1);else if("Polygon"===g)ft(u,f,n,r,i,!0);else if("MultiPolygon"===g)for(var p=0;p<u.length;p++){var I=[];ft(u[p],I,n,r,i,!0),I.length&&f.push(I)}if(f.length){if(s.lineMetrics&&"LineString"===g){for(p=0;p<f.length;p++)A.push(rt(c.id,g,f[p],c.tags));continue}"LineString"!==g&&"MultiLineString"!==g||(1===f.length?(g="LineString",f=f[0]):g="MultiLineString"),"Point"!==g&&"MultiPoint"!==g||(g=3===f.length?"Point":"MultiPoint"),A.push(rt(c.id,g,f,c.tags))}}}return A.length?A:null}function gt(t,e,n,r,i){for(var o=0;o<t.length;o+=3){var a=t[o+i];a>=n&&a<=r&&(e.push(t[o]),e.push(t[o+1]),e.push(t[o+2]))}}function ht(t,e,n,r,i,o,a){for(var s,A,l=dt(t),c=0===i?It:Ct,u=t.start,g=0;g<t.length-3;g+=3){var h=t[g],d=t[g+1],f=t[g+2],p=t[g+3],I=t[g+4],C=0===i?h:d,m=0===i?p:I,y=!1;a&&(s=Math.sqrt(Math.pow(h-p,2)+Math.pow(d-I,2))),C<n?m>n&&(A=c(l,h,d,p,I,n),a&&(l.start=u+s*A)):C>r?m<r&&(A=c(l,h,d,p,I,r),a&&(l.start=u+s*A)):pt(l,h,d,f),m<n&&C>=n&&(A=c(l,h,d,p,I,n),y=!0),m>r&&C<=r&&(A=c(l,h,d,p,I,r),y=!0),!o&&y&&(a&&(l.end=u+s*A),e.push(l),l=dt(t)),a&&(u+=s)}var E=t.length-3;h=t[E],d=t[E+1],f=t[E+2],(C=0===i?h:d)>=n&&C<=r&&pt(l,h,d,f),E=l.length-3,o&&E>=3&&(l[E]!==l[0]||l[E+1]!==l[1])&&pt(l,l[0],l[1],l[2]),l.length&&e.push(l)}function dt(t){var e=[];return e.size=t.size,e.start=t.start,e.end=t.end,e}function ft(t,e,n,r,i,o){for(var a=0;a<t.length;a++)ht(t[a],e,n,r,i,o,!1)}function pt(t,e,n,r){t.push(e),t.push(n),t.push(r)}function It(t,e,n,r,i,o){var a=(o-e)/(r-e);return t.push(o),t.push(n+(i-n)*a),t.push(1),a}function Ct(t,e,n,r,i,o){var a=(o-n)/(i-n);return t.push(e+(r-e)*a),t.push(o),t.push(1),a}function mt(t,e){for(var n=[],r=0;r<t.length;r++){var i,o=t[r],a=o.type;if("Point"===a||"MultiPoint"===a||"LineString"===a)i=yt(o.geometry,e);else if("MultiLineString"===a||"Polygon"===a){i=[];for(var s=0;s<o.geometry.length;s++)i.push(yt(o.geometry[s],e))}else if("MultiPolygon"===a)for(i=[],s=0;s<o.geometry.length;s++){for(var A=[],l=0;l<o.geometry[s].length;l++)A.push(yt(o.geometry[s][l],e));i.push(A)}n.push(rt(o.id,a,i,o.tags))}return n}function yt(t,e){var n=[];n.size=t.size,void 0!==t.start&&(n.start=t.start,n.end=t.end);for(var r=0;r<t.length;r+=3)n.push(t[r]+e,t[r+1],t[r+2]);return n}function Et(t,e){if(t.transformed)return t;var n,r,i,o=1<<t.z,a=t.x,s=t.y;for(n=0;n<t.features.length;n++){var A=t.features[n],l=A.geometry,c=A.type;if(A.geometry=[],1===c)for(r=0;r<l.length;r+=2)A.geometry.push(vt(l[r],l[r+1],e,o,a,s));else for(r=0;r<l.length;r++){var u=[];for(i=0;i<l[r].length;i+=2)u.push(vt(l[r][i],l[r][i+1],e,o,a,s));A.geometry.push(u)}}return t.transformed=!0,t}function vt(t,e,n,r,i,o){return[Math.round(n*(t*r-i)),Math.round(n*(e*r-o))]}function Bt(t,e,n,r,i){for(var o=e===i.maxZoom?0:i.tolerance/((1<<e)*i.extent),a={features:[],numPoints:0,numSimplified:0,numFeatures:0,source:null,x:n,y:r,z:e,transformed:!1,minX:2,minY:1,maxX:-1,maxY:0},s=0;s<t.length;s++){a.numFeatures++,bt(a,t[s],o,i);var A=t[s].minX,l=t[s].minY,c=t[s].maxX,u=t[s].maxY;A<a.minX&&(a.minX=A),l<a.minY&&(a.minY=l),c>a.maxX&&(a.maxX=c),u>a.maxY&&(a.maxY=u)}return a}function bt(t,e,n,r){var i=e.geometry,o=e.type,a=[];if("Point"===o||"MultiPoint"===o)for(var s=0;s<i.length;s+=3)a.push(i[s]),a.push(i[s+1]),t.numPoints++,t.numSimplified++;else if("LineString"===o)Qt(a,i,t,n,!1,!1);else if("MultiLineString"===o||"Polygon"===o)for(s=0;s<i.length;s++)Qt(a,i[s],t,n,"Polygon"===o,0===s);else if("MultiPolygon"===o)for(var A=0;A<i.length;A++){var l=i[A];for(s=0;s<l.length;s++)Qt(a,l[s],t,n,!0,0===s)}if(a.length){var c=e.tags||null;if("LineString"===o&&r.lineMetrics){for(var u in c={},e.tags)c[u]=e.tags[u];c.mapbox_clip_start=i.start/i.size,c.mapbox_clip_end=i.end/i.size}var g={geometry:a,type:"Polygon"===o||"MultiPolygon"===o?3:"LineString"===o||"MultiLineString"===o?2:1,tags:c};null!==e.id&&(g.id=e.id),t.features.push(g)}}function Qt(t,e,n,r,i,o){var a=r*r;if(r>0&&e.size<(i?a:r))n.numPoints+=e.length/3;else{for(var s=[],A=0;A<e.length;A+=3)(0===r||e[A+2]>a)&&(n.numSimplified++,s.push(e[A]),s.push(e[A+1])),n.numPoints++;i&&function(t,e){for(var n=0,r=0,i=t.length,o=i-2;r<i;o=r,r+=2)n+=(t[r]-t[o])*(t[r+1]+t[o+1]);if(n>0===e)for(r=0,i=t.length;r<i/2;r+=2){var a=t[r],s=t[r+1];t[r]=t[i-2-r],t[r+1]=t[i-1-r],t[i-2-r]=a,t[i-1-r]=s}}(s,o),t.push(s)}}function wt(t,e){var n=(e=this.options=function(t,e){for(var n in e)t[n]=e[n];return t}(Object.create(this.options),e)).debug;if(n&&console.time("preprocess data"),e.maxZoom<0||e.maxZoom>24)throw new Error("maxZoom should be in the 0-24 range");if(e.promoteId&&e.generateId)throw new Error("promoteId and generateId cannot be used together.");var r=function(t,e){var n=[];if("FeatureCollection"===t.type)for(var r=0;r<t.features.length;r++)ot(n,t.features[r],e,r);else"Feature"===t.type?ot(n,t,e):ot(n,{geometry:t},e);return n}(t,e);this.tiles={},this.tileCoords=[],n&&(console.timeEnd("preprocess data"),console.log("index: maxZoom: %d, maxPoints: %d",e.indexMaxZoom,e.indexMaxPoints),console.time("generate tiles"),this.stats={},this.total=0),(r=function(t,e){var n=e.buffer/e.extent,r=t,i=ut(t,1,-1-n,n,0,-1,2,e),o=ut(t,1,1-n,2+n,0,-1,2,e);return(i||o)&&(r=ut(t,1,-n,1+n,0,-1,2,e)||[],i&&(r=mt(i,1).concat(r)),o&&(r=r.concat(mt(o,-1)))),r}(r,e)).length&&this.splitTile(r,0,0,0),n&&(r.length&&console.log("features: %d, points: %d",this.tiles[0].numFeatures,this.tiles[0].numPoints),console.timeEnd("generate tiles"),console.log("tiles generated:",this.total,JSON.stringify(this.stats)))}function xt(t,e,n){return 32*((1<<t)*n+e)+t}function _t(t,e){var n=t.tileID.canonical;if(!this._geoJSONIndex)return e(null,null);var r=this._geoJSONIndex.getTile(n.z,n.x,n.y);if(!r)return e(null,null);var i=new B(r.features),o=_(i);0===o.byteOffset&&o.byteLength===o.buffer.byteLength||(o=new Uint8Array(o)),e(null,{vectorTile:i,rawData:o.buffer})}z.prototype.load=function(t){var e=this.options,n=e.log,r=e.minZoom,i=e.maxZoom,o=e.nodeSize;n&&console.time("total time");var a="prepare "+t.length+" points";n&&console.time(a),this.points=t;for(var s=[],A=0;A<t.length;A++)t[A].geometry&&s.push(q(t[A],A));this.trees[i+1]=new P(s,tt,et,o,Float32Array),n&&console.timeEnd(a);for(var l=i;l>=r;l--){var c=+Date.now();s=this._cluster(s,l),this.trees[l]=new P(s,tt,et,o,Float32Array),n&&console.log("z%d: %d clusters in %dms",l,s.length,+Date.now()-c)}return n&&console.timeEnd("total time"),this},z.prototype.getClusters=function(t,e){var n=((t[0]+180)%360+360)%360-180,r=Math.max(-90,Math.min(90,t[1])),i=180===t[2]?180:((t[2]+180)%360+360)%360-180,o=Math.max(-90,Math.min(90,t[3]));if(t[2]-t[0]>=360)n=-180,i=180;else if(n>i){var a=this.getClusters([n,r,180,o],e),s=this.getClusters([-180,r,i,o],e);return a.concat(s)}for(var A=this.trees[this._limitZoom(e)],l=[],c=0,u=A.range(X(n),Z(o),X(i),Z(r));c<u.length;c+=1){var g=u[c],h=A.points[g];l.push(h.numPoints?V(h):this.points[h.index])}return l},z.prototype.getChildren=function(t){var e=t>>5,n=t%32,r="No cluster with the specified id.",i=this.trees[n];if(!i)throw new Error(r);var o=i.points[e];if(!o)throw new Error(r);for(var a=this.options.radius/(this.options.extent*Math.pow(2,n-1)),s=[],A=0,l=i.within(o.x,o.y,a);A<l.length;A+=1){var c=l[A],u=i.points[c];u.parentId===t&&s.push(u.numPoints?V(u):this.points[u.index])}if(0===s.length)throw new Error(r);return s},z.prototype.getLeaves=function(t,e,n){e=e||10,n=n||0;var r=[];return this._appendLeaves(r,t,e,n,0),r},z.prototype.getTile=function(t,e,n){var r=this.trees[this._limitZoom(t)],i=Math.pow(2,t),o=this.options,a=o.extent,s=o.radius/a,A=(n-s)/i,l=(n+1+s)/i,c={features:[]};return this._addTileFeatures(r.range((e-s)/i,A,(e+1+s)/i,l),r.points,e,n,i,c),0===e&&this._addTileFeatures(r.range(1-s/i,A,1,l),r.points,i,n,i,c),e===i-1&&this._addTileFeatures(r.range(0,A,s/i,l),r.points,-1,n,i,c),c.features.length?c:null},z.prototype.getClusterExpansionZoom=function(t){for(var e=t%32-1;e<=this.options.maxZoom;){var n=this.getChildren(t);if(e++,1!==n.length)break;t=n[0].properties.cluster_id}return e},z.prototype._appendLeaves=function(t,e,n,r,i){for(var o=0,a=this.getChildren(e);o<a.length;o+=1){var s=a[o],A=s.properties;if(A&&A.cluster?i+A.point_count<=r?i+=A.point_count:i=this._appendLeaves(t,A.cluster_id,n,r,i):i<r?i++:t.push(s),t.length===n)break}return i},z.prototype._addTileFeatures=function(t,e,n,r,i,o){for(var a=0,s=t;a<s.length;a+=1){var A=e[s[a]],l={type:1,geometry:[[Math.round(this.options.extent*(A.x*i-n)),Math.round(this.options.extent*(A.y*i-r))]],tags:A.numPoints?W(A):this.points[A.index].properties},c=A.numPoints?A.id:this.points[A.index].id;void 0!==c&&(l.id=c),o.features.push(l)}},z.prototype._limitZoom=function(t){return Math.max(this.options.minZoom,Math.min(t,this.options.maxZoom+1))},z.prototype._cluster=function(t,e){for(var n=[],r=this.options,i=r.radius,o=r.extent,a=r.reduce,s=i/(o*Math.pow(2,e)),A=0;A<t.length;A++){var l=t[A];if(!(l.zoom<=e)){l.zoom=e;for(var c=this.trees[e+1],u=c.within(l.x,l.y,s),g=l.numPoints||1,h=l.x*g,d=l.y*g,f=a?this._map(l,!0):null,p=(A<<5)+(e+1),I=0,C=u;I<C.length;I+=1){var m=C[I],y=c.points[m];if(!(y.zoom<=e)){y.zoom=e;var E=y.numPoints||1;h+=y.x*E,d+=y.y*E,g+=E,y.parentId=p,a&&a(f,this._map(y))}}1===g?n.push(l):(l.parentId=p,n.push(J(h/g,d/g,p,g,f)))}}return n},z.prototype._map=function(t,e){if(t.numPoints)return e?$({},t.properties):t.properties;var n=this.points[t.index].properties,r=this.options.map(n);return e&&r===n?$({},r):r},wt.prototype.options={maxZoom:14,indexMaxZoom:5,indexMaxPoints:1e5,tolerance:3,extent:4096,buffer:64,lineMetrics:!1,promoteId:null,generateId:!1,debug:0},wt.prototype.splitTile=function(t,e,n,r,i,o,a){for(var s=[t,e,n,r],A=this.options,l=A.debug;s.length;){r=s.pop(),n=s.pop(),e=s.pop(),t=s.pop();var c=1<<e,u=xt(e,n,r),g=this.tiles[u];if(!g&&(l>1&&console.time("creation"),g=this.tiles[u]=Bt(t,e,n,r,A),this.tileCoords.push({z:e,x:n,y:r}),l)){l>1&&(console.log("tile z%d-%d-%d (features: %d, points: %d, simplified: %d)",e,n,r,g.numFeatures,g.numPoints,g.numSimplified),console.timeEnd("creation"));var h="z"+e;this.stats[h]=(this.stats[h]||0)+1,this.total++}if(g.source=t,i){if(e===A.maxZoom||e===i)continue;var d=1<<i-e;if(n!==Math.floor(o/d)||r!==Math.floor(a/d))continue}else if(e===A.indexMaxZoom||g.numPoints<=A.indexMaxPoints)continue;if(g.source=null,0!==t.length){l>1&&console.time("clipping");var f,p,I,C,m,y,E=.5*A.buffer/A.extent,v=.5-E,B=.5+E,b=1+E;f=p=I=C=null,m=ut(t,c,n-E,n+B,0,g.minX,g.maxX,A),y=ut(t,c,n+v,n+b,0,g.minX,g.maxX,A),t=null,m&&(f=ut(m,c,r-E,r+B,1,g.minY,g.maxY,A),p=ut(m,c,r+v,r+b,1,g.minY,g.maxY,A),m=null),y&&(I=ut(y,c,r-E,r+B,1,g.minY,g.maxY,A),C=ut(y,c,r+v,r+b,1,g.minY,g.maxY,A),y=null),l>1&&console.timeEnd("clipping"),s.push(f||[],e+1,2*n,2*r),s.push(p||[],e+1,2*n,2*r+1),s.push(I||[],e+1,2*n+1,2*r),s.push(C||[],e+1,2*n+1,2*r+1)}}},wt.prototype.getTile=function(t,e,n){var r=this.options,i=r.extent,o=r.debug;if(t<0||t>24)return null;var a=1<<t,s=xt(t,e=(e%a+a)%a,n);if(this.tiles[s])return Et(this.tiles[s],i);o>1&&console.log("drilling down to z%d-%d-%d",t,e,n);for(var A,l=t,c=e,u=n;!A&&l>0;)l--,c=Math.floor(c/2),u=Math.floor(u/2),A=this.tiles[xt(l,c,u)];return A&&A.source?(o>1&&console.log("found parent tile z%d-%d-%d",l,c,u),o>1&&console.time("drilling down"),this.splitTile(A.source,l,c,u,t,e,n),o>1&&console.timeEnd("drilling down"),this.tiles[s]?Et(this.tiles[s],i):null):null};var Dt=function(e){function n(t,n,r){e.call(this,t,n,_t),r&&(this.loadGeoJSON=r)}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.loadData=function(t,e){this._pendingCallback&&this._pendingCallback(null,{abandoned:!0}),this._pendingCallback=e,this._pendingLoadDataParams=t,this._state&&"Idle"!==this._state?this._state="NeedsLoadData":(this._state="Coalescing",this._loadData())},n.prototype._loadData=function(){var e=this;if(this._pendingCallback&&this._pendingLoadDataParams){var n=this._pendingCallback,r=this._pendingLoadDataParams;delete this._pendingCallback,delete this._pendingLoadDataParams;var i=!!(r&&r.request&&r.request.collectResourceTiming)&&new A.Performance(r.request);this.loadGeoJSON(r,function(o,a){if(o||!a)return n(o);if("object"!=typeof a)return n(new Error("Input data given to '"+r.source+"' is not a valid GeoJSON object."));!function t(e,n){switch(e&&e.type||null){case"FeatureCollection":return e.features=e.features.map(C(t,n)),e;case"GeometryCollection":return e.geometries=e.geometries.map(C(t,n)),e;case"Feature":return e.geometry=t(e.geometry,n),e;case"Polygon":case"MultiPolygon":return function(t,e){return"Polygon"===t.type?t.coordinates=m(t.coordinates,e):"MultiPolygon"===t.type&&(t.coordinates=t.coordinates.map(C(m,e))),t}(e,n);default:return e}}(a,!0);try{e._geoJSONIndex=r.cluster?new z(function(e){var n=e.superclusterOptions,r=e.clusterProperties;if(!r||!n)return n;for(var i={},o={},a={accumulated:null,zoom:0},s={properties:null},A=Object.keys(r),l=0,c=A;l<c.length;l+=1){var u=c[l],g=r[u],h=g[0],d=g[1],f=t.createExpression(d),p=t.createExpression("string"==typeof h?[h,["accumulated"],["get",u]]:h);i[u]=f.value,o[u]=p.value}return n.map=function(t){s.properties=t;for(var e={},n=0,r=A;n<r.length;n+=1){var o=r[n];e[o]=i[o].evaluate(a,s)}return e},n.reduce=function(t,e){s.properties=e;for(var n=0,r=A;n<r.length;n+=1){var i=r[n];a.accumulated=t[i],t[i]=o[i].evaluate(a,s)}},n}(r)).load(a.features):new wt(a,r.geojsonVtOptions)}catch(o){return n(o)}e.loaded={};var s={};if(i){var A=i.finish();A&&(s.resourceTiming={},s.resourceTiming[r.source]=JSON.parse(JSON.stringify(A)))}n(null,s)})}},n.prototype.coalesce=function(){"Coalescing"===this._state?this._state="Idle":"NeedsLoadData"===this._state&&(this._state="Coalescing",this._loadData())},n.prototype.reloadTile=function(t,n){var r=this.loaded,i=t.uid;return r&&r[i]?e.prototype.reloadTile.call(this,t,n):this.loadTile(t,n)},n.prototype.loadGeoJSON=function(e,n){if(e.request)t.getJSON(e.request,n);else{if("string"!=typeof e.data)return n(new Error("Input data given to '"+e.source+"' is not a valid GeoJSON object."));try{return n(null,JSON.parse(e.data))}catch(t){return n(new Error("Input data given to '"+e.source+"' is not a valid GeoJSON object."))}}},n.prototype.removeSource=function(t,e){this._pendingCallback&&this._pendingCallback(null,{abandoned:!0}),e()},n.prototype.getClusterExpansionZoom=function(t,e){e(null,this._geoJSONIndex.getClusterExpansionZoom(t.clusterId))},n.prototype.getClusterChildren=function(t,e){e(null,this._geoJSONIndex.getChildren(t.clusterId))},n.prototype.getClusterLeaves=function(t,e){e(null,this._geoJSONIndex.getLeaves(t.clusterId,t.limit,t.offset))},n}(u),Mt=function(e){var n=this;this.self=e,this.actor=new t.Actor(e,this),this.layerIndexes={},this.workerSourceTypes={vector:u,geojson:Dt},this.workerSources={},this.demWorkerSources={},this.self.registerWorkerSource=function(t,e){if(n.workerSourceTypes[t])throw new Error('Worker source with name "'+t+'" already registered.');n.workerSourceTypes[t]=e},this.self.registerRTLTextPlugin=function(e){if(t.plugin.isLoaded())throw new Error("RTL text plugin already registered.");t.plugin.applyArabicShaping=e.applyArabicShaping,t.plugin.processBidirectionalText=e.processBidirectionalText,t.plugin.processStyledBidirectionalText=e.processStyledBidirectionalText}};return Mt.prototype.setReferrer=function(t,e){this.referrer=e},Mt.prototype.setLayers=function(t,e,n){this.getLayerIndex(t).replace(e),n()},Mt.prototype.updateLayers=function(t,e,n){this.getLayerIndex(t).update(e.layers,e.removedIds),n()},Mt.prototype.loadTile=function(t,e,n){this.getWorkerSource(t,e.type,e.source).loadTile(e,n)},Mt.prototype.loadDEMTile=function(t,e,n){this.getDEMWorkerSource(t,e.source).loadTile(e,n)},Mt.prototype.reloadTile=function(t,e,n){this.getWorkerSource(t,e.type,e.source).reloadTile(e,n)},Mt.prototype.abortTile=function(t,e,n){this.getWorkerSource(t,e.type,e.source).abortTile(e,n)},Mt.prototype.removeTile=function(t,e,n){this.getWorkerSource(t,e.type,e.source).removeTile(e,n)},Mt.prototype.removeDEMTile=function(t,e){this.getDEMWorkerSource(t,e.source).removeTile(e)},Mt.prototype.removeSource=function(t,e,n){if(this.workerSources[t]&&this.workerSources[t][e.type]&&this.workerSources[t][e.type][e.source]){var r=this.workerSources[t][e.type][e.source];delete this.workerSources[t][e.type][e.source],void 0!==r.removeSource?r.removeSource(e,n):n()}},Mt.prototype.loadWorkerSource=function(t,e,n){try{this.self.importScripts(e.url),n()}catch(t){n(t.toString())}},Mt.prototype.loadRTLTextPlugin=function(e,n,r){try{t.plugin.isLoaded()||(this.self.importScripts(n),r(t.plugin.isLoaded()?null:new Error("RTL Text Plugin failed to import scripts from "+n)))}catch(t){r(t.toString())}},Mt.prototype.getLayerIndex=function(t){var e=this.layerIndexes[t];return e||(e=this.layerIndexes[t]=new r),e},Mt.prototype.getWorkerSource=function(t,e,n){var r=this;if(this.workerSources[t]||(this.workerSources[t]={}),this.workerSources[t][e]||(this.workerSources[t][e]={}),!this.workerSources[t][e][n]){var i={send:function(e,n,i){r.actor.send(e,n,i,t)}};this.workerSources[t][e][n]=new this.workerSourceTypes[e](i,this.getLayerIndex(t))}return this.workerSources[t][e][n]},Mt.prototype.getDEMWorkerSource=function(t,e){return this.demWorkerSources[t]||(this.demWorkerSources[t]={}),this.demWorkerSources[t][e]||(this.demWorkerSources[t][e]=new g),this.demWorkerSources[t][e]},Mt.prototype.enforceCacheSizeLimit=function(e,n){t.enforceCacheSizeLimit(n)},"undefined"!=typeof WorkerGlobalScope&&void 0!==t.window&&t.window instanceof WorkerGlobalScope&&(t.window.worker=new Mt(t.window)),Mt}),r(0,function(t){var e=t.createCommonjsModule(function(t){function e(t){return!!("undefined"!=typeof window&&"undefined"!=typeof document&&Array.prototype&&Array.prototype.every&&Array.prototype.filter&&Array.prototype.forEach&&Array.prototype.indexOf&&Array.prototype.lastIndexOf&&Array.prototype.map&&Array.prototype.some&&Array.prototype.reduce&&Array.prototype.reduceRight&&Array.isArray&&Function.prototype&&Function.prototype.bind&&Object.keys&&Object.create&&Object.getPrototypeOf&&Object.getOwnPropertyNames&&Object.isSealed&&Object.isFrozen&&Object.isExtensible&&Object.getOwnPropertyDescriptor&&Object.defineProperty&&Object.defineProperties&&Object.seal&&Object.freeze&&Object.preventExtensions&&"JSON"in window&&"parse"in JSON&&"stringify"in JSON&&function(){if(!("Worker"in window&&"Blob"in window&&"URL"in window))return!1;var t,e,n=new Blob([""],{type:"text/javascript"}),r=URL.createObjectURL(n);try{e=new Worker(r),t=!0}catch(e){t=!1}return e&&e.terminate(),URL.revokeObjectURL(r),t}()&&"Uint8ClampedArray"in window&&ArrayBuffer.isView&&function(t){return void 0===n[t]&&(n[t]=function(t){var n=document.createElement("canvas"),r=Object.create(e.webGLContextAttributes);return r.failIfMajorPerformanceCaveat=t,n.probablySupportsContext?n.probablySupportsContext("webgl",r)||n.probablySupportsContext("experimental-webgl",r):n.supportsContext?n.supportsContext("webgl",r)||n.supportsContext("experimental-webgl",r):n.getContext("webgl",r)||n.getContext("experimental-webgl",r)}(t)),n[t]}(t&&t.failIfMajorPerformanceCaveat))}t.exports?t.exports=e:window&&(window.mapboxgl=window.mapboxgl||{},window.mapboxgl.supported=e);var n={};e.webGLContextAttributes={antialias:!1,alpha:!0,stencil:!0,depth:!0}}),n={create:function(e,n,r){var i=t.window.document.createElement(e);return void 0!==n&&(i.className=n),r&&r.appendChild(i),i},createNS:function(e,n){return t.window.document.createElementNS(e,n)}},r=t.window.document.documentElement.style;function i(t){if(!r)return t[0];for(var e=0;e<t.length;e++)if(t[e]in r)return t[e];return t[0]}var o,a=i(["userSelect","MozUserSelect","WebkitUserSelect","msUserSelect"]);n.disableDrag=function(){r&&a&&(o=r[a],r[a]="none")},n.enableDrag=function(){r&&a&&(r[a]=o)};var s=i(["transform","WebkitTransform"]);n.setTransform=function(t,e){t.style[s]=e};var A=!1;try{var l=Object.defineProperty({},"passive",{get:function(){A=!0}});t.window.addEventListener("test",l,l),t.window.removeEventListener("test",l,l)}catch(t){A=!1}n.addEventListener=function(t,e,n,r){void 0===r&&(r={}),"passive"in r&&A?t.addEventListener(e,n,r):t.addEventListener(e,n,r.capture)},n.removeEventListener=function(t,e,n,r){void 0===r&&(r={}),"passive"in r&&A?t.removeEventListener(e,n,r):t.removeEventListener(e,n,r.capture)};var c=function(e){e.preventDefault(),e.stopPropagation(),t.window.removeEventListener("click",c,!0)};function u(t){var e=t.userImage;return!!(e&&e.render&&e.render())&&(t.data.replace(new Uint8Array(e.data.buffer)),!0)}n.suppressClick=function(){t.window.addEventListener("click",c,!0),t.window.setTimeout(function(){t.window.removeEventListener("click",c,!0)},0)},n.mousePos=function(e,n){var r=e.getBoundingClientRect(),i=t.window.TouchEvent&&n instanceof t.window.TouchEvent?n.touches[0]:n;return new t.Point(i.clientX-r.left-e.clientLeft,i.clientY-r.top-e.clientTop)},n.touchPos=function(e,n){for(var r=e.getBoundingClientRect(),i=[],o="touchend"===n.type?n.changedTouches:n.touches,a=0;a<o.length;a++)i.push(new t.Point(o[a].clientX-r.left-e.clientLeft,o[a].clientY-r.top-e.clientTop));return i},n.mouseButton=function(e){return void 0!==t.window.InstallTrigger&&2===e.button&&e.ctrlKey&&t.window.navigator.platform.toUpperCase().indexOf("MAC")>=0?0:e.button},n.remove=function(t){t.parentNode&&t.parentNode.removeChild(t)};var g=function(e){function n(){e.call(this),this.images={},this.updatedImages={},this.callbackDispatchedThisFrame={},this.loaded=!1,this.requestors=[],this.patterns={},this.atlasImage=new t.RGBAImage({width:1,height:1}),this.dirty=!0}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.isLoaded=function(){return this.loaded},n.prototype.setLoaded=function(t){if(this.loaded!==t&&(this.loaded=t,t)){for(var e=0,n=this.requestors;e<n.length;e+=1){var r=n[e],i=r.ids,o=r.callback;this._notify(i,o)}this.requestors=[]}},n.prototype.getImage=function(t){return this.images[t]},n.prototype.addImage=function(t,e){this.images[t]=e},n.prototype.updateImage=function(t,e){var n=this.images[t];e.version=n.version+1,this.images[t]=e,this.updatedImages[t]=!0},n.prototype.removeImage=function(t){var e=this.images[t];delete this.images[t],delete this.patterns[t],e.userImage&&e.userImage.onRemove&&e.userImage.onRemove()},n.prototype.listImages=function(){return Object.keys(this.images)},n.prototype.getImages=function(t,e){var n=!0;if(!this.isLoaded())for(var r=0,i=t;r<i.length;r+=1){var o=i[r];this.images[o]||(n=!1)}this.isLoaded()||n?this._notify(t,e):this.requestors.push({ids:t,callback:e})},n.prototype._notify=function(e,n){for(var r={},i=0,o=e;i<o.length;i+=1){var a=o[i];this.images[a]||this.fire(new t.Event("styleimagemissing",{id:a}));var s=this.images[a];s?r[a]={data:s.data.clone(),pixelRatio:s.pixelRatio,sdf:s.sdf,version:s.version,hasRenderCallback:Boolean(s.userImage&&s.userImage.render)}:t.warnOnce('Image "'+a+'" could not be loaded. Please make sure you have added the image with map.addImage() or a "sprite" property in your style. You can provide missing images by listening for the "styleimagemissing" map event.')}n(null,r)},n.prototype.getPixelSize=function(){var t=this.atlasImage;return{width:t.width,height:t.height}},n.prototype.getPattern=function(e){var n=this.patterns[e],r=this.getImage(e);if(!r)return null;if(n&&n.position.version===r.version)return n.position;if(n)n.position.version=r.version;else{var i={w:r.data.width+2,h:r.data.height+2,x:0,y:0},o=new t.ImagePosition(i,r);this.patterns[e]={bin:i,position:o}}return this._updatePatternAtlas(),this.patterns[e].position},n.prototype.bind=function(e){var n=e.gl;this.atlasTexture?this.dirty&&(this.atlasTexture.update(this.atlasImage),this.dirty=!1):this.atlasTexture=new t.Texture(e,this.atlasImage,n.RGBA),this.atlasTexture.bind(n.LINEAR,n.CLAMP_TO_EDGE)},n.prototype._updatePatternAtlas=function(){var e=[];for(var n in this.patterns)e.push(this.patterns[n].bin);var r=t.potpack(e),i=r.w,o=r.h,a=this.atlasImage;for(var s in a.resize({width:i||1,height:o||1}),this.patterns){var A=this.patterns[s].bin,l=A.x+1,c=A.y+1,u=this.images[s].data,g=u.width,h=u.height;t.RGBAImage.copy(u,a,{x:0,y:0},{x:l,y:c},{width:g,height:h}),t.RGBAImage.copy(u,a,{x:0,y:h-1},{x:l,y:c-1},{width:g,height:1}),t.RGBAImage.copy(u,a,{x:0,y:0},{x:l,y:c+h},{width:g,height:1}),t.RGBAImage.copy(u,a,{x:g-1,y:0},{x:l-1,y:c},{width:1,height:h}),t.RGBAImage.copy(u,a,{x:0,y:0},{x:l+g,y:c},{width:1,height:h})}this.dirty=!0},n.prototype.beginFrame=function(){this.callbackDispatchedThisFrame={}},n.prototype.dispatchRenderCallbacks=function(t){for(var e=0,n=t;e<n.length;e+=1){var r=n[e];if(!this.callbackDispatchedThisFrame[r]){this.callbackDispatchedThisFrame[r]=!0;var i=this.images[r];u(i)&&this.updateImage(r,i)}}},n}(t.Evented),h=f,d=1e20;function f(t,e,n,r,i,o){this.fontSize=t||24,this.buffer=void 0===e?3:e,this.cutoff=r||.25,this.fontFamily=i||"sans-serif",this.fontWeight=o||"normal",this.radius=n||8;var a=this.size=this.fontSize+2*this.buffer;this.canvas=document.createElement("canvas"),this.canvas.width=this.canvas.height=a,this.ctx=this.canvas.getContext("2d"),this.ctx.font=this.fontWeight+" "+this.fontSize+"px "+this.fontFamily,this.ctx.textBaseline="middle",this.ctx.fillStyle="black",this.gridOuter=new Float64Array(a*a),this.gridInner=new Float64Array(a*a),this.f=new Float64Array(a),this.d=new Float64Array(a),this.z=new Float64Array(a+1),this.v=new Int16Array(a),this.middle=Math.round(a/2*(navigator.userAgent.indexOf("Gecko/")>=0?1.2:1))}function p(t,e,n,r,i,o,a){for(var s=0;s<e;s++){for(var A=0;A<n;A++)r[A]=t[A*e+s];for(I(r,i,o,a,n),A=0;A<n;A++)t[A*e+s]=i[A]}for(A=0;A<n;A++){for(s=0;s<e;s++)r[s]=t[A*e+s];for(I(r,i,o,a,e),s=0;s<e;s++)t[A*e+s]=Math.sqrt(i[s])}}function I(t,e,n,r,i){n[0]=0,r[0]=-d,r[1]=+d;for(var o=1,a=0;o<i;o++){for(var s=(t[o]+o*o-(t[n[a]]+n[a]*n[a]))/(2*o-2*n[a]);s<=r[a];)a--,s=(t[o]+o*o-(t[n[a]]+n[a]*n[a]))/(2*o-2*n[a]);n[++a]=o,r[a]=s,r[a+1]=+d}for(o=0,a=0;o<i;o++){for(;r[a+1]<o;)a++;e[o]=(o-n[a])*(o-n[a])+t[n[a]]}}f.prototype.draw=function(t){this.ctx.clearRect(0,0,this.size,this.size),this.ctx.fillText(t,this.buffer,this.middle);for(var e=this.ctx.getImageData(0,0,this.size,this.size),n=new Uint8ClampedArray(this.size*this.size),r=0;r<this.size*this.size;r++){var i=e.data[4*r+3]/255;this.gridOuter[r]=1===i?0:0===i?d:Math.pow(Math.max(0,.5-i),2),this.gridInner[r]=1===i?d:0===i?0:Math.pow(Math.max(0,i-.5),2)}for(p(this.gridOuter,this.size,this.size,this.f,this.d,this.v,this.z),p(this.gridInner,this.size,this.size,this.f,this.d,this.v,this.z),r=0;r<this.size*this.size;r++){var o=this.gridOuter[r]-this.gridInner[r];n[r]=Math.max(0,Math.min(255,Math.round(255-255*(o/this.radius+this.cutoff))))}return n};var C=function(t,e){this.requestManager=t,this.localIdeographFontFamily=e,this.entries={}};C.prototype.setURL=function(t){this.url=t},C.prototype.getGlyphs=function(e,n){var r=this,i=[];for(var o in e)for(var a=0,s=e[o];a<s.length;a+=1){var A=s[a];i.push({stack:o,id:A})}t.asyncAll(i,function(t,e){var n=t.stack,i=t.id,o=r.entries[n];o||(o=r.entries[n]={glyphs:{},requests:{}});var a=o.glyphs[i];if(void 0===a)if(a=r._tinySDF(o,n,i))e(null,{stack:n,id:i,glyph:a});else{var s=Math.floor(i/256);if(256*s>65535)e(new Error("glyphs > 65535 not supported"));else{var A=o.requests[s];A||(A=o.requests[s]=[],C.loadGlyphRange(n,s,r.url,r.requestManager,function(t,e){if(e)for(var n in e)o.glyphs[+n]=e[+n];for(var r=0,i=A;r<i.length;r+=1)(0,i[r])(t,e);delete o.requests[s]})),A.push(function(t,r){t?e(t):r&&e(null,{stack:n,id:i,glyph:r[i]||null})})}}else e(null,{stack:n,id:i,glyph:a})},function(t,e){if(t)n(t);else if(e){for(var r={},i=0,o=e;i<o.length;i+=1){var a=o[i],s=a.stack,A=a.id,l=a.glyph;(r[s]||(r[s]={}))[A]=l&&{id:l.id,bitmap:l.bitmap.clone(),metrics:l.metrics}}n(null,r)}})},C.prototype._tinySDF=function(e,n,r){var i=this.localIdeographFontFamily;if(i&&(t.isChar["CJK Unified Ideographs"](r)||t.isChar["Hangul Syllables"](r)||t.isChar.Hiragana(r)||t.isChar.Katakana(r))){var o=e.tinySDF;if(!o){var a="400";/bold/i.test(n)?a="900":/medium/i.test(n)?a="500":/light/i.test(n)&&(a="200"),o=e.tinySDF=new C.TinySDF(24,3,8,.25,i,a)}return{id:r,bitmap:new t.AlphaImage({width:30,height:30},o.draw(String.fromCharCode(r))),metrics:{width:24,height:24,left:0,top:-8,advance:24}}}},C.loadGlyphRange=function(e,n,r,i,o){var a=256*n,s=a+255,A=i.transformRequest(i.normalizeGlyphsURL(r).replace("{fontstack}",e).replace("{range}",a+"-"+s),t.ResourceType.Glyphs);t.getArrayBuffer(A,function(e,n){if(e)o(e);else if(n){for(var r={},i=0,a=t.parseGlyphPBF(n);i<a.length;i+=1){var s=a[i];r[s.id]=s}o(null,r)}})},C.TinySDF=h;var m=function(){this.specification=t.styleSpec.light.position};m.prototype.possiblyEvaluate=function(e,n){return t.sphericalToCartesian(e.expression.evaluate(n))},m.prototype.interpolate=function(e,n,r){return{x:t.number(e.x,n.x,r),y:t.number(e.y,n.y,r),z:t.number(e.z,n.z,r)}};var y=new t.Properties({anchor:new t.DataConstantProperty(t.styleSpec.light.anchor),position:new m,color:new t.DataConstantProperty(t.styleSpec.light.color),intensity:new t.DataConstantProperty(t.styleSpec.light.intensity)}),E=function(e){function n(n){e.call(this),this._transitionable=new t.Transitionable(y),this.setLight(n),this._transitioning=this._transitionable.untransitioned()}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.getLight=function(){return this._transitionable.serialize()},n.prototype.setLight=function(e,n){if(void 0===n&&(n={}),!this._validate(t.validateLight,e,n))for(var r in e){var i=e[r];t.endsWith(r,"-transition")?this._transitionable.setTransition(r.slice(0,-"-transition".length),i):this._transitionable.setValue(r,i)}},n.prototype.updateTransitions=function(t){this._transitioning=this._transitionable.transitioned(t,this._transitioning)},n.prototype.hasTransition=function(){return this._transitioning.hasTransition()},n.prototype.recalculate=function(t){this.properties=this._transitioning.possiblyEvaluate(t)},n.prototype._validate=function(e,n,r){return(!r||!1!==r.validate)&&t.emitValidationErrors(this,e.call(t.validateStyle,t.extend({value:n,style:{glyphs:!0,sprite:!0},styleSpec:t.styleSpec})))},n}(t.Evented),v=function(t,e){this.width=t,this.height=e,this.nextRow=0,this.bytes=4,this.data=new Uint8Array(this.width*this.height*this.bytes),this.positions={}};v.prototype.getDash=function(t,e){var n=t.join(",")+String(e);return this.positions[n]||(this.positions[n]=this.addDash(t,e)),this.positions[n]},v.prototype.addDash=function(e,n){var r=n?7:0,i=2*r+1;if(this.nextRow+i>this.height)return t.warnOnce("LineAtlas out of space"),null;for(var o=0,a=0;a<e.length;a++)o+=e[a];for(var s=this.width/o,A=s/2,l=e.length%2==1,c=-r;c<=r;c++)for(var u=this.nextRow+r+c,g=this.width*u,h=l?-e[e.length-1]:0,d=e[0],f=1,p=0;p<this.width;p++){for(;d<p/s;)h=d,d+=e[f],l&&f===e.length-1&&(d+=e[0]),f++;var I=Math.abs(p-h*s),C=Math.abs(p-d*s),m=Math.min(I,C),y=f%2==1,E=void 0;if(n){var v=r?c/r*(A+1):0;if(y){var B=A-Math.abs(v);E=Math.sqrt(m*m+B*B)}else E=A-Math.sqrt(m*m+v*v)}else E=(y?1:-1)*m;this.data[3+4*(g+p)]=Math.max(0,Math.min(255,E+128))}var b={y:(this.nextRow+r+.5)/this.height,height:2*r/this.height,width:o};return this.nextRow+=i,this.dirty=!0,b},v.prototype.bind=function(t){var e=t.gl;this.texture?(e.bindTexture(e.TEXTURE_2D,this.texture),this.dirty&&(this.dirty=!1,e.texSubImage2D(e.TEXTURE_2D,0,0,0,this.width,this.height,e.RGBA,e.UNSIGNED_BYTE,this.data))):(this.texture=e.createTexture(),e.bindTexture(e.TEXTURE_2D,this.texture),e.texParameteri(e.TEXTURE_2D,e.TEXTURE_WRAP_S,e.REPEAT),e.texParameteri(e.TEXTURE_2D,e.TEXTURE_WRAP_T,e.REPEAT),e.texParameteri(e.TEXTURE_2D,e.TEXTURE_MIN_FILTER,e.LINEAR),e.texParameteri(e.TEXTURE_2D,e.TEXTURE_MAG_FILTER,e.LINEAR),e.texImage2D(e.TEXTURE_2D,0,e.RGBA,this.width,this.height,0,e.RGBA,e.UNSIGNED_BYTE,this.data))};var B=function e(n,r){this.workerPool=n,this.actors=[],this.currentActor=0,this.id=t.uniqueId();for(var i=this.workerPool.acquire(this.id),o=0;o<i.length;o++){var a=i[o],s=new e.Actor(a,r,this.id);s.name="Worker "+o,this.actors.push(s)}};function b(e,n,r){var i=function(i,o){if(i)return r(i);if(o){var a=t.pick(o,["tiles","minzoom","maxzoom","attribution","mapbox_logo","bounds"]);o.vector_layers&&(a.vectorLayers=o.vector_layers,a.vectorLayerIds=a.vectorLayers.map(function(t){return t.id})),e.url&&(a.tiles=n.canonicalizeTileset(a,e.url)),r(null,a)}};return e.url?t.getJSON(n.transformRequest(n.normalizeSourceURL(e.url),t.ResourceType.Source),i):t.browser.frame(function(){return i(null,e)})}B.prototype.broadcast=function(e,n,r){r=r||function(){},t.asyncAll(this.actors,function(t,r){t.send(e,n,r)},r)},B.prototype.send=function(t,e,n,r){return("number"!=typeof r||isNaN(r))&&(r=this.currentActor=(this.currentActor+1)%this.actors.length),this.actors[r].send(t,e,n),r},B.prototype.remove=function(){this.actors.forEach(function(t){t.remove()}),this.actors=[],this.workerPool.release(this.id)},B.Actor=t.Actor;var Q=function(e,n,r){this.bounds=t.LngLatBounds.convert(this.validateBounds(e)),this.minzoom=n||0,this.maxzoom=r||24};Q.prototype.validateBounds=function(t){return Array.isArray(t)&&4===t.length?[Math.max(-180,t[0]),Math.max(-90,t[1]),Math.min(180,t[2]),Math.min(90,t[3])]:[-180,-90,180,90]},Q.prototype.contains=function(e){var n=Math.pow(2,e.z),r=Math.floor(t.mercatorXfromLng(this.bounds.getWest())*n),i=Math.floor(t.mercatorYfromLat(this.bounds.getNorth())*n),o=Math.ceil(t.mercatorXfromLng(this.bounds.getEast())*n),a=Math.ceil(t.mercatorYfromLat(this.bounds.getSouth())*n);return e.x>=r&&e.x<o&&e.y>=i&&e.y<a};var w=function(e){function n(n,r,i,o){if(e.call(this),this.id=n,this.dispatcher=i,this.type="vector",this.minzoom=0,this.maxzoom=22,this.scheme="xyz",this.tileSize=512,this.reparseOverscaled=!0,this.isTileClipped=!0,t.extend(this,t.pick(r,["url","scheme","tileSize"])),this._options=t.extend({type:"vector"},r),this._collectResourceTiming=r.collectResourceTiming,512!==this.tileSize)throw new Error("vector tile sources must have a tileSize of 512");this.setEventedParent(o)}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.load=function(){var e=this;this.fire(new t.Event("dataloading",{dataType:"source"})),this._tileJSONRequest=b(this._options,this.map._requestManager,function(n,r){e._tileJSONRequest=null,n?e.fire(new t.ErrorEvent(n)):r&&(t.extend(e,r),r.bounds&&(e.tileBounds=new Q(r.bounds,e.minzoom,e.maxzoom)),t.postTurnstileEvent(r.tiles),t.postMapLoadEvent(r.tiles,e.map._getMapId(),e.map._requestManager._skuToken),e.fire(new t.Event("data",{dataType:"source",sourceDataType:"metadata"})),e.fire(new t.Event("data",{dataType:"source",sourceDataType:"content"})))})},n.prototype.hasTile=function(t){return!this.tileBounds||this.tileBounds.contains(t.canonical)},n.prototype.onAdd=function(t){this.map=t,this.load()},n.prototype.onRemove=function(){this._tileJSONRequest&&(this._tileJSONRequest.cancel(),this._tileJSONRequest=null)},n.prototype.serialize=function(){return t.extend({},this._options)},n.prototype.loadTile=function(e,n){var r=this.map._requestManager.normalizeTileURL(e.tileID.canonical.url(this.tiles,this.scheme),this.url,null),i={request:this.map._requestManager.transformRequest(r,t.ResourceType.Tile),uid:e.uid,tileID:e.tileID,zoom:e.tileID.overscaledZ,tileSize:this.tileSize*e.tileID.overscaleFactor(),type:this.type,source:this.id,pixelRatio:t.browser.devicePixelRatio,showCollisionBoxes:this.map.showCollisionBoxes};function o(r,i){return e.aborted?n(null):r&&404!==r.status?n(r):(i&&i.resourceTiming&&(e.resourceTiming=i.resourceTiming),this.map._refreshExpiredTiles&&i&&e.setExpiryData(i),e.loadVectorData(i,this.map.painter),t.cacheEntryPossiblyAdded(this.dispatcher),n(null),void(e.reloadCallback&&(this.loadTile(e,e.reloadCallback),e.reloadCallback=null)))}i.request.collectResourceTiming=this._collectResourceTiming,void 0===e.workerID||"expired"===e.state?e.workerID=this.dispatcher.send("loadTile",i,o.bind(this)):"loading"===e.state?e.reloadCallback=n:this.dispatcher.send("reloadTile",i,o.bind(this),e.workerID)},n.prototype.abortTile=function(t){this.dispatcher.send("abortTile",{uid:t.uid,type:this.type,source:this.id},void 0,t.workerID)},n.prototype.unloadTile=function(t){t.unloadVectorData(),this.dispatcher.send("removeTile",{uid:t.uid,type:this.type,source:this.id},void 0,t.workerID)},n.prototype.hasTransition=function(){return!1},n}(t.Evented),x=function(e){function n(n,r,i,o){e.call(this),this.id=n,this.dispatcher=i,this.setEventedParent(o),this.type="raster",this.minzoom=0,this.maxzoom=22,this.roundZoom=!0,this.scheme="xyz",this.tileSize=512,this._loaded=!1,this._options=t.extend({},r),t.extend(this,t.pick(r,["url","scheme","tileSize"]))}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.load=function(){var e=this;this.fire(new t.Event("dataloading",{dataType:"source"})),this._tileJSONRequest=b(this._options,this.map._requestManager,function(n,r){e._tileJSONRequest=null,n?e.fire(new t.ErrorEvent(n)):r&&(t.extend(e,r),r.bounds&&(e.tileBounds=new Q(r.bounds,e.minzoom,e.maxzoom)),t.postTurnstileEvent(r.tiles),t.postMapLoadEvent(r.tiles,e.map._getMapId(),e.map._requestManager._skuToken),e.fire(new t.Event("data",{dataType:"source",sourceDataType:"metadata"})),e.fire(new t.Event("data",{dataType:"source",sourceDataType:"content"})))})},n.prototype.onAdd=function(t){this.map=t,this.load()},n.prototype.onRemove=function(){this._tileJSONRequest&&(this._tileJSONRequest.cancel(),this._tileJSONRequest=null)},n.prototype.serialize=function(){return t.extend({},this._options)},n.prototype.hasTile=function(t){return!this.tileBounds||this.tileBounds.contains(t.canonical)},n.prototype.loadTile=function(e,n){var r=this,i=this.map._requestManager.normalizeTileURL(e.tileID.canonical.url(this.tiles,this.scheme),this.url,this.tileSize);e.request=t.getImage(this.map._requestManager.transformRequest(i,t.ResourceType.Tile),function(i,o){if(delete e.request,e.aborted)e.state="unloaded",n(null);else if(i)e.state="errored",n(i);else if(o){r.map._refreshExpiredTiles&&e.setExpiryData(o),delete o.cacheControl,delete o.expires;var a=r.map.painter.context,s=a.gl;e.texture=r.map.painter.getTileTexture(o.width),e.texture?e.texture.update(o,{useMipmap:!0}):(e.texture=new t.Texture(a,o,s.RGBA,{useMipmap:!0}),e.texture.bind(s.LINEAR,s.CLAMP_TO_EDGE,s.LINEAR_MIPMAP_NEAREST),a.extTextureFilterAnisotropic&&s.texParameterf(s.TEXTURE_2D,a.extTextureFilterAnisotropic.TEXTURE_MAX_ANISOTROPY_EXT,a.extTextureFilterAnisotropicMax)),e.state="loaded",t.cacheEntryPossiblyAdded(r.dispatcher),n(null)}})},n.prototype.abortTile=function(t,e){t.request&&(t.request.cancel(),delete t.request),e()},n.prototype.unloadTile=function(t,e){t.texture&&this.map.painter.saveTileTexture(t.texture),e()},n.prototype.hasTransition=function(){return!1},n}(t.Evented),_=function(e){function n(n,r,i,o){e.call(this,n,r,i,o),this.type="raster-dem",this.maxzoom=22,this._options=t.extend({},r),this.encoding=r.encoding||"mapbox"}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.serialize=function(){return{type:"raster-dem",url:this.url,tileSize:this.tileSize,tiles:this.tiles,bounds:this.bounds,encoding:this.encoding}},n.prototype.loadTile=function(e,n){var r=this.map._requestManager.normalizeTileURL(e.tileID.canonical.url(this.tiles,this.scheme),this.url,this.tileSize);e.request=t.getImage(this.map._requestManager.transformRequest(r,t.ResourceType.Tile),function(r,i){if(delete e.request,e.aborted)e.state="unloaded",n(null);else if(r)e.state="errored",n(r);else if(i){this.map._refreshExpiredTiles&&e.setExpiryData(i),delete i.cacheControl,delete i.expires;var o=t.browser.getImageData(i),a={uid:e.uid,coord:e.tileID,source:this.id,rawImageData:o,encoding:this.encoding};e.workerID&&"expired"!==e.state||(e.workerID=this.dispatcher.send("loadDEMTile",a,function(t,r){t&&(e.state="errored",n(t)),r&&(e.dem=r,e.needsHillshadePrepare=!0,e.state="loaded",n(null))}.bind(this)))}}.bind(this)),e.neighboringTiles=this._getNeighboringTiles(e.tileID)},n.prototype._getNeighboringTiles=function(e){var n=e.canonical,r=Math.pow(2,n.z),i=(n.x-1+r)%r,o=0===n.x?e.wrap-1:e.wrap,a=(n.x+1+r)%r,s=n.x+1===r?e.wrap+1:e.wrap,A={};return A[new t.OverscaledTileID(e.overscaledZ,o,n.z,i,n.y).key]={backfilled:!1},A[new t.OverscaledTileID(e.overscaledZ,s,n.z,a,n.y).key]={backfilled:!1},n.y>0&&(A[new t.OverscaledTileID(e.overscaledZ,o,n.z,i,n.y-1).key]={backfilled:!1},A[new t.OverscaledTileID(e.overscaledZ,e.wrap,n.z,n.x,n.y-1).key]={backfilled:!1},A[new t.OverscaledTileID(e.overscaledZ,s,n.z,a,n.y-1).key]={backfilled:!1}),n.y+1<r&&(A[new t.OverscaledTileID(e.overscaledZ,o,n.z,i,n.y+1).key]={backfilled:!1},A[new t.OverscaledTileID(e.overscaledZ,e.wrap,n.z,n.x,n.y+1).key]={backfilled:!1},A[new t.OverscaledTileID(e.overscaledZ,s,n.z,a,n.y+1).key]={backfilled:!1}),A},n.prototype.unloadTile=function(t){t.demTexture&&this.map.painter.saveTileTexture(t.demTexture),t.fbo&&(t.fbo.destroy(),delete t.fbo),t.dem&&delete t.dem,delete t.neighboringTiles,t.state="unloaded",this.dispatcher.send("removeDEMTile",{uid:t.uid,source:this.id},void 0,t.workerID)},n}(x),D=function(e){function n(n,r,i,o){e.call(this),this.id=n,this.type="geojson",this.minzoom=0,this.maxzoom=18,this.tileSize=512,this.isTileClipped=!0,this.reparseOverscaled=!0,this._removed=!1,this.dispatcher=i,this.setEventedParent(o),this._data=r.data,this._options=t.extend({},r),this._collectResourceTiming=r.collectResourceTiming,this._resourceTiming=[],void 0!==r.maxzoom&&(this.maxzoom=r.maxzoom),r.type&&(this.type=r.type),r.attribution&&(this.attribution=r.attribution);var a=t.EXTENT/this.tileSize;this.workerOptions=t.extend({source:this.id,cluster:r.cluster||!1,geojsonVtOptions:{buffer:(void 0!==r.buffer?r.buffer:128)*a,tolerance:(void 0!==r.tolerance?r.tolerance:.375)*a,extent:t.EXTENT,maxZoom:this.maxzoom,lineMetrics:r.lineMetrics||!1,generateId:r.generateId||!1},superclusterOptions:{maxZoom:void 0!==r.clusterMaxZoom?Math.min(r.clusterMaxZoom,this.maxzoom-1):this.maxzoom-1,extent:t.EXTENT,radius:(r.clusterRadius||50)*a,log:!1},clusterProperties:r.clusterProperties},r.workerOptions)}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.load=function(){var e=this;this.fire(new t.Event("dataloading",{dataType:"source"})),this._updateWorkerData(function(n){if(n)e.fire(new t.ErrorEvent(n));else{var r={dataType:"source",sourceDataType:"metadata"};e._collectResourceTiming&&e._resourceTiming&&e._resourceTiming.length>0&&(r.resourceTiming=e._resourceTiming,e._resourceTiming=[]),e.fire(new t.Event("data",r))}})},n.prototype.onAdd=function(t){this.map=t,this.load()},n.prototype.setData=function(e){var n=this;return this._data=e,this.fire(new t.Event("dataloading",{dataType:"source"})),this._updateWorkerData(function(e){if(e)n.fire(new t.ErrorEvent(e));else{var r={dataType:"source",sourceDataType:"content"};n._collectResourceTiming&&n._resourceTiming&&n._resourceTiming.length>0&&(r.resourceTiming=n._resourceTiming,n._resourceTiming=[]),n.fire(new t.Event("data",r))}}),this},n.prototype.getClusterExpansionZoom=function(t,e){return this.dispatcher.send("geojson.getClusterExpansionZoom",{clusterId:t,source:this.id},e,this.workerID),this},n.prototype.getClusterChildren=function(t,e){return this.dispatcher.send("geojson.getClusterChildren",{clusterId:t,source:this.id},e,this.workerID),this},n.prototype.getClusterLeaves=function(t,e,n,r){return this.dispatcher.send("geojson.getClusterLeaves",{source:this.id,clusterId:t,limit:e,offset:n},r,this.workerID),this},n.prototype._updateWorkerData=function(e){var n=this,r=t.extend({},this.workerOptions),i=this._data;"string"==typeof i?(r.request=this.map._requestManager.transformRequest(t.browser.resolveURL(i),t.ResourceType.Source),r.request.collectResourceTiming=this._collectResourceTiming):r.data=JSON.stringify(i),this.workerID=this.dispatcher.send(this.type+".loadData",r,function(t,i){n._removed||i&&i.abandoned||(n._loaded=!0,i&&i.resourceTiming&&i.resourceTiming[n.id]&&(n._resourceTiming=i.resourceTiming[n.id].slice(0)),n.dispatcher.send(n.type+".coalesce",{source:r.source},null,n.workerID),e(t))},this.workerID)},n.prototype.loadTile=function(e,n){var r=this,i=void 0===e.workerID?"loadTile":"reloadTile",o={type:this.type,uid:e.uid,tileID:e.tileID,zoom:e.tileID.overscaledZ,maxZoom:this.maxzoom,tileSize:this.tileSize,source:this.id,pixelRatio:t.browser.devicePixelRatio,showCollisionBoxes:this.map.showCollisionBoxes};e.workerID=this.dispatcher.send(i,o,function(t,o){return e.unloadVectorData(),e.aborted?n(null):t?n(t):(e.loadVectorData(o,r.map.painter,"reloadTile"===i),n(null))},this.workerID)},n.prototype.abortTile=function(t){t.aborted=!0},n.prototype.unloadTile=function(t){t.unloadVectorData(),this.dispatcher.send("removeTile",{uid:t.uid,type:this.type,source:this.id},null,t.workerID)},n.prototype.onRemove=function(){this._removed=!0,this.dispatcher.send("removeSource",{type:this.type,source:this.id},null,this.workerID)},n.prototype.serialize=function(){return t.extend({},this._options,{type:this.type,data:this._data})},n.prototype.hasTransition=function(){return!1},n}(t.Evented),M=function(e){function n(t,n,r,i){e.call(this),this.id=t,this.dispatcher=r,this.coordinates=n.coordinates,this.type="image",this.minzoom=0,this.maxzoom=22,this.tileSize=512,this.tiles={},this.setEventedParent(i),this.options=n}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.load=function(e,n){var r=this;this.fire(new t.Event("dataloading",{dataType:"source"})),this.url=this.options.url,t.getImage(this.map._requestManager.transformRequest(this.url,t.ResourceType.Image),function(i,o){i?r.fire(new t.ErrorEvent(i)):o&&(r.image=o,e&&(r.coordinates=e),n&&n(),r._finishLoading())})},n.prototype.updateImage=function(t){var e=this;return this.image&&t.url?(this.options.url=t.url,this.load(t.coordinates,function(){e.texture=null}),this):this},n.prototype._finishLoading=function(){this.map&&(this.setCoordinates(this.coordinates),this.fire(new t.Event("data",{dataType:"source",sourceDataType:"metadata"})))},n.prototype.onAdd=function(t){this.map=t,this.load()},n.prototype.setCoordinates=function(e){var n=this;this.coordinates=e;var r=e.map(t.MercatorCoordinate.fromLngLat);this.tileID=function(e){for(var n=1/0,r=1/0,i=-1/0,o=-1/0,a=0,s=e;a<s.length;a+=1){var A=s[a];n=Math.min(n,A.x),r=Math.min(r,A.y),i=Math.max(i,A.x),o=Math.max(o,A.y)}var l=i-n,c=o-r,u=Math.max(l,c),g=Math.max(0,Math.floor(-Math.log(u)/Math.LN2)),h=Math.pow(2,g);return new t.CanonicalTileID(g,Math.floor((n+i)/2*h),Math.floor((r+o)/2*h))}(r),this.minzoom=this.maxzoom=this.tileID.z;var i=r.map(function(t){return n.tileID.getTilePoint(t)._round()});return this._boundsArray=new t.StructArrayLayout4i8,this._boundsArray.emplaceBack(i[0].x,i[0].y,0,0),this._boundsArray.emplaceBack(i[1].x,i[1].y,t.EXTENT,0),this._boundsArray.emplaceBack(i[3].x,i[3].y,0,t.EXTENT),this._boundsArray.emplaceBack(i[2].x,i[2].y,t.EXTENT,t.EXTENT),this.boundsBuffer&&(this.boundsBuffer.destroy(),delete this.boundsBuffer),this.fire(new t.Event("data",{dataType:"source",sourceDataType:"content"})),this},n.prototype.prepare=function(){if(0!==Object.keys(this.tiles).length&&this.image){var e=this.map.painter.context,n=e.gl;for(var r in this.boundsBuffer||(this.boundsBuffer=e.createVertexBuffer(this._boundsArray,t.rasterBoundsAttributes.members)),this.boundsSegments||(this.boundsSegments=t.SegmentVector.simpleSegment(0,0,4,2)),this.texture||(this.texture=new t.Texture(e,this.image,n.RGBA),this.texture.bind(n.LINEAR,n.CLAMP_TO_EDGE)),this.tiles){var i=this.tiles[r];"loaded"!==i.state&&(i.state="loaded",i.texture=this.texture)}}},n.prototype.loadTile=function(t,e){this.tileID&&this.tileID.equals(t.tileID.canonical)?(this.tiles[String(t.tileID.wrap)]=t,t.buckets={},e(null)):(t.state="errored",e(null))},n.prototype.serialize=function(){return{type:"image",url:this.options.url,coordinates:this.coordinates}},n.prototype.hasTransition=function(){return!1},n}(t.Evented),k=function(e){function n(t,n,r,i){e.call(this,t,n,r,i),this.roundZoom=!0,this.type="video",this.options=n}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.load=function(){var e=this,n=this.options;this.urls=[];for(var r=0,i=n.urls;r<i.length;r+=1){var o=i[r];this.urls.push(this.map._requestManager.transformRequest(o,t.ResourceType.Source).url)}t.getVideo(this.urls,function(n,r){n?e.fire(new t.ErrorEvent(n)):r&&(e.video=r,e.video.loop=!0,e.video.addEventListener("playing",function(){e.map.triggerRepaint()}),e.map&&e.video.play(),e._finishLoading())})},n.prototype.getVideo=function(){return this.video},n.prototype.onAdd=function(t){this.map||(this.map=t,this.load(),this.video&&(this.video.play(),this.setCoordinates(this.coordinates)))},n.prototype.prepare=function(){if(!(0===Object.keys(this.tiles).length||this.video.readyState<2)){var e=this.map.painter.context,n=e.gl;for(var r in this.boundsBuffer||(this.boundsBuffer=e.createVertexBuffer(this._boundsArray,t.rasterBoundsAttributes.members)),this.boundsSegments||(this.boundsSegments=t.SegmentVector.simpleSegment(0,0,4,2)),this.texture?this.video.paused||(this.texture.bind(n.LINEAR,n.CLAMP_TO_EDGE),n.texSubImage2D(n.TEXTURE_2D,0,0,0,n.RGBA,n.UNSIGNED_BYTE,this.video)):(this.texture=new t.Texture(e,this.video,n.RGBA),this.texture.bind(n.LINEAR,n.CLAMP_TO_EDGE)),this.tiles){var i=this.tiles[r];"loaded"!==i.state&&(i.state="loaded",i.texture=this.texture)}}},n.prototype.serialize=function(){return{type:"video",urls:this.urls,coordinates:this.coordinates}},n.prototype.hasTransition=function(){return this.video&&!this.video.paused},n}(M),S=function(e){function n(n,r,i,o){e.call(this,n,r,i,o),r.coordinates?Array.isArray(r.coordinates)&&4===r.coordinates.length&&!r.coordinates.some(function(t){return!Array.isArray(t)||2!==t.length||t.some(function(t){return"number"!=typeof t})})||this.fire(new t.ErrorEvent(new t.ValidationError("sources."+n,null,'"coordinates" property must be an array of 4 longitude/latitude array pairs'))):this.fire(new t.ErrorEvent(new t.ValidationError("sources."+n,null,'missing required property "coordinates"'))),r.animate&&"boolean"!=typeof r.animate&&this.fire(new t.ErrorEvent(new t.ValidationError("sources."+n,null,'optional "animate" property must be a boolean value'))),r.canvas?"string"==typeof r.canvas||r.canvas instanceof t.window.HTMLCanvasElement||this.fire(new t.ErrorEvent(new t.ValidationError("sources."+n,null,'"canvas" must be either a string representing the ID of the canvas element from which to read, or an HTMLCanvasElement instance'))):this.fire(new t.ErrorEvent(new t.ValidationError("sources."+n,null,'missing required property "canvas"'))),this.options=r,this.animate=void 0===r.animate||r.animate}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.load=function(){this.canvas||(this.canvas=this.options.canvas instanceof t.window.HTMLCanvasElement?this.options.canvas:t.window.document.getElementById(this.options.canvas)),this.width=this.canvas.width,this.height=this.canvas.height,this._hasInvalidDimensions()?this.fire(new t.ErrorEvent(new Error("Canvas dimensions cannot be less than or equal to zero."))):(this.play=function(){this._playing=!0,this.map.triggerRepaint()},this.pause=function(){this._playing&&(this.prepare(),this._playing=!1)},this._finishLoading())},n.prototype.getCanvas=function(){return this.canvas},n.prototype.onAdd=function(t){this.map=t,this.load(),this.canvas&&this.animate&&this.play()},n.prototype.onRemove=function(){this.pause()},n.prototype.prepare=function(){var e=!1;if(this.canvas.width!==this.width&&(this.width=this.canvas.width,e=!0),this.canvas.height!==this.height&&(this.height=this.canvas.height,e=!0),!this._hasInvalidDimensions()&&0!==Object.keys(this.tiles).length){var n=this.map.painter.context,r=n.gl;for(var i in this.boundsBuffer||(this.boundsBuffer=n.createVertexBuffer(this._boundsArray,t.rasterBoundsAttributes.members)),this.boundsSegments||(this.boundsSegments=t.SegmentVector.simpleSegment(0,0,4,2)),this.texture?(e||this._playing)&&this.texture.update(this.canvas,{premultiply:!0}):this.texture=new t.Texture(n,this.canvas,r.RGBA,{premultiply:!0}),this.tiles){var o=this.tiles[i];"loaded"!==o.state&&(o.state="loaded",o.texture=this.texture)}}},n.prototype.serialize=function(){return{type:"canvas",coordinates:this.coordinates}},n.prototype.hasTransition=function(){return this._playing},n.prototype._hasInvalidDimensions=function(){for(var t=0,e=[this.canvas.width,this.canvas.height];t<e.length;t+=1){var n=e[t];if(isNaN(n)||n<=0)return!0}return!1},n}(M),L={vector:w,raster:x,"raster-dem":_,geojson:D,video:k,image:M,canvas:S},R=function(e,n,r,i){var o=new L[n.type](e,n,r,i);if(o.id!==e)throw new Error("Expected Source id to be "+e+" instead of "+o.id);return t.bindAll(["load","abort","unload","serialize","prepare"],o),o};function N(e,n){var r=t.identity([]);return t.translate(r,r,[1,1,0]),t.scale(r,r,[.5*e.width,.5*e.height,1]),t.multiply(r,r,e.calculatePosMatrix(n.toUnwrapped()))}function F(t,e,n,r,i){var o=function(t,e,n){if(t)for(var r=0,i=t;r<i.length;r+=1){var o=e[i[r]];if(o&&o.source===n&&"fill-extrusion"===o.type)return!0}else for(var a in e){var s=e[a];if(s.source===n&&"fill-extrusion"===s.type)return!0}return!1}(r&&r.layers,e,t.id),a=i.maxPitchScaleFactor(),s=t.tilesIn(n,a,o);s.sort(T);for(var A=[],l=0,c=s;l<c.length;l+=1){var u=c[l];A.push({wrappedTileID:u.tileID.wrapped().key,queryResults:u.tile.queryRenderedFeatures(e,t._state,u.queryGeometry,u.cameraQueryGeometry,u.scale,r,i,a,N(t.transform,u.tileID))})}var g=function(t){for(var e={},n={},r=0,i=t;r<i.length;r+=1){var o=i[r],a=o.queryResults,s=o.wrappedTileID,A=n[s]=n[s]||{};for(var l in a)for(var c=a[l],u=A[l]=A[l]||{},g=e[l]=e[l]||[],h=0,d=c;h<d.length;h+=1){var f=d[h];u[f.featureIndex]||(u[f.featureIndex]=!0,g.push(f))}}return e}(A);for(var h in g)g[h].forEach(function(e){var n=e.feature,r=t.getFeatureState(n.layer["source-layer"],n.id);n.source=n.layer.source,n.layer["source-layer"]&&(n.sourceLayer=n.layer["source-layer"]),n.state=r});return g}function T(t,e){var n=t.tileID,r=e.tileID;return n.overscaledZ-r.overscaledZ||n.canonical.y-r.canonical.y||n.wrap-r.wrap||n.canonical.x-r.canonical.x}var U=function(t,e){this.max=t,this.onRemove=e,this.reset()};U.prototype.reset=function(){for(var t in this.data)for(var e=0,n=this.data[t];e<n.length;e+=1){var r=n[e];r.timeout&&clearTimeout(r.timeout),this.onRemove(r.value)}return this.data={},this.order=[],this},U.prototype.add=function(t,e,n){var r=this,i=t.wrapped().key;void 0===this.data[i]&&(this.data[i]=[]);var o={value:e,timeout:void 0};if(void 0!==n&&(o.timeout=setTimeout(function(){r.remove(t,o)},n)),this.data[i].push(o),this.order.push(i),this.order.length>this.max){var a=this._getAndRemoveByKey(this.order[0]);a&&this.onRemove(a)}return this},U.prototype.has=function(t){return t.wrapped().key in this.data},U.prototype.getAndRemove=function(t){return this.has(t)?this._getAndRemoveByKey(t.wrapped().key):null},U.prototype._getAndRemoveByKey=function(t){var e=this.data[t].shift();return e.timeout&&clearTimeout(e.timeout),0===this.data[t].length&&delete this.data[t],this.order.splice(this.order.indexOf(t),1),e.value},U.prototype.get=function(t){return this.has(t)?this.data[t.wrapped().key][0].value:null},U.prototype.remove=function(t,e){if(!this.has(t))return this;var n=t.wrapped().key,r=void 0===e?0:this.data[n].indexOf(e),i=this.data[n][r];return this.data[n].splice(r,1),i.timeout&&clearTimeout(i.timeout),0===this.data[n].length&&delete this.data[n],this.onRemove(i.value),this.order.splice(this.order.indexOf(n),1),this},U.prototype.setMaxSize=function(t){for(this.max=t;this.order.length>this.max;){var e=this._getAndRemoveByKey(this.order[0]);e&&this.onRemove(e)}return this};var O=function(t,e,n){this.context=t;var r=t.gl;this.buffer=r.createBuffer(),this.dynamicDraw=Boolean(n),this.context.unbindVAO(),t.bindElementBuffer.set(this.buffer),r.bufferData(r.ELEMENT_ARRAY_BUFFER,e.arrayBuffer,this.dynamicDraw?r.DYNAMIC_DRAW:r.STATIC_DRAW),this.dynamicDraw||delete e.arrayBuffer};O.prototype.bind=function(){this.context.bindElementBuffer.set(this.buffer)},O.prototype.updateData=function(t){var e=this.context.gl;this.context.unbindVAO(),this.bind(),e.bufferSubData(e.ELEMENT_ARRAY_BUFFER,0,t.arrayBuffer)},O.prototype.destroy=function(){var t=this.context.gl;this.buffer&&(t.deleteBuffer(this.buffer),delete this.buffer)};var G={Int8:"BYTE",Uint8:"UNSIGNED_BYTE",Int16:"SHORT",Uint16:"UNSIGNED_SHORT",Int32:"INT",Uint32:"UNSIGNED_INT",Float32:"FLOAT"},H=function(t,e,n,r){this.length=e.length,this.attributes=n,this.itemSize=e.bytesPerElement,this.dynamicDraw=r,this.context=t;var i=t.gl;this.buffer=i.createBuffer(),t.bindVertexBuffer.set(this.buffer),i.bufferData(i.ARRAY_BUFFER,e.arrayBuffer,this.dynamicDraw?i.DYNAMIC_DRAW:i.STATIC_DRAW),this.dynamicDraw||delete e.arrayBuffer};H.prototype.bind=function(){this.context.bindVertexBuffer.set(this.buffer)},H.prototype.updateData=function(t){var e=this.context.gl;this.bind(),e.bufferSubData(e.ARRAY_BUFFER,0,t.arrayBuffer)},H.prototype.enableAttributes=function(t,e){for(var n=0;n<this.attributes.length;n++){var r=this.attributes[n],i=e.attributes[r.name];void 0!==i&&t.enableVertexAttribArray(i)}},H.prototype.setVertexAttribPointers=function(t,e,n){for(var r=0;r<this.attributes.length;r++){var i=this.attributes[r],o=e.attributes[i.name];void 0!==o&&t.vertexAttribPointer(o,i.components,t[G[i.type]],!1,this.itemSize,i.offset+this.itemSize*(n||0))}},H.prototype.destroy=function(){var t=this.context.gl;this.buffer&&(t.deleteBuffer(this.buffer),delete this.buffer)};var j=function(t){this.gl=t.gl,this.default=this.getDefault(),this.current=this.default,this.dirty=!1};j.prototype.get=function(){return this.current},j.prototype.set=function(t){},j.prototype.getDefault=function(){return this.default},j.prototype.setDefault=function(){this.set(this.default)};var Y=function(e){function n(){e.apply(this,arguments)}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.getDefault=function(){return t.Color.transparent},n.prototype.set=function(t){var e=this.current;(t.r!==e.r||t.g!==e.g||t.b!==e.b||t.a!==e.a||this.dirty)&&(this.gl.clearColor(t.r,t.g,t.b,t.a),this.current=t,this.dirty=!1)},n}(j),P=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return 1},e.prototype.set=function(t){(t!==this.current||this.dirty)&&(this.gl.clearDepth(t),this.current=t,this.dirty=!1)},e}(j),K=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return 0},e.prototype.set=function(t){(t!==this.current||this.dirty)&&(this.gl.clearStencil(t),this.current=t,this.dirty=!1)},e}(j),z=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return[!0,!0,!0,!0]},e.prototype.set=function(t){var e=this.current;(t[0]!==e[0]||t[1]!==e[1]||t[2]!==e[2]||t[3]!==e[3]||this.dirty)&&(this.gl.colorMask(t[0],t[1],t[2],t[3]),this.current=t,this.dirty=!1)},e}(j),J=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return!0},e.prototype.set=function(t){(t!==this.current||this.dirty)&&(this.gl.depthMask(t),this.current=t,this.dirty=!1)},e}(j),q=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return 255},e.prototype.set=function(t){(t!==this.current||this.dirty)&&(this.gl.stencilMask(t),this.current=t,this.dirty=!1)},e}(j),V=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return{func:this.gl.ALWAYS,ref:0,mask:255}},e.prototype.set=function(t){var e=this.current;(t.func!==e.func||t.ref!==e.ref||t.mask!==e.mask||this.dirty)&&(this.gl.stencilFunc(t.func,t.ref,t.mask),this.current=t,this.dirty=!1)},e}(j),W=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){var t=this.gl;return[t.KEEP,t.KEEP,t.KEEP]},e.prototype.set=function(t){var e=this.current;(t[0]!==e[0]||t[1]!==e[1]||t[2]!==e[2]||this.dirty)&&(this.gl.stencilOp(t[0],t[1],t[2]),this.current=t,this.dirty=!1)},e}(j),X=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return!1},e.prototype.set=function(t){if(t!==this.current||this.dirty){var e=this.gl;t?e.enable(e.STENCIL_TEST):e.disable(e.STENCIL_TEST),this.current=t,this.dirty=!1}},e}(j),Z=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return[0,1]},e.prototype.set=function(t){var e=this.current;(t[0]!==e[0]||t[1]!==e[1]||this.dirty)&&(this.gl.depthRange(t[0],t[1]),this.current=t,this.dirty=!1)},e}(j),$=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return!1},e.prototype.set=function(t){if(t!==this.current||this.dirty){var e=this.gl;t?e.enable(e.DEPTH_TEST):e.disable(e.DEPTH_TEST),this.current=t,this.dirty=!1}},e}(j),tt=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return this.gl.LESS},e.prototype.set=function(t){(t!==this.current||this.dirty)&&(this.gl.depthFunc(t),this.current=t,this.dirty=!1)},e}(j),et=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return!1},e.prototype.set=function(t){if(t!==this.current||this.dirty){var e=this.gl;t?e.enable(e.BLEND):e.disable(e.BLEND),this.current=t,this.dirty=!1}},e}(j),nt=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){var t=this.gl;return[t.ONE,t.ZERO]},e.prototype.set=function(t){var e=this.current;(t[0]!==e[0]||t[1]!==e[1]||this.dirty)&&(this.gl.blendFunc(t[0],t[1]),this.current=t,this.dirty=!1)},e}(j),rt=function(e){function n(){e.apply(this,arguments)}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.getDefault=function(){return t.Color.transparent},n.prototype.set=function(t){var e=this.current;(t.r!==e.r||t.g!==e.g||t.b!==e.b||t.a!==e.a||this.dirty)&&(this.gl.blendColor(t.r,t.g,t.b,t.a),this.current=t,this.dirty=!1)},n}(j),it=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return this.gl.FUNC_ADD},e.prototype.set=function(t){(t!==this.current||this.dirty)&&(this.gl.blendEquation(t),this.current=t,this.dirty=!1)},e}(j),ot=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return!1},e.prototype.set=function(t){if(t!==this.current||this.dirty){var e=this.gl;t?e.enable(e.CULL_FACE):e.disable(e.CULL_FACE),this.current=t,this.dirty=!1}},e}(j),at=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return this.gl.BACK},e.prototype.set=function(t){(t!==this.current||this.dirty)&&(this.gl.cullFace(t),this.current=t,this.dirty=!1)},e}(j),st=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return this.gl.CCW},e.prototype.set=function(t){(t!==this.current||this.dirty)&&(this.gl.frontFace(t),this.current=t,this.dirty=!1)},e}(j),At=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return null},e.prototype.set=function(t){(t!==this.current||this.dirty)&&(this.gl.useProgram(t),this.current=t,this.dirty=!1)},e}(j),lt=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return this.gl.TEXTURE0},e.prototype.set=function(t){(t!==this.current||this.dirty)&&(this.gl.activeTexture(t),this.current=t,this.dirty=!1)},e}(j),ct=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){var t=this.gl;return[0,0,t.drawingBufferWidth,t.drawingBufferHeight]},e.prototype.set=function(t){var e=this.current;(t[0]!==e[0]||t[1]!==e[1]||t[2]!==e[2]||t[3]!==e[3]||this.dirty)&&(this.gl.viewport(t[0],t[1],t[2],t[3]),this.current=t,this.dirty=!1)},e}(j),ut=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return null},e.prototype.set=function(t){if(t!==this.current||this.dirty){var e=this.gl;e.bindFramebuffer(e.FRAMEBUFFER,t),this.current=t,this.dirty=!1}},e}(j),gt=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return null},e.prototype.set=function(t){if(t!==this.current||this.dirty){var e=this.gl;e.bindRenderbuffer(e.RENDERBUFFER,t),this.current=t,this.dirty=!1}},e}(j),ht=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return null},e.prototype.set=function(t){if(t!==this.current||this.dirty){var e=this.gl;e.bindTexture(e.TEXTURE_2D,t),this.current=t,this.dirty=!1}},e}(j),dt=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return null},e.prototype.set=function(t){if(t!==this.current||this.dirty){var e=this.gl;e.bindBuffer(e.ARRAY_BUFFER,t),this.current=t,this.dirty=!1}},e}(j),ft=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return null},e.prototype.set=function(t){var e=this.gl;e.bindBuffer(e.ELEMENT_ARRAY_BUFFER,t),this.current=t,this.dirty=!1},e}(j),pt=function(t){function e(e){t.call(this,e),this.vao=e.extVertexArrayObject}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return null},e.prototype.set=function(t){this.vao&&(t!==this.current||this.dirty)&&(this.vao.bindVertexArrayOES(t),this.current=t,this.dirty=!1)},e}(j),It=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return 4},e.prototype.set=function(t){if(t!==this.current||this.dirty){var e=this.gl;e.pixelStorei(e.UNPACK_ALIGNMENT,t),this.current=t,this.dirty=!1}},e}(j),Ct=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return!1},e.prototype.set=function(t){if(t!==this.current||this.dirty){var e=this.gl;e.pixelStorei(e.UNPACK_PREMULTIPLY_ALPHA_WEBGL,t),this.current=t,this.dirty=!1}},e}(j),mt=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return!1},e.prototype.set=function(t){if(t!==this.current||this.dirty){var e=this.gl;e.pixelStorei(e.UNPACK_FLIP_Y_WEBGL,t),this.current=t,this.dirty=!1}},e}(j),yt=function(t){function e(e,n){t.call(this,e),this.context=e,this.parent=n}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.getDefault=function(){return null},e}(j),Et=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.setDirty=function(){this.dirty=!0},e.prototype.set=function(t){if(t!==this.current||this.dirty){this.context.bindFramebuffer.set(this.parent);var e=this.gl;e.framebufferTexture2D(e.FRAMEBUFFER,e.COLOR_ATTACHMENT0,e.TEXTURE_2D,t,0),this.current=t,this.dirty=!1}},e}(yt),vt=function(t){function e(){t.apply(this,arguments)}return t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e,e.prototype.set=function(t){if(t!==this.current||this.dirty){this.context.bindFramebuffer.set(this.parent);var e=this.gl;e.framebufferRenderbuffer(e.FRAMEBUFFER,e.DEPTH_ATTACHMENT,e.RENDERBUFFER,t),this.current=t,this.dirty=!1}},e}(yt),Bt=function(t,e,n){this.context=t,this.width=e,this.height=n;var r=t.gl,i=this.framebuffer=r.createFramebuffer();this.colorAttachment=new Et(t,i),this.depthAttachment=new vt(t,i)};Bt.prototype.destroy=function(){var t=this.context.gl,e=this.colorAttachment.get();e&&t.deleteTexture(e);var n=this.depthAttachment.get();n&&t.deleteRenderbuffer(n),t.deleteFramebuffer(this.framebuffer)};var bt=function(t,e,n){this.func=t,this.mask=e,this.range=n};bt.ReadOnly=!1,bt.ReadWrite=!0,bt.disabled=new bt(519,bt.ReadOnly,[0,1]);var Qt=function(t,e,n,r,i,o){this.test=t,this.ref=e,this.mask=n,this.fail=r,this.depthFail=i,this.pass=o};Qt.disabled=new Qt({func:519,mask:0},0,0,7680,7680,7680);var wt=function(t,e,n){this.blendFunction=t,this.blendColor=e,this.mask=n};wt.disabled=new wt(wt.Replace=[1,0],t.Color.transparent,[!1,!1,!1,!1]),wt.unblended=new wt(wt.Replace,t.Color.transparent,[!0,!0,!0,!0]),wt.alphaBlended=new wt([1,771],t.Color.transparent,[!0,!0,!0,!0]);var xt=function(t,e,n){this.enable=t,this.mode=e,this.frontFace=n};xt.disabled=new xt(!1,1029,2305),xt.backCCW=new xt(!0,1029,2305);var _t=function(t){this.gl=t,this.extVertexArrayObject=this.gl.getExtension("OES_vertex_array_object"),this.clearColor=new Y(this),this.clearDepth=new P(this),this.clearStencil=new K(this),this.colorMask=new z(this),this.depthMask=new J(this),this.stencilMask=new q(this),this.stencilFunc=new V(this),this.stencilOp=new W(this),this.stencilTest=new X(this),this.depthRange=new Z(this),this.depthTest=new $(this),this.depthFunc=new tt(this),this.blend=new et(this),this.blendFunc=new nt(this),this.blendColor=new rt(this),this.blendEquation=new it(this),this.cullFace=new ot(this),this.cullFaceSide=new at(this),this.frontFace=new st(this),this.program=new At(this),this.activeTexture=new lt(this),this.viewport=new ct(this),this.bindFramebuffer=new ut(this),this.bindRenderbuffer=new gt(this),this.bindTexture=new ht(this),this.bindVertexBuffer=new dt(this),this.bindElementBuffer=new ft(this),this.bindVertexArrayOES=this.extVertexArrayObject&&new pt(this),this.pixelStoreUnpack=new It(this),this.pixelStoreUnpackPremultiplyAlpha=new Ct(this),this.pixelStoreUnpackFlipY=new mt(this),this.extTextureFilterAnisotropic=t.getExtension("EXT_texture_filter_anisotropic")||t.getExtension("MOZ_EXT_texture_filter_anisotropic")||t.getExtension("WEBKIT_EXT_texture_filter_anisotropic"),this.extTextureFilterAnisotropic&&(this.extTextureFilterAnisotropicMax=t.getParameter(this.extTextureFilterAnisotropic.MAX_TEXTURE_MAX_ANISOTROPY_EXT)),this.extTextureHalfFloat=t.getExtension("OES_texture_half_float"),this.extTextureHalfFloat&&t.getExtension("OES_texture_half_float_linear")};_t.prototype.setDefault=function(){this.unbindVAO(),this.clearColor.setDefault(),this.clearDepth.setDefault(),this.clearStencil.setDefault(),this.colorMask.setDefault(),this.depthMask.setDefault(),this.stencilMask.setDefault(),this.stencilFunc.setDefault(),this.stencilOp.setDefault(),this.stencilTest.setDefault(),this.depthRange.setDefault(),this.depthTest.setDefault(),this.depthFunc.setDefault(),this.blend.setDefault(),this.blendFunc.setDefault(),this.blendColor.setDefault(),this.blendEquation.setDefault(),this.cullFace.setDefault(),this.cullFaceSide.setDefault(),this.frontFace.setDefault(),this.program.setDefault(),this.activeTexture.setDefault(),this.bindFramebuffer.setDefault(),this.pixelStoreUnpack.setDefault(),this.pixelStoreUnpackPremultiplyAlpha.setDefault(),this.pixelStoreUnpackFlipY.setDefault()},_t.prototype.setDirty=function(){this.clearColor.dirty=!0,this.clearDepth.dirty=!0,this.clearStencil.dirty=!0,this.colorMask.dirty=!0,this.depthMask.dirty=!0,this.stencilMask.dirty=!0,this.stencilFunc.dirty=!0,this.stencilOp.dirty=!0,this.stencilTest.dirty=!0,this.depthRange.dirty=!0,this.depthTest.dirty=!0,this.depthFunc.dirty=!0,this.blend.dirty=!0,this.blendFunc.dirty=!0,this.blendColor.dirty=!0,this.blendEquation.dirty=!0,this.cullFace.dirty=!0,this.cullFaceSide.dirty=!0,this.frontFace.dirty=!0,this.program.dirty=!0,this.activeTexture.dirty=!0,this.viewport.dirty=!0,this.bindFramebuffer.dirty=!0,this.bindRenderbuffer.dirty=!0,this.bindTexture.dirty=!0,this.bindVertexBuffer.dirty=!0,this.bindElementBuffer.dirty=!0,this.extVertexArrayObject&&(this.bindVertexArrayOES.dirty=!0),this.pixelStoreUnpack.dirty=!0,this.pixelStoreUnpackPremultiplyAlpha.dirty=!0,this.pixelStoreUnpackFlipY.dirty=!0},_t.prototype.createIndexBuffer=function(t,e){return new O(this,t,e)},_t.prototype.createVertexBuffer=function(t,e,n){return new H(this,t,e,n)},_t.prototype.createRenderbuffer=function(t,e,n){var r=this.gl,i=r.createRenderbuffer();return this.bindRenderbuffer.set(i),r.renderbufferStorage(r.RENDERBUFFER,t,e,n),this.bindRenderbuffer.set(null),i},_t.prototype.createFramebuffer=function(t,e){return new Bt(this,t,e)},_t.prototype.clear=function(t){var e=t.color,n=t.depth,r=this.gl,i=0;e&&(i|=r.COLOR_BUFFER_BIT,this.clearColor.set(e),this.colorMask.set([!0,!0,!0,!0])),void 0!==n&&(i|=r.DEPTH_BUFFER_BIT,this.depthRange.set([0,1]),this.clearDepth.set(n),this.depthMask.set(!0)),r.clear(i)},_t.prototype.setCullFace=function(t){!1===t.enable?this.cullFace.set(!1):(this.cullFace.set(!0),this.cullFaceSide.set(t.mode),this.frontFace.set(t.frontFace))},_t.prototype.setDepthMode=function(t){t.func!==this.gl.ALWAYS||t.mask?(this.depthTest.set(!0),this.depthFunc.set(t.func),this.depthMask.set(t.mask),this.depthRange.set(t.range)):this.depthTest.set(!1)},_t.prototype.setStencilMode=function(t){t.test.func!==this.gl.ALWAYS||t.mask?(this.stencilTest.set(!0),this.stencilMask.set(t.mask),this.stencilOp.set([t.fail,t.depthFail,t.pass]),this.stencilFunc.set({func:t.test.func,ref:t.ref,mask:t.test.mask})):this.stencilTest.set(!1)},_t.prototype.setColorMode=function(e){t.deepEqual(e.blendFunction,wt.Replace)?this.blend.set(!1):(this.blend.set(!0),this.blendFunc.set(e.blendFunction),this.blendColor.set(e.blendColor)),this.colorMask.set(e.mask)},_t.prototype.unbindVAO=function(){this.extVertexArrayObject&&this.bindVertexArrayOES.set(null)};var Dt=function(e){function n(n,r,i){var o=this;e.call(this),this.id=n,this.dispatcher=i,this.on("data",function(t){"source"===t.dataType&&"metadata"===t.sourceDataType&&(o._sourceLoaded=!0),o._sourceLoaded&&!o._paused&&"source"===t.dataType&&"content"===t.sourceDataType&&(o.reload(),o.transform&&o.update(o.transform))}),this.on("error",function(){o._sourceErrored=!0}),this._source=R(n,r,i,this),this._tiles={},this._cache=new U(0,this._unloadTile.bind(this)),this._timers={},this._cacheTimers={},this._maxTileCacheSize=null,this._coveredTiles={},this._state=new t.SourceFeatureState}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.onAdd=function(t){this.map=t,this._maxTileCacheSize=t?t._maxTileCacheSize:null,this._source&&this._source.onAdd&&this._source.onAdd(t)},n.prototype.onRemove=function(t){this._source&&this._source.onRemove&&this._source.onRemove(t)},n.prototype.loaded=function(){if(this._sourceErrored)return!0;if(!this._sourceLoaded)return!1;for(var t in this._tiles){var e=this._tiles[t];if("loaded"!==e.state&&"errored"!==e.state)return!1}return!0},n.prototype.getSource=function(){return this._source},n.prototype.pause=function(){this._paused=!0},n.prototype.resume=function(){if(this._paused){var t=this._shouldReloadOnResume;this._paused=!1,this._shouldReloadOnResume=!1,t&&this.reload(),this.transform&&this.update(this.transform)}},n.prototype._loadTile=function(t,e){return this._source.loadTile(t,e)},n.prototype._unloadTile=function(t){if(this._source.unloadTile)return this._source.unloadTile(t,function(){})},n.prototype._abortTile=function(t){if(this._source.abortTile)return this._source.abortTile(t,function(){})},n.prototype.serialize=function(){return this._source.serialize()},n.prototype.prepare=function(t){for(var e in this._source.prepare&&this._source.prepare(),this._state.coalesceChanges(this._tiles,this.map?this.map.painter:null),this._tiles){var n=this._tiles[e];n.upload(t),n.prepare(this.map.style.imageManager)}},n.prototype.getIds=function(){return Object.keys(this._tiles).map(Number).sort(Mt)},n.prototype.getRenderableIds=function(e){var n=this,r=[];for(var i in this._tiles)this._isIdRenderable(+i,e)&&r.push(+i);return e?r.sort(function(e,r){var i=n._tiles[e].tileID,o=n._tiles[r].tileID,a=new t.Point(i.canonical.x,i.canonical.y)._rotate(n.transform.angle),s=new t.Point(o.canonical.x,o.canonical.y)._rotate(n.transform.angle);return i.overscaledZ-o.overscaledZ||s.y-a.y||s.x-a.x}):r.sort(Mt)},n.prototype.hasRenderableParent=function(t){var e=this.findLoadedParent(t,0);return!!e&&this._isIdRenderable(e.tileID.key)},n.prototype._isIdRenderable=function(t,e){return this._tiles[t]&&this._tiles[t].hasData()&&!this._coveredTiles[t]&&(e||!this._tiles[t].holdingForFade())},n.prototype.reload=function(){if(this._paused)this._shouldReloadOnResume=!0;else for(var t in this._cache.reset(),this._tiles)"errored"!==this._tiles[t].state&&this._reloadTile(t,"reloading")},n.prototype._reloadTile=function(t,e){var n=this._tiles[t];n&&("loading"!==n.state&&(n.state=e),this._loadTile(n,this._tileLoaded.bind(this,n,t,e)))},n.prototype._tileLoaded=function(e,n,r,i){if(i)return e.state="errored",void(404!==i.status?this._source.fire(new t.ErrorEvent(i,{tile:e})):this.update(this.transform));e.timeAdded=t.browser.now(),"expired"===r&&(e.refreshedUponExpiration=!0),this._setTileReloadTimer(n,e),"raster-dem"===this.getSource().type&&e.dem&&this._backfillDEM(e),this._state.initializeTileState(e,this.map?this.map.painter:null),this._source.fire(new t.Event("data",{dataType:"source",tile:e,coord:e.tileID}))},n.prototype._backfillDEM=function(t){for(var e=this.getRenderableIds(),n=0;n<e.length;n++){var r=e[n];if(t.neighboringTiles&&t.neighboringTiles[r]){var i=this.getTileByID(r);o(t,i),o(i,t)}}function o(t,e){t.needsHillshadePrepare=!0;var n=e.tileID.canonical.x-t.tileID.canonical.x,r=e.tileID.canonical.y-t.tileID.canonical.y,i=Math.pow(2,t.tileID.canonical.z),o=e.tileID.key;0===n&&0===r||Math.abs(r)>1||(Math.abs(n)>1&&(1===Math.abs(n+i)?n+=i:1===Math.abs(n-i)&&(n-=i)),e.dem&&t.dem&&(t.dem.backfillBorder(e.dem,n,r),t.neighboringTiles&&t.neighboringTiles[o]&&(t.neighboringTiles[o].backfilled=!0)))}},n.prototype.getTile=function(t){return this.getTileByID(t.key)},n.prototype.getTileByID=function(t){return this._tiles[t]},n.prototype.getZoom=function(t){return t.zoom+t.scaleZoom(t.tileSize/this._source.tileSize)},n.prototype._retainLoadedChildren=function(t,e,n,r){for(var i in this._tiles){var o=this._tiles[i];if(!(r[i]||!o.hasData()||o.tileID.overscaledZ<=e||o.tileID.overscaledZ>n)){for(var a=o.tileID;o&&o.tileID.overscaledZ>e+1;){var s=o.tileID.scaledTo(o.tileID.overscaledZ-1);(o=this._tiles[s.key])&&o.hasData()&&(a=s)}for(var A=a;A.overscaledZ>e;)if(t[(A=A.scaledTo(A.overscaledZ-1)).key]){r[a.key]=a;break}}}},n.prototype.findLoadedParent=function(t,e){for(var n=t.overscaledZ-1;n>=e;n--){var r=t.scaledTo(n);if(!r)return;var i=String(r.key),o=this._tiles[i];if(o&&o.hasData())return o;if(this._cache.has(r))return this._cache.get(r)}},n.prototype.updateCacheSize=function(t){var e=(Math.ceil(t.width/this._source.tileSize)+1)*(Math.ceil(t.height/this._source.tileSize)+1),n=Math.floor(5*e),r="number"==typeof this._maxTileCacheSize?Math.min(this._maxTileCacheSize,n):n;this._cache.setMaxSize(r)},n.prototype.handleWrapJump=function(t){var e=(t-(void 0===this._prevLng?t:this._prevLng))/360,n=Math.round(e);if(this._prevLng=t,n){var r={};for(var i in this._tiles){var o=this._tiles[i];o.tileID=o.tileID.unwrapTo(o.tileID.wrap+n),r[o.tileID.key]=o}for(var a in this._tiles=r,this._timers)clearTimeout(this._timers[a]),delete this._timers[a];for(var s in this._tiles){var A=this._tiles[s];this._setTileReloadTimer(s,A)}}},n.prototype.update=function(e){var r=this;if(this.transform=e,this._sourceLoaded&&!this._paused){var i;this.updateCacheSize(e),this.handleWrapJump(this.transform.center.lng),this._coveredTiles={},this.used?this._source.tileID?i=e.getVisibleUnwrappedCoordinates(this._source.tileID).map(function(e){return new t.OverscaledTileID(e.canonical.z,e.wrap,e.canonical.z,e.canonical.x,e.canonical.y)}):(i=e.coveringTiles({tileSize:this._source.tileSize,minzoom:this._source.minzoom,maxzoom:this._source.maxzoom,roundZoom:this._source.roundZoom,reparseOverscaled:this._source.reparseOverscaled}),this._source.hasTile&&(i=i.filter(function(t){return r._source.hasTile(t)}))):i=[];var o=(this._source.roundZoom?Math.round:Math.floor)(this.getZoom(e)),a=Math.max(o-n.maxOverzooming,this._source.minzoom),s=Math.max(o+n.maxUnderzooming,this._source.minzoom),A=this._updateRetainedTiles(i,o);if(kt(this._source.type)){for(var l={},c={},u=0,g=Object.keys(A);u<g.length;u+=1){var h=g[u],d=A[h],f=this._tiles[h];if(f&&!(f.fadeEndTime&&f.fadeEndTime<=t.browser.now())){var p=this.findLoadedParent(d,a);p&&(this._addTile(p.tileID),l[p.tileID.key]=p.tileID),c[h]=d}}for(var I in this._retainLoadedChildren(c,o,s,A),l)A[I]||(this._coveredTiles[I]=!0,A[I]=l[I])}for(var C in A)this._tiles[C].clearFadeHold();for(var m=0,y=t.keysDifference(this._tiles,A);m<y.length;m+=1){var E=y[m],v=this._tiles[E];v.hasSymbolBuckets&&!v.holdingForFade()?v.setHoldDuration(this.map._fadeDuration):v.hasSymbolBuckets&&!v.symbolFadeFinished()||this._removeTile(E)}}},n.prototype.releaseSymbolFadeTiles=function(){for(var t in this._tiles)this._tiles[t].holdingForFade()&&this._removeTile(t)},n.prototype._updateRetainedTiles=function(t,e){for(var r={},i={},o=Math.max(e-n.maxOverzooming,this._source.minzoom),a=Math.max(e+n.maxUnderzooming,this._source.minzoom),s={},A=0,l=t;A<l.length;A+=1){var c=l[A],u=this._addTile(c);r[c.key]=c,u.hasData()||e<this._source.maxzoom&&(s[c.key]=c)}this._retainLoadedChildren(s,e,a,r);for(var g=0,h=t;g<h.length;g+=1){var d=h[g],f=this._tiles[d.key];if(!f.hasData()){if(e+1>this._source.maxzoom){var p=d.children(this._source.maxzoom)[0],I=this.getTile(p);if(I&&I.hasData()){r[p.key]=p;continue}}else{var C=d.children(this._source.maxzoom);if(r[C[0].key]&&r[C[1].key]&&r[C[2].key]&&r[C[3].key])continue}for(var m=f.wasRequested(),y=d.overscaledZ-1;y>=o;--y){var E=d.scaledTo(y);if(i[E.key])break;if(i[E.key]=!0,!(f=this.getTile(E))&&m&&(f=this._addTile(E)),f&&(r[E.key]=E,m=f.wasRequested(),f.hasData()))break}}}return r},n.prototype._addTile=function(e){var n=this._tiles[e.key];if(n)return n;(n=this._cache.getAndRemove(e))&&(this._setTileReloadTimer(e.key,n),n.tileID=e,this._state.initializeTileState(n,this.map?this.map.painter:null),this._cacheTimers[e.key]&&(clearTimeout(this._cacheTimers[e.key]),delete this._cacheTimers[e.key],this._setTileReloadTimer(e.key,n)));var r=Boolean(n);return r||(n=new t.Tile(e,this._source.tileSize*e.overscaleFactor()),this._loadTile(n,this._tileLoaded.bind(this,n,e.key,n.state))),n?(n.uses++,this._tiles[e.key]=n,r||this._source.fire(new t.Event("dataloading",{tile:n,coord:n.tileID,dataType:"source"})),n):null},n.prototype._setTileReloadTimer=function(t,e){var n=this;t in this._timers&&(clearTimeout(this._timers[t]),delete this._timers[t]);var r=e.getExpiryTimeout();r&&(this._timers[t]=setTimeout(function(){n._reloadTile(t,"expired"),delete n._timers[t]},r))},n.prototype._removeTile=function(t){var e=this._tiles[t];e&&(e.uses--,delete this._tiles[t],this._timers[t]&&(clearTimeout(this._timers[t]),delete this._timers[t]),e.uses>0||(e.hasData()?this._cache.add(e.tileID,e,e.getExpiryTimeout()):(e.aborted=!0,this._abortTile(e),this._unloadTile(e))))},n.prototype.clearTiles=function(){for(var t in this._shouldReloadOnResume=!1,this._paused=!1,this._tiles)this._removeTile(t);this._cache.reset()},n.prototype.tilesIn=function(e,n,r){var i=this,o=[],a=this.transform;if(!a)return o;for(var s=r?a.getCameraQueryGeometry(e):e,A=e.map(function(t){return a.pointCoordinate(t)}),l=s.map(function(t){return a.pointCoordinate(t)}),c=this.getIds(),u=1/0,g=1/0,h=-1/0,d=-1/0,f=0,p=l;f<p.length;f+=1){var I=p[f];u=Math.min(u,I.x),g=Math.min(g,I.y),h=Math.max(h,I.x),d=Math.max(d,I.y)}for(var C=function(e){var r=i._tiles[c[e]];if(!r.holdingForFade()){var s=r.tileID,f=Math.pow(2,a.zoom-r.tileID.overscaledZ),p=n*r.queryPadding*t.EXTENT/r.tileSize/f,I=[s.getTilePoint(new t.MercatorCoordinate(u,g)),s.getTilePoint(new t.MercatorCoordinate(h,d))];if(I[0].x-p<t.EXTENT&&I[0].y-p<t.EXTENT&&I[1].x+p>=0&&I[1].y+p>=0){var C=A.map(function(t){return s.getTilePoint(t)}),m=l.map(function(t){return s.getTilePoint(t)});o.push({tile:r,tileID:s,queryGeometry:C,cameraQueryGeometry:m,scale:f})}}},m=0;m<c.length;m++)C(m);return o},n.prototype.getVisibleCoordinates=function(t){for(var e=this,n=this.getRenderableIds(t).map(function(t){return e._tiles[t].tileID}),r=0,i=n;r<i.length;r+=1){var o=i[r];o.posMatrix=this.transform.calculatePosMatrix(o.toUnwrapped())}return n},n.prototype.hasTransition=function(){if(this._source.hasTransition())return!0;if(kt(this._source.type))for(var e in this._tiles){var n=this._tiles[e];if(void 0!==n.fadeEndTime&&n.fadeEndTime>=t.browser.now())return!0}return!1},n.prototype.setFeatureState=function(t,e,n){t=t||"_geojsonTileLayer",this._state.updateState(t,e,n)},n.prototype.removeFeatureState=function(t,e,n){t=t||"_geojsonTileLayer",this._state.removeFeatureState(t,e,n)},n.prototype.getFeatureState=function(t,e){return t=t||"_geojsonTileLayer",this._state.getState(t,e)},n}(t.Evented);function Mt(t,e){return t%32-e%32||e-t}function kt(t){return"raster"===t||"image"===t||"video"===t}function St(){return new t.window.Worker(Wr.workerUrl)}Dt.maxOverzooming=10,Dt.maxUnderzooming=3;var Lt=function(){this.active={}};Lt.prototype.acquire=function(t){if(!this.workers)for(this.workers=[];this.workers.length<Lt.workerCount;)this.workers.push(new St);return this.active[t]=!0,this.workers.slice()},Lt.prototype.release=function(t){delete this.active[t],0===Object.keys(this.active).length&&(this.workers.forEach(function(t){t.terminate()}),this.workers=null)};var Rt,Nt=Math.floor(t.browser.hardwareConcurrency/2);function Ft(e,n){var r={};for(var i in e)"ref"!==i&&(r[i]=e[i]);return t.refProperties.forEach(function(t){t in n&&(r[t]=n[t])}),r}function Tt(t){t=t.slice();for(var e=Object.create(null),n=0;n<t.length;n++)e[t[n].id]=t[n];for(var r=0;r<t.length;r++)"ref"in t[r]&&(t[r]=Ft(t[r],e[t[r].ref]));return t}Lt.workerCount=Math.max(Math.min(Nt,6),1);var Ut={setStyle:"setStyle",addLayer:"addLayer",removeLayer:"removeLayer",setPaintProperty:"setPaintProperty",setLayoutProperty:"setLayoutProperty",setFilter:"setFilter",addSource:"addSource",removeSource:"removeSource",setGeoJSONSourceData:"setGeoJSONSourceData",setLayerZoomRange:"setLayerZoomRange",setLayerProperty:"setLayerProperty",setCenter:"setCenter",setZoom:"setZoom",setBearing:"setBearing",setPitch:"setPitch",setSprite:"setSprite",setGlyphs:"setGlyphs",setTransition:"setTransition",setLight:"setLight"};function Ot(t,e,n){n.push({command:Ut.addSource,args:[t,e[t]]})}function Gt(t,e,n){e.push({command:Ut.removeSource,args:[t]}),n[t]=!0}function Ht(t,e,n,r){Gt(t,n,r),Ot(t,e,n)}function jt(e,n,r){var i;for(i in e[r])if(e[r].hasOwnProperty(i)&&"data"!==i&&!t.deepEqual(e[r][i],n[r][i]))return!1;for(i in n[r])if(n[r].hasOwnProperty(i)&&"data"!==i&&!t.deepEqual(e[r][i],n[r][i]))return!1;return!0}function Yt(e,n,r,i,o,a){var s;for(s in n=n||{},e=e||{})e.hasOwnProperty(s)&&(t.deepEqual(e[s],n[s])||r.push({command:a,args:[i,s,n[s],o]}));for(s in n)n.hasOwnProperty(s)&&!e.hasOwnProperty(s)&&(t.deepEqual(e[s],n[s])||r.push({command:a,args:[i,s,n[s],o]}))}function Pt(t){return t.id}function Kt(t,e){return t[e.id]=e,t}var zt=function(t,e,n){var r=this.boxCells=[],i=this.circleCells=[];this.xCellCount=Math.ceil(t/n),this.yCellCount=Math.ceil(e/n);for(var o=0;o<this.xCellCount*this.yCellCount;o++)r.push([]),i.push([]);this.circleKeys=[],this.boxKeys=[],this.bboxes=[],this.circles=[],this.width=t,this.height=e,this.xScale=this.xCellCount/t,this.yScale=this.yCellCount/e,this.boxUid=0,this.circleUid=0};function Jt(e,n,r,i,o){var a=t.create();return n?(t.scale(a,a,[1/o,1/o,1]),r||t.rotateZ(a,a,i.angle)):t.multiply(a,i.labelPlaneMatrix,e),a}function qt(e,n,r,i,o){if(n){var a=t.clone(e);return t.scale(a,a,[o,o,1]),r||t.rotateZ(a,a,-i.angle),a}return i.glCoordMatrix}function Vt(e,n){var r=[e.x,e.y,0,1];oe(r,r,n);var i=r[3];return{point:new t.Point(r[0]/i,r[1]/i),signedDistanceFromCamera:i}}function Wt(t,e){var n=t[0]/t[3],r=t[1]/t[3];return n>=-e[0]&&n<=e[0]&&r>=-e[1]&&r<=e[1]}function Xt(e,n,r,i,o,a,s,A){var l=i?e.textSizeData:e.iconSizeData,c=t.evaluateSizeForZoom(l,r.transform.zoom),u=[256/r.width*2+1,256/r.height*2+1],g=i?e.text.dynamicLayoutVertexArray:e.icon.dynamicLayoutVertexArray;g.clear();for(var h=e.lineVertexArray,d=i?e.text.placedSymbolArray:e.icon.placedSymbolArray,f=r.transform.width/r.transform.height,p=!1,I=0;I<d.length;I++){var C=d.get(I);if(C.hidden||C.writingMode===t.WritingMode.vertical&&!p)ie(C.numGlyphs,g);else{p=!1;var m=[C.anchorX,C.anchorY,0,1];if(t.transformMat4(m,m,n),Wt(m,u)){var y=.5+m[3]/r.transform.cameraToCenterDistance*.5,E=t.evaluateSizeForFeature(l,c,C),v=s?E*y:E/y,B=new t.Point(C.anchorX,C.anchorY),b=Vt(B,o).point,Q={},w=te(C,v,!1,A,n,o,a,e.glyphOffsetArray,h,g,b,B,Q,f);p=w.useVertical,(w.notEnoughRoom||p||w.needsFlipping&&te(C,v,!0,A,n,o,a,e.glyphOffsetArray,h,g,b,B,Q,f).notEnoughRoom)&&ie(C.numGlyphs,g)}else ie(C.numGlyphs,g)}}i?e.text.dynamicLayoutVertexBuffer.updateData(g):e.icon.dynamicLayoutVertexBuffer.updateData(g)}function Zt(t,e,n,r,i,o,a,s,A,l,c,u){var g=s.glyphStartIndex+s.numGlyphs,h=s.lineStartIndex,d=s.lineStartIndex+s.lineLength,f=e.getoffsetX(s.glyphStartIndex),p=e.getoffsetX(g-1),I=ne(t*f,n,r,i,o,a,s.segment,h,d,A,l,c,u);if(!I)return null;var C=ne(t*p,n,r,i,o,a,s.segment,h,d,A,l,c,u);return C?{first:I,last:C}:null}function $t(e,n,r,i){return e===t.WritingMode.horizontal&&Math.abs(r.y-n.y)>Math.abs(r.x-n.x)*i?{useVertical:!0}:(e===t.WritingMode.vertical?n.y<r.y:n.x>r.x)?{needsFlipping:!0}:null}function te(e,n,r,i,o,a,s,A,l,c,u,g,h,d){var f,p=n/24,I=e.lineOffsetX*p,C=e.lineOffsetY*p;if(e.numGlyphs>1){var m=e.glyphStartIndex+e.numGlyphs,y=e.lineStartIndex,E=e.lineStartIndex+e.lineLength,v=Zt(p,A,I,C,r,u,g,e,l,a,h,!1);if(!v)return{notEnoughRoom:!0};var B=Vt(v.first.point,s).point,b=Vt(v.last.point,s).point;if(i&&!r){var Q=$t(e.writingMode,B,b,d);if(Q)return Q}f=[v.first];for(var w=e.glyphStartIndex+1;w<m-1;w++)f.push(ne(p*A.getoffsetX(w),I,C,r,u,g,e.segment,y,E,l,a,h,!1));f.push(v.last)}else{if(i&&!r){var x=Vt(g,o).point,_=e.lineStartIndex+e.segment+1,D=new t.Point(l.getx(_),l.gety(_)),M=Vt(D,o),k=M.signedDistanceFromCamera>0?M.point:ee(g,D,x,1,o),S=$t(e.writingMode,x,k,d);if(S)return S}var L=ne(p*A.getoffsetX(e.glyphStartIndex),I,C,r,u,g,e.segment,e.lineStartIndex,e.lineStartIndex+e.lineLength,l,a,h,!1);if(!L)return{notEnoughRoom:!0};f=[L]}for(var R=0,N=f;R<N.length;R+=1){var F=N[R];t.addDynamicAttributes(c,F.point,F.angle)}return{}}function ee(t,e,n,r,i){var o=Vt(t.add(t.sub(e)._unit()),i).point,a=n.sub(o);return n.add(a._mult(r/a.mag()))}function ne(e,n,r,i,o,a,s,A,l,c,u,g,h){var d=i?e-n:e+n,f=d>0?1:-1,p=0;i&&(f*=-1,p=Math.PI),f<0&&(p+=Math.PI);for(var I=f>0?A+s:A+s+1,C=I,m=o,y=o,E=0,v=0,B=Math.abs(d);E+v<=B;){if((I+=f)<A||I>=l)return null;if(y=m,void 0===(m=g[I])){var b=new t.Point(c.getx(I),c.gety(I)),Q=Vt(b,u);if(Q.signedDistanceFromCamera>0)m=g[I]=Q.point;else{var w=I-f;m=ee(0===E?a:new t.Point(c.getx(w),c.gety(w)),b,y,B-E+1,u)}}E+=v,v=y.dist(m)}var x=(B-E)/v,_=m.sub(y),D=_.mult(x)._add(y);return D._add(_._unit()._perp()._mult(r*f)),{point:D,angle:p+Math.atan2(m.y-y.y,m.x-y.x),tileDistance:h?{prevTileDistance:I-f===C?0:c.gettileUnitDistanceFromAnchor(I-f),lastSegmentViewportDistance:B-E}:null}}zt.prototype.keysLength=function(){return this.boxKeys.length+this.circleKeys.length},zt.prototype.insert=function(t,e,n,r,i){this._forEachCell(e,n,r,i,this._insertBoxCell,this.boxUid++),this.boxKeys.push(t),this.bboxes.push(e),this.bboxes.push(n),this.bboxes.push(r),this.bboxes.push(i)},zt.prototype.insertCircle=function(t,e,n,r){this._forEachCell(e-r,n-r,e+r,n+r,this._insertCircleCell,this.circleUid++),this.circleKeys.push(t),this.circles.push(e),this.circles.push(n),this.circles.push(r)},zt.prototype._insertBoxCell=function(t,e,n,r,i,o){this.boxCells[i].push(o)},zt.prototype._insertCircleCell=function(t,e,n,r,i,o){this.circleCells[i].push(o)},zt.prototype._query=function(t,e,n,r,i,o){if(n<0||t>this.width||r<0||e>this.height)return!i&&[];var a=[];if(t<=0&&e<=0&&this.width<=n&&this.height<=r){if(i)return!0;for(var s=0;s<this.boxKeys.length;s++)a.push({key:this.boxKeys[s],x1:this.bboxes[4*s],y1:this.bboxes[4*s+1],x2:this.bboxes[4*s+2],y2:this.bboxes[4*s+3]});for(var A=0;A<this.circleKeys.length;A++){var l=this.circles[3*A],c=this.circles[3*A+1],u=this.circles[3*A+2];a.push({key:this.circleKeys[A],x1:l-u,y1:c-u,x2:l+u,y2:c+u})}return o?a.filter(o):a}var g={hitTest:i,seenUids:{box:{},circle:{}}};return this._forEachCell(t,e,n,r,this._queryCell,a,g,o),i?a.length>0:a},zt.prototype._queryCircle=function(t,e,n,r,i){var o=t-n,a=t+n,s=e-n,A=e+n;if(a<0||o>this.width||A<0||s>this.height)return!r&&[];var l=[],c={hitTest:r,circle:{x:t,y:e,radius:n},seenUids:{box:{},circle:{}}};return this._forEachCell(o,s,a,A,this._queryCellCircle,l,c,i),r?l.length>0:l},zt.prototype.query=function(t,e,n,r,i){return this._query(t,e,n,r,!1,i)},zt.prototype.hitTest=function(t,e,n,r,i){return this._query(t,e,n,r,!0,i)},zt.prototype.hitTestCircle=function(t,e,n,r){return this._queryCircle(t,e,n,!0,r)},zt.prototype._queryCell=function(t,e,n,r,i,o,a,s){var A=a.seenUids,l=this.boxCells[i];if(null!==l)for(var c=this.bboxes,u=0,g=l;u<g.length;u+=1){var h=g[u];if(!A.box[h]){A.box[h]=!0;var d=4*h;if(t<=c[d+2]&&e<=c[d+3]&&n>=c[d+0]&&r>=c[d+1]&&(!s||s(this.boxKeys[h]))){if(a.hitTest)return o.push(!0),!0;o.push({key:this.boxKeys[h],x1:c[d],y1:c[d+1],x2:c[d+2],y2:c[d+3]})}}}var f=this.circleCells[i];if(null!==f)for(var p=this.circles,I=0,C=f;I<C.length;I+=1){var m=C[I];if(!A.circle[m]){A.circle[m]=!0;var y=3*m;if(this._circleAndRectCollide(p[y],p[y+1],p[y+2],t,e,n,r)&&(!s||s(this.circleKeys[m]))){if(a.hitTest)return o.push(!0),!0;var E=p[y],v=p[y+1],B=p[y+2];o.push({key:this.circleKeys[m],x1:E-B,y1:v-B,x2:E+B,y2:v+B})}}}},zt.prototype._queryCellCircle=function(t,e,n,r,i,o,a,s){var A=a.circle,l=a.seenUids,c=this.boxCells[i];if(null!==c)for(var u=this.bboxes,g=0,h=c;g<h.length;g+=1){var d=h[g];if(!l.box[d]){l.box[d]=!0;var f=4*d;if(this._circleAndRectCollide(A.x,A.y,A.radius,u[f+0],u[f+1],u[f+2],u[f+3])&&(!s||s(this.boxKeys[d])))return o.push(!0),!0}}var p=this.circleCells[i];if(null!==p)for(var I=this.circles,C=0,m=p;C<m.length;C+=1){var y=m[C];if(!l.circle[y]){l.circle[y]=!0;var E=3*y;if(this._circlesCollide(I[E],I[E+1],I[E+2],A.x,A.y,A.radius)&&(!s||s(this.circleKeys[y])))return o.push(!0),!0}}},zt.prototype._forEachCell=function(t,e,n,r,i,o,a,s){for(var A=this._convertToXCellCoord(t),l=this._convertToYCellCoord(e),c=this._convertToXCellCoord(n),u=this._convertToYCellCoord(r),g=A;g<=c;g++)for(var h=l;h<=u;h++){var d=this.xCellCount*h+g;if(i.call(this,t,e,n,r,d,o,a,s))return}},zt.prototype._convertToXCellCoord=function(t){return Math.max(0,Math.min(this.xCellCount-1,Math.floor(t*this.xScale)))},zt.prototype._convertToYCellCoord=function(t){return Math.max(0,Math.min(this.yCellCount-1,Math.floor(t*this.yScale)))},zt.prototype._circlesCollide=function(t,e,n,r,i,o){var a=r-t,s=i-e,A=n+o;return A*A>a*a+s*s},zt.prototype._circleAndRectCollide=function(t,e,n,r,i,o,a){var s=(o-r)/2,A=Math.abs(t-(r+s));if(A>s+n)return!1;var l=(a-i)/2,c=Math.abs(e-(i+l));if(c>l+n)return!1;if(A<=s||c<=l)return!0;var u=A-s,g=c-l;return u*u+g*g<=n*n};var re=new Float32Array([-1/0,-1/0,0,-1/0,-1/0,0,-1/0,-1/0,0,-1/0,-1/0,0]);function ie(t,e){for(var n=0;n<t;n++){var r=e.length;e.resize(r+4),e.float32.set(re,3*r)}}function oe(t,e,n){var r=e[0],i=e[1];return t[0]=n[0]*r+n[4]*i+n[12],t[1]=n[1]*r+n[5]*i+n[13],t[3]=n[3]*r+n[7]*i+n[15],t}var ae=function(t,e,n){void 0===e&&(e=new zt(t.width+200,t.height+200,25)),void 0===n&&(n=new zt(t.width+200,t.height+200,25)),this.transform=t,this.grid=e,this.ignoredGrid=n,this.pitchfactor=Math.cos(t._pitch)*t.cameraToCenterDistance,this.screenRightBoundary=t.width+100,this.screenBottomBoundary=t.height+100,this.gridRightBoundary=t.width+200,this.gridBottomBoundary=t.height+200};function se(t,e,n){t[e+4]=n?1:0}function Ae(e,n,r){return n*(t.EXTENT/(e.tileSize*Math.pow(2,r-e.tileID.overscaledZ)))}ae.prototype.placeCollisionBox=function(t,e,n,r,i){var o=this.projectAndGetPerspectiveRatio(r,t.anchorPointX,t.anchorPointY),a=n*o.perspectiveRatio,s=t.x1*a+o.point.x,A=t.y1*a+o.point.y,l=t.x2*a+o.point.x,c=t.y2*a+o.point.y;return!this.isInsideGrid(s,A,l,c)||!e&&this.grid.hitTest(s,A,l,c,i)?{box:[],offscreen:!1}:{box:[s,A,l,c],offscreen:this.isOffscreen(s,A,l,c)}},ae.prototype.approximateTileDistance=function(t,e,n,r,i){var o=i?1:r/this.pitchfactor,a=t.lastSegmentViewportDistance*n;return t.prevTileDistance+a+(o-1)*a*Math.abs(Math.sin(e))},ae.prototype.placeCollisionCircles=function(e,n,r,i,o,a,s,A,l,c,u,g,h){var d=[],f=this.projectAnchor(l,o.anchorX,o.anchorY),p=A/24,I=o.lineOffsetX*A,C=o.lineOffsetY*A,m=new t.Point(o.anchorX,o.anchorY),y=Zt(p,s,I,C,!1,Vt(m,c).point,m,o,a,c,{},!0),E=!1,v=!1,B=!0,b=f.perspectiveRatio*i,Q=1/(i*r),w=0,x=0;y&&(w=this.approximateTileDistance(y.first.tileDistance,y.first.angle,Q,f.cameraDistance,g),x=this.approximateTileDistance(y.last.tileDistance,y.last.angle,Q,f.cameraDistance,g));for(var _=0;_<e.length;_+=5){var D=e[_],M=e[_+1],k=e[_+2],S=e[_+3];if(!y||S<-w||S>x)se(e,_,!1);else{var L=this.projectPoint(l,D,M),R=k*b;if(d.length>0){var N=L.x-d[d.length-4],F=L.y-d[d.length-3];if(R*R*2>N*N+F*F&&_+8<e.length){var T=e[_+8];if(T>-w&&T<x){se(e,_,!1);continue}}}var U=_/5;d.push(L.x,L.y,R,U),se(e,_,!0);var O=L.x-R,G=L.y-R,H=L.x+R,j=L.y+R;if(B=B&&this.isOffscreen(O,G,H,j),v=v||this.isInsideGrid(O,G,H,j),!n&&this.grid.hitTestCircle(L.x,L.y,R,h)){if(!u)return{circles:[],offscreen:!1};E=!0}}}return{circles:E||!v?[]:d,offscreen:B}},ae.prototype.queryRenderedSymbols=function(e){if(0===e.length||0===this.grid.keysLength()&&0===this.ignoredGrid.keysLength())return{};for(var n=[],r=1/0,i=1/0,o=-1/0,a=-1/0,s=0,A=e;s<A.length;s+=1){var l=A[s],c=new t.Point(l.x+100,l.y+100);r=Math.min(r,c.x),i=Math.min(i,c.y),o=Math.max(o,c.x),a=Math.max(a,c.y),n.push(c)}for(var u={},g={},h=0,d=this.grid.query(r,i,o,a).concat(this.ignoredGrid.query(r,i,o,a));h<d.length;h+=1){var f=d[h],p=f.key;if(void 0===u[p.bucketInstanceId]&&(u[p.bucketInstanceId]={}),!u[p.bucketInstanceId][p.featureIndex]){var I=[new t.Point(f.x1,f.y1),new t.Point(f.x2,f.y1),new t.Point(f.x2,f.y2),new t.Point(f.x1,f.y2)];t.polygonIntersectsPolygon(n,I)&&(u[p.bucketInstanceId][p.featureIndex]=!0,void 0===g[p.bucketInstanceId]&&(g[p.bucketInstanceId]=[]),g[p.bucketInstanceId].push(p.featureIndex))}}return g},ae.prototype.insertCollisionBox=function(t,e,n,r,i){var o={bucketInstanceId:n,featureIndex:r,collisionGroupID:i};(e?this.ignoredGrid:this.grid).insert(o,t[0],t[1],t[2],t[3])},ae.prototype.insertCollisionCircles=function(t,e,n,r,i){for(var o=e?this.ignoredGrid:this.grid,a={bucketInstanceId:n,featureIndex:r,collisionGroupID:i},s=0;s<t.length;s+=4)o.insertCircle(a,t[s],t[s+1],t[s+2])},ae.prototype.projectAnchor=function(t,e,n){var r=[e,n,0,1];return oe(r,r,t),{perspectiveRatio:.5+this.transform.cameraToCenterDistance/r[3]*.5,cameraDistance:r[3]}},ae.prototype.projectPoint=function(e,n,r){var i=[n,r,0,1];return oe(i,i,e),new t.Point((i[0]/i[3]+1)/2*this.transform.width+100,(-i[1]/i[3]+1)/2*this.transform.height+100)},ae.prototype.projectAndGetPerspectiveRatio=function(e,n,r){var i=[n,r,0,1];return oe(i,i,e),{point:new t.Point((i[0]/i[3]+1)/2*this.transform.width+100,(-i[1]/i[3]+1)/2*this.transform.height+100),perspectiveRatio:.5+this.transform.cameraToCenterDistance/i[3]*.5}},ae.prototype.isOffscreen=function(t,e,n,r){return n<100||t>=this.screenRightBoundary||r<100||e>this.screenBottomBoundary},ae.prototype.isInsideGrid=function(t,e,n,r){return n>=0&&t<this.gridRightBoundary&&r>=0&&e<this.gridBottomBoundary};var le=function(t,e,n,r){this.opacity=t?Math.max(0,Math.min(1,t.opacity+(t.placed?e:-e))):r&&n?1:0,this.placed=n};le.prototype.isHidden=function(){return 0===this.opacity&&!this.placed};var ce=function(t,e,n,r,i){this.text=new le(t?t.text:null,e,n,i),this.icon=new le(t?t.icon:null,e,r,i)};ce.prototype.isHidden=function(){return this.text.isHidden()&&this.icon.isHidden()};var ue=function(t,e,n){this.text=t,this.icon=e,this.skipFade=n},ge=function(t,e,n,r,i){this.bucketInstanceId=t,this.featureIndex=e,this.sourceLayerIndex=n,this.bucketIndex=r,this.tileID=i},he=function(t){this.crossSourceCollisions=t,this.maxGroupID=0,this.collisionGroups={}};function de(e,n,r,i,o){var a=t.getAnchorAlignment(e),s=-(a.horizontalAlign-.5)*n,A=-(a.verticalAlign-.5)*r,l=t.evaluateRadialOffset(e,i);return new t.Point(s+l[0]*o,A+l[1]*o)}he.prototype.get=function(t){if(this.crossSourceCollisions)return{ID:0,predicate:null};if(!this.collisionGroups[t]){var e=++this.maxGroupID;this.collisionGroups[t]={ID:e,predicate:function(t){return t.collisionGroupID===e}}}return this.collisionGroups[t]};var fe=function(t,e,n,r){this.transform=t.clone(),this.collisionIndex=new ae(this.transform),this.placements={},this.opacities={},this.variableOffsets={},this.stale=!1,this.commitTime=0,this.fadeDuration=e,this.retainedQueryData={},this.collisionGroups=new he(n),this.prevPlacement=r,r&&(r.prevPlacement=void 0)};function pe(t,e,n,r,i){t.emplaceBack(e?1:0,n?1:0,r||0,i||0),t.emplaceBack(e?1:0,n?1:0,r||0,i||0),t.emplaceBack(e?1:0,n?1:0,r||0,i||0),t.emplaceBack(e?1:0,n?1:0,r||0,i||0)}fe.prototype.placeLayerTile=function(e,n,r,i){var o=n.getBucket(e),a=n.latestFeatureIndex;if(o&&a&&e.id===o.layerIds[0]){var s=n.collisionBoxArray,A=o.layers[0].layout,l=Math.pow(2,this.transform.zoom-n.tileID.overscaledZ),c=n.tileSize/t.EXTENT,u=this.transform.calculatePosMatrix(n.tileID.toUnwrapped()),g=Jt(u,"map"===A.get("text-pitch-alignment"),"map"===A.get("text-rotation-alignment"),this.transform,Ae(n,1,this.transform.zoom)),h=Jt(u,"map"===A.get("icon-pitch-alignment"),"map"===A.get("icon-rotation-alignment"),this.transform,Ae(n,1,this.transform.zoom));this.retainedQueryData[o.bucketInstanceId]=new ge(o.bucketInstanceId,a,o.sourceLayerIndex,o.index,n.tileID),this.placeLayerBucket(o,u,g,h,l,c,r,n.holdingForFade(),i,s)}},fe.prototype.attemptAnchorPlacement=function(e,n,r,i,o,a,s,A,l,c,u,g,h,d){var f,p=de(e,r,i,o,a),I=this.collisionIndex.placeCollisionBox(function(e,n,r,i,o,a){var s=e.x1,A=e.x2,l=e.y1,c=e.y2,u=e.anchorPointX,g=e.anchorPointY,h=new t.Point(n,r);return i&&h._rotate(o?a:-a),{x1:s+h.x,y1:l+h.y,x2:A+h.x,y2:c+h.y,anchorPointX:u,anchorPointY:g}}(n,p.x,p.y,s,A,this.transform.angle),g,l,c,u.predicate);if(I.box.length>0)return this.prevPlacement&&this.prevPlacement.variableOffsets[h.crossTileID]&&this.prevPlacement.placements[h.crossTileID]&&this.prevPlacement.placements[h.crossTileID].text&&(f=this.prevPlacement.variableOffsets[h.crossTileID].anchor),this.variableOffsets[h.crossTileID]={radialOffset:o,width:r,height:i,anchor:e,textBoxScale:a,prevAnchor:f},this.markUsedJustification(d,e,h),I},fe.prototype.placeLayerBucket=function(e,n,r,i,o,a,s,A,l,c){var u=this,g=e.layers[0].layout,h=t.evaluateSizeForZoom(e.textSizeData,this.transform.zoom),d=g.get("text-optional"),f=g.get("icon-optional"),p=g.get("text-allow-overlap"),I=g.get("icon-allow-overlap"),C=p&&(I||!e.hasIconData()||f),m=I&&(p||!e.hasTextData()||d),y=this.collisionGroups.get(e.sourceID),E="map"===g.get("text-rotation-alignment"),v="map"===g.get("text-pitch-alignment"),B="viewport-y"===g.get("symbol-z-order");!e.collisionArrays&&c&&e.deserializeCollisionBoxes(c);var b=function(i,c){if(!l[i.crossTileID])if(A)u.placements[i.crossTileID]=new ue(!1,!1,!1);else{var I=!1,B=!1,b=!0,Q=null,w=null,x=null,_=0,D=0;c.textFeatureIndex&&(_=c.textFeatureIndex);var M=c.textBox;if(M)if(g.get("text-variable-anchor")){var k=M.x2-M.x1,S=M.y2-M.y1,L=i.textBoxScale,R=g.get("text-variable-anchor");if(u.prevPlacement&&u.prevPlacement.variableOffsets[i.crossTileID]){var N=u.prevPlacement.variableOffsets[i.crossTileID];R[0]!==N.anchor&&(R=R.filter(function(t){return t!==N.anchor})).unshift(N.anchor)}for(var F=0,T=R;F<T.length;F+=1){var U=T[F];if(Q=u.attemptAnchorPlacement(U,M,k,S,i.radialTextOffset,L,E,v,a,n,y,p,i,e)){I=!0;break}}if(!u.variableOffsets[i.crossTileID]&&u.prevPlacement){var O=u.prevPlacement.variableOffsets[i.crossTileID];O&&(u.variableOffsets[i.crossTileID]=O,u.markUsedJustification(e,O.anchor,i))}}else I=(Q=u.collisionIndex.placeCollisionBox(M,g.get("text-allow-overlap"),a,n,y.predicate)).box.length>0;b=Q&&Q.offscreen;var G=c.textCircles;if(G){var H=e.text.placedSymbolArray.get(i.centerJustifiedTextSymbolIndex),j=t.evaluateSizeForFeature(e.textSizeData,h,H);w=u.collisionIndex.placeCollisionCircles(G,g.get("text-allow-overlap"),o,a,H,e.lineVertexArray,e.glyphOffsetArray,j,n,r,s,v,y.predicate),I=g.get("text-allow-overlap")||w.circles.length>0,b=b&&w.offscreen}c.iconFeatureIndex&&(D=c.iconFeatureIndex),c.iconBox&&(B=(x=u.collisionIndex.placeCollisionBox(c.iconBox,g.get("icon-allow-overlap"),a,n,y.predicate)).box.length>0,b=b&&x.offscreen);var Y=d||0===i.numHorizontalGlyphVertices&&0===i.numVerticalGlyphVertices,P=f||0===i.numIconVertices;Y||P?P?Y||(B=B&&I):I=B&&I:B=I=B&&I,I&&Q&&u.collisionIndex.insertCollisionBox(Q.box,g.get("text-ignore-placement"),e.bucketInstanceId,_,y.ID),B&&x&&u.collisionIndex.insertCollisionBox(x.box,g.get("icon-ignore-placement"),e.bucketInstanceId,D,y.ID),I&&w&&u.collisionIndex.insertCollisionCircles(w.circles,g.get("text-ignore-placement"),e.bucketInstanceId,_,y.ID),u.placements[i.crossTileID]=new ue(I||C,B||m,b||e.justReloaded),l[i.crossTileID]=!0}};if(B)for(var Q=e.getSortedSymbolIndexes(this.transform.angle),w=Q.length-1;w>=0;--w){var x=Q[w];b(e.symbolInstances.get(x),e.collisionArrays[x])}else for(var _=0;_<e.symbolInstances.length;++_)b(e.symbolInstances.get(_),e.collisionArrays[_]);e.justReloaded=!1},fe.prototype.markUsedJustification=function(e,n,r){var i={left:r.leftJustifiedTextSymbolIndex,center:r.centerJustifiedTextSymbolIndex,right:r.rightJustifiedTextSymbolIndex},o=i[t.getAnchorJustification(n)];for(var a in i){var s=i[a];s>=0&&(e.text.placedSymbolArray.get(s).crossTileID=o>=0&&s!==o?0:r.crossTileID)}},fe.prototype.commit=function(t){this.commitTime=t;var e=this.prevPlacement,n=!1,r=e&&0!==this.fadeDuration?(this.commitTime-e.commitTime)/this.fadeDuration:1,i=e?e.opacities:{},o=e?e.variableOffsets:{};for(var a in this.placements){var s=this.placements[a],A=i[a];A?(this.opacities[a]=new ce(A,r,s.text,s.icon),n=n||s.text!==A.text.placed||s.icon!==A.icon.placed):(this.opacities[a]=new ce(null,r,s.text,s.icon,s.skipFade),n=n||s.text||s.icon)}for(var l in i){var c=i[l];if(!this.opacities[l]){var u=new ce(c,r,!1,!1);u.isHidden()||(this.opacities[l]=u,n=n||c.text.placed||c.icon.placed)}}for(var g in o)this.variableOffsets[g]||!this.opacities[g]||this.opacities[g].isHidden()||(this.variableOffsets[g]=o[g]);n?this.lastPlacementChangeTime=t:"number"!=typeof this.lastPlacementChangeTime&&(this.lastPlacementChangeTime=e?e.lastPlacementChangeTime:t)},fe.prototype.updateLayerOpacities=function(t,e){for(var n={},r=0,i=e;r<i.length;r+=1){var o=i[r],a=o.getBucket(t);a&&o.latestFeatureIndex&&t.id===a.layerIds[0]&&this.updateBucketOpacities(a,n,o.collisionBoxArray)}},fe.prototype.updateBucketOpacities=function(e,n,r){e.hasTextData()&&e.text.opacityVertexArray.clear(),e.hasIconData()&&e.icon.opacityVertexArray.clear(),e.hasCollisionBoxData()&&e.collisionBox.collisionVertexArray.clear(),e.hasCollisionCircleData()&&e.collisionCircle.collisionVertexArray.clear();var i=e.layers[0].layout,o=new ce(null,0,!1,!1,!0),a=i.get("text-allow-overlap"),s=i.get("icon-allow-overlap"),A=i.get("text-variable-anchor"),l="map"===i.get("text-rotation-alignment"),c="map"===i.get("text-pitch-alignment"),u=new ce(null,0,a&&(s||!e.hasIconData()||i.get("icon-optional")),s&&(a||!e.hasTextData()||i.get("text-optional")),!0);!e.collisionArrays&&r&&(e.hasCollisionBoxData()||e.hasCollisionCircleData())&&e.deserializeCollisionBoxes(r);for(var g=0;g<e.symbolInstances.length;g++){var h=e.symbolInstances.get(g),d=h.numHorizontalGlyphVertices,f=h.numVerticalGlyphVertices,p=h.crossTileID,I=n[p],C=this.opacities[p];I?C=o:C||(C=u,this.opacities[p]=C),n[p]=!0;var m=d>0||f>0,y=h.numIconVertices>0;if(m){for(var E=be(C.text),v=(d+f)/4,B=0;B<v;B++)e.text.opacityVertexArray.emplaceBack(E);var b=C.text.isHidden()?1:0;[h.rightJustifiedTextSymbolIndex,h.centerJustifiedTextSymbolIndex,h.leftJustifiedTextSymbolIndex,h.verticalPlacedTextSymbolIndex].forEach(function(t){t>=0&&(e.text.placedSymbolArray.get(t).hidden=b)});var Q=this.variableOffsets[h.crossTileID];Q&&this.markUsedJustification(e,Q.anchor,h)}if(y){for(var w=be(C.icon),x=0;x<h.numIconVertices/4;x++)e.icon.opacityVertexArray.emplaceBack(w);e.icon.placedSymbolArray.get(g).hidden=C.icon.isHidden()}if(e.hasCollisionBoxData()||e.hasCollisionCircleData()){var _=e.collisionArrays[g];if(_){if(_.textBox){var D=new t.Point(0,0),M=!0;if(A){var k=this.variableOffsets[p];k?(D=de(k.anchor,k.width,k.height,k.radialOffset,k.textBoxScale),l&&D._rotate(c?this.transform.angle:-this.transform.angle)):M=!1}pe(e.collisionBox.collisionVertexArray,C.text.placed,!M,D.x,D.y)}_.iconBox&&pe(e.collisionBox.collisionVertexArray,C.icon.placed,!1);var S=_.textCircles;if(S&&e.hasCollisionCircleData())for(var L=0;L<S.length;L+=5){var R=I||0===S[L+4];pe(e.collisionCircle.collisionVertexArray,C.text.placed,R)}}}}e.sortFeatures(this.transform.angle),this.retainedQueryData[e.bucketInstanceId]&&(this.retainedQueryData[e.bucketInstanceId].featureSortOrder=e.featureSortOrder),e.hasTextData()&&e.text.opacityVertexBuffer&&e.text.opacityVertexBuffer.updateData(e.text.opacityVertexArray),e.hasIconData()&&e.icon.opacityVertexBuffer&&e.icon.opacityVertexBuffer.updateData(e.icon.opacityVertexArray),e.hasCollisionBoxData()&&e.collisionBox.collisionVertexBuffer&&e.collisionBox.collisionVertexBuffer.updateData(e.collisionBox.collisionVertexArray),e.hasCollisionCircleData()&&e.collisionCircle.collisionVertexBuffer&&e.collisionCircle.collisionVertexBuffer.updateData(e.collisionCircle.collisionVertexArray)},fe.prototype.symbolFadeChange=function(t){return 0===this.fadeDuration?1:(t-this.commitTime)/this.fadeDuration},fe.prototype.hasTransitions=function(t){return this.stale||t-this.lastPlacementChangeTime<this.fadeDuration},fe.prototype.stillRecent=function(t){return this.commitTime+this.fadeDuration>t},fe.prototype.setStale=function(){this.stale=!0};var Ie=Math.pow(2,25),Ce=Math.pow(2,24),me=Math.pow(2,17),ye=Math.pow(2,16),Ee=Math.pow(2,9),ve=Math.pow(2,8),Be=Math.pow(2,1);function be(t){if(0===t.opacity&&!t.placed)return 0;if(1===t.opacity&&t.placed)return 4294967295;var e=t.placed?1:0,n=Math.floor(127*t.opacity);return n*Ie+e*Ce+n*me+e*ye+n*Ee+e*ve+n*Be+e}var Qe=function(){this._currentTileIndex=0,this._seenCrossTileIDs={}};Qe.prototype.continuePlacement=function(t,e,n,r,i){for(;this._currentTileIndex<t.length;){var o=t[this._currentTileIndex];if(e.placeLayerTile(r,o,n,this._seenCrossTileIDs),this._currentTileIndex++,i())return!0}};var we=function(t,e,n,r,i,o,a){this.placement=new fe(t,i,o,a),this._currentPlacementIndex=e.length-1,this._forceFullPlacement=n,this._showCollisionBoxes=r,this._done=!1};we.prototype.isDone=function(){return this._done},we.prototype.continuePlacement=function(e,n,r){for(var i=this,o=t.browser.now(),a=function(){var e=t.browser.now()-o;return!i._forceFullPlacement&&e>2};this._currentPlacementIndex>=0;){var s=n[e[this._currentPlacementIndex]],A=this.placement.collisionIndex.transform.zoom;if("symbol"===s.type&&(!s.minzoom||s.minzoom<=A)&&(!s.maxzoom||s.maxzoom>A)){if(this._inProgressLayer||(this._inProgressLayer=new Qe),this._inProgressLayer.continuePlacement(r[s.source],this.placement,this._showCollisionBoxes,s,a))return;delete this._inProgressLayer}this._currentPlacementIndex--}this._done=!0},we.prototype.commit=function(t){return this.placement.commit(t),this.placement};var xe=512/t.EXTENT/2,_e=function(t,e,n){this.tileID=t,this.indexedSymbolInstances={},this.bucketInstanceId=n;for(var r=0;r<e.length;r++){var i=e.get(r),o=i.key;this.indexedSymbolInstances[o]||(this.indexedSymbolInstances[o]=[]),this.indexedSymbolInstances[o].push({crossTileID:i.crossTileID,coord:this.getScaledCoordinates(i,t)})}};_e.prototype.getScaledCoordinates=function(e,n){var r=n.canonical.z-this.tileID.canonical.z,i=xe/Math.pow(2,r);return{x:Math.floor((n.canonical.x*t.EXTENT+e.anchorX)*i),y:Math.floor((n.canonical.y*t.EXTENT+e.anchorY)*i)}},_e.prototype.findMatches=function(t,e,n){for(var r=this.tileID.canonical.z<e.canonical.z?1:Math.pow(2,this.tileID.canonical.z-e.canonical.z),i=0;i<t.length;i++){var o=t.get(i);if(!o.crossTileID){var a=this.indexedSymbolInstances[o.key];if(a)for(var s=this.getScaledCoordinates(o,e),A=0,l=a;A<l.length;A+=1){var c=l[A];if(Math.abs(c.coord.x-s.x)<=r&&Math.abs(c.coord.y-s.y)<=r&&!n[c.crossTileID]){n[c.crossTileID]=!0,o.crossTileID=c.crossTileID;break}}}}};var De=function(){this.maxCrossTileID=0};De.prototype.generate=function(){return++this.maxCrossTileID};var Me=function(){this.indexes={},this.usedCrossTileIDs={},this.lng=0};Me.prototype.handleWrapJump=function(t){var e=Math.round((t-this.lng)/360);if(0!==e)for(var n in this.indexes){var r=this.indexes[n],i={};for(var o in r){var a=r[o];a.tileID=a.tileID.unwrapTo(a.tileID.wrap+e),i[a.tileID.key]=a}this.indexes[n]=i}this.lng=t},Me.prototype.addBucket=function(t,e,n){if(this.indexes[t.overscaledZ]&&this.indexes[t.overscaledZ][t.key]){if(this.indexes[t.overscaledZ][t.key].bucketInstanceId===e.bucketInstanceId)return!1;this.removeBucketCrossTileIDs(t.overscaledZ,this.indexes[t.overscaledZ][t.key])}for(var r=0;r<e.symbolInstances.length;r++)e.symbolInstances.get(r).crossTileID=0;this.usedCrossTileIDs[t.overscaledZ]||(this.usedCrossTileIDs[t.overscaledZ]={});var i=this.usedCrossTileIDs[t.overscaledZ];for(var o in this.indexes){var a=this.indexes[o];if(Number(o)>t.overscaledZ)for(var s in a){var A=a[s];A.tileID.isChildOf(t)&&A.findMatches(e.symbolInstances,t,i)}else{var l=a[t.scaledTo(Number(o)).key];l&&l.findMatches(e.symbolInstances,t,i)}}for(var c=0;c<e.symbolInstances.length;c++){var u=e.symbolInstances.get(c);u.crossTileID||(u.crossTileID=n.generate(),i[u.crossTileID]=!0)}return void 0===this.indexes[t.overscaledZ]&&(this.indexes[t.overscaledZ]={}),this.indexes[t.overscaledZ][t.key]=new _e(t,e.symbolInstances,e.bucketInstanceId),!0},Me.prototype.removeBucketCrossTileIDs=function(t,e){for(var n in e.indexedSymbolInstances)for(var r=0,i=e.indexedSymbolInstances[n];r<i.length;r+=1){var o=i[r];delete this.usedCrossTileIDs[t][o.crossTileID]}},Me.prototype.removeStaleBuckets=function(t){var e=!1;for(var n in this.indexes){var r=this.indexes[n];for(var i in r)t[r[i].bucketInstanceId]||(this.removeBucketCrossTileIDs(n,r[i]),delete r[i],e=!0)}return e};var ke=function(){this.layerIndexes={},this.crossTileIDs=new De,this.maxBucketInstanceId=0,this.bucketsInCurrentPlacement={}};ke.prototype.addLayer=function(t,e,n){var r=this.layerIndexes[t.id];void 0===r&&(r=this.layerIndexes[t.id]=new Me);var i=!1,o={};r.handleWrapJump(n);for(var a=0,s=e;a<s.length;a+=1){var A=s[a],l=A.getBucket(t);l&&t.id===l.layerIds[0]&&(l.bucketInstanceId||(l.bucketInstanceId=++this.maxBucketInstanceId),r.addBucket(A.tileID,l,this.crossTileIDs)&&(i=!0),o[l.bucketInstanceId]=!0)}return r.removeStaleBuckets(o)&&(i=!0),i},ke.prototype.pruneUnusedLayers=function(t){var e={};for(var n in t.forEach(function(t){e[t]=!0}),this.layerIndexes)e[n]||delete this.layerIndexes[n]};var Se=function(e,n){return t.emitValidationErrors(e,n&&n.filter(function(t){return"source.canvas"!==t.identifier}))},Le=t.pick(Ut,["addLayer","removeLayer","setPaintProperty","setLayoutProperty","setFilter","addSource","removeSource","setLayerZoomRange","setLight","setTransition","setGeoJSONSourceData"]),Re=t.pick(Ut,["setCenter","setZoom","setBearing","setPitch"]),Ne=function(e){function n(r,i){var o=this;void 0===i&&(i={}),e.call(this),this.map=r,this.dispatcher=new B((Rt||(Rt=new Lt),Rt),this),this.imageManager=new g,this.imageManager.setEventedParent(this),this.glyphManager=new C(r._requestManager,i.localIdeographFontFamily),this.lineAtlas=new v(256,512),this.crossTileSymbolIndex=new ke,this._layers={},this._order=[],this.sourceCaches={},this.zoomHistory=new t.ZoomHistory,this._loaded=!1,this._resetUpdates(),this.dispatcher.broadcast("setReferrer",t.getReferrer());var a=this;this._rtlTextPluginCallback=n.registerForPluginAvailability(function(t){for(var e in a.dispatcher.broadcast("loadRTLTextPlugin",t.pluginURL,t.completionCallback),a.sourceCaches)a.sourceCaches[e].reload()}),this.on("data",function(t){if("source"===t.dataType&&"metadata"===t.sourceDataType){var e=o.sourceCaches[t.sourceId];if(e){var n=e.getSource();if(n&&n.vectorLayerIds)for(var r in o._layers){var i=o._layers[r];i.source===n.id&&o._validateLayer(i)}}}})}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.loadURL=function(e,n){var r=this;void 0===n&&(n={}),this.fire(new t.Event("dataloading",{dataType:"style"}));var i="boolean"==typeof n.validate?n.validate:!t.isMapboxURL(e);e=this.map._requestManager.normalizeStyleURL(e,n.accessToken);var o=this.map._requestManager.transformRequest(e,t.ResourceType.Style);this._request=t.getJSON(o,function(e,n){r._request=null,e?r.fire(new t.ErrorEvent(e)):n&&r._load(n,i)})},n.prototype.loadJSON=function(e,n){var r=this;void 0===n&&(n={}),this.fire(new t.Event("dataloading",{dataType:"style"})),this._request=t.browser.frame(function(){r._request=null,r._load(e,!1!==n.validate)})},n.prototype._load=function(e,n){var r=this;if(!n||!Se(this,t.validateStyle(e))){for(var i in this._loaded=!0,this.stylesheet=e,e.sources)this.addSource(i,e.sources[i],{validate:!1});e.sprite?this._spriteRequest=function(e,n,r){var i,o,a,s=t.browser.devicePixelRatio>1?"@2x":"",A=t.getJSON(n.transformRequest(n.normalizeSpriteURL(e,s,".json"),t.ResourceType.SpriteJSON),function(t,e){A=null,a||(a=t,i=e,c())}),l=t.getImage(n.transformRequest(n.normalizeSpriteURL(e,s,".png"),t.ResourceType.SpriteImage),function(t,e){l=null,a||(a=t,o=e,c())});function c(){if(a)r(a);else if(i&&o){var e=t.browser.getImageData(o),n={};for(var s in i){var A=i[s],l=A.width,c=A.height,u=A.x,g=A.y,h=A.sdf,d=A.pixelRatio,f=new t.RGBAImage({width:l,height:c});t.RGBAImage.copy(e,f,{x:u,y:g},{x:0,y:0},{width:l,height:c}),n[s]={data:f,pixelRatio:d,sdf:h}}r(null,n)}}return{cancel:function(){A&&(A.cancel(),A=null),l&&(l.cancel(),l=null)}}}(e.sprite,this.map._requestManager,function(e,n){if(r._spriteRequest=null,e)r.fire(new t.ErrorEvent(e));else if(n)for(var i in n)r.imageManager.addImage(i,n[i]);r.imageManager.setLoaded(!0),r.fire(new t.Event("data",{dataType:"style"}))}):this.imageManager.setLoaded(!0),this.glyphManager.setURL(e.glyphs);var o=Tt(this.stylesheet.layers);this._order=o.map(function(t){return t.id}),this._layers={};for(var a=0,s=o;a<s.length;a+=1){var A=s[a];(A=t.createStyleLayer(A)).setEventedParent(this,{layer:{id:A.id}}),this._layers[A.id]=A}this.dispatcher.broadcast("setLayers",this._serializeLayers(this._order)),this.light=new E(this.stylesheet.light),this.fire(new t.Event("data",{dataType:"style"})),this.fire(new t.Event("style.load"))}},n.prototype._validateLayer=function(e){var n=this.sourceCaches[e.source];if(n){var r=e.sourceLayer;if(r){var i=n.getSource();("geojson"===i.type||i.vectorLayerIds&&-1===i.vectorLayerIds.indexOf(r))&&this.fire(new t.ErrorEvent(new Error('Source layer "'+r+'" does not exist on source "'+i.id+'" as specified by style layer "'+e.id+'"')))}}},n.prototype.loaded=function(){if(!this._loaded)return!1;if(Object.keys(this._updatedSources).length)return!1;for(var t in this.sourceCaches)if(!this.sourceCaches[t].loaded())return!1;return!!this.imageManager.isLoaded()},n.prototype._serializeLayers=function(t){for(var e=[],n=0,r=t;n<r.length;n+=1){var i=r[n],o=this._layers[i];"custom"!==o.type&&e.push(o.serialize())}return e},n.prototype.hasTransitions=function(){if(this.light&&this.light.hasTransition())return!0;for(var t in this.sourceCaches)if(this.sourceCaches[t].hasTransition())return!0;for(var e in this._layers)if(this._layers[e].hasTransition())return!0;return!1},n.prototype._checkLoaded=function(){if(!this._loaded)throw new Error("Style is not done loading")},n.prototype.update=function(e){if(this._loaded){var n=this._changed;if(this._changed){var r=Object.keys(this._updatedLayers),i=Object.keys(this._removedLayers);for(var o in(r.length||i.length)&&this._updateWorkerLayers(r,i),this._updatedSources){var a=this._updatedSources[o];"reload"===a?this._reloadSource(o):"clear"===a&&this._clearSource(o)}for(var s in this._updatedPaintProps)this._layers[s].updateTransitions(e);this.light.updateTransitions(e),this._resetUpdates()}for(var A in this.sourceCaches)this.sourceCaches[A].used=!1;for(var l=0,c=this._order;l<c.length;l+=1){var u=c[l],g=this._layers[u];g.recalculate(e),!g.isHidden(e.zoom)&&g.source&&(this.sourceCaches[g.source].used=!0)}this.light.recalculate(e),this.z=e.zoom,n&&this.fire(new t.Event("data",{dataType:"style"}))}},n.prototype._updateWorkerLayers=function(t,e){this.dispatcher.broadcast("updateLayers",{layers:this._serializeLayers(t),removedIds:e})},n.prototype._resetUpdates=function(){this._changed=!1,this._updatedLayers={},this._removedLayers={},this._updatedSources={},this._updatedPaintProps={}},n.prototype.setState=function(e){var n=this;if(this._checkLoaded(),Se(this,t.validateStyle(e)))return!1;(e=t.clone$1(e)).layers=Tt(e.layers);var r=function(e,n){if(!e)return[{command:Ut.setStyle,args:[n]}];var r=[];try{if(!t.deepEqual(e.version,n.version))return[{command:Ut.setStyle,args:[n]}];t.deepEqual(e.center,n.center)||r.push({command:Ut.setCenter,args:[n.center]}),t.deepEqual(e.zoom,n.zoom)||r.push({command:Ut.setZoom,args:[n.zoom]}),t.deepEqual(e.bearing,n.bearing)||r.push({command:Ut.setBearing,args:[n.bearing]}),t.deepEqual(e.pitch,n.pitch)||r.push({command:Ut.setPitch,args:[n.pitch]}),t.deepEqual(e.sprite,n.sprite)||r.push({command:Ut.setSprite,args:[n.sprite]}),t.deepEqual(e.glyphs,n.glyphs)||r.push({command:Ut.setGlyphs,args:[n.glyphs]}),t.deepEqual(e.transition,n.transition)||r.push({command:Ut.setTransition,args:[n.transition]}),t.deepEqual(e.light,n.light)||r.push({command:Ut.setLight,args:[n.light]});var i={},o=[];!function(e,n,r,i){var o;for(o in n=n||{},e=e||{})e.hasOwnProperty(o)&&(n.hasOwnProperty(o)||Gt(o,r,i));for(o in n)n.hasOwnProperty(o)&&(e.hasOwnProperty(o)?t.deepEqual(e[o],n[o])||("geojson"===e[o].type&&"geojson"===n[o].type&&jt(e,n,o)?r.push({command:Ut.setGeoJSONSourceData,args:[o,n[o].data]}):Ht(o,n,r,i)):Ot(o,n,r))}(e.sources,n.sources,o,i);var a=[];e.layers&&e.layers.forEach(function(t){i[t.source]?r.push({command:Ut.removeLayer,args:[t.id]}):a.push(t)}),r=r.concat(o),function(e,n,r){n=n||[];var i,o,a,s,A,l,c,u=(e=e||[]).map(Pt),g=n.map(Pt),h=e.reduce(Kt,{}),d=n.reduce(Kt,{}),f=u.slice(),p=Object.create(null);for(i=0,o=0;i<u.length;i++)a=u[i],d.hasOwnProperty(a)?o++:(r.push({command:Ut.removeLayer,args:[a]}),f.splice(f.indexOf(a,o),1));for(i=0,o=0;i<g.length;i++)a=g[g.length-1-i],f[f.length-1-i]!==a&&(h.hasOwnProperty(a)?(r.push({command:Ut.removeLayer,args:[a]}),f.splice(f.lastIndexOf(a,f.length-o),1)):o++,l=f[f.length-i],r.push({command:Ut.addLayer,args:[d[a],l]}),f.splice(f.length-i,0,a),p[a]=!0);for(i=0;i<g.length;i++)if(s=h[a=g[i]],A=d[a],!p[a]&&!t.deepEqual(s,A))if(t.deepEqual(s.source,A.source)&&t.deepEqual(s["source-layer"],A["source-layer"])&&t.deepEqual(s.type,A.type)){for(c in Yt(s.layout,A.layout,r,a,null,Ut.setLayoutProperty),Yt(s.paint,A.paint,r,a,null,Ut.setPaintProperty),t.deepEqual(s.filter,A.filter)||r.push({command:Ut.setFilter,args:[a,A.filter]}),t.deepEqual(s.minzoom,A.minzoom)&&t.deepEqual(s.maxzoom,A.maxzoom)||r.push({command:Ut.setLayerZoomRange,args:[a,A.minzoom,A.maxzoom]}),s)s.hasOwnProperty(c)&&"layout"!==c&&"paint"!==c&&"filter"!==c&&"metadata"!==c&&"minzoom"!==c&&"maxzoom"!==c&&(0===c.indexOf("paint.")?Yt(s[c],A[c],r,a,c.slice(6),Ut.setPaintProperty):t.deepEqual(s[c],A[c])||r.push({command:Ut.setLayerProperty,args:[a,c,A[c]]}));for(c in A)A.hasOwnProperty(c)&&!s.hasOwnProperty(c)&&"layout"!==c&&"paint"!==c&&"filter"!==c&&"metadata"!==c&&"minzoom"!==c&&"maxzoom"!==c&&(0===c.indexOf("paint.")?Yt(s[c],A[c],r,a,c.slice(6),Ut.setPaintProperty):t.deepEqual(s[c],A[c])||r.push({command:Ut.setLayerProperty,args:[a,c,A[c]]}))}else r.push({command:Ut.removeLayer,args:[a]}),l=f[f.lastIndexOf(a)+1],r.push({command:Ut.addLayer,args:[A,l]})}(a,n.layers,r)}catch(t){console.warn("Unable to compute style diff:",t),r=[{command:Ut.setStyle,args:[n]}]}return r}(this.serialize(),e).filter(function(t){return!(t.command in Re)});if(0===r.length)return!1;var i=r.filter(function(t){return!(t.command in Le)});if(i.length>0)throw new Error("Unimplemented: "+i.map(function(t){return t.command}).join(", ")+".");return r.forEach(function(t){"setTransition"!==t.command&&n[t.command].apply(n,t.args)}),this.stylesheet=e,!0},n.prototype.addImage=function(e,n){if(this.getImage(e))return this.fire(new t.ErrorEvent(new Error("An image with this name already exists.")));this.imageManager.addImage(e,n),this.fire(new t.Event("data",{dataType:"style"}))},n.prototype.updateImage=function(t,e){this.imageManager.updateImage(t,e)},n.prototype.getImage=function(t){return this.imageManager.getImage(t)},n.prototype.removeImage=function(e){if(!this.getImage(e))return this.fire(new t.ErrorEvent(new Error("No image with this name exists.")));this.imageManager.removeImage(e),this.fire(new t.Event("data",{dataType:"style"}))},n.prototype.listImages=function(){return this._checkLoaded(),this.imageManager.listImages()},n.prototype.addSource=function(e,n,r){var i=this;if(void 0===r&&(r={}),this._checkLoaded(),void 0!==this.sourceCaches[e])throw new Error("There is already a source with this ID");if(!n.type)throw new Error("The type property must be defined, but the only the following properties were given: "+Object.keys(n).join(", ")+".");if(!(["vector","raster","geojson","video","image"].indexOf(n.type)>=0&&this._validate(t.validateStyle.source,"sources."+e,n,null,r))){this.map&&this.map._collectResourceTiming&&(n.collectResourceTiming=!0);var o=this.sourceCaches[e]=new Dt(e,n,this.dispatcher);o.style=this,o.setEventedParent(this,function(){return{isSourceLoaded:i.loaded(),source:o.serialize(),sourceId:e}}),o.onAdd(this.map),this._changed=!0}},n.prototype.removeSource=function(e){if(this._checkLoaded(),void 0===this.sourceCaches[e])throw new Error("There is no source with this ID");for(var n in this._layers)if(this._layers[n].source===e)return this.fire(new t.ErrorEvent(new Error('Source "'+e+'" cannot be removed while layer "'+n+'" is using it.')));var r=this.sourceCaches[e];delete this.sourceCaches[e],delete this._updatedSources[e],r.fire(new t.Event("data",{sourceDataType:"metadata",dataType:"source",sourceId:e})),r.setEventedParent(null),r.clearTiles(),r.onRemove&&r.onRemove(this.map),this._changed=!0},n.prototype.setGeoJSONSourceData=function(t,e){this._checkLoaded(),this.sourceCaches[t].getSource().setData(e),this._changed=!0},n.prototype.getSource=function(t){return this.sourceCaches[t]&&this.sourceCaches[t].getSource()},n.prototype.addLayer=function(e,n,r){void 0===r&&(r={}),this._checkLoaded();var i=e.id;if(this.getLayer(i))this.fire(new t.ErrorEvent(new Error('Layer with id "'+i+'" already exists on this map')));else{var o;if("custom"===e.type){if(Se(this,t.validateCustomStyleLayer(e)))return;o=t.createStyleLayer(e)}else{if("object"==typeof e.source&&(this.addSource(i,e.source),e=t.clone$1(e),e=t.extend(e,{source:i})),this._validate(t.validateStyle.layer,"layers."+i,e,{arrayIndex:-1},r))return;o=t.createStyleLayer(e),this._validateLayer(o),o.setEventedParent(this,{layer:{id:i}})}var a=n?this._order.indexOf(n):this._order.length;if(n&&-1===a)this.fire(new t.ErrorEvent(new Error('Layer with id "'+n+'" does not exist on this map.')));else{if(this._order.splice(a,0,i),this._layerOrderChanged=!0,this._layers[i]=o,this._removedLayers[i]&&o.source&&"custom"!==o.type){var s=this._removedLayers[i];delete this._removedLayers[i],s.type!==o.type?this._updatedSources[o.source]="clear":(this._updatedSources[o.source]="reload",this.sourceCaches[o.source].pause())}this._updateLayer(o),o.onAdd&&o.onAdd(this.map)}}},n.prototype.moveLayer=function(e,n){if(this._checkLoaded(),this._changed=!0,this._layers[e]){if(e!==n){var r=this._order.indexOf(e);this._order.splice(r,1);var i=n?this._order.indexOf(n):this._order.length;n&&-1===i?this.fire(new t.ErrorEvent(new Error('Layer with id "'+n+'" does not exist on this map.'))):(this._order.splice(i,0,e),this._layerOrderChanged=!0)}}else this.fire(new t.ErrorEvent(new Error("The layer '"+e+"' does not exist in the map's style and cannot be moved.")))},n.prototype.removeLayer=function(e){this._checkLoaded();var n=this._layers[e];if(n){n.setEventedParent(null);var r=this._order.indexOf(e);this._order.splice(r,1),this._layerOrderChanged=!0,this._changed=!0,this._removedLayers[e]=n,delete this._layers[e],delete this._updatedLayers[e],delete this._updatedPaintProps[e],n.onRemove&&n.onRemove(this.map)}else this.fire(new t.ErrorEvent(new Error("The layer '"+e+"' does not exist in the map's style and cannot be removed.")))},n.prototype.getLayer=function(t){return this._layers[t]},n.prototype.setLayerZoomRange=function(e,n,r){this._checkLoaded();var i=this.getLayer(e);i?i.minzoom===n&&i.maxzoom===r||(null!=n&&(i.minzoom=n),null!=r&&(i.maxzoom=r),this._updateLayer(i)):this.fire(new t.ErrorEvent(new Error("The layer '"+e+"' does not exist in the map's style and cannot have zoom extent.")))},n.prototype.setFilter=function(e,n,r){void 0===r&&(r={}),this._checkLoaded();var i=this.getLayer(e);if(i){if(!t.deepEqual(i.filter,n))return null==n?(i.filter=void 0,void this._updateLayer(i)):void(this._validate(t.validateStyle.filter,"layers."+i.id+".filter",n,null,r)||(i.filter=t.clone$1(n),this._updateLayer(i)))}else this.fire(new t.ErrorEvent(new Error("The layer '"+e+"' does not exist in the map's style and cannot be filtered.")))},n.prototype.getFilter=function(e){return t.clone$1(this.getLayer(e).filter)},n.prototype.setLayoutProperty=function(e,n,r,i){void 0===i&&(i={}),this._checkLoaded();var o=this.getLayer(e);o?t.deepEqual(o.getLayoutProperty(n),r)||(o.setLayoutProperty(n,r,i),this._updateLayer(o)):this.fire(new t.ErrorEvent(new Error("The layer '"+e+"' does not exist in the map's style and cannot be styled.")))},n.prototype.getLayoutProperty=function(e,n){var r=this.getLayer(e);if(r)return r.getLayoutProperty(n);this.fire(new t.ErrorEvent(new Error("The layer '"+e+"' does not exist in the map's style.")))},n.prototype.setPaintProperty=function(e,n,r,i){void 0===i&&(i={}),this._checkLoaded();var o=this.getLayer(e);o?t.deepEqual(o.getPaintProperty(n),r)||(o.setPaintProperty(n,r,i)&&this._updateLayer(o),this._changed=!0,this._updatedPaintProps[e]=!0):this.fire(new t.ErrorEvent(new Error("The layer '"+e+"' does not exist in the map's style and cannot be styled.")))},n.prototype.getPaintProperty=function(t,e){return this.getLayer(t).getPaintProperty(e)},n.prototype.setFeatureState=function(e,n){this._checkLoaded();var r=e.source,i=e.sourceLayer,o=this.sourceCaches[r],a=parseInt(e.id,10);if(void 0!==o){var s=o.getSource().type;"geojson"===s&&i?this.fire(new t.ErrorEvent(new Error("GeoJSON sources cannot have a sourceLayer parameter."))):"vector"!==s||i?isNaN(a)||a<0?this.fire(new t.ErrorEvent(new Error("The feature id parameter must be provided and non-negative."))):o.setFeatureState(i,a,n):this.fire(new t.ErrorEvent(new Error("The sourceLayer parameter must be provided for vector source types.")))}else this.fire(new t.ErrorEvent(new Error("The source '"+r+"' does not exist in the map's style.")))},n.prototype.removeFeatureState=function(e,n){this._checkLoaded();var r=e.source,i=this.sourceCaches[r];if(void 0!==i){var o=i.getSource().type,a="vector"===o?e.sourceLayer:void 0,s=parseInt(e.id,10);"vector"!==o||a?void 0!==e.id&&isNaN(s)||s<0?this.fire(new t.ErrorEvent(new Error("The feature id parameter must be non-negative."))):n&&"string"!=typeof e.id&&"number"!=typeof e.id?this.fire(new t.ErrorEvent(new Error("A feature id is requred to remove its specific state property."))):i.removeFeatureState(a,s,n):this.fire(new t.ErrorEvent(new Error("The sourceLayer parameter must be provided for vector source types.")))}else this.fire(new t.ErrorEvent(new Error("The source '"+r+"' does not exist in the map's style.")))},n.prototype.getFeatureState=function(e){this._checkLoaded();var n=e.source,r=e.sourceLayer,i=this.sourceCaches[n],o=parseInt(e.id,10);if(void 0!==i)if("vector"!==i.getSource().type||r){if(!(isNaN(o)||o<0))return i.getFeatureState(r,o);this.fire(new t.ErrorEvent(new Error("The feature id parameter must be provided and non-negative.")))}else this.fire(new t.ErrorEvent(new Error("The sourceLayer parameter must be provided for vector source types.")));else this.fire(new t.ErrorEvent(new Error("The source '"+n+"' does not exist in the map's style.")))},n.prototype.getTransition=function(){return t.extend({duration:300,delay:0},this.stylesheet&&this.stylesheet.transition)},n.prototype.serialize=function(){return t.filterObject({version:this.stylesheet.version,name:this.stylesheet.name,metadata:this.stylesheet.metadata,light:this.stylesheet.light,center:this.stylesheet.center,zoom:this.stylesheet.zoom,bearing:this.stylesheet.bearing,pitch:this.stylesheet.pitch,sprite:this.stylesheet.sprite,glyphs:this.stylesheet.glyphs,transition:this.stylesheet.transition,sources:t.mapObject(this.sourceCaches,function(t){return t.serialize()}),layers:this._serializeLayers(this._order)},function(t){return void 0!==t})},n.prototype._updateLayer=function(t){this._updatedLayers[t.id]=!0,t.source&&!this._updatedSources[t.source]&&(this._updatedSources[t.source]="reload",this.sourceCaches[t.source].pause()),this._changed=!0},n.prototype._flattenAndSortRenderedFeatures=function(t){for(var e=this,n=function(t){return"fill-extrusion"===e._layers[t].type},r={},i=[],o=this._order.length-1;o>=0;o--){var a=this._order[o];if(n(a)){r[a]=o;for(var s=0,A=t;s<A.length;s+=1){var l=A[s][a];if(l)for(var c=0,u=l;c<u.length;c+=1){var g=u[c];i.push(g)}}}}i.sort(function(t,e){return e.intersectionZ-t.intersectionZ});for(var h=[],d=this._order.length-1;d>=0;d--){var f=this._order[d];if(n(f))for(var p=i.length-1;p>=0;p--){var I=i[p].feature;if(r[I.layer.id]<d)break;h.push(I),i.pop()}else for(var C=0,m=t;C<m.length;C+=1){var y=m[C][f];if(y)for(var E=0,v=y;E<v.length;E+=1){var B=v[E];h.push(B.feature)}}}return h},n.prototype.queryRenderedFeatures=function(e,n,r){n&&n.filter&&this._validate(t.validateStyle.filter,"queryRenderedFeatures.filter",n.filter,null,n);var i={};if(n&&n.layers){if(!Array.isArray(n.layers))return this.fire(new t.ErrorEvent(new Error("parameters.layers must be an Array."))),[];for(var o=0,a=n.layers;o<a.length;o+=1){var s=a[o],A=this._layers[s];if(!A)return this.fire(new t.ErrorEvent(new Error("The layer '"+s+"' does not exist in the map's style and cannot be queried for features."))),[];i[A.source]=!0}}var l=[];for(var c in this.sourceCaches)n.layers&&!i[c]||l.push(F(this.sourceCaches[c],this._layers,e,n,r));return this.placement&&l.push(function(t,e,n,r,i,o){for(var a={},s=i.queryRenderedSymbols(n),A=[],l=0,c=Object.keys(s).map(Number);l<c.length;l+=1){var u=c[l];A.push(o[u])}A.sort(T);for(var g=function(){var e=d[h],n=e.featureIndex.lookupSymbolFeatures(s[e.bucketInstanceId],e.bucketIndex,e.sourceLayerIndex,r.filter,r.layers,t);for(var i in n){var o=a[i]=a[i]||[],A=n[i];A.sort(function(t,n){var r=e.featureSortOrder;if(r){var i=r.indexOf(t.featureIndex);return r.indexOf(n.featureIndex)-i}return n.featureIndex-t.featureIndex});for(var l=0,c=A;l<c.length;l+=1){var u=c[l];o.push(u)}}},h=0,d=A;h<d.length;h+=1)g();var f=function(n){a[n].forEach(function(r){var i=r.feature,o=t[n],a=e[o.source].getFeatureState(i.layer["source-layer"],i.id);i.source=i.layer.source,i.layer["source-layer"]&&(i.sourceLayer=i.layer["source-layer"]),i.state=a})};for(var p in a)f(p);return a}(this._layers,this.sourceCaches,e,n,this.placement.collisionIndex,this.placement.retainedQueryData)),this._flattenAndSortRenderedFeatures(l)},n.prototype.querySourceFeatures=function(e,n){n&&n.filter&&this._validate(t.validateStyle.filter,"querySourceFeatures.filter",n.filter,null,n);var r=this.sourceCaches[e];return r?function(t,e){for(var n=t.getRenderableIds().map(function(e){return t.getTileByID(e)}),r=[],i={},o=0;o<n.length;o++){var a=n[o],s=a.tileID.canonical.key;i[s]||(i[s]=!0,a.querySourceFeatures(r,e))}return r}(r,n):[]},n.prototype.addSourceType=function(t,e,r){return n.getSourceType(t)?r(new Error('A source type called "'+t+'" already exists.')):(n.setSourceType(t,e),e.workerSourceURL?void this.dispatcher.broadcast("loadWorkerSource",{name:t,url:e.workerSourceURL},r):r(null,null))},n.prototype.getLight=function(){return this.light.getLight()},n.prototype.setLight=function(e,n){void 0===n&&(n={}),this._checkLoaded();var r=this.light.getLight(),i=!1;for(var o in e)if(!t.deepEqual(e[o],r[o])){i=!0;break}if(i){var a={now:t.browser.now(),transition:t.extend({duration:300,delay:0},this.stylesheet.transition)};this.light.setLight(e,n),this.light.updateTransitions(a)}},n.prototype._validate=function(e,n,r,i,o){return void 0===o&&(o={}),(!o||!1!==o.validate)&&Se(this,e.call(t.validateStyle,t.extend({key:n,style:this.serialize(),value:r,styleSpec:t.styleSpec},i)))},n.prototype._remove=function(){for(var e in this._request&&(this._request.cancel(),this._request=null),this._spriteRequest&&(this._spriteRequest.cancel(),this._spriteRequest=null),t.evented.off("pluginAvailable",this._rtlTextPluginCallback),this.sourceCaches)this.sourceCaches[e].clearTiles();this.dispatcher.remove()},n.prototype._clearSource=function(t){this.sourceCaches[t].clearTiles()},n.prototype._reloadSource=function(t){this.sourceCaches[t].resume(),this.sourceCaches[t].reload()},n.prototype._updateSources=function(t){for(var e in this.sourceCaches)this.sourceCaches[e].update(t)},n.prototype._generateCollisionBoxes=function(){for(var t in this.sourceCaches)this._reloadSource(t)},n.prototype._updatePlacement=function(e,n,r,i){for(var o=!1,a=!1,s={},A=0,l=this._order;A<l.length;A+=1){var c=l[A],u=this._layers[c];if("symbol"===u.type){if(!s[u.source]){var g=this.sourceCaches[u.source];s[u.source]=g.getRenderableIds(!0).map(function(t){return g.getTileByID(t)}).sort(function(t,e){return e.tileID.overscaledZ-t.tileID.overscaledZ||(t.tileID.isLessThan(e.tileID)?-1:1)})}var h=this.crossTileSymbolIndex.addLayer(u,s[u.source],e.center.lng);o=o||h}}this.crossTileSymbolIndex.pruneUnusedLayers(this._order);var d=this._layerOrderChanged||0===r;if((d||!this.pauseablePlacement||this.pauseablePlacement.isDone()&&!this.placement.stillRecent(t.browser.now()))&&(this.pauseablePlacement=new we(e,this._order,d,n,r,i,this.placement),this._layerOrderChanged=!1),this.pauseablePlacement.isDone()?this.placement.setStale():(this.pauseablePlacement.continuePlacement(this._order,this._layers,s),this.pauseablePlacement.isDone()&&(this.placement=this.pauseablePlacement.commit(t.browser.now()),a=!0),o&&this.pauseablePlacement.placement.setStale()),a||o)for(var f=0,p=this._order;f<p.length;f+=1){var I=p[f],C=this._layers[I];"symbol"===C.type&&this.placement.updateLayerOpacities(C,s[C.source])}return!this.pauseablePlacement.isDone()||this.placement.hasTransitions(t.browser.now())},n.prototype._releaseSymbolFadeTiles=function(){for(var t in this.sourceCaches)this.sourceCaches[t].releaseSymbolFadeTiles()},n.prototype.getImages=function(t,e,n){this.imageManager.getImages(e.icons,n)},n.prototype.getGlyphs=function(t,e,n){this.glyphManager.getGlyphs(e.stacks,n)},n.prototype.getResource=function(e,n,r){return t.makeRequest(n,r)},n}(t.Evented);Ne.getSourceType=function(t){return L[t]},Ne.setSourceType=function(t,e){L[t]=e},Ne.registerForPluginAvailability=t.registerForPluginAvailability;var Fe=t.createLayout([{name:"a_pos",type:"Int16",components:2}]),Te=ln("#ifdef GL_ES\nprecision mediump float;\n#else\n#if !defined(lowp)\n#define lowp\n#endif\n#if !defined(mediump)\n#define mediump\n#endif\n#if !defined(highp)\n#define highp\n#endif\n#endif","#ifdef GL_ES\nprecision highp float;\n#else\n#if !defined(lowp)\n#define lowp\n#endif\n#if !defined(mediump)\n#define mediump\n#endif\n#if !defined(highp)\n#define highp\n#endif\n#endif\nvec2 unpack_float(const float packedValue) {int packedIntValue=int(packedValue);int v0=packedIntValue/256;return vec2(v0,packedIntValue-v0*256);}vec2 unpack_opacity(const float packedOpacity) {int intOpacity=int(packedOpacity)/2;return vec2(float(intOpacity)/127.0,mod(packedOpacity,2.0));}vec4 decode_color(const vec2 encodedColor) {return vec4(unpack_float(encodedColor[0])/255.0,unpack_float(encodedColor[1])/255.0\n);}float unpack_mix_vec2(const vec2 packedValue,const float t) {return mix(packedValue[0],packedValue[1],t);}vec4 unpack_mix_color(const vec4 packedColors,const float t) {vec4 minColor=decode_color(vec2(packedColors[0],packedColors[1]));vec4 maxColor=decode_color(vec2(packedColors[2],packedColors[3]));return mix(minColor,maxColor,t);}vec2 get_pattern_pos(const vec2 pixel_coord_upper,const vec2 pixel_coord_lower,const vec2 pattern_size,const float tile_units_to_pixels,const vec2 pos) {vec2 offset=mod(mod(mod(pixel_coord_upper,pattern_size)*256.0,pattern_size)*256.0+pixel_coord_lower,pattern_size);return (tile_units_to_pixels*pos+offset)/pattern_size;}"),Ue=ln("uniform vec4 u_color;uniform float u_opacity;void main() {gl_FragColor=u_color*u_opacity;\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","attribute vec2 a_pos;uniform mat4 u_matrix;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);}"),Oe=ln("uniform vec2 u_pattern_tl_a;uniform vec2 u_pattern_br_a;uniform vec2 u_pattern_tl_b;uniform vec2 u_pattern_br_b;uniform vec2 u_texsize;uniform float u_mix;uniform float u_opacity;uniform sampler2D u_image;varying vec2 v_pos_a;varying vec2 v_pos_b;void main() {vec2 imagecoord=mod(v_pos_a,1.0);vec2 pos=mix(u_pattern_tl_a/u_texsize,u_pattern_br_a/u_texsize,imagecoord);vec4 color1=texture2D(u_image,pos);vec2 imagecoord_b=mod(v_pos_b,1.0);vec2 pos2=mix(u_pattern_tl_b/u_texsize,u_pattern_br_b/u_texsize,imagecoord_b);vec4 color2=texture2D(u_image,pos2);gl_FragColor=mix(color1,color2,u_mix)*u_opacity;\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","uniform mat4 u_matrix;uniform vec2 u_pattern_size_a;uniform vec2 u_pattern_size_b;uniform vec2 u_pixel_coord_upper;uniform vec2 u_pixel_coord_lower;uniform float u_scale_a;uniform float u_scale_b;uniform float u_tile_units_to_pixels;attribute vec2 a_pos;varying vec2 v_pos_a;varying vec2 v_pos_b;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);v_pos_a=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,u_scale_a*u_pattern_size_a,u_tile_units_to_pixels,a_pos);v_pos_b=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,u_scale_b*u_pattern_size_b,u_tile_units_to_pixels,a_pos);}"),Ge=ln("varying vec3 v_data;\n#pragma mapbox: define highp vec4 color\n#pragma mapbox: define mediump float radius\n#pragma mapbox: define lowp float blur\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define highp vec4 stroke_color\n#pragma mapbox: define mediump float stroke_width\n#pragma mapbox: define lowp float stroke_opacity\nvoid main() {\n#pragma mapbox: initialize highp vec4 color\n#pragma mapbox: initialize mediump float radius\n#pragma mapbox: initialize lowp float blur\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize highp vec4 stroke_color\n#pragma mapbox: initialize mediump float stroke_width\n#pragma mapbox: initialize lowp float stroke_opacity\nvec2 extrude=v_data.xy;float extrude_length=length(extrude);lowp float antialiasblur=v_data.z;float antialiased_blur=-max(blur,antialiasblur);float opacity_t=smoothstep(0.0,antialiased_blur,extrude_length-1.0);float color_t=stroke_width < 0.01 ? 0.0 : smoothstep(antialiased_blur,0.0,extrude_length-radius/(radius+stroke_width));gl_FragColor=opacity_t*mix(color*opacity,stroke_color*stroke_opacity,color_t);\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","uniform mat4 u_matrix;uniform bool u_scale_with_map;uniform bool u_pitch_with_map;uniform vec2 u_extrude_scale;uniform lowp float u_device_pixel_ratio;uniform highp float u_camera_to_center_distance;attribute vec2 a_pos;varying vec3 v_data;\n#pragma mapbox: define highp vec4 color\n#pragma mapbox: define mediump float radius\n#pragma mapbox: define lowp float blur\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define highp vec4 stroke_color\n#pragma mapbox: define mediump float stroke_width\n#pragma mapbox: define lowp float stroke_opacity\nvoid main(void) {\n#pragma mapbox: initialize highp vec4 color\n#pragma mapbox: initialize mediump float radius\n#pragma mapbox: initialize lowp float blur\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize highp vec4 stroke_color\n#pragma mapbox: initialize mediump float stroke_width\n#pragma mapbox: initialize lowp float stroke_opacity\nvec2 extrude=vec2(mod(a_pos,2.0)*2.0-1.0);vec2 circle_center=floor(a_pos*0.5);if (u_pitch_with_map) {vec2 corner_position=circle_center;if (u_scale_with_map) {corner_position+=extrude*(radius+stroke_width)*u_extrude_scale;} else {vec4 projected_center=u_matrix*vec4(circle_center,0,1);corner_position+=extrude*(radius+stroke_width)*u_extrude_scale*(projected_center.w/u_camera_to_center_distance);}gl_Position=u_matrix*vec4(corner_position,0,1);} else {gl_Position=u_matrix*vec4(circle_center,0,1);if (u_scale_with_map) {gl_Position.xy+=extrude*(radius+stroke_width)*u_extrude_scale*u_camera_to_center_distance;} else {gl_Position.xy+=extrude*(radius+stroke_width)*u_extrude_scale*gl_Position.w;}}lowp float antialiasblur=1.0/u_device_pixel_ratio/(radius+stroke_width);v_data=vec3(extrude.x,extrude.y,antialiasblur);}"),He=ln("void main() {gl_FragColor=vec4(1.0);}","attribute vec2 a_pos;uniform mat4 u_matrix;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);}"),je=ln("uniform highp float u_intensity;varying vec2 v_extrude;\n#pragma mapbox: define highp float weight\n#define GAUSS_COEF 0.3989422804014327\nvoid main() {\n#pragma mapbox: initialize highp float weight\nfloat d=-0.5*3.0*3.0*dot(v_extrude,v_extrude);float val=weight*u_intensity*GAUSS_COEF*exp(d);gl_FragColor=vec4(val,1.0,1.0,1.0);\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","uniform mat4 u_matrix;uniform float u_extrude_scale;uniform float u_opacity;uniform float u_intensity;attribute vec2 a_pos;varying vec2 v_extrude;\n#pragma mapbox: define highp float weight\n#pragma mapbox: define mediump float radius\nconst highp float ZERO=1.0/255.0/16.0;\n#define GAUSS_COEF 0.3989422804014327\nvoid main(void) {\n#pragma mapbox: initialize highp float weight\n#pragma mapbox: initialize mediump float radius\nvec2 unscaled_extrude=vec2(mod(a_pos,2.0)*2.0-1.0);float S=sqrt(-2.0*log(ZERO/weight/u_intensity/GAUSS_COEF))/3.0;v_extrude=S*unscaled_extrude;vec2 extrude=v_extrude*radius*u_extrude_scale;vec4 pos=vec4(floor(a_pos*0.5)+extrude,0,1);gl_Position=u_matrix*pos;}"),Ye=ln("uniform sampler2D u_image;uniform sampler2D u_color_ramp;uniform float u_opacity;varying vec2 v_pos;void main() {float t=texture2D(u_image,v_pos).r;vec4 color=texture2D(u_color_ramp,vec2(t,0.5));gl_FragColor=color*u_opacity;\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(0.0);\n#endif\n}","uniform mat4 u_matrix;uniform vec2 u_world;attribute vec2 a_pos;varying vec2 v_pos;void main() {gl_Position=u_matrix*vec4(a_pos*u_world,0,1);v_pos.x=a_pos.x;v_pos.y=1.0-a_pos.y;}"),Pe=ln("varying float v_placed;varying float v_notUsed;void main() {float alpha=0.5;gl_FragColor=vec4(1.0,0.0,0.0,1.0)*alpha;if (v_placed > 0.5) {gl_FragColor=vec4(0.0,0.0,1.0,0.5)*alpha;}if (v_notUsed > 0.5) {gl_FragColor*=.1;}}","attribute vec2 a_pos;attribute vec2 a_anchor_pos;attribute vec2 a_extrude;attribute vec2 a_placed;attribute vec2 a_shift;uniform mat4 u_matrix;uniform vec2 u_extrude_scale;uniform float u_camera_to_center_distance;varying float v_placed;varying float v_notUsed;void main() {vec4 projectedPoint=u_matrix*vec4(a_anchor_pos,0,1);highp float camera_to_anchor_distance=projectedPoint.w;highp float collision_perspective_ratio=clamp(0.5+0.5*(u_camera_to_center_distance/camera_to_anchor_distance),0.0,4.0);gl_Position=u_matrix*vec4(a_pos,0.0,1.0);gl_Position.xy+=(a_extrude+a_shift)*u_extrude_scale*gl_Position.w*collision_perspective_ratio;v_placed=a_placed.x;v_notUsed=a_placed.y;}"),Ke=ln("uniform float u_overscale_factor;varying float v_placed;varying float v_notUsed;varying float v_radius;varying vec2 v_extrude;varying vec2 v_extrude_scale;void main() {float alpha=0.5;vec4 color=vec4(1.0,0.0,0.0,1.0)*alpha;if (v_placed > 0.5) {color=vec4(0.0,0.0,1.0,0.5)*alpha;}if (v_notUsed > 0.5) {color*=.2;}float extrude_scale_length=length(v_extrude_scale);float extrude_length=length(v_extrude)*extrude_scale_length;float stroke_width=15.0*extrude_scale_length/u_overscale_factor;float radius=v_radius*extrude_scale_length;float distance_to_edge=abs(extrude_length-radius);float opacity_t=smoothstep(-stroke_width,0.0,-distance_to_edge);gl_FragColor=opacity_t*color;}","attribute vec2 a_pos;attribute vec2 a_anchor_pos;attribute vec2 a_extrude;attribute vec2 a_placed;uniform mat4 u_matrix;uniform vec2 u_extrude_scale;uniform float u_camera_to_center_distance;varying float v_placed;varying float v_notUsed;varying float v_radius;varying vec2 v_extrude;varying vec2 v_extrude_scale;void main() {vec4 projectedPoint=u_matrix*vec4(a_anchor_pos,0,1);highp float camera_to_anchor_distance=projectedPoint.w;highp float collision_perspective_ratio=clamp(0.5+0.5*(u_camera_to_center_distance/camera_to_anchor_distance),0.0,4.0);gl_Position=u_matrix*vec4(a_pos,0.0,1.0);highp float padding_factor=1.2;gl_Position.xy+=a_extrude*u_extrude_scale*padding_factor*gl_Position.w*collision_perspective_ratio;v_placed=a_placed.x;v_notUsed=a_placed.y;v_radius=abs(a_extrude.y);v_extrude=a_extrude*padding_factor;v_extrude_scale=u_extrude_scale*u_camera_to_center_distance*collision_perspective_ratio;}"),ze=ln("uniform highp vec4 u_color;void main() {gl_FragColor=u_color;}","attribute vec2 a_pos;uniform mat4 u_matrix;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);}"),Je=ln("#pragma mapbox: define highp vec4 color\n#pragma mapbox: define lowp float opacity\nvoid main() {\n#pragma mapbox: initialize highp vec4 color\n#pragma mapbox: initialize lowp float opacity\ngl_FragColor=color*opacity;\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","attribute vec2 a_pos;uniform mat4 u_matrix;\n#pragma mapbox: define highp vec4 color\n#pragma mapbox: define lowp float opacity\nvoid main() {\n#pragma mapbox: initialize highp vec4 color\n#pragma mapbox: initialize lowp float opacity\ngl_Position=u_matrix*vec4(a_pos,0,1);}"),qe=ln("varying vec2 v_pos;\n#pragma mapbox: define highp vec4 outline_color\n#pragma mapbox: define lowp float opacity\nvoid main() {\n#pragma mapbox: initialize highp vec4 outline_color\n#pragma mapbox: initialize lowp float opacity\nfloat dist=length(v_pos-gl_FragCoord.xy);float alpha=1.0-smoothstep(0.0,1.0,dist);gl_FragColor=outline_color*(alpha*opacity);\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","attribute vec2 a_pos;uniform mat4 u_matrix;uniform vec2 u_world;varying vec2 v_pos;\n#pragma mapbox: define highp vec4 outline_color\n#pragma mapbox: define lowp float opacity\nvoid main() {\n#pragma mapbox: initialize highp vec4 outline_color\n#pragma mapbox: initialize lowp float opacity\ngl_Position=u_matrix*vec4(a_pos,0,1);v_pos=(gl_Position.xy/gl_Position.w+1.0)/2.0*u_world;}"),Ve=ln("uniform vec2 u_texsize;uniform sampler2D u_image;uniform float u_fade;varying vec2 v_pos_a;varying vec2 v_pos_b;varying vec2 v_pos;\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define lowp vec4 pattern_from\n#pragma mapbox: define lowp vec4 pattern_to\nvoid main() {\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize mediump vec4 pattern_from\n#pragma mapbox: initialize mediump vec4 pattern_to\nvec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;vec2 imagecoord=mod(v_pos_a,1.0);vec2 pos=mix(pattern_tl_a/u_texsize,pattern_br_a/u_texsize,imagecoord);vec4 color1=texture2D(u_image,pos);vec2 imagecoord_b=mod(v_pos_b,1.0);vec2 pos2=mix(pattern_tl_b/u_texsize,pattern_br_b/u_texsize,imagecoord_b);vec4 color2=texture2D(u_image,pos2);float dist=length(v_pos-gl_FragCoord.xy);float alpha=1.0-smoothstep(0.0,1.0,dist);gl_FragColor=mix(color1,color2,u_fade)*alpha*opacity;\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","uniform mat4 u_matrix;uniform vec2 u_world;uniform vec2 u_pixel_coord_upper;uniform vec2 u_pixel_coord_lower;uniform vec4 u_scale;attribute vec2 a_pos;varying vec2 v_pos_a;varying vec2 v_pos_b;varying vec2 v_pos;\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define lowp vec4 pattern_from\n#pragma mapbox: define lowp vec4 pattern_to\nvoid main() {\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize mediump vec4 pattern_from\n#pragma mapbox: initialize mediump vec4 pattern_to\nvec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;float pixelRatio=u_scale.x;float tileRatio=u_scale.y;float fromScale=u_scale.z;float toScale=u_scale.w;gl_Position=u_matrix*vec4(a_pos,0,1);vec2 display_size_a=vec2((pattern_br_a.x-pattern_tl_a.x)/pixelRatio,(pattern_br_a.y-pattern_tl_a.y)/pixelRatio);vec2 display_size_b=vec2((pattern_br_b.x-pattern_tl_b.x)/pixelRatio,(pattern_br_b.y-pattern_tl_b.y)/pixelRatio);v_pos_a=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,fromScale*display_size_a,tileRatio,a_pos);v_pos_b=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,toScale*display_size_b,tileRatio,a_pos);v_pos=(gl_Position.xy/gl_Position.w+1.0)/2.0*u_world;}"),We=ln("uniform vec2 u_texsize;uniform float u_fade;uniform sampler2D u_image;varying vec2 v_pos_a;varying vec2 v_pos_b;\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define lowp vec4 pattern_from\n#pragma mapbox: define lowp vec4 pattern_to\nvoid main() {\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize mediump vec4 pattern_from\n#pragma mapbox: initialize mediump vec4 pattern_to\nvec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;vec2 imagecoord=mod(v_pos_a,1.0);vec2 pos=mix(pattern_tl_a/u_texsize,pattern_br_a/u_texsize,imagecoord);vec4 color1=texture2D(u_image,pos);vec2 imagecoord_b=mod(v_pos_b,1.0);vec2 pos2=mix(pattern_tl_b/u_texsize,pattern_br_b/u_texsize,imagecoord_b);vec4 color2=texture2D(u_image,pos2);gl_FragColor=mix(color1,color2,u_fade)*opacity;\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","uniform mat4 u_matrix;uniform vec2 u_pixel_coord_upper;uniform vec2 u_pixel_coord_lower;uniform vec4 u_scale;attribute vec2 a_pos;varying vec2 v_pos_a;varying vec2 v_pos_b;\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define lowp vec4 pattern_from\n#pragma mapbox: define lowp vec4 pattern_to\nvoid main() {\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize mediump vec4 pattern_from\n#pragma mapbox: initialize mediump vec4 pattern_to\nvec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;float pixelRatio=u_scale.x;float tileZoomRatio=u_scale.y;float fromScale=u_scale.z;float toScale=u_scale.w;vec2 display_size_a=vec2((pattern_br_a.x-pattern_tl_a.x)/pixelRatio,(pattern_br_a.y-pattern_tl_a.y)/pixelRatio);vec2 display_size_b=vec2((pattern_br_b.x-pattern_tl_b.x)/pixelRatio,(pattern_br_b.y-pattern_tl_b.y)/pixelRatio);gl_Position=u_matrix*vec4(a_pos,0,1);v_pos_a=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,fromScale*display_size_a,tileZoomRatio,a_pos);v_pos_b=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,toScale*display_size_b,tileZoomRatio,a_pos);}"),Xe=ln("varying vec4 v_color;void main() {gl_FragColor=v_color;\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","uniform mat4 u_matrix;uniform vec3 u_lightcolor;uniform lowp vec3 u_lightpos;uniform lowp float u_lightintensity;uniform float u_vertical_gradient;uniform lowp float u_opacity;attribute vec2 a_pos;attribute vec4 a_normal_ed;varying vec4 v_color;\n#pragma mapbox: define highp float base\n#pragma mapbox: define highp float height\n#pragma mapbox: define highp vec4 color\nvoid main() {\n#pragma mapbox: initialize highp float base\n#pragma mapbox: initialize highp float height\n#pragma mapbox: initialize highp vec4 color\nvec3 normal=a_normal_ed.xyz;base=max(0.0,base);height=max(0.0,height);float t=mod(normal.x,2.0);gl_Position=u_matrix*vec4(a_pos,t > 0.0 ? height : base,1);float colorvalue=color.r*0.2126+color.g*0.7152+color.b*0.0722;v_color=vec4(0.0,0.0,0.0,1.0);vec4 ambientlight=vec4(0.03,0.03,0.03,1.0);color+=ambientlight;float directional=clamp(dot(normal/16384.0,u_lightpos),0.0,1.0);directional=mix((1.0-u_lightintensity),max((1.0-colorvalue+u_lightintensity),1.0),directional);if (normal.y !=0.0) {directional*=((1.0-u_vertical_gradient)+(u_vertical_gradient*clamp((t+base)*pow(height/150.0,0.5),mix(0.7,0.98,1.0-u_lightintensity),1.0)));}v_color.r+=clamp(color.r*directional*u_lightcolor.r,mix(0.0,0.3,1.0-u_lightcolor.r),1.0);v_color.g+=clamp(color.g*directional*u_lightcolor.g,mix(0.0,0.3,1.0-u_lightcolor.g),1.0);v_color.b+=clamp(color.b*directional*u_lightcolor.b,mix(0.0,0.3,1.0-u_lightcolor.b),1.0);v_color*=u_opacity;}"),Ze=ln("uniform vec2 u_texsize;uniform float u_fade;uniform sampler2D u_image;varying vec2 v_pos_a;varying vec2 v_pos_b;varying vec4 v_lighting;\n#pragma mapbox: define lowp float base\n#pragma mapbox: define lowp float height\n#pragma mapbox: define lowp vec4 pattern_from\n#pragma mapbox: define lowp vec4 pattern_to\nvoid main() {\n#pragma mapbox: initialize lowp float base\n#pragma mapbox: initialize lowp float height\n#pragma mapbox: initialize mediump vec4 pattern_from\n#pragma mapbox: initialize mediump vec4 pattern_to\nvec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;vec2 imagecoord=mod(v_pos_a,1.0);vec2 pos=mix(pattern_tl_a/u_texsize,pattern_br_a/u_texsize,imagecoord);vec4 color1=texture2D(u_image,pos);vec2 imagecoord_b=mod(v_pos_b,1.0);vec2 pos2=mix(pattern_tl_b/u_texsize,pattern_br_b/u_texsize,imagecoord_b);vec4 color2=texture2D(u_image,pos2);vec4 mixedColor=mix(color1,color2,u_fade);gl_FragColor=mixedColor*v_lighting;\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","uniform mat4 u_matrix;uniform vec2 u_pixel_coord_upper;uniform vec2 u_pixel_coord_lower;uniform float u_height_factor;uniform vec4 u_scale;uniform float u_vertical_gradient;uniform lowp float u_opacity;uniform vec3 u_lightcolor;uniform lowp vec3 u_lightpos;uniform lowp float u_lightintensity;attribute vec2 a_pos;attribute vec4 a_normal_ed;varying vec2 v_pos_a;varying vec2 v_pos_b;varying vec4 v_lighting;\n#pragma mapbox: define lowp float base\n#pragma mapbox: define lowp float height\n#pragma mapbox: define lowp vec4 pattern_from\n#pragma mapbox: define lowp vec4 pattern_to\nvoid main() {\n#pragma mapbox: initialize lowp float base\n#pragma mapbox: initialize lowp float height\n#pragma mapbox: initialize mediump vec4 pattern_from\n#pragma mapbox: initialize mediump vec4 pattern_to\nvec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;float pixelRatio=u_scale.x;float tileRatio=u_scale.y;float fromScale=u_scale.z;float toScale=u_scale.w;vec3 normal=a_normal_ed.xyz;float edgedistance=a_normal_ed.w;vec2 display_size_a=vec2((pattern_br_a.x-pattern_tl_a.x)/pixelRatio,(pattern_br_a.y-pattern_tl_a.y)/pixelRatio);vec2 display_size_b=vec2((pattern_br_b.x-pattern_tl_b.x)/pixelRatio,(pattern_br_b.y-pattern_tl_b.y)/pixelRatio);base=max(0.0,base);height=max(0.0,height);float t=mod(normal.x,2.0);float z=t > 0.0 ? height : base;gl_Position=u_matrix*vec4(a_pos,z,1);vec2 pos=normal.x==1.0 && normal.y==0.0 && normal.z==16384.0\n? a_pos\n: vec2(edgedistance,z*u_height_factor);v_pos_a=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,fromScale*display_size_a,tileRatio,pos);v_pos_b=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,toScale*display_size_b,tileRatio,pos);v_lighting=vec4(0.0,0.0,0.0,1.0);float directional=clamp(dot(normal/16383.0,u_lightpos),0.0,1.0);directional=mix((1.0-u_lightintensity),max((0.5+u_lightintensity),1.0),directional);if (normal.y !=0.0) {directional*=((1.0-u_vertical_gradient)+(u_vertical_gradient*clamp((t+base)*pow(height/150.0,0.5),mix(0.7,0.98,1.0-u_lightintensity),1.0)));}v_lighting.rgb+=clamp(directional*u_lightcolor,mix(vec3(0.0),vec3(0.3),1.0-u_lightcolor),vec3(1.0));v_lighting*=u_opacity;}"),$e=ln("#ifdef GL_ES\nprecision highp float;\n#endif\nuniform sampler2D u_image;varying vec2 v_pos;uniform vec2 u_dimension;uniform float u_zoom;uniform float u_maxzoom;float getElevation(vec2 coord,float bias) {vec4 data=texture2D(u_image,coord)*255.0;return (data.r+data.g*256.0+data.b*256.0*256.0)/4.0;}void main() {vec2 epsilon=1.0/u_dimension;float a=getElevation(v_pos+vec2(-epsilon.x,-epsilon.y),0.0);float b=getElevation(v_pos+vec2(0,-epsilon.y),0.0);float c=getElevation(v_pos+vec2(epsilon.x,-epsilon.y),0.0);float d=getElevation(v_pos+vec2(-epsilon.x,0),0.0);float e=getElevation(v_pos,0.0);float f=getElevation(v_pos+vec2(epsilon.x,0),0.0);float g=getElevation(v_pos+vec2(-epsilon.x,epsilon.y),0.0);float h=getElevation(v_pos+vec2(0,epsilon.y),0.0);float i=getElevation(v_pos+vec2(epsilon.x,epsilon.y),0.0);float exaggeration=u_zoom < 2.0 ? 0.4 : u_zoom < 4.5 ? 0.35 : 0.3;vec2 deriv=vec2((c+f+f+i)-(a+d+d+g),(g+h+h+i)-(a+b+b+c))/ pow(2.0,(u_zoom-u_maxzoom)*exaggeration+19.2562-u_zoom);gl_FragColor=clamp(vec4(deriv.x/2.0+0.5,deriv.y/2.0+0.5,1.0,1.0),0.0,1.0);\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","uniform mat4 u_matrix;uniform vec2 u_dimension;attribute vec2 a_pos;attribute vec2 a_texture_pos;varying vec2 v_pos;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);highp vec2 epsilon=1.0/u_dimension;float scale=(u_dimension.x-2.0)/u_dimension.x;v_pos=(a_texture_pos/8192.0)*scale+epsilon;}"),tn=ln("uniform sampler2D u_image;varying vec2 v_pos;uniform vec2 u_latrange;uniform vec2 u_light;uniform vec4 u_shadow;uniform vec4 u_highlight;uniform vec4 u_accent;\n#define PI 3.141592653589793\nvoid main() {vec4 pixel=texture2D(u_image,v_pos);vec2 deriv=((pixel.rg*2.0)-1.0);float scaleFactor=cos(radians((u_latrange[0]-u_latrange[1])*(1.0-v_pos.y)+u_latrange[1]));float slope=atan(1.25*length(deriv)/scaleFactor);float aspect=deriv.x !=0.0 ? atan(deriv.y,-deriv.x) : PI/2.0*(deriv.y > 0.0 ? 1.0 :-1.0);float intensity=u_light.x;float azimuth=u_light.y+PI;float base=1.875-intensity*1.75;float maxValue=0.5*PI;float scaledSlope=intensity !=0.5 ? ((pow(base,slope)-1.0)/(pow(base,maxValue)-1.0))*maxValue : slope;float accent=cos(scaledSlope);vec4 accent_color=(1.0-accent)*u_accent*clamp(intensity*2.0,0.0,1.0);float shade=abs(mod((aspect+azimuth)/PI+0.5,2.0)-1.0);vec4 shade_color=mix(u_shadow,u_highlight,shade)*sin(scaledSlope)*clamp(intensity*2.0,0.0,1.0);gl_FragColor=accent_color*(1.0-shade_color.a)+shade_color;\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","uniform mat4 u_matrix;attribute vec2 a_pos;attribute vec2 a_texture_pos;varying vec2 v_pos;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);v_pos=a_texture_pos/8192.0;}"),en=ln("uniform lowp float u_device_pixel_ratio;varying vec2 v_width2;varying vec2 v_normal;varying float v_gamma_scale;\n#pragma mapbox: define highp vec4 color\n#pragma mapbox: define lowp float blur\n#pragma mapbox: define lowp float opacity\nvoid main() {\n#pragma mapbox: initialize highp vec4 color\n#pragma mapbox: initialize lowp float blur\n#pragma mapbox: initialize lowp float opacity\nfloat dist=length(v_normal)*v_width2.s;float blur2=(blur+1.0/u_device_pixel_ratio)*v_gamma_scale;float alpha=clamp(min(dist-(v_width2.t-blur2),v_width2.s-dist)/blur2,0.0,1.0);gl_FragColor=color*(alpha*opacity);\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","\n#define scale 0.015873016\nattribute vec2 a_pos_normal;attribute vec4 a_data;uniform mat4 u_matrix;uniform mediump float u_ratio;uniform vec2 u_units_to_pixels;uniform lowp float u_device_pixel_ratio;varying vec2 v_normal;varying vec2 v_width2;varying float v_gamma_scale;varying highp float v_linesofar;\n#pragma mapbox: define highp vec4 color\n#pragma mapbox: define lowp float blur\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define mediump float gapwidth\n#pragma mapbox: define lowp float offset\n#pragma mapbox: define mediump float width\nvoid main() {\n#pragma mapbox: initialize highp vec4 color\n#pragma mapbox: initialize lowp float blur\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize mediump float gapwidth\n#pragma mapbox: initialize lowp float offset\n#pragma mapbox: initialize mediump float width\nfloat ANTIALIASING=1.0/u_device_pixel_ratio/2.0;vec2 a_extrude=a_data.xy-128.0;float a_direction=mod(a_data.z,4.0)-1.0;v_linesofar=(floor(a_data.z/4.0)+a_data.w*64.0)*2.0;vec2 pos=floor(a_pos_normal*0.5);mediump vec2 normal=a_pos_normal-2.0*pos;normal.y=normal.y*2.0-1.0;v_normal=normal;gapwidth=gapwidth/2.0;float halfwidth=width/2.0;offset=-1.0*offset;float inset=gapwidth+(gapwidth > 0.0 ? ANTIALIASING : 0.0);float outset=gapwidth+halfwidth*(gapwidth > 0.0 ? 2.0 : 1.0)+(halfwidth==0.0 ? 0.0 : ANTIALIASING);mediump vec2 dist=outset*a_extrude*scale;mediump float u=0.5*a_direction;mediump float t=1.0-abs(u);mediump vec2 offset2=offset*a_extrude*scale*normal.y*mat2(t,-u,u,t);vec4 projected_extrude=u_matrix*vec4(dist/u_ratio,0.0,0.0);gl_Position=u_matrix*vec4(pos+offset2/u_ratio,0.0,1.0)+projected_extrude;float extrude_length_without_perspective=length(dist);float extrude_length_with_perspective=length(projected_extrude.xy/gl_Position.w*u_units_to_pixels);v_gamma_scale=extrude_length_without_perspective/extrude_length_with_perspective;v_width2=vec2(outset,inset);}"),nn=ln("uniform lowp float u_device_pixel_ratio;uniform sampler2D u_image;varying vec2 v_width2;varying vec2 v_normal;varying float v_gamma_scale;varying highp float v_lineprogress;\n#pragma mapbox: define lowp float blur\n#pragma mapbox: define lowp float opacity\nvoid main() {\n#pragma mapbox: initialize lowp float blur\n#pragma mapbox: initialize lowp float opacity\nfloat dist=length(v_normal)*v_width2.s;float blur2=(blur+1.0/u_device_pixel_ratio)*v_gamma_scale;float alpha=clamp(min(dist-(v_width2.t-blur2),v_width2.s-dist)/blur2,0.0,1.0);vec4 color=texture2D(u_image,vec2(v_lineprogress,0.5));gl_FragColor=color*(alpha*opacity);\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","\n#define MAX_LINE_DISTANCE 32767.0\n#define scale 0.015873016\nattribute vec2 a_pos_normal;attribute vec4 a_data;uniform mat4 u_matrix;uniform mediump float u_ratio;uniform lowp float u_device_pixel_ratio;uniform vec2 u_units_to_pixels;varying vec2 v_normal;varying vec2 v_width2;varying float v_gamma_scale;varying highp float v_lineprogress;\n#pragma mapbox: define lowp float blur\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define mediump float gapwidth\n#pragma mapbox: define lowp float offset\n#pragma mapbox: define mediump float width\nvoid main() {\n#pragma mapbox: initialize lowp float blur\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize mediump float gapwidth\n#pragma mapbox: initialize lowp float offset\n#pragma mapbox: initialize mediump float width\nfloat ANTIALIASING=1.0/u_device_pixel_ratio/2.0;vec2 a_extrude=a_data.xy-128.0;float a_direction=mod(a_data.z,4.0)-1.0;v_lineprogress=(floor(a_data.z/4.0)+a_data.w*64.0)*2.0/MAX_LINE_DISTANCE;vec2 pos=floor(a_pos_normal*0.5);mediump vec2 normal=a_pos_normal-2.0*pos;normal.y=normal.y*2.0-1.0;v_normal=normal;gapwidth=gapwidth/2.0;float halfwidth=width/2.0;offset=-1.0*offset;float inset=gapwidth+(gapwidth > 0.0 ? ANTIALIASING : 0.0);float outset=gapwidth+halfwidth*(gapwidth > 0.0 ? 2.0 : 1.0)+(halfwidth==0.0 ? 0.0 : ANTIALIASING);mediump vec2 dist=outset*a_extrude*scale;mediump float u=0.5*a_direction;mediump float t=1.0-abs(u);mediump vec2 offset2=offset*a_extrude*scale*normal.y*mat2(t,-u,u,t);vec4 projected_extrude=u_matrix*vec4(dist/u_ratio,0.0,0.0);gl_Position=u_matrix*vec4(pos+offset2/u_ratio,0.0,1.0)+projected_extrude;float extrude_length_without_perspective=length(dist);float extrude_length_with_perspective=length(projected_extrude.xy/gl_Position.w*u_units_to_pixels);v_gamma_scale=extrude_length_without_perspective/extrude_length_with_perspective;v_width2=vec2(outset,inset);}"),rn=ln("uniform lowp float u_device_pixel_ratio;uniform vec2 u_texsize;uniform float u_fade;uniform mediump vec4 u_scale;uniform sampler2D u_image;varying vec2 v_normal;varying vec2 v_width2;varying float v_linesofar;varying float v_gamma_scale;\n#pragma mapbox: define lowp vec4 pattern_from\n#pragma mapbox: define lowp vec4 pattern_to\n#pragma mapbox: define lowp float blur\n#pragma mapbox: define lowp float opacity\nvoid main() {\n#pragma mapbox: initialize mediump vec4 pattern_from\n#pragma mapbox: initialize mediump vec4 pattern_to\n#pragma mapbox: initialize lowp float blur\n#pragma mapbox: initialize lowp float opacity\nvec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;float pixelRatio=u_scale.x;float tileZoomRatio=u_scale.y;float fromScale=u_scale.z;float toScale=u_scale.w;vec2 display_size_a=vec2((pattern_br_a.x-pattern_tl_a.x)/pixelRatio,(pattern_br_a.y-pattern_tl_a.y)/pixelRatio);vec2 display_size_b=vec2((pattern_br_b.x-pattern_tl_b.x)/pixelRatio,(pattern_br_b.y-pattern_tl_b.y)/pixelRatio);vec2 pattern_size_a=vec2(display_size_a.x*fromScale/tileZoomRatio,display_size_a.y);vec2 pattern_size_b=vec2(display_size_b.x*toScale/tileZoomRatio,display_size_b.y);float dist=length(v_normal)*v_width2.s;float blur2=(blur+1.0/u_device_pixel_ratio)*v_gamma_scale;float alpha=clamp(min(dist-(v_width2.t-blur2),v_width2.s-dist)/blur2,0.0,1.0);float x_a=mod(v_linesofar/pattern_size_a.x,1.0);float x_b=mod(v_linesofar/pattern_size_b.x,1.0);float y_a=0.5+(v_normal.y*clamp(v_width2.s,0.0,(pattern_size_a.y+2.0)/2.0)/pattern_size_a.y);float y_b=0.5+(v_normal.y*clamp(v_width2.s,0.0,(pattern_size_b.y+2.0)/2.0)/pattern_size_b.y);vec2 pos_a=mix(pattern_tl_a/u_texsize,pattern_br_a/u_texsize,vec2(x_a,y_a));vec2 pos_b=mix(pattern_tl_b/u_texsize,pattern_br_b/u_texsize,vec2(x_b,y_b));vec4 color=mix(texture2D(u_image,pos_a),texture2D(u_image,pos_b),u_fade);gl_FragColor=color*alpha*opacity;\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","\n#define scale 0.015873016\n#define LINE_DISTANCE_SCALE 2.0\nattribute vec2 a_pos_normal;attribute vec4 a_data;uniform mat4 u_matrix;uniform vec2 u_units_to_pixels;uniform mediump float u_ratio;uniform lowp float u_device_pixel_ratio;varying vec2 v_normal;varying vec2 v_width2;varying float v_linesofar;varying float v_gamma_scale;\n#pragma mapbox: define lowp float blur\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define lowp float offset\n#pragma mapbox: define mediump float gapwidth\n#pragma mapbox: define mediump float width\n#pragma mapbox: define lowp vec4 pattern_from\n#pragma mapbox: define lowp vec4 pattern_to\nvoid main() {\n#pragma mapbox: initialize lowp float blur\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize lowp float offset\n#pragma mapbox: initialize mediump float gapwidth\n#pragma mapbox: initialize mediump float width\n#pragma mapbox: initialize mediump vec4 pattern_from\n#pragma mapbox: initialize mediump vec4 pattern_to\nfloat ANTIALIASING=1.0/u_device_pixel_ratio/2.0;vec2 a_extrude=a_data.xy-128.0;float a_direction=mod(a_data.z,4.0)-1.0;float a_linesofar=(floor(a_data.z/4.0)+a_data.w*64.0)*LINE_DISTANCE_SCALE;vec2 pos=floor(a_pos_normal*0.5);mediump vec2 normal=a_pos_normal-2.0*pos;normal.y=normal.y*2.0-1.0;v_normal=normal;gapwidth=gapwidth/2.0;float halfwidth=width/2.0;offset=-1.0*offset;float inset=gapwidth+(gapwidth > 0.0 ? ANTIALIASING : 0.0);float outset=gapwidth+halfwidth*(gapwidth > 0.0 ? 2.0 : 1.0)+(halfwidth==0.0 ? 0.0 : ANTIALIASING);mediump vec2 dist=outset*a_extrude*scale;mediump float u=0.5*a_direction;mediump float t=1.0-abs(u);mediump vec2 offset2=offset*a_extrude*scale*normal.y*mat2(t,-u,u,t);vec4 projected_extrude=u_matrix*vec4(dist/u_ratio,0.0,0.0);gl_Position=u_matrix*vec4(pos+offset2/u_ratio,0.0,1.0)+projected_extrude;float extrude_length_without_perspective=length(dist);float extrude_length_with_perspective=length(projected_extrude.xy/gl_Position.w*u_units_to_pixels);v_gamma_scale=extrude_length_without_perspective/extrude_length_with_perspective;v_linesofar=a_linesofar;v_width2=vec2(outset,inset);}"),on=ln("uniform lowp float u_device_pixel_ratio;uniform sampler2D u_image;uniform float u_sdfgamma;uniform float u_mix;varying vec2 v_normal;varying vec2 v_width2;varying vec2 v_tex_a;varying vec2 v_tex_b;varying float v_gamma_scale;\n#pragma mapbox: define highp vec4 color\n#pragma mapbox: define lowp float blur\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define mediump float width\n#pragma mapbox: define lowp float floorwidth\nvoid main() {\n#pragma mapbox: initialize highp vec4 color\n#pragma mapbox: initialize lowp float blur\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize mediump float width\n#pragma mapbox: initialize lowp float floorwidth\nfloat dist=length(v_normal)*v_width2.s;float blur2=(blur+1.0/u_device_pixel_ratio)*v_gamma_scale;float alpha=clamp(min(dist-(v_width2.t-blur2),v_width2.s-dist)/blur2,0.0,1.0);float sdfdist_a=texture2D(u_image,v_tex_a).a;float sdfdist_b=texture2D(u_image,v_tex_b).a;float sdfdist=mix(sdfdist_a,sdfdist_b,u_mix);alpha*=smoothstep(0.5-u_sdfgamma/floorwidth,0.5+u_sdfgamma/floorwidth,sdfdist);gl_FragColor=color*(alpha*opacity);\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","\n#define scale 0.015873016\n#define LINE_DISTANCE_SCALE 2.0\nattribute vec2 a_pos_normal;attribute vec4 a_data;uniform mat4 u_matrix;uniform mediump float u_ratio;uniform lowp float u_device_pixel_ratio;uniform vec2 u_patternscale_a;uniform float u_tex_y_a;uniform vec2 u_patternscale_b;uniform float u_tex_y_b;uniform vec2 u_units_to_pixels;varying vec2 v_normal;varying vec2 v_width2;varying vec2 v_tex_a;varying vec2 v_tex_b;varying float v_gamma_scale;\n#pragma mapbox: define highp vec4 color\n#pragma mapbox: define lowp float blur\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define mediump float gapwidth\n#pragma mapbox: define lowp float offset\n#pragma mapbox: define mediump float width\n#pragma mapbox: define lowp float floorwidth\nvoid main() {\n#pragma mapbox: initialize highp vec4 color\n#pragma mapbox: initialize lowp float blur\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize mediump float gapwidth\n#pragma mapbox: initialize lowp float offset\n#pragma mapbox: initialize mediump float width\n#pragma mapbox: initialize lowp float floorwidth\nfloat ANTIALIASING=1.0/u_device_pixel_ratio/2.0;vec2 a_extrude=a_data.xy-128.0;float a_direction=mod(a_data.z,4.0)-1.0;float a_linesofar=(floor(a_data.z/4.0)+a_data.w*64.0)*LINE_DISTANCE_SCALE;vec2 pos=floor(a_pos_normal*0.5);mediump vec2 normal=a_pos_normal-2.0*pos;normal.y=normal.y*2.0-1.0;v_normal=normal;gapwidth=gapwidth/2.0;float halfwidth=width/2.0;offset=-1.0*offset;float inset=gapwidth+(gapwidth > 0.0 ? ANTIALIASING : 0.0);float outset=gapwidth+halfwidth*(gapwidth > 0.0 ? 2.0 : 1.0)+(halfwidth==0.0 ? 0.0 : ANTIALIASING);mediump vec2 dist=outset*a_extrude*scale;mediump float u=0.5*a_direction;mediump float t=1.0-abs(u);mediump vec2 offset2=offset*a_extrude*scale*normal.y*mat2(t,-u,u,t);vec4 projected_extrude=u_matrix*vec4(dist/u_ratio,0.0,0.0);gl_Position=u_matrix*vec4(pos+offset2/u_ratio,0.0,1.0)+projected_extrude;float extrude_length_without_perspective=length(dist);float extrude_length_with_perspective=length(projected_extrude.xy/gl_Position.w*u_units_to_pixels);v_gamma_scale=extrude_length_without_perspective/extrude_length_with_perspective;v_tex_a=vec2(a_linesofar*u_patternscale_a.x/floorwidth,normal.y*u_patternscale_a.y+u_tex_y_a);v_tex_b=vec2(a_linesofar*u_patternscale_b.x/floorwidth,normal.y*u_patternscale_b.y+u_tex_y_b);v_width2=vec2(outset,inset);}"),an=ln("uniform float u_fade_t;uniform float u_opacity;uniform sampler2D u_image0;uniform sampler2D u_image1;varying vec2 v_pos0;varying vec2 v_pos1;uniform float u_brightness_low;uniform float u_brightness_high;uniform float u_saturation_factor;uniform float u_contrast_factor;uniform vec3 u_spin_weights;void main() {vec4 color0=texture2D(u_image0,v_pos0);vec4 color1=texture2D(u_image1,v_pos1);if (color0.a > 0.0) {color0.rgb=color0.rgb/color0.a;}if (color1.a > 0.0) {color1.rgb=color1.rgb/color1.a;}vec4 color=mix(color0,color1,u_fade_t);color.a*=u_opacity;vec3 rgb=color.rgb;rgb=vec3(dot(rgb,u_spin_weights.xyz),dot(rgb,u_spin_weights.zxy),dot(rgb,u_spin_weights.yzx));float average=(color.r+color.g+color.b)/3.0;rgb+=(average-rgb)*u_saturation_factor;rgb=(rgb-0.5)*u_contrast_factor+0.5;vec3 u_high_vec=vec3(u_brightness_low,u_brightness_low,u_brightness_low);vec3 u_low_vec=vec3(u_brightness_high,u_brightness_high,u_brightness_high);gl_FragColor=vec4(mix(u_high_vec,u_low_vec,rgb)*color.a,color.a);\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","uniform mat4 u_matrix;uniform vec2 u_tl_parent;uniform float u_scale_parent;uniform float u_buffer_scale;attribute vec2 a_pos;attribute vec2 a_texture_pos;varying vec2 v_pos0;varying vec2 v_pos1;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);v_pos0=(((a_texture_pos/8192.0)-0.5)/u_buffer_scale )+0.5;v_pos1=(v_pos0*u_scale_parent)+u_tl_parent;}"),sn=ln("uniform sampler2D u_texture;varying vec2 v_tex;varying float v_fade_opacity;\n#pragma mapbox: define lowp float opacity\nvoid main() {\n#pragma mapbox: initialize lowp float opacity\nlowp float alpha=opacity*v_fade_opacity;gl_FragColor=texture2D(u_texture,v_tex)*alpha;\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","const float PI=3.141592653589793;attribute vec4 a_pos_offset;attribute vec4 a_data;attribute vec3 a_projected_pos;attribute float a_fade_opacity;uniform bool u_is_size_zoom_constant;uniform bool u_is_size_feature_constant;uniform highp float u_size_t;uniform highp float u_size;uniform highp float u_camera_to_center_distance;uniform highp float u_pitch;uniform bool u_rotate_symbol;uniform highp float u_aspect_ratio;uniform float u_fade_change;uniform mat4 u_matrix;uniform mat4 u_label_plane_matrix;uniform mat4 u_coord_matrix;uniform bool u_is_text;uniform bool u_pitch_with_map;uniform vec2 u_texsize;varying vec2 v_tex;varying float v_fade_opacity;\n#pragma mapbox: define lowp float opacity\nvoid main() {\n#pragma mapbox: initialize lowp float opacity\nvec2 a_pos=a_pos_offset.xy;vec2 a_offset=a_pos_offset.zw;vec2 a_tex=a_data.xy;vec2 a_size=a_data.zw;highp float segment_angle=-a_projected_pos[2];float size;if (!u_is_size_zoom_constant && !u_is_size_feature_constant) {size=mix(a_size[0],a_size[1],u_size_t)/256.0;} else if (u_is_size_zoom_constant && !u_is_size_feature_constant) {size=a_size[0]/256.0;} else if (!u_is_size_zoom_constant && u_is_size_feature_constant) {size=u_size;} else {size=u_size;}vec4 projectedPoint=u_matrix*vec4(a_pos,0,1);highp float camera_to_anchor_distance=projectedPoint.w;highp float distance_ratio=u_pitch_with_map ?\ncamera_to_anchor_distance/u_camera_to_center_distance :\nu_camera_to_center_distance/camera_to_anchor_distance;highp float perspective_ratio=clamp(0.5+0.5*distance_ratio,0.0,4.0);size*=perspective_ratio;float fontScale=u_is_text ? size/24.0 : size;highp float symbol_rotation=0.0;if (u_rotate_symbol) {vec4 offsetProjectedPoint=u_matrix*vec4(a_pos+vec2(1,0),0,1);vec2 a=projectedPoint.xy/projectedPoint.w;vec2 b=offsetProjectedPoint.xy/offsetProjectedPoint.w;symbol_rotation=atan((b.y-a.y)/u_aspect_ratio,b.x-a.x);}highp float angle_sin=sin(segment_angle+symbol_rotation);highp float angle_cos=cos(segment_angle+symbol_rotation);mat2 rotation_matrix=mat2(angle_cos,-1.0*angle_sin,angle_sin,angle_cos);vec4 projected_pos=u_label_plane_matrix*vec4(a_projected_pos.xy,0.0,1.0);gl_Position=u_coord_matrix*vec4(projected_pos.xy/projected_pos.w+rotation_matrix*(a_offset/32.0*fontScale),0.0,1.0);v_tex=a_tex/u_texsize;vec2 fade_opacity=unpack_opacity(a_fade_opacity);float fade_change=fade_opacity[1] > 0.5 ? u_fade_change :-u_fade_change;v_fade_opacity=max(0.0,min(1.0,fade_opacity[0]+fade_change));}"),An=ln("#define SDF_PX 8.0\nuniform bool u_is_halo;uniform sampler2D u_texture;uniform highp float u_gamma_scale;uniform lowp float u_device_pixel_ratio;uniform bool u_is_text;varying vec2 v_data0;varying vec3 v_data1;\n#pragma mapbox: define highp vec4 fill_color\n#pragma mapbox: define highp vec4 halo_color\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define lowp float halo_width\n#pragma mapbox: define lowp float halo_blur\nvoid main() {\n#pragma mapbox: initialize highp vec4 fill_color\n#pragma mapbox: initialize highp vec4 halo_color\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize lowp float halo_width\n#pragma mapbox: initialize lowp float halo_blur\nfloat EDGE_GAMMA=0.105/u_device_pixel_ratio;vec2 tex=v_data0.xy;float gamma_scale=v_data1.x;float size=v_data1.y;float fade_opacity=v_data1[2];float fontScale=u_is_text ? size/24.0 : size;lowp vec4 color=fill_color;highp float gamma=EDGE_GAMMA/(fontScale*u_gamma_scale);lowp float buff=(256.0-64.0)/256.0;if (u_is_halo) {color=halo_color;gamma=(halo_blur*1.19/SDF_PX+EDGE_GAMMA)/(fontScale*u_gamma_scale);buff=(6.0-halo_width/fontScale)/SDF_PX;}lowp float dist=texture2D(u_texture,tex).a;highp float gamma_scaled=gamma*gamma_scale;highp float alpha=smoothstep(buff-gamma_scaled,buff+gamma_scaled,dist);gl_FragColor=color*(alpha*opacity*fade_opacity);\n#ifdef OVERDRAW_INSPECTOR\ngl_FragColor=vec4(1.0);\n#endif\n}","const float PI=3.141592653589793;attribute vec4 a_pos_offset;attribute vec4 a_data;attribute vec3 a_projected_pos;attribute float a_fade_opacity;uniform bool u_is_size_zoom_constant;uniform bool u_is_size_feature_constant;uniform highp float u_size_t;uniform highp float u_size;uniform mat4 u_matrix;uniform mat4 u_label_plane_matrix;uniform mat4 u_coord_matrix;uniform bool u_is_text;uniform bool u_pitch_with_map;uniform highp float u_pitch;uniform bool u_rotate_symbol;uniform highp float u_aspect_ratio;uniform highp float u_camera_to_center_distance;uniform float u_fade_change;uniform vec2 u_texsize;varying vec2 v_data0;varying vec3 v_data1;\n#pragma mapbox: define highp vec4 fill_color\n#pragma mapbox: define highp vec4 halo_color\n#pragma mapbox: define lowp float opacity\n#pragma mapbox: define lowp float halo_width\n#pragma mapbox: define lowp float halo_blur\nvoid main() {\n#pragma mapbox: initialize highp vec4 fill_color\n#pragma mapbox: initialize highp vec4 halo_color\n#pragma mapbox: initialize lowp float opacity\n#pragma mapbox: initialize lowp float halo_width\n#pragma mapbox: initialize lowp float halo_blur\nvec2 a_pos=a_pos_offset.xy;vec2 a_offset=a_pos_offset.zw;vec2 a_tex=a_data.xy;vec2 a_size=a_data.zw;highp float segment_angle=-a_projected_pos[2];float size;if (!u_is_size_zoom_constant && !u_is_size_feature_constant) {size=mix(a_size[0],a_size[1],u_size_t)/256.0;} else if (u_is_size_zoom_constant && !u_is_size_feature_constant) {size=a_size[0]/256.0;} else if (!u_is_size_zoom_constant && u_is_size_feature_constant) {size=u_size;} else {size=u_size;}vec4 projectedPoint=u_matrix*vec4(a_pos,0,1);highp float camera_to_anchor_distance=projectedPoint.w;highp float distance_ratio=u_pitch_with_map ?\ncamera_to_anchor_distance/u_camera_to_center_distance :\nu_camera_to_center_distance/camera_to_anchor_distance;highp float perspective_ratio=clamp(0.5+0.5*distance_ratio,0.0,4.0);size*=perspective_ratio;float fontScale=u_is_text ? size/24.0 : size;highp float symbol_rotation=0.0;if (u_rotate_symbol) {vec4 offsetProjectedPoint=u_matrix*vec4(a_pos+vec2(1,0),0,1);vec2 a=projectedPoint.xy/projectedPoint.w;vec2 b=offsetProjectedPoint.xy/offsetProjectedPoint.w;symbol_rotation=atan((b.y-a.y)/u_aspect_ratio,b.x-a.x);}highp float angle_sin=sin(segment_angle+symbol_rotation);highp float angle_cos=cos(segment_angle+symbol_rotation);mat2 rotation_matrix=mat2(angle_cos,-1.0*angle_sin,angle_sin,angle_cos);vec4 projected_pos=u_label_plane_matrix*vec4(a_projected_pos.xy,0.0,1.0);gl_Position=u_coord_matrix*vec4(projected_pos.xy/projected_pos.w+rotation_matrix*(a_offset/32.0*fontScale),0.0,1.0);float gamma_scale=gl_Position.w;vec2 tex=a_tex/u_texsize;vec2 fade_opacity=unpack_opacity(a_fade_opacity);float fade_change=fade_opacity[1] > 0.5 ? u_fade_change :-u_fade_change;float interpolated_fade_opacity=max(0.0,min(1.0,fade_opacity[0]+fade_change));v_data0=vec2(tex.x,tex.y);v_data1=vec3(gamma_scale,size,interpolated_fade_opacity);}");function ln(t,e){var n=/#pragma mapbox: ([\w]+) ([\w]+) ([\w]+) ([\w]+)/g,r={};return{fragmentSource:t=t.replace(n,function(t,e,n,i,o){return r[o]=!0,"define"===e?"\n#ifndef HAS_UNIFORM_u_"+o+"\nvarying "+n+" "+i+" "+o+";\n#else\nuniform "+n+" "+i+" u_"+o+";\n#endif\n":"\n#ifdef HAS_UNIFORM_u_"+o+"\n    "+n+" "+i+" "+o+" = u_"+o+";\n#endif\n"}),vertexSource:e=e.replace(n,function(t,e,n,i,o){var a="float"===i?"vec2":"vec4",s=o.match(/color/)?"color":a;return r[o]?"define"===e?"\n#ifndef HAS_UNIFORM_u_"+o+"\nuniform lowp float u_"+o+"_t;\nattribute "+n+" "+a+" a_"+o+";\nvarying "+n+" "+i+" "+o+";\n#else\nuniform "+n+" "+i+" u_"+o+";\n#endif\n":"vec4"===s?"\n#ifndef HAS_UNIFORM_u_"+o+"\n    "+o+" = a_"+o+";\n#else\n    "+n+" "+i+" "+o+" = u_"+o+";\n#endif\n":"\n#ifndef HAS_UNIFORM_u_"+o+"\n    "+o+" = unpack_mix_"+s+"(a_"+o+", u_"+o+"_t);\n#else\n    "+n+" "+i+" "+o+" = u_"+o+";\n#endif\n":"define"===e?"\n#ifndef HAS_UNIFORM_u_"+o+"\nuniform lowp float u_"+o+"_t;\nattribute "+n+" "+a+" a_"+o+";\n#else\nuniform "+n+" "+i+" u_"+o+";\n#endif\n":"vec4"===s?"\n#ifndef HAS_UNIFORM_u_"+o+"\n    "+n+" "+i+" "+o+" = a_"+o+";\n#else\n    "+n+" "+i+" "+o+" = u_"+o+";\n#endif\n":"\n#ifndef HAS_UNIFORM_u_"+o+"\n    "+n+" "+i+" "+o+" = unpack_mix_"+s+"(a_"+o+", u_"+o+"_t);\n#else\n    "+n+" "+i+" "+o+" = u_"+o+";\n#endif\n"})}}var cn=Object.freeze({prelude:Te,background:Ue,backgroundPattern:Oe,circle:Ge,clippingMask:He,heatmap:je,heatmapTexture:Ye,collisionBox:Pe,collisionCircle:Ke,debug:ze,fill:Je,fillOutline:qe,fillOutlinePattern:Ve,fillPattern:We,fillExtrusion:Xe,fillExtrusionPattern:Ze,hillshadePrepare:$e,hillshade:tn,line:en,lineGradient:nn,linePattern:rn,lineSDF:on,raster:an,symbolIcon:sn,symbolSDF:An}),un=function(){this.boundProgram=null,this.boundLayoutVertexBuffer=null,this.boundPaintVertexBuffers=[],this.boundIndexBuffer=null,this.boundVertexOffset=null,this.boundDynamicVertexBuffer=null,this.vao=null};un.prototype.bind=function(t,e,n,r,i,o,a,s){this.context=t;for(var A=this.boundPaintVertexBuffers.length!==r.length,l=0;!A&&l<r.length;l++)this.boundPaintVertexBuffers[l]!==r[l]&&(A=!0);var c=!this.vao||this.boundProgram!==e||this.boundLayoutVertexBuffer!==n||A||this.boundIndexBuffer!==i||this.boundVertexOffset!==o||this.boundDynamicVertexBuffer!==a||this.boundDynamicVertexBuffer2!==s;!t.extVertexArrayObject||c?this.freshBind(e,n,r,i,o,a,s):(t.bindVertexArrayOES.set(this.vao),a&&a.bind(),i&&i.dynamicDraw&&i.bind(),s&&s.bind())},un.prototype.freshBind=function(t,e,n,r,i,o,a){var s,A=t.numAttributes,l=this.context,c=l.gl;if(l.extVertexArrayObject)this.vao&&this.destroy(),this.vao=l.extVertexArrayObject.createVertexArrayOES(),l.bindVertexArrayOES.set(this.vao),s=0,this.boundProgram=t,this.boundLayoutVertexBuffer=e,this.boundPaintVertexBuffers=n,this.boundIndexBuffer=r,this.boundVertexOffset=i,this.boundDynamicVertexBuffer=o,this.boundDynamicVertexBuffer2=a;else{s=l.currentNumAttributes||0;for(var u=A;u<s;u++)c.disableVertexAttribArray(u)}e.enableAttributes(c,t);for(var g=0,h=n;g<h.length;g+=1)h[g].enableAttributes(c,t);o&&o.enableAttributes(c,t),a&&a.enableAttributes(c,t),e.bind(),e.setVertexAttribPointers(c,t,i);for(var d=0,f=n;d<f.length;d+=1){var p=f[d];p.bind(),p.setVertexAttribPointers(c,t,i)}o&&(o.bind(),o.setVertexAttribPointers(c,t,i)),r&&r.bind(),a&&(a.bind(),a.setVertexAttribPointers(c,t,i)),l.currentNumAttributes=A},un.prototype.destroy=function(){this.vao&&(this.context.extVertexArrayObject.deleteVertexArrayOES(this.vao),this.vao=null)};var gn=function(t,e,n,r,i){var o=t.gl;this.program=o.createProgram();var a=n.defines();i&&a.push("#define OVERDRAW_INSPECTOR;");var s=a.concat(Te.fragmentSource,e.fragmentSource).join("\n"),A=a.concat(Te.vertexSource,e.vertexSource).join("\n"),l=o.createShader(o.FRAGMENT_SHADER);o.shaderSource(l,s),o.compileShader(l),o.attachShader(this.program,l);var c=o.createShader(o.VERTEX_SHADER);o.shaderSource(c,A),o.compileShader(c),o.attachShader(this.program,c);for(var u=n.layoutAttributes||[],g=0;g<u.length;g++)o.bindAttribLocation(this.program,g,u[g].name);o.linkProgram(this.program),this.numAttributes=o.getProgramParameter(this.program,o.ACTIVE_ATTRIBUTES),this.attributes={};for(var h={},d=0;d<this.numAttributes;d++){var f=o.getActiveAttrib(this.program,d);f&&(this.attributes[f.name]=o.getAttribLocation(this.program,f.name))}for(var p=o.getProgramParameter(this.program,o.ACTIVE_UNIFORMS),I=0;I<p;I++){var C=o.getActiveUniform(this.program,I);C&&(h[C.name]=o.getUniformLocation(this.program,C.name))}this.fixedUniforms=r(t,h),this.binderUniforms=n.getUniforms(t,h)};function hn(e,n,r){var i=1/Ae(r,1,n.transform.tileZoom),o=Math.pow(2,r.tileID.overscaledZ),a=r.tileSize*Math.pow(2,n.transform.tileZoom)/o,s=a*(r.tileID.canonical.x+r.tileID.wrap*o),A=a*r.tileID.canonical.y;return{u_image:0,u_texsize:r.imageAtlasTexture.size,u_scale:[t.browser.devicePixelRatio,i,e.fromScale,e.toScale],u_fade:e.t,u_pixel_coord_upper:[s>>16,A>>16],u_pixel_coord_lower:[65535&s,65535&A]}}gn.prototype.draw=function(t,e,n,r,i,o,a,s,A,l,c,u,g,h,d,f){var p,I=t.gl;for(var C in t.program.set(this.program),t.setDepthMode(n),t.setStencilMode(r),t.setColorMode(i),t.setCullFace(o),this.fixedUniforms)this.fixedUniforms[C].set(a[C]);h&&h.setUniforms(t,this.binderUniforms,u,{zoom:g});for(var m=(p={},p[I.LINES]=2,p[I.TRIANGLES]=3,p[I.LINE_STRIP]=1,p)[e],y=0,E=c.get();y<E.length;y+=1){var v=E[y],B=v.vaos||(v.vaos={});(B[s]||(B[s]=new un)).bind(t,this,A,h?h.getPaintVertexBuffers():[],l,v.vertexOffset,d,f),I.drawElements(e,v.primitiveLength*m,I.UNSIGNED_SHORT,v.primitiveOffset*m*2)}};var dn=function(e,n,r,i){var o=n.style.light,a=o.properties.get("position"),s=[a.x,a.y,a.z],A=t.create$1();"viewport"===o.properties.get("anchor")&&t.fromRotation(A,-n.transform.angle),t.transformMat3(s,s,A);var l=o.properties.get("color");return{u_matrix:e,u_lightpos:s,u_lightintensity:o.properties.get("intensity"),u_lightcolor:[l.r,l.g,l.b],u_vertical_gradient:+r,u_opacity:i}},fn=function(e,n,r,i,o,a,s){return t.extend(dn(e,n,r,i),hn(a,n,s),{u_height_factor:-Math.pow(2,o.overscaledZ)/s.tileSize/8})},pn=function(t){return{u_matrix:t}},In=function(e,n,r,i){return t.extend(pn(e),hn(r,n,i))},Cn=function(t,e){return{u_matrix:t,u_world:e}},mn=function(e,n,r,i,o){return t.extend(In(e,n,r,i),{u_world:o})},yn=function(e,n,r,i){var o,a,s=e.transform;if("map"===i.paint.get("circle-pitch-alignment")){var A=Ae(r,1,s.zoom);o=!0,a=[A,A]}else o=!1,a=s.pixelsToGLUnits;return{u_camera_to_center_distance:s.cameraToCenterDistance,u_scale_with_map:+("map"===i.paint.get("circle-pitch-scale")),u_matrix:e.translatePosMatrix(n.posMatrix,r,i.paint.get("circle-translate"),i.paint.get("circle-translate-anchor")),u_pitch_with_map:+o,u_device_pixel_ratio:t.browser.devicePixelRatio,u_extrude_scale:a}},En=function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_camera_to_center_distance:new t.Uniform1f(e,n.u_camera_to_center_distance),u_pixels_to_tile_units:new t.Uniform1f(e,n.u_pixels_to_tile_units),u_extrude_scale:new t.Uniform2f(e,n.u_extrude_scale),u_overscale_factor:new t.Uniform1f(e,n.u_overscale_factor)}},vn=function(t,e,n){var r=Ae(n,1,e.zoom),i=Math.pow(2,e.zoom-n.tileID.overscaledZ),o=n.tileID.overscaleFactor();return{u_matrix:t,u_camera_to_center_distance:e.cameraToCenterDistance,u_pixels_to_tile_units:r,u_extrude_scale:[e.pixelsToGLUnits[0]/(r*i),e.pixelsToGLUnits[1]/(r*i)],u_overscale_factor:o}},Bn=function(t,e){return{u_matrix:t,u_color:e}},bn=function(t){return{u_matrix:t}},Qn=function(t,e,n,r){return{u_matrix:t,u_extrude_scale:Ae(e,1,n),u_intensity:r}},wn=function(t,e,n){var r=n.paint.get("hillshade-shadow-color"),i=n.paint.get("hillshade-highlight-color"),o=n.paint.get("hillshade-accent-color"),a=n.paint.get("hillshade-illumination-direction")*(Math.PI/180);"viewport"===n.paint.get("hillshade-illumination-anchor")&&(a-=t.transform.angle);var s=!t.options.moving;return{u_matrix:t.transform.calculatePosMatrix(e.tileID.toUnwrapped(),s),u_image:0,u_latrange:_n(0,e.tileID),u_light:[n.paint.get("hillshade-exaggeration"),a],u_shadow:r,u_highlight:i,u_accent:o}},xn=function(e,n){var r=e.dem.stride,i=t.create();return t.ortho(i,0,t.EXTENT,-t.EXTENT,0,0,1),t.translate(i,i,[0,-t.EXTENT,0]),{u_matrix:i,u_image:1,u_dimension:[r,r],u_zoom:e.tileID.overscaledZ,u_maxzoom:n}};function _n(e,n){var r=Math.pow(2,n.canonical.z),i=n.canonical.y;return[new t.MercatorCoordinate(0,i/r).toLngLat().lat,new t.MercatorCoordinate(0,(i+1)/r).toLngLat().lat]}var Dn=function(e,n,r){var i=e.transform;return{u_matrix:Rn(e,n,r),u_ratio:1/Ae(n,1,i.zoom),u_device_pixel_ratio:t.browser.devicePixelRatio,u_units_to_pixels:[1/i.pixelsToGLUnits[0],1/i.pixelsToGLUnits[1]]}},Mn=function(e,n,r){return t.extend(Dn(e,n,r),{u_image:0})},kn=function(e,n,r,i){var o=e.transform,a=Ln(n,o);return{u_matrix:Rn(e,n,r),u_texsize:n.imageAtlasTexture.size,u_ratio:1/Ae(n,1,o.zoom),u_device_pixel_ratio:t.browser.devicePixelRatio,u_image:0,u_scale:[t.browser.devicePixelRatio,a,i.fromScale,i.toScale],u_fade:i.t,u_units_to_pixels:[1/o.pixelsToGLUnits[0],1/o.pixelsToGLUnits[1]]}},Sn=function(e,n,r,i,o){var a=e.transform,s=e.lineAtlas,A=Ln(n,a),l="round"===r.layout.get("line-cap"),c=s.getDash(i.from,l),u=s.getDash(i.to,l),g=c.width*o.fromScale,h=u.width*o.toScale;return t.extend(Dn(e,n,r),{u_patternscale_a:[A/g,-c.height/2],u_patternscale_b:[A/h,-u.height/2],u_sdfgamma:s.width/(256*Math.min(g,h)*t.browser.devicePixelRatio)/2,u_image:0,u_tex_y_a:c.y,u_tex_y_b:u.y,u_mix:o.t})};function Ln(t,e){return 1/Ae(t,1,e.tileZoom)}function Rn(t,e,n){return t.translatePosMatrix(e.tileID.posMatrix,e,n.paint.get("line-translate"),n.paint.get("line-translate-anchor"))}var Nn=function(t,e,n,r,i){return{u_matrix:t,u_tl_parent:e,u_scale_parent:n,u_buffer_scale:1,u_fade_t:r.mix,u_opacity:r.opacity*i.paint.get("raster-opacity"),u_image0:0,u_image1:1,u_brightness_low:i.paint.get("raster-brightness-min"),u_brightness_high:i.paint.get("raster-brightness-max"),u_saturation_factor:(a=i.paint.get("raster-saturation"),a>0?1-1/(1.001-a):-a),u_contrast_factor:(o=i.paint.get("raster-contrast"),o>0?1/(1-o):1+o),u_spin_weights:function(t){t*=Math.PI/180;var e=Math.sin(t),n=Math.cos(t);return[(2*n+1)/3,(-Math.sqrt(3)*e-n+1)/3,(Math.sqrt(3)*e-n+1)/3]}(i.paint.get("raster-hue-rotate"))};var o,a},Fn=function(t,e,n,r,i,o,a,s,A,l){var c=i.transform;return{u_is_size_zoom_constant:+("constant"===t||"source"===t),u_is_size_feature_constant:+("constant"===t||"camera"===t),u_size_t:e?e.uSizeT:0,u_size:e?e.uSize:0,u_camera_to_center_distance:c.cameraToCenterDistance,u_pitch:c.pitch/360*2*Math.PI,u_rotate_symbol:+n,u_aspect_ratio:c.width/c.height,u_fade_change:i.options.fadeDuration?i.symbolFadeChange:1,u_matrix:o,u_label_plane_matrix:a,u_coord_matrix:s,u_is_text:+A,u_pitch_with_map:+r,u_texsize:l,u_texture:0}},Tn=function(e,n,r,i,o,a,s,A,l,c,u){var g=o.transform;return t.extend(Fn(e,n,r,i,o,a,s,A,l,c),{u_gamma_scale:i?Math.cos(g._pitch)*g.cameraToCenterDistance:1,u_device_pixel_ratio:t.browser.devicePixelRatio,u_is_halo:+u})},Un=function(t,e,n){return{u_matrix:t,u_opacity:e,u_color:n}},On=function(e,n,r,i,o,a){return t.extend(function(t,e,n,r){var i=n.imageManager.getPattern(t.from),o=n.imageManager.getPattern(t.to),a=n.imageManager.getPixelSize(),s=a.width,A=a.height,l=Math.pow(2,r.tileID.overscaledZ),c=r.tileSize*Math.pow(2,n.transform.tileZoom)/l,u=c*(r.tileID.canonical.x+r.tileID.wrap*l),g=c*r.tileID.canonical.y;return{u_image:0,u_pattern_tl_a:i.tl,u_pattern_br_a:i.br,u_pattern_tl_b:o.tl,u_pattern_br_b:o.br,u_texsize:[s,A],u_mix:e.t,u_pattern_size_a:i.displaySize,u_pattern_size_b:o.displaySize,u_scale_a:e.fromScale,u_scale_b:e.toScale,u_tile_units_to_pixels:1/Ae(r,1,n.transform.tileZoom),u_pixel_coord_upper:[u>>16,g>>16],u_pixel_coord_lower:[65535&u,65535&g]}}(i,a,r,o),{u_matrix:e,u_opacity:n})},Gn={fillExtrusion:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_lightpos:new t.Uniform3f(e,n.u_lightpos),u_lightintensity:new t.Uniform1f(e,n.u_lightintensity),u_lightcolor:new t.Uniform3f(e,n.u_lightcolor),u_vertical_gradient:new t.Uniform1f(e,n.u_vertical_gradient),u_opacity:new t.Uniform1f(e,n.u_opacity)}},fillExtrusionPattern:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_lightpos:new t.Uniform3f(e,n.u_lightpos),u_lightintensity:new t.Uniform1f(e,n.u_lightintensity),u_lightcolor:new t.Uniform3f(e,n.u_lightcolor),u_vertical_gradient:new t.Uniform1f(e,n.u_vertical_gradient),u_height_factor:new t.Uniform1f(e,n.u_height_factor),u_image:new t.Uniform1i(e,n.u_image),u_texsize:new t.Uniform2f(e,n.u_texsize),u_pixel_coord_upper:new t.Uniform2f(e,n.u_pixel_coord_upper),u_pixel_coord_lower:new t.Uniform2f(e,n.u_pixel_coord_lower),u_scale:new t.Uniform4f(e,n.u_scale),u_fade:new t.Uniform1f(e,n.u_fade),u_opacity:new t.Uniform1f(e,n.u_opacity)}},fill:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix)}},fillPattern:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_image:new t.Uniform1i(e,n.u_image),u_texsize:new t.Uniform2f(e,n.u_texsize),u_pixel_coord_upper:new t.Uniform2f(e,n.u_pixel_coord_upper),u_pixel_coord_lower:new t.Uniform2f(e,n.u_pixel_coord_lower),u_scale:new t.Uniform4f(e,n.u_scale),u_fade:new t.Uniform1f(e,n.u_fade)}},fillOutline:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_world:new t.Uniform2f(e,n.u_world)}},fillOutlinePattern:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_world:new t.Uniform2f(e,n.u_world),u_image:new t.Uniform1i(e,n.u_image),u_texsize:new t.Uniform2f(e,n.u_texsize),u_pixel_coord_upper:new t.Uniform2f(e,n.u_pixel_coord_upper),u_pixel_coord_lower:new t.Uniform2f(e,n.u_pixel_coord_lower),u_scale:new t.Uniform4f(e,n.u_scale),u_fade:new t.Uniform1f(e,n.u_fade)}},circle:function(e,n){return{u_camera_to_center_distance:new t.Uniform1f(e,n.u_camera_to_center_distance),u_scale_with_map:new t.Uniform1i(e,n.u_scale_with_map),u_pitch_with_map:new t.Uniform1i(e,n.u_pitch_with_map),u_extrude_scale:new t.Uniform2f(e,n.u_extrude_scale),u_device_pixel_ratio:new t.Uniform1f(e,n.u_device_pixel_ratio),u_matrix:new t.UniformMatrix4f(e,n.u_matrix)}},collisionBox:En,collisionCircle:En,debug:function(e,n){return{u_color:new t.UniformColor(e,n.u_color),u_matrix:new t.UniformMatrix4f(e,n.u_matrix)}},clippingMask:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix)}},heatmap:function(e,n){return{u_extrude_scale:new t.Uniform1f(e,n.u_extrude_scale),u_intensity:new t.Uniform1f(e,n.u_intensity),u_matrix:new t.UniformMatrix4f(e,n.u_matrix)}},heatmapTexture:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_world:new t.Uniform2f(e,n.u_world),u_image:new t.Uniform1i(e,n.u_image),u_color_ramp:new t.Uniform1i(e,n.u_color_ramp),u_opacity:new t.Uniform1f(e,n.u_opacity)}},hillshade:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_image:new t.Uniform1i(e,n.u_image),u_latrange:new t.Uniform2f(e,n.u_latrange),u_light:new t.Uniform2f(e,n.u_light),u_shadow:new t.UniformColor(e,n.u_shadow),u_highlight:new t.UniformColor(e,n.u_highlight),u_accent:new t.UniformColor(e,n.u_accent)}},hillshadePrepare:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_image:new t.Uniform1i(e,n.u_image),u_dimension:new t.Uniform2f(e,n.u_dimension),u_zoom:new t.Uniform1f(e,n.u_zoom),u_maxzoom:new t.Uniform1f(e,n.u_maxzoom)}},line:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_ratio:new t.Uniform1f(e,n.u_ratio),u_device_pixel_ratio:new t.Uniform1f(e,n.u_device_pixel_ratio),u_units_to_pixels:new t.Uniform2f(e,n.u_units_to_pixels)}},lineGradient:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_ratio:new t.Uniform1f(e,n.u_ratio),u_device_pixel_ratio:new t.Uniform1f(e,n.u_device_pixel_ratio),u_units_to_pixels:new t.Uniform2f(e,n.u_units_to_pixels),u_image:new t.Uniform1i(e,n.u_image)}},linePattern:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_texsize:new t.Uniform2f(e,n.u_texsize),u_ratio:new t.Uniform1f(e,n.u_ratio),u_device_pixel_ratio:new t.Uniform1f(e,n.u_device_pixel_ratio),u_image:new t.Uniform1i(e,n.u_image),u_units_to_pixels:new t.Uniform2f(e,n.u_units_to_pixels),u_scale:new t.Uniform4f(e,n.u_scale),u_fade:new t.Uniform1f(e,n.u_fade)}},lineSDF:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_ratio:new t.Uniform1f(e,n.u_ratio),u_device_pixel_ratio:new t.Uniform1f(e,n.u_device_pixel_ratio),u_units_to_pixels:new t.Uniform2f(e,n.u_units_to_pixels),u_patternscale_a:new t.Uniform2f(e,n.u_patternscale_a),u_patternscale_b:new t.Uniform2f(e,n.u_patternscale_b),u_sdfgamma:new t.Uniform1f(e,n.u_sdfgamma),u_image:new t.Uniform1i(e,n.u_image),u_tex_y_a:new t.Uniform1f(e,n.u_tex_y_a),u_tex_y_b:new t.Uniform1f(e,n.u_tex_y_b),u_mix:new t.Uniform1f(e,n.u_mix)}},raster:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_tl_parent:new t.Uniform2f(e,n.u_tl_parent),u_scale_parent:new t.Uniform1f(e,n.u_scale_parent),u_buffer_scale:new t.Uniform1f(e,n.u_buffer_scale),u_fade_t:new t.Uniform1f(e,n.u_fade_t),u_opacity:new t.Uniform1f(e,n.u_opacity),u_image0:new t.Uniform1i(e,n.u_image0),u_image1:new t.Uniform1i(e,n.u_image1),u_brightness_low:new t.Uniform1f(e,n.u_brightness_low),u_brightness_high:new t.Uniform1f(e,n.u_brightness_high),u_saturation_factor:new t.Uniform1f(e,n.u_saturation_factor),u_contrast_factor:new t.Uniform1f(e,n.u_contrast_factor),u_spin_weights:new t.Uniform3f(e,n.u_spin_weights)}},symbolIcon:function(e,n){return{u_is_size_zoom_constant:new t.Uniform1i(e,n.u_is_size_zoom_constant),u_is_size_feature_constant:new t.Uniform1i(e,n.u_is_size_feature_constant),u_size_t:new t.Uniform1f(e,n.u_size_t),u_size:new t.Uniform1f(e,n.u_size),u_camera_to_center_distance:new t.Uniform1f(e,n.u_camera_to_center_distance),u_pitch:new t.Uniform1f(e,n.u_pitch),u_rotate_symbol:new t.Uniform1i(e,n.u_rotate_symbol),u_aspect_ratio:new t.Uniform1f(e,n.u_aspect_ratio),u_fade_change:new t.Uniform1f(e,n.u_fade_change),u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_label_plane_matrix:new t.UniformMatrix4f(e,n.u_label_plane_matrix),u_coord_matrix:new t.UniformMatrix4f(e,n.u_coord_matrix),u_is_text:new t.Uniform1f(e,n.u_is_text),u_pitch_with_map:new t.Uniform1i(e,n.u_pitch_with_map),u_texsize:new t.Uniform2f(e,n.u_texsize),u_texture:new t.Uniform1i(e,n.u_texture)}},symbolSDF:function(e,n){return{u_is_size_zoom_constant:new t.Uniform1i(e,n.u_is_size_zoom_constant),u_is_size_feature_constant:new t.Uniform1i(e,n.u_is_size_feature_constant),u_size_t:new t.Uniform1f(e,n.u_size_t),u_size:new t.Uniform1f(e,n.u_size),u_camera_to_center_distance:new t.Uniform1f(e,n.u_camera_to_center_distance),u_pitch:new t.Uniform1f(e,n.u_pitch),u_rotate_symbol:new t.Uniform1i(e,n.u_rotate_symbol),u_aspect_ratio:new t.Uniform1f(e,n.u_aspect_ratio),u_fade_change:new t.Uniform1f(e,n.u_fade_change),u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_label_plane_matrix:new t.UniformMatrix4f(e,n.u_label_plane_matrix),u_coord_matrix:new t.UniformMatrix4f(e,n.u_coord_matrix),u_is_text:new t.Uniform1f(e,n.u_is_text),u_pitch_with_map:new t.Uniform1i(e,n.u_pitch_with_map),u_texsize:new t.Uniform2f(e,n.u_texsize),u_texture:new t.Uniform1i(e,n.u_texture),u_gamma_scale:new t.Uniform1f(e,n.u_gamma_scale),u_device_pixel_ratio:new t.Uniform1f(e,n.u_device_pixel_ratio),u_is_halo:new t.Uniform1f(e,n.u_is_halo)}},background:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_opacity:new t.Uniform1f(e,n.u_opacity),u_color:new t.UniformColor(e,n.u_color)}},backgroundPattern:function(e,n){return{u_matrix:new t.UniformMatrix4f(e,n.u_matrix),u_opacity:new t.Uniform1f(e,n.u_opacity),u_image:new t.Uniform1i(e,n.u_image),u_pattern_tl_a:new t.Uniform2f(e,n.u_pattern_tl_a),u_pattern_br_a:new t.Uniform2f(e,n.u_pattern_br_a),u_pattern_tl_b:new t.Uniform2f(e,n.u_pattern_tl_b),u_pattern_br_b:new t.Uniform2f(e,n.u_pattern_br_b),u_texsize:new t.Uniform2f(e,n.u_texsize),u_mix:new t.Uniform1f(e,n.u_mix),u_pattern_size_a:new t.Uniform2f(e,n.u_pattern_size_a),u_pattern_size_b:new t.Uniform2f(e,n.u_pattern_size_b),u_scale_a:new t.Uniform1f(e,n.u_scale_a),u_scale_b:new t.Uniform1f(e,n.u_scale_b),u_pixel_coord_upper:new t.Uniform2f(e,n.u_pixel_coord_upper),u_pixel_coord_lower:new t.Uniform2f(e,n.u_pixel_coord_lower),u_tile_units_to_pixels:new t.Uniform1f(e,n.u_tile_units_to_pixels)}}};function Hn(e,n){for(var r=e.sort(function(t,e){return t.tileID.isLessThan(e.tileID)?-1:e.tileID.isLessThan(t.tileID)?1:0}),i=0;i<r.length;i++){var o={},a=r[i],s=r.slice(i+1);jn(a.tileID.wrapped(),a.tileID,s,new t.OverscaledTileID(0,a.tileID.wrap+1,0,0,0),o),a.setMask(o,n)}}function jn(e,n,r,i,o){for(var a=0;a<r.length;a++){var s=r[a];if(i.isLessThan(s.tileID))break;if(n.key===s.tileID.key)return;if(s.tileID.isChildOf(n)){for(var A=n.children(1/0),l=0;l<A.length;l++)jn(e,A[l],r.slice(a),i,o);return}}var c=n.overscaledZ-e.overscaledZ,u=new t.CanonicalTileID(c,n.canonical.x-(e.canonical.x<<c),n.canonical.y-(e.canonical.y<<c));o[u.key]=o[u.key]||u}function Yn(t,e,n,r,i){for(var o=t.context,a=o.gl,s=i?t.useProgram("collisionCircle"):t.useProgram("collisionBox"),A=0;A<r.length;A++){var l=r[A],c=e.getTile(l),u=c.getBucket(n);if(u){var g=i?u.collisionCircle:u.collisionBox;g&&s.draw(o,i?a.TRIANGLES:a.LINES,bt.disabled,Qt.disabled,t.colorModeForRenderPass(),xt.disabled,vn(l.posMatrix,t.transform,c),n.id,g.layoutVertexBuffer,g.indexBuffer,g.segments,null,t.transform.zoom,null,null,g.collisionVertexBuffer)}}}var Pn=t.identity(new Float32Array(16));function Kn(e,n,r,i,o,a){var s=t.getAnchorAlignment(e),A=-(s.horizontalAlign-.5)*n,l=-(s.verticalAlign-.5)*r,c=t.evaluateRadialOffset(e,i);return new t.Point((A/o+c[0])*a,(l/o+c[1])*a)}function zn(e,n,r,i,o,a,s,A,l,c){var u=e.text.placedSymbolArray,g=e.text.dynamicLayoutVertexArray;g.clear();for(var h=0;h<u.length;h++){var d=u.get(h),f=!d.hidden&&d.crossTileID?i[d.crossTileID]:null;if(f){var p=new t.Point(d.anchorX,d.anchorY),I=Vt(p,r?A:s),C=.5+a.cameraToCenterDistance/I.signedDistanceFromCamera*.5,m=o.evaluateSizeForFeature(e.textSizeData,c,d)*C/t.ONE_EM;r&&(m*=e.tilePixelRatio/l);for(var y=f.width,E=f.height,v=f.radialOffset,B=f.textBoxScale,b=Kn(f.anchor,y,E,v,B,m),Q=r?Vt(p.add(b),s).point:I.point.add(n?b.rotate(-a.angle):b),w=0;w<d.numGlyphs;w++)t.addDynamicAttributes(g,Q,0)}else ie(d.numGlyphs,g)}e.text.dynamicLayoutVertexBuffer.updateData(g)}function Jn(e,n,r,i,o,a,s,A,l,c,u,g,h){for(var d,f,p=e.context,I=p.gl,C=e.transform,m="map"===A,y="map"===l,E=m&&"point"!==r.layout.get("symbol-placement"),v=m&&!y&&!E,B=void 0!==r.layout.get("symbol-sort-key").constantOr(1),b=e.depthModeForSublayer(0,bt.ReadOnly),Q=r.layout.get("text-variable-anchor"),w=[],x=0,_=i;x<_.length;x+=1){var D=_[x],M=n.getTile(D),k=M.getBucket(r);if(k){var S=o?k.text:k.icon;if(S&&S.segments.get().length){var L=S.programConfigurations.get(r.id),R=o||k.sdfIcons,N=o?k.textSizeData:k.iconSizeData;d||(d=e.useProgram(R?"symbolSDF":"symbolIcon",L),f=t.evaluateSizeForZoom(N,C.zoom)),p.activeTexture.set(I.TEXTURE0);var F=void 0,T=void 0,U=void 0;if(o)T=M.glyphAtlasTexture,U=I.LINEAR,F=M.glyphAtlasTexture.size;else{var O=1!==r.layout.get("icon-size").constantOr(0)||k.iconsNeedLinear,G=y||0!==C.pitch;T=M.imageAtlasTexture,U=R||e.options.rotating||e.options.zooming||O||G?I.LINEAR:I.NEAREST,F=M.imageAtlasTexture.size}var H=Ae(M,1,e.transform.zoom),j=Jt(D.posMatrix,y,m,e.transform,H),Y=qt(D.posMatrix,y,m,e.transform,H);if(E)Xt(k,D.posMatrix,e,o,j,Y,y,c);else if(o&&f&&Q){var P=Math.pow(2,C.zoom-M.tileID.overscaledZ);zn(k,m,y,h,t.symbolSize,C,j,D.posMatrix,P,f)}var K=e.translatePosMatrix(D.posMatrix,M,a,s),z=E||o&&Q?Pn:j,J=e.translatePosMatrix(Y,M,a,s,!0),q=R&&0!==r.paint.get(o?"text-halo-width":"icon-halo-width").constantOr(1),V={program:d,buffers:S,uniformValues:R?Tn(N.kind,f,v,y,e,K,z,J,o,F,!0):Fn(N.kind,f,v,y,e,K,z,J,o,F),atlasTexture:T,atlasInterpolation:U,isSDF:R,hasHalo:q};if(B)for(var W=0,X=S.segments.get();W<X.length;W+=1){var Z=X[W];w.push({segments:new t.SegmentVector([Z]),sortKey:Z.sortKey,state:V})}else w.push({segments:S.segments,sortKey:0,state:V})}}}B&&w.sort(function(t,e){return t.sortKey-e.sortKey});for(var $=0,tt=w;$<tt.length;$+=1){var et=tt[$],nt=et.state;if(nt.atlasTexture.bind(nt.atlasInterpolation,I.CLAMP_TO_EDGE),nt.isSDF){var rt=nt.uniformValues;nt.hasHalo&&(rt.u_is_halo=1,qn(nt.buffers,et.segments,r,e,nt.program,b,u,g,rt)),rt.u_is_halo=0}qn(nt.buffers,et.segments,r,e,nt.program,b,u,g,nt.uniformValues)}}function qn(t,e,n,r,i,o,a,s,A){var l=r.context,c=l.gl;i.draw(l,c.TRIANGLES,o,a,s,xt.disabled,A,n.id,t.layoutVertexBuffer,t.indexBuffer,e,n.paint,r.transform.zoom,t.programConfigurations.get(n.id),t.dynamicLayoutVertexBuffer,t.opacityVertexBuffer)}function Vn(t,e,n,r,i,o,a){var s,A,l,c,u,g=t.context.gl,h=n.paint.get("fill-pattern"),d=h&&h.constantOr(1),f=n.getCrossfadeParameters();a?(A=d&&!n.getPaintProperty("fill-outline-color")?"fillOutlinePattern":"fillOutline",s=g.LINES):(A=d?"fillPattern":"fill",s=g.TRIANGLES);for(var p=0,I=r;p<I.length;p+=1){var C=I[p],m=e.getTile(C);if(!d||m.patternsLoaded()){var y=m.getBucket(n);if(y){var E=y.programConfigurations.get(n.id),v=t.useProgram(A,E);d&&(t.context.activeTexture.set(g.TEXTURE0),m.imageAtlasTexture.bind(g.LINEAR,g.CLAMP_TO_EDGE),E.updatePatternPaintBuffers(f));var B=h.constantOr(null);if(B&&m.imageAtlas){var b=m.imageAtlas.patternPositions[B.to],Q=m.imageAtlas.patternPositions[B.from];b&&Q&&E.setConstantPatternPositions(b,Q)}var w=t.translatePosMatrix(C.posMatrix,m,n.paint.get("fill-translate"),n.paint.get("fill-translate-anchor"));if(a){c=y.indexBuffer2,u=y.segments2;var x=[g.drawingBufferWidth,g.drawingBufferHeight];l="fillOutlinePattern"===A&&d?mn(w,t,f,m,x):Cn(w,x)}else c=y.indexBuffer,u=y.segments,l=d?In(w,t,f,m):pn(w);v.draw(t.context,s,i,t.stencilModeForClipping(C),o,xt.disabled,l,n.id,y.layoutVertexBuffer,c,u,n.paint,t.transform.zoom,E)}}}}function Wn(t,e,n,r,i,o,a){for(var s=t.context,A=s.gl,l=n.paint.get("fill-extrusion-pattern"),c=l.constantOr(1),u=n.getCrossfadeParameters(),g=n.paint.get("fill-extrusion-opacity"),h=0,d=r;h<d.length;h+=1){var f=d[h],p=e.getTile(f),I=p.getBucket(n);if(I){var C=I.programConfigurations.get(n.id),m=t.useProgram(c?"fillExtrusionPattern":"fillExtrusion",C);c&&(t.context.activeTexture.set(A.TEXTURE0),p.imageAtlasTexture.bind(A.LINEAR,A.CLAMP_TO_EDGE),C.updatePatternPaintBuffers(u));var y=l.constantOr(null);if(y&&p.imageAtlas){var E=p.imageAtlas.patternPositions[y.to],v=p.imageAtlas.patternPositions[y.from];E&&v&&C.setConstantPatternPositions(E,v)}var B=t.translatePosMatrix(f.posMatrix,p,n.paint.get("fill-extrusion-translate"),n.paint.get("fill-extrusion-translate-anchor")),b=n.paint.get("fill-extrusion-vertical-gradient"),Q=c?fn(B,t,b,g,f,u,p):dn(B,t,b,g);m.draw(s,s.gl.TRIANGLES,i,o,a,xt.backCCW,Q,n.id,I.layoutVertexBuffer,I.indexBuffer,I.segments,n.paint,t.transform.zoom,C)}}}function Xn(t,e,n,r,i,o){var a=t.context,s=a.gl,A=e.fbo;if(A){var l=t.useProgram("hillshade");a.activeTexture.set(s.TEXTURE0),s.bindTexture(s.TEXTURE_2D,A.colorAttachment.get());var c=wn(t,e,n);e.maskedBoundsBuffer&&e.maskedIndexBuffer&&e.segments?l.draw(a,s.TRIANGLES,r,i,o,xt.disabled,c,n.id,e.maskedBoundsBuffer,e.maskedIndexBuffer,e.segments):l.draw(a,s.TRIANGLES,r,i,o,xt.disabled,c,n.id,t.rasterBoundsBuffer,t.quadTriangleIndexBuffer,t.rasterBoundsSegments)}}function Zn(e,n,r,i,o,a,s){var A=e.context,l=A.gl;if(n.dem&&n.dem.data){var c=n.dem.dim,u=n.dem.stride,g=n.dem.getPixels();if(A.activeTexture.set(l.TEXTURE1),A.pixelStoreUnpackPremultiplyAlpha.set(!1),n.demTexture=n.demTexture||e.getTileTexture(u),n.demTexture){var h=n.demTexture;h.update(g,{premultiply:!1}),h.bind(l.NEAREST,l.CLAMP_TO_EDGE)}else n.demTexture=new t.Texture(A,g,l.RGBA,{premultiply:!1}),n.demTexture.bind(l.NEAREST,l.CLAMP_TO_EDGE);A.activeTexture.set(l.TEXTURE0);var d=n.fbo;if(!d){var f=new t.Texture(A,{width:c,height:c,data:null},l.RGBA);f.bind(l.LINEAR,l.CLAMP_TO_EDGE),(d=n.fbo=A.createFramebuffer(c,c)).colorAttachment.set(f.texture)}A.bindFramebuffer.set(d.framebuffer),A.viewport.set([0,0,c,c]),e.useProgram("hillshadePrepare").draw(A,l.TRIANGLES,o,a,s,xt.disabled,xn(n,i),r.id,e.rasterBoundsBuffer,e.quadTriangleIndexBuffer,e.rasterBoundsSegments),n.needsHillshadePrepare=!1}}function $n(e,n,r,i,o){var a=i.paint.get("raster-fade-duration");if(a>0){var s=t.browser.now(),A=(s-e.timeAdded)/a,l=n?(s-n.timeAdded)/a:-1,c=r.getSource(),u=o.coveringZoomLevel({tileSize:c.tileSize,roundZoom:c.roundZoom}),g=!n||Math.abs(n.tileID.overscaledZ-u)>Math.abs(e.tileID.overscaledZ-u),h=g&&e.refreshedUponExpiration?1:t.clamp(g?A:1-l,0,1);return e.refreshedUponExpiration&&A>=1&&(e.refreshedUponExpiration=!1),n?{opacity:1,mix:1-h}:{opacity:h,mix:0}}return{opacity:1,mix:0}}function tr(e,n,r){var i=e.context,o=i.gl,a=r.posMatrix,s=e.useProgram("debug"),A=bt.disabled,l=Qt.disabled,c=e.colorModeForRenderPass(),u="$debug";s.draw(i,o.LINE_STRIP,A,l,c,xt.disabled,Bn(a,t.Color.red),u,e.debugBuffer,e.tileBorderIndexBuffer,e.debugSegments);for(var g=n.getTileByID(r.key).latestRawTileData,h=g&&g.byteLength||0,d=Math.floor(h/1024),f=function(t,e,n,r){r=r||1;var i,o,a,s,A,l,c,u,g=[];for(i=0,o=t.length;i<o;i++)if(A=er[t[i]]){for(u=null,a=0,s=A[1].length;a<s;a+=2)-1===A[1][a]&&-1===A[1][a+1]?u=null:(l=e+A[1][a]*r,c=200-A[1][a+1]*r,u&&g.push(u.x,u.y,l,c),u={x:l,y:c});e+=A[0]*r}return g}(r.toString()+" "+d+"kb",50,0,5),p=new t.StructArrayLayout2i4,I=new t.StructArrayLayout2ui4,C=0;C<f.length;C+=2)p.emplaceBack(f[C],f[C+1]),I.emplaceBack(C,C+1);for(var m=i.createVertexBuffer(p,Fe.members),y=i.createIndexBuffer(I),E=t.SegmentVector.simpleSegment(0,0,p.length/2,p.length/2),v=n.getTile(r).tileSize,B=t.EXTENT/(Math.pow(2,e.transform.zoom-r.overscaledZ)*v),b=[[-1,-1],[-1,1],[1,-1],[1,1]],Q=0;Q<b.length;Q++){var w=b[Q];s.draw(i,o.LINES,A,l,c,xt.disabled,Bn(t.translate([],a,[B*w[0],B*w[1],0]),t.Color.white),u,m,y,E)}s.draw(i,o.LINES,A,l,c,xt.disabled,Bn(a,t.Color.black),u,m,y,E)}var er={" ":[16,[]],"!":[10,[5,21,5,7,-1,-1,5,2,4,1,5,0,6,1,5,2]],'"':[16,[4,21,4,14,-1,-1,12,21,12,14]],"#":[21,[11,25,4,-7,-1,-1,17,25,10,-7,-1,-1,4,12,18,12,-1,-1,3,6,17,6]],$:[20,[8,25,8,-4,-1,-1,12,25,12,-4,-1,-1,17,18,15,20,12,21,8,21,5,20,3,18,3,16,4,14,5,13,7,12,13,10,15,9,16,8,17,6,17,3,15,1,12,0,8,0,5,1,3,3]],"%":[24,[21,21,3,0,-1,-1,8,21,10,19,10,17,9,15,7,14,5,14,3,16,3,18,4,20,6,21,8,21,10,20,13,19,16,19,19,20,21,21,-1,-1,17,7,15,6,14,4,14,2,16,0,18,0,20,1,21,3,21,5,19,7,17,7]],"&":[26,[23,12,23,13,22,14,21,14,20,13,19,11,17,6,15,3,13,1,11,0,7,0,5,1,4,2,3,4,3,6,4,8,5,9,12,13,13,14,14,16,14,18,13,20,11,21,9,20,8,18,8,16,9,13,11,10,16,3,18,1,20,0,22,0,23,1,23,2]],"'":[10,[5,19,4,20,5,21,6,20,6,18,5,16,4,15]],"(":[14,[11,25,9,23,7,20,5,16,4,11,4,7,5,2,7,-2,9,-5,11,-7]],")":[14,[3,25,5,23,7,20,9,16,10,11,10,7,9,2,7,-2,5,-5,3,-7]],"*":[16,[8,21,8,9,-1,-1,3,18,13,12,-1,-1,13,18,3,12]],"+":[26,[13,18,13,0,-1,-1,4,9,22,9]],",":[10,[6,1,5,0,4,1,5,2,6,1,6,-1,5,-3,4,-4]],"-":[26,[4,9,22,9]],".":[10,[5,2,4,1,5,0,6,1,5,2]],"/":[22,[20,25,2,-7]],0:[20,[9,21,6,20,4,17,3,12,3,9,4,4,6,1,9,0,11,0,14,1,16,4,17,9,17,12,16,17,14,20,11,21,9,21]],1:[20,[6,17,8,18,11,21,11,0]],2:[20,[4,16,4,17,5,19,6,20,8,21,12,21,14,20,15,19,16,17,16,15,15,13,13,10,3,0,17,0]],3:[20,[5,21,16,21,10,13,13,13,15,12,16,11,17,8,17,6,16,3,14,1,11,0,8,0,5,1,4,2,3,4]],4:[20,[13,21,3,7,18,7,-1,-1,13,21,13,0]],5:[20,[15,21,5,21,4,12,5,13,8,14,11,14,14,13,16,11,17,8,17,6,16,3,14,1,11,0,8,0,5,1,4,2,3,4]],6:[20,[16,18,15,20,12,21,10,21,7,20,5,17,4,12,4,7,5,3,7,1,10,0,11,0,14,1,16,3,17,6,17,7,16,10,14,12,11,13,10,13,7,12,5,10,4,7]],7:[20,[17,21,7,0,-1,-1,3,21,17,21]],8:[20,[8,21,5,20,4,18,4,16,5,14,7,13,11,12,14,11,16,9,17,7,17,4,16,2,15,1,12,0,8,0,5,1,4,2,3,4,3,7,4,9,6,11,9,12,13,13,15,14,16,16,16,18,15,20,12,21,8,21]],9:[20,[16,14,15,11,13,9,10,8,9,8,6,9,4,11,3,14,3,15,4,18,6,20,9,21,10,21,13,20,15,18,16,14,16,9,15,4,13,1,10,0,8,0,5,1,4,3]],":":[10,[5,14,4,13,5,12,6,13,5,14,-1,-1,5,2,4,1,5,0,6,1,5,2]],";":[10,[5,14,4,13,5,12,6,13,5,14,-1,-1,6,1,5,0,4,1,5,2,6,1,6,-1,5,-3,4,-4]],"<":[24,[20,18,4,9,20,0]],"=":[26,[4,12,22,12,-1,-1,4,6,22,6]],">":[24,[4,18,20,9,4,0]],"?":[18,[3,16,3,17,4,19,5,20,7,21,11,21,13,20,14,19,15,17,15,15,14,13,13,12,9,10,9,7,-1,-1,9,2,8,1,9,0,10,1,9,2]],"@":[27,[18,13,17,15,15,16,12,16,10,15,9,14,8,11,8,8,9,6,11,5,14,5,16,6,17,8,-1,-1,12,16,10,14,9,11,9,8,10,6,11,5,-1,-1,18,16,17,8,17,6,19,5,21,5,23,7,24,10,24,12,23,15,22,17,20,19,18,20,15,21,12,21,9,20,7,19,5,17,4,15,3,12,3,9,4,6,5,4,7,2,9,1,12,0,15,0,18,1,20,2,21,3,-1,-1,19,16,18,8,18,6,19,5]],A:[18,[9,21,1,0,-1,-1,9,21,17,0,-1,-1,4,7,14,7]],B:[21,[4,21,4,0,-1,-1,4,21,13,21,16,20,17,19,18,17,18,15,17,13,16,12,13,11,-1,-1,4,11,13,11,16,10,17,9,18,7,18,4,17,2,16,1,13,0,4,0]],C:[21,[18,16,17,18,15,20,13,21,9,21,7,20,5,18,4,16,3,13,3,8,4,5,5,3,7,1,9,0,13,0,15,1,17,3,18,5]],D:[21,[4,21,4,0,-1,-1,4,21,11,21,14,20,16,18,17,16,18,13,18,8,17,5,16,3,14,1,11,0,4,0]],E:[19,[4,21,4,0,-1,-1,4,21,17,21,-1,-1,4,11,12,11,-1,-1,4,0,17,0]],F:[18,[4,21,4,0,-1,-1,4,21,17,21,-1,-1,4,11,12,11]],G:[21,[18,16,17,18,15,20,13,21,9,21,7,20,5,18,4,16,3,13,3,8,4,5,5,3,7,1,9,0,13,0,15,1,17,3,18,5,18,8,-1,-1,13,8,18,8]],H:[22,[4,21,4,0,-1,-1,18,21,18,0,-1,-1,4,11,18,11]],I:[8,[4,21,4,0]],J:[16,[12,21,12,5,11,2,10,1,8,0,6,0,4,1,3,2,2,5,2,7]],K:[21,[4,21,4,0,-1,-1,18,21,4,7,-1,-1,9,12,18,0]],L:[17,[4,21,4,0,-1,-1,4,0,16,0]],M:[24,[4,21,4,0,-1,-1,4,21,12,0,-1,-1,20,21,12,0,-1,-1,20,21,20,0]],N:[22,[4,21,4,0,-1,-1,4,21,18,0,-1,-1,18,21,18,0]],O:[22,[9,21,7,20,5,18,4,16,3,13,3,8,4,5,5,3,7,1,9,0,13,0,15,1,17,3,18,5,19,8,19,13,18,16,17,18,15,20,13,21,9,21]],P:[21,[4,21,4,0,-1,-1,4,21,13,21,16,20,17,19,18,17,18,14,17,12,16,11,13,10,4,10]],Q:[22,[9,21,7,20,5,18,4,16,3,13,3,8,4,5,5,3,7,1,9,0,13,0,15,1,17,3,18,5,19,8,19,13,18,16,17,18,15,20,13,21,9,21,-1,-1,12,4,18,-2]],R:[21,[4,21,4,0,-1,-1,4,21,13,21,16,20,17,19,18,17,18,15,17,13,16,12,13,11,4,11,-1,-1,11,11,18,0]],S:[20,[17,18,15,20,12,21,8,21,5,20,3,18,3,16,4,14,5,13,7,12,13,10,15,9,16,8,17,6,17,3,15,1,12,0,8,0,5,1,3,3]],T:[16,[8,21,8,0,-1,-1,1,21,15,21]],U:[22,[4,21,4,6,5,3,7,1,10,0,12,0,15,1,17,3,18,6,18,21]],V:[18,[1,21,9,0,-1,-1,17,21,9,0]],W:[24,[2,21,7,0,-1,-1,12,21,7,0,-1,-1,12,21,17,0,-1,-1,22,21,17,0]],X:[20,[3,21,17,0,-1,-1,17,21,3,0]],Y:[18,[1,21,9,11,9,0,-1,-1,17,21,9,11]],Z:[20,[17,21,3,0,-1,-1,3,21,17,21,-1,-1,3,0,17,0]],"[":[14,[4,25,4,-7,-1,-1,5,25,5,-7,-1,-1,4,25,11,25,-1,-1,4,-7,11,-7]],"\\":[14,[0,21,14,-3]],"]":[14,[9,25,9,-7,-1,-1,10,25,10,-7,-1,-1,3,25,10,25,-1,-1,3,-7,10,-7]],"^":[16,[6,15,8,18,10,15,-1,-1,3,12,8,17,13,12,-1,-1,8,17,8,0]],_:[16,[0,-2,16,-2]],"`":[10,[6,21,5,20,4,18,4,16,5,15,6,16,5,17]],a:[19,[15,14,15,0,-1,-1,15,11,13,13,11,14,8,14,6,13,4,11,3,8,3,6,4,3,6,1,8,0,11,0,13,1,15,3]],b:[19,[4,21,4,0,-1,-1,4,11,6,13,8,14,11,14,13,13,15,11,16,8,16,6,15,3,13,1,11,0,8,0,6,1,4,3]],c:[18,[15,11,13,13,11,14,8,14,6,13,4,11,3,8,3,6,4,3,6,1,8,0,11,0,13,1,15,3]],d:[19,[15,21,15,0,-1,-1,15,11,13,13,11,14,8,14,6,13,4,11,3,8,3,6,4,3,6,1,8,0,11,0,13,1,15,3]],e:[18,[3,8,15,8,15,10,14,12,13,13,11,14,8,14,6,13,4,11,3,8,3,6,4,3,6,1,8,0,11,0,13,1,15,3]],f:[12,[10,21,8,21,6,20,5,17,5,0,-1,-1,2,14,9,14]],g:[19,[15,14,15,-2,14,-5,13,-6,11,-7,8,-7,6,-6,-1,-1,15,11,13,13,11,14,8,14,6,13,4,11,3,8,3,6,4,3,6,1,8,0,11,0,13,1,15,3]],h:[19,[4,21,4,0,-1,-1,4,10,7,13,9,14,12,14,14,13,15,10,15,0]],i:[8,[3,21,4,20,5,21,4,22,3,21,-1,-1,4,14,4,0]],j:[10,[5,21,6,20,7,21,6,22,5,21,-1,-1,6,14,6,-3,5,-6,3,-7,1,-7]],k:[17,[4,21,4,0,-1,-1,14,14,4,4,-1,-1,8,8,15,0]],l:[8,[4,21,4,0]],m:[30,[4,14,4,0,-1,-1,4,10,7,13,9,14,12,14,14,13,15,10,15,0,-1,-1,15,10,18,13,20,14,23,14,25,13,26,10,26,0]],n:[19,[4,14,4,0,-1,-1,4,10,7,13,9,14,12,14,14,13,15,10,15,0]],o:[19,[8,14,6,13,4,11,3,8,3,6,4,3,6,1,8,0,11,0,13,1,15,3,16,6,16,8,15,11,13,13,11,14,8,14]],p:[19,[4,14,4,-7,-1,-1,4,11,6,13,8,14,11,14,13,13,15,11,16,8,16,6,15,3,13,1,11,0,8,0,6,1,4,3]],q:[19,[15,14,15,-7,-1,-1,15,11,13,13,11,14,8,14,6,13,4,11,3,8,3,6,4,3,6,1,8,0,11,0,13,1,15,3]],r:[13,[4,14,4,0,-1,-1,4,8,5,11,7,13,9,14,12,14]],s:[17,[14,11,13,13,10,14,7,14,4,13,3,11,4,9,6,8,11,7,13,6,14,4,14,3,13,1,10,0,7,0,4,1,3,3]],t:[12,[5,21,5,4,6,1,8,0,10,0,-1,-1,2,14,9,14]],u:[19,[4,14,4,4,5,1,7,0,10,0,12,1,15,4,-1,-1,15,14,15,0]],v:[16,[2,14,8,0,-1,-1,14,14,8,0]],w:[22,[3,14,7,0,-1,-1,11,14,7,0,-1,-1,11,14,15,0,-1,-1,19,14,15,0]],x:[17,[3,14,14,0,-1,-1,14,14,3,0]],y:[16,[2,14,8,0,-1,-1,14,14,8,0,6,-4,4,-6,2,-7,1,-7]],z:[17,[14,14,3,0,-1,-1,3,14,14,14,-1,-1,3,0,14,0]],"{":[14,[9,25,7,24,6,23,5,21,5,19,6,17,7,16,8,14,8,12,6,10,-1,-1,7,24,6,22,6,20,7,18,8,17,9,15,9,13,8,11,4,9,8,7,9,5,9,3,8,1,7,0,6,-2,6,-4,7,-6,-1,-1,6,8,8,6,8,4,7,2,6,1,5,-1,5,-3,6,-5,7,-6,9,-7]],"|":[8,[4,25,4,-7]],"}":[14,[5,25,7,24,8,23,9,21,9,19,8,17,7,16,6,14,6,12,8,10,-1,-1,7,24,8,22,8,20,7,18,6,17,5,15,5,13,6,11,10,9,6,7,5,5,5,3,6,1,7,0,8,-2,8,-4,7,-6,-1,-1,8,8,6,6,6,4,7,2,8,1,9,-1,9,-3,8,-5,7,-6,5,-7]],"~":[24,[3,6,3,8,4,11,6,12,8,12,10,11,14,8,16,7,18,7,20,8,21,10,-1,-1,3,8,4,10,6,11,8,11,10,10,14,7,16,6,18,6,20,7,21,10,21,12]]},nr={symbol:function(t,e,n,r,i){if("translucent"===t.renderPass){var o=Qt.disabled,a=t.colorModeForRenderPass();0!==n.paint.get("icon-opacity").constantOr(1)&&Jn(t,e,n,r,!1,n.paint.get("icon-translate"),n.paint.get("icon-translate-anchor"),n.layout.get("icon-rotation-alignment"),n.layout.get("icon-pitch-alignment"),n.layout.get("icon-keep-upright"),o,a,i),0!==n.paint.get("text-opacity").constantOr(1)&&Jn(t,e,n,r,!0,n.paint.get("text-translate"),n.paint.get("text-translate-anchor"),n.layout.get("text-rotation-alignment"),n.layout.get("text-pitch-alignment"),n.layout.get("text-keep-upright"),o,a,i),e.map.showCollisionBoxes&&function(t,e,n,r){Yn(t,e,n,r,!1),Yn(t,e,n,r,!0)}(t,e,n,r)}},circle:function(t,e,n,r){if("translucent"===t.renderPass){var i=n.paint.get("circle-opacity"),o=n.paint.get("circle-stroke-width"),a=n.paint.get("circle-stroke-opacity");if(0!==i.constantOr(1)||0!==o.constantOr(1)&&0!==a.constantOr(1))for(var s=t.context,A=s.gl,l=t.depthModeForSublayer(0,bt.ReadOnly),c=Qt.disabled,u=t.colorModeForRenderPass(),g=0;g<r.length;g++){var h=r[g],d=e.getTile(h),f=d.getBucket(n);if(f){var p=f.programConfigurations.get(n.id);t.useProgram("circle",p).draw(s,A.TRIANGLES,l,c,u,xt.disabled,yn(t,h,d,n),n.id,f.layoutVertexBuffer,f.indexBuffer,f.segments,n.paint,t.transform.zoom,p)}}}},heatmap:function(e,n,r,i){if(0!==r.paint.get("heatmap-opacity"))if("offscreen"===e.renderPass){var o=e.context,a=o.gl,s=e.depthModeForSublayer(0,bt.ReadOnly),A=Qt.disabled,l=new wt([a.ONE,a.ONE],t.Color.transparent,[!0,!0,!0,!0]);!function(t,e,n){var r=t.gl;t.activeTexture.set(r.TEXTURE1),t.viewport.set([0,0,e.width/4,e.height/4]);var i=n.heatmapFbo;if(i)r.bindTexture(r.TEXTURE_2D,i.colorAttachment.get()),t.bindFramebuffer.set(i.framebuffer);else{var o=r.createTexture();r.bindTexture(r.TEXTURE_2D,o),r.texParameteri(r.TEXTURE_2D,r.TEXTURE_WRAP_S,r.CLAMP_TO_EDGE),r.texParameteri(r.TEXTURE_2D,r.TEXTURE_WRAP_T,r.CLAMP_TO_EDGE),r.texParameteri(r.TEXTURE_2D,r.TEXTURE_MIN_FILTER,r.LINEAR),r.texParameteri(r.TEXTURE_2D,r.TEXTURE_MAG_FILTER,r.LINEAR),i=n.heatmapFbo=t.createFramebuffer(e.width/4,e.height/4),function t(e,n,r,i){var o=e.gl;o.texImage2D(o.TEXTURE_2D,0,o.RGBA,n.width/4,n.height/4,0,o.RGBA,e.extTextureHalfFloat?e.extTextureHalfFloat.HALF_FLOAT_OES:o.UNSIGNED_BYTE,null),i.colorAttachment.set(r),e.extTextureHalfFloat&&o.checkFramebufferStatus(o.FRAMEBUFFER)!==o.FRAMEBUFFER_COMPLETE&&(e.extTextureHalfFloat=null,i.colorAttachment.setDirty(),t(e,n,r,i))}(t,e,o,i)}}(o,e,r),o.clear({color:t.Color.transparent});for(var c=0;c<i.length;c++){var u=i[c];if(!n.hasRenderableParent(u)){var g=n.getTile(u),h=g.getBucket(r);if(h){var d=h.programConfigurations.get(r.id),f=e.useProgram("heatmap",d),p=e.transform.zoom;f.draw(o,a.TRIANGLES,s,A,l,xt.disabled,Qn(u.posMatrix,g,p,r.paint.get("heatmap-intensity")),r.id,h.layoutVertexBuffer,h.indexBuffer,h.segments,r.paint,e.transform.zoom,d)}}}o.viewport.set([0,0,e.width,e.height])}else"translucent"===e.renderPass&&(e.context.setColorMode(e.colorModeForRenderPass()),function(e,n){var r=e.context,i=r.gl,o=n.heatmapFbo;if(o){r.activeTexture.set(i.TEXTURE0),i.bindTexture(i.TEXTURE_2D,o.colorAttachment.get()),r.activeTexture.set(i.TEXTURE1);var a=n.colorRampTexture;a||(a=n.colorRampTexture=new t.Texture(r,n.colorRamp,i.RGBA)),a.bind(i.LINEAR,i.CLAMP_TO_EDGE),e.useProgram("heatmapTexture").draw(r,i.TRIANGLES,bt.disabled,Qt.disabled,e.colorModeForRenderPass(),xt.disabled,function(e,n,r,i){var o=t.create();t.ortho(o,0,e.width,e.height,0,0,1);var a=e.context.gl;return{u_matrix:o,u_world:[a.drawingBufferWidth,a.drawingBufferHeight],u_image:0,u_color_ramp:1,u_opacity:n.paint.get("heatmap-opacity")}}(e,n),n.id,e.viewportBuffer,e.quadTriangleIndexBuffer,e.viewportSegments,n.paint,e.transform.zoom)}}(e,r))},line:function(e,n,r,i){if("translucent"===e.renderPass){var o=r.paint.get("line-opacity"),a=r.paint.get("line-width");if(0!==o.constantOr(1)&&0!==a.constantOr(1)){var s=e.depthModeForSublayer(0,bt.ReadOnly),A=e.colorModeForRenderPass(),l=r.paint.get("line-dasharray"),c=r.paint.get("line-pattern"),u=c.constantOr(1),g=r.paint.get("line-gradient"),h=r.getCrossfadeParameters(),d=l?"lineSDF":u?"linePattern":g?"lineGradient":"line",f=e.context,p=f.gl,I=!0;if(g){f.activeTexture.set(p.TEXTURE0);var C=r.gradientTexture;if(!r.gradient)return;C||(C=r.gradientTexture=new t.Texture(f,r.gradient,p.RGBA)),C.bind(p.LINEAR,p.CLAMP_TO_EDGE)}for(var m=0,y=i;m<y.length;m+=1){var E=y[m],v=n.getTile(E);if(!u||v.patternsLoaded()){var B=v.getBucket(r);if(B){var b=B.programConfigurations.get(r.id),Q=e.context.program.get(),w=e.useProgram(d,b),x=I||w.program!==Q,_=c.constantOr(null);if(_&&v.imageAtlas){var D=v.imageAtlas.patternPositions[_.to],M=v.imageAtlas.patternPositions[_.from];D&&M&&b.setConstantPatternPositions(D,M)}var k=l?Sn(e,v,r,l,h):u?kn(e,v,r,h):g?Mn(e,v,r):Dn(e,v,r);l&&(x||e.lineAtlas.dirty)?(f.activeTexture.set(p.TEXTURE0),e.lineAtlas.bind(f)):u&&(f.activeTexture.set(p.TEXTURE0),v.imageAtlasTexture.bind(p.LINEAR,p.CLAMP_TO_EDGE),b.updatePatternPaintBuffers(h)),w.draw(f,p.TRIANGLES,s,e.stencilModeForClipping(E),A,xt.disabled,k,r.id,B.layoutVertexBuffer,B.indexBuffer,B.segments,r.paint,e.transform.zoom,b),I=!1}}}}}},fill:function(e,n,r,i){var o=r.paint.get("fill-color"),a=r.paint.get("fill-opacity");if(0!==a.constantOr(1)){var s=e.colorModeForRenderPass(),A=r.paint.get("fill-pattern"),l=e.opaquePassEnabledForLayer()&&!A.constantOr(1)&&1===o.constantOr(t.Color.transparent).a&&1===a.constantOr(0)?"opaque":"translucent";if(e.renderPass===l){var c=e.depthModeForSublayer(1,"opaque"===e.renderPass?bt.ReadWrite:bt.ReadOnly);Vn(e,n,r,i,c,s,!1)}if("translucent"===e.renderPass&&r.paint.get("fill-antialias")){var u=e.depthModeForSublayer(r.getPaintProperty("fill-outline-color")?2:0,bt.ReadOnly);Vn(e,n,r,i,u,s,!0)}}},"fill-extrusion":function(t,e,n,r){var i=n.paint.get("fill-extrusion-opacity");if(0!==i&&"translucent"===t.renderPass){var o=new bt(t.context.gl.LEQUAL,bt.ReadWrite,t.depthRangeFor3D);if(1!==i||n.paint.get("fill-extrusion-pattern").constantOr(1))Wn(t,e,n,r,o,Qt.disabled,wt.disabled),Wn(t,e,n,r,o,t.stencilModeFor3D(),t.colorModeForRenderPass());else{var a=t.colorModeForRenderPass();Wn(t,e,n,r,o,Qt.disabled,a)}}},hillshade:function(t,e,n,r){if("offscreen"===t.renderPass||"translucent"===t.renderPass){for(var i=t.context,o=e.getSource().maxzoom,a=t.depthModeForSublayer(0,bt.ReadOnly),s=Qt.disabled,A=t.colorModeForRenderPass(),l=0,c=r;l<c.length;l+=1){var u=c[l],g=e.getTile(u);g.needsHillshadePrepare&&"offscreen"===t.renderPass?Zn(t,g,n,o,a,s,A):"translucent"===t.renderPass&&Xn(t,g,n,a,s,A)}i.viewport.set([0,0,t.width,t.height])}},raster:function(t,e,n,r){if("translucent"===t.renderPass&&0!==n.paint.get("raster-opacity"))for(var i=t.context,o=i.gl,a=e.getSource(),s=t.useProgram("raster"),A=Qt.disabled,l=t.colorModeForRenderPass(),c=r.length&&r[0].overscaledZ,u=!t.options.moving,g=0,h=r;g<h.length;g+=1){var d=h[g],f=t.depthModeForSublayer(d.overscaledZ-c,1===n.paint.get("raster-opacity")?bt.ReadWrite:bt.ReadOnly,o.LESS),p=e.getTile(d),I=t.transform.calculatePosMatrix(d.toUnwrapped(),u);p.registerFadeDuration(n.paint.get("raster-fade-duration"));var C=e.findLoadedParent(d,0),m=$n(p,C,e,n,t.transform),y=void 0,E=void 0,v="nearest"===n.paint.get("raster-resampling")?o.NEAREST:o.LINEAR;i.activeTexture.set(o.TEXTURE0),p.texture.bind(v,o.CLAMP_TO_EDGE,o.LINEAR_MIPMAP_NEAREST),i.activeTexture.set(o.TEXTURE1),C?(C.texture.bind(v,o.CLAMP_TO_EDGE,o.LINEAR_MIPMAP_NEAREST),y=Math.pow(2,C.tileID.overscaledZ-p.tileID.overscaledZ),E=[p.tileID.canonical.x*y%1,p.tileID.canonical.y*y%1]):p.texture.bind(v,o.CLAMP_TO_EDGE,o.LINEAR_MIPMAP_NEAREST);var B=Nn(I,E||[0,0],y||1,m,n);a instanceof M?s.draw(i,o.TRIANGLES,f,A,l,xt.disabled,B,n.id,a.boundsBuffer,t.quadTriangleIndexBuffer,a.boundsSegments):p.maskedBoundsBuffer&&p.maskedIndexBuffer&&p.segments?s.draw(i,o.TRIANGLES,f,A,l,xt.disabled,B,n.id,p.maskedBoundsBuffer,p.maskedIndexBuffer,p.segments,n.paint,t.transform.zoom):s.draw(i,o.TRIANGLES,f,A,l,xt.disabled,B,n.id,t.rasterBoundsBuffer,t.quadTriangleIndexBuffer,t.rasterBoundsSegments)}},background:function(t,e,n){var r=n.paint.get("background-color"),i=n.paint.get("background-opacity");if(0!==i){var o=t.context,a=o.gl,s=t.transform,A=s.tileSize,l=n.paint.get("background-pattern");if(!t.isPatternMissing(l)){var c=l||1!==r.a||1!==i?"translucent":"opaque";if(t.renderPass===c){var u=Qt.disabled,g=t.depthModeForSublayer(0,"opaque"===c?bt.ReadWrite:bt.ReadOnly),h=t.colorModeForRenderPass(),d=t.useProgram(l?"backgroundPattern":"background"),f=s.coveringTiles({tileSize:A});l&&(o.activeTexture.set(a.TEXTURE0),t.imageManager.bind(t.context));for(var p=n.getCrossfadeParameters(),I=0,C=f;I<C.length;I+=1){var m=C[I],y=t.transform.calculatePosMatrix(m.toUnwrapped()),E=l?On(y,i,t,l,{tileID:m,tileSize:A},p):Un(y,i,r);d.draw(o,a.TRIANGLES,g,u,h,xt.disabled,E,n.id,t.tileExtentBuffer,t.quadTriangleIndexBuffer,t.tileExtentSegments)}}}}},debug:function(t,e,n){for(var r=0;r<n.length;r++)tr(t,e,n[r])},custom:function(t,e,n){var r=t.context,i=n.implementation;if("offscreen"===t.renderPass){var o=i.prerender;o&&(t.setCustomLayerDefaults(),r.setColorMode(t.colorModeForRenderPass()),o.call(i,r.gl,t.transform.customLayerMatrix()),r.setDirty(),t.setBaseState())}else if("translucent"===t.renderPass){t.setCustomLayerDefaults(),r.setColorMode(t.colorModeForRenderPass()),r.setStencilMode(Qt.disabled);var a="3d"===i.renderingMode?new bt(t.context.gl.LEQUAL,bt.ReadWrite,t.depthRangeFor3D):t.depthModeForSublayer(0,bt.ReadOnly);r.setDepthMode(a),i.render(r.gl,t.transform.customLayerMatrix()),r.setDirty(),t.setBaseState(),r.bindFramebuffer.set(null)}}},rr=function(e,n){this.context=new _t(e),this.transform=n,this._tileTextures={},this.setup(),this.numSublayers=Dt.maxUnderzooming+Dt.maxOverzooming+1,this.depthEpsilon=1/Math.pow(2,16),this.depthRboNeedsClear=!0,this.emptyProgramConfiguration=new t.ProgramConfiguration,this.crossTileSymbolIndex=new ke};function ir(t,e){if(t.y>e.y){var n=t;t=e,e=n}return{x0:t.x,y0:t.y,x1:e.x,y1:e.y,dx:e.x-t.x,dy:e.y-t.y}}function or(t,e,n,r,i){var o=Math.max(n,Math.floor(e.y0)),a=Math.min(r,Math.ceil(e.y1));if(t.x0===e.x0&&t.y0===e.y0?t.x0+e.dy/t.dy*t.dx<e.x1:t.x1-e.dy/t.dy*t.dx<e.x0){var s=t;t=e,e=s}for(var A=t.dx/t.dy,l=e.dx/e.dy,c=t.dx>0,u=e.dx<0,g=o;g<a;g++){var h=A*Math.max(0,Math.min(t.dy,g+c-t.y0))+t.x0,d=l*Math.max(0,Math.min(e.dy,g+u-e.y0))+e.x0;i(Math.floor(d),Math.ceil(h),g)}}function ar(t,e,n,r,i,o){var a,s=ir(t,e),A=ir(e,n),l=ir(n,t);s.dy>A.dy&&(a=s,s=A,A=a),s.dy>l.dy&&(a=s,s=l,l=a),A.dy>l.dy&&(a=A,A=l,l=a),s.dy&&or(l,s,r,i,o),A.dy&&or(l,A,r,i,o)}rr.prototype.resize=function(e,n){var r=this.context.gl;if(this.width=e*t.browser.devicePixelRatio,this.height=n*t.browser.devicePixelRatio,this.context.viewport.set([0,0,this.width,this.height]),this.style)for(var i=0,o=this.style._order;i<o.length;i+=1){var a=o[i];this.style._layers[a].resize()}this.depthRbo&&(r.deleteRenderbuffer(this.depthRbo),this.depthRbo=null)},rr.prototype.setup=function(){var e=this.context,n=new t.StructArrayLayout2i4;n.emplaceBack(0,0),n.emplaceBack(t.EXTENT,0),n.emplaceBack(0,t.EXTENT),n.emplaceBack(t.EXTENT,t.EXTENT),this.tileExtentBuffer=e.createVertexBuffer(n,Fe.members),this.tileExtentSegments=t.SegmentVector.simpleSegment(0,0,4,2);var r=new t.StructArrayLayout2i4;r.emplaceBack(0,0),r.emplaceBack(t.EXTENT,0),r.emplaceBack(0,t.EXTENT),r.emplaceBack(t.EXTENT,t.EXTENT),this.debugBuffer=e.createVertexBuffer(r,Fe.members),this.debugSegments=t.SegmentVector.simpleSegment(0,0,4,5);var i=new t.StructArrayLayout4i8;i.emplaceBack(0,0,0,0),i.emplaceBack(t.EXTENT,0,t.EXTENT,0),i.emplaceBack(0,t.EXTENT,0,t.EXTENT),i.emplaceBack(t.EXTENT,t.EXTENT,t.EXTENT,t.EXTENT),this.rasterBoundsBuffer=e.createVertexBuffer(i,t.rasterBoundsAttributes.members),this.rasterBoundsSegments=t.SegmentVector.simpleSegment(0,0,4,2);var o=new t.StructArrayLayout2i4;o.emplaceBack(0,0),o.emplaceBack(1,0),o.emplaceBack(0,1),o.emplaceBack(1,1),this.viewportBuffer=e.createVertexBuffer(o,Fe.members),this.viewportSegments=t.SegmentVector.simpleSegment(0,0,4,2);var a=new t.StructArrayLayout1ui2;a.emplaceBack(0),a.emplaceBack(1),a.emplaceBack(3),a.emplaceBack(2),a.emplaceBack(0),this.tileBorderIndexBuffer=e.createIndexBuffer(a);var s=new t.StructArrayLayout3ui6;s.emplaceBack(0,1,2),s.emplaceBack(2,1,3),this.quadTriangleIndexBuffer=e.createIndexBuffer(s);var A=this.context.gl;this.stencilClearMode=new Qt({func:A.ALWAYS,mask:0},0,255,A.ZERO,A.ZERO,A.ZERO)},rr.prototype.clearStencil=function(){var e=this.context,n=e.gl;this.nextStencilID=1,this.currentStencilSource=void 0;var r=t.create();t.ortho(r,0,this.width,this.height,0,0,1),t.scale(r,r,[n.drawingBufferWidth,n.drawingBufferHeight,0]),this.useProgram("clippingMask").draw(e,n.TRIANGLES,bt.disabled,this.stencilClearMode,wt.disabled,xt.disabled,bn(r),"$clipping",this.viewportBuffer,this.quadTriangleIndexBuffer,this.viewportSegments)},rr.prototype._renderTileClippingMasks=function(t,e){if(this.currentStencilSource!==t.source&&t.isTileClipped()&&e&&e.length){this.currentStencilSource=t.source;var n=this.context,r=n.gl;this.nextStencilID+e.length>256&&this.clearStencil(),n.setColorMode(wt.disabled),n.setDepthMode(bt.disabled);var i=this.useProgram("clippingMask");this._tileClippingMaskIDs={};for(var o=0,a=e;o<a.length;o+=1){var s=a[o],A=this._tileClippingMaskIDs[s.key]=this.nextStencilID++;i.draw(n,r.TRIANGLES,bt.disabled,new Qt({func:r.ALWAYS,mask:0},A,255,r.KEEP,r.KEEP,r.REPLACE),wt.disabled,xt.disabled,bn(s.posMatrix),"$clipping",this.tileExtentBuffer,this.quadTriangleIndexBuffer,this.tileExtentSegments)}}},rr.prototype.stencilModeFor3D=function(){this.nextStencilID+1>256&&this.clearStencil();var t=this.nextStencilID++,e=this.context.gl;return new Qt({func:e.NOTEQUAL,mask:255},t,255,e.KEEP,e.KEEP,e.REPLACE)},rr.prototype.stencilModeForClipping=function(t){var e=this.context.gl;return new Qt({func:e.EQUAL,mask:255},this._tileClippingMaskIDs[t.key],0,e.KEEP,e.KEEP,e.REPLACE)},rr.prototype.colorModeForRenderPass=function(){var e=this.context.gl;return this._showOverdrawInspector?new wt([e.CONSTANT_COLOR,e.ONE],new t.Color(1/8,1/8,1/8,0),[!0,!0,!0,!0]):"opaque"===this.renderPass?wt.unblended:wt.alphaBlended},rr.prototype.depthModeForSublayer=function(t,e,n){if(!this.opaquePassEnabledForLayer())return bt.disabled;var r=1-((1+this.currentLayer)*this.numSublayers+t)*this.depthEpsilon;return new bt(n||this.context.gl.LEQUAL,e,[r,r])},rr.prototype.opaquePassEnabledForLayer=function(){return this.currentLayer<this.opaquePassCutoff},rr.prototype.render=function(e,n){this.style=e,this.options=n,this.lineAtlas=e.lineAtlas,this.imageManager=e.imageManager,this.glyphManager=e.glyphManager,this.symbolFadeChange=e.placement.symbolFadeChange(t.browser.now()),this.imageManager.beginFrame();var r=this.style._order,i=this.style.sourceCaches;for(var o in i){var a=i[o];a.used&&a.prepare(this.context)}var s={},A={},l={};for(var c in i){var u=i[c];s[c]=u.getVisibleCoordinates(),A[c]=s[c].slice().reverse(),l[c]=u.getVisibleCoordinates(!0).reverse()}for(var g in i){var h=i[g],d=h.getSource();if("raster"===d.type||"raster-dem"===d.type){for(var f=[],p=0,I=s[g];p<I.length;p+=1){var C=I[p];f.push(h.getTile(C))}Hn(f,this.context)}}this.opaquePassCutoff=1/0;for(var m=0;m<r.length;m++){var y=r[m];if(this.style._layers[y].is3D()){this.opaquePassCutoff=m;break}}this.renderPass="offscreen",this.depthRboNeedsClear=!0;for(var E=0,v=r;E<v.length;E+=1){var B=v[E],b=this.style._layers[B];if(b.hasOffscreenPass()&&!b.isHidden(this.transform.zoom)){var Q=A[b.source];("custom"===b.type||Q.length)&&this.renderLayer(this,i[b.source],b,Q)}}for(this.context.bindFramebuffer.set(null),this.context.clear({color:n.showOverdrawInspector?t.Color.black:t.Color.transparent,depth:1}),this.clearStencil(),this._showOverdrawInspector=n.showOverdrawInspector,this.depthRangeFor3D=[0,1-(e._order.length+2)*this.numSublayers*this.depthEpsilon],this.renderPass="opaque",this.currentLayer=r.length-1;this.currentLayer>=0;this.currentLayer--){var w=this.style._layers[r[this.currentLayer]],x=i[w.source],_=s[w.source];this._renderTileClippingMasks(w,_),this.renderLayer(this,x,w,_)}for(this.renderPass="translucent",this.currentLayer=0;this.currentLayer<r.length;this.currentLayer++){var D=this.style._layers[r[this.currentLayer]],M=i[D.source],k=("symbol"===D.type?l:A)[D.source];this._renderTileClippingMasks(D,s[D.source]),this.renderLayer(this,M,D,k)}if(this.options.showTileBoundaries)for(var S in i){nr.debug(this,i[S],s[S]);break}this.context.setDefault()},rr.prototype.setupOffscreenDepthRenderbuffer=function(){var t=this.context;this.depthRbo||(this.depthRbo=t.createRenderbuffer(t.gl.DEPTH_COMPONENT16,this.width,this.height))},rr.prototype.renderLayer=function(t,e,n,r){n.isHidden(this.transform.zoom)||("background"===n.type||"custom"===n.type||r.length)&&(this.id=n.id,nr[n.type](t,e,n,r,this.style.placement.variableOffsets))},rr.prototype.translatePosMatrix=function(e,n,r,i,o){if(!r[0]&&!r[1])return e;var a=o?"map"===i?this.transform.angle:0:"viewport"===i?-this.transform.angle:0;if(a){var s=Math.sin(a),A=Math.cos(a);r=[r[0]*A-r[1]*s,r[0]*s+r[1]*A]}var l=[o?r[0]:Ae(n,r[0],this.transform.zoom),o?r[1]:Ae(n,r[1],this.transform.zoom),0],c=new Float32Array(16);return t.translate(c,e,l),c},rr.prototype.saveTileTexture=function(t){var e=this._tileTextures[t.size[0]];e?e.push(t):this._tileTextures[t.size[0]]=[t]},rr.prototype.getTileTexture=function(t){var e=this._tileTextures[t];return e&&e.length>0?e.pop():null},rr.prototype.isPatternMissing=function(t){if(!t)return!1;var e=this.imageManager.getPattern(t.from),n=this.imageManager.getPattern(t.to);return!e||!n},rr.prototype.useProgram=function(t,e){void 0===e&&(e=this.emptyProgramConfiguration),this.cache=this.cache||{};var n=""+t+(e.cacheKey||"")+(this._showOverdrawInspector?"/overdraw":"");return this.cache[n]||(this.cache[n]=new gn(this.context,cn[t],e,Gn[t],this._showOverdrawInspector)),this.cache[n]},rr.prototype.setCustomLayerDefaults=function(){this.context.unbindVAO(),this.context.cullFace.setDefault(),this.context.activeTexture.setDefault(),this.context.pixelStoreUnpack.setDefault(),this.context.pixelStoreUnpackPremultiplyAlpha.setDefault(),this.context.pixelStoreUnpackFlipY.setDefault()},rr.prototype.setBaseState=function(){var t=this.context.gl;this.context.cullFace.set(!1),this.context.viewport.set([0,0,this.width,this.height]),this.context.blendEquation.set(t.FUNC_ADD)};var sr=function(e,n,r){this.tileSize=512,this.maxValidLatitude=85.051129,this._renderWorldCopies=void 0===r||r,this._minZoom=e||0,this._maxZoom=n||22,this.setMaxBounds(),this.width=0,this.height=0,this._center=new t.LngLat(0,0),this.zoom=0,this.angle=0,this._fov=.6435011087932844,this._pitch=0,this._unmodified=!0,this._posMatrixCache={},this._alignedPosMatrixCache={}},Ar={minZoom:{configurable:!0},maxZoom:{configurable:!0},renderWorldCopies:{configurable:!0},worldSize:{configurable:!0},centerPoint:{configurable:!0},size:{configurable:!0},bearing:{configurable:!0},pitch:{configurable:!0},fov:{configurable:!0},zoom:{configurable:!0},center:{configurable:!0},unmodified:{configurable:!0},point:{configurable:!0}};sr.prototype.clone=function(){var t=new sr(this._minZoom,this._maxZoom,this._renderWorldCopies);return t.tileSize=this.tileSize,t.latRange=this.latRange,t.width=this.width,t.height=this.height,t._center=this._center,t.zoom=this.zoom,t.angle=this.angle,t._fov=this._fov,t._pitch=this._pitch,t._unmodified=this._unmodified,t._calcMatrices(),t},Ar.minZoom.get=function(){return this._minZoom},Ar.minZoom.set=function(t){this._minZoom!==t&&(this._minZoom=t,this.zoom=Math.max(this.zoom,t))},Ar.maxZoom.get=function(){return this._maxZoom},Ar.maxZoom.set=function(t){this._maxZoom!==t&&(this._maxZoom=t,this.zoom=Math.min(this.zoom,t))},Ar.renderWorldCopies.get=function(){return this._renderWorldCopies},Ar.renderWorldCopies.set=function(t){void 0===t?t=!0:null===t&&(t=!1),this._renderWorldCopies=t},Ar.worldSize.get=function(){return this.tileSize*this.scale},Ar.centerPoint.get=function(){return this.size._div(2)},Ar.size.get=function(){return new t.Point(this.width,this.height)},Ar.bearing.get=function(){return-this.angle/Math.PI*180},Ar.bearing.set=function(e){var n=-t.wrap(e,-180,180)*Math.PI/180;this.angle!==n&&(this._unmodified=!1,this.angle=n,this._calcMatrices(),this.rotationMatrix=t.create$2(),t.rotate(this.rotationMatrix,this.rotationMatrix,this.angle))},Ar.pitch.get=function(){return this._pitch/Math.PI*180},Ar.pitch.set=function(e){var n=t.clamp(e,0,60)/180*Math.PI;this._pitch!==n&&(this._unmodified=!1,this._pitch=n,this._calcMatrices())},Ar.fov.get=function(){return this._fov/Math.PI*180},Ar.fov.set=function(t){t=Math.max(.01,Math.min(60,t)),this._fov!==t&&(this._unmodified=!1,this._fov=t/180*Math.PI,this._calcMatrices())},Ar.zoom.get=function(){return this._zoom},Ar.zoom.set=function(t){var e=Math.min(Math.max(t,this.minZoom),this.maxZoom);this._zoom!==e&&(this._unmodified=!1,this._zoom=e,this.scale=this.zoomScale(e),this.tileZoom=Math.floor(e),this.zoomFraction=e-this.tileZoom,this._constrain(),this._calcMatrices())},Ar.center.get=function(){return this._center},Ar.center.set=function(t){t.lat===this._center.lat&&t.lng===this._center.lng||(this._unmodified=!1,this._center=t,this._constrain(),this._calcMatrices())},sr.prototype.coveringZoomLevel=function(t){return(t.roundZoom?Math.round:Math.floor)(this.zoom+this.scaleZoom(this.tileSize/t.tileSize))},sr.prototype.getVisibleUnwrappedCoordinates=function(e){var n=[new t.UnwrappedTileID(0,e)];if(this._renderWorldCopies)for(var r=this.pointCoordinate(new t.Point(0,0)),i=this.pointCoordinate(new t.Point(this.width,0)),o=this.pointCoordinate(new t.Point(this.width,this.height)),a=this.pointCoordinate(new t.Point(0,this.height)),s=Math.floor(Math.min(r.x,i.x,o.x,a.x)),A=Math.floor(Math.max(r.x,i.x,o.x,a.x)),l=s-1;l<=A+1;l++)0!==l&&n.push(new t.UnwrappedTileID(l,e));return n},sr.prototype.coveringTiles=function(e){var n=this.coveringZoomLevel(e),r=n;if(void 0!==e.minzoom&&n<e.minzoom)return[];void 0!==e.maxzoom&&n>e.maxzoom&&(n=e.maxzoom);var i=t.MercatorCoordinate.fromLngLat(this.center),o=Math.pow(2,n),a=new t.Point(o*i.x-.5,o*i.y-.5);return function(e,n,r,i){void 0===i&&(i=!0);var o=1<<e,a={};function s(n,s,A){var l,c,u,g;if(A>=0&&A<=o)for(l=n;l<s;l++)c=Math.floor(l/o),u=(l%o+o)%o,0!==c&&!0!==i||(g=new t.OverscaledTileID(r,c,e,u,A),a[g.key]=g)}var A=n.map(function(e){return new t.Point(e.x,e.y)._mult(o)});return ar(A[0],A[1],A[2],0,o,s),ar(A[2],A[3],A[0],0,o,s),Object.keys(a).map(function(t){return a[t]})}(n,[this.pointCoordinate(new t.Point(0,0)),this.pointCoordinate(new t.Point(this.width,0)),this.pointCoordinate(new t.Point(this.width,this.height)),this.pointCoordinate(new t.Point(0,this.height))],e.reparseOverscaled?r:n,this._renderWorldCopies).sort(function(t,e){return a.dist(t.canonical)-a.dist(e.canonical)})},sr.prototype.resize=function(t,e){this.width=t,this.height=e,this.pixelsToGLUnits=[2/t,-2/e],this._constrain(),this._calcMatrices()},Ar.unmodified.get=function(){return this._unmodified},sr.prototype.zoomScale=function(t){return Math.pow(2,t)},sr.prototype.scaleZoom=function(t){return Math.log(t)/Math.LN2},sr.prototype.project=function(e){var n=t.clamp(e.lat,-this.maxValidLatitude,this.maxValidLatitude);return new t.Point(t.mercatorXfromLng(e.lng)*this.worldSize,t.mercatorYfromLat(n)*this.worldSize)},sr.prototype.unproject=function(e){return new t.MercatorCoordinate(e.x/this.worldSize,e.y/this.worldSize).toLngLat()},Ar.point.get=function(){return this.project(this.center)},sr.prototype.setLocationAtPoint=function(e,n){var r=this.pointCoordinate(n),i=this.pointCoordinate(this.centerPoint),o=this.locationCoordinate(e),a=new t.MercatorCoordinate(o.x-(r.x-i.x),o.y-(r.y-i.y));this.center=this.coordinateLocation(a),this._renderWorldCopies&&(this.center=this.center.wrap())},sr.prototype.locationPoint=function(t){return this.coordinatePoint(this.locationCoordinate(t))},sr.prototype.pointLocation=function(t){return this.coordinateLocation(this.pointCoordinate(t))},sr.prototype.locationCoordinate=function(e){return t.MercatorCoordinate.fromLngLat(e)},sr.prototype.coordinateLocation=function(t){return t.toLngLat()},sr.prototype.pointCoordinate=function(e){var n=[e.x,e.y,0,1],r=[e.x,e.y,1,1];t.transformMat4(n,n,this.pixelMatrixInverse),t.transformMat4(r,r,this.pixelMatrixInverse);var i=n[3],o=r[3],a=n[0]/i,s=r[0]/o,A=n[1]/i,l=r[1]/o,c=n[2]/i,u=r[2]/o,g=c===u?0:(0-c)/(u-c);return new t.MercatorCoordinate(t.number(a,s,g)/this.worldSize,t.number(A,l,g)/this.worldSize)},sr.prototype.coordinatePoint=function(e){var n=[e.x*this.worldSize,e.y*this.worldSize,0,1];return t.transformMat4(n,n,this.pixelMatrix),new t.Point(n[0]/n[3],n[1]/n[3])},sr.prototype.getBounds=function(){return(new t.LngLatBounds).extend(this.pointLocation(new t.Point(0,0))).extend(this.pointLocation(new t.Point(this.width,0))).extend(this.pointLocation(new t.Point(this.width,this.height))).extend(this.pointLocation(new t.Point(0,this.height)))},sr.prototype.getMaxBounds=function(){return this.latRange&&2===this.latRange.length&&this.lngRange&&2===this.lngRange.length?new t.LngLatBounds([this.lngRange[0],this.latRange[0]],[this.lngRange[1],this.latRange[1]]):null},sr.prototype.setMaxBounds=function(t){t?(this.lngRange=[t.getWest(),t.getEast()],this.latRange=[t.getSouth(),t.getNorth()],this._constrain()):(this.lngRange=null,this.latRange=[-this.maxValidLatitude,this.maxValidLatitude])},sr.prototype.calculatePosMatrix=function(e,n){void 0===n&&(n=!1);var r=e.key,i=n?this._alignedPosMatrixCache:this._posMatrixCache;if(i[r])return i[r];var o=e.canonical,a=this.worldSize/this.zoomScale(o.z),s=o.x+Math.pow(2,o.z)*e.wrap,A=t.identity(new Float64Array(16));return t.translate(A,A,[s*a,o.y*a,0]),t.scale(A,A,[a/t.EXTENT,a/t.EXTENT,1]),t.multiply(A,n?this.alignedProjMatrix:this.projMatrix,A),i[r]=new Float32Array(A),i[r]},sr.prototype.customLayerMatrix=function(){return this.mercatorMatrix.slice()},sr.prototype._constrain=function(){if(this.center&&this.width&&this.height&&!this._constraining){this._constraining=!0;var e,n,r,i,o=-90,a=90,s=-180,A=180,l=this.size,c=this._unmodified;if(this.latRange){var u=this.latRange;o=t.mercatorYfromLat(u[1])*this.worldSize,e=(a=t.mercatorYfromLat(u[0])*this.worldSize)-o<l.y?l.y/(a-o):0}if(this.lngRange){var g=this.lngRange;s=t.mercatorXfromLng(g[0])*this.worldSize,n=(A=t.mercatorXfromLng(g[1])*this.worldSize)-s<l.x?l.x/(A-s):0}var h=this.point,d=Math.max(n||0,e||0);if(d)return this.center=this.unproject(new t.Point(n?(A+s)/2:h.x,e?(a+o)/2:h.y)),this.zoom+=this.scaleZoom(d),this._unmodified=c,void(this._constraining=!1);if(this.latRange){var f=h.y,p=l.y/2;f-p<o&&(i=o+p),f+p>a&&(i=a-p)}if(this.lngRange){var I=h.x,C=l.x/2;I-C<s&&(r=s+C),I+C>A&&(r=A-C)}void 0===r&&void 0===i||(this.center=this.unproject(new t.Point(void 0!==r?r:h.x,void 0!==i?i:h.y))),this._unmodified=c,this._constraining=!1}},sr.prototype._calcMatrices=function(){if(this.height){this.cameraToCenterDistance=.5/Math.tan(this._fov/2)*this.height;var e=this._fov/2,n=Math.PI/2+this._pitch,r=Math.sin(e)*this.cameraToCenterDistance/Math.sin(Math.PI-n-e),i=this.point,o=i.x,a=i.y,s=1.01*(Math.cos(Math.PI/2-this._pitch)*r+this.cameraToCenterDistance),A=new Float64Array(16);t.perspective(A,this._fov,this.width/this.height,1,s),t.scale(A,A,[1,-1,1]),t.translate(A,A,[0,0,-this.cameraToCenterDistance]),t.rotateX(A,A,this._pitch),t.rotateZ(A,A,this.angle),t.translate(A,A,[-o,-a,0]),this.mercatorMatrix=t.scale([],A,[this.worldSize,this.worldSize,this.worldSize]),t.scale(A,A,[1,1,t.mercatorZfromAltitude(1,this.center.lat)*this.worldSize,1]),this.projMatrix=A;var l=this.width%2/2,c=this.height%2/2,u=Math.cos(this.angle),g=Math.sin(this.angle),h=o-Math.round(o)+u*l+g*c,d=a-Math.round(a)+u*c+g*l,f=new Float64Array(A);if(t.translate(f,f,[h>.5?h-1:h,d>.5?d-1:d,0]),this.alignedProjMatrix=f,A=t.create(),t.scale(A,A,[this.width/2,-this.height/2,1]),t.translate(A,A,[1,-1,0]),this.labelPlaneMatrix=A,A=t.create(),t.scale(A,A,[1,-1,1]),t.translate(A,A,[-1,-1,0]),t.scale(A,A,[2/this.width,2/this.height,1]),this.glCoordMatrix=A,this.pixelMatrix=t.multiply(new Float64Array(16),this.labelPlaneMatrix,this.projMatrix),!(A=t.invert(new Float64Array(16),this.pixelMatrix)))throw new Error("failed to invert matrix");this.pixelMatrixInverse=A,this._posMatrixCache={},this._alignedPosMatrixCache={}}},sr.prototype.maxPitchScaleFactor=function(){if(!this.pixelMatrixInverse)return 1;var e=this.pointCoordinate(new t.Point(0,0)),n=[e.x*this.worldSize,e.y*this.worldSize,0,1];return t.transformMat4(n,n,this.pixelMatrix)[3]/this.cameraToCenterDistance},sr.prototype.getCameraPoint=function(){var e=this._pitch,n=Math.tan(e)*(this.cameraToCenterDistance||1);return this.centerPoint.add(new t.Point(0,n))},sr.prototype.getCameraQueryGeometry=function(e){var n=this.getCameraPoint();if(1===e.length)return[e[0],n];for(var r=n.x,i=n.y,o=n.x,a=n.y,s=0,A=e;s<A.length;s+=1){var l=A[s];r=Math.min(r,l.x),i=Math.min(i,l.y),o=Math.max(o,l.x),a=Math.max(a,l.y)}return[new t.Point(r,i),new t.Point(o,i),new t.Point(o,a),new t.Point(r,a),new t.Point(r,i)]},Object.defineProperties(sr.prototype,Ar);var lr=function(){var e,n,r,i;t.bindAll(["_onHashChange","_updateHash"],this),this._updateHash=(e=this._updateHashUnthrottled.bind(this),n=!1,r=null,i=function(){r=null,n&&(e(),r=setTimeout(i,300),n=!1)},function(){return n=!0,r||i(),r})};lr.prototype.addTo=function(e){return this._map=e,t.window.addEventListener("hashchange",this._onHashChange,!1),this._map.on("moveend",this._updateHash),this},lr.prototype.remove=function(){return t.window.removeEventListener("hashchange",this._onHashChange,!1),this._map.off("moveend",this._updateHash),clearTimeout(this._updateHash()),delete this._map,this},lr.prototype.getHashString=function(t){var e=this._map.getCenter(),n=Math.round(100*this._map.getZoom())/100,r=Math.ceil((n*Math.LN2+Math.log(512/360/.5))/Math.LN10),i=Math.pow(10,r),o=Math.round(e.lng*i)/i,a=Math.round(e.lat*i)/i,s=this._map.getBearing(),A=this._map.getPitch(),l="";return l+=t?"#/"+o+"/"+a+"/"+n:"#"+n+"/"+a+"/"+o,(s||A)&&(l+="/"+Math.round(10*s)/10),A&&(l+="/"+Math.round(A)),l},lr.prototype._onHashChange=function(){var e=t.window.location.hash.replace("#","").split("/");return e.length>=3&&(this._map.jumpTo({center:[+e[2],+e[1]],zoom:+e[0],bearing:+(e[3]||0),pitch:+(e[4]||0)}),!0)},lr.prototype._updateHashUnthrottled=function(){var e=this.getHashString();try{t.window.history.replaceState(t.window.history.state,"",e)}catch(t){}};var cr=function(e){function r(r,i,o,a){void 0===a&&(a={});var s=n.mousePos(i.getCanvasContainer(),o),A=i.unproject(s);e.call(this,r,t.extend({point:s,lngLat:A,originalEvent:o},a)),this._defaultPrevented=!1,this.target=i}e&&(r.__proto__=e),r.prototype=Object.create(e&&e.prototype),r.prototype.constructor=r;var i={defaultPrevented:{configurable:!0}};return r.prototype.preventDefault=function(){this._defaultPrevented=!0},i.defaultPrevented.get=function(){return this._defaultPrevented},Object.defineProperties(r.prototype,i),r}(t.Event),ur=function(e){function r(r,i,o){var a=n.touchPos(i.getCanvasContainer(),o),s=a.map(function(t){return i.unproject(t)}),A=a.reduce(function(t,e,n,r){return t.add(e.div(r.length))},new t.Point(0,0)),l=i.unproject(A);e.call(this,r,{points:a,point:A,lngLats:s,lngLat:l,originalEvent:o}),this._defaultPrevented=!1}e&&(r.__proto__=e),r.prototype=Object.create(e&&e.prototype),r.prototype.constructor=r;var i={defaultPrevented:{configurable:!0}};return r.prototype.preventDefault=function(){this._defaultPrevented=!0},i.defaultPrevented.get=function(){return this._defaultPrevented},Object.defineProperties(r.prototype,i),r}(t.Event),gr=function(t){function e(e,n,r){t.call(this,e,{originalEvent:r}),this._defaultPrevented=!1}t&&(e.__proto__=t),e.prototype=Object.create(t&&t.prototype),e.prototype.constructor=e;var n={defaultPrevented:{configurable:!0}};return e.prototype.preventDefault=function(){this._defaultPrevented=!0},n.defaultPrevented.get=function(){return this._defaultPrevented},Object.defineProperties(e.prototype,n),e}(t.Event),hr=function(e){this._map=e,this._el=e.getCanvasContainer(),this._delta=0,this._defaultZoomRate=.01,this._wheelZoomRate=1/450,t.bindAll(["_onWheel","_onTimeout","_onScrollFrame","_onScrollFinished"],this)};hr.prototype.setZoomRate=function(t){this._defaultZoomRate=t},hr.prototype.setWheelZoomRate=function(t){this._wheelZoomRate=t},hr.prototype.isEnabled=function(){return!!this._enabled},hr.prototype.isActive=function(){return!!this._active},hr.prototype.isZooming=function(){return!!this._zooming},hr.prototype.enable=function(t){this.isEnabled()||(this._enabled=!0,this._aroundCenter=t&&"center"===t.around)},hr.prototype.disable=function(){this.isEnabled()&&(this._enabled=!1)},hr.prototype.onWheel=function(e){if(this.isEnabled()){var n=e.deltaMode===t.window.WheelEvent.DOM_DELTA_LINE?40*e.deltaY:e.deltaY,r=t.browser.now(),i=r-(this._lastWheelEventTime||0);this._lastWheelEventTime=r,0!==n&&n%4.000244140625==0?this._type="wheel":0!==n&&Math.abs(n)<4?this._type="trackpad":i>400?(this._type=null,this._lastValue=n,this._timeout=setTimeout(this._onTimeout,40,e)):this._type||(this._type=Math.abs(i*n)<200?"trackpad":"wheel",this._timeout&&(clearTimeout(this._timeout),this._timeout=null,n+=this._lastValue)),e.shiftKey&&n&&(n/=4),this._type&&(this._lastWheelEvent=e,this._delta-=n,this.isActive()||this._start(e)),e.preventDefault()}},hr.prototype._onTimeout=function(t){this._type="wheel",this._delta-=this._lastValue,this.isActive()||this._start(t)},hr.prototype._start=function(e){if(this._delta){this._frameId&&(this._map._cancelRenderFrame(this._frameId),this._frameId=null),this._active=!0,this.isZooming()||(this._zooming=!0,this._map.fire(new t.Event("movestart",{originalEvent:e})),this._map.fire(new t.Event("zoomstart",{originalEvent:e}))),this._finishTimeout&&clearTimeout(this._finishTimeout);var r=n.mousePos(this._el,e);this._around=t.LngLat.convert(this._aroundCenter?this._map.getCenter():this._map.unproject(r)),this._aroundPoint=this._map.transform.locationPoint(this._around),this._frameId||(this._frameId=this._map._requestRenderFrame(this._onScrollFrame))}},hr.prototype._onScrollFrame=function(){var e=this;if(this._frameId=null,this.isActive()){var n=this._map.transform;if(0!==this._delta){var r="wheel"===this._type&&Math.abs(this._delta)>4.000244140625?this._wheelZoomRate:this._defaultZoomRate,i=2/(1+Math.exp(-Math.abs(this._delta*r)));this._delta<0&&0!==i&&(i=1/i);var o="number"==typeof this._targetZoom?n.zoomScale(this._targetZoom):n.scale;this._targetZoom=Math.min(n.maxZoom,Math.max(n.minZoom,n.scaleZoom(o*i))),"wheel"===this._type&&(this._startZoom=n.zoom,this._easing=this._smoothOutEasing(200)),this._delta=0}var a="number"==typeof this._targetZoom?this._targetZoom:n.zoom,s=this._startZoom,A=this._easing,l=!1;if("wheel"===this._type&&s&&A){var c=Math.min((t.browser.now()-this._lastWheelEventTime)/200,1),u=A(c);n.zoom=t.number(s,a,u),c<1?this._frameId||(this._frameId=this._map._requestRenderFrame(this._onScrollFrame)):l=!0}else n.zoom=a,l=!0;n.setLocationAtPoint(this._around,this._aroundPoint),this._map.fire(new t.Event("move",{originalEvent:this._lastWheelEvent})),this._map.fire(new t.Event("zoom",{originalEvent:this._lastWheelEvent})),l&&(this._active=!1,this._finishTimeout=setTimeout(function(){e._zooming=!1,e._map.fire(new t.Event("zoomend",{originalEvent:e._lastWheelEvent})),e._map.fire(new t.Event("moveend",{originalEvent:e._lastWheelEvent})),delete e._targetZoom},200))}},hr.prototype._smoothOutEasing=function(e){var n=t.ease;if(this._prevEase){var r=this._prevEase,i=(t.browser.now()-r.start)/r.duration,o=r.easing(i+.01)-r.easing(i),a=.27/Math.sqrt(o*o+1e-4)*.01,s=Math.sqrt(.0729-a*a);n=t.bezier(a,s,.25,1)}return this._prevEase={start:t.browser.now(),duration:e,easing:n},n};var dr=function(e,n){this._map=e,this._el=e.getCanvasContainer(),this._container=e.getContainer(),this._clickTolerance=n.clickTolerance||1,t.bindAll(["_onMouseMove","_onMouseUp","_onKeyDown"],this)};dr.prototype.isEnabled=function(){return!!this._enabled},dr.prototype.isActive=function(){return!!this._active},dr.prototype.enable=function(){this.isEnabled()||(this._enabled=!0)},dr.prototype.disable=function(){this.isEnabled()&&(this._enabled=!1)},dr.prototype.onMouseDown=function(e){this.isEnabled()&&e.shiftKey&&0===e.button&&(t.window.document.addEventListener("mousemove",this._onMouseMove,!1),t.window.document.addEventListener("keydown",this._onKeyDown,!1),t.window.document.addEventListener("mouseup",this._onMouseUp,!1),n.disableDrag(),this._startPos=this._lastPos=n.mousePos(this._el,e),this._active=!0)},dr.prototype._onMouseMove=function(t){var e=n.mousePos(this._el,t);if(!(this._lastPos.equals(e)||!this._box&&e.dist(this._startPos)<this._clickTolerance)){var r=this._startPos;this._lastPos=e,this._box||(this._box=n.create("div","mapboxgl-boxzoom",this._container),this._container.classList.add("mapboxgl-crosshair"),this._fireEvent("boxzoomstart",t));var i=Math.min(r.x,e.x),o=Math.max(r.x,e.x),a=Math.min(r.y,e.y),s=Math.max(r.y,e.y);n.setTransform(this._box,"translate("+i+"px,"+a+"px)"),this._box.style.width=o-i+"px",this._box.style.height=s-a+"px"}},dr.prototype._onMouseUp=function(e){if(0===e.button){var r=this._startPos,i=n.mousePos(this._el,e);this._finish(),n.suppressClick(),r.x===i.x&&r.y===i.y?this._fireEvent("boxzoomcancel",e):this._map.fitScreenCoordinates(r,i,this._map.getBearing(),{linear:!0}).fire(new t.Event("boxzoomend",{originalEvent:e}))}},dr.prototype._onKeyDown=function(t){27===t.keyCode&&(this._finish(),this._fireEvent("boxzoomcancel",t))},dr.prototype._finish=function(){this._active=!1,t.window.document.removeEventListener("mousemove",this._onMouseMove,!1),t.window.document.removeEventListener("keydown",this._onKeyDown,!1),t.window.document.removeEventListener("mouseup",this._onMouseUp,!1),this._container.classList.remove("mapboxgl-crosshair"),this._box&&(n.remove(this._box),this._box=null),n.enableDrag(),delete this._startPos,delete this._lastPos},dr.prototype._fireEvent=function(e,n){return this._map.fire(new t.Event(e,{originalEvent:n}))};var fr=t.bezier(0,0,.25,1),pr=function(e,n){this._map=e,this._el=n.element||e.getCanvasContainer(),this._state="disabled",this._button=n.button||"right",this._bearingSnap=n.bearingSnap||0,this._pitchWithRotate=!1!==n.pitchWithRotate,t.bindAll(["onMouseDown","_onMouseMove","_onMouseUp","_onBlur","_onDragFrame"],this)};pr.prototype.isEnabled=function(){return"disabled"!==this._state},pr.prototype.isActive=function(){return"active"===this._state},pr.prototype.enable=function(){this.isEnabled()||(this._state="enabled")},pr.prototype.disable=function(){if(this.isEnabled())switch(this._state){case"active":this._state="disabled",this._unbind(),this._deactivate(),this._fireEvent("rotateend"),this._pitchWithRotate&&this._fireEvent("pitchend"),this._fireEvent("moveend");break;case"pending":this._state="disabled",this._unbind();break;default:this._state="disabled"}},pr.prototype.onMouseDown=function(e){if("enabled"===this._state){if("right"===this._button){if(this._eventButton=n.mouseButton(e),this._eventButton!==(e.ctrlKey?0:2))return}else{if(e.ctrlKey||0!==n.mouseButton(e))return;this._eventButton=0}n.disableDrag(),t.window.document.addEventListener("mousemove",this._onMouseMove,{capture:!0}),t.window.document.addEventListener("mouseup",this._onMouseUp),t.window.addEventListener("blur",this._onBlur),this._state="pending",this._inertia=[[t.browser.now(),this._map.getBearing()]],this._startPos=this._lastPos=n.mousePos(this._el,e),this._center=this._map.transform.centerPoint,e.preventDefault()}},pr.prototype._onMouseMove=function(t){var e=n.mousePos(this._el,t);this._lastPos.equals(e)||(this._lastMoveEvent=t,this._lastPos=e,"pending"===this._state&&(this._state="active",this._fireEvent("rotatestart",t),this._fireEvent("movestart",t),this._pitchWithRotate&&this._fireEvent("pitchstart",t)),this._frameId||(this._frameId=this._map._requestRenderFrame(this._onDragFrame)))},pr.prototype._onDragFrame=function(){this._frameId=null;var e=this._lastMoveEvent;if(e){var n=this._map.transform,r=this._startPos,i=this._lastPos,o=.8*(r.x-i.x),a=-.5*(r.y-i.y),s=n.bearing-o,A=n.pitch-a,l=this._inertia,c=l[l.length-1];this._drainInertiaBuffer(),l.push([t.browser.now(),this._map._normalizeBearing(s,c[1])]),n.bearing=s,this._pitchWithRotate&&(this._fireEvent("pitch",e),n.pitch=A),this._fireEvent("rotate",e),this._fireEvent("move",e),delete this._lastMoveEvent,this._startPos=this._lastPos}},pr.prototype._onMouseUp=function(t){if(n.mouseButton(t)===this._eventButton)switch(this._state){case"active":this._state="enabled",n.suppressClick(),this._unbind(),this._deactivate(),this._inertialRotate(t);break;case"pending":this._state="enabled",this._unbind()}},pr.prototype._onBlur=function(t){switch(this._state){case"active":this._state="enabled",this._unbind(),this._deactivate(),this._fireEvent("rotateend",t),this._pitchWithRotate&&this._fireEvent("pitchend",t),this._fireEvent("moveend",t);break;case"pending":this._state="enabled",this._unbind()}},pr.prototype._unbind=function(){t.window.document.removeEventListener("mousemove",this._onMouseMove,{capture:!0}),t.window.document.removeEventListener("mouseup",this._onMouseUp),t.window.removeEventListener("blur",this._onBlur),n.enableDrag()},pr.prototype._deactivate=function(){this._frameId&&(this._map._cancelRenderFrame(this._frameId),this._frameId=null),delete this._lastMoveEvent,delete this._startPos,delete this._lastPos},pr.prototype._inertialRotate=function(t){var e=this;this._fireEvent("rotateend",t),this._drainInertiaBuffer();var n=this._map,r=n.getBearing(),i=this._inertia,o=function(){Math.abs(r)<e._bearingSnap?n.resetNorth({noMoveStart:!0},{originalEvent:t}):e._fireEvent("moveend",t),e._pitchWithRotate&&e._fireEvent("pitchend",t)};if(i.length<2)o();else{var a=i[0],s=i[i.length-1],A=i[i.length-2],l=n._normalizeBearing(r,A[1]),c=s[1]-a[1],u=c<0?-1:1,g=(s[0]-a[0])/1e3;if(0!==c&&0!==g){var h=Math.abs(c*(.25/g));h>180&&(h=180);var d=h/180;l+=u*h*(d/2),Math.abs(n._normalizeBearing(l,0))<this._bearingSnap&&(l=n._normalizeBearing(0,l)),n.rotateTo(l,{duration:1e3*d,easing:fr,noMoveStart:!0},{originalEvent:t})}else o()}},pr.prototype._fireEvent=function(e,n){return this._map.fire(new t.Event(e,n?{originalEvent:n}:{}))},pr.prototype._drainInertiaBuffer=function(){for(var e=this._inertia,n=t.browser.now();e.length>0&&n-e[0][0]>160;)e.shift()};var Ir=t.bezier(0,0,.3,1),Cr=function(e,n){this._map=e,this._el=e.getCanvasContainer(),this._state="disabled",this._clickTolerance=n.clickTolerance||1,t.bindAll(["_onMove","_onMouseUp","_onTouchEnd","_onBlur","_onDragFrame"],this)};Cr.prototype.isEnabled=function(){return"disabled"!==this._state},Cr.prototype.isActive=function(){return"active"===this._state},Cr.prototype.enable=function(){this.isEnabled()||(this._el.classList.add("mapboxgl-touch-drag-pan"),this._state="enabled")},Cr.prototype.disable=function(){if(this.isEnabled())switch(this._el.classList.remove("mapboxgl-touch-drag-pan"),this._state){case"active":this._state="disabled",this._unbind(),this._deactivate(),this._fireEvent("dragend"),this._fireEvent("moveend");break;case"pending":this._state="disabled",this._unbind();break;default:this._state="disabled"}},Cr.prototype.onMouseDown=function(e){"enabled"===this._state&&(e.ctrlKey||0!==n.mouseButton(e)||(n.addEventListener(t.window.document,"mousemove",this._onMove,{capture:!0}),n.addEventListener(t.window.document,"mouseup",this._onMouseUp),this._start(e)))},Cr.prototype.onTouchStart=function(e){"enabled"===this._state&&(e.touches.length>1||(n.addEventListener(t.window.document,"touchmove",this._onMove,{capture:!0,passive:!1}),n.addEventListener(t.window.document,"touchend",this._onTouchEnd),this._start(e)))},Cr.prototype._start=function(e){t.window.addEventListener("blur",this._onBlur),this._state="pending",this._startPos=this._mouseDownPos=this._lastPos=n.mousePos(this._el,e),this._inertia=[[t.browser.now(),this._startPos]]},Cr.prototype._onMove=function(e){e.preventDefault();var r=n.mousePos(this._el,e);this._lastPos.equals(r)||"pending"===this._state&&r.dist(this._mouseDownPos)<this._clickTolerance||(this._lastMoveEvent=e,this._lastPos=r,this._drainInertiaBuffer(),this._inertia.push([t.browser.now(),this._lastPos]),"pending"===this._state&&(this._state="active",this._fireEvent("dragstart",e),this._fireEvent("movestart",e)),this._frameId||(this._frameId=this._map._requestRenderFrame(this._onDragFrame)))},Cr.prototype._onDragFrame=function(){this._frameId=null;var t=this._lastMoveEvent;if(t){var e=this._map.transform;e.setLocationAtPoint(e.pointLocation(this._startPos),this._lastPos),this._fireEvent("drag",t),this._fireEvent("move",t),this._startPos=this._lastPos,delete this._lastMoveEvent}},Cr.prototype._onMouseUp=function(t){if(0===n.mouseButton(t))switch(this._state){case"active":this._state="enabled",n.suppressClick(),this._unbind(),this._deactivate(),this._inertialPan(t);break;case"pending":this._state="enabled",this._unbind()}},Cr.prototype._onTouchEnd=function(t){switch(this._state){case"active":this._state="enabled",this._unbind(),this._deactivate(),this._inertialPan(t);break;case"pending":this._state="enabled",this._unbind()}},Cr.prototype._onBlur=function(t){switch(this._state){case"active":this._state="enabled",this._unbind(),this._deactivate(),this._fireEvent("dragend",t),this._fireEvent("moveend",t);break;case"pending":this._state="enabled",this._unbind()}},Cr.prototype._unbind=function(){n.removeEventListener(t.window.document,"touchmove",this._onMove,{capture:!0,passive:!1}),n.removeEventListener(t.window.document,"touchend",this._onTouchEnd),n.removeEventListener(t.window.document,"mousemove",this._onMove,{capture:!0}),n.removeEventListener(t.window.document,"mouseup",this._onMouseUp),n.removeEventListener(t.window,"blur",this._onBlur)},Cr.prototype._deactivate=function(){this._frameId&&(this._map._cancelRenderFrame(this._frameId),this._frameId=null),delete this._lastMoveEvent,delete this._startPos,delete this._mouseDownPos,delete this._lastPos},Cr.prototype._inertialPan=function(t){this._fireEvent("dragend",t),this._drainInertiaBuffer();var e=this._inertia;if(e.length<2)this._fireEvent("moveend",t);else{var n=e[e.length-1],r=e[0],i=n[1].sub(r[1]),o=(n[0]-r[0])/1e3;if(0===o||n[1].equals(r[1]))this._fireEvent("moveend",t);else{var a=i.mult(.3/o),s=a.mag();s>1400&&(s=1400,a._unit()._mult(s));var A=s/750,l=a.mult(-A/2);this._map.panBy(l,{duration:1e3*A,easing:Ir,noMoveStart:!0},{originalEvent:t})}}},Cr.prototype._fireEvent=function(e,n){return this._map.fire(new t.Event(e,n?{originalEvent:n}:{}))},Cr.prototype._drainInertiaBuffer=function(){for(var e=this._inertia,n=t.browser.now();e.length>0&&n-e[0][0]>160;)e.shift()};var mr=function(e){this._map=e,this._el=e.getCanvasContainer(),t.bindAll(["_onKeyDown"],this)};function yr(t){return t*(2-t)}mr.prototype.isEnabled=function(){return!!this._enabled},mr.prototype.enable=function(){this.isEnabled()||(this._el.addEventListener("keydown",this._onKeyDown,!1),this._enabled=!0)},mr.prototype.disable=function(){this.isEnabled()&&(this._el.removeEventListener("keydown",this._onKeyDown),this._enabled=!1)},mr.prototype._onKeyDown=function(t){if(!(t.altKey||t.ctrlKey||t.metaKey)){var e=0,n=0,r=0,i=0,o=0;switch(t.keyCode){case 61:case 107:case 171:case 187:e=1;break;case 189:case 109:case 173:e=-1;break;case 37:t.shiftKey?n=-1:(t.preventDefault(),i=-1);break;case 39:t.shiftKey?n=1:(t.preventDefault(),i=1);break;case 38:t.shiftKey?r=1:(t.preventDefault(),o=-1);break;case 40:t.shiftKey?r=-1:(o=1,t.preventDefault());break;default:return}var a=this._map,s=a.getZoom(),A={duration:300,delayEndEvents:500,easing:yr,zoom:e?Math.round(s)+e*(t.shiftKey?2:1):s,bearing:a.getBearing()+15*n,pitch:a.getPitch()+10*r,offset:[100*-i,100*-o],center:a.getCenter()};a.easeTo(A,{originalEvent:t})}};var Er=function(e){this._map=e,t.bindAll(["_onDblClick","_onZoomEnd"],this)};Er.prototype.isEnabled=function(){return!!this._enabled},Er.prototype.isActive=function(){return!!this._active},Er.prototype.enable=function(){this.isEnabled()||(this._enabled=!0)},Er.prototype.disable=function(){this.isEnabled()&&(this._enabled=!1)},Er.prototype.onTouchStart=function(t){var e=this;if(this.isEnabled()&&!(t.points.length>1))if(this._tapped){var n=t.points[0],r=this._tappedPoint;if(r&&r.dist(n)<=30){t.originalEvent.preventDefault();var i=function(){e._tapped&&e._zoom(t),e._map.off("touchcancel",o),e._resetTapped()},o=function(){e._map.off("touchend",i),e._resetTapped()};this._map.once("touchend",i),this._map.once("touchcancel",o)}else this._resetTapped()}else this._tappedPoint=t.points[0],this._tapped=setTimeout(function(){e._tapped=null,e._tappedPoint=null},300)},Er.prototype._resetTapped=function(){clearTimeout(this._tapped),this._tapped=null,this._tappedPoint=null},Er.prototype.onDblClick=function(t){this.isEnabled()&&(t.originalEvent.preventDefault(),this._zoom(t))},Er.prototype._zoom=function(t){this._active=!0,this._map.on("zoomend",this._onZoomEnd),this._map.zoomTo(this._map.getZoom()+(t.originalEvent.shiftKey?-1:1),{around:t.lngLat},t)},Er.prototype._onZoomEnd=function(){this._active=!1,this._map.off("zoomend",this._onZoomEnd)};var vr=t.bezier(0,0,.15,1),Br=function(e){this._map=e,this._el=e.getCanvasContainer(),t.bindAll(["_onMove","_onEnd","_onTouchFrame"],this)};Br.prototype.isEnabled=function(){return!!this._enabled},Br.prototype.enable=function(t){this.isEnabled()||(this._el.classList.add("mapboxgl-touch-zoom-rotate"),this._enabled=!0,this._aroundCenter=!!t&&"center"===t.around)},Br.prototype.disable=function(){this.isEnabled()&&(this._el.classList.remove("mapboxgl-touch-zoom-rotate"),this._enabled=!1)},Br.prototype.disableRotation=function(){this._rotationDisabled=!0},Br.prototype.enableRotation=function(){this._rotationDisabled=!1},Br.prototype.onStart=function(e){if(this.isEnabled()&&2===e.touches.length){var r=n.mousePos(this._el,e.touches[0]),i=n.mousePos(this._el,e.touches[1]),o=r.add(i).div(2);this._startVec=r.sub(i),this._startAround=this._map.transform.pointLocation(o),this._gestureIntent=void 0,this._inertia=[],n.addEventListener(t.window.document,"touchmove",this._onMove,{passive:!1}),n.addEventListener(t.window.document,"touchend",this._onEnd)}},Br.prototype._getTouchEventData=function(t){var e=n.mousePos(this._el,t.touches[0]),r=n.mousePos(this._el,t.touches[1]),i=e.sub(r);return{vec:i,center:e.add(r).div(2),scale:i.mag()/this._startVec.mag(),bearing:this._rotationDisabled?0:180*i.angleWith(this._startVec)/Math.PI}},Br.prototype._onMove=function(e){if(2===e.touches.length){var n=this._getTouchEventData(e),r=n.vec,i=n.scale,o=n.bearing;if(!this._gestureIntent){var a=this._rotationDisabled&&1!==i||Math.abs(1-i)>.15;Math.abs(o)>10?this._gestureIntent="rotate":a&&(this._gestureIntent="zoom"),this._gestureIntent&&(this._map.fire(new t.Event(this._gestureIntent+"start",{originalEvent:e})),this._map.fire(new t.Event("movestart",{originalEvent:e})),this._startVec=r)}this._lastTouchEvent=e,this._frameId||(this._frameId=this._map._requestRenderFrame(this._onTouchFrame)),e.preventDefault()}},Br.prototype._onTouchFrame=function(){this._frameId=null;var e=this._gestureIntent;if(e){var n=this._map.transform;this._startScale||(this._startScale=n.scale,this._startBearing=n.bearing);var r=this._getTouchEventData(this._lastTouchEvent),i=r.center,o=r.bearing,a=r.scale,s=n.pointLocation(i),A=n.locationPoint(s);"rotate"===e&&(n.bearing=this._startBearing+o),n.zoom=n.scaleZoom(this._startScale*a),n.setLocationAtPoint(this._startAround,A),this._map.fire(new t.Event(e,{originalEvent:this._lastTouchEvent})),this._map.fire(new t.Event("move",{originalEvent:this._lastTouchEvent})),this._drainInertiaBuffer(),this._inertia.push([t.browser.now(),a,i])}},Br.prototype._onEnd=function(e){n.removeEventListener(t.window.document,"touchmove",this._onMove,{passive:!1}),n.removeEventListener(t.window.document,"touchend",this._onEnd);var r=this._gestureIntent,i=this._startScale;if(this._frameId&&(this._map._cancelRenderFrame(this._frameId),this._frameId=null),delete this._gestureIntent,delete this._startScale,delete this._startBearing,delete this._lastTouchEvent,r){this._map.fire(new t.Event(r+"end",{originalEvent:e})),this._drainInertiaBuffer();var o=this._inertia,a=this._map;if(o.length<2)a.snapToNorth({},{originalEvent:e});else{var s=o[o.length-1],A=o[0],l=a.transform.scaleZoom(i*s[1]),c=a.transform.scaleZoom(i*A[1]),u=l-c,g=(s[0]-A[0])/1e3,h=s[2];if(0!==g&&l!==c){var d=.15*u/g;Math.abs(d)>2.5&&(d=d>0?2.5:-2.5);var f=1e3*Math.abs(d/(12*.15)),p=l+d*f/2e3;p<0&&(p=0),a.easeTo({zoom:p,duration:f,easing:vr,around:this._aroundCenter?a.getCenter():a.unproject(h),noMoveStart:!0},{originalEvent:e})}else a.snapToNorth({},{originalEvent:e})}}},Br.prototype._drainInertiaBuffer=function(){for(var e=this._inertia,n=t.browser.now();e.length>2&&n-e[0][0]>160;)e.shift()};var br={scrollZoom:hr,boxZoom:dr,dragRotate:pr,dragPan:Cr,keyboard:mr,doubleClickZoom:Er,touchZoomRotate:Br},Qr=function(e){function n(n,r){e.call(this),this._moving=!1,this._zooming=!1,this.transform=n,this._bearingSnap=r.bearingSnap,t.bindAll(["_renderFrameCallback"],this)}return e&&(n.__proto__=e),n.prototype=Object.create(e&&e.prototype),n.prototype.constructor=n,n.prototype.getCenter=function(){return new t.LngLat(this.transform.center.lng,this.transform.center.lat)},n.prototype.setCenter=function(t,e){return this.jumpTo({center:t},e)},n.prototype.panBy=function(e,n,r){return e=t.Point.convert(e).mult(-1),this.panTo(this.transform.center,t.extend({offset:e},n),r)},n.prototype.panTo=function(e,n,r){return this.easeTo(t.extend({center:e},n),r)},n.prototype.getZoom=function(){return this.transform.zoom},n.prototype.setZoom=function(t,e){return this.jumpTo({zoom:t},e),this},n.prototype.zoomTo=function(e,n,r){return this.easeTo(t.extend({zoom:e},n),r)},n.prototype.zoomIn=function(t,e){return this.zoomTo(this.getZoom()+1,t,e),this},n.prototype.zoomOut=function(t,e){return this.zoomTo(this.getZoom()-1,t,e),this},n.prototype.getBearing=function(){return this.transform.bearing},n.prototype.setBearing=function(t,e){return this.jumpTo({bearing:t},e),this},n.prototype.rotateTo=function(e,n,r){return this.easeTo(t.extend({bearing:e},n),r)},n.prototype.resetNorth=function(e,n){return this.rotateTo(0,t.extend({duration:1e3},e),n),this},n.prototype.snapToNorth=function(t,e){return Math.abs(this.getBearing())<this._bearingSnap?this.resetNorth(t,e):this},n.prototype.getPitch=function(){return this.transform.pitch},n.prototype.setPitch=function(t,e){return this.jumpTo({pitch:t},e),this},n.prototype.cameraForBounds=function(e,n){return e=t.LngLatBounds.convert(e),this._cameraForBoxAndBearing(e.getNorthWest(),e.getSouthEast(),0,n)},n.prototype._cameraForBoxAndBearing=function(e,n,r,i){if("number"==typeof(i=t.extend({padding:{top:0,bottom:0,right:0,left:0},offset:[0,0],maxZoom:this.transform.maxZoom},i)).padding){var o=i.padding;i.padding={top:o,bottom:o,right:o,left:o}}if(t.deepEqual(Object.keys(i.padding).sort(function(t,e){return t<e?-1:t>e?1:0}),["bottom","left","right","top"])){var a=this.transform,s=a.project(t.LngLat.convert(e)),A=a.project(t.LngLat.convert(n)),l=s.rotate(-r*Math.PI/180),c=A.rotate(-r*Math.PI/180),u=new t.Point(Math.max(l.x,c.x),Math.max(l.y,c.y)),g=new t.Point(Math.min(l.x,c.x),Math.min(l.y,c.y)),h=u.sub(g),d=(a.width-i.padding.left-i.padding.right)/h.x,f=(a.height-i.padding.top-i.padding.bottom)/h.y;if(!(f<0||d<0)){var p=Math.min(a.scaleZoom(a.scale*Math.min(d,f)),i.maxZoom),I=t.Point.convert(i.offset),C=(i.padding.left-i.padding.right)/2,m=(i.padding.top-i.padding.bottom)/2,y=new t.Point(I.x+C,I.y+m).mult(a.scale/a.zoomScale(p));return{center:a.unproject(s.add(A).div(2).sub(y)),zoom:p,bearing:r}}t.warnOnce("Map cannot fit within canvas with the given bounds, padding, and/or offset.")}else t.warnOnce("options.padding must be a positive number, or an Object with keys 'bottom', 'left', 'right', 'top'")},n.prototype.fitBounds=function(t,e,n){return this._fitInternal(this.cameraForBounds(t,e),e,n)},n.prototype.fitScreenCoordinates=function(e,n,r,i,o){return this._fitInternal(this._cameraForBoxAndBearing(this.transform.pointLocation(t.Point.convert(e)),this.transform.pointLocation(t.Point.convert(n)),r,i),i,o)},n.prototype._fitInternal=function(e,n,r){return e?(n=t.extend(e,n)).linear?this.easeTo(n,r):this.flyTo(n,r):this},n.prototype.jumpTo=function(e,n){this.stop();var r=this.transform,i=!1,o=!1,a=!1;return"zoom"in e&&r.zoom!==+e.zoom&&(i=!0,r.zoom=+e.zoom),void 0!==e.center&&(r.center=t.LngLat.convert(e.center)),"bearing"in e&&r.bearing!==+e.bearing&&(o=!0,r.bearing=+e.bearing),"pitch"in e&&r.pitch!==+e.pitch&&(a=!0,r.pitch=+e.pitch),this.fire(new t.Event("movestart",n)).fire(new t.Event("move",n)),i&&this.fire(new t.Event("zoomstart",n)).fire(new t.Event("zoom",n)).fire(new t.Event("zoomend",n)),o&&this.fire(new t.Event("rotatestart",n)).fire(new t.Event("rotate",n)).fire(new t.Event("rotateend",n)),a&&this.fire(new t.Event("pitchstart",n)).fire(new t.Event("pitch",n)).fire(new t.Event("pitchend",n)),this.fire(new t.Event("moveend",n))},n.prototype.easeTo=function(e,n){var r=this;this.stop(),!1===(e=t.extend({offset:[0,0],duration:500,easing:t.ease},e)).animate&&(e.duration=0);var i=this.transform,o=this.getZoom(),a=this.getBearing(),s=this.getPitch(),A="zoom"in e?+e.zoom:o,l="bearing"in e?this._normalizeBearing(e.bearing,a):a,c="pitch"in e?+e.pitch:s,u=i.centerPoint.add(t.Point.convert(e.offset)),g=i.pointLocation(u),h=t.LngLat.convert(e.center||g);this._normalizeCenter(h);var d,f,p=i.project(g),I=i.project(h).sub(p),C=i.zoomScale(A-o);return e.around&&(d=t.LngLat.convert(e.around),f=i.locationPoint(d)),this._zooming=A!==o,this._rotating=a!==l,this._pitching=c!==s,this._prepareEase(n,e.noMoveStart),clearTimeout(this._easeEndTimeoutID),this._ease(function(e){if(r._zooming&&(i.zoom=t.number(o,A,e)),r._rotating&&(i.bearing=t.number(a,l,e)),r._pitching&&(i.pitch=t.number(s,c,e)),d)i.setLocationAtPoint(d,f);else{var g=i.zoomScale(i.zoom-o),h=A>o?Math.min(2,C):Math.max(.5,C),m=Math.pow(h,1-e),y=i.unproject(p.add(I.mult(e*m)).mult(g));i.setLocationAtPoint(i.renderWorldCopies?y.wrap():y,u)}r._fireMoveEvents(n)},function(){e.delayEndEvents?r._easeEndTimeoutID=setTimeout(function(){return r._afterEase(n)},e.delayEndEvents):r._afterEase(n)},e),this},n.prototype._prepareEase=function(e,n){this._moving=!0,n||this.fire(new t.Event("movestart",e)),this._zooming&&this.fire(new t.Event("zoomstart",e)),this._rotating&&this.fire(new t.Event("rotatestart",e)),this._pitching&&this.fire(new t.Event("pitchstart",e))},n.prototype._fireMoveEvents=function(e){this.fire(new t.Event("move",e)),this._zooming&&this.fire(new t.Event("zoom",e)),this._rotating&&this.fire(new t.Event("rotate",e)),this._pitching&&this.fire(new t.Event("pitch",e))},n.prototype._afterEase=function(e){var n=this._zooming,r=this._rotating,i=this._pitching;this._moving=!1,this._zooming=!1,this._rotating=!1,this._pitching=!1,n&&this.fire(new t.Event("zoomend",e)),r&&this.fire(new t.Event("rotateend",e)),i&&this.fire(new t.Event("pitchend",e)),this.fire(new t.Event("moveend",e))},n.prototype.flyTo=function(e,n){var r=this;this.stop(),e=t.extend({offset:[0,0],speed:1.2,curve:1.42,easing:t.ease},e);var i=this.transform,o=this.getZoom(),a=this.getBearing(),s=this.getPitch(),A="zoom"in e?t.clamp(+e.zoom,i.minZoom,i.maxZoom):o,l="bearing"in e?this._normalizeBearing(e.bearing,a):a,c="pitch"in e?+e.pitch:s,u=i.zoomScale(A-o),g=i.centerPoint.add(t.Point.convert(e.offset)),h=i.pointLocation(g),d=t.LngLat.convert(e.center||h);this._normalizeCenter(d);var f=i.project(h),p=i.project(d).sub(f),I=e.curve,C=Math.max(i.width,i.height),m=C/u,y=p.mag();if("minZoom"in e){var E=t.clamp(Math.min(e.minZoom,o,A),i.minZoom,i.maxZoom),v=C/i.zoomScale(E-o);I=Math.sqrt(v/y*2)}var B=I*I;function b(t){var e=(m*m-C*C+(t?-1:1)*B*B*y*y)/(2*(t?m:C)*B*y);return Math.log(Math.sqrt(e*e+1)-e)}function Q(t){return(Math.exp(t)-Math.exp(-t))/2}function w(t){return(Math.exp(t)+Math.exp(-t))/2}var x=b(0),_=function(t){return w(x)/w(x+I*t)},D=function(t){return C*((w(x)*(Q(e=x+I*t)/w(e))-Q(x))/B)/y;var e},M=(b(1)-x)/I;if(Math.abs(y)<1e-6||!isFinite(M)){if(Math.abs(C-m)<1e-6)return this.easeTo(e,n);var k=m<C?-1:1;M=Math.abs(Math.log(m/C))/I,D=function(){return 0},_=function(t){return Math.exp(k*I*t)}}if("duration"in e)e.duration=+e.duration;else{var S="screenSpeed"in e?+e.screenSpeed/I:+e.speed;e.duration=1e3*M/S}return e.maxDuration&&e.duration>e.maxDuration&&(e.duration=0),this._zooming=!0,this._rotating=a!==l,this._pitching=c!==s,this._prepareEase(n,!1),this._ease(function(e){var u=e*M,h=1/_(u);i.zoom=1===e?A:o+i.scaleZoom(h),r._rotating&&(i.bearing=t.number(a,l,e)),r._pitching&&(i.pitch=t.number(s,c,e));var I=1===e?d:i.unproject(f.add(p.mult(D(u))).mult(h));i.setLocationAtPoint(i.renderWorldCopies?I.wrap():I,g),r._fireMoveEvents(n)},function(){return r._afterEase(n)},e),this},n.prototype.isEasing=function(){return!!this._easeFrameId},n.prototype.stop=function(){if(this._easeFrameId&&(this._cancelRenderFrame(this._easeFrameId),delete this._easeFrameId,delete this._onEaseFrame),this._onEaseEnd){var t=this._onEaseEnd;delete this._onEaseEnd,t.call(this)}return this},n.prototype._ease=function(e,n,r){!1===r.animate||0===r.duration?(e(1),n()):(this._easeStart=t.browser.now(),this._easeOptions=r,this._onEaseFrame=e,this._onEaseEnd=n,this._easeFrameId=this._requestRenderFrame(this._renderFrameCallback))},n.prototype._renderFrameCallback=function(){var e=Math.min((t.browser.now()-this._easeStart)/this._easeOptions.duration,1);this._onEaseFrame(this._easeOptions.easing(e)),e<1?this._easeFrameId=this._requestRenderFrame(this._renderFrameCallback):this.stop()},n.prototype._normalizeBearing=function(e,n){e=t.wrap(e,-180,180);var r=Math.abs(e-n);return Math.abs(e-360-n)<r&&(e-=360),Math.abs(e+360-n)<r&&(e+=360),e},n.prototype._normalizeCenter=function(t){var e=this.transform;if(e.renderWorldCopies&&!e.lngRange){var n=t.lng-e.center.lng;t.lng+=n>180?-360:n<-180?360:0}},n}(t.Evented),wr=function(e){void 0===e&&(e={}),this.options=e,t.bindAll(["_updateEditLink","_updateData","_updateCompact"],this)};wr.prototype.getDefaultPosition=function(){return"bottom-right"},wr.prototype.onAdd=function(t){var e=this.options&&this.options.compact;return this._map=t,this._container=n.create("div","mapboxgl-ctrl mapboxgl-ctrl-attrib"),this._innerContainer=n.create("div","mapboxgl-ctrl-attrib-inner",this._container),e&&this._container.classList.add("mapboxgl-compact"),this._updateAttributions(),this._updateEditLink(),this._map.on("styledata",this._updateData),this._map.on("sourcedata",this._updateData),this._map.on("moveend",this._updateEditLink),void 0===e&&(this._map.on("resize",this._updateCompact),this._updateCompact()),this._container},wr.prototype.onRemove=function(){n.remove(this._container),this._map.off("styledata",this._updateData),this._map.off("sourcedata",this._updateData),this._map.off("moveend",this._updateEditLink),this._map.off("resize",this._updateCompact),this._map=void 0},wr.prototype._updateEditLink=function(){var e=this._editLink;e||(e=this._editLink=this._container.querySelector(".mapbox-improve-map"));var n=[{key:"owner",value:this.styleOwner},{key:"id",value:this.styleId},{key:"access_token",value:t.config.ACCESS_TOKEN}];if(e){var r=n.reduce(function(t,e,r){return e.value&&(t+=e.key+"="+e.value+(r<n.length-1?"&":"")),t},"?");e.href=t.config.FEEDBACK_URL+"/"+r+(this._map._hash?this._map._hash.getHashString(!0):""),e.rel="noopener nofollow"}},wr.prototype._updateData=function(t){!t||"metadata"!==t.sourceDataType&&"style"!==t.dataType||(this._updateAttributions(),this._updateEditLink())},wr.prototype._updateAttributions=function(){if(this._map.style){var t=[];if(this.options.customAttribution&&(Array.isArray(this.options.customAttribution)?t=t.concat(this.options.customAttribution.map(function(t){return"string"!=typeof t?"":t})):"string"==typeof this.options.customAttribution&&t.push(this.options.customAttribution)),this._map.style.stylesheet){var e=this._map.style.stylesheet;this.styleOwner=e.owner,this.styleId=e.id}var n=this._map.style.sourceCaches;for(var r in n){var i=n[r];if(i.used){var o=i.getSource();o.attribution&&t.indexOf(o.attribution)<0&&t.push(o.attribution)}}t.sort(function(t,e){return t.length-e.length});var a=(t=t.filter(function(e,n){for(var r=n+1;r<t.length;r++)if(t[r].indexOf(e)>=0)return!1;return!0})).join(" | ");a!==this._attribHTML&&(this._attribHTML=a,t.length?(this._innerContainer.innerHTML=a,this._container.classList.remove("mapboxgl-attrib-empty")):this._container.classList.add("mapboxgl-attrib-empty"),this._editLink=null)}},wr.prototype._updateCompact=function(){this._map.getCanvasContainer().offsetWidth<=640?this._container.classList.add("mapboxgl-compact"):this._container.classList.remove("mapboxgl-compact")};var xr=function(){t.bindAll(["_updateLogo"],this),t.bindAll(["_updateCompact"],this)};xr.prototype.onAdd=function(t){this._map=t,this._container=n.create("div","mapboxgl-ctrl");var e=n.create("a","mapboxgl-ctrl-logo");return e.target="_blank",e.rel="noopener nofollow",e.href="https://www.mapbox.com/",e.setAttribute("aria-label","Mapbox logo"),e.setAttribute("rel","noopener nofollow"),this._container.appendChild(e),this._container.style.display="none",this._map.on("sourcedata",this._updateLogo),this._updateLogo(),this._map.on("resize",this._updateCompact),this._updateCompact(),this._container},xr.prototype.onRemove=function(){n.remove(this._container),this._map.off("sourcedata",this._updateLogo),this._map.off("resize",this._updateCompact)},xr.prototype.getDefaultPosition=function(){return"bottom-left"},xr.prototype._updateLogo=function(t){t&&"metadata"!==t.sourceDataType||(this._container.style.display=this._logoRequired()?"block":"none")},xr.prototype._logoRequired=function(){if(this._map.style){var t=this._map.style.sourceCaches;for(var e in t)if(t[e].getSource().mapbox_logo)return!0;return!1}},xr.prototype._updateCompact=function(){var t=this._container.children;if(t.length){var e=t[0];this._map.getCanvasContainer().offsetWidth<250?e.classList.add("mapboxgl-compact"):e.classList.remove("mapboxgl-compact")}};var _r=function(){this._queue=[],this._id=0,this._cleared=!1,this._currentlyRunning=!1};_r.prototype.add=function(t){var e=++this._id;return this._queue.push({callback:t,id:e,cancelled:!1}),e},_r.prototype.remove=function(t){for(var e=this._currentlyRunning,n=0,r=e?this._queue.concat(e):this._queue;n<r.length;n+=1){var i=r[n];if(i.id===t)return void(i.cancelled=!0)}},_r.prototype.run=function(){var t=this._currentlyRunning=this._queue;this._queue=[];for(var e=0,n=t;e<n.length;e+=1){var r=n[e];if(!r.cancelled&&(r.callback(),this._cleared))break}this._cleared=!1,this._currentlyRunning=!1},_r.prototype.clear=function(){this._currentlyRunning&&(this._cleared=!0),this._queue=[]};var Dr=t.window.HTMLImageElement,Mr=t.window.HTMLElement,kr={center:[0,0],zoom:0,bearing:0,pitch:0,minZoom:0,maxZoom:22,interactive:!0,scrollZoom:!0,boxZoom:!0,dragRotate:!0,dragPan:!0,keyboard:!0,doubleClickZoom:!0,touchZoomRotate:!0,bearingSnap:7,clickTolerance:3,hash:!1,attributionControl:!0,failIfMajorPerformanceCaveat:!1,preserveDrawingBuffer:!1,trackResize:!0,renderWorldCopies:!0,refreshExpiredTiles:!0,maxTileCacheSize:null,localIdeographFontFamily:"sans-serif",transformRequest:null,fadeDuration:300,crossSourceCollisions:!0},Sr=function(r){function i(e){var i=this;if(null!=(e=t.extend({},kr,e)).minZoom&&null!=e.maxZoom&&e.minZoom>e.maxZoom)throw new Error("maxZoom must be greater than minZoom");var o=new sr(e.minZoom,e.maxZoom,e.renderWorldCopies);if(r.call(this,o,e),this._interactive=e.interactive,this._maxTileCacheSize=e.maxTileCacheSize,this._failIfMajorPerformanceCaveat=e.failIfMajorPerformanceCaveat,this._preserveDrawingBuffer=e.preserveDrawingBuffer,this._antialias=e.antialias,this._trackResize=e.trackResize,this._bearingSnap=e.bearingSnap,this._refreshExpiredTiles=e.refreshExpiredTiles,this._fadeDuration=e.fadeDuration,this._crossSourceCollisions=e.crossSourceCollisions,this._crossFadingFactor=1,this._collectResourceTiming=e.collectResourceTiming,this._renderTaskQueue=new _r,this._controls=[],this._mapId=t.uniqueId(),this._requestManager=new t.RequestManager(e.transformRequest),"string"==typeof e.container){if(this._container=t.window.document.getElementById(e.container),!this._container)throw new Error("Container '"+e.container+"' not found.")}else{if(!(e.container instanceof Mr))throw new Error("Invalid type: 'container' must be a String or HTMLElement.");this._container=e.container}if(e.maxBounds&&this.setMaxBounds(e.maxBounds),t.bindAll(["_onWindowOnline","_onWindowResize","_contextLost","_contextRestored"],this),this._setupContainer(),this._setupPainter(),void 0===this.painter)throw new Error("Failed to initialize WebGL.");this.on("move",function(){return i._update(!1)}),this.on("moveend",function(){return i._update(!1)}),this.on("zoom",function(){return i._update(!0)}),void 0!==t.window&&(t.window.addEventListener("online",this._onWindowOnline,!1),t.window.addEventListener("resize",this._onWindowResize,!1)),function(t,e){var r=t.getCanvasContainer(),i=null,o=!1,a=null;for(var s in br)t[s]=new br[s](t,e),e.interactive&&e[s]&&t[s].enable(e[s]);n.addEventListener(r,"mouseout",function(e){t.fire(new cr("mouseout",t,e))}),n.addEventListener(r,"mousedown",function(i){o=!0,a=n.mousePos(r,i);var s=new cr("mousedown",t,i);t.fire(s),s.defaultPrevented||(e.interactive&&!t.doubleClickZoom.isActive()&&t.stop(),t.boxZoom.onMouseDown(i),t.boxZoom.isActive()||t.dragPan.isActive()||t.dragRotate.onMouseDown(i),t.boxZoom.isActive()||t.dragRotate.isActive()||t.dragPan.onMouseDown(i))}),n.addEventListener(r,"mouseup",function(e){var n=t.dragRotate.isActive();i&&!n&&t.fire(new cr("contextmenu",t,i)),i=null,o=!1,t.fire(new cr("mouseup",t,e))}),n.addEventListener(r,"mousemove",function(e){if(!t.dragPan.isActive()&&!t.dragRotate.isActive()){for(var n=e.target;n&&n!==r;)n=n.parentNode;n===r&&t.fire(new cr("mousemove",t,e))}}),n.addEventListener(r,"mouseover",function(e){for(var n=e.target;n&&n!==r;)n=n.parentNode;n===r&&t.fire(new cr("mouseover",t,e))}),n.addEventListener(r,"touchstart",function(n){var r=new ur("touchstart",t,n);t.fire(r),r.defaultPrevented||(e.interactive&&t.stop(),t.boxZoom.isActive()||t.dragRotate.isActive()||t.dragPan.onTouchStart(n),t.touchZoomRotate.onStart(n),t.doubleClickZoom.onTouchStart(r))},{passive:!1}),n.addEventListener(r,"touchmove",function(e){t.fire(new ur("touchmove",t,e))},{passive:!1}),n.addEventListener(r,"touchend",function(e){t.fire(new ur("touchend",t,e))}),n.addEventListener(r,"touchcancel",function(e){t.fire(new ur("touchcancel",t,e))}),n.addEventListener(r,"click",function(i){var o=n.mousePos(r,i);(o.equals(a)||o.dist(a)<e.clickTolerance)&&t.fire(new cr("click",t,i))}),n.addEventListener(r,"dblclick",function(e){var n=new cr("dblclick",t,e);t.fire(n),n.defaultPrevented||t.doubleClickZoom.onDblClick(n)}),n.addEventListener(r,"contextmenu",function(e){var n=t.dragRotate.isActive();o||n?o&&(i=e):t.fire(new cr("contextmenu",t,e)),(t.dragRotate.isEnabled()||t.listens("contextmenu"))&&e.preventDefault()}),n.addEventListener(r,"wheel",function(n){e.interactive&&t.stop();var r=new gr("wheel",t,n);t.fire(r),r.defaultPrevented||t.scrollZoom.onWheel(n)},{passive:!1})}(this,e),this._hash=e.hash&&(new lr).addTo(this),this._hash&&this._hash._onHashChange()||(this.jumpTo({center:e.center,zoom:e.zoom,bearing:e.bearing,pitch:e.pitch}),e.bounds&&(this.resize(),this.fitBounds(e.bounds,t.extend({},e.fitBoundsOptions,{duration:0})))),this.resize(),this._localIdeographFontFamily=e.localIdeographFontFamily,e.style&&this.setStyle(e.style,{localIdeographFontFamily:e.localIdeographFontFamily}),e.attributionControl&&this.addControl(new wr({customAttribution:e.customAttribution})),this.addControl(new xr,e.logoPosition),this.on("style.load",function(){i.transform.unmodified&&i.jumpTo(i.style.stylesheet)}),this.on("data",function(e){i._update("style"===e.dataType),i.fire(new t.Event(e.dataType+"data",e))}),this.on("dataloading",function(e){i.fire(new t.Event(e.dataType+"dataloading",e))})}r&&(i.__proto__=r),i.prototype=Object.create(r&&r.prototype),i.prototype.constructor=i;var o={showTileBoundaries:{configurable:!0},showCollisionBoxes:{configurable:!0},showOverdrawInspector:{configurable:!0},repaint:{configurable:!0},vertices:{configurable:!0}};return i.prototype._getMapId=function(){return this._mapId},i.prototype.addControl=function(e,n){if(void 0===n&&e.getDefaultPosition&&(n=e.getDefaultPosition()),void 0===n&&(n="top-right"),!e||!e.onAdd)return this.fire(new t.ErrorEvent(new Error("Invalid argument to map.addControl(). Argument must be a control with onAdd and onRemove methods.")));var r=e.onAdd(this);this._controls.push(e);var i=this._controlPositions[n];return-1!==n.indexOf("bottom")?i.insertBefore(r,i.firstChild):i.appendChild(r),this},i.prototype.removeControl=function(e){if(!e||!e.onRemove)return this.fire(new t.ErrorEvent(new Error("Invalid argument to map.removeControl(). Argument must be a control with onAdd and onRemove methods.")));var n=this._controls.indexOf(e);return n>-1&&this._controls.splice(n,1),e.onRemove(this),this},i.prototype.resize=function(e){var n=this._containerDimensions(),r=n[0],i=n[1];return this._resizeCanvas(r,i),this.transform.resize(r,i),this.painter.resize(r,i),this.fire(new t.Event("movestart",e)).fire(new t.Event("move",e)).fire(new t.Event("resize",e)).fire(new t.Event("moveend",e)),this},i.prototype.getBounds=function(){return this.transform.getBounds()},i.prototype.getMaxBounds=function(){return this.transform.getMaxBounds()},i.prototype.setMaxBounds=function(e){return this.transform.setMaxBounds(t.LngLatBounds.convert(e)),this._update()},i.prototype.setMinZoom=function(t){if((t=null==t?0:t)>=0&&t<=this.transform.maxZoom)return this.transform.minZoom=t,this._update(),this.getZoom()<t&&this.setZoom(t),this;throw new Error("minZoom must be between 0 and the current maxZoom, inclusive")},i.prototype.getMinZoom=function(){return this.transform.minZoom},i.prototype.setMaxZoom=function(t){if((t=null==t?22:t)>=this.transform.minZoom)return this.transform.maxZoom=t,this._update(),this.getZoom()>t&&this.setZoom(t),this;throw new Error("maxZoom must be greater than the current minZoom")},i.prototype.getRenderWorldCopies=function(){return this.transform.renderWorldCopies},i.prototype.setRenderWorldCopies=function(t){return this.transform.renderWorldCopies=t,this._update()},i.prototype.getMaxZoom=function(){return this.transform.maxZoom},i.prototype.project=function(e){return this.transform.locationPoint(t.LngLat.convert(e))},i.prototype.unproject=function(e){return this.transform.pointLocation(t.Point.convert(e))},i.prototype.isMoving=function(){return this._moving||this.dragPan.isActive()||this.dragRotate.isActive()||this.scrollZoom.isActive()},i.prototype.isZooming=function(){return this._zooming||this.scrollZoom.isZooming()},i.prototype.isRotating=function(){return this._rotating||this.dragRotate.isActive()},i.prototype.on=function(t,e,n){var i=this;if(void 0===n)return r.prototype.on.call(this,t,e);var o=function(){var r;if("mouseenter"===t||"mouseover"===t){var o=!1;return{layer:e,listener:n,delegates:{mousemove:function(r){var a=i.getLayer(e)?i.queryRenderedFeatures(r.point,{layers:[e]}):[];a.length?o||(o=!0,n.call(i,new cr(t,i,r.originalEvent,{features:a}))):o=!1},mouseout:function(){o=!1}}}}if("mouseleave"===t||"mouseout"===t){var a=!1;return{layer:e,listener:n,delegates:{mousemove:function(r){(i.getLayer(e)?i.queryRenderedFeatures(r.point,{layers:[e]}):[]).length?a=!0:a&&(a=!1,n.call(i,new cr(t,i,r.originalEvent)))},mouseout:function(e){a&&(a=!1,n.call(i,new cr(t,i,e.originalEvent)))}}}}return{layer:e,listener:n,delegates:(r={},r[t]=function(t){var r=i.getLayer(e)?i.queryRenderedFeatures(t.point,{layers:[e]}):[];r.length&&(t.features=r,n.call(i,t),delete t.features)},r)}}();for(var a in this._delegatedListeners=this._delegatedListeners||{},this._delegatedListeners[t]=this._delegatedListeners[t]||[],this._delegatedListeners[t].push(o),o.delegates)this.on(a,o.delegates[a]);return this},i.prototype.off=function(t,e,n){if(void 0===n)return r.prototype.off.call(this,t,e);if(this._delegatedListeners&&this._delegatedListeners[t])for(var i=this._delegatedListeners[t],o=0;o<i.length;o++){var a=i[o];if(a.layer===e&&a.listener===n){for(var s in a.delegates)this.off(s,a.delegates[s]);return i.splice(o,1),this}}return this},i.prototype.queryRenderedFeatures=function(e,n){if(!this.style)return[];var r;if(void 0!==n||void 0===e||e instanceof t.Point||Array.isArray(e)||(n=e,e=void 0),n=n||{},(e=e||[[0,0],[this.transform.width,this.transform.height]])instanceof t.Point||"number"==typeof e[0])r=[t.Point.convert(e)];else{var i=t.Point.convert(e[0]),o=t.Point.convert(e[1]);r=[i,new t.Point(o.x,i.y),o,new t.Point(i.x,o.y),i]}return this.style.queryRenderedFeatures(r,n,this.transform)},i.prototype.querySourceFeatures=function(t,e){return this.style.querySourceFeatures(t,e)},i.prototype.setStyle=function(e,n){return!1!==(n=t.extend({},{localIdeographFontFamily:kr.localIdeographFontFamily},n)).diff&&n.localIdeographFontFamily===this._localIdeographFontFamily&&this.style&&e?(this._diffStyle(e,n),this):(this._localIdeographFontFamily=n.localIdeographFontFamily,this._updateStyle(e,n))},i.prototype._updateStyle=function(t,e){return this.style&&(this.style.setEventedParent(null),this.style._remove()),t?(this.style=new Ne(this,e||{}),this.style.setEventedParent(this,{style:this.style}),"string"==typeof t?this.style.loadURL(t):this.style.loadJSON(t),this):(delete this.style,this)},i.prototype._diffStyle=function(e,n){var r=this;if("string"==typeof e){var i=this._requestManager.normalizeStyleURL(e),o=this._requestManager.transformRequest(i,t.ResourceType.Style);t.getJSON(o,function(e,i){e?r.fire(new t.ErrorEvent(e)):i&&r._updateDiff(i,n)})}else"object"==typeof e&&this._updateDiff(e,n)},i.prototype._updateDiff=function(e,n){try{this.style.setState(e)&&this._update(!0)}catch(r){t.warnOnce("Unable to perform style diff: "+(r.message||r.error||r)+".  Rebuilding the style from scratch."),this._updateStyle(e,n)}},i.prototype.getStyle=function(){if(this.style)return this.style.serialize()},i.prototype.isStyleLoaded=function(){return this.style?this.style.loaded():t.warnOnce("There is no style added to the map.")},i.prototype.addSource=function(t,e){return this.style.addSource(t,e),this._update(!0)},i.prototype.isSourceLoaded=function(e){var n=this.style&&this.style.sourceCaches[e];if(void 0!==n)return n.loaded();this.fire(new t.ErrorEvent(new Error("There is no source with ID '"+e+"'")))},i.prototype.areTilesLoaded=function(){var t=this.style&&this.style.sourceCaches;for(var e in t){var n=t[e]._tiles;for(var r in n){var i=n[r];if("loaded"!==i.state&&"errored"!==i.state)return!1}}return!0},i.prototype.addSourceType=function(t,e,n){return this.style.addSourceType(t,e,n)},i.prototype.removeSource=function(t){return this.style.removeSource(t),this._update(!0)},i.prototype.getSource=function(t){return this.style.getSource(t)},i.prototype.addImage=function(e,n,r){void 0===r&&(r={});var i=r.pixelRatio;void 0===i&&(i=1);var o=r.sdf;if(void 0===o&&(o=!1),n instanceof Dr){var a=t.browser.getImageData(n),s=a.width,A=a.height,l=a.data;this.style.addImage(e,{data:new t.RGBAImage({width:s,height:A},l),pixelRatio:i,sdf:o,version:0})}else{if(void 0===n.width||void 0===n.height)return this.fire(new t.ErrorEvent(new Error("Invalid arguments to map.addImage(). The second argument must be an `HTMLImageElement`, `ImageData`, or object with `width`, `height`, and `data` properties with the same format as `ImageData`")));var c=n.width,u=n.height,g=n.data,h=n;this.style.addImage(e,{data:new t.RGBAImage({width:c,height:u},new Uint8Array(g)),pixelRatio:i,sdf:o,version:0,userImage:h}),h.onAdd&&h.onAdd(this,e)}},i.prototype.updateImage=function(e,n){var r=this.style.getImage(e);if(!r)return this.fire(new t.ErrorEvent(new Error("The map has no image with that id. If you are adding a new image use `map.addImage(...)` instead.")));var i=n instanceof Dr?t.browser.getImageData(n):n,o=i.width,a=i.height,s=i.data;if(void 0===o||void 0===a)return this.fire(new t.ErrorEvent(new Error("Invalid arguments to map.updateImage(). The second argument must be an `HTMLImageElement`, `ImageData`, or object with `width`, `height`, and `data` properties with the same format as `ImageData`")));if(o!==r.data.width||a!==r.data.height)return this.fire(new t.ErrorEvent(new Error("The width and height of the updated image must be that same as the previous version of the image")));var A=!(n instanceof Dr);r.data.replace(s,A),this.style.updateImage(e,r)},i.prototype.hasImage=function(e){return e?!!this.style.getImage(e):(this.fire(new t.ErrorEvent(new Error("Missing required image id"))),!1)},i.prototype.removeImage=function(t){this.style.removeImage(t)},i.prototype.loadImage=function(e,n){t.getImage(this._requestManager.transformRequest(e,t.ResourceType.Image),n)},i.prototype.listImages=function(){return this.style.listImages()},i.prototype.addLayer=function(t,e){return this.style.addLayer(t,e),this._update(!0)},i.prototype.moveLayer=function(t,e){return this.style.moveLayer(t,e),this._update(!0)},i.prototype.removeLayer=function(t){return this.style.removeLayer(t),this._update(!0)},i.prototype.getLayer=function(t){return this.style.getLayer(t)},i.prototype.setFilter=function(t,e,n){return void 0===n&&(n={}),this.style.setFilter(t,e,n),this._update(!0)},i.prototype.setLayerZoomRange=function(t,e,n){return this.style.setLayerZoomRange(t,e,n),this._update(!0)},i.prototype.getFilter=function(t){return this.style.getFilter(t)},i.prototype.setPaintProperty=function(t,e,n,r){return void 0===r&&(r={}),this.style.setPaintProperty(t,e,n,r),this._update(!0)},i.prototype.getPaintProperty=function(t,e){return this.style.getPaintProperty(t,e)},i.prototype.setLayoutProperty=function(t,e,n,r){return void 0===r&&(r={}),this.style.setLayoutProperty(t,e,n,r),this._update(!0)},i.prototype.getLayoutProperty=function(t,e){return this.style.getLayoutProperty(t,e)},i.prototype.setLight=function(t,e){return void 0===e&&(e={}),this.style.setLight(t,e),this._update(!0)},i.prototype.getLight=function(){return this.style.getLight()},i.prototype.setFeatureState=function(t,e){return this.style.setFeatureState(t,e),this._update()},i.prototype.removeFeatureState=function(t,e){return this.style.removeFeatureState(t,e),this._update()},i.prototype.getFeatureState=function(t){return this.style.getFeatureState(t)},i.prototype.getContainer=function(){return this._container},i.prototype.getCanvasContainer=function(){return this._canvasContainer},i.prototype.getCanvas=function(){return this._canvas},i.prototype._containerDimensions=function(){var t=0,e=0;return this._container&&(t=this._container.clientWidth||400,e=this._container.clientHeight||300),[t,e]},i.prototype._detectMissingCSS=function(){"rgb(250, 128, 114)"!==t.window.getComputedStyle(this._missingCSSCanary).getPropertyValue("background-color")&&t.warnOnce("This page appears to be missing CSS declarations for Mapbox GL JS, which may cause the map to display incorrectly. Please ensure your page includes mapbox-gl.css, as described in https://www.mapbox.com/mapbox-gl-js/api/.")},i.prototype._setupContainer=function(){var t=this._container;t.classList.add("mapboxgl-map"),(this._missingCSSCanary=n.create("div","mapboxgl-canary",t)).style.visibility="hidden",this._detectMissingCSS();var e=this._canvasContainer=n.create("div","mapboxgl-canvas-container",t);this._interactive&&e.classList.add("mapboxgl-interactive"),this._canvas=n.create("canvas","mapboxgl-canvas",e),this._canvas.style.position="absolute",this._canvas.addEventListener("webglcontextlost",this._contextLost,!1),this._canvas.addEventListener("webglcontextrestored",this._contextRestored,!1),this._canvas.setAttribute("tabindex","0"),this._canvas.setAttribute("aria-label","Map");var r=this._containerDimensions();this._resizeCanvas(r[0],r[1]);var i=this._controlContainer=n.create("div","mapboxgl-control-container",t),o=this._controlPositions={};["top-left","top-right","bottom-left","bottom-right"].forEach(function(t){o[t]=n.create("div","mapboxgl-ctrl-"+t,i)})},i.prototype._resizeCanvas=function(e,n){var r=t.window.devicePixelRatio||1;this._canvas.width=r*e,this._canvas.height=r*n,this._canvas.style.width=e+"px",this._canvas.style.height=n+"px"},i.prototype._setupPainter=function(){var n=t.extend({},e.webGLContextAttributes,{failIfMajorPerformanceCaveat:this._failIfMajorPerformanceCaveat,preserveDrawingBuffer:this._preserveDrawingBuffer,antialias:this._antialias||!1}),r=this._canvas.getContext("webgl",n)||this._canvas.getContext("experimental-webgl",n);r?(this.painter=new rr(r,this.transform),t.webpSupported.testSupport(r)):this.fire(new t.ErrorEvent(new Error("Failed to initialize WebGL")))},i.prototype._contextLost=function(e){e.preventDefault(),this._frame&&(this._frame.cancel(),this._frame=null),this.fire(new t.Event("webglcontextlost",{originalEvent:e}))},i.prototype._contextRestored=function(e){this._setupPainter(),this.resize(),this._update(),this.fire(new t.Event("webglcontextrestored",{originalEvent:e}))},i.prototype.loaded=function(){return!this._styleDirty&&!this._sourcesDirty&&!!this.style&&this.style.loaded()},i.prototype._update=function(t){return this.style?(this._styleDirty=this._styleDirty||t,this._sourcesDirty=!0,this.triggerRepaint(),this):this},i.prototype._requestRenderFrame=function(t){return this._update(),this._renderTaskQueue.add(t)},i.prototype._cancelRenderFrame=function(t){this._renderTaskQueue.remove(t)},i.prototype._render=function(){this.painter.context.setDirty(),this.painter.setBaseState(),this._renderTaskQueue.run();var e=!1;if(this.style&&this._styleDirty){this._styleDirty=!1;var n=this.transform.zoom,r=t.browser.now();this.style.zoomHistory.update(n,r);var i=new t.EvaluationParameters(n,{now:r,fadeDuration:this._fadeDuration,zoomHistory:this.style.zoomHistory,transition:this.style.getTransition()}),o=i.crossFadingFactor();1===o&&o===this._crossFadingFactor||(e=!0,this._crossFadingFactor=o),this.style.update(i)}return this.style&&this._sourcesDirty&&(this._sourcesDirty=!1,this.style._updateSources(this.transform)),this._placementDirty=this.style&&this.style._updatePlacement(this.painter.transform,this.showCollisionBoxes,this._fadeDuration,this._crossSourceCollisions),this.painter.render(this.style,{showTileBoundaries:this.showTileBoundaries,showOverdrawInspector:this._showOverdrawInspector,rotating:this.isRotating(),zooming:this.isZooming(),moving:this.isMoving(),fadeDuration:this._fadeDuration}),this.fire(new t.Event("render")),this.loaded()&&!this._loaded&&(this._loaded=!0,this.fire(new t.Event("load"))),this.style&&(this.style.hasTransitions()||e)&&(this._styleDirty=!0),this.style&&!this._placementDirty&&this.style._releaseSymbolFadeTiles(),this._sourcesDirty||this._repaint||this._styleDirty||this._placementDirty?this.triggerRepaint():!this.isMoving()&&this.loaded()&&this.fire(new t.Event("idle")),this},i.prototype.remove=function(){this._hash&&this._hash.remove();for(var e=0,n=this._controls;e<n.length;e+=1)n[e].onRemove(this);this._controls=[],this._frame&&(this._frame.cancel(),this._frame=null),this._renderTaskQueue.clear(),this.setStyle(null),void 0!==t.window&&(t.window.removeEventListener("resize",this._onWindowResize,!1),t.window.removeEventListener("online",this._onWindowOnline,!1));var r=this.painter.context.gl.getExtension("WEBGL_lose_context");r&&r.loseContext(),Lr(this._canvasContainer),Lr(this._controlContainer),Lr(this._missingCSSCanary),this._container.classList.remove("mapboxgl-map"),this.fire(new t.Event("remove"))},i.prototype.triggerRepaint=function(){var e=this;this.style&&!this._frame&&(this._frame=t.browser.frame(function(){e._frame=null,e._render()}))},i.prototype._onWindowOnline=function(){this._update()},i.prototype._onWindowResize=function(){this._trackResize&&this.resize()._update()},o.showTileBoundaries.get=function(){return!!this._showTileBoundaries},o.showTileBoundaries.set=function(t){this._showTileBoundaries!==t&&(this._showTileBoundaries=t,this._update())},o.showCollisionBoxes.get=function(){return!!this._showCollisionBoxes},o.showCollisionBoxes.set=function(t){this._showCollisionBoxes!==t&&(this._showCollisionBoxes=t,t?this.style._generateCollisionBoxes():this._update())},o.showOverdrawInspector.get=function(){return!!this._showOverdrawInspector},o.showOverdrawInspector.set=function(t){this._showOverdrawInspector!==t&&(this._showOverdrawInspector=t,this._update())},o.repaint.get=function(){return!!this._repaint},o.repaint.set=function(t){this._repaint!==t&&(this._repaint=t,this.triggerRepaint())},o.vertices.get=function(){return!!this._vertices},o.vertices.set=function(t){this._vertices=t,this._update()},i.prototype._setCacheLimits=function(e,n){t.setCacheLimits(e,n)},Object.defineProperties(i.prototype,o),i}(Qr);function Lr(t){t.parentNode&&t.parentNode.removeChild(t)}var Rr={showCompass:!0,showZoom:!0},Nr=function(e){var r=this;this.options=t.extend({},Rr,e),this._container=n.create("div","mapboxgl-ctrl mapboxgl-ctrl-group"),this._container.addEventListener("contextmenu",function(t){return t.preventDefault()}),this.options.showZoom&&(t.bindAll(["_updateZoomButtons"],this),this._zoomInButton=this._createButton("mapboxgl-ctrl-icon mapboxgl-ctrl-zoom-in","Zoom in",function(){return r._map.zoomIn()}),this._zoomOutButton=this._createButton("mapboxgl-ctrl-icon mapboxgl-ctrl-zoom-out","Zoom out",function(){return r._map.zoomOut()})),this.options.showCompass&&(t.bindAll(["_rotateCompassArrow"],this),this._compass=this._createButton("mapboxgl-ctrl-icon mapboxgl-ctrl-compass","Reset bearing to north",function(){return r._map.resetNorth()}),this._compassArrow=n.create("span","mapboxgl-ctrl-compass-arrow",this._compass))};function Fr(e,n,r){if(e=new t.LngLat(e.lng,e.lat),n){var i=new t.LngLat(e.lng-360,e.lat),o=new t.LngLat(e.lng+360,e.lat),a=r.locationPoint(e).distSqr(n);r.locationPoint(i).distSqr(n)<a?e=i:r.locationPoint(o).distSqr(n)<a&&(e=o)}for(;Math.abs(e.lng-r.center.lng)>180;){var s=r.locationPoint(e);if(s.x>=0&&s.y>=0&&s.x<=r.width&&s.y<=r.height)break;e.lng>r.center.lng?e.lng-=360:e.lng+=360}return e}Nr.prototype._updateZoomButtons=function(){var t=this._map.getZoom();this._zoomInButton.classList.toggle("mapboxgl-ctrl-icon-disabled",t===this._map.getMaxZoom()),this._zoomOutButton.classList.toggle("mapboxgl-ctrl-icon-disabled",t===this._map.getMinZoom())},Nr.prototype._rotateCompassArrow=function(){var t="rotate("+this._map.transform.angle*(180/Math.PI)+"deg)";this._compassArrow.style.transform=t},Nr.prototype.onAdd=function(t){return this._map=t,this.options.showZoom&&(this._map.on("zoom",this._updateZoomButtons),this._updateZoomButtons()),this.options.showCompass&&(this._map.on("rotate",this._rotateCompassArrow),this._rotateCompassArrow(),this._handler=new pr(t,{button:"left",element:this._compass}),n.addEventListener(this._compass,"mousedown",this._handler.onMouseDown),this._handler.enable()),this._container},Nr.prototype.onRemove=function(){n.remove(this._container),this.options.showZoom&&this._map.off("zoom",this._updateZoomButtons),this.options.showCompass&&(this._map.off("rotate",this._rotateCompassArrow),n.removeEventListener(this._compass,"mousedown",this._handler.onMouseDown),this._handler.disable(),delete this._handler),delete this._map},Nr.prototype._createButton=function(t,e,r){var i=n.create("button",t,this._container);return i.type="button",i.title=e,i.setAttribute("aria-label",e),i.addEventListener("click",r),i};var Tr={center:"translate(-50%,-50%)",top:"translate(-50%,0)","top-left":"translate(0,0)","top-right":"translate(-100%,0)",bottom:"translate(-50%,-100%)","bottom-left":"translate(0,-100%)","bottom-right":"translate(-100%,-100%)",left:"translate(0,-50%)",right:"translate(-100%,-50%)"};function Ur(t,e,n){var r=t.classList;for(var i in Tr)r.remove("mapboxgl-"+n+"-anchor-"+i);r.add("mapboxgl-"+n+"-anchor-"+e)}var Or,Gr=function(e){function r(r,i){if(e.call(this),(r instanceof t.window.HTMLElement||i)&&(r=t.extend({element:r},i)),t.bindAll(["_update","_onMove","_onUp","_addDragHandler","_onMapClick"],this),this._anchor=r&&r.anchor||"center",this._color=r&&r.color||"#3FB1CE",this._draggable=r&&r.draggable||!1,this._state="inactive",r&&r.element)this._element=r.element,this._offset=t.Point.convert(r&&r.offset||[0,0]);else{this._defaultMarker=!0,this._element=n.create("div");var o=n.createNS("http://www.w3.org/2000/svg","svg");o.setAttributeNS(null,"display","block"),o.setAttributeNS(null,"height","41px"),o.setAttributeNS(null,"width","27px"),o.setAttributeNS(null,"viewBox","0 0 27 41");var a=n.createNS("http://www.w3.org/2000/svg","g");a.setAttributeNS(null,"stroke","none"),a.setAttributeNS(null,"stroke-width","1"),a.setAttributeNS(null,"fill","none"),a.setAttributeNS(null,"fill-rule","evenodd");var s=n.createNS("http://www.w3.org/2000/svg","g");s.setAttributeNS(null,"fill-rule","nonzero");var A=n.createNS("http://www.w3.org/2000/svg","g");A.setAttributeNS(null,"transform","translate(3.0, 29.0)"),A.setAttributeNS(null,"fill","#000000");for(var l=0,c=[{rx:"10.5",ry:"5.25002273"},{rx:"10.5",ry:"5.25002273"},{rx:"9.5",ry:"4.77275007"},{rx:"8.5",ry:"4.29549936"},{rx:"7.5",ry:"3.81822308"},{rx:"6.5",ry:"3.34094679"},{rx:"5.5",ry:"2.86367051"},{rx:"4.5",ry:"2.38636864"}];l<c.length;l+=1){var u=c[l],g=n.createNS("http://www.w3.org/2000/svg","ellipse");g.setAttributeNS(null,"opacity","0.04"),g.setAttributeNS(null,"cx","10.5"),g.setAttributeNS(null,"cy","5.80029008"),g.setAttributeNS(null,"rx",u.rx),g.setAttributeNS(null,"ry",u.ry),A.appendChild(g)}var h=n.createNS("http://www.w3.org/2000/svg","g");h.setAttributeNS(null,"fill",this._color);var d=n.createNS("http://www.w3.org/2000/svg","path");d.setAttributeNS(null,"d","M27,13.5 C27,19.074644 20.250001,27.000002 14.75,34.500002 C14.016665,35.500004 12.983335,35.500004 12.25,34.500002 C6.7499993,27.000002 0,19.222562 0,13.5 C0,6.0441559 6.0441559,0 13.5,0 C20.955844,0 27,6.0441559 27,13.5 Z"),h.appendChild(d);var f=n.createNS("http://www.w3.org/2000/svg","g");f.setAttributeNS(null,"opacity","0.25"),f.setAttributeNS(null,"fill","#000000");var p=n.createNS("http://www.w3.org/2000/svg","path");p.setAttributeNS(null,"d","M13.5,0 C6.0441559,0 0,6.0441559 0,13.5 C0,19.222562 6.7499993,27 12.25,34.5 C13,35.522727 14.016664,35.500004 14.75,34.5 C20.250001,27 27,19.074644 27,13.5 C27,6.0441559 20.955844,0 13.5,0 Z M13.5,1 C20.415404,1 26,6.584596 26,13.5 C26,15.898657 24.495584,19.181431 22.220703,22.738281 C19.945823,26.295132 16.705119,30.142167 13.943359,33.908203 C13.743445,34.180814 13.612715,34.322738 13.5,34.441406 C13.387285,34.322738 13.256555,34.180814 13.056641,33.908203 C10.284481,30.127985 7.4148684,26.314159 5.015625,22.773438 C2.6163816,19.232715 1,15.953538 1,13.5 C1,6.584596 6.584596,1 13.5,1 Z"),f.appendChild(p);var I=n.createNS("http://www.w3.org/2000/svg","g");I.setAttributeNS(null,"transform","translate(6.0, 7.0)"),I.setAttributeNS(null,"fill","#FFFFFF");var C=n.createNS("http://www.w3.org/2000/svg","g");C.setAttributeNS(null,"transform","translate(8.0, 8.0)");var m=n.createNS("http://www.w3.org/2000/svg","circle");m.setAttributeNS(null,"fill","#000000"),m.setAttributeNS(null,"opacity","0.25"),m.setAttributeNS(null,"cx","5.5"),m.setAttributeNS(null,"cy","5.5"),m.setAttributeNS(null,"r","5.4999962");var y=n.createNS("http://www.w3.org/2000/svg","circle");y.setAttributeNS(null,"fill","#FFFFFF"),y.setAttributeNS(null,"cx","5.5"),y.setAttributeNS(null,"cy","5.5"),y.setAttributeNS(null,"r","5.4999962"),C.appendChild(m),C.appendChild(y),s.appendChild(A),s.appendChild(h),s.appendChild(f),s.appendChild(I),s.appendChild(C),o.appendChild(s),this._element.appendChild(o),this._offset=t.Point.convert(r&&r.offset||[0,-14])}this._element.classList.add("mapboxgl-marker"),this._element.addEventListener("dragstart",function(t){t.preventDefault()}),Ur(this._element,this._anchor,"marker"),this._popup=null}return e&&(r.__proto__=e),r.prototype=Object.create(e&&e.prototype),r.prototype.constructor=r,r.prototype.addTo=function(t){return this.remove(),this._map=t,t.getCanvasContainer().appendChild(this._element),t.on("move",this._update),t.on("moveend",this._update),this.setDraggable(this._draggable),this._update(),this._map.on("click",this._onMapClick),this},r.prototype.remove=function(){return this._map&&(this._map.off("click",this._onMapClick),this._map.off("move",this._update),this._map.off("moveend",this._update),this._map.off("mousedown",this._addDragHandler),this._map.off("touchstart",this._addDragHandler),this._map.off("mouseup",this._onUp),this._map.off("touchend",this._onUp),delete this._map),n.remove(this._element),this._popup&&this._popup.remove(),this},r.prototype.getLngLat=function(){return this._lngLat},r.prototype.setLngLat=function(e){return this._lngLat=t.LngLat.convert(e),this._pos=null,this._popup&&this._popup.setLngLat(this._lngLat),this._update(),this},r.prototype.getElement=function(){return this._element},r.prototype.setPopup=function(t){if(this._popup&&(this._popup.remove(),this._popup=null),t){if(!("offset"in t.options)){var e=Math.sqrt(Math.pow(13.5,2)/2);t.options.offset=this._defaultMarker?{top:[0,0],"top-left":[0,0],"top-right":[0,0],bottom:[0,-38.1],"bottom-left":[e,-1*(24.6+e)],"bottom-right":[-e,-1*(24.6+e)],left:[13.5,-24.6],right:[-13.5,-24.6]}:this._offset}this._popup=t,this._lngLat&&this._popup.setLngLat(this._lngLat)}return this},r.prototype._onMapClick=function(t){var e=t.originalEvent.target,n=this._element;this._popup&&(e===n||n.contains(e))&&this.togglePopup()},r.prototype.getPopup=function(){return this._popup},r.prototype.togglePopup=function(){var t=this._popup;return t?(t.isOpen()?t.remove():t.addTo(this._map),this):this},r.prototype._update=function(t){this._map&&(this._map.transform.renderWorldCopies&&(this._lngLat=Fr(this._lngLat,this._pos,this._map.transform)),this._pos=this._map.project(this._lngLat)._add(this._offset),t&&"moveend"!==t.type||(this._pos=this._pos.round()),n.setTransform(this._element,Tr[this._anchor]+" translate("+this._pos.x+"px, "+this._pos.y+"px)"))},r.prototype.getOffset=function(){return this._offset},r.prototype.setOffset=function(e){return this._offset=t.Point.convert(e),this._update(),this},r.prototype._onMove=function(e){this._pos=e.point.sub(this._positionDelta),this._lngLat=this._map.unproject(this._pos),this.setLngLat(this._lngLat),this._element.style.pointerEvents="none","pending"===this._state&&(this._state="active",this.fire(new t.Event("dragstart"))),this.fire(new t.Event("drag"))},r.prototype._onUp=function(){this._element.style.pointerEvents="auto",this._positionDelta=null,this._map.off("mousemove",this._onMove),this._map.off("touchmove",this._onMove),"active"===this._state&&this.fire(new t.Event("dragend")),this._state="inactive"},r.prototype._addDragHandler=function(t){this._element.contains(t.originalEvent.target)&&(t.preventDefault(),this._positionDelta=t.point.sub(this._pos).add(this._offset),this._state="pending",this._map.on("mousemove",this._onMove),this._map.on("touchmove",this._onMove),this._map.once("mouseup",this._onUp),this._map.once("touchend",this._onUp))},r.prototype.setDraggable=function(t){return this._draggable=!!t,this._map&&(t?(this._map.on("mousedown",this._addDragHandler),this._map.on("touchstart",this._addDragHandler)):(this._map.off("mousedown",this._addDragHandler),this._map.off("touchstart",this._addDragHandler))),this},r.prototype.isDraggable=function(){return this._draggable},r}(t.Evented),Hr={positionOptions:{enableHighAccuracy:!1,maximumAge:0,timeout:6e3},fitBoundsOptions:{maxZoom:15},trackUserLocation:!1,showUserLocation:!0},jr=function(e){function r(n){e.call(this),this.options=t.extend({},Hr,n),t.bindAll(["_onSuccess","_onError","_finish","_setupUI","_updateCamera","_updateMarker"],this)}return e&&(r.__proto__=e),r.prototype=Object.create(e&&e.prototype),r.prototype.constructor=r,r.prototype.onAdd=function(e){var r;return this._map=e,this._container=n.create("div","mapboxgl-ctrl mapboxgl-ctrl-group"),r=this._setupUI,void 0!==Or?r(Or):void 0!==t.window.navigator.permissions?t.window.navigator.permissions.query({name:"geolocation"}).then(function(t){Or="denied"!==t.state,r(Or)}):(Or=!!t.window.navigator.geolocation,r(Or)),this._container},r.prototype.onRemove=function(){void 0!==this._geolocationWatchID&&(t.window.navigator.geolocation.clearWatch(this._geolocationWatchID),this._geolocationWatchID=void 0),this.options.showUserLocation&&this._userLocationDotMarker&&this._userLocationDotMarker.remove(),n.remove(this._container),this._map=void 0},r.prototype._onSuccess=function(e){if(this.options.trackUserLocation)switch(this._lastKnownPosition=e,this._watchState){case"WAITING_ACTIVE":case"ACTIVE_LOCK":case"ACTIVE_ERROR":this._watchState="ACTIVE_LOCK",this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-waiting"),this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active-error"),this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-active");break;case"BACKGROUND":case"BACKGROUND_ERROR":this._watchState="BACKGROUND",this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-waiting"),this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background-error"),this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-background")}this.options.showUserLocation&&"OFF"!==this._watchState&&this._updateMarker(e),this.options.trackUserLocation&&"ACTIVE_LOCK"!==this._watchState||this._updateCamera(e),this.options.showUserLocation&&this._dotElement.classList.remove("mapboxgl-user-location-dot-stale"),this.fire(new t.Event("geolocate",e)),this._finish()},r.prototype._updateCamera=function(e){var n=new t.LngLat(e.coords.longitude,e.coords.latitude),r=e.coords.accuracy,i=this._map.getBearing(),o=t.extend({bearing:i},this.options.fitBoundsOptions);this._map.fitBounds(n.toBounds(r),o,{geolocateSource:!0})},r.prototype._updateMarker=function(t){t?this._userLocationDotMarker.setLngLat([t.coords.longitude,t.coords.latitude]).addTo(this._map):this._userLocationDotMarker.remove()},r.prototype._onError=function(e){if(this.options.trackUserLocation)if(1===e.code)this._watchState="OFF",this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-waiting"),this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active"),this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active-error"),this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background"),this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background-error"),void 0!==this._geolocationWatchID&&this._clearWatch();else switch(this._watchState){case"WAITING_ACTIVE":this._watchState="ACTIVE_ERROR",this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active"),this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-active-error");break;case"ACTIVE_LOCK":this._watchState="ACTIVE_ERROR",this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active"),this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-active-error"),this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-waiting");break;case"BACKGROUND":this._watchState="BACKGROUND_ERROR",this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background"),this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-background-error"),this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-waiting")}"OFF"!==this._watchState&&this.options.showUserLocation&&this._dotElement.classList.add("mapboxgl-user-location-dot-stale"),this.fire(new t.Event("error",e)),this._finish()},r.prototype._finish=function(){this._timeoutId&&clearTimeout(this._timeoutId),this._timeoutId=void 0},r.prototype._setupUI=function(e){var r=this;!1!==e?(this._container.addEventListener("contextmenu",function(t){return t.preventDefault()}),this._geolocateButton=n.create("button","mapboxgl-ctrl-icon mapboxgl-ctrl-geolocate",this._container),this._geolocateButton.type="button",this._geolocateButton.setAttribute("aria-label","Geolocate"),this.options.trackUserLocation&&(this._geolocateButton.setAttribute("aria-pressed","false"),this._watchState="OFF"),this.options.showUserLocation&&(this._dotElement=n.create("div","mapboxgl-user-location-dot"),this._userLocationDotMarker=new Gr(this._dotElement),this.options.trackUserLocation&&(this._watchState="OFF")),this._geolocateButton.addEventListener("click",this.trigger.bind(this)),this._setup=!0,this.options.trackUserLocation&&this._map.on("movestart",function(e){e.geolocateSource||"ACTIVE_LOCK"!==r._watchState||(r._watchState="BACKGROUND",r._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-background"),r._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active"),r.fire(new t.Event("trackuserlocationend")))})):t.warnOnce("Geolocation support is not available, the GeolocateControl will not be visible.")},r.prototype.trigger=function(){if(!this._setup)return t.warnOnce("Geolocate control triggered before added to a map"),!1;if(this.options.trackUserLocation){switch(this._watchState){case"OFF":this._watchState="WAITING_ACTIVE",this.fire(new t.Event("trackuserlocationstart"));break;case"WAITING_ACTIVE":case"ACTIVE_LOCK":case"ACTIVE_ERROR":case"BACKGROUND_ERROR":this._watchState="OFF",this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-waiting"),this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active"),this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active-error"),this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background"),this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background-error"),this.fire(new t.Event("trackuserlocationend"));break;case"BACKGROUND":this._watchState="ACTIVE_LOCK",this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background"),this._lastKnownPosition&&this._updateCamera(this._lastKnownPosition),this.fire(new t.Event("trackuserlocationstart"))}switch(this._watchState){case"WAITING_ACTIVE":this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-waiting"),this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-active");break;case"ACTIVE_LOCK":this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-active");break;case"ACTIVE_ERROR":this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-waiting"),this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-active-error");break;case"BACKGROUND":this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-background");break;case"BACKGROUND_ERROR":this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-waiting"),this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-background-error")}"OFF"===this._watchState&&void 0!==this._geolocationWatchID?this._clearWatch():void 0===this._geolocationWatchID&&(this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-waiting"),this._geolocateButton.setAttribute("aria-pressed","true"),this._geolocationWatchID=t.window.navigator.geolocation.watchPosition(this._onSuccess,this._onError,this.options.positionOptions))}else t.window.navigator.geolocation.getCurrentPosition(this._onSuccess,this._onError,this.options.positionOptions),this._timeoutId=setTimeout(this._finish,1e4);return!0},r.prototype._clearWatch=function(){t.window.navigator.geolocation.clearWatch(this._geolocationWatchID),this._geolocationWatchID=void 0,this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-waiting"),this._geolocateButton.setAttribute("aria-pressed","false"),this.options.showUserLocation&&this._updateMarker(null)},r}(t.Evented),Yr={maxWidth:100,unit:"metric"},Pr=function(e){this.options=t.extend({},Yr,e),t.bindAll(["_onMove","setUnit"],this)};function Kr(t,e,n){var r,i,o,a,s,A,l=n&&n.maxWidth||100,c=t._container.clientHeight/2,u=(r=t.unproject([0,c]),i=t.unproject([l,c]),o=Math.PI/180,a=r.lat*o,s=i.lat*o,A=Math.sin(a)*Math.sin(s)+Math.cos(a)*Math.cos(s)*Math.cos((i.lng-r.lng)*o),6371e3*Math.acos(Math.min(A,1)));if(n&&"imperial"===n.unit){var g=3.2808*u;g>5280?zr(e,l,g/5280,"mi"):zr(e,l,g,"ft")}else n&&"nautical"===n.unit?zr(e,l,u/1852,"nm"):zr(e,l,u,"m")}function zr(t,e,n,r){var i,o,a,s=(i=n,(o=Math.pow(10,(""+Math.floor(i)).length-1))*(a=(a=i/o)>=10?10:a>=5?5:a>=3?3:a>=2?2:a>=1?1:function(t){var e=Math.pow(10,Math.ceil(-Math.log(t)/Math.LN10));return Math.round(t*e)/e}(a))),A=s/n;"m"===r&&s>=1e3&&(s/=1e3,r="km"),t.style.width=e*A+"px",t.innerHTML=s+r}Pr.prototype.getDefaultPosition=function(){return"bottom-left"},Pr.prototype._onMove=function(){Kr(this._map,this._container,this.options)},Pr.prototype.onAdd=function(t){return this._map=t,this._container=n.create("div","mapboxgl-ctrl mapboxgl-ctrl-scale",t.getContainer()),this._map.on("move",this._onMove),this._onMove(),this._container},Pr.prototype.onRemove=function(){n.remove(this._container),this._map.off("move",this._onMove),this._map=void 0},Pr.prototype.setUnit=function(t){this.options.unit=t,Kr(this._map,this._container,this.options)};var Jr=function(e){this._fullscreen=!1,e&&e.container&&(e.container instanceof t.window.HTMLElement?this._container=e.container:t.warnOnce("Full screen control 'container' must be a DOM element.")),t.bindAll(["_onClickFullscreen","_changeIcon"],this),"onfullscreenchange"in t.window.document?this._fullscreenchange="fullscreenchange":"onmozfullscreenchange"in t.window.document?this._fullscreenchange="mozfullscreenchange":"onwebkitfullscreenchange"in t.window.document?this._fullscreenchange="webkitfullscreenchange":"onmsfullscreenchange"in t.window.document&&(this._fullscreenchange="MSFullscreenChange"),this._className="mapboxgl-ctrl"};Jr.prototype.onAdd=function(e){return this._map=e,this._container||(this._container=this._map.getContainer()),this._controlContainer=n.create("div",this._className+" mapboxgl-ctrl-group"),this._checkFullscreenSupport()?this._setupUI():(this._controlContainer.style.display="none",t.warnOnce("This device does not support fullscreen mode.")),this._controlContainer},Jr.prototype.onRemove=function(){n.remove(this._controlContainer),this._map=null,t.window.document.removeEventListener(this._fullscreenchange,this._changeIcon)},Jr.prototype._checkFullscreenSupport=function(){return!!(t.window.document.fullscreenEnabled||t.window.document.mozFullScreenEnabled||t.window.document.msFullscreenEnabled||t.window.document.webkitFullscreenEnabled)},Jr.prototype._setupUI=function(){(this._fullscreenButton=n.create("button",this._className+"-icon "+this._className+"-fullscreen",this._controlContainer)).type="button",this._updateTitle(),this._fullscreenButton.addEventListener("click",this._onClickFullscreen),t.window.document.addEventListener(this._fullscreenchange,this._changeIcon)},Jr.prototype._updateTitle=function(){var t=this._isFullscreen()?"Exit fullscreen":"Enter fullscreen";this._fullscreenButton.setAttribute("aria-label",t),this._fullscreenButton.title=t},Jr.prototype._isFullscreen=function(){return this._fullscreen},Jr.prototype._changeIcon=function(){(t.window.document.fullscreenElement||t.window.document.mozFullScreenElement||t.window.document.webkitFullscreenElement||t.window.document.msFullscreenElement)===this._container!==this._fullscreen&&(this._fullscreen=!this._fullscreen,this._fullscreenButton.classList.toggle(this._className+"-shrink"),this._fullscreenButton.classList.toggle(this._className+"-fullscreen"),this._updateTitle())},Jr.prototype._onClickFullscreen=function(){this._isFullscreen()?t.window.document.exitFullscreen?t.window.document.exitFullscreen():t.window.document.mozCancelFullScreen?t.window.document.mozCancelFullScreen():t.window.document.msExitFullscreen?t.window.document.msExitFullscreen():t.window.document.webkitCancelFullScreen&&t.window.document.webkitCancelFullScreen():this._container.requestFullscreen?this._container.requestFullscreen():this._container.mozRequestFullScreen?this._container.mozRequestFullScreen():this._container.msRequestFullscreen?this._container.msRequestFullscreen():this._container.webkitRequestFullscreen&&this._container.webkitRequestFullscreen()};var qr={closeButton:!0,closeOnClick:!0,className:"",maxWidth:"240px"},Vr=function(e){function r(n){e.call(this),this.options=t.extend(Object.create(qr),n),t.bindAll(["_update","_onClickClose","remove"],this)}return e&&(r.__proto__=e),r.prototype=Object.create(e&&e.prototype),r.prototype.constructor=r,r.prototype.addTo=function(e){var n=this;return this._map=e,this.options.closeOnClick&&this._map.on("click",this._onClickClose),this._map.on("remove",this.remove),this._update(),this._trackPointer?(this._map.on("mousemove",function(t){n._update(t.point)}),this._map.on("mouseup",function(t){n._update(t.point)}),this._container.classList.add("mapboxgl-popup-track-pointer"),this._map._canvasContainer.classList.add("mapboxgl-track-pointer")):this._map.on("move",this._update),this.fire(new t.Event("open")),this},r.prototype.isOpen=function(){return!!this._map},r.prototype.remove=function(){return this._content&&n.remove(this._content),this._container&&(n.remove(this._container),delete this._container),this._map&&(this._map.off("move",this._update),this._map.off("click",this._onClickClose),this._map.off("remove",this.remove),this._map.off("mousemove"),delete this._map),this.fire(new t.Event("close")),this},r.prototype.getLngLat=function(){return this._lngLat},r.prototype.setLngLat=function(e){return this._lngLat=t.LngLat.convert(e),this._pos=null,this._map&&(this._map.on("move",this._update),this._map.off("mousemove"),this._container.classList.remove("mapboxgl-popup-track-pointer"),this._map._canvasContainer.classList.remove("mapboxgl-track-pointer")),this._trackPointer=!1,this._update(),this},r.prototype.trackPointer=function(){var t=this;return this._trackPointer=!0,this._pos=null,this._map&&(this._map.off("move",this._update),this._map.on("mousemove",function(e){t._update(e.point)}),this._map.on("drag",function(e){t._update(e.point)}),this._container.classList.add("mapboxgl-popup-track-pointer"),this._map._canvasContainer.classList.add("mapboxgl-track-pointer")),this},r.prototype.getElement=function(){return this._container},r.prototype.setText=function(e){return this.setDOMContent(t.window.document.createTextNode(e))},r.prototype.setHTML=function(e){var n,r=t.window.document.createDocumentFragment(),i=t.window.document.createElement("body");for(i.innerHTML=e;n=i.firstChild;)r.appendChild(n);return this.setDOMContent(r)},r.prototype.getMaxWidth=function(){return this._container.style.maxWidth},r.prototype.setMaxWidth=function(t){return this.options.maxWidth=t,this._update(),this},r.prototype.setDOMContent=function(t){return this._createContent(),this._content.appendChild(t),this._update(),this},r.prototype._createContent=function(){this._content&&n.remove(this._content),this._content=n.create("div","mapboxgl-popup-content",this._container),this.options.closeButton&&(this._closeButton=n.create("button","mapboxgl-popup-close-button",this._content),this._closeButton.type="button",this._closeButton.setAttribute("aria-label","Close popup"),this._closeButton.innerHTML="&#215;",this._closeButton.addEventListener("click",this._onClickClose))},r.prototype._update=function(e){var r=this,i=this._lngLat||this._trackPointer;if(this._map&&i&&this._content&&(this._container||(this._container=n.create("div","mapboxgl-popup",this._map.getContainer()),this._tip=n.create("div","mapboxgl-popup-tip",this._container),this._container.appendChild(this._content),this.options.className&&this.options.className.split(" ").forEach(function(t){return r._container.classList.add(t)})),this.options.maxWidth&&this._container.style.maxWidth!==this.options.maxWidth&&(this._container.style.maxWidth=this.options.maxWidth),this._map.transform.renderWorldCopies&&!this._trackPointer&&(this._lngLat=Fr(this._lngLat,this._pos,this._map.transform)),!this._trackPointer||e)){var o=this._pos=this._trackPointer&&e?e:this._map.project(this._lngLat),a=this.options.anchor,s=function e(n){if(n){if("number"==typeof n){var r=Math.round(Math.sqrt(.5*Math.pow(n,2)));return{center:new t.Point(0,0),top:new t.Point(0,n),"top-left":new t.Point(r,r),"top-right":new t.Point(-r,r),bottom:new t.Point(0,-n),"bottom-left":new t.Point(r,-r),"bottom-right":new t.Point(-r,-r),left:new t.Point(n,0),right:new t.Point(-n,0)}}if(n instanceof t.Point||Array.isArray(n)){var i=t.Point.convert(n);return{center:i,top:i,"top-left":i,"top-right":i,bottom:i,"bottom-left":i,"bottom-right":i,left:i,right:i}}return{center:t.Point.convert(n.center||[0,0]),top:t.Point.convert(n.top||[0,0]),"top-left":t.Point.convert(n["top-left"]||[0,0]),"top-right":t.Point.convert(n["top-right"]||[0,0]),bottom:t.Point.convert(n.bottom||[0,0]),"bottom-left":t.Point.convert(n["bottom-left"]||[0,0]),"bottom-right":t.Point.convert(n["bottom-right"]||[0,0]),left:t.Point.convert(n.left||[0,0]),right:t.Point.convert(n.right||[0,0])}}return e(new t.Point(0,0))}(this.options.offset);if(!a){var A,l=this._container.offsetWidth,c=this._container.offsetHeight;A=o.y+s.bottom.y<c?["top"]:o.y>this._map.transform.height-c?["bottom"]:[],o.x<l/2?A.push("left"):o.x>this._map.transform.width-l/2&&A.push("right"),a=0===A.length?"bottom":A.join("-")}var u=o.add(s[a]).round();n.setTransform(this._container,Tr[a]+" translate("+u.x+"px,"+u.y+"px)"),Ur(this._container,a,"popup")}},r.prototype._onClickClose=function(){this.remove()},r}(t.Evented),Wr={version:t.version,supported:e,setRTLTextPlugin:t.setRTLTextPlugin,Map:Sr,NavigationControl:Nr,GeolocateControl:jr,AttributionControl:wr,ScaleControl:Pr,FullscreenControl:Jr,Popup:Vr,Marker:Gr,Style:Ne,LngLat:t.LngLat,LngLatBounds:t.LngLatBounds,Point:t.Point,MercatorCoordinate:t.MercatorCoordinate,Evented:t.Evented,config:t.config,get accessToken(){return t.config.ACCESS_TOKEN},set accessToken(e){t.config.ACCESS_TOKEN=e},get baseApiUrl(){return t.config.API_URL},set baseApiUrl(e){t.config.API_URL=e},get workerCount(){return Lt.workerCount},set workerCount(t){Lt.workerCount=t},get maxParallelImageRequests(){return t.config.MAX_PARALLEL_IMAGE_REQUESTS},set maxParallelImageRequests(e){t.config.MAX_PARALLEL_IMAGE_REQUESTS=e},clearStorage:function(e){t.clearTileCache(e)},workerUrl:""};return Wr}),n},"object"==typeof n&&void 0!==e?e.exports=i():(r=r||self).mapboxgl=i()},{}],427:[function(t,e,n){"use strict";e.exports=function(t){for(var e=1<<t+1,n=new Array(e),r=0;r<e;++r)n[r]=o(t,r);return n};var r=t("convex-hull");function i(t,e,n){for(var r=new Array(t),i=0;i<t;++i)r[i]=0,i===e&&(r[i]+=.5),i===n&&(r[i]+=.5);return r}function o(t,e){if(0===e||e===(1<<t+1)-1)return[];for(var n=[],o=[],a=0;a<=t;++a)if(e&1<<a){n.push(i(t,a-1,a-1)),o.push(null);for(var s=0;s<=t;++s)~e&1<<s&&(n.push(i(t,a-1,s-1)),o.push([a,s]))}var A=r(n),l=[];t:for(a=0;a<A.length;++a){var c=A[a],u=[];for(s=0;s<c.length;++s){if(!o[c[s]])continue t;u.push(o[c[s]].slice())}l.push(u)}return l}},{"convex-hull":130}],428:[function(t,e,n){var r=t("./normalize"),i=t("gl-mat4/create"),o=t("gl-mat4/clone"),a=t("gl-mat4/determinant"),s=t("gl-mat4/invert"),A=t("gl-mat4/transpose"),l={length:t("gl-vec3/length"),normalize:t("gl-vec3/normalize"),dot:t("gl-vec3/dot"),cross:t("gl-vec3/cross")},c=i(),u=i(),g=[0,0,0,0],h=[[0,0,0],[0,0,0],[0,0,0]],d=[0,0,0];function f(t,e,n,r,i){t[0]=e[0]*r+n[0]*i,t[1]=e[1]*r+n[1]*i,t[2]=e[2]*r+n[2]*i}e.exports=function(t,e,n,i,p,I){if(e||(e=[0,0,0]),n||(n=[0,0,0]),i||(i=[0,0,0]),p||(p=[0,0,0,1]),I||(I=[0,0,0,1]),!r(c,t))return!1;if(o(u,c),u[3]=0,u[7]=0,u[11]=0,u[15]=1,Math.abs(a(u)<1e-8))return!1;var C,m,y,E,v,B,b,Q=c[3],w=c[7],x=c[11],_=c[12],D=c[13],M=c[14],k=c[15];if(0!==Q||0!==w||0!==x){if(g[0]=Q,g[1]=w,g[2]=x,g[3]=k,!s(u,u))return!1;A(u,u),C=p,y=u,E=(m=g)[0],v=m[1],B=m[2],b=m[3],C[0]=y[0]*E+y[4]*v+y[8]*B+y[12]*b,C[1]=y[1]*E+y[5]*v+y[9]*B+y[13]*b,C[2]=y[2]*E+y[6]*v+y[10]*B+y[14]*b,C[3]=y[3]*E+y[7]*v+y[11]*B+y[15]*b}else p[0]=p[1]=p[2]=0,p[3]=1;if(e[0]=_,e[1]=D,e[2]=M,function(t,e){t[0][0]=e[0],t[0][1]=e[1],t[0][2]=e[2],t[1][0]=e[4],t[1][1]=e[5],t[1][2]=e[6],t[2][0]=e[8],t[2][1]=e[9],t[2][2]=e[10]}(h,c),n[0]=l.length(h[0]),l.normalize(h[0],h[0]),i[0]=l.dot(h[0],h[1]),f(h[1],h[1],h[0],1,-i[0]),n[1]=l.length(h[1]),l.normalize(h[1],h[1]),i[0]/=n[1],i[1]=l.dot(h[0],h[2]),f(h[2],h[2],h[0],1,-i[1]),i[2]=l.dot(h[1],h[2]),f(h[2],h[2],h[1],1,-i[2]),n[2]=l.length(h[2]),l.normalize(h[2],h[2]),i[1]/=n[2],i[2]/=n[2],l.cross(d,h[1],h[2]),l.dot(h[0],d)<0)for(var S=0;S<3;S++)n[S]*=-1,h[S][0]*=-1,h[S][1]*=-1,h[S][2]*=-1;return I[0]=.5*Math.sqrt(Math.max(1+h[0][0]-h[1][1]-h[2][2],0)),I[1]=.5*Math.sqrt(Math.max(1-h[0][0]+h[1][1]-h[2][2],0)),I[2]=.5*Math.sqrt(Math.max(1-h[0][0]-h[1][1]+h[2][2],0)),I[3]=.5*Math.sqrt(Math.max(1+h[0][0]+h[1][1]+h[2][2],0)),h[2][1]>h[1][2]&&(I[0]=-I[0]),h[0][2]>h[2][0]&&(I[1]=-I[1]),h[1][0]>h[0][1]&&(I[2]=-I[2]),!0}},{"./normalize":429,"gl-mat4/clone":259,"gl-mat4/create":260,"gl-mat4/determinant":261,"gl-mat4/invert":265,"gl-mat4/transpose":276,"gl-vec3/cross":334,"gl-vec3/dot":339,"gl-vec3/length":349,"gl-vec3/normalize":356}],429:[function(t,e,n){e.exports=function(t,e){var n=e[15];if(0===n)return!1;for(var r=1/n,i=0;i<16;i++)t[i]=e[i]*r;return!0}},{}],430:[function(t,e,n){var r=t("gl-vec3/lerp"),i=t("mat4-recompose"),o=t("mat4-decompose"),a=t("gl-mat4/determinant"),s=t("quat-slerp"),A=u(),l=u(),c=u();function u(){return{translate:g(),scale:g(1),skew:g(),perspective:[0,0,0,1],quaternion:[0,0,0,1]}}function g(t){return[t||0,t||0,t||0]}e.exports=function(t,e,n,u){if(0===a(e)||0===a(n))return!1;var g=o(e,A.translate,A.scale,A.skew,A.perspective,A.quaternion),h=o(n,l.translate,l.scale,l.skew,l.perspective,l.quaternion);return!(!g||!h||(r(c.translate,A.translate,l.translate,u),r(c.skew,A.skew,l.skew,u),r(c.scale,A.scale,l.scale,u),r(c.perspective,A.perspective,l.perspective,u),s(c.quaternion,A.quaternion,l.quaternion,u),i(t,c.translate,c.scale,c.skew,c.perspective,c.quaternion),0))}},{"gl-mat4/determinant":261,"gl-vec3/lerp":350,"mat4-decompose":428,"mat4-recompose":431,"quat-slerp":483}],431:[function(t,e,n){var r={identity:t("gl-mat4/identity"),translate:t("gl-mat4/translate"),multiply:t("gl-mat4/multiply"),create:t("gl-mat4/create"),scale:t("gl-mat4/scale"),fromRotationTranslation:t("gl-mat4/fromRotationTranslation")},i=(r.create(),r.create());e.exports=function(t,e,n,o,a,s){return r.identity(t),r.fromRotationTranslation(t,s,e),t[3]=a[0],t[7]=a[1],t[11]=a[2],t[15]=a[3],r.identity(i),0!==o[2]&&(i[9]=o[2],r.multiply(t,t,i)),0!==o[1]&&(i[9]=0,i[8]=o[1],r.multiply(t,t,i)),0!==o[0]&&(i[8]=0,i[4]=o[0],r.multiply(t,t,i)),r.scale(t,t,n),t}},{"gl-mat4/create":260,"gl-mat4/fromRotationTranslation":263,"gl-mat4/identity":264,"gl-mat4/multiply":267,"gl-mat4/scale":274,"gl-mat4/translate":275}],432:[function(t,e,n){"use strict";e.exports=Math.log2||function(t){return Math.log(t)*Math.LOG2E}},{}],433:[function(t,e,n){"use strict";var r=t("binary-search-bounds"),i=t("mat4-interpolate"),o=t("gl-mat4/invert"),a=t("gl-mat4/rotateX"),s=t("gl-mat4/rotateY"),A=t("gl-mat4/rotateZ"),l=t("gl-mat4/lookAt"),c=t("gl-mat4/translate"),u=(t("gl-mat4/scale"),t("gl-vec3/normalize")),g=[0,0,0];function h(t){this._components=t.slice(),this._time=[0],this.prevMatrix=t.slice(),this.nextMatrix=t.slice(),this.computedMatrix=t.slice(),this.computedInverse=t.slice(),this.computedEye=[0,0,0],this.computedUp=[0,0,0],this.computedCenter=[0,0,0],this.computedRadius=[0],this._limits=[-1/0,1/0]}e.exports=function(t){return new h((t=t||{}).matrix||[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1])};var d=h.prototype;d.recalcMatrix=function(t){var e=this._time,n=r.le(e,t),a=this.computedMatrix;if(!(n<0)){var s=this._components;if(n===e.length-1)for(var A=16*n,l=0;l<16;++l)a[l]=s[A++];else{var c=e[n+1]-e[n],g=(A=16*n,this.prevMatrix),h=!0;for(l=0;l<16;++l)g[l]=s[A++];var d=this.nextMatrix;for(l=0;l<16;++l)d[l]=s[A++],h=h&&g[l]===d[l];if(c<1e-6||h)for(l=0;l<16;++l)a[l]=g[l];else i(a,g,d,(t-e[n])/c)}var f=this.computedUp;f[0]=a[1],f[1]=a[5],f[2]=a[9],u(f,f);var p=this.computedInverse;o(p,a);var I=this.computedEye,C=p[15];I[0]=p[12]/C,I[1]=p[13]/C,I[2]=p[14]/C;var m=this.computedCenter,y=Math.exp(this.computedRadius[0]);for(l=0;l<3;++l)m[l]=I[l]-a[2+4*l]*y}},d.idle=function(t){if(!(t<this.lastT())){for(var e=this._components,n=e.length-16,r=0;r<16;++r)e.push(e[n++]);this._time.push(t)}},d.flush=function(t){var e=r.gt(this._time,t)-2;e<0||(this._time.splice(0,e),this._components.splice(0,16*e))},d.lastT=function(){return this._time[this._time.length-1]},d.lookAt=function(t,e,n,r){this.recalcMatrix(t),e=e||this.computedEye,n=n||g,r=r||this.computedUp,this.setMatrix(t,l(this.computedMatrix,e,n,r));for(var i=0,o=0;o<3;++o)i+=Math.pow(n[o]-e[o],2);i=Math.log(Math.sqrt(i)),this.computedRadius[0]=i},d.rotate=function(t,e,n,r){this.recalcMatrix(t);var i=this.computedInverse;e&&s(i,i,e),n&&a(i,i,n),r&&A(i,i,r),this.setMatrix(t,o(this.computedMatrix,i))};var f=[0,0,0];d.pan=function(t,e,n,r){f[0]=-(e||0),f[1]=-(n||0),f[2]=-(r||0),this.recalcMatrix(t);var i=this.computedInverse;c(i,i,f),this.setMatrix(t,o(i,i))},d.translate=function(t,e,n,r){f[0]=e||0,f[1]=n||0,f[2]=r||0,this.recalcMatrix(t);var i=this.computedMatrix;c(i,i,f),this.setMatrix(t,i)},d.setMatrix=function(t,e){if(!(t<this.lastT())){this._time.push(t);for(var n=0;n<16;++n)this._components.push(e[n])}},d.setDistance=function(t,e){this.computedRadius[0]=e},d.setDistanceLimits=function(t,e){var n=this._limits;n[0]=t,n[1]=e},d.getDistanceLimits=function(t){var e=this._limits;return t?(t[0]=e[0],t[1]=e[1],t):e}},{"binary-search-bounds":91,"gl-mat4/invert":265,"gl-mat4/lookAt":266,"gl-mat4/rotateX":271,"gl-mat4/rotateY":272,"gl-mat4/rotateZ":273,"gl-mat4/scale":274,"gl-mat4/translate":275,"gl-vec3/normalize":356,"mat4-interpolate":430}],434:[function(t,e,n){"use strict";e.exports=function(t){var e=t.length;if(e<3){for(var n=new Array(e),i=0;i<e;++i)n[i]=i;return 2===e&&t[0][0]===t[1][0]&&t[0][1]===t[1][1]?[0]:n}for(var o=new Array(e),i=0;i<e;++i)o[i]=i;o.sort(function(e,n){var r=t[e][0]-t[n][0];return r||t[e][1]-t[n][1]});for(var a=[o[0],o[1]],s=[o[0],o[1]],i=2;i<e;++i){for(var A=o[i],l=t[A],c=a.length;c>1&&r(t[a[c-2]],t[a[c-1]],l)<=0;)c-=1,a.pop();for(a.push(A),c=s.length;c>1&&r(t[s[c-2]],t[s[c-1]],l)>=0;)c-=1,s.pop();s.push(A)}for(var n=new Array(s.length+a.length-2),u=0,i=0,g=a.length;i<g;++i)n[u++]=a[i];for(var h=s.length-2;h>0;--h)n[u++]=s[h];return n};var r=t("robust-orientation")[3]},{"robust-orientation":510}],435:[function(t,e,n){"use strict";e.exports=function(t,e){e||(e=t,t=window);var n=0,i=0,o=0,a={shift:!1,alt:!1,control:!1,meta:!1},s=!1;function A(t){var e=!1;return"altKey"in t&&(e=e||t.altKey!==a.alt,a.alt=!!t.altKey),"shiftKey"in t&&(e=e||t.shiftKey!==a.shift,a.shift=!!t.shiftKey),"ctrlKey"in t&&(e=e||t.ctrlKey!==a.control,a.control=!!t.ctrlKey),"metaKey"in t&&(e=e||t.metaKey!==a.meta,a.meta=!!t.metaKey),e}function l(t,s){var l=r.x(s),c=r.y(s);"buttons"in s&&(t=0|s.buttons),(t!==n||l!==i||c!==o||A(s))&&(n=0|t,i=l||0,o=c||0,e&&e(n,i,o,a))}function c(t){l(0,t)}function u(){(n||i||o||a.shift||a.alt||a.meta||a.control)&&(i=o=0,n=0,a.shift=a.alt=a.control=a.meta=!1,e&&e(0,0,0,a))}function g(t){A(t)&&e&&e(n,i,o,a)}function h(t){0===r.buttons(t)?l(0,t):l(n,t)}function d(t){l(n|r.buttons(t),t)}function f(t){l(n&~r.buttons(t),t)}function p(){s||(s=!0,t.addEventListener("mousemove",h),t.addEventListener("mousedown",d),t.addEventListener("mouseup",f),t.addEventListener("mouseleave",c),t.addEventListener("mouseenter",c),t.addEventListener("mouseout",c),t.addEventListener("mouseover",c),t.addEventListener("blur",u),t.addEventListener("keyup",g),t.addEventListener("keydown",g),t.addEventListener("keypress",g),t!==window&&(window.addEventListener("blur",u),window.addEventListener("keyup",g),window.addEventListener("keydown",g),window.addEventListener("keypress",g)))}p();var I={element:t};return Object.defineProperties(I,{enabled:{get:function(){return s},set:function(e){e?p():s&&(s=!1,t.removeEventListener("mousemove",h),t.removeEventListener("mousedown",d),t.removeEventListener("mouseup",f),t.removeEventListener("mouseleave",c),t.removeEventListener("mouseenter",c),t.removeEventListener("mouseout",c),t.removeEventListener("mouseover",c),t.removeEventListener("blur",u),t.removeEventListener("keyup",g),t.removeEventListener("keydown",g),t.removeEventListener("keypress",g),t!==window&&(window.removeEventListener("blur",u),window.removeEventListener("keyup",g),window.removeEventListener("keydown",g),window.removeEventListener("keypress",g)))},enumerable:!0},buttons:{get:function(){return n},enumerable:!0},x:{get:function(){return i},enumerable:!0},y:{get:function(){return o},enumerable:!0},mods:{get:function(){return a},enumerable:!0}}),I};var r=t("mouse-event")},{"mouse-event":437}],436:[function(t,e,n){var r={left:0,top:0};e.exports=function(t,e,n){e=e||t.currentTarget||t.srcElement,Array.isArray(n)||(n=[0,0]);var i,o=t.clientX||0,a=t.clientY||0,s=(i=e)===window||i===document||i===document.body?r:i.getBoundingClientRect();return n[0]=o-s.left,n[1]=a-s.top,n}},{}],437:[function(t,e,n){"use strict";function r(t){return t.target||t.srcElement||window}n.buttons=function(t){if("object"==typeof t){if("buttons"in t)return t.buttons;if("which"in t){if(2===(e=t.which))return 4;if(3===e)return 2;if(e>0)return 1<<e-1}else if("button"in t){var e;if(1===(e=t.button))return 4;if(2===e)return 2;if(e>=0)return 1<<e}}return 0},n.element=r,n.x=function(t){if("object"==typeof t){if("offsetX"in t)return t.offsetX;var e=r(t).getBoundingClientRect();return t.clientX-e.left}return 0},n.y=function(t){if("object"==typeof t){if("offsetY"in t)return t.offsetY;var e=r(t).getBoundingClientRect();return t.clientY-e.top}return 0}},{}],438:[function(t,e,n){"use strict";var r=t("to-px");e.exports=function(t,e,n){"function"==typeof t&&(n=!!e,e=t,t=window);var i=r("ex",t),o=function(t){n&&t.preventDefault();var r=t.deltaX||0,o=t.deltaY||0,a=t.deltaZ||0,s=t.deltaMode,A=1;switch(s){case 1:A=i;break;case 2:A=window.innerHeight}if(o*=A,a*=A,(r*=A)||o||a)return e(r,o,a,t)};return t.addEventListener("wheel",o),o}},{"to-px":539}],439:[function(t,e,n){"use strict";var r=t("typedarray-pool");function i(t){return"a"+t}function o(t){return"d"+t}function a(t,e){return"c"+t+"_"+e}function s(t){return"s"+t}function A(t,e){return"t"+t+"_"+e}function l(t){return"o"+t}function c(t){return"x"+t}function u(t){return"p"+t}function g(t,e){return"d"+t+"_"+e}function h(t){return"i"+t}function d(t,e){return"u"+t+"_"+e}function f(t){return"b"+t}function p(t){return"y"+t}function I(t){return"e"+t}function C(t){return"v"+t}e.exports=function(t){function e(t){throw new Error("ndarray-extract-contour: "+t)}"object"!=typeof t&&e("Must specify arguments");var n=t.order;Array.isArray(n)||e("Must specify order");var w=t.arrayArguments||1;w<1&&e("Must have at least one array argument");var x=t.scalarArguments||0;x<0&&e("Scalar arg count must be > 0"),"function"!=typeof t.vertex&&e("Must specify vertex creation function"),"function"!=typeof t.cell&&e("Must specify cell creation function"),"function"!=typeof t.phase&&e("Must specify phase function");for(var _=t.getters||[],D=new Array(w),M=0;M<w;++M)_.indexOf(M)>=0?D[M]=!0:D[M]=!1;return function(t,e,n,w,x,_){var D=_.length,M=x.length;if(M<2)throw new Error("ndarray-extract-contour: Dimension must be at least 2");for(var k="extractContour"+x.join("_"),S=[],L=[],R=[],N=0;N<D;++N)R.push(i(N));for(var N=0;N<w;++N)R.push(c(N));for(var N=0;N<M;++N)L.push(s(N)+"="+i(0)+".shape["+N+"]|0");for(var N=0;N<D;++N){L.push(o(N)+"="+i(N)+".data",l(N)+"="+i(N)+".offset|0");for(var F=0;F<M;++F)L.push(A(N,F)+"="+i(N)+".stride["+F+"]|0")}for(var N=0;N<D;++N){L.push(u(N)+"="+l(N)),L.push(a(N,0));for(var F=1;F<1<<M;++F){for(var T=[],U=0;U<M;++U)F&1<<U&&T.push("-"+A(N,U));L.push(g(N,F)+"=("+T.join("")+")|0"),L.push(a(N,F)+"=0")}}for(var N=0;N<D;++N)for(var F=0;F<M;++F){var O=[A(N,x[F])];F>0&&O.push(A(N,x[F-1])+"*"+s(x[F-1])),L.push(d(N,x[F])+"=("+O.join("-")+")|0")}for(var N=0;N<M;++N)L.push(h(N)+"=0");L.push(E+"=0");for(var G=["2"],N=M-2;N>=0;--N)G.push(s(x[N]));L.push(v+"=("+G.join("*")+")|0",y+"=mallocUint32("+v+")",m+"=mallocUint32("+v+")",B+"=0"),L.push(f(0)+"=0");for(var F=1;F<1<<M;++F){for(var H=[],j=[],U=0;U<M;++U)F&1<<U&&(0===j.length?H.push("1"):H.unshift(j.join("*"))),j.push(s(x[U]));var Y="";H[0].indexOf(s(x[M-2]))<0&&(Y="-");var P=Q(M,F,x);L.push(I(P)+"=(-"+H.join("-")+")|0",p(P)+"=("+Y+H.join("-")+")|0",f(P)+"=0")}function K(t,e){S.push("for(",h(x[t]),"=",e,";",h(x[t]),"<",s(x[t]),";","++",h(x[t]),"){")}function z(t){for(var e=0;e<D;++e)S.push(u(e),"+=",d(e,x[t]),";");S.push("}")}function J(){for(var t=1;t<1<<M;++t)S.push(b,"=",I(t),";",I(t),"=",p(t),";",p(t),"=",b,";")}L.push(C(0)+"=0",b+"=0"),function t(e,n){if(e<0)!function(t){for(var e=0;e<D;++e)_[e]?S.push(a(e,0),"=",o(e),".get(",u(e),");"):S.push(a(e,0),"=",o(e),"[",u(e),"];");for(var n=[],e=0;e<D;++e)n.push(a(e,0));for(var e=0;e<w;++e)n.push(c(e));S.push(f(0),"=",y,"[",B,"]=phase(",n.join(),");");for(var r=1;r<1<<M;++r)S.push(f(r),"=",y,"[",B,"+",I(r),"];");for(var i=[],r=1;r<1<<M;++r)i.push("("+f(0)+"!=="+f(r)+")");S.push("if(",i.join("||"),"){");for(var s=[],e=0;e<M;++e)s.push(h(e));for(var e=0;e<D;++e){s.push(a(e,0));for(var r=1;r<1<<M;++r)_[e]?S.push(a(e,r),"=",o(e),".get(",u(e),"+",g(e,r),");"):S.push(a(e,r),"=",o(e),"[",u(e),"+",g(e,r),"];"),s.push(a(e,r))}for(var e=0;e<1<<M;++e)s.push(f(e));for(var e=0;e<w;++e)s.push(c(e));S.push("vertex(",s.join(),");",C(0),"=",m,"[",B,"]=",E,"++;");for(var A=(1<<M)-1,l=f(A),r=0;r<M;++r)if(0==(t&~(1<<r))){for(var d=A^1<<r,p=f(d),v=[],b=d;b>0;b=b-1&d)v.push(m+"["+B+"+"+I(b)+"]");v.push(C(0));for(var b=0;b<D;++b)1&r?v.push(a(b,A),a(b,d)):v.push(a(b,d),a(b,A));1&r?v.push(l,p):v.push(p,l);for(var b=0;b<w;++b)v.push(c(b));S.push("if(",l,"!==",p,"){","face(",v.join(),")}")}S.push("}",B,"+=1;")}(n);else{!function(t){for(var e=t-1;e>=0;--e)K(e,0);for(var n=[],e=0;e<D;++e)_[e]?n.push(o(e)+".get("+u(e)+")"):n.push(o(e)+"["+u(e)+"]");for(var e=0;e<w;++e)n.push(c(e));S.push(y,"[",B,"++]=phase(",n.join(),");");for(var e=0;e<t;++e)z(e);for(var r=0;r<D;++r)S.push(u(r),"+=",d(r,x[t]),";")}(e),S.push("if(",s(x[e]),">0){",h(x[e]),"=1;"),t(e-1,n|1<<x[e]);for(var r=0;r<D;++r)S.push(u(r),"+=",d(r,x[e]),";");e===M-1&&(S.push(B,"=0;"),J()),K(e,2),t(e-1,n),e===M-1&&(S.push("if(",h(x[M-1]),"&1){",B,"=0;}"),J()),z(e),S.push("}")}}(M-1,0),S.push("freeUint32(",m,");freeUint32(",y,");");var q=["'use strict';","function ",k,"(",R.join(),"){","var ",L.join(),";",S.join(""),"}","return ",k].join("");return new Function("vertex","face","phase","mallocUint32","freeUint32",q)(t,e,n,r.mallocUint32,r.freeUint32)}(t.vertex,t.cell,t.phase,x,n,D)};var m="V",y="P",E="N",v="Q",B="X",b="T";function Q(t,e,n){for(var r=0,i=0;i<t;++i)e&1<<i&&(r|=1<<n[i]);return r}},{"typedarray-pool":545}],440:[function(t,e,n){"use strict";var r=t("cwise/lib/wrapper")({args:["index","array","scalar"],pre:{body:"{}",args:[],thisVars:[],localVars:[]},body:{body:"{_inline_1_arg1_=_inline_1_arg2_.apply(void 0,_inline_1_arg0_)}",args:[{name:"_inline_1_arg0_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_1_arg1_",lvalue:!0,rvalue:!1,count:1},{name:"_inline_1_arg2_",lvalue:!1,rvalue:!0,count:1}],thisVars:[],localVars:[]},post:{body:"{}",args:[],thisVars:[],localVars:[]},debug:!1,funcName:"cwise",blockSize:64});e.exports=function(t,e){return r(t,e),t}},{"cwise/lib/wrapper":149}],441:[function(t,e,n){"use strict";e.exports=function(t,e,n){if(Array.isArray(n)){if(n.length!==e.dimension)throw new Error("ndarray-gradient: invalid boundary conditions")}else n=r(e.dimension,"string"==typeof n?n:"clamp");if(t.dimension!==e.dimension+1)throw new Error("ndarray-gradient: output dimension must be +1 input dimension");if(t.shape[e.dimension]!==e.dimension)throw new Error("ndarray-gradient: output shape must match input shape");for(var i=0;i<e.dimension;++i)if(t.shape[i]!==e.shape[i])throw new Error("ndarray-gradient: shape mismatch");return 0===e.size?t:e.dimension<=0?(t.set(0),t):function(t){var e=t.join();if(I=a[e])return I;var n=t.length,r=["function gradient(dst,src){var s=src.shape.slice();"];function i(e){for(var i=n-e.length,o=[],a=[],s=[],A=0;A<n;++A)e.indexOf(A+1)>=0?s.push("0"):e.indexOf(-(A+1))>=0?s.push("s["+A+"]-1"):(s.push("-1"),o.push("1"),a.push("s["+A+"]-2"));var l=".lo("+o.join()+").hi("+a.join()+")";if(0===o.length&&(l=""),i>0){r.push("if(1");for(var A=0;A<n;++A)e.indexOf(A+1)>=0||e.indexOf(-(A+1))>=0||r.push("&&s[",A,"]>2");r.push("){grad",i,"(src.pick(",s.join(),")",l);for(var A=0;A<n;++A)e.indexOf(A+1)>=0||e.indexOf(-(A+1))>=0||r.push(",dst.pick(",s.join(),",",A,")",l);r.push(");")}for(var A=0;A<e.length;++A){var c=Math.abs(e[A])-1,u="dst.pick("+s.join()+","+c+")"+l;switch(t[c]){case"clamp":var g=s.slice(),h=s.slice();e[A]<0?g[c]="s["+c+"]-2":h[c]="1",0===i?r.push("if(s[",c,"]>1){dst.set(",s.join(),",",c,",0.5*(src.get(",g.join(),")-src.get(",h.join(),")))}else{dst.set(",s.join(),",",c,",0)};"):r.push("if(s[",c,"]>1){diff(",u,",src.pick(",g.join(),")",l,",src.pick(",h.join(),")",l,");}else{zero(",u,");};");break;case"mirror":0===i?r.push("dst.set(",s.join(),",",c,",0);"):r.push("zero(",u,");");break;case"wrap":var d=s.slice(),f=s.slice();e[A]<0?(d[c]="s["+c+"]-2",f[c]="0"):(d[c]="s["+c+"]-1",f[c]="1"),0===i?r.push("if(s[",c,"]>2){dst.set(",s.join(),",",c,",0.5*(src.get(",d.join(),")-src.get(",f.join(),")))}else{dst.set(",s.join(),",",c,",0)};"):r.push("if(s[",c,"]>2){diff(",u,",src.pick(",d.join(),")",l,",src.pick(",f.join(),")",l,");}else{zero(",u,");};");break;default:throw new Error("ndarray-gradient: Invalid boundary condition")}}i>0&&r.push("};")}for(var s=0;s<1<<n;++s){for(var u=[],g=0;g<n;++g)s&1<<g&&u.push(g+1);for(var h=0;h<1<<u.length;++h){for(var d=u.slice(),g=0;g<u.length;++g)h&1<<g&&(d[g]=-d[g]);i(d)}}r.push("return dst;};return gradient");for(var f=["diff","zero"],p=[A,l],s=1;s<=n;++s)f.push("grad"+s),p.push(c(s));f.push(r.join(""));var I=Function.apply(void 0,f).apply(void 0,p);return o[e]=I,I}(n)(t,e)};var r=t("dup"),i=t("cwise-compiler"),o={},a={},s={body:"",args:[],thisVars:[],localVars:[]},A=i({args:["array","array","array"],pre:s,post:s,body:{args:[{name:"out",lvalue:!0,rvalue:!1,count:1},{name:"left",lvalue:!1,rvalue:!0,count:1},{name:"right",lvalue:!1,rvalue:!0,count:1}],body:"out=0.5*(left-right)",thisVars:[],localVars:[]},funcName:"cdiff"}),l=i({args:["array"],pre:s,post:s,body:{args:[{name:"out",lvalue:!0,rvalue:!1,count:1}],body:"out=0",thisVars:[],localVars:[]},funcName:"zero"});function c(t){if(t in o)return o[t];for(var e=[],n=0;n<t;++n)e.push("out",n,"s=0.5*(inp",n,"l-inp",n,"r);");var a=["array"],A=["junk"];for(n=0;n<t;++n){a.push("array"),A.push("out"+n+"s");var l=r(t);l[n]=-1,a.push({array:0,offset:l.slice()}),l[n]=1,a.push({array:0,offset:l.slice()}),A.push("inp"+n+"l","inp"+n+"r")}return o[t]=i({args:a,pre:s,post:s,body:{body:e.join(""),args:A.map(function(t){return{name:t,lvalue:0===t.indexOf("out"),rvalue:0===t.indexOf("inp"),count:"junk"!==t|0}}),thisVars:[],localVars:[]},funcName:"fdTemplate"+t})}},{"cwise-compiler":146,dup:170}],442:[function(t,e,n){"use strict";var r=t("ndarray-warp"),i=t("gl-matrix-invert");e.exports=function(t,e,n){var o=e.dimension,a=i([],n);return r(t,e,function(t,e){for(var n=0;n<o;++n){t[n]=a[(o+1)*o+n];for(var r=0;r<o;++r)t[n]+=a[(o+1)*r+n]*e[r]}var i=a[(o+1)*(o+1)-1];for(r=0;r<o;++r)i+=a[(o+1)*r+o]*e[r];var s=1/i;for(n=0;n<o;++n)t[n]*=s;return t}),t}},{"gl-matrix-invert":277,"ndarray-warp":449}],443:[function(t,e,n){"use strict";function r(t,e){var n=Math.floor(e),r=e-n,i=0<=n&&n<t.shape[0],o=0<=n+1&&n+1<t.shape[0];return(1-r)*(i?+t.get(n):0)+r*(o?+t.get(n+1):0)}function i(t,e,n){var r=Math.floor(e),i=e-r,o=0<=r&&r<t.shape[0],a=0<=r+1&&r+1<t.shape[0],s=Math.floor(n),A=n-s,l=0<=s&&s<t.shape[1],c=0<=s+1&&s+1<t.shape[1],u=o&&l?t.get(r,s):0,g=o&&c?t.get(r,s+1):0;return(1-A)*((1-i)*u+i*(a&&l?t.get(r+1,s):0))+A*((1-i)*g+i*(a&&c?t.get(r+1,s+1):0))}function o(t,e,n,r){var i=Math.floor(e),o=e-i,a=0<=i&&i<t.shape[0],s=0<=i+1&&i+1<t.shape[0],A=Math.floor(n),l=n-A,c=0<=A&&A<t.shape[1],u=0<=A+1&&A+1<t.shape[1],g=Math.floor(r),h=r-g,d=0<=g&&g<t.shape[2],f=0<=g+1&&g+1<t.shape[2],p=a&&c&&d?t.get(i,A,g):0,I=a&&u&&d?t.get(i,A+1,g):0,C=s&&c&&d?t.get(i+1,A,g):0,m=s&&u&&d?t.get(i+1,A+1,g):0,y=a&&c&&f?t.get(i,A,g+1):0,E=a&&u&&f?t.get(i,A+1,g+1):0;return(1-h)*((1-l)*((1-o)*p+o*C)+l*((1-o)*I+o*m))+h*((1-l)*((1-o)*y+o*(s&&c&&f?t.get(i+1,A,g+1):0))+l*((1-o)*E+o*(s&&u&&f?t.get(i+1,A+1,g+1):0)))}e.exports=function(t,e,n,a){switch(t.shape.length){case 0:return 0;case 1:return r(t,e);case 2:return i(t,e,n);case 3:return o(t,e,n,a);default:return function(t){var e,n,r=0|t.shape.length,i=new Array(r),o=new Array(r),a=new Array(r),s=new Array(r);for(e=0;e<r;++e)n=+arguments[e+1],i[e]=Math.floor(n),o[e]=n-i[e],a[e]=0<=i[e]&&i[e]<t.shape[e],s[e]=0<=i[e]+1&&i[e]+1<t.shape[e];var A,l,c,u=0;t:for(e=0;e<1<<r;++e){for(l=1,c=t.offset,A=0;A<r;++A)if(e&1<<A){if(!s[A])continue t;l*=o[A],c+=t.stride[A]*(i[A]+1)}else{if(!a[A])continue t;l*=1-o[A],c+=t.stride[A]*i[A]}u+=l*t.data[c]}return u}.apply(void 0,arguments)}},e.exports.d1=r,e.exports.d2=i,e.exports.d3=o},{}],444:[function(t,e,n){"use strict";var r=t("cwise-compiler"),i={body:"",args:[],thisVars:[],localVars:[]};function o(t){if(!t)return i;for(var e=0;e<t.args.length;++e){var n=t.args[e];t.args[e]=0===e?{name:n,lvalue:!0,rvalue:!!t.rvalue,count:t.count||1}:{name:n,lvalue:!1,rvalue:!0,count:1}}return t.thisVars||(t.thisVars=[]),t.localVars||(t.localVars=[]),t}function a(t){for(var e=[],n=0;n<t.args.length;++n)e.push("a"+n);return new Function("P",["return function ",t.funcName,"_ndarrayops(",e.join(","),") {P(",e.join(","),");return a0}"].join(""))(function(t){return r({args:t.args,pre:o(t.pre),body:o(t.body),post:o(t.proc),funcName:t.funcName})}(t))}var s={add:"+",sub:"-",mul:"*",div:"/",mod:"%",band:"&",bor:"|",bxor:"^",lshift:"<<",rshift:">>",rrshift:">>>"};!function(){for(var t in s){var e=s[t];n[t]=a({args:["array","array","array"],body:{args:["a","b","c"],body:"a=b"+e+"c"},funcName:t}),n[t+"eq"]=a({args:["array","array"],body:{args:["a","b"],body:"a"+e+"=b"},rvalue:!0,funcName:t+"eq"}),n[t+"s"]=a({args:["array","array","scalar"],body:{args:["a","b","s"],body:"a=b"+e+"s"},funcName:t+"s"}),n[t+"seq"]=a({args:["array","scalar"],body:{args:["a","s"],body:"a"+e+"=s"},rvalue:!0,funcName:t+"seq"})}}();var A={not:"!",bnot:"~",neg:"-",recip:"1.0/"};!function(){for(var t in A){var e=A[t];n[t]=a({args:["array","array"],body:{args:["a","b"],body:"a="+e+"b"},funcName:t}),n[t+"eq"]=a({args:["array"],body:{args:["a"],body:"a="+e+"a"},rvalue:!0,count:2,funcName:t+"eq"})}}();var l={and:"&&",or:"||",eq:"===",neq:"!==",lt:"<",gt:">",leq:"<=",geq:">="};!function(){for(var t in l){var e=l[t];n[t]=a({args:["array","array","array"],body:{args:["a","b","c"],body:"a=b"+e+"c"},funcName:t}),n[t+"s"]=a({args:["array","array","scalar"],body:{args:["a","b","s"],body:"a=b"+e+"s"},funcName:t+"s"}),n[t+"eq"]=a({args:["array","array"],body:{args:["a","b"],body:"a=a"+e+"b"},rvalue:!0,count:2,funcName:t+"eq"}),n[t+"seq"]=a({args:["array","scalar"],body:{args:["a","s"],body:"a=a"+e+"s"},rvalue:!0,count:2,funcName:t+"seq"})}}();var c=["abs","acos","asin","atan","ceil","cos","exp","floor","log","round","sin","sqrt","tan"];!function(){for(var t=0;t<c.length;++t){var e=c[t];n[e]=a({args:["array","array"],pre:{args:[],body:"this_f=Math."+e,thisVars:["this_f"]},body:{args:["a","b"],body:"a=this_f(b)",thisVars:["this_f"]},funcName:e}),n[e+"eq"]=a({args:["array"],pre:{args:[],body:"this_f=Math."+e,thisVars:["this_f"]},body:{args:["a"],body:"a=this_f(a)",thisVars:["this_f"]},rvalue:!0,count:2,funcName:e+"eq"})}}();var u=["max","min","atan2","pow"];!function(){for(var t=0;t<u.length;++t){var e=u[t];n[e]=a({args:["array","array","array"],pre:{args:[],body:"this_f=Math."+e,thisVars:["this_f"]},body:{args:["a","b","c"],body:"a=this_f(b,c)",thisVars:["this_f"]},funcName:e}),n[e+"s"]=a({args:["array","array","scalar"],pre:{args:[],body:"this_f=Math."+e,thisVars:["this_f"]},body:{args:["a","b","c"],body:"a=this_f(b,c)",thisVars:["this_f"]},funcName:e+"s"}),n[e+"eq"]=a({args:["array","array"],pre:{args:[],body:"this_f=Math."+e,thisVars:["this_f"]},body:{args:["a","b"],body:"a=this_f(a,b)",thisVars:["this_f"]},rvalue:!0,count:2,funcName:e+"eq"}),n[e+"seq"]=a({args:["array","scalar"],pre:{args:[],body:"this_f=Math."+e,thisVars:["this_f"]},body:{args:["a","b"],body:"a=this_f(a,b)",thisVars:["this_f"]},rvalue:!0,count:2,funcName:e+"seq"})}}();var g=["atan2","pow"];!function(){for(var t=0;t<g.length;++t){var e=g[t];n[e+"op"]=a({args:["array","array","array"],pre:{args:[],body:"this_f=Math."+e,thisVars:["this_f"]},body:{args:["a","b","c"],body:"a=this_f(c,b)",thisVars:["this_f"]},funcName:e+"op"}),n[e+"ops"]=a({args:["array","array","scalar"],pre:{args:[],body:"this_f=Math."+e,thisVars:["this_f"]},body:{args:["a","b","c"],body:"a=this_f(c,b)",thisVars:["this_f"]},funcName:e+"ops"}),n[e+"opeq"]=a({args:["array","array"],pre:{args:[],body:"this_f=Math."+e,thisVars:["this_f"]},body:{args:["a","b"],body:"a=this_f(b,a)",thisVars:["this_f"]},rvalue:!0,count:2,funcName:e+"opeq"}),n[e+"opseq"]=a({args:["array","scalar"],pre:{args:[],body:"this_f=Math."+e,thisVars:["this_f"]},body:{args:["a","b"],body:"a=this_f(b,a)",thisVars:["this_f"]},rvalue:!0,count:2,funcName:e+"opseq"})}}(),n.any=r({args:["array"],pre:i,body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:1}],body:"if(a){return true}",localVars:[],thisVars:[]},post:{args:[],localVars:[],thisVars:[],body:"return false"},funcName:"any"}),n.all=r({args:["array"],pre:i,body:{args:[{name:"x",lvalue:!1,rvalue:!0,count:1}],body:"if(!x){return false}",localVars:[],thisVars:[]},post:{args:[],localVars:[],thisVars:[],body:"return true"},funcName:"all"}),n.sum=r({args:["array"],pre:{args:[],localVars:[],thisVars:["this_s"],body:"this_s=0"},body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:1}],body:"this_s+=a",localVars:[],thisVars:["this_s"]},post:{args:[],localVars:[],thisVars:["this_s"],body:"return this_s"},funcName:"sum"}),n.prod=r({args:["array"],pre:{args:[],localVars:[],thisVars:["this_s"],body:"this_s=1"},body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:1}],body:"this_s*=a",localVars:[],thisVars:["this_s"]},post:{args:[],localVars:[],thisVars:["this_s"],body:"return this_s"},funcName:"prod"}),n.norm2squared=r({args:["array"],pre:{args:[],localVars:[],thisVars:["this_s"],body:"this_s=0"},body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:2}],body:"this_s+=a*a",localVars:[],thisVars:["this_s"]},post:{args:[],localVars:[],thisVars:["this_s"],body:"return this_s"},funcName:"norm2squared"}),n.norm2=r({args:["array"],pre:{args:[],localVars:[],thisVars:["this_s"],body:"this_s=0"},body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:2}],body:"this_s+=a*a",localVars:[],thisVars:["this_s"]},post:{args:[],localVars:[],thisVars:["this_s"],body:"return Math.sqrt(this_s)"},funcName:"norm2"}),n.norminf=r({args:["array"],pre:{args:[],localVars:[],thisVars:["this_s"],body:"this_s=0"},body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:4}],body:"if(-a>this_s){this_s=-a}else if(a>this_s){this_s=a}",localVars:[],thisVars:["this_s"]},post:{args:[],localVars:[],thisVars:["this_s"],body:"return this_s"},funcName:"norminf"}),n.norm1=r({args:["array"],pre:{args:[],localVars:[],thisVars:["this_s"],body:"this_s=0"},body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:3}],body:"this_s+=a<0?-a:a",localVars:[],thisVars:["this_s"]},post:{args:[],localVars:[],thisVars:["this_s"],body:"return this_s"},funcName:"norm1"}),n.sup=r({args:["array"],pre:{body:"this_h=-Infinity",args:[],thisVars:["this_h"],localVars:[]},body:{body:"if(_inline_1_arg0_>this_h)this_h=_inline_1_arg0_",args:[{name:"_inline_1_arg0_",lvalue:!1,rvalue:!0,count:2}],thisVars:["this_h"],localVars:[]},post:{body:"return this_h",args:[],thisVars:["this_h"],localVars:[]}}),n.inf=r({args:["array"],pre:{body:"this_h=Infinity",args:[],thisVars:["this_h"],localVars:[]},body:{body:"if(_inline_1_arg0_<this_h)this_h=_inline_1_arg0_",args:[{name:"_inline_1_arg0_",lvalue:!1,rvalue:!0,count:2}],thisVars:["this_h"],localVars:[]},post:{body:"return this_h",args:[],thisVars:["this_h"],localVars:[]}}),n.argmin=r({args:["index","array","shape"],pre:{body:"{this_v=Infinity;this_i=_inline_0_arg2_.slice(0)}",args:[{name:"_inline_0_arg0_",lvalue:!1,rvalue:!1,count:0},{name:"_inline_0_arg1_",lvalue:!1,rvalue:!1,count:0},{name:"_inline_0_arg2_",lvalue:!1,rvalue:!0,count:1}],thisVars:["this_i","this_v"],localVars:[]},body:{body:"{if(_inline_1_arg1_<this_v){this_v=_inline_1_arg1_;for(var _inline_1_k=0;_inline_1_k<_inline_1_arg0_.length;++_inline_1_k){this_i[_inline_1_k]=_inline_1_arg0_[_inline_1_k]}}}",args:[{name:"_inline_1_arg0_",lvalue:!1,rvalue:!0,count:2},{name:"_inline_1_arg1_",lvalue:!1,rvalue:!0,count:2}],thisVars:["this_i","this_v"],localVars:["_inline_1_k"]},post:{body:"{return this_i}",args:[],thisVars:["this_i"],localVars:[]}}),n.argmax=r({args:["index","array","shape"],pre:{body:"{this_v=-Infinity;this_i=_inline_0_arg2_.slice(0)}",args:[{name:"_inline_0_arg0_",lvalue:!1,rvalue:!1,count:0},{name:"_inline_0_arg1_",lvalue:!1,rvalue:!1,count:0},{name:"_inline_0_arg2_",lvalue:!1,rvalue:!0,count:1}],thisVars:["this_i","this_v"],localVars:[]},body:{body:"{if(_inline_1_arg1_>this_v){this_v=_inline_1_arg1_;for(var _inline_1_k=0;_inline_1_k<_inline_1_arg0_.length;++_inline_1_k){this_i[_inline_1_k]=_inline_1_arg0_[_inline_1_k]}}}",args:[{name:"_inline_1_arg0_",lvalue:!1,rvalue:!0,count:2},{name:"_inline_1_arg1_",lvalue:!1,rvalue:!0,count:2}],thisVars:["this_i","this_v"],localVars:["_inline_1_k"]},post:{body:"{return this_i}",args:[],thisVars:["this_i"],localVars:[]}}),n.random=a({args:["array"],pre:{args:[],body:"this_f=Math.random",thisVars:["this_f"]},body:{args:["a"],body:"a=this_f()",thisVars:["this_f"]},funcName:"random"}),n.assign=a({args:["array","array"],body:{args:["a","b"],body:"a=b"},funcName:"assign"}),n.assigns=a({args:["array","scalar"],body:{args:["a","b"],body:"a=b"},funcName:"assigns"}),n.equals=r({args:["array","array"],pre:i,body:{args:[{name:"x",lvalue:!1,rvalue:!0,count:1},{name:"y",lvalue:!1,rvalue:!0,count:1}],body:"if(x!==y){return false}",localVars:[],thisVars:[]},post:{args:[],localVars:[],thisVars:[],body:"return true"},funcName:"equals"})},{"cwise-compiler":146}],445:[function(t,e,n){"use strict";var r=t("ndarray"),i=t("./doConvert.js");e.exports=function(t,e){for(var n=[],o=t,a=1;Array.isArray(o);)n.push(o.length),a*=o.length,o=o[0];return 0===n.length?r():(e||(e=r(new Float64Array(a),n)),i(e,t),e)}},{"./doConvert.js":446,ndarray:450}],446:[function(t,e,n){e.exports=t("cwise-compiler")({args:["array","scalar","index"],pre:{body:"{}",args:[],thisVars:[],localVars:[]},body:{body:"{\nvar _inline_1_v=_inline_1_arg1_,_inline_1_i\nfor(_inline_1_i=0;_inline_1_i<_inline_1_arg2_.length-1;++_inline_1_i) {\n_inline_1_v=_inline_1_v[_inline_1_arg2_[_inline_1_i]]\n}\n_inline_1_arg0_=_inline_1_v[_inline_1_arg2_[_inline_1_arg2_.length-1]]\n}",args:[{name:"_inline_1_arg0_",lvalue:!0,rvalue:!1,count:1},{name:"_inline_1_arg1_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_1_arg2_",lvalue:!1,rvalue:!0,count:4}],thisVars:[],localVars:["_inline_1_i","_inline_1_v"]},post:{body:"{}",args:[],thisVars:[],localVars:[]},funcName:"convert",blockSize:64})},{"cwise-compiler":146}],447:[function(t,e,n){"use strict";var r=t("typedarray-pool"),i=32;function o(t){switch(t){case"uint8":return[r.mallocUint8,r.freeUint8];case"uint16":return[r.mallocUint16,r.freeUint16];case"uint32":return[r.mallocUint32,r.freeUint32];case"int8":return[r.mallocInt8,r.freeInt8];case"int16":return[r.mallocInt16,r.freeInt16];case"int32":return[r.mallocInt32,r.freeInt32];case"float32":return[r.mallocFloat,r.freeFloat];case"float64":return[r.mallocDouble,r.freeDouble];default:return null}}function a(t){for(var e=[],n=0;n<t;++n)e.push("s"+n);for(n=0;n<t;++n)e.push("n"+n);for(n=1;n<t;++n)e.push("d"+n);for(n=1;n<t;++n)e.push("e"+n);for(n=1;n<t;++n)e.push("f"+n);return e}e.exports=function(t,e){var n=["'use strict'"],r=["ndarraySortWrapper",t.join("d"),e].join("");n.push(["function ",r,"(",["array"].join(","),"){"].join(""));for(var s=["data=array.data,offset=array.offset|0,shape=array.shape,stride=array.stride"],A=0;A<t.length;++A)s.push(["s",A,"=stride[",A,"]|0,n",A,"=shape[",A,"]|0"].join(""));var l=new Array(t.length),c=[];for(A=0;A<t.length;++A)0!==(h=t[A])&&(0===c.length?l[h]="1":l[h]=c.join("*"),c.push("n"+h));var u=-1,g=-1;for(A=0;A<t.length;++A){var h,d=t[A];0!==d&&(u>0?s.push(["d",d,"=s",d,"-d",u,"*n",u].join("")):s.push(["d",d,"=s",d].join("")),u=d),0!=(h=t.length-1-A)&&(g>0?s.push(["e",h,"=s",h,"-e",g,"*n",g,",f",h,"=",l[h],"-f",g,"*n",g].join("")):s.push(["e",h,"=s",h,",f",h,"=",l[h]].join("")),g=h)}n.push("var "+s.join(","));var f=["0","n0-1","data","offset"].concat(a(t.length));n.push(["if(n0<=",i,"){","insertionSort(",f.join(","),")}else{","quickSort(",f.join(","),")}"].join("")),n.push("}return "+r);var p=new Function("insertionSort","quickSort",n.join("\n")),I=function(t,e){var n=["'use strict'"],r=["ndarrayInsertionSort",t.join("d"),e].join(""),i=["left","right","data","offset"].concat(a(t.length)),s=o(e),A=["i,j,cptr,ptr=left*s0+offset"];if(t.length>1){for(var l=[],c=1;c<t.length;++c)A.push("i"+c),l.push("n"+c);s?A.push("scratch=malloc("+l.join("*")+")"):A.push("scratch=new Array("+l.join("*")+")"),A.push("dptr","sptr","a","b")}else A.push("scratch");function u(t){return"generic"===e?["data.get(",t,")"].join(""):["data[",t,"]"].join("")}function g(t,n){return"generic"===e?["data.set(",t,",",n,")"].join(""):["data[",t,"]=",n].join("")}if(n.push(["function ",r,"(",i.join(","),"){var ",A.join(",")].join(""),"for(i=left+1;i<=right;++i){","j=i;ptr+=s0","cptr=ptr"),t.length>1){for(n.push("dptr=0;sptr=ptr"),c=t.length-1;c>=0;--c)0!==(h=t[c])&&n.push(["for(i",h,"=0;i",h,"<n",h,";++i",h,"){"].join(""));for(n.push("scratch[dptr++]=",u("sptr")),c=0;c<t.length;++c)0!==(h=t[c])&&n.push("sptr+=d"+h,"}");for(n.push("__g:while(j--\x3eleft){","dptr=0","sptr=cptr-s0"),c=1;c<t.length;++c)1===c&&n.push("__l:"),n.push(["for(i",c,"=0;i",c,"<n",c,";++i",c,"){"].join(""));for(n.push(["a=",u("sptr"),"\nb=scratch[dptr]\nif(a<b){break __g}\nif(a>b){break __l}"].join("")),c=t.length-1;c>=1;--c)n.push("sptr+=e"+c,"dptr+=f"+c,"}");for(n.push("dptr=cptr;sptr=cptr-s0"),c=t.length-1;c>=0;--c)0!==(h=t[c])&&n.push(["for(i",h,"=0;i",h,"<n",h,";++i",h,"){"].join(""));for(n.push(g("dptr",u("sptr"))),c=0;c<t.length;++c)0!==(h=t[c])&&n.push(["dptr+=d",h,";sptr+=d",h].join(""),"}");for(n.push("cptr-=s0\n}"),n.push("dptr=cptr;sptr=0"),c=t.length-1;c>=0;--c)0!==(h=t[c])&&n.push(["for(i",h,"=0;i",h,"<n",h,";++i",h,"){"].join(""));for(n.push(g("dptr","scratch[sptr++]")),c=0;c<t.length;++c){var h;0!==(h=t[c])&&n.push("dptr+=d"+h,"}")}}else n.push("scratch="+u("ptr"),"while((j--\x3eleft)&&("+u("cptr-s0")+">scratch)){",g("cptr",u("cptr-s0")),"cptr-=s0","}",g("cptr","scratch"));return n.push("}"),t.length>1&&s&&n.push("free(scratch)"),n.push("} return "+r),s?new Function("malloc","free",n.join("\n"))(s[0],s[1]):new Function(n.join("\n"))()}(t,e),C=function(t,e,n){var r=["'use strict'"],s=["ndarrayQuickSort",t.join("d"),e].join(""),A=["left","right","data","offset"].concat(a(t.length)),l=o(e),c=0;r.push(["function ",s,"(",A.join(","),"){"].join(""));var u=["sixth=((right-left+1)/6)|0","index1=left+sixth","index5=right-sixth","index3=(left+right)>>1","index2=index3-sixth","index4=index3+sixth","el1=index1","el2=index2","el3=index3","el4=index4","el5=index5","less=left+1","great=right-1","pivots_are_equal=true","tmp","tmp0","x","y","z","k","ptr0","ptr1","ptr2","comp_pivot1=0","comp_pivot2=0","comp=0"];if(t.length>1){for(var g=[],h=1;h<t.length;++h)g.push("n"+h),u.push("i"+h);for(h=0;h<8;++h)u.push("b_ptr"+h);u.push("ptr3","ptr4","ptr5","ptr6","ptr7","pivot_ptr","ptr_shift","elementSize="+g.join("*")),l?u.push("pivot1=malloc(elementSize)","pivot2=malloc(elementSize)"):u.push("pivot1=new Array(elementSize),pivot2=new Array(elementSize)")}else u.push("pivot1","pivot2");function d(t){return["(offset+",t,"*s0)"].join("")}function f(t){return"generic"===e?["data.get(",t,")"].join(""):["data[",t,"]"].join("")}function p(t,n){return"generic"===e?["data.set(",t,",",n,")"].join(""):["data[",t,"]=",n].join("")}function I(e,n,i){if(1===e.length)r.push("ptr0="+d(e[0]));else for(var o=0;o<e.length;++o)r.push(["b_ptr",o,"=s0*",e[o]].join(""));for(n&&r.push("pivot_ptr=0"),r.push("ptr_shift=offset"),o=t.length-1;o>=0;--o)0!==(a=t[o])&&r.push(["for(i",a,"=0;i",a,"<n",a,";++i",a,"){"].join(""));if(e.length>1)for(o=0;o<e.length;++o)r.push(["ptr",o,"=b_ptr",o,"+ptr_shift"].join(""));for(r.push(i),n&&r.push("++pivot_ptr"),o=0;o<t.length;++o){var a;0!==(a=t[o])&&(e.length>1?r.push("ptr_shift+=d"+a):r.push("ptr0+=d"+a),r.push("}"))}}function C(e,n,i,o){if(1===n.length)r.push("ptr0="+d(n[0]));else{for(var a=0;a<n.length;++a)r.push(["b_ptr",a,"=s0*",n[a]].join(""));r.push("ptr_shift=offset")}for(i&&r.push("pivot_ptr=0"),e&&r.push(e+":"),a=1;a<t.length;++a)r.push(["for(i",a,"=0;i",a,"<n",a,";++i",a,"){"].join(""));if(n.length>1)for(a=0;a<n.length;++a)r.push(["ptr",a,"=b_ptr",a,"+ptr_shift"].join(""));for(r.push(o),a=t.length-1;a>=1;--a)i&&r.push("pivot_ptr+=f"+a),n.length>1?r.push("ptr_shift+=e"+a):r.push("ptr0+=e"+a),r.push("}")}function m(){t.length>1&&l&&r.push("free(pivot1)","free(pivot2)")}function y(e,n){var i="el"+e,o="el"+n;if(t.length>1){var a="__l"+ ++c;C(a,[i,o],!1,["comp=",f("ptr0"),"-",f("ptr1"),"\n","if(comp>0){tmp0=",i,";",i,"=",o,";",o,"=tmp0;break ",a,"}\n","if(comp<0){break ",a,"}"].join(""))}else r.push(["if(",f(d(i)),">",f(d(o)),"){tmp0=",i,";",i,"=",o,";",o,"=tmp0}"].join(""))}function E(e,n){t.length>1?I([e,n],!1,p("ptr0",f("ptr1"))):r.push(p(d(e),f(d(n))))}function v(e,n,i){if(t.length>1){var o="__l"+ ++c;C(o,[n],!0,[e,"=",f("ptr0"),"-pivot",i,"[pivot_ptr]\n","if(",e,"!==0){break ",o,"}"].join(""))}else r.push([e,"=",f(d(n)),"-pivot",i].join(""))}function B(e,n){t.length>1?I([e,n],!1,["tmp=",f("ptr0"),"\n",p("ptr0",f("ptr1")),"\n",p("ptr1","tmp")].join("")):r.push(["ptr0=",d(e),"\n","ptr1=",d(n),"\n","tmp=",f("ptr0"),"\n",p("ptr0",f("ptr1")),"\n",p("ptr1","tmp")].join(""))}function b(e,n,i){t.length>1?(I([e,n,i],!1,["tmp=",f("ptr0"),"\n",p("ptr0",f("ptr1")),"\n",p("ptr1",f("ptr2")),"\n",p("ptr2","tmp")].join("")),r.push("++"+n,"--"+i)):r.push(["ptr0=",d(e),"\n","ptr1=",d(n),"\n","ptr2=",d(i),"\n","++",n,"\n","--",i,"\n","tmp=",f("ptr0"),"\n",p("ptr0",f("ptr1")),"\n",p("ptr1",f("ptr2")),"\n",p("ptr2","tmp")].join(""))}function Q(t,e){B(t,e),r.push("--"+e)}function w(e,n,i){t.length>1?I([e,n],!0,[p("ptr0",f("ptr1")),"\n",p("ptr1",["pivot",i,"[pivot_ptr]"].join(""))].join("")):r.push(p(d(e),f(d(n))),p(d(n),"pivot"+i))}function x(e,n){r.push(["if((",n,"-",e,")<=",i,"){\n","insertionSort(",e,",",n,",data,offset,",a(t.length).join(","),")\n","}else{\n",s,"(",e,",",n,",data,offset,",a(t.length).join(","),")\n","}"].join(""))}function _(e,n,i){t.length>1?(r.push(["__l",++c,":while(true){"].join("")),I([e],!0,["if(",f("ptr0"),"!==pivot",n,"[pivot_ptr]){break __l",c,"}"].join("")),r.push(i,"}")):r.push(["while(",f(d(e)),"===pivot",n,"){",i,"}"].join(""))}return r.push("var "+u.join(",")),y(1,2),y(4,5),y(1,3),y(2,3),y(1,4),y(3,4),y(2,5),y(2,3),y(4,5),t.length>1?I(["el1","el2","el3","el4","el5","index1","index3","index5"],!0,["pivot1[pivot_ptr]=",f("ptr1"),"\n","pivot2[pivot_ptr]=",f("ptr3"),"\n","pivots_are_equal=pivots_are_equal&&(pivot1[pivot_ptr]===pivot2[pivot_ptr])\n","x=",f("ptr0"),"\n","y=",f("ptr2"),"\n","z=",f("ptr4"),"\n",p("ptr5","x"),"\n",p("ptr6","y"),"\n",p("ptr7","z")].join("")):r.push(["pivot1=",f(d("el2")),"\n","pivot2=",f(d("el4")),"\n","pivots_are_equal=pivot1===pivot2\n","x=",f(d("el1")),"\n","y=",f(d("el3")),"\n","z=",f(d("el5")),"\n",p(d("index1"),"x"),"\n",p(d("index3"),"y"),"\n",p(d("index5"),"z")].join("")),E("index2","left"),E("index4","right"),r.push("if(pivots_are_equal){"),r.push("for(k=less;k<=great;++k){"),v("comp","k",1),r.push("if(comp===0){continue}"),r.push("if(comp<0){"),r.push("if(k!==less){"),B("k","less"),r.push("}"),r.push("++less"),r.push("}else{"),r.push("while(true){"),v("comp","great",1),r.push("if(comp>0){"),r.push("great--"),r.push("}else if(comp<0){"),b("k","less","great"),r.push("break"),r.push("}else{"),Q("k","great"),r.push("break"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),r.push("}else{"),r.push("for(k=less;k<=great;++k){"),v("comp_pivot1","k",1),r.push("if(comp_pivot1<0){"),r.push("if(k!==less){"),B("k","less"),r.push("}"),r.push("++less"),r.push("}else{"),v("comp_pivot2","k",2),r.push("if(comp_pivot2>0){"),r.push("while(true){"),v("comp","great",2),r.push("if(comp>0){"),r.push("if(--great<k){break}"),r.push("continue"),r.push("}else{"),v("comp","great",1),r.push("if(comp<0){"),b("k","less","great"),r.push("}else{"),Q("k","great"),r.push("}"),r.push("break"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),w("left","(less-1)",1),w("right","(great+1)",2),x("left","(less-2)"),x("(great+2)","right"),r.push("if(pivots_are_equal){"),m(),r.push("return"),r.push("}"),r.push("if(less<index1&&great>index5){"),_("less",1,"++less"),_("great",2,"--great"),r.push("for(k=less;k<=great;++k){"),v("comp_pivot1","k",1),r.push("if(comp_pivot1===0){"),r.push("if(k!==less){"),B("k","less"),r.push("}"),r.push("++less"),r.push("}else{"),v("comp_pivot2","k",2),r.push("if(comp_pivot2===0){"),r.push("while(true){"),v("comp","great",2),r.push("if(comp===0){"),r.push("if(--great<k){break}"),r.push("continue"),r.push("}else{"),v("comp","great",1),r.push("if(comp<0){"),b("k","less","great"),r.push("}else{"),Q("k","great"),r.push("}"),r.push("break"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),m(),x("less","great"),r.push("}return "+s),t.length>1&&l?new Function("insertionSort","malloc","free",r.join("\n"))(n,l[0],l[1]):new Function("insertionSort",r.join("\n"))(n)}(t,e,I);return p(I,C)}},{"typedarray-pool":545}],448:[function(t,e,n){"use strict";var r=t("./lib/compile_sort.js"),i={};e.exports=function(t){var e=t.order,n=t.dtype,o=[e,n].join(":"),a=i[o];return a||(i[o]=a=r(e,n)),a(t),t}},{"./lib/compile_sort.js":447}],449:[function(t,e,n){"use strict";var r=t("ndarray-linear-interpolate"),i=t("cwise/lib/wrapper")({args:["index","array","scalar","scalar","scalar"],pre:{body:"{this_warped=new Array(_inline_3_arg4_)}",args:[{name:"_inline_3_arg0_",lvalue:!1,rvalue:!1,count:0},{name:"_inline_3_arg1_",lvalue:!1,rvalue:!1,count:0},{name:"_inline_3_arg2_",lvalue:!1,rvalue:!1,count:0},{name:"_inline_3_arg3_",lvalue:!1,rvalue:!1,count:0},{name:"_inline_3_arg4_",lvalue:!1,rvalue:!0,count:1}],thisVars:["this_warped"],localVars:[]},body:{body:"{_inline_4_arg2_(this_warped,_inline_4_arg0_),_inline_4_arg1_=_inline_4_arg3_.apply(void 0,this_warped)}",args:[{name:"_inline_4_arg0_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_4_arg1_",lvalue:!0,rvalue:!1,count:1},{name:"_inline_4_arg2_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_4_arg3_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_4_arg4_",lvalue:!1,rvalue:!1,count:0}],thisVars:["this_warped"],localVars:[]},post:{body:"{}",args:[],thisVars:[],localVars:[]},debug:!1,funcName:"warpND",blockSize:64}),o=t("cwise/lib/wrapper")({args:["index","array","scalar","scalar","scalar"],pre:{body:"{this_warped=[0]}",args:[],thisVars:["this_warped"],localVars:[]},body:{body:"{_inline_7_arg2_(this_warped,_inline_7_arg0_),_inline_7_arg1_=_inline_7_arg3_(_inline_7_arg4_,this_warped[0])}",args:[{name:"_inline_7_arg0_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_7_arg1_",lvalue:!0,rvalue:!1,count:1},{name:"_inline_7_arg2_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_7_arg3_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_7_arg4_",lvalue:!1,rvalue:!0,count:1}],thisVars:["this_warped"],localVars:[]},post:{body:"{}",args:[],thisVars:[],localVars:[]},debug:!1,funcName:"warp1D",blockSize:64}),a=t("cwise/lib/wrapper")({args:["index","array","scalar","scalar","scalar"],pre:{body:"{this_warped=[0,0]}",args:[],thisVars:["this_warped"],localVars:[]},body:{body:"{_inline_10_arg2_(this_warped,_inline_10_arg0_),_inline_10_arg1_=_inline_10_arg3_(_inline_10_arg4_,this_warped[0],this_warped[1])}",args:[{name:"_inline_10_arg0_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_10_arg1_",lvalue:!0,rvalue:!1,count:1},{name:"_inline_10_arg2_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_10_arg3_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_10_arg4_",lvalue:!1,rvalue:!0,count:1}],thisVars:["this_warped"],localVars:[]},post:{body:"{}",args:[],thisVars:[],localVars:[]},debug:!1,funcName:"warp2D",blockSize:64}),s=t("cwise/lib/wrapper")({args:["index","array","scalar","scalar","scalar"],pre:{body:"{this_warped=[0,0,0]}",args:[],thisVars:["this_warped"],localVars:[]},body:{body:"{_inline_13_arg2_(this_warped,_inline_13_arg0_),_inline_13_arg1_=_inline_13_arg3_(_inline_13_arg4_,this_warped[0],this_warped[1],this_warped[2])}",args:[{name:"_inline_13_arg0_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_13_arg1_",lvalue:!0,rvalue:!1,count:1},{name:"_inline_13_arg2_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_13_arg3_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_13_arg4_",lvalue:!1,rvalue:!0,count:1}],thisVars:["this_warped"],localVars:[]},post:{body:"{}",args:[],thisVars:[],localVars:[]},debug:!1,funcName:"warp3D",blockSize:64});e.exports=function(t,e,n){switch(e.shape.length){case 1:o(t,n,r.d1,e);break;case 2:a(t,n,r.d2,e);break;case 3:s(t,n,r.d3,e);break;default:i(t,n,r.bind(void 0,e),e.shape.length)}return t}},{"cwise/lib/wrapper":149,"ndarray-linear-interpolate":443}],450:[function(t,e,n){var r=t("iota-array"),i=t("is-buffer"),o="undefined"!=typeof Float64Array;function a(t,e){return t[0]-e[0]}function s(){var t,e=this.stride,n=new Array(e.length);for(t=0;t<n.length;++t)n[t]=[Math.abs(e[t]),t];n.sort(a);var r=new Array(n.length);for(t=0;t<r.length;++t)r[t]=n[t][1];return r}function A(t,e){var n=["View",e,"d",t].join("");e<0&&(n="View_Nil"+t);var i="generic"===t;if(-1===e){var o="function "+n+"(a){this.data=a;};var proto="+n+".prototype;proto.dtype='"+t+"';proto.index=function(){return -1};proto.size=0;proto.dimension=-1;proto.shape=proto.stride=proto.order=[];proto.lo=proto.hi=proto.transpose=proto.step=function(){return new "+n+"(this.data);};proto.get=proto.set=function(){};proto.pick=function(){return null};return function construct_"+n+"(a){return new "+n+"(a);}";return new Function(o)()}if(0===e)return o="function "+n+"(a,d) {this.data = a;this.offset = d};var proto="+n+".prototype;proto.dtype='"+t+"';proto.index=function(){return this.offset};proto.dimension=0;proto.size=1;proto.shape=proto.stride=proto.order=[];proto.lo=proto.hi=proto.transpose=proto.step=function "+n+"_copy() {return new "+n+"(this.data,this.offset)};proto.pick=function "+n+"_pick(){return TrivialArray(this.data);};proto.valueOf=proto.get=function "+n+"_get(){return "+(i?"this.data.get(this.offset)":"this.data[this.offset]")+"};proto.set=function "+n+"_set(v){return "+(i?"this.data.set(this.offset,v)":"this.data[this.offset]=v")+"};return function construct_"+n+"(a,b,c,d){return new "+n+"(a,d)}",new Function("TrivialArray",o)(l[t][0]);o=["'use strict'"];var a=r(e),A=a.map(function(t){return"i"+t}),c="this.offset+"+a.map(function(t){return"this.stride["+t+"]*i"+t}).join("+"),u=a.map(function(t){return"b"+t}).join(","),g=a.map(function(t){return"c"+t}).join(",");o.push("function "+n+"(a,"+u+","+g+",d){this.data=a","this.shape=["+u+"]","this.stride=["+g+"]","this.offset=d|0}","var proto="+n+".prototype","proto.dtype='"+t+"'","proto.dimension="+e),o.push("Object.defineProperty(proto,'size',{get:function "+n+"_size(){return "+a.map(function(t){return"this.shape["+t+"]"}).join("*"),"}})"),1===e?o.push("proto.order=[0]"):(o.push("Object.defineProperty(proto,'order',{get:"),e<4?(o.push("function "+n+"_order(){"),2===e?o.push("return (Math.abs(this.stride[0])>Math.abs(this.stride[1]))?[1,0]:[0,1]}})"):3===e&&o.push("var s0=Math.abs(this.stride[0]),s1=Math.abs(this.stride[1]),s2=Math.abs(this.stride[2]);if(s0>s1){if(s1>s2){return [2,1,0];}else if(s0>s2){return [1,2,0];}else{return [1,0,2];}}else if(s0>s2){return [2,0,1];}else if(s2>s1){return [0,1,2];}else{return [0,2,1];}}})")):o.push("ORDER})")),o.push("proto.set=function "+n+"_set("+A.join(",")+",v){"),i?o.push("return this.data.set("+c+",v)}"):o.push("return this.data["+c+"]=v}"),o.push("proto.get=function "+n+"_get("+A.join(",")+"){"),i?o.push("return this.data.get("+c+")}"):o.push("return this.data["+c+"]}"),o.push("proto.index=function "+n+"_index(",A.join(),"){return "+c+"}"),o.push("proto.hi=function "+n+"_hi("+A.join(",")+"){return new "+n+"(this.data,"+a.map(function(t){return["(typeof i",t,"!=='number'||i",t,"<0)?this.shape[",t,"]:i",t,"|0"].join("")}).join(",")+","+a.map(function(t){return"this.stride["+t+"]"}).join(",")+",this.offset)}");var h=a.map(function(t){return"a"+t+"=this.shape["+t+"]"}),d=a.map(function(t){return"c"+t+"=this.stride["+t+"]"});o.push("proto.lo=function "+n+"_lo("+A.join(",")+"){var b=this.offset,d=0,"+h.join(",")+","+d.join(","));for(var f=0;f<e;++f)o.push("if(typeof i"+f+"==='number'&&i"+f+">=0){d=i"+f+"|0;b+=c"+f+"*d;a"+f+"-=d}");for(o.push("return new "+n+"(this.data,"+a.map(function(t){return"a"+t}).join(",")+","+a.map(function(t){return"c"+t}).join(",")+",b)}"),o.push("proto.step=function "+n+"_step("+A.join(",")+"){var "+a.map(function(t){return"a"+t+"=this.shape["+t+"]"}).join(",")+","+a.map(function(t){return"b"+t+"=this.stride["+t+"]"}).join(",")+",c=this.offset,d=0,ceil=Math.ceil"),f=0;f<e;++f)o.push("if(typeof i"+f+"==='number'){d=i"+f+"|0;if(d<0){c+=b"+f+"*(a"+f+"-1);a"+f+"=ceil(-a"+f+"/d)}else{a"+f+"=ceil(a"+f+"/d)}b"+f+"*=d}");o.push("return new "+n+"(this.data,"+a.map(function(t){return"a"+t}).join(",")+","+a.map(function(t){return"b"+t}).join(",")+",c)}");var p=new Array(e),I=new Array(e);for(f=0;f<e;++f)p[f]="a[i"+f+"]",I[f]="b[i"+f+"]";for(o.push("proto.transpose=function "+n+"_transpose("+A+"){"+A.map(function(t,e){return t+"=("+t+"===undefined?"+e+":"+t+"|0)"}).join(";"),"var a=this.shape,b=this.stride;return new "+n+"(this.data,"+p.join(",")+","+I.join(",")+",this.offset)}"),o.push("proto.pick=function "+n+"_pick("+A+"){var a=[],b=[],c=this.offset"),f=0;f<e;++f)o.push("if(typeof i"+f+"==='number'&&i"+f+">=0){c=(c+this.stride["+f+"]*i"+f+")|0}else{a.push(this.shape["+f+"]);b.push(this.stride["+f+"])}");return o.push("var ctor=CTOR_LIST[a.length+1];return ctor(this.data,a,b,c)}"),o.push("return function construct_"+n+"(data,shape,stride,offset){return new "+n+"(data,"+a.map(function(t){return"shape["+t+"]"}).join(",")+","+a.map(function(t){return"stride["+t+"]"}).join(",")+",offset)}"),new Function("CTOR_LIST","ORDER",o.join("\n"))(l[t],s)}var l={float32:[],float64:[],int8:[],int16:[],int32:[],uint8:[],uint16:[],uint32:[],array:[],uint8_clamped:[],buffer:[],generic:[]};e.exports=function(t,e,n,r){if(void 0===t)return(0,l.array[0])([]);"number"==typeof t&&(t=[t]),void 0===e&&(e=[t.length]);var a=e.length;if(void 0===n){n=new Array(a);for(var s=a-1,c=1;s>=0;--s)n[s]=c,c*=e[s]}if(void 0===r)for(r=0,s=0;s<a;++s)n[s]<0&&(r-=(e[s]-1)*n[s]);for(var u=function(t){if(i(t))return"buffer";if(o)switch(Object.prototype.toString.call(t)){case"[object Float64Array]":return"float64";case"[object Float32Array]":return"float32";case"[object Int8Array]":return"int8";case"[object Int16Array]":return"int16";case"[object Int32Array]":return"int32";case"[object Uint8Array]":return"uint8";case"[object Uint16Array]":return"uint16";case"[object Uint32Array]":return"uint32";case"[object Uint8ClampedArray]":return"uint8_clamped"}return Array.isArray(t)?"array":"generic"}(t),g=l[u];g.length<=a+1;)g.push(A(u,g.length-1));return(0,g[a+1])(t,e,n,r)}},{"iota-array":416,"is-buffer":418}],451:[function(t,e,n){"use strict";var r=t("double-bits"),i=Math.pow(2,-1074),o=-1>>>0;e.exports=function(t,e){if(isNaN(t)||isNaN(e))return NaN;if(t===e)return t;if(0===t)return e<0?-i:i;var n=r.hi(t),a=r.lo(t);return e>t==t>0?a===o?(n+=1,a=0):a+=1:0===a?(a=o,n-=1):a-=1,r.pack(a,n)}},{"double-bits":167}],452:[function(t,e,n){var r=Math.PI,i=l(120);function o(t,e,n,r){return["C",t,e,n,r,n,r]}function a(t,e,n,r,i,o){return["C",t/3+2/3*n,e/3+2/3*r,i/3+2/3*n,o/3+2/3*r,i,o]}function s(t,e,n,o,a,l,c,u,g,h){if(h)B=h[0],b=h[1],E=h[2],v=h[3];else{var d=A(t,e,-a);t=d.x,e=d.y;var f=(t-(u=(d=A(u,g,-a)).x))/2,p=(e-(g=d.y))/2,I=f*f/(n*n)+p*p/(o*o);I>1&&(n*=I=Math.sqrt(I),o*=I);var C=n*n,m=o*o,y=(l==c?-1:1)*Math.sqrt(Math.abs((C*m-C*p*p-m*f*f)/(C*p*p+m*f*f)));y==1/0&&(y=1);var E=y*n*p/o+(t+u)/2,v=y*-o*f/n+(e+g)/2,B=Math.asin(((e-v)/o).toFixed(9)),b=Math.asin(((g-v)/o).toFixed(9));(B=t<E?r-B:B)<0&&(B=2*r+B),(b=u<E?r-b:b)<0&&(b=2*r+b),c&&B>b&&(B-=2*r),!c&&b>B&&(b-=2*r)}if(Math.abs(b-B)>i){var Q=b,w=u,x=g;b=B+i*(c&&b>B?1:-1);var _=s(u=E+n*Math.cos(b),g=v+o*Math.sin(b),n,o,a,0,c,w,x,[b,Q,E,v])}var D=Math.tan((b-B)/4),M=4/3*n*D,k=4/3*o*D,S=[2*t-(t+M*Math.sin(B)),2*e-(e-k*Math.cos(B)),u+M*Math.sin(b),g-k*Math.cos(b),u,g];if(h)return S;_&&(S=S.concat(_));for(var L=0;L<S.length;){var R=A(S[L],S[L+1],a);S[L++]=R.x,S[L++]=R.y}return S}function A(t,e,n){return{x:t*Math.cos(n)-e*Math.sin(n),y:t*Math.sin(n)+e*Math.cos(n)}}function l(t){return t*(r/180)}e.exports=function(t){for(var e,n=[],r=0,i=0,A=0,c=0,u=null,g=null,h=0,d=0,f=0,p=t.length;f<p;f++){var I=t[f],C=I[0];switch(C){case"M":A=I[1],c=I[2];break;case"A":(I=s(h,d,I[1],I[2],l(I[3]),I[4],I[5],I[6],I[7])).unshift("C"),I.length>7&&(n.push(I.splice(0,7)),I.unshift("C"));break;case"S":var m=h,y=d;"C"!=e&&"S"!=e||(m+=m-r,y+=y-i),I=["C",m,y,I[1],I[2],I[3],I[4]];break;case"T":"Q"==e||"T"==e?(u=2*h-u,g=2*d-g):(u=h,g=d),I=a(h,d,u,g,I[1],I[2]);break;case"Q":u=I[1],g=I[2],I=a(h,d,I[1],I[2],I[3],I[4]);break;case"L":I=o(h,d,I[1],I[2]);break;case"H":I=o(h,d,I[1],d);break;case"V":I=o(h,d,h,I[1]);break;case"Z":I=o(h,d,A,c)}e=C,h=I[I.length-2],d=I[I.length-1],I.length>4?(r=I[I.length-4],i=I[I.length-3]):(r=h,i=d),n.push(I)}return n}},{}],453:[function(t,e,n){n.vertexNormals=function(t,e,n){for(var r=e.length,i=new Array(r),o=void 0===n?1e-6:n,a=0;a<r;++a)i[a]=[0,0,0];for(a=0;a<t.length;++a)for(var s=t[a],A=0,l=s[s.length-1],c=s[0],u=0;u<s.length;++u){A=l,l=c,c=s[(u+1)%s.length];for(var g=e[A],h=e[l],d=e[c],f=new Array(3),p=0,I=new Array(3),C=0,m=0;m<3;++m)f[m]=g[m]-h[m],p+=f[m]*f[m],I[m]=d[m]-h[m],C+=I[m]*I[m];if(p*C>o){var y=i[l],E=1/Math.sqrt(p*C);for(m=0;m<3;++m){var v=(m+1)%3,B=(m+2)%3;y[m]+=E*(I[v]*f[B]-I[B]*f[v])}}}for(a=0;a<r;++a){y=i[a];var b=0;for(m=0;m<3;++m)b+=y[m]*y[m];if(b>o)for(E=1/Math.sqrt(b),m=0;m<3;++m)y[m]*=E;else for(m=0;m<3;++m)y[m]=0}return i},n.faceNormals=function(t,e,n){for(var r=t.length,i=new Array(r),o=void 0===n?1e-6:n,a=0;a<r;++a){for(var s=t[a],A=new Array(3),l=0;l<3;++l)A[l]=e[s[l]];var c=new Array(3),u=new Array(3);for(l=0;l<3;++l)c[l]=A[1][l]-A[0][l],u[l]=A[2][l]-A[0][l];var g=new Array(3),h=0;for(l=0;l<3;++l){var d=(l+1)%3,f=(l+2)%3;g[l]=c[d]*u[f]-c[f]*u[d],h+=g[l]*g[l]}for(h=h>o?1/Math.sqrt(h):0,l=0;l<3;++l)g[l]*=h;i[a]=g}return i}},{}],454:[function(t,e,n){"use strict";var r=Object.getOwnPropertySymbols,i=Object.prototype.hasOwnProperty,o=Object.prototype.propertyIsEnumerable;e.exports=function(){try{if(!Object.assign)return!1;var t=new String("abc");if(t[5]="de","5"===Object.getOwnPropertyNames(t)[0])return!1;for(var e={},n=0;n<10;n++)e["_"+String.fromCharCode(n)]=n;if("0123456789"!==Object.getOwnPropertyNames(e).map(function(t){return e[t]}).join(""))return!1;var r={};return"abcdefghijklmnopqrst".split("").forEach(function(t){r[t]=t}),"abcdefghijklmnopqrst"===Object.keys(Object.assign({},r)).join("")}catch(t){return!1}}()?Object.assign:function(t,e){for(var n,a,s=function(t){if(null==t)throw new TypeError("Object.assign cannot be called with null or undefined");return Object(t)}(t),A=1;A<arguments.length;A++){for(var l in n=Object(arguments[A]))i.call(n,l)&&(s[l]=n[l]);if(r){a=r(n);for(var c=0;c<a.length;c++)o.call(n,a[c])&&(s[a[c]]=n[a[c]])}}return s}},{}],455:[function(t,e,n){"use strict";e.exports=function(t,e,n,r,i,o,a,s,A,l){var c=e+o+l;if(u>0){var u=Math.sqrt(c+1);t[0]=.5*(a-A)/u,t[1]=.5*(s-r)/u,t[2]=.5*(n-o)/u,t[3]=.5*u}else{var g=Math.max(e,o,l),u=Math.sqrt(2*g-c+1);e>=g?(t[0]=.5*u,t[1]=.5*(i+n)/u,t[2]=.5*(s+r)/u,t[3]=.5*(a-A)/u):o>=g?(t[0]=.5*(n+i)/u,t[1]=.5*u,t[2]=.5*(A+a)/u,t[3]=.5*(s-r)/u):(t[0]=.5*(r+s)/u,t[1]=.5*(a+A)/u,t[2]=.5*u,t[3]=.5*(n-i)/u)}return t}},{}],456:[function(t,e,n){"use strict";e.exports=function(t){var e=(t=t||{}).center||[0,0,0],n=t.rotation||[0,0,0,1],r=t.radius||1;e=[].slice.call(e,0,3),c(n=[].slice.call(n,0,4),n);var i=new u(n,e,Math.log(r));return i.setDistanceLimits(t.zoomMin,t.zoomMax),("eye"in t||"up"in t)&&i.lookAt(0,t.eye,t.center,t.up),i};var r=t("filtered-vector"),i=t("gl-mat4/lookAt"),o=t("gl-mat4/fromQuat"),a=t("gl-mat4/invert"),s=t("./lib/quatFromFrame");function A(t,e,n){return Math.sqrt(Math.pow(t,2)+Math.pow(e,2)+Math.pow(n,2))}function l(t,e,n,r){return Math.sqrt(Math.pow(t,2)+Math.pow(e,2)+Math.pow(n,2)+Math.pow(r,2))}function c(t,e){var n=e[0],r=e[1],i=e[2],o=e[3],a=l(n,r,i,o);a>1e-6?(t[0]=n/a,t[1]=r/a,t[2]=i/a,t[3]=o/a):(t[0]=t[1]=t[2]=0,t[3]=1)}function u(t,e,n){this.radius=r([n]),this.center=r(e),this.rotation=r(t),this.computedRadius=this.radius.curve(0),this.computedCenter=this.center.curve(0),this.computedRotation=this.rotation.curve(0),this.computedUp=[.1,0,0],this.computedEye=[.1,0,0],this.computedMatrix=[.1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],this.recalcMatrix(0)}var g=u.prototype;g.lastT=function(){return Math.max(this.radius.lastT(),this.center.lastT(),this.rotation.lastT())},g.recalcMatrix=function(t){this.radius.curve(t),this.center.curve(t),this.rotation.curve(t);var e=this.computedRotation;c(e,e);var n=this.computedMatrix;o(n,e);var r=this.computedCenter,i=this.computedEye,a=this.computedUp,s=Math.exp(this.computedRadius[0]);i[0]=r[0]+s*n[2],i[1]=r[1]+s*n[6],i[2]=r[2]+s*n[10],a[0]=n[1],a[1]=n[5],a[2]=n[9];for(var A=0;A<3;++A){for(var l=0,u=0;u<3;++u)l+=n[A+4*u]*i[u];n[12+A]=-l}},g.getMatrix=function(t,e){this.recalcMatrix(t);var n=this.computedMatrix;if(e){for(var r=0;r<16;++r)e[r]=n[r];return e}return n},g.idle=function(t){this.center.idle(t),this.radius.idle(t),this.rotation.idle(t)},g.flush=function(t){this.center.flush(t),this.radius.flush(t),this.rotation.flush(t)},g.pan=function(t,e,n,r){e=e||0,n=n||0,r=r||0,this.recalcMatrix(t);var i=this.computedMatrix,o=i[1],a=i[5],s=i[9],l=A(o,a,s);o/=l,a/=l,s/=l;var c=i[0],u=i[4],g=i[8],h=c*o+u*a+g*s,d=A(c-=o*h,u-=a*h,g-=s*h);c/=d,u/=d,g/=d;var f=i[2],p=i[6],I=i[10],C=f*o+p*a+I*s,m=f*c+p*u+I*g,y=A(f-=C*o+m*c,p-=C*a+m*u,I-=C*s+m*g);f/=y,p/=y,I/=y;var E=c*e+o*n,v=u*e+a*n,B=g*e+s*n;this.center.move(t,E,v,B);var b=Math.exp(this.computedRadius[0]);b=Math.max(1e-4,b+r),this.radius.set(t,Math.log(b))},g.rotate=function(t,e,n,r){this.recalcMatrix(t),e=e||0,n=n||0;var i=this.computedMatrix,o=i[0],a=i[4],s=i[8],c=i[1],u=i[5],g=i[9],h=i[2],d=i[6],f=i[10],p=e*o+n*c,I=e*a+n*u,C=e*s+n*g,m=-(d*C-f*I),y=-(f*p-h*C),E=-(h*I-d*p),v=Math.sqrt(Math.max(0,1-Math.pow(m,2)-Math.pow(y,2)-Math.pow(E,2))),B=l(m,y,E,v);B>1e-6?(m/=B,y/=B,E/=B,v/=B):(m=y=E=0,v=1);var b=this.computedRotation,Q=b[0],w=b[1],x=b[2],_=b[3],D=Q*v+_*m+w*E-x*y,M=w*v+_*y+x*m-Q*E,k=x*v+_*E+Q*y-w*m,S=_*v-Q*m-w*y-x*E;if(r){m=h,y=d,E=f;var L=Math.sin(r)/A(m,y,E);m*=L,y*=L,E*=L,S=S*(v=Math.cos(e))-(D=D*v+S*m+M*E-k*y)*m-(M=M*v+S*y+k*m-D*E)*y-(k=k*v+S*E+D*y-M*m)*E}var R=l(D,M,k,S);R>1e-6?(D/=R,M/=R,k/=R,S/=R):(D=M=k=0,S=1),this.rotation.set(t,D,M,k,S)},g.lookAt=function(t,e,n,r){this.recalcMatrix(t),n=n||this.computedCenter,e=e||this.computedEye,r=r||this.computedUp;var o=this.computedMatrix;i(o,e,n,r);var a=this.computedRotation;s(a,o[0],o[1],o[2],o[4],o[5],o[6],o[8],o[9],o[10]),c(a,a),this.rotation.set(t,a[0],a[1],a[2],a[3]);for(var A=0,l=0;l<3;++l)A+=Math.pow(n[l]-e[l],2);this.radius.set(t,.5*Math.log(Math.max(A,1e-6))),this.center.set(t,n[0],n[1],n[2])},g.translate=function(t,e,n,r){this.center.move(t,e||0,n||0,r||0)},g.setMatrix=function(t,e){var n=this.computedRotation;s(n,e[0],e[1],e[2],e[4],e[5],e[6],e[8],e[9],e[10]),c(n,n),this.rotation.set(t,n[0],n[1],n[2],n[3]);var r=this.computedMatrix;a(r,e);var i=r[15];if(Math.abs(i)>1e-6){var o=r[12]/i,A=r[13]/i,l=r[14]/i;this.recalcMatrix(t);var u=Math.exp(this.computedRadius[0]);this.center.set(t,o-r[2]*u,A-r[6]*u,l-r[10]*u),this.radius.idle(t)}else this.center.idle(t),this.radius.idle(t)},g.setDistance=function(t,e){e>0&&this.radius.set(t,Math.log(e))},g.setDistanceLimits=function(t,e){t=t>0?Math.log(t):-1/0,e=e>0?Math.log(e):1/0,e=Math.max(e,t),this.radius.bounds[0][0]=t,this.radius.bounds[1][0]=e},g.getDistanceLimits=function(t){var e=this.radius.bounds;return t?(t[0]=Math.exp(e[0][0]),t[1]=Math.exp(e[1][0]),t):[Math.exp(e[0][0]),Math.exp(e[1][0])]},g.toJSON=function(){return this.recalcMatrix(this.lastT()),{center:this.computedCenter.slice(),rotation:this.computedRotation.slice(),distance:Math.log(this.computedRadius[0]),zoomMin:this.radius.bounds[0][0],zoomMax:this.radius.bounds[1][0]}},g.fromJSON=function(t){var e=this.lastT(),n=t.center;n&&this.center.set(e,n[0],n[1],n[2]);var r=t.rotation;r&&this.rotation.set(e,r[0],r[1],r[2],r[3]);var i=t.distance;i&&i>0&&this.radius.set(e,Math.log(i)),this.setDistanceLimits(t.zoomMin,t.zoomMax)}},{"./lib/quatFromFrame":455,"filtered-vector":226,"gl-mat4/fromQuat":262,"gl-mat4/invert":265,"gl-mat4/lookAt":266}],457:[function(t,e,n){"use strict";var r=t("repeat-string");e.exports=function(t,e,n){return r(n=void 0!==n?n+"":" ",e)+t}},{"repeat-string":503}],458:[function(t,e,n){"use strict";function r(t,e){if("string"!=typeof t)return[t];var n=[t];"string"==typeof e||Array.isArray(e)?e={brackets:e}:e||(e={});var r=e.brackets?Array.isArray(e.brackets)?e.brackets:[e.brackets]:["{}","[]","()"],i=e.escape||"___",o=!!e.flat;r.forEach(function(t){var e=new RegExp(["\\",t[0],"[^\\",t[0],"\\",t[1],"]*\\",t[1]].join("")),r=[];function o(e,o,a){var s=n.push(e.slice(t[0].length,-t[1].length))-1;return r.push(s),i+s+i}n.forEach(function(t,r){for(var i,a=0;t!=i;)if(i=t,t=t.replace(e,o),a++>1e4)throw Error("References have circular dependency. Please, check them.");n[r]=t}),r=r.reverse(),n=n.map(function(e){return r.forEach(function(n){e=e.replace(new RegExp("(\\"+i+n+"\\"+i+")","g"),t[0]+"$1"+t[1])}),e})});var a=new RegExp("\\"+i+"([0-9]+)\\"+i);return o?n:function t(e,n,r){for(var i,o=[],s=0;i=a.exec(e);){if(s++>1e4)throw Error("Circular references in parenthesis");o.push(e.slice(0,i.index)),o.push(t(n[i[1]],n)),e=e.slice(i.index+i[0].length)}return o.push(e),o}(n[0],n)}function i(t,e){if(e&&e.flat){var n,r=e&&e.escape||"___",i=t[0];if(!i)return"";for(var o=new RegExp("\\"+r+"([0-9]+)\\"+r),a=0;i!=n;){if(a++>1e4)throw Error("Circular references in "+t);n=i,i=i.replace(o,s)}return i}return t.reduce(function t(e,n){return Array.isArray(n)&&(n=n.reduce(t,"")),e+n},"");function s(e,n){if(null==t[n])throw Error("Reference "+n+"is undefined");return t[n]}}function o(t,e){return Array.isArray(t)?i(t,e):r(t,e)}o.parse=r,o.stringify=i,e.exports=o},{}],459:[function(t,e,n){"use strict";var r=t("pick-by-alias");e.exports=function(t){var e;return arguments.length>1&&(t=arguments),"string"==typeof t?t=t.split(/\s/).map(parseFloat):"number"==typeof t&&(t=[t]),t.length&&"number"==typeof t[0]?e=1===t.length?{width:t[0],height:t[0],x:0,y:0}:2===t.length?{width:t[0],height:t[1],x:0,y:0}:{x:t[0],y:t[1],width:t[2]-t[0]||0,height:t[3]-t[1]||0}:t&&(t=r(t,{left:"x l left Left",top:"y t top Top",width:"w width W Width",height:"h height W Width",bottom:"b bottom Bottom",right:"r right Right"}),e={x:t.left||0,y:t.top||0},null==t.width?t.right?e.width=t.right-e.x:e.width=0:e.width=t.width,null==t.height?t.bottom?e.height=t.bottom-e.y:e.height=0:e.height=t.height),e}},{"pick-by-alias":465}],460:[function(t,e,n){e.exports=function(t){var e=[];return t.replace(i,function(t,n,i){var a=n.toLowerCase();for(i=function(t){var e=t.match(o);return e?e.map(Number):[]}(i),"m"==a&&i.length>2&&(e.push([n].concat(i.splice(0,2))),a="l",n="m"==n?"l":"L");;){if(i.length==r[a])return i.unshift(n),e.push(i);if(i.length<r[a])throw new Error("malformed path data");e.push([n].concat(i.splice(0,r[a])))}}),e};var r={a:7,c:6,h:1,l:2,m:2,q:4,s:4,t:2,v:1,z:0},i=/([astvzqmhlc])([^astvzqmhlc]*)/gi,o=/-?[0-9]*\.?[0-9]+(?:e[-+]?\d+)?/gi},{}],461:[function(t,e,n){e.exports=function(t,e){e||(e=[0,""]),t=String(t);var n=parseFloat(t,10);return e[0]=n,e[1]=t.match(/[\d.\-\+]*\s*(.*)/)[1]||"",e}},{}],462:[function(t,e,n){(function(t){(function(){var n,r,i,o,a,s;"undefined"!=typeof performance&&null!==performance&&performance.now?e.exports=function(){return performance.now()}:null!=t&&t.hrtime?(e.exports=function(){return(n()-a)/1e6},r=t.hrtime,o=(n=function(){var t;return 1e9*(t=r())[0]+t[1]})(),s=1e9*t.uptime(),a=o-s):Date.now?(e.exports=function(){return Date.now()-i},i=Date.now()):(e.exports=function(){return(new Date).getTime()-i},i=(new Date).getTime())}).call(this)}).call(this,t("_process"))},{_process:482}],463:[function(t,e,n){"use strict";e.exports=function(t){var e=t.length;if(e<r){for(var n=1,o=0;o<e;++o)for(var a=0;a<o;++a)if(t[o]<t[a])n=-n;else if(t[o]===t[a])return 0;return n}for(var s=i.mallocUint8(e),o=0;o<e;++o)s[o]=0;for(var n=1,o=0;o<e;++o)if(!s[o]){var A=1;s[o]=1;for(var a=t[o];a!==o;a=t[a]){if(s[a])return i.freeUint8(s),0;A+=1,s[a]=1}1&A||(n=-n)}return i.freeUint8(s),n};var r=32,i=t("typedarray-pool")},{"typedarray-pool":545}],464:[function(t,e,n){"use strict";var r=t("typedarray-pool"),i=t("invert-permutation");n.rank=function(t){var e=t.length;switch(e){case 0:case 1:return 0;case 2:return t[1]}var n,o,a,s=r.mallocUint32(e),A=r.mallocUint32(e),l=0;for(i(t,A),a=0;a<e;++a)s[a]=t[a];for(a=e-1;a>0;--a)o=A[a],n=s[a],s[a]=s[o],s[o]=n,A[a]=A[n],A[n]=o,l=(l+n)*a;return r.freeUint32(A),r.freeUint32(s),l},n.unrank=function(t,e,n){switch(t){case 0:return n||[];case 1:return n?(n[0]=0,n):[0];case 2:return n?(e?(n[0]=0,n[1]=1):(n[0]=1,n[1]=0),n):e?[0,1]:[1,0]}var r,i,o,a=1;for((n=n||new Array(t))[0]=0,o=1;o<t;++o)n[o]=o,a=a*o|0;for(o=t-1;o>0;--o)e=e-(r=e/a|0)*a|0,a=a/o|0,i=0|n[o],n[o]=0|n[r],n[r]=0|i;return n}},{"invert-permutation":415,"typedarray-pool":545}],465:[function(t,e,n){"use strict";e.exports=function(t,e,n){var r,o,a={};if("string"==typeof e&&(e=i(e)),Array.isArray(e)){var s={};for(o=0;o<e.length;o++)s[e[o]]=!0;e=s}for(r in e)e[r]=i(e[r]);var A={};for(r in e){var l=e[r];if(Array.isArray(l))for(o=0;o<l.length;o++){var c=l[o];if(n&&(A[c]=!0),c in t){if(a[r]=t[c],n)for(var u=o;u<l.length;u++)A[l[u]]=!0;break}}else r in t&&(e[r]&&(a[r]=t[r]),n&&(A[r]=!0))}if(n)for(r in t)A[r]||(a[r]=t[r]);return a};var r={};function i(t){return r[t]?r[t]:("string"==typeof t&&(t=r[t]=t.split(/\s*,\s*|\s+/)),t)}},{}],466:[function(t,e,n){"use strict";e.exports=function(t,e){for(var n=0|e.length,i=t.length,o=[new Array(n),new Array(n)],a=0;a<n;++a)o[0][a]=[],o[1][a]=[];for(var a=0;a<i;++a){var s=t[a];o[0][s[0]].push(s),o[1][s[1]].push(s)}for(var A=[],a=0;a<n;++a)o[0][a].length+o[1][a].length===0&&A.push([a]);function l(t,e){var n=o[e][t[e]];n.splice(n.indexOf(t),1)}function c(t,n,i){for(var a,s,A,c=0;c<2;++c)if(o[c][n].length>0){a=o[c][n][0],A=c;break}s=a[1^A];for(var u=0;u<2;++u)for(var g=o[u][n],h=0;h<g.length;++h){var d=g[h],f=d[1^u],p=r(e[t],e[n],e[s],e[f]);p>0&&(a=d,s=f,A=u)}return i?s:(a&&l(a,A),s)}function u(t,n){var i=o[n][t][0],a=[t];l(i,n);for(var s=i[1^n];;){for(;s!==t;)a.push(s),s=c(a[a.length-2],s,!1);if(o[0][t].length+o[1][t].length===0)break;var A=a[a.length-1],u=t,g=a[1],h=c(A,u,!0);if(r(e[A],e[u],e[g],e[h])<0)break;a.push(t),s=c(A,u)}return a}function g(t,e){return e[1]===e[e.length-1]}for(var a=0;a<n;++a)for(var h=0;h<2;++h){for(var d=[];o[h][a].length>0;){o[0][a].length;var f=u(a,h);g(0,f)?d.push.apply(d,f):(d.length>0&&A.push(d),d=f)}d.length>0&&A.push(d)}return A};var r=t("compare-angle")},{"compare-angle":127}],467:[function(t,e,n){"use strict";e.exports=function(t,e){for(var n=r(t,e.length),i=new Array(e.length),o=new Array(e.length),a=[],s=0;s<e.length;++s){var A=n[s].length;o[s]=A,i[s]=!0,A<=1&&a.push(s)}for(;a.length>0;){var l=a.pop();i[l]=!1;for(var c=n[l],s=0;s<c.length;++s){var u=c[s];0==--o[u]&&a.push(u)}}for(var g=new Array(e.length),h=[],s=0;s<e.length;++s)if(i[s]){var l=h.length;g[s]=l,h.push(e[s])}else g[s]=-1;for(var d=[],s=0;s<t.length;++s){var f=t[s];i[f[0]]&&i[f[1]]&&d.push([g[f[0]],g[f[1]]])}return[d,h]};var r=t("edges-to-adjacency-list")},{"edges-to-adjacency-list":171}],468:[function(t,e,n){"use strict";e.exports=function(t,e){var n=l(t,e);t=n[0];for(var u=(e=n[1]).length,g=(t.length,r(t,e.length)),h=0;h<u;++h)if(g[h].length%2==1)throw new Error("planar-graph-to-polyline: graph must be manifold");for(var d=i(t,e),f=(d=d.filter(function(t){for(var n=t.length,r=[0],i=0;i<n;++i){var o=e[t[i]],A=e[t[(i+1)%n]],l=a(-o[0],o[1]),c=a(-o[0],A[1]),u=a(A[0],o[1]),g=a(A[0],A[1]);r=s(r,s(s(l,c),s(u,g)))}return r[r.length-1]>0})).length,p=new Array(f),I=new Array(f),h=0;h<f;++h){p[h]=h;var C=new Array(f),m=d[h].map(function(t){return e[t]}),y=o([m]),E=0;t:for(var v=0;v<f;++v)if(C[v]=0,h!==v){for(var B=d[v],b=B.length,Q=0;Q<b;++Q){var w=y(e[B[Q]]);if(0!==w){w<0&&(C[v]=1,E+=1);continue t}}C[v]=1,E+=1}I[h]=[E,h,C]}I.sort(function(t,e){return e[0]-t[0]});for(var h=0;h<f;++h)for(var C=I[h],x=C[1],_=C[2],v=0;v<f;++v)_[v]&&(p[v]=x);for(var D=function(t){for(var e=new Array(t),n=0;n<t;++n)e[n]=[];return e}(f),h=0;h<f;++h)D[h].push(p[h]),D[p[h]].push(h);for(var M={},k=c(u,!1),h=0;h<f;++h)for(var B=d[h],b=B.length,v=0;v<b;++v){var S=B[v],L=B[(v+1)%b],R=Math.min(S,L)+":"+Math.max(S,L);if(R in M){var N=M[R];D[N].push(h),D[h].push(N),k[S]=k[L]=!0}else M[R]=h}function F(t){for(var e=t.length,n=0;n<e;++n)if(!k[t[n]])return!1;return!0}for(var T=[],U=c(f,-1),h=0;h<f;++h)p[h]!==h||F(d[h])?U[h]=-1:(T.push(h),U[h]=0);for(var n=[];T.length>0;){var O=T.pop(),G=D[O];A(G,function(t,e){return t-e});var H,j=G.length,Y=U[O];if(0===Y){var B=d[O];H=[B]}for(var h=0;h<j;++h){var P=G[h];if(!(U[P]>=0)&&(U[P]=1^Y,T.push(P),0===Y)){var B=d[P];F(B)||(B.reverse(),H.push(B))}}0===Y&&n.push(H)}return n};var r=t("edges-to-adjacency-list"),i=t("planar-dual"),o=t("point-in-big-polygon"),a=t("two-product"),s=t("robust-sum"),A=t("uniq"),l=t("./lib/trim-leaves");function c(t,e){for(var n=new Array(t),r=0;r<t;++r)n[r]=e;return n}},{"./lib/trim-leaves":467,"edges-to-adjacency-list":171,"planar-dual":466,"point-in-big-polygon":472,"robust-sum":515,"two-product":543,uniq:547}],469:[function(t,e,n){"use strict";e.exports=t("./quad")},{"./quad":471}],470:[function(t,e,n){arguments[4][111][0].apply(n,arguments)},{dup:111}],471:[function(t,e,n){"use strict";var r=t("binary-search-bounds"),i=t("clamp"),o=t("parse-rect"),a=t("array-bounds"),s=t("pick-by-alias"),A=t("defined"),l=t("flatten-vertex-data"),c=t("is-obj"),u=t("dtype"),g=t("math-log2");function h(t,e){for(var n=e[0],r=e[1],o=1/(e[2]-n),a=1/(e[3]-r),s=new Array(t.length),A=0,l=t.length/2;A<l;A++)s[2*A]=i((t[2*A]-n)*o,0,1),s[2*A+1]=i((t[2*A+1]-r)*a,0,1);return s}e.exports=function(t,e){e||(e={}),t=l(t,"float64"),e=s(e,{bounds:"range bounds dataBox databox",maxDepth:"depth maxDepth maxdepth level maxLevel maxlevel levels",dtype:"type dtype format out dst output destination"});var n=A(e.maxDepth,255),i=A(e.bounds,a(t,2));i[0]===i[2]&&i[2]++,i[1]===i[3]&&i[3]++;var d,f=h(t,i),p=t.length>>>1;e.dtype||(e.dtype="array"),"string"==typeof e.dtype?d=new(u(e.dtype))(p):e.dtype&&(d=e.dtype,Array.isArray(d)&&(d.length=p));for(var I=0;I<p;++I)d[I]=I;var C=[],m=[],y=[],E=[];!function t(e,r,i,o,a,s){if(!o.length)return null;var A=C[a]||(C[a]=[]),l=y[a]||(y[a]=[]),c=m[a]||(m[a]=[]),u=A.length;if(++a>n){for(var g=0;g<o.length;g++)A.push(o[g]),l.push(s),c.push(null,null,null,null);return u}if(A.push(o[0]),l.push(s),o.length<=1)return c.push(null,null,null,null),u;for(var h=.5*i,d=e+h,p=r+h,I=[],E=[],v=[],B=[],b=1,Q=o.length;b<Q;b++){var w=o[b],x=f[2*w],_=f[2*w+1];x<d?_<p?I.push(w):E.push(w):_<p?v.push(w):B.push(w)}return s<<=2,c.push(t(e,r,h,I,a,s),t(e,p,h,E,a,s+1),t(d,r,h,v,a,s+2),t(d,p,h,B,a,s+3)),u}(0,0,1,d,0,1);for(var v=0,B=0;B<C.length;B++){var b=C[B];if(d.set)d.set(b,v);else for(var Q=0,w=b.length;Q<w;Q++)d[Q+v]=b[Q];var x=v+C[B].length;E[B]=[v,x],v=x}return d.range=function(){for(var e,n=[],a=arguments.length;a--;)n[a]=arguments[a];if(c(n[n.length-1])){var l=n.pop();n.length||null==l.x&&null==l.l&&null==l.left||(n=[l],e={}),e=s(l,{level:"level maxLevel",d:"d diam diameter r radius px pxSize pixel pixelSize maxD size minSize",lod:"lod details ranges offsets"})}else e={};n.length||(n=i);var u,d=o.apply(void 0,n),f=[Math.min(d.x,d.x+d.width),Math.min(d.y,d.y+d.height),Math.max(d.x,d.x+d.width),Math.max(d.y,d.y+d.height)],p=f[0],I=f[1],v=f[2],B=f[3],b=h([p,I,v,B],i),Q=b[0],w=b[1],x=b[2],D=b[3],M=A(e.level,C.length);if(null!=e.d&&("number"==typeof e.d?u=[e.d,e.d]:e.d.length&&(u=e.d),M=Math.min(Math.max(Math.ceil(-g(Math.abs(u[0])/(i[2]-i[0]))),Math.ceil(-g(Math.abs(u[1])/(i[3]-i[1])))),M)),M=Math.min(M,C.length),e.lod)return function(t,e,n,i,o){for(var a=[],s=0;s<o;s++){var A=y[s],l=E[s][0],c=_(t,e,s),u=_(n,i,s),g=r.ge(A,c),h=r.gt(A,u,g,A.length-1);a[s]=[g+l,h+l]}return a}(Q,w,x,D,M);var k=[];return function e(n,r,i,o,a,s){if(null!==a&&null!==s){var A=n+i,l=r+i;if(!(Q>A||w>l||x<n||D<r||o>=M||a===s)){var c=C[o];void 0===s&&(s=c.length);for(var u=a;u<s;u++){var g=c[u],h=t[2*g],d=t[2*g+1];h>=p&&h<=v&&d>=I&&d<=B&&k.push(g)}var f=m[o],y=f[4*a+0],E=f[4*a+1],b=f[4*a+2],_=f[4*a+3],S=function(t,e){for(var n=null,r=0;null===n;)if(n=t[4*e+r],++r>t.length)return null;return n}(f,a+1),L=.5*i,R=o+1;e(n,r,L,R,y,E||b||_||S),e(n,r+L,L,R,E,b||_||S),e(n+L,r,L,R,b,_||S),e(n+L,r+L,L,R,_,S)}}}(0,0,1,0,0,1),k},d;function _(t,e,n){for(var r=1,i=.5,o=.5,a=.5,s=0;s<n;s++)r<<=2,r+=t<i?e<o?0:1:e<o?2:3,a*=.5,i+=t<i?-a:a,o+=e<o?-a:a;return r}}},{"array-bounds":65,"binary-search-bounds":470,clamp:115,defined:164,dtype:169,"flatten-vertex-data":227,"is-obj":421,"math-log2":432,"parse-rect":459,"pick-by-alias":465}],472:[function(t,e,n){e.exports=function(t){for(var e=t.length,n=[],o=[],s=0;s<e;++s)for(var c=t[s],u=c.length,g=u-1,h=0;h<u;g=h++){var d=c[g],f=c[h];d[0]===f[0]?o.push([d,f]):n.push([d,f])}if(0===n.length)return 0===o.length?l:(p=A(o),function(t){return p(t[0],t[1])?0:1});var p,I=i(n),C=function(t,e){return function(n){var i=a.le(e,n[0]);if(i<0)return 1;var o=t[i];if(!o){if(!(i>0&&e[i]===n[0]))return 1;o=t[i-1]}for(var s=1;o;){var A=o.key,l=r(n,A[0],A[1]);if(A[0][0]<A[1][0])if(l<0)o=o.left;else{if(!(l>0))return 0;s=-1,o=o.right}else if(l>0)o=o.left;else{if(!(l<0))return 0;s=1,o=o.right}}return s}}(I.slabs,I.coordinates);return 0===o.length?C:function(t,e){return function(n){return t(n[0],n[1])?0:e(n)}}(A(o),C)};var r=t("robust-orientation")[3],i=t("slab-decomposition"),o=t("interval-tree-1d"),a=t("binary-search-bounds");function s(){return!0}function A(t){for(var e={},n=0;n<t.length;++n){var r=t[n],i=r[0][0],a=r[0][1],A=r[1][1],l=[Math.min(a,A),Math.max(a,A)];i in e?e[i].push(l):e[i]=[l]}var c={},u=Object.keys(e);for(n=0;n<u.length;++n){var g=e[u[n]];c[u[n]]=o(g)}return function(t){return function(e,n){var r=t[e];return!!r&&!!r.queryPoint(n,s)}}(c)}function l(t){return 1}},{"binary-search-bounds":91,"interval-tree-1d":414,"robust-orientation":510,"slab-decomposition":526}],473:[function(t,e,n){var r,i=t("./lib/build-log"),o=t("./lib/epsilon"),a=t("./lib/intersecter"),s=t("./lib/segment-chainer"),A=t("./lib/segment-selector"),l=t("./lib/geojson"),c=!1,u=o();function g(t,e,n){var i=r.segments(t),o=r.segments(e),a=n(r.combine(i,o));return r.polygon(a)}r={buildLog:function(t){return!0===t?c=i():!1===t&&(c=!1),!1!==c&&c.list},epsilon:function(t){return u.epsilon(t)},segments:function(t){var e=a(!0,u,c);return t.regions.forEach(e.addRegion),{segments:e.calculate(t.inverted),inverted:t.inverted}},combine:function(t,e){return{combined:a(!1,u,c).calculate(t.segments,t.inverted,e.segments,e.inverted),inverted1:t.inverted,inverted2:e.inverted}},selectUnion:function(t){return{segments:A.union(t.combined,c),inverted:t.inverted1||t.inverted2}},selectIntersect:function(t){return{segments:A.intersect(t.combined,c),inverted:t.inverted1&&t.inverted2}},selectDifference:function(t){return{segments:A.difference(t.combined,c),inverted:t.inverted1&&!t.inverted2}},selectDifferenceRev:function(t){return{segments:A.differenceRev(t.combined,c),inverted:!t.inverted1&&t.inverted2}},selectXor:function(t){return{segments:A.xor(t.combined,c),inverted:t.inverted1!==t.inverted2}},polygon:function(t){return{regions:s(t.segments,u,c),inverted:t.inverted}},polygonFromGeoJSON:function(t){return l.toPolygon(r,t)},polygonToGeoJSON:function(t){return l.fromPolygon(r,u,t)},union:function(t,e){return g(t,e,r.selectUnion)},intersect:function(t,e){return g(t,e,r.selectIntersect)},difference:function(t,e){return g(t,e,r.selectDifference)},differenceRev:function(t,e){return g(t,e,r.selectDifferenceRev)},xor:function(t,e){return g(t,e,r.selectXor)}},"object"==typeof window&&(window.PolyBool=r),e.exports=r},{"./lib/build-log":474,"./lib/epsilon":475,"./lib/geojson":476,"./lib/intersecter":477,"./lib/segment-chainer":479,"./lib/segment-selector":480}],474:[function(t,e,n){e.exports=function(){var t,e=0,n=!1;function r(e,n){return t.list.push({type:e,data:n?JSON.parse(JSON.stringify(n)):void 0}),t}return t={list:[],segmentId:function(){return e++},checkIntersection:function(t,e){return r("check",{seg1:t,seg2:e})},segmentChop:function(t,e){return r("div_seg",{seg:t,pt:e}),r("chop",{seg:t,pt:e})},statusRemove:function(t){return r("pop_seg",{seg:t})},segmentUpdate:function(t){return r("seg_update",{seg:t})},segmentNew:function(t,e){return r("new_seg",{seg:t,primary:e})},segmentRemove:function(t){return r("rem_seg",{seg:t})},tempStatus:function(t,e,n){return r("temp_status",{seg:t,above:e,below:n})},rewind:function(t){return r("rewind",{seg:t})},status:function(t,e,n){return r("status",{seg:t,above:e,below:n})},vert:function(e){return e===n?t:(n=e,r("vert",{x:e}))},log:function(t){return"string"!=typeof t&&(t=JSON.stringify(t,!1,"  ")),r("log",{txt:t})},reset:function(){return r("reset")},selected:function(t){return r("selected",{segs:t})},chainStart:function(t){return r("chain_start",{seg:t})},chainRemoveHead:function(t,e){return r("chain_rem_head",{index:t,pt:e})},chainRemoveTail:function(t,e){return r("chain_rem_tail",{index:t,pt:e})},chainNew:function(t,e){return r("chain_new",{pt1:t,pt2:e})},chainMatch:function(t){return r("chain_match",{index:t})},chainClose:function(t){return r("chain_close",{index:t})},chainAddHead:function(t,e){return r("chain_add_head",{index:t,pt:e})},chainAddTail:function(t,e){return r("chain_add_tail",{index:t,pt:e})},chainConnect:function(t,e){return r("chain_con",{index1:t,index2:e})},chainReverse:function(t){return r("chain_rev",{index:t})},chainJoin:function(t,e){return r("chain_join",{index1:t,index2:e})},done:function(){return r("done")}}}},{}],475:[function(t,e,n){e.exports=function(t){"number"!=typeof t&&(t=1e-10);var e={epsilon:function(e){return"number"==typeof e&&(t=e),t},pointAboveOrOnLine:function(e,n,r){var i=n[0],o=n[1],a=r[0],s=r[1],A=e[0];return(a-i)*(e[1]-o)-(s-o)*(A-i)>=-t},pointBetween:function(e,n,r){var i=e[1]-n[1],o=r[0]-n[0],a=e[0]-n[0],s=r[1]-n[1],A=a*o+i*s;return!(A<t||A-(o*o+s*s)>-t)},pointsSameX:function(e,n){return Math.abs(e[0]-n[0])<t},pointsSameY:function(e,n){return Math.abs(e[1]-n[1])<t},pointsSame:function(t,n){return e.pointsSameX(t,n)&&e.pointsSameY(t,n)},pointsCompare:function(t,n){return e.pointsSameX(t,n)?e.pointsSameY(t,n)?0:t[1]<n[1]?-1:1:t[0]<n[0]?-1:1},pointsCollinear:function(e,n,r){var i=e[0]-n[0],o=e[1]-n[1],a=n[0]-r[0],s=n[1]-r[1];return Math.abs(i*s-a*o)<t},linesIntersect:function(e,n,r,i){var o=n[0]-e[0],a=n[1]-e[1],s=i[0]-r[0],A=i[1]-r[1],l=o*A-a*s;if(Math.abs(l)<t)return!1;var c=e[0]-r[0],u=e[1]-r[1],g=(s*u-A*c)/l,h=(o*u-a*c)/l,d={alongA:0,alongB:0,pt:[e[0]+g*o,e[1]+g*a]};return d.alongA=g<=-t?-2:g<t?-1:g-1<=-t?0:g-1<t?1:2,d.alongB=h<=-t?-2:h<t?-1:h-1<=-t?0:h-1<t?1:2,d},pointInsideRegion:function(e,n){for(var r=e[0],i=e[1],o=n[n.length-1][0],a=n[n.length-1][1],s=!1,A=0;A<n.length;A++){var l=n[A][0],c=n[A][1];c-i>t!=a-i>t&&(o-l)*(i-c)/(a-c)+l-r>t&&(s=!s),o=l,a=c}return s}};return e}},{}],476:[function(t,e,n){var r={toPolygon:function(t,e){function n(e){if(e.length<=0)return t.segments({inverted:!1,regions:[]});function n(e){var n=e.slice(0,e.length-1);return t.segments({inverted:!1,regions:[n]})}for(var r=n(e[0]),i=1;i<e.length;i++)r=t.selectDifference(t.combine(r,n(e[i])));return r}if("Polygon"===e.type)return t.polygon(n(e.coordinates));if("MultiPolygon"===e.type){for(var r=t.segments({inverted:!1,regions:[]}),i=0;i<e.coordinates.length;i++)r=t.selectUnion(t.combine(r,n(e.coordinates[i])));return t.polygon(r)}throw new Error("PolyBool: Cannot convert GeoJSON object to PolyBool polygon")},fromPolygon:function(t,e,n){function r(t,n){return e.pointInsideRegion([.5*(t[0][0]+t[1][0]),.5*(t[0][1]+t[1][1])],n)}function i(t){return{region:t,children:[]}}n=t.polygon(t.segments(n));var o=i(null);function a(t,e){for(var n=0;n<t.children.length;n++)if(r(e,(s=t.children[n]).region))return void a(s,e);var o=i(e);for(n=0;n<t.children.length;n++){var s;r((s=t.children[n]).region,e)&&(o.children.push(s),t.children.splice(n,1),n--)}t.children.push(o)}for(var s=0;s<n.regions.length;s++){var A=n.regions[s];A.length<3||a(o,A)}function l(t,e){for(var n=0,r=t[t.length-1][0],i=t[t.length-1][1],o=[],a=0;a<t.length;a++){var s=t[a][0],A=t[a][1];o.push([s,A]),n+=A*r-s*i,r=s,i=A}return n<0!==e&&o.reverse(),o.push([o[0][0],o[0][1]]),o}var c=[];function u(t){var e=[l(t.region,!1)];c.push(e);for(var n=0;n<t.children.length;n++)e.push(g(t.children[n]))}function g(t){for(var e=0;e<t.children.length;e++)u(t.children[e]);return l(t.region,!0)}for(s=0;s<o.children.length;s++)u(o.children[s]);return c.length<=0?{type:"Polygon",coordinates:[]}:1==c.length?{type:"Polygon",coordinates:c[0]}:{type:"MultiPolygon",coordinates:c}}};e.exports=r},{}],477:[function(t,e,n){var r=t("./linked-list");e.exports=function(t,e,n){function i(t,e,r){return{id:n?n.segmentId():-1,start:t,end:e,myFill:{above:r.myFill.above,below:r.myFill.below},otherFill:null}}var o=r.create();function a(t,n){o.insertBefore(t,function(r){return function(t,n,r,i,o,a){var s=e.pointsCompare(n,o);return 0!==s?s:e.pointsSame(r,a)?0:t!==i?t?1:-1:e.pointAboveOrOnLine(r,i?o:a,i?a:o)?1:-1}(t.isStart,t.pt,n,r.isStart,r.pt,r.other.pt)<0})}function s(t,e){var n=function(t,e){var n=r.node({isStart:!0,pt:t.start,seg:t,primary:e,other:null,status:null});return a(n,t.end),n}(t,e);return function(t,e,n){var i=r.node({isStart:!1,pt:e.end,seg:e,primary:n,other:t,status:null});t.other=i,a(i,t.pt)}(n,t,e),n}function A(t,e){var r=i(e,t.seg.end,t.seg);return function(t,e){n&&n.segmentChop(t.seg,e),t.other.remove(),t.seg.end=e,t.other.pt=e,a(t.other,t.pt)}(t,e),s(r,t.primary)}function l(i,a){var s=r.create();function l(t){return s.findTransition(function(n){var r,i,o,a,s,A;return r=t,i=n.ev,o=r.seg.start,a=r.seg.end,s=i.seg.start,A=i.seg.end,(e.pointsCollinear(o,s,A)?e.pointsCollinear(a,s,A)?1:e.pointAboveOrOnLine(a,s,A)?1:-1:e.pointAboveOrOnLine(o,s,A)?1:-1)>0})}function c(t,r){var i=t.seg,o=r.seg,a=i.start,s=i.end,l=o.start,c=o.end;n&&n.checkIntersection(i,o);var u=e.linesIntersect(a,s,l,c);if(!1===u){if(!e.pointsCollinear(a,s,l))return!1;if(e.pointsSame(a,c)||e.pointsSame(s,l))return!1;var g=e.pointsSame(a,l),h=e.pointsSame(s,c);if(g&&h)return r;var d=!g&&e.pointBetween(a,l,c),f=!h&&e.pointBetween(s,l,c);if(g)return f?A(r,s):A(t,c),r;d&&(h||(f?A(r,s):A(t,c)),A(r,a))}else 0===u.alongA&&(-1===u.alongB?A(t,l):0===u.alongB?A(t,u.pt):1===u.alongB&&A(t,c)),0===u.alongB&&(-1===u.alongA?A(r,a):0===u.alongA?A(r,u.pt):1===u.alongA&&A(r,s));return!1}for(var u=[];!o.isEmpty();){var g=o.getHead();if(n&&n.vert(g.pt[0]),g.isStart){n&&n.segmentNew(g.seg,g.primary);var h=l(g),d=h.before?h.before.ev:null,f=h.after?h.after.ev:null;function p(){if(d){var t=c(g,d);if(t)return t}return!!f&&c(g,f)}n&&n.tempStatus(g.seg,!!d&&d.seg,!!f&&f.seg);var I,C,m=p();if(m&&(t?(C=null===g.seg.myFill.below||g.seg.myFill.above!==g.seg.myFill.below)&&(m.seg.myFill.above=!m.seg.myFill.above):m.seg.otherFill=g.seg.myFill,n&&n.segmentUpdate(m.seg),g.other.remove(),g.remove()),o.getHead()!==g){n&&n.rewind(g.seg);continue}t?(C=null===g.seg.myFill.below||g.seg.myFill.above!==g.seg.myFill.below,g.seg.myFill.below=f?f.seg.myFill.above:i,g.seg.myFill.above=C?!g.seg.myFill.below:g.seg.myFill.below):null===g.seg.otherFill&&(I=f?g.primary===f.primary?f.seg.otherFill.above:f.seg.myFill.above:g.primary?a:i,g.seg.otherFill={above:I,below:I}),n&&n.status(g.seg,!!d&&d.seg,!!f&&f.seg),g.other.status=h.insert(r.node({ev:g}))}else{var y=g.status;if(null===y)throw new Error("PolyBool: Zero-length segment detected; your epsilon is probably too small or too large");if(s.exists(y.prev)&&s.exists(y.next)&&c(y.prev.ev,y.next.ev),n&&n.statusRemove(y.ev.seg),y.remove(),!g.primary){var E=g.seg.myFill;g.seg.myFill=g.seg.otherFill,g.seg.otherFill=E}u.push(g.seg)}o.getHead().remove()}return n&&n.done(),u}return t?{addRegion:function(t){for(var r,i,o,a=t[t.length-1],A=0;A<t.length;A++){r=a,a=t[A];var l=e.pointsCompare(r,a);0!==l&&s((i=l<0?r:a,o=l<0?a:r,{id:n?n.segmentId():-1,start:i,end:o,myFill:{above:null,below:null},otherFill:null}),!0)}},calculate:function(t){return l(t,!1)}}:{calculate:function(t,e,n,r){return t.forEach(function(t){s(i(t.start,t.end,t),!0)}),n.forEach(function(t){s(i(t.start,t.end,t),!1)}),l(e,r)}}}},{"./linked-list":478}],478:[function(t,e,n){e.exports={create:function(){var t={root:{root:!0,next:null},exists:function(e){return null!==e&&e!==t.root},isEmpty:function(){return null===t.root.next},getHead:function(){return t.root.next},insertBefore:function(e,n){for(var r=t.root,i=t.root.next;null!==i;){if(n(i))return e.prev=i.prev,e.next=i,i.prev.next=e,void(i.prev=e);r=i,i=i.next}r.next=e,e.prev=r,e.next=null},findTransition:function(e){for(var n=t.root,r=t.root.next;null!==r&&!e(r);)n=r,r=r.next;return{before:n===t.root?null:n,after:r,insert:function(t){return t.prev=n,t.next=r,n.next=t,null!==r&&(r.prev=t),t}}}};return t},node:function(t){return t.prev=null,t.next=null,t.remove=function(){t.prev.next=t.next,t.next&&(t.next.prev=t.prev),t.prev=null,t.next=null},t}}},{}],479:[function(t,e,n){e.exports=function(t,e,n){var r=[],i=[];return t.forEach(function(t){var o=t.start,a=t.end;if(e.pointsSame(o,a))console.warn("PolyBool: Warning: Zero-length segment detected; your epsilon is probably too small or too large");else{n&&n.chainStart(t);for(var s={index:0,matches_head:!1,matches_pt1:!1},A={index:0,matches_head:!1,matches_pt1:!1},l=s,c=0;c<r.length;c++){var u=(p=r[c])[0],g=(p[1],p[p.length-1]);if(p[p.length-2],e.pointsSame(u,o)){if(b(c,!0,!0))break}else if(e.pointsSame(u,a)){if(b(c,!0,!1))break}else if(e.pointsSame(g,o)){if(b(c,!1,!0))break}else if(e.pointsSame(g,a)&&b(c,!1,!1))break}if(l===s)return r.push([o,a]),void(n&&n.chainNew(o,a));if(l===A){n&&n.chainMatch(s.index);var h=s.index,d=s.matches_pt1?a:o,f=s.matches_head,p=r[h],I=f?p[0]:p[p.length-1],C=f?p[1]:p[p.length-2],m=f?p[p.length-1]:p[0],y=f?p[p.length-2]:p[1];return e.pointsCollinear(C,I,d)&&(f?(n&&n.chainRemoveHead(s.index,d),p.shift()):(n&&n.chainRemoveTail(s.index,d),p.pop()),I=C),e.pointsSame(m,d)?(r.splice(h,1),e.pointsCollinear(y,m,I)&&(f?(n&&n.chainRemoveTail(s.index,I),p.pop()):(n&&n.chainRemoveHead(s.index,I),p.shift())),n&&n.chainClose(s.index),void i.push(p)):void(f?(n&&n.chainAddHead(s.index,d),p.unshift(d)):(n&&n.chainAddTail(s.index,d),p.push(d)))}var E=s.index,v=A.index;n&&n.chainConnect(E,v);var B=r[E].length<r[v].length;s.matches_head?A.matches_head?B?(Q(E),w(E,v)):(Q(v),w(v,E)):w(v,E):A.matches_head?w(E,v):B?(Q(E),w(v,E)):(Q(v),w(E,v))}function b(t,e,n){return l.index=t,l.matches_head=e,l.matches_pt1=n,l===s?(l=A,!1):(l=null,!0)}function Q(t){n&&n.chainReverse(t),r[t].reverse()}function w(t,i){var o=r[t],a=r[i],s=o[o.length-1],A=o[o.length-2],l=a[0],c=a[1];e.pointsCollinear(A,s,l)&&(n&&n.chainRemoveTail(t,s),o.pop(),s=A),e.pointsCollinear(s,l,c)&&(n&&n.chainRemoveHead(i,l),a.shift()),n&&n.chainJoin(t,i),r[t]=o.concat(a),r.splice(i,1)}}),i}},{}],480:[function(t,e,n){function r(t,e,n){var r=[];return t.forEach(function(t){var i=(t.myFill.above?8:0)+(t.myFill.below?4:0)+(t.otherFill&&t.otherFill.above?2:0)+(t.otherFill&&t.otherFill.below?1:0);0!==e[i]&&r.push({id:n?n.segmentId():-1,start:t.start,end:t.end,myFill:{above:1===e[i],below:2===e[i]},otherFill:null})}),n&&n.selected(r),r}var i={union:function(t,e){return r(t,[0,2,1,0,2,2,0,0,1,0,1,0,0,0,0,0],e)},intersect:function(t,e){return r(t,[0,0,0,0,0,2,0,2,0,0,1,1,0,2,1,0],e)},difference:function(t,e){return r(t,[0,0,0,0,2,0,2,0,1,1,0,0,0,1,2,0],e)},differenceRev:function(t,e){return r(t,[0,2,1,0,0,0,1,1,0,2,0,2,0,0,0,0],e)},xor:function(t,e){return r(t,[0,2,1,0,2,0,0,1,1,0,0,2,0,1,2,0],e)}};e.exports=i},{}],481:[function(t,e,n){"use strict";var r=new Float64Array(4),i=new Float64Array(4),o=new Float64Array(4);e.exports=function(t,e,n,a,s){r.length<a.length&&(r=new Float64Array(a.length),i=new Float64Array(a.length),o=new Float64Array(a.length));for(var A=0;A<a.length;++A)r[A]=t[A]-a[A],i[A]=e[A]-t[A],o[A]=n[A]-t[A];var l=0,c=0,u=0,g=0,h=0,d=0;for(A=0;A<a.length;++A){var f=i[A],p=o[A],I=r[A];l+=f*f,c+=f*p,u+=p*p,g+=I*f,h+=I*p,d+=I*I}var C,m,y,E,v,B=Math.abs(l*u-c*c),b=c*h-u*g,Q=c*g-l*h;if(b+Q<=B)if(b<0)Q<0&&g<0?(Q=0,-g>=l?(b=1,C=l+2*g+d):C=g*(b=-g/l)+d):(b=0,h>=0?(Q=0,C=d):-h>=u?(Q=1,C=u+2*h+d):C=h*(Q=-h/u)+d);else if(Q<0)Q=0,g>=0?(b=0,C=d):-g>=l?(b=1,C=l+2*g+d):C=g*(b=-g/l)+d;else{var w=1/B;C=(b*=w)*(l*b+c*(Q*=w)+2*g)+Q*(c*b+u*Q+2*h)+d}else b<0?(y=u+h)>(m=c+g)?(E=y-m)>=(v=l-2*c+u)?(b=1,Q=0,C=l+2*g+d):C=(b=E/v)*(l*b+c*(Q=1-b)+2*g)+Q*(c*b+u*Q+2*h)+d:(b=0,y<=0?(Q=1,C=u+2*h+d):h>=0?(Q=0,C=d):C=h*(Q=-h/u)+d):Q<0?(y=l+g)>(m=c+h)?(E=y-m)>=(v=l-2*c+u)?(Q=1,b=0,C=u+2*h+d):C=(b=1-(Q=E/v))*(l*b+c*Q+2*g)+Q*(c*b+u*Q+2*h)+d:(Q=0,y<=0?(b=1,C=l+2*g+d):g>=0?(b=0,C=d):C=g*(b=-g/l)+d):(E=u+h-c-g)<=0?(b=0,Q=1,C=u+2*h+d):E>=(v=l-2*c+u)?(b=1,Q=0,C=l+2*g+d):C=(b=E/v)*(l*b+c*(Q=1-b)+2*g)+Q*(c*b+u*Q+2*h)+d;var x=1-b-Q;for(A=0;A<a.length;++A)s[A]=x*t[A]+b*e[A]+Q*n[A];return C<0?0:C}},{}],482:[function(t,e,n){var r,i,o=e.exports={};function a(){throw new Error("setTimeout has not been defined")}function s(){throw new Error("clearTimeout has not been defined")}function A(t){if(r===setTimeout)return setTimeout(t,0);if((r===a||!r)&&setTimeout)return r=setTimeout,setTimeout(t,0);try{return r(t,0)}catch(e){try{return r.call(null,t,0)}catch(e){return r.call(this,t,0)}}}!function(){try{r="function"==typeof setTimeout?setTimeout:a}catch(t){r=a}try{i="function"==typeof clearTimeout?clearTimeout:s}catch(t){i=s}}();var l,c=[],u=!1,g=-1;function h(){u&&l&&(u=!1,l.length?c=l.concat(c):g=-1,c.length&&d())}function d(){if(!u){var t=A(h);u=!0;for(var e=c.length;e;){for(l=c,c=[];++g<e;)l&&l[g].run();g=-1,e=c.length}l=null,u=!1,function(t){if(i===clearTimeout)return clearTimeout(t);if((i===s||!i)&&clearTimeout)return i=clearTimeout,clearTimeout(t);try{i(t)}catch(e){try{return i.call(null,t)}catch(e){return i.call(this,t)}}}(t)}}function f(t,e){this.fun=t,this.array=e}function p(){}o.nextTick=function(t){var e=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)e[n-1]=arguments[n];c.push(new f(t,e)),1!==c.length||u||A(d)},f.prototype.run=function(){this.fun.apply(null,this.array)},o.title="browser",o.browser=!0,o.env={},o.argv=[],o.version="",o.versions={},o.on=p,o.addListener=p,o.once=p,o.off=p,o.removeListener=p,o.removeAllListeners=p,o.emit=p,o.prependListener=p,o.prependOnceListener=p,o.listeners=function(t){return[]},o.binding=function(t){throw new Error("process.binding is not supported")},o.cwd=function(){return"/"},o.chdir=function(t){throw new Error("process.chdir is not supported")},o.umask=function(){return 0}},{}],483:[function(t,e,n){e.exports=t("gl-quat/slerp")},{"gl-quat/slerp":293}],484:[function(t,n,r){(function(e){for(var r=t("performance-now"),i="undefined"==typeof window?e:window,o=["moz","webkit"],a="AnimationFrame",s=i["request"+a],A=i["cancel"+a]||i["cancelRequest"+a],l=0;!s&&l<o.length;l++)s=i[o[l]+"Request"+a],A=i[o[l]+"Cancel"+a]||i[o[l]+"CancelRequest"+a];if(!s||!A){var c=0,u=0,g=[];s=function(t){if(0===g.length){var e=r(),n=Math.max(0,1e3/60-(e-c));c=n+e,setTimeout(function(){var t=g.slice(0);g.length=0;for(var e=0;e<t.length;e++)if(!t[e].cancelled)try{t[e].callback(c)}catch(t){setTimeout(function(){throw t},0)}},Math.round(n))}return g.push({handle:++u,callback:t,cancelled:!1}),u},A=function(t){for(var e=0;e<g.length;e++)g[e].handle===t&&(g[e].cancelled=!0)}}n.exports=function(t){return s.call(i,t)},n.exports.cancel=function(){A.apply(i,arguments)},n.exports.polyfill=function(t){t||(t=i),t.requestAnimationFrame=s,t.cancelAnimationFrame=A}}).call(this,void 0!==e?e:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{"performance-now":462}],485:[function(t,e,n){"use strict";var r=t("big-rat/add");e.exports=function(t,e){for(var n=t.length,i=new Array(n),o=0;o<n;++o)i[o]=r(t[o],e[o]);return i}},{"big-rat/add":75}],486:[function(t,e,n){"use strict";e.exports=function(t){for(var e=new Array(t.length),n=0;n<t.length;++n)e[n]=r(t[n]);return e};var r=t("big-rat")},{"big-rat":78}],487:[function(t,e,n){"use strict";var r=t("big-rat"),i=t("big-rat/mul");e.exports=function(t,e){for(var n=r(e),o=t.length,a=new Array(o),s=0;s<o;++s)a[s]=i(t[s],n);return a}},{"big-rat":78,"big-rat/mul":87}],488:[function(t,e,n){"use strict";var r=t("big-rat/sub");e.exports=function(t,e){for(var n=t.length,i=new Array(n),o=0;o<n;++o)i[o]=r(t[o],e[o]);return i}},{"big-rat/sub":89}],489:[function(t,e,n){"use strict";var r=t("compare-cell"),i=t("compare-oriented-cell"),o=t("cell-orientation");e.exports=function(t){t.sort(i);for(var e=t.length,n=0,a=0;a<e;++a){var s=t[a],A=o(s);if(0!==A){if(n>0){var l=t[n-1];if(0===r(s,l)&&o(l)!==A){n-=1;continue}}t[n++]=s}}return t.length=n,t}},{"cell-orientation":112,"compare-cell":128,"compare-oriented-cell":129}],490:[function(t,e,n){"use strict";var r=t("array-bounds"),i=t("color-normalize"),o=t("update-diff"),a=t("pick-by-alias"),s=t("object-assign"),A=t("flatten-vertex-data"),l=t("to-float32"),c=l.float32,u=l.fract32;e.exports=function(t,e){if("function"==typeof t?(e||(e={}),e.regl=t):e=t,e.length&&(e.positions=e),!(t=e.regl).hasExtension("ANGLE_instanced_arrays"))throw Error("regl-error2d: `ANGLE_instanced_arrays` extension should be enabled");var n,l,h,d,f,p,I=t._gl,C={color:"black",capSize:5,lineWidth:1,opacity:1,viewport:null,range:null,offset:0,count:0,bounds:null,positions:[],errors:[]},m=[];return d=t.buffer({usage:"dynamic",type:"uint8",data:new Uint8Array(0)}),l=t.buffer({usage:"dynamic",type:"float",data:new Uint8Array(0)}),h=t.buffer({usage:"dynamic",type:"float",data:new Uint8Array(0)}),f=t.buffer({usage:"dynamic",type:"float",data:new Uint8Array(0)}),p=t.buffer({usage:"static",type:"float",data:g}),B(e),n=t({vert:"\n\t\tprecision highp float;\n\n\t\tattribute vec2 position, positionFract;\n\t\tattribute vec4 error;\n\t\tattribute vec4 color;\n\n\t\tattribute vec2 direction, lineOffset, capOffset;\n\n\t\tuniform vec4 viewport;\n\t\tuniform float lineWidth, capSize;\n\t\tuniform vec2 scale, scaleFract, translate, translateFract;\n\n\t\tvarying vec4 fragColor;\n\n\t\tvoid main() {\n\t\t\tfragColor = color / 255.;\n\n\t\t\tvec2 pixelOffset = lineWidth * lineOffset + (capSize + lineWidth) * capOffset;\n\n\t\t\tvec2 dxy = -step(.5, direction.xy) * error.xz + step(direction.xy, vec2(-.5)) * error.yw;\n\n\t\t\tvec2 position = position + dxy;\n\n\t\t\tvec2 pos = (position + translate) * scale\n\t\t\t\t+ (positionFract + translateFract) * scale\n\t\t\t\t+ (position + translate) * scaleFract\n\t\t\t\t+ (positionFract + translateFract) * scaleFract;\n\n\t\t\tpos += pixelOffset / viewport.zw;\n\n\t\t\tgl_Position = vec4(pos * 2. - 1., 0, 1);\n\t\t}\n\t\t",frag:"\n\t\tprecision highp float;\n\n\t\tvarying vec4 fragColor;\n\n\t\tuniform float opacity;\n\n\t\tvoid main() {\n\t\t\tgl_FragColor = fragColor;\n\t\t\tgl_FragColor.a *= opacity;\n\t\t}\n\t\t",uniforms:{range:t.prop("range"),lineWidth:t.prop("lineWidth"),capSize:t.prop("capSize"),opacity:t.prop("opacity"),scale:t.prop("scale"),translate:t.prop("translate"),scaleFract:t.prop("scaleFract"),translateFract:t.prop("translateFract"),viewport:function(t,e){return[e.viewport.x,e.viewport.y,t.viewportWidth,t.viewportHeight]}},attributes:{color:{buffer:d,offset:function(t,e){return 4*e.offset},divisor:1},position:{buffer:l,offset:function(t,e){return 8*e.offset},divisor:1},positionFract:{buffer:h,offset:function(t,e){return 8*e.offset},divisor:1},error:{buffer:f,offset:function(t,e){return 16*e.offset},divisor:1},direction:{buffer:p,stride:24,offset:0},lineOffset:{buffer:p,stride:24,offset:8},capOffset:{buffer:p,stride:24,offset:16}},primitive:"triangles",blend:{enable:!0,color:[0,0,0,0],equation:{rgb:"add",alpha:"add"},func:{srcRGB:"src alpha",dstRGB:"one minus src alpha",srcAlpha:"one minus dst alpha",dstAlpha:"one"}},depth:{enable:!1},scissor:{enable:!0,box:t.prop("viewport")},viewport:t.prop("viewport"),stencil:!1,instances:t.prop("count"),count:g.length}),s(y,{update:B,draw:E,destroy:b,regl:t,gl:I,canvas:I.canvas,groups:m}),y;function y(t){t?B(t):null===t&&b(),E()}function E(e){if("number"==typeof e)return v(e);e&&!Array.isArray(e)&&(e=[e]),t._refresh(),m.forEach(function(t,n){t&&(e&&(e[n]?t.draw=!0:t.draw=!1),t.draw?v(n):t.draw=!0)})}function v(t){"number"==typeof t&&(t=m[t]),null!=t&&t&&t.count&&t.color&&t.opacity&&t.positions&&t.positions.length>1&&(t.scaleRatio=[t.scale[0]*t.viewport.width,t.scale[1]*t.viewport.height],n(t),t.after&&t.after(t))}function B(t){if(t){null!=t.length?"number"==typeof t[0]&&(t=[{positions:t}]):Array.isArray(t)||(t=[t]);var e=0,n=0;if(y.groups=m=t.map(function(t,l){var c=m[l];return t?("function"==typeof t?t={after:t}:"number"==typeof t[0]&&(t={positions:t}),t=a(t,{color:"color colors fill",capSize:"capSize cap capsize cap-size",lineWidth:"lineWidth line-width width line thickness",opacity:"opacity alpha",range:"range dataBox",viewport:"viewport viewBox",errors:"errors error",positions:"positions position data points"}),c||(m[l]=c={id:l,scale:null,translate:null,scaleFract:null,translateFract:null,draw:!0},t=s({},C,t)),o(c,t,[{lineWidth:function(t){return.5*+t},capSize:function(t){return.5*+t},opacity:parseFloat,errors:function(t){return t=A(t),n+=t.length,t},positions:function(t,n){return t=A(t,"float64"),n.count=Math.floor(t.length/2),n.bounds=r(t,2),n.offset=e,e+=n.count,t}},{color:function(t,e){var n=e.count;if(t||(t="transparent"),!Array.isArray(t)||"number"==typeof t[0]){var r=t;t=Array(n);for(var o=0;o<n;o++)t[o]=r}if(t.length<n)throw Error("Not enough colors");for(var a=new Uint8Array(4*n),s=0;s<n;s++){var A=i(t[s],"uint8");a.set(A,4*s)}return a},range:function(t,e,n){var r=e.bounds;return t||(t=r),e.scale=[1/(t[2]-t[0]),1/(t[3]-t[1])],e.translate=[-t[0],-t[1]],e.scaleFract=u(e.scale),e.translateFract=u(e.translate),t},viewport:function(t){var e;return Array.isArray(t)?e={x:t[0],y:t[1],width:t[2]-t[0],height:t[3]-t[1]}:t?(e={x:t.x||t.left||0,y:t.y||t.top||0},t.right?e.width=t.right-e.x:e.width=t.w||t.width||0,t.bottom?e.height=t.bottom-e.y:e.height=t.h||t.height||0):e={x:0,y:0,width:I.drawingBufferWidth,height:I.drawingBufferHeight},e}}]),c):c}),e||n){var g=m.reduce(function(t,e,n){return t+(e?e.count:0)},0),p=new Float64Array(2*g),E=new Uint8Array(4*g),v=new Float32Array(4*g);m.forEach(function(t,e){if(t){var n=t.positions,r=t.count,i=t.offset,o=t.color,a=t.errors;r&&(E.set(o,4*i),v.set(a,4*i),p.set(n,2*i))}}),l(c(p)),h(u(p)),d(E),f(v)}}}function b(){l.destroy(),h.destroy(),d.destroy(),f.destroy(),p.destroy()}};var g=[[1,0,0,1,0,0],[1,0,0,-1,0,0],[-1,0,0,-1,0,0],[-1,0,0,-1,0,0],[-1,0,0,1,0,0],[1,0,0,1,0,0],[1,0,-1,0,0,1],[1,0,-1,0,0,-1],[1,0,1,0,0,-1],[1,0,1,0,0,-1],[1,0,1,0,0,1],[1,0,-1,0,0,1],[-1,0,-1,0,0,1],[-1,0,-1,0,0,-1],[-1,0,1,0,0,-1],[-1,0,1,0,0,-1],[-1,0,1,0,0,1],[-1,0,-1,0,0,1],[0,1,1,0,0,0],[0,1,-1,0,0,0],[0,-1,-1,0,0,0],[0,-1,-1,0,0,0],[0,1,1,0,0,0],[0,-1,1,0,0,0],[0,1,0,-1,1,0],[0,1,0,-1,-1,0],[0,1,0,1,-1,0],[0,1,0,1,1,0],[0,1,0,-1,1,0],[0,1,0,1,-1,0],[0,-1,0,-1,1,0],[0,-1,0,-1,-1,0],[0,-1,0,1,-1,0],[0,-1,0,1,1,0],[0,-1,0,-1,1,0],[0,-1,0,1,-1,0]]},{"array-bounds":65,"color-normalize":120,"flatten-vertex-data":227,"object-assign":454,"pick-by-alias":465,"to-float32":538,"update-diff":549}],491:[function(t,e,n){"use strict";var r=t("color-normalize"),i=t("array-bounds"),o=t("object-assign"),a=t("glslify"),s=t("pick-by-alias"),A=t("flatten-vertex-data"),l=t("earcut"),c=t("array-normalize"),u=t("to-float32"),g=u.float32,h=u.fract32,d=t("es6-weak-map"),f=t("parse-rect");function p(t,e){if(!(this instanceof p))return new p(t,e);if("function"==typeof t?(e||(e={}),e.regl=t):e=t,e.length&&(e.positions=e),!(t=e.regl).hasExtension("ANGLE_instanced_arrays"))throw Error("regl-error2d: `ANGLE_instanced_arrays` extension should be enabled");this.gl=t._gl,this.regl=t,this.passes=[],this.shaders=p.shaders.has(t)?p.shaders.get(t):p.shaders.set(t,p.createShaders(t)).get(t),this.update(e)}e.exports=p,p.dashMult=2,p.maxPatternLength=256,p.precisionThreshold=3e6,p.maxPoints=1e4,p.maxLines=2048,p.shaders=new d,p.createShaders=function(t){var e,n=t.buffer({usage:"static",type:"float",data:[0,1,0,0,1,1,1,0]}),r={primitive:"triangle strip",instances:t.prop("count"),count:4,offset:0,uniforms:{miterMode:function(t,e){return"round"===e.join?2:1},miterLimit:t.prop("miterLimit"),scale:t.prop("scale"),scaleFract:t.prop("scaleFract"),translateFract:t.prop("translateFract"),translate:t.prop("translate"),thickness:t.prop("thickness"),dashPattern:t.prop("dashTexture"),opacity:t.prop("opacity"),pixelRatio:t.context("pixelRatio"),id:t.prop("id"),dashSize:t.prop("dashLength"),viewport:function(t,e){return[e.viewport.x,e.viewport.y,t.viewportWidth,t.viewportHeight]},depth:t.prop("depth")},blend:{enable:!0,color:[0,0,0,0],equation:{rgb:"add",alpha:"add"},func:{srcRGB:"src alpha",dstRGB:"one minus src alpha",srcAlpha:"one minus dst alpha",dstAlpha:"one"}},depth:{enable:function(t,e){return!e.overlay}},stencil:{enable:!1},scissor:{enable:!0,box:t.prop("viewport")},viewport:t.prop("viewport")},i=t(o({vert:a(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec2 aCoord, bCoord, aCoordFract, bCoordFract;\nattribute vec4 color;\nattribute float lineEnd, lineTop;\n\nuniform vec2 scale, scaleFract, translate, translateFract;\nuniform float thickness, pixelRatio, id, depth;\nuniform vec4 viewport;\n\nvarying vec4 fragColor;\nvarying vec2 tangent;\n\nvec2 project(vec2 position, vec2 positionFract, vec2 scale, vec2 scaleFract, vec2 translate, vec2 translateFract) {\n\t// the order is important\n\treturn position * scale + translate\n       + positionFract * scale + translateFract\n       + position * scaleFract\n       + positionFract * scaleFract;\n}\n\nvoid main() {\n\tfloat lineStart = 1. - lineEnd;\n\tfloat lineOffset = lineTop * 2. - 1.;\n\n\tvec2 diff = (bCoord + bCoordFract - aCoord - aCoordFract);\n\ttangent = normalize(diff * scale * viewport.zw);\n\tvec2 normal = vec2(-tangent.y, tangent.x);\n\n\tvec2 position = project(aCoord, aCoordFract, scale, scaleFract, translate, translateFract) * lineStart\n\t\t+ project(bCoord, bCoordFract, scale, scaleFract, translate, translateFract) * lineEnd\n\n\t\t+ thickness * normal * .5 * lineOffset / viewport.zw;\n\n\tgl_Position = vec4(position * 2.0 - 1.0, depth, 1);\n\n\tfragColor = color / 255.;\n}\n"]),frag:a(["precision highp float;\n#define GLSLIFY 1\n\nuniform sampler2D dashPattern;\n\nuniform float dashSize, pixelRatio, thickness, opacity, id;\n\nvarying vec4 fragColor;\nvarying vec2 tangent;\n\nvoid main() {\n\tfloat alpha = 1.;\n\n\tfloat t = fract(dot(tangent, gl_FragCoord.xy) / dashSize) * .5 + .25;\n\tfloat dash = texture2D(dashPattern, vec2(t, .5)).r;\n\n\tgl_FragColor = fragColor;\n\tgl_FragColor.a *= alpha * opacity * dash;\n}\n"]),attributes:{lineEnd:{buffer:n,divisor:0,stride:8,offset:0},lineTop:{buffer:n,divisor:0,stride:8,offset:4},aCoord:{buffer:t.prop("positionBuffer"),stride:8,offset:8,divisor:1},bCoord:{buffer:t.prop("positionBuffer"),stride:8,offset:16,divisor:1},aCoordFract:{buffer:t.prop("positionFractBuffer"),stride:8,offset:8,divisor:1},bCoordFract:{buffer:t.prop("positionFractBuffer"),stride:8,offset:16,divisor:1},color:{buffer:t.prop("colorBuffer"),stride:4,offset:0,divisor:1}}},r));try{e=t(o({cull:{enable:!0,face:"back"},vert:a(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec2 aCoord, bCoord, nextCoord, prevCoord;\nattribute vec4 aColor, bColor;\nattribute float lineEnd, lineTop;\n\nuniform vec2 scale, translate;\nuniform float thickness, pixelRatio, id, depth;\nuniform vec4 viewport;\nuniform float miterLimit, miterMode;\n\nvarying vec4 fragColor;\nvarying vec4 startCutoff, endCutoff;\nvarying vec2 tangent;\nvarying vec2 startCoord, endCoord;\nvarying float enableStartMiter, enableEndMiter;\n\nconst float REVERSE_THRESHOLD = -.875;\nconst float MIN_DIFF = 1e-6;\n\n// TODO: possible optimizations: avoid overcalculating all for vertices and calc just one instead\n// TODO: precalculate dot products, normalize things beforehead etc.\n// TODO: refactor to rectangular algorithm\n\nfloat distToLine(vec2 p, vec2 a, vec2 b) {\n\tvec2 diff = b - a;\n\tvec2 perp = normalize(vec2(-diff.y, diff.x));\n\treturn dot(p - a, perp);\n}\n\nbool isNaN( float val ){\n  return ( val < 0.0 || 0.0 < val || val == 0.0 ) ? false : true;\n}\n\nvoid main() {\n\tvec2 aCoord = aCoord, bCoord = bCoord, prevCoord = prevCoord, nextCoord = nextCoord;\n\n  vec2 adjustedScale;\n  adjustedScale.x = (abs(scale.x) < MIN_DIFF) ? MIN_DIFF : scale.x;\n  adjustedScale.y = (abs(scale.y) < MIN_DIFF) ? MIN_DIFF : scale.y;\n\n  vec2 scaleRatio = adjustedScale * viewport.zw;\n\tvec2 normalWidth = thickness / scaleRatio;\n\n\tfloat lineStart = 1. - lineEnd;\n\tfloat lineBot = 1. - lineTop;\n\n\tfragColor = (lineStart * aColor + lineEnd * bColor) / 255.;\n\n\tif (isNaN(aCoord.x) || isNaN(aCoord.y) || isNaN(bCoord.x) || isNaN(bCoord.y)) return;\n\n\tif (aCoord == prevCoord) prevCoord = aCoord + normalize(bCoord - aCoord);\n\tif (bCoord == nextCoord) nextCoord = bCoord - normalize(bCoord - aCoord);\n\n\tvec2 prevDiff = aCoord - prevCoord;\n\tvec2 currDiff = bCoord - aCoord;\n\tvec2 nextDiff = nextCoord - bCoord;\n\n\tvec2 prevTangent = normalize(prevDiff * scaleRatio);\n\tvec2 currTangent = normalize(currDiff * scaleRatio);\n\tvec2 nextTangent = normalize(nextDiff * scaleRatio);\n\n\tvec2 prevNormal = vec2(-prevTangent.y, prevTangent.x);\n\tvec2 currNormal = vec2(-currTangent.y, currTangent.x);\n\tvec2 nextNormal = vec2(-nextTangent.y, nextTangent.x);\n\n\tvec2 startJoinDirection = normalize(prevTangent - currTangent);\n\tvec2 endJoinDirection = normalize(currTangent - nextTangent);\n\n\t// collapsed/unidirectional segment cases\n\t// FIXME: there should be more elegant solution\n\tvec2 prevTanDiff = abs(prevTangent - currTangent);\n\tvec2 nextTanDiff = abs(nextTangent - currTangent);\n\tif (max(prevTanDiff.x, prevTanDiff.y) < MIN_DIFF) {\n\t\tstartJoinDirection = currNormal;\n\t}\n\tif (max(nextTanDiff.x, nextTanDiff.y) < MIN_DIFF) {\n\t\tendJoinDirection = currNormal;\n\t}\n\tif (aCoord == bCoord) {\n\t\tendJoinDirection = startJoinDirection;\n\t\tcurrNormal = prevNormal;\n\t\tcurrTangent = prevTangent;\n\t}\n\n\ttangent = currTangent;\n\n\t//calculate join shifts relative to normals\n\tfloat startJoinShift = dot(currNormal, startJoinDirection);\n\tfloat endJoinShift = dot(currNormal, endJoinDirection);\n\n\tfloat startMiterRatio = abs(1. / startJoinShift);\n\tfloat endMiterRatio = abs(1. / endJoinShift);\n\n\tvec2 startJoin = startJoinDirection * startMiterRatio;\n\tvec2 endJoin = endJoinDirection * endMiterRatio;\n\n\tvec2 startTopJoin, startBotJoin, endTopJoin, endBotJoin;\n\tstartTopJoin = sign(startJoinShift) * startJoin * .5;\n\tstartBotJoin = -startTopJoin;\n\n\tendTopJoin = sign(endJoinShift) * endJoin * .5;\n\tendBotJoin = -endTopJoin;\n\n\tvec2 aTopCoord = aCoord + normalWidth * startTopJoin;\n\tvec2 bTopCoord = bCoord + normalWidth * endTopJoin;\n\tvec2 aBotCoord = aCoord + normalWidth * startBotJoin;\n\tvec2 bBotCoord = bCoord + normalWidth * endBotJoin;\n\n\t//miter anti-clipping\n\tfloat baClipping = distToLine(bCoord, aCoord, aBotCoord) / dot(normalize(normalWidth * endBotJoin), normalize(normalWidth.yx * vec2(-startBotJoin.y, startBotJoin.x)));\n\tfloat abClipping = distToLine(aCoord, bCoord, bTopCoord) / dot(normalize(normalWidth * startBotJoin), normalize(normalWidth.yx * vec2(-endBotJoin.y, endBotJoin.x)));\n\n\t//prevent close to reverse direction switch\n\tbool prevReverse = dot(currTangent, prevTangent) <= REVERSE_THRESHOLD && abs(dot(currTangent, prevNormal)) * min(length(prevDiff), length(currDiff)) <  length(normalWidth * currNormal);\n\tbool nextReverse = dot(currTangent, nextTangent) <= REVERSE_THRESHOLD && abs(dot(currTangent, nextNormal)) * min(length(nextDiff), length(currDiff)) <  length(normalWidth * currNormal);\n\n\tif (prevReverse) {\n\t\t//make join rectangular\n\t\tvec2 miterShift = normalWidth * startJoinDirection * miterLimit * .5;\n\t\tfloat normalAdjust = 1. - min(miterLimit / startMiterRatio, 1.);\n\t\taBotCoord = aCoord + miterShift - normalAdjust * normalWidth * currNormal * .5;\n\t\taTopCoord = aCoord + miterShift + normalAdjust * normalWidth * currNormal * .5;\n\t}\n\telse if (!nextReverse && baClipping > 0. && baClipping < length(normalWidth * endBotJoin)) {\n\t\t//handle miter clipping\n\t\tbTopCoord -= normalWidth * endTopJoin;\n\t\tbTopCoord += normalize(endTopJoin * normalWidth) * baClipping;\n\t}\n\n\tif (nextReverse) {\n\t\t//make join rectangular\n\t\tvec2 miterShift = normalWidth * endJoinDirection * miterLimit * .5;\n\t\tfloat normalAdjust = 1. - min(miterLimit / endMiterRatio, 1.);\n\t\tbBotCoord = bCoord + miterShift - normalAdjust * normalWidth * currNormal * .5;\n\t\tbTopCoord = bCoord + miterShift + normalAdjust * normalWidth * currNormal * .5;\n\t}\n\telse if (!prevReverse && abClipping > 0. && abClipping < length(normalWidth * startBotJoin)) {\n\t\t//handle miter clipping\n\t\taBotCoord -= normalWidth * startBotJoin;\n\t\taBotCoord += normalize(startBotJoin * normalWidth) * abClipping;\n\t}\n\n\tvec2 aTopPosition = (aTopCoord) * adjustedScale + translate;\n\tvec2 aBotPosition = (aBotCoord) * adjustedScale + translate;\n\n\tvec2 bTopPosition = (bTopCoord) * adjustedScale + translate;\n\tvec2 bBotPosition = (bBotCoord) * adjustedScale + translate;\n\n\t//position is normalized 0..1 coord on the screen\n\tvec2 position = (aTopPosition * lineTop + aBotPosition * lineBot) * lineStart + (bTopPosition * lineTop + bBotPosition * lineBot) * lineEnd;\n\n\tstartCoord = aCoord * scaleRatio + translate * viewport.zw + viewport.xy;\n\tendCoord = bCoord * scaleRatio + translate * viewport.zw + viewport.xy;\n\n\tgl_Position = vec4(position  * 2.0 - 1.0, depth, 1);\n\n\tenableStartMiter = step(dot(currTangent, prevTangent), .5);\n\tenableEndMiter = step(dot(currTangent, nextTangent), .5);\n\n\t//bevel miter cutoffs\n\tif (miterMode == 1.) {\n\t\tif (enableStartMiter == 1.) {\n\t\t\tvec2 startMiterWidth = vec2(startJoinDirection) * thickness * miterLimit * .5;\n\t\t\tstartCutoff = vec4(aCoord, aCoord);\n\t\t\tstartCutoff.zw += vec2(-startJoinDirection.y, startJoinDirection.x) / scaleRatio;\n\t\t\tstartCutoff = startCutoff * scaleRatio.xyxy + translate.xyxy * viewport.zwzw;\n\t\t\tstartCutoff += viewport.xyxy;\n\t\t\tstartCutoff += startMiterWidth.xyxy;\n\t\t}\n\n\t\tif (enableEndMiter == 1.) {\n\t\t\tvec2 endMiterWidth = vec2(endJoinDirection) * thickness * miterLimit * .5;\n\t\t\tendCutoff = vec4(bCoord, bCoord);\n\t\t\tendCutoff.zw += vec2(-endJoinDirection.y, endJoinDirection.x)  / scaleRatio;\n\t\t\tendCutoff = endCutoff * scaleRatio.xyxy + translate.xyxy * viewport.zwzw;\n\t\t\tendCutoff += viewport.xyxy;\n\t\t\tendCutoff += endMiterWidth.xyxy;\n\t\t}\n\t}\n\n\t//round miter cutoffs\n\telse if (miterMode == 2.) {\n\t\tif (enableStartMiter == 1.) {\n\t\t\tvec2 startMiterWidth = vec2(startJoinDirection) * thickness * abs(dot(startJoinDirection, currNormal)) * .5;\n\t\t\tstartCutoff = vec4(aCoord, aCoord);\n\t\t\tstartCutoff.zw += vec2(-startJoinDirection.y, startJoinDirection.x) / scaleRatio;\n\t\t\tstartCutoff = startCutoff * scaleRatio.xyxy + translate.xyxy * viewport.zwzw;\n\t\t\tstartCutoff += viewport.xyxy;\n\t\t\tstartCutoff += startMiterWidth.xyxy;\n\t\t}\n\n\t\tif (enableEndMiter == 1.) {\n\t\t\tvec2 endMiterWidth = vec2(endJoinDirection) * thickness * abs(dot(endJoinDirection, currNormal)) * .5;\n\t\t\tendCutoff = vec4(bCoord, bCoord);\n\t\t\tendCutoff.zw += vec2(-endJoinDirection.y, endJoinDirection.x)  / scaleRatio;\n\t\t\tendCutoff = endCutoff * scaleRatio.xyxy + translate.xyxy * viewport.zwzw;\n\t\t\tendCutoff += viewport.xyxy;\n\t\t\tendCutoff += endMiterWidth.xyxy;\n\t\t}\n\t}\n}\n"]),frag:a(["precision highp float;\n#define GLSLIFY 1\n\nuniform sampler2D dashPattern;\nuniform float dashSize, pixelRatio, thickness, opacity, id, miterMode;\n\nvarying vec4 fragColor;\nvarying vec2 tangent;\nvarying vec4 startCutoff, endCutoff;\nvarying vec2 startCoord, endCoord;\nvarying float enableStartMiter, enableEndMiter;\n\nfloat distToLine(vec2 p, vec2 a, vec2 b) {\n\tvec2 diff = b - a;\n\tvec2 perp = normalize(vec2(-diff.y, diff.x));\n\treturn dot(p - a, perp);\n}\n\nvoid main() {\n\tfloat alpha = 1., distToStart, distToEnd;\n\tfloat cutoff = thickness * .5;\n\n\t//bevel miter\n\tif (miterMode == 1.) {\n\t\tif (enableStartMiter == 1.) {\n\t\t\tdistToStart = distToLine(gl_FragCoord.xy, startCutoff.xy, startCutoff.zw);\n\t\t\tif (distToStart < -1.) {\n\t\t\t\tdiscard;\n\t\t\t\treturn;\n\t\t\t}\n\t\t\talpha *= min(max(distToStart + 1., 0.), 1.);\n\t\t}\n\n\t\tif (enableEndMiter == 1.) {\n\t\t\tdistToEnd = distToLine(gl_FragCoord.xy, endCutoff.xy, endCutoff.zw);\n\t\t\tif (distToEnd < -1.) {\n\t\t\t\tdiscard;\n\t\t\t\treturn;\n\t\t\t}\n\t\t\talpha *= min(max(distToEnd + 1., 0.), 1.);\n\t\t}\n\t}\n\n\t// round miter\n\telse if (miterMode == 2.) {\n\t\tif (enableStartMiter == 1.) {\n\t\t\tdistToStart = distToLine(gl_FragCoord.xy, startCutoff.xy, startCutoff.zw);\n\t\t\tif (distToStart < 0.) {\n\t\t\t\tfloat radius = length(gl_FragCoord.xy - startCoord);\n\n\t\t\t\tif(radius > cutoff + .5) {\n\t\t\t\t\tdiscard;\n\t\t\t\t\treturn;\n\t\t\t\t}\n\n\t\t\t\talpha -= smoothstep(cutoff - .5, cutoff + .5, radius);\n\t\t\t}\n\t\t}\n\n\t\tif (enableEndMiter == 1.) {\n\t\t\tdistToEnd = distToLine(gl_FragCoord.xy, endCutoff.xy, endCutoff.zw);\n\t\t\tif (distToEnd < 0.) {\n\t\t\t\tfloat radius = length(gl_FragCoord.xy - endCoord);\n\n\t\t\t\tif(radius > cutoff + .5) {\n\t\t\t\t\tdiscard;\n\t\t\t\t\treturn;\n\t\t\t\t}\n\n\t\t\t\talpha -= smoothstep(cutoff - .5, cutoff + .5, radius);\n\t\t\t}\n\t\t}\n\t}\n\n\tfloat t = fract(dot(tangent, gl_FragCoord.xy) / dashSize) * .5 + .25;\n\tfloat dash = texture2D(dashPattern, vec2(t, .5)).r;\n\n\tgl_FragColor = fragColor;\n\tgl_FragColor.a *= alpha * opacity * dash;\n}\n"]),attributes:{lineEnd:{buffer:n,divisor:0,stride:8,offset:0},lineTop:{buffer:n,divisor:0,stride:8,offset:4},aColor:{buffer:t.prop("colorBuffer"),stride:4,offset:0,divisor:1},bColor:{buffer:t.prop("colorBuffer"),stride:4,offset:4,divisor:1},prevCoord:{buffer:t.prop("positionBuffer"),stride:8,offset:0,divisor:1},aCoord:{buffer:t.prop("positionBuffer"),stride:8,offset:8,divisor:1},bCoord:{buffer:t.prop("positionBuffer"),stride:8,offset:16,divisor:1},nextCoord:{buffer:t.prop("positionBuffer"),stride:8,offset:24,divisor:1}}},r))}catch(t){e=i}return{fill:t({primitive:"triangle",elements:function(t,e){return e.triangles},offset:0,vert:a(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec2 position, positionFract;\n\nuniform vec4 color;\nuniform vec2 scale, scaleFract, translate, translateFract;\nuniform float pixelRatio, id;\nuniform vec4 viewport;\nuniform float opacity;\n\nvarying vec4 fragColor;\n\nconst float MAX_LINES = 256.;\n\nvoid main() {\n\tfloat depth = (MAX_LINES - 4. - id) / (MAX_LINES);\n\n\tvec2 position = position * scale + translate\n       + positionFract * scale + translateFract\n       + position * scaleFract\n       + positionFract * scaleFract;\n\n\tgl_Position = vec4(position * 2.0 - 1.0, depth, 1);\n\n\tfragColor = color / 255.;\n\tfragColor.a *= opacity;\n}\n"]),frag:a(["precision highp float;\n#define GLSLIFY 1\n\nvarying vec4 fragColor;\n\nvoid main() {\n\tgl_FragColor = fragColor;\n}\n"]),uniforms:{scale:t.prop("scale"),color:t.prop("fill"),scaleFract:t.prop("scaleFract"),translateFract:t.prop("translateFract"),translate:t.prop("translate"),opacity:t.prop("opacity"),pixelRatio:t.context("pixelRatio"),id:t.prop("id"),viewport:function(t,e){return[e.viewport.x,e.viewport.y,t.viewportWidth,t.viewportHeight]}},attributes:{position:{buffer:t.prop("positionBuffer"),stride:8,offset:8},positionFract:{buffer:t.prop("positionFractBuffer"),stride:8,offset:8}},blend:r.blend,depth:{enable:!1},scissor:r.scissor,stencil:r.stencil,viewport:r.viewport}),rect:i,miter:e}},p.defaults={dashes:null,join:"miter",miterLimit:1,thickness:10,cap:"square",color:"black",opacity:1,overlay:!1,viewport:null,range:null,close:!1,fill:null},p.prototype.render=function(){for(var t,e=[],n=arguments.length;n--;)e[n]=arguments[n];e.length&&(t=this).update.apply(t,e),this.draw()},p.prototype.draw=function(){for(var t=this,e=[],n=arguments.length;n--;)e[n]=arguments[n];return(e.length?e:this.passes).forEach(function(e,n){var r;if(e&&Array.isArray(e))return(r=t).draw.apply(r,e);"number"==typeof e&&(e=t.passes[e]),e&&e.count>1&&e.opacity&&(t.regl._refresh(),e.fill&&e.triangles&&e.triangles.length>2&&t.shaders.fill(e),e.thickness&&(e.scale[0]*e.viewport.width>p.precisionThreshold||e.scale[1]*e.viewport.height>p.precisionThreshold?t.shaders.rect(e):"rect"===e.join||!e.join&&(e.thickness<=2||e.count>=p.maxPoints)?t.shaders.rect(e):t.shaders.miter(e)))}),this},p.prototype.update=function(t){var e=this;if(t){null!=t.length?"number"==typeof t[0]&&(t=[{positions:t}]):Array.isArray(t)||(t=[t]);var n=this.regl,a=this.gl;if(t.forEach(function(t,u){var d=e.passes[u];if(void 0!==t)if(null!==t){if("number"==typeof t[0]&&(t={positions:t}),t=s(t,{positions:"positions points data coords",thickness:"thickness lineWidth lineWidths line-width linewidth width stroke-width strokewidth strokeWidth",join:"lineJoin linejoin join type mode",miterLimit:"miterlimit miterLimit",dashes:"dash dashes dasharray dash-array dashArray",color:"color colour stroke colors colours stroke-color strokeColor",fill:"fill fill-color fillColor",opacity:"alpha opacity",overlay:"overlay crease overlap intersect",close:"closed close closed-path closePath",range:"range dataBox",viewport:"viewport viewBox",hole:"holes hole hollow"}),d||(e.passes[u]=d={id:u,scale:null,scaleFract:null,translate:null,translateFract:null,count:0,hole:[],depth:0,dashLength:1,dashTexture:n.texture({channels:1,data:new Uint8Array([255]),width:1,height:1,mag:"linear",min:"linear"}),colorBuffer:n.buffer({usage:"dynamic",type:"uint8",data:new Uint8Array}),positionBuffer:n.buffer({usage:"dynamic",type:"float",data:new Uint8Array}),positionFractBuffer:n.buffer({usage:"dynamic",type:"float",data:new Uint8Array})},t=o({},p.defaults,t)),null!=t.thickness&&(d.thickness=parseFloat(t.thickness)),null!=t.opacity&&(d.opacity=parseFloat(t.opacity)),null!=t.miterLimit&&(d.miterLimit=parseFloat(t.miterLimit)),null!=t.overlay&&(d.overlay=!!t.overlay,u<p.maxLines&&(d.depth=2*(p.maxLines-1-u%p.maxLines)/p.maxLines-1)),null!=t.join&&(d.join=t.join),null!=t.hole&&(d.hole=t.hole),null!=t.fill&&(d.fill=t.fill?r(t.fill,"uint8"):null),null!=t.viewport&&(d.viewport=f(t.viewport)),d.viewport||(d.viewport=f([a.drawingBufferWidth,a.drawingBufferHeight])),null!=t.close&&(d.close=t.close),null===t.positions&&(t.positions=[]),t.positions){var I,C;if(t.positions.x&&t.positions.y){var m=t.positions.x,y=t.positions.y;C=d.count=Math.max(m.length,y.length),I=new Float64Array(2*C);for(var E=0;E<C;E++)I[2*E]=m[E],I[2*E+1]=y[E]}else I=A(t.positions,"float64"),C=d.count=Math.floor(I.length/2);var v=d.bounds=i(I,2);if(d.fill){for(var B=[],b={},Q=0,w=0,x=0,_=d.count;w<_;w++){var D=I[2*w],M=I[2*w+1];isNaN(D)||isNaN(M)||null==D||null==M?(D=I[2*Q],M=I[2*Q+1],b[w]=Q):Q=w,B[x++]=D,B[x++]=M}for(var k=l(B,d.hole||[]),S=0,L=k.length;S<L;S++)null!=b[k[S]]&&(k[S]=b[k[S]]);d.triangles=k}var R=new Float64Array(I);c(R,2,v);var N=new Float64Array(2*C+6);d.close?I[0]===I[2*C-2]&&I[1]===I[2*C-1]?(N[0]=R[2*C-4],N[1]=R[2*C-3]):(N[0]=R[2*C-2],N[1]=R[2*C-1]):(N[0]=R[0],N[1]=R[1]),N.set(R,2),d.close?I[0]===I[2*C-2]&&I[1]===I[2*C-1]?(N[2*C+2]=R[2],N[2*C+3]=R[3],d.count-=1):(N[2*C+2]=R[0],N[2*C+3]=R[1],N[2*C+4]=R[2],N[2*C+5]=R[3]):(N[2*C+2]=R[2*C-2],N[2*C+3]=R[2*C-1],N[2*C+4]=R[2*C-2],N[2*C+5]=R[2*C-1]),d.positionBuffer(g(N)),d.positionFractBuffer(h(N))}if(t.range?d.range=t.range:d.range||(d.range=d.bounds),(t.range||t.positions)&&d.count){var F=d.bounds,T=F[2]-F[0],U=F[3]-F[1],O=d.range[2]-d.range[0],G=d.range[3]-d.range[1];d.scale=[T/O,U/G],d.translate=[-d.range[0]/O+F[0]/O||0,-d.range[1]/G+F[1]/G||0],d.scaleFract=h(d.scale),d.translateFract=h(d.translate)}if(t.dashes){var H,j=0;if(!t.dashes||t.dashes.length<2)j=1,H=new Uint8Array([255,255,255,255,255,255,255,255]);else{j=0;for(var Y=0;Y<t.dashes.length;++Y)j+=t.dashes[Y];H=new Uint8Array(j*p.dashMult);for(var P=0,K=255,z=0;z<2;z++)for(var J=0;J<t.dashes.length;++J){for(var q=0,V=t.dashes[J]*p.dashMult*.5;q<V;++q)H[P++]=K;K^=255}}d.dashLength=j,d.dashTexture({channels:1,data:H,width:H.length,height:1,mag:"linear",min:"linear"},0,0)}if(t.color){var W=d.count,X=t.color;X||(X="transparent");var Z=new Uint8Array(4*W+4);if(Array.isArray(X)&&"number"!=typeof X[0]){for(var $=0;$<W;$++){var tt=r(X[$],"uint8");Z.set(tt,4*$)}Z.set(r(X[0],"uint8"),4*W)}else for(var et=r(X,"uint8"),nt=0;nt<W+1;nt++)Z.set(et,4*nt);d.colorBuffer({usage:"dynamic",type:"uint8",data:Z})}}else e.passes[u]=null}),t.length<this.passes.length){for(var u=t.length;u<this.passes.length;u++){var d=e.passes[u];d&&(d.colorBuffer.destroy(),d.positionBuffer.destroy(),d.dashTexture.destroy())}this.passes.length=t.length}for(var I=[],C=0;C<this.passes.length;C++)null!==e.passes[C]&&I.push(e.passes[C]);return this.passes=I,this}},p.prototype.destroy=function(){return this.passes.forEach(function(t){t.colorBuffer.destroy(),t.positionBuffer.destroy(),t.dashTexture.destroy()}),this.passes.length=0,this}},{"array-bounds":65,"array-normalize":66,"color-normalize":120,earcut:492,"es6-weak-map":493,"flatten-vertex-data":227,glslify:409,"object-assign":454,"parse-rect":459,"pick-by-alias":465,"to-float32":538}],492:[function(t,e,n){"use strict";function r(t,e,n){n=n||2;var r,s,A,l,c,h,f,p=e&&e.length,I=p?e[0]*n:t.length,C=i(t,0,I,n,!0),m=[];if(!C||C.next===C.prev)return m;if(p&&(C=function(t,e,n,r){var a,s,A,l,c,h=[];for(a=0,s=e.length;a<s;a++)A=e[a]*r,l=a<s-1?e[a+1]*r:t.length,(c=i(t,A,l,r,!1))===c.next&&(c.steiner=!0),h.push(d(c));for(h.sort(u),a=0;a<h.length;a++)g(h[a],n),n=o(n,n.next);return n}(t,e,C,n)),t.length>80*n){r=A=t[0],s=l=t[1];for(var y=n;y<I;y+=n)(c=t[y])<r&&(r=c),(h=t[y+1])<s&&(s=h),c>A&&(A=c),h>l&&(l=h);f=0!==(f=Math.max(A-r,l-s))?1/f:0}return a(C,m,n,r,s,f),m}function i(t,e,n,r,i){var o,a;if(i===Q(t,e,n,r)>0)for(o=e;o<n;o+=r)a=v(o,t[o],t[o+1],a);else for(o=n-r;o>=e;o-=r)a=v(o,t[o],t[o+1],a);return a&&C(a,a.next)&&(B(a),a=a.next),a}function o(t,e){if(!t)return t;e||(e=t);var n,r=t;do{if(n=!1,r.steiner||!C(r,r.next)&&0!==I(r.prev,r,r.next))r=r.next;else{if(B(r),(r=e=r.prev)===r.next)break;n=!0}}while(n||r!==e);return e}function a(t,e,n,r,i,u,g){if(t){!g&&u&&function(t,e,n,r){var i=t;do{null===i.z&&(i.z=h(i.x,i.y,e,n,r)),i.prevZ=i.prev,i.nextZ=i.next,i=i.next}while(i!==t);i.prevZ.nextZ=null,i.prevZ=null,function(t){var e,n,r,i,o,a,s,A,l=1;do{for(n=t,t=null,o=null,a=0;n;){for(a++,r=n,s=0,e=0;e<l&&(s++,r=r.nextZ);e++);for(A=l;s>0||A>0&&r;)0!==s&&(0===A||!r||n.z<=r.z)?(i=n,n=n.nextZ,s--):(i=r,r=r.nextZ,A--),o?o.nextZ=i:t=i,i.prevZ=o,o=i;n=r}o.nextZ=null,l*=2}while(a>1)}(i)}(t,r,i,u);for(var d,f,p=t;t.prev!==t.next;)if(d=t.prev,f=t.next,u?A(t,r,i,u):s(t))e.push(d.i/n),e.push(t.i/n),e.push(f.i/n),B(t),t=f.next,p=f.next;else if((t=f)===p){g?1===g?a(t=l(t,e,n),e,n,r,i,u,2):2===g&&c(t,e,n,r,i,u):a(o(t),e,n,r,i,u,1);break}}}function s(t){var e=t.prev,n=t,r=t.next;if(I(e,n,r)>=0)return!1;for(var i=t.next.next;i!==t.prev;){if(f(e.x,e.y,n.x,n.y,r.x,r.y,i.x,i.y)&&I(i.prev,i,i.next)>=0)return!1;i=i.next}return!0}function A(t,e,n,r){var i=t.prev,o=t,a=t.next;if(I(i,o,a)>=0)return!1;for(var s=i.x<o.x?i.x<a.x?i.x:a.x:o.x<a.x?o.x:a.x,A=i.y<o.y?i.y<a.y?i.y:a.y:o.y<a.y?o.y:a.y,l=i.x>o.x?i.x>a.x?i.x:a.x:o.x>a.x?o.x:a.x,c=i.y>o.y?i.y>a.y?i.y:a.y:o.y>a.y?o.y:a.y,u=h(s,A,e,n,r),g=h(l,c,e,n,r),d=t.prevZ,p=t.nextZ;d&&d.z>=u&&p&&p.z<=g;){if(d!==t.prev&&d!==t.next&&f(i.x,i.y,o.x,o.y,a.x,a.y,d.x,d.y)&&I(d.prev,d,d.next)>=0)return!1;if(d=d.prevZ,p!==t.prev&&p!==t.next&&f(i.x,i.y,o.x,o.y,a.x,a.y,p.x,p.y)&&I(p.prev,p,p.next)>=0)return!1;p=p.nextZ}for(;d&&d.z>=u;){if(d!==t.prev&&d!==t.next&&f(i.x,i.y,o.x,o.y,a.x,a.y,d.x,d.y)&&I(d.prev,d,d.next)>=0)return!1;d=d.prevZ}for(;p&&p.z<=g;){if(p!==t.prev&&p!==t.next&&f(i.x,i.y,o.x,o.y,a.x,a.y,p.x,p.y)&&I(p.prev,p,p.next)>=0)return!1;p=p.nextZ}return!0}function l(t,e,n){var r=t;do{var i=r.prev,o=r.next.next;!C(i,o)&&m(i,r,r.next,o)&&y(i,o)&&y(o,i)&&(e.push(i.i/n),e.push(r.i/n),e.push(o.i/n),B(r),B(r.next),r=t=o),r=r.next}while(r!==t);return r}function c(t,e,n,r,i,s){var A=t;do{for(var l=A.next.next;l!==A.prev;){if(A.i!==l.i&&p(A,l)){var c=E(A,l);return A=o(A,A.next),c=o(c,c.next),a(A,e,n,r,i,s),void a(c,e,n,r,i,s)}l=l.next}A=A.next}while(A!==t)}function u(t,e){return t.x-e.x}function g(t,e){if(e=function(t,e){var n,r=e,i=t.x,o=t.y,a=-1/0;do{if(o<=r.y&&o>=r.next.y&&r.next.y!==r.y){var s=r.x+(o-r.y)*(r.next.x-r.x)/(r.next.y-r.y);if(s<=i&&s>a){if(a=s,s===i){if(o===r.y)return r;if(o===r.next.y)return r.next}n=r.x<r.next.x?r:r.next}}r=r.next}while(r!==e);if(!n)return null;if(i===a)return n.prev;var A,l=n,c=n.x,u=n.y,g=1/0;for(r=n.next;r!==l;)i>=r.x&&r.x>=c&&i!==r.x&&f(o<u?i:a,o,c,u,o<u?a:i,o,r.x,r.y)&&((A=Math.abs(o-r.y)/(i-r.x))<g||A===g&&r.x>n.x)&&y(r,t)&&(n=r,g=A),r=r.next;return n}(t,e)){var n=E(e,t);o(n,n.next)}}function h(t,e,n,r,i){return(t=1431655765&((t=858993459&((t=252645135&((t=16711935&((t=32767*(t-n)*i)|t<<8))|t<<4))|t<<2))|t<<1))|(e=1431655765&((e=858993459&((e=252645135&((e=16711935&((e=32767*(e-r)*i)|e<<8))|e<<4))|e<<2))|e<<1))<<1}function d(t){var e=t,n=t;do{(e.x<n.x||e.x===n.x&&e.y<n.y)&&(n=e),e=e.next}while(e!==t);return n}function f(t,e,n,r,i,o,a,s){return(i-a)*(e-s)-(t-a)*(o-s)>=0&&(t-a)*(r-s)-(n-a)*(e-s)>=0&&(n-a)*(o-s)-(i-a)*(r-s)>=0}function p(t,e){return t.next.i!==e.i&&t.prev.i!==e.i&&!function(t,e){var n=t;do{if(n.i!==t.i&&n.next.i!==t.i&&n.i!==e.i&&n.next.i!==e.i&&m(n,n.next,t,e))return!0;n=n.next}while(n!==t);return!1}(t,e)&&y(t,e)&&y(e,t)&&function(t,e){var n=t,r=!1,i=(t.x+e.x)/2,o=(t.y+e.y)/2;do{n.y>o!=n.next.y>o&&n.next.y!==n.y&&i<(n.next.x-n.x)*(o-n.y)/(n.next.y-n.y)+n.x&&(r=!r),n=n.next}while(n!==t);return r}(t,e)}function I(t,e,n){return(e.y-t.y)*(n.x-e.x)-(e.x-t.x)*(n.y-e.y)}function C(t,e){return t.x===e.x&&t.y===e.y}function m(t,e,n,r){return!!(C(t,e)&&C(n,r)||C(t,r)&&C(n,e))||I(t,e,n)>0!=I(t,e,r)>0&&I(n,r,t)>0!=I(n,r,e)>0}function y(t,e){return I(t.prev,t,t.next)<0?I(t,e,t.next)>=0&&I(t,t.prev,e)>=0:I(t,e,t.prev)<0||I(t,t.next,e)<0}function E(t,e){var n=new b(t.i,t.x,t.y),r=new b(e.i,e.x,e.y),i=t.next,o=e.prev;return t.next=e,e.prev=t,n.next=i,i.prev=n,r.next=n,n.prev=r,o.next=r,r.prev=o,r}function v(t,e,n,r){var i=new b(t,e,n);return r?(i.next=r.next,i.prev=r,r.next.prev=i,r.next=i):(i.prev=i,i.next=i),i}function B(t){t.next.prev=t.prev,t.prev.next=t.next,t.prevZ&&(t.prevZ.nextZ=t.nextZ),t.nextZ&&(t.nextZ.prevZ=t.prevZ)}function b(t,e,n){this.i=t,this.x=e,this.y=n,this.prev=null,this.next=null,this.z=null,this.prevZ=null,this.nextZ=null,this.steiner=!1}function Q(t,e,n,r){for(var i=0,o=e,a=n-r;o<n;o+=r)i+=(t[a]-t[o])*(t[o+1]+t[a+1]),a=o;return i}e.exports=r,e.exports.default=r,r.deviation=function(t,e,n,r){var i=e&&e.length,o=i?e[0]*n:t.length,a=Math.abs(Q(t,0,o,n));if(i)for(var s=0,A=e.length;s<A;s++){var l=e[s]*n,c=s<A-1?e[s+1]*n:t.length;a-=Math.abs(Q(t,l,c,n))}var u=0;for(s=0;s<r.length;s+=3){var g=r[s]*n,h=r[s+1]*n,d=r[s+2]*n;u+=Math.abs((t[g]-t[d])*(t[h+1]-t[g+1])-(t[g]-t[h])*(t[d+1]-t[g+1]))}return 0===a&&0===u?0:Math.abs((u-a)/a)},r.flatten=function(t){for(var e=t[0][0].length,n={vertices:[],holes:[],dimensions:e},r=0,i=0;i<t.length;i++){for(var o=0;o<t[i].length;o++)for(var a=0;a<e;a++)n.vertices.push(t[i][o][a]);i>0&&(r+=t[i-1].length,n.holes.push(r))}return n}},{}],493:[function(t,e,n){arguments[4][318][0].apply(n,arguments)},{"./is-implemented":494,"./polyfill":496,dup:318}],494:[function(t,e,n){arguments[4][319][0].apply(n,arguments)},{dup:319}],495:[function(t,e,n){arguments[4][320][0].apply(n,arguments)},{dup:320}],496:[function(t,e,n){arguments[4][321][0].apply(n,arguments)},{"./is-native-implemented":495,d:151,dup:321,"es5-ext/object/is-value":194,"es5-ext/object/set-prototype-of":200,"es5-ext/object/valid-object":204,"es5-ext/object/valid-value":205,"es5-ext/string/random-uniq":210,"es6-iterator/for-of":212,"es6-iterator/get":213,"es6-symbol":219}],497:[function(t,e,n){"use strict";function r(t,e){return function(t){if(Array.isArray(t))return t}(t)||function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!e||n.length!==e);r=!0);}catch(t){i=!0,o=t}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(t,e)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}function i(t){return function(t){if(Array.isArray(t)){for(var e=0,n=new Array(t.length);e<t.length;e++)n[e]=t[e];return n}}(t)||function(t){if(Symbol.iterator in Object(t)||"[object Arguments]"===Object.prototype.toString.call(t))return Array.from(t)}(t)||function(){throw new TypeError("Invalid attempt to spread non-iterable instance")}()}var o=t("color-normalize"),a=t("array-bounds"),s=t("color-id"),A=t("point-cluster"),l=t("object-assign"),c=t("glslify"),u=t("pick-by-alias"),g=t("update-diff"),h=t("flatten-vertex-data"),d=t("is-iexplorer"),f=t("to-float32"),p=t("parse-rect"),I=C;function C(t,e){var n=this;if(!(this instanceof C))return new C(t,e);"function"==typeof t?(e||(e={}),e.regl=t):(e=t,t=null),e&&e.length&&(e.positions=e);var r,i=(t=e.regl)._gl,o=[];this.tooManyColors=d,r=t.texture({data:new Uint8Array(1020),width:255,height:1,type:"uint8",format:"rgba",wrapS:"clamp",wrapT:"clamp",mag:"nearest",min:"nearest"}),l(this,{regl:t,gl:i,groups:[],markerCache:[null],markerTextures:[null],palette:o,paletteIds:{},paletteTexture:r,maxColors:255,maxSize:100,canvas:i.canvas}),this.update(e);var a={uniforms:{pixelRatio:t.context("pixelRatio"),palette:r,paletteSize:function(t,e){return[n.tooManyColors?0:255,r.height]},scale:t.prop("scale"),scaleFract:t.prop("scaleFract"),translate:t.prop("translate"),translateFract:t.prop("translateFract"),opacity:t.prop("opacity"),marker:t.prop("markerTexture")},attributes:{x:function(t,e){return e.xAttr||{buffer:e.positionBuffer,stride:8,offset:0}},y:function(t,e){return e.yAttr||{buffer:e.positionBuffer,stride:8,offset:4}},xFract:function(t,e){return e.xAttr?{constant:[0,0]}:{buffer:e.positionFractBuffer,stride:8,offset:0}},yFract:function(t,e){return e.yAttr?{constant:[0,0]}:{buffer:e.positionFractBuffer,stride:8,offset:4}},size:function(t,e){return e.size.length?{buffer:e.sizeBuffer,stride:2,offset:0}:{constant:[Math.round(255*e.size/n.maxSize)]}},borderSize:function(t,e){return e.borderSize.length?{buffer:e.sizeBuffer,stride:2,offset:1}:{constant:[Math.round(255*e.borderSize/n.maxSize)]}},colorId:function(t,e){return e.color.length?{buffer:e.colorBuffer,stride:n.tooManyColors?8:4,offset:0}:{constant:n.tooManyColors?o.slice(4*e.color,4*e.color+4):[e.color]}},borderColorId:function(t,e){return e.borderColor.length?{buffer:e.colorBuffer,stride:n.tooManyColors?8:4,offset:n.tooManyColors?4:2}:{constant:n.tooManyColors?o.slice(4*e.borderColor,4*e.borderColor+4):[e.borderColor]}},isActive:function(t,e){return!0===e.activation?{constant:[1]}:e.activation?e.activation:{constant:[0]}}},blend:{enable:!0,color:[0,0,0,1],func:{srcRGB:"src alpha",dstRGB:"one minus src alpha",srcAlpha:"one minus dst alpha",dstAlpha:"one"}},scissor:{enable:!0,box:t.prop("viewport")},viewport:t.prop("viewport"),stencil:{enable:!1},depth:{enable:!1},elements:t.prop("elements"),count:t.prop("count"),offset:t.prop("offset"),primitive:"points"},s=l({},a);s.frag=c(["precision highp float;\n#define GLSLIFY 1\n\nvarying vec4 fragColor, fragBorderColor;\nvarying float fragWidth, fragBorderColorLevel, fragColorLevel;\n\nuniform sampler2D marker;\nuniform float pixelRatio, opacity;\n\nfloat smoothStep(float x, float y) {\n  return 1.0 / (1.0 + exp(50.0*(x - y)));\n}\n\nvoid main() {\n  float dist = texture2D(marker, gl_PointCoord).r, delta = fragWidth;\n\n  // max-distance alpha\n  if (dist < 0.003) discard;\n\n  // null-border case\n  if (fragBorderColorLevel == fragColorLevel || fragBorderColor.a == 0.) {\n    float colorAmt = smoothstep(.5 - delta, .5 + delta, dist);\n    gl_FragColor = vec4(fragColor.rgb, colorAmt * fragColor.a * opacity);\n  }\n  else {\n    float borderColorAmt = smoothstep(fragBorderColorLevel - delta, fragBorderColorLevel + delta, dist);\n    float colorAmt = smoothstep(fragColorLevel - delta, fragColorLevel + delta, dist);\n\n    vec4 color = fragBorderColor;\n    color.a *= borderColorAmt;\n    color = mix(color, fragColor, colorAmt);\n    color.a *= opacity;\n\n    gl_FragColor = color;\n  }\n\n}\n"]),s.vert=c(["precision highp float;\n#define GLSLIFY 1\n\nattribute float x, y, xFract, yFract;\nattribute float size, borderSize;\nattribute vec4 colorId, borderColorId;\nattribute float isActive;\n\nuniform vec2 scale, scaleFract, translate, translateFract, paletteSize;\nuniform float pixelRatio;\nuniform sampler2D palette;\n\nconst float maxSize = 100.;\nconst float borderLevel = .5;\n\nvarying vec4 fragColor, fragBorderColor;\nvarying float fragPointSize, fragBorderRadius, fragWidth, fragBorderColorLevel, fragColorLevel;\n\nbool isDirect = (paletteSize.x < 1.);\n\nvec4 getColor(vec4 id) {\n  return isDirect ? id / 255. : texture2D(palette,\n    vec2(\n      (id.x + .5) / paletteSize.x,\n      (id.y + .5) / paletteSize.y\n    )\n  );\n}\n\nvoid main() {\n  if (isActive == 0.) return;\n\n  vec2 position = vec2(x, y);\n  vec2 positionFract = vec2(xFract, yFract);\n\n  vec4 color = getColor(colorId);\n  vec4 borderColor = getColor(borderColorId);\n\n  float size = size * maxSize / 255.;\n  float borderSize = borderSize * maxSize / 255.;\n\n  gl_PointSize = 2. * size * pixelRatio;\n  fragPointSize = size * pixelRatio;\n\n  vec2 pos = (position + translate) * scale\n      + (positionFract + translateFract) * scale\n      + (position + translate) * scaleFract\n      + (positionFract + translateFract) * scaleFract;\n\n  gl_Position = vec4(pos * 2. - 1., 0, 1);\n\n  fragColor = color;\n  fragBorderColor = borderColor;\n  fragWidth = 1. / gl_PointSize;\n\n  fragBorderColorLevel = clamp(borderLevel - borderLevel * borderSize / size, 0., 1.);\n  fragColorLevel = clamp(borderLevel + (1. - borderLevel) * borderSize / size, 0., 1.);\n}"]),this.drawMarker=t(s);var A=l({},a);A.frag=c(["precision highp float;\n#define GLSLIFY 1\n\nvarying vec4 fragColor, fragBorderColor;\n\nuniform float opacity;\nvarying float fragBorderRadius, fragWidth;\n\nfloat smoothStep(float edge0, float edge1, float x) {\n\tfloat t;\n\tt = clamp((x - edge0) / (edge1 - edge0), 0.0, 1.0);\n\treturn t * t * (3.0 - 2.0 * t);\n}\n\nvoid main() {\n\tfloat radius, alpha = 1.0, delta = fragWidth;\n\n\tradius = length(2.0 * gl_PointCoord.xy - 1.0);\n\n\tif (radius > 1.0 + delta) {\n\t\tdiscard;\n\t}\n\n\talpha -= smoothstep(1.0 - delta, 1.0 + delta, radius);\n\n\tfloat borderRadius = fragBorderRadius;\n\tfloat ratio = smoothstep(borderRadius - delta, borderRadius + delta, radius);\n\tvec4 color = mix(fragColor, fragBorderColor, ratio);\n\tcolor.a *= alpha * opacity;\n\tgl_FragColor = color;\n}\n"]),A.vert=c(["precision highp float;\n#define GLSLIFY 1\n\nattribute float x, y, xFract, yFract;\nattribute float size, borderSize;\nattribute vec4 colorId, borderColorId;\nattribute float isActive;\n\nuniform vec2 scale, scaleFract, translate, translateFract;\nuniform float pixelRatio;\nuniform sampler2D palette;\nuniform vec2 paletteSize;\n\nconst float maxSize = 100.;\n\nvarying vec4 fragColor, fragBorderColor;\nvarying float fragBorderRadius, fragWidth;\n\nbool isDirect = (paletteSize.x < 1.);\n\nvec4 getColor(vec4 id) {\n  return isDirect ? id / 255. : texture2D(palette,\n    vec2(\n      (id.x + .5) / paletteSize.x,\n      (id.y + .5) / paletteSize.y\n    )\n  );\n}\n\nvoid main() {\n  // ignore inactive points\n  if (isActive == 0.) return;\n\n  vec2 position = vec2(x, y);\n  vec2 positionFract = vec2(xFract, yFract);\n\n  vec4 color = getColor(colorId);\n  vec4 borderColor = getColor(borderColorId);\n\n  float size = size * maxSize / 255.;\n  float borderSize = borderSize * maxSize / 255.;\n\n  gl_PointSize = (size + borderSize) * pixelRatio;\n\n  vec2 pos = (position + translate) * scale\n      + (positionFract + translateFract) * scale\n      + (position + translate) * scaleFract\n      + (positionFract + translateFract) * scaleFract;\n\n  gl_Position = vec4(pos * 2. - 1., 0, 1);\n\n  fragBorderRadius = 1. - 2. * borderSize / (size + borderSize);\n  fragColor = color;\n  fragBorderColor = borderColor.a == 0. || borderSize == 0. ? vec4(color.rgb, 0.) : borderColor;\n  fragWidth = 1. / gl_PointSize;\n}\n"]),d&&(A.frag=A.frag.replace("smoothstep","smoothStep"),s.frag=s.frag.replace("smoothstep","smoothStep")),this.drawCircle=t(A)}C.defaults={color:"black",borderColor:"transparent",borderSize:0,size:12,opacity:1,marker:void 0,viewport:null,range:null,pixelSize:null,count:0,offset:0,bounds:null,positions:[],snap:1e4},C.prototype.render=function(){return arguments.length&&this.update.apply(this,arguments),this.draw(),this},C.prototype.draw=function(){for(var t=this,e=arguments.length,n=new Array(e),r=0;r<e;r++)n[r]=arguments[r];var i=this.groups;if(1===n.length&&Array.isArray(n[0])&&(null===n[0][0]||Array.isArray(n[0][0]))&&(n=n[0]),this.regl._refresh(),n.length)for(var o=0;o<n.length;o++)this.drawItem(o,n[o]);else i.forEach(function(e,n){t.drawItem(n)});return this},C.prototype.drawItem=function(t,e){var n=this.groups,r=n[t];if("number"==typeof e&&(t=e,r=n[e],e=null),r&&r.count&&r.opacity){r.activation[0]&&this.drawCircle(this.getMarkerDrawOptions(0,r,e));for(var o=[],a=1;a<r.activation.length;a++)r.activation[a]&&(!0===r.activation[a]||r.activation[a].data.length)&&o.push.apply(o,i(this.getMarkerDrawOptions(a,r,e)));o.length&&this.drawMarker(o)}},C.prototype.getMarkerDrawOptions=function(t,e,n){var i=e.range,o=e.tree,a=e.viewport,s=e.activation,A=e.selectionBuffer,c=e.count;if(this.regl,!o)return n?[l({},e,{markerTexture:this.markerTextures[t],activation:s[t],count:n.length,elements:n,offset:0})]:[l({},e,{markerTexture:this.markerTextures[t],activation:s[t],offset:0})];var u=[],g=o.range(i,{lod:!0,px:[(i[2]-i[0])/a.width,(i[3]-i[1])/a.height]});if(n){for(var h=s[t].data,d=new Uint8Array(c),f=0;f<n.length;f++){var p=n[f];d[p]=h?h[p]:1}A.subdata(d)}for(var I=g.length;I--;){var C=r(g[I],2),m=C[0],y=C[1];u.push(l({},e,{markerTexture:this.markerTextures[t],activation:n?A:s[t],offset:m,count:y-m}))}return u},C.prototype.update=function(){for(var t=this,e=arguments.length,n=new Array(e),r=0;r<e;r++)n[r]=arguments[r];if(n.length){1===n.length&&Array.isArray(n[0])&&(n=n[0]);var i=this.groups,o=this.gl,s=this.regl,c=this.maxSize,d=this.maxColors,I=this.palette;this.groups=i=n.map(function(e,n){var r=i[n];if(void 0===e)return r;null===e?e={positions:null}:"function"==typeof e?e={ondraw:e}:"number"==typeof e[0]&&(e={positions:e}),null===(e=u(e,{positions:"positions data points",snap:"snap cluster lod tree",size:"sizes size radius",borderSize:"borderSizes borderSize border-size bordersize borderWidth borderWidths border-width borderwidth stroke-width strokeWidth strokewidth outline",color:"colors color fill fill-color fillColor",borderColor:"borderColors borderColor stroke stroke-color strokeColor",marker:"markers marker shape",range:"range dataBox databox",viewport:"viewport viewPort viewBox viewbox",opacity:"opacity alpha transparency",bounds:"bound bounds boundaries limits",tooManyColors:"tooManyColors palette paletteMode optimizePalette enablePalette"})).positions&&(e.positions=[]),null!=e.tooManyColors&&(t.tooManyColors=e.tooManyColors),r||(i[n]=r={id:n,scale:null,translate:null,scaleFract:null,translateFract:null,activation:[],selectionBuffer:s.buffer({data:new Uint8Array(0),usage:"stream",type:"uint8"}),sizeBuffer:s.buffer({data:new Uint8Array(0),usage:"dynamic",type:"uint8"}),colorBuffer:s.buffer({data:new Uint8Array(0),usage:"dynamic",type:"uint8"}),positionBuffer:s.buffer({data:new Uint8Array(0),usage:"dynamic",type:"float"}),positionFractBuffer:s.buffer({data:new Uint8Array(0),usage:"dynamic",type:"float"})},e=l({},C.defaults,e)),!e.positions||"marker"in e||(e.marker=r.marker,delete r.marker),!e.marker||"positions"in e||(e.positions=r.positions,delete r.positions);var m=0,y=0;if(g(r,e,[{snap:!0,size:function(t,e){return null==t&&(t=C.defaults.size),m+=t&&t.length?1:0,t},borderSize:function(t,e){return null==t&&(t=C.defaults.borderSize),m+=t&&t.length?1:0,t},opacity:parseFloat,color:function(e,n){return null==e&&(e=C.defaults.color),e=t.updateColor(e),y++,e},borderColor:function(e,n){return null==e&&(e=C.defaults.borderColor),e=t.updateColor(e),y++,e},bounds:function(t,e,n){return"range"in n||(n.range=null),t},positions:function(t,e,n){var r=e.snap,i=e.positionBuffer,o=e.positionFractBuffer,l=e.selectionBuffer;if(t.x||t.y)return t.x.length?e.xAttr={buffer:s.buffer(t.x),offset:0,stride:4,count:t.x.length}:e.xAttr={buffer:t.x.buffer,offset:4*t.x.offset||0,stride:4*(t.x.stride||1),count:t.x.count},t.y.length?e.yAttr={buffer:s.buffer(t.y),offset:0,stride:4,count:t.y.length}:e.yAttr={buffer:t.y.buffer,offset:4*t.y.offset||0,stride:4*(t.y.stride||1),count:t.y.count},e.count=Math.max(e.xAttr.count,e.yAttr.count),t;t=h(t,"float64");var c=e.count=Math.floor(t.length/2),u=e.bounds=c?a(t,2):null;if(n.range||e.range||(delete e.range,n.range=u),n.marker||e.marker||(delete e.marker,n.marker=null),r&&(!0===r||c>r)?e.tree=A(t,{bounds:u}):r&&r.length&&(e.tree=r),e.tree){var g={primitive:"points",usage:"static",data:e.tree,type:"uint32"};e.elements?e.elements(g):e.elements=s.elements(g)}return i({data:f.float(t),usage:"dynamic"}),o({data:f.fract(t),usage:"dynamic"}),l({data:new Uint8Array(c),type:"uint8",usage:"stream"}),t}},{marker:function(e,n,r){var i=n.activation;if(i.forEach(function(t){return t&&t.destroy&&t.destroy()}),i.length=0,e&&"number"!=typeof e[0]){for(var o=[],a=0,A=Math.min(e.length,n.count);a<A;a++){var l=t.addMarker(e[a]);o[l]||(o[l]=new Uint8Array(n.count)),o[l][a]=1}for(var c=0;c<o.length;c++)if(o[c]){var u={data:o[c],type:"uint8",usage:"static"};i[c]?i[c](u):i[c]=s.buffer(u),i[c].data=o[c]}}else i[t.addMarker(e)]=!0;return e},range:function(t,e,n){var r=e.bounds;if(r)return t||(t=r),e.scale=[1/(t[2]-t[0]),1/(t[3]-t[1])],e.translate=[-t[0],-t[1]],e.scaleFract=f.fract(e.scale),e.translateFract=f.fract(e.translate),t},viewport:function(t){return p(t||[o.drawingBufferWidth,o.drawingBufferHeight])}}]),m){var E=r,v=E.count,B=E.size,b=E.borderSize,Q=E.sizeBuffer,w=new Uint8Array(2*v);if(B.length||b.length)for(var x=0;x<v;x++)w[2*x]=Math.round(255*(null==B[x]?B:B[x])/c),w[2*x+1]=Math.round(255*(null==b[x]?b:b[x])/c);Q({data:w,usage:"dynamic"})}if(y){var _,D=r,M=D.count,k=D.color,S=D.borderColor,L=D.colorBuffer;if(t.tooManyColors){if(k.length||S.length){_=new Uint8Array(8*M);for(var R=0;R<M;R++){var N=k[R];_[8*R]=I[4*N],_[8*R+1]=I[4*N+1],_[8*R+2]=I[4*N+2],_[8*R+3]=I[4*N+3];var F=S[R];_[8*R+4]=I[4*F],_[8*R+5]=I[4*F+1],_[8*R+6]=I[4*F+2],_[8*R+7]=I[4*F+3]}}}else if(k.length||S.length){_=new Uint8Array(4*M+2);for(var T=0;T<M;T++)null!=k[T]&&(_[4*T]=k[T]%d,_[4*T+1]=Math.floor(k[T]/d)),null!=S[T]&&(_[4*T+2]=S[T]%d,_[4*T+3]=Math.floor(S[T]/d))}L({data:_||new Uint8Array(0),type:"uint8",usage:"dynamic"})}return r})}},C.prototype.addMarker=function(t){var e,n=this.markerTextures,r=this.regl,i=this.markerCache,o=null==t?0:i.indexOf(t);if(o>=0)return o;if(t instanceof Uint8Array||t instanceof Uint8ClampedArray)e=t;else{e=new Uint8Array(t.length);for(var a=0,s=t.length;a<s;a++)e[a]=255*t[a]}var A=Math.floor(Math.sqrt(e.length));return o=n.length,i.push(t),n.push(r.texture({channels:1,data:e,radius:A,mag:"linear",min:"linear"})),o},C.prototype.updateColor=function(t){var e=this.paletteIds,n=this.palette,r=this.maxColors;Array.isArray(t)||(t=[t]);var i=[];if("number"==typeof t[0]){var a=[];if(Array.isArray(t))for(var A=0;A<t.length;A+=4)a.push(t.slice(A,A+4));else for(var l=0;l<t.length;l+=4)a.push(t.subarray(l,l+4));t=a}for(var c=0;c<t.length;c++){var u=t[c];u=o(u,"uint8");var g=s(u,!1);if(null==e[g]){var h=n.length;e[g]=Math.floor(h/4),n[h]=u[0],n[h+1]=u[1],n[h+2]=u[2],n[h+3]=u[3]}i[c]=e[g]}return!this.tooManyColors&&n.length>4*r&&(this.tooManyColors=!0),this.updatePalette(n),1===i.length?i[0]:i},C.prototype.updatePalette=function(t){if(!this.tooManyColors){var e=this.maxColors,n=this.paletteTexture,r=Math.ceil(.25*t.length/e);if(r>1)for(var i=.25*(t=t.slice()).length%e;i<r*e;i++)t.push(0,0,0,0);n.height<r&&n.resize(e,r),n.subimage({width:Math.min(.25*t.length,e),height:r,data:t},0,0)}},C.prototype.destroy=function(){return this.groups.forEach(function(t){t.sizeBuffer.destroy(),t.positionBuffer.destroy(),t.positionFractBuffer.destroy(),t.colorBuffer.destroy(),t.activation.forEach(function(t){return t&&t.destroy&&t.destroy()}),t.selectionBuffer.destroy(),t.elements&&t.elements.destroy()}),this.groups.length=0,this.paletteTexture.destroy(),this.markerTextures.forEach(function(t){return t&&t.destroy&&t.destroy()}),this};var m=t("object-assign");e.exports=function(t,e){var n=new I(t,e),r=n.render.bind(n);return m(r,{render:r,update:n.update.bind(n),draw:n.draw.bind(n),destroy:n.destroy.bind(n),regl:n.regl,gl:n.gl,canvas:n.gl.canvas,groups:n.groups,markers:n.markerCache,palette:n.palette}),r}},{"array-bounds":65,"color-id":118,"color-normalize":120,"flatten-vertex-data":227,glslify:409,"is-iexplorer":419,"object-assign":454,"parse-rect":459,"pick-by-alias":465,"point-cluster":499,"to-float32":538,"update-diff":549}],498:[function(t,e,n){arguments[4][111][0].apply(n,arguments)},{dup:111}],499:[function(t,e,n){"use strict";e.exports=t("./quad")},{"./quad":500}],500:[function(t,e,n){"use strict";var r=t("binary-search-bounds"),i=t("clamp"),o=t("parse-rect"),a=t("array-bounds"),s=t("pick-by-alias"),A=t("defined"),l=t("flatten-vertex-data"),c=t("is-obj"),u=t("dtype"),g=t("math-log2");function h(t,e){for(var n=e[0],r=e[1],o=1/(e[2]-n),a=1/(e[3]-r),s=new Array(t.length),A=0,l=t.length/2;A<l;A++)s[2*A]=i((t[2*A]-n)*o,0,1),s[2*A+1]=i((t[2*A+1]-r)*a,0,1);return s}e.exports=function(t,e){e||(e={}),t=l(t,"float64"),e=s(e,{bounds:"range bounds dataBox databox",maxDepth:"depth maxDepth maxdepth level maxLevel maxlevel levels",dtype:"type dtype format out dst output destination"});var n=A(e.maxDepth,255),i=A(e.bounds,a(t,2));i[0]===i[2]&&i[2]++,i[1]===i[3]&&i[3]++;var d,f=h(t,i),p=t.length>>>1;e.dtype||(e.dtype="array"),"string"==typeof e.dtype?d=new(u(e.dtype))(p):e.dtype&&(d=e.dtype,Array.isArray(d)&&(d.length=p));for(var I=0;I<p;++I)d[I]=I;var C=[],m=[],y=[],E=[];!function t(e,r,i,o,a,s){if(!o.length)return null;var A=C[a]||(C[a]=[]),l=y[a]||(y[a]=[]),c=m[a]||(m[a]=[]),u=A.length;if(++a>n||s>1073741824){for(var g=0;g<o.length;g++)A.push(o[g]),l.push(s),c.push(null,null,null,null);return u}if(A.push(o[0]),l.push(s),o.length<=1)return c.push(null,null,null,null),u;for(var h=.5*i,d=e+h,p=r+h,I=[],E=[],v=[],B=[],b=1,Q=o.length;b<Q;b++){var w=o[b],x=f[2*w],_=f[2*w+1];x<d?_<p?I.push(w):E.push(w):_<p?v.push(w):B.push(w)}return s<<=2,c.push(t(e,r,h,I,a,s),t(e,p,h,E,a,s+1),t(d,r,h,v,a,s+2),t(d,p,h,B,a,s+3)),u}(0,0,1,d,0,1);for(var v=0,B=0;B<C.length;B++){var b=C[B];if(d.set)d.set(b,v);else for(var Q=0,w=b.length;Q<w;Q++)d[Q+v]=b[Q];var x=v+C[B].length;E[B]=[v,x],v=x}return d.range=function(){for(var e,n=[],a=arguments.length;a--;)n[a]=arguments[a];if(c(n[n.length-1])){var l=n.pop();n.length||null==l.x&&null==l.l&&null==l.left||(n=[l],e={}),e=s(l,{level:"level maxLevel",d:"d diam diameter r radius px pxSize pixel pixelSize maxD size minSize",lod:"lod details ranges offsets"})}else e={};n.length||(n=i);var u,d=o.apply(void 0,n),f=[Math.min(d.x,d.x+d.width),Math.min(d.y,d.y+d.height),Math.max(d.x,d.x+d.width),Math.max(d.y,d.y+d.height)],p=f[0],I=f[1],v=f[2],B=f[3],b=h([p,I,v,B],i),Q=b[0],w=b[1],x=b[2],D=b[3],M=A(e.level,C.length);if(null!=e.d&&("number"==typeof e.d?u=[e.d,e.d]:e.d.length&&(u=e.d),M=Math.min(Math.max(Math.ceil(-g(Math.abs(u[0])/(i[2]-i[0]))),Math.ceil(-g(Math.abs(u[1])/(i[3]-i[1])))),M)),M=Math.min(M,C.length),e.lod)return function(t,e,n,i,o){for(var a=[],s=0;s<o;s++){var A=y[s],l=E[s][0],c=_(t,e,s),u=_(n,i,s),g=r.ge(A,c),h=r.gt(A,u,g,A.length-1);a[s]=[g+l,h+l]}return a}(Q,w,x,D,M);var k=[];return function e(n,r,i,o,a,s){if(null!==a&&null!==s){var A=n+i,l=r+i;if(!(Q>A||w>l||x<n||D<r||o>=M||a===s)){var c=C[o];void 0===s&&(s=c.length);for(var u=a;u<s;u++){var g=c[u],h=t[2*g],d=t[2*g+1];h>=p&&h<=v&&d>=I&&d<=B&&k.push(g)}var f=m[o],y=f[4*a+0],E=f[4*a+1],b=f[4*a+2],_=f[4*a+3],S=function(t,e){for(var n=null,r=0;null===n;)if(n=t[4*e+r],++r>t.length)return null;return n}(f,a+1),L=.5*i,R=o+1;e(n,r,L,R,y,E||b||_||S),e(n,r+L,L,R,E,b||_||S),e(n+L,r,L,R,b,_||S),e(n+L,r+L,L,R,_,S)}}}(0,0,1,0,0,1),k},d;function _(t,e,n){for(var r=1,i=.5,o=.5,a=.5,s=0;s<n;s++)r<<=2,r+=t<i?e<o?0:1:e<o?2:3,a*=.5,i+=t<i?-a:a,o+=e<o?-a:a;return r}}},{"array-bounds":65,"binary-search-bounds":498,clamp:115,defined:164,dtype:169,"flatten-vertex-data":227,"is-obj":421,"math-log2":432,"parse-rect":459,"pick-by-alias":465}],501:[function(t,e,n){"use strict";var r=t("regl-scatter2d"),i=t("pick-by-alias"),o=t("array-bounds"),a=t("raf"),s=t("array-range"),A=t("parse-rect"),l=t("flatten-vertex-data");function c(t,e){if(!(this instanceof c))return new c(t,e);this.traces=[],this.passes={},this.regl=t,this.scatter=r(t),this.canvas=this.scatter.canvas}function u(t,e,n){return(null!=t.id?t.id:t)<<16|(255&e)<<8|255&n}function g(t,e,n){var r,i,o,a,s=t[e],A=t[n];return s.length>2?(s[0],s[2],r=s[1],i=s[3]):s.length?(r=s[0],i=s[1]):(s.x,r=s.y,s.x,s.width,i=s.y+s.height),A.length>2?(o=A[0],a=A[2],A[1],A[3]):A.length?(o=A[0],a=A[1]):(o=A.x,A.y,a=A.x+A.width,A.y,A.height),[o,r,a,i]}function h(t){if("number"==typeof t)return[t,t,t,t];if(2===t.length)return[t[0],t[1],t[0],t[1]];var e=A(t);return[e.x,e.y,e.x+e.width,e.y+e.height]}e.exports=c,c.prototype.render=function(){for(var t,e=this,n=[],r=arguments.length;r--;)n[r]=arguments[r];return n.length&&(t=this).update.apply(t,n),this.regl.attributes.preserveDrawingBuffer?this.draw():(this.dirty?null==this.planned&&(this.planned=a(function(){e.draw(),e.dirty=!0,e.planned=null})):(this.draw(),this.dirty=!0,a(function(){e.dirty=!1})),this)},c.prototype.update=function(){for(var t,e=[],n=arguments.length;n--;)e[n]=arguments[n];if(e.length){for(var r=0;r<e.length;r++)this.updateItem(r,e[r]);this.traces=this.traces.filter(Boolean);for(var i=[],o=0,a=0;a<this.traces.length;a++){for(var s=this.traces[a],A=this.traces[a].passes,l=0;l<A.length;l++)i.push(this.passes[A[l]]);s.passOffset=o,o+=s.passes.length}return(t=this.scatter).update.apply(t,i),this}},c.prototype.updateItem=function(t,e){var n=this.regl;if(null===e)return this.traces[t]=null,this;if(!e)return this;var r,a=i(e,{data:"data items columns rows values dimensions samples x",snap:"snap cluster",size:"sizes size radius",color:"colors color fill fill-color fillColor",opacity:"opacity alpha transparency opaque",borderSize:"borderSizes borderSize border-size bordersize borderWidth borderWidths border-width borderwidth stroke-width strokeWidth strokewidth outline",borderColor:"borderColors borderColor bordercolor stroke stroke-color strokeColor",marker:"markers marker shape",range:"range ranges databox dataBox",viewport:"viewport viewBox viewbox",domain:"domain domains area areas",padding:"pad padding paddings pads margin margins",transpose:"transpose transposed",diagonal:"diagonal diag showDiagonal",upper:"upper up top upperhalf upperHalf showupperhalf showUpper showUpperHalf",lower:"lower low bottom lowerhalf lowerHalf showlowerhalf showLowerHalf showLower"}),s=this.traces[t]||(this.traces[t]={id:t,buffer:n.buffer({usage:"dynamic",type:"float",data:new Uint8Array}),color:"black",marker:null,size:12,borderColor:"transparent",borderSize:1,viewport:A([n._gl.drawingBufferWidth,n._gl.drawingBufferHeight]),padding:[0,0,0,0],opacity:1,diagonal:!0,upper:!0,lower:!0});if(null!=a.color&&(s.color=a.color),null!=a.size&&(s.size=a.size),null!=a.marker&&(s.marker=a.marker),null!=a.borderColor&&(s.borderColor=a.borderColor),null!=a.borderSize&&(s.borderSize=a.borderSize),null!=a.opacity&&(s.opacity=a.opacity),a.viewport&&(s.viewport=A(a.viewport)),null!=a.diagonal&&(s.diagonal=a.diagonal),null!=a.upper&&(s.upper=a.upper),null!=a.lower&&(s.lower=a.lower),a.data){s.buffer(l(a.data)),s.columns=a.data.length,s.count=a.data[0].length,s.bounds=[];for(var c=0;c<s.columns;c++)s.bounds[c]=o(a.data[c],1)}a.range&&(s.range=a.range,r=s.range&&"number"!=typeof s.range[0]),a.domain&&(s.domain=a.domain);var d=!1;null!=a.padding&&(Array.isArray(a.padding)&&a.padding.length===s.columns&&"number"==typeof a.padding[a.padding.length-1]?(s.padding=a.padding.map(h),d=!0):s.padding=h(a.padding));var f=s.columns,p=s.count,I=s.viewport.width,C=s.viewport.height,m=s.viewport.x,y=s.viewport.y,E=I/f,v=C/f;s.passes=[];for(var B=0;B<f;B++)for(var b=0;b<f;b++)if((s.diagonal||b!==B)&&(s.upper||!(B>b))&&(s.lower||!(B<b))){var Q=u(s.id,B,b),w=this.passes[Q]||(this.passes[Q]={});if(a.data&&(a.transpose?w.positions={x:{buffer:s.buffer,offset:b,count:p,stride:f},y:{buffer:s.buffer,offset:B,count:p,stride:f}}:w.positions={x:{buffer:s.buffer,offset:b*p,count:p},y:{buffer:s.buffer,offset:B*p,count:p}},w.bounds=g(s.bounds,B,b)),a.domain||a.viewport||a.data){var x=d?g(s.padding,B,b):s.padding;if(s.domain){var _=g(s.domain,B,b),D=_[0],M=_[1],k=_[2],S=_[3];w.viewport=[m+D*I+x[0],y+M*C+x[1],m+k*I-x[2],y+S*C-x[3]]}else w.viewport=[m+b*E+E*x[0],y+B*v+v*x[1],m+(b+1)*E-E*x[2],y+(B+1)*v-v*x[3]]}a.color&&(w.color=s.color),a.size&&(w.size=s.size),a.marker&&(w.marker=s.marker),a.borderSize&&(w.borderSize=s.borderSize),a.borderColor&&(w.borderColor=s.borderColor),a.opacity&&(w.opacity=s.opacity),a.range&&(w.range=r?g(s.range,B,b):s.range||w.bounds),s.passes.push(Q)}return this},c.prototype.draw=function(){for(var t,e=[],n=arguments.length;n--;)e[n]=arguments[n];if(e.length){for(var r=[],i=0;i<e.length;i++)if("number"==typeof e[i]){var o=this.traces[e[i]],a=o.passes,A=o.passOffset;r.push.apply(r,s(A,A+a.length))}else if(e[i].length){var l=e[i],c=this.traces[i],u=c.passes,g=c.passOffset;u=u.map(function(t,e){r[g+e]=l})}(t=this.scatter).draw.apply(t,r)}else this.scatter.draw();return this},c.prototype.destroy=function(){return this.traces.forEach(function(t){t.buffer&&t.buffer.destroy&&t.buffer.destroy()}),this.traces=null,this.passes=null,this.scatter.destroy(),this}},{"array-bounds":65,"array-range":67,"flatten-vertex-data":227,"parse-rect":459,"pick-by-alias":465,raf:484,"regl-scatter2d":497}],502:[function(t,e,n){var r;r=function(){function t(t,e){this.id=H++,this.type=t,this.data=e}function e(t){return"["+function t(e){if(0===e.length)return[];var n=e.charAt(0),r=e.charAt(e.length-1);if(1<e.length&&n===r&&('"'===n||"'"===n))return['"'+e.substr(1,e.length-2).replace(/\\/g,"\\\\").replace(/"/g,'\\"')+'"'];if(n=/\[(false|true|null|\d+|'[^']*'|"[^"]*")\]/.exec(e))return t(e.substr(0,n.index)).concat(t(n[1])).concat(t(e.substr(n.index+n[0].length)));if(1===(n=e.split(".")).length)return['"'+e.replace(/\\/g,"\\\\").replace(/"/g,'\\"')+'"'];for(e=[],r=0;r<n.length;++r)e=e.concat(t(n[r]));return e}(t).join("][")+"]"}function n(t){return"string"==typeof t?t.split():t}function r(t){return"string"==typeof t?document.querySelector(t):t}function i(t){var e,i,o,a,s=t||{};t={};var A=[],l=[],c="undefined"==typeof window?1:window.devicePixelRatio,u=!1,g=function(t){},h=function(){};if("string"==typeof s?e=document.querySelector(s):"object"==typeof s&&("string"==typeof s.nodeName&&"function"==typeof s.appendChild&&"function"==typeof s.getBoundingClientRect?e=s:"function"==typeof s.drawArrays||"function"==typeof s.drawElements?o=(a=s).canvas:("gl"in s?a=s.gl:"canvas"in s?o=r(s.canvas):"container"in s&&(i=r(s.container)),"attributes"in s&&(t=s.attributes),"extensions"in s&&(A=n(s.extensions)),"optionalExtensions"in s&&(l=n(s.optionalExtensions)),"onDone"in s&&(g=s.onDone),"profile"in s&&(u=!!s.profile),"pixelRatio"in s&&(c=+s.pixelRatio))),e&&("canvas"===e.nodeName.toLowerCase()?o=e:i=e),!a){if(!o){if(!(e=function(t,e,n){function r(){var e=window.innerWidth,r=window.innerHeight;t!==document.body&&(e=(r=t.getBoundingClientRect()).right-r.left,r=r.bottom-r.top),i.width=n*e,i.height=n*r,G(i.style,{width:e+"px",height:r+"px"})}var i=document.createElement("canvas");return G(i.style,{border:0,margin:0,padding:0,top:0,left:0}),t.appendChild(i),t===document.body&&(i.style.position="absolute",G(t.style,{margin:0,padding:0})),window.addEventListener("resize",r,!1),r(),{canvas:i,onDestroy:function(){window.removeEventListener("resize",r),t.removeChild(i)}}}(i||document.body,0,c)))return null;o=e.canvas,h=e.onDestroy}a=function(t,e){function n(n){try{return t.getContext(n,e)}catch(t){return null}}return n("webgl")||n("experimental-webgl")||n("webgl-experimental")}(o,t)}return a?{gl:a,canvas:o,container:i,extensions:A,optionalExtensions:l,pixelRatio:c,profile:u,onDone:g,onDestroy:h}:(h(),g("webgl not supported, try upgrading your browser or graphics drivers http://get.webgl.org"),null)}function o(t,e){for(var n=Array(t),r=0;r<t;++r)n[r]=e(r);return n}function a(t){var e,n;return e=(65535<t)<<4,e|=n=(255<(t>>>=e))<<3,(e|=n=(15<(t>>>=n))<<2)|(n=(3<(t>>>=n))<<1)|t>>>n>>1}function s(){function t(t){t:{for(var e=16;268435456>=e;e*=16)if(t<=e){t=e;break t}t=0}return 0<(e=n[a(t)>>2]).length?e.pop():new ArrayBuffer(t)}function e(t){n[a(t.byteLength)>>2].push(t)}var n=o(8,function(){return[]});return{alloc:t,free:e,allocType:function(e,n){var r=null;switch(e){case 5120:r=new Int8Array(t(n),0,n);break;case 5121:r=new Uint8Array(t(n),0,n);break;case 5122:r=new Int16Array(t(2*n),0,n);break;case 5123:r=new Uint16Array(t(2*n),0,n);break;case 5124:r=new Int32Array(t(4*n),0,n);break;case 5125:r=new Uint32Array(t(4*n),0,n);break;case 5126:r=new Float32Array(t(4*n),0,n);break;default:return null}return r.length!==n?r.subarray(0,n):r},freeType:function(t){e(t.buffer)}}}function A(t){return!!t&&"object"==typeof t&&Array.isArray(t.shape)&&Array.isArray(t.stride)&&"number"==typeof t.offset&&t.shape.length===t.stride.length&&(Array.isArray(t.data)||J(t.data))}function l(t,e,n,r,i,o){for(var a=0;a<e;++a)for(var s=t[a],A=0;A<n;++A)for(var l=s[A],c=0;c<r;++c)i[o++]=l[c]}function c(t){return 0|W[Object.prototype.toString.call(t)]}function u(t,e){for(var n=0;n<e.length;++n)t[n]=e[n]}function g(t,e,n,r,i,o,a){for(var s=0,A=0;A<n;++A)for(var l=0;l<r;++l)t[s++]=e[i*A+o*l+a]}function h(t,e,n,r){function i(e){this.id=l++,this.buffer=t.createBuffer(),this.type=e,this.usage=35044,this.byteLength=0,this.dimension=1,this.dtype=5121,this.persistentData=null,n.profile&&(this.stats={size:0})}function o(e,n,r){e.byteLength=n.byteLength,t.bufferData(e.type,n,r)}function a(t,e,n,r,i,a){if(t.usage=n,Array.isArray(e)){if(t.dtype=r||5126,0<e.length)if(Array.isArray(e[0])){i=tt(e);for(var s=r=1;s<i.length;++s)r*=i[s];t.dimension=r,o(t,e=$(e,i,t.dtype),n),a?t.persistentData=e:K.freeType(e)}else"number"==typeof e[0]?(t.dimension=i,u(i=K.allocType(t.dtype,e.length),e),o(t,i,n),a?t.persistentData=i:K.freeType(i)):J(e[0])&&(t.dimension=e[0].length,t.dtype=r||c(e[0])||5126,o(t,e=$(e,[e.length,e[0].length],t.dtype),n),a?t.persistentData=e:K.freeType(e))}else if(J(e))t.dtype=r||c(e),t.dimension=i,o(t,e,n),a&&(t.persistentData=new Uint8Array(new Uint8Array(e.buffer)));else if(A(e)){i=e.shape;var l=e.stride,h=(s=e.offset,0),d=0,f=0,p=0;1===i.length?(h=i[0],d=1,f=l[0],p=0):2===i.length&&(h=i[0],d=i[1],f=l[0],p=l[1]),t.dtype=r||c(e.data)||5126,t.dimension=d,g(i=K.allocType(t.dtype,h*d),e.data,h,d,f,p,s),o(t,i,n),a?t.persistentData=i:K.freeType(i)}}function s(n){e.bufferCount--;for(var i=0;i<r.state.length;++i){var o=r.state[i];o.buffer===n&&(t.disableVertexAttribArray(i),o.buffer=null)}t.deleteBuffer(n.buffer),n.buffer=null,delete h[n.id]}var l=0,h={};i.prototype.bind=function(){t.bindBuffer(this.type,this.buffer)},i.prototype.destroy=function(){s(this)};var d=[];return n.profile&&(e.getTotalBufferSize=function(){var t=0;return Object.keys(h).forEach(function(e){t+=h[e].stats.size}),t}),{create:function(r,o,l,d){function f(e){var r=35044,i=null,o=0,s=0,l=1;return Array.isArray(e)||J(e)||A(e)?i=e:"number"==typeof e?o=0|e:e&&("data"in e&&(i=e.data),"usage"in e&&(r=Z[e.usage]),"type"in e&&(s=X[e.type]),"dimension"in e&&(l=0|e.dimension),"length"in e&&(o=0|e.length)),p.bind(),i?a(p,i,r,s,l,d):(o&&t.bufferData(p.type,o,r),p.dtype=s||5121,p.usage=r,p.dimension=l,p.byteLength=o),n.profile&&(p.stats.size=p.byteLength*et[p.dtype]),f}e.bufferCount++;var p=new i(o);return h[p.id]=p,l||f(r),f._reglType="buffer",f._buffer=p,f.subdata=function(e,n){var r,i=0|(n||0);if(p.bind(),J(e))t.bufferSubData(p.type,i,e);else if(Array.isArray(e)){if(0<e.length)if("number"==typeof e[0]){var o=K.allocType(p.dtype,e.length);u(o,e),t.bufferSubData(p.type,i,o),K.freeType(o)}else(Array.isArray(e[0])||J(e[0]))&&(r=tt(e),o=$(e,r,p.dtype),t.bufferSubData(p.type,i,o),K.freeType(o))}else if(A(e)){r=e.shape;var a=e.stride,s=o=0,l=0,h=0;1===r.length?(o=r[0],s=1,l=a[0],h=0):2===r.length&&(o=r[0],s=r[1],l=a[0],h=a[1]),r=Array.isArray(e.data)?p.dtype:c(e.data),g(r=K.allocType(r,o*s),e.data,o,s,l,h,e.offset),t.bufferSubData(p.type,i,r),K.freeType(r)}return f},n.profile&&(f.stats=p.stats),f.destroy=function(){s(p)},f},createStream:function(t,e){var n=d.pop();return n||(n=new i(t)),n.bind(),a(n,e,35040,0,1,!1),n},destroyStream:function(t){d.push(t)},clear:function(){q(h).forEach(s),d.forEach(s)},getBuffer:function(t){return t&&t._buffer instanceof i?t._buffer:null},restore:function(){q(h).forEach(function(e){e.buffer=t.createBuffer(),t.bindBuffer(e.type,e.buffer),t.bufferData(e.type,e.persistentData||e.byteLength,e.usage)})},_initBuffer:a}}function d(t,e,n,r){function i(t){this.id=l++,s[this.id]=this,this.buffer=t,this.primType=4,this.type=this.vertCount=0}function o(r,i,o,a,s,l,c){if(r.buffer.bind(),i){var u=c;c||J(i)&&(!A(i)||J(i.data))||(u=e.oes_element_index_uint?5125:5123),n._initBuffer(r.buffer,i,o,u,3)}else t.bufferData(34963,l,o),r.buffer.dtype=u||5121,r.buffer.usage=o,r.buffer.dimension=3,r.buffer.byteLength=l;if(u=c,!c){switch(r.buffer.dtype){case 5121:case 5120:u=5121;break;case 5123:case 5122:u=5123;break;case 5125:case 5124:u=5125}r.buffer.dtype=u}r.type=u,0>(i=s)&&(i=r.buffer.byteLength,5123===u?i>>=1:5125===u&&(i>>=2)),r.vertCount=i,i=a,0>a&&(i=4,1===(a=r.buffer.dimension)&&(i=0),2===a&&(i=1),3===a&&(i=4)),r.primType=i}function a(t){r.elementsCount--,delete s[t.id],t.buffer.destroy(),t.buffer=null}var s={},l=0,c={uint8:5121,uint16:5123};e.oes_element_index_uint&&(c.uint32=5125),i.prototype.bind=function(){this.buffer.bind()};var u=[];return{create:function(t,e){function s(t){if(t)if("number"==typeof t)l(t),u.primType=4,u.vertCount=0|t,u.type=5121;else{var e=null,n=35044,r=-1,i=-1,a=0,g=0;Array.isArray(t)||J(t)||A(t)?e=t:("data"in t&&(e=t.data),"usage"in t&&(n=Z[t.usage]),"primitive"in t&&(r=nt[t.primitive]),"count"in t&&(i=0|t.count),"type"in t&&(g=c[t.type]),"length"in t?a=0|t.length:(a=i,5123===g||5122===g?a*=2:5125!==g&&5124!==g||(a*=4))),o(u,e,n,r,i,a,g)}else l(),u.primType=4,u.vertCount=0,u.type=5121;return s}var l=n.create(null,34963,!0),u=new i(l._buffer);return r.elementsCount++,s(t),s._reglType="elements",s._elements=u,s.subdata=function(t,e){return l.subdata(t,e),s},s.destroy=function(){a(u)},s},createStream:function(t){var e=u.pop();return e||(e=new i(n.create(null,34963,!0,!1)._buffer)),o(e,t,35040,-1,-1,0,0),e},destroyStream:function(t){u.push(t)},getElements:function(t){return"function"==typeof t&&t._elements instanceof i?t._elements:null},clear:function(){q(s).forEach(a)}}}function f(t){for(var e=K.allocType(5123,t.length),n=0;n<t.length;++n)if(isNaN(t[n]))e[n]=65535;else if(1/0===t[n])e[n]=31744;else if(-1/0===t[n])e[n]=64512;else{rt[0]=t[n];var r=(o=it[0])>>>31<<15,i=(o<<1>>>24)-127,o=o>>13&1023;e[n]=-24>i?r:-14>i?r+(o+1024>>-14-i):15<i?r+31744:r+(i+15<<10)+o}return e}function p(t){return Array.isArray(t)||J(t)}function I(t){return"[object "+t+"]"}function C(t){return Array.isArray(t)&&(0===t.length||"number"==typeof t[0])}function m(t){return!(!Array.isArray(t)||0===t.length||!p(t[0]))}function y(t){return Object.prototype.toString.call(t)}function E(t){if(!t)return!1;var e=y(t);return 0<=ht.indexOf(e)||C(t)||m(t)||A(t)}function v(t,e){36193===t.type?(t.data=f(e),K.freeType(e)):t.data=e}function B(t,e,n,r,i,o){if(t=void 0!==ft[t]?ft[t]:st[t]*dt[e],o&&(t*=6),i){for(r=0;1<=n;)r+=t*n*n,n/=2;return r}return t*n*r}function b(t,e,n,r,i,o,a){function s(){this.format=this.internalformat=6408,this.type=5121,this.flipY=this.premultiplyAlpha=this.compressed=!1,this.unpackAlignment=1,this.colorSpace=37444,this.channels=this.height=this.width=0}function l(t,e){t.internalformat=e.internalformat,t.format=e.format,t.type=e.type,t.compressed=e.compressed,t.premultiplyAlpha=e.premultiplyAlpha,t.flipY=e.flipY,t.unpackAlignment=e.unpackAlignment,t.colorSpace=e.colorSpace,t.width=e.width,t.height=e.height,t.channels=e.channels}function c(t,e){if("object"==typeof e&&e){"premultiplyAlpha"in e&&(t.premultiplyAlpha=e.premultiplyAlpha),"flipY"in e&&(t.flipY=e.flipY),"alignment"in e&&(t.unpackAlignment=e.alignment),"colorSpace"in e&&(t.colorSpace=Y[e.colorSpace]),"type"in e&&(t.type=P[e.type]);var n=t.width,r=t.height,i=t.channels,o=!1;"shape"in e?(n=e.shape[0],r=e.shape[1],3===e.shape.length&&(i=e.shape[2],o=!0)):("radius"in e&&(n=r=e.radius),"width"in e&&(n=e.width),"height"in e&&(r=e.height),"channels"in e&&(i=e.channels,o=!0)),t.width=0|n,t.height=0|r,t.channels=0|i,n=!1,"format"in e&&(n=e.format,r=t.internalformat=z[n],t.format=ht[r],n in P&&!("type"in e)&&(t.type=P[n]),n in X&&(t.compressed=!0),n=!0),!o&&n?t.channels=st[t.format]:o&&!n&&t.channels!==at[t.format]&&(t.format=t.internalformat=at[t.channels])}}function u(e){t.pixelStorei(37440,e.flipY),t.pixelStorei(37441,e.premultiplyAlpha),t.pixelStorei(37443,e.colorSpace),t.pixelStorei(3317,e.unpackAlignment)}function g(){s.call(this),this.yOffset=this.xOffset=0,this.data=null,this.needsFree=!1,this.element=null,this.needsCopy=!1}function h(t,e){var n=null;if(E(e)?n=e:e&&(c(t,e),"x"in e&&(t.xOffset=0|e.x),"y"in e&&(t.yOffset=0|e.y),E(e.data)&&(n=e.data)),e.copy){var r=i.viewportWidth,o=i.viewportHeight;t.width=t.width||r-t.xOffset,t.height=t.height||o-t.yOffset,t.needsCopy=!0}else if(n){if(J(n))t.channels=t.channels||4,t.data=n,"type"in e||5121!==t.type||(t.type=0|W[Object.prototype.toString.call(n)]);else if(C(n)){switch(t.channels=t.channels||4,o=(r=n).length,t.type){case 5121:case 5123:case 5125:case 5126:(o=K.allocType(t.type,o)).set(r),t.data=o;break;case 36193:t.data=f(r)}t.alignment=1,t.needsFree=!0}else if(A(n)){r=n.data,Array.isArray(r)||5121!==t.type||(t.type=0|W[Object.prototype.toString.call(r)]),o=n.shape;var a,s,l,u,g=n.stride;3===o.length?(l=o[2],u=g[2]):u=l=1,a=o[0],s=o[1],o=g[0],g=g[1],t.alignment=1,t.width=a,t.height=s,t.channels=l,t.format=t.internalformat=at[l],t.needsFree=!0,a=u,n=n.offset,l=t.width,u=t.height,s=t.channels;for(var h=K.allocType(36193===t.type?5126:t.type,l*u*s),d=0,I=0;I<u;++I)for(var B=0;B<l;++B)for(var b=0;b<s;++b)h[d++]=r[o*B+g*I+a*b+n];v(t,h)}else if(y(n)===At||y(n)===lt)y(n)===At?t.element=n:t.element=n.canvas,t.width=t.element.width,t.height=t.element.height,t.channels=4;else if(y(n)===ct)t.element=n,t.width=n.width,t.height=n.height,t.channels=4;else if(y(n)===ut)t.element=n,t.width=n.naturalWidth,t.height=n.naturalHeight,t.channels=4;else if(y(n)===gt)t.element=n,t.width=n.videoWidth,t.height=n.videoHeight,t.channels=4;else if(m(n)){for(r=t.width||n[0].length,o=t.height||n.length,g=t.channels,g=p(n[0][0])?g||n[0][0].length:g||1,a=V.shape(n),l=1,u=0;u<a.length;++u)l*=a[u];l=K.allocType(36193===t.type?5126:t.type,l),V.flatten(n,a,"",l),v(t,l),t.alignment=1,t.width=r,t.height=o,t.channels=g,t.format=t.internalformat=at[g],t.needsFree=!0}}else t.width=t.width||1,t.height=t.height||1,t.channels=t.channels||4}function d(e,n,i,o,a){var s=e.element,A=e.data,l=e.internalformat,c=e.format,g=e.type,h=e.width,d=e.height;u(e),s?t.texSubImage2D(n,a,i,o,c,g,s):e.compressed?t.compressedTexSubImage2D(n,a,i,o,l,h,d,A):e.needsCopy?(r(),t.copyTexSubImage2D(n,a,i,o,e.xOffset,e.yOffset,h,d)):t.texSubImage2D(n,a,i,o,h,d,c,g,A)}function I(){return dt.pop()||new g}function b(t){t.needsFree&&K.freeType(t.data),g.call(t),dt.push(t)}function Q(){s.call(this),this.genMipmaps=!1,this.mipmapHint=4352,this.mipmask=0,this.images=Array(16)}function w(t,e,n){var r=t.images[0]=I();t.mipmask=1,r.width=t.width=e,r.height=t.height=n,r.channels=t.channels=4}function x(t,e){var n=null;if(E(e))l(n=t.images[0]=I(),t),h(n,e),t.mipmask=1;else if(c(t,e),Array.isArray(e.mipmap))for(var r=e.mipmap,i=0;i<r.length;++i)l(n=t.images[i]=I(),t),n.width>>=i,n.height>>=i,h(n,r[i]),t.mipmask|=1<<i;else l(n=t.images[0]=I(),t),h(n,e),t.mipmask=1;l(t,t.images[0])}function _(e,n){for(var i=e.images,o=0;o<i.length&&i[o];++o){var a=i[o],s=n,A=o,l=a.element,c=a.data,g=a.internalformat,h=a.format,d=a.type,f=a.width,p=a.height,I=a.channels;u(a),l?t.texImage2D(s,A,h,h,d,l):a.compressed?t.compressedTexImage2D(s,A,g,f,p,0,c):a.needsCopy?(r(),t.copyTexImage2D(s,A,h,a.xOffset,a.yOffset,f,p,0)):((a=!c)&&(c=K.zero.allocType(d,f*p*I)),t.texImage2D(s,A,h,f,p,0,h,d,c),a&&c&&K.zero.freeType(c))}}function D(){var t=ft.pop()||new Q;s.call(t);for(var e=t.mipmask=0;16>e;++e)t.images[e]=null;return t}function M(t){for(var e=t.images,n=0;n<e.length;++n)e[n]&&b(e[n]),e[n]=null;ft.push(t)}function k(){this.magFilter=this.minFilter=9728,this.wrapT=this.wrapS=33071,this.anisotropic=1,this.genMipmaps=!1,this.mipmapHint=4352}function S(t,e){"min"in e&&(t.minFilter=j[e.min],0<=ot.indexOf(t.minFilter)&&!("faces"in e)&&(t.genMipmaps=!0)),"mag"in e&&(t.magFilter=H[e.mag]);var n=t.wrapS,r=t.wrapT;if("wrap"in e){var i=e.wrap;"string"==typeof i?n=r=O[i]:Array.isArray(i)&&(n=O[i[0]],r=O[i[1]])}else"wrapS"in e&&(n=O[e.wrapS]),"wrapT"in e&&(r=O[e.wrapT]);if(t.wrapS=n,t.wrapT=r,"anisotropic"in e&&(t.anisotropic=e.anisotropic),"mipmap"in e){switch(n=!1,typeof e.mipmap){case"string":t.mipmapHint=U[e.mipmap],n=t.genMipmaps=!0;break;case"boolean":n=t.genMipmaps=e.mipmap;break;case"object":t.genMipmaps=!1,n=!0}!n||"min"in e||(t.minFilter=9984)}}function L(n,r){t.texParameteri(r,10241,n.minFilter),t.texParameteri(r,10240,n.magFilter),t.texParameteri(r,10242,n.wrapS),t.texParameteri(r,10243,n.wrapT),e.ext_texture_filter_anisotropic&&t.texParameteri(r,34046,n.anisotropic),n.genMipmaps&&(t.hint(33170,n.mipmapHint),t.generateMipmap(r))}function R(e){s.call(this),this.mipmask=0,this.internalformat=6408,this.id=pt++,this.refCount=1,this.target=e,this.texture=t.createTexture(),this.unit=-1,this.bindCount=0,this.texInfo=new k,a.profile&&(this.stats={size:0})}function N(e){t.activeTexture(33984),t.bindTexture(e.target,e.texture)}function F(){var e=mt[0];e?t.bindTexture(e.target,e.texture):t.bindTexture(3553,null)}function T(e){var n=e.texture,r=e.unit,i=e.target;0<=r&&(t.activeTexture(33984+r),t.bindTexture(i,null),mt[r]=null),t.deleteTexture(n),e.texture=null,e.params=null,e.pixels=null,e.refCount=0,delete It[e.id],o.textureCount--}var U={"don't care":4352,"dont care":4352,nice:4354,fast:4353},O={repeat:10497,clamp:33071,mirror:33648},H={nearest:9728,linear:9729},j=G({mipmap:9987,"nearest mipmap nearest":9984,"linear mipmap nearest":9985,"nearest mipmap linear":9986,"linear mipmap linear":9987},H),Y={none:0,browser:37444},P={uint8:5121,rgba4:32819,rgb565:33635,"rgb5 a1":32820},z={alpha:6406,luminance:6409,"luminance alpha":6410,rgb:6407,rgba:6408,rgba4:32854,"rgb5 a1":32855,rgb565:36194},X={};e.ext_srgb&&(z.srgb=35904,z.srgba=35906),e.oes_texture_float&&(P.float32=P.float=5126),e.oes_texture_half_float&&(P.float16=P["half float"]=36193),e.webgl_depth_texture&&(G(z,{depth:6402,"depth stencil":34041}),G(P,{uint16:5123,uint32:5125,"depth stencil":34042})),e.webgl_compressed_texture_s3tc&&G(X,{"rgb s3tc dxt1":33776,"rgba s3tc dxt1":33777,"rgba s3tc dxt3":33778,"rgba s3tc dxt5":33779}),e.webgl_compressed_texture_atc&&G(X,{"rgb atc":35986,"rgba atc explicit alpha":35987,"rgba atc interpolated alpha":34798}),e.webgl_compressed_texture_pvrtc&&G(X,{"rgb pvrtc 4bppv1":35840,"rgb pvrtc 2bppv1":35841,"rgba pvrtc 4bppv1":35842,"rgba pvrtc 2bppv1":35843}),e.webgl_compressed_texture_etc1&&(X["rgb etc1"]=36196);var Z=Array.prototype.slice.call(t.getParameter(34467));Object.keys(X).forEach(function(t){var e=X[t];0<=Z.indexOf(e)&&(z[t]=e)});var $=Object.keys(z);n.textureFormats=$;var tt=[];Object.keys(z).forEach(function(t){tt[z[t]]=t});var et=[];Object.keys(P).forEach(function(t){et[P[t]]=t});var nt=[];Object.keys(H).forEach(function(t){nt[H[t]]=t});var rt=[];Object.keys(j).forEach(function(t){rt[j[t]]=t});var it=[];Object.keys(O).forEach(function(t){it[O[t]]=t});var ht=$.reduce(function(t,e){var n=z[e];return 6409===n||6406===n||6409===n||6410===n||6402===n||34041===n?t[n]=n:32855===n||0<=e.indexOf("rgba")?t[n]=6408:t[n]=6407,t},{}),dt=[],ft=[],pt=0,It={},Ct=n.maxTextureUnits,mt=Array(Ct).map(function(){return null});return G(R.prototype,{bind:function(){this.bindCount+=1;var e=this.unit;if(0>e){for(var n=0;n<Ct;++n){var r=mt[n];if(r){if(0<r.bindCount)continue;r.unit=-1}mt[n]=this,e=n;break}a.profile&&o.maxTextureUnits<e+1&&(o.maxTextureUnits=e+1),this.unit=e,t.activeTexture(33984+e),t.bindTexture(this.target,this.texture)}return e},unbind:function(){--this.bindCount},decRef:function(){0>=--this.refCount&&T(this)}}),a.profile&&(o.getTotalTextureSize=function(){var t=0;return Object.keys(It).forEach(function(e){t+=It[e].stats.size}),t}),{create2D:function(e,n){function r(t,e){var n=i.texInfo;k.call(n);var o=D();return"number"==typeof t?w(o,0|t,"number"==typeof e?0|e:0|t):t?(S(n,t),x(o,t)):w(o,1,1),n.genMipmaps&&(o.mipmask=(o.width<<1)-1),i.mipmask=o.mipmask,l(i,o),i.internalformat=o.internalformat,r.width=o.width,r.height=o.height,N(i),_(o,3553),L(n,3553),F(),M(o),a.profile&&(i.stats.size=B(i.internalformat,i.type,o.width,o.height,n.genMipmaps,!1)),r.format=tt[i.internalformat],r.type=et[i.type],r.mag=nt[n.magFilter],r.min=rt[n.minFilter],r.wrapS=it[n.wrapS],r.wrapT=it[n.wrapT],r}var i=new R(3553);return It[i.id]=i,o.textureCount++,r(e,n),r.subimage=function(t,e,n,o){e|=0,n|=0,o|=0;var a=I();return l(a,i),a.width=0,a.height=0,h(a,t),a.width=a.width||(i.width>>o)-e,a.height=a.height||(i.height>>o)-n,N(i),d(a,3553,e,n,o),F(),b(a),r},r.resize=function(e,n){var o=0|e,s=0|n||o;if(o===i.width&&s===i.height)return r;r.width=i.width=o,r.height=i.height=s,N(i);for(var A,l=i.channels,c=i.type,u=0;i.mipmask>>u;++u){var g=o>>u,h=s>>u;if(!g||!h)break;A=K.zero.allocType(c,g*h*l),t.texImage2D(3553,u,i.format,g,h,0,i.format,i.type,A),A&&K.zero.freeType(A)}return F(),a.profile&&(i.stats.size=B(i.internalformat,i.type,o,s,!1,!1)),r},r._reglType="texture2d",r._texture=i,a.profile&&(r.stats=i.stats),r.destroy=function(){i.decRef()},r},createCube:function(e,n,r,i,s,A){function u(t,e,n,r,i,o){var s,A=g.texInfo;for(k.call(A),s=0;6>s;++s)f[s]=D();if("number"!=typeof t&&t){if("object"==typeof t)if(e)x(f[0],t),x(f[1],e),x(f[2],n),x(f[3],r),x(f[4],i),x(f[5],o);else if(S(A,t),c(g,t),"faces"in t)for(t=t.faces,s=0;6>s;++s)l(f[s],g),x(f[s],t[s]);else for(s=0;6>s;++s)x(f[s],t)}else for(t=0|t||1,s=0;6>s;++s)w(f[s],t,t);for(l(g,f[0]),g.mipmask=A.genMipmaps?(f[0].width<<1)-1:f[0].mipmask,g.internalformat=f[0].internalformat,u.width=f[0].width,u.height=f[0].height,N(g),s=0;6>s;++s)_(f[s],34069+s);for(L(A,34067),F(),a.profile&&(g.stats.size=B(g.internalformat,g.type,u.width,u.height,A.genMipmaps,!0)),u.format=tt[g.internalformat],u.type=et[g.type],u.mag=nt[A.magFilter],u.min=rt[A.minFilter],u.wrapS=it[A.wrapS],u.wrapT=it[A.wrapT],s=0;6>s;++s)M(f[s]);return u}var g=new R(34067);It[g.id]=g,o.cubeCount++;var f=Array(6);return u(e,n,r,i,s,A),u.subimage=function(t,e,n,r,i){n|=0,r|=0,i|=0;var o=I();return l(o,g),o.width=0,o.height=0,h(o,e),o.width=o.width||(g.width>>i)-n,o.height=o.height||(g.height>>i)-r,N(g),d(o,34069+t,n,r,i),F(),b(o),u},u.resize=function(e){if((e|=0)!==g.width){u.width=g.width=e,u.height=g.height=e,N(g);for(var n=0;6>n;++n)for(var r=0;g.mipmask>>r;++r)t.texImage2D(34069+n,r,g.format,e>>r,e>>r,0,g.format,g.type,null);return F(),a.profile&&(g.stats.size=B(g.internalformat,g.type,u.width,u.height,!1,!0)),u}},u._reglType="textureCube",u._texture=g,a.profile&&(u.stats=g.stats),u.destroy=function(){g.decRef()},u},clear:function(){for(var e=0;e<Ct;++e)t.activeTexture(33984+e),t.bindTexture(3553,null),mt[e]=null;q(It).forEach(T),o.cubeCount=0,o.textureCount=0},getTexture:function(t){return null},restore:function(){for(var e=0;e<Ct;++e){var n=mt[e];n&&(n.bindCount=0,n.unit=-1,mt[e]=null)}q(It).forEach(function(e){e.texture=t.createTexture(),t.bindTexture(e.target,e.texture);for(var n=0;32>n;++n)if(0!=(e.mipmask&1<<n))if(3553===e.target)t.texImage2D(3553,n,e.internalformat,e.width>>n,e.height>>n,0,e.internalformat,e.type,null);else for(var r=0;6>r;++r)t.texImage2D(34069+r,n,e.internalformat,e.width>>n,e.height>>n,0,e.internalformat,e.type,null);L(e.texInfo,e.target)})}}}function Q(t,e,n,r,i,o){function a(t,e,n){this.target=t,this.texture=e,this.renderbuffer=n;var r=t=0;e?(t=e.width,r=e.height):n&&(t=n.width,r=n.height),this.width=t,this.height=r}function s(t){t&&(t.texture&&t.texture._texture.decRef(),t.renderbuffer&&t.renderbuffer._renderbuffer.decRef())}function A(t,e,n){t&&(t.texture?t.texture._texture.refCount+=1:t.renderbuffer._renderbuffer.refCount+=1)}function l(e,n){n&&(n.texture?t.framebufferTexture2D(36160,e,n.target,n.texture._texture.texture,0):t.framebufferRenderbuffer(36160,e,36161,n.renderbuffer._renderbuffer.renderbuffer))}function c(t){var e=3553,n=null,r=null,i=t;return"object"==typeof t&&(i=t.data,"target"in t&&(e=0|t.target)),"texture2d"===(t=i._reglType)?n=i:"textureCube"===t?n=i:"renderbuffer"===t&&(r=i,e=36161),new a(e,n,r)}function u(t,e,n,o,s){return n?((t=r.create2D({width:t,height:e,format:o,type:s}))._texture.refCount=0,new a(3553,t,null)):((t=i.create({width:t,height:e,format:o}))._renderbuffer.refCount=0,new a(36161,null,t))}function g(t){return t&&(t.texture||t.renderbuffer)}function h(t,e,n){t&&(t.texture?t.texture.resize(e,n):t.renderbuffer&&t.renderbuffer.resize(e,n),t.width=e,t.height=n)}function d(){this.id=B++,b[this.id]=this,this.framebuffer=t.createFramebuffer(),this.height=this.width=0,this.colorAttachments=[],this.depthStencilAttachment=this.stencilAttachment=this.depthAttachment=null}function f(t){t.colorAttachments.forEach(s),s(t.depthAttachment),s(t.stencilAttachment),s(t.depthStencilAttachment)}function p(e){t.deleteFramebuffer(e.framebuffer),e.framebuffer=null,o.framebufferCount--,delete b[e.id]}function I(e){var r;t.bindFramebuffer(36160,e.framebuffer);var i=e.colorAttachments;for(r=0;r<i.length;++r)l(36064+r,i[r]);for(r=i.length;r<n.maxColorAttachments;++r)t.framebufferTexture2D(36160,36064+r,3553,null,0);t.framebufferTexture2D(36160,33306,3553,null,0),t.framebufferTexture2D(36160,36096,3553,null,0),t.framebufferTexture2D(36160,36128,3553,null,0),l(36096,e.depthAttachment),l(36128,e.stencilAttachment),l(33306,e.depthStencilAttachment),t.checkFramebufferStatus(36160),t.isContextLost(),t.bindFramebuffer(36160,m.next?m.next.framebuffer:null),m.cur=m.next,t.getError()}function C(t,e){function n(t,e){var i,o=0,a=0,s=!0,l=!0;i=null;var h=!0,d="rgba",p="uint8",C=1,m=null,v=null,B=null,b=!1;"number"==typeof t?(o=0|t,a=0|e||o):t?("shape"in t?(o=(a=t.shape)[0],a=a[1]):("radius"in t&&(o=a=t.radius),"width"in t&&(o=t.width),"height"in t&&(a=t.height)),("color"in t||"colors"in t)&&(i=t.color||t.colors,Array.isArray(i)),i||("colorCount"in t&&(C=0|t.colorCount),"colorTexture"in t&&(h=!!t.colorTexture,d="rgba4"),"colorType"in t&&(p=t.colorType,!h)&&("half float"===p||"float16"===p?d="rgba16f":"float"!==p&&"float32"!==p||(d="rgba32f")),"colorFormat"in t&&(d=t.colorFormat,0<=y.indexOf(d)?h=!0:0<=E.indexOf(d)&&(h=!1))),("depthTexture"in t||"depthStencilTexture"in t)&&(b=!(!t.depthTexture&&!t.depthStencilTexture)),"depth"in t&&("boolean"==typeof t.depth?s=t.depth:(m=t.depth,l=!1)),"stencil"in t&&("boolean"==typeof t.stencil?l=t.stencil:(v=t.stencil,s=!1)),"depthStencil"in t&&("boolean"==typeof t.depthStencil?s=l=t.depthStencil:(B=t.depthStencil,l=s=!1))):o=a=1;var Q=null,w=null,x=null,_=null;if(Array.isArray(i))Q=i.map(c);else if(i)Q=[c(i)];else for(Q=Array(C),i=0;i<C;++i)Q[i]=u(o,a,h,d,p);for(o=o||Q[0].width,a=a||Q[0].height,m?w=c(m):s&&!l&&(w=u(o,a,b,"depth","uint32")),v?x=c(v):l&&!s&&(x=u(o,a,!1,"stencil","uint8")),B?_=c(B):!m&&!v&&l&&s&&(_=u(o,a,b,"depth stencil","depth stencil")),s=null,i=0;i<Q.length;++i)A(Q[i]),Q[i]&&Q[i].texture&&(l=Ct[Q[i].texture._texture.format]*mt[Q[i].texture._texture.type],null===s&&(s=l));return A(w),A(x),A(_),f(r),r.width=o,r.height=a,r.colorAttachments=Q,r.depthAttachment=w,r.stencilAttachment=x,r.depthStencilAttachment=_,n.color=Q.map(g),n.depth=g(w),n.stencil=g(x),n.depthStencil=g(_),n.width=r.width,n.height=r.height,I(r),n}var r=new d;return o.framebufferCount++,n(t,e),G(n,{resize:function(t,e){var i=Math.max(0|t,1),o=Math.max(0|e||i,1);if(i===r.width&&o===r.height)return n;for(var a=r.colorAttachments,s=0;s<a.length;++s)h(a[s],i,o);return h(r.depthAttachment,i,o),h(r.stencilAttachment,i,o),h(r.depthStencilAttachment,i,o),r.width=n.width=i,r.height=n.height=o,I(r),n},_reglType:"framebuffer",_framebuffer:r,destroy:function(){p(r),f(r)},use:function(t){m.setFBO({framebuffer:n},t)}})}var m={cur:null,next:null,dirty:!1,setFBO:null},y=["rgba"],E=["rgba4","rgb565","rgb5 a1"];e.ext_srgb&&E.push("srgba"),e.ext_color_buffer_half_float&&E.push("rgba16f","rgb16f"),e.webgl_color_buffer_float&&E.push("rgba32f");var v=["uint8"];e.oes_texture_half_float&&v.push("half float","float16"),e.oes_texture_float&&v.push("float","float32");var B=0,b={};return G(m,{getFramebuffer:function(t){return"function"==typeof t&&"framebuffer"===t._reglType&&(t=t._framebuffer)instanceof d?t:null},create:C,createCube:function(t){function e(t){var i,o={color:null},a=0,s=null;i="rgba";var A="uint8",l=1;if("number"==typeof t?a=0|t:t?("shape"in t?a=t.shape[0]:("radius"in t&&(a=0|t.radius),"width"in t?a=0|t.width:"height"in t&&(a=0|t.height)),("color"in t||"colors"in t)&&(s=t.color||t.colors,Array.isArray(s)),s||("colorCount"in t&&(l=0|t.colorCount),"colorType"in t&&(A=t.colorType),"colorFormat"in t&&(i=t.colorFormat)),"depth"in t&&(o.depth=t.depth),"stencil"in t&&(o.stencil=t.stencil),"depthStencil"in t&&(o.depthStencil=t.depthStencil)):a=1,s)if(Array.isArray(s))for(t=[],i=0;i<s.length;++i)t[i]=s[i];else t=[s];else for(t=Array(l),s={radius:a,format:i,type:A},i=0;i<l;++i)t[i]=r.createCube(s);for(o.color=Array(t.length),i=0;i<t.length;++i)l=t[i],a=a||l.width,o.color[i]={target:34069,data:t[i]};for(i=0;6>i;++i){for(l=0;l<t.length;++l)o.color[l].target=34069+i;0<i&&(o.depth=n[0].depth,o.stencil=n[0].stencil,o.depthStencil=n[0].depthStencil),n[i]?n[i](o):n[i]=C(o)}return G(e,{width:a,height:a,color:t})}var n=Array(6);return e(t),G(e,{faces:n,resize:function(t){var r=0|t;if(r===e.width)return e;var i=e.color;for(t=0;t<i.length;++t)i[t].resize(r);for(t=0;6>t;++t)n[t].resize(r);return e.width=e.height=r,e},_reglType:"framebufferCube",destroy:function(){n.forEach(function(t){t.destroy()})}})},clear:function(){q(b).forEach(p)},restore:function(){m.cur=null,m.next=null,m.dirty=!0,q(b).forEach(function(e){e.framebuffer=t.createFramebuffer(),I(e)})}})}function w(){this.w=this.z=this.y=this.x=this.state=0,this.buffer=null,this.size=0,this.normalized=!1,this.type=5126,this.divisor=this.stride=this.offset=0}function x(t,e,n,r){function i(t,e,n,r){this.name=t,this.id=e,this.location=n,this.info=r}function o(t,e){for(var n=0;n<t.length;++n)if(t[n].id===e.id)return void(t[n].location=e.location);t.push(e)}function a(n,r,i){if(!(a=(i=35632===n?l:c)[r])){var o=e.str(r),a=t.createShader(n);t.shaderSource(a,o),t.compileShader(a),i[r]=a}return a}function s(t,e){this.id=h++,this.fragId=t,this.vertId=e,this.program=null,this.uniforms=[],this.attributes=[],r.profile&&(this.stats={uniformsCount:0,attributesCount:0})}function A(n,s){var A,l;A=a(35632,n.fragId),l=a(35633,n.vertId);var c=n.program=t.createProgram();t.attachShader(c,A),t.attachShader(c,l),t.linkProgram(c);var u=t.getProgramParameter(c,35718);r.profile&&(n.stats.uniformsCount=u);var g=n.uniforms;for(A=0;A<u;++A)if(l=t.getActiveUniform(c,A))if(1<l.size)for(var h=0;h<l.size;++h){var d=l.name.replace("[0]","["+h+"]");o(g,new i(d,e.id(d),t.getUniformLocation(c,d),l))}else o(g,new i(l.name,e.id(l.name),t.getUniformLocation(c,l.name),l));for(u=t.getProgramParameter(c,35721),r.profile&&(n.stats.attributesCount=u),g=n.attributes,A=0;A<u;++A)(l=t.getActiveAttrib(c,A))&&o(g,new i(l.name,e.id(l.name),t.getAttribLocation(c,l.name),l))}var l={},c={},u={},g=[],h=0;return r.profile&&(n.getMaxUniformsCount=function(){var t=0;return g.forEach(function(e){e.stats.uniformsCount>t&&(t=e.stats.uniformsCount)}),t},n.getMaxAttributesCount=function(){var t=0;return g.forEach(function(e){e.stats.attributesCount>t&&(t=e.stats.attributesCount)}),t}),{clear:function(){var e=t.deleteShader.bind(t);q(l).forEach(e),l={},q(c).forEach(e),c={},g.forEach(function(e){t.deleteProgram(e.program)}),g.length=0,u={},n.shaderCount=0},program:function(t,e,r){var i=u[e];i||(i=u[e]={});var o=i[t];return o||(o=new s(e,t),n.shaderCount++,A(o),i[t]=o,g.push(o)),o},restore:function(){l={},c={};for(var t=0;t<g.length;++t)A(g[t])},shader:a,frag:-1,vert:-1}}function _(t,e,n,r,i,o,a){function s(i){var o;o=null===e.next?5121:e.next.colorAttachments[0].texture._texture.type;var a=0,s=0,A=r.framebufferWidth,l=r.framebufferHeight,c=null;return J(i)?c=i:i&&(a=0|i.x,s=0|i.y,A=0|(i.width||r.framebufferWidth-a),l=0|(i.height||r.framebufferHeight-s),c=i.data||null),n(),i=A*l*4,c||(5121===o?c=new Uint8Array(i):5126===o&&(c=c||new Float32Array(i))),t.pixelStorei(3333,4),t.readPixels(a,s,A,l,6408,o,c),c}return function(t){return t&&"framebuffer"in t?function(t){var n;return e.setFBO({framebuffer:t.framebuffer},function(){n=s(t)}),n}(t):s(t)}}function D(t){return Array.prototype.slice.call(t)}function M(t){return D(t).join("")}function k(){function t(){var t=[],e=[];return G(function(){t.push.apply(t,D(arguments))},{def:function(){var r="v"+n++;return e.push(r),0<arguments.length&&(t.push(r,"="),t.push.apply(t,D(arguments)),t.push(";")),r},toString:function(){return M([0<e.length?"var "+e+";":"",M(t)])}})}function e(){function e(t,e){r(t,e,"=",n.def(t,e),";")}var n=t(),r=t(),i=n.toString,o=r.toString;return G(function(){n.apply(n,D(arguments))},{def:n.def,entry:n,exit:r,save:e,set:function(t,r,i){e(t,r),n(t,r,"=",i,";")},toString:function(){return i()+o()}})}var n=0,r=[],i=[],o=t(),a={};return{global:o,link:function(t){for(var e=0;e<i.length;++e)if(i[e]===t)return r[e];return e="g"+n++,r.push(e),i.push(t),e},block:t,proc:function(t,n){function r(){var t="a"+i.length;return i.push(t),t}var i=[];n=n||0;for(var o=0;o<n;++o)r();var s=(o=e()).toString;return a[t]=G(o,{arg:r,toString:function(){return M(["function(",i.join(),"){",s(),"}"])}})},scope:e,cond:function(){var t=M(arguments),n=e(),r=e(),i=n.toString,o=r.toString;return G(n,{then:function(){return n.apply(n,D(arguments)),this},else:function(){return r.apply(r,D(arguments)),this},toString:function(){var e=o();return e&&(e="else{"+e+"}"),M(["if(",t,"){",i(),"}",e])}})},compile:function(){var t=['"use strict";',o,"return {"];Object.keys(a).forEach(function(e){t.push('"',e,'":',a[e].toString(),",")}),t.push("}");var e=M(t).replace(/;/g,";\n").replace(/}/g,"}\n").replace(/{/g,"{\n");return Function.apply(null,r.concat(e)).apply(null,i)}}}function S(t){return Array.isArray(t)||J(t)||A(t)}function L(t){return t.sort(function(t,e){return"viewport"===t?-1:"viewport"===e?1:t<e?-1:1})}function R(t,e,n,r){this.thisDep=t,this.contextDep=e,this.propDep=n,this.append=r}function N(t){return t&&!(t.thisDep||t.contextDep||t.propDep)}function F(t){return new R(!1,!1,!1,t)}function T(t,e){var n=t.type;return 0===n?new R(!0,1<=(n=t.data.length),2<=n,e):4===n?new R((n=t.data).thisDep,n.contextDep,n.propDep,e):new R(3===n,2===n,1===n,e)}function U(t,e,n,r,i,a,s,A,l,c,u,g,h,d,f){function I(t){return t.replace(".","_")}function C(t,e,n){var r=I(t);rt.push(t),et[r]=tt[r]=!!n,it[r]=e}function m(t,e,n){var r=I(t);rt.push(t),Array.isArray(n)?(tt[r]=n.slice(),et[r]=n.slice()):tt[r]=et[r]=n,ot[r]=e}function y(){var t=k(),n=t.link,r=t.global;t.id=At++,t.batchId="0";var i=n(at),o=t.shared={props:"a0"};Object.keys(at).forEach(function(t){o[t]=r.def(i,".",t)});var a=t.next={},s=t.current={};Object.keys(ot).forEach(function(t){Array.isArray(tt[t])&&(a[t]=r.def(o.next,".",t),s[t]=r.def(o.current,".",t))});var A=t.constants={};Object.keys(st).forEach(function(t){A[t]=r.def(JSON.stringify(st[t]))}),t.invoke=function(e,r){switch(r.type){case 0:var i=["this",o.context,o.props,t.batchId];return e.def(n(r.data),".call(",i.slice(0,Math.max(r.data.length+1,4)),")");case 1:return e.def(o.props,r.data);case 2:return e.def(o.context,r.data);case 3:return e.def("this",r.data);case 4:return r.data.append(t,e),r.data.ref}},t.attribCache={};var l={};return t.scopeAttrib=function(t){if((t=e.id(t))in l)return l[t];var r=c.scope[t];return r||(r=c.scope[t]=new V),l[t]=n(r)},t}function E(t,e){var n=t.static,r=t.dynamic;if("framebuffer"in n){var i=n.framebuffer;return i?(i=A.getFramebuffer(i),F(function(t,e){var n=t.link(i),r=t.shared;return e.set(r.framebuffer,".next",n),r=r.context,e.set(r,".framebufferWidth",n+".width"),e.set(r,".framebufferHeight",n+".height"),n})):F(function(t,e){var n=t.shared;return e.set(n.framebuffer,".next","null"),n=n.context,e.set(n,".framebufferWidth",n+".drawingBufferWidth"),e.set(n,".framebufferHeight",n+".drawingBufferHeight"),"null"})}if("framebuffer"in r){var o=r.framebuffer;return T(o,function(t,e){var n=t.invoke(e,o),r=t.shared,i=r.framebuffer;return n=e.def(i,".getFramebuffer(",n,")"),e.set(i,".next",n),r=r.context,e.set(r,".framebufferWidth",n+"?"+n+".width:"+r+".drawingBufferWidth"),e.set(r,".framebufferHeight",n+"?"+n+".height:"+r+".drawingBufferHeight"),n})}return null}function v(t){function n(t){if(t in r){var n=e.id(r[t]);return(t=F(function(){return n})).id=n,t}if(t in i){var o=i[t];return T(o,function(t,e){var n=t.invoke(e,o);return e.def(t.shared.strings,".id(",n,")")})}return null}var r=t.static,i=t.dynamic,o=n("frag"),a=n("vert"),s=null;return N(o)&&N(a)?(s=u.program(a.id,o.id),t=F(function(t,e){return t.link(s)})):t=new R(o&&o.thisDep||a&&a.thisDep,o&&o.contextDep||a&&a.contextDep,o&&o.propDep||a&&a.propDep,function(t,e){var n,r,i=t.shared.shader;return n=o?o.append(t,e):e.def(i,".","frag"),r=a?a.append(t,e):e.def(i,".","vert"),e.def(i+".program("+r+","+n+")")}),{frag:o,vert:a,progVar:t,program:s}}function B(t,e){function n(t,e){if(t in r){var n=0|r[t];return F(function(t,r){return e&&(t.OFFSET=n),n})}if(t in i){var a=i[t];return T(a,function(t,n){var r=t.invoke(n,a);return e&&(t.OFFSET=r),r})}return e&&o?F(function(t,e){return t.OFFSET="0",0}):null}var r=t.static,i=t.dynamic,o=function(){if("elements"in r){var t=r.elements;S(t)?t=a.getElements(a.create(t,!0)):t&&(t=a.getElements(t));var e=F(function(e,n){if(t){var r=e.link(t);return e.ELEMENTS=r}return e.ELEMENTS=null});return e.value=t,e}if("elements"in i){var n=i.elements;return T(n,function(t,e){var r=(i=t.shared).isBufferArgs,i=i.elements,o=t.invoke(e,n),a=e.def("null");return r=e.def(r,"(",o,")"),o=t.cond(r).then(a,"=",i,".createStream(",o,");").else(a,"=",i,".getElements(",o,");"),e.entry(o),e.exit(t.cond(r).then(i,".destroyStream(",a,");")),t.ELEMENTS=a})}return null}(),s=n("offset",!0);return{elements:o,primitive:function(){if("primitive"in r){var t=r.primitive;return F(function(e,n){return nt[t]})}if("primitive"in i){var e=i.primitive;return T(e,function(t,n){var r=t.constants.primTypes,i=t.invoke(n,e);return n.def(r,"[",i,"]")})}return o?N(o)?o.value?F(function(t,e){return e.def(t.ELEMENTS,".primType")}):F(function(){return 4}):new R(o.thisDep,o.contextDep,o.propDep,function(t,e){var n=t.ELEMENTS;return e.def(n,"?",n,".primType:",4)}):null}(),count:function(){if("count"in r){var t=0|r.count;return F(function(){return t})}if("count"in i){var e=i.count;return T(e,function(t,n){return t.invoke(n,e)})}return o?N(o)?o?s?new R(s.thisDep,s.contextDep,s.propDep,function(t,e){return e.def(t.ELEMENTS,".vertCount-",t.OFFSET)}):F(function(t,e){return e.def(t.ELEMENTS,".vertCount")}):F(function(){return-1}):new R(o.thisDep||s.thisDep,o.contextDep||s.contextDep,o.propDep||s.propDep,function(t,e){var n=t.ELEMENTS;return t.OFFSET?e.def(n,"?",n,".vertCount-",t.OFFSET,":-1"):e.def(n,"?",n,".vertCount:-1")}):null}(),instances:n("instances",!1),offset:s}}function b(t,n){var r=t.static,o=t.dynamic,a={};return Object.keys(r).forEach(function(t){var n=r[t],o=e.id(t),s=new V;if(S(n))s.state=1,s.buffer=i.getBuffer(i.create(n,34962,!1,!0)),s.type=0;else if(l=i.getBuffer(n))s.state=1,s.buffer=l,s.type=0;else if("constant"in n){var A=n.constant;s.buffer="null",s.state=2,"number"==typeof A?s.x=A:yt.forEach(function(t,e){e<A.length&&(s[t]=A[e])})}else{var l=S(n.buffer)?i.getBuffer(i.create(n.buffer,34962,!1,!0)):i.getBuffer(n.buffer),c=0|n.offset,u=0|n.stride,g=0|n.size,h=!!n.normalized,d=0;"type"in n&&(d=X[n.type]),n=0|n.divisor,s.buffer=l,s.state=1,s.size=g,s.normalized=h,s.type=d||l.dtype,s.offset=c,s.stride=u,s.divisor=n}a[t]=F(function(t,e){var n=t.attribCache;if(o in n)return n[o];var r={isStream:!1};return Object.keys(s).forEach(function(t){r[t]=s[t]}),s.buffer&&(r.buffer=t.link(s.buffer),r.type=r.type||r.buffer+".dtype"),n[o]=r})}),Object.keys(o).forEach(function(t){var e=o[t];a[t]=T(e,function(t,n){function r(t){n(A[t],"=",i,".",t,"|0;")}var i=t.invoke(n,e),o=t.shared,a=o.isBufferArgs,s=o.buffer,A={isStream:n.def(!1)},l=new V;l.state=1,Object.keys(l).forEach(function(t){A[t]=n.def(""+l[t])});var c=A.buffer,u=A.type;return n("if(",a,"(",i,")){",A.isStream,"=true;",c,"=",s,".createStream(",34962,",",i,");",u,"=",c,".dtype;","}else{",c,"=",s,".getBuffer(",i,");","if(",c,"){",u,"=",c,".dtype;",'}else if("constant" in ',i,"){",A.state,"=",2,";","if(typeof "+i+'.constant === "number"){',A[yt[0]],"=",i,".constant;",yt.slice(1).map(function(t){return A[t]}).join("="),"=0;","}else{",yt.map(function(t,e){return A[t]+"="+i+".constant.length>"+e+"?"+i+".constant["+e+"]:0;"}).join(""),"}}else{","if(",a,"(",i,".buffer)){",c,"=",s,".createStream(",34962,",",i,".buffer);","}else{",c,"=",s,".getBuffer(",i,".buffer);","}",u,'="type" in ',i,"?",o.glTypes,"[",i,".type]:",c,".dtype;",A.normalized,"=!!",i,".normalized;"),r("size"),r("offset"),r("stride"),r("divisor"),n("}}"),n.exit("if(",A.isStream,"){",s,".destroyStream(",c,");","}"),A})}),a}function Q(t,e,n,r,i){var a=E(t),s=function(t,e,n){function r(t){if(t in i){var n=i[t];t=!0;var r,a,s=0|n.x,A=0|n.y;return"width"in n?r=0|n.width:t=!1,"height"in n?a=0|n.height:t=!1,new R(!t&&e&&e.thisDep,!t&&e&&e.contextDep,!t&&e&&e.propDep,function(t,e){var i=t.shared.context,o=r;"width"in n||(o=e.def(i,".","framebufferWidth","-",s));var l=a;return"height"in n||(l=e.def(i,".","framebufferHeight","-",A)),[s,A,o,l]})}if(t in o){var l=o[t];return t=T(l,function(t,e){var n=t.invoke(e,l),r=t.shared.context,i=e.def(n,".x|0"),o=e.def(n,".y|0");return[i,o,e.def('"width" in ',n,"?",n,".width|0:","(",r,".","framebufferWidth","-",i,")"),n=e.def('"height" in ',n,"?",n,".height|0:","(",r,".","framebufferHeight","-",o,")")]}),e&&(t.thisDep=t.thisDep||e.thisDep,t.contextDep=t.contextDep||e.contextDep,t.propDep=t.propDep||e.propDep),t}return e?new R(e.thisDep,e.contextDep,e.propDep,function(t,e){var n=t.shared.context;return[0,0,e.def(n,".","framebufferWidth"),e.def(n,".","framebufferHeight")]}):null}var i=t.static,o=t.dynamic;if(t=r("viewport")){var a=t;t=new R(t.thisDep,t.contextDep,t.propDep,function(t,e){var n=a.append(t,e),r=t.shared.context;return e.set(r,".viewportWidth",n[2]),e.set(r,".viewportHeight",n[3]),n})}return{viewport:t,scissor_box:r("scissor.box")}}(t,a),A=B(t),l=function(t,e){var n=t.static,r=t.dynamic,i={};return rt.forEach(function(t){function e(e,o){if(t in n){var s=e(n[t]);i[a]=F(function(){return s})}else if(t in r){var A=r[t];i[a]=T(A,function(t,e){return o(t,e,t.invoke(e,A))})}}var a=I(t);switch(t){case"cull.enable":case"blend.enable":case"dither":case"stencil.enable":case"depth.enable":case"scissor.enable":case"polygonOffset.enable":case"sample.alpha":case"sample.enable":case"depth.mask":return e(function(t){return t},function(t,e,n){return n});case"depth.func":return e(function(t){return Bt[t]},function(t,e,n){return e.def(t.constants.compareFuncs,"[",n,"]")});case"depth.range":return e(function(t){return t},function(t,e,n){return[e.def("+",n,"[0]"),e=e.def("+",n,"[1]")]});case"blend.func":return e(function(t){return[vt["srcRGB"in t?t.srcRGB:t.src],vt["dstRGB"in t?t.dstRGB:t.dst],vt["srcAlpha"in t?t.srcAlpha:t.src],vt["dstAlpha"in t?t.dstAlpha:t.dst]]},function(t,e,n){function r(t,r){return e.def('"',t,r,'" in ',n,"?",n,".",t,r,":",n,".",t)}t=t.constants.blendFuncs;var i=r("src","RGB"),o=r("dst","RGB"),a=(i=e.def(t,"[",i,"]"),e.def(t,"[",r("src","Alpha"),"]"));return[i,o=e.def(t,"[",o,"]"),a,t=e.def(t,"[",r("dst","Alpha"),"]")]});case"blend.equation":return e(function(t){return"string"==typeof t?[W[t],W[t]]:"object"==typeof t?[W[t.rgb],W[t.alpha]]:void 0},function(t,e,n){var r=t.constants.blendEquations,i=e.def(),o=e.def();return(t=t.cond("typeof ",n,'==="string"')).then(i,"=",o,"=",r,"[",n,"];"),t.else(i,"=",r,"[",n,".rgb];",o,"=",r,"[",n,".alpha];"),e(t),[i,o]});case"blend.color":return e(function(t){return o(4,function(e){return+t[e]})},function(t,e,n){return o(4,function(t){return e.def("+",n,"[",t,"]")})});case"stencil.mask":return e(function(t){return 0|t},function(t,e,n){return e.def(n,"|0")});case"stencil.func":return e(function(t){return[Bt[t.cmp||"keep"],t.ref||0,"mask"in t?t.mask:-1]},function(t,e,n){return[t=e.def('"cmp" in ',n,"?",t.constants.compareFuncs,"[",n,".cmp]",":",7680),e.def(n,".ref|0"),e=e.def('"mask" in ',n,"?",n,".mask|0:-1")]});case"stencil.opFront":case"stencil.opBack":return e(function(e){return["stencil.opBack"===t?1029:1028,bt[e.fail||"keep"],bt[e.zfail||"keep"],bt[e.zpass||"keep"]]},function(e,n,r){function i(t){return n.def('"',t,'" in ',r,"?",o,"[",r,".",t,"]:",7680)}var o=e.constants.stencilOps;return["stencil.opBack"===t?1029:1028,i("fail"),i("zfail"),i("zpass")]});case"polygonOffset.offset":return e(function(t){return[0|t.factor,0|t.units]},function(t,e,n){return[e.def(n,".factor|0"),e=e.def(n,".units|0")]});case"cull.face":return e(function(t){var e=0;return"front"===t?e=1028:"back"===t&&(e=1029),e},function(t,e,n){return e.def(n,'==="front"?',1028,":",1029)});case"lineWidth":return e(function(t){return t},function(t,e,n){return n});case"frontFace":return e(function(t){return Qt[t]},function(t,e,n){return e.def(n+'==="cw"?2304:2305')});case"colorMask":return e(function(t){return t.map(function(t){return!!t})},function(t,e,n){return o(4,function(t){return"!!"+n+"["+t+"]"})});case"sample.coverage":return e(function(t){return["value"in t?t.value:1,!!t.invert]},function(t,e,n){return[e.def('"value" in ',n,"?+",n,".value:1"),e=e.def("!!",n,".invert")]})}}),i}(t),c=v(t),u=s.viewport;return u&&(l.viewport=u),(s=s[u=I("scissor.box")])&&(l[u]=s),(a={framebuffer:a,draw:A,shader:c,state:l,dirty:s=0<Object.keys(l).length}).profile=function(t){var e,n=t.static;if(t=t.dynamic,"profile"in n){var r=!!n.profile;(e=F(function(t,e){return r})).enable=r}else if("profile"in t){var i=t.profile;e=T(i,function(t,e){return t.invoke(e,i)})}return e}(t),a.uniforms=function(t,e){var n=t.static,r=t.dynamic,i={};return Object.keys(n).forEach(function(t){var e,r=n[t];if("number"==typeof r||"boolean"==typeof r)e=F(function(){return r});else if("function"==typeof r){var a=r._reglType;"texture2d"===a||"textureCube"===a?e=F(function(t){return t.link(r)}):"framebuffer"!==a&&"framebufferCube"!==a||(e=F(function(t){return t.link(r.color[0])}))}else p(r)&&(e=F(function(t){return t.global.def("[",o(r.length,function(t){return r[t]}),"]")}));e.value=r,i[t]=e}),Object.keys(r).forEach(function(t){var e=r[t];i[t]=T(e,function(t,n){return t.invoke(n,e)})}),i}(n),a.attributes=b(e),a.context=function(t){var e=t.static,n=t.dynamic,r={};return Object.keys(e).forEach(function(t){var n=e[t];r[t]=F(function(t,e){return"number"==typeof n||"boolean"==typeof n?""+n:t.link(n)})}),Object.keys(n).forEach(function(t){var e=n[t];r[t]=T(e,function(t,n){return t.invoke(n,e)})}),r}(r),a}function w(t,e,n){var r=t.shared.context,i=t.scope();Object.keys(n).forEach(function(o){e.save(r,"."+o),i(r,".",o,"=",n[o].append(t,e),";")}),e(i)}function x(t,e,n,r){var i,o=(s=t.shared).gl,a=s.framebuffer;$&&(i=e.def(s.extensions,".webgl_draw_buffers"));var s=(A=t.constants).drawBuffer,A=A.backBuffer;t=n?n.append(t,e):e.def(a,".next"),r||e("if(",t,"!==",a,".cur){"),e("if(",t,"){",o,".bindFramebuffer(",36160,",",t,".framebuffer);"),$&&e(i,".drawBuffersWEBGL(",s,"[",t,".colorAttachments.length]);"),e("}else{",o,".bindFramebuffer(",36160,",null);"),$&&e(i,".drawBuffersWEBGL(",A,");"),e("}",a,".cur=",t,";"),r||e("}")}function _(t,e,n){var r=t.shared,i=r.gl,a=t.current,s=t.next,A=r.current,l=r.next,c=t.cond(A,".dirty");rt.forEach(function(e){var r,u;if(!((e=I(e))in n.state))if(e in s){r=s[e],u=a[e];var g=o(tt[e].length,function(t){return c.def(r,"[",t,"]")});c(t.cond(g.map(function(t,e){return t+"!=="+u+"["+e+"]"}).join("||")).then(i,".",ot[e],"(",g,");",g.map(function(t,e){return u+"["+e+"]="+t}).join(";"),";"))}else r=c.def(l,".",e),g=t.cond(r,"!==",A,".",e),c(g),e in it?g(t.cond(r).then(i,".enable(",it[e],");").else(i,".disable(",it[e],");"),A,".",e,"=",r,";"):g(i,".",ot[e],"(",r,");",A,".",e,"=",r,";")}),0===Object.keys(n.state).length&&c(A,".dirty=false;"),e(c)}function D(t,e,n,r){var i=t.shared,o=t.current,a=i.current,s=i.gl;L(Object.keys(n)).forEach(function(i){var A=n[i];if(!r||r(A)){var l=A.append(t,e);if(it[i]){var c=it[i];N(A)?e(s,l?".enable(":".disable(",c,");"):e(t.cond(l).then(s,".enable(",c,");").else(s,".disable(",c,");")),e(a,".",i,"=",l,";")}else if(p(l)){var u=o[i];e(s,".",ot[i],"(",l,");",l.map(function(t,e){return u+"["+e+"]="+t}).join(";"),";")}else e(s,".",ot[i],"(",l,");",a,".",i,"=",l,";")}})}function M(t,e){Z&&(t.instancing=e.def(t.shared.extensions,".angle_instanced_arrays"))}function U(t,e,n,r,i){function o(){return"undefined"==typeof performance?"Date.now()":"performance.now()"}function a(t){t(l=e.def(),"=",o(),";"),"string"==typeof i?t(g,".count+=",i,";"):t(g,".count++;"),d&&(r?t(c=e.def(),"=",f,".getNumPendingQueries();"):t(f,".beginQuery(",g,");"))}function s(t){t(g,".cpuTime+=",o(),"-",l,";"),d&&(r?t(f,".pushScopeStats(",c,",",f,".getNumPendingQueries(),",g,");"):t(f,".endQuery();"))}function A(t){var n=e.def(h,".profile");e(h,".profile=",t,";"),e.exit(h,".profile=",n,";")}var l,c,u=t.shared,g=t.stats,h=u.current,f=u.timer;if(n=n.profile){if(N(n))return void(n.enable?(a(e),s(e.exit),A("true")):A("false"));A(n=n.append(t,e))}else n=e.def(h,".profile");a(u=t.block()),e("if(",n,"){",u,"}"),s(t=t.block()),e.exit("if(",n,"){",t,"}")}function O(t,e,n,r,i){function o(n,r,i){function o(){e("if(!",c,".buffer){",A,".enableVertexAttribArray(",l,");}");var n,o=i.type;n=i.size?e.def(i.size,"||",r):r,e("if(",c,".type!==",o,"||",c,".size!==",n,"||",h.map(function(t){return c+"."+t+"!=="+i[t]}).join("||"),"){",A,".bindBuffer(",34962,",",u,".buffer);",A,".vertexAttribPointer(",[l,n,o,i.normalized,i.stride,i.offset],");",c,".type=",o,";",c,".size=",n,";",h.map(function(t){return c+"."+t+"="+i[t]+";"}).join(""),"}"),Z&&(o=i.divisor,e("if(",c,".divisor!==",o,"){",t.instancing,".vertexAttribDivisorANGLE(",[l,o],");",c,".divisor=",o,";}"))}function s(){e("if(",c,".buffer){",A,".disableVertexAttribArray(",l,");","}if(",yt.map(function(t,e){return c+"."+t+"!=="+g[e]}).join("||"),"){",A,".vertexAttrib4f(",l,",",g,");",yt.map(function(t,e){return c+"."+t+"="+g[e]+";"}).join(""),"}")}var A=a.gl,l=e.def(n,".location"),c=e.def(a.attributes,"[",l,"]");n=i.state;var u=i.buffer,g=[i.x,i.y,i.z,i.w],h=["buffer","normalized","offset","stride"];1===n?o():2===n?s():(e("if(",n,"===",1,"){"),o(),e("}else{"),s(),e("}"))}var a=t.shared;r.forEach(function(r){var a,s=r.name,A=n.attributes[s];if(A){if(!i(A))return;a=A.append(t,e)}else{if(!i(wt))return;var l=t.scopeAttrib(s);a={},Object.keys(new V).forEach(function(t){a[t]=e.def(l,".",t)})}o(t.link(r),(r.info.type,1),a)})}function G(t,n,r,i,a){for(var s,A=t.shared,l=A.gl,c=0;c<i.length;++c){var u,g=(f=i[c]).name,h=f.info.type,d=r.uniforms[g],f=t.link(f)+".location";if(d){if(!a(d))continue;if(N(d)){if(g=d.value,35678===h||35680===h)n(l,".uniform1i(",f,",",(h=t.link(g._texture||g.color[0]._texture))+".bind());"),n.exit(h,".unbind();");else if(35674===h||35675===h||35676===h)d=2,35675===h?d=3:35676===h&&(d=4),n(l,".uniformMatrix",d,"fv(",f,",false,",g=t.global.def("new Float32Array(["+Array.prototype.slice.call(g)+"])"),");");else{switch(h){case 5126:s="1f";break;case 35664:s="2f";break;case 35665:s="3f";break;case 35666:s="4f";break;case 35670:case 5124:s="1i";break;case 35671:case 35667:s="2i";break;case 35672:case 35668:s="3i";break;case 35673:s="4i";break;case 35669:s="4i"}n(l,".uniform",s,"(",f,",",p(g)?Array.prototype.slice.call(g):g,");")}continue}u=d.append(t,n)}else{if(!a(wt))continue;u=n.def(A.uniforms,"[",e.id(g),"]")}switch(35678===h?n("if(",u,"&&",u,'._reglType==="framebuffer"){',u,"=",u,".color[0];","}"):35680===h&&n("if(",u,"&&",u,'._reglType==="framebufferCube"){',u,"=",u,".color[0];","}"),g=1,h){case 35678:case 35680:h=n.def(u,"._texture"),n(l,".uniform1i(",f,",",h,".bind());"),n.exit(h,".unbind();");continue;case 5124:case 35670:s="1i";break;case 35667:case 35671:s="2i",g=2;break;case 35668:case 35672:s="3i",g=3;break;case 35669:case 35673:s="4i",g=4;break;case 5126:s="1f";break;case 35664:s="2f",g=2;break;case 35665:s="3f",g=3;break;case 35666:s="4f",g=4;break;case 35674:s="Matrix2fv";break;case 35675:s="Matrix3fv";break;case 35676:s="Matrix4fv"}if(n(l,".uniform",s,"(",f,","),"M"===s.charAt(0)){f=Math.pow(h-35674+2,2);var I=t.global.def("new Float32Array(",f,")");n("false,(Array.isArray(",u,")||",u," instanceof Float32Array)?",u,":(",o(f,function(t){return I+"["+t+"]="+u+"["+t+"]"}),",",I,")")}else n(1<g?o(g,function(t){return u+"["+t+"]"}):u);n(");")}}function H(t,e,n,r){function i(i){var o=g[i];return o?o.contextDep&&r.contextDynamic||o.propDep?o.append(t,n):o.append(t,e):e.def(u,".",i)}function o(){function t(){n(A,".drawElementsInstancedANGLE(",[d,p,I,f+"<<(("+I+"-5121)>>1)",s],");")}function e(){n(A,".drawArraysInstancedANGLE(",[d,f,p,s],");")}h?C?t():(n("if(",h,"){"),t(),n("}else{"),e(),n("}")):e()}function a(){function t(){n(c+".drawElements("+[d,p,I,f+"<<(("+I+"-5121)>>1)"]+");")}function e(){n(c+".drawArrays("+[d,f,p]+");")}h?C?t():(n("if(",h,"){"),t(),n("}else{"),e(),n("}")):e()}var s,A,l=t.shared,c=l.gl,u=l.draw,g=r.draw,h=function(){var i=g.elements,o=e;return i?((i.contextDep&&r.contextDynamic||i.propDep)&&(o=n),i=i.append(t,o)):i=o.def(u,".","elements"),i&&o("if("+i+")"+c+".bindBuffer(34963,"+i+".buffer.buffer);"),i}(),d=i("primitive"),f=i("offset"),p=function(){var i=g.count,o=e;return i?((i.contextDep&&r.contextDynamic||i.propDep)&&(o=n),i=i.append(t,o)):i=o.def(u,".","count"),i}();if("number"==typeof p){if(0===p)return}else n("if(",p,"){"),n.exit("}");Z&&(s=i("instances"),A=t.instancing);var I=h+".type",C=g.elements&&N(g.elements);Z&&("number"!=typeof s||0<=s)?"string"==typeof s?(n("if(",s,">0){"),o(),n("}else if(",s,"<0){"),a(),n("}")):o():a()}function Y(t,e,n,r,i){return i=(e=y()).proc("body",i),Z&&(e.instancing=i.def(e.shared.extensions,".angle_instanced_arrays")),t(e,i,n,r),e.compile().body}function P(t,e,n,r){M(t,e),O(t,e,n,r.attributes,function(){return!0}),G(t,e,n,r.uniforms,function(){return!0}),H(t,e,e,n)}function K(t,e,n,r){function i(){return!0}t.batchId="a1",M(t,e),O(t,e,n,r.attributes,i),G(t,e,n,r.uniforms,i),H(t,e,e,n)}function z(t,e,n,r){function i(t){return t.contextDep&&a||t.propDep}function o(t){return!i(t)}M(t,e);var a=n.contextDep,s=e.def(),A=e.def();t.shared.props=A,t.batchId=s;var l=t.scope(),c=t.scope();e(l.entry,"for(",s,"=0;",s,"<","a1",";++",s,"){",A,"=","a0","[",s,"];",c,"}",l.exit),n.needsContext&&w(t,c,n.context),n.needsFramebuffer&&x(t,c,n.framebuffer),D(t,c,n.state,i),n.profile&&i(n.profile)&&U(t,c,n,!1,!0),r?(O(t,l,n,r.attributes,o),O(t,c,n,r.attributes,i),G(t,l,n,r.uniforms,o),G(t,c,n,r.uniforms,i),H(t,l,c,n)):(e=t.global.def("{}"),r=n.shader.progVar.append(t,c),A=c.def(r,".id"),l=c.def(e,"[",A,"]"),c(t.shared.gl,".useProgram(",r,".program);","if(!",l,"){",l,"=",e,"[",A,"]=",t.link(function(e){return Y(K,t,n,e,2)}),"(",r,");}",l,".call(this,a0[",s,"],",s,");"))}function J(t,n){function r(e){var r=n.shader[e];r&&i.set(o.shader,"."+e,r.append(t,i))}var i=t.proc("scope",3);t.batchId="a2";var o=t.shared,a=o.current;w(t,i,n.context),n.framebuffer&&n.framebuffer.append(t,i),L(Object.keys(n.state)).forEach(function(e){var r=n.state[e].append(t,i);p(r)?r.forEach(function(n,r){i.set(t.next[e],"["+r+"]",n)}):i.set(o.next,"."+e,r)}),U(t,i,n,!0,!0),["elements","offset","count","instances","primitive"].forEach(function(e){var r=n.draw[e];r&&i.set(o.draw,"."+e,""+r.append(t,i))}),Object.keys(n.uniforms).forEach(function(r){i.set(o.uniforms,"["+e.id(r)+"]",n.uniforms[r].append(t,i))}),Object.keys(n.attributes).forEach(function(e){var r=n.attributes[e].append(t,i),o=t.scopeAttrib(e);Object.keys(new V).forEach(function(t){i.set(o,"."+t,r[t])})}),r("vert"),r("frag"),0<Object.keys(n.state).length&&(i(a,".dirty=true;"),i.exit(a,".dirty=true;")),i("a1(",t.shared.context,",a0,",t.batchId,");")}function q(t,e,n){var r=e.static[n];if(r&&function(t){if("object"==typeof t&&!p(t)){for(var e=Object.keys(t),n=0;n<e.length;++n)if(j.isDynamic(t[e[n]]))return!0;return!1}}(r)){var i=t.global,o=Object.keys(r),a=!1,s=!1,A=!1,l=t.global.def("{}");o.forEach(function(e){var n=r[e];if(j.isDynamic(n))"function"==typeof n&&(n=r[e]=j.unbox(n)),e=T(n,null),a=a||e.thisDep,A=A||e.propDep,s=s||e.contextDep;else{switch(i(l,".",e,"="),typeof n){case"number":i(n);break;case"string":i('"',n,'"');break;case"object":Array.isArray(n)&&i("[",n.join(),"]");break;default:i(t.link(n))}i(";")}}),e.dynamic[n]=new j.DynamicVariable(4,{thisDep:a,contextDep:s,propDep:A,ref:l,append:function(t,e){o.forEach(function(n){var i=r[n];j.isDynamic(i)&&(i=t.invoke(e,i),e(l,".",n,"=",i,";"))})}}),delete e.static[n]}}var V=c.Record,W={add:32774,subtract:32778,"reverse subtract":32779};n.ext_blend_minmax&&(W.min=32775,W.max=32776);var Z=n.angle_instanced_arrays,$=n.webgl_draw_buffers,tt={dirty:!0,profile:f.profile},et={},rt=[],it={},ot={};C("dither",3024),C("blend.enable",3042),m("blend.color","blendColor",[0,0,0,0]),m("blend.equation","blendEquationSeparate",[32774,32774]),m("blend.func","blendFuncSeparate",[1,0,1,0]),C("depth.enable",2929,!0),m("depth.func","depthFunc",513),m("depth.range","depthRange",[0,1]),m("depth.mask","depthMask",!0),m("colorMask","colorMask",[!0,!0,!0,!0]),C("cull.enable",2884),m("cull.face","cullFace",1029),m("frontFace","frontFace",2305),m("lineWidth","lineWidth",1),C("polygonOffset.enable",32823),m("polygonOffset.offset","polygonOffset",[0,0]),C("sample.alpha",32926),C("sample.enable",32928),m("sample.coverage","sampleCoverage",[1,!1]),C("stencil.enable",2960),m("stencil.mask","stencilMask",-1),m("stencil.func","stencilFunc",[519,0,-1]),m("stencil.opFront","stencilOpSeparate",[1028,7680,7680,7680]),m("stencil.opBack","stencilOpSeparate",[1029,7680,7680,7680]),C("scissor.enable",3089),m("scissor.box","scissor",[0,0,t.drawingBufferWidth,t.drawingBufferHeight]),m("viewport","viewport",[0,0,t.drawingBufferWidth,t.drawingBufferHeight]);var at={gl:t,context:h,strings:e,next:et,current:tt,draw:g,elements:a,buffer:i,shader:u,attributes:c.state,uniforms:l,framebuffer:A,extensions:n,timer:d,isBufferArgs:S},st={primTypes:nt,compareFuncs:Bt,blendFuncs:vt,blendEquations:W,stencilOps:bt,glTypes:X,orientationType:Qt};$&&(st.backBuffer=[1029],st.drawBuffer=o(r.maxDrawbuffers,function(t){return 0===t?[0]:o(t,function(t){return 36064+t})}));var At=0;return{next:et,current:tt,procs:function(){var t=y(),e=t.proc("poll"),n=t.proc("refresh"),i=t.block();e(i),n(i);var a,s=t.shared,A=s.gl,l=s.next,c=s.current;i(c,".dirty=false;"),x(t,e),x(t,n,null,!0),Z&&(a=t.link(Z));for(var u=0;u<r.maxAttributes;++u){var g=n.def(s.attributes,"[",u,"]"),h=t.cond(g,".buffer");h.then(A,".enableVertexAttribArray(",u,");",A,".bindBuffer(",34962,",",g,".buffer.buffer);",A,".vertexAttribPointer(",u,",",g,".size,",g,".type,",g,".normalized,",g,".stride,",g,".offset);").else(A,".disableVertexAttribArray(",u,");",A,".vertexAttrib4f(",u,",",g,".x,",g,".y,",g,".z,",g,".w);",g,".buffer=null;"),n(h),Z&&n(a,".vertexAttribDivisorANGLE(",u,",",g,".divisor);")}return Object.keys(it).forEach(function(r){var o=it[r],a=i.def(l,".",r),s=t.block();s("if(",a,"){",A,".enable(",o,")}else{",A,".disable(",o,")}",c,".",r,"=",a,";"),n(s),e("if(",a,"!==",c,".",r,"){",s,"}")}),Object.keys(ot).forEach(function(r){var a,s,u=ot[r],g=tt[r],h=t.block();h(A,".",u,"("),p(g)?(u=g.length,a=t.global.def(l,".",r),s=t.global.def(c,".",r),h(o(u,function(t){return a+"["+t+"]"}),");",o(u,function(t){return s+"["+t+"]="+a+"["+t+"];"}).join("")),e("if(",o(u,function(t){return a+"["+t+"]!=="+s+"["+t+"]"}).join("||"),"){",h,"}")):(a=i.def(l,".",r),s=i.def(c,".",r),h(a,");",c,".",r,"=",a,";"),e("if(",a,"!==",s,"){",h,"}")),n(h)}),t.compile()}(),compile:function(t,e,n,r,i){var o=y();return o.stats=o.link(i),Object.keys(e.static).forEach(function(t){q(o,e,t)}),Et.forEach(function(e){q(o,t,e)}),n=Q(t,e,n,r),function(t,e){var n=t.proc("draw",1);M(t,n),w(t,n,e.context),x(t,n,e.framebuffer),_(t,n,e),D(t,n,e.state),U(t,n,e,!1,!0);var r=e.shader.progVar.append(t,n);if(n(t.shared.gl,".useProgram(",r,".program);"),e.shader.program)P(t,n,e,e.shader.program);else{var i=t.global.def("{}"),o=n.def(r,".id"),a=n.def(i,"[",o,"]");n(t.cond(a).then(a,".call(this,a0);").else(a,"=",i,"[",o,"]=",t.link(function(n){return Y(P,t,e,n,1)}),"(",r,");",a,".call(this,a0);"))}0<Object.keys(e.state).length&&n(t.shared.current,".dirty=true;")}(o,n),J(o,n),function(t,e){function n(t){return t.contextDep&&i||t.propDep}var r=t.proc("batch",2);t.batchId="0",M(t,r);var i=!1,o=!0;Object.keys(e.context).forEach(function(t){i=i||e.context[t].propDep}),i||(w(t,r,e.context),o=!1);var a=!1;if((s=e.framebuffer)?(s.propDep?i=a=!0:s.contextDep&&i&&(a=!0),a||x(t,r,s)):x(t,r,null),e.state.viewport&&e.state.viewport.propDep&&(i=!0),_(t,r,e),D(t,r,e.state,function(t){return!n(t)}),e.profile&&n(e.profile)||U(t,r,e,!1,"a1"),e.contextDep=i,e.needsContext=o,e.needsFramebuffer=a,(o=e.shader.progVar).contextDep&&i||o.propDep)z(t,r,e,null);else if(o=o.append(t,r),r(t.shared.gl,".useProgram(",o,".program);"),e.shader.program)z(t,r,e,e.shader.program);else{var s=t.global.def("{}"),A=(a=r.def(o,".id"),r.def(s,"[",a,"]"));r(t.cond(A).then(A,".call(this,a0,a1);").else(A,"=",s,"[",a,"]=",t.link(function(n){return Y(z,t,e,n,2)}),"(",o,");",A,".call(this,a0,a1);"))}0<Object.keys(e.state).length&&r(t.shared.current,".dirty=true;")}(o,n),o.compile()}}}function O(t,e){for(var n=0;n<t.length;++n)if(t[n]===e)return n;return-1}var G=function(t,e){for(var n=Object.keys(e),r=0;r<n.length;++r)t[n[r]]=e[n[r]];return t},H=0,j={DynamicVariable:t,define:function(n,r){return new t(n,e(r+""))},isDynamic:function(e){return"function"==typeof e&&!e._reglType||e instanceof t},unbox:function(e,n){return"function"==typeof e?new t(0,e):e},accessor:e},Y={next:"function"==typeof requestAnimationFrame?function(t){return requestAnimationFrame(t)}:function(t){return setTimeout(t,16)},cancel:"function"==typeof cancelAnimationFrame?function(t){return cancelAnimationFrame(t)}:clearTimeout},P="undefined"!=typeof performance&&performance.now?function(){return performance.now()}:function(){return+new Date},K=s();K.zero=s();var z=function(t,e){var n=1;e.ext_texture_filter_anisotropic&&(n=t.getParameter(34047));var r=1,i=1;e.webgl_draw_buffers&&(r=t.getParameter(34852),i=t.getParameter(36063));var o=!!e.oes_texture_float;if(o){o=t.createTexture(),t.bindTexture(3553,o),t.texImage2D(3553,0,6408,1,1,0,6408,5126,null);var a=t.createFramebuffer();if(t.bindFramebuffer(36160,a),t.framebufferTexture2D(36160,36064,3553,o,0),t.bindTexture(3553,null),36053!==t.checkFramebufferStatus(36160))o=!1;else{t.viewport(0,0,1,1),t.clearColor(1,0,0,1),t.clear(16384);var s=K.allocType(5126,4);t.readPixels(0,0,1,1,6408,5126,s),t.getError()?o=!1:(t.deleteFramebuffer(a),t.deleteTexture(o),o=1===s[0]),K.freeType(s)}}return s=!0,"undefined"!=typeof navigator&&(/MSIE/.test(navigator.userAgent)||/Trident\//.test(navigator.appVersion)||/Edge/.test(navigator.userAgent))||(s=t.createTexture(),a=K.allocType(5121,36),t.activeTexture(33984),t.bindTexture(34067,s),t.texImage2D(34069,0,6408,3,3,0,6408,5121,a),K.freeType(a),t.bindTexture(34067,null),t.deleteTexture(s),s=!t.getError()),{colorBits:[t.getParameter(3410),t.getParameter(3411),t.getParameter(3412),t.getParameter(3413)],depthBits:t.getParameter(3414),stencilBits:t.getParameter(3415),subpixelBits:t.getParameter(3408),extensions:Object.keys(e).filter(function(t){return!!e[t]}),maxAnisotropic:n,maxDrawbuffers:r,maxColorAttachments:i,pointSizeDims:t.getParameter(33901),lineWidthDims:t.getParameter(33902),maxViewportDims:t.getParameter(3386),maxCombinedTextureUnits:t.getParameter(35661),maxCubeMapSize:t.getParameter(34076),maxRenderbufferSize:t.getParameter(34024),maxTextureUnits:t.getParameter(34930),maxTextureSize:t.getParameter(3379),maxAttributes:t.getParameter(34921),maxVertexUniforms:t.getParameter(36347),maxVertexTextureUnits:t.getParameter(35660),maxVaryingVectors:t.getParameter(36348),maxFragmentUniforms:t.getParameter(36349),glsl:t.getParameter(35724),renderer:t.getParameter(7937),vendor:t.getParameter(7936),version:t.getParameter(7938),readFloat:o,npotTextureCube:s}},J=function(t){return t instanceof Uint8Array||t instanceof Uint16Array||t instanceof Uint32Array||t instanceof Int8Array||t instanceof Int16Array||t instanceof Int32Array||t instanceof Float32Array||t instanceof Float64Array||t instanceof Uint8ClampedArray},q=function(t){return Object.keys(t).map(function(e){return t[e]})},V={shape:function(t){for(var e=[];t.length;t=t[0])e.push(t.length);return e},flatten:function(t,e,n,r){var i=1;if(e.length)for(var o=0;o<e.length;++o)i*=e[o];else i=0;switch(n=r||K.allocType(n,i),e.length){case 0:break;case 1:for(r=e[0],e=0;e<r;++e)n[e]=t[e];break;case 2:for(r=e[0],e=e[1],o=i=0;o<r;++o)for(var a=t[o],s=0;s<e;++s)n[i++]=a[s];break;case 3:l(t,e[0],e[1],e[2],n,0);break;default:!function t(e,n,r,i,o){for(var a=1,s=r+1;s<n.length;++s)a*=n[s];var A=n[r];if(4==n.length-r){var c=n[r+1],u=n[r+2];for(n=n[r+3],s=0;s<A;++s)l(e[s],c,u,n,i,o),o+=a}else for(s=0;s<A;++s)t(e[s],n,r+1,i,o),o+=a}(t,e,0,n,0)}return n}},W={"[object Int8Array]":5120,"[object Int16Array]":5122,"[object Int32Array]":5124,"[object Uint8Array]":5121,"[object Uint8ClampedArray]":5121,"[object Uint16Array]":5123,"[object Uint32Array]":5125,"[object Float32Array]":5126,"[object Float64Array]":5121,"[object ArrayBuffer]":5121},X={int8:5120,int16:5122,int32:5124,uint8:5121,uint16:5123,uint32:5125,float:5126,float32:5126},Z={dynamic:35048,stream:35040,static:35044},$=V.flatten,tt=V.shape,et=[];et[5120]=1,et[5122]=2,et[5124]=4,et[5121]=1,et[5123]=2,et[5125]=4,et[5126]=4;var nt={points:0,point:0,lines:1,line:1,triangles:4,triangle:4,"line loop":2,"line strip":3,"triangle strip":5,"triangle fan":6},rt=new Float32Array(1),it=new Uint32Array(rt.buffer),ot=[9984,9986,9985,9987],at=[0,6409,6410,6407,6408],st={};st[6409]=st[6406]=st[6402]=1,st[34041]=st[6410]=2,st[6407]=st[35904]=3,st[6408]=st[35906]=4;var At=I("HTMLCanvasElement"),lt=I("CanvasRenderingContext2D"),ct=I("ImageBitmap"),ut=I("HTMLImageElement"),gt=I("HTMLVideoElement"),ht=Object.keys(W).concat([At,lt,ct,ut,gt]),dt=[];dt[5121]=1,dt[5126]=4,dt[36193]=2,dt[5123]=2,dt[5125]=4;var ft=[];ft[32854]=2,ft[32855]=2,ft[36194]=2,ft[34041]=4,ft[33776]=.5,ft[33777]=.5,ft[33778]=1,ft[33779]=1,ft[35986]=.5,ft[35987]=1,ft[34798]=1,ft[35840]=.5,ft[35841]=.25,ft[35842]=.5,ft[35843]=.25,ft[36196]=.5;var pt=[];pt[32854]=2,pt[32855]=2,pt[36194]=2,pt[33189]=2,pt[36168]=1,pt[34041]=4,pt[35907]=4,pt[34836]=16,pt[34842]=8,pt[34843]=6;var It=function(t,e,n,r,i){function o(t){this.id=l++,this.refCount=1,this.renderbuffer=t,this.format=32854,this.height=this.width=0,i.profile&&(this.stats={size:0})}function a(e){var n=e.renderbuffer;t.bindRenderbuffer(36161,null),t.deleteRenderbuffer(n),e.renderbuffer=null,e.refCount=0,delete c[e.id],r.renderbufferCount--}var s={rgba4:32854,rgb565:36194,"rgb5 a1":32855,depth:33189,stencil:36168,"depth stencil":34041};e.ext_srgb&&(s.srgba=35907),e.ext_color_buffer_half_float&&(s.rgba16f=34842,s.rgb16f=34843),e.webgl_color_buffer_float&&(s.rgba32f=34836);var A=[];Object.keys(s).forEach(function(t){A[s[t]]=t});var l=0,c={};return o.prototype.decRef=function(){0>=--this.refCount&&a(this)},i.profile&&(r.getTotalRenderbufferSize=function(){var t=0;return Object.keys(c).forEach(function(e){t+=c[e].stats.size}),t}),{create:function(e,n){function a(e,n){var r=0,o=0,c=32854;if("object"==typeof e&&e?("shape"in e?(r=0|(o=e.shape)[0],o=0|o[1]):("radius"in e&&(r=o=0|e.radius),"width"in e&&(r=0|e.width),"height"in e&&(o=0|e.height)),"format"in e&&(c=s[e.format])):"number"==typeof e?(r=0|e,o="number"==typeof n?0|n:r):e||(r=o=1),r!==l.width||o!==l.height||c!==l.format)return a.width=l.width=r,a.height=l.height=o,l.format=c,t.bindRenderbuffer(36161,l.renderbuffer),t.renderbufferStorage(36161,c,r,o),i.profile&&(l.stats.size=pt[l.format]*l.width*l.height),a.format=A[l.format],a}var l=new o(t.createRenderbuffer());return c[l.id]=l,r.renderbufferCount++,a(e,n),a.resize=function(e,n){var r=0|e,o=0|n||r;return r===l.width&&o===l.height?a:(a.width=l.width=r,a.height=l.height=o,t.bindRenderbuffer(36161,l.renderbuffer),t.renderbufferStorage(36161,l.format,r,o),i.profile&&(l.stats.size=pt[l.format]*l.width*l.height),a)},a._reglType="renderbuffer",a._renderbuffer=l,i.profile&&(a.stats=l.stats),a.destroy=function(){l.decRef()},a},clear:function(){q(c).forEach(a)},restore:function(){q(c).forEach(function(e){e.renderbuffer=t.createRenderbuffer(),t.bindRenderbuffer(36161,e.renderbuffer),t.renderbufferStorage(36161,e.format,e.width,e.height)}),t.bindRenderbuffer(36161,null)}}},Ct=[];Ct[6408]=4,Ct[6407]=3;var mt=[];mt[5121]=1,mt[5126]=4,mt[36193]=2;var yt=["x","y","z","w"],Et="blend.func blend.equation stencil.func stencil.opFront stencil.opBack sample.coverage viewport scissor.box polygonOffset.offset".split(" "),vt={0:0,1:1,zero:0,one:1,"src color":768,"one minus src color":769,"src alpha":770,"one minus src alpha":771,"dst color":774,"one minus dst color":775,"dst alpha":772,"one minus dst alpha":773,"constant color":32769,"one minus constant color":32770,"constant alpha":32771,"one minus constant alpha":32772,"src alpha saturate":776},Bt={never:512,less:513,"<":513,equal:514,"=":514,"==":514,"===":514,lequal:515,"<=":515,greater:516,">":516,notequal:517,"!=":517,"!==":517,gequal:518,">=":518,always:519},bt={0:0,zero:0,keep:7680,replace:7681,increment:7682,decrement:7683,"increment wrap":34055,"decrement wrap":34056,invert:5386},Qt={cw:2304,ccw:2305},wt=new R(!1,!1,!1,function(){});return function(t){function e(){if(0===V.length)v&&v.update(),$=null;else{$=Y.next(e),u();for(var t=V.length-1;0<=t;--t){var n=V[t];n&&n(k,null,0)}p.flush(),v&&v.update()}}function n(){!$&&0<V.length&&($=Y.next(e))}function r(){$&&(Y.cancel(e),$=null)}function o(t){t.preventDefault(),r(),W.forEach(function(t){t()})}function a(t){p.getError(),C.restore(),N.restore(),L.restore(),F.restore(),T.restore(),H.restore(),v&&v.restore(),K.procs.refresh(),n(),X.forEach(function(t){t()})}function s(t){function e(t){var e={},n={};return Object.keys(t).forEach(function(r){var i=t[r];j.isDynamic(i)?n[r]=j.unbox(i,r):e[r]=i}),{dynamic:n,static:e}}var n=e(t.context||{}),r=e(t.uniforms||{}),i=e(t.attributes||{}),o=e(function(t){function e(t){if(t in n){var e=n[t];delete n[t],Object.keys(e).forEach(function(r){n[t+"."+r]=e[r]})}}var n=G({},t);return delete n.uniforms,delete n.attributes,delete n.context,"stencil"in n&&n.stencil.op&&(n.stencil.opBack=n.stencil.opFront=n.stencil.op,delete n.stencil.op),e("blend"),e("depth"),e("cull"),e("stencil"),e("polygonOffset"),e("scissor"),e("sample"),n}(t));t={gpuTime:0,cpuTime:0,count:0};var a=(n=K.compile(o,i,r,n,t)).draw,s=n.batch,A=n.scope,l=[];return G(function(t,e){var n;if("function"==typeof t)return A.call(this,null,t,0);if("function"==typeof e)if("number"==typeof t)for(n=0;n<t;++n)A.call(this,null,e,n);else{if(!Array.isArray(t))return A.call(this,t,e,0);for(n=0;n<t.length;++n)A.call(this,t[n],e,n)}else if("number"==typeof t){if(0<t)return s.call(this,function(t){for(;l.length<t;)l.push(null);return l}(0|t),0|t)}else{if(!Array.isArray(t))return a.call(this,t);if(t.length)return s.call(this,t,t.length)}},{stats:t})}function A(t,e){var n=0;K.procs.poll();var r=e.color;r&&(p.clearColor(+r[0]||0,+r[1]||0,+r[2]||0,+r[3]||0),n|=16384),"depth"in e&&(p.clearDepth(+e.depth),n|=256),"stencil"in e&&(p.clearStencil(0|e.stencil),n|=1024),p.clear(n)}function l(t){return V.push(t),n(),{cancel:function(){var e=O(V,t);V[e]=function t(){var e=O(V,t);V[e]=V[V.length-1],--V.length,0>=V.length&&r()}}}}function c(){var t=J.viewport,e=J.scissor_box;t[0]=t[1]=e[0]=e[1]=0,k.viewportWidth=k.framebufferWidth=k.drawingBufferWidth=t[2]=e[2]=p.drawingBufferWidth,k.viewportHeight=k.framebufferHeight=k.drawingBufferHeight=t[3]=e[3]=p.drawingBufferHeight}function u(){k.tick+=1,k.time=f(),c(),K.procs.poll()}function g(){c(),K.procs.refresh(),v&&v.update()}function f(){return(P()-B)/1e3}if(!(t=i(t)))return null;var p=t.gl,I=p.getContextAttributes();p.isContextLost();var C=function(t,e){function n(e){var n;e=e.toLowerCase();try{n=r[e]=t.getExtension(e)}catch(t){}return!!n}for(var r={},i=0;i<e.extensions.length;++i){var o=e.extensions[i];if(!n(o))return e.onDestroy(),e.onDone('"'+o+'" extension is not supported by the current WebGL context, try upgrading your system or a different browser'),null}return e.optionalExtensions.forEach(n),{extensions:r,restore:function(){Object.keys(r).forEach(function(t){if(r[t]&&!n(t))throw Error("(regl): error restoring extension "+t)})}}}(p,t);if(!C)return null;var m=function(){var t={"":0},e=[""];return{id:function(n){var r=t[n];return r||(r=t[n]=e.length,e.push(n),r)},str:function(t){return e[t]}}}(),y={bufferCount:0,elementsCount:0,framebufferCount:0,shaderCount:0,textureCount:0,cubeCount:0,renderbufferCount:0,maxTextureUnits:0},E=C.extensions,v=function(t,e){function n(){this.endQueryIndex=this.startQueryIndex=-1,this.sum=0,this.stats=null}function r(t,e,r){var i=a.pop()||new n;i.startQueryIndex=t,i.endQueryIndex=e,i.sum=0,i.stats=r,s.push(i)}if(!e.ext_disjoint_timer_query)return null;var i=[],o=[],a=[],s=[],A=[],l=[];return{beginQuery:function(t){var n=i.pop()||e.ext_disjoint_timer_query.createQueryEXT();e.ext_disjoint_timer_query.beginQueryEXT(35007,n),o.push(n),r(o.length-1,o.length,t)},endQuery:function(){e.ext_disjoint_timer_query.endQueryEXT(35007)},pushScopeStats:r,update:function(){var t,n;if(0!==(t=o.length)){l.length=Math.max(l.length,t+1),A.length=Math.max(A.length,t+1),A[0]=0;var r=l[0]=0;for(n=t=0;n<o.length;++n){var c=o[n];e.ext_disjoint_timer_query.getQueryObjectEXT(c,34919)?(r+=e.ext_disjoint_timer_query.getQueryObjectEXT(c,34918),i.push(c)):o[t++]=c,A[n+1]=r,l[n+1]=t}for(o.length=t,n=t=0;n<s.length;++n){var u=(r=s[n]).startQueryIndex;c=r.endQueryIndex,r.sum+=A[c]-A[u],u=l[u],(c=l[c])===u?(r.stats.gpuTime+=r.sum/1e6,a.push(r)):(r.startQueryIndex=u,r.endQueryIndex=c,s[t++]=r)}s.length=t}},getNumPendingQueries:function(){return o.length},clear:function(){i.push.apply(i,o);for(var t=0;t<i.length;t++)e.ext_disjoint_timer_query.deleteQueryEXT(i[t]);o.length=0,i.length=0},restore:function(){o.length=0,i.length=0}}}(0,E),B=P(),D=p.drawingBufferWidth,M=p.drawingBufferHeight,k={tick:0,time:0,viewportWidth:D,viewportHeight:M,framebufferWidth:D,framebufferHeight:M,drawingBufferWidth:D,drawingBufferHeight:M,pixelRatio:t.pixelRatio},S=z(p,E),L=(D=function(t,e,n,r){for(t=n.maxAttributes,e=Array(t),n=0;n<t;++n)e[n]=new w;return{Record:w,scope:{},state:e}}(p,E,S),h(p,y,t,D)),R=d(p,E,L,y),N=x(p,m,y,t),F=b(p,E,S,function(){K.procs.poll()},k,y,t),T=It(p,E,0,y,t),H=Q(p,E,S,F,T,y),K=U(p,m,E,S,L,R,0,H,{},D,N,{elements:null,primitive:4,count:-1,offset:0,instances:-1},k,v,t),J=(m=_(p,H,K.procs.poll,k),K.next),q=p.canvas,V=[],W=[],X=[],Z=[t.onDestroy],$=null;q&&(q.addEventListener("webglcontextlost",o,!1),q.addEventListener("webglcontextrestored",a,!1));var tt=H.setFBO=s({framebuffer:j.define.call(null,1,"framebuffer")});return g(),I=G(s,{clear:function(t){if("framebuffer"in t)if(t.framebuffer&&"framebufferCube"===t.framebuffer_reglType)for(var e=0;6>e;++e)tt(G({framebuffer:t.framebuffer.faces[e]},t),A);else tt(t,A);else A(0,t)},prop:j.define.bind(null,1),context:j.define.bind(null,2),this:j.define.bind(null,3),draw:s({}),buffer:function(t){return L.create(t,34962,!1,!1)},elements:function(t){return R.create(t,!1)},texture:F.create2D,cube:F.createCube,renderbuffer:T.create,framebuffer:H.create,framebufferCube:H.createCube,attributes:I,frame:l,on:function(t,e){var n;switch(t){case"frame":return l(e);case"lost":n=W;break;case"restore":n=X;break;case"destroy":n=Z}return n.push(e),{cancel:function(){for(var t=0;t<n.length;++t)if(n[t]===e){n[t]=n[n.length-1],n.pop();break}}}},limits:S,hasExtension:function(t){return 0<=S.extensions.indexOf(t.toLowerCase())},read:m,destroy:function(){V.length=0,r(),q&&(q.removeEventListener("webglcontextlost",o),q.removeEventListener("webglcontextrestored",a)),N.clear(),H.clear(),T.clear(),F.clear(),R.clear(),L.clear(),v&&v.clear(),Z.forEach(function(t){t()})},_gl:p,_refresh:g,poll:function(){u(),v&&v.update()},now:f,stats:y}),t.onDone(null,I),I}},"object"==typeof n&&void 0!==e?e.exports=r():this.createREGL=r()},{}],503:[function(t,e,n){"use strict";var r,i="";e.exports=function(t,e){if("string"!=typeof t)throw new TypeError("expected a string");if(1===e)return t;if(2===e)return t+t;var n=t.length*e;if(r!==t||void 0===r)r=t,i="";else if(i.length>=n)return i.substr(0,n);for(;n>i.length&&e>1;)1&e&&(i+=t),e>>=1,t+=t;return i=(i+=t).substr(0,n)}},{}],504:[function(t,n,r){(function(t){n.exports=t.performance&&t.performance.now?function(){return performance.now()}:Date.now||function(){return+new Date}}).call(this,void 0!==e?e:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{}],505:[function(t,e,n){"use strict";e.exports=function(t){for(var e=t.length,n=t[t.length-1],r=e,i=e-2;i>=0;--i){var o=n,a=t[i],s=(n=o+a)-o,A=a-s;A&&(t[--r]=n,n=A)}for(var l=0,i=r;i<e;++i){var o=t[i],a=n,s=(n=o+a)-o,A=a-s;A&&(t[l++]=A)}return t[l++]=n,t.length=l,t}},{}],506:[function(t,e,n){"use strict";var r=t("two-product"),i=t("robust-sum"),o=t("robust-scale"),a=t("robust-compress");function s(t,e){for(var n=new Array(t.length-1),r=1;r<t.length;++r)for(var i=n[r-1]=new Array(t.length-1),o=0,a=0;o<t.length;++o)o!==e&&(i[a++]=t[r][o]);return n}function A(t){if(2===t.length)return["sum(prod(",t[0][0],",",t[1][1],"),prod(-",t[0][1],",",t[1][0],"))"].join("");for(var e=[],n=0;n<t.length;++n)e.push(["scale(",A(s(t,n)),",",(r=n,1&r?"-":""),t[0][n],")"].join(""));return function t(e){if(1===e.length)return e[0];if(2===e.length)return["sum(",e[0],",",e[1],")"].join("");var n=e.length>>1;return["sum(",t(e.slice(0,n)),",",t(e.slice(n)),")"].join("")}(e);var r}function l(t){return new Function("sum","scale","prod","compress",["function robustDeterminant",t,"(m){return compress(",A(function(t){for(var e=new Array(t),n=0;n<t;++n){e[n]=new Array(t);for(var r=0;r<t;++r)e[n][r]=["m[",n,"][",r,"]"].join("")}return e}(t)),")};return robustDeterminant",t].join(""))(i,o,r,a)}var c=[function(){return[0]},function(t){return[t[0][0]]}];!function(){for(;c.length<6;)c.push(l(c.length));for(var t=[],n=["function robustDeterminant(m){switch(m.length){"],r=0;r<6;++r)t.push("det"+r),n.push("case ",r,":return det",r,"(m);");n.push("}var det=CACHE[m.length];if(!det)det=CACHE[m.length]=gen(m.length);return det(m);}return robustDeterminant"),t.push("CACHE","gen",n.join(""));var i=Function.apply(void 0,t);for(e.exports=i.apply(void 0,c.concat([c,l])),r=0;r<c.length;++r)e.exports[r]=c[r]}()},{"robust-compress":505,"robust-scale":512,"robust-sum":515,"two-product":543}],507:[function(t,e,n){"use strict";var r=t("two-product"),i=t("robust-sum");e.exports=function(t,e){for(var n=r(t[0],e[0]),o=1;o<t.length;++o)n=i(n,r(t[o],e[o]));return n}},{"robust-sum":515,"two-product":543}],508:[function(t,e,n){"use strict";var r=t("two-product"),i=t("robust-sum"),o=t("robust-subtract"),a=t("robust-scale");function s(t,e){for(var n=new Array(t.length-1),r=1;r<t.length;++r)for(var i=n[r-1]=new Array(t.length-1),o=0,a=0;o<t.length;++o)o!==e&&(i[a++]=t[r][o]);return n}function A(t){if(1===t.length)return t[0];if(2===t.length)return["sum(",t[0],",",t[1],")"].join("");var e=t.length>>1;return["sum(",A(t.slice(0,e)),",",A(t.slice(e)),")"].join("")}function l(t,e){if("m"===t.charAt(0)){if("w"===e.charAt(0)){var n=t.split("[");return["w",e.substr(1),"m",n[0].substr(1)].join("")}return["prod(",t,",",e,")"].join("")}return l(e,t)}function c(t){if(2===t.length)return[["diff(",l(t[0][0],t[1][1]),",",l(t[1][0],t[0][1]),")"].join("")];for(var e=[],n=0;n<t.length;++n)e.push(["scale(",A(c(s(t,n))),",",(r=n,!0&r?"-":""),t[0][n],")"].join(""));return e;var r}function u(t,e){for(var n=[],r=0;r<e-2;++r)n.push(["prod(m",t,"[",r,"],m",t,"[",r,"])"].join(""));return A(n)}function g(t){for(var e=[],n=[],l=function(t){for(var e=new Array(t),n=0;n<t;++n){e[n]=new Array(t);for(var r=0;r<t;++r)e[n][r]=["m",r,"[",t-n-2,"]"].join("")}return e}(t),g=0;g<t;++g)l[0][g]="1",l[t-1][g]="w"+g;for(g=0;g<t;++g)0==(1&g)?e.push.apply(e,c(s(l,g))):n.push.apply(n,c(s(l,g)));var h=A(e),d=A(n),f="exactInSphere"+t,p=[];for(g=0;g<t;++g)p.push("m"+g);var I=["function ",f,"(",p.join(),"){"];for(g=0;g<t;++g){I.push("var w",g,"=",u(g,t),";");for(var C=0;C<t;++C)C!==g&&I.push("var w",g,"m",C,"=scale(w",g,",m",C,"[0]);")}return I.push("var p=",h,",n=",d,",d=diff(p,n);return d[d.length-1];}return ",f),new Function("sum","diff","prod","scale",I.join(""))(i,o,r,a)}var h=[function(){return 0},function(){return 0},function(){return 0}];!function(){for(;h.length<=6;)h.push(g(h.length));for(var t=[],n=["slow"],r=0;r<=6;++r)t.push("a"+r),n.push("o"+r);var i=["function testInSphere(",t.join(),"){switch(arguments.length){case 0:case 1:return 0;"];for(r=2;r<=6;++r)i.push("case ",r,":return o",r,"(",t.slice(0,r).join(),");");i.push("}var s=new Array(arguments.length);for(var i=0;i<arguments.length;++i){s[i]=arguments[i]};return slow(s);}return testInSphere"),n.push(i.join(""));var o=Function.apply(void 0,n);for(e.exports=o.apply(void 0,[function(t){var e=h[t.length];return e||(e=h[t.length]=g(t.length)),e.apply(void 0,t)}].concat(h)),r=0;r<=6;++r)e.exports[r]=h[r]}()},{"robust-scale":512,"robust-subtract":514,"robust-sum":515,"two-product":543}],509:[function(t,e,n){"use strict";var r=t("robust-determinant");function i(t){for(var e="robustLinearSolve"+t+"d",n=["function ",e,"(A,b){return ["],i=0;i<t;++i){n.push("det([");for(var o=0;o<t;++o){o>0&&n.push(","),n.push("[");for(var a=0;a<t;++a)a>0&&n.push(","),a===i?n.push("+b[",o,"]"):n.push("+A[",o,"][",a,"]");n.push("]")}n.push("]),")}n.push("det(A)]}return ",e);var s=new Function("det",n.join(""));return s(t<6?r[t]:r)}var o=[function(){return[0]},function(t,e){return[[e[0]],[t[0][0]]]}];!function(){for(;o.length<6;)o.push(i(o.length));for(var t=[],n=["function dispatchLinearSolve(A,b){switch(A.length){"],r=0;r<6;++r)t.push("s"+r),n.push("case ",r,":return s",r,"(A,b);");n.push("}var s=CACHE[A.length];if(!s)s=CACHE[A.length]=g(A.length);return s(A,b)}return dispatchLinearSolve"),t.push("CACHE","g",n.join(""));var a=Function.apply(void 0,t);for(e.exports=a.apply(void 0,o.concat([o,i])),r=0;r<6;++r)e.exports[r]=o[r]}()},{"robust-determinant":506}],510:[function(t,e,n){"use strict";var r=t("two-product"),i=t("robust-sum"),o=t("robust-scale"),a=t("robust-subtract");function s(t,e){for(var n=new Array(t.length-1),r=1;r<t.length;++r)for(var i=n[r-1]=new Array(t.length-1),o=0,a=0;o<t.length;++o)o!==e&&(i[a++]=t[r][o]);return n}function A(t){if(1===t.length)return t[0];if(2===t.length)return["sum(",t[0],",",t[1],")"].join("");var e=t.length>>1;return["sum(",A(t.slice(0,e)),",",A(t.slice(e)),")"].join("")}function l(t){if(2===t.length)return[["sum(prod(",t[0][0],",",t[1][1],"),prod(-",t[0][1],",",t[1][0],"))"].join("")];for(var e=[],n=0;n<t.length;++n)e.push(["scale(",A(l(s(t,n))),",",(r=n,1&r?"-":""),t[0][n],")"].join(""));return e;var r}function c(t){for(var e=[],n=[],c=function(t){for(var e=new Array(t),n=0;n<t;++n){e[n]=new Array(t);for(var r=0;r<t;++r)e[n][r]=["m",r,"[",t-n-1,"]"].join("")}return e}(t),u=[],g=0;g<t;++g)0==(1&g)?e.push.apply(e,l(s(c,g))):n.push.apply(n,l(s(c,g))),u.push("m"+g);var h=A(e),d=A(n),f="orientation"+t+"Exact",p=["function ",f,"(",u.join(),"){var p=",h,",n=",d,",d=sub(p,n);return d[d.length-1];};return ",f].join("");return new Function("sum","prod","scale","sub",p)(i,r,o,a)}var u=c(3),g=c(4),h=[function(){return 0},function(){return 0},function(t,e){return e[0]-t[0]},function(t,e,n){var r,i=(t[1]-n[1])*(e[0]-n[0]),o=(t[0]-n[0])*(e[1]-n[1]),a=i-o;if(i>0){if(o<=0)return a;r=i+o}else{if(!(i<0))return a;if(o>=0)return a;r=-(i+o)}var s=33306690738754716e-32*r;return a>=s||a<=-s?a:u(t,e,n)},function(t,e,n,r){var i=t[0]-r[0],o=e[0]-r[0],a=n[0]-r[0],s=t[1]-r[1],A=e[1]-r[1],l=n[1]-r[1],c=t[2]-r[2],u=e[2]-r[2],h=n[2]-r[2],d=o*l,f=a*A,p=a*s,I=i*l,C=i*A,m=o*s,y=c*(d-f)+u*(p-I)+h*(C-m),E=7771561172376103e-31*((Math.abs(d)+Math.abs(f))*Math.abs(c)+(Math.abs(p)+Math.abs(I))*Math.abs(u)+(Math.abs(C)+Math.abs(m))*Math.abs(h));return y>E||-y>E?y:g(t,e,n,r)}];!function(){for(;h.length<=5;)h.push(c(h.length));for(var t=[],n=["slow"],r=0;r<=5;++r)t.push("a"+r),n.push("o"+r);var i=["function getOrientation(",t.join(),"){switch(arguments.length){case 0:case 1:return 0;"];for(r=2;r<=5;++r)i.push("case ",r,":return o",r,"(",t.slice(0,r).join(),");");i.push("}var s=new Array(arguments.length);for(var i=0;i<arguments.length;++i){s[i]=arguments[i]};return slow(s);}return getOrientation"),n.push(i.join(""));var o=Function.apply(void 0,n);for(e.exports=o.apply(void 0,[function(t){var e=h[t.length];return e||(e=h[t.length]=c(t.length)),e.apply(void 0,t)}].concat(h)),r=0;r<=5;++r)e.exports[r]=h[r]}()},{"robust-scale":512,"robust-subtract":514,"robust-sum":515,"two-product":543}],511:[function(t,e,n){"use strict";var r=t("robust-sum"),i=t("robust-scale");e.exports=function(t,e){if(1===t.length)return i(e,t[0]);if(1===e.length)return i(t,e[0]);if(0===t.length||0===e.length)return[0];var n=[0];if(t.length<e.length)for(var o=0;o<t.length;++o)n=r(n,i(e,t[o]));else for(var o=0;o<e.length;++o)n=r(n,i(t,e[o]));return n}},{"robust-scale":512,"robust-sum":515}],512:[function(t,e,n){"use strict";var r=t("two-product"),i=t("two-sum");e.exports=function(t,e){var n=t.length;if(1===n){var o=r(t[0],e);return o[0]?o:[o[1]]}var a=new Array(2*n),s=[.1,.1],A=[.1,.1],l=0;r(t[0],e,s),s[0]&&(a[l++]=s[0]);for(var c=1;c<n;++c){r(t[c],e,A);var u=s[1];i(u,A[0],s),s[0]&&(a[l++]=s[0]);var g=A[1],h=s[1],d=g+h,f=d-g,p=h-f;s[1]=d,p&&(a[l++]=p)}return s[1]&&(a[l++]=s[1]),0===l&&(a[l++]=0),a.length=l,a}},{"two-product":543,"two-sum":544}],513:[function(t,e,n){"use strict";e.exports=function(t,e,n,i){var o=r(t,n,i),a=r(e,n,i);if(o>0&&a>0||o<0&&a<0)return!1;var s=r(n,t,e),A=r(i,t,e);return!(s>0&&A>0||s<0&&A<0)&&(0!==o||0!==a||0!==s||0!==A||function(t,e,n,r){for(var i=0;i<2;++i){var o=t[i],a=e[i],s=Math.min(o,a),A=Math.max(o,a),l=n[i],c=r[i],u=Math.min(l,c),g=Math.max(l,c);if(g<s||A<u)return!1}return!0}(t,e,n,i))};var r=t("robust-orientation")[3]},{"robust-orientation":510}],514:[function(t,e,n){"use strict";e.exports=function(t,e){var n=0|t.length,r=0|e.length;if(1===n&&1===r)return function(t,e){var n=t+e,r=n-t,i=t-(n-r)+(e-r);return i?[i,n]:[n]}(t[0],-e[0]);var i,o,a=new Array(n+r),s=0,A=0,l=0,c=Math.abs,u=t[A],g=c(u),h=-e[l],d=c(h);g<d?(o=u,(A+=1)<n&&(u=t[A],g=c(u))):(o=h,(l+=1)<r&&(h=-e[l],d=c(h))),A<n&&g<d||l>=r?(i=u,(A+=1)<n&&(u=t[A],g=c(u))):(i=h,(l+=1)<r&&(h=-e[l],d=c(h)));for(var f,p,I=i+o,C=I-i,m=o-C,y=m,E=I;A<n&&l<r;)g<d?(i=u,(A+=1)<n&&(u=t[A],g=c(u))):(i=h,(l+=1)<r&&(h=-e[l],d=c(h))),(m=(o=y)-(C=(I=i+o)-i))&&(a[s++]=m),y=E-((f=E+I)-(p=f-E))+(I-p),E=f;for(;A<n;)(m=(o=y)-(C=(I=(i=u)+o)-i))&&(a[s++]=m),y=E-((f=E+I)-(p=f-E))+(I-p),E=f,(A+=1)<n&&(u=t[A]);for(;l<r;)(m=(o=y)-(C=(I=(i=h)+o)-i))&&(a[s++]=m),y=E-((f=E+I)-(p=f-E))+(I-p),E=f,(l+=1)<r&&(h=-e[l]);return y&&(a[s++]=y),E&&(a[s++]=E),s||(a[s++]=0),a.length=s,a}},{}],515:[function(t,e,n){"use strict";e.exports=function(t,e){var n=0|t.length,r=0|e.length;if(1===n&&1===r)return function(t,e){var n=t+e,r=n-t,i=t-(n-r)+(e-r);return i?[i,n]:[n]}(t[0],e[0]);var i,o,a=new Array(n+r),s=0,A=0,l=0,c=Math.abs,u=t[A],g=c(u),h=e[l],d=c(h);g<d?(o=u,(A+=1)<n&&(u=t[A],g=c(u))):(o=h,(l+=1)<r&&(h=e[l],d=c(h))),A<n&&g<d||l>=r?(i=u,(A+=1)<n&&(u=t[A],g=c(u))):(i=h,(l+=1)<r&&(h=e[l],d=c(h)));for(var f,p,I=i+o,C=I-i,m=o-C,y=m,E=I;A<n&&l<r;)g<d?(i=u,(A+=1)<n&&(u=t[A],g=c(u))):(i=h,(l+=1)<r&&(h=e[l],d=c(h))),(m=(o=y)-(C=(I=i+o)-i))&&(a[s++]=m),y=E-((f=E+I)-(p=f-E))+(I-p),E=f;for(;A<n;)(m=(o=y)-(C=(I=(i=u)+o)-i))&&(a[s++]=m),y=E-((f=E+I)-(p=f-E))+(I-p),E=f,(A+=1)<n&&(u=t[A]);for(;l<r;)(m=(o=y)-(C=(I=(i=h)+o)-i))&&(a[s++]=m),y=E-((f=E+I)-(p=f-E))+(I-p),E=f,(l+=1)<r&&(h=e[l]);return y&&(a[s++]=y),E&&(a[s++]=E),s||(a[s++]=0),a.length=s,a}},{}],516:[function(t,e,n){"use strict";e.exports=function(t){return t<0?-1:t>0?1:0}},{}],517:[function(t,e,n){"use strict";e.exports=function(t){return i(r(t))};var r=t("boundary-cells"),i=t("reduce-simplicial-complex")},{"boundary-cells":95,"reduce-simplicial-complex":489}],518:[function(t,e,n){"use strict";e.exports=function(t,e,n,s){if(n=n||0,void 0===s&&(s=function(t){for(var e=t.length,n=0,r=0;r<e;++r)n=0|Math.max(n,t[r].length);return n-1}(t)),0===t.length||s<1)return{cells:[],vertexIds:[],vertexWeights:[]};var A=function(t,e){for(var n=t.length,r=i.mallocUint8(n),o=0;o<n;++o)r[o]=t[o]<e|0;return r}(e,+n),l=function(t,e){for(var n=t.length,a=e*(e+1)/2*n|0,s=i.mallocUint32(2*a),A=0,l=0;l<n;++l)for(var c=t[l],e=c.length,u=0;u<e;++u)for(var g=0;g<u;++g){var h=c[g],d=c[u];s[A++]=0|Math.min(h,d),s[A++]=0|Math.max(h,d)}o(r(s,[A/2|0,2]));for(var f=2,l=2;l<A;l+=2)s[l-2]===s[l]&&s[l-1]===s[l+1]||(s[f++]=s[l],s[f++]=s[l+1]);return r(s,[f/2|0,2])}(t,s),c=function(t,e,n,o){for(var a=t.data,s=t.shape[0],A=i.mallocDouble(s),l=0,c=0;c<s;++c){var u=a[2*c],g=a[2*c+1];if(n[u]!==n[g]){var h=e[u],d=e[g];a[2*l]=u,a[2*l+1]=g,A[l++]=(d-o)/(d-h)}}return t.shape[0]=l,r(A,[l])}(l,e,A,+n),u=function(t,e){var n=i.mallocInt32(2*e),r=t.shape[0],o=t.data;n[0]=0;for(var a=0,s=0;s<r;++s){var A=o[2*s];if(A!==a){for(n[2*a+1]=s;++a<A;)n[2*a]=s,n[2*a+1]=s;n[2*a]=s}}for(n[2*a+1]=r;++a<e;)n[2*a]=n[2*a+1]=r;return n}(l,0|e.length),g=a(s)(t,l.data,u,A),h=function(t){for(var e=0|t.shape[0],n=t.data,r=new Array(e),i=0;i<e;++i)r[i]=[n[2*i],n[2*i+1]];return r}(l),d=[].slice.call(c.data,0,c.shape[0]);return i.free(A),i.free(l.data),i.free(c.data),i.free(u),{cells:g,vertexIds:h,vertexWeights:d}};var r=t("ndarray"),i=t("typedarray-pool"),o=t("ndarray-sort"),a=t("./lib/codegen")},{"./lib/codegen":519,ndarray:450,"ndarray-sort":448,"typedarray-pool":545}],519:[function(t,e,n){"use strict";e.exports=function(t){var e=o[t];return e||(e=o[t]=function(t){var e=0,n=new Array(t+1);n[0]=[[]];for(var o=1;o<=t;++o)for(var a=n[o]=i(o),s=0;s<a.length;++s)e=Math.max(e,a[o].length);var A=["function B(C,E,i,j){","var a=Math.min(i,j)|0,b=Math.max(i,j)|0,l=C[2*a],h=C[2*a+1];","while(l<h){","var m=(l+h)>>1,v=E[2*m+1];","if(v===b){return m}","if(b<v){h=m}else{l=m+1}","}","return l;","};","function getContour",t,"d(F,E,C,S){","var n=F.length,R=[];","for(var i=0;i<n;++i){var c=F[i],l=c.length;"];function l(t){if(!(t.length<=0)){A.push("R.push(");for(var e=0;e<t.length;++e){var n=t[e];e>0&&A.push(","),A.push("[");for(var r=0;r<n.length;++r){var i=n[r];r>0&&A.push(","),A.push("B(C,E,c[",i[0],"],c[",i[1],"])")}A.push("]")}A.push(");")}}for(var o=t+1;o>1;--o){o<t+1&&A.push("else "),A.push("if(l===",o,"){");for(var c=[],s=0;s<o;++s)c.push("(S[c["+s+"]]<<"+s+")");A.push("var M=",c.join("+"),";if(M===0||M===",(1<<o)-1,"){continue}switch(M){");for(var a=n[o-1],s=0;s<a.length;++s)A.push("case ",s,":"),l(a[s]),A.push("break;");A.push("}}")}return A.push("}return R;};return getContour",t,"d"),new Function("pool",A.join(""))(r)}(t)),e};var r=t("typedarray-pool"),i=t("marching-simplex-table"),o={}},{"marching-simplex-table":427,"typedarray-pool":545}],520:[function(t,e,n){"use strict";var r=t("bit-twiddle"),i=t("union-find");function o(t,e){var n=t.length,r=t.length-e.length,i=Math.min;if(r)return r;switch(n){case 0:return 0;case 1:return t[0]-e[0];case 2:return(s=t[0]+t[1]-e[0]-e[1])||i(t[0],t[1])-i(e[0],e[1]);case 3:var o=t[0]+t[1],a=e[0]+e[1];if(s=o+t[2]-(a+e[2]))return s;var s,A=i(t[0],t[1]),l=i(e[0],e[1]);return(s=i(A,t[2])-i(l,e[2]))||i(A+t[2],o)-i(l+e[2],a);default:var c=t.slice(0);c.sort();var u=e.slice(0);u.sort();for(var g=0;g<n;++g)if(r=c[g]-u[g])return r;return 0}}function a(t,e){return o(t[0],e[0])}function s(t,e){if(e){for(var n=t.length,r=new Array(n),i=0;i<n;++i)r[i]=[t[i],e[i]];for(r.sort(a),i=0;i<n;++i)t[i]=r[i][0],e[i]=r[i][1];return t}return t.sort(o),t}function A(t){if(0===t.length)return[];for(var e=1,n=t.length,r=1;r<n;++r){var i=t[r];if(o(i,t[r-1])){if(r===e){e++;continue}t[e++]=i}}return t.length=e,t}function l(t,e){for(var n=0,r=t.length-1,i=-1;n<=r;){var a=n+r>>1,s=o(t[a],e);s<=0?(0===s&&(i=a),n=a+1):s>0&&(r=a-1)}return i}function c(t,e){for(var n=new Array(t.length),i=0,a=n.length;i<a;++i)n[i]=[];for(var s=[],A=(i=0,e.length);i<A;++i)for(var c=e[i],u=c.length,g=1,h=1<<u;g<h;++g){s.length=r.popCount(g);for(var d=0,f=0;f<u;++f)g&1<<f&&(s[d++]=c[f]);var p=l(t,s);if(!(p<0))for(;n[p++].push(i),!(p>=t.length||0!==o(t[p],s)););}return n}function u(t,e){if(e<0)return[];for(var n=[],i=(1<<e+1)-1,o=0;o<t.length;++o)for(var a=t[o],A=i;A<1<<a.length;A=r.nextCombination(A)){for(var l=new Array(e+1),c=0,u=0;u<a.length;++u)A&1<<u&&(l[c++]=a[u]);n.push(l)}return s(n)}n.dimension=function(t){for(var e=0,n=Math.max,r=0,i=t.length;r<i;++r)e=n(e,t[r].length);return e-1},n.countVertices=function(t){for(var e=-1,n=Math.max,r=0,i=t.length;r<i;++r)for(var o=t[r],a=0,s=o.length;a<s;++a)e=n(e,o[a]);return e+1},n.cloneCells=function(t){for(var e=new Array(t.length),n=0,r=t.length;n<r;++n)e[n]=t[n].slice(0);return e},n.compareCells=o,n.normalize=s,n.unique=A,n.findCell=l,n.incidence=c,n.dual=function(t,e){if(!e)return c(A(u(t,0)),t);for(var n=new Array(e),r=0;r<e;++r)n[r]=[];r=0;for(var i=t.length;r<i;++r)for(var o=t[r],a=0,s=o.length;a<s;++a)n[o[a]].push(r);return n},n.explode=function(t){for(var e=[],n=0,r=t.length;n<r;++n)for(var i=t[n],o=0|i.length,a=1,A=1<<o;a<A;++a){for(var l=[],c=0;c<o;++c)a>>>c&1&&l.push(i[c]);e.push(l)}return s(e)},n.skeleton=u,n.boundary=function(t){for(var e=[],n=0,r=t.length;n<r;++n)for(var i=t[n],o=0,a=i.length;o<a;++o){for(var A=new Array(i.length-1),l=0,c=0;l<a;++l)l!==o&&(A[c++]=i[l]);e.push(A)}return s(e)},n.connectedComponents=function(t,e){return e?function(t,e){for(var n=new i(e),r=0;r<t.length;++r)for(var o=t[r],a=0;a<o.length;++a)for(var s=a+1;s<o.length;++s)n.link(o[a],o[s]);var A=[],l=n.ranks;for(r=0;r<l.length;++r)l[r]=-1;for(r=0;r<t.length;++r){var c=n.find(t[r][0]);l[c]<0?(l[c]=A.length,A.push([t[r].slice(0)])):A[l[c]].push(t[r].slice(0))}return A}(t,e):function(t){for(var e=A(s(u(t,0))),n=new i(e.length),r=0;r<t.length;++r)for(var o=t[r],a=0;a<o.length;++a)for(var c=l(e,[o[a]]),g=a+1;g<o.length;++g)n.link(c,l(e,[o[g]]));var h=[],d=n.ranks;for(r=0;r<d.length;++r)d[r]=-1;for(r=0;r<t.length;++r){var f=n.find(l(e,[t[r][0]]));d[f]<0?(d[f]=h.length,h.push([t[r].slice(0)])):h[d[f]].push(t[r].slice(0))}return h}(t)}},{"bit-twiddle":92,"union-find":546}],521:[function(t,e,n){arguments[4][92][0].apply(n,arguments)},{dup:92}],522:[function(t,e,n){arguments[4][520][0].apply(n,arguments)},{"bit-twiddle":521,dup:520,"union-find":523}],523:[function(t,e,n){"use strict";function r(t){this.roots=new Array(t),this.ranks=new Array(t);for(var e=0;e<t;++e)this.roots[e]=e,this.ranks[e]=0}e.exports=r,r.prototype.length=function(){return this.roots.length},r.prototype.makeSet=function(){var t=this.roots.length;return this.roots.push(t),this.ranks.push(0),t},r.prototype.find=function(t){for(var e=this.roots;e[t]!==t;){var n=e[t];e[t]=e[n],t=n}return t},r.prototype.link=function(t,e){var n=this.find(t),r=this.find(e);if(n!==r){var i=this.ranks,o=this.roots,a=i[n],s=i[r];a<s?o[n]=r:s<a?o[r]=n:(o[r]=n,++i[n])}}},{}],524:[function(t,e,n){"use strict";e.exports=function(t,e,n){for(var o=e.length,a=t.length,s=new Array(o),A=new Array(o),l=new Array(o),c=new Array(o),u=0;u<o;++u)s[u]=A[u]=-1,l[u]=1/0,c[u]=!1;for(var u=0;u<a;++u){var g=t[u];if(2!==g.length)throw new Error("Input must be a graph");var h=g[1],d=g[0];-1!==A[d]?A[d]=-2:A[d]=h,-1!==s[h]?s[h]=-2:s[h]=d}function f(t){if(c[t])return 1/0;var n,i,o,a,l,u=s[t],g=A[t];return u<0||g<0?1/0:(n=e[t],i=e[u],o=e[g],a=Math.abs(r(n,i,o)),l=Math.sqrt(Math.pow(i[0]-o[0],2)+Math.pow(i[1]-o[1],2)),a/l)}function p(t,e){var n=b[t],r=b[e];b[t]=r,b[e]=n,Q[n]=e,Q[r]=t}function I(t){return l[b[t]]}function C(t){return 1&t?t-1>>1:(t>>1)-1}function m(t){for(var e=I(t);;){var n=e,r=2*t+1,i=2*(t+1),o=t;if(r<x){var a=I(r);a<n&&(o=r,n=a)}if(i<x){var s=I(i);s<n&&(o=i)}if(o===t)return t;p(t,o),t=o}}function y(t){for(var e=I(t);t>0;){var n=C(t);if(n>=0){var r=I(n);if(e<r){p(t,n),t=n;continue}}return t}}function E(){if(x>0){var t=b[0];return p(0,x-1),x-=1,m(0),t}return-1}function v(t,e){var n=b[t];return l[n]===e?t:(l[n]=-1/0,y(t),E(),l[n]=e,y((x+=1)-1))}function B(t){if(!c[t]){c[t]=!0;var e=s[t],n=A[t];s[n]>=0&&(s[n]=e),A[e]>=0&&(A[e]=n),Q[e]>=0&&v(Q[e],f(e)),Q[n]>=0&&v(Q[n],f(n))}}for(var b=[],Q=new Array(o),u=0;u<o;++u){var w=l[u]=f(u);w<1/0?(Q[u]=b.length,b.push(u)):Q[u]=-1}for(var x=b.length,u=x>>1;u>=0;--u)m(u);for(;;){var _=E();if(_<0||l[_]>n)break;B(_)}for(var D=[],u=0;u<o;++u)c[u]||(Q[u]=D.length,D.push(e[u].slice()));function M(t,e){if(t[e]<0)return e;var n=e,r=e;do{var i=t[r];if(!c[r]||i<0||i===r)break;if(i=t[r=i],!c[r]||i<0||i===r)break;r=i,n=t[n]}while(n!==r);for(var o=e;o!==r;o=t[o])t[o]=r;return r}D.length;var k=[];return t.forEach(function(t){var e=M(s,t[0]),n=M(A,t[1]);if(e>=0&&n>=0&&e!==n){var r=Q[e],i=Q[n];r!==i&&k.push([r,i])}}),i.unique(i.normalize(k)),{positions:D,edges:k}};var r=t("robust-orientation"),i=t("simplicial-complex")},{"robust-orientation":510,"simplicial-complex":522}],525:[function(t,e,n){"use strict";e.exports=function(t,e){var n,o,a,s;if(e[0][0]<e[1][0])n=e[0],o=e[1];else{if(!(e[0][0]>e[1][0]))return i(e,t);n=e[1],o=e[0]}if(t[0][0]<t[1][0])a=t[0],s=t[1];else{if(!(t[0][0]>t[1][0]))return-i(t,e);a=t[1],s=t[0]}var A=r(n,o,s),l=r(n,o,a);if(A<0){if(l<=0)return A}else if(A>0){if(l>=0)return A}else if(l)return l;if(A=r(s,a,o),l=r(s,a,n),A<0){if(l<=0)return A}else if(A>0){if(l>=0)return A}else if(l)return l;return o[0]-s[0]};var r=t("robust-orientation");function i(t,e){var n,i,o,a;if(e[0][0]<e[1][0])n=e[0],i=e[1];else{if(!(e[0][0]>e[1][0])){var s=Math.min(t[0][1],t[1][1]),A=Math.max(t[0][1],t[1][1]),l=Math.min(e[0][1],e[1][1]),c=Math.max(e[0][1],e[1][1]);return A<l?A-l:s>c?s-c:A-c}n=e[1],i=e[0]}t[0][1]<t[1][1]?(o=t[0],a=t[1]):(o=t[1],a=t[0]);var u=r(i,n,o);return u||(u=r(i,n,a))||a-i}},{"robust-orientation":510}],526:[function(t,e,n){"use strict";e.exports=function(t){for(var e=t.length,n=2*e,r=new Array(n),o=0;o<e;++o){var A=t[o],l=A[0][0]<A[1][0];r[2*o]=new u(A[0][0],A,l,o),r[2*o+1]=new u(A[1][0],A,!l,o)}r.sort(function(t,e){var n=t.x-e.x;return n||(n=t.create-e.create)||Math.min(t.segment[0][1],t.segment[1][1])-Math.min(e.segment[0][1],e.segment[1][1])});for(var g=i(a),h=[],d=[],f=[],o=0;o<n;){for(var p=r[o].x,I=[];o<n;){var C=r[o];if(C.x!==p)break;o+=1,C.segment[0][0]===C.x&&C.segment[1][0]===C.x?C.create&&(C.segment[0][1]<C.segment[1][1]?(I.push(new c(C.segment[0][1],C.index,!0,!0)),I.push(new c(C.segment[1][1],C.index,!1,!1))):(I.push(new c(C.segment[1][1],C.index,!0,!1)),I.push(new c(C.segment[0][1],C.index,!1,!0)))):g=C.create?g.insert(C.segment,C.index):g.remove(C.segment)}h.push(g.root),d.push(p),f.push(I)}return new s(h,d,f)};var r=t("binary-search-bounds"),i=t("functional-red-black-tree"),o=t("robust-orientation"),a=t("./lib/order-segments");function s(t,e,n){this.slabs=t,this.coordinates=e,this.horizontal=n}function A(t,e){return t.y-e}function l(t,e){for(var n=null;t;){var r,i,a=t.key;a[0][0]<a[1][0]?(r=a[0],i=a[1]):(r=a[1],i=a[0]);var s=o(r,i,e);if(s<0)t=t.left;else if(s>0)if(e[0]!==a[1][0])n=t,t=t.right;else{if(A=l(t.right,e))return A;t=t.left}else{if(e[0]!==a[1][0])return t;var A;if(A=l(t.right,e))return A;t=t.left}}return n}function c(t,e,n,r){this.y=t,this.index=e,this.start=n,this.closed=r}function u(t,e,n,r){this.x=t,this.segment=e,this.create=n,this.index=r}s.prototype.castUp=function(t){var e=r.le(this.coordinates,t[0]);if(e<0)return-1;this.slabs[e];var n=l(this.slabs[e],t),i=-1;if(n&&(i=n.value),this.coordinates[e]===t[0]){var s=null;if(n&&(s=n.key),e>0){var c=l(this.slabs[e-1],t);c&&(s?a(c.key,s)>0&&(s=c.key,i=c.value):(i=c.value,s=c.key))}var u=this.horizontal[e];if(u.length>0){var g=r.ge(u,t[1],A);if(g<u.length){var h=u[g];if(t[1]===h.y){if(h.closed)return h.index;for(;g<u.length-1&&u[g+1].y===t[1];)if((h=u[g+=1]).closed)return h.index;if(h.y===t[1]&&!h.start){if((g+=1)>=u.length)return i;h=u[g]}}if(h.start)if(s){var d=o(s[0],s[1],[t[0],h.y]);s[0][0]>s[1][0]&&(d=-d),d>0&&(i=h.index)}else i=h.index;else h.y!==t[1]&&(i=h.index)}}}return i}},{"./lib/order-segments":525,"binary-search-bounds":91,"functional-red-black-tree":230,"robust-orientation":510}],527:[function(t,e,n){"use strict";var r=t("robust-dot-product"),i=t("robust-sum");function o(t,e){var n=i(r(t,e),[e[e.length-1]]);return n[n.length-1]}function a(t,e,n,r){var i=-e/(r-e);i<0?i=0:i>1&&(i=1);for(var o=1-i,a=t.length,s=new Array(a),A=0;A<a;++A)s[A]=i*t[A]+o*n[A];return s}e.exports=function(t,e){for(var n=[],r=[],i=o(t[t.length-1],e),s=t[t.length-1],A=t[0],l=0;l<t.length;++l,s=A){var c=o(A=t[l],e);if(i<0&&c>0||i>0&&c<0){var u=a(s,c,A,i);n.push(u),r.push(u.slice())}c<0?r.push(A.slice()):c>0?n.push(A.slice()):(n.push(A.slice()),r.push(A.slice())),i=c}return{positive:n,negative:r}},e.exports.positive=function(t,e){for(var n=[],r=o(t[t.length-1],e),i=t[t.length-1],s=t[0],A=0;A<t.length;++A,i=s){var l=o(s=t[A],e);(r<0&&l>0||r>0&&l<0)&&n.push(a(i,l,s,r)),l>=0&&n.push(s.slice()),r=l}return n},e.exports.negative=function(t,e){for(var n=[],r=o(t[t.length-1],e),i=t[t.length-1],s=t[0],A=0;A<t.length;++A,i=s){var l=o(s=t[A],e);(r<0&&l>0||r>0&&l<0)&&n.push(a(i,l,s,r)),l<=0&&n.push(s.slice()),r=l}return n}},{"robust-dot-product":507,"robust-sum":515}],528:[function(t,e,n){!function(){"use strict";var t={not_string:/[^s]/,not_bool:/[^t]/,not_type:/[^T]/,not_primitive:/[^v]/,number:/[diefg]/,numeric_arg:/[bcdiefguxX]/,json:/[j]/,not_json:/[^j]/,text:/^[^\x25]+/,modulo:/^\x25{2}/,placeholder:/^\x25(?:([1-9]\d*)\$|\(([^)]+)\))?(\+)?(0|'[^$])?(-)?(\d+)?(?:\.(\d+))?([b-gijostTuvxX])/,key:/^([a-z_][a-z_\d]*)/i,key_access:/^\.([a-z_][a-z_\d]*)/i,index_access:/^\[(\d+)\]/,sign:/^[+-]/};function e(n){return function(n,r){var i,o,a,s,A,l,c,u,g,h=1,d=n.length,f="";for(o=0;o<d;o++)if("string"==typeof n[o])f+=n[o];else if("object"==typeof n[o]){if((s=n[o]).keys)for(i=r[h],a=0;a<s.keys.length;a++){if(null==i)throw new Error(e('[sprintf] Cannot access property "%s" of undefined value "%s"',s.keys[a],s.keys[a-1]));i=i[s.keys[a]]}else i=s.param_no?r[s.param_no]:r[h++];if(t.not_type.test(s.type)&&t.not_primitive.test(s.type)&&i instanceof Function&&(i=i()),t.numeric_arg.test(s.type)&&"number"!=typeof i&&isNaN(i))throw new TypeError(e("[sprintf] expecting number but found %T",i));switch(t.number.test(s.type)&&(u=i>=0),s.type){case"b":i=parseInt(i,10).toString(2);break;case"c":i=String.fromCharCode(parseInt(i,10));break;case"d":case"i":i=parseInt(i,10);break;case"j":i=JSON.stringify(i,null,s.width?parseInt(s.width):0);break;case"e":i=s.precision?parseFloat(i).toExponential(s.precision):parseFloat(i).toExponential();break;case"f":i=s.precision?parseFloat(i).toFixed(s.precision):parseFloat(i);break;case"g":i=s.precision?String(Number(i.toPrecision(s.precision))):parseFloat(i);break;case"o":i=(parseInt(i,10)>>>0).toString(8);break;case"s":i=String(i),i=s.precision?i.substring(0,s.precision):i;break;case"t":i=String(!!i),i=s.precision?i.substring(0,s.precision):i;break;case"T":i=Object.prototype.toString.call(i).slice(8,-1).toLowerCase(),i=s.precision?i.substring(0,s.precision):i;break;case"u":i=parseInt(i,10)>>>0;break;case"v":i=i.valueOf(),i=s.precision?i.substring(0,s.precision):i;break;case"x":i=(parseInt(i,10)>>>0).toString(16);break;case"X":i=(parseInt(i,10)>>>0).toString(16).toUpperCase()}t.json.test(s.type)?f+=i:(!t.number.test(s.type)||u&&!s.sign?g="":(g=u?"+":"-",i=i.toString().replace(t.sign,"")),l=s.pad_char?"0"===s.pad_char?"0":s.pad_char.charAt(1):" ",c=s.width-(g+i).length,A=s.width&&c>0?l.repeat(c):"",f+=s.align?g+i+A:"0"===l?g+A+i:A+g+i)}return f}(function(e){if(i[e])return i[e];for(var n,r=e,o=[],a=0;r;){if(null!==(n=t.text.exec(r)))o.push(n[0]);else if(null!==(n=t.modulo.exec(r)))o.push("%");else{if(null===(n=t.placeholder.exec(r)))throw new SyntaxError("[sprintf] unexpected placeholder");if(n[2]){a|=1;var s=[],A=n[2],l=[];if(null===(l=t.key.exec(A)))throw new SyntaxError("[sprintf] failed to parse named argument key");for(s.push(l[1]);""!==(A=A.substring(l[0].length));)if(null!==(l=t.key_access.exec(A)))s.push(l[1]);else{if(null===(l=t.index_access.exec(A)))throw new SyntaxError("[sprintf] failed to parse named argument key");s.push(l[1])}n[2]=s}else a|=2;if(3===a)throw new Error("[sprintf] mixing positional and named placeholders is not (yet) supported");o.push({placeholder:n[0],param_no:n[1],keys:n[2],sign:n[3],pad_char:n[4],align:n[5],width:n[6],precision:n[7],type:n[8]})}r=r.substring(n[0].length)}return i[e]=o}(n),arguments)}function r(t,n){return e.apply(null,[t].concat(n||[]))}var i=Object.create(null);void 0!==n&&(n.sprintf=e,n.vsprintf=r),"undefined"!=typeof window&&(window.sprintf=e,window.vsprintf=r)}()},{}],529:[function(t,e,n){"use strict";var r=t("parenthesis");e.exports=function(t,e,n){if(null==t)throw Error("First argument should be a string");if(null==e)throw Error("Separator should be a string or a RegExp");n?("string"==typeof n||Array.isArray(n))&&(n={ignore:n}):n={},null==n.escape&&(n.escape=!0),null==n.ignore?n.ignore=["[]","()","{}","<>",'""',"''","``","“”","«»"]:("string"==typeof n.ignore&&(n.ignore=[n.ignore]),n.ignore=n.ignore.map(function(t){return 1===t.length&&(t+=t),t}));var i=r.parse(t,{flat:!0,brackets:n.ignore}),o=i[0].split(e);if(n.escape){for(var a=[],s=0;s<o.length;s++){var A=o[s],l=o[s+1];"\\"===A[A.length-1]&&"\\"!==A[A.length-2]?(a.push(A+e+l),s++):a.push(A)}o=a}for(s=0;s<o.length;s++)i[0]=o[s],o[s]=r.stringify(i,{flat:!0});return o}},{parenthesis:458}],530:[function(t,e,n){"use strict";e.exports=function(t){for(var e=t.length,n=new Array(e),r=new Array(e),i=new Array(e),o=new Array(e),a=new Array(e),s=new Array(e),A=0;A<e;++A)n[A]=-1,r[A]=0,i[A]=!1,o[A]=0,a[A]=-1,s[A]=[];var l,c=0,u=[],g=[];function h(e){var A=[e],l=[e];for(n[e]=r[e]=c,i[e]=!0,c+=1;l.length>0;){e=l[l.length-1];var h=t[e];if(o[e]<h.length){for(var d=o[e];d<h.length;++d){var f=h[d];if(n[f]<0){n[f]=r[f]=c,i[f]=!0,c+=1,A.push(f),l.push(f);break}i[f]&&(r[e]=0|Math.min(r[e],r[f])),a[f]>=0&&s[e].push(a[f])}o[e]=d}else{if(r[e]===n[e]){for(var p=[],I=[],C=0,d=A.length-1;d>=0;--d){var m=A[d];if(i[m]=!1,p.push(m),I.push(s[m]),C+=s[m].length,a[m]=u.length,m===e){A.length=d;break}}u.push(p);for(var y=new Array(C),d=0;d<I.length;d++)for(var E=0;E<I[d].length;E++)y[--C]=I[d][E];g.push(y)}l.pop()}}}for(var A=0;A<e;++A)n[A]<0&&h(A);for(var A=0;A<g.length;A++){var d=g[A];if(0!==d.length){d.sort(function(t,e){return t-e}),l=[d[0]];for(var f=1;f<d.length;f++)d[f]!==d[f-1]&&l.push(d[f]);g[A]=l}}return{components:u,adjacencyList:g}}},{}],531:[function(t,e,n){"use strict";e.exports=function(t,e){if(t.dimension<=0)return{positions:[],cells:[]};if(1===t.dimension)return function(t,e){for(var n=o(t,e),r=n.length,i=new Array(r),a=new Array(r),s=0;s<r;++s)i[s]=[n[s]],a[s]=[s];return{positions:i,cells:a}}(t,e);var n=t.order.join()+"-"+t.dtype,s=a[n],e=+e||0;return s||(s=a[n]=function(t,e){var n=t.length,o=["'use strict';"],a="surfaceNets"+t.join("_")+"d"+e;o.push("var contour=genContour({","order:[",t.join(),"],","scalarArguments: 3,","phase:function phaseFunc(p,a,b,c) { return (p > c)|0 },"),"generic"===e&&o.push("getters:[0],");for(var s=[],A=[],l=0;l<n;++l)s.push("d"+l),A.push("d"+l);for(var l=0;l<1<<n;++l)s.push("v"+l),A.push("v"+l);for(var l=0;l<1<<n;++l)s.push("p"+l),A.push("p"+l);s.push("a","b","c"),A.push("a","c"),o.push("vertex:function vertexFunc(",s.join(),"){");for(var c=[],l=0;l<1<<n;++l)c.push("(p"+l+"<<"+l+")");o.push("var m=(",c.join("+"),")|0;if(m===0||m===",(1<<(1<<n))-1,"){return}");var u=[],g=[];1<<(1<<n)<=128?(o.push("switch(m){"),g=o):o.push("switch(m>>>7){");for(var l=0;l<1<<(1<<n);++l){if(1<<(1<<n)>128&&l%128==0){u.length>0&&g.push("}}");var h="vExtra"+u.length;o.push("case ",l>>>7,":",h,"(m&0x7f,",A.join(),");break;"),g=["function ",h,"(m,",A.join(),"){switch(m){"],u.push(g)}g.push("case ",127&l,":");for(var d=new Array(n),f=new Array(n),p=new Array(n),I=new Array(n),C=0,m=0;m<n;++m)d[m]=[],f[m]=[],p[m]=0,I[m]=0;for(var m=0;m<1<<n;++m)for(var y=0;y<n;++y){var E=m^1<<y;if(!(E>m)&&!(l&1<<E)!=!(l&1<<m)){var v=1;l&1<<E?f[y].push("v"+E+"-v"+m):(f[y].push("v"+m+"-v"+E),v=-v),v<0?(d[y].push("-v"+m+"-v"+E),p[y]+=2):(d[y].push("v"+m+"+v"+E),p[y]-=2),C+=1;for(var B=0;B<n;++B)B!==y&&(E&1<<B?I[B]+=1:I[B]-=1)}}for(var b=[],y=0;y<n;++y)if(0===d[y].length)b.push("d"+y+"-0.5");else{var Q="";p[y]<0?Q=p[y]+"*c":p[y]>0&&(Q="+"+p[y]+"*c");var w=d[y].length/C*.5,x=.5+I[y]/C*.5;b.push("d"+y+"-"+x+"-"+w+"*("+d[y].join("+")+Q+")/("+f[y].join("+")+")")}g.push("a.push([",b.join(),"]);","break;")}o.push("}},"),u.length>0&&g.push("}}");for(var _=[],l=0;l<1<<n-1;++l)_.push("v"+l);_.push("c0","c1","p0","p1","a","b","c"),o.push("cell:function cellFunc(",_.join(),"){");var D=i(n-1);o.push("if(p0){b.push(",D.map(function(t){return"["+t.map(function(t){return"v"+t})+"]"}).join(),")}else{b.push(",D.map(function(t){var e=t.slice();return e.reverse(),"["+e.map(function(t){return"v"+t})+"]"}).join(),")}}});function ",a,"(array,level){var verts=[],cells=[];contour(array,verts,cells,level);return {positions:verts,cells:cells};} return ",a,";");for(var l=0;l<u.length;++l)o.push(u[l].join(""));return new Function("genContour",o.join(""))(r)}(t.order,t.dtype)),s(t,e)};var r=t("ndarray-extract-contour"),i=t("triangulate-hypercube"),o=t("zero-crossings"),a={}},{"ndarray-extract-contour":439,"triangulate-hypercube":541,"zero-crossings":574}],532:[function(t,e,n){"use strict";Object.defineProperty(n,"__esModule",{value:!0});var r=function(t,e){if(Array.isArray(t))return t;if(Symbol.iterator in Object(t))return function(t,e){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!e||n.length!==e);r=!0);}catch(t){i=!0,o=t}finally{try{!r&&s.return&&s.return()}finally{if(i)throw o}}return n}(t,e);throw new TypeError("Invalid attempt to destructure non-iterable instance")},i=2*Math.PI,o=function(t,e,n,r,i,o,a){var s=t.x,A=t.y;return{x:r*(s*=e)-i*(A*=n)+o,y:i*s+r*A+a}},a=function(t,e){var n=.551915024494*(e<0?-1:1),r=Math.cos(t),i=Math.sin(t),o=Math.cos(t+e),a=Math.sin(t+e);return[{x:r-i*n,y:i+r*n},{x:o+a*n,y:a-o*n},{x:o,y:a}]},s=function(t,e,n,r){var i=t*r-e*n<0?-1:1,o=(t*n+e*r)/(Math.sqrt(t*t+e*e)*Math.sqrt(t*t+e*e));return o>1&&(o=1),o<-1&&(o=-1),i*Math.acos(o)};n.default=function(t){var e=t.px,n=t.py,A=t.cx,l=t.cy,c=t.rx,u=t.ry,g=t.xAxisRotation,h=void 0===g?0:g,d=t.largeArcFlag,f=void 0===d?0:d,p=t.sweepFlag,I=void 0===p?0:p,C=[];if(0===c||0===u)return[];var m=Math.sin(h*i/360),y=Math.cos(h*i/360),E=y*(e-A)/2+m*(n-l)/2,v=-m*(e-A)/2+y*(n-l)/2;if(0===E&&0===v)return[];c=Math.abs(c),u=Math.abs(u);var B=Math.pow(E,2)/Math.pow(c,2)+Math.pow(v,2)/Math.pow(u,2);B>1&&(c*=Math.sqrt(B),u*=Math.sqrt(B));var b=function(t,e,n,r,o,a,A,l,c,u,g,h){var d=Math.pow(o,2),f=Math.pow(a,2),p=Math.pow(g,2),I=Math.pow(h,2),C=d*f-d*I-f*p;C<0&&(C=0),C/=d*I+f*p;var m=(C=Math.sqrt(C)*(A===l?-1:1))*o/a*h,y=C*-a/o*g,E=u*m-c*y+(t+n)/2,v=c*m+u*y+(e+r)/2,B=(g-m)/o,b=(h-y)/a,Q=(-g-m)/o,w=(-h-y)/a,x=s(1,0,B,b),_=s(B,b,Q,w);return 0===l&&_>0&&(_-=i),1===l&&_<0&&(_+=i),[E,v,x,_]}(e,n,A,l,c,u,f,I,m,y,E,v),Q=r(b,4),w=Q[0],x=Q[1],_=Q[2],D=Q[3],M=Math.abs(D)/(i/4);Math.abs(1-M)<1e-7&&(M=1);var k=Math.max(Math.ceil(M),1);D/=k;for(var S=0;S<k;S++)C.push(a(_,D)),_+=D;return C.map(function(t){var e=o(t[0],c,u,y,m,w,x),n=e.x,r=e.y,i=o(t[1],c,u,y,m,w,x),a=i.x,s=i.y,A=o(t[2],c,u,y,m,w,x);return{x1:n,y1:r,x2:a,y2:s,x:A.x,y:A.y}})},e.exports=n.default},{}],533:[function(t,e,n){"use strict";var r=t("parse-svg-path"),i=t("abs-svg-path"),o=t("normalize-svg-path"),a=t("is-svg-path"),s=t("assert");e.exports=function(t){if(Array.isArray(t)&&1===t.length&&"string"==typeof t[0]&&(t=t[0]),"string"==typeof t&&(s(a(t),"String is not an SVG path."),t=r(t)),s(Array.isArray(t),"Argument should be a string or an array of path segments."),t=i(t),!(t=o(t)).length)return[0,0,0,0];for(var e=[1/0,1/0,-1/0,-1/0],n=0,A=t.length;n<A;n++)for(var l=t[n].slice(1),c=0;c<l.length;c+=2)l[c+0]<e[0]&&(e[0]=l[c+0]),l[c+1]<e[1]&&(e[1]=l[c+1]),l[c+0]>e[2]&&(e[2]=l[c+0]),l[c+1]>e[3]&&(e[3]=l[c+1]);return e}},{"abs-svg-path":60,assert:68,"is-svg-path":424,"normalize-svg-path":534,"parse-svg-path":460}],534:[function(t,e,n){"use strict";e.exports=function(t){for(var e,n=[],a=0,s=0,A=0,l=0,c=null,u=null,g=0,h=0,d=0,f=t.length;d<f;d++){var p=t[d],I=p[0];switch(I){case"M":A=p[1],l=p[2];break;case"A":var C=r({px:g,py:h,cx:p[6],cy:p[7],rx:p[1],ry:p[2],xAxisRotation:p[3],largeArcFlag:p[4],sweepFlag:p[5]});if(!C.length)continue;for(var m,y=0;y<C.length;y++)m=C[y],p=["C",m.x1,m.y1,m.x2,m.y2,m.x,m.y],y<C.length-1&&n.push(p);break;case"S":var E=g,v=h;"C"!=e&&"S"!=e||(E+=E-a,v+=v-s),p=["C",E,v,p[1],p[2],p[3],p[4]];break;case"T":"Q"==e||"T"==e?(c=2*g-c,u=2*h-u):(c=g,u=h),p=o(g,h,c,u,p[1],p[2]);break;case"Q":c=p[1],u=p[2],p=o(g,h,p[1],p[2],p[3],p[4]);break;case"L":p=i(g,h,p[1],p[2]);break;case"H":p=i(g,h,p[1],h);break;case"V":p=i(g,h,g,p[1]);break;case"Z":p=i(g,h,A,l)}e=I,g=p[p.length-2],h=p[p.length-1],p.length>4?(a=p[p.length-4],s=p[p.length-3]):(a=g,s=h),n.push(p)}return n};var r=t("svg-arc-to-cubic-bezier");function i(t,e,n,r){return["C",t,e,n,r,n,r]}function o(t,e,n,r,i,o){return["C",t/3+2/3*n,e/3+2/3*r,i/3+2/3*n,o/3+2/3*r,i,o]}},{"svg-arc-to-cubic-bezier":532}],535:[function(t,e,n){"use strict";var r,i=t("svg-path-bounds"),o=t("parse-svg-path"),a=t("draw-svg-path"),s=t("is-svg-path"),A=t("bitmap-sdf"),l=document.createElement("canvas"),c=l.getContext("2d");e.exports=function(t,e){if(!s(t))throw Error("Argument should be valid svg path string");var n,u;e||(e={}),e.shape?(n=e.shape[0],u=e.shape[1]):(n=l.width=e.w||e.width||200,u=l.height=e.h||e.height||200);var g=Math.min(n,u),h=e.stroke||0,d=e.viewbox||e.viewBox||i(t),f=[n/(d[2]-d[0]),u/(d[3]-d[1])],p=Math.min(f[0]||0,f[1]||0)/2;if(c.fillStyle="black",c.fillRect(0,0,n,u),c.fillStyle="white",h&&("number"!=typeof h&&(h=1),c.strokeStyle=h>0?"white":"black",c.lineWidth=Math.abs(h)),c.translate(.5*n,.5*u),c.scale(p,p),function(){if(null!=r)return r;var t=document.createElement("canvas").getContext("2d");if(t.canvas.width=t.canvas.height=1,!window.Path2D)return r=!1;var e=new Path2D("M0,0h1v1h-1v-1Z");t.fillStyle="black",t.fill(e);var n=t.getImageData(0,0,1,1);return r=n&&n.data&&255===n.data[3]}()){var I=new Path2D(t);c.fill(I),h&&c.stroke(I)}else{var C=o(t);a(c,C),c.fill(),h&&c.stroke()}return c.setTransform(1,0,0,1,0,0),A(c,{cutoff:null!=e.cutoff?e.cutoff:.5,radius:null!=e.radius?e.radius:.5*g})}},{"bitmap-sdf":93,"draw-svg-path":168,"is-svg-path":424,"parse-svg-path":460,"svg-path-bounds":533}],536:[function(t,e,n){(function(n){"use strict";e.exports=function t(e,n,i){var i=i||{},a=o[e];a||(a=o[e]={" ":{data:new Float32Array(0),shape:.2}});var s=a[n];if(!s)if(n.length<=1||!/\d/.test(n))s=a[n]=function(t){for(var e=t.cells,n=t.positions,r=new Float32Array(6*e.length),i=0,o=0,a=0;a<e.length;++a)for(var s=e[a],A=0;A<3;++A){var l=n[s[A]];r[i++]=l[0],r[i++]=l[1]+1.4,o=Math.max(l[0],o)}return{data:r,shape:o}}(r(n,{triangles:!0,font:e,textAlign:i.textAlign||"left",textBaseline:"alphabetic",styletags:{breaklines:!0,bolds:!0,italics:!0,subscripts:!0,superscripts:!0}}));else{for(var A=n.split(/(\d|\s)/),l=new Array(A.length),c=0,u=0,g=0;g<A.length;++g)l[g]=t(e,A[g]),c+=l[g].data.length,u+=l[g].shape,g>0&&(u+=.02);for(var h=new Float32Array(c),d=0,f=-.5*u,g=0;g<l.length;++g){for(var p=l[g].data,I=0;I<p.length;I+=2)h[d++]=p[I]+f,h[d++]=p[I+1];f+=l[g].shape+.02}s=a[n]={data:h,shape:u}}return s};var r=t("vectorize-text"),i=window||n.global||{},o=i.__TEXT_CACHE||{};i.__TEXT_CACHE={}}).call(this,t("_process"))},{_process:482,"vectorize-text":550}],537:[function(t,e,n){!function(t){var n=/^\s+/,r=/\s+$/,i=0,o=t.round,a=t.min,s=t.max,A=t.random;function l(e,A){if(A=A||{},(e=e||"")instanceof l)return e;if(!(this instanceof l))return new l(e,A);var c=function(e){var i,o,A,l={r:0,g:0,b:0},c=1,u=null,g=null,h=null,d=!1,f=!1;return"string"==typeof e&&(e=function(t){t=t.replace(n,"").replace(r,"").toLowerCase();var e,i=!1;if(x[t])t=x[t],i=!0;else if("transparent"==t)return{r:0,g:0,b:0,a:0,format:"name"};return(e=G.rgb.exec(t))?{r:e[1],g:e[2],b:e[3]}:(e=G.rgba.exec(t))?{r:e[1],g:e[2],b:e[3],a:e[4]}:(e=G.hsl.exec(t))?{h:e[1],s:e[2],l:e[3]}:(e=G.hsla.exec(t))?{h:e[1],s:e[2],l:e[3],a:e[4]}:(e=G.hsv.exec(t))?{h:e[1],s:e[2],v:e[3]}:(e=G.hsva.exec(t))?{h:e[1],s:e[2],v:e[3],a:e[4]}:(e=G.hex8.exec(t))?{r:S(e[1]),g:S(e[2]),b:S(e[3]),a:F(e[4]),format:i?"name":"hex8"}:(e=G.hex6.exec(t))?{r:S(e[1]),g:S(e[2]),b:S(e[3]),format:i?"name":"hex"}:(e=G.hex4.exec(t))?{r:S(e[1]+""+e[1]),g:S(e[2]+""+e[2]),b:S(e[3]+""+e[3]),a:F(e[4]+""+e[4]),format:i?"name":"hex8"}:!!(e=G.hex3.exec(t))&&{r:S(e[1]+""+e[1]),g:S(e[2]+""+e[2]),b:S(e[3]+""+e[3]),format:i?"name":"hex"}}(e)),"object"==typeof e&&(H(e.r)&&H(e.g)&&H(e.b)?(i=e.r,o=e.g,A=e.b,l={r:255*M(i,255),g:255*M(o,255),b:255*M(A,255)},d=!0,f="%"===String(e.r).substr(-1)?"prgb":"rgb"):H(e.h)&&H(e.s)&&H(e.v)?(u=R(e.s),g=R(e.v),l=function(e,n,r){e=6*M(e,360),n=M(n,100),r=M(r,100);var i=t.floor(e),o=e-i,a=r*(1-n),s=r*(1-o*n),A=r*(1-(1-o)*n),l=i%6;return{r:255*[r,s,a,a,A,r][l],g:255*[A,r,r,s,a,a][l],b:255*[a,a,A,r,r,s][l]}}(e.h,u,g),d=!0,f="hsv"):H(e.h)&&H(e.s)&&H(e.l)&&(u=R(e.s),h=R(e.l),l=function(t,e,n){var r,i,o;function a(t,e,n){return n<0&&(n+=1),n>1&&(n-=1),n<1/6?t+6*(e-t)*n:n<.5?e:n<2/3?t+(e-t)*(2/3-n)*6:t}if(t=M(t,360),e=M(e,100),n=M(n,100),0===e)r=i=o=n;else{var s=n<.5?n*(1+e):n+e-n*e,A=2*n-s;r=a(A,s,t+1/3),i=a(A,s,t),o=a(A,s,t-1/3)}return{r:255*r,g:255*i,b:255*o}}(e.h,u,h),d=!0,f="hsl"),e.hasOwnProperty("a")&&(c=e.a)),c=D(c),{ok:d,format:e.format||f,r:a(255,s(l.r,0)),g:a(255,s(l.g,0)),b:a(255,s(l.b,0)),a:c}}(e);this._originalInput=e,this._r=c.r,this._g=c.g,this._b=c.b,this._a=c.a,this._roundA=o(100*this._a)/100,this._format=A.format||c.format,this._gradientType=A.gradientType,this._r<1&&(this._r=o(this._r)),this._g<1&&(this._g=o(this._g)),this._b<1&&(this._b=o(this._b)),this._ok=c.ok,this._tc_id=i++}function c(t,e,n){t=M(t,255),e=M(e,255),n=M(n,255);var r,i,o=s(t,e,n),A=a(t,e,n),l=(o+A)/2;if(o==A)r=i=0;else{var c=o-A;switch(i=l>.5?c/(2-o-A):c/(o+A),o){case t:r=(e-n)/c+(e<n?6:0);break;case e:r=(n-t)/c+2;break;case n:r=(t-e)/c+4}r/=6}return{h:r,s:i,l:l}}function u(t,e,n){t=M(t,255),e=M(e,255),n=M(n,255);var r,i,o=s(t,e,n),A=a(t,e,n),l=o,c=o-A;if(i=0===o?0:c/o,o==A)r=0;else{switch(o){case t:r=(e-n)/c+(e<n?6:0);break;case e:r=(n-t)/c+2;break;case n:r=(t-e)/c+4}r/=6}return{h:r,s:i,v:l}}function g(t,e,n,r){var i=[L(o(t).toString(16)),L(o(e).toString(16)),L(o(n).toString(16))];return r&&i[0].charAt(0)==i[0].charAt(1)&&i[1].charAt(0)==i[1].charAt(1)&&i[2].charAt(0)==i[2].charAt(1)?i[0].charAt(0)+i[1].charAt(0)+i[2].charAt(0):i.join("")}function h(t,e,n,r){return[L(N(r)),L(o(t).toString(16)),L(o(e).toString(16)),L(o(n).toString(16))].join("")}function d(t,e){e=0===e?0:e||10;var n=l(t).toHsl();return n.s-=e/100,n.s=k(n.s),l(n)}function f(t,e){e=0===e?0:e||10;var n=l(t).toHsl();return n.s+=e/100,n.s=k(n.s),l(n)}function p(t){return l(t).desaturate(100)}function I(t,e){e=0===e?0:e||10;var n=l(t).toHsl();return n.l+=e/100,n.l=k(n.l),l(n)}function C(t,e){e=0===e?0:e||10;var n=l(t).toRgb();return n.r=s(0,a(255,n.r-o(-e/100*255))),n.g=s(0,a(255,n.g-o(-e/100*255))),n.b=s(0,a(255,n.b-o(-e/100*255))),l(n)}function m(t,e){e=0===e?0:e||10;var n=l(t).toHsl();return n.l-=e/100,n.l=k(n.l),l(n)}function y(t,e){var n=l(t).toHsl(),r=(n.h+e)%360;return n.h=r<0?360+r:r,l(n)}function E(t){var e=l(t).toHsl();return e.h=(e.h+180)%360,l(e)}function v(t){var e=l(t).toHsl(),n=e.h;return[l(t),l({h:(n+120)%360,s:e.s,l:e.l}),l({h:(n+240)%360,s:e.s,l:e.l})]}function B(t){var e=l(t).toHsl(),n=e.h;return[l(t),l({h:(n+90)%360,s:e.s,l:e.l}),l({h:(n+180)%360,s:e.s,l:e.l}),l({h:(n+270)%360,s:e.s,l:e.l})]}function b(t){var e=l(t).toHsl(),n=e.h;return[l(t),l({h:(n+72)%360,s:e.s,l:e.l}),l({h:(n+216)%360,s:e.s,l:e.l})]}function Q(t,e,n){e=e||6,n=n||30;var r=l(t).toHsl(),i=360/n,o=[l(t)];for(r.h=(r.h-(i*e>>1)+720)%360;--e;)r.h=(r.h+i)%360,o.push(l(r));return o}function w(t,e){e=e||6;for(var n=l(t).toHsv(),r=n.h,i=n.s,o=n.v,a=[],s=1/e;e--;)a.push(l({h:r,s:i,v:o})),o=(o+s)%1;return a}l.prototype={isDark:function(){return this.getBrightness()<128},isLight:function(){return!this.isDark()},isValid:function(){return this._ok},getOriginalInput:function(){return this._originalInput},getFormat:function(){return this._format},getAlpha:function(){return this._a},getBrightness:function(){var t=this.toRgb();return(299*t.r+587*t.g+114*t.b)/1e3},getLuminance:function(){var e,n,r,i=this.toRgb();return e=i.r/255,n=i.g/255,r=i.b/255,.2126*(e<=.03928?e/12.92:t.pow((e+.055)/1.055,2.4))+.7152*(n<=.03928?n/12.92:t.pow((n+.055)/1.055,2.4))+.0722*(r<=.03928?r/12.92:t.pow((r+.055)/1.055,2.4))},setAlpha:function(t){return this._a=D(t),this._roundA=o(100*this._a)/100,this},toHsv:function(){var t=u(this._r,this._g,this._b);return{h:360*t.h,s:t.s,v:t.v,a:this._a}},toHsvString:function(){var t=u(this._r,this._g,this._b),e=o(360*t.h),n=o(100*t.s),r=o(100*t.v);return 1==this._a?"hsv("+e+", "+n+"%, "+r+"%)":"hsva("+e+", "+n+"%, "+r+"%, "+this._roundA+")"},toHsl:function(){var t=c(this._r,this._g,this._b);return{h:360*t.h,s:t.s,l:t.l,a:this._a}},toHslString:function(){var t=c(this._r,this._g,this._b),e=o(360*t.h),n=o(100*t.s),r=o(100*t.l);return 1==this._a?"hsl("+e+", "+n+"%, "+r+"%)":"hsla("+e+", "+n+"%, "+r+"%, "+this._roundA+")"},toHex:function(t){return g(this._r,this._g,this._b,t)},toHexString:function(t){return"#"+this.toHex(t)},toHex8:function(t){return function(t,e,n,r,i){var a=[L(o(t).toString(16)),L(o(e).toString(16)),L(o(n).toString(16)),L(N(r))];return i&&a[0].charAt(0)==a[0].charAt(1)&&a[1].charAt(0)==a[1].charAt(1)&&a[2].charAt(0)==a[2].charAt(1)&&a[3].charAt(0)==a[3].charAt(1)?a[0].charAt(0)+a[1].charAt(0)+a[2].charAt(0)+a[3].charAt(0):a.join("")}(this._r,this._g,this._b,this._a,t)},toHex8String:function(t){return"#"+this.toHex8(t)},toRgb:function(){return{r:o(this._r),g:o(this._g),b:o(this._b),a:this._a}},toRgbString:function(){return 1==this._a?"rgb("+o(this._r)+", "+o(this._g)+", "+o(this._b)+")":"rgba("+o(this._r)+", "+o(this._g)+", "+o(this._b)+", "+this._roundA+")"},toPercentageRgb:function(){return{r:o(100*M(this._r,255))+"%",g:o(100*M(this._g,255))+"%",b:o(100*M(this._b,255))+"%",a:this._a}},toPercentageRgbString:function(){return 1==this._a?"rgb("+o(100*M(this._r,255))+"%, "+o(100*M(this._g,255))+"%, "+o(100*M(this._b,255))+"%)":"rgba("+o(100*M(this._r,255))+"%, "+o(100*M(this._g,255))+"%, "+o(100*M(this._b,255))+"%, "+this._roundA+")"},toName:function(){return 0===this._a?"transparent":!(this._a<1)&&(_[g(this._r,this._g,this._b,!0)]||!1)},toFilter:function(t){var e="#"+h(this._r,this._g,this._b,this._a),n=e,r=this._gradientType?"GradientType = 1, ":"";if(t){var i=l(t);n="#"+h(i._r,i._g,i._b,i._a)}return"progid:DXImageTransform.Microsoft.gradient("+r+"startColorstr="+e+",endColorstr="+n+")"},toString:function(t){var e=!!t;t=t||this._format;var n=!1,r=this._a<1&&this._a>=0;return e||!r||"hex"!==t&&"hex6"!==t&&"hex3"!==t&&"hex4"!==t&&"hex8"!==t&&"name"!==t?("rgb"===t&&(n=this.toRgbString()),"prgb"===t&&(n=this.toPercentageRgbString()),"hex"!==t&&"hex6"!==t||(n=this.toHexString()),"hex3"===t&&(n=this.toHexString(!0)),"hex4"===t&&(n=this.toHex8String(!0)),"hex8"===t&&(n=this.toHex8String()),"name"===t&&(n=this.toName()),"hsl"===t&&(n=this.toHslString()),"hsv"===t&&(n=this.toHsvString()),n||this.toHexString()):"name"===t&&0===this._a?this.toName():this.toRgbString()},clone:function(){return l(this.toString())},_applyModification:function(t,e){var n=t.apply(null,[this].concat([].slice.call(e)));return this._r=n._r,this._g=n._g,this._b=n._b,this.setAlpha(n._a),this},lighten:function(){return this._applyModification(I,arguments)},brighten:function(){return this._applyModification(C,arguments)},darken:function(){return this._applyModification(m,arguments)},desaturate:function(){return this._applyModification(d,arguments)},saturate:function(){return this._applyModification(f,arguments)},greyscale:function(){return this._applyModification(p,arguments)},spin:function(){return this._applyModification(y,arguments)},_applyCombination:function(t,e){return t.apply(null,[this].concat([].slice.call(e)))},analogous:function(){return this._applyCombination(Q,arguments)},complement:function(){return this._applyCombination(E,arguments)},monochromatic:function(){return this._applyCombination(w,arguments)},splitcomplement:function(){return this._applyCombination(b,arguments)},triad:function(){return this._applyCombination(v,arguments)},tetrad:function(){return this._applyCombination(B,arguments)}},l.fromRatio=function(t,e){if("object"==typeof t){var n={};for(var r in t)t.hasOwnProperty(r)&&(n[r]="a"===r?t[r]:R(t[r]));t=n}return l(t,e)},l.equals=function(t,e){return!(!t||!e)&&l(t).toRgbString()==l(e).toRgbString()},l.random=function(){return l.fromRatio({r:A(),g:A(),b:A()})},l.mix=function(t,e,n){n=0===n?0:n||50;var r=l(t).toRgb(),i=l(e).toRgb(),o=n/100;return l({r:(i.r-r.r)*o+r.r,g:(i.g-r.g)*o+r.g,b:(i.b-r.b)*o+r.b,a:(i.a-r.a)*o+r.a})},l.readability=function(e,n){var r=l(e),i=l(n);return(t.max(r.getLuminance(),i.getLuminance())+.05)/(t.min(r.getLuminance(),i.getLuminance())+.05)},l.isReadable=function(t,e,n){var r,i,o=l.readability(t,e);switch(i=!1,(r=function(t){var e,n;return e=((t=t||{level:"AA",size:"small"}).level||"AA").toUpperCase(),n=(t.size||"small").toLowerCase(),"AA"!==e&&"AAA"!==e&&(e="AA"),"small"!==n&&"large"!==n&&(n="small"),{level:e,size:n}}(n)).level+r.size){case"AAsmall":case"AAAlarge":i=o>=4.5;break;case"AAlarge":i=o>=3;break;case"AAAsmall":i=o>=7}return i},l.mostReadable=function(t,e,n){var r,i,o,a,s=null,A=0;i=(n=n||{}).includeFallbackColors,o=n.level,a=n.size;for(var c=0;c<e.length;c++)(r=l.readability(t,e[c]))>A&&(A=r,s=l(e[c]));return l.isReadable(t,s,{level:o,size:a})||!i?s:(n.includeFallbackColors=!1,l.mostReadable(t,["#fff","#000"],n))};var x=l.names={aliceblue:"f0f8ff",antiquewhite:"faebd7",aqua:"0ff",aquamarine:"7fffd4",azure:"f0ffff",beige:"f5f5dc",bisque:"ffe4c4",black:"000",blanchedalmond:"ffebcd",blue:"00f",blueviolet:"8a2be2",brown:"a52a2a",burlywood:"deb887",burntsienna:"ea7e5d",cadetblue:"5f9ea0",chartreuse:"7fff00",chocolate:"d2691e",coral:"ff7f50",cornflowerblue:"6495ed",cornsilk:"fff8dc",crimson:"dc143c",cyan:"0ff",darkblue:"00008b",darkcyan:"008b8b",darkgoldenrod:"b8860b",darkgray:"a9a9a9",darkgreen:"006400",darkgrey:"a9a9a9",darkkhaki:"bdb76b",darkmagenta:"8b008b",darkolivegreen:"556b2f",darkorange:"ff8c00",darkorchid:"9932cc",darkred:"8b0000",darksalmon:"e9967a",darkseagreen:"8fbc8f",darkslateblue:"483d8b",darkslategray:"2f4f4f",darkslategrey:"2f4f4f",darkturquoise:"00ced1",darkviolet:"9400d3",deeppink:"ff1493",deepskyblue:"00bfff",dimgray:"696969",dimgrey:"696969",dodgerblue:"1e90ff",firebrick:"b22222",floralwhite:"fffaf0",forestgreen:"228b22",fuchsia:"f0f",gainsboro:"dcdcdc",ghostwhite:"f8f8ff",gold:"ffd700",goldenrod:"daa520",gray:"808080",green:"008000",greenyellow:"adff2f",grey:"808080",honeydew:"f0fff0",hotpink:"ff69b4",indianred:"cd5c5c",indigo:"4b0082",ivory:"fffff0",khaki:"f0e68c",lavender:"e6e6fa",lavenderblush:"fff0f5",lawngreen:"7cfc00",lemonchiffon:"fffacd",lightblue:"add8e6",lightcoral:"f08080",lightcyan:"e0ffff",lightgoldenrodyellow:"fafad2",lightgray:"d3d3d3",lightgreen:"90ee90",lightgrey:"d3d3d3",lightpink:"ffb6c1",lightsalmon:"ffa07a",lightseagreen:"20b2aa",lightskyblue:"87cefa",lightslategray:"789",lightslategrey:"789",lightsteelblue:"b0c4de",lightyellow:"ffffe0",lime:"0f0",limegreen:"32cd32",linen:"faf0e6",magenta:"f0f",maroon:"800000",mediumaquamarine:"66cdaa",mediumblue:"0000cd",mediumorchid:"ba55d3",mediumpurple:"9370db",mediumseagreen:"3cb371",mediumslateblue:"7b68ee",mediumspringgreen:"00fa9a",mediumturquoise:"48d1cc",mediumvioletred:"c71585",midnightblue:"191970",mintcream:"f5fffa",mistyrose:"ffe4e1",moccasin:"ffe4b5",navajowhite:"ffdead",navy:"000080",oldlace:"fdf5e6",olive:"808000",olivedrab:"6b8e23",orange:"ffa500",orangered:"ff4500",orchid:"da70d6",palegoldenrod:"eee8aa",palegreen:"98fb98",paleturquoise:"afeeee",palevioletred:"db7093",papayawhip:"ffefd5",peachpuff:"ffdab9",peru:"cd853f",pink:"ffc0cb",plum:"dda0dd",powderblue:"b0e0e6",purple:"800080",rebeccapurple:"663399",red:"f00",rosybrown:"bc8f8f",royalblue:"4169e1",saddlebrown:"8b4513",salmon:"fa8072",sandybrown:"f4a460",seagreen:"2e8b57",seashell:"fff5ee",sienna:"a0522d",silver:"c0c0c0",skyblue:"87ceeb",slateblue:"6a5acd",slategray:"708090",slategrey:"708090",snow:"fffafa",springgreen:"00ff7f",steelblue:"4682b4",tan:"d2b48c",teal:"008080",thistle:"d8bfd8",tomato:"ff6347",turquoise:"40e0d0",violet:"ee82ee",wheat:"f5deb3",white:"fff",whitesmoke:"f5f5f5",yellow:"ff0",yellowgreen:"9acd32"},_=l.hexNames=function(t){var e={};for(var n in t)t.hasOwnProperty(n)&&(e[t[n]]=n);return e}(x);function D(t){return t=parseFloat(t),(isNaN(t)||t<0||t>1)&&(t=1),t}function M(e,n){(function(t){return"string"==typeof t&&-1!=t.indexOf(".")&&1===parseFloat(t)})(e)&&(e="100%");var r=function(t){return"string"==typeof t&&-1!=t.indexOf("%")}(e);return e=a(n,s(0,parseFloat(e))),r&&(e=parseInt(e*n,10)/100),t.abs(e-n)<1e-6?1:e%n/parseFloat(n)}function k(t){return a(1,s(0,t))}function S(t){return parseInt(t,16)}function L(t){return 1==t.length?"0"+t:""+t}function R(t){return t<=1&&(t=100*t+"%"),t}function N(e){return t.round(255*parseFloat(e)).toString(16)}function F(t){return S(t)/255}var T,U,O,G=(U="[\\s|\\(]+("+(T="(?:[-\\+]?\\d*\\.\\d+%?)|(?:[-\\+]?\\d+%?)")+")[,|\\s]+("+T+")[,|\\s]+("+T+")\\s*\\)?",O="[\\s|\\(]+("+T+")[,|\\s]+("+T+")[,|\\s]+("+T+")[,|\\s]+("+T+")\\s*\\)?",{CSS_UNIT:new RegExp(T),rgb:new RegExp("rgb"+U),rgba:new RegExp("rgba"+O),hsl:new RegExp("hsl"+U),hsla:new RegExp("hsla"+O),hsv:new RegExp("hsv"+U),hsva:new RegExp("hsva"+O),hex3:/^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex6:/^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/,hex4:/^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex8:/^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/});function H(t){return!!G.CSS_UNIT.exec(t)}void 0!==e&&e.exports?e.exports=l:window.tinycolor=l}(Math)},{}],538:[function(t,e,n){"use strict";e.exports=i,e.exports.float32=e.exports.float=i,e.exports.fract32=e.exports.fract=function(t){if(t.length){for(var e=i(t),n=0,r=e.length;n<r;n++)e[n]=t[n]-e[n];return e}return i(t-i(t))};var r=new Float32Array(1);function i(t){if(t.length){if(t instanceof Float32Array)return t;var e=new Float32Array(t);return e.set(t),e}return r[0]=t,r[0]}},{}],539:[function(t,e,n){"use strict";var r=t("parse-unit");e.exports=a;var i=96;function o(t,e){var n=r(getComputedStyle(t).getPropertyValue(e));return n[0]*a(n[1],t)}function a(t,e){switch(e=e||document.body,t=(t||"px").trim().toLowerCase(),e!==window&&e!==document||(e=document.body),t){case"%":return e.clientHeight/100;case"ch":case"ex":return function(t,e){var n=document.createElement("div");n.style["font-size"]="128"+t,e.appendChild(n);var r=o(n,"font-size")/128;return e.removeChild(n),r}(t,e);case"em":return o(e,"font-size");case"rem":return o(document.body,"font-size");case"vw":return window.innerWidth/100;case"vh":return window.innerHeight/100;case"vmin":return Math.min(window.innerWidth,window.innerHeight)/100;case"vmax":return Math.max(window.innerWidth,window.innerHeight)/100;case"in":return i;case"cm":return i/2.54;case"mm":return i/25.4;case"pt":return i/72;case"pc":return i/6}return 1}},{"parse-unit":461}],540:[function(t,e,n){!function(t){"use strict";var e=function(t){return t},n=function(t){if(null==(n=t.transform))return e;var n,r,i,o=n.scale[0],a=n.scale[1],s=n.translate[0],A=n.translate[1];return function(t,e){return e||(r=i=0),t[0]=(r+=t[0])*o+s,t[1]=(i+=t[1])*a+A,t}},r=function(t){var e=t.bbox;function r(t){A[0]=t[0],A[1]=t[1],s(A),A[0]<l&&(l=A[0]),A[0]>u&&(u=A[0]),A[1]<c&&(c=A[1]),A[1]>g&&(g=A[1])}function i(t){switch(t.type){case"GeometryCollection":t.geometries.forEach(i);break;case"Point":r(t.coordinates);break;case"MultiPoint":t.coordinates.forEach(r)}}if(!e){var o,a,s=n(t),A=new Array(2),l=1/0,c=l,u=-l,g=-l;for(a in t.arcs.forEach(function(t){for(var e=-1,n=t.length;++e<n;)o=t[e],A[0]=o[0],A[1]=o[1],s(A,e),A[0]<l&&(l=A[0]),A[0]>u&&(u=A[0]),A[1]<c&&(c=A[1]),A[1]>g&&(g=A[1])}),t.objects)i(t.objects[a]);e=t.bbox=[l,c,u,g]}return e},i=function(t,e){for(var n,r=t.length,i=r-e;i<--r;)n=t[i],t[i++]=t[r],t[r]=n};function o(t,e){var n=e.id,r=e.bbox,i=null==e.properties?{}:e.properties,o=a(t,e);return null==n&&null==r?{type:"Feature",properties:i,geometry:o}:null==r?{type:"Feature",id:n,properties:i,geometry:o}:{type:"Feature",id:n,bbox:r,properties:i,geometry:o}}function a(t,e){var r=n(t),o=t.arcs;function a(t,e){e.length&&e.pop();for(var n=o[t<0?~t:t],a=0,s=n.length;a<s;++a)e.push(r(n[a].slice(),a));t<0&&i(e,s)}function s(t){return r(t.slice())}function A(t){for(var e=[],n=0,r=t.length;n<r;++n)a(t[n],e);return e.length<2&&e.push(e[0].slice()),e}function l(t){for(var e=A(t);e.length<4;)e.push(e[0].slice());return e}function c(t){return t.map(l)}return function t(e){var n,r=e.type;switch(r){case"GeometryCollection":return{type:r,geometries:e.geometries.map(t)};case"Point":n=s(e.coordinates);break;case"MultiPoint":n=e.coordinates.map(s);break;case"LineString":n=A(e.arcs);break;case"MultiLineString":n=e.arcs.map(A);break;case"Polygon":n=c(e.arcs);break;case"MultiPolygon":n=e.arcs.map(c);break;default:return null}return{type:r,coordinates:n}}(e)}var s=function(t,e){var n={},r={},i={},o=[],a=-1;function s(t,e){for(var r in t){var i=t[r];delete e[i.start],delete i.start,delete i.end,i.forEach(function(t){n[t<0?~t:t]=1}),o.push(i)}}return e.forEach(function(n,r){var i,o=t.arcs[n<0?~n:n];o.length<3&&!o[1][0]&&!o[1][1]&&(i=e[++a],e[a]=n,e[r]=i)}),e.forEach(function(e){var n,o,a=function(e){var n,r=t.arcs[e<0?~e:e],i=r[0];return t.transform?(n=[0,0],r.forEach(function(t){n[0]+=t[0],n[1]+=t[1]})):n=r[r.length-1],e<0?[n,i]:[i,n]}(e),s=a[0],A=a[1];if(n=i[s])if(delete i[n.end],n.push(e),n.end=A,o=r[A]){delete r[o.start];var l=o===n?n:n.concat(o);r[l.start=n.start]=i[l.end=o.end]=l}else r[n.start]=i[n.end]=n;else if(n=r[A])if(delete r[n.start],n.unshift(e),n.start=s,o=i[s]){delete i[o.end];var c=o===n?n:o.concat(n);r[c.start=o.start]=i[c.end=n.end]=c}else r[n.start]=i[n.end]=n;else r[(n=[e]).start=s]=i[n.end=A]=n}),s(i,r),s(r,i),e.forEach(function(t){n[t<0?~t:t]||o.push([t])}),o};function A(t,e,n){var r,i,o;if(arguments.length>1)r=function(t,e,n){var r,i=[],o=[];function a(t){var e=t<0?~t:t;(o[e]||(o[e]=[])).push({i:t,g:r})}function s(t){t.forEach(a)}function A(t){t.forEach(s)}return function t(e){switch(r=e,e.type){case"GeometryCollection":e.geometries.forEach(t);break;case"LineString":s(e.arcs);break;case"MultiLineString":case"Polygon":A(e.arcs);break;case"MultiPolygon":e.arcs.forEach(A)}}(e),o.forEach(null==n?function(t){i.push(t[0].i)}:function(t){n(t[0].g,t[t.length-1].g)&&i.push(t[0].i)}),i}(0,e,n);else for(i=0,r=new Array(o=t.arcs.length);i<o;++i)r[i]=i;return{type:"MultiLineString",arcs:s(t,r)}}function l(t,e){var n={},r=[],i=[];function o(t){t.forEach(function(e){e.forEach(function(e){(n[e=e<0?~e:e]||(n[e]=[])).push(t)})}),r.push(t)}function A(e){return function(t){for(var e,n=-1,r=t.length,i=t[r-1],o=0;++n<r;)e=i,i=t[n],o+=e[0]*i[1]-e[1]*i[0];return Math.abs(o)}(a(t,{type:"Polygon",arcs:[e]}).coordinates[0])}return e.forEach(function t(e){switch(e.type){case"GeometryCollection":e.geometries.forEach(t);break;case"Polygon":o(e.arcs);break;case"MultiPolygon":e.arcs.forEach(o)}}),r.forEach(function(t){if(!t._){var e=[],r=[t];for(t._=1,i.push(e);t=r.pop();)e.push(t),t.forEach(function(t){t.forEach(function(t){n[t<0?~t:t].forEach(function(t){t._||(t._=1,r.push(t))})})})}}),r.forEach(function(t){delete t._}),{type:"MultiPolygon",arcs:i.map(function(e){var r,i=[];if(e.forEach(function(t){t.forEach(function(t){t.forEach(function(t){n[t<0?~t:t].length<2&&i.push(t)})})}),(r=(i=s(t,i)).length)>1)for(var o,a,l=1,c=A(i[0]);l<r;++l)(o=A(i[l]))>c&&(a=i[0],i[0]=i[l],i[l]=a,c=o);return i})}}var c=function(t,e){for(var n=0,r=t.length;n<r;){var i=n+r>>>1;t[i]<e?n=i+1:r=i}return n};t.bbox=r,t.feature=function(t,e){return"GeometryCollection"===e.type?{type:"FeatureCollection",features:e.geometries.map(function(e){return o(t,e)})}:o(t,e)},t.mesh=function(t){return a(t,A.apply(this,arguments))},t.meshArcs=A,t.merge=function(t){return a(t,l.apply(this,arguments))},t.mergeArcs=l,t.neighbors=function(t){var e={},n=t.map(function(){return[]});function r(t,n){t.forEach(function(t){t<0&&(t=~t);var r=e[t];r?r.push(n):e[t]=[n]})}function i(t,e){t.forEach(function(t){r(t,e)})}var o={LineString:r,MultiLineString:i,Polygon:i,MultiPolygon:function(t,e){t.forEach(function(t){i(t,e)})}};for(var a in t.forEach(function t(e,n){"GeometryCollection"===e.type?e.geometries.forEach(function(e){t(e,n)}):e.type in o&&o[e.type](e.arcs,n)}),e)for(var s=e[a],A=s.length,l=0;l<A;++l)for(var u=l+1;u<A;++u){var g,h=s[l],d=s[u];(g=n[h])[a=c(g,d)]!==d&&g.splice(a,0,d),(g=n[d])[a=c(g,h)]!==h&&g.splice(a,0,h)}return n},t.quantize=function(t,e){if(!((e=Math.floor(e))>=2))throw new Error("n must be ≥2");if(t.transform)throw new Error("already quantized");var n,i=r(t),o=i[0],a=(i[2]-o)/(e-1)||1,s=i[1],A=(i[3]-s)/(e-1)||1;function l(t){t[0]=Math.round((t[0]-o)/a),t[1]=Math.round((t[1]-s)/A)}function c(t){switch(t.type){case"GeometryCollection":t.geometries.forEach(c);break;case"Point":l(t.coordinates);break;case"MultiPoint":t.coordinates.forEach(l)}}for(n in t.arcs.forEach(function(t){for(var e,n,r,i=1,l=1,c=t.length,u=t[0],g=u[0]=Math.round((u[0]-o)/a),h=u[1]=Math.round((u[1]-s)/A);i<c;++i)u=t[i],n=Math.round((u[0]-o)/a),r=Math.round((u[1]-s)/A),n===g&&r===h||((e=t[l++])[0]=n-g,g=n,e[1]=r-h,h=r);l<2&&((e=t[l++])[0]=0,e[1]=0),t.length=l}),t.objects)c(t.objects[n]);return t.transform={scale:[a,A],translate:[o,s]},t},t.transform=n,t.untransform=function(t){if(null==(n=t.transform))return e;var n,r,i,o=n.scale[0],a=n.scale[1],s=n.translate[0],A=n.translate[1];return function(t,e){e||(r=i=0);var n=Math.round((t[0]-s)/o),l=Math.round((t[1]-A)/a);return t[0]=n-r,r=n,t[1]=l-i,i=l,t}},Object.defineProperty(t,"__esModule",{value:!0})}("object"==typeof n&&void 0!==e?n:this.topojson=this.topojson||{})},{}],541:[function(t,e,n){"use strict";e.exports=function(t){if(t<0)return[];if(0===t)return[[0]];for(var e=0|Math.round(o(t+1)),n=[],a=0;a<e;++a){for(var s=r.unrank(t,a),A=[0],l=0,c=0;c<s.length;++c)l+=1<<s[c],A.push(l);i(s)<1&&(A[0]=l,A[t]=0),n.push(A)}return n};var r=t("permutation-rank"),i=t("permutation-parity"),o=t("gamma")},{gamma:231,"permutation-parity":463,"permutation-rank":464}],542:[function(t,e,n){"use strict";e.exports=function(t){var e=(t=t||{}).center||[0,0,0],n=t.up||[0,1,0],r=t.right||u(n),i=t.radius||1,o=t.theta||0,c=t.phi||0;if(e=[].slice.call(e,0,3),n=[].slice.call(n,0,3),s(n,n),r=[].slice.call(r,0,3),s(r,r),"eye"in t){var h=t.eye,d=[h[0]-e[0],h[1]-e[1],h[2]-e[2]];a(r,d,n),l(r[0],r[1],r[2])<1e-6?r=u(n):s(r,r),i=l(d[0],d[1],d[2]);var f=A(n,d)/i,p=A(r,d)/i;c=Math.acos(f),o=Math.acos(p)}return i=Math.log(i),new g(t.zoomMin,t.zoomMax,e,n,r,i,o,c)};var r=t("filtered-vector"),i=t("gl-mat4/invert"),o=t("gl-mat4/rotate"),a=t("gl-vec3/cross"),s=t("gl-vec3/normalize"),A=t("gl-vec3/dot");function l(t,e,n){return Math.sqrt(Math.pow(t,2)+Math.pow(e,2)+Math.pow(n,2))}function c(t){return Math.min(1,Math.max(-1,t))}function u(t){var e=Math.abs(t[0]),n=Math.abs(t[1]),r=Math.abs(t[2]),i=[0,0,0];e>Math.max(n,r)?i[2]=1:n>Math.max(e,r)?i[0]=1:i[1]=1;for(var o=0,a=0,A=0;A<3;++A)o+=t[A]*t[A],a+=i[A]*t[A];for(A=0;A<3;++A)i[A]-=a/o*t[A];return s(i,i),i}function g(t,e,n,i,o,a,s,A){this.center=r(n),this.up=r(i),this.right=r(o),this.radius=r([a]),this.angle=r([s,A]),this.angle.bounds=[[-1/0,-Math.PI/2],[1/0,Math.PI/2]],this.setDistanceLimits(t,e),this.computedCenter=this.center.curve(0),this.computedUp=this.up.curve(0),this.computedRight=this.right.curve(0),this.computedRadius=this.radius.curve(0),this.computedAngle=this.angle.curve(0),this.computedToward=[0,0,0],this.computedEye=[0,0,0],this.computedMatrix=new Array(16);for(var l=0;l<16;++l)this.computedMatrix[l]=.5;this.recalcMatrix(0)}var h=g.prototype;h.setDistanceLimits=function(t,e){t=t>0?Math.log(t):-1/0,e=e>0?Math.log(e):1/0,e=Math.max(e,t),this.radius.bounds[0][0]=t,this.radius.bounds[1][0]=e},h.getDistanceLimits=function(t){var e=this.radius.bounds[0];return t?(t[0]=Math.exp(e[0][0]),t[1]=Math.exp(e[1][0]),t):[Math.exp(e[0][0]),Math.exp(e[1][0])]},h.recalcMatrix=function(t){this.center.curve(t),this.up.curve(t),this.right.curve(t),this.radius.curve(t),this.angle.curve(t);for(var e=this.computedUp,n=this.computedRight,r=0,i=0,o=0;o<3;++o)i+=e[o]*n[o],r+=e[o]*e[o];var A=Math.sqrt(r),c=0;for(o=0;o<3;++o)n[o]-=e[o]*i/r,c+=n[o]*n[o],e[o]/=A;var u=Math.sqrt(c);for(o=0;o<3;++o)n[o]/=u;var g=this.computedToward;a(g,e,n),s(g,g);var h=Math.exp(this.computedRadius[0]),d=this.computedAngle[0],f=this.computedAngle[1],p=Math.cos(d),I=Math.sin(d),C=Math.cos(f),m=Math.sin(f),y=this.computedCenter,E=p*C,v=I*C,B=m,b=-p*m,Q=-I*m,w=C,x=this.computedEye,_=this.computedMatrix;for(o=0;o<3;++o){var D=E*n[o]+v*g[o]+B*e[o];_[4*o+1]=b*n[o]+Q*g[o]+w*e[o],_[4*o+2]=D,_[4*o+3]=0}var M=_[1],k=_[5],S=_[9],L=_[2],R=_[6],N=_[10],F=k*N-S*R,T=S*L-M*N,U=M*R-k*L,O=l(F,T,U);for(F/=O,T/=O,U/=O,_[0]=F,_[4]=T,_[8]=U,o=0;o<3;++o)x[o]=y[o]+_[2+4*o]*h;for(o=0;o<3;++o){c=0;for(var G=0;G<3;++G)c+=_[o+4*G]*x[G];_[12+o]=-c}_[15]=1},h.getMatrix=function(t,e){this.recalcMatrix(t);var n=this.computedMatrix;if(e){for(var r=0;r<16;++r)e[r]=n[r];return e}return n};var d=[0,0,0];h.rotate=function(t,e,n,r){if(this.angle.move(t,e,n),r){this.recalcMatrix(t);var i=this.computedMatrix;d[0]=i[2],d[1]=i[6],d[2]=i[10];for(var a=this.computedUp,s=this.computedRight,A=this.computedToward,l=0;l<3;++l)i[4*l]=a[l],i[4*l+1]=s[l],i[4*l+2]=A[l];for(o(i,i,r,d),l=0;l<3;++l)a[l]=i[4*l],s[l]=i[4*l+1];this.up.set(t,a[0],a[1],a[2]),this.right.set(t,s[0],s[1],s[2])}},h.pan=function(t,e,n,r){e=e||0,n=n||0,r=r||0,this.recalcMatrix(t);var i=this.computedMatrix,o=(Math.exp(this.computedRadius[0]),i[1]),a=i[5],s=i[9],A=l(o,a,s);o/=A,a/=A,s/=A;var c=i[0],u=i[4],g=i[8],h=c*o+u*a+g*s,d=l(c-=o*h,u-=a*h,g-=s*h),f=(c/=d)*e+o*n,p=(u/=d)*e+a*n,I=(g/=d)*e+s*n;this.center.move(t,f,p,I);var C=Math.exp(this.computedRadius[0]);C=Math.max(1e-4,C+r),this.radius.set(t,Math.log(C))},h.translate=function(t,e,n,r){this.center.move(t,e||0,n||0,r||0)},h.setMatrix=function(t,e,n,r){var o=1;"number"==typeof n&&(o=0|n),(o<0||o>3)&&(o=1);var a=(o+2)%3;e||(this.recalcMatrix(t),e=this.computedMatrix);var s=e[o],A=e[o+4],u=e[o+8];if(r){var g=Math.abs(s),h=Math.abs(A),d=Math.abs(u),f=Math.max(g,h,d);g===f?(s=s<0?-1:1,A=u=0):d===f?(u=u<0?-1:1,s=A=0):(A=A<0?-1:1,s=u=0)}else{var p=l(s,A,u);s/=p,A/=p,u/=p}var I,C,m=e[a],y=e[a+4],E=e[a+8],v=m*s+y*A+E*u,B=l(m-=s*v,y-=A*v,E-=u*v),b=A*(E/=B)-u*(y/=B),Q=u*(m/=B)-s*E,w=s*y-A*m,x=l(b,Q,w);if(b/=x,Q/=x,w/=x,this.center.jump(t,P,K,z),this.radius.idle(t),this.up.jump(t,s,A,u),this.right.jump(t,m,y,E),2===o){var _=e[1],D=e[5],M=e[9],k=_*m+D*y+M*E,S=_*b+D*Q+M*w;I=F<0?-Math.PI/2:Math.PI/2,C=Math.atan2(S,k)}else{var L=e[2],R=e[6],N=e[10],F=L*s+R*A+N*u,T=L*m+R*y+N*E,U=L*b+R*Q+N*w;I=Math.asin(c(F)),C=Math.atan2(U,T)}this.angle.jump(t,C,I),this.recalcMatrix(t);var O=e[2],G=e[6],H=e[10],j=this.computedMatrix;i(j,e);var Y=j[15],P=j[12]/Y,K=j[13]/Y,z=j[14]/Y,J=Math.exp(this.computedRadius[0]);this.center.jump(t,P-O*J,K-G*J,z-H*J)},h.lastT=function(){return Math.max(this.center.lastT(),this.up.lastT(),this.right.lastT(),this.radius.lastT(),this.angle.lastT())},h.idle=function(t){this.center.idle(t),this.up.idle(t),this.right.idle(t),this.radius.idle(t),this.angle.idle(t)},h.flush=function(t){this.center.flush(t),this.up.flush(t),this.right.flush(t),this.radius.flush(t),this.angle.flush(t)},h.setDistance=function(t,e){e>0&&this.radius.set(t,Math.log(e))},h.lookAt=function(t,e,n,r){this.recalcMatrix(t),e=e||this.computedEye,n=n||this.computedCenter;var i=(r=r||this.computedUp)[0],o=r[1],a=r[2],s=l(i,o,a);if(!(s<1e-6)){i/=s,o/=s,a/=s;var A=e[0]-n[0],u=e[1]-n[1],g=e[2]-n[2],h=l(A,u,g);if(!(h<1e-6)){A/=h,u/=h,g/=h;var d=this.computedRight,f=d[0],p=d[1],I=d[2],C=i*f+o*p+a*I,m=l(f-=C*i,p-=C*o,I-=C*a);if(!(m<.01&&(m=l(f=o*g-a*u,p=a*A-i*g,I=i*u-o*A))<1e-6)){f/=m,p/=m,I/=m,this.up.set(t,i,o,a),this.right.set(t,f,p,I),this.center.set(t,n[0],n[1],n[2]),this.radius.set(t,Math.log(h));var y=o*I-a*p,E=a*f-i*I,v=i*p-o*f,B=l(y,E,v),b=i*A+o*u+a*g,Q=f*A+p*u+I*g,w=(y/=B)*A+(E/=B)*u+(v/=B)*g,x=Math.asin(c(b)),_=Math.atan2(w,Q),D=this.angle._state,M=D[D.length-1],k=D[D.length-2];M%=2*Math.PI;var S=Math.abs(M+2*Math.PI-_),L=Math.abs(M-_),R=Math.abs(M-2*Math.PI-_);S<L&&(M+=2*Math.PI),R<L&&(M-=2*Math.PI),this.angle.jump(this.angle.lastT(),M,k),this.angle.set(t,_,x)}}}}},{"filtered-vector":226,"gl-mat4/invert":265,"gl-mat4/rotate":270,"gl-vec3/cross":334,"gl-vec3/dot":339,"gl-vec3/normalize":356}],543:[function(t,e,n){"use strict";e.exports=function(t,e,n){var i=t*e,o=r*t,a=o-(o-t),s=t-a,A=r*e,l=A-(A-e),c=e-l,u=s*c-(i-a*l-s*l-a*c);return n?(n[0]=u,n[1]=i,n):[u,i]};var r=+(Math.pow(2,27)+1)},{}],544:[function(t,e,n){"use strict";e.exports=function(t,e,n){var r=t+e,i=r-t,o=e-i,a=t-(r-i);return n?(n[0]=a+o,n[1]=r,n):[a+o,r]}},{}],545:[function(t,n,r){(function(e,n){"use strict";var i=t("bit-twiddle"),o=t("dup");e.__TYPEDARRAY_POOL||(e.__TYPEDARRAY_POOL={UINT8:o([32,0]),UINT16:o([32,0]),UINT32:o([32,0]),INT8:o([32,0]),INT16:o([32,0]),INT32:o([32,0]),FLOAT:o([32,0]),DOUBLE:o([32,0]),DATA:o([32,0]),UINT8C:o([32,0]),BUFFER:o([32,0])});var a="undefined"!=typeof Uint8ClampedArray,s=e.__TYPEDARRAY_POOL;s.UINT8C||(s.UINT8C=o([32,0])),s.BUFFER||(s.BUFFER=o([32,0]));var A=s.DATA,l=s.BUFFER;function c(t){if(t){var e=t.length||t.byteLength,n=i.log2(e);A[n].push(t)}}function u(t){t=i.nextPow2(t);var e=i.log2(t),n=A[e];return n.length>0?n.pop():new ArrayBuffer(t)}function g(t){return new Uint8Array(u(t),0,t)}function h(t){return new Uint16Array(u(2*t),0,t)}function d(t){return new Uint32Array(u(4*t),0,t)}function f(t){return new Int8Array(u(t),0,t)}function p(t){return new Int16Array(u(2*t),0,t)}function I(t){return new Int32Array(u(4*t),0,t)}function C(t){return new Float32Array(u(4*t),0,t)}function m(t){return new Float64Array(u(8*t),0,t)}function y(t){return a?new Uint8ClampedArray(u(t),0,t):g(t)}function E(t){return new DataView(u(t),0,t)}function v(t){t=i.nextPow2(t);var e=i.log2(t),r=l[e];return r.length>0?r.pop():new n(t)}r.free=function(t){if(n.isBuffer(t))l[i.log2(t.length)].push(t);else{if("[object ArrayBuffer]"!==Object.prototype.toString.call(t)&&(t=t.buffer),!t)return;var e=t.length||t.byteLength,r=0|i.log2(e);A[r].push(t)}},r.freeUint8=r.freeUint16=r.freeUint32=r.freeInt8=r.freeInt16=r.freeInt32=r.freeFloat32=r.freeFloat=r.freeFloat64=r.freeDouble=r.freeUint8Clamped=r.freeDataView=function(t){c(t.buffer)},r.freeArrayBuffer=c,r.freeBuffer=function(t){l[i.log2(t.length)].push(t)},r.malloc=function(t,e){if(void 0===e||"arraybuffer"===e)return u(t);switch(e){case"uint8":return g(t);case"uint16":return h(t);case"uint32":return d(t);case"int8":return f(t);case"int16":return p(t);case"int32":return I(t);case"float":case"float32":return C(t);case"double":case"float64":return m(t);case"uint8_clamped":return y(t);case"buffer":return v(t);case"data":case"dataview":return E(t);default:return null}return null},r.mallocArrayBuffer=u,r.mallocUint8=g,r.mallocUint16=h,r.mallocUint32=d,r.mallocInt8=f,r.mallocInt16=p,r.mallocInt32=I,r.mallocFloat32=r.mallocFloat=C,r.mallocFloat64=r.mallocDouble=m,r.mallocUint8Clamped=y,r.mallocDataView=E,r.mallocBuffer=v,r.clearCache=function(){for(var t=0;t<32;++t)s.UINT8[t].length=0,s.UINT16[t].length=0,s.UINT32[t].length=0,s.INT8[t].length=0,s.INT16[t].length=0,s.INT32[t].length=0,s.FLOAT[t].length=0,s.DOUBLE[t].length=0,s.UINT8C[t].length=0,A[t].length=0,l[t].length=0}}).call(this,void 0!==e?e:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{},t("buffer").Buffer)},{"bit-twiddle":92,buffer:105,dup:170}],546:[function(t,e,n){"use strict";function r(t){this.roots=new Array(t),this.ranks=new Array(t);for(var e=0;e<t;++e)this.roots[e]=e,this.ranks[e]=0}e.exports=r;var i=r.prototype;Object.defineProperty(i,"length",{get:function(){return this.roots.length}}),i.makeSet=function(){var t=this.roots.length;return this.roots.push(t),this.ranks.push(0),t},i.find=function(t){for(var e=t,n=this.roots;n[t]!==t;)t=n[t];for(;n[e]!==t;){var r=n[e];n[e]=t,e=r}return t},i.link=function(t,e){var n=this.find(t),r=this.find(e);if(n!==r){var i=this.ranks,o=this.roots,a=i[n],s=i[r];a<s?o[n]=r:s<a?o[r]=n:(o[r]=n,++i[n])}}},{}],547:[function(t,e,n){"use strict";e.exports=function(t,e,n){return 0===t.length?t:e?(n||t.sort(e),function(t,e){for(var n=1,r=t.length,i=t[0],o=t[0],a=1;a<r;++a)if(o=i,e(i=t[a],o)){if(a===n){n++;continue}t[n++]=i}return t.length=n,t}(t,e)):(n||t.sort(),function(t){for(var e=1,n=t.length,r=t[0],i=t[0],o=1;o<n;++o,i=r)if(i=r,(r=t[o])!==i){if(o===e){e++;continue}t[e++]=r}return t.length=e,t}(t))}},{}],548:[function(t,e,n){var r=/[\'\"]/;e.exports=function(t){return t?(r.test(t.charAt(0))&&(t=t.substr(1)),r.test(t.charAt(t.length-1))&&(t=t.substr(0,t.length-1)),t):""}},{}],549:[function(t,e,n){"use strict";e.exports=function(t,e,n){Array.isArray(n)||(n=[].slice.call(arguments,2));for(var r=0,i=n.length;r<i;r++){var o=n[r];for(var a in o)if((void 0===e[a]||Array.isArray(e[a])||t[a]!==e[a])&&a in e){var s;if(!0===o[a])s=e[a];else{if(!1===o[a])continue;if("function"==typeof o[a]&&void 0===(s=o[a](e[a],t,e)))continue}t[a]=s}}return t}},{}],550:[function(t,e,n){"use strict";e.exports=function(t,e){return"object"==typeof e&&null!==e||(e={}),r(t,e.canvas||i,e.context||o,e)};var r=t("./lib/vtext"),i=null,o=null;"undefined"!=typeof document&&((i=document.createElement("canvas")).width=8192,i.height=1024,o=i.getContext("2d"))},{"./lib/vtext":551}],551:[function(t,e,n){e.exports=function(t,e,n,r){var o=64,a=1.25,s={breaklines:!1,bolds:!1,italics:!1,subscripts:!1,superscripts:!1};return r&&(r.size&&r.size>0&&(o=r.size),r.lineSpacing&&r.lineSpacing>0&&(a=r.lineSpacing),r.styletags&&r.styletags.breaklines&&(s.breaklines=!!r.styletags.breaklines),r.styletags&&r.styletags.bolds&&(s.bolds=!!r.styletags.bolds),r.styletags&&r.styletags.italics&&(s.italics=!!r.styletags.italics),r.styletags&&r.styletags.subscripts&&(s.subscripts=!!r.styletags.subscripts),r.styletags&&r.styletags.superscripts&&(s.superscripts=!!r.styletags.superscripts)),n.font=[r.fontStyle,r.fontVariant,r.fontWeight,o+"px",r.font].filter(function(t){return t}).join(" "),n.textAlign="start",n.textBaseline="alphabetic",n.direction="ltr",v(function(t,e,n,r,o,a){n=n.replace(/\n/g,""),n=!0===a.breaklines?n.replace(/\<br\>/g,"\n"):n.replace(/\<br\>/g," ");var s="",A=[];for(B=0;B<n.length;++B)A[B]=s;!0===a.bolds&&(A=m(l,c,n,A)),!0===a.italics&&(A=m(u,g,n,A)),!0===a.superscripts&&(A=m(h,f,n,A)),!0===a.subscripts&&(A=m(p,C,n,A));var y=[],E="";for(B=0;B<n.length;++B)null!==A[B]&&(E+=n[B],y.push(A[B]));var v,B,b,Q,w,x=E.split("\n"),_=x.length,D=Math.round(o*r),M=r,k=2*r,S=0,L=_*D+k;t.height<L&&(t.height=L),e.fillStyle="#000",e.fillRect(0,0,t.width,t.height),e.fillStyle="#fff";var R=0,N="";function F(){if(""!==N){var t=e.measureText(N).width;e.fillText(N,M+b,k+Q),b+=t}}function T(){return Math.round(w)+"px "}function U(t,n){var r=""+e.font;if(!0===a.subscripts){var i=t.indexOf(I),o=n.indexOf(I),s=i>-1?parseInt(t[1+i]):0,A=o>-1?parseInt(n[1+o]):0;s!==A&&(r=r.replace(T(),"?px "),w*=Math.pow(.75,A-s),r=r.replace("?px ",T())),Q+=.25*D*(A-s)}if(!0===a.superscripts){var l=t.indexOf(d),u=n.indexOf(d),h=l>-1?parseInt(t[1+l]):0,f=u>-1?parseInt(n[1+u]):0;h!==f&&(r=r.replace(T(),"?px "),w*=Math.pow(.75,f-h),r=r.replace("?px ",T())),Q-=.25*D*(f-h)}if(!0===a.bolds){var p=t.indexOf(c)>-1,C=n.indexOf(c)>-1;!p&&C&&(r=m?r.replace("italic ","italic bold "):"bold "+r),p&&!C&&(r=r.replace("bold ",""))}if(!0===a.italics){var m=t.indexOf(g)>-1,y=n.indexOf(g)>-1;!m&&y&&(r="italic "+r),m&&!y&&(r=r.replace("italic ",""))}e.font=r}for(v=0;v<_;++v){var O=x[v]+"\n";for(b=0,Q=v*D,w=r,N="",B=0;B<O.length;++B){var G=B+R<y.length?y[B+R]:y[y.length-1];s===G?N+=O[B]:(F(),N=O[B],void 0!==G&&(U(s,G),s=G))}F(),R+=O.length;var H=0|Math.round(b+2*M);S<H&&(S=H)}var j=S,Y=k+D*_;return i(e.getImageData(0,0,j,Y).data,[Y,j,4]).pick(-1,-1,0).transpose(1,0)}(e,n,t,o,a,s),r,o)},e.exports.processPixels=v;var r=t("surface-nets"),i=t("ndarray"),o=t("simplify-planar-graph"),a=t("clean-pslg"),s=t("cdt2d"),A=t("planar-graph-to-polyline"),l="b",c="b|",u="i",g="i|",h="sup",d="+",f="+1",p="sub",I="-",C="-1";function m(t,e,n,r){for(var i="<"+t+">",o="</"+t+">",a=i.length,s=o.length,A=e[0]===d||e[0]===I,l=0,c=-s;l>-1&&-1!==(l=n.indexOf(i,l))&&-1!==(c=n.indexOf(o,l+a))&&!(c<=l);){for(var u=l;u<c+s;++u)if(u<l+a||u>=c)r[u]=null,n=n.substr(0,u)+" "+n.substr(u+1);else if(null!==r[u]){var g=r[u].indexOf(e[0]);-1===g?r[u]+=e:A&&(r[u]=r[u].substr(0,g+1)+(1+parseInt(r[u][g+1]))+r[u].substr(g+2))}var h=l+a,f=n.substr(h,c-h).indexOf(i);l=-1!==f?f:c+s}return r}function y(t,e){var n=r(t,128);return e?o(n.cells,n.positions,.25):{edges:n.cells,positions:n.positions}}function E(t,e,n,r){var i=y(t,r),o=function(t,e,n){for(var r=e.textAlign||"start",i=e.textBaseline||"alphabetic",o=[1<<30,1<<30],a=[0,0],s=t.length,A=0;A<s;++A)for(var l=t[A],c=0;c<2;++c)o[c]=0|Math.min(o[c],l[c]),a[c]=0|Math.max(a[c],l[c]);var u=0;switch(r){case"center":u=-.5*(o[0]+a[0]);break;case"right":case"end":u=-a[0];break;case"left":case"start":u=-o[0];break;default:throw new Error("vectorize-text: Unrecognized textAlign: '"+r+"'")}var g=0;switch(i){case"hanging":case"top":g=-o[1];break;case"middle":g=-.5*(o[1]+a[1]);break;case"alphabetic":case"ideographic":g=-3*n;break;case"bottom":g=-a[1];break;default:throw new Error("vectorize-text: Unrecoginized textBaseline: '"+i+"'")}var h=1/n;return"lineHeight"in e?h*=+e.lineHeight:"width"in e?h=e.width/(a[0]-o[0]):"height"in e&&(h=e.height/(a[1]-o[1])),t.map(function(t){return[h*(t[0]+u),h*(t[1]+g)]})}(i.positions,e,n),l=i.edges,c="ccw"===e.orientation;if(a(o,l),e.polygons||e.polygon||e.polyline){for(var u=A(l,o),g=new Array(u.length),h=0;h<u.length;++h){for(var d=u[h],f=new Array(d.length),p=0;p<d.length;++p){for(var I=d[p],C=new Array(I.length),m=0;m<I.length;++m)C[m]=o[I[m]].slice();c&&C.reverse(),f[p]=C}g[h]=f}return g}return e.triangles||e.triangulate||e.triangle?{cells:s(o,l,{delaunay:!1,exterior:!1,interior:!0}),positions:o}:{edges:l,positions:o}}function v(t,e,n){try{return E(t,e,n,!0)}catch(t){}try{return E(t,e,n,!1)}catch(t){}return e.polygons||e.polyline||e.polygon?[]:e.triangles||e.triangulate||e.triangle?{cells:[],positions:[]}:{edges:[],positions:[]}}},{cdt2d:106,"clean-pslg":116,ndarray:450,"planar-graph-to-polyline":468,"simplify-planar-graph":524,"surface-nets":531}],552:[function(t,e,n){!function(){"use strict";if("undefined"==typeof ses||!ses.ok||ses.ok()){"undefined"!=typeof ses&&(ses.weakMapPermitHostObjects=p);var t=!1;if("function"==typeof WeakMap){var n=WeakMap;if("undefined"!=typeof navigator&&/Firefox/.test(navigator.userAgent));else{var r=new n,i=Object.freeze({});if(r.set(i,1),1===r.get(i))return void(e.exports=WeakMap);t=!0}}Object.prototype.hasOwnProperty;var o=Object.getOwnPropertyNames,a=Object.defineProperty,s=Object.isExtensible,A="weakmap:",l=A+"ident:"+Math.random()+"___";if("undefined"!=typeof crypto&&"function"==typeof crypto.getRandomValues&&"function"==typeof ArrayBuffer&&"function"==typeof Uint8Array){var c=new ArrayBuffer(25),u=new Uint8Array(c);crypto.getRandomValues(u),l=A+"rand:"+Array.prototype.map.call(u,function(t){return(t%36).toString(36)}).join("")+"___"}if(a(Object,"getOwnPropertyNames",{value:function(t){return o(t).filter(I)}}),"getPropertyNames"in Object){var g=Object.getPropertyNames;a(Object,"getPropertyNames",{value:function(t){return g(t).filter(I)}})}!function(){var t=Object.freeze;a(Object,"freeze",{value:function(e){return C(e),t(e)}});var e=Object.seal;a(Object,"seal",{value:function(t){return C(t),e(t)}});var n=Object.preventExtensions;a(Object,"preventExtensions",{value:function(t){return C(t),n(t)}})}();var h=!1,d=0,f=function(){this instanceof f||y();var t=[],e=[],n=d++;return Object.create(f.prototype,{get___:{value:m(function(r,i){var o,a=C(r);return a?n in a?a[n]:i:(o=t.indexOf(r))>=0?e[o]:i})},has___:{value:m(function(e){var r=C(e);return r?n in r:t.indexOf(e)>=0})},set___:{value:m(function(r,i){var o,a=C(r);return a?a[n]=i:(o=t.indexOf(r))>=0?e[o]=i:(o=t.length,e[o]=i,t[o]=r),this})},delete___:{value:m(function(r){var i,o,a=C(r);return a?n in a&&delete a[n]:!((i=t.indexOf(r))<0||(o=t.length-1,t[i]=void 0,e[i]=e[o],t[i]=t[o],t.length=o,e.length=o,0))})}})};f.prototype=Object.create(Object.prototype,{get:{value:function(t,e){return this.get___(t,e)},writable:!0,configurable:!0},has:{value:function(t){return this.has___(t)},writable:!0,configurable:!0},set:{value:function(t,e){return this.set___(t,e)},writable:!0,configurable:!0},delete:{value:function(t){return this.delete___(t)},writable:!0,configurable:!0}}),"function"==typeof n?function(){function r(){this instanceof f||y();var e,r=new n,i=void 0,o=!1;return e=t?function(t,e){return r.set(t,e),r.has(t)||(i||(i=new f),i.set(t,e)),this}:function(t,e){if(o)try{r.set(t,e)}catch(n){i||(i=new f),i.set___(t,e)}else r.set(t,e);return this},Object.create(f.prototype,{get___:{value:m(function(t,e){return i?r.has(t)?r.get(t):i.get___(t,e):r.get(t,e)})},has___:{value:m(function(t){return r.has(t)||!!i&&i.has___(t)})},set___:{value:m(e)},delete___:{value:m(function(t){var e=!!r.delete(t);return i&&i.delete___(t)||e})},permitHostObjects___:{value:m(function(t){if(t!==p)throw new Error("bogus call to permitHostObjects___");o=!0})}})}t&&"undefined"!=typeof Proxy&&(Proxy=void 0),r.prototype=f.prototype,e.exports=r,Object.defineProperty(WeakMap.prototype,"constructor",{value:WeakMap,enumerable:!1,configurable:!0,writable:!0})}():("undefined"!=typeof Proxy&&(Proxy=void 0),e.exports=f)}function p(t){t.permitHostObjects___&&t.permitHostObjects___(p)}function I(t){return!(t.substr(0,A.length)==A&&"___"===t.substr(t.length-3))}function C(t){if(t!==Object(t))throw new TypeError("Not an object: "+t);var e=t[l];if(e&&e.key===t)return e;if(s(t)){e={key:t};try{return a(t,l,{value:e,writable:!1,enumerable:!1,configurable:!1}),e}catch(t){return}}}function m(t){return t.prototype=null,Object.freeze(t)}function y(){h||"undefined"==typeof console||(h=!0,console.warn("WeakMap should be invoked as new WeakMap(), not WeakMap(). This will be an error in the future."))}}()},{}],553:[function(t,e,n){var r=t("./hidden-store.js");e.exports=function(){var t={};return function(e){if(("object"!=typeof e||null===e)&&"function"!=typeof e)throw new Error("Weakmap-shim: Key must be object");var n=e.valueOf(t);return n&&n.identity===t?n:r(e,t)}}},{"./hidden-store.js":554}],554:[function(t,e,n){e.exports=function(t,e){var n={identity:e},r=t.valueOf;return Object.defineProperty(t,"valueOf",{value:function(t){return t!==e?r.apply(this,arguments):n},writable:!0}),n}},{}],555:[function(t,e,n){var r=t("./create-store.js");e.exports=function(){var t=r();return{get:function(e,n){var r=t(e);return r.hasOwnProperty("value")?r.value:n},set:function(e,n){return t(e).value=n,this},has:function(e){return"value"in t(e)},delete:function(e){return delete t(e).value}}}},{"./create-store.js":553}],556:[function(t,e,n){var r=t("get-canvas-context");e.exports=function(t){return r("webgl",t)}},{"get-canvas-context":232}],557:[function(t,e,n){var r=t("../main"),i=t("object-assign"),o=r.instance();function a(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}a.prototype=new r.baseCalendar,i(a.prototype,{name:"Chinese",jdEpoch:1721425.5,hasYearZero:!1,minMonth:0,firstMonth:0,minDay:1,regionalOptions:{"":{name:"Chinese",epochs:["BEC","EC"],monthNumbers:function(t,e){if("string"==typeof t){var n=t.match(A);return n?n[0]:""}var r=this._validateYear(t),i=t.month(),o=""+this.toChineseMonth(r,i);return e&&o.length<2&&(o="0"+o),this.isIntercalaryMonth(r,i)&&(o+="i"),o},monthNames:function(t){if("string"==typeof t){var e=t.match(l);return e?e[0]:""}var n=this._validateYear(t),r=t.month(),i=["一月","二月","三月","四月","五月","六月","七月","八月","九月","十月","十一月","十二月"][this.toChineseMonth(n,r)-1];return this.isIntercalaryMonth(n,r)&&(i="闰"+i),i},monthNamesShort:function(t){if("string"==typeof t){var e=t.match(c);return e?e[0]:""}var n=this._validateYear(t),r=t.month(),i=["一","二","三","四","五","六","七","八","九","十","十一","十二"][this.toChineseMonth(n,r)-1];return this.isIntercalaryMonth(n,r)&&(i="闰"+i),i},parseMonth:function(t,e){t=this._validateYear(t);var n,r=parseInt(e);if(isNaN(r))"闰"===e[0]&&(n=!0,e=e.substring(1)),"月"===e[e.length-1]&&(e=e.substring(0,e.length-1)),r=1+["一","二","三","四","五","六","七","八","九","十","十一","十二"].indexOf(e);else{var i=e[e.length-1];n="i"===i||"I"===i}return this.toMonthIndex(t,r,n)},dayNames:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],dayNamesShort:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],dayNamesMin:["Su","Mo","Tu","We","Th","Fr","Sa"],digits:null,dateFormat:"yyyy/mm/dd",firstDay:1,isRTL:!1}},_validateYear:function(t,e){if(t.year&&(t=t.year()),"number"!=typeof t||t<1888||t>2111)throw e.replace(/\{0\}/,this.local.name);return t},toMonthIndex:function(t,e,n){var i=this.intercalaryMonth(t);if(n&&e!==i||e<1||e>12)throw r.local.invalidMonth.replace(/\{0\}/,this.local.name);return i?!n&&e<=i?e-1:e:e-1},toChineseMonth:function(t,e){t.year&&(e=(t=t.year()).month());var n=this.intercalaryMonth(t);if(e<0||e>(n?12:11))throw r.local.invalidMonth.replace(/\{0\}/,this.local.name);return n?e<n?e+1:e:e+1},intercalaryMonth:function(t){return t=this._validateYear(t),u[t-u[0]]>>13},isIntercalaryMonth:function(t,e){t.year&&(e=(t=t.year()).month());var n=this.intercalaryMonth(t);return!!n&&n===e},leapYear:function(t){return 0!==this.intercalaryMonth(t)},weekOfYear:function(t,e,n){var i,a=this._validateYear(t,r.local.invalidyear),s=g[a-g[0]],A=s>>9&4095,l=s>>5&15,c=31&s;(i=o.newDate(A,l,c)).add(4-(i.dayOfWeek()||7),"d");var u=this.toJD(t,e,n)-i.toJD();return 1+Math.floor(u/7)},monthsInYear:function(t){return this.leapYear(t)?13:12},daysInMonth:function(t,e){t.year&&(e=t.month(),t=t.year()),t=this._validateYear(t);var n=u[t-u[0]];if(e>(n>>13?12:11))throw r.local.invalidMonth.replace(/\{0\}/,this.local.name);return n&1<<12-e?30:29},weekDay:function(t,e,n){return(this.dayOfWeek(t,e,n)||7)<6},toJD:function(t,e,n){var i=this._validate(t,s,n,r.local.invalidDate);t=this._validateYear(i.year()),e=i.month(),n=i.day();var a=this.isIntercalaryMonth(t,e),s=this.toChineseMonth(t,e),A=function(t,e,n,r,i){var o,a,s;if("object"==typeof t)a=t,o=e||{};else{var A="number"==typeof t&&t>=1888&&t<=2111;if(!A)throw new Error("Lunar year outside range 1888-2111");var l="number"==typeof e&&e>=1&&e<=12;if(!l)throw new Error("Lunar month outside range 1 - 12");var c,h="number"==typeof n&&n>=1&&n<=30;if(!h)throw new Error("Lunar day outside range 1 - 30");"object"==typeof r?(c=!1,o=r):(c=!!r,o={}),a={year:t,month:e,day:n,isIntercalary:c}}s=a.day-1;var d,f=u[a.year-u[0]],p=f>>13;d=p?a.month>p?a.month:a.isIntercalary?a.month:a.month-1:a.month-1;for(var I=0;I<d;I++){var C=f&1<<12-I?30:29;s+=C}var m=g[a.year-g[0]],y=new Date(m>>9&4095,(m>>5&15)-1,(31&m)+s);return o.year=y.getFullYear(),o.month=1+y.getMonth(),o.day=y.getDate(),o}(t,s,n,a);return o.toJD(A.year,A.month,A.day)},fromJD:function(t){var e=o.fromJD(t),n=function(t,e,n,r){var i,o;if("object"==typeof t)i=t,o=e||{};else{var a="number"==typeof t&&t>=1888&&t<=2111;if(!a)throw new Error("Solar year outside range 1888-2111");var s="number"==typeof e&&e>=1&&e<=12;if(!s)throw new Error("Solar month outside range 1 - 12");var A="number"==typeof n&&n>=1&&n<=31;if(!A)throw new Error("Solar day outside range 1 - 31");i={year:t,month:e,day:n},o={}}var l=g[i.year-g[0]],c=i.year<<9|i.month<<5|i.day;o.year=c>=l?i.year:i.year-1,l=g[o.year-g[0]];var h,d=new Date(l>>9&4095,(l>>5&15)-1,31&l),f=new Date(i.year,i.month-1,i.day);h=Math.round((f-d)/864e5);var p,I=u[o.year-u[0]];for(p=0;p<13;p++){var C=I&1<<12-p?30:29;if(h<C)break;h-=C}var m=I>>13;return!m||p<m?(o.isIntercalary=!1,o.month=1+p):p===m?(o.isIntercalary=!0,o.month=p):(o.isIntercalary=!1,o.month=p),o.day=1+h,o}(e.year(),e.month(),e.day()),r=this.toMonthIndex(n.year,n.month,n.isIntercalary);return this.newDate(n.year,r,n.day)},fromString:function(t){var e=t.match(s),n=this._validateYear(+e[1]),r=+e[2],i=!!e[3],o=this.toMonthIndex(n,r,i),a=+e[4];return this.newDate(n,o,a)},add:function(t,e,n){var r=t.year(),i=t.month(),o=this.isIntercalaryMonth(r,i),s=this.toChineseMonth(r,i),A=Object.getPrototypeOf(a.prototype).add.call(this,t,e,n);if("y"===n){var l=A.year(),c=A.month(),u=this.isIntercalaryMonth(l,s),g=o&&u?this.toMonthIndex(l,s,!0):this.toMonthIndex(l,s,!1);g!==c&&A.month(g)}return A}});var s=/^\s*(-?\d\d\d\d|\d\d)[-\/](\d?\d)([iI]?)[-\/](\d?\d)/m,A=/^\d?\d[iI]?/m,l=/^\u95f0?\u5341?[\u4e00\u4e8c\u4e09\u56db\u4e94\u516d\u4e03\u516b\u4e5d]?\u6708/m,c=/^\u95f0?\u5341?[\u4e00\u4e8c\u4e09\u56db\u4e94\u516d\u4e03\u516b\u4e5d]?/m;r.calendars.chinese=a;var u=[1887,5780,5802,19157,2742,50359,1198,2646,46378,7466,3412,30122,5482,67949,2396,5294,43597,6732,6954,36181,2772,4954,18781,2396,54427,5274,6730,47781,5800,6868,21210,4790,59703,2350,5270,46667,3402,3496,38325,1388,4782,18735,2350,52374,6804,7498,44457,2906,1388,29294,4700,63789,6442,6804,56138,5802,2772,38235,1210,4698,22827,5418,63125,3476,5802,43701,2484,5302,27223,2646,70954,7466,3412,54698,5482,2412,38062,5294,2636,32038,6954,60245,2772,4826,43357,2394,5274,39501,6730,72357,5800,5844,53978,4790,2358,38039,5270,87627,3402,3496,54708,5484,4782,43311,2350,3222,27978,7498,68965,2904,5484,45677,4700,6444,39573,6804,6986,19285,2772,62811,1210,4698,47403,5418,5780,38570,5546,76469,2420,5302,51799,2646,5414,36501,3412,5546,18869,2412,54446,5276,6732,48422,6822,2900,28010,4826,92509,2394,5274,55883,6730,6820,47956,5812,2778,18779,2358,62615,5270,5450,46757,3492,5556,27318,4718,67887,2350,3222,52554,7498,3428,38252,5468,4700,31022,6444,64149,6804,6986,43861,2772,5338,35421,2650,70955,5418,5780,54954,5546,2740,38074,5302,2646,29991,3366,61011,3412,5546,43445,2412,5294,35406,6732,72998,6820,6996,52586,2778,2396,38045,5274,6698,23333,6820,64338,5812,2746,43355,2358,5270,39499,5450,79525,3492,5548],g=[1887,966732,967231,967733,968265,968766,969297,969798,970298,970829,971330,971830,972362,972863,973395,973896,974397,974928,975428,975929,976461,976962,977462,977994,978494,979026,979526,980026,980558,981059,981559,982091,982593,983124,983624,984124,984656,985157,985656,986189,986690,987191,987722,988222,988753,989254,989754,990286,990788,991288,991819,992319,992851,993352,993851,994383,994885,995385,995917,996418,996918,997450,997949,998481,998982,999483,1000014,1000515,1001016,1001548,1002047,1002578,1003080,1003580,1004111,1004613,1005113,1005645,1006146,1006645,1007177,1007678,1008209,1008710,1009211,1009743,1010243,1010743,1011275,1011775,1012306,1012807,1013308,1013840,1014341,1014841,1015373,1015874,1016404,1016905,1017405,1017937,1018438,1018939,1019471,1019972,1020471,1021002,1021503,1022035,1022535,1023036,1023568,1024069,1024568,1025100,1025601,1026102,1026633,1027133,1027666,1028167,1028666,1029198,1029699,1030199,1030730,1031231,1031763,1032264,1032764,1033296,1033797,1034297,1034828,1035329,1035830,1036362,1036861,1037393,1037894,1038394,1038925,1039427,1039927,1040459,1040959,1041491,1041992,1042492,1043023,1043524,1044024,1044556,1045057,1045558,1046090,1046590,1047121,1047622,1048122,1048654,1049154,1049655,1050187,1050689,1051219,1051720,1052220,1052751,1053252,1053752,1054284,1054786,1055285,1055817,1056317,1056849,1057349,1057850,1058382,1058883,1059383,1059915,1060415,1060947,1061447,1061947,1062479,1062981,1063480,1064012,1064514,1065014,1065545,1066045,1066577,1067078,1067578,1068110,1068611,1069112,1069642,1070142,1070674,1071175,1071675,1072207,1072709,1073209,1073740,1074241,1074741,1075273,1075773,1076305,1076807,1077308,1077839,1078340,1078840,1079372,1079871,1080403,1080904]},{"../main":571,"object-assign":454}],558:[function(t,e,n){var r=t("../main"),i=t("object-assign");function o(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}o.prototype=new r.baseCalendar,i(o.prototype,{name:"Coptic",jdEpoch:1825029.5,daysPerMonth:[30,30,30,30,30,30,30,30,30,30,30,30,5],hasYearZero:!1,minMonth:1,firstMonth:1,minDay:1,regionalOptions:{"":{name:"Coptic",epochs:["BAM","AM"],monthNames:["Thout","Paopi","Hathor","Koiak","Tobi","Meshir","Paremhat","Paremoude","Pashons","Paoni","Epip","Mesori","Pi Kogi Enavot"],monthNamesShort:["Tho","Pao","Hath","Koi","Tob","Mesh","Pat","Pad","Pash","Pao","Epi","Meso","PiK"],dayNames:["Tkyriaka","Pesnau","Pshoment","Peftoou","Ptiou","Psoou","Psabbaton"],dayNamesShort:["Tky","Pes","Psh","Pef","Pti","Pso","Psa"],dayNamesMin:["Tk","Pes","Psh","Pef","Pt","Pso","Psa"],digits:null,dateFormat:"dd/mm/yyyy",firstDay:0,isRTL:!1}},leapYear:function(t){var e=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear);return(t=e.year()+(e.year()<0?1:0))%4==3||t%4==-1},monthsInYear:function(t){return this._validate(t,this.minMonth,this.minDay,r.local.invalidYear||r.regionalOptions[""].invalidYear),13},weekOfYear:function(t,e,n){var r=this.newDate(t,e,n);return r.add(-r.dayOfWeek(),"d"),Math.floor((r.dayOfYear()-1)/7)+1},daysInMonth:function(t,e){var n=this._validate(t,e,this.minDay,r.local.invalidMonth);return this.daysPerMonth[n.month()-1]+(13===n.month()&&this.leapYear(n.year())?1:0)},weekDay:function(t,e,n){return(this.dayOfWeek(t,e,n)||7)<6},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);return(t=i.year())<0&&t++,i.day()+30*(i.month()-1)+365*(t-1)+Math.floor(t/4)+this.jdEpoch-1},fromJD:function(t){var e=Math.floor(t)+.5-this.jdEpoch,n=Math.floor((e-Math.floor((e+366)/1461))/365)+1;n<=0&&n--,e=Math.floor(t)+.5-this.newDate(n,1,1).toJD();var r=Math.floor(e/30)+1,i=e-30*(r-1)+1;return this.newDate(n,r,i)}}),r.calendars.coptic=o},{"../main":571,"object-assign":454}],559:[function(t,e,n){var r=t("../main"),i=t("object-assign");function o(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}o.prototype=new r.baseCalendar,i(o.prototype,{name:"Discworld",jdEpoch:1721425.5,daysPerMonth:[16,32,32,32,32,32,32,32,32,32,32,32,32],hasYearZero:!1,minMonth:1,firstMonth:1,minDay:1,regionalOptions:{"":{name:"Discworld",epochs:["BUC","UC"],monthNames:["Ick","Offle","February","March","April","May","June","Grune","August","Spune","Sektober","Ember","December"],monthNamesShort:["Ick","Off","Feb","Mar","Apr","May","Jun","Gru","Aug","Spu","Sek","Emb","Dec"],dayNames:["Sunday","Octeday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],dayNamesShort:["Sun","Oct","Mon","Tue","Wed","Thu","Fri","Sat"],dayNamesMin:["Su","Oc","Mo","Tu","We","Th","Fr","Sa"],digits:null,dateFormat:"yyyy/mm/dd",firstDay:2,isRTL:!1}},leapYear:function(t){return this._validate(t,this.minMonth,this.minDay,r.local.invalidYear),!1},monthsInYear:function(t){return this._validate(t,this.minMonth,this.minDay,r.local.invalidYear),13},daysInYear:function(t){return this._validate(t,this.minMonth,this.minDay,r.local.invalidYear),400},weekOfYear:function(t,e,n){var r=this.newDate(t,e,n);return r.add(-r.dayOfWeek(),"d"),Math.floor((r.dayOfYear()-1)/8)+1},daysInMonth:function(t,e){var n=this._validate(t,e,this.minDay,r.local.invalidMonth);return this.daysPerMonth[n.month()-1]},daysInWeek:function(){return 8},dayOfWeek:function(t,e,n){return(this._validate(t,e,n,r.local.invalidDate).day()+1)%8},weekDay:function(t,e,n){var r=this.dayOfWeek(t,e,n);return r>=2&&r<=6},extraInfo:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);return{century:a[Math.floor((i.year()-1)/100)+1]||""}},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);return t=i.year()+(i.year()<0?1:0),e=i.month(),(n=i.day())+(e>1?16:0)+(e>2?32*(e-2):0)+400*(t-1)+this.jdEpoch-1},fromJD:function(t){t=Math.floor(t+.5)-Math.floor(this.jdEpoch)-1;var e=Math.floor(t/400)+1;t-=400*(e-1),t+=t>15?16:0;var n=Math.floor(t/32)+1,r=t-32*(n-1)+1;return this.newDate(e<=0?e-1:e,n,r)}});var a={20:"Fruitbat",21:"Anchovy"};r.calendars.discworld=o},{"../main":571,"object-assign":454}],560:[function(t,e,n){var r=t("../main"),i=t("object-assign");function o(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}o.prototype=new r.baseCalendar,i(o.prototype,{name:"Ethiopian",jdEpoch:1724220.5,daysPerMonth:[30,30,30,30,30,30,30,30,30,30,30,30,5],hasYearZero:!1,minMonth:1,firstMonth:1,minDay:1,regionalOptions:{"":{name:"Ethiopian",epochs:["BEE","EE"],monthNames:["Meskerem","Tikemet","Hidar","Tahesas","Tir","Yekatit","Megabit","Miazia","Genbot","Sene","Hamle","Nehase","Pagume"],monthNamesShort:["Mes","Tik","Hid","Tah","Tir","Yek","Meg","Mia","Gen","Sen","Ham","Neh","Pag"],dayNames:["Ehud","Segno","Maksegno","Irob","Hamus","Arb","Kidame"],dayNamesShort:["Ehu","Seg","Mak","Iro","Ham","Arb","Kid"],dayNamesMin:["Eh","Se","Ma","Ir","Ha","Ar","Ki"],digits:null,dateFormat:"dd/mm/yyyy",firstDay:0,isRTL:!1}},leapYear:function(t){var e=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear);return(t=e.year()+(e.year()<0?1:0))%4==3||t%4==-1},monthsInYear:function(t){return this._validate(t,this.minMonth,this.minDay,r.local.invalidYear||r.regionalOptions[""].invalidYear),13},weekOfYear:function(t,e,n){var r=this.newDate(t,e,n);return r.add(-r.dayOfWeek(),"d"),Math.floor((r.dayOfYear()-1)/7)+1},daysInMonth:function(t,e){var n=this._validate(t,e,this.minDay,r.local.invalidMonth);return this.daysPerMonth[n.month()-1]+(13===n.month()&&this.leapYear(n.year())?1:0)},weekDay:function(t,e,n){return(this.dayOfWeek(t,e,n)||7)<6},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);return(t=i.year())<0&&t++,i.day()+30*(i.month()-1)+365*(t-1)+Math.floor(t/4)+this.jdEpoch-1},fromJD:function(t){var e=Math.floor(t)+.5-this.jdEpoch,n=Math.floor((e-Math.floor((e+366)/1461))/365)+1;n<=0&&n--,e=Math.floor(t)+.5-this.newDate(n,1,1).toJD();var r=Math.floor(e/30)+1,i=e-30*(r-1)+1;return this.newDate(n,r,i)}}),r.calendars.ethiopian=o},{"../main":571,"object-assign":454}],561:[function(t,e,n){var r=t("../main"),i=t("object-assign");function o(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}function a(t,e){return t-e*Math.floor(t/e)}o.prototype=new r.baseCalendar,i(o.prototype,{name:"Hebrew",jdEpoch:347995.5,daysPerMonth:[30,29,30,29,30,29,30,29,30,29,30,29,29],hasYearZero:!1,minMonth:1,firstMonth:7,minDay:1,regionalOptions:{"":{name:"Hebrew",epochs:["BAM","AM"],monthNames:["Nisan","Iyar","Sivan","Tammuz","Av","Elul","Tishrei","Cheshvan","Kislev","Tevet","Shevat","Adar","Adar II"],monthNamesShort:["Nis","Iya","Siv","Tam","Av","Elu","Tis","Che","Kis","Tev","She","Ada","Ad2"],dayNames:["Yom Rishon","Yom Sheni","Yom Shlishi","Yom Revi'i","Yom Chamishi","Yom Shishi","Yom Shabbat"],dayNamesShort:["Ris","She","Shl","Rev","Cha","Shi","Sha"],dayNamesMin:["Ri","She","Shl","Re","Ch","Shi","Sha"],digits:null,dateFormat:"dd/mm/yyyy",firstDay:0,isRTL:!1}},leapYear:function(t){var e=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear);return this._leapYear(e.year())},_leapYear:function(t){return a(7*(t=t<0?t+1:t)+1,19)<7},monthsInYear:function(t){return this._validate(t,this.minMonth,this.minDay,r.local.invalidYear),this._leapYear(t.year?t.year():t)?13:12},weekOfYear:function(t,e,n){var r=this.newDate(t,e,n);return r.add(-r.dayOfWeek(),"d"),Math.floor((r.dayOfYear()-1)/7)+1},daysInYear:function(t){return t=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear).year(),this.toJD(-1===t?1:t+1,7,1)-this.toJD(t,7,1)},daysInMonth:function(t,e){return t.year&&(e=t.month(),t=t.year()),this._validate(t,e,this.minDay,r.local.invalidMonth),12===e&&this.leapYear(t)?30:8===e&&5===a(this.daysInYear(t),10)?30:9===e&&3===a(this.daysInYear(t),10)?29:this.daysPerMonth[e-1]},weekDay:function(t,e,n){return 6!==this.dayOfWeek(t,e,n)},extraInfo:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);return{yearType:(this.leapYear(i)?"embolismic":"common")+" "+["deficient","regular","complete"][this.daysInYear(i)%10-3]}},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);t=i.year(),e=i.month(),n=i.day();var o=t<=0?t+1:t,a=this.jdEpoch+this._delay1(o)+this._delay2(o)+n+1;if(e<7){for(var s=7;s<=this.monthsInYear(t);s++)a+=this.daysInMonth(t,s);for(s=1;s<e;s++)a+=this.daysInMonth(t,s)}else for(s=7;s<e;s++)a+=this.daysInMonth(t,s);return a},_delay1:function(t){var e=Math.floor((235*t-234)/19),n=12084+13753*e,r=29*e+Math.floor(n/25920);return a(3*(r+1),7)<3&&r++,r},_delay2:function(t){var e=this._delay1(t-1),n=this._delay1(t);return this._delay1(t+1)-n==356?2:n-e==382?1:0},fromJD:function(t){t=Math.floor(t)+.5;for(var e=Math.floor(98496*(t-this.jdEpoch)/35975351)-1;t>=this.toJD(-1===e?1:e+1,7,1);)e++;for(var n=t<this.toJD(e,1,1)?7:1;t>this.toJD(e,n,this.daysInMonth(e,n));)n++;var r=t-this.toJD(e,n,1)+1;return this.newDate(e,n,r)}}),r.calendars.hebrew=o},{"../main":571,"object-assign":454}],562:[function(t,e,n){var r=t("../main"),i=t("object-assign");function o(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}o.prototype=new r.baseCalendar,i(o.prototype,{name:"Islamic",jdEpoch:1948439.5,daysPerMonth:[30,29,30,29,30,29,30,29,30,29,30,29],hasYearZero:!1,minMonth:1,firstMonth:1,minDay:1,regionalOptions:{"":{name:"Islamic",epochs:["BH","AH"],monthNames:["Muharram","Safar","Rabi' al-awwal","Rabi' al-thani","Jumada al-awwal","Jumada al-thani","Rajab","Sha'aban","Ramadan","Shawwal","Dhu al-Qi'dah","Dhu al-Hijjah"],monthNamesShort:["Muh","Saf","Rab1","Rab2","Jum1","Jum2","Raj","Sha'","Ram","Shaw","DhuQ","DhuH"],dayNames:["Yawm al-ahad","Yawm al-ithnayn","Yawm ath-thulaathaa'","Yawm al-arbi'aa'","Yawm al-khamīs","Yawm al-jum'a","Yawm as-sabt"],dayNamesShort:["Aha","Ith","Thu","Arb","Kha","Jum","Sab"],dayNamesMin:["Ah","It","Th","Ar","Kh","Ju","Sa"],digits:null,dateFormat:"yyyy/mm/dd",firstDay:6,isRTL:!1}},leapYear:function(t){return(11*this._validate(t,this.minMonth,this.minDay,r.local.invalidYear).year()+14)%30<11},weekOfYear:function(t,e,n){var r=this.newDate(t,e,n);return r.add(-r.dayOfWeek(),"d"),Math.floor((r.dayOfYear()-1)/7)+1},daysInYear:function(t){return this.leapYear(t)?355:354},daysInMonth:function(t,e){var n=this._validate(t,e,this.minDay,r.local.invalidMonth);return this.daysPerMonth[n.month()-1]+(12===n.month()&&this.leapYear(n.year())?1:0)},weekDay:function(t,e,n){return 5!==this.dayOfWeek(t,e,n)},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);return t=i.year(),e=i.month(),t=t<=0?t+1:t,(n=i.day())+Math.ceil(29.5*(e-1))+354*(t-1)+Math.floor((3+11*t)/30)+this.jdEpoch-1},fromJD:function(t){t=Math.floor(t)+.5;var e=Math.floor((30*(t-this.jdEpoch)+10646)/10631);e=e<=0?e-1:e;var n=Math.min(12,Math.ceil((t-29-this.toJD(e,1,1))/29.5)+1),r=t-this.toJD(e,n,1)+1;return this.newDate(e,n,r)}}),r.calendars.islamic=o},{"../main":571,"object-assign":454}],563:[function(t,e,n){var r=t("../main"),i=t("object-assign");function o(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}o.prototype=new r.baseCalendar,i(o.prototype,{name:"Julian",jdEpoch:1721423.5,daysPerMonth:[31,28,31,30,31,30,31,31,30,31,30,31],hasYearZero:!1,minMonth:1,firstMonth:1,minDay:1,regionalOptions:{"":{name:"Julian",epochs:["BC","AD"],monthNames:["January","February","March","April","May","June","July","August","September","October","November","December"],monthNamesShort:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],dayNames:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],dayNamesShort:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],dayNamesMin:["Su","Mo","Tu","We","Th","Fr","Sa"],digits:null,dateFormat:"mm/dd/yyyy",firstDay:0,isRTL:!1}},leapYear:function(t){var e=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear);return(t=e.year()<0?e.year()+1:e.year())%4==0},weekOfYear:function(t,e,n){var r=this.newDate(t,e,n);return r.add(4-(r.dayOfWeek()||7),"d"),Math.floor((r.dayOfYear()-1)/7)+1},daysInMonth:function(t,e){var n=this._validate(t,e,this.minDay,r.local.invalidMonth);return this.daysPerMonth[n.month()-1]+(2===n.month()&&this.leapYear(n.year())?1:0)},weekDay:function(t,e,n){return(this.dayOfWeek(t,e,n)||7)<6},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);return t=i.year(),e=i.month(),n=i.day(),t<0&&t++,e<=2&&(t--,e+=12),Math.floor(365.25*(t+4716))+Math.floor(30.6001*(e+1))+n-1524.5},fromJD:function(t){var e=Math.floor(t+.5)+1524,n=Math.floor((e-122.1)/365.25),r=Math.floor(365.25*n),i=Math.floor((e-r)/30.6001),o=i-Math.floor(i<14?1:13),a=n-Math.floor(o>2?4716:4715),s=e-r-Math.floor(30.6001*i);return a<=0&&a--,this.newDate(a,o,s)}}),r.calendars.julian=o},{"../main":571,"object-assign":454}],564:[function(t,e,n){var r=t("../main"),i=t("object-assign");function o(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}function a(t,e){return t-e*Math.floor(t/e)}function s(t,e){return a(t-1,e)+1}o.prototype=new r.baseCalendar,i(o.prototype,{name:"Mayan",jdEpoch:584282.5,hasYearZero:!0,minMonth:0,firstMonth:0,minDay:0,regionalOptions:{"":{name:"Mayan",epochs:["",""],monthNames:["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17"],monthNamesShort:["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17"],dayNames:["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19"],dayNamesShort:["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19"],dayNamesMin:["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19"],digits:null,dateFormat:"YYYY.m.d",firstDay:0,isRTL:!1,haabMonths:["Pop","Uo","Zip","Zotz","Tzec","Xul","Yaxkin","Mol","Chen","Yax","Zac","Ceh","Mac","Kankin","Muan","Pax","Kayab","Cumku","Uayeb"],tzolkinMonths:["Imix","Ik","Akbal","Kan","Chicchan","Cimi","Manik","Lamat","Muluc","Oc","Chuen","Eb","Ben","Ix","Men","Cib","Caban","Etznab","Cauac","Ahau"]}},leapYear:function(t){return this._validate(t,this.minMonth,this.minDay,r.local.invalidYear),!1},formatYear:function(t){t=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear).year();var e=Math.floor(t/400);return t%=400,t+=t<0?400:0,e+"."+Math.floor(t/20)+"."+t%20},forYear:function(t){if((t=t.split(".")).length<3)throw"Invalid Mayan year";for(var e=0,n=0;n<t.length;n++){var r=parseInt(t[n],10);if(Math.abs(r)>19||n>0&&r<0)throw"Invalid Mayan year";e=20*e+r}return e},monthsInYear:function(t){return this._validate(t,this.minMonth,this.minDay,r.local.invalidYear),18},weekOfYear:function(t,e,n){return this._validate(t,e,n,r.local.invalidDate),0},daysInYear:function(t){return this._validate(t,this.minMonth,this.minDay,r.local.invalidYear),360},daysInMonth:function(t,e){return this._validate(t,e,this.minDay,r.local.invalidMonth),20},daysInWeek:function(){return 5},dayOfWeek:function(t,e,n){return this._validate(t,e,n,r.local.invalidDate).day()},weekDay:function(t,e,n){return this._validate(t,e,n,r.local.invalidDate),!0},extraInfo:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate).toJD(),o=this._toHaab(i),a=this._toTzolkin(i);return{haabMonthName:this.local.haabMonths[o[0]-1],haabMonth:o[0],haabDay:o[1],tzolkinDayName:this.local.tzolkinMonths[a[0]-1],tzolkinDay:a[0],tzolkinTrecena:a[1]}},_toHaab:function(t){var e=a(8+(t-=this.jdEpoch)+340,365);return[Math.floor(e/20)+1,a(e,20)]},_toTzolkin:function(t){return[s(20+(t-=this.jdEpoch),20),s(t+4,13)]},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);return i.day()+20*i.month()+360*i.year()+this.jdEpoch},fromJD:function(t){t=Math.floor(t)+.5-this.jdEpoch;var e=Math.floor(t/360);t%=360,t+=t<0?360:0;var n=Math.floor(t/20),r=t%20;return this.newDate(e,n,r)}}),r.calendars.mayan=o},{"../main":571,"object-assign":454}],565:[function(t,e,n){var r=t("../main"),i=t("object-assign");function o(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}o.prototype=new r.baseCalendar;var a=r.instance("gregorian");i(o.prototype,{name:"Nanakshahi",jdEpoch:2257673.5,daysPerMonth:[31,31,31,31,31,30,30,30,30,30,30,30],hasYearZero:!1,minMonth:1,firstMonth:1,minDay:1,regionalOptions:{"":{name:"Nanakshahi",epochs:["BN","AN"],monthNames:["Chet","Vaisakh","Jeth","Harh","Sawan","Bhadon","Assu","Katak","Maghar","Poh","Magh","Phagun"],monthNamesShort:["Che","Vai","Jet","Har","Saw","Bha","Ass","Kat","Mgr","Poh","Mgh","Pha"],dayNames:["Somvaar","Mangalvar","Budhvaar","Veervaar","Shukarvaar","Sanicharvaar","Etvaar"],dayNamesShort:["Som","Mangal","Budh","Veer","Shukar","Sanichar","Et"],dayNamesMin:["So","Ma","Bu","Ve","Sh","Sa","Et"],digits:null,dateFormat:"dd-mm-yyyy",firstDay:0,isRTL:!1}},leapYear:function(t){var e=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear||r.regionalOptions[""].invalidYear);return a.leapYear(e.year()+(e.year()<1?1:0)+1469)},weekOfYear:function(t,e,n){var r=this.newDate(t,e,n);return r.add(1-(r.dayOfWeek()||7),"d"),Math.floor((r.dayOfYear()-1)/7)+1},daysInMonth:function(t,e){var n=this._validate(t,e,this.minDay,r.local.invalidMonth);return this.daysPerMonth[n.month()-1]+(12===n.month()&&this.leapYear(n.year())?1:0)},weekDay:function(t,e,n){return(this.dayOfWeek(t,e,n)||7)<6},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidMonth);(t=i.year())<0&&t++;for(var o=i.day(),s=1;s<i.month();s++)o+=this.daysPerMonth[s-1];return o+a.toJD(t+1468,3,13)},fromJD:function(t){t=Math.floor(t+.5);for(var e=Math.floor((t-(this.jdEpoch-1))/366);t>=this.toJD(e+1,1,1);)e++;for(var n=t-Math.floor(this.toJD(e,1,1)+.5)+1,r=1;n>this.daysInMonth(e,r);)n-=this.daysInMonth(e,r),r++;return this.newDate(e,r,n)}}),r.calendars.nanakshahi=o},{"../main":571,"object-assign":454}],566:[function(t,e,n){var r=t("../main"),i=t("object-assign");function o(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}o.prototype=new r.baseCalendar,i(o.prototype,{name:"Nepali",jdEpoch:1700709.5,daysPerMonth:[31,31,32,32,31,30,30,29,30,29,30,30],hasYearZero:!1,minMonth:1,firstMonth:1,minDay:1,daysPerYear:365,regionalOptions:{"":{name:"Nepali",epochs:["BBS","ABS"],monthNames:["Baisakh","Jestha","Ashadh","Shrawan","Bhadra","Ashwin","Kartik","Mangsir","Paush","Mangh","Falgun","Chaitra"],monthNamesShort:["Bai","Je","As","Shra","Bha","Ash","Kar","Mang","Pau","Ma","Fal","Chai"],dayNames:["Aaitabaar","Sombaar","Manglbaar","Budhabaar","Bihibaar","Shukrabaar","Shanibaar"],dayNamesShort:["Aaita","Som","Mangl","Budha","Bihi","Shukra","Shani"],dayNamesMin:["Aai","So","Man","Bu","Bi","Shu","Sha"],digits:null,dateFormat:"dd/mm/yyyy",firstDay:1,isRTL:!1}},leapYear:function(t){return this.daysInYear(t)!==this.daysPerYear},weekOfYear:function(t,e,n){var r=this.newDate(t,e,n);return r.add(-r.dayOfWeek(),"d"),Math.floor((r.dayOfYear()-1)/7)+1},daysInYear:function(t){if(t=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear).year(),void 0===this.NEPALI_CALENDAR_DATA[t])return this.daysPerYear;for(var e=0,n=this.minMonth;n<=12;n++)e+=this.NEPALI_CALENDAR_DATA[t][n];return e},daysInMonth:function(t,e){return t.year&&(e=t.month(),t=t.year()),this._validate(t,e,this.minDay,r.local.invalidMonth),void 0===this.NEPALI_CALENDAR_DATA[t]?this.daysPerMonth[e-1]:this.NEPALI_CALENDAR_DATA[t][e]},weekDay:function(t,e,n){return 6!==this.dayOfWeek(t,e,n)},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);t=i.year(),e=i.month(),n=i.day();var o=r.instance(),a=0,s=e,A=t;this._createMissingCalendarData(t);var l=t-(s>9||9===s&&n>=this.NEPALI_CALENDAR_DATA[A][0]?56:57);for(9!==e&&(a=n,s--);9!==s;)s<=0&&(s=12,A--),a+=this.NEPALI_CALENDAR_DATA[A][s],s--;return 9===e?(a+=n-this.NEPALI_CALENDAR_DATA[A][0])<0&&(a+=o.daysInYear(l)):a+=this.NEPALI_CALENDAR_DATA[A][9]-this.NEPALI_CALENDAR_DATA[A][0],o.newDate(l,1,1).add(a,"d").toJD()},fromJD:function(t){var e=r.instance().fromJD(t),n=e.year(),i=e.dayOfYear(),o=n+56;this._createMissingCalendarData(o);for(var a=9,s=this.NEPALI_CALENDAR_DATA[o][0],A=this.NEPALI_CALENDAR_DATA[o][a]-s+1;i>A;)++a>12&&(a=1,o++),A+=this.NEPALI_CALENDAR_DATA[o][a];var l=this.NEPALI_CALENDAR_DATA[o][a]-(A-i);return this.newDate(o,a,l)},_createMissingCalendarData:function(t){var e=this.daysPerMonth.slice(0);e.unshift(17);for(var n=t-1;n<t+2;n++)void 0===this.NEPALI_CALENDAR_DATA[n]&&(this.NEPALI_CALENDAR_DATA[n]=e)},NEPALI_CALENDAR_DATA:{1970:[18,31,31,32,31,31,31,30,29,30,29,30,30],1971:[18,31,31,32,31,32,30,30,29,30,29,30,30],1972:[17,31,32,31,32,31,30,30,30,29,29,30,30],1973:[19,30,32,31,32,31,30,30,30,29,30,29,31],1974:[19,31,31,32,30,31,31,30,29,30,29,30,30],1975:[18,31,31,32,32,30,31,30,29,30,29,30,30],1976:[17,31,32,31,32,31,30,30,30,29,29,30,31],1977:[18,31,32,31,32,31,31,29,30,29,30,29,31],1978:[18,31,31,32,31,31,31,30,29,30,29,30,30],1979:[18,31,31,32,32,31,30,30,29,30,29,30,30],1980:[17,31,32,31,32,31,30,30,30,29,29,30,31],1981:[18,31,31,31,32,31,31,29,30,30,29,30,30],1982:[18,31,31,32,31,31,31,30,29,30,29,30,30],1983:[18,31,31,32,32,31,30,30,29,30,29,30,30],1984:[17,31,32,31,32,31,30,30,30,29,29,30,31],1985:[18,31,31,31,32,31,31,29,30,30,29,30,30],1986:[18,31,31,32,31,31,31,30,29,30,29,30,30],1987:[18,31,32,31,32,31,30,30,29,30,29,30,30],1988:[17,31,32,31,32,31,30,30,30,29,29,30,31],1989:[18,31,31,31,32,31,31,30,29,30,29,30,30],1990:[18,31,31,32,31,31,31,30,29,30,29,30,30],1991:[18,31,32,31,32,31,30,30,29,30,29,30,30],1992:[17,31,32,31,32,31,30,30,30,29,30,29,31],1993:[18,31,31,31,32,31,31,30,29,30,29,30,30],1994:[18,31,31,32,31,31,31,30,29,30,29,30,30],1995:[17,31,32,31,32,31,30,30,30,29,29,30,30],1996:[17,31,32,31,32,31,30,30,30,29,30,29,31],1997:[18,31,31,32,31,31,31,30,29,30,29,30,30],1998:[18,31,31,32,31,31,31,30,29,30,29,30,30],1999:[17,31,32,31,32,31,30,30,30,29,29,30,31],2e3:[17,30,32,31,32,31,30,30,30,29,30,29,31],2001:[18,31,31,32,31,31,31,30,29,30,29,30,30],2002:[18,31,31,32,32,31,30,30,29,30,29,30,30],2003:[17,31,32,31,32,31,30,30,30,29,29,30,31],2004:[17,30,32,31,32,31,30,30,30,29,30,29,31],2005:[18,31,31,32,31,31,31,30,29,30,29,30,30],2006:[18,31,31,32,32,31,30,30,29,30,29,30,30],2007:[17,31,32,31,32,31,30,30,30,29,29,30,31],2008:[17,31,31,31,32,31,31,29,30,30,29,29,31],2009:[18,31,31,32,31,31,31,30,29,30,29,30,30],2010:[18,31,31,32,32,31,30,30,29,30,29,30,30],2011:[17,31,32,31,32,31,30,30,30,29,29,30,31],2012:[17,31,31,31,32,31,31,29,30,30,29,30,30],2013:[18,31,31,32,31,31,31,30,29,30,29,30,30],2014:[18,31,31,32,32,31,30,30,29,30,29,30,30],2015:[17,31,32,31,32,31,30,30,30,29,29,30,31],2016:[17,31,31,31,32,31,31,29,30,30,29,30,30],2017:[18,31,31,32,31,31,31,30,29,30,29,30,30],2018:[18,31,32,31,32,31,30,30,29,30,29,30,30],2019:[17,31,32,31,32,31,30,30,30,29,30,29,31],2020:[17,31,31,31,32,31,31,30,29,30,29,30,30],2021:[18,31,31,32,31,31,31,30,29,30,29,30,30],2022:[17,31,32,31,32,31,30,30,30,29,29,30,30],2023:[17,31,32,31,32,31,30,30,30,29,30,29,31],2024:[17,31,31,31,32,31,31,30,29,30,29,30,30],2025:[18,31,31,32,31,31,31,30,29,30,29,30,30],2026:[17,31,32,31,32,31,30,30,30,29,29,30,31],2027:[17,30,32,31,32,31,30,30,30,29,30,29,31],2028:[17,31,31,32,31,31,31,30,29,30,29,30,30],2029:[18,31,31,32,31,32,30,30,29,30,29,30,30],2030:[17,31,32,31,32,31,30,30,30,30,30,30,31],2031:[17,31,32,31,32,31,31,31,31,31,31,31,31],2032:[17,32,32,32,32,32,32,32,32,32,32,32,32],2033:[18,31,31,32,32,31,30,30,29,30,29,30,30],2034:[17,31,32,31,32,31,30,30,30,29,29,30,31],2035:[17,30,32,31,32,31,31,29,30,30,29,29,31],2036:[17,31,31,32,31,31,31,30,29,30,29,30,30],2037:[18,31,31,32,32,31,30,30,29,30,29,30,30],2038:[17,31,32,31,32,31,30,30,30,29,29,30,31],2039:[17,31,31,31,32,31,31,29,30,30,29,30,30],2040:[17,31,31,32,31,31,31,30,29,30,29,30,30],2041:[18,31,31,32,32,31,30,30,29,30,29,30,30],2042:[17,31,32,31,32,31,30,30,30,29,29,30,31],2043:[17,31,31,31,32,31,31,29,30,30,29,30,30],2044:[17,31,31,32,31,31,31,30,29,30,29,30,30],2045:[18,31,32,31,32,31,30,30,29,30,29,30,30],2046:[17,31,32,31,32,31,30,30,30,29,29,30,31],2047:[17,31,31,31,32,31,31,30,29,30,29,30,30],2048:[17,31,31,32,31,31,31,30,29,30,29,30,30],2049:[17,31,32,31,32,31,30,30,30,29,29,30,30],2050:[17,31,32,31,32,31,30,30,30,29,30,29,31],2051:[17,31,31,31,32,31,31,30,29,30,29,30,30],2052:[17,31,31,32,31,31,31,30,29,30,29,30,30],2053:[17,31,32,31,32,31,30,30,30,29,29,30,30],2054:[17,31,32,31,32,31,30,30,30,29,30,29,31],2055:[17,31,31,32,31,31,31,30,29,30,30,29,30],2056:[17,31,31,32,31,32,30,30,29,30,29,30,30],2057:[17,31,32,31,32,31,30,30,30,29,29,30,31],2058:[17,30,32,31,32,31,30,30,30,29,30,29,31],2059:[17,31,31,32,31,31,31,30,29,30,29,30,30],2060:[17,31,31,32,32,31,30,30,29,30,29,30,30],2061:[17,31,32,31,32,31,30,30,30,29,29,30,31],2062:[17,30,32,31,32,31,31,29,30,29,30,29,31],2063:[17,31,31,32,31,31,31,30,29,30,29,30,30],2064:[17,31,31,32,32,31,30,30,29,30,29,30,30],2065:[17,31,32,31,32,31,30,30,30,29,29,30,31],2066:[17,31,31,31,32,31,31,29,30,30,29,29,31],2067:[17,31,31,32,31,31,31,30,29,30,29,30,30],2068:[17,31,31,32,32,31,30,30,29,30,29,30,30],2069:[17,31,32,31,32,31,30,30,30,29,29,30,31],2070:[17,31,31,31,32,31,31,29,30,30,29,30,30],2071:[17,31,31,32,31,31,31,30,29,30,29,30,30],2072:[17,31,32,31,32,31,30,30,29,30,29,30,30],2073:[17,31,32,31,32,31,30,30,30,29,29,30,31],2074:[17,31,31,31,32,31,31,30,29,30,29,30,30],2075:[17,31,31,32,31,31,31,30,29,30,29,30,30],2076:[16,31,32,31,32,31,30,30,30,29,29,30,30],2077:[17,31,32,31,32,31,30,30,30,29,30,29,31],2078:[17,31,31,31,32,31,31,30,29,30,29,30,30],2079:[17,31,31,32,31,31,31,30,29,30,29,30,30],2080:[16,31,32,31,32,31,30,30,30,29,29,30,30],2081:[17,31,31,32,32,31,30,30,30,29,30,30,30],2082:[17,31,32,31,32,31,30,30,30,29,30,30,30],2083:[17,31,31,32,31,31,30,30,30,29,30,30,30],2084:[17,31,31,32,31,31,30,30,30,29,30,30,30],2085:[17,31,32,31,32,31,31,30,30,29,30,30,30],2086:[17,31,32,31,32,31,30,30,30,29,30,30,30],2087:[16,31,31,32,31,31,31,30,30,29,30,30,30],2088:[16,30,31,32,32,30,31,30,30,29,30,30,30],2089:[17,31,32,31,32,31,30,30,30,29,30,30,30],2090:[17,31,32,31,32,31,30,30,30,29,30,30,30],2091:[16,31,31,32,31,31,31,30,30,29,30,30,30],2092:[16,31,31,32,32,31,30,30,30,29,30,30,30],2093:[17,31,32,31,32,31,30,30,30,29,30,30,30],2094:[17,31,31,32,31,31,30,30,30,29,30,30,30],2095:[17,31,31,32,31,31,31,30,29,30,30,30,30],2096:[17,30,31,32,32,31,30,30,29,30,29,30,30],2097:[17,31,32,31,32,31,30,30,30,29,30,30,30],2098:[17,31,31,32,31,31,31,29,30,29,30,30,31],2099:[17,31,31,32,31,31,31,30,29,29,30,30,30],2100:[17,31,32,31,32,30,31,30,29,30,29,30,30]}}),r.calendars.nepali=o},{"../main":571,"object-assign":454}],567:[function(t,e,n){var r=t("../main"),i=t("object-assign");function o(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}function a(t,e){return t-e*Math.floor(t/e)}o.prototype=new r.baseCalendar,i(o.prototype,{name:"Persian",jdEpoch:1948320.5,daysPerMonth:[31,31,31,31,31,31,30,30,30,30,30,29],hasYearZero:!1,minMonth:1,firstMonth:1,minDay:1,regionalOptions:{"":{name:"Persian",epochs:["BP","AP"],monthNames:["Farvardin","Ordibehesht","Khordad","Tir","Mordad","Shahrivar","Mehr","Aban","Azar","Day","Bahman","Esfand"],monthNamesShort:["Far","Ord","Kho","Tir","Mor","Sha","Meh","Aba","Aza","Day","Bah","Esf"],dayNames:["Yekshambe","Doshambe","Seshambe","Chæharshambe","Panjshambe","Jom'e","Shambe"],dayNamesShort:["Yek","Do","Se","Chæ","Panj","Jom","Sha"],dayNamesMin:["Ye","Do","Se","Ch","Pa","Jo","Sh"],digits:null,dateFormat:"yyyy/mm/dd",firstDay:6,isRTL:!1}},leapYear:function(t){var e=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear);return 682*((e.year()-(e.year()>0?474:473))%2820+474+38)%2816<682},weekOfYear:function(t,e,n){var r=this.newDate(t,e,n);return r.add(-(r.dayOfWeek()+1)%7,"d"),Math.floor((r.dayOfYear()-1)/7)+1},daysInMonth:function(t,e){var n=this._validate(t,e,this.minDay,r.local.invalidMonth);return this.daysPerMonth[n.month()-1]+(12===n.month()&&this.leapYear(n.year())?1:0)},weekDay:function(t,e,n){return 5!==this.dayOfWeek(t,e,n)},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);t=i.year(),e=i.month(),n=i.day();var o=t-(t>=0?474:473),s=474+a(o,2820);return n+(e<=7?31*(e-1):30*(e-1)+6)+Math.floor((682*s-110)/2816)+365*(s-1)+1029983*Math.floor(o/2820)+this.jdEpoch-1},fromJD:function(t){var e=(t=Math.floor(t)+.5)-this.toJD(475,1,1),n=Math.floor(e/1029983),r=a(e,1029983),i=2820;if(1029982!==r){var o=Math.floor(r/366),s=a(r,366);i=Math.floor((2134*o+2816*s+2815)/1028522)+o+1}var A=i+2820*n+474;A=A<=0?A-1:A;var l=t-this.toJD(A,1,1)+1,c=l<=186?Math.ceil(l/31):Math.ceil((l-6)/30),u=t-this.toJD(A,c,1)+1;return this.newDate(A,c,u)}}),r.calendars.persian=o,r.calendars.jalali=o},{"../main":571,"object-assign":454}],568:[function(t,e,n){var r=t("../main"),i=t("object-assign"),o=r.instance();function a(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}a.prototype=new r.baseCalendar,i(a.prototype,{name:"Taiwan",jdEpoch:2419402.5,yearsOffset:1911,daysPerMonth:[31,28,31,30,31,30,31,31,30,31,30,31],hasYearZero:!1,minMonth:1,firstMonth:1,minDay:1,regionalOptions:{"":{name:"Taiwan",epochs:["BROC","ROC"],monthNames:["January","February","March","April","May","June","July","August","September","October","November","December"],monthNamesShort:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],dayNames:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],dayNamesShort:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],dayNamesMin:["Su","Mo","Tu","We","Th","Fr","Sa"],digits:null,dateFormat:"yyyy/mm/dd",firstDay:1,isRTL:!1}},leapYear:function(t){var e=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear);return t=this._t2gYear(e.year()),o.leapYear(t)},weekOfYear:function(t,e,n){var i=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear);return t=this._t2gYear(i.year()),o.weekOfYear(t,i.month(),i.day())},daysInMonth:function(t,e){var n=this._validate(t,e,this.minDay,r.local.invalidMonth);return this.daysPerMonth[n.month()-1]+(2===n.month()&&this.leapYear(n.year())?1:0)},weekDay:function(t,e,n){return(this.dayOfWeek(t,e,n)||7)<6},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);return t=this._t2gYear(i.year()),o.toJD(t,i.month(),i.day())},fromJD:function(t){var e=o.fromJD(t),n=this._g2tYear(e.year());return this.newDate(n,e.month(),e.day())},_t2gYear:function(t){return t+this.yearsOffset+(t>=-this.yearsOffset&&t<=-1?1:0)},_g2tYear:function(t){return t-this.yearsOffset-(t>=1&&t<=this.yearsOffset?1:0)}}),r.calendars.taiwan=a},{"../main":571,"object-assign":454}],569:[function(t,e,n){var r=t("../main"),i=t("object-assign"),o=r.instance();function a(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}a.prototype=new r.baseCalendar,i(a.prototype,{name:"Thai",jdEpoch:1523098.5,yearsOffset:543,daysPerMonth:[31,28,31,30,31,30,31,31,30,31,30,31],hasYearZero:!1,minMonth:1,firstMonth:1,minDay:1,regionalOptions:{"":{name:"Thai",epochs:["BBE","BE"],monthNames:["January","February","March","April","May","June","July","August","September","October","November","December"],monthNamesShort:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],dayNames:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],dayNamesShort:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],dayNamesMin:["Su","Mo","Tu","We","Th","Fr","Sa"],digits:null,dateFormat:"dd/mm/yyyy",firstDay:0,isRTL:!1}},leapYear:function(t){var e=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear);return t=this._t2gYear(e.year()),o.leapYear(t)},weekOfYear:function(t,e,n){var i=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear);return t=this._t2gYear(i.year()),o.weekOfYear(t,i.month(),i.day())},daysInMonth:function(t,e){var n=this._validate(t,e,this.minDay,r.local.invalidMonth);return this.daysPerMonth[n.month()-1]+(2===n.month()&&this.leapYear(n.year())?1:0)},weekDay:function(t,e,n){return(this.dayOfWeek(t,e,n)||7)<6},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate);return t=this._t2gYear(i.year()),o.toJD(t,i.month(),i.day())},fromJD:function(t){var e=o.fromJD(t),n=this._g2tYear(e.year());return this.newDate(n,e.month(),e.day())},_t2gYear:function(t){return t-this.yearsOffset-(t>=1&&t<=this.yearsOffset?1:0)},_g2tYear:function(t){return t+this.yearsOffset+(t>=-this.yearsOffset&&t<=-1?1:0)}}),r.calendars.thai=a},{"../main":571,"object-assign":454}],570:[function(t,e,n){var r=t("../main"),i=t("object-assign");function o(t){this.local=this.regionalOptions[t||""]||this.regionalOptions[""]}o.prototype=new r.baseCalendar,i(o.prototype,{name:"UmmAlQura",hasYearZero:!1,minMonth:1,firstMonth:1,minDay:1,regionalOptions:{"":{name:"Umm al-Qura",epochs:["BH","AH"],monthNames:["Al-Muharram","Safar","Rabi' al-awwal","Rabi' Al-Thani","Jumada Al-Awwal","Jumada Al-Thani","Rajab","Sha'aban","Ramadan","Shawwal","Dhu al-Qi'dah","Dhu al-Hijjah"],monthNamesShort:["Muh","Saf","Rab1","Rab2","Jum1","Jum2","Raj","Sha'","Ram","Shaw","DhuQ","DhuH"],dayNames:["Yawm al-Ahad","Yawm al-Ithnain","Yawm al-Thalāthā’","Yawm al-Arba‘ā’","Yawm al-Khamīs","Yawm al-Jum‘a","Yawm al-Sabt"],dayNamesMin:["Ah","Ith","Th","Ar","Kh","Ju","Sa"],digits:null,dateFormat:"yyyy/mm/dd",firstDay:6,isRTL:!0}},leapYear:function(t){var e=this._validate(t,this.minMonth,this.minDay,r.local.invalidYear);return 355===this.daysInYear(e.year())},weekOfYear:function(t,e,n){var r=this.newDate(t,e,n);return r.add(-r.dayOfWeek(),"d"),Math.floor((r.dayOfYear()-1)/7)+1},daysInYear:function(t){for(var e=0,n=1;n<=12;n++)e+=this.daysInMonth(t,n);return e},daysInMonth:function(t,e){for(var n=this._validate(t,e,this.minDay,r.local.invalidMonth).toJD()-24e5+.5,i=0,o=0;o<a.length;o++){if(a[o]>n)return a[i]-a[i-1];i++}return 30},weekDay:function(t,e,n){return 5!==this.dayOfWeek(t,e,n)},toJD:function(t,e,n){var i=this._validate(t,e,n,r.local.invalidDate),o=12*(i.year()-1)+i.month()-15292;return i.day()+a[o-1]-1+24e5-.5},fromJD:function(t){for(var e=t-24e5+.5,n=0,r=0;r<a.length&&!(a[r]>e);r++)n++;var i=n+15292,o=Math.floor((i-1)/12),s=o+1,A=i-12*o,l=e-a[n-1]+1;return this.newDate(s,A,l)},isValid:function(t,e,n){var i=r.baseCalendar.prototype.isValid.apply(this,arguments);return i&&(i=(t=null!=t.year?t.year:t)>=1276&&t<=1500),i},_validate:function(t,e,n,i){var o=r.baseCalendar.prototype._validate.apply(this,arguments);if(o.year<1276||o.year>1500)throw i.replace(/\{0\}/,this.local.name);return o}}),r.calendars.ummalqura=o;var a=[20,50,79,109,138,168,197,227,256,286,315,345,374,404,433,463,492,522,551,581,611,641,670,700,729,759,788,818,847,877,906,936,965,995,1024,1054,1083,1113,1142,1172,1201,1231,1260,1290,1320,1350,1379,1409,1438,1468,1497,1527,1556,1586,1615,1645,1674,1704,1733,1763,1792,1822,1851,1881,1910,1940,1969,1999,2028,2058,2087,2117,2146,2176,2205,2235,2264,2294,2323,2353,2383,2413,2442,2472,2501,2531,2560,2590,2619,2649,2678,2708,2737,2767,2796,2826,2855,2885,2914,2944,2973,3003,3032,3062,3091,3121,3150,3180,3209,3239,3268,3298,3327,3357,3386,3416,3446,3476,3505,3535,3564,3594,3623,3653,3682,3712,3741,3771,3800,3830,3859,3889,3918,3948,3977,4007,4036,4066,4095,4125,4155,4185,4214,4244,4273,4303,4332,4362,4391,4421,4450,4480,4509,4539,4568,4598,4627,4657,4686,4716,4745,4775,4804,4834,4863,4893,4922,4952,4981,5011,5040,5070,5099,5129,5158,5188,5218,5248,5277,5307,5336,5366,5395,5425,5454,5484,5513,5543,5572,5602,5631,5661,5690,5720,5749,5779,5808,5838,5867,5897,5926,5956,5985,6015,6044,6074,6103,6133,6162,6192,6221,6251,6281,6311,6340,6370,6399,6429,6458,6488,6517,6547,6576,6606,6635,6665,6694,6724,6753,6783,6812,6842,6871,6901,6930,6960,6989,7019,7048,7078,7107,7137,7166,7196,7225,7255,7284,7314,7344,7374,7403,7433,7462,7492,7521,7551,7580,7610,7639,7669,7698,7728,7757,7787,7816,7846,7875,7905,7934,7964,7993,8023,8053,8083,8112,8142,8171,8201,8230,8260,8289,8319,8348,8378,8407,8437,8466,8496,8525,8555,8584,8614,8643,8673,8702,8732,8761,8791,8821,8850,8880,8909,8938,8968,8997,9027,9056,9086,9115,9145,9175,9205,9234,9264,9293,9322,9352,9381,9410,9440,9470,9499,9529,9559,9589,9618,9648,9677,9706,9736,9765,9794,9824,9853,9883,9913,9943,9972,10002,10032,10061,10090,10120,10149,10178,10208,10237,10267,10297,10326,10356,10386,10415,10445,10474,10504,10533,10562,10592,10621,10651,10680,10710,10740,10770,10799,10829,10858,10888,10917,10947,10976,11005,11035,11064,11094,11124,11153,11183,11213,11242,11272,11301,11331,11360,11389,11419,11448,11478,11507,11537,11567,11596,11626,11655,11685,11715,11744,11774,11803,11832,11862,11891,11921,11950,11980,12010,12039,12069,12099,12128,12158,12187,12216,12246,12275,12304,12334,12364,12393,12423,12453,12483,12512,12542,12571,12600,12630,12659,12688,12718,12747,12777,12807,12837,12866,12896,12926,12955,12984,13014,13043,13072,13102,13131,13161,13191,13220,13250,13280,13310,13339,13368,13398,13427,13456,13486,13515,13545,13574,13604,13634,13664,13693,13723,13752,13782,13811,13840,13870,13899,13929,13958,13988,14018,14047,14077,14107,14136,14166,14195,14224,14254,14283,14313,14342,14372,14401,14431,14461,14490,14520,14550,14579,14609,14638,14667,14697,14726,14756,14785,14815,14844,14874,14904,14933,14963,14993,15021,15051,15081,15110,15140,15169,15199,15228,15258,15287,15317,15347,15377,15406,15436,15465,15494,15524,15553,15582,15612,15641,15671,15701,15731,15760,15790,15820,15849,15878,15908,15937,15966,15996,16025,16055,16085,16114,16144,16174,16204,16233,16262,16292,16321,16350,16380,16409,16439,16468,16498,16528,16558,16587,16617,16646,16676,16705,16734,16764,16793,16823,16852,16882,16912,16941,16971,17001,17030,17060,17089,17118,17148,17177,17207,17236,17266,17295,17325,17355,17384,17414,17444,17473,17502,17532,17561,17591,17620,17650,17679,17709,17738,17768,17798,17827,17857,17886,17916,17945,17975,18004,18034,18063,18093,18122,18152,18181,18211,18241,18270,18300,18330,18359,18388,18418,18447,18476,18506,18535,18565,18595,18625,18654,18684,18714,18743,18772,18802,18831,18860,18890,18919,18949,18979,19008,19038,19068,19098,19127,19156,19186,19215,19244,19274,19303,19333,19362,19392,19422,19452,19481,19511,19540,19570,19599,19628,19658,19687,19717,19746,19776,19806,19836,19865,19895,19924,19954,19983,20012,20042,20071,20101,20130,20160,20190,20219,20249,20279,20308,20338,20367,20396,20426,20455,20485,20514,20544,20573,20603,20633,20662,20692,20721,20751,20780,20810,20839,20869,20898,20928,20957,20987,21016,21046,21076,21105,21135,21164,21194,21223,21253,21282,21312,21341,21371,21400,21430,21459,21489,21519,21548,21578,21607,21637,21666,21696,21725,21754,21784,21813,21843,21873,21902,21932,21962,21991,22021,22050,22080,22109,22138,22168,22197,22227,22256,22286,22316,22346,22375,22405,22434,22464,22493,22522,22552,22581,22611,22640,22670,22700,22730,22759,22789,22818,22848,22877,22906,22936,22965,22994,23024,23054,23083,23113,23143,23173,23202,23232,23261,23290,23320,23349,23379,23408,23438,23467,23497,23527,23556,23586,23616,23645,23674,23704,23733,23763,23792,23822,23851,23881,23910,23940,23970,23999,24029,24058,24088,24117,24147,24176,24206,24235,24265,24294,24324,24353,24383,24413,24442,24472,24501,24531,24560,24590,24619,24648,24678,24707,24737,24767,24796,24826,24856,24885,24915,24944,24974,25003,25032,25062,25091,25121,25150,25180,25210,25240,25269,25299,25328,25358,25387,25416,25446,25475,25505,25534,25564,25594,25624,25653,25683,25712,25742,25771,25800,25830,25859,25888,25918,25948,25977,26007,26037,26067,26096,26126,26155,26184,26214,26243,26272,26302,26332,26361,26391,26421,26451,26480,26510,26539,26568,26598,26627,26656,26686,26715,26745,26775,26805,26834,26864,26893,26923,26952,26982,27011,27041,27070,27099,27129,27159,27188,27218,27248,27277,27307,27336,27366,27395,27425,27454,27484,27513,27542,27572,27602,27631,27661,27691,27720,27750,27779,27809,27838,27868,27897,27926,27956,27985,28015,28045,28074,28104,28134,28163,28193,28222,28252,28281,28310,28340,28369,28399,28428,28458,28488,28517,28547,28577,28607,28636,28665,28695,28724,28754,28783,28813,28843,28872,28901,28931,28960,28990,29019,29049,29078,29108,29137,29167,29196,29226,29255,29285,29315,29345,29375,29404,29434,29463,29492,29522,29551,29580,29610,29640,29669,29699,29729,29759,29788,29818,29847,29876,29906,29935,29964,29994,30023,30053,30082,30112,30141,30171,30200,30230,30259,30289,30318,30348,30378,30408,30437,30467,30496,30526,30555,30585,30614,30644,30673,30703,30732,30762,30791,30821,30850,30880,30909,30939,30968,30998,31027,31057,31086,31116,31145,31175,31204,31234,31263,31293,31322,31352,31381,31411,31441,31471,31500,31530,31559,31589,31618,31648,31676,31706,31736,31766,31795,31825,31854,31884,31913,31943,31972,32002,32031,32061,32090,32120,32150,32180,32209,32239,32268,32298,32327,32357,32386,32416,32445,32475,32504,32534,32563,32593,32622,32652,32681,32711,32740,32770,32799,32829,32858,32888,32917,32947,32976,33006,33035,33065,33094,33124,33153,33183,33213,33243,33272,33302,33331,33361,33390,33420,33450,33479,33509,33539,33568,33598,33627,33657,33686,33716,33745,33775,33804,33834,33863,33893,33922,33952,33981,34011,34040,34069,34099,34128,34158,34187,34217,34247,34277,34306,34336,34365,34395,34424,34454,34483,34512,34542,34571,34601,34631,34660,34690,34719,34749,34778,34808,34837,34867,34896,34926,34955,34985,35015,35044,35074,35103,35133,35162,35192,35222,35251,35280,35310,35340,35370,35399,35429,35458,35488,35517,35547,35576,35605,35635,35665,35694,35723,35753,35782,35811,35841,35871,35901,35930,35960,35989,36019,36048,36078,36107,36136,36166,36195,36225,36254,36284,36314,36343,36373,36403,36433,36462,36492,36521,36551,36580,36610,36639,36669,36698,36728,36757,36786,36816,36845,36875,36904,36934,36963,36993,37022,37052,37081,37111,37141,37170,37200,37229,37259,37288,37318,37347,37377,37406,37436,37465,37495,37524,37554,37584,37613,37643,37672,37701,37731,37760,37790,37819,37849,37878,37908,37938,37967,37997,38027,38056,38085,38115,38144,38174,38203,38233,38262,38292,38322,38351,38381,38410,38440,38469,38499,38528,38558,38587,38617,38646,38676,38705,38735,38764,38794,38823,38853,38882,38912,38941,38971,39001,39030,39059,39089,39118,39148,39178,39208,39237,39267,39297,39326,39355,39385,39414,39444,39473,39503,39532,39562,39592,39621,39650,39680,39709,39739,39768,39798,39827,39857,39886,39916,39946,39975,40005,40035,40064,40094,40123,40153,40182,40212,40241,40271,40300,40330,40359,40389,40418,40448,40477,40507,40536,40566,40595,40625,40655,40685,40714,40744,40773,40803,40832,40862,40892,40921,40951,40980,41009,41039,41068,41098,41127,41157,41186,41216,41245,41275,41304,41334,41364,41393,41422,41452,41481,41511,41540,41570,41599,41629,41658,41688,41718,41748,41777,41807,41836,41865,41894,41924,41953,41983,42012,42042,42072,42102,42131,42161,42190,42220,42249,42279,42308,42337,42367,42397,42426,42456,42485,42515,42545,42574,42604,42633,42662,42692,42721,42751,42780,42810,42839,42869,42899,42929,42958,42988,43017,43046,43076,43105,43135,43164,43194,43223,43253,43283,43312,43342,43371,43401,43430,43460,43489,43519,43548,43578,43607,43637,43666,43696,43726,43755,43785,43814,43844,43873,43903,43932,43962,43991,44021,44050,44080,44109,44139,44169,44198,44228,44258,44287,44317,44346,44375,44405,44434,44464,44493,44523,44553,44582,44612,44641,44671,44700,44730,44759,44788,44818,44847,44877,44906,44936,44966,44996,45025,45055,45084,45114,45143,45172,45202,45231,45261,45290,45320,45350,45380,45409,45439,45468,45498,45527,45556,45586,45615,45644,45674,45704,45733,45763,45793,45823,45852,45882,45911,45940,45970,45999,46028,46058,46088,46117,46147,46177,46206,46236,46265,46295,46324,46354,46383,46413,46442,46472,46501,46531,46560,46590,46620,46649,46679,46708,46738,46767,46797,46826,46856,46885,46915,46944,46974,47003,47033,47063,47092,47122,47151,47181,47210,47240,47269,47298,47328,47357,47387,47417,47446,47476,47506,47535,47565,47594,47624,47653,47682,47712,47741,47771,47800,47830,47860,47890,47919,47949,47978,48008,48037,48066,48096,48125,48155,48184,48214,48244,48273,48303,48333,48362,48392,48421,48450,48480,48509,48538,48568,48598,48627,48657,48687,48717,48746,48776,48805,48834,48864,48893,48922,48952,48982,49011,49041,49071,49100,49130,49160,49189,49218,49248,49277,49306,49336,49365,49395,49425,49455,49484,49514,49543,49573,49602,49632,49661,49690,49720,49749,49779,49809,49838,49868,49898,49927,49957,49986,50016,50045,50075,50104,50133,50163,50192,50222,50252,50281,50311,50340,50370,50400,50429,50459,50488,50518,50547,50576,50606,50635,50665,50694,50724,50754,50784,50813,50843,50872,50902,50931,50960,50990,51019,51049,51078,51108,51138,51167,51197,51227,51256,51286,51315,51345,51374,51403,51433,51462,51492,51522,51552,51582,51611,51641,51670,51699,51729,51758,51787,51816,51846,51876,51906,51936,51965,51995,52025,52054,52083,52113,52142,52171,52200,52230,52260,52290,52319,52349,52379,52408,52438,52467,52497,52526,52555,52585,52614,52644,52673,52703,52733,52762,52792,52822,52851,52881,52910,52939,52969,52998,53028,53057,53087,53116,53146,53176,53205,53235,53264,53294,53324,53353,53383,53412,53441,53471,53500,53530,53559,53589,53619,53648,53678,53708,53737,53767,53796,53825,53855,53884,53913,53943,53973,54003,54032,54062,54092,54121,54151,54180,54209,54239,54268,54297,54327,54357,54387,54416,54446,54476,54505,54535,54564,54593,54623,54652,54681,54711,54741,54770,54800,54830,54859,54889,54919,54948,54977,55007,55036,55066,55095,55125,55154,55184,55213,55243,55273,55302,55332,55361,55391,55420,55450,55479,55508,55538,55567,55597,55627,55657,55686,55716,55745,55775,55804,55834,55863,55892,55922,55951,55981,56011,56040,56070,56100,56129,56159,56188,56218,56247,56276,56306,56335,56365,56394,56424,56454,56483,56513,56543,56572,56601,56631,56660,56690,56719,56749,56778,56808,56837,56867,56897,56926,56956,56985,57015,57044,57074,57103,57133,57162,57192,57221,57251,57280,57310,57340,57369,57399,57429,57458,57487,57517,57546,57576,57605,57634,57664,57694,57723,57753,57783,57813,57842,57871,57901,57930,57959,57989,58018,58048,58077,58107,58137,58167,58196,58226,58255,58285,58314,58343,58373,58402,58432,58461,58491,58521,58551,58580,58610,58639,58669,58698,58727,58757,58786,58816,58845,58875,58905,58934,58964,58994,59023,59053,59082,59111,59141,59170,59200,59229,59259,59288,59318,59348,59377,59407,59436,59466,59495,59525,59554,59584,59613,59643,59672,59702,59731,59761,59791,59820,59850,59879,59909,59939,59968,59997,60027,60056,60086,60115,60145,60174,60204,60234,60264,60293,60323,60352,60381,60411,60440,60469,60499,60528,60558,60588,60618,60648,60677,60707,60736,60765,60795,60824,60853,60883,60912,60942,60972,61002,61031,61061,61090,61120,61149,61179,61208,61237,61267,61296,61326,61356,61385,61415,61445,61474,61504,61533,61563,61592,61621,61651,61680,61710,61739,61769,61799,61828,61858,61888,61917,61947,61976,62006,62035,62064,62094,62123,62153,62182,62212,62242,62271,62301,62331,62360,62390,62419,62448,62478,62507,62537,62566,62596,62625,62655,62685,62715,62744,62774,62803,62832,62862,62891,62921,62950,62980,63009,63039,63069,63099,63128,63157,63187,63216,63246,63275,63305,63334,63363,63393,63423,63453,63482,63512,63541,63571,63600,63630,63659,63689,63718,63747,63777,63807,63836,63866,63895,63925,63955,63984,64014,64043,64073,64102,64131,64161,64190,64220,64249,64279,64309,64339,64368,64398,64427,64457,64486,64515,64545,64574,64603,64633,64663,64692,64722,64752,64782,64811,64841,64870,64899,64929,64958,64987,65017,65047,65076,65106,65136,65166,65195,65225,65254,65283,65313,65342,65371,65401,65431,65460,65490,65520,65549,65579,65608,65638,65667,65697,65726,65755,65785,65815,65844,65874,65903,65933,65963,65992,66022,66051,66081,66110,66140,66169,66199,66228,66258,66287,66317,66346,66376,66405,66435,66465,66494,66524,66553,66583,66612,66641,66671,66700,66730,66760,66789,66819,66849,66878,66908,66937,66967,66996,67025,67055,67084,67114,67143,67173,67203,67233,67262,67292,67321,67351,67380,67409,67439,67468,67497,67527,67557,67587,67617,67646,67676,67705,67735,67764,67793,67823,67852,67882,67911,67941,67971,68e3,68030,68060,68089,68119,68148,68177,68207,68236,68266,68295,68325,68354,68384,68414,68443,68473,68502,68532,68561,68591,68620,68650,68679,68708,68738,68768,68797,68827,68857,68886,68916,68946,68975,69004,69034,69063,69092,69122,69152,69181,69211,69240,69270,69300,69330,69359,69388,69418,69447,69476,69506,69535,69565,69595,69624,69654,69684,69713,69743,69772,69802,69831,69861,69890,69919,69949,69978,70008,70038,70067,70097,70126,70156,70186,70215,70245,70274,70303,70333,70362,70392,70421,70451,70481,70510,70540,70570,70599,70629,70658,70687,70717,70746,70776,70805,70835,70864,70894,70924,70954,70983,71013,71042,71071,71101,71130,71159,71189,71218,71248,71278,71308,71337,71367,71397,71426,71455,71485,71514,71543,71573,71602,71632,71662,71691,71721,71751,71781,71810,71839,71869,71898,71927,71957,71986,72016,72046,72075,72105,72135,72164,72194,72223,72253,72282,72311,72341,72370,72400,72429,72459,72489,72518,72548,72577,72607,72637,72666,72695,72725,72754,72784,72813,72843,72872,72902,72931,72961,72991,73020,73050,73080,73109,73139,73168,73197,73227,73256,73286,73315,73345,73375,73404,73434,73464,73493,73523,73552,73581,73611,73640,73669,73699,73729,73758,73788,73818,73848,73877,73907,73936,73965,73995,74024,74053,74083,74113,74142,74172,74202,74231,74261,74291,74320,74349,74379,74408,74437,74467,74497,74526,74556,74586,74615,74645,74675,74704,74733,74763,74792,74822,74851,74881,74910,74940,74969,74999,75029,75058,75088,75117,75147,75176,75206,75235,75264,75294,75323,75353,75383,75412,75442,75472,75501,75531,75560,75590,75619,75648,75678,75707,75737,75766,75796,75826,75856,75885,75915,75944,75974,76003,76032,76062,76091,76121,76150,76180,76210,76239,76269,76299,76328,76358,76387,76416,76446,76475,76505,76534,76564,76593,76623,76653,76682,76712,76741,76771,76801,76830,76859,76889,76918,76948,76977,77007,77036,77066,77096,77125,77155,77185,77214,77243,77273,77302,77332,77361,77390,77420,77450,77479,77509,77539,77569,77598,77627,77657,77686,77715,77745,77774,77804,77833,77863,77893,77923,77952,77982,78011,78041,78070,78099,78129,78158,78188,78217,78247,78277,78307,78336,78366,78395,78425,78454,78483,78513,78542,78572,78601,78631,78661,78690,78720,78750,78779,78808,78838,78867,78897,78926,78956,78985,79015,79044,79074,79104,79133,79163,79192,79222,79251,79281,79310,79340,79369,79399,79428,79458,79487,79517,79546,79576,79606,79635,79665,79695,79724,79753,79783,79812,79841,79871,79900,79930,79960,79990]},{"../main":571,"object-assign":454}],571:[function(t,e,n){var r=t("object-assign");function i(){this.regionalOptions=[],this.regionalOptions[""]={invalidCalendar:"Calendar {0} not found",invalidDate:"Invalid {0} date",invalidMonth:"Invalid {0} month",invalidYear:"Invalid {0} year",differentCalendars:"Cannot mix {0} and {1} dates"},this.local=this.regionalOptions[""],this.calendars={},this._localCals={}}function o(t,e,n,r){if(this._calendar=t,this._year=e,this._month=n,this._day=r,0===this._calendar._validateLevel&&!this._calendar.isValid(this._year,this._month,this._day))throw(l.local.invalidDate||l.regionalOptions[""].invalidDate).replace(/\{0\}/,this._calendar.local.name)}function a(t,e){return"000000".substring(0,e-(t=""+t).length)+t}function s(){this.shortYearCutoff="+10"}function A(t){this.local=this.regionalOptions[t]||this.regionalOptions[""]}r(i.prototype,{instance:function(t,e){t=(t||"gregorian").toLowerCase(),e=e||"";var n=this._localCals[t+"-"+e];if(!n&&this.calendars[t]&&(n=new this.calendars[t](e),this._localCals[t+"-"+e]=n),!n)throw(this.local.invalidCalendar||this.regionalOptions[""].invalidCalendar).replace(/\{0\}/,t);return n},newDate:function(t,e,n,r,i){return(r=(null!=t&&t.year?t.calendar():"string"==typeof r?this.instance(r,i):r)||this.instance()).newDate(t,e,n)},substituteDigits:function(t){return function(e){return(e+"").replace(/[0-9]/g,function(e){return t[e]})}},substituteChineseDigits:function(t,e){return function(n){for(var r="",i=0;n>0;){var o=n%10;r=(0===o?"":t[o]+e[i])+r,i++,n=Math.floor(n/10)}return 0===r.indexOf(t[1]+e[1])&&(r=r.substr(1)),r||t[0]}}}),r(o.prototype,{newDate:function(t,e,n){return this._calendar.newDate(null==t?this:t,e,n)},year:function(t){return 0===arguments.length?this._year:this.set(t,"y")},month:function(t){return 0===arguments.length?this._month:this.set(t,"m")},day:function(t){return 0===arguments.length?this._day:this.set(t,"d")},date:function(t,e,n){if(!this._calendar.isValid(t,e,n))throw(l.local.invalidDate||l.regionalOptions[""].invalidDate).replace(/\{0\}/,this._calendar.local.name);return this._year=t,this._month=e,this._day=n,this},leapYear:function(){return this._calendar.leapYear(this)},epoch:function(){return this._calendar.epoch(this)},formatYear:function(){return this._calendar.formatYear(this)},monthOfYear:function(){return this._calendar.monthOfYear(this)},weekOfYear:function(){return this._calendar.weekOfYear(this)},daysInYear:function(){return this._calendar.daysInYear(this)},dayOfYear:function(){return this._calendar.dayOfYear(this)},daysInMonth:function(){return this._calendar.daysInMonth(this)},dayOfWeek:function(){return this._calendar.dayOfWeek(this)},weekDay:function(){return this._calendar.weekDay(this)},extraInfo:function(){return this._calendar.extraInfo(this)},add:function(t,e){return this._calendar.add(this,t,e)},set:function(t,e){return this._calendar.set(this,t,e)},compareTo:function(t){if(this._calendar.name!==t._calendar.name)throw(l.local.differentCalendars||l.regionalOptions[""].differentCalendars).replace(/\{0\}/,this._calendar.local.name).replace(/\{1\}/,t._calendar.local.name);var e=this._year!==t._year?this._year-t._year:this._month!==t._month?this.monthOfYear()-t.monthOfYear():this._day-t._day;return 0===e?0:e<0?-1:1},calendar:function(){return this._calendar},toJD:function(){return this._calendar.toJD(this)},fromJD:function(t){return this._calendar.fromJD(t)},toJSDate:function(){return this._calendar.toJSDate(this)},fromJSDate:function(t){return this._calendar.fromJSDate(t)},toString:function(){return(this.year()<0?"-":"")+a(Math.abs(this.year()),4)+"-"+a(this.month(),2)+"-"+a(this.day(),2)}}),r(s.prototype,{_validateLevel:0,newDate:function(t,e,n){return null==t?this.today():(t.year&&(this._validate(t,e,n,l.local.invalidDate||l.regionalOptions[""].invalidDate),n=t.day(),e=t.month(),t=t.year()),new o(this,t,e,n))},today:function(){return this.fromJSDate(new Date)},epoch:function(t){return this._validate(t,this.minMonth,this.minDay,l.local.invalidYear||l.regionalOptions[""].invalidYear).year()<0?this.local.epochs[0]:this.local.epochs[1]},formatYear:function(t){var e=this._validate(t,this.minMonth,this.minDay,l.local.invalidYear||l.regionalOptions[""].invalidYear);return(e.year()<0?"-":"")+a(Math.abs(e.year()),4)},monthsInYear:function(t){return this._validate(t,this.minMonth,this.minDay,l.local.invalidYear||l.regionalOptions[""].invalidYear),12},monthOfYear:function(t,e){var n=this._validate(t,e,this.minDay,l.local.invalidMonth||l.regionalOptions[""].invalidMonth);return(n.month()+this.monthsInYear(n)-this.firstMonth)%this.monthsInYear(n)+this.minMonth},fromMonthOfYear:function(t,e){var n=(e+this.firstMonth-2*this.minMonth)%this.monthsInYear(t)+this.minMonth;return this._validate(t,n,this.minDay,l.local.invalidMonth||l.regionalOptions[""].invalidMonth),n},daysInYear:function(t){var e=this._validate(t,this.minMonth,this.minDay,l.local.invalidYear||l.regionalOptions[""].invalidYear);return this.leapYear(e)?366:365},dayOfYear:function(t,e,n){var r=this._validate(t,e,n,l.local.invalidDate||l.regionalOptions[""].invalidDate);return r.toJD()-this.newDate(r.year(),this.fromMonthOfYear(r.year(),this.minMonth),this.minDay).toJD()+1},daysInWeek:function(){return 7},dayOfWeek:function(t,e,n){var r=this._validate(t,e,n,l.local.invalidDate||l.regionalOptions[""].invalidDate);return(Math.floor(this.toJD(r))+2)%this.daysInWeek()},extraInfo:function(t,e,n){return this._validate(t,e,n,l.local.invalidDate||l.regionalOptions[""].invalidDate),{}},add:function(t,e,n){return this._validate(t,this.minMonth,this.minDay,l.local.invalidDate||l.regionalOptions[""].invalidDate),this._correctAdd(t,this._add(t,e,n),e,n)},_add:function(t,e,n){if(this._validateLevel++,"d"===n||"w"===n){var r=t.toJD()+e*("w"===n?this.daysInWeek():1),i=t.calendar().fromJD(r);return this._validateLevel--,[i.year(),i.month(),i.day()]}try{var o=t.year()+("y"===n?e:0),a=t.monthOfYear()+("m"===n?e:0);i=t.day(),"y"===n?(t.month()!==this.fromMonthOfYear(o,a)&&(a=this.newDate(o,t.month(),this.minDay).monthOfYear()),a=Math.min(a,this.monthsInYear(o)),i=Math.min(i,this.daysInMonth(o,this.fromMonthOfYear(o,a)))):"m"===n&&(function(t){for(;a<t.minMonth;)o--,a+=t.monthsInYear(o);for(var e=t.monthsInYear(o);a>e-1+t.minMonth;)o++,a-=e,e=t.monthsInYear(o)}(this),i=Math.min(i,this.daysInMonth(o,this.fromMonthOfYear(o,a))));var s=[o,this.fromMonthOfYear(o,a),i];return this._validateLevel--,s}catch(t){throw this._validateLevel--,t}},_correctAdd:function(t,e,n,r){if(!(this.hasYearZero||"y"!==r&&"m"!==r||0!==e[0]&&t.year()>0==e[0]>0)){var i={y:[1,1,"y"],m:[1,this.monthsInYear(-1),"m"],w:[this.daysInWeek(),this.daysInYear(-1),"d"],d:[1,this.daysInYear(-1),"d"]}[r],o=n<0?-1:1;e=this._add(t,n*i[0]+o*i[1],i[2])}return t.date(e[0],e[1],e[2])},set:function(t,e,n){this._validate(t,this.minMonth,this.minDay,l.local.invalidDate||l.regionalOptions[""].invalidDate);var r="y"===n?e:t.year(),i="m"===n?e:t.month(),o="d"===n?e:t.day();return"y"!==n&&"m"!==n||(o=Math.min(o,this.daysInMonth(r,i))),t.date(r,i,o)},isValid:function(t,e,n){this._validateLevel++;var r=this.hasYearZero||0!==t;if(r){var i=this.newDate(t,e,this.minDay);r=e>=this.minMonth&&e-this.minMonth<this.monthsInYear(i)&&n>=this.minDay&&n-this.minDay<this.daysInMonth(i)}return this._validateLevel--,r},toJSDate:function(t,e,n){var r=this._validate(t,e,n,l.local.invalidDate||l.regionalOptions[""].invalidDate);return l.instance().fromJD(this.toJD(r)).toJSDate()},fromJSDate:function(t){return this.fromJD(l.instance().fromJSDate(t).toJD())},_validate:function(t,e,n,r){if(t.year){if(0===this._validateLevel&&this.name!==t.calendar().name)throw(l.local.differentCalendars||l.regionalOptions[""].differentCalendars).replace(/\{0\}/,this.local.name).replace(/\{1\}/,t.calendar().local.name);return t}try{if(this._validateLevel++,1===this._validateLevel&&!this.isValid(t,e,n))throw r.replace(/\{0\}/,this.local.name);var i=this.newDate(t,e,n);return this._validateLevel--,i}catch(t){throw this._validateLevel--,t}}}),A.prototype=new s,r(A.prototype,{name:"Gregorian",jdEpoch:1721425.5,daysPerMonth:[31,28,31,30,31,30,31,31,30,31,30,31],hasYearZero:!1,minMonth:1,firstMonth:1,minDay:1,regionalOptions:{"":{name:"Gregorian",epochs:["BCE","CE"],monthNames:["January","February","March","April","May","June","July","August","September","October","November","December"],monthNamesShort:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],dayNames:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],dayNamesShort:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],dayNamesMin:["Su","Mo","Tu","We","Th","Fr","Sa"],digits:null,dateFormat:"mm/dd/yyyy",firstDay:0,isRTL:!1}},leapYear:function(t){var e=this._validate(t,this.minMonth,this.minDay,l.local.invalidYear||l.regionalOptions[""].invalidYear);return(t=e.year()+(e.year()<0?1:0))%4==0&&(t%100!=0||t%400==0)},weekOfYear:function(t,e,n){var r=this.newDate(t,e,n);return r.add(4-(r.dayOfWeek()||7),"d"),Math.floor((r.dayOfYear()-1)/7)+1},daysInMonth:function(t,e){var n=this._validate(t,e,this.minDay,l.local.invalidMonth||l.regionalOptions[""].invalidMonth);return this.daysPerMonth[n.month()-1]+(2===n.month()&&this.leapYear(n.year())?1:0)},weekDay:function(t,e,n){return(this.dayOfWeek(t,e,n)||7)<6},toJD:function(t,e,n){var r=this._validate(t,e,n,l.local.invalidDate||l.regionalOptions[""].invalidDate);t=r.year(),e=r.month(),n=r.day(),t<0&&t++,e<3&&(e+=12,t--);var i=Math.floor(t/100),o=2-i+Math.floor(i/4);return Math.floor(365.25*(t+4716))+Math.floor(30.6001*(e+1))+n+o-1524.5},fromJD:function(t){var e=Math.floor(t+.5),n=Math.floor((e-1867216.25)/36524.25),r=1524+(n=e+1+n-Math.floor(n/4)),i=Math.floor((r-122.1)/365.25),o=Math.floor(365.25*i),a=Math.floor((r-o)/30.6001),s=r-o-Math.floor(30.6001*a),A=a-(a>13.5?13:1),l=i-(A>2.5?4716:4715);return l<=0&&l--,this.newDate(l,A,s)},toJSDate:function(t,e,n){var r=this._validate(t,e,n,l.local.invalidDate||l.regionalOptions[""].invalidDate),i=new Date(r.year(),r.month()-1,r.day());return i.setHours(0),i.setMinutes(0),i.setSeconds(0),i.setMilliseconds(0),i.setHours(i.getHours()>12?i.getHours()+2:0),i},fromJSDate:function(t){return this.newDate(t.getFullYear(),t.getMonth()+1,t.getDate())}});var l=e.exports=new i;l.cdate=o,l.baseCalendar=s,l.calendars.gregorian=A},{"object-assign":454}],572:[function(t,e,n){var r=t("object-assign"),i=t("./main");r(i.regionalOptions[""],{invalidArguments:"Invalid arguments",invalidFormat:"Cannot format a date from another calendar",missingNumberAt:"Missing number at position {0}",unknownNameAt:"Unknown name at position {0}",unexpectedLiteralAt:"Unexpected literal at position {0}",unexpectedText:"Additional text found at end"}),i.local=i.regionalOptions[""],r(i.cdate.prototype,{formatDate:function(t,e){return"string"!=typeof t&&(e=t,t=""),this._calendar.formatDate(t||"",this,e)}}),r(i.baseCalendar.prototype,{UNIX_EPOCH:i.instance().newDate(1970,1,1).toJD(),SECS_PER_DAY:86400,TICKS_EPOCH:i.instance().jdEpoch,TICKS_PER_DAY:864e9,ATOM:"yyyy-mm-dd",COOKIE:"D, dd M yyyy",FULL:"DD, MM d, yyyy",ISO_8601:"yyyy-mm-dd",JULIAN:"J",RFC_822:"D, d M yy",RFC_850:"DD, dd-M-yy",RFC_1036:"D, d M yy",RFC_1123:"D, d M yyyy",RFC_2822:"D, d M yyyy",RSS:"D, d M yy",TICKS:"!",TIMESTAMP:"@",W3C:"yyyy-mm-dd",formatDate:function(t,e,n){if("string"!=typeof t&&(n=e,e=t,t=""),!e)return"";if(e.calendar()!==this)throw i.local.invalidFormat||i.regionalOptions[""].invalidFormat;t=t||this.local.dateFormat;for(var r,o,a,s=(n=n||{}).dayNamesShort||this.local.dayNamesShort,A=n.dayNames||this.local.dayNames,l=n.monthNumbers||this.local.monthNumbers,c=n.monthNamesShort||this.local.monthNamesShort,u=n.monthNames||this.local.monthNames,g=(n.calculateWeek||this.local.calculateWeek,function(e,n){for(var r=1;E+r<t.length&&t.charAt(E+r)===e;)r++;return E+=r-1,Math.floor(r/(n||1))>1}),h=function(t,e,n,r){var i=""+e;if(g(t,r))for(;i.length<n;)i="0"+i;return i},d=this,f=function(t){return"function"==typeof l?l.call(d,t,g("m")):C(h("m",t.month(),2))},p=function(t,e){return e?"function"==typeof u?u.call(d,t):u[t.month()-d.minMonth]:"function"==typeof c?c.call(d,t):c[t.month()-d.minMonth]},I=this.local.digits,C=function(t){return n.localNumbers&&I?I(t):t},m="",y=!1,E=0;E<t.length;E++)if(y)"'"!==t.charAt(E)||g("'")?m+=t.charAt(E):y=!1;else switch(t.charAt(E)){case"d":m+=C(h("d",e.day(),2));break;case"D":m+=(r=e.dayOfWeek(),o=s,a=A,g("D")?a[r]:o[r]);break;case"o":m+=h("o",e.dayOfYear(),3);break;case"w":m+=h("w",e.weekOfYear(),2);break;case"m":m+=f(e);break;case"M":m+=p(e,g("M"));break;case"y":m+=g("y",2)?e.year():(e.year()%100<10?"0":"")+e.year()%100;break;case"Y":g("Y",2),m+=e.formatYear();break;case"J":m+=e.toJD();break;case"@":m+=(e.toJD()-this.UNIX_EPOCH)*this.SECS_PER_DAY;break;case"!":m+=(e.toJD()-this.TICKS_EPOCH)*this.TICKS_PER_DAY;break;case"'":g("'")?m+="'":y=!0;break;default:m+=t.charAt(E)}return m},parseDate:function(t,e,n){if(null==e)throw i.local.invalidArguments||i.regionalOptions[""].invalidArguments;if(""===(e="object"==typeof e?e.toString():e+""))return null;t=t||this.local.dateFormat;var r=(n=n||{}).shortYearCutoff||this.shortYearCutoff;r="string"!=typeof r?r:this.today().year()%100+parseInt(r,10);for(var o=n.dayNamesShort||this.local.dayNamesShort,a=n.dayNames||this.local.dayNames,s=n.parseMonth||this.local.parseMonth,A=n.monthNumbers||this.local.monthNumbers,l=n.monthNamesShort||this.local.monthNamesShort,c=n.monthNames||this.local.monthNames,u=-1,g=-1,h=-1,d=-1,f=-1,p=!1,I=!1,C=function(e,n){for(var r=1;w+r<t.length&&t.charAt(w+r)===e;)r++;return w+=r-1,Math.floor(r/(n||1))>1},m=function(t,n){var r=C(t,n),o=[2,3,r?4:2,r?4:2,10,11,20]["oyYJ@!".indexOf(t)+1],a=new RegExp("^-?\\d{1,"+o+"}"),s=e.substring(Q).match(a);if(!s)throw(i.local.missingNumberAt||i.regionalOptions[""].missingNumberAt).replace(/\{0\}/,Q);return Q+=s[0].length,parseInt(s[0],10)},y=this,E=function(){if("function"==typeof A){C("m");var t=A.call(y,e.substring(Q));return Q+=t.length,t}return m("m")},v=function(t,n,r,o){for(var a=C(t,o)?r:n,s=0;s<a.length;s++)if(e.substr(Q,a[s].length).toLowerCase()===a[s].toLowerCase())return Q+=a[s].length,s+y.minMonth;throw(i.local.unknownNameAt||i.regionalOptions[""].unknownNameAt).replace(/\{0\}/,Q)},B=function(){if("function"==typeof c){var t=C("M")?c.call(y,e.substring(Q)):l.call(y,e.substring(Q));return Q+=t.length,t}return v("M",l,c)},b=function(){if(e.charAt(Q)!==t.charAt(w))throw(i.local.unexpectedLiteralAt||i.regionalOptions[""].unexpectedLiteralAt).replace(/\{0\}/,Q);Q++},Q=0,w=0;w<t.length;w++)if(I)"'"!==t.charAt(w)||C("'")?b():I=!1;else switch(t.charAt(w)){case"d":d=m("d");break;case"D":v("D",o,a);break;case"o":f=m("o");break;case"w":m("w");break;case"m":h=E();break;case"M":h=B();break;case"y":var x=w;p=!C("y",2),w=x,g=m("y",2);break;case"Y":g=m("Y",2);break;case"J":u=m("J")+.5,"."===e.charAt(Q)&&(Q++,m("J"));break;case"@":u=m("@")/this.SECS_PER_DAY+this.UNIX_EPOCH;break;case"!":u=m("!")/this.TICKS_PER_DAY+this.TICKS_EPOCH;break;case"*":Q=e.length;break;case"'":C("'")?b():I=!0;break;default:b()}if(Q<e.length)throw i.local.unexpectedText||i.regionalOptions[""].unexpectedText;if(-1===g?g=this.today().year():g<100&&p&&(g+=-1===r?1900:this.today().year()-this.today().year()%100-(g<=r?0:100)),"string"==typeof h&&(h=s.call(this,g,h)),f>-1){h=1,d=f;for(var _=this.daysInMonth(g,h);d>_;_=this.daysInMonth(g,h))h++,d-=_}return u>-1?this.fromJD(u):this.newDate(g,h,d)},determineDate:function(t,e,n,r,i){n&&"object"!=typeof n&&(i=r,r=n,n=null),"string"!=typeof r&&(i=r,r="");var o=this;return e=e?e.newDate():null,t=null==t?e:"string"==typeof t?function(t){try{return o.parseDate(r,t,i)}catch(t){}for(var e=((t=t.toLowerCase()).match(/^c/)&&n?n.newDate():null)||o.today(),a=/([+-]?[0-9]+)\s*(d|w|m|y)?/g,s=a.exec(t);s;)e.add(parseInt(s[1],10),s[2]||"d"),s=a.exec(t);return e}(t):"number"==typeof t?isNaN(t)||t===1/0||t===-1/0?e:o.today().add(t,"d"):o.newDate(t)}})},{"./main":571,"object-assign":454}],573:[function(t,e,n){e.exports=t("cwise-compiler")({args:["array",{offset:[1],array:0},"scalar","scalar","index"],pre:{body:"{}",args:[],thisVars:[],localVars:[]},post:{body:"{}",args:[],thisVars:[],localVars:[]},body:{body:"{\n        var _inline_1_da = _inline_1_arg0_ - _inline_1_arg3_\n        var _inline_1_db = _inline_1_arg1_ - _inline_1_arg3_\n        if((_inline_1_da >= 0) !== (_inline_1_db >= 0)) {\n          _inline_1_arg2_.push(_inline_1_arg4_[0] + 0.5 + 0.5 * (_inline_1_da + _inline_1_db) / (_inline_1_da - _inline_1_db))\n        }\n      }",args:[{name:"_inline_1_arg0_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_1_arg1_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_1_arg2_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_1_arg3_",lvalue:!1,rvalue:!0,count:2},{name:"_inline_1_arg4_",lvalue:!1,rvalue:!0,count:1}],thisVars:[],localVars:["_inline_1_da","_inline_1_db"]},funcName:"zeroCrossings"})},{"cwise-compiler":146}],574:[function(t,e,n){"use strict";e.exports=function(t,e){var n=[];return e=+e||0,r(t.hi(t.shape[0]-1),n,e),n};var r=t("./lib/zc-core")},{"./lib/zc-core":573}],575:[function(t,e,n){"use strict";e.exports=[{path:"",backoff:0},{path:"M-2.4,-3V3L0.6,0Z",backoff:.6},{path:"M-3.7,-2.5V2.5L1.3,0Z",backoff:1.3},{path:"M-4.45,-3L-1.65,-0.2V0.2L-4.45,3L1.55,0Z",backoff:1.55},{path:"M-2.2,-2.2L-0.2,-0.2V0.2L-2.2,2.2L-1.4,3L1.6,0L-1.4,-3Z",backoff:1.6},{path:"M-4.4,-2.1L-0.6,-0.2V0.2L-4.4,2.1L-4,3L2,0L-4,-3Z",backoff:2},{path:"M2,0A2,2 0 1,1 0,-2A2,2 0 0,1 2,0Z",backoff:0,noRotate:!0},{path:"M2,2V-2H-2V2Z",backoff:0,noRotate:!0}]},{}],576:[function(t,e,n){"use strict";var r=t("./arrow_paths"),i=t("../../plots/font_attributes"),o=t("../../plots/cartesian/constants"),a=t("../../plot_api/plot_template").templatedArray;e.exports=a("annotation",{visible:{valType:"boolean",dflt:!0,editType:"calc+arraydraw"},text:{valType:"string",editType:"calc+arraydraw"},textangle:{valType:"angle",dflt:0,editType:"calc+arraydraw"},font:i({editType:"calc+arraydraw",colorEditType:"arraydraw"}),width:{valType:"number",min:1,dflt:null,editType:"calc+arraydraw"},height:{valType:"number",min:1,dflt:null,editType:"calc+arraydraw"},opacity:{valType:"number",min:0,max:1,dflt:1,editType:"arraydraw"},align:{valType:"enumerated",values:["left","center","right"],dflt:"center",editType:"arraydraw"},valign:{valType:"enumerated",values:["top","middle","bottom"],dflt:"middle",editType:"arraydraw"},bgcolor:{valType:"color",dflt:"rgba(0,0,0,0)",editType:"arraydraw"},bordercolor:{valType:"color",dflt:"rgba(0,0,0,0)",editType:"arraydraw"},borderpad:{valType:"number",min:0,dflt:1,editType:"calc+arraydraw"},borderwidth:{valType:"number",min:0,dflt:1,editType:"calc+arraydraw"},showarrow:{valType:"boolean",dflt:!0,editType:"calc+arraydraw"},arrowcolor:{valType:"color",editType:"arraydraw"},arrowhead:{valType:"integer",min:0,max:r.length,dflt:1,editType:"arraydraw"},startarrowhead:{valType:"integer",min:0,max:r.length,dflt:1,editType:"arraydraw"},arrowside:{valType:"flaglist",flags:["end","start"],extras:["none"],dflt:"end",editType:"arraydraw"},arrowsize:{valType:"number",min:.3,dflt:1,editType:"calc+arraydraw"},startarrowsize:{valType:"number",min:.3,dflt:1,editType:"calc+arraydraw"},arrowwidth:{valType:"number",min:.1,editType:"calc+arraydraw"},standoff:{valType:"number",min:0,dflt:0,editType:"calc+arraydraw"},startstandoff:{valType:"number",min:0,dflt:0,editType:"calc+arraydraw"},ax:{valType:"any",editType:"calc+arraydraw"},ay:{valType:"any",editType:"calc+arraydraw"},axref:{valType:"enumerated",dflt:"pixel",values:["pixel",o.idRegex.x.toString()],editType:"calc"},ayref:{valType:"enumerated",dflt:"pixel",values:["pixel",o.idRegex.y.toString()],editType:"calc"},xref:{valType:"enumerated",values:["paper",o.idRegex.x.toString()],editType:"calc"},x:{valType:"any",editType:"calc+arraydraw"},xanchor:{valType:"enumerated",values:["auto","left","center","right"],dflt:"auto",editType:"calc+arraydraw"},xshift:{valType:"number",dflt:0,editType:"calc+arraydraw"},yref:{valType:"enumerated",values:["paper",o.idRegex.y.toString()],editType:"calc"},y:{valType:"any",editType:"calc+arraydraw"},yanchor:{valType:"enumerated",values:["auto","top","middle","bottom"],dflt:"auto",editType:"calc+arraydraw"},yshift:{valType:"number",dflt:0,editType:"calc+arraydraw"},clicktoshow:{valType:"enumerated",values:[!1,"onoff","onout"],dflt:!1,editType:"arraydraw"},xclick:{valType:"any",editType:"arraydraw"},yclick:{valType:"any",editType:"arraydraw"},hovertext:{valType:"string",editType:"arraydraw"},hoverlabel:{bgcolor:{valType:"color",editType:"arraydraw"},bordercolor:{valType:"color",editType:"arraydraw"},font:i({editType:"arraydraw"}),editType:"arraydraw"},captureevents:{valType:"boolean",editType:"arraydraw"},editType:"calc",_deprecated:{ref:{valType:"string",editType:"calc"}}})},{"../../plot_api/plot_template":757,"../../plots/cartesian/constants":773,"../../plots/font_attributes":793,"./arrow_paths":575}],577:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axes"),o=t("./draw").draw;function a(t){var e=t._fullLayout;r.filterVisible(e.annotations).forEach(function(e){var n=i.getFromId(t,e.xref),r=i.getFromId(t,e.yref);e._extremes={},n&&s(e,n),r&&s(e,r)})}function s(t,e){var n,r=e._id,o=r.charAt(0),a=t[o],s=t["a"+o],A=t[o+"ref"],l=t["a"+o+"ref"],c=t["_"+o+"padplus"],u=t["_"+o+"padminus"],g={x:1,y:-1}[o]*t[o+"shift"],h=3*t.arrowsize*t.arrowwidth||0,d=h+g,f=h-g,p=3*t.startarrowsize*t.arrowwidth||0,I=p+g,C=p-g;if(l===A){var m=i.findExtremes(e,[e.r2c(a)],{ppadplus:d,ppadminus:f}),y=i.findExtremes(e,[e.r2c(s)],{ppadplus:Math.max(c,I),ppadminus:Math.max(u,C)});n={min:[m.min[0],y.min[0]],max:[m.max[0],y.max[0]]}}else I=s?I+s:I,C=s?C-s:C,n=i.findExtremes(e,[e.r2c(a)],{ppadplus:Math.max(c,d,I),ppadminus:Math.max(u,f,C)});t._extremes[r]=n}e.exports=function(t){var e=t._fullLayout;if(r.filterVisible(e.annotations).length&&t._fullData.length)return r.syncOrAsync([o,a],t)}},{"../../lib":719,"../../plots/cartesian/axes":767,"./draw":582}],578:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../registry"),o=t("../../plot_api/plot_template").arrayEditor;function a(t,e){var n,r,i,o,a,A,l,c=t._fullLayout.annotations,u=[],g=[],h=[],d=(e||[]).length;for(n=0;n<c.length;n++)if(o=(i=c[n]).clicktoshow){for(r=0;r<d;r++)if(A=(a=e[r]).xaxis,l=a.yaxis,A._id===i.xref&&l._id===i.yref&&A.d2r(a.x)===s(i._xclick,A)&&l.d2r(a.y)===s(i._yclick,l)){(i.visible?"onout"===o?g:h:u).push(n);break}r===d&&i.visible&&"onout"===o&&g.push(n)}return{on:u,off:g,explicitOff:h}}function s(t,e){return"log"===e.type?e.l2r(t):e.d2r(t)}e.exports={hasClickToShow:function(t,e){var n=a(t,e);return n.on.length>0||n.explicitOff.length>0},onClick:function(t,e){var n,s,A=a(t,e),l=A.on,c=A.off.concat(A.explicitOff),u={},g=t._fullLayout.annotations;if(l.length||c.length){for(n=0;n<l.length;n++)(s=o(t.layout,"annotations",g[l[n]])).modifyItem("visible",!0),r.extendFlat(u,s.getUpdateObj());for(n=0;n<c.length;n++)(s=o(t.layout,"annotations",g[c[n]])).modifyItem("visible",!1),r.extendFlat(u,s.getUpdateObj());return i.call("update",t,{},u)}}}},{"../../lib":719,"../../plot_api/plot_template":757,"../../registry":847}],579:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../color");e.exports=function(t,e,n,o){o("opacity");var a=o("bgcolor"),s=o("bordercolor"),A=i.opacity(s);o("borderpad");var l=o("borderwidth"),c=o("showarrow");if(o("text",c?" ":n._dfltTitle.annotation),o("textangle"),r.coerceFont(o,"font",n.font),o("width"),o("align"),o("height")&&o("valign"),c){var u,g,h=o("arrowside");-1!==h.indexOf("end")&&(u=o("arrowhead"),g=o("arrowsize")),-1!==h.indexOf("start")&&(o("startarrowhead",u),o("startarrowsize",g)),o("arrowcolor",A?e.bordercolor:i.defaultLine),o("arrowwidth",2*(A&&l||1)),o("standoff"),o("startstandoff")}var d=o("hovertext"),f=n.hoverlabel||{};if(d){var p=o("hoverlabel.bgcolor",f.bgcolor||(i.opacity(a)?i.rgb(a):i.defaultLine)),I=o("hoverlabel.bordercolor",f.bordercolor||i.contrast(p));r.coerceFont(o,"hoverlabel.font",{family:f.font.family,size:f.font.size,color:f.font.color||I})}o("captureevents",!!d)}},{"../../lib":719,"../color":593}],580:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib/to_log_range");e.exports=function(t,e,n,o){e=e||{};var a="log"===n&&"linear"===e.type,s="linear"===n&&"log"===e.type;if(a||s)for(var A,l,c=t._fullLayout.annotations,u=e._id.charAt(0),g=0;g<c.length;g++)A=c[g],l="annotations["+g+"].",A[u+"ref"]===e._id&&h(u),A["a"+u+"ref"]===e._id&&h("a"+u);function h(t){var n=A[t],s=null;s=a?i(n,e.range):Math.pow(10,n),r(s)||(s=null),o(l+t,s)}}},{"../../lib/to_log_range":745,"fast-isnumeric":225}],581:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axes"),o=t("../../plots/array_container_defaults"),a=t("./common_defaults"),s=t("./attributes");function A(t,e,n){function o(n,i){return r.coerce(t,e,s,n,i)}var A=o("visible"),l=o("clicktoshow");if(A||l){a(t,e,n,o);for(var c=e.showarrow,u=["x","y"],g=[-10,-30],h={_fullLayout:n},d=0;d<2;d++){var f=u[d],p=i.coerceRef(t,e,h,f,"","paper");if("paper"!==p&&i.getFromId(h,p)._annIndices.push(e._index),i.coercePosition(e,h,o,p,f,.5),c){var I="a"+f,C=i.coerceRef(t,e,h,I,"pixel");"pixel"!==C&&C!==p&&(C=e[I]="pixel");var m="pixel"===C?g[d]:.4;i.coercePosition(e,h,o,C,I,m)}o(f+"anchor"),o(f+"shift")}if(r.noneOrAll(t,e,["x","y"]),c&&r.noneOrAll(t,e,["ax","ay"]),l){var y=o("xclick"),E=o("yclick");e._xclick=void 0===y?e.x:i.cleanPosition(y,h,e.xref),e._yclick=void 0===E?e.y:i.cleanPosition(E,h,e.yref)}}}e.exports=function(t,e){o(t,e,{name:"annotations",handleItemDefaults:A})}},{"../../lib":719,"../../plots/array_container_defaults":763,"../../plots/cartesian/axes":767,"./attributes":576,"./common_defaults":579}],582:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../registry"),o=t("../../plots/plots"),a=t("../../lib"),s=t("../../plots/cartesian/axes"),A=t("../color"),l=t("../drawing"),c=t("../fx"),u=t("../../lib/svg_text_utils"),g=t("../../lib/setcursor"),h=t("../dragelement"),d=t("../../plot_api/plot_template").arrayEditor,f=t("./draw_arrow_head");function p(t,e){var n=t._fullLayout.annotations[e]||{},r=s.getFromId(t,n.xref),i=s.getFromId(t,n.yref);r&&r.setScale(),i&&i.setScale(),I(t,n,e,!1,r,i)}function I(t,e,n,o,s,p){var I,C,m=t._fullLayout,y=t._fullLayout._size,E=t._context.edits;o?(I="annotation-"+o,C=o+".annotations"):(I="annotation",C="annotations");var v=d(t.layout,C,e),B=v.modifyBase,b=v.modifyItem,Q=v.getUpdateObj;m._infolayer.selectAll("."+I+'[data-index="'+n+'"]').remove();var w="clip"+m._uid+"_ann"+n;if(e._input&&!1!==e.visible){var x={x:{},y:{}},_=+e.textangle||0,D=m._infolayer.append("g").classed(I,!0).attr("data-index",String(n)).style("opacity",e.opacity),M=D.append("g").classed("annotation-text-g",!0),k=E[e.showarrow?"annotationTail":"annotationPosition"],S=e.captureevents||E.annotationText||k,L=M.append("g").style("pointer-events",S?"all":null).call(g,"pointer").on("click",function(){t._dragging=!1,t.emit("plotly_clickannotation",Y(r.event))});e.hovertext&&L.on("mouseover",function(){var n=e.hoverlabel,r=n.font,i=this.getBoundingClientRect(),o=t.getBoundingClientRect();c.loneHover({x0:i.left-o.left,x1:i.right-o.left,y:(i.top+i.bottom)/2-o.top,text:e.hovertext,color:n.bgcolor,borderColor:n.bordercolor,fontFamily:r.family,fontSize:r.size,fontColor:r.color},{container:m._hoverlayer.node(),outerContainer:m._paper.node(),gd:t})}).on("mouseout",function(){c.loneUnhover(m._hoverlayer.node())});var R=e.borderwidth,N=e.borderpad,F=R+N,T=L.append("rect").attr("class","bg").style("stroke-width",R+"px").call(A.stroke,e.bordercolor).call(A.fill,e.bgcolor),U=e.width||e.height,O=m._topclips.selectAll("#"+w).data(U?[0]:[]);O.enter().append("clipPath").classed("annclip",!0).attr("id",w).append("rect"),O.exit().remove();var G=e.font,H=m._meta?a.templateString(e.text,m._meta):e.text,j=L.append("text").classed("annotation-text",!0).text(H);E.annotationText?j.call(u.makeEditable,{delegate:L,gd:t}).call(P).on("edit",function(n){e.text=n,this.call(P),b("text",n),s&&s.autorange&&B(s._name+".autorange",!0),p&&p.autorange&&B(p._name+".autorange",!0),i.call("_guiRelayout",t,Q())}):j.call(P)}else r.selectAll("#"+w).remove();function Y(t){var r={index:n,annotation:e._input,fullAnnotation:e,event:t};return o&&(r.subplotId=o),r}function P(n){return n.call(l.font,G).attr({"text-anchor":{left:"start",right:"end"}[e.align]||"middle"}),u.convertToTspans(n,t,K),n}function K(){var n=j.selectAll("a");1===n.size()&&n.text()===j.text()&&L.insert("a",":first-child").attr({"xlink:xlink:href":n.attr("xlink:href"),"xlink:xlink:show":n.attr("xlink:show")}).style({cursor:"pointer"}).node().appendChild(T.node());var r=L.select(".annotation-text-math-group"),c=!r.empty(),d=l.bBox((c?r:j).node()),I=d.width,C=d.height,v=e.width||I,S=e.height||C,N=Math.round(v+2*F),G=Math.round(S+2*F);function H(t,e){return"auto"===e&&(e=t<1/3?"left":t>2/3?"right":"center"),{center:0,middle:0,left:.5,bottom:-.5,right:-.5,top:.5}[e]}for(var P=!1,K=["x","y"],z=0;z<K.length;z++){var J,q,V,W,X,Z=K[z],$=e[Z+"ref"]||Z,tt=e["a"+Z+"ref"],et={x:s,y:p}[Z],nt=(_+("x"===Z?0:-90))*Math.PI/180,rt=N*Math.cos(nt),it=G*Math.sin(nt),ot=Math.abs(rt)+Math.abs(it),at=e[Z+"anchor"],st=e[Z+"shift"]*("x"===Z?1:-1),At=x[Z];if(et){var lt=et.r2fraction(e[Z]);(lt<0||lt>1)&&(tt===$?((lt=et.r2fraction(e["a"+Z]))<0||lt>1)&&(P=!0):P=!0),J=et._offset+et.r2p(e[Z]),W=.5}else"x"===Z?(V=e[Z],J=y.l+y.w*V):(V=1-e[Z],J=y.t+y.h*V),W=e.showarrow?.5:V;if(e.showarrow){At.head=J;var ct=e["a"+Z];X=rt*H(.5,e.xanchor)-it*H(.5,e.yanchor),tt===$?(At.tail=et._offset+et.r2p(ct),q=X):(At.tail=J+ct,q=X+ct),At.text=At.tail+X;var ut=m["x"===Z?"width":"height"];if("paper"===$&&(At.head=a.constrain(At.head,1,ut-1)),"pixel"===tt){var gt=-Math.max(At.tail-3,At.text),ht=Math.min(At.tail+3,At.text)-ut;gt>0?(At.tail+=gt,At.text+=gt):ht>0&&(At.tail-=ht,At.text-=ht)}At.tail+=st,At.head+=st}else q=X=ot*H(W,at),At.text=J+X;At.text+=st,X+=st,q+=st,e["_"+Z+"padplus"]=ot/2+q,e["_"+Z+"padminus"]=ot/2-q,e["_"+Z+"size"]=ot,e["_"+Z+"shift"]=X}if(t._dragging||!P){var dt=0,ft=0;if("left"!==e.align&&(dt=(v-I)*("center"===e.align?.5:1)),"top"!==e.valign&&(ft=(S-C)*("middle"===e.valign?.5:1)),c)r.select("svg").attr({x:F+dt-1,y:F+ft}).call(l.setClipUrl,U?w:null,t);else{var pt=F+ft-d.top,It=F+dt-d.left;j.call(u.positionText,It,pt).call(l.setClipUrl,U?w:null,t)}O.select("rect").call(l.setRect,F,F,v,S),T.call(l.setRect,R/2,R/2,N-R,G-R),L.call(l.setTranslate,Math.round(x.x.text-N/2),Math.round(x.y.text-G/2)),M.attr({transform:"rotate("+_+","+x.x.text+","+x.y.text+")"});var Ct,mt=function(n,r){D.selectAll(".annotation-arrow-g").remove();var c=x.x.head,u=x.y.head,g=x.x.tail+n,d=x.y.tail+r,I=x.x.text+n,C=x.y.text+r,m=a.rotationXYMatrix(_,I,C),v=a.apply2DTransform(m),w=a.apply2DTransform2(m),k=+T.attr("width"),S=+T.attr("height"),R=I-.5*k,N=R+k,F=C-.5*S,U=F+S,O=[[R,F,R,U],[R,U,N,U],[N,U,N,F],[N,F,R,F]].map(w);if(!O.reduce(function(t,e){return t^!!a.segmentsIntersect(c,u,c+1e6,u+1e6,e[0],e[1],e[2],e[3])},!1)){O.forEach(function(t){var e=a.segmentsIntersect(g,d,c,u,t[0],t[1],t[2],t[3]);e&&(g=e.x,d=e.y)});var G=e.arrowwidth,H=e.arrowcolor,j=e.arrowside,Y=D.append("g").style({opacity:A.opacity(H)}).classed("annotation-arrow-g",!0),P=Y.append("path").attr("d","M"+g+","+d+"L"+c+","+u).style("stroke-width",G+"px").call(A.stroke,A.rgb(H));if(f(P,j,e),E.annotationPosition&&P.node().parentNode&&!o){var K=c,z=u;if(e.standoff){var J=Math.sqrt(Math.pow(c-g,2)+Math.pow(u-d,2));K+=e.standoff*(g-c)/J,z+=e.standoff*(d-u)/J}var q,V,W=Y.append("path").classed("annotation-arrow",!0).classed("anndrag",!0).classed("cursor-move",!0).attr({d:"M3,3H-3V-3H3ZM0,0L"+(g-K)+","+(d-z),transform:"translate("+K+","+z+")"}).style("stroke-width",G+6+"px").call(A.stroke,"rgba(0,0,0,0)").call(A.fill,"rgba(0,0,0,0)");h.init({element:W.node(),gd:t,prepFn:function(){var t=l.getTranslate(L);q=t.x,V=t.y,s&&s.autorange&&B(s._name+".autorange",!0),p&&p.autorange&&B(p._name+".autorange",!0)},moveFn:function(t,n){var r=v(q,V),i=r[0]+t,o=r[1]+n;L.call(l.setTranslate,i,o),b("x",s?s.p2r(s.r2p(e.x)+t):e.x+t/y.w),b("y",p?p.p2r(p.r2p(e.y)+n):e.y-n/y.h),e.axref===e.xref&&b("ax",s.p2r(s.r2p(e.ax)+t)),e.ayref===e.yref&&b("ay",p.p2r(p.r2p(e.ay)+n)),Y.attr("transform","translate("+t+","+n+")"),M.attr({transform:"rotate("+_+","+i+","+o+")"})},doneFn:function(){i.call("_guiRelayout",t,Q());var e=document.querySelector(".js-notes-box-panel");e&&e.redraw(e.selectedObj)}})}}};e.showarrow&&mt(0,0),k&&h.init({element:L.node(),gd:t,prepFn:function(){Ct=M.attr("transform")},moveFn:function(t,n){var r="pointer";if(e.showarrow)e.axref===e.xref?b("ax",s.p2r(s.r2p(e.ax)+t)):b("ax",e.ax+t),e.ayref===e.yref?b("ay",p.p2r(p.r2p(e.ay)+n)):b("ay",e.ay+n),mt(t,n);else{if(o)return;var i,a;if(s)i=s.p2r(s.r2p(e.x)+t);else{var A=e._xsize/y.w,l=e.x+(e._xshift-e.xshift)/y.w-A/2;i=h.align(l+t/y.w,A,0,1,e.xanchor)}if(p)a=p.p2r(p.r2p(e.y)+n);else{var c=e._ysize/y.h,u=e.y-(e._yshift+e.yshift)/y.h-c/2;a=h.align(u-n/y.h,c,0,1,e.yanchor)}b("x",i),b("y",a),s&&p||(r=h.getCursor(s?.5:i,p?.5:a,e.xanchor,e.yanchor))}M.attr({transform:"translate("+t+","+n+")"+Ct}),g(L,r)},clickFn:function(n,r){e.captureevents&&t.emit("plotly_clickannotation",Y(r))},doneFn:function(){g(L),i.call("_guiRelayout",t,Q());var e=document.querySelector(".js-notes-box-panel");e&&e.redraw(e.selectedObj)}})}else L.remove()}}e.exports={draw:function(t){var e=t._fullLayout;e._infolayer.selectAll(".annotation").remove();for(var n=0;n<e.annotations.length;n++)e.annotations[n].visible&&p(t,n);return o.previousPromises(t)},drawOne:p,drawRaw:I}},{"../../lib":719,"../../lib/setcursor":739,"../../lib/svg_text_utils":743,"../../plot_api/plot_template":757,"../../plots/cartesian/axes":767,"../../plots/plots":828,"../../registry":847,"../color":593,"../dragelement":611,"../drawing":614,"../fx":632,"./draw_arrow_head":583,d3:163}],583:[function(t,e,n){"use strict";var r=t("d3"),i=t("../color"),o=t("./arrow_paths");e.exports=function(t,e,n){var a,s,A,l,c=t.node(),u=o[n.arrowhead||0],g=o[n.startarrowhead||0],h=(n.arrowwidth||1)*(n.arrowsize||1),d=(n.arrowwidth||1)*(n.startarrowsize||1),f=e.indexOf("start")>=0,p=e.indexOf("end")>=0,I=u.backoff*h+n.standoff,C=g.backoff*d+n.startstandoff;if("line"===c.nodeName){a={x:+t.attr("x1"),y:+t.attr("y1")},s={x:+t.attr("x2"),y:+t.attr("y2")};var m=a.x-s.x,y=a.y-s.y;if(l=(A=Math.atan2(y,m))+Math.PI,I&&C&&I+C>Math.sqrt(m*m+y*y))return void k();if(I){if(I*I>m*m+y*y)return void k();var E=I*Math.cos(A),v=I*Math.sin(A);s.x+=E,s.y+=v,t.attr({x2:s.x,y2:s.y})}if(C){if(C*C>m*m+y*y)return void k();var B=C*Math.cos(A),b=C*Math.sin(A);a.x-=B,a.y-=b,t.attr({x1:a.x,y1:a.y})}}else if("path"===c.nodeName){var Q=c.getTotalLength(),w="";if(Q<I+C)return void k();var x=c.getPointAtLength(0),_=c.getPointAtLength(.1);A=Math.atan2(x.y-_.y,x.x-_.x),a=c.getPointAtLength(Math.min(C,Q)),w="0px,"+C+"px,";var D=c.getPointAtLength(Q),M=c.getPointAtLength(Q-.1);l=Math.atan2(D.y-M.y,D.x-M.x),s=c.getPointAtLength(Math.max(0,Q-I)),w+=Q-(w?C+I:I)+"px,"+Q+"px",t.style("stroke-dasharray",w)}function k(){t.style("stroke-dasharray","0px,100px")}function S(e,o,a,s){e.path&&(e.noRotate&&(a=0),r.select(c.parentNode).append("path").attr({class:t.attr("class"),d:e.path,transform:"translate("+o.x+","+o.y+")"+(a?"rotate("+180*a/Math.PI+")":"")+"scale("+s+")"}).style({fill:i.rgb(n.arrowcolor),"stroke-width":0}))}f&&S(g,a,A,d),p&&S(u,s,l,h)}},{"../color":593,"./arrow_paths":575,d3:163}],584:[function(t,e,n){"use strict";var r=t("./draw"),i=t("./click");e.exports={moduleType:"component",name:"annotations",layoutAttributes:t("./attributes"),supplyLayoutDefaults:t("./defaults"),includeBasePlot:t("../../plots/cartesian/include_components")("annotations"),calcAutorange:t("./calc_autorange"),draw:r.draw,drawOne:r.drawOne,drawRaw:r.drawRaw,hasClickToShow:i.hasClickToShow,onClick:i.onClick,convertCoords:t("./convert_coords")}},{"../../plots/cartesian/include_components":777,"./attributes":576,"./calc_autorange":577,"./click":578,"./convert_coords":580,"./defaults":581,"./draw":582}],585:[function(t,e,n){"use strict";var r=t("../annotations/attributes"),i=t("../../plot_api/edit_types").overrideAll,o=t("../../plot_api/plot_template").templatedArray;e.exports=i(o("annotation",{visible:r.visible,x:{valType:"any"},y:{valType:"any"},z:{valType:"any"},ax:{valType:"number"},ay:{valType:"number"},xanchor:r.xanchor,xshift:r.xshift,yanchor:r.yanchor,yshift:r.yshift,text:r.text,textangle:r.textangle,font:r.font,width:r.width,height:r.height,opacity:r.opacity,align:r.align,valign:r.valign,bgcolor:r.bgcolor,bordercolor:r.bordercolor,borderpad:r.borderpad,borderwidth:r.borderwidth,showarrow:r.showarrow,arrowcolor:r.arrowcolor,arrowhead:r.arrowhead,startarrowhead:r.startarrowhead,arrowside:r.arrowside,arrowsize:r.arrowsize,startarrowsize:r.startarrowsize,arrowwidth:r.arrowwidth,standoff:r.standoff,startstandoff:r.startstandoff,hovertext:r.hovertext,hoverlabel:r.hoverlabel,captureevents:r.captureevents}),"calc","from-root")},{"../../plot_api/edit_types":750,"../../plot_api/plot_template":757,"../annotations/attributes":576}],586:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axes");function o(t,e){var n=e.fullSceneLayout.domain,o=e.fullLayout._size,a={pdata:null,type:"linear",autorange:!1,range:[-1/0,1/0]};t._xa={},r.extendFlat(t._xa,a),i.setConvert(t._xa),t._xa._offset=o.l+n.x[0]*o.w,t._xa.l2p=function(){return.5*(1+t._pdata[0]/t._pdata[3])*o.w*(n.x[1]-n.x[0])},t._ya={},r.extendFlat(t._ya,a),i.setConvert(t._ya),t._ya._offset=o.t+(1-n.y[1])*o.h,t._ya.l2p=function(){return.5*(1-t._pdata[1]/t._pdata[3])*o.h*(n.y[1]-n.y[0])}}e.exports=function(t){for(var e=t.fullSceneLayout.annotations,n=0;n<e.length;n++)o(e[n],t);t.fullLayout._infolayer.selectAll(".annotation-"+t.id).remove()}},{"../../lib":719,"../../plots/cartesian/axes":767}],587:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axes"),o=t("../../plots/array_container_defaults"),a=t("../annotations/common_defaults"),s=t("./attributes");function A(t,e,n,o){function A(n,i){return r.coerce(t,e,s,n,i)}function l(t){var r=t+"axis",o={_fullLayout:{}};return o._fullLayout[r]=n[r],i.coercePosition(e,o,A,t,t,.5)}A("visible")&&(a(t,e,o.fullLayout,A),l("x"),l("y"),l("z"),r.noneOrAll(t,e,["x","y","z"]),e.xref="x",e.yref="y",e.zref="z",A("xanchor"),A("yanchor"),A("xshift"),A("yshift"),e.showarrow&&(e.axref="pixel",e.ayref="pixel",A("ax",-10),A("ay",-30),r.noneOrAll(t,e,["ax","ay"])))}e.exports=function(t,e,n){o(t,e,{name:"annotations",handleItemDefaults:A,fullLayout:n.fullLayout})}},{"../../lib":719,"../../plots/array_container_defaults":763,"../../plots/cartesian/axes":767,"../annotations/common_defaults":579,"./attributes":585}],588:[function(t,e,n){"use strict";var r=t("../annotations/draw").drawRaw,i=t("../../plots/gl3d/project"),o=["x","y","z"];e.exports=function(t){for(var e=t.fullSceneLayout,n=t.dataScale,a=e.annotations,s=0;s<a.length;s++){for(var A=a[s],l=!1,c=0;c<3;c++){var u=o[c],g=A[u],h=e[u+"axis"].r2fraction(g);if(h<0||h>1){l=!0;break}}l?t.fullLayout._infolayer.select(".annotation-"+t.id+'[data-index="'+s+'"]').remove():(A._pdata=i(t.glplot.cameraParams,[e.xaxis.r2l(A.x)*n[0],e.yaxis.r2l(A.y)*n[1],e.zaxis.r2l(A.z)*n[2]]),r(t.graphDiv,A,s,t.id,A._xa,A._ya))}}},{"../../plots/gl3d/project":816,"../annotations/draw":582}],589:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib");e.exports={moduleType:"component",name:"annotations3d",schema:{subplots:{scene:{annotations:t("./attributes")}}},layoutAttributes:t("./attributes"),handleDefaults:t("./defaults"),includeBasePlot:function(t,e){var n=r.subplotsRegistry.gl3d;if(n)for(var o=n.attrRegex,a=Object.keys(t),s=0;s<a.length;s++){var A=a[s];o.test(A)&&(t[A].annotations||[]).length&&(i.pushUnique(e._basePlotModules,n),i.pushUnique(e._subplots.gl3d,A))}},convert:t("./convert"),draw:t("./draw")}},{"../../lib":719,"../../registry":847,"./attributes":585,"./convert":586,"./defaults":587,"./draw":588}],590:[function(t,e,n){"use strict";e.exports=t("world-calendars/dist/main"),t("world-calendars/dist/plus"),t("world-calendars/dist/calendars/chinese"),t("world-calendars/dist/calendars/coptic"),t("world-calendars/dist/calendars/discworld"),t("world-calendars/dist/calendars/ethiopian"),t("world-calendars/dist/calendars/hebrew"),t("world-calendars/dist/calendars/islamic"),t("world-calendars/dist/calendars/julian"),t("world-calendars/dist/calendars/mayan"),t("world-calendars/dist/calendars/nanakshahi"),t("world-calendars/dist/calendars/nepali"),t("world-calendars/dist/calendars/persian"),t("world-calendars/dist/calendars/taiwan"),t("world-calendars/dist/calendars/thai"),t("world-calendars/dist/calendars/ummalqura")},{"world-calendars/dist/calendars/chinese":557,"world-calendars/dist/calendars/coptic":558,"world-calendars/dist/calendars/discworld":559,"world-calendars/dist/calendars/ethiopian":560,"world-calendars/dist/calendars/hebrew":561,"world-calendars/dist/calendars/islamic":562,"world-calendars/dist/calendars/julian":563,"world-calendars/dist/calendars/mayan":564,"world-calendars/dist/calendars/nanakshahi":565,"world-calendars/dist/calendars/nepali":566,"world-calendars/dist/calendars/persian":567,"world-calendars/dist/calendars/taiwan":568,"world-calendars/dist/calendars/thai":569,"world-calendars/dist/calendars/ummalqura":570,"world-calendars/dist/main":571,"world-calendars/dist/plus":572}],591:[function(t,e,n){"use strict";var r=t("./calendars"),i=t("../../lib"),o=t("../../constants/numerical"),a=o.EPOCHJD,s=o.ONEDAY,A={valType:"enumerated",values:Object.keys(r.calendars),editType:"calc",dflt:"gregorian"},l=function(t,e,n,r){var o={};return o[n]=A,i.coerce(t,e,o,n,r)},c="##",u={d:{0:"dd","-":"d"},e:{0:"d","-":"d"},a:{0:"D","-":"D"},A:{0:"DD","-":"DD"},j:{0:"oo","-":"o"},W:{0:"ww","-":"w"},m:{0:"mm","-":"m"},b:{0:"M","-":"M"},B:{0:"MM","-":"MM"},y:{0:"yy","-":"yy"},Y:{0:"yyyy","-":"yyyy"},U:c,w:c,c:{0:"D M d %X yyyy","-":"D M d %X yyyy"},x:{0:"mm/dd/yyyy","-":"mm/dd/yyyy"}},g={};function h(t){var e=g[t];return e||(e=g[t]=r.instance(t))}function d(t){return i.extendFlat({},A,{description:t})}function f(t){return"Sets the calendar system to use with `"+t+"` date data."}var p={xcalendar:d(f("x"))},I=i.extendFlat({},p,{ycalendar:d(f("y"))}),C=i.extendFlat({},I,{zcalendar:d(f("z"))}),m=d(["Sets the calendar system to use for `range` and `tick0`","if this is a date axis. This does not set the calendar for","interpreting data on this axis, that's specified in the trace","or via the global `layout.calendar`"].join(" "));e.exports={moduleType:"component",name:"calendars",schema:{traces:{scatter:I,bar:I,box:I,heatmap:I,contour:I,histogram:I,histogram2d:I,histogram2dcontour:I,scatter3d:C,surface:C,mesh3d:C,scattergl:I,ohlc:p,candlestick:p},layout:{calendar:d(["Sets the default calendar system to use for interpreting and","displaying dates throughout the plot."].join(" "))},subplots:{xaxis:{calendar:m},yaxis:{calendar:m},scene:{xaxis:{calendar:m},yaxis:{calendar:m},zaxis:{calendar:m}},polar:{radialaxis:{calendar:m}}},transforms:{filter:{valuecalendar:d(["Sets the calendar system to use for `value`, if it is a date."].join(" ")),targetcalendar:d(["Sets the calendar system to use for `target`, if it is an","array of dates. If `target` is a string (eg *x*) we use the","corresponding trace attribute (eg `xcalendar`) if it exists,","even if `targetcalendar` is provided."].join(" "))}}},layoutAttributes:A,handleDefaults:l,handleTraceDefaults:function(t,e,n,r){for(var i=0;i<n.length;i++)l(t,e,n[i]+"calendar",r.calendar)},CANONICAL_SUNDAY:{chinese:"2000-01-02",coptic:"2000-01-03",discworld:"2000-01-03",ethiopian:"2000-01-05",hebrew:"5000-01-01",islamic:"1000-01-02",julian:"2000-01-03",mayan:"5000-01-01",nanakshahi:"1000-01-05",nepali:"2000-01-05",persian:"1000-01-01",jalali:"1000-01-01",taiwan:"1000-01-04",thai:"2000-01-04",ummalqura:"1400-01-06"},CANONICAL_TICK:{chinese:"2000-01-01",coptic:"2000-01-01",discworld:"2000-01-01",ethiopian:"2000-01-01",hebrew:"5000-01-01",islamic:"1000-01-01",julian:"2000-01-01",mayan:"5000-01-01",nanakshahi:"1000-01-01",nepali:"2000-01-01",persian:"1000-01-01",jalali:"1000-01-01",taiwan:"1000-01-01",thai:"2000-01-01",ummalqura:"1400-01-01"},DFLTRANGE:{chinese:["2000-01-01","2001-01-01"],coptic:["1700-01-01","1701-01-01"],discworld:["1800-01-01","1801-01-01"],ethiopian:["2000-01-01","2001-01-01"],hebrew:["5700-01-01","5701-01-01"],islamic:["1400-01-01","1401-01-01"],julian:["2000-01-01","2001-01-01"],mayan:["5200-01-01","5201-01-01"],nanakshahi:["0500-01-01","0501-01-01"],nepali:["2000-01-01","2001-01-01"],persian:["1400-01-01","1401-01-01"],jalali:["1400-01-01","1401-01-01"],taiwan:["0100-01-01","0101-01-01"],thai:["2500-01-01","2501-01-01"],ummalqura:["1400-01-01","1401-01-01"]},getCal:h,worldCalFmt:function(t,e,n){for(var r,i,o,A,l,g=Math.floor((e+.05)/s)+a,d=h(n).fromJD(g),f=0;-1!==(f=t.indexOf("%",f));)"0"===(r=t.charAt(f+1))||"-"===r||"_"===r?(o=3,i=t.charAt(f+2),"_"===r&&(r="-")):(i=r,r="0",o=2),(A=u[i])?(l=A===c?c:d.formatDate(A[r]),t=t.substr(0,f)+l+t.substr(f+o),f+=l.length):f+=o;return t}}},{"../../constants/numerical":695,"../../lib":719,"./calendars":590}],592:[function(t,e,n){"use strict";n.defaults=["#1f77b4","#ff7f0e","#2ca02c","#d62728","#9467bd","#8c564b","#e377c2","#7f7f7f","#bcbd22","#17becf"],n.defaultLine="#444",n.lightLine="#eee",n.background="#fff",n.borderLine="#BEC8D9",n.lightFraction=1e3/11},{}],593:[function(t,e,n){"use strict";var r=t("tinycolor2"),i=t("fast-isnumeric"),o=e.exports={},a=t("./attributes");o.defaults=a.defaults;var s=o.defaultLine=a.defaultLine;o.lightLine=a.lightLine;var A=o.background=a.background;function l(t){if(i(t)||"string"!=typeof t)return t;var e=t.trim();if("rgb"!==e.substr(0,3))return t;var n=e.match(/^rgba?\s*\(([^()]*)\)$/);if(!n)return t;var r=n[1].trim().split(/\s*[\s,]\s*/),o="a"===e.charAt(3)&&4===r.length;if(!o&&3!==r.length)return t;for(var a=0;a<r.length;a++){if(!r[a].length)return t;if(r[a]=Number(r[a]),!(r[a]>=0))return t;if(3===a)r[a]>1&&(r[a]=1);else if(r[a]>=1)return t}var s=Math.round(255*r[0])+", "+Math.round(255*r[1])+", "+Math.round(255*r[2]);return o?"rgba("+s+", "+r[3]+")":"rgb("+s+")"}o.tinyRGB=function(t){var e=t.toRgb();return"rgb("+Math.round(e.r)+", "+Math.round(e.g)+", "+Math.round(e.b)+")"},o.rgb=function(t){return o.tinyRGB(r(t))},o.opacity=function(t){return t?r(t).getAlpha():0},o.addOpacity=function(t,e){var n=r(t).toRgb();return"rgba("+Math.round(n.r)+", "+Math.round(n.g)+", "+Math.round(n.b)+", "+e+")"},o.combine=function(t,e){var n=r(t).toRgb();if(1===n.a)return r(t).toRgbString();var i=r(e||A).toRgb(),o=1===i.a?i:{r:255*(1-i.a)+i.r*i.a,g:255*(1-i.a)+i.g*i.a,b:255*(1-i.a)+i.b*i.a},a={r:o.r*(1-n.a)+n.r*n.a,g:o.g*(1-n.a)+n.g*n.a,b:o.b*(1-n.a)+n.b*n.a};return r(a).toRgbString()},o.contrast=function(t,e,n){var i=r(t);return 1!==i.getAlpha()&&(i=r(o.combine(t,A))),(i.isDark()?e?i.lighten(e):A:n?i.darken(n):s).toString()},o.stroke=function(t,e){var n=r(e);t.style({stroke:o.tinyRGB(n),"stroke-opacity":n.getAlpha()})},o.fill=function(t,e){var n=r(e);t.style({fill:o.tinyRGB(n),"fill-opacity":n.getAlpha()})},o.clean=function(t){if(t&&"object"==typeof t){var e,n,r,i,a=Object.keys(t);for(e=0;e<a.length;e++)if(i=t[r=a[e]],"color"===r.substr(r.length-5))if(Array.isArray(i))for(n=0;n<i.length;n++)i[n]=l(i[n]);else t[r]=l(i);else if("colorscale"===r.substr(r.length-10)&&Array.isArray(i))for(n=0;n<i.length;n++)Array.isArray(i[n])&&(i[n][1]=l(i[n][1]));else if(Array.isArray(i)){var s=i[0];if(!Array.isArray(s)&&s&&"object"==typeof s)for(n=0;n<i.length;n++)o.clean(i[n])}else i&&"object"==typeof i&&o.clean(i)}}},{"./attributes":592,"fast-isnumeric":225,tinycolor2:537}],594:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/layout_attributes"),i=t("../../plots/font_attributes"),o=t("../../lib/extend").extendFlat,a=t("../../plot_api/edit_types").overrideAll;e.exports=a({thicknessmode:{valType:"enumerated",values:["fraction","pixels"],dflt:"pixels"},thickness:{valType:"number",min:0,dflt:30},lenmode:{valType:"enumerated",values:["fraction","pixels"],dflt:"fraction"},len:{valType:"number",min:0,dflt:1},x:{valType:"number",dflt:1.02,min:-2,max:3},xanchor:{valType:"enumerated",values:["left","center","right"],dflt:"left"},xpad:{valType:"number",min:0,dflt:10},y:{valType:"number",dflt:.5,min:-2,max:3},yanchor:{valType:"enumerated",values:["top","middle","bottom"],dflt:"middle"},ypad:{valType:"number",min:0,dflt:10},outlinecolor:r.linecolor,outlinewidth:r.linewidth,bordercolor:r.linecolor,borderwidth:{valType:"number",min:0,dflt:0},bgcolor:{valType:"color",dflt:"rgba(0,0,0,0)"},tickmode:r.tickmode,nticks:r.nticks,tick0:r.tick0,dtick:r.dtick,tickvals:r.tickvals,ticktext:r.ticktext,ticks:o({},r.ticks,{dflt:""}),ticklen:r.ticklen,tickwidth:r.tickwidth,tickcolor:r.tickcolor,showticklabels:r.showticklabels,tickfont:i({}),tickangle:r.tickangle,tickformat:r.tickformat,tickformatstops:r.tickformatstops,tickprefix:r.tickprefix,showtickprefix:r.showtickprefix,ticksuffix:r.ticksuffix,showticksuffix:r.showticksuffix,separatethousands:r.separatethousands,exponentformat:r.exponentformat,showexponent:r.showexponent,title:{text:{valType:"string"},font:i({}),side:{valType:"enumerated",values:["right","top","bottom"],dflt:"top"}},_deprecated:{title:{valType:"string"},titlefont:i({}),titleside:{valType:"enumerated",values:["right","top","bottom"],dflt:"top"}}},"colorbars","from-root")},{"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plots/cartesian/layout_attributes":779,"../../plots/font_attributes":793}],595:[function(t,e,n){"use strict";e.exports={cn:{colorbar:"colorbar",cbbg:"cbbg",cbfill:"cbfill",cbfills:"cbfills",cbline:"cbline",cblines:"cblines",cbaxis:"cbaxis",cbtitleunshift:"cbtitleunshift",cbtitle:"cbtitle",cboutline:"cboutline",crisp:"crisp",jsPlaceholder:"js-placeholder"}}},{}],596:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plot_api/plot_template"),o=t("../../plots/cartesian/tick_value_defaults"),a=t("../../plots/cartesian/tick_mark_defaults"),s=t("../../plots/cartesian/tick_label_defaults"),A=t("./attributes");e.exports=function(t,e,n){var l=i.newContainer(e,"colorbar"),c=t.colorbar||{};function u(t,e){return r.coerce(c,l,A,t,e)}var g=u("thicknessmode");u("thickness","fraction"===g?30/(n.width-n.margin.l-n.margin.r):30);var h=u("lenmode");u("len","fraction"===h?1:n.height-n.margin.t-n.margin.b),u("x"),u("xanchor"),u("xpad"),u("y"),u("yanchor"),u("ypad"),r.noneOrAll(c,l,["x","y"]),u("outlinecolor"),u("outlinewidth"),u("bordercolor"),u("borderwidth"),u("bgcolor"),o(c,l,u,"linear");var d={outerTicks:!1,font:n.font};s(c,l,u,"linear",d),a(c,l,u,"linear",d),u("title.text",n._dfltTitle.colorbar),r.coerceFont(u,"title.font",n.font),u("title.side")}},{"../../lib":719,"../../plot_api/plot_template":757,"../../plots/cartesian/tick_label_defaults":786,"../../plots/cartesian/tick_mark_defaults":787,"../../plots/cartesian/tick_value_defaults":788,"./attributes":594}],597:[function(t,e,n){"use strict";var r=t("d3"),i=t("tinycolor2"),o=t("../../plots/plots"),a=t("../../registry"),s=t("../../plots/cartesian/axes"),A=t("../dragelement"),l=t("../../lib"),c=t("../../lib/extend").extendFlat,u=t("../../lib/setcursor"),g=t("../drawing"),h=t("../color"),d=t("../titles"),f=t("../../lib/svg_text_utils"),p=t("../colorscale/helpers").flipScale,I=t("../../plots/cartesian/axis_defaults"),C=t("../../plots/cartesian/position_defaults"),m=t("../../plots/cartesian/layout_attributes"),y=t("../../constants/alignment"),E=y.LINE_SPACING,v=y.FROM_TL,B=y.FROM_BR,b=t("./constants").cn;e.exports={draw:function(t){var e=t._fullLayout._infolayer.selectAll("g."+b.colorbar).data(function(t){var e,n,r,i,o=t._fullLayout,a=t.calcdata,s=[];function A(t){return c(t,{_fillcolor:null,_line:{color:null,width:null,dash:null},_levels:{start:null,end:null,size:null},_filllevels:null,_fillgradient:null,_zrange:null})}function l(){"function"==typeof i.calc?i.calc(t,r,e):(e._fillgradient=n.reversescale?p(n.colorscale):n.colorscale,e._zrange=[n[i.min],n[i.max]])}for(var u=0;u<a.length;u++){var g=a[u],h=(r=g[0].trace)._module.colorbar;if(!0===r.visible&&h)for(var d=Array.isArray(h),f=d?h:[h],I=0;I<f.length;I++){var C=(i=f[I]).container;(n=C?r[C]:r)&&n.showscale&&((e=A(n.colorbar))._id="cb"+r.uid+(d&&C?"-"+C:""),e._traceIndex=r.index,e._propPrefix=(C?C+".":"")+"colorbar.",e._meta=r._meta,l(),s.push(e))}}for(var m in o._colorAxes)if((n=o[m]).showscale){var y=o._colorAxes[m];(e=A(n.colorbar))._id="cb"+m,e._propPrefix=m+".colorbar.",e._meta=o._meta,i={min:"cmin",max:"cmax"},"heatmap"!==y[0]&&(r=y[1],i.calc=r._module.colorbar.calc),l(),s.push(e)}return s}(t),function(t){return t._id});e.enter().append("g").attr("class",function(t){return t._id}).classed(b.colorbar,!0),e.each(function(e){var n=r.select(this);l.ensureSingle(n,"rect",b.cbbg),l.ensureSingle(n,"g",b.cbfills),l.ensureSingle(n,"g",b.cblines),l.ensureSingle(n,"g",b.cbaxis,function(t){t.classed(b.crisp,!0)}),l.ensureSingle(n,"g",b.cbtitleunshift,function(t){t.append("g").classed(b.cbtitle,!0)}),l.ensureSingle(n,"rect",b.cboutline);var p=function(t,e,n){var a=n._fullLayout,A=a._size,u=e._fillcolor,p=e._line,y=e.title,Q=y.side,w=e._zrange||r.extent(("function"==typeof u?u:p.color).domain()),x="function"==typeof p.color?p.color:function(){return p.color},_="function"==typeof u?u:function(){return u},D=e._levels,M=function(t,e,n){var r,i,o=e._levels,a=[],s=[],A=o.end+o.size/100,l=o.size,c=1.001*n[0]-.001*n[1],u=1.001*n[1]-.001*n[0];for(i=0;i<1e5&&(r=o.start+i*l,!(l>0?r>=A:r<=A));i++)r>c&&r<u&&a.push(r);if(e._fillgradient)s=[0];else if("function"==typeof e._fillcolor){var g=e._filllevels;if(g)for(A=g.end+g.size/100,l=g.size,i=0;i<1e5&&(r=g.start+i*l,!(l>0?r>=A:r<=A));i++)r>n[0]&&r<n[1]&&s.push(r);else(s=a.map(function(t){return t-o.size/2})).push(s[s.length-1]+o.size)}else e._fillcolor&&"string"==typeof e._fillcolor&&(s=[0]);return o.size<0&&(a.reverse(),s.reverse()),{line:a,fill:s}}(0,e,w),k=M.fill,S=M.line,L=Math.round(e.thickness*("fraction"===e.thicknessmode?A.w:1)),R=L/A.w,N=Math.round(e.len*("fraction"===e.lenmode?A.h:1)),F=N/A.h,T=e.xpad/A.w,U=(e.borderwidth+e.outlinewidth)/2,O=e.ypad/A.h,G=Math.round(e.x*A.w+e.xpad),H=e.x-R*({middle:.5,right:1}[e.xanchor]||0),j=e.y+F*(({top:-.5,bottom:.5}[e.yanchor]||0)-.5),Y=Math.round(A.h*(1-j)),P=Y-N;e._lenFrac=F,e._thickFrac=R,e._xLeftFrac=H,e._yBottomFrac=j;var K=function(t,e,n){var r=t._fullLayout,i={type:"linear",range:n,tickmode:e.tickmode,nticks:e.nticks,tick0:e.tick0,dtick:e.dtick,tickvals:e.tickvals,ticktext:e.ticktext,ticks:e.ticks,ticklen:e.ticklen,tickwidth:e.tickwidth,tickcolor:e.tickcolor,showticklabels:e.showticklabels,tickfont:e.tickfont,tickangle:e.tickangle,tickformat:e.tickformat,exponentformat:e.exponentformat,separatethousands:e.separatethousands,showexponent:e.showexponent,showtickprefix:e.showtickprefix,tickprefix:e.tickprefix,showticksuffix:e.showticksuffix,ticksuffix:e.ticksuffix,title:e.title,showline:!0,anchor:"free",side:"right",position:1},o={type:"linear",_id:"y"+e._id},a={letter:"y",font:r.font,noHover:!0,noTickson:!0,calendar:r.calendar};function s(t,e){return l.coerce(i,o,m,t,e)}return I(i,o,s,a,r),C(i,o,s,a),o}(n,e,w);if(K.position=e.x+T+R,-1!==["top","bottom"].indexOf(Q)&&(K.title.side=Q,K.titlex=e.x+T,K.titley=j+("top"===y.side?F-O:O)),p.color&&"auto"===e.tickmode){K.tickmode="linear",K.tick0=D.start;var z=D.size,J=l.constrain((Y-P)/50,4,15)+1,q=(w[1]-w[0])/((e.nticks||J)*z);if(q>1){var V=Math.pow(10,Math.floor(Math.log(q)/Math.LN10));z*=V*l.roundUp(q/V,[2,5,10]),(Math.abs(D.start)/D.size+1e-6)%1<2e-6&&(K.tick0=0)}K.dtick=z}K.domain=[j+O,j+F-O],K.setScale(),t.attr("transform","translate("+Math.round(A.l)+","+Math.round(A.t)+")");var W,X=t.select("."+b.cbtitleunshift).attr("transform","translate(-"+Math.round(A.l)+",-"+Math.round(A.t)+")"),Z=t.select("."+b.cbaxis),$=0;function tt(r,i){var o={propContainer:K,propName:e._propPrefix+"title",traceIndex:e._traceIndex,_meta:e._meta,placeholder:a._dfltTitle.colorbar,containerGroup:t.select("."+b.cbtitle)},s="h"===r.charAt(0)?r.substr(1):"h"+r;t.selectAll("."+s+",."+s+"-math-group").remove(),d.draw(n,r,c(o,i||{}))}return l.syncOrAsync([o.previousPromises,function(){if(-1!==["top","bottom"].indexOf(Q)){var t,n=A.l+(e.x+T)*A.w,r=K.title.font.size;t="top"===Q?(1-(j+F-O))*A.h+A.t+3+.75*r:(1-(j+O))*A.h+A.t-3-.25*r,tt(K._id+"title",{attributes:{x:n,y:t,"text-anchor":"start"}})}},function(){if(-1!==["top","bottom"].indexOf(Q)){var o=t.select("."+b.cbtitle),a=o.select("text"),c=[-e.outlinewidth/2,e.outlinewidth/2],u=o.select(".h"+K._id+"title-math-group").node(),h=15.6;if(a.node()&&(h=parseInt(a.node().style.fontSize,10)*E),u?($=g.bBox(u).height)>h&&(c[1]-=($-h)/2):a.node()&&!a.classed(b.jsPlaceholder)&&($=g.bBox(a.node()).height),$){if($+=5,"top"===Q)K.domain[1]-=$/A.h,c[1]*=-1;else{K.domain[0]+=$/A.h;var d=f.lineCount(a);c[1]+=(1-d)*h}o.attr("transform","translate("+c+")"),K.setScale()}}t.selectAll("."+b.cbfills+",."+b.cblines).attr("transform","translate(0,"+Math.round(A.h*(1-K.domain[1]))+")"),Z.attr("transform","translate(0,"+Math.round(-A.t)+")");var I=t.select("."+b.cbfills).selectAll("rect."+b.cbfill).data(k);I.enter().append("rect").classed(b.cbfill,!0).style("stroke","none"),I.exit().remove();var C=w.map(K.c2p).map(Math.round).sort(function(t,e){return t-e});I.each(function(t,o){var a=[0===o?w[0]:(k[o]+k[o-1])/2,o===k.length-1?w[1]:(k[o]+k[o+1])/2].map(K.c2p).map(Math.round);a[1]=l.constrain(a[1]+(a[1]>a[0])?1:-1,C[0],C[1]);var s=r.select(this).attr({x:G,width:Math.max(L,2),y:r.min(a),height:Math.max(r.max(a)-r.min(a),2)});if(e._fillgradient)g.gradient(s,n,e._id,"vertical",e._fillgradient,"fill");else{var A=_(t).replace("e-","");s.attr("fill",i(A).toHexString())}});var m=t.select("."+b.cblines).selectAll("path."+b.cbline).data(p.color&&p.width?S:[]);m.enter().append("path").classed(b.cbline,!0),m.exit().remove(),m.each(function(t){r.select(this).attr("d","M"+G+","+(Math.round(K.c2p(t))+p.width/2%1)+"h"+L).call(g.lineGroupStyle,p.width,x(t),p.dash)}),Z.selectAll("g."+K._id+"tick,path").remove();var y=G+L+(e.outlinewidth||0)/2-("outside"===e.ticks?1:0),v=s.calcTicks(K),B=s.makeTransFn(K),D=s.getTickSigns(K)[2];return s.drawTicks(n,K,{vals:"inside"===K.ticks?s.clipEnds(K,v):v,layer:Z,path:s.makeTickPath(K,y,D),transFn:B}),s.drawLabels(n,K,{vals:v,layer:Z,transFn:B,labelFns:s.makeLabelFns(K,y)})},function(){if(-1===["top","bottom"].indexOf(Q)){var t=K.title.font.size,e=K._offset+K._length/2,i=A.l+(K.position||0)*A.w+("right"===K.side?10+t*(K.showticklabels?1:.5):-10-t*(K.showticklabels?.5:0));tt("h"+K._id+"title",{avoid:{selection:r.select(n).selectAll("g."+K._id+"tick"),side:Q,offsetLeft:A.l,offsetTop:0,maxShift:a.width},attributes:{x:i,y:e,"text-anchor":"middle"},transform:{rotate:"-90",offset:0}})}},o.previousPromises,function(){var r=L+e.outlinewidth/2+g.bBox(Z.node()).width;if((W=X.select("text")).node()&&!W.classed(b.jsPlaceholder)){var i,a=X.select(".h"+K._id+"title-math-group").node();i=a&&-1!==["top","bottom"].indexOf(Q)?g.bBox(a).width:g.bBox(X.node()).right-G-A.l,r=Math.max(r,i)}var s=2*e.xpad+r+e.borderwidth+e.outlinewidth/2,l=Y-P;t.select("."+b.cbbg).attr({x:G-e.xpad-(e.borderwidth+e.outlinewidth)/2,y:P-U,width:Math.max(s,2),height:Math.max(l+2*U,2)}).call(h.fill,e.bgcolor).call(h.stroke,e.bordercolor).style("stroke-width",e.borderwidth),t.selectAll("."+b.cboutline).attr({x:G,y:P+e.ypad+("top"===Q?$:0),width:Math.max(L,2),height:Math.max(l-2*e.ypad-$,2)}).call(h.stroke,e.outlinecolor).style({fill:"none","stroke-width":e.outlinewidth});var c=({center:.5,right:1}[e.xanchor]||0)*s;t.attr("transform","translate("+(A.l-c)+","+A.t+")");var u={},d=v[e.yanchor],f=B[e.yanchor];"pixels"===e.lenmode?(u.y=e.y,u.t=l*d,u.b=l*f):(u.t=u.b=0,u.yt=e.y+e.len*d,u.yb=e.y-e.len*f);var p=v[e.xanchor],I=B[e.xanchor];if("pixels"===e.thicknessmode)u.x=e.x,u.l=s*p,u.r=s*I;else{var C=s-L;u.l=C*p,u.r=C*I,u.xl=e.x-e.thickness*p,u.xr=e.x+e.thickness*I}o.autoMargin(n,e._id,u)}],n)}(n,e,t);p&&p.then&&(t._promises||[]).push(p),t._context.edits.colorbarPosition&&function(t,e,n){var r,i,o,s=n._fullLayout._size;A.init({element:t.node(),gd:n,prepFn:function(){r=t.attr("transform"),u(t)},moveFn:function(n,a){t.attr("transform",r+" translate("+n+","+a+")"),i=A.align(e._xLeftFrac+n/s.w,e._thickFrac,0,1,e.xanchor),o=A.align(e._yBottomFrac-a/s.h,e._lenFrac,0,1,e.yanchor);var l=A.getCursor(i,o,e.xanchor,e.yanchor);u(t,l)},doneFn:function(){if(u(t),void 0!==i&&void 0!==o){var r={};r[e._propPrefix+"x"]=i,r[e._propPrefix+"y"]=o,void 0!==e._traceIndex?a.call("_guiRestyle",n,r,e._traceIndex):a.call("_guiRelayout",n,r)}}})}(n,e,t)}),e.exit().each(function(e){o.autoMargin(t,e._id)}).remove(),e.order()}}},{"../../constants/alignment":688,"../../lib":719,"../../lib/extend":710,"../../lib/setcursor":739,"../../lib/svg_text_utils":743,"../../plots/cartesian/axes":767,"../../plots/cartesian/axis_defaults":769,"../../plots/cartesian/layout_attributes":779,"../../plots/cartesian/position_defaults":782,"../../plots/plots":828,"../../registry":847,"../color":593,"../colorscale/helpers":604,"../dragelement":611,"../drawing":614,"../titles":681,"./constants":595,d3:163,tinycolor2:537}],598:[function(t,e,n){"use strict";var r=t("../../lib");e.exports=function(t){return r.isPlainObject(t.colorbar)}},{"../../lib":719}],599:[function(t,e,n){"use strict";e.exports={moduleType:"component",name:"colorbar",attributes:t("./attributes"),supplyDefaults:t("./defaults"),draw:t("./draw").draw,hasColorbar:t("./has_colorbar")}},{"./attributes":594,"./defaults":596,"./draw":597,"./has_colorbar":598}],600:[function(t,e,n){"use strict";var r=t("../colorbar/attributes"),i=t("../../lib/regex").counter,o=t("./scales.js").scales;function a(t){return"`"+t+"`"}Object.keys(o),e.exports=function(t,e){t=t||"";var n,s=(e=e||{}).cLetter||"c",A=("onlyIfNumerical"in e?e.onlyIfNumerical:Boolean(t),"noScale"in e?e.noScale:"marker.line"===t),l="showScaleDflt"in e?e.showScaleDflt:"z"===s,c="string"==typeof e.colorscaleDflt?o[e.colorscaleDflt]:null,u=e.editTypeOverride||"",g=t?t+".":"";"colorAttr"in e?(n=e.colorAttr,e.colorAttr):a(g+(n={z:"z",c:"color"}[s]));var h=s+"auto",d=s+"min",f=s+"max",p=s+"mid",I=(a(g+h),a(g+d),a(g+f),{});I[d]=I[f]=void 0;var C={};C[h]=!1;var m={};return"color"===n&&(m.color={valType:"color",arrayOk:!0,editType:u||"style"},e.anim&&(m.color.anim=!0)),m[h]={valType:"boolean",dflt:!0,editType:"calc",impliedEdits:I},m[d]={valType:"number",dflt:null,editType:u||"plot",impliedEdits:C},m[f]={valType:"number",dflt:null,editType:u||"plot",impliedEdits:C},m[p]={valType:"number",dflt:null,editType:"calc",impliedEdits:I},m.colorscale={valType:"colorscale",editType:"calc",dflt:c,impliedEdits:{autocolorscale:!1}},m.autocolorscale={valType:"boolean",dflt:!1!==e.autoColorDflt,editType:"calc",impliedEdits:{colorscale:void 0}},m.reversescale={valType:"boolean",dflt:!1,editType:"plot"},A||(m.showscale={valType:"boolean",dflt:l,editType:"calc"},m.colorbar=r),e.noColorAxis||(m.coloraxis={valType:"subplotid",regex:i("coloraxis"),dflt:null,editType:"calc"}),m}},{"../../lib/regex":735,"../colorbar/attributes":594,"./scales.js":608}],601:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib"),o=t("./helpers").extractOpts;e.exports=function(t,e,n){var a,s=t._fullLayout,A=n.vals,l=n.containerStr,c=l?i.nestedProperty(e,l).get():e,u=o(c),g=!1!==u.auto,h=u.min,d=u.max,f=u.mid,p=function(){return i.aggNums(Math.min,null,A)},I=function(){return i.aggNums(Math.max,null,A)};void 0===h?h=p():g&&(h=c._colorAx&&r(h)?Math.min(h,p()):p()),void 0===d?d=I():g&&(d=c._colorAx&&r(d)?Math.max(d,I()):I()),g&&void 0!==f&&(d-f>f-h?h=f-(d-f):d-f<f-h&&(d=f+(f-h))),h===d&&(h-=.5,d+=.5),u._sync("min",h),u._sync("max",d),u.autocolorscale&&(a=h*d<0?s.colorscale.diverging:h>=0?s.colorscale.sequential:s.colorscale.sequentialminus,u._sync("colorscale",a))}},{"../../lib":719,"./helpers":604,"fast-isnumeric":225}],602:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./helpers").hasColorscale,o=t("./helpers").extractOpts;e.exports=function(t,e){function n(t,e){var n=t["_"+e];void 0!==n&&(t[e]=n)}function a(t,i){var a=i.container?r.nestedProperty(t,i.container).get():t;if(a)if(a.coloraxis)a._colorAx=e[a.coloraxis];else{var s=o(a),A=s.auto;(A||void 0===s.min)&&n(a,i.min),(A||void 0===s.max)&&n(a,i.max),s.autocolorscale&&n(a,"colorscale")}}for(var s=0;s<t.length;s++){var A=t[s],l=A._module.colorbar;if(l)if(Array.isArray(l))for(var c=0;c<l.length;c++)a(A,l[c]);else a(A,l);i(A,"marker.line")&&a(A,{container:"marker.line",min:"cmin",max:"cmax"})}for(var u in e._colorAxes)a(e[u],{min:"cmin",max:"cmax"})}},{"../../lib":719,"./helpers":604}],603:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib"),o=t("../colorbar/has_colorbar"),a=t("../colorbar/defaults"),s=t("./scales").isValid,A=t("../../registry").traceIs;function l(t,e){var n=e.slice(0,e.length-1);return e?i.nestedProperty(t,n).get()||{}:t}e.exports=function t(e,n,c,u,g){var h=g.prefix,d=g.cLetter,f="_module"in n,p=l(e,h),I=l(n,h),C=l(n._template||{},h)||{},m=function(){return delete e.coloraxis,delete n.coloraxis,t(e,n,c,u,g)};if(f){var y=c._colorAxes||{},E=u(h+"coloraxis");if(E){var v=A(n,"contour")&&i.nestedProperty(n,"contours.coloring").get()||"heatmap",B=y[E];return void(B?(B[2].push(m),B[0]!==v&&(B[0]=!1,i.warn(["Ignoring coloraxis:",E,"setting","as it is linked to incompatible colorscales."].join(" ")))):y[E]=[v,n,[m]])}}var b=p[d+"min"],Q=p[d+"max"],w=r(b)&&r(Q)&&b<Q;u(h+d+"auto",!w)?u(h+d+"mid"):(u(h+d+"min"),u(h+d+"max"));var x,_,D=p.colorscale,M=C.colorscale;void 0!==D&&(x=!s(D)),void 0!==M&&(x=!s(M)),u(h+"autocolorscale",x),u(h+"colorscale"),u(h+"reversescale"),"marker.line."!==h&&(h&&f&&(_=o(p)),u(h+"showscale",_)&&a(p,I,c))}},{"../../lib":719,"../../registry":847,"../colorbar/defaults":596,"../colorbar/has_colorbar":598,"./scales":608,"fast-isnumeric":225}],604:[function(t,e,n){"use strict";var r=t("d3"),i=t("tinycolor2"),o=t("fast-isnumeric"),a=t("../../lib"),s=t("../color"),A=t("./scales").isValid,l=["showscale","autocolorscale","colorscale","reversescale","colorbar"],c=["min","max","mid","auto"];function u(t){var e,n,r,i=t._colorAx,o=i||t,a={};for(n=0;n<l.length;n++)a[r=l[n]]=o[r];if(i)for(e="c",n=0;n<c.length;n++)a[r=c[n]]=o["c"+r];else{var s;for(n=0;n<c.length;n++)(s="c"+(r=c[n]))in o?a[r]=o[s]:(s="z"+r)in o&&(a[r]=o[s]);e=s.charAt(0)}return a._sync=function(t,n){var r=-1!==c.indexOf(t)?e+t:t;o[r]=o["_"+r]=n},a}function g(t){for(var e=u(t),n=e.min,r=e.max,i=e.reversescale?h(e.colorscale):e.colorscale,o=i.length,a=new Array(o),s=new Array(o),A=0;A<o;A++){var l=i[A];a[A]=n+l[0]*(r-n),s[A]=l[1]}return{domain:a,range:s}}function h(t){for(var e=t.length,n=new Array(e),r=e-1,i=0;r>=0;r--,i++){var o=t[r];n[i]=[1-o[0],o[1]]}return n}function d(t,e){e=e||{};for(var n=t.domain,a=t.range,A=a.length,l=new Array(A),c=0;c<A;c++){var u=i(a[c]).toRgb();l[c]=[u.r,u.g,u.b,u.a]}var g,h=r.scale.linear().domain(n).range(l).clamp(!0),d=e.noNumericCheck,p=e.returnArray;return(g=d&&p?h:d?function(t){return f(h(t))}:p?function(t){return o(t)?h(t):i(t).isValid()?t:s.defaultLine}:function(t){return o(t)?f(h(t)):i(t).isValid()?t:s.defaultLine}).domain=h.domain,g.range=function(){return a},g}function f(t){var e={r:t[0],g:t[1],b:t[2],a:t[3]};return i(e).toRgbString()}e.exports={hasColorscale:function(t,e){var n=e?a.nestedProperty(t,e).get()||{}:t,r=n.color,i=!1;if(a.isArrayOrTypedArray(r))for(var s=0;s<r.length;s++)if(o(r[s])){i=!0;break}return a.isPlainObject(n)&&(i||!0===n.showscale||o(n.cmin)&&o(n.cmax)||A(n.colorscale)||a.isPlainObject(n.colorbar))},extractOpts:u,extractScale:g,flipScale:h,makeColorScaleFunc:d,makeColorScaleFuncFromTrace:function(t,e){return d(g(t),e)}}},{"../../lib":719,"../color":593,"./scales":608,d3:163,"fast-isnumeric":225,tinycolor2:537}],605:[function(t,e,n){"use strict";var r=t("./scales"),i=t("./helpers");e.exports={moduleType:"component",name:"colorscale",attributes:t("./attributes"),layoutAttributes:t("./layout_attributes"),supplyLayoutDefaults:t("./layout_defaults"),handleDefaults:t("./defaults"),crossTraceDefaults:t("./cross_trace_defaults"),calc:t("./calc"),scales:r.scales,defaultScale:r.defaultScale,getScale:r.get,isValidScale:r.isValid,hasColorscale:i.hasColorscale,extractOpts:i.extractOpts,extractScale:i.extractScale,flipScale:i.flipScale,makeColorScaleFunc:i.makeColorScaleFunc,makeColorScaleFuncFromTrace:i.makeColorScaleFuncFromTrace}},{"./attributes":600,"./calc":601,"./cross_trace_defaults":602,"./defaults":603,"./helpers":604,"./layout_attributes":606,"./layout_defaults":607,"./scales":608}],606:[function(t,e,n){"use strict";var r=t("../../lib/extend").extendFlat,i=t("./attributes"),o=t("./scales").scales;e.exports={editType:"calc",colorscale:{editType:"calc",sequential:{valType:"colorscale",dflt:o.Reds,editType:"calc"},sequentialminus:{valType:"colorscale",dflt:o.Blues,editType:"calc"},diverging:{valType:"colorscale",dflt:o.RdBu,editType:"calc"}},coloraxis:r({_isSubplotObj:!0,editType:"calc"},i("",{colorAttr:"corresponding trace color array(s)",noColorAxis:!0,showScaleDflt:!0}))}},{"../../lib/extend":710,"./attributes":600,"./scales":608}],607:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plot_api/plot_template"),o=t("./layout_attributes"),a=t("./defaults");e.exports=function(t,e){function n(n,i){return r.coerce(t,e,o,n,i)}n("colorscale.sequential"),n("colorscale.sequentialminus"),n("colorscale.diverging");var s,A,l=e._colorAxes;function c(t,e){return r.coerce(s,A,o.coloraxis,t,e)}for(var u in l){var g=l[u];if(g[0])s=t[u]||{},(A=i.newContainer(e,u,"coloraxis"))._name=u,a(s,A,e,c,{prefix:"",cLetter:"c"});else{for(var h=0;h<g[2].length;h++)g[2][h]();delete e._colorAxes[u]}}}},{"../../lib":719,"../../plot_api/plot_template":757,"./defaults":603,"./layout_attributes":606}],608:[function(t,e,n){"use strict";var r=t("tinycolor2"),i={Greys:[[0,"rgb(0,0,0)"],[1,"rgb(255,255,255)"]],YlGnBu:[[0,"rgb(8,29,88)"],[.125,"rgb(37,52,148)"],[.25,"rgb(34,94,168)"],[.375,"rgb(29,145,192)"],[.5,"rgb(65,182,196)"],[.625,"rgb(127,205,187)"],[.75,"rgb(199,233,180)"],[.875,"rgb(237,248,217)"],[1,"rgb(255,255,217)"]],Greens:[[0,"rgb(0,68,27)"],[.125,"rgb(0,109,44)"],[.25,"rgb(35,139,69)"],[.375,"rgb(65,171,93)"],[.5,"rgb(116,196,118)"],[.625,"rgb(161,217,155)"],[.75,"rgb(199,233,192)"],[.875,"rgb(229,245,224)"],[1,"rgb(247,252,245)"]],YlOrRd:[[0,"rgb(128,0,38)"],[.125,"rgb(189,0,38)"],[.25,"rgb(227,26,28)"],[.375,"rgb(252,78,42)"],[.5,"rgb(253,141,60)"],[.625,"rgb(254,178,76)"],[.75,"rgb(254,217,118)"],[.875,"rgb(255,237,160)"],[1,"rgb(255,255,204)"]],Bluered:[[0,"rgb(0,0,255)"],[1,"rgb(255,0,0)"]],RdBu:[[0,"rgb(5,10,172)"],[.35,"rgb(106,137,247)"],[.5,"rgb(190,190,190)"],[.6,"rgb(220,170,132)"],[.7,"rgb(230,145,90)"],[1,"rgb(178,10,28)"]],Reds:[[0,"rgb(220,220,220)"],[.2,"rgb(245,195,157)"],[.4,"rgb(245,160,105)"],[1,"rgb(178,10,28)"]],Blues:[[0,"rgb(5,10,172)"],[.35,"rgb(40,60,190)"],[.5,"rgb(70,100,245)"],[.6,"rgb(90,120,245)"],[.7,"rgb(106,137,247)"],[1,"rgb(220,220,220)"]],Picnic:[[0,"rgb(0,0,255)"],[.1,"rgb(51,153,255)"],[.2,"rgb(102,204,255)"],[.3,"rgb(153,204,255)"],[.4,"rgb(204,204,255)"],[.5,"rgb(255,255,255)"],[.6,"rgb(255,204,255)"],[.7,"rgb(255,153,255)"],[.8,"rgb(255,102,204)"],[.9,"rgb(255,102,102)"],[1,"rgb(255,0,0)"]],Rainbow:[[0,"rgb(150,0,90)"],[.125,"rgb(0,0,200)"],[.25,"rgb(0,25,255)"],[.375,"rgb(0,152,255)"],[.5,"rgb(44,255,150)"],[.625,"rgb(151,255,0)"],[.75,"rgb(255,234,0)"],[.875,"rgb(255,111,0)"],[1,"rgb(255,0,0)"]],Portland:[[0,"rgb(12,51,131)"],[.25,"rgb(10,136,186)"],[.5,"rgb(242,211,56)"],[.75,"rgb(242,143,56)"],[1,"rgb(217,30,30)"]],Jet:[[0,"rgb(0,0,131)"],[.125,"rgb(0,60,170)"],[.375,"rgb(5,255,255)"],[.625,"rgb(255,255,0)"],[.875,"rgb(250,0,0)"],[1,"rgb(128,0,0)"]],Hot:[[0,"rgb(0,0,0)"],[.3,"rgb(230,0,0)"],[.6,"rgb(255,210,0)"],[1,"rgb(255,255,255)"]],Blackbody:[[0,"rgb(0,0,0)"],[.2,"rgb(230,0,0)"],[.4,"rgb(230,210,0)"],[.7,"rgb(255,255,255)"],[1,"rgb(160,200,255)"]],Earth:[[0,"rgb(0,0,130)"],[.1,"rgb(0,180,180)"],[.2,"rgb(40,210,40)"],[.4,"rgb(230,230,50)"],[.6,"rgb(120,70,20)"],[1,"rgb(255,255,255)"]],Electric:[[0,"rgb(0,0,0)"],[.15,"rgb(30,0,100)"],[.4,"rgb(120,0,100)"],[.6,"rgb(160,90,0)"],[.8,"rgb(230,200,0)"],[1,"rgb(255,250,220)"]],Viridis:[[0,"#440154"],[.06274509803921569,"#48186a"],[.12549019607843137,"#472d7b"],[.18823529411764706,"#424086"],[.25098039215686274,"#3b528b"],[.3137254901960784,"#33638d"],[.3764705882352941,"#2c728e"],[.4392156862745098,"#26828e"],[.5019607843137255,"#21918c"],[.5647058823529412,"#1fa088"],[.6274509803921569,"#28ae80"],[.6901960784313725,"#3fbc73"],[.7529411764705882,"#5ec962"],[.8156862745098039,"#84d44b"],[.8784313725490196,"#addc30"],[.9411764705882353,"#d8e219"],[1,"#fde725"]],Cividis:[[0,"rgb(0,32,76)"],[.058824,"rgb(0,42,102)"],[.117647,"rgb(0,52,110)"],[.176471,"rgb(39,63,108)"],[.235294,"rgb(60,74,107)"],[.294118,"rgb(76,85,107)"],[.352941,"rgb(91,95,109)"],[.411765,"rgb(104,106,112)"],[.470588,"rgb(117,117,117)"],[.529412,"rgb(131,129,120)"],[.588235,"rgb(146,140,120)"],[.647059,"rgb(161,152,118)"],[.705882,"rgb(176,165,114)"],[.764706,"rgb(192,177,109)"],[.823529,"rgb(209,191,102)"],[.882353,"rgb(225,204,92)"],[.941176,"rgb(243,219,79)"],[1,"rgb(255,233,69)"]]},o=i.RdBu;function a(t){var e=0;if(!Array.isArray(t)||t.length<2)return!1;if(!t[0]||!t[t.length-1])return!1;if(0!=+t[0][0]||1!=+t[t.length-1][0])return!1;for(var n=0;n<t.length;n++){var i=t[n];if(2!==i.length||+i[0]<e||!r(i[1]).isValid())return!1;e=+i[0]}return!0}e.exports={scales:i,defaultScale:o,get:function(t,e){if(e||(e=o),!t)return e;function n(){try{t=i[t]||JSON.parse(t)}catch(n){t=e}}return"string"==typeof t&&(n(),"string"==typeof t&&n()),a(t)?t:e},isValid:function(t){return void 0!==i[t]||a(t)}}},{tinycolor2:537}],609:[function(t,e,n){"use strict";e.exports=function(t,e,n,r,i){var o=(t-n)/(r-n),a=o+e/(r-n),s=(o+a)/2;return"left"===i||"bottom"===i?o:"center"===i||"middle"===i?s:"right"===i||"top"===i?a:o<2/3-s?o:a>4/3-s?a:s}},{}],610:[function(t,e,n){"use strict";var r=t("../../lib"),i=[["sw-resize","s-resize","se-resize"],["w-resize","move","e-resize"],["nw-resize","n-resize","ne-resize"]];e.exports=function(t,e,n,o){return t="left"===n?0:"center"===n?1:"right"===n?2:r.constrain(Math.floor(3*t),0,2),e="bottom"===o?0:"middle"===o?1:"top"===o?2:r.constrain(Math.floor(3*e),0,2),i[e][t]}},{"../../lib":719}],611:[function(t,e,n){"use strict";var r=t("mouse-event-offset"),i=t("has-hover"),o=t("has-passive-events"),a=t("../../lib").removeElement,s=t("../../plots/cartesian/constants"),A=e.exports={};A.align=t("./align"),A.getCursor=t("./cursor");var l=t("./unhover");function c(){var t=document.createElement("div");t.className="dragcover";var e=t.style;return e.position="fixed",e.left=0,e.right=0,e.top=0,e.bottom=0,e.zIndex=999999999,e.background="none",document.body.appendChild(t),t}function u(t){return r(t.changedTouches?t.changedTouches[0]:t,document.body)}A.unhover=l.wrapped,A.unhoverRaw=l.raw,A.init=function(t){var e,n,r,l,g,h,d,f,p=t.gd,I=1,C=p._context.doubleClickDelay,m=t.element;p._mouseDownTime||(p._mouseDownTime=0),m.style.pointerEvents="all",m.onmousedown=E,o?(m._ontouchstart&&m.removeEventListener("touchstart",m._ontouchstart),m._ontouchstart=E,m.addEventListener("touchstart",E,{passive:!1})):m.ontouchstart=E;var y=t.clampFn||function(t,e,n){return Math.abs(t)<n&&(t=0),Math.abs(e)<n&&(e=0),[t,e]};function E(o){p._dragged=!1,p._dragging=!0;var a=u(o);e=a[0],n=a[1],d=o.target,h=o,f=2===o.buttons||o.ctrlKey,void 0===o.clientX&&void 0===o.clientY&&(o.clientX=e,o.clientY=n),(r=(new Date).getTime())-p._mouseDownTime<C?I+=1:(I=1,p._mouseDownTime=r),t.prepFn&&t.prepFn(o,e,n),i&&!f?(g=c()).style.cursor=window.getComputedStyle(m).cursor:i||(g=document,l=window.getComputedStyle(document.documentElement).cursor,document.documentElement.style.cursor=window.getComputedStyle(m).cursor),document.addEventListener("mouseup",B),document.addEventListener("touchend",B),!1!==t.dragmode&&(o.preventDefault(),document.addEventListener("mousemove",v),document.addEventListener("touchmove",v))}function v(r){r.preventDefault();var i=u(r),o=t.minDrag||s.MINDRAG,a=y(i[0]-e,i[1]-n,o),l=a[0],c=a[1];(l||c)&&(p._dragged=!0,A.unhover(p)),p._dragged&&t.moveFn&&!f&&(p._dragdata={element:m,dx:l,dy:c},t.moveFn(l,c))}function B(e){if(delete p._dragdata,!1!==t.dragmode&&(e.preventDefault(),document.removeEventListener("mousemove",v),document.removeEventListener("touchmove",v)),document.removeEventListener("mouseup",B),document.removeEventListener("touchend",B),i?a(g):l&&(g.documentElement.style.cursor=l,l=null),p._dragging){if(p._dragging=!1,(new Date).getTime()-p._mouseDownTime>C&&(I=Math.max(I-1,1)),p._dragged)t.doneFn&&t.doneFn();else if(t.clickFn&&t.clickFn(I,h),!f){var n;try{n=new MouseEvent("click",e)}catch(t){var r=u(e);(n=document.createEvent("MouseEvents")).initMouseEvent("click",e.bubbles,e.cancelable,e.view,e.detail,e.screenX,e.screenY,r[0],r[1],e.ctrlKey,e.altKey,e.shiftKey,e.metaKey,e.button,e.relatedTarget)}d.dispatchEvent(n)}p._dragging=!1,p._dragged=!1}else p._dragged=!1}},A.coverSlip=c},{"../../lib":719,"../../plots/cartesian/constants":773,"./align":609,"./cursor":610,"./unhover":612,"has-hover":410,"has-passive-events":411,"mouse-event-offset":436}],612:[function(t,e,n){"use strict";var r=t("../../lib/events"),i=t("../../lib/throttle"),o=t("../../lib/dom").getGraphDiv,a=t("../fx/constants"),s=e.exports={};s.wrapped=function(t,e,n){(t=o(t))._fullLayout&&i.clear(t._fullLayout._uid+a.HOVERID),s.raw(t,e,n)},s.raw=function(t,e){var n=t._fullLayout,i=t._hoverdata;e||(e={}),e.target&&!1===r.triggerHandler(t,"plotly_beforehover",e)||(n._hoverlayer.selectAll("g").remove(),n._hoverlayer.selectAll("line").remove(),n._hoverlayer.selectAll("circle").remove(),t._hoverdata=void 0,e.target&&i&&t.emit("plotly_unhover",{event:e,points:i}))}},{"../../lib/dom":708,"../../lib/events":709,"../../lib/throttle":744,"../fx/constants":626}],613:[function(t,e,n){"use strict";n.dash={valType:"string",values:["solid","dot","dash","longdash","dashdot","longdashdot"],dflt:"solid",editType:"style"}},{}],614:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=t("tinycolor2"),a=t("../../registry"),s=t("../color"),A=t("../colorscale"),l=t("../../lib"),c=t("../../lib/svg_text_utils"),u=t("../../constants/xmlns_namespaces"),g=t("../../constants/alignment").LINE_SPACING,h=t("../../constants/interactions").DESELECTDIM,d=t("../../traces/scatter/subtypes"),f=t("../../traces/scatter/make_bubble_size_func"),p=e.exports={};p.font=function(t,e,n,r){l.isPlainObject(e)&&(r=e.color,n=e.size,e=e.family),e&&t.style("font-family",e),n+1&&t.style("font-size",n+"px"),r&&t.call(s.fill,r)},p.setPosition=function(t,e,n){t.attr("x",e).attr("y",n)},p.setSize=function(t,e,n){t.attr("width",e).attr("height",n)},p.setRect=function(t,e,n,r,i){t.call(p.setPosition,e,n).call(p.setSize,r,i)},p.translatePoint=function(t,e,n,r){var o=n.c2p(t.x),a=r.c2p(t.y);return!!(i(o)&&i(a)&&e.node())&&("text"===e.node().nodeName?e.attr("x",o).attr("y",a):e.attr("transform","translate("+o+","+a+")"),!0)},p.translatePoints=function(t,e,n){t.each(function(t){var i=r.select(this);p.translatePoint(t,i,e,n)})},p.hideOutsideRangePoint=function(t,e,n,r,i,o){e.attr("display",n.isPtWithinRange(t,i)&&r.isPtWithinRange(t,o)?null:"none")},p.hideOutsideRangePoints=function(t,e){if(e._hasClipOnAxisFalse){var n=e.xaxis,i=e.yaxis;t.each(function(e){var o=e[0].trace,s=o.xcalendar,A=o.ycalendar,l=a.traceIs(o,"bar-like")?".bartext":".point,.textpoint";t.selectAll(l).each(function(t){p.hideOutsideRangePoint(t,r.select(this),n,i,s,A)})})}},p.crispRound=function(t,e,n){return e&&i(e)?t._context.staticPlot?e:e<1?1:Math.round(e):n||0},p.singleLineStyle=function(t,e,n,r,i){e.style("fill","none");var o=(((t||[])[0]||{}).trace||{}).line||{},a=n||o.width||0,A=i||o.dash||"";s.stroke(e,r||o.color),p.dashLine(e,A,a)},p.lineGroupStyle=function(t,e,n,i){t.style("fill","none").each(function(t){var o=(((t||[])[0]||{}).trace||{}).line||{},a=e||o.width||0,A=i||o.dash||"";r.select(this).call(s.stroke,n||o.color).call(p.dashLine,A,a)})},p.dashLine=function(t,e,n){n=+n||0,e=p.dashStyle(e,n),t.style({"stroke-dasharray":e,"stroke-width":n+"px"})},p.dashStyle=function(t,e){e=+e||1;var n=Math.max(e,3);return"solid"===t?t="":"dot"===t?t=n+"px,"+n+"px":"dash"===t?t=3*n+"px,"+3*n+"px":"longdash"===t?t=5*n+"px,"+5*n+"px":"dashdot"===t?t=3*n+"px,"+n+"px,"+n+"px,"+n+"px":"longdashdot"===t&&(t=5*n+"px,"+2*n+"px,"+n+"px,"+2*n+"px"),t},p.singleFillStyle=function(t){var e=(((r.select(t.node()).data()[0]||[])[0]||{}).trace||{}).fillcolor;e&&t.call(s.fill,e)},p.fillGroupStyle=function(t){t.style("stroke-width",0).each(function(t){var e=r.select(this);t[0].trace&&e.call(s.fill,t[0].trace.fillcolor)})};var I=t("./symbol_defs");p.symbolNames=[],p.symbolFuncs=[],p.symbolNeedLines={},p.symbolNoDot={},p.symbolNoFill={},p.symbolList=[],Object.keys(I).forEach(function(t){var e=I[t];p.symbolList=p.symbolList.concat([e.n,t,e.n+100,t+"-open"]),p.symbolNames[e.n]=t,p.symbolFuncs[e.n]=e.f,e.needLine&&(p.symbolNeedLines[e.n]=!0),e.noDot?p.symbolNoDot[e.n]=!0:p.symbolList=p.symbolList.concat([e.n+200,t+"-dot",e.n+300,t+"-open-dot"]),e.noFill&&(p.symbolNoFill[e.n]=!0)});var C=p.symbolNames.length,m="M0,0.5L0.5,0L0,-0.5L-0.5,0Z";function y(t,e){var n=t%100;return p.symbolFuncs[n](e)+(t>=200?m:"")}p.symbolNumber=function(t){if("string"==typeof t){var e=0;t.indexOf("-open")>0&&(e=100,t=t.replace("-open","")),t.indexOf("-dot")>0&&(e+=200,t=t.replace("-dot","")),(t=p.symbolNames.indexOf(t))>=0&&(t+=e)}return t%100>=C||t>=400?0:Math.floor(Math.max(t,0))};var E={x1:1,x2:0,y1:0,y2:0},v={x1:0,x2:0,y1:1,y2:0},B=r.format("~.1f"),b={radial:{node:"radialGradient"},radialreversed:{node:"radialGradient",reversed:!0},horizontal:{node:"linearGradient",attrs:E},horizontalreversed:{node:"linearGradient",attrs:E,reversed:!0},vertical:{node:"linearGradient",attrs:v},verticalreversed:{node:"linearGradient",attrs:v,reversed:!0}};p.gradient=function(t,e,n,i,a,A){for(var c=a.length,u=b[i],g=new Array(c),h=0;h<c;h++)u.reversed?g[c-1-h]=[B(100*(1-a[h][0])),a[h][1]]:g[h]=[B(100*a[h][0]),a[h][1]];var d="g"+e._fullLayout._uid+"-"+n,f=e._fullLayout._defs.select(".gradients").selectAll("#"+d).data([i+g.join(";")],l.identity);f.exit().remove(),f.enter().append(u.node).each(function(){var t=r.select(this);u.attrs&&t.attr(u.attrs),t.attr("id",d);var e=t.selectAll("stop").data(g);e.exit().remove(),e.enter().append("stop"),e.each(function(t){var e=o(t[1]);r.select(this).attr({offset:t[0]+"%","stop-color":s.tinyRGB(e),"stop-opacity":e.getAlpha()})})}),t.style(A,R(d,e)).style(A+"-opacity",null)},p.initGradients=function(t){l.ensureSingle(t._fullLayout._defs,"g","gradients").selectAll("linearGradient,radialGradient").remove()},p.pointStyle=function(t,e,n){if(t.size()){var i=p.makePointStyleFns(e);t.each(function(t){p.singlePointStyle(t,r.select(this),e,i,n)})}},p.singlePointStyle=function(t,e,n,r,i){var o=n.marker,a=o.line;if(e.style("opacity",r.selectedOpacityFn?r.selectedOpacityFn(t):void 0===t.mo?o.opacity:t.mo),r.ms2mrc){var A;A="various"===t.ms||"various"===o.size?3:r.ms2mrc(t.ms),t.mrc=A,r.selectedSizeFn&&(A=t.mrc=r.selectedSizeFn(t));var c=p.symbolNumber(t.mx||o.symbol)||0;t.om=c%200>=100,e.attr("d",y(c,A))}var u,g,h,d=!1;if(t.so)h=a.outlierwidth,g=a.outliercolor,u=o.outliercolor;else{var f=(a||{}).width;h=(t.mlw+1||f+1||(t.trace?(t.trace.marker.line||{}).width:0)+1)-1||0,g="mlc"in t?t.mlcc=r.lineScale(t.mlc):l.isArrayOrTypedArray(a.color)?s.defaultLine:a.color,l.isArrayOrTypedArray(o.color)&&(u=s.defaultLine,d=!0),u="mc"in t?t.mcc=r.markerScale(t.mc):o.color||"rgba(0,0,0,0)",r.selectedColorFn&&(u=r.selectedColorFn(t))}if(t.om)e.call(s.stroke,u).style({"stroke-width":(h||1)+"px",fill:"none"});else{e.style("stroke-width",h+"px");var I=o.gradient,C=t.mgt;if(C?d=!0:C=I&&I.type,Array.isArray(C)&&(C=C[0],b[C]||(C=0)),C&&"none"!==C){var m=t.mgc;m?d=!0:m=I.color;var E=n.uid;d&&(E+="-"+t.i),p.gradient(e,i,E,C,[[0,m],[1,u]],"fill")}else s.fill(e,u);h&&s.stroke(e,g)}},p.makePointStyleFns=function(t){var e={},n=t.marker;return e.markerScale=p.tryColorscale(n,""),e.lineScale=p.tryColorscale(n,"line"),a.traceIs(t,"symbols")&&(e.ms2mrc=d.isBubble(t)?f(t):function(){return(n.size||6)/2}),t.selectedpoints&&l.extendFlat(e,p.makeSelectedPointStyleFns(t)),e},p.makeSelectedPointStyleFns=function(t){var e={},n=t.selected||{},r=t.unselected||{},i=t.marker||{},o=n.marker||{},s=r.marker||{},A=i.opacity,c=o.opacity,u=s.opacity,g=void 0!==c,d=void 0!==u;(l.isArrayOrTypedArray(A)||g||d)&&(e.selectedOpacityFn=function(t){var e=void 0===t.mo?i.opacity:t.mo;return t.selected?g?c:e:d?u:h*e});var f=i.color,p=o.color,I=s.color;(p||I)&&(e.selectedColorFn=function(t){var e=t.mcc||f;return t.selected?p||e:I||e});var C=i.size,m=o.size,y=s.size,E=void 0!==m,v=void 0!==y;return a.traceIs(t,"symbols")&&(E||v)&&(e.selectedSizeFn=function(t){var e=t.mrc||C/2;return t.selected?E?m/2:e:v?y/2:e}),e},p.makeSelectedTextStyleFns=function(t){var e={},n=t.selected||{},r=t.unselected||{},i=t.textfont||{},o=n.textfont||{},a=r.textfont||{},A=i.color,l=o.color,c=a.color;return e.selectedTextColorFn=function(t){var e=t.tc||A;return t.selected?l||e:c||(l?e:s.addOpacity(e,h))},e},p.selectedPointStyle=function(t,e){if(t.size()&&e.selectedpoints){var n=p.makeSelectedPointStyleFns(e),i=e.marker||{},o=[];n.selectedOpacityFn&&o.push(function(t,e){t.style("opacity",n.selectedOpacityFn(e))}),n.selectedColorFn&&o.push(function(t,e){s.fill(t,n.selectedColorFn(e))}),n.selectedSizeFn&&o.push(function(t,e){var r=e.mx||i.symbol||0,o=n.selectedSizeFn(e);t.attr("d",y(p.symbolNumber(r),o)),e.mrc2=o}),o.length&&t.each(function(t){for(var e=r.select(this),n=0;n<o.length;n++)o[n](e,t)})}},p.tryColorscale=function(t,e){var n=e?l.nestedProperty(t,e).get():t;if(n){var r=n.color;if((n.colorscale||n._colorAx)&&l.isArrayOrTypedArray(r))return A.makeColorScaleFuncFromTrace(n)}return l.identity};var Q={start:1,end:-1,middle:0,bottom:1,top:-1};function w(t,e,n,i){var o=r.select(t.node().parentNode),a=-1!==e.indexOf("top")?"top":-1!==e.indexOf("bottom")?"bottom":"middle",s=-1!==e.indexOf("left")?"end":-1!==e.indexOf("right")?"start":"middle",A=i?i/.8+1:0,l=(c.lineCount(t)-1)*g+1,u=Q[s]*A,h=.75*n+Q[a]*A+(Q[a]-1)*l*n/2;t.attr("text-anchor",s),o.attr("transform","translate("+u+","+h+")")}function x(t,e){var n=t.ts||e.textfont.size;return i(n)&&n>0?n:0}p.textPointStyle=function(t,e,n){if(t.size()){var i;if(e.selectedpoints){var o=p.makeSelectedTextStyleFns(e);i=o.selectedTextColorFn}t.each(function(t){var o=r.select(this),a=l.extractOption(t,e,"tx","text");if(a||0===a){var s=t.tp||e.textposition,A=x(t,e),u=i?i(t):t.tc||e.textfont.color;o.call(p.font,t.tf||e.textfont.family,A,u).text(a).call(c.convertToTspans,n).call(w,s,A,t.mrc)}else o.remove()})}},p.selectedTextStyle=function(t,e){if(t.size()&&e.selectedpoints){var n=p.makeSelectedTextStyleFns(e);t.each(function(t){var i=r.select(this),o=n.selectedTextColorFn(t),a=t.tp||e.textposition,A=x(t,e);s.fill(i,o),w(i,a,A,t.mrc2||t.mrc)})}};var _=.5;function D(t,e,n,i){var o=t[0]-e[0],a=t[1]-e[1],s=n[0]-e[0],A=n[1]-e[1],l=Math.pow(o*o+a*a,_/2),c=Math.pow(s*s+A*A,_/2),u=(c*c*o-l*l*s)*i,g=(c*c*a-l*l*A)*i,h=3*c*(l+c),d=3*l*(l+c);return[[r.round(e[0]+(h&&u/h),2),r.round(e[1]+(h&&g/h),2)],[r.round(e[0]-(d&&u/d),2),r.round(e[1]-(d&&g/d),2)]]}p.smoothopen=function(t,e){if(t.length<3)return"M"+t.join("L");var n,r="M"+t[0],i=[];for(n=1;n<t.length-1;n++)i.push(D(t[n-1],t[n],t[n+1],e));for(r+="Q"+i[0][0]+" "+t[1],n=2;n<t.length-1;n++)r+="C"+i[n-2][1]+" "+i[n-1][0]+" "+t[n];return r+="Q"+i[t.length-3][1]+" "+t[t.length-1]},p.smoothclosed=function(t,e){if(t.length<3)return"M"+t.join("L")+"Z";var n,r="M"+t[0],i=t.length-1,o=[D(t[i],t[0],t[1],e)];for(n=1;n<i;n++)o.push(D(t[n-1],t[n],t[n+1],e));for(o.push(D(t[i-1],t[i],t[0],e)),n=1;n<=i;n++)r+="C"+o[n-1][1]+" "+o[n][0]+" "+t[n];return r+="C"+o[i][1]+" "+o[0][0]+" "+t[0]+"Z"};var M={hv:function(t,e){return"H"+r.round(e[0],2)+"V"+r.round(e[1],2)},vh:function(t,e){return"V"+r.round(e[1],2)+"H"+r.round(e[0],2)},hvh:function(t,e){return"H"+r.round((t[0]+e[0])/2,2)+"V"+r.round(e[1],2)+"H"+r.round(e[0],2)},vhv:function(t,e){return"V"+r.round((t[1]+e[1])/2,2)+"H"+r.round(e[0],2)+"V"+r.round(e[1],2)}},k=function(t,e){return"L"+r.round(e[0],2)+","+r.round(e[1],2)};p.steps=function(t){var e=M[t]||k;return function(t){for(var n="M"+r.round(t[0][0],2)+","+r.round(t[0][1],2),i=1;i<t.length;i++)n+=e(t[i-1],t[i]);return n}},p.makeTester=function(){var t=l.ensureSingleById(r.select("body"),"svg","js-plotly-tester",function(t){t.attr(u.svgAttrs).style({position:"absolute",left:"-10000px",top:"-10000px",width:"9000px",height:"9000px","z-index":"1"})}),e=l.ensureSingle(t,"path","js-reference-point",function(t){t.attr("d","M0,0H1V1H0Z").style({"stroke-width":0,fill:"black"})});p.tester=t,p.testref=e},p.savedBBoxes={};var S=0;function L(t){var e=t.getAttribute("data-unformatted");if(null!==e)return e+t.getAttribute("data-math")+t.getAttribute("text-anchor")+t.getAttribute("style")}function R(t,e){if(!t)return null;var n=e._context;return"url('"+(n._exportedPlot?"":n._baseUrl||"")+"#"+t+"')"}p.bBox=function(t,e,n){var i,o,a;if(n||(n=L(t)),n){if(i=p.savedBBoxes[n])return l.extendFlat({},i)}else if(1===t.childNodes.length){var s=t.childNodes[0];if(n=L(s)){var A=+s.getAttribute("x")||0,u=+s.getAttribute("y")||0,g=s.getAttribute("transform");if(!g){var h=p.bBox(s,!1,n);return A&&(h.left+=A,h.right+=A),u&&(h.top+=u,h.bottom+=u),h}if(n+="~"+A+"~"+u+"~"+g,i=p.savedBBoxes[n])return l.extendFlat({},i)}}e?o=t:(a=p.tester.node(),o=t.cloneNode(!0),a.appendChild(o)),r.select(o).attr("transform",null).call(c.positionText,0,0);var d=o.getBoundingClientRect(),f=p.testref.node().getBoundingClientRect();e||a.removeChild(o);var I={height:d.height,width:d.width,left:d.left-f.left,top:d.top-f.top,right:d.right-f.left,bottom:d.bottom-f.top};return S>=1e4&&(p.savedBBoxes={},S=0),n&&(p.savedBBoxes[n]=I),S++,l.extendFlat({},I)},p.setClipUrl=function(t,e,n){t.attr("clip-path",R(e,n))},p.getTranslate=function(t){var e=(t[t.attr?"attr":"getAttribute"]("transform")||"").replace(/.*\btranslate\((-?\d*\.?\d*)[^-\d]*(-?\d*\.?\d*)[^\d].*/,function(t,e,n){return[e,n].join(" ")}).split(" ");return{x:+e[0]||0,y:+e[1]||0}},p.setTranslate=function(t,e,n){var r=t.attr?"attr":"getAttribute",i=t.attr?"attr":"setAttribute",o=t[r]("transform")||"";return e=e||0,n=n||0,o=o.replace(/(\btranslate\(.*?\);?)/,"").trim(),o=(o+=" translate("+e+", "+n+")").trim(),t[i]("transform",o),o},p.getScale=function(t){var e=(t[t.attr?"attr":"getAttribute"]("transform")||"").replace(/.*\bscale\((\d*\.?\d*)[^\d]*(\d*\.?\d*)[^\d].*/,function(t,e,n){return[e,n].join(" ")}).split(" ");return{x:+e[0]||1,y:+e[1]||1}},p.setScale=function(t,e,n){var r=t.attr?"attr":"getAttribute",i=t.attr?"attr":"setAttribute",o=t[r]("transform")||"";return e=e||1,n=n||1,o=o.replace(/(\bscale\(.*?\);?)/,"").trim(),o=(o+=" scale("+e+", "+n+")").trim(),t[i]("transform",o),o};var N=/\s*sc.*/;p.setPointGroupScale=function(t,e,n){if(e=e||1,n=n||1,t){var r=1===e&&1===n?"":" scale("+e+","+n+")";t.each(function(){var t=(this.getAttribute("transform")||"").replace(N,"");t=(t+=r).trim(),this.setAttribute("transform",t)})}};var F=/translate\([^)]*\)\s*$/;p.setTextPointsScale=function(t,e,n){t&&t.each(function(){var t,i=r.select(this),o=i.select("text");if(o.node()){var a=parseFloat(o.attr("x")||0),s=parseFloat(o.attr("y")||0),A=(i.attr("transform")||"").match(F);t=1===e&&1===n?[]:["translate("+a+","+s+")","scale("+e+","+n+")","translate("+-a+","+-s+")"],A&&t.push(A),i.attr("transform",t.join(" "))}})}},{"../../constants/alignment":688,"../../constants/interactions":694,"../../constants/xmlns_namespaces":696,"../../lib":719,"../../lib/svg_text_utils":743,"../../registry":847,"../../traces/scatter/make_bubble_size_func":1128,"../../traces/scatter/subtypes":1135,"../color":593,"../colorscale":605,"./symbol_defs":615,d3:163,"fast-isnumeric":225,tinycolor2:537}],615:[function(t,e,n){"use strict";var r=t("d3");e.exports={circle:{n:0,f:function(t){var e=r.round(t,2);return"M"+e+",0A"+e+","+e+" 0 1,1 0,-"+e+"A"+e+","+e+" 0 0,1 "+e+",0Z"}},square:{n:1,f:function(t){var e=r.round(t,2);return"M"+e+","+e+"H-"+e+"V-"+e+"H"+e+"Z"}},diamond:{n:2,f:function(t){var e=r.round(1.3*t,2);return"M"+e+",0L0,"+e+"L-"+e+",0L0,-"+e+"Z"}},cross:{n:3,f:function(t){var e=r.round(.4*t,2),n=r.round(1.2*t,2);return"M"+n+","+e+"H"+e+"V"+n+"H-"+e+"V"+e+"H-"+n+"V-"+e+"H-"+e+"V-"+n+"H"+e+"V-"+e+"H"+n+"Z"}},x:{n:4,f:function(t){var e=r.round(.8*t/Math.sqrt(2),2),n="l"+e+","+e,i="l"+e+",-"+e,o="l-"+e+",-"+e,a="l-"+e+","+e;return"M0,"+e+n+i+o+i+o+a+o+a+n+a+n+"Z"}},"triangle-up":{n:5,f:function(t){var e=r.round(2*t/Math.sqrt(3),2);return"M-"+e+","+r.round(t/2,2)+"H"+e+"L0,-"+r.round(t,2)+"Z"}},"triangle-down":{n:6,f:function(t){var e=r.round(2*t/Math.sqrt(3),2);return"M-"+e+",-"+r.round(t/2,2)+"H"+e+"L0,"+r.round(t,2)+"Z"}},"triangle-left":{n:7,f:function(t){var e=r.round(2*t/Math.sqrt(3),2);return"M"+r.round(t/2,2)+",-"+e+"V"+e+"L-"+r.round(t,2)+",0Z"}},"triangle-right":{n:8,f:function(t){var e=r.round(2*t/Math.sqrt(3),2);return"M-"+r.round(t/2,2)+",-"+e+"V"+e+"L"+r.round(t,2)+",0Z"}},"triangle-ne":{n:9,f:function(t){var e=r.round(.6*t,2),n=r.round(1.2*t,2);return"M-"+n+",-"+e+"H"+e+"V"+n+"Z"}},"triangle-se":{n:10,f:function(t){var e=r.round(.6*t,2),n=r.round(1.2*t,2);return"M"+e+",-"+n+"V"+e+"H-"+n+"Z"}},"triangle-sw":{n:11,f:function(t){var e=r.round(.6*t,2),n=r.round(1.2*t,2);return"M"+n+","+e+"H-"+e+"V-"+n+"Z"}},"triangle-nw":{n:12,f:function(t){var e=r.round(.6*t,2),n=r.round(1.2*t,2);return"M-"+e+","+n+"V-"+e+"H"+n+"Z"}},pentagon:{n:13,f:function(t){var e=r.round(.951*t,2),n=r.round(.588*t,2),i=r.round(-t,2),o=r.round(-.309*t,2);return"M"+e+","+o+"L"+n+","+r.round(.809*t,2)+"H-"+n+"L-"+e+","+o+"L0,"+i+"Z"}},hexagon:{n:14,f:function(t){var e=r.round(t,2),n=r.round(t/2,2),i=r.round(t*Math.sqrt(3)/2,2);return"M"+i+",-"+n+"V"+n+"L0,"+e+"L-"+i+","+n+"V-"+n+"L0,-"+e+"Z"}},hexagon2:{n:15,f:function(t){var e=r.round(t,2),n=r.round(t/2,2),i=r.round(t*Math.sqrt(3)/2,2);return"M-"+n+","+i+"H"+n+"L"+e+",0L"+n+",-"+i+"H-"+n+"L-"+e+",0Z"}},octagon:{n:16,f:function(t){var e=r.round(.924*t,2),n=r.round(.383*t,2);return"M-"+n+",-"+e+"H"+n+"L"+e+",-"+n+"V"+n+"L"+n+","+e+"H-"+n+"L-"+e+","+n+"V-"+n+"Z"}},star:{n:17,f:function(t){var e=1.4*t,n=r.round(.225*e,2),i=r.round(.951*e,2),o=r.round(.363*e,2),a=r.round(.588*e,2),s=r.round(-e,2),A=r.round(-.309*e,2),l=r.round(.118*e,2),c=r.round(.809*e,2);return"M"+n+","+A+"H"+i+"L"+o+","+l+"L"+a+","+c+"L0,"+r.round(.382*e,2)+"L-"+a+","+c+"L-"+o+","+l+"L-"+i+","+A+"H-"+n+"L0,"+s+"Z"}},hexagram:{n:18,f:function(t){var e=r.round(.66*t,2),n=r.round(.38*t,2),i=r.round(.76*t,2);return"M-"+i+",0l-"+n+",-"+e+"h"+i+"l"+n+",-"+e+"l"+n+","+e+"h"+i+"l-"+n+","+e+"l"+n+","+e+"h-"+i+"l-"+n+","+e+"l-"+n+",-"+e+"h-"+i+"Z"}},"star-triangle-up":{n:19,f:function(t){var e=r.round(t*Math.sqrt(3)*.8,2),n=r.round(.8*t,2),i=r.round(1.6*t,2),o=r.round(4*t,2),a="A "+o+","+o+" 0 0 1 ";return"M-"+e+","+n+a+e+","+n+a+"0,-"+i+a+"-"+e+","+n+"Z"}},"star-triangle-down":{n:20,f:function(t){var e=r.round(t*Math.sqrt(3)*.8,2),n=r.round(.8*t,2),i=r.round(1.6*t,2),o=r.round(4*t,2),a="A "+o+","+o+" 0 0 1 ";return"M"+e+",-"+n+a+"-"+e+",-"+n+a+"0,"+i+a+e+",-"+n+"Z"}},"star-square":{n:21,f:function(t){var e=r.round(1.1*t,2),n=r.round(2*t,2),i="A "+n+","+n+" 0 0 1 ";return"M-"+e+",-"+e+i+"-"+e+","+e+i+e+","+e+i+e+",-"+e+i+"-"+e+",-"+e+"Z"}},"star-diamond":{n:22,f:function(t){var e=r.round(1.4*t,2),n=r.round(1.9*t,2),i="A "+n+","+n+" 0 0 1 ";return"M-"+e+",0"+i+"0,"+e+i+e+",0"+i+"0,-"+e+i+"-"+e+",0Z"}},"diamond-tall":{n:23,f:function(t){var e=r.round(.7*t,2),n=r.round(1.4*t,2);return"M0,"+n+"L"+e+",0L0,-"+n+"L-"+e+",0Z"}},"diamond-wide":{n:24,f:function(t){var e=r.round(1.4*t,2),n=r.round(.7*t,2);return"M0,"+n+"L"+e+",0L0,-"+n+"L-"+e+",0Z"}},hourglass:{n:25,f:function(t){var e=r.round(t,2);return"M"+e+","+e+"H-"+e+"L"+e+",-"+e+"H-"+e+"Z"},noDot:!0},bowtie:{n:26,f:function(t){var e=r.round(t,2);return"M"+e+","+e+"V-"+e+"L-"+e+","+e+"V-"+e+"Z"},noDot:!0},"circle-cross":{n:27,f:function(t){var e=r.round(t,2);return"M0,"+e+"V-"+e+"M"+e+",0H-"+e+"M"+e+",0A"+e+","+e+" 0 1,1 0,-"+e+"A"+e+","+e+" 0 0,1 "+e+",0Z"},needLine:!0,noDot:!0},"circle-x":{n:28,f:function(t){var e=r.round(t,2),n=r.round(t/Math.sqrt(2),2);return"M"+n+","+n+"L-"+n+",-"+n+"M"+n+",-"+n+"L-"+n+","+n+"M"+e+",0A"+e+","+e+" 0 1,1 0,-"+e+"A"+e+","+e+" 0 0,1 "+e+",0Z"},needLine:!0,noDot:!0},"square-cross":{n:29,f:function(t){var e=r.round(t,2);return"M0,"+e+"V-"+e+"M"+e+",0H-"+e+"M"+e+","+e+"H-"+e+"V-"+e+"H"+e+"Z"},needLine:!0,noDot:!0},"square-x":{n:30,f:function(t){var e=r.round(t,2);return"M"+e+","+e+"L-"+e+",-"+e+"M"+e+",-"+e+"L-"+e+","+e+"M"+e+","+e+"H-"+e+"V-"+e+"H"+e+"Z"},needLine:!0,noDot:!0},"diamond-cross":{n:31,f:function(t){var e=r.round(1.3*t,2);return"M"+e+",0L0,"+e+"L-"+e+",0L0,-"+e+"ZM0,-"+e+"V"+e+"M-"+e+",0H"+e},needLine:!0,noDot:!0},"diamond-x":{n:32,f:function(t){var e=r.round(1.3*t,2),n=r.round(.65*t,2);return"M"+e+",0L0,"+e+"L-"+e+",0L0,-"+e+"ZM-"+n+",-"+n+"L"+n+","+n+"M-"+n+","+n+"L"+n+",-"+n},needLine:!0,noDot:!0},"cross-thin":{n:33,f:function(t){var e=r.round(1.4*t,2);return"M0,"+e+"V-"+e+"M"+e+",0H-"+e},needLine:!0,noDot:!0,noFill:!0},"x-thin":{n:34,f:function(t){var e=r.round(t,2);return"M"+e+","+e+"L-"+e+",-"+e+"M"+e+",-"+e+"L-"+e+","+e},needLine:!0,noDot:!0,noFill:!0},asterisk:{n:35,f:function(t){var e=r.round(1.2*t,2),n=r.round(.85*t,2);return"M0,"+e+"V-"+e+"M"+e+",0H-"+e+"M"+n+","+n+"L-"+n+",-"+n+"M"+n+",-"+n+"L-"+n+","+n},needLine:!0,noDot:!0,noFill:!0},hash:{n:36,f:function(t){var e=r.round(t/2,2),n=r.round(t,2);return"M"+e+","+n+"V-"+n+"m-"+n+",0V"+n+"M"+n+","+e+"H-"+n+"m0,-"+n+"H"+n},needLine:!0,noFill:!0},"y-up":{n:37,f:function(t){var e=r.round(1.2*t,2),n=r.round(1.6*t,2),i=r.round(.8*t,2);return"M-"+e+","+i+"L0,0M"+e+","+i+"L0,0M0,-"+n+"L0,0"},needLine:!0,noDot:!0,noFill:!0},"y-down":{n:38,f:function(t){var e=r.round(1.2*t,2),n=r.round(1.6*t,2),i=r.round(.8*t,2);return"M-"+e+",-"+i+"L0,0M"+e+",-"+i+"L0,0M0,"+n+"L0,0"},needLine:!0,noDot:!0,noFill:!0},"y-left":{n:39,f:function(t){var e=r.round(1.2*t,2),n=r.round(1.6*t,2),i=r.round(.8*t,2);return"M"+i+","+e+"L0,0M"+i+",-"+e+"L0,0M-"+n+",0L0,0"},needLine:!0,noDot:!0,noFill:!0},"y-right":{n:40,f:function(t){var e=r.round(1.2*t,2),n=r.round(1.6*t,2),i=r.round(.8*t,2);return"M-"+i+","+e+"L0,0M-"+i+",-"+e+"L0,0M"+n+",0L0,0"},needLine:!0,noDot:!0,noFill:!0},"line-ew":{n:41,f:function(t){var e=r.round(1.4*t,2);return"M"+e+",0H-"+e},needLine:!0,noDot:!0,noFill:!0},"line-ns":{n:42,f:function(t){var e=r.round(1.4*t,2);return"M0,"+e+"V-"+e},needLine:!0,noDot:!0,noFill:!0},"line-ne":{n:43,f:function(t){var e=r.round(t,2);return"M"+e+",-"+e+"L-"+e+","+e},needLine:!0,noDot:!0,noFill:!0},"line-nw":{n:44,f:function(t){var e=r.round(t,2);return"M"+e+","+e+"L-"+e+",-"+e},needLine:!0,noDot:!0,noFill:!0}}},{d3:163}],616:[function(t,e,n){"use strict";e.exports={visible:{valType:"boolean",editType:"calc"},type:{valType:"enumerated",values:["percent","constant","sqrt","data"],editType:"calc"},symmetric:{valType:"boolean",editType:"calc"},array:{valType:"data_array",editType:"calc"},arrayminus:{valType:"data_array",editType:"calc"},value:{valType:"number",min:0,dflt:10,editType:"calc"},valueminus:{valType:"number",min:0,dflt:10,editType:"calc"},traceref:{valType:"integer",min:0,dflt:0,editType:"style"},tracerefminus:{valType:"integer",min:0,dflt:0,editType:"style"},copy_ystyle:{valType:"boolean",editType:"plot"},copy_zstyle:{valType:"boolean",editType:"style"},color:{valType:"color",editType:"style"},thickness:{valType:"number",min:0,dflt:2,editType:"style"},width:{valType:"number",min:0,editType:"plot"},editType:"calc",_deprecated:{opacity:{valType:"number",editType:"style"}}}},{}],617:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../registry"),o=t("../../plots/cartesian/axes"),a=t("../../lib"),s=t("./compute_error");function A(t,e,n,i){var A=e["error_"+i]||{},l=[];if(A.visible&&-1!==["linear","log"].indexOf(n.type)){for(var c=s(A),u=0;u<t.length;u++){var g=t[u],h=g.i;if(void 0===h)h=u;else if(null===h)continue;var d=g[i];if(r(n.c2l(d))){var f=c(d,h);if(r(f[0])&&r(f[1])){var p=g[i+"s"]=d-f[0],I=g[i+"h"]=d+f[1];l.push(p,I)}}}var C=n._id,m=e._extremes[C],y=o.findExtremes(n,l,a.extendFlat({tozero:m.opts.tozero},{padded:!0}));m.min=m.min.concat(y.min),m.max=m.max.concat(y.max)}}e.exports=function(t){for(var e=t.calcdata,n=0;n<e.length;n++){var r=e[n],a=r[0].trace;if(!0===a.visible&&i.traceIs(a,"errorBarsOK")){var s=o.getFromId(t,a.xaxis),l=o.getFromId(t,a.yaxis);A(r,a,s,"x"),A(r,a,l,"y")}}}},{"../../lib":719,"../../plots/cartesian/axes":767,"../../registry":847,"./compute_error":618,"fast-isnumeric":225}],618:[function(t,e,n){"use strict";function r(t,e){return"percent"===t?function(t){return Math.abs(t*e/100)}:"constant"===t?function(){return Math.abs(e)}:"sqrt"===t?function(t){return Math.sqrt(Math.abs(t))}:void 0}e.exports=function(t){var e=t.type,n=t.symmetric;if("data"===e){var i=t.array||[];if(n)return function(t,e){var n=+i[e];return[n,n]};var o=t.arrayminus||[];return function(t,e){var n=+i[e],r=+o[e];return isNaN(n)&&isNaN(r)?[NaN,NaN]:[r||0,n||0]}}var a=r(e,t.value),s=r(e,t.valueminus);return n||void 0===t.valueminus?function(t){var e=a(t);return[e,e]}:function(t){return[s(t),a(t)]}}},{}],619:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../registry"),o=t("../../lib"),a=t("../../plot_api/plot_template"),s=t("./attributes");e.exports=function(t,e,n,A){var l="error_"+A.axis,c=a.newContainer(e,l),u=t[l]||{};function g(t,e){return o.coerce(u,c,s,t,e)}if(!1!==g("visible",void 0!==u.array||void 0!==u.value||"sqrt"===u.type)){var h=g("type","array"in u?"data":"percent"),d=!0;"sqrt"!==h&&(d=g("symmetric",!(("data"===h?"arrayminus":"valueminus")in u))),"data"===h?(g("array"),g("traceref"),d||(g("arrayminus"),g("tracerefminus"))):"percent"!==h&&"constant"!==h||(g("value"),d||g("valueminus"));var f="copy_"+A.inherit+"style";A.inherit&&(e["error_"+A.inherit]||{}).visible&&g(f,!(u.color||r(u.thickness)||r(u.width))),A.inherit&&c[f]||(g("color",n),g("thickness"),g("width",i.traceIs(e,"gl3d")?0:4))}}},{"../../lib":719,"../../plot_api/plot_template":757,"../../registry":847,"./attributes":616,"fast-isnumeric":225}],620:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plot_api/edit_types").overrideAll,o=t("./attributes"),a={error_x:r.extendFlat({},o),error_y:r.extendFlat({},o)};delete a.error_x.copy_zstyle,delete a.error_y.copy_zstyle,delete a.error_y.copy_ystyle;var s={error_x:r.extendFlat({},o),error_y:r.extendFlat({},o),error_z:r.extendFlat({},o)};delete s.error_x.copy_ystyle,delete s.error_y.copy_ystyle,delete s.error_z.copy_ystyle,delete s.error_z.copy_zstyle,e.exports={moduleType:"component",name:"errorbars",schema:{traces:{scatter:a,bar:a,histogram:a,scatter3d:i(s,"calc","nested"),scattergl:i(a,"calc","nested")}},supplyDefaults:t("./defaults"),calc:t("./calc"),makeComputeError:t("./compute_error"),plot:t("./plot"),style:t("./style"),hoverInfo:function(t,e,n){(e.error_y||{}).visible&&(n.yerr=t.yh-t.y,e.error_y.symmetric||(n.yerrneg=t.y-t.ys)),(e.error_x||{}).visible&&(n.xerr=t.xh-t.x,e.error_x.symmetric||(n.xerrneg=t.x-t.xs))}}},{"../../lib":719,"../../plot_api/edit_types":750,"./attributes":616,"./calc":617,"./compute_error":618,"./defaults":619,"./plot":621,"./style":622}],621:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=t("../drawing"),a=t("../../traces/scatter/subtypes");e.exports=function(t,e,n,s){var A=n.xaxis,l=n.yaxis,c=s&&s.duration>0;e.each(function(e){var u,g=e[0].trace,h=g.error_x||{},d=g.error_y||{};g.ids&&(u=function(t){return t.id});var f=a.hasMarkers(g)&&g.marker.maxdisplayed>0;d.visible||h.visible||(e=[]);var p=r.select(this).selectAll("g.errorbar").data(e,u);if(p.exit().remove(),e.length){h.visible||p.selectAll("path.xerror").remove(),d.visible||p.selectAll("path.yerror").remove(),p.style("opacity",1);var I=p.enter().append("g").classed("errorbar",!0);c&&I.style("opacity",0).transition().duration(s.duration).style("opacity",1),o.setClipUrl(p,n.layerClipId,t),p.each(function(t){var e=r.select(this),n=function(t,e,n){var r={x:e.c2p(t.x),y:n.c2p(t.y)};return void 0!==t.yh&&(r.yh=n.c2p(t.yh),r.ys=n.c2p(t.ys),i(r.ys)||(r.noYS=!0,r.ys=n.c2p(t.ys,!0))),void 0!==t.xh&&(r.xh=e.c2p(t.xh),r.xs=e.c2p(t.xs),i(r.xs)||(r.noXS=!0,r.xs=e.c2p(t.xs,!0))),r}(t,A,l);if(!f||t.vis){var o,a=e.select("path.yerror");if(d.visible&&i(n.x)&&i(n.yh)&&i(n.ys)){var u=d.width;o="M"+(n.x-u)+","+n.yh+"h"+2*u+"m-"+u+",0V"+n.ys,n.noYS||(o+="m-"+u+",0h"+2*u),a.size()?c&&(a=a.transition().duration(s.duration).ease(s.easing)):a=e.append("path").style("vector-effect","non-scaling-stroke").classed("yerror",!0),a.attr("d",o)}else a.remove();var g=e.select("path.xerror");if(h.visible&&i(n.y)&&i(n.xh)&&i(n.xs)){var p=(h.copy_ystyle?d:h).width;o="M"+n.xh+","+(n.y-p)+"v"+2*p+"m0,-"+p+"H"+n.xs,n.noXS||(o+="m0,-"+p+"v"+2*p),g.size()?c&&(g=g.transition().duration(s.duration).ease(s.easing)):g=e.append("path").style("vector-effect","non-scaling-stroke").classed("xerror",!0),g.attr("d",o)}else g.remove()}})}})}},{"../../traces/scatter/subtypes":1135,"../drawing":614,d3:163,"fast-isnumeric":225}],622:[function(t,e,n){"use strict";var r=t("d3"),i=t("../color");e.exports=function(t){t.each(function(t){var e=t[0].trace,n=e.error_y||{},o=e.error_x||{},a=r.select(this);a.selectAll("path.yerror").style("stroke-width",n.thickness+"px").call(i.stroke,n.color),o.copy_ystyle&&(o=n),a.selectAll("path.xerror").style("stroke-width",o.thickness+"px").call(i.stroke,o.color)})}},{"../color":593,d3:163}],623:[function(t,e,n){"use strict";var r=t("../../plots/font_attributes"),i=t("./layout_attributes").hoverlabel,o=t("../../lib/extend").extendFlat;e.exports={hoverlabel:{bgcolor:o({},i.bgcolor,{arrayOk:!0}),bordercolor:o({},i.bordercolor,{arrayOk:!0}),font:r({arrayOk:!0,editType:"none"}),align:o({},i.align,{arrayOk:!0}),namelength:o({},i.namelength,{arrayOk:!0}),editType:"none"}}},{"../../lib/extend":710,"../../plots/font_attributes":793,"./layout_attributes":633}],624:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../registry");function o(t,e,n,i){i=i||r.identity,Array.isArray(t)&&(e[0][n]=i(t))}e.exports=function(t){var e=t.calcdata,n=t._fullLayout;function a(t){return function(e){return r.coerceHoverinfo({hoverinfo:e},{_module:t._module},n)}}for(var s=0;s<e.length;s++){var A=e[s],l=A[0].trace;if(!i.traceIs(l,"pie-like")){var c=i.traceIs(l,"2dMap")?o:r.fillArray;c(l.hoverinfo,A,"hi",a(l)),l.hovertemplate&&c(l.hovertemplate,A,"ht"),l.hoverlabel&&(c(l.hoverlabel.bgcolor,A,"hbg"),c(l.hoverlabel.bordercolor,A,"hbc"),c(l.hoverlabel.font.size,A,"hts"),c(l.hoverlabel.font.color,A,"htc"),c(l.hoverlabel.font.family,A,"htf"),c(l.hoverlabel.namelength,A,"hnl"),c(l.hoverlabel.align,A,"hta"))}}}},{"../../lib":719,"../../registry":847}],625:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("./hover").hover;e.exports=function(t,e,n){var o=r.getComponentMethod("annotations","onClick")(t,t._hoverdata);function a(){t.emit("plotly_click",{points:t._hoverdata,event:e})}void 0!==n&&i(t,e,n,!0),t._hoverdata&&e&&e.target&&(o&&o.then?o.then(a):a(),e.stopImmediatePropagation&&e.stopImmediatePropagation())}},{"../../registry":847,"./hover":629}],626:[function(t,e,n){"use strict";e.exports={YANGLE:60,HOVERARROWSIZE:6,HOVERTEXTPAD:3,HOVERFONTSIZE:13,HOVERFONT:"Arial, sans-serif",HOVERMINTIME:50,HOVERID:"-hover"}},{}],627:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./attributes"),o=t("./hoverlabel_defaults");e.exports=function(t,e,n,a){var s=r.extendFlat({},a.hoverlabel);e.hovertemplate&&(s.namelength=-1),o(t,e,function(n,o){return r.coerce(t,e,i,n,o)},s)}},{"../../lib":719,"./attributes":623,"./hoverlabel_defaults":630}],628:[function(t,e,n){"use strict";var r=t("../../lib");n.getSubplot=function(t){return t.subplot||t.xaxis+t.yaxis||t.geo},n.isTraceInSubplots=function(t,e){if("splom"===t.type){for(var r=t.xaxes||[],i=t.yaxes||[],o=0;o<r.length;o++)for(var a=0;a<i.length;a++)if(-1!==e.indexOf(r[o]+i[a]))return!0;return!1}return-1!==e.indexOf(n.getSubplot(t))},n.flat=function(t,e){for(var n=new Array(t.length),r=0;r<t.length;r++)n[r]=e;return n},n.p2c=function(t,e){for(var n=new Array(t.length),r=0;r<t.length;r++)n[r]=t[r].p2c(e);return n},n.getDistanceFunction=function(t,e,r,i){return"closest"===t?i||n.quadrature(e,r):"x"===t?e:r},n.getClosest=function(t,e,n){if(!1!==n.index)n.index>=0&&n.index<t.length?n.distance=0:n.index=!1;else for(var r=0;r<t.length;r++){var i=e(t[r]);i<=n.distance&&(n.index=r,n.distance=i)}return n},n.inbox=function(t,e,n){return t*e<0||0===t?n:1/0},n.quadrature=function(t,e){return function(n){var r=t(n),i=e(n);return Math.sqrt(r*r+i*i)}},n.makeEventData=function(t,e,r){var i="index"in t?t.index:t.pointNumber,o={data:e._input,fullData:e,curveNumber:e.index,pointNumber:i};if(e._indexToPoints){var a=e._indexToPoints[i];1===a.length?o.pointIndex=a[0]:o.pointIndices=a}else o.pointIndex=i;return e._module.eventData?o=e._module.eventData(o,t,e,r,i):("xVal"in t?o.x=t.xVal:"x"in t&&(o.x=t.x),"yVal"in t?o.y=t.yVal:"y"in t&&(o.y=t.y),t.xa&&(o.xaxis=t.xa),t.ya&&(o.yaxis=t.ya),void 0!==t.zLabelVal&&(o.z=t.zLabelVal)),n.appendArrayPointValue(o,e,i),o},n.appendArrayPointValue=function(t,e,n){var i=e._arrayAttrs;if(i)for(var s=0;s<i.length;s++){var A=i[s],l=o(A);if(void 0===t[l]){var c=a(r.nestedProperty(e,A).get(),n);void 0!==c&&(t[l]=c)}}},n.appendArrayMultiPointValues=function(t,e,n){var i=e._arrayAttrs;if(i)for(var s=0;s<i.length;s++){var A=i[s],l=o(A);if(void 0===t[l]){for(var c=r.nestedProperty(e,A).get(),u=new Array(n.length),g=0;g<n.length;g++)u[g]=a(c,n[g]);t[l]=u}}};var i={ids:"id",locations:"location",labels:"label",values:"value","marker.colors":"color",parents:"parent"};function o(t){return i[t]||t}function a(t,e){return Array.isArray(e)?Array.isArray(t)&&Array.isArray(t[e[0]])?t[e[0]][e[1]]:void 0:t[e]}},{"../../lib":719}],629:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=t("tinycolor2"),a=t("../../lib"),s=t("../../lib/events"),A=t("../../lib/svg_text_utils"),l=t("../../lib/override_cursor"),c=t("../drawing"),u=t("../color"),g=t("../dragelement"),h=t("../../plots/cartesian/axes"),d=t("../../registry"),f=t("./helpers"),p=t("./constants"),I=p.YANGLE,C=Math.PI*I/180,m=1/Math.sin(C),y=Math.cos(C),E=Math.sin(C),v=p.HOVERARROWSIZE,B=p.HOVERTEXTPAD;n.hover=function(t,e,n,o){t=a.getGraphDiv(t),a.throttle(t._fullLayout._uid+p.HOVERID,p.HOVERMINTIME,function(){!function(t,e,n,o){n||(n="xy");var A=Array.isArray(n)?n:[n],c=t._fullLayout,p=c._plots||[],I=p[n],C=c._has("cartesian");if(I){var y=I.overlays.map(function(t){return t.id});A=A.concat(y)}for(var E=A.length,v=new Array(E),B=new Array(E),b=!1,M=0;M<E;M++){var k=A[M],S=p[k];if(S)b=!0,v[M]=h.getFromId(t,S.xaxis._id),B[M]=h.getFromId(t,S.yaxis._id);else{var L=c[k]._subplot;v[M]=L.xaxis,B[M]=L.yaxis}}var R=e.hovermode||c.hovermode;if(R&&!b&&(R="closest"),-1===["x","y","closest"].indexOf(R)||!t.calcdata||t.querySelector(".zoombox")||t._dragging)return g.unhoverRaw(t,e);var N,F,T,U,O,G,H,j,Y,P,K,z,J,q=-1===c.hoverdistance?1/0:c.hoverdistance,V=-1===c.spikedistance?1/0:c.spikedistance,W=[],X=[],Z={hLinePoint:null,vLinePoint:null},$=!1;if(Array.isArray(e))for(R="array",T=0;T<e.length;T++)(O=t.calcdata[e[T].curveNumber||0])&&(G=O[0].trace,"skip"!==O[0].trace.hoverinfo&&(X.push(O),"h"===G.orientation&&($=!0)));else{for(U=0;U<t.calcdata.length;U++)O=t.calcdata[U],"skip"!==(G=O[0].trace).hoverinfo&&f.isTraceInSubplots(G,A)&&(X.push(O),"h"===G.orientation&&($=!0));var tt,et,nt=!e.target;if(nt)tt="xpx"in e?e.xpx:v[0]._length/2,et="ypx"in e?e.ypx:B[0]._length/2;else{if(!1===s.triggerHandler(t,"plotly_beforehover",e))return;var rt=e.target.getBoundingClientRect();if(tt=e.clientX-rt.left,et=e.clientY-rt.top,tt<0||tt>v[0]._length||et<0||et>B[0]._length)return g.unhoverRaw(t,e)}if(e.pointerX=tt+v[0]._offset,e.pointerY=et+B[0]._offset,N="xval"in e?f.flat(A,e.xval):f.p2c(v,tt),F="yval"in e?f.flat(A,e.yval):f.p2c(B,et),!i(N[0])||!i(F[0]))return a.warn("Fx.hover failed",e,t),g.unhoverRaw(t,e)}var it=1/0;for(U=0;U<X.length;U++)if((O=X[U])&&O[0]&&O[0].trace&&!0===(G=O[0].trace).visible&&0!==G._length&&-1===["carpet","contourcarpet"].indexOf(G._module.name)){if("splom"===G.type?H=A[j=0]:(H=f.getSubplot(G),j=A.indexOf(H)),Y=R,z={cd:O,trace:G,xa:v[j],ya:B[j],maxHoverDistance:q,maxSpikeDistance:V,index:!1,distance:Math.min(it,q),spikeDistance:1/0,xSpike:void 0,ySpike:void 0,color:u.defaultLine,name:G.name,x0:void 0,x1:void 0,y0:void 0,y1:void 0,xLabelVal:void 0,yLabelVal:void 0,zLabelVal:void 0,text:void 0},c[H]&&(z.subplot=c[H]._subplot),c._splomScenes&&c._splomScenes[G.uid]&&(z.scene=c._splomScenes[G.uid]),J=W.length,"array"===Y){var ot=e[U];"pointNumber"in ot?(z.index=ot.pointNumber,Y="closest"):(Y="","xval"in ot&&(P=ot.xval,Y="x"),"yval"in ot&&(K=ot.yval,Y=Y?"closest":"y"))}else P=N[j],K=F[j];if(0!==q)if(G._module&&G._module.hoverPoints){var at=G._module.hoverPoints(z,P,K,Y,c._hoverlayer);if(at)for(var st,At=0;At<at.length;At++)st=at[At],i(st.x0)&&i(st.y0)&&W.push(x(st,R))}else a.log("Unrecognized trace type in hover:",G);if("closest"===R&&W.length>J&&(W.splice(0,J),it=W[0].distance),C&&0!==V&&0===W.length){z.distance=V,z.index=!1;var lt=G._module.hoverPoints(z,P,K,"closest",c._hoverlayer);if(lt&&(lt=lt.filter(function(t){return t.spikeDistance<=V})),lt&&lt.length){var ct,ut=lt.filter(function(t){return t.xa.showspikes});if(ut.length){var gt=ut[0];i(gt.x0)&&i(gt.y0)&&(ct=pt(gt),(!Z.vLinePoint||Z.vLinePoint.spikeDistance>ct.spikeDistance)&&(Z.vLinePoint=ct))}var ht=lt.filter(function(t){return t.ya.showspikes});if(ht.length){var dt=ht[0];i(dt.x0)&&i(dt.y0)&&(ct=pt(dt),(!Z.hLinePoint||Z.hLinePoint.spikeDistance>ct.spikeDistance)&&(Z.hLinePoint=ct))}}}}function ft(t,e){for(var n,r=null,i=1/0,o=0;o<t.length;o++)(n=t[o].spikeDistance)<i&&n<=e&&(r=t[o],i=n);return r}function pt(t){return t?{xa:t.xa,ya:t.ya,x:void 0!==t.xSpike?t.xSpike:(t.x0+t.x1)/2,y:void 0!==t.ySpike?t.ySpike:(t.y0+t.y1)/2,distance:t.distance,spikeDistance:t.spikeDistance,curveNumber:t.trace.index,color:t.color,pointNumber:t.index}:null}var It={fullLayout:c,container:c._hoverlayer,outerContainer:c._paperdiv,event:e},Ct=t._spikepoints,mt={vLinePoint:Z.vLinePoint,hLinePoint:Z.hLinePoint};if(t._spikepoints=mt,C&&0!==V&&0!==W.length){var yt=W.filter(function(t){return t.ya.showspikes}),Et=ft(yt,V);Z.hLinePoint=pt(Et);var vt=W.filter(function(t){return t.xa.showspikes}),Bt=ft(vt,V);Z.vLinePoint=pt(Bt)}if(0===W.length){var bt=g.unhoverRaw(t,e);return!C||null===Z.hLinePoint&&null===Z.vLinePoint||D(Ct)&&_(Z,It),bt}C&&D(Ct)&&_(Z,It),W.sort(function(t,e){return t.distance-e.distance});var Qt=t._hoverdata,wt=[];for(T=0;T<W.length;T++){var xt=W[T],_t=f.makeEventData(xt,xt.trace,xt.cd);if(!1!==xt.hovertemplate){var Dt=!1;xt.cd[xt.index]&&xt.cd[xt.index].ht&&(Dt=xt.cd[xt.index].ht),xt.hovertemplate=Dt||xt.trace.hovertemplate||!1}xt.eventData=[_t],wt.push(_t)}t._hoverdata=wt;var Mt="y"===R&&(X.length>1||W.length>1)||"closest"===R&&$&&W.length>1,kt=u.combine(c.plot_bgcolor||u.background,c.paper_bgcolor),St={hovermode:R,rotateLabels:Mt,bgColor:kt,container:c._hoverlayer,outerContainer:c._paperdiv,commonLabelOpts:c.hoverlabel,hoverdistance:c.hoverdistance},Lt=Q(W,St,t);if(function(t,e,n){var r,i,o,a,s,A,l,c=0,u=1,g=t.size(),h=new Array(g),d=0;function f(t){var e=t[0],n=t[t.length-1];if(i=e.pmin-e.pos-e.dp+e.size,o=n.pos+n.dp+n.size-e.pmax,i>.01){for(s=t.length-1;s>=0;s--)t[s].dp+=i;r=!1}if(!(o<.01)){if(i<-.01){for(s=t.length-1;s>=0;s--)t[s].dp-=o;r=!1}if(r){var l=0;for(a=0;a<t.length;a++)(A=t[a]).pos+A.dp+A.size>e.pmax&&l++;for(a=t.length-1;a>=0&&!(l<=0);a--)(A=t[a]).pos>e.pmax-1&&(A.del=!0,l--);for(a=0;a<t.length&&!(l<=0);a++)if((A=t[a]).pos<e.pmin+1)for(A.del=!0,l--,o=2*A.size,s=t.length-1;s>=0;s--)t[s].dp-=o;for(a=t.length-1;a>=0&&!(l<=0);a--)(A=t[a]).pos+A.dp+A.size>e.pmax&&(A.del=!0,l--)}}}for(t.each(function(t){var r=t[e],i="x"===r._id.charAt(0),o=r.range;0===d&&o&&o[0]>o[1]!==i&&(u=-1),h[d++]=[{datum:t,traceIndex:t.trace.index,dp:0,pos:t.pos,posref:t.posref,size:t.by*(i?m:1)/2,pmin:0,pmax:i?n.width:n.height}]}),h.sort(function(t,e){return t[0].posref-e[0].posref||u*(e[0].traceIndex-t[0].traceIndex)});!r&&c<=g;){for(c++,r=!0,a=0;a<h.length-1;){var p=h[a],I=h[a+1],C=p[p.length-1],y=I[0];if((i=C.pos+C.dp+C.size-y.pos-y.dp+y.size)>.01&&C.pmin===y.pmin&&C.pmax===y.pmax){for(s=I.length-1;s>=0;s--)I[s].dp+=i;for(p.push.apply(p,I),h.splice(a+1,1),l=0,s=p.length-1;s>=0;s--)l+=p[s].dp;for(o=l/p.length,s=p.length-1;s>=0;s--)p[s].dp-=o;r=!1}else a++}h.forEach(f)}for(a=h.length-1;a>=0;a--){var E=h[a];for(s=E.length-1;s>=0;s--){var v=E[s],B=v.datum;B.offset=v.dp,B.del=v.del}}}(Lt,Mt?"xa":"ya",c),w(Lt,Mt),e.target&&e.target.tagName){var Rt=d.getComponentMethod("annotations","hasClickToShow")(t,wt);l(r.select(e.target),Rt?"pointer":"")}e.target&&!o&&function(t,e,n){if(!n||n.length!==t._hoverdata.length)return!0;for(var r=n.length-1;r>=0;r--){var i=n[r],o=t._hoverdata[r];if(i.curveNumber!==o.curveNumber||String(i.pointNumber)!==String(o.pointNumber)||String(i.pointNumbers)!==String(o.pointNumbers))return!0}return!1}(t,0,Qt)&&(Qt&&t.emit("plotly_unhover",{event:e,points:Qt}),t.emit("plotly_hover",{event:e,points:t._hoverdata,xaxes:v,yaxes:B,xvals:N,yvals:F}))}(t,e,n,o)})},n.loneHover=function(t,e){var n=!0;Array.isArray(t)||(n=!1,t=[t]);var i=t.map(function(t){return{color:t.color||u.defaultLine,x0:t.x0||t.x||0,x1:t.x1||t.x||0,y0:t.y0||t.y||0,y1:t.y1||t.y||0,xLabel:t.xLabel,yLabel:t.yLabel,zLabel:t.zLabel,text:t.text,name:t.name,idealAlign:t.idealAlign,borderColor:t.borderColor,fontFamily:t.fontFamily,fontSize:t.fontSize,fontColor:t.fontColor,nameLength:t.nameLength,textAlign:t.textAlign,trace:t.trace||{index:0,hoverinfo:""},xa:{_offset:0},ya:{_offset:0},index:0,hovertemplate:t.hovertemplate||!1,eventData:t.eventData||!1,hovertemplateLabels:t.hovertemplateLabels||!1}}),o=r.select(e.container),a=e.outerContainer?r.select(e.outerContainer):o,s={hovermode:"closest",rotateLabels:!1,bgColor:e.bgColor||u.background,container:o,outerContainer:a},A=Q(i,s,e.gd),l=0,c=0;return A.sort(function(t,e){return t.y0-e.y0}).each(function(t,n){var r=t.y0-t.by/2;t.offset=r-5<l?l-r+5:0,l=r+t.by+t.offset,n===e.anchorIndex&&(c=t.offset)}).each(function(t){t.offset-=c}),w(A,s.rotateLabels),n?A:A.node()};var b=/<extra>([\s\S]*)<\/extra>/;function Q(t,e,n){var i=n._fullLayout,o=e.hovermode,s=e.rotateLabels,l=e.bgColor,g=e.container,h=e.outerContainer,d=e.commonLabelOpts||{},f=e.fontFamily||p.HOVERFONT,C=e.fontSize||p.HOVERFONTSIZE,m=t[0],y=m.xa,E=m.ya,Q="y"===o?"yLabel":"xLabel",w=m[Q],x=(String(w)||"").split(" ")[0],_=h.node().getBoundingClientRect(),D=_.top,k=_.width,S=_.height,L=void 0!==w&&m.distance<=e.hoverdistance&&("x"===o||"y"===o);if(L){var R,N,F=!0;for(R=0;R<t.length;R++)if(F&&void 0===t[R].zLabel&&(F=!1),N=t[R].hoverinfo||t[R].trace.hoverinfo){var T=Array.isArray(N)?N:N.split("+");if(-1===T.indexOf("all")&&-1===T.indexOf(o)){L=!1;break}}F&&(L=!1)}var U=g.selectAll("g.axistext").data(L?[0]:[]);U.enter().append("g").classed("axistext",!0),U.exit().remove(),U.each(function(){var e=r.select(this),i=a.ensureSingle(e,"path","",function(t){t.style({"stroke-width":"1px"})}),s=a.ensureSingle(e,"text","",function(t){t.attr("data-notex",1)}),l=d.bgcolor||u.defaultLine,g=d.bordercolor||u.contrast(l),h=u.contrast(l);i.style({fill:l,stroke:g}),s.text(w).call(c.font,d.font.family||f,d.font.size||C,d.font.color||h).call(A.positionText,0,0).call(A.convertToTspans,n),e.attr("transform","");var p=s.node().getBoundingClientRect();if("x"===o){s.attr("text-anchor","middle").call(A.positionText,0,"top"===y.side?D-p.bottom-v-B:D-p.top+v+B);var I="top"===y.side?"-":"";i.attr("d","M0,0L"+v+","+I+v+"H"+(B+p.width/2)+"v"+I+(2*B+p.height)+"H-"+(B+p.width/2)+"V"+I+v+"H-"+v+"Z"),e.attr("transform","translate("+(y._offset+(m.x0+m.x1)/2)+","+(E._offset+("top"===y.side?0:E._length))+")")}else{s.attr("text-anchor","right"===E.side?"start":"end").call(A.positionText,("right"===E.side?1:-1)*(B+v),D-p.top-p.height/2);var b="right"===E.side?"":"-";i.attr("d","M0,0L"+b+v+","+v+"V"+(B+p.height/2)+"h"+b+(2*B+p.width)+"V-"+(B+p.height/2)+"H"+b+v+"V-"+v+"Z"),e.attr("transform","translate("+(y._offset+("right"===E.side?y._length:0))+","+(E._offset+(m.y0+m.y1)/2)+")")}t=t.filter(function(t){return void 0!==t.zLabelVal||(t[Q]||"").split(" ")[0]===x})});var O=g.selectAll("g.hovertext").data(t,function(t){return[t.trace.index,t.index,t.x0,t.y0,t.name,t.attr,t.xa,t.ya||""].join(",")});return O.enter().append("g").classed("hovertext",!0).each(function(){var t=r.select(this);t.append("rect").call(u.fill,u.addOpacity(l,.8)),t.append("text").classed("name",!0),t.append("path").style("stroke-width","1px"),t.append("text").classed("nums",!0).call(c.font,f,C)}),O.exit().remove(),O.each(function(t){var e=r.select(this).attr("transform",""),g="",h="",d=t.bgcolor||t.color,p=u.combine(u.opacity(d)?d:u.defaultLine,l),m=u.combine(u.opacity(t.color)?t.color:u.defaultLine,l),y=t.borderColor||u.contrast(p);void 0!==t.nameOverride&&(t.name=t.nameOverride),t.name&&(t.trace._meta&&(t.name=a.templateString(t.name,t.trace._meta)),g=M(t.name,t.nameLength)),void 0!==t.zLabel?(void 0!==t.xLabel&&(h+="x: "+t.xLabel+"<br>"),void 0!==t.yLabel&&(h+="y: "+t.yLabel+"<br>"),"choropleth"!==t.trace.type&&"choroplethmapbox"!==t.trace.type&&(h+=(h?"z: ":"")+t.zLabel)):L&&t[o+"Label"]===w?h=t[("x"===o?"y":"x")+"Label"]||"":void 0===t.xLabel?void 0!==t.yLabel&&"scattercarpet"!==t.trace.type&&(h=t.yLabel):h=void 0===t.yLabel?t.xLabel:"("+t.xLabel+", "+t.yLabel+")",!t.text&&0!==t.text||Array.isArray(t.text)||(h+=(h?"<br>":"")+t.text),void 0!==t.extraText&&(h+=(h?"<br>":"")+t.extraText),""!==h||t.hovertemplate||(""===g&&e.remove(),h=g);var E=i._d3locale,Q=t.hovertemplate||!1,x=t.hovertemplateLabels||t,_=t.eventData[0]||{};Q&&(h=(h=a.hovertemplateString(Q,x,E,_,t.trace._meta)).replace(b,function(e,n){return g=M(n,t.nameLength),""}));var R=e.select("text.nums").call(c.font,t.fontFamily||f,t.fontSize||C,t.fontColor||y).text(h).attr("data-notex",1).call(A.positionText,0,0).call(A.convertToTspans,n),N=e.select("text.name"),F=0,T=0;if(g&&g!==h){N.call(c.font,t.fontFamily||f,t.fontSize||C,m).text(g).attr("data-notex",1).call(A.positionText,0,0).call(A.convertToTspans,n);var U=N.node().getBoundingClientRect();F=U.width+2*B,T=U.height+2*B}else N.remove(),e.select("rect").remove();e.select("path").style({fill:p,stroke:y});var O,G,H=R.node().getBoundingClientRect(),j=t.xa._offset+(t.x0+t.x1)/2,Y=t.ya._offset+(t.y0+t.y1)/2,P=Math.abs(t.x1-t.x0),K=Math.abs(t.y1-t.y0),z=H.width+v+B+F;if(t.ty0=D-H.top,t.bx=H.width+2*B,t.by=Math.max(H.height+2*B,T),t.anchor="start",t.txwidth=H.width,t.tx2width=F,t.offset=0,s)t.pos=j,O=Y+K/2+z<=S,G=Y-K/2-z>=0,"top"!==t.idealAlign&&O||!G?O?(Y+=K/2,t.anchor="start"):t.anchor="middle":(Y-=K/2,t.anchor="end");else if(t.pos=Y,O=j+P/2+z<=k,G=j-P/2-z>=0,"left"!==t.idealAlign&&O||!G)if(O)j+=P/2,t.anchor="start";else{t.anchor="middle";var J=z/2,q=j+J-k,V=j-J;q>0&&(j-=q),V<0&&(j+=-V)}else j-=P/2,t.anchor="end";R.attr("text-anchor",t.anchor),F&&N.attr("text-anchor",t.anchor),e.attr("transform","translate("+j+","+Y+")"+(s?"rotate("+I+")":""))}),O}function w(t,e){t.each(function(t){var n=r.select(this);if(t.del)return n.remove();var i=n.select("text.nums"),o=t.anchor,a="end"===o?-1:1,s={start:1,end:-1,middle:0}[o],l=s*(v+B),u=l+s*(t.txwidth+B),g=0,h=t.offset;"middle"===o&&(l-=t.tx2width/2,u+=t.txwidth/2+B),e&&(h*=-E,g=t.offset*y),n.select("path").attr("d","middle"===o?"M-"+(t.bx/2+t.tx2width/2)+","+(h-t.by/2)+"h"+t.bx+"v"+t.by+"h-"+t.bx+"Z":"M0,0L"+(a*v+g)+","+(v+h)+"v"+(t.by/2-v)+"h"+a*t.bx+"v-"+t.by+"H"+(a*v+g)+"V"+(h-v)+"Z");var d=l+g,f=h+t.ty0-t.by/2+B,p=t.textAlign||"auto";"auto"!==p&&("left"===p&&"start"!==o?(i.attr("text-anchor","start"),d="middle"===o?-t.bx/2-t.tx2width/2+B:-t.bx-B):"right"===p&&"end"!==o&&(i.attr("text-anchor","end"),d="middle"===o?t.bx/2-t.tx2width/2-B:t.bx+B)),i.call(A.positionText,d,f),t.tx2width&&(n.select("text.name").call(A.positionText,u+s*B+g,h+t.ty0-t.by/2+B),n.select("rect").call(c.setRect,u+(s-1)*t.tx2width/2+g,h-t.by/2-1,t.tx2width,t.by+2))})}function x(t,e){var n=t.index,r=t.trace||{},o=t.cd[0],s=t.cd[n]||{};function A(t){return t||i(t)&&0===t}var l=Array.isArray(n)?function(t,e){var i=a.castOption(o,n,t);return A(i)?i:a.extractOption({},r,"",e)}:function(t,e){return a.extractOption(s,r,t,e)};function c(e,n,r){var i=l(n,r);A(i)&&(t[e]=i)}if(c("hoverinfo","hi","hoverinfo"),c("bgcolor","hbg","hoverlabel.bgcolor"),c("borderColor","hbc","hoverlabel.bordercolor"),c("fontFamily","htf","hoverlabel.font.family"),c("fontSize","hts","hoverlabel.font.size"),c("fontColor","htc","hoverlabel.font.color"),c("nameLength","hnl","hoverlabel.namelength"),c("textAlign","hta","hoverlabel.align"),t.posref="y"===e||"closest"===e&&"h"===r.orientation?t.xa._offset+(t.x0+t.x1)/2:t.ya._offset+(t.y0+t.y1)/2,t.x0=a.constrain(t.x0,0,t.xa._length),t.x1=a.constrain(t.x1,0,t.xa._length),t.y0=a.constrain(t.y0,0,t.ya._length),t.y1=a.constrain(t.y1,0,t.ya._length),void 0!==t.xLabelVal&&(t.xLabel="xLabel"in t?t.xLabel:h.hoverLabelText(t.xa,t.xLabelVal),t.xVal=t.xa.c2d(t.xLabelVal)),void 0!==t.yLabelVal&&(t.yLabel="yLabel"in t?t.yLabel:h.hoverLabelText(t.ya,t.yLabelVal),t.yVal=t.ya.c2d(t.yLabelVal)),void 0!==t.zLabelVal&&void 0===t.zLabel&&(t.zLabel=String(t.zLabelVal)),!(isNaN(t.xerr)||"log"===t.xa.type&&t.xerr<=0)){var u=h.tickText(t.xa,t.xa.c2l(t.xerr),"hover").text;void 0!==t.xerrneg?t.xLabel+=" +"+u+" / -"+h.tickText(t.xa,t.xa.c2l(t.xerrneg),"hover").text:t.xLabel+=" ± "+u,"x"===e&&(t.distance+=1)}if(!(isNaN(t.yerr)||"log"===t.ya.type&&t.yerr<=0)){var g=h.tickText(t.ya,t.ya.c2l(t.yerr),"hover").text;void 0!==t.yerrneg?t.yLabel+=" +"+g+" / -"+h.tickText(t.ya,t.ya.c2l(t.yerrneg),"hover").text:t.yLabel+=" ± "+g,"y"===e&&(t.distance+=1)}var d=t.hoverinfo||t.trace.hoverinfo;return d&&"all"!==d&&(-1===(d=Array.isArray(d)?d:d.split("+")).indexOf("x")&&(t.xLabel=void 0),-1===d.indexOf("y")&&(t.yLabel=void 0),-1===d.indexOf("z")&&(t.zLabel=void 0),-1===d.indexOf("text")&&(t.text=void 0),-1===d.indexOf("name")&&(t.name=void 0)),t}function _(t,e){var n,r,i=e.container,a=e.fullLayout,s=e.event,A=!!t.hLinePoint,l=!!t.vLinePoint;if(i.selectAll(".spikeline").remove(),l||A){var g=u.combine(a.plot_bgcolor,a.paper_bgcolor);if(A){var h,d,f=t.hLinePoint;n=f&&f.xa,"cursor"===(r=f&&f.ya).spikesnap?(h=s.pointerX,d=s.pointerY):(h=n._offset+f.x,d=r._offset+f.y);var p,I,C=o.readability(f.color,g)<1.5?u.contrast(g):f.color,m=r.spikemode,y=r.spikethickness,E=r.spikecolor||C,v=r._boundingBox,B=(v.left+v.right)/2<h?v.right:v.left;-1===m.indexOf("toaxis")&&-1===m.indexOf("across")||(-1!==m.indexOf("toaxis")&&(p=B,I=h),-1!==m.indexOf("across")&&(p=r._counterSpan[0],I=r._counterSpan[1]),i.insert("line",":first-child").attr({x1:p,x2:I,y1:d,y2:d,"stroke-width":y,stroke:E,"stroke-dasharray":c.dashStyle(r.spikedash,y)}).classed("spikeline",!0).classed("crisp",!0),i.insert("line",":first-child").attr({x1:p,x2:I,y1:d,y2:d,"stroke-width":y+2,stroke:g}).classed("spikeline",!0).classed("crisp",!0)),-1!==m.indexOf("marker")&&i.insert("circle",":first-child").attr({cx:B+("right"!==r.side?y:-y),cy:d,r:y,fill:E}).classed("spikeline",!0)}if(l){var b,Q,w=t.vLinePoint;n=w&&w.xa,r=w&&w.ya,"cursor"===n.spikesnap?(b=s.pointerX,Q=s.pointerY):(b=n._offset+w.x,Q=r._offset+w.y);var x,_,D=o.readability(w.color,g)<1.5?u.contrast(g):w.color,M=n.spikemode,k=n.spikethickness,S=n.spikecolor||D,L=n._boundingBox,R=(L.top+L.bottom)/2<Q?L.bottom:L.top;-1===M.indexOf("toaxis")&&-1===M.indexOf("across")||(-1!==M.indexOf("toaxis")&&(x=R,_=Q),-1!==M.indexOf("across")&&(x=n._counterSpan[0],_=n._counterSpan[1]),i.insert("line",":first-child").attr({x1:b,x2:b,y1:x,y2:_,"stroke-width":k,stroke:S,"stroke-dasharray":c.dashStyle(n.spikedash,k)}).classed("spikeline",!0).classed("crisp",!0),i.insert("line",":first-child").attr({x1:b,x2:b,y1:x,y2:_,"stroke-width":k+2,stroke:g}).classed("spikeline",!0).classed("crisp",!0)),-1!==M.indexOf("marker")&&i.insert("circle",":first-child").attr({cx:b,cy:R-("top"!==n.side?k:-k),r:k,fill:S}).classed("spikeline",!0)}}}function D(t,e){return!e||e.vLinePoint!==t._spikepoints.vLinePoint||e.hLinePoint!==t._spikepoints.hLinePoint}function M(t,e){return A.plainText(t||"",{len:e,allowedTags:["br","sub","sup","b","i","em"]})}},{"../../lib":719,"../../lib/events":709,"../../lib/override_cursor":730,"../../lib/svg_text_utils":743,"../../plots/cartesian/axes":767,"../../registry":847,"../color":593,"../dragelement":611,"../drawing":614,"./constants":626,"./helpers":628,d3:163,"fast-isnumeric":225,tinycolor2:537}],630:[function(t,e,n){"use strict";var r=t("../../lib");e.exports=function(t,e,n,i){n("hoverlabel.bgcolor",(i=i||{}).bgcolor),n("hoverlabel.bordercolor",i.bordercolor),n("hoverlabel.namelength",i.namelength),r.coerceFont(n,"hoverlabel.font",i.font),n("hoverlabel.align",i.align)}},{"../../lib":719}],631:[function(t,e,n){"use strict";t("../../constants/docs").FORMAT_LINK,e.exports=function(t,e){t=t||{},(e=e||{}).description&&e.description;var n=e.keys||[];if(n.length>0){for(var r=[],i=0;i<n.length;i++)r[i]="`"+n[i]+"`";1===n.length?r[0]:(r.slice(0,-1).join(", "),r.slice(-1))}var o={valType:"string",dflt:"",editType:t.editType||"none"};return!1!==t.arrayOk&&(o.arrayOk=!0),o}},{"../../constants/docs":690}],632:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../lib"),o=t("../dragelement"),a=t("./helpers"),s=t("./layout_attributes"),A=t("./hover");e.exports={moduleType:"component",name:"fx",constants:t("./constants"),schema:{layout:s},attributes:t("./attributes"),layoutAttributes:s,supplyLayoutGlobalDefaults:t("./layout_global_defaults"),supplyDefaults:t("./defaults"),supplyLayoutDefaults:t("./layout_defaults"),calc:t("./calc"),getDistanceFunction:a.getDistanceFunction,getClosest:a.getClosest,inbox:a.inbox,quadrature:a.quadrature,appendArrayPointValue:a.appendArrayPointValue,castHoverOption:function(t,e,n){return i.castOption(t,e,"hoverlabel."+n)},castHoverinfo:function(t,e,n){return i.castOption(t,n,"hoverinfo",function(n){return i.coerceHoverinfo({hoverinfo:n},{_module:t._module},e)})},hover:A.hover,unhover:o.unhover,loneHover:A.loneHover,loneUnhover:function(t){var e=i.isD3Selection(t)?t:r.select(t);e.selectAll("g.hovertext").remove(),e.selectAll(".spikeline").remove()},click:t("./click")}},{"../../lib":719,"../dragelement":611,"./attributes":623,"./calc":624,"./click":625,"./constants":626,"./defaults":627,"./helpers":628,"./hover":629,"./layout_attributes":633,"./layout_defaults":634,"./layout_global_defaults":635,d3:163}],633:[function(t,e,n){"use strict";var r=t("./constants"),i=t("../../plots/font_attributes")({editType:"none"});i.family.dflt=r.HOVERFONT,i.size.dflt=r.HOVERFONTSIZE,e.exports={clickmode:{valType:"flaglist",flags:["event","select"],dflt:"event",editType:"plot",extras:["none"]},dragmode:{valType:"enumerated",values:["zoom","pan","select","lasso","orbit","turntable",!1],dflt:"zoom",editType:"modebar"},hovermode:{valType:"enumerated",values:["x","y","closest",!1],editType:"modebar"},hoverdistance:{valType:"integer",min:-1,dflt:20,editType:"none"},spikedistance:{valType:"integer",min:-1,dflt:20,editType:"none"},hoverlabel:{bgcolor:{valType:"color",editType:"none"},bordercolor:{valType:"color",editType:"none"},font:i,align:{valType:"enumerated",values:["left","right","auto"],dflt:"auto",editType:"none"},namelength:{valType:"integer",min:-1,dflt:15,editType:"none"},editType:"none"},selectdirection:{valType:"enumerated",values:["h","v","d","any"],dflt:"any",editType:"none"}}},{"../../plots/font_attributes":793,"./constants":626}],634:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./layout_attributes");e.exports=function(t,e,n){function o(n,o){return r.coerce(t,e,i,n,o)}var a,s=o("clickmode");"select"===o("dragmode")&&o("selectdirection"),e._has("cartesian")?s.indexOf("select")>-1?a="closest":(e._isHoriz=function(t,e){for(var n=e._scatterStackOpts||{},r=0;r<t.length;r++){var i=t[r],o=i.xaxis+i.yaxis,a=n[o]||{},s=a[i.stackgroup]||{};if("h"!==i.orientation&&"h"!==s.orientation)return!1}return!0}(n,e),a=e._isHoriz?"y":"x"):a="closest",o("hovermode",a)&&(o("hoverdistance"),o("spikedistance"));var A=e._has("mapbox"),l=e._has("geo"),c=e._basePlotModules.length;"zoom"===e.dragmode&&((A||l)&&1===c||A&&l&&2===c)&&(e.dragmode="pan")}},{"../../lib":719,"./layout_attributes":633}],635:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./hoverlabel_defaults"),o=t("./layout_attributes");e.exports=function(t,e){i(t,e,function(n,i){return r.coerce(t,e,o,n,i)})}},{"../../lib":719,"./hoverlabel_defaults":630,"./layout_attributes":633}],636:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../lib/regex").counter,o=t("../../plots/domain").attributes,a=t("../../plots/cartesian/constants").idRegex,s=t("../../plot_api/plot_template"),A={rows:{valType:"integer",min:1,editType:"plot"},roworder:{valType:"enumerated",values:["top to bottom","bottom to top"],dflt:"top to bottom",editType:"plot"},columns:{valType:"integer",min:1,editType:"plot"},subplots:{valType:"info_array",freeLength:!0,dimensions:2,items:{valType:"enumerated",values:[i("xy").toString(),""],editType:"plot"},editType:"plot"},xaxes:{valType:"info_array",freeLength:!0,items:{valType:"enumerated",values:[a.x.toString(),""],editType:"plot"},editType:"plot"},yaxes:{valType:"info_array",freeLength:!0,items:{valType:"enumerated",values:[a.y.toString(),""],editType:"plot"},editType:"plot"},pattern:{valType:"enumerated",values:["independent","coupled"],dflt:"coupled",editType:"plot"},xgap:{valType:"number",min:0,max:1,editType:"plot"},ygap:{valType:"number",min:0,max:1,editType:"plot"},domain:o({name:"grid",editType:"plot",noGridCell:!0},{}),xside:{valType:"enumerated",values:["bottom","bottom plot","top plot","top"],dflt:"bottom plot",editType:"plot"},yside:{valType:"enumerated",values:["left","left plot","right plot","right"],dflt:"left plot",editType:"plot"},editType:"plot"};function l(t,e,n){var r=e[n+"axes"],i=Object.keys((t._splomAxes||{})[n]||{});return Array.isArray(r)?r:i.length?i:void 0}function c(t,e,n,r,i,o){var a=e(t+"gap",n),s=e("domain."+t);e(t+"side",r);for(var A=new Array(i),l=s[0],c=(s[1]-l)/(i-a),u=c*(1-a),g=0;g<i;g++){var h=l+c*g;A[o?i-1-g:g]=[h,h+u]}return A}function u(t,e,n,r,i){var o,a=new Array(n);function s(t,n){-1!==e.indexOf(n)&&void 0===r[n]?(a[t]=n,r[n]=t):a[t]=""}if(Array.isArray(t))for(o=0;o<n;o++)s(o,t[o]);else for(s(0,i),o=1;o<n;o++)s(o,i+(o+1));return a}e.exports={moduleType:"component",name:"grid",schema:{layout:{grid:A}},layoutAttributes:A,sizeDefaults:function(t,e){var n=t.grid||{},i=l(e,n,"x"),o=l(e,n,"y");if(t.grid||i||o){var a,u,g=Array.isArray(n.subplots)&&Array.isArray(n.subplots[0]),h=Array.isArray(i),d=Array.isArray(o),f=h&&i!==n.xaxes&&d&&o!==n.yaxes;g?(a=n.subplots.length,u=n.subplots[0].length):(d&&(a=o.length),h&&(u=i.length));var p=s.newContainer(e,"grid"),I=b("rows",a),C=b("columns",u);if(I*C>1){g||h||d||"independent"===b("pattern")&&(g=!0),p._hasSubplotGrid=g;var m,y,E="top to bottom"===b("roworder"),v=g?.2:.1,B=g?.3:.1;f&&e._splomGridDflt&&(m=e._splomGridDflt.xside,y=e._splomGridDflt.yside),p._domains={x:c("x",b,v,m,C),y:c("y",b,B,y,I,E)}}else delete e.grid}function b(t,e){return r.coerce(n,p,A,t,e)}},contentDefaults:function(t,e){var n=e.grid;if(n&&n._domains){var r,i,o,a,s,A,c,g=t.grid||{},h=e._subplots,d=n._hasSubplotGrid,f=n.rows,p=n.columns,I="independent"===n.pattern,C=n._axisMap={};if(d){var m=g.subplots||[];A=n.subplots=new Array(f);var y=1;for(r=0;r<f;r++){var E=A[r]=new Array(p),v=m[r]||[];for(i=0;i<p;i++)if(I?(s=1===y?"xy":"x"+y+"y"+y,y++):s=v[i],E[i]="",-1!==h.cartesian.indexOf(s)){if(c=s.indexOf("y"),o=s.slice(0,c),a=s.slice(c),void 0!==C[o]&&C[o]!==i||void 0!==C[a]&&C[a]!==r)continue;E[i]=s,C[o]=i,C[a]=r}}}else{var B=l(e,g,"x"),b=l(e,g,"y");n.xaxes=u(B,h.xaxis,p,C,"x"),n.yaxes=u(b,h.yaxis,f,C,"y")}var Q=n._anchors={},w="top to bottom"===n.roworder;for(var x in C){var _,D,M,k=x.charAt(0),S=n[k+"side"];if(S.length<8)Q[x]="free";else if("x"===k){if("t"===S.charAt(0)===w?(_=0,D=1,M=f):(_=f-1,D=-1,M=-1),d){var L=C[x];for(r=_;r!==M;r+=D)if((s=A[r][L])&&(c=s.indexOf("y"),s.slice(0,c)===x)){Q[x]=s.slice(c);break}}else for(r=_;r!==M;r+=D)if(a=n.yaxes[r],-1!==h.cartesian.indexOf(x+a)){Q[x]=a;break}}else if("l"===S.charAt(0)?(_=0,D=1,M=p):(_=p-1,D=-1,M=-1),d){var R=C[x];for(r=_;r!==M;r+=D)if((s=A[R][r])&&(c=s.indexOf("y"),s.slice(c)===x)){Q[x]=s.slice(0,c);break}}else for(r=_;r!==M;r+=D)if(o=n.xaxes[r],-1!==h.cartesian.indexOf(o+x)){Q[x]=o;break}}}}}},{"../../lib":719,"../../lib/regex":735,"../../plot_api/plot_template":757,"../../plots/cartesian/constants":773,"../../plots/domain":792}],637:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/constants"),i=t("../../plot_api/plot_template").templatedArray;e.exports=i("image",{visible:{valType:"boolean",dflt:!0,editType:"arraydraw"},source:{valType:"string",editType:"arraydraw"},layer:{valType:"enumerated",values:["below","above"],dflt:"above",editType:"arraydraw"},sizex:{valType:"number",dflt:0,editType:"arraydraw"},sizey:{valType:"number",dflt:0,editType:"arraydraw"},sizing:{valType:"enumerated",values:["fill","contain","stretch"],dflt:"contain",editType:"arraydraw"},opacity:{valType:"number",min:0,max:1,dflt:1,editType:"arraydraw"},x:{valType:"any",dflt:0,editType:"arraydraw"},y:{valType:"any",dflt:0,editType:"arraydraw"},xanchor:{valType:"enumerated",values:["left","center","right"],dflt:"left",editType:"arraydraw"},yanchor:{valType:"enumerated",values:["top","middle","bottom"],dflt:"top",editType:"arraydraw"},xref:{valType:"enumerated",values:["paper",r.idRegex.x.toString()],dflt:"paper",editType:"arraydraw"},yref:{valType:"enumerated",values:["paper",r.idRegex.y.toString()],dflt:"paper",editType:"arraydraw"},editType:"arraydraw"})},{"../../plot_api/plot_template":757,"../../plots/cartesian/constants":773}],638:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib/to_log_range");e.exports=function(t,e,n,o){e=e||{};var a="log"===n&&"linear"===e.type,s="linear"===n&&"log"===e.type;if(a||s)for(var A,l,c=t._fullLayout.images,u=e._id.charAt(0),g=0;g<c.length;g++)if(l="images["+g+"].",(A=c[g])[u+"ref"]===e._id){var h=A[u],d=A["size"+u],f=null,p=null;if(a){f=i(h,e.range);var I=d/Math.pow(10,f)/2;p=2*Math.log(I+Math.sqrt(1+I*I))/Math.LN10}else p=(f=Math.pow(10,h))*(Math.pow(10,d/2)-Math.pow(10,-d/2));r(f)?r(p)||(p=null):(f=null,p=null),o(l+u,f),o(l+"size"+u,p)}}},{"../../lib/to_log_range":745,"fast-isnumeric":225}],639:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axes"),o=t("../../plots/array_container_defaults"),a=t("./attributes");function s(t,e,n){function o(n,i){return r.coerce(t,e,a,n,i)}var s=o("source");if(!o("visible",!!s))return e;o("layer"),o("xanchor"),o("yanchor"),o("sizex"),o("sizey"),o("sizing"),o("opacity");for(var A={_fullLayout:n},l=["x","y"],c=0;c<2;c++){var u=l[c],g=i.coerceRef(t,e,A,u,"paper");"paper"!==g&&i.getFromId(A,g)._imgIndices.push(e._index),i.coercePosition(e,A,o,g,u,0)}return e}e.exports=function(t,e){o(t,e,{name:"images",handleItemDefaults:s})}},{"../../lib":719,"../../plots/array_container_defaults":763,"../../plots/cartesian/axes":767,"./attributes":637}],640:[function(t,e,n){"use strict";var r=t("d3"),i=t("../drawing"),o=t("../../plots/cartesian/axes"),a=t("../../constants/xmlns_namespaces");e.exports=function(t){var e,n,s=t._fullLayout,A=[],l={},c=[];for(n=0;n<s.images.length;n++){var u=s.images[n];if(u.visible)if("below"===u.layer&&"paper"!==u.xref&&"paper"!==u.yref){e=u.xref+u.yref;var g=s._plots[e];if(!g){c.push(u);continue}g.mainplot&&(e=g.mainplot.id),l[e]||(l[e]=[]),l[e].push(u)}else"above"===u.layer?A.push(u):c.push(u)}var h={x:{left:{sizing:"xMin",offset:0},center:{sizing:"xMid",offset:-.5},right:{sizing:"xMax",offset:-1}},y:{top:{sizing:"YMin",offset:0},middle:{sizing:"YMid",offset:-.5},bottom:{sizing:"YMax",offset:-1}}};function d(e){var n=r.select(this);if(!this.img||this.img.src!==e.source){n.attr("xmlns",a.svg);var i=new Promise(function(t){var r=new Image;function i(){n.remove(),t()}this.img=r,r.setAttribute("crossOrigin","anonymous"),r.onerror=i,r.onload=function(){var e=document.createElement("canvas");e.width=this.width,e.height=this.height,e.getContext("2d").drawImage(this,0,0);var r=e.toDataURL("image/png");n.attr("xlink:href",r),t()},n.on("error",i),r.src=e.source}.bind(this));t._promises.push(i)}}function f(e){var n=r.select(this),a=o.getFromId(t,e.xref),A=o.getFromId(t,e.yref),l=s._size,c=a?Math.abs(a.l2p(e.sizex)-a.l2p(0)):e.sizex*l.w,u=A?Math.abs(A.l2p(e.sizey)-A.l2p(0)):e.sizey*l.h,g=c*h.x[e.xanchor].offset,d=u*h.y[e.yanchor].offset,f=h.x[e.xanchor].sizing+h.y[e.yanchor].sizing,p=(a?a.r2p(e.x)+a._offset:e.x*l.w+l.l)+g,I=(A?A.r2p(e.y)+A._offset:l.h-e.y*l.h+l.t)+d;switch(e.sizing){case"fill":f+=" slice";break;case"stretch":f="none"}n.attr({x:p,y:I,width:c,height:u,preserveAspectRatio:f,opacity:e.opacity});var C=(a?a._id:"")+(A?A._id:"");i.setClipUrl(n,C?"clip"+s._uid+C:null,t)}var p=s._imageLowerLayer.selectAll("image").data(c),I=s._imageUpperLayer.selectAll("image").data(A);p.enter().append("image"),I.enter().append("image"),p.exit().remove(),I.exit().remove(),p.each(function(t){d.bind(this)(t),f.bind(this)(t)}),I.each(function(t){d.bind(this)(t),f.bind(this)(t)});var C=Object.keys(s._plots);for(n=0;n<C.length;n++){e=C[n];var m=s._plots[e];if(m.imagelayer){var y=m.imagelayer.selectAll("image").data(l[e]||[]);y.enter().append("image"),y.exit().remove(),y.each(function(t){d.bind(this)(t),f.bind(this)(t)})}}}},{"../../constants/xmlns_namespaces":696,"../../plots/cartesian/axes":767,"../drawing":614,d3:163}],641:[function(t,e,n){"use strict";e.exports={moduleType:"component",name:"images",layoutAttributes:t("./attributes"),supplyLayoutDefaults:t("./defaults"),includeBasePlot:t("../../plots/cartesian/include_components")("images"),draw:t("./draw"),convertCoords:t("./convert_coords")}},{"../../plots/cartesian/include_components":777,"./attributes":637,"./convert_coords":638,"./defaults":639,"./draw":640}],642:[function(t,e,n){"use strict";var r=t("../../plots/font_attributes"),i=t("../color/attributes");e.exports={bgcolor:{valType:"color",editType:"legend"},bordercolor:{valType:"color",dflt:i.defaultLine,editType:"legend"},borderwidth:{valType:"number",min:0,dflt:0,editType:"legend"},font:r({editType:"legend"}),orientation:{valType:"enumerated",values:["v","h"],dflt:"v",editType:"legend"},traceorder:{valType:"flaglist",flags:["reversed","grouped"],extras:["normal"],editType:"legend"},tracegroupgap:{valType:"number",min:0,dflt:10,editType:"legend"},itemsizing:{valType:"enumerated",values:["trace","constant"],dflt:"trace",editType:"legend"},itemclick:{valType:"enumerated",values:["toggle","toggleothers",!1],dflt:"toggle",editType:"legend"},itemdoubleclick:{valType:"enumerated",values:["toggle","toggleothers",!1],dflt:"toggleothers",editType:"legend"},x:{valType:"number",min:-2,max:3,dflt:1.02,editType:"legend"},xanchor:{valType:"enumerated",values:["auto","left","center","right"],dflt:"left",editType:"legend"},y:{valType:"number",min:-2,max:3,dflt:1,editType:"legend"},yanchor:{valType:"enumerated",values:["auto","top","middle","bottom"],dflt:"auto",editType:"legend"},uirevision:{valType:"any",editType:"none"},valign:{valType:"enumerated",values:["top","middle","bottom"],dflt:"middle",editType:"legend"},editType:"legend"}},{"../../plots/font_attributes":793,"../color/attributes":592}],643:[function(t,e,n){"use strict";e.exports={scrollBarWidth:6,scrollBarMinHeight:20,scrollBarColor:"#808BA4",scrollBarMargin:4,textOffsetX:40}},{}],644:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib"),o=t("../../plot_api/plot_template"),a=t("./attributes"),s=t("../../plots/layout_attributes"),A=t("./helpers");e.exports=function(t,e,n){for(var l,c,u,g,h=t.legend||{},d=0,f=!1,p="normal",I=0;I<n.length;I++){var C=n[I];C.visible&&((C.showlegend||C._dfltShowLegend)&&(d++,C.showlegend&&(f=!0,(r.traceIs(C,"pie-like")||!0===C._input.showlegend)&&d++)),(r.traceIs(C,"bar")&&"stack"===e.barmode||-1!==["tonextx","tonexty"].indexOf(C.fill))&&(p=A.isGrouped({traceorder:p})?"grouped+reversed":"reversed"),void 0!==C.legendgroup&&""!==C.legendgroup&&(p=A.isReversed({traceorder:p})?"reversed+grouped":"grouped"))}var m=i.coerce(t,e,s,"showlegend",f&&d>1);if(!1!==m||h.uirevision){var y=o.newContainer(e,"legend");if(v("uirevision",e.uirevision),!1!==m){if(v("bgcolor",e.paper_bgcolor),v("bordercolor"),v("borderwidth"),i.coerceFont(v,"font",e.font),v("orientation"),"h"===y.orientation){var E=t.xaxis;r.getComponentMethod("rangeslider","isVisible")(E)?(l=0,u="left",c=1.1,g="bottom"):(l=0,u="left",c=-.1,g="top")}v("traceorder",p),A.isGrouped(e.legend)&&v("tracegroupgap"),v("itemsizing"),v("itemclick"),v("itemdoubleclick"),v("x",l),v("xanchor",u),v("y",c),v("yanchor",g),v("valign"),i.noneOrAll(h,y,["x","y"])}}function v(t,e){return i.coerce(h,y,a,t,e)}}},{"../../lib":719,"../../plot_api/plot_template":757,"../../plots/layout_attributes":819,"../../registry":847,"./attributes":642,"./helpers":648}],645:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../lib"),o=t("../../plots/plots"),a=t("../../registry"),s=t("../../lib/events"),A=t("../dragelement"),l=t("../drawing"),c=t("../color"),u=t("../../lib/svg_text_utils"),g=t("./handle_click"),h=t("./constants"),d=t("../../constants/alignment"),f=d.LINE_SPACING,p=d.FROM_TL,I=d.FROM_BR,C=t("./get_legend_data"),m=t("./style"),y=t("./helpers");function E(t,e,n,r,i){var o=n.data()[0][0].trace,A={event:i,node:n.node(),curveNumber:o.index,expandedIndex:o._expandedIndex,data:t.data,layout:t.layout,frames:t._transitionData._frames,config:t._context,fullData:t._fullData,fullLayout:t._fullLayout};o._group&&(A.group=o._group),a.traceIs(o,"pie-like")&&(A.label=n.datum()[0].label),!1!==s.triggerHandler(t,"plotly_legendclick",A)&&(1===r?e._clickTimeout=setTimeout(function(){g(n,t,r)},t._context.doubleClickDelay):2===r&&(e._clickTimeout&&clearTimeout(e._clickTimeout),t._legendMouseDownTime=0,!1!==s.triggerHandler(t,"plotly_legenddoubleclick",A)&&g(n,t,r)))}function v(t,e,n){var r=t.data()[0][0],o=e._fullLayout,s=r.trace,A=a.traceIs(s,"pie-like"),c=s.index,g=e._context.edits.legendText&&!A,d=A?r.label:s.name;s._meta&&(d=i.templateString(d,s._meta));var p=i.ensureSingle(t,"text","legendtext");function I(n){u.convertToTspans(n,e,function(){!function(t,e){var n=t.data()[0][0];if(n.trace.showlegend){var r,i,o=t.select("g[class*=math-group]"),a=o.node(),s=e._fullLayout.legend.font.size*f;if(a){var A=l.bBox(a);r=A.height,i=A.width,l.setTranslate(o,0,r/4)}else{var c=t.select(".legendtext"),g=u.lineCount(c),d=c.node();r=s*g,i=d?l.bBox(d).width:0;var p=s*(.3+(1-g)/2);u.positionText(c,h.textOffsetX,p)}n.lineHeight=s,n.height=Math.max(r,16)+3,n.width=i}else t.remove()}(t,e)})}p.attr("text-anchor","start").classed("user-select-none",!0).call(l.font,o.legend.font).text(g?B(d,n):d),u.positionText(p,h.textOffsetX,0),g?p.call(u.makeEditable,{gd:e,text:d}).call(I).on("edit",function(t){this.text(B(t,n)).call(I);var o=r.trace._fullInput||{},s={};if(a.hasTransform(o,"groupby")){var A=a.getTransformIndices(o,"groupby"),l=A[A.length-1],u=i.keyedContainer(o,"transforms["+l+"].styles","target","value.name");u.set(r.trace._group,t),s=u.constructUpdate()}else s.name=t;return a.call("_guiRestyle",e,s,c)}):I(p)}function B(t,e){var n=Math.max(4,e);if(t&&t.trim().length>=n/2)return t;for(var r=n-(t=t||"").length;r>0;r--)t+=" ";return t}function b(t,e){var n,o=e._context.doubleClickDelay,a=1,s=i.ensureSingle(t,"rect","legendtoggle",function(t){t.style("cursor","pointer").attr("pointer-events","all").call(c.fill,"rgba(0,0,0,0)")});s.on("mousedown",function(){(n=(new Date).getTime())-e._legendMouseDownTime<o?a+=1:(a=1,e._legendMouseDownTime=n)}),s.on("mouseup",function(){if(!e._dragged&&!e._editing){var n=e._fullLayout.legend;(new Date).getTime()-e._legendMouseDownTime>o&&(a=Math.max(a-1,1)),E(e,n,t,a,r.event)}})}function Q(t,e,n){var o=t._fullLayout,a=o.legend,s=a.borderwidth,A=y.isGrouped(a),c=0;if(a._width=0,a._height=0,y.isVertical(a))A&&e.each(function(t,e){l.setTranslate(this,0,e*a.tracegroupgap)}),n.each(function(t){var e=t[0],n=e.height,r=e.width;l.setTranslate(this,s,5+s+a._height+n/2),a._height+=n,a._width=Math.max(a._width,r)}),a._width+=45+2*s,a._height+=10+2*s,A&&(a._height+=(a._lgroupsLength-1)*a.tracegroupgap),c=40;else if(A){var u,g=0,h=0,d=e.data(),f=0;for(u=0;u<d.length;u++){var p=d[u],I=p.map(function(t){return t[0].width}),C=i.aggNums(Math.max,null,I),m=p.reduce(function(t,e){return t+e[0].height},0);h=Math.max(h,C),g=Math.max(g,m),f=Math.max(f,p.length)}h+=5,h+=40;var E=[a._width],v=[],B=0;for(u=0;u<d.length;u++){o._size.w<s+a._width+5+h?(E[E.length-1]=E[0],a._width=h,B++):a._width+=h+s;var b=B*g;b+=B>0?a.tracegroupgap:0,v.push(b),E.push(a._width)}e.each(function(t,e){l.setTranslate(this,E[e],v[e])}),e.each(function(){var t=r.select(this).selectAll("g.traces"),e=0;t.each(function(t){var n=t[0].height;l.setTranslate(this,0,5+s+e+n/2),e+=n})});var Q=v[v.length-1]+g;a._height=10+2*s+Q;var w=Math.max.apply(null,E);a._width=w+h+40,a._width+=2*s}else{var x=0,_=0,D=0,M=0,k=0;n.each(function(t){D=Math.max(40+t[0].width,D),k+=40+t[0].width+5});var S=o._size.w>s+k-5;n.each(function(t){var e=t[0],n=S?40+t[0].width:D;s+M+5+n>o._size.w&&(M=0,x+=_,a._height+=_,_=0),l.setTranslate(this,s+M,5+s+e.height/2+x),a._width+=5+n,M+=5+n,_=Math.max(e.height,_)}),S?a._height=_:a._height+=_,a._width+=2*s,a._height+=10+2*s}a._width=Math.ceil(a._width),a._height=Math.ceil(a._height);var L=t._context.edits.legendText||t._context.edits.legendPosition;n.each(function(t){var e=t[0],n=r.select(this).select(".legendtoggle");l.setRect(n,0,-e.height/2,(L?0:a._width)+c,e.height)})}function w(t){var e=t._fullLayout.legend,n="left";i.isRightAnchor(e)?n="right":i.isCenterAnchor(e)&&(n="center");var r="top";i.isBottomAnchor(e)?r="bottom":i.isMiddleAnchor(e)&&(r="middle"),o.autoMargin(t,"legend",{x:e.x,y:e.y,l:e._width*p[n],r:e._width*I[n],b:e._height*I[r],t:e._height*p[r]})}e.exports=function(t){var e=t._fullLayout,n="legend"+e._uid;if(e._infolayer&&t.calcdata){t._legendMouseDownTime||(t._legendMouseDownTime=0);var s=e.legend,u=e.showlegend&&C(t.calcdata,s),g=e.hiddenlabels||[];if(!e.showlegend||!u.length)return e._infolayer.selectAll(".legend").remove(),e._topdefs.select("#"+n).remove(),void o.autoMargin(t,"legend");for(var d=0,f=0;f<u.length;f++)for(var y=0;y<u[f].length;y++){var B=u[f][y][0],x=B.trace,_=a.traceIs(x,"pie-like")?B.label:x.name;d=Math.max(d,_&&_.length||0)}var D=!1,M=i.ensureSingle(e._infolayer,"g","legend",function(t){t.attr("pointer-events","all"),D=!0}),k=i.ensureSingleById(e._topdefs,"clipPath",n,function(t){t.append("rect")}),S=i.ensureSingle(M,"rect","bg",function(t){t.attr("shape-rendering","crispEdges")});S.call(c.stroke,s.bordercolor).call(c.fill,s.bgcolor).style("stroke-width",s.borderwidth+"px");var L=i.ensureSingle(M,"g","scrollbox"),R=i.ensureSingle(M,"rect","scrollbar",function(t){t.attr({rx:20,ry:3,width:0,height:0}).call(c.fill,"#808BA4")}),N=L.selectAll("g.groups").data(u);N.enter().append("g").attr("class","groups"),N.exit().remove();var F=N.selectAll("g.traces").data(i.identity);F.enter().append("g").attr("class","traces"),F.exit().remove(),F.style("opacity",function(t){var e=t[0].trace;return a.traceIs(e,"pie-like")?-1!==g.indexOf(t[0].label)?.5:1:"legendonly"===e.visible?.5:1}).each(function(){r.select(this).call(v,t,d)}).call(m,t).each(function(){r.select(this).call(b,t)}),i.syncOrAsync([o.previousPromises,function(){D&&(Q(t,N,F),w(t));var c=e.width,u=e.height;Q(t,N,F),s._height>u?function(t){var e=t._fullLayout.legend,n="left";i.isRightAnchor(e)?n="right":i.isCenterAnchor(e)&&(n="center"),o.autoMargin(t,"legend",{x:e.x,y:.5,l:e._width*p[n],r:e._width*I[n],b:0,t:0})}(t):w(t);var g=e._size,d=g.l+g.w*s.x,f=g.t+g.h*(1-s.y);i.isRightAnchor(s)?d-=s._width:i.isCenterAnchor(s)&&(d-=s._width/2),i.isBottomAnchor(s)?f-=s._height:i.isMiddleAnchor(s)&&(f-=s._height/2);var C=s._width,m=g.w;C>m?(d=g.l,C=m):(d+C>c&&(d=c-C),d<0&&(d=0),C=Math.min(c-d,s._width));var y,v,B,b,x=s._height,_=g.h;if(x>_?(f=g.t,x=_):(f+x>u&&(f=u-x),f<0&&(f=0),x=Math.min(u-f,s._height)),l.setTranslate(M,d,f),R.on(".drag",null),M.on("wheel",null),s._height<=x||t._context.staticPlot)S.attr({width:C-s.borderwidth,height:x-s.borderwidth,x:s.borderwidth/2,y:s.borderwidth/2}),l.setTranslate(L,0,0),k.select("rect").attr({width:C-2*s.borderwidth,height:x-2*s.borderwidth,x:s.borderwidth,y:s.borderwidth}),l.setClipUrl(L,n,t),l.setRect(R,0,0,0,0),delete s._scrollY;else{var T,U,O=Math.max(h.scrollBarMinHeight,x*x/s._height),G=x-O-2*h.scrollBarMargin,H=s._height-x,j=G/H,Y=Math.min(s._scrollY||0,H);S.attr({width:C-2*s.borderwidth+h.scrollBarWidth+h.scrollBarMargin,height:x-s.borderwidth,x:s.borderwidth/2,y:s.borderwidth/2}),k.select("rect").attr({width:C-2*s.borderwidth+h.scrollBarWidth+h.scrollBarMargin,height:x-2*s.borderwidth,x:s.borderwidth,y:s.borderwidth+Y}),l.setClipUrl(L,n,t),K(Y,O,j),M.on("wheel",function(){K(Y=i.constrain(s._scrollY+r.event.deltaY/G*H,0,H),O,j),0!==Y&&Y!==H&&r.event.preventDefault()});var P=r.behavior.drag().on("dragstart",function(){T=r.event.sourceEvent.clientY,U=Y}).on("drag",function(){var t=r.event.sourceEvent;2===t.buttons||t.ctrlKey||K(Y=i.constrain((t.clientY-T)/j+U,0,H),O,j)});R.call(P)}function K(e,n,r){s._scrollY=t._fullLayout.legend._scrollY=e,l.setTranslate(L,0,-e),l.setRect(R,C,h.scrollBarMargin+e*r,h.scrollBarWidth,n),k.select("rect").attr({y:s.borderwidth+e})}t._context.edits.legendPosition&&(M.classed("cursor-move",!0),A.init({element:M.node(),gd:t,prepFn:function(){var t=l.getTranslate(M);B=t.x,b=t.y},moveFn:function(t,e){var n=B+t,r=b+e;l.setTranslate(M,n,r),y=A.align(n,0,g.l,g.l+g.w,s.xanchor),v=A.align(r,0,g.t+g.h,g.t,s.yanchor)},doneFn:function(){void 0!==y&&void 0!==v&&a.call("_guiRelayout",t,{"legend.x":y,"legend.y":v})},clickFn:function(n,r){var i=e._infolayer.selectAll("g.traces").filter(function(){var t=this.getBoundingClientRect();return r.clientX>=t.left&&r.clientX<=t.right&&r.clientY>=t.top&&r.clientY<=t.bottom});i.size()>0&&E(t,M,i,n,r)}}))}],t)}}},{"../../constants/alignment":688,"../../lib":719,"../../lib/events":709,"../../lib/svg_text_utils":743,"../../plots/plots":828,"../../registry":847,"../color":593,"../dragelement":611,"../drawing":614,"./constants":643,"./get_legend_data":646,"./handle_click":647,"./helpers":648,"./style":650,d3:163}],646:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("./helpers");e.exports=function(t,e){var n,o,a={},s=[],A=!1,l={},c=0;function u(t,n){if(""!==t&&i.isGrouped(e))-1===s.indexOf(t)?(s.push(t),A=!0,a[t]=[[n]]):a[t].push([n]);else{var r="~~i"+c;s.push(r),a[r]=[[n]],c++}}for(n=0;n<t.length;n++){var g=t[n],h=g[0],d=h.trace,f=d.legendgroup;if(d.visible&&d.showlegend)if(r.traceIs(d,"pie-like"))for(l[f]||(l[f]={}),o=0;o<g.length;o++){var p=g[o].label;l[f][p]||(u(f,{label:p,color:g[o].color,i:g[o].i,trace:d,pts:g[o].pts}),l[f][p]=!0)}else u(f,h)}if(!s.length)return[];var I,C,m=s.length;if(A&&i.isGrouped(e))for(C=new Array(m),n=0;n<m;n++)I=a[s[n]],C[n]=i.isReversed(e)?I.reverse():I;else{for(C=[new Array(m)],n=0;n<m;n++)I=a[s[n]][0],C[0][i.isReversed(e)?m-n-1:n]=I;m=1}return e._lgroupsLength=m,C}},{"../../registry":847,"./helpers":648}],647:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../registry"),o=!0;e.exports=function(t,e,n){var a=e._fullLayout;if(!e._dragged&&!e._editing){var s,A=a.legend.itemclick,l=a.legend.itemdoubleclick;if(1===n&&"toggle"===A&&"toggleothers"===l&&o&&e.data&&e._context.showTips?(r.notifier(r._(e,"Double-click on legend to isolate one trace"),"long"),o=!1):o=!1,1===n?s=A:2===n&&(s=l),s){var c,u,g,h,d,f=a.hiddenlabels?a.hiddenlabels.slice():[],p=t.data()[0][0],I=e._fullData,C=p.trace,m=C.legendgroup,y={},E=[],v=[],B=[];if(i.traceIs(C,"pie-like")){var b=p.label,Q=f.indexOf(b);"toggle"===s?-1===Q?f.push(b):f.splice(Q,1):"toggleothers"===s&&(f=[],e.calcdata[0].forEach(function(t){b!==t.label&&f.push(t.label)}),e._fullLayout.hiddenlabels&&e._fullLayout.hiddenlabels.length===f.length&&-1===Q&&(f=[])),i.call("_guiRelayout",e,"hiddenlabels",f)}else{var w,x=m&&m.length,_=[];if(x)for(c=0;c<I.length;c++)(w=I[c]).visible&&w.legendgroup===m&&_.push(c);if("toggle"===s){var D;switch(C.visible){case!0:D="legendonly";break;case!1:D=!1;break;case"legendonly":D=!0}if(x)for(c=0;c<I.length;c++)!1!==I[c].visible&&I[c].legendgroup===m&&F(I[c],D);else F(C,D)}else if("toggleothers"===s){var M,k,S=!0;for(c=0;c<I.length;c++)if(I[c]!==C&&!(M=x&&I[c].legendgroup===m)&&!0===I[c].visible&&!i.traceIs(I[c],"notLegendIsolatable")){S=!1;break}for(c=0;c<I.length;c++)if(!1!==I[c].visible&&!i.traceIs(I[c],"notLegendIsolatable"))switch(C.visible){case"legendonly":F(I[c],!0);break;case!0:k=!!S||"legendonly",M=I[c]===C||x&&I[c].legendgroup===m,F(I[c],!!M||k)}}for(c=0;c<v.length;c++)if(g=v[c]){var L=g.constructUpdate(),R=Object.keys(L);for(u=0;u<R.length;u++)h=R[u],(y[h]=y[h]||[])[B[c]]=L[h]}for(d=Object.keys(y),c=0;c<d.length;c++)for(h=d[c],u=0;u<E.length;u++)y[h].hasOwnProperty(u)||(y[h][u]=void 0);i.call("_guiRestyle",e,y,E)}}}function N(t,e,n){var r=E.indexOf(t),i=y[e];return i||(i=y[e]=[]),-1===E.indexOf(t)&&(E.push(t),r=E.length-1),i[r]=n,r}function F(t,e){var n=t._fullInput;if(i.hasTransform(n,"groupby")){var o=v[n.index];if(!o){var a=i.getTransformIndices(n,"groupby"),s=a[a.length-1];o=r.keyedContainer(n,"transforms["+s+"].styles","target","value.visible"),v[n.index]=o}var A=o.get(t._group);void 0===A&&(A=!0),!1!==A&&o.set(t._group,e),B[n.index]=N(n.index,"visible",!1!==n.visible)}else{var l=!1!==n.visible&&e;N(n.index,"visible",l)}}}},{"../../lib":719,"../../registry":847}],648:[function(t,e,n){"use strict";n.isGrouped=function(t){return-1!==(t.traceorder||"").indexOf("grouped")},n.isVertical=function(t){return"h"!==t.orientation},n.isReversed=function(t){return-1!==(t.traceorder||"").indexOf("reversed")}},{}],649:[function(t,e,n){"use strict";e.exports={moduleType:"component",name:"legend",layoutAttributes:t("./attributes"),supplyLayoutDefaults:t("./defaults"),draw:t("./draw"),style:t("./style")}},{"./attributes":642,"./defaults":644,"./draw":645,"./style":650}],650:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../registry"),o=t("../../lib"),a=t("../drawing"),s=t("../color"),A=t("../../traces/scatter/subtypes"),l=t("../../traces/pie/style_one"),c=t("../../traces/pie/helpers").castOption,u=2,g=5;e.exports=function(t,e){var n=e._fullLayout.legend,h="constant"===n.itemsizing;function d(t,e,n,r){var i;if(t+1)i=t;else{if(!(e&&e.width>0))return 0;i=e.width}return h?r:Math.min(i,n)}function f(t,e,n){var o=t[0].trace,a=o.marker||{},A=a.line||{},l=n?o.type===n&&o.visible:i.traceIs(o,"bar"),c=r.select(e).select("g.legendpoints").selectAll("path.legend"+n).data(l?[t]:[]);c.enter().append("path").classed("legend"+n,!0).attr("d","M6,6H-6V-6H6Z").attr("transform","translate(20,0)"),c.exit().remove(),c.each(function(t){var e=r.select(this),n=t[0],i=d(n.mlw,a.line,g,u);e.style("stroke-width",i+"px").call(s.fill,n.mc||a.color),i&&s.stroke(e,n.mlc||A.color)})}function p(t,e,n){var a=t[0],s=a.trace,A=n?s.type===n&&s.visible:i.traceIs(s,n),h=r.select(e).select("g.legendpoints").selectAll("path.legend"+n).data(A?[t]:[]);if(h.enter().append("path").classed("legend"+n,!0).attr("d","M6,6H-6V-6H6Z").attr("transform","translate(20,0)"),h.exit().remove(),h.size()){var f=(s.marker||{}).line,p=d(c(f.width,a.pts),f,g,u),I=o.minExtend(s,{marker:{line:{width:p}}});I.marker.line.color=f.color;var C=o.minExtend(a,{trace:I});l(h,C,I)}}t.each(function(t){var e=r.select(this),i=o.ensureSingle(e,"g","layers");i.style("opacity",t[0].trace.opacity);var a=n.valign,s=t[0].lineHeight,A=t[0].height;if("middle"!==a&&s&&A){var l={top:1,bottom:-1}[a]*(.5*(s-A+3));i.attr("transform","translate(0,"+l+")")}else i.attr("transform",null);i.selectAll("g.legendfill").data([t]).enter().append("g").classed("legendfill",!0),i.selectAll("g.legendlines").data([t]).enter().append("g").classed("legendlines",!0);var c=i.selectAll("g.legendsymbols").data([t]);c.enter().append("g").classed("legendsymbols",!0),c.selectAll("g.legendpoints").data([t]).enter().append("g").classed("legendpoints",!0)}).each(function(t){var e=t[0].trace,n=[];"waterfall"===e.type&&e.visible&&(n=t[0].hasTotals?[["increasing","M-6,-6V6H0Z"],["totals","M6,6H0L-6,-6H-0Z"],["decreasing","M6,6V-6H0Z"]]:[["increasing","M-6,-6V6H6Z"],["decreasing","M6,6V-6H-6Z"]]);var i=r.select(this).select("g.legendpoints").selectAll("path.legendwaterfall").data(n);i.enter().append("path").classed("legendwaterfall",!0).attr("transform","translate(20,0)").style("stroke-miterlimit",1),i.exit().remove(),i.each(function(t){var n=r.select(this),i=e[t[0]].marker,o=d(void 0,i.line,g,u);n.attr("d",t[1]).style("stroke-width",o+"px").call(s.fill,i.color),o&&n.call(s.stroke,i.line.color)})}).each(function(t){f(t,this,"funnel")}).each(function(t){f(t,this)}).each(function(t){var n=t[0].trace,A=r.select(this).select("g.legendpoints").selectAll("path.legendbox").data(i.traceIs(n,"box-violin")&&n.visible?[t]:[]);A.enter().append("path").classed("legendbox",!0).attr("d","M6,6H-6V-6H6Z").attr("transform","translate(20,0)"),A.exit().remove(),A.each(function(){var t=r.select(this);if("all"!==n.boxpoints&&"all"!==n.points||0!==s.opacity(n.fillcolor)||0!==s.opacity((n.line||{}).color)){var i=d(void 0,n.line,g,u);t.style("stroke-width",i+"px").call(s.fill,n.fillcolor),i&&s.stroke(t,n.line.color)}else{var l=o.minExtend(n,{marker:{size:h?12:o.constrain(n.marker.size,2,16),sizeref:1,sizemin:1,sizemode:"diameter"}});A.call(a.pointStyle,l,e)}})}).each(function(t){p(t,this,"funnelarea")}).each(function(t){p(t,this,"pie")}).each(function(t){var n,i,s=t[0],l=s.trace,c=l.visible&&l.fill&&"none"!==l.fill,u=A.hasLines(l),g=l.contours,h=!1,f=!1;if(g){var p=g.coloring;"lines"===p?h=!0:u="none"===p||"heatmap"===p||g.showlines,"constraint"===g.type?c="="!==g._operation:"fill"!==p&&"heatmap"!==p||(f=!0)}var I=A.hasMarkers(l)||A.hasText(l),C=c||f,m=u||h,y=I||!C?"M5,0":m?"M5,-2":"M5,-3",E=r.select(this),v=E.select(".legendfill").selectAll("path").data(c||f?[t]:[]);if(v.enter().append("path").classed("js-fill",!0),v.exit().remove(),v.attr("d",y+"h30v6h-30z").call(c?a.fillGroupStyle:function(t){if(t.size()){var n="legendfill-"+l.uid;a.gradient(t,e,n,"horizontalreversed",l.colorscale,"fill")}}),u||h){var B=d(void 0,l.line,10,5);i=o.minExtend(l,{line:{width:B}}),n=[o.minExtend(s,{trace:i})]}var b=E.select(".legendlines").selectAll("path").data(u||h?[n]:[]);b.enter().append("path").classed("js-line",!0),b.exit().remove(),b.attr("d",y+(h?"l30,0.0001":"h30")).call(u?a.lineGroupStyle:function(t){if(t.size()){var n="legendline-"+l.uid;a.lineGroupStyle(t),a.gradient(t,e,n,"horizontalreversed",l.colorscale,"stroke")}})}).each(function(t){var n,i,s=t[0],l=s.trace,c=A.hasMarkers(l),g=A.hasText(l),d=A.hasLines(l);function f(t,e,n,r){var i=o.nestedProperty(l,t).get(),a=o.isArrayOrTypedArray(i)&&e?e(i):i;if(h&&a&&void 0!==r&&(a=r),n){if(a<n[0])return n[0];if(a>n[1])return n[1]}return a}function p(t){return t[0]}if(c||g||d){var I={},C={};if(c){I.mc=f("marker.color",p),I.mx=f("marker.symbol",p),I.mo=f("marker.opacity",o.mean,[.2,1]),I.mlc=f("marker.line.color",p),I.mlw=f("marker.line.width",o.mean,[0,5],u),C.marker={sizeref:1,sizemin:1,sizemode:"diameter"};var m=f("marker.size",o.mean,[2,16],12);I.ms=m,C.marker.size=m}d&&(C.line={width:f("line.width",p,[0,10],5)}),g&&(I.tx="Aa",I.tp=f("textposition",p),I.ts=10,I.tc=f("textfont.color",p),I.tf=f("textfont.family",p)),n=[o.minExtend(s,I)],(i=o.minExtend(l,C)).selectedpoints=null}var y=r.select(this).select("g.legendpoints"),E=y.selectAll("path.scatterpts").data(c?n:[]);E.enter().insert("path",":first-child").classed("scatterpts",!0).attr("transform","translate(20,0)"),E.exit().remove(),E.call(a.pointStyle,i,e),c&&(n[0].mrc=3);var v=y.selectAll("g.pointtext").data(g?n:[]);v.enter().append("g").classed("pointtext",!0).append("text").attr("transform","translate(20,0)"),v.exit().remove(),v.selectAll("text").call(a.textPointStyle,i,e)}).each(function(t){var e=t[0].trace,n=r.select(this).select("g.legendpoints").selectAll("path.legendcandle").data("candlestick"===e.type&&e.visible?[t,t]:[]);n.enter().append("path").classed("legendcandle",!0).attr("d",function(t,e){return e?"M-15,0H-8M-8,6V-6H8Z":"M15,0H8M8,-6V6H-8Z"}).attr("transform","translate(20,0)").style("stroke-miterlimit",1),n.exit().remove(),n.each(function(t,n){var i=r.select(this),o=e[n?"increasing":"decreasing"],a=d(void 0,o.line,g,u);i.style("stroke-width",a+"px").call(s.fill,o.fillcolor),a&&s.stroke(i,o.line.color)})}).each(function(t){var e=t[0].trace,n=r.select(this).select("g.legendpoints").selectAll("path.legendohlc").data("ohlc"===e.type&&e.visible?[t,t]:[]);n.enter().append("path").classed("legendohlc",!0).attr("d",function(t,e){return e?"M-15,0H0M-8,-6V0":"M15,0H0M8,6V0"}).attr("transform","translate(20,0)").style("stroke-miterlimit",1),n.exit().remove(),n.each(function(t,n){var i=r.select(this),o=e[n?"increasing":"decreasing"],A=d(void 0,o.line,g,u);i.style("fill","none").call(a.dashLine,o.line.dash,A),A&&s.stroke(i,o.line.color)})})}},{"../../lib":719,"../../registry":847,"../../traces/pie/helpers":1091,"../../traces/pie/style_one":1097,"../../traces/scatter/subtypes":1135,"../color":593,"../drawing":614,d3:163}],651:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../plots/plots"),o=t("../../plots/cartesian/axis_ids"),a=t("../../lib"),s=t("../../fonts/ploticon"),A=a._,l=e.exports={};function c(t,e){var n,i,a=e.currentTarget,s=a.getAttribute("data-attr"),A=a.getAttribute("data-val")||!0,l=t._fullLayout,c={},u=o.list(t,null,!0),g="on";if("zoom"===s){var h,d="in"===A?.5:2,f=(1+d)/2,p=(1-d)/2;for(i=0;i<u.length;i++)if(!(n=u[i]).fixedrange)if(h=n._name,"auto"===A)c[h+".autorange"]=!0;else if("reset"===A){if(void 0===n._rangeInitial)c[h+".autorange"]=!0;else{var I=n._rangeInitial.slice();c[h+".range[0]"]=I[0],c[h+".range[1]"]=I[1]}void 0!==n._showSpikeInitial&&(c[h+".showspikes"]=n._showSpikeInitial,"on"!==g||n._showSpikeInitial||(g="off"))}else{var C=[n.r2l(n.range[0]),n.r2l(n.range[1])],m=[f*C[0]+p*C[1],f*C[1]+p*C[0]];c[h+".range[0]"]=n.l2r(m[0]),c[h+".range[1]"]=n.l2r(m[1])}l._cartesianSpikesEnabled=g}else{if("hovermode"!==s||"x"!==A&&"y"!==A){if("hovermode"===s&&"closest"===A){for(i=0;i<u.length;i++)n=u[i],"on"!==g||n.showspikes||(g="off");l._cartesianSpikesEnabled=g}}else A=l._isHoriz?"y":"x",a.setAttribute("data-val",A);c[s]=A}r.call("_guiRelayout",t,c)}function u(t,e){for(var n=e.currentTarget,i=n.getAttribute("data-attr"),o=n.getAttribute("data-val")||!0,a=t._fullLayout._subplots.gl3d,s={},A=i.split("."),l=0;l<a.length;l++)s[a[l]+"."+A[1]]=o;var c="pan"===o?o:"zoom";s.dragmode=c,r.call("_guiRelayout",t,s)}function g(t,e){for(var n=e.currentTarget.getAttribute("data-attr"),i=t._fullLayout,o=i._subplots.gl3d,a={},s=0;s<o.length;s++){var A=o[s],l=A+".camera",c=i[A]._scene;"resetLastSave"===n?(a[l+".up"]=c.viewInitial.up,a[l+".eye"]=c.viewInitial.eye,a[l+".center"]=c.viewInitial.center):"resetDefault"===n&&(a[l+".up"]=null,a[l+".eye"]=null,a[l+".center"]=null)}r.call("_guiRelayout",t,a)}function h(t,e){var n=e.currentTarget,r=n._previousVal,i=t._fullLayout,o=i._subplots.gl3d,a=["xaxis","yaxis","zaxis"],s={},A={};if(r)A=r,n._previousVal=null;else{for(var l=0;l<o.length;l++){var c=o[l],u=i[c],g=c+".hovermode";s[g]=u.hovermode,A[g]=!1;for(var h=0;h<3;h++){var d=a[h],f=c+"."+d+".showspikes";A[f]=!1,s[f]=u[d].showspikes}}n._previousVal=s}return A}function d(t,e){for(var n=e.currentTarget,i=n.getAttribute("data-attr"),o=n.getAttribute("data-val")||!0,a=t._fullLayout,s=a._subplots.geo,A=0;A<s.length;A++){var l=s[A],c=a[l];if("zoom"===i){var u=c.projection.scale,g="in"===o?2*u:.5*u;r.call("_guiRelayout",t,l+".projection.scale",g)}else"reset"===i&&I(t,"geo")}}function f(t){var e=t._fullLayout;return!e.hovermode&&(e._has("cartesian")?e._isHoriz?"y":"x":"closest")}function p(t){var e=f(t);r.call("_guiRelayout",t,"hovermode",e)}function I(t,e){for(var n=t._fullLayout,i=n._subplots[e],o={},a=0;a<i.length;a++)for(var s=i[a],A=n[s]._subplot.viewInitial,l=Object.keys(A),c=0;c<l.length;c++){var u=l[c];o[s+"."+u]=A[u]}r.call("_guiRelayout",t,o)}l.toImage={name:"toImage",title:function(t){var e=(t._context.toImageButtonOptions||{}).format||"png";return A(t,"png"===e?"Download plot as a png":"Download plot")},icon:s.camera,click:function(t){var e=t._context.toImageButtonOptions,n={format:e.format||"png"};a.notifier(A(t,"Taking snapshot - this may take a few seconds"),"long"),"svg"!==n.format&&a.isIE()&&(a.notifier(A(t,"IE only supports svg.  Changing format to svg."),"long"),n.format="svg"),["filename","width","height","scale"].forEach(function(t){t in e&&(n[t]=e[t])}),r.call("downloadImage",t,n).then(function(e){a.notifier(A(t,"Snapshot succeeded")+" - "+e,"long")}).catch(function(){a.notifier(A(t,"Sorry, there was a problem downloading your snapshot!"),"long")})}},l.sendDataToCloud={name:"sendDataToCloud",title:function(t){return A(t,"Edit in Chart Studio")},icon:s.disk,click:function(t){i.sendDataToCloud(t)}},l.editInChartStudio={name:"editInChartStudio",title:function(t){return A(t,"Edit in Chart Studio")},icon:s.pencil,click:function(t){i.sendDataToCloud(t)}},l.zoom2d={name:"zoom2d",title:function(t){return A(t,"Zoom")},attr:"dragmode",val:"zoom",icon:s.zoombox,click:c},l.pan2d={name:"pan2d",title:function(t){return A(t,"Pan")},attr:"dragmode",val:"pan",icon:s.pan,click:c},l.select2d={name:"select2d",title:function(t){return A(t,"Box Select")},attr:"dragmode",val:"select",icon:s.selectbox,click:c},l.lasso2d={name:"lasso2d",title:function(t){return A(t,"Lasso Select")},attr:"dragmode",val:"lasso",icon:s.lasso,click:c},l.zoomIn2d={name:"zoomIn2d",title:function(t){return A(t,"Zoom in")},attr:"zoom",val:"in",icon:s.zoom_plus,click:c},l.zoomOut2d={name:"zoomOut2d",title:function(t){return A(t,"Zoom out")},attr:"zoom",val:"out",icon:s.zoom_minus,click:c},l.autoScale2d={name:"autoScale2d",title:function(t){return A(t,"Autoscale")},attr:"zoom",val:"auto",icon:s.autoscale,click:c},l.resetScale2d={name:"resetScale2d",title:function(t){return A(t,"Reset axes")},attr:"zoom",val:"reset",icon:s.home,click:c},l.hoverClosestCartesian={name:"hoverClosestCartesian",title:function(t){return A(t,"Show closest data on hover")},attr:"hovermode",val:"closest",icon:s.tooltip_basic,gravity:"ne",click:c},l.hoverCompareCartesian={name:"hoverCompareCartesian",title:function(t){return A(t,"Compare data on hover")},attr:"hovermode",val:function(t){return t._fullLayout._isHoriz?"y":"x"},icon:s.tooltip_compare,gravity:"ne",click:c},l.zoom3d={name:"zoom3d",title:function(t){return A(t,"Zoom")},attr:"scene.dragmode",val:"zoom",icon:s.zoombox,click:u},l.pan3d={name:"pan3d",title:function(t){return A(t,"Pan")},attr:"scene.dragmode",val:"pan",icon:s.pan,click:u},l.orbitRotation={name:"orbitRotation",title:function(t){return A(t,"Orbital rotation")},attr:"scene.dragmode",val:"orbit",icon:s["3d_rotate"],click:u},l.tableRotation={name:"tableRotation",title:function(t){return A(t,"Turntable rotation")},attr:"scene.dragmode",val:"turntable",icon:s["z-axis"],click:u},l.resetCameraDefault3d={name:"resetCameraDefault3d",title:function(t){return A(t,"Reset camera to default")},attr:"resetDefault",icon:s.home,click:g},l.resetCameraLastSave3d={name:"resetCameraLastSave3d",title:function(t){return A(t,"Reset camera to last save")},attr:"resetLastSave",icon:s.movie,click:g},l.hoverClosest3d={name:"hoverClosest3d",title:function(t){return A(t,"Toggle show closest data on hover")},attr:"hovermode",val:null,toggle:!0,icon:s.tooltip_basic,gravity:"ne",click:function(t,e){var n=h(t,e);r.call("_guiRelayout",t,n)}},l.zoomInGeo={name:"zoomInGeo",title:function(t){return A(t,"Zoom in")},attr:"zoom",val:"in",icon:s.zoom_plus,click:d},l.zoomOutGeo={name:"zoomOutGeo",title:function(t){return A(t,"Zoom out")},attr:"zoom",val:"out",icon:s.zoom_minus,click:d},l.resetGeo={name:"resetGeo",title:function(t){return A(t,"Reset")},attr:"reset",val:null,icon:s.autoscale,click:d},l.hoverClosestGeo={name:"hoverClosestGeo",title:function(t){return A(t,"Toggle show closest data on hover")},attr:"hovermode",val:null,toggle:!0,icon:s.tooltip_basic,gravity:"ne",click:p},l.hoverClosestGl2d={name:"hoverClosestGl2d",title:function(t){return A(t,"Toggle show closest data on hover")},attr:"hovermode",val:null,toggle:!0,icon:s.tooltip_basic,gravity:"ne",click:p},l.hoverClosestPie={name:"hoverClosestPie",title:function(t){return A(t,"Toggle show closest data on hover")},attr:"hovermode",val:"closest",icon:s.tooltip_basic,gravity:"ne",click:p},l.resetViewSankey={name:"resetSankeyGroup",title:function(t){return A(t,"Reset view")},icon:s.home,click:function(t){for(var e={"node.groups":[],"node.x":[],"node.y":[]},n=0;n<t._fullData.length;n++){var i=t._fullData[n]._viewInitial;e["node.groups"].push(i.node.groups.slice()),e["node.x"].push(i.node.x.slice()),e["node.y"].push(i.node.y.slice())}r.call("restyle",t,e)}},l.toggleHover={name:"toggleHover",title:function(t){return A(t,"Toggle show closest data on hover")},attr:"hovermode",val:null,toggle:!0,icon:s.tooltip_basic,gravity:"ne",click:function(t,e){var n=h(t,e);n.hovermode=f(t),r.call("_guiRelayout",t,n)}},l.resetViews={name:"resetViews",title:function(t){return A(t,"Reset views")},icon:s.home,click:function(t,e){var n=e.currentTarget;n.setAttribute("data-attr","zoom"),n.setAttribute("data-val","reset"),c(t,e),n.setAttribute("data-attr","resetLastSave"),g(t,e),I(t,"geo"),I(t,"mapbox")}},l.toggleSpikelines={name:"toggleSpikelines",title:function(t){return A(t,"Toggle Spike Lines")},icon:s.spikeline,attr:"_cartesianSpikesEnabled",val:"on",click:function(t){var e=t._fullLayout;e._cartesianSpikesEnabled="on"===e._cartesianSpikesEnabled?"off":"on";var n=function(t){for(var e,n,r=t._fullLayout,i=o.list(t,null,!0),a={},s=0;s<i.length;s++)e=i[s],n=e._name,a[n+".showspikes"]="on"===r._cartesianSpikesEnabled||e._showSpikeInitial;return a}(t);r.call("_guiRelayout",t,n)}},l.resetViewMapbox={name:"resetViewMapbox",title:function(t){return A(t,"Reset view")},attr:"reset",icon:s.home,click:function(t){I(t,"mapbox")}}},{"../../fonts/ploticon":699,"../../lib":719,"../../plots/cartesian/axis_ids":770,"../../plots/plots":828,"../../registry":847}],652:[function(t,e,n){"use strict";n.manage=t("./manage")},{"./manage":653}],653:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axis_ids"),i=t("../../traces/scatter/subtypes"),o=t("../../registry"),a=t("./modebar"),s=t("./buttons");e.exports=function(t){var e=t._fullLayout,n=t._context,A=e._modeBar;if(n.displayModeBar||n.watermark){if(!Array.isArray(n.modeBarButtonsToRemove))throw new Error(["*modeBarButtonsToRemove* configuration options","must be an array."].join(" "));if(!Array.isArray(n.modeBarButtonsToAdd))throw new Error(["*modeBarButtonsToAdd* configuration options","must be an array."].join(" "));var l,c=n.modeBarButtons;l=Array.isArray(c)&&c.length?function(t){for(var e=0;e<t.length;e++)for(var n=t[e],r=0;r<n.length;r++){var i=n[r];if("string"==typeof i){if(void 0===s[i])throw new Error(["*modeBarButtons* configuration options","invalid button name"].join(" "));t[e][r]=s[i]}}return t}(c):!n.displayModeBar&&n.watermark?[]:function(t){var e=t._fullLayout,n=t._fullData,a=t._context,A=a.modeBarButtonsToRemove,l=a.modeBarButtonsToAdd,c=e._has("cartesian"),u=e._has("gl3d"),g=e._has("geo"),h=e._has("pie"),d=e._has("funnelarea"),f=e._has("gl2d"),p=e._has("ternary"),I=e._has("mapbox"),C=e._has("polar"),m=e._has("sankey"),y=function(t){for(var e=r.list({_fullLayout:t},null,!0),n=0;n<e.length;n++)if(!e[n].fixedrange)return!1;return!0}(e),E=[];function v(t){if(t.length){for(var e=[],n=0;n<t.length;n++){var r=t[n];-1===A.indexOf(r)&&e.push(s[r])}E.push(e)}}var B=["toImage"];a.showEditInChartStudio?B.push("editInChartStudio"):a.showSendToCloud&&B.push("sendDataToCloud"),v(B);var b=[],Q=[],w=[],x=[];return(c||f||h||d||p)+g+u+I+C>1?(Q=["toggleHover"],w=["resetViews"]):g?(b=["zoomInGeo","zoomOutGeo"],Q=["hoverClosestGeo"],w=["resetGeo"]):u?(Q=["hoverClosest3d"],w=["resetCameraDefault3d","resetCameraLastSave3d"]):I?(Q=["toggleHover"],w=["resetViewMapbox"]):f?Q=["hoverClosestGl2d"]:h?Q=["hoverClosestPie"]:m?(Q=["hoverClosestCartesian","hoverCompareCartesian"],w=["resetViewSankey"]):Q=["toggleHover"],c&&(Q=["toggleSpikelines","hoverClosestCartesian","hoverCompareCartesian"]),function(t){for(var e=0;e<t.length;e++)if(!o.traceIs(t[e],"noHover"))return!1;return!0}(n)&&(Q=[]),!c&&!f||y||(b=["zoomIn2d","zoomOut2d","autoScale2d"],"resetViews"!==w[0]&&(w=["resetScale2d"])),u?x=["zoom3d","pan3d","orbitRotation","tableRotation"]:(c||f)&&!y||p?x=["zoom2d","pan2d"]:I||g?x=["pan2d"]:C&&(x=["zoom2d"]),function(t){for(var e=!1,n=0;n<t.length&&!e;n++){var r=t[n];r._module&&r._module.selectPoints&&(o.traceIs(r,"scatter-like")?(i.hasMarkers(r)||i.hasText(r))&&(e=!0):o.traceIs(r,"box-violin")&&"all"!==r.boxpoints&&"all"!==r.points||(e=!0))}return e}(n)&&x.push("select2d","lasso2d"),v(x),v(b.concat(w)),v(Q),function(t,e){if(e.length)if(Array.isArray(e[0]))for(var n=0;n<e.length;n++)t.push(e[n]);else t.push(e);return t}(E,l)}(t),A?A.update(t,l):e._modeBar=a(t,l)}else A&&(A.destroy(),delete e._modeBar)}},{"../../plots/cartesian/axis_ids":770,"../../registry":847,"../../traces/scatter/subtypes":1135,"./buttons":651,"./modebar":654}],654:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=t("../../lib"),a=t("../../fonts/ploticon"),s=new DOMParser;function A(t){this.container=t.container,this.element=document.createElement("div"),this.update(t.graphInfo,t.buttons),this.container.appendChild(this.element)}var l=A.prototype;l.update=function(t,e){this.graphInfo=t;var n=this.graphInfo._context,r=this.graphInfo._fullLayout,i="modebar-"+r._uid;this.element.setAttribute("id",i),this._uid=i,this.element.className="modebar","hover"===n.displayModeBar&&(this.element.className+=" modebar--hover ease-bg"),"v"===r.modebar.orientation&&(this.element.className+=" vertical",e=e.reverse());var a=r.modebar,s="hover"===n.displayModeBar?".js-plotly-plot .plotly:hover ":"";o.deleteRelatedStyleRule(i),o.addRelatedStyleRule(i,s+"#"+i+" .modebar-group","background-color: "+a.bgcolor),o.addRelatedStyleRule(i,"#"+i+" .modebar-btn .icon path","fill: "+a.color),o.addRelatedStyleRule(i,"#"+i+" .modebar-btn:hover .icon path","fill: "+a.activecolor),o.addRelatedStyleRule(i,"#"+i+" .modebar-btn.active .icon path","fill: "+a.activecolor);var A=!this.hasButtons(e),l=this.hasLogo!==n.displaylogo,c=this.locale!==n.locale;if(this.locale=n.locale,(A||l||c)&&(this.removeAllButtons(),this.updateButtons(e),n.watermark||n.displaylogo)){var u=this.getLogo();n.watermark&&(u.className=u.className+" watermark"),"v"===r.modebar.orientation?this.element.insertBefore(u,this.element.childNodes[0]):this.element.appendChild(u),this.hasLogo=!0}this.updateActiveButton()},l.updateButtons=function(t){var e=this;this.buttons=t,this.buttonElements=[],this.buttonsNames=[],this.buttons.forEach(function(t){var n=e.createGroup();t.forEach(function(t){var r=t.name;if(!r)throw new Error("must provide button 'name' in button config");if(-1!==e.buttonsNames.indexOf(r))throw new Error("button name '"+r+"' is taken");e.buttonsNames.push(r);var i=e.createButton(t);e.buttonElements.push(i),n.appendChild(i)}),e.element.appendChild(n)})},l.createGroup=function(){var t=document.createElement("div");return t.className="modebar-group",t},l.createButton=function(t){var e=this,n=document.createElement("a");n.setAttribute("rel","tooltip"),n.className="modebar-btn";var i=t.title;void 0===i?i=t.name:"function"==typeof i&&(i=i(this.graphInfo)),(i||0===i)&&n.setAttribute("data-title",i),void 0!==t.attr&&n.setAttribute("data-attr",t.attr);var o=t.val;if(void 0!==o&&("function"==typeof o&&(o=o(this.graphInfo)),n.setAttribute("data-val",o)),"function"!=typeof t.click)throw new Error("must provide button 'click' function in button config");n.addEventListener("click",function(n){t.click(e.graphInfo,n),e.updateActiveButton(n.currentTarget)}),n.setAttribute("data-toggle",t.toggle||!1),t.toggle&&r.select(n).classed("active",!0);var s=t.icon;return"function"==typeof s?n.appendChild(s()):n.appendChild(this.createIcon(s||a.question)),n.setAttribute("data-gravity",t.gravity||"n"),n},l.createIcon=function(t){var e,n=i(t.height)?Number(t.height):t.ascent-t.descent,r="http://www.w3.org/2000/svg";if(t.path){(e=document.createElementNS(r,"svg")).setAttribute("viewBox",[0,0,t.width,n].join(" ")),e.setAttribute("class","icon");var o=document.createElementNS(r,"path");o.setAttribute("d",t.path),t.transform?o.setAttribute("transform",t.transform):void 0!==t.ascent&&o.setAttribute("transform","matrix(1 0 0 -1 0 "+t.ascent+")"),e.appendChild(o)}return t.svg&&(e=s.parseFromString(t.svg,"application/xml").childNodes[0]),e.setAttribute("height","1em"),e.setAttribute("width","1em"),e},l.updateActiveButton=function(t){var e=this.graphInfo._fullLayout,n=void 0!==t?t.getAttribute("data-attr"):null;this.buttonElements.forEach(function(t){var i=t.getAttribute("data-val")||!0,a=t.getAttribute("data-attr"),s="true"===t.getAttribute("data-toggle"),A=r.select(t);if(s)a===n&&A.classed("active",!A.classed("active"));else{var l=null===a?a:o.nestedProperty(e,a).get();A.classed("active",l===i)}})},l.hasButtons=function(t){var e=this.buttons;if(!e)return!1;if(t.length!==e.length)return!1;for(var n=0;n<t.length;++n){if(t[n].length!==e[n].length)return!1;for(var r=0;r<t[n].length;r++)if(t[n][r].name!==e[n][r].name)return!1}return!0},l.getLogo=function(){var t=this.createGroup(),e=document.createElement("a");return e.href="https://plot.ly/",e.target="_blank",e.setAttribute("data-title",o._(this.graphInfo,"Produced with Plotly")),e.className="modebar-btn plotlyjsicon modebar-btn--logo",e.appendChild(this.createIcon(a.newplotlylogo)),t.appendChild(e),t},l.removeAllButtons=function(){for(;this.element.firstChild;)this.element.removeChild(this.element.firstChild);this.hasLogo=!1},l.destroy=function(){o.removeElement(this.container.querySelector(".modebar")),o.deleteRelatedStyleRule(this._uid)},e.exports=function(t,e){var n=t._fullLayout,i=new A({graphInfo:t,container:n._modebardiv.node(),buttons:e});return n._privateplot&&r.select(i.element).append("span").classed("badge-private float--left",!0).text("PRIVATE"),i}},{"../../fonts/ploticon":699,"../../lib":719,d3:163,"fast-isnumeric":225}],655:[function(t,e,n){"use strict";var r=t("../../plots/font_attributes"),i=t("../color/attributes"),o=(0,t("../../plot_api/plot_template").templatedArray)("button",{visible:{valType:"boolean",dflt:!0,editType:"plot"},step:{valType:"enumerated",values:["month","year","day","hour","minute","second","all"],dflt:"month",editType:"plot"},stepmode:{valType:"enumerated",values:["backward","todate"],dflt:"backward",editType:"plot"},count:{valType:"number",min:0,dflt:1,editType:"plot"},label:{valType:"string",editType:"plot"},editType:"plot"});e.exports={visible:{valType:"boolean",editType:"plot"},buttons:o,x:{valType:"number",min:-2,max:3,editType:"plot"},xanchor:{valType:"enumerated",values:["auto","left","center","right"],dflt:"left",editType:"plot"},y:{valType:"number",min:-2,max:3,editType:"plot"},yanchor:{valType:"enumerated",values:["auto","top","middle","bottom"],dflt:"bottom",editType:"plot"},font:r({editType:"plot"}),bgcolor:{valType:"color",dflt:i.lightLine,editType:"plot"},activecolor:{valType:"color",editType:"plot"},bordercolor:{valType:"color",dflt:i.defaultLine,editType:"plot"},borderwidth:{valType:"number",min:0,dflt:0,editType:"plot"},editType:"plot"}},{"../../plot_api/plot_template":757,"../../plots/font_attributes":793,"../color/attributes":592}],656:[function(t,e,n){"use strict";e.exports={yPad:.02,minButtonWidth:30,rx:3,ry:3,lightAmount:25,darkAmount:10}},{}],657:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../color"),o=t("../../plot_api/plot_template"),a=t("../../plots/array_container_defaults"),s=t("./attributes"),A=t("./constants");function l(t,e,n,i){var o=i.calendar;function a(n,i){return r.coerce(t,e,s.buttons,n,i)}if(a("visible")){var A=a("step");"all"!==A&&(!o||"gregorian"===o||"month"!==A&&"year"!==A?a("stepmode"):e.stepmode="backward",a("count")),a("label")}}e.exports=function(t,e,n,c,u){var g=t.rangeselector||{},h=o.newContainer(e,"rangeselector");function d(t,e){return r.coerce(g,h,s,t,e)}if(d("visible",a(g,h,{name:"buttons",handleItemDefaults:l,calendar:u}).length>0)){var f=function(t,e,n){for(var r=n.filter(function(n){return e[n].anchor===t._id}),i=0,o=0;o<r.length;o++){var a=e[r[o]].domain;a&&(i=Math.max(a[1],i))}return[t.domain[0],i+A.yPad]}(e,n,c);d("x",f[0]),d("y",f[1]),r.noneOrAll(t,e,["x","y"]),d("xanchor"),d("yanchor"),r.coerceFont(d,"font",n.font);var p=d("bgcolor");d("activecolor",i.contrast(p,A.lightAmount,A.darkAmount)),d("bordercolor"),d("borderwidth")}}},{"../../lib":719,"../../plot_api/plot_template":757,"../../plots/array_container_defaults":763,"../color":593,"./attributes":655,"./constants":656}],658:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../registry"),o=t("../../plots/plots"),a=t("../color"),s=t("../drawing"),A=t("../../lib"),l=t("../../lib/svg_text_utils"),c=t("../../plots/cartesian/axis_ids"),u=t("../../constants/alignment"),g=u.LINE_SPACING,h=u.FROM_TL,d=u.FROM_BR,f=t("./constants"),p=t("./get_update_object");function I(t){return t._id}function C(t,e,n){var r=A.ensureSingle(t,"rect","selector-rect",function(t){t.attr("shape-rendering","crispEdges")});r.attr({rx:f.rx,ry:f.ry}),r.call(a.stroke,e.bordercolor).call(a.fill,function(t,e){return e._isActive||e._isHovered?t.activecolor:t.bgcolor}(e,n)).style("stroke-width",e.borderwidth+"px")}function m(t,e,n,r){A.ensureSingle(t,"text","selector-text",function(t){t.classed("user-select-none",!0).attr("text-anchor","middle")}).call(s.font,e.font).text(function(t,e){return t.label?e?A.templateString(t.label,e):t.label:"all"===t.step?"all":t.count+t.step.charAt(0)}(n,r._fullLayout._meta)).call(function(t){l.convertToTspans(t,r)})}e.exports=function(t){var e=t._fullLayout._infolayer.selectAll(".rangeselector").data(function(t){for(var e=c.list(t,"x",!0),n=[],r=0;r<e.length;r++){var i=e[r];i.rangeselector&&i.rangeselector.visible&&n.push(i)}return n}(t),I);e.enter().append("g").classed("rangeselector",!0),e.exit().remove(),e.style({cursor:"pointer","pointer-events":"all"}),e.each(function(e){var n=r.select(this),a=e,c=a.rangeselector,u=n.selectAll("g.button").data(A.filterVisible(c.buttons));u.enter().append("g").classed("button",!0),u.exit().remove(),u.each(function(e){var n=r.select(this),o=p(a,e);e._isActive=function(t,e,n){if("all"===e.step)return!0===t.autorange;var r=Object.keys(n);return t.range[0]===n[r[0]]&&t.range[1]===n[r[1]]}(a,e,o),n.call(C,c,e),n.call(m,c,e,t),n.on("click",function(){t._dragged||i.call("_guiRelayout",t,o)}),n.on("mouseover",function(){e._isHovered=!0,n.call(C,c,e)}),n.on("mouseout",function(){e._isHovered=!1,n.call(C,c,e)})}),function(t,e,n,i,a){var c=0,u=0,p=n.borderwidth;e.each(function(){var t=r.select(this),e=t.select(".selector-text"),i=n.font.size*g,o=Math.max(i*l.lineCount(e),16)+3;u=Math.max(u,o)}),e.each(function(){var t=r.select(this),e=t.select(".selector-rect"),i=t.select(".selector-text"),o=i.node()&&s.bBox(i.node()).width,a=n.font.size*g,A=l.lineCount(i),h=Math.max(o+10,f.minButtonWidth);t.attr("transform","translate("+(p+c)+","+p+")"),e.attr({x:0,y:0,width:h,height:u}),l.positionText(i,h/2,u/2-(A-1)*a/2+3),c+=h+5});var I=t._fullLayout._size,C=I.l+I.w*n.x,m=I.t+I.h*(1-n.y),y="left";A.isRightAnchor(n)&&(C-=c,y="right"),A.isCenterAnchor(n)&&(C-=c/2,y="center");var E="top";A.isBottomAnchor(n)&&(m-=u,E="bottom"),A.isMiddleAnchor(n)&&(m-=u/2,E="middle"),c=Math.ceil(c),u=Math.ceil(u),C=Math.round(C),m=Math.round(m),o.autoMargin(t,i+"-range-selector",{x:n.x,y:n.y,l:c*h[y],r:c*d[y],b:u*d[E],t:u*h[E]}),a.attr("transform","translate("+C+","+m+")")}(t,u,c,a._name,n)})}},{"../../constants/alignment":688,"../../lib":719,"../../lib/svg_text_utils":743,"../../plots/cartesian/axis_ids":770,"../../plots/plots":828,"../../registry":847,"../color":593,"../drawing":614,"./constants":656,"./get_update_object":659,d3:163}],659:[function(t,e,n){"use strict";var r=t("d3");e.exports=function(t,e){var n=t._name,i={};if("all"===e.step)i[n+".autorange"]=!0;else{var o=function(t,e){var n,i=t.range,o=new Date(t.r2l(i[1])),a=e.step,s=e.count;switch(e.stepmode){case"backward":n=t.l2r(+r.time[a].utc.offset(o,-s));break;case"todate":var A=r.time[a].utc.offset(o,-s);n=t.l2r(+r.time[a].utc.ceil(A))}var l=i[1];return[n,l]}(t,e);i[n+".range[0]"]=o[0],i[n+".range[1]"]=o[1]}return i}},{d3:163}],660:[function(t,e,n){"use strict";e.exports={moduleType:"component",name:"rangeselector",schema:{subplots:{xaxis:{rangeselector:t("./attributes")}}},layoutAttributes:t("./attributes"),handleDefaults:t("./defaults"),draw:t("./draw")}},{"./attributes":655,"./defaults":657,"./draw":658}],661:[function(t,e,n){"use strict";var r=t("../color/attributes");e.exports={bgcolor:{valType:"color",dflt:r.background,editType:"plot"},bordercolor:{valType:"color",dflt:r.defaultLine,editType:"plot"},borderwidth:{valType:"integer",dflt:0,min:0,editType:"plot"},autorange:{valType:"boolean",dflt:!0,editType:"calc",impliedEdits:{"range[0]":void 0,"range[1]":void 0}},range:{valType:"info_array",items:[{valType:"any",editType:"calc",impliedEdits:{"^autorange":!1}},{valType:"any",editType:"calc",impliedEdits:{"^autorange":!1}}],editType:"calc",impliedEdits:{autorange:!1}},thickness:{valType:"number",dflt:.15,min:0,max:1,editType:"plot"},visible:{valType:"boolean",dflt:!0,editType:"calc"},editType:"calc"}},{"../color/attributes":592}],662:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axis_ids").list,i=t("../../plots/cartesian/autorange").getAutoRange,o=t("./constants");e.exports=function(t){for(var e=r(t,"x",!0),n=0;n<e.length;n++){var a=e[n],s=a[o.name];s&&s.visible&&s.autorange&&(s._input.autorange=!0,s._input.range=s.range=i(t,a))}}},{"../../plots/cartesian/autorange":766,"../../plots/cartesian/axis_ids":770,"./constants":663}],663:[function(t,e,n){"use strict";e.exports={name:"rangeslider",containerClassName:"rangeslider-container",bgClassName:"rangeslider-bg",rangePlotClassName:"rangeslider-rangeplot",maskMinClassName:"rangeslider-mask-min",maskMaxClassName:"rangeslider-mask-max",slideBoxClassName:"rangeslider-slidebox",grabberMinClassName:"rangeslider-grabber-min",grabAreaMinClassName:"rangeslider-grabarea-min",handleMinClassName:"rangeslider-handle-min",grabberMaxClassName:"rangeslider-grabber-max",grabAreaMaxClassName:"rangeslider-grabarea-max",handleMaxClassName:"rangeslider-handle-max",maskMinOppAxisClassName:"rangeslider-mask-min-opp-axis",maskMaxOppAxisClassName:"rangeslider-mask-max-opp-axis",maskColor:"rgba(0,0,0,0.4)",maskOppAxisColor:"rgba(0,0,0,0.2)",slideBoxFill:"transparent",slideBoxCursor:"ew-resize",grabAreaFill:"transparent",grabAreaCursor:"col-resize",grabAreaWidth:10,handleWidth:4,handleRadius:1,handleStrokeWidth:1,extraPad:15}},{}],664:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plot_api/plot_template"),o=t("../../plots/cartesian/axis_ids"),a=t("./attributes"),s=t("./oppaxis_attributes");e.exports=function(t,e,n){var A=t[n],l=e[n];if(A.rangeslider||e._requestRangeslider[l._id]){r.isPlainObject(A.rangeslider)||(A.rangeslider={});var c,u,g=A.rangeslider,h=i.newContainer(l,"rangeslider");if(E("visible")){E("bgcolor",e.plot_bgcolor),E("bordercolor"),E("borderwidth"),E("thickness"),E("autorange",!l.isValidRange(g.range)),E("range");var d=e._subplots;if(d)for(var f=d.cartesian.filter(function(t){return t.substr(0,t.indexOf("y"))===o.name2id(n)}).map(function(t){return t.substr(t.indexOf("y"),t.length)}),p=r.simpleMap(f,o.id2name),I=0;I<p.length;I++){var C=p[I];c=g[C]||{},u=i.newContainer(h,C,"yaxis");var m,y=e[C];c.range&&y.isValidRange(c.range)&&(m="fixed"),"match"!==v("rangemode",m)&&v("range",y.range.slice())}h._input=g}}function E(t,e){return r.coerce(g,h,a,t,e)}function v(t,e){return r.coerce(c,u,s,t,e)}}},{"../../lib":719,"../../plot_api/plot_template":757,"../../plots/cartesian/axis_ids":770,"./attributes":661,"./oppaxis_attributes":668}],665:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../registry"),o=t("../../plots/plots"),a=t("../../lib"),s=t("../drawing"),A=t("../color"),l=t("../titles"),c=t("../../plots/cartesian"),u=t("../../plots/cartesian/axis_ids"),g=t("../dragelement"),h=t("../../lib/setcursor"),d=t("./constants");function f(t,e,n,r){var i=a.ensureSingle(t,"rect",d.bgClassName,function(t){t.attr({x:0,y:0,"shape-rendering":"crispEdges"})}),o=r.borderwidth%2==0?r.borderwidth:r.borderwidth-1,A=-r._offsetShift,l=s.crispRound(e,r.borderwidth);i.attr({width:r._width+o,height:r._height+o,transform:"translate("+A+","+A+")",fill:r.bgcolor,stroke:r.bordercolor,"stroke-width":l})}function p(t,e,n,r){var i=e._fullLayout;a.ensureSingleById(i._topdefs,"clipPath",r._clipId,function(t){t.append("rect").attr({x:0,y:0})}).select("rect").attr({width:r._width,height:r._height})}function I(t,e,n,i){var A,l=e.calcdata,g=t.selectAll("g."+d.rangePlotClassName).data(n._subplotsWith,a.identity);g.enter().append("g").attr("class",function(t){return d.rangePlotClassName+" "+t}).call(s.setClipUrl,i._clipId,e),g.order(),g.exit().remove(),g.each(function(t,a){var s=r.select(this),g=0===a,h=u.getFromId(e,t,"y"),d=h._name,f=i[d],p={data:[],layout:{xaxis:{type:n.type,domain:[0,1],range:i.range.slice(),calendar:n.calendar},width:i._width,height:i._height,margin:{t:0,b:0,l:0,r:0}},_context:e._context};p.layout[d]={type:h.type,domain:[0,1],range:"match"!==f.rangemode?f.range.slice():h.range.slice(),calendar:h.calendar},o.supplyDefaults(p);var I=p._fullLayout.xaxis,C=p._fullLayout[d];I.clearCalc(),I.setScale(),C.clearCalc(),C.setScale();var m={id:t,plotgroup:s,xaxis:I,yaxis:C,isRangePlot:!0};g?A=m:(m.mainplot="xy",m.mainplotinfo=A),c.rangePlot(e,m,function(t,e){for(var n=[],r=0;r<t.length;r++){var i=t[r],o=i[0].trace;o.xaxis+o.yaxis===e&&n.push(i)}return n}(l,t))})}function C(t,e,n,r,i){a.ensureSingle(t,"rect",d.maskMinClassName,function(t){t.attr({x:0,y:0,"shape-rendering":"crispEdges"})}).attr("height",r._height).call(A.fill,d.maskColor),a.ensureSingle(t,"rect",d.maskMaxClassName,function(t){t.attr({y:0,"shape-rendering":"crispEdges"})}).attr("height",r._height).call(A.fill,d.maskColor),"match"!==i.rangemode&&(a.ensureSingle(t,"rect",d.maskMinOppAxisClassName,function(t){t.attr({y:0,"shape-rendering":"crispEdges"})}).attr("width",r._width).call(A.fill,d.maskOppAxisColor),a.ensureSingle(t,"rect",d.maskMaxOppAxisClassName,function(t){t.attr({y:0,"shape-rendering":"crispEdges"})}).attr("width",r._width).style("border-top",d.maskOppBorder).call(A.fill,d.maskOppAxisColor))}function m(t,e,n,r){e._context.staticPlot||a.ensureSingle(t,"rect",d.slideBoxClassName,function(t){t.attr({y:0,cursor:d.slideBoxCursor,"shape-rendering":"crispEdges"})}).attr({height:r._height,fill:d.slideBoxFill})}function y(t,e,n,r){var i=a.ensureSingle(t,"g",d.grabberMinClassName),o=a.ensureSingle(t,"g",d.grabberMaxClassName),s={x:0,width:d.handleWidth,rx:d.handleRadius,fill:A.background,stroke:A.defaultLine,"stroke-width":d.handleStrokeWidth,"shape-rendering":"crispEdges"},l={y:Math.round(r._height/4),height:Math.round(r._height/2)};if(a.ensureSingle(i,"rect",d.handleMinClassName,function(t){t.attr(s)}).attr(l),a.ensureSingle(o,"rect",d.handleMaxClassName,function(t){t.attr(s)}).attr(l),!e._context.staticPlot){var c={width:d.grabAreaWidth,x:0,y:0,fill:d.grabAreaFill,cursor:d.grabAreaCursor};a.ensureSingle(i,"rect",d.grabAreaMinClassName,function(t){t.attr(c)}).attr("height",r._height),a.ensureSingle(o,"rect",d.grabAreaMaxClassName,function(t){t.attr(c)}).attr("height",r._height)}}e.exports=function(t){for(var e=t._fullLayout,n=e._rangeSliderData,o=0;o<n.length;o++){var s=n[o][d.name];s._clipId=s._id+"-"+e._uid}var A=e._infolayer.selectAll("g."+d.containerClassName).data(n,function(t){return t._name});A.exit().each(function(t){var n=t[d.name];e._topdefs.select("#"+n._clipId).remove()}).remove(),0!==n.length&&(A.enter().append("g").classed(d.containerClassName,!0).attr("pointer-events","all"),A.each(function(n){var o=r.select(this),s=n[d.name],A=e[u.id2name(n.anchor)],c=s[u.id2name(n.anchor)];if(s.range){var E,v=a.simpleMap(s.range,n.r2l),B=a.simpleMap(n.range,n.r2l);E=B[0]<B[1]?[Math.min(v[0],B[0]),Math.max(v[1],B[1])]:[Math.max(v[0],B[0]),Math.min(v[1],B[1])],s.range=s._input.range=a.simpleMap(E,n.l2r)}n.cleanRange("rangeslider.range");var b=e.margin,Q=e._size,w=n.domain,x=s._tickHeight,_=s._oppBottom;s._width=Q.w*(w[1]-w[0]);var D=Math.round(b.l+Q.w*w[0]),M=Math.round(Q.t+Q.h*(1-_)+x+s._offsetShift+d.extraPad);o.attr("transform","translate("+D+","+M+")");var k=n.r2l(s.range[0]),S=n.r2l(s.range[1]),L=S-k;if(s.p2d=function(t){return t/s._width*L+k},s.d2p=function(t){return(t-k)/L*s._width},s._rl=[k,S],"match"!==c.rangemode){var R=A.r2l(c.range[0]),N=A.r2l(c.range[1])-R;s.d2pOppAxis=function(t){return(t-R)/N*s._height}}o.call(f,t,n,s).call(p,t,n,s).call(I,t,n,s).call(C,t,n,s,c).call(m,t,n,s).call(y,t,n,s),function(t,e,n,o){var s=t.select("rect."+d.slideBoxClassName).node(),A=t.select("rect."+d.grabAreaMinClassName).node(),l=t.select("rect."+d.grabAreaMaxClassName).node();t.on("mousedown",function(){var c=r.event,u=c.target,d=c.clientX,f=d-t.node().getBoundingClientRect().left,p=o.d2p(n._rl[0]),I=o.d2p(n._rl[1]),C=g.coverSlip();function m(t){var c,g,m,y=+t.clientX-d;switch(u){case s:m="ew-resize",c=p+y,g=I+y;break;case A:m="col-resize",c=p+y,g=I;break;case l:m="col-resize",c=p,g=I+y;break;default:m="ew-resize",c=f,g=f+y}if(g<c){var E=g;g=c,c=E}o._pixelMin=c,o._pixelMax=g,h(r.select(C),m),function(t,e,n,r){function o(t){return n.l2r(a.constrain(t,r._rl[0],r._rl[1]))}var s=o(r.p2d(r._pixelMin)),A=o(r.p2d(r._pixelMax));window.requestAnimationFrame(function(){i.call("_guiRelayout",e,n._name+".range",[s,A])})}(0,e,n,o)}C.addEventListener("mousemove",m),C.addEventListener("mouseup",function t(){C.removeEventListener("mousemove",m),C.removeEventListener("mouseup",t),a.removeElement(C)})})}(o,t,n,s),function(t,e,n,r,i,o){var s=d.handleWidth/2;function A(t){return a.constrain(t,0,r._width)}function l(t){return a.constrain(t,0,r._height)}function c(t){return a.constrain(t,-s,r._width+s)}var u=A(r.d2p(n._rl[0])),g=A(r.d2p(n._rl[1]));if(t.select("rect."+d.slideBoxClassName).attr("x",u).attr("width",g-u),t.select("rect."+d.maskMinClassName).attr("width",u),t.select("rect."+d.maskMaxClassName).attr("x",g).attr("width",r._width-g),"match"!==o.rangemode){var h=r._height-l(r.d2pOppAxis(i._rl[1])),f=r._height-l(r.d2pOppAxis(i._rl[0]));t.select("rect."+d.maskMinOppAxisClassName).attr("x",u).attr("height",h).attr("width",g-u),t.select("rect."+d.maskMaxOppAxisClassName).attr("x",u).attr("y",f).attr("height",r._height-f).attr("width",g-u),t.select("rect."+d.slideBoxClassName).attr("y",h).attr("height",f-h)}var p=Math.round(c(u-s))-.5,I=Math.round(c(g-s))+.5;t.select("g."+d.grabberMinClassName).attr("transform","translate("+p+",0.5)"),t.select("g."+d.grabberMaxClassName).attr("transform","translate("+I+",0.5)")}(o,0,n,s,A,c),"bottom"===n.side&&l.draw(t,n._id+"title",{propContainer:n,propName:n._name+".title",placeholder:e._dfltTitle.x,attributes:{x:n._offset+n._length/2,y:M+s._height+s._offsetShift+10+1.5*n.title.font.size,"text-anchor":"middle"}})}))}},{"../../lib":719,"../../lib/setcursor":739,"../../plots/cartesian":778,"../../plots/cartesian/axis_ids":770,"../../plots/plots":828,"../../registry":847,"../color":593,"../dragelement":611,"../drawing":614,"../titles":681,"./constants":663,d3:163}],666:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axis_ids"),i=t("./constants"),o=i.name;function a(t){var e=t&&t[o];return e&&e.visible}n.isVisible=a,n.makeData=function(t){var e=r.list({_fullLayout:t},"x",!0),n=t.margin,i=[];if(!t._has("gl2d"))for(var s=0;s<e.length;s++){var A=e[s];if(a(A)){i.push(A);var l=A[o];l._id=o+A._id,l._height=(t.height-n.b-n.t)*l.thickness,l._offsetShift=Math.floor(l.borderwidth/2)}}t._rangeSliderData=i},n.autoMarginOpts=function(t,e){for(var n=e[o],a=1/0,s=e._counterAxes,A=0;A<s.length;A++){var l=s[A],c=r.getFromId(t,l);a=Math.min(a,c.domain[0])}n._oppBottom=a;var u="bottom"===e.side&&e._boundingBox.height||0;return n._tickHeight=u,{x:0,y:a,l:0,r:0,t:0,b:n._height+t._fullLayout.margin.b+u,pad:i.extraPad+2*n._offsetShift}}},{"../../plots/cartesian/axis_ids":770,"./constants":663}],667:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./attributes"),o=t("./oppaxis_attributes"),a=t("./helpers");e.exports={moduleType:"component",name:"rangeslider",schema:{subplots:{xaxis:{rangeslider:r.extendFlat({},i,{yaxis:o})}}},layoutAttributes:t("./attributes"),handleDefaults:t("./defaults"),calcAutorange:t("./calc_autorange"),draw:t("./draw"),isVisible:a.isVisible,makeData:a.makeData,autoMarginOpts:a.autoMarginOpts}},{"../../lib":719,"./attributes":661,"./calc_autorange":662,"./defaults":664,"./draw":665,"./helpers":666,"./oppaxis_attributes":668}],668:[function(t,e,n){"use strict";e.exports={_isSubplotObj:!0,rangemode:{valType:"enumerated",values:["auto","fixed","match"],dflt:"match",editType:"calc"},range:{valType:"info_array",items:[{valType:"any",editType:"plot"},{valType:"any",editType:"plot"}],editType:"plot"},editType:"calc"}},{}],669:[function(t,e,n){"use strict";var r=t("../annotations/attributes"),i=t("../../traces/scatter/attributes").line,o=t("../drawing/attributes").dash,a=t("../../lib/extend").extendFlat,s=t("../../plot_api/plot_template").templatedArray;e.exports=s("shape",{visible:{valType:"boolean",dflt:!0,editType:"calc+arraydraw"},type:{valType:"enumerated",values:["circle","rect","path","line"],editType:"calc+arraydraw"},layer:{valType:"enumerated",values:["below","above"],dflt:"above",editType:"arraydraw"},xref:a({},r.xref,{}),xsizemode:{valType:"enumerated",values:["scaled","pixel"],dflt:"scaled",editType:"calc+arraydraw"},xanchor:{valType:"any",editType:"calc+arraydraw"},x0:{valType:"any",editType:"calc+arraydraw"},x1:{valType:"any",editType:"calc+arraydraw"},yref:a({},r.yref,{}),ysizemode:{valType:"enumerated",values:["scaled","pixel"],dflt:"scaled",editType:"calc+arraydraw"},yanchor:{valType:"any",editType:"calc+arraydraw"},y0:{valType:"any",editType:"calc+arraydraw"},y1:{valType:"any",editType:"calc+arraydraw"},path:{valType:"string",editType:"calc+arraydraw"},opacity:{valType:"number",min:0,max:1,dflt:1,editType:"arraydraw"},line:{color:a({},i.color,{editType:"arraydraw"}),width:a({},i.width,{editType:"calc+arraydraw"}),dash:a({},o,{editType:"arraydraw"}),editType:"calc+arraydraw"},fillcolor:{valType:"color",dflt:"rgba(0,0,0,0)",editType:"arraydraw"},editType:"arraydraw"})},{"../../lib/extend":710,"../../plot_api/plot_template":757,"../../traces/scatter/attributes":1112,"../annotations/attributes":576,"../drawing/attributes":613}],670:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axes"),o=t("./constants"),a=t("./helpers");function s(t){return l(t.line.width,t.xsizemode,t.x0,t.x1,t.path,!1)}function A(t){return l(t.line.width,t.ysizemode,t.y0,t.y1,t.path,!0)}function l(t,e,n,i,s,A){var l=t/2,c=A;if("pixel"===e){var u=s?a.extractPathCoords(s,A?o.paramIsY:o.paramIsX):[n,i],g=r.aggNums(Math.max,null,u),h=r.aggNums(Math.min,null,u),d=h<0?Math.abs(h)+l:l,f=g>0?g+l:l;return{ppad:l,ppadplus:c?d:f,ppadminus:c?f:d}}return{ppad:l}}function c(t,e,n,r,i){var s="category"===t.type||"multicategory"===t.type?t.r2c:t.d2c;if(void 0!==e)return[s(e),s(n)];if(r){var A,l,c,u,g=1/0,h=-1/0,d=r.match(o.segmentRE);for("date"===t.type&&(s=a.decodeDate(s)),A=0;A<d.length;A++)void 0!==(l=i[d[A].charAt(0)].drawn)&&(!(c=d[A].substr(1).match(o.paramRE))||c.length<l||((u=s(c[l]))<g&&(g=u),u>h&&(h=u)));return h>=g?[g,h]:void 0}}e.exports=function(t){var e=t._fullLayout,n=r.filterVisible(e.shapes);if(n.length&&t._fullData.length)for(var a=0;a<n.length;a++){var l,u,g=n[a];if(g._extremes={},"paper"!==g.xref){var h="pixel"===g.xsizemode?g.xanchor:g.x0,d="pixel"===g.xsizemode?g.xanchor:g.x1;(u=c(l=i.getFromId(t,g.xref),h,d,g.path,o.paramIsX))&&(g._extremes[l._id]=i.findExtremes(l,u,s(g)))}if("paper"!==g.yref){var f="pixel"===g.ysizemode?g.yanchor:g.y0,p="pixel"===g.ysizemode?g.yanchor:g.y1;(u=c(l=i.getFromId(t,g.yref),f,p,g.path,o.paramIsY))&&(g._extremes[l._id]=i.findExtremes(l,u,A(g)))}}}},{"../../lib":719,"../../plots/cartesian/axes":767,"./constants":671,"./helpers":674}],671:[function(t,e,n){"use strict";e.exports={segmentRE:/[MLHVQCTSZ][^MLHVQCTSZ]*/g,paramRE:/[^\s,]+/g,paramIsX:{M:{0:!0,drawn:0},L:{0:!0,drawn:0},H:{0:!0,drawn:0},V:{},Q:{0:!0,2:!0,drawn:2},C:{0:!0,2:!0,4:!0,drawn:4},T:{0:!0,drawn:0},S:{0:!0,2:!0,drawn:2},Z:{}},paramIsY:{M:{1:!0,drawn:1},L:{1:!0,drawn:1},H:{},V:{0:!0,drawn:0},Q:{1:!0,3:!0,drawn:3},C:{1:!0,3:!0,5:!0,drawn:5},T:{1:!0,drawn:1},S:{1:!0,3:!0,drawn:5},Z:{}},numParams:{M:2,L:2,H:1,V:1,Q:4,C:6,T:2,S:4,Z:0}}},{}],672:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axes"),o=t("../../plots/array_container_defaults"),a=t("./attributes"),s=t("./helpers");function A(t,e,n){function o(n,i){return r.coerce(t,e,a,n,i)}if(o("visible")){o("layer"),o("opacity"),o("fillcolor"),o("line.color"),o("line.width"),o("line.dash");for(var A=o("type",t.path?"path":"rect"),l=o("xsizemode"),c=o("ysizemode"),u=["x","y"],g=0;g<2;g++){var h,d,f,p=u[g],I=p+"anchor",C="x"===p?l:c,m={_fullLayout:n},y=i.coerceRef(t,e,m,p,"","paper");if("paper"!==y?((h=i.getFromId(m,y))._shapeIndices.push(e._index),f=s.rangeToShapePosition(h),d=s.shapePositionToRange(h)):d=f=r.identity,"path"!==A){var E=p+"0",v=p+"1",B=t[E],b=t[v];t[E]=d(t[E],!0),t[v]=d(t[v],!0),"pixel"===C?(o(E,0),o(v,10)):(i.coercePosition(e,m,o,y,E,.25),i.coercePosition(e,m,o,y,v,.75)),e[E]=f(e[E]),e[v]=f(e[v]),t[E]=B,t[v]=b}if("pixel"===C){var Q=t[I];t[I]=d(t[I],!0),i.coercePosition(e,m,o,y,I,.25),e[I]=f(e[I]),t[I]=Q}}"path"===A?o("path"):r.noneOrAll(t,e,["x0","x1","y0","y1"])}}e.exports=function(t,e){o(t,e,{name:"shapes",handleItemDefaults:A})}},{"../../lib":719,"../../plots/array_container_defaults":763,"../../plots/cartesian/axes":767,"./attributes":669,"./helpers":674}],673:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib"),o=t("../../plots/cartesian/axes"),a=t("../color"),s=t("../drawing"),A=t("../../plot_api/plot_template").arrayEditor,l=t("../dragelement"),c=t("../../lib/setcursor"),u=t("./constants"),g=t("./helpers");function h(t,e){t._fullLayout._paperdiv.selectAll('.shapelayer [data-index="'+e+'"]').remove();var n=t._fullLayout.shapes[e]||{};if(n._input&&!1!==n.visible)if("below"!==n.layer)I(t._fullLayout._shapeUpperLayer);else if("paper"===n.xref||"paper"===n.yref)I(t._fullLayout._shapeLowerLayer);else{var h=t._fullLayout._plots[n.xref+n.yref];I(h?(h.mainplotinfo||h).shapelayer:t._fullLayout._shapeLowerLayer)}function I(h){var I={"data-index":e,"fill-rule":"evenodd",d:f(t,n)},C=n.line.width?n.line.color:"rgba(0,0,0,0)",m=h.append("path").attr(I).style("opacity",n.opacity).call(a.stroke,C).call(a.fill,n.fillcolor).call(s.dashLine,n.line.dash,n.line.width);d(m,t,n),t._context.edits.shapePosition&&function(t,e,n,a,h){var I,C,m,y,E,v,B,b,Q,w,x,_,D,M,k,S,L=10,R=10,N="pixel"===n.xsizemode,F="pixel"===n.ysizemode,T="line"===n.type,U="path"===n.type,O=A(t.layout,"shapes",n),G=O.modifyItem,H=o.getFromId(t,n.xref),j=o.getFromId(t,n.yref),Y=g.getDataToPixel(t,H),P=g.getDataToPixel(t,j,!0),K=g.getPixelToData(t,H),z=g.getPixelToData(t,j,!0),J=T?function(){var t=Math.max(n.line.width,10),r=h.append("g").attr("data-index",a);r.append("path").attr("d",e.attr("d")).style({cursor:"move","stroke-width":t,"stroke-opacity":"0"});var i={"fill-opacity":"0"},o=t/2>10?t/2:10;return r.append("circle").attr({"data-line-point":"start-point",cx:N?Y(n.xanchor)+n.x0:Y(n.x0),cy:F?P(n.yanchor)-n.y0:P(n.y0),r:o}).style(i).classed("cursor-grab",!0),r.append("circle").attr({"data-line-point":"end-point",cx:N?Y(n.xanchor)+n.x1:Y(n.x1),cy:F?P(n.yanchor)-n.y1:P(n.y1),r:o}).style(i).classed("cursor-grab",!0),r}():e,q={element:J.node(),gd:t,prepFn:function(r){N&&(E=Y(n.xanchor)),F&&(v=P(n.yanchor)),"path"===n.type?k=n.path:(I=N?n.x0:Y(n.x0),C=F?n.y0:P(n.y0),m=N?n.x1:Y(n.x1),y=F?n.y1:P(n.y1)),I<m?(Q=I,D="x0",w=m,M="x1"):(Q=m,D="x1",w=I,M="x0"),!F&&C<y||F&&C>y?(B=C,x="y0",b=y,_="y1"):(B=y,x="y1",b=C,_="y0"),V(r),Z(h,n),function(t,e,n){var r=e.xref,i=e.yref,a=o.getFromId(n,r),A=o.getFromId(n,i),l="";"paper"===r||a.autorange||(l+=r),"paper"===i||A.autorange||(l+=i),s.setClipUrl(t,l?"clip"+n._fullLayout._uid+l:null,n)}(e,n,t),q.moveFn="move"===S?W:X},doneFn:function(){c(e),$(h),d(e,t,n),r.call("_guiRelayout",t,O.getUpdateObj())},clickFn:function(){$(h)}};function V(t){if(T)S="path"===t.target.tagName?"move":"start-point"===t.target.attributes["data-line-point"].value?"resize-over-start-point":"resize-over-end-point";else{var n=q.element.getBoundingClientRect(),r=n.right-n.left,i=n.bottom-n.top,o=t.clientX-n.left,a=t.clientY-n.top,s=!U&&r>L&&i>R&&!t.shiftKey?l.getCursor(o/r,1-a/i):"move";c(e,s),S=s.split("-")[0]}}function W(r,i){if("path"===n.type){var o=function(t){return t},a=o,s=o;N?G("xanchor",n.xanchor=K(E+r)):(a=function(t){return K(Y(t)+r)},H&&"date"===H.type&&(a=g.encodeDate(a))),F?G("yanchor",n.yanchor=z(v+i)):(s=function(t){return z(P(t)+i)},j&&"date"===j.type&&(s=g.encodeDate(s))),G("path",n.path=p(k,a,s))}else N?G("xanchor",n.xanchor=K(E+r)):(G("x0",n.x0=K(I+r)),G("x1",n.x1=K(m+r))),F?G("yanchor",n.yanchor=z(v+i)):(G("y0",n.y0=z(C+i)),G("y1",n.y1=z(y+i)));e.attr("d",f(t,n)),Z(h,n)}function X(r,i){if(U){var o=function(t){return t},a=o,s=o;N?G("xanchor",n.xanchor=K(E+r)):(a=function(t){return K(Y(t)+r)},H&&"date"===H.type&&(a=g.encodeDate(a))),F?G("yanchor",n.yanchor=z(v+i)):(s=function(t){return z(P(t)+i)},j&&"date"===j.type&&(s=g.encodeDate(s))),G("path",n.path=p(k,a,s))}else if(T){if("resize-over-start-point"===S){var A=I+r,l=F?C-i:C+i;G("x0",n.x0=N?A:K(A)),G("y0",n.y0=F?l:z(l))}else if("resize-over-end-point"===S){var c=m+r,u=F?y-i:y+i;G("x1",n.x1=N?c:K(c)),G("y1",n.y1=F?u:z(u))}}else{var d=~S.indexOf("n")?B+i:B,O=~S.indexOf("s")?b+i:b,J=~S.indexOf("w")?Q+r:Q,q=~S.indexOf("e")?w+r:w;~S.indexOf("n")&&F&&(d=B-i),~S.indexOf("s")&&F&&(O=b-i),(!F&&O-d>R||F&&d-O>R)&&(G(x,n[x]=F?d:z(d)),G(_,n[_]=F?O:z(O))),q-J>L&&(G(D,n[D]=N?J:K(J)),G(M,n[M]=N?q:K(q)))}e.attr("d",f(t,n)),Z(h,n)}function Z(t,e){(N||F)&&function(){var n="path"!==e.type,r=t.selectAll(".visual-cue").data([0]);r.enter().append("path").attr({fill:"#fff","fill-rule":"evenodd",stroke:"#000","stroke-width":1}).classed("visual-cue",!0);var o=Y(N?e.xanchor:i.midRange(n?[e.x0,e.x1]:g.extractPathCoords(e.path,u.paramIsX))),a=P(F?e.yanchor:i.midRange(n?[e.y0,e.y1]:g.extractPathCoords(e.path,u.paramIsY)));if(o=g.roundPositionForSharpStrokeRendering(o,1),a=g.roundPositionForSharpStrokeRendering(a,1),N&&F){var s="M"+(o-1-1)+","+(a-1-1)+"h-8v2h8 v8h2v-8 h8v-2h-8 v-8h-2 Z";r.attr("d",s)}else if(N){var A="M"+(o-1-1)+","+(a-9-1)+"v18 h2 v-18 Z";r.attr("d",A)}else{var l="M"+(o-9-1)+","+(a-1-1)+"h18 v2 h-18 Z";r.attr("d",l)}}()}function $(t){t.selectAll(".visual-cue").remove()}l.init(q),J.node().onmousemove=V}(t,m,n,e,h)}}function d(t,e,n){var r=(n.xref+n.yref).replace(/paper/g,"");s.setClipUrl(t,r?"clip"+e._fullLayout._uid+r:null,e)}function f(t,e){var n,r,a,s,A,l,c,h,d=e.type,f=o.getFromId(t,e.xref),p=o.getFromId(t,e.yref),I=t._fullLayout._size;if(f?(n=g.shapePositionToRange(f),r=function(t){return f._offset+f.r2p(n(t,!0))}):r=function(t){return I.l+I.w*t},p?(a=g.shapePositionToRange(p),s=function(t){return p._offset+p.r2p(a(t,!0))}):s=function(t){return I.t+I.h*(1-t)},"path"===d)return f&&"date"===f.type&&(r=g.decodeDate(r)),p&&"date"===p.type&&(s=g.decodeDate(s)),function(t,e,n){var r=t.path,o=t.xsizemode,a=t.ysizemode,s=t.xanchor,A=t.yanchor;return r.replace(u.segmentRE,function(t){var r=0,l=t.charAt(0),c=u.paramIsX[l],g=u.paramIsY[l],h=u.numParams[l],d=t.substr(1).replace(u.paramRE,function(t){return c[r]?t="pixel"===o?e(s)+Number(t):e(t):g[r]&&(t="pixel"===a?n(A)-Number(t):n(t)),++r>h&&(t="X"),t});return r>h&&(d=d.replace(/[\s,]*X.*/,""),i.log("Ignoring extra params in segment "+t)),l+d})}(e,r,s);if("pixel"===e.xsizemode){var C=r(e.xanchor);A=C+e.x0,l=C+e.x1}else A=r(e.x0),l=r(e.x1);if("pixel"===e.ysizemode){var m=s(e.yanchor);c=m-e.y0,h=m-e.y1}else c=s(e.y0),h=s(e.y1);if("line"===d)return"M"+A+","+c+"L"+l+","+h;if("rect"===d)return"M"+A+","+c+"H"+l+"V"+h+"H"+A+"Z";var y=(A+l)/2,E=(c+h)/2,v=Math.abs(y-A),B=Math.abs(E-c),b="A"+v+","+B,Q=y+v+","+E;return"M"+Q+b+" 0 1,1 "+y+","+(E-B)+b+" 0 0,1 "+Q+"Z"}function p(t,e,n){return t.replace(u.segmentRE,function(t){var r=0,i=t.charAt(0),o=u.paramIsX[i],a=u.paramIsY[i],s=u.numParams[i];return i+t.substr(1).replace(u.paramRE,function(t){return r>=s?t:(o[r]?t=e(t):a[r]&&(t=n(t)),r++,t)})})}e.exports={draw:function(t){var e=t._fullLayout;for(var n in e._shapeUpperLayer.selectAll("path").remove(),e._shapeLowerLayer.selectAll("path").remove(),e._plots){var r=e._plots[n].shapelayer;r&&r.selectAll("path").remove()}for(var i=0;i<e.shapes.length;i++)e.shapes[i].visible&&h(t,i)},drawOne:h}},{"../../lib":719,"../../lib/setcursor":739,"../../plot_api/plot_template":757,"../../plots/cartesian/axes":767,"../../registry":847,"../color":593,"../dragelement":611,"../drawing":614,"./constants":671,"./helpers":674}],674:[function(t,e,n){"use strict";var r=t("./constants"),i=t("../../lib");n.rangeToShapePosition=function(t){return"log"===t.type?t.r2d:function(t){return t}},n.shapePositionToRange=function(t){return"log"===t.type?t.d2r:function(t){return t}},n.decodeDate=function(t){return function(e){return e.replace&&(e=e.replace("_"," ")),t(e)}},n.encodeDate=function(t){return function(e){return t(e).replace(" ","_")}},n.extractPathCoords=function(t,e){var n=[];return t.match(r.segmentRE).forEach(function(t){var o=e[t.charAt(0)].drawn;if(void 0!==o){var a=t.substr(1).match(r.paramRE);!a||a.length<o||n.push(i.cleanNumber(a[o]))}}),n},n.getDataToPixel=function(t,e,r){var i,o=t._fullLayout._size;if(e){var a=n.shapePositionToRange(e);i=function(t){return e._offset+e.r2p(a(t,!0))},"date"===e.type&&(i=n.decodeDate(i))}else i=r?function(t){return o.t+o.h*(1-t)}:function(t){return o.l+o.w*t};return i},n.getPixelToData=function(t,e,r){var i,o=t._fullLayout._size;if(e){var a=n.rangeToShapePosition(e);i=function(t){return a(e.p2r(t-e._offset))}}else i=r?function(t){return 1-(t-o.t)/o.h}:function(t){return(t-o.l)/o.w};return i},n.roundPositionForSharpStrokeRendering=function(t,e){var n=1===Math.round(e%2),r=Math.round(t);return n?r+.5:r}},{"../../lib":719,"./constants":671}],675:[function(t,e,n){"use strict";var r=t("./draw");e.exports={moduleType:"component",name:"shapes",layoutAttributes:t("./attributes"),supplyLayoutDefaults:t("./defaults"),includeBasePlot:t("../../plots/cartesian/include_components")("shapes"),calcAutorange:t("./calc_autorange"),draw:r.draw,drawOne:r.drawOne}},{"../../plots/cartesian/include_components":777,"./attributes":669,"./calc_autorange":670,"./defaults":672,"./draw":673}],676:[function(t,e,n){"use strict";var r=t("../../plots/font_attributes"),i=t("../../plots/pad_attributes"),o=t("../../lib/extend").extendDeepAll,a=t("../../plot_api/edit_types").overrideAll,s=t("../../plots/animation_attributes"),A=t("../../plot_api/plot_template").templatedArray,l=t("./constants"),c=A("step",{visible:{valType:"boolean",dflt:!0},method:{valType:"enumerated",values:["restyle","relayout","animate","update","skip"],dflt:"restyle"},args:{valType:"info_array",freeLength:!0,items:[{valType:"any"},{valType:"any"},{valType:"any"}]},label:{valType:"string"},value:{valType:"string"},execute:{valType:"boolean",dflt:!0}});e.exports=a(A("slider",{visible:{valType:"boolean",dflt:!0},active:{valType:"number",min:0,dflt:0},steps:c,lenmode:{valType:"enumerated",values:["fraction","pixels"],dflt:"fraction"},len:{valType:"number",min:0,dflt:1},x:{valType:"number",min:-2,max:3,dflt:0},pad:o(i({editType:"arraydraw"}),{},{t:{dflt:20}}),xanchor:{valType:"enumerated",values:["auto","left","center","right"],dflt:"left"},y:{valType:"number",min:-2,max:3,dflt:0},yanchor:{valType:"enumerated",values:["auto","top","middle","bottom"],dflt:"top"},transition:{duration:{valType:"number",min:0,dflt:150},easing:{valType:"enumerated",values:s.transition.easing.values,dflt:"cubic-in-out"}},currentvalue:{visible:{valType:"boolean",dflt:!0},xanchor:{valType:"enumerated",values:["left","center","right"],dflt:"left"},offset:{valType:"number",dflt:10},prefix:{valType:"string"},suffix:{valType:"string"},font:r({})},font:r({}),activebgcolor:{valType:"color",dflt:l.gripBgActiveColor},bgcolor:{valType:"color",dflt:l.railBgColor},bordercolor:{valType:"color",dflt:l.railBorderColor},borderwidth:{valType:"number",min:0,dflt:l.railBorderWidth},ticklen:{valType:"number",min:0,dflt:l.tickLength},tickcolor:{valType:"color",dflt:l.tickColor},tickwidth:{valType:"number",min:0,dflt:1},minorticklen:{valType:"number",min:0,dflt:l.minorTickLength}}),"arraydraw","from-root")},{"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plot_api/plot_template":757,"../../plots/animation_attributes":762,"../../plots/font_attributes":793,"../../plots/pad_attributes":827,"./constants":677}],677:[function(t,e,n){"use strict";e.exports={name:"sliders",containerClassName:"slider-container",groupClassName:"slider-group",inputAreaClass:"slider-input-area",railRectClass:"slider-rail-rect",railTouchRectClass:"slider-rail-touch-rect",gripRectClass:"slider-grip-rect",tickRectClass:"slider-tick-rect",inputProxyClass:"slider-input-proxy",labelsClass:"slider-labels",labelGroupClass:"slider-label-group",labelClass:"slider-label",currentValueClass:"slider-current-value",railHeight:5,menuIndexAttrName:"slider-active-index",autoMarginIdRoot:"slider-",minWidth:30,minHeight:30,textPadX:40,arrowOffsetX:4,railRadius:2,railWidth:5,railBorder:4,railBorderWidth:1,railBorderColor:"#bec8d9",railBgColor:"#f8fafc",railInset:8,stepInset:10,gripRadius:10,gripWidth:20,gripHeight:20,gripBorder:20,gripBorderWidth:1,gripBorderColor:"#bec8d9",gripBgColor:"#f6f8fa",gripBgActiveColor:"#dbdde0",labelPadding:8,labelOffset:0,tickWidth:1,tickColor:"#333",tickOffset:25,tickLength:7,minorTickOffset:25,minorTickColor:"#333",minorTickLength:4,currentValuePadding:8,currentValueInset:0}},{}],678:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/array_container_defaults"),o=t("./attributes"),a=t("./constants").name,s=o.steps;function A(t,e,n){function a(n,i){return r.coerce(t,e,o,n,i)}for(var s=i(t,e,{name:"steps",handleItemDefaults:l}),A=0,c=0;c<s.length;c++)s[c].visible&&A++;if(A<2?e.visible=!1:a("visible")){e._stepCount=A;var u=e._visibleSteps=r.filterVisible(s);(s[a("active")]||{}).visible||(e.active=u[0]._index),a("x"),a("y"),r.noneOrAll(t,e,["x","y"]),a("xanchor"),a("yanchor"),a("len"),a("lenmode"),a("pad.t"),a("pad.r"),a("pad.b"),a("pad.l"),r.coerceFont(a,"font",n.font),a("currentvalue.visible")&&(a("currentvalue.xanchor"),a("currentvalue.prefix"),a("currentvalue.suffix"),a("currentvalue.offset"),r.coerceFont(a,"currentvalue.font",e.font)),a("transition.duration"),a("transition.easing"),a("bgcolor"),a("activebgcolor"),a("bordercolor"),a("borderwidth"),a("ticklen"),a("tickwidth"),a("tickcolor"),a("minorticklen")}}function l(t,e){function n(n,i){return r.coerce(t,e,s,n,i)}if("skip"===t.method||Array.isArray(t.args)?n("visible"):e.visible=!1){n("method"),n("args");var i=n("label","step-"+e._index);n("value",i),n("execute")}}e.exports=function(t,e){i(t,e,{name:a,handleItemDefaults:A})}},{"../../lib":719,"../../plots/array_container_defaults":763,"./attributes":676,"./constants":677}],679:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../plots/plots"),o=t("../color"),a=t("../drawing"),s=t("../../lib"),A=t("../../lib/svg_text_utils"),l=t("../../plot_api/plot_template").arrayEditor,c=t("./constants"),u=t("../../constants/alignment"),g=u.LINE_SPACING,h=u.FROM_TL,d=u.FROM_BR;function f(t){return c.autoMarginIdRoot+t._index}function p(t){return t._index}function I(t,e){var n=a.tester.selectAll("g."+c.labelGroupClass).data(e._visibleSteps);n.enter().append("g").classed(c.labelGroupClass,!0);var o=0,l=0;n.each(function(t){var n=y(r.select(this),{step:t},e).node();if(n){var i=a.bBox(n);l=Math.max(l,i.height),o=Math.max(o,i.width)}}),n.remove();var u=e._dims={};u.inputAreaWidth=Math.max(c.railWidth,c.gripHeight);var g=t._fullLayout._size;u.lx=g.l+g.w*e.x,u.ly=g.t+g.h*(1-e.y),"fraction"===e.lenmode?u.outerLength=Math.round(g.w*e.len):u.outerLength=e.len,u.inputAreaStart=0,u.inputAreaLength=Math.round(u.outerLength-e.pad.l-e.pad.r);var p=(u.inputAreaLength-2*c.stepInset)/(e._stepCount-1),I=o+c.labelPadding;if(u.labelStride=Math.max(1,Math.ceil(I/p)),u.labelHeight=l,u.currentValueMaxWidth=0,u.currentValueHeight=0,u.currentValueTotalHeight=0,u.currentValueMaxLines=1,e.currentvalue.visible){var m=a.tester.append("g");n.each(function(t){var n=C(m,e,t.label),r=n.node()&&a.bBox(n.node())||{width:0,height:0},i=A.lineCount(n);u.currentValueMaxWidth=Math.max(u.currentValueMaxWidth,Math.ceil(r.width)),u.currentValueHeight=Math.max(u.currentValueHeight,Math.ceil(r.height)),u.currentValueMaxLines=Math.max(u.currentValueMaxLines,i)}),u.currentValueTotalHeight=u.currentValueHeight+e.currentvalue.offset,m.remove()}u.height=u.currentValueTotalHeight+c.tickOffset+e.ticklen+c.labelOffset+u.labelHeight+e.pad.t+e.pad.b;var E="left";s.isRightAnchor(e)&&(u.lx-=u.outerLength,E="right"),s.isCenterAnchor(e)&&(u.lx-=u.outerLength/2,E="center");var v="top";s.isBottomAnchor(e)&&(u.ly-=u.height,v="bottom"),s.isMiddleAnchor(e)&&(u.ly-=u.height/2,v="middle"),u.outerLength=Math.ceil(u.outerLength),u.height=Math.ceil(u.height),u.lx=Math.round(u.lx),u.ly=Math.round(u.ly);var B={y:e.y,b:u.height*d[v],t:u.height*h[v]};"fraction"===e.lenmode?(B.l=0,B.xl=e.x-e.len*h[E],B.r=0,B.xr=e.x+e.len*d[E]):(B.x=e.x,B.l=u.outerLength*h[E],B.r=u.outerLength*d[E]),i.autoMargin(t,f(e),B)}function C(t,e,n){if(e.currentvalue.visible){var r,i,o=e._dims;switch(e.currentvalue.xanchor){case"right":r=o.inputAreaLength-c.currentValueInset-o.currentValueMaxWidth,i="left";break;case"center":r=.5*o.inputAreaLength,i="middle";break;default:r=c.currentValueInset,i="left"}var l=s.ensureSingle(t,"text",c.labelClass,function(t){t.classed("user-select-none",!0).attr({"text-anchor":i,"data-notex":1})}),u=e.currentvalue.prefix?e.currentvalue.prefix:"";if("string"==typeof n)u+=n;else{var h=e.steps[e.active].label,d=e._gd._fullLayout._meta;d&&(h=s.templateString(h,d)),u+=h}e.currentvalue.suffix&&(u+=e.currentvalue.suffix),l.call(a.font,e.currentvalue.font).text(u).call(A.convertToTspans,e._gd);var f=A.lineCount(l),p=(o.currentValueMaxLines+1-f)*e.currentvalue.font.size*g;return A.positionText(l,r,p),l}}function m(t,e,n){s.ensureSingle(t,"rect",c.gripRectClass,function(r){r.call(b,e,t,n).style("pointer-events","all")}).attr({width:c.gripWidth,height:c.gripHeight,rx:c.gripRadius,ry:c.gripRadius}).call(o.stroke,n.bordercolor).call(o.fill,n.bgcolor).style("stroke-width",n.borderwidth+"px")}function y(t,e,n){var r=s.ensureSingle(t,"text",c.labelClass,function(t){t.classed("user-select-none",!0).attr({"text-anchor":"middle","data-notex":1})}),i=e.step.label,o=n._gd._fullLayout._meta;return o&&(i=s.templateString(i,o)),r.call(a.font,n.font).text(i).call(A.convertToTspans,n._gd),r}function E(t,e){var n=s.ensureSingle(t,"g",c.labelsClass),i=e._dims,o=n.selectAll("g."+c.labelGroupClass).data(i.labelSteps);o.enter().append("g").classed(c.labelGroupClass,!0),o.exit().remove(),o.each(function(t){var n=r.select(this);n.call(y,t,e),a.setTranslate(n,x(e,t.fraction),c.tickOffset+e.ticklen+e.font.size*g+c.labelOffset+i.currentValueTotalHeight)})}function v(t,e,n,r,i){var o=Math.round(r*(n._stepCount-1)),a=n._visibleSteps[o]._index;a!==n.active&&B(t,e,n,a,!0,i)}function B(t,e,n,r,o,a){var s=n.active;n.active=r,l(t.layout,c.name,n).applyUpdate("active",r);var A=n.steps[n.active];e.call(w,n,a),e.call(C,n),t.emit("plotly_sliderchange",{slider:n,step:n.steps[n.active],interaction:o,previousActive:s}),A&&A.method&&o&&(e._nextMethod?(e._nextMethod.step=A,e._nextMethod.doCallback=o,e._nextMethod.doTransition=a):(e._nextMethod={step:A,doCallback:o,doTransition:a},e._nextMethodRaf=window.requestAnimationFrame(function(){var n=e._nextMethod.step;n.method&&(n.execute&&i.executeAPICommand(t,n.method,n.args),e._nextMethod=null,e._nextMethodRaf=null)})))}function b(t,e,n){var i=n.node(),a=r.select(e);function s(){return n.data()[0]}t.on("mousedown",function(){var t=s();e.emit("plotly_sliderstart",{slider:t});var A=n.select("."+c.gripRectClass);r.event.stopPropagation(),r.event.preventDefault(),A.call(o.fill,t.activebgcolor);var l=_(t,r.mouse(i)[0]);v(e,n,t,l,!0),t._dragging=!0,a.on("mousemove",function(){var t=s(),o=_(t,r.mouse(i)[0]);v(e,n,t,o,!1)}),a.on("mouseup",function(){var t=s();t._dragging=!1,A.call(o.fill,t.bgcolor),a.on("mouseup",null),a.on("mousemove",null),e.emit("plotly_sliderend",{slider:t,step:t.steps[t.active]})})})}function Q(t,e){var n=t.selectAll("rect."+c.tickRectClass).data(e._visibleSteps),i=e._dims;n.enter().append("rect").classed(c.tickRectClass,!0),n.exit().remove(),n.attr({width:e.tickwidth+"px","shape-rendering":"crispEdges"}),n.each(function(t,n){var s=n%i.labelStride==0,A=r.select(this);A.attr({height:s?e.ticklen:e.minorticklen}).call(o.fill,e.tickcolor),a.setTranslate(A,x(e,n/(e._stepCount-1))-.5*e.tickwidth,(s?c.tickOffset:c.minorTickOffset)+i.currentValueTotalHeight)})}function w(t,e,n){for(var r=t.select("rect."+c.gripRectClass),i=0,o=0;o<e._stepCount;o++)if(e._visibleSteps[o]._index===e.active){i=o;break}var a=x(e,i/(e._stepCount-1));if(!e._invokingCommand){var s=r;n&&e.transition.duration>0&&(s=s.transition().duration(e.transition.duration).ease(e.transition.easing)),s.attr("transform","translate("+(a-.5*c.gripWidth)+","+e._dims.currentValueTotalHeight+")")}}function x(t,e){var n=t._dims;return n.inputAreaStart+c.stepInset+(n.inputAreaLength-2*c.stepInset)*Math.min(1,Math.max(0,e))}function _(t,e){var n=t._dims;return Math.min(1,Math.max(0,(e-c.stepInset-n.inputAreaStart)/(n.inputAreaLength-2*c.stepInset-2*n.inputAreaStart)))}function D(t,e,n){var r=n._dims,i=s.ensureSingle(t,"rect",c.railTouchRectClass,function(r){r.call(b,e,t,n).style("pointer-events","all")});i.attr({width:r.inputAreaLength,height:Math.max(r.inputAreaWidth,c.tickOffset+n.ticklen+r.labelHeight)}).call(o.fill,n.bgcolor).attr("opacity",0),a.setTranslate(i,0,r.currentValueTotalHeight)}function M(t,e){var n=e._dims,r=n.inputAreaLength-2*c.railInset,i=s.ensureSingle(t,"rect",c.railRectClass);i.attr({width:r,height:c.railWidth,rx:c.railRadius,ry:c.railRadius,"shape-rendering":"crispEdges"}).call(o.stroke,e.bordercolor).call(o.fill,e.bgcolor).style("stroke-width",e.borderwidth+"px"),a.setTranslate(i,c.railInset,.5*(n.inputAreaWidth-c.railWidth)+n.currentValueTotalHeight)}e.exports=function(t){var e=t._fullLayout,n=function(t,e){for(var n=t[c.name],r=[],i=0;i<n.length;i++){var o=n[i];o.visible&&(o._gd=e,r.push(o))}return r}(e,t),o=e._infolayer.selectAll("g."+c.containerClassName).data(n.length>0?[0]:[]);function s(e){e._commandObserver&&(e._commandObserver.remove(),delete e._commandObserver),i.autoMargin(t,f(e))}if(o.enter().append("g").classed(c.containerClassName,!0).style("cursor","ew-resize"),o.exit().each(function(){r.select(this).selectAll("g."+c.groupClassName).each(s)}).remove(),0!==n.length){var A=o.selectAll("g."+c.groupClassName).data(n,p);A.enter().append("g").classed(c.groupClassName,!0),A.exit().each(s).remove();for(var l=0;l<n.length;l++){var u=n[l];I(t,u)}A.each(function(e){var n=r.select(this);!function(t){var e=t._dims;e.labelSteps=[];for(var n=t._stepCount,r=0;r<n;r+=e.labelStride)e.labelSteps.push({fraction:r/(n-1),step:t._visibleSteps[r]})}(e),i.manageCommandObserver(t,e,e._visibleSteps,function(e){var r=n.data()[0];r.active!==e.index&&(r._dragging||B(t,n,r,e.index,!1,!0))}),function(t,e,n){(n.steps[n.active]||{}).visible||(n.active=n._visibleSteps[0]._index),e.call(C,n).call(M,n).call(E,n).call(Q,n).call(D,t,n).call(m,t,n);var r=n._dims;a.setTranslate(e,r.lx+n.pad.l,r.ly+n.pad.t),e.call(w,n,!1),e.call(C,n)}(t,r.select(this),e)})}}},{"../../constants/alignment":688,"../../lib":719,"../../lib/svg_text_utils":743,"../../plot_api/plot_template":757,"../../plots/plots":828,"../color":593,"../drawing":614,"./constants":677,d3:163}],680:[function(t,e,n){"use strict";var r=t("./constants");e.exports={moduleType:"component",name:r.name,layoutAttributes:t("./attributes"),supplyLayoutDefaults:t("./defaults"),draw:t("./draw")}},{"./attributes":676,"./constants":677,"./defaults":678,"./draw":679}],681:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=t("../../plots/plots"),a=t("../../registry"),s=t("../../lib"),A=t("../drawing"),l=t("../color"),c=t("../../lib/svg_text_utils"),u=t("../../constants/interactions");e.exports={draw:function(t,e,n){var h,d=n.propContainer,f=n.propName,p=n.placeholder,I=n.traceIndex,C=n.avoid||{},m=n.attributes,y=n.transform,E=n.containerGroup,v=t._fullLayout,B=1,b=!1,Q=d.title,w=(Q&&Q.text?Q.text:"").trim(),x=Q&&Q.font?Q.font:{},_=x.family,D=x.size,M=x.color;"title.text"===f?h="titleText":-1!==f.indexOf("axis")?h="axisTitleText":f.indexOf(!0)&&(h="colorbarTitleText");var k=t._context.edits[h];""===w?B=0:w.replace(g," % ")===p.replace(g," % ")&&(B=.2,b=!0,k||(w="")),n._meta?w=s.templateString(w,n._meta):v._meta&&(w=s.templateString(w,v._meta));var S=w||k;E||(E=s.ensureSingle(v._infolayer,"g","g-"+e));var L=E.selectAll("text").data(S?[0]:[]);if(L.enter().append("text"),L.text(w).attr("class",e),L.exit().remove(),!S)return E;function R(t){s.syncOrAsync([N,F],t)}function N(e){var n;return y?(n="",y.rotate&&(n+="rotate("+[y.rotate,m.x,m.y]+")"),y.offset&&(n+="translate(0, "+y.offset+")")):n=null,e.attr("transform",n),e.style({"font-family":_,"font-size":r.round(D,2)+"px",fill:l.rgb(M),opacity:B*l.opacity(M),"font-weight":o.fontWeight}).attr(m).call(c.convertToTspans,t),o.previousPromises(t)}function F(t){var e=r.select(t.node().parentNode);if(C&&C.selection&&C.side&&w){e.attr("transform",null);var n=0,o={left:"right",right:"left",top:"bottom",bottom:"top"}[C.side],a=-1!==["left","top"].indexOf(C.side)?-1:1,l=i(C.pad)?C.pad:2,c=A.bBox(e.node()),u={left:0,top:0,right:v.width,bottom:v.height},g=C.maxShift||(u[C.side]-c[C.side])*("left"===C.side||"top"===C.side?-1:1);if(g<0)n=g;else{var h=C.offsetLeft||0,d=C.offsetTop||0;c.left-=h,c.right-=h,c.top-=d,c.bottom-=d,C.selection.each(function(){var t=A.bBox(this);s.bBoxIntersect(c,t,l)&&(n=Math.max(n,a*(t[C.side]-c[o])+l))}),n=Math.min(g,n)}if(n>0||g<0){var f={left:[-n,0],right:[n,0],top:[0,-n],bottom:[0,n]}[C.side];e.attr("transform","translate("+f+")")}}}return L.call(R),k&&(w?L.on(".opacity",null):(B=0,b=!0,L.text(p).on("mouseover.opacity",function(){r.select(this).transition().duration(u.SHOW_PLACEHOLDER).style("opacity",1)}).on("mouseout.opacity",function(){r.select(this).transition().duration(u.HIDE_PLACEHOLDER).style("opacity",0)})),L.call(c.makeEditable,{gd:t}).on("edit",function(e){void 0!==I?a.call("_guiRestyle",t,f,e,I):a.call("_guiRelayout",t,f,e)}).on("cancel",function(){this.text(this.attr("data-unformatted")).call(R)}).on("input",function(t){this.text(t||" ").call(c.positionText,m.x,m.y)})),L.classed("js-placeholder",b),E}};var g=/ [XY][0-9]* /},{"../../constants/interactions":694,"../../lib":719,"../../lib/svg_text_utils":743,"../../plots/plots":828,"../../registry":847,"../color":593,"../drawing":614,d3:163,"fast-isnumeric":225}],682:[function(t,e,n){"use strict";var r=t("../../plots/font_attributes"),i=t("../color/attributes"),o=t("../../lib/extend").extendFlat,a=t("../../plot_api/edit_types").overrideAll,s=t("../../plots/pad_attributes"),A=t("../../plot_api/plot_template").templatedArray,l=A("button",{visible:{valType:"boolean"},method:{valType:"enumerated",values:["restyle","relayout","animate","update","skip"],dflt:"restyle"},args:{valType:"info_array",freeLength:!0,items:[{valType:"any"},{valType:"any"},{valType:"any"}]},label:{valType:"string",dflt:""},execute:{valType:"boolean",dflt:!0}});e.exports=a(A("updatemenu",{_arrayAttrRegexps:[/^updatemenus\[(0|[1-9][0-9]+)\]\.buttons/],visible:{valType:"boolean"},type:{valType:"enumerated",values:["dropdown","buttons"],dflt:"dropdown"},direction:{valType:"enumerated",values:["left","right","up","down"],dflt:"down"},active:{valType:"integer",min:-1,dflt:0},showactive:{valType:"boolean",dflt:!0},buttons:l,x:{valType:"number",min:-2,max:3,dflt:-.05},xanchor:{valType:"enumerated",values:["auto","left","center","right"],dflt:"right"},y:{valType:"number",min:-2,max:3,dflt:1},yanchor:{valType:"enumerated",values:["auto","top","middle","bottom"],dflt:"top"},pad:o(s({editType:"arraydraw"}),{}),font:r({}),bgcolor:{valType:"color"},bordercolor:{valType:"color",dflt:i.borderLine},borderwidth:{valType:"number",min:0,dflt:1,editType:"arraydraw"}}),"arraydraw","from-root")},{"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plot_api/plot_template":757,"../../plots/font_attributes":793,"../../plots/pad_attributes":827,"../color/attributes":592}],683:[function(t,e,n){"use strict";e.exports={name:"updatemenus",containerClassName:"updatemenu-container",headerGroupClassName:"updatemenu-header-group",headerClassName:"updatemenu-header",headerArrowClassName:"updatemenu-header-arrow",dropdownButtonGroupClassName:"updatemenu-dropdown-button-group",dropdownButtonClassName:"updatemenu-dropdown-button",buttonClassName:"updatemenu-button",itemRectClassName:"updatemenu-item-rect",itemTextClassName:"updatemenu-item-text",menuIndexAttrName:"updatemenu-active-index",autoMarginIdRoot:"updatemenu-",blankHeaderOpts:{label:"  "},minWidth:30,minHeight:30,textPadX:24,arrowPadX:16,rx:2,ry:2,textOffsetX:12,textOffsetY:3,arrowOffsetX:4,gapButtonHeader:5,gapButton:2,activeColor:"#F4FAFF",hoverColor:"#F4FAFF",arrowSymbol:{left:"◄",right:"►",up:"▲",down:"▼"}}},{}],684:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/array_container_defaults"),o=t("./attributes"),a=t("./constants").name,s=o.buttons;function A(t,e,n){function a(n,i){return r.coerce(t,e,o,n,i)}a("visible",i(t,e,{name:"buttons",handleItemDefaults:l}).length>0)&&(a("active"),a("direction"),a("type"),a("showactive"),a("x"),a("y"),r.noneOrAll(t,e,["x","y"]),a("xanchor"),a("yanchor"),a("pad.t"),a("pad.r"),a("pad.b"),a("pad.l"),r.coerceFont(a,"font",n.font),a("bgcolor",n.paper_bgcolor),a("bordercolor"),a("borderwidth"))}function l(t,e){function n(n,i){return r.coerce(t,e,s,n,i)}n("visible","skip"===t.method||Array.isArray(t.args))&&(n("method"),n("args"),n("label"),n("execute"))}e.exports=function(t,e){i(t,e,{name:a,handleItemDefaults:A})}},{"../../lib":719,"../../plots/array_container_defaults":763,"./attributes":682,"./constants":683}],685:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../plots/plots"),o=t("../color"),a=t("../drawing"),s=t("../../lib"),A=t("../../lib/svg_text_utils"),l=t("../../plot_api/plot_template").arrayEditor,c=t("../../constants/alignment").LINE_SPACING,u=t("./constants"),g=t("./scrollbox");function h(t){return t._index}function d(t,e){return+t.attr(u.menuIndexAttrName)===e._index}function f(t,e,n,r,i,o,a,s){e.active=a,l(t.layout,u.name,e).applyUpdate("active",a),"buttons"===e.type?I(t,r,null,null,e):"dropdown"===e.type&&(i.attr(u.menuIndexAttrName,"-1"),p(t,r,i,o,e),s||I(t,r,i,o,e))}function p(t,e,n,r,i){var o=s.ensureSingle(e,"g",u.headerClassName,function(t){t.style("pointer-events","all")}),A=i._dims,l=i.active,c=i.buttons[l]||u.blankHeaderOpts,g={y:i.pad.t,yPad:0,x:i.pad.l,xPad:0,index:0},h={width:A.headerWidth,height:A.headerHeight};o.call(C,i,c,t).call(w,i,g,h),s.ensureSingle(e,"text",u.headerArrowClassName,function(t){t.classed("user-select-none",!0).attr("text-anchor","end").call(a.font,i.font).text(u.arrowSymbol[i.direction])}).attr({x:A.headerWidth-u.arrowOffsetX+i.pad.l,y:A.headerHeight/2+u.textOffsetY+i.pad.t}),o.on("click",function(){n.call(x,String(d(n,i)?-1:i._index)),I(t,e,n,r,i)}),o.on("mouseover",function(){o.call(v)}),o.on("mouseout",function(){o.call(B,i)}),a.setTranslate(e,A.lx,A.ly)}function I(t,e,n,o,a){n||(n=e).attr("pointer-events","all");var A=function(t){return-1==+t.attr(u.menuIndexAttrName)}(n)&&"buttons"!==a.type?[]:a.buttons,l="dropdown"===a.type?u.dropdownButtonClassName:u.buttonClassName,c=n.selectAll("g."+l).data(s.filterVisible(A)),g=c.enter().append("g").classed(l,!0),h=c.exit();"dropdown"===a.type?(g.attr("opacity","0").transition().attr("opacity","1"),h.transition().attr("opacity","0").remove()):h.remove();var d=0,p=0,I=a._dims,m=-1!==["up","down"].indexOf(a.direction);"dropdown"===a.type&&(m?p=I.headerHeight+u.gapButtonHeader:d=I.headerWidth+u.gapButtonHeader),"dropdown"===a.type&&"up"===a.direction&&(p=-u.gapButtonHeader+u.gapButton-I.openHeight),"dropdown"===a.type&&"left"===a.direction&&(d=-u.gapButtonHeader+u.gapButton-I.openWidth);var y={x:I.lx+d+a.pad.l,y:I.ly+p+a.pad.t,yPad:u.gapButton,xPad:u.gapButton,index:0},b={l:y.x+a.borderwidth,t:y.y+a.borderwidth};c.each(function(s,A){var l=r.select(this);l.call(C,a,s,t).call(w,a,y),l.on("click",function(){r.event.defaultPrevented||(f(t,a,0,e,n,o,A),s.execute&&i.executeAPICommand(t,s.method,s.args),t.emit("plotly_buttonclicked",{menu:a,button:s,active:a.active}))}),l.on("mouseover",function(){l.call(v)}),l.on("mouseout",function(){l.call(B,a),c.call(E,a)})}),c.call(E,a),m?(b.w=Math.max(I.openWidth,I.headerWidth),b.h=y.y-b.t):(b.w=y.x-b.l,b.h=Math.max(I.openHeight,I.headerHeight)),b.direction=a.direction,o&&(c.size()?function(t,e,n,r,i,o){var a,s,A,l=i.direction,c="up"===l||"down"===l,g=i._dims,h=i.active;if(c)for(s=0,A=0;A<h;A++)s+=g.heights[A]+u.gapButton;else for(a=0,A=0;A<h;A++)a+=g.widths[A]+u.gapButton;r.enable(o,a,s),r.hbar&&r.hbar.attr("opacity","0").transition().attr("opacity","1"),r.vbar&&r.vbar.attr("opacity","0").transition().attr("opacity","1")}(0,0,0,o,a,b):function(t){var e=!!t.hbar,n=!!t.vbar;e&&t.hbar.transition().attr("opacity","0").each("end",function(){e=!1,n||t.disable()}),n&&t.vbar.transition().attr("opacity","0").each("end",function(){n=!1,e||t.disable()})}(o))}function C(t,e,n,r){t.call(m,e).call(y,e,n,r)}function m(t,e){s.ensureSingle(t,"rect",u.itemRectClassName,function(t){t.attr({rx:u.rx,ry:u.ry,"shape-rendering":"crispEdges"})}).call(o.stroke,e.bordercolor).call(o.fill,e.bgcolor).style("stroke-width",e.borderwidth+"px")}function y(t,e,n,r){var i=s.ensureSingle(t,"text",u.itemTextClassName,function(t){t.classed("user-select-none",!0).attr({"text-anchor":"start","data-notex":1})}),o=n.label,l=r._fullLayout._meta;l&&(o=s.templateString(o,l)),i.call(a.font,e.font).text(o).call(A.convertToTspans,r)}function E(t,e){var n=e.active;t.each(function(t,i){var a=r.select(this);i===n&&e.showactive&&a.select("rect."+u.itemRectClassName).call(o.fill,u.activeColor)})}function v(t){t.select("rect."+u.itemRectClassName).call(o.fill,u.hoverColor)}function B(t,e){t.select("rect."+u.itemRectClassName).call(o.fill,e.bgcolor)}function b(t,e){var n=e._dims={width1:0,height1:0,heights:[],widths:[],totalWidth:0,totalHeight:0,openWidth:0,openHeight:0,lx:0,ly:0},o=a.tester.selectAll("g."+u.dropdownButtonClassName).data(s.filterVisible(e.buttons));o.enter().append("g").classed(u.dropdownButtonClassName,!0);var l=-1!==["up","down"].indexOf(e.direction);o.each(function(i,o){var s=r.select(this);s.call(C,e,i,t);var g=s.select("."+u.itemTextClassName),h=g.node()&&a.bBox(g.node()).width,d=Math.max(h+u.textPadX,u.minWidth),f=e.font.size*c,p=A.lineCount(g),I=Math.max(f*p,u.minHeight)+u.textOffsetY;I=Math.ceil(I),d=Math.ceil(d),n.widths[o]=d,n.heights[o]=I,n.height1=Math.max(n.height1,I),n.width1=Math.max(n.width1,d),l?(n.totalWidth=Math.max(n.totalWidth,d),n.openWidth=n.totalWidth,n.totalHeight+=I+u.gapButton,n.openHeight+=I+u.gapButton):(n.totalWidth+=d+u.gapButton,n.openWidth+=d+u.gapButton,n.totalHeight=Math.max(n.totalHeight,I),n.openHeight=n.totalHeight)}),l?n.totalHeight-=u.gapButton:n.totalWidth-=u.gapButton,n.headerWidth=n.width1+u.arrowPadX,n.headerHeight=n.height1,"dropdown"===e.type&&(l?(n.width1+=u.arrowPadX,n.totalHeight=n.height1):n.totalWidth=n.width1,n.totalWidth+=u.arrowPadX),o.remove();var g=n.totalWidth+e.pad.l+e.pad.r,h=n.totalHeight+e.pad.t+e.pad.b,d=t._fullLayout._size;n.lx=d.l+d.w*e.x,n.ly=d.t+d.h*(1-e.y);var f="left";s.isRightAnchor(e)&&(n.lx-=g,f="right"),s.isCenterAnchor(e)&&(n.lx-=g/2,f="center");var p="top";s.isBottomAnchor(e)&&(n.ly-=h,p="bottom"),s.isMiddleAnchor(e)&&(n.ly-=h/2,p="middle"),n.totalWidth=Math.ceil(n.totalWidth),n.totalHeight=Math.ceil(n.totalHeight),n.lx=Math.round(n.lx),n.ly=Math.round(n.ly),i.autoMargin(t,Q(e),{x:e.x,y:e.y,l:g*({right:1,center:.5}[f]||0),r:g*({left:1,center:.5}[f]||0),b:h*({top:1,middle:.5}[p]||0),t:h*({bottom:1,middle:.5}[p]||0)})}function Q(t){return u.autoMarginIdRoot+t._index}function w(t,e,n,r){r=r||{};var i=t.select("."+u.itemRectClassName),o=t.select("."+u.itemTextClassName),s=e.borderwidth,l=n.index,g=e._dims;a.setTranslate(t,s+n.x,s+n.y);var h=-1!==["up","down"].indexOf(e.direction),d=r.height||(h?g.heights[l]:g.height1);i.attr({x:0,y:0,width:r.width||(h?g.width1:g.widths[l]),height:d});var f=e.font.size*c,p=(A.lineCount(o)-1)*f/2;A.positionText(o,u.textOffsetX,d/2-p+u.textOffsetY),h?n.y+=g.heights[l]+n.yPad:n.x+=g.widths[l]+n.xPad,n.index++}function x(t,e){t.attr(u.menuIndexAttrName,e||"-1").selectAll("g."+u.dropdownButtonClassName).remove()}e.exports=function(t){var e=t._fullLayout,n=s.filterVisible(e[u.name]);function o(e){i.autoMargin(t,Q(e))}var a=e._menulayer.selectAll("g."+u.containerClassName).data(n.length>0?[0]:[]);if(a.enter().append("g").classed(u.containerClassName,!0).style("cursor","pointer"),a.exit().each(function(){r.select(this).selectAll("g."+u.headerGroupClassName).each(o)}).remove(),0!==n.length){var A=a.selectAll("g."+u.headerGroupClassName).data(n,h);A.enter().append("g").classed(u.headerGroupClassName,!0);for(var l=s.ensureSingle(a,"g",u.dropdownButtonGroupClassName,function(t){t.style("pointer-events","all")}),c=0;c<n.length;c++){var C=n[c];b(t,C)}var m="updatemenus"+e._uid,y=new g(t,l,m);A.enter().size()&&(l.node().parentNode.appendChild(l.node()),l.call(x)),A.exit().each(function(t){l.call(x),o(t)}).remove(),A.each(function(e){var n=r.select(this),o="dropdown"===e.type?l:null;i.manageCommandObserver(t,e,e.buttons,function(r){f(t,e,e.buttons[r.index],n,o,y,r.index,!0)}),"dropdown"===e.type?(p(t,n,l,y,e),d(l,e)&&I(t,n,l,y,e)):I(t,n,null,null,e)})}}},{"../../constants/alignment":688,"../../lib":719,"../../lib/svg_text_utils":743,"../../plot_api/plot_template":757,"../../plots/plots":828,"../color":593,"../drawing":614,"./constants":683,"./scrollbox":687,d3:163}],686:[function(t,e,n){arguments[4][680][0].apply(n,arguments)},{"./attributes":682,"./constants":683,"./defaults":684,"./draw":685,dup:680}],687:[function(t,e,n){"use strict";e.exports=s;var r=t("d3"),i=t("../color"),o=t("../drawing"),a=t("../../lib");function s(t,e,n){this.gd=t,this.container=e,this.id=n,this.position=null,this.translateX=null,this.translateY=null,this.hbar=null,this.vbar=null,this.bg=this.container.selectAll("rect.scrollbox-bg").data([0]),this.bg.exit().on(".drag",null).on("wheel",null).remove(),this.bg.enter().append("rect").classed("scrollbox-bg",!0).style("pointer-events","all").attr({opacity:0,x:0,y:0,width:0,height:0})}s.barWidth=2,s.barLength=20,s.barRadius=2,s.barPad=1,s.barColor="#808BA4",s.prototype.enable=function(t,e,n){var a=this.gd._fullLayout,A=a.width,l=a.height;this.position=t;var c,u,g,h,d=this.position.l,f=this.position.w,p=this.position.t,I=this.position.h,C=this.position.direction,m="down"===C,y="left"===C,E="up"===C,v=f,B=I;m||y||"right"===C||E||(this.position.direction="down",m=!0),m||E?(u=(c=d)+v,m?(g=p,B=(h=Math.min(g+B,l))-g):B=(h=p+B)-(g=Math.max(h-B,0))):(h=(g=p)+B,y?v=(u=d+v)-(c=Math.max(u-v,0)):(c=d,v=(u=Math.min(c+v,A))-c)),this._box={l:c,t:g,w:v,h:B};var b=f>v,Q=s.barLength+2*s.barPad,w=s.barWidth+2*s.barPad,x=d,_=p+I;_+w>l&&(_=l-w);var D=this.container.selectAll("rect.scrollbar-horizontal").data(b?[0]:[]);D.exit().on(".drag",null).remove(),D.enter().append("rect").classed("scrollbar-horizontal",!0).call(i.fill,s.barColor),b?(this.hbar=D.attr({rx:s.barRadius,ry:s.barRadius,x:x,y:_,width:Q,height:w}),this._hbarXMin=x+Q/2,this._hbarTranslateMax=v-Q):(delete this.hbar,delete this._hbarXMin,delete this._hbarTranslateMax);var M=I>B,k=s.barWidth+2*s.barPad,S=s.barLength+2*s.barPad,L=d+f,R=p;L+k>A&&(L=A-k);var N=this.container.selectAll("rect.scrollbar-vertical").data(M?[0]:[]);N.exit().on(".drag",null).remove(),N.enter().append("rect").classed("scrollbar-vertical",!0).call(i.fill,s.barColor),M?(this.vbar=N.attr({rx:s.barRadius,ry:s.barRadius,x:L,y:R,width:k,height:S}),this._vbarYMin=R+S/2,this._vbarTranslateMax=B-S):(delete this.vbar,delete this._vbarYMin,delete this._vbarTranslateMax);var F=this.id,T=c-.5,U=M?u+k+.5:u+.5,O=g-.5,G=b?h+w+.5:h+.5,H=a._topdefs.selectAll("#"+F).data(b||M?[0]:[]);if(H.exit().remove(),H.enter().append("clipPath").attr("id",F).append("rect"),b||M?(this._clipRect=H.select("rect").attr({x:Math.floor(T),y:Math.floor(O),width:Math.ceil(U)-Math.floor(T),height:Math.ceil(G)-Math.floor(O)}),this.container.call(o.setClipUrl,F,this.gd),this.bg.attr({x:d,y:p,width:f,height:I})):(this.bg.attr({width:0,height:0}),this.container.on("wheel",null).on(".drag",null).call(o.setClipUrl,null),delete this._clipRect),b||M){var j=r.behavior.drag().on("dragstart",function(){r.event.sourceEvent.preventDefault()}).on("drag",this._onBoxDrag.bind(this));this.container.on("wheel",null).on("wheel",this._onBoxWheel.bind(this)).on(".drag",null).call(j);var Y=r.behavior.drag().on("dragstart",function(){r.event.sourceEvent.preventDefault(),r.event.sourceEvent.stopPropagation()}).on("drag",this._onBarDrag.bind(this));b&&this.hbar.on(".drag",null).call(Y),M&&this.vbar.on(".drag",null).call(Y)}this.setTranslate(e,n)},s.prototype.disable=function(){(this.hbar||this.vbar)&&(this.bg.attr({width:0,height:0}),this.container.on("wheel",null).on(".drag",null).call(o.setClipUrl,null),delete this._clipRect),this.hbar&&(this.hbar.on(".drag",null),this.hbar.remove(),delete this.hbar,delete this._hbarXMin,delete this._hbarTranslateMax),this.vbar&&(this.vbar.on(".drag",null),this.vbar.remove(),delete this.vbar,delete this._vbarYMin,delete this._vbarTranslateMax)},s.prototype._onBoxDrag=function(){var t=this.translateX,e=this.translateY;this.hbar&&(t-=r.event.dx),this.vbar&&(e-=r.event.dy),this.setTranslate(t,e)},s.prototype._onBoxWheel=function(){var t=this.translateX,e=this.translateY;this.hbar&&(t+=r.event.deltaY),this.vbar&&(e+=r.event.deltaY),this.setTranslate(t,e)},s.prototype._onBarDrag=function(){var t=this.translateX,e=this.translateY;if(this.hbar){var n=t+this._hbarXMin,i=n+this._hbarTranslateMax;t=(a.constrain(r.event.x,n,i)-n)/(i-n)*(this.position.w-this._box.w)}if(this.vbar){var o=e+this._vbarYMin,s=o+this._vbarTranslateMax;e=(a.constrain(r.event.y,o,s)-o)/(s-o)*(this.position.h-this._box.h)}this.setTranslate(t,e)},s.prototype.setTranslate=function(t,e){var n=this.position.w-this._box.w,r=this.position.h-this._box.h;if(t=a.constrain(t||0,0,n),e=a.constrain(e||0,0,r),this.translateX=t,this.translateY=e,this.container.call(o.setTranslate,this._box.l-this.position.l-t,this._box.t-this.position.t-e),this._clipRect&&this._clipRect.attr({x:Math.floor(this.position.l+t-.5),y:Math.floor(this.position.t+e-.5)}),this.hbar){var i=t/n;this.hbar.call(o.setTranslate,t+i*this._hbarTranslateMax,e)}if(this.vbar){var s=e/r;this.vbar.call(o.setTranslate,t,e+s*this._vbarTranslateMax)}}},{"../../lib":719,"../color":593,"../drawing":614,d3:163}],688:[function(t,e,n){"use strict";e.exports={FROM_BL:{left:0,center:.5,right:1,bottom:0,middle:.5,top:1},FROM_TL:{left:0,center:.5,right:1,bottom:1,middle:.5,top:0},FROM_BR:{left:1,center:.5,right:0,bottom:0,middle:.5,top:1},LINE_SPACING:1.3,CAP_SHIFT:.7,MID_SHIFT:.35,OPPOSITE_SIDE:{left:"right",right:"left",top:"bottom",bottom:"top"}}},{}],689:[function(t,e,n){"use strict";e.exports={INCREASING:{COLOR:"#3D9970",SYMBOL:"▲"},DECREASING:{COLOR:"#FF4136",SYMBOL:"▼"}}},{}],690:[function(t,e,n){"use strict";e.exports={FORMAT_LINK:"https://github.com/d3/d3-3.x-api-reference/blob/master/Formatting.md#d3_format",DATE_FORMAT_LINK:"https://github.com/d3/d3-3.x-api-reference/blob/master/Time-Formatting.md#format"}},{}],691:[function(t,e,n){"use strict";e.exports={COMPARISON_OPS:["=","!=","<",">=",">","<="],COMPARISON_OPS2:["=","<",">=",">","<="],INTERVAL_OPS:["[]","()","[)","(]","][",")(","](",")["],SET_OPS:["{}","}{"],CONSTRAINT_REDUCTION:{"=":"=","<":"<","<=":"<",">":">",">=":">","[]":"[]","()":"[]","[)":"[]","(]":"[]","][":"][",")(":"][","](":"][",")[":"]["}}},{}],692:[function(t,e,n){"use strict";e.exports={solid:[[],0],dot:[[.5,1],200],dash:[[.5,1],50],longdash:[[.5,1],10],dashdot:[[.5,.625,.875,1],50],longdashdot:[[.5,.7,.8,1],10]}},{}],693:[function(t,e,n){"use strict";e.exports={circle:"●","circle-open":"○",square:"■","square-open":"□",diamond:"◆","diamond-open":"◇",cross:"+",x:"❌"}},{}],694:[function(t,e,n){"use strict";e.exports={SHOW_PLACEHOLDER:100,HIDE_PLACEHOLDER:1e3,DESELECTDIM:.2}},{}],695:[function(t,e,n){"use strict";e.exports={BADNUM:void 0,FP_SAFE:Number.MAX_VALUE/1e4,ONEAVGYEAR:315576e5,ONEAVGMONTH:26298e5,ONEDAY:864e5,ONEHOUR:36e5,ONEMIN:6e4,ONESEC:1e3,EPOCHJD:2440587.5,ALMOST_EQUAL:1-1e-6,LOG_CLIP:10,MINUS_SIGN:"−"}},{}],696:[function(t,e,n){"use strict";n.xmlns="http://www.w3.org/2000/xmlns/",n.svg="http://www.w3.org/2000/svg",n.xlink="http://www.w3.org/1999/xlink",n.svgAttrs={xmlns:n.svg,"xmlns:xlink":n.xlink}},{}],697:[function(t,e,n){"use strict";n.version="1.49.1",t("es6-promise").polyfill(),t("../build/plotcss"),t("./fonts/mathjax_config")();for(var r=t("./registry"),i=n.register=r.register,o=t("./plot_api"),a=Object.keys(o),s=0;s<a.length;s++){var A=a[s];"_"!==A.charAt(0)&&(n[A]=o[A]),i({moduleType:"apiMethod",name:A,fn:o[A]})}i(t("./traces/scatter")),i([t("./components/fx"),t("./components/legend"),t("./components/annotations"),t("./components/annotations3d"),t("./components/shapes"),t("./components/images"),t("./components/updatemenus"),t("./components/sliders"),t("./components/rangeslider"),t("./components/rangeselector"),t("./components/grid"),t("./components/errorbars"),t("./components/colorscale"),t("./components/colorbar")]),i([t("./locale-en"),t("./locale-en-us")]),n.Icons=t("./fonts/ploticon"),n.Plots=t("./plots/plots"),n.Fx=t("./components/fx"),n.Snapshot=t("./snapshot"),n.PlotSchema=t("./plot_api/plot_schema"),n.Queue=t("./lib/queue"),n.d3=t("d3")},{"../build/plotcss":1,"./components/annotations":584,"./components/annotations3d":589,"./components/colorbar":599,"./components/colorscale":605,"./components/errorbars":620,"./components/fx":632,"./components/grid":636,"./components/images":641,"./components/legend":649,"./components/rangeselector":660,"./components/rangeslider":667,"./components/shapes":675,"./components/sliders":680,"./components/updatemenus":686,"./fonts/mathjax_config":698,"./fonts/ploticon":699,"./lib/queue":734,"./locale-en":748,"./locale-en-us":747,"./plot_api":752,"./plot_api/plot_schema":756,"./plots/plots":828,"./registry":847,"./snapshot":852,"./traces/scatter":1123,d3:163,"es6-promise":218}],698:[function(t,e,n){"use strict";e.exports=function(){"undefined"!=typeof MathJax&&"local"!==(window.PlotlyConfig||{}).MathJaxConfig&&(MathJax.Hub.Config({messageStyle:"none",skipStartupTypeset:!0,displayAlign:"left",tex2jax:{inlineMath:[["$","$"],["\\(","\\)"]]}}),MathJax.Hub.Configured())}},{}],699:[function(t,e,n){"use strict";e.exports={undo:{width:857.1,height:1e3,path:"m857 350q0-87-34-166t-91-137-137-92-166-34q-96 0-183 41t-147 114q-4 6-4 13t5 11l76 77q6 5 14 5 9-1 13-7 41-53 100-82t126-29q58 0 110 23t92 61 61 91 22 111-22 111-61 91-92 61-110 23q-55 0-105-20t-90-57l77-77q17-16 8-38-10-23-33-23h-250q-15 0-25 11t-11 25v250q0 24 22 33 22 10 39-8l72-72q60 57 137 88t159 31q87 0 166-34t137-92 91-137 34-166z",transform:"matrix(1 0 0 -1 0 850)"},home:{width:928.6,height:1e3,path:"m786 296v-267q0-15-11-26t-25-10h-214v214h-143v-214h-214q-15 0-25 10t-11 26v267q0 1 0 2t0 2l321 264 321-264q1-1 1-4z m124 39l-34-41q-5-5-12-6h-2q-7 0-12 3l-386 322-386-322q-7-4-13-4-7 2-12 7l-35 41q-4 5-3 13t6 12l401 334q18 15 42 15t43-15l136-114v109q0 8 5 13t13 5h107q8 0 13-5t5-13v-227l122-102q5-5 6-12t-4-13z",transform:"matrix(1 0 0 -1 0 850)"},"camera-retro":{width:1e3,height:1e3,path:"m518 386q0 8-5 13t-13 5q-37 0-63-27t-26-63q0-8 5-13t13-5 12 5 5 13q0 23 16 38t38 16q8 0 13 5t5 13z m125-73q0-59-42-101t-101-42-101 42-42 101 42 101 101 42 101-42 42-101z m-572-320h858v71h-858v-71z m643 320q0 89-62 152t-152 62-151-62-63-152 63-151 151-63 152 63 62 151z m-571 358h214v72h-214v-72z m-72-107h858v143h-462l-36-71h-360v-72z m929 143v-714q0-30-21-51t-50-21h-858q-29 0-50 21t-21 51v714q0 30 21 51t50 21h858q29 0 50-21t21-51z",transform:"matrix(1 0 0 -1 0 850)"},zoombox:{width:1e3,height:1e3,path:"m1000-25l-250 251c40 63 63 138 63 218 0 224-182 406-407 406-224 0-406-182-406-406s183-406 407-406c80 0 155 22 218 62l250-250 125 125z m-812 250l0 438 437 0 0-438-437 0z m62 375l313 0 0-312-313 0 0 312z",transform:"matrix(1 0 0 -1 0 850)"},pan:{width:1e3,height:1e3,path:"m1000 350l-187 188 0-125-250 0 0 250 125 0-188 187-187-187 125 0 0-250-250 0 0 125-188-188 186-187 0 125 252 0 0-250-125 0 187-188 188 188-125 0 0 250 250 0 0-126 187 188z",transform:"matrix(1 0 0 -1 0 850)"},zoom_plus:{width:875,height:1e3,path:"m1 787l0-875 875 0 0 875-875 0z m687-500l-187 0 0-187-125 0 0 187-188 0 0 125 188 0 0 187 125 0 0-187 187 0 0-125z",transform:"matrix(1 0 0 -1 0 850)"},zoom_minus:{width:875,height:1e3,path:"m0 788l0-876 875 0 0 876-875 0z m688-500l-500 0 0 125 500 0 0-125z",transform:"matrix(1 0 0 -1 0 850)"},autoscale:{width:1e3,height:1e3,path:"m250 850l-187 0-63 0 0-62 0-188 63 0 0 188 187 0 0 62z m688 0l-188 0 0-62 188 0 0-188 62 0 0 188 0 62-62 0z m-875-938l0 188-63 0 0-188 0-62 63 0 187 0 0 62-187 0z m875 188l0-188-188 0 0-62 188 0 62 0 0 62 0 188-62 0z m-125 188l-1 0-93-94-156 156 156 156 92-93 2 0 0 250-250 0 0-2 93-92-156-156-156 156 94 92 0 2-250 0 0-250 0 0 93 93 157-156-157-156-93 94 0 0 0-250 250 0 0 0-94 93 156 157 156-157-93-93 0 0 250 0 0 250z",transform:"matrix(1 0 0 -1 0 850)"},tooltip_basic:{width:1500,height:1e3,path:"m375 725l0 0-375-375 375-374 0-1 1125 0 0 750-1125 0z",transform:"matrix(1 0 0 -1 0 850)"},tooltip_compare:{width:1125,height:1e3,path:"m187 786l0 2-187-188 188-187 0 0 937 0 0 373-938 0z m0-499l0 1-187-188 188-188 0 0 937 0 0 376-938-1z",transform:"matrix(1 0 0 -1 0 850)"},plotlylogo:{width:1542,height:1e3,path:"m0-10h182v-140h-182v140z m228 146h183v-286h-183v286z m225 714h182v-1000h-182v1000z m225-285h182v-715h-182v715z m225 142h183v-857h-183v857z m231-428h182v-429h-182v429z m225-291h183v-138h-183v138z",transform:"matrix(1 0 0 -1 0 850)"},"z-axis":{width:1e3,height:1e3,path:"m833 5l-17 108v41l-130-65 130-66c0 0 0 38 0 39 0-1 36-14 39-25 4-15-6-22-16-30-15-12-39-16-56-20-90-22-187-23-279-23-261 0-341 34-353 59 3 60 228 110 228 110-140-8-351-35-351-116 0-120 293-142 474-142 155 0 477 22 477 142 0 50-74 79-163 96z m-374 94c-58-5-99-21-99-40 0-24 65-43 144-43 79 0 143 19 143 43 0 19-42 34-98 40v216h87l-132 135-133-135h88v-216z m167 515h-136v1c16 16 31 34 46 52l84 109v54h-230v-71h124v-1c-16-17-28-32-44-51l-89-114v-51h245v72z",transform:"matrix(1 0 0 -1 0 850)"},"3d_rotate":{width:1e3,height:1e3,path:"m922 660c-5 4-9 7-14 11-359 263-580-31-580-31l-102 28 58-400c0 1 1 1 2 2 118 108 351 249 351 249s-62 27-100 42c88 83 222 183 347 122 16-8 30-17 44-27-2 1-4 2-6 4z m36-329c0 0 64 229-88 296-62 27-124 14-175-11 157-78 225-208 249-266 8-19 11-31 11-31 2 5 6 15 11 32-5-13-8-20-8-20z m-775-239c70-31 117-50 198-32-121 80-199 346-199 346l-96-15-58-12c0 0 55-226 155-287z m603 133l-317-139c0 0 4-4 19-14 7-5 24-15 24-15s-177-147-389 4c235-287 536-112 536-112l31-22 100 299-4-1z m-298-153c6-4 14-9 24-15 0 0-17 10-24 15z",transform:"matrix(1 0 0 -1 0 850)"},camera:{width:1e3,height:1e3,path:"m500 450c-83 0-150-67-150-150 0-83 67-150 150-150 83 0 150 67 150 150 0 83-67 150-150 150z m400 150h-120c-16 0-34 13-39 29l-31 93c-6 15-23 28-40 28h-340c-16 0-34-13-39-28l-31-94c-6-15-23-28-40-28h-120c-55 0-100-45-100-100v-450c0-55 45-100 100-100h800c55 0 100 45 100 100v450c0 55-45 100-100 100z m-400-550c-138 0-250 112-250 250 0 138 112 250 250 250 138 0 250-112 250-250 0-138-112-250-250-250z m365 380c-19 0-35 16-35 35 0 19 16 35 35 35 19 0 35-16 35-35 0-19-16-35-35-35z",transform:"matrix(1 0 0 -1 0 850)"},movie:{width:1e3,height:1e3,path:"m938 413l-188-125c0 37-17 71-44 94 64 38 107 107 107 187 0 121-98 219-219 219-121 0-219-98-219-219 0-61 25-117 66-156h-115c30 33 49 76 49 125 0 103-84 187-187 187s-188-84-188-187c0-57 26-107 65-141-38-22-65-62-65-109v-250c0-70 56-126 125-126h500c69 0 125 56 125 126l188-126c34 0 62 28 62 63v375c0 35-28 63-62 63z m-750 0c-69 0-125 56-125 125s56 125 125 125 125-56 125-125-56-125-125-125z m406-1c-87 0-157 70-157 157 0 86 70 156 157 156s156-70 156-156-70-157-156-157z",transform:"matrix(1 0 0 -1 0 850)"},question:{width:857.1,height:1e3,path:"m500 82v107q0 8-5 13t-13 5h-107q-8 0-13-5t-5-13v-107q0-8 5-13t13-5h107q8 0 13 5t5 13z m143 375q0 49-31 91t-77 65-95 23q-136 0-207-119-9-14 4-24l74-55q4-4 10-4 9 0 14 7 30 38 48 51 19 14 48 14 27 0 48-15t21-33q0-21-11-34t-38-25q-35-16-65-48t-29-70v-20q0-8 5-13t13-5h107q8 0 13 5t5 13q0 10 12 27t30 28q18 10 28 16t25 19 25 27 16 34 7 45z m214-107q0-117-57-215t-156-156-215-58-216 58-155 156-58 215 58 215 155 156 216 58 215-58 156-156 57-215z",transform:"matrix(1 0 0 -1 0 850)"},disk:{width:857.1,height:1e3,path:"m214-7h429v214h-429v-214z m500 0h72v500q0 8-6 21t-11 20l-157 156q-5 6-19 12t-22 5v-232q0-22-15-38t-38-16h-322q-22 0-37 16t-16 38v232h-72v-714h72v232q0 22 16 38t37 16h465q22 0 38-16t15-38v-232z m-214 518v178q0 8-5 13t-13 5h-107q-7 0-13-5t-5-13v-178q0-8 5-13t13-5h107q7 0 13 5t5 13z m357-18v-518q0-22-15-38t-38-16h-750q-23 0-38 16t-16 38v750q0 22 16 38t38 16h517q23 0 50-12t42-26l156-157q16-15 27-42t11-49z",transform:"matrix(1 0 0 -1 0 850)"},lasso:{width:1031,height:1e3,path:"m1018 538c-36 207-290 336-568 286-277-48-473-256-436-463 10-57 36-108 76-151-13-66 11-137 68-183 34-28 75-41 114-42l-55-70 0 0c-2-1-3-2-4-3-10-14-8-34 5-45 14-11 34-8 45 4 1 1 2 3 2 5l0 0 113 140c16 11 31 24 45 40 4 3 6 7 8 11 48-3 100 0 151 9 278 48 473 255 436 462z m-624-379c-80 14-149 48-197 96 42 42 109 47 156 9 33-26 47-66 41-105z m-187-74c-19 16-33 37-39 60 50-32 109-55 174-68-42-25-95-24-135 8z m360 75c-34-7-69-9-102-8 8 62-16 128-68 170-73 59-175 54-244-5-9 20-16 40-20 61-28 159 121 317 333 354s407-60 434-217c28-159-121-318-333-355z",transform:"matrix(1 0 0 -1 0 850)"},selectbox:{width:1e3,height:1e3,path:"m0 850l0-143 143 0 0 143-143 0z m286 0l0-143 143 0 0 143-143 0z m285 0l0-143 143 0 0 143-143 0z m286 0l0-143 143 0 0 143-143 0z m-857-286l0-143 143 0 0 143-143 0z m857 0l0-143 143 0 0 143-143 0z m-857-285l0-143 143 0 0 143-143 0z m857 0l0-143 143 0 0 143-143 0z m-857-286l0-143 143 0 0 143-143 0z m286 0l0-143 143 0 0 143-143 0z m285 0l0-143 143 0 0 143-143 0z m286 0l0-143 143 0 0 143-143 0z",transform:"matrix(1 0 0 -1 0 850)"},spikeline:{width:1e3,height:1e3,path:"M512 409c0-57-46-104-103-104-57 0-104 47-104 104 0 57 47 103 104 103 57 0 103-46 103-103z m-327-39l92 0 0 92-92 0z m-185 0l92 0 0 92-92 0z m370-186l92 0 0 93-92 0z m0-184l92 0 0 92-92 0z",transform:"matrix(1.5 0 0 -1.5 0 850)"},pencil:{width:1792,height:1792,path:"M491 1536l91-91-235-235-91 91v107h128v128h107zm523-928q0-22-22-22-10 0-17 7l-542 542q-7 7-7 17 0 22 22 22 10 0 17-7l542-542q7-7 7-17zm-54-192l416 416-832 832h-416v-416zm683 96q0 53-37 90l-166 166-416-416 166-165q36-38 90-38 53 0 91 38l235 234q37 39 37 91z",transform:"matrix(1 0 0 1 0 1)"},newplotlylogo:{name:"newplotlylogo",svg:"<svg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 132 132'><defs><style>.cls-1 {fill: #119dff;} .cls-2 {fill: #25fefd;} .cls-3 {fill: #fff;}</style></defs><title>plotly-logomark</title><g id='symbol'><rect class='cls-1' width='132' height='132' rx='6' ry='6'/><circle class='cls-2' cx='78' cy='54' r='6'/><circle class='cls-2' cx='102' cy='30' r='6'/><circle class='cls-2' cx='78' cy='30' r='6'/><circle class='cls-2' cx='54' cy='30' r='6'/><circle class='cls-2' cx='30' cy='30' r='6'/><circle class='cls-2' cx='30' cy='54' r='6'/><path class='cls-3' d='M30,72a6,6,0,0,0-6,6v24a6,6,0,0,0,12,0V78A6,6,0,0,0,30,72Z'/><path class='cls-3' d='M78,72a6,6,0,0,0-6,6v24a6,6,0,0,0,12,0V78A6,6,0,0,0,78,72Z'/><path class='cls-3' d='M54,48a6,6,0,0,0-6,6v48a6,6,0,0,0,12,0V54A6,6,0,0,0,54,48Z'/><path class='cls-3' d='M102,48a6,6,0,0,0-6,6v48a6,6,0,0,0,12,0V54A6,6,0,0,0,102,48Z'/></g></svg>"}}},{}],700:[function(t,e,n){"use strict";n.isLeftAnchor=function(t){return"left"===t.xanchor||"auto"===t.xanchor&&t.x<=1/3},n.isCenterAnchor=function(t){return"center"===t.xanchor||"auto"===t.xanchor&&t.x>1/3&&t.x<2/3},n.isRightAnchor=function(t){return"right"===t.xanchor||"auto"===t.xanchor&&t.x>=2/3},n.isTopAnchor=function(t){return"top"===t.yanchor||"auto"===t.yanchor&&t.y>=2/3},n.isMiddleAnchor=function(t){return"middle"===t.yanchor||"auto"===t.yanchor&&t.y>1/3&&t.y<2/3},n.isBottomAnchor=function(t){return"bottom"===t.yanchor||"auto"===t.yanchor&&t.y<=1/3}},{}],701:[function(t,e,n){"use strict";var r=t("./mod"),i=r.mod,o=r.modHalf,a=Math.PI,s=2*a;function A(t){return Math.abs(t[1]-t[0])>s-1e-14}function l(t,e){return o(e-t,s)}function c(t,e){if(A(e))return!0;var n,r;e[0]<e[1]?(n=e[0],r=e[1]):(n=e[1],r=e[0]),(n=i(n,s))>(r=i(r,s))&&(r+=s);var o=i(t,s),a=o+s;return o>=n&&o<=r||a>=n&&a<=r}function u(t,e,n,r,i,o,l){i=i||0,o=o||0;var c,u,g,h,d,f=A([n,r]);function p(t,e){return[t*Math.cos(e)+i,o-t*Math.sin(e)]}f?(c=0,u=a,g=s):n<r?(c=n,g=r):(c=r,g=n),t<e?(h=t,d=e):(h=e,d=t);var I,C=Math.abs(g-c)<=a?0:1;function m(t,e,n){return"A"+[t,t]+" "+[0,C,n]+" "+p(t,e)}return f?I=null===h?"M"+p(d,c)+m(d,u,0)+m(d,g,0)+"Z":"M"+p(h,c)+m(h,u,0)+m(h,g,0)+"ZM"+p(d,c)+m(d,u,1)+m(d,g,1)+"Z":null===h?(I="M"+p(d,c)+m(d,g,0),l&&(I+="L0,0Z")):I="M"+p(h,c)+"L"+p(d,c)+m(d,g,0)+"L"+p(h,g)+m(h,c,1)+"Z",I}e.exports={deg2rad:function(t){return t/180*a},rad2deg:function(t){return t/a*180},angleDelta:l,angleDist:function(t,e){return Math.abs(l(t,e))},isFullCircle:A,isAngleInsideSector:c,isPtInsideSector:function(t,e,n,r){return!!c(e,r)&&(n[0]<n[1]?(i=n[0],o=n[1]):(i=n[1],o=n[0]),t>=i&&t<=o);var i,o},pathArc:function(t,e,n,r,i){return u(null,t,e,n,r,i,0)},pathSector:function(t,e,n,r,i){return u(null,t,e,n,r,i,1)},pathAnnulus:function(t,e,n,r,i,o){return u(t,e,n,r,i,o,1)}}},{"./mod":726}],702:[function(t,e,n){"use strict";var r=Array.isArray,i="undefined"!=typeof ArrayBuffer&&ArrayBuffer.isView?ArrayBuffer:{isView:function(){return!1}},o="undefined"==typeof DataView?function(){}:DataView;function a(t){return i.isView(t)&&!(t instanceof o)}function s(t){return r(t)||a(t)}function A(t,e,n){if(s(t)){if(s(t[0])){for(var r=n,i=0;i<t.length;i++)r=e(r,t[i].length);return r}return t.length}return 0}n.isTypedArray=a,n.isArrayOrTypedArray=s,n.isArray1D=function(t){return!s(t[0])},n.ensureArray=function(t,e){return r(t)||(t=[]),t.length=e,t},n.concat=function(){var t,e,n,i,o,a,s,A,l=[],c=!0,u=0;for(n=0;n<arguments.length;n++)(a=(i=arguments[n]).length)&&(e?l.push(i):(e=i,o=a),r(i)?t=!1:(c=!1,u?t!==i.constructor&&(t=!1):t=i.constructor),u+=a);if(!u)return[];if(!l.length)return e;if(c)return e.concat.apply(e,l);if(t){for((s=new t(u)).set(e),n=0;n<l.length;n++)i=l[n],s.set(i,o),o+=i.length;return s}for(s=new Array(u),A=0;A<e.length;A++)s[A]=e[A];for(n=0;n<l.length;n++){for(i=l[n],A=0;A<i.length;A++)s[o+A]=i[A];o+=A}return s},n.maxRowLength=function(t){return A(t,Math.max,0)},n.minRowLength=function(t){return A(t,Math.min,1/0)}},{}],703:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../constants/numerical").BADNUM,o=/^['"%,$#\s']+|[, ]|['"%,$#\s']+$/g;e.exports=function(t){return"string"==typeof t&&(t=t.replace(o,"")),r(t)?Number(t):i}},{"../constants/numerical":695,"fast-isnumeric":225}],704:[function(t,e,n){"use strict";e.exports=function(t){var e=t._fullLayout;e._glcanvas&&e._glcanvas.size()&&e._glcanvas.each(function(t){t.regl&&t.regl.clear({color:!0,depth:!0})})}},{}],705:[function(t,e,n){"use strict";e.exports=function(t){t._responsiveChartHandler&&(window.removeEventListener("resize",t._responsiveChartHandler),delete t._responsiveChartHandler)}},{}],706:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("tinycolor2"),o=t("../plots/attributes"),a=t("../components/colorscale/scales"),s=t("../constants/interactions").DESELECTDIM,A=t("./nested_property"),l=t("./regex").counter,c=t("./mod").modHalf,u=t("./array").isArrayOrTypedArray;function g(t,e){var r=n.valObjectMeta[e.valType];if(e.arrayOk&&u(t))return!0;if(r.validateFunction)return r.validateFunction(t,e);var i={},o=i,a={set:function(t){o=t}};return r.coerceFunction(t,a,i,e),o!==i}n.valObjectMeta={data_array:{coerceFunction:function(t,e,n){u(t)?e.set(t):void 0!==n&&e.set(n)}},enumerated:{coerceFunction:function(t,e,n,r){r.coerceNumber&&(t=+t),-1===r.values.indexOf(t)?e.set(n):e.set(t)},validateFunction:function(t,e){e.coerceNumber&&(t=+t);for(var n=e.values,r=0;r<n.length;r++){var i=String(n[r]);if("/"===i.charAt(0)&&"/"===i.charAt(i.length-1)){if(new RegExp(i.substr(1,i.length-2)).test(t))return!0}else if(t===n[r])return!0}return!1}},boolean:{coerceFunction:function(t,e,n){!0===t||!1===t?e.set(t):e.set(n)}},number:{coerceFunction:function(t,e,n,i){!r(t)||void 0!==i.min&&t<i.min||void 0!==i.max&&t>i.max?e.set(n):e.set(+t)}},integer:{coerceFunction:function(t,e,n,i){t%1||!r(t)||void 0!==i.min&&t<i.min||void 0!==i.max&&t>i.max?e.set(n):e.set(+t)}},string:{coerceFunction:function(t,e,n,r){if("string"!=typeof t){var i="number"==typeof t;!0!==r.strict&&i?e.set(String(t)):e.set(n)}else r.noBlank&&!t?e.set(n):e.set(t)}},color:{coerceFunction:function(t,e,n){i(t).isValid()?e.set(t):e.set(n)}},colorlist:{coerceFunction:function(t,e,n){Array.isArray(t)&&t.length&&t.every(function(t){return i(t).isValid()})?e.set(t):e.set(n)}},colorscale:{coerceFunction:function(t,e,n){e.set(a.get(t,n))}},angle:{coerceFunction:function(t,e,n){"auto"===t?e.set("auto"):r(t)?e.set(c(+t,360)):e.set(n)}},subplotid:{coerceFunction:function(t,e,n,r){var i=r.regex||l(n);"string"==typeof t&&i.test(t)?e.set(t):e.set(n)},validateFunction:function(t,e){var n=e.dflt;return t===n||"string"==typeof t&&!!l(n).test(t)}},flaglist:{coerceFunction:function(t,e,n,r){if("string"==typeof t)if(-1===(r.extras||[]).indexOf(t)){for(var i=t.split("+"),o=0;o<i.length;){var a=i[o];-1===r.flags.indexOf(a)||i.indexOf(a)<o?i.splice(o,1):o++}i.length?e.set(i.join("+")):e.set(n)}else e.set(t);else e.set(n)}},any:{coerceFunction:function(t,e,n){void 0===t?e.set(n):e.set(t)}},info_array:{coerceFunction:function(t,e,r,i){function o(t,e,r){var i,o={set:function(t){i=t}};return void 0===r&&(r=e.dflt),n.valObjectMeta[e.valType].coerceFunction(t,o,r,e),i}var a=2===i.dimensions||"1-2"===i.dimensions&&Array.isArray(t)&&Array.isArray(t[0]);if(Array.isArray(t)){var s,A,l,c,u,g,h=i.items,d=[],f=Array.isArray(h),p=f&&a&&Array.isArray(h[0]),I=a&&f&&!p,C=f&&!I?h.length:t.length;if(r=Array.isArray(r)?r:[],a)for(s=0;s<C;s++)for(d[s]=[],l=Array.isArray(t[s])?t[s]:[],u=I?h.length:f?h[s].length:l.length,A=0;A<u;A++)c=I?h[A]:f?h[s][A]:h,void 0!==(g=o(l[A],c,(r[s]||[])[A]))&&(d[s][A]=g);else for(s=0;s<C;s++)void 0!==(g=o(t[s],f?h[s]:h,r[s]))&&(d[s]=g);e.set(d)}else e.set(r)},validateFunction:function(t,e){if(!Array.isArray(t))return!1;var n=e.items,r=Array.isArray(n),i=2===e.dimensions;if(!e.freeLength&&t.length!==n.length)return!1;for(var o=0;o<t.length;o++)if(i){if(!Array.isArray(t[o])||!e.freeLength&&t[o].length!==n[o].length)return!1;for(var a=0;a<t[o].length;a++)if(!g(t[o][a],r?n[o][a]:n))return!1}else if(!g(t[o],r?n[o]:n))return!1;return!0}}},n.coerce=function(t,e,r,i,o){var a=A(r,i).get(),s=A(t,i),l=A(e,i),c=s.get(),h=e._template;if(void 0===c&&h&&(c=A(h,i).get(),h=0),void 0===o&&(o=a.dflt),a.arrayOk&&u(c))return l.set(c),c;var d=n.valObjectMeta[a.valType].coerceFunction;d(c,l,o,a);var f=l.get();return h&&f===o&&!g(c,a)&&(d(c=A(h,i).get(),l,o,a),f=l.get()),f},n.coerce2=function(t,e,r,i,o){var a=A(t,i),s=n.coerce(t,e,r,i,o),l=a.get();return null!=l&&s},n.coerceFont=function(t,e,n){var r={};return n=n||{},r.family=t(e+".family",n.family),r.size=t(e+".size",n.size),r.color=t(e+".color",n.color),r},n.coerceHoverinfo=function(t,e,r){var i,a=e._module.attributes,s=a.hoverinfo?a:o,A=s.hoverinfo;if(1===r._dataLength){var l="all"===A.dflt?A.flags.slice():A.dflt.split("+");l.splice(l.indexOf("name"),1),i=l.join("+")}return n.coerce(t,e,s,"hoverinfo",i)},n.coerceSelectionMarkerOpacity=function(t,e){if(t.marker){var n,r,i=t.marker.opacity;void 0!==i&&(u(i)||t.selected||t.unselected||(n=i,r=s*i),e("selected.marker.opacity",n),e("unselected.marker.opacity",r))}},n.validate=g},{"../components/colorscale/scales":608,"../constants/interactions":694,"../plots/attributes":764,"./array":702,"./mod":726,"./nested_property":727,"./regex":735,"fast-isnumeric":225,tinycolor2:537}],707:[function(t,e,n){"use strict";var r,i,o=t("d3"),a=t("fast-isnumeric"),s=t("./loggers"),A=t("./mod").mod,l=t("../constants/numerical"),c=l.BADNUM,u=l.ONEDAY,g=l.ONEHOUR,h=l.ONEMIN,d=l.ONESEC,f=l.EPOCHJD,p=t("../registry"),I=o.time.format.utc,C=/^\s*(-?\d\d\d\d|\d\d)(-(\d?\d)(-(\d?\d)([ Tt]([01]?\d|2[0-3])(:([0-5]\d)(:([0-5]\d(\.\d+)?))?(Z|z|[+\-]\d\d:?\d\d)?)?)?)?)?\s*$/m,m=/^\s*(-?\d\d\d\d|\d\d)(-(\d?\di?)(-(\d?\d)([ Tt]([01]?\d|2[0-3])(:([0-5]\d)(:([0-5]\d(\.\d+)?))?(Z|z|[+\-]\d\d:?\d\d)?)?)?)?)?\s*$/m,y=(new Date).getFullYear()-70;function E(t){return t&&p.componentsRegistry.calendars&&"string"==typeof t&&"gregorian"!==t}function v(t,e){return String(t+Math.pow(10,e)).substr(1)}n.dateTick0=function(t,e){return E(t)?e?p.getComponentMethod("calendars","CANONICAL_SUNDAY")[t]:p.getComponentMethod("calendars","CANONICAL_TICK")[t]:e?"2000-01-02":"2000-01-01"},n.dfltRange=function(t){return E(t)?p.getComponentMethod("calendars","DFLTRANGE")[t]:["2000-01-01","2001-01-01"]},n.isJSDate=function(t){return"object"==typeof t&&null!==t&&"function"==typeof t.getTime},n.dateTime2ms=function(t,e){if(n.isJSDate(t)){var o=t.getTimezoneOffset()*h,a=(t.getUTCMinutes()-t.getMinutes())*h+(t.getUTCSeconds()-t.getSeconds())*d+(t.getUTCMilliseconds()-t.getMilliseconds());if(a){var s=3*h;o=o-s/2+A(a-o+s/2,s)}return(t=Number(t)-o)>=r&&t<=i?t:c}if("string"!=typeof t&&"number"!=typeof t)return c;t=String(t);var l=E(e),I=t.charAt(0);!l||"G"!==I&&"g"!==I||(t=t.substr(1),e="");var v=l&&"chinese"===e.substr(0,7),B=t.match(v?m:C);if(!B)return c;var b=B[1],Q=B[3]||"1",w=Number(B[5]||1),x=Number(B[7]||0),_=Number(B[9]||0),D=Number(B[11]||0);if(l){if(2===b.length)return c;var M;b=Number(b);try{var k=p.getComponentMethod("calendars","getCal")(e);if(v){var S="i"===Q.charAt(Q.length-1);Q=parseInt(Q,10),M=k.newDate(b,k.toMonthIndex(b,Q,S),w)}else M=k.newDate(b,Number(Q),w)}catch(t){return c}return M?(M.toJD()-f)*u+x*g+_*h+D*d:c}b=2===b.length?(Number(b)+2e3-y)%100+y:Number(b),Q-=1;var L=new Date(Date.UTC(2e3,Q,w,x,_));return L.setUTCFullYear(b),L.getUTCMonth()!==Q?c:L.getUTCDate()!==w?c:L.getTime()+D*d},r=n.MIN_MS=n.dateTime2ms("-9999"),i=n.MAX_MS=n.dateTime2ms("9999-12-31 23:59:59.9999"),n.isDateTime=function(t,e){return n.dateTime2ms(t,e)!==c};var B=90*u,b=3*g,Q=5*h;function w(t,e,n,r,i){if((e||n||r||i)&&(t+=" "+v(e,2)+":"+v(n,2),(r||i)&&(t+=":"+v(r,2),i))){for(var o=4;i%10==0;)o-=1,i/=10;t+="."+v(i,o)}return t}n.ms2DateTime=function(t,e,n){if("number"!=typeof t||!(t>=r&&t<=i))return c;e||(e=0);var o,a,s,l,C,m,y=Math.floor(10*A(t+.05,1)),v=Math.round(t-y/10);if(E(n)){var x=Math.floor(v/u)+f,_=Math.floor(A(t,u));try{o=p.getComponentMethod("calendars","getCal")(n).fromJD(x).formatDate("yyyy-mm-dd")}catch(t){o=I("G%Y-%m-%d")(new Date(v))}if("-"===o.charAt(0))for(;o.length<11;)o="-0"+o.substr(1);else for(;o.length<10;)o="0"+o;a=e<B?Math.floor(_/g):0,s=e<B?Math.floor(_%g/h):0,l=e<b?Math.floor(_%h/d):0,C=e<Q?_%d*10+y:0}else m=new Date(v),o=I("%Y-%m-%d")(m),a=e<B?m.getUTCHours():0,s=e<B?m.getUTCMinutes():0,l=e<b?m.getUTCSeconds():0,C=e<Q?10*m.getUTCMilliseconds()+y:0;return w(o,a,s,l,C)},n.ms2DateTimeLocal=function(t){if(!(t>=r+u&&t<=i-u))return c;var e=Math.floor(10*A(t+.05,1)),n=new Date(Math.round(t-e/10));return w(o.time.format("%Y-%m-%d")(n),n.getHours(),n.getMinutes(),n.getSeconds(),10*n.getUTCMilliseconds()+e)},n.cleanDate=function(t,e,r){if(t===c)return e;if(n.isJSDate(t)||"number"==typeof t&&isFinite(t)){if(E(r))return s.error("JS Dates and milliseconds are incompatible with world calendars",t),e;if(!(t=n.ms2DateTimeLocal(+t))&&void 0!==e)return e}else if(!n.isDateTime(t,r))return s.error("unrecognized date",t),e;return t};var x=/%\d?f/g;function _(t,e,n,r){t=t.replace(x,function(t){var n=Math.min(+t.charAt(1)||6,6);return(e/1e3%1+2).toFixed(n).substr(2).replace(/0+$/,"")||"0"});var i=new Date(Math.floor(e+.05));if(E(r))try{t=p.getComponentMethod("calendars","worldCalFmt")(t,e,r)}catch(t){return"Invalid"}return n(t)(i)}var D=[59,59.9,59.99,59.999,59.9999];n.formatDate=function(t,e,n,r,i,o){if(i=E(i)&&i,!e)if("y"===n)e=o.year;else if("m"===n)e=o.month;else{if("d"!==n)return function(t,e){var n=A(t+.05,u),r=v(Math.floor(n/g),2)+":"+v(A(Math.floor(n/h),60),2);if("M"!==e){a(e)||(e=0);var i=(100+Math.min(A(t/d,60),D[e])).toFixed(e).substr(1);e>0&&(i=i.replace(/0+$/,"").replace(/[\.]$/,"")),r+=":"+i}return r}(t,n)+"\n"+_(o.dayMonthYear,t,r,i);e=o.dayMonth+"\n"+o.year}return _(e,t,r,i)};var M=3*u;n.incrementMonth=function(t,e,n){n=E(n)&&n;var r=A(t,u);if(t=Math.round(t-r),n)try{var i=Math.round(t/u)+f,o=p.getComponentMethod("calendars","getCal")(n),a=o.fromJD(i);return e%12?o.add(a,e,"m"):o.add(a,e/12,"y"),(a.toJD()-f)*u+r}catch(e){s.error("invalid ms "+t+" in calendar "+n)}var l=new Date(t+M);return l.setUTCMonth(l.getUTCMonth()+e)+r-M},n.findExactDates=function(t,e){for(var n,r,i=0,o=0,s=0,A=0,l=E(e)&&p.getComponentMethod("calendars","getCal")(e),c=0;c<t.length;c++)if(r=t[c],a(r)){if(!(r%u))if(l)try{1===(n=l.fromJD(r/u+f)).day()?1===n.month()?i++:o++:s++}catch(t){}else 1===(n=new Date(r)).getUTCDate()?0===n.getUTCMonth()?i++:o++:s++}else A++;s+=o+=i;var g=t.length-A;return{exactYears:i/g,exactMonths:o/g,exactDays:s/g}}},{"../constants/numerical":695,"../registry":847,"./loggers":723,"./mod":726,d3:163,"fast-isnumeric":225}],708:[function(t,e,n){"use strict";var r=t("d3"),i=t("./loggers");function o(t){var e=t&&t.parentNode;e&&e.removeChild(t)}function a(t,e,n){var r="plotly.js-style-"+t,o=document.getElementById(r);o||((o=document.createElement("style")).setAttribute("id",r),o.appendChild(document.createTextNode("")),document.head.appendChild(o));var a=o.sheet;a.insertRule?a.insertRule(e+"{"+n+"}",0):a.addRule?a.addRule(e,n,0):i.warn("addStyleRule failed")}e.exports={getGraphDiv:function(t){var e;if("string"==typeof t){if(null===(e=document.getElementById(t)))throw new Error("No DOM element with id '"+t+"' exists on the page.");return e}if(null==t)throw new Error("DOM element provided is null or undefined");return t},isPlotDiv:function(t){var e=r.select(t);return e.node()instanceof HTMLElement&&e.size()&&e.classed("js-plotly-plot")},removeElement:o,addStyleRule:function(t,e){a("global",t,e)},addRelatedStyleRule:a,deleteRelatedStyleRule:function(t){var e="plotly.js-style-"+t,n=document.getElementById(e);n&&o(n)}}},{"./loggers":723,d3:163}],709:[function(t,e,n){"use strict";var r=t("events").EventEmitter,i={init:function(t){if(t._ev instanceof r)return t;var e=new r,n=new r;return t._ev=e,t._internalEv=n,t.on=e.on.bind(e),t.once=e.once.bind(e),t.removeListener=e.removeListener.bind(e),t.removeAllListeners=e.removeAllListeners.bind(e),t._internalOn=n.on.bind(n),t._internalOnce=n.once.bind(n),t._removeInternalListener=n.removeListener.bind(n),t._removeAllInternalListeners=n.removeAllListeners.bind(n),t.emit=function(r,i){"undefined"!=typeof jQuery&&jQuery(t).trigger(r,i),e.emit(r,i),n.emit(r,i)},t},triggerHandler:function(t,e,n){var r,i;"undefined"!=typeof jQuery&&(r=jQuery(t).triggerHandler(e,n));var o=t._ev;if(!o)return r;var a,s=o._events[e];if(!s)return r;function A(t){return t.listener?(o.removeListener(e,t.listener),t.fired?void 0:(t.fired=!0,t.listener.apply(o,[n]))):t.apply(o,[n])}for(s=Array.isArray(s)?s:[s],a=0;a<s.length-1;a++)A(s[a]);return i=A(s[a]),void 0!==r?r:i},purge:function(t){return delete t._ev,delete t.on,delete t.once,delete t.removeListener,delete t.removeAllListeners,delete t.emit,delete t._ev,delete t._internalEv,delete t._internalOn,delete t._internalOnce,delete t._removeInternalListener,delete t._removeAllInternalListeners,t}};e.exports=i},{events:104}],710:[function(t,e,n){"use strict";var r=t("./is_plain_object.js"),i=Array.isArray;function o(t,e,n,a){var s,A,l,c,u,g,h=t[0],d=t.length;if(2===d&&i(h)&&i(t[1])&&0===h.length){if(function(t,e){var n,r;for(n=0;n<t.length;n++){if(null!==(r=t[n])&&"object"==typeof r)return!1;void 0!==r&&(e[n]=r)}return!0}(t[1],h))return h;h.splice(0,h.length)}for(var f=1;f<d;f++)for(A in s=t[f])l=h[A],c=s[A],a&&i(c)?h[A]=c:e&&c&&(r(c)||(u=i(c)))?(u?(u=!1,g=l&&i(l)?l:[]):g=l&&r(l)?l:{},h[A]=o([g,c],e,n,a)):(void 0!==c||n)&&(h[A]=c);return h}n.extendFlat=function(){return o(arguments,!1,!1,!1)},n.extendDeep=function(){return o(arguments,!0,!1,!1)},n.extendDeepAll=function(){return o(arguments,!0,!0,!1)},n.extendDeepNoArrays=function(){return o(arguments,!0,!1,!0)}},{"./is_plain_object.js":720}],711:[function(t,e,n){"use strict";e.exports=function(t){for(var e={},n=[],r=0,i=0;i<t.length;i++){var o=t[i];1!==e[o]&&(e[o]=1,n[r++]=o)}return n}},{}],712:[function(t,e,n){"use strict";function r(t){return!0===t.visible}function i(t){var e=t[0].trace;return!0===e.visible&&0!==e._length}e.exports=function(t){for(var e,n=(e=t,Array.isArray(e)&&Array.isArray(e[0])&&e[0][0]&&e[0][0].trace?i:r),o=[],a=0;a<t.length;a++){var s=t[a];n(s)&&o.push(s)}return o}},{}],713:[function(t,e,n){"use strict";var r=t("country-regex"),i=t("../lib"),o=Object.keys(r),a={"ISO-3":i.identity,"USA-states":i.identity,"country names":function(t){for(var e=0;e<o.length;e++){var n=o[e],a=new RegExp(r[n]);if(a.test(t.trim().toLowerCase()))return n}return i.log("Unrecognized country name: "+t+"."),!1}};e.exports={locationToFeature:function(t,e,n){if(!e||"string"!=typeof e)return!1;var r,o,s,A=a[t](e);if(A){if("USA-states"===t)for(r=[],s=0;s<n.length;s++)(o=n[s]).properties&&o.properties.gu&&"USA"===o.properties.gu&&r.push(o);else r=n;for(s=0;s<r.length;s++)if((o=r[s]).id===A)return o;i.log(["Location with id",A,"does not have a matching topojson feature at this resolution."].join(" "))}return!1}}},{"../lib":719,"country-regex":134}],714:[function(t,e,n){"use strict";var r=t("../constants/numerical").BADNUM;n.calcTraceToLineCoords=function(t){for(var e=t[0].trace.connectgaps,n=[],i=[],o=0;o<t.length;o++){var a=t[o].lonlat;a[0]!==r?i.push(a):!e&&i.length>0&&(n.push(i),i=[])}return i.length>0&&n.push(i),n},n.makeLine=function(t){return 1===t.length?{type:"LineString",coordinates:t[0]}:{type:"MultiLineString",coordinates:t}},n.makePolygon=function(t){if(1===t.length)return{type:"Polygon",coordinates:t};for(var e=new Array(t.length),n=0;n<t.length;n++)e[n]=[t[n]];return{type:"MultiPolygon",coordinates:e}},n.makeBlank=function(){return{type:"Point",coordinates:[]}}},{"../constants/numerical":695}],715:[function(t,e,n){"use strict";var r,i,o,a=t("./mod").mod;function s(t,e,n,r,i,o,a,s){var A=n-t,l=i-t,c=a-i,u=r-e,g=o-e,h=s-o,d=A*h-c*u;if(0===d)return null;var f=(l*h-c*g)/d,p=(l*u-A*g)/d;return p<0||p>1||f<0||f>1?null:{x:t+A*f,y:e+u*f}}function A(t,e,n,r,i){var o=r*t+i*e;if(o<0)return r*r+i*i;if(o>n){var a=r-t,s=i-e;return a*a+s*s}var A=r*e-i*t;return A*A/n}n.segmentsIntersect=s,n.segmentDistance=function(t,e,n,r,i,o,a,l){if(s(t,e,n,r,i,o,a,l))return 0;var c=n-t,u=r-e,g=a-i,h=l-o,d=c*c+u*u,f=g*g+h*h,p=Math.min(A(c,u,d,i-t,o-e),A(c,u,d,a-t,l-e),A(g,h,f,t-i,e-o),A(g,h,f,n-i,r-o));return Math.sqrt(p)},n.getTextLocation=function(t,e,n,s){if(t===i&&s===o||(r={},i=t,o=s),r[n])return r[n];var A=t.getPointAtLength(a(n-s/2,e)),l=t.getPointAtLength(a(n+s/2,e)),c=Math.atan((l.y-A.y)/(l.x-A.x)),u=t.getPointAtLength(a(n,e)),g={x:(4*u.x+A.x+l.x)/6,y:(4*u.y+A.y+l.y)/6,theta:c};return r[n]=g,g},n.clearLocationCache=function(){i=null},n.getVisibleSegment=function(t,e,n){var r,i,o=e.left,a=e.right,s=e.top,A=e.bottom,l=0,c=t.getTotalLength(),u=c;function g(e){var n=t.getPointAtLength(e);0===e?r=n:e===c&&(i=n);var l=n.x<o?o-n.x:n.x>a?n.x-a:0,u=n.y<s?s-n.y:n.y>A?n.y-A:0;return Math.sqrt(l*l+u*u)}for(var h=g(l);h;){if((l+=h+n)>u)return;h=g(l)}for(h=g(u);h;){if(l>(u-=h+n))return;h=g(u)}return{min:l,max:u,len:u-l,total:c,isClosed:0===l&&u===c&&Math.abs(r.x-i.x)<.1&&Math.abs(r.y-i.y)<.1}},n.findPointOnPath=function(t,e,n,r){for(var i,o,a,s=(r=r||{}).pathLength||t.getTotalLength(),A=r.tolerance||.001,l=r.iterationLimit||30,c=t.getPointAtLength(0)[n]>t.getPointAtLength(s)[n]?-1:1,u=0,g=0,h=s;u<l;){if(i=(g+h)/2,a=(o=t.getPointAtLength(i))[n]-e,Math.abs(a)<A)return o;c*a>0?h=i:g=i,u++}return o}},{"./mod":726}],716:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("tinycolor2"),o=t("color-normalize"),a=t("../components/colorscale"),s=t("../components/color/attributes").defaultLine,A=t("./array").isArrayOrTypedArray,l=o(s),c=1;function u(t,e){var n=t;return n[3]*=e,n}function g(t){if(r(t))return l;var e=o(t);return e.length?e:l}function h(t){return r(t)?t:c}e.exports={formatColor:function(t,e,n){var r,i,s,d,f,p=t.color,I=A(p),C=A(e),m=a.extractOpts(t),y=[];if(r=void 0!==m.colorscale?a.makeColorScaleFuncFromTrace(t):g,i=I?function(t,e){return void 0===t[e]?l:o(r(t[e]))}:g,s=C?function(t,e){return void 0===t[e]?c:h(t[e])}:h,I||C)for(var E=0;E<n;E++)d=i(p,E),f=s(e,E),y[E]=u(d,f);else y=u(o(p),e);return y},parseColorScale:function(t,e){void 0===e&&(e=1);var n=a.extractOpts(t);return(n.reversescale?a.flipScale(n.colorscale):n.colorscale).map(function(t){var n=t[0],r=i(t[1]).toRgb();return{index:n,rgb:[r.r,r.g,r.b,e]}})}}},{"../components/color/attributes":592,"../components/colorscale":605,"./array":702,"color-normalize":120,"fast-isnumeric":225,tinycolor2:537}],717:[function(t,e,n){"use strict";var r=t("./identity");function i(t){return[t]}e.exports={keyFun:function(t){return t.key},repeat:i,descend:r,wrap:i,unwrap:function(t){return t[0]}}},{"./identity":718}],718:[function(t,e,n){"use strict";e.exports=function(t){return t}},{}],719:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=t("../constants/numerical"),a=o.FP_SAFE,s=o.BADNUM,A=e.exports={};A.nestedProperty=t("./nested_property"),A.keyedContainer=t("./keyed_container"),A.relativeAttr=t("./relative_attr"),A.isPlainObject=t("./is_plain_object"),A.toLogRange=t("./to_log_range"),A.relinkPrivateKeys=t("./relink_private");var l=t("./array");A.isTypedArray=l.isTypedArray,A.isArrayOrTypedArray=l.isArrayOrTypedArray,A.isArray1D=l.isArray1D,A.ensureArray=l.ensureArray,A.concat=l.concat,A.maxRowLength=l.maxRowLength,A.minRowLength=l.minRowLength;var c=t("./mod");A.mod=c.mod,A.modHalf=c.modHalf;var u=t("./coerce");A.valObjectMeta=u.valObjectMeta,A.coerce=u.coerce,A.coerce2=u.coerce2,A.coerceFont=u.coerceFont,A.coerceHoverinfo=u.coerceHoverinfo,A.coerceSelectionMarkerOpacity=u.coerceSelectionMarkerOpacity,A.validate=u.validate;var g=t("./dates");A.dateTime2ms=g.dateTime2ms,A.isDateTime=g.isDateTime,A.ms2DateTime=g.ms2DateTime,A.ms2DateTimeLocal=g.ms2DateTimeLocal,A.cleanDate=g.cleanDate,A.isJSDate=g.isJSDate,A.formatDate=g.formatDate,A.incrementMonth=g.incrementMonth,A.dateTick0=g.dateTick0,A.dfltRange=g.dfltRange,A.findExactDates=g.findExactDates,A.MIN_MS=g.MIN_MS,A.MAX_MS=g.MAX_MS;var h=t("./search");A.findBin=h.findBin,A.sorterAsc=h.sorterAsc,A.sorterDes=h.sorterDes,A.distinctVals=h.distinctVals,A.roundUp=h.roundUp,A.sort=h.sort,A.findIndexOfMin=h.findIndexOfMin;var d=t("./stats");A.aggNums=d.aggNums,A.len=d.len,A.mean=d.mean,A.median=d.median,A.midRange=d.midRange,A.variance=d.variance,A.stdev=d.stdev,A.interp=d.interp;var f=t("./matrix");A.init2dArray=f.init2dArray,A.transposeRagged=f.transposeRagged,A.dot=f.dot,A.translationMatrix=f.translationMatrix,A.rotationMatrix=f.rotationMatrix,A.rotationXYMatrix=f.rotationXYMatrix,A.apply2DTransform=f.apply2DTransform,A.apply2DTransform2=f.apply2DTransform2;var p=t("./angles");A.deg2rad=p.deg2rad,A.rad2deg=p.rad2deg,A.angleDelta=p.angleDelta,A.angleDist=p.angleDist,A.isFullCircle=p.isFullCircle,A.isAngleInsideSector=p.isAngleInsideSector,A.isPtInsideSector=p.isPtInsideSector,A.pathArc=p.pathArc,A.pathSector=p.pathSector,A.pathAnnulus=p.pathAnnulus;var I=t("./anchor_utils");A.isLeftAnchor=I.isLeftAnchor,A.isCenterAnchor=I.isCenterAnchor,A.isRightAnchor=I.isRightAnchor,A.isTopAnchor=I.isTopAnchor,A.isMiddleAnchor=I.isMiddleAnchor,A.isBottomAnchor=I.isBottomAnchor;var C=t("./geometry2d");A.segmentsIntersect=C.segmentsIntersect,A.segmentDistance=C.segmentDistance,A.getTextLocation=C.getTextLocation,A.clearLocationCache=C.clearLocationCache,A.getVisibleSegment=C.getVisibleSegment,A.findPointOnPath=C.findPointOnPath;var m=t("./extend");A.extendFlat=m.extendFlat,A.extendDeep=m.extendDeep,A.extendDeepAll=m.extendDeepAll,A.extendDeepNoArrays=m.extendDeepNoArrays;var y=t("./loggers");A.log=y.log,A.warn=y.warn,A.error=y.error;var E=t("./regex");A.counterRegex=E.counter;var v=t("./throttle");A.throttle=v.throttle,A.throttleDone=v.done,A.clearThrottle=v.clear;var B=t("./dom");function b(t){var e={};for(var n in t)for(var r=t[n],i=0;i<r.length;i++)e[r[i]]=+n;return e}A.getGraphDiv=B.getGraphDiv,A.isPlotDiv=B.isPlotDiv,A.removeElement=B.removeElement,A.addStyleRule=B.addStyleRule,A.addRelatedStyleRule=B.addRelatedStyleRule,A.deleteRelatedStyleRule=B.deleteRelatedStyleRule,A.clearResponsive=t("./clear_responsive"),A.makeTraceGroups=t("./make_trace_groups"),A._=t("./localize"),A.notifier=t("./notifier"),A.filterUnique=t("./filter_unique"),A.filterVisible=t("./filter_visible"),A.pushUnique=t("./push_unique"),A.cleanNumber=t("./clean_number"),A.ensureNumber=function(t){return i(t)?(t=Number(t))<-a||t>a?s:i(t)?Number(t):s:s},A.isIndex=function(t,e){return!(void 0!==e&&t>=e)&&i(t)&&t>=0&&t%1==0},A.noop=t("./noop"),A.identity=t("./identity"),A.repeat=function(t,e){for(var n=new Array(e),r=0;r<e;r++)n[r]=t;return n},A.swapAttrs=function(t,e,n,r){n||(n="x"),r||(r="y");for(var i=0;i<e.length;i++){var o=e[i],a=A.nestedProperty(t,o.replace("?",n)),s=A.nestedProperty(t,o.replace("?",r)),l=a.get();a.set(s.get()),s.set(l)}},A.raiseToTop=function(t){t.parentNode.appendChild(t)},A.cancelTransition=function(t){return t.transition().duration(0)},A.constrain=function(t,e,n){return e>n?Math.max(n,Math.min(e,t)):Math.max(e,Math.min(n,t))},A.bBoxIntersect=function(t,e,n){return n=n||0,t.left<=e.right+n&&e.left<=t.right+n&&t.top<=e.bottom+n&&e.top<=t.bottom+n},A.simpleMap=function(t,e,n,r){for(var i=t.length,o=new Array(i),a=0;a<i;a++)o[a]=e(t[a],n,r);return o},A.randstr=function t(e,n,r,i){if(r||(r=16),void 0===n&&(n=24),n<=0)return"0";var o,a,s=Math.log(Math.pow(2,n))/Math.log(r),l="";for(o=2;s===1/0;o*=2)s=Math.log(Math.pow(2,n/o))/Math.log(r)*o;var c=s-Math.floor(s);for(o=0;o<Math.floor(s);o++)l=Math.floor(Math.random()*r).toString(r)+l;c&&(a=Math.pow(r,c),l=Math.floor(Math.random()*a).toString(r)+l);var u=parseInt(l,r);return e&&e[l]||u!==1/0&&u>=Math.pow(2,n)?i>10?(A.warn("randstr failed uniqueness"),l):t(e,n,r,(i||0)+1):l},A.OptionControl=function(t,e){t||(t={}),e||(e="opt");var n={optionList:[],_newoption:function(r){r[e]=t,n[r.name]=r,n.optionList.push(r)}};return n["_"+e]=t,n},A.smooth=function(t,e){if((e=Math.round(e)||0)<2)return t;var n,r,i,o,a=t.length,s=2*a,A=2*e-1,l=new Array(A),c=new Array(a);for(n=0;n<A;n++)l[n]=(1-Math.cos(Math.PI*(n+1)/e))/(2*e);for(n=0;n<a;n++){for(o=0,r=0;r<A;r++)(i=n+r+1-e)<-a?i-=s*Math.round(i/s):i>=s&&(i-=s*Math.floor(i/s)),i<0?i=-1-i:i>=a&&(i=s-1-i),o+=t[i]*l[r];c[n]=o}return c},A.syncOrAsync=function(t,e,n){var r;function i(){return A.syncOrAsync(t,e,n)}for(;t.length;)if((r=(0,t.splice(0,1)[0])(e))&&r.then)return r.then(i).then(void 0,A.promiseError);return n&&n(e)},A.stripTrailingSlash=function(t){return"/"===t.substr(-1)?t.substr(0,t.length-1):t},A.noneOrAll=function(t,e,n){if(t){var r,i=!1,o=!0;for(r=0;r<n.length;r++)null!=t[n[r]]?i=!0:o=!1;if(i&&!o)for(r=0;r<n.length;r++)t[n[r]]=e[n[r]]}},A.mergeArray=function(t,e,n,r){var i="function"==typeof r;if(A.isArrayOrTypedArray(t))for(var o=Math.min(t.length,e.length),a=0;a<o;a++){var s=t[a];e[a][n]=i?r(s):s}},A.mergeArrayCastPositive=function(t,e,n){return A.mergeArray(t,e,n,function(t){var e=+t;return isFinite(e)&&e>0?e:0})},A.fillArray=function(t,e,n,r){if(r=r||A.identity,A.isArrayOrTypedArray(t))for(var i=0;i<e.length;i++)e[i][n]=r(t[i])},A.castOption=function(t,e,n,r){r=r||A.identity;var i=A.nestedProperty(t,n).get();return A.isArrayOrTypedArray(i)?Array.isArray(e)&&A.isArrayOrTypedArray(i[e[0]])?r(i[e[0]][e[1]]):r(i[e]):i},A.extractOption=function(t,e,n,r){if(n in t)return t[n];var i=A.nestedProperty(e,r).get();return Array.isArray(i)?void 0:i},A.tagSelected=function(t,e,n){var r,i,o=e.selectedpoints,a=e._indexToPoints;a&&(r=b(a));for(var s=0;s<o.length;s++){var l=o[s];if(A.isIndex(l)){var c=r?r[l]:l,u=n?n[c]:c;void 0!==(i=u)&&i<t.length&&(t[u].selected=1)}}},A.selIndices2selPoints=function(t){var e=t.selectedpoints,n=t._indexToPoints;if(n){for(var r=b(n),i=[],o=0;o<e.length;o++){var a=e[o];if(A.isIndex(a)){var s=r[a];A.isIndex(s)&&i.push(s)}}return i}return e},A.getTargetArray=function(t,e){var n=e.target;if("string"==typeof n&&n){var r=A.nestedProperty(t,n).get();return!!Array.isArray(r)&&r}return!!Array.isArray(n)&&n},A.minExtend=function(t,e){var n={};"object"!=typeof e&&(e={});var r,i,o,a=Object.keys(t);for(r=0;r<a.length;r++)o=t[i=a[r]],"_"!==i.charAt(0)&&"function"!=typeof o&&("module"===i?n[i]=o:Array.isArray(o)?n[i]="colorscale"===i?o.slice():o.slice(0,3):n[i]=o&&"object"==typeof o?A.minExtend(t[i],e[i]):o);for(a=Object.keys(e),r=0;r<a.length;r++)"object"==typeof(o=e[i=a[r]])&&i in n&&"object"==typeof n[i]||(n[i]=o);return n},A.titleCase=function(t){return t.charAt(0).toUpperCase()+t.substr(1)},A.containsAny=function(t,e){for(var n=0;n<e.length;n++)if(-1!==t.indexOf(e[n]))return!0;return!1},A.isIE=function(){return void 0!==window.navigator.msSaveBlob};var Q=/MSIE [1-9]\./;A.isIE9orBelow=function(){return A.isIE()&&Q.test(window.navigator.userAgent)};var w=/Version\/[\d\.]+.*Safari/;A.isSafari=function(){return w.test(window.navigator.userAgent)},A.isD3Selection=function(t){return t&&"function"==typeof t.classed},A.ensureSingle=function(t,e,n,r){var i=t.select(e+(n?"."+n:""));if(i.size())return i;var o=t.append(e);return n&&o.classed(n,!0),r&&o.call(r),o},A.ensureSingleById=function(t,e,n,r){var i=t.select(e+"#"+n);if(i.size())return i;var o=t.append(e).attr("id",n);return r&&o.call(r),o},A.objectFromPath=function(t,e){for(var n,r=t.split("."),i=n={},o=0;o<r.length;o++){var a=r[o],s=null,A=r[o].match(/(.*)\[([0-9]+)\]/);A?(a=A[1],s=A[2],n=n[a]=[],o===r.length-1?n[s]=e:n[s]={},n=n[s]):(o===r.length-1?n[a]=e:n[a]={},n=n[a])}return i};var x=/^([^\[\.]+)\.(.+)?/,_=/^([^\.]+)\[([0-9]+)\](\.)?(.+)?/;A.expandObjectPaths=function(t){var e,n,r,i,o,a,s;if("object"==typeof t&&!Array.isArray(t))for(n in t)t.hasOwnProperty(n)&&((e=n.match(x))?(i=t[n],r=e[1],delete t[n],t[r]=A.extendDeepNoArrays(t[r]||{},A.objectFromPath(n,A.expandObjectPaths(i))[r])):(e=n.match(_))?(i=t[n],r=e[1],o=parseInt(e[2]),delete t[n],t[r]=t[r]||[],"."===e[3]?(s=e[4],a=t[r][o]=t[r][o]||{},A.extendDeepNoArrays(a,A.objectFromPath(s,A.expandObjectPaths(i)))):t[r][o]=A.expandObjectPaths(i)):t[n]=A.expandObjectPaths(t[n]));return t},A.numSeparate=function(t,e,n){if(n||(n=!1),"string"!=typeof e||0===e.length)throw new Error("Separator string required for formatting!");"number"==typeof t&&(t=String(t));var r=/(\d+)(\d{3})/,i=e.charAt(0),o=e.charAt(1),a=t.split("."),s=a[0],A=a.length>1?i+a[1]:"";if(o&&(a.length>1||s.length>4||n))for(;r.test(s);)s=s.replace(r,"$1"+o+"$2");return s+A},A.TEMPLATE_STRING_REGEX=/%{([^\s%{}:]*)(:[^}]*)?}/g;var D=/^\w*$/;A.templateString=function(t,e){var n={};return t.replace(A.TEMPLATE_STRING_REGEX,function(t,r){return D.test(r)?e[r]||"":(n[r]=n[r]||A.nestedProperty(e,r).get,n[r]()||"")})};var M=/^:/,k=0;A.hovertemplateString=function(t,e,n){var i=arguments,o={};return t.replace(A.TEMPLATE_STRING_REGEX,function(t,a,s){var l,c,u;for(u=3;u<i.length;u++){if((l=i[u]).hasOwnProperty(a)){c=l[a];break}if(D.test(a)||(c=o[a]||A.nestedProperty(l,a).get())&&(o[a]=c),void 0!==c)break}return void 0===c&&(k<10&&(A.warn("Variable '"+a+"' in hovertemplate could not be found!"),c=t),10===k&&A.warn("Too many hovertemplate warnings - additional warnings will be suppressed"),k++),s?c=(n?n.numberFormat:r.format)(s.replace(M,""))(c):e.hasOwnProperty(a+"Label")&&(c=e[a+"Label"]),c})},A.subplotSort=function(t,e){for(var n=Math.min(t.length,e.length)+1,r=0,i=0,o=0;o<n;o++){var a=t.charCodeAt(o)||0,s=e.charCodeAt(o)||0,A=a>=48&&a<=57,l=s>=48&&s<=57;if(A&&(r=10*r+a-48),l&&(i=10*i+s-48),!A||!l){if(r!==i)return r-i;if(a!==s)return a-s}}return i-r};var S=2e9;A.seedPseudoRandom=function(){S=2e9},A.pseudoRandom=function(){var t=S;return S=(69069*S+1)%4294967296,Math.abs(S-t)<429496729?A.pseudoRandom():S/4294967296},A.fillText=function(t,e,n){var r=Array.isArray(n)?function(t){n.push(t)}:function(t){n.text=t},i=A.extractOption(t,e,"htx","hovertext");if(A.isValidTextValue(i))return r(i);var o=A.extractOption(t,e,"tx","text");return A.isValidTextValue(o)?r(o):void 0},A.isValidTextValue=function(t){return t||0===t},A.formatPercent=function(t,e){e=e||0;for(var n=(Math.round(100*t*Math.pow(10,e))*Math.pow(.1,e)).toFixed(e)+"%",r=0;r<e;r++)-1!==n.indexOf(".")&&(n=(n=n.replace("0%","%")).replace(".%","%"));return n},A.isHidden=function(t){var e=window.getComputedStyle(t).display;return!e||"none"===e}},{"../constants/numerical":695,"./anchor_utils":700,"./angles":701,"./array":702,"./clean_number":703,"./clear_responsive":705,"./coerce":706,"./dates":707,"./dom":708,"./extend":710,"./filter_unique":711,"./filter_visible":712,"./geometry2d":715,"./identity":718,"./is_plain_object":720,"./keyed_container":721,"./localize":722,"./loggers":723,"./make_trace_groups":724,"./matrix":725,"./mod":726,"./nested_property":727,"./noop":728,"./notifier":729,"./push_unique":733,"./regex":735,"./relative_attr":736,"./relink_private":737,"./search":738,"./stats":741,"./throttle":744,"./to_log_range":745,d3:163,"fast-isnumeric":225}],720:[function(t,e,n){"use strict";e.exports=function(t){return window&&window.process&&window.process.versions?"[object Object]"===Object.prototype.toString.call(t):"[object Object]"===Object.prototype.toString.call(t)&&Object.getPrototypeOf(t)===Object.prototype}},{}],721:[function(t,e,n){"use strict";var r=t("./nested_property"),i=/^\w*$/;e.exports=function(t,e,n,o){var a,s,A;n=n||"name",o=o||"value";var l={};e&&e.length?(A=r(t,e),s=A.get()):s=t,e=e||"";var c={};if(s)for(a=0;a<s.length;a++)c[s[a][n]]=a;var u=i.test(o),g={set:function(t,e){var i=null===e?4:0;if(!s){if(!A||4===i)return;s=[],A.set(s)}var a=c[t];if(void 0===a){if(4===i)return;i|=3,a=s.length,c[t]=a}else e!==(u?s[a][o]:r(s[a],o).get())&&(i|=2);var h=s[a]=s[a]||{};return h[n]=t,u?h[o]=e:r(h,o).set(e),null!==e&&(i&=-5),l[a]=l[a]|i,g},get:function(t){if(s){var e=c[t];return void 0===e?void 0:u?s[e][o]:r(s[e],o).get()}},rename:function(t,e){var r=c[t];return void 0===r?g:(l[r]=1|l[r],c[e]=r,delete c[t],s[r][n]=e,g)},remove:function(t){var e=c[t];if(void 0===e)return g;var i=s[e];if(Object.keys(i).length>2)return l[e]=2|l[e],g.set(t,null);if(u){for(a=e;a<s.length;a++)l[a]=3|l[a];for(a=e;a<s.length;a++)c[s[a][n]]--;s.splice(e,1),delete c[t]}else r(i,o).set(null),l[e]=6|l[e];return g},constructUpdate:function(){for(var t,i,a={},A=Object.keys(l),c=0;c<A.length;c++)i=A[c],t=e+"["+i+"]",s[i]?(1&l[i]&&(a[t+"."+n]=s[i][n]),2&l[i]&&(a[t+"."+o]=u?4&l[i]?null:s[i][o]:4&l[i]?null:r(s[i],o).get())):a[t]=null;return a}};return g}},{"./nested_property":727}],722:[function(t,e,n){"use strict";var r=t("../registry");e.exports=function(t,e){for(var n=t._context.locale,i=0;i<2;i++){for(var o=t._context.locales,a=0;a<2;a++){var s=(o[n]||{}).dictionary;if(s){var A=s[e];if(A)return A}o=r.localeRegistry}var l=n.split("-")[0];if(l===n)break;n=l}return e}},{"../registry":847}],723:[function(t,e,n){"use strict";var r=t("../plot_api/plot_config").dfltConfig,i=e.exports={};function o(t,e){if(t&&t.apply)try{return void t.apply(console,e)}catch(t){}for(var n=0;n<e.length;n++)try{t(e[n])}catch(t){console.log(e[n])}}i.log=function(){if(r.logging>1){for(var t=["LOG:"],e=0;e<arguments.length;e++)t.push(arguments[e]);o(console.trace||console.log,t)}},i.warn=function(){if(r.logging>0){for(var t=["WARN:"],e=0;e<arguments.length;e++)t.push(arguments[e]);o(console.trace||console.log,t)}},i.error=function(){if(r.logging>0){for(var t=["ERROR:"],e=0;e<arguments.length;e++)t.push(arguments[e]);o(console.error,t)}}},{"../plot_api/plot_config":755}],724:[function(t,e,n){"use strict";var r=t("d3");e.exports=function(t,e,n){var i=t.selectAll("g."+n.replace(/\s/g,".")).data(e,function(t){return t[0].trace.uid});i.exit().remove(),i.enter().append("g").attr("class",n),i.order();var o=t.classed("rangeplot")?"nodeRangePlot3":"node3";return i.each(function(t){t[0][o]=r.select(this)}),i}},{d3:163}],725:[function(t,e,n){"use strict";n.init2dArray=function(t,e){for(var n=new Array(t),r=0;r<t;r++)n[r]=new Array(e);return n},n.transposeRagged=function(t){var e,n,r=0,i=t.length;for(e=0;e<i;e++)r=Math.max(r,t[e].length);var o=new Array(r);for(e=0;e<r;e++)for(o[e]=new Array(i),n=0;n<i;n++)o[e][n]=t[n][e];return o},n.dot=function(t,e){if(!t.length||!e.length||t.length!==e.length)return null;var r,i,o=t.length;if(t[0].length)for(r=new Array(o),i=0;i<o;i++)r[i]=n.dot(t[i],e);else if(e[0].length){var a=n.transposeRagged(e);for(r=new Array(a.length),i=0;i<a.length;i++)r[i]=n.dot(t,a[i])}else for(r=0,i=0;i<o;i++)r+=t[i]*e[i];return r},n.translationMatrix=function(t,e){return[[1,0,t],[0,1,e],[0,0,1]]},n.rotationMatrix=function(t){var e=t*Math.PI/180;return[[Math.cos(e),-Math.sin(e),0],[Math.sin(e),Math.cos(e),0],[0,0,1]]},n.rotationXYMatrix=function(t,e,r){return n.dot(n.dot(n.translationMatrix(e,r),n.rotationMatrix(t)),n.translationMatrix(-e,-r))},n.apply2DTransform=function(t){return function(){var e=arguments;3===e.length&&(e=e[0]);var r=1===arguments.length?e[0]:[e[0],e[1]];return n.dot(t,[r[0],r[1],1]).slice(0,2)}},n.apply2DTransform2=function(t){var e=n.apply2DTransform(t);return function(t){return e(t.slice(0,2)).concat(e(t.slice(2,4)))}}},{}],726:[function(t,e,n){"use strict";e.exports={mod:function(t,e){var n=t%e;return n<0?n+e:n},modHalf:function(t,e){return Math.abs(t)>e/2?t-Math.round(t/e)*e:t}}},{}],727:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("./array").isArrayOrTypedArray;e.exports=function(t,e){if(r(e))e=String(e);else if("string"!=typeof e||"[-1]"===e.substr(e.length-4))throw"bad property string";for(var n,o,a,A=0,l=e.split(".");A<l.length;){if(n=String(l[A]).match(/^([^\[\]]*)((\[\-?[0-9]*\])+)$/)){if(n[1])l[A]=n[1];else{if(0!==A)throw"bad property string";l.splice(0,1)}for(o=n[2].substr(1,n[2].length-2).split("]["),a=0;a<o.length;a++)A++,l.splice(A,0,Number(o[a]))}A++}return"object"!=typeof t?function(t,e,n){return{set:function(){throw"bad container"},get:function(){},astr:e,parts:n,obj:t}}(t,e,l):{set:s(t,l,e),get:function t(e,n){return function(){var r,o,a,s,A,l=e;for(s=0;s<n.length-1;s++){if(-1===(r=n[s])){for(o=!0,a=[],A=0;A<l.length;A++)a[A]=t(l[A],n.slice(s+1))(),a[A]!==a[0]&&(o=!1);return o?a[0]:a}if("number"==typeof r&&!i(l))return;if("object"!=typeof(l=l[r])||null===l)return}if("object"==typeof l&&null!==l&&null!==(a=l[n[s]]))return a}}(t,l),astr:e,parts:l,obj:t}};var o=/(^|\.)args\[/;function a(t,e){return void 0===t||null===t&&!e.match(o)}function s(t,e,n){return function(r){var o,s,u=t,g="",h=[[t,g]],d=a(r,n);for(s=0;s<e.length-1;s++){if("number"==typeof(o=e[s])&&!i(u))throw"array index but container is not an array";if(-1===o){if(d=!l(u,e.slice(s+1),r,n))break;return}if(!c(u,o,e[s+1],d))break;if("object"!=typeof(u=u[o])||null===u)throw"container is not an object";g=A(g,o),h.push([u,g])}if(d){if(s===e.length-1&&(delete u[e[s]],Array.isArray(u)&&+e[s]==u.length-1))for(;u.length&&void 0===u[u.length-1];)u.pop()}else u[e[s]]=r}}function A(t,e){var n=e;return r(e)?n="["+e+"]":t&&(n="."+e),t+n}function l(t,e,n,r){var o,A=i(n),l=!0,u=n,g=r.replace("-1",0),h=!A&&a(n,g),d=e[0];for(o=0;o<t.length;o++)g=r.replace("-1",o),A&&(h=a(u=n[o%n.length],g)),h&&(l=!1),c(t,o,d,h)&&s(t[o],e,r.replace("-1",o))(u);return l}function c(t,e,n,r){if(void 0===t[e]){if(r)return!1;t[e]="number"==typeof n?[]:{}}return!0}},{"./array":702,"fast-isnumeric":225}],728:[function(t,e,n){"use strict";e.exports=function(){}},{}],729:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=[];e.exports=function(t,e){if(-1===o.indexOf(t)){o.push(t);var n=1e3;i(e)?n=e:"long"===e&&(n=3e3);var a=r.select("body").selectAll(".plotly-notifier").data([0]);a.enter().append("div").classed("plotly-notifier",!0),a.selectAll(".notifier-note").data(o).enter().append("div").classed("notifier-note",!0).style("opacity",0).each(function(t){var e=r.select(this);e.append("button").classed("notifier-close",!0).html("&times;").on("click",function(){e.transition().call(s)});for(var i=e.append("p"),o=t.split(/<br\s*\/?>/g),a=0;a<o.length;a++)a&&i.append("br"),i.append("span").text(o[a]);e.transition().duration(700).style("opacity",1).transition().delay(n).call(s)})}function s(t){t.duration(700).style("opacity",0).each("end",function(t){var e=o.indexOf(t);-1!==e&&o.splice(e,1),r.select(this).remove()})}}},{d3:163,"fast-isnumeric":225}],730:[function(t,e,n){"use strict";var r=t("./setcursor"),i="data-savedcursor";e.exports=function(t,e){var n=t.attr(i);if(e){if(!n){for(var o=(t.attr("class")||"").split(" "),a=0;a<o.length;a++){var s=o[a];0===s.indexOf("cursor-")&&t.attr(i,s.substr(7)).classed(s,!1)}t.attr(i)||t.attr(i,"!!")}r(t,e)}else n&&(t.attr(i,null),"!!"===n?r(t):r(t,n))}},{"./setcursor":739}],731:[function(t,e,n){"use strict";var r=t("./matrix").dot,i=t("../constants/numerical").BADNUM,o=e.exports={};o.tester=function(t){var e,n=t.slice(),r=n[0][0],o=r,a=n[0][1],s=a;for(n.push(n[0]),e=1;e<n.length;e++)r=Math.min(r,n[e][0]),o=Math.max(o,n[e][0]),a=Math.min(a,n[e][1]),s=Math.max(s,n[e][1]);var A,l=!1;5===n.length&&(n[0][0]===n[1][0]?n[2][0]===n[3][0]&&n[0][1]===n[3][1]&&n[1][1]===n[2][1]&&(l=!0,A=function(t){return t[0]===n[0][0]}):n[0][1]===n[1][1]&&n[2][1]===n[3][1]&&n[0][0]===n[3][0]&&n[1][0]===n[2][0]&&(l=!0,A=function(t){return t[1]===n[0][1]}));var c=!0,u=n[0];for(e=1;e<n.length;e++)if(u[0]!==n[e][0]||u[1]!==n[e][1]){c=!1;break}return{xmin:r,xmax:o,ymin:a,ymax:s,pts:n,contains:l?function(t,e){var n=t[0],l=t[1];return!(n===i||n<r||n>o||l===i||l<a||l>s||e&&A(t))}:function(t,e){var A=t[0],l=t[1];if(A===i||A<r||A>o||l===i||l<a||l>s)return!1;var c,u,g,h,d,f=n.length,p=n[0][0],I=n[0][1],C=0;for(c=1;c<f;c++)if(u=p,g=I,p=n[c][0],I=n[c][1],!(A<(h=Math.min(u,p))||A>Math.max(u,p)||l>Math.max(g,I)))if(l<Math.min(g,I))A!==h&&C++;else{if(l===(d=p===u?l:g+(A-u)*(I-g)/(p-u)))return 1!==c||!e;l<=d&&A!==h&&C++}return C%2==1},isRect:l,degenerate:c}},o.isSegmentBent=function(t,e,n,i){var o,a,s,A=t[e],l=[t[n][0]-A[0],t[n][1]-A[1]],c=r(l,l),u=Math.sqrt(c),g=[-l[1]/u,l[0]/u];for(o=e+1;o<n;o++)if(a=[t[o][0]-A[0],t[o][1]-A[1]],(s=r(a,l))<0||s>c||Math.abs(r(a,g))>i)return!0;return!1},o.filter=function(t,e){var n=[t[0]],r=0,i=0;function a(a){t.push(a);var s=n.length,A=r;n.splice(i+1);for(var l=A+1;l<t.length;l++)(l===t.length-1||o.isSegmentBent(t,A,l+1,e))&&(n.push(t[l]),n.length<s-2&&(r=l,i=n.length-1),A=l)}return t.length>1&&a(t.pop()),{addPt:a,raw:t,filtered:n}}},{"../constants/numerical":695,"./matrix":725}],732:[function(t,n,r){(function(e){"use strict";var r=t("./show_no_webgl_msg"),i=t("regl");n.exports=function(t,n){var o=t._fullLayout,a=!0;return o._glcanvas.each(function(r){if(!r.regl&&(!r.pick||o._has("parcoords"))){try{r.regl=i({canvas:this,attributes:{antialias:!r.pick,preserveDrawingBuffer:!0},pixelRatio:t._context.plotGlPixelRatio||e.devicePixelRatio,extensions:n||[]})}catch(t){a=!1}a&&this.addEventListener("webglcontextlost",function(e){t&&t.emit&&t.emit("plotly_webglcontextlost",{event:e,layer:r.key})},!1)}}),a||r({container:o._glcontainer.node()}),a}}).call(this,void 0!==e?e:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{"./show_no_webgl_msg":740,regl:502}],733:[function(t,e,n){"use strict";e.exports=function(t,e){if(e instanceof RegExp){for(var n=e.toString(),r=0;r<t.length;r++)if(t[r]instanceof RegExp&&t[r].toString()===n)return t;t.push(e)}else!e&&0!==e||-1!==t.indexOf(e)||t.push(e);return t}},{}],734:[function(t,e,n){"use strict";var r=t("../lib"),i=t("../plot_api/plot_config").dfltConfig,o={add:function(t,e,n,r,o){var a,s;t.undoQueue=t.undoQueue||{index:0,queue:[],sequence:!1},s=t.undoQueue.index,t.autoplay?t.undoQueue.inSequence||(t.autoplay=!1):(!t.undoQueue.sequence||t.undoQueue.beginSequence?(a={undo:{calls:[],args:[]},redo:{calls:[],args:[]}},t.undoQueue.queue.splice(s,t.undoQueue.queue.length-s,a),t.undoQueue.index+=1):a=t.undoQueue.queue[s-1],t.undoQueue.beginSequence=!1,a&&(a.undo.calls.unshift(e),a.undo.args.unshift(n),a.redo.calls.push(r),a.redo.args.push(o)),t.undoQueue.queue.length>i.queueLength&&(t.undoQueue.queue.shift(),t.undoQueue.index--))},startSequence:function(t){t.undoQueue=t.undoQueue||{index:0,queue:[],sequence:!1},t.undoQueue.sequence=!0,t.undoQueue.beginSequence=!0},stopSequence:function(t){t.undoQueue=t.undoQueue||{index:0,queue:[],sequence:!1},t.undoQueue.sequence=!1,t.undoQueue.beginSequence=!1},undo:function(t){var e,n;if(t.framework&&t.framework.isPolar)t.framework.undo();else if(!(void 0===t.undoQueue||isNaN(t.undoQueue.index)||t.undoQueue.index<=0)){for(t.undoQueue.index--,e=t.undoQueue.queue[t.undoQueue.index],t.undoQueue.inSequence=!0,n=0;n<e.undo.calls.length;n++)o.plotDo(t,e.undo.calls[n],e.undo.args[n]);t.undoQueue.inSequence=!1,t.autoplay=!1}},redo:function(t){var e,n;if(t.framework&&t.framework.isPolar)t.framework.redo();else if(!(void 0===t.undoQueue||isNaN(t.undoQueue.index)||t.undoQueue.index>=t.undoQueue.queue.length)){for(e=t.undoQueue.queue[t.undoQueue.index],t.undoQueue.inSequence=!0,n=0;n<e.redo.calls.length;n++)o.plotDo(t,e.redo.calls[n],e.redo.args[n]);t.undoQueue.inSequence=!1,t.autoplay=!1,t.undoQueue.index++}},plotDo:function(t,e,n){t.autoplay=!0,n=function(t,e){for(var n,i=[],o=0;o<e.length;o++)n=e[o],i[o]=n===t?n:"object"==typeof n?Array.isArray(n)?r.extendDeep([],n):r.extendDeepAll({},n):n;return i}(t,n),e.apply(null,n)}};e.exports=o},{"../lib":719,"../plot_api/plot_config":755}],735:[function(t,e,n){"use strict";n.counter=function(t,e,n,r){var i=(e||"")+(n?"":"$"),o=!1===r?"":"^";return"xy"===t?new RegExp(o+"x([2-9]|[1-9][0-9]+)?y([2-9]|[1-9][0-9]+)?"+i):new RegExp(o+t+"([2-9]|[1-9][0-9]+)?"+i)}},{}],736:[function(t,e,n){"use strict";var r=/^(.*)(\.[^\.\[\]]+|\[\d\])$/,i=/^[^\.\[\]]+$/;e.exports=function(t,e){for(;e;){var n=t.match(r);if(n)t=n[1];else{if(!t.match(i))throw new Error("bad relativeAttr call:"+[t,e]);t=""}if("^"!==e.charAt(0))break;e=e.slice(1)}return t&&"["!==e.charAt(0)?t+"."+e:t+e}},{}],737:[function(t,e,n){"use strict";var r=t("./array").isArrayOrTypedArray,i=t("./is_plain_object");e.exports=function t(e,n){for(var o in n){var a=n[o],s=e[o];if(s!==a)if("_"===o.charAt(0)||"function"==typeof a){if(o in e)continue;e[o]=a}else if(r(a)&&r(s)&&i(a[0])){if("customdata"===o||"ids"===o)continue;for(var A=Math.min(a.length,s.length),l=0;l<A;l++)s[l]!==a[l]&&i(a[l])&&i(s[l])&&t(s[l],a[l])}else i(a)&&i(s)&&(t(s,a),Object.keys(s).length||delete e[o])}}},{"./array":702,"./is_plain_object":720}],738:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("./loggers"),o=t("./identity");function a(t,e){return t<e}function s(t,e){return t<=e}function A(t,e){return t>e}function l(t,e){return t>=e}n.findBin=function(t,e,n){if(r(e.start))return n?Math.ceil((t-e.start)/e.size-1e-9)-1:Math.floor((t-e.start)/e.size+1e-9);var o,c,u=0,g=e.length,h=0,d=g>1?(e[g-1]-e[0])/(g-1):1;for(c=d>=0?n?a:s:n?l:A,t+=1e-9*d*(n?-1:1)*(d>=0?1:-1);u<g&&h++<100;)c(e[o=Math.floor((u+g)/2)],t)?u=o+1:g=o;return h>90&&i.log("Long binary search..."),u-1},n.sorterAsc=function(t,e){return t-e},n.sorterDes=function(t,e){return e-t},n.distinctVals=function(t){var e=t.slice();e.sort(n.sorterAsc);for(var r=e.length-1,i=e[r]-e[0]||1,o=i/(r||1)/1e4,a=[e[0]],s=0;s<r;s++)e[s+1]>e[s]+o&&(i=Math.min(i,e[s+1]-e[s]),a.push(e[s+1]));return{vals:a,minDiff:i}},n.roundUp=function(t,e,n){for(var r,i=0,o=e.length-1,a=0,s=n?0:1,A=n?1:0,l=n?Math.ceil:Math.floor;i<o&&a++<100;)e[r=l((i+o)/2)]<=t?i=r+s:o=r-A;return e[i]},n.sort=function(t,e){for(var n=0,r=0,i=1;i<t.length;i++){var o=e(t[i],t[i-1]);if(o<0?n=1:o>0&&(r=1),n&&r)return t.sort(e)}return r?t:t.reverse()},n.findIndexOfMin=function(t,e){e=e||o;for(var n,r=1/0,i=0;i<t.length;i++){var a=e(t[i]);a<r&&(r=a,n=i)}return n}},{"./identity":718,"./loggers":723,"fast-isnumeric":225}],739:[function(t,e,n){"use strict";e.exports=function(t,e){(t.attr("class")||"").split(" ").forEach(function(e){0===e.indexOf("cursor-")&&t.classed(e,!1)}),e&&t.classed("cursor-"+e,!0)}},{}],740:[function(t,e,n){"use strict";var r=t("../components/color"),i=function(){};e.exports=function(t){for(var e in t)"function"==typeof t[e]&&(t[e]=i);t.destroy=function(){t.container.parentNode.removeChild(t.container)};var n=document.createElement("div");n.className="no-webgl",n.style.cursor="pointer",n.style.fontSize="24px",n.style.color=r.defaults[0],n.style.position="absolute",n.style.left=n.style.top="0px",n.style.width=n.style.height="100%",n.style["background-color"]=r.lightLine,n.style["z-index"]=30;var o=document.createElement("p");return o.textContent="WebGL is not supported by your browser - visit https://get.webgl.org for more info",o.style.position="relative",o.style.top="50%",o.style.left="50%",o.style.height="30%",o.style.width="50%",o.style.margin="-15% 0 0 -25%",n.appendChild(o),t.container.appendChild(n),t.container.style.background="#FFFFFF",t.container.onclick=function(){window.open("https://get.webgl.org")},!1}},{"../components/color":593}],741:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("./array").isArrayOrTypedArray;n.aggNums=function(t,e,o,a){var s,A;if((!a||a>o.length)&&(a=o.length),r(e)||(e=!1),i(o[0])){for(A=new Array(a),s=0;s<a;s++)A[s]=n.aggNums(t,e,o[s]);o=A}for(s=0;s<a;s++)r(e)?r(o[s])&&(e=t(+e,+o[s])):e=o[s];return e},n.len=function(t){return n.aggNums(function(t){return t+1},0,t)},n.mean=function(t,e){return e||(e=n.len(t)),n.aggNums(function(t,e){return t+e},0,t)/e},n.midRange=function(t){if(void 0!==t&&0!==t.length)return(n.aggNums(Math.max,null,t)+n.aggNums(Math.min,null,t))/2},n.variance=function(t,e,i){return e||(e=n.len(t)),r(i)||(i=n.mean(t,e)),n.aggNums(function(t,e){return t+Math.pow(e-i,2)},0,t)/e},n.stdev=function(t,e,r){return Math.sqrt(n.variance(t,e,r))},n.median=function(t){var e=t.slice().sort();return n.interp(e,.5)},n.interp=function(t,e){if(!r(e))throw"n should be a finite number";if((e=e*t.length-.5)<0)return t[0];if(e>t.length-1)return t[t.length-1];var n=e%1;return n*t[Math.ceil(e)]+(1-n)*t[Math.floor(e)]}},{"./array":702,"fast-isnumeric":225}],742:[function(t,e,n){"use strict";var r=t("color-normalize");e.exports=function(t){return t?r(t):[0,0,0,1]}},{"color-normalize":120}],743:[function(t,e,n){"use strict";var r=t("d3"),i=t("../lib"),o=t("../constants/xmlns_namespaces"),a=t("../constants/alignment").LINE_SPACING;function s(t,e){return t.node().getBoundingClientRect()[e]}var A=/([^$]*)([$]+[^$]*[$]+)([^$]*)/;n.convertToTspans=function(t,e,w){var x=t.text(),D=!t.attr("data-notex")&&"undefined"!=typeof MathJax&&x.match(A),M=r.select(t.node().parentNode);if(!M.empty()){var k=t.attr("class")?t.attr("class").split(" ")[0]:"text";return k+="-math",M.selectAll("svg."+k).remove(),M.selectAll("g."+k+"-group").remove(),t.style("display",null).attr({"data-unformatted":x,"data-math":"N"}),D?(e&&e._promises||[]).push(new Promise(function(e){t.style("display","none");var n=parseInt(t.node().style.fontSize,10),o={fontSize:n};!function(t,e,n){var o,a,s,A;MathJax.Hub.Queue(function(){return a=i.extendDeepAll({},MathJax.Hub.config),s=MathJax.Hub.processSectionDelay,void 0!==MathJax.Hub.processSectionDelay&&(MathJax.Hub.processSectionDelay=0),MathJax.Hub.Config({messageStyle:"none",tex2jax:{inlineMath:[["$","$"],["\\(","\\)"]]},displayAlign:"left"})},function(){if("SVG"!==(o=MathJax.Hub.config.menuSettings.renderer))return MathJax.Hub.setRenderer("SVG")},function(){var n="math-output-"+i.randstr({},64);return A=r.select("body").append("div").attr({id:n}).style({visibility:"hidden",position:"absolute"}).style({"font-size":e.fontSize+"px"}).text(t.replace(l,"\\lt ").replace(c,"\\gt ")),MathJax.Hub.Typeset(A.node())},function(){var e=r.select("body").select("#MathJax_SVG_glyphs");if(A.select(".MathJax_SVG").empty()||!A.select("svg").node())i.log("There was an error in the tex syntax.",t),n();else{var a=A.select("svg").node().getBoundingClientRect();n(A.select(".MathJax_SVG"),e,a)}if(A.remove(),"SVG"!==o)return MathJax.Hub.setRenderer(o)},function(){return void 0!==s&&(MathJax.Hub.processSectionDelay=s),MathJax.Hub.Config(a)})}(D[2],o,function(r,i,o){M.selectAll("svg."+k).remove(),M.selectAll("g."+k+"-group").remove();var a=r&&r.select("svg");if(!a||!a.node())return S(),void e();var A=M.append("g").classed(k+"-group",!0).attr({"pointer-events":"none","data-unformatted":x,"data-math":"Y"});A.node().appendChild(a.node()),i&&i.node()&&a.node().insertBefore(i.node().cloneNode(!0),a.node().firstChild),a.attr({class:k,height:o.height,preserveAspectRatio:"xMinYMin meet"}).style({overflow:"visible","pointer-events":"none"});var l=t.node().style.fill||"black",c=a.select("g");c.attr({fill:l,stroke:l});var u=s(c,"width"),g=s(c,"height"),h=+t.attr("x")-u*{start:0,middle:.5,end:1}[t.attr("text-anchor")||"start"],d=-(n||s(t,"height"))/4;"y"===k[0]?(A.attr({transform:"rotate("+[-90,+t.attr("x"),+t.attr("y")]+") translate("+[-u/2,d-g/2]+")"}),a.attr({x:+t.attr("x"),y:+t.attr("y")})):"l"===k[0]?a.attr({x:t.attr("x"),y:d-g/2}):"a"===k[0]&&0!==k.indexOf("atitle")?a.attr({x:0,y:d}):a.attr({x:h,y:+t.attr("y")+d-g/2}),w&&w.call(t,A),e(A)})})):S(),t}function S(){M.empty()||(k=t.attr("class")+"-math",M.select("svg."+k).remove()),t.text("").style("white-space","pre"),function(t,e){e=e.replace(p," ");var n,s=!1,A=[],l=-1;function c(){l++;var e=document.createElementNS(o.svg,"tspan");r.select(e).attr({class:"line",dy:l*a+"em"}),t.appendChild(e),n=e;var i=A;if(A=[{node:e}],i.length>1)for(var s=1;s<i.length;s++)w(i[s])}function w(t){var e,i=t.type,a={};if("a"===i){e="a";var s=t.target,l=t.href,c=t.popup;l&&(a={"xlink:xlink:show":"_blank"===s||"_"!==s.charAt(0)?"new":"replace",target:s,"xlink:xlink:href":l},c&&(a.onclick='window.open(this.href.baseVal,this.target.baseVal,"'+c+'");return false;'))}else e="tspan";t.style&&(a.style=t.style);var u=document.createElementNS(o.svg,e);if("sup"===i||"sub"===i){x(n,d),n.appendChild(u);var f=document.createElementNS(o.svg,"tspan");x(f,d),r.select(f).attr("dy",h[i]),a.dy=g[i],n.appendChild(u),n.appendChild(f)}else n.appendChild(u);r.select(u).attr(a),n=t.node=u,A.push(t)}function x(t,e){t.appendChild(document.createTextNode(e))}function D(t){if(1!==A.length){var r=A.pop();t!==r.type&&i.log("Start tag <"+r.type+"> doesnt match end tag <"+t+">. Pretending it did match.",e),n=A[A.length-1].node}else i.log("Ignoring unexpected end tag </"+t+">.",e)}m.test(e)?c():(n=t,A=[{node:t}]);for(var M=e.split(I),k=0;k<M.length;k++){var S=M[k],L=S.match(C),R=L&&L[2].toLowerCase(),N=u[R];if("br"===R)c();else if(void 0===N)x(n,_(S));else if(L[1])D(R);else{var F=L[4],T={type:R},U=b(F,y);if(U?(U=U.replace(Q,"$1 fill:"),N&&(U+=";"+N)):N&&(U=N),U&&(T.style=U),"a"===R){s=!0;var O=b(F,E);if(O){var G=document.createElement("a");G.href=O,-1!==f.indexOf(G.protocol)&&(T.href=encodeURI(decodeURI(O)),T.target=b(F,v)||"_blank",T.popup=b(F,B))}}w(T)}}return s}(t.node(),x)&&t.style("pointer-events","all"),n.positionText(t),w&&w.call(t)}};var l=/(<|&lt;|&#60;)/g,c=/(>|&gt;|&#62;)/g,u={sup:"font-size:70%",sub:"font-size:70%",b:"font-weight:bold",i:"font-style:italic",a:"cursor:pointer",span:"",em:"font-style:italic;font-weight:bold"},g={sub:"0.3em",sup:"-0.6em"},h={sub:"-0.21em",sup:"0.42em"},d="​",f=["http:","https:","mailto:","",void 0,":"],p=/(\r\n?|\n)/g,I=/(<[^<>]*>)/,C=/<(\/?)([^ >]*)(\s+(.*))?>/i,m=/<br(\s+.*)?>/i,y=/(^|[\s"'])style\s*=\s*("([^"]*);?"|'([^']*);?')/i,E=/(^|[\s"'])href\s*=\s*("([^"]*)"|'([^']*)')/i,v=/(^|[\s"'])target\s*=\s*("([^"\s]*)"|'([^'\s]*)')/i,B=/(^|[\s"'])popup\s*=\s*("([\w=,]*)"|'([\w=,]*)')/i;function b(t,e){if(!t)return null;var n=t.match(e),r=n&&(n[3]||n[4]);return r&&_(r)}var Q=/(^|;)\s*color:/;n.plainText=function(t,e){for(var n=void 0!==(e=e||{}).len&&-1!==e.len?e.len:1/0,r=void 0!==e.allowedTags?e.allowedTags:["br"],i="...".length,o=t.split(I),a=[],s="",A=0,l=0;l<o.length;l++){var c=o[l],u=c.match(C),g=u&&u[2].toLowerCase();if(g)-1!==r.indexOf(g)&&(a.push(c),s=g);else{var h=c.length;if(A+h<n)a.push(c),A+=h;else if(A<n){var d=n-A;s&&("br"!==s||d<=i||h<=i)&&a.pop(),n>i?a.push(c.substr(0,d-i)+"..."):a.push(c.substr(0,d));break}s=""}}return a.join("")};var w={mu:"μ",amp:"&",lt:"<",gt:">",nbsp:" ",times:"×",plusmn:"±",deg:"°"},x=/&(#\d+|#x[\da-fA-F]+|[a-z]+);/g;function _(t){return t.replace(x,function(t,e){return("#"===e.charAt(0)?function(t){if(!(t>1114111)){var e=String.fromCodePoint;if(e)return e(t);var n=String.fromCharCode;return t<=65535?n(t):n(55232+(t>>10),t%1024+56320)}}("x"===e.charAt(1)?parseInt(e.substr(2),16):parseInt(e.substr(1),10)):w[e])||t})}function D(t,e,n){var r,i,o,a=n.horizontalAlign,s=n.verticalAlign||"top",A=t.node().getBoundingClientRect(),l=e.node().getBoundingClientRect();return i="bottom"===s?function(){return A.bottom-r.height}:"middle"===s?function(){return A.top+(A.height-r.height)/2}:function(){return A.top},o="right"===a?function(){return A.right-r.width}:"center"===a?function(){return A.left+(A.width-r.width)/2}:function(){return A.left},function(){return r=this.node().getBoundingClientRect(),this.style({top:i()-l.top+"px",left:o()-l.left+"px","z-index":1e3}),this}}n.convertEntities=_,n.lineCount=function(t){return t.selectAll("tspan.line").size()||1},n.positionText=function(t,e,n){return t.each(function(){var t=r.select(this);function i(e,n){return void 0===n?null===(n=t.attr(e))&&(t.attr(e,0),n=0):t.attr(e,n),n}var o=i("x",e),a=i("y",n);"text"===this.nodeName&&t.selectAll("tspan.line").attr({x:o,y:a})})},n.makeEditable=function(t,e){var n=e.gd,i=e.delegate,o=r.dispatch("edit","input","cancel"),a=i||t;if(t.style({"pointer-events":i?"none":"all"}),1!==t.size())throw new Error("boo");function s(){!function(){var i=r.select(n).select(".svg-container"),a=i.append("div"),s=t.node().style,l=parseFloat(s.fontSize||12),c=e.text;void 0===c&&(c=t.attr("data-unformatted")),a.classed("plugin-editable editable",!0).style({position:"absolute","font-family":s.fontFamily||"Arial","font-size":l,color:e.fill||s.fill||"black",opacity:1,"background-color":e.background||"transparent",outline:"#ffffff33 1px solid",margin:[-l/8+1,0,0,-1].join("px ")+"px",padding:"0","box-sizing":"border-box"}).attr({contenteditable:!0}).text(c).call(D(t,i,e)).on("blur",function(){n._editing=!1,t.text(this.textContent).style({opacity:1});var e,i=r.select(this).attr("class");(e=i?"."+i.split(" ")[0]+"-math-group":"[class*=-math-group]")&&r.select(t.node().parentNode).select(e).style({opacity:0});var a=this.textContent;r.select(this).transition().duration(0).remove(),r.select(document).on("mouseup",null),o.edit.call(t,a)}).on("focus",function(){var t=this;n._editing=!0,r.select(document).on("mouseup",function(){if(r.event.target===t)return!1;document.activeElement===a.node()&&a.node().blur()})}).on("keyup",function(){27===r.event.which?(n._editing=!1,t.style({opacity:1}),r.select(this).style({opacity:0}).on("blur",function(){return!1}).transition().remove(),o.cancel.call(t,this.textContent)):(o.input.call(t,this.textContent),r.select(this).call(D(t,i,e)))}).on("keydown",function(){13===r.event.which&&this.blur()}).call(A)}(),t.style({opacity:0});var i,s=a.attr("class");(i=s?"."+s.split(" ")[0]+"-math-group":"[class*=-math-group]")&&r.select(t.node().parentNode).select(i).style({opacity:0})}function A(t){var e=t.node(),n=document.createRange();n.selectNodeContents(e);var r=window.getSelection();r.removeAllRanges(),r.addRange(n),e.focus()}return e.immediate?s():a.on("click",s),r.rebind(t,o,"on")}},{"../constants/alignment":688,"../constants/xmlns_namespaces":696,"../lib":719,d3:163}],744:[function(t,e,n){"use strict";var r={};function i(t){t&&null!==t.timer&&(clearTimeout(t.timer),t.timer=null)}n.throttle=function(t,e,n){var o=r[t],a=Date.now();if(!o){for(var s in r)r[s].ts<a-6e4&&delete r[s];o=r[t]={ts:0,timer:null}}function A(){n(),o.ts=Date.now(),o.onDone&&(o.onDone(),o.onDone=null)}i(o),a>o.ts+e?A():o.timer=setTimeout(function(){A(),o.timer=null},e)},n.done=function(t){var e=r[t];return e&&e.timer?new Promise(function(t){var n=e.onDone;e.onDone=function(){n&&n(),t(),e.onDone=null}}):Promise.resolve()},n.clear=function(t){if(t)i(r[t]),delete r[t];else for(var e in r)n.clear(e)}},{}],745:[function(t,e,n){"use strict";var r=t("fast-isnumeric");e.exports=function(t,e){if(t>0)return Math.log(t)/Math.LN10;var n=Math.log(Math.min(e[0],e[1]))/Math.LN10;return r(n)||(n=Math.log(Math.max(e[0],e[1]))/Math.LN10-6),n}},{"fast-isnumeric":225}],746:[function(t,e,n){"use strict";var r=e.exports={},i=t("../plots/geo/constants").locationmodeToLayer,o=t("topojson-client").feature;r.getTopojsonName=function(t){return[t.scope.replace(/ /g,"-"),"_",t.resolution.toString(),"m"].join("")},r.getTopojsonPath=function(t,e){return t+e+".json"},r.getTopojsonFeatures=function(t,e){var n=i[t.locationmode],r=e.objects[n];return o(e,r).features}},{"../plots/geo/constants":795,"topojson-client":540}],747:[function(t,e,n){"use strict";e.exports={moduleType:"locale",name:"en-US",dictionary:{"Click to enter Colorscale title":"Click to enter Colorscale title"},format:{date:"%m/%d/%Y"}}},{}],748:[function(t,e,n){"use strict";e.exports={moduleType:"locale",name:"en",dictionary:{"Click to enter Colorscale title":"Click to enter Colourscale title"},format:{days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],periods:["AM","PM"],dateTime:"%a %b %e %X %Y",date:"%d/%m/%Y",time:"%H:%M:%S",decimal:".",thousands:",",grouping:[3],currency:["$",""],year:"%Y",month:"%b %Y",dayMonth:"%b %-d",dayMonthYear:"%b %-d, %Y"}}},{}],749:[function(t,e,n){"use strict";var r=t("../registry");e.exports=function(t){for(var e,n,i=r.layoutArrayContainers,o=r.layoutArrayRegexes,a=t.split("[")[0],s=0;s<o.length;s++)if((n=t.match(o[s]))&&0===n.index){e=n[0];break}if(e||(e=i[i.indexOf(a)]),!e)return!1;var A=t.substr(e.length);return A?!!(n=A.match(/^\[(0|[1-9][0-9]*)\](\.(.+))?$/))&&{array:e,index:Number(n[1]),property:n[3]||""}:{array:e,index:"",property:""}}},{"../registry":847}],750:[function(t,e,n){"use strict";var r=t("../lib"),i=r.extendFlat,o=r.isPlainObject,a={valType:"flaglist",extras:["none"],flags:["calc","clearAxisTypes","plot","style","markerSize","colorbars"]},s={valType:"flaglist",extras:["none"],flags:["calc","plot","legend","ticks","axrange","layoutstyle","modebar","camera","arraydraw","colorbars"]},A=a.flags.slice().concat(["fullReplot"]),l=s.flags.slice().concat("layoutReplot");function c(t){for(var e={},n=0;n<t.length;n++)e[t[n]]=!1;return e}function u(t,e,n){var r=i({},t);for(var a in r){var s=r[a];o(s)&&(r[a]=g(s,e,0,a))}return"from-root"===n&&(r.editType=e),r}function g(t,e,n,r){if(t.valType){var o=i({},t);if(o.editType=e,Array.isArray(t.items)){o.items=new Array(t.items.length);for(var a=0;a<t.items.length;a++)o.items[a]=g(t.items[a],e)}return o}return u(t,e,"_"===r.charAt(0)?"nested":"from-root")}e.exports={traces:a,layout:s,traceFlags:function(){return c(A)},layoutFlags:function(){return c(l)},update:function(t,e){var n=e.editType;if(n&&"none"!==n)for(var r=n.split("+"),i=0;i<r.length;i++)t[r[i]]=!0},overrideAll:u}},{"../lib":719}],751:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("gl-mat4/fromQuat"),o=t("../registry"),a=t("../lib"),s=t("../plots/plots"),A=t("../plots/cartesian/axis_ids"),l=t("../components/color"),c=A.cleanId,u=A.getFromTrace,g=o.traceIs;function h(t,e){var n=t[e],r=e.charAt(0);n&&"paper"!==n&&(t[e]=c(n,r))}function d(t){function e(e,n){var r=t[e],i=t.title&&t.title[n];r&&!i&&(t.title||(t.title={}),t.title[n]=t[e],delete t[e])}t&&("string"!=typeof t.title&&"number"!=typeof t.title||(t.title={text:t.title}),e("titlefont","font"),e("titleposition","position"),e("titleside","side"),e("titleoffset","offset"))}function f(t){if(!a.isPlainObject(t))return!1;var e=t.name;return delete t.name,delete t.showlegend,("string"==typeof e||"number"==typeof e)&&String(e)}function p(t,e,n,r){if(n&&!r)return t;if(r&&!n)return e;if(!t.trim())return e;if(!e.trim())return t;var i,o=Math.min(t.length,e.length);for(i=0;i<o&&t.charAt(i)===e.charAt(i);i++);return t.substr(0,i).trim()}function I(t){var e="middle",n="center";return"string"==typeof t&&(-1!==t.indexOf("top")?e="top":-1!==t.indexOf("bottom")&&(e="bottom"),-1!==t.indexOf("left")?n="left":-1!==t.indexOf("right")&&(n="right")),e+" "+n}function C(t,e){return e in t&&"object"==typeof t[e]&&0===Object.keys(t[e]).length}n.clearPromiseQueue=function(t){Array.isArray(t._promises)&&t._promises.length>0&&a.log("Clearing previous rejected promises from queue."),t._promises=[]},n.cleanLayout=function(t){var e,r;t||(t={}),t.xaxis1&&(t.xaxis||(t.xaxis=t.xaxis1),delete t.xaxis1),t.yaxis1&&(t.yaxis||(t.yaxis=t.yaxis1),delete t.yaxis1),t.scene1&&(t.scene||(t.scene=t.scene1),delete t.scene1);var o=(s.subplotsRegistry.cartesian||{}).attrRegex,A=(s.subplotsRegistry.polar||{}).attrRegex,u=(s.subplotsRegistry.ternary||{}).attrRegex,g=(s.subplotsRegistry.gl3d||{}).attrRegex,f=Object.keys(t);for(e=0;e<f.length;e++){var p=f[e];if(o&&o.test(p)){var I=t[p];I.anchor&&"free"!==I.anchor&&(I.anchor=c(I.anchor)),I.overlaying&&(I.overlaying=c(I.overlaying)),I.type||(I.isdate?I.type="date":I.islog?I.type="log":!1===I.isdate&&!1===I.islog&&(I.type="linear")),"withzero"!==I.autorange&&"tozero"!==I.autorange||(I.autorange=!0,I.rangemode="tozero"),delete I.islog,delete I.isdate,delete I.categories,C(I,"domain")&&delete I.domain,void 0!==I.autotick&&(void 0===I.tickmode&&(I.tickmode=I.autotick?"auto":"linear"),delete I.autotick),d(I)}else if(A&&A.test(p))d(t[p].radialaxis);else if(u&&u.test(p)){var m=t[p];d(m.aaxis),d(m.baxis),d(m.caxis)}else if(g&&g.test(p)){var y=t[p],E=y.cameraposition;if(Array.isArray(E)&&4===E[0].length){var v=E[0],B=E[1],b=E[2],Q=i([],v),w=[];for(r=0;r<3;++r)w[r]=B[r]+b*Q[2+4*r];y.camera={eye:{x:w[0],y:w[1],z:w[2]},center:{x:B[0],y:B[1],z:B[2]},up:{x:0,y:0,z:1}},delete y.cameraposition}d(y.xaxis),d(y.yaxis),d(y.zaxis)}}var x=Array.isArray(t.annotations)?t.annotations.length:0;for(e=0;e<x;e++){var _=t.annotations[e];a.isPlainObject(_)&&(_.ref&&("paper"===_.ref?(_.xref="paper",_.yref="paper"):"data"===_.ref&&(_.xref="x",_.yref="y"),delete _.ref),h(_,"xref"),h(_,"yref"))}var D=Array.isArray(t.shapes)?t.shapes.length:0;for(e=0;e<D;e++){var M=t.shapes[e];a.isPlainObject(M)&&(h(M,"xref"),h(M,"yref"))}var k=t.legend;return k&&(k.x>3?(k.x=1.02,k.xanchor="left"):k.x<-2&&(k.x=-.02,k.xanchor="right"),k.y>3?(k.y=1.02,k.yanchor="bottom"):k.y<-2&&(k.y=-.02,k.yanchor="top")),d(t),"rotate"===t.dragmode&&(t.dragmode="orbit"),l.clean(t),t.template&&t.template.layout&&n.cleanLayout(t.template.layout),t},n.cleanData=function(t){for(var e=0;e<t.length;e++){var r,i=t[e];if("histogramy"===i.type&&"xbins"in i&&!("ybins"in i)&&(i.ybins=i.xbins,delete i.xbins),i.error_y&&"opacity"in i.error_y){var A=l.defaults,u=i.error_y.color||(g(i,"bar")?l.defaultLine:A[e%A.length]);i.error_y.color=l.addOpacity(l.rgb(u),l.opacity(u)*i.error_y.opacity),delete i.error_y.opacity}if("bardir"in i&&("h"!==i.bardir||!g(i,"bar")&&"histogram"!==i.type.substr(0,9)||(i.orientation="h",n.swapXYData(i)),delete i.bardir),"histogramy"===i.type&&n.swapXYData(i),"histogramx"!==i.type&&"histogramy"!==i.type||(i.type="histogram"),"scl"in i&&!("colorscale"in i)&&(i.colorscale=i.scl,delete i.scl),"reversescl"in i&&!("reversescale"in i)&&(i.reversescale=i.reversescl,delete i.reversescl),i.xaxis&&(i.xaxis=c(i.xaxis,"x")),i.yaxis&&(i.yaxis=c(i.yaxis,"y")),g(i,"gl3d")&&i.scene&&(i.scene=s.subplotsRegistry.gl3d.cleanId(i.scene)),!g(i,"pie-like")&&!g(i,"bar-like"))if(Array.isArray(i.textposition))for(r=0;r<i.textposition.length;r++)i.textposition[r]=I(i.textposition[r]);else i.textposition&&(i.textposition=I(i.textposition));var h=o.getModule(i);if(h&&h.colorbar){var m=h.colorbar.container,y=m?i[m]:i;y&&y.colorscale&&("YIGnBu"===y.colorscale&&(y.colorscale="YlGnBu"),"YIOrRd"===y.colorscale&&(y.colorscale="YlOrRd"))}if("surface"===i.type&&a.isPlainObject(i.contours)){var E=["x","y","z"];for(r=0;r<E.length;r++){var v=i.contours[E[r]];a.isPlainObject(v)&&(v.highlightColor&&(v.highlightcolor=v.highlightColor,delete v.highlightColor),v.highlightWidth&&(v.highlightwidth=v.highlightWidth,delete v.highlightWidth))}}if("candlestick"===i.type||"ohlc"===i.type){var B=!1!==(i.increasing||{}).showlegend,b=!1!==(i.decreasing||{}).showlegend,Q=f(i.increasing),w=f(i.decreasing);if(!1!==Q&&!1!==w){var x=p(Q,w,B,b);x&&(i.name=x)}else!Q&&!w||i.name||(i.name=Q||w)}if(Array.isArray(i.transforms)){var _=i.transforms;for(r=0;r<_.length;r++){var D=_[r];if(a.isPlainObject(D))switch(D.type){case"filter":D.filtersrc&&(D.target=D.filtersrc,delete D.filtersrc),D.calendar&&(D.valuecalendar||(D.valuecalendar=D.calendar),delete D.calendar);break;case"groupby":if(D.styles=D.styles||D.style,D.styles&&!Array.isArray(D.styles)){var M=D.styles,k=Object.keys(M);D.styles=[];for(var S=0;S<k.length;S++)D.styles.push({target:k[S],value:M[k[S]]})}}}}C(i,"line")&&delete i.line,"marker"in i&&(C(i.marker,"line")&&delete i.marker.line,C(i,"marker")&&delete i.marker),l.clean(i),i.autobinx&&(delete i.autobinx,delete i.xbins),i.autobiny&&(delete i.autobiny,delete i.ybins),d(i),i.colorbar&&d(i.colorbar),i.marker&&i.marker.colorbar&&d(i.marker.colorbar),i.line&&i.line.colorbar&&d(i.line.colorbar),i.aaxis&&d(i.aaxis),i.baxis&&d(i.baxis)}},n.swapXYData=function(t){var e;if(a.swapAttrs(t,["?","?0","d?","?bins","nbins?","autobin?","?src","error_?"]),Array.isArray(t.z)&&Array.isArray(t.z[0])&&(t.transpose?delete t.transpose:t.transpose=!0),t.error_x&&t.error_y){var n=t.error_y,r="copy_ystyle"in n?n.copy_ystyle:!(n.color||n.thickness||n.width);a.swapAttrs(t,["error_?.copy_ystyle"]),r&&a.swapAttrs(t,["error_?.color","error_?.thickness","error_?.width"])}if("string"==typeof t.hoverinfo){var i=t.hoverinfo.split("+");for(e=0;e<i.length;e++)"x"===i[e]?i[e]="y":"y"===i[e]&&(i[e]="x");t.hoverinfo=i.join("+")}},n.coerceTraceIndices=function(t,e){if(r(e))return[e];if(!Array.isArray(e)||!e.length)return t.data.map(function(t,e){return e});if(Array.isArray(e)){for(var n=[],i=0;i<e.length;i++)a.isIndex(e[i],t.data.length)?n.push(e[i]):a.warn("trace index (",e[i],") is not a number or is out of bounds");return n}return e},n.manageArrayContainers=function(t,e,n){var i=t.obj,o=t.parts,s=o.length,A=o[s-1],l=r(A);if(l&&null===e){var c=o.slice(0,s-1).join(".");a.nestedProperty(i,c).get().splice(A,1)}else l&&void 0===t.get()?(void 0===t.get()&&(n[t.astr]=null),t.set(e)):t.set(e)};var m=/(\.[^\[\]\.]+|\[[^\[\]\.]+\])$/;function y(t){var e=t.search(m);if(e>0)return t.substr(0,e)}n.hasParent=function(t,e){for(var n=y(e);n;){if(n in t)return!0;n=y(n)}return!1};var E=["x","y","z"];n.clearAxisTypes=function(t,e,n){for(var r=0;r<e.length;r++)for(var i=t._fullData[r],o=0;o<3;o++){var s=u(t,i,E[o]);if(s&&"log"!==s.type){var A=s._name,l=s._id.substr(1);if("scene"===l.substr(0,5)){if(void 0!==n[l])continue;A=l+"."+A}var c=A+".type";void 0===n[A]&&void 0===n[c]&&a.nestedProperty(t.layout,c).set(null)}}}},{"../components/color":593,"../lib":719,"../plots/cartesian/axis_ids":770,"../plots/plots":828,"../registry":847,"fast-isnumeric":225,"gl-mat4/fromQuat":262}],752:[function(t,e,n){"use strict";var r=t("./plot_api");n.plot=r.plot,n.newPlot=r.newPlot,n.restyle=r.restyle,n.relayout=r.relayout,n.redraw=r.redraw,n.update=r.update,n._guiRestyle=r._guiRestyle,n._guiRelayout=r._guiRelayout,n._guiUpdate=r._guiUpdate,n._storeDirectGUIEdit=r._storeDirectGUIEdit,n.react=r.react,n.extendTraces=r.extendTraces,n.prependTraces=r.prependTraces,n.addTraces=r.addTraces,n.deleteTraces=r.deleteTraces,n.moveTraces=r.moveTraces,n.purge=r.purge,n.addFrames=r.addFrames,n.deleteFrames=r.deleteFrames,n.animate=r.animate,n.setPlotConfig=r.setPlotConfig,n.toImage=t("./to_image"),n.validate=t("./validate"),n.downloadImage=t("../snapshot/download");var i=t("./template_api");n.makeTemplate=i.makeTemplate,n.validateTemplate=i.validateTemplate},{"../snapshot/download":849,"./plot_api":754,"./template_api":759,"./to_image":760,"./validate":761}],753:[function(t,e,n){"use strict";var r=t("../lib/is_plain_object"),i=t("../lib/noop"),o=t("../lib/loggers"),a=t("../lib/search").sorterAsc,s=t("../registry");n.containerArrayMatch=t("./container_array_match");var A=n.isAddVal=function(t){return"add"===t||r(t)},l=n.isRemoveVal=function(t){return null===t||"remove"===t};n.applyContainerArrayChanges=function(t,e,n,r,c){var u=e.astr,g=s.getComponentMethod(u,"supplyLayoutDefaults"),h=s.getComponentMethod(u,"draw"),d=s.getComponentMethod(u,"drawOne"),f=r.replot||r.recalc||g===i||h===i,p=t.layout,I=t._fullLayout;if(n[""]){Object.keys(n).length>1&&o.warn("Full array edits are incompatible with other edits",u);var C=n[""][""];if(l(C))e.set(null);else{if(!Array.isArray(C))return o.warn("Unrecognized full array edit value",u,C),!0;e.set(C)}return!f&&(g(p,I),h(t),!0)}var m,y,E,v,B,b,Q,w,x=Object.keys(n).map(Number).sort(a),_=e.get(),D=_||[],M=c(I,u).get(),k=[],S=-1,L=D.length;for(m=0;m<x.length;m++)if(v=n[E=x[m]],B=Object.keys(v),b=v[""],Q=A(b),E<0||E>D.length-(Q?0:1))o.warn("index out of range",u,E);else if(void 0!==b)B.length>1&&o.warn("Insertion & removal are incompatible with edits to the same index.",u,E),l(b)?k.push(E):Q?("add"===b&&(b={}),D.splice(E,0,b),M&&M.splice(E,0,{})):o.warn("Unrecognized full object edit value",u,E,b),-1===S&&(S=E);else for(y=0;y<B.length;y++)w=u+"["+E+"].",c(D[E],B[y],w).set(v[B[y]]);for(m=k.length-1;m>=0;m--)D.splice(k[m],1),M&&M.splice(k[m],1);if(D.length?_||e.set(D):e.set(null),f)return!1;if(g(p,I),d!==i){var R;if(-1===S)R=x;else{for(L=Math.max(D.length,L),R=[],m=0;m<x.length&&!((E=x[m])>=S);m++)R.push(E);for(m=S;m<L;m++)R.push(m)}for(m=0;m<R.length;m++)d(t,R[m])}else h(t);return!0}},{"../lib/is_plain_object":720,"../lib/loggers":723,"../lib/noop":728,"../lib/search":738,"../registry":847,"./container_array_match":749}],754:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=t("has-hover"),a=t("../lib"),s=a.nestedProperty,A=t("../lib/events"),l=t("../lib/queue"),c=t("../registry"),u=t("./plot_schema"),g=t("../plots/plots"),h=t("../plots/polar/legacy"),d=t("../plots/cartesian/axes"),f=t("../components/drawing"),p=t("../components/color"),I=t("../plots/cartesian/graph_interact").initInteractions,C=t("../constants/xmlns_namespaces"),m=t("../lib/svg_text_utils"),y=t("../plots/cartesian/select").clearSelect,E=t("./plot_config").dfltConfig,v=t("./manage_arrays"),B=t("./helpers"),b=t("./subroutines"),Q=t("./edit_types"),w=t("../plots/cartesian/constants").AX_NAME_PATTERN,x=0;function _(t){var e=t._fullLayout;e._redrawFromAutoMarginCount?e._redrawFromAutoMarginCount--:t.emit("plotly_afterplot")}function D(t,e){try{t._fullLayout._paper.style("background",e)}catch(t){a.error(t)}}function M(t,e){D(t,p.combine(e,"white"))}function k(t,e){if(!t._context){t._context=a.extendDeep({},E);var n=r.select("base");t._context._baseUrl=n.size()&&n.attr("href")?window.location.href.split("#")[0]:""}var i,s,A,l=t._context;if(e){for(s=Object.keys(e),i=0;i<s.length;i++)"editable"!==(A=s[i])&&"edits"!==A&&A in l&&("setBackground"===A&&"opaque"===e[A]?l[A]=M:l[A]=e[A]);e.plot3dPixelRatio&&!l.plotGlPixelRatio&&(l.plotGlPixelRatio=l.plot3dPixelRatio);var c=e.editable;if(void 0!==c)for(l.editable=c,s=Object.keys(l.edits),i=0;i<s.length;i++)l.edits[s[i]]=c;if(e.edits)for(s=Object.keys(e.edits),i=0;i<s.length;i++)(A=s[i])in l.edits&&(l.edits[A]=e.edits[A]);l._exportedPlot=e._exportedPlot}l.staticPlot&&(l.editable=!1,l.edits={},l.autosizable=!1,l.scrollZoom=!1,l.doubleClick=!1,l.showTips=!1,l.showLink=!1,l.displayModeBar=!1),"hover"!==l.displayModeBar||o||(l.displayModeBar=!0),"transparent"!==l.setBackground&&"function"==typeof l.setBackground||(l.setBackground=D),l._hasZeroHeight=l._hasZeroHeight||0===t.clientHeight,l._hasZeroWidth=l._hasZeroWidth||0===t.clientWidth;var u=l.scrollZoom,g=l._scrollZoom={};if(!0===u)g.cartesian=1,g.gl3d=1,g.geo=1,g.mapbox=1;else if("string"==typeof u){var h=u.split("+");for(i=0;i<h.length;i++)g[h[i]]=1}else!1!==u&&(g.gl3d=1,g.geo=1,g.mapbox=1)}function S(t,e){var n,r,i=e+1,o=[];for(n=0;n<t.length;n++)(r=t[n])<0?o.push(i+r):o.push(r);return o}function L(t,e,n){var r,i;for(r=0;r<e.length;r++){if((i=e[r])!==parseInt(i,10))throw new Error("all values in "+n+" must be integers");if(i>=t.data.length||i<-t.data.length)throw new Error(n+" must be valid indices for gd.data.");if(e.indexOf(i,r+1)>-1||i>=0&&e.indexOf(-t.data.length+i)>-1||i<0&&e.indexOf(t.data.length+i)>-1)throw new Error("each index in "+n+" must be unique.")}}function R(t,e,n){if(!Array.isArray(t.data))throw new Error("gd.data must be an array.");if(void 0===e)throw new Error("currentIndices is a required argument.");if(Array.isArray(e)||(e=[e]),L(t,e,"currentIndices"),void 0===n||Array.isArray(n)||(n=[n]),void 0!==n&&L(t,n,"newIndices"),void 0!==n&&e.length!==n.length)throw new Error("current and new indices must be of equal length.")}function N(t,e,n,r,o){!function(t,e,n,r){var i=a.isPlainObject(r);if(!Array.isArray(t.data))throw new Error("gd.data must be an array");if(!a.isPlainObject(e))throw new Error("update must be a key:value object");if(void 0===n)throw new Error("indices must be an integer or array of integers");for(var o in L(t,n,"indices"),e){if(!Array.isArray(e[o])||e[o].length!==n.length)throw new Error("attribute "+o+" must be an array of length equal to indices array length");if(i&&(!(o in r)||!Array.isArray(r[o])||r[o].length!==e[o].length))throw new Error("when maxPoints is set as a key:value object it must contain a 1:1 corrispondence with the keys and number of traces in the update object")}}(t,e,n,r);for(var A=function(t,e,n,r){var o,A,l,c,u,g=a.isPlainObject(r),h=[];for(var d in Array.isArray(n)||(n=[n]),n=S(n,t.data.length-1),e)for(var f=0;f<n.length;f++){if(o=t.data[n[f]],A=(l=s(o,d)).get(),c=e[d][f],!a.isArrayOrTypedArray(c))throw new Error("attribute: "+d+" index: "+f+" must be an array");if(!a.isArrayOrTypedArray(A))throw new Error("cannot extend missing or non-array attribute: "+d);if(A.constructor!==c.constructor)throw new Error("cannot extend array with an array of a different type: "+d);u=g?r[d][f]:r,i(u)||(u=-1),h.push({prop:l,target:A,insert:c,maxp:Math.floor(u)})}return h}(t,e,n,r),l={},c={},u=0;u<A.length;u++){var g=A[u].prop,h=A[u].maxp,d=o(A[u].target,A[u].insert,h);g.set(d[0]),Array.isArray(l[g.astr])||(l[g.astr]=[]),l[g.astr].push(d[1]),Array.isArray(c[g.astr])||(c[g.astr]=[]),c[g.astr].push(A[u].target.length)}return{update:l,maxPoints:c}}function F(t,e){var n=new t.constructor(t.length+e.length);return n.set(t),n.set(e,t.length),n}function T(t,e,r,i){t=a.getGraphDiv(t),B.clearPromiseQueue(t);var o={};if("string"==typeof e)o[e]=r;else{if(!a.isPlainObject(e))return a.warn("Restyle fail.",e,r,i),Promise.reject();o=a.extendFlat({},e),void 0===i&&(i=r)}Object.keys(o).length&&(t.changed=!0);var s=B.coerceTraceIndices(t,i),A=H(t,o,s),c=A.flags;c.calc&&(t.calcdata=void 0),c.clearAxisTypes&&B.clearAxisTypes(t,s,{});var u=[];c.fullReplot?u.push(n.plot):(u.push(g.previousPromises),g.supplyDefaults(t),c.markerSize&&(g.doCalcdata(t),K(u)),c.style&&u.push(b.doTraceStyle),c.colorbars&&u.push(b.doColorBars),u.push(_)),u.push(g.rehover,g.redrag),l.add(t,T,[t,A.undoit,A.traces],T,[t,A.redoit,A.traces]);var h=a.syncOrAsync(u,t);return h&&h.then||(h=Promise.resolve()),h.then(function(){return t.emit("plotly_restyle",A.eventData),t})}function U(t){return void 0===t?null:t}function O(t,e){return e?function(e,n,r){var i=s(e,n),o=i.set;return i.set=function(e){G((r||"")+n,i.get(),e,t),o(e)},i}:s}function G(t,e,n,r){if(Array.isArray(e)||Array.isArray(n))for(var i=Array.isArray(e)?e:[],o=Array.isArray(n)?n:[],s=Math.max(i.length,o.length),A=0;A<s;A++)G(t+"["+A+"]",i[A],o[A],r);else if(a.isPlainObject(e)||a.isPlainObject(n)){var l=a.isPlainObject(e)?e:{},c=a.isPlainObject(n)?n:{},u=a.extendFlat({},l,c);for(var g in u)G(t+"."+g,l[g],c[g],r)}else void 0===r[t]&&(r[t]=U(e))}function H(t,e,n){var r,i=t._fullLayout,o=t._fullData,A=t.data,l=i._guiEditing,h=O(i._preGUI,l),f=a.extendDeepAll({},e);j(e);var p,I=Q.traceFlags(),C={},m={};function y(){return n.map(function(){})}function E(t){var e=d.id2name(t);-1===p.indexOf(e)&&p.push(e)}function v(t){return"LAYOUT"+t+".autorange"}function b(t){return"LAYOUT"+t+".range"}function w(t){for(var e=t;e<o.length;e++)if(o[e]._input===A[t])return o[e]}function x(r,o,a){if(Array.isArray(r))r.forEach(function(t){x(t,o,a)});else if(!(r in e||B.hasParent(e,r))){var s;if("LAYOUT"===r.substr(0,6))s=h(t.layout,r.replace("LAYOUT",""));else{var c=n[a];s=O(i._tracePreGUI[w(c)._fullInput.uid],l)(A[c],r)}r in m||(m[r]=y()),void 0===m[r][a]&&(m[r][a]=U(s.get())),void 0!==o&&s.set(o)}}function _(t){return function(e){return o[e][t]}}function D(t){return function(e,r){return!1===e?o[n[r]][t]:null}}for(var M in e){if(B.hasParent(e,M))throw new Error("cannot set "+M+" and a parent attribute simultaneously");var k,S,L,R,N,F,T=e[M];if("autobinx"!==M&&"autobiny"!==M||(M=M.charAt(M.length-1)+"bins",T=Array.isArray(T)?T.map(D(M)):!1===T?n.map(_(M)):null),C[M]=T,"LAYOUT"!==M.substr(0,6)){for(m[M]=y(),r=0;r<n.length;r++)if(k=A[n[r]],S=w(n[r]),R=(L=O(i._tracePreGUI[S._fullInput.uid],l)(k,M)).get(),void 0!==(N=Array.isArray(T)?T[r%T.length]:T)){var G=L.parts[L.parts.length-1],H=M.substr(0,M.length-G.length-1),Y=H?H+".":"",P=H?s(S,H).get():S;if((F=u.getTraceValObject(S,L.parts))&&F.impliedEdits&&null!==N)for(var K in F.impliedEdits)x(a.relativeAttr(M,K),F.impliedEdits[K],r);else if("thicknessmode"!==G&&"lenmode"!==G||R===N||"fraction"!==N&&"pixels"!==N||!P){if("type"===M&&("pie"===N!=("pie"===R)||"funnelarea"===N!=("funnelarea"===R))){var z="x",J="y";"bar"!==N&&"bar"!==R||"h"!==k.orientation||(z="y",J="x"),a.swapAttrs(k,["?","?src"],"labels",z),a.swapAttrs(k,["d?","?0"],"label",z),a.swapAttrs(k,["?","?src"],"values",J),"pie"===R||"funnelarea"===R?(s(k,"marker.color").set(s(k,"marker.colors").get()),i._pielayer.selectAll("g.trace").remove()):c.traceIs(k,"cartesian")&&s(k,"marker.colors").set(s(k,"marker.color").get())}}else{var q=i._size,V=P.orient,W="top"===V||"bottom"===V;if("thicknessmode"===G){var X=W?q.h:q.w;x(Y+"thickness",P.thickness*("fraction"===N?1/X:X),r)}else{var Z=W?q.w:q.h;x(Y+"len",P.len*("fraction"===N?1/Z:Z),r)}}if(m[M][r]=U(R),-1!==["swapxy","swapxyaxes","orientation","orientationaxes"].indexOf(M)){if("orientation"===M){L.set(N);var $=k.x&&!k.y?"h":"v";if((L.get()||$)===S.orientation)continue}else"orientationaxes"===M&&(k.orientation={v:"h",h:"v"}[S.orientation]);B.swapXYData(k),I.calc=I.clearAxisTypes=!0}else-1!==g.dataArrayContainers.indexOf(L.parts[0])?(B.manageArrayContainers(L,N,m),I.calc=!0):(F?F.arrayOk&&!c.traceIs(S,"regl")&&(a.isArrayOrTypedArray(N)||a.isArrayOrTypedArray(R))?I.calc=!0:Q.update(I,F):I.calc=!0,L.set(N))}if(-1!==["swapxyaxes","orientationaxes"].indexOf(M)&&d.swap(t,n),"orientationaxes"===M){var tt=s(t.layout,"hovermode");"x"===tt.get()?tt.set("y"):"y"===tt.get()&&tt.set("x")}if(-1!==["orientation","type"].indexOf(M)){for(p=[],r=0;r<n.length;r++){var et=A[n[r]];c.traceIs(et,"cartesian")&&(E(et.xaxis||"x"),E(et.yaxis||"y"))}x(p.map(v),!0,0),x(p.map(b),[0,1],0)}}else L=h(t.layout,M.replace("LAYOUT","")),m[M]=[U(L.get())],L.set(Array.isArray(T)?T[0]:T),I.calc=!0}return(I.calc||I.plot)&&(I.fullReplot=!0),{flags:I,undoit:m,redoit:C,traces:n,eventData:a.extendDeepNoArrays([],[f,n])}}function j(t){var e,n,r,i=a.counterRegex("axis",".title",!1,!1),o=/colorbar\.title$/,s=Object.keys(t);for(e=0;e<s.length;e++)n=s[e],r=t[n],"title"!==n&&!i.test(n)&&!o.test(n)||"string"!=typeof r&&"number"!=typeof r?n.indexOf("titlefont")>-1?A(n,n.replace("titlefont","title.font")):n.indexOf("titleposition")>-1?A(n,n.replace("titleposition","title.position")):n.indexOf("titleside")>-1?A(n,n.replace("titleside","title.side")):n.indexOf("titleoffset")>-1&&A(n,n.replace("titleoffset","title.offset")):A(n,n.replace("title","title.text"));function A(e,n){t[n]=t[e],delete t[e]}}function Y(t,e,n){if(t=a.getGraphDiv(t),B.clearPromiseQueue(t),t.framework&&t.framework.isPolar)return Promise.resolve(t);var r={};if("string"==typeof e)r[e]=n;else{if(!a.isPlainObject(e))return a.warn("Relayout fail.",e,n),Promise.reject();r=a.extendFlat({},e)}Object.keys(r).length&&(t.changed=!0);var i=V(t,r),o=i.flags;o.calc&&(t.calcdata=void 0);var s=[g.previousPromises];o.layoutReplot?s.push(b.layoutReplot):Object.keys(r).length&&(P(t,o,i)||g.supplyDefaults(t),o.legend&&s.push(b.doLegend),o.layoutstyle&&s.push(b.layoutStyles),o.axrange&&K(s,i.rangesAltered),o.ticks&&s.push(b.doTicksRelayout),o.modebar&&s.push(b.doModeBar),o.camera&&s.push(b.doCamera),o.colorbars&&s.push(b.doColorBars),s.push(_)),s.push(g.rehover,g.redrag),l.add(t,Y,[t,i.undoit],Y,[t,i.redoit]);var A=a.syncOrAsync(s,t);return A&&A.then||(A=Promise.resolve(t)),A.then(function(){return t.emit("plotly_relayout",i.eventData),t})}function P(t,e,n){var r=t._fullLayout;if(!e.axrange)return!1;for(var i in e)if("axrange"!==i&&e[i])return!1;for(var o in n.rangesAltered){var a=d.id2name(o),s=t.layout[a],A=r[a];if(A.autorange=s.autorange,A.range=s.range.slice(),A.cleanRange(),A._matchGroup)for(var l in A._matchGroup)if(l!==o){var c=r[d.id2name(l)];c.autorange=A.autorange,c.range=A.range.slice(),c._input.range=A.range.slice()}}return!0}function K(t,e){var n=e?function(t){var n=[],r=!0;for(var i in e){var o=d.getFromId(t,i);if(n.push(i),o._matchGroup)for(var a in o._matchGroup)e[a]||n.push(a);o.automargin&&(r=!1)}return d.draw(t,n,{skipTitle:r})}:function(t){return d.draw(t,"redraw")};t.push(y,b.doAutoRangeAndConstraints,n,b.drawData,b.finalDraw)}var z=/^[xyz]axis[0-9]*\.range(\[[0|1]\])?$/,J=/^[xyz]axis[0-9]*\.autorange$/,q=/^[xyz]axis[0-9]*\.domain(\[[0|1]\])?$/;function V(t,e){var n,r,i,o=t.layout,A=t._fullLayout,l=A._guiEditing,g=O(A._preGUI,l),h=Object.keys(e),f=d.list(t),p=a.extendDeepAll({},e),I={};for(j(e),h=Object.keys(e),r=0;r<h.length;r++)if(0===h[r].indexOf("allaxes")){for(i=0;i<f.length;i++){var C=f[i]._id.substr(1),m=-1!==C.indexOf("scene")?C+".":"",y=h[r].replace("allaxes",m+f[i]._name);e[y]||(e[y]=e[h[r]])}delete e[h[r]]}var E=Q.layoutFlags(),b={},x={};function _(t,n){if(Array.isArray(t))t.forEach(function(t){_(t,n)});else if(!(t in e||B.hasParent(e,t))){var r=g(o,t);t in x||(x[t]=U(r.get())),void 0!==n&&r.set(n)}}var D,M={};function k(t){var e=d.name2id(t.split(".")[0]);return M[e]=1,e}for(var S in e){if(B.hasParent(e,S))throw new Error("cannot set "+S+" and a parent attribute simultaneously");for(var L=g(o,S),R=e[S],N=L.parts.length-1;N>0&&"string"!=typeof L.parts[N];)N--;var F=L.parts[N],T=L.parts[N-1]+"."+F,G=L.parts.slice(0,N).join("."),H=s(t.layout,G).get(),Y=s(A,G).get(),P=L.get();if(void 0!==R){b[S]=R,x[S]="reverse"===F?R:U(P);var K=u.getLayoutValObject(A,L.parts);if(K&&K.impliedEdits&&null!==R)for(var V in K.impliedEdits)_(a.relativeAttr(S,V),K.impliedEdits[V]);if(-1!==["width","height"].indexOf(S))if(R){_("autosize",null);var X="height"===S?"width":"height";_(X,A[X])}else A[S]=t._initialAutoSize[S];else if("autosize"===S)_("width",R?null:A.width),_("height",R?null:A.height);else if(T.match(z))k(T),s(A,G+"._inputRange").set(null);else if(T.match(J)){k(T),s(A,G+"._inputRange").set(null);var Z=s(A,G).get();Z._inputDomain&&(Z._input.domain=Z._inputDomain.slice())}else T.match(q)&&s(A,G+"._inputDomain").set(null);if("type"===F){var $=H,tt="linear"===Y.type&&"log"===R,et="log"===Y.type&&"linear"===R;if(tt||et){if($&&$.range)if(Y.autorange)tt&&($.range=$.range[1]>$.range[0]?[1,2]:[2,1]);else{var nt=$.range[0],rt=$.range[1];tt?(nt<=0&&rt<=0&&_(G+".autorange",!0),nt<=0?nt=rt/1e6:rt<=0&&(rt=nt/1e6),_(G+".range[0]",Math.log(nt)/Math.LN10),_(G+".range[1]",Math.log(rt)/Math.LN10)):(_(G+".range[0]",Math.pow(10,nt)),_(G+".range[1]",Math.pow(10,rt)))}else _(G+".autorange",!0);Array.isArray(A._subplots.polar)&&A._subplots.polar.length&&A[L.parts[0]]&&"radialaxis"===L.parts[1]&&delete A[L.parts[0]]._subplot.viewInitial["radialaxis.range"],c.getComponentMethod("annotations","convertCoords")(t,Y,R,_),c.getComponentMethod("images","convertCoords")(t,Y,R,_)}else _(G+".autorange",!0),_(G+".range",null);s(A,G+"._inputRange").set(null)}else if(F.match(w)){var it=s(A,S).get(),ot=(R||{}).type;ot&&"-"!==ot||(ot="linear"),c.getComponentMethod("annotations","convertCoords")(t,it,ot,_),c.getComponentMethod("images","convertCoords")(t,it,ot,_)}var at=v.containerArrayMatch(S);if(at){n=at.array,r=at.index;var st=at.property,At=K||{editType:"calc"};""!==r&&""===st&&(v.isAddVal(R)?x[S]=null:v.isRemoveVal(R)?x[S]=(s(o,n).get()||[])[r]:a.warn("unrecognized full object value",e)),Q.update(E,At),I[n]||(I[n]={});var lt=I[n][r];lt||(lt=I[n][r]={}),lt[st]=R,delete e[S]}else"reverse"===F?(H.range?H.range.reverse():(_(G+".autorange",!0),H.range=[1,0]),Y.autorange?E.calc=!0:E.plot=!0):(A._has("scatter-like")&&A._has("regl")&&"dragmode"===S&&("lasso"===R||"select"===R)&&"lasso"!==P&&"select"!==P?E.plot=!0:A._has("gl2d")?E.plot=!0:K?Q.update(E,K):E.calc=!0,L.set(R))}}for(n in I)v.applyContainerArrayChanges(t,g(o,n),I[n],E,g)||(E.plot=!0);var ct=A._axisConstraintGroups||[];for(D in M)for(r=0;r<ct.length;r++){var ut=ct[r];if(ut[D])for(var gt in E.calc=!0,ut)M[gt]||(d.getFromId(t,gt)._constraintShrinkable=!0)}return(W(t)||e.height||e.width)&&(E.plot=!0),(E.plot||E.calc)&&(E.layoutReplot=!0),{flags:E,rangesAltered:M,undoit:x,redoit:b,eventData:p}}function W(t){var e=t._fullLayout,n=e.width,r=e.height;return t.layout.autosize&&g.plotAutoSize(t,t.layout,e),e.width!==n||e.height!==r}function X(t,e,r,i){if(t=a.getGraphDiv(t),B.clearPromiseQueue(t),t.framework&&t.framework.isPolar)return Promise.resolve(t);a.isPlainObject(e)||(e={}),a.isPlainObject(r)||(r={}),Object.keys(e).length&&(t.changed=!0),Object.keys(r).length&&(t.changed=!0);var o=B.coerceTraceIndices(t,i),s=H(t,a.extendFlat({},e),o),A=s.flags,c=V(t,a.extendFlat({},r)),u=c.flags;(A.calc||u.calc)&&(t.calcdata=void 0),A.clearAxisTypes&&B.clearAxisTypes(t,o,r);var h=[];u.layoutReplot?h.push(b.layoutReplot):A.fullReplot?h.push(n.plot):(h.push(g.previousPromises),P(t,u,c)||g.supplyDefaults(t),A.style&&h.push(b.doTraceStyle),(A.colorbars||u.colorbars)&&h.push(b.doColorBars),u.legend&&h.push(b.doLegend),u.layoutstyle&&h.push(b.layoutStyles),u.axrange&&K(h,c.rangesAltered),u.ticks&&h.push(b.doTicksRelayout),u.modebar&&h.push(b.doModeBar),u.camera&&h.push(b.doCamera),h.push(_)),h.push(g.rehover,g.redrag),l.add(t,X,[t,s.undoit,c.undoit,s.traces],X,[t,s.redoit,c.redoit,s.traces]);var d=a.syncOrAsync(h,t);return d&&d.then||(d=Promise.resolve(t)),d.then(function(){return t.emit("plotly_update",{data:s.eventData,layout:c.eventData}),t})}function Z(t){return function(e){e._fullLayout._guiEditing=!0;var n=t.apply(null,arguments);return e._fullLayout._guiEditing=!1,n}}var $=[{pattern:/^hiddenlabels/,attr:"legend.uirevision"},{pattern:/^((x|y)axis\d*)\.((auto)?range|title\.text)/},{pattern:/axis\d*\.showspikes$/,attr:"modebar.uirevision"},{pattern:/(hover|drag)mode$/,attr:"modebar.uirevision"},{pattern:/^(scene\d*)\.camera/},{pattern:/^(geo\d*)\.(projection|center)/},{pattern:/^(ternary\d*\.[abc]axis)\.(min|title\.text)$/},{pattern:/^(polar\d*\.radialaxis)\.((auto)?range|angle|title\.text)/},{pattern:/^(polar\d*\.angularaxis)\.rotation/},{pattern:/^(mapbox\d*)\.(center|zoom|bearing|pitch)/},{pattern:/^legend\.(x|y)$/,attr:"editrevision"},{pattern:/^(shapes|annotations)/,attr:"editrevision"},{pattern:/^title\.text$/,attr:"editrevision"}],tt=[{pattern:/^selectedpoints$/,attr:"selectionrevision"},{pattern:/(^|value\.)visible$/,attr:"legend.uirevision"},{pattern:/^dimensions\[\d+\]\.constraintrange/},{pattern:/^node\.(x|y|groups)/},{pattern:/^level$/},{pattern:/(^|value\.)name$/},{pattern:/colorbar\.title\.text$/},{pattern:/colorbar\.(x|y)$/,attr:"editrevision"}];function et(t,e){for(var n=0;n<e.length;n++){var r=e[n],i=t.match(r.pattern);if(i)return{head:i[1],attr:r.attr}}}function nt(t,e){var n=s(e,t).get();if(void 0!==n)return n;var r=t.split(".");for(r.pop();r.length>1;)if(r.pop(),void 0!==(n=s(e,r.join(".")+".uirevision").get()))return n;return e.uirevision}function rt(t,e){for(var n=0;n<e.length;n++)if(e[n]._fullInput.uid===t)return n;return-1}function it(t,e,n){for(var r=0;r<e.length;r++)if(e[r].uid===t)return r;return!e[n]||e[n].uid?-1:n}function ot(t,e){var n=a.isPlainObject(t),r=Array.isArray(t);return n||r?(n&&a.isPlainObject(e)||r&&Array.isArray(e))&&JSON.stringify(t)===JSON.stringify(e):t===e}function at(t,e,n,r){var i,o,A,l=r.getValObject,c=r.flags,u=r.immutable,g=r.inArray,h=r.arrayIndex;function d(){var t=i.editType;g&&-1!==t.indexOf("arraydraw")?a.pushUnique(c.arrays[g],h):(Q.update(c,i),"none"!==t&&c.nChanges++,r.transition&&i.anim&&c.nChangesAnim++,(z.test(A)||J.test(A))&&(c.rangesAltered[n[0]]=1),q.test(A)&&s(e,"_inputDomain").set(null),"datarevision"===o&&(c.newDataRevision=1))}function f(t){return"data_array"===t.valType||t.arrayOk}for(o in t){if(c.calc&&!r.transition)return;var p=t[o],I=e[o],C=n.concat(o);if(A=C.join("."),"_"!==o.charAt(0)&&"function"!=typeof p&&p!==I){if(("tick0"===o||"dtick"===o)&&"geo"!==n[0]){var m=e.tickmode;if("auto"===m||"array"===m||!m)continue}if(("range"!==o||!e.autorange)&&("zmin"!==o&&"zmax"!==o||"contourcarpet"!==e.type)&&(i=l(C))&&(!i._compareAsJSON||JSON.stringify(p)!==JSON.stringify(I))){var y,E=i.valType,v=f(i),B=Array.isArray(p),b=Array.isArray(I);if(B&&b){var w="_input_"+o,x=t[w],_=e[w];if(Array.isArray(x)&&x===_)continue}if(void 0===I)v&&B?c.calc=!0:d();else if(i._isLinkedToArray){var D=[],M=!1;g||(c.arrays[o]=D);var k=Math.min(p.length,I.length),S=Math.max(p.length,I.length);if(k!==S){if("arraydraw"!==i.editType){d();continue}M=!0}for(y=0;y<k;y++)at(p[y],I[y],C.concat(y),a.extendFlat({inArray:o,arrayIndex:y},r));if(M)for(y=k;y<S;y++)D.push(y)}else!E&&a.isPlainObject(p)?at(p,I,C,r):v?B&&b?(u&&(c.calc=!0),(u||r.newDataRevision)&&d()):B!==b?c.calc=!0:d():B&&b&&p.length===I.length&&String(p)===String(I)||d()}}}for(o in e)if(!(o in t||"_"===o.charAt(0)||"function"==typeof e[o])){if(f(i=l(n.concat(o)))&&Array.isArray(e[o]))return void(c.calc=!0);d()}}function st(t){var e=r.select(t),n=t._fullLayout;if(n._container=e.selectAll(".plot-container").data([0]),n._container.enter().insert("div",":first-child").classed("plot-container",!0).classed("plotly",!0),n._paperdiv=n._container.selectAll(".svg-container").data([0]),n._paperdiv.enter().append("div").classed("svg-container",!0).style("position","relative"),n._glcontainer=n._paperdiv.selectAll(".gl-container").data([{}]),n._glcontainer.enter().append("div").classed("gl-container",!0),n._paperdiv.selectAll(".main-svg").remove(),n._paperdiv.select(".modebar-container").remove(),n._paper=n._paperdiv.insert("svg",":first-child").classed("main-svg",!0),n._toppaper=n._paperdiv.append("svg").classed("main-svg",!0),n._modebardiv=n._paperdiv.append("div"),n._hoverpaper=n._paperdiv.append("svg").classed("main-svg",!0),!n._uid){var i={};r.selectAll("defs").each(function(){this.id&&(i[this.id.split("-")[1]]=1)}),n._uid=a.randstr(i)}n._paperdiv.selectAll(".main-svg").attr(C.svgAttrs),n._defs=n._paper.append("defs").attr("id","defs-"+n._uid),n._clips=n._defs.append("g").classed("clips",!0),n._topdefs=n._toppaper.append("defs").attr("id","topdefs-"+n._uid),n._topclips=n._topdefs.append("g").classed("clips",!0),n._bgLayer=n._paper.append("g").classed("bglayer",!0),n._draggers=n._paper.append("g").classed("draglayer",!0);var o=n._paper.append("g").classed("layer-below",!0);n._imageLowerLayer=o.append("g").classed("imagelayer",!0),n._shapeLowerLayer=o.append("g").classed("shapelayer",!0),n._cartesianlayer=n._paper.append("g").classed("cartesianlayer",!0),n._polarlayer=n._paper.append("g").classed("polarlayer",!0),n._ternarylayer=n._paper.append("g").classed("ternarylayer",!0),n._geolayer=n._paper.append("g").classed("geolayer",!0),n._funnelarealayer=n._paper.append("g").classed("funnelarealayer",!0),n._pielayer=n._paper.append("g").classed("pielayer",!0),n._sunburstlayer=n._paper.append("g").classed("sunburstlayer",!0),n._indicatorlayer=n._toppaper.append("g").classed("indicatorlayer",!0),n._glimages=n._paper.append("g").classed("glimages",!0);var s=n._toppaper.append("g").classed("layer-above",!0);n._imageUpperLayer=s.append("g").classed("imagelayer",!0),n._shapeUpperLayer=s.append("g").classed("shapelayer",!0),n._infolayer=n._toppaper.append("g").classed("infolayer",!0),n._menulayer=n._toppaper.append("g").classed("menulayer",!0),n._zoomlayer=n._toppaper.append("g").classed("zoomlayer",!0),n._hoverlayer=n._hoverpaper.append("g").classed("hoverlayer",!0),n._modebardiv.classed("modebar-container",!0).style("position","absolute").style("top","0px").style("right","0px"),t.emit("plotly_framework")}n.animate=function(t,e,n){if(t=a.getGraphDiv(t),!a.isPlotDiv(t))throw new Error("This element is not a Plotly plot: "+t+". It's likely that you've failed to create a plot before animating it. For more details, see https://plot.ly/javascript/animations/");var r=t._transitionData;r._frameQueue||(r._frameQueue=[]);var i=(n=g.supplyAnimationDefaults(n)).transition,o=n.frame;function s(t){return Array.isArray(i)?t>=i.length?i[0]:i[t]:i}function A(t){return Array.isArray(o)?t>=o.length?o[0]:o[t]:o}function l(t,e){var n=0;return function(){if(t&&++n===e)return t()}}return void 0===r._frameWaitingCnt&&(r._frameWaitingCnt=0),new Promise(function(o,c){function u(){t.emit("plotly_animating"),r._lastFrameAt=-1/0,r._timeToNext=0,r._runningTransitions=0,r._currentFrame=null;var e=function(){r._animationRaf=window.requestAnimationFrame(e),Date.now()-r._lastFrameAt>r._timeToNext&&function(){r._currentFrame&&r._currentFrame.onComplete&&r._currentFrame.onComplete();var e=r._currentFrame=r._frameQueue.shift();if(e){var n=e.name?e.name.toString():null;t._fullLayout._currentFrame=n,r._lastFrameAt=Date.now(),r._timeToNext=e.frameOpts.duration,g.transition(t,e.frame.data,e.frame.layout,B.coerceTraceIndices(t,e.frame.traces),e.frameOpts,e.transitionOpts).then(function(){e.onComplete&&e.onComplete()}),t.emit("plotly_animatingframe",{name:n,frame:e.frame,animation:{frame:e.frameOpts,transition:e.transitionOpts}})}else t.emit("plotly_animated"),window.cancelAnimationFrame(r._animationRaf),r._animationRaf=null}()};e()}var h,d,f=0;function p(t){return Array.isArray(i)?f>=i.length?t.transitionOpts=i[f]:t.transitionOpts=i[0]:t.transitionOpts=i,f++,t}var I=[],C=null==e,m=Array.isArray(e);if(C||m||!a.isPlainObject(e)){if(C||-1!==["string","number"].indexOf(typeof e))for(h=0;h<r._frames.length;h++)(d=r._frames[h])&&(C||String(d.group)===String(e))&&I.push({type:"byname",name:String(d.name),data:p({name:d.name})});else if(m)for(h=0;h<e.length;h++){var y=e[h];-1!==["number","string"].indexOf(typeof y)?(y=String(y),I.push({type:"byname",name:y,data:p({name:y})})):a.isPlainObject(y)&&I.push({type:"object",data:p(a.extendFlat({},y))})}}else I.push({type:"object",data:p(a.extendFlat({},e))});for(h=0;h<I.length;h++)if("byname"===(d=I[h]).type&&!r._frameHash[d.data.name])return a.warn('animate failure: frame not found: "'+d.data.name+'"'),void c();-1!==["next","immediate"].indexOf(n.mode)&&function(){if(0!==r._frameQueue.length){for(;r._frameQueue.length;){var e=r._frameQueue.pop();e.onInterrupt&&e.onInterrupt()}t.emit("plotly_animationinterrupted",[])}}(),"reverse"===n.direction&&I.reverse();var E=t._fullLayout._currentFrame;if(E&&n.fromcurrent){var v=-1;for(h=0;h<I.length;h++)if("byname"===(d=I[h]).type&&d.name===E){v=h;break}if(v>0&&v<I.length-1){var b=[];for(h=0;h<I.length;h++)d=I[h],("byname"!==I[h].type||h>v)&&b.push(d);I=b}}I.length>0?function(e){if(0!==e.length){for(var i=0;i<e.length;i++){var a;a="byname"===e[i].type?g.computeFrame(t,e[i].name):e[i].data;var h=A(i),d=s(i);d.duration=Math.min(d.duration,h.duration);var f={frame:a,name:e[i].name,frameOpts:h,transitionOpts:d};i===e.length-1&&(f.onComplete=l(o,2),f.onInterrupt=c),r._frameQueue.push(f)}"immediate"===n.mode&&(r._lastFrameAt=-1/0),r._animationRaf||u()}}(I):(t.emit("plotly_animated"),o())})},n.addFrames=function(t,e,n){if(t=a.getGraphDiv(t),null==e)return Promise.resolve();if(!a.isPlotDiv(t))throw new Error("This element is not a Plotly plot: "+t+". It's likely that you've failed to create a plot before adding frames. For more details, see https://plot.ly/javascript/animations/");var r,i,o,s,A=t._transitionData._frames,c=t._transitionData._frameHash;if(!Array.isArray(e))throw new Error("addFrames failure: frameList must be an Array of frame definitions"+e);var u=A.length+2*e.length,h=[],d={};for(r=e.length-1;r>=0;r--)if(a.isPlainObject(e[r])){var f=e[r].name,p=(c[f]||d[f]||{}).name,I=e[r].name,C=c[p]||d[p];p&&I&&"number"==typeof I&&C&&x<5&&(x++,a.warn('addFrames: overwriting frame "'+(c[p]||d[p]).name+'" with a frame whose name of type "number" also equates to "'+p+'". This is valid but may potentially lead to unexpected behavior since all plotly.js frame names are stored internally as strings.'),5===x&&a.warn("addFrames: This API call has yielded too many of these warnings. For the rest of this call, further warnings about numeric frame names will be suppressed.")),d[f]={name:f},h.push({frame:g.supplyFrameDefaults(e[r]),index:n&&void 0!==n[r]&&null!==n[r]?n[r]:u+r})}h.sort(function(t,e){return t.index>e.index?-1:t.index<e.index?1:0});var m=[],y=[],E=A.length;for(r=h.length-1;r>=0;r--){if("number"==typeof(i=h[r].frame).name&&a.warn("Warning: addFrames accepts frames with numeric names, but the numbers areimplicitly cast to strings"),!i.name)for(;c[i.name="frame "+t._transitionData._counter++];);if(c[i.name]){for(o=0;o<A.length&&(A[o]||{}).name!==i.name;o++);m.push({type:"replace",index:o,value:i}),y.unshift({type:"replace",index:o,value:A[o]})}else s=Math.max(0,Math.min(h[r].index,E)),m.push({type:"insert",index:s,value:i}),y.unshift({type:"delete",index:s}),E++}var v=g.modifyFrames,B=g.modifyFrames,b=[t,y],Q=[t,m];return l&&l.add(t,v,b,B,Q),g.modifyFrames(t,m)},n.deleteFrames=function(t,e){if(t=a.getGraphDiv(t),!a.isPlotDiv(t))throw new Error("This element is not a Plotly plot: "+t);var n,r,i=t._transitionData._frames,o=[],s=[];if(!e)for(e=[],n=0;n<i.length;n++)e.push(n);for((e=e.slice()).sort(),n=e.length-1;n>=0;n--)r=e[n],o.push({type:"delete",index:r}),s.unshift({type:"insert",index:r,value:i[r]});var A=g.modifyFrames,c=g.modifyFrames,u=[t,s],h=[t,o];return l&&l.add(t,A,u,c,h),g.modifyFrames(t,o)},n.addTraces=function t(e,r,i){e=a.getGraphDiv(e);var o,s,A=[],c=n.deleteTraces,u=t,g=[e,A],h=[e,r];for(function(t,e,n){var r,i;if(!Array.isArray(t.data))throw new Error("gd.data must be an array.");if(void 0===e)throw new Error("traces must be defined.");for(Array.isArray(e)||(e=[e]),r=0;r<e.length;r++)if("object"!=typeof(i=e[r])||Array.isArray(i)||null===i)throw new Error("all values in traces array must be non-array objects");if(void 0===n||Array.isArray(n)||(n=[n]),void 0!==n&&n.length!==e.length)throw new Error("if indices is specified, traces.length must equal indices.length")}(e,r,i),Array.isArray(r)||(r=[r]),r=r.map(function(t){return a.extendFlat({},t)}),B.cleanData(r),o=0;o<r.length;o++)e.data.push(r[o]);for(o=0;o<r.length;o++)A.push(-r.length+o);if(void 0===i)return s=n.redraw(e),l.add(e,c,g,u,h),s;Array.isArray(i)||(i=[i]);try{R(e,A,i)}catch(t){throw e.data.splice(e.data.length-r.length,r.length),t}return l.startSequence(e),l.add(e,c,g,u,h),s=n.moveTraces(e,A,i),l.stopSequence(e),s},n.deleteTraces=function t(e,r){e=a.getGraphDiv(e);var i,o,s=[],A=n.addTraces,c=t,u=[e,s,r],g=[e,r];if(void 0===r)throw new Error("indices must be an integer or array of integers.");for(Array.isArray(r)||(r=[r]),L(e,r,"indices"),(r=S(r,e.data.length-1)).sort(a.sorterDes),i=0;i<r.length;i+=1)o=e.data.splice(r[i],1)[0],s.push(o);var h=n.redraw(e);return l.add(e,A,u,c,g),h},n.extendTraces=function t(e,r,i,o){var s=N(e=a.getGraphDiv(e),r,i,o,function(t,e,n){var r,i;if(a.isTypedArray(t))if(n<0){var o=new t.constructor(0),s=F(t,e);n<0?(r=s,i=o):(r=o,i=s)}else if(r=new t.constructor(n),i=new t.constructor(t.length+e.length-n),n===e.length)r.set(e),i.set(t);else if(n<e.length){var A=e.length-n;r.set(e.subarray(A)),i.set(t),i.set(e.subarray(0,A),t.length)}else{var l=n-e.length,c=t.length-l;r.set(t.subarray(c)),r.set(e,l),i.set(t.subarray(0,c))}else r=t.concat(e),i=n>=0&&n<r.length?r.splice(0,r.length-n):[];return[r,i]}),A=n.redraw(e),c=[e,s.update,i,s.maxPoints];return l.add(e,n.prependTraces,c,t,arguments),A},n.moveTraces=function t(e,r,i){var o,s=[],A=[],c=t,u=t,g=[e=a.getGraphDiv(e),i,r],h=[e,r,i];if(R(e,r,i),r=Array.isArray(r)?r:[r],void 0===i)for(i=[],o=0;o<r.length;o++)i.push(-r.length+o);for(i=Array.isArray(i)?i:[i],r=S(r,e.data.length-1),i=S(i,e.data.length-1),o=0;o<e.data.length;o++)-1===r.indexOf(o)&&s.push(e.data[o]);for(o=0;o<r.length;o++)A.push({newIndex:i[o],trace:e.data[r[o]]});for(A.sort(function(t,e){return t.newIndex-e.newIndex}),o=0;o<A.length;o+=1)s.splice(A[o].newIndex,0,A[o].trace);e.data=s;var d=n.redraw(e);return l.add(e,c,g,u,h),d},n.prependTraces=function t(e,r,i,o){var s=N(e=a.getGraphDiv(e),r,i,o,function(t,e,n){var r,i;if(a.isTypedArray(t))if(n<=0){var o=new t.constructor(0),s=F(e,t);n<0?(r=s,i=o):(r=o,i=s)}else if(r=new t.constructor(n),i=new t.constructor(t.length+e.length-n),n===e.length)r.set(e),i.set(t);else if(n<e.length){var A=e.length-n;r.set(e.subarray(0,A)),i.set(e.subarray(A)),i.set(t,A)}else{var l=n-e.length;r.set(e),r.set(t.subarray(0,l),e.length),i.set(t.subarray(l))}else r=e.concat(t),i=n>=0&&n<r.length?r.splice(n,r.length):[];return[r,i]}),A=n.redraw(e),c=[e,s.update,i,s.maxPoints];return l.add(e,n.extendTraces,c,t,arguments),A},n.newPlot=function(t,e,r,i){return t=a.getGraphDiv(t),g.cleanPlot([],{},t._fullData||[],t._fullLayout||{}),g.purge(t),n.plot(t,e,r,i)},n.plot=function(t,e,i,o){var s;if(t=a.getGraphDiv(t),A.init(t),a.isPlainObject(e)){var l=e;e=l.data,i=l.layout,o=l.config,s=l.frames}if(!1===A.triggerHandler(t,"plotly_beforeplot",[e,i,o]))return Promise.reject();e||i||a.isPlotDiv(t)||a.warn("Calling Plotly.plot as if redrawing but this container doesn't yet have a plot.",t),k(t,o),i||(i={}),r.select(t).classed("js-plotly-plot",!0),f.makeTester(),Array.isArray(t._promises)||(t._promises=[]);var u=0===(t.data||[]).length&&Array.isArray(e);Array.isArray(e)&&(B.cleanData(e),u?t.data=e:t.data.push.apply(t.data,e),t.empty=!1),t.layout&&!u||(t.layout=B.cleanLayout(i)),g.supplyDefaults(t);var p=t._fullLayout,C=p._has("cartesian");if(!p._has("polar")&&e&&e[0]&&e[0].r)return a.log("Legacy polar charts are deprecated!"),function(t,e,n){var i=r.select(t).selectAll(".plot-container").data([0]);i.enter().insert("div",":first-child").classed("plot-container plotly",!0);var o=i.selectAll(".svg-container").data([0]);o.enter().append("div").classed("svg-container",!0).style("position","relative"),o.html(""),e&&(t.data=e),n&&(t.layout=n),h.manager.fillLayout(t),o.style({width:t._fullLayout.width+"px",height:t._fullLayout.height+"px"}),t.framework=h.manager.framework(t),t.framework({data:t.data,layout:t.layout},o.node()),t.framework.setUndoPoint();var s=t.framework.svg(),A=1,l=t._fullLayout.title?t._fullLayout.title.text:"";""!==l&&l||(A=0);var c=function(){this.call(m.convertToTspans,t)},u=s.select(".title-group text").call(c);if(t._context.edits.titleText){var d=a._(t,"Click to enter Plot title");l&&l!==d||(A=.2,u.attr({"data-unformatted":d}).text(d).style({opacity:A}).on("mouseover.opacity",function(){r.select(this).transition().duration(100).style("opacity",1)}).on("mouseout.opacity",function(){r.select(this).transition().duration(1e3).style("opacity",0)}));var f=function(){this.call(m.makeEditable,{gd:t}).on("edit",function(e){t.framework({layout:{title:{text:e}}}),this.text(e).call(c),this.call(f)}).on("cancel",function(){var t=this.attr("data-unformatted");this.text(t).call(c)})};u.call(f)}return t._context.setBackground(t,t._fullLayout.paper_bgcolor),g.addLinks(t),Promise.resolve()}(t,e,i);p._replotting=!0,u&&st(t),t.framework!==st&&(t.framework=st,st(t)),f.initGradients(t),u&&d.saveShowSpikeInitial(t);var y=!t.calcdata||t.calcdata.length!==(t._fullData||[]).length;y&&g.doCalcdata(t);for(var E=0;E<t.calcdata.length;E++)t.calcdata[E][0].trace=t._fullData[E];t._context.responsive?t._responsiveChartHandler||(t._responsiveChartHandler=function(){a.isHidden(t)||g.resize(t)},window.addEventListener("resize",t._responsiveChartHandler)):a.clearResponsive(t);var v=a.extendFlat({},p._size),Q=0;function w(){return g.clearAutoMarginIds(t),b.drawMarginPushers(t),d.allowAutoMargin(t),g.doAutoMargin(t),g.previousPromises(t)}function x(){t._transitioning||(b.doAutoRangeAndConstraints(t),u&&d.saveRangeInitial(t),c.getComponentMethod("rangeslider","calcAutorange")(t))}var D=[g.previousPromises,function(){if(s)return n.addFrames(t,s)},function e(){for(var n=p._basePlotModules,r=0;r<n.length;r++)n[r].drawFramework&&n[r].drawFramework(t);if(!p._glcanvas&&p._has("gl")&&(p._glcanvas=p._glcontainer.selectAll(".gl-canvas").data([{key:"contextLayer",context:!0,pick:!1},{key:"focusLayer",context:!1,pick:!1},{key:"pickLayer",context:!1,pick:!0}],function(t){return t.key}),p._glcanvas.enter().append("canvas").attr("class",function(t){return"gl-canvas gl-canvas-"+t.key.replace("Layer","")}).style({position:"absolute",top:0,left:0,overflow:"visible","pointer-events":"none"})),p._glcanvas){p._glcanvas.attr("width",p.width).attr("height",p.height);var i=p._glcanvas.data()[0].regl;if(i&&(Math.floor(p.width)!==i._gl.drawingBufferWidth||Math.floor(p.height)!==i._gl.drawingBufferHeight)){var o="WebGL context buffer and canvas dimensions do not match due to browser/WebGL bug.";if(!Q)return a.log(o+" Clearing graph and plotting again."),g.cleanPlot([],{},t._fullData,p),g.supplyDefaults(t),p=t._fullLayout,g.doCalcdata(t),Q++,e();a.error(o)}}return"h"===p.modebar.orientation?p._modebardiv.style("height",null).style("width","100%"):p._modebardiv.style("width",null).style("height",p.height+"px"),g.previousPromises(t)},w,function(){if(g.didMarginChange(v,p._size))return a.syncOrAsync([w,b.layoutStyles],t)}];C&&D.push(function(){if(y)return a.syncOrAsync([c.getComponentMethod("shapes","calcAutorange"),c.getComponentMethod("annotations","calcAutorange"),x],t);x()}),D.push(b.layoutStyles),C&&D.push(function(){return d.draw(t,u?"":"redraw")}),D.push(b.drawData,b.finalDraw,I,g.addLinks,g.rehover,g.redrag,g.doAutoMargin,g.previousPromises);var M=a.syncOrAsync(D,t);return M&&M.then||(M=Promise.resolve()),M.then(function(){return _(t),t})},n.purge=function(t){var e=(t=a.getGraphDiv(t))._fullLayout||{},n=t._fullData||[];return g.cleanPlot([],{},n,e),g.purge(t),A.purge(t),e._container&&e._container.remove(),delete t._context,t},n.react=function(t,e,r,i){var o,A,l=(t=a.getGraphDiv(t))._fullData,h=t._fullLayout;if(a.isPlotDiv(t)&&l&&h){if(a.isPlainObject(e)){var d=e;e=d.data,r=d.layout,i=d.config,o=d.frames}var f=!1;if(i){var p=a.extendDeep({},t._context);t._context=void 0,k(t,i),f=function t(e,n){var r;for(r in e)if("_"!==r.charAt(0)){var i=e[r],o=n[r];if(i!==o)if(a.isPlainObject(i)&&a.isPlainObject(o)){if(t(i,o))return!0}else{if(!Array.isArray(i)||!Array.isArray(o))return!0;if(i.length!==o.length)return!0;for(var s=0;s<i.length;s++)if(i[s]!==o[s]){if(!a.isPlainObject(i[s])||!a.isPlainObject(o[s]))return!0;if(t(i[s],o[s]))return!0}}}}(p,t._context)}t.data=e||[],B.cleanData(t.data),t.layout=r||{},B.cleanLayout(t.layout),function(t,e,n,r){var i,o,A,l,c,u,g,h,d=r._preGUI,f=[],p={};for(i in d){if(c=et(i,$)){if(o=c.attr||c.head+".uirevision",(l=(A=s(r,o).get())&&nt(o,e))&&l===A&&(null===(u=d[i])&&(u=void 0),ot(h=(g=s(e,i)).get(),u))){void 0===h&&"autorange"===i.substr(i.length-9)&&f.push(i.substr(0,i.length-10)),g.set(U(s(r,i).get()));continue}}else a.warn("unrecognized GUI edit: "+i);delete d[i],"range["===i.substr(i.length-8,6)&&(p[i.substr(0,i.length-9)]=1)}for(var I=0;I<f.length;I++){var C=f[I];if(p[C]){var m=s(e,C).get();m&&delete m.autorange}}var y=r._tracePreGUI;for(var E in y){var v,B=y[E],b=null;for(i in B){if(!b){var Q=rt(E,n);if(Q<0){delete y[E];break}var w=it(E,t,(v=n[Q]._fullInput).index);if(w<0){delete y[E];break}b=t[w]}if(c=et(i,tt)){if(c.attr?l=(A=s(r,c.attr).get())&&nt(c.attr,e):(A=v.uirevision,void 0===(l=b.uirevision)&&(l=e.uirevision)),l&&l===A&&(null===(u=B[i])&&(u=void 0),ot(h=(g=s(b,i)).get(),u))){g.set(U(s(v,i).get()));continue}}else a.warn("unrecognized GUI edit: "+i+" in trace uid "+E);delete B[i]}}}(t.data,t.layout,l,h),g.supplyDefaults(t,{skipUpdateCalc:!0});var I=t._fullData,C=t._fullLayout,m=void 0===C.datarevision,y=C.transition,E=function(t,e,n,r,i){var o=Q.layoutFlags();return o.arrays={},o.rangesAltered={},o.nChanges=0,o.nChangesAnim=0,at(e,n,[],{getValObject:function(t){return u.getLayoutValObject(n,t)},flags:o,immutable:r,transition:i,gd:t}),(o.plot||o.calc)&&(o.layoutReplot=!0),i&&o.nChanges&&o.nChangesAnim&&(o.anim=o.nChanges===o.nChangesAnim?"all":"some"),o}(t,h,C,m,y),v=E.newDataRevision,w=function(t,e,n,r,i,o){var a=e.length===n.length;if(!i&&!a)return{fullReplot:!0,calc:!0};var s,A,l=Q.traceFlags();l.arrays={},l.nChanges=0,l.nChangesAnim=0;var c={getValObject:function(t){var e=u.getTraceValObject(A,t);return!A._module.animatable&&e.anim&&(e.anim=!1),e},flags:l,immutable:r,transition:i,newDataRevision:o,gd:t},h={};for(s=0;s<e.length;s++)if(n[s]){if(A=n[s]._fullInput,g.hasMakesDataTransform(A)&&(A=n[s]),h[A.uid])continue;h[A.uid]=1,at(e[s]._fullInput,A,[],c)}return(l.calc||l.plot)&&(l.fullReplot=!0),i&&l.nChanges&&l.nChangesAnim&&(l.anim=l.nChanges===l.nChangesAnim&&a?"all":"some"),l}(t,l,I,m,y,v);W(t)&&(E.layoutReplot=!0),w.calc||E.calc?t.calcdata=void 0:g.supplyDefaultsUpdateCalc(t.calcdata,I);var x=[];if(o&&(t._transitionData={},g.createTransitionData(t),x.push(function(){return n.addFrames(t,o)})),C.transition&&!f&&(w.anim||E.anim))g.doCalcdata(t),b.doAutoRangeAndConstraints(t),x.push(function(){return g.transitionFromReact(t,w,E,h)});else if(w.fullReplot||E.layoutReplot||f)t._fullLayout._skipDefaults=!0,x.push(n.plot);else{for(var D in E.arrays){var M=E.arrays[D];if(M.length){var S=c.getComponentMethod(D,"drawOne");if(S!==a.noop)for(var L=0;L<M.length;L++)S(t,M[L]);else{var R=c.getComponentMethod(D,"draw");if(R===a.noop)throw new Error("cannot draw components: "+D);R(t)}}}x.push(g.previousPromises),w.style&&x.push(b.doTraceStyle),(w.colorbars||E.colorbars)&&x.push(b.doColorBars),E.legend&&x.push(b.doLegend),E.layoutstyle&&x.push(b.layoutStyles),E.axrange&&K(x),E.ticks&&x.push(b.doTicksRelayout),E.modebar&&x.push(b.doModeBar),E.camera&&x.push(b.doCamera),x.push(_)}x.push(g.rehover,g.redrag),(A=a.syncOrAsync(x,t))&&A.then||(A=Promise.resolve(t))}else A=n.newPlot(t,e,r,i);return A.then(function(){return t.emit("plotly_react",{data:e,layout:r}),t})},n.redraw=function(t){if(t=a.getGraphDiv(t),!a.isPlotDiv(t))throw new Error("This element is not a Plotly plot: "+t);return B.cleanData(t.data),B.cleanLayout(t.layout),t.calcdata=void 0,n.plot(t).then(function(){return t.emit("plotly_redraw"),t})},n.relayout=Y,n.restyle=T,n.setPlotConfig=function(t){return a.extendFlat(E,t)},n.update=X,n._guiRelayout=Z(Y),n._guiRestyle=Z(T),n._guiUpdate=Z(X),n._storeDirectGUIEdit=function(t,e,n){for(var r in n)G(r,s(t,r).get(),n[r],e)}},{"../components/color":593,"../components/drawing":614,"../constants/xmlns_namespaces":696,"../lib":719,"../lib/events":709,"../lib/queue":734,"../lib/svg_text_utils":743,"../plots/cartesian/axes":767,"../plots/cartesian/constants":773,"../plots/cartesian/graph_interact":776,"../plots/cartesian/select":784,"../plots/plots":828,"../plots/polar/legacy":836,"../registry":847,"./edit_types":750,"./helpers":751,"./manage_arrays":753,"./plot_config":755,"./plot_schema":756,"./subroutines":758,d3:163,"fast-isnumeric":225,"has-hover":410}],755:[function(t,e,n){"use strict";var r={staticPlot:{valType:"boolean",dflt:!1},plotlyServerURL:{valType:"string",dflt:"https://plot.ly"},editable:{valType:"boolean",dflt:!1},edits:{annotationPosition:{valType:"boolean",dflt:!1},annotationTail:{valType:"boolean",dflt:!1},annotationText:{valType:"boolean",dflt:!1},axisTitleText:{valType:"boolean",dflt:!1},colorbarPosition:{valType:"boolean",dflt:!1},colorbarTitleText:{valType:"boolean",dflt:!1},legendPosition:{valType:"boolean",dflt:!1},legendText:{valType:"boolean",dflt:!1},shapePosition:{valType:"boolean",dflt:!1},titleText:{valType:"boolean",dflt:!1}},autosizable:{valType:"boolean",dflt:!1},responsive:{valType:"boolean",dflt:!1},fillFrame:{valType:"boolean",dflt:!1},frameMargins:{valType:"number",dflt:0,min:0,max:.5},scrollZoom:{valType:"flaglist",flags:["cartesian","gl3d","geo","mapbox"],extras:[!0,!1],dflt:"gl3d+geo+mapbox"},doubleClick:{valType:"enumerated",values:[!1,"reset","autosize","reset+autosize"],dflt:"reset+autosize"},doubleClickDelay:{valType:"number",dflt:300,min:0},showAxisDragHandles:{valType:"boolean",dflt:!0},showAxisRangeEntryBoxes:{valType:"boolean",dflt:!0},showTips:{valType:"boolean",dflt:!0},showLink:{valType:"boolean",dflt:!1},linkText:{valType:"string",dflt:"Edit chart",noBlank:!0},sendData:{valType:"boolean",dflt:!0},showSources:{valType:"any",dflt:!1},displayModeBar:{valType:"enumerated",values:["hover",!0,!1],dflt:"hover"},showSendToCloud:{valType:"boolean",dflt:!1},showEditInChartStudio:{valType:"boolean",dflt:!1},modeBarButtonsToRemove:{valType:"any",dflt:[]},modeBarButtonsToAdd:{valType:"any",dflt:[]},modeBarButtons:{valType:"any",dflt:!1},toImageButtonOptions:{valType:"any",dflt:{}},displaylogo:{valType:"boolean",dflt:!0},watermark:{valType:"boolean",dflt:!1},plotGlPixelRatio:{valType:"number",dflt:2,min:1,max:4},setBackground:{valType:"any",dflt:"transparent"},topojsonURL:{valType:"string",noBlank:!0,dflt:"https://cdn.plot.ly/"},mapboxAccessToken:{valType:"string",dflt:null},logging:{valType:"boolean",dflt:1},queueLength:{valType:"integer",min:0,dflt:0},globalTransforms:{valType:"any",dflt:[]},locale:{valType:"string",dflt:"en-US"},locales:{valType:"any",dflt:{}}},i={};!function t(e,n){for(var r in e){var i=e[r];i.valType?n[r]=i.dflt:(n[r]||(n[r]={}),t(i,n[r]))}}(r,i),e.exports={configAttributes:r,dfltConfig:i}},{}],756:[function(t,e,n){"use strict";var r=t("../registry"),i=t("../lib"),o=t("../plots/attributes"),a=t("../plots/layout_attributes"),s=t("../plots/frame_attributes"),A=t("../plots/animation_attributes"),l=t("./plot_config").configAttributes,c=t("../plots/polar/legacy/area_attributes"),u=t("../plots/polar/legacy/axis_attributes"),g=t("./edit_types"),h=i.extendFlat,d=i.extendDeepAll,f=i.isPlainObject,p=i.isArrayOrTypedArray,I=i.nestedProperty,C=i.valObjectMeta,m="_isSubplotObj",y="_isLinkedToArray",E=[m,y,"_arrayAttrRegexps","_deprecated"];function v(t,e,n){if(!t)return!1;if(t._isLinkedToArray)if(B(e[n]))n++;else if(n<e.length)return!1;for(;n<e.length;n++){var r=t[e[n]];if(!f(r))break;if(t=r,n===e.length-1)break;if(t._isLinkedToArray){if(!B(e[++n]))return!1}else if("info_array"===t.valType){var i=e[++n];if(!B(i))return!1;var o=t.items;if(Array.isArray(o)){if(i>=o.length)return!1;if(2===t.dimensions){if(n++,e.length===n)return t;var a=e[n];if(!B(a))return!1;t=o[i][a]}else t=o[i]}else t=o}}return t}function B(t){return t===Math.round(t)&&t>=0}function b(t){return function(t){n.crawl(t,function(t,e,r){n.isValObject(t)?"data_array"===t.valType?(t.role="data",r[e+"src"]={valType:"string",editType:"none"}):!0===t.arrayOk&&(r[e+"src"]={valType:"string",editType:"none"}):f(t)&&(t.role="object")})}(t),function(t){n.crawl(t,function(t,e,n){if(t){var r=t[y];r&&(delete t[y],n[e]={items:{}},n[e].items[r]=t,n[e].role="object")}})}(t),function(t){!function t(e){for(var n in e)if(f(e[n]))t(e[n]);else if(Array.isArray(e[n]))for(var r=0;r<e[n].length;r++)t(e[n][r]);else e[n]instanceof RegExp&&(e[n]=e[n].toString())}(t)}(t),t}function Q(t,e,n){var r=I(t,n),i=d({},e.layoutAttributes);i[m]=!0,r.set(i)}function w(t,e,n){var r=I(t,n);r.set(d(r.get()||{},e))}n.IS_SUBPLOT_OBJ=m,n.IS_LINKED_TO_ARRAY=y,n.DEPRECATED="_deprecated",n.UNDERSCORE_ATTRS=E,n.get=function(){var t={};r.allTypes.concat("area").forEach(function(e){t[e]=function(t){var e,i;"area"===t?(e={attributes:c},i={}):(e=r.modules[t]._module,i=e.basePlotModule);var a={type:null},s=d({},o),A=d({},e.attributes);n.crawl(A,function(t,e,n,r,i){I(s,i).set(void 0),void 0===t&&I(A,i).set(void 0)}),d(a,s),r.traceIs(t,"noOpacity")&&delete a.opacity,r.traceIs(t,"showLegend")||(delete a.showlegend,delete a.legendgroup),r.traceIs(t,"noHover")&&(delete a.hoverinfo,delete a.hoverlabel),e.selectPoints||delete a.selectedpoints,d(a,A),i.attributes&&d(a,i.attributes),a.type=t;var l={meta:e.meta||{},categories:e.categories||{},animatable:Boolean(e.animatable),type:t,attributes:b(a)};if(e.layoutAttributes){var u={};d(u,e.layoutAttributes),l.layoutAttributes=b(u)}return e.animatable||n.crawl(l,function(t){n.isValObject(t)&&"anim"in t&&delete t.anim}),l}(e)});var e,i={};return Object.keys(r.transformsRegistry).forEach(function(t){i[t]=function(t){var e=r.transformsRegistry[t],n=d({},e.attributes);return Object.keys(r.componentsRegistry).forEach(function(e){var i=r.componentsRegistry[e];i.schema&&i.schema.transforms&&i.schema.transforms[t]&&Object.keys(i.schema.transforms[t]).forEach(function(e){w(n,i.schema.transforms[t][e],e)})}),{attributes:b(n)}}(t)}),{defs:{valObjects:C,metaKeys:E.concat(["description","role","editType","impliedEdits"]),editType:{traces:g.traces,layout:g.layout},impliedEdits:{}},traces:t,layout:function(){var t,e,n={};for(t in d(n,a),r.subplotsRegistry)if((e=r.subplotsRegistry[t]).layoutAttributes)if(Array.isArray(e.attr))for(var i=0;i<e.attr.length;i++)Q(n,e,e.attr[i]);else{var o="subplot"===e.attr?e.name:e.attr;Q(n,e,o)}for(t in n=function(t){return h(t,{radialaxis:u.radialaxis,angularaxis:u.angularaxis}),h(t,u.layout),t}(n),r.componentsRegistry){var s=(e=r.componentsRegistry[t]).schema;if(s&&(s.subplots||s.layout)){var A=s.subplots;if(A&&A.xaxis&&!A.yaxis)for(var l in A.xaxis)delete n.yaxis[l]}else"colorscale"===e.name?d(n,e.layoutAttributes):e.layoutAttributes&&w(n,e.layoutAttributes,e.name)}return{layoutAttributes:b(n)}}(),transforms:i,frames:(e={frames:d({},s)},b(e),e.frames),animation:b(A),config:b(l)}},n.crawl=function(t,e,r,i){var o=r||0;i=i||"",Object.keys(t).forEach(function(r){var a=t[r];if(-1===E.indexOf(r)){var s=(i?i+".":"")+r;e(a,r,t,o,s),n.isValObject(a)||f(a)&&"impliedEdits"!==r&&n.crawl(a,e,o+1,s)}})},n.isValObject=function(t){return t&&void 0!==t.valType},n.findArrayAttributes=function(t){var e,r,i=[],a=[],s=[];function A(t,n,o,A){a=a.slice(0,A).concat([n]),s=s.slice(0,A).concat([t&&t._isLinkedToArray]),t&&("data_array"===t.valType||!0===t.arrayOk)&&("colorbar"!==a[A-1]||"ticktext"!==n&&"tickvals"!==n)&&function t(e,n,o){var A=e[a[n]],l=o+a[n];if(n===a.length-1)p(A)&&i.push(r+l);else if(s[n]){if(Array.isArray(A))for(var c=0;c<A.length;c++)f(A[c])&&t(A[c],n+1,l+"["+c+"].")}else f(A)&&t(A,n+1,l+".")}(e,0,"")}e=t,r="",n.crawl(o,A),t._module&&t._module.attributes&&n.crawl(t._module.attributes,A);var l=t.transforms;if(l)for(var c=0;c<l.length;c++){var u=l[c],g=u._module;g&&(r="transforms["+c+"].",e=u,n.crawl(g.attributes,A))}return i},n.getTraceValObject=function(t,e){var n,i,a=e[0],s=1;if("transforms"===a){if(1===e.length)return o.transforms;var A=t.transforms;if(!Array.isArray(A)||!A.length)return!1;var l=e[1];if(!B(l)||l>=A.length)return!1;i=(n=(r.transformsRegistry[A[l].type]||{}).attributes)&&n[e[2]],s=3}else if("area"===t.type)i=c[a];else{var u=t._module;if(u||(u=(r.modules[t.type||o.type.dflt]||{})._module),!u)return!1;if(!(i=(n=u.attributes)&&n[a])){var g=u.basePlotModule;g&&g.attributes&&(i=g.attributes[a])}i||(i=o[a])}return v(i,e,s)},n.getLayoutValObject=function(t,e){return v(function(t,e){var n,i,o,s,A=t._basePlotModules;if(A){var l;for(n=0;n<A.length;n++){if((o=A[n]).attrRegex&&o.attrRegex.test(e)){if(o.layoutAttrOverrides)return o.layoutAttrOverrides;!l&&o.layoutAttributes&&(l=o.layoutAttributes)}var c=o.baseLayoutAttrOverrides;if(c&&e in c)return c[e]}if(l)return l}var g=t._modules;if(g)for(n=0;n<g.length;n++)if((s=g[n].layoutAttributes)&&e in s)return s[e];for(i in r.componentsRegistry){if("colorscale"===(o=r.componentsRegistry[i]).name&&0===e.indexOf("coloraxis"))return o.layoutAttributes[e];if(!o.schema&&e===o.name)return o.layoutAttributes}return e in a?a[e]:"radialaxis"===e||"angularaxis"===e?u[e]:u.layout[e]||!1}(t,e[0]),e,1)}},{"../lib":719,"../plots/animation_attributes":762,"../plots/attributes":764,"../plots/frame_attributes":794,"../plots/layout_attributes":819,"../plots/polar/legacy/area_attributes":834,"../plots/polar/legacy/axis_attributes":835,"../registry":847,"./edit_types":750,"./plot_config":755}],757:[function(t,e,n){"use strict";var r=t("../lib"),i=t("../plots/attributes"),o="templateitemname",a={name:{valType:"string",editType:"none"}};function s(t){return t&&"string"==typeof t}function A(t){var e=t.length-1;return"s"!==t.charAt(e)&&r.warn("bad argument to arrayDefaultKey: "+t),t.substr(0,t.length-1)+"defaults"}a[o]={valType:"string",editType:"calc"},n.templatedArray=function(t,e){return e._isLinkedToArray=t,e.name=a.name,e[o]=a[o],e},n.traceTemplater=function(t){var e,n,o={};for(e in t)n=t[e],Array.isArray(n)&&n.length&&(o[e]=0);return{newTrace:function(a){var s={type:e=r.coerce(a,{},i,"type"),_template:null};if(e in o){n=t[e];var A=o[e]%n.length;o[e]++,s._template=n[A]}return s}}},n.newContainer=function(t,e,n){var i=t._template,o=i&&(i[e]||n&&i[n]);return r.isPlainObject(o)||(o=null),t[e]={_template:o}},n.arrayTemplater=function(t,e,n){var r=t._template,i=r&&r[A(e)],a=r&&r[e];Array.isArray(a)&&a.length||(a=[]);var l={};return{newItem:function(t){var e={name:t.name,_input:t},r=e[o]=t[o];if(!s(r))return e._template=i,e;for(var A=0;A<a.length;A++){var c=a[A];if(c.name===r)return l[r]=1,e._template=c,e}return e[n]=t[n]||!1,e._template=!1,e},defaultItems:function(){for(var t=[],e=0;e<a.length;e++){var n=a[e],r=n.name;if(s(r)&&!l[r]){var i={_template:n,name:r,_input:{_templateitemname:r}};i[o]=n[o],t.push(i),l[r]=1}}return t}}},n.arrayDefaultKey=A,n.arrayEditor=function(t,e,n){var i=(r.nestedProperty(t,e).get()||[]).length,a=n._index,s=a>=i&&(n._input||{})._templateitemname;s&&(a=i);var A,l=e+"["+a+"]";function c(){A={},s&&(A[l]={},A[l][o]=s)}function u(t,e){s?r.nestedProperty(A[l],t).set(e):A[l+"."+t]=e}function g(){var t=A;return c(),t}return c(),{modifyBase:function(t,e){A[t]=e},modifyItem:u,getUpdateObj:g,applyUpdate:function(e,n){e&&u(e,n);var i=g();for(var o in i)r.nestedProperty(t,o).set(i[o])}}}},{"../lib":719,"../plots/attributes":764}],758:[function(t,e,n){"use strict";var r=t("d3"),i=t("../registry"),o=t("../plots/plots"),a=t("../lib"),s=t("../lib/clear_gl_canvases"),A=t("../components/color"),l=t("../components/drawing"),c=t("../components/titles"),u=t("../components/modebar"),g=t("../plots/cartesian/axes"),h=t("../constants/alignment"),d=t("../plots/cartesian/constraints"),f=d.enforce,p=d.clean,I=t("../plots/cartesian/autorange").doAutoRange,C="start";function m(t,e,n){for(var r=0;r<n.length;r++){var i=n[r][0],o=n[r][1];if(!(i[0]>=t[1]||i[1]<=t[0])&&o[0]<e[1]&&o[1]>e[0])return!0}return!1}function y(t){var e,i,o,s,c,d,f=t._fullLayout,p=f._size,I=p.p,C=g.list(t,"",!0);if(f._paperdiv.style({width:t._context.responsive&&f.autosize&&!t._context._hasZeroWidth&&!t.layout.width?"100%":f.width+"px",height:t._context.responsive&&f.autosize&&!t._context._hasZeroHeight&&!t.layout.height?"100%":f.height+"px"}).selectAll(".main-svg").call(l.setSize,f.width,f.height),t._context.setBackground(t,f.paper_bgcolor),n.drawMainTitle(t),u.manage(t),!f._has("cartesian"))return t._promises.length&&Promise.all(t._promises);function y(t,e,n){var r=t._lw/2;return"x"===t._id.charAt(0)?e?"top"===n?e._offset-I-r:e._offset+e._length+I+r:p.t+p.h*(1-(t.position||0))+r%1:e?"right"===n?e._offset+e._length+I+r:e._offset-I-r:p.l+p.w*(t.position||0)+r%1}for(e=0;e<C.length;e++){var v=(s=C[e])._anchorAxis;s._linepositions={},s._lw=l.crispRound(t,s.linewidth,1),s._mainLinePosition=y(s,v,s.side),s._mainMirrorPosition=s.mirror&&v?y(s,v,h.OPPOSITE_SIDE[s.side]):null}var b=[],Q=[],w=[],x=1===A.opacity(f.paper_bgcolor)&&1===A.opacity(f.plot_bgcolor)&&f.paper_bgcolor===f.plot_bgcolor;for(i in f._plots)if((o=f._plots[i]).mainplot)o.bg&&o.bg.remove(),o.bg=void 0;else{var _=o.xaxis.domain,D=o.yaxis.domain,M=o.plotgroup;if(m(_,D,w)){var k=M.node(),S=o.bg=a.ensureSingle(M,"rect","bg");k.insertBefore(S.node(),k.childNodes[0]),Q.push(i)}else M.select("rect.bg").remove(),w.push([_,D]),x||(b.push(i),Q.push(i))}var L,R,N,F,T,U,O,G,H,j,Y,P,K,z=f._bgLayer.selectAll(".bg").data(b);for(z.enter().append("rect").classed("bg",!0),z.exit().remove(),z.each(function(t){f._plots[t].bg=r.select(this)}),e=0;e<Q.length;e++)o=f._plots[Q[e]],c=o.xaxis,d=o.yaxis,o.bg&&o.bg.call(l.setRect,c._offset-I,d._offset-I,c._length+2*I,d._length+2*I).call(A.fill,f.plot_bgcolor).style("stroke-width",0);if(!f._hasOnlyLargeSploms)for(i in f._plots){o=f._plots[i],c=o.xaxis,d=o.yaxis;var J,q,V=o.clipId="clip"+f._uid+i+"plot",W=a.ensureSingleById(f._clips,"clipPath",V,function(t){t.classed("plotclip",!0).append("rect")});o.clipRect=W.select("rect").attr({width:c._length,height:d._length}),l.setTranslate(o.plot,c._offset,d._offset),o._hasClipOnAxisFalse?(J=null,q=V):(J=V,q=null),l.setClipUrl(o.plot,J,t),o.layerClipId=q}function X(t){return"M"+L+","+t+"H"+R}function Z(t){return"M"+c._offset+","+t+"h"+c._length}function $(t){return"M"+t+","+G+"V"+O}function tt(t){return"M"+t+","+d._offset+"v"+d._length}function et(t,e,n){if(!t.showline||i!==t._mainSubplot)return"";if(!t._anchorAxis)return n(t._mainLinePosition);var r=e(t._mainLinePosition);return t.mirror&&(r+=e(t._mainMirrorPosition)),r}for(i in f._plots){o=f._plots[i],c=o.xaxis,d=o.yaxis;var nt="M0,0";E(c,i)&&(T=B(c,"left",d,C),L=c._offset-(T?I+T:0),U=B(c,"right",d,C),R=c._offset+c._length+(U?I+U:0),N=y(c,d,"bottom"),F=y(c,d,"top"),!(K=!c._anchorAxis||i!==c._mainSubplot)||"allticks"!==c.mirror&&"all"!==c.mirror||(c._linepositions[i]=[N,F]),nt=et(c,X,Z),K&&c.showline&&("all"===c.mirror||"allticks"===c.mirror)&&(nt+=X(N)+X(F)),o.xlines.style("stroke-width",c._lw+"px").call(A.stroke,c.showline?c.linecolor:"rgba(0,0,0,0)")),o.xlines.attr("d",nt);var rt="M0,0";E(d,i)&&(Y=B(d,"bottom",c,C),O=d._offset+d._length+(Y?I:0),P=B(d,"top",c,C),G=d._offset-(P?I:0),H=y(d,c,"left"),j=y(d,c,"right"),!(K=!d._anchorAxis||i!==d._mainSubplot)||"allticks"!==d.mirror&&"all"!==d.mirror||(d._linepositions[i]=[H,j]),rt=et(d,$,tt),K&&d.showline&&("all"===d.mirror||"allticks"===d.mirror)&&(rt+=$(H)+$(j)),o.ylines.style("stroke-width",d._lw+"px").call(A.stroke,d.showline?d.linecolor:"rgba(0,0,0,0)")),o.ylines.attr("d",rt)}return g.makeClipPaths(t),t._promises.length&&Promise.all(t._promises)}function E(t,e){return(t.ticks||t.showline)&&(e===t._mainSubplot||"all"===t.mirror||"allticks"===t.mirror)}function v(t,e,n){if(!n.showline||!n._lw)return!1;if("all"===n.mirror||"allticks"===n.mirror)return!0;var r=n._anchorAxis;if(!r)return!1;var i=h.FROM_BL[e];return n.side===e?r.domain[i]===t.domain[i]:n.mirror&&r.domain[1-i]===t.domain[1-i]}function B(t,e,n,r){if(v(t,e,n))return n._lw;for(var i=0;i<r.length;i++){var o=r[i];if(o._mainAxis===n._mainAxis&&v(t,e,o))return o._lw}return 0}n.layoutStyles=function(t){return a.syncOrAsync([o.doAutoMargin,y],t)},n.drawMainTitle=function(t){var e=t._fullLayout,n=function(t){var e=t.title,n="middle";return a.isRightAnchor(e)?n="end":a.isLeftAnchor(e)&&(n=C),n}(e),r=function(t){var e=t.title,n="0em";return a.isTopAnchor(e)?n=h.CAP_SHIFT+"em":a.isMiddleAnchor(e)&&(n=h.MID_SHIFT+"em"),n}(e);c.draw(t,"gtitle",{propContainer:e,propName:"title.text",placeholder:e._dfltTitle.plot,attributes:{x:function(t,e){var n=t.title,r=t._size,i=0;switch(e===C?i=n.pad.l:"end"===e&&(i=-n.pad.r),n.xref){case"paper":return r.l+r.w*n.x+i;case"container":default:return t.width*n.x+i}}(e,n),y:function(t,e){var n=t.title,r=t._size,i=0;if("0em"!==e&&e?e===h.CAP_SHIFT+"em"&&(i=n.pad.t):i=-n.pad.b,"auto"===n.y)return r.t/2;switch(n.yref){case"paper":return r.t+r.h-r.h*n.y+i;case"container":default:return t.height-t.height*n.y+i}}(e,r),"text-anchor":n,dy:r}})},n.doTraceStyle=function(t){var e,r=t.calcdata,a=[];for(e=0;e<r.length;e++){var A=r[e],l=A[0]||{},c=l.trace||{},u=c._module||{},g=u.arraysToCalcdata;g&&g(A,c);var h=u.editStyle;h&&a.push({fn:h,cd0:l})}if(a.length){for(e=0;e<a.length;e++){var d=a[e];d.fn(t,d.cd0)}s(t),n.redrawReglTraces(t)}return o.style(t),i.getComponentMethod("legend","draw")(t),o.previousPromises(t)},n.doColorBars=function(t){return i.getComponentMethod("colorbar","draw")(t),o.previousPromises(t)},n.layoutReplot=function(t){var e=t.layout;return t.layout=void 0,i.call("plot",t,"",e)},n.doLegend=function(t){return i.getComponentMethod("legend","draw")(t),o.previousPromises(t)},n.doTicksRelayout=function(t){return g.draw(t,"redraw"),t._fullLayout._hasOnlyLargeSploms&&(i.subplotsRegistry.splom.updateGrid(t),s(t),n.redrawReglTraces(t)),n.drawMainTitle(t),o.previousPromises(t)},n.doModeBar=function(t){var e=t._fullLayout;u.manage(t);for(var n=0;n<e._basePlotModules.length;n++){var r=e._basePlotModules[n].updateFx;r&&r(t)}return o.previousPromises(t)},n.doCamera=function(t){for(var e=t._fullLayout,n=e._subplots.gl3d,r=0;r<n.length;r++){var i=e[n[r]],o=i._scene,a=i.camera;o.setCamera(a)}},n.drawData=function(t){var e=t._fullLayout;s(t);for(var r=e._basePlotModules,a=0;a<r.length;a++)r[a].plot(t);return n.redrawReglTraces(t),o.style(t),i.getComponentMethod("shapes","draw")(t),i.getComponentMethod("annotations","draw")(t),e._replotting=!1,o.previousPromises(t)},n.redrawReglTraces=function(t){var e=t._fullLayout;if(e._has("regl")){var n,r,i=t._fullData,o=[],s=[];for(e._hasOnlyLargeSploms&&e._splomGrid.draw(),n=0;n<i.length;n++){var A=i[n];!0===A.visible&&0!==A._length&&("splom"===A.type?e._splomScenes[A.uid].draw():"scattergl"===A.type?a.pushUnique(o,A.xaxis+A.yaxis):"scatterpolargl"===A.type&&a.pushUnique(s,A.subplot))}for(n=0;n<o.length;n++)(r=e._plots[o[n]])._scene&&r._scene.draw();for(n=0;n<s.length;n++)(r=e[s[n]]._subplot)._scene&&r._scene.draw()}},n.doAutoRangeAndConstraints=function(t){for(var e,n,r=t._fullLayout,i=g.list(t,"",!0),o=r._axisMatchGroups||[],s=0;s<i.length;s++)e=i[s],p(t,e),I(t,e);f(t);t:for(var A=0;A<o.length;A++){var l,c=o[A],u=null;for(l in c){if(!1===(e=g.getFromId(t,l)).autorange)continue t;n=a.simpleMap(e.range,e.r2l),u?u[0]<u[1]?(u[0]=Math.min(u[0],n[0]),u[1]=Math.max(u[1],n[1])):(u[0]=Math.max(u[0],n[0]),u[1]=Math.min(u[1],n[1])):u=n}for(l in c)(e=g.getFromId(t,l)).range=a.simpleMap(u,e.l2r),e._input.range=e.range.slice(),e.setScale()}},n.finalDraw=function(t){i.getComponentMethod("shapes","draw")(t),i.getComponentMethod("images","draw")(t),i.getComponentMethod("annotations","draw")(t),i.getComponentMethod("rangeslider","draw")(t),i.getComponentMethod("rangeselector","draw")(t)},n.drawMarginPushers=function(t){i.getComponentMethod("legend","draw")(t),i.getComponentMethod("rangeselector","draw")(t),i.getComponentMethod("sliders","draw")(t),i.getComponentMethod("updatemenus","draw")(t),i.getComponentMethod("colorbar","draw")(t)}},{"../components/color":593,"../components/drawing":614,"../components/modebar":652,"../components/titles":681,"../constants/alignment":688,"../lib":719,"../lib/clear_gl_canvases":704,"../plots/cartesian/autorange":766,"../plots/cartesian/axes":767,"../plots/cartesian/constraints":774,"../plots/plots":828,"../registry":847,d3:163}],759:[function(t,e,n){"use strict";var r=t("../lib"),i=r.isPlainObject,o=t("./plot_schema"),a=t("../plots/plots"),s=t("../plots/attributes"),A=t("./plot_template"),l=t("./plot_config").dfltConfig;function c(t,e){t=r.extendDeep({},t);var n,o,a=Object.keys(t).sort();function s(e,n,r){if(i(n)&&i(e))c(e,n);else if(Array.isArray(n)&&Array.isArray(e)){var a=A.arrayTemplater({_template:t},r);for(o=0;o<n.length;o++){var s=n[o],l=a.newItem(s)._template;l&&c(l,s)}var u=a.defaultItems();for(o=0;o<u.length;o++)n.push(u[o]._template);for(o=0;o<n.length;o++)delete n[o].templateitemname}}for(n=0;n<a.length;n++){var l=a[n],g=t[l];if(l in e?s(g,e[l],l):e[l]=g,u(l)===l)for(var h in e){var d=u(h);h===d||d!==l||h in t||s(g,e[h],l)}}}function u(t){return t.replace(/[0-9]+$/,"")}function g(t,e,n,o,a){var s=a&&n(a);for(var l in t){var c=t[l],d=h(t,l,o),f=h(t,l,a),p=n(f);if(!p){var I=u(l);I!==l&&(p=n(f=h(t,I,a)))}if(!(s&&s===p||!p||p._noTemplating||"data_array"===p.valType||p.arrayOk&&Array.isArray(c)))if(!p.valType&&i(c))g(c,e,n,d,f);else if(p._isLinkedToArray&&Array.isArray(c))for(var C=!1,m=0,y={},E=0;E<c.length;E++){var v=c[E];if(i(v)){var B=v.name;if(B)y[B]||(g(v,e,n,h(c,m,d),h(c,m,f)),m++,y[B]=1);else if(!C){var b=h(t,A.arrayDefaultKey(l),o),Q=h(c,m,d);g(v,e,n,Q,h(c,m,f));var w=r.nestedProperty(e,Q);r.nestedProperty(e,b).set(w.get()),w.set(null),C=!0}}}else r.nestedProperty(e,d).set(c)}}function h(t,e,n){return n?Array.isArray(t)?n+"["+e+"]":n+"."+e:e}function d(t){for(var e=0;e<t.length;e++)if(i(t[e]))return!0}function f(t){var e;switch(t.code){case"data":e="The template has no key data.";break;case"layout":e="The template has no key layout.";break;case"missing":e=t.path?"There are no templates for item "+t.path+" with name "+t.templateitemname:"There are no templates for trace "+t.index+", of type "+t.traceType+".";break;case"unused":e=t.path?"The template item at "+t.path+" was not used in constructing the plot.":t.dataCount?"Some of the templates of type "+t.traceType+" were not used. The template has "+t.templateCount+" traces, the data only has "+t.dataCount+" of this type.":"The template has "+t.templateCount+" traces of type "+t.traceType+" but there are none in the data.";break;case"reused":e="Some of the templates of type "+t.traceType+" were used more than once. The template has "+t.templateCount+" traces, the data has "+t.dataCount+" of this type."}return t.msg=e,t}n.makeTemplate=function(t){t=r.isPlainObject(t)?t:r.getGraphDiv(t),t=r.extendDeep({_context:l},{data:t.data,layout:t.layout}),a.supplyDefaults(t);var e=t.data||[],n=t.layout||{};n._basePlotModules=t._fullLayout._basePlotModules,n._modules=t._fullLayout._modules;var A={data:{},layout:{}};e.forEach(function(t){var e={};g(t,e,function(t,e){return o.getTraceValObject(t,r.nestedProperty({},e).parts)}.bind(null,t));var n=r.coerce(t,{},s,"type"),i=A.data[n];i||(i=A.data[n]=[]),i.push(e)}),g(n,A.layout,function(t,e){return o.getLayoutValObject(t,r.nestedProperty({},e).parts)}.bind(null,n)),delete A.layout.template;var u=n.template;if(i(u)){var h,d,f,p,I,C,m=u.layout;i(m)&&c(m,A.layout);var y=u.data;if(i(y)){for(d in A.data)if(f=y[d],Array.isArray(f)){for(C=(I=A.data[d]).length,p=f.length,h=0;h<C;h++)c(f[h%p],I[h]);for(h=C;h<p;h++)I.push(r.extendDeep({},f[h]))}for(d in y)d in A.data||(A.data[d]=r.extendDeep([],y[d]))}}return A},n.validateTemplate=function(t,e){var n=r.extendDeep({},{_context:l,data:t.data,layout:t.layout}),o=n.layout||{};i(e)||(e=o.template||{});var s=e.layout,A=e.data,c=[];n.layout=o,n.layout.template=e,a.supplyDefaults(n);var g=n._fullLayout,p=n._fullData,I={};if(i(s)?(function t(e,n){for(var r in e)if("_"!==r.charAt(0)&&i(e[r])){var o,a=u(r),s=[];for(o=0;o<n.length;o++)s.push(h(e,r,n[o])),a!==r&&s.push(h(e,a,n[o]));for(o=0;o<s.length;o++)I[s[o]]=1;t(e[r],s)}}(g,["layout"]),function t(e,n){for(var r in e)if(-1===r.indexOf("defaults")&&i(e[r])){var o=h(e,r,n);I[o]?t(e[r],o):c.push({code:"unused",path:o})}}(s,"layout")):c.push({code:"layout"}),i(A)){for(var C,m={},y=0;y<p.length;y++){var E=p[y];m[C=E.type]=(m[C]||0)+1,E._fullInput._template||c.push({code:"missing",index:E._fullInput.index,traceType:C})}for(C in A){var v=A[C].length,B=m[C]||0;v>B?c.push({code:"unused",traceType:C,templateCount:v,dataCount:B}):B>v&&c.push({code:"reused",traceType:C,templateCount:v,dataCount:B})}}else c.push({code:"data"});if(function t(e,n){for(var r in e)if("_"!==r.charAt(0)){var o=e[r],a=h(e,r,n);i(o)?(Array.isArray(e)&&!1===o._template&&o.templateitemname&&c.push({code:"missing",path:a,templateitemname:o.templateitemname}),t(o,a)):Array.isArray(o)&&d(o)&&t(o,a)}}({data:p,layout:g},""),c.length)return c.map(f)}},{"../lib":719,"../plots/attributes":764,"../plots/plots":828,"./plot_config":755,"./plot_schema":756,"./plot_template":757}],760:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("./plot_api"),o=t("../lib"),a=t("../snapshot/helpers"),s=t("../snapshot/tosvg"),A=t("../snapshot/svgtoimg"),l={format:{valType:"enumerated",values:["png","jpeg","webp","svg"],dflt:"png"},width:{valType:"number",min:1},height:{valType:"number",min:1},scale:{valType:"number",min:0,dflt:1},setBackground:{valType:"any",dflt:!1},imageDataOnly:{valType:"boolean",dflt:!1}};e.exports=function(t,e){var n,c,u,g;function h(t){return!(t in e)||o.validate(e[t],l[t])}if(e=e||{},o.isPlainObject(t)?(n=t.data||[],c=t.layout||{},u=t.config||{},g={}):(t=o.getGraphDiv(t),n=o.extendDeep([],t.data),c=o.extendDeep({},t.layout),u=t._context,g=t._fullLayout||{}),!h("width")&&null!==e.width||!h("height")&&null!==e.height)throw new Error("Height and width should be pixel values.");if(!h("format"))throw new Error("Image format is not jpeg, png, svg or webp.");var d={};function f(t,n){return o.coerce(e,d,l,t,n)}var p=f("format"),I=f("width"),C=f("height"),m=f("scale"),y=f("setBackground"),E=f("imageDataOnly"),v=document.createElement("div");v.style.position="absolute",v.style.left="-5000px",document.body.appendChild(v);var B=o.extendFlat({},c);I?B.width=I:null===e.width&&r(g.width)&&(B.width=g.width),C?B.height=C:null===e.height&&r(g.height)&&(B.height=g.height);var b=o.extendFlat({},u,{_exportedPlot:!0,staticPlot:!0,setBackground:y}),Q=a.getRedrawFunc(v);function w(){return new Promise(function(t){setTimeout(t,a.getDelay(v._fullLayout))})}function x(){return new Promise(function(t,e){var n=s(v,p,m),r=v._fullLayout.width,l=v._fullLayout.height;if(i.purge(v),document.body.removeChild(v),"svg"===p)return t(E?n:a.encodeSVG(n));var c=document.createElement("canvas");c.id=o.randstr(),A({format:p,width:r,height:l,scale:m,canvas:c,svg:n,promise:!0}).then(t).catch(e)})}return new Promise(function(t,e){i.plot(v,n,B,b).then(Q).then(w).then(x).then(function(e){t(function(t){return E?t.replace(a.IMAGE_URL_PREFIX,""):t}(e))}).catch(function(t){e(t)})})}},{"../lib":719,"../snapshot/helpers":851,"../snapshot/svgtoimg":853,"../snapshot/tosvg":855,"./plot_api":754,"fast-isnumeric":225}],761:[function(t,e,n){"use strict";var r=t("../lib"),i=t("../plots/plots"),o=t("./plot_schema"),a=t("./plot_config").dfltConfig,s=r.isPlainObject,A=Array.isArray,l=r.isArrayOrTypedArray;function c(t,e,n,i,o,a){a=a||[];for(var u=Object.keys(t),g=0;g<u.length;g++){var p=u[g];if("transforms"!==p){var I=a.slice();I.push(p);var C=t[p],m=e[p],y=f(n,p),E="info_array"===(y||{}).valType,v="colorscale"===(y||{}).valType,B=(y||{}).items;if(d(n,p))if(s(C)&&s(m))c(C,m,y,i,o,I);else if(E&&A(C)){C.length>m.length&&i.push(h("unused",o,I.concat(m.length)));var b,Q,w,x,_,D=m.length,M=Array.isArray(B);if(M&&(D=Math.min(D,B.length)),2===y.dimensions)for(Q=0;Q<D;Q++)if(A(C[Q])){C[Q].length>m[Q].length&&i.push(h("unused",o,I.concat(Q,m[Q].length)));var k=m[Q].length;for(b=0;b<(M?Math.min(k,B[Q].length):k);b++)w=M?B[Q][b]:B,x=C[Q][b],_=m[Q][b],r.validate(x,w)?_!==x&&_!==+x&&i.push(h("dynamic",o,I.concat(Q,b),x,_)):i.push(h("value",o,I.concat(Q,b),x))}else i.push(h("array",o,I.concat(Q),C[Q]));else for(Q=0;Q<D;Q++)w=M?B[Q]:B,x=C[Q],_=m[Q],r.validate(x,w)?_!==x&&_!==+x&&i.push(h("dynamic",o,I.concat(Q),x,_)):i.push(h("value",o,I.concat(Q),x))}else if(y.items&&!E&&A(C)){var S,L,R=B[Object.keys(B)[0]],N=[];for(S=0;S<m.length;S++){var F=m[S]._index||S;if((L=I.slice()).push(F),s(C[F])&&s(m[S])){N.push(F);var T=C[F],U=m[S];s(T)&&!1!==T.visible&&!1===U.visible?i.push(h("invisible",o,L)):c(T,U,R,i,o,L)}}for(S=0;S<C.length;S++)(L=I.slice()).push(S),s(C[S])?-1===N.indexOf(S)&&i.push(h("unused",o,L)):i.push(h("object",o,L,C[S]))}else!s(C)&&s(m)?i.push(h("object",o,I,C)):l(C)||!l(m)||E||v?p in e?r.validate(C,y)?"enumerated"===y.valType&&(y.coerceNumber&&C!==+m||C!==m)&&i.push(h("dynamic",o,I,C,m)):i.push(h("value",o,I,C)):i.push(h("unused",o,I,C)):i.push(h("array",o,I,C));else i.push(h("schema",o,I))}}return i}e.exports=function(t,e){var n,l,u=o.get(),g=[],d={_context:r.extendFlat({},a)};A(t)?(d.data=r.extendDeep([],t),n=t):(d.data=[],n=[],g.push(h("array","data"))),s(e)?(d.layout=r.extendDeep({},e),l=e):(d.layout={},l={},arguments.length>1&&g.push(h("object","layout"))),i.supplyDefaults(d);for(var f=d._fullData,p=n.length,I=0;I<p;I++){var C=n[I],m=["data",I];if(s(C)){var y=f[I],E=y.type,v=u.traces[E].attributes;v.type={valType:"enumerated",values:[E]},!1===y.visible&&!1!==C.visible&&g.push(h("invisible",m)),c(C,y,v,g,m);var B=C.transforms,b=y.transforms;if(B){A(B)||g.push(h("array",m,["transforms"])),m.push("transforms");for(var Q=0;Q<B.length;Q++){var w=["transforms",Q],x=B[Q].type;if(s(B[Q])){var _=u.transforms[x]?u.transforms[x].attributes:{};_.type={valType:"enumerated",values:Object.keys(u.transforms)},c(B[Q],b[Q],_,g,m,w)}else g.push(h("object",m,w))}}}else g.push(h("object",m))}return c(l,d._fullLayout,function(t,e){for(var n=t.layout.layoutAttributes,i=0;i<e.length;i++){var o=e[i],a=t.traces[o.type],s=a.layoutAttributes;s&&(o.subplot?r.extendFlat(n[a.attributes.subplot.dflt],s):r.extendFlat(n,s))}return n}(u,f),g,"layout"),0===g.length?void 0:g};var u={object:function(t,e){return("layout"===t&&""===e?"The layout argument":"data"===t[0]&&""===e?"Trace "+t[1]+" in the data argument":g(t)+"key "+e)+" must be linked to an object container"},array:function(t,e){return("data"===t?"The data argument":g(t)+"key "+e)+" must be linked to an array container"},schema:function(t,e){return g(t)+"key "+e+" is not part of the schema"},unused:function(t,e,n){var r=s(n)?"container":"key";return g(t)+r+" "+e+" did not get coerced"},dynamic:function(t,e,n,r){return[g(t)+"key",e,"(set to '"+n+"')","got reset to","'"+r+"'","during defaults."].join(" ")},invisible:function(t,e){return(e?g(t)+"item "+e:"Trace "+t[1])+" got defaulted to be not visible"},value:function(t,e,n){return[g(t)+"key "+e,"is set to an invalid value ("+n+")"].join(" ")}};function g(t){return A(t)?"In data trace "+t[1]+", ":"In "+t+", "}function h(t,e,n,i,o){var a,s;n=n||"",A(e)?(a=e[0],s=e[1]):(a=e,s=null);var l=function(t){if(!A(t))return String(t);for(var e="",n=0;n<t.length;n++){var r=t[n];"number"==typeof r?e=e.substr(0,e.length-1)+"["+r+"]":e+=r,n<t.length-1&&(e+=".")}return e}(n),c=u[t](e,l,i,o);return r.log(c),{code:t,container:a,trace:s,path:n,astr:l,msg:c}}function d(t,e){var n=I(e),r=n.keyMinusId,i=n.id;return!!(r in t&&t[r]._isSubplotObj&&i)||e in t}function f(t,e){return e in t?t[e]:t[I(e).keyMinusId]}var p=r.counterRegex("([a-z]+)");function I(t){var e=t.match(p);return{keyMinusId:e&&e[1],id:e&&e[2]}}},{"../lib":719,"../plots/plots":828,"./plot_config":755,"./plot_schema":756}],762:[function(t,e,n){"use strict";e.exports={mode:{valType:"enumerated",dflt:"afterall",values:["immediate","next","afterall"]},direction:{valType:"enumerated",values:["forward","reverse"],dflt:"forward"},fromcurrent:{valType:"boolean",dflt:!1},frame:{duration:{valType:"number",min:0,dflt:500},redraw:{valType:"boolean",dflt:!0}},transition:{duration:{valType:"number",min:0,dflt:500,editType:"none"},easing:{valType:"enumerated",dflt:"cubic-in-out",values:["linear","quad","cubic","sin","exp","circle","elastic","back","bounce","linear-in","quad-in","cubic-in","sin-in","exp-in","circle-in","elastic-in","back-in","bounce-in","linear-out","quad-out","cubic-out","sin-out","exp-out","circle-out","elastic-out","back-out","bounce-out","linear-in-out","quad-in-out","cubic-in-out","sin-in-out","exp-in-out","circle-in-out","elastic-in-out","back-in-out","bounce-in-out"],editType:"none"},ordering:{valType:"enumerated",values:["layout first","traces first"],dflt:"layout first",editType:"none"}}}},{}],763:[function(t,e,n){"use strict";var r=t("../lib"),i=t("../plot_api/plot_template");e.exports=function(t,e,n){var o,a,s=n.name,A=n.inclusionAttr||"visible",l=e[s],c=r.isArrayOrTypedArray(t[s])?t[s]:[],u=e[s]=[],g=i.arrayTemplater(e,s,A);for(o=0;o<c.length;o++){var h=c[o];r.isPlainObject(h)?a=g.newItem(h):(a=g.newItem({}))[A]=!1,a._index=o,!1!==a[A]&&n.handleItemDefaults(h,a,e,n),u.push(a)}var d=g.defaultItems();for(o=0;o<d.length;o++)(a=d[o])._index=u.length,n.handleItemDefaults({},a,e,n,{}),u.push(a);if(r.isArrayOrTypedArray(l)){var f=Math.min(l.length,u.length);for(o=0;o<f;o++)r.relinkPrivateKeys(u[o],l[o])}return u}},{"../lib":719,"../plot_api/plot_template":757}],764:[function(t,e,n){"use strict";var r=t("../components/fx/attributes");e.exports={type:{valType:"enumerated",values:[],dflt:"scatter",editType:"calc+clearAxisTypes",_noTemplating:!0},visible:{valType:"enumerated",values:[!0,!1,"legendonly"],dflt:!0,editType:"calc"},showlegend:{valType:"boolean",dflt:!0,editType:"style"},legendgroup:{valType:"string",dflt:"",editType:"style"},opacity:{valType:"number",min:0,max:1,dflt:1,editType:"style"},name:{valType:"string",editType:"style"},uid:{valType:"string",editType:"plot",anim:!0},ids:{valType:"data_array",editType:"calc",anim:!0},customdata:{valType:"data_array",editType:"calc"},meta:{valType:"any",arrayOk:!0,editType:"plot"},selectedpoints:{valType:"any",editType:"calc"},hoverinfo:{valType:"flaglist",flags:["x","y","z","text","name"],extras:["all","none","skip"],arrayOk:!0,dflt:"all",editType:"none"},hoverlabel:r.hoverlabel,stream:{token:{valType:"string",noBlank:!0,strict:!0,editType:"calc"},maxpoints:{valType:"number",min:0,max:1e4,dflt:500,editType:"calc"},editType:"calc"},transforms:{_isLinkedToArray:"transform",editType:"calc"},uirevision:{valType:"any",editType:"none"}}},{"../components/fx/attributes":623}],765:[function(t,e,n){"use strict";e.exports={xaxis:{valType:"subplotid",dflt:"x",editType:"calc+clearAxisTypes"},yaxis:{valType:"subplotid",dflt:"y",editType:"calc+clearAxisTypes"}}},{}],766:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib"),o=t("../../constants/numerical").FP_SAFE,a=t("../../registry");function s(t,e){var n,r,o=[],a=A(e),s=l(t,e),c=s.min,u=s.max;if(0===c.length||0===u.length)return i.simpleMap(e.range,e.r2l);var g=c[0].val,h=u[0].val;for(n=1;n<c.length&&g===h;n++)g=Math.min(g,c[n].val);for(n=1;n<u.length&&g===h;n++)h=Math.max(h,u[n].val);var d=!1;if(e.range){var f=i.simpleMap(e.range,e.r2l);d=f[1]<f[0]}"reversed"===e.autorange&&(d=!0,e.autorange=!0);var p,I,C,m,y,E,v=e.rangemode,B="tozero"===v,b="nonnegative"===v,Q=e._length,w=Q/10,x=0;for(n=0;n<c.length;n++)for(p=c[n],r=0;r<u.length;r++)(E=(I=u[r]).val-p.val)>0&&((y=Q-a(p)-a(I))>w?E/y>x&&(C=p,m=I,x=E/y):E/Q>x&&(C={val:p.val,pad:0},m={val:I.val,pad:0},x=E/Q));if(g===h){var _=g-1,D=g+1;if(B)if(0===g)o=[0,1];else{var M=(g>0?u:c).reduce(function(t,e){return Math.max(t,a(e))},0),k=g/(1-Math.min(.5,M/Q));o=g>0?[0,k]:[k,0]}else o=b?[Math.max(0,_),Math.max(1,D)]:[_,D]}else B?(C.val>=0&&(C={val:0,pad:0}),m.val<=0&&(m={val:0,pad:0})):b&&(C.val-x*a(C)<0&&(C={val:0,pad:0}),m.val<=0&&(m={val:1,pad:0})),x=(m.val-C.val)/(Q-a(C)-a(m)),o=[C.val-x*a(C),m.val+x*a(m)];return d&&o.reverse(),i.simpleMap(o,e.l2r||Number)}function A(t){var e=t._length/20;return"domain"===t.constrain&&t._inputDomain&&(e*=(t._inputDomain[1]-t._inputDomain[0])/(t.domain[1]-t.domain[0])),function(t){return t.pad+(t.extrapad?e:0)}}function l(t,e){var n,r,i,o=e._id,a=t._fullData,s=t._fullLayout,A=[],l=[];function g(t,e){for(n=0;n<e.length;n++){var a=t[e[n]],s=(a._extremes||{})[o];if(!0===a.visible&&s){for(r=0;r<s.min.length;r++)i=s.min[r],c(A,i.val,i.pad,{extrapad:i.extrapad});for(r=0;r<s.max.length;r++)i=s.max[r],u(l,i.val,i.pad,{extrapad:i.extrapad})}}}return g(a,e._traceIndices),g(s.annotations||[],e._annIndices||[]),g(s.shapes||[],e._shapeIndices||[]),{min:A,max:l}}function c(t,e,n,r){g(t,e,n,r,d)}function u(t,e,n,r){g(t,e,n,r,f)}function g(t,e,n,r,i){for(var o=r.tozero,a=r.extrapad,s=!0,A=0;A<t.length&&s;A++){var l=t[A];if(i(l.val,e)&&l.pad>=n&&(l.extrapad||!a)){s=!1;break}i(e,l.val)&&l.pad<=n&&(a||!l.extrapad)&&(t.splice(A,1),A--)}if(s){var c=o&&0===e;t.push({val:e,pad:c?0:n,extrapad:!c&&a})}}function h(t){return r(t)&&Math.abs(t)<o}function d(t,e){return t<=e}function f(t,e){return t>=e}e.exports={getAutoRange:s,makePadFn:A,doAutoRange:function(t,e){if(e.setScale(),e.autorange){e.range=s(t,e),e._r=e.range.slice(),e._rl=i.simpleMap(e._r,e.r2l);var n=e._input,r={};r[e._attr+".range"]=e.range,r[e._attr+".autorange"]=e.autorange,a.call("_storeDirectGUIEdit",t.layout,t._fullLayout._preGUI,r),n.range=e.range.slice(),n.autorange=e.autorange}var o=e._anchorAxis;if(o&&o.rangeslider){var A=o.rangeslider[e._name];A&&"auto"===A.rangemode&&(A.range=s(t,e)),o._input.rangeslider[e._name]=i.extendFlat({},A)}},findExtremes:function(t,e,n){n||(n={}),t._m||t.setScale();var i,a,s,A,l,g,d,f,p,I=[],C=[],m=e.length,y=n.padded||!1,E=n.tozero&&("linear"===t.type||"-"===t.type),v="log"===t.type,B=!1;function b(t){if(Array.isArray(t))return B=!0,function(e){return Math.max(Number(t[e]||0),0)};var e=Math.max(Number(t||0),0);return function(){return e}}var Q=b((t._m>0?n.ppadplus:n.ppadminus)||n.ppad||0),w=b((t._m>0?n.ppadminus:n.ppadplus)||n.ppad||0),x=b(n.vpadplus||n.vpad),_=b(n.vpadminus||n.vpad);if(!B){if(f=1/0,p=-1/0,v)for(i=0;i<m;i++)(a=e[i])<f&&a>0&&(f=a),a>p&&a<o&&(p=a);else for(i=0;i<m;i++)(a=e[i])<f&&a>-o&&(f=a),a>p&&a<o&&(p=a);e=[f,p],m=2}var D={tozero:E,extrapad:y};function M(n){s=e[n],r(s)&&(g=Q(n),d=w(n),f=s-_(n),p=s+x(n),v&&f<p/10&&(f=p/10),A=t.c2l(f),l=t.c2l(p),E&&(A=Math.min(0,A),l=Math.max(0,l)),h(A)&&c(I,A,d,D),h(l)&&u(C,l,g,D))}var k=Math.min(6,m);for(i=0;i<k;i++)M(i);for(i=m-1;i>=k;i--)M(i);return{min:I,max:C,opts:n}},concatExtremes:l}},{"../../constants/numerical":695,"../../lib":719,"../../registry":847,"fast-isnumeric":225}],767:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=t("../../plots/plots"),a=t("../../registry"),s=t("../../lib"),A=t("../../lib/svg_text_utils"),l=t("../../components/titles"),c=t("../../components/color"),u=t("../../components/drawing"),g=t("./layout_attributes"),h=t("./clean_ticks"),d=t("../../constants/numerical"),f=d.ONEAVGYEAR,p=d.ONEAVGMONTH,I=d.ONEDAY,C=d.ONEHOUR,m=d.ONEMIN,y=d.ONESEC,E=d.MINUS_SIGN,v=d.BADNUM,B=t("../../constants/alignment").MID_SHIFT,b=t("../../constants/alignment").LINE_SPACING,Q=e.exports={};Q.setConvert=t("./set_convert");var w=t("./axis_autotype"),x=t("./axis_ids");Q.id2name=x.id2name,Q.name2id=x.name2id,Q.cleanId=x.cleanId,Q.list=x.list,Q.listIds=x.listIds,Q.getFromId=x.getFromId,Q.getFromTrace=x.getFromTrace;var _=t("./autorange");Q.getAutoRange=_.getAutoRange,Q.findExtremes=_.findExtremes,Q.coerceRef=function(t,e,n,r,i,o){var a=r.charAt(r.length-1),A=n._fullLayout._subplots[a+"axis"],l=r+"ref",c={};return i||(i=A[0]||o),o||(o=i),c[l]={valType:"enumerated",values:A.concat(o?[o]:[]),dflt:i},s.coerce(t,e,c,l)},Q.coercePosition=function(t,e,n,r,i,o){var a,A;if("paper"===r||"pixel"===r)a=s.ensureNumber,A=n(i,o);else{var l=Q.getFromId(e,r);A=n(i,o=l.fraction2r(o)),a=l.cleanPos}t[i]=a(A)},Q.cleanPosition=function(t,e,n){return("paper"===n||"pixel"===n?s.ensureNumber:Q.getFromId(e,n).cleanPos)(t)},Q.redrawComponents=function(t,e){e=e||Q.listIds(t);var n=t._fullLayout;function r(r,i,o,s){for(var A=a.getComponentMethod(r,i),l={},c=0;c<e.length;c++)for(var u=n[Q.id2name(e[c])][o],g=0;g<u.length;g++){var h=u[g];if(!l[h]&&(A(t,h),l[h]=1,s))return}}r("annotations","drawOne","_annIndices"),r("shapes","drawOne","_shapeIndices"),r("images","draw","_imgIndices",!0)};var D=Q.getDataConversions=function(t,e,n,r){var i,o="x"===n||"y"===n||"z"===n?n:r;if(Array.isArray(o)){if(i={type:w(r),_categories:[]},Q.setConvert(i),"category"===i.type)for(var a=0;a<r.length;a++)i.d2c(r[a])}else i=Q.getFromTrace(t,e,o);return i?{d2c:i.d2c,c2d:i.c2d}:"ids"===o?{d2c:k,c2d:k}:{d2c:M,c2d:M}};function M(t){return+t}function k(t){return String(t)}Q.getDataToCoordFunc=function(t,e,n,r){return D(t,e,n,r).d2c},Q.counterLetter=function(t){var e=t.charAt(0);return"x"===e?"y":"y"===e?"x":void 0},Q.minDtick=function(t,e,n,r){-1===["log","category","multicategory"].indexOf(t.type)&&r?void 0===t._minDtick?(t._minDtick=e,t._forceTick0=n):t._minDtick&&((t._minDtick/e+1e-6)%1<2e-6&&((n-t._forceTick0)/e%1+1.000001)%1<2e-6?(t._minDtick=e,t._forceTick0=n):((e/t._minDtick+1e-6)%1>2e-6||((n-t._forceTick0)/t._minDtick%1+1.000001)%1>2e-6)&&(t._minDtick=0)):t._minDtick=0},Q.saveRangeInitial=function(t,e){for(var n=Q.list(t,"",!0),r=!1,i=0;i<n.length;i++){var o=n[i],a=void 0===o._rangeInitial,s=a||!(o.range[0]===o._rangeInitial[0]&&o.range[1]===o._rangeInitial[1]);(a&&!1===o.autorange||e&&s)&&(o._rangeInitial=o.range.slice(),r=!0)}return r},Q.saveShowSpikeInitial=function(t,e){for(var n=Q.list(t,"",!0),r=!1,i="on",o=0;o<n.length;o++){var a=n[o],s=void 0===a._showSpikeInitial,A=s||!(a.showspikes===a._showspikes);(s||e&&A)&&(a._showSpikeInitial=a.showspikes,r=!0),"on"!==i||a.showspikes||(i="off")}return t._fullLayout._cartesianSpikesEnabled=i,r},Q.autoBin=function(t,e,n,r,o,a){var A,l=s.aggNums(Math.min,null,t),c=s.aggNums(Math.max,null,t);if("category"===e.type||"multicategory"===e.type)return{start:l-.5,end:c+.5,size:Math.max(1,Math.round(a)||1),_dataSpan:c-l};if(o||(o=e.calendar),A="log"===e.type?{type:"linear",range:[l,c]}:{type:e.type,range:s.simpleMap([l,c],e.c2r,0,o),calendar:o},Q.setConvert(A),a=a&&h.dtick(a,A.type))A.dtick=a,A.tick0=h.tick0(void 0,A.type,o);else{var u;if(n)u=(c-l)/n;else{var g=s.distinctVals(t),d=Math.pow(10,Math.floor(Math.log(g.minDiff)/Math.LN10)),f=d*s.roundUp(g.minDiff/d,[.9,1.9,4.9,9.9],!0);u=Math.max(f,2*s.stdev(t)/Math.pow(t.length,r?.25:.4)),i(u)||(u=1)}Q.autoTicks(A,u)}var p,C=A.dtick,m=Q.tickIncrement(Q.tickFirst(A),C,"reverse",o);if("number"==typeof C)p=(m=function(t,e,n,r,o){var a=0,s=0,A=0,l=0;function c(e){return(1+100*(e-t)/n.dtick)%100<2}for(var u=0;u<e.length;u++)e[u]%1==0?A++:i(e[u])||l++,c(e[u])&&a++,c(e[u]+n.dtick/2)&&s++;var g=e.length-l;if(A===g&&"date"!==n.type)n.dtick<1?t=r-.5*n.dtick:(t-=.5)+n.dtick<r&&(t+=n.dtick);else if(s<.1*g&&(a>.3*g||c(r)||c(o))){var h=n.dtick/2;t+=t+h<r?h:-h}return t}(m,t,A,l,c))+(1+Math.floor((c-m)/C))*C;else for("M"===A.dtick.charAt(0)&&(m=function(t,e,n,r,i){var o=s.findExactDates(e,i);if(o.exactDays>.8){var a=Number(n.substr(1));o.exactYears>.8&&a%12==0?t=Q.tickIncrement(t,"M6","reverse")+1.5*I:o.exactMonths>.8?t=Q.tickIncrement(t,"M1","reverse")+15.5*I:t-=I/2;var A=Q.tickIncrement(t,n);if(A<=r)return A}return t}(m,t,C,l,o)),p=m;p<=c;)p=Q.tickIncrement(p,C,!1,o);return{start:e.c2r(m,0,o),end:e.c2r(p,0,o),size:C,_dataSpan:c-l}},Q.prepTicks=function(t){var e=s.simpleMap(t.range,t.r2l);if("auto"===t.tickmode||!t.dtick){var n,r=t.nticks;r||("category"===t.type||"multicategory"===t.type?(n=t.tickfont?1.2*(t.tickfont.size||12):15,r=t._length/n):(n="y"===t._id.charAt(0)?40:80,r=s.constrain(t._length/n,4,9)+1),"radialaxis"===t._name&&(r*=2)),"array"===t.tickmode&&(r*=100),Q.autoTicks(t,Math.abs(e[1]-e[0])/r),t._minDtick>0&&t.dtick<2*t._minDtick&&(t.dtick=t._minDtick,t.tick0=t.l2r(t._forceTick0))}t.tick0||(t.tick0="date"===t.type?"2000-01-01":0),"date"===t.type&&t.dtick<.1&&(t.dtick=.1),G(t)},Q.calcTicks=function(t){Q.prepTicks(t);var e=s.simpleMap(t.range,t.r2l);if("array"===t.tickmode)return function(t){var e=t.tickvals,n=t.ticktext,r=new Array(e.length),i=s.simpleMap(t.range,t.r2l),o=1.0001*i[0]-1e-4*i[1],a=1.0001*i[1]-1e-4*i[0],A=Math.min(o,a),l=Math.max(o,a),c=0;Array.isArray(n)||(n=[]);var u="category"===t.type?t.d2l_noadd:t.d2l;"log"===t.type&&"L"!==String(t.dtick).charAt(0)&&(t.dtick="L"+Math.pow(10,Math.floor(Math.min(t.range[0],t.range[1]))-1));for(var g=0;g<e.length;g++){var h=u(e[g]);h>A&&h<l&&(void 0===n[g]?r[c]=Q.tickText(t,h):r[c]=H(t,h,String(n[g])),c++)}return c<e.length&&r.splice(c,e.length-c),r}(t);t._tmin=Q.tickFirst(t);var n=1.0001*e[0]-1e-4*e[1],r=1.0001*e[1]-1e-4*e[0],o=e[1]<e[0];if(t._tmin<n!==o)return[];var a=[];"category"!==t.type&&"multicategory"!==t.type||(r=o?Math.max(-.5,r):Math.min(t._categories.length-.5,r));for(var A="log"===t.type&&!(i(t.dtick)||"L"===t.dtick.charAt(0)),l=null,c=Math.max(1e3,t._length||0),u=t._tmin;(o?u>=r:u<=r)&&!(a.length>c||u===l);u=Q.tickIncrement(u,t.dtick,o,t.calendar)){l=u;var g=!1;A&&u!==(0|u)&&(g=!0),a.push({minor:g,value:u})}nt(t)&&360===Math.abs(e[1]-e[0])&&a.pop(),t._tmax=(a[a.length-1]||{}).value,t._prevDateHead="",t._inCalcTicks=!0;for(var h=new Array(a.length),d=0;d<a.length;d++)h[d]=Q.tickText(t,a[d].value,!1,a[d].minor);return t._inCalcTicks=!1,h};var S=[2,5,10],L=[1,2,3,6,12],R=[1,2,5,10,15,30],N=[1,2,3,7,14],F=[-.046,0,.301,.477,.602,.699,.778,.845,.903,.954,1],T=[-.301,0,.301,.699,1],U=[15,30,45,90,180];function O(t,e,n){return e*s.roundUp(t/e,n)}function G(t){var e=t.dtick;if(t._tickexponent=0,i(e)||"string"==typeof e||(e=1),"category"!==t.type&&"multicategory"!==t.type||(t._tickround=null),"date"===t.type){var n=t.r2l(t.tick0),r=t.l2r(n).replace(/(^-|i)/g,""),o=r.length;if("M"===String(e).charAt(0))o>10||"01-01"!==r.substr(5)?t._tickround="d":t._tickround=+e.substr(1)%12==0?"y":"m";else if(e>=I&&o<=10||e>=15*I)t._tickround="d";else if(e>=m&&o<=16||e>=C)t._tickround="M";else if(e>=y&&o<=19||e>=m)t._tickround="S";else{var a=t.l2r(n+e).replace(/^-/,"").length;t._tickround=Math.max(o,a)-20,t._tickround<0&&(t._tickround=4)}}else if(i(e)||"L"===e.charAt(0)){var s=t.range.map(t.r2d||Number);i(e)||(e=Number(e.substr(1))),t._tickround=2-Math.floor(Math.log(e)/Math.LN10+.01);var A=Math.max(Math.abs(s[0]),Math.abs(s[1])),l=Math.floor(Math.log(A)/Math.LN10+.01);Math.abs(l)>3&&(Y(t.exponentformat)&&!P(l)?t._tickexponent=3*Math.round((l-1)/3):t._tickexponent=l)}else t._tickround=null}function H(t,e,n){var r=t.tickfont||{};return{x:e,dx:0,dy:0,text:n||"",fontSize:r.size,font:r.family,fontColor:r.color}}Q.autoTicks=function(t,e){var n;function r(t){return Math.pow(t,Math.floor(Math.log(e)/Math.LN10))}if("date"===t.type){t.tick0=s.dateTick0(t.calendar);var o=2*e;o>f?(e/=f,n=r(10),t.dtick="M"+12*O(e,n,S)):o>p?(e/=p,t.dtick="M"+O(e,1,L)):o>I?(t.dtick=O(e,I,N),t.tick0=s.dateTick0(t.calendar,!0)):o>C?t.dtick=O(e,C,L):o>m?t.dtick=O(e,m,R):o>y?t.dtick=O(e,y,R):(n=r(10),t.dtick=O(e,n,S))}else if("log"===t.type){t.tick0=0;var a=s.simpleMap(t.range,t.r2l);if(e>.7)t.dtick=Math.ceil(e);else if(Math.abs(a[1]-a[0])<1){var A=1.5*Math.abs((a[1]-a[0])/e);e=Math.abs(Math.pow(10,a[1])-Math.pow(10,a[0]))/A,n=r(10),t.dtick="L"+O(e,n,S)}else t.dtick=e>.3?"D2":"D1"}else"category"===t.type||"multicategory"===t.type?(t.tick0=0,t.dtick=Math.ceil(Math.max(e,1))):nt(t)?(t.tick0=0,n=1,t.dtick=O(e,n,U)):(t.tick0=0,n=r(10),t.dtick=O(e,n,S));if(0===t.dtick&&(t.dtick=1),!i(t.dtick)&&"string"!=typeof t.dtick){var l=t.dtick;throw t.dtick=1,"ax.dtick error: "+String(l)}},Q.tickIncrement=function(t,e,n,o){var a=n?-1:1;if(i(e))return t+a*e;var A=e.charAt(0),l=a*Number(e.substr(1));if("M"===A)return s.incrementMonth(t,l,o);if("L"===A)return Math.log(Math.pow(10,t)+l)/Math.LN10;if("D"===A){var c="D2"===e?T:F,u=t+.01*a,g=s.roundUp(s.mod(u,1),c,n);return Math.floor(u)+Math.log(r.round(Math.pow(10,g),1))/Math.LN10}throw"unrecognized dtick "+String(e)},Q.tickFirst=function(t){var e=t.r2l||Number,n=s.simpleMap(t.range,e),o=n[1]<n[0],a=o?Math.floor:Math.ceil,A=1.0001*n[0]-1e-4*n[1],l=t.dtick,c=e(t.tick0);if(i(l)){var u=a((A-c)/l)*l+c;return"category"!==t.type&&"multicategory"!==t.type||(u=s.constrain(u,0,t._categories.length-1)),u}var g=l.charAt(0),h=Number(l.substr(1));if("M"===g){for(var d,f,p,I=0,C=c;I<10;){if(((d=Q.tickIncrement(C,l,o,t.calendar))-A)*(C-A)<=0)return o?Math.min(C,d):Math.max(C,d);f=(A-(C+d)/2)/(d-C),p=g+(Math.abs(Math.round(f))||1)*h,C=Q.tickIncrement(C,p,f<0?!o:o,t.calendar),I++}return s.error("tickFirst did not converge",t),C}if("L"===g)return Math.log(a((Math.pow(10,A)-c)/h)*h+c)/Math.LN10;if("D"===g){var m="D2"===l?T:F,y=s.roundUp(s.mod(A,1),m,o);return Math.floor(A)+Math.log(r.round(Math.pow(10,y),1))/Math.LN10}throw"unrecognized dtick "+String(l)},Q.tickText=function(t,e,n,r){var o,a=H(t,e),A="array"===t.tickmode,l=n||A,c=t.type,u="category"===c?t.d2l_noadd:t.d2l;if(A&&Array.isArray(t.ticktext)){var g=s.simpleMap(t.range,t.r2l),h=Math.abs(g[1]-g[0])/1e4;for(o=0;o<t.ticktext.length&&!(Math.abs(e-u(t.tickvals[o]))<h);o++);if(o<t.ticktext.length)return a.text=String(t.ticktext[o]),a}function d(r){if(void 0===r)return!0;if(n)return"none"===r;var i={first:t._tmin,last:t._tmax}[r];return"all"!==r&&e!==i}var f=n?"never":"none"!==t.exponentformat&&d(t.showexponent)?"hide":"";if("date"===c?function(t,e,n,r){var o=t._tickround,a=n&&t.hoverformat||Q.getTickFormat(t);r&&(o=i(o)?4:{y:"m",m:"d",d:"M",M:"S",S:4}[o]);var A,l=s.formatDate(e.x,a,o,t._dateFormat,t.calendar,t._extraFormat),c=l.indexOf("\n");-1!==c&&(A=l.substr(c+1),l=l.substr(0,c)),r&&("00:00:00"===l||"00:00"===l?(l=A,A=""):8===l.length&&(l=l.replace(/:00$/,""))),A&&(n?"d"===o?l+=", "+A:l=A+(l?", "+l:""):t._inCalcTicks&&A===t._prevDateHead||(l+="<br>"+A,t._prevDateHead=A)),e.text=l}(t,a,n,l):"log"===c?function(t,e,n,r,o){var a=t.dtick,A=e.x,l=t.tickformat,c="string"==typeof a&&a.charAt(0);if("never"===o&&(o=""),r&&"L"!==c&&(a="L3",c="L"),l||"L"===c)e.text=K(Math.pow(10,A),t,o,r);else if(i(a)||"D"===c&&s.mod(A+.01,1)<.1){var u=Math.round(A),g=Math.abs(u),h=t.exponentformat;"power"===h||Y(h)&&P(u)?(e.text=0===u?1:1===u?"10":"10<sup>"+(u>1?"":E)+g+"</sup>",e.fontSize*=1.25):("e"===h||"E"===h)&&g>2?e.text="1"+h+(u>0?"+":E)+g:(e.text=K(Math.pow(10,A),t,"","fakehover"),"D1"===a&&"y"===t._id.charAt(0)&&(e.dy-=e.fontSize/6))}else{if("D"!==c)throw"unrecognized dtick "+String(a);e.text=String(Math.round(Math.pow(10,s.mod(A,1)))),e.fontSize*=.75}if("D1"===t.dtick){var d=String(e.text).charAt(0);"0"!==d&&"1"!==d||("y"===t._id.charAt(0)?e.dx-=e.fontSize/4:(e.dy+=e.fontSize/2,e.dx+=(t.range[1]>t.range[0]?1:-1)*e.fontSize*(A<0?.5:.25)))}}(t,a,0,l,f):"category"===c?function(t,e){var n=t._categories[Math.round(e.x)];void 0===n&&(n=""),e.text=String(n)}(t,a):"multicategory"===c?function(t,e,n){var r=Math.round(e.x),i=t._categories[r]||[],o=void 0===i[1]?"":String(i[1]),a=void 0===i[0]?"":String(i[0]);n?e.text=a+" - "+o:(e.text=o,e.text2=a)}(t,a,n):nt(t)?function(t,e,n,r,i){if("radians"!==t.thetaunit||n)e.text=K(e.x,t,i,r);else{var o=e.x/180;if(0===o)e.text="0";else{var a=function(t){function e(t,e){return Math.abs(t-e)<=1e-6}var n=function(t){for(var n=1;!e(Math.round(t*n)/n,t);)n*=10;return n}(t),r=t*n,i=Math.abs(function t(n,r){return e(r,0)?n:t(r,n%r)}(r,n));return[Math.round(r/i),Math.round(n/i)]}(o);if(a[1]>=100)e.text=K(s.deg2rad(e.x),t,i,r);else{var A=e.x<0;1===a[1]?1===a[0]?e.text="π":e.text=a[0]+"π":e.text=["<sup>",a[0],"</sup>","⁄","<sub>",a[1],"</sub>","π"].join(""),A&&(e.text=E+e.text)}}}}(t,a,n,l,f):function(t,e,n,r,i){"never"===i?i="":"all"===t.showexponent&&Math.abs(e.x/t.dtick)<1e-6&&(i="hide"),e.text=K(e.x,t,i,r)}(t,a,0,l,f),r||(t.tickprefix&&!d(t.showtickprefix)&&(a.text=t.tickprefix+a.text),t.ticksuffix&&!d(t.showticksuffix)&&(a.text+=t.ticksuffix)),"boundaries"===t.tickson||t.showdividers){var p=function(e){var n=t.l2p(e);return n>=0&&n<=t._length?e:null};a.xbnd=[p(a.x-.5),p(a.x+t.dtick-.5)]}return a},Q.hoverLabelText=function(t,e,n){if(n!==v&&n!==e)return Q.hoverLabelText(t,e)+" - "+Q.hoverLabelText(t,n);var r="log"===t.type&&e<=0,i=Q.tickText(t,t.c2l(r?-e:e),"hover").text;return r?0===e?"0":E+i:i};var j=["f","p","n","μ","m","","k","M","G","T"];function Y(t){return"SI"===t||"B"===t}function P(t){return t>14||t<-15}function K(t,e,n,r){var o=t<0,a=e._tickround,A=n||e.exponentformat||"B",l=e._tickexponent,c=Q.getTickFormat(e),u=e.separatethousands;if(r){var g={exponentformat:A,dtick:"none"===e.showexponent?e.dtick:i(t)&&Math.abs(t)||1,range:"none"===e.showexponent?e.range.map(e.r2d):[0,t||1]};G(g),a=(Number(g._tickround)||0)+4,l=g._tickexponent,e.hoverformat&&(c=e.hoverformat)}if(c)return e._numFormat(c)(t).replace(/-/g,E);var h,d=Math.pow(10,-a)/2;if("none"===A&&(l=0),(t=Math.abs(t))<d)t="0",o=!1;else{if(t+=d,l&&(t*=Math.pow(10,-l),a+=l),0===a)t=String(Math.floor(t));else if(a<0){t=(t=String(Math.round(t))).substr(0,t.length+a);for(var f=a;f<0;f++)t+="0"}else{var p=(t=String(t)).indexOf(".")+1;p&&(t=t.substr(0,p+a).replace(/\.?0+$/,""))}t=s.numSeparate(t,e._separators,u)}return l&&"hide"!==A&&(Y(A)&&P(l)&&(A="power"),h=l<0?E+-l:"power"!==A?"+"+l:String(l),"e"===A||"E"===A?t+=A+h:"power"===A?t+="×10<sup>"+h+"</sup>":"B"===A&&9===l?t+="B":Y(A)&&(t+=j[l/3+5])),o?E+t:t}function z(t,e){var n=t._id.charAt(0),r=t._tickAngles[e]||0,i=s.deg2rad(r),o=Math.sin(i),a=Math.cos(i),A=0,l=0;return t._selections[e].each(function(){var t=W(this),e=u.bBox(t.node()),n=e.width,r=e.height;A=Math.max(A,a*n,o*r),l=Math.max(l,o*n,a*r)}),{x:l,y:A}[n]}function J(t){return[t.text,t.x,t.axInfo,t.font,t.fontSize,t.fontColor].join("_")}function q(t,e){var n,r=t._fullLayout._size,i=e._id.charAt(0),o=e.side;return"free"!==e.anchor?n=x.getFromId(t,e.anchor):"x"===i?n={_offset:r.t+(1-(e.position||0))*r.h,_length:0}:"y"===i&&(n={_offset:r.l+(e.position||0)*r.w,_length:0}),"top"===o||"left"===o?n._offset:"bottom"===o||"right"===o?n._offset+n._length:void 0}function V(t,e){var n=t.l2p(e);return n>1&&n<t._length-1}function W(t){var e=r.select(t),n=e.select(".text-math-group");return n.empty()?e.select("text"):n}function X(t){return t._id+".automargin"}function Z(t){return t._id+".rangeslider"}function $(t,e){for(var n=0;n<e.length;n++)-1===t.indexOf(e[n])&&t.push(e[n])}function tt(t,e,n){var r,i,o=[],a=[],A=t.layout;for(r=0;r<e.length;r++)o.push(Q.getFromId(t,e[r]));for(r=0;r<n.length;r++)a.push(Q.getFromId(t,n[r]));var l=Object.keys(g),c=["anchor","domain","overlaying","position","side","tickangle","editType"],u=["linear","log"];for(r=0;r<l.length;r++){var h=l[r],d=o[0][h],f=a[0][h],p=!0,I=!1,C=!1;if("_"!==h.charAt(0)&&"function"!=typeof d&&-1===c.indexOf(h)){for(i=1;i<o.length&&p;i++){var m=o[i][h];"type"===h&&-1!==u.indexOf(d)&&-1!==u.indexOf(m)&&d!==m?I=!0:m!==d&&(p=!1)}for(i=1;i<a.length&&p;i++){var y=a[i][h];"type"===h&&-1!==u.indexOf(f)&&-1!==u.indexOf(y)&&f!==y?C=!0:a[i][h]!==f&&(p=!1)}p&&(I&&(A[o[0]._name].type="linear"),C&&(A[a[0]._name].type="linear"),et(A,h,o,a,t._fullLayout._dfltTitle))}}for(r=0;r<t._fullLayout.annotations.length;r++){var E=t._fullLayout.annotations[r];-1!==e.indexOf(E.xref)&&-1!==n.indexOf(E.yref)&&s.swapAttrs(A.annotations[r],["?"])}}function et(t,e,n,r,i){var o,a=s.nestedProperty,A=a(t[n[0]._name],e).get(),l=a(t[r[0]._name],e).get();for("title"===e&&(A&&A.text===i.x&&(A.text=i.y),l&&l.text===i.y&&(l.text=i.x)),o=0;o<n.length;o++)a(t,n[o]._name+"."+e).set(l);for(o=0;o<r.length;o++)a(t,r[o]._name+"."+e).set(A)}function nt(t){return"angularaxis"===t._id}Q.getTickFormat=function(t){var e,n,r,i,o,a,s,A;function l(t){return"string"!=typeof t?t:Number(t.replace("M",""))*p}function c(t,e){var n=["L","D"];if(typeof t==typeof e){if("number"==typeof t)return t-e;var r=n.indexOf(t.charAt(0)),i=n.indexOf(e.charAt(0));return r===i?Number(t.replace(/(L|D)/g,""))-Number(e.replace(/(L|D)/g,"")):r-i}return"number"==typeof t?1:-1}function u(t,e){var n=null===e[0],r=null===e[1],i=c(t,e[0])>=0,o=c(t,e[1])<=0;return(n||i)&&(r||o)}if(t.tickformatstops&&t.tickformatstops.length>0)switch(t.type){case"date":case"linear":for(e=0;e<t.tickformatstops.length;e++)if((r=t.tickformatstops[e]).enabled&&(i=t.dtick,o=r.dtickrange,a=void 0,a=l||function(t){return t},s=o[0],A=o[1],(!s&&"number"!=typeof s||a(s)<=a(i))&&(!A&&"number"!=typeof A||a(A)>=a(i)))){n=r;break}break;case"log":for(e=0;e<t.tickformatstops.length;e++)if((r=t.tickformatstops[e]).enabled&&u(t.dtick,r.dtickrange)){n=r;break}}return n?n.value:t.tickformat},Q.getSubplots=function(t,e){var n=t._fullLayout._subplots,r=n.cartesian.concat(n.gl2d||[]),i=e?Q.findSubplotsWithAxis(r,e):r;return i.sort(function(t,e){var n=t.substr(1).split("y"),r=e.substr(1).split("y");return n[0]===r[0]?+n[1]-+r[1]:+n[0]-+r[0]}),i},Q.findSubplotsWithAxis=function(t,e){for(var n=new RegExp("x"===e._id.charAt(0)?"^"+e._id+"y":e._id+"$"),r=[],i=0;i<t.length;i++){var o=t[i];n.test(o)&&r.push(o)}return r},Q.makeClipPaths=function(t){var e=t._fullLayout;if(!e._hasOnlyLargeSploms){var n,i,o={_offset:0,_length:e.width,_id:""},a={_offset:0,_length:e.height,_id:""},s=Q.list(t,"x",!0),A=Q.list(t,"y",!0),l=[];for(n=0;n<s.length;n++)for(l.push({x:s[n],y:a}),i=0;i<A.length;i++)0===n&&l.push({x:o,y:A[i]}),l.push({x:s[n],y:A[i]});var c=e._clips.selectAll(".axesclip").data(l,function(t){return t.x._id+t.y._id});c.enter().append("clipPath").classed("axesclip",!0).attr("id",function(t){return"clip"+e._uid+t.x._id+t.y._id}).append("rect"),c.exit().remove(),c.each(function(t){r.select(this).select("rect").attr({x:t.x._offset||0,y:t.y._offset||0,width:t.x._length||1,height:t.y._length||1})})}},Q.draw=function(t,e,n){var r=t._fullLayout;"redraw"===e&&r._paper.selectAll("g.subplot").each(function(t){var e=t[0],n=r._plots[e],i=n.xaxis,o=n.yaxis;n.xaxislayer.selectAll("."+i._id+"tick").remove(),n.yaxislayer.selectAll("."+o._id+"tick").remove(),n.xaxislayer.selectAll("."+i._id+"tick2").remove(),n.yaxislayer.selectAll("."+o._id+"tick2").remove(),n.xaxislayer.selectAll("."+i._id+"divider").remove(),n.yaxislayer.selectAll("."+o._id+"divider").remove(),n.gridlayer&&n.gridlayer.selectAll("path").remove(),n.zerolinelayer&&n.zerolinelayer.selectAll("path").remove(),r._infolayer.select(".g-"+i._id+"title").remove(),r._infolayer.select(".g-"+o._id+"title").remove()});var i=e&&"redraw"!==e?e:Q.listIds(t);return s.syncOrAsync(i.map(function(e){return function(){if(e){var r=Q.getFromId(t,e),i=Q.drawOne(t,r,n);return r._r=r.range.slice(),r._rl=s.simpleMap(r._r,r.r2l),i}}}))},Q.drawOne=function(t,e,n){var r,i,A;n=n||{},e.setScale();var g=t._fullLayout,h=e._id,d=h.charAt(0),f=Q.counterLetter(h),p=e._mainSubplot,I=e._mainLinePosition,C=e._mainMirrorPosition,m=g._plots[p][d+"axislayer"],y=e._subplotsWith,E=e._vals=Q.calcTicks(e),v=[e.mirror,I,C].join("_");for(r=0;r<E.length;r++)E[r].axInfo=v;if(e.visible){e._selections={},e._tickAngles={};var B,w,x=Q.makeTransFn(e);if("boundaries"===e.tickson){var _=function(t,e){var n,r=[],i=function(t,e){var n=t.xbnd[e];null!==n&&r.push(s.extendFlat({},t,{x:n}))};if(e.length){for(n=0;n<e.length;n++)i(e[n],0);i(e[n-1],1)}return r}(0,E);w=Q.clipEnds(e,_),B="inside"===e.ticks?w:_}else w=Q.clipEnds(e,E),B="inside"===e.ticks?w:E;var D=e._gridVals=w,M=function(t,e){var n,r,i=[],o=function(t,e){var n=t.xbnd[e];null!==n&&i.push(s.extendFlat({},t,{x:n}))};if(t.showdividers&&e.length){for(n=0;n<e.length;n++){var a=e[n];a.text2!==r&&o(a,0),r=a.text2}o(e[n-1],1)}return i}(e,E);if(!g._hasOnlyLargeSploms){var k={};for(r=0;r<y.length;r++){i=y[r];var S=(A=g._plots[i])[f+"axis"],L=S._mainAxis._id;if(!k[L]){k[L]=1;var R="x"===d?"M0,"+S._offset+"v"+S._length:"M"+S._offset+",0h"+S._length;Q.drawGrid(t,e,{vals:D,counterAxis:S,layer:A.gridlayer.select("."+h),path:R,transFn:x}),Q.drawZeroLine(t,e,{counterAxis:S,layer:A.zerolinelayer,path:R,transFn:x})}}}var N=Q.getTickSigns(e),F=[];if(e.ticks){var T,U,O,G=Q.makeTickPath(e,I,N[2]);if(e._anchorAxis&&e.mirror&&!0!==e.mirror?(T=Q.makeTickPath(e,C,N[3]),U=G+T):(T="",U=G),e.showdividers&&"outside"===e.ticks&&"boundaries"===e.tickson){var j={};for(r=0;r<M.length;r++)j[M[r].x]=1;O=function(t){return j[t.x]?T:U}}else O=U;Q.drawTicks(t,e,{vals:B,layer:m,path:O,transFn:x}),F=Object.keys(e._linepositions||{})}for(r=0;r<F.length;r++){i=F[r],A=g._plots[i];var Y=e._linepositions[i]||[],P=Q.makeTickPath(e,Y[0],N[0])+Q.makeTickPath(e,Y[1],N[1]);Q.drawTicks(t,e,{vals:B,layer:A[d+"axislayer"],path:P,transFn:x})}var K=[];if(K.push(function(){return Q.drawLabels(t,e,{vals:E,layer:m,transFn:x,labelFns:Q.makeLabelFns(e,I)})}),"multicategory"===e.type){var V=0,W={x:2,y:10}[d],$=N[2]*("inside"===e.ticks?-1:1);K.push(function(){return V+=z(e,h+"tick")+W,V+=e._tickAngles[h+"tick"]?e.tickfont.size*b:0,Q.drawLabels(t,e,{vals:function(t,e){for(var n=[],r={},i=0;i<e.length;i++){var o=e[i];r[o.text2]?r[o.text2].push(o.x):r[o.text2]=[o.x]}for(var a in r)n.push(H(t,s.interp(r[a],.5),a));return n}(e,E),layer:m,cls:h+"tick2",repositionOnUpdate:!0,secondary:!0,transFn:x,labelFns:Q.makeLabelFns(e,I+V*$)})}),K.push(function(){return V+=z(e,h+"tick2"),e._labelLength=V,function(t,e,n){var r=e._id+"divider",i=n.vals,o=n.layer.selectAll("path."+r).data(i,J);o.exit().remove(),o.enter().insert("path",":first-child").classed(r,1).classed("crisp",1).call(c.stroke,e.dividercolor).style("stroke-width",u.crispRound(t,e.dividerwidth,1)+"px"),o.attr("transform",n.transFn).attr("d",n.path)}(t,e,{vals:M,layer:m,path:Q.makeTickPath(e,I,$,V),transFn:x})})}var tt=a.getComponentMethod("rangeslider","isVisible")(e);return K.push(function(){if(e.showticklabels){var n=t.getBoundingClientRect(),r=m.node().getBoundingClientRect();e._boundingBox={width:r.width,height:r.height,left:r.left-n.left,right:r.right-n.left,top:r.top-n.top,bottom:r.bottom-n.top}}else{var i,o=g._size;"x"===d?(i="free"===e.anchor?o.t+o.h*(1-e.position):o.t+o.h*(1-e._anchorAxis.domain[{bottom:0,top:1}[e.side]]),e._boundingBox={top:i,bottom:i,left:e._offset,right:e._offset+e._length,width:e._length,height:0}):(i="free"===e.anchor?o.l+o.w*e.position:o.l+o.w*e._anchorAxis.domain[{left:0,right:1}[e.side]],e._boundingBox={left:i,right:i,bottom:e._offset+e._length,top:e._offset,height:e._length,width:0})}if(y){for(var a=e._counterSpan=[1/0,-1/0],s=0;s<y.length;s++){var A=g._plots[y[s]]["x"===d?"yaxis":"xaxis"];et(a,[A._offset,A._offset+A._length])}"free"===e.anchor&&et(a,"x"===d?[e._boundingBox.bottom,e._boundingBox.top]:[e._boundingBox.right,e._boundingBox.left])}},function(){var n,r,i=e.side.charAt(0);if(tt&&(r=a.getComponentMethod("rangeslider","autoMarginOpts")(t,e)),o.autoMargin(t,Z(e),r),e.automargin&&(!tt||"b"!==i)){n={x:0,y:0,r:0,l:0,t:0,b:0};var s,A,l=e._boundingBox,c=q(t,e);switch(d+i){case"xb":s=0,A=l.top-c,n[i]=l.height;break;case"xt":s=1,A=c-l.bottom,n[i]=l.height;break;case"yl":s=0,A=c-l.right,n[i]=l.width;break;case"yr":s=1,A=l.left-c,n[i]=l.width}if(n[f]="free"===e.anchor?e.position:e._anchorAxis.domain[s],n[i]>0&&(n[i]+=A),e.title.text!==g._dfltTitle[d]&&(n[i]+=e.title.font.size),"x"===d&&l.width>0){var u=l.right-(e._offset+e._length);u>0&&(n.x=1,n.r=u);var h=e._offset-l.left;h>0&&(n.x=0,n.l=h)}else if("y"===d&&l.height>0){var p=l.bottom-(e._offset+e._length);p>0&&(n.y=0,n.b=p);var I=e._offset-l.top;I>0&&(n.y=1,n.t=I)}}o.autoMargin(t,X(e),n)}),n.skipTitle||tt&&e._boundingBox&&"bottom"===e.side||K.push(function(){return function(t,e){var n,r=t._fullLayout,i=e._id,o=i.charAt(0),a=e.title.font.size;n="multicategory"===e.type?e._labelLength:10+1.5*a+(e.linewidth?e.linewidth-1:0);var s,A,c,g,h=q(t,e);if("x"===o?(A=e._offset+e._length/2,c="top"===e.side?-n-a*(e.showticklabels?1:0):n+a*(e.showticklabels?1.5:.5),c+=h):(c=e._offset+e._length/2,A="right"===e.side?n+a*(e.showticklabels?1:.5):-n-a*(e.showticklabels?.5:0),A+=h,s={rotate:"-90",offset:0}),"multicategory"!==e.type){var d=e._selections[e._id+"tick"];if(g={selection:d,side:e.side},d&&d.node()&&d.node().parentNode){var f=u.getTranslate(d.node().parentNode);g.offsetLeft=f.x,g.offsetTop=f.y}}return l.draw(t,i+"title",{propContainer:e,propName:e._name+".title.text",placeholder:r._dfltTitle[o],avoid:g,transform:s,attributes:{x:A,y:c,"text-anchor":"middle"}})}(t,e)}),s.syncOrAsync(K)}function et(t,e){t[0]=Math.min(t[0],e[0]),t[1]=Math.max(t[1],e[1])}},Q.getTickSigns=function(t){var e=t._id.charAt(0),n={x:"top",y:"right"}[e],r=t.side===n?1:-1,i=[-1,1,r,-r];return"inside"!==t.ticks==("x"===e)&&(i=i.map(function(t){return-t})),i},Q.makeTransFn=function(t){var e=t._id.charAt(0),n=t._offset;return"x"===e?function(e){return"translate("+(n+t.l2p(e.x))+",0)"}:function(e){return"translate(0,"+(n+t.l2p(e.x))+")"}},Q.makeTickPath=function(t,e,n,r){r=void 0!==r?r:t.ticklen;var i=t._id.charAt(0),o=(t.linewidth||1)/2;return"x"===i?"M0,"+(e+o*n)+"v"+r*n:"M"+(e+o*n)+",0h"+r*n},Q.makeLabelFns=function(t,e,n){var r=t._id.charAt(0),o="boundaries"!==t.tickson&&"outside"===t.ticks,a=0,A=0;if(o&&(a+=t.ticklen),n&&"outside"===t.ticks){var l=s.deg2rad(n);a=t.ticklen*Math.cos(l)+1,A=t.ticklen*Math.sin(l)}t.showticklabels&&(o||t.showline)&&(a+=.2*t.tickfont.size);var c,u,g,h,d={labelStandoff:a+=(t.linewidth||1)/2,labelShift:A};return"x"===r?(h="bottom"===t.side?1:-1,c=A*h,u=e+a*h,g="bottom"===t.side?1:-.2,d.xFn=function(t){return t.dx+c},d.yFn=function(t){return t.dy+u+t.fontSize*g},d.anchorFn=function(t,e){return i(e)&&0!==e&&180!==e?e*h<0?"end":"start":"middle"},d.heightFn=function(e,n,r){return n<-60||n>60?-.5*r:"top"===t.side?-r:0}):"y"===r&&(h="right"===t.side?1:-1,c=a,u=-A*h,g=90===Math.abs(t.tickangle)?.5:0,d.xFn=function(t){return t.dx+e+(c+t.fontSize*g)*h},d.yFn=function(t){return t.dy+u+t.fontSize*B},d.anchorFn=function(e,n){return i(n)&&90===Math.abs(n)?"middle":"right"===t.side?"start":"end"},d.heightFn=function(e,n,r){return(n*="left"===t.side?1:-1)<-30?-r:n<30?-.5*r:0}),d},Q.drawTicks=function(t,e,n){n=n||{};var r=e._id+"tick",i=n.layer.selectAll("path."+r).data(e.ticks?n.vals:[],J);i.exit().remove(),i.enter().append("path").classed(r,1).classed("ticks",1).classed("crisp",!1!==n.crisp).call(c.stroke,e.tickcolor).style("stroke-width",u.crispRound(t,e.tickwidth,1)+"px").attr("d",n.path),i.attr("transform",n.transFn)},Q.drawGrid=function(t,e,n){n=n||{};var r=e._id+"grid",i=n.vals,o=n.counterAxis;if(!1===e.showgrid)i=[];else if(o&&Q.shouldShowZeroLine(t,e,o))for(var a="array"===e.tickmode,s=0;s<i.length;s++){var A=i[s].x;if(a?!A:Math.abs(A)<e.dtick/100){if(i=i.slice(0,s).concat(i.slice(s+1)),!a)break;s--}}var l=n.layer.selectAll("path."+r).data(i,J);l.exit().remove(),l.enter().append("path").classed(r,1).classed("crisp",!1!==n.crisp),e._gw=u.crispRound(t,e.gridwidth,1),l.attr("transform",n.transFn).attr("d",n.path).call(c.stroke,e.gridcolor||"#ddd").style("stroke-width",e._gw+"px"),"function"==typeof n.path&&l.attr("d",n.path)},Q.drawZeroLine=function(t,e,n){n=n||n;var r=e._id+"zl",i=Q.shouldShowZeroLine(t,e,n.counterAxis),o=n.layer.selectAll("path."+r).data(i?[{x:0,id:e._id}]:[]);o.exit().remove(),o.enter().append("path").classed(r,1).classed("zl",1).classed("crisp",!1!==n.crisp).each(function(){n.layer.selectAll("path").sort(function(t,e){return x.idSort(t.id,e.id)})}),o.attr("transform",n.transFn).attr("d",n.path).call(c.stroke,e.zerolinecolor||c.defaultLine).style("stroke-width",u.crispRound(t,e.zerolinewidth,e._gw||1)+"px")},Q.drawLabels=function(t,e,n){n=n||{};var o=e._id,a=o.charAt(0),l=n.cls||o+"tick",c=n.vals,g=n.labelFns,h=n.secondary?0:e.tickangle,d=(e._tickAngles||{})[l],f=n.layer.selectAll("g."+l).data(e.showticklabels?c:[],J),p=[];function I(t,e){t.each(function(t){var o=r.select(this),a=o.select(".text-math-group"),s=g.anchorFn(t,e),l=n.transFn.call(o.node(),t)+(i(e)&&0!=+e?" rotate("+e+","+g.xFn(t)+","+(g.yFn(t)-t.fontSize/2)+")":""),c=A.lineCount(o),h=b*t.fontSize,d=g.heightFn(t,i(e)?+e:0,(c-1)*h);if(d&&(l+=" translate(0, "+d+")"),a.empty())o.select("text").attr({transform:l,"text-anchor":s});else{var f=u.bBox(a.node()).width*{end:-.5,start:.5}[s];a.attr("transform",l+(f?"translate("+f+",0)":""))}})}f.enter().append("g").classed(l,1).append("text").attr("text-anchor","middle").each(function(e){var n=r.select(this),i=t._promises.length;n.call(A.positionText,g.xFn(e),g.yFn(e)).call(u.font,e.font,e.fontSize,e.fontColor).text(e.text).call(A.convertToTspans,t),t._promises[i]?p.push(t._promises.pop().then(function(){I(n,h)})):I(n,h)}),f.exit().remove(),n.repositionOnUpdate&&f.each(function(t){r.select(this).select("text").call(A.positionText,g.xFn(t),g.yFn(t))}),I(f,d||h),e._selections&&(e._selections[l]=f);var C=s.syncOrAsync([function(){return p.length&&Promise.all(p)},function(){I(f,h);var t=null;if(c.length&&"x"===a&&!i(h)&&("log"!==e.type||"D"!==String(e.dtick).charAt(0))){t=0;var r,o=0,A=[];if(f.each(function(t){o=Math.max(o,t.fontSize);var n=e.l2p(t.x),r=W(this),i=u.bBox(r.node());A.push({top:0,bottom:10,height:10,left:n-i.width/2,right:n+i.width/2+2,width:i.width+2})}),"boundaries"!==e.tickson&&!e.showdividers||n.secondary){var g=c.length,d=Math.abs((c[g-1].x-c[0].x)*e._m)/(g-1)<2.5*o||"multicategory"===e.type;for(r=0;r<A.length-1;r++)if(s.bBoxIntersect(A[r],A[r+1])){t=d?90:30;break}}else{var p=2;for(e.ticks&&(p+=e.tickwidth/2),r=0;r<A.length;r++){var C=c[r].xbnd,m=A[r];if(null!==C[0]&&m.left-e.l2p(C[0])<p||null!==C[1]&&e.l2p(C[1])-m.right<p){t=90;break}}}t&&I(f,t)}e._tickAngles&&(e._tickAngles[l]=null===t?i(h)?h:0:t)}]);return C&&C.then&&t._promises.push(C),C},Q.shouldShowZeroLine=function(t,e,n){var r=s.simpleMap(e.range,e.r2l);return r[0]*r[1]<=0&&e.zeroline&&("linear"===e.type||"-"===e.type)&&e._gridVals.length&&(V(e,0)||!function(t,e,n,r){var i=n._mainAxis;if(i){var o=t._fullLayout,a=e._id.charAt(0),s=Q.counterLetter(e._id),A=e._offset+(Math.abs(r[0])<Math.abs(r[1])==("x"===a)?0:e._length),l=o._plots[n._mainSubplot];if(!(l.mainplotinfo||l).overlays.length)return h(n);for(var c=Q.list(t,s),u=0;u<c.length;u++){var g=c[u];if(g._mainAxis===i&&h(g))return!0}}function h(t){if(!t.showline||!t.linewidth)return!1;var n=Math.max((t.linewidth+e.zerolinewidth)/2,1);function r(t){return"number"==typeof t&&Math.abs(t-A)<n}if(r(t._mainLinePosition)||r(t._mainMirrorPosition))return!0;var i=t._linepositions||{};for(var o in i)if(r(i[o][0])||r(i[o][1]))return!0}}(t,e,n,r)||function(t,e){for(var n=t._fullData,r=e._mainSubplot,i=e._id.charAt(0),o=0;o<n.length;o++){var s=n[o];if(!0===s.visible&&s.xaxis+s.yaxis===r){if(a.traceIs(s,"bar-like")&&s.orientation==={x:"h",y:"v"}[i])return!0;if(s.fill&&s.fill.charAt(s.fill.length-1)===i)return!0}}return!1}(t,e))},Q.clipEnds=function(t,e){return e.filter(function(e){return V(t,e.x)})},Q.allowAutoMargin=function(t){for(var e=Q.list(t,"",!0),n=0;n<e.length;n++){var r=e[n];r.automargin&&o.allowAutoMargin(t,X(r)),a.getComponentMethod("rangeslider","isVisible")(r)&&o.allowAutoMargin(t,Z(r))}},Q.swap=function(t,e){for(var n=function(t,e){var n,r,i=[];for(n=0;n<e.length;n++){var o=[],a=t._fullData[e[n]].xaxis,s=t._fullData[e[n]].yaxis;if(a&&s){for(r=0;r<i.length;r++)-1===i[r].x.indexOf(a)&&-1===i[r].y.indexOf(s)||o.push(r);if(o.length){var A,l=i[o[0]];if(o.length>1)for(r=1;r<o.length;r++)A=i[o[r]],$(l.x,A.x),$(l.y,A.y);$(l.x,[a]),$(l.y,[s])}else i.push({x:[a],y:[s]})}}return i}(t,e),r=0;r<n.length;r++)tt(t,n[r].x,n[r].y)}},{"../../components/color":593,"../../components/drawing":614,"../../components/titles":681,"../../constants/alignment":688,"../../constants/numerical":695,"../../lib":719,"../../lib/svg_text_utils":743,"../../plots/plots":828,"../../registry":847,"./autorange":766,"./axis_autotype":768,"./axis_ids":770,"./clean_ticks":772,"./layout_attributes":779,"./set_convert":785,d3:163,"fast-isnumeric":225}],768:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib"),o=t("../../constants/numerical").BADNUM;e.exports=function(t,e,n){return!(n=n||{}).noMultiCategory&&(a=t,i.isArrayOrTypedArray(a[0])&&i.isArrayOrTypedArray(a[1]))?"multicategory":function(t,e){for(var n=Math.max(1,(t.length-1)/1e3),o=0,a=0,s={},A=0;A<t.length;A+=n){var l=t[Math.round(A)],c=String(l);s[c]||(s[c]=1,i.isDateTime(l,e)&&(o+=1),r(l)&&(a+=1))}return o>2*a}(t,e)?"date":function(t){for(var e=Math.max(1,(t.length-1)/1e3),n=0,r=0,a={},s=0;s<t.length;s+=e){var A=t[Math.round(s)],l=String(A);a[l]||(a[l]=1,"boolean"==typeof A?r++:i.cleanNumber(A)!==o?n++:"string"==typeof A&&r++)}return r>2*n}(t)?"category":function(t){if(!t)return!1;for(var e=0;e<t.length;e++)if(r(t[e]))return!0;return!1}(t)?"linear":"-";var a}},{"../../constants/numerical":695,"../../lib":719,"fast-isnumeric":225}],769:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib"),o=t("./layout_attributes"),a=t("./tick_value_defaults"),s=t("./tick_mark_defaults"),A=t("./tick_label_defaults"),l=t("./category_order_defaults"),c=t("./line_grid_defaults"),u=t("./set_convert");e.exports=function(t,e,n,g,h){var d=g.letter,f=g.font||{},p=g.splomStash||{},I=n("visible",!g.visibleDflt),C=e.type;"date"===C&&r.getComponentMethod("calendars","handleDefaults")(t,e,"calendar",g.calendar),u(e,h);var m=!e.isValidRange(t.range);m&&g.reverseDflt&&(m="reversed"),!n("autorange",m)||"linear"!==C&&"-"!==C||n("rangemode"),n("range"),e.cleanRange(),l(t,e,n,g),"category"===C||g.noHover||n("hoverformat");var y=n("color"),E=y!==o.color.dflt?y:f.color,v=p.label||h._dfltTitle[d];if(A(t,e,n,C,g,{pass:1}),!I)return e;n("title.text",v),i.coerceFont(n,"title.font",{family:f.family,size:Math.round(1.2*f.size),color:E}),a(t,e,n,C),A(t,e,n,C,g,{pass:2}),s(t,e,n,g),c(t,e,n,{dfltColor:y,bgColor:g.bgColor,showGrid:g.showGrid,attributes:o}),(e.showline||e.ticks)&&n("mirror"),g.automargin&&n("automargin");var B,b="multicategory"===e.type;return g.noTickson||"category"!==e.type&&!b||!e.ticks&&!e.showgrid||(b&&(B="boundaries"),n("tickson",B)),b&&n("showdividers")&&(n("dividercolor"),n("dividerwidth")),e}},{"../../lib":719,"../../registry":847,"./category_order_defaults":771,"./layout_attributes":779,"./line_grid_defaults":781,"./set_convert":785,"./tick_label_defaults":786,"./tick_mark_defaults":787,"./tick_value_defaults":788}],770:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("./constants");n.id2name=function(t){if("string"==typeof t&&t.match(i.AX_ID_PATTERN)){var e=t.substr(1);return"1"===e&&(e=""),t.charAt(0)+"axis"+e}},n.name2id=function(t){if(t.match(i.AX_NAME_PATTERN)){var e=t.substr(5);return"1"===e&&(e=""),t.charAt(0)+e}},n.cleanId=function(t,e){if(t.match(i.AX_ID_PATTERN)&&(!e||t.charAt(0)===e)){var n=t.substr(1).replace(/^0+/,"");return"1"===n&&(n=""),t.charAt(0)+n}},n.list=function(t,e,r){var i=t._fullLayout;if(!i)return[];var o,a=n.listIds(t,e),s=new Array(a.length);for(o=0;o<a.length;o++){var A=a[o];s[o]=i[A.charAt(0)+"axis"+A.substr(1)]}if(!r){var l=i._subplots.gl3d||[];for(o=0;o<l.length;o++){var c=i[l[o]];e?s.push(c[e+"axis"]):s.push(c.xaxis,c.yaxis,c.zaxis)}}return s},n.listIds=function(t,e){var n=t._fullLayout;if(!n)return[];var r=n._subplots;return e?r[e+"axis"]:r.xaxis.concat(r.yaxis)},n.getFromId=function(t,e,r){var i=t._fullLayout;return"x"===r?e=e.replace(/y[0-9]*/,""):"y"===r&&(e=e.replace(/x[0-9]*/,"")),i[n.id2name(e)]},n.getFromTrace=function(t,e,i){var o=t._fullLayout,a=null;if(r.traceIs(e,"gl3d")){var s=e.scene;"scene"===s.substr(0,5)&&(a=o[s][i+"axis"])}else a=n.getFromId(t,e[i+"axis"]||i);return a},n.idSort=function(t,e){var n=t.charAt(0),r=e.charAt(0);return n!==r?n>r?1:-1:+(t.substr(1)||1)-+(e.substr(1)||1)},n.getAxisGroup=function(t,e){for(var n=t._axisMatchGroups,r=0;r<n.length;r++)if(n[r][e])return"g"+r;return e}},{"../../registry":847,"./constants":773}],771:[function(t,e,n){"use strict";e.exports=function(t,e,n,r){if("category"===e.type){var i,o=t.categoryarray,a=Array.isArray(o)&&o.length>0;a&&(i="array");var s,A=n("categoryorder",i);"array"===A&&(s=n("categoryarray")),a||"array"!==A||(A=e.categoryorder="trace"),"trace"===A?e._initialCategories=[]:"array"===A?e._initialCategories=s.slice():(s=function(t,e){var n,r,i,o=e.dataAttr||t._id.charAt(0),a={};if(e.axData)n=e.axData;else for(n=[],r=0;r<e.data.length;r++){var s=e.data[r];s[o+"axis"]===t._id&&n.push(s)}for(r=0;r<n.length;r++){var A=n[r][o];for(i=0;i<A.length;i++){var l=A[i];null!=l&&(a[l]=1)}}return Object.keys(a)}(e,r).sort(),"category ascending"===A?e._initialCategories=s:"category descending"===A&&(e._initialCategories=s.reverse()))}}},{}],772:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib"),o=t("../../constants/numerical").ONEDAY;n.dtick=function(t,e){var n="log"===e,i="date"===e,a="category"===e,s=i?o:1;if(!t)return s;if(r(t))return(t=Number(t))<=0?s:a?Math.max(1,Math.round(t)):i?Math.max(.1,t):t;if("string"!=typeof t||!i&&!n)return s;var A=t.charAt(0),l=t.substr(1);return(l=r(l)?Number(l):0)<=0||!(i&&"M"===A&&l===Math.round(l)||n&&"L"===A||n&&"D"===A&&(1===l||2===l))?s:t},n.tick0=function(t,e,n,o){return"date"===e?i.cleanDate(t,i.dateTick0(n)):"D1"!==o&&"D2"!==o?r(t)?Number(t):0:void 0}},{"../../constants/numerical":695,"../../lib":719,"fast-isnumeric":225}],773:[function(t,e,n){"use strict";var r=t("../../lib/regex").counter;e.exports={idRegex:{x:r("x"),y:r("y")},attrRegex:r("[xy]axis"),xAxisMatch:r("xaxis"),yAxisMatch:r("yaxis"),AX_ID_PATTERN:/^[xyz][0-9]*$/,AX_NAME_PATTERN:/^[xyz]axis[0-9]*$/,SUBPLOT_PATTERN:/^x([0-9]*)y([0-9]*)$/,MINDRAG:8,MINSELECT:12,MINZOOM:20,DRAGGERSIZE:20,BENDPX:1.5,REDRAWDELAY:50,SELECTDELAY:100,SELECTID:"-select",DFLTRANGEX:[-1,6],DFLTRANGEY:[-1,4],traceLayerClasses:["heatmaplayer","contourcarpetlayer","contourlayer","funnellayer","waterfalllayer","barlayer","carpetlayer","violinlayer","boxlayer","ohlclayer","scattercarpetlayer","scatterlayer"],clipOnAxisFalseQuery:[".scatterlayer",".barlayer",".funnellayer",".waterfalllayer"],layerValue2layerClass:{"above traces":"above","below traces":"below"}}},{"../../lib/regex":735}],774:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./axis_ids").id2name,o=t("./scale_zoom"),a=t("./autorange").makePadFn,s=t("./autorange").concatExtremes,A=t("../../constants/numerical").ALMOST_EQUAL,l=t("../../constants/alignment").FROM_BL;function c(t,e,n,r,o){var a,s,A,l,c="range"!==o,u=r[i(e)].type,g=[];for(s=0;s<n.length;s++)(A=n[s])!==e&&(l=r[i(A)]).type===u&&(l.fixedrange?c&&l.anchor&&r[i(l.anchor)].fixedrange&&g.push(A):g.push(A));for(a=0;a<t.length;a++)if(t[a][e]){var h=t[a],d=[];for(s=0;s<g.length;s++)h[A=g[s]]||d.push(A);return{linkableAxes:d,thisGroup:h}}return{linkableAxes:g,thisGroup:null}}function u(t,e,n,r,i){var o,a,s,A,l;null===e?((e={})[n]=1,l=t.length,t.push(e)):l=t.indexOf(e);var c=Object.keys(e);for(o=0;o<t.length;o++)if(s=t[o],o!==l&&s[r]){var u=s[r];for(a=0;a<c.length;a++)s[A=c[a]]=u*i*e[A];return void t.splice(l,1)}if(1!==i)for(a=0;a<c.length;a++)e[c[a]]*=i;e[r]=1}function g(t,e){var n=t._inputDomain,r=l[t.constraintoward],i=n[0]+(n[1]-n[0])*r;t.domain=t._input.domain=[i+(n[0]-i)/e,i+(n[1]-i)/e],t.setScale()}n.handleConstraintDefaults=function(t,e,n,i,o){var a,s,A,l,g=o._axisConstraintGroups,h=o._axisMatchGroups,d=e._id,f=d.charAt(0),p=((o._splomAxes||{})[f]||{})[d]||{},I=e._id,C=I.charAt(0),m=n("constrain");if(r.coerce(t,e,{constraintoward:{valType:"enumerated",values:"x"===C?["left","center","right"]:["bottom","middle","top"],dflt:"x"===C?"center":"middle"}},"constraintoward"),!t.matches&&!p.matches||e.fixedrange||(s=c(h,I,i,o),a=r.coerce(t,e,{matches:{valType:"enumerated",values:s.linkableAxes||[],dflt:p.matches}},"matches")),a||!t.scaleanchor||e.fixedrange&&"domain"!==m||(l=c(g,I,i,o,m),A=r.coerce(t,e,{scaleanchor:{valType:"enumerated",values:l.linkableAxes||[]}},"scaleanchor")),a?(delete e.constrain,u(h,s.thisGroup,I,a,1)):-1!==i.indexOf(t.matches)&&r.warn("ignored "+e._name+'.matches: "'+t.matches+'" to avoid either an infinite loop or because the target axis has fixed range.'),A){var y=n("scaleratio");y||(y=e.scaleratio=1),u(g,l.thisGroup,I,A,y)}else-1!==i.indexOf(t.scaleanchor)&&r.warn("ignored "+e._name+'.scaleanchor: "'+t.scaleanchor+'" to avoid either an infinite loop and possibly inconsistent scaleratios, or because the target axis has fixed range or this axis declares a *matches* constraint.')},n.enforce=function(t){var e,n,r,l,c,u,h,d=t._fullLayout,f=d._axisConstraintGroups||[];for(e=0;e<f.length;e++){var p=f[e],I=Object.keys(p),C=1/0,m=0,y=1/0,E={},v={},B=!1;for(n=0;n<I.length;n++)v[r=I[n]]=l=d[i(r)],l._inputDomain?l.domain=l._inputDomain.slice():l._inputDomain=l.domain.slice(),l._inputRange||(l._inputRange=l.range.slice()),l.setScale(),E[r]=c=Math.abs(l._m)/p[r],C=Math.min(C,c),"domain"!==l.constrain&&l._constraintShrinkable||(y=Math.min(y,c)),delete l._constraintShrinkable,m=Math.max(m,c),"domain"===l.constrain&&(B=!0);if(!(C>A*m)||B)for(n=0;n<I.length;n++)if(c=E[r=I[n]],u=(l=v[r]).constrain,c!==y||"domain"===u)if(h=c/y,"range"===u)o(l,h);else{var b=l._inputDomain,Q=(l.domain[1]-l.domain[0])/(b[1]-b[0]),w=(l.r2l(l.range[1])-l.r2l(l.range[0]))/(l.r2l(l._inputRange[1])-l.r2l(l._inputRange[0]));if((h/=Q)*w<1){l.domain=l._input.domain=b.slice(),o(l,h);continue}if(w<1&&(l.range=l._input.range=l._inputRange.slice(),h*=w),l.autorange){var x=l.r2l(l.range[0]),_=l.r2l(l.range[1]),D=(x+_)/2,M=D,k=D,S=Math.abs(_-D),L=D-S*h*1.0001,R=D+S*h*1.0001,N=a(l);g(l,h);var F,T,U=Math.abs(l._m),O=s(t,l),G=O.min,H=O.max;for(T=0;T<G.length;T++)(F=G[T].val-N(G[T])/U)>L&&F<M&&(M=F);for(T=0;T<H.length;T++)(F=H[T].val+N(H[T])/U)<R&&F>k&&(k=F);h/=(k-M)/(2*S),M=l.l2r(M),k=l.l2r(k),l.range=l._input.range=x<_?[M,k]:[k,M]}g(l,h)}}},n.clean=function(t,e){if(e._inputDomain){for(var n=!1,r=e._id,i=t._fullLayout._axisConstraintGroups,o=0;o<i.length;o++)if(i[o][r]){n=!0;break}n&&"domain"===e.constrain||(e._input.domain=e.domain=e._inputDomain,delete e._inputDomain)}}},{"../../constants/alignment":688,"../../constants/numerical":695,"../../lib":719,"./autorange":766,"./axis_ids":770,"./scale_zoom":783}],775:[function(t,e,n){"use strict";var r=t("d3"),i=t("tinycolor2"),o=t("has-passive-events"),a=t("../../registry"),s=t("../../lib"),A=t("../../lib/svg_text_utils"),l=t("../../components/color"),c=t("../../components/drawing"),u=t("../../components/fx"),g=t("./axes"),h=t("../../lib/setcursor"),d=t("../../components/dragelement"),f=t("../../constants/alignment").FROM_TL,p=t("../../lib/clear_gl_canvases"),I=t("../../plot_api/subroutines").redrawReglTraces,C=t("../plots"),m=t("./axis_ids").getFromId,y=t("./select").prepSelect,E=t("./select").clearSelect,v=t("./select").selectOnClick,B=t("./scale_zoom"),b=t("./constants"),Q=b.MINDRAG,w=b.MINZOOM,x=!0;function _(t,e,n,r){var i=s.ensureSingle(t.draglayer,e,n,function(e){e.classed("drag",!0).style({fill:"transparent","stroke-width":0}).attr("data-subplot",t.id)});return i.call(h,r),i.node()}function D(t,e,n,i,o,a,s){var A=_(t,"rect",e,n);return r.select(A).call(c.setRect,i,o,a,s),A}function M(t,e){for(var n=0;n<t.length;n++)if(!t[n].fixedrange)return e;return""}function k(t,e,n,r,i){for(var o=0;o<t.length;o++){var a=t[o];if(!a.fixedrange){var s=a._rl[0],A=a._rl[1]-s;r[a._name+".range[0]"]=a.l2r(s+A*e),r[a._name+".range[1]"]=a.l2r(s+A*n)}}if(i&&i.length){var l=(e+(1-n))/2;k(i,l,1-l,r,[])}}function S(t,e){for(var n=0;n<t.length;n++){var r=t[n];r.fixedrange||(r.range=[r.l2r(r._rl[0]-e/r._m),r.l2r(r._rl[1]-e/r._m)])}}function L(t){return 1-(t>=0?Math.min(t,.9):1/(1/Math.max(t,-.3)+3.222))}function R(t,e,n,r,i){return t.append("path").attr("class","zoombox").style({fill:e>.2?"rgba(0,0,0,0)":"rgba(255,255,255,0)","stroke-width":0}).attr("transform","translate("+n+", "+r+")").attr("d",i+"Z")}function N(t,e,n){return t.append("path").attr("class","zoombox-corners").style({fill:l.background,stroke:l.defaultLine,"stroke-width":1,opacity:0}).attr("transform","translate("+e+", "+n+")").attr("d","M0,0Z")}function F(t,e,n,r,i,o){t.attr("d",r+"M"+n.l+","+n.t+"v"+n.h+"h"+n.w+"v-"+n.h+"h-"+n.w+"Z"),T(t,e,i,o)}function T(t,e,n,r){n||(t.transition().style("fill",r>.2?"rgba(0,0,0,0.4)":"rgba(255,255,255,0.3)").duration(200),e.transition().style("opacity",1).duration(200))}function U(t){r.select(t).selectAll(".zoombox,.js-zoombox-backdrop,.js-zoombox-menu,.zoombox-corners").remove()}function O(t){x&&t.data&&t._context.showTips&&(s.notifier(s._(t,"Double-click to zoom back out"),"long"),x=!1)}function G(t){return"lasso"===t||"select"===t}function H(t){var e=Math.floor(Math.min(t.b-t.t,t.r-t.l,w)/2);return"M"+(t.l-3.5)+","+(t.t-.5+e)+"h3v"+-e+"h"+e+"v-3h-"+(e+3)+"ZM"+(t.r+3.5)+","+(t.t-.5+e)+"h-3v"+-e+"h"+-e+"v-3h"+(e+3)+"ZM"+(t.r+3.5)+","+(t.b+.5-e)+"h-3v"+e+"h"+-e+"v3h"+(e+3)+"ZM"+(t.l-3.5)+","+(t.b+.5-e)+"h3v"+e+"h"+e+"v3h-"+(e+3)+"Z"}function j(t,e,n,r){for(var i,o,a,A,l=!1,c={},u={},g=0;g<e.length;g++){var h=e[g];for(i in n)if(h[i]){for(a in h)("x"===a.charAt(0)?n:r)[a]||(c[a]=i);for(o in r)h[o]&&(l=!0)}for(o in r)if(h[o])for(A in h)("x"===A.charAt(0)?n:r)[A]||(u[A]=o)}l&&(s.extendFlat(c,u),u={});var d={},f=[];for(a in c){var p=m(t,a);f.push(p),d[p._id]=p}var I={},C=[];for(A in u){var y=m(t,A);C.push(y),I[y._id]=y}return{xaHash:d,yaHash:I,xaxes:f,yaxes:C,xLinks:c,yLinks:u,isSubplotConstrained:l}}function Y(t,e){if(o){var n=void 0!==t.onwheel?"wheel":"mousewheel";t._onwheel&&t.removeEventListener(n,t._onwheel),t._onwheel=e,t.addEventListener(n,e,{passive:!1})}else void 0!==t.onwheel?t.onwheel=e:void 0!==t.onmousewheel&&(t.onmousewheel=e)}function P(t){var e=[];for(var n in t)e.push(t[n]);return e}e.exports={makeDragBox:function(t,e,n,o,l,h,x,_){var T,K,z,J,q,V,W,X,Z,$,tt,et,nt,rt,it,ot,at,st,At,lt,ct,ut=t._fullLayout._zoomlayer,gt=x+_==="nsew",ht=1===(x+_).length;function dt(){if(T=e.xaxis,K=e.yaxis,Z=T._length,$=K._length,W=T._offset,X=K._offset,(z={})[T._id]=T,(J={})[K._id]=K,x&&_)for(var n=e.overlays,r=0;r<n.length;r++){var i=n[r].xaxis;z[i._id]=i;var o=n[r].yaxis;J[o._id]=o}q=P(z),V=P(J),nt=M(q,_),rt=M(V,x),it=!rt&&!nt,tt=j(t,t._fullLayout._axisConstraintGroups,z,J),et=j(t,t._fullLayout._axisMatchGroups,z,J),ot=_||tt.isSubplotConstrained||et.isSubplotConstrained,at=x||tt.isSubplotConstrained||et.isSubplotConstrained;var a=t._fullLayout;st=a._has("scattergl"),At=a._has("splom"),lt=a._has("svg")}dt();var ft=function(t,e,n){return t?"nsew"===t?"pan"===e?"move":"crosshair":t.toLowerCase()+"-resize":"pointer"}(rt+nt,t._fullLayout.dragmode),pt=D(e,x+_+"drag",ft,n,o,l,h);if(it&&!gt)return pt.onmousedown=null,pt.style.pointerEvents="none",pt;var It,Ct,mt,yt,Et,vt,Bt,bt,Qt,wt,xt={element:pt,gd:t,plotinfo:e};function _t(){xt.plotinfo.selection=!1,E(t)}function Dt(n,i){var o=t._fullLayout.clickmode;if(U(t),2!==n||ht||function(){if(!t._transitioningWithDuration){var e=t._context.doubleClick,n=[];nt&&(n=n.concat(q)),rt&&(n=n.concat(V)),et.xaxes&&(n=n.concat(et.xaxes)),et.yaxes&&(n=n.concat(et.yaxes));var r,i,o,s={};if("reset+autosize"===e)for(e="autosize",i=0;i<n.length;i++)if((r=n[i])._rangeInitial&&(r.range[0]!==r._rangeInitial[0]||r.range[1]!==r._rangeInitial[1])||!r._rangeInitial&&!r.autorange){e="reset";break}if("autosize"===e)for(i=0;i<n.length;i++)(r=n[i]).fixedrange||(s[r._name+".autorange"]=!0);else if("reset"===e)for((nt||tt.isSubplotConstrained)&&(n=n.concat(tt.xaxes)),rt&&!tt.isSubplotConstrained&&(n=n.concat(tt.yaxes)),tt.isSubplotConstrained&&(nt?rt||(n=n.concat(V)):n=n.concat(q)),i=0;i<n.length;i++)(r=n[i]).fixedrange||(r._rangeInitial?(o=r._rangeInitial,s[r._name+".range[0]"]=o[0],s[r._name+".range[1]"]=o[1]):s[r._name+".autorange"]=!0);t.emit("plotly_doubleclick",null),a.call("_guiRelayout",t,s)}}(),gt)o.indexOf("select")>-1&&v(i,t,q,V,e.id,xt),o.indexOf("event")>-1&&u.click(t,i,e.id);else if(1===n&&ht){var s=x?K:T,l="s"===x||"w"===_?0:1,c=s._name+".range["+l+"]",g=function(t,e){var n,i=t.range[e],o=Math.abs(i-t.range[1-e]);return"date"===t.type?i:"log"===t.type?(n=Math.ceil(Math.max(0,-Math.log(o)/Math.LN10))+3,r.format("."+n+"g")(Math.pow(10,i))):(n=Math.floor(Math.log(Math.abs(i))/Math.LN10)-Math.floor(Math.log(o)/Math.LN10)+4,r.format("."+String(n)+"g")(i))}(s,l),h="left",d="middle";if(s.fixedrange)return;x?(d="n"===x?"top":"bottom","right"===s.side&&(h="right")):"e"===_&&(h="right"),t._context.showAxisRangeEntryBoxes&&r.select(pt).call(A.makeEditable,{gd:t,immediate:!0,background:t._fullLayout.paper_bgcolor,text:String(g),fill:s.tickfont?s.tickfont.color:"#444",horizontalAlign:h,verticalAlign:d}).on("edit",function(e){var n=s.d2r(e);void 0!==n&&a.call("_guiRelayout",t,c,n)})}}function Mt(e,n){if(t._transitioningWithDuration)return!1;var r=Math.max(0,Math.min(Z,e+It)),i=Math.max(0,Math.min($,n+Ct)),o=Math.abs(r-It),a=Math.abs(i-Ct);function s(){Bt="",mt.r=mt.l,mt.t=mt.b,Qt.attr("d","M0,0Z")}if(mt.l=Math.min(It,r),mt.r=Math.max(It,r),mt.t=Math.min(Ct,i),mt.b=Math.max(Ct,i),tt.isSubplotConstrained)o>w||a>w?(Bt="xy",o/Z>a/$?(a=o*$/Z,Ct>i?mt.t=Ct-a:mt.b=Ct+a):(o=a*Z/$,It>r?mt.l=It-o:mt.r=It+o),Qt.attr("d",H(mt))):s();else if(et.isSubplotConstrained)if(o>w||a>w){Bt="xy";var A=Math.min(mt.l/Z,($-mt.b)/$),l=Math.max(mt.r/Z,($-mt.t)/$);mt.l=A*Z,mt.r=l*Z,mt.b=(1-A)*$,mt.t=(1-l)*$,Qt.attr("d",H(mt))}else s();else!rt||a<Math.min(Math.max(.6*o,Q),w)?o<Q||!nt?s():(mt.t=0,mt.b=$,Bt="x",Qt.attr("d",function(t,e){return"M"+(t.l-.5)+","+(e-w-.5)+"h-3v"+(2*w+1)+"h3ZM"+(t.r+.5)+","+(e-w-.5)+"h3v"+(2*w+1)+"h-3Z"}(mt,Ct))):!nt||o<Math.min(.6*a,w)?(mt.l=0,mt.r=Z,Bt="y",Qt.attr("d",function(t,e){return"M"+(e-w-.5)+","+(t.t-.5)+"v-3h"+(2*w+1)+"v3ZM"+(e-w-.5)+","+(t.b+.5)+"v3h"+(2*w+1)+"v-3Z"}(mt,It))):(Bt="xy",Qt.attr("d",H(mt)));mt.w=mt.r-mt.l,mt.h=mt.b-mt.t,Bt&&(wt=!0),t._dragged=wt,F(bt,Qt,mt,Et,vt,yt),kt(),t.emit("plotly_relayouting",ct),vt=!0}function kt(){ct={},"xy"!==Bt&&"x"!==Bt||(k(q,mt.l/Z,mt.r/Z,ct,tt.xaxes),Ut("x",ct)),"xy"!==Bt&&"y"!==Bt||(k(V,($-mt.b)/$,($-mt.t)/$,ct,tt.yaxes),Ut("y",ct))}function St(){if(Math.min(mt.h,mt.w)<2*Q)return U(t);kt(),U(t),Gt(),O(t)}xt.prepFn=function(e,n,r){var o=xt.dragmode,a=t._fullLayout.dragmode;a!==o&&(xt.dragmode=a),dt(),it||(gt?e.shiftKey?"pan"===a?a="zoom":G(a)||(a="pan"):e.ctrlKey&&(a="pan"):a="pan"),xt.minDrag="lasso"===a?1:void 0,G(a)?(xt.xaxes=q,xt.yaxes=V,y(e,n,r,xt,a)):(xt.clickFn=Dt,G(o)&&_t(),it||("zoom"===a?(xt.moveFn=Mt,xt.doneFn=St,xt.minDrag=1,function(e,n,r){var o=pt.getBoundingClientRect();It=n-o.left,Ct=r-o.top,mt={l:It,r:It,w:0,t:Ct,b:Ct,h:0},yt=t._hmpixcount?t._hmlumcount/t._hmpixcount:i(t._fullLayout.plot_bgcolor).getLuminance(),vt=!1,Bt="xy",wt=!1,bt=R(ut,yt,W,X,Et="M0,0H"+Z+"V"+$+"H0V0"),Qt=N(ut,W,X)}(0,n,r)):"pan"===a&&(xt.moveFn=Tt,xt.doneFn=Gt))),t._fullLayout._redrag=function(){var e=t._dragdata;e&&e.element===pt&&(G(t._fullLayout.dragmode)||(dt(),Ht([0,0,Z,$]),xt.moveFn(e.dx,e.dy)))}},d.init(xt);var Lt=[0,0,Z,$],Rt=null,Nt=b.REDRAWDELAY,Ft=e.mainplot?t._fullLayout._plots[e.mainplot]:e;function Tt(e,n){if(!t._transitioningWithDuration){if(t._fullLayout._replotting=!0,"ew"===nt||"ns"===rt)return nt&&(S(q,e),Ut("x")),rt&&(S(V,n),Ut("y")),Ht([nt?-e:0,rt?-n:0,Z,$]),Ot(),void t.emit("plotly_relayouting",ct);if(tt.isSubplotConstrained&&nt&&rt){var r="w"===nt==("n"===rt)?1:-1,i=(e/Z+r*n/$)/2;e=i*Z,n=r*i*$}"w"===nt?e=A(q,0,e):"e"===nt?e=A(q,1,-e):nt||(e=0),"n"===rt?n=A(V,1,n):"s"===rt?n=A(V,0,-n):rt||(n=0);var o="w"===nt?e:0,a="n"===rt?n:0;if(tt.isSubplotConstrained){var s;if(!nt&&1===rt.length){for(s=0;s<q.length;s++)q[s].range=q[s]._r.slice(),B(q[s],1-n/$);o=(e=n*Z/$)/2}if(!rt&&1===nt.length){for(s=0;s<V.length;s++)V[s].range=V[s]._r.slice(),B(V[s],1-e/Z);a=(n=e*$/Z)/2}}Ut("x"),Ut("y"),Ht([o,a,Z-e,$-n]),Ot(),t.emit("plotly_relayouting",ct)}function A(t,e,n){for(var r,i,o=1-e,a=0;a<t.length;a++){var s=t[a];if(!s.fixedrange){r=s,i=s._rl[o]+(s._rl[e]-s._rl[o])/L(n/s._length);var A=s.l2r(i);!1!==A&&void 0!==A&&(s.range[e]=A)}}return r._length*(r._rl[e]-i)/(r._rl[e]-r._rl[o])}}function Ut(t,e){for(var n=et.isSubplotConstrained?{x:V,y:q}[t]:et[t+"axes"],r=et.isSubplotConstrained?{x:q,y:V}[t]:[],i=0;i<n.length;i++){var o=n[i],a=o._id,s=et.xLinks[a]||et.yLinks[a],A=r[0]||z[s]||J[s];A&&(e?(e[o._name+".range[0]"]=e[A._name+".range[0]"],e[o._name+".range[1]"]=e[A._name+".range[1]"]):o.range=A.range.slice())}}function Ot(){var e,n=[];function r(t){for(e=0;e<t.length;e++)t[e].fixedrange||n.push(t[e]._id)}for(ot&&(r(q),r(tt.xaxes),r(et.xaxes)),at&&(r(V),r(tt.yaxes),r(et.yaxes)),ct={},e=0;e<n.length;e++){var i=n[e],o=m(t,i);g.drawOne(t,o,{skipTitle:!0}),ct[o._name+".range[0]"]=o.range[0],ct[o._name+".range[1]"]=o.range[1]}g.redrawComponents(t,n)}function Gt(){Ht([0,0,Z,$]),s.syncOrAsync([C.previousPromises,function(){t._fullLayout._replotting=!1,a.call("_guiRelayout",t,ct)}],t)}function Ht(e){var n,r,i,o,A=t._fullLayout,l=A._plots,u=A._subplots.cartesian;if(At&&a.subplotsRegistry.splom.drag(t),st)for(n=0;n<u.length;n++)if(i=(r=l[u[n]]).xaxis,o=r.yaxis,r._scene){var g=s.simpleMap(i.range,i.r2l),h=s.simpleMap(o.range,o.r2l);r._scene.update({range:[g[0],h[0],g[1],h[1]]})}if((At||st)&&(p(t),I(t)),lt){var d=e[2]/T._length,f=e[3]/K._length;for(n=0;n<u.length;n++){i=(r=l[u[n]]).xaxis,o=r.yaxis;var C,m,y,E,v=ot&&!i.fixedrange&&z[i._id],B=at&&!o.fixedrange&&J[o._id];if(v?(C=d,y=_?e[0]:Pt(i,C)):et.xaHash[i._id]?(C=d,y=e[0]*i._length/T._length):et.yaHash[i._id]?(C=f,y="ns"===rt?-e[1]*i._length/K._length:Pt(i,C,{n:"top",s:"bottom"}[rt])):y=Yt(i,C=jt(i,d,f)),B?(m=f,E=x?e[1]:Pt(o,m)):et.yaHash[o._id]?(m=f,E=e[1]*o._length/K._length):et.xaHash[o._id]?(m=d,E="ew"===nt?-e[0]*o._length/T._length:Pt(o,m,{e:"right",w:"left"}[nt])):E=Yt(o,m=jt(o,d,f)),C||m){C||(C=1),m||(m=1);var b=i._offset-y/C,Q=o._offset-E/m;r.clipRect.call(c.setTranslate,y,E).call(c.setScale,C,m),r.plot.call(c.setTranslate,b,Q).call(c.setScale,1/C,1/m),C===r.xScaleFactor&&m===r.yScaleFactor||(c.setPointGroupScale(r.zoomScalePts,C,m),c.setTextPointsScale(r.zoomScaleTxt,C,m)),c.hideOutsideRangePoints(r.clipOnAxisFalseTraces,r),r.xScaleFactor=C,r.yScaleFactor=m}}}}function jt(t,e,n){return t.fixedrange?0:ot&&tt.xaHash[t._id]?e:at&&(tt.isSubplotConstrained?tt.xaHash:tt.yaHash)[t._id]?n:0}function Yt(t,e){return e?(t.range=t._r.slice(),B(t,e),Pt(t,e)):0}function Pt(t,e,n){return t._length*(1-e)*f[n||t.constraintoward||"middle"]}return x.length*_.length!=1&&Y(pt,function(e){if(t._context._scrollZoom.cartesian||t._fullLayout._enablescrollzoom){if(_t(),t._transitioningWithDuration)return e.preventDefault(),void e.stopPropagation();dt(),clearTimeout(Rt);var n=-e.deltaY;if(isFinite(n)||(n=e.wheelDelta/10),isFinite(n)){var r,i=Math.exp(-Math.min(Math.max(n,-20),20)/200),o=Ft.draglayer.select(".nsewdrag").node().getBoundingClientRect(),a=(e.clientX-o.left)/o.width,A=(o.bottom-e.clientY)/o.height;if(ot){for(_||(a=.5),r=0;r<q.length;r++)l(q[r],a,i);Ut("x"),Lt[2]*=i,Lt[0]+=Lt[2]*a*(1/i-1)}if(at){for(x||(A=.5),r=0;r<V.length;r++)l(V[r],A,i);Ut("y"),Lt[3]*=i,Lt[1]+=Lt[3]*(1-A)*(1/i-1)}Ht(Lt),Ot(),t.emit("plotly_relayouting",ct),Rt=setTimeout(function(){Lt=[0,0,Z,$],Gt()},Nt),e.preventDefault()}else s.log("Did not find wheel motion attributes: ",e)}function l(t,e,n){if(!t.fixedrange){var r=s.simpleMap(t.range,t.r2l),i=r[0]+(r[1]-r[0])*e;t.range=r.map(function(e){return t.l2r(i+(e-i)*n)})}}}),pt},makeDragger:_,makeRectDragger:D,makeZoombox:R,makeCorners:N,updateZoombox:F,xyCorners:H,transitionZoombox:T,removeZoombox:U,showDoubleClickNotifier:O,attachWheelEventHandler:Y}},{"../../components/color":593,"../../components/dragelement":611,"../../components/drawing":614,"../../components/fx":632,"../../constants/alignment":688,"../../lib":719,"../../lib/clear_gl_canvases":704,"../../lib/setcursor":739,"../../lib/svg_text_utils":743,"../../plot_api/subroutines":758,"../../registry":847,"../plots":828,"./axes":767,"./axis_ids":770,"./constants":773,"./scale_zoom":783,"./select":784,d3:163,"has-passive-events":411,tinycolor2:537}],776:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/fx"),o=t("../../components/dragelement"),a=t("../../lib/setcursor"),s=t("./dragbox").makeDragBox,A=t("./constants").DRAGGERSIZE;n.initInteractions=function(t){var e=t._fullLayout;if(t._context.staticPlot)r.select(t).selectAll(".drag").remove();else if(e._has("cartesian")||e._has("splom")){Object.keys(e._plots||{}).sort(function(t,n){if((e._plots[t].mainplot&&!0)===(e._plots[n].mainplot&&!0)){var r=t.split("y"),i=n.split("y");return r[0]===i[0]?Number(r[1]||1)-Number(i[1]||1):Number(r[0]||1)-Number(i[0]||1)}return e._plots[t].mainplot?1:-1}).forEach(function(n){var r=e._plots[n],a=r.xaxis,l=r.yaxis;if(!r.mainplot){var c=s(t,r,a._offset,l._offset,a._length,l._length,"ns","ew");c.onmousemove=function(e){t._fullLayout._rehover=function(){t._fullLayout._hoversubplot===n&&i.hover(t,e,n)},i.hover(t,e,n),t._fullLayout._lasthover=c,t._fullLayout._hoversubplot=n},c.onmouseout=function(e){t._dragging||(t._fullLayout._hoversubplot=null,o.unhover(t,e))},t._context.showAxisDragHandles&&(s(t,r,a._offset-A,l._offset-A,A,A,"n","w"),s(t,r,a._offset+a._length,l._offset-A,A,A,"n","e"),s(t,r,a._offset-A,l._offset+l._length,A,A,"s","w"),s(t,r,a._offset+a._length,l._offset+l._length,A,A,"s","e"))}if(t._context.showAxisDragHandles){if(n===a._mainSubplot){var u=a._mainLinePosition;"top"===a.side&&(u-=A),s(t,r,a._offset+.1*a._length,u,.8*a._length,A,"","ew"),s(t,r,a._offset,u,.1*a._length,A,"","w"),s(t,r,a._offset+.9*a._length,u,.1*a._length,A,"","e")}if(n===l._mainSubplot){var g=l._mainLinePosition;"right"!==l.side&&(g-=A),s(t,r,g,l._offset+.1*l._length,A,.8*l._length,"ns",""),s(t,r,g,l._offset+.9*l._length,A,.1*l._length,"s",""),s(t,r,g,l._offset,A,.1*l._length,"n","")}}});var a=e._hoverlayer.node();a.onmousemove=function(n){n.target=t._fullLayout._lasthover,i.hover(t,n,e._hoversubplot)},a.onclick=function(e){e.target=t._fullLayout._lasthover,i.click(t,e)},a.onmousedown=function(e){t._fullLayout._lasthover.onmousedown(e)},n.updateFx(t)}},n.updateFx=function(t){var e=t._fullLayout,n="pan"===e.dragmode?"move":"crosshair";a(e._draggers,n)}},{"../../components/dragelement":611,"../../components/fx":632,"../../lib/setcursor":739,"./constants":773,"./dragbox":775,d3:163}],777:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib");e.exports=function(t){return function(e,n){var o=e[t];if(Array.isArray(o))for(var a=r.subplotsRegistry.cartesian,s=a.idRegex,A=n._subplots,l=A.xaxis,c=A.yaxis,u=A.cartesian,g=n._has("cartesian")||n._has("gl2d"),h=0;h<o.length;h++){var d=o[h];if(i.isPlainObject(d)){var f=d.xref,p=d.yref,I=s.x.test(f),C=s.y.test(p);if(I||C){g||i.pushUnique(n._basePlotModules,a);var m=!1;I&&-1===l.indexOf(f)&&(l.push(f),m=!0),C&&-1===c.indexOf(p)&&(c.push(p),m=!0),m&&I&&C&&u.push(f+p)}}}}}},{"../../lib":719,"../../registry":847}],778:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../registry"),o=t("../../lib"),a=t("../plots"),s=t("../../components/drawing"),A=t("../get_data").getModuleCalcData,l=t("./axis_ids"),c=t("./constants"),u=t("../../constants/xmlns_namespaces"),g=o.ensureSingle;function h(t,e,n){return o.ensureSingle(t,e,n,function(t){t.datum(n)})}function d(t,e,n,o,a){for(var l,u,g,h=c.traceLayerClasses,d=t._fullLayout,f=d._modules,p=[],I=[],C=0;C<f.length;C++){var m=(l=f[C]).name,y=i.modules[m].categories;if(y.svg){var E=l.layerName||m+"layer",v=l.plot;g=(u=A(n,v))[0],n=u[1],g.length&&p.push({i:h.indexOf(E),className:E,plotMethod:v,cdModule:g}),y.zoomScale&&I.push("."+E)}}p.sort(function(t,e){return t.i-e.i});var B=e.plot.selectAll("g.mlayer").data(p,function(t){return t.className});if(B.enter().append("g").attr("class",function(t){return t.className}).classed("mlayer",!0).classed("rangeplot",e.isRangePlot),B.exit().remove(),B.order(),B.each(function(n){var i=r.select(this),A=n.className;n.plotMethod(t,e,n.cdModule,i,o,a),-1===c.clipOnAxisFalseQuery.indexOf("."+A)&&s.setClipUrl(i,e.layerClipId,t)}),d._has("scattergl")&&(l=i.getModule("scattergl"),g=A(n,l)[0],l.plot(t,e,g)),!t._context.staticPlot&&(e._hasClipOnAxisFalse&&(e.clipOnAxisFalseTraces=e.plot.selectAll(c.clipOnAxisFalseQuery.join(",")).selectAll(".trace")),I.length)){var b=e.plot.selectAll(I.join(",")).selectAll(".trace");e.zoomScalePts=b.selectAll("path.point"),e.zoomScaleTxt=b.selectAll(".textpoint")}}function f(t,e){var n=e.plotgroup,r=e.id,i=c.layerValue2layerClass[e.xaxis.layer],o=c.layerValue2layerClass[e.yaxis.layer],a=t._fullLayout._hasOnlyLargeSploms;if(e.mainplot){var s=e.mainplotinfo,A=s.plotgroup,u=r+"-x",d=r+"-y";e.gridlayer=s.gridlayer,e.zerolinelayer=s.zerolinelayer,g(s.overlinesBelow,"path",u),g(s.overlinesBelow,"path",d),g(s.overaxesBelow,"g",u),g(s.overaxesBelow,"g",d),e.plot=g(s.overplot,"g",r),g(s.overlinesAbove,"path",u),g(s.overlinesAbove,"path",d),g(s.overaxesAbove,"g",u),g(s.overaxesAbove,"g",d),e.xlines=A.select(".overlines-"+i).select("."+u),e.ylines=A.select(".overlines-"+o).select("."+d),e.xaxislayer=A.select(".overaxes-"+i).select("."+u),e.yaxislayer=A.select(".overaxes-"+o).select("."+d)}else if(a)e.xlines=g(n,"path","xlines-above"),e.ylines=g(n,"path","ylines-above"),e.xaxislayer=g(n,"g","xaxislayer-above"),e.yaxislayer=g(n,"g","yaxislayer-above");else{var f=g(n,"g","layer-subplot");e.shapelayer=g(f,"g","shapelayer"),e.imagelayer=g(f,"g","imagelayer"),e.gridlayer=g(n,"g","gridlayer"),e.zerolinelayer=g(n,"g","zerolinelayer"),g(n,"path","xlines-below"),g(n,"path","ylines-below"),e.overlinesBelow=g(n,"g","overlines-below"),g(n,"g","xaxislayer-below"),g(n,"g","yaxislayer-below"),e.overaxesBelow=g(n,"g","overaxes-below"),e.plot=g(n,"g","plot"),e.overplot=g(n,"g","overplot"),e.xlines=g(n,"path","xlines-above"),e.ylines=g(n,"path","ylines-above"),e.overlinesAbove=g(n,"g","overlines-above"),g(n,"g","xaxislayer-above"),g(n,"g","yaxislayer-above"),e.overaxesAbove=g(n,"g","overaxes-above"),e.xlines=n.select(".xlines-"+i),e.ylines=n.select(".ylines-"+o),e.xaxislayer=n.select(".xaxislayer-"+i),e.yaxislayer=n.select(".yaxislayer-"+o)}a||(h(e.gridlayer,"g",e.xaxis._id),h(e.gridlayer,"g",e.yaxis._id),e.gridlayer.selectAll("g").map(function(t){return t[0]}).sort(l.idSort)),e.xlines.style("fill","none").classed("crisp",!0),e.ylines.style("fill","none").classed("crisp",!0)}function p(t,e){if(t){var n={};for(var i in t.each(function(t){var i=t[0];r.select(this).remove(),I(i,e),n[i]=!0}),e._plots)for(var o=e._plots[i].overlays||[],a=0;a<o.length;a++){var s=o[a];n[s.id]&&s.plot.selectAll(".trace").remove()}}}function I(t,e){e._draggers.selectAll("g."+t).remove(),e._defs.select("#clip"+e._uid+t+"plot").remove()}n.name="cartesian",n.attr=["xaxis","yaxis"],n.idRoot=["x","y"],n.idRegex=c.idRegex,n.attrRegex=c.attrRegex,n.attributes=t("./attributes"),n.layoutAttributes=t("./layout_attributes"),n.supplyLayoutDefaults=t("./layout_defaults"),n.transitionAxes=t("./transition_axes"),n.finalizeSubplots=function(t,e){var n,r,i,a=e._subplots,s=a.xaxis,A=a.yaxis,u=a.cartesian,g=u.concat(a.gl2d||[]),h={},d={};for(n=0;n<g.length;n++){var f=g[n].split("y");h[f[0]]=1,d["y"+f[1]]=1}for(n=0;n<s.length;n++)h[r=s[n]]||(i=(t[l.id2name(r)]||{}).anchor,c.idRegex.y.test(i)||(i="y"),u.push(r+i),g.push(r+i),d[i]||(d[i]=1,o.pushUnique(A,i)));for(n=0;n<A.length;n++)d[i=A[n]]||(r=(t[l.id2name(i)]||{}).anchor,c.idRegex.x.test(r)||(r="x"),u.push(r+i),g.push(r+i),h[r]||(h[r]=1,o.pushUnique(s,r)));if(!g.length){for(var p in r="",i="",t)c.attrRegex.test(p)&&("x"===p.charAt(0)?(!r||+p.substr(5)<+r.substr(5))&&(r=p):(!i||+p.substr(5)<+i.substr(5))&&(i=p));r=r?l.name2id(r):"x",i=i?l.name2id(i):"y",s.push(r),A.push(i),u.push(r+i)}},n.plot=function(t,e,n,r){var i,o=t._fullLayout,a=o._subplots.cartesian,s=t.calcdata;if(!Array.isArray(e))for(e=[],i=0;i<s.length;i++)e.push(i);for(i=0;i<a.length;i++){for(var A,l=a[i],c=o._plots[l],u=[],g=0;g<s.length;g++){var h=s[g],f=h[0].trace;f.xaxis+f.yaxis===l&&((-1!==e.indexOf(f.index)||f.carpet)&&(A&&A[0].trace.xaxis+A[0].trace.yaxis===l&&-1!==["tonextx","tonexty","tonext"].indexOf(f.fill)&&-1===u.indexOf(A)&&u.push(A),u.push(h)),A=h)}d(t,c,u,n,r)}},n.clean=function(t,e,n,r){var i,o,a,s=r._plots||{},A=e._plots||{},c=r._subplots||{};if(r._hasOnlyLargeSploms&&!e._hasOnlyLargeSploms)for(a in s)(i=s[a]).plotgroup&&i.plotgroup.remove();var u=r._has&&r._has("gl"),g=e._has&&e._has("gl");if(u&&!g)for(a in s)(i=s[a])._scene&&i._scene.destroy();if(c.xaxis&&c.yaxis){var h=l.listIds({_fullLayout:r});for(o=0;o<h.length;o++){var d=h[o];e[l.id2name(d)]||r._infolayer.selectAll(".g-"+d+"title").remove()}}var f=r._has&&r._has("cartesian"),C=e._has&&e._has("cartesian");if(f&&!C)p(r._cartesianlayer.selectAll(".subplot"),r),r._defs.selectAll(".axesclip").remove(),delete r._axisConstraintGroups;else if(c.cartesian)for(o=0;o<c.cartesian.length;o++){var m=c.cartesian[o];if(!A[m]){var y="."+m+",."+m+"-x,."+m+"-y";r._cartesianlayer.selectAll(y).remove(),I(m,r)}}},n.drawFramework=function(t){var e=t._fullLayout,n=function(t){var e,n,r,i,o,a,s=t._fullLayout,A=s._subplots.cartesian,l=A.length,c=[],u=[];for(e=0;e<l;e++){r=A[e],i=s._plots[r],o=i.xaxis,a=i.yaxis;var g=o._mainAxis,h=a._mainAxis,d=g._id+h._id,f=s._plots[d];i.overlays=[],d!==r&&f?(i.mainplot=d,i.mainplotinfo=f,u.push(r)):(i.mainplot=void 0,i.mainPlotinfo=void 0,c.push(r))}for(e=0;e<u.length;e++)r=u[e],(i=s._plots[r]).mainplotinfo.overlays.push(i);var p=c.concat(u),I=new Array(l);for(e=0;e<l;e++){r=p[e],i=s._plots[r],o=i.xaxis,a=i.yaxis;var C=[r,o.layer,a.layer,o.overlaying||"",a.overlaying||""];for(n=0;n<i.overlays.length;n++)C.push(i.overlays[n].id);I[e]=C}return I}(t),i=e._cartesianlayer.selectAll(".subplot").data(n,String);i.enter().append("g").attr("class",function(t){return"subplot "+t[0]}),i.order(),i.exit().call(p,e),i.each(function(n){var i=n[0],o=e._plots[i];o.plotgroup=r.select(this),f(t,o),o.draglayer=g(e._draggers,"g",i)})},n.rangePlot=function(t,e,n){f(t,e),d(t,e,n),a.style(t)},n.toSVG=function(t){var e=t._fullLayout._glimages,n=r.select(t).selectAll(".svg-container");n.filter(function(t,e){return e===n.size()-1}).selectAll(".gl-canvas-context, .gl-canvas-focus").each(function(){var t=this.toDataURL("image/png");e.append("svg:image").attr({xmlns:u.svg,"xlink:href":t,preserveAspectRatio:"none",x:0,y:0,width:this.width,height:this.height})})},n.updateFx=t("./graph_interact").updateFx},{"../../components/drawing":614,"../../constants/xmlns_namespaces":696,"../../lib":719,"../../registry":847,"../get_data":802,"../plots":828,"./attributes":765,"./axis_ids":770,"./constants":773,"./graph_interact":776,"./layout_attributes":779,"./layout_defaults":780,"./transition_axes":789,d3:163}],779:[function(t,e,n){"use strict";var r=t("../font_attributes"),i=t("../../components/color/attributes"),o=t("../../components/drawing/attributes").dash,a=t("../../lib/extend").extendFlat,s=t("../../plot_api/plot_template").templatedArray,A=(t("../../constants/docs").FORMAT_LINK,t("../../constants/docs").DATE_FORMAT_LINK,t("./constants"));e.exports={visible:{valType:"boolean",editType:"plot"},color:{valType:"color",dflt:i.defaultLine,editType:"ticks"},title:{text:{valType:"string",editType:"ticks"},font:r({editType:"ticks"}),editType:"ticks"},type:{valType:"enumerated",values:["-","linear","log","date","category","multicategory"],dflt:"-",editType:"calc",_noTemplating:!0},autorange:{valType:"enumerated",values:[!0,!1,"reversed"],dflt:!0,editType:"axrange",impliedEdits:{"range[0]":void 0,"range[1]":void 0}},rangemode:{valType:"enumerated",values:["normal","tozero","nonnegative"],dflt:"normal",editType:"plot"},range:{valType:"info_array",items:[{valType:"any",editType:"axrange",impliedEdits:{"^autorange":!1},anim:!0},{valType:"any",editType:"axrange",impliedEdits:{"^autorange":!1},anim:!0}],editType:"axrange",impliedEdits:{autorange:!1},anim:!0},fixedrange:{valType:"boolean",dflt:!1,editType:"calc"},scaleanchor:{valType:"enumerated",values:[A.idRegex.x.toString(),A.idRegex.y.toString()],editType:"plot"},scaleratio:{valType:"number",min:0,dflt:1,editType:"plot"},constrain:{valType:"enumerated",values:["range","domain"],dflt:"range",editType:"plot"},constraintoward:{valType:"enumerated",values:["left","center","right","top","middle","bottom"],editType:"plot"},matches:{valType:"enumerated",values:[A.idRegex.x.toString(),A.idRegex.y.toString()],editType:"calc"},tickmode:{valType:"enumerated",values:["auto","linear","array"],editType:"ticks",impliedEdits:{tick0:void 0,dtick:void 0}},nticks:{valType:"integer",min:0,dflt:0,editType:"ticks"},tick0:{valType:"any",editType:"ticks",impliedEdits:{tickmode:"linear"}},dtick:{valType:"any",editType:"ticks",impliedEdits:{tickmode:"linear"}},tickvals:{valType:"data_array",editType:"ticks"},ticktext:{valType:"data_array",editType:"ticks"},ticks:{valType:"enumerated",values:["outside","inside",""],editType:"ticks"},tickson:{valType:"enumerated",values:["labels","boundaries"],dflt:"labels",editType:"ticks"},mirror:{valType:"enumerated",values:[!0,"ticks",!1,"all","allticks"],dflt:!1,editType:"ticks+layoutstyle"},ticklen:{valType:"number",min:0,dflt:5,editType:"ticks"},tickwidth:{valType:"number",min:0,dflt:1,editType:"ticks"},tickcolor:{valType:"color",dflt:i.defaultLine,editType:"ticks"},showticklabels:{valType:"boolean",dflt:!0,editType:"ticks"},automargin:{valType:"boolean",dflt:!1,editType:"ticks"},showspikes:{valType:"boolean",dflt:!1,editType:"modebar"},spikecolor:{valType:"color",dflt:null,editType:"none"},spikethickness:{valType:"number",dflt:3,editType:"none"},spikedash:a({},o,{dflt:"dash",editType:"none"}),spikemode:{valType:"flaglist",flags:["toaxis","across","marker"],dflt:"toaxis",editType:"none"},spikesnap:{valType:"enumerated",values:["data","cursor"],dflt:"data",editType:"none"},tickfont:r({editType:"ticks"}),tickangle:{valType:"angle",dflt:"auto",editType:"ticks"},tickprefix:{valType:"string",dflt:"",editType:"ticks"},showtickprefix:{valType:"enumerated",values:["all","first","last","none"],dflt:"all",editType:"ticks"},ticksuffix:{valType:"string",dflt:"",editType:"ticks"},showticksuffix:{valType:"enumerated",values:["all","first","last","none"],dflt:"all",editType:"ticks"},showexponent:{valType:"enumerated",values:["all","first","last","none"],dflt:"all",editType:"ticks"},exponentformat:{valType:"enumerated",values:["none","e","E","power","SI","B"],dflt:"B",editType:"ticks"},separatethousands:{valType:"boolean",dflt:!1,editType:"ticks"},tickformat:{valType:"string",dflt:"",editType:"ticks"},tickformatstops:s("tickformatstop",{enabled:{valType:"boolean",dflt:!0,editType:"ticks"},dtickrange:{valType:"info_array",items:[{valType:"any",editType:"ticks"},{valType:"any",editType:"ticks"}],editType:"ticks"},value:{valType:"string",dflt:"",editType:"ticks"},editType:"ticks"}),hoverformat:{valType:"string",dflt:"",editType:"none"},showline:{valType:"boolean",dflt:!1,editType:"ticks+layoutstyle"},linecolor:{valType:"color",dflt:i.defaultLine,editType:"layoutstyle"},linewidth:{valType:"number",min:0,dflt:1,editType:"ticks+layoutstyle"},showgrid:{valType:"boolean",editType:"ticks"},gridcolor:{valType:"color",dflt:i.lightLine,editType:"ticks"},gridwidth:{valType:"number",min:0,dflt:1,editType:"ticks"},zeroline:{valType:"boolean",editType:"ticks"},zerolinecolor:{valType:"color",dflt:i.defaultLine,editType:"ticks"},zerolinewidth:{valType:"number",dflt:1,editType:"ticks"},showdividers:{valType:"boolean",dflt:!0,editType:"ticks"},dividercolor:{valType:"color",dflt:i.defaultLine,editType:"ticks"},dividerwidth:{valType:"number",dflt:1,editType:"ticks"},anchor:{valType:"enumerated",values:["free",A.idRegex.x.toString(),A.idRegex.y.toString()],editType:"plot"},side:{valType:"enumerated",values:["top","bottom","left","right"],editType:"plot"},overlaying:{valType:"enumerated",values:["free",A.idRegex.x.toString(),A.idRegex.y.toString()],editType:"plot"},layer:{valType:"enumerated",values:["above traces","below traces"],dflt:"above traces",editType:"plot"},domain:{valType:"info_array",items:[{valType:"number",min:0,max:1,editType:"plot"},{valType:"number",min:0,max:1,editType:"plot"}],dflt:[0,1],editType:"plot"},position:{valType:"number",min:0,max:1,dflt:0,editType:"plot"},categoryorder:{valType:"enumerated",values:["trace","category ascending","category descending","array","total ascending","total descending","min ascending","min descending","max ascending","max descending","sum ascending","sum descending","mean ascending","mean descending","median ascending","median descending"],dflt:"trace",editType:"calc"},categoryarray:{valType:"data_array",editType:"calc"},uirevision:{valType:"any",editType:"none"},editType:"calc",_deprecated:{autotick:{valType:"boolean",editType:"ticks"},title:{valType:"string",editType:"ticks"},titlefont:r({editType:"ticks"})}}},{"../../components/color/attributes":592,"../../components/drawing/attributes":613,"../../constants/docs":690,"../../lib/extend":710,"../../plot_api/plot_template":757,"../font_attributes":793,"./constants":773}],780:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/color"),o=t("../../plot_api/plot_template"),a=t("../layout_attributes"),s=t("./layout_attributes"),A=t("./type_defaults"),l=t("./axis_defaults"),c=t("./constraints").handleConstraintDefaults,u=t("./position_defaults"),g=t("./axis_ids"),h=g.id2name,d=g.name2id,f=t("../../registry"),p=f.traceIs,I=f.getComponentMethod;function C(t,e,n){Array.isArray(t[e])?t[e].push(n):t[e]=[n]}e.exports=function(t,e,n){var g,f,m={},y={},E={},v={},B={},b={},Q={},w={},x={};for(g=0;g<n.length;g++){var _=n[g];if(p(_,"cartesian")||p(_,"gl2d")){var D,M;if(_.xaxis)C(m,D=h(_.xaxis),_);else if(_.xaxes)for(f=0;f<_.xaxes.length;f++)C(m,h(_.xaxes[f]),_);if(_.yaxis)C(m,M=h(_.yaxis),_);else if(_.yaxes)for(f=0;f<_.yaxes.length;f++)C(m,h(_.yaxes[f]),_);"funnel"===_.type?"h"===_.orientation?(D&&(y[D]=!0),M&&(Q[M]=!0)):M&&(E[M]=!0):(M&&(B[M]=!0,b[M]=!0),p(_,"carpet")&&("carpet"!==_.type||_._cheater)||D&&(v[D]=!0)),"carpet"===_.type&&_._cheater&&D&&(y[D]=!0),p(_,"2dMap")&&(w[D]=!0,w[M]=!0),p(_,"oriented")&&(x["h"===_.orientation?M:D]=!0)}}var k=e._subplots,S=k.xaxis,L=k.yaxis,R=r.simpleMap(S,h),N=r.simpleMap(L,h),F=R.concat(N),T=i.background;S.length&&L.length&&(T=r.coerce(t,e,a,"plot_bgcolor"));var U,O,G,H,j=i.combine(T,e.paper_bgcolor);function Y(t,e){return r.coerce(G,H,s,t,e)}function P(t,e){return r.coerce2(G,H,s,t,e)}function K(t){return"x"===t?L:S}var z={x:K("x"),y:K("y")},J=z.x.concat(z.y);function q(e,n){for(var r="x"===e?R:N,i=[],o=0;o<r.length;o++){var a=r[o];a===n||(t[a]||{}).overlaying||i.push(d(a))}return i}for(g=0;g<F.length;g++){O=(U=F[g]).charAt(0),r.isPlainObject(t[U])||(t[U]={}),G=t[U],H=o.newContainer(e,U,O+"axis");var V=m[U]||[];H._traceIndices=V.map(function(t){return t._expandedIndex}),H._annIndices=[],H._shapeIndices=[],H._imgIndices=[],H._subplotsWith=[],H._counterAxes=[],H._name=H._attr=U;var W=H._id=d(U),X=q(O,U),Z="x"===O&&!v[U]&&y[U]||"y"===O&&!B[U]&&E[U],$="y"===O&&!b[U]&&Q[U],tt={letter:O,font:e.font,outerTicks:w[U],showGrid:!x[U],data:V,bgColor:j,calendar:e.calendar,automargin:!0,visibleDflt:Z,reverseDflt:$,splomStash:((e._splomAxes||{})[O]||{})[W]};Y("uirevision",e.uirevision),A(G,H,Y,tt),l(G,H,Y,tt,e);var et=P("spikecolor"),nt=P("spikethickness"),rt=P("spikedash"),it=P("spikemode"),ot=P("spikesnap");Y("showspikes",!!(et||nt||rt||it||ot))||(delete H.spikecolor,delete H.spikethickness,delete H.spikedash,delete H.spikemode,delete H.spikesnap),u(G,H,Y,{letter:O,counterAxes:z[O],overlayableAxes:X,grid:e.grid}),H._input=G}var at=I("rangeslider","handleDefaults"),st=I("rangeselector","handleDefaults");for(g=0;g<R.length;g++)U=R[g],G=t[U],H=e[U],at(t,e,U),"date"===H.type&&st(G,H,e,N,H.calendar),Y("fixedrange");for(g=0;g<N.length;g++){U=N[g],G=t[U],H=e[U];var At=e[h(H.anchor)];Y("fixedrange",I("rangeslider","isVisible")(At))}var lt=e._axisConstraintGroups=[],ct=e._axisMatchGroups=[];for(g=0;g<F.length;g++)O=(U=F[g]).charAt(0),G=t[U],H=e[U],c(G,H,Y,J,e);for(g=0;g<ct.length;g++){var ut,gt=ct[g],ht=null,dt=null;for(ut in gt)(H=e[h(ut)]).matches||(ht=H.range,dt=H.autorange);if(null===ht||null===dt)for(ut in gt){ht=(H=e[h(ut)]).range,dt=H.autorange;break}for(ut in gt)(H=e[h(ut)]).matches&&(H.range=ht.slice(),H.autorange=dt),H._matchGroup=gt;if(lt.length)for(ut in gt)for(f=0;f<lt.length;f++){var ft=lt[f];for(var pt in ft)ut===pt&&(r.warn("Axis "+pt+" is set with both a *scaleanchor* and *matches* constraint; ignoring the scale constraint."),delete ft[pt],Object.keys(ft).length<2&&lt.splice(f,1))}}}},{"../../components/color":593,"../../lib":719,"../../plot_api/plot_template":757,"../../registry":847,"../layout_attributes":819,"./axis_defaults":769,"./axis_ids":770,"./constraints":774,"./layout_attributes":779,"./position_defaults":782,"./type_defaults":790}],781:[function(t,e,n){"use strict";var r=t("tinycolor2").mix,i=t("../../components/color/attributes").lightFraction,o=t("../../lib");e.exports=function(t,e,n,a){var s=(a=a||{}).dfltColor;function A(n,r){return o.coerce2(t,e,a.attributes,n,r)}var l=A("linecolor",s),c=A("linewidth");n("showline",a.showLine||!!l||!!c)||(delete e.linecolor,delete e.linewidth);var u=A("gridcolor",r(s,a.bgColor,a.blend||i).toRgbString()),g=A("gridwidth");if(n("showgrid",a.showGrid||!!u||!!g)||(delete e.gridcolor,delete e.gridwidth),!a.noZeroLine){var h=A("zerolinecolor",s),d=A("zerolinewidth");n("zeroline",a.showGrid||!!h||!!d)||(delete e.zerolinecolor,delete e.zerolinewidth)}}},{"../../components/color/attributes":592,"../../lib":719,tinycolor2:537}],782:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib");e.exports=function(t,e,n,o){var a,s,A,l,c=o.counterAxes||[],u=o.overlayableAxes||[],g=o.letter,h=o.grid;h&&(s=h._domains[g][h._axisMap[e._id]],a=h._anchors[e._id],s&&(A=h[g+"side"].split(" ")[0],l=h.domain[g]["right"===A||"top"===A?1:0])),s=s||[0,1],a=a||(r(t.position)?"free":c[0]||"free"),A=A||("x"===g?"bottom":"left"),l=l||0,"free"===i.coerce(t,e,{anchor:{valType:"enumerated",values:["free"].concat(c),dflt:a}},"anchor")&&n("position",l),i.coerce(t,e,{side:{valType:"enumerated",values:"x"===g?["bottom","top"]:["left","right"],dflt:A}},"side");var d=!1;if(u.length&&(d=i.coerce(t,e,{overlaying:{valType:"enumerated",values:[!1].concat(u),dflt:!1}},"overlaying")),!d){var f=n("domain",s);f[0]>f[1]-1/4096&&(e.domain=s),i.noneOrAll(t.domain,e.domain,s)}return n("layer"),e}},{"../../lib":719,"fast-isnumeric":225}],783:[function(t,e,n){"use strict";var r=t("../../constants/alignment").FROM_BL;e.exports=function(t,e,n){void 0===n&&(n=r[t.constraintoward||"center"]);var i=[t.r2l(t.range[0]),t.r2l(t.range[1])],o=i[0]+(i[1]-i[0])*n;t.range=t._input.range=[t.l2r(o+(i[0]-o)*e),t.l2r(o+(i[1]-o)*e)]}},{"../../constants/alignment":688}],784:[function(t,e,n){"use strict";var r=t("polybooljs"),i=t("../../registry"),o=t("../../components/color"),a=t("../../components/fx"),s=t("../../lib"),A=t("../../lib/polygon"),l=t("../../lib/throttle"),c=t("../../components/fx/helpers").makeEventData,u=t("./axis_ids").getFromId,g=t("../../lib/clear_gl_canvases"),h=t("../../plot_api/subroutines").redrawReglTraces,d=t("./constants"),f=d.MINSELECT,p=A.filter,I=A.tester;function C(t){return t._id}function m(t,e,n,r,i,o,a){var s,A,l,c,u,g,h,d,f,p=e._hoverdata,I=e._fullLayout.clickmode.indexOf("event")>-1,C=[];if(function(t){return t&&Array.isArray(t)&&!0!==t[0].hoverOnBox}(p)){B(t,e,o);var m=function(t,e){var n,r,i=t[0],o=-1,a=[];for(r=0;r<e.length;r++)if(n=e[r],i.fullData._expandedIndex===n.cd[0].trace._expandedIndex){if(!0===i.hoverOnBox)break;void 0!==i.pointNumber?o=i.pointNumber:void 0!==i.binNumber&&(o=i.binNumber,a=i.pointNumbers);break}return{pointNumber:o,pointNumbers:a,searchInfo:n}}(p,s=Q(e,n,r,i));if(m.pointNumbers.length>0?function(t,e){var n,r,i,o=[];for(i=0;i<t.length;i++)(n=t[i]).cd[0].trace.selectedpoints&&n.cd[0].trace.selectedpoints.length>0&&o.push(n);if(1===o.length&&o[0]===e.searchInfo&&(r=e.searchInfo.cd[0].trace).selectedpoints.length===e.pointNumbers.length){for(i=0;i<e.pointNumbers.length;i++)if(r.selectedpoints.indexOf(e.pointNumbers[i])<0)return!1;return!0}return!1}(s,m):function(t){var e,n,r,i=0;for(r=0;r<t.length;r++)if(e=t[r],(n=e.cd[0].trace).selectedpoints){if(n.selectedpoints.length>1)return!1;if((i+=n.selectedpoints.length)>1)return!1}return 1===i}(s)&&(g=x(m))){for(a&&a.remove(),f=0;f<s.length;f++)(A=s[f])._module.selectPoints(A,!1);_(e,s),b(o),I&&e.emit("plotly_deselect",null)}else{for(h=t.shiftKey&&(void 0!==g?g:x(m)),l=function(t,e,n){return{pointNumber:t,searchInfo:e,subtract:n}}(m.pointNumber,m.searchInfo,h),c=v(o.selectionDefs.concat([l])),f=0;f<s.length;f++)if(u=D(s[f]._module.selectPoints(s[f],c),s[f]),C.length)for(var y=0;y<u.length;y++)C.push(u[y]);else C=u;_(e,s,d={points:C}),l&&o&&o.selectionDefs.push(l),a&&w(o.mergedPolygons,a),I&&e.emit("plotly_selected",d)}}}function y(t){return"pointNumber"in t&&"searchInfo"in t}function E(t){return{xmin:0,xmax:0,ymin:0,ymax:0,pts:[],contains:function(e,n,r,i){var o=t.searchInfo.cd[0].trace._expandedIndex;return i.cd[0].trace._expandedIndex===o&&r===t.pointNumber},isRect:!1,degenerate:!1,subtract:t.subtract}}function v(t){for(var e=[],n=y(t[0])?0:t[0][0][0],r=n,i=y(t[0])?0:t[0][0][1],o=i,a=0;a<t.length;a++)if(y(t[a]))e.push(E(t[a]));else{var s=A.tester(t[a]);s.subtract=t[a].subtract,e.push(s),n=Math.min(n,s.xmin),r=Math.max(r,s.xmax),i=Math.min(i,s.ymin),o=Math.max(o,s.ymax)}return{xmin:n,xmax:r,ymin:i,ymax:o,pts:[],contains:function(t,n,r,i){for(var o=!1,a=0;a<e.length;a++)e[a].contains(t,n,r,i)&&(o=!1===e[a].subtract);return o},isRect:!1,degenerate:!1}}function B(t,e,n){var r=e._fullLayout,i=n.plotinfo,o=r._lastSelectedSubplot&&r._lastSelectedSubplot===i.id,a=t.shiftKey||t.altKey;o&&a&&i.selection&&i.selection.selectionDefs&&!n.selectionDefs?(n.selectionDefs=i.selection.selectionDefs,n.mergedPolygons=i.selection.mergedPolygons):a&&i.selection||b(n),o||(M(e),r._lastSelectedSubplot=i.id)}function b(t){var e=t.plotinfo;e.selection={},e.selection.selectionDefs=t.selectionDefs=[],e.selection.mergedPolygons=t.mergedPolygons=[]}function Q(t,e,n,r){var i,o,a,s=[],A=e.map(C),l=n.map(C);for(a=0;a<t.calcdata.length;a++)if(!0===(o=(i=t.calcdata[a])[0].trace).visible&&o._module&&o._module.selectPoints)if(!r||o.subplot!==r&&o.geo!==r)if("splom"===o.type&&o._xaxes[A[0]]&&o._yaxes[l[0]]){var c=h(o._module,i,e[0],n[0]);c.scene=t._fullLayout._splomScenes[o.uid],s.push(c)}else if("sankey"===o.type){var g=h(o._module,i,e[0],n[0]);s.push(g)}else{if(-1===A.indexOf(o.xaxis))continue;if(-1===l.indexOf(o.yaxis))continue;s.push(h(o._module,i,u(t,o.xaxis),u(t,o.yaxis)))}else s.push(h(o._module,i,e[0],n[0]));return s;function h(t,e,n,r){return{_module:t,cd:e,xaxis:n,yaxis:r}}}function w(t,e){var n,r,i=[];for(n=0;n<t.length;n++){var o=t[n];i.push(o.join("L")+"L"+o[0])}r=t.length>0?"M"+i.join("M")+"Z":"M0,0Z",e.attr("d",r)}function x(t){var e=t.searchInfo.cd[0].trace,n=t.pointNumber,r=t.pointNumbers,i=r.length>0?r[0]:n;return!!e.selectedpoints&&e.selectedpoints.indexOf(i)>-1}function _(t,e,n){var r,o,a,s;for(r=0;r<e.length;r++){var A=e[r].cd[0].trace._fullInput,l=t._fullLayout._tracePreGUI[A.uid]||{};void 0===l.selectedpoints&&(l.selectedpoints=A._input.selectedpoints||null)}if(n){var c=n.points||[];for(r=0;r<e.length;r++)(s=e[r].cd[0].trace)._input.selectedpoints=s._fullInput.selectedpoints=[],s._fullInput!==s&&(s.selectedpoints=[]);for(r=0;r<c.length;r++){var u=c[r],d=u.data,f=u.fullData;u.pointIndices?([].push.apply(d.selectedpoints,u.pointIndices),s._fullInput!==s&&[].push.apply(f.selectedpoints,u.pointIndices)):(d.selectedpoints.push(u.pointIndex),s._fullInput!==s&&f.selectedpoints.push(u.pointIndex))}}else for(r=0;r<e.length;r++)delete(s=e[r].cd[0].trace).selectedpoints,delete s._input.selectedpoints,s._fullInput!==s&&delete s._fullInput.selectedpoints;var p=!1;for(r=0;r<e.length;r++){s=(a=(o=e[r]).cd)[0].trace,i.traceIs(s,"regl")&&(p=!0);var I=o._module,C=I.styleOnSelect||I.style;C&&(C(t,a,a[0].node3),a[0].nodeRangePlot3&&C(t,a,a[0].nodeRangePlot3))}p&&(g(t),h(t))}function D(t,e){if(Array.isArray(t))for(var n=e.cd,r=e.cd[0].trace,i=0;i<t.length;i++)t[i]=c(t[i],r,n);return t}function M(t){var e=(t._fullLayout||{})._zoomlayer;e&&e.selectAll(".select-outline").remove()}e.exports={prepSelect:function(t,e,n,i,A){var c,u,g,h,C,y,E,x=i.gd,M=x._fullLayout,k=M._zoomlayer,S=i.element.getBoundingClientRect(),L=i.plotinfo,R=L.xaxis._offset,N=L.yaxis._offset,F=e-S.left,T=n-S.top,U=F,O=T,G="M"+F+","+T,H=i.xaxes[0]._length,j=i.yaxes[0]._length,Y=i.xaxes.concat(i.yaxes),P=t.altKey;B(t,x,i),"lasso"===A&&(c=p([[F,T]],d.BENDPX));var K=k.selectAll("path.select-outline-"+L.id).data([1,2]);K.enter().append("path").attr("class",function(t){return"select-outline select-outline-"+t+" select-outline-"+L.id}).attr("transform","translate("+R+", "+N+")").attr("d",G+"Z");var z,J=k.append("path").attr("class","zoombox-corners").style({fill:o.background,stroke:o.defaultLine,"stroke-width":1}).attr("transform","translate("+R+", "+N+")").attr("d","M0,0Z"),q=M._uid+d.SELECTID,V=[],W=Q(x,i.xaxes,i.yaxes,i.subplot);function X(t,e){return"log"===t.type?t.p2d(e):t.p2r(e)}function Z(t){var e="y"===t._id.charAt(0)?1:0;return function(n){return X(t,n[e])}}function $(t,e){return t-e}z=L.fillRangeItems?L.fillRangeItems:"select"===A?function(t,e){var n=t.range={};for(C=0;C<Y.length;C++){var r=Y[C],i=r._id.charAt(0);n[r._id]=[X(r,e[i+"min"]),X(r,e[i+"max"])].sort($)}}:function(t,e,n){var r=t.lassoPoints={};for(C=0;C<Y.length;C++){var i=Y[C];r[i._id]=n.filtered.map(Z(i))}},i.moveFn=function(t,e){U=Math.max(0,Math.min(H,t+F)),O=Math.max(0,Math.min(j,e+T));var n=Math.abs(U-F),o=Math.abs(O-T);if("select"===A){var a=M.selectdirection;"h"===(a="any"===M.selectdirection?o<Math.min(.6*n,f)?"h":n<Math.min(.6*o,f)?"v":"d":M.selectdirection)?((h=[[F,0],[F,j],[U,j],[U,0]]).xmin=Math.min(F,U),h.xmax=Math.max(F,U),h.ymin=Math.min(0,j),h.ymax=Math.max(0,j),J.attr("d","M"+h.xmin+","+(T-f)+"h-4v"+2*f+"h4ZM"+(h.xmax-1)+","+(T-f)+"h4v"+2*f+"h-4Z")):"v"===a?((h=[[0,T],[0,O],[H,O],[H,T]]).xmin=Math.min(0,H),h.xmax=Math.max(0,H),h.ymin=Math.min(T,O),h.ymax=Math.max(T,O),J.attr("d","M"+(F-f)+","+h.ymin+"v-4h"+2*f+"v4ZM"+(F-f)+","+(h.ymax-1)+"v4h"+2*f+"v-4Z")):"d"===a&&((h=[[F,T],[F,O],[U,O],[U,T]]).xmin=Math.min(F,U),h.xmax=Math.max(F,U),h.ymin=Math.min(T,O),h.ymax=Math.max(T,O),J.attr("d","M0,0Z"))}else"lasso"===A&&(c.addPt([U,O]),h=c.filtered);i.selectionDefs&&i.selectionDefs.length?(g=function(t,e,n){return r.union({regions:t,inverted:!1},{regions:[e],inverted:!1}).regions}(i.mergedPolygons,h),h.subtract=P,u=v(i.selectionDefs.concat([h]))):(g=[h],u=I(h)),w(g,K),l.throttle(q,d.SELECTDELAY,function(){var t;V=[];var e,n=[];for(C=0;C<W.length;C++)if(e=(y=W[C])._module.selectPoints(y,u),n.push(e),t=D(e,y),V.length)for(var r=0;r<t.length;r++)V.push(t[r]);else V=t;_(x,W,E={points:V}),z(E,h,c),i.gd.emit("plotly_selecting",E)})},i.clickFn=function(t,e){var n=M.clickmode;J.remove(),l.done(q).then(function(){if(l.clear(q),2===t){for(K.remove(),C=0;C<W.length;C++)(y=W[C])._module.selectPoints(y,!1);_(x,W),b(i),x.emit("plotly_deselect",null)}else n.indexOf("select")>-1&&m(e,x,i.xaxes,i.yaxes,i.subplot,i,K),"event"===n&&x.emit("plotly_selected",void 0);a.click(x,e)}).catch(s.error)},i.doneFn=function(){J.remove(),l.done(q).then(function(){l.clear(q),i.gd.emit("plotly_selected",E),h&&i.selectionDefs&&(h.subtract=P,i.selectionDefs.push(h),i.mergedPolygons.length=0,[].push.apply(i.mergedPolygons,g)),i.doneFnCompleted&&i.doneFnCompleted(V)}).catch(s.error)}},clearSelect:M,selectOnClick:m}},{"../../components/color":593,"../../components/fx":632,"../../components/fx/helpers":628,"../../lib":719,"../../lib/clear_gl_canvases":704,"../../lib/polygon":731,"../../lib/throttle":744,"../../plot_api/subroutines":758,"../../registry":847,"./axis_ids":770,"./constants":773,polybooljs:473}],785:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=t("../../lib"),a=o.cleanNumber,s=o.ms2DateTime,A=o.dateTime2ms,l=o.ensureNumber,c=o.isArrayOrTypedArray,u=t("../../constants/numerical"),g=u.FP_SAFE,h=u.BADNUM,d=u.LOG_CLIP,f=t("./constants"),p=t("./axis_ids");function I(t){return Math.pow(10,t)}function C(t){return null!=t}e.exports=function(t,e){e=e||{};var n=t._id||"x",u=n.charAt(0);function m(e,n){if(e>0)return Math.log(e)/Math.LN10;if(e<=0&&n&&t.range&&2===t.range.length){var r=t.range[0],i=t.range[1];return.5*(r+i-2*d*Math.abs(r-i))}return h}function y(e,n,r){var a=A(e,r||t.calendar);if(a===h){if(!i(e))return h;e=+e;var s=Math.floor(10*o.mod(e+.05,1)),l=Math.round(e-s/10);a=A(new Date(l))+s/10}return a}function E(e,n,r){return s(e,n,r||t.calendar)}function v(e){return t._categories[Math.round(e)]}function B(e){if(C(e)){if(void 0===t._categoriesMap&&(t._categoriesMap={}),void 0!==t._categoriesMap[e])return t._categoriesMap[e];t._categories.push("number"==typeof e?String(e):e);var n=t._categories.length-1;return t._categoriesMap[e]=n,n}return h}function b(e){if(t._categoriesMap)return t._categoriesMap[e]}function Q(t){var e=b(t);return void 0!==e?e:i(t)?+t:void 0}function w(e){return i(e)?r.round(t._b+t._m*e,2):h}function x(e){return(e-t._b)/t._m}t.c2l="log"===t.type?m:l,t.l2c="log"===t.type?I:l,t.l2p=w,t.p2l=x,t.c2p="log"===t.type?function(t,e){return w(m(t,e))}:w,t.p2c="log"===t.type?function(t){return I(x(t))}:x,-1!==["linear","-"].indexOf(t.type)?(t.d2r=t.r2d=t.d2c=t.r2c=t.d2l=t.r2l=a,t.c2d=t.c2r=t.l2d=t.l2r=l,t.d2p=t.r2p=function(e){return t.l2p(a(e))},t.p2d=t.p2r=x,t.cleanPos=l):"log"===t.type?(t.d2r=t.d2l=function(t,e){return m(a(t),e)},t.r2d=t.r2c=function(t){return I(a(t))},t.d2c=t.r2l=a,t.c2d=t.l2r=l,t.c2r=m,t.l2d=I,t.d2p=function(e,n){return t.l2p(t.d2r(e,n))},t.p2d=function(t){return I(x(t))},t.r2p=function(e){return t.l2p(a(e))},t.p2r=x,t.cleanPos=l):"date"===t.type?(t.d2r=t.r2d=o.identity,t.d2c=t.r2c=t.d2l=t.r2l=y,t.c2d=t.c2r=t.l2d=t.l2r=E,t.d2p=t.r2p=function(e,n,r){return t.l2p(y(e,0,r))},t.p2d=t.p2r=function(t,e,n){return E(x(t),e,n)},t.cleanPos=function(e){return o.cleanDate(e,h,t.calendar)}):"category"===t.type?(t.d2c=t.d2l=B,t.r2d=t.c2d=t.l2d=v,t.d2r=t.d2l_noadd=Q,t.r2c=function(e){var n=Q(e);return void 0!==n?n:t.fraction2r(.5)},t.l2r=t.c2r=l,t.r2l=Q,t.d2p=function(e){return t.l2p(t.r2c(e))},t.p2d=function(t){return v(x(t))},t.r2p=t.d2p,t.p2r=x,t.cleanPos=function(t){return"string"==typeof t&&""!==t?t:l(t)}):"multicategory"===t.type&&(t.r2d=t.c2d=t.l2d=v,t.d2r=t.d2l_noadd=Q,t.r2c=function(e){var n=Q(e);return void 0!==n?n:t.fraction2r(.5)},t.r2c_just_indices=b,t.l2r=t.c2r=l,t.r2l=Q,t.d2p=function(e){return t.l2p(t.r2c(e))},t.p2d=function(t){return v(x(t))},t.r2p=t.d2p,t.p2r=x,t.cleanPos=function(t){return Array.isArray(t)||"string"==typeof t&&""!==t?t:l(t)},t.setupMultiCategory=function(r){var i,a,s=t._traceIndices,A=e._axisMatchGroups;if(A&&A.length&&0===t._categories.length)for(i=0;i<A.length;i++){var l=A[i];if(l[n])for(var g in l)if(g!==n){var h=e[p.id2name(g)];s=s.concat(h._traceIndices)}}var d=[[0,{}],[0,{}]],f=[];for(i=0;i<s.length;i++){var I=r[s[i]];if(u in I){var m=I[u],y=I._length||o.minRowLength(m);if(c(m[0])&&c(m[1]))for(a=0;a<y;a++){var E=m[0][a],v=m[1][a];C(E)&&C(v)&&(f.push([E,v]),E in d[0][1]||(d[0][1][E]=d[0][0]++),v in d[1][1]||(d[1][1][v]=d[1][0]++))}}}for(f.sort(function(t,e){var n=d[0][1],r=n[t[0]]-n[e[0]];if(r)return r;var i=d[1][1];return i[t[1]]-i[e[1]]}),i=0;i<f.length;i++)B(f[i])}),t.fraction2r=function(e){var n=t.r2l(t.range[0]),r=t.r2l(t.range[1]);return t.l2r(n+e*(r-n))},t.r2fraction=function(e){var n=t.r2l(t.range[0]),r=t.r2l(t.range[1]);return(t.r2l(e)-n)/(r-n)},t.cleanRange=function(e,n){n||(n={}),e||(e="range");var r,a,s=o.nestedProperty(t,e).get();if(a=(a="date"===t.type?o.dfltRange(t.calendar):"y"===u?f.DFLTRANGEY:n.dfltRange||f.DFLTRANGEX).slice(),s&&2===s.length)for("date"!==t.type||t.autorange||(s[0]=o.cleanDate(s[0],h,t.calendar),s[1]=o.cleanDate(s[1],h,t.calendar)),r=0;r<2;r++)if("date"===t.type){if(!o.isDateTime(s[r],t.calendar)){t[e]=a;break}if(t.r2l(s[0])===t.r2l(s[1])){var A=o.constrain(t.r2l(s[0]),o.MIN_MS+1e3,o.MAX_MS-1e3);s[0]=t.l2r(A-1e3),s[1]=t.l2r(A+1e3);break}}else{if(!i(s[r])){if(!i(s[1-r])){t[e]=a;break}s[r]=s[1-r]*(r?10:.1)}if(s[r]<-g?s[r]=-g:s[r]>g&&(s[r]=g),s[0]===s[1]){var l=Math.max(1,Math.abs(1e-6*s[0]));s[0]-=l,s[1]+=l}}else o.nestedProperty(t,e).set(a)},t.setScale=function(n){var r=e._size;if(t.overlaying){var i=p.getFromId({_fullLayout:e},t.overlaying);t.domain=i.domain}var o=n&&t._r?"_r":"range",a=t.calendar;t.cleanRange(o);var s=t.r2l(t[o][0],a),A=t.r2l(t[o][1],a);if("y"===u?(t._offset=r.t+(1-t.domain[1])*r.h,t._length=r.h*(t.domain[1]-t.domain[0]),t._m=t._length/(s-A),t._b=-t._m*A):(t._offset=r.l+t.domain[0]*r.w,t._length=r.w*(t.domain[1]-t.domain[0]),t._m=t._length/(A-s),t._b=-t._m*s),!isFinite(t._m)||!isFinite(t._b)||t._length<0)throw e._replotting=!1,new Error("Something went wrong with axis scaling")},t.makeCalcdata=function(e,n){var r,i,a,s,A=t.type,l="date"===A&&e[n+"calendar"];if(n in e){if(r=e[n],s=e._length||o.minRowLength(r),o.isTypedArray(r)&&("linear"===A||"log"===A)){if(s===r.length)return r;if(r.subarray)return r.subarray(0,s)}if("multicategory"===A)return function(t,e){for(var n=new Array(e),r=0;r<e;r++){var i=(t[0]||[])[r],o=(t[1]||[])[r];n[r]=b([i,o])}return n}(r,s);for(i=new Array(s),a=0;a<s;a++)i[a]=t.d2c(r[a],0,l)}else{var c=n+"0"in e?t.d2c(e[n+"0"],0,l):0,u=e["d"+n]?Number(e["d"+n]):1;for(r=e[{x:"y",y:"x"}[n]],s=e._length||r.length,i=new Array(s),a=0;a<s;a++)i[a]=c+a*u}return i},t.isValidRange=function(e){return Array.isArray(e)&&2===e.length&&i(t.r2l(e[0]))&&i(t.r2l(e[1]))},t.isPtWithinRange=function(e,n){var r=t.c2l(e[u],null,n),i=t.r2l(t.range[0]),o=t.r2l(t.range[1]);return i<o?i<=r&&r<=o:o<=r&&r<=i},t.clearCalc=function(){var r=function(){t._categories=[],t._categoriesMap={}},i=e._axisMatchGroups;if(i&&i.length){for(var o=!1,a=0;a<i.length;a++){var s=i[a];if(s[n]){o=!0;var A=null,l=null;for(var c in s){var u=e[p.id2name(c)];if(u._categories){A=u._categories,l=u._categoriesMap;break}}A&&l?(t._categories=A,t._categoriesMap=l):r();break}}o||r()}else r();if(t._initialCategories)for(var g=0;g<t._initialCategories.length;g++)B(t._initialCategories[g])},t.sortByInitialCategories=function(){var r=[];if(t._categories=[],t._categoriesMap={},t._initialCategories)for(var i=0;i<t._initialCategories.length;i++)B(t._initialCategories[i]);r=r.concat(t._traceIndices);var o=t._matchGroup;for(var a in o)if(n!==a){var s=e[p.id2name(a)];s._categories=t._categories,s._categoriesMap=t._categoriesMap,r=r.concat(s._traceIndices)}return r};var _=e._d3locale;"date"===t.type&&(t._dateFormat=_?_.timeFormat.utc:r.time.format.utc,t._extraFormat=e._extraFormat),t._separators=e.separators,t._numFormat=_?_.numberFormat:r.format,delete t._minDtick,delete t._forceTick0}},{"../../constants/numerical":695,"../../lib":719,"./axis_ids":770,"./constants":773,d3:163,"fast-isnumeric":225}],786:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./layout_attributes"),o=t("../array_container_defaults");function a(t){var e=["showexponent","showtickprefix","showticksuffix"].filter(function(e){return void 0!==t[e]});if(e.every(function(n){return t[n]===t[e[0]]})||1===e.length)return t[e[0]]}function s(t,e){function n(n,o){return r.coerce(t,e,i.tickformatstops,n,o)}n("enabled")&&(n("dtickrange"),n("value"))}e.exports=function(t,e,n,A,l,c){c&&1!==c.pass||function(t,e,n,r,i){var o=a(t);n("tickprefix")&&n("showtickprefix",o),n("ticksuffix",i.tickSuffixDflt)&&n("showticksuffix",o)}(t,0,n,0,l),c&&2!==c.pass||function(t,e,n,A,l){var c=a(t);if(n("tickprefix")&&n("showtickprefix",c),n("ticksuffix",l.tickSuffixDflt)&&n("showticksuffix",c),n("showticklabels")){var u=l.font||{},g=e.color,h=g&&g!==i.color.dflt?g:u.color;if(r.coerceFont(n,"tickfont",{family:u.family,size:u.size,color:h}),n("tickangle"),"category"!==A){var d=n("tickformat"),f=t.tickformatstops;Array.isArray(f)&&f.length&&o(t,e,{name:"tickformatstops",inclusionAttr:"enabled",handleItemDefaults:s}),d||"date"===A||(n("showexponent",c),n("exponentformat"),n("separatethousands"))}}}(t,e,n,A,l)}},{"../../lib":719,"../array_container_defaults":763,"./layout_attributes":779}],787:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./layout_attributes");e.exports=function(t,e,n,o){var a=r.coerce2(t,e,i,"ticklen"),s=r.coerce2(t,e,i,"tickwidth"),A=r.coerce2(t,e,i,"tickcolor",e.color);n("ticks",o.outerTicks||a||s||A?"outside":"")||(delete e.ticklen,delete e.tickwidth,delete e.tickcolor)}},{"../../lib":719,"./layout_attributes":779}],788:[function(t,e,n){"use strict";var r=t("./clean_ticks");e.exports=function(t,e,n,i){var o;if("auto"===(o="array"!==t.tickmode||"log"!==i&&"date"!==i?n("tickmode",Array.isArray(t.tickvals)?"array":t.dtick?"linear":"auto"):e.tickmode="auto"))n("nticks");else if("linear"===o){var a=e.dtick=r.dtick(t.dtick,i);e.tick0=r.tick0(t.tick0,i,e.calendar,a)}else"multicategory"!==i&&(void 0===n("tickvals")?e.tickmode="auto":n("ticktext"))}},{"./clean_ticks":772}],789:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../registry"),o=t("../../components/drawing"),a=t("./axes");e.exports=function(t,e,n,s){var A=t._fullLayout;if(0!==e.length){var l,c,u,g;s&&(l=s());var h=r.ease(n.easing);return t._transitionData._interruptCallbacks.push(function(){return window.cancelAnimationFrame(g),g=null,function(){for(var n={},r=0;r<e.length;r++){var o=e[r];o.xr0&&(n[o.plotinfo.xaxis._name+".range"]=o.xr0.slice()),o.yr0&&(n[o.plotinfo.yaxis._name+".range"]=o.yr0.slice())}return i.call("relayout",t,n).then(function(){for(var t=0;t<e.length;t++)d(e[t].plotinfo)})}()}),c=Date.now(),g=window.requestAnimationFrame(function r(){u=Date.now();for(var o=Math.min(1,(u-c)/n.duration),a=h(o),s=0;s<e.length;s++)f(e[s],a);u-c>n.duration?(function(){for(var n={},r=0;r<e.length;r++){var o=e[r];o.xr1&&(n[o.plotinfo.xaxis._name+".range"]=o.xr1.slice()),o.yr1&&(n[o.plotinfo.yaxis._name+".range"]=o.yr1.slice())}l&&l(),i.call("relayout",t,n).then(function(){for(var t=0;t<e.length;t++)d(e[t].plotinfo)})}(),g=window.cancelAnimationFrame(r)):g=window.requestAnimationFrame(r)}),Promise.resolve()}function d(t){var e=t.xaxis,n=t.yaxis;A._defs.select("#"+t.clipId+"> rect").call(o.setTranslate,0,0).call(o.setScale,1,1),t.plot.call(o.setTranslate,e._offset,n._offset).call(o.setScale,1,1);var r=t.plot.selectAll(".scatterlayer .trace");r.selectAll(".point").call(o.setPointGroupScale,1,1),r.selectAll(".textpoint").call(o.setTextPointsScale,1,1),r.call(o.hideOutsideRangePoints,t)}function f(e,n){var r=e.plotinfo,i=r.xaxis,s=r.yaxis,A=e.xr0,l=e.xr1,c=i._length,u=e.yr0,g=e.yr1,h=s._length,d=!!l,f=!!g,p=[];if(d){var I=A[1]-A[0],C=l[1]-l[0];p[0]=(A[0]*(1-n)+n*l[0]-A[0])/(A[1]-A[0])*c,p[2]=c*(1-n+n*C/I),i.range[0]=A[0]*(1-n)+n*l[0],i.range[1]=A[1]*(1-n)+n*l[1]}else p[0]=0,p[2]=c;if(f){var m=u[1]-u[0],y=g[1]-g[0];p[1]=(u[1]*(1-n)+n*g[1]-u[1])/(u[0]-u[1])*h,p[3]=h*(1-n+n*y/m),s.range[0]=u[0]*(1-n)+n*g[0],s.range[1]=u[1]*(1-n)+n*g[1]}else p[1]=0,p[3]=h;a.drawOne(t,i,{skipTitle:!0}),a.drawOne(t,s,{skipTitle:!0}),a.redrawComponents(t,[i._id,s._id]);var E=d?c/p[2]:1,v=f?h/p[3]:1,B=d?p[0]:0,b=f?p[1]:0,Q=d?p[0]/p[2]*c:0,w=f?p[1]/p[3]*h:0,x=i._offset-Q,_=s._offset-w;r.clipRect.call(o.setTranslate,B,b).call(o.setScale,1/E,1/v),r.plot.call(o.setTranslate,x,_).call(o.setScale,E,v),o.setPointGroupScale(r.zoomScalePts,1/E,1/v),o.setTextPointsScale(r.zoomScaleTxt,1/E,1/v)}a.redrawComponents(t)}},{"../../components/drawing":614,"../../registry":847,"./axes":767,d3:163}],790:[function(t,e,n){"use strict";var r=t("../../registry").traceIs,i=t("./axis_autotype");function o(t){return{v:"x",h:"y"}[t.orientation||"v"]}function a(t,e){var n=o(t),i=r(t,"box-violin"),a=r(t._fullInput||{},"candlestick");return i&&!a&&e===n&&void 0===t[n]&&void 0===t[n+"0"]}e.exports=function(t,e,n,s){"-"===n("type",(s.splomStash||{}).type)&&(function(t,e){if("-"===t.type){var n=t._id,s=n.charAt(0);-1!==n.indexOf("scene")&&(n=s);var A=function(t,e,n){for(var r=0;r<t.length;r++){var i=t[r];if("splom"===i.type&&i._length>0&&(i["_"+n+"axes"]||{})[e])return i;if((i[n+"axis"]||n)===e){if(a(i,n))return i;if((i[n]||[]).length||i[n+"0"])return i}}}(e,n,s);if(A)if("histogram"!==A.type||s!=={v:"y",h:"x"}[A.orientation||"v"]){var l,c=s+"calendar",u=A[c],g={noMultiCategory:!r(A,"cartesian")||r(A,"noMultiCategory")};if(a(A,s)){var h=o(A),d=[];for(l=0;l<e.length;l++){var f=e[l];r(f,"box-violin")&&(f[s+"axis"]||s)===n&&(void 0!==f[h]?d.push(f[h][0]):void 0!==f.name?d.push(f.name):d.push("text"),f[c]!==u&&(u=void 0))}t.type=i(d,u,g)}else if("splom"===A.type){var p=A.dimensions,I=p[A._axesDim[n]];I.visible&&(t.type=i(I.values,u,g))}else t.type=i(A[s]||[A[s+"0"]],u,g)}else t.type="linear"}}(e,s.data),"-"===e.type?e.type="linear":t.type=e.type)}},{"../../registry":847,"./axis_autotype":768}],791:[function(t,e,n){"use strict";var r=t("../registry"),i=t("../lib");function o(t,e,n){var r,o,a,s=!1;if("data"===e.type)r=t._fullData[null!==e.traces?e.traces[0]:0];else{if("layout"!==e.type)return!1;r=t._fullLayout}return o=i.nestedProperty(r,e.prop).get(),(a=n[e.type]=n[e.type]||{}).hasOwnProperty(e.prop)&&a[e.prop]!==o&&(s=!0),a[e.prop]=o,{changed:s,value:o}}function a(t,e){var n=[],r=e[0],o={};if("string"==typeof r)o[r]=e[1];else{if(!i.isPlainObject(r))return n;o=r}return A(o,function(t,e,r){n.push({type:"layout",prop:t,value:r})},"",0),n}function s(t,e){var n,r,o,a,s=[];if(r=e[0],o=e[1],n=e[2],a={},"string"==typeof r)a[r]=o;else{if(!i.isPlainObject(r))return s;a=r,void 0===n&&(n=o)}return void 0===n&&(n=null),A(a,function(e,r,i){var o,a;if(Array.isArray(i)){a=i.slice();var A=Math.min(a.length,t.data.length);n&&(A=Math.min(A,n.length)),o=[];for(var l=0;l<A;l++)o[l]=n?n[l]:l}else a=i,o=n?n.slice():null;if(null===o)Array.isArray(a)&&(a=a[0]);else if(Array.isArray(o)){if(!Array.isArray(a)){var c=a;a=[];for(var u=0;u<o.length;u++)a[u]=c}a.length=Math.min(o.length,a.length)}s.push({type:"data",prop:e,traces:o,value:a})},"",0),s}function A(t,e,n,r){Object.keys(t).forEach(function(o){var a=t[o];if("_"!==o[0]){var s=n+(r>0?".":"")+o;i.isPlainObject(a)?A(a,e,s,r+1):e(s,o,a)}})}n.manageCommandObserver=function(t,e,r,a){var s={},A=!0;e&&e._commandObserver&&(s=e._commandObserver),s.cache||(s.cache={}),s.lookupTable={};var l=n.hasSimpleAPICommandBindings(t,r,s.lookupTable);if(e&&e._commandObserver){if(l)return s;if(e._commandObserver.remove)return e._commandObserver.remove(),e._commandObserver=null,s}if(l){o(t,l,s.cache),s.check=function(){if(A){var e=o(t,l,s.cache);return e.changed&&a&&void 0!==s.lookupTable[e.value]&&(s.disable(),Promise.resolve(a({value:e.value,type:l.type,prop:l.prop,traces:l.traces,index:s.lookupTable[e.value]})).then(s.enable,s.enable)),e.changed}};for(var c=["plotly_relayout","plotly_redraw","plotly_restyle","plotly_update","plotly_animatingframe","plotly_afterplot"],u=0;u<c.length;u++)t._internalOn(c[u],s.check);s.remove=function(){for(var e=0;e<c.length;e++)t._removeInternalListener(c[e],s.check)}}else i.log("Unable to automatically bind plot updates to API command"),s.lookupTable={},s.remove=function(){};return s.disable=function(){A=!1},s.enable=function(){A=!0},e&&(e._commandObserver=s),s},n.hasSimpleAPICommandBindings=function(t,e,r){var i,o,a=e.length;for(i=0;i<a;i++){var s,A=e[i],l=A.method,c=A.args;if(Array.isArray(c)||(c=[]),!l)return!1;var u=n.computeAPICommandBindings(t,l,c);if(1!==u.length)return!1;if(o){if((s=u[0]).type!==o.type)return!1;if(s.prop!==o.prop)return!1;if(Array.isArray(o.traces)){if(!Array.isArray(s.traces))return!1;s.traces.sort();for(var g=0;g<o.traces.length;g++)if(o.traces[g]!==s.traces[g])return!1}else if(s.prop!==o.prop)return!1}else o=u[0],Array.isArray(o.traces)&&o.traces.sort();var h=(s=u[0]).value;if(Array.isArray(h)){if(1!==h.length)return!1;h=h[0]}r&&(r[h]=i)}return o},n.executeAPICommand=function(t,e,n){if("skip"===e)return Promise.resolve();var o=r.apiMethodRegistry[e],a=[t];Array.isArray(n)||(n=[]);for(var s=0;s<n.length;s++)a.push(n[s]);return o.apply(null,a).catch(function(t){return i.warn("API call to Plotly."+e+" rejected.",t),Promise.reject(t)})},n.computeAPICommandBindings=function(t,e,n){var r;switch(Array.isArray(n)||(n=[]),e){case"restyle":r=s(t,n);break;case"relayout":r=a(0,n);break;case"update":r=s(t,[n[0],n[2]]).concat(a(0,[n[1]]));break;case"animate":r=function(t,e){return Array.isArray(e[0])&&1===e[0].length&&-1!==["string","number"].indexOf(typeof e[0][0])?[{type:"layout",prop:"_currentFrame",value:e[0][0].toString()}]:[]}(0,n);break;default:r=[]}return r}},{"../lib":719,"../registry":847}],792:[function(t,e,n){"use strict";var r=t("../lib/extend").extendFlat;n.attributes=function(t,e){e=e||{};var n={valType:"info_array",editType:(t=t||{}).editType,items:[{valType:"number",min:0,max:1,editType:t.editType},{valType:"number",min:0,max:1,editType:t.editType}],dflt:[0,1]},i=(t.name&&t.name,t.trace,e.description&&e.description,{x:r({},n,{}),y:r({},n,{}),editType:t.editType});return t.noGridCell||(i.row={valType:"integer",min:0,dflt:0,editType:t.editType},i.column={valType:"integer",min:0,dflt:0,editType:t.editType}),i},n.defaults=function(t,e,n,r){var i=r&&r.x||[0,1],o=r&&r.y||[0,1],a=e.grid;if(a){var s=n("domain.column");void 0!==s&&(s<a.columns?i=a._domains.x[s]:delete t.domain.column);var A=n("domain.row");void 0!==A&&(A<a.rows?o=a._domains.y[A]:delete t.domain.row)}n("domain.x",i),n("domain.y",o)}},{"../lib/extend":710}],793:[function(t,e,n){"use strict";e.exports=function(t){var e=t.editType,n=t.colorEditType;void 0===n&&(n=e);var r={family:{valType:"string",noBlank:!0,strict:!0,editType:e},size:{valType:"number",min:1,editType:e},color:{valType:"color",editType:n},editType:e};return t.arrayOk&&(r.family.arrayOk=!0,r.size.arrayOk=!0,r.color.arrayOk=!0),r}},{}],794:[function(t,e,n){"use strict";e.exports={_isLinkedToArray:"frames_entry",group:{valType:"string"},name:{valType:"string"},traces:{valType:"any"},baseframe:{valType:"string"},data:{valType:"any"},layout:{valType:"any"}}},{}],795:[function(t,e,n){"use strict";n.projNames={equirectangular:"equirectangular",mercator:"mercator",orthographic:"orthographic","natural earth":"naturalEarth",kavrayskiy7:"kavrayskiy7",miller:"miller",robinson:"robinson",eckert4:"eckert4","azimuthal equal area":"azimuthalEqualArea","azimuthal equidistant":"azimuthalEquidistant","conic equal area":"conicEqualArea","conic conformal":"conicConformal","conic equidistant":"conicEquidistant",gnomonic:"gnomonic",stereographic:"stereographic",mollweide:"mollweide",hammer:"hammer","transverse mercator":"transverseMercator","albers usa":"albersUsa","winkel tripel":"winkel3",aitoff:"aitoff",sinusoidal:"sinusoidal"},n.axesNames=["lonaxis","lataxis"],n.lonaxisSpan={orthographic:180,"azimuthal equal area":360,"azimuthal equidistant":360,"conic conformal":180,gnomonic:160,stereographic:180,"transverse mercator":180,"*":360},n.lataxisSpan={"conic conformal":150,stereographic:179.5,"*":180},n.scopeDefaults={world:{lonaxisRange:[-180,180],lataxisRange:[-90,90],projType:"equirectangular",projRotate:[0,0,0]},usa:{lonaxisRange:[-180,-50],lataxisRange:[15,80],projType:"albers usa"},europe:{lonaxisRange:[-30,60],lataxisRange:[30,85],projType:"conic conformal",projRotate:[15,0,0],projParallels:[0,60]},asia:{lonaxisRange:[22,160],lataxisRange:[-15,55],projType:"mercator",projRotate:[0,0,0]},africa:{lonaxisRange:[-30,60],lataxisRange:[-40,40],projType:"mercator",projRotate:[0,0,0]},"north america":{lonaxisRange:[-180,-45],lataxisRange:[5,85],projType:"conic conformal",projRotate:[-100,0,0],projParallels:[29.5,45.5]},"south america":{lonaxisRange:[-100,-30],lataxisRange:[-60,15],projType:"mercator",projRotate:[0,0,0]}},n.clipPad=.001,n.precision=.1,n.landColor="#F0DC82",n.waterColor="#3399FF",n.locationmodeToLayer={"ISO-3":"countries","USA-states":"subunits","country names":"countries"},n.sphereSVG={type:"Sphere"},n.fillLayers={ocean:1,land:1,lakes:1},n.lineLayers={subunits:1,countries:1,coastlines:1,rivers:1,frame:1},n.layers=["bg","ocean","land","lakes","subunits","countries","coastlines","rivers","lataxis","lonaxis","frame","backplot","frontplot"],n.layersForChoropleth=["bg","ocean","land","subunits","countries","coastlines","lataxis","lonaxis","frame","backplot","rivers","lakes","frontplot"],n.layerNameToAdjective={ocean:"ocean",land:"land",lakes:"lake",subunits:"subunit",countries:"country",coastlines:"coastline",rivers:"river",frame:"frame"}},{}],796:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../registry"),o=t("../../lib"),a=t("../../components/color"),s=t("../../components/drawing"),A=t("../../components/fx"),l=t("../plots"),c=t("../cartesian/axes"),u=t("../../components/dragelement"),g=t("../cartesian/select").prepSelect,h=t("../cartesian/select").selectOnClick,d=t("./zoom"),f=t("./constants"),p=t("../../lib/topojson_utils"),I=t("topojson-client").feature;function C(t){this.id=t.id,this.graphDiv=t.graphDiv,this.container=t.container,this.topojsonURL=t.topojsonURL,this.isStatic=t.staticPlot,this.topojsonName=null,this.topojson=null,this.projection=null,this.scope=null,this.viewInitial=null,this.fitScale=null,this.bounds=null,this.midPt=null,this.hasChoropleth=!1,this.traceHash={},this.layers={},this.basePaths={},this.dataPaths={},this.dataPoints={},this.clipDef=null,this.clipRect=null,this.bgRect=null,this.makeFramework()}t("./projections")(r);var m=C.prototype;e.exports=function(t){return new C(t)},m.plot=function(t,e,n){var r=this,i=e[this.id],o=!1;for(var a in f.layerNameToAdjective)if("frame"!==a&&i["show"+a]){o=!0;break}for(var s=0;s<t.length;s++)if(t[0][0].trace.locationmode){o=!0;break}if(!o)return r.update(t,e);var A=p.getTopojsonName(i);null===r.topojson||A!==r.topojsonName?(r.topojsonName=A,void 0===PlotlyGeoAssets.topojson[r.topojsonName]?n.push(r.fetchTopojson().then(function(n){PlotlyGeoAssets.topojson[r.topojsonName]=n,r.topojson=n,r.update(t,e)})):(r.topojson=PlotlyGeoAssets.topojson[r.topojsonName],r.update(t,e))):r.update(t,e)},m.fetchTopojson=function(){var t=p.getTopojsonPath(this.topojsonURL,this.topojsonName);return new Promise(function(e,n){r.json(t,function(r,i){if(r)return 404===r.status?n(new Error(["plotly.js could not find topojson file at",t,".","Make sure the *topojsonURL* plot config option","is set properly."].join(" "))):n(new Error(["unexpected error while fetching topojson file at",t].join(" ")));e(i)})})},m.update=function(t,e){var n=e[this.id];if(!this.updateProjection(e,n)){this.hasChoropleth=!1;for(var r=0;r<t.length;r++)if("choropleth"===t[r][0].trace.type){this.hasChoropleth=!0;break}this.viewInitial&&this.scope===n.scope||this.saveViewInitial(n),this.scope=n.scope,this.updateBaseLayers(e,n),this.updateDims(e,n),this.updateFx(e,n),l.generalUpdatePerTraceModule(this.graphDiv,this,t,n);var i=this.layers.frontplot.select(".scatterlayer");this.dataPoints.point=i.selectAll(".point"),this.dataPoints.text=i.selectAll("text"),this.dataPaths.line=i.selectAll(".js-line");var o=this.layers.backplot.select(".choroplethlayer");this.dataPaths.choropleth=o.selectAll("path"),this.render()}},m.updateProjection=function(t,e){var n=t._size,a=e.domain,s=e.projection,A=s.rotation||{},l=e.center||{},c=this.projection=function(t){for(var e=t.projection.type,n=r.geo[f.projNames[e]](),i=t._isClipped?f.lonaxisSpan[e]/2:null,o=["center","rotate","parallels","clipExtent"],a=function(t){return t?n:[]},s=0;s<o.length;s++){var A=o[s];"function"!=typeof n[A]&&(n[A]=a)}return n.isLonLatOverEdges=function(t){if(null===n(t))return!0;if(i){var e=n.rotate();return r.geo.distance(t,[-e[0],-e[1]])>i*Math.PI/180}return!1},n.getPath=function(){return r.geo.path().projection(n)},n.getBounds=function(t){return n.getPath().bounds(t)},n.fitExtent=function(t,e){var r=t[1][0]-t[0][0],i=t[1][1]-t[0][1],o=n.clipExtent&&n.clipExtent();n.scale(150).translate([0,0]),o&&n.clipExtent(null);var a=n.getBounds(e),s=Math.min(r/(a[1][0]-a[0][0]),i/(a[1][1]-a[0][1])),A=+t[0][0]+(r-s*(a[1][0]+a[0][0]))/2,l=+t[0][1]+(i-s*(a[1][1]+a[0][1]))/2;return o&&n.clipExtent(o),n.scale(150*s).translate([A,l])},n.precision(f.precision),i&&n.clipAngle(i-f.clipPad),n}(e);c.center([l.lon-A.lon,l.lat-A.lat]).rotate([-A.lon,-A.lat,A.roll]).parallels(s.parallels);var u=[[n.l+n.w*a.x[0],n.t+n.h*(1-a.y[1])],[n.l+n.w*a.x[1],n.t+n.h*(1-a.y[0])]],g=e.lonaxis,h=e.lataxis,d=function(t,e){var n=f.clipPad,r=t[0]+n,i=t[1]-n,o=e[0]+n,a=e[1]-n;r>0&&i<0&&(i+=360);var s=(i-r)/4;return{type:"Polygon",coordinates:[[[r,o],[r,a],[r+s,a],[r+2*s,a],[r+3*s,a],[i,a],[i,o],[i-s,o],[i-2*s,o],[i-3*s,o],[r,o]]]}}(g.range,h.range);c.fitExtent(u,d);var p=this.bounds=c.getBounds(d),I=this.fitScale=c.scale(),C=c.translate();if(!isFinite(p[0][0])||!isFinite(p[0][1])||!isFinite(p[1][0])||!isFinite(p[1][1])||isNaN(C[0])||isNaN(C[0])){for(var m=this.graphDiv,y=["projection.rotation","center","lonaxis.range","lataxis.range"],E="Invalid geo settings, relayout'ing to default view.",v={},B=0;B<y.length;B++)v[this.id+"."+y[B]]=null;return this.viewInitial=null,o.warn(E),m._promises.push(i.call("relayout",m,v)),E}var b=this.midPt=[(p[0][0]+p[1][0])/2,(p[0][1]+p[1][1])/2];if(c.scale(s.scale*I).translate([C[0]+(b[0]-C[0]),C[1]+(b[1]-C[1])]).clipExtent(p),e._isAlbersUsa){var Q=c([l.lon,l.lat]),w=c.translate();c.translate([w[0]-(Q[0]-w[0]),w[1]-(Q[1]-w[1])])}},m.updateBaseLayers=function(t,e){var n=this,i=n.topojson,o=n.layers,A=n.basePaths;function l(t){return"lonaxis"===t||"lataxis"===t}function u(t){return Boolean(f.lineLayers[t])}function g(t){return Boolean(f.fillLayers[t])}var h=(this.hasChoropleth?f.layersForChoropleth:f.layers).filter(function(t){return u(t)||g(t)?e["show"+t]:!l(t)||e[t].showgrid}),d=n.framework.selectAll(".layer").data(h,String);d.exit().each(function(t){delete o[t],delete A[t],r.select(this).remove()}),d.enter().append("g").attr("class",function(t){return"layer "+t}).each(function(t){var e=o[t]=r.select(this);"bg"===t?n.bgRect=e.append("rect").style("pointer-events","all"):l(t)?A[t]=e.append("path").style("fill","none"):"backplot"===t?e.append("g").classed("choroplethlayer",!0):"frontplot"===t?e.append("g").classed("scatterlayer",!0):u(t)?A[t]=e.append("path").style("fill","none").style("stroke-miterlimit",2):g(t)&&(A[t]=e.append("path").style("stroke","none"))}),d.order(),d.each(function(n){var r=A[n],o=f.layerNameToAdjective[n];"frame"===n?r.datum(f.sphereSVG):u(n)||g(n)?r.datum(I(i,i.objects[n])):l(n)&&r.datum(function(t,e,n){var r,i,o,a=e[t],s=f.scopeDefaults[e.scope];"lonaxis"===t?(r=s.lonaxisRange,i=s.lataxisRange,o=function(t,e){return[t,e]}):"lataxis"===t&&(r=s.lataxisRange,i=s.lonaxisRange,o=function(t,e){return[e,t]});var A={type:"linear",range:[r[0],r[1]-1e-6],tick0:a.tick0,dtick:a.dtick};c.setConvert(A,n);var l=c.calcTicks(A);e.isScoped||"lonaxis"!==t||l.pop();for(var u=l.length,g=new Array(u),h=0;h<u;h++)for(var d=l[h].x,p=g[h]=[],I=i[0];I<i[1]+2.5;I+=2.5)p.push(o(d,I));return{type:"MultiLineString",coordinates:g}}(n,e,t)).call(a.stroke,e[n].gridcolor).call(s.dashLine,"",e[n].gridwidth),u(n)?r.call(a.stroke,e[o+"color"]).call(s.dashLine,"",e[o+"width"]):g(n)&&r.call(a.fill,e[o+"color"])})},m.updateDims=function(t,e){var n=this.bounds,r=(e.framewidth||0)/2,i=n[0][0]-r,o=n[0][1]-r,A=n[1][0]-i+r,l=n[1][1]-o+r;s.setRect(this.clipRect,i,o,A,l),this.bgRect.call(s.setRect,i,o,A,l).call(a.fill,e.bgcolor),this.xaxis._offset=i,this.xaxis._length=A,this.yaxis._offset=o,this.yaxis._length=l},m.updateFx=function(t,e){var n=this,o=n.graphDiv,a=n.bgRect,s=t.dragmode,l=t.clickmode;if(!n.isStatic){var c;"select"===s?c=function(t,e){(t.range={})[n.id]=[p([e.xmin,e.ymin]),p([e.xmax,e.ymax])]}:"lasso"===s&&(c=function(t,e,r){(t.lassoPoints={})[n.id]=r.filtered.map(p)});var f={element:n.bgRect.node(),gd:o,plotinfo:{id:n.id,xaxis:n.xaxis,yaxis:n.yaxis,fillRangeItems:c},xaxes:[n.xaxis],yaxes:[n.yaxis],subplot:n.id,clickFn:function(e){2===e&&t._zoomlayer.selectAll(".select-outline").remove()}};"pan"===s?(a.node().onmousedown=null,a.call(d(n,e)),a.on("dblclick.zoom",function(){var t=n.viewInitial,e={};for(var r in t)e[n.id+"."+r]=t[r];i.call("_guiRelayout",o,e),o.emit("plotly_doubleclick",null)}),o._context._scrollZoom.geo||a.on("wheel.zoom",null)):"select"!==s&&"lasso"!==s||(a.on(".zoom",null),f.prepFn=function(t,e,n){g(t,e,n,f,s)},u.init(f)),a.on("mousemove",function(){var t=n.projection.invert(r.mouse(this));if(!t||isNaN(t[0])||isNaN(t[1]))return u.unhover(o,r.event);n.xaxis.p2c=function(){return t[0]},n.yaxis.p2c=function(){return t[1]},A.hover(o,r.event,n.id)}),a.on("mouseout",function(){o._dragging||u.unhover(o,r.event)}),a.on("click",function(){"select"!==s&&"lasso"!==s&&(l.indexOf("select")>-1&&h(r.event,o,[n.xaxis],[n.yaxis],n.id,f),l.indexOf("event")>-1&&A.click(o,r.event))})}function p(t){return n.projection.invert([t[0]+n.xaxis._offset,t[1]+n.yaxis._offset])}},m.makeFramework=function(){var t=this,e=t.graphDiv,n=e._fullLayout,i="clip"+n._uid+t.id;t.clipDef=n._clips.append("clipPath").attr("id",i),t.clipRect=t.clipDef.append("rect"),t.framework=r.select(t.container).append("g").attr("class","geo "+t.id).call(s.setClipUrl,i,e),t.project=function(e){var n=t.projection(e);return n?[n[0]-t.xaxis._offset,n[1]-t.yaxis._offset]:[null,null]},t.xaxis={_id:"x",c2p:function(e){return t.project(e)[0]}},t.yaxis={_id:"y",c2p:function(e){return t.project(e)[1]}},t.mockAxis={type:"linear",showexponent:"all",exponentformat:"B"},c.setConvert(t.mockAxis,n)},m.saveViewInitial=function(t){var e=t.center||{},n=t.projection,r=n.rotation||{};t._isScoped?this.viewInitial={"center.lon":e.lon,"center.lat":e.lat,"projection.scale":n.scale}:t._isClipped?this.viewInitial={"projection.scale":n.scale,"projection.rotation.lon":r.lon,"projection.rotation.lat":r.lat}:this.viewInitial={"center.lon":e.lon,"center.lat":e.lat,"projection.scale":n.scale,"projection.rotation.lon":r.lon}},m.render=function(){var t,e=this.projection,n=e.getPath();function r(t){var n=e(t.lonlat);return n?"translate("+n[0]+","+n[1]+")":null}function i(t){return e.isLonLatOverEdges(t.lonlat)?"none":null}for(t in this.basePaths)this.basePaths[t].attr("d",n);for(t in this.dataPaths)this.dataPaths[t].attr("d",function(t){return n(t.geojson)});for(t in this.dataPoints)this.dataPoints[t].attr("display",i).attr("transform",r)}},{"../../components/color":593,"../../components/dragelement":611,"../../components/drawing":614,"../../components/fx":632,"../../lib":719,"../../lib/topojson_utils":746,"../../registry":847,"../cartesian/axes":767,"../cartesian/select":784,"../plots":828,"./constants":795,"./projections":800,"./zoom":801,d3:163,"topojson-client":540}],797:[function(t,e,n){"use strict";var r=t("../../plots/get_data").getSubplotCalcData,i=t("../../lib").counterRegex,o=t("./geo"),a="geo",s=i(a),A={};A[a]={valType:"subplotid",dflt:a,editType:"calc"},e.exports={attr:a,name:a,idRoot:a,idRegex:s,attrRegex:s,attributes:A,layoutAttributes:t("./layout_attributes"),supplyLayoutDefaults:t("./layout_defaults"),plot:function(t){for(var e=t._fullLayout,n=t.calcdata,i=e._subplots[a],s=0;s<i.length;s++){var A=i[s],l=r(n,a,A),c=e[A]._subplot;c||(c=o({id:A,graphDiv:t,container:e._geolayer.node(),topojsonURL:t._context.topojsonURL,staticPlot:t._context.staticPlot}),e[A]._subplot=c),c.plot(l,e,t._promises)}},updateFx:function(t){for(var e=t._fullLayout,n=e._subplots[a],r=0;r<n.length;r++){var i=e[n[r]];i._subplot.updateFx(e,i)}},clean:function(t,e,n,r){for(var i=r._subplots[a]||[],o=0;o<i.length;o++){var s=i[o],A=r[s]._subplot;!e[s]&&A&&(A.framework.remove(),A.clipDef.remove())}}}},{"../../lib":719,"../../plots/get_data":802,"./geo":796,"./layout_attributes":798,"./layout_defaults":799}],798:[function(t,e,n){"use strict";var r=t("../../components/color/attributes"),i=t("../domain").attributes,o=t("./constants"),a=t("../../plot_api/edit_types").overrideAll,s={range:{valType:"info_array",items:[{valType:"number"},{valType:"number"}]},showgrid:{valType:"boolean",dflt:!1},tick0:{valType:"number",dflt:0},dtick:{valType:"number"},gridcolor:{valType:"color",dflt:r.lightLine},gridwidth:{valType:"number",min:0,dflt:1}};(e.exports=a({domain:i({name:"geo"},{}),resolution:{valType:"enumerated",values:[110,50],dflt:110,coerceNumber:!0},scope:{valType:"enumerated",values:Object.keys(o.scopeDefaults),dflt:"world"},projection:{type:{valType:"enumerated",values:Object.keys(o.projNames)},rotation:{lon:{valType:"number"},lat:{valType:"number"},roll:{valType:"number"}},parallels:{valType:"info_array",items:[{valType:"number"},{valType:"number"}]},scale:{valType:"number",min:0,dflt:1}},center:{lon:{valType:"number"},lat:{valType:"number"}},showcoastlines:{valType:"boolean"},coastlinecolor:{valType:"color",dflt:r.defaultLine},coastlinewidth:{valType:"number",min:0,dflt:1},showland:{valType:"boolean",dflt:!1},landcolor:{valType:"color",dflt:o.landColor},showocean:{valType:"boolean",dflt:!1},oceancolor:{valType:"color",dflt:o.waterColor},showlakes:{valType:"boolean",dflt:!1},lakecolor:{valType:"color",dflt:o.waterColor},showrivers:{valType:"boolean",dflt:!1},rivercolor:{valType:"color",dflt:o.waterColor},riverwidth:{valType:"number",min:0,dflt:1},showcountries:{valType:"boolean"},countrycolor:{valType:"color",dflt:r.defaultLine},countrywidth:{valType:"number",min:0,dflt:1},showsubunits:{valType:"boolean"},subunitcolor:{valType:"color",dflt:r.defaultLine},subunitwidth:{valType:"number",min:0,dflt:1},showframe:{valType:"boolean"},framecolor:{valType:"color",dflt:r.defaultLine},framewidth:{valType:"number",min:0,dflt:1},bgcolor:{valType:"color",dflt:r.background},lonaxis:s,lataxis:s},"plot","from-root")).uirevision={valType:"any",editType:"none"}},{"../../components/color/attributes":592,"../../plot_api/edit_types":750,"../domain":792,"./constants":795}],799:[function(t,e,n){"use strict";var r=t("../subplot_defaults"),i=t("./constants"),o=t("./layout_attributes"),a=i.axesNames;function s(t,e,n){var r=n("resolution"),o=n("scope"),s=i.scopeDefaults[o],A=n("projection.type",s.projType),l=e._isAlbersUsa="albers usa"===A;l&&(o=e.scope="usa");var c=e._isScoped="world"!==o,u=e._isConic=-1!==A.indexOf("conic");e._isClipped=!!i.lonaxisSpan[A];for(var g=0;g<a.length;g++){var h,d=a[g],f=[30,10][g];if(c)h=s[d+"Range"];else{var p=i[d+"Span"],I=(p[A]||p["*"])/2,C=n("projection.rotation."+d.substr(0,3),s.projRotate[g]);h=[C-I,C+I]}n(d+".range",h),n(d+".tick0"),n(d+".dtick",f),n(d+".showgrid")&&(n(d+".gridcolor"),n(d+".gridwidth"))}var m=e.lonaxis.range,y=e.lataxis.range,E=m[0],v=m[1];E>0&&v<0&&(v+=360);var B,b,Q,w=(E+v)/2;if(!l){var x=c?s.projRotate:[w,0,0];B=n("projection.rotation.lon",x[0]),n("projection.rotation.lat",x[1]),n("projection.rotation.roll",x[2]),n("showcoastlines",!c)&&(n("coastlinecolor"),n("coastlinewidth")),n("showocean")&&n("oceancolor")}l?(b=-96.6,Q=38.7):(b=c?w:B,Q=(y[0]+y[1])/2),n("center.lon",b),n("center.lat",Q),u&&n("projection.parallels",s.projParallels||[0,60]),n("projection.scale"),n("showland")&&n("landcolor"),n("showlakes")&&n("lakecolor"),n("showrivers")&&(n("rivercolor"),n("riverwidth")),n("showcountries",c&&"usa"!==o)&&(n("countrycolor"),n("countrywidth")),("usa"===o||"north america"===o&&50===r)&&(n("showsubunits",!0),n("subunitcolor"),n("subunitwidth")),c||n("showframe",!0)&&(n("framecolor"),n("framewidth")),n("bgcolor")}e.exports=function(t,e,n){r(t,e,n,{type:"geo",attributes:o,handleDefaults:s,partition:"y"})}},{"../subplot_defaults":842,"./constants":795,"./layout_attributes":798}],800:[function(t,e,n){"use strict";e.exports=function(t){function e(t,e){return{type:"Feature",id:t.id,properties:t.properties,geometry:n(t.geometry,e)}}function n(e,r){if(!e)return null;if("GeometryCollection"===e.type)return{type:"GeometryCollection",geometries:object.geometries.map(function(t){return n(t,r)})};if(!l.hasOwnProperty(e.type))return null;var i=l[e.type];return t.geo.stream(e,r(i)),i.result()}t.geo.project=function(t,e){var i=e.stream;if(!i)throw new Error("not yet supported");return(t&&r.hasOwnProperty(t.type)?r[t.type]:n)(t,i)};var r={Feature:e,FeatureCollection:function(t,n){return{type:"FeatureCollection",features:t.features.map(function(t){return e(t,n)})}}},i=[],o=[],a={point:function(t,e){i.push([t,e])},result:function(){var t=i.length?i.length<2?{type:"Point",coordinates:i[0]}:{type:"MultiPoint",coordinates:i}:null;return i=[],t}},s={lineStart:c,point:function(t,e){i.push([t,e])},lineEnd:function(){i.length&&(o.push(i),i=[])},result:function(){var t=o.length?o.length<2?{type:"LineString",coordinates:o[0]}:{type:"MultiLineString",coordinates:o}:null;return o=[],t}},A={polygonStart:c,lineStart:c,point:function(t,e){i.push([t,e])},lineEnd:function(){var t=i.length;if(t){do{i.push(i[0].slice())}while(++t<4);o.push(i),i=[]}},polygonEnd:c,result:function(){if(!o.length)return null;var t=[],e=[];return o.forEach(function(n){!function(t){if((e=t.length)<4)return!1;for(var e,n=0,r=t[e-1][1]*t[0][0]-t[e-1][0]*t[0][1];++n<e;)r+=t[n-1][1]*t[n][0]-t[n-1][0]*t[n][1];return r<=0}(n)?e.push(n):t.push([n])}),e.forEach(function(e){var n=e[0];t.some(function(t){if(function(t,e){for(var n=e[0],r=e[1],i=!1,o=0,a=t.length,s=a-1;o<a;s=o++){var A=t[o],l=A[0],c=A[1],u=t[s],g=u[0],h=u[1];c>r^h>r&&n<(g-l)*(r-c)/(h-c)+l&&(i=!i)}return i}(t[0],n))return t.push(e),!0})||t.push([e])}),o=[],t.length?t.length>1?{type:"MultiPolygon",coordinates:t}:{type:"Polygon",coordinates:t[0]}:null}},l={Point:a,MultiPoint:a,LineString:s,MultiLineString:s,Polygon:A,MultiPolygon:A,Sphere:A};function c(){}var u=1e-6,g=u*u,h=Math.PI,d=h/2,f=(Math.sqrt(h),h/180),p=180/h;function I(t){return t>1?d:t<-1?-d:Math.asin(t)}function C(t){return t>1?0:t<-1?h:Math.acos(t)}var m=t.geo.projection,y=t.geo.projectionMutator;function E(t,e){var n=(2+d)*Math.sin(e);e/=2;for(var r=0,i=1/0;r<10&&Math.abs(i)>u;r++){var o=Math.cos(e);e-=i=(e+Math.sin(e)*(o+2)-n)/(2*o*(1+o))}return[2/Math.sqrt(h*(4+h))*t*(1+Math.cos(e)),2*Math.sqrt(h/(4+h))*Math.sin(e)]}t.geo.interrupt=function(e){var n,r=[[[[-h,0],[0,d],[h,0]]],[[[-h,0],[0,-d],[h,0]]]];function i(t,n){for(var i=n<0?-1:1,o=r[+(n<0)],a=0,s=o.length-1;a<s&&t>o[a][2][0];++a);var A=e(t-o[a][1][0],n);return A[0]+=e(o[a][1][0],i*n>i*o[a][0][1]?o[a][0][1]:n)[0],A}e.invert&&(i.invert=function(t,o){for(var a=n[+(o<0)],s=r[+(o<0)],l=0,c=a.length;l<c;++l){var u=a[l];if(u[0][0]<=t&&t<u[1][0]&&u[0][1]<=o&&o<u[1][1]){var g=e.invert(t-e(s[l][1][0],0)[0],o);return g[0]+=s[l][1][0],A(i(g[0],g[1]),[t,o])?g:null}}});var o=t.geo.projection(i),a=o.stream;function s(t,e){for(var n,r,i,o=-1,a=t.length,s=t[0],A=[];++o<a;){r=((n=t[o])[0]-s[0])/e,i=(n[1]-s[1])/e;for(var l=0;l<e;++l)A.push([s[0]+l*r,s[1]+l*i]);s=n}return A.push(n),A}function A(t,e){return Math.abs(t[0]-e[0])<u&&Math.abs(t[1]-e[1])<u}return o.stream=function(e){var n=o.rotate(),i=a(e),A=(o.rotate([0,0]),a(e));return o.rotate(n),i.sphere=function(){t.geo.stream(function(){for(var e=1e-6,n=[],i=0,o=r[0].length;i<o;++i){var a=r[0][i],A=180*a[0][0]/h,l=180*a[0][1]/h,c=180*a[1][1]/h,u=180*a[2][0]/h,g=180*a[2][1]/h;n.push(s([[A+e,l+e],[A+e,c-e],[u-e,c-e],[u-e,g+e]],30))}for(var i=r[1].length-1;i>=0;--i){var a=r[1][i],A=180*a[0][0]/h,l=180*a[0][1]/h,c=180*a[1][1]/h,u=180*a[2][0]/h,g=180*a[2][1]/h;n.push(s([[u-e,g-e],[u-e,c+e],[A+e,c+e],[A+e,l-e]],30))}return{type:"Polygon",coordinates:[t.merge(n)]}}(),A)},i},o.lobes=function(t){return arguments.length?(r=t.map(function(t){return t.map(function(t){return[[t[0][0]*h/180,t[0][1]*h/180],[t[1][0]*h/180,t[1][1]*h/180],[t[2][0]*h/180,t[2][1]*h/180]]})}),n=r.map(function(t){return t.map(function(t){var n,r=e(t[0][0],t[0][1])[0],i=e(t[2][0],t[2][1])[0],o=e(t[1][0],t[0][1])[1],a=e(t[1][0],t[1][1])[1];return o>a&&(n=o,o=a,a=n),[[r,o],[i,a]]})}),o):r.map(function(t){return t.map(function(t){return[[180*t[0][0]/h,180*t[0][1]/h],[180*t[1][0]/h,180*t[1][1]/h],[180*t[2][0]/h,180*t[2][1]/h]]})})},o},E.invert=function(t,e){var n=.5*e*Math.sqrt((4+h)/h),r=I(n),i=Math.cos(r);return[t/(2/Math.sqrt(h*(4+h))*(1+i)),I((r+n*(i+2))/(2+d))]},(t.geo.eckert4=function(){return m(E)}).raw=E;var v=t.geo.azimuthalEqualArea.raw;function B(t,e){if(arguments.length<2&&(e=t),1===e)return v;if(e===1/0)return b;function n(n,r){var i=v(n/e,r);return i[0]*=t,i}return n.invert=function(n,r){var i=v.invert(n/t,r);return i[0]*=e,i},n}function b(t,e){return[t*Math.cos(e)/Math.cos(e/=2),2*Math.sin(e)]}function Q(t,e){return[3*t/(2*h)*Math.sqrt(h*h/3-e*e),e]}function w(t,e){return[t,1.25*Math.log(Math.tan(h/4+.4*e))]}function x(t){return function(e){var n,r=t*Math.sin(e),i=30;do{e-=n=(e+Math.sin(e)-r)/(1+Math.cos(e))}while(Math.abs(n)>u&&--i>0);return e/2}}b.invert=function(t,e){var n=2*I(e/2);return[t*Math.cos(n/2)/Math.cos(n),n]},(t.geo.hammer=function(){var t=2,e=y(B),n=e(t);return n.coefficient=function(n){return arguments.length?e(t=+n):t},n}).raw=B,Q.invert=function(t,e){return[2/3*h*t/Math.sqrt(h*h/3-e*e),e]},(t.geo.kavrayskiy7=function(){return m(Q)}).raw=Q,w.invert=function(t,e){return[t,2.5*Math.atan(Math.exp(.8*e))-.625*h]},(t.geo.miller=function(){return m(w)}).raw=w,x(h);var _=function(t,e,n){var r=x(n);function i(n,i){return[t*n*Math.cos(i=r(i)),e*Math.sin(i)]}return i.invert=function(r,i){var o=I(i/e);return[r/(t*Math.cos(o)),I((2*o+Math.sin(2*o))/n)]},i}(Math.SQRT2/d,Math.SQRT2,h);function D(t,e){var n=e*e,r=n*n;return[t*(.8707-.131979*n+r*(r*(.003971*n-.001529*r)-.013791)),e*(1.007226+n*(.015085+r*(.028874*n-.044475-.005916*r)))]}(t.geo.mollweide=function(){return m(_)}).raw=_,D.invert=function(t,e){var n,r=e,i=25;do{var o=r*r,a=o*o;r-=n=(r*(1.007226+o*(.015085+a*(.028874*o-.044475-.005916*a)))-e)/(1.007226+o*(.045255+a*(.259866*o-.311325-.005916*11*a)))}while(Math.abs(n)>u&&--i>0);return[t/(.8707+(o=r*r)*(o*(o*o*o*(.003971-.001529*o)-.013791)-.131979)),r]},(t.geo.naturalEarth=function(){return m(D)}).raw=D;var M=[[.9986,-.062],[1,0],[.9986,.062],[.9954,.124],[.99,.186],[.9822,.248],[.973,.31],[.96,.372],[.9427,.434],[.9216,.4958],[.8962,.5571],[.8679,.6176],[.835,.6769],[.7986,.7346],[.7597,.7903],[.7186,.8435],[.6732,.8936],[.6213,.9394],[.5722,.9761],[.5322,1]];function k(t,e){var n,r=Math.min(18,36*Math.abs(e)/h),i=Math.floor(r),o=r-i,a=(n=M[i])[0],s=n[1],A=(n=M[++i])[0],l=n[1],c=(n=M[Math.min(19,++i)])[0],u=n[1];return[t*(A+o*(c-a)/2+o*o*(c-2*A+a)/2),(e>0?d:-d)*(l+o*(u-s)/2+o*o*(u-2*l+s)/2)]}function S(t,e){return[t*Math.cos(e),e]}function L(t,e){var n,r=Math.cos(e),i=(n=C(r*Math.cos(t/=2)))?n/Math.sin(n):1;return[2*r*Math.sin(t)*i,Math.sin(e)*i]}function R(t,e){var n=L(t,e);return[(n[0]+t/d)/2,(n[1]+e)/2]}M.forEach(function(t){t[1]*=1.0144}),k.invert=function(t,e){var n=e/d,r=90*n,i=Math.min(18,Math.abs(r/5)),o=Math.max(0,Math.floor(i));do{var a=M[o][1],s=M[o+1][1],A=M[Math.min(19,o+2)][1],l=A-a,c=A-2*s+a,u=2*(Math.abs(n)-s)/l,h=c/l,I=u*(1-h*u*(1-2*h*u));if(I>=0||1===o){r=(e>=0?5:-5)*(I+i);var C,m=50;do{I=(i=Math.min(18,Math.abs(r)/5))-(o=Math.floor(i)),a=M[o][1],s=M[o+1][1],A=M[Math.min(19,o+2)][1],r-=(C=(e>=0?d:-d)*(s+I*(A-a)/2+I*I*(A-2*s+a)/2)-e)*p}while(Math.abs(C)>g&&--m>0);break}}while(--o>=0);var y=M[o][0],E=M[o+1][0],v=M[Math.min(19,o+2)][0];return[t/(E+I*(v-y)/2+I*I*(v-2*E+y)/2),r*f]},(t.geo.robinson=function(){return m(k)}).raw=k,S.invert=function(t,e){return[t/Math.cos(e),e]},(t.geo.sinusoidal=function(){return m(S)}).raw=S,L.invert=function(t,e){if(!(t*t+4*e*e>h*h+u)){var n=t,r=e,i=25;do{var o,a=Math.sin(n),s=Math.sin(n/2),A=Math.cos(n/2),l=Math.sin(r),c=Math.cos(r),g=Math.sin(2*r),d=l*l,f=c*c,p=s*s,I=1-f*A*A,m=I?C(c*A)*Math.sqrt(o=1/I):o=0,y=2*m*c*s-t,E=m*l-e,v=o*(f*p+m*c*A*d),B=o*(.5*a*g-2*m*l*s),b=.25*o*(g*s-m*l*f*a),Q=o*(d*A+m*p*c),w=B*b-Q*v;if(!w)break;var x=(E*B-y*Q)/w,_=(y*b-E*v)/w;n-=x,r-=_}while((Math.abs(x)>u||Math.abs(_)>u)&&--i>0);return[n,r]}},(t.geo.aitoff=function(){return m(L)}).raw=L,R.invert=function(t,e){var n=t,r=e,i=25;do{var o,a=Math.cos(r),s=Math.sin(r),A=Math.sin(2*r),l=s*s,c=a*a,g=Math.sin(n),h=Math.cos(n/2),f=Math.sin(n/2),p=f*f,I=1-c*h*h,m=I?C(a*h)*Math.sqrt(o=1/I):o=0,y=.5*(2*m*a*f+n/d)-t,E=.5*(m*s+r)-e,v=.5*o*(c*p+m*a*h*l)+.5/d,B=o*(g*A/4-m*s*f),b=.125*o*(A*f-m*s*c*g),Q=.5*o*(l*h+m*p*a)+.5,w=B*b-Q*v,x=(E*B-y*Q)/w,_=(y*b-E*v)/w;n-=x,r-=_}while((Math.abs(x)>u||Math.abs(_)>u)&&--i>0);return[n,r]},(t.geo.winkel3=function(){return m(R)}).raw=R}},{}],801:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../lib"),o=t("../../registry"),a=Math.PI/180,s=180/Math.PI,A={cursor:"pointer"},l={cursor:"auto"};function c(t,e){return r.behavior.zoom().translate(e.translate()).scale(e.scale())}function u(t,e,n){var r=t.id,a=t.graphDiv,s=a.layout,A=s[r],l=a._fullLayout,c=l[r],u={},g={};function h(t,e){u[r+"."+t]=i.nestedProperty(A,t).get(),o.call("_storeDirectGUIEdit",s,l._preGUI,u);var n=i.nestedProperty(c,t);n.get()!==e&&(n.set(e),i.nestedProperty(A,t).set(e),g[r+"."+t]=e)}n(h),h("projection.scale",e.scale()/t.fitScale),a.emit("plotly_relayout",g)}function g(t,e){var n=t.invert(e);return n&&isFinite(n[0])&&isFinite(n[1])&&function(t){var e=t[0]*a,n=t[1]*a,r=Math.cos(n);return[r*Math.cos(e),r*Math.sin(e),Math.sin(n)]}(n)}function h(t,e,n,r){var i=d(n-t),o=d(r-e);return Math.sqrt(i*i+o*o)}function d(t){return(t%360+540)%360-180}function f(t,e,n){var r=n*a,i=t.slice(),o=0===e?1:0,s=2===e?1:2,A=Math.cos(r),l=Math.sin(r);return i[o]=t[o]*A-t[s]*l,i[s]=t[s]*A+t[o]*l,i}function p(t,e){for(var n=0,r=0,i=t.length;r<i;++r)n+=t[r]*e[r];return n}e.exports=function(t,e){var n=t.projection;return(e._isScoped?function(t,e){var n=c(0,e);function i(n){var r=e.invert(t.midPt);n("center.lon",r[0]),n("center.lat",r[1])}return n.on("zoomstart",function(){r.select(this).style(A)}).on("zoom",function(){e.scale(r.event.scale).translate(r.event.translate),t.render();var n=e.invert(t.midPt);t.graphDiv.emit("plotly_relayouting",{"geo.projection.scale":e.scale()/t.fitScale,"geo.center.lon":n[0],"geo.center.lat":n[1]})}).on("zoomend",function(){r.select(this).style(l),u(t,e,i)}),n}:e._isClipped?function(t,e){var n,i={r:e.rotate(),k:e.scale()},o=c(0,e),d=function(t){for(var e=0,n=arguments.length,i=[];++e<n;)i.push(arguments[e]);var o=r.dispatch.apply(null,i);return o.of=function(e,n){return function(i){var a;try{a=i.sourceEvent=r.event,i.target=t,r.event=i,o[i.type].apply(e,n)}finally{r.event=a}}},o}(o,"zoomstart","zoom","zoomend"),I=0,C=o.on;function m(t){var n=e.rotate();t("projection.rotation.lon",-n[0]),t("projection.rotation.lat",-n[1])}return o.on("zoomstart",function(){r.select(this).style(A);var t,l,c,u,m,y,E,v,B,b,Q,w=r.mouse(this),x=e.rotate(),_=x,D=e.translate(),M=(l=.5*(t=x)[0]*a,c=.5*t[1]*a,u=.5*t[2]*a,m=Math.sin(l),y=Math.cos(l),E=Math.sin(c),v=Math.cos(c),B=Math.sin(u),b=Math.cos(u),[y*v*b+m*E*B,m*v*b-y*E*B,y*E*b+m*v*B,y*v*B-m*E*b]);n=g(e,w),C.call(o,"zoom",function(){var t,o,a,A,l,c,u,I,C,m,y=r.mouse(this);if(e.scale(i.k=r.event.scale),n){if(g(e,y)){e.rotate(x).translate(D);var E=g(e,y),v=function(t,e){if(t&&e){var n=function(t,e){return[t[1]*e[2]-t[2]*e[1],t[2]*e[0]-t[0]*e[2],t[0]*e[1]-t[1]*e[0]]}(t,e),r=Math.sqrt(p(n,n)),i=.5*Math.acos(Math.max(-1,Math.min(1,p(t,e)))),o=Math.sin(i)/r;return r&&[Math.cos(i),n[2]*o,-n[1]*o,n[0]*o]}}(n,E),B=function(t){return[Math.atan2(2*(t[0]*t[1]+t[2]*t[3]),1-2*(t[1]*t[1]+t[2]*t[2]))*s,Math.asin(Math.max(-1,Math.min(1,2*(t[0]*t[2]-t[3]*t[1]))))*s,Math.atan2(2*(t[0]*t[3]+t[1]*t[2]),1-2*(t[2]*t[2]+t[3]*t[3]))*s]}((o=v,a=(t=M)[0],A=t[1],l=t[2],c=t[3],u=o[0],I=o[1],C=o[2],m=o[3],[a*u-A*I-l*C-c*m,a*I+A*u+l*m-c*C,a*C-A*m+l*u+c*I,a*m+A*C-l*I+c*u])),b=i.r=function(t,e,n){var r=f(e,2,t[0]);r=f(r,1,t[1]),r=f(r,0,t[2]-n[2]);var i,o,a=e[0],A=e[1],l=e[2],c=r[0],u=r[1],g=r[2],d=Math.atan2(A,a)*s,p=Math.sqrt(a*a+A*A);Math.abs(u)>p?(o=(u>0?90:-90)-d,i=0):(o=Math.asin(u/p)*s-d,i=Math.sqrt(p*p-u*u));var I=180-o-2*d,C=(Math.atan2(g,c)-Math.atan2(l,i))*s,m=(Math.atan2(g,c)-Math.atan2(l,-i))*s,y=h(n[0],n[1],o,C),E=h(n[0],n[1],I,m);return y<=E?[o,C,n[2]]:[I,m,n[2]]}(B,n,_);isFinite(b[0])&&isFinite(b[1])&&isFinite(b[2])||(b=_),e.rotate(b),_=b}}else n=g(e,w=y);d.of(this,arguments)({type:"zoom"})}),Q=d.of(this,arguments),I++||Q({type:"zoomstart"})}).on("zoomend",function(){var n;r.select(this).style(l),C.call(o,"zoom",null),n=d.of(this,arguments),--I||n({type:"zoomend"}),u(t,e,m)}).on("zoom.redraw",function(){t.render();var n=e.rotate();t.graphDiv.emit("plotly_relayouting",{"geo.projection.scale":e.scale()/t.fitScale,"geo.projection.rotation.lon":-n[0],"geo.projection.rotation.lat":-n[1]})}),r.rebind(o,d,"on")}:function(t,e){var n,i,o,a,s,g,h,d,f,p=c(0,e);function I(t){return e.invert(t)}function C(n){var r=e.rotate(),i=e.invert(t.midPt);n("projection.rotation.lon",-r[0]),n("center.lon",i[0]),n("center.lat",i[1])}return p.on("zoomstart",function(){r.select(this).style(A),n=r.mouse(this),i=e.rotate(),o=e.translate(),a=i,s=I(n)}).on("zoom",function(){if(g=r.mouse(this),function(t){var n=I(t);if(!n)return!0;var r=e(n);return Math.abs(r[0]-t[0])>2||Math.abs(r[1]-t[1])>2}(n))return p.scale(e.scale()),void p.translate(e.translate());e.scale(r.event.scale),e.translate([o[0],r.event.translate[1]]),s?I(g)&&(d=I(g),h=[a[0]+(d[0]-s[0]),i[1],i[2]],e.rotate(h),a=h):s=I(n=g),f=!0,t.render();var A=e.rotate(),l=e.invert(t.midPt);t.graphDiv.emit("plotly_relayouting",{"geo.projection.scale":e.scale()/t.fitScale,"geo.center.lon":l[0],"geo.center.lat":l[1],"geo.projection.rotation.lon":-A[0]})}).on("zoomend",function(){r.select(this).style(l),f&&u(t,e,C)}),p})(t,n)}},{"../../lib":719,"../../registry":847,d3:163}],802:[function(t,e,n){"use strict";var r=t("../registry"),i=t("./cartesian/constants").SUBPLOT_PATTERN;n.getSubplotCalcData=function(t,e,n){var i=r.subplotsRegistry[e];if(!i)return[];for(var o=i.attr,a=[],s=0;s<t.length;s++){var A=t[s];A[0].trace[o]===n&&a.push(A)}return a},n.getModuleCalcData=function(t,e){var n,i=[],o=[];if(!(n="string"==typeof e?r.getModule(e).plot:"function"==typeof e?e:e.plot))return[i,t];for(var a=0;a<t.length;a++){var s=t[a],A=s[0].trace;!0===A.visible&&0!==A._length&&(A._module.plot===n?i.push(s):o.push(s))}return[i,o]},n.getSubplotData=function(t,e,n){if(!r.subplotsRegistry[e])return[];var o,a,s,A=r.subplotsRegistry[e].attr,l=[];if("gl2d"===e){var c=n.match(i);a="x"+c[1],s="y"+c[2]}for(var u=0;u<t.length;u++)o=t[u],"gl2d"===e&&r.traceIs(o,"gl2d")?o[A[0]]===a&&o[A[1]]===s&&l.push(o):o[A]===n&&l.push(o);return l}},{"../registry":847,"./cartesian/constants":773}],803:[function(t,e,n){"use strict";var r=t("mouse-change"),i=t("mouse-wheel"),o=t("mouse-event-offset"),a=t("../cartesian/constants"),s=t("has-passive-events");function A(t,e){this.element=t,this.plot=e,this.mouseListener=null,this.wheelListener=null,this.lastInputTime=Date.now(),this.lastPos=[0,0],this.boxEnabled=!1,this.boxInited=!1,this.boxStart=[0,0],this.boxEnd=[0,0],this.dragStart=[0,0]}e.exports=function(t){var e=t.mouseContainer,n=t.glplot,l=new A(e,n);function c(){t.xaxis.autorange=!1,t.yaxis.autorange=!1}function u(e,r,i){var o,s,A=t.calcDataBox(),u=n.viewBox,g=l.lastPos[0],h=l.lastPos[1],d=a.MINDRAG*n.pixelRatio,f=a.MINZOOM*n.pixelRatio;function p(e,n,r){var i=Math.min(n,r),o=Math.max(n,r);i!==o?(A[e]=i,A[e+2]=o,l.dataBox=A,t.setRanges(A)):(t.selectBox.selectBox=[0,0,1,1],t.glplot.setDirty())}switch(r*=n.pixelRatio,i*=n.pixelRatio,i=u[3]-u[1]-i,t.fullLayout.dragmode){case"zoom":if(e){var I=r/(u[2]-u[0])*(A[2]-A[0])+A[0],C=i/(u[3]-u[1])*(A[3]-A[1])+A[1];l.boxInited||(l.boxStart[0]=I,l.boxStart[1]=C,l.dragStart[0]=r,l.dragStart[1]=i),l.boxEnd[0]=I,l.boxEnd[1]=C,l.boxInited=!0,l.boxEnabled||l.boxStart[0]===l.boxEnd[0]&&l.boxStart[1]===l.boxEnd[1]||(l.boxEnabled=!0);var m=Math.abs(l.dragStart[0]-r)<f,y=Math.abs(l.dragStart[1]-i)<f;if(!function(){for(var e=t.graphDiv._fullLayout._axisConstraintGroups,n=t.xaxis._id,r=t.yaxis._id,i=0;i<e.length;i++)if(-1!==e[i][n]){if(-1!==e[i][r])return!0;break}return!1}()||m&&y)m&&(l.boxEnd[0]=l.boxStart[0]),y&&(l.boxEnd[1]=l.boxStart[1]);else{o=l.boxEnd[0]-l.boxStart[0],s=l.boxEnd[1]-l.boxStart[1];var E=(A[3]-A[1])/(A[2]-A[0]);Math.abs(o*E)>Math.abs(s)?(l.boxEnd[1]=l.boxStart[1]+Math.abs(o)*E*(s>=0?1:-1),l.boxEnd[1]<A[1]?(l.boxEnd[1]=A[1],l.boxEnd[0]=l.boxStart[0]+(A[1]-l.boxStart[1])/Math.abs(E)):l.boxEnd[1]>A[3]&&(l.boxEnd[1]=A[3],l.boxEnd[0]=l.boxStart[0]+(A[3]-l.boxStart[1])/Math.abs(E))):(l.boxEnd[0]=l.boxStart[0]+Math.abs(s)/E*(o>=0?1:-1),l.boxEnd[0]<A[0]?(l.boxEnd[0]=A[0],l.boxEnd[1]=l.boxStart[1]+(A[0]-l.boxStart[0])*Math.abs(E)):l.boxEnd[0]>A[2]&&(l.boxEnd[0]=A[2],l.boxEnd[1]=l.boxStart[1]+(A[2]-l.boxStart[0])*Math.abs(E)))}}else l.boxEnabled?(o=l.boxStart[0]!==l.boxEnd[0],s=l.boxStart[1]!==l.boxEnd[1],o||s?(o&&(p(0,l.boxStart[0],l.boxEnd[0]),t.xaxis.autorange=!1),s&&(p(1,l.boxStart[1],l.boxEnd[1]),t.yaxis.autorange=!1),t.relayoutCallback()):t.glplot.setDirty(),l.boxEnabled=!1,l.boxInited=!1):l.boxInited&&(l.boxInited=!1);break;case"pan":l.boxEnabled=!1,l.boxInited=!1,e?(l.panning||(l.dragStart[0]=r,l.dragStart[1]=i),Math.abs(l.dragStart[0]-r)<d&&(r=l.dragStart[0]),Math.abs(l.dragStart[1]-i)<d&&(i=l.dragStart[1]),o=(g-r)*(A[2]-A[0])/(n.viewBox[2]-n.viewBox[0]),s=(h-i)*(A[3]-A[1])/(n.viewBox[3]-n.viewBox[1]),A[0]+=o,A[2]+=o,A[1]+=s,A[3]+=s,t.setRanges(A),l.panning=!0,l.lastInputTime=Date.now(),c(),t.cameraChanged(),t.handleAnnotations()):l.panning&&(l.panning=!1,t.relayoutCallback())}l.lastPos[0]=r,l.lastPos[1]=i}return l.mouseListener=r(e,u),e.addEventListener("touchstart",function(t){var n=o(t.changedTouches[0],e);u(0,n[0],n[1]),u(1,n[0],n[1]),t.preventDefault()},!!s&&{passive:!1}),e.addEventListener("touchmove",function(t){t.preventDefault();var n=o(t.changedTouches[0],e);u(1,n[0],n[1]),t.preventDefault()},!!s&&{passive:!1}),e.addEventListener("touchend",function(t){u(0,l.lastPos[0],l.lastPos[1]),t.preventDefault()},!!s&&{passive:!1}),l.wheelListener=i(e,function(e,r){if(!t.scrollZoom)return!1;var i=t.calcDataBox(),o=n.viewBox,a=l.lastPos[0],s=l.lastPos[1],A=Math.exp(5*r/(o[3]-o[1])),u=a/(o[2]-o[0])*(i[2]-i[0])+i[0],g=s/(o[3]-o[1])*(i[3]-i[1])+i[1];return i[0]=(i[0]-u)*A+u,i[2]=(i[2]-u)*A+u,i[1]=(i[1]-g)*A+g,i[3]=(i[3]-g)*A+g,t.setRanges(i),l.lastInputTime=Date.now(),c(),t.cameraChanged(),t.handleAnnotations(),t.relayoutCallback(),!0},!0),l}},{"../cartesian/constants":773,"has-passive-events":411,"mouse-change":435,"mouse-event-offset":436,"mouse-wheel":438}],804:[function(t,e,n){"use strict";var r=t("../cartesian/axes"),i=t("../../lib/str2rgbarray");function o(t){this.scene=t,this.gl=t.gl,this.pixelRatio=t.pixelRatio,this.screenBox=[0,0,1,1],this.viewBox=[0,0,1,1],this.dataBox=[-1,-1,1,1],this.borderLineEnable=[!1,!1,!1,!1],this.borderLineWidth=[1,1,1,1],this.borderLineColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.ticks=[[],[]],this.tickEnable=[!0,!0,!1,!1],this.tickPad=[15,15,15,15],this.tickAngle=[0,0,0,0],this.tickColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.tickMarkLength=[0,0,0,0],this.tickMarkWidth=[0,0,0,0],this.tickMarkColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.labels=["x","y"],this.labelEnable=[!0,!0,!1,!1],this.labelAngle=[0,Math.PI/2,0,3*Math.PI/2],this.labelPad=[15,15,15,15],this.labelSize=[12,12],this.labelFont=["sans-serif","sans-serif"],this.labelColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.title="",this.titleEnable=!0,this.titleCenter=[0,0,0,0],this.titleAngle=0,this.titleColor=[0,0,0,1],this.titleFont="sans-serif",this.titleSize=18,this.gridLineEnable=[!0,!0],this.gridLineColor=[[0,0,0,.5],[0,0,0,.5]],this.gridLineWidth=[1,1],this.zeroLineEnable=[!0,!0],this.zeroLineWidth=[1,1],this.zeroLineColor=[[0,0,0,1],[0,0,0,1]],this.borderColor=[0,0,0,0],this.backgroundColor=[0,0,0,0],this.static=this.scene.staticPlot}var a=o.prototype,s=["xaxis","yaxis"];a.merge=function(t){var e,n,r,o,a,A,l,c,u,g,h;for(this.titleEnable=!1,this.backgroundColor=i(t.plot_bgcolor),g=0;g<2;++g){var d=(e=s[g]).charAt(0);for(r=(n=t[this.scene[e]._name]).title.text===this.scene.fullLayout._dfltTitle[d]?"":n.title.text,h=0;h<=2;h+=2)this.labelEnable[g+h]=!1,this.labels[g+h]=r,this.labelColor[g+h]=i(n.title.font.color),this.labelFont[g+h]=n.title.font.family,this.labelSize[g+h]=n.title.font.size,this.labelPad[g+h]=this.getLabelPad(e,n),this.tickEnable[g+h]=!1,this.tickColor[g+h]=i((n.tickfont||{}).color),this.tickAngle[g+h]="auto"===n.tickangle?0:Math.PI*-n.tickangle/180,this.tickPad[g+h]=this.getTickPad(n),this.tickMarkLength[g+h]=0,this.tickMarkWidth[g+h]=n.tickwidth||0,this.tickMarkColor[g+h]=i(n.tickcolor),this.borderLineEnable[g+h]=!1,this.borderLineColor[g+h]=i(n.linecolor),this.borderLineWidth[g+h]=n.linewidth||0;l=this.hasSharedAxis(n),a=this.hasAxisInDfltPos(e,n)&&!l,A=this.hasAxisInAltrPos(e,n)&&!l,o=n.mirror||!1,c=l?-1!==String(o).indexOf("all"):!!o,u=l?"allticks"===o:-1!==String(o).indexOf("ticks"),a?this.labelEnable[g]=!0:A&&(this.labelEnable[g+2]=!0),a?this.tickEnable[g]=n.showticklabels:A&&(this.tickEnable[g+2]=n.showticklabels),(a||c)&&(this.borderLineEnable[g]=n.showline),(A||c)&&(this.borderLineEnable[g+2]=n.showline),(a||u)&&(this.tickMarkLength[g]=this.getTickMarkLength(n)),(A||u)&&(this.tickMarkLength[g+2]=this.getTickMarkLength(n)),this.gridLineEnable[g]=n.showgrid,this.gridLineColor[g]=i(n.gridcolor),this.gridLineWidth[g]=n.gridwidth,this.zeroLineEnable[g]=n.zeroline,this.zeroLineColor[g]=i(n.zerolinecolor),this.zeroLineWidth[g]=n.zerolinewidth}},a.hasSharedAxis=function(t){var e=this.scene,n=e.fullLayout._subplots.gl2d;return 0!==r.findSubplotsWithAxis(n,t).indexOf(e.id)},a.hasAxisInDfltPos=function(t,e){var n=e.side;return"xaxis"===t?"bottom"===n:"yaxis"===t?"left"===n:void 0},a.hasAxisInAltrPos=function(t,e){var n=e.side;return"xaxis"===t?"top"===n:"yaxis"===t?"right"===n:void 0},a.getLabelPad=function(t,e){var n=e.title.font.size,r=e.showticklabels;return"xaxis"===t?"top"===e.side?n*(1.5+(r?1:0))-10:n*(1.5+(r?.5:0))-10:"yaxis"===t?"right"===e.side?10+n*(1.5+(r?1:.5)):10+n*(1.5+(r?.5:0)):void 0},a.getTickPad=function(t){return"outside"===t.ticks?10+t.ticklen:15},a.getTickMarkLength=function(t){if(!t.ticks)return 0;var e=t.ticklen;return"inside"===t.ticks?-e:e},e.exports=function(t){return new o(t)}},{"../../lib/str2rgbarray":742,"../cartesian/axes":767}],805:[function(t,e,n){"use strict";var r=t("../../plot_api/edit_types").overrideAll,i=t("./scene2d"),o=t("../layout_attributes"),a=t("../../constants/xmlns_namespaces"),s=t("../cartesian/constants"),A=t("../cartesian"),l=t("../../components/fx/layout_attributes"),c=t("../get_data").getSubplotData;n.name="gl2d",n.attr=["xaxis","yaxis"],n.idRoot=["x","y"],n.idRegex=s.idRegex,n.attrRegex=s.attrRegex,n.attributes=t("../cartesian/attributes"),n.supplyLayoutDefaults=function(t,e,n){e._has("cartesian")||A.supplyLayoutDefaults(t,e,n)},n.layoutAttrOverrides=r(A.layoutAttributes,"plot","from-root"),n.baseLayoutAttrOverrides=r({plot_bgcolor:o.plot_bgcolor,hoverlabel:l.hoverlabel},"plot","nested"),n.plot=function(t){for(var e=t._fullLayout,n=t._fullData,r=e._subplots.gl2d,o=0;o<r.length;o++){var a=r[o],s=e._plots[a],A=c(n,"gl2d",a),l=s._scene2d;void 0===l&&(l=new i({id:a,graphDiv:t,container:t.querySelector(".gl-container"),staticPlot:t._context.staticPlot,plotGlPixelRatio:t._context.plotGlPixelRatio},e),s._scene2d=l),l.plot(A,t.calcdata,e,t.layout)}},n.clean=function(t,e,n,r){for(var i=r._subplots.gl2d||[],o=0;o<i.length;o++){var a=i[o],s=r._plots[a];s._scene2d&&0===c(t,"gl2d",a).length&&(s._scene2d.destroy(),delete r._plots[a])}A.clean.apply(this,arguments)},n.drawFramework=function(t){t._context.staticPlot||A.drawFramework(t)},n.toSVG=function(t){for(var e=t._fullLayout,n=e._subplots.gl2d,r=0;r<n.length;r++){var i=e._plots[n[r]]._scene2d,o=i.toImage("png");e._glimages.append("svg:image").attr({xmlns:a.svg,"xlink:href":o,x:0,y:0,width:"100%",height:"100%",preserveAspectRatio:"none"}),i.destroy()}},n.updateFx=function(t){for(var e=t._fullLayout,n=e._subplots.gl2d,r=0;r<n.length;r++)e._plots[n[r]]._scene2d.updateFx(e.dragmode)}},{"../../components/fx/layout_attributes":633,"../../constants/xmlns_namespaces":696,"../../plot_api/edit_types":750,"../cartesian":778,"../cartesian/attributes":765,"../cartesian/constants":773,"../get_data":802,"../layout_attributes":819,"./scene2d":806}],806:[function(t,e,n){"use strict";var r,i,o=t("../../registry"),a=t("../../plots/cartesian/axes"),s=t("../../components/fx"),A=t("gl-plot2d"),l=t("gl-spikes2d"),c=t("gl-select-box"),u=t("webgl-context"),g=t("./convert"),h=t("./camera"),d=t("../../lib/show_no_webgl_msg"),f=t("../cartesian/constraints"),p=f.enforce,I=f.clean,C=t("../cartesian/autorange").doAutoRange,m=["xaxis","yaxis"],y=t("../cartesian/constants").SUBPLOT_PATTERN;function E(t,e){this.container=t.container,this.graphDiv=t.graphDiv,this.pixelRatio=t.plotGlPixelRatio||window.devicePixelRatio,this.id=t.id,this.staticPlot=!!t.staticPlot,this.scrollZoom=this.graphDiv._context._scrollZoom.cartesian,this.fullData=null,this.updateRefs(e),this.makeFramework(),this.stopped||(this.glplotOptions=g(this),this.glplotOptions.merge(e),this.glplot=A(this.glplotOptions),this.camera=h(this),this.traces={},this.spikes=l(this.glplot),this.selectBox=c(this.glplot,{innerFill:!1,outerFill:!0}),this.lastButtonState=0,this.pickResult=null,this.isMouseOver=!0,this.stopped=!1,this.redraw=this.draw.bind(this),this.redraw())}e.exports=E;var v=E.prototype;v.makeFramework=function(){if(this.staticPlot){if(!(i||(r=document.createElement("canvas"),i=u({canvas:r,preserveDrawingBuffer:!1,premultipliedAlpha:!0,antialias:!0}))))throw new Error("Error creating static canvas/context for image server");this.canvas=r,this.gl=i}else{var t=this.container.querySelector(".gl-canvas-focus"),e=u({canvas:t,preserveDrawingBuffer:!0,premultipliedAlpha:!0});if(!e)return d(this),void(this.stopped=!0);this.canvas=t,this.gl=e}var n=this.canvas;n.style.width="100%",n.style.height="100%",n.style.position="absolute",n.style.top="0px",n.style.left="0px",n.style["pointer-events"]="none",this.updateSize(n),n.className+=" user-select-none";var o=this.svgContainer=document.createElementNS("http://www.w3.org/2000/svg","svg");o.style.position="absolute",o.style.top=o.style.left="0px",o.style.width=o.style.height="100%",o.style["z-index"]=20,o.style["pointer-events"]="none";var a=this.mouseContainer=document.createElement("div");a.style.position="absolute",a.style["pointer-events"]="auto",this.pickCanvas=this.container.querySelector(".gl-canvas-pick");var s=this.container;s.appendChild(o),s.appendChild(a);var A=this;a.addEventListener("mouseout",function(){A.isMouseOver=!1,A.unhover()}),a.addEventListener("mouseover",function(){A.isMouseOver=!0})},v.toImage=function(t){t||(t="png"),this.stopped=!0,this.staticPlot&&this.container.appendChild(r),this.updateSize(this.canvas);var e=this.glplot.gl,n=e.drawingBufferWidth,i=e.drawingBufferHeight;e.clearColor(1,1,1,0),e.clear(e.COLOR_BUFFER_BIT|e.DEPTH_BUFFER_BIT),this.glplot.setDirty(),this.glplot.draw(),e.bindFramebuffer(e.FRAMEBUFFER,null);var o=new Uint8Array(n*i*4);e.readPixels(0,0,n,i,e.RGBA,e.UNSIGNED_BYTE,o);for(var a=0,s=i-1;a<s;++a,--s)for(var A=0;A<n;++A)for(var l=0;l<4;++l){var c=o[4*(n*a+A)+l];o[4*(n*a+A)+l]=o[4*(n*s+A)+l],o[4*(n*s+A)+l]=c}var u=document.createElement("canvas");u.width=n,u.height=i;var g,h=u.getContext("2d"),d=h.createImageData(n,i);switch(d.data.set(o),h.putImageData(d,0,0),t){case"jpeg":g=u.toDataURL("image/jpeg");break;case"webp":g=u.toDataURL("image/webp");break;default:g=u.toDataURL("image/png")}return this.staticPlot&&this.container.removeChild(r),g},v.updateSize=function(t){t||(t=this.canvas);var e=this.pixelRatio,n=this.fullLayout,r=n.width,i=n.height,o=0|Math.ceil(e*r),a=0|Math.ceil(e*i);return t.width===o&&t.height===a||(t.width=o,t.height=a),t},v.computeTickMarks=function(){this.xaxis.setScale(),this.yaxis.setScale();for(var t=[a.calcTicks(this.xaxis),a.calcTicks(this.yaxis)],e=0;e<2;++e)for(var n=0;n<t[e].length;++n)t[e][n].text=t[e][n].text+"";return t},v.updateRefs=function(t){this.fullLayout=t;var e=this.id.match(y),n="xaxis"+e[1],r="yaxis"+e[2];this.xaxis=this.fullLayout[n],this.yaxis=this.fullLayout[r]},v.relayoutCallback=function(){var t=this.graphDiv,e=this.xaxis,n=this.yaxis,r=t.layout,i={},a=i[e._name+".range"]=e.range.slice(),s=i[n._name+".range"]=n.range.slice();i[e._name+".autorange"]=e.autorange,i[n._name+".autorange"]=n.autorange,o.call("_storeDirectGUIEdit",t.layout,t._fullLayout._preGUI,i);var A=r[e._name];A.range=a,A.autorange=e.autorange;var l=r[n._name];l.range=s,l.autorange=n.autorange,i.lastInputTime=this.camera.lastInputTime,t.emit("plotly_relayout",i)},v.cameraChanged=function(){var t=this.camera;this.glplot.setDataBox(this.calcDataBox());var e=this.computeTickMarks();(function(t,e){for(var n=0;n<2;++n){var r=t[n],i=e[n];if(r.length!==i.length)return!0;for(var o=0;o<r.length;++o)if(r[o].x!==i[o].x)return!0}return!1})(e,this.glplotOptions.ticks)&&(this.glplotOptions.ticks=e,this.glplotOptions.dataBox=t.dataBox,this.glplot.update(this.glplotOptions),this.handleAnnotations())},v.handleAnnotations=function(){for(var t=this.graphDiv,e=this.fullLayout.annotations,n=0;n<e.length;n++){var r=e[n];r.xref===this.xaxis._id&&r.yref===this.yaxis._id&&o.getComponentMethod("annotations","drawOne")(t,n)}},v.destroy=function(){if(this.glplot){var t=this.traces;t&&Object.keys(t).map(function(e){t[e].dispose(),delete t[e]}),this.glplot.dispose(),this.container.removeChild(this.svgContainer),this.container.removeChild(this.mouseContainer),this.fullData=null,this.glplot=null,this.stopped=!0,this.camera.mouseListener.enabled=!1,this.mouseContainer.removeEventListener("wheel",this.camera.wheelListener),this.camera=null}},v.plot=function(t,e,n){var r=this.glplot;this.updateRefs(n),this.xaxis.clearCalc(),this.yaxis.clearCalc(),this.updateTraces(t,e),this.updateFx(n.dragmode);var i=n.width,o=n.height;this.updateSize(this.canvas);var a=this.glplotOptions;a.merge(n),a.screenBox=[0,0,i,o];var s={_fullLayout:{_axisConstraintGroups:this.graphDiv._fullLayout._axisConstraintGroups,xaxis:this.xaxis,yaxis:this.yaxis}};I(s,this.xaxis),I(s,this.yaxis);var A,l,c=n._size,u=this.xaxis.domain,g=this.yaxis.domain;for(a.viewBox=[c.l+u[0]*c.w,c.b+g[0]*c.h,i-c.r-(1-u[1])*c.w,o-c.t-(1-g[1])*c.h],this.mouseContainer.style.width=c.w*(u[1]-u[0])+"px",this.mouseContainer.style.height=c.h*(g[1]-g[0])+"px",this.mouseContainer.height=c.h*(g[1]-g[0]),this.mouseContainer.style.left=c.l+u[0]*c.w+"px",this.mouseContainer.style.top=c.t+(1-g[1])*c.h+"px",l=0;l<2;++l)(A=this[m[l]])._length=a.viewBox[l+2]-a.viewBox[l],C(this.graphDiv,A),A.setScale();p(s),a.ticks=this.computeTickMarks(),a.dataBox=this.calcDataBox(),a.merge(n),r.update(a),this.glplot.draw()},v.calcDataBox=function(){var t=this.xaxis,e=this.yaxis,n=t.range,r=e.range,i=t.r2l,o=e.r2l;return[i(n[0]),o(r[0]),i(n[1]),o(r[1])]},v.setRanges=function(t){var e=this.xaxis,n=this.yaxis,r=e.l2r,i=n.l2r;e.range=[r(t[0]),r(t[2])],n.range=[i(t[1]),i(t[3])]},v.updateTraces=function(t,e){var n,r,i,o=Object.keys(this.traces);this.fullData=t;t:for(n=0;n<o.length;n++){var a=o[n],s=this.traces[a];for(r=0;r<t.length;r++)if((i=t[r]).uid===a&&i.type===s.type)continue t;s.dispose(),delete this.traces[a]}for(n=0;n<t.length;n++){i=t[n];var A=e[n],l=this.traces[i.uid];l?l.update(i,A):(l=i._module.plot(this,i,A),this.traces[i.uid]=l)}this.glplot.objects.sort(function(t,e){return t._trace.index-e._trace.index})},v.updateFx=function(t){"lasso"===t||"select"===t?(this.pickCanvas.style["pointer-events"]="none",this.mouseContainer.style["pointer-events"]="none"):(this.pickCanvas.style["pointer-events"]="auto",this.mouseContainer.style["pointer-events"]="auto"),this.mouseContainer.style.cursor="pan"===t?"move":"zoom"===t?"crosshair":null},v.emitPointAction=function(t,e){for(var n,r=t.trace.uid,i=t.pointIndex,o=0;o<this.fullData.length;o++)this.fullData[o].uid===r&&(n=this.fullData[o]);var a={x:t.traceCoord[0],y:t.traceCoord[1],curveNumber:n.index,pointNumber:i,data:n._input,fullData:this.fullData,xaxis:this.xaxis,yaxis:this.yaxis};s.appendArrayPointValue(a,n,i),this.graphDiv.emit(e,{points:[a]})},v.draw=function(){if(!this.stopped){requestAnimationFrame(this.redraw);var t=this.glplot,e=this.camera,n=e.mouseListener,r=1===this.lastButtonState&&0===n.buttons,i=this.fullLayout;this.lastButtonState=n.buttons,this.cameraChanged();var o,a=n.x*t.pixelRatio,A=this.canvas.height-t.pixelRatio*n.y;if(e.boxEnabled&&"zoom"===i.dragmode){this.selectBox.enabled=!0;for(var l=this.selectBox.selectBox=[Math.min(e.boxStart[0],e.boxEnd[0]),Math.min(e.boxStart[1],e.boxEnd[1]),Math.max(e.boxStart[0],e.boxEnd[0]),Math.max(e.boxStart[1],e.boxEnd[1])],c=0;c<2;c++)e.boxStart[c]===e.boxEnd[c]&&(l[c]=t.dataBox[c],l[c+2]=t.dataBox[c+2]);t.setDirty()}else if(!e.panning&&this.isMouseOver){this.selectBox.enabled=!1;var u=i._size,g=this.xaxis.domain,h=this.yaxis.domain,d=(o=t.pick(a/t.pixelRatio+u.l+g[0]*u.w,A/t.pixelRatio-(u.t+(1-h[1])*u.h)))&&o.object._trace.handlePick(o);if(d&&r&&this.emitPointAction(d,"plotly_click"),o&&"skip"!==o.object._trace.hoverinfo&&i.hovermode&&d&&(!this.lastPickResult||this.lastPickResult.traceUid!==d.trace.uid||this.lastPickResult.dataCoord[0]!==d.dataCoord[0]||this.lastPickResult.dataCoord[1]!==d.dataCoord[1])){var f=d;this.lastPickResult={traceUid:d.trace?d.trace.uid:null,dataCoord:d.dataCoord.slice()},this.spikes.update({center:o.dataCoord}),f.screenCoord=[((t.viewBox[2]-t.viewBox[0])*(o.dataCoord[0]-t.dataBox[0])/(t.dataBox[2]-t.dataBox[0])+t.viewBox[0])/t.pixelRatio,(this.canvas.height-(t.viewBox[3]-t.viewBox[1])*(o.dataCoord[1]-t.dataBox[1])/(t.dataBox[3]-t.dataBox[1])-t.viewBox[1])/t.pixelRatio],this.emitPointAction(d,"plotly_hover");var p=this.fullData[f.trace.index]||{},I=f.pointIndex,C=s.castHoverinfo(p,i,I);if(C&&"all"!==C){var m=C.split("+");-1===m.indexOf("x")&&(f.traceCoord[0]=void 0),-1===m.indexOf("y")&&(f.traceCoord[1]=void 0),-1===m.indexOf("z")&&(f.traceCoord[2]=void 0),-1===m.indexOf("text")&&(f.textLabel=void 0),-1===m.indexOf("name")&&(f.name=void 0)}s.loneHover({x:f.screenCoord[0],y:f.screenCoord[1],xLabel:this.hoverFormatter("xaxis",f.traceCoord[0]),yLabel:this.hoverFormatter("yaxis",f.traceCoord[1]),zLabel:f.traceCoord[2],text:f.textLabel,name:f.name,color:s.castHoverOption(p,I,"bgcolor")||f.color,borderColor:s.castHoverOption(p,I,"bordercolor"),fontFamily:s.castHoverOption(p,I,"font.family"),fontSize:s.castHoverOption(p,I,"font.size"),fontColor:s.castHoverOption(p,I,"font.color"),nameLength:s.castHoverOption(p,I,"namelength"),textAlign:s.castHoverOption(p,I,"align")},{container:this.svgContainer,gd:this.graphDiv})}}o||this.unhover(),t.draw()}},v.unhover=function(){this.lastPickResult&&(this.spikes.update({}),this.lastPickResult=null,this.graphDiv.emit("plotly_unhover"),s.loneUnhover(this.svgContainer))},v.hoverFormatter=function(t,e){if(void 0!==e){var n=this[t];return a.tickText(n,n.c2l(e),"hover").text}}},{"../../components/fx":632,"../../lib/show_no_webgl_msg":740,"../../plots/cartesian/axes":767,"../../registry":847,"../cartesian/autorange":766,"../cartesian/constants":773,"../cartesian/constraints":774,"./camera":803,"./convert":804,"gl-plot2d":287,"gl-select-box":299,"gl-spikes2d":308,"webgl-context":556}],807:[function(t,e,n){"use strict";var r=t("../../plot_api/edit_types").overrideAll,i=t("../../components/fx/layout_attributes"),o=t("./scene"),a=t("../get_data").getSubplotData,s=t("../../lib"),A=t("../../constants/xmlns_namespaces");n.name="gl3d",n.attr="scene",n.idRoot="scene",n.idRegex=n.attrRegex=s.counterRegex("scene"),n.attributes=t("./layout/attributes"),n.layoutAttributes=t("./layout/layout_attributes"),n.baseLayoutAttrOverrides=r({hoverlabel:i.hoverlabel},"plot","nested"),n.supplyLayoutDefaults=t("./layout/defaults"),n.plot=function(t){for(var e=t._fullLayout,n=t._fullData,r=e._subplots.gl3d,i=0;i<r.length;i++){var s=r[i],A=a(n,"gl3d",s),l=e[s],c=l.camera,u=l._scene;u||(u=new o({id:s,graphDiv:t,container:t.querySelector(".gl-container"),staticPlot:t._context.staticPlot,plotGlPixelRatio:t._context.plotGlPixelRatio,camera:c},e),l._scene=u),u.viewInitial||(u.viewInitial={up:{x:c.up.x,y:c.up.y,z:c.up.z},eye:{x:c.eye.x,y:c.eye.y,z:c.eye.z},center:{x:c.center.x,y:c.center.y,z:c.center.z}}),u.plot(A,e,t.layout)}},n.clean=function(t,e,n,r){for(var i=r._subplots.gl3d||[],o=0;o<i.length;o++){var a=i[o];!e[a]&&r[a]._scene&&(r[a]._scene.destroy(),r._infolayer&&r._infolayer.selectAll(".annotation-"+a).remove())}},n.toSVG=function(t){for(var e=t._fullLayout,n=e._subplots.gl3d,r=e._size,i=0;i<n.length;i++){var o=e[n[i]],a=o.domain,s=o._scene,l=s.toImage("png");e._glimages.append("svg:image").attr({xmlns:A.svg,"xlink:href":l,x:r.l+r.w*a.x[0],y:r.t+r.h*(1-a.y[1]),width:r.w*(a.x[1]-a.x[0]),height:r.h*(a.y[1]-a.y[0]),preserveAspectRatio:"none"}),s.destroy()}},n.cleanId=function(t){if(t.match(/^scene[0-9]*$/)){var e=t.substr(5);return"1"===e&&(e=""),"scene"+e}},n.updateFx=function(t){for(var e=t._fullLayout,n=e._subplots.gl3d,r=0;r<n.length;r++)e[n[r]]._scene.updateFx(e.dragmode,e.hovermode)}},{"../../components/fx/layout_attributes":633,"../../constants/xmlns_namespaces":696,"../../lib":719,"../../plot_api/edit_types":750,"../get_data":802,"./layout/attributes":808,"./layout/defaults":812,"./layout/layout_attributes":813,"./scene":817}],808:[function(t,e,n){"use strict";e.exports={scene:{valType:"subplotid",dflt:"scene",editType:"calc+clearAxisTypes"}}},{}],809:[function(t,e,n){"use strict";var r=t("../../../components/color"),i=t("../../cartesian/layout_attributes"),o=t("../../../lib/extend").extendFlat,a=t("../../../plot_api/edit_types").overrideAll;e.exports=a({visible:i.visible,showspikes:{valType:"boolean",dflt:!0},spikesides:{valType:"boolean",dflt:!0},spikethickness:{valType:"number",min:0,dflt:2},spikecolor:{valType:"color",dflt:r.defaultLine},showbackground:{valType:"boolean",dflt:!1},backgroundcolor:{valType:"color",dflt:"rgba(204, 204, 204, 0.5)"},showaxeslabels:{valType:"boolean",dflt:!0},color:i.color,categoryorder:i.categoryorder,categoryarray:i.categoryarray,title:i.title,type:o({},i.type,{values:["-","linear","log","date","category"]}),autorange:i.autorange,rangemode:i.rangemode,range:o({},i.range,{items:[{valType:"any",editType:"plot",impliedEdits:{"^autorange":!1}},{valType:"any",editType:"plot",impliedEdits:{"^autorange":!1}}],anim:!1}),tickmode:i.tickmode,nticks:i.nticks,tick0:i.tick0,dtick:i.dtick,tickvals:i.tickvals,ticktext:i.ticktext,ticks:i.ticks,mirror:i.mirror,ticklen:i.ticklen,tickwidth:i.tickwidth,tickcolor:i.tickcolor,showticklabels:i.showticklabels,tickfont:i.tickfont,tickangle:i.tickangle,tickprefix:i.tickprefix,showtickprefix:i.showtickprefix,ticksuffix:i.ticksuffix,showticksuffix:i.showticksuffix,showexponent:i.showexponent,exponentformat:i.exponentformat,separatethousands:i.separatethousands,tickformat:i.tickformat,tickformatstops:i.tickformatstops,hoverformat:i.hoverformat,showline:i.showline,linecolor:i.linecolor,linewidth:i.linewidth,showgrid:i.showgrid,gridcolor:o({},i.gridcolor,{dflt:"rgb(204, 204, 204)"}),gridwidth:i.gridwidth,zeroline:i.zeroline,zerolinecolor:i.zerolinecolor,zerolinewidth:i.zerolinewidth,_deprecated:{title:i._deprecated.title,titlefont:i._deprecated.titlefont}},"plot","from-root")},{"../../../components/color":593,"../../../lib/extend":710,"../../../plot_api/edit_types":750,"../../cartesian/layout_attributes":779}],810:[function(t,e,n){"use strict";var r=t("tinycolor2").mix,i=t("../../../lib"),o=t("../../../plot_api/plot_template"),a=t("./axis_attributes"),s=t("../../cartesian/type_defaults"),A=t("../../cartesian/axis_defaults"),l=["xaxis","yaxis","zaxis"];e.exports=function(t,e,n){var c,u;function g(t,e){return i.coerce(c,u,a,t,e)}for(var h=0;h<l.length;h++){var d=l[h];c=t[d]||{},(u=o.newContainer(e,d))._id=d[0]+n.scene,u._name=d,s(c,u,g,n),A(c,u,g,{font:n.font,letter:d[0],data:n.data,showGrid:!0,noTickson:!0,bgColor:n.bgColor,calendar:n.calendar},n.fullLayout),g("gridcolor",r(u.color,n.bgColor,13600/187).toRgbString()),g("title.text",d[0]),u.setScale=i.noop,g("showspikes")&&(g("spikesides"),g("spikethickness"),g("spikecolor",u.color)),g("showaxeslabels"),g("showbackground")&&g("backgroundcolor")}}},{"../../../lib":719,"../../../plot_api/plot_template":757,"../../cartesian/axis_defaults":769,"../../cartesian/type_defaults":790,"./axis_attributes":809,tinycolor2:537}],811:[function(t,e,n){"use strict";var r=t("../../../lib/str2rgbarray"),i=t("../../../lib"),o=["xaxis","yaxis","zaxis"];function a(){this.bounds=[[-10,-10,-10],[10,10,10]],this.ticks=[[],[],[]],this.tickEnable=[!0,!0,!0],this.tickFont=["sans-serif","sans-serif","sans-serif"],this.tickSize=[12,12,12],this.tickAngle=[0,0,0],this.tickColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.tickPad=[18,18,18],this.labels=["x","y","z"],this.labelEnable=[!0,!0,!0],this.labelFont=["Open Sans","Open Sans","Open Sans"],this.labelSize=[20,20,20],this.labelColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.labelPad=[30,30,30],this.lineEnable=[!0,!0,!0],this.lineMirror=[!1,!1,!1],this.lineWidth=[1,1,1],this.lineColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.lineTickEnable=[!0,!0,!0],this.lineTickMirror=[!1,!1,!1],this.lineTickLength=[10,10,10],this.lineTickWidth=[1,1,1],this.lineTickColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.gridEnable=[!0,!0,!0],this.gridWidth=[1,1,1],this.gridColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.zeroEnable=[!0,!0,!0],this.zeroLineColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.zeroLineWidth=[2,2,2],this.backgroundEnable=[!0,!0,!0],this.backgroundColor=[[.8,.8,.8,.5],[.8,.8,.8,.5],[.8,.8,.8,.5]],this._defaultTickPad=this.tickPad.slice(),this._defaultLabelPad=this.labelPad.slice(),this._defaultLineTickLength=this.lineTickLength.slice()}a.prototype.merge=function(t,e){for(var n=0;n<3;++n){var a=e[o[n]];a.visible?(this.labels[n]=t._meta?i.templateString(a.title.text,t._meta):a.title.text,"font"in a.title&&(a.title.font.color&&(this.labelColor[n]=r(a.title.font.color)),a.title.font.family&&(this.labelFont[n]=a.title.font.family),a.title.font.size&&(this.labelSize[n]=a.title.font.size)),"showline"in a&&(this.lineEnable[n]=a.showline),"linecolor"in a&&(this.lineColor[n]=r(a.linecolor)),"linewidth"in a&&(this.lineWidth[n]=a.linewidth),"showgrid"in a&&(this.gridEnable[n]=a.showgrid),"gridcolor"in a&&(this.gridColor[n]=r(a.gridcolor)),"gridwidth"in a&&(this.gridWidth[n]=a.gridwidth),"log"===a.type?this.zeroEnable[n]=!1:"zeroline"in a&&(this.zeroEnable[n]=a.zeroline),"zerolinecolor"in a&&(this.zeroLineColor[n]=r(a.zerolinecolor)),"zerolinewidth"in a&&(this.zeroLineWidth[n]=a.zerolinewidth),"ticks"in a&&a.ticks?this.lineTickEnable[n]=!0:this.lineTickEnable[n]=!1,"ticklen"in a&&(this.lineTickLength[n]=this._defaultLineTickLength[n]=a.ticklen),"tickcolor"in a&&(this.lineTickColor[n]=r(a.tickcolor)),"tickwidth"in a&&(this.lineTickWidth[n]=a.tickwidth),"tickangle"in a&&(this.tickAngle[n]="auto"===a.tickangle?-3600:Math.PI*-a.tickangle/180),"showticklabels"in a&&(this.tickEnable[n]=a.showticklabels),"tickfont"in a&&(a.tickfont.color&&(this.tickColor[n]=r(a.tickfont.color)),a.tickfont.family&&(this.tickFont[n]=a.tickfont.family),a.tickfont.size&&(this.tickSize[n]=a.tickfont.size)),"mirror"in a?-1!==["ticks","all","allticks"].indexOf(a.mirror)?(this.lineTickMirror[n]=!0,this.lineMirror[n]=!0):!0===a.mirror?(this.lineTickMirror[n]=!1,this.lineMirror[n]=!0):(this.lineTickMirror[n]=!1,this.lineMirror[n]=!1):this.lineMirror[n]=!1,"showbackground"in a&&!1!==a.showbackground?(this.backgroundEnable[n]=!0,this.backgroundColor[n]=r(a.backgroundcolor)):this.backgroundEnable[n]=!1):(this.tickEnable[n]=!1,this.labelEnable[n]=!1,this.lineEnable[n]=!1,this.lineTickEnable[n]=!1,this.gridEnable[n]=!1,this.zeroEnable[n]=!1,this.backgroundEnable[n]=!1)}},e.exports=function(t,e){var n=new a;return n.merge(t,e),n}},{"../../../lib":719,"../../../lib/str2rgbarray":742}],812:[function(t,e,n){"use strict";var r=t("../../../lib"),i=t("../../../components/color"),o=t("../../../registry"),a=t("../../subplot_defaults"),s=t("./axis_defaults"),A=t("./layout_attributes"),l=t("../../get_data").getSubplotData,c="gl3d";function u(t,e,n,r){for(var a=n("bgcolor"),A=i.combine(a,r.paper_bgcolor),u=["up","center","eye"],g=0;g<u.length;g++)n("camera."+u[g]+".x"),n("camera."+u[g]+".y"),n("camera."+u[g]+".z");n("camera.projection.type");var h=!!n("aspectratio.x")&&!!n("aspectratio.y")&&!!n("aspectratio.z"),d=n("aspectmode",h?"manual":"auto");h||(t.aspectratio=e.aspectratio={x:1,y:1,z:1},"manual"===d&&(e.aspectmode="auto"),t.aspectmode=e.aspectmode);var f=l(r.fullData,c,r.id);s(t,e,{font:r.font,scene:r.id,data:f,bgColor:A,calendar:r.calendar,fullLayout:r.fullLayout}),o.getComponentMethod("annotations3d","handleDefaults")(t,e,r);var p=r.getDfltFromLayout("dragmode");if(!1!==p&&!p)if(p="orbit",t.camera&&t.camera.up){var I=t.camera.up.x,C=t.camera.up.y,m=t.camera.up.z;0!==m&&(I&&C&&m?m/Math.sqrt(I*I+C*C+m*m)>.999&&(p="turntable"):p="turntable")}else p="turntable";n("dragmode",p),n("hovermode",r.getDfltFromLayout("hovermode"))}e.exports=function(t,e,n){var i=e._basePlotModules.length>1;a(t,e,n,{type:c,attributes:A,handleDefaults:u,fullLayout:e,font:e.font,fullData:n,getDfltFromLayout:function(e){if(!i)return r.validate(t[e],A[e])?t[e]:void 0},paper_bgcolor:e.paper_bgcolor,calendar:e.calendar})}},{"../../../components/color":593,"../../../lib":719,"../../../registry":847,"../../get_data":802,"../../subplot_defaults":842,"./axis_defaults":810,"./layout_attributes":813}],813:[function(t,e,n){"use strict";var r=t("./axis_attributes"),i=t("../../domain").attributes,o=t("../../../lib/extend").extendFlat,a=t("../../../lib").counterRegex;function s(t,e,n){return{x:{valType:"number",dflt:t,editType:"camera"},y:{valType:"number",dflt:e,editType:"camera"},z:{valType:"number",dflt:n,editType:"camera"},editType:"camera"}}e.exports={_arrayAttrRegexps:[a("scene",".annotations",!0)],bgcolor:{valType:"color",dflt:"rgba(0,0,0,0)",editType:"plot"},camera:{up:o(s(0,0,1),{}),center:o(s(0,0,0),{}),eye:o(s(1.25,1.25,1.25),{}),projection:{type:{valType:"enumerated",values:["perspective","orthographic"],dflt:"perspective",editType:"calc"},editType:"calc"},editType:"camera"},domain:i({name:"scene",editType:"plot"}),aspectmode:{valType:"enumerated",values:["auto","cube","data","manual"],dflt:"auto",editType:"plot",impliedEdits:{"aspectratio.x":void 0,"aspectratio.y":void 0,"aspectratio.z":void 0}},aspectratio:{x:{valType:"number",min:0,editType:"plot",impliedEdits:{"^aspectmode":"manual"}},y:{valType:"number",min:0,editType:"plot",impliedEdits:{"^aspectmode":"manual"}},z:{valType:"number",min:0,editType:"plot",impliedEdits:{"^aspectmode":"manual"}},editType:"plot",impliedEdits:{aspectmode:"manual"}},xaxis:r,yaxis:r,zaxis:r,dragmode:{valType:"enumerated",values:["orbit","turntable","zoom","pan",!1],editType:"plot"},hovermode:{valType:"enumerated",values:["closest",!1],dflt:"closest",editType:"modebar"},uirevision:{valType:"any",editType:"none"},editType:"plot",_deprecated:{cameraposition:{valType:"info_array",editType:"camera"}}}},{"../../../lib":719,"../../../lib/extend":710,"../../domain":792,"./axis_attributes":809}],814:[function(t,e,n){"use strict";var r=t("../../../lib/str2rgbarray"),i=["xaxis","yaxis","zaxis"];function o(){this.enabled=[!0,!0,!0],this.colors=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.drawSides=[!0,!0,!0],this.lineWidth=[1,1,1]}o.prototype.merge=function(t){for(var e=0;e<3;++e){var n=t[i[e]];n.visible?(this.enabled[e]=n.showspikes,this.colors[e]=r(n.spikecolor),this.drawSides[e]=n.spikesides,this.lineWidth[e]=n.spikethickness):(this.enabled[e]=!1,this.drawSides[e]=!1)}},e.exports=function(t){var e=new o;return e.merge(t),e}},{"../../../lib/str2rgbarray":742}],815:[function(t,e,n){"use strict";e.exports=function(t){for(var e=t.axesOptions,n=t.glplot.axesPixels,s=t.fullSceneLayout,A=[[],[],[]],l=0;l<3;++l){var c=s[o[l]];if(c._length=(n[l].hi-n[l].lo)*n[l].pixelsPerDataUnit/t.dataScale[l],Math.abs(c._length)===1/0||isNaN(c._length))A[l]=[];else{c._input_range=c.range.slice(),c.range[0]=n[l].lo/t.dataScale[l],c.range[1]=n[l].hi/t.dataScale[l],c._m=1/(t.dataScale[l]*n[l].pixelsPerDataUnit),c.range[0]===c.range[1]&&(c.range[0]-=1,c.range[1]+=1);var u=c.tickmode;if("auto"===c.tickmode){c.tickmode="linear";var g=c.nticks||i.constrain(c._length/40,4,9);r.autoTicks(c,Math.abs(c.range[1]-c.range[0])/g)}for(var h=r.calcTicks(c),d=0;d<h.length;++d)h[d].x=h[d].x*t.dataScale[l],"date"===c.type&&(h[d].text=h[d].text.replace(/\<br\>/g," "));A[l]=h,c.tickmode=u}}e.ticks=A;for(var l=0;l<3;++l){a[l]=.5*(t.glplot.bounds[0][l]+t.glplot.bounds[1][l]);for(var d=0;d<2;++d)e.bounds[d][l]=t.glplot.bounds[d][l]}t.contourLevels=function(t){for(var e=new Array(3),n=0;n<3;++n){for(var r=t[n],i=new Array(r.length),o=0;o<r.length;++o)i[o]=r[o].x;e[n]=i}return e}(A)};var r=t("../../cartesian/axes"),i=t("../../../lib"),o=["xaxis","yaxis","zaxis"],a=[0,0,0]},{"../../../lib":719,"../../cartesian/axes":767}],816:[function(t,e,n){"use strict";function r(t,e){var n,r,i=[0,0,0,0];for(n=0;n<4;++n)for(r=0;r<4;++r)i[r]+=t[4*n+r]*e[n];return i}e.exports=function(t,e){return r(t.projection,r(t.view,r(t.model,[e[0],e[1],e[2],1])))}},{}],817:[function(t,e,n){"use strict";var r,i,o=t("gl-plot3d").createCamera,a=t("gl-plot3d").createScene,s=t("webgl-context"),A=t("has-passive-events"),l=t("../../registry"),c=t("../../lib"),u=t("../../plots/cartesian/axes"),g=t("../../components/fx"),h=t("../../lib/str2rgbarray"),d=t("../../lib/show_no_webgl_msg"),f=t("./project"),p=t("./layout/convert"),I=t("./layout/spikes"),C=t("./layout/tick_marks");function m(t,e,n,o){if(t.initializeGLCamera(),!function(t,e,n,o,A){var l={canvas:o,gl:A,container:t.container,axes:t.axesOptions,spikes:t.spikeOptions,pickRadius:10,snapToData:!0,autoScale:!0,autoBounds:!1,cameraObject:e,pixelRatio:n};if(t.staticMode){if(!(i||(r=document.createElement("canvas"),i=s({canvas:r,preserveDrawingBuffer:!0,premultipliedAlpha:!0,antialias:!0}))))throw new Error("error creating static canvas/context for image server");l.pixelRatio=t.pixelRatio,l.gl=i,l.canvas=r}try{t.glplot=a(l)}catch(t){return!1}return!0}(t,t.camera,e,n,o))return d(t);var l=t.graphDiv,h=function(t){if(!1!==t.fullSceneLayout.dragmode){var e={};e[t.id+".camera"]=b(t.camera),t.saveCamera(l.layout),t.graphDiv.emit("plotly_relayout",e)}};return t.glplot.canvas.addEventListener("mouseup",function(){h(t)}),t.glplot.canvas.addEventListener("wheel",function(){l._context._scrollZoom.gl3d&&h(t)},!!A&&{passive:!1}),t.glplot.canvas.addEventListener("mousemove",function(){if(!1!==t.fullSceneLayout.dragmode&&0!==t.camera.mouseListener.buttons){var e={};e[t.id+".camera"]=b(t.camera),t.graphDiv.emit("plotly_relayouting",e)}}),t.staticMode||t.glplot.canvas.addEventListener("webglcontextlost",function(e){l&&l.emit&&l.emit("plotly_webglcontextlost",{event:e,layer:t.id})},!1),t.glplot.camera=t.camera,t.glplot.oncontextloss=function(){t.recoverContext()},t.glplot.onrender=function(t){var e,n=t.graphDiv,r=t.svgContainer,i=t.container.getBoundingClientRect(),o=i.width,a=i.height;r.setAttributeNS(null,"viewBox","0 0 "+o+" "+a),r.setAttributeNS(null,"width",o),r.setAttributeNS(null,"height",a),C(t),t.glplot.axes.update(t.axesOptions);for(var s,A=Object.keys(t.traces),l=null,h=t.glplot.selection,d=0;d<A.length;++d)"skip"!==(e=t.traces[A[d]]).data.hoverinfo&&e.handlePick(h)&&(l=e),e.setContourLevels&&e.setContourLevels();function p(e,n){var r=t.fullSceneLayout[e];return u.tickText(r,r.d2l(n),"hover").text}if(null!==l){var I=f(t.glplot.cameraParams,h.dataCoordinate);e=l.data;var m,y=n._fullData[e.index],E=h.index,v={xLabel:p("xaxis",h.traceCoordinate[0]),yLabel:p("yaxis",h.traceCoordinate[1]),zLabel:p("zaxis",h.traceCoordinate[2])},B=g.castHoverinfo(y,t.fullLayout,E),b=(B||"").split("+"),Q=B&&"all"===B;y.hovertemplate||Q||(-1===b.indexOf("x")&&(v.xLabel=void 0),-1===b.indexOf("y")&&(v.yLabel=void 0),-1===b.indexOf("z")&&(v.zLabel=void 0),-1===b.indexOf("text")&&(h.textLabel=void 0),-1===b.indexOf("name")&&(l.name=void 0));var w=[];"cone"===e.type||"streamtube"===e.type?(v.uLabel=p("xaxis",h.traceCoordinate[3]),(Q||-1!==b.indexOf("u"))&&w.push("u: "+v.uLabel),v.vLabel=p("yaxis",h.traceCoordinate[4]),(Q||-1!==b.indexOf("v"))&&w.push("v: "+v.vLabel),v.wLabel=p("zaxis",h.traceCoordinate[5]),(Q||-1!==b.indexOf("w"))&&w.push("w: "+v.wLabel),v.normLabel=h.traceCoordinate[6].toPrecision(3),(Q||-1!==b.indexOf("norm"))&&w.push("norm: "+v.normLabel),"streamtube"===e.type&&(v.divergenceLabel=h.traceCoordinate[7].toPrecision(3),(Q||-1!==b.indexOf("divergence"))&&w.push("divergence: "+v.divergenceLabel)),h.textLabel&&w.push(h.textLabel),m=w.join("<br>")):"isosurface"===e.type||"volume"===e.type?(v.valueLabel=u.tickText(t.mockAxis,t.mockAxis.d2l(h.traceCoordinate[3]),"hover").text,w.push("value: "+v.valueLabel),h.textLabel&&w.push(h.textLabel),m=w.join("<br>")):m=h.textLabel;var x={x:h.traceCoordinate[0],y:h.traceCoordinate[1],z:h.traceCoordinate[2],data:y._input,fullData:y,curveNumber:y.index,pointNumber:E};g.appendArrayPointValue(x,y,E),e._module.eventData&&(x=y._module.eventData(x,h,y,{},E));var _={points:[x]};t.fullSceneLayout.hovermode&&g.loneHover({trace:y,x:(.5+.5*I[0]/I[3])*o,y:(.5-.5*I[1]/I[3])*a,xLabel:v.xLabel,yLabel:v.yLabel,zLabel:v.zLabel,text:m,name:l.name,color:g.castHoverOption(y,E,"bgcolor")||l.color,borderColor:g.castHoverOption(y,E,"bordercolor"),fontFamily:g.castHoverOption(y,E,"font.family"),fontSize:g.castHoverOption(y,E,"font.size"),fontColor:g.castHoverOption(y,E,"font.color"),nameLength:g.castHoverOption(y,E,"namelength"),textAlign:g.castHoverOption(y,E,"align"),hovertemplate:c.castOption(y,E,"hovertemplate"),hovertemplateLabels:c.extendFlat({},x,v),eventData:[x]},{container:r,gd:n}),h.buttons&&h.distance<5?n.emit("plotly_click",_):n.emit("plotly_hover",_),s=_}else g.loneUnhover(r),n.emit("plotly_unhover",s);t.drawAnnotations(t)}.bind(null,t),t.traces={},t.make4thDimension(),!0}function y(t,e){var n=document.createElement("div"),r=t.container;this.graphDiv=t.graphDiv;var i=document.createElementNS("http://www.w3.org/2000/svg","svg");i.style.position="absolute",i.style.top=i.style.left="0px",i.style.width=i.style.height="100%",i.style["z-index"]=20,i.style["pointer-events"]="none",n.appendChild(i),this.svgContainer=i,n.id=t.id,n.style.position="absolute",n.style.top=n.style.left="0px",n.style.width=n.style.height="100%",r.appendChild(n),this.fullLayout=e,this.id=t.id||"scene",this.fullSceneLayout=e[this.id],this.plotArgs=[[],{},{}],this.axesOptions=p(e,e[this.id]),this.spikeOptions=I(e[this.id]),this.container=n,this.staticMode=!!t.staticPlot,this.pixelRatio=this.pixelRatio||t.plotGlPixelRatio||2,this.dataScale=[1,1,1],this.contourLevels=[[],[],[]],this.convertAnnotations=l.getComponentMethod("annotations3d","convert"),this.drawAnnotations=l.getComponentMethod("annotations3d","draw"),m(this,this.pixelRatio)}var E=y.prototype;E.initializeGLCamera=function(){var t=this.fullSceneLayout.camera,e="orthographic"===t.projection.type;this.camera=o(this.container,{center:[t.center.x,t.center.y,t.center.z],eye:[t.eye.x,t.eye.y,t.eye.z],up:[t.up.x,t.up.y,t.up.z],_ortho:e,zoomMin:.01,zoomMax:100,mode:"orbit"})},E.recoverContext=function(){var t=this,e=this.glplot.gl,n=this.glplot.canvas,r=this.glplot.camera,i=this.glplot.pixelRatio;this.glplot.dispose(),requestAnimationFrame(function o(){e.isContextLost()?requestAnimationFrame(o):m(t,r,i,n)?t.plot.apply(t,t.plotArgs):c.error("Catastrophic and unrecoverable WebGL error. Context lost.")})};var v=["xaxis","yaxis","zaxis"];function B(t,e,n){for(var r=t.fullSceneLayout,i=0;i<3;i++){var o=v[i],a=o.charAt(0),s=r[o],A=e[a],l=e[a+"calendar"],u=e["_"+a+"length"];if(c.isArrayOrTypedArray(A))for(var g,h=0;h<(u||A.length);h++)if(c.isArrayOrTypedArray(A[h]))for(var d=0;d<A[h].length;++d)g=s.d2l(A[h][d],0,l),!isNaN(g)&&isFinite(g)&&(n[0][i]=Math.min(n[0][i],g),n[1][i]=Math.max(n[1][i],g));else g=s.d2l(A[h],0,l),!isNaN(g)&&isFinite(g)&&(n[0][i]=Math.min(n[0][i],g),n[1][i]=Math.max(n[1][i],g));else n[0][i]=Math.min(n[0][i],0),n[1][i]=Math.max(n[1][i],u-1)}}function b(t){return{up:{x:t.up[0],y:t.up[1],z:t.up[2]},center:{x:t.center[0],y:t.center[1],z:t.center[2]},eye:{x:t.eye[0],y:t.eye[1],z:t.eye[2]},projection:{type:!0===t._ortho?"orthographic":"perspective"}}}E.plot=function(t,e,n){if(this.plotArgs=[t,e,n],!this.glplot.contextLost){var r,i,o,a,s,A,l=e[this.id],c=n[this.id];l.bgcolor?this.glplot.clearColor=h(l.bgcolor):this.glplot.clearColor=[0,0,0,0],this.glplot.snapToData=!0,this.fullLayout=e,this.fullSceneLayout=l,this.glplotLayout=l,this.axesOptions.merge(e,l),this.spikeOptions.merge(l),this.setCamera(l.camera),this.updateFx(l.dragmode,l.hovermode),this.camera.enableWheel=this.graphDiv._context._scrollZoom.gl3d,this.glplot.update({}),this.setConvert(s),t?Array.isArray(t)||(t=[t]):t=[];var u=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]];for(o=0;o<t.length;++o)!0===(r=t[o]).visible&&0!==r._length&&B(this,r,u);!function(t,e){for(var n=t.fullSceneLayout,r=n.annotations||[],i=0;i<3;i++)for(var o=v[i],a=o.charAt(0),s=n[o],A=0;A<r.length;A++){var l=r[A];if(l.visible){var c=s.r2l(l[a]);!isNaN(c)&&isFinite(c)&&(e[0][i]=Math.min(e[0][i],c),e[1][i]=Math.max(e[1][i],c))}}}(this,u);var g=[1,1,1];for(a=0;a<3;++a)u[1][a]===u[0][a]?g[a]=1:g[a]=1/(u[1][a]-u[0][a]);for(this.dataScale=g,this.convertAnnotations(this),o=0;o<t.length;++o)!0===(r=t[o]).visible&&0!==r._length&&((i=this.traces[r.uid])?i.data.type===r.type?i.update(r):(i.dispose(),i=r._module.plot(this,r),this.traces[r.uid]=i):(i=r._module.plot(this,r),this.traces[r.uid]=i),i.name=r.name);var d=Object.keys(this.traces);t:for(o=0;o<d.length;++o){for(a=0;a<t.length;++a)if(t[a].uid===d[o]&&!0===t[a].visible&&0!==t[a]._length)continue t;(i=this.traces[d[o]]).dispose(),delete this.traces[d[o]]}this.glplot.objects.sort(function(t,e){return t._trace.data.index-e._trace.data.index});var f=[[0,0,0],[0,0,0]],p=[],I={};for(o=0;o<3;++o){if((A=(s=l[v[o]]).type)in I?(I[A].acc*=g[o],I[A].count+=1):I[A]={acc:g[o],count:1},s.autorange){f[0][o]=1/0,f[1][o]=-1/0;var C=this.glplot.objects,m=this.fullSceneLayout.annotations||[],y=s._name.charAt(0);for(a=0;a<C.length;a++){var E=C[a],b=E.bounds,Q=E._trace.data._pad||0;"ErrorBars"===E.constructor.name&&s._lowerLogErrorBound?f[0][o]=Math.min(f[0][o],s._lowerLogErrorBound):f[0][o]=Math.min(f[0][o],b[0][o]/g[o]-Q),f[1][o]=Math.max(f[1][o],b[1][o]/g[o]+Q)}for(a=0;a<m.length;a++){var w=m[a];if(w.visible){var x=s.r2l(w[y]);f[0][o]=Math.min(f[0][o],x),f[1][o]=Math.max(f[1][o],x)}}if("rangemode"in s&&"tozero"===s.rangemode&&(f[0][o]=Math.min(f[0][o],0),f[1][o]=Math.max(f[1][o],0)),f[0][o]>f[1][o])f[0][o]=-1,f[1][o]=1;else{var _=f[1][o]-f[0][o];f[0][o]-=_/32,f[1][o]+=_/32}if("reversed"===s.autorange){var D=f[0][o];f[0][o]=f[1][o],f[1][o]=D}}else{var M=s.range;f[0][o]=s.r2l(M[0]),f[1][o]=s.r2l(M[1])}f[0][o]===f[1][o]&&(f[0][o]-=1,f[1][o]+=1),p[o]=f[1][o]-f[0][o],this.glplot.bounds[0][o]=f[0][o]*g[o],this.glplot.bounds[1][o]=f[1][o]*g[o]}var k,S=[1,1,1];for(o=0;o<3;++o){var L=I[A=(s=l[v[o]]).type];S[o]=Math.pow(L.acc,1/L.count)/g[o]}if("auto"===l.aspectmode)k=Math.max.apply(null,S)/Math.min.apply(null,S)<=4?S:[1,1,1];else if("cube"===l.aspectmode)k=[1,1,1];else if("data"===l.aspectmode)k=S;else{if("manual"!==l.aspectmode)throw new Error("scene.js aspectRatio was not one of the enumerated types");var R=l.aspectratio;k=[R.x,R.y,R.z]}l.aspectratio.x=c.aspectratio.x=k[0],l.aspectratio.y=c.aspectratio.y=k[1],l.aspectratio.z=c.aspectratio.z=k[2],this.glplot.aspect=k;var N=l.domain||null,F=e._size||null;if(N&&F){var T=this.container.style;T.position="absolute",T.left=F.l+N.x[0]*F.w+"px",T.top=F.t+(1-N.y[1])*F.h+"px",T.width=F.w*(N.x[1]-N.x[0])+"px",T.height=F.h*(N.y[1]-N.y[0])+"px"}this.glplot.redraw()}},E.destroy=function(){this.glplot&&(this.camera.mouseListener.enabled=!1,this.container.removeEventListener("wheel",this.camera.wheelListener),this.camera=this.glplot.camera=null,this.glplot.dispose(),this.container.parentNode.removeChild(this.container),this.glplot=null)},E.getCamera=function(){return this.glplot.camera.view.recalcMatrix(this.camera.view.lastT()),b(this.glplot.camera)},E.setCamera=function(t){var e;this.glplot.camera.lookAt.apply(this,[[(e=t).eye.x,e.eye.y,e.eye.z],[e.center.x,e.center.y,e.center.z],[e.up.x,e.up.y,e.up.z]]);var n="orthographic"===t.projection.type;if(n!==this.glplot.camera._ortho){this.glplot.redraw();var r=this.glplot.pixelRatio,i=this.glplot.clearColor;this.glplot.gl.clearColor(i[0],i[1],i[2],i[3]),this.glplot.gl.clear(this.glplot.gl.DEPTH_BUFFER_BIT|this.glplot.gl.COLOR_BUFFER_BIT),this.glplot.dispose(),m(this,r),this.glplot.camera._ortho=n}},E.saveCamera=function(t){var e=this.fullLayout,n=this.getCamera(),r=c.nestedProperty(t,this.id+".camera"),i=r.get(),o=!1;function a(t,e,n,r){var i=["up","center","eye"],o=["x","y","z"];return e[i[n]]&&t[i[n]][o[r]]===e[i[n]][o[r]]}if(void 0===i)o=!0;else{for(var s=0;s<3;s++)for(var A=0;A<3;A++)if(!a(n,i,s,A)){o=!0;break}(!i.projection||n.projection&&n.projection.type!==i.projection.type)&&(o=!0)}if(o){var u={};u[this.id+".camera"]=i,l.call("_storeDirectGUIEdit",t,e._preGUI,u),r.set(n),c.nestedProperty(e,this.id+".camera").set(n)}return o},E.updateFx=function(t,e){var n=this.camera;if(n)if("orbit"===t)n.mode="orbit",n.keyBindingMode="rotate";else if("turntable"===t){n.up=[0,0,1],n.mode="turntable",n.keyBindingMode="rotate";var r=this.graphDiv,i=r._fullLayout,o=this.fullSceneLayout.camera,a=o.up.x,s=o.up.y,A=o.up.z;if(A/Math.sqrt(a*a+s*s+A*A)<.999){var u=this.id+".camera.up",g={x:0,y:0,z:1},h={};h[u]=g;var d=r.layout;l.call("_storeDirectGUIEdit",d,i._preGUI,h),o.up=g,c.nestedProperty(d,u).set(g)}}else n.keyBindingMode=t;this.fullSceneLayout.hovermode=e},E.toImage=function(t){t||(t="png"),this.staticMode&&this.container.appendChild(r),this.glplot.redraw();var e=this.glplot.gl,n=e.drawingBufferWidth,i=e.drawingBufferHeight;e.bindFramebuffer(e.FRAMEBUFFER,null);var o=new Uint8Array(n*i*4);e.readPixels(0,0,n,i,e.RGBA,e.UNSIGNED_BYTE,o);for(var a=0,s=i-1;a<s;++a,--s)for(var A=0;A<n;++A)for(var l=0;l<4;++l){var c=o[4*(n*a+A)+l];o[4*(n*a+A)+l]=o[4*(n*s+A)+l],o[4*(n*s+A)+l]=c}var u=document.createElement("canvas");u.width=n,u.height=i;var g,h=u.getContext("2d"),d=h.createImageData(n,i);switch(d.data.set(o),h.putImageData(d,0,0),t){case"jpeg":g=u.toDataURL("image/jpeg");break;case"webp":g=u.toDataURL("image/webp");break;default:g=u.toDataURL("image/png")}return this.staticMode&&this.container.removeChild(r),g},E.setConvert=function(){for(var t=0;t<3;t++){var e=this.fullSceneLayout[v[t]];u.setConvert(e,this.fullLayout),e.setScale=c.noop}},E.make4thDimension=function(){var t=this.graphDiv._fullLayout;this.mockAxis={type:"linear",showexponent:"all",exponentformat:"B"},u.setConvert(this.mockAxis,t)},e.exports=y},{"../../components/fx":632,"../../lib":719,"../../lib/show_no_webgl_msg":740,"../../lib/str2rgbarray":742,"../../plots/cartesian/axes":767,"../../registry":847,"./layout/convert":811,"./layout/spikes":814,"./layout/tick_marks":815,"./project":816,"gl-plot3d":290,"has-passive-events":411,"webgl-context":556}],818:[function(t,e,n){"use strict";e.exports=function(t,e,n,r){r=r||t.length;for(var i=new Array(r),o=0;o<r;o++)i[o]=[t[o],e[o],n[o]];return i}},{}],819:[function(t,e,n){"use strict";var r=t("./font_attributes"),i=t("./animation_attributes"),o=t("../components/color/attributes"),a=t("./pad_attributes"),s=t("../lib/extend").extendFlat,A=r({editType:"calc"});A.family.dflt='"Open Sans", verdana, arial, sans-serif',A.size.dflt=12,A.color.dflt=o.defaultLine,e.exports={font:A,title:{text:{valType:"string",editType:"layoutstyle"},font:r({editType:"layoutstyle"}),xref:{valType:"enumerated",dflt:"container",values:["container","paper"],editType:"layoutstyle"},yref:{valType:"enumerated",dflt:"container",values:["container","paper"],editType:"layoutstyle"},x:{valType:"number",min:0,max:1,dflt:.5,editType:"layoutstyle"},y:{valType:"number",min:0,max:1,dflt:"auto",editType:"layoutstyle"},xanchor:{valType:"enumerated",dflt:"auto",values:["auto","left","center","right"],editType:"layoutstyle"},yanchor:{valType:"enumerated",dflt:"auto",values:["auto","top","middle","bottom"],editType:"layoutstyle"},pad:s(a({editType:"layoutstyle"}),{}),editType:"layoutstyle"},autosize:{valType:"boolean",dflt:!1,editType:"none"},width:{valType:"number",min:10,dflt:700,editType:"plot"},height:{valType:"number",min:10,dflt:450,editType:"plot"},margin:{l:{valType:"number",min:0,dflt:80,editType:"plot"},r:{valType:"number",min:0,dflt:80,editType:"plot"},t:{valType:"number",min:0,dflt:100,editType:"plot"},b:{valType:"number",min:0,dflt:80,editType:"plot"},pad:{valType:"number",min:0,dflt:0,editType:"plot"},autoexpand:{valType:"boolean",dflt:!0,editType:"plot"},editType:"plot"},paper_bgcolor:{valType:"color",dflt:o.background,editType:"plot"},plot_bgcolor:{valType:"color",dflt:o.background,editType:"layoutstyle"},separators:{valType:"string",editType:"plot"},hidesources:{valType:"boolean",dflt:!1,editType:"plot"},showlegend:{valType:"boolean",editType:"legend"},colorway:{valType:"colorlist",dflt:o.defaults,editType:"calc"},datarevision:{valType:"any",editType:"calc"},uirevision:{valType:"any",editType:"none"},editrevision:{valType:"any",editType:"none"},selectionrevision:{valType:"any",editType:"none"},template:{valType:"any",editType:"calc"},modebar:{orientation:{valType:"enumerated",values:["v","h"],dflt:"h",editType:"modebar"},bgcolor:{valType:"color",editType:"modebar"},color:{valType:"color",editType:"modebar"},activecolor:{valType:"color",editType:"modebar"},uirevision:{valType:"any",editType:"none"},editType:"modebar"},meta:{valType:"any",arrayOk:!0,editType:"plot"},transition:s({},i.transition,{editType:"none"}),_deprecated:{title:{valType:"string",editType:"layoutstyle"},titlefont:r({editType:"layoutstyle"})}}},{"../components/color/attributes":592,"../lib/extend":710,"./animation_attributes":762,"./font_attributes":793,"./pad_attributes":827}],820:[function(t,e,n){"use strict";var r={"open-street-map":{id:"osm",version:8,sources:{"plotly-osm-tiles":{type:"raster",attribution:'<a href="http://www.openstreetmap.org/about/" target="_blank">© OpenStreetMap</a>',tiles:["https://a.tile.openstreetmap.org/{z}/{x}/{y}.png","https://b.tile.openstreetmap.org/{z}/{x}/{y}.png"],tileSize:256}},layers:[{id:"plotly-osm-tiles",type:"raster",source:"plotly-osm-tiles",minzoom:0,maxzoom:22}]},"white-bg":{id:"white-bg",version:8,sources:{},layers:[{id:"white-bg",type:"background",paint:{"background-color":"#FFFFFF"},minzoom:0,maxzoom:22}]},"carto-positron":{id:"carto-positron",version:8,sources:{"plotly-carto-positron":{type:"raster",attribution:'<a href="https://carto.com/" target="_blank">© CARTO</a>',tiles:["https://cartodb-basemaps-c.global.ssl.fastly.net/light_all/{z}/{x}/{y}.png"],tileSize:256}},layers:[{id:"plotly-carto-positron",type:"raster",source:"plotly-carto-positron",minzoom:0,maxzoom:22}]},"carto-darkmatter":{id:"carto-darkmatter",version:8,sources:{"plotly-carto-darkmatter":{type:"raster",attribution:'<a href="https://carto.com/" target="_blank">© CARTO</a>',tiles:["https://cartodb-basemaps-c.global.ssl.fastly.net/dark_all/{z}/{x}/{y}.png"],tileSize:256}},layers:[{id:"plotly-carto-darkmatter",type:"raster",source:"plotly-carto-darkmatter",minzoom:0,maxzoom:22}]},"stamen-terrain":{id:"stamen-terrain",version:8,sources:{"plotly-stamen-terrain":{type:"raster",attribution:'Map tiles by <a href="http://stamen.com">Stamen Design</a>, under <a href="http://creativecommons.org/licenses/by/3.0">CC BY 3.0</a> | Data by <a href="http://openstreetmap.org">OpenStreetMap</a>, under <a href="http://www.openstreetmap.org/copyright">ODbL</a>.',tiles:["https://stamen-tiles.a.ssl.fastly.net/terrain/{z}/{x}/{y}.png"],tileSize:256}},layers:[{id:"plotly-stamen-terrain",type:"raster",source:"plotly-stamen-terrain",minzoom:0,maxzoom:22}]},"stamen-toner":{id:"stamen-toner",version:8,sources:{"plotly-stamen-toner":{type:"raster",attribution:'Map tiles by <a href="http://stamen.com">Stamen Design</a>, under <a href="http://creativecommons.org/licenses/by/3.0">CC BY 3.0</a> | Data by <a href="http://openstreetmap.org">OpenStreetMap</a>, under <a href="http://www.openstreetmap.org/copyright">ODbL</a>.',tiles:["https://stamen-tiles.a.ssl.fastly.net/toner/{z}/{x}/{y}.png"],tileSize:256}},layers:[{id:"plotly-stamen-toner",type:"raster",source:"plotly-stamen-toner",minzoom:0,maxzoom:22}]},"stamen-watercolor":{id:"stamen-watercolor",version:8,sources:{"plotly-stamen-watercolor":{type:"raster",attribution:'Map tiles by <a href="http://stamen.com">Stamen Design</a>, under <a href="http://creativecommons.org/licenses/by/3.0">CC BY 3.0</a> | Data by <a href="http://openstreetmap.org">OpenStreetMap</a>, under <a href="http://creativecommons.org/licenses/by-sa/3.0">CC BY SA</a>.',tiles:["https://stamen-tiles.a.ssl.fastly.net/watercolor/{z}/{x}/{y}.png"],tileSize:256}},layers:[{id:"plotly-stamen-watercolor",type:"raster",source:"plotly-stamen-watercolor",minzoom:0,maxzoom:22}]}},i=Object.keys(r);e.exports={requiredVersion:"1.1.1",styleUrlPrefix:"mapbox://styles/mapbox/",styleUrlSuffix:"v9",styleValuesMapbox:["basic","streets","outdoors","light","dark","satellite","satellite-streets"],styleValueDflt:"basic",stylesNonMapbox:r,styleValuesNonMapbox:i,traceLayerPrefix:"plotly-trace-layer-",layoutLayerPrefix:"plotly-layout-layer-",wrongVersionErrorMsg:["Your custom plotly.js bundle is not using the correct mapbox-gl version","Please install mapbox-gl@1.1.1."].join("\n"),noAccessTokenErrorMsg:["Missing Mapbox access token.","Mapbox trace type require a Mapbox access token to be registered.","For example:","  Plotly.plot(gd, data, layout, { mapboxAccessToken: 'my-access-token' });","More info here: https://www.mapbox.com/help/define-access-token/"].join("\n"),missingStyleErrorMsg:["No valid mapbox style found, please set `mapbox.style` to one of:",i.join(", "),"or register a Mapbox access token to use a Mapbox-served style."].join("\n"),multipleTokensErrorMsg:["Set multiple mapbox access token across different mapbox subplot,","using first token found as mapbox-gl does not allow multipleaccess tokens on the same page."].join("\n"),mapOnErrorMsg:"Mapbox error.",mapboxLogo:{path0:"m 10.5,1.24 c -5.11,0 -9.25,4.15 -9.25,9.25 0,5.1 4.15,9.25 9.25,9.25 5.1,0 9.25,-4.15 9.25,-9.25 0,-5.11 -4.14,-9.25 -9.25,-9.25 z m 4.39,11.53 c -1.93,1.93 -4.78,2.31 -6.7,2.31 -0.7,0 -1.41,-0.05 -2.1,-0.16 0,0 -1.02,-5.64 2.14,-8.81 0.83,-0.83 1.95,-1.28 3.13,-1.28 1.27,0 2.49,0.51 3.39,1.42 1.84,1.84 1.89,4.75 0.14,6.52 z",path1:"M 10.5,-0.01 C 4.7,-0.01 0,4.7 0,10.49 c 0,5.79 4.7,10.5 10.5,10.5 5.8,0 10.5,-4.7 10.5,-10.5 C 20.99,4.7 16.3,-0.01 10.5,-0.01 Z m 0,19.75 c -5.11,0 -9.25,-4.15 -9.25,-9.25 0,-5.1 4.14,-9.26 9.25,-9.26 5.11,0 9.25,4.15 9.25,9.25 0,5.13 -4.14,9.26 -9.25,9.26 z",path2:"M 14.74,6.25 C 12.9,4.41 9.98,4.35 8.23,6.1 5.07,9.27 6.09,14.91 6.09,14.91 c 0,0 5.64,1.02 8.81,-2.14 C 16.64,11 16.59,8.09 14.74,6.25 Z m -2.27,4.09 -0.91,1.87 -0.9,-1.87 -1.86,-0.91 1.86,-0.9 0.9,-1.87 0.91,1.87 1.86,0.9 z",polygon:"11.56,12.21 10.66,10.34 8.8,9.43 10.66,8.53 11.56,6.66 12.47,8.53 14.33,9.43 12.47,10.34"},styleRules:{map:"overflow:hidden;position:relative;","missing-css":"display:none;",canary:"background-color:salmon;","ctrl-bottom-left":"position: absolute; pointer-events: none; z-index: 2; bottom: 0; left: 0;","ctrl-bottom-right":"position: absolute; pointer-events: none; z-index: 2; right: 0; bottom: 0;",ctrl:"clear: both; pointer-events: auto; transform: translate(0, 0);","ctrl-attrib.mapboxgl-compact .mapboxgl-ctrl-attrib-inner":"display: none;","ctrl-attrib.mapboxgl-compact:hover .mapboxgl-ctrl-attrib-inner":"display: block; margin-top:2px","ctrl-attrib.mapboxgl-compact:hover":"padding: 2px 24px 2px 4px; visibility: visible; margin-top: 6px;","ctrl-attrib.mapboxgl-compact::after":'content: ""; cursor: pointer; position: absolute; background-image: url(\'data:image/svg+xml;charset=utf-8,%3Csvg viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg"%3E %3Cpath fill="%23333333" fill-rule="evenodd" d="M4,10a6,6 0 1,0 12,0a6,6 0 1,0 -12,0 M9,7a1,1 0 1,0 2,0a1,1 0 1,0 -2,0 M9,10a1,1 0 1,1 2,0l0,3a1,1 0 1,1 -2,0"/%3E %3C/svg%3E\'); background-color: rgba(255, 255, 255, 0.5); width: 24px; height: 24px; box-sizing: border-box; border-radius: 12px;',"ctrl-attrib.mapboxgl-compact":"min-height: 20px; padding: 0; margin: 10px; position: relative; background-color: #fff; border-radius: 3px 12px 12px 3px;","ctrl-bottom-right > .mapboxgl-ctrl-attrib.mapboxgl-compact::after":"bottom: 0; right: 0","ctrl-bottom-left > .mapboxgl-ctrl-attrib.mapboxgl-compact::after":"bottom: 0; left: 0","ctrl-bottom-left .mapboxgl-ctrl":"margin: 0 0 10px 10px; float: left;","ctrl-bottom-right .mapboxgl-ctrl":"margin: 0 10px 10px 0; float: right;","ctrl-attrib":"color: rgba(0, 0, 0, 0.75); text-decoration: none; font-size: 12px","ctrl-attrib a":"color: rgba(0, 0, 0, 0.75); text-decoration: none; font-size: 12px","ctrl-attrib a:hover":"color: inherit; text-decoration: underline;","ctrl-attrib .mapbox-improve-map":"font-weight: bold; margin-left: 2px;","attrib-empty":"display: none;","ctrl-logo":'display:block; width: 21px; height: 21px; background-image: url(\'data:image/svg+xml;charset=utf-8,%3C?xml version="1.0" encoding="utf-8"?%3E %3Csvg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 21 21" style="enable-background:new 0 0 21 21;" xml:space="preserve"%3E%3Cg transform="translate(0,0.01)"%3E%3Cpath d="m 10.5,1.24 c -5.11,0 -9.25,4.15 -9.25,9.25 0,5.1 4.15,9.25 9.25,9.25 5.1,0 9.25,-4.15 9.25,-9.25 0,-5.11 -4.14,-9.25 -9.25,-9.25 z m 4.39,11.53 c -1.93,1.93 -4.78,2.31 -6.7,2.31 -0.7,0 -1.41,-0.05 -2.1,-0.16 0,0 -1.02,-5.64 2.14,-8.81 0.83,-0.83 1.95,-1.28 3.13,-1.28 1.27,0 2.49,0.51 3.39,1.42 1.84,1.84 1.89,4.75 0.14,6.52 z" style="opacity:0.9;fill:%23ffffff;enable-background:new" class="st0"/%3E%3Cpath d="M 10.5,-0.01 C 4.7,-0.01 0,4.7 0,10.49 c 0,5.79 4.7,10.5 10.5,10.5 5.8,0 10.5,-4.7 10.5,-10.5 C 20.99,4.7 16.3,-0.01 10.5,-0.01 Z m 0,19.75 c -5.11,0 -9.25,-4.15 -9.25,-9.25 0,-5.1 4.14,-9.26 9.25,-9.26 5.11,0 9.25,4.15 9.25,9.25 0,5.13 -4.14,9.26 -9.25,9.26 z" style="opacity:0.35;enable-background:new" class="st1"/%3E%3Cpath d="M 14.74,6.25 C 12.9,4.41 9.98,4.35 8.23,6.1 5.07,9.27 6.09,14.91 6.09,14.91 c 0,0 5.64,1.02 8.81,-2.14 C 16.64,11 16.59,8.09 14.74,6.25 Z m -2.27,4.09 -0.91,1.87 -0.9,-1.87 -1.86,-0.91 1.86,-0.9 0.9,-1.87 0.91,1.87 1.86,0.9 z" style="opacity:0.35;enable-background:new" class="st1"/%3E%3Cpolygon points="11.56,12.21 10.66,10.34 8.8,9.43 10.66,8.53 11.56,6.66 12.47,8.53 14.33,9.43 12.47,10.34 " style="opacity:0.9;fill:%23ffffff;enable-background:new" class="st0"/%3E%3C/g%3E%3C/svg%3E\')'}}},{}],821:[function(t,e,n){"use strict";var r=t("../../lib");e.exports=function(t,e){var n=t.split(" "),i=n[0],o=n[1],a=r.isArrayOrTypedArray(e)?r.mean(e):e,s=.5+a/100,A=1.5+a/100,l=["",""],c=[0,0];switch(i){case"top":l[0]="top",c[1]=-A;break;case"bottom":l[0]="bottom",c[1]=A}switch(o){case"left":l[1]="right",c[0]=-s;break;case"right":l[1]="left",c[0]=s}return{anchor:l[0]&&l[1]?l.join("-"):l[0]?l[0]:l[1]?l[1]:"center",offset:c}}},{"../../lib":719}],822:[function(t,e,n){"use strict";var r=t("mapbox-gl"),i=t("../../lib"),o=t("../../plots/get_data").getSubplotCalcData,a=t("../../constants/xmlns_namespaces"),s=t("d3"),A=t("../../components/drawing"),l=t("../../lib/svg_text_utils"),c=t("./mapbox"),u=n.constants=t("./constants");function g(t){return"string"==typeof t&&-1!==u.styleValuesMapbox.indexOf(t)}n.name="mapbox",n.attr="subplot",n.idRoot="mapbox",n.idRegex=n.attrRegex=i.counterRegex("mapbox"),n.attributes={subplot:{valType:"subplotid",dflt:"mapbox",editType:"calc"}},n.layoutAttributes=t("./layout_attributes"),n.supplyLayoutDefaults=t("./layout_defaults"),n.plot=function(t){var e=t._fullLayout,n=t.calcdata,a=e._subplots.mapbox;if(r.version!==u.requiredVersion)throw new Error(u.wrongVersionErrorMsg);var s=function(t,e){var n=t._fullLayout;if(""===t._context.mapboxAccessToken)return"";for(var r=[],o=[],a=!1,s=!1,A=0;A<e.length;A++){var l=n[e[A]],c=l.accesstoken;g(l.style)&&(c?i.pushUnique(r,c):(g(l._input.style)&&(i.error("Uses Mapbox map style, but did not set an access token."),a=!0),s=!0)),c&&i.pushUnique(o,c)}if(s){var h=a?u.noAccessTokenErrorMsg:u.missingStyleErrorMsg;throw new Error(h)}return r.length?(r.length>1&&i.warn(u.multipleTokensErrorMsg),r[0]):(o.length&&i.log(["Listed mapbox access token(s)",o.join(","),"but did not use a Mapbox map style, ignoring token(s)."].join(" ")),"")}(t,a);r.accessToken=s;for(var A=0;A<a.length;A++){var l=a[A],h=o(n,"mapbox",l),d=e[l],f=d._subplot;f||(f=new c(t,l),e[l]._subplot=f),f.viewInitial||(f.viewInitial={center:i.extendFlat({},d.center),zoom:d.zoom,bearing:d.bearing,pitch:d.pitch}),f.plot(h,e,t._promises)}},n.clean=function(t,e,n,r){for(var i=r._subplots.mapbox||[],o=0;o<i.length;o++){var a=i[o];!e[a]&&r[a]._subplot&&r[a]._subplot.destroy()}},n.toSVG=function(t){for(var e=t._fullLayout,n=e._subplots.mapbox,r=e._size,i=0;i<n.length;i++){var o=e[n[i]],c=o.domain,g=o._subplot.toImage("png");e._glimages.append("svg:image").attr({xmlns:a.svg,"xlink:href":g,x:r.l+r.w*c.x[0],y:r.t+r.h*(1-c.y[1]),width:r.w*(c.x[1]-c.x[0]),height:r.h*(c.y[1]-c.y[0]),preserveAspectRatio:"none"});var h=s.select(o._subplot.div);if(null!==h.select(".mapboxgl-ctrl-logo").node().offsetParent){var d=e._glimages.append("g");d.attr("transform","translate("+(r.l+r.w*c.x[0]+10)+", "+(r.t+r.h*(1-c.y[0])-31)+")"),d.append("path").attr("d",u.mapboxLogo.path0).style({opacity:.9,fill:"#ffffff","enable-background":"new"}),d.append("path").attr("d",u.mapboxLogo.path1).style("opacity",.35).style("enable-background","new"),d.append("path").attr("d",u.mapboxLogo.path2).style("opacity",.35).style("enable-background","new"),d.append("polygon").attr("points",u.mapboxLogo.polygon).style({opacity:.9,fill:"#ffffff","enable-background":"new"})}var f=h.select(".mapboxgl-ctrl-attrib").text().replace("Improve this map",""),p=e._glimages.append("g"),I=p.append("text");I.text(f).classed("static-attribution",!0).attr({"font-size":12,"font-family":"Arial",color:"rgba(0, 0, 0, 0.75)","text-anchor":"end","data-unformatted":f});var C=A.bBox(I.node()),m=r.w*(c.x[1]-c.x[0]);if(C.width>m/2){var y=f.split("|").join("<br>");I.text(y).attr("data-unformatted",y).call(l.convertToTspans,t),C=A.bBox(I.node())}I.attr("transform","translate(-3, "+(8-C.height)+")"),p.insert("rect",".static-attribution").attr({x:-C.width-6,y:-C.height-3,width:C.width+6,height:C.height+3,fill:"rgba(255, 255, 255, 0.75)"});var E=1;C.width+6>m&&(E=m/(C.width+6));var v=[r.l+r.w*c.x[1],r.t+r.h*(1-c.y[0])];p.attr("transform","translate("+v[0]+","+v[1]+") scale("+E+")")}},n.updateFx=function(t){for(var e=t._fullLayout,n=e._subplots.mapbox,r=0;r<n.length;r++)e[n[r]]._subplot.updateFx(e)}},{"../../components/drawing":614,"../../constants/xmlns_namespaces":696,"../../lib":719,"../../lib/svg_text_utils":743,"../../plots/get_data":802,"./constants":820,"./layout_attributes":824,"./layout_defaults":825,"./mapbox":826,d3:163,"mapbox-gl":426}],823:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./convert_text_opts"),o=t("./constants");function a(t,e){this.subplot=t,this.uid=t.uid+"-"+e,this.index=e,this.idSource="source-"+this.uid,this.idLayer=o.layoutLayerPrefix+this.uid,this.sourceType=null,this.source=null,this.layerType=null,this.below=null,this.visible=!1}var s=a.prototype;function A(t){var e=t.source;return t.visible&&(r.isPlainObject(e)||("string"==typeof e||Array.isArray(e))&&e.length>0)}function l(t){var e={},n={};switch(t.type){case"circle":r.extendFlat(n,{"circle-radius":t.circle.radius,"circle-color":t.color,"circle-opacity":t.opacity});break;case"line":r.extendFlat(n,{"line-width":t.line.width,"line-color":t.color,"line-opacity":t.opacity,"line-dasharray":t.line.dash});break;case"fill":r.extendFlat(n,{"fill-color":t.color,"fill-outline-color":t.fill.outlinecolor,"fill-opacity":t.opacity});break;case"symbol":var o=t.symbol,a=i(o.textposition,o.iconsize);r.extendFlat(e,{"icon-image":o.icon+"-15","icon-size":o.iconsize/10,"text-field":o.text,"text-size":o.textfont.size,"text-anchor":a.anchor,"text-offset":a.offset,"symbol-placement":o.placement}),r.extendFlat(n,{"icon-color":t.color,"text-color":o.textfont.color,"text-opacity":t.opacity})}return{layout:e,paint:n}}s.update=function(t){this.visible?this.needsNewSource(t)?(this.removeLayer(),this.updateSource(t),this.updateLayer(t)):this.needsNewLayer(t)?this.updateLayer(t):this.updateStyle(t):(this.updateSource(t),this.updateLayer(t)),this.visible=A(t)},s.needsNewSource=function(t){return this.sourceType!==t.sourcetype||this.source!==t.source||this.layerType!==t.type},s.needsNewLayer=function(t){return this.layerType!==t.type||this.below!==this.subplot.belowLookup["layout-"+this.index]},s.updateSource=function(t){var e=this.subplot.map;if(e.getSource(this.idSource)&&e.removeSource(this.idSource),this.sourceType=t.sourcetype,this.source=t.source,A(t)){var n=function(t){var e,n=t.sourcetype,r=t.source,i={type:n};return"geojson"===n?e="data":"vector"===n?e="string"==typeof r?"url":"tiles":"raster"===n?(e="tiles",i.tileSize=256):"image"===n&&(e="url",i.coordinates=t.coordinates),i[e]=r,t.sourceattribution&&(i.attribution=t.sourceattribution),i}(t);e.addSource(this.idSource,n)}},s.updateLayer=function(t){var e,n=this.subplot,r=l(t),i=this.subplot.belowLookup["layout-"+this.index];if("traces"===i)for(var a=n.getMapLayers(),s=0;s<a.length;s++){var c=a[s].id;if("string"==typeof c&&0===c.indexOf(o.traceLayerPrefix)){e=c;break}}else e=i;this.removeLayer(),A(t)&&n.addLayer({id:this.idLayer,source:this.idSource,"source-layer":t.sourcelayer||"",type:t.type,minzoom:t.minzoom,maxzoom:t.maxzoom,layout:r.layout,paint:r.paint},e),this.layerType=t.type,this.below=i},s.updateStyle=function(t){if(A(t)){var e=l(t);this.subplot.setOptions(this.idLayer,"setLayoutProperty",e.layout),this.subplot.setOptions(this.idLayer,"setPaintProperty",e.paint)}},s.removeLayer=function(){var t=this.subplot.map;t.getLayer(this.idLayer)&&t.removeLayer(this.idLayer)},s.dispose=function(){var t=this.subplot.map;t.removeLayer(this.idLayer),t.removeSource(this.idSource)},e.exports=function(t,e,n){var r=new a(t,e);return r.update(n),r}},{"../../lib":719,"./constants":820,"./convert_text_opts":821}],824:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/color").defaultLine,o=t("../domain").attributes,a=t("../font_attributes"),s=t("../../traces/scatter/attributes").textposition,A=t("../../plot_api/edit_types").overrideAll,l=t("../../plot_api/plot_template").templatedArray,c=t("./constants"),u=a({});u.family.dflt="Open Sans Regular, Arial Unicode MS Regular",(e.exports=A({_arrayAttrRegexps:[r.counterRegex("mapbox",".layers",!0)],domain:o({name:"mapbox"}),accesstoken:{valType:"string",noBlank:!0,strict:!0},style:{valType:"any",values:c.styleValuesMapbox.concat(c.styleValuesNonMapbox),dflt:c.styleValueDflt},center:{lon:{valType:"number",dflt:0},lat:{valType:"number",dflt:0}},zoom:{valType:"number",dflt:1},bearing:{valType:"number",dflt:0},pitch:{valType:"number",dflt:0},layers:l("layer",{visible:{valType:"boolean",dflt:!0},sourcetype:{valType:"enumerated",values:["geojson","vector","raster","image"],dflt:"geojson"},source:{valType:"any"},sourcelayer:{valType:"string",dflt:""},sourceattribution:{valType:"string"},type:{valType:"enumerated",values:["circle","line","fill","symbol","raster"],dflt:"circle"},coordinates:{valType:"any"},below:{valType:"string"},color:{valType:"color",dflt:i},opacity:{valType:"number",min:0,max:1,dflt:1},minzoom:{valType:"number",min:0,max:24,dflt:0},maxzoom:{valType:"number",min:0,max:24,dflt:24},circle:{radius:{valType:"number",dflt:15}},line:{width:{valType:"number",dflt:2},dash:{valType:"data_array"}},fill:{outlinecolor:{valType:"color",dflt:i}},symbol:{icon:{valType:"string",dflt:"marker"},iconsize:{valType:"number",dflt:10},text:{valType:"string",dflt:""},placement:{valType:"enumerated",values:["point","line","line-center"],dflt:"point"},textfont:u,textposition:r.extendFlat({},s,{arrayOk:!1})}})},"plot","from-root")).uirevision={valType:"any",editType:"none"}},{"../../components/color":593,"../../lib":719,"../../plot_api/edit_types":750,"../../plot_api/plot_template":757,"../../traces/scatter/attributes":1112,"../domain":792,"../font_attributes":793,"./constants":820}],825:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../subplot_defaults"),o=t("../array_container_defaults"),a=t("./layout_attributes");function s(t,e,n,r){n("accesstoken",r.accessToken),n("style"),n("center.lon"),n("center.lat"),n("zoom"),n("bearing"),n("pitch"),o(t,e,{name:"layers",handleItemDefaults:A}),e._input=t}function A(t,e){function n(n,i){return r.coerce(t,e,a.layers,n,i)}if(n("visible")){var i,o=n("sourcetype"),s="raster"===o||"image"===o;n("source"),n("sourceattribution"),"vector"===o&&n("sourcelayer"),"image"===o&&n("coordinates"),s&&(i="raster");var A=n("type",i);s&&"raster"!==A&&(A=e.type="raster",r.log("Source types *raster* and *image* must drawn *raster* layer type.")),n("below"),n("color"),n("opacity"),n("minzoom"),n("maxzoom"),"circle"===A&&n("circle.radius"),"line"===A&&(n("line.width"),n("line.dash")),"fill"===A&&n("fill.outlinecolor"),"symbol"===A&&(n("symbol.icon"),n("symbol.iconsize"),n("symbol.text"),r.coerceFont(n,"symbol.textfont"),n("symbol.textposition"),n("symbol.placement"))}}e.exports=function(t,e,n){i(t,e,n,{type:"mapbox",attributes:a,handleDefaults:s,partition:"y",accessToken:e._mapboxAccessToken})}},{"../../lib":719,"../array_container_defaults":763,"../subplot_defaults":842,"./layout_attributes":824}],826:[function(t,e,n){"use strict";var r=t("mapbox-gl"),i=t("d3"),o=t("../../components/fx"),a=t("../../lib"),s=t("../../registry"),A=t("../cartesian/axes"),l=t("../../components/dragelement"),c=t("../cartesian/select").prepSelect,u=t("../cartesian/select").selectOnClick,g=t("./constants"),h=t("./layers");function d(t,e){this.id=e,this.gd=t;var n=t._fullLayout,r=t._context;this.container=n._glcontainer.node(),this.isStatic=r.staticPlot,this.uid=n._uid+"-"+this.id,this.div=null,this.xaxis=null,this.yaxis=null,this.createFramework(n),this.map=null,this.accessToken=null,this.styleObj=null,this.traceHash={},this.layerList=[],this.belowLookup={}}var f=d.prototype;f.plot=function(t,e,n){var r,i=this,o=e[i.id];i.map&&o.accesstoken!==i.accessToken&&(i.map.remove(),i.map=null,i.styleObj=null,i.traceHash=[],i.layerList={}),r=i.map?new Promise(function(n,r){i.updateMap(t,e,n,r)}):new Promise(function(n,r){i.createMap(t,e,n,r)}),n.push(r)},f.createMap=function(t,e,n,i){var o=this,a=e[o.id],s=o.styleObj=I(a.style);o.accessToken=a.accesstoken;var A=o.map=new r.Map({container:o.div,style:s.style,center:m(a.center),zoom:a.zoom,bearing:a.bearing,pitch:a.pitch,interactive:!o.isStatic,preserveDrawingBuffer:o.isStatic,doubleClickZoom:!1,boxZoom:!1,attributionControl:!1}).addControl(new r.AttributionControl({compact:!0}));A._canvas.style.left="0px",A._canvas.style.top="0px",o.rejectOnError(i),o.isStatic||o.initFx(t,e);var l=[];l.push(new Promise(function(t){A.once("load",t)})),l=l.concat(o.fetchMapData(t,e)),Promise.all(l).then(function(){o.fillBelowLookup(t,e),o.updateData(t),o.updateLayout(e),o.resolveOnRender(n)}).catch(i)},f.fetchMapData=function(t){var e=[];function n(t){return new Promise(function(e,n){i.json(t,function(r,i){if(r){delete PlotlyGeoAssets[t];var o=404===r.status?'GeoJSON at URL "'+t+'" does not exist.':"Unexpected error while fetching from "+t;return n(new Error(o))}PlotlyGeoAssets[t]=i,e(i)})})}for(var r=0;r<t.length;r++){var o=t[r][0].trace.geojson;"string"!=typeof o||PlotlyGeoAssets[o]||(PlotlyGeoAssets[o]="pending",e.push(n(o)))}return e},f.updateMap=function(t,e,n,r){var i=this,o=i.map,a=e[this.id];i.rejectOnError(r);var s=[],A=I(a.style);i.styleObj.id!==A.id&&(i.styleObj=A,o.setStyle(A.style),i.traceHash={},s.push(new Promise(function(t){o.once("styledata",t)}))),s=s.concat(i.fetchMapData(t,e)),Promise.all(s).then(function(){i.fillBelowLookup(t,e),i.updateData(t),i.updateLayout(e),i.resolveOnRender(n)}).catch(r)},f.fillBelowLookup=function(t,e){var n,r,i=e[this.id].layers,o=this.belowLookup={},a=!1;for(n=0;n<t.length;n++){var s=t[n][0].trace,A=s._module;"string"==typeof s.below?r=s.below:A.getBelow&&(r=A.getBelow(s,this)),""===r&&(a=!0),o["trace-"+s.uid]=r||""}for(n=0;n<i.length;n++){var l=i[n];r="string"==typeof l.below?l.below:a?"traces":"",o["layout-"+n]=r}var c,u,g={};for(c in o)g[r=o[c]]?g[r].push(c):g[r]=[c];for(r in g){var h=g[r];if(h.length>1)for(n=0;n<h.length;n++)0===(c=h[n]).indexOf("trace-")?(u=c.split("trace-")[1],this.traceHash[u]&&(this.traceHash[u].below=null)):0===c.indexOf("layout-")&&(u=c.split("layout-")[1],this.layerList[u]&&(this.layerList[u].below=null))}};var p={choroplethmapbox:0,densitymapbox:1,scattermapbox:2};function I(t){var e={};return a.isPlainObject(t)?(e.id=t.id,e.style=t):"string"==typeof t?(e.id=t,-1!==g.styleValuesMapbox.indexOf(t)?e.style=C(t):g.stylesNonMapbox[t]?e.style=g.stylesNonMapbox[t]:e.style=t):(e.id=g.styleValueDflt,e.style=C(g.styleValueDflt)),e.transition={duration:0,delay:0},e}function C(t){return g.styleUrlPrefix+t+"-"+g.styleUrlSuffix}function m(t){return[t.lon,t.lat]}f.updateData=function(t){var e,n,r,i,o=this.traceHash,a=t.slice().sort(function(t,e){return p[t[0].trace.type]-p[e[0].trace.type]});for(r=0;r<a.length;r++){var s=a[r];(e=o[(n=s[0].trace).uid])?e.update(s):n._module&&(o[n.uid]=n._module.plot(this,s))}var A=Object.keys(o);t:for(r=0;r<A.length;r++){var l=A[r];for(i=0;i<t.length;i++)if(l===(n=t[i][0].trace).uid)continue t;(e=o[l]).dispose(),delete o[l]}},f.updateLayout=function(t){var e=this.map,n=t[this.id];e.setCenter(m(n.center)),e.setZoom(n.zoom),e.setBearing(n.bearing),e.setPitch(n.pitch),this.updateLayers(t),this.updateFramework(t),this.updateFx(t),this.map.resize(),this.gd._context._scrollZoom.mapbox?e.scrollZoom.enable():e.scrollZoom.disable()},f.resolveOnRender=function(t){var e=this.map;e.on("render",function n(){e.loaded()&&(e.off("render",n),setTimeout(t,0))})},f.rejectOnError=function(t){var e=this.map;function n(){t(new Error(g.mapOnErrorMsg))}e.once("error",n),e.once("style.error",n),e.once("source.error",n),e.once("tile.error",n),e.once("layer.error",n)},f.createFramework=function(t){var e=this,n=e.div=document.createElement("div");n.id=e.uid,n.style.position="absolute",e.container.appendChild(n),e.xaxis={_id:"x",c2p:function(t){return e.project(t).x}},e.yaxis={_id:"y",c2p:function(t){return e.project(t).y}},e.updateFramework(t),e.mockAxis={type:"linear",showexponent:"all",exponentformat:"B"},A.setConvert(e.mockAxis,t)},f.initFx=function(t,e){var n=this,r=n.gd,i=n.map,a=!1;function A(){o.loneUnhover(e._hoverlayer)}function l(){var t=n.getView();r.emit("plotly_relayouting",n.getViewEdits(t))}i.on("moveend",function(t){if(n.map){var e=r._fullLayout;if(t.originalEvent||a){var i=e[n.id];s.call("_storeDirectGUIEdit",r.layout,e._preGUI,n.getViewEdits(i));var o=n.getView();i._input.center=i.center=o.center,i._input.zoom=i.zoom=o.zoom,i._input.bearing=i.bearing=o.bearing,i._input.pitch=i.pitch=o.pitch,r.emit("plotly_relayout",n.getViewEdits(o))}a=!1,e._rehover&&e._rehover()}}),i.on("wheel",function(){a=!0}),i.on("mousemove",function(t){var e=n.div.getBoundingClientRect();t.clientX=t.point.x+e.left,t.clientY=t.point.y+e.top,t.target.getBoundingClientRect=function(){return e},n.xaxis.p2c=function(){return t.lngLat.lng},n.yaxis.p2c=function(){return t.lngLat.lat},r._fullLayout._rehover=function(){r._fullLayout._hoversubplot===n.id&&o.hover(r,t,n.id)},o.hover(r,t,n.id),r._fullLayout._hoversubplot=n.id}),i.on("dragstart",A),i.on("zoomstart",A),i.on("mouseout",function(){r._fullLayout._hoversubplot=null}),i.on("drag",l),i.on("zoom",l),i.on("dblclick",function(){var t=r._fullLayout[n.id];s.call("_storeDirectGUIEdit",r.layout,r._fullLayout._preGUI,n.getViewEdits(t));var e=n.viewInitial;i.setCenter(m(e.center)),i.setZoom(e.zoom),i.setBearing(e.bearing),i.setPitch(e.pitch);var o=n.getView();t._input.center=t.center=o.center,t._input.zoom=t.zoom=o.zoom,t._input.bearing=t.bearing=o.bearing,t._input.pitch=t.pitch=o.pitch,r.emit("plotly_doubleclick",null),r.emit("plotly_relayout",n.getViewEdits(o))}),n.clearSelect=function(){r._fullLayout._zoomlayer.selectAll(".select-outline").remove()},n.onClickInPanFn=function(t){return function(e){var i=r._fullLayout.clickmode;i.indexOf("select")>-1&&u(e.originalEvent,r,[n.xaxis],[n.yaxis],n.id,t),i.indexOf("event")>-1&&o.click(r,e.originalEvent)}}},f.updateFx=function(t){var e=this,n=e.map,r=e.gd;if(!e.isStatic){var i,o=t.dragmode;i="select"===o?function(t,n){(t.range={})[e.id]=[A([n.xmin,n.ymin]),A([n.xmax,n.ymax])]}:function(t,n,r){(t.lassoPoints={})[e.id]=r.filtered.map(A)};var s=e.dragOptions;e.dragOptions=a.extendDeep(s||{},{element:e.div,gd:r,plotinfo:{id:e.id,xaxis:e.xaxis,yaxis:e.yaxis,fillRangeItems:i},xaxes:[e.xaxis],yaxes:[e.yaxis],subplot:e.id}),n.off("click",e.onClickInPanHandler),"select"===o||"lasso"===o?(n.dragPan.disable(),n.on("zoomstart",e.clearSelect),e.dragOptions.prepFn=function(t,n,r){c(t,n,r,e.dragOptions,o)},l.init(e.dragOptions)):(n.dragPan.enable(),n.off("zoomstart",e.clearSelect),e.div.onmousedown=null,e.onClickInPanHandler=e.onClickInPanFn(e.dragOptions),n.on("click",e.onClickInPanHandler))}function A(t){var n=e.map.unproject(t);return[n.lng,n.lat]}},f.updateFramework=function(t){var e=t[this.id].domain,n=t._size,r=this.div.style;r.width=n.w*(e.x[1]-e.x[0])+"px",r.height=n.h*(e.y[1]-e.y[0])+"px",r.left=n.l+e.x[0]*n.w+"px",r.top=n.t+(1-e.y[1])*n.h+"px",this.xaxis._offset=n.l+e.x[0]*n.w,this.xaxis._length=n.w*(e.x[1]-e.x[0]),this.yaxis._offset=n.t+(1-e.y[1])*n.h,this.yaxis._length=n.h*(e.y[1]-e.y[0])},f.updateLayers=function(t){var e,n=t[this.id].layers,r=this.layerList;if(n.length!==r.length){for(e=0;e<r.length;e++)r[e].dispose();for(r=this.layerList=[],e=0;e<n.length;e++)r.push(h(this,e,n[e]))}else for(e=0;e<n.length;e++)r[e].update(n[e])},f.destroy=function(){this.map&&(this.map.remove(),this.map=null,this.container.removeChild(this.div))},f.toImage=function(){return this.map.stop(),this.map.getCanvas().toDataURL()},f.setOptions=function(t,e,n){for(var r in n)this.map[e](t,r,n[r])},f.getMapLayers=function(){return this.map.getStyle().layers},f.addLayer=function(t,e){var n=this.map;if("string"==typeof e){if(""===e)return void n.addLayer(t,e);for(var r=this.getMapLayers(),i=0;i<r.length;i++)if(e===r[i].id)return void n.addLayer(t,e);a.warn(["Trying to add layer with *below* value",e,"referencing a layer that does not exist","or that does not yet exist."].join(" "))}n.addLayer(t)},f.project=function(t){return this.map.project(new r.LngLat(t[0],t[1]))},f.getView=function(){var t=this.map,e=t.getCenter();return{center:{lon:e.lng,lat:e.lat},zoom:t.getZoom(),bearing:t.getBearing(),pitch:t.getPitch()}},f.getViewEdits=function(t){for(var e=this.id,n=["center","zoom","bearing","pitch"],r={},i=0;i<n.length;i++){var o=n[i];r[e+"."+o]=t[o]}return r},e.exports=d},{"../../components/dragelement":611,"../../components/fx":632,"../../lib":719,"../../registry":847,"../cartesian/axes":767,"../cartesian/select":784,"./constants":820,"./layers":823,d3:163,"mapbox-gl":426}],827:[function(t,e,n){"use strict";e.exports=function(t){var e=t.editType;return{t:{valType:"number",dflt:0,editType:e},r:{valType:"number",dflt:0,editType:e},b:{valType:"number",dflt:0,editType:e},l:{valType:"number",dflt:0,editType:e},editType:e}}},{}],828:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=t("../registry"),a=t("../plot_api/plot_schema"),s=t("../plot_api/plot_template"),A=t("../lib"),l=t("../components/color"),c=t("../constants/numerical").BADNUM,u=t("./cartesian/axis_ids"),g=t("./animation_attributes"),h=t("./frame_attributes"),d=A.relinkPrivateKeys,f=A._,p=e.exports={};A.extendFlat(p,o),p.attributes=t("./attributes"),p.attributes.type.values=p.allTypes,p.fontAttrs=t("./font_attributes"),p.layoutAttributes=t("./layout_attributes"),p.fontWeight="normal";var I=p.transformsRegistry,C=t("./command");p.executeAPICommand=C.executeAPICommand,p.computeAPICommandBindings=C.computeAPICommandBindings,p.manageCommandObserver=C.manageCommandObserver,p.hasSimpleAPICommandBindings=C.hasSimpleAPICommandBindings,p.redrawText=function(t){var e=(t=A.getGraphDiv(t))._fullLayout||{};if(e._has&&e._has("polar")||!t.data||!t.data[0]||!t.data[0].r)return new Promise(function(e){setTimeout(function(){o.getComponentMethod("annotations","draw")(t),o.getComponentMethod("legend","draw")(t),o.getComponentMethod("colorbar","draw")(t),e(p.previousPromises(t))},300)})},p.resize=function(t){return t=A.getGraphDiv(t),new Promise(function(e,n){t&&!A.isHidden(t)||n(new Error("Resize must be passed a displayed plot div element.")),t._redrawTimer&&clearTimeout(t._redrawTimer),t._redrawTimer=setTimeout(function(){if(!t.layout||t.layout.width&&t.layout.height||A.isHidden(t))e(t);else{delete t.layout.width,delete t.layout.height;var n=t.changed;t.autoplay=!0,o.call("relayout",t,{autosize:!0}).then(function(){t.changed=n,e(t)})}},100)})},p.previousPromises=function(t){if((t._promises||[]).length)return Promise.all(t._promises).then(function(){t._promises=[]})},p.addLinks=function(t){if(t._context.showLink||t._context.showSources){var e=t._fullLayout,n=A.ensureSingle(e._paper,"text","js-plot-link-container",function(t){t.style({"font-family":'"Open Sans", Arial, sans-serif',"font-size":"12px",fill:l.defaultLine,"pointer-events":"all"}).each(function(){var t=r.select(this);t.append("tspan").classed("js-link-to-tool",!0),t.append("tspan").classed("js-link-spacer",!0),t.append("tspan").classed("js-sourcelinks",!0)})}),i=n.node(),o={y:e._paper.attr("height")-9};document.body.contains(i)&&i.getComputedTextLength()>=e.width-20?(o["text-anchor"]="start",o.x=5):(o["text-anchor"]="end",o.x=e._paper.attr("width")-7),n.attr(o);var a=n.select(".js-link-to-tool"),s=n.select(".js-link-spacer"),c=n.select(".js-sourcelinks");t._context.showSources&&t._context.showSources(t),t._context.showLink&&function(t,e){e.text("");var n=e.append("a").attr({"xlink:xlink:href":"#",class:"link--impt link--embedview","font-weight":"bold"}).text(t._context.linkText+" "+String.fromCharCode(187));if(t._context.sendData)n.on("click",function(){p.sendDataToCloud(t)});else{var r=window.location.pathname.split("/"),i=window.location.search;n.attr({"xlink:xlink:show":"new","xlink:xlink:href":"/"+r[2].split(".")[0]+"/"+r[1]+i})}}(t,a),s.text(a.text()&&c.text()?" - ":"")}},p.sendDataToCloud=function(t){t.emit("plotly_beforeexport");var e=(window.PLOTLYENV||{}).BASE_URL||t._context.plotlyServerURL,n=r.select(t).append("div").attr("id","hiddenform").style("display","none"),i=n.append("form").attr({action:e+"/external",method:"post",target:"_blank"});return i.append("input").attr({type:"text",name:"data"}).node().value=p.graphJson(t,!1,"keepdata"),i.node().submit(),n.remove(),t.emit("plotly_afterexport"),!1};var m=["days","shortDays","months","shortMonths","periods","dateTime","date","time","decimal","thousands","grouping","currency"],y=["year","month","dayMonth","dayMonthYear"];function E(t,e){var n=t._context.locale,r=!1,i={};function a(t){for(var n=!0,o=0;o<e.length;o++){var a=e[o];i[a]||(t[a]?i[a]=t[a]:n=!1)}n&&(r=!0)}for(var s=0;s<2;s++){for(var A=t._context.locales,l=0;l<2;l++){var c=(A[n]||{}).format;if(c&&(a(c),r))break;A=o.localeRegistry}var u=n.split("-")[0];if(r||u===n)break;n=u}return r||a(o.localeRegistry.en.format),i}function v(t,e){var n={_fullLayout:e},r="x"===t._id.charAt(0),i=t._mainAxis._anchorAxis,o="",a="",s="";if(i&&(s=i._mainAxis._id,o=r?t._id+s:s+t._id),!o||!e._plots[o]){o="";for(var A=t._counterAxes,l=0;l<A.length;l++){var c=A[l],g=r?t._id+c:c+t._id;a||(a=g);var h=u.getFromId(n,c);if(s&&h.overlaying===s){o=g;break}}}return o||a}function B(t){var e=t.transforms;if(Array.isArray(e)&&e.length)for(var n=0;n<e.length;n++){var r=e[n],i=r._module||I[r.type];if(i&&i.makesData)return!0}return!1}function b(t,e,n,r){for(var i=t.transforms,o=[t],a=0;a<i.length;a++){var s=i[a],A=I[s.type];A&&A.transform&&(o=A.transform(o,{transform:s,fullTrace:t,fullData:e,layout:n,fullLayout:r,transformIndex:a}))}return o}function Q(t){var e=t.margin;if(!t._size){var n=t._size={l:Math.round(e.l),r:Math.round(e.r),t:Math.round(e.t),b:Math.round(e.b),p:Math.round(e.pad)};n.w=Math.round(t.width)-n.l-n.r,n.h=Math.round(t.height)-n.t-n.b}t._pushmargin||(t._pushmargin={}),t._pushmarginIds||(t._pushmarginIds={})}p.supplyDefaults=function(t,e){var n=e&&e.skipUpdateCalc,i=t._fullLayout||{};if(i._skipDefaults)delete i._skipDefaults;else{var a,s=t._fullLayout={},l=t.layout||{},c=t._fullData||[],u=t._fullData=[],g=t.data||[],h=t.calcdata||[],I=t._context||{};t._transitionData||p.createTransitionData(t),s._dfltTitle={plot:f(t,"Click to enter Plot title"),x:f(t,"Click to enter X axis title"),y:f(t,"Click to enter Y axis title"),colorbar:f(t,"Click to enter Colorscale title"),annotation:f(t,"new text")},s._traceWord=f(t,"trace");var C=E(t,m);if(s._mapboxAccessToken=I.mapboxAccessToken,i._initialAutoSizeIsDone){var v=i.width,B=i.height;p.supplyLayoutGlobalDefaults(l,s,C),l.width||(s.width=v),l.height||(s.height=B),p.sanitizeMargins(s)}else{p.supplyLayoutGlobalDefaults(l,s,C);var b=!l.width||!l.height,w=s.autosize,x=I.autosizable;b&&(w||x)?p.plotAutoSize(t,l,s):b&&p.sanitizeMargins(s),!w&&b&&(l.width=s.width,l.height=s.height)}s._d3locale=function(t,e){return t.decimal=e.charAt(0),t.thousands=e.charAt(1),r.locale(t)}(C,s.separators),s._extraFormat=E(t,y),s._initialAutoSizeIsDone=!0,s._dataLength=g.length,s._modules=[],s._visibleModules=[],s._basePlotModules=[];var _=s._subplots=function(){var t,e,n=o.collectableSubplotTypes,r={};if(!n){n=[];var i=o.subplotsRegistry;for(var a in i){var s=i[a],l=s.attr;if(l&&(n.push(a),Array.isArray(l)))for(e=0;e<l.length;e++)A.pushUnique(n,l[e])}}for(t=0;t<n.length;t++)r[n[t]]=[];return r}(),D=s._splomAxes={x:{},y:{}},M=s._splomSubplots={};s._splomGridDflt={},s._scatterStackOpts={},s._firstScatter={},s._alignmentOpts={},s._colorAxes={},s._requestRangeslider={},s._traceUids=function(t,e){var n,r,i=e.length,o=[];for(n=0;n<t.length;n++){var a=t[n]._fullInput;a!==r&&o.push(a),r=a}var s=o.length,l=new Array(i),c={};function u(t,e){l[e]=t,c[t]=1}function g(t,e){if(t&&"string"==typeof t&&!c[t])return u(t,e),!0}for(n=0;n<i;n++){var h=e[n].uid;"number"==typeof h&&(h=String(h)),g(h,n)||n<s&&g(o[n].uid,n)||u(A.randstr(c),n)}return l}(c,g),s._globalTransforms=(t._context||{}).globalTransforms,p.supplyDataDefaults(g,u,l,s);var k=Object.keys(D.x),S=Object.keys(D.y);if(k.length>1&&S.length>1){for(o.getComponentMethod("grid","sizeDefaults")(l,s),a=0;a<k.length;a++)A.pushUnique(_.xaxis,k[a]);for(a=0;a<S.length;a++)A.pushUnique(_.yaxis,S[a]);for(var L in M)A.pushUnique(_.cartesian,L)}if(s._has=p._hasPlotType.bind(s),c.length===u.length)for(a=0;a<u.length;a++)d(u[a],c[a]);p.supplyLayoutModuleDefaults(l,s,u,t._transitionData);var R=s._visibleModules,N=[];for(a=0;a<R.length;a++){var F=R[a].crossTraceDefaults;F&&A.pushUnique(N,F)}for(a=0;a<N.length;a++)N[a](u,s);s._hasOnlyLargeSploms=1===s._basePlotModules.length&&"splom"===s._basePlotModules[0].name&&k.length>15&&S.length>15&&0===s.shapes.length&&0===s.images.length,s._hasCartesian=s._has("cartesian"),s._hasGeo=s._has("geo"),s._hasGL3D=s._has("gl3d"),s._hasGL2D=s._has("gl2d"),s._hasTernary=s._has("ternary"),s._hasPie=s._has("pie"),p.linkSubplots(u,s,c,i),p.cleanPlot(u,s,c,i),i._zoomlayer&&!t._dragging&&i._zoomlayer.selectAll(".select-outline").remove(),function(t,e){var n,r=[];e.meta&&(n=e._meta={meta:e.meta,layout:{meta:e.meta}});for(var i=0;i<t.length;i++){var o=t[i];o.meta?r[o.index]=o._meta={meta:o.meta}:e.meta&&(o._meta={meta:e.meta}),e.meta&&(o._meta.layout={meta:e.meta})}r.length&&(n||(n=e._meta={}),n.data=r)}(u,s),d(s,i),o.getComponentMethod("colorscale","crossTraceDefaults")(u,s),s._preGUI||(s._preGUI={}),s._tracePreGUI||(s._tracePreGUI={});var T,U=s._tracePreGUI,O={};for(T in U)O[T]="old";for(a=0;a<u.length;a++)O[T=u[a]._fullInput.uid]||(U[T]={}),O[T]="new";for(T in O)"old"===O[T]&&delete U[T];Q(s),o.getComponentMethod("rangeslider","makeData")(s),n||h.length!==u.length||p.supplyDefaultsUpdateCalc(h,u)}},p.supplyDefaultsUpdateCalc=function(t,e){for(var n=0;n<e.length;n++){var r=e[n],i=(t[n]||[])[0];if(i&&i.trace){var o=i.trace;if(o._hasCalcTransform){var a,s,l,c=o._arrayAttrs;for(a=0;a<c.length;a++)s=c[a],l=A.nestedProperty(o,s).get().slice(),A.nestedProperty(r,s).set(l)}i.trace=r}}},p.createTransitionData=function(t){t._transitionData||(t._transitionData={}),t._transitionData._frames||(t._transitionData._frames=[]),t._transitionData._frameHash||(t._transitionData._frameHash={}),t._transitionData._counter||(t._transitionData._counter=0),t._transitionData._interruptCallbacks||(t._transitionData._interruptCallbacks=[])},p._hasPlotType=function(t){var e,n=this._basePlotModules||[];for(e=0;e<n.length;e++)if(n[e].name===t)return!0;var r=this._modules||[];for(e=0;e<r.length;e++){var i=r[e].name;if(i===t)return!0;var a=o.modules[i];if(a&&a.categories[t])return!0}return!1},p.cleanPlot=function(t,e,n,r){var i,o,a=r._basePlotModules||[];for(i=0;i<a.length;i++){var s=a[i];s.clean&&s.clean(t,e,n,r)}var A=r._has&&r._has("gl"),l=e._has&&e._has("gl");A&&!l&&void 0!==r._glcontainer&&(r._glcontainer.selectAll(".gl-canvas").remove(),r._glcontainer.selectAll(".no-webgl").remove(),r._glcanvas=null);var c=!!r._infolayer;t:for(i=0;i<n.length;i++){var u=n[i].uid;for(o=0;o<t.length;o++)if(u===t[o].uid)continue t;c&&r._infolayer.select(".cb"+u).remove()}},p.linkSubplots=function(t,e,n,r){var i,o,a=r._plots||{},s=e._plots={},l=e._subplots,c={_fullData:t,_fullLayout:e},g=l.cartesian.concat(l.gl2d||[]);for(i=0;i<g.length;i++){var h,d=g[i],f=a[d],p=u.getFromId(c,d,"x"),I=u.getFromId(c,d,"y");for(f?h=s[d]=f:(h=s[d]={}).id=d,p._counterAxes.push(I._id),I._counterAxes.push(p._id),p._subplotsWith.push(d),I._subplotsWith.push(d),h.xaxis=p,h.yaxis=I,h._hasClipOnAxisFalse=!1,o=0;o<t.length;o++){var C=t[o];if(C.xaxis===h.xaxis._id&&C.yaxis===h.yaxis._id&&!1===C.cliponaxis){h._hasClipOnAxisFalse=!0;break}}}var m,y=u.list(c,null,!0);for(i=0;i<y.length;i++){var E=null;(m=y[i]).overlaying&&(E=u.getFromId(c,m.overlaying))&&E.overlaying&&(m.overlaying=!1,E=null),m._mainAxis=E||m,E&&(m.domain=E.domain.slice()),m._anchorAxis="free"===m.anchor?null:u.getFromId(c,m.anchor)}for(i=0;i<y.length;i++)(m=y[i])._counterAxes.sort(u.idSort),m._subplotsWith.sort(A.subplotSort),m._mainSubplot=v(m,e)},p.clearExpandedTraceDefaultColors=function(t){var e,n,r;for(n=[],(e=t._module._colorAttrs)||(t._module._colorAttrs=e=[],a.crawl(t._module.attributes,function(t,r,i,o){n[o]=r,n.length=o+1,"color"===t.valType&&void 0===t.dflt&&e.push(n.join("."))})),r=0;r<e.length;r++)A.nestedProperty(t,"_input."+e[r]).get()||A.nestedProperty(t,e[r]).set(null)},p.supplyDataDefaults=function(t,e,n,r){var i,a,l,c=r._modules,u=r._visibleModules,g=r._basePlotModules,h=0,f=0;function I(t){e.push(t);var n=t._module;n&&(A.pushUnique(c,n),!0===t.visible&&A.pushUnique(u,n),A.pushUnique(g,t._module.basePlotModule),h++,!1!==t._input.visible&&f++)}r._transformModules=[];var C={},m=[],y=(n.template||{}).data||{},E=s.traceTemplater(y);for(i=0;i<t.length;i++){if(l=t[i],(a=E.newTrace(l)).uid=r._traceUids[i],p.supplyTraceDefaults(l,a,f,r,i),a.index=i,a._input=l,a._expandedIndex=h,a.transforms&&a.transforms.length)for(var v=!1!==l.visible&&!1===a.visible,B=b(a,e,n,r),Q=0;Q<B.length;Q++){var w=B[Q],x={_template:a._template,type:a.type,uid:a.uid+Q};v&&!1===w.visible&&delete w.visible,p.supplyTraceDefaults(w,x,h,r,i),d(x,w),x.index=i,x._input=l,x._fullInput=a,x._expandedIndex=h,x._expandedInput=w,I(x)}else a._fullInput=a,a._expandedInput=a,I(a);o.traceIs(a,"carpetAxis")&&(C[a.carpet]=a),o.traceIs(a,"carpetDependent")&&m.push(i)}for(i=0;i<m.length;i++)if((a=e[m[i]]).visible){var _=C[a.carpet];a._carpet=_,_&&_.visible?(a.xaxis=_.xaxis,a.yaxis=_.yaxis):a.visible=!1}},p.supplyAnimationDefaults=function(t){var e;t=t||{};var n={};function r(e,r){return A.coerce(t||{},n,g,e,r)}if(r("mode"),r("direction"),r("fromcurrent"),Array.isArray(t.frame))for(n.frame=[],e=0;e<t.frame.length;e++)n.frame[e]=p.supplyAnimationFrameDefaults(t.frame[e]||{});else n.frame=p.supplyAnimationFrameDefaults(t.frame||{});if(Array.isArray(t.transition))for(n.transition=[],e=0;e<t.transition.length;e++)n.transition[e]=p.supplyAnimationTransitionDefaults(t.transition[e]||{});else n.transition=p.supplyAnimationTransitionDefaults(t.transition||{});return n},p.supplyAnimationFrameDefaults=function(t){var e={};function n(n,r){return A.coerce(t||{},e,g.frame,n,r)}return n("duration"),n("redraw"),e},p.supplyAnimationTransitionDefaults=function(t){var e={};function n(n,r){return A.coerce(t||{},e,g.transition,n,r)}return n("duration"),n("easing"),e},p.supplyFrameDefaults=function(t){var e={};function n(n,r){return A.coerce(t,e,h,n,r)}return n("group"),n("name"),n("traces"),n("baseframe"),n("data"),n("layout"),e},p.supplyTraceDefaults=function(t,e,n,r,i){var a,s=r.colorway||l.defaults,c=s[n%s.length];function u(n,r){return A.coerce(t,e,p.attributes,n,r)}var g=u("visible");u("type"),u("name",r._traceWord+" "+i),u("uirevision",r.uirevision);var h=p.getModule(e);if(e._module=h,h){var d=h.basePlotModule,f=d.attr,I=d.attributes;if(f&&I){var C=r._subplots,m="";if("gl2d"!==d.name||g){if(Array.isArray(f))for(a=0;a<f.length;a++){var y=f[a],E=A.coerce(t,e,I,y);C[y]&&A.pushUnique(C[y],E),m+=E}else m=A.coerce(t,e,I,f);C[d.name]&&A.pushUnique(C[d.name],m)}}}return g&&(u("customdata"),u("ids"),u("meta"),o.traceIs(e,"showLegend")?(e._dfltShowLegend=!0,u("showlegend"),u("legendgroup")):e._dfltShowLegend=!1,h&&h.supplyDefaults(t,e,c,r),o.traceIs(e,"noOpacity")||u("opacity"),o.traceIs(e,"notLegendIsolatable")&&(e.visible=!!e.visible),o.traceIs(e,"noHover")||(e.hovertemplate||A.coerceHoverinfo(t,e,r),"parcats"!==e.type&&o.getComponentMethod("fx","supplyDefaults")(t,e,c,r)),h&&h.selectPoints&&u("selectedpoints"),p.supplyTransformDefaults(t,e,r)),e},p.hasMakesDataTransform=B,p.supplyTransformDefaults=function(t,e,n){if(e._length||B(t)){var r=n._globalTransforms||[],i=n._transformModules||[];if(Array.isArray(t.transforms)||0!==r.length)for(var o=t.transforms||[],a=r.concat(o),s=e.transforms=[],l=0;l<a.length;l++){var c,u=a[l],g=u.type,h=I[g],d=!(u._module&&u._module===h),f=h&&"function"==typeof h.transform;h||A.warn("Unrecognized transform type "+g+"."),h&&h.supplyDefaults&&(d||f)?((c=h.supplyDefaults(u,e,n,t)).type=g,c._module=h,A.pushUnique(i,h)):c=A.extendFlat({},u),s.push(c)}}},p.supplyLayoutGlobalDefaults=function(t,e,n){function r(n,r){return A.coerce(t,e,p.layoutAttributes,n,r)}var i=t.template;A.isPlainObject(i)&&(e.template=i,e._template=i.layout,e._dataTemplate=i.data);var a=A.coerceFont(r,"font");r("title.text",e._dfltTitle.plot),A.coerceFont(r,"title.font",{family:a.family,size:Math.round(1.4*a.size),color:a.color}),r("title.xref"),r("title.yref"),r("title.x"),r("title.y"),r("title.xanchor"),r("title.yanchor"),r("title.pad.t"),r("title.pad.r"),r("title.pad.b"),r("title.pad.l"),r("autosize",!(t.width&&t.height)),r("width"),r("height"),r("margin.l"),r("margin.r"),r("margin.t"),r("margin.b"),r("margin.pad"),r("margin.autoexpand"),t.width&&t.height&&p.sanitizeMargins(e),o.getComponentMethod("grid","sizeDefaults")(t,e),r("paper_bgcolor"),r("separators",n.decimal+n.thousands),r("hidesources"),r("colorway"),r("datarevision");var s=r("uirevision");r("editrevision",s),r("selectionrevision",s),r("modebar.orientation"),r("modebar.bgcolor",l.addOpacity(e.paper_bgcolor,.5));var c=l.contrast(l.rgb(e.modebar.bgcolor));r("modebar.color",l.addOpacity(c,.3)),r("modebar.activecolor",l.addOpacity(c,.7)),r("modebar.uirevision",s),r("meta"),A.isPlainObject(t.transition)&&(r("transition.duration"),r("transition.easing"),r("transition.ordering")),o.getComponentMethod("calendars","handleDefaults")(t,e,"calendar"),o.getComponentMethod("fx","supplyLayoutGlobalDefaults")(t,e,r)},p.plotAutoSize=function(t,e,n){var r,o,a=t._context||{},s=a.frameMargins,l=A.isPlotDiv(t);if(l&&t.emit("plotly_autosize"),a.fillFrame)r=window.innerWidth,o=window.innerHeight,document.body.style.overflow="hidden";else{var c=l?window.getComputedStyle(t):{};if(r=parseFloat(c.width)||parseFloat(c.maxWidth)||n.width,o=parseFloat(c.height)||parseFloat(c.maxHeight)||n.height,i(s)&&s>0){var u=1-2*s;r=Math.round(u*r),o=Math.round(u*o)}}var g=p.layoutAttributes.width.min,h=p.layoutAttributes.height.min;r<g&&(r=g),o<h&&(o=h);var d=!e.width&&Math.abs(n.width-r)>1,f=!e.height&&Math.abs(n.height-o)>1;(f||d)&&(d&&(n.width=r),f&&(n.height=o)),t._initialAutoSize||(t._initialAutoSize={width:r,height:o}),p.sanitizeMargins(n)},p.supplyLayoutModuleDefaults=function(t,e,n,r){var i,a,s,l=o.componentsRegistry,c=e._basePlotModules,u=o.subplotsRegistry.cartesian;for(i in l)(s=l[i]).includeBasePlot&&s.includeBasePlot(t,e);for(var g in c.length||c.push(u),e._has("cartesian")&&(o.getComponentMethod("grid","contentDefaults")(t,e),u.finalizeSubplots(t,e)),e._subplots)e._subplots[g].sort(A.subplotSort);for(a=0;a<c.length;a++)(s=c[a]).supplyLayoutDefaults&&s.supplyLayoutDefaults(t,e,n);var h=e._modules;for(a=0;a<h.length;a++)(s=h[a]).supplyLayoutDefaults&&s.supplyLayoutDefaults(t,e,n);var d=e._transformModules;for(a=0;a<d.length;a++)(s=d[a]).supplyLayoutDefaults&&s.supplyLayoutDefaults(t,e,n,r);for(i in l)(s=l[i]).supplyLayoutDefaults&&s.supplyLayoutDefaults(t,e,n)},p.purge=function(t){var e=t._fullLayout||{};void 0!==e._glcontainer&&(e._glcontainer.selectAll(".gl-canvas").remove(),e._glcontainer.remove(),e._glcanvas=null),void 0!==e._geocontainer&&e._geocontainer.remove(),e._modeBar&&e._modeBar.destroy(),t._transitionData&&(t._transitionData._interruptCallbacks&&(t._transitionData._interruptCallbacks.length=0),t._transitionData._animationRaf&&window.cancelAnimationFrame(t._transitionData._animationRaf)),A.clearThrottle(),A.clearResponsive(t),delete t.data,delete t.layout,delete t._fullData,delete t._fullLayout,delete t.calcdata,delete t.framework,delete t.empty,delete t.fid,delete t.undoqueue,delete t.undonum,delete t.autoplay,delete t.changed,delete t._promises,delete t._redrawTimer,delete t._hmlumcount,delete t._hmpixcount,delete t._transitionData,delete t._transitioning,delete t._initialAutoSize,delete t._transitioningWithDuration,delete t._dragging,delete t._dragged,delete t._dragdata,delete t._hoverdata,delete t._snapshotInProgress,delete t._editing,delete t._mouseDownTime,delete t._legendMouseDownTime,t.removeAllListeners&&t.removeAllListeners()},p.style=function(t){var e,n=t._fullLayout._visibleModules,r=[];for(e=0;e<n.length;e++){var i=n[e];i.style&&A.pushUnique(r,i.style)}for(e=0;e<r.length;e++)r[e](t)},p.sanitizeMargins=function(t){if(t&&t.margin){var e,n=t.width,r=t.height,i=t.margin,o=n-(i.l+i.r),a=r-(i.t+i.b);o<0&&(e=(n-1)/(i.l+i.r),i.l=Math.floor(e*i.l),i.r=Math.floor(e*i.r)),a<0&&(e=(r-1)/(i.t+i.b),i.t=Math.floor(e*i.t),i.b=Math.floor(e*i.b))}},p.clearAutoMarginIds=function(t){t._fullLayout._pushmarginIds={}},p.allowAutoMargin=function(t,e){t._fullLayout._pushmarginIds[e]=1},p.autoMargin=function(t,e,n){var r=t._fullLayout,i=r._pushmargin,o=r._pushmarginIds;if(!1!==r.margin.autoexpand){if(n){var a=n.pad;if(void 0===a){var s=r.margin;a=Math.min(12,s.l,s.r,s.t,s.b)}n.l+n.r>.5*r.width&&(n.l=n.r=0),n.b+n.t>.5*r.height&&(n.b=n.t=0);var A=void 0!==n.xl?n.xl:n.x,l=void 0!==n.xr?n.xr:n.x,c=void 0!==n.yt?n.yt:n.y,u=void 0!==n.yb?n.yb:n.y;i[e]={l:{val:A,size:n.l+a},r:{val:l,size:n.r+a},b:{val:u,size:n.b+a},t:{val:c,size:n.t+a}},o[e]=1}else delete i[e],delete o[e];r._replotting||p.doAutoMargin(t)}},p.doAutoMargin=function(t){var e=t._fullLayout;e._size||(e._size={}),Q(e);var n=e._size,r=e.margin,a=A.extendFlat({},n),s=r.l,l=r.r,c=r.t,u=r.b,g=e.width,h=e.height,d=e._pushmargin,f=e._pushmarginIds;if(!1!==e.margin.autoexpand){for(var I in d)f[I]||delete d[I];for(var C in d.base={l:{val:0,size:s},r:{val:1,size:l},t:{val:1,size:c},b:{val:0,size:u}},d){var m=d[C].l||{},y=d[C].b||{},E=m.val,v=m.size,B=y.val,b=y.size;for(var w in d){if(i(v)&&d[w].r){var x=d[w].r.val,_=d[w].r.size;if(x>E){var D=(v*x+(_-g)*E)/(x-E),M=(_*(1-E)+(v-g)*(1-x))/(x-E);D>=0&&M>=0&&g-(D+M)>0&&D+M>s+l&&(s=D,l=M)}}if(i(b)&&d[w].t){var k=d[w].t.val,S=d[w].t.size;if(k>B){var L=(b*k+(S-h)*B)/(k-B),R=(S*(1-B)+(b-h)*(1-k))/(k-B);L>=0&&R>=0&&h-(R+L)>0&&L+R>u+c&&(u=L,c=R)}}}}}if(n.l=Math.round(s),n.r=Math.round(l),n.t=Math.round(c),n.b=Math.round(u),n.p=Math.round(r.pad),n.w=Math.round(g)-n.l-n.r,n.h=Math.round(h)-n.t-n.b,!e._replotting&&p.didMarginChange(a,n))return"_redrawFromAutoMarginCount"in e?e._redrawFromAutoMarginCount++:e._redrawFromAutoMarginCount=1,o.call("plot",t)};var w=["l","r","t","b","p","w","h"];function x(t,e,n){var r=!1,i=[p.previousPromises,function(){if(t._transitionData)return t._transitioning=!1,function(t){var e=Promise.resolve();if(!t)return e;for(;t.length;)e=e.then(t.shift());return e}(t._transitionData._interruptCallbacks)},n.prepareFn,p.rehover,function(){return t.emit("plotly_transitioning",[]),new Promise(function(i){t._transitioning=!0,e.duration>0&&(t._transitioningWithDuration=!0),t._transitionData._interruptCallbacks.push(function(){r=!0}),n.redraw&&t._transitionData._interruptCallbacks.push(function(){return o.call("redraw",t)}),t._transitionData._interruptCallbacks.push(function(){t.emit("plotly_transitioninterrupted",[])});var a=0,s=0;function A(){return a++,function(){var e;s++,r||s!==a||(e=i,t._transitionData&&(function(t){if(t)for(;t.length;)t.shift()}(t._transitionData._interruptCallbacks),Promise.resolve().then(function(){if(n.redraw)return o.call("redraw",t)}).then(function(){t._transitioning=!1,t._transitioningWithDuration=!1,t.emit("plotly_transitioned",[])}).then(e)))}}n.runFn(A),setTimeout(A())})}],a=A.syncOrAsync(i,t);return a&&a.then||(a=Promise.resolve()),a.then(function(){return t})}p.didMarginChange=function(t,e){for(var n=0;n<w.length;n++){var r=w[n],o=t[r],a=e[r];if(!i(o)||Math.abs(a-o)>1)return!0}return!1},p.graphJson=function(t,e,n,r,i){(i&&e&&!t._fullData||i&&!e&&!t._fullLayout)&&p.supplyDefaults(t);var o=i?t._fullData:t.data,a=i?t._fullLayout:t.layout,s=(t._transitionData||{})._frames;function l(t){if("function"==typeof t)return null;if(A.isPlainObject(t)){var e,r,i={};for(e in t)if("function"!=typeof t[e]&&-1===["_","["].indexOf(e.charAt(0))){if("keepdata"===n){if("src"===e.substr(e.length-3))continue}else if("keepstream"===n){if("string"==typeof(r=t[e+"src"])&&r.indexOf(":")>0&&!A.isPlainObject(t.stream))continue}else if("keepall"!==n&&"string"==typeof(r=t[e+"src"])&&r.indexOf(":")>0)continue;i[e]=l(t[e])}return i}return Array.isArray(t)?t.map(l):A.isTypedArray(t)?A.simpleMap(t,A.identity):A.isJSDate(t)?A.ms2DateTimeLocal(+t):t}var c={data:(o||[]).map(function(t){var n=l(t);return e&&delete n.fit,n})};return e||(c.layout=l(a)),t.framework&&t.framework.isPolar&&(c=t.framework.getConfig()),s&&(c.frames=l(s)),"object"===r?c:JSON.stringify(c)},p.modifyFrames=function(t,e){var n,r,i,o=t._transitionData._frames,a=t._transitionData._frameHash;for(n=0;n<e.length;n++)switch((r=e[n]).type){case"replace":i=r.value;var s=(o[r.index]||{}).name,A=i.name;o[r.index]=a[A]=i,A!==s&&(delete a[s],a[A]=i);break;case"insert":a[(i=r.value).name]=i,o.splice(r.index,0,i);break;case"delete":delete a[(i=o[r.index]).name],o.splice(r.index,1)}return Promise.resolve()},p.computeFrame=function(t,e){var n,r,i,o,a=t._transitionData._frameHash;if(!e)throw new Error("computeFrame must be given a string frame name");var s=a[e.toString()];if(!s)return!1;for(var A=[s],l=[s.name];s.baseframe&&(s=a[s.baseframe.toString()])&&-1===l.indexOf(s.name);)A.push(s),l.push(s.name);for(var c={};s=A.pop();)if(s.layout&&(c.layout=p.extendLayout(c.layout,s.layout)),s.data){if(c.data||(c.data=[]),!(r=s.traces))for(r=[],n=0;n<s.data.length;n++)r[n]=n;for(c.traces||(c.traces=[]),n=0;n<s.data.length;n++)null!=(i=r[n])&&(-1===(o=c.traces.indexOf(i))&&(o=c.data.length,c.traces[o]=i),c.data[o]=p.extendTrace(c.data[o],s.data[n]))}return c},p.recomputeFrameHash=function(t){for(var e=t._transitionData._frameHash={},n=t._transitionData._frames,r=0;r<n.length;r++){var i=n[r];i&&i.name&&(e[i.name]=i)}},p.extendObjectWithContainers=function(t,e,n){var r,i,o,a,s,l,c,u=A.extendDeepNoArrays({},e||{}),g=A.expandObjectPaths(u),h={};if(n&&n.length)for(o=0;o<n.length;o++)void 0===(i=(r=A.nestedProperty(g,n[o])).get())?A.nestedProperty(h,n[o]).set(null):(r.set(null),A.nestedProperty(h,n[o]).set(i));if(t=A.extendDeepNoArrays(t||{},g),n&&n.length)for(o=0;o<n.length;o++)if(l=A.nestedProperty(h,n[o]).get()){for(c=(s=A.nestedProperty(t,n[o])).get(),Array.isArray(c)||(c=[],s.set(c)),a=0;a<l.length;a++){var d=l[a];c[a]=null===d?null:p.extendObjectWithContainers(c[a],d)}s.set(c)}return t},p.dataArrayContainers=["transforms","dimensions"],p.layoutArrayContainers=o.layoutArrayContainers,p.extendTrace=function(t,e){return p.extendObjectWithContainers(t,e,p.dataArrayContainers)},p.extendLayout=function(t,e){return p.extendObjectWithContainers(t,e,p.layoutArrayContainers)},p.transition=function(t,e,n,r,i,o){var a={redraw:i.redraw},s={},l=[];return a.prepareFn=function(){for(var i=Array.isArray(e)?e.length:0,o=r.slice(0,i),a=0;a<o.length;a++){var c=o[a],u=t._fullData[c]._module;if(u){if(u.animatable){var g=u.basePlotModule.name;s[g]||(s[g]=[]),s[g].push(c)}t.data[o[a]]=p.extendTrace(t.data[o[a]],e[a])}}var h=A.expandObjectPaths(A.extendDeepNoArrays({},n)),d=/^[xy]axis[0-9]*$/;for(var f in h)d.test(f)&&delete h[f].range;p.extendLayout(t.layout,h),delete t.calcdata,p.supplyDefaults(t),p.doCalcdata(t);var I=A.expandObjectPaths(n);if(I){var C=t._fullLayout._plots;for(var m in C){var y,E,v,B,b=C[m],Q=b.xaxis,w=b.yaxis,x=Q.range.slice(),_=w.range.slice();Array.isArray(I[Q._name+".range"])?y=I[Q._name+".range"].slice():Array.isArray((I[Q._name]||{}).range)&&(y=I[Q._name].range.slice()),Array.isArray(I[w._name+".range"])?E=I[w._name+".range"].slice():Array.isArray((I[w._name]||{}).range)&&(E=I[w._name].range.slice()),x&&y&&(x[0]!==y[0]||x[1]!==y[1])&&(v={xr0:x,xr1:y}),_&&E&&(_[0]!==E[0]||_[1]!==E[1])&&(B={yr0:_,yr1:E}),(v||B)&&l.push(A.extendFlat({plotinfo:b},v,B))}}return Promise.resolve()},a.runFn=function(e){var r,i,a=t._fullLayout._basePlotModules,c=l.length;if(n)for(i=0;i<a.length;i++)a[i].transitionAxes&&a[i].transitionAxes(t,l,o,e);for(var u in c?((r=A.extendFlat({},o)).duration=0,delete s.cartesian):r=o,s){var g=s[u];t._fullData[g[0]]._module.basePlotModule.plot(t,g,r,e)}},x(t,o,a)},p.transitionFromReact=function(t,e,n,r){var i=t._fullLayout,o=i.transition,a={},s=[];return a.prepareFn=function(){var t=i._plots;for(var o in a.redraw=!1,"some"===e.anim&&(a.redraw=!0),"some"===n.anim&&(a.redraw=!0),t){var l,c,u=t[o],g=u.xaxis,h=u.yaxis,d=r[g._name].range.slice(),f=r[h._name].range.slice(),p=g.range.slice(),I=h.range.slice();g.setScale(),h.setScale(),d[0]===p[0]&&d[1]===p[1]||(l={xr0:d,xr1:p}),f[0]===I[0]&&f[1]===I[1]||(c={yr0:f,yr1:I}),(l||c)&&s.push(A.extendFlat({plotinfo:u},l,c))}return Promise.resolve()},a.runFn=function(n){for(var r,i,a,l=t._fullData,c=t._fullLayout._basePlotModules,u=[],g=0;g<l.length;g++)u.push(g);function h(){for(var e=0;e<c.length;e++)c[e].transitionAxes&&c[e].transitionAxes(t,s,r,n)}function d(){for(var e=0;e<c.length;e++)c[e].plot(t,a,i,n)}s.length&&e.anim?"traces first"===o.ordering?(r=A.extendFlat({},o,{duration:0}),a=u,i=o,d(),setTimeout(h,o.duration)):(r=o,a=null,i=A.extendFlat({},o,{duration:0}),h(),d()):s.length?(r=o,h()):e.anim&&(a=u,i=o,d())},x(t,o,a)},p.doCalcdata=function(t,e){var n,r,i,s,l=u.list(t),g=t._fullData,h=t._fullLayout,d=new Array(g.length),f=(t.calcdata||[]).slice();for(t.calcdata=d,h._numBoxes=0,h._numViolins=0,h._violinScaleGroupStats={},t._hmpixcount=0,t._hmlumcount=0,h._piecolormap={},h._sunburstcolormap={},h._funnelareacolormap={},i=0;i<g.length;i++)Array.isArray(e)&&-1===e.indexOf(i)&&(d[i]=f[i]);for(i=0;i<g.length;i++)(n=g[i])._arrayAttrs=a.findArrayAttributes(n),n._extremes={};var p=h._subplots.polar||[];for(i=0;i<p.length;i++)l.push(h[p[i]].radialaxis,h[p[i]].angularaxis);var C=!1;function m(e){if(n=g[e],r=n._module,!0===n.visible&&n.transforms){if(r&&r.calc){var i=r.calc(t,n);i[0]&&i[0].t&&i[0].t._scene&&delete i[0].t._scene.dirty}for(s=0;s<n.transforms.length;s++){var o=n.transforms[s];(r=I[o.type])&&r.calcTransform&&(n._hasCalcTransform=!0,C=!0,r.calcTransform(t,n,o))}}}function y(e,i){if(n=g[e],!!(r=n._module).isContainer===i){var o=[];if(!0===n.visible&&0!==n._length){delete n._indexToPoints;var a=n.transforms||[];for(s=a.length-1;s>=0;s--)if(a[s].enabled){n._indexToPoints=a[s]._indexToPoints;break}r&&r.calc&&(o=r.calc(t,n))}Array.isArray(o)&&o[0]||(o=[{x:c,y:c}]),o[0].t||(o[0].t={}),o[0].trace=n,d[e]=o}}for(D(l,g),i=0;i<g.length;i++)y(i,!0);for(i=0;i<g.length;i++)m(i);for(C&&D(l,g),i=0;i<g.length;i++)y(i,!0);for(i=0;i<g.length;i++)y(i,!1);M(t);var E=function(t,e){var n,r,i,a,s,l=[];function c(t,n,r){var i=n._id.charAt(0);if("histogram2dcontour"===t){var o=n._counterAxes[0],a=u.getFromId(e,o),s="x"===i||"x"===o&&"category"===a.type,A="y"===i||"y"===o&&"category"===a.type;return function(t,e){return 0===t||0===e?-1:s&&t===r[e].length-1?-1:A&&e===r.length-1?-1:("y"===i?e:t)-1}}return function(t,e){return"y"===i?e:t}}var g={min:function(t){return A.aggNums(Math.min,null,t)},max:function(t){return A.aggNums(Math.max,null,t)},sum:function(t){return A.aggNums(function(t,e){return t+e},null,t)},total:function(t){return A.aggNums(function(t,e){return t+e},null,t)},mean:function(t){return A.mean(t)},median:function(t){return A.median(t)}};for(n=0;n<t.length;n++){var h=t[n];if("category"===h.type){var d=h.categoryorder.match(_);if(d){var f=d[1],p=d[2],I=[];for(r=0;r<h._categories.length;r++)I.push([h._categories[r],[]]);for(r=0;r<h._traceIndices.length;r++){var C=h._traceIndices[r],m=e._fullData[C],y=h._id.charAt(0);if(!0===m.visible){var E=m.type;o.traceIs(m,"histogram")&&(delete m._xautoBinFinished,delete m._yautoBinFinished);var v=e.calcdata[C];for(i=0;i<v.length;i++){var B,b,Q,w=v[i];if("splom"===E){var x=m._axesDim[h._id];if("y"===y){var D=m._diag[x][0];D&&(h=e._fullLayout[u.id2name(D)])}var M=w.trace.dimensions[x].values;for(a=0;a<M.length;a++)for(B=M[a],b=h._categoriesMap[B],s=0;s<w.trace.dimensions.length;s++)if(s!==x){var k=w.trace.dimensions[s];I[b][1].push(k.values[a])}}else if("scattergl"===E){for(a=0;a<w.t.x.length;a++)"x"===y&&(B=w.t.x[a],b=B,Q=w.t.y[a]),"y"===y&&(B=w.t.y[a],b=B,Q=w.t.x[a]),I[b][1].push(Q);w.t&&w.t._scene&&delete w.t._scene.dirty}else if(w.hasOwnProperty("z")){Q=w.z;var S=c(m.type,h,Q);for(a=0;a<Q.length;a++)for(s=0;s<Q[a].length;s++)(b=S(s,a))+1&&I[b][1].push(Q[a][s])}else for("x"===y?(B=w.p+1?w.p:w.x,Q=w.s||w.v||w.y):"y"===y&&(B=w.p+1?w.p:w.y,Q=w.s||w.v||w.x),Array.isArray(Q)||(Q=[Q]),a=0;a<Q.length;a++)I[B][1].push(Q[a])}}}h._categoriesValue=I;var L=[];for(r=0;r<I.length;r++)L.push([I[r][0],g[f](I[r][1])]);L.sort(function(t,e){return t[1]-e[1]}),h._categoriesAggregatedValue=L,h._initialCategories=L.map(function(t){return t[0]}),"descending"===p&&h._initialCategories.reverse(),l=l.concat(h.sortByInitialCategories())}}}return l}(l,t);if(E.length){for(h._numBoxes=0,h._numViolins=0,i=0;i<E.length;i++)y(E[i],!0);for(i=0;i<E.length;i++)y(E[i],!1);M(t)}o.getComponentMethod("fx","calc")(t),o.getComponentMethod("errorbars","calc")(t)};var _=/(total|sum|min|max|mean|median) (ascending|descending)/;function D(t,e){for(var n=0;n<t.length;n++){var r=t[n];r.clearCalc(),"multicategory"===r.type&&r.setupMultiCategory(e)}}function M(t){var e,n,r,i=t._fullLayout,o=i._visibleModules,a={};for(n=0;n<o.length;n++){var s=o[n],l=s.crossTraceCalc;if(l){var c=s.basePlotModule.name;a[c]?A.pushUnique(a[c],l):a[c]=[l]}}for(r in a){var u=a[r],g=i._subplots[r];if(Array.isArray(g))for(e=0;e<g.length;e++){var h=g[e],d="cartesian"===r?i._plots[h]:i[h];for(n=0;n<u.length;n++)u[n](t,d,h)}else for(n=0;n<u.length;n++)u[n](t)}}p.rehover=function(t){t._fullLayout._rehover&&t._fullLayout._rehover()},p.redrag=function(t){t._fullLayout._redrag&&t._fullLayout._redrag()},p.generalUpdatePerTraceModule=function(t,e,n,r){var i,o=e.traceHash,a={};for(i=0;i<n.length;i++){var s=n[i],l=s[0].trace;l.visible&&(a[l.type]=a[l.type]||[],a[l.type].push(s))}for(var c in o)if(!a[c]){var u=o[c][0];u[0].trace.visible=!1,a[c]=[u]}for(var g in a){var h=a[g];h[0][0].trace._module.plot(t,e,A.filterVisible(h),r)}e.traceHash=a}},{"../components/color":593,"../constants/numerical":695,"../lib":719,"../plot_api/plot_schema":756,"../plot_api/plot_template":757,"../registry":847,"./animation_attributes":762,"./attributes":764,"./cartesian/axis_ids":770,"./command":791,"./font_attributes":793,"./frame_attributes":794,"./layout_attributes":819,d3:163,"fast-isnumeric":225}],829:[function(t,e,n){"use strict";e.exports={attr:"subplot",name:"polar",axisNames:["angularaxis","radialaxis"],axisName2dataArray:{angularaxis:"theta",radialaxis:"r"},layerNames:["draglayer","plotbg","backplot","angular-grid","radial-grid","frontplot","angular-line","radial-line","angular-axis","radial-axis"],radialDragBoxSize:50,angularDragBoxSize:30,cornerLen:25,cornerHalfWidth:2,MINDRAG:8,MINZOOM:20,OFFEDGE:20}},{}],830:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../lib/polygon").tester,o=r.findIndexOfMin,a=r.isAngleInsideSector,s=r.angleDelta,A=r.angleDist;function l(t,e,n,r){var i,o,a=r[0],s=r[1],A=u(Math.sin(e)-Math.sin(t)),l=u(Math.cos(e)-Math.cos(t)),c=Math.tan(n),g=u(1/c),h=A/l,d=s-h*a;return g?A&&l?o=c*(i=d/(c-h)):l?(i=s*g,o=s):(i=a,o=a*c):A&&l?(i=0,o=d):l?(i=0,o=s):i=o=NaN,[i,o]}function c(t,e,n,i){return r.isFullCircle([e,n])?function(t,e){var n,r=e.length,i=new Array(r+1);for(n=0;n<r;n++){var o=e[n];i[n]=[t*Math.cos(o),t*Math.sin(o)]}return i[n]=i[0].slice(),i}(t,i):function(t,e,n,i){var s,c,u=i.length,g=[];function h(e){return[t*Math.cos(e),t*Math.sin(e)]}function d(t,e,n){return l(t,e,n,h(t))}function f(t){return r.mod(t,u)}function p(t){return a(t,[e,n])}var I=o(i,function(t){return p(t)?A(t,e):1/0}),C=d(i[I],i[f(I-1)],e);for(g.push(C),s=I,c=0;c<u;s++,c++){var m=i[f(s)];if(!p(m))break;g.push(h(m))}var y=o(i,function(t){return p(t)?A(t,n):1/0}),E=d(i[y],i[f(y+1)],n);return g.push(E),g.push([0,0]),g.push(g[0].slice()),g}(t,e,n,i)}function u(t){return Math.abs(t)>1e-10?t:0}function g(t,e,n){e=e||0,n=n||0;for(var r=t.length,i=new Array(r),o=0;o<r;o++){var a=t[o];i[o]=[e+a[0],n-a[1]]}return i}e.exports={isPtInsidePolygon:function(t,e,n,r,o){if(!a(e,r))return!1;var s,A;n[0]<n[1]?(s=n[0],A=n[1]):(s=n[1],A=n[0]);var l=i(c(s,r[0],r[1],o)),u=i(c(A,r[0],r[1],o)),g=[t*Math.cos(e),t*Math.sin(e)];return u.contains(g)&&!l.contains(g)},findPolygonOffset:function(t,e,n,r){for(var i=1/0,o=1/0,a=c(t,e,n,r),s=0;s<a.length;s++){var A=a[s];i=Math.min(i,A[0]),o=Math.min(o,-A[1])}return[i,o]},findEnclosingVertexAngles:function(t,e){var n=o(e,function(e){var n=s(e,t);return n>0?n:1/0}),i=r.mod(n+1,e.length);return[e[n],e[i]]},findIntersectionXY:l,findXYatLength:function(t,e,n,r){var i=-e*n,o=e*e+1,a=2*(e*i-n),s=i*i+n*n-t*t,A=Math.sqrt(a*a-4*o*s),l=(-a+A)/(2*o),c=(-a-A)/(2*o);return[[l,e*l+i+r],[c,e*c+i+r]]},clampTiny:u,pathPolygon:function(t,e,n,r,i,o){return"M"+g(c(t,e,n,r),i,o).join("L")},pathPolygonAnnulus:function(t,e,n,r,i,o,a){var s,A;t<e?(s=t,A=e):(s=e,A=t);var l=g(c(s,n,r,i),o,a);return"M"+g(c(A,n,r,i),o,a).reverse().join("L")+"M"+l.join("L")}}},{"../../lib":719,"../../lib/polygon":731}],831:[function(t,e,n){"use strict";var r=t("../get_data").getSubplotCalcData,i=t("../../lib").counterRegex,o=t("./polar"),a=t("./constants"),s=a.attr,A=a.name,l=i(A),c={};c[s]={valType:"subplotid",dflt:A,editType:"calc"},e.exports={attr:s,name:A,idRoot:A,idRegex:l,attrRegex:l,attributes:c,layoutAttributes:t("./layout_attributes"),supplyLayoutDefaults:t("./layout_defaults"),plot:function(t){for(var e=t._fullLayout,n=t.calcdata,i=e._subplots[A],a=0;a<i.length;a++){var s=i[a],l=r(n,A,s),c=e[s]._subplot;c||(c=o(t,s),e[s]._subplot=c),c.plot(l,e,t._promises)}},clean:function(t,e,n,r){for(var i=r._subplots[A]||[],o=r._has&&r._has("gl"),a=e._has&&e._has("gl"),s=o&&!a,l=0;l<i.length;l++){var c=i[l],u=r[c]._subplot;if(!e[c]&&u)for(var g in u.framework.remove(),u.layers["radial-axis-title"].remove(),u.clipPaths)u.clipPaths[g].remove();s&&u._scene&&(u._scene.destroy(),u._scene=null)}},toSVG:t("../cartesian").toSVG}},{"../../lib":719,"../cartesian":778,"../get_data":802,"./constants":829,"./layout_attributes":832,"./layout_defaults":833,"./polar":840}],832:[function(t,e,n){"use strict";var r=t("../../components/color/attributes"),i=t("../cartesian/layout_attributes"),o=t("../domain").attributes,a=t("../../lib").extendFlat,s=t("../../plot_api/edit_types").overrideAll,A=s({color:i.color,showline:a({},i.showline,{dflt:!0}),linecolor:i.linecolor,linewidth:i.linewidth,showgrid:a({},i.showgrid,{dflt:!0}),gridcolor:i.gridcolor,gridwidth:i.gridwidth},"plot","from-root"),l=s({tickmode:i.tickmode,nticks:i.nticks,tick0:i.tick0,dtick:i.dtick,tickvals:i.tickvals,ticktext:i.ticktext,ticks:i.ticks,ticklen:i.ticklen,tickwidth:i.tickwidth,tickcolor:i.tickcolor,showticklabels:i.showticklabels,showtickprefix:i.showtickprefix,tickprefix:i.tickprefix,showticksuffix:i.showticksuffix,ticksuffix:i.ticksuffix,showexponent:i.showexponent,exponentformat:i.exponentformat,separatethousands:i.separatethousands,tickfont:i.tickfont,tickangle:i.tickangle,tickformat:i.tickformat,tickformatstops:i.tickformatstops,layer:i.layer},"plot","from-root"),c={visible:a({},i.visible,{dflt:!0}),type:a({},i.type,{values:["-","linear","log","date","category"]}),autorange:a({},i.autorange,{editType:"plot"}),rangemode:{valType:"enumerated",values:["tozero","nonnegative","normal"],dflt:"tozero",editType:"calc"},range:a({},i.range,{items:[{valType:"any",editType:"plot",impliedEdits:{"^autorange":!1}},{valType:"any",editType:"plot",impliedEdits:{"^autorange":!1}}],editType:"plot"}),categoryorder:i.categoryorder,categoryarray:i.categoryarray,angle:{valType:"angle",editType:"plot"},side:{valType:"enumerated",values:["clockwise","counterclockwise"],dflt:"clockwise",editType:"plot"},title:s(i.title,"plot","from-root"),hoverformat:i.hoverformat,uirevision:{valType:"any",editType:"none"},editType:"calc",_deprecated:{title:i._deprecated.title,titlefont:i._deprecated.titlefont}};c.title.text.dflt="",a(c,A,l);var u={visible:a({},i.visible,{dflt:!0}),type:{valType:"enumerated",values:["-","linear","category"],dflt:"-",editType:"calc",_noTemplating:!0},categoryorder:i.categoryorder,categoryarray:i.categoryarray,thetaunit:{valType:"enumerated",values:["radians","degrees"],dflt:"degrees",editType:"calc"},period:{valType:"number",editType:"calc",min:0},direction:{valType:"enumerated",values:["counterclockwise","clockwise"],dflt:"counterclockwise",editType:"calc"},rotation:{valType:"angle",editType:"calc"},hoverformat:i.hoverformat,uirevision:{valType:"any",editType:"none"},editType:"calc"};a(u,A,l),e.exports={domain:o({name:"polar",editType:"plot"}),sector:{valType:"info_array",items:[{valType:"number",editType:"plot"},{valType:"number",editType:"plot"}],dflt:[0,360],editType:"plot"},hole:{valType:"number",min:0,max:1,dflt:0,editType:"plot"},bgcolor:{valType:"color",editType:"plot",dflt:r.background},radialaxis:c,angularaxis:u,gridshape:{valType:"enumerated",values:["circular","linear"],dflt:"circular",editType:"plot"},uirevision:{valType:"any",editType:"none"},editType:"calc"}},{"../../components/color/attributes":592,"../../lib":719,"../../plot_api/edit_types":750,"../cartesian/layout_attributes":779,"../domain":792}],833:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/color"),o=t("../../plot_api/plot_template"),a=t("../subplot_defaults"),s=t("../get_data").getSubplotData,A=t("../cartesian/tick_value_defaults"),l=t("../cartesian/tick_mark_defaults"),c=t("../cartesian/tick_label_defaults"),u=t("../cartesian/category_order_defaults"),g=t("../cartesian/line_grid_defaults"),h=t("../cartesian/axis_autotype"),d=t("./layout_attributes"),f=t("./set_convert"),p=t("./constants"),I=p.axisNames;function C(t,e,n,a){var h=n("bgcolor");a.bgColor=i.combine(h,a.paper_bgcolor);var C=n("sector");n("hole");var y,E=s(a.fullData,p.name,a.id),v=a.layoutOut;function B(t,e){return n(y+"."+t,e)}for(var b=0;b<I.length;b++){y=I[b],r.isPlainObject(t[y])||(t[y]={});var Q=t[y],w=o.newContainer(e,y);w._id=w._name=y,w._attr=a.id+"."+y,w._traceIndices=E.map(function(t){return t._expandedIndex});var x=p.axisName2dataArray[y],_=m(Q,w,B,E,x);u(Q,w,B,{axData:E,dataAttr:x});var D,M,k=B("visible");switch(f(w,e,v),B("uirevision",e.uirevision),k&&(M=(D=B("color"))===Q.color?D:a.font.color),w._m=1,y){case"radialaxis":var S=B("autorange",!w.isValidRange(Q.range));Q.autorange=S,!S||"linear"!==_&&"-"!==_||B("rangemode"),"reversed"===S&&(w._m=-1),B("range"),w.cleanRange("range",{dfltRange:[0,1]}),k&&(B("side"),B("angle",C[0]),B("title.text"),r.coerceFont(B,"title.font",{family:a.font.family,size:Math.round(1.2*a.font.size),color:M}));break;case"angularaxis":if("date"===_){r.log("Polar plots do not support date angular axes yet.");for(var L=0;L<E.length;L++)E[L].visible=!1;_=Q.type=w.type="linear"}B("linear"===_?"thetaunit":"period");var R=B("direction");B("rotation",{counterclockwise:0,clockwise:90}[R])}k&&(A(Q,w,B,w.type),c(Q,w,B,w.type,{tickSuffixDflt:"degrees"===w.thetaunit?"°":void 0}),l(Q,w,B,{outerTicks:!0}),B("showticklabels")&&(r.coerceFont(B,"tickfont",{family:a.font.family,size:a.font.size,color:M}),B("tickangle"),B("tickformat")),g(Q,w,B,{dfltColor:D,bgColor:a.bgColor,blend:60,showLine:!0,showGrid:!0,noZeroLine:!0,attributes:d[y]}),B("layer")),"category"!==_&&B("hoverformat"),w._input=Q}"category"===e.angularaxis.type&&n("gridshape")}function m(t,e,n,r,i){if("-"===n("type")){for(var o,a=0;a<r.length;a++)if(r[a].visible){o=r[a];break}o&&o[i]&&(e.type=h(o[i],"gregorian")),"-"===e.type?e.type="linear":t.type=e.type}return e.type}e.exports=function(t,e,n){a(t,e,n,{type:p.name,attributes:d,handleDefaults:C,font:e.font,paper_bgcolor:e.paper_bgcolor,fullData:n,layoutOut:e})}},{"../../components/color":593,"../../lib":719,"../../plot_api/plot_template":757,"../cartesian/axis_autotype":768,"../cartesian/category_order_defaults":771,"../cartesian/line_grid_defaults":781,"../cartesian/tick_label_defaults":786,"../cartesian/tick_mark_defaults":787,"../cartesian/tick_value_defaults":788,"../get_data":802,"../subplot_defaults":842,"./constants":829,"./layout_attributes":832,"./set_convert":841}],834:[function(t,e,n){"use strict";var r=t("../../../traces/scatter/attributes"),i=r.marker,o=t("../../../lib/extend").extendFlat;["Area traces are deprecated!","Please switch to the *barpolar* trace type."].join(" "),e.exports={r:o({},r.r,{}),t:o({},r.t,{}),marker:{color:o({},i.color,{}),size:o({},i.size,{}),symbol:o({},i.symbol,{}),opacity:o({},i.opacity,{}),editType:"calc"}}},{"../../../lib/extend":710,"../../../traces/scatter/attributes":1112}],835:[function(t,e,n){"use strict";var r=t("../../cartesian/layout_attributes"),i=t("../../../lib/extend").extendFlat,o=t("../../../plot_api/edit_types").overrideAll,a=["Legacy polar charts are deprecated!","Please switch to *polar* subplots."].join(" "),s=i({},r.domain,{});function A(t,e){return i({},e,{showline:{valType:"boolean"},showticklabels:{valType:"boolean"},tickorientation:{valType:"enumerated",values:["horizontal","vertical"]},ticklen:{valType:"number",min:0},tickcolor:{valType:"color"},ticksuffix:{valType:"string"},endpadding:{valType:"number",description:a},visible:{valType:"boolean"}})}e.exports=o({radialaxis:A(0,{range:{valType:"info_array",items:[{valType:"number"},{valType:"number"}]},domain:s,orientation:{valType:"number"}}),angularaxis:A(0,{range:{valType:"info_array",items:[{valType:"number",dflt:0},{valType:"number",dflt:360}]},domain:s}),layout:{direction:{valType:"enumerated",values:["clockwise","counterclockwise"]},orientation:{valType:"angle"}}},"plot","nested")},{"../../../lib/extend":710,"../../../plot_api/edit_types":750,"../../cartesian/layout_attributes":779}],836:[function(t,e,n){"use strict";(e.exports=t("./micropolar")).manager=t("./micropolar_manager")},{"./micropolar":837,"./micropolar_manager":838}],837:[function(t,e,n){var r=t("d3"),i=t("../../../lib").extendDeepAll,o=t("../../../constants/alignment").MID_SHIFT,a=e.exports={version:"0.2.2"};a.Axis=function(){var t,e,n,s,A={data:[],layout:{}},l={},c={},u=r.dispatch("hover"),g={render:function(l){return function(l){e=l||e;var u=A.data,g=A.layout;("string"==typeof e||e.nodeName)&&(e=r.select(e)),e.datum(u).each(function(e,A){var l=e.slice();c={data:a.util.cloneJson(l),layout:a.util.cloneJson(g)};var u=0;l.forEach(function(t,e){t.color||(t.color=g.defaultColorRange[u],u=(u+1)%g.defaultColorRange.length),t.strokeColor||(t.strokeColor="LinePlot"===t.geometry?t.color:r.rgb(t.color).darker().toString()),c.data[e].color=t.color,c.data[e].strokeColor=t.strokeColor,c.data[e].strokeDash=t.strokeDash,c.data[e].strokeSize=t.strokeSize});var h=l.filter(function(t,e){var n=t.visible;return void 0===n||!0===n}),d=!1,f=h.map(function(t,e){return d=d||void 0!==t.groupId,t});if(d){var p=r.nest().key(function(t,e){return void 0!==t.groupId?t.groupId:"unstacked"}).entries(f),I=[],C=p.map(function(t,e){if("unstacked"===t.key)return t.values;var n=t.values[0].r.map(function(t,e){return 0});return t.values.forEach(function(t,e,r){t.yStack=[n],I.push(n),n=a.util.sumArrays(t.r,n)}),t.values});h=r.merge(C)}h.forEach(function(t,e){t.t=Array.isArray(t.t[0])?t.t:[t.t],t.r=Array.isArray(t.r[0])?t.r:[t.r]});var m=Math.min(g.width-g.margin.left-g.margin.right,g.height-g.margin.top-g.margin.bottom)/2;m=Math.max(10,m);var y,E=[g.margin.left+m,g.margin.top+m];y=d?[0,r.max(a.util.sumArrays(a.util.arrayLast(h).r[0],a.util.arrayLast(I)))]:r.extent(a.util.flattenArray(h.map(function(t,e){return t.r}))),g.radialAxis.domain!=a.DATAEXTENT&&(y[0]=0),n=r.scale.linear().domain(g.radialAxis.domain!=a.DATAEXTENT&&g.radialAxis.domain?g.radialAxis.domain:y).range([0,m]),c.layout.radialAxis.domain=n.domain();var v,B=a.util.flattenArray(h.map(function(t,e){return t.t})),b="string"==typeof B[0];b&&(B=a.util.deduplicate(B),v=B.slice(),B=r.range(B.length),h=h.map(function(t,e){var n=t;return t.t=[B],d&&(n.yStack=t.yStack),n}));var Q=h.filter(function(t,e){return"LinePlot"===t.geometry||"DotPlot"===t.geometry}).length===h.length,w=null===g.needsEndSpacing?b||!Q:g.needsEndSpacing,x=g.angularAxis.domain&&g.angularAxis.domain!=a.DATAEXTENT&&!b&&g.angularAxis.domain[0]>=0?g.angularAxis.domain:r.extent(B),_=Math.abs(B[1]-B[0]);Q&&!b&&(_=0);var D=x.slice();w&&b&&(D[1]+=_);var M=g.angularAxis.ticksCount||4;M>8&&(M=M/(M/8)+M%8),g.angularAxis.ticksStep&&(M=(D[1]-D[0])/M);var k=g.angularAxis.ticksStep||(D[1]-D[0])/(M*(g.minorTicks+1));v&&(k=Math.max(Math.round(k),1)),D[2]||(D[2]=k);var S=r.range.apply(this,D);if(S=S.map(function(t,e){return parseFloat(t.toPrecision(12))}),s=r.scale.linear().domain(D.slice(0,2)).range("clockwise"===g.direction?[0,360]:[360,0]),c.layout.angularAxis.domain=s.domain(),c.layout.angularAxis.endPadding=w?_:0,void 0===(t=r.select(this).select("svg.chart-root"))||t.empty()){var L=(new DOMParser).parseFromString("<svg xmlns='http://www.w3.org/2000/svg' class='chart-root'>' + '<g class='outer-group'>' + '<g class='chart-group'>' + '<circle class='background-circle'></circle>' + '<g class='geometry-group'></g>' + '<g class='radial axis-group'>' + '<circle class='outside-circle'></circle>' + '</g>' + '<g class='angular axis-group'></g>' + '<g class='guides-group'><line></line><circle r='0'></circle></g>' + '</g>' + '<g class='legend-group'></g>' + '<g class='tooltips-group'></g>' + '<g class='title-group'><text></text></g>' + '</g>' + '</svg>","application/xml"),R=this.appendChild(this.ownerDocument.importNode(L.documentElement,!0));t=r.select(R)}t.select(".guides-group").style({"pointer-events":"none"}),t.select(".angular.axis-group").style({"pointer-events":"none"}),t.select(".radial.axis-group").style({"pointer-events":"none"});var N,F=t.select(".chart-group"),T={fill:"none",stroke:g.tickColor},U={"font-size":g.font.size,"font-family":g.font.family,fill:g.font.color,"text-shadow":["-1px 0px","1px -1px","-1px 1px","1px 1px"].map(function(t,e){return" "+t+" 0 "+g.font.outlineColor}).join(",")};if(g.showLegend){N=t.select(".legend-group").attr({transform:"translate("+[m,g.margin.top]+")"}).style({display:"block"});var O=h.map(function(t,e){var n=a.util.cloneJson(t);return n.symbol="DotPlot"===t.geometry?t.dotType||"circle":"LinePlot"!=t.geometry?"square":"line",n.visibleInLegend=void 0===t.visibleInLegend||t.visibleInLegend,n.color="LinePlot"===t.geometry?t.strokeColor:t.color,n});a.Legend().config({data:h.map(function(t,e){return t.name||"Element"+e}),legendConfig:i({},a.Legend.defaultConfig().legendConfig,{container:N,elements:O,reverseOrder:g.legend.reverseOrder})})();var G=N.node().getBBox();m=Math.min(g.width-G.width-g.margin.left-g.margin.right,g.height-g.margin.top-g.margin.bottom)/2,m=Math.max(10,m),E=[g.margin.left+m,g.margin.top+m],n.range([0,m]),c.layout.radialAxis.domain=n.domain(),N.attr("transform","translate("+[E[0]+m,E[1]-m]+")")}else N=t.select(".legend-group").style({display:"none"});t.attr({width:g.width,height:g.height}).style({opacity:g.opacity}),F.attr("transform","translate("+E+")").style({cursor:"crosshair"});var H=[(g.width-(g.margin.left+g.margin.right+2*m+(G?G.width:0)))/2,(g.height-(g.margin.top+g.margin.bottom+2*m))/2];if(H[0]=Math.max(0,H[0]),H[1]=Math.max(0,H[1]),t.select(".outer-group").attr("transform","translate("+H+")"),g.title&&g.title.text){var j=t.select("g.title-group text").style(U).text(g.title.text),Y=j.node().getBBox();j.attr({x:E[0]-Y.width/2,y:E[1]-m-20})}var P=t.select(".radial.axis-group");if(g.radialAxis.gridLinesVisible){var K=P.selectAll("circle.grid-circle").data(n.ticks(5));K.enter().append("circle").attr({class:"grid-circle"}).style(T),K.attr("r",n),K.exit().remove()}P.select("circle.outside-circle").attr({r:m}).style(T);var z=t.select("circle.background-circle").attr({r:m}).style({fill:g.backgroundColor,stroke:g.stroke});function J(t,e){return s(t)%360+g.orientation}if(g.radialAxis.visible){var q=r.svg.axis().scale(n).ticks(5).tickSize(5);P.call(q).attr({transform:"rotate("+g.radialAxis.orientation+")"}),P.selectAll(".domain").style(T),P.selectAll("g>text").text(function(t,e){return this.textContent+g.radialAxis.ticksSuffix}).style(U).style({"text-anchor":"start"}).attr({x:0,y:0,dx:0,dy:0,transform:function(t,e){return"horizontal"===g.radialAxis.tickOrientation?"rotate("+-g.radialAxis.orientation+") translate("+[0,U["font-size"]]+")":"translate("+[0,U["font-size"]]+")"}}),P.selectAll("g>line").style({stroke:"black"})}var V=t.select(".angular.axis-group").selectAll("g.angular-tick").data(S),W=V.enter().append("g").classed("angular-tick",!0);V.attr({transform:function(t,e){return"rotate("+J(t)+")"}}).style({display:g.angularAxis.visible?"block":"none"}),V.exit().remove(),W.append("line").classed("grid-line",!0).classed("major",function(t,e){return e%(g.minorTicks+1)==0}).classed("minor",function(t,e){return!(e%(g.minorTicks+1)==0)}).style(T),W.selectAll(".minor").style({stroke:g.minorTickColor}),V.select("line.grid-line").attr({x1:g.tickLength?m-g.tickLength:0,x2:m}).style({display:g.angularAxis.gridLinesVisible?"block":"none"}),W.append("text").classed("axis-text",!0).style(U);var X=V.select("text.axis-text").attr({x:m+g.labelOffset,dy:o+"em",transform:function(t,e){var n=J(t),r=m+g.labelOffset,i=g.angularAxis.tickOrientation;return"horizontal"==i?"rotate("+-n+" "+r+" 0)":"radial"==i?n<270&&n>90?"rotate(180 "+r+" 0)":null:"rotate("+(n<=180&&n>0?-90:90)+" "+r+" 0)"}}).style({"text-anchor":"middle",display:g.angularAxis.labelsVisible?"block":"none"}).text(function(t,e){return e%(g.minorTicks+1)!=0?"":v?v[t]+g.angularAxis.ticksSuffix:t+g.angularAxis.ticksSuffix}).style(U);g.angularAxis.rewriteTicks&&X.text(function(t,e){return e%(g.minorTicks+1)!=0?"":g.angularAxis.rewriteTicks(this.textContent,e)});var Z=r.max(F.selectAll(".angular-tick text")[0].map(function(t,e){return t.getCTM().e+t.getBBox().width}));N.attr({transform:"translate("+[m+Z,g.margin.top]+")"});var $=t.select("g.geometry-group").selectAll("g").size()>0,tt=t.select("g.geometry-group").selectAll("g.geometry").data(h);if(tt.enter().append("g").attr({class:function(t,e){return"geometry geometry"+e}}),tt.exit().remove(),h[0]||$){var et=[];h.forEach(function(t,e){var r={};r.radialScale=n,r.angularScale=s,r.container=tt.filter(function(t,n){return n==e}),r.geometry=t.geometry,r.orientation=g.orientation,r.direction=g.direction,r.index=e,et.push({data:t,geometryConfig:r})});var nt=r.nest().key(function(t,e){return void 0!==t.data.groupId||"unstacked"}).entries(et),rt=[];nt.forEach(function(t,e){"unstacked"===t.key?rt=rt.concat(t.values.map(function(t,e){return[t]})):rt.push(t.values)}),rt.forEach(function(t,e){var n;n=Array.isArray(t)?t[0].geometryConfig.geometry:t.geometryConfig.geometry;var r=t.map(function(t,e){return i(a[n].defaultConfig(),t)});a[n]().config(r)()})}var it,ot,at=t.select(".guides-group"),st=t.select(".tooltips-group"),At=a.tooltipPanel().config({container:st,fontSize:8})(),lt=a.tooltipPanel().config({container:st,fontSize:8})(),ct=a.tooltipPanel().config({container:st,hasTick:!0})();if(!b){var ut=at.select("line").attr({x1:0,y1:0,y2:0}).style({stroke:"grey","pointer-events":"none"});F.on("mousemove.angular-guide",function(t,e){var n=a.util.getMousePos(z).angle;ut.attr({x2:-m,transform:"rotate("+n+")"}).style({opacity:.5});var r=(n+180+360-g.orientation)%360;it=s.invert(r);var i=a.util.convertToCartesian(m+12,n+180);At.text(a.util.round(it)).move([i[0]+E[0],i[1]+E[1]])}).on("mouseout.angular-guide",function(t,e){at.select("line").style({opacity:0})})}var gt=at.select("circle").style({stroke:"grey",fill:"none"});F.on("mousemove.radial-guide",function(t,e){var r=a.util.getMousePos(z).radius;gt.attr({r:r}).style({opacity:.5}),ot=n.invert(a.util.getMousePos(z).radius);var i=a.util.convertToCartesian(r,g.radialAxis.orientation);lt.text(a.util.round(ot)).move([i[0]+E[0],i[1]+E[1]])}).on("mouseout.radial-guide",function(t,e){gt.style({opacity:0}),ct.hide(),At.hide(),lt.hide()}),t.selectAll(".geometry-group .mark").on("mouseover.tooltip",function(e,n){var i=r.select(this),o=this.style.fill,s="black",A=this.style.opacity||1;if(i.attr({"data-opacity":A}),o&&"none"!==o){i.attr({"data-fill":o}),s=r.hsl(o).darker().toString(),i.style({fill:s,opacity:1});var l={t:a.util.round(e[0]),r:a.util.round(e[1])};b&&(l.t=v[e[0]]);var c="t: "+l.t+", r: "+l.r,u=this.getBoundingClientRect(),g=t.node().getBoundingClientRect(),h=[u.left+u.width/2-H[0]-g.left,u.top+u.height/2-H[1]-g.top];ct.config({color:s}).text(c),ct.move(h)}else o=this.style.stroke||"black",i.attr({"data-stroke":o}),s=r.hsl(o).darker().toString(),i.style({stroke:s,opacity:1})}).on("mousemove.tooltip",function(t,e){if(0!=r.event.which)return!1;r.select(this).attr("data-fill")&&ct.show()}).on("mouseout.tooltip",function(t,e){ct.hide();var n=r.select(this),i=n.attr("data-fill");i?n.style({fill:i,opacity:n.attr("data-opacity")}):n.style({stroke:n.attr("data-stroke"),opacity:n.attr("data-opacity")})})})}(l),this},config:function(t){if(!arguments.length)return A;var e=a.util.cloneJson(t);return e.data.forEach(function(t,e){A.data[e]||(A.data[e]={}),i(A.data[e],a.Axis.defaultConfig().data[0]),i(A.data[e],t)}),i(A.layout,a.Axis.defaultConfig().layout),i(A.layout,e.layout),this},getLiveConfig:function(){return c},getinputConfig:function(){return l},radialScale:function(t){return n},angularScale:function(t){return s},svg:function(){return t}};return r.rebind(g,u,"on"),g},a.Axis.defaultConfig=function(t,e){return{data:[{t:[1,2,3,4],r:[10,11,12,13],name:"Line1",geometry:"LinePlot",color:null,strokeDash:"solid",strokeColor:null,strokeSize:"1",visibleInLegend:!0,opacity:1}],layout:{defaultColorRange:r.scale.category10().range(),title:null,height:450,width:500,margin:{top:40,right:40,bottom:40,left:40},font:{size:12,color:"gray",outlineColor:"white",family:"Tahoma, sans-serif"},direction:"clockwise",orientation:0,labelOffset:10,radialAxis:{domain:null,orientation:-45,ticksSuffix:"",visible:!0,gridLinesVisible:!0,tickOrientation:"horizontal",rewriteTicks:null},angularAxis:{domain:[0,360],ticksSuffix:"",visible:!0,gridLinesVisible:!0,labelsVisible:!0,tickOrientation:"horizontal",rewriteTicks:null,ticksCount:null,ticksStep:null},minorTicks:0,tickLength:null,tickColor:"silver",minorTickColor:"#eee",backgroundColor:"none",needsEndSpacing:null,showLegend:!0,legend:{reverseOrder:!1},opacity:1}}},a.util={},a.DATAEXTENT="dataExtent",a.AREA="AreaChart",a.LINE="LinePlot",a.DOT="DotPlot",a.BAR="BarChart",a.util._override=function(t,e){for(var n in t)n in e&&(e[n]=t[n])},a.util._extend=function(t,e){for(var n in t)e[n]=t[n]},a.util._rndSnd=function(){return 2*Math.random()-1+(2*Math.random()-1)+(2*Math.random()-1)},a.util.dataFromEquation2=function(t,e){var n=e||6;return r.range(0,360+n,n).map(function(e,n){var r=e*Math.PI/180;return[e,t(r)]})},a.util.dataFromEquation=function(t,e,n){var i=e||6,o=[],a=[];r.range(0,360+i,i).forEach(function(e,n){var r=e*Math.PI/180,i=t(r);o.push(e),a.push(i)});var s={t:o,r:a};return n&&(s.name=n),s},a.util.ensureArray=function(t,e){if(void 0===t)return null;var n=[].concat(t);return r.range(e).map(function(t,e){return n[e]||n[0]})},a.util.fillArrays=function(t,e,n){return e.forEach(function(e,r){t[e]=a.util.ensureArray(t[e],n)}),t},a.util.cloneJson=function(t){return JSON.parse(JSON.stringify(t))},a.util.validateKeys=function(t,e){"string"==typeof e&&(e=e.split("."));var n=e.shift();return t[n]&&(!e.length||objHasKeys(t[n],e))},a.util.sumArrays=function(t,e){return r.zip(t,e).map(function(t,e){return r.sum(t)})},a.util.arrayLast=function(t){return t[t.length-1]},a.util.arrayEqual=function(t,e){for(var n=Math.max(t.length,e.length,1);n-- >=0&&t[n]===e[n];);return-2===n},a.util.flattenArray=function(t){for(var e=[];!a.util.arrayEqual(e,t);)e=t,t=[].concat.apply([],t);return t},a.util.deduplicate=function(t){return t.filter(function(t,e,n){return n.indexOf(t)==e})},a.util.convertToCartesian=function(t,e){var n=e*Math.PI/180;return[t*Math.cos(n),t*Math.sin(n)]},a.util.round=function(t,e){var n=e||2,r=Math.pow(10,n);return Math.round(t*r)/r},a.util.getMousePos=function(t){var e=r.mouse(t.node()),n=e[0],i=e[1],o={};return o.x=n,o.y=i,o.pos=e,o.angle=180*(Math.atan2(i,n)+Math.PI)/Math.PI,o.radius=Math.sqrt(n*n+i*i),o},a.util.duplicatesCount=function(t){for(var e,n={},r={},i=0,o=t.length;i<o;i++)(e=t[i])in n?(n[e]++,r[e]=n[e]):n[e]=1;return r},a.util.duplicates=function(t){return Object.keys(a.util.duplicatesCount(t))},a.util.translator=function(t,e,n,r){if(r){var i=n.slice();n=e,e=i}var o=e.reduce(function(t,e){if(void 0!==t)return t[e]},t);void 0!==o&&(e.reduce(function(t,n,r){if(void 0!==t)return r===e.length-1&&delete t[n],t[n]},t),n.reduce(function(t,e,r){return void 0===t[e]&&(t[e]={}),r===n.length-1&&(t[e]=o),t[e]},t))},a.PolyChart=function(){var t=[a.PolyChart.defaultConfig()],e=r.dispatch("hover"),n={solid:"none",dash:[5,2],dot:[2,5]};function o(){var e=t[0].geometryConfig,i=e.container;"string"==typeof i&&(i=r.select(i)),i.datum(t).each(function(t,i){var o=!!t[0].data.yStack,a=t.map(function(t,e){return o?r.zip(t.data.t[0],t.data.r[0],t.data.yStack[0]):r.zip(t.data.t[0],t.data.r[0])}),s=e.angularScale,A=e.radialScale.domain()[0],l={bar:function(n,i,o){var a=t[o].data,A=e.radialScale(n[1])-e.radialScale(0),l=e.radialScale(n[2]||0),c=a.barWidth;r.select(this).attr({class:"mark bar",d:"M"+[[A+l,-c/2],[A+l,c/2],[l,c/2],[l,-c/2]].join("L")+"Z",transform:function(t,n){return"rotate("+(e.orientation+s(t[0]))+")"}})},dot:function(n,i,o){var a=n[2]?[n[0],n[1]+n[2]]:n,s=r.svg.symbol().size(t[o].data.dotSize).type(t[o].data.dotType)(n,i);r.select(this).attr({class:"mark dot",d:s,transform:function(t,n){var r,i,o,s=(r=function(t,n){var r=e.radialScale(t[1]),i=(e.angularScale(t[0])+e.orientation)*Math.PI/180;return{r:r,t:i}}(a),i=r.r*Math.cos(r.t),o=r.r*Math.sin(r.t),{x:i,y:o});return"translate("+[s.x,s.y]+")"}})}},c=r.svg.line.radial().interpolate(t[0].data.lineInterpolation).radius(function(t){return e.radialScale(t[1])}).angle(function(t){return e.angularScale(t[0])*Math.PI/180});l.line=function(n,i,o){var s=n[2]?a[o].map(function(t,e){return[t[0],t[1]+t[2]]}):a[o];if(r.select(this).each(l.dot).style({opacity:function(e,n){return+t[o].data.dotVisible},fill:d.stroke(n,i,o)}).attr({class:"mark dot"}),!(i>0)){var A=r.select(this.parentNode).selectAll("path.line").data([0]);A.enter().insert("path"),A.attr({class:"line",d:c(s),transform:function(t,n){return"rotate("+(e.orientation+90)+")"},"pointer-events":"none"}).style({fill:function(t,e){return d.fill(n,i,o)},"fill-opacity":0,stroke:function(t,e){return d.stroke(n,i,o)},"stroke-width":function(t,e){return d["stroke-width"](n,i,o)},"stroke-dasharray":function(t,e){return d["stroke-dasharray"](n,i,o)},opacity:function(t,e){return d.opacity(n,i,o)},display:function(t,e){return d.display(n,i,o)}})}};var u=e.angularScale.range(),g=Math.abs(u[1]-u[0])/a[0].length*Math.PI/180,h=r.svg.arc().startAngle(function(t){return-g/2}).endAngle(function(t){return g/2}).innerRadius(function(t){return e.radialScale(A+(t[2]||0))}).outerRadius(function(t){return e.radialScale(A+(t[2]||0))+e.radialScale(t[1])});l.arc=function(t,n,i){r.select(this).attr({class:"mark arc",d:h,transform:function(t,n){return"rotate("+(e.orientation+s(t[0])+90)+")"}})};var d={fill:function(e,n,r){return t[r].data.color},stroke:function(e,n,r){return t[r].data.strokeColor},"stroke-width":function(e,n,r){return t[r].data.strokeSize+"px"},"stroke-dasharray":function(e,r,i){return n[t[i].data.strokeDash]},opacity:function(e,n,r){return t[r].data.opacity},display:function(e,n,r){return void 0===t[r].data.visible||t[r].data.visible?"block":"none"}},f=r.select(this).selectAll("g.layer").data(a);f.enter().append("g").attr({class:"layer"});var p=f.selectAll("path.mark").data(function(t,e){return t});p.enter().append("path").attr({class:"mark"}),p.style(d).each(l[e.geometryType]),p.exit().remove(),f.exit().remove()})}return o.config=function(e){return arguments.length?(e.forEach(function(e,n){t[n]||(t[n]={}),i(t[n],a.PolyChart.defaultConfig()),i(t[n],e)}),this):t},o.getColorScale=function(){},r.rebind(o,e,"on"),o},a.PolyChart.defaultConfig=function(){return{data:{name:"geom1",t:[[1,2,3,4]],r:[[1,2,3,4]],dotType:"circle",dotSize:64,dotVisible:!1,barWidth:20,color:"#ffa500",strokeSize:1,strokeColor:"silver",strokeDash:"solid",opacity:1,index:0,visible:!0,visibleInLegend:!0},geometryConfig:{geometry:"LinePlot",geometryType:"arc",direction:"clockwise",orientation:0,container:"body",radialScale:null,angularScale:null,colorScale:r.scale.category20()}}},a.BarChart=function(){return a.PolyChart()},a.BarChart.defaultConfig=function(){return{geometryConfig:{geometryType:"bar"}}},a.AreaChart=function(){return a.PolyChart()},a.AreaChart.defaultConfig=function(){return{geometryConfig:{geometryType:"arc"}}},a.DotPlot=function(){return a.PolyChart()},a.DotPlot.defaultConfig=function(){return{geometryConfig:{geometryType:"dot",dotType:"circle"}}},a.LinePlot=function(){return a.PolyChart()},a.LinePlot.defaultConfig=function(){return{geometryConfig:{geometryType:"line"}}},a.Legend=function(){var t=a.Legend.defaultConfig(),e=r.dispatch("hover");function n(){var e=t.legendConfig,o=t.data.map(function(t,n){return[].concat(t).map(function(t,r){var o=i({},e.elements[n]);return o.name=t,o.color=[].concat(e.elements[n].color)[r],o})}),a=r.merge(o);a=a.filter(function(t,n){return e.elements[n]&&(e.elements[n].visibleInLegend||void 0===e.elements[n].visibleInLegend)}),e.reverseOrder&&(a=a.reverse());var s=e.container;("string"==typeof s||s.nodeName)&&(s=r.select(s));var A=a.map(function(t,e){return t.color}),l=e.fontSize,c=null==e.isContinuous?"number"==typeof a[0]:e.isContinuous,u=c?e.height:l*a.length,g=s.classed("legend-group",!0).selectAll("svg").data([0]),h=g.enter().append("svg").attr({width:300,height:u+l,xmlns:"http://www.w3.org/2000/svg","xmlns:xlink":"http://www.w3.org/1999/xlink",version:"1.1"});h.append("g").classed("legend-axis",!0),h.append("g").classed("legend-marks",!0);var d=r.range(a.length),f=r.scale[c?"linear":"ordinal"]().domain(d).range(A),p=r.scale[c?"linear":"ordinal"]().domain(d)[c?"range":"rangePoints"]([0,u]);if(c){var I=g.select(".legend-marks").append("defs").append("linearGradient").attr({id:"grad1",x1:"0%",y1:"0%",x2:"0%",y2:"100%"}).selectAll("stop").data(A);I.enter().append("stop"),I.attr({offset:function(t,e){return e/(A.length-1)*100+"%"}}).style({"stop-color":function(t,e){return t}}),g.append("rect").classed("legend-mark",!0).attr({height:e.height,width:e.colorBandWidth,fill:"url(#grad1)"})}else{var C=g.select(".legend-marks").selectAll("path.legend-mark").data(a);C.enter().append("path").classed("legend-mark",!0),C.attr({transform:function(t,e){return"translate("+[l/2,p(e)+l/2]+")"},d:function(t,e){var n,i,o,a=t.symbol;return o=3*(i=l),"line"===(n=a)?"M"+[[-i/2,-i/12],[i/2,-i/12],[i/2,i/12],[-i/2,i/12]]+"Z":-1!=r.svg.symbolTypes.indexOf(n)?r.svg.symbol().type(n).size(o)():r.svg.symbol().type("square").size(o)()},fill:function(t,e){return f(e)}}),C.exit().remove()}var m=r.svg.axis().scale(p).orient("right"),y=g.select("g.legend-axis").attr({transform:"translate("+[c?e.colorBandWidth:l,l/2]+")"}).call(m);return y.selectAll(".domain").style({fill:"none",stroke:"none"}),y.selectAll("line").style({fill:"none",stroke:c?e.textColor:"none"}),y.selectAll("text").style({fill:e.textColor,"font-size":e.fontSize}).text(function(t,e){return a[e].name}),n}return n.config=function(e){return arguments.length?(i(t,e),this):t},r.rebind(n,e,"on"),n},a.Legend.defaultConfig=function(t,e){return{data:["a","b","c"],legendConfig:{elements:[{symbol:"line",color:"red"},{symbol:"square",color:"yellow"},{symbol:"diamond",color:"limegreen"}],height:150,colorBandWidth:30,fontSize:12,container:"body",isContinuous:null,textColor:"grey",reverseOrder:!1}}},a.tooltipPanel=function(){var t,e,n,o={container:null,hasTick:!1,fontSize:12,color:"white",padding:5},s="tooltip-"+a.tooltipPanel.uid++,A=10,l=function(){var r=(t=o.container.selectAll("g."+s).data([0])).enter().append("g").classed(s,!0).style({"pointer-events":"none",display:"none"});return n=r.append("path").style({fill:"white","fill-opacity":.9}).attr({d:"M0 0"}),e=r.append("text").attr({dx:o.padding+A,dy:.3*+o.fontSize}),l};return l.text=function(i){var a=r.hsl(o.color).l,s=a>=.5?"#aaa":"white",c=a>=.5?"black":"white",u=i||"";e.style({fill:c,"font-size":o.fontSize+"px"}).text(u);var g=o.padding,h=e.node().getBBox(),d={fill:o.color,stroke:s,"stroke-width":"2px"},f=h.width+2*g+A,p=h.height+2*g;return n.attr({d:"M"+[[A,-p/2],[A,-p/4],[o.hasTick?0:A,0],[A,p/4],[A,p/2],[f,p/2],[f,-p/2]].join("L")+"Z"}).style(d),t.attr({transform:"translate("+[A,-p/2+2*g]+")"}),t.style({display:"block"}),l},l.move=function(e){if(t)return t.attr({transform:"translate("+[e[0],e[1]]+")"}).style({display:"block"}),l},l.hide=function(){if(t)return t.style({display:"none"}),l},l.show=function(){if(t)return t.style({display:"block"}),l},l.config=function(t){return i(o,t),l},l},a.tooltipPanel.uid=1,a.adapter={},a.adapter.plotly=function(){var t={convert:function(t,e){var n={};if(t.data&&(n.data=t.data.map(function(t,n){var r=i({},t);return[[r,["marker","color"],["color"]],[r,["marker","opacity"],["opacity"]],[r,["marker","line","color"],["strokeColor"]],[r,["marker","line","dash"],["strokeDash"]],[r,["marker","line","width"],["strokeSize"]],[r,["marker","symbol"],["dotType"]],[r,["marker","size"],["dotSize"]],[r,["marker","barWidth"],["barWidth"]],[r,["line","interpolation"],["lineInterpolation"]],[r,["showlegend"],["visibleInLegend"]]].forEach(function(t,n){a.util.translator.apply(null,t.concat(e))}),e||delete r.marker,e&&delete r.groupId,e?("LinePlot"===r.geometry?(r.type="scatter",!0===r.dotVisible?(delete r.dotVisible,r.mode="lines+markers"):r.mode="lines"):"DotPlot"===r.geometry?(r.type="scatter",r.mode="markers"):"AreaChart"===r.geometry?r.type="area":"BarChart"===r.geometry&&(r.type="bar"),delete r.geometry):("scatter"===r.type?"lines"===r.mode?r.geometry="LinePlot":"markers"===r.mode?r.geometry="DotPlot":"lines+markers"===r.mode&&(r.geometry="LinePlot",r.dotVisible=!0):"area"===r.type?r.geometry="AreaChart":"bar"===r.type&&(r.geometry="BarChart"),delete r.mode,delete r.type),r}),!e&&t.layout&&"stack"===t.layout.barmode)){var o=a.util.duplicates(n.data.map(function(t,e){return t.geometry}));n.data.forEach(function(t,e){var r=o.indexOf(t.geometry);-1!=r&&(n.data[e].groupId=r)})}if(t.layout){var s=i({},t.layout);if([[s,["plot_bgcolor"],["backgroundColor"]],[s,["showlegend"],["showLegend"]],[s,["radialaxis"],["radialAxis"]],[s,["angularaxis"],["angularAxis"]],[s.angularaxis,["showline"],["gridLinesVisible"]],[s.angularaxis,["showticklabels"],["labelsVisible"]],[s.angularaxis,["nticks"],["ticksCount"]],[s.angularaxis,["tickorientation"],["tickOrientation"]],[s.angularaxis,["ticksuffix"],["ticksSuffix"]],[s.angularaxis,["range"],["domain"]],[s.angularaxis,["endpadding"],["endPadding"]],[s.radialaxis,["showline"],["gridLinesVisible"]],[s.radialaxis,["tickorientation"],["tickOrientation"]],[s.radialaxis,["ticksuffix"],["ticksSuffix"]],[s.radialaxis,["range"],["domain"]],[s.angularAxis,["showline"],["gridLinesVisible"]],[s.angularAxis,["showticklabels"],["labelsVisible"]],[s.angularAxis,["nticks"],["ticksCount"]],[s.angularAxis,["tickorientation"],["tickOrientation"]],[s.angularAxis,["ticksuffix"],["ticksSuffix"]],[s.angularAxis,["range"],["domain"]],[s.angularAxis,["endpadding"],["endPadding"]],[s.radialAxis,["showline"],["gridLinesVisible"]],[s.radialAxis,["tickorientation"],["tickOrientation"]],[s.radialAxis,["ticksuffix"],["ticksSuffix"]],[s.radialAxis,["range"],["domain"]],[s.font,["outlinecolor"],["outlineColor"]],[s.legend,["traceorder"],["reverseOrder"]],[s,["labeloffset"],["labelOffset"]],[s,["defaultcolorrange"],["defaultColorRange"]]].forEach(function(t,n){a.util.translator.apply(null,t.concat(e))}),e?(void 0!==s.tickLength&&(s.angularaxis.ticklen=s.tickLength,delete s.tickLength),s.tickColor&&(s.angularaxis.tickcolor=s.tickColor,delete s.tickColor)):(s.angularAxis&&void 0!==s.angularAxis.ticklen&&(s.tickLength=s.angularAxis.ticklen),s.angularAxis&&void 0!==s.angularAxis.tickcolor&&(s.tickColor=s.angularAxis.tickcolor)),s.legend&&"boolean"!=typeof s.legend.reverseOrder&&(s.legend.reverseOrder="normal"!=s.legend.reverseOrder),s.legend&&"boolean"==typeof s.legend.traceorder&&(s.legend.traceorder=s.legend.traceorder?"reversed":"normal",delete s.legend.reverseOrder),s.margin&&void 0!==s.margin.t){var A=["t","r","b","l","pad"],l=["top","right","bottom","left","pad"],c={};r.entries(s.margin).forEach(function(t,e){c[l[A.indexOf(t.key)]]=t.value}),s.margin=c}e&&(delete s.needsEndSpacing,delete s.minorTickColor,delete s.minorTicks,delete s.angularaxis.ticksCount,delete s.angularaxis.ticksCount,delete s.angularaxis.ticksStep,delete s.angularaxis.rewriteTicks,delete s.angularaxis.nticks,delete s.radialaxis.ticksCount,delete s.radialaxis.ticksCount,delete s.radialaxis.ticksStep,delete s.radialaxis.rewriteTicks,delete s.radialaxis.nticks),n.layout=s}return n}};return t}},{"../../../constants/alignment":688,"../../../lib":719,d3:163}],838:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../../lib"),o=t("../../../components/color"),a=t("./micropolar"),s=t("./undo_manager"),A=i.extendDeepAll,l=e.exports={};l.framework=function(t){var e,n,i,o,c,u=new s;function g(n,s){return s&&(c=s),r.select(r.select(c).node().parentNode).selectAll(".svg-container>*:not(.chart-root)").remove(),e=e?A(e,n):n,i||(i=a.Axis()),o=a.adapter.plotly().convert(e),i.config(o).render(c),t.data=e.data,t.layout=e.layout,l.fillLayout(t),e}return g.isPolar=!0,g.svg=function(){return i.svg()},g.getConfig=function(){return e},g.getLiveConfig=function(){return a.adapter.plotly().convert(i.getLiveConfig(),!0)},g.getLiveScales=function(){return{t:i.angularScale(),r:i.radialScale()}},g.setUndoPoint=function(){var t,r,i=this,o=a.util.cloneJson(e);t=o,r=n,u.add({undo:function(){r&&i(r)},redo:function(){i(t)}}),n=a.util.cloneJson(o)},g.undo=function(){u.undo()},g.redo=function(){u.redo()},g},l.fillLayout=function(t){var e=r.select(t).selectAll(".plot-container"),n=e.selectAll(".svg-container"),i=t.framework&&t.framework.svg&&t.framework.svg(),a={width:800,height:600,paper_bgcolor:o.background,_container:e,_paperdiv:n,_paper:i};t._fullLayout=A(a,t.layout)}},{"../../../components/color":593,"../../../lib":719,"./micropolar":837,"./undo_manager":839,d3:163}],839:[function(t,e,n){"use strict";e.exports=function(){var t,e=[],n=-1,r=!1;function i(t,e){return t?(r=!0,t[e](),r=!1,this):this}return{add:function(t){return r?this:(e.splice(n+1,e.length-n),e.push(t),n=e.length-1,this)},setCallback:function(e){t=e},undo:function(){var r=e[n];return r?(i(r,"undo"),n-=1,t&&t(r.undo),this):this},redo:function(){var r=e[n+1];return r?(i(r,"redo"),n+=1,t&&t(r.redo),this):this},clear:function(){e=[],n=-1},hasUndo:function(){return-1!==n},hasRedo:function(){return n<e.length-1},getCommands:function(){return e},getPreviousCommand:function(){return e[n-1]},getIndex:function(){return n}}}},{}],840:[function(t,e,n){"use strict";var r=t("d3"),i=t("tinycolor2"),o=t("../../registry"),a=t("../../lib"),s=t("../../components/color"),A=t("../../components/drawing"),l=t("../plots"),c=t("../../plots/cartesian/axes"),u=t("../cartesian/set_convert"),g=t("./set_convert"),h=t("../cartesian/autorange").doAutoRange,d=t("../cartesian/dragbox"),f=t("../../components/dragelement"),p=t("../../components/fx"),I=t("../../components/titles"),C=t("../cartesian/select").prepSelect,m=t("../cartesian/select").selectOnClick,y=t("../cartesian/select").clearSelect,E=t("../../lib/setcursor"),v=t("../../lib/clear_gl_canvases"),B=t("../../plot_api/subroutines").redrawReglTraces,b=t("../../constants/alignment").MID_SHIFT,Q=t("./constants"),w=t("./helpers"),x=a._,_=a.mod,D=a.deg2rad,M=a.rad2deg;function k(t,e){this.id=e,this.gd=t,this._hasClipOnAxisFalse=null,this.vangles=null,this.radialAxisAngle=null,this.traceHash={},this.layers={},this.clipPaths={},this.clipIds={},this.viewInitial={};var n=t._fullLayout,r="clip"+n._uid+e;this.clipIds.forTraces=r+"-for-traces",this.clipPaths.forTraces=n._clips.append("clipPath").attr("id",this.clipIds.forTraces),this.clipPaths.forTraces.append("path"),this.framework=n._polarlayer.append("g").attr("class",e),this.radialTickLayout=null,this.angularTickLayout=null}var S=k.prototype;function L(t){var e=t.ticks+String(t.ticklen)+String(t.showticklabels);return"side"in t&&(e+=t.side),e}function R(t,e){return e[a.findIndexOfMin(e,function(e){return a.angleDist(t,e)})]}function N(t,e,n){return e?(t.attr("display",null),t.attr(n)):t&&t.attr("display","none"),t}function F(t,e){return"translate("+t+","+e+")"}function T(t){return"rotate("+t+")"}e.exports=function(t,e){return new k(t,e)},S.plot=function(t,e){var n=e[this.id];this._hasClipOnAxisFalse=!1;for(var r=0;r<t.length;r++)if(!1===t[r][0].trace.cliponaxis){this._hasClipOnAxisFalse=!0;break}this.updateLayers(e,n),this.updateLayout(e,n),l.generalUpdatePerTraceModule(this.gd,this,t,n),this.updateFx(e,n)},S.updateLayers=function(t,e){var n=this.layers,i=e.radialaxis,o=e.angularaxis,a=Q.layerNames,s=a.indexOf("frontplot"),A=a.slice(0,s),l="below traces"===o.layer,c="below traces"===i.layer;l&&A.push("angular-line"),c&&A.push("radial-line"),l&&A.push("angular-axis"),c&&A.push("radial-axis"),A.push("frontplot"),l||A.push("angular-line"),c||A.push("radial-line"),l||A.push("angular-axis"),c||A.push("radial-axis");var u=this.framework.selectAll(".polarsublayer").data(A,String);u.enter().append("g").attr("class",function(t){return"polarsublayer "+t}).each(function(t){var e=n[t]=r.select(this);switch(t){case"frontplot":e.append("g").classed("barlayer",!0),e.append("g").classed("scatterlayer",!0);break;case"backplot":e.append("g").classed("maplayer",!0);break;case"plotbg":n.bg=e.append("path");break;case"radial-grid":case"angular-grid":e.style("fill","none");break;case"radial-line":e.append("line").style("fill","none");break;case"angular-line":e.append("path").style("fill","none")}}),u.order()},S.updateLayout=function(t,e){var n=this.layers,r=t._size,i=e.radialaxis,o=e.angularaxis,a=e.domain.x,l=e.domain.y;this.xOffset=r.l+r.w*a[0],this.yOffset=r.t+r.h*(1-l[1]);var c=this.xLength=r.w*(a[1]-a[0]),u=this.yLength=r.h*(l[1]-l[0]),g=e.sector;this.sectorInRad=g.map(D);var h,d,f,p,I,C=this.sectorBBox=function(t){var e,n,r,i,o=t[0],a=t[1]-o,s=_(o,360),A=s+a,l=Math.cos(D(s)),c=Math.sin(D(s)),u=Math.cos(D(A)),g=Math.sin(D(A));return i=s<=90&&A>=90||s>90&&A>=450?1:c<=0&&g<=0?0:Math.max(c,g),e=s<=180&&A>=180||s>180&&A>=540?-1:l>=0&&u>=0?0:Math.min(l,u),n=s<=270&&A>=270||s>270&&A>=630?-1:c>=0&&g>=0?0:Math.min(c,g),r=A>=360?1:l<=0&&u<=0?0:Math.max(l,u),[e,n,r,i]}(g),m=C[2]-C[0],y=C[3]-C[1],E=u/c,v=Math.abs(y/m);E>v?(h=c,I=(u-(d=c*v))/r.h/2,f=[a[0],a[1]],p=[l[0]+I,l[1]-I]):(d=u,I=(c-(h=u/v))/r.w/2,f=[a[0]+I,a[1]-I],p=[l[0],l[1]]),this.xLength2=h,this.yLength2=d,this.xDomain2=f,this.yDomain2=p;var B=this.xOffset2=r.l+r.w*f[0],b=this.yOffset2=r.t+r.h*(1-p[1]),Q=this.radius=h/m,w=this.innerRadius=e.hole*Q,x=this.cx=B-Q*C[0],M=this.cy=b+Q*C[3],k=this.cxx=x-B,S=this.cyy=M-b;this.radialAxis=this.mockAxis(t,e,i,{_id:"x",side:{counterclockwise:"top",clockwise:"bottom"}[i.side],domain:[w/r.w,Q/r.w]}),this.angularAxis=this.mockAxis(t,e,o,{side:"right",domain:[0,Math.PI],autorange:!1}),this.doAutoRange(t,e),this.updateAngularAxis(t,e),this.updateRadialAxis(t,e),this.updateRadialAxisTitle(t,e),this.xaxis=this.mockCartesianAxis(t,e,{_id:"x",domain:f}),this.yaxis=this.mockCartesianAxis(t,e,{_id:"y",domain:p});var L=this.pathSubplot();this.clipPaths.forTraces.select("path").attr("d",L).attr("transform",F(k,S)),n.frontplot.attr("transform",F(B,b)).call(A.setClipUrl,this._hasClipOnAxisFalse?null:this.clipIds.forTraces,this.gd),n.bg.attr("d",L).attr("transform",F(x,M)).call(s.fill,e.bgcolor)},S.mockAxis=function(t,e,n,r){var i=a.extendFlat({anchor:"free",position:0},n,r);return g(i,e,t),i},S.mockCartesianAxis=function(t,e,n){var r=this,i=n._id,o=a.extendFlat({type:"linear"},n);u(o,t);var s={x:[0,2],y:[1,3]};return o.setRange=function(){var t=r.sectorBBox,n=s[i],a=r.radialAxis._rl,A=(a[1]-a[0])/(1-e.hole);o.range=[t[n[0]]*A,t[n[1]]*A]},o.isPtWithinRange="x"===i?function(t){return r.isPtInside(t)}:function(){return!0},o.setRange(),o.setScale(),o},S.doAutoRange=function(t,e){var n=this.gd,r=this.radialAxis,i=e.radialaxis;r.setScale(),h(n,r);var o=r.range;i.range=o.slice(),i._input.range=o.slice(),r._rl=[r.r2l(o[0],null,"gregorian"),r.r2l(o[1],null,"gregorian")]},S.updateRadialAxis=function(t,e){var n=this,r=n.gd,i=n.layers,o=n.radius,A=n.innerRadius,l=n.cx,u=n.cy,g=e.radialaxis,h=_(e.sector[0],360),d=n.radialAxis,f=A<o;n.fillViewInitialKey("radialaxis.angle",g.angle),n.fillViewInitialKey("radialaxis.range",d.range.slice()),d.setGeometry(),"auto"===d.tickangle&&h>90&&h<=270&&(d.tickangle=180);var p=function(t){return"translate("+(d.l2p(t.x)+A)+",0)"},I=L(g);if(n.radialTickLayout!==I&&(i["radial-axis"].selectAll(".xtick").remove(),n.radialTickLayout=I),f){d.setScale();var C=c.calcTicks(d),m=c.clipEnds(d,C),y=c.getTickSigns(d)[2];c.drawTicks(r,d,{vals:C,layer:i["radial-axis"],path:c.makeTickPath(d,0,y),transFn:p,crisp:!1}),c.drawGrid(r,d,{vals:m,layer:i["radial-grid"],path:function(t){return n.pathArc(d.r2p(t.x)+A)},transFn:a.noop,crisp:!1}),c.drawLabels(r,d,{vals:C,layer:i["radial-axis"],transFn:p,labelFns:c.makeLabelFns(d,0)})}var E=n.radialAxisAngle=n.vangles?M(R(D(g.angle),n.vangles)):g.angle,v=F(l,u),B=v+T(-E);N(i["radial-axis"],f&&(g.showticklabels||g.ticks),{transform:B}),N(i["radial-grid"],f&&g.showgrid,{transform:v}),N(i["radial-line"].select("line"),f&&g.showline,{x1:A,y1:0,x2:o,y2:0,transform:B}).attr("stroke-width",g.linewidth).call(s.stroke,g.linecolor)},S.updateRadialAxisTitle=function(t,e,n){var r=this.gd,i=this.radius,o=this.cx,a=this.cy,s=e.radialaxis,l=this.id+"title",c=void 0!==n?n:this.radialAxisAngle,u=D(c),g=Math.cos(u),h=Math.sin(u),d=0;if(s.title){var f=A.bBox(this.layers["radial-axis"].node()).height,p=s.title.font.size;d="counterclockwise"===s.side?-f-.4*p:f+.8*p}this.layers["radial-axis-title"]=I.draw(r,l,{propContainer:s,propName:this.id+".radialaxis.title",placeholder:x(r,"Click to enter radial axis title"),attributes:{x:o+i/2*g+d*h,y:a-i/2*h+d*g,"text-anchor":"middle"},transform:{rotate:-c}})},S.updateAngularAxis=function(t,e){var n=this,r=n.gd,i=n.layers,o=n.radius,A=n.innerRadius,l=n.cx,u=n.cy,g=e.angularaxis,h=n.angularAxis;n.fillViewInitialKey("angularaxis.rotation",g.rotation),h.setGeometry(),h.setScale();var d=function(t){return h.t2g(t.x)};"linear"===h.type&&"radians"===h.thetaunit&&(h.tick0=M(h.tick0),h.dtick=M(h.dtick));var f=function(t){return F(l+o*Math.cos(t),u-o*Math.sin(t))},p=c.makeLabelFns(h,0).labelStandoff,I={xFn:function(t){var e=d(t);return Math.cos(e)*p},yFn:function(t){var e=d(t),n=Math.sin(e)>0?.2:1;return-Math.sin(e)*(p+t.fontSize*n)+Math.abs(Math.cos(e))*(t.fontSize*b)},anchorFn:function(t){var e=d(t),n=Math.cos(e);return Math.abs(n)<.1?"middle":n>0?"start":"end"},heightFn:function(t,e,n){var r=d(t);return-.5*(1+Math.sin(r))*n}},C=L(g);n.angularTickLayout!==C&&(i["angular-axis"].selectAll("."+h._id+"tick").remove(),n.angularTickLayout=C);var m,y=c.calcTicks(h);if("linear"===e.gridshape?(m=y.map(d),a.angleDelta(m[0],m[1])<0&&(m=m.slice().reverse())):m=null,n.vangles=m,"category"===h.type&&(y=y.filter(function(t){return a.isAngleInsideSector(d(t),n.sectorInRad)})),h.visible){var E="inside"===h.ticks?-1:1,v=(h.linewidth||1)/2;c.drawTicks(r,h,{vals:y,layer:i["angular-axis"],path:"M"+E*v+",0h"+E*h.ticklen,transFn:function(t){var e=d(t);return f(e)+T(-M(e))},crisp:!1}),c.drawGrid(r,h,{vals:y,layer:i["angular-grid"],path:function(t){var e=d(t),n=Math.cos(e),r=Math.sin(e);return"M"+[l+A*n,u-A*r]+"L"+[l+o*n,u-o*r]},transFn:a.noop,crisp:!1}),c.drawLabels(r,h,{vals:y,layer:i["angular-axis"],repositionOnUpdate:!0,transFn:function(t){return f(d(t))},labelFns:I})}N(i["angular-line"].select("path"),g.showline,{d:n.pathSubplot(),transform:F(l,u)}).attr("stroke-width",g.linewidth).call(s.stroke,g.linecolor)},S.updateFx=function(t,e){this.gd._context.staticPlot||(this.updateAngularDrag(t),this.updateRadialDrag(t,e,0),this.updateRadialDrag(t,e,1),this.updateMainDrag(t))},S.updateMainDrag=function(t){var e=this,n=e.gd,a=e.layers,s=t._zoomlayer,A=Q.MINZOOM,l=Q.OFFEDGE,c=e.radius,u=e.innerRadius,g=e.cx,h=e.cy,I=e.cxx,E=e.cyy,v=e.sectorInRad,B=e.vangles,b=e.radialAxis,x=w.clampTiny,_=w.findXYatLength,D=w.findEnclosingVertexAngles,M=Q.cornerHalfWidth,k=Q.cornerLen/2,S=d.makeDragger(a,"path","maindrag","crosshair");r.select(S).attr("d",e.pathSubplot()).attr("transform",F(g,h));var L,R,N,T,U,O,G,H,j,Y={element:S,gd:n,subplot:e.id,plotinfo:{id:e.id,xaxis:e.xaxis,yaxis:e.yaxis},xaxes:[e.xaxis],yaxes:[e.yaxis]};function P(t,e){return Math.sqrt(t*t+e*e)}function K(t,e){return P(t-I,e-E)}function z(t,e){return Math.atan2(E-e,t-I)}function J(t,e){return[t*Math.cos(e),t*Math.sin(-e)]}function q(t,n){if(0===t)return e.pathSector(2*M);var r=k/t,i=n-r,o=n+r,a=Math.max(0,Math.min(t,c)),s=a-M,A=a+M;return"M"+J(s,i)+"A"+[s,s]+" 0,0,0 "+J(s,o)+"L"+J(A,o)+"A"+[A,A]+" 0,0,1 "+J(A,i)+"Z"}function V(t,n,r){if(0===t)return e.pathSector(2*M);var i,o,a=J(t,n),s=J(t,r),A=x((a[0]+s[0])/2),l=x((a[1]+s[1])/2);if(A&&l){var c=l/A,u=-1/c,g=_(M,c,A,l);i=_(k,u,g[0][0],g[0][1]),o=_(k,u,g[1][0],g[1][1])}else{var h,d;l?(h=k,d=M):(h=M,d=k),i=[[A-h,l-d],[A+h,l-d]],o=[[A-h,l+d],[A+h,l+d]]}return"M"+i.join("L")+"L"+o.reverse().join("L")+"Z"}function W(t,e){return e=Math.max(Math.min(e,c),u),t<l?t=0:c-t<l?t=c:e<l?e=0:c-e<l&&(e=c),Math.abs(e-t)>A?(t<e?(N=t,T=e):(N=e,T=t),!0):(N=null,T=null,!1)}function X(t,e){t=t||U,e=e||"M0,0Z",H.attr("d",t),j.attr("d",e),d.transitionZoombox(H,j,O,G),O=!0;var r={};nt(r),n.emit("plotly_relayouting",r)}function Z(t,n){var r,i,o=L+t,a=R+n,s=K(L,R),A=Math.min(K(o,a),c),l=z(L,R);W(s,A)&&(r=U+e.pathSector(T),N&&(r+=e.pathSector(N)),i=q(N,l)+q(T,l)),X(r,i)}function $(t,e,n,r){var i=w.findIntersectionXY(n,r,n,[t-I,E-e]);return P(i[0],i[1])}function tt(t,n){var r,i,o=L+t,a=R+n,s=z(L,R),A=z(o,a),l=D(s,B),u=D(A,B);W($(L,R,l[0],l[1]),Math.min($(o,a,u[0],u[1]),c))&&(r=U+e.pathSector(T),N&&(r+=e.pathSector(N)),i=[V(N,l[0],l[1]),V(T,l[0],l[1])].join(" ")),X(r,i)}function et(){if(d.removeZoombox(n),null!==N&&null!==T){var t={};nt(t),d.showDoubleClickNotifier(n),o.call("_guiRelayout",n,t)}}function nt(t){var n=b._rl,r=(n[1]-n[0])/(1-u/c)/c,i=[n[0]+(N-u)*r,n[0]+(T-u)*r];t[e.id+".radialaxis.range"]=i}function rt(t,r){var i=n._fullLayout.clickmode;if(d.removeZoombox(n),2===t){var a={};for(var s in e.viewInitial)a[e.id+"."+s]=e.viewInitial[s];n.emit("plotly_doubleclick",null),o.call("_guiRelayout",n,a)}i.indexOf("select")>-1&&1===t&&m(r,n,[e.xaxis],[e.yaxis],e.id,Y),i.indexOf("event")>-1&&p.click(n,r,e.id)}Y.prepFn=function(t,r,o){var a=n._fullLayout.dragmode,A=S.getBoundingClientRect();if(L=r-A.left,R=o-A.top,B){var l=w.findPolygonOffset(c,v[0],v[1],B);L+=I+l[0],R+=E+l[1]}switch(a){case"zoom":Y.moveFn=B?tt:Z,Y.clickFn=rt,Y.doneFn=et,function(){N=null,T=null,U=e.pathSubplot(),O=!1;var t=n._fullLayout[e.id];G=i(t.bgcolor).getLuminance(),(H=d.makeZoombox(s,G,g,h,U)).attr("fill-rule","evenodd"),j=d.makeCorners(s,g,h),y(n)}();break;case"select":case"lasso":C(t,r,o,Y,a)}},S.onmousemove=function(t){p.hover(n,t,e.id),n._fullLayout._lasthover=S,n._fullLayout._hoversubplot=e.id},S.onmouseout=function(t){n._dragging||f.unhover(n,t)},f.init(Y)},S.updateRadialDrag=function(t,e,n){var i=this,s=i.gd,A=i.layers,l=i.radius,c=i.innerRadius,u=i.cx,g=i.cy,h=i.radialAxis,p=Q.radialDragBoxSize,I=p/2;if(h.visible){var C,m,E,b=D(i.radialAxisAngle),w=h._rl,x=w[0],_=w[1],k=w[n],S=.75*(w[1]-w[0])/(1-e.hole)/l;n?(C=u+(l+I)*Math.cos(b),m=g-(l+I)*Math.sin(b),E="radialdrag"):(C=u+(c-I)*Math.cos(b),m=g-(c-I)*Math.sin(b),E="radialdrag-inner");var L,U,O,G=d.makeRectDragger(A,E,"crosshair",-I,-I,p,p),H={element:G,gd:s};N(r.select(G),h.visible&&c<l,{transform:F(C,m)}),H.prepFn=function(){L=null,U=null,O=null,H.moveFn=j,H.doneFn=Y,y(s)},H.clampFn=function(t,e){return Math.sqrt(t*t+e*e)<Q.MINDRAG&&(t=0,e=0),[t,e]},f.init(H)}function j(t,e){if(L)L(t,e);else{var r=[t,-e],o=[Math.cos(b),Math.sin(b)],A=Math.abs(a.dot(r,o)/Math.sqrt(a.dot(r,r)));isNaN(A)||(L=A<.5?P:K)}var l={};!function(t){null!==U?t[i.id+".radialaxis.angle"]=U:null!==O&&(t[i.id+".radialaxis.range["+n+"]"]=O)}(l),s.emit("plotly_relayouting",l)}function Y(){null!==U?o.call("_guiRelayout",s,i.id+".radialaxis.angle",U):null!==O&&o.call("_guiRelayout",s,i.id+".radialaxis.range["+n+"]",O)}function P(t,e){if(0!==n){var r=C+t,o=m+e;U=Math.atan2(g-o,r-u),i.vangles&&(U=R(U,i.vangles)),U=M(U);var a=F(u,g)+T(-U);A["radial-axis"].attr("transform",a),A["radial-line"].select("line").attr("transform",a);var s=i.gd._fullLayout,l=s[i.id];i.updateRadialAxisTitle(s,l,U)}}function K(t,e){var r=a.dot([t,-e],[Math.cos(b),Math.sin(b)]);if(O=k-S*r,S>0==(n?O>x:O<_)){var A=s._fullLayout,l=A[i.id];h.range[n]=O,h._rl[n]=O,i.updateRadialAxis(A,l),i.xaxis.setRange(),i.xaxis.setScale(),i.yaxis.setRange(),i.yaxis.setScale();var c=!1;for(var u in i.traceHash){var g=i.traceHash[u],d=a.filterVisible(g);g[0][0].trace._module.plot(s,i,d,l),o.traceIs(u,"gl")&&d.length&&(c=!0)}c&&(v(s),B(s))}else O=null}},S.updateAngularDrag=function(t){var e=this,n=e.gd,i=e.layers,s=e.radius,l=e.angularAxis,c=e.cx,u=e.cy,g=e.cxx,h=e.cyy,p=Q.angularDragBoxSize,I=d.makeDragger(i,"path","angulardrag","move"),C={element:I,gd:n};function m(t,e){return Math.atan2(h+p-e,t-g-p)}r.select(I).attr("d",e.pathAnnulus(s,s+p)).attr("transform",F(c,u)).call(E,"move");var b,w,x,_,D,k,S=i.frontplot.select(".scatterlayer").selectAll(".trace"),L=S.selectAll(".point"),R=S.selectAll(".textpoint");function N(t,s){var d=e.gd._fullLayout,f=d[e.id],p=m(b+t,w+s),I=M(p-k);if(_=x+I,i.frontplot.attr("transform",F(e.xOffset2,e.yOffset2)+T([-I,g,h])),e.vangles){D=e.radialAxisAngle+I;var C=F(c,u)+T(-I),y=F(c,u)+T(-D);i.bg.attr("transform",C),i["radial-grid"].attr("transform",C),i["radial-axis"].attr("transform",y),i["radial-line"].select("line").attr("transform",y),e.updateRadialAxisTitle(d,f,D)}else e.clipPaths.forTraces.select("path").attr("transform",F(g,h)+T(I));L.each(function(){var t=r.select(this),e=A.getTranslate(t);t.attr("transform",F(e.x,e.y)+T([I]))}),R.each(function(){var t=r.select(this),e=t.select("text"),n=A.getTranslate(t);t.attr("transform",T([I,e.attr("x"),e.attr("y")])+F(n.x,n.y))}),l.rotation=a.modHalf(_,360),e.updateAngularAxis(d,f),e._hasClipOnAxisFalse&&!a.isFullCircle(e.sectorInRad)&&S.call(A.hideOutsideRangePoints,e);var E=!1;for(var Q in e.traceHash)if(o.traceIs(Q,"gl")){var N=e.traceHash[Q],O=a.filterVisible(N);N[0][0].trace._module.plot(n,e,O,f),O.length&&(E=!0)}E&&(v(n),B(n));var G={};U(G),n.emit("plotly_relayouting",G)}function U(t){t[e.id+".angularaxis.rotation"]=_,e.vangles&&(t[e.id+".radialaxis.angle"]=D)}function O(){R.select("text").attr("transform",null);var t={};U(t),o.call("_guiRelayout",n,t)}C.prepFn=function(r,i,o){var a=t[e.id];x=a.angularaxis.rotation;var s=I.getBoundingClientRect();b=i-s.left,w=o-s.top,k=m(b,w),C.moveFn=N,C.doneFn=O,y(n)},e.vangles&&!a.isFullCircle(e.sectorInRad)&&(C.prepFn=a.noop,E(r.select(I),null)),f.init(C)},S.isPtInside=function(t){var e=this.sectorInRad,n=this.vangles,r=this.angularAxis.c2g(t.theta),i=this.radialAxis,o=i.c2l(t.r),s=i._rl;return(n?w.isPtInsidePolygon:a.isPtInsideSector)(o,r,s,e,n)},S.pathArc=function(t){var e=this.sectorInRad,n=this.vangles;return(n?w.pathPolygon:a.pathArc)(t,e[0],e[1],n)},S.pathSector=function(t){var e=this.sectorInRad,n=this.vangles;return(n?w.pathPolygon:a.pathSector)(t,e[0],e[1],n)},S.pathAnnulus=function(t,e){var n=this.sectorInRad,r=this.vangles;return(r?w.pathPolygonAnnulus:a.pathAnnulus)(t,e,n[0],n[1],r)},S.pathSubplot=function(){var t=this.innerRadius,e=this.radius;return t?this.pathAnnulus(t,e):this.pathSector(e)},S.fillViewInitialKey=function(t,e){t in this.viewInitial||(this.viewInitial[t]=e)}},{"../../components/color":593,"../../components/dragelement":611,"../../components/drawing":614,"../../components/fx":632,"../../components/titles":681,"../../constants/alignment":688,"../../lib":719,"../../lib/clear_gl_canvases":704,"../../lib/setcursor":739,"../../plot_api/subroutines":758,"../../plots/cartesian/axes":767,"../../registry":847,"../cartesian/autorange":766,"../cartesian/dragbox":775,"../cartesian/select":784,"../cartesian/set_convert":785,"../plots":828,"./constants":829,"./helpers":830,"./set_convert":841,d3:163,tinycolor2:537}],841:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../cartesian/set_convert"),o=r.deg2rad,a=r.rad2deg;e.exports=function(t,e,n){switch(i(t,n),t._id){case"x":case"radialaxis":!function(t,e){var n=e._subplot;t.setGeometry=function(){var e=t._rl[0],r=t._rl[1],i=n.innerRadius,o=(n.radius-i)/(r-e),a=i/o,s=e>r?function(t){return t<=0}:function(t){return t>=0};t.c2g=function(n){var r=t.c2l(n)-e;return(s(r)?r:0)+a},t.g2c=function(n){return t.l2c(n+e-a)},t.g2p=function(t){return t*o},t.c2p=function(e){return t.g2p(t.c2g(e))}}}(t,e);break;case"angularaxis":!function(t,e){var n=t.type;if("linear"===n){var i=t.d2c,s=t.c2d;t.d2c=function(t,e){return function(t,e){return"degrees"===e?o(t):t}(i(t),e)},t.c2d=function(t,e){return s(function(t,e){return"degrees"===e?a(t):t}(t,e))}}t.makeCalcdata=function(e,i){var o,a,s=e[i],A=e._length,l=function(n){return t.d2c(n,e.thetaunit)};if(s){if(r.isTypedArray(s)&&"linear"===n){if(A===s.length)return s;if(s.subarray)return s.subarray(0,A)}for(o=new Array(A),a=0;a<A;a++)o[a]=l(s[a])}else{var c=i+"0",u="d"+i,g=c in e?l(e[c]):0,h=e[u]?l(e[u]):(t.period||2*Math.PI)/A;for(o=new Array(A),a=0;a<A;a++)o[a]=g+a*h}return o},t.setGeometry=function(){var i,s,A,l,c=e.sector,u=c.map(o),g={clockwise:-1,counterclockwise:1}[t.direction],h=o(t.rotation),d=function(t){return g*t+h},f=function(t){return(t-h)/g};switch(n){case"linear":s=i=r.identity,l=o,A=a,t.range=r.isFullCircle(u)?[c[0],c[0]+360]:u.map(f).map(a);break;case"category":var p=t._categories.length,I=t.period?Math.max(t.period,p):p;0===I&&(I=1),s=l=function(t){return 2*t*Math.PI/I},i=A=function(t){return t*I/Math.PI/2},t.range=[0,I]}t.c2g=function(t){return d(s(t))},t.g2c=function(t){return i(f(t))},t.t2g=function(t){return d(l(t))},t.g2t=function(t){return A(f(t))}}}(t,e)}}},{"../../lib":719,"../cartesian/set_convert":785}],842:[function(t,e,n){"use strict";var r=t("../lib"),i=t("../plot_api/plot_template"),o=t("./domain").defaults;e.exports=function(t,e,n,a){var s,A,l=a.type,c=a.attributes,u=a.handleDefaults,g=a.partition||"x",h=e._subplots[l],d=h.length,f=d&&h[0].replace(/\d+$/,"");function p(t,e){return r.coerce(s,A,c,t,e)}for(var I=0;I<d;I++){var C=h[I];s=t[C]?t[C]:t[C]={},A=i.newContainer(e,C,f),p("uirevision",e.uirevision);var m={};m[g]=[I/d,(I+1)/d],o(A,e,p,m),a.id=C,u(s,A,p,a)}}},{"../lib":719,"../plot_api/plot_template":757,"./domain":792}],843:[function(t,e,n){"use strict";var r=t("./ternary"),i=t("../../plots/get_data").getSubplotCalcData,o=t("../../lib").counterRegex;n.name="ternary";var a=n.attr="subplot";n.idRoot="ternary",n.idRegex=n.attrRegex=o("ternary"),(n.attributes={})[a]={valType:"subplotid",dflt:"ternary",editType:"calc"},n.layoutAttributes=t("./layout_attributes"),n.supplyLayoutDefaults=t("./layout_defaults"),n.plot=function(t){for(var e=t._fullLayout,n=t.calcdata,o=e._subplots.ternary,a=0;a<o.length;a++){var s=o[a],A=i(n,"ternary",s),l=e[s]._subplot;l||(l=new r({id:s,graphDiv:t,container:e._ternarylayer.node()},e),e[s]._subplot=l),l.plot(A,e,t._promises)}},n.clean=function(t,e,n,r){for(var i=r._subplots.ternary||[],o=0;o<i.length;o++){var a=i[o],s=r[a]._subplot;!e[a]&&s&&(s.plotContainer.remove(),s.clipDef.remove(),s.clipDefRelative.remove(),s.layers["a-title"].remove(),s.layers["b-title"].remove(),s.layers["c-title"].remove())}}},{"../../lib":719,"../../plots/get_data":802,"./layout_attributes":844,"./layout_defaults":845,"./ternary":846}],844:[function(t,e,n){"use strict";var r=t("../../components/color/attributes"),i=t("../domain").attributes,o=t("../cartesian/layout_attributes"),a=t("../../plot_api/edit_types").overrideAll,s=t("../../lib/extend").extendFlat,A={title:o.title,color:o.color,tickmode:o.tickmode,nticks:s({},o.nticks,{dflt:6,min:1}),tick0:o.tick0,dtick:o.dtick,tickvals:o.tickvals,ticktext:o.ticktext,ticks:o.ticks,ticklen:o.ticklen,tickwidth:o.tickwidth,tickcolor:o.tickcolor,showticklabels:o.showticklabels,showtickprefix:o.showtickprefix,tickprefix:o.tickprefix,showticksuffix:o.showticksuffix,ticksuffix:o.ticksuffix,showexponent:o.showexponent,exponentformat:o.exponentformat,separatethousands:o.separatethousands,tickfont:o.tickfont,tickangle:o.tickangle,tickformat:o.tickformat,tickformatstops:o.tickformatstops,hoverformat:o.hoverformat,showline:s({},o.showline,{dflt:!0}),linecolor:o.linecolor,linewidth:o.linewidth,showgrid:s({},o.showgrid,{dflt:!0}),gridcolor:o.gridcolor,gridwidth:o.gridwidth,layer:o.layer,min:{valType:"number",dflt:0,min:0},_deprecated:{title:o._deprecated.title,titlefont:o._deprecated.titlefont}},l=e.exports=a({domain:i({name:"ternary"}),bgcolor:{valType:"color",dflt:r.background},sum:{valType:"number",dflt:1,min:0},aaxis:A,baxis:A,caxis:A},"plot","from-root");l.uirevision={valType:"any",editType:"none"},l.aaxis.uirevision=l.baxis.uirevision=l.caxis.uirevision={valType:"any",editType:"none"}},{"../../components/color/attributes":592,"../../lib/extend":710,"../../plot_api/edit_types":750,"../cartesian/layout_attributes":779,"../domain":792}],845:[function(t,e,n){"use strict";var r=t("../../components/color"),i=t("../../plot_api/plot_template"),o=t("../../lib"),a=t("../subplot_defaults"),s=t("../cartesian/tick_label_defaults"),A=t("../cartesian/tick_mark_defaults"),l=t("../cartesian/tick_value_defaults"),c=t("../cartesian/line_grid_defaults"),u=t("./layout_attributes"),g=["aaxis","baxis","caxis"];function h(t,e,n,o){var a,s,A,l=n("bgcolor"),c=n("sum");o.bgColor=r.combine(l,o.paper_bgcolor);for(var u=0;u<g.length;u++)s=t[a=g[u]]||{},(A=i.newContainer(e,a))._name=a,d(s,A,o,e);var h=e.aaxis,f=e.baxis,p=e.caxis;h.min+f.min+p.min>=c&&(h.min=0,f.min=0,p.min=0,t.aaxis&&delete t.aaxis.min,t.baxis&&delete t.baxis.min,t.caxis&&delete t.caxis.min)}function d(t,e,n,r){var i=u[e._name];function a(n,r){return o.coerce(t,e,i,n,r)}a("uirevision",r.uirevision),e.type="linear";var g=a("color"),h=g!==i.color.dflt?g:n.font.color,d=e._name.charAt(0).toUpperCase(),f="Component "+d,p=a("title.text",f);e._hovertitle=p===f?p:d,o.coerceFont(a,"title.font",{family:n.font.family,size:Math.round(1.2*n.font.size),color:h}),a("min"),l(t,e,a,"linear"),s(t,e,a,"linear",{}),A(t,e,a,{outerTicks:!0}),a("showticklabels")&&(o.coerceFont(a,"tickfont",{family:n.font.family,size:n.font.size,color:h}),a("tickangle"),a("tickformat")),c(t,e,a,{dfltColor:g,bgColor:n.bgColor,blend:60,showLine:!0,showGrid:!0,noZeroLine:!0,attributes:i}),a("hoverformat"),a("layer")}e.exports=function(t,e,n){a(t,e,n,{type:"ternary",attributes:u,handleDefaults:h,font:e.font,paper_bgcolor:e.paper_bgcolor})}},{"../../components/color":593,"../../lib":719,"../../plot_api/plot_template":757,"../cartesian/line_grid_defaults":781,"../cartesian/tick_label_defaults":786,"../cartesian/tick_mark_defaults":787,"../cartesian/tick_value_defaults":788,"../subplot_defaults":842,"./layout_attributes":844}],846:[function(t,e,n){"use strict";var r=t("d3"),i=t("tinycolor2"),o=t("../../registry"),a=t("../../lib"),s=a._,A=t("../../components/color"),l=t("../../components/drawing"),c=t("../cartesian/set_convert"),u=t("../../lib/extend").extendFlat,g=t("../plots"),h=t("../cartesian/axes"),d=t("../../components/dragelement"),f=t("../../components/fx"),p=t("../../components/titles"),I=t("../cartesian/select").prepSelect,C=t("../cartesian/select").selectOnClick,m=t("../cartesian/select").clearSelect,y=t("../cartesian/constants");function E(t,e){this.id=t.id,this.graphDiv=t.graphDiv,this.init(e),this.makeFramework(e),this.aTickLayout=null,this.bTickLayout=null,this.cTickLayout=null}e.exports=E;var v=E.prototype;v.init=function(t){this.container=t._ternarylayer,this.defs=t._defs,this.layoutId=t._uid,this.traceHash={},this.layers={}},v.plot=function(t,e){var n=e[this.id],r=e._size;this._hasClipOnAxisFalse=!1;for(var i=0;i<t.length;i++)if(!1===t[i][0].trace.cliponaxis){this._hasClipOnAxisFalse=!0;break}this.updateLayers(n),this.adjustLayout(n,r),g.generalUpdatePerTraceModule(this.graphDiv,this,t,n),this.layers.plotbg.select("path").call(A.fill,n.bgcolor)},v.makeFramework=function(t){var e=this.graphDiv,n=t[this.id],r=this.clipId="clip"+this.layoutId+this.id,i=this.clipIdRelative="clip-relative"+this.layoutId+this.id;this.clipDef=a.ensureSingleById(t._clips,"clipPath",r,function(t){t.append("path").attr("d","M0,0Z")}),this.clipDefRelative=a.ensureSingleById(t._clips,"clipPath",i,function(t){t.append("path").attr("d","M0,0Z")}),this.plotContainer=a.ensureSingle(this.container,"g",this.id),this.updateLayers(n),l.setClipUrl(this.layers.backplot,r,e),l.setClipUrl(this.layers.grids,r,e)},v.updateLayers=function(t){var e=this.layers,n=["draglayer","plotbg","backplot","grids"];"below traces"===t.aaxis.layer&&n.push("aaxis","aline"),"below traces"===t.baxis.layer&&n.push("baxis","bline"),"below traces"===t.caxis.layer&&n.push("caxis","cline"),n.push("frontplot"),"above traces"===t.aaxis.layer&&n.push("aaxis","aline"),"above traces"===t.baxis.layer&&n.push("baxis","bline"),"above traces"===t.caxis.layer&&n.push("caxis","cline");var i=this.plotContainer.selectAll("g.toplevel").data(n,String),o=["agrid","bgrid","cgrid"];i.enter().append("g").attr("class",function(t){return"toplevel "+t}).each(function(t){var n=r.select(this);e[t]=n,"frontplot"===t?n.append("g").classed("scatterlayer",!0):"backplot"===t?n.append("g").classed("maplayer",!0):"plotbg"===t?n.append("path").attr("d","M0,0Z"):"aline"===t||"bline"===t||"cline"===t?n.append("path"):"grids"===t&&o.forEach(function(t){e[t]=n.append("g").classed("grid "+t,!0)})}),i.order()};var B=Math.sqrt(4/3);v.adjustLayout=function(t,e){var n,r,i,o,a,s,g=this,h=t.domain,d=(h.x[0]+h.x[1])/2,f=(h.y[0]+h.y[1])/2,p=h.x[1]-h.x[0],I=h.y[1]-h.y[0],C=p*e.w,m=I*e.h,y=t.sum,E=t.aaxis.min,v=t.baxis.min,b=t.caxis.min;C>B*m?i=(o=m)*B:o=(i=C)/B,a=p*i/C,s=I*o/m,n=e.l+e.w*d-i/2,r=e.t+e.h*(1-f)-o/2,g.x0=n,g.y0=r,g.w=i,g.h=o,g.sum=y,g.xaxis={type:"linear",range:[E+2*b-y,y-E-2*v],domain:[d-a/2,d+a/2],_id:"x"},c(g.xaxis,g.graphDiv._fullLayout),g.xaxis.setScale(),g.xaxis.isPtWithinRange=function(t){return t.a>=g.aaxis.range[0]&&t.a<=g.aaxis.range[1]&&t.b>=g.baxis.range[1]&&t.b<=g.baxis.range[0]&&t.c>=g.caxis.range[1]&&t.c<=g.caxis.range[0]},g.yaxis={type:"linear",range:[E,y-v-b],domain:[f-s/2,f+s/2],_id:"y"},c(g.yaxis,g.graphDiv._fullLayout),g.yaxis.setScale(),g.yaxis.isPtWithinRange=function(){return!0};var Q=g.yaxis.domain[0],w=g.aaxis=u({},t.aaxis,{range:[E,y-v-b],side:"left",tickangle:(+t.aaxis.tickangle||0)-30,domain:[Q,Q+s*B],anchor:"free",position:0,_id:"y",_length:i});c(w,g.graphDiv._fullLayout),w.setScale();var x=g.baxis=u({},t.baxis,{range:[y-E-b,v],side:"bottom",domain:g.xaxis.domain,anchor:"free",position:0,_id:"x",_length:i});c(x,g.graphDiv._fullLayout),x.setScale();var _=g.caxis=u({},t.caxis,{range:[y-E-v,b],side:"right",tickangle:(+t.caxis.tickangle||0)+30,domain:[Q,Q+s*B],anchor:"free",position:0,_id:"y",_length:i});c(_,g.graphDiv._fullLayout),_.setScale();var D="M"+n+","+(r+o)+"h"+i+"l-"+i/2+",-"+o+"Z";g.clipDef.select("path").attr("d",D),g.layers.plotbg.select("path").attr("d",D);var M="M0,"+o+"h"+i+"l-"+i/2+",-"+o+"Z";g.clipDefRelative.select("path").attr("d",M);var k="translate("+n+","+r+")";g.plotContainer.selectAll(".scatterlayer,.maplayer").attr("transform",k),g.clipDefRelative.select("path").attr("transform",null);var S="translate("+(n-x._offset)+","+(r+o)+")";g.layers.baxis.attr("transform",S),g.layers.bgrid.attr("transform",S);var L="translate("+(n+i/2)+","+r+")rotate(30)translate(0,"+-w._offset+")";g.layers.aaxis.attr("transform",L),g.layers.agrid.attr("transform",L);var R="translate("+(n+i/2)+","+r+")rotate(-30)translate(0,"+-_._offset+")";g.layers.caxis.attr("transform",R),g.layers.cgrid.attr("transform",R),g.drawAxes(!0),g.layers.aline.select("path").attr("d",w.showline?"M"+n+","+(r+o)+"l"+i/2+",-"+o:"M0,0").call(A.stroke,w.linecolor||"#000").style("stroke-width",(w.linewidth||0)+"px"),g.layers.bline.select("path").attr("d",x.showline?"M"+n+","+(r+o)+"h"+i:"M0,0").call(A.stroke,x.linecolor||"#000").style("stroke-width",(x.linewidth||0)+"px"),g.layers.cline.select("path").attr("d",_.showline?"M"+(n+i/2)+","+r+"l"+i/2+","+o:"M0,0").call(A.stroke,_.linecolor||"#000").style("stroke-width",(_.linewidth||0)+"px"),g.graphDiv._context.staticPlot||g.initInteractions(),l.setClipUrl(g.layers.frontplot,g._hasClipOnAxisFalse?null:g.clipId,g.graphDiv)},v.drawAxes=function(t){var e=this.graphDiv,n=this.id.substr(7)+"title",r=this.layers,i=this.aaxis,o=this.baxis,a=this.caxis;if(this.drawAx(i),this.drawAx(o),this.drawAx(a),t){var A=Math.max(i.showticklabels?i.tickfont.size/2:0,(a.showticklabels?.75*a.tickfont.size:0)+("outside"===a.ticks?.87*a.ticklen:0)),l=(o.showticklabels?o.tickfont.size:0)+("outside"===o.ticks?o.ticklen:0)+3;r["a-title"]=p.draw(e,"a"+n,{propContainer:i,propName:this.id+".aaxis.title",placeholder:s(e,"Click to enter Component A title"),attributes:{x:this.x0+this.w/2,y:this.y0-i.title.font.size/3-A,"text-anchor":"middle"}}),r["b-title"]=p.draw(e,"b"+n,{propContainer:o,propName:this.id+".baxis.title",placeholder:s(e,"Click to enter Component B title"),attributes:{x:this.x0-l,y:this.y0+this.h+.83*o.title.font.size+l,"text-anchor":"middle"}}),r["c-title"]=p.draw(e,"c"+n,{propContainer:a,propName:this.id+".caxis.title",placeholder:s(e,"Click to enter Component C title"),attributes:{x:this.x0+this.w+l,y:this.y0+this.h+.83*a.title.font.size+l,"text-anchor":"middle"}})}},v.drawAx=function(t){var e,n=this.graphDiv,r=t._name,i=r.charAt(0),o=t._id,s=this.layers[r],A=i+"tickLayout",l=(e=t).ticks+String(e.ticklen)+String(e.showticklabels);this[A]!==l&&(s.selectAll("."+o+"tick").remove(),this[A]=l),t.setScale();var c=h.calcTicks(t),u=h.clipEnds(t,c),g=h.makeTransFn(t),d=h.getTickSigns(t)[2],f=a.deg2rad(30),p=d*(t.linewidth||1)/2,I=d*t.ticklen,C=this.w,m=this.h,y="b"===i?"M0,"+p+"l"+Math.sin(f)*I+","+Math.cos(f)*I:"M"+p+",0l"+Math.cos(f)*I+","+-Math.sin(f)*I,E={a:"M0,0l"+m+",-"+C/2,b:"M0,0l-"+C/2+",-"+m,c:"M0,0l-"+m+","+C/2}[i];h.drawTicks(n,t,{vals:"inside"===t.ticks?u:c,layer:s,path:y,transFn:g,crisp:!1}),h.drawGrid(n,t,{vals:u,layer:this.layers[i+"grid"],path:E,transFn:g,crisp:!1}),h.drawLabels(n,t,{vals:c,layer:s,transFn:g,labelFns:h.makeLabelFns(t,0,30)})};var b=y.MINZOOM/2+.87,Q="m-0.87,.5h"+b+"v3h-"+(b+5.2)+"l"+(b/2+2.6)+",-"+(.87*b+4.5)+"l2.6,1.5l-"+b/2+","+.87*b+"Z",w="m0.87,.5h-"+b+"v3h"+(b+5.2)+"l-"+(b/2+2.6)+",-"+(.87*b+4.5)+"l-2.6,1.5l"+b/2+","+.87*b+"Z",x="m0,1l"+b/2+","+.87*b+"l2.6,-1.5l-"+(b/2+2.6)+",-"+(.87*b+4.5)+"l-"+(b/2+2.6)+","+(.87*b+4.5)+"l2.6,1.5l"+b/2+",-"+.87*b+"Z",_="m0.5,0.5h5v-2h-5v-5h-2v5h-5v2h5v5h2Z",D=!0;function M(t){r.select(t).selectAll(".zoombox,.js-zoombox-backdrop,.js-zoombox-menu,.zoombox-corners").remove()}v.initInteractions=function(){var t,e,n,r,c,u,g,h,p,E,v=this,b=v.layers.plotbg.select("path").node(),k=v.graphDiv,S=k._fullLayout._zoomlayer,L={element:b,gd:k,plotinfo:{id:v.id,xaxis:v.xaxis,yaxis:v.yaxis},subplot:v.id,prepFn:function(o,a,s){L.xaxes=[v.xaxis],L.yaxes=[v.yaxis];var l=k._fullLayout.dragmode;L.minDrag="lasso"===l?1:void 0,"zoom"===l?(L.moveFn=O,L.clickFn=N,L.doneFn=G,function(o,a,s){var l=b.getBoundingClientRect();t=a-l.left,e=s-l.top,n={a:v.aaxis.range[0],b:v.baxis.range[1],c:v.caxis.range[1]},c=n,r=v.aaxis.range[1]-n.a,u=i(v.graphDiv._fullLayout[v.id].bgcolor).getLuminance(),g="M0,"+v.h+"L"+v.w/2+", 0L"+v.w+","+v.h+"Z",h=!1,p=S.append("path").attr("class","zoombox").attr("transform","translate("+v.x0+", "+v.y0+")").style({fill:u>.2?"rgba(0,0,0,0)":"rgba(255,255,255,0)","stroke-width":0}).attr("d",g),E=S.append("path").attr("class","zoombox-corners").attr("transform","translate("+v.x0+", "+v.y0+")").style({fill:A.background,stroke:A.defaultLine,"stroke-width":1,opacity:0}).attr("d","M0,0Z"),m(k)}(0,a,s)):"pan"===l?(L.moveFn=H,L.clickFn=N,L.doneFn=j,n={a:v.aaxis.range[0],b:v.baxis.range[1],c:v.caxis.range[1]},c=n,m(k)):"select"!==l&&"lasso"!==l||I(o,a,s,L,l)}};function R(t){var e={};return e[v.id+".aaxis.min"]=t.a,e[v.id+".baxis.min"]=t.b,e[v.id+".caxis.min"]=t.c,e}function N(t,e){var n=k._fullLayout.clickmode;M(k),2===t&&(k.emit("plotly_doubleclick",null),o.call("_guiRelayout",k,R({a:0,b:0,c:0}))),n.indexOf("select")>-1&&1===t&&C(e,k,[v.xaxis],[v.yaxis],v.id,L),n.indexOf("event")>-1&&f.click(k,e,v.id)}function F(t,e){return 1-e/v.h}function T(t,e){return 1-(t+(v.h-e)/Math.sqrt(3))/v.w}function U(t,e){return(t-(v.h-e)/Math.sqrt(3))/v.w}function O(i,o){var a=t+i,s=e+o,A=Math.max(0,Math.min(1,F(0,e),F(0,s))),l=Math.max(0,Math.min(1,T(t,e),T(a,s))),d=Math.max(0,Math.min(1,U(t,e),U(a,s))),f=(A/2+d)*v.w,I=(1-A/2-l)*v.w,C=(f+I)/2,m=I-f,b=(1-A)*v.h,D=b-m/B;m<y.MINZOOM?(c=n,p.attr("d",g),E.attr("d","M0,0Z")):(c={a:n.a+A*r,b:n.b+l*r,c:n.c+d*r},p.attr("d",g+"M"+f+","+b+"H"+I+"L"+C+","+D+"L"+f+","+b+"Z"),E.attr("d","M"+t+","+e+_+"M"+f+","+b+Q+"M"+I+","+b+w+"M"+C+","+D+x)),h||(p.transition().style("fill",u>.2?"rgba(0,0,0,0.4)":"rgba(255,255,255,0.3)").duration(200),E.transition().style("opacity",1).duration(200),h=!0),k.emit("plotly_relayouting",R(c))}function G(){M(k),c!==n&&(o.call("_guiRelayout",k,R(c)),D&&k.data&&k._context.showTips&&(a.notifier(s(k,"Double-click to zoom back out"),"long"),D=!1))}function H(t,e){var r=t/v.xaxis._m,i=e/v.yaxis._m,o=[(c={a:n.a-i,b:n.b+(r+i)/2,c:n.c-(r-i)/2}).a,c.b,c.c].sort(),a=o.indexOf(c.a),s=o.indexOf(c.b),A=o.indexOf(c.c);o[0]<0&&(o[1]+o[0]/2<0?(o[2]+=o[0]+o[1],o[0]=o[1]=0):(o[2]+=o[0]/2,o[1]+=o[0]/2,o[0]=0),c={a:o[a],b:o[s],c:o[A]},e=(n.a-c.a)*v.yaxis._m,t=(n.c-c.c-n.b+c.b)*v.xaxis._m);var u="translate("+(v.x0+t)+","+(v.y0+e)+")";v.plotContainer.selectAll(".scatterlayer,.maplayer").attr("transform",u);var g="translate("+-t+","+-e+")";v.clipDefRelative.select("path").attr("transform",g),v.aaxis.range=[c.a,v.sum-c.b-c.c],v.baxis.range=[v.sum-c.a-c.c,c.b],v.caxis.range=[v.sum-c.a-c.b,c.c],v.drawAxes(!1),v._hasClipOnAxisFalse&&v.plotContainer.select(".scatterlayer").selectAll(".trace").call(l.hideOutsideRangePoints,v),k.emit("plotly_relayouting",R(c))}function j(){o.call("_guiRelayout",k,R(c))}b.onmousemove=function(t){f.hover(k,t,v.id),k._fullLayout._lasthover=b,k._fullLayout._hoversubplot=v.id},b.onmouseout=function(t){k._dragging||d.unhover(k,t)},d.init(L)}},{"../../components/color":593,"../../components/dragelement":611,"../../components/drawing":614,"../../components/fx":632,"../../components/titles":681,"../../lib":719,"../../lib/extend":710,"../../registry":847,"../cartesian/axes":767,"../cartesian/constants":773,"../cartesian/select":784,"../cartesian/set_convert":785,"../plots":828,d3:163,tinycolor2:537}],847:[function(t,e,n){"use strict";var r=t("./lib/loggers"),i=t("./lib/noop"),o=t("./lib/push_unique"),a=t("./lib/is_plain_object"),s=t("./lib/dom").addStyleRule,A=t("./lib/extend"),l=t("./plots/attributes"),c=t("./plots/layout_attributes"),u=A.extendFlat,g=A.extendDeepAll;function h(t){var e=t.name,i=t.categories,o=t.meta;if(n.modules[e])r.log("Type "+e+" already registered");else{n.subplotsRegistry[t.basePlotModule.name]||function(t){var e=t.name;if(n.subplotsRegistry[e])r.log("Plot type "+e+" already registered.");else for(var i in I(t),n.subplotsRegistry[e]=t,n.componentsRegistry)y(i,t.name)}(t.basePlotModule);for(var a={},A=0;A<i.length;A++)a[i[A]]=!0,n.allCategories[i[A]]=!0;for(var l in n.modules[e]={_module:t,categories:a},o&&Object.keys(o).length&&(n.modules[e].meta=o),n.allTypes.push(e),n.componentsRegistry)C(l,e);t.layoutAttributes&&u(n.traceLayoutAttributes,t.layoutAttributes);var c=t.basePlotModule,g=c.name;if("mapbox"===g){var h=c.constants.styleRules;for(var d in h)s(".js-plotly-plot .plotly .mapboxgl-"+d,h[d])}"geo"!==g&&"mapbox"!==g||void 0===typeof window||void 0!==window.PlotlyGeoAssets||(window.PlotlyGeoAssets={topojson:{}})}}function d(t){if("string"!=typeof t.name)throw new Error("Component module *name* must be a string.");var e=t.name;for(var r in n.componentsRegistry[e]=t,t.layoutAttributes&&(t.layoutAttributes._isLinkedToArray&&o(n.layoutArrayContainers,e),I(t)),n.modules)C(e,r);for(var i in n.subplotsRegistry)y(e,i);for(var a in n.transformsRegistry)m(e,a);t.schema&&t.schema.layout&&g(c,t.schema.layout)}function f(t){if("string"!=typeof t.name)throw new Error("Transform module *name* must be a string.");var e="Transform module "+t.name,i="function"==typeof t.transform,o="function"==typeof t.calcTransform;if(!i&&!o)throw new Error(e+" is missing a *transform* or *calcTransform* method.");for(var s in i&&o&&r.log([e+" has both a *transform* and *calcTransform* methods.","Please note that all *transform* methods are executed","before all *calcTransform* methods."].join(" ")),a(t.attributes)||r.log(e+" registered without an *attributes* object."),"function"!=typeof t.supplyDefaults&&r.log(e+" registered without a *supplyDefaults* method."),n.transformsRegistry[t.name]=t,n.componentsRegistry)m(s,t.name)}function p(t){var e=t.name,r=e.split("-")[0],i=t.dictionary,o=t.format,a=i&&Object.keys(i).length,s=o&&Object.keys(o).length,A=n.localeRegistry,l=A[e];if(l||(A[e]=l={}),r!==e){var c=A[r];c||(A[r]=c={}),a&&c.dictionary===l.dictionary&&(c.dictionary=i),s&&c.format===l.format&&(c.format=o)}a&&(l.dictionary=i),s&&(l.format=o)}function I(t){if(t.layoutAttributes){var e=t.layoutAttributes._arrayAttrRegexps;if(e)for(var r=0;r<e.length;r++)o(n.layoutArrayRegexes,e[r])}}function C(t,e){var r=n.componentsRegistry[t].schema;if(r&&r.traces){var i=r.traces[e];i&&g(n.modules[e]._module.attributes,i)}}function m(t,e){var r=n.componentsRegistry[t].schema;if(r&&r.transforms){var i=r.transforms[e];i&&g(n.transformsRegistry[e].attributes,i)}}function y(t,e){var r=n.componentsRegistry[t].schema;if(r&&r.subplots){var i=n.subplotsRegistry[e],o=i.layoutAttributes,a="subplot"===i.attr?i.name:i.attr;Array.isArray(a)&&(a=a[0]);var s=r.subplots[a];o&&s&&g(o,s)}}function E(t){return"object"==typeof t&&(t=t.type),t}n.modules={},n.allCategories={},n.allTypes=[],n.subplotsRegistry={},n.transformsRegistry={},n.componentsRegistry={},n.layoutArrayContainers=[],n.layoutArrayRegexes=[],n.traceLayoutAttributes={},n.localeRegistry={},n.apiMethodRegistry={},n.collectableSubplotTypes=null,n.register=function(t){if(n.collectableSubplotTypes=null,!t)throw new Error("No argument passed to Plotly.register.");t&&!Array.isArray(t)&&(t=[t]);for(var e=0;e<t.length;e++){var r=t[e];if(!r)throw new Error("Invalid module was attempted to be registered!");switch(r.moduleType){case"trace":h(r);break;case"transform":f(r);break;case"component":d(r);break;case"locale":p(r);break;case"apiMethod":var i=r.name;n.apiMethodRegistry[i]=r.fn;break;default:throw new Error("Invalid module was attempted to be registered!")}}},n.getModule=function(t){var e=n.modules[E(t)];return!!e&&e._module},n.traceIs=function(t,e){if("various"===(t=E(t)))return!1;var i=n.modules[t];return i||(t&&"area"!==t&&r.log("Unrecognized trace type "+t+"."),i=n.modules[l.type.dflt]),!!i.categories[e]},n.getTransformIndices=function(t,e){for(var n=[],r=t.transforms||[],i=0;i<r.length;i++)r[i].type===e&&n.push(i);return n},n.hasTransform=function(t,e){for(var n=t.transforms||[],r=0;r<n.length;r++)if(n[r].type===e)return!0;return!1},n.getComponentMethod=function(t,e){var r=n.componentsRegistry[t];return r&&r[e]||i},n.call=function(){var t=arguments[0],e=[].slice.call(arguments,1);return n.apiMethodRegistry[t].apply(null,e)}},{"./lib/dom":708,"./lib/extend":710,"./lib/is_plain_object":720,"./lib/loggers":723,"./lib/noop":728,"./lib/push_unique":733,"./plots/attributes":764,"./plots/layout_attributes":819}],848:[function(t,e,n){"use strict";var r=t("../registry"),i=t("../lib"),o=i.extendFlat,a=i.extendDeep;function s(t){var e;switch(t){case"themes__thumb":e={autosize:!0,width:150,height:150,title:{text:""},showlegend:!1,margin:{l:5,r:5,t:5,b:5,pad:0},annotations:[]};break;case"thumbnail":e={title:{text:""},hidesources:!0,showlegend:!1,borderwidth:0,bordercolor:"",margin:{l:1,r:1,t:1,b:1,pad:0},annotations:[]};break;default:e={}}return e}e.exports=function(t,e){var n;t.framework&&t.framework.isPolar&&(t=t.framework.getConfig());var i,A=t.data,l=t.layout,c=a([],A),u=a({},l,s(e.tileClass)),g=t._context||{};if(e.width&&(u.width=e.width),e.height&&(u.height=e.height),"thumbnail"===e.tileClass||"themes__thumb"===e.tileClass){u.annotations=[];var h=Object.keys(u);for(n=0;n<h.length;n++)i=h[n],["xaxis","yaxis","zaxis"].indexOf(i.slice(0,5))>-1&&(u[h[n]].title={text:""});for(n=0;n<c.length;n++){var d=c[n];d.showscale=!1,d.marker&&(d.marker.showscale=!1),r.traceIs(d,"pie-like")&&(d.textposition="none")}}if(Array.isArray(e.annotations))for(n=0;n<e.annotations.length;n++)u.annotations.push(e.annotations[n]);var f=Object.keys(u).filter(function(t){return t.match(/^scene\d*$/)});if(f.length){var p={};for("thumbnail"===e.tileClass&&(p={title:{text:""},showaxeslabels:!1,showticklabels:!1,linetickenable:!1}),n=0;n<f.length;n++){var I=u[f[n]];I.xaxis||(I.xaxis={}),I.yaxis||(I.yaxis={}),I.zaxis||(I.zaxis={}),o(I.xaxis,p),o(I.yaxis,p),o(I.zaxis,p),I._scene=null}}var C=document.createElement("div");e.tileClass&&(C.className=e.tileClass);var m={gd:C,td:C,layout:u,data:c,config:{staticPlot:void 0===e.staticPlot||e.staticPlot,plotGlPixelRatio:void 0===e.plotGlPixelRatio?2:e.plotGlPixelRatio,displaylogo:e.displaylogo||!1,showLink:e.showLink||!1,showTips:e.showTips||!1,mapboxAccessToken:g.mapboxAccessToken}};return"transparent"!==e.setBackground&&(m.config.setBackground=e.setBackground||"opaque"),m.gd.defaultLayout=s(e.tileClass),m}},{"../lib":719,"../registry":847}],849:[function(t,e,n){"use strict";var r=t("../lib"),i=t("../plot_api/to_image"),o=t("./filesaver"),a=t("./helpers");e.exports=function(t,e){var n;return r.isPlainObject(t)||(n=r.getGraphDiv(t)),(e=e||{}).format=e.format||"png",e.imageDataOnly=!0,new Promise(function(s,A){n&&n._snapshotInProgress&&A(new Error("Snapshotting already in progress.")),r.isIE()&&"svg"!==e.format&&A(new Error(a.MSG_IE_BAD_FORMAT)),n&&(n._snapshotInProgress=!0);var l=i(t,e),c=e.filename||t.fn||"newplot";c+="."+e.format,l.then(function(t){return n&&(n._snapshotInProgress=!1),o(t,c,e.format)}).then(function(t){s(t)}).catch(function(t){n&&(n._snapshotInProgress=!1),A(t)})})}},{"../lib":719,"../plot_api/to_image":760,"./filesaver":850,"./helpers":851}],850:[function(t,e,n){"use strict";var r=t("../lib"),i=t("./helpers");e.exports=function(t,e,n){var o=document.createElement("a"),a="download"in o;return new Promise(function(s,A){var l,c;if(r.isIE9orBelow()&&A(new Error("IE < 10 unsupported")),r.isSafari()){var u="svg"===n?",":";base64,";return i.octetStream(u+encodeURIComponent(t)),s(e)}return r.isIE()?(l=i.createBlob(t,"svg"),window.navigator.msSaveBlob(l,e),l=null,s(e)):a?(l=i.createBlob(t,n),c=i.createObjectURL(l),o.href=c,o.download=e,document.body.appendChild(o),o.click(),document.body.removeChild(o),i.revokeObjectURL(c),l=null,s(e)):void A(new Error("download error"))})}},{"../lib":719,"./helpers":851}],851:[function(t,e,n){"use strict";var r=t("../registry");n.getDelay=function(t){return t._has&&(t._has("gl3d")||t._has("gl2d")||t._has("mapbox"))?500:0},n.getRedrawFunc=function(t){return function(){var e=t._fullLayout||{};(!e._has||!e._has("polar"))&&t.data&&t.data[0]&&t.data[0].r||r.getComponentMethod("colorbar","draw")(t)}},n.encodeSVG=function(t){return"data:image/svg+xml,"+encodeURIComponent(t)};var i=window.URL||window.webkitURL;n.createObjectURL=function(t){return i.createObjectURL(t)},n.revokeObjectURL=function(t){return i.revokeObjectURL(t)},n.createBlob=function(t,e){if("svg"===e)return new window.Blob([t],{type:"image/svg+xml;charset=utf-8"});var n=function(t){for(var e=t.length,n=new ArrayBuffer(e),r=new Uint8Array(n),i=0;i<e;i++)r[i]=t.charCodeAt(i);return n}(window.atob(t));return new window.Blob([n],{type:"image/"+e})},n.octetStream=function(t){document.location.href="data:application/octet-stream"+t},n.IMAGE_URL_PREFIX=/^data:image\/\w+;base64,/,n.MSG_IE_BAD_FORMAT="Sorry IE does not support downloading from canvas. Try {format:'svg'} instead."},{"../registry":847}],852:[function(t,e,n){"use strict";var r=t("./helpers"),i={getDelay:r.getDelay,getRedrawFunc:r.getRedrawFunc,clone:t("./cloneplot"),toSVG:t("./tosvg"),svgToImg:t("./svgtoimg"),toImage:t("./toimage"),downloadImage:t("./download")};e.exports=i},{"./cloneplot":848,"./download":849,"./helpers":851,"./svgtoimg":853,"./toimage":854,"./tosvg":855}],853:[function(t,e,n){"use strict";var r=t("../lib"),i=t("events").EventEmitter,o=t("./helpers");e.exports=function(t){var e=t.emitter||new i,n=new Promise(function(i,a){var s=window.Image,A=t.svg,l=t.format||"png";if(r.isIE()&&"svg"!==l){var c=new Error(o.MSG_IE_BAD_FORMAT);return a(c),t.promise?n:e.emit("error",c)}var u,g,h=t.canvas,d=t.scale||1,f=t.width||300,p=t.height||150,I=d*f,C=d*p,m=h.getContext("2d"),y=new s;"svg"===l||r.isIE9orBelow()||r.isSafari()?g=o.encodeSVG(A):(u=o.createBlob(A,"svg"),g=o.createObjectURL(u)),h.width=I,h.height=C,y.onload=function(){var n;switch(u=null,o.revokeObjectURL(g),"svg"!==l&&m.drawImage(y,0,0,I,C),l){case"jpeg":n=h.toDataURL("image/jpeg");break;case"png":n=h.toDataURL("image/png");break;case"webp":n=h.toDataURL("image/webp");break;case"svg":n=g;break;default:var r="Image format is not jpeg, png, svg or webp.";if(a(new Error(r)),!t.promise)return e.emit("error",r)}i(n),t.promise||e.emit("success",n)},y.onerror=function(n){if(u=null,o.revokeObjectURL(g),a(n),!t.promise)return e.emit("error",n)},y.src=g});return t.promise?n:e}},{"../lib":719,"./helpers":851,events:104}],854:[function(t,e,n){"use strict";var r=t("events").EventEmitter,i=t("../registry"),o=t("../lib"),a=t("./helpers"),s=t("./cloneplot"),A=t("./tosvg"),l=t("./svgtoimg");e.exports=function(t,e){var n=new r,c=s(t,{format:"png"}),u=c.gd;u.style.position="absolute",u.style.left="-5000px",document.body.appendChild(u);var g=a.getRedrawFunc(u);return i.call("plot",u,c.data,c.layout,c.config).then(g).then(function(){var t=a.getDelay(u._fullLayout);setTimeout(function(){var t=A(u),r=document.createElement("canvas");r.id=o.randstr(),(n=l({format:e.format,width:u._fullLayout.width,height:u._fullLayout.height,canvas:r,emitter:n,svg:t})).clean=function(){u&&document.body.removeChild(u)}},t)}).catch(function(t){n.emit("error",t)}),n}},{"../lib":719,"../registry":847,"./cloneplot":848,"./helpers":851,"./svgtoimg":853,"./tosvg":855,events:104}],855:[function(t,e,n){"use strict";var r=t("d3"),i=t("../lib"),o=t("../components/drawing"),a=t("../components/color"),s=t("../constants/xmlns_namespaces"),A=/"/g,l=new RegExp('("TOBESTRIPPED)|(TOBESTRIPPED")',"g");e.exports=function(t,e,n){var c,u=t._fullLayout,g=u._paper,h=u._toppaper,d=u.width,f=u.height;g.insert("rect",":first-child").call(o.setRect,0,0,d,f).call(a.fill,u.paper_bgcolor);var p=u._basePlotModules||[];for(c=0;c<p.length;c++){var I=p[c];I.toSVG&&I.toSVG(t)}if(h){var C=h.node().childNodes,m=Array.prototype.slice.call(C);for(c=0;c<m.length;c++){var y=m[c];y.childNodes.length&&g.node().appendChild(y)}}u._draggers&&u._draggers.remove(),g.node().style.background="",g.selectAll("text").attr({"data-unformatted":null,"data-math":null}).each(function(){var t=r.select(this);if("hidden"!==this.style.visibility&&"none"!==this.style.display){t.style({visibility:null,display:null});var e=this.style.fontFamily;e&&-1!==e.indexOf('"')&&t.style("font-family",e.replace(A,"TOBESTRIPPED"))}else t.remove()}),g.selectAll(".point, .scatterpts, .legendfill>path, .legendlines>path, .cbfill").each(function(){var t=r.select(this),e=this.style.fill;e&&-1!==e.indexOf("url(")&&t.style("fill",e.replace(A,"TOBESTRIPPED"));var n=this.style.stroke;n&&-1!==n.indexOf("url(")&&t.style("stroke",n.replace(A,"TOBESTRIPPED"))}),"pdf"!==e&&"eps"!==e||g.selectAll("#MathJax_SVG_glyphs path").attr("stroke-width",0),g.node().setAttributeNS(s.xmlns,"xmlns",s.svg),g.node().setAttributeNS(s.xmlns,"xmlns:xlink",s.xlink),"svg"===e&&n&&(g.attr("width",n*d),g.attr("height",n*f),g.attr("viewBox","0 0 "+d+" "+f));var E=(new window.XMLSerializer).serializeToString(g.node());return E=(E=(E=function(t){var e=r.select("body").append("div").style({display:"none"}).html(""),n=t.replace(/(&[^;]*;)/gi,function(t){return"&lt;"===t?"&#60;":"&rt;"===t?"&#62;":-1!==t.indexOf("<")||-1!==t.indexOf(">")?"":e.html(t).text()});return e.remove(),n}(E)).replace(/&(?!\w+;|\#[0-9]+;| \#x[0-9A-F]+;)/g,"&amp;")).replace(l,"'"),i.isIE()&&(E=(E=(E=E.replace(/"/gi,"'")).replace(/(\('#)([^']*)('\))/gi,'("#$2")')).replace(/(\\')/gi,'"')),E}},{"../components/color":593,"../components/drawing":614,"../constants/xmlns_namespaces":696,"../lib":719,d3:163}],856:[function(t,e,n){"use strict";var r=t("../../lib");e.exports=function(t,e){for(var n=0;n<t.length;n++)t[n].i=n;r.mergeArray(e.text,t,"tx"),r.mergeArray(e.hovertext,t,"htx");var i=e.marker;if(i){r.mergeArray(i.opacity,t,"mo",!0),r.mergeArray(i.color,t,"mc");var o=i.line;o&&(r.mergeArray(o.color,t,"mlc"),r.mergeArrayCastPositive(o.width,t,"mlw"))}}},{"../../lib":719}],857:[function(t,e,n){"use strict";var r=t("../scatter/attributes"),i=t("../../components/fx/hovertemplate_attributes"),o=t("../../components/colorscale/attributes"),a=t("../../plots/font_attributes"),s=t("./constants.js"),A=t("../../lib/extend").extendFlat,l=a({editType:"calc",arrayOk:!0,colorEditType:"style"}),c=A({},r.marker.line.width,{dflt:0}),u=A({width:c,editType:"calc"},o("marker.line")),g=A({line:u,editType:"calc"},o("marker"),{opacity:{valType:"number",arrayOk:!0,dflt:1,min:0,max:1,editType:"style"}});e.exports={x:r.x,x0:r.x0,dx:r.dx,y:r.y,y0:r.y0,dy:r.dy,text:r.text,hovertext:r.hovertext,hovertemplate:i({},{keys:s.eventDataKeys}),textposition:{valType:"enumerated",values:["inside","outside","auto","none"],dflt:"none",arrayOk:!0,editType:"calc"},insidetextanchor:{valType:"enumerated",values:["end","middle","start"],dflt:"end",editType:"plot"},textangle:{valType:"angle",dflt:"auto",editType:"plot"},textfont:A({},l,{}),insidetextfont:A({},l,{}),outsidetextfont:A({},l,{}),constraintext:{valType:"enumerated",values:["inside","outside","both","none"],dflt:"both",editType:"calc"},cliponaxis:A({},r.cliponaxis,{}),orientation:{valType:"enumerated",values:["v","h"],editType:"calc+clearAxisTypes"},base:{valType:"any",dflt:null,arrayOk:!0,editType:"calc"},offset:{valType:"number",dflt:null,arrayOk:!0,editType:"calc"},width:{valType:"number",dflt:null,min:0,arrayOk:!0,editType:"calc"},marker:g,offsetgroup:{valType:"string",dflt:"",editType:"calc"},alignmentgroup:{valType:"string",dflt:"",editType:"calc"},selected:{marker:{opacity:r.selected.marker.opacity,color:r.selected.marker.color,editType:"style"},textfont:r.selected.textfont,editType:"style"},unselected:{marker:{opacity:r.unselected.marker.opacity,color:r.unselected.marker.color,editType:"style"},textfont:r.unselected.textfont,editType:"style"},r:r.r,t:r.t,_deprecated:{bardir:{valType:"enumerated",editType:"calc",values:["v","h"]}}}},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/font_attributes":793,"../scatter/attributes":1112,"./constants.js":859}],858:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axes"),i=t("../../components/colorscale/helpers").hasColorscale,o=t("../../components/colorscale/calc"),a=t("./arrays_to_calcdata"),s=t("../scatter/calc_selection");e.exports=function(t,e){var n,A,l=r.getFromId(t,e.xaxis||"x"),c=r.getFromId(t,e.yaxis||"y");"h"===e.orientation?(n=l.makeCalcdata(e,"x"),A=c.makeCalcdata(e,"y")):(n=c.makeCalcdata(e,"y"),A=l.makeCalcdata(e,"x"));for(var u=Math.min(A.length,n.length),g=new Array(u),h=0;h<u;h++)g[h]={p:A[h],s:n[h]},e.ids&&(g[h].id=String(e.ids[h]));return i(e,"marker")&&o(t,e,{vals:e.marker.color,containerStr:"marker",cLetter:"c"}),i(e,"marker.line")&&o(t,e,{vals:e.marker.line.color,containerStr:"marker.line",cLetter:"c"}),a(g,e),s(g,e),g}},{"../../components/colorscale/calc":601,"../../components/colorscale/helpers":604,"../../plots/cartesian/axes":767,"../scatter/calc_selection":1114,"./arrays_to_calcdata":856}],859:[function(t,e,n){"use strict";e.exports={eventDataKeys:[]}},{}],860:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib").isArrayOrTypedArray,o=t("../../constants/numerical").BADNUM,a=t("../../registry"),s=t("../../plots/cartesian/axes"),A=t("../../plots/cartesian/axis_ids").getAxisGroup,l=t("./sieve.js");function c(t,e,n,a,c){if(a.length){var y,E,v,B;switch(function(t,e){var n,o;for(n=0;n<e.length;n++){var a,s=e[n],A=s[0].trace,l="funnel"===A.type?A._base:A.base,c="h"===A.orientation?A.xcalendar:A.ycalendar,u="category"===t.type||"multicategory"===t.type?function(){return null}:t.d2c;if(i(l)){for(o=0;o<Math.min(l.length,s.length);o++)a=u(l[o],0,c),r(a)?(s[o].b=+a,s[o].hasB=1):s[o].b=0;for(;o<s.length;o++)s[o].b=0}else{a=u(l,0,c);var g=r(a);for(a=g?a:0,o=0;o<s.length;o++)s[o].b=a,g&&(s[o].hasB=1)}}}(n,a),c.mode){case"overlay":u(e,n,a,c);break;case"group":for(y=[],E=[],v=0;v<a.length;v++)void 0===(B=a[v])[0].trace.offset?E.push(B):y.push(B);E.length&&function(t,e,n,r,i){var a=new l(r,{sepNegVal:!1,overlapNoMerge:!i.norm});(function(t,e,n,r){for(var i=t._fullLayout,o=n.positions,a=n.distinctPositions,s=n.minDiff,l=n.traces,c=l.length,u=o.length!==a.length,g=s*(1-r.gap),p=A(i,e._id)+l[0][0].trace.orientation,I=i._alignmentOpts[p]||{},C=0;C<c;C++){var m,y,E=l[C],v=E[0].trace,B=I[v.alignmentgroup]||{},b=Object.keys(B.offsetGroups||{}).length,Q=(m=b?g/b:u?g/c:g)*(1-(r.groupgap||0));y=b?((2*v._offsetIndex+1-b)*m-Q)/2:u?((2*C+1-c)*m-Q)/2:-Q/2;var w=E[0].t;w.barwidth=Q,w.poffset=y,w.bargroupwidth=g,w.bardelta=s}n.binWidth=l[0][0].t.barwidth/100,h(n),d(e,n),f(e,n,u)})(t,e,a,i),function(t){for(var e=t.traces,n=0;n<e.length;n++){var r=e[n],i=r[0].trace;if(void 0===i.base)for(var a=new l([r],{sepNegVal:!0,overlapNoMerge:!0}),s=0;s<r.length;s++){var A=r[s];if(A.p!==o){var c=a.put(A.p,A.b+A.s);c&&(A.b=c)}}}}(a),i.norm?(I(a),C(n,a,i)):p(n,a)}(t,e,n,E,c),y.length&&u(e,n,y,c);break;case"stack":case"relative":for(y=[],E=[],v=0;v<a.length;v++)void 0===(B=a[v])[0].trace.base?E.push(B):y.push(B);E.length&&function(t,e,n,r,i){var a=new l(r,{sepNegVal:"relative"===i.mode,overlapNoMerge:!(i.norm||"stack"===i.mode||"relative"===i.mode)});g(e,a,i),function(t,e,n){var r,i,a,A,l,c,u=m(t),g=e.traces;for(A=0;A<g.length;A++)if(r=g[A],"funnel"===(i=r[0].trace).type)for(l=0;l<r.length;l++)(c=r[l]).s!==o&&e.put(c.p,-.5*c.s);for(A=0;A<g.length;A++){r=g[A],i=r[0].trace,a="funnel"===i.type;var h=[];for(l=0;l<r.length;l++)if((c=r[l]).s!==o){var d;d=a?c.s:c.s+c.b;var f=e.put(c.p,d),p=f+d;c.b=f,c[u]=p,n.norm||(h.push(p),c.hasB&&h.push(f))}n.norm||(i._extremes[t._id]=s.findExtremes(t,h,{tozero:!0,padded:!0}))}}(n,a,i);for(var A=0;A<r.length;A++)for(var c=r[A],u=0;u<c.length;u++){var h=c[u];if(h.s!==o){var d=h.b+h.s===a.get(h.p,h.s);d&&(h._outmost=!0)}}i.norm&&C(n,a,i)}(0,e,n,E,c),y.length&&u(e,n,y,c)}!function(t,e){var n,i,o,a=m(e),s={},A=1/0,l=-1/0;for(n=0;n<t.length;n++)for(o=t[n],i=0;i<o.length;i++){var c=o[i].p;r(c)&&(A=Math.min(A,c),l=Math.max(l,c))}var u=1e4/(l-A),g=s.round=function(t){return String(Math.round(u*(t-A)))};for(n=0;n<t.length;n++){(o=t[n])[0].t.extents=s;var h=o[0].t.poffset,d=Array.isArray(h);for(i=0;i<o.length;i++){var f=o[i],p=f[a]-f.w/2;if(r(p)){var I=f[a]+f.w/2,C=g(f.p);s[C]?s[C]=[Math.min(p,s[C][0]),Math.max(I,s[C][1])]:s[C]=[p,I]}f.p0=f.p+(d?h[i]:h),f.p1=f.p0+f.w,f.s0=f.b,f.s1=f.s0+f.s}}}(a,e)}}function u(t,e,n,r){for(var i=0;i<n.length;i++){var o=n[i],a=new l([o],{sepNegVal:!1,overlapNoMerge:!r.norm});g(t,a,r),r.norm?(I(a),C(e,a,r)):p(e,a)}}function g(t,e,n){for(var r=e.minDiff,i=e.traces,o=r*(1-n.gap),a=o*(1-(n.groupgap||0)),s=-a/2,A=0;A<i.length;A++){var l=i[A][0].t;l.barwidth=a,l.poffset=s,l.bargroupwidth=o,l.bardelta=r}e.binWidth=i[0][0].t.barwidth/100,h(e),d(t,e),f(t,e)}function h(t){var e,n,o=t.traces;for(e=0;e<o.length;e++){var a,s=o[e],A=s[0],l=A.trace,c=A.t,u=l._offset||l.offset,g=c.poffset;if(i(u)){for(a=Array.prototype.slice.call(u,0,s.length),n=0;n<a.length;n++)r(a[n])||(a[n]=g);for(n=a.length;n<s.length;n++)a.push(g);c.poffset=a}else void 0!==u&&(c.poffset=u);var h=l._width||l.width,d=c.barwidth;if(i(h)){var f=Array.prototype.slice.call(h,0,s.length);for(n=0;n<f.length;n++)r(f[n])||(f[n]=d);for(n=f.length;n<s.length;n++)f.push(d);if(c.barwidth=f,void 0===u){for(a=[],n=0;n<s.length;n++)a.push(g+(d-f[n])/2);c.poffset=a}}else void 0!==h&&(c.barwidth=h,void 0===u&&(c.poffset=g+(d-h)/2))}}function d(t,e){for(var n=e.traces,r=m(t),i=0;i<n.length;i++)for(var o=n[i],a=o[0].t,s=a.poffset,A=Array.isArray(s),l=a.barwidth,c=Array.isArray(l),u=0;u<o.length;u++){var g=o[u],h=g.w=c?l[u]:l;g[r]=g.p+(A?s[u]:s)+h/2}}function f(t,e,n){var r=e.traces,i=e.minDiff/2;s.minDtick(t,e.minDiff,e.distinctPositions[0],n);for(var o=0;o<r.length;o++){var a,A,l,c,u=r[o],g=u[0],h=g.trace,d=[];for(c=0;c<u.length;c++)A=(a=u[c]).p-i,l=a.p+i,d.push(A,l);if(h.width||h.offset){var f=g.t,p=f.poffset,I=f.barwidth,C=Array.isArray(p),m=Array.isArray(I);for(c=0;c<u.length;c++){a=u[c];var y=C?p[c]:p,E=m?I[c]:I;l=(A=a.p+y)+E,d.push(A,l)}}h._extremes[t._id]=s.findExtremes(t,d,{padded:!1})}}function p(t,e){for(var n=e.traces,r=m(t),i=0;i<n.length;i++){for(var o=n[i],a=o[0].trace,A=[],l=!0,c=0;c<o.length;c++){var u=o[c],g=u.b,h=g+u.s;u[r]=h,A.push(h),u.hasB&&A.push(g),u.hasB&&u.b>0&&u.s>0||(l=!1)}a._extremes[t._id]=s.findExtremes(t,A,{tozero:!l,padded:!0})}}function I(t){for(var e=t.traces,n=0;n<e.length;n++)for(var r=e[n],i=0;i<r.length;i++){var a=r[i];a.s!==o&&t.put(a.p,a.b+a.s)}}function C(t,e,n){var i=e.traces,a=m(t),A="fraction"===n.norm?1:100,l=A/1e9,c=t.l2c(t.c2l(0)),u="stack"===n.mode?A:c;function g(e){return r(t.c2l(e))&&(e<c-l||e>u+l||!r(c))}for(var h=0;h<i.length;h++){for(var d=i[h],f=d[0].trace,p=[],I=!0,C=!1,y=0;y<d.length;y++){var E=d[y];if(E.s!==o){var v=Math.abs(A/e.get(E.p,E.s));E.b*=v,E.s*=v;var B=E.b,b=B+E.s;E[a]=b,p.push(b),C=C||g(b),E.hasB&&(p.push(B),C=C||g(B)),E.hasB&&E.b>0&&E.s>0||(I=!1)}}f._extremes[t._id]=s.findExtremes(t,p,{tozero:!I,padded:C})}}function m(t){return t._id.charAt(0)}e.exports={crossTraceCalc:function(t,e){for(var n=e.xaxis,r=e.yaxis,i=t._fullLayout,o=t._fullData,s=t.calcdata,A=[],l=[],u=0;u<o.length;u++){var g=o[u];!0===g.visible&&a.traceIs(g,"bar")&&g.xaxis===n._id&&g.yaxis===r._id&&("h"===g.orientation?A.push(s[u]):l.push(s[u]))}var h={mode:i.barmode,norm:i.barnorm,gap:i.bargap,groupgap:i.bargroupgap};c(t,n,r,l,h),c(t,r,n,A,h)},setGroupPositions:c}},{"../../constants/numerical":695,"../../lib":719,"../../plots/cartesian/axes":767,"../../plots/cartesian/axis_ids":770,"../../registry":847,"./sieve.js":869,"fast-isnumeric":225}],861:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/color"),o=t("../../registry"),a=t("../scatter/xy_defaults"),s=t("./style_defaults"),A=t("../../plots/cartesian/axis_ids").getAxisGroup,l=t("./attributes"),c=r.coerceFont;function u(t,e,n,r){var i=e.orientation,o=e[{v:"x",h:"y"}[i]+"axis"],a=A(n,o)+i,s=n._alignmentOpts||{},l=r("alignmentgroup"),c=s[a];c||(c=s[a]={});var u=c[l];u?u.traces.push(e):u=c[l]={traces:[e],alignmentIndex:Object.keys(c).length,offsetGroups:{}};var g=r("offsetgroup"),h=u.offsetGroups,d=h[g];g&&(d||(d=h[g]={offsetIndex:Object.keys(h).length}),e._offsetIndex=d.offsetIndex)}function g(t,e,n,i,o,a){var s=!(!1===(a=a||{}).moduleHasSelected),A=!(!1===a.moduleHasUnselected),l=!(!1===a.moduleHasConstrain),u=!(!1===a.moduleHasCliponaxis),g=!(!1===a.moduleHasTextangle),h=!(!1===a.moduleHasInsideanchor),d=Array.isArray(o)||"auto"===o,f=d||"inside"===o,p=d||"outside"===o;if(f||p){var I=c(i,"textfont",n.font),C=r.extendFlat({},I);!(t.textfont&&t.textfont.color)&&delete C.color,c(i,"insidetextfont",C),p&&c(i,"outsidetextfont",I),s&&i("selected.textfont.color"),A&&i("unselected.textfont.color"),l&&i("constraintext"),u&&i("cliponaxis"),g&&i("textangle")}f&&h&&i("insidetextanchor")}e.exports={supplyDefaults:function(t,e,n,A){function c(n,i){return r.coerce(t,e,l,n,i)}if(a(t,e,A,c)){c("orientation",e.x&&!e.y?"h":"v"),c("base"),c("offset"),c("width"),c("text"),c("hovertext"),c("hovertemplate");var u=c("textposition");g(t,0,A,c,u,{moduleHasSelected:!0,moduleHasUnselected:!0,moduleHasConstrain:!0,moduleHasCliponaxis:!0,moduleHasTextangle:!0,moduleHasInsideanchor:!0}),s(t,e,c,n,A);var h=(e.marker.line||{}).color,d=o.getComponentMethod("errorbars","supplyDefaults");d(t,e,h||i.defaultLine,{axis:"y"}),d(t,e,h||i.defaultLine,{axis:"x",inherit:"y"}),r.coerceSelectionMarkerOpacity(e,c)}else e.visible=!1},crossTraceDefaults:function(t,e){var n;function i(t){return r.coerce(n._input,n,l,t)}if("group"===e.barmode)for(var o=0;o<t.length;o++)"bar"===(n=t[o]).type&&(n._input,u(0,n,e,i))},handleGroupingDefaults:u,handleText:g}},{"../../components/color":593,"../../lib":719,"../../plots/cartesian/axis_ids":770,"../../registry":847,"../scatter/xy_defaults":1137,"./attributes":857,"./style_defaults":871}],862:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("tinycolor2"),o=t("../../lib").isArrayOrTypedArray;n.coerceString=function(t,e,n){if("string"==typeof e){if(e||!t.noBlank)return e}else if(("number"==typeof e||!0===e)&&!t.strict)return String(e);return void 0!==n?n:t.dflt},n.coerceNumber=function(t,e,n){if(r(e)){e=+e;var i=t.min,o=t.max;if(!(void 0!==i&&e<i||void 0!==o&&e>o))return e}return void 0!==n?n:t.dflt},n.coerceColor=function(t,e,n){return i(e).isValid()?e:void 0!==n?n:t.dflt},n.coerceEnumerated=function(t,e,n){return t.coerceNumber&&(e=+e),-1!==t.values.indexOf(e)?e:void 0!==n?n:t.dflt},n.getValue=function(t,e){var n;return Array.isArray(t)?e<t.length&&(n=t[e]):n=t,n},n.getLineWidth=function(t,e){return 0<e.mlw?e.mlw:o(t.marker.line.width)?0:t.marker.line.width}},{"../../lib":719,"fast-isnumeric":225,tinycolor2:537}],863:[function(t,e,n){"use strict";var r=t("../../components/fx"),i=t("../../registry"),o=t("../../components/color"),a=t("../../lib").fillText,s=t("./helpers").getLineWidth;function A(t,e,n,i){var o,s,A,l,c,u,g,h=t.cd,d=h[0].trace,f=h[0].t,p="closest"===i,I="waterfall"===d.type,C=t.maxHoverDistance,m=t.maxSpikeDistance;function y(t){return t[A]-t.w/2}function E(t){return t[A]+t.w/2}var v=p?y:function(t){return Math.min(y(t),t.p-f.bardelta/2)},B=p?E:function(t){return Math.max(E(t),t.p+f.bardelta/2)};function b(t,e){return r.inbox(t-o,e-o,C+Math.min(1,Math.abs(e-t)/g)-1)}function Q(t){return b(v(t),B(t))}function w(t){var e=s,n=t.b,i=t[l];return I&&(i+=Math.abs(t.rawS||0)),r.inbox(n-e,i-e,C+(i-e)/(i-n)-1)}"h"===d.orientation?(o=n,s=e,A="y",l="x",c=w,u=Q):(o=e,s=n,A="x",l="y",u=w,c=Q);var x=t[A+"a"],_=t[l+"a"];g=Math.abs(x.r2c(x.range[1])-x.r2c(x.range[0]));var D=r.getDistanceFunction(i,c,u,function(t){return(c(t)+u(t))/2});if(r.getClosest(h,D,t),!1!==t.index){p||(v=function(t){return Math.min(y(t),t.p-f.bargroupwidth/2)},B=function(t){return Math.max(E(t),t.p+f.bargroupwidth/2)});var M=h[t.index],k=d.base?M.b+M.s:M.s;t[l+"0"]=t[l+"1"]=_.c2p(M[l],!0),t[l+"LabelVal"]=k;var S=f.extents[f.extents.round(M.p)];return t[A+"0"]=x.c2p(p?v(M):S[0],!0),t[A+"1"]=x.c2p(p?B(M):S[1],!0),t[A+"LabelVal"]=M.p,t.spikeDistance=(w(M)+function(t){return b(y(t),E(t))}(M))/2+m-C,t[A+"Spike"]=x.c2p(M.p,!0),a(M,d,t),t.hovertemplate=d.hovertemplate,t}}function l(t,e){var n=e.mcc||t.marker.color,r=e.mlcc||t.marker.line.color,i=s(t,e);return o.opacity(n)?n:o.opacity(r)&&i?r:void 0}e.exports={hoverPoints:function(t,e,n,r){var o=A(t,e,n,r);if(o){var a=o.cd,s=a[0].trace,c=a[o.index];return o.color=l(s,c),i.getComponentMethod("errorbars","hoverInfo")(c,s,o),[o]}},hoverOnBars:A,getTraceColor:l}},{"../../components/color":593,"../../components/fx":632,"../../lib":719,"../../registry":847,"./helpers":862}],864:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),layoutAttributes:t("./layout_attributes"),supplyDefaults:t("./defaults").supplyDefaults,crossTraceDefaults:t("./defaults").crossTraceDefaults,supplyLayoutDefaults:t("./layout_defaults"),calc:t("./calc"),crossTraceCalc:t("./cross_trace_calc").crossTraceCalc,colorbar:t("../scatter/marker_colorbar"),arraysToCalcdata:t("./arrays_to_calcdata"),plot:t("./plot").plot,style:t("./style").style,styleOnSelect:t("./style").styleOnSelect,hoverPoints:t("./hover").hoverPoints,selectPoints:t("./select"),moduleType:"trace",name:"bar",basePlotModule:t("../../plots/cartesian"),categories:["bar-like","cartesian","svg","bar","oriented","errorBarsOK","showLegend","zoomScale"],meta:{}}},{"../../plots/cartesian":778,"../scatter/marker_colorbar":1129,"./arrays_to_calcdata":856,"./attributes":857,"./calc":858,"./cross_trace_calc":860,"./defaults":861,"./hover":863,"./layout_attributes":865,"./layout_defaults":866,"./plot":867,"./select":868,"./style":870}],865:[function(t,e,n){"use strict";e.exports={barmode:{valType:"enumerated",values:["stack","group","overlay","relative"],dflt:"group",editType:"calc"},barnorm:{valType:"enumerated",values:["","fraction","percent"],dflt:"",editType:"calc"},bargap:{valType:"number",min:0,max:1,editType:"calc"},bargroupgap:{valType:"number",min:0,max:1,dflt:0,editType:"calc"}}},{}],866:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../plots/cartesian/axes"),o=t("../../lib"),a=t("./layout_attributes");e.exports=function(t,e,n){function s(n,r){return o.coerce(t,e,a,n,r)}for(var A=!1,l=!1,c=!1,u={},g=s("barmode"),h=0;h<n.length;h++){var d=n[h];if(r.traceIs(d,"bar")&&d.visible){if(A=!0,"group"===g){var f=d.xaxis+d.yaxis;u[f]&&(c=!0),u[f]=!0}d.visible&&"histogram"===d.type&&"category"!==i.getFromId({_fullLayout:e},d["v"===d.orientation?"xaxis":"yaxis"]).type&&(l=!0)}}A?("overlay"!==g&&s("barnorm"),s("bargap",l&&!c?0:.2),s("bargroupgap")):delete e.barmode}},{"../../lib":719,"../../plots/cartesian/axes":767,"../../registry":847,"./layout_attributes":865}],867:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=t("../../lib"),a=t("../../lib/svg_text_utils"),s=t("../../components/color"),A=t("../../components/drawing"),l=t("../../registry"),c=t("../../plots/cartesian/axes").tickText,u=t("./style"),g=t("./helpers"),h=t("./attributes"),d=h.text,f=h.textposition,p=3;function I(t,e){return t<e?1:-1}function C(t){return"auto"===t?0:t}function m(t,e,n,r,i,o){var a=!!o.isHorizontal,s=!!o.constrained,A=o.angle||0,l=o.anchor||0,c=i.width,u=i.height,g=Math.abs(e-t),h=Math.abs(r-n),d=g>2*p&&h>2*p?p:0;g-=2*d,h-=2*d;var f=!1;if(!("auto"===A)||c<=g&&u<=h||!(c>g||u>h)||(c>h||u>g)&&c<u==g<h||(f=!0),f){var m=h;h=g,g=m}var y=C(A),E=Math.abs(Math.sin(Math.PI/180*y)),v=Math.abs(Math.cos(Math.PI/180*y)),B=Math.max(g*v,h*E),b=Math.max(g*E,h*v),Q=s?Math.min(B/c,b/u):Math.max(v,E);Q=Math.min(1,Q);var w=(t+e)/2,x=(n+r)/2;return"middle"!==l&&(d+=.5*(Q*(a!==f?u:c)*E+Q*(a!==f?c:u)*v),a?(d*=I(t,e),w="start"===l?t+d:e-d):(d*=I(n,r),x="start"===l?n+d:r-d)),f&&(y+=90),{textX:(i.left+i.right)/2,textY:(i.top+i.bottom)/2,targetX:w,targetY:x,scale:Q,rotate:y}}function y(t,e,n,r,i,o){var a,s=!!o.isHorizontal,A=!!o.constrained,l=o.angle||0,c=i.width,u=i.height,g=Math.abs(e-t),h=Math.abs(r-n);a=s?h>2*p?p:0:g>2*p?p:0;var d=1;A&&(d=s?Math.min(1,h/u):Math.min(1,g/c));var f=C(l);a+=.5*(d*(s?u:c)*Math.abs(Math.sin(Math.PI/180*f))+d*(s?c:u)*Math.abs(Math.cos(Math.PI/180*f)));var m=(t+e)/2,y=(n+r)/2;return s?m=e-a*I(e,t):y=r+a*I(n,r),{textX:(i.left+i.right)/2,textY:(i.top+i.bottom)/2,targetX:m,targetY:y,scale:d,rotate:f}}function E(t){var e,n=t.textX,r=t.textY,i=t.targetX,o=t.targetY,a=t.scale,s=t.rotate;return a<1?e="scale("+a+") ":(a=1,e=""),"translate("+(i-a*n)+" "+(o-a*r)+")"+e+(s?"rotate("+s+" "+n+" "+r+") ":"")}e.exports={plot:function(t,e,n,h,C){var v=e.xaxis,B=e.yaxis,b=t._fullLayout;C||(C={mode:b.barmode,norm:b.barmode,gap:b.bargap,groupgap:b.bargroupgap});var Q=o.makeTraceGroups(h,n,"trace bars").each(function(n){var l=r.select(this),h=n[0].trace,b="waterfall"===h.type,Q="funnel"===h.type,w="bar"===h.type||Q,x=0;b&&h.connector.visible&&"between"===h.connector.mode&&(x=h.connector.line.width/2);var _="h"===h.orientation,D=o.ensureSingle(l,"g","points").selectAll("g.point").data(o.identity);D.enter().append("g").classed("point",!0),D.exit().remove(),D.each(function(l,b){var Q,D,M=r.select(this),k=function(t,e,n,r){var i=[],o=[],a=r?e:n,s=r?n:e;return i[0]=a.c2p(t.s0,!0),o[0]=s.c2p(t.p0,!0),i[1]=a.c2p(t.s1,!0),o[1]=s.c2p(t.p1,!0),r?[i,o]:[o,i]}(l,v,B,_),S=k[0][0],L=k[0][1],R=k[1][0],N=k[1][1],F=!(S!==L&&R!==N&&i(S)&&i(L)&&i(R)&&i(N));if(F&&w&&g.getLineWidth(h,l)&&(_?L-S==0:N-R==0)&&(F=!1),l.isBlank=F,x&&(_?(S-=I(S,L)*x,L+=I(S,L)*x):(R-=I(R,N)*x,N+=I(R,N)*x)),"waterfall"===h.type){if(!F){var T=h[l.dir].marker;Q=T.line.width,D=T.color}}else Q=g.getLineWidth(h,l),D=l.mc||h.marker.color;var U=r.round(Q/2%1,2);function O(t){return 0===C.gap&&0===C.groupgap?r.round(Math.round(t)-U,2):t}if(!t._context.staticPlot){var G=s.opacity(D)<1||Q>.01?O:function(t,e){return Math.abs(t-e)>=2?O(t):t>e?Math.ceil(t):Math.floor(t)};S=G(S,L),L=G(L,S),R=G(R,N),N=G(N,R)}o.ensureSingle(M,"path").style("vector-effect","non-scaling-stroke").attr("d",F?"M0,0Z":"M"+S+","+R+"V"+N+"H"+L+"V"+R+"Z").call(A.setClipUrl,e.layerClipId,t),function(t,e,n,r,i,s,l,h,I,C){var v,B=e.xaxis,b=e.yaxis,Q=t._fullLayout;function w(e,n,r){var i=o.ensureSingle(e,"text").text(n).attr({class:"bartext bartext-"+v,transform:"","text-anchor":"middle","data-notex":1}).call(A.font,r).call(a.convertToTspans,t);return i}var x=r[0].trace,_="h"===x.orientation,D=function(t,e,n,r){var i,a=t[0].trace;return i=a.textinfo?function(t,e,n,r){var i=t[0].trace,a="h"===i.orientation,s="waterfall"===i.type,A="funnel"===i.type;function l(t){var e=a?n:r;return c(e,+t,!0).text}var u,g,h=i.textinfo,d=t[e],f=h.split("+"),p=[],I=function(t){return-1!==f.indexOf(t)};if(I("label")&&p.push((g=t[e].p,c(a?r:n,g,!0).text)),I("text")&&(0===(u=o.castOption(i,d.i,"text"))||u)&&p.push(u),s){var C=+d.rawS||d.s,m=d.v,y=m-C;I("initial")&&p.push(l(y)),I("delta")&&p.push(l(C)),I("final")&&p.push(l(m))}if(A){I("value")&&p.push(l(d.s));var E=0;I("percent initial")&&E++,I("percent previous")&&E++,I("percent total")&&E++;var v=E>1;I("percent initial")&&(u=o.formatPercent(d.begR),v&&(u+=" of initial"),p.push(u)),I("percent previous")&&(u=o.formatPercent(d.difR),v&&(u+=" of previous"),p.push(u)),I("percent total")&&(u=o.formatPercent(d.sumR),v&&(u+=" of total"),p.push(u))}return p.join("<br>")}(t,e,n,r):g.getValue(a.text,e),g.coerceString(d,i)}(r,i,B,b);v=function(t,e){var n=g.getValue(t.textposition,e);return g.coerceEnumerated(f,n)}(x,i);var M="stack"===C.mode||"relative"===C.mode,k=r[i],S=!M||k._outmost;if(D&&"none"!==v&&(!k.isBlank&&s!==l&&h!==I||"auto"!==v&&"inside"!==v)){var L=Q.font,R=u.getBarColor(r[i],x),N=u.getInsideTextFont(x,i,L,R),F=u.getOutsideTextFont(x,i,L),T=n.datum();_?"log"===B.type&&T.s0<=0&&(s=B.range[0]<B.range[1]?0:B._length):"log"===b.type&&T.s0<=0&&(h=b.range[0]<b.range[1]?b._length:0);var U,O,G,H,j,Y,P=Math.abs(l-s)-2*p,K=Math.abs(I-h)-2*p;if("outside"===v&&(S||k.hasB||(v="inside")),"auto"===v)if(S){v="inside",U=w(n,D,N),O=A.bBox(U.node()),G=O.width,H=O.height;var z=G>0&&H>0,J=G<=P&&H<=K,q=G<=K&&H<=P,V=_?P>=G*(K/H):K>=H*(P/G);z&&(J||q||V)?v="inside":(v="outside",U.remove(),U=null)}else v="inside";!U&&(U=w(n,D,"outside"===v?F:N),O=A.bBox(U.node()),G=O.width,H=O.height,G<=0||H<=0)?U.remove():("outside"===v?(Y="both"===x.constraintext||"outside"===x.constraintext,j=E(y(s,l,h,I,O,{isHorizontal:_,constrained:Y,angle:x.textangle}))):(Y="both"===x.constraintext||"inside"===x.constraintext,j=E(m(s,l,h,I,O,{isHorizontal:_,constrained:Y,angle:x.textangle,anchor:x.insidetextanchor}))),U.attr("transform",j))}else n.select("text").remove()}(t,e,M,n,b,S,L,R,N,C),e.layerClipId&&A.hideOutsideRangePoint(l,M.select("text"),v,B,h.xcalendar,h.ycalendar)});var M=!1===h.cliponaxis;A.setClipUrl(l,M?null:e.layerClipId,t)});l.getComponentMethod("errorbars","plot")(t,Q,e)},getTransform:E,toMoveInsideBar:m,toMoveOutsideBar:y}},{"../../components/color":593,"../../components/drawing":614,"../../lib":719,"../../lib/svg_text_utils":743,"../../plots/cartesian/axes":767,"../../registry":847,"./attributes":857,"./helpers":862,"./style":870,d3:163,"fast-isnumeric":225}],868:[function(t,e,n){"use strict";function r(t,e,n,r,i){var o=e.c2p(r?t.s0:t.p0,!0),a=e.c2p(r?t.s1:t.p1,!0),s=n.c2p(r?t.p0:t.s0,!0),A=n.c2p(r?t.p1:t.s1,!0);return i?[(o+a)/2,(s+A)/2]:r?[a,(s+A)/2]:[(o+a)/2,A]}e.exports=function(t,e){var n,i=t.cd,o=t.xaxis,a=t.yaxis,s=i[0].trace,A="funnel"===s.type,l="h"===s.orientation,c=[];if(!1===e)for(n=0;n<i.length;n++)i[n].selected=0;else for(n=0;n<i.length;n++){var u=i[n],g="ct"in u?u.ct:r(u,o,a,l,A);e.contains(g,!1,n,t)?(c.push({pointNumber:n,x:o.c2d(u.x),y:a.c2d(u.y)}),u.selected=1):u.selected=0}return c}},{}],869:[function(t,e,n){"use strict";e.exports=o;var r=t("../../lib").distinctVals,i=t("../../constants/numerical").BADNUM;function o(t,e){this.traces=t,this.sepNegVal=e.sepNegVal,this.overlapNoMerge=e.overlapNoMerge;for(var n=1/0,o=[],a=0;a<t.length;a++){for(var s=t[a],A=0;A<s.length;A++){var l=s[A];l.p!==i&&o.push(l.p)}s[0]&&s[0].width1&&(n=Math.min(s[0].width1,n))}this.positions=o;var c=r(o);this.distinctPositions=c.vals,1===c.vals.length&&n!==1/0?this.minDiff=n:this.minDiff=Math.min(c.minDiff,n),this.binWidth=this.minDiff,this.bins={}}o.prototype.put=function(t,e){var n=this.getLabel(t,e),r=this.bins[n]||0;return this.bins[n]=r+e,r},o.prototype.get=function(t,e){var n=this.getLabel(t,e);return this.bins[n]||0},o.prototype.getLabel=function(t,e){return(e<0&&this.sepNegVal?"v":"^")+(this.overlapNoMerge?t:Math.round(t/this.binWidth))}},{"../../constants/numerical":695,"../../lib":719}],870:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/color"),o=t("../../components/drawing"),a=t("../../lib"),s=t("../../registry"),A=t("./attributes"),l=A.textfont,c=A.insidetextfont,u=A.outsidetextfont,g=t("./helpers");function h(t,e,n){o.pointStyle(t.selectAll("path"),e,n),d(t,e,n)}function d(t,e,n){t.selectAll("text").each(function(t){var i=r.select(this),a=f(i,t,e,n);o.font(i,a)})}function f(t,e,n,r){var i=r._fullLayout.font,o=n.textfont;if(t.classed("bartext-inside")){var a=y(e,n);o=I(n,e.i,i,a)}else t.classed("bartext-outside")&&(o=C(n,e.i,i));return o}function p(t,e,n){return m(l,t.textfont,e,n)}function I(t,e,n,r){var o=p(t,e,n);return(void 0===t._input.textfont||void 0===t._input.textfont.color||Array.isArray(t.textfont.color)&&void 0===t.textfont.color[e])&&(o={color:i.contrast(r),family:o.family,size:o.size}),m(c,t.insidetextfont,e,o)}function C(t,e,n){var r=p(t,e,n);return m(u,t.outsidetextfont,e,r)}function m(t,e,n,r){e=e||{};var i=g.getValue(e.family,n),o=g.getValue(e.size,n),a=g.getValue(e.color,n);return{family:g.coerceString(t.family,i,r.family),size:g.coerceNumber(t.size,o,r.size),color:g.coerceColor(t.color,a,r.color)}}function y(t,e){return"waterfall"===e.type?e[t.dir].marker.color:t.mc||e.marker.color}e.exports={style:function(t){var e=r.select(t).selectAll("g.barlayer").selectAll("g.trace"),n=e.size(),i=t._fullLayout;e.style("opacity",function(t){return t[0].trace.opacity}).each(function(t){("stack"===i.barmode&&n>1||0===i.bargap&&0===i.bargroupgap&&!t[0].trace.marker.line.width)&&r.select(this).attr("shape-rendering","crispEdges")}),e.selectAll("g.points").each(function(e){h(r.select(this),e[0].trace,t)}),s.getComponentMethod("errorbars","style")(e)},styleTextPoints:d,styleOnSelect:function(t,e,n){var i=e[0].trace;i.selectedpoints?function(t,e,n){o.selectedPointStyle(t.selectAll("path"),e),function(t,e,n){t.each(function(t){var i,s=r.select(this);if(t.selected){i=a.extendFlat({},f(s,t,e,n));var A=e.selected.textfont&&e.selected.textfont.color;A&&(i.color=A),o.font(s,i)}else o.selectedTextStyle(s,e)})}(t.selectAll("text"),e,n)}(n,i,t):(h(n,i,t),s.getComponentMethod("errorbars","style")(n))},getInsideTextFont:I,getOutsideTextFont:C,getBarColor:y}},{"../../components/color":593,"../../components/drawing":614,"../../lib":719,"../../registry":847,"./attributes":857,"./helpers":862,d3:163}],871:[function(t,e,n){"use strict";var r=t("../../components/color"),i=t("../../components/colorscale/helpers").hasColorscale,o=t("../../components/colorscale/defaults");e.exports=function(t,e,n,a,s){n("marker.color",a),i(t,"marker")&&o(t,e,s,n,{prefix:"marker.",cLetter:"c"}),n("marker.line.color",r.defaultLine),i(t,"marker.line")&&o(t,e,s,n,{prefix:"marker.line.",cLetter:"c"}),n("marker.line.width"),n("marker.opacity"),n("selected.marker.color"),n("unselected.marker.color")}},{"../../components/color":593,"../../components/colorscale/defaults":603,"../../components/colorscale/helpers":604}],872:[function(t,e,n){"use strict";var r=t("../../components/fx/hovertemplate_attributes"),i=t("../../lib/extend").extendFlat,o=t("../scatterpolar/attributes"),a=t("../bar/attributes");e.exports={r:o.r,theta:o.theta,r0:o.r0,dr:o.dr,theta0:o.theta0,dtheta:o.dtheta,thetaunit:o.thetaunit,base:i({},a.base,{}),offset:i({},a.offset,{}),width:i({},a.width,{}),text:i({},a.text,{}),hovertext:i({},a.hovertext,{}),marker:a.marker,hoverinfo:o.hoverinfo,hovertemplate:r(),selected:a.selected,unselected:a.unselected}},{"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../bar/attributes":857,"../scatterpolar/attributes":1179}],873:[function(t,e,n){"use strict";var r=t("../../components/colorscale/helpers").hasColorscale,i=t("../../components/colorscale/calc"),o=t("../bar/arrays_to_calcdata"),a=t("../bar/cross_trace_calc").setGroupPositions,s=t("../scatter/calc_selection"),A=t("../../registry").traceIs,l=t("../../lib").extendFlat;e.exports={calc:function(t,e){for(var n=t._fullLayout,a=e.subplot,A=n[a].radialaxis,l=n[a].angularaxis,c=A.makeCalcdata(e,"r"),u=l.makeCalcdata(e,"theta"),g=e._length,h=new Array(g),d=c,f=u,p=0;p<g;p++)h[p]={p:f[p],s:d[p]};function I(t){var n=e[t];void 0!==n&&(e["_"+t]=Array.isArray(n)?l.makeCalcdata(e,t):l.d2c(n,e.thetaunit))}return"linear"===l.type&&(I("width"),I("offset")),r(e,"marker")&&i(t,e,{vals:e.marker.color,containerStr:"marker",cLetter:"c"}),r(e,"marker.line")&&i(t,e,{vals:e.marker.line.color,containerStr:"marker.line",cLetter:"c"}),o(h,e),s(h,e),h},crossTraceCalc:function(t,e,n){for(var r=t.calcdata,i=[],o=0;o<r.length;o++){var s=r[o],c=s[0].trace;!0===c.visible&&A(c,"bar")&&c.subplot===n&&i.push(s)}var u=l({},e.radialaxis,{_id:"x"}),g=e.angularaxis;a(t,g,u,i,{mode:e.barmode,norm:e.barnorm,gap:e.bargap,groupgap:e.bargroupgap})}}},{"../../components/colorscale/calc":601,"../../components/colorscale/helpers":604,"../../lib":719,"../../registry":847,"../bar/arrays_to_calcdata":856,"../bar/cross_trace_calc":860,"../scatter/calc_selection":1114}],874:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../scatterpolar/defaults").handleRThetaDefaults,o=t("../bar/style_defaults"),a=t("./attributes");e.exports=function(t,e,n,s){function A(n,i){return r.coerce(t,e,a,n,i)}i(t,e,s,A)?(A("thetaunit"),A("base"),A("offset"),A("width"),A("text"),A("hovertext"),A("hovertemplate"),o(t,e,A,n,s),r.coerceSelectionMarkerOpacity(e,A)):e.visible=!1}},{"../../lib":719,"../bar/style_defaults":871,"../scatterpolar/defaults":1181,"./attributes":872}],875:[function(t,e,n){"use strict";var r=t("../../components/fx"),i=t("../../lib"),o=t("../bar/hover").getTraceColor,a=i.fillText,s=t("../scatterpolar/hover").makeHoverPointText,A=t("../../plots/polar/helpers").isPtInsidePolygon;e.exports=function(t,e,n){var l=t.cd,c=l[0].trace,u=t.subplot,g=u.radialAxis,h=u.angularAxis,d=u.vangles,f=d?A:i.isPtInsideSector,p=t.maxHoverDistance,I=h._period||2*Math.PI,C=Math.abs(g.g2p(Math.sqrt(e*e+n*n))),m=Math.atan2(n,e);if(g.range[0]>g.range[1]&&(m+=Math.PI),r.getClosest(l,function(t){return f(C,m,[t.rp0,t.rp1],[t.thetag0,t.thetag1],d)?p+Math.min(1,Math.abs(t.thetag1-t.thetag0)/I)-1+(t.rp1-C)/(t.rp1-t.rp0)-1:1/0},t),!1!==t.index){var y=l[t.index];t.x0=t.x1=y.ct[0],t.y0=t.y1=y.ct[1];var E=i.extendFlat({},y,{r:y.s,theta:y.p});return a(y,c,t),s(E,c,u,t),t.hovertemplate=c.hovertemplate,t.color=o(c,y),t.xLabelVal=t.yLabelVal=void 0,y.s<0&&(t.idealAlign="left"),[t]}}},{"../../components/fx":632,"../../lib":719,"../../plots/polar/helpers":830,"../bar/hover":863,"../scatterpolar/hover":1182}],876:[function(t,e,n){"use strict";e.exports={moduleType:"trace",name:"barpolar",basePlotModule:t("../../plots/polar"),categories:["polar","bar","showLegend"],attributes:t("./attributes"),layoutAttributes:t("./layout_attributes"),supplyDefaults:t("./defaults"),supplyLayoutDefaults:t("./layout_defaults"),calc:t("./calc").calc,crossTraceCalc:t("./calc").crossTraceCalc,plot:t("./plot"),colorbar:t("../scatter/marker_colorbar"),style:t("../bar/style").style,styleOnSelect:t("../bar/style").styleOnSelect,hoverPoints:t("./hover"),selectPoints:t("../bar/select"),meta:{}}},{"../../plots/polar":831,"../bar/select":868,"../bar/style":870,"../scatter/marker_colorbar":1129,"./attributes":872,"./calc":873,"./defaults":874,"./hover":875,"./layout_attributes":877,"./layout_defaults":878,"./plot":879}],877:[function(t,e,n){"use strict";e.exports={barmode:{valType:"enumerated",values:["stack","overlay"],dflt:"stack",editType:"calc"},bargap:{valType:"number",dflt:.1,min:0,max:1,editType:"calc"}}},{}],878:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./layout_attributes");e.exports=function(t,e,n){var o,a={};function s(n,a){return r.coerce(t[o]||{},e[o],i,n,a)}for(var A=0;A<n.length;A++){var l=n[A];"barpolar"===l.type&&!0===l.visible&&(a[o=l.subplot]||(s("barmode"),s("bargap"),a[o]=1))}}},{"../../lib":719,"./layout_attributes":877}],879:[function(t,e,n){"use strict";var r=t("d3"),i=t("fast-isnumeric"),o=t("../../lib"),a=t("../../components/drawing"),s=t("../../plots/polar/helpers");e.exports=function(t,e,n){var A=e.xaxis,l=e.yaxis,c=e.radialAxis,u=e.angularAxis,g=function(t){var e=t.cxx,n=t.cyy;return t.vangles?function(r,i,a,A){var l,c;o.angleDelta(a,A)>0?(l=a,c=A):(l=A,c=a);var u=s.findEnclosingVertexAngles(l,t.vangles)[0],g=s.findEnclosingVertexAngles(c,t.vangles)[1],h=[u,(l+c)/2,g];return s.pathPolygonAnnulus(r,i,l,c,h,e,n)}:function(t,r,i,a){return o.pathAnnulus(t,r,i,a,e,n)}}(e),h=e.layers.frontplot.select("g.barlayer");o.makeTraceGroups(h,n,"trace bars").each(function(){var n=r.select(this),s=o.ensureSingle(n,"g","points").selectAll("g.point").data(o.identity);s.enter().append("g").style("vector-effect","non-scaling-stroke").style("stroke-miterlimit",2).classed("point",!0),s.exit().remove(),s.each(function(t){var e,n=r.select(this),a=t.rp0=c.c2p(t.s0),s=t.rp1=c.c2p(t.s1),h=t.thetag0=u.c2g(t.p0),d=t.thetag1=u.c2g(t.p1);if(i(a)&&i(s)&&i(h)&&i(d)&&a!==s&&h!==d){var f=c.c2g(t.s1),p=(h+d)/2;t.ct=[A.c2p(f*Math.cos(p)),l.c2p(f*Math.sin(p))],e=g(a,s,h,d)}else e="M0,0Z";o.ensureSingle(n,"path").attr("d",e)}),a.setClipUrl(n,e._hasClipOnAxisFalse?e.clipIds.forTraces:null,t)})}},{"../../components/drawing":614,"../../lib":719,"../../plots/polar/helpers":830,d3:163,"fast-isnumeric":225}],880:[function(t,e,n){"use strict";var r=t("../scatter/attributes"),i=t("../bar/attributes"),o=t("../../components/color/attributes"),a=t("../../components/fx/hovertemplate_attributes"),s=t("../../lib/extend").extendFlat,A=r.marker,l=A.line;e.exports={y:{valType:"data_array",editType:"calc+clearAxisTypes"},x:{valType:"data_array",editType:"calc+clearAxisTypes"},x0:{valType:"any",editType:"calc+clearAxisTypes"},y0:{valType:"any",editType:"calc+clearAxisTypes"},name:{valType:"string",editType:"calc+clearAxisTypes"},text:s({},r.text,{}),hovertext:s({},r.hovertext,{}),hovertemplate:a({}),whiskerwidth:{valType:"number",min:0,max:1,dflt:.5,editType:"calc"},notched:{valType:"boolean",editType:"calc"},notchwidth:{valType:"number",min:0,max:.5,dflt:.25,editType:"calc"},boxpoints:{valType:"enumerated",values:["all","outliers","suspectedoutliers",!1],dflt:"outliers",editType:"calc"},boxmean:{valType:"enumerated",values:[!0,"sd",!1],dflt:!1,editType:"calc"},jitter:{valType:"number",min:0,max:1,editType:"calc"},pointpos:{valType:"number",min:-2,max:2,editType:"calc"},orientation:{valType:"enumerated",values:["v","h"],editType:"calc+clearAxisTypes"},width:{valType:"number",min:0,dflt:0,editType:"calc"},marker:{outliercolor:{valType:"color",dflt:"rgba(0, 0, 0, 0)",editType:"style"},symbol:s({},A.symbol,{arrayOk:!1,editType:"plot"}),opacity:s({},A.opacity,{arrayOk:!1,dflt:1,editType:"style"}),size:s({},A.size,{arrayOk:!1,editType:"calc"}),color:s({},A.color,{arrayOk:!1,editType:"style"}),line:{color:s({},l.color,{arrayOk:!1,dflt:o.defaultLine,editType:"style"}),width:s({},l.width,{arrayOk:!1,dflt:0,editType:"style"}),outliercolor:{valType:"color",editType:"style"},outlierwidth:{valType:"number",min:0,dflt:1,editType:"style"},editType:"style"},editType:"plot"},line:{color:{valType:"color",editType:"style"},width:{valType:"number",min:0,dflt:2,editType:"style"},editType:"plot"},fillcolor:r.fillcolor,offsetgroup:i.offsetgroup,alignmentgroup:i.alignmentgroup,selected:{marker:r.selected.marker,editType:"style"},unselected:{marker:r.unselected.marker,editType:"style"},hoveron:{valType:"flaglist",flags:["boxes","points"],dflt:"boxes+points",editType:"style"}}},{"../../components/color/attributes":592,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../bar/attributes":857,"../scatter/attributes":1112}],881:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib"),o=i._,a=t("../../plots/cartesian/axes");function s(t,e,n){var r={text:"tx",hovertext:"htx"};for(var i in r)Array.isArray(e[i])&&(t[r[i]]=e[i][n])}function A(t,e){return t.v-e.v}function l(t){return t.v}e.exports=function(t,e){var n,c,u,g,h,d=t._fullLayout,f=a.getFromId(t,e.xaxis||"x"),p=a.getFromId(t,e.yaxis||"y"),I=[],C="violin"===e.type?"_numViolins":"_numBoxes";"h"===e.orientation?(c=f,u="x",g=p,h="y"):(c=p,u="y",g=f,h="x");var m,y=c.makeCalcdata(e,u),E=function(t,e,n,o,a){if(e in t)return n.makeCalcdata(t,e);var s;s=e+"0"in t?t[e+"0"]:"name"in t&&("category"===n.type||r(t.name)&&-1!==["linear","log"].indexOf(n.type)||i.isDateTime(t.name)&&"date"===n.type)?t.name:a;var A="multicategory"===n.type?n.r2c_just_indices(s):n.d2c(s,0,t[e+"calendar"]);return o.map(function(){return A})}(e,h,g,y,d[C]),v=i.distinctVals(E),B=v.vals,b=v.minDiff/2,Q=function(t,e){for(var n=t.length,r=new Array(n+1),i=0;i<n;i++)r[i]=t[i]-e;return r[n]=t[n-1]+e,r}(B,b),w=B.length,x=function(t){for(var e=new Array(t),n=0;n<t;n++)e[n]=[];return e}(w);for(n=0;n<e._length;n++){var _=y[n];if(r(_)){var D=i.findBin(E[n],Q);if(D>=0&&D<w){var M={v:_,i:n};s(M,e,n),x[D].push(M)}}}var k="all"===(e.boxpoints||e.points)?i.identity:function(t){return t.v<m.lf||t.v>m.uf};for(n=0;n<w;n++)if(x[n].length>0){var S=x[n].sort(A),L=S.map(l),R=L.length;(m={}).pos=B[n],m.pts=S,m[h]=m.pos,m[u]=m.pts.map(function(t){return t.v}),m.min=L[0],m.max=L[R-1],m.mean=i.mean(L,R),m.sd=i.stdev(L,R,m.mean),m.q1=i.interp(L,.25),m.med=i.interp(L,.5),m.q3=i.interp(L,.75),m.lf=Math.min(m.q1,L[Math.min(i.findBin(2.5*m.q1-1.5*m.q3,L,!0)+1,R-1)]),m.uf=Math.max(m.q3,L[Math.max(i.findBin(2.5*m.q3-1.5*m.q1,L),0)]),m.lo=4*m.q1-3*m.q3,m.uo=4*m.q3-3*m.q1;var N=1.57*(m.q3-m.q1)/Math.sqrt(R);m.ln=m.med-N,m.un=m.med+N,m.pts2=S.filter(k),I.push(m)}!function(t,e){if(i.isArrayOrTypedArray(e.selectedpoints))for(var n=0;n<t.length;n++){for(var r=t[n].pts||[],o={},a=0;a<r.length;a++)o[r[a].i]=a;i.tagSelected(r,e,o)}}(I,e);var F=a.findExtremes(c,y,{padded:!0});return e._extremes[c._id]=F,I.length>0?(I[0].t={num:d[C],dPos:b,posLetter:h,valLetter:u,labels:{med:o(t,"median:"),min:o(t,"min:"),q1:o(t,"q1:"),q3:o(t,"q3:"),max:o(t,"max:"),mean:"sd"===e.boxmean?o(t,"mean ± σ:"):o(t,"mean:"),lf:o(t,"lower fence:"),uf:o(t,"upper fence:")}},d[C]++,I):[{t:{empty:!0}}]}},{"../../lib":719,"../../plots/cartesian/axes":767,"fast-isnumeric":225}],882:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axes"),i=t("../../lib"),o=t("../../plots/cartesian/axis_ids").getAxisGroup,a=["v","h"];function s(t,e,n,a){var s,A,l,c=e.calcdata,u=e._fullLayout,g=a._id,h=g.charAt(0),d=[],f=0;for(s=0;s<n.length;s++)for(l=c[n[s]],A=0;A<l.length;A++)d.push(l[A].pos),f+=(l[A].pts2||[]).length;if(d.length){var p=i.distinctVals(d),I=p.minDiff/2;r.minDtick(a,p.minDiff,p.vals[0],!0);var C=u["violin"===t?"_numViolins":"_numBoxes"],m="group"===u[t+"mode"]&&C>1,y=1-u[t+"gap"],E=1-u[t+"groupgap"];for(s=0;s<n.length;s++){var v,B,b,Q,w,x,_=(l=c[n[s]])[0].trace,D=l[0].t,M=_.width,k=_.side;if(M)v=B=Q=M/2,b=0;else if(v=I,m){var S=o(u,a._id)+_.orientation,L=(u._alignmentOpts[S]||{})[_.alignmentgroup]||{},R=Object.keys(L.offsetGroups||{}).length,N=R||C;B=v*y*E/N,b=2*v*(((R?_._offsetIndex:D.num)+.5)/N-.5)*y,Q=v*y/N}else B=v*y*E,b=0,Q=v;D.dPos=v,D.bPos=b,D.bdPos=B,D.wHover=Q;var F,T,U,O,G,H,j=b+B,Y=Boolean(M);if("positive"===k?(w=v*(M?1:.5),F=j,x=F=b):"negative"===k?(w=F=b,x=v*(M?1:.5),T=j):(w=x=v,F=T=j),(_.boxpoints||_.points)&&f>0){var P=_.pointpos,K=_.jitter,z=_.marker.size/2,J=0;P+K>=0&&((J=j*(P+K))>w?(Y=!0,G=z,U=J):J>F&&(G=z,U=w)),J<=w&&(U=w);var q=0;P-K<=0&&((q=-j*(P-K))>x?(Y=!0,H=z,O=q):q>T&&(H=z,O=x)),q<=x&&(O=x)}else U=w,O=x;var V=new Array(l.length);for(A=0;A<l.length;A++)V[A]=l[A].pos;_._extremes[g]=r.findExtremes(a,V,{padded:Y,vpadminus:O,vpadplus:U,ppadminus:{x:H,y:G}[h],ppadplus:{x:G,y:H}[h]})}}}e.exports={crossTraceCalc:function(t,e){for(var n=t.calcdata,r=e.xaxis,i=e.yaxis,o=0;o<a.length;o++){for(var A=a[o],l="h"===A?i:r,c=[],u=0;u<n.length;u++){var g=n[u],h=g[0].t,d=g[0].trace;!0!==d.visible||"box"!==d.type&&"candlestick"!==d.type||h.empty||(d.orientation||"v")!==A||d.xaxis!==r._id||d.yaxis!==i._id||c.push(u)}s("box",t,c,l)}},setPositionOffset:s}},{"../../lib":719,"../../plots/cartesian/axes":767,"../../plots/cartesian/axis_ids":770}],883:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../registry"),o=t("../../components/color"),a=t("../bar/defaults").handleGroupingDefaults,s=t("./attributes");function A(t,e,n,o){var a,s,A=n("y"),l=n("x"),c=l&&l.length;if(A&&A.length)a="v",c?s=Math.min(r.minRowLength(l),r.minRowLength(A)):(n("x0"),s=r.minRowLength(A));else{if(!c)return void(e.visible=!1);a="h",n("y0"),s=r.minRowLength(l)}e._length=s,i.getComponentMethod("calendars","handleTraceDefaults")(t,e,["x","y"],o),n("orientation",a)}function l(t,e,n,i){var o=i.prefix,a=r.coerce2(t,e,s,"marker.outliercolor"),A=n("marker.line.outliercolor"),l=n(o+"points",a||A?"suspectedoutliers":void 0);l?(n("jitter","all"===l?.3:0),n("pointpos","all"===l?-1.5:0),n("marker.symbol"),n("marker.opacity"),n("marker.size"),n("marker.color",e.line.color),n("marker.line.color"),n("marker.line.width"),"suspectedoutliers"===l&&(n("marker.line.outliercolor",e.marker.color),n("marker.line.outlierwidth")),n("selected.marker.color"),n("unselected.marker.color"),n("selected.marker.size"),n("unselected.marker.size"),n("text"),n("hovertext")):delete e.marker;var c=n("hoveron");"all"!==c&&-1===c.indexOf("points")||n("hovertemplate"),r.coerceSelectionMarkerOpacity(e,n)}e.exports={supplyDefaults:function(t,e,n,i){function a(n,i){return r.coerce(t,e,s,n,i)}A(t,e,a,i),!1!==e.visible&&(a("line.color",(t.marker||{}).color||n),a("line.width"),a("fillcolor",o.addOpacity(e.line.color,.5)),a("whiskerwidth"),a("boxmean"),a("width"),a("notched",void 0!==t.notchwidth)&&a("notchwidth"),l(t,e,a,{prefix:"box"}))},crossTraceDefaults:function(t,e){var n,i;function o(t){return r.coerce(i._input,i,s,t)}for(var A=0;A<t.length;A++){var l=(i=t[A]).type;"box"!==l&&"violin"!==l||(n=i._input,"group"===e[l+"mode"]&&a(n,i,e,o))}},handleSampleDefaults:A,handlePointsDefaults:l}},{"../../components/color":593,"../../lib":719,"../../registry":847,"../bar/defaults":861,"./attributes":880}],884:[function(t,e,n){"use strict";e.exports=function(t,e){return e.hoverOnBox&&(t.hoverOnBox=e.hoverOnBox),"xVal"in e&&(t.x=e.xVal),"yVal"in e&&(t.y=e.yVal),e.xa&&(t.xaxis=e.xa),e.ya&&(t.yaxis=e.ya),t}},{}],885:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axes"),i=t("../../lib"),o=t("../../components/fx"),a=t("../../components/color"),s=i.fillText;function A(t,e,n,s){var A,l,c,u,g,h,d,f,p,I,C,m,y,E,v=t.cd,B=t.xa,b=t.ya,Q=v[0].trace,w=v[0].t,x="violin"===Q.type,_=[],D=w.bdPos,M=w.wHover,k=function(t){return t.pos+w.bPos-h};x&&"both"!==Q.side?("positive"===Q.side&&(p=function(t){var e=k(t);return o.inbox(e,e+M,I)},m=D,y=0),"negative"===Q.side&&(p=function(t){var e=k(t);return o.inbox(e-M,e,I)},m=0,y=D)):(p=function(t){var e=k(t);return o.inbox(e-M,e+M,I)},m=y=D),E=x?function(t){return o.inbox(t.span[0]-g,t.span[1]-g,I)}:function(t){return o.inbox(t.min-g,t.max-g,I)},"h"===Q.orientation?(g=e,h=n,d=E,f=p,A="y",c=b,l="x",u=B):(g=n,h=e,d=p,f=E,A="x",c=B,l="y",u=b);var S=Math.min(1,D/Math.abs(c.r2c(c.range[1])-c.r2c(c.range[0])));function L(t){return(d(t)+f(t))/2}I=t.maxHoverDistance-S,C=t.maxSpikeDistance-S;var R=o.getDistanceFunction(s,d,f,L);if(o.getClosest(v,R,t),!1===t.index)return[];var N=v[t.index],F=Q.line.color,T=(Q.marker||{}).color;a.opacity(F)&&Q.line.width?t.color=F:a.opacity(T)&&Q.boxpoints?t.color=T:t.color=Q.fillcolor,t[A+"0"]=c.c2p(N.pos+w.bPos-y,!0),t[A+"1"]=c.c2p(N.pos+w.bPos+m,!0),t[A+"LabelVal"]=N.pos;var U=A+"Spike";t.spikeDistance=L(N)*C/I,t[U]=c.c2p(N.pos,!0);var O={},G=["med","q1","q3","min","max"];(Q.boxmean||(Q.meanline||{}).visible)&&G.push("mean"),(Q.boxpoints||Q.points)&&G.push("lf","uf");for(var H=0;H<G.length;H++){var j=G[H];if(j in N&&!(N[j]in O)){O[N[j]]=!0;var Y=N[j],P=u.c2p(Y,!0),K=i.extendFlat({},t);K.attr=j,K[l+"0"]=K[l+"1"]=P,K[l+"LabelVal"]=Y,K[l+"Label"]=(w.labels?w.labels[j]+" ":"")+r.hoverLabelText(u,Y),K.hoverOnBox=!0,"mean"===j&&"sd"in N&&"sd"===Q.boxmean&&(K[l+"err"]=N.sd),t.name="",t.spikeDistance=void 0,t[U]=void 0,K.hovertemplate=!1,_.push(K)}}return _}function l(t,e,n){for(var r,a,A,l=t.cd,c=t.xa,u=t.ya,g=l[0].trace,h=c.c2p(e),d=u.c2p(n),f=o.quadrature(function(t){var e=Math.max(3,t.mrc||0);return Math.max(Math.abs(c.c2p(t.x)-h)-e,1-3/e)},function(t){var e=Math.max(3,t.mrc||0);return Math.max(Math.abs(u.c2p(t.y)-d)-e,1-3/e)}),p=!1,I=0;I<l.length;I++){a=l[I];for(var C=0;C<(a.pts||[]).length;C++){var m=f(A=a.pts[C]);m<=t.distance&&(t.distance=m,p=[I,C])}}if(!p)return!1;A=(a=l[p[0]]).pts[p[1]];var y,E=c.c2p(A.x,!0),v=u.c2p(A.y,!0),B=A.mrc||1;return r=i.extendFlat({},t,{index:A.i,color:(g.marker||{}).color,name:g.name,x0:E-B,x1:E+B,y0:v-B,y1:v+B,spikeDistance:t.distance,hovertemplate:g.hovertemplate}),"h"===g.orientation?(y=u,r.xLabelVal=A.x,r.yLabelVal=a.pos):(y=c,r.xLabelVal=a.pos,r.yLabelVal=A.y),r[y._id.charAt(0)+"Spike"]=y.c2p(a.pos,!0),s(A,g,r),r}e.exports={hoverPoints:function(t,e,n,r){var i,o=t.cd[0].trace.hoveron,a=[];return-1!==o.indexOf("boxes")&&(a=a.concat(A(t,e,n,r))),-1!==o.indexOf("points")&&(i=l(t,e,n)),"closest"===r?i?[i]:a:i?(a.push(i),a):a},hoverOnBoxes:A,hoverOnPoints:l}},{"../../components/color":593,"../../components/fx":632,"../../lib":719,"../../plots/cartesian/axes":767}],886:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),layoutAttributes:t("./layout_attributes"),supplyDefaults:t("./defaults").supplyDefaults,crossTraceDefaults:t("./defaults").crossTraceDefaults,supplyLayoutDefaults:t("./layout_defaults").supplyLayoutDefaults,calc:t("./calc"),crossTraceCalc:t("./cross_trace_calc").crossTraceCalc,plot:t("./plot").plot,style:t("./style").style,styleOnSelect:t("./style").styleOnSelect,hoverPoints:t("./hover").hoverPoints,eventData:t("./event_data"),selectPoints:t("./select"),moduleType:"trace",name:"box",basePlotModule:t("../../plots/cartesian"),categories:["cartesian","svg","symbols","oriented","box-violin","showLegend","boxLayout","zoomScale"],meta:{}}},{"../../plots/cartesian":778,"./attributes":880,"./calc":881,"./cross_trace_calc":882,"./defaults":883,"./event_data":884,"./hover":885,"./layout_attributes":887,"./layout_defaults":888,"./plot":889,"./select":890,"./style":891}],887:[function(t,e,n){"use strict";e.exports={boxmode:{valType:"enumerated",values:["group","overlay"],dflt:"overlay",editType:"calc"},boxgap:{valType:"number",min:0,max:1,dflt:.3,editType:"calc"},boxgroupgap:{valType:"number",min:0,max:1,dflt:.3,editType:"calc"}}},{}],888:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib"),o=t("./layout_attributes");function a(t,e,n,i,o){for(var a=o+"Layout",s=!1,A=0;A<n.length;A++){var l=n[A];if(r.traceIs(l,a)){s=!0;break}}s&&(i(o+"mode"),i(o+"gap"),i(o+"groupgap"))}e.exports={supplyLayoutDefaults:function(t,e,n){a(0,0,n,function(n,r){return i.coerce(t,e,o,n,r)},"box")},_supply:a}},{"../../lib":719,"../../registry":847,"./layout_attributes":887}],889:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../lib"),o=t("../../components/drawing"),a=5,s=.01;function A(t,e,n,o){var a,s,A=e.pos,l=e.val,c=o.bPos,u=o.wdPos||0,g=o.bPosPxOffset||0,h=n.whiskerwidth||0,d=n.notched||!1,f=d?1-2*n.notchwidth:1;Array.isArray(o.bdPos)?(a=o.bdPos[0],s=o.bdPos[1]):(a=o.bdPos,s=o.bdPos);var p=t.selectAll("path.box").data("violin"!==n.type||n.box.visible?i.identity:[]);p.enter().append("path").style("vector-effect","non-scaling-stroke").attr("class","box"),p.exit().remove(),p.each(function(t){if(t.empty)return"M0,0Z";var e=t.pos,o=A.c2p(e+c,!0)+g,p=A.c2p(e+c-a,!0)+g,I=A.c2p(e+c+s,!0)+g,C=A.c2p(e+c-u,!0)+g,m=A.c2p(e+c+u,!0)+g,y=A.c2p(e+c-a*f,!0)+g,E=A.c2p(e+c+s*f,!0)+g,v=l.c2p(t.q1,!0),B=l.c2p(t.q3,!0),b=i.constrain(l.c2p(t.med,!0),Math.min(v,B)+1,Math.max(v,B)-1),Q=void 0===t.lf||!1===n.boxpoints,w=l.c2p(Q?t.min:t.lf,!0),x=l.c2p(Q?t.max:t.uf,!0),_=l.c2p(t.ln,!0),D=l.c2p(t.un,!0);"h"===n.orientation?r.select(this).attr("d","M"+b+","+y+"V"+E+"M"+v+","+p+"V"+I+(d?"H"+_+"L"+b+","+E+"L"+D+","+I:"")+"H"+B+"V"+p+(d?"H"+D+"L"+b+","+y+"L"+_+","+p:"")+"ZM"+v+","+o+"H"+w+"M"+B+","+o+"H"+x+(0===h?"":"M"+w+","+C+"V"+m+"M"+x+","+C+"V"+m)):r.select(this).attr("d","M"+y+","+b+"H"+E+"M"+p+","+v+"H"+I+(d?"V"+_+"L"+E+","+b+"L"+I+","+D:"")+"V"+B+"H"+p+(d?"V"+D+"L"+y+","+b+"L"+p+","+_:"")+"ZM"+o+","+v+"V"+w+"M"+o+","+B+"V"+x+(0===h?"":"M"+C+","+w+"H"+m+"M"+C+","+x+"H"+m))})}function l(t,e,n,r){var A=e.x,l=e.y,c=r.bdPos,u=r.bPos,g=n.boxpoints||n.points;i.seedPseudoRandom();var h=t.selectAll("g.points").data(g?function(t){return t.forEach(function(t){t.t=r,t.trace=n}),t}:[]);h.enter().append("g").attr("class","points"),h.exit().remove();var d=h.selectAll("path").data(function(t){var e,r,o=t.pts2,A=Math.max((t.max-t.min)/10,t.q3-t.q1),l=1e-9*A,h=A*s,d=[],f=0;if(n.jitter){if(0===A)for(f=1,d=new Array(o.length),e=0;e<o.length;e++)d[e]=1;else for(e=0;e<o.length;e++){var p=Math.max(0,e-a),I=o[p].v,C=Math.min(o.length-1,e+a),m=o[C].v;"all"!==g&&(o[e].v<t.lf?m=Math.min(m,t.lf):I=Math.max(I,t.uf));var y=Math.sqrt(h*(C-p)/(m-I+l))||0;y=i.constrain(Math.abs(y),0,1),d.push(y),f=Math.max(y,f)}r=2*n.jitter/(f||1)}for(e=0;e<o.length;e++){var E=o[e],v=E.v,B=n.jitter?r*d[e]*(i.pseudoRandom()-.5):0,b=t.pos+u+c*(n.pointpos+B);"h"===n.orientation?(E.y=b,E.x=v):(E.x=b,E.y=v),"suspectedoutliers"===g&&v<t.uo&&v>t.lo&&(E.so=!0)}return o});d.enter().append("path").classed("point",!0),d.exit().remove(),d.call(o.translatePoints,A,l)}function c(t,e,n,o){var a,s,A=e.pos,l=e.val,c=o.bPos,u=o.bPosPxOffset||0,g=n.boxmean||(n.meanline||{}).visible;Array.isArray(o.bdPos)?(a=o.bdPos[0],s=o.bdPos[1]):(a=o.bdPos,s=o.bdPos);var h=t.selectAll("path.mean").data("box"===n.type&&n.boxmean||"violin"===n.type&&n.box.visible&&n.meanline.visible?i.identity:[]);h.enter().append("path").attr("class","mean").style({fill:"none","vector-effect":"non-scaling-stroke"}),h.exit().remove(),h.each(function(t){var e=A.c2p(t.pos+c,!0)+u,i=A.c2p(t.pos+c-a,!0)+u,o=A.c2p(t.pos+c+s,!0)+u,h=l.c2p(t.mean,!0),d=l.c2p(t.mean-t.sd,!0),f=l.c2p(t.mean+t.sd,!0);"h"===n.orientation?r.select(this).attr("d","M"+h+","+i+"V"+o+("sd"===g?"m0,0L"+d+","+e+"L"+h+","+i+"L"+f+","+e+"Z":"")):r.select(this).attr("d","M"+i+","+h+"H"+o+("sd"===g?"m0,0L"+e+","+d+"L"+i+","+h+"L"+e+","+f+"Z":""))})}e.exports={plot:function(t,e,n,o){var a=e.xaxis,s=e.yaxis;i.makeTraceGroups(o,n,"trace boxes").each(function(t){var e,n,i=r.select(this),o=t[0],u=o.t,g=o.trace;u.wdPos=u.bdPos*g.whiskerwidth,!0!==g.visible||u.empty?i.remove():("h"===g.orientation?(e=s,n=a):(e=a,n=s),A(i,{pos:e,val:n},g,u),l(i,{x:a,y:s},g,u),c(i,{pos:e,val:n},g,u))})},plotBoxAndWhiskers:A,plotPoints:l,plotBoxMean:c}},{"../../components/drawing":614,"../../lib":719,d3:163}],890:[function(t,e,n){"use strict";e.exports=function(t,e){var n,r,i=t.cd,o=t.xaxis,a=t.yaxis,s=[];if(!1===e)for(n=0;n<i.length;n++)for(r=0;r<(i[n].pts||[]).length;r++)i[n].pts[r].selected=0;else for(n=0;n<i.length;n++)for(r=0;r<(i[n].pts||[]).length;r++){var A=i[n].pts[r],l=o.c2p(A.x),c=a.c2p(A.y);e.contains([l,c],null,A.i,t)?(s.push({pointNumber:A.i,x:o.c2d(A.x),y:a.c2d(A.y)}),A.selected=1):A.selected=0}return s}},{}],891:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/color"),o=t("../../components/drawing");e.exports={style:function(t,e,n){var a=n||r.select(t).selectAll("g.trace.boxes");a.style("opacity",function(t){return t[0].trace.opacity}),a.each(function(e){var n=r.select(this),a=e[0].trace,s=a.line.width;function A(t,e,n,r){t.style("stroke-width",e+"px").call(i.stroke,n).call(i.fill,r)}var l=n.selectAll("path.box");if("candlestick"===a.type)l.each(function(t){if(!t.empty){var e=r.select(this),n=a[t.dir];A(e,n.line.width,n.line.color,n.fillcolor),e.style("opacity",a.selectedpoints&&!t.selected?.3:1)}});else{A(l,s,a.line.color,a.fillcolor),n.selectAll("path.mean").style({"stroke-width":s,"stroke-dasharray":2*s+"px,"+s+"px"}).call(i.stroke,a.line.color);var c=n.selectAll("path.point");o.pointStyle(c,a,t)}})},styleOnSelect:function(t,e,n){var r=e[0].trace,i=n.selectAll("path.point");r.selectedpoints?o.selectedPointStyle(i,r):o.pointStyle(i,r,t)}}},{"../../components/color":593,"../../components/drawing":614,d3:163}],892:[function(t,e,n){"use strict";var r=t("../../lib").extendFlat,i=t("../ohlc/attributes"),o=t("../box/attributes");function a(t){return{line:{color:r({},o.line.color,{dflt:t}),width:o.line.width,editType:"style"},fillcolor:o.fillcolor,editType:"style"}}e.exports={x:i.x,open:i.open,high:i.high,low:i.low,close:i.close,line:{width:r({},o.line.width,{}),editType:"style"},increasing:a(i.increasing.line.color.dflt),decreasing:a(i.decreasing.line.color.dflt),text:i.text,hovertext:i.hovertext,whiskerwidth:r({},o.whiskerwidth,{dflt:0}),hoverlabel:i.hoverlabel}},{"../../lib":719,"../box/attributes":880,"../ohlc/attributes":1058}],893:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axes"),o=t("../ohlc/calc").calcCommon;function a(t,e,n,r){return{min:n,q1:Math.min(t,r),med:r,q3:Math.max(t,r),max:e}}e.exports=function(t,e){var n=t._fullLayout,s=i.getFromId(t,e.xaxis),A=i.getFromId(t,e.yaxis),l=s.makeCalcdata(e,"x"),c=o(t,e,l,A,a);return c.length?(r.extendFlat(c[0].t,{num:n._numBoxes,dPos:r.distinctVals(l).minDiff/2,posLetter:"x",valLetter:"y"}),n._numBoxes++,c):[{t:{empty:!0}}]}},{"../../lib":719,"../../plots/cartesian/axes":767,"../ohlc/calc":1059}],894:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/color"),o=t("../ohlc/ohlc_defaults"),a=t("./attributes");function s(t,e,n,r){var o=n(r+".line.color");n(r+".line.width",e.line.width),n(r+".fillcolor",i.addOpacity(o,.5))}e.exports=function(t,e,n,i){function A(n,i){return r.coerce(t,e,a,n,i)}o(t,e,A,i)?(A("line.width"),s(0,e,A,"increasing"),s(0,e,A,"decreasing"),A("text"),A("hovertext"),A("whiskerwidth"),i._requestRangeslider[e.xaxis]=!0):e.visible=!1}},{"../../components/color":593,"../../lib":719,"../ohlc/ohlc_defaults":1063,"./attributes":892}],895:[function(t,e,n){"use strict";e.exports={moduleType:"trace",name:"candlestick",basePlotModule:t("../../plots/cartesian"),categories:["cartesian","svg","showLegend","candlestick","boxLayout"],meta:{},attributes:t("./attributes"),layoutAttributes:t("../box/layout_attributes"),supplyLayoutDefaults:t("../box/layout_defaults").supplyLayoutDefaults,crossTraceCalc:t("../box/cross_trace_calc").crossTraceCalc,supplyDefaults:t("./defaults"),calc:t("./calc"),plot:t("../box/plot").plot,layerName:"boxlayer",style:t("../box/style").style,hoverPoints:t("../ohlc/hover").hoverPoints,selectPoints:t("../ohlc/select")}},{"../../plots/cartesian":778,"../box/cross_trace_calc":882,"../box/layout_attributes":887,"../box/layout_defaults":888,"../box/plot":889,"../box/style":891,"../ohlc/hover":1061,"../ohlc/select":1065,"./attributes":892,"./calc":893,"./defaults":894}],896:[function(t,e,n){"use strict";var r=t("./axis_defaults"),i=t("../../plot_api/plot_template");e.exports=function(t,e,n,o,a){o("a")||(o("da"),o("a0")),o("b")||(o("db"),o("b0")),function(t,e,n,o){["aaxis","baxis"].forEach(function(a){var s=a.charAt(0),A=t[a]||{},l=i.newContainer(e,a),c={tickfont:"x",id:s+"axis",letter:s,font:e.font,name:a,data:t[s],calendar:e.calendar,dfltColor:o,bgColor:n.paper_bgcolor,fullLayout:n};r(A,l,c),l._categories=l._categories||[],t[a]||"-"===A.type||(t[a]={type:A.type})})}(t,e,n,a)}},{"../../plot_api/plot_template":757,"./axis_defaults":901}],897:[function(t,e,n){"use strict";var r=t("../../lib").isArrayOrTypedArray;e.exports=function(t){return function t(e,n){if(!r(e)||n>=10)return null;for(var i=1/0,o=-1/0,a=e.length,s=0;s<a;s++){var A=e[s];if(r(A)){var l=t(A,n+1);l&&(i=Math.min(l[0],i),o=Math.max(l[1],o))}else i=Math.min(A,i),o=Math.max(A,o)}return[i,o]}(t,0)}},{"../../lib":719}],898:[function(t,e,n){"use strict";var r=t("../../plots/font_attributes"),i=t("./axis_attributes"),o=t("../../components/color/attributes"),a=r({editType:"calc"});a.family.dflt='"Open Sans", verdana, arial, sans-serif',a.size.dflt=12,a.color.dflt=o.defaultLine,e.exports={carpet:{valType:"string",editType:"calc"},x:{valType:"data_array",editType:"calc+clearAxisTypes"},y:{valType:"data_array",editType:"calc+clearAxisTypes"},a:{valType:"data_array",editType:"calc"},a0:{valType:"number",dflt:0,editType:"calc"},da:{valType:"number",dflt:1,editType:"calc"},b:{valType:"data_array",editType:"calc"},b0:{valType:"number",dflt:0,editType:"calc"},db:{valType:"number",dflt:1,editType:"calc"},cheaterslope:{valType:"number",dflt:1,editType:"calc"},aaxis:i,baxis:i,font:a,color:{valType:"color",dflt:o.defaultLine,editType:"plot"},transforms:void 0}},{"../../components/color/attributes":592,"../../plots/font_attributes":793,"./axis_attributes":900}],899:[function(t,e,n){"use strict";var r=t("../../lib").isArrayOrTypedArray;e.exports=function(t,e,n,i){var o,a,s,A,l,c,u,g,h,d,f,p,I,C=r(n)?"a":"b",m=("a"===C?t.aaxis:t.baxis).smoothing,y="a"===C?t.a2i:t.b2j,E="a"===C?n:i,v="a"===C?i:n,B="a"===C?e.a.length:e.b.length,b="a"===C?e.b.length:e.a.length,Q=Math.floor("a"===C?t.b2j(v):t.a2i(v)),w="a"===C?function(e){return t.evalxy([],e,Q)}:function(e){return t.evalxy([],Q,e)};m&&(s=Math.max(0,Math.min(b-2,Q)),A=Q-s,a="a"===C?function(e,n){return t.dxydi([],e,s,n,A)}:function(e,n){return t.dxydj([],s,e,A,n)});var x=y(E[0]),_=y(E[1]),D=x<_?1:-1,M=1e-8*(_-x),k=D>0?Math.floor:Math.ceil,S=D>0?Math.ceil:Math.floor,L=D>0?Math.min:Math.max,R=D>0?Math.max:Math.min,N=k(x+M),F=S(_-M),T=[[u=w(x)]];for(o=N;o*D<F*D;o+=D)l=[],f=R(x,o),I=(p=L(_,o+D))-f,c=Math.max(0,Math.min(B-2,Math.floor(.5*(f+p)))),g=w(p),m&&(h=a(c,f-c),d=a(c,p-c),l.push([u[0]+h[0]/3*I,u[1]+h[1]/3*I]),l.push([g[0]-d[0]/3*I,g[1]-d[1]/3*I])),l.push(g),T.push(l),u=g;return T}},{"../../lib":719}],900:[function(t,e,n){"use strict";var r=t("../../plots/font_attributes"),i=t("../../components/color/attributes"),o=t("../../plots/cartesian/layout_attributes"),a=t("../../plot_api/edit_types").overrideAll;t("../../constants/docs").FORMAT_LINK,t("../../constants/docs").TIME_FORMAT_LINK,e.exports={color:{valType:"color",editType:"calc"},smoothing:{valType:"number",dflt:1,min:0,max:1.3,editType:"calc"},title:{text:{valType:"string",dflt:"",editType:"calc"},font:r({editType:"calc"}),offset:{valType:"number",dflt:10,editType:"calc"},editType:"calc"},type:{valType:"enumerated",values:["-","linear","date","category"],dflt:"-",editType:"calc"},autorange:{valType:"enumerated",values:[!0,!1,"reversed"],dflt:!0,editType:"calc"},rangemode:{valType:"enumerated",values:["normal","tozero","nonnegative"],dflt:"normal",editType:"calc"},range:{valType:"info_array",editType:"calc",items:[{valType:"any",editType:"calc"},{valType:"any",editType:"calc"}]},fixedrange:{valType:"boolean",dflt:!1,editType:"calc"},cheatertype:{valType:"enumerated",values:["index","value"],dflt:"value",editType:"calc"},tickmode:{valType:"enumerated",values:["linear","array"],dflt:"array",editType:"calc"},nticks:{valType:"integer",min:0,dflt:0,editType:"calc"},tickvals:{valType:"data_array",editType:"calc"},ticktext:{valType:"data_array",editType:"calc"},showticklabels:{valType:"enumerated",values:["start","end","both","none"],dflt:"start",editType:"calc"},tickfont:r({editType:"calc"}),tickangle:{valType:"angle",dflt:"auto",editType:"calc"},tickprefix:{valType:"string",dflt:"",editType:"calc"},showtickprefix:{valType:"enumerated",values:["all","first","last","none"],dflt:"all",editType:"calc"},ticksuffix:{valType:"string",dflt:"",editType:"calc"},showticksuffix:{valType:"enumerated",values:["all","first","last","none"],dflt:"all",editType:"calc"},showexponent:{valType:"enumerated",values:["all","first","last","none"],dflt:"all",editType:"calc"},exponentformat:{valType:"enumerated",values:["none","e","E","power","SI","B"],dflt:"B",editType:"calc"},separatethousands:{valType:"boolean",dflt:!1,editType:"calc"},tickformat:{valType:"string",dflt:"",editType:"calc"},tickformatstops:a(o.tickformatstops,"calc","from-root"),categoryorder:{valType:"enumerated",values:["trace","category ascending","category descending","array"],dflt:"trace",editType:"calc"},categoryarray:{valType:"data_array",editType:"calc"},labelpadding:{valType:"integer",dflt:10,editType:"calc"},labelprefix:{valType:"string",editType:"calc"},labelsuffix:{valType:"string",dflt:"",editType:"calc"},showline:{valType:"boolean",dflt:!1,editType:"calc"},linecolor:{valType:"color",dflt:i.defaultLine,editType:"calc"},linewidth:{valType:"number",min:0,dflt:1,editType:"calc"},gridcolor:{valType:"color",editType:"calc"},gridwidth:{valType:"number",min:0,dflt:1,editType:"calc"},showgrid:{valType:"boolean",dflt:!0,editType:"calc"},minorgridcount:{valType:"integer",min:0,dflt:0,editType:"calc"},minorgridwidth:{valType:"number",min:0,dflt:1,editType:"calc"},minorgridcolor:{valType:"color",dflt:i.lightLine,editType:"calc"},startline:{valType:"boolean",editType:"calc"},startlinecolor:{valType:"color",editType:"calc"},startlinewidth:{valType:"number",dflt:1,editType:"calc"},endline:{valType:"boolean",editType:"calc"},endlinewidth:{valType:"number",dflt:1,editType:"calc"},endlinecolor:{valType:"color",editType:"calc"},tick0:{valType:"number",min:0,dflt:0,editType:"calc"},dtick:{valType:"number",min:0,dflt:1,editType:"calc"},arraytick0:{valType:"integer",min:0,dflt:0,editType:"calc"},arraydtick:{valType:"integer",min:1,dflt:1,editType:"calc"},_deprecated:{title:{valType:"string",editType:"calc"},titlefont:r({editType:"calc"}),titleoffset:{valType:"number",dflt:10,editType:"calc"}},editType:"calc"}},{"../../components/color/attributes":592,"../../constants/docs":690,"../../plot_api/edit_types":750,"../../plots/cartesian/layout_attributes":779,"../../plots/font_attributes":793}],901:[function(t,e,n){"use strict";var r=t("./attributes"),i=t("../../components/color").addOpacity,o=t("../../registry"),a=t("../../lib"),s=t("../../plots/cartesian/tick_value_defaults"),A=t("../../plots/cartesian/tick_label_defaults"),l=t("../../plots/cartesian/category_order_defaults"),c=t("../../plots/cartesian/set_convert"),u=t("../../plots/cartesian/axis_autotype");e.exports=function(t,e,n){var g=n.letter,h=n.font||{},d=r[g+"axis"];function f(n,r){return a.coerce(t,e,d,n,r)}function p(n,r){return a.coerce2(t,e,d,n,r)}n.name&&(e._name=n.name,e._id=n.name);var I=f("type");"-"===I&&(n.data&&function(t,e){if("-"===t.type){var n=t._id.charAt(0),r=t[n+"calendar"];t.type=u(e,r)}}(e,n.data),"-"===e.type?e.type="linear":I=t.type=e.type),f("smoothing"),f("cheatertype"),f("showticklabels"),f("labelprefix",g+" = "),f("labelsuffix"),f("showtickprefix"),f("showticksuffix"),f("separatethousands"),f("tickformat"),f("exponentformat"),f("showexponent"),f("categoryorder"),f("tickmode"),f("tickvals"),f("ticktext"),f("tick0"),f("dtick"),"array"===e.tickmode&&(f("arraytick0"),f("arraydtick")),f("labelpadding"),e._hovertitle=g,"date"===I&&o.getComponentMethod("calendars","handleDefaults")(t,e,"calendar",n.calendar),c(e,n.fullLayout),e.c2p=a.identity;var C=f("color",n.dfltColor),m=C===t.color?C:h.color;f("title.text")&&(a.coerceFont(f,"title.font",{family:h.family,size:Math.round(1.2*h.size),color:m}),f("title.offset")),f("tickangle"),f("autorange",!e.isValidRange(t.range))&&f("rangemode"),f("range"),e.cleanRange(),f("fixedrange"),s(t,e,f,I),A(t,e,f,I,n),l(t,e,f,{data:n.data,dataAttr:g});var y=p("gridcolor",i(C,.3)),E=p("gridwidth"),v=f("showgrid");v||(delete e.gridcolor,delete e.gridwidth);var B=p("startlinecolor",C),b=p("startlinewidth",E);f("startline",e.showgrid||!!B||!!b)||(delete e.startlinecolor,delete e.startlinewidth);var Q=p("endlinecolor",C),w=p("endlinewidth",E);return f("endline",e.showgrid||!!Q||!!w)||(delete e.endlinecolor,delete e.endlinewidth),v?(f("minorgridcount"),f("minorgridwidth",E),f("minorgridcolor",i(y,.06)),e.minorgridcount||(delete e.minorgridwidth,delete e.minorgridcolor)):(delete e.gridcolor,delete e.gridWidth),"none"===e.showticklabels&&(delete e.tickfont,delete e.tickangle,delete e.showexponent,delete e.exponentformat,delete e.tickformat,delete e.showticksuffix,delete e.showtickprefix),e.showticksuffix||delete e.ticksuffix,e.showtickprefix||delete e.tickprefix,f("tickmode"),e}},{"../../components/color":593,"../../lib":719,"../../plots/cartesian/axis_autotype":768,"../../plots/cartesian/category_order_defaults":771,"../../plots/cartesian/set_convert":785,"../../plots/cartesian/tick_label_defaults":786,"../../plots/cartesian/tick_value_defaults":788,"../../registry":847,"./attributes":898}],902:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axes"),i=t("../../lib").isArray1D,o=t("./cheater_basis"),a=t("./array_minmax"),s=t("./calc_gridlines"),A=t("./calc_labels"),l=t("./calc_clippath"),c=t("../heatmap/clean_2d_array"),u=t("./smooth_fill_2d_array"),g=t("../heatmap/convert_column_xyz"),h=t("./set_convert");e.exports=function(t,e){var n=r.getFromId(t,e.xaxis),d=r.getFromId(t,e.yaxis),f=e.aaxis,p=e.baxis,I=e.x,C=e.y,m=[];I&&i(I)&&m.push("x"),C&&i(C)&&m.push("y"),m.length&&g(e,f,p,"a","b",m);var y=e._a=e._a||e.a,E=e._b=e._b||e.b;I=e._x||e.x,C=e._y||e.y;var v={};if(e._cheater){var B="index"===f.cheatertype?y.length:y,b="index"===p.cheatertype?E.length:E;I=o(B,b,e.cheaterslope)}e._x=I=c(I),e._y=C=c(C),u(I,y,E),u(C,y,E),h(e),e.setScale();var Q=a(I),w=a(C),x=.5*(Q[1]-Q[0]),_=.5*(Q[1]+Q[0]),D=.5*(w[1]-w[0]),M=.5*(w[1]+w[0]);return Q=[_-1.3*x,_+1.3*x],w=[M-1.3*D,M+1.3*D],e._extremes[n._id]=r.findExtremes(n,Q,{padded:!0}),e._extremes[d._id]=r.findExtremes(d,w,{padded:!0}),s(e,"a","b"),s(e,"b","a"),A(e,f),A(e,p),v.clipsegments=l(e._xctrl,e._yctrl,f,p),v.x=I,v.y=C,v.a=y,v.b=E,[v]}},{"../../lib":719,"../../plots/cartesian/axes":767,"../heatmap/clean_2d_array":1003,"../heatmap/convert_column_xyz":1005,"./array_minmax":897,"./calc_clippath":903,"./calc_gridlines":904,"./calc_labels":905,"./cheater_basis":907,"./set_convert":920,"./smooth_fill_2d_array":921}],903:[function(t,e,n){"use strict";e.exports=function(t,e,n,r){var i,o,a,s=[],A=!!n.smoothing,l=!!r.smoothing,c=t[0].length-1,u=t.length-1;for(i=0,o=[],a=[];i<=c;i++)o[i]=t[0][i],a[i]=e[0][i];for(s.push({x:o,y:a,bicubic:A}),i=0,o=[],a=[];i<=u;i++)o[i]=t[i][c],a[i]=e[i][c];for(s.push({x:o,y:a,bicubic:l}),i=c,o=[],a=[];i>=0;i--)o[c-i]=t[u][i],a[c-i]=e[u][i];for(s.push({x:o,y:a,bicubic:A}),i=u,o=[],a=[];i>=0;i--)o[u-i]=t[i][0],a[u-i]=e[i][0];return s.push({x:o,y:a,bicubic:l}),s}},{}],904:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axes"),i=t("../../lib/extend").extendFlat;e.exports=function(t,e,n){var o,a,s,A,l,c,u,g,h,d,f,p,I,C,m=t["_"+e],y=t[e+"axis"],E=y._gridlines=[],v=y._minorgridlines=[],B=y._boundarylines=[],b=t["_"+n],Q=t[n+"axis"];"array"===y.tickmode&&(y.tickvals=m.slice());var w=t._xctrl,x=t._yctrl,_=w[0].length,D=w.length,M=t._a.length,k=t._b.length;r.prepTicks(y),"array"===y.tickmode&&delete y.tickvals;var S=y.smoothing?3:1;function L(r){var i,o,a,s,A,l,c,u,h,d,f,p,I=[],C=[],m={};if("b"===e)for(o=t.b2j(r),a=Math.floor(Math.max(0,Math.min(k-2,o))),s=o-a,m.length=k,m.crossLength=M,m.xy=function(e){return t.evalxy([],e,o)},m.dxy=function(e,n){return t.dxydi([],e,a,n,s)},i=0;i<M;i++)l=Math.min(M-2,i),c=i-l,u=t.evalxy([],i,o),Q.smoothing&&i>0&&(h=t.dxydi([],i-1,a,0,s),I.push(A[0]+h[0]/3),C.push(A[1]+h[1]/3),d=t.dxydi([],i-1,a,1,s),I.push(u[0]-d[0]/3),C.push(u[1]-d[1]/3)),I.push(u[0]),C.push(u[1]),A=u;else for(i=t.a2i(r),l=Math.floor(Math.max(0,Math.min(M-2,i))),c=i-l,m.length=M,m.crossLength=k,m.xy=function(e){return t.evalxy([],i,e)},m.dxy=function(e,n){return t.dxydj([],l,e,c,n)},o=0;o<k;o++)a=Math.min(k-2,o),s=o-a,u=t.evalxy([],i,o),Q.smoothing&&o>0&&(f=t.dxydj([],l,o-1,c,0),I.push(A[0]+f[0]/3),C.push(A[1]+f[1]/3),p=t.dxydj([],l,o-1,c,1),I.push(u[0]-p[0]/3),C.push(u[1]-p[1]/3)),I.push(u[0]),C.push(u[1]),A=u;return m.axisLetter=e,m.axis=y,m.crossAxis=Q,m.value=r,m.constvar=n,m.index=g,m.x=I,m.y=C,m.smoothing=Q.smoothing,m}function R(r){var i,o,a,s,A,l=[],c=[],u={};if(u.length=m.length,u.crossLength=b.length,"b"===e)for(a=Math.max(0,Math.min(k-2,r)),A=Math.min(1,Math.max(0,r-a)),u.xy=function(e){return t.evalxy([],e,r)},u.dxy=function(e,n){return t.dxydi([],e,a,n,A)},i=0;i<_;i++)l[i]=w[r*S][i],c[i]=x[r*S][i];else for(o=Math.max(0,Math.min(M-2,r)),s=Math.min(1,Math.max(0,r-o)),u.xy=function(e){return t.evalxy([],r,e)},u.dxy=function(e,n){return t.dxydj([],o,e,s,n)},i=0;i<D;i++)l[i]=w[i][r*S],c[i]=x[i][r*S];return u.axisLetter=e,u.axis=y,u.crossAxis=Q,u.value=m[r],u.constvar=n,u.index=r,u.x=l,u.y=c,u.smoothing=Q.smoothing,u}if("array"===y.tickmode){for(A=5e-15,c=(l=[Math.floor((m.length-1-y.arraytick0)/y.arraydtick*(1+A)),Math.ceil(-y.arraytick0/y.arraydtick/(1+A))].sort(function(t,e){return t-e}))[0]-1,u=l[1]+1,g=c;g<u;g++)(a=y.arraytick0+y.arraydtick*g)<0||a>m.length-1||E.push(i(R(a),{color:y.gridcolor,width:y.gridwidth}));for(g=c;g<u;g++)if(s=y.arraytick0+y.arraydtick*g,f=Math.min(s+y.arraydtick,m.length-1),!(s<0||s>m.length-1||f<0||f>m.length-1))for(p=m[s],I=m[f],o=0;o<y.minorgridcount;o++)(C=f-s)<=0||(d=p+(I-p)*(o+1)/(y.minorgridcount+1)*(y.arraydtick/C))<m[0]||d>m[m.length-1]||v.push(i(L(d),{color:y.minorgridcolor,width:y.minorgridwidth}));y.startline&&B.push(i(R(0),{color:y.startlinecolor,width:y.startlinewidth})),y.endline&&B.push(i(R(m.length-1),{color:y.endlinecolor,width:y.endlinewidth}))}else{for(A=5e-15,c=(l=[Math.floor((m[m.length-1]-y.tick0)/y.dtick*(1+A)),Math.ceil((m[0]-y.tick0)/y.dtick/(1+A))].sort(function(t,e){return t-e}))[0],u=l[1],g=c;g<=u;g++)h=y.tick0+y.dtick*g,E.push(i(L(h),{color:y.gridcolor,width:y.gridwidth}));for(g=c-1;g<u+1;g++)for(h=y.tick0+y.dtick*g,o=0;o<y.minorgridcount;o++)(d=h+y.dtick*(o+1)/(y.minorgridcount+1))<m[0]||d>m[m.length-1]||v.push(i(L(d),{color:y.minorgridcolor,width:y.minorgridwidth}));y.startline&&B.push(i(L(m[0]),{color:y.startlinecolor,width:y.startlinewidth})),y.endline&&B.push(i(L(m[m.length-1]),{color:y.endlinecolor,width:y.endlinewidth}))}}},{"../../lib/extend":710,"../../plots/cartesian/axes":767}],905:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axes"),i=t("../../lib/extend").extendFlat;e.exports=function(t,e){var n,o,a,s=e._labels=[],A=e._gridlines;for(n=0;n<A.length;n++)a=A[n],-1!==["start","both"].indexOf(e.showticklabels)&&(o=r.tickText(e,a.value),i(o,{prefix:void 0,suffix:void 0,endAnchor:!0,xy:a.xy(0),dxy:a.dxy(0,0),axis:a.axis,length:a.crossAxis.length,font:a.axis.tickfont,isFirst:0===n,isLast:n===A.length-1}),s.push(o)),-1!==["end","both"].indexOf(e.showticklabels)&&(o=r.tickText(e,a.value),i(o,{endAnchor:!1,xy:a.xy(a.crossLength-1),dxy:a.dxy(a.crossLength-2,1),axis:a.axis,length:a.crossAxis.length,font:a.axis.tickfont,isFirst:0===n,isLast:n===A.length-1}),s.push(o))}},{"../../lib/extend":710,"../../plots/cartesian/axes":767}],906:[function(t,e,n){"use strict";e.exports=function(t,e,n,r){var i=t[0]-e[0],o=t[1]-e[1],a=n[0]-e[0],s=n[1]-e[1],A=Math.pow(i*i+o*o,.25),l=Math.pow(a*a+s*s,.25),c=(l*l*i-A*A*a)*r,u=(l*l*o-A*A*s)*r,g=l*(A+l)*3,h=A*(A+l)*3;return[[e[0]+(g&&c/g),e[1]+(g&&u/g)],[e[0]-(h&&c/h),e[1]-(h&&u/h)]]}},{}],907:[function(t,e,n){"use strict";var r=t("../../lib").isArrayOrTypedArray;e.exports=function(t,e,n){var i,o,a,s,A,l,c=[],u=r(t)?t.length:t,g=r(e)?e.length:e,h=r(t)?t:null,d=r(e)?e:null;h&&(a=(h.length-1)/(h[h.length-1]-h[0])/(u-1)),d&&(s=(d.length-1)/(d[d.length-1]-d[0])/(g-1));var f=1/0,p=-1/0;for(o=0;o<g;o++)for(c[o]=[],A=d?(d[o]-d[0])*s:o/(g-1),i=0;i<u;i++)l=(h?(h[i]-h[0])*a:i/(u-1))-A*n,f=Math.min(l,f),p=Math.max(l,p),c[o][i]=l;var I=1/(p-f),C=-f*I;for(o=0;o<g;o++)for(i=0;i<u;i++)c[o][i]=I*c[o][i]+C;return c}},{"../../lib":719}],908:[function(t,e,n){"use strict";var r=t("./catmull_rom"),i=t("../../lib").ensureArray;function o(t,e,n){var r=-.5*n[0]+1.5*e[0],i=-.5*n[1]+1.5*e[1];return[(2*r+t[0])/3,(2*i+t[1])/3]}e.exports=function(t,e,n,a,s,A){var l,c,u,g,h,d,f,p,I,C,m=n[0].length,y=n.length,E=s?3*m-2:m,v=A?3*y-2:y;for(t=i(t,v),e=i(e,v),u=0;u<v;u++)t[u]=i(t[u],E),e[u]=i(e[u],E);for(c=0,g=0;c<y;c++,g+=A?3:1)for(h=t[g],d=e[g],f=n[c],p=a[c],l=0,u=0;l<m;l++,u+=s?3:1)h[u]=f[l],d[u]=p[l];if(s)for(c=0,g=0;c<y;c++,g+=A?3:1){for(l=1,u=3;l<m-1;l++,u+=3)I=r([n[c][l-1],a[c][l-1]],[n[c][l],a[c][l]],[n[c][l+1],a[c][l+1]],s),t[g][u-1]=I[0][0],e[g][u-1]=I[0][1],t[g][u+1]=I[1][0],e[g][u+1]=I[1][1];C=o([t[g][0],e[g][0]],[t[g][2],e[g][2]],[t[g][3],e[g][3]]),t[g][1]=C[0],e[g][1]=C[1],C=o([t[g][E-1],e[g][E-1]],[t[g][E-3],e[g][E-3]],[t[g][E-4],e[g][E-4]]),t[g][E-2]=C[0],e[g][E-2]=C[1]}if(A)for(u=0;u<E;u++){for(g=3;g<v-3;g+=3)I=r([t[g-3][u],e[g-3][u]],[t[g][u],e[g][u]],[t[g+3][u],e[g+3][u]],A),t[g-1][u]=I[0][0],e[g-1][u]=I[0][1],t[g+1][u]=I[1][0],e[g+1][u]=I[1][1];C=o([t[0][u],e[0][u]],[t[2][u],e[2][u]],[t[3][u],e[3][u]]),t[1][u]=C[0],e[1][u]=C[1],C=o([t[v-1][u],e[v-1][u]],[t[v-3][u],e[v-3][u]],[t[v-4][u],e[v-4][u]]),t[v-2][u]=C[0],e[v-2][u]=C[1]}if(s&&A)for(g=1;g<v;g+=(g+1)%3==0?2:1){for(u=3;u<E-3;u+=3)I=r([t[g][u-3],e[g][u-3]],[t[g][u],e[g][u]],[t[g][u+3],e[g][u+3]],s),t[g][u-1]=.5*(t[g][u-1]+I[0][0]),e[g][u-1]=.5*(e[g][u-1]+I[0][1]),t[g][u+1]=.5*(t[g][u+1]+I[1][0]),e[g][u+1]=.5*(e[g][u+1]+I[1][1]);C=o([t[g][0],e[g][0]],[t[g][2],e[g][2]],[t[g][3],e[g][3]]),t[g][1]=.5*(t[g][1]+C[0]),e[g][1]=.5*(e[g][1]+C[1]),C=o([t[g][E-1],e[g][E-1]],[t[g][E-3],e[g][E-3]],[t[g][E-4],e[g][E-4]]),t[g][E-2]=.5*(t[g][E-2]+C[0]),e[g][E-2]=.5*(e[g][E-2]+C[1])}return[t,e]}},{"../../lib":719,"./catmull_rom":906}],909:[function(t,e,n){"use strict";e.exports={RELATIVE_CULL_TOLERANCE:1e-6}},{}],910:[function(t,e,n){"use strict";e.exports=function(t,e,n){return e&&n?function(e,n,r,i,o){var a,s,A,l,c,u;e||(e=[]),n*=3,r*=3;var g=i*i,h=1-i,d=h*h,f=h*i*2,p=-3*d,I=3*(d-f),C=3*(f-g),m=3*g,y=o*o,E=y*o,v=1-o,B=v*v,b=B*v;for(u=0;u<t.length;u++)a=p*(c=t[u])[r][n]+I*c[r][n+1]+C*c[r][n+2]+m*c[r][n+3],s=p*c[r+1][n]+I*c[r+1][n+1]+C*c[r+1][n+2]+m*c[r+1][n+3],A=p*c[r+2][n]+I*c[r+2][n+1]+C*c[r+2][n+2]+m*c[r+2][n+3],l=p*c[r+3][n]+I*c[r+3][n+1]+C*c[r+3][n+2]+m*c[r+3][n+3],e[u]=b*a+3*(B*o*s+v*y*A)+E*l;return e}:e?function(e,n,r,i,o){var a,s,A,l;e||(e=[]),n*=3;var c=i*i,u=1-i,g=u*u,h=u*i*2,d=-3*g,f=3*(g-h),p=3*(h-c),I=3*c,C=1-o;for(A=0;A<t.length;A++)a=d*(l=t[A])[r][n]+f*l[r][n+1]+p*l[r][n+2]+I*l[r][n+3],s=d*l[r+1][n]+f*l[r+1][n+1]+p*l[r+1][n+2]+I*l[r+1][n+3],e[A]=C*a+o*s;return e}:n?function(e,n,r,i,o){var a,s,A,l,c,u;e||(e=[]),r*=3;var g=o*o,h=g*o,d=1-o,f=d*d,p=f*d;for(c=0;c<t.length;c++)a=(u=t[c])[r][n+1]-u[r][n],s=u[r+1][n+1]-u[r+1][n],A=u[r+2][n+1]-u[r+2][n],l=u[r+3][n+1]-u[r+3][n],e[c]=p*a+3*(f*o*s+d*g*A)+h*l;return e}:function(e,n,r,i,o){var a,s,A,l;e||(e=[]);var c=1-o;for(A=0;A<t.length;A++)a=(l=t[A])[r][n+1]-l[r][n],s=l[r+1][n+1]-l[r+1][n],e[A]=c*a+o*s;return e}}},{}],911:[function(t,e,n){"use strict";e.exports=function(t,e,n){return e&&n?function(e,n,r,i,o){var a,s,A,l,c,u;e||(e=[]),n*=3,r*=3;var g=i*i,h=g*i,d=1-i,f=d*d,p=f*d,I=o*o,C=1-o,m=C*C,y=C*o*2,E=-3*m,v=3*(m-y),B=3*(y-I),b=3*I;for(u=0;u<t.length;u++)a=E*(c=t[u])[r][n]+v*c[r+1][n]+B*c[r+2][n]+b*c[r+3][n],s=E*c[r][n+1]+v*c[r+1][n+1]+B*c[r+2][n+1]+b*c[r+3][n+1],A=E*c[r][n+2]+v*c[r+1][n+2]+B*c[r+2][n+2]+b*c[r+3][n+2],l=E*c[r][n+3]+v*c[r+1][n+3]+B*c[r+2][n+3]+b*c[r+3][n+3],e[u]=p*a+3*(f*i*s+d*g*A)+h*l;return e}:e?function(e,n,r,i,o){var a,s,A,l,c,u;e||(e=[]),n*=3;var g=o*o,h=g*o,d=1-o,f=d*d,p=f*d;for(c=0;c<t.length;c++)a=(u=t[c])[r+1][n]-u[r][n],s=u[r+1][n+1]-u[r][n+1],A=u[r+1][n+2]-u[r][n+2],l=u[r+1][n+3]-u[r][n+3],e[c]=p*a+3*(f*o*s+d*g*A)+h*l;return e}:n?function(e,n,r,i,o){var a,s,A,l;e||(e=[]),r*=3;var c=1-i,u=o*o,g=1-o,h=g*g,d=g*o*2,f=-3*h,p=3*(h-d),I=3*(d-u),C=3*u;for(A=0;A<t.length;A++)a=f*(l=t[A])[r][n]+p*l[r+1][n]+I*l[r+2][n]+C*l[r+3][n],s=f*l[r][n+1]+p*l[r+1][n+1]+I*l[r+2][n+1]+C*l[r+3][n+1],e[A]=c*a+i*s;return e}:function(e,n,r,i,o){var a,s,A,l;e||(e=[]);var c=1-i;for(A=0;A<t.length;A++)a=(l=t[A])[r+1][n]-l[r][n],s=l[r+1][n+1]-l[r][n+1],e[A]=c*a+i*s;return e}}},{}],912:[function(t,e,n){"use strict";e.exports=function(t,e,n,r,i){var o=e-2,a=n-2;return r&&i?function(e,n,r){var i,s,A,l,c,u;e||(e=[]);var g=Math.max(0,Math.min(Math.floor(n),o)),h=Math.max(0,Math.min(Math.floor(r),a)),d=Math.max(0,Math.min(1,n-g)),f=Math.max(0,Math.min(1,r-h));g*=3,h*=3;var p=d*d,I=p*d,C=1-d,m=C*C,y=m*C,E=f*f,v=E*f,B=1-f,b=B*B,Q=b*B;for(u=0;u<t.length;u++)i=y*(c=t[u])[h][g]+3*(m*d*c[h][g+1]+C*p*c[h][g+2])+I*c[h][g+3],s=y*c[h+1][g]+3*(m*d*c[h+1][g+1]+C*p*c[h+1][g+2])+I*c[h+1][g+3],A=y*c[h+2][g]+3*(m*d*c[h+2][g+1]+C*p*c[h+2][g+2])+I*c[h+2][g+3],l=y*c[h+3][g]+3*(m*d*c[h+3][g+1]+C*p*c[h+3][g+2])+I*c[h+3][g+3],e[u]=Q*i+3*(b*f*s+B*E*A)+v*l;return e}:r?function(e,n,r){e||(e=[]);var i,s,A,l,c,u,g=Math.max(0,Math.min(Math.floor(n),o)),h=Math.max(0,Math.min(Math.floor(r),a)),d=Math.max(0,Math.min(1,n-g)),f=Math.max(0,Math.min(1,r-h));g*=3;var p=d*d,I=p*d,C=1-d,m=C*C,y=m*C,E=1-f;for(c=0;c<t.length;c++)i=E*(u=t[c])[h][g]+f*u[h+1][g],s=E*u[h][g+1]+f*u[h+1][g+1],A=E*u[h][g+2]+f*u[h+1][g+1],l=E*u[h][g+3]+f*u[h+1][g+1],e[c]=y*i+3*(m*d*s+C*p*A)+I*l;return e}:i?function(e,n,r){e||(e=[]);var i,s,A,l,c,u,g=Math.max(0,Math.min(Math.floor(n),o)),h=Math.max(0,Math.min(Math.floor(r),a)),d=Math.max(0,Math.min(1,n-g)),f=Math.max(0,Math.min(1,r-h));h*=3;var p=f*f,I=p*f,C=1-f,m=C*C,y=m*C,E=1-d;for(c=0;c<t.length;c++)i=E*(u=t[c])[h][g]+d*u[h][g+1],s=E*u[h+1][g]+d*u[h+1][g+1],A=E*u[h+2][g]+d*u[h+2][g+1],l=E*u[h+3][g]+d*u[h+3][g+1],e[c]=y*i+3*(m*f*s+C*p*A)+I*l;return e}:function(e,n,r){e||(e=[]);var i,s,A,l,c=Math.max(0,Math.min(Math.floor(n),o)),u=Math.max(0,Math.min(Math.floor(r),a)),g=Math.max(0,Math.min(1,n-c)),h=Math.max(0,Math.min(1,r-u)),d=1-h,f=1-g;for(A=0;A<t.length;A++)i=f*(l=t[A])[u][c]+g*l[u][c+1],s=f*l[u+1][c]+g*l[u+1][c+1],e[A]=d*i+h*s;return e}}},{}],913:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./xy_defaults"),o=t("./ab_defaults"),a=t("./attributes"),s=t("../../components/color/attributes");e.exports=function(t,e,n,A){function l(n,i){return r.coerce(t,e,a,n,i)}e._clipPathId="clip"+e.uid+"carpet";var c=l("color",s.defaultLine);r.coerceFont(l,"font"),l("carpet"),o(t,e,A,l,c),e.a&&e.b?(e.a.length<3&&(e.aaxis.smoothing=0),e.b.length<3&&(e.baxis.smoothing=0),i(t,e,l)||(e.visible=!1),e._cheater&&l("cheaterslope")):e.visible=!1}},{"../../components/color/attributes":592,"../../lib":719,"./ab_defaults":896,"./attributes":898,"./xy_defaults":922}],914:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),plot:t("./plot"),calc:t("./calc"),animatable:!0,isContainer:!0,moduleType:"trace",name:"carpet",basePlotModule:t("../../plots/cartesian"),categories:["cartesian","svg","carpet","carpetAxis","notLegendIsolatable","noMultiCategory"],meta:{}}},{"../../plots/cartesian":778,"./attributes":898,"./calc":902,"./defaults":913,"./plot":919}],915:[function(t,e,n){"use strict";e.exports=function(t,e){for(var n,r=t._fullData.length,i=0;i<r;i++){var o=t._fullData[i];if(o.index!==e.index&&"carpet"===o.type&&(n||(n=o),o.carpet===e.carpet))return o}return n}},{}],916:[function(t,e,n){"use strict";e.exports=function(t,e,n){if(0===t.length)return"";var r,i=[],o=n?3:1;for(r=0;r<t.length;r+=o)i.push(t[r]+","+e[r]),n&&r<t.length-o&&(i.push("C"),i.push([t[r+1]+","+e[r+1],t[r+2]+","+e[r+2]+" "].join(" ")));return i.join(n?"":"L")}},{}],917:[function(t,e,n){"use strict";var r=t("../../lib").isArrayOrTypedArray;e.exports=function(t,e,n){var i;for(r(t)?t.length>e.length&&(t=t.slice(0,e.length)):t=[],i=0;i<e.length;i++)t[i]=n(e[i]);return t}},{"../../lib":719}],918:[function(t,e,n){"use strict";e.exports=function(t,e,n,r,i,o){var a=i[0]*t.dpdx(e),s=i[1]*t.dpdy(n),A=1,l=1;if(o){var c=Math.sqrt(i[0]*i[0]+i[1]*i[1]),u=Math.sqrt(o[0]*o[0]+o[1]*o[1]),g=(i[0]*o[0]+i[1]*o[1])/c/u;l=Math.max(0,g)}var h=180*Math.atan2(s,a)/Math.PI;return h<-90?(h+=180,A=-A):h>90&&(h-=180,A=-A),{angle:h,flip:A,p:t.c2p(r,e,n),offsetMultplier:l}}},{}],919:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/drawing"),o=t("./map_1d_array"),a=t("./makepath"),s=t("./orient_text"),A=t("../../lib/svg_text_utils"),l=t("../../lib"),c=t("../../constants/alignment");function u(t,e,n,i,s,A){var l="const-"+s+"-lines",c=n.selectAll("."+l).data(A);c.enter().append("path").classed(l,!0).style("vector-effect","non-scaling-stroke"),c.each(function(n){var i=n,s=i.x,A=i.y,l=o([],s,t.c2p),c=o([],A,e.c2p),u="M"+a(l,c,i.smoothing);r.select(this).attr("d",u).style("stroke-width",i.width).style("stroke",i.color).style("fill","none")}),c.exit().remove()}function g(t,e,n,o,a,l,c,u){var g=l.selectAll("text."+u).data(c);g.enter().append("text").classed(u,!0);var h=0,d={};return g.each(function(a,l){var c;if("auto"===a.axis.tickangle)c=s(o,e,n,a.xy,a.dxy);else{var u=(a.axis.tickangle+180)*Math.PI/180;c=s(o,e,n,a.xy,[Math.cos(u),Math.sin(u)])}l||(d={angle:c.angle,flip:c.flip});var g=(a.endAnchor?-1:1)*c.flip,f=r.select(this).attr({"text-anchor":g>0?"start":"end","data-notex":1}).call(i.font,a.font).text(a.text).call(A.convertToTspans,t),p=i.bBox(this);f.attr("transform","translate("+c.p[0]+","+c.p[1]+") rotate("+c.angle+")translate("+a.axis.labelpadding*g+","+.3*p.height+")"),h=Math.max(h,p.width+a.axis.labelpadding)}),g.exit().remove(),d.maxExtent=h,d}e.exports=function(t,e,n,i){var A=e.xaxis,c=e.yaxis,h=t._fullLayout._clips;l.makeTraceGroups(i,n,"trace").each(function(e){var n=r.select(this),i=e[0],d=i.trace,p=d.aaxis,I=d.baxis,C=l.ensureSingle(n,"g","minorlayer"),m=l.ensureSingle(n,"g","majorlayer"),y=l.ensureSingle(n,"g","boundarylayer"),E=l.ensureSingle(n,"g","labellayer");n.style("opacity",d.opacity),u(A,c,m,0,"a",p._gridlines),u(A,c,m,0,"b",I._gridlines),u(A,c,C,0,"a",p._minorgridlines),u(A,c,C,0,"b",I._minorgridlines),u(A,c,y,0,"a-boundary",p._boundarylines),u(A,c,y,0,"b-boundary",I._boundarylines);var v=g(t,A,c,d,0,E,p._labels,"a-label"),B=g(t,A,c,d,0,E,I._labels,"b-label");!function(t,e,n,r,i,o,a,A){var c,u,g,h,d=l.aggNums(Math.min,null,n.a),p=l.aggNums(Math.max,null,n.a),I=l.aggNums(Math.min,null,n.b),C=l.aggNums(Math.max,null,n.b);c=.5*(d+p),u=I,g=n.ab2xy(c,u,!0),h=n.dxyda_rough(c,u),void 0===a.angle&&l.extendFlat(a,s(n,i,o,g,n.dxydb_rough(c,u))),f(t,e,n,0,g,h,n.aaxis,i,o,a,"a-title"),c=d,u=.5*(I+C),g=n.ab2xy(c,u,!0),h=n.dxydb_rough(c,u),void 0===A.angle&&l.extendFlat(A,s(n,i,o,g,n.dxyda_rough(c,u))),f(t,e,n,0,g,h,n.baxis,i,o,A,"b-title")}(t,E,d,0,A,c,v,B),function(t,e,n,r,i){var s,A,c,u,g=n.select("#"+t._clipPathId);g.size()||(g=n.append("clipPath").classed("carpetclip",!0));var h=l.ensureSingle(g,"path","carpetboundary"),d=e.clipsegments,f=[];for(u=0;u<d.length;u++)s=d[u],A=o([],s.x,r.c2p),c=o([],s.y,i.c2p),f.push(a(A,c,s.bicubic));var p="M"+f.join("L")+"Z";g.attr("id",t._clipPathId),h.attr("d",p)}(d,i,h,A,c)})};var h=c.LINE_SPACING,d=(1-c.MID_SHIFT)/h+1;function f(t,e,n,o,a,l,c,u,g,f,p){var I=[];c.title.text&&I.push(c.title.text);var C=e.selectAll("text."+p).data(I),m=f.maxExtent;C.enter().append("text").classed(p,!0),C.each(function(){var e=s(n,u,g,a,l);-1===["start","both"].indexOf(c.showticklabels)&&(m=0);var o=c.title.font.size;m+=o+c.title.offset;var p=(f.angle+(f.flip<0?180:0)-e.angle+450)%360,I=p>90&&p<270,C=r.select(this);C.text(c.title.text).call(A.convertToTspans,t),I&&(m=(-A.lineCount(C)+d)*h*o-m),C.attr("transform","translate("+e.p[0]+","+e.p[1]+") rotate("+e.angle+") translate(0,"+m+")").classed("user-select-none",!0).attr("text-anchor","middle").call(i.font,c.title.font)}),C.exit().remove()}},{"../../components/drawing":614,"../../constants/alignment":688,"../../lib":719,"../../lib/svg_text_utils":743,"./makepath":916,"./map_1d_array":917,"./orient_text":918,d3:163}],920:[function(t,e,n){"use strict";var r=t("./constants"),i=t("../../lib/search").findBin,o=t("./compute_control_points"),a=t("./create_spline_evaluator"),s=t("./create_i_derivative_evaluator"),A=t("./create_j_derivative_evaluator");e.exports=function(t){var e=t._a,n=t._b,l=e.length,c=n.length,u=t.aaxis,g=t.baxis,h=e[0],d=e[l-1],f=n[0],p=n[c-1],I=e[e.length-1]-e[0],C=n[n.length-1]-n[0],m=I*r.RELATIVE_CULL_TOLERANCE,y=C*r.RELATIVE_CULL_TOLERANCE;h-=m,d+=m,f-=y,p+=y,t.isVisible=function(t,e){return t>h&&t<d&&e>f&&e<p},t.isOccluded=function(t,e){return t<h||t>d||e<f||e>p},t.setScale=function(){var e=t._x,n=t._y,r=o(t._xctrl,t._yctrl,e,n,u.smoothing,g.smoothing);t._xctrl=r[0],t._yctrl=r[1],t.evalxy=a([t._xctrl,t._yctrl],l,c,u.smoothing,g.smoothing),t.dxydi=s([t._xctrl,t._yctrl],u.smoothing,g.smoothing),t.dxydj=A([t._xctrl,t._yctrl],u.smoothing,g.smoothing)},t.i2a=function(t){var n=Math.max(0,Math.floor(t[0]),l-2),r=t[0]-n;return(1-r)*e[n]+r*e[n+1]},t.j2b=function(t){var e=Math.max(0,Math.floor(t[1]),l-2),r=t[1]-e;return(1-r)*n[e]+r*n[e+1]},t.ij2ab=function(e){return[t.i2a(e[0]),t.j2b(e[1])]},t.a2i=function(t){var n=Math.max(0,Math.min(i(t,e),l-2)),r=e[n],o=e[n+1];return Math.max(0,Math.min(l-1,n+(t-r)/(o-r)))},t.b2j=function(t){var e=Math.max(0,Math.min(i(t,n),c-2)),r=n[e],o=n[e+1];return Math.max(0,Math.min(c-1,e+(t-r)/(o-r)))},t.ab2ij=function(e){return[t.a2i(e[0]),t.b2j(e[1])]},t.i2c=function(e,n){return t.evalxy([],e,n)},t.ab2xy=function(r,i,o){if(!o&&(r<e[0]||r>e[l-1]|i<n[0]||i>n[c-1]))return[!1,!1];var a=t.a2i(r),s=t.b2j(i),A=t.evalxy([],a,s);if(o){var u,g,h,d,f=0,p=0,I=[];r<e[0]?(u=0,g=0,f=(r-e[0])/(e[1]-e[0])):r>e[l-1]?(u=l-2,g=1,f=(r-e[l-1])/(e[l-1]-e[l-2])):g=a-(u=Math.max(0,Math.min(l-2,Math.floor(a)))),i<n[0]?(h=0,d=0,p=(i-n[0])/(n[1]-n[0])):i>n[c-1]?(h=c-2,d=1,p=(i-n[c-1])/(n[c-1]-n[c-2])):d=s-(h=Math.max(0,Math.min(c-2,Math.floor(s)))),f&&(t.dxydi(I,u,h,g,d),A[0]+=I[0]*f,A[1]+=I[1]*f),p&&(t.dxydj(I,u,h,g,d),A[0]+=I[0]*p,A[1]+=I[1]*p)}return A},t.c2p=function(t,e,n){return[e.c2p(t[0]),n.c2p(t[1])]},t.p2x=function(t,e,n){return[e.p2c(t[0]),n.p2c(t[1])]},t.dadi=function(t){var n=Math.max(0,Math.min(e.length-2,t));return e[n+1]-e[n]},t.dbdj=function(t){var e=Math.max(0,Math.min(n.length-2,t));return n[e+1]-n[e]},t.dxyda=function(e,n,r,i){var o=t.dxydi(null,e,n,r,i),a=t.dadi(e,r);return[o[0]/a,o[1]/a]},t.dxydb=function(e,n,r,i){var o=t.dxydj(null,e,n,r,i),a=t.dbdj(n,i);return[o[0]/a,o[1]/a]},t.dxyda_rough=function(e,n,r){var i=I*(r||.1),o=t.ab2xy(e+i,n,!0),a=t.ab2xy(e-i,n,!0);return[.5*(o[0]-a[0])/i,.5*(o[1]-a[1])/i]},t.dxydb_rough=function(e,n,r){var i=C*(r||.1),o=t.ab2xy(e,n+i,!0),a=t.ab2xy(e,n-i,!0);return[.5*(o[0]-a[0])/i,.5*(o[1]-a[1])/i]},t.dpdx=function(t){return t._m},t.dpdy=function(t){return t._m}}},{"../../lib/search":738,"./compute_control_points":908,"./constants":909,"./create_i_derivative_evaluator":910,"./create_j_derivative_evaluator":911,"./create_spline_evaluator":912}],921:[function(t,e,n){"use strict";var r=t("../../lib");e.exports=function(t,e,n){var i,o,a,s=[],A=[],l=t[0].length,c=t.length;function u(e,n){var r,i=0,o=0;return e>0&&void 0!==(r=t[n][e-1])&&(o++,i+=r),e<l-1&&void 0!==(r=t[n][e+1])&&(o++,i+=r),n>0&&void 0!==(r=t[n-1][e])&&(o++,i+=r),n<c-1&&void 0!==(r=t[n+1][e])&&(o++,i+=r),i/Math.max(1,o)}var g,h,d,f,p,I,C,m,y,E,v,B=0;for(i=0;i<l;i++)for(o=0;o<c;o++)void 0===t[o][i]&&(s.push(i),A.push(o),t[o][i]=u(i,o)),B=Math.max(B,Math.abs(t[o][i]));if(!s.length)return t;var b=0,Q=0,w=s.length;do{for(b=0,a=0;a<w;a++){i=s[a],o=A[a];var x,_,D,M,k,S,L=0,R=0;0===i?(D=e[k=Math.min(l-1,2)],M=e[1],x=t[o][k],R+=(_=t[o][1])+(_-x)*(e[0]-M)/(M-D),L++):i===l-1&&(D=e[k=Math.max(0,l-3)],M=e[l-2],x=t[o][k],R+=(_=t[o][l-2])+(_-x)*(e[l-1]-M)/(M-D),L++),(0===i||i===l-1)&&o>0&&o<c-1&&(g=n[o+1]-n[o],R+=((h=n[o]-n[o-1])*t[o+1][i]+g*t[o-1][i])/(h+g),L++),0===o?(D=n[S=Math.min(c-1,2)],M=n[1],x=t[S][i],R+=(_=t[1][i])+(_-x)*(n[0]-M)/(M-D),L++):o===c-1&&(D=n[S=Math.max(0,c-3)],M=n[c-2],x=t[S][i],R+=(_=t[c-2][i])+(_-x)*(n[c-1]-M)/(M-D),L++),(0===o||o===c-1)&&i>0&&i<l-1&&(g=e[i+1]-e[i],R+=((h=e[i]-e[i-1])*t[o][i+1]+g*t[o][i-1])/(h+g),L++),L?R/=L:(d=e[i+1]-e[i],f=e[i]-e[i-1],m=(p=n[o+1]-n[o])*(I=n[o]-n[o-1])*(p+I),R=((C=d*f*(d+f))*(I*t[o+1][i]+p*t[o-1][i])+m*(f*t[o][i+1]+d*t[o][i-1]))/(m*(f+d)+C*(I+p))),b+=(E=(y=R-t[o][i])/B)*E,v=L?0:.85,t[o][i]+=y*(1+v)}b=Math.sqrt(b)}while(Q++<100&&b>1e-5);return r.log("Smoother converged to",b,"after",Q,"iterations"),t}},{"../../lib":719}],922:[function(t,e,n){"use strict";var r=t("../../lib").isArray1D;e.exports=function(t,e,n){var i=n("x"),o=i&&i.length,a=n("y"),s=a&&a.length;if(!o&&!s)return!1;if(e._cheater=!i,o&&!r(i)||s&&!r(a))e._length=null;else{var A=o?i.length:1/0;s&&(A=Math.min(A,a.length)),e.a&&e.a.length&&(A=Math.min(A,e.a.length)),e.b&&e.b.length&&(A=Math.min(A,e.b.length)),e._length=A}return!0}},{"../../lib":719}],923:[function(t,e,n){"use strict";var r=t("../../components/fx/hovertemplate_attributes"),i=t("../scattergeo/attributes"),o=t("../../components/colorscale/attributes"),a=t("../../plots/attributes"),s=t("../../components/color/attributes").defaultLine,A=t("../../lib/extend").extendFlat,l=i.marker.line;e.exports=A({locations:{valType:"data_array",editType:"calc"},locationmode:i.locationmode,z:{valType:"data_array",editType:"calc"},text:A({},i.text,{}),hovertext:A({},i.hovertext,{}),marker:{line:{color:A({},l.color,{dflt:s}),width:A({},l.width,{dflt:1}),editType:"calc"},opacity:{valType:"number",arrayOk:!0,min:0,max:1,dflt:1,editType:"style"},editType:"calc"},selected:{marker:{opacity:i.selected.marker.opacity,editType:"plot"},editType:"plot"},unselected:{marker:{opacity:i.unselected.marker.opacity,editType:"plot"},editType:"plot"},hoverinfo:A({},a.hoverinfo,{editType:"calc",flags:["location","z","text","name"]}),hovertemplate:r()},o("",{cLetter:"z",editTypeOverride:"calc"}))},{"../../components/color/attributes":592,"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../scattergeo/attributes":1151}],924:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../constants/numerical").BADNUM,o=t("../../components/colorscale/calc"),a=t("../scatter/arrays_to_calcdata"),s=t("../scatter/calc_selection");function A(t){return t&&"string"==typeof t}e.exports=function(t,e){var n,l=e._length,c=new Array(l);n=e.geojson?function(t){return A(t)||r(t)}:A;for(var u=0;u<l;u++){var g=c[u]={},h=e.locations[u],d=e.z[u];n(h)&&r(d)?(g.loc=h,g.z=d):(g.loc=null,g.z=i),g.index=u}return a(c,e),o(t,e,{vals:e.z,containerStr:"",cLetter:"z"}),s(c,e),c}},{"../../components/colorscale/calc":601,"../../constants/numerical":695,"../scatter/arrays_to_calcdata":1111,"../scatter/calc_selection":1114,"fast-isnumeric":225}],925:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/colorscale/defaults"),o=t("./attributes");e.exports=function(t,e,n,a){function s(n,i){return r.coerce(t,e,o,n,i)}var A=s("locations"),l=s("z");A&&A.length&&r.isArrayOrTypedArray(l)&&l.length?(e._length=Math.min(A.length,l.length),s("locationmode"),s("text"),s("hovertext"),s("hovertemplate"),s("marker.line.width")&&s("marker.line.color"),s("marker.opacity"),i(t,e,a,s,{prefix:"",cLetter:"z"}),r.coerceSelectionMarkerOpacity(e,s)):e.visible=!1}},{"../../components/colorscale/defaults":603,"../../lib":719,"./attributes":923}],926:[function(t,e,n){"use strict";e.exports=function(t,e,n,r,i){t.location=e.location,t.z=e.z;var o=r[i];return o.fIn&&(t.properties=o.fIn.properties),t}},{}],927:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axes"),i=t("./attributes"),o=t("../../lib").fillText;e.exports=function(t,e,n){var a,s,A,l,c=t.cd,u=c[0].trace,g=t.subplot;for(s=0;s<c.length;s++)if(l=!1,(a=c[s])._polygons){for(A=0;A<a._polygons.length;A++)a._polygons[A].contains([e,n])&&(l=!l),a._polygons[A].contains([e+360,n])&&(l=!l);if(l)break}if(l&&a)return t.x0=t.x1=t.xa.c2p(a.ct),t.y0=t.y1=t.ya.c2p(a.ct),t.index=a.index,t.location=a.loc,t.z=a.z,t.zLabel=r.tickText(g.mockAxis,g.mockAxis.c2l(a.z),"hover").text,t.hovertemplate=a.hovertemplate,function(t,e,n){if(!e.hovertemplate){var r=n.hi||e.hoverinfo,a="all"===r?i.hoverinfo.flags:r.split("+"),s=-1!==a.indexOf("name"),A=-1!==a.indexOf("location"),l=-1!==a.indexOf("z"),c=-1!==a.indexOf("text"),u=[];!s&&A?t.nameOverride=n.loc:(s&&(t.nameOverride=e.name),A&&u.push(n.loc)),l&&u.push(t.zLabel),c&&o(n,e,u),t.extraText=u.join("<br>")}}(t,u,a,g.mockAxis),[t]}},{"../../lib":719,"../../plots/cartesian/axes":767,"./attributes":923}],928:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:t("../heatmap/colorbar"),calc:t("./calc"),plot:t("./plot").plot,style:t("./style").style,styleOnSelect:t("./style").styleOnSelect,hoverPoints:t("./hover"),eventData:t("./event_data"),selectPoints:t("./select"),moduleType:"trace",name:"choropleth",basePlotModule:t("../../plots/geo"),categories:["geo","noOpacity"],meta:{}}},{"../../plots/geo":797,"../heatmap/colorbar":1004,"./attributes":923,"./calc":924,"./defaults":925,"./event_data":926,"./hover":927,"./plot":929,"./select":930,"./style":931}],929:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../lib"),o=t("../../lib/polygon"),a=t("../../lib/topojson_utils").getTopojsonFeatures,s=t("../../lib/geo_location_utils").locationToFeature,A=t("./style").style;function l(t,e){for(var n=t[0].trace,r=t.length,i=a(n,e),o=0;o<r;o++){var A=t[o],l=s(n.locationmode,A.loc,i);l?(A.geojson=l,A.ct=l.properties.ct,A._polygons=c(l)):A.geojson=null}}function c(t){var e,n,r,i,a=t.geometry,s=a.coordinates,A=t.id,l=[];function c(t){for(var e=0;e<t.length-1;e++)if(t[e][0]>0&&t[e+1][0]<0)return e;return null}switch(e="RUS"===A||"FJI"===A?function(t){var e;if(null===c(t))e=t;else for(e=new Array(t.length),i=0;i<t.length;i++)e[i]=[t[i][0]<0?t[i][0]+360:t[i][0],t[i][1]];l.push(o.tester(e))}:"ATA"===A?function(t){var e=c(t);if(null===e)return l.push(o.tester(t));var n=new Array(t.length+1),r=0;for(i=0;i<t.length;i++)i>e?n[r++]=[t[i][0]+360,t[i][1]]:i===e?(n[r++]=t[i],n[r++]=[t[i][0],-90]):n[r++]=t[i];var a=o.tester(n);a.pts.pop(),l.push(a)}:function(t){l.push(o.tester(t))},a.type){case"MultiPolygon":for(n=0;n<s.length;n++)for(r=0;r<s[n].length;r++)e(s[n][r]);break;case"Polygon":for(n=0;n<s.length;n++)e(s[n])}return l}e.exports={plot:function(t,e,n){for(var o=0;o<n.length;o++)l(n[o],e.topojson);var a=e.layers.backplot.select(".choroplethlayer");i.makeTraceGroups(a,n,"trace choropleth").each(function(e){var n=r.select(this).selectAll("path.choroplethlocation").data(i.identity);n.enter().append("path").classed("choroplethlocation",!0),n.exit().remove(),A(t,e)})},feature2polygons:c}},{"../../lib":719,"../../lib/geo_location_utils":713,"../../lib/polygon":731,"../../lib/topojson_utils":746,"./style":931,d3:163}],930:[function(t,e,n){"use strict";e.exports=function(t,e){var n,r,i,o,a,s=t.cd,A=t.xaxis,l=t.yaxis,c=[];if(!1===e)for(n=0;n<s.length;n++)s[n].selected=0;else for(n=0;n<s.length;n++)(i=(r=s[n]).ct)&&(o=A.c2p(i),a=l.c2p(i),e.contains([o,a],null,n,t)?(c.push({pointNumber:n,lon:i[0],lat:i[1]}),r.selected=1):r.selected=0);return c}},{}],931:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/color"),o=t("../../components/drawing"),a=t("../../components/colorscale");function s(t,e){var n=e[0].trace,s=e[0].node3.selectAll(".choroplethlocation"),A=n.marker||{},l=A.line||{},c=a.makeColorScaleFuncFromTrace(n);s.each(function(t){r.select(this).attr("fill",c(t.z)).call(i.stroke,t.mlc||l.color).call(o.dashLine,"",t.mlw||l.width||0).style("opacity",A.opacity)}),o.selectedPointStyle(s,n,t)}e.exports={style:function(t,e){e&&s(t,e)},styleOnSelect:function(t,e){var n=e[0].node3,r=e[0].trace;r.selectedpoints?o.selectedPointStyle(n.selectAll(".choroplethlocation"),r,t):s(t,e)}}},{"../../components/color":593,"../../components/colorscale":605,"../../components/drawing":614,d3:163}],932:[function(t,e,n){"use strict";var r=t("../choropleth/attributes"),i=t("../../components/colorscale/attributes"),o=t("../../components/fx/hovertemplate_attributes"),a=t("../../lib/extend").extendFlat;e.exports=a({locations:{valType:"data_array",editType:"calc"},z:{valType:"data_array",editType:"calc"},geojson:{valType:"any",editType:"calc"},below:{valType:"string",editType:"plot"},text:r.text,hovertext:r.hovertext,marker:{line:{color:a({},r.marker.line.color,{editType:"plot"}),width:a({},r.marker.line.width,{editType:"plot"}),editType:"calc"},opacity:a({},r.marker.opacity,{editType:"plot"}),editType:"calc"},selected:{marker:{opacity:a({},r.selected.marker.opacity,{editType:"plot"}),editType:"plot"},editType:"plot"},unselected:{marker:{opacity:a({},r.unselected.marker.opacity,{editType:"plot"}),editType:"plot"},editType:"plot"},hoverinfo:r.hoverinfo,hovertemplate:o({},{keys:["properties"]})},i("",{cLetter:"z",editTypeOverride:"calc"}))},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../choropleth/attributes":923}],933:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("@turf/area"),o=t("@turf/centroid"),a=t("../../lib"),s=t("../../components/colorscale"),A=t("../../components/drawing"),l=t("../../lib/geojson_utils").makeBlank,c=t("../choropleth/plot").feature2polygons;function u(t){var e,n=t[0].trace,r=n._opts;if(n.selectedpoints){for(var i=A.makeSelectedPointStyleFns(n),o=0;o<t.length;o++){var s=t[o];s.fOut&&(s.fOut.properties.mo2=i.selectedOpacityFn(s))}e={type:"identity",property:"mo2"}}else e=a.isArrayOrTypedArray(n.marker.opacity)?{type:"identity",property:"mo"}:n.marker.opacity;return a.extendFlat(r.fill.paint,{"fill-opacity":e}),a.extendFlat(r.line.paint,{"line-opacity":e}),r}e.exports={convert:function(t){var e=t[0].trace,n=!0===e.visible&&0!==e._length,A={layout:{visibility:"none"},paint:{}},g={layout:{visibility:"none"},paint:{}},h=e._opts={fill:A,line:g,geojson:l()};if(!n)return h;var d="string"==typeof e.geojson?(window.PlotlyGeoAssets||{})[e.geojson]:e.geojson;if(!a.isPlainObject(d))return a.error("Oops ... something when wrong when fetching "+e.geojson),h;var f,p={},I=[];for(f=0;f<t.length;f++){var C=t[f];C.loc&&(p[C.loc]=C)}var m,y,E,v=s.makeColorScaleFuncFromTrace(e),B=e.marker,b=B.line||{};function Q(t){var e=p[t.id];if(e){var n=t.geometry;if("Polygon"===n.type||"MultiPolygon"===n.type){var r={fc:v(e.z)};m&&(r.mo=m(e)),y&&(r.mlc=y(e)),E&&(r.mlw=E(e));var s={type:"Feature",geometry:n,properties:r};e._polygons=c(s),e.ct=function(t){var e,n=t.geometry;if("MultiPolygon"===n.type)for(var r=n.coordinates,a=0,s=0;s<r.length;s++){var A={type:"Polygon",coordinates:r[s]},l=i.default(A);l>a&&(a=l,e=A)}else e=n;return o.default(e).geometry.coordinates}(s),e.fIn=t,e.fOut=s,I.push(s)}else a.log(["Location with id",e.loc,"does not have a valid GeoJSON geometry,","choroplethmapbox traces only support *Polygon* and *MultiPolygon* geometries."].join(" "))}delete p[t.id]}switch(a.isArrayOrTypedArray(B.opacity)&&(m=function(t){var e=t.mo;return r(e)?+a.constrain(e,0,1):0}),a.isArrayOrTypedArray(b.color)&&(y=function(t){return t.mlc}),a.isArrayOrTypedArray(b.width)&&(E=function(t){return t.mlw}),d.type){case"FeatureCollection":var w=d.features;for(f=0;f<w.length;f++)Q(w[f]);break;case"Feature":Q(d);break;default:return a.warn(["Invalid GeoJSON type",(d.type||"none")+",","choroplethmapbox traces only support *FeatureCollection* and *Feature* types."].join(" ")),h}for(var x in p)a.log("Location with id "+x+" does not have a matching feature");var _=m?{type:"identity",property:"mo"}:B.opacity;return a.extendFlat(A.paint,{"fill-color":{type:"identity",property:"fc"},"fill-opacity":_}),a.extendFlat(g.paint,{"line-color":y?{type:"identity",property:"mlc"}:b.color,"line-width":E?{type:"identity",property:"mlw"}:b.width,"line-opacity":_}),A.layout.visibility="visible",g.layout.visibility="visible",h.geojson={type:"FeatureCollection",features:I},u(t),h},convertOnSelect:u}},{"../../components/colorscale":605,"../../components/drawing":614,"../../lib":719,"../../lib/geojson_utils":714,"../choropleth/plot":929,"@turf/area":55,"@turf/centroid":56,"fast-isnumeric":225}],934:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/colorscale/defaults"),o=t("./attributes");e.exports=function(t,e,n,a){function s(n,i){return r.coerce(t,e,o,n,i)}var A=s("locations"),l=s("z"),c=s("geojson");r.isArrayOrTypedArray(A)&&A.length&&r.isArrayOrTypedArray(l)&&l.length&&("string"==typeof c&&""!==c||r.isPlainObject(c))?(e._length=Math.min(A.length,l.length),s("below"),s("text"),s("hovertext"),s("hovertemplate"),s("marker.line.width")&&s("marker.line.color"),s("marker.opacity"),i(t,e,a,s,{prefix:"",cLetter:"z"}),r.coerceSelectionMarkerOpacity(e,s)):e.visible=!1}},{"../../components/colorscale/defaults":603,"../../lib":719,"./attributes":932}],935:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:t("../heatmap/colorbar"),calc:t("../choropleth/calc"),plot:t("./plot"),hoverPoints:t("../choropleth/hover"),eventData:t("../choropleth/event_data"),selectPoints:t("../choropleth/select"),styleOnSelect:function(t,e){e&&e[0].trace._glTrace.updateOnSelect(e)},getBelow:function(t,e){for(var n=e.getMapLayers(),r=n.length-2;r>=0;r--){var i=n[r].id;if("string"==typeof i&&0===i.indexOf("water"))for(var o=r+1;o<n.length;o++)if("string"==typeof(i=n[o].id)&&-1===i.indexOf("plotly-"))return i}},moduleType:"trace",name:"choroplethmapbox",basePlotModule:t("../../plots/mapbox"),categories:["mapbox","gl","noOpacity"],meta:{hr_name:"choropleth_mapbox"}}},{"../../plots/mapbox":822,"../choropleth/calc":924,"../choropleth/event_data":926,"../choropleth/hover":927,"../choropleth/select":930,"../heatmap/colorbar":1004,"./attributes":932,"./defaults":934,"./plot":936}],936:[function(t,e,n){"use strict";var r=t("./convert").convert,i=t("./convert").convertOnSelect,o=t("../../plots/mapbox/constants").traceLayerPrefix;function a(t,e){this.subplot=t,this.uid=e,this.sourceId="source-"+e,this.layerList=[["fill",o+e+"-fill"],["line",o+e+"-line"]],this.below=null}var s=a.prototype;s.update=function(t){this._update(r(t))},s.updateOnSelect=function(t){this._update(i(t))},s._update=function(t){var e=this.subplot,n=this.layerList,r=e.belowLookup["trace-"+this.uid];e.map.getSource(this.sourceId).setData(t.geojson),r!==this.below&&(this._removeLayers(),this._addLayers(t,r),this.below=r);for(var i=0;i<n.length;i++){var o=n[i],a=o[0],s=o[1],A=t[a];e.setOptions(s,"setLayoutProperty",A.layout),"visible"===A.layout.visibility&&e.setOptions(s,"setPaintProperty",A.paint)}},s._addLayers=function(t,e){for(var n=this.subplot,r=this.layerList,i=this.sourceId,o=0;o<r.length;o++){var a=r[o],s=a[0],A=t[s];n.addLayer({type:s,id:a[1],source:i,layout:A.layout,paint:A.paint},e)}},s._removeLayers=function(){for(var t=this.subplot.map,e=this.layerList,n=e.length-1;n>=0;n--)t.removeLayer(e[n][1])},s.dispose=function(){var t=this.subplot.map;this._removeLayers(),t.removeSource(this.sourceId)},e.exports=function(t,e){var n=e[0].trace,i=new a(t,n.uid),o=i.sourceId,s=r(e),A=i.below=t.belowLookup["trace-"+n.uid];return t.map.addSource(o,{type:"geojson",data:s.geojson}),i._addLayers(s,A),e[0].trace._glTrace=i,i}},{"../../plots/mapbox/constants":820,"./convert":933}],937:[function(t,e,n){"use strict";var r=t("../../components/colorscale/attributes"),i=t("../../components/fx/hovertemplate_attributes"),o=t("../mesh3d/attributes"),a=t("../../plots/attributes"),s=t("../../lib/extend").extendFlat,A={x:{valType:"data_array",editType:"calc+clearAxisTypes"},y:{valType:"data_array",editType:"calc+clearAxisTypes"},z:{valType:"data_array",editType:"calc+clearAxisTypes"},u:{valType:"data_array",editType:"calc"},v:{valType:"data_array",editType:"calc"},w:{valType:"data_array",editType:"calc"},sizemode:{valType:"enumerated",values:["scaled","absolute"],editType:"calc",dflt:"scaled"},sizeref:{valType:"number",editType:"calc",min:0},anchor:{valType:"enumerated",editType:"calc",values:["tip","tail","cm","center"],dflt:"cm"},text:{valType:"string",dflt:"",arrayOk:!0,editType:"calc"},hovertext:{valType:"string",dflt:"",arrayOk:!0,editType:"calc"},hovertemplate:i({editType:"calc"},{keys:["norm"]})};s(A,r("",{colorAttr:"u/v/w norm",showScaleDflt:!0,editTypeOverride:"calc"})),["opacity","lightposition","lighting"].forEach(function(t){A[t]=o[t]}),A.hoverinfo=s({},a.hoverinfo,{editType:"calc",flags:["x","y","z","u","v","w","norm","text","name"],dflt:"x+y+z+norm+text+name"}),A.transforms=void 0,e.exports=A},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../mesh3d/attributes":1053}],938:[function(t,e,n){"use strict";var r=t("../../components/colorscale/calc");e.exports=function(t,e){for(var n=e.u,i=e.v,o=e.w,a=Math.min(e.x.length,e.y.length,e.z.length,n.length,i.length,o.length),s=-1/0,A=1/0,l=0;l<a;l++){var c=n[l],u=i[l],g=o[l],h=Math.sqrt(c*c+u*u+g*g);s=Math.max(s,h),A=Math.min(A,h)}e._len=a,e._normMax=s,r(t,e,{vals:[A,s],containerStr:"",cLetter:"c"})}},{"../../components/colorscale/calc":601}],939:[function(t,e,n){"use strict";var r=t("gl-cone3d"),i=t("gl-cone3d").createConeMesh,o=t("../../lib").simpleMap,a=t("../../lib/gl_format_color").parseColorScale,s=t("../../components/colorscale").extractOpts,A=t("../../plots/gl3d/zip3");function l(t,e){this.scene=t,this.uid=e,this.mesh=null,this.data=null}var c=l.prototype;c.handlePick=function(t){if(t.object===this.mesh){var e=t.index=t.data.index,n=this.data.x[e],r=this.data.y[e],i=this.data.z[e],o=this.data.u[e],a=this.data.v[e],s=this.data.w[e];t.traceCoordinate=[n,r,i,o,a,s,Math.sqrt(o*o+a*a+s*s)];var A=this.data.hovertext||this.data.text;return Array.isArray(A)&&void 0!==A[e]?t.textLabel=A[e]:A&&(t.textLabel=A),!0}};var u={xaxis:0,yaxis:1,zaxis:2},g={tip:1,tail:0,cm:.25,center:.5},h={tip:1,tail:1,cm:.75,center:.5};function d(t,e){var n=t.fullSceneLayout,i=t.dataScale,l={};function c(t,e){var r=n[e],a=i[u[e]];return o(t,function(t){return r.d2l(t)*a})}l.vectors=A(c(e.u,"xaxis"),c(e.v,"yaxis"),c(e.w,"zaxis"),e._len),l.positions=A(c(e.x,"xaxis"),c(e.y,"yaxis"),c(e.z,"zaxis"),e._len);var d=s(e);l.colormap=a(e),l.vertexIntensityBounds=[d.min/e._normMax,d.max/e._normMax],l.coneOffset=g[e.anchor],"scaled"===e.sizemode?l.coneSize=e.sizeref||.5:l.coneSize=e.sizeref&&e._normMax?e.sizeref/e._normMax:.5;var f=r(l),p=e.lightposition;return f.lightPosition=[p.x,p.y,p.z],f.ambient=e.lighting.ambient,f.diffuse=e.lighting.diffuse,f.specular=e.lighting.specular,f.roughness=e.lighting.roughness,f.fresnel=e.lighting.fresnel,f.opacity=e.opacity,e._pad=h[e.anchor]*f.vectorScale*f.coneScale*e._normMax,f}c.update=function(t){this.data=t;var e=d(this.scene,t);this.mesh.update(e)},c.dispose=function(){this.scene.glplot.remove(this.mesh),this.mesh.dispose()},e.exports=function(t,e){var n=t.glplot.gl,r=d(t,e),o=i(n,r),a=new l(t,e.uid);return a.mesh=o,a.data=e,o._trace=a,t.glplot.add(o),a}},{"../../components/colorscale":605,"../../lib":719,"../../lib/gl_format_color":716,"../../plots/gl3d/zip3":818,"gl-cone3d":242}],940:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/colorscale/defaults"),o=t("./attributes");e.exports=function(t,e,n,a){function s(n,i){return r.coerce(t,e,o,n,i)}var A=s("u"),l=s("v"),c=s("w"),u=s("x"),g=s("y"),h=s("z");A&&A.length&&l&&l.length&&c&&c.length&&u&&u.length&&g&&g.length&&h&&h.length?(s("sizeref"),s("sizemode"),s("anchor"),s("lighting.ambient"),s("lighting.diffuse"),s("lighting.specular"),s("lighting.roughness"),s("lighting.fresnel"),s("lightposition.x"),s("lightposition.y"),s("lightposition.z"),i(t,e,a,s,{prefix:"",cLetter:"c"}),s("text"),s("hovertext"),s("hovertemplate"),e._length=null):e.visible=!1}},{"../../components/colorscale/defaults":603,"../../lib":719,"./attributes":937}],941:[function(t,e,n){"use strict";e.exports={moduleType:"trace",name:"cone",basePlotModule:t("../../plots/gl3d"),categories:["gl3d"],attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:{min:"cmin",max:"cmax"},calc:t("./calc"),plot:t("./convert"),eventData:function(t,e){return t.norm=e.traceCoordinate[6],t},meta:{}}},{"../../plots/gl3d":807,"./attributes":937,"./calc":938,"./convert":939,"./defaults":940}],942:[function(t,e,n){"use strict";var r=t("../heatmap/attributes"),i=t("../scatter/attributes"),o=t("../../components/colorscale/attributes"),a=t("../../components/drawing/attributes").dash,s=t("../../plots/font_attributes"),A=t("../../lib/extend").extendFlat,l=t("../../constants/filter_ops"),c=l.COMPARISON_OPS2,u=l.INTERVAL_OPS,g=(t("../../constants/docs").FORMAT_LINK,i.line);e.exports=A({z:r.z,x:r.x,x0:r.x0,dx:r.dx,y:r.y,y0:r.y0,dy:r.dy,text:r.text,hovertext:r.hovertext,transpose:r.transpose,xtype:r.xtype,ytype:r.ytype,zhoverformat:r.zhoverformat,hovertemplate:r.hovertemplate,connectgaps:r.connectgaps,fillcolor:{valType:"color",editType:"calc"},autocontour:{valType:"boolean",dflt:!0,editType:"calc",impliedEdits:{"contours.start":void 0,"contours.end":void 0,"contours.size":void 0}},ncontours:{valType:"integer",dflt:15,min:1,editType:"calc"},contours:{type:{valType:"enumerated",values:["levels","constraint"],dflt:"levels",editType:"calc"},start:{valType:"number",dflt:null,editType:"plot",impliedEdits:{"^autocontour":!1}},end:{valType:"number",dflt:null,editType:"plot",impliedEdits:{"^autocontour":!1}},size:{valType:"number",dflt:null,min:0,editType:"plot",impliedEdits:{"^autocontour":!1}},coloring:{valType:"enumerated",values:["fill","heatmap","lines","none"],dflt:"fill",editType:"calc"},showlines:{valType:"boolean",dflt:!0,editType:"plot"},showlabels:{valType:"boolean",dflt:!1,editType:"plot"},labelfont:s({editType:"plot",colorEditType:"style"}),labelformat:{valType:"string",dflt:"",editType:"plot"},operation:{valType:"enumerated",values:[].concat(c).concat(u),dflt:"=",editType:"calc"},value:{valType:"any",dflt:0,editType:"calc"},editType:"calc",impliedEdits:{autocontour:!1}},line:{color:A({},g.color,{editType:"style+colorbars"}),width:A({},g.width,{editType:"style+colorbars"}),dash:a,smoothing:A({},g.smoothing,{}),editType:"plot"}},o("",{cLetter:"z",autoColorDflt:!1,editTypeOverride:"calc"}))},{"../../components/colorscale/attributes":600,"../../components/drawing/attributes":613,"../../constants/docs":690,"../../constants/filter_ops":691,"../../lib/extend":710,"../../plots/font_attributes":793,"../heatmap/attributes":1001,"../scatter/attributes":1112}],943:[function(t,e,n){"use strict";var r=t("../../components/colorscale"),i=t("../heatmap/calc"),o=t("./set_contours"),a=t("./end_plus");e.exports=function(t,e){var n=i(t,e),s=n[0].z;o(e,s);var A,l=e.contours,c=r.extractOpts(e);if("heatmap"===l.coloring&&c.auto&&!1===e.autocontour){var u=l.start,g=a(l),h=l.size||1,d=Math.floor((g-u)/h)+1;isFinite(h)||(h=1,d=1);var f=u-h/2;A=[f,f+d*h]}else A=s;return r.calc(t,e,{vals:A,cLetter:"z"}),n}},{"../../components/colorscale":605,"../heatmap/calc":1002,"./end_plus":953,"./set_contours":961}],944:[function(t,e,n){"use strict";e.exports=function(t,e,n,r){var i,o,a,s=t[0],A=s.x.length,l=s.y.length,c=s.z,u=r.contours,g=-1/0,h=1/0;for(i=0;i<l;i++)h=Math.min(h,c[i][0]),h=Math.min(h,c[i][A-1]),g=Math.max(g,c[i][0]),g=Math.max(g,c[i][A-1]);for(i=1;i<A-1;i++)h=Math.min(h,c[0][i]),h=Math.min(h,c[l-1][i]),g=Math.max(g,c[0][i]),g=Math.max(g,c[l-1][i]);switch(s.prefixBoundary=!1,e){case">":u.value>g&&(s.prefixBoundary=!0);break;case"<":u.value<h&&(s.prefixBoundary=!0);break;case"[]":o=Math.min.apply(null,u.value),((a=Math.max.apply(null,u.value))<h||o>g)&&(s.prefixBoundary=!0);break;case"][":o=Math.min.apply(null,u.value),a=Math.max.apply(null,u.value),o<h&&a>g&&(s.prefixBoundary=!0)}}},{}],945:[function(t,e,n){"use strict";var r=t("../../components/colorscale").extractOpts,i=t("./make_color_map"),o=t("./end_plus");e.exports={min:"zmin",max:"zmax",calc:function(t,e,n){var a=e.contours,s=e.line,A=a.size||1,l=a.coloring,c=i(e,{isColorbar:!0});if("heatmap"===l){var u=r(e);n._fillgradient=e.colorscale,n._zrange=[u.min,u.max]}else"fill"===l&&(n._fillcolor=c);n._line={color:"lines"===l?c:s.color,width:!1!==a.showlines?s.width:0,dash:s.dash},n._levels={start:a.start,end:o(a),size:A}}}},{"../../components/colorscale":605,"./end_plus":953,"./make_color_map":958}],946:[function(t,e,n){"use strict";e.exports={BOTTOMSTART:[1,9,13,104,713],TOPSTART:[4,6,7,104,713],LEFTSTART:[8,12,14,208,1114],RIGHTSTART:[2,3,11,208,1114],NEWDELTA:[null,[-1,0],[0,-1],[-1,0],[1,0],null,[0,-1],[-1,0],[0,1],[0,1],null,[0,1],[1,0],[1,0],[0,-1]],CHOOSESADDLE:{104:[4,1],208:[2,8],713:[7,13],1114:[11,14]},SADDLEREMAINDER:{1:4,2:8,4:1,7:13,8:2,11:14,13:7,14:11},LABELDISTANCE:2,LABELINCREASE:10,LABELMIN:3,LABELMAX:10,LABELOPTIMIZER:{EDGECOST:1,ANGLECOST:1,NEIGHBORCOST:5,SAMELEVELFACTOR:10,SAMELEVELDISTANCE:5,MAXCOST:100,INITIALSEARCHPOINTS:10,ITERATIONS:5}}},{}],947:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("./label_defaults"),o=t("../../components/color"),a=o.addOpacity,s=o.opacity,A=t("../../constants/filter_ops"),l=A.CONSTRAINT_REDUCTION,c=A.COMPARISON_OPS2;e.exports=function(t,e,n,o,A,u){var g,h,d,f=e.contours,p=n("contours.operation");f._operation=l[p],function(t,e){var n;-1===c.indexOf(e.operation)?(t("contours.value",[0,1]),Array.isArray(e.value)?e.value.length>2?e.value=e.value.slice(2):0===e.length?e.value=[0,1]:e.length<2?(n=parseFloat(e.value[0]),e.value=[n,n+1]):e.value=[parseFloat(e.value[0]),parseFloat(e.value[1])]:r(e.value)&&(n=parseFloat(e.value),e.value=[n,n+1])):(t("contours.value",0),r(e.value)||(Array.isArray(e.value)?e.value=parseFloat(e.value[0]):e.value=0))}(n,f),"="===p?g=f.showlines=!0:(g=n("contours.showlines"),d=n("fillcolor",a((t.line||{}).color||A,.5))),g&&(h=n("line.color",d&&s(d)?a(e.fillcolor,1):A),n("line.width",2),n("line.dash")),n("line.smoothing"),i(n,o,h,u)}},{"../../components/color":593,"../../constants/filter_ops":691,"./label_defaults":957,"fast-isnumeric":225}],948:[function(t,e,n){"use strict";var r=t("../../constants/filter_ops"),i=t("fast-isnumeric");function o(t,e){var n,o=Array.isArray(e);function a(t){return i(t)?+t:null}return-1!==r.COMPARISON_OPS2.indexOf(t)?n=a(o?e[0]:e):-1!==r.INTERVAL_OPS.indexOf(t)?n=o?[a(e[0]),a(e[1])]:[a(e),a(e)]:-1!==r.SET_OPS.indexOf(t)&&(n=o?e.map(a):[a(e)]),n}function a(t){return function(e){e=o(t,e);var n=Math.min(e[0],e[1]),r=Math.max(e[0],e[1]);return{start:n,end:r,size:r-n}}}function s(t){return function(e){return{start:e=o(t,e),end:1/0,size:1/0}}}e.exports={"[]":a("[]"),"][":a("]["),">":s(">"),"<":s("<"),"=":s("=")}},{"../../constants/filter_ops":691,"fast-isnumeric":225}],949:[function(t,e,n){"use strict";e.exports=function(t,e,n,r){var i=r("contours.start"),o=r("contours.end"),a=!1===i||!1===o,s=n("contours.size");!(a?e.autocontour=!0:n("autocontour",!1))&&s||n("ncontours")}},{}],950:[function(t,e,n){"use strict";var r=t("../../lib");function i(t){return r.extendFlat({},t,{edgepaths:r.extendDeep([],t.edgepaths),paths:r.extendDeep([],t.paths)})}e.exports=function(t,e){var n,o,a,s=function(t){return t.reverse()},A=function(t){return t};switch(e){case"=":case"<":return t;case">":for(1!==t.length&&r.warn("Contour data invalid for the specified inequality operation."),o=t[0],n=0;n<o.edgepaths.length;n++)o.edgepaths[n]=s(o.edgepaths[n]);for(n=0;n<o.paths.length;n++)o.paths[n]=s(o.paths[n]);return t;case"][":var l=s;s=A,A=l;case"[]":for(2!==t.length&&r.warn("Contour data invalid for the specified inequality range operation."),o=i(t[0]),a=i(t[1]),n=0;n<o.edgepaths.length;n++)o.edgepaths[n]=s(o.edgepaths[n]);for(n=0;n<o.paths.length;n++)o.paths[n]=s(o.paths[n]);for(;a.edgepaths.length;)o.edgepaths.push(A(a.edgepaths.shift()));for(;a.paths.length;)o.paths.push(A(a.paths.shift()));return[o]}}},{"../../lib":719}],951:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../heatmap/xyz_defaults"),o=t("./constraint_defaults"),a=t("./contours_defaults"),s=t("./style_defaults"),A=t("./attributes");e.exports=function(t,e,n,l){function c(n,i){return r.coerce(t,e,A,n,i)}if(i(t,e,c,l)){c("text"),c("hovertext"),c("hovertemplate");var u="constraint"===c("contours.type");c("connectgaps",r.isArray1D(e.z)),u?o(t,e,c,l,n):(a(t,e,c,function(n){return r.coerce2(t,e,A,n)}),s(t,e,c,l))}else e.visible=!1}},{"../../lib":719,"../heatmap/xyz_defaults":1015,"./attributes":942,"./constraint_defaults":947,"./contours_defaults":949,"./style_defaults":963}],952:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./constraint_mapping"),o=t("./end_plus");e.exports=function(t,e,n){for(var a="constraint"===t.type?i[t._operation](t.value):t,s=a.size,A=[],l=o(a),c=n.trace._carpetTrace,u=c?{xaxis:c.aaxis,yaxis:c.baxis,x:n.a,y:n.b}:{xaxis:e.xaxis,yaxis:e.yaxis,x:n.x,y:n.y},g=a.start;g<l;g+=s)if(A.push(r.extendFlat({level:g,crossings:{},starts:[],edgepaths:[],paths:[],z:n.z,smoothing:n.trace.line.smoothing},u)),A.length>1e3){r.warn("Too many contours, clipping at 1000",t);break}return A}},{"../../lib":719,"./constraint_mapping":948,"./end_plus":953}],953:[function(t,e,n){"use strict";e.exports=function(t){return t.end+t.size/1e6}},{}],954:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./constants");function o(t,e,n,r){return Math.abs(t[0]-e[0])<n&&Math.abs(t[1]-e[1])<r}function a(t,e,n,a,A){var l,c=e.join(","),u=c,g=t.crossings[u],h=function(t,e,n){var r=0,o=0;return t>20&&e?208===t||1114===t?r=0===n[0]?1:-1:o=0===n[1]?1:-1:-1!==i.BOTTOMSTART.indexOf(t)?o=1:-1!==i.LEFTSTART.indexOf(t)?r=1:-1!==i.TOPSTART.indexOf(t)?o=-1:r=-1,[r,o]}(g,n,e),d=[s(t,e,[-h[0],-h[1]])],f=h.join(","),p=t.z.length,I=t.z[0].length;for(l=0;l<1e4;l++){if(g>20?(g=i.CHOOSESADDLE[g][(h[0]||h[1])<0?0:1],t.crossings[u]=i.SADDLEREMAINDER[g]):delete t.crossings[u],!(h=i.NEWDELTA[g])){r.log("Found bad marching index:",g,e,t.level);break}d.push(s(t,e,h)),e[0]+=h[0],e[1]+=h[1],o(d[d.length-1],d[d.length-2],a,A)&&d.pop(),u=e.join(",");var C=h[0]&&(e[0]<0||e[0]>I-2)||h[1]&&(e[1]<0||e[1]>p-2);if(u===c&&h.join(",")===f||n&&C)break;g=t.crossings[u]}1e4===l&&r.log("Infinite loop in contour?");var m,y,E,v,B,b,Q,w,x,_,D,M,k,S,L,R=o(d[0],d[d.length-1],a,A),N=0,F=.2*t.smoothing,T=[],U=0;for(l=1;l<d.length;l++)M=d[l],k=d[l-1],S=M[2]-k[2],L=M[3]-k[3],N+=Q=Math.sqrt(S*S+L*L),T.push(Q);var O=N/T.length*F;function G(t){return d[t%d.length]}for(l=d.length-2;l>=U;l--)if((m=T[l])<O){for(E=0,y=l-1;y>=U&&m+T[y]<O;y--)m+=T[y];if(R&&l===d.length-2)for(E=0;E<y&&m+T[E]<O;E++)m+=T[E];B=l-y+E+1,b=Math.floor((l+y+E+2)/2),v=R||l!==d.length-2?R||-1!==y?B%2?G(b):[(G(b)[0]+G(b+1)[0])/2,(G(b)[1]+G(b+1)[1])/2]:d[0]:d[d.length-1],d.splice(y+1,l-y+1,v),l=y+1,E&&(U=E),R&&(l===d.length-2?d[E]=d[d.length-1]:0===l&&(d[d.length-1]=d[0]))}for(d.splice(0,U),l=0;l<d.length;l++)d[l].length=2;if(!(d.length<2))if(R)d.pop(),t.paths.push(d);else{n||r.log("Unclosed interior contour?",t.level,c,d.join("L"));var H=!1;for(w=0;w<t.edgepaths.length;w++)if(_=t.edgepaths[w],!H&&o(_[0],d[d.length-1],a,A)){d.pop(),H=!0;var j=!1;for(x=0;x<t.edgepaths.length;x++)if(o((D=t.edgepaths[x])[D.length-1],d[0],a,A)){j=!0,d.shift(),t.edgepaths.splice(w,1),x===w?t.paths.push(d.concat(D)):(x>w&&x--,t.edgepaths[x]=D.concat(d,_));break}j||(t.edgepaths[w]=d.concat(_))}for(w=0;w<t.edgepaths.length&&!H;w++)o((_=t.edgepaths[w])[_.length-1],d[0],a,A)&&(d.shift(),t.edgepaths[w]=_.concat(d),H=!0);H||t.edgepaths.push(d)}}function s(t,e,n){var r=e[0]+Math.max(n[0],0),i=e[1]+Math.max(n[1],0),o=t.z[i][r],a=t.xaxis,s=t.yaxis;if(n[1]){var A=(t.level-o)/(t.z[i][r+1]-o);return[a.c2p((1-A)*t.x[r]+A*t.x[r+1],!0),s.c2p(t.y[i],!0),r+A,i]}var l=(t.level-o)/(t.z[i+1][r]-o);return[a.c2p(t.x[r],!0),s.c2p((1-l)*t.y[i]+l*t.y[i+1],!0),r,i+l]}e.exports=function(t,e,n){var i,o,s,A;for(e=e||.01,n=n||.01,o=0;o<t.length;o++){for(s=t[o],A=0;A<s.starts.length;A++)a(s,s.starts[A],"edge",e,n);for(i=0;Object.keys(s.crossings).length&&i<1e4;)i++,a(s,Object.keys(s.crossings)[0].split(",").map(Number),void 0,e,n);1e4===i&&r.log("Infinite loop in contour?")}}},{"../../lib":719,"./constants":946}],955:[function(t,e,n){"use strict";var r=t("../../components/color"),i=t("../heatmap/hover");e.exports=function(t,e,n,o,a){var s=i(t,e,n,o,a,!0);return s&&s.forEach(function(t){var e=t.trace;"constraint"===e.contours.type&&(e.fillcolor&&r.opacity(e.fillcolor)?t.color=r.addOpacity(e.fillcolor,1):e.contours.showlines&&r.opacity(e.line.color)&&(t.color=r.addOpacity(e.line.color,1)))}),s}},{"../../components/color":593,"../heatmap/hover":1008}],956:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),calc:t("./calc"),plot:t("./plot").plot,style:t("./style"),colorbar:t("./colorbar"),hoverPoints:t("./hover"),moduleType:"trace",name:"contour",basePlotModule:t("../../plots/cartesian"),categories:["cartesian","svg","2dMap","contour","showLegend"],meta:{}}},{"../../plots/cartesian":778,"./attributes":942,"./calc":943,"./colorbar":945,"./defaults":951,"./hover":955,"./plot":960,"./style":962}],957:[function(t,e,n){"use strict";var r=t("../../lib");e.exports=function(t,e,n,i){if(i||(i={}),t("contours.showlabels")){var o=e.font;r.coerceFont(t,"contours.labelfont",{family:o.family,size:o.size,color:n}),t("contours.labelformat")}!1!==i.hasHover&&t("zhoverformat")}},{"../../lib":719}],958:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/colorscale"),o=t("./end_plus");e.exports=function(t){var e=t.contours,n=e.start,a=o(e),s=e.size||1,A=Math.floor((a-n)/s)+1,l="lines"===e.coloring?0:1,c=i.extractOpts(t);isFinite(s)||(s=1,A=1);var u,g,h=c.reversescale?i.flipScale(c.colorscale):c.colorscale,d=h.length,f=new Array(d),p=new Array(d);if("heatmap"===e.coloring){var I=c.min,C=c.max;for(g=0;g<d;g++)u=h[g],f[g]=u[0]*(C-I)+I,p[g]=u[1];var m=r.extent([I,C,e.start,e.start+s*(A-1)]),y=m[I<C?0:1],E=m[I<C?1:0];y!==I&&(f.splice(0,0,y),p.splice(0,0,p[0])),E!==C&&(f.push(E),p.push(p[p.length-1]))}else for(g=0;g<d;g++)u=h[g],f[g]=(u[0]*(A+l-1)-l/2)*s+n,p[g]=u[1];return i.makeColorScaleFunc({domain:f,range:p},{noNumericCheck:!0})}},{"../../components/colorscale":605,"./end_plus":953,d3:163}],959:[function(t,e,n){"use strict";var r=t("./constants");function i(t,e){var n=(e[0][0]>t?0:1)+(e[0][1]>t?0:2)+(e[1][1]>t?0:4)+(e[1][0]>t?0:8);return 5===n||10===n?t>(e[0][0]+e[0][1]+e[1][0]+e[1][1])/4?5===n?713:1114:5===n?104:208:15===n?0:n}e.exports=function(t){var e,n,o,a,s,A,l,c,u,g=t[0].z,h=g.length,d=g[0].length,f=2===h||2===d;for(n=0;n<h-1;n++)for(a=[],0===n&&(a=a.concat(r.BOTTOMSTART)),n===h-2&&(a=a.concat(r.TOPSTART)),e=0;e<d-1;e++)for(o=a.slice(),0===e&&(o=o.concat(r.LEFTSTART)),e===d-2&&(o=o.concat(r.RIGHTSTART)),s=e+","+n,A=[[g[n][e],g[n][e+1]],[g[n+1][e],g[n+1][e+1]]],u=0;u<t.length;u++)(l=i((c=t[u]).level,A))&&(c.crossings[s]=l,-1!==o.indexOf(l)&&(c.starts.push([e,n]),f&&-1!==o.indexOf(l,o.indexOf(l)+1)&&c.starts.push([e,n])))}},{"./constants":946}],960:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../lib"),o=t("../../components/drawing"),a=t("../../lib/svg_text_utils"),s=t("../../plots/cartesian/axes"),A=t("../../plots/cartesian/set_convert"),l=t("../heatmap/plot"),c=t("./make_crossings"),u=t("./find_all_paths"),g=t("./empty_pathinfo"),h=t("./convert_to_constraints"),d=t("./close_boundaries"),f=t("./constants"),p=f.LABELOPTIMIZER;function I(t,e){var n,r,a,s,A,l,c,u=function(t,e){var n=t.prefixBoundary;if(void 0===n){var r=Math.min(t.z[0][0],t.z[0][1]);n=!t.edgepaths.length&&r>t.level}return n?"M"+e.join("L")+"Z":""}(t,e),g=0,h=t.edgepaths.map(function(t,e){return e}),d=!0;function f(t){return Math.abs(t[1]-e[2][1])<.01}function p(t){return Math.abs(t[0]-e[0][0])<.01}function I(t){return Math.abs(t[0]-e[2][0])<.01}for(;h.length;){for(l=o.smoothopen(t.edgepaths[g],t.smoothing),u+=d?l:l.replace(/^M/,"L"),h.splice(h.indexOf(g),1),n=t.edgepaths[g][t.edgepaths[g].length-1],s=-1,a=0;a<4;a++){if(!n){i.log("Missing end?",g,t);break}for(c=n,Math.abs(c[1]-e[0][1])<.01&&!I(n)?r=e[1]:p(n)?r=e[0]:f(n)?r=e[3]:I(n)&&(r=e[2]),A=0;A<t.edgepaths.length;A++){var C=t.edgepaths[A][0];Math.abs(n[0]-r[0])<.01?Math.abs(n[0]-C[0])<.01&&(C[1]-n[1])*(r[1]-C[1])>=0&&(r=C,s=A):Math.abs(n[1]-r[1])<.01?Math.abs(n[1]-C[1])<.01&&(C[0]-n[0])*(r[0]-C[0])>=0&&(r=C,s=A):i.log("endpt to newendpt is not vert. or horz.",n,r,C)}if(n=r,s>=0)break;u+="L"+r}if(s===t.edgepaths.length){i.log("unclosed perimeter path");break}g=s,(d=-1===h.indexOf(g))&&(g=h[0],u+="Z")}for(g=0;g<t.paths.length;g++)u+=o.smoothclosed(t.paths[g],t.smoothing);return u}function C(t,e,n,r){var o=e.width/2,a=e.height/2,s=t.x,A=t.y,l=t.theta,c=Math.cos(l)*o,u=Math.sin(l)*o,g=(s>r.center?r.right-s:s-r.left)/(c+Math.abs(Math.sin(l)*a)),h=(A>r.middle?r.bottom-A:A-r.top)/(Math.abs(u)+Math.cos(l)*a);if(g<1||h<1)return 1/0;var d=p.EDGECOST*(1/(g-1)+1/(h-1));d+=p.ANGLECOST*l*l;for(var f=s-c,I=A-u,C=s+c,m=A+u,y=0;y<n.length;y++){var E=n[y],v=Math.cos(E.theta)*E.width/2,B=Math.sin(E.theta)*E.width/2,b=2*i.segmentDistance(f,I,C,m,E.x-v,E.y-B,E.x+v,E.y+B)/(e.height+E.height),Q=E.level===e.level,w=Q?p.SAMELEVELDISTANCE:1;if(b<=w)return 1/0;d+=p.NEIGHBORCOST*(Q?p.SAMELEVELFACTOR:1)/(b-w)}return d}n.plot=function(t,e,a,s){var A=e.xaxis,p=e.yaxis;i.makeTraceGroups(s,a,"contour").each(function(a){var s=r.select(this),C=a[0],m=C.trace,y=C.x,E=C.y,v=m.contours,B=g(v,e,C),b=i.ensureSingle(s,"g","heatmapcoloring"),Q=[];"heatmap"===v.coloring&&(Q=[a]),l(t,e,Q,b),c(B),u(B);var w=A.c2p(y[0],!0),x=A.c2p(y[y.length-1],!0),_=p.c2p(E[0],!0),D=p.c2p(E[E.length-1],!0),M=[[w,D],[x,D],[x,_],[w,_]],k=B;"constraint"===v.type&&(k=h(B,v._operation),d(k,v._operation,M,m)),function(t,e,n){var r=i.ensureSingle(t,"g","contourbg").selectAll("path").data("fill"===n.coloring?[0]:[]);r.enter().append("path"),r.exit().remove(),r.attr("d","M"+e.join("L")+"Z").style("stroke","none")}(s,M,v),function(t,e,n,o){var a=i.ensureSingle(t,"g","contourfill").selectAll("path").data("fill"===o.coloring||"constraint"===o.type&&"="!==o._operation?e:[]);a.enter().append("path"),a.exit().remove(),a.each(function(t){var e=I(t,n);e?r.select(this).attr("d",e).style("stroke","none"):r.select(this).remove()})}(s,k,M,v),function(t,e,a,s,A){var l=i.ensureSingle(t,"g","contourlines"),c=!1!==A.showlines,u=A.showlabels,g=c&&u,h=n.createLines(l,c||u,e),d=n.createLineClip(l,g,a,s.trace.uid),p=t.selectAll("g.contourlabels").data(u?[0]:[]);if(p.exit().remove(),p.enter().append("g").classed("contourlabels",!0),u){var I=[],C=[];i.clearLocationCache();var m=n.labelFormatter(A,s.t.cb,a._fullLayout),y=o.tester.append("text").attr("data-notex",1).call(o.font,A.labelfont),E=e[0].xaxis,v=e[0].yaxis,B=E._length,b=v._length,Q=E.range,w=v.range,x=i.aggNums(Math.min,null,s.x),_=i.aggNums(Math.max,null,s.x),D=i.aggNums(Math.min,null,s.y),M=i.aggNums(Math.max,null,s.y),k=Math.max(E.c2p(x,!0),0),S=Math.min(E.c2p(_,!0),B),L=Math.max(v.c2p(M,!0),0),R=Math.min(v.c2p(D,!0),b),N={};Q[0]<Q[1]?(N.left=k,N.right=S):(N.left=S,N.right=k),w[0]<w[1]?(N.top=L,N.bottom=R):(N.top=R,N.bottom=L),N.middle=(N.top+N.bottom)/2,N.center=(N.left+N.right)/2,I.push([[N.left,N.top],[N.right,N.top],[N.right,N.bottom],[N.left,N.bottom]]);var F=Math.sqrt(B*B+b*b),T=f.LABELDISTANCE*F/Math.max(1,e.length/f.LABELINCREASE);h.each(function(t){var e=n.calcTextOpts(t.level,m,y,a);r.select(this).selectAll("path").each(function(){var t=i.getVisibleSegment(this,N,e.height/2);if(t&&!(t.len<(e.width+e.height)*f.LABELMIN))for(var r=Math.min(Math.ceil(t.len/T),f.LABELMAX),o=0;o<r;o++){var a=n.findBestTextLocation(this,t,e,C,N);if(!a)break;n.addLabelData(a,e,C,I)}})}),y.remove(),n.drawLabels(p,C,a,d,g?I:null)}u&&!c&&h.remove()}(s,B,t,C,v),function(t,e,n,r,a){var s=n._fullLayout._clips,A="clip"+r.trace.uid,l=s.selectAll("#"+A).data(r.trace.connectgaps?[]:[0]);if(l.enter().append("clipPath").classed("contourclip",!0).attr("id",A),l.exit().remove(),!1===r.trace.connectgaps){var g={level:.9,crossings:{},starts:[],edgepaths:[],paths:[],xaxis:e.xaxis,yaxis:e.yaxis,x:r.x,y:r.y,z:function(t){var e,n,r=t.trace._emptypoints,i=[],o=t.z.length,a=t.z[0].length,s=[];for(e=0;e<a;e++)s.push(1);for(e=0;e<o;e++)i.push(s.slice());for(e=0;e<r.length;e++)n=r[e],i[n[0]][n[1]]=0;return t.zmask=i,i}(r),smoothing:0};c([g]),u([g]);var h=I(g,a),d=i.ensureSingle(l,"path","");d.attr("d",h)}else A=null;o.setClipUrl(t,A,n)}(s,e,t,C,M)})},n.createLines=function(t,e,n){var r=n[0].smoothing,i=t.selectAll("g.contourlevel").data(e?n:[]);if(i.exit().remove(),i.enter().append("g").classed("contourlevel",!0),e){var a=i.selectAll("path.openline").data(function(t){return t.pedgepaths||t.edgepaths});a.exit().remove(),a.enter().append("path").classed("openline",!0),a.attr("d",function(t){return o.smoothopen(t,r)}).style("stroke-miterlimit",1).style("vector-effect","non-scaling-stroke");var s=i.selectAll("path.closedline").data(function(t){return t.ppaths||t.paths});s.exit().remove(),s.enter().append("path").classed("closedline",!0),s.attr("d",function(t){return o.smoothclosed(t,r)}).style("stroke-miterlimit",1).style("vector-effect","non-scaling-stroke")}return i},n.createLineClip=function(t,e,n,r){var i=e?"clipline"+r:null,a=n._fullLayout._clips.selectAll("#"+i).data(e?[0]:[]);return a.exit().remove(),a.enter().append("clipPath").classed("contourlineclip",!0).attr("id",i),o.setClipUrl(t,i,n),a},n.labelFormatter=function(t,e,n){if(t.labelformat)return n._d3locale.numberFormat(t.labelformat);var r;if(e)r=e.axis;else{if(r={type:"linear",_id:"ycontour",showexponent:"all",exponentformat:"B"},"constraint"===t.type){var i=t.value;Array.isArray(i)?r.range=[i[0],i[i.length-1]]:r.range=[i,i]}else r.range=[t.start,t.end],r.nticks=(t.end-t.start)/t.size;r.range[0]===r.range[1]&&(r.range[1]+=r.range[0]||1),r.nticks||(r.nticks=1e3),A(r,n),s.prepTicks(r),r._tmin=null,r._tmax=null}return function(t){return s.tickText(r,t).text}},n.calcTextOpts=function(t,e,n,r){var i=e(t);n.text(i).call(a.convertToTspans,r);var s=o.bBox(n.node(),!0);return{text:i,width:s.width,height:s.height,level:t,dy:(s.top+s.bottom)/2}},n.findBestTextLocation=function(t,e,n,r,o){var a,s,A,l,c,u=n.width;e.isClosed?(s=e.len/p.INITIALSEARCHPOINTS,a=e.min+s/2,A=e.max):(s=(e.len-u)/(p.INITIALSEARCHPOINTS+1),a=e.min+s+u/2,A=e.max-(s+u)/2);for(var g=1/0,h=0;h<p.ITERATIONS;h++){for(var d=a;d<A;d+=s){var f=i.getTextLocation(t,e.total,d,u),I=C(f,n,r,o);I<g&&(g=I,c=f,l=d)}if(g>2*p.MAXCOST)break;h&&(s/=2),A=(a=l-s/2)+1.5*s}if(g<=p.MAXCOST)return c},n.addLabelData=function(t,e,n,r){var i=e.width/2,o=e.height/2,a=t.x,s=t.y,A=t.theta,l=Math.sin(A),c=Math.cos(A),u=i*c,g=o*l,h=i*l,d=-o*c,f=[[a-u-g,s-h-d],[a+u-g,s+h-d],[a+u+g,s+h+d],[a-u+g,s-h+d]];n.push({text:e.text,x:a,y:s,dy:e.dy,theta:A,level:e.level,width:e.width,height:e.height}),r.push(f)},n.drawLabels=function(t,e,n,o,s){var A=t.selectAll("text").data(e,function(t){return t.text+","+t.x+","+t.y+","+t.theta});if(A.exit().remove(),A.enter().append("text").attr({"data-notex":1,"text-anchor":"middle"}).each(function(t){var e=t.x+Math.sin(t.theta)*t.dy,i=t.y-Math.cos(t.theta)*t.dy;r.select(this).text(t.text).attr({x:e,y:i,transform:"rotate("+180*t.theta/Math.PI+" "+e+" "+i+")"}).call(a.convertToTspans,n)}),s){for(var l="",c=0;c<s.length;c++)l+="M"+s[c].join("L")+"Z";i.ensureSingle(o,"path","").attr("d",l)}}},{"../../components/drawing":614,"../../lib":719,"../../lib/svg_text_utils":743,"../../plots/cartesian/axes":767,"../../plots/cartesian/set_convert":785,"../heatmap/plot":1012,"./close_boundaries":944,"./constants":946,"./convert_to_constraints":950,"./empty_pathinfo":952,"./find_all_paths":954,"./make_crossings":959,d3:163}],961:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axes"),i=t("../../lib");function o(t,e,n){var i={type:"linear",range:[t,e]};return r.autoTicks(i,(e-t)/(n||15)),i}e.exports=function(t,e){var n=t.contours;if(t.autocontour){var a=t.zmin,s=t.zmax;(t.zauto||void 0===a)&&(a=i.aggNums(Math.min,null,e)),(t.zauto||void 0===s)&&(s=i.aggNums(Math.max,null,e));var A=o(a,s,t.ncontours);n.size=A.dtick,n.start=r.tickFirst(A),A.range.reverse(),n.end=r.tickFirst(A),n.start===a&&(n.start+=n.size),n.end===s&&(n.end-=n.size),n.start>n.end&&(n.start=n.end=(n.start+n.end)/2),t._input.contours||(t._input.contours={}),i.extendFlat(t._input.contours,{start:n.start,end:n.end,size:n.size}),t._input.autocontour=!0}else if("constraint"!==n.type){var l,c=n.start,u=n.end,g=t._input.contours;c>u&&(n.start=g.start=u,u=n.end=g.end=c,c=n.start),n.size>0||(l=c===u?1:o(c,u,t.ncontours).dtick,g.size=n.size=l)}}},{"../../lib":719,"../../plots/cartesian/axes":767}],962:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/drawing"),o=t("../heatmap/style"),a=t("./make_color_map");e.exports=function(t){var e=r.select(t).selectAll("g.contour");e.style("opacity",function(t){return t[0].trace.opacity}),e.each(function(t){var e=r.select(this),n=t[0].trace,o=n.contours,s=n.line,A=o.size||1,l=o.start,c="constraint"===o.type,u=!c&&"lines"===o.coloring,g=!c&&"fill"===o.coloring,h=u||g?a(n):null;e.selectAll("g.contourlevel").each(function(t){r.select(this).selectAll("path").call(i.lineGroupStyle,s.width,u?h(t.level):s.color,s.dash)});var d=o.labelfont;if(e.selectAll("g.contourlabels text").each(function(t){i.font(r.select(this),{family:d.family,size:d.size,color:d.color||(u?h(t.level):s.color)})}),c)e.selectAll("g.contourfill path").style("fill",n.fillcolor);else if(g){var f;e.selectAll("g.contourfill path").style("fill",function(t){return void 0===f&&(f=t.level),h(t.level+.5*A)}),void 0===f&&(f=l),e.selectAll("g.contourbg path").style("fill",h(f-.5*A))}}),o(t)}},{"../../components/drawing":614,"../heatmap/style":1013,"./make_color_map":958,d3:163}],963:[function(t,e,n){"use strict";var r=t("../../components/colorscale/defaults"),i=t("./label_defaults");e.exports=function(t,e,n,o,a){var s,A=n("contours.coloring"),l="";"fill"===A&&(s=n("contours.showlines")),!1!==s&&("lines"!==A&&(l=n("line.color","#000")),n("line.width",.5),n("line.dash")),"none"!==A&&(!0!==t.showlegend&&(e.showlegend=!1),e._dfltShowLegend=!1,r(t,e,o,n,{prefix:"",cLetter:"z"})),n("line.smoothing"),i(n,o,l,a)}},{"../../components/colorscale/defaults":603,"./label_defaults":957}],964:[function(t,e,n){"use strict";var r=t("../heatmap/attributes"),i=t("../contour/attributes"),o=i.contours,a=t("../scatter/attributes"),s=t("../../components/colorscale/attributes"),A=t("../../lib/extend").extendFlat,l=a.line;e.exports=A({carpet:{valType:"string",editType:"calc"},z:r.z,a:r.x,a0:r.x0,da:r.dx,b:r.y,b0:r.y0,db:r.dy,text:r.text,hovertext:r.hovertext,transpose:r.transpose,atype:r.xtype,btype:r.ytype,fillcolor:i.fillcolor,autocontour:i.autocontour,ncontours:i.ncontours,contours:{type:o.type,start:o.start,end:o.end,size:o.size,coloring:{valType:"enumerated",values:["fill","lines","none"],dflt:"fill",editType:"calc"},showlines:o.showlines,showlabels:o.showlabels,labelfont:o.labelfont,labelformat:o.labelformat,operation:o.operation,value:o.value,editType:"calc",impliedEdits:{autocontour:!1}},line:{color:A({},l.color,{}),width:l.width,dash:l.dash,smoothing:A({},l.smoothing,{}),editType:"plot"},transforms:void 0},s("",{cLetter:"z",autoColorDflt:!1}))},{"../../components/colorscale/attributes":600,"../../lib/extend":710,"../contour/attributes":942,"../heatmap/attributes":1001,"../scatter/attributes":1112}],965:[function(t,e,n){"use strict";var r=t("../../components/colorscale/calc"),i=t("../../lib"),o=t("../heatmap/convert_column_xyz"),a=t("../heatmap/clean_2d_array"),s=t("../heatmap/interp2d"),A=t("../heatmap/find_empties"),l=t("../heatmap/make_bound_array"),c=t("./defaults"),u=t("../carpet/lookup_carpetid"),g=t("../contour/set_contours");e.exports=function(t,e){var n=e._carpetTrace=u(t,e);if(n&&n.visible&&"legendonly"!==n.visible){if(!e.a||!e.b){var h=t.data[n.index],d=t.data[e.index];d.a||(d.a=h.a),d.b||(d.b=h.b),c(d,e,e._defaultColor,t._fullLayout)}var f=function(t,e){var n,c,u,g,h,d,f,p=e._carpetTrace,I=p.aaxis,C=p.baxis;I._minDtick=0,C._minDtick=0,i.isArray1D(e.z)&&o(e,I,C,"a","b",["z"]),n=e._a=e._a||e.a,g=e._b=e._b||e.b,n=n?I.makeCalcdata(e,"_a"):[],g=g?C.makeCalcdata(e,"_b"):[],c=e.a0||0,u=e.da||1,h=e.b0||0,d=e.db||1,f=e._z=a(e._z||e.z,e.transpose),e._emptypoints=A(f),s(f,e._emptypoints);var m=i.maxRowLength(f),y="scaled"===e.xtype?"":n,E=l(e,y,c,u,m,I),v="scaled"===e.ytype?"":g,B=l(e,v,h,d,f.length,C),b={a:E,b:B,z:f};return"levels"===e.contours.type&&"none"!==e.contours.coloring&&r(t,e,{vals:f,containerStr:"",cLetter:"z"}),[b]}(t,e);return g(e,e._z),f}}},{"../../components/colorscale/calc":601,"../../lib":719,"../carpet/lookup_carpetid":915,"../contour/set_contours":961,"../heatmap/clean_2d_array":1003,"../heatmap/convert_column_xyz":1005,"../heatmap/find_empties":1007,"../heatmap/interp2d":1010,"../heatmap/make_bound_array":1011,"./defaults":966}],966:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../heatmap/xyz_defaults"),o=t("./attributes"),a=t("../contour/constraint_defaults"),s=t("../contour/contours_defaults"),A=t("../contour/style_defaults");e.exports=function(t,e,n,l){function c(n,i){return r.coerce(t,e,o,n,i)}if(c("carpet"),t.a&&t.b){if(!i(t,e,c,l,"a","b"))return void(e.visible=!1);c("text"),"constraint"===c("contours.type")?a(t,e,c,l,n,{hasHover:!1}):(s(t,e,c,function(n){return r.coerce2(t,e,o,n)}),A(t,e,c,l,{hasHover:!1}))}else e._defaultColor=n,e._length=null}},{"../../lib":719,"../contour/constraint_defaults":947,"../contour/contours_defaults":949,"../contour/style_defaults":963,"../heatmap/xyz_defaults":1015,"./attributes":964}],967:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:t("../contour/colorbar"),calc:t("./calc"),plot:t("./plot"),style:t("../contour/style"),moduleType:"trace",name:"contourcarpet",basePlotModule:t("../../plots/cartesian"),categories:["cartesian","svg","carpet","contour","symbols","showLegend","hasLines","carpetDependent"],meta:{}}},{"../../plots/cartesian":778,"../contour/colorbar":945,"../contour/style":962,"./attributes":964,"./calc":965,"./defaults":966,"./plot":970}],968:[function(t,e,n){"use strict";var r=t("../../components/drawing"),i=t("../carpet/axis_aligned_line"),o=t("../../lib");e.exports=function(t,e,n,a,s,A,l,c){var u,g,h,d,f,p,I,C="",m=e.edgepaths.map(function(t,e){return e}),y=!0,E=1e-4*Math.abs(n[0][0]-n[2][0]),v=1e-4*Math.abs(n[0][1]-n[2][1]);function B(t){return Math.abs(t[1]-n[0][1])<v}function b(t){return Math.abs(t[1]-n[2][1])<v}function Q(t){return Math.abs(t[0]-n[0][0])<E}function w(t){return Math.abs(t[0]-n[2][0])<E}function x(t,e){var n,r,o,a,u="";for(B(t)&&!w(t)||b(t)&&!Q(t)?(a=s.aaxis,o=i(s,A,[t[0],e[0]],.5*(t[1]+e[1]))):(a=s.baxis,o=i(s,A,.5*(t[0]+e[0]),[t[1],e[1]])),n=1;n<o.length;n++)for(u+=a.smoothing?"C":"L",r=0;r<o[n].length;r++){var g=o[n][r];u+=[l.c2p(g[0]),c.c2p(g[1])]+" "}return u}for(u=0,g=null;m.length;){var _=e.edgepaths[u][0];for(g&&(C+=x(g,_)),I=r.smoothopen(e.edgepaths[u].map(a),e.smoothing),C+=y?I:I.replace(/^M/,"L"),m.splice(m.indexOf(u),1),g=e.edgepaths[u][e.edgepaths[u].length-1],f=-1,d=0;d<4;d++){if(!g){o.log("Missing end?",u,e);break}for(B(g)&&!w(g)?h=n[1]:Q(g)?h=n[0]:b(g)?h=n[3]:w(g)&&(h=n[2]),p=0;p<e.edgepaths.length;p++){var D=e.edgepaths[p][0];Math.abs(g[0]-h[0])<E?Math.abs(g[0]-D[0])<E&&(D[1]-g[1])*(h[1]-D[1])>=0&&(h=D,f=p):Math.abs(g[1]-h[1])<v?Math.abs(g[1]-D[1])<v&&(D[0]-g[0])*(h[0]-D[0])>=0&&(h=D,f=p):o.log("endpt to newendpt is not vert. or horz.",g,h,D)}if(f>=0)break;C+=x(g,h),g=h}if(f===e.edgepaths.length){o.log("unclosed perimeter path");break}u=f,(y=-1===m.indexOf(u))&&(u=m[0],C+=x(g,h)+"Z",g=null)}for(u=0;u<e.paths.length;u++)C+=r.smoothclosed(e.paths[u].map(a),e.smoothing);return C}},{"../../components/drawing":614,"../../lib":719,"../carpet/axis_aligned_line":899}],969:[function(t,e,n){"use strict";e.exports=function(t,e){var n,r,i,o,a,s,A,l,c;for(n=0;n<t.length;n++){for(a=(o=t[n]).pedgepaths=[],s=o.ppaths=[],r=0;r<o.edgepaths.length;r++){for(c=o.edgepaths[r],A=[],i=0;i<c.length;i++)A[i]=e(c[i]);a.push(A)}for(r=0;r<o.paths.length;r++){for(c=o.paths[r],l=[],i=0;i<c.length;i++)l[i]=e(c[i]);s.push(l)}}}},{}],970:[function(t,e,n){"use strict";var r=t("d3"),i=t("../carpet/map_1d_array"),o=t("../carpet/makepath"),a=t("../../components/drawing"),s=t("../../lib"),A=t("../contour/make_crossings"),l=t("../contour/find_all_paths"),c=t("../contour/plot"),u=t("../contour/constants"),g=t("../contour/convert_to_constraints"),h=t("./join_all_paths"),d=t("../contour/empty_pathinfo"),f=t("./map_pathinfo"),p=t("../carpet/lookup_carpetid"),I=t("../contour/close_boundaries");function C(t,e,n){var r=t.getPointAtLength(e),i=t.getPointAtLength(n),o=i.x-r.x,a=i.y-r.y,s=Math.sqrt(o*o+a*a);return[o/s,a/s]}function m(t){var e=Math.sqrt(t[0]*t[0]+t[1]*t[1]);return[t[0]/e,t[1]/e]}function y(t,e){var n=Math.abs(t[0]*e[0]+t[1]*e[1]);return Math.sqrt(1-n*n)/n}e.exports=function(t,e,n,E){var v=e.xaxis,B=e.yaxis;s.makeTraceGroups(E,n,"contour").each(function(n){var E=r.select(this),b=n[0],Q=b.trace,w=Q._carpetTrace=p(t,Q),x=t.calcdata[w.index][0];if(w.visible&&"legendonly"!==w.visible){var _=b.a,D=b.b,M=Q.contours,k=d(M,e,b),S="constraint"===M.type,L=M._operation,R=S?"="===L?"lines":"fill":M.coloring,N=[[_[0],D[D.length-1]],[_[_.length-1],D[D.length-1]],[_[_.length-1],D[0]],[_[0],D[0]]];A(k);var F=1e-8*(_[_.length-1]-_[0]),T=1e-8*(D[D.length-1]-D[0]);l(k,F,T);var U,O,G,H,j=k;"constraint"===M.type&&(j=g(k,L),I(j,L,N,Q)),f(k,K);var Y=[];for(H=x.clipsegments.length-1;H>=0;H--)U=x.clipsegments[H],O=i([],U.x,v.c2p),G=i([],U.y,B.c2p),O.reverse(),G.reverse(),Y.push(o(O,G,U.bicubic));var P="M"+Y.join("L")+"Z";!function(t,e,n,r,a,A){var l,c,u,g,h=s.ensureSingle(t,"g","contourbg").selectAll("path").data("fill"!==A||a?[]:[0]);h.enter().append("path"),h.exit().remove();var d=[];for(g=0;g<e.length;g++)l=e[g],c=i([],l.x,n.c2p),u=i([],l.y,r.c2p),d.push(o(c,u,l.bicubic));h.attr("d","M"+d.join("L")+"Z").style("stroke","none")}(E,x.clipsegments,v,B,S,R),function(t,e,n,i,o,a,A,l,c,u,g){var d=s.ensureSingle(e,"g","contourfill").selectAll("path").data("fill"===u?o:[]);d.enter().append("path"),d.exit().remove(),d.each(function(e){var o=h(t,e,a,A,l,c,n,i);e.prefixBoundary&&(o=g+o),o?r.select(this).attr("d",o).style("stroke","none"):r.select(this).remove()})}(Q,E,v,B,j,N,K,w,x,R,P),function(t,e,n,i,o,A,l){var g=s.ensureSingle(t,"g","contourlines"),h=!1!==o.showlines,d=o.showlabels,f=h&&d,p=c.createLines(g,h||d,e),I=c.createLineClip(g,f,n,i.trace.uid),E=t.selectAll("g.contourlabels").data(d?[0]:[]);if(E.exit().remove(),E.enter().append("g").classed("contourlabels",!0),d){var v=A.xaxis,B=A.yaxis,b=v._length,Q=B._length,w=[[[0,0],[b,0],[b,Q],[0,Q]]],x=[];s.clearLocationCache();var _=c.labelFormatter(o,i.t.cb,n._fullLayout),D=a.tester.append("text").attr("data-notex",1).call(a.font,o.labelfont),M={left:0,right:b,center:b/2,top:0,bottom:Q,middle:Q/2},k=Math.sqrt(b*b+Q*Q),S=u.LABELDISTANCE*k/Math.max(1,e.length/u.LABELINCREASE);p.each(function(t){var e=c.calcTextOpts(t.level,_,D,n);r.select(this).selectAll("path").each(function(n){var r=s.getVisibleSegment(this,M,e.height/2);if(r&&(function(t,e,n,r,i,o){for(var a,s=0;s<n.pedgepaths.length;s++)e===n.pedgepaths[s]&&(a=n.edgepaths[s]);if(a){var A=i.a[0],l=i.a[i.a.length-1],c=i.b[0],u=i.b[i.b.length-1],g=C(t,0,1),h=C(t,r.total,r.total-1),d=p(a[0],g),f=r.total-p(a[a.length-1],h);r.min<d&&(r.min=d),r.max>f&&(r.max=f),r.len=r.max-r.min}function p(t,e){var n,r=0;return(Math.abs(t[0]-A)<.1||Math.abs(t[0]-l)<.1)&&(n=m(i.dxydb_rough(t[0],t[1],.1)),r=Math.max(r,o*y(e,n)/2)),(Math.abs(t[1]-c)<.1||Math.abs(t[1]-u)<.1)&&(n=m(i.dxyda_rough(t[0],t[1],.1)),r=Math.max(r,o*y(e,n)/2)),r}}(this,n,t,r,l,e.height),!(r.len<(e.width+e.height)*u.LABELMIN)))for(var i=Math.min(Math.ceil(r.len/S),u.LABELMAX),o=0;o<i;o++){var a=c.findBestTextLocation(this,r,e,x,M);if(!a)break;c.addLabelData(a,e,x,w)}})}),D.remove(),c.drawLabels(E,x,n,I,f?w:null)}d&&!h&&p.remove()}(E,k,t,b,M,e,w),a.setClipUrl(E,w._clipPathId,t)}function K(t){var e=w.ab2xy(t[0],t[1],!0);return[v.c2p(e[0]),B.c2p(e[1])]}})}},{"../../components/drawing":614,"../../lib":719,"../carpet/lookup_carpetid":915,"../carpet/makepath":916,"../carpet/map_1d_array":917,"../contour/close_boundaries":944,"../contour/constants":946,"../contour/convert_to_constraints":950,"../contour/empty_pathinfo":952,"../contour/find_all_paths":954,"../contour/make_crossings":959,"../contour/plot":960,"./join_all_paths":968,"./map_pathinfo":969,d3:163}],971:[function(t,e,n){"use strict";var r=t("../../components/colorscale/attributes"),i=t("../../components/fx/hovertemplate_attributes"),o=t("../../plots/attributes"),a=t("../scattermapbox/attributes"),s=t("../../lib/extend").extendFlat;e.exports=s({lon:a.lon,lat:a.lat,z:{valType:"data_array",editType:"calc"},radius:{valType:"number",editType:"plot",arrayOk:!0,min:1,dflt:30},below:{valType:"string",editType:"plot"},text:a.text,hovertext:a.hovertext,hoverinfo:s({},o.hoverinfo,{flags:["lon","lat","z","text","name"]}),hovertemplate:i()},r("",{cLetter:"z",editTypeOverride:"calc"}))},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../scattermapbox/attributes":1171}],972:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib").isArrayOrTypedArray,o=t("../../constants/numerical").BADNUM,a=t("../../components/colorscale/calc"),s=t("../../lib")._;e.exports=function(t,e){for(var n=e._length,A=new Array(n),l=e.z,c=i(l)&&l.length,u=0;u<n;u++){var g=A[u]={},h=e.lon[u],d=e.lat[u];if(g.lonlat=r(h)&&r(d)?[+h,+d]:[o,o],c){var f=l[u];g.z=r(f)?f:o}}return a(t,e,{vals:c?l:[0,1],containerStr:"",cLetter:"z"}),n&&(A[0].t={labels:{lat:s(t,"lat:")+" ",lon:s(t,"lon:")+" "}}),A}},{"../../components/colorscale/calc":601,"../../constants/numerical":695,"../../lib":719,"fast-isnumeric":225}],973:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib"),o=t("../../components/color"),a=t("../../components/colorscale"),s=t("../../constants/numerical").BADNUM,A=t("../../lib/geojson_utils").makeBlank;e.exports=function(t){var e=t[0].trace,n=!0===e.visible&&0!==e._length,l=e._opts={heatmap:{layout:{visibility:"none"},paint:{}},geojson:A()};if(!n)return l;var c,u=[],g=e.z,h=e.radius,d=i.isArrayOrTypedArray(g)&&g.length,f=i.isArrayOrTypedArray(h);for(c=0;c<t.length;c++){var p=t[c],I=p.lonlat;if(I[0]!==s){var C={};if(d){var m=p.z;C.z=m!==s?m:0}f&&(C.r=r(h[c])&&h[c]>0?+h[c]:0),u.push({type:"Feature",geometry:{type:"Point",coordinates:I},properties:C})}}var y=a.extractOpts(e),E=y.reversescale?a.flipScale(y.colorscale):y.colorscale,v=E[0][1],B=["interpolate",["linear"],["heatmap-density"],0,o.opacity(v)<1?v:o.addOpacity(v,0)];for(c=1;c<E.length;c++)B.push(E[c][0],E[c][1]);var b=["interpolate",["linear"],["get","z"],y.min,0,y.max,1];return i.extendFlat(l.heatmap.paint,{"heatmap-weight":d?b:1/(y.max-y.min),"heatmap-color":B,"heatmap-radius":f?{type:"identity",property:"r"}:e.radius,"heatmap-opacity":e.opacity}),l.geojson={type:"FeatureCollection",features:u},l.heatmap.layout.visibility="visible",l}},{"../../components/color":593,"../../components/colorscale":605,"../../constants/numerical":695,"../../lib":719,"../../lib/geojson_utils":714,"fast-isnumeric":225}],974:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/colorscale/defaults"),o=t("./attributes");e.exports=function(t,e,n,a){function s(n,i){return r.coerce(t,e,o,n,i)}var A=s("lon")||[],l=s("lat")||[],c=Math.min(A.length,l.length);c?(e._length=c,s("z"),s("radius"),s("below"),s("text"),s("hovertext"),s("hovertemplate"),i(t,e,a,s,{prefix:"",cLetter:"z"})):e.visible=!1}},{"../../components/colorscale/defaults":603,"../../lib":719,"./attributes":971}],975:[function(t,e,n){"use strict";e.exports=function(t,e){return t.lon=e.lon,t.lat=e.lat,t.z=e.z,t}},{}],976:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axes"),o=t("../scattermapbox/hover");e.exports=function(t,e,n){var a=o(t,e,n);if(a){var s=a[0],A=s.cd,l=A[0].trace,c=A[s.index];if(delete s.color,"z"in c){var u=s.subplot.mockAxis;s.z=c.z,s.zLabel=i.tickText(u,u.c2l(c.z),"hover").text}return s.extraText=function(t,e,n){if(!t.hovertemplate){var i=(e.hi||t.hoverinfo).split("+"),o=-1!==i.indexOf("all"),a=-1!==i.indexOf("lon"),s=-1!==i.indexOf("lat"),A=e.lonlat,l=[];return o||a&&s?l.push("("+c(A[0])+", "+c(A[1])+")"):a?l.push(n.lon+c(A[0])):s&&l.push(n.lat+c(A[1])),(o||-1!==i.indexOf("text"))&&r.fillText(e,t,l),l.join("<br>")}function c(t){return t+"°"}}(l,c,A[0].t.labels),[s]}}},{"../../lib":719,"../../plots/cartesian/axes":767,"../scattermapbox/hover":1175}],977:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:t("../heatmap/colorbar"),calc:t("./calc"),plot:t("./plot"),hoverPoints:t("./hover"),eventData:t("./event_data"),getBelow:function(t,e){for(var n=e.getMapLayers(),r=0;r<n.length;r++){var i=n[r],o=i.id;if("symbol"===i.type&&"string"==typeof o&&-1===o.indexOf("plotly-"))return o}},moduleType:"trace",name:"densitymapbox",basePlotModule:t("../../plots/mapbox"),categories:["mapbox","gl"],meta:{hr_name:"density_mapbox"}}},{"../../plots/mapbox":822,"../heatmap/colorbar":1004,"./attributes":971,"./calc":972,"./defaults":974,"./event_data":975,"./hover":976,"./plot":978}],978:[function(t,e,n){"use strict";var r=t("./convert"),i=t("../../plots/mapbox/constants").traceLayerPrefix;function o(t,e){this.subplot=t,this.uid=e,this.sourceId="source-"+e,this.layerList=[["heatmap",i+e+"-heatmap"]],this.below=null}var a=o.prototype;a.update=function(t){var e=this.subplot,n=this.layerList,i=r(t),o=e.belowLookup["trace-"+this.uid];e.map.getSource(this.sourceId).setData(i.geojson),o!==this.below&&(this._removeLayers(),this._addLayers(i,o),this.below=o);for(var a=0;a<n.length;a++){var s=n[a],A=s[0],l=s[1],c=i[A];e.setOptions(l,"setLayoutProperty",c.layout),"visible"===c.layout.visibility&&e.setOptions(l,"setPaintProperty",c.paint)}},a._addLayers=function(t,e){for(var n=this.subplot,r=this.layerList,i=this.sourceId,o=0;o<r.length;o++){var a=r[o],s=a[0],A=t[s];n.addLayer({type:s,id:a[1],source:i,layout:A.layout,paint:A.paint},e)}},a._removeLayers=function(){for(var t=this.subplot.map,e=this.layerList,n=e.length-1;n>=0;n--)t.removeLayer(e[n][1])},a.dispose=function(){var t=this.subplot.map;this._removeLayers(),t.removeSource(this.sourceId)},e.exports=function(t,e){var n=e[0].trace,i=new o(t,n.uid),a=i.sourceId,s=r(e),A=i.below=t.belowLookup["trace-"+n.uid];return t.map.addSource(a,{type:"geojson",data:s.geojson}),i._addLayers(s,A),i}},{"../../plots/mapbox/constants":820,"./convert":973}],979:[function(t,e,n){"use strict";var r=t("../../lib");e.exports=function(t,e){for(var n=0;n<t.length;n++)t[n].i=n;r.mergeArray(e.text,t,"tx"),r.mergeArray(e.hovertext,t,"htx");var i=e.marker;if(i){r.mergeArray(i.opacity,t,"mo"),r.mergeArray(i.color,t,"mc");var o=i.line;o&&(r.mergeArray(o.color,t,"mlc"),r.mergeArrayCastPositive(o.width,t,"mlw"))}}},{"../../lib":719}],980:[function(t,e,n){"use strict";var r=t("../bar/attributes"),i=t("../scatter/attributes").line,o=t("../../plots/attributes"),a=t("../../components/fx/hovertemplate_attributes"),s=t("./constants"),A=t("../../lib/extend").extendFlat,l=t("../../components/color");e.exports={x:r.x,x0:r.x0,dx:r.dx,y:r.y,y0:r.y0,dy:r.dy,hovertext:r.hovertext,hovertemplate:a({},{keys:s.eventDataKeys}),hoverinfo:A({},o.hoverinfo,{flags:["name","x","y","text","percent initial","percent previous","percent total"]}),textinfo:{valType:"flaglist",flags:["label","text","percent initial","percent previous","percent total","value"],extras:["none"],editType:"plot",arrayOk:!1},text:r.text,textposition:A({},r.textposition,{dflt:"auto"}),insidetextanchor:A({},r.insidetextanchor,{dflt:"middle"}),textangle:A({},r.textangle,{dflt:0}),textfont:r.textfont,insidetextfont:r.insidetextfont,outsidetextfont:r.outsidetextfont,constraintext:r.constraintext,cliponaxis:r.cliponaxis,orientation:A({},r.orientation,{}),offset:A({},r.offset,{arrayOk:!1}),width:A({},r.width,{arrayOk:!1}),marker:r.marker,connector:{fillcolor:{valType:"color",editType:"style"},line:{color:A({},i.color,{dflt:l.defaultLine}),width:A({},i.width,{dflt:0,editType:"plot"}),dash:i.dash,editType:"style"},visible:{valType:"boolean",dflt:!0,editType:"plot"},editType:"plot"},offsetgroup:r.offsetgroup,alignmentgroup:r.alignmentgroup}},{"../../components/color":593,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../bar/attributes":857,"../scatter/attributes":1112,"./constants":982}],981:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axes"),i=t("./arrays_to_calcdata"),o=t("../scatter/calc_selection"),a=t("../../constants/numerical").BADNUM;function s(t){return t===a?0:t}e.exports=function(t,e){var n,A,l,c,u=r.getFromId(t,e.xaxis||"x"),g=r.getFromId(t,e.yaxis||"y");"h"===e.orientation?(n=u.makeCalcdata(e,"x"),A=g.makeCalcdata(e,"y")):(n=g.makeCalcdata(e,"y"),A=u.makeCalcdata(e,"x"));var h,d=Math.min(A.length,n.length),f=new Array(d);for(e._base=[],l=0;l<d;l++){n[l]<0&&(n[l]=a);var p=!1;n[l]!==a&&l+1<d&&n[l+1]!==a&&(p=!0),c=f[l]={p:A[l],s:n[l],cNext:p},e._base[l]=-.5*c.s,e.ids&&(c.id=String(e.ids[l])),0===l&&(f[0].vTotal=0),f[0].vTotal+=s(c.s),c.begR=s(c.s)/s(f[0].s)}for(l=0;l<d;l++)(c=f[l]).s!==a&&(c.sumR=c.s/f[0].vTotal,c.difR=void 0!==h?c.s/h:1,h=c.s);return i(f,e),o(f,e),f}},{"../../constants/numerical":695,"../../plots/cartesian/axes":767,"../scatter/calc_selection":1114,"./arrays_to_calcdata":979}],982:[function(t,e,n){"use strict";e.exports={eventDataKeys:["percentInitial","percentPrevious","percentTotal"]}},{}],983:[function(t,e,n){"use strict";var r=t("../bar/cross_trace_calc").setGroupPositions;e.exports=function(t,e){var n,i,o=t._fullLayout,a=t._fullData,s=t.calcdata,A=e.xaxis,l=e.yaxis,c=[],u=[],g=[];for(i=0;i<a.length;i++){var h=a[i],d="h"===h.orientation;!0===h.visible&&h.xaxis===A._id&&h.yaxis===l._id&&"funnel"===h.type&&(n=s[i],d?g.push(n):u.push(n),c.push(n))}var f={mode:o.funnelmode,norm:o.funnelnorm,gap:o.funnelgap,groupgap:o.funnelgroupgap};for(r(t,A,l,u,f),r(t,l,A,g,f),i=0;i<c.length;i++){n=c[i];for(var p=0;p<n.length;p++)p+1<n.length&&(n[p].nextP0=n[p+1].p0,n[p].nextS0=n[p+1].s0,n[p].nextP1=n[p+1].p1,n[p].nextS1=n[p+1].s1)}}},{"../bar/cross_trace_calc":860}],984:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../bar/defaults").handleGroupingDefaults,o=t("../bar/defaults").handleText,a=t("../scatter/xy_defaults"),s=t("./attributes"),A=t("../../components/color");e.exports={supplyDefaults:function(t,e,n,i){function l(n,i){return r.coerce(t,e,s,n,i)}if(a(t,e,i,l)){l("orientation",e.y&&!e.x?"v":"h"),l("offset"),l("width");var c=l("text");l("hovertext"),l("hovertemplate");var u=l("textposition");o(t,e,i,l,u,{moduleHasSelected:!1,moduleHasUnselected:!1,moduleHasConstrain:!0,moduleHasCliponaxis:!0,moduleHasTextangle:!0,moduleHasInsideanchor:!0}),"none"!==e.textposition&&l("textinfo",Array.isArray(c)?"text+value":"value");var g=l("marker.color",n);l("marker.line.color",A.defaultLine),l("marker.line.width"),l("connector.visible")&&(l("connector.fillcolor",function(t){var e=r.isArrayOrTypedArray(t)?"#000":t;return A.addOpacity(e,.5*A.opacity(e))}(g)),l("connector.line.width")&&(l("connector.line.color"),l("connector.line.dash")))}else e.visible=!1},crossTraceDefaults:function(t,e){var n,o;function a(t){return r.coerce(o._input,o,s,t)}if("group"===e.funnelmode)for(var A=0;A<t.length;A++)n=(o=t[A])._input,i(n,o,e,a)}}},{"../../components/color":593,"../../lib":719,"../bar/defaults":861,"../scatter/xy_defaults":1137,"./attributes":980}],985:[function(t,e,n){"use strict";e.exports=function(t,e){return t.x="xVal"in e?e.xVal:e.x,t.y="yVal"in e?e.yVal:e.y,"percentInitial"in e&&(t.percentInitial=e.percentInitial),"percentPrevious"in e&&(t.percentPrevious=e.percentPrevious),"percentTotal"in e&&(t.percentTotal=e.percentTotal),e.xa&&(t.xaxis=e.xa),e.ya&&(t.yaxis=e.ya),t}},{}],986:[function(t,e,n){"use strict";var r=t("../../components/color").opacity,i=t("../bar/hover").hoverOnBars,o=t("../../lib").formatPercent;e.exports=function(t,e,n,a){var s=i(t,e,n,a);if(s){var A=s.cd,l=A[0].trace,c="h"===l.orientation,u=A[s.index];s[(c?"x":"y")+"LabelVal"]=u.s,s.percentInitial=u.begR,s.percentInitialLabel=o(u.begR,1),s.percentPrevious=u.difR,s.percentPreviousLabel=o(u.difR,1),s.percentTotal=u.sumR,s.percentTotalLabel=o(u.sumR,1);var g=u.hi||l.hoverinfo,h=[];if(g&&"none"!==g&&"skip"!==g){var d="all"===g,f=g.split("+"),p=function(t){return d||-1!==f.indexOf(t)};p("percent initial")&&h.push(s.percentInitialLabel+" of initial"),p("percent previous")&&h.push(s.percentPreviousLabel+" of previous"),p("percent total")&&h.push(s.percentTotalLabel+" of total")}return s.extraText=h.join("<br>"),s.color=function(t,e){var n=t.marker,i=e.mc||n.color,o=e.mlc||n.line.color,a=e.mlw||n.line.width;return r(i)?i:r(o)&&a?o:void 0}(l,u),[s]}}},{"../../components/color":593,"../../lib":719,"../bar/hover":863}],987:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),layoutAttributes:t("./layout_attributes"),supplyDefaults:t("./defaults").supplyDefaults,crossTraceDefaults:t("./defaults").crossTraceDefaults,supplyLayoutDefaults:t("./layout_defaults"),calc:t("./calc"),crossTraceCalc:t("./cross_trace_calc"),plot:t("./plot"),style:t("./style").style,hoverPoints:t("./hover"),eventData:t("./event_data"),selectPoints:t("../bar/select"),moduleType:"trace",name:"funnel",basePlotModule:t("../../plots/cartesian"),categories:["bar-like","cartesian","svg","oriented","showLegend","zoomScale"],meta:{}}},{"../../plots/cartesian":778,"../bar/select":868,"./attributes":980,"./calc":981,"./cross_trace_calc":983,"./defaults":984,"./event_data":985,"./hover":986,"./layout_attributes":988,"./layout_defaults":989,"./plot":990,"./style":991}],988:[function(t,e,n){"use strict";e.exports={funnelmode:{valType:"enumerated",values:["stack","group","overlay"],dflt:"stack",editType:"calc"},funnelgap:{valType:"number",min:0,max:1,editType:"calc"},funnelgroupgap:{valType:"number",min:0,max:1,dflt:0,editType:"calc"}}},{}],989:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./layout_attributes");e.exports=function(t,e,n){var o=!1;function a(n,o){return r.coerce(t,e,i,n,o)}for(var s=0;s<n.length;s++){var A=n[s];if(A.visible&&"funnel"===A.type){o=!0;break}}o&&(a("funnelmode"),a("funnelgap",.2),a("funnelgroupgap"))}},{"../../lib":719,"./layout_attributes":988}],990:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../lib"),o=t("../../components/drawing"),a=t("../bar/plot").plot;function s(t,e,n,r){var i=[],o=[],a=r?e:n,s=r?n:e;return i[0]=a.c2p(t.s0,!0),o[0]=s.c2p(t.p0,!0),i[1]=a.c2p(t.s1,!0),o[1]=s.c2p(t.p1,!0),i[2]=a.c2p(t.nextS0,!0),o[2]=s.c2p(t.nextP0,!0),i[3]=a.c2p(t.nextS1,!0),o[3]=s.c2p(t.nextP1,!0),r?[i,o]:[o,i]}e.exports=function(t,e,n,A){var l=t._fullLayout;!function(t,e,n,a){var A=e.xaxis,l=e.yaxis;i.makeTraceGroups(a,n,"trace bars").each(function(n){var a=r.select(this),c=n[0].trace,u=i.ensureSingle(a,"g","regions");if(c.connector&&c.connector.visible){var g="h"===c.orientation,h=u.selectAll("g.region").data(i.identity);h.enter().append("g").classed("region",!0),h.exit().remove();var d=h.size();h.each(function(n,a){if(a===d-1||n.cNext){var c=s(n,A,l,g),u=c[0],h=c[1],f="";void 0!==u[3]&&void 0!==h[3]&&(f+=g?"M"+u[0]+","+h[1]+"L"+u[2]+","+h[2]+"H"+u[3]+"L"+u[1]+","+h[1]+"Z":"M"+u[1]+","+h[1]+"L"+u[2]+","+h[3]+"V"+h[2]+"L"+u[1]+","+h[0]+"Z"),i.ensureSingle(r.select(this),"path").attr("d",f).call(o.setClipUrl,e.layerClipId,t)}})}else u.remove()})}(t,e,n,A),function(t,e,n,a){var A=e.xaxis,l=e.yaxis;i.makeTraceGroups(a,n,"trace bars").each(function(n){var a=r.select(this),c=n[0].trace,u=i.ensureSingle(a,"g","lines");if(c.connector&&c.connector.visible&&c.connector.line.width){var g="h"===c.orientation,h=u.selectAll("g.line").data(i.identity);h.enter().append("g").classed("line",!0),h.exit().remove();var d=h.size();h.each(function(n,a){if(a===d-1||n.cNext){var c=s(n,A,l,g),u=c[0],h=c[1],f="";void 0!==u[3]&&void 0!==h[3]&&(g?(f+="M"+u[0]+","+h[1]+"L"+u[2]+","+h[2],f+="M"+u[1]+","+h[1]+"L"+u[3]+","+h[2]):(f+="M"+u[1]+","+h[1]+"L"+u[2]+","+h[3],f+="M"+u[1]+","+h[0]+"L"+u[2]+","+h[2])),""===f&&(f="M0,0Z"),i.ensureSingle(r.select(this),"path").attr("d",f).call(o.setClipUrl,e.layerClipId,t)}})}else u.remove()})}(t,e,n,A),a(t,e,n,A,{mode:l.funnelmode,norm:l.funnelmode,gap:l.funnelgap,groupgap:l.funnelgroupgap})}},{"../../components/drawing":614,"../../lib":719,"../bar/plot":867,d3:163}],991:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/drawing"),o=t("../../components/color"),a=t("../../constants/interactions").DESELECTDIM,s=t("../bar/style").styleTextPoints;e.exports={style:function(t,e,n){var A=n||r.select(t).selectAll("g.funnellayer").selectAll("g.trace");A.style("opacity",function(t){return t[0].trace.opacity}),A.each(function(e){var n=r.select(this),A=e[0].trace;n.selectAll(".point > path").each(function(t){if(!t.isBlank){var e=A.marker;r.select(this).call(o.fill,t.mc||e.color).call(o.stroke,t.mlc||e.line.color).call(i.dashLine,e.line.dash,t.mlw||e.line.width).style("opacity",A.selectedpoints&&!t.selected?a:1)}}),s(n,A,t),n.selectAll(".regions").each(function(){r.select(this).selectAll("path").style("stroke-width",0).call(o.fill,A.connector.fillcolor)}),n.selectAll(".lines").each(function(){var t=A.connector.line;i.lineGroupStyle(r.select(this).selectAll("path"),t.width,t.color,t.dash)})})}}},{"../../components/color":593,"../../components/drawing":614,"../../constants/interactions":694,"../bar/style":870,d3:163}],992:[function(t,e,n){"use strict";var r=t("../pie/attributes"),i=t("../../plots/attributes"),o=t("../../plots/domain").attributes,a=t("../../components/fx/hovertemplate_attributes"),s=t("../../lib/extend").extendFlat;e.exports={labels:r.labels,label0:r.label0,dlabel:r.dlabel,values:r.values,marker:{colors:r.marker.colors,line:{color:s({},r.marker.line.color,{dflt:null}),width:s({},r.marker.line.width,{dflt:1}),editType:"calc"},editType:"calc"},text:r.text,hovertext:r.hovertext,scalegroup:s({},r.scalegroup,{}),textinfo:s({},r.textinfo,{flags:["label","text","value","percent"]}),hoverinfo:s({},i.hoverinfo,{flags:["label","text","value","percent","name"]}),hovertemplate:a({},{keys:["label","color","value","percent","text"]}),textposition:s({},r.textposition,{values:["inside","none"],dflt:"inside"}),textfont:r.textfont,insidetextfont:r.insidetextfont,title:{text:r.title.text,font:r.title.font,position:s({},r.title.position,{values:["top left","top center","top right"],dflt:"top center"}),editType:"plot"},domain:o({name:"funnelarea",trace:!0,editType:"calc"}),aspectratio:{valType:"number",min:0,dflt:1,editType:"plot"},baseratio:{valType:"number",min:0,max:1,dflt:.333,editType:"plot"}}},{"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../../plots/domain":792,"../pie/attributes":1086}],993:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../plots/get_data").getModuleCalcData;n.name="funnelarea",n.plot=function(t){var e=r.getModule("funnelarea"),n=i(t.calcdata,e)[0];e.plot(t,n)},n.clean=function(t,e,n,r){var i=r._has&&r._has("funnelarea"),o=e._has&&e._has("funnelarea");i&&!o&&r._funnelarealayer.selectAll("g.trace").remove()}},{"../../plots/get_data":802,"../../registry":847}],994:[function(t,e,n){"use strict";var r=t("../pie/calc");e.exports={calc:function(t,e){return r.calc(t,e)},crossTraceCalc:function(t){r.crossTraceCalc(t,{type:"funnelarea"})}}},{"../pie/calc":1088}],995:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./attributes"),o=t("../../plots/domain").defaults,a=t("../bar/defaults").handleText;e.exports=function(t,e,n,s){function A(n,o){return r.coerce(t,e,i,n,o)}var l,c=A("values"),u=r.isArrayOrTypedArray(c),g=A("labels");if(Array.isArray(g)?(l=g.length,u&&(l=Math.min(l,c.length))):u&&(l=c.length,A("label0"),A("dlabel")),l){e._length=l,A("marker.line.width")&&A("marker.line.color",s.paper_bgcolor),A("marker.colors"),A("scalegroup");var h=A("text"),d=A("textinfo",Array.isArray(h)?"text+percent":"percent");if(A("hovertext"),A("hovertemplate"),d&&"none"!==d){var f=A("textposition");a(t,e,s,A,f,{moduleHasSelected:!1,moduleHasUnselected:!1,moduleHasConstrain:!1,moduleHasCliponaxis:!1,moduleHasTextangle:!1,moduleHasInsideanchor:!1})}o(e,s,A),A("title.text")&&(A("title.position"),r.coerceFont(A,"title.font",s.font)),A("aspectratio"),A("baseratio")}else e.visible=!1}},{"../../lib":719,"../../plots/domain":792,"../bar/defaults":861,"./attributes":992}],996:[function(t,e,n){"use strict";e.exports={moduleType:"trace",name:"funnelarea",basePlotModule:t("./base_plot"),categories:["pie-like","funnelarea","showLegend"],attributes:t("./attributes"),layoutAttributes:t("./layout_attributes"),supplyDefaults:t("./defaults"),supplyLayoutDefaults:t("./layout_defaults"),calc:t("./calc").calc,crossTraceCalc:t("./calc").crossTraceCalc,plot:t("./plot"),style:t("./style"),styleOne:t("../pie/style_one"),meta:{}}},{"../pie/style_one":1097,"./attributes":992,"./base_plot":993,"./calc":994,"./defaults":995,"./layout_attributes":997,"./layout_defaults":998,"./plot":999,"./style":1e3}],997:[function(t,e,n){"use strict";var r=t("../pie/layout_attributes").hiddenlabels;e.exports={hiddenlabels:r,funnelareacolorway:{valType:"colorlist",editType:"calc"},extendfunnelareacolors:{valType:"boolean",dflt:!0,editType:"calc"}}},{"../pie/layout_attributes":1093}],998:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./layout_attributes");e.exports=function(t,e){function n(n,o){return r.coerce(t,e,i,n,o)}n("hiddenlabels"),n("funnelareacolorway",e.colorway),n("extendfunnelareacolors")}},{"../../lib":719,"./layout_attributes":997}],999:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/drawing"),o=t("../../lib"),a=t("../../lib/svg_text_utils"),s=t("../bar/plot"),A=s.getTransform,l=s.toMoveInsideBar,c=t("../pie/helpers"),u=t("../pie/plot"),g=u.attachFxHandlers,h=u.determineInsideTextFont,d=u.layoutAreas,f=u.prerenderTitles,p=u.positionTitleOutside;function I(t,e){return"l"+(e[0]-t[0])+","+(e[1]-t[1])}e.exports=function(t,e){var n=t._fullLayout;f(e,t),d(e,n._size),o.makeTraceGroups(n._funnelarealayer,e,"trace").each(function(e){var s=r.select(this),u=e[0],d=u.trace;!function(t){if(t.length){var e=t[0],n=e.trace,r=n.aspectratio,i=n.baseratio;i>.999&&(i=.999);var o,a,s,A=Math.pow(i,2),l=e.vTotal,c=l,u=l*A/(1-A)/l,g=[];for(g.push(_()),a=t.length-1;a>-1;a--)if(!(s=t[a]).hidden){var h=s.v/c;u+=h,g.push(_())}var d=1/0,f=-1/0;for(a=0;a<g.length;a++)o=g[a],d=Math.min(d,o[1]),f=Math.max(f,o[1]);for(a=0;a<g.length;a++)g[a][1]-=(f+d)/2;var p=g[g.length-1][0],I=e.r,C=(f-d)/2,m=I/p,y=I/C*r;for(e.r=y*C,a=0;a<g.length;a++)g[a][0]*=m,g[a][1]*=y;var E,v,B=[-(o=g[0])[0],o[1]],b=[o[0],o[1]],Q=0;for(a=t.length-1;a>-1;a--)if(!(s=t[a]).hidden){var w=g[Q+=1][0],x=g[Q][1];s.TL=[-w,x],s.TR=[w,x],s.BL=B,s.BR=b,s.pxmid=(E=s.TR,v=s.BR,[.5*(E[0]+v[0]),.5*(E[1]+v[1])]),B=s.TL,b=s.TR}}function _(){var t,e={x:t=Math.sqrt(u),y:-t};return[e.x,e.y]}}(e),s.each(function(){var s=r.select(this).selectAll("g.slice").data(e);s.enter().append("g").classed("slice",!0),s.exit().remove(),s.each(function(n){if(n.hidden)r.select(this).selectAll("path,g").remove();else{n.pointNumber=n.i,n.curveNumber=d.index;var s=u.cx,f=u.cy,p=r.select(this),C=p.selectAll("path.surface").data([n]);C.enter().append("path").classed("surface",!0).style({"pointer-events":"all"}),p.call(g,t,e);var m="M"+(s+n.TR[0])+","+(f+n.TR[1])+I(n.TR,n.BR)+I(n.BR,n.BL)+I(n.BL,n.TL)+"Z";C.attr("d",m);var y=c.castOption(d.textposition,n.pts),E=p.selectAll("g.slicetext").data(n.text&&"none"!==y?[0]:[]);E.enter().append("g").classed("slicetext",!0),E.exit().remove(),E.each(function(){var e=o.ensureSingle(r.select(this),"text","",function(t){t.attr("data-notex",1)});e.text(n.text).attr({class:"slicetext",transform:"","text-anchor":"middle"}).call(i.font,h(d,n,t._fullLayout.font)).call(a.convertToTspans,t);var c,u,g,p=i.bBox(e.node()),I=Math.min(n.BL[1],n.BR[1]),C=Math.max(n.TL[1],n.TR[1]);u=Math.max(n.TL[0],n.BL[0]),g=Math.min(n.TR[0],n.BR[0]),c=A(l(u,g,I,C,p,{isHorizontal:!0,constrained:!0,angle:0,anchor:"middle"})),e.attr("transform","translate("+s+","+f+")"+c)})}});var f=r.select(this).selectAll("g.titletext").data(d.title.text?[0]:[]);f.enter().append("g").classed("titletext",!0),f.exit().remove(),f.each(function(){var e=o.ensureSingle(r.select(this),"text","",function(t){t.attr("data-notex",1)}),s=d.title.text;d._meta&&(s=o.templateString(s,d._meta)),e.text(s).attr({class:"titletext",transform:"","text-anchor":"middle"}).call(i.font,d.title.font).call(a.convertToTspans,t);var A=p(u,n._size);e.attr("transform","translate("+A.x+","+A.y+")"+(A.scale<1?"scale("+A.scale+")":"")+"translate("+A.tx+","+A.ty+")")})})})}},{"../../components/drawing":614,"../../lib":719,"../../lib/svg_text_utils":743,"../bar/plot":867,"../pie/helpers":1091,"../pie/plot":1095,d3:163}],1e3:[function(t,e,n){"use strict";var r=t("d3"),i=t("../pie/style_one");e.exports=function(t){t._fullLayout._funnelarealayer.selectAll(".trace").each(function(t){var e=t[0].trace,n=r.select(this);n.style({opacity:e.opacity}),n.selectAll("path.surface").each(function(t){r.select(this).call(i,t,e)})})}},{"../pie/style_one":1097,d3:163}],1001:[function(t,e,n){"use strict";var r=t("../scatter/attributes"),i=t("../../components/fx/hovertemplate_attributes"),o=t("../../components/colorscale/attributes"),a=(t("../../constants/docs").FORMAT_LINK,t("../../lib/extend").extendFlat);e.exports=a({z:{valType:"data_array",editType:"calc"},x:a({},r.x,{impliedEdits:{xtype:"array"}}),x0:a({},r.x0,{impliedEdits:{xtype:"scaled"}}),dx:a({},r.dx,{impliedEdits:{xtype:"scaled"}}),y:a({},r.y,{impliedEdits:{ytype:"array"}}),y0:a({},r.y0,{impliedEdits:{ytype:"scaled"}}),dy:a({},r.dy,{impliedEdits:{ytype:"scaled"}}),text:{valType:"data_array",editType:"calc"},hovertext:{valType:"data_array",editType:"calc"},transpose:{valType:"boolean",dflt:!1,editType:"calc"},xtype:{valType:"enumerated",values:["array","scaled"],editType:"calc+clearAxisTypes"},ytype:{valType:"enumerated",values:["array","scaled"],editType:"calc+clearAxisTypes"},zsmooth:{valType:"enumerated",values:["fast","best",!1],dflt:!1,editType:"calc"},connectgaps:{valType:"boolean",dflt:!1,editType:"calc"},xgap:{valType:"number",dflt:0,min:0,editType:"plot"},ygap:{valType:"number",dflt:0,min:0,editType:"plot"},zhoverformat:{valType:"string",dflt:"",editType:"none"},hovertemplate:i()},{transforms:void 0},o("",{cLetter:"z",autoColorDflt:!1}))},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../constants/docs":690,"../../lib/extend":710,"../scatter/attributes":1112}],1002:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib"),o=t("../../plots/cartesian/axes"),a=t("../histogram2d/calc"),s=t("../../components/colorscale/calc"),A=t("./convert_column_xyz"),l=t("./clean_2d_array"),c=t("./interp2d"),u=t("./find_empties"),g=t("./make_bound_array");e.exports=function(t,e){var n,h,d,f,p,I,C,m,y,E=o.getFromId(t,e.xaxis||"x"),v=o.getFromId(t,e.yaxis||"y"),B=r.traceIs(e,"contour"),b=r.traceIs(e,"histogram"),Q=r.traceIs(e,"gl2d"),w=B?"best":e.zsmooth;if(E._minDtick=0,v._minDtick=0,b)n=(y=a(t,e)).x,h=y.x0,d=y.dx,f=y.y,p=y.y0,I=y.dy,C=y.z;else{var x=e.z;i.isArray1D(x)?(A(e,E,v,"x","y",["z"]),n=e._x,f=e._y,x=e._z):(n=e._x=e.x?E.makeCalcdata(e,"x"):[],f=e._y=e.y?v.makeCalcdata(e,"y"):[]),h=e.x0,d=e.dx,p=e.y0,I=e.dy,C=l(x,e,E,v),(B||e.connectgaps)&&(e._emptypoints=u(C),c(C,e._emptypoints))}function _(t){w=e._input.zsmooth=e.zsmooth=!1,i.warn('cannot use zsmooth: "fast": '+t)}if("fast"===w)if("log"===E.type||"log"===v.type)_("log axis found");else if(!b){if(n.length){var D=(n[n.length-1]-n[0])/(n.length-1),M=Math.abs(D/100);for(m=0;m<n.length-1;m++)if(Math.abs(n[m+1]-n[m]-D)>M){_("x scale is not linear");break}}if(f.length&&"fast"===w){var k=(f[f.length-1]-f[0])/(f.length-1),S=Math.abs(k/100);for(m=0;m<f.length-1;m++)if(Math.abs(f[m+1]-f[m]-k)>S){_("y scale is not linear");break}}}var L=i.maxRowLength(C),R="scaled"===e.xtype?"":n,N=g(e,R,h,d,L,E),F="scaled"===e.ytype?"":f,T=g(e,F,p,I,C.length,v);Q||(e._extremes[E._id]=o.findExtremes(E,N),e._extremes[v._id]=o.findExtremes(v,T));var U={x:N,y:T,z:C,text:e._text||e.text,hovertext:e._hovertext||e.hovertext};if(R&&R.length===N.length-1&&(U.xCenter=R),F&&F.length===T.length-1&&(U.yCenter=F),b&&(U.xRanges=y.xRanges,U.yRanges=y.yRanges,U.pts=y.pts),B||s(t,e,{vals:C,cLetter:"z"}),B&&e.contours&&"heatmap"===e.contours.coloring){var O={type:"contour"===e.type?"heatmap":"histogram2d",xcalendar:e.xcalendar,ycalendar:e.ycalendar};U.xfill=g(O,R,h,d,L,E),U.yfill=g(O,F,p,I,C.length,v)}return[U]}},{"../../components/colorscale/calc":601,"../../lib":719,"../../plots/cartesian/axes":767,"../../registry":847,"../histogram2d/calc":1033,"./clean_2d_array":1003,"./convert_column_xyz":1005,"./find_empties":1007,"./interp2d":1010,"./make_bound_array":1011}],1003:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib"),o=t("../../constants/numerical").BADNUM;e.exports=function(t,e,n,a){var s,A,l,c,u,g;function h(t){if(r(t))return+t}if(e&&e.transpose){for(s=0,u=0;u<t.length;u++)s=Math.max(s,t[u].length);if(0===s)return!1;l=function(t){return t.length},c=function(t,e,n){return(t[n]||[])[e]}}else s=t.length,l=function(t,e){return t[e].length},c=function(t,e,n){return(t[e]||[])[n]};var d=function(t,e,n){return e===o||n===o?o:c(t,e,n)};function f(t){if(e&&"carpet"!==e.type&&"contourcarpet"!==e.type&&t&&"category"===t.type&&e["_"+t._id.charAt(0)].length){var n=t._id.charAt(0),r={},a=e["_"+n+"CategoryMap"]||e[n];for(u=0;u<a.length;u++)r[a[u]]=u;return function(e){var n=r[t._categories[e]];return n+1?n:o}}return i.identity}var p=f(n),I=f(a);a&&"category"===a.type&&(s=a._categories.length);var C=new Array(s);for(u=0;u<s;u++)for(A=n&&"category"===n.type?n._categories.length:l(t,u),C[u]=new Array(A),g=0;g<A;g++)C[u][g]=h(d(t,I(u),p(g)));return C}},{"../../constants/numerical":695,"../../lib":719,"fast-isnumeric":225}],1004:[function(t,e,n){"use strict";e.exports={min:"zmin",max:"zmax"}},{}],1005:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../constants/numerical").BADNUM;e.exports=function(t,e,n,o,a,s){var A,l,c,u,g=t._length,h=e.makeCalcdata(t,o),d=n.makeCalcdata(t,a),f=t.text,p=void 0!==f&&r.isArray1D(f),I=t.hovertext,C=void 0!==I&&r.isArray1D(I),m=r.distinctVals(h),y=m.vals,E=r.distinctVals(d),v=E.vals,B=[];for(A=0;A<s.length;A++)B[A]=r.init2dArray(v.length,y.length);for(p&&(c=r.init2dArray(v.length,y.length)),C&&(u=r.init2dArray(v.length,y.length)),A=0;A<g;A++)if(h[A]!==i&&d[A]!==i){var b=r.findBin(h[A]+m.minDiff/2,y),Q=r.findBin(d[A]+E.minDiff/2,v);for(l=0;l<s.length;l++){var w=t[s[l]];B[l][Q][b]=w[A]}p&&(c[Q][b]=f[A]),C&&(u[Q][b]=I[A])}for(t["_"+o]=y,t["_"+a]=v,l=0;l<s.length;l++)t["_"+s[l]]=B[l];p&&(t._text=c),C&&(t._hovertext=u),e&&"category"===e.type&&(t["_"+o+"CategoryMap"]=y.map(function(t){return e._categories[t]})),n&&"category"===n.type&&(t["_"+a+"CategoryMap"]=v.map(function(t){return n._categories[t]}))}},{"../../constants/numerical":695,"../../lib":719}],1006:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./xyz_defaults"),o=t("./style_defaults"),a=t("../../components/colorscale/defaults"),s=t("./attributes");e.exports=function(t,e,n,A){function l(n,i){return r.coerce(t,e,s,n,i)}i(t,e,l,A)?(l("text"),l("hovertext"),l("hovertemplate"),o(t,e,l,A),l("connectgaps",r.isArray1D(e.z)&&!1!==e.zsmooth),a(t,e,A,l,{prefix:"",cLetter:"z"})):e.visible=!1}},{"../../components/colorscale/defaults":603,"../../lib":719,"./attributes":1001,"./style_defaults":1014,"./xyz_defaults":1015}],1007:[function(t,e,n){"use strict";var r=t("../../lib").maxRowLength;e.exports=function(t){var e,n,i,o,a,s,A,l,c=[],u={},g=[],h=t[0],d=[],f=[0,0,0],p=r(t);for(n=0;n<t.length;n++)for(e=d,d=h,h=t[n+1]||[],i=0;i<p;i++)void 0===d[i]&&((s=(void 0!==d[i-1]?1:0)+(void 0!==d[i+1]?1:0)+(void 0!==e[i]?1:0)+(void 0!==h[i]?1:0))?(0===n&&s++,0===i&&s++,n===t.length-1&&s++,i===d.length-1&&s++,s<4&&(u[[n,i]]=[n,i,s]),c.push([n,i,s])):g.push([n,i]));for(;g.length;){for(A={},l=!1,a=g.length-1;a>=0;a--)(s=((u[[(n=(o=g[a])[0])-1,i=o[1]]]||f)[2]+(u[[n+1,i]]||f)[2]+(u[[n,i-1]]||f)[2]+(u[[n,i+1]]||f)[2])/20)&&(A[o]=[n,i,s],g.splice(a,1),l=!0);if(!l)throw"findEmpties iterated with no new neighbors";for(o in A)u[o]=A[o],c.push(A[o])}return c.sort(function(t,e){return e[2]-t[2]})}},{"../../lib":719}],1008:[function(t,e,n){"use strict";var r=t("../../components/fx"),i=t("../../lib"),o=t("../../plots/cartesian/axes"),a=t("../../components/colorscale").extractOpts;e.exports=function(t,e,n,s,A,l){var c,u,g,h,d=t.cd[0],f=d.trace,p=t.xa,I=t.ya,C=d.x,m=d.y,y=d.z,E=d.xCenter,v=d.yCenter,B=d.zmask,b=f.zhoverformat,Q=C,w=m;if(!1!==t.index){try{g=Math.round(t.index[1]),h=Math.round(t.index[0])}catch(e){return void i.error("Error hovering on heatmap, pointNumber must be [row,col], found:",t.index)}if(g<0||g>=y[0].length||h<0||h>y.length)return}else{if(r.inbox(e-C[0],e-C[C.length-1],0)>0||r.inbox(n-m[0],n-m[m.length-1],0)>0)return;if(l){var x;for(Q=[2*C[0]-C[1]],x=1;x<C.length;x++)Q.push((C[x]+C[x-1])/2);for(Q.push([2*C[C.length-1]-C[C.length-2]]),w=[2*m[0]-m[1]],x=1;x<m.length;x++)w.push((m[x]+m[x-1])/2);w.push([2*m[m.length-1]-m[m.length-2]])}g=Math.max(0,Math.min(Q.length-2,i.findBin(e,Q))),h=Math.max(0,Math.min(w.length-2,i.findBin(n,w)))}var _=p.c2p(C[g]),D=p.c2p(C[g+1]),M=I.c2p(m[h]),k=I.c2p(m[h+1]);l?(D=_,c=C[g],k=M,u=m[h]):(c=E?E[g]:(C[g]+C[g+1])/2,u=v?v[h]:(m[h]+m[h+1])/2,p&&"category"===p.type&&(c=C[g]),I&&"category"===I.type&&(u=m[h]),f.zsmooth&&(_=D=p.c2p(c),M=k=I.c2p(u)));var S,L=y[h][g];B&&!B[h][g]&&(L=void 0),Array.isArray(d.hovertext)&&Array.isArray(d.hovertext[h])?S=d.hovertext[h][g]:Array.isArray(d.text)&&Array.isArray(d.text[h])&&(S=d.text[h][g]);var R=a(f),N={type:"linear",range:[R.min,R.max],hoverformat:b,_separators:p._separators,_numFormat:p._numFormat},F=o.tickText(N,L,"hover").text;return[i.extendFlat(t,{index:[h,g],distance:t.maxHoverDistance,spikeDistance:t.maxSpikeDistance,x0:_,x1:D,y0:M,y1:k,xLabelVal:c,yLabelVal:u,zLabelVal:L,zLabel:F,text:S})]}},{"../../components/colorscale":605,"../../components/fx":632,"../../lib":719,"../../plots/cartesian/axes":767}],1009:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),calc:t("./calc"),plot:t("./plot"),colorbar:t("./colorbar"),style:t("./style"),hoverPoints:t("./hover"),moduleType:"trace",name:"heatmap",basePlotModule:t("../../plots/cartesian"),categories:["cartesian","svg","2dMap"],meta:{}}},{"../../plots/cartesian":778,"./attributes":1001,"./calc":1002,"./colorbar":1004,"./defaults":1006,"./hover":1008,"./plot":1012,"./style":1013}],1010:[function(t,e,n){"use strict";var r=t("../../lib"),i=[[-1,0],[1,0],[0,-1],[0,1]];function o(t){return.5-.25*Math.min(1,.5*t)}function a(t,e,n){var r,o,a,s,A,l,c,u,g,h,d,f,p,I=0;for(s=0;s<e.length;s++){for(o=(r=e[s])[0],a=r[1],d=t[o][a],h=0,g=0,A=0;A<4;A++)(c=t[o+(l=i[A])[0]])&&void 0!==(u=c[a+l[1]])&&(0===h?f=p=u:(f=Math.min(f,u),p=Math.max(p,u)),g++,h+=u);if(0===g)throw"iterateInterp2d order is wrong: no defined neighbors";t[o][a]=h/g,void 0===d?g<4&&(I=1):(t[o][a]=(1+n)*t[o][a]-n*d,p>f&&(I=Math.max(I,Math.abs(t[o][a]-d)/(p-f))))}return I}e.exports=function(t,e){var n,i=1;for(a(t,e),n=0;n<e.length&&!(e[n][2]<4);n++);for(e=e.slice(n),n=0;n<100&&i>.01;n++)i=a(t,e,o(i));return i>.01&&r.log("interp2d didn't converge quickly",i),t}},{"../../lib":719}],1011:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib").isArrayOrTypedArray;e.exports=function(t,e,n,o,a,s){var A,l,c,u=[],g=r.traceIs(t,"contour"),h=r.traceIs(t,"histogram"),d=r.traceIs(t,"gl2d");if(i(e)&&e.length>1&&!h&&"category"!==s.type){var f=e.length;if(!(f<=a))return g?e.slice(0,a):e.slice(0,a+1);if(g||d)u=e.slice(0,a);else if(1===a)u=[e[0]-.5,e[0]+.5];else{for(u=[1.5*e[0]-.5*e[1]],c=1;c<f;c++)u.push(.5*(e[c-1]+e[c]));u.push(1.5*e[f-1]-.5*e[f-2])}if(f<a){var p=u[u.length-1],I=p-u[u.length-2];for(c=f;c<a;c++)p+=I,u.push(p)}}else{var C=t[s._id.charAt(0)+"calendar"];for(A=h?s.r2c(n,0,C):i(e)&&1===e.length?e[0]:void 0===n?0:("log"===s.type?s.d2c:s.r2c)(n,0,C),l=o||1,c=g||d?0:-.5;c<a;c++)u.push(A+l*c)}return u}},{"../../lib":719,"../../registry":847}],1012:[function(t,e,n){"use strict";var r=t("d3"),i=t("tinycolor2"),o=t("../../registry"),a=t("../../lib"),s=t("../../components/colorscale").makeColorScaleFuncFromTrace,A=t("../../constants/xmlns_namespaces");function l(t,e){var n=e.length-2,r=a.constrain(a.findBin(t,e),0,n),i=e[r],o=e[r+1],s=a.constrain(r+(t-i)/(o-i)-.5,0,n),A=Math.round(s),l=Math.abs(s-A);return s&&s!==n&&l?{bin0:A,frac:l,bin1:Math.round(A+l/(s-A))}:{bin0:A,bin1:A,frac:0}}function c(t,e){var n=e.length-1,r=a.constrain(a.findBin(t,e),0,n),i=e[r],o=(t-i)/(e[r+1]-i)||0;return o<=0?{bin0:r,bin1:r,frac:0}:o<.5?{bin0:r,bin1:r+1,frac:o}:{bin0:r+1,bin1:r,frac:1-o}}function u(t,e,n){t[e]=n[0],t[e+1]=n[1],t[e+2]=n[2],t[e+3]=Math.round(255*n[3])}e.exports=function(t,e,n,g){var h=e.xaxis,d=e.yaxis;a.makeTraceGroups(g,n,"hm").each(function(e){var n,g,f,p,I,C,m=r.select(this),y=e[0],E=y.trace,v=y.z,B=y.x,b=y.y,Q=y.xCenter,w=y.yCenter,x=o.traceIs(E,"contour"),_=x?"best":E.zsmooth,D=v.length,M=a.maxRowLength(v),k=!1,S=!1;for(C=0;void 0===n&&C<B.length-1;)n=h.c2p(B[C]),C++;for(C=B.length-1;void 0===g&&C>0;)g=h.c2p(B[C]),C--;for(g<n&&(f=g,g=n,n=f,k=!0),C=0;void 0===p&&C<b.length-1;)p=d.c2p(b[C]),C++;for(C=b.length-1;void 0===I&&C>0;)I=d.c2p(b[C]),C--;if(I<p&&(f=p,p=I,I=f,S=!0),x&&(Q=B,w=b,B=y.xfill,b=y.yfill),"fast"!==_){var L="best"===_?0:.5;n=Math.max(-L*h._length,n),g=Math.min((1+L)*h._length,g),p=Math.max(-L*d._length,p),I=Math.min((1+L)*d._length,I)}var R=Math.round(g-n),N=Math.round(I-p);if(R<=0||N<=0)m.selectAll("image").data([]).exit().remove();else{var F,T;"fast"===_?(F=M,T=D):(F=R,T=N);var U=document.createElement("canvas");U.width=F,U.height=T;var O,G,H=U.getContext("2d"),j=s(E,{noNumericCheck:!0,returnArray:!0});"fast"===_?(O=k?function(t){return M-1-t}:a.identity,G=S?function(t){return D-1-t}:a.identity):(O=function(t){return a.constrain(Math.round(h.c2p(B[t])-n),0,R)},G=function(t){return a.constrain(Math.round(d.c2p(b[t])-p),0,N)});var Y,P,K,z,J,q=G(0),V=[q,q],W=k?0:1,X=S?0:1,Z=0,$=0,tt=0,et=0;if(_){var nt,rt=0;try{nt=new Uint8Array(R*N*4)}catch(t){nt=new Array(R*N*4)}if("best"===_){var it,ot,at,st=Q||B,At=w||b,lt=new Array(st.length),ct=new Array(At.length),ut=new Array(R),gt=Q?c:l,ht=w?c:l;for(C=0;C<st.length;C++)lt[C]=Math.round(h.c2p(st[C])-n);for(C=0;C<At.length;C++)ct[C]=Math.round(d.c2p(At[C])-p);for(C=0;C<R;C++)ut[C]=gt(C,lt);for(P=0;P<N;P++)for(ot=v[(it=ht(P,ct)).bin0],at=v[it.bin1],C=0;C<R;C++,rt+=4)u(nt,rt,J=bt(ot,at,ut[C],it))}else for(P=0;P<D;P++)for(z=v[P],V=G(P),C=0;C<R;C++)J=Bt(z[C],1),u(nt,rt=4*(V*R+O(C)),J);var dt=H.createImageData(R,N);try{dt.data.set(nt)}catch(t){var ft=dt.data,pt=ft.length;for(P=0;P<pt;P++)ft[P]=nt[P]}H.putImageData(dt,0,0)}else{var It=E.xgap,Ct=E.ygap,mt=Math.floor(It/2),yt=Math.floor(Ct/2);for(P=0;P<D;P++)if(z=v[P],V.reverse(),V[X]=G(P+1),V[0]!==V[1]&&void 0!==V[0]&&void 0!==V[1])for(Y=[K=O(0),K],C=0;C<M;C++)Y.reverse(),Y[W]=O(C+1),Y[0]!==Y[1]&&void 0!==Y[0]&&void 0!==Y[1]&&(J=Bt(z[C],(Y[1]-Y[0])*(V[1]-V[0])),H.fillStyle="rgba("+J.join(",")+")",H.fillRect(Y[0]+mt,V[0]+yt,Y[1]-Y[0]-It,V[1]-V[0]-Ct))}$=Math.round($/Z),tt=Math.round(tt/Z),et=Math.round(et/Z);var Et=i("rgb("+$+","+tt+","+et+")");t._hmpixcount=(t._hmpixcount||0)+Z,t._hmlumcount=(t._hmlumcount||0)+Z*Et.getLuminance();var vt=m.selectAll("image").data(e);vt.enter().append("svg:image").attr({xmlns:A.svg,preserveAspectRatio:"none"}),vt.attr({height:N,width:R,x:n,y:p,"xlink:href":U.toDataURL("image/png")})}function Bt(t,e){if(void 0!==t){var n=j(t);return n[0]=Math.round(n[0]),n[1]=Math.round(n[1]),n[2]=Math.round(n[2]),Z+=e,$+=n[0]*e,tt+=n[1]*e,et+=n[2]*e,n}return[0,0,0,0]}function bt(t,e,n,r){var i=t[n.bin0];if(void 0===i)return Bt(void 0,1);var o,a=t[n.bin1],s=e[n.bin0],A=e[n.bin1],l=a-i||0,c=s-i||0;return o=void 0===a?void 0===A?0:void 0===s?2*(A-i):2*(2*A-s-i)/3:void 0===A?void 0===s?0:2*(2*i-a-s)/3:void 0===s?2*(2*A-a-i)/3:A+i-a-s,Bt(i+n.frac*l+r.frac*(c+n.frac*o))}})}},{"../../components/colorscale":605,"../../constants/xmlns_namespaces":696,"../../lib":719,"../../registry":847,d3:163,tinycolor2:537}],1013:[function(t,e,n){"use strict";var r=t("d3");e.exports=function(t){r.select(t).selectAll(".hm image").style("opacity",function(t){return t.trace.opacity})}},{d3:163}],1014:[function(t,e,n){"use strict";e.exports=function(t,e,n){!1===n("zsmooth")&&(n("xgap"),n("ygap")),n("zhoverformat")}},{}],1015:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib"),o=t("../../registry");function a(t,e){var n=e(t);return"scaled"===(n?e(t+"type","array"):"scaled")&&(e(t+"0"),e("d"+t)),n}e.exports=function(t,e,n,s,A,l){var c,u,g=n("z");if(A=A||"x",l=l||"y",void 0===g||!g.length)return 0;if(i.isArray1D(t.z)){c=n(A),u=n(l);var h=i.minRowLength(c),d=i.minRowLength(u);if(0===h||0===d)return 0;e._length=Math.min(h,d,g.length)}else{if(c=a(A,n),u=a(l,n),!function(t){for(var e,n=!0,o=!1,a=!1,s=0;s<t.length;s++){if(e=t[s],!i.isArrayOrTypedArray(e)){n=!1;break}e.length>0&&(o=!0);for(var A=0;A<e.length;A++)if(r(e[A])){a=!0;break}}return n&&o&&a}(g))return 0;n("transpose"),e._length=null}return o.getComponentMethod("calendars","handleTraceDefaults")(t,e,[A,l],s),!0}},{"../../lib":719,"../../registry":847,"fast-isnumeric":225}],1016:[function(t,e,n){"use strict";for(var r=t("../heatmap/attributes"),i=t("../../components/colorscale/attributes"),o=t("../../lib/extend").extendFlat,a=t("../../plot_api/edit_types").overrideAll,s=["z","x","x0","dx","y","y0","dy","text","transpose","xtype","ytype"],A={},l=0;l<s.length;l++){var c=s[l];A[c]=r[c]}o(A,i("",{cLetter:"z",autoColorDflt:!1})),e.exports=a(A,"calc","nested")},{"../../components/colorscale/attributes":600,"../../lib/extend":710,"../../plot_api/edit_types":750,"../heatmap/attributes":1001}],1017:[function(t,e,n){"use strict";var r=t("gl-heatmap2d"),i=t("../../plots/cartesian/axes"),o=t("../../lib/str2rgbarray");function a(t,e){this.scene=t,this.uid=e,this.type="heatmapgl",this.name="",this.hoverinfo="all",this.xData=[],this.yData=[],this.zData=[],this.textLabels=[],this.idToIndex=[],this.bounds=[0,0,0,0],this.options={z:[],x:[],y:[],shape:[0,0],colorLevels:[0],colorValues:[0,0,0,1]},this.heatmap=r(t.glplot,this.options),this.heatmap._trace=this}var s=a.prototype;s.handlePick=function(t){var e=this.options,n=e.shape,r=t.pointId,i=r%n[0],o=Math.floor(r/n[0]),a=r;return{trace:this,dataCoord:t.dataCoord,traceCoord:[e.x[i],e.y[o],e.z[a]],textLabel:this.textLabels[r],name:this.name,pointIndex:[o,i],hoverinfo:this.hoverinfo}},s.update=function(t,e){var n=e[0];this.index=t.index,this.name=t.name,this.hoverinfo=t.hoverinfo;var r=n.z;this.options.z=[].concat.apply([],r);var a=r[0].length,s=r.length;this.options.shape=[a,s],this.options.x=n.x,this.options.y=n.y;var A=function(t){for(var e=t.colorscale,n=t.zmin,r=t.zmax,i=e.length,a=new Array(i),s=new Array(4*i),A=0;A<i;A++){var l=e[A],c=o(l[1]);a[A]=n+l[0]*(r-n);for(var u=0;u<4;u++)s[4*A+u]=c[u]}return{colorLevels:a,colorValues:s}}(t);this.options.colorLevels=A.colorLevels,this.options.colorValues=A.colorValues,this.textLabels=[].concat.apply([],t.text),this.heatmap.update(this.options);var l=this.scene.xaxis,c=this.scene.yaxis;t._extremes[l._id]=i.findExtremes(l,n.x),t._extremes[c._id]=i.findExtremes(c,n.y)},s.dispose=function(){this.heatmap.dispose()},e.exports=function(t,e,n){var r=new a(t,e.uid);return r.update(e,n),r}},{"../../lib/str2rgbarray":742,"../../plots/cartesian/axes":767,"gl-heatmap2d":251}],1018:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("../heatmap/defaults"),colorbar:t("../heatmap/colorbar"),calc:t("../heatmap/calc"),plot:t("./convert"),moduleType:"trace",name:"heatmapgl",basePlotModule:t("../../plots/gl2d"),categories:["gl","gl2d","2dMap"],meta:{}}},{"../../plots/gl2d":805,"../heatmap/calc":1002,"../heatmap/colorbar":1004,"../heatmap/defaults":1006,"./attributes":1016,"./convert":1017}],1019:[function(t,e,n){"use strict";var r=t("../bar/attributes"),i=t("../../components/fx/hovertemplate_attributes"),o=t("./bin_attributes"),a=t("./constants"),s=t("../../lib/extend").extendFlat;e.exports={x:{valType:"data_array",editType:"calc+clearAxisTypes"},y:{valType:"data_array",editType:"calc+clearAxisTypes"},text:s({},r.text,{}),hovertext:s({},r.hovertext,{}),orientation:r.orientation,histfunc:{valType:"enumerated",values:["count","sum","avg","min","max"],dflt:"count",editType:"calc"},histnorm:{valType:"enumerated",values:["","percent","probability","density","probability density"],dflt:"",editType:"calc"},cumulative:{enabled:{valType:"boolean",dflt:!1,editType:"calc"},direction:{valType:"enumerated",values:["increasing","decreasing"],dflt:"increasing",editType:"calc"},currentbin:{valType:"enumerated",values:["include","exclude","half"],dflt:"include",editType:"calc"},editType:"calc"},nbinsx:{valType:"integer",min:0,dflt:0,editType:"calc"},xbins:o("x",!0),nbinsy:{valType:"integer",min:0,dflt:0,editType:"calc"},ybins:o("y",!0),autobinx:{valType:"boolean",dflt:null,editType:"calc"},autobiny:{valType:"boolean",dflt:null,editType:"calc"},bingroup:{valType:"string",dflt:"",editType:"calc"},hovertemplate:i({},{keys:a.eventDataKeys}),marker:r.marker,offsetgroup:r.offsetgroup,alignmentgroup:r.alignmentgroup,selected:r.selected,unselected:r.unselected,_deprecated:{bardir:r._deprecated.bardir}}},{"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../bar/attributes":857,"./bin_attributes":1021,"./constants":1025}],1020:[function(t,e,n){"use strict";e.exports=function(t,e){for(var n=t.length,r=0,i=0;i<n;i++)e[i]?(t[i]/=e[i],r+=t[i]):t[i]=null;return r}},{}],1021:[function(t,e,n){"use strict";e.exports=function(t,e){return{start:{valType:"any",editType:"calc"},end:{valType:"any",editType:"calc"},size:{valType:"any",editType:"calc"},editType:"calc"}}},{}],1022:[function(t,e,n){"use strict";var r=t("fast-isnumeric");e.exports={count:function(t,e,n){return n[t]++,1},sum:function(t,e,n,i){var o=i[e];return r(o)?(o=Number(o),n[t]+=o,o):0},avg:function(t,e,n,i,o){var a=i[e];return r(a)&&(a=Number(a),n[t]+=a,o[t]++),0},min:function(t,e,n,i){var o=i[e];if(r(o)){if(o=Number(o),!r(n[t]))return n[t]=o,o;if(n[t]>o){var a=o-n[t];return n[t]=o,a}}return 0},max:function(t,e,n,i){var o=i[e];if(r(o)){if(o=Number(o),!r(n[t]))return n[t]=o,o;if(n[t]<o){var a=o-n[t];return n[t]=o,a}}return 0}}},{"fast-isnumeric":225}],1023:[function(t,e,n){"use strict";var r=t("../../constants/numerical"),i=r.ONEAVGYEAR,o=r.ONEAVGMONTH,a=r.ONEDAY,s=r.ONEHOUR,A=r.ONEMIN,l=r.ONESEC,c=t("../../plots/cartesian/axes").tickIncrement;function u(t,e,n,r){if(t*e<=0)return 1/0;for(var i=Math.abs(e-t),o="date"===n.type,a=g(i,o),s=0;s<10;s++){var A=g(80*a,o);if(a===A)break;if(!h(A,t,e,o,n,r))break;a=A}return a}function g(t,e){return e&&t>l?t>a?t>1.1*i?i:t>1.1*o?o:a:t>s?s:t>A?A:l:Math.pow(10,Math.floor(Math.log(t)/Math.LN10))}function h(t,e,n,r,o,s){if(r&&t>a){var A=d(e,o,s),l=d(n,o,s),c=t===i?0:1;return A[c]!==l[c]}return Math.floor(n/t)-Math.floor(e/t)>.1}function d(t,e,n){var r=e.c2d(t,i,n).split("-");return""===r[0]&&(r.unshift(),r[0]="-"+r[0]),r}e.exports=function(t,e,n,r,o){var s,A,l=-1.1*e,g=-.1*e,h=t-g,d=n[0],f=n[1],p=Math.min(u(d+g,d+h,r,o),u(f+g,f+h,r,o)),I=Math.min(u(d+l,d+g,r,o),u(f+l,f+g,r,o));if(p>I&&I<Math.abs(f-d)/4e3?(s=p,A=!1):(s=Math.min(p,I),A=!0),"date"===r.type&&s>a){var C=s===i?1:6,m=s===i?"M12":"M1";return function(e,n){var a=r.c2d(e,i,o),s=a.indexOf("-",C);s>0&&(a=a.substr(0,s));var l=r.d2c(a,0,o);if(l<e){var u=c(l,m,!1,o);(l+u)/2<e+t&&(l=u)}return n&&A?c(l,m,!0,o):l}}return function(e,n){var r=s*Math.round(e/s);return r+s/10<e&&r+.9*s<e+t&&(r+=s),n&&A&&(r-=s),r}}},{"../../constants/numerical":695,"../../plots/cartesian/axes":767}],1024:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib"),o=t("../../registry"),a=t("../../plots/cartesian/axes"),s=t("../bar/arrays_to_calcdata"),A=t("./bin_functions"),l=t("./norm_functions"),c=t("./average"),u=t("./bin_label_vals");function g(t,e,n,s,A){var l,c,u,h,d,f,p,I=s+"bins",C=t._fullLayout,m=e["_"+s+"bingroup"],y=C._histogramBinOpts[m],E="overlay"===C.barmode,v=function(t){return n.r2c(t,0,h)},B=function(t){return n.c2r(t,0,h)},b="date"===n.type?function(t){return t||0===t?i.cleanDate(t,null,h):null}:function(t){return r(t)?Number(t):null};function Q(t,e,n){e[t+"Found"]?(e[t]=b(e[t]),null===e[t]&&(e[t]=n[t])):(f[t]=e[t]=n[t],i.nestedProperty(c[0],I+"."+t).set(n[t]))}if(e["_"+s+"autoBinFinished"])delete e["_"+s+"autoBinFinished"];else{c=y.traces;var w=[],x=!0,_=!1,D=!1;for(l=0;l<c.length;l++)if((u=c[l]).visible){var M=y.dirs[l];d=u["_"+M+"pos0"]=n.makeCalcdata(u,M),w=i.concat(w,d),delete u["_"+s+"autoBinFinished"],!0===e.visible&&(x?x=!1:(delete u._autoBin,u["_"+s+"autoBinFinished"]=1),o.traceIs(u,"2dMap")&&(_=!0),"histogram2dcontour"===u.type&&(D=!0))}h=c[0][s+"calendar"];var k=a.autoBin(w,n,y.nbins,_,h,y.sizeFound&&y.size),S=c[0]._autoBin={};if(f=S[y.dirs[0]]={},D&&(y.size||(k.start=B(a.tickIncrement(v(k.start),k.size,!0,h))),void 0===y.end&&(k.end=B(a.tickIncrement(v(k.end),k.size,!1,h)))),E&&!o.traceIs(e,"2dMap")&&0===k._dataSpan&&"category"!==n.type&&"multicategory"!==n.type){if(A)return[k,d,!0];k=function(t,e,n,r,o){var a,s,A,l=t._fullLayout,c=function(t,e){for(var n=e.xaxis,r=e.yaxis,i=e.orientation,o=[],a=t._fullData,s=0;s<a.length;s++){var A=a[s];"histogram"===A.type&&!0===A.visible&&A.orientation===i&&A.xaxis===n&&A.yaxis===r&&o.push(A)}return o}(t,e),u=!1,h=1/0,d=[e];for(a=0;a<c.length;a++)if((s=c[a])===e)u=!0;else if(u){var f=g(t,s,n,r,!0),p=f[0],I=f[2];s["_"+r+"autoBinFinished"]=1,s["_"+r+"pos0"]=f[1],I?d.push(s):h=Math.min(h,p.size)}else A=l._histogramBinOpts[s["_"+r+"bingroup"]],h=Math.min(h,A.size||s[o].size);var C=new Array(d.length);for(a=0;a<d.length;a++)for(var m=d[a]["_"+r+"pos0"],y=0;y<m.length;y++)if(void 0!==m[y]){C[a]=m[y];break}for(isFinite(h)||(h=i.distinctVals(C).minDiff),a=0;a<d.length;a++){var E=(s=d[a])[r+"calendar"],v={start:n.c2r(C[a]-h/2,0,E),end:n.c2r(C[a]+h/2,0,E),size:h};s._input[o]=s[o]=v,(A=l._histogramBinOpts[s["_"+r+"bingroup"]])&&i.extendFlat(A,v)}return e[o]}(t,e,n,s,I)}(p=u.cumulative||{}).enabled&&"include"!==p.currentbin&&("decreasing"===p.direction?k.start=B(a.tickIncrement(v(k.start),k.size,!0,h)):k.end=B(a.tickIncrement(v(k.end),k.size,!1,h))),y.size=k.size,y.sizeFound||(f.size=k.size,i.nestedProperty(c[0],I+".size").set(k.size)),Q("start",y,k),Q("end",y,k)}d=e["_"+s+"pos0"],delete e["_"+s+"pos0"];var L=e._input[I]||{},R=i.extendFlat({},y),N=y.start,F=n.r2l(L.start),T=void 0!==F;if((y.startFound||T)&&F!==n.r2l(N)){var U=T?F:i.aggNums(Math.min,null,d),O={type:"category"===n.type||"multicategory"===n.type?"linear":n.type,r2l:n.r2l,dtick:y.size,tick0:N,calendar:h,range:[U,a.tickIncrement(U,y.size,!1,h)].map(n.l2r)},G=a.tickFirst(O);G>n.r2l(U)&&(G=a.tickIncrement(G,y.size,!0,h)),R.start=n.l2r(G),T||i.nestedProperty(e,I+".start").set(R.start)}var H=y.end,j=n.r2l(L.end),Y=void 0!==j;if((y.endFound||Y)&&j!==n.r2l(H)){var P=Y?j:i.aggNums(Math.max,null,d);R.end=n.l2r(P),Y||i.nestedProperty(e,I+".start").set(R.end)}var K="autobin"+s;return!1===e._input[K]&&(e._input[I]=i.extendFlat({},e[I]||{}),delete e._input[K],delete e[K]),[R,d]}e.exports={calc:function(t,e){var n,o,h,d,f=[],p=[],I=a.getFromId(t,"h"===e.orientation?e.yaxis:e.xaxis),C="h"===e.orientation?"y":"x",m={x:"y",y:"x"}[C],y=e[C+"calendar"],E=e.cumulative,v=g(t,e,I,C),B=v[0],b=v[1],Q="string"==typeof B.size,w=[],x=Q?w:B,_=[],D=[],M=[],k=0,S=e.histnorm,L=e.histfunc,R=-1!==S.indexOf("density");E.enabled&&R&&(S=S.replace(/ ?density$/,""),R=!1);var N,F="max"===L||"min"===L?null:0,T=A.count,U=l[S],O=!1,G=function(t){return I.r2c(t,0,y)};for(i.isArrayOrTypedArray(e[m])&&"count"!==L&&(N=e[m],O="avg"===L,T=A[L]),n=G(B.start),h=G(B.end)+(n-a.tickIncrement(n,B.size,!1,y))/1e6;n<h&&f.length<1e6&&(o=a.tickIncrement(n,B.size,!1,y),f.push((n+o)/2),p.push(F),M.push([]),w.push(n),R&&_.push(1/(o-n)),O&&D.push(0),!(o<=n));)n=o;w.push(n),Q||"date"!==I.type||(x={start:G(x.start),end:G(x.end),size:x.size});var H,j=p.length,Y=!0,P=1/0,K=1/0,z={};for(n=0;n<b.length;n++){var J=b[n];(d=i.findBin(J,x))>=0&&d<j&&(k+=T(d,n,p,N,D),Y&&M[d].length&&J!==b[M[d][0]]&&(Y=!1),M[d].push(n),z[n]=d,P=Math.min(P,J-w[d]),K=Math.min(K,w[d+1]-J))}Y||(H=u(P,K,w,I,y)),O&&(k=c(p,D)),U&&U(p,k,_),E.enabled&&function(t,e,n){var r,i,o;function a(e){o=t[e],t[e]/=2}function s(e){i=t[e],t[e]=o+i/2,o+=i}if("half"===n)if("increasing"===e)for(a(0),r=1;r<t.length;r++)s(r);else for(a(t.length-1),r=t.length-2;r>=0;r--)s(r);else if("increasing"===e){for(r=1;r<t.length;r++)t[r]+=t[r-1];"exclude"===n&&(t.unshift(0),t.pop())}else{for(r=t.length-2;r>=0;r--)t[r]+=t[r+1];"exclude"===n&&(t.push(0),t.shift())}}(p,E.direction,E.currentbin);var q=Math.min(f.length,p.length),V=[],W=0,X=q-1;for(n=0;n<q;n++)if(p[n]){W=n;break}for(n=q-1;n>=W;n--)if(p[n]){X=n;break}for(n=W;n<=X;n++)if(r(f[n])&&r(p[n])){var Z={p:f[n],s:p[n],b:0};E.enabled||(Z.pts=M[n],Y?Z.ph0=Z.ph1=M[n].length?b[M[n][0]]:f[n]:(Z.ph0=H(w[n]),Z.ph1=H(w[n+1],!0))),V.push(Z)}return 1===V.length&&(V[0].width1=a.tickIncrement(V[0].p,B.size,!1,y)-V[0].p),s(V,e),i.isArrayOrTypedArray(e.selectedpoints)&&i.tagSelected(V,e,z),V},calcAllAutoBins:g}},{"../../lib":719,"../../plots/cartesian/axes":767,"../../registry":847,"../bar/arrays_to_calcdata":856,"./average":1020,"./bin_functions":1022,"./bin_label_vals":1023,"./norm_functions":1031,"fast-isnumeric":225}],1025:[function(t,e,n){"use strict";e.exports={eventDataKeys:["binNumber"]}},{}],1026:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axis_ids"),o=t("../../registry").traceIs,a=t("../bar/defaults").handleGroupingDefaults,s=r.nestedProperty,A=i.getAxisGroup,l=[{aStr:{x:"xbins.start",y:"ybins.start"},name:"start"},{aStr:{x:"xbins.end",y:"ybins.end"},name:"end"},{aStr:{x:"xbins.size",y:"ybins.size"},name:"size"},{aStr:{x:"nbinsx",y:"nbinsy"},name:"nbins"}],c=["x","y"];e.exports=function(t,e){var n,u,g,h,d,f,p,I=e._histogramBinOpts={},C=[],m={},y=[];function E(t,e){return r.coerce(n._input,n,n._module.attributes,t,e)}function v(t){return"v"===t.orientation?"x":"y"}function B(t,n,o){var a=t.uid+"__"+o;n||(n=a);var s=function(t,n){return i.getFromTrace({_fullLayout:e},t,n).type}(t,o),A=t[o+"calendar"],l=I[n],c=!0;l&&(s===l.axType&&A===l.calendar?(c=!1,l.traces.push(t),l.dirs.push(o)):(n=a,s!==l.axType&&r.warn(["Attempted to group the bins of trace",t.index,"set on a","type:"+s,"axis","with bins on","type:"+l.axType,"axis."].join(" ")),A!==l.calendar&&r.warn(["Attempted to group the bins of trace",t.index,"set with a",A,"calendar","with bins",l.calendar?"on a "+l.calendar+" calendar":"w/o a set calendar"].join(" ")))),c&&(I[n]={traces:[t],dirs:[o],axType:s,calendar:t[o+"calendar"]||""}),t["_"+o+"bingroup"]=n}for(d=0;d<t.length;d++)n=t[d],o(n,"histogram")&&(C.push(n),delete n._xautoBinFinished,delete n._yautoBinFinished,o(n,"2dMap")||a(n._input,n,e,E));var b=e._alignmentOpts||{};for(d=0;d<C.length;d++){if(n=C[d],g="",!o(n,"2dMap")){if(h=v(n),"group"===e.barmode&&n.alignmentgroup){var Q=n[h+"axis"],w=A(e,Q)+n.orientation;(b[w]||{})[n.alignmentgroup]&&(g=w)}g||"overlay"===e.barmode||(g=A(e,n.xaxis)+A(e,n.yaxis)+v(n))}g?(m[g]||(m[g]=[]),m[g].push(n)):y.push(n)}for(g in m)if(1!==(u=m[g]).length){var x=!1;for(d=0;d<u.length;d++){n=u[d],x=E("bingroup");break}for(g=x||g,d=0;d<u.length;d++){var _=(n=u[d])._input.bingroup;_&&_!==g&&r.warn(["Trace",n.index,"must match","within bingroup",g+".","Ignoring its bingroup:",_,"setting."].join(" ")),n.bingroup=g,B(n,g,v(n))}}else y.push(u[0]);for(d=0;d<y.length;d++){n=y[d];var D=E("bingroup");if(o(n,"2dMap"))for(p=0;p<2;p++){var M=E((h=c[p])+"bingroup",D?D+"__"+h:null);B(n,M,h)}else B(n,D,v(n))}for(g in I){var k=I[g];for(u=k.traces,f=0;f<l.length;f++){var S,L,R=l[f],N=R.name;if("nbins"!==N||!k.sizeFound){for(d=0;d<u.length;d++){if(n=u[d],h=k.dirs[d],S=R.aStr[h],void 0!==s(n._input,S).get()){k[N]=E(S),k[N+"Found"]=!0;break}(L=(n._autoBin||{})[h]||{})[N]&&s(n,S).set(L[N])}if("start"===N||"end"===N)for(;d<u.length;d++)(n=u[d])["_"+h+"bingroup"]&&E(S,(L=(n._autoBin||{})[h]||{})[N]);"nbins"!==N||k.sizeFound||k.nbinsFound||(n=u[0],k[N]=E(S))}}}}},{"../../lib":719,"../../plots/cartesian/axis_ids":770,"../../registry":847,"../bar/defaults":861}],1027:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib"),o=t("../../components/color"),a=t("../bar/style_defaults"),s=t("./attributes");e.exports=function(t,e,n,A){function l(n,r){return i.coerce(t,e,s,n,r)}var c=l("x"),u=l("y");l("cumulative.enabled")&&(l("cumulative.direction"),l("cumulative.currentbin")),l("text"),l("hovertext"),l("hovertemplate");var g=l("orientation",u&&!c?"h":"v"),h="v"===g?"x":"y",d="v"===g?"y":"x",f=c&&u?Math.min(i.minRowLength(c)&&i.minRowLength(u)):i.minRowLength(e[h]||[]);if(f){e._length=f,r.getComponentMethod("calendars","handleTraceDefaults")(t,e,["x","y"],A),e[d]&&l("histfunc"),l("histnorm"),l("autobin"+h),a(t,e,l,n,A),i.coerceSelectionMarkerOpacity(e,l);var p=(e.marker.line||{}).color,I=r.getComponentMethod("errorbars","supplyDefaults");I(t,e,p||o.defaultLine,{axis:"y"}),I(t,e,p||o.defaultLine,{axis:"x",inherit:"y"})}else e.visible=!1}},{"../../components/color":593,"../../lib":719,"../../registry":847,"../bar/style_defaults":871,"./attributes":1019}],1028:[function(t,e,n){"use strict";e.exports=function(t,e,n,r,i){if(t.x="xVal"in e?e.xVal:e.x,t.y="yVal"in e?e.yVal:e.y,"zLabelVal"in e&&(t.z=e.zLabelVal),e.xa&&(t.xaxis=e.xa),e.ya&&(t.yaxis=e.ya),!(n.cumulative||{}).enabled){var o,a=Array.isArray(i)?r[0].pts[i[0]][i[1]]:r[i].pts;if(t.pointNumbers=a,t.binNumber=t.pointNumber,delete t.pointNumber,delete t.pointIndex,n._indexToPoints){o=[];for(var s=0;s<a.length;s++)o=o.concat(n._indexToPoints[a[s]])}else o=a;t.pointIndices=o}return t}},{}],1029:[function(t,e,n){"use strict";var r=t("../bar/hover").hoverPoints,i=t("../../plots/cartesian/axes").hoverLabelText;e.exports=function(t,e,n,o){var a=r(t,e,n,o);if(a){var s=(t=a[0]).cd[t.index],A=t.cd[0].trace;if(!A.cumulative.enabled){var l="h"===A.orientation?"y":"x";t[l+"Label"]=i(t[l+"a"],s.ph0,s.ph1)}return A.hovermplate&&(t.hovertemplate=A.hovertemplate),a}}},{"../../plots/cartesian/axes":767,"../bar/hover":863}],1030:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),layoutAttributes:t("../bar/layout_attributes"),supplyDefaults:t("./defaults"),crossTraceDefaults:t("./cross_trace_defaults"),supplyLayoutDefaults:t("../bar/layout_defaults"),calc:t("./calc").calc,crossTraceCalc:t("../bar/cross_trace_calc").crossTraceCalc,plot:t("../bar/plot").plot,layerName:"barlayer",style:t("../bar/style").style,styleOnSelect:t("../bar/style").styleOnSelect,colorbar:t("../scatter/marker_colorbar"),hoverPoints:t("./hover"),selectPoints:t("../bar/select"),eventData:t("./event_data"),moduleType:"trace",name:"histogram",basePlotModule:t("../../plots/cartesian"),categories:["bar-like","cartesian","svg","bar","histogram","oriented","errorBarsOK","showLegend"],meta:{}}},{"../../plots/cartesian":778,"../bar/cross_trace_calc":860,"../bar/layout_attributes":865,"../bar/layout_defaults":866,"../bar/plot":867,"../bar/select":868,"../bar/style":870,"../scatter/marker_colorbar":1129,"./attributes":1019,"./calc":1024,"./cross_trace_defaults":1026,"./defaults":1027,"./event_data":1028,"./hover":1029}],1031:[function(t,e,n){"use strict";e.exports={percent:function(t,e){for(var n=t.length,r=100/e,i=0;i<n;i++)t[i]*=r},probability:function(t,e){for(var n=t.length,r=0;r<n;r++)t[r]/=e},density:function(t,e,n,r){var i=t.length;r=r||1;for(var o=0;o<i;o++)t[o]*=n[o]*r},"probability density":function(t,e,n,r){var i=t.length;r&&(e/=r);for(var o=0;o<i;o++)t[o]*=n[o]/e}}},{}],1032:[function(t,e,n){"use strict";var r=t("../histogram/attributes"),i=t("../histogram/bin_attributes"),o=t("../heatmap/attributes"),a=t("../../components/fx/hovertemplate_attributes"),s=t("../../components/colorscale/attributes"),A=t("../../lib/extend").extendFlat;e.exports=A({x:r.x,y:r.y,z:{valType:"data_array",editType:"calc"},marker:{color:{valType:"data_array",editType:"calc"},editType:"calc"},histnorm:r.histnorm,histfunc:r.histfunc,nbinsx:r.nbinsx,xbins:i("x"),nbinsy:r.nbinsy,ybins:i("y"),autobinx:r.autobinx,autobiny:r.autobiny,bingroup:A({},r.bingroup,{}),xbingroup:A({},r.bingroup,{}),ybingroup:A({},r.bingroup,{}),xgap:o.xgap,ygap:o.ygap,zsmooth:o.zsmooth,zhoverformat:o.zhoverformat,hovertemplate:a({},{keys:"z"})},s("",{cLetter:"z",autoColorDflt:!1}))},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../heatmap/attributes":1001,"../histogram/attributes":1019,"../histogram/bin_attributes":1021}],1033:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axes"),o=t("../histogram/bin_functions"),a=t("../histogram/norm_functions"),s=t("../histogram/average"),A=t("../histogram/bin_label_vals"),l=t("../histogram/calc").calcAllAutoBins;function c(t,e,n,r){var i,o=new Array(t);if(r)for(i=0;i<t;i++)o[i]=1/(e[i+1]-e[i]);else{var a=1/n;for(i=0;i<t;i++)o[i]=a}return o}function u(t,e){return{start:t(e.start),end:t(e.end),size:e.size}}function g(t,e,n,r,i,o){var a,s=t.length-1,l=new Array(s),c=A(n,r,t,i,o);for(a=0;a<s;a++){var u=(e||[])[a];l[a]=void 0===u?[c(t[a]),c(t[a+1],!0)]:[u,u]}return l}e.exports=function(t,e){var n,A,h,d,f=i.getFromId(t,e.xaxis),p=i.getFromId(t,e.yaxis),I=e.xcalendar,C=e.ycalendar,m=function(t){return f.r2c(t,0,I)},y=function(t){return p.r2c(t,0,C)},E=l(t,e,f,"x"),v=E[0],B=E[1],b=l(t,e,p,"y"),Q=b[0],w=b[1],x=e._length;B.length>x&&B.splice(x,B.length-x),w.length>x&&w.splice(x,w.length-x);var _=[],D=[],M=[],k="string"==typeof v.size,S="string"==typeof Q.size,L=[],R=[],N=k?L:v,F=S?R:Q,T=0,U=[],O=[],G=e.histnorm,H=e.histfunc,j=-1!==G.indexOf("density"),Y="max"===H||"min"===H?null:0,P=o.count,K=a[G],z=!1,J=[],q=[],V="z"in e?e.z:"marker"in e&&Array.isArray(e.marker.color)?e.marker.color:"";V&&"count"!==H&&(z="avg"===H,P=o[H]);var W=v.size,X=m(v.start),Z=m(v.end)+(X-i.tickIncrement(X,W,!1,I))/1e6;for(n=X;n<Z;n=i.tickIncrement(n,W,!1,I))D.push(Y),L.push(n),z&&M.push(0);L.push(n);var $,tt=D.length,et=(n-X)/tt,nt=($=X+et/2,f.c2r($,0,I)),rt=Q.size,it=y(Q.start),ot=y(Q.end)+(it-i.tickIncrement(it,rt,!1,C))/1e6;for(n=it;n<ot;n=i.tickIncrement(n,rt,!1,C)){_.push(D.slice()),R.push(n);var at=new Array(tt);for(A=0;A<tt;A++)at[A]=[];O.push(at),z&&U.push(M.slice())}R.push(n);var st=_.length,At=(n-it)/st,lt=function(t){return p.c2r(t,0,C)}(it+At/2);j&&(J=c(D.length,N,et,k),q=c(_.length,F,At,S)),k||"date"!==f.type||(N=u(m,N)),S||"date"!==p.type||(F=u(y,F));var ct=!0,ut=!0,gt=new Array(tt),ht=new Array(st),dt=1/0,ft=1/0,pt=1/0,It=1/0;for(n=0;n<x;n++){var Ct=B[n],mt=w[n];h=r.findBin(Ct,N),d=r.findBin(mt,F),h>=0&&h<tt&&d>=0&&d<st&&(T+=P(h,n,_[d],V,U[d]),O[d][h].push(n),ct&&(void 0===gt[h]?gt[h]=Ct:gt[h]!==Ct&&(ct=!1)),ut&&(void 0===ht[d]?ht[d]=mt:ht[d]!==mt&&(ut=!1)),dt=Math.min(dt,Ct-L[h]),ft=Math.min(ft,L[h+1]-Ct),pt=Math.min(pt,mt-R[d]),It=Math.min(It,R[d+1]-mt))}if(z)for(d=0;d<st;d++)T+=s(_[d],U[d]);if(K)for(d=0;d<st;d++)K(_[d],T,J,q[d]);return{x:B,xRanges:g(L,ct&&gt,dt,ft,f,I),x0:nt,dx:et,y:w,yRanges:g(R,ut&&ht,pt,It,p,C),y0:lt,dy:At,z:_,pts:O}}},{"../../lib":719,"../../plots/cartesian/axes":767,"../histogram/average":1020,"../histogram/bin_functions":1022,"../histogram/bin_label_vals":1023,"../histogram/calc":1024,"../histogram/norm_functions":1031}],1034:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./sample_defaults"),o=t("../heatmap/style_defaults"),a=t("../../components/colorscale/defaults"),s=t("./attributes");e.exports=function(t,e,n,A){function l(n,i){return r.coerce(t,e,s,n,i)}i(t,e,l,A),!1!==e.visible&&(o(t,e,l,A),a(t,e,A,l,{prefix:"",cLetter:"z"}),l("hovertemplate"))}},{"../../components/colorscale/defaults":603,"../../lib":719,"../heatmap/style_defaults":1014,"./attributes":1032,"./sample_defaults":1037}],1035:[function(t,e,n){"use strict";var r=t("../heatmap/hover"),i=t("../../plots/cartesian/axes").hoverLabelText;e.exports=function(t,e,n,o,a,s){var A=r(t,e,n,o,a,s);if(A){var l=(t=A[0]).index,c=l[0],u=l[1],g=t.cd[0],h=g.xRanges[u],d=g.yRanges[c];return t.xLabel=i(t.xa,h[0],h[1]),t.yLabel=i(t.ya,d[0],d[1]),A}}},{"../../plots/cartesian/axes":767,"../heatmap/hover":1008}],1036:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),crossTraceDefaults:t("../histogram/cross_trace_defaults"),calc:t("../heatmap/calc"),plot:t("../heatmap/plot"),layerName:"heatmaplayer",colorbar:t("../heatmap/colorbar"),style:t("../heatmap/style"),hoverPoints:t("./hover"),eventData:t("../histogram/event_data"),moduleType:"trace",name:"histogram2d",basePlotModule:t("../../plots/cartesian"),categories:["cartesian","svg","2dMap","histogram"],meta:{}}},{"../../plots/cartesian":778,"../heatmap/calc":1002,"../heatmap/colorbar":1004,"../heatmap/plot":1012,"../heatmap/style":1013,"../histogram/cross_trace_defaults":1026,"../histogram/event_data":1028,"./attributes":1032,"./defaults":1034,"./hover":1035}],1037:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib");e.exports=function(t,e,n,o){var a=n("x"),s=n("y"),A=i.minRowLength(a),l=i.minRowLength(s);A&&l?(e._length=Math.min(A,l),r.getComponentMethod("calendars","handleTraceDefaults")(t,e,["x","y"],o),(n("z")||n("marker.color"))&&n("histfunc"),n("histnorm"),n("autobinx"),n("autobiny")):e.visible=!1}},{"../../lib":719,"../../registry":847}],1038:[function(t,e,n){"use strict";var r=t("../histogram2d/attributes"),i=t("../contour/attributes"),o=t("../../components/colorscale/attributes"),a=t("../../lib/extend").extendFlat;e.exports=a({x:r.x,y:r.y,z:r.z,marker:r.marker,histnorm:r.histnorm,histfunc:r.histfunc,nbinsx:r.nbinsx,xbins:r.xbins,nbinsy:r.nbinsy,ybins:r.ybins,autobinx:r.autobinx,autobiny:r.autobiny,bingroup:r.bingroup,xbingroup:r.xbingroup,ybingroup:r.ybingroup,autocontour:i.autocontour,ncontours:i.ncontours,contours:i.contours,line:i.line,zhoverformat:r.zhoverformat,hovertemplate:r.hovertemplate},o("",{cLetter:"z",editTypeOverride:"calc"}))},{"../../components/colorscale/attributes":600,"../../lib/extend":710,"../contour/attributes":942,"../histogram2d/attributes":1032}],1039:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../histogram2d/sample_defaults"),o=t("../contour/contours_defaults"),a=t("../contour/style_defaults"),s=t("./attributes");e.exports=function(t,e,n,A){function l(n,i){return r.coerce(t,e,s,n,i)}i(t,e,l,A),!1!==e.visible&&(o(t,e,l,function(n){return r.coerce2(t,e,s,n)}),a(t,e,l,A),l("hovertemplate"))}},{"../../lib":719,"../contour/contours_defaults":949,"../contour/style_defaults":963,"../histogram2d/sample_defaults":1037,"./attributes":1038}],1040:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),crossTraceDefaults:t("../histogram/cross_trace_defaults"),calc:t("../contour/calc"),plot:t("../contour/plot").plot,layerName:"contourlayer",style:t("../contour/style"),colorbar:t("../contour/colorbar"),hoverPoints:t("../contour/hover"),moduleType:"trace",name:"histogram2dcontour",basePlotModule:t("../../plots/cartesian"),categories:["cartesian","svg","2dMap","contour","histogram","showLegend"],meta:{}}},{"../../plots/cartesian":778,"../contour/calc":943,"../contour/colorbar":945,"../contour/hover":955,"../contour/plot":960,"../contour/style":962,"../histogram/cross_trace_defaults":1026,"./attributes":1038,"./defaults":1039}],1041:[function(t,e,n){"use strict";var r=t("../../lib/extend").extendFlat,i=t("../../lib/extend").extendDeep,o=t("../../plot_api/edit_types").overrideAll,a=t("../../plots/font_attributes"),s=t("../../components/color/attributes"),A=t("../../plots/domain").attributes,l=t("../../plots/cartesian/layout_attributes"),c=t("../../plot_api/plot_template").templatedArray,u=t("../../constants/delta.js"),g=(t("../../constants/docs").FORMAT_LINK,a({editType:"plot",colorEditType:"plot"})),h={color:{valType:"color",editType:"plot"},line:{color:{valType:"color",dflt:s.defaultLine,editType:"plot"},width:{valType:"number",min:0,dflt:0,editType:"plot"},editType:"calc"},thickness:{valType:"number",min:0,max:1,dflt:1,editType:"plot"},editType:"calc"},d={valType:"info_array",items:[{valType:"number",editType:"plot"},{valType:"number",editType:"plot"}],editType:"plot"},f=c("steps",i({},h,{range:d}));e.exports={mode:{valType:"flaglist",editType:"calc",flags:["number","delta","gauge"],dflt:"number"},value:{valType:"number",editType:"calc",anim:!0},align:{valType:"enumerated",values:["left","center","right"],editType:"plot"},domain:A({name:"indicator",trace:!0,editType:"calc"}),title:{text:{valType:"string",editType:"plot"},align:{valType:"enumerated",values:["left","center","right"],editType:"plot"},font:r({},g,{}),editType:"plot"},number:{valueformat:{valType:"string",dflt:"",editType:"plot"},font:r({},g,{}),prefix:{valType:"string",dflt:"",editType:"plot"},suffix:{valType:"string",dflt:"",editType:"plot"},editType:"plot"},delta:{reference:{valType:"number",editType:"calc"},position:{valType:"enumerated",values:["top","bottom","left","right"],dflt:"bottom",editType:"plot"},relative:{valType:"boolean",editType:"plot",dflt:!1},valueformat:{valType:"string",editType:"plot"},increasing:{symbol:{valType:"string",dflt:u.INCREASING.SYMBOL,editType:"plot"},color:{valType:"color",dflt:u.INCREASING.COLOR,editType:"plot"},editType:"plot"},decreasing:{symbol:{valType:"string",dflt:u.DECREASING.SYMBOL,editType:"plot"},color:{valType:"color",dflt:u.DECREASING.COLOR,editType:"plot"},editType:"plot"},font:r({},g,{}),editType:"calc"},gauge:{shape:{valType:"enumerated",editType:"plot",dflt:"angular",values:["angular","bullet"]},bar:i({},h,{color:{dflt:"green"}}),bgcolor:{valType:"color",editType:"plot"},bordercolor:{valType:"color",dflt:s.defaultLine,editType:"plot"},borderwidth:{valType:"number",min:0,dflt:1,editType:"plot"},axis:o({range:d,visible:r({},l.visible,{dflt:!0}),tickmode:l.tickmode,nticks:l.nticks,tick0:l.tick0,dtick:l.dtick,tickvals:l.tickvals,ticktext:l.ticktext,ticks:r({},l.ticks,{dflt:"outside"}),ticklen:l.ticklen,tickwidth:l.tickwidth,tickcolor:l.tickcolor,showticklabels:l.showticklabels,tickfont:a({}),tickangle:l.tickangle,tickformat:l.tickformat,tickformatstops:l.tickformatstops,tickprefix:l.tickprefix,showtickprefix:l.showtickprefix,ticksuffix:l.ticksuffix,showticksuffix:l.showticksuffix,separatethousands:l.separatethousands,exponentformat:l.exponentformat,showexponent:l.showexponent,editType:"plot"},"plot"),steps:f,threshold:{line:{color:r({},h.line.color,{}),width:r({},h.line.width,{dflt:1}),editType:"plot"},thickness:r({},h.thickness,{dflt:.85}),value:{valType:"number",editType:"calc",dflt:!1},editType:"plot"},editType:"plot"}}},{"../../components/color/attributes":592,"../../constants/delta.js":689,"../../constants/docs":690,"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plot_api/plot_template":757,"../../plots/cartesian/layout_attributes":779,"../../plots/domain":792,"../../plots/font_attributes":793}],1042:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../plots/get_data").getModuleCalcData,o=n.name="indicator";n.plot=function(t,e,n,a){var s=r.getModule(o),A=i(t.calcdata,s)[0];s.plot(t,A,n,a)},n.clean=function(t,e,n,r){var i=r._has&&r._has(o),a=e._has&&e._has(o);i&&!a&&r._indicatorlayer.selectAll("g.trace").remove()}},{"../../plots/get_data":802,"../../registry":847}],1043:[function(t,e,n){"use strict";e.exports={calc:function(t,e){var n=[],r=e.value;"number"!=typeof e._lastValue&&(e._lastValue=e.value);var i=e._lastValue,o=i;return e._hasDelta&&"number"==typeof e.delta.reference&&(o=e.delta.reference),n[0]={y:r,lastY:i,delta:r-o,relativeDelta:(r-o)/o},n}}},{}],1044:[function(t,e,n){"use strict";e.exports={defaultNumberFontSize:80,bulletNumberDomainSize:.25,bulletPadding:.025,innerRadius:.75,valueThickness:.5,titlePadding:5,horizontalPadding:10}},{}],1045:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./attributes"),o=t("../../plots/domain").defaults,a=t("../../plot_api/plot_template"),s=t("../../plots/array_container_defaults"),A=t("./constants.js"),l=t("../../plots/cartesian/tick_value_defaults"),c=t("../../plots/cartesian/tick_mark_defaults"),u=t("../../plots/cartesian/tick_label_defaults");function g(t,e){function n(n,o){return r.coerce(t,e,i.gauge.steps,n,o)}n("color"),n("line.color"),n("line.width"),n("range"),n("thickness")}e.exports={supplyDefaults:function(t,e,n,h){function d(n,o){return r.coerce(t,e,i,n,o)}o(e,h,d),d("mode"),e._hasNumber=-1!==e.mode.indexOf("number"),e._hasDelta=-1!==e.mode.indexOf("delta"),e._hasGauge=-1!==e.mode.indexOf("gauge");var f=d("value");e._range=[0,"number"==typeof f?1.5*f:1];var p,I,C,m,y,E,v=new Array(2);function B(t,e){return r.coerce(C,m,i.gauge,t,e)}function b(t,e){return r.coerce(y,E,i.gauge.axis,t,e)}if(e._hasNumber&&(d("number.valueformat"),d("number.font.color",h.font.color),d("number.font.family",h.font.family),d("number.font.size"),void 0===e.number.font.size&&(e.number.font.size=A.defaultNumberFontSize,v[0]=!0),d("number.prefix"),d("number.suffix"),p=e.number.font.size),e._hasDelta&&(d("delta.font.color",h.font.color),d("delta.font.family",h.font.family),d("delta.font.size"),void 0===e.delta.font.size&&(e.delta.font.size=(e._hasNumber?.5:1)*(p||A.defaultNumberFontSize),v[1]=!0),d("delta.reference",e.value),d("delta.relative"),d("delta.valueformat",e.delta.relative?"2%":""),d("delta.increasing.symbol"),d("delta.increasing.color"),d("delta.decreasing.symbol"),d("delta.decreasing.color"),d("delta.position"),I=e.delta.font.size),e._scaleNumbers=(!e._hasNumber||v[0])&&(!e._hasDelta||v[1])||!1,d("title.font.color",h.font.color),d("title.font.family",h.font.family),d("title.font.size",.25*(p||I||A.defaultNumberFontSize)),d("title.text"),e._hasGauge){(C=t.gauge)||(C={}),m=a.newContainer(e,"gauge"),B("shape"),(e._isBullet="bullet"===e.gauge.shape)||d("title.align","center"),(e._isAngular="angular"===e.gauge.shape)||d("align","center"),B("bgcolor",h.paper_bgcolor),B("borderwidth"),B("bordercolor"),B("bar.color"),B("bar.line.color"),B("bar.line.width"),B("bar.thickness",A.valueThickness*("bullet"===e.gauge.shape?.5:1)),s(C,m,{name:"steps",handleItemDefaults:g}),B("threshold.value"),B("threshold.thickness"),B("threshold.line.width"),B("threshold.line.color"),y={},C&&(y=C.axis||{}),E=a.newContainer(m,"axis"),b("visible"),e._range=b("range",e._range);var Q={outerTicks:!0};l(y,E,b,"linear"),u(y,E,b,"linear",Q),c(y,E,b,Q)}else d("title.align","center"),d("align","center"),e._isAngular=e._isBullet=!1;e._length=null}}},{"../../lib":719,"../../plot_api/plot_template":757,"../../plots/array_container_defaults":763,"../../plots/cartesian/tick_label_defaults":786,"../../plots/cartesian/tick_mark_defaults":787,"../../plots/cartesian/tick_value_defaults":788,"../../plots/domain":792,"./attributes":1041,"./constants.js":1044}],1046:[function(t,e,n){"use strict";e.exports={moduleType:"trace",name:"indicator",basePlotModule:t("./base_plot"),categories:["svg","noOpacity","noHover"],animatable:!0,attributes:t("./attributes"),supplyDefaults:t("./defaults").supplyDefaults,calc:t("./calc").calc,plot:t("./plot"),meta:{}}},{"./attributes":1041,"./base_plot":1042,"./calc":1043,"./defaults":1045,"./plot":1047}],1047:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../lib"),o=i.rad2deg,a=t("../../constants/alignment").MID_SHIFT,s=t("../../components/drawing"),A=t("./constants"),l=t("../../lib/svg_text_utils"),c=t("../../plots/cartesian/axes"),u=t("../../plots/cartesian/axis_defaults"),g=t("../../plots/cartesian/position_defaults"),h=t("../../plots/cartesian/layout_attributes"),d=t("../../components/color"),f={left:"start",center:"middle",right:"end"},p={left:0,center:.5,right:1},I=/[yzafpn\xb5mkMGTPEZY]/;function C(t){return t&&t.duration>0}function m(t){t.each(function(t){d.stroke(r.select(this),t.line.color)}).each(function(t){d.fill(r.select(this),t.color)}).style("stroke-width",function(t){return t.line.width})}function y(t,e,n){var r=t._fullLayout,o=i.extendFlat({type:"linear",ticks:"outside",range:n,showline:!0},e),a={type:"linear",_id:"x"+e._id},s={letter:"x",font:r.font,noHover:!0,noTickson:!0};function A(t,e){return i.coerce(o,a,h,t,e)}return u(o,a,A,s,r),g(o,a,A,s),a}function E(t,e){return"translate("+t+","+e+")"}function v(t,e,n){return[Math.min(e/t.width,n/t.height),t,e+"x"+n]}function B(t,e,n,i){var o=document.createElementNS("http://www.w3.org/2000/svg","text"),a=r.select(o);return a.text(t).attr("x",0).attr("y",0).attr("text-anchor",n).attr("data-unformatted",t).call(l.convertToTspans,i).call(s.font,e),s.bBox(a.node())}function b(t,e,n,r,o,a){var s="_cache"+e;t[s]&&t[s].key===o||(t[s]={key:o,value:n});var A=i.aggNums(a,null,[t[s].value,r],2);return t[s].value=A,A}e.exports=function(t,e,n,u){var g,h=t._fullLayout;C(n)&&u&&(g=u()),i.makeTraceGroups(h._indicatorlayer,e,"trace").each(function(e){var u,Q,w,x,_,D=e[0].trace,M=r.select(this),k=D._hasGauge,S=D._isAngular,L=D._isBullet,R=D.domain,N={w:h._size.w*(R.x[1]-R.x[0]),h:h._size.h*(R.y[1]-R.y[0]),l:h._size.l+h._size.w*R.x[0],r:h._size.r+h._size.w*(1-R.x[1]),t:h._size.t+h._size.h*(1-R.y[1]),b:h._size.b+h._size.h*R.y[0]},F=N.l+N.w/2,T=N.t+N.h/2,U=Math.min(N.w/2,N.h),O=A.innerRadius*U,G=D.align||"center";if(Q=T,k){if(S&&(u=F,Q=T+U/2,w=function(t){return e=t,n=.9*O,r=Math.sqrt(e.width/2*(e.width/2)+e.height*e.height),[n/r,e,n];var e,n,r}),L){var H=A.bulletPadding,j=1-A.bulletNumberDomainSize+H;u=N.l+(j+(1-j)*p[G])*N.w,w=function(t){return v(t,(A.bulletNumberDomainSize-H)*N.w,N.h)}}}else u=N.l+p[G]*N.w,w=function(t){return v(t,N.w,N.h)};!function(t,e,n,o){var a,A,u,g=n[0].trace,h=o.numbersX,m=o.numbersY,v=g.align||"center",Q=f[v],w=o.transitionOpts,x=o.onComplete,_=i.ensureSingle(e,"g","numbers"),D=[];g._hasNumber&&D.push("number"),g._hasDelta&&(D.push("delta"),"left"===g.delta.position&&D.reverse());var M=_.selectAll("text").data(D);function k(e,n,r,i){if(!e.match("s")||r>=0==i>=0||n(r).slice(-1).match(I)||n(i).slice(-1).match(I))return n;var o=e.slice().replace("s","f").replace(/\d+/,function(t){return parseInt(t)-1}),a=y(t,{tickformat:o});return function(t){return Math.abs(t)<1?c.tickText(a,t).text:n(t)}}M.enter().append("text"),M.attr("text-anchor",function(){return Q}).attr("class",function(t){return t}).attr("x",null).attr("y",null).attr("dx",null).attr("dy",null),M.exit().remove();var S,L=g.mode+g.align;if(g._hasDelta&&(S=function(){var e=y(t,{tickformat:g.delta.valueformat},g._range);e.setScale(),c.calcTicks(e);var i=function(t){return c.tickText(e,t).text},o=function(t){var e=g.delta.relative?t.relativeDelta:t.delta;return e},a=function(t,e){return 0===t||"number"!=typeof t||isNaN(t)?"-":(t>0?g.delta.increasing.symbol:g.delta.decreasing.symbol)+e(t)},u=function(t){return t.delta>=0?g.delta.increasing.color:g.delta.decreasing.color};void 0===g._deltaLastValue&&(g._deltaLastValue=o(n[0]));var h=_.select("text.delta");function f(){h.text(a(o(n[0]),i)).call(d.fill,u(n[0])).call(l.convertToTspans,t)}return h.call(s.font,g.delta.font).call(d.fill,u({delta:g._deltaLastValue})),C(w)?h.transition().duration(w.duration).ease(w.easing).tween("text",function(){var t=r.select(this),e=o(n[0]),s=g._deltaLastValue,A=k(g.delta.valueformat,i,s,e),l=r.interpolateNumber(s,e);return g._deltaLastValue=e,function(e){t.text(a(l(e),A)),t.call(d.fill,u({delta:l(e)}))}}).each("end",function(){f(),x&&x()}).each("interrupt",function(){f(),x&&x()}):f(),A=B(a(o(n[0]),i),g.delta.font,Q,t),h}(),L+=g.delta.position+g.delta.font.size+g.delta.font.family+g.delta.valueformat,L+=g.delta.increasing.symbol+g.delta.decreasing.symbol,u=A),g._hasNumber&&(function(){var e=y(t,{tickformat:g.number.valueformat},g._range);e.setScale(),c.calcTicks(e);var i=function(t){return c.tickText(e,t).text},o=g.number.suffix,A=g.number.prefix,u=_.select("text.number");function h(){var e="number"==typeof n[0].y?A+i(n[0].y)+o:"-";u.text(e).call(s.font,g.number.font).call(l.convertToTspans,t)}C(w)?u.transition().duration(w.duration).ease(w.easing).each("end",function(){h(),x&&x()}).each("interrupt",function(){h(),x&&x()}).attrTween("text",function(){var t=r.select(this),e=r.interpolateNumber(n[0].lastY,n[0].y);g._lastValue=n[0].y;var a=k(g.number.valueformat,i,n[0].lastY,n[0].y);return function(n){t.text(A+a(e(n))+o)}}):h(),a=B(A+i(n[0].y)+o,g.number.font,Q,t)}(),L+=g.number.font.size+g.number.font.family+g.number.valueformat+g.number.suffix+g.number.prefix,u=a),g._hasDelta&&g._hasNumber){var R,N,F=[(a.left+a.right)/2,(a.top+a.bottom)/2],T=[(A.left+A.right)/2,(A.top+A.bottom)/2],U=.75*g.delta.font.size;"left"===g.delta.position&&(R=b(g,"deltaPos",0,-1*(a.width*p[g.align]+A.width*(1-p[g.align])+U),L,Math.min),N=F[1]-T[1],u={width:a.width+A.width+U,height:Math.max(a.height,A.height),left:A.left+R,right:a.right,top:Math.min(a.top,A.top+N),bottom:Math.max(a.bottom,A.bottom+N)}),"right"===g.delta.position&&(R=b(g,"deltaPos",0,a.width*(1-p[g.align])+A.width*p[g.align]+U,L,Math.max),N=F[1]-T[1],u={width:a.width+A.width+U,height:Math.max(a.height,A.height),left:a.left,right:A.right+R,top:Math.min(a.top,A.top+N),bottom:Math.max(a.bottom,A.bottom+N)}),"bottom"===g.delta.position&&(R=null,N=A.height,u={width:Math.max(a.width,A.width),height:a.height+A.height,left:Math.min(a.left,A.left),right:Math.max(a.right,A.right),top:a.bottom-a.height,bottom:a.bottom+A.height}),"top"===g.delta.position&&(R=null,N=a.top,u={width:Math.max(a.width,A.width),height:a.height+A.height,left:Math.min(a.left,A.left),right:Math.max(a.right,A.right),top:a.bottom-a.height-A.height,bottom:a.bottom}),S.attr({dx:R,dy:N})}(g._hasNumber||g._hasDelta)&&_.attr("transform",function(){var t=o.numbersScaler(u);L+=t[2];var e,n=b(g,"numbersScale",1,t[0],L,Math.min);g._scaleNumbers||(n=1),e=g._isAngular?m-n*u.bottom:m-n*(u.top+u.bottom)/2,g._numbersTop=n*u.top+e;var r=u[v];"center"===v&&(r=(u.left+u.right)/2);var i=h-n*r;return E(i=b(g,"numbersTranslate",0,i,L,Math.max),e)+" scale("+n+")"})}(t,M,e,{numbersX:u,numbersY:Q,numbersScaler:w,transitionOpts:n,onComplete:g}),k&&(x={range:D.gauge.axis.range,color:D.gauge.bgcolor,line:{color:D.gauge.bordercolor,width:0},thickness:1},_={range:D.gauge.axis.range,color:"rgba(0, 0, 0, 0)",line:{color:D.gauge.bordercolor,width:D.gauge.borderwidth},thickness:1});var Y=M.selectAll("g.angular").data(S?e:[]);Y.exit().remove();var P=M.selectAll("g.angularaxis").data(S?e:[]);P.exit().remove(),S&&function(t,e,n,i){var s,A,l,u,g=n[0].trace,h=i.size,d=i.radius,f=i.innerRadius,p=i.gaugeBg,I=i.gaugeOutline,v=[h.l+h.w/2,h.t+h.h/2+d/2],B=i.gauge,b=i.layer,Q=i.transitionOpts,w=i.onComplete,x=Math.PI/2;function _(t){var e=g.gauge.axis.range[0],n=g.gauge.axis.range[1],r=(t-e)/(n-e)*Math.PI-x;return r<-x?-x:r>x?x:r}function D(t){return r.svg.arc().innerRadius((f+d)/2-t/2*(d-f)).outerRadius((f+d)/2+t/2*(d-f)).startAngle(-x)}function M(t){t.attr("d",function(t){return D(t.thickness).startAngle(_(t.range[0])).endAngle(_(t.range[1]))()})}B.enter().append("g").classed("angular",!0),B.attr("transform",E(v[0],v[1])),b.enter().append("g").classed("angularaxis",!0).classed("crisp",!0),b.selectAll("g.xangularaxistick,path,text").remove(),(s=y(t,g.gauge.axis)).type="linear",s.range=g.gauge.axis.range,s._id="xangularaxis",s.setScale();var k=function(t){return(s.range[0]-t.x)/(s.range[1]-s.range[0])*Math.PI+Math.PI},S={},L=c.makeLabelFns(s,0).labelStandoff;S.xFn=function(t){var e=k(t);return Math.cos(e)*L},S.yFn=function(t){var e=k(t),n=Math.sin(e)>0?.2:1;return-Math.sin(e)*(L+t.fontSize*n)+Math.abs(Math.cos(e))*(t.fontSize*a)},S.anchorFn=function(t){var e=k(t),n=Math.cos(e);return Math.abs(n)<.1?"middle":n>0?"start":"end"},S.heightFn=function(t,e,n){var r=k(t);return-.5*(1+Math.sin(r))*n};var R=function(t){return E(v[0]+d*Math.cos(t),v[1]-d*Math.sin(t))};if(l=function(t){return R(k(t))},A=c.calcTicks(s),u=c.getTickSigns(s)[2],s.visible){u="inside"===s.ticks?-1:1;var N=(s.linewidth||1)/2;c.drawTicks(t,s,{vals:A,layer:b,path:"M"+u*N+",0h"+u*s.ticklen,transFn:function(t){var e=k(t);return R(e)+"rotate("+-o(e)+")"}}),c.drawLabels(t,s,{vals:A,layer:b,transFn:l,labelFns:S})}var F=[p].concat(g.gauge.steps),T=B.selectAll("g.bg-arc").data(F);T.enter().append("g").classed("bg-arc",!0).append("path"),T.select("path").call(M).call(m),T.exit().remove();var U=D(g.gauge.bar.thickness),O=B.selectAll("g.value-arc").data([g.gauge.bar]);O.enter().append("g").classed("value-arc",!0).append("path");var G,H,j,Y=O.select("path");C(Q)?(Y.transition().duration(Q.duration).ease(Q.easing).each("end",function(){w&&w()}).each("interrupt",function(){w&&w()}).attrTween("d",(G=U,H=_(n[0].lastY),j=_(n[0].y),function(){var t=r.interpolate(H,j);return function(e){return G.endAngle(t(e))()}})),g._lastValue=n[0].y):Y.attr("d","number"==typeof n[0].y?U.endAngle(_(n[0].y)):"M0,0Z"),Y.call(m),O.exit().remove(),F=[];var P=g.gauge.threshold.value;P&&F.push({range:[P,P],color:g.gauge.threshold.color,line:{color:g.gauge.threshold.line.color,width:g.gauge.threshold.line.width},thickness:g.gauge.threshold.thickness});var K=B.selectAll("g.threshold-arc").data(F);K.enter().append("g").classed("threshold-arc",!0).append("path"),K.select("path").call(M).call(m),K.exit().remove();var z=B.selectAll("g.gauge-outline").data([I]);z.enter().append("g").classed("gauge-outline",!0).append("path"),z.select("path").call(M).call(m),z.exit().remove()}(t,0,e,{radius:U,innerRadius:O,gauge:Y,layer:P,size:N,gaugeBg:x,gaugeOutline:_,transitionOpts:n,onComplete:g});var K=M.selectAll("g.bullet").data(L?e:[]);K.exit().remove();var z=M.selectAll("g.bulletaxis").data(L?e:[]);z.exit().remove(),L&&function(t,e,n,r){var i,o,a,s,l,u=n[0].trace,g=r.gauge,h=r.layer,f=r.gaugeBg,p=r.gaugeOutline,I=r.size,E=u.domain,v=r.transitionOpts,B=r.onComplete;g.enter().append("g").classed("bullet",!0),g.attr("transform","translate("+I.l+", "+I.t+")"),h.enter().append("g").classed("bulletaxis",!0).classed("crisp",!0),h.selectAll("g.xbulletaxistick,path,text").remove();var b=I.h,Q=u.gauge.bar.thickness*b,w=E.x[0],x=E.x[0]+(E.x[1]-E.x[0])*(u._hasNumber||u._hasDelta?1-A.bulletNumberDomainSize:1);function _(t){t.attr("width",function(t){return Math.max(0,i.c2p(t.range[1])-i.c2p(t.range[0]))}).attr("x",function(t){return i.c2p(t.range[0])}).attr("y",function(t){return.5*(1-t.thickness)*b}).attr("height",function(t){return t.thickness*b})}(i=y(t,u.gauge.axis))._id="xbulletaxis",i.domain=[w,x],i.setScale(),o=c.calcTicks(i),a=c.makeTransFn(i),s=c.getTickSigns(i)[2],l=I.t+I.h,i.visible&&(c.drawTicks(t,i,{vals:"inside"===i.ticks?c.clipEnds(i,o):o,layer:h,path:c.makeTickPath(i,l,s),transFn:a}),c.drawLabels(t,i,{vals:o,layer:h,transFn:a,labelFns:c.makeLabelFns(i,l)}));var D=[f].concat(u.gauge.steps),M=g.selectAll("g.bg-bullet").data(D);M.enter().append("g").classed("bg-bullet",!0).append("rect"),M.select("rect").call(_).call(m),M.exit().remove();var k=g.selectAll("g.value-bullet").data([u.gauge.bar]);k.enter().append("g").classed("value-bullet",!0).append("rect"),k.select("rect").attr("height",Q).attr("y",(b-Q)/2).call(m),C(v)?k.select("rect").transition().duration(v.duration).ease(v.easing).each("end",function(){B&&B()}).each("interrupt",function(){B&&B()}).attr("width",Math.max(0,i.c2p(Math.min(u.gauge.axis.range[1],n[0].y)))):k.select("rect").attr("width","number"==typeof n[0].y?Math.max(0,i.c2p(Math.min(u.gauge.axis.range[1],n[0].y))):0),k.exit().remove();var S=n.filter(function(){return u.gauge.threshold.value}),L=g.selectAll("g.threshold-bullet").data(S);L.enter().append("g").classed("threshold-bullet",!0).append("line"),L.select("line").attr("x1",i.c2p(u.gauge.threshold.value)).attr("x2",i.c2p(u.gauge.threshold.value)).attr("y1",(1-u.gauge.threshold.thickness)/2*b).attr("y2",(1-(1-u.gauge.threshold.thickness)/2)*b).call(d.stroke,u.gauge.threshold.line.color).style("stroke-width",u.gauge.threshold.line.width),L.exit().remove();var R=g.selectAll("g.gauge-outline").data([p]);R.enter().append("g").classed("gauge-outline",!0).append("rect"),R.select("rect").call(_).call(m),R.exit().remove()}(t,0,e,{gauge:K,layer:z,size:N,gaugeBg:x,gaugeOutline:_,transitionOpts:n,onComplete:g});var J=M.selectAll("text.title").data(e);J.exit().remove(),J.enter().append("text").classed("title",!0),J.attr("text-anchor",function(){return L?f.right:f[D.title.align]}).text(D.title.text).call(s.font,D.title.font).call(l.convertToTspans,t),J.attr("transform",function(){var t,e=N.l+N.w*p[D.title.align],n=A.titlePadding,r=s.bBox(J.node());return k?(S&&(t=D.gauge.axis.visible?s.bBox(P.node()).top-n-r.bottom:N.t+N.h/2-U/2-r.bottom-n),L&&(t=Q-(r.top+r.bottom)/2,e=N.l-A.bulletPadding*N.w)):t=D._numbersTop-n-r.bottom,E(e,t)})})}},{"../../components/color":593,"../../components/drawing":614,"../../constants/alignment":688,"../../lib":719,"../../lib/svg_text_utils":743,"../../plots/cartesian/axes":767,"../../plots/cartesian/axis_defaults":769,"../../plots/cartesian/layout_attributes":779,"../../plots/cartesian/position_defaults":782,"./constants":1044,d3:163}],1048:[function(t,e,n){"use strict";var r=t("../../components/colorscale/attributes"),i=t("../../components/fx/hovertemplate_attributes"),o=t("../mesh3d/attributes"),a=t("../../plots/attributes"),s=t("../../lib/extend").extendFlat,A=t("../../plot_api/edit_types").overrideAll,l=e.exports=A(s({x:{valType:"data_array"},y:{valType:"data_array"},z:{valType:"data_array"},value:{valType:"data_array"},isomin:{valType:"number"},isomax:{valType:"number"},surface:{show:{valType:"boolean",dflt:!0},count:{valType:"integer",dflt:2,min:1},fill:{valType:"number",min:0,max:1,dflt:1},pattern:{valType:"flaglist",flags:["A","B","C","D","E"],extras:["all","odd","even"],dflt:"all"}},spaceframe:{show:{valType:"boolean",dflt:!1},fill:{valType:"number",min:0,max:1,dflt:.15}},slices:{x:{show:{valType:"boolean",dflt:!1},locations:{valType:"data_array",dflt:[]},fill:{valType:"number",min:0,max:1,dflt:1}},y:{show:{valType:"boolean",dflt:!1},locations:{valType:"data_array",dflt:[]},fill:{valType:"number",min:0,max:1,dflt:1}},z:{show:{valType:"boolean",dflt:!1},locations:{valType:"data_array",dflt:[]},fill:{valType:"number",min:0,max:1,dflt:1}}},caps:{x:{show:{valType:"boolean",dflt:!0},fill:{valType:"number",min:0,max:1,dflt:1}},y:{show:{valType:"boolean",dflt:!0},fill:{valType:"number",min:0,max:1,dflt:1}},z:{show:{valType:"boolean",dflt:!0},fill:{valType:"number",min:0,max:1,dflt:1}}},text:{valType:"string",dflt:"",arrayOk:!0},hovertext:{valType:"string",dflt:"",arrayOk:!0},hovertemplate:i()},r("",{colorAttr:"`value`",showScaleDflt:!0,editTypeOverride:"calc"}),{opacity:o.opacity,lightposition:o.lightposition,lighting:o.lighting,flatshading:o.flatshading,contour:o.contour,hoverinfo:s({},a.hoverinfo)}),"calc","nested");l.flatshading.dflt=!0,l.lighting.facenormalsepsilon.dflt=0,l.x.editType=l.y.editType=l.z.editType=l.value.editType="calc+clearAxisTypes",l.transforms=void 0},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plots/attributes":764,"../mesh3d/attributes":1053}],1049:[function(t,e,n){"use strict";var r=t("../../components/colorscale/calc");e.exports=function(t,e){e._len=Math.min(e.x.length,e.y.length,e.z.length,e.value.length);for(var n=1/0,i=-1/0,o=e.value.length,a=0;a<o;a++){var s=e.value[a];n=Math.min(n,s),i=Math.max(i,s)}e._minValues=n,e._maxValues=i,e._vMin=void 0===e.isomin||null===e.isomin?n:e.isomin,e._vMax=void 0===e.isomax||null===e.isomin?i:e.isomax,r(t,e,{vals:[e._vMin,e._vMax],containerStr:"",cLetter:"c"})}},{"../../components/colorscale/calc":601}],1050:[function(t,e,n){"use strict";var r=t("gl-mesh3d"),i=t("../../lib"),o=t("../../lib/gl_format_color").parseColorScale,a=t("../../lib/str2rgbarray"),s=t("../../components/colorscale").extractOpts,A=t("../../plots/gl3d/zip3");function l(t){return i.distinctVals(t).vals}var c=function(t,e){for(var n=e.length-1;n>0;n--){var r=Math.min(e[n],e[n-1]),i=Math.max(e[n],e[n-1]);if(i>r&&r<t&&t<=i)return{id:n,distRatio:(i-t)/(i-r)}}return{id:0,distRatio:0}};function u(t,e,n){this.scene=t,this.uid=n,this.mesh=e,this.name="",this.data=null,this.showContour=!1}var g=u.prototype;function h(t){t._i=[],t._j=[],t._k=[];var e,n,r=t.surface.show,i=t.spaceframe.show,o=t.surface.fill,a=t.spaceframe.fill,s=!1,A=!1,u=0,g=l(t.x.slice(0,t._len)),h=l(t.y.slice(0,t._len)),d=l(t.z.slice(0,t._len)),f=g.length,p=h.length,I=d.length;function C(t,e,n){return n+I*e+I*p*t}var m,y,E,v,B,b=t._minValues,Q=t._maxValues,w=t._vMin,x=t._vMax;function _(t,e,r){for(var i=v.length,o=n;o<i;o++)if(t===m[o]&&e===y[o]&&r===E[o])return o;return-1}function D(){n=e}function M(){m=[],y=[],E=[],v=[],e=0,D()}function k(t,n,r,i){return m.push(t),y.push(n),E.push(r),v.push(i),++e-1}function S(t,e,n){for(var r=[],i=0;i<t.length;i++)r[i]=t[i]*(1-n)+n*e[i];return r}function L(t){B=t}function R(t,e){return"all"===t||null===t||t.indexOf(e)>-1}function N(t,e){return null===t?e:t}function F(e,n,r){D();var i,o,a,s=[n],A=[r];if(B>=1)s=[n],A=[r];else if(B>0){var l=function(t,e){var n=t[0],r=t[1],i=t[2],o=function(t,e,n){for(var r=[],i=0;i<t.length;i++)r[i]=(t[i]+e[i]+n[i])/3;return r}(n,r,i),a=Math.sqrt(1-B),s=S(o,n,a),A=S(o,r,a),l=S(o,i,a),c=e[0],u=e[1],g=e[2];return{xyzv:[[n,r,A],[A,s,n],[r,i,l],[l,A,r],[i,n,s],[s,l,i]],abc:[[c,u,-1],[-1,-1,c],[u,g,-1],[-1,-1,u],[g,c,-1],[-1,-1,g]]}}(n,r);s=l.xyzv,A=l.abc}for(var c=0;c<s.length;c++){n=s[c],r=A[c];for(var g=[],h=0;h<3;h++){var d=n[h][0],f=n[h][1],p=n[h][2],I=n[h][3],C=r[h]>-1?r[h]:_(d,f,p);g[h]=C>-1?C:k(d,f,p,N(e,I))}i=g[0],o=g[1],a=g[2],t._i.push(i),t._j.push(o),t._k.push(a),++u}}function T(t,e,n,r){var i=t[3];i<n&&(i=n),i>r&&(i=r);for(var o=(t[3]-i)/(t[3]-e[3]+1e-9),a=[],s=0;s<4;s++)a[s]=(1-o)*t[s]+o*e[s];return a}function U(t,e,n){return t>=e&&t<=n}function O(t){var e=.001*(x-w);return t>=w-e&&t<=x+e}function G(e){for(var n=[],r=0;r<4;r++){var i=e[r];n.push([t.x[i],t.y[i],t.z[i],t.value[i]])}return n}var H=3;function j(t,e,n,r,i,o){o||(o=1),n=[-1,-1,-1];var a=!1,s=[U(e[0][3],r,i),U(e[1][3],r,i),U(e[2][3],r,i)];if(!s[0]&&!s[1]&&!s[2])return!1;var A=function(t,e,n){return O(e[0][3])&&O(e[1][3])&&O(e[2][3])?(F(t,e,n),!0):o<H&&j(t,e,n,w,x,++o)};if(s[0]&&s[1]&&s[2])return A(t,e,n)||a;var l=!1;return[[0,1,2],[2,0,1],[1,2,0]].forEach(function(o){if(s[o[0]]&&s[o[1]]&&!s[o[2]]){var c=e[o[0]],u=e[o[1]],g=e[o[2]],h=T(g,c,r,i),d=T(g,u,r,i);a=A(t,[d,h,c],[-1,-1,n[o[0]]])||a,a=A(t,[c,u,d],[n[o[0]],n[o[1]],-1])||a,l=!0}}),l?a:([[0,1,2],[1,2,0],[2,0,1]].forEach(function(o){if(s[o[0]]&&!s[o[1]]&&!s[o[2]]){var c=e[o[0]],u=e[o[1]],g=e[o[2]],h=T(u,c,r,i),d=T(g,c,r,i);a=A(t,[d,h,c],[-1,-1,n[o[0]]])||a,l=!0}}),a)}function Y(t,e,n,r){var i=!1,o=G(e),a=[U(o[0][3],n,r),U(o[1][3],n,r),U(o[2][3],n,r),U(o[3][3],n,r)];if(!(a[0]||a[1]||a[2]||a[3]))return i;if(a[0]&&a[1]&&a[2]&&a[3])return A&&(i=function(t,e,n){var r=function(r,i,o){F(t,[e[r],e[i],e[o]],[n[r],n[i],n[o]])};r(0,1,2),r(3,0,1),r(2,3,0),r(1,2,3)}(t,o,e)||i),i;var s=!1;return[[0,1,2,3],[3,0,1,2],[2,3,0,1],[1,2,3,0]].forEach(function(l){if(a[l[0]]&&a[l[1]]&&a[l[2]]&&!a[l[3]]){var c=o[l[0]],u=o[l[1]],g=o[l[2]],h=o[l[3]];if(A)i=F(t,[c,u,g],[e[l[0]],e[l[1]],e[l[2]]])||i;else{var d=T(h,c,n,r),f=T(h,u,n,r),p=T(h,g,n,r);i=F(null,[d,f,p],[-1,-1,-1])||i}s=!0}}),s?i:([[0,1,2,3],[1,2,3,0],[2,3,0,1],[3,0,1,2],[0,2,3,1],[1,3,2,0]].forEach(function(l){if(a[l[0]]&&a[l[1]]&&!a[l[2]]&&!a[l[3]]){var c=o[l[0]],u=o[l[1]],g=o[l[2]],h=o[l[3]],d=T(g,c,n,r),f=T(g,u,n,r),p=T(h,u,n,r),I=T(h,c,n,r);A?(i=F(t,[c,I,d],[e[l[0]],-1,-1])||i,i=F(t,[u,f,p],[e[l[1]],-1,-1])||i):i=function(t,e,n){var r=function(t,r,i){F(null,[e[t],e[r],e[i]],[n[t],n[r],n[i]])};r(0,1,2),r(2,3,0)}(0,[d,f,p,I],[-1,-1,-1,-1])||i,s=!0}}),s?i:([[0,1,2,3],[1,2,3,0],[2,3,0,1],[3,0,1,2]].forEach(function(l){if(a[l[0]]&&!a[l[1]]&&!a[l[2]]&&!a[l[3]]){var c=o[l[0]],u=o[l[1]],g=o[l[2]],h=o[l[3]],d=T(u,c,n,r),f=T(g,c,n,r),p=T(h,c,n,r);A?(i=F(t,[c,d,f],[e[l[0]],-1,-1])||i,i=F(t,[c,f,p],[e[l[0]],-1,-1])||i,i=F(t,[c,p,d],[e[l[0]],-1,-1])||i):i=F(null,[d,f,p],[-1,-1,-1])||i,s=!0}}),i))}function P(t,e,n,r,i,o,a,l,c,u,g){var h=!1;return s&&(R(t,"A")&&(h=Y(null,[e,n,r,o],u,g)||h),R(t,"B")&&(h=Y(null,[n,r,i,c],u,g)||h),R(t,"C")&&(h=Y(null,[n,o,a,c],u,g)||h),R(t,"D")&&(h=Y(null,[r,o,l,c],u,g)||h),R(t,"E")&&(h=Y(null,[n,r,o,c],u,g)||h)),A&&(h=Y(t,[n,r,o,c],u,g)||h),h}function K(t,e,n,r,i,o,a,s){return[!0===s[0]||j(t,G([e,n,r]),[e,n,r],o,a),!0===s[1]||j(t,G([r,i,e]),[r,i,e],o,a)]}function z(t,e,n,r,i,o,a,s,A){return s?K(t,e,n,i,r,o,a,A):K(t,n,i,r,e,o,a,A)}function J(t,e,n,r,i,o,a){var s,A,l,c,u=!1,g=function(){u=j(t,[s,A,l],[-1,-1,-1],i,o)||u,u=j(t,[l,c,s],[-1,-1,-1],i,o)||u},h=a[0],d=a[1],f=a[2];return h&&(s=S(G([C(e,n-0,r-0)])[0],G([C(e-1,n-0,r-0)])[0],h),A=S(G([C(e,n-0,r-1)])[0],G([C(e-1,n-0,r-1)])[0],h),l=S(G([C(e,n-1,r-1)])[0],G([C(e-1,n-1,r-1)])[0],h),c=S(G([C(e,n-1,r-0)])[0],G([C(e-1,n-1,r-0)])[0],h),g()),d&&(s=S(G([C(e-0,n,r-0)])[0],G([C(e-0,n-1,r-0)])[0],d),A=S(G([C(e-0,n,r-1)])[0],G([C(e-0,n-1,r-1)])[0],d),l=S(G([C(e-1,n,r-1)])[0],G([C(e-1,n-1,r-1)])[0],d),c=S(G([C(e-1,n,r-0)])[0],G([C(e-1,n-1,r-0)])[0],d),g()),f&&(s=S(G([C(e-0,n-0,r)])[0],G([C(e-0,n-0,r-1)])[0],f),A=S(G([C(e-0,n-1,r)])[0],G([C(e-0,n-1,r-1)])[0],f),l=S(G([C(e-1,n-1,r)])[0],G([C(e-1,n-1,r-1)])[0],f),c=S(G([C(e-1,n-0,r)])[0],G([C(e-1,n-0,r-1)])[0],f),g()),u}function q(t,e,n,r,i,o,a,A,l,c,u,g){var h=t;return g?(s&&"even"===t&&(h=null),P(h,e,n,r,i,o,a,A,l,c,u)):(s&&"odd"===t&&(h=null),P(h,l,A,a,o,i,r,n,e,c,u))}function V(t,e,n,r,i){for(var o=[],a=0,s=0;s<e.length;s++)for(var A=e[s],l=1;l<I;l++)for(var c=1;c<p;c++)o.push(z(t,C(A,c-1,l-1),C(A,c-1,l),C(A,c,l-1),C(A,c,l),n,r,(A+c+l)%2,i&&i[a]?i[a]:[])),a++;return o}function W(t,e,n,r,i){for(var o=[],a=0,s=0;s<e.length;s++)for(var A=e[s],l=1;l<f;l++)for(var c=1;c<I;c++)o.push(z(t,C(l-1,A,c-1),C(l,A,c-1),C(l-1,A,c),C(l,A,c),n,r,(l+A+c)%2,i&&i[a]?i[a]:[])),a++;return o}function X(t,e,n,r,i){for(var o=[],a=0,s=0;s<e.length;s++)for(var A=e[s],l=1;l<p;l++)for(var c=1;c<f;c++)o.push(z(t,C(c-1,l-1,A),C(c-1,l,A),C(c,l-1,A),C(c,l,A),n,r,(c+l+A)%2,i&&i[a]?i[a]:[])),a++;return o}function Z(t,e,n){for(var r=1;r<I;r++)for(var i=1;i<p;i++)for(var o=1;o<f;o++)q(t,C(o-1,i-1,r-1),C(o-1,i-1,r),C(o-1,i,r-1),C(o-1,i,r),C(o,i-1,r-1),C(o,i-1,r),C(o,i,r-1),C(o,i,r),e,n,(o+i+r)%2)}function $(t,e,n){s=!0,Z(t,e,n),s=!1}function tt(t,e,n,r,i,o){for(var a=[],s=0,A=0;A<e.length;A++)for(var l=e[A],c=1;c<I;c++)for(var u=1;u<p;u++)a.push(J(t,l,u,c,n,r,i[A],o&&o[s]&&o[s])),s++;return a}function et(t,e,n,r,i,o){for(var a=[],s=0,A=0;A<e.length;A++)for(var l=e[A],c=1;c<f;c++)for(var u=1;u<I;u++)a.push(J(t,c,l,u,n,r,i[A],o&&o[s]&&o[s])),s++;return a}function nt(t,e,n,r,i,o){for(var a=[],s=0,A=0;A<e.length;A++)for(var l=e[A],c=1;c<p;c++)for(var u=1;u<f;u++)a.push(J(t,u,c,l,n,r,i[A],o&&o[s]&&o[s])),s++;return a}function rt(t,e){for(var n=[],r=t;r<e;r++)n.push(r);return n}return function(){if(M(),function(){for(var e=0;e<f;e++)for(var n=0;n<p;n++)for(var r=0;r<I;r++){var i=C(e,n,r);k(t.x[i],t.y[i],t.z[i],t.value[i])}}(),i&&a&&(L(a),A=!0,Z(null,w,x),A=!1),r&&o){L(o);for(var e=t.surface.pattern,n=t.surface.count,s=0;s<n;s++){var l=1===n?.5:s/(n-1),B=(1-l)*w+l*x,_=Math.abs(B-b)>Math.abs(B-Q)?[b,B]:[B,Q];$(e,_[0],_[1])}}var D=[[Math.min(w,Q),Math.max(w,Q)],[Math.min(b,x),Math.max(b,x)]];["x","y","z"].forEach(function(e){for(var n=[],r=0;r<D.length;r++){var i=0,o=D[r][0],a=D[r][1],s=t.slices[e];if(s.show&&s.fill){L(s.fill);var A=[],l=[],u=[];if(s.locations.length)for(var C=0;C<s.locations.length;C++){var m=c(s.locations[C],"x"===e?g:"y"===e?h:d);0===m.distRatio?A.push(m.id):m.id>0&&(l.push(m.id),"x"===e?u.push([m.distRatio,0,0]):"y"===e?u.push([0,m.distRatio,0]):u.push([0,0,m.distRatio]))}else A=rt(1,"x"===e?f-1:"y"===e?p-1:I-1);l.length>0&&(n[i]="x"===e?tt(null,l,o,a,u,n[i]):"y"===e?et(null,l,o,a,u,n[i]):nt(null,l,o,a,u,n[i]),i++),A.length>0&&(n[i]="x"===e?V(null,A,o,a,n[i]):"y"===e?W(null,A,o,a,n[i]):X(null,A,o,a,n[i]),i++)}var y=t.caps[e];y.show&&y.fill&&(L(y.fill),n[i]="x"===e?V(null,[0,f-1],o,a,n[i]):"y"===e?W(null,[0,p-1],o,a,n[i]):X(null,[0,I-1],o,a,n[i]),i++)}}),0===u&&M(),t._x=m,t._y=y,t._z=E,t._intensity=v,t._Xs=g,t._Ys=h,t._Zs=d}(),t}g.handlePick=function(t){if(t.object===this.mesh){var e=t.data.index,n=this.data._x[e],r=this.data._y[e],i=this.data._z[e],o=this.data._Ys.length,a=this.data._Zs.length,s=c(n,this.data._Xs).id,A=c(r,this.data._Ys).id,l=c(i,this.data._Zs).id,u=t.index=l+a*A+a*o*s;t.traceCoordinate=[this.data._x[u],this.data._y[u],this.data._z[u],this.data.value[u]];var g=this.data.hovertext||this.data.text;return Array.isArray(g)&&void 0!==g[u]?t.textLabel=g[u]:g&&(t.textLabel=g),!0}},g.update=function(t){var e=this.scene,n=e.fullSceneLayout;function r(t,e,n,r){return e.map(function(e){return t.d2l(e,0,r)*n})}this.data=h(t);var i={positions:A(r(n.xaxis,t._x,e.dataScale[0],t.xcalendar),r(n.yaxis,t._y,e.dataScale[1],t.ycalendar),r(n.zaxis,t._z,e.dataScale[2],t.zcalendar)),cells:A(t._i,t._j,t._k),lightPosition:[t.lightposition.x,t.lightposition.y,t.lightposition.z],ambient:t.lighting.ambient,diffuse:t.lighting.diffuse,specular:t.lighting.specular,roughness:t.lighting.roughness,fresnel:t.lighting.fresnel,vertexNormalsEpsilon:t.lighting.vertexnormalsepsilon,faceNormalsEpsilon:t.lighting.facenormalsepsilon,opacity:t.opacity,contourEnable:t.contour.show,contourColor:a(t.contour.color).slice(0,3),contourWidth:t.contour.width,useFacetNormals:t.flatshading},l=s(t);i.vertexIntensity=t._intensity,i.vertexIntensityBounds=[l.min,l.max],i.colormap=o(t),this.mesh.update(i)},g.dispose=function(){this.scene.glplot.remove(this.mesh),this.mesh.dispose()},e.exports={findNearestOnAxis:c,generateIsoMeshes:h,createIsosurfaceTrace:function(t,e){var n=t.glplot.gl,i=r({gl:n}),o=new u(t,i,e.uid);return i._trace=o,o.update(e),t.glplot.add(i),o}}},{"../../components/colorscale":605,"../../lib":719,"../../lib/gl_format_color":716,"../../lib/str2rgbarray":742,"../../plots/gl3d/zip3":818,"gl-mesh3d":280}],1051:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../registry"),o=t("./attributes"),a=t("../../components/colorscale/defaults");function s(t,e,n,r,o){var s=o("isomin"),A=o("isomax");null!=A&&null!=s&&s>A&&(e.isomin=null,e.isomax=null);var l=o("x"),c=o("y"),u=o("z"),g=o("value");l&&l.length&&c&&c.length&&u&&u.length&&g&&g.length?(i.getComponentMethod("calendars","handleTraceDefaults")(t,e,["x","y","z"],r),["x","y","z"].forEach(function(t){var e="caps."+t;o(e+".show")&&o(e+".fill");var n="slices."+t;o(n+".show")&&(o(n+".fill"),o(n+".locations"))}),o("spaceframe.show")&&o("spaceframe.fill"),o("surface.show")&&(o("surface.count"),o("surface.fill"),o("surface.pattern")),o("contour.show")&&(o("contour.color"),o("contour.width")),["text","hovertext","hovertemplate","lighting.ambient","lighting.diffuse","lighting.specular","lighting.roughness","lighting.fresnel","lighting.vertexnormalsepsilon","lighting.facenormalsepsilon","lightposition.x","lightposition.y","lightposition.z","flatshading","opacity"].forEach(function(t){o(t)}),a(t,e,r,o,{prefix:"",cLetter:"c"}),e._length=null):e.visible=!1}e.exports={supplyDefaults:function(t,e,n,i){s(t,e,0,i,function(n,i){return r.coerce(t,e,o,n,i)})},supplyIsoDefaults:s}},{"../../components/colorscale/defaults":603,"../../lib":719,"../../registry":847,"./attributes":1048}],1052:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults").supplyDefaults,calc:t("./calc"),colorbar:{min:"cmin",max:"cmax"},plot:t("./convert").createIsosurfaceTrace,moduleType:"trace",name:"isosurface",basePlotModule:t("../../plots/gl3d"),categories:["gl3d"],meta:{}}},{"../../plots/gl3d":807,"./attributes":1048,"./calc":1049,"./convert":1050,"./defaults":1051}],1053:[function(t,e,n){"use strict";var r=t("../../components/colorscale/attributes"),i=t("../../components/fx/hovertemplate_attributes"),o=t("../surface/attributes"),a=t("../../plots/attributes"),s=t("../../lib/extend").extendFlat;e.exports=s({x:{valType:"data_array",editType:"calc+clearAxisTypes"},y:{valType:"data_array",editType:"calc+clearAxisTypes"},z:{valType:"data_array",editType:"calc+clearAxisTypes"},i:{valType:"data_array",editType:"calc"},j:{valType:"data_array",editType:"calc"},k:{valType:"data_array",editType:"calc"},text:{valType:"string",dflt:"",arrayOk:!0,editType:"calc"},hovertext:{valType:"string",dflt:"",arrayOk:!0,editType:"calc"},hovertemplate:i({editType:"calc"}),delaunayaxis:{valType:"enumerated",values:["x","y","z"],dflt:"z",editType:"calc"},alphahull:{valType:"number",dflt:-1,editType:"calc"},intensity:{valType:"data_array",editType:"calc"},color:{valType:"color",editType:"calc"},vertexcolor:{valType:"data_array",editType:"calc"},facecolor:{valType:"data_array",editType:"calc"},transforms:void 0},r("",{colorAttr:"`intensity`",showScaleDflt:!0,editTypeOverride:"calc"}),{opacity:o.opacity,flatshading:{valType:"boolean",dflt:!1,editType:"calc"},contour:{show:s({},o.contours.x.show,{}),color:o.contours.x.color,width:o.contours.x.width,editType:"calc"},lightposition:{x:s({},o.lightposition.x,{dflt:1e5}),y:s({},o.lightposition.y,{dflt:1e5}),z:s({},o.lightposition.z,{dflt:0}),editType:"calc"},lighting:s({vertexnormalsepsilon:{valType:"number",min:0,max:1,dflt:1e-12,editType:"calc"},facenormalsepsilon:{valType:"number",min:0,max:1,dflt:1e-6,editType:"calc"},editType:"calc"},o.lighting),hoverinfo:s({},a.hoverinfo,{editType:"calc"})})},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../surface/attributes":1224}],1054:[function(t,e,n){"use strict";var r=t("../../components/colorscale/calc");e.exports=function(t,e){e.intensity&&r(t,e,{vals:e.intensity,containerStr:"",cLetter:"c"})}},{"../../components/colorscale/calc":601}],1055:[function(t,e,n){"use strict";var r=t("gl-mesh3d"),i=t("delaunay-triangulate"),o=t("alpha-shape"),a=t("convex-hull"),s=t("../../lib/gl_format_color").parseColorScale,A=t("../../lib/str2rgbarray"),l=t("../../components/colorscale").extractOpts,c=t("../../plots/gl3d/zip3");function u(t,e,n){this.scene=t,this.uid=n,this.mesh=e,this.name="",this.color="#fff",this.data=null,this.showContour=!1}var g=u.prototype;function h(t){for(var e=[],n=t.length,r=0;r<n;r++)e[r]=A(t[r]);return e}function d(t,e,n,r){for(var i=[],o=e.length,a=0;a<o;a++)i[a]=t.d2l(e[a],0,r)*n;return i}function f(t){for(var e=[],n=t.length,r=0;r<n;r++)e[r]=Math.round(t[r]);return e}function p(t,e){for(var n=t.length,r=0;r<n;r++)if(t[r]<=-.5||t[r]>=e-.5)return!1;return!0}g.handlePick=function(t){if(t.object===this.mesh){var e=t.index=t.data.index;t.traceCoordinate=[this.data.x[e],this.data.y[e],this.data.z[e]];var n=this.data.hovertext||this.data.text;return Array.isArray(n)&&void 0!==n[e]?t.textLabel=n[e]:n&&(t.textLabel=n),!0}},g.update=function(t){var e=this.scene,n=e.fullSceneLayout;this.data=t;var r,u=t.x.length,g=c(d(n.xaxis,t.x,e.dataScale[0],t.xcalendar),d(n.yaxis,t.y,e.dataScale[1],t.ycalendar),d(n.zaxis,t.z,e.dataScale[2],t.zcalendar));if(t.i&&t.j&&t.k){if(t.i.length!==t.j.length||t.j.length!==t.k.length||!p(t.i,u)||!p(t.j,u)||!p(t.k,u))return;r=c(f(t.i),f(t.j),f(t.k))}else r=0===t.alphahull?a(g):t.alphahull>0?o(t.alphahull,g):function(t,e){for(var n=["x","y","z"].indexOf(t),r=[],o=e.length,a=0;a<o;a++)r[a]=[e[a][(n+1)%3],e[a][(n+2)%3]];return i(r)}(t.delaunayaxis,g);var I={positions:g,cells:r,lightPosition:[t.lightposition.x,t.lightposition.y,t.lightposition.z],ambient:t.lighting.ambient,diffuse:t.lighting.diffuse,specular:t.lighting.specular,roughness:t.lighting.roughness,fresnel:t.lighting.fresnel,vertexNormalsEpsilon:t.lighting.vertexnormalsepsilon,faceNormalsEpsilon:t.lighting.facenormalsepsilon,opacity:t.opacity,contourEnable:t.contour.show,contourColor:A(t.contour.color).slice(0,3),contourWidth:t.contour.width,useFacetNormals:t.flatshading};if(t.intensity){var C=l(t);this.color="#fff",I.vertexIntensity=t.intensity,I.vertexIntensityBounds=[C.min,C.max],I.colormap=s(t)}else t.vertexcolor?(this.color=t.vertexcolor[0],I.vertexColors=h(t.vertexcolor)):t.facecolor?(this.color=t.facecolor[0],I.cellColors=h(t.facecolor)):(this.color=t.color,I.meshColor=A(t.color));this.mesh.update(I)},g.dispose=function(){this.scene.glplot.remove(this.mesh),this.mesh.dispose()},e.exports=function(t,e){var n=t.glplot.gl,i=r({gl:n}),o=new u(t,i,e.uid);return i._trace=o,o.update(e),t.glplot.add(i),o}},{"../../components/colorscale":605,"../../lib/gl_format_color":716,"../../lib/str2rgbarray":742,"../../plots/gl3d/zip3":818,"alpha-shape":64,"convex-hull":130,"delaunay-triangulate":165,"gl-mesh3d":280}],1056:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib"),o=t("../../components/colorscale/defaults"),a=t("./attributes");e.exports=function(t,e,n,s){function A(n,r){return i.coerce(t,e,a,n,r)}function l(t){var e=t.map(function(t){var e=A(t);return e&&i.isArrayOrTypedArray(e)?e:null});return e.every(function(t){return t&&t.length===e[0].length})&&e}l(["x","y","z"])?(l(["i","j","k"]),(!e.i||e.j&&e.k)&&(!e.j||e.k&&e.i)&&(!e.k||e.i&&e.j)?(r.getComponentMethod("calendars","handleTraceDefaults")(t,e,["x","y","z"],s),["lighting.ambient","lighting.diffuse","lighting.specular","lighting.roughness","lighting.fresnel","lighting.vertexnormalsepsilon","lighting.facenormalsepsilon","lightposition.x","lightposition.y","lightposition.z","contour.show","contour.color","contour.width","colorscale","reversescale","flatshading","alphahull","delaunayaxis","opacity"].forEach(function(t){A(t)}),"intensity"in t?(A("intensity"),o(t,e,s,A,{prefix:"",cLetter:"c"})):(e.showscale=!1,"facecolor"in t?A("facecolor"):"vertexcolor"in t?A("vertexcolor"):A("color",n)),A("text"),A("hovertext"),A("hovertemplate"),e._length=null):e.visible=!1):e.visible=!1}},{"../../components/colorscale/defaults":603,"../../lib":719,"../../registry":847,"./attributes":1053}],1057:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),calc:t("./calc"),colorbar:{min:"cmin",max:"cmax"},plot:t("./convert"),moduleType:"trace",name:"mesh3d",basePlotModule:t("../../plots/gl3d"),categories:["gl3d"],meta:{}}},{"../../plots/gl3d":807,"./attributes":1053,"./calc":1054,"./convert":1055,"./defaults":1056}],1058:[function(t,e,n){"use strict";var r=t("../../lib").extendFlat,i=t("../scatter/attributes"),o=t("../../components/drawing/attributes").dash,a=t("../../components/fx/attributes"),s=t("../../constants/delta.js"),A=s.INCREASING.COLOR,l=s.DECREASING.COLOR,c=i.line;function u(t){return{line:{color:r({},c.color,{dflt:t}),width:c.width,dash:o,editType:"style"},editType:"style"}}e.exports={x:{valType:"data_array",editType:"calc+clearAxisTypes"},open:{valType:"data_array",editType:"calc"},high:{valType:"data_array",editType:"calc"},low:{valType:"data_array",editType:"calc"},close:{valType:"data_array",editType:"calc"},line:{width:r({},c.width,{}),dash:r({},o,{}),editType:"style"},increasing:u(A),decreasing:u(l),text:{valType:"string",dflt:"",arrayOk:!0,editType:"calc"},hovertext:{valType:"string",dflt:"",arrayOk:!0,editType:"calc"},tickwidth:{valType:"number",min:0,max:.5,dflt:.3,editType:"calc"},hoverlabel:r({},a.hoverlabel,{split:{valType:"boolean",dflt:!1,editType:"style"}})}},{"../../components/drawing/attributes":613,"../../components/fx/attributes":623,"../../constants/delta.js":689,"../../lib":719,"../scatter/attributes":1112}],1059:[function(t,e,n){"use strict";var r=t("../../lib"),i=r._,o=t("../../plots/cartesian/axes"),a=t("../../constants/numerical").BADNUM;function s(t,e,n,r){return{o:t,h:e,l:n,c:r}}function A(t,e,n,s,A){for(var l=s.makeCalcdata(e,"open"),c=s.makeCalcdata(e,"high"),u=s.makeCalcdata(e,"low"),g=s.makeCalcdata(e,"close"),h=Array.isArray(e.text),d=Array.isArray(e.hovertext),f=!0,p=null,I=[],C=0;C<n.length;C++){var m=n[C],y=l[C],E=c[C],v=u[C],B=g[C];if(m!==a&&y!==a&&E!==a&&v!==a&&B!==a){B===y?null!==p&&B!==p&&(f=B>p):f=B>y,p=B;var b=A(y,E,v,B);b.pos=m,b.yc=(y+B)/2,b.i=C,b.dir=f?"increasing":"decreasing",b.x=b.pos,b.y=[v,E],h&&(b.tx=e.text[C]),d&&(b.htx=e.hovertext[C]),I.push(b)}else I.push({pos:m,empty:!0})}return e._extremes[s._id]=o.findExtremes(s,r.concat(u,c),{padded:!0}),I.length&&(I[0].t={labels:{open:i(t,"open:")+" ",high:i(t,"high:")+" ",low:i(t,"low:")+" ",close:i(t,"close:")+" "}}),I}e.exports={calc:function(t,e){var n=o.getFromId(t,e.xaxis),i=o.getFromId(t,e.yaxis),a=function(t,e,n){var i=n._minDiff;if(!i){var o,a=t._fullData,s=[];for(i=1/0,o=0;o<a.length;o++){var A=a[o];if("ohlc"===A.type&&!0===A.visible&&A.xaxis===e._id){s.push(A);var l=e.makeCalcdata(A,"x");A._xcalc=l;var c=r.distinctVals(l).minDiff;c&&isFinite(c)&&(i=Math.min(i,c))}}for(i===1/0&&(i=1),o=0;o<s.length;o++)s[o]._minDiff=i}return i*n.tickwidth}(t,n,e),l=e._minDiff;e._minDiff=null;var c=e._xcalc;e._xcalc=null;var u=A(t,e,c,i,s);return e._extremes[n._id]=o.findExtremes(n,c,{vpad:l/2}),u.length?(r.extendFlat(u[0].t,{wHover:l/2,tickLen:a}),u):[{t:{empty:!0}}]},calcCommon:A}},{"../../constants/numerical":695,"../../lib":719,"../../plots/cartesian/axes":767}],1060:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./ohlc_defaults"),o=t("./attributes");function a(t,e,n,r){n(r+".line.color"),n(r+".line.width",e.line.width),n(r+".line.dash",e.line.dash)}e.exports=function(t,e,n,s){function A(n,i){return r.coerce(t,e,o,n,i)}i(t,e,A,s)?(A("line.width"),A("line.dash"),a(0,e,A,"increasing"),a(0,e,A,"decreasing"),A("text"),A("hovertext"),A("tickwidth"),s._requestRangeslider[e.xaxis]=!0):e.visible=!1}},{"../../lib":719,"./attributes":1058,"./ohlc_defaults":1063}],1061:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axes"),i=t("../../lib"),o=t("../../components/fx"),a=t("../../components/color"),s=t("../../lib").fillText,A=t("../../constants/delta.js"),l={increasing:A.INCREASING.SYMBOL,decreasing:A.DECREASING.SYMBOL};function c(t,e,n,r){var i,s,A=t.cd,l=t.xa,c=A[0].trace,u=A[0].t,g=c.type,h="ohlc"===g?"l":"min",d="ohlc"===g?"h":"max",f=u.bPos||0,p=function(t){return t.pos+f-e},I=u.bdPos||u.tickLen,C=u.wHover,m=Math.min(1,I/Math.abs(l.r2c(l.range[1])-l.r2c(l.range[0])));function y(t){var e=p(t);return o.inbox(e-C,e+C,i)}function E(t){var e=t[h],r=t[d];return e===r||o.inbox(e-n,r-n,i)}function v(t){return(y(t)+E(t))/2}i=t.maxHoverDistance-m,s=t.maxSpikeDistance-m;var B=o.getDistanceFunction(r,y,E,v);if(o.getClosest(A,B,t),!1===t.index)return null;var b=A[t.index];if(b.empty)return null;var Q=c[b.dir],w=Q.line.color;return a.opacity(w)&&Q.line.width?t.color=w:t.color=Q.fillcolor,t.x0=l.c2p(b.pos+f-I,!0),t.x1=l.c2p(b.pos+f+I,!0),t.xLabelVal=b.pos,t.spikeDistance=v(b)*s/i,t.xSpike=l.c2p(b.pos,!0),t}function u(t,e,n,o){var a=t.cd,s=t.ya,A=a[0].trace,l=a[0].t,u=[],g=c(t,e,n,o);if(!g)return[];var h=a[g.index].hi||A.hoverinfo,d=h.split("+");if("all"!==h&&-1===d.indexOf("y"))return[];for(var f=["high","open","close","low"],p={},I=0;I<f.length;I++){var C,m=f[I],y=A[m][g.index],E=s.c2p(y,!0);y in p?(C=p[y]).yLabel+="<br>"+l.labels[m]+r.hoverLabelText(s,y):((C=i.extendFlat({},g)).y0=C.y1=E,C.yLabelVal=y,C.yLabel=l.labels[m]+r.hoverLabelText(s,y),C.name="",u.push(C),p[y]=C)}return u}function g(t,e,n,i){var o=t.cd,a=t.ya,A=o[0].trace,u=o[0].t,g=c(t,e,n,i);if(!g)return[];var h=o[g.index],d=g.index=h.i,f=h.dir;function p(t){return u.labels[t]+r.hoverLabelText(a,A[t][d])}var I=h.hi||A.hoverinfo,C=I.split("+"),m="all"===I,y=m||-1!==C.indexOf("y"),E=m||-1!==C.indexOf("text"),v=y?[p("open"),p("high"),p("low"),p("close")+"  "+l[f]]:[];return E&&s(h,A,v),g.extraText=v.join("<br>"),g.y0=g.y1=a.c2p(h.yc,!0),[g]}e.exports={hoverPoints:function(t,e,n,r){return t.cd[0].trace.hoverlabel.split?u(t,e,n,r):g(t,e,n,r)},hoverSplit:u,hoverOnPoints:g}},{"../../components/color":593,"../../components/fx":632,"../../constants/delta.js":689,"../../lib":719,"../../plots/cartesian/axes":767}],1062:[function(t,e,n){"use strict";e.exports={moduleType:"trace",name:"ohlc",basePlotModule:t("../../plots/cartesian"),categories:["cartesian","svg","showLegend"],meta:{},attributes:t("./attributes"),supplyDefaults:t("./defaults"),calc:t("./calc").calc,plot:t("./plot"),style:t("./style"),hoverPoints:t("./hover").hoverPoints,selectPoints:t("./select")}},{"../../plots/cartesian":778,"./attributes":1058,"./calc":1059,"./defaults":1060,"./hover":1061,"./plot":1064,"./select":1065,"./style":1066}],1063:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib");e.exports=function(t,e,n,o){var a=n("x"),s=n("open"),A=n("high"),l=n("low"),c=n("close");if(n("hoverlabel.split"),r.getComponentMethod("calendars","handleTraceDefaults")(t,e,["x"],o),s&&A&&l&&c){var u=Math.min(s.length,A.length,l.length,c.length);return a&&(u=Math.min(u,i.minRowLength(a))),e._length=u,u}}},{"../../lib":719,"../../registry":847}],1064:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../lib");e.exports=function(t,e,n,o){var a=e.xaxis,s=e.yaxis;i.makeTraceGroups(o,n,"trace ohlc").each(function(t){var e=r.select(this),n=t[0],o=n.t;if(!0!==n.trace.visible||o.empty)e.remove();else{var A=o.tickLen,l=e.selectAll("path").data(i.identity);l.enter().append("path"),l.exit().remove(),l.attr("d",function(t){if(t.empty)return"M0,0Z";var e=a.c2p(t.pos,!0),n=a.c2p(t.pos-A,!0),r=a.c2p(t.pos+A,!0);return"M"+n+","+s.c2p(t.o,!0)+"H"+e+"M"+e+","+s.c2p(t.h,!0)+"V"+s.c2p(t.l,!0)+"M"+r+","+s.c2p(t.c,!0)+"H"+e})}})}},{"../../lib":719,d3:163}],1065:[function(t,e,n){"use strict";e.exports=function(t,e){var n,r=t.cd,i=t.xaxis,o=t.yaxis,a=[],s=r[0].t.bPos||0;if(!1===e)for(n=0;n<r.length;n++)r[n].selected=0;else for(n=0;n<r.length;n++){var A=r[n];e.contains([i.c2p(A.pos+s),o.c2p(A.yc)],null,A.i,t)?(a.push({pointNumber:A.i,x:i.c2d(A.pos),y:o.c2d(A.yc)}),A.selected=1):A.selected=0}return a}},{}],1066:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/drawing"),o=t("../../components/color");e.exports=function(t,e,n){var a=n||r.select(t).selectAll("g.ohlclayer").selectAll("g.trace");a.style("opacity",function(t){return t[0].trace.opacity}),a.each(function(t){var e=t[0].trace;r.select(this).selectAll("path").each(function(t){if(!t.empty){var n=e[t.dir].line;r.select(this).style("fill","none").call(o.stroke,n.color).call(i.dashLine,n.dash,n.width).style("opacity",e.selectedpoints&&!t.selected?.3:1)}})})}},{"../../components/color":593,"../../components/drawing":614,d3:163}],1067:[function(t,e,n){"use strict";var r=t("../../lib/extend").extendFlat,i=t("../../plots/attributes"),o=t("../../plots/font_attributes"),a=t("../../components/colorscale/attributes"),s=t("../../components/fx/hovertemplate_attributes"),A=t("../../plots/domain").attributes,l=r({editType:"calc"},a("line",{editTypeOverride:"calc"}),{shape:{valType:"enumerated",values:["linear","hspline"],dflt:"linear",editType:"plot"},hovertemplate:s({editType:"plot",arrayOk:!1},{keys:["count","probability"]})});e.exports={domain:A({name:"parcats",trace:!0,editType:"calc"}),hoverinfo:r({},i.hoverinfo,{flags:["count","probability"],editType:"plot",arrayOk:!1}),hoveron:{valType:"enumerated",values:["category","color","dimension"],dflt:"category",editType:"plot"},hovertemplate:s({editType:"plot",arrayOk:!1},{keys:["count","probability","category","categorycount","colorcount","bandcolorcount"]}),arrangement:{valType:"enumerated",values:["perpendicular","freeform","fixed"],dflt:"perpendicular",editType:"plot"},bundlecolors:{valType:"boolean",dflt:!0,editType:"plot"},sortpaths:{valType:"enumerated",values:["forward","backward"],dflt:"forward",editType:"plot"},labelfont:o({editType:"calc"}),tickfont:o({editType:"calc"}),dimensions:{_isLinkedToArray:"dimension",label:{valType:"string",editType:"calc"},categoryorder:{valType:"enumerated",values:["trace","category ascending","category descending","array"],dflt:"trace",editType:"calc"},categoryarray:{valType:"data_array",editType:"calc"},ticktext:{valType:"data_array",editType:"calc"},values:{valType:"data_array",dflt:[],editType:"calc"},displayindex:{valType:"integer",editType:"calc"},editType:"calc",visible:{valType:"boolean",dflt:!0,editType:"calc"}},line:l,counts:{valType:"number",min:0,dflt:1,arrayOk:!0,editType:"calc"},customdata:void 0,hoverlabel:void 0,ids:void 0,legendgroup:void 0,opacity:void 0,selectedpoints:void 0,showlegend:void 0}},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../../plots/domain":792,"../../plots/font_attributes":793}],1068:[function(t,e,n){"use strict";var r=t("../../plots/get_data").getModuleCalcData,i=t("./plot");n.name="parcats",n.plot=function(t,e,n,o){var a=r(t.calcdata,"parcats");if(a.length){var s=a[0];i(t,s,n,o)}},n.clean=function(t,e,n,r){var i=r._has&&r._has("parcats"),o=e._has&&e._has("parcats");i&&!o&&r._paperdiv.selectAll(".parcats").remove()}},{"../../plots/get_data":802,"./plot":1073}],1069:[function(t,e,n){"use strict";var r=t("../../lib/gup").wrap,i=t("../../components/colorscale/helpers").hasColorscale,o=t("../../components/colorscale/calc"),a=t("../../lib/filter_unique.js"),s=t("../../components/drawing"),A=t("../../lib");function l(t,e,n){t.valueInds.push(e),t.count+=n}function c(t,e,n){t.valueInds.push(e),t.count+=n}e.exports=function(t,e){var n=A.filterVisible(e.dimensions);if(0===n.length)return[];var u,g,h,d=n.map(function(t){var e;return"trace"===t.categoryorder?e=null:"array"===t.categoryorder?e=t.categoryarray:(e=a(t.values).sort(),"category descending"===t.categoryorder&&(e=e.reverse())),function(t,e){e=null==e?[]:e.map(function(t){return t});var n={},r={},i=[];e.forEach(function(t,e){n[t]=0,r[t]=e});for(var o=0;o<t.length;o++){var a,s=t[o];void 0===n[s]?(n[s]=1,a=e.push(s)-1,r[s]=a):(n[s]++,a=r[s]),i.push(a)}var A=e.map(function(t){return n[t]});return{uniqueValues:e,uniqueCounts:A,inds:i}}(t.values,e)});u=A.isArrayOrTypedArray(e.counts)?e.counts:[e.counts],function(t){var e;if(function(t){for(var e=new Array(t.length),n=0;n<t.length;n++){if(t[n]<0||t[n]>=t.length)return!1;if(void 0!==e[t[n]])return!1;e[t[n]]=!0}return!0}(t.map(function(t){return t.displayindex})))for(e=0;e<t.length;e++)t[e]._displayindex=t[e].displayindex;else for(e=0;e<t.length;e++)t[e]._displayindex=e}(n),n.forEach(function(t,e){!function(t,e){t._categoryarray=e.uniqueValues,null===t.ticktext||void 0===t.ticktext?t._ticktext=[]:t._ticktext=t.ticktext.slice();for(var n=t._ticktext.length;n<e.uniqueValues.length;n++)t._ticktext.push(e.uniqueValues[n])}(t,d[e])});var f,p=e.line;p?(i(e,"line")&&o(t,e,{vals:e.line.color,containerStr:"line",cLetter:"c"}),f=s.tryColorscale(p)):f=A.identity;var I,C,m,y,E=n[0].values.length,v={},B=d.map(function(t){return t.inds});for(h=0,I=0;I<E;I++){var b=[];for(C=0;C<B.length;C++)b.push(B[C][I]);g=u[I%u.length],h+=g;var Q=(m=I,y=void 0,y=A.isArrayOrTypedArray(p.color)?p.color[m%p.color.length]:p.color,{color:f(y),rawColor:y}),w=b+"-"+Q.rawColor;void 0===v[w]&&(v[w]={categoryInds:b,color:Q.color,rawColor:Q.rawColor,valueInds:[],count:0}),c(v[w],I,g)}var x,_=n.map(function(t,e){return n=e,r=t._index,i=t._displayindex,o=t.label,{dimensionInd:n,containerInd:r,displayInd:i,dimensionLabel:o,count:h,categories:[],dragX:null};var n,r,i,o});for(I=0;I<E;I++)for(g=u[I%u.length],C=0;C<_.length;C++){var D=_[C].containerInd,M=d[C].inds[I],k=_[C].categories;if(void 0===k[M]){var S=e.dimensions[D]._categoryarray[M],L=e.dimensions[D]._ticktext[M];k[M]={dimensionInd:C,categoryInd:x=M,categoryValue:S,displayInd:x,categoryLabel:L,valueInds:[],count:0,dragY:null}}l(k[M],I,g)}return r(function(t,e,n){var r=t.map(function(t){return t.categories.length}).reduce(function(t,e){return Math.max(t,e)});return{dimensions:t,paths:e,trace:void 0,maxCats:r,count:n}}(_,v,h))}},{"../../components/colorscale/calc":601,"../../components/colorscale/helpers":604,"../../components/drawing":614,"../../lib":719,"../../lib/filter_unique.js":711,"../../lib/gup":717}],1070:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/colorscale/helpers").hasColorscale,o=t("../../components/colorscale/defaults"),a=t("../../plots/domain").defaults,s=t("../../plots/array_container_defaults"),A=t("./attributes"),l=t("../parcoords/merge_length");function c(t,e){function n(n,i){return r.coerce(t,e,A.dimensions,n,i)}var i=n("values"),o=n("visible");if(i&&i.length||(o=e.visible=!1),o){n("label"),n("displayindex",e._index);var a,s=t.categoryarray,l=Array.isArray(s)&&s.length>0;l&&(a="array");var c=n("categoryorder",a);"array"===c?(n("categoryarray"),n("ticktext")):(delete t.categoryarray,delete t.ticktext),l||"array"!==c||(e.categoryorder="trace")}}e.exports=function(t,e,n,u){function g(n,i){return r.coerce(t,e,A,n,i)}var h=s(t,e,{name:"dimensions",handleItemDefaults:c}),d=function(t,e,n,a,s){s("line.shape"),s("line.hovertemplate");var A=s("line.color",a.colorway[0]);if(i(t,"line")&&r.isArrayOrTypedArray(A)){if(A.length)return s("line.colorscale"),o(t,e,a,s,{prefix:"line.",cLetter:"c"}),A.length;e.line.color=n}return 1/0}(t,e,n,u,g);a(e,u,g),Array.isArray(h)&&h.length||(e.visible=!1),l(e,h,"values",d),g("hoveron"),g("hovertemplate"),g("arrangement"),g("bundlecolors"),g("sortpaths"),g("counts");var f={family:u.font.family,size:Math.round(u.font.size),color:u.font.color};r.coerceFont(g,"labelfont",f);var p={family:u.font.family,size:Math.round(u.font.size/1.2),color:u.font.color};r.coerceFont(g,"tickfont",p)}},{"../../components/colorscale/defaults":603,"../../components/colorscale/helpers":604,"../../lib":719,"../../plots/array_container_defaults":763,"../../plots/domain":792,"../parcoords/merge_length":1083,"./attributes":1067}],1071:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),calc:t("./calc"),plot:t("./plot"),colorbar:{container:"line",min:"cmin",max:"cmax"},moduleType:"trace",name:"parcats",basePlotModule:t("./base_plot"),categories:["noOpacity"],meta:{}}},{"./attributes":1067,"./base_plot":1068,"./calc":1069,"./defaults":1070,"./plot":1073}],1072:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../plot_api/plot_api"),o=t("../../components/fx"),a=t("../../lib"),s=t("../../components/drawing"),A=t("tinycolor2"),l=t("../../lib/svg_text_utils");function c(t,e,n,i){var o=t.map(function(t,e,n){var r,i=n[0],o=e.margin||{l:80,r:80,t:100,b:80},a=i.trace,s=a.domain,A=e.width,l=e.height,c=Math.floor(A*(s.x[1]-s.x[0])),u=Math.floor(l*(s.y[1]-s.y[0])),g=s.x[0]*A+o.l,h=e.height-s.y[1]*e.height+o.t,d=a.line.shape;r="all"===a.hoverinfo?["count","probability"]:(a.hoverinfo||"").split("+");var f={trace:a,key:a.uid,model:i,x:g,y:h,width:c,height:u,hoveron:a.hoveron,hoverinfoItems:r,arrangement:a.arrangement,bundlecolors:a.bundlecolors,sortpaths:a.sortpaths,labelfont:a.labelfont,categorylabelfont:a.tickfont,pathShape:d,dragDimension:null,margin:o,paths:[],dimensions:[],graphDiv:t,traceSelection:null,pathSelection:null,dimensionSelection:null};return i.dimensions&&(F(f),N(f)),f}.bind(0,e,n)),A=i.selectAll("g.parcatslayer").data([null]);A.enter().append("g").attr("class","parcatslayer").style("pointer-events","all");var c=A.selectAll("g.trace.parcats").data(o,u),p=c.enter().append("g").attr("class","trace parcats");c.attr("transform",function(t){return"translate("+t.x+", "+t.y+")"}),p.append("g").attr("class","paths");var m=c.select("g.paths").selectAll("path.path").data(function(t){return t.paths},u);m.attr("fill",function(t){return t.model.color});var v=m.enter().append("path").attr("class","path").attr("stroke-opacity",0).attr("fill",function(t){return t.model.color}).attr("fill-opacity",0);C(v),m.attr("d",function(t){return t.svgD}),v.empty()||m.sort(h),m.exit().remove(),m.on("mouseover",d).on("mouseout",f).on("click",I),p.append("g").attr("class","dimensions");var B=c.select("g.dimensions").selectAll("g.dimension").data(function(t){return t.dimensions},u);B.enter().append("g").attr("class","dimension"),B.attr("transform",function(t){return"translate("+t.x+", 0)"}),B.exit().remove();var b=B.selectAll("g.category").data(function(t){return t.categories},u),Q=b.enter().append("g").attr("class","category");b.attr("transform",function(t){return"translate(0, "+t.y+")"}),Q.append("rect").attr("class","catrect").attr("pointer-events","none"),b.select("rect.catrect").attr("fill","none").attr("width",function(t){return t.width}).attr("height",function(t){return t.height}),y(Q);var k=b.selectAll("rect.bandrect").data(function(t){return t.bands},u);k.each(function(){a.raiseToTop(this)}),k.attr("fill",function(t){return t.color});var S=k.enter().append("rect").attr("class","bandrect").attr("stroke-opacity",0).attr("fill",function(t){return t.color}).attr("fill-opacity",0);k.attr("fill",function(t){return t.color}).attr("width",function(t){return t.width}).attr("height",function(t){return t.height}).attr("y",function(t){return t.y}).attr("cursor",function(t){return"fixed"===t.parcatsViewModel.arrangement?"default":"perpendicular"===t.parcatsViewModel.arrangement?"ns-resize":"move"}),E(S),k.exit().remove(),Q.append("text").attr("class","catlabel").attr("pointer-events","none");var L=e._fullLayout.paper_bgcolor;b.select("text.catlabel").attr("text-anchor",function(t){return g(t)?"start":"end"}).attr("alignment-baseline","middle").style("text-shadow",L+" -1px  1px 2px, "+L+" 1px  1px 2px, "+L+"  1px -1px 2px, "+L+" -1px -1px 2px").style("fill","rgb(0, 0, 0)").attr("x",function(t){return g(t)?t.width+5:-5}).attr("y",function(t){return t.height/2}).text(function(t){return t.model.categoryLabel}).each(function(t){s.font(r.select(this),t.parcatsViewModel.categorylabelfont),l.convertToTspans(r.select(this),e)}),Q.append("text").attr("class","dimlabel"),b.select("text.dimlabel").attr("text-anchor","middle").attr("alignment-baseline","baseline").attr("cursor",function(t){return"fixed"===t.parcatsViewModel.arrangement?"default":"ew-resize"}).attr("x",function(t){return t.width/2}).attr("y",-5).text(function(t,e){return 0===e?t.parcatsViewModel.model.dimensions[t.model.dimensionInd].dimensionLabel:null}).each(function(t){s.font(r.select(this),t.parcatsViewModel.labelfont)}),b.selectAll("rect.bandrect").on("mouseover",w).on("mouseout",x),b.exit().remove(),B.call(r.behavior.drag().origin(function(t){return{x:t.x,y:0}}).on("dragstart",_).on("drag",D).on("dragend",M)),c.each(function(t){t.traceSelection=r.select(this),t.pathSelection=r.select(this).selectAll("g.paths").selectAll("path.path"),t.dimensionSelection=r.select(this).selectAll("g.dimensions").selectAll("g.dimension")}),c.exit().remove()}function u(t){return t.key}function g(t){var e=t.parcatsViewModel.dimensions.length,n=t.parcatsViewModel.dimensions[e-1].model.dimensionInd;return t.model.dimensionInd===n}function h(t,e){return t.model.rawColor>e.model.rawColor?1:t.model.rawColor<e.model.rawColor?-1:0}function d(t){if(!t.parcatsViewModel.dragDimension&&-1===t.parcatsViewModel.hoverinfoItems.indexOf("skip")){a.raiseToTop(this),m(r.select(this));var e=p(t);if(t.parcatsViewModel.graphDiv.emit("plotly_hover",{points:e,event:r.event}),-1===t.parcatsViewModel.hoverinfoItems.indexOf("none")){var n,i,s,l=r.mouse(this)[0],c=t.parcatsViewModel.graphDiv,u=t.parcatsViewModel.trace,g=c._fullLayout,h=g._paperdiv.node().getBoundingClientRect(),d=t.parcatsViewModel.graphDiv.getBoundingClientRect();for(s=0;s<t.leftXs.length-1;s++)if(t.leftXs[s]+t.dimWidths[s]-2<=l&&l<=t.leftXs[s+1]+2){var f=t.parcatsViewModel.dimensions[s],I=t.parcatsViewModel.dimensions[s+1];n=(f.x+f.width+I.x)/2,i=(t.topYs[s]+t.topYs[s+1]+t.height)/2;break}var C=t.parcatsViewModel.x+n,y=t.parcatsViewModel.y+i,E=A.mostReadable(t.model.color,["black","white"]),v=t.model.count,B=v/t.parcatsViewModel.model.count,b={countLabel:v,probabilityLabel:B.toFixed(3)},Q=[];-1!==t.parcatsViewModel.hoverinfoItems.indexOf("count")&&Q.push(["Count:",b.countLabel].join(" ")),-1!==t.parcatsViewModel.hoverinfoItems.indexOf("probability")&&Q.push(["P:",b.probabilityLabel].join(" "));var w=Q.join("<br>"),x=r.mouse(c)[0];o.loneHover({trace:u,x:C-h.left+d.left,y:y-h.top+d.top,text:w,color:t.model.color,borderColor:"black",fontFamily:'Monaco, "Courier New", monospace',fontSize:10,fontColor:E,idealAlign:x<C?"right":"left",hovertemplate:(u.line||{}).hovertemplate,hovertemplateLabels:b,eventData:[{data:u._input,fullData:u,count:v,probability:B}]},{container:g._hoverlayer.node(),outerContainer:g._paper.node(),gd:c})}}}function f(t){if(!t.parcatsViewModel.dragDimension&&(C(r.select(this)),o.loneUnhover(t.parcatsViewModel.graphDiv._fullLayout._hoverlayer.node()),t.parcatsViewModel.pathSelection.sort(h),-1===t.parcatsViewModel.hoverinfoItems.indexOf("skip"))){var e=p(t);t.parcatsViewModel.graphDiv.emit("plotly_unhover",{points:e,event:r.event})}}function p(t){for(var e=[],n=k(t.parcatsViewModel),r=0;r<t.model.valueInds.length;r++){var i=t.model.valueInds[r];e.push({curveNumber:n,pointNumber:i})}return e}function I(t){if(-1===t.parcatsViewModel.hoverinfoItems.indexOf("skip")){var e=p(t);t.parcatsViewModel.graphDiv.emit("plotly_click",{points:e,event:r.event})}}function C(t){t.attr("fill",function(t){return t.model.color}).attr("fill-opacity",.6).attr("stroke","lightgray").attr("stroke-width",.2).attr("stroke-opacity",1)}function m(t){t.attr("fill-opacity",.8).attr("stroke",function(t){return A.mostReadable(t.model.color,["black","white"])}).attr("stroke-width",.3)}function y(t){t.select("rect.catrect").attr("stroke","black").attr("stroke-width",1).attr("stroke-opacity",1)}function E(t){t.attr("stroke","black").attr("stroke-width",.2).attr("stroke-opacity",1).attr("fill-opacity",1)}function v(t){var e=t.parcatsViewModel.pathSelection,n=t.categoryViewModel.model.dimensionInd,r=t.categoryViewModel.model.categoryInd;return e.filter(function(e){return e.model.categoryInds[n]===r&&e.model.color===t.color})}function B(t,e,n){var i=r.select(t).datum().parcatsViewModel.graphDiv,o=r.select(t.parentNode).selectAll("rect.bandrect"),a=[];o.each(function(t){v(t).each(function(t){Array.prototype.push.apply(a,p(t))})}),i.emit(e,{points:a,event:n})}function b(t,e,n){var i=r.select(t).datum(),o=i.parcatsViewModel.graphDiv,a=v(i),s=[];a.each(function(t){Array.prototype.push.apply(s,p(t))}),o.emit(e,{points:s,event:n})}function Q(t,e){var n,i,o=r.select(e.parentNode).select("rect.catrect"),a=o.node().getBoundingClientRect(),s=o.datum(),A=s.parcatsViewModel,l=A.model.dimensions[s.model.dimensionInd],c=A.trace,u=a.top+a.height/2;A.dimensions.length>1&&l.displayInd===A.dimensions.length-1?(n=a.left,i="left"):(n=a.left+a.width,i="right");var g=s.model.count,h=s.model.categoryLabel,d=g/s.parcatsViewModel.model.count,f={countLabel:g,categoryLabel:h,probabilityLabel:d.toFixed(3)},p=[];-1!==s.parcatsViewModel.hoverinfoItems.indexOf("count")&&p.push(["Count:",f.countLabel].join(" ")),-1!==s.parcatsViewModel.hoverinfoItems.indexOf("probability")&&p.push(["P("+f.categoryLabel+"):",f.probabilityLabel].join(" "));var I=p.join("<br>");return{trace:c,x:n-t.left,y:u-t.top,text:I,color:"lightgray",borderColor:"black",fontFamily:'Monaco, "Courier New", monospace',fontSize:12,fontColor:"black",idealAlign:i,hovertemplate:c.hovertemplate,hovertemplateLabels:f,eventData:[{data:c._input,fullData:c,count:g,category:h,probability:d}]}}function w(t){if(!t.parcatsViewModel.dragDimension&&-1===t.parcatsViewModel.hoverinfoItems.indexOf("skip")){if(r.mouse(this)[1]<-1)return;var e,n=t.parcatsViewModel.graphDiv,i=n._fullLayout,s=i._paperdiv.node().getBoundingClientRect(),l=t.parcatsViewModel.hoveron;"color"===l?(function(t){var e=r.select(t).datum(),n=v(e);m(n),n.each(function(){a.raiseToTop(this)}),r.select(t.parentNode).selectAll("rect.bandrect").filter(function(t){return t.color===e.color}).each(function(){a.raiseToTop(this),r.select(this).attr("stroke","black").attr("stroke-width",1.5)})}(this),b(this,"plotly_hover",r.event)):(function(t){r.select(t.parentNode).selectAll("rect.bandrect").each(function(t){var e=v(t);m(e),e.each(function(){a.raiseToTop(this)})}),r.select(t.parentNode).select("rect.catrect").attr("stroke","black").attr("stroke-width",2.5)}(this),B(this,"plotly_hover",r.event)),-1===t.parcatsViewModel.hoverinfoItems.indexOf("none")&&("category"===l?e=Q(s,this):"color"===l?e=function(t,e){var n,i,o=e.getBoundingClientRect(),a=r.select(e).datum(),s=a.categoryViewModel,l=s.parcatsViewModel,c=l.model.dimensions[s.model.dimensionInd],u=l.trace,g=o.y+o.height/2;l.dimensions.length>1&&c.displayInd===l.dimensions.length-1?(n=o.left,i="left"):(n=o.left+o.width,i="right");var h=s.model.categoryLabel,d=a.parcatsViewModel.model.count,f=0;a.categoryViewModel.bands.forEach(function(t){t.color===a.color&&(f+=t.count)});var p=s.model.count,I=0;l.pathSelection.each(function(t){t.model.color===a.color&&(I+=t.model.count)});var C=f/d,m=f/I,y=f/p,E={countLabel:d,categoryLabel:h,probabilityLabel:C.toFixed(3)},v=[];-1!==s.parcatsViewModel.hoverinfoItems.indexOf("count")&&v.push(["Count:",E.countLabel].join(" ")),-1!==s.parcatsViewModel.hoverinfoItems.indexOf("probability")&&(v.push("P(color ∩ "+h+"): "+E.probabilityLabel),v.push("P("+h+" | color): "+m.toFixed(3)),v.push("P(color | "+h+"): "+y.toFixed(3)));var B=v.join("<br>"),b=A.mostReadable(a.color,["black","white"]);return{trace:u,x:n-t.left,y:g-t.top,text:B,color:a.color,borderColor:"black",fontFamily:'Monaco, "Courier New", monospace',fontColor:b,fontSize:10,idealAlign:i,hovertemplate:u.hovertemplate,hovertemplateLabels:E,eventData:[{data:u._input,fullData:u,category:h,count:d,probability:C,categorycount:p,colorcount:I,bandcolorcount:f}]}}(s,this):"dimension"===l&&(e=function(t,e){var n=[];return r.select(e.parentNode.parentNode).selectAll("g.category").select("rect.catrect").each(function(){n.push(Q(t,this))}),n}(s,this)),e&&o.loneHover(e,{container:i._hoverlayer.node(),outerContainer:i._paper.node(),gd:n}))}}function x(t){var e=t.parcatsViewModel;e.dragDimension||(C(e.pathSelection),y(e.dimensionSelection.selectAll("g.category")),E(e.dimensionSelection.selectAll("g.category").selectAll("rect.bandrect")),o.loneUnhover(e.graphDiv._fullLayout._hoverlayer.node()),e.pathSelection.sort(h),-1!==e.hoverinfoItems.indexOf("skip"))||("color"===t.parcatsViewModel.hoveron?b(this,"plotly_unhover",r.event):B(this,"plotly_unhover",r.event))}function _(t){"fixed"!==t.parcatsViewModel.arrangement&&(t.dragDimensionDisplayInd=t.model.displayInd,t.initialDragDimensionDisplayInds=t.parcatsViewModel.model.dimensions.map(function(t){return t.displayInd}),t.dragHasMoved=!1,t.dragCategoryDisplayInd=null,r.select(this).selectAll("g.category").select("rect.catrect").each(function(e){var n=r.mouse(this)[0],i=r.mouse(this)[1];-2<=n&&n<=e.width+2&&-2<=i&&i<=e.height+2&&(t.dragCategoryDisplayInd=e.model.displayInd,t.initialDragCategoryDisplayInds=t.model.categories.map(function(t){return t.displayInd}),e.model.dragY=e.y,a.raiseToTop(this.parentNode),r.select(this.parentNode).selectAll("rect.bandrect").each(function(e){e.y<i&&i<=e.y+e.height&&(t.potentialClickBand=this)}))}),t.parcatsViewModel.dragDimension=t,o.loneUnhover(t.parcatsViewModel.graphDiv._fullLayout._hoverlayer.node()))}function D(t){if("fixed"!==t.parcatsViewModel.arrangement&&(t.dragHasMoved=!0,null!==t.dragDimensionDisplayInd)){var e=t.dragDimensionDisplayInd,n=e-1,i=e+1,o=t.parcatsViewModel.dimensions[e];if(null!==t.dragCategoryDisplayInd){var a=o.categories[t.dragCategoryDisplayInd];a.model.dragY+=r.event.dy;var s=a.model.dragY,A=a.model.displayInd,l=o.categories,c=l[A-1],u=l[A+1];void 0!==c&&s<c.y+c.height/2&&(a.model.displayInd=c.model.displayInd,c.model.displayInd=A),void 0!==u&&s+a.height>u.y+u.height/2&&(a.model.displayInd=u.model.displayInd,u.model.displayInd=A),t.dragCategoryDisplayInd=a.model.displayInd}if(null===t.dragCategoryDisplayInd||"freeform"===t.parcatsViewModel.arrangement){o.model.dragX=r.event.x;var g=t.parcatsViewModel.dimensions[n],h=t.parcatsViewModel.dimensions[i];void 0!==g&&o.model.dragX<g.x+g.width&&(o.model.displayInd=g.model.displayInd,g.model.displayInd=e),void 0!==h&&o.model.dragX+o.width>h.x&&(o.model.displayInd=h.model.displayInd,h.model.displayInd=t.dragDimensionDisplayInd),t.dragDimensionDisplayInd=o.model.displayInd}F(t.parcatsViewModel),N(t.parcatsViewModel),L(t.parcatsViewModel),S(t.parcatsViewModel)}}function M(t){if("fixed"!==t.parcatsViewModel.arrangement&&null!==t.dragDimensionDisplayInd){r.select(this).selectAll("text").attr("font-weight","normal");var e={},n=k(t.parcatsViewModel),o=t.parcatsViewModel.model.dimensions.map(function(t){return t.displayInd}),a=t.initialDragDimensionDisplayInds.some(function(t,e){return t!==o[e]});a&&o.forEach(function(n,r){var i=t.parcatsViewModel.model.dimensions[r].containerInd;e["dimensions["+i+"].displayindex"]=n});var s=!1;if(null!==t.dragCategoryDisplayInd){var A=t.model.categories.map(function(t){return t.displayInd});if(s=t.initialDragCategoryDisplayInds.some(function(t,e){return t!==A[e]})){var l=t.model.categories.slice().sort(function(t,e){return t.displayInd-e.displayInd}),c=l.map(function(t){return t.categoryValue}),u=l.map(function(t){return t.categoryLabel});e["dimensions["+t.model.containerInd+"].categoryarray"]=[c],e["dimensions["+t.model.containerInd+"].ticktext"]=[u],e["dimensions["+t.model.containerInd+"].categoryorder"]="array"}}-1===t.parcatsViewModel.hoverinfoItems.indexOf("skip")&&!t.dragHasMoved&&t.potentialClickBand&&("color"===t.parcatsViewModel.hoveron?b(t.potentialClickBand,"plotly_click",r.event.sourceEvent):B(t.potentialClickBand,"plotly_click",r.event.sourceEvent)),t.model.dragX=null,null!==t.dragCategoryDisplayInd&&(t.parcatsViewModel.dimensions[t.dragDimensionDisplayInd].categories[t.dragCategoryDisplayInd].model.dragY=null,t.dragCategoryDisplayInd=null),t.dragDimensionDisplayInd=null,t.parcatsViewModel.dragDimension=null,t.dragHasMoved=null,t.potentialClickBand=null,F(t.parcatsViewModel),N(t.parcatsViewModel),r.transition().duration(300).ease("cubic-in-out").each(function(){L(t.parcatsViewModel,!0),S(t.parcatsViewModel,!0)}).each("end",function(){(a||s)&&i.restyle(t.parcatsViewModel.graphDiv,e,[n])})}}function k(t){for(var e,n=t.graphDiv._fullData,r=0;r<n.length;r++)if(t.key===n[r].uid){e=r;break}return e}function S(t,e){var n;void 0===e&&(e=!1),t.pathSelection.data(function(t){return t.paths},u),(n=t.pathSelection,e?n.transition():n).attr("d",function(t){return t.svgD})}function L(t,e){function n(t){return e?t.transition():t}void 0===e&&(e=!1),t.dimensionSelection.data(function(t){return t.dimensions},u);var i=t.dimensionSelection.selectAll("g.category").data(function(t){return t.categories},u);n(t.dimensionSelection).attr("transform",function(t){return"translate("+t.x+", 0)"}),n(i).attr("transform",function(t){return"translate(0, "+t.y+")"}),i.select(".dimlabel").text(function(t,e){return 0===e?t.parcatsViewModel.model.dimensions[t.model.dimensionInd].dimensionLabel:null}),i.select(".catlabel").attr("text-anchor",function(t){return g(t)?"start":"end"}).attr("x",function(t){return g(t)?t.width+5:-5}).each(function(t){var e,n;g(t)?(e=t.width+5,n="start"):(e=-5,n="end"),r.select(this).selectAll("tspan").attr("x",e).attr("text-anchor",n)});var o=i.selectAll("rect.bandrect").data(function(t){return t.bands},u),s=o.enter().append("rect").attr("class","bandrect").attr("cursor","move").attr("stroke-opacity",0).attr("fill",function(t){return t.color}).attr("fill-opacity",0);o.attr("fill",function(t){return t.color}).attr("width",function(t){return t.width}).attr("height",function(t){return t.height}).attr("y",function(t){return t.y}),E(s),o.each(function(){a.raiseToTop(this)}),o.exit().remove()}function R(t,e,n,i,o){var a,s,A=[],l=[];for(s=0;s<n.length-1;s++)a=r.interpolateNumber(n[s]+t[s],t[s+1]),A.push(a(o)),l.push(a(1-o));var c="M "+t[0]+","+e[0];for(c+="l"+n[0]+",0 ",s=1;s<n.length;s++)c+="C"+A[s-1]+","+e[s-1]+" "+l[s-1]+","+e[s]+" "+t[s]+","+e[s],c+="l"+n[s]+",0 ";for(c+="l0,"+i+" ",c+="l -"+n[n.length-1]+",0 ",s=n.length-2;s>=0;s--)c+="C"+l[s]+","+(e[s+1]+i)+" "+A[s]+","+(e[s]+i)+" "+(t[s]+n[s])+","+(e[s]+i),c+="l-"+n[s]+",0 ";return c+="Z"}function N(t){var e=t.dimensions,n=t.model,r=e.map(function(t){return t.categories.map(function(t){return t.y})}),i=t.model.dimensions.map(function(t){return t.categories.map(function(t){return t.displayInd})}),o=t.model.dimensions.map(function(t){return t.displayInd}),a=t.dimensions.map(function(t){return t.model.dimensionInd}),s=e.map(function(t){return t.x}),A=e.map(function(t){return t.width}),l=[];for(var c in n.paths)n.paths.hasOwnProperty(c)&&l.push(n.paths[c]);function u(t){var e=t.categoryInds.map(function(t,e){return i[e][t]});return a.map(function(t){return e[t]})}l.sort(function(e,n){var r=u(e),i=u(n);return"backward"===t.sortpaths&&(r.reverse(),i.reverse()),r.push(e.valueInds[0]),i.push(n.valueInds[0]),t.bundlecolors&&(r.unshift(e.rawColor),i.unshift(n.rawColor)),r<i?-1:r>i?1:0});for(var g=new Array(l.length),h=e[0].model.count,d=e[0].categories.map(function(t){return t.height}).reduce(function(t,e){return t+e}),f=0;f<l.length;f++){var p,I=l[f];p=h>0?d*(I.count/h):0;for(var C,m=new Array(r.length),y=0;y<I.categoryInds.length;y++){var E=I.categoryInds[y],v=i[y][E],B=o[y];m[B]=r[B][v],r[B][v]+=p;var b=t.dimensions[B].categories[v],Q=b.bands.length,w=b.bands[Q-1];if(void 0===w||I.rawColor!==w.rawColor){var x=void 0===w?0:w.y+w.height;b.bands.push({key:x,color:I.color,rawColor:I.rawColor,height:p,width:b.width,count:I.count,y:x,categoryViewModel:b,parcatsViewModel:t})}else{var _=b.bands[Q-1];_.height+=p,_.count+=I.count}}C="hspline"===t.pathShape?R(s,m,A,p,.5):R(s,m,A,p,0),g[f]={key:I.valueInds[0],model:I,height:p,leftXs:s,topYs:m,dimWidths:A,svgD:C,parcatsViewModel:t}}t.paths=g}function F(t){var e=t.model.dimensions.map(function(t){return{displayInd:t.displayInd,dimensionInd:t.dimensionInd}});e.sort(function(t,e){return t.displayInd-e.displayInd});var n=[];for(var r in e){var i=e[r].dimensionInd,o=t.model.dimensions[i];n.push(T(t,o))}t.dimensions=n}function T(t,e){var n,r=t.model.dimensions.length,i=e.displayInd;n=40+(r>1?(t.width-80-16)/(r-1):0)*i;var o,a,s,A,l,c=[],u=t.model.maxCats,g=e.categories.length,h=e.count,d=t.height-8*(u-1),f=8*(u-g)/2,p=e.categories.map(function(t){return{displayInd:t.displayInd,categoryInd:t.categoryInd}});for(p.sort(function(t,e){return t.displayInd-e.displayInd}),l=0;l<g;l++)A=p[l].categoryInd,a=e.categories[A],o=h>0?a.count/h*d:0,s={key:a.valueInds[0],model:a,width:16,height:o,y:null!==a.dragY?a.dragY:f,bands:[],parcatsViewModel:t},f=f+o+8,c.push(s);return{key:e.dimensionInd,x:null!==e.dragX?e.dragX:n,y:0,width:16,model:e,categories:c,parcatsViewModel:t,dragCategoryDisplayInd:null,dragDimensionDisplayInd:null,initialDragDimensionDisplayInds:null,initialDragCategoryDisplayInds:null,dragHasMoved:null,potentialClickBand:null}}e.exports=function(t,e,n,r){c(n,t,r,e)}},{"../../components/drawing":614,"../../components/fx":632,"../../lib":719,"../../lib/svg_text_utils":743,"../../plot_api/plot_api":754,d3:163,tinycolor2:537}],1073:[function(t,e,n){"use strict";var r=t("./parcats");e.exports=function(t,e,n,i){var o=t._fullLayout,a=o._paper,s=o._size;r(t,a,e,{width:s.w,height:s.h,margin:{t:s.t,r:s.r,b:s.b,l:s.l}},n,i)}},{"./parcats":1072}],1074:[function(t,e,n){"use strict";var r=t("../../components/colorscale/attributes"),i=t("../../plots/cartesian/layout_attributes"),o=t("../../plots/font_attributes"),a=t("../../plots/domain").attributes,s=t("../../lib/extend").extendFlat,A=t("../../plot_api/plot_template").templatedArray;e.exports={domain:a({name:"parcoords",trace:!0,editType:"plot"}),labelangle:{valType:"angle",dflt:0,editType:"plot"},labelside:{valType:"enumerated",values:["top","bottom"],dflt:"top",editType:"plot"},labelfont:o({editType:"plot"}),tickfont:o({editType:"plot"}),rangefont:o({editType:"plot"}),dimensions:A("dimension",{label:{valType:"string",editType:"plot"},tickvals:s({},i.tickvals,{editType:"plot"}),ticktext:s({},i.ticktext,{editType:"plot"}),tickformat:s({},i.tickformat,{editType:"plot"}),visible:{valType:"boolean",dflt:!0,editType:"plot"},range:{valType:"info_array",items:[{valType:"number",editType:"plot"},{valType:"number",editType:"plot"}],editType:"plot"},constraintrange:{valType:"info_array",freeLength:!0,dimensions:"1-2",items:[{valType:"number",editType:"plot"},{valType:"number",editType:"plot"}],editType:"plot"},multiselect:{valType:"boolean",dflt:!0,editType:"plot"},values:{valType:"data_array",editType:"calc"},editType:"calc"}),line:s({editType:"calc"},r("line",{colorscaleDflt:"Viridis",autoColorDflt:!1,editTypeOverride:"calc"}))}},{"../../components/colorscale/attributes":600,"../../lib/extend":710,"../../plot_api/plot_template":757,"../../plots/cartesian/layout_attributes":779,"../../plots/domain":792,"../../plots/font_attributes":793}],1075:[function(t,e,n){"use strict";var r=t("./constants"),i=t("d3"),o=t("../../lib/gup").keyFun,a=t("../../lib/gup").repeat,s=t("../../lib").sorterAsc,A=r.bar.snapRatio;function l(t,e){return t*(1-A)+e*A}var c=r.bar.snapClose;function u(t,e){return t*(1-c)+e*c}function g(t,e,n,r){if(function(t,e){for(var n=0;n<e.length;n++)if(t>=e[n][0]&&t<=e[n][1])return!0;return!1}(n,r))return n;var i=t?-1:1,o=0,a=e.length-1;if(i<0){var s=o;o=a,a=s}for(var A=e[o],c=A,g=o;i*g<i*a;g+=i){var h=g+i,d=e[h];if(i*n<i*u(A,d))return l(A,c);if(i*n<i*d||h===a)return l(d,A);c=A,A=d}}function h(t){t.attr("x",-r.bar.captureWidth/2).attr("width",r.bar.captureWidth)}function d(t){t.attr("visibility","visible").style("visibility","visible").attr("fill","yellow").attr("opacity",0)}function f(t){if(!t.brush.filterSpecified)return"0,"+t.height;for(var e,n,r,i=p(t.brush.filter.getConsolidated(),t.height),o=[0],a=i.length?i[0][0]:null,s=0;s<i.length;s++)n=(e=i[s])[1]-e[0],o.push(a),o.push(n),(r=s+1)<i.length&&(a=i[r][0]-e[1]);return o.push(t.height),o}function p(t,e){return t.map(function(t){return t.map(function(t){return Math.max(0,t*e)}).sort(s)})}function I(){i.select(document.body).style("cursor",null)}function C(t){t.attr("stroke-dasharray",f)}function m(t,e){var n=i.select(t).selectAll(".highlight, .highlight-shadow");C(e?n.transition().duration(r.bar.snapDuration).each("end",e):n)}function y(t,e){var n,i=t.brush,o=NaN,a={};if(i.filterSpecified){var s=t.height,A=i.filter.getConsolidated(),l=p(A,s),c=NaN,u=NaN,g=NaN;for(n=0;n<=l.length;n++){var h=l[n];if(h&&h[0]<=e&&e<=h[1]){c=n;break}if(u=n?n-1:NaN,h&&h[0]>e){g=n;break}}if(o=c,isNaN(o)&&(o=isNaN(u)||isNaN(g)?isNaN(u)?g:u:e-l[u][1]<l[g][0]-e?u:g),!isNaN(o)){var d=l[o],f=function(t,e){var n=r.bar.handleHeight;if(!(e>t[1]+n||e<t[0]-n))return e>=.9*t[1]+.1*t[0]?"n":e<=.9*t[0]+.1*t[1]?"s":"ns"}(d,e);f&&(a.interval=A[o],a.intervalPix=d,a.region=f)}}if(t.ordinal&&!a.region){var I=t.unitTickvals,C=t.unitToPaddedPx.invert(e);for(n=0;n<I.length;n++){var m=[.25*I[Math.max(n-1,0)]+.75*I[n],.25*I[Math.min(n+1,I.length-1)]+.75*I[n]];if(C>=m[0]&&C<=m[1]){a.clickableOrdinalRange=m;break}}}return a}function E(t,e){i.event.sourceEvent.stopPropagation();var n=e.height-i.mouse(t)[1]-2*r.verticalPadding,o=e.brush.svgBrush;o.wasDragged=!0,o._dragging=!0,o.grabbingBar?o.newExtent=[n-o.grabPoint,n+o.barLength-o.grabPoint].map(e.unitToPaddedPx.invert):o.newExtent=[o.startExtent,e.unitToPaddedPx.invert(n)].sort(s),e.brush.filterSpecified=!0,o.extent=o.stayingIntervals.concat([o.newExtent]),o.brushCallback(e),m(t.parentNode)}function v(t,e){var n=y(e,e.height-i.mouse(t)[1]-2*r.verticalPadding),o="crosshair";n.clickableOrdinalRange?o="pointer":n.region&&(o=n.region+"-resize"),i.select(document.body).style("cursor",o)}function B(t){t.on("mousemove",function(t){i.event.preventDefault(),t.parent.inBrushDrag||v(this,t)}).on("mouseleave",function(t){t.parent.inBrushDrag||I()}).call(i.behavior.drag().on("dragstart",function(t){!function(t,e){i.event.sourceEvent.stopPropagation();var n=e.height-i.mouse(t)[1]-2*r.verticalPadding,o=e.unitToPaddedPx.invert(n),a=e.brush,s=y(e,n),A=s.interval,l=a.svgBrush;if(l.wasDragged=!1,l.grabbingBar="ns"===s.region,l.grabbingBar){var c=A.map(e.unitToPaddedPx);l.grabPoint=n-c[0]-r.verticalPadding,l.barLength=c[1]-c[0]}l.clickableOrdinalRange=s.clickableOrdinalRange,l.stayingIntervals=e.multiselect&&a.filterSpecified?a.filter.getConsolidated():[],A&&(l.stayingIntervals=l.stayingIntervals.filter(function(t){return t[0]!==A[0]&&t[1]!==A[1]})),l.startExtent=s.region?A["s"===s.region?1:0]:o,e.parent.inBrushDrag=!0,l.brushStartCallback()}(this,t)}).on("drag",function(t){E(this,t)}).on("dragend",function(t){!function(t,e){var n=e.brush,r=n.filter,o=n.svgBrush;o._dragging||(v(t,e),E(t,e),e.brush.svgBrush.wasDragged=!1),o._dragging=!1,i.event.sourceEvent.stopPropagation();var a=o.grabbingBar;if(o.grabbingBar=!1,o.grabLocation=void 0,e.parent.inBrushDrag=!1,I(),!o.wasDragged)return o.wasDragged=void 0,o.clickableOrdinalRange?n.filterSpecified&&e.multiselect?o.extent.push(o.clickableOrdinalRange):(o.extent=[o.clickableOrdinalRange],n.filterSpecified=!0):a?(o.extent=o.stayingIntervals,0===o.extent.length&&Q(n)):Q(n),o.brushCallback(e),m(t.parentNode),void o.brushEndCallback(n.filterSpecified?r.getConsolidated():[]);var s=function(){r.set(r.getConsolidated())};if(e.ordinal){var A=e.unitTickvals;A[A.length-1]<A[0]&&A.reverse(),o.newExtent=[g(0,A,o.newExtent[0],o.stayingIntervals),g(1,A,o.newExtent[1],o.stayingIntervals)];var l=o.newExtent[1]>o.newExtent[0];o.extent=o.stayingIntervals.concat(l?[o.newExtent]:[]),o.extent.length||Q(n),o.brushCallback(e),l?m(t.parentNode,s):(s(),m(t.parentNode))}else s();o.brushEndCallback(n.filterSpecified?r.getConsolidated():[])}(this,t)}))}function b(t,e){return t[0]-e[0]}function Q(t){t.filterSpecified=!1,t.svgBrush.extent=[[-1/0,1/0]]}function w(t){for(var e,n=t.slice(),r=[],i=n.shift();i;){for(e=i.slice();(i=n.shift())&&i[0]<=e[1];)e[1]=Math.max(e[1],i[1]);r.push(e)}return r}e.exports={makeBrush:function(t,e,n,r,i,o){var a,A=function(){var t,e,n=[];return{set:function(r){1===(n=r.map(function(t){return t.slice().sort(s)}).sort(b)).length&&n[0][0]===-1/0&&n[0][1]===1/0&&(n=[[0,-1]]),t=w(n),e=n.reduce(function(t,e){return[Math.min(t[0],e[0]),Math.max(t[1],e[1])]},[1/0,-1/0])},get:function(){return n.slice()},getConsolidated:function(){return t},getBounds:function(){return e}}}();return A.set(n),{filter:A,filterSpecified:e,svgBrush:{extent:[],brushStartCallback:r,brushCallback:(a=i,function(t){var e=t.brush,n=function(t){return t.svgBrush.extent.map(function(t){return t.slice()})}(e).slice();e.filter.set(n),a()}),brushEndCallback:o}}},ensureAxisBrush:function(t){var e=t.selectAll("."+r.cn.axisBrush).data(a,o);e.enter().append("g").classed(r.cn.axisBrush,!0),function(t){var e=t.selectAll(".background").data(a);e.enter().append("rect").classed("background",!0).call(h).call(d).style("pointer-events","auto").attr("transform","translate(0 "+r.verticalPadding+")"),e.call(B).attr("height",function(t){return t.height-r.verticalPadding});var n=t.selectAll(".highlight-shadow").data(a);n.enter().append("line").classed("highlight-shadow",!0).attr("x",-r.bar.width/2).attr("stroke-width",r.bar.width+r.bar.strokeWidth).attr("stroke",r.bar.strokeColor).attr("opacity",r.bar.strokeOpacity).attr("stroke-linecap","butt"),n.attr("y1",function(t){return t.height}).call(C);var i=t.selectAll(".highlight").data(a);i.enter().append("line").classed("highlight",!0).attr("x",-r.bar.width/2).attr("stroke-width",r.bar.width-r.bar.strokeWidth).attr("stroke",r.bar.fillColor).attr("opacity",r.bar.fillOpacity).attr("stroke-linecap","butt"),i.attr("y1",function(t){return t.height}).call(C)}(e)},cleanRanges:function(t,e){if(Array.isArray(t[0])?(t=t.map(function(t){return t.sort(s)}),t=e.multiselect?w(t.sort(b)):[t[0]]):t=[t.sort(s)],e.tickvals){var n=e.tickvals.slice().sort(s);if(!(t=t.map(function(t){var e=[g(0,n,t[0],[]),g(1,n,t[1],[])];if(e[1]>e[0])return e}).filter(function(t){return t})).length)return}return t.length>1?t:t[0]}}},{"../../lib":719,"../../lib/gup":717,"./constants":1078,d3:163}],1076:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../plots/get_data").getModuleCalcData,o=t("./plot"),a=t("../../constants/xmlns_namespaces");n.name="parcoords",n.plot=function(t){var e=i(t.calcdata,"parcoords")[0];e.length&&o(t,e)},n.clean=function(t,e,n,r){var i=r._has&&r._has("parcoords"),o=e._has&&e._has("parcoords");i&&!o&&(r._paperdiv.selectAll(".parcoords").remove(),r._glimages.selectAll("*").remove())},n.toSVG=function(t){var e=t._fullLayout._glimages,n=r.select(t).selectAll(".svg-container");n.filter(function(t,e){return e===n.size()-1}).selectAll(".gl-canvas-context, .gl-canvas-focus").each(function(){var t=this.toDataURL("image/png");e.append("svg:image").attr({xmlns:a.svg,"xlink:href":t,preserveAspectRatio:"none",x:0,y:0,width:this.width,height:this.height})}),window.setTimeout(function(){r.selectAll("#filterBarPattern").attr("id","filterBarPattern")},60)}},{"../../constants/xmlns_namespaces":696,"../../plots/get_data":802,"./plot":1085,d3:163}],1077:[function(t,e,n){"use strict";var r=t("../../lib").isArrayOrTypedArray,i=t("../../components/colorscale"),o=t("../../lib/gup").wrap;e.exports=function(t,e){var n,a;return i.hasColorscale(e,"line")&&r(e.line.color)?(n=e.line.color,a=i.extractOpts(e.line).colorscale,i.calc(t,e,{vals:n,containerStr:"line",cLetter:"c"})):(n=function(t){for(var e=new Array(t),n=0;n<t;n++)e[n]=.5;return e}(e._length),a=[[0,e.line.color],[1,e.line.color]]),o({lineColor:n,cscale:a})}},{"../../components/colorscale":605,"../../lib":719,"../../lib/gup":717}],1078:[function(t,e,n){"use strict";e.exports={maxDimensionCount:60,overdrag:45,verticalPadding:2,tickDistance:50,canvasPixelRatio:1,blockLineCount:5e3,layers:["contextLineLayer","focusLineLayer","pickLineLayer"],axisTitleOffset:28,axisExtentOffset:10,deselectedLineColor:"#777",bar:{width:4,captureWidth:10,fillColor:"magenta",fillOpacity:1,snapDuration:150,snapRatio:.25,snapClose:.01,strokeColor:"white",strokeOpacity:1,strokeWidth:1,handleHeight:8,handleOpacity:1,handleOverlap:0},cn:{axisExtentText:"axis-extent-text",parcoordsLineLayers:"parcoords-line-layers",parcoordsLineLayer:"parcoords-lines",parcoords:"parcoords",parcoordsControlView:"parcoords-control-view",yAxis:"y-axis",axisOverlays:"axis-overlays",axis:"axis",axisHeading:"axis-heading",axisTitle:"axis-title",axisExtent:"axis-extent",axisExtentTop:"axis-extent-top",axisExtentTopText:"axis-extent-top-text",axisExtentBottom:"axis-extent-bottom",axisExtentBottomText:"axis-extent-bottom-text",axisBrush:"axis-brush"},id:{filterBarPattern:"filter-bar-pattern"}}},{}],1079:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/colorscale/helpers").hasColorscale,o=t("../../components/colorscale/defaults"),a=t("../../plots/domain").defaults,s=t("../../plots/array_container_defaults"),A=t("../../plots/cartesian/axes"),l=t("./attributes"),c=t("./axisbrush"),u=t("./constants").maxDimensionCount,g=t("./merge_length");function h(t,e,n,i){function o(n,i){return r.coerce(t,e,l.dimensions,n,i)}var a=o("values"),s=o("visible");if(a&&a.length||(s=e.visible=!1),s){o("label"),o("tickvals"),o("ticktext"),o("tickformat");var u=o("range");e._ax={_id:"y",type:"linear",showexponent:"all",exponentformat:"B",range:u},A.setConvert(e._ax,i.layout),o("multiselect");var g=o("constraintrange");g&&(e.constraintrange=c.cleanRanges(g,e))}}e.exports=function(t,e,n,A){function c(n,i){return r.coerce(t,e,l,n,i)}var d=t.dimensions;Array.isArray(d)&&d.length>u&&(r.log("parcoords traces support up to "+u+" dimensions at the moment"),d.splice(u));var f=s(t,e,{name:"dimensions",layout:A,handleItemDefaults:h}),p=function(t,e,n,a,s){var A=s("line.color",n);if(i(t,"line")&&r.isArrayOrTypedArray(A)){if(A.length)return s("line.colorscale"),o(t,e,a,s,{prefix:"line.",cLetter:"c"}),A.length;e.line.color=n}return 1/0}(t,e,n,A,c);a(e,A,c),Array.isArray(f)&&f.length||(e.visible=!1),g(e,f,"values",p);var I={family:A.font.family,size:Math.round(A.font.size/1.2),color:A.font.color};r.coerceFont(c,"labelfont",I),r.coerceFont(c,"tickfont",I),r.coerceFont(c,"rangefont",I),c("labelangle"),c("labelside")}},{"../../components/colorscale/defaults":603,"../../components/colorscale/helpers":604,"../../lib":719,"../../plots/array_container_defaults":763,"../../plots/cartesian/axes":767,"../../plots/domain":792,"./attributes":1074,"./axisbrush":1075,"./constants":1078,"./merge_length":1083}],1080:[function(t,e,n){"use strict";var r=t("../../lib").isTypedArray;n.convertTypedArray=function(t){return r(t)?Array.prototype.slice.call(t):t},n.isOrdinal=function(t){return!!t.tickvals},n.isVisible=function(t){return t.visible||!("visible"in t)}},{"../../lib":719}],1081:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),calc:t("./calc"),plot:t("./plot"),colorbar:{container:"line",min:"cmin",max:"cmax"},moduleType:"trace",name:"parcoords",basePlotModule:t("./base_plot"),categories:["gl","regl","noOpacity","noHover"],meta:{}}},{"./attributes":1074,"./base_plot":1076,"./calc":1077,"./defaults":1079,"./plot":1085}],1082:[function(t,e,n){"use strict";var r=t("glslify"),i=r(["precision highp float;\n#define GLSLIFY 1\n\nvarying vec4 fragColor;\n\nattribute vec4 p01_04, p05_08, p09_12, p13_16,\n               p17_20, p21_24, p25_28, p29_32,\n               p33_36, p37_40, p41_44, p45_48,\n               p49_52, p53_56, p57_60, colors;\n\nuniform mat4 dim0A, dim1A, dim0B, dim1B, dim0C, dim1C, dim0D, dim1D,\n             loA, hiA, loB, hiB, loC, hiC, loD, hiD;\n\nuniform vec2 resolution, viewBoxPos, viewBoxSize;\nuniform sampler2D mask, palette;\nuniform float maskHeight;\nuniform float drwLayer; // 0: context, 1: focus, 2: pick\nuniform vec4 contextColor;\n\nbool isPick    = (drwLayer > 1.5);\nbool isContext = (drwLayer < 0.5);\n\nconst vec4 ZEROS = vec4(0.0, 0.0, 0.0, 0.0);\nconst vec4 UNITS = vec4(1.0, 1.0, 1.0, 1.0);\n\nfloat val(mat4 p, mat4 v) {\n    return dot(matrixCompMult(p, v) * UNITS, UNITS);\n}\n\nfloat axisY(float ratio, mat4 A, mat4 B, mat4 C, mat4 D) {\n    float y1 = val(A, dim0A) + val(B, dim0B) + val(C, dim0C) + val(D, dim0D);\n    float y2 = val(A, dim1A) + val(B, dim1B) + val(C, dim1C) + val(D, dim1D);\n    return y1 * (1.0 - ratio) + y2 * ratio;\n}\n\nint iMod(int a, int b) {\n    return a - b * (a / b);\n}\n\nbool fOutside(float p, float lo, float hi) {\n    return (lo < hi) && (lo > p || p > hi);\n}\n\nbool vOutside(vec4 p, vec4 lo, vec4 hi) {\n    return (\n        fOutside(p[0], lo[0], hi[0]) ||\n        fOutside(p[1], lo[1], hi[1]) ||\n        fOutside(p[2], lo[2], hi[2]) ||\n        fOutside(p[3], lo[3], hi[3])\n    );\n}\n\nbool mOutside(mat4 p, mat4 lo, mat4 hi) {\n    return (\n        vOutside(p[0], lo[0], hi[0]) ||\n        vOutside(p[1], lo[1], hi[1]) ||\n        vOutside(p[2], lo[2], hi[2]) ||\n        vOutside(p[3], lo[3], hi[3])\n    );\n}\n\nbool outsideBoundingBox(mat4 A, mat4 B, mat4 C, mat4 D) {\n    return mOutside(A, loA, hiA) ||\n           mOutside(B, loB, hiB) ||\n           mOutside(C, loC, hiC) ||\n           mOutside(D, loD, hiD);\n}\n\nbool outsideRasterMask(mat4 A, mat4 B, mat4 C, mat4 D) {\n    mat4 pnts[4];\n    pnts[0] = A;\n    pnts[1] = B;\n    pnts[2] = C;\n    pnts[3] = D;\n\n    for(int i = 0; i < 4; ++i) {\n        for(int j = 0; j < 4; ++j) {\n            for(int k = 0; k < 4; ++k) {\n                if(0 == iMod(\n                    int(255.0 * texture2D(mask,\n                        vec2(\n                            (float(i * 2 + j / 2) + 0.5) / 8.0,\n                            (pnts[i][j][k] * (maskHeight - 1.0) + 1.0) / maskHeight\n                        ))[3]\n                    ) / int(pow(2.0, float(iMod(j * 4 + k, 8)))),\n                    2\n                )) return true;\n            }\n        }\n    }\n    return false;\n}\n\nvec4 position(bool isContext, float v, mat4 A, mat4 B, mat4 C, mat4 D) {\n    float x = 0.5 * sign(v) + 0.5;\n    float y = axisY(x, A, B, C, D);\n    float z = 1.0 - abs(v);\n\n    z += isContext ? 0.0 : 2.0 * float(\n        outsideBoundingBox(A, B, C, D) ||\n        outsideRasterMask(A, B, C, D)\n    );\n\n    return vec4(\n        2.0 * (vec2(x, y) * viewBoxSize + viewBoxPos) / resolution - 1.0,\n        z,\n        1.0\n    );\n}\n\nvoid main() {\n    mat4 A = mat4(p01_04, p05_08, p09_12, p13_16);\n    mat4 B = mat4(p17_20, p21_24, p25_28, p29_32);\n    mat4 C = mat4(p33_36, p37_40, p41_44, p45_48);\n    mat4 D = mat4(p49_52, p53_56, p57_60, ZEROS);\n\n    float v = colors[3];\n\n    gl_Position = position(isContext, v, A, B, C, D);\n\n    fragColor =\n        isContext ? vec4(contextColor) :\n        isPick ? vec4(colors.rgb, 1.0) : texture2D(palette, vec2(abs(v), 0.5));\n}\n"]),o=r(["precision highp float;\n#define GLSLIFY 1\n\nvarying vec4 fragColor;\n\nvoid main() {\n    gl_FragColor = fragColor;\n}\n"]),a=t("./constants").maxDimensionCount,s=t("../../lib"),A=1e-6,l=2048,c=new Uint8Array(4),u=new Uint8Array(4),g={shape:[256,1],format:"rgba",type:"uint8",mag:"nearest",min:"nearest"};function h(t,e,n,r,i){var o=t._gl;o.enable(o.SCISSOR_TEST),o.scissor(e,n,r,i),t.clear({color:[0,0,0,0],depth:1})}function d(t,e,n,r,i,o){var a=o.key;n.drawCompleted||(function(t){t.read({x:0,y:0,width:1,height:1,data:c})}(t),n.drawCompleted=!0),function s(A){var l=Math.min(r,i-A*r);0===A&&(window.cancelAnimationFrame(n.currentRafs[a]),delete n.currentRafs[a],h(t,o.scissorX,o.scissorY,o.scissorWidth,o.viewBoxSize[1])),n.clearOnly||(o.count=2*l,o.offset=2*A*r,e(o),A*r+l<i&&(n.currentRafs[a]=window.requestAnimationFrame(function(){s(A+1)})),n.drawCompleted=!1)}(0)}function f(t,e){return(t>>>8*e)%256/255}function p(t,e,n){for(var r=new Array(8*e),i=0,o=0;o<e;o++)for(var a=0;a<2;a++)for(var s=0;s<4;s++){var A=4*t+s,l=n[64*o+A];63===A&&0===a&&(l*=-1),r[i++]=l}return r}function I(t){var e="0"+t;return e.substr(e.length-2)}function C(t){return t<a?"p"+I(t+1)+"_"+I(t+4):"colors"}function m(t,e,n,r,i,o,a,A,l,c,u,g,h){for(var d=[[],[]],f=0;f<64;f++)d[0][f]=f===i?1:0,d[1][f]=f===o?1:0;var p=t.lines.canvasOverdrag,I=t.domain,C=t.canvasWidth,m=t.canvasHeight,y=t.deselectedLines.color;return s.extendFlat({key:u,resolution:[C,m],viewBoxPos:[a+p,A],viewBoxSize:[l,c],i0:i,i1:o,dim0A:d[0].slice(0,16),dim0B:d[0].slice(16,32),dim0C:d[0].slice(32,48),dim0D:d[0].slice(48,64),dim1A:d[1].slice(0,16),dim1B:d[1].slice(16,32),dim1C:d[1].slice(32,48),dim1D:d[1].slice(48,64),drwLayer:g,contextColor:[y[0]/255,y[1]/255,y[2]/255,y[3]<1?y[3]:Math.max(1/255,Math.pow(1/t.lines.color.length,1/3))],scissorX:(r===e?0:a+p)+(t.pad.l-p)+t.layoutWidth*I.x[0],scissorWidth:(r===n?C-a+p:l+.5)+(r===e?a+p:0),scissorY:A+t.pad.b+t.layoutHeight*I.y[0],scissorHeight:c,viewportX:t.pad.l-p+t.layoutWidth*I.x[0],viewportY:t.pad.b+t.layoutHeight*I.y[0],viewportWidth:C,viewportHeight:m},h)}function y(t){var e=l-1,n=Math.max(0,Math.floor(t[0]*e),0),r=Math.min(e,Math.ceil(t[1]*e),e);return[Math.min(n,r),Math.max(n,r)]}e.exports=function(t,e){var n,r,c,I,E,v=e.context,B=e.pick,b=e.regl,Q={currentRafs:{},drawCompleted:!0,clearOnly:!1},w=function(t){for(var e={},n=0;n<=a;n+=4)e[C(n)]=t.buffer({usage:"dynamic",type:"float",data:new Uint8Array(0)});return e}(b),x=b.texture(g),_=[];M(e);var D=b({profile:!1,blend:{enable:v,func:{srcRGB:"src alpha",dstRGB:"one minus src alpha",srcAlpha:1,dstAlpha:1},equation:{rgb:"add",alpha:"add"},color:[0,0,0,0]},depth:{enable:!v,mask:!0,func:"less",range:[0,1]},cull:{enable:!0,face:"back"},scissor:{enable:!0,box:{x:b.prop("scissorX"),y:b.prop("scissorY"),width:b.prop("scissorWidth"),height:b.prop("scissorHeight")}},viewport:{x:b.prop("viewportX"),y:b.prop("viewportY"),width:b.prop("viewportWidth"),height:b.prop("viewportHeight")},dither:!1,vert:i,frag:o,primitive:"lines",lineWidth:1,attributes:w,uniforms:{resolution:b.prop("resolution"),viewBoxPos:b.prop("viewBoxPos"),viewBoxSize:b.prop("viewBoxSize"),dim0A:b.prop("dim0A"),dim1A:b.prop("dim1A"),dim0B:b.prop("dim0B"),dim1B:b.prop("dim1B"),dim0C:b.prop("dim0C"),dim1C:b.prop("dim1C"),dim0D:b.prop("dim0D"),dim1D:b.prop("dim1D"),loA:b.prop("loA"),hiA:b.prop("hiA"),loB:b.prop("loB"),hiB:b.prop("hiB"),loC:b.prop("loC"),hiC:b.prop("hiC"),loD:b.prop("loD"),hiD:b.prop("hiD"),palette:x,contextColor:b.prop("contextColor"),mask:b.prop("maskTexture"),drwLayer:b.prop("drwLayer"),maskHeight:b.prop("maskHeight")},offset:b.prop("offset"),count:b.prop("count")});function M(t){n=t.model,r=t.viewModel,c=r.dimensions.slice(),I=c[0]?c[0].values.length:0;var e=n.lines,i=B?e.color.map(function(t,n){return n/e.color.length}):e.color,o=function(t,e,n){for(var r,i=new Array(t*(a+4)),o=0,s=0;s<t;s++){for(var l=0;l<a;l++)i[o++]=l<e.length?e[l].paddedUnitValues[s]:.5;i[o++]=f(s,2),i[o++]=f(s,1),i[o++]=f(s,0),i[o++]=(r=n[s],Math.max(A,Math.min(1-A,r)))}return i}(I,c,i);!function(t,e,n){for(var r=0;r<=a;r+=4)t[C(r)](p(r/4,e,n))}(w,I,o),v||B||(x=b.texture(s.extendFlat({data:function(t,e){for(var n=new Array(256),r=0;r<256;r++)n[r]=t(r/255).concat(255);return n}(n.unitToColor)},g)))}return{render:function(t,e,r){var i,o,a,s=t.length,A=1/0,u=-1/0;for(i=0;i<s;i++)t[i].dim0.canvasX<A&&(A=t[i].dim0.canvasX,o=i),t[i].dim1.canvasX>u&&(u=t[i].dim1.canvasX,a=i);0===s&&h(b,0,0,n.canvasWidth,n.canvasHeight);var g=function(t){var e,n,r,i=[[],[]];for(r=0;r<64;r++){var o=!t&&r<c.length?c[r].brush.filter.getBounds():[-1/0,1/0];i[0][r]=o[0],i[1][r]=o[1]}var a=8*l,s=new Array(a);for(e=0;e<a;e++)s[e]=255;if(!t)for(e=0;e<c.length;e++){var A=e%8,u=(e-A)/8,g=Math.pow(2,A),h=c[e].brush.filter.get();if(!(h.length<2)){var d=y(h[0])[1];for(n=1;n<h.length;n++){var f=y(h[n]);for(r=d+1;r<f[0];r++)s[8*r+u]&=~g;d=Math.max(d,f[1])}}}var p={shape:[8,l],format:"alpha",type:"uint8",mag:"nearest",min:"nearest",data:s};return E?E(p):E=b.texture(p),{maskTexture:E,maskHeight:l,loA:i[0].slice(0,16),loB:i[0].slice(16,32),loC:i[0].slice(32,48),loD:i[0].slice(48,64),hiA:i[1].slice(0,16),hiB:i[1].slice(16,32),hiC:i[1].slice(32,48),hiD:i[1].slice(48,64)}}(v);for(i=0;i<s;i++){var f=t[i],p=f.dim0.crossfilterDimensionIndex,C=f.dim1.crossfilterDimensionIndex,w=f.canvasX,x=f.canvasY,M=w+f.panelSizeX;if(e||!_[p]||_[p][0]!==w||_[p][1]!==M){_[p]=[w,M];var k=m(n,o,a,i,p,C,w,x,f.panelSizeX,f.panelSizeY,f.dim0.crossfilterDimensionIndex,v?0:B?2:1,g);Q.clearOnly=r;var S=e?n.lines.blockLineCount:I;d(b,D,Q,S,I,k)}}},readPixel:function(t,e){return b.read({x:t,y:e,width:1,height:1,data:u}),u},readPixels:function(t,e,n,r){var i=new Uint8Array(4*n*r);return b.read({x:t,y:e,width:n,height:r,data:i}),i},destroy:function(){for(var e in t.style["pointer-events"]="none",x.destroy(),E&&E.destroy(),w)w[e].destroy()},update:M}}},{"../../lib":719,"./constants":1078,glslify:409}],1083:[function(t,e,n){"use strict";e.exports=function(t,e,n,r){var i,o;for(r||(r=1/0),i=0;i<e.length;i++)(o=e[i]).visible&&(r=Math.min(r,o[n].length));for(r===1/0&&(r=0),t._length=r,i=0;i<e.length;i++)(o=e[i]).visible&&(o._length=r);return r}},{}],1084:[function(t,e,n){"use strict";var r=t("d3"),i=t("color-rgba"),o=t("../../plots/cartesian/axes"),a=t("../../lib"),s=t("../../lib/svg_text_utils"),A=t("../../components/drawing"),l=t("../../components/colorscale"),c=t("../../lib/gup"),u=c.keyFun,g=c.repeat,h=c.unwrap,d=t("./helpers"),f=t("./constants"),p=t("./axisbrush"),I=t("./lines");function C(t,e,n){return a.aggNums(t,null,e,n)}function m(t,e){return E(C(Math.min,t,e),C(Math.max,t,e))}function y(t){var e=t.range;return e?E(e[0],e[1]):m(t.values,t._length)}function E(t,e){return!isNaN(t)&&isFinite(t)||(t=0),!isNaN(e)&&isFinite(e)||(e=0),t===e&&(0===t?(t-=1,e+=1):(t*=.9,e*=1.1)),[t,e]}function v(t){return t.dimensions.some(function(t){return t.brush.filterSpecified})}function B(t,e,n){var o=h(e),s=o.trace,A=d.convertTypedArray(o.lineColor),c=s.line,u={color:i(f.deselectedLineColor)},g=l.extractOpts(c),p=g.reversescale?l.flipScale(o.cscale):o.cscale,I=s.domain,C=s.dimensions,m=t.width,E=s.labelangle,v=s.labelside,B=s.labelfont,b=s.tickfont,Q=s.rangefont,w=a.extendDeepNoArrays({},c,{color:A.map(r.scale.linear().domain(y({values:A,range:[g.min,g.max],_length:s._length}))),blockLineCount:f.blockLineCount,canvasOverdrag:f.overdrag*f.canvasPixelRatio}),x=Math.floor(m*(I.x[1]-I.x[0])),_=Math.floor(t.height*(I.y[1]-I.y[0])),D=t.margin||{l:80,r:80,t:100,b:80},M=x,k=_;return{key:n,colCount:C.filter(d.isVisible).length,dimensions:C,tickDistance:f.tickDistance,unitToColor:function(t){var e=t.map(function(t){return t[0]}),n=t.map(function(t){var e=i(t[1]);return r.rgb("rgb("+e[0]+","+e[1]+","+e[2]+")")}),o="rgb".split("").map(function(t){return r.scale.linear().clamp(!0).domain(e).range(n.map((i=t,function(t){return t[i]})));var i});return function(t){return o.map(function(e){return e(t)})}}(p),lines:w,deselectedLines:u,labelAngle:E,labelSide:v,labelFont:B,tickFont:b,rangeFont:Q,layoutWidth:m,layoutHeight:t.height,domain:I,translateX:I.x[0]*m,translateY:t.height-I.y[1]*t.height,pad:D,canvasWidth:M*f.canvasPixelRatio+2*w.canvasOverdrag,canvasHeight:k*f.canvasPixelRatio,width:M,height:k,canvasPixelRatio:f.canvasPixelRatio}}function b(t,e,n){var i=n.width,o=n.height,s=n.dimensions,A=n.canvasPixelRatio,l=function(t){return i*t/Math.max(1,n.colCount-1)},c=f.verticalPadding/o,u=function(t,e){return r.scale.linear().range([e,t-e])}(o,f.verticalPadding),g={key:n.key,xScale:l,model:n,inBrushDrag:!1},h={};return g.dimensions=s.filter(d.isVisible).map(function(i,s){var I=function(t,e){return r.scale.linear().domain(y(t)).range([e,1-e])}(i,c),C=h[i.label];h[i.label]=(C||0)+1;var m=i.label+(C?"__"+C:""),E=i.constraintrange,B=E&&E.length;B&&!Array.isArray(E[0])&&(E=[E]);var b=B?E.map(function(t){return t.map(I)}):[[-1/0,1/0]],Q=i.values;Q.length>i._length&&(Q=Q.slice(0,i._length));var w,x=i.tickvals;function _(t,e){return{val:t,text:w[e]}}function D(t,e){return t.val-e.val}if(Array.isArray(x)&&x.length){w=i.ticktext,Array.isArray(w)&&w.length?w.length>x.length?w=w.slice(0,x.length):x.length>w.length&&(x=x.slice(0,w.length)):w=x.map(r.format(i.tickformat));for(var M=1;M<x.length;M++)if(x[M]<x[M-1]){for(var k=x.map(_).sort(D),S=0;S<x.length;S++)x[S]=k[S].val,w[S]=k[S].text;break}}else x=void 0;return Q=d.convertTypedArray(Q),Q=d.convertTypedArray(Q),{key:m,label:i.label,tickFormat:i.tickformat,tickvals:x,ticktext:w,ordinal:d.isOrdinal(i),multiselect:i.multiselect,xIndex:s,crossfilterDimensionIndex:s,visibleIndex:i._index,height:o,values:Q,paddedUnitValues:Q.map(I),unitTickvals:x&&x.map(I),xScale:l,x:l(s),canvasX:l(s)*A,unitToPaddedPx:u,domainScale:function(t,e,n,i,o){var a,s,A=y(n);return i?r.scale.ordinal().domain(i.map((a=r.format(n.tickformat),s=o,s?function(t,e){var n=s[e];return null==n?a(t):n}:a))).range(i.map(function(n){var r=(n-A[0])/(A[1]-A[0]);return t-e+r*(2*e-t)})):r.scale.linear().domain(A).range([t-e,e])}(o,f.verticalPadding,i,x,w),ordinalScale:function(t){if(t.tickvals){var e=y(t);return r.scale.ordinal().domain(t.tickvals).range(t.tickvals.map(function(t){return(t-e[0])/(e[1]-e[0])}))}}(i),parent:g,model:n,brush:p.makeBrush(t,B,b,function(){t.linePickActive(!1)},function(){var e=g;e.focusLayer&&e.focusLayer.render(e.panels,!0);var n=v(e);!t.contextShown()&&n?(e.contextLayer&&e.contextLayer.render(e.panels,!0),t.contextShown(!0)):t.contextShown()&&!n&&(e.contextLayer&&e.contextLayer.render(e.panels,!0,!0),t.contextShown(!1))},function(n){if(g.focusLayer.render(g.panels,!0),g.pickLayer&&g.pickLayer.render(g.panels,!0),t.linePickActive(!0),e&&e.filterChanged){var r=I.invert,o=n.map(function(t){return t.map(r).sort(a.sorterAsc)}).sort(function(t,e){return t[0]-e[0]});e.filterChanged(g.key,i._index,o)}})}}),g}function Q(t){t.classed(f.cn.axisExtentText,!0).attr("text-anchor","middle").style("cursor","default").style("user-select","none")}function w(t,e){var n="top"===e?1:-1,r=t*Math.PI/180;return{dir:n,dx:Math.sin(r),dy:Math.cos(r),degrees:t}}function x(t,e){for(var n=e.panels||(e.panels=[]),r=t.data(),i=0;i<r.length-1;i++){var o=n[i]||(n[i]={}),a=r[i],s=r[i+1];o.dim0=a,o.dim1=s,o.canvasX=a.canvasX,o.panelSizeX=s.canvasX-a.canvasX,o.panelSizeY=e.model.canvasHeight,o.y=0,o.canvasY=0}}function _(t,e){return o.tickText(t._ax,e,!1).text}function D(t,e){if(t.ordinal)return"";var n=t.domainScale.domain(),r=n[e?n.length-1:0];return _(t.model.dimensions[t.visibleIndex],r)}e.exports=function(t,e,n,i){var l=t._fullLayout,c=l._toppaper,C=l._glcontainer;!function(t){for(var e=0;e<t.length;e++)for(var n=0;n<t[e].length;n++)for(var r=t[e][n].trace,i=r.dimensions,a=0;a<i.length;a++){var s=i[a].values,A=i[a]._ax;A&&(A.range?A.range=E(A.range[0],A.range[1]):A.range=m(s,r._length),A.dtick||(A.dtick=.01*(Math.abs(A.range[1]-A.range[0])||1)),A.tickformat=i[a].tickformat,o.calcTicks(A),A.cleanRange())}}(e);var y,M,k=(y=!0,M=!1,{linePickActive:function(t){return arguments.length?y=!!t:y},contextShown:function(t){return arguments.length?M=!!t:M}}),S=e.filter(function(t){return h(t).trace.visible}).map(B.bind(0,n)).map(b.bind(0,k,i));C.each(function(t,e){return a.extendFlat(t,S[e])});var L=C.selectAll(".gl-canvas").each(function(t){t.viewModel=S[0],t.model=t.viewModel?t.viewModel.model:null}),R=null;L.filter(function(t){return t.pick}).style("pointer-events","auto").on("mousemove",function(t){if(k.linePickActive()&&t.lineLayer&&i&&i.hover){var e=r.event,n=this.width,o=this.height,a=r.mouse(this),s=a[0],A=a[1];if(s<0||A<0||s>=n||A>=o)return;var l=t.lineLayer.readPixel(s,o-1-A),c=0!==l[3],u=c?l[2]+256*(l[1]+256*l[0]):null,g={x:s,y:A,clientX:e.clientX,clientY:e.clientY,dataIndex:t.model.key,curveNumber:u};u!==R&&(c?i.hover(g):i.unhover&&i.unhover(g),R=u)}}),L.style("opacity",function(t){return t.pick?0:1}),c.style("background","rgba(255, 255, 255, 0)");var N=c.selectAll("."+f.cn.parcoords).data(S,u);N.exit().remove(),N.enter().append("g").classed(f.cn.parcoords,!0).style("shape-rendering","crispEdges").style("pointer-events","none"),N.attr("transform",function(t){return"translate("+t.model.translateX+","+t.model.translateY+")"});var F=N.selectAll("."+f.cn.parcoordsControlView).data(g,u);F.enter().append("g").classed(f.cn.parcoordsControlView,!0),F.attr("transform",function(t){return"translate("+t.model.pad.l+","+t.model.pad.t+")"});var T=F.selectAll("."+f.cn.yAxis).data(function(t){return t.dimensions},u);T.enter().append("g").classed(f.cn.yAxis,!0),F.each(function(t){x(T,t)}),L.each(function(t){if(t.viewModel){!t.lineLayer||i?t.lineLayer=I(this,t):t.lineLayer.update(t),(t.key||0===t.key)&&(t.viewModel[t.key]=t.lineLayer);var e=!t.context||i;t.lineLayer.render(t.viewModel.panels,e)}}),T.attr("transform",function(t){return"translate("+t.xScale(t.xIndex)+", 0)"}),T.call(r.behavior.drag().origin(function(t){return t}).on("drag",function(t){var e=t.parent;k.linePickActive(!1),t.x=Math.max(-f.overdrag,Math.min(t.model.width+f.overdrag,r.event.x)),t.canvasX=t.x*t.model.canvasPixelRatio,T.sort(function(t,e){return t.x-e.x}).each(function(e,n){e.xIndex=n,e.x=t===e?e.x:e.xScale(e.xIndex),e.canvasX=e.x*e.model.canvasPixelRatio}),x(T,e),T.filter(function(e){return 0!==Math.abs(t.xIndex-e.xIndex)}).attr("transform",function(t){return"translate("+t.xScale(t.xIndex)+", 0)"}),r.select(this).attr("transform","translate("+t.x+", 0)"),T.each(function(n,r,i){i===t.parent.key&&(e.dimensions[r]=n)}),e.contextLayer&&e.contextLayer.render(e.panels,!1,!v(e)),e.focusLayer.render&&e.focusLayer.render(e.panels)}).on("dragend",function(t){var e=t.parent;t.x=t.xScale(t.xIndex),t.canvasX=t.x*t.model.canvasPixelRatio,x(T,e),r.select(this).attr("transform",function(t){return"translate("+t.x+", 0)"}),e.contextLayer&&e.contextLayer.render(e.panels,!1,!v(e)),e.focusLayer&&e.focusLayer.render(e.panels),e.pickLayer&&e.pickLayer.render(e.panels,!0),k.linePickActive(!0),i&&i.axesMoved&&i.axesMoved(e.key,e.dimensions.map(function(t){return t.crossfilterDimensionIndex}))})),T.exit().remove();var U=T.selectAll("."+f.cn.axisOverlays).data(g,u);U.enter().append("g").classed(f.cn.axisOverlays,!0),U.selectAll("."+f.cn.axis).remove();var O=U.selectAll("."+f.cn.axis).data(g,u);O.enter().append("g").classed(f.cn.axis,!0),O.each(function(t){var e=t.model.height/t.model.tickDistance,n=t.domainScale,i=n.domain();r.select(this).call(r.svg.axis().orient("left").tickSize(4).outerTickSize(2).ticks(e,t.tickFormat).tickValues(t.ordinal?i:null).tickFormat(function(e){return d.isOrdinal(t)?e:_(t.model.dimensions[t.visibleIndex],e)}).scale(n)),A.font(O.selectAll("text"),t.model.tickFont)}),O.selectAll(".domain, .tick>line").attr("fill","none").attr("stroke","black").attr("stroke-opacity",.25).attr("stroke-width","1px"),O.selectAll("text").style("text-shadow","1px 1px 1px #fff, -1px -1px 1px #fff, 1px -1px 1px #fff, -1px 1px 1px #fff").style("cursor","default").style("user-select","none");var G=U.selectAll("."+f.cn.axisHeading).data(g,u);G.enter().append("g").classed(f.cn.axisHeading,!0);var H=G.selectAll("."+f.cn.axisTitle).data(g,u);H.enter().append("text").classed(f.cn.axisTitle,!0).attr("text-anchor","middle").style("cursor","ew-resize").style("user-select","none").style("pointer-events","auto"),H.text(function(t){return t.label}).each(function(e){var n=r.select(this);A.font(n,e.model.labelFont),s.convertToTspans(n,t)}).attr("transform",function(t){var e=w(t.model.labelAngle,t.model.labelSide),n=f.axisTitleOffset;return(e.dir>0?"":"translate(0,"+(2*n+t.model.height)+")")+"rotate("+e.degrees+")translate("+-n*e.dx+","+-n*e.dy+")"}).attr("text-anchor",function(t){var e=w(t.model.labelAngle,t.model.labelSide);return 2*Math.abs(e.dx)>Math.abs(e.dy)?e.dir*e.dx<0?"start":"end":"middle"});var j=U.selectAll("."+f.cn.axisExtent).data(g,u);j.enter().append("g").classed(f.cn.axisExtent,!0);var Y=j.selectAll("."+f.cn.axisExtentTop).data(g,u);Y.enter().append("g").classed(f.cn.axisExtentTop,!0),Y.attr("transform","translate(0,"+-f.axisExtentOffset+")");var P=Y.selectAll("."+f.cn.axisExtentTopText).data(g,u);P.enter().append("text").classed(f.cn.axisExtentTopText,!0).call(Q),P.text(function(t){return D(t,!0)}).each(function(t){A.font(r.select(this),t.model.rangeFont)});var K=j.selectAll("."+f.cn.axisExtentBottom).data(g,u);K.enter().append("g").classed(f.cn.axisExtentBottom,!0),K.attr("transform",function(t){return"translate(0,"+(t.model.height+f.axisExtentOffset)+")"});var z=K.selectAll("."+f.cn.axisExtentBottomText).data(g,u);z.enter().append("text").classed(f.cn.axisExtentBottomText,!0).attr("dy","0.75em").call(Q),z.text(function(t){return D(t,!1)}).each(function(t){A.font(r.select(this),t.model.rangeFont)}),p.ensureAxisBrush(U)}},{"../../components/colorscale":605,"../../components/drawing":614,"../../lib":719,"../../lib/gup":717,"../../lib/svg_text_utils":743,"../../plots/cartesian/axes":767,"./axisbrush":1075,"./constants":1078,"./helpers":1080,"./lines":1082,"color-rgba":122,d3:163}],1085:[function(t,e,n){"use strict";var r=t("./parcoords"),i=t("../../lib/prepare_regl"),o=t("./helpers").isVisible;function a(t,e,n){var r=e.indexOf(n),i=t.indexOf(r);return-1===i&&(i+=e.length),i}e.exports=function(t,e){var n=t._fullLayout;if(i(t)){var s={},A={},l={},c={},u=n._size;e.forEach(function(e,n){var r=e[0].trace;l[n]=r.index;var i=c[n]=r._fullInput.index;s[n]=t.data[i].dimensions,A[n]=t.data[i].dimensions.slice()}),r(t,e,{width:u.w,height:u.h,margin:{t:u.t,r:u.r,b:u.b,l:u.l}},{filterChanged:function(e,r,i){var o=A[e][r],a=i.map(function(t){return t.slice()}),s="dimensions["+r+"].constraintrange",u=n._tracePreGUI[t._fullData[l[e]]._fullInput.uid];if(void 0===u[s]){var g=o.constraintrange;u[s]=g||null}var h=t._fullData[l[e]].dimensions[r];a.length?(1===a.length&&(a=a[0]),o.constraintrange=a,h.constraintrange=a.slice(),a=[a]):(delete o.constraintrange,delete h.constraintrange,a=null);var d={};d[s]=a,t.emit("plotly_restyle",[d,[c[e]]])},hover:function(e){t.emit("plotly_hover",e)},unhover:function(e){t.emit("plotly_unhover",e)},axesMoved:function(e,n){var r=function(t,e){return function(n,r){return a(t,e,n)-a(t,e,r)}}(n,A[e].filter(o));s[e].sort(r),A[e].filter(function(t){return!o(t)}).sort(function(t){return A[e].indexOf(t)}).forEach(function(t){s[e].splice(s[e].indexOf(t),1),s[e].splice(A[e].indexOf(t),0,t)}),t.emit("plotly_restyle",[{dimensions:[s[e]]},[c[e]]])}})}}},{"../../lib/prepare_regl":732,"./helpers":1080,"./parcoords":1084}],1086:[function(t,e,n){"use strict";var r=t("../../plots/attributes"),i=t("../../plots/domain").attributes,o=t("../../plots/font_attributes"),a=t("../../components/color/attributes"),s=t("../../components/fx/hovertemplate_attributes"),A=t("../../lib/extend").extendFlat,l=o({editType:"plot",arrayOk:!0,colorEditType:"plot"});e.exports={labels:{valType:"data_array",editType:"calc"},label0:{valType:"number",dflt:0,editType:"calc"},dlabel:{valType:"number",dflt:1,editType:"calc"},values:{valType:"data_array",editType:"calc"},marker:{colors:{valType:"data_array",editType:"calc"},line:{color:{valType:"color",dflt:a.defaultLine,arrayOk:!0,editType:"style"},width:{valType:"number",min:0,dflt:0,arrayOk:!0,editType:"style"},editType:"calc"},editType:"calc"},text:{valType:"data_array",editType:"calc"},hovertext:{valType:"string",dflt:"",arrayOk:!0,editType:"style"},scalegroup:{valType:"string",dflt:"",editType:"calc"},textinfo:{valType:"flaglist",flags:["label","text","value","percent"],extras:["none"],editType:"calc"},hoverinfo:A({},r.hoverinfo,{flags:["label","text","value","percent","name"]}),hovertemplate:s({},{keys:["label","color","value","percent","text"]}),textposition:{valType:"enumerated",values:["inside","outside","auto","none"],dflt:"auto",arrayOk:!0,editType:"plot"},textfont:A({},l,{}),insidetextfont:A({},l,{}),outsidetextfont:A({},l,{}),title:{text:{valType:"string",dflt:"",editType:"plot"},font:A({},l,{}),position:{valType:"enumerated",values:["top left","top center","top right","middle center","bottom left","bottom center","bottom right"],editType:"plot"},editType:"plot"},domain:i({name:"pie",trace:!0,editType:"calc"}),hole:{valType:"number",min:0,max:1,dflt:0,editType:"calc"},sort:{valType:"boolean",dflt:!0,editType:"calc"},direction:{valType:"enumerated",values:["clockwise","counterclockwise"],dflt:"counterclockwise",editType:"calc"},rotation:{valType:"number",min:-360,max:360,dflt:0,editType:"calc"},pull:{valType:"number",min:0,max:1,dflt:0,arrayOk:!0,editType:"calc"},_deprecated:{title:{valType:"string",dflt:"",editType:"calc"},titlefont:A({},l,{}),titleposition:{valType:"enumerated",values:["top left","top center","top right","middle center","bottom left","bottom center","bottom right"],editType:"calc"}}}},{"../../components/color/attributes":592,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../../plots/domain":792,"../../plots/font_attributes":793}],1087:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../plots/get_data").getModuleCalcData;n.name="pie",n.plot=function(t){var e=r.getModule("pie"),n=i(t.calcdata,e)[0];e.plot(t,n)},n.clean=function(t,e,n,r){var i=r._has&&r._has("pie"),o=e._has&&e._has("pie");i&&!o&&r._pielayer.selectAll("g.trace").remove()}},{"../../plots/get_data":802,"../../registry":847}],1088:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib").isArrayOrTypedArray,o=t("tinycolor2"),a=t("../../components/color"),s=t("./helpers"),A=t("../../lib").isValidTextValue,l={};function c(t){return function(e,n){return!!e&&!!(e=o(e)).isValid()&&(e=a.addOpacity(e,e.getAlpha()),t[n]||(t[n]=e),e)}}function u(t,e){var n,r=JSON.stringify(t),i=e[r];if(!i){for(i=t.slice(),n=0;n<t.length;n++)i.push(o(t[n]).lighten(20).toHexString());for(n=0;n<t.length;n++)i.push(o(t[n]).darken(20).toHexString());e[r]=i}return i}e.exports={calc:function(t,e){var n,o,a=[],l=t._fullLayout,u=l.hiddenlabels||[],g=e.labels,h=e.marker.colors||[],d=e.values,f=i(d)&&d.length;if(e.dlabel)for(g=new Array(d.length),n=0;n<d.length;n++)g[n]=String(e.label0+n*e.dlabel);var p={},I=c(l["_"+e.type+"colormap"]),C=(f?d:g).length,m=0,y=!1;for(n=0;n<C;n++){var E,v,B;if(f){if(E=d[n],!r(E))continue;if((E=+E)<0)continue}else E=1;void 0!==(v=g[n])&&""!==v||(v=n);var b=p[v=String(v)];void 0===b?(p[v]=a.length,(B=-1!==u.indexOf(v))||(m+=E),a.push({v:E,label:v,color:I(h[n],v),i:n,pts:[n],hidden:B})):(y=!0,(o=a[b]).v+=E,o.pts.push(n),o.hidden||(m+=E),!1===o.color&&h[n]&&(o.color=I(h[n],v)))}("funnelarea"===e.type?y:e.sort)&&a.sort(function(t,e){return e.v-t.v}),a[0]&&(a[0].vTotal=m);var Q=e.textinfo;if(Q&&"none"!==Q){var w,x=Q.split("+"),_=function(t){return-1!==x.indexOf(t)},D=_("label"),M=_("text"),k=_("value"),S=_("percent"),L=l.separators;for(n=0;n<a.length;n++){if(o=a[n],w=D?[o.label]:[],M){var R=s.getFirstFilled(e.text,o.pts);A(R)&&w.push(R)}k&&w.push(s.formatPieValue(o.v,L)),S&&w.push(s.formatPiePercent(o.v/m,L)),o.text=w.join("<br>")}}return a},crossTraceCalc:function(t,e){var n=(e||{}).type;n||(n="pie");var r=t._fullLayout,i=t.calcdata,o=r[n+"colorway"],a=r["_"+n+"colormap"];r["extend"+n+"colors"]&&(o=u(o,l));for(var s=0,A=0;A<i.length;A++){var c=i[A];if(c[0].trace.type===n)for(var g=0;g<c.length;g++){var h=c[g];!1===h.color&&(a[h.label]?h.color=a[h.label]:(a[h.label]=h.color=o[s%o.length],s++))}}},makePullColorFn:c,generateExtendedColors:u}},{"../../components/color":593,"../../lib":719,"./helpers":1091,"fast-isnumeric":225,tinycolor2:537}],1089:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./attributes"),o=t("../../plots/domain").defaults,a=t("../bar/defaults").handleText;e.exports=function(t,e,n,s){function A(n,o){return r.coerce(t,e,i,n,o)}var l,c=A("values"),u=r.isArrayOrTypedArray(c),g=A("labels");if(Array.isArray(g)?(l=g.length,u&&(l=Math.min(l,c.length))):u&&(l=c.length,A("label0"),A("dlabel")),l){e._length=l,A("marker.line.width")&&A("marker.line.color"),A("marker.colors"),A("scalegroup");var h=A("text"),d=A("textinfo",Array.isArray(h)?"text+percent":"percent");if(A("hovertext"),A("hovertemplate"),d&&"none"!==d){var f=A("textposition");a(t,e,s,A,f,{moduleHasSelected:!1,moduleHasUnselected:!1,moduleHasConstrain:!1,moduleHasCliponaxis:!1,moduleHasTextangle:!1,moduleHasInsideanchor:!1})}o(e,s,A);var p=A("hole");if(A("title.text")){var I=A("title.position",p?"middle center":"top center");p||"middle center"!==I||(e.title.position="top center"),r.coerceFont(A,"title.font",s.font)}A("sort"),A("direction"),A("rotation"),A("pull")}else e.visible=!1}},{"../../lib":719,"../../plots/domain":792,"../bar/defaults":861,"./attributes":1086}],1090:[function(t,e,n){"use strict";var r=t("../../components/fx/helpers").appendArrayMultiPointValues;e.exports=function(t,e){var n={curveNumber:e.index,pointNumbers:t.pts,data:e._input,fullData:e,label:t.label,color:t.color,value:t.v,percent:t.percent,text:t.text,v:t.v};return 1===t.pts.length&&(n.pointNumber=n.i=t.pts[0]),r(n,e,t.pts),"funnelarea"===e.type&&(delete n.v,delete n.i),n}},{"../../components/fx/helpers":628}],1091:[function(t,e,n){"use strict";var r=t("../../lib");n.formatPiePercent=function(t,e){var n=(100*t).toPrecision(3);return-1!==n.lastIndexOf(".")&&(n=n.replace(/[.]?0+$/,"")),r.numSeparate(n,e)+"%"},n.formatPieValue=function(t,e){var n=t.toPrecision(10);return-1!==n.lastIndexOf(".")&&(n=n.replace(/[.]?0+$/,"")),r.numSeparate(n,e)},n.getFirstFilled=function(t,e){if(Array.isArray(t))for(var n=0;n<e.length;n++){var r=t[e[n]];if(r||0===r)return r}},n.castOption=function(t,e){return Array.isArray(t)?n.getFirstFilled(t,e):t||void 0}},{"../../lib":719}],1092:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),supplyLayoutDefaults:t("./layout_defaults"),layoutAttributes:t("./layout_attributes"),calc:t("./calc").calc,crossTraceCalc:t("./calc").crossTraceCalc,plot:t("./plot").plot,style:t("./style"),styleOne:t("./style_one"),moduleType:"trace",name:"pie",basePlotModule:t("./base_plot"),categories:["pie-like","pie","showLegend"],meta:{}}},{"./attributes":1086,"./base_plot":1087,"./calc":1088,"./defaults":1089,"./layout_attributes":1093,"./layout_defaults":1094,"./plot":1095,"./style":1096,"./style_one":1097}],1093:[function(t,e,n){"use strict";e.exports={hiddenlabels:{valType:"data_array",editType:"calc"},piecolorway:{valType:"colorlist",editType:"calc"},extendpiecolors:{valType:"boolean",dflt:!0,editType:"calc"}}},{}],1094:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./layout_attributes");e.exports=function(t,e){function n(n,o){return r.coerce(t,e,i,n,o)}n("hiddenlabels"),n("piecolorway",e.colorway),n("extendpiecolors")}},{"../../lib":719,"./layout_attributes":1093}],1095:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/fx"),o=t("../../components/color"),a=t("../../components/drawing"),s=t("../../lib"),A=t("../../lib/svg_text_utils"),l=t("./helpers"),c=t("./event_data");function u(t,e,n){var o=n[0],a=o.trace,A=o.cx,u=o.cy;"_hasHoverLabel"in a||(a._hasHoverLabel=!1),"_hasHoverEvent"in a||(a._hasHoverEvent=!1),t.on("mouseover",function(t){var n=e._fullLayout,g=e._fullData[a.index];if(!e._dragging&&!1!==n.hovermode){var h=g.hoverinfo;if(Array.isArray(h)&&(h=i.castHoverinfo({hoverinfo:[l.castOption(h,t.pts)],_module:a._module},n,0)),"all"===h&&(h="label+text+value+percent+name"),g.hovertemplate||"none"!==h&&"skip"!==h&&h){var d=t.rInscribed||0,f=A+t.pxmid[0]*(1-d),p=u+t.pxmid[1]*(1-d),I=n.separators,C=[];if(h&&-1!==h.indexOf("label")&&C.push(t.label),t.text=l.castOption(g.hovertext||g.text,t.pts),h&&-1!==h.indexOf("text")){var m=t.text;s.isValidTextValue(m)&&C.push(m)}t.value=t.v,t.valueLabel=l.formatPieValue(t.v,I),h&&-1!==h.indexOf("value")&&C.push(t.valueLabel),t.percent=t.v/o.vTotal,t.percentLabel=l.formatPiePercent(t.percent,I),h&&-1!==h.indexOf("percent")&&C.push(t.percentLabel);var y=g.hoverlabel,E=y.font;i.loneHover({trace:a,x0:f-d*o.r,x1:f+d*o.r,y:p,text:C.join("<br>"),name:g.hovertemplate||-1!==h.indexOf("name")?g.name:void 0,idealAlign:t.pxmid[0]<0?"left":"right",color:l.castOption(y.bgcolor,t.pts)||t.color,borderColor:l.castOption(y.bordercolor,t.pts),fontFamily:l.castOption(E.family,t.pts),fontSize:l.castOption(E.size,t.pts),fontColor:l.castOption(E.color,t.pts),nameLength:l.castOption(y.namelength,t.pts),textAlign:l.castOption(y.align,t.pts),hovertemplate:l.castOption(g.hovertemplate,t.pts),hovertemplateLabels:t,eventData:[c(t,g)]},{container:n._hoverlayer.node(),outerContainer:n._paper.node(),gd:e}),a._hasHoverLabel=!0}a._hasHoverEvent=!0,e.emit("plotly_hover",{points:[c(t,g)],event:r.event})}}),t.on("mouseout",function(t){var n=e._fullLayout,o=e._fullData[a.index],s=r.select(this).datum();a._hasHoverEvent&&(t.originalEvent=r.event,e.emit("plotly_unhover",{points:[c(s,o)],event:r.event}),a._hasHoverEvent=!1),a._hasHoverLabel&&(i.loneUnhover(n._hoverlayer.node()),a._hasHoverLabel=!1)}),t.on("click",function(t){var n=e._fullLayout,o=e._fullData[a.index];e._dragging||!1===n.hovermode||(e._hoverdata=[c(t,o)],i.click(e,r.event))})}function g(t,e,n){var r=l.castOption(t.insidetextfont.color,e.pts);!r&&t._input.textfont&&(r=l.castOption(t._input.textfont.color,e.pts));var i=l.castOption(t.insidetextfont.family,e.pts)||l.castOption(t.textfont.family,e.pts)||n.family,a=l.castOption(t.insidetextfont.size,e.pts)||l.castOption(t.textfont.size,e.pts)||n.size;return{color:r||o.contrast(e.color),family:i,size:a}}function h(t,e){for(var n,r,i=0;i<t.length;i++)if((r=(n=t[i][0]).trace).title.text){var o=r.title.text;r._meta&&(o=s.templateString(o,r._meta));var l=a.tester.append("text").attr("data-notex",1).text(o).call(a.font,r.title.font).call(A.convertToTspans,e),c=a.bBox(l.node(),!0);n.titleBox={width:c.width,height:c.height},l.remove()}}function d(t,e,n){var r=Math.sqrt(t.width*t.width+t.height*t.height),i=t.width/t.height,o=e.halfangle,a=e.ring,s=e.rInscribed,A=n.r||e.rpx1,l={scale:s*A*2/r,rCenter:1-s,rotate:0};if(l.scale>=1)return l;var c=i+1/(2*Math.tan(o)),u=A*Math.min(1/(Math.sqrt(c*c+.5)+c),a/(Math.sqrt(i*i+a/2)+i)),g={scale:2*u/t.height,rCenter:Math.cos(u/A)-u*i/A,rotate:(180/Math.PI*e.midangle+720)%180-90},h=1/i,d=h+1/(2*Math.tan(o)),f=A*Math.min(1/(Math.sqrt(d*d+.5)+d),a/(Math.sqrt(h*h+a/2)+h)),p={scale:2*f/t.width,rCenter:Math.cos(f/A)-f/i/A,rotate:(180/Math.PI*e.midangle+810)%180-90},I=p.scale>g.scale?p:g;return l.scale<1&&I.scale>l.scale?I:l}function f(t,e){return t.v!==e.vTotal||e.trace.hole?Math.min(1/(1+1/Math.sin(t.halfangle)),t.ring/2):1}function p(t,e){var n=e.pxmid[0],r=e.pxmid[1],i=t.width/2,o=t.height/2;return n<0&&(i*=-1),r<0&&(o*=-1),{scale:1,rCenter:1,rotate:0,x:i+Math.abs(o)*(i>0?1:-1)/2,y:o/(1+n*n/(r*r)),outside:!0}}function I(t,e){var n,r,i,o=t.trace,a={x:t.cx,y:t.cy},s={tx:0,ty:0};s.ty+=o.title.font.size,i=m(o),-1!==o.title.position.indexOf("top")?(a.y-=(1+i)*t.r,s.ty-=t.titleBox.height):-1!==o.title.position.indexOf("bottom")&&(a.y+=(1+i)*t.r);var A,l,c=(A=t.r,l=t.trace.aspectratio,A/(void 0===l?1:l)),u=e.w*(o.domain.x[1]-o.domain.x[0])/2;return-1!==o.title.position.indexOf("left")?(u+=c,a.x-=(1+i)*c,s.tx+=t.titleBox.width/2):-1!==o.title.position.indexOf("center")?u*=2:-1!==o.title.position.indexOf("right")&&(u+=c,a.x+=(1+i)*c,s.tx-=t.titleBox.width/2),n=u/t.titleBox.width,r=C(t,e)/t.titleBox.height,{x:a.x,y:a.y,scale:Math.min(n,r),tx:s.tx,ty:s.ty}}function C(t,e){var n=t.trace,r=e.h*(n.domain.y[1]-n.domain.y[0]);return Math.min(t.titleBox.height,r/2)}function m(t){var e,n=t.pull;if(!n)return 0;if(Array.isArray(n))for(n=0,e=0;e<t.pull.length;e++)t.pull[e]>n&&(n=t.pull[e]);return n}function y(t,e){for(var n=[],r=0;r<t.length;r++){var i=t[r][0],o=i.trace,a=o.domain,s=e.w*(a.x[1]-a.x[0]),A=e.h*(a.y[1]-a.y[0]);o.title.text&&"middle center"!==o.title.position&&(A-=C(i,e));var l=s/2,c=A/2;"funnelarea"!==o.type||o.scalegroup||(c/=o.aspectratio),i.r=Math.min(l,c)/(1+m(o)),i.cx=e.l+e.w*(o.domain.x[1]+o.domain.x[0])/2,i.cy=e.t+e.h*(1-o.domain.y[0])-A/2,o.title.text&&-1!==o.title.position.indexOf("bottom")&&(i.cy-=C(i,e)),o.scalegroup&&-1===n.indexOf(o.scalegroup)&&n.push(o.scalegroup)}!function(t,e){for(var n,r,i,o=0;o<e.length;o++){var a=1/0,s=e[o];for(r=0;r<t.length;r++)if(n=t[r][0],(i=n.trace).scalegroup===s){var A;if("pie"===i.type)A=n.r*n.r;else if("funnelarea"===i.type){var l,c;i.aspectratio>1?(l=n.r,c=l/i.aspectratio):(c=n.r,l=c*i.aspectratio),l*=(1+i.baseratio)/2,A=l*c}a=Math.min(a,A/n.vTotal)}for(r=0;r<t.length;r++)if(n=t[r][0],(i=n.trace).scalegroup===s){var u=a*n.vTotal;"funnelarea"===i.type&&(u/=(1+i.baseratio)/2,u/=i.aspectratio),n.r=Math.sqrt(u)}}}(t,n)}e.exports={plot:function(t,e){var n=t._fullLayout;h(e,t),y(e,n._size);var i=s.makeTraceGroups(n._pielayer,e,"trace").each(function(e){var i=r.select(this),c=e[0],h=c.trace;!function(t){var e,n,r,i=t[0],o=i.trace,a=o.rotation*Math.PI/180,s=2*Math.PI/i.vTotal,A="px0",l="px1";if("counterclockwise"===o.direction){for(e=0;e<t.length&&t[e].hidden;e++);if(e===t.length)return;a+=s*t[e].v,s*=-1,A="px1",l="px0"}function c(t){return[i.r*Math.sin(t),-i.r*Math.cos(t)]}for(r=c(a),e=0;e<t.length;e++)(n=t[e]).hidden||(n[A]=r,a+=s*n.v/2,n.pxmid=c(a),n.midangle=a,a+=s*n.v/2,r=c(a),n[l]=r,n.largeArc=n.v>i.vTotal/2?1:0,n.halfangle=Math.PI*Math.min(n.v/i.vTotal,.5),n.ring=1-o.hole,n.rInscribed=f(n,i))}(e),i.attr("stroke-linejoin","round"),i.each(function(){var i=r.select(this).selectAll("g.slice").data(e);i.enter().append("g").classed("slice",!0),i.exit().remove();var f=[[[],[]],[[],[]]],C=!1;i.each(function(n){if(n.hidden)r.select(this).selectAll("path,g").remove();else{n.pointNumber=n.i,n.curveNumber=h.index,f[n.pxmid[1]<0?0:1][n.pxmid[0]<0?0:1].push(n);var i=c.cx,o=c.cy,I=r.select(this),m=I.selectAll("path.surface").data([n]);if(m.enter().append("path").classed("surface",!0).style({"pointer-events":"all"}),I.call(u,t,e),h.pull){var y=+l.castOption(h.pull,n.pts)||0;y>0&&(i+=y*n.pxmid[0],o+=y*n.pxmid[1])}n.cxFinal=i,n.cyFinal=o;var E=h.hole;if(n.v===c.vTotal){var v="M"+(i+n.px0[0])+","+(o+n.px0[1])+x(n.px0,n.pxmid,!0,1)+x(n.pxmid,n.px0,!0,1)+"Z";E?m.attr("d","M"+(i+E*n.px0[0])+","+(o+E*n.px0[1])+x(n.px0,n.pxmid,!1,E)+x(n.pxmid,n.px0,!1,E)+"Z"+v):m.attr("d",v)}else{var B=x(n.px0,n.px1,!0,1);if(E){var b=1-E;m.attr("d","M"+(i+E*n.px1[0])+","+(o+E*n.px1[1])+x(n.px1,n.px0,!1,E)+"l"+b*n.px0[0]+","+b*n.px0[1]+B+"Z")}else m.attr("d","M"+i+","+o+"l"+n.px0[0]+","+n.px0[1]+B+"Z")}var Q=l.castOption(h.textposition,n.pts),w=I.selectAll("g.slicetext").data(n.text&&"none"!==Q?[0]:[]);w.enter().append("g").classed("slicetext",!0),w.exit().remove(),w.each(function(){var e=s.ensureSingle(r.select(this),"text","",function(t){t.attr("data-notex",1)});e.text(n.text).attr({class:"slicetext",transform:"","text-anchor":"middle"}).call(a.font,"outside"===Q?function(t,e,n){var r=l.castOption(t.outsidetextfont.color,e.pts)||l.castOption(t.textfont.color,e.pts)||n.color,i=l.castOption(t.outsidetextfont.family,e.pts)||l.castOption(t.textfont.family,e.pts)||n.family,o=l.castOption(t.outsidetextfont.size,e.pts)||l.castOption(t.textfont.size,e.pts)||n.size;return{color:r,family:i,size:o}}(h,n,t._fullLayout.font):g(h,n,t._fullLayout.font)).call(A.convertToTspans,t);var u,f=a.bBox(e.node());"outside"===Q?u=p(f,n):(u=d(f,n,c),"auto"===Q&&u.scale<1&&(e.call(a.font,h.outsidetextfont),h.outsidetextfont.family===h.insidetextfont.family&&h.outsidetextfont.size===h.insidetextfont.size||(f=a.bBox(e.node())),u=p(f,n)));var I=i+n.pxmid[0]*u.rCenter+(u.x||0),m=o+n.pxmid[1]*u.rCenter+(u.y||0);u.outside&&(n.yLabelMin=m-f.height/2,n.yLabelMid=m,n.yLabelMax=m+f.height/2,n.labelExtraX=0,n.labelExtraY=0,C=!0),e.attr("transform","translate("+I+","+m+")"+(u.scale<1?"scale("+u.scale+")":"")+(u.rotate?"rotate("+u.rotate+")":"")+"translate("+-(f.left+f.right)/2+","+-(f.top+f.bottom)/2+")")})}function x(t,e,r,i){var o=i*(e[0]-t[0]),a=i*(e[1]-t[1]);return"a"+i*c.r+","+i*c.r+" 0 "+n.largeArc+(r?" 1 ":" 0 ")+o+","+a}});var m=r.select(this).selectAll("g.titletext").data(h.title.text?[0]:[]);m.enter().append("g").classed("titletext",!0),m.exit().remove(),m.each(function(){var e,i=s.ensureSingle(r.select(this),"text","",function(t){t.attr("data-notex",1)}),o=h.title.text;h._meta&&(o=s.templateString(o,h._meta)),i.text(o).attr({class:"titletext",transform:"","text-anchor":"middle"}).call(a.font,h.title.font).call(A.convertToTspans,t),e="middle center"===h.title.position?function(t){var e=Math.sqrt(t.titleBox.width*t.titleBox.width+t.titleBox.height*t.titleBox.height);return{x:t.cx,y:t.cy,scale:t.trace.hole*t.r*2/e,tx:0,ty:-t.titleBox.height/2+t.trace.title.font.size}}(c):I(c,n._size),i.attr("transform","translate("+e.x+","+e.y+")"+(e.scale<1?"scale("+e.scale+")":"")+"translate("+e.tx+","+e.ty+")")}),C&&function(t,e){var n,r,i,o,a,s,A,c,u,g,h,d,f;function p(t,e){return t.pxmid[1]-e.pxmid[1]}function I(t,e){return e.pxmid[1]-t.pxmid[1]}function C(t,n){n||(n={});var i,c,u,h,d,f,p=n.labelExtraY+(r?n.yLabelMax:n.yLabelMin),I=r?t.yLabelMin:t.yLabelMax,C=r?t.yLabelMax:t.yLabelMin,m=t.cyFinal+a(t.px0[1],t.px1[1]),y=p-I;if(y*A>0&&(t.labelExtraY=y),Array.isArray(e.pull))for(c=0;c<g.length;c++)(u=g[c])===t||(l.castOption(e.pull,t.pts)||0)>=(l.castOption(e.pull,u.pts)||0)||((t.pxmid[1]-u.pxmid[1])*A>0?(h=u.cyFinal+a(u.px0[1],u.px1[1]),(y=h-I-t.labelExtraY)*A>0&&(t.labelExtraY+=y)):(C+t.labelExtraY-m)*A>0&&(i=3*s*Math.abs(c-g.indexOf(t)),d=u.cxFinal+o(u.px0[0],u.px1[0]),(f=d+i-(t.cxFinal+t.pxmid[0])-t.labelExtraX)*s>0&&(t.labelExtraX+=f)))}for(r=0;r<2;r++)for(i=r?p:I,a=r?Math.max:Math.min,A=r?1:-1,n=0;n<2;n++){for(o=n?Math.max:Math.min,s=n?1:-1,(c=t[r][n]).sort(i),u=t[1-r][n],g=u.concat(c),d=[],h=0;h<c.length;h++)void 0!==c[h].yLabelMid&&d.push(c[h]);for(f=!1,h=0;r&&h<u.length;h++)if(void 0!==u[h].yLabelMid){f=u[h];break}for(h=0;h<d.length;h++){var m=h&&d[h-1];f&&!h&&(m=f),C(d[h],m)}}}(f,h),function(t,e){t.each(function(t){var n=r.select(this);if(t.labelExtraX||t.labelExtraY){var i=n.select("g.slicetext text");i.attr("transform","translate("+t.labelExtraX+","+t.labelExtraY+")"+i.attr("transform"));var a=t.cxFinal+t.pxmid[0],A=t.cyFinal+t.pxmid[1],l="M"+a+","+A,c=(t.yLabelMax-t.yLabelMin)*(t.pxmid[0]<0?-1:1)/4;if(t.labelExtraX){var u=t.labelExtraX*t.pxmid[1]/t.pxmid[0],g=t.yLabelMid+t.labelExtraY-(t.cyFinal+t.pxmid[1]);Math.abs(u)>Math.abs(g)?l+="l"+g*t.pxmid[0]/t.pxmid[1]+","+g+"H"+(a+t.labelExtraX+c):l+="l"+t.labelExtraX+","+u+"v"+(g-u)+"h"+c}else l+="V"+(t.yLabelMid+t.labelExtraY)+"h"+c;s.ensureSingle(n,"path","textline").call(o.stroke,e.outsidetextfont.color).attr({"stroke-width":Math.min(2,e.outsidetextfont.size/8),d:l,fill:"none"})}else n.select("path.textline").remove()})}(i,h)})});setTimeout(function(){i.selectAll("tspan").each(function(){var t=r.select(this);t.attr("dy")&&t.attr("dy",t.attr("dy"))})},0)},transformInsideText:d,determineInsideTextFont:g,positionTitleOutside:I,prerenderTitles:h,layoutAreas:y,attachFxHandlers:u}},{"../../components/color":593,"../../components/drawing":614,"../../components/fx":632,"../../lib":719,"../../lib/svg_text_utils":743,"./event_data":1090,"./helpers":1091,d3:163}],1096:[function(t,e,n){"use strict";var r=t("d3"),i=t("./style_one");e.exports=function(t){t._fullLayout._pielayer.selectAll(".trace").each(function(t){var e=t[0].trace,n=r.select(this);n.style({opacity:e.opacity}),n.selectAll("path.surface").each(function(t){r.select(this).call(i,t,e)})})}},{"./style_one":1097,d3:163}],1097:[function(t,e,n){"use strict";var r=t("../../components/color"),i=t("./helpers").castOption;e.exports=function(t,e,n){var o=n.marker.line,a=i(o.color,e.pts)||r.defaultLine,s=i(o.width,e.pts)||0;t.style("stroke-width",s).call(r.fill,e.color).call(r.stroke,a)}},{"../../components/color":593,"./helpers":1091}],1098:[function(t,e,n){"use strict";var r=t("../scatter/attributes");e.exports={x:r.x,y:r.y,xy:{valType:"data_array",editType:"calc"},indices:{valType:"data_array",editType:"calc"},xbounds:{valType:"data_array",editType:"calc"},ybounds:{valType:"data_array",editType:"calc"},text:r.text,marker:{color:{valType:"color",arrayOk:!1,editType:"calc"},opacity:{valType:"number",min:0,max:1,dflt:1,arrayOk:!1,editType:"calc"},blend:{valType:"boolean",dflt:null,editType:"calc"},sizemin:{valType:"number",min:.1,max:2,dflt:.5,editType:"calc"},sizemax:{valType:"number",min:.1,dflt:20,editType:"calc"},border:{color:{valType:"color",arrayOk:!1,editType:"calc"},arearatio:{valType:"number",min:0,max:1,dflt:0,editType:"calc"},editType:"calc"},editType:"calc"},transforms:void 0}},{"../scatter/attributes":1112}],1099:[function(t,e,n){"use strict";var r=t("gl-pointcloud2d"),i=t("../../lib/str2rgbarray"),o=t("../../plots/cartesian/autorange").findExtremes,a=t("../scatter/get_trace_color");function s(t,e){this.scene=t,this.uid=e,this.type="pointcloud",this.pickXData=[],this.pickYData=[],this.xData=[],this.yData=[],this.textLabels=[],this.color="rgb(0, 0, 0)",this.name="",this.hoverinfo="all",this.idToIndex=new Int32Array(0),this.bounds=[0,0,0,0],this.pointcloudOptions={positions:new Float32Array(0),idToIndex:this.idToIndex,sizemin:.5,sizemax:12,color:[0,0,0,1],areaRatio:1,borderColor:[0,0,0,1]},this.pointcloud=r(t.glplot,this.pointcloudOptions),this.pointcloud._trace=this}var A=s.prototype;A.handlePick=function(t){var e=this.idToIndex[t.pointId];return{trace:this,dataCoord:t.dataCoord,traceCoord:this.pickXYData?[this.pickXYData[2*e],this.pickXYData[2*e+1]]:[this.pickXData[e],this.pickYData[e]],textLabel:Array.isArray(this.textLabels)?this.textLabels[e]:this.textLabels,color:this.color,name:this.name,pointIndex:e,hoverinfo:this.hoverinfo}},A.update=function(t){this.index=t.index,this.textLabels=t.text,this.name=t.name,this.hoverinfo=t.hoverinfo,this.bounds=[1/0,1/0,-1/0,-1/0],this.updateFast(t),this.color=a(t,{})},A.updateFast=function(t){var e,n,r,a,s,A,l=this.xData=this.pickXData=t.x,c=this.yData=this.pickYData=t.y,u=this.pickXYData=t.xy,g=t.xbounds&&t.ybounds,h=t.indices,d=this.bounds;if(u){if(r=u,e=u.length>>>1,g)d[0]=t.xbounds[0],d[2]=t.xbounds[1],d[1]=t.ybounds[0],d[3]=t.ybounds[1];else for(A=0;A<e;A++)a=r[2*A],s=r[2*A+1],a<d[0]&&(d[0]=a),a>d[2]&&(d[2]=a),s<d[1]&&(d[1]=s),s>d[3]&&(d[3]=s);if(h)n=h;else for(n=new Int32Array(e),A=0;A<e;A++)n[A]=A}else for(e=l.length,r=new Float32Array(2*e),n=new Int32Array(e),A=0;A<e;A++)a=l[A],s=c[A],n[A]=A,r[2*A]=a,r[2*A+1]=s,a<d[0]&&(d[0]=a),a>d[2]&&(d[2]=a),s<d[1]&&(d[1]=s),s>d[3]&&(d[3]=s);this.idToIndex=n,this.pointcloudOptions.idToIndex=n,this.pointcloudOptions.positions=r;var f=i(t.marker.color),p=i(t.marker.border.color),I=t.opacity*t.marker.opacity;f[3]*=I,this.pointcloudOptions.color=f;var C=t.marker.blend;null===C&&(C=l.length<100||c.length<100),this.pointcloudOptions.blend=C,p[3]*=I,this.pointcloudOptions.borderColor=p;var m=t.marker.sizemin,y=Math.max(t.marker.sizemax,t.marker.sizemin);this.pointcloudOptions.sizeMin=m,this.pointcloudOptions.sizeMax=y,this.pointcloudOptions.areaRatio=t.marker.border.arearatio,this.pointcloud.update(this.pointcloudOptions);var E=this.scene.xaxis,v=this.scene.yaxis,B=y/2||.5;t._extremes[E._id]=o(E,[d[0],d[2]],{ppad:B}),t._extremes[v._id]=o(v,[d[1],d[3]],{ppad:B})},A.dispose=function(){this.pointcloud.dispose()},e.exports=function(t,e){var n=new s(t,e.uid);return n.update(e),n}},{"../../lib/str2rgbarray":742,"../../plots/cartesian/autorange":766,"../scatter/get_trace_color":1121,"gl-pointcloud2d":292}],1100:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./attributes");e.exports=function(t,e,n){function o(n,o){return r.coerce(t,e,i,n,o)}o("x"),o("y"),o("xbounds"),o("ybounds"),t.xy&&t.xy instanceof Float32Array&&(e.xy=t.xy),t.indices&&t.indices instanceof Int32Array&&(e.indices=t.indices),o("text"),o("marker.color",n),o("marker.opacity"),o("marker.blend"),o("marker.sizemin"),o("marker.sizemax"),o("marker.border.color",n),o("marker.border.arearatio"),e._length=null}},{"../../lib":719,"./attributes":1098}],1101:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),calc:t("../scatter3d/calc"),plot:t("./convert"),moduleType:"trace",name:"pointcloud",basePlotModule:t("../../plots/gl2d"),categories:["gl","gl2d","showLegend"],meta:{}}},{"../../plots/gl2d":805,"../scatter3d/calc":1139,"./attributes":1098,"./convert":1099,"./defaults":1100}],1102:[function(t,e,n){"use strict";var r=t("../../plots/font_attributes"),i=t("../../plots/attributes"),o=t("../../components/color/attributes"),a=t("../../components/fx/attributes"),s=t("../../plots/domain").attributes,A=t("../../components/fx/hovertemplate_attributes"),l=t("../../components/colorscale/attributes"),c=t("../../plot_api/plot_template").templatedArray,u=t("../../lib/extend").extendFlat,g=t("../../plot_api/edit_types").overrideAll;t("../../constants/docs").FORMAT_LINK,(e.exports=g({hoverinfo:u({},i.hoverinfo,{flags:[],arrayOk:!1}),hoverlabel:a.hoverlabel,domain:s({name:"sankey",trace:!0}),orientation:{valType:"enumerated",values:["v","h"],dflt:"h"},valueformat:{valType:"string",dflt:".3s"},valuesuffix:{valType:"string",dflt:""},arrangement:{valType:"enumerated",values:["snap","perpendicular","freeform","fixed"],dflt:"snap"},textfont:r({}),node:{label:{valType:"data_array",dflt:[]},groups:{valType:"info_array",impliedEdits:{x:[],y:[]},dimensions:2,freeLength:!0,dflt:[],items:{valType:"number",editType:"calc"}},x:{valType:"data_array",dflt:[]},y:{valType:"data_array",dflt:[]},color:{valType:"color",arrayOk:!0},line:{color:{valType:"color",dflt:o.defaultLine,arrayOk:!0},width:{valType:"number",min:0,dflt:.5,arrayOk:!0}},pad:{valType:"number",arrayOk:!1,min:0,dflt:20},thickness:{valType:"number",arrayOk:!1,min:1,dflt:20},hoverinfo:{valType:"enumerated",values:["all","none","skip"],dflt:"all"},hoverlabel:a.hoverlabel,hovertemplate:A({},{keys:["value","label"]})},link:{label:{valType:"data_array",dflt:[]},color:{valType:"color",arrayOk:!0},line:{color:{valType:"color",dflt:o.defaultLine,arrayOk:!0},width:{valType:"number",min:0,dflt:0,arrayOk:!0}},source:{valType:"data_array",dflt:[]},target:{valType:"data_array",dflt:[]},value:{valType:"data_array",dflt:[]},hoverinfo:{valType:"enumerated",values:["all","none","skip"],dflt:"all"},hoverlabel:a.hoverlabel,hovertemplate:A({},{keys:["value","label"]}),colorscales:c("concentrationscales",{editType:"calc",label:{valType:"string",editType:"calc",dflt:""},cmax:{valType:"number",editType:"calc",dflt:1},cmin:{valType:"number",editType:"calc",dflt:0},colorscale:u(l().colorscale,{dflt:[[0,"white"],[1,"black"]]})})}},"calc","nested")).transforms=void 0},{"../../components/color/attributes":592,"../../components/colorscale/attributes":600,"../../components/fx/attributes":623,"../../components/fx/hovertemplate_attributes":631,"../../constants/docs":690,"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plot_api/plot_template":757,"../../plots/attributes":764,"../../plots/domain":792,"../../plots/font_attributes":793}],1103:[function(t,e,n){"use strict";var r=t("../../plot_api/edit_types").overrideAll,i=t("../../plots/get_data").getModuleCalcData,o=t("./plot"),a=t("../../components/fx/layout_attributes"),s=t("../../lib/setcursor"),A=t("../../components/dragelement"),l=t("../../plots/cartesian/select").prepSelect,c=t("../../lib"),u=t("../../registry");function g(t,e){var n=t._fullData[e],r=t._fullLayout,i=r.dragmode,o="pan"===r.dragmode?"move":"crosshair",a=n._bgRect;if("pan"!==i&&"zoom"!==i){s(a,o);var g={_id:"x",c2p:c.identity,_offset:n._sankey.translateX,_length:n._sankey.width},h={_id:"y",c2p:c.identity,_offset:n._sankey.translateY,_length:n._sankey.height},d={gd:t,element:a.node(),plotinfo:{id:e,xaxis:g,yaxis:h,fillRangeItems:c.noop},subplot:e,xaxes:[g],yaxes:[h],doneFnCompleted:function(n){var r,i=t._fullData[e],o=i.node.groups.slice(),a=[];function s(t){for(var e=i._sankey.graph.nodes,n=0;n<e.length;n++)if(e[n].pointNumber===t)return e[n]}for(var A=0;A<n.length;A++){var l=s(n[A].pointNumber);if(l)if(l.group){for(var c=0;c<l.childrenNodes.length;c++)a.push(l.childrenNodes[c].pointNumber);o[l.pointNumber-i.node._count]=!1}else a.push(l.pointNumber)}r=o.filter(Boolean).concat([a]),u.call("_guiRestyle",t,{"node.groups":[r]},e)},prepFn:function(t,e,n){l(t,e,n,d,i)}};A.init(d)}}n.name="sankey",n.baseLayoutAttrOverrides=r({hoverlabel:a.hoverlabel},"plot","nested"),n.plot=function(t){var e=i(t.calcdata,"sankey")[0];o(t,e),n.updateFx(t)},n.clean=function(t,e,n,r){var i=r._has&&r._has("sankey"),o=e._has&&e._has("sankey");i&&!o&&(r._paperdiv.selectAll(".sankey").remove(),r._paperdiv.selectAll(".bgsankey").remove())},n.updateFx=function(t){for(var e=0;e<t._fullData.length;e++)g(t,e)}},{"../../components/dragelement":611,"../../components/fx/layout_attributes":633,"../../lib":719,"../../lib/setcursor":739,"../../plot_api/edit_types":750,"../../plots/cartesian/select":784,"../../plots/get_data":802,"../../registry":847,"./plot":1108}],1104:[function(t,e,n){"use strict";var r=t("strongly-connected-components"),i=t("../../lib"),o=t("../../lib/gup").wrap,a=i.isArrayOrTypedArray,s=i.isIndex,A=t("../../components/colorscale");e.exports=function(t,e){var n=function(t){var e,n=t.node,o=t.link,l=[],c=a(o.color),u={},g={},h=o.colorscales.length;for(e=0;e<h;e++){var d=o.colorscales[e],f=A.extractScale(d,{cLetter:"c"}),p=A.makeColorScaleFunc(f);g[d.label]=p}var I=0;for(e=0;e<o.value.length;e++)o.source[e]>I&&(I=o.source[e]),o.target[e]>I&&(I=o.target[e]);var C,m=I+1;t.node._count=m;var y=t.node.groups,E={};for(e=0;e<y.length;e++){var v=y[e];for(C=0;C<v.length;C++){var B=v[C],b=m+e;E.hasOwnProperty(B)?i.warn("Node "+B+" is already part of a group."):E[B]=b}}var Q={source:[],target:[]};for(e=0;e<o.value.length;e++){var w=o.value[e],x=o.source[e],_=o.target[e];if(w>0&&s(x,m)&&s(_,m)&&(!E.hasOwnProperty(x)||!E.hasOwnProperty(_)||E[x]!==E[_])){E.hasOwnProperty(_)&&(_=E[_]),E.hasOwnProperty(x)&&(x=E[x]),_=+_,u[x=+x]=u[_]=!0;var D="";o.label&&o.label[e]&&(D=o.label[e]);var M=null;D&&g.hasOwnProperty(D)&&(M=g[D]),l.push({pointNumber:e,label:D,color:c?o.color[e]:o.color,concentrationscale:M,source:x,target:_,value:+w}),Q.source.push(x),Q.target.push(_)}}var k=m+y.length,S=a(n.color),L=[];for(e=0;e<k;e++)if(u[e]){var R=n.label[e];L.push({group:e>m-1,childrenNodes:[],pointNumber:e,label:R,color:S?n.color[e]:n.color})}var N=!1;return function(t,e,n){for(var o=i.init2dArray(t,0),a=0;a<Math.min(e.length,n.length);a++)if(i.isIndex(e[a],t)&&i.isIndex(n[a],t)){if(e[a]===n[a])return!0;o[e[a]].push(n[a])}return r(o).components.some(function(t){return t.length>1})}(k,Q.source,Q.target)&&(N=!0),{circular:N,links:l,nodes:L,groups:y,groupLookup:E}}(e);return o({circular:n.circular,_nodes:n.nodes,_links:n.links,_groups:n.groups,_groupLookup:n.groupLookup})}},{"../../components/colorscale":605,"../../lib":719,"../../lib/gup":717,"strongly-connected-components":530}],1105:[function(t,e,n){"use strict";e.exports={nodeTextOffsetHorizontal:4,nodeTextOffsetVertical:3,nodePadAcross:10,sankeyIterations:50,forceIterations:5,forceTicksPerFrame:10,duration:500,ease:"linear",cn:{sankey:"sankey",sankeyLinks:"sankey-links",sankeyLink:"sankey-link",sankeyNodeSet:"sankey-node-set",sankeyNode:"sankey-node",nodeRect:"node-rect",nodeCapture:"node-capture",nodeCentered:"node-entered",nodeLabelGuide:"node-label-guide",nodeLabel:"node-label",nodeLabelTextPath:"node-label-text-path"}}},{}],1106:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./attributes"),o=t("../../components/color"),a=t("tinycolor2"),s=t("../../plots/domain").defaults,A=t("../../components/fx/hoverlabel_defaults"),l=t("../../plot_api/plot_template"),c=t("../../plots/array_container_defaults");function u(t,e){function n(n,o){return r.coerce(t,e,i.link.colorscales,n,o)}n("label"),n("cmin"),n("cmax"),n("colorscale")}e.exports=function(t,e,n,g){function h(n,o){return r.coerce(t,e,i,n,o)}var d=r.extendDeep(g.hoverlabel,t.hoverlabel),f=t.node,p=l.newContainer(e,"node");function I(t,e){return r.coerce(f,p,i.node,t,e)}I("label"),I("groups"),I("x"),I("y"),I("pad"),I("thickness"),I("line.color"),I("line.width"),I("hoverinfo",t.hoverinfo),A(f,p,I,d),I("hovertemplate");var C=g.colorway;I("color",p.label.map(function(t,e){return o.addOpacity(function(t){return C[t%C.length]}(e),.8)}));var m=t.link||{},y=l.newContainer(e,"link");function E(t,e){return r.coerce(m,y,i.link,t,e)}E("label"),E("source"),E("target"),E("value"),E("line.color"),E("line.width"),E("hoverinfo",t.hoverinfo),A(m,y,E,d),E("hovertemplate");var v,B=a(g.paper_bgcolor).getLuminance()<.333?"rgba(255, 255, 255, 0.6)":"rgba(0, 0, 0, 0.2)";E("color",r.repeat(B,y.value.length)),c(m,y,{name:"colorscales",handleItemDefaults:u}),s(e,g,h),h("orientation"),h("valueformat"),h("valuesuffix"),p.x.length&&p.y.length&&(v="freeform"),h("arrangement",v),r.coerceFont(h,"textfont",r.extendFlat({},g.font)),e._length=null}},{"../../components/color":593,"../../components/fx/hoverlabel_defaults":630,"../../lib":719,"../../plot_api/plot_template":757,"../../plots/array_container_defaults":763,"../../plots/domain":792,"./attributes":1102,tinycolor2:537}],1107:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),calc:t("./calc"),plot:t("./plot"),moduleType:"trace",name:"sankey",basePlotModule:t("./base_plot"),selectPoints:t("./select.js"),categories:["noOpacity"],meta:{}}},{"./attributes":1102,"./base_plot":1103,"./calc":1104,"./defaults":1106,"./plot":1108,"./select.js":1110}],1108:[function(t,e,n){"use strict";var r=t("d3"),i=t("./render"),o=t("../../components/fx"),a=t("../../components/color"),s=t("../../lib"),A=t("./constants").cn,l=s._;function c(t){return""!==t}function u(t,e){return t.filter(function(t){return t.key===e.traceId})}function g(t,e){r.select(t).select("path").style("fill-opacity",e),r.select(t).select("rect").style("fill-opacity",e)}function h(t){r.select(t).select("text.name").style("fill","black")}function d(t){return function(e){return-1!==t.node.sourceLinks.indexOf(e.link)||-1!==t.node.targetLinks.indexOf(e.link)}}function f(t){return function(e){return-1!==e.node.sourceLinks.indexOf(t.link)||-1!==e.node.targetLinks.indexOf(t.link)}}function p(t,e,n){e&&n&&u(n,e).selectAll("."+A.sankeyLink).filter(d(e)).call(C.bind(0,e,n,!1))}function I(t,e,n){e&&n&&u(n,e).selectAll("."+A.sankeyLink).filter(d(e)).call(m.bind(0,e,n,!1))}function C(t,e,n,r){var i=r.datum().link.label;r.style("fill-opacity",function(t){if(!t.link.concentrationscale)return.4}),i&&u(e,t).selectAll("."+A.sankeyLink).filter(function(t){return t.link.label===i}).style("fill-opacity",function(t){if(!t.link.concentrationscale)return.4}),n&&u(e,t).selectAll("."+A.sankeyNode).filter(f(t)).call(p)}function m(t,e,n,r){var i=r.datum().link.label;r.style("fill-opacity",function(t){return t.tinyColorAlpha}),i&&u(e,t).selectAll("."+A.sankeyLink).filter(function(t){return t.link.label===i}).style("fill-opacity",function(t){return t.tinyColorAlpha}),n&&u(e,t).selectAll(A.sankeyNode).filter(f(t)).call(I)}function y(t,e){var n=t.hoverlabel||{},r=s.nestedProperty(n,e).get();return!Array.isArray(r)&&r}e.exports=function(t,e){for(var n=t._fullLayout,s=n._paper,u=n._size,d=0;d<t._fullData.length;d++)if(t._fullData[d].type===A.sankey&&!t._fullData[d]._viewInitial){var f=t._fullData[d].node;t._fullData[d]._viewInitial={node:{groups:f.groups.slice(),x:f.x.slice(),y:f.y.slice()}}}var E=l(t,"source:")+" ",v=l(t,"target:")+" ",B=l(t,"concentration:")+" ",b=l(t,"incoming flow count:")+" ",Q=l(t,"outgoing flow count:")+" ";i(t,s,e,{width:u.w,height:u.h,margin:{t:u.t,r:u.r,b:u.b,l:u.l}},{linkEvents:{hover:function(e,n,i){!1!==t._fullLayout.hovermode&&(r.select(e).call(C.bind(0,n,i,!0)),"skip"!==n.link.trace.link.hoverinfo&&(n.link.fullData=n.link.trace,t.emit("plotly_hover",{event:r.event,points:[n.link]})))},follow:function(e,i){if(!1!==t._fullLayout.hovermode){var s=i.link.trace.link;if("none"!==s.hoverinfo&&"skip"!==s.hoverinfo){for(var A=[],l=0,u=0;u<i.flow.links.length;u++){var d=i.flow.links[u];if("closest"!==t._fullLayout.hovermode||i.link.pointNumber===d.pointNumber){i.link.pointNumber===d.pointNumber&&(l=u),d.fullData=d.trace,s=i.link.trace.link;var f=I(d),p={valueLabel:r.format(i.valueFormat)(d.value)+i.valueSuffix};A.push({x:f[0],y:f[1],name:p.valueLabel,text:[d.label||"",E+d.source.label,v+d.target.label,d.concentrationscale?B+r.format("%0.2f")(d.flow.labelConcentration):""].filter(c).join("<br>"),color:y(s,"bgcolor")||a.addOpacity(d.color,1),borderColor:y(s,"bordercolor"),fontFamily:y(s,"font.family"),fontSize:y(s,"font.size"),fontColor:y(s,"font.color"),nameLength:y(s,"namelength"),textAlign:y(s,"align"),idealAlign:r.event.x<f[0]?"right":"left",hovertemplate:s.hovertemplate,hovertemplateLabels:p,eventData:[d]})}}o.loneHover(A,{container:n._hoverlayer.node(),outerContainer:n._paper.node(),gd:t,anchorIndex:l}).each(function(){i.link.concentrationscale||g(this,.65),h(this)})}}function I(t){var e,n;return t.circular?(e=(t.circularPathData.leftInnerExtent+t.circularPathData.rightInnerExtent)/2+i.parent.translateX,n=t.circularPathData.verticalFullExtent+i.parent.translateY):(e=(t.source.x1+t.target.x0)/2+i.parent.translateX,n=(t.y0+t.y1)/2+i.parent.translateY),[e,n]}},unhover:function(e,i,a){!1!==t._fullLayout.hovermode&&(r.select(e).call(m.bind(0,i,a,!0)),"skip"!==i.link.trace.link.hoverinfo&&(i.link.fullData=i.link.trace,t.emit("plotly_unhover",{event:r.event,points:[i.link]})),o.loneUnhover(n._hoverlayer.node()))},select:function(e,n){var i=n.link;i.originalEvent=r.event,t._hoverdata=[i],o.click(t,{target:!0})}},nodeEvents:{hover:function(e,n,i){!1!==t._fullLayout.hovermode&&(r.select(e).call(p,n,i),"skip"!==n.node.trace.node.hoverinfo&&(n.node.fullData=n.node.trace,t.emit("plotly_hover",{event:r.event,points:[n.node]})))},follow:function(e,i){if(!1!==t._fullLayout.hovermode){var a=i.node.trace.node;if("none"!==a.hoverinfo&&"skip"!==a.hoverinfo){var s=r.select(e).select("."+A.nodeRect),l=t._fullLayout._paperdiv.node().getBoundingClientRect(),u=s.node().getBoundingClientRect(),d=u.left-2-l.left,f=u.right+2-l.left,p=u.top+u.height/4-l.top,I={valueLabel:r.format(i.valueFormat)(i.node.value)+i.valueSuffix};i.node.fullData=i.node.trace;var C=o.loneHover({x0:d,x1:f,y:p,name:r.format(i.valueFormat)(i.node.value)+i.valueSuffix,text:[i.node.label,b+i.node.targetLinks.length,Q+i.node.sourceLinks.length].filter(c).join("<br>"),color:y(a,"bgcolor")||i.tinyColorHue,borderColor:y(a,"bordercolor"),fontFamily:y(a,"font.family"),fontSize:y(a,"font.size"),fontColor:y(a,"font.color"),nameLength:y(a,"namelength"),textAlign:y(a,"align"),idealAlign:"left",hovertemplate:a.hovertemplate,hovertemplateLabels:I,eventData:[i.node]},{container:n._hoverlayer.node(),outerContainer:n._paper.node(),gd:t});g(C,.85),h(C)}}},unhover:function(e,i,a){!1!==t._fullLayout.hovermode&&(r.select(e).call(I,i,a),"skip"!==i.node.trace.node.hoverinfo&&(i.node.fullData=i.node.trace,t.emit("plotly_unhover",{event:r.event,points:[i.node]})),o.loneUnhover(n._hoverlayer.node()))},select:function(e,n,i){var a=n.node;a.originalEvent=r.event,t._hoverdata=[a],r.select(e).call(I,n,i),o.click(t,{target:!0})}}})}},{"../../components/color":593,"../../components/fx":632,"../../lib":719,"./constants":1105,"./render":1109,d3:163}],1109:[function(t,e,n){"use strict";var r=t("./constants"),i=t("d3"),o=t("tinycolor2"),a=t("../../components/color"),s=t("../../components/drawing"),A=t("@plotly/d3-sankey"),l=t("@plotly/d3-sankey-circular"),c=t("d3-force"),u=t("../../lib"),g=t("../../lib/gup"),h=g.keyFun,d=g.repeat,f=g.unwrap,p=t("d3-interpolate").interpolateNumber,I=t("../../registry");function C(){return function(t){if(t.link.circular)return e=t.link,n=e.width/2,r=e.circularPathData,"top"===e.circularLinkType?"M "+r.targetX+" "+(r.targetY+n)+" L"+r.rightInnerExtent+" "+(r.targetY+n)+"A"+(r.rightLargeArcRadius+n)+" "+(r.rightSmallArcRadius+n)+" 0 0 1 "+(r.rightFullExtent-n)+" "+(r.targetY-r.rightSmallArcRadius)+"L"+(r.rightFullExtent-n)+" "+r.verticalRightInnerExtent+"A"+(r.rightLargeArcRadius+n)+" "+(r.rightLargeArcRadius+n)+" 0 0 1 "+r.rightInnerExtent+" "+(r.verticalFullExtent-n)+"L"+r.leftInnerExtent+" "+(r.verticalFullExtent-n)+"A"+(r.leftLargeArcRadius+n)+" "+(r.leftLargeArcRadius+n)+" 0 0 1 "+(r.leftFullExtent+n)+" "+r.verticalLeftInnerExtent+"L"+(r.leftFullExtent+n)+" "+(r.sourceY-r.leftSmallArcRadius)+"A"+(r.leftLargeArcRadius+n)+" "+(r.leftSmallArcRadius+n)+" 0 0 1 "+r.leftInnerExtent+" "+(r.sourceY+n)+"L"+r.sourceX+" "+(r.sourceY+n)+"L"+r.sourceX+" "+(r.sourceY-n)+"L"+r.leftInnerExtent+" "+(r.sourceY-n)+"A"+(r.leftLargeArcRadius-n)+" "+(r.leftSmallArcRadius-n)+" 0 0 0 "+(r.leftFullExtent-n)+" "+(r.sourceY-r.leftSmallArcRadius)+"L"+(r.leftFullExtent-n)+" "+r.verticalLeftInnerExtent+"A"+(r.leftLargeArcRadius-n)+" "+(r.leftLargeArcRadius-n)+" 0 0 0 "+r.leftInnerExtent+" "+(r.verticalFullExtent+n)+"L"+r.rightInnerExtent+" "+(r.verticalFullExtent+n)+"A"+(r.rightLargeArcRadius-n)+" "+(r.rightLargeArcRadius-n)+" 0 0 0 "+(r.rightFullExtent+n)+" "+r.verticalRightInnerExtent+"L"+(r.rightFullExtent+n)+" "+(r.targetY-r.rightSmallArcRadius)+"A"+(r.rightLargeArcRadius-n)+" "+(r.rightSmallArcRadius-n)+" 0 0 0 "+r.rightInnerExtent+" "+(r.targetY-n)+"L"+r.targetX+" "+(r.targetY-n)+"Z":"M "+r.targetX+" "+(r.targetY-n)+" L"+r.rightInnerExtent+" "+(r.targetY-n)+"A"+(r.rightLargeArcRadius+n)+" "+(r.rightSmallArcRadius+n)+" 0 0 0 "+(r.rightFullExtent-n)+" "+(r.targetY+r.rightSmallArcRadius)+"L"+(r.rightFullExtent-n)+" "+r.verticalRightInnerExtent+"A"+(r.rightLargeArcRadius+n)+" "+(r.rightLargeArcRadius+n)+" 0 0 0 "+r.rightInnerExtent+" "+(r.verticalFullExtent+n)+"L"+r.leftInnerExtent+" "+(r.verticalFullExtent+n)+"A"+(r.leftLargeArcRadius+n)+" "+(r.leftLargeArcRadius+n)+" 0 0 0 "+(r.leftFullExtent+n)+" "+r.verticalLeftInnerExtent+"L"+(r.leftFullExtent+n)+" "+(r.sourceY+r.leftSmallArcRadius)+"A"+(r.leftLargeArcRadius+n)+" "+(r.leftSmallArcRadius+n)+" 0 0 0 "+r.leftInnerExtent+" "+(r.sourceY-n)+"L"+r.sourceX+" "+(r.sourceY-n)+"L"+r.sourceX+" "+(r.sourceY+n)+"L"+r.leftInnerExtent+" "+(r.sourceY+n)+"A"+(r.leftLargeArcRadius-n)+" "+(r.leftSmallArcRadius-n)+" 0 0 1 "+(r.leftFullExtent-n)+" "+(r.sourceY+r.leftSmallArcRadius)+"L"+(r.leftFullExtent-n)+" "+r.verticalLeftInnerExtent+"A"+(r.leftLargeArcRadius-n)+" "+(r.leftLargeArcRadius-n)+" 0 0 1 "+r.leftInnerExtent+" "+(r.verticalFullExtent-n)+"L"+r.rightInnerExtent+" "+(r.verticalFullExtent-n)+"A"+(r.rightLargeArcRadius-n)+" "+(r.rightLargeArcRadius-n)+" 0 0 1 "+(r.rightFullExtent+n)+" "+r.verticalRightInnerExtent+"L"+(r.rightFullExtent+n)+" "+(r.targetY+r.rightSmallArcRadius)+"A"+(r.rightLargeArcRadius-n)+" "+(r.rightSmallArcRadius-n)+" 0 0 1 "+r.rightInnerExtent+" "+(r.targetY+n)+"L"+r.targetX+" "+(r.targetY+n)+"Z";var e,n,r,i=t.link.source.x1,o=t.link.target.x0,a=p(i,o),s=a(.5),A=a(.5),l=t.link.y0-t.link.width/2,c=t.link.y0+t.link.width/2,u=t.link.y1-t.link.width/2,g=t.link.y1+t.link.width/2;return"M"+i+","+l+"C"+s+","+l+" "+A+","+u+" "+o+","+u+"L"+o+","+g+"C"+A+","+g+" "+s+","+c+" "+i+","+c+"Z"}}function m(t){t.attr("transform",function(t){return"translate("+t.node.x0.toFixed(3)+", "+t.node.y0.toFixed(3)+")"})}function y(t){t.call(m)}function E(t,e){t.call(y),e.attr("d",C())}function v(t){t.attr("width",function(t){return t.node.x1-t.node.x0}).attr("height",function(t){return t.visibleHeight})}function B(t){return t.link.width>1||t.linkLineWidth>0}function b(t){return"translate("+t.translateX+","+t.translateY+")"+(t.horizontal?"matrix(1 0 0 1 0 0)":"matrix(0 1 1 0 0 0)")}function Q(t){return"translate("+(t.horizontal?0:t.labelY)+" "+(t.horizontal?t.labelY:0)+")"}function w(t){return i.svg.line()([[t.horizontal?t.left?-t.sizeAcross:t.visibleWidth+r.nodeTextOffsetHorizontal:r.nodeTextOffsetHorizontal,0],[t.horizontal?t.left?-r.nodeTextOffsetHorizontal:t.sizeAcross:t.visibleHeight-r.nodeTextOffsetHorizontal,0]])}function x(t){return t.horizontal?"matrix(1 0 0 1 0 0)":"matrix(0 1 1 0 0 0)"}function _(t){return t.horizontal?"scale(1 1)":"scale(-1 1)"}function D(t){return t.darkBackground&&!t.horizontal?"rgb(255,255,255)":"rgb(0,0,0)"}function M(t){return t.horizontal&&t.left?"100%":"0%"}function k(t,e,n){t.on(".basic",null).on("mouseover.basic",function(t){t.interactionState.dragInProgress||t.partOfGroup||(n.hover(this,t,e),t.interactionState.hovered=[this,t])}).on("mousemove.basic",function(t){t.interactionState.dragInProgress||t.partOfGroup||(n.follow(this,t),t.interactionState.hovered=[this,t])}).on("mouseout.basic",function(t){t.interactionState.dragInProgress||t.partOfGroup||(n.unhover(this,t,e),t.interactionState.hovered=!1)}).on("click.basic",function(t){t.interactionState.hovered&&(n.unhover(this,t,e),t.interactionState.hovered=!1),t.interactionState.dragInProgress||t.partOfGroup||n.select(this,t,e)})}function S(t,e,n,o){var a=i.behavior.drag().origin(function(t){return{x:t.node.x0+t.visibleWidth/2,y:t.node.y0+t.visibleHeight/2}}).on("dragstart",function(i){if("fixed"!==i.arrangement&&(u.ensureSingle(o._fullLayout._infolayer,"g","dragcover",function(t){o._fullLayout._dragCover=t}),u.raiseToTop(this),i.interactionState.dragInProgress=i.node,R(i.node),i.interactionState.hovered&&(n.nodeEvents.unhover.apply(0,i.interactionState.hovered),i.interactionState.hovered=!1),"snap"===i.arrangement)){var a=i.traceId+"|"+i.key;i.forceLayouts[a]?i.forceLayouts[a].alpha(1):function(t,e,n,i){!function(t){for(var e=0;e<t.length;e++)t[e].y=(t[e].y0+t[e].y1)/2,t[e].x=(t[e].x0+t[e].x1)/2}(n.graph.nodes);var o=n.graph.nodes.filter(function(t){return t.originalX===n.node.originalX}).filter(function(t){return!t.partOfGroup});n.forceLayouts[e]=c.forceSimulation(o).alphaDecay(0).force("collide",c.forceCollide().radius(function(t){return t.dy/2+n.nodePad/2}).strength(1).iterations(r.forceIterations)).force("constrain",function(t,e,n,i){return function(){for(var t=0,o=0;o<n.length;o++){var a=n[o];a===i.interactionState.dragInProgress?(a.x=a.lastDraggedX,a.y=a.lastDraggedY):(a.vx=(a.originalX-a.x)/r.forceTicksPerFrame,a.y=Math.min(i.size-a.dy/2,Math.max(a.dy/2,a.y))),t=Math.max(t,Math.abs(a.vx),Math.abs(a.vy))}!i.interactionState.dragInProgress&&t<.1&&i.forceLayouts[e].alpha()>0&&i.forceLayouts[e].alpha(0)}}(0,e,o,n)).stop()}(0,a,i),function(t,e,n,i,o){window.requestAnimationFrame(function a(){var s;for(s=0;s<r.forceTicksPerFrame;s++)n.forceLayouts[i].tick();var A=n.graph.nodes;if(function(t){for(var e=0;e<t.length;e++)t[e].y0=t[e].y-t[e].dy/2,t[e].y1=t[e].y0+t[e].dy,t[e].x0=t[e].x-t[e].dx/2,t[e].x1=t[e].x0+t[e].dx}(A),n.sankey.update(n.graph),E(t.filter(N(n)),e),n.forceLayouts[i].alpha()>0)window.requestAnimationFrame(a);else{var l=n.node.originalX;n.node.x0=l-n.visibleWidth/2,n.node.x1=l+n.visibleWidth/2,L(n,o)}})}(t,e,i,a,o)}}).on("drag",function(n){if("fixed"!==n.arrangement){var r=i.event.x,o=i.event.y;"snap"===n.arrangement?(n.node.x0=r-n.visibleWidth/2,n.node.x1=r+n.visibleWidth/2,n.node.y0=o-n.visibleHeight/2,n.node.y1=o+n.visibleHeight/2):("freeform"===n.arrangement&&(n.node.x0=r-n.visibleWidth/2,n.node.x1=r+n.visibleWidth/2),o=Math.max(0,Math.min(n.size-n.visibleHeight/2,o)),n.node.y0=o-n.visibleHeight/2,n.node.y1=o+n.visibleHeight/2),R(n.node),"snap"!==n.arrangement&&(n.sankey.update(n.graph),E(t.filter(N(n)),e))}}).on("dragend",function(t){if("fixed"!==t.arrangement){t.interactionState.dragInProgress=!1;for(var e=0;e<t.node.childrenNodes.length;e++)t.node.childrenNodes[e].x=t.node.x,t.node.childrenNodes[e].y=t.node.y;"snap"!==t.arrangement&&L(t,o)}});t.on(".drag",null).call(a)}function L(t,e){for(var n=[],r=[],i=0;i<t.graph.nodes.length;i++){var o=(t.graph.nodes[i].x0+t.graph.nodes[i].x1)/2,a=(t.graph.nodes[i].y0+t.graph.nodes[i].y1)/2;n.push(o/t.figure.width),r.push(a/t.figure.height)}I.call("_guiRestyle",e,{"node.x":[n],"node.y":[r]},t.trace.index).then(function(){e._fullLayout._dragCover&&e._fullLayout._dragCover.remove()})}function R(t){t.lastDraggedX=t.x0+t.dx/2,t.lastDraggedY=t.y0+t.dy/2}function N(t){return function(e){return e.node.originalX===t.node.originalX}}e.exports=function(t,e,n,c,g){var p=!1;u.ensureSingle(t._fullLayout._infolayer,"g","first-render",function(){p=!0});var I=t._fullLayout._dragCover,y=n.filter(function(t){return f(t).trace.visible}).map(function(t,e,n){var i,a=f(e),s=a.trace,c=s.domain,g="h"===s.orientation,h=s.node.pad,d=s.node.thickness,p=t.width*(c.x[1]-c.x[0]),I=t.height*(c.y[1]-c.y[0]),C=a._nodes,m=a._links,y=a.circular;(i=y?l.sankeyCircular().circularLinkGap(0):A.sankey()).iterations(r.sankeyIterations).size(g?[p,I]:[I,p]).nodeWidth(d).nodePadding(h).nodeId(function(t){return t.pointNumber}).nodes(C).links(m);var E,v,B,b=i();for(var Q in i.nodePadding()<h&&u.warn("node.pad was reduced to ",i.nodePadding()," to fit within the figure."),a._groupLookup){var w,x=parseInt(a._groupLookup[Q]);for(E=0;E<b.nodes.length;E++)if(b.nodes[E].pointNumber===x){w=b.nodes[E];break}if(w){var _={pointNumber:parseInt(Q),x0:w.x0,x1:w.x1,y0:w.y0,y1:w.y1,partOfGroup:!0,sourceLinks:[],targetLinks:[]};b.nodes.unshift(_),w.childrenNodes.unshift(_)}}if(function(){for(E=0;E<b.nodes.length;E++){var t,e,n=b.nodes[E],r={};for(v=0;v<n.targetLinks.length;v++)t=(e=n.targetLinks[v]).source.pointNumber+":"+e.target.pointNumber,r.hasOwnProperty(t)||(r[t]=[]),r[t].push(e);var i=Object.keys(r);for(v=0;v<i.length;v++){var a=r[t=i[v]],s=0,A={};for(B=0;B<a.length;B++)A[(e=a[B]).label]||(A[e.label]=0),A[e.label]+=e.value,s+=e.value;for(B=0;B<a.length;B++)(e=a[B]).flow={value:s,labelConcentration:A[e.label]/s,concentration:e.value/s,links:a},e.concentrationscale&&(e.color=o(e.concentrationscale(e.flow.labelConcentration)))}var l=0;for(v=0;v<n.sourceLinks.length;v++)l+=n.sourceLinks[v].value;for(v=0;v<n.sourceLinks.length;v++)(e=n.sourceLinks[v]).concentrationOut=e.value/l;var c=0;for(v=0;v<n.targetLinks.length;v++)c+=n.targetLinks[v].value;for(v=0;v<n.targetLinks.length;v++)(e=n.targetLinks[v]).concenrationIn=e.value/c}}(),s.node.x.length&&s.node.y.length){for(E=0;E<Math.min(s.node.x.length,s.node.y.length,b.nodes.length);E++)if(s.node.x[E]&&s.node.y[E]){var D=[s.node.x[E]*p,s.node.y[E]*I];b.nodes[E].x0=D[0]-d/2,b.nodes[E].x1=D[0]+d/2;var M=b.nodes[E].y1-b.nodes[E].y0;b.nodes[E].y0=D[1]-M/2,b.nodes[E].y1=D[1]+M/2}"snap"===s.arrangement&&function(t){t.forEach(function(t){var e,n,r,i=0,o=t.length;for(t.sort(function(t,e){return t.y0-e.y0}),r=0;r<o;++r)(e=t[r]).y0>=i||(n=i-e.y0)>1e-6&&(e.y0+=n,e.y1+=n),i=e.y1+h})}(function(t){var e,n,r=t.map(function(t,e){return{x0:t.x0,index:e}}).sort(function(t,e){return t.x0-e.x0}),i=[],o=-1,a=-1/0;for(E=0;E<r.length;E++){var s=t[r[E].index];s.x0>a+d&&(o+=1,e=s.x0),a=s.x0,i[o]||(i[o]=[]),i[o].push(s),n=e-s.x0,s.x0+=n,s.x1+=n}return i}(C=b.nodes)),i.update(b)}return{circular:y,key:n,trace:s,guid:u.randstr(),horizontal:g,width:p,height:I,nodePad:s.node.pad,nodeLineColor:s.node.line.color,nodeLineWidth:s.node.line.width,linkLineColor:s.link.line.color,linkLineWidth:s.link.line.width,valueFormat:s.valueformat,valueSuffix:s.valuesuffix,textFont:s.textfont,translateX:c.x[0]*t.width+t.margin.l,translateY:t.height-c.y[1]*t.height+t.margin.t,dragParallel:g?I:p,dragPerpendicular:g?p:I,arrangement:s.arrangement,sankey:i,graph:b,forceLayouts:{},interactionState:{dragInProgress:!1,hovered:!1}}}.bind(null,c)),E=e.selectAll("."+r.cn.sankey).data(y,h);E.exit().remove(),E.enter().append("g").classed(r.cn.sankey,!0).style("box-sizing","content-box").style("position","absolute").style("left",0).style("shape-rendering","geometricPrecision").style("pointer-events","auto").attr("transform",b),E.each(function(e,n){t._fullData[n]._sankey=e;var r="bgsankey-"+e.trace.uid+"-"+n;u.ensureSingle(t._fullLayout._draggers,"rect",r),t._fullData[n]._bgRect=i.select("."+r),t._fullData[n]._bgRect.style("pointer-events","all").attr("width",e.width).attr("height",e.height).attr("x",e.translateX).attr("y",e.translateY).classed("bgsankey",!0).style({fill:"transparent","stroke-width":0})}),E.transition().ease(r.ease).duration(r.duration).attr("transform",b);var L=E.selectAll("."+r.cn.sankeyLinks).data(d,h);L.enter().append("g").classed(r.cn.sankeyLinks,!0).style("fill","none");var R=L.selectAll("."+r.cn.sankeyLink).data(function(t){return t.graph.links.filter(function(t){return t.value}).map(function(t,e,n){var r=o(e.color),i=e.source.label+"|"+e.target.label+"__"+n;return e.trace=t.trace,e.curveNumber=t.trace.index,{circular:t.circular,key:i,traceId:t.key,pointNumber:e.pointNumber,link:e,tinyColorHue:a.tinyRGB(r),tinyColorAlpha:r.getAlpha(),linkPath:C,linkLineColor:t.linkLineColor,linkLineWidth:t.linkLineWidth,valueFormat:t.valueFormat,valueSuffix:t.valueSuffix,sankey:t.sankey,parent:t,interactionState:t.interactionState,flow:e.flow}}.bind(null,t))},h);R.enter().append("path").classed(r.cn.sankeyLink,!0).call(k,E,g.linkEvents),R.style("stroke",function(t){return B(t)?a.tinyRGB(o(t.linkLineColor)):t.tinyColorHue}).style("stroke-opacity",function(t){return B(t)?a.opacity(t.linkLineColor):t.tinyColorAlpha}).style("fill",function(t){return t.tinyColorHue}).style("fill-opacity",function(t){return t.tinyColorAlpha}).style("stroke-width",function(t){return B(t)?t.linkLineWidth:1}).attr("d",C()),R.style("opacity",function(){return t._context.staticPlot||p||I?1:0}).transition().ease(r.ease).duration(r.duration).style("opacity",1),R.exit().transition().ease(r.ease).duration(r.duration).style("opacity",0).remove();var N=E.selectAll("."+r.cn.sankeyNodeSet).data(d,h);N.enter().append("g").classed(r.cn.sankeyNodeSet,!0),N.style("cursor",function(t){switch(t.arrangement){case"fixed":return"default";case"perpendicular":return"ns-resize";default:return"move"}});var F=N.selectAll("."+r.cn.sankeyNode).data(function(t){var e=t.graph.nodes;return function(t){var e,n=[];for(e=0;e<t.length;e++)t[e].originalX=(t[e].x0+t[e].x1)/2,t[e].originalY=(t[e].y0+t[e].y1)/2,-1===n.indexOf(t[e].originalX)&&n.push(t[e].originalX);for(n.sort(function(t,e){return t-e}),e=0;e<t.length;e++)t[e].originalLayerIndex=n.indexOf(t[e].originalX),t[e].originalLayer=t[e].originalLayerIndex/(n.length-1)}(e),e.map(function(t,e){var n=o(e.color),i=r.nodePadAcross,s=t.nodePad/2;e.dx=e.x1-e.x0,e.dy=e.y1-e.y0;var A=e.dx,l=Math.max(.5,e.dy),c="node_"+e.pointNumber;return e.group&&(c=u.randstr()),e.trace=t.trace,e.curveNumber=t.trace.index,{index:e.pointNumber,key:c,partOfGroup:e.partOfGroup||!1,group:e.group,traceId:t.key,trace:t.trace,node:e,nodePad:t.nodePad,nodeLineColor:t.nodeLineColor,nodeLineWidth:t.nodeLineWidth,textFont:t.textFont,size:t.horizontal?t.height:t.width,visibleWidth:Math.ceil(A),visibleHeight:l,zoneX:-i,zoneY:-s,zoneWidth:A+2*i,zoneHeight:l+2*s,labelY:t.horizontal?e.dy/2+1:e.dx/2+1,left:1===e.originalLayer,sizeAcross:t.width,forceLayouts:t.forceLayouts,horizontal:t.horizontal,darkBackground:n.getBrightness()<=128,tinyColorHue:a.tinyRGB(n),tinyColorAlpha:n.getAlpha(),valueFormat:t.valueFormat,valueSuffix:t.valueSuffix,sankey:t.sankey,graph:t.graph,arrangement:t.arrangement,uniqueNodeLabelPathId:[t.guid,t.key,c].join("_"),interactionState:t.interactionState,figure:t}}.bind(null,t))},h);F.enter().append("g").classed(r.cn.sankeyNode,!0).call(m).style("opacity",function(e){return!t._context.staticPlot&&!p||e.partOfGroup?0:1}),F.call(k,E,g.nodeEvents).call(S,R,g,t),F.transition().ease(r.ease).duration(r.duration).call(m).style("opacity",function(t){return t.partOfGroup?0:1}),F.exit().transition().ease(r.ease).duration(r.duration).style("opacity",0).remove();var T=F.selectAll("."+r.cn.nodeRect).data(d);T.enter().append("rect").classed(r.cn.nodeRect,!0).call(v),T.style("stroke-width",function(t){return t.nodeLineWidth}).style("stroke",function(t){return a.tinyRGB(o(t.nodeLineColor))}).style("stroke-opacity",function(t){return a.opacity(t.nodeLineColor)}).style("fill",function(t){return t.tinyColorHue}).style("fill-opacity",function(t){return t.tinyColorAlpha}),T.transition().ease(r.ease).duration(r.duration).call(v);var U=F.selectAll("."+r.cn.nodeCapture).data(d);U.enter().append("rect").classed(r.cn.nodeCapture,!0).style("fill-opacity",0),U.attr("x",function(t){return t.zoneX}).attr("y",function(t){return t.zoneY}).attr("width",function(t){return t.zoneWidth}).attr("height",function(t){return t.zoneHeight});var O=F.selectAll("."+r.cn.nodeCentered).data(d);O.enter().append("g").classed(r.cn.nodeCentered,!0).attr("transform",Q),O.transition().ease(r.ease).duration(r.duration).attr("transform",Q);var G=O.selectAll("."+r.cn.nodeLabelGuide).data(d);G.enter().append("path").classed(r.cn.nodeLabelGuide,!0).attr("id",function(t){return t.uniqueNodeLabelPathId}).attr("d",w).attr("transform",x),G.transition().ease(r.ease).duration(r.duration).attr("d",w).attr("transform",x);var H=O.selectAll("."+r.cn.nodeLabel).data(d);H.enter().append("text").classed(r.cn.nodeLabel,!0).attr("transform",_).style("user-select","none").style("cursor","default").style("fill","black"),H.style("text-shadow",function(t){return t.horizontal?"-1px 1px 1px #fff, 1px 1px 1px #fff, 1px -1px 1px #fff, -1px -1px 1px #fff":"none"}).each(function(t){s.font(H,t.textFont)}),H.transition().ease(r.ease).duration(r.duration).attr("transform",_);var j=H.selectAll("."+r.cn.nodeLabelTextPath).data(d);j.enter().append("textPath").classed(r.cn.nodeLabelTextPath,!0).attr("alignment-baseline","middle").attr("xlink:href",function(t){return"#"+t.uniqueNodeLabelPathId}).attr("startOffset",M).style("fill",D),j.text(function(t){return t.horizontal||t.node.dy>5?t.node.label:""}).attr("text-anchor",function(t){return t.horizontal&&t.left?"end":"start"}),j.transition().ease(r.ease).duration(r.duration).attr("startOffset",M).style("fill",D)}},{"../../components/color":593,"../../components/drawing":614,"../../lib":719,"../../lib/gup":717,"../../registry":847,"./constants":1105,"@plotly/d3-sankey":54,"@plotly/d3-sankey-circular":53,d3:163,"d3-force":156,"d3-interpolate":158,tinycolor2:537}],1110:[function(t,e,n){"use strict";e.exports=function(t,e){for(var n=[],r=t.cd[0].trace,i=r._sankey.graph.nodes,o=0;o<i.length;o++){var a=i[o];if(!a.partOfGroup){var s=[(a.x0+a.x1)/2,(a.y0+a.y1)/2];"v"===r.orientation&&s.reverse(),e&&e.contains(s,!1,o,t)&&n.push({pointNumber:a.pointNumber})}}return n}},{}],1111:[function(t,e,n){"use strict";var r=t("../../lib");e.exports=function(t,e){for(var n=0;n<t.length;n++)t[n].i=n;r.mergeArray(e.text,t,"tx"),r.mergeArray(e.hovertext,t,"htx"),r.mergeArray(e.customdata,t,"data"),r.mergeArray(e.textposition,t,"tp"),e.textfont&&(r.mergeArrayCastPositive(e.textfont.size,t,"ts"),r.mergeArray(e.textfont.color,t,"tc"),r.mergeArray(e.textfont.family,t,"tf"));var i=e.marker;if(i){r.mergeArrayCastPositive(i.size,t,"ms"),r.mergeArrayCastPositive(i.opacity,t,"mo"),r.mergeArray(i.symbol,t,"mx"),r.mergeArray(i.color,t,"mc");var o=i.line;i.line&&(r.mergeArray(o.color,t,"mlc"),r.mergeArrayCastPositive(o.width,t,"mlw"));var a=i.gradient;a&&"none"!==a.type&&(r.mergeArray(a.type,t,"mgt"),r.mergeArray(a.color,t,"mgc"))}}},{"../../lib":719}],1112:[function(t,e,n){"use strict";var r=t("../../components/fx/hovertemplate_attributes"),i=t("../../components/colorscale/attributes"),o=t("../../plots/font_attributes"),a=t("../../components/drawing/attributes").dash,s=t("../../components/drawing"),A=t("./constants"),l=t("../../lib/extend").extendFlat;e.exports={x:{valType:"data_array",editType:"calc+clearAxisTypes",anim:!0},x0:{valType:"any",dflt:0,editType:"calc+clearAxisTypes",anim:!0},dx:{valType:"number",dflt:1,editType:"calc",anim:!0},y:{valType:"data_array",editType:"calc+clearAxisTypes",anim:!0},y0:{valType:"any",dflt:0,editType:"calc+clearAxisTypes",anim:!0},dy:{valType:"number",dflt:1,editType:"calc",anim:!0},stackgroup:{valType:"string",dflt:"",editType:"calc"},orientation:{valType:"enumerated",values:["v","h"],editType:"calc"},groupnorm:{valType:"enumerated",values:["","fraction","percent"],dflt:"",editType:"calc"},stackgaps:{valType:"enumerated",values:["infer zero","interpolate"],dflt:"infer zero",editType:"calc"},text:{valType:"string",dflt:"",arrayOk:!0,editType:"calc"},hovertext:{valType:"string",dflt:"",arrayOk:!0,editType:"style"},mode:{valType:"flaglist",flags:["lines","markers","text"],extras:["none"],editType:"calc"},hoveron:{valType:"flaglist",flags:["points","fills"],editType:"style"},hovertemplate:r({},{keys:A.eventDataKeys}),line:{color:{valType:"color",editType:"style",anim:!0},width:{valType:"number",min:0,dflt:2,editType:"style",anim:!0},shape:{valType:"enumerated",values:["linear","spline","hv","vh","hvh","vhv"],dflt:"linear",editType:"plot"},smoothing:{valType:"number",min:0,max:1.3,dflt:1,editType:"plot"},dash:l({},a,{editType:"style"}),simplify:{valType:"boolean",dflt:!0,editType:"plot"},editType:"plot"},connectgaps:{valType:"boolean",dflt:!1,editType:"calc"},cliponaxis:{valType:"boolean",dflt:!0,editType:"plot"},fill:{valType:"enumerated",values:["none","tozeroy","tozerox","tonexty","tonextx","toself","tonext"],editType:"calc"},fillcolor:{valType:"color",editType:"style",anim:!0},marker:l({symbol:{valType:"enumerated",values:s.symbolList,dflt:"circle",arrayOk:!0,editType:"style"},opacity:{valType:"number",min:0,max:1,arrayOk:!0,editType:"style",anim:!0},size:{valType:"number",min:0,dflt:6,arrayOk:!0,editType:"calc",anim:!0},maxdisplayed:{valType:"number",min:0,dflt:0,editType:"plot"},sizeref:{valType:"number",dflt:1,editType:"calc"},sizemin:{valType:"number",min:0,dflt:0,editType:"calc"},sizemode:{valType:"enumerated",values:["diameter","area"],dflt:"diameter",editType:"calc"},line:l({width:{valType:"number",min:0,arrayOk:!0,editType:"style",anim:!0},editType:"calc"},i("marker.line",{anim:!0})),gradient:{type:{valType:"enumerated",values:["radial","horizontal","vertical","none"],arrayOk:!0,dflt:"none",editType:"calc"},color:{valType:"color",arrayOk:!0,editType:"calc"},editType:"calc"},editType:"calc"},i("marker",{anim:!0})),selected:{marker:{opacity:{valType:"number",min:0,max:1,editType:"style"},color:{valType:"color",editType:"style"},size:{valType:"number",min:0,editType:"style"},editType:"style"},textfont:{color:{valType:"color",editType:"style"},editType:"style"},editType:"style"},unselected:{marker:{opacity:{valType:"number",min:0,max:1,editType:"style"},color:{valType:"color",editType:"style"},size:{valType:"number",min:0,editType:"style"},editType:"style"},textfont:{color:{valType:"color",editType:"style"},editType:"style"},editType:"style"},textposition:{valType:"enumerated",values:["top left","top center","top right","middle left","middle center","middle right","bottom left","bottom center","bottom right"],dflt:"middle center",arrayOk:!0,editType:"calc"},textfont:o({editType:"calc",colorEditType:"style",arrayOk:!0}),r:{valType:"data_array",editType:"calc"},t:{valType:"data_array",editType:"calc"}}},{"../../components/colorscale/attributes":600,"../../components/drawing":614,"../../components/drawing/attributes":613,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/font_attributes":793,"./constants":1116}],1113:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib"),o=t("../../plots/cartesian/axes"),a=t("../../constants/numerical").BADNUM,s=t("./subtypes"),A=t("./colorscale_calc"),l=t("./arrays_to_calcdata"),c=t("./calc_selection");function u(t,e,n,r,i,a,A){var l=e._length,c=t._fullLayout,u=n._id,g=r._id,h=c._firstScatter[d(e)]===e.uid,p=(f(e,c,n,r)||{}).orientation,I=e.fill;n._minDtick=0,r._minDtick=0;var C={padded:!0},m={padded:!0};A&&(C.ppad=m.ppad=A);var y=l<2||i[0]!==i[l-1]||a[0]!==a[l-1];y&&("tozerox"===I||"tonextx"===I&&(h||"h"===p))?C.tozero=!0:(e.error_y||{}).visible||"tonexty"!==I&&"tozeroy"!==I&&(s.hasMarkers(e)||s.hasText(e))||(C.padded=!1,C.ppad=0),y&&("tozeroy"===I||"tonexty"===I&&(h||"v"===p))?m.tozero=!0:"tonextx"!==I&&"tozerox"!==I||(m.padded=!1),u&&(e._extremes[u]=o.findExtremes(n,i,C)),g&&(e._extremes[g]=o.findExtremes(r,a,m))}function g(t,e){if(s.hasMarkers(t)){var n,r=t.marker,a=1.6*(t.marker.sizeref||1);if(n="area"===t.marker.sizemode?function(t){return Math.max(Math.sqrt((t||0)/a),3)}:function(t){return Math.max((t||0)/a,3)},i.isArrayOrTypedArray(r.size)){var A={type:"linear"};o.setConvert(A);for(var l=A.makeCalcdata(t.marker,"size"),c=new Array(e),u=0;u<e;u++)c[u]=n(l[u]);return c}return n(r.size)}}function h(t,e){var n=d(e),r=t._firstScatter;r[n]||(r[n]=e.uid)}function d(t){var e=t.stackgroup;return t.xaxis+t.yaxis+t.type+(e?"-"+e:"")}function f(t,e,n,r){var i=t.stackgroup;if(i){var o=e._scatterStackOpts[n._id+r._id][i],a="v"===o.orientation?r:n;return"linear"===a.type||"log"===a.type?o:void 0}}e.exports={calc:function(t,e){var n,s,d,p,I,C,m=t._fullLayout,y=o.getFromId(t,e.xaxis||"x"),E=o.getFromId(t,e.yaxis||"y"),v=y.makeCalcdata(e,"x"),B=E.makeCalcdata(e,"y"),b=e._length,Q=new Array(b),w=e.ids,x=f(e,m,y,E),_=!1;h(m,e);var D,M="x",k="y";for(x?(i.pushUnique(x.traceIndices,e._expandedIndex),(n="v"===x.orientation)?(k="s",D="x"):(M="s",D="y"),I="interpolate"===x.stackgaps):u(t,e,y,E,v,B,g(e,b)),s=0;s<b;s++){var S=Q[s]={},L=r(v[s]),R=r(B[s]);L&&R?(S[M]=v[s],S[k]=B[s]):x&&(n?L:R)?(S[D]=n?v[s]:B[s],S.gap=!0,I?(S.s=a,_=!0):S.s=0):S[M]=S[k]=a,w&&(S.id=String(w[s]))}if(l(Q,e),A(t,e),c(Q,e),x){for(s=0;s<Q.length;)Q[s][D]===a?Q.splice(s,1):s++;if(i.sort(Q,function(t,e){return t[D]-e[D]||t.i-e.i}),_){for(s=0;s<Q.length-1&&Q[s].gap;)s++;for((C=Q[s].s)||(C=Q[s].s=0),d=0;d<s;d++)Q[d].s=C;for(p=Q.length-1;p>s&&Q[p].gap;)p--;for(C=Q[p].s,d=Q.length-1;d>p;d--)Q[d].s=C;for(;s<p;)if(Q[++s].gap){for(d=s+1;Q[d].gap;)d++;for(var N=Q[s-1][D],F=Q[s-1].s,T=(Q[d].s-F)/(Q[d][D]-N);s<d;)Q[s].s=F+(Q[s][D]-N)*T,s++}}}return Q},calcMarkerSize:g,calcAxisExpansion:u,setFirstScatter:h,getStackOpts:f}},{"../../constants/numerical":695,"../../lib":719,"../../plots/cartesian/axes":767,"./arrays_to_calcdata":1111,"./calc_selection":1114,"./colorscale_calc":1115,"./subtypes":1135,"fast-isnumeric":225}],1114:[function(t,e,n){"use strict";var r=t("../../lib");e.exports=function(t,e){r.isArrayOrTypedArray(e.selectedpoints)&&r.tagSelected(t,e)}},{"../../lib":719}],1115:[function(t,e,n){"use strict";var r=t("../../components/colorscale/helpers").hasColorscale,i=t("../../components/colorscale/calc"),o=t("./subtypes");e.exports=function(t,e){o.hasLines(e)&&r(e,"line")&&i(t,e,{vals:e.line.color,containerStr:"line",cLetter:"c"}),o.hasMarkers(e)&&(r(e,"marker")&&i(t,e,{vals:e.marker.color,containerStr:"marker",cLetter:"c"}),r(e,"marker.line")&&i(t,e,{vals:e.marker.line.color,containerStr:"marker.line",cLetter:"c"}))}},{"../../components/colorscale/calc":601,"../../components/colorscale/helpers":604,"./subtypes":1135}],1116:[function(t,e,n){"use strict";e.exports={PTS_LINESONLY:20,minTolerance:.2,toleranceGrowth:10,maxScreensAway:20,eventDataKeys:[]}},{}],1117:[function(t,e,n){"use strict";var r=t("./calc");function i(t,e,n,r,i,o,a){i[r]=!0;var s={i:null,gap:!0,s:0};if(s[a]=n,t.splice(e,0,s),e&&n===t[e-1][a]){var A=t[e-1];s.s=A.s,s.i=A.i,s.gap=A.gap}else o&&(s.s=function(t,e,n,r){var i=t[e-1],o=t[e+1];return o?i?i.s+(o.s-i.s)*(n-i[r])/(o[r]-i[r]):o.s:i.s}(t,e,n,a));e||(t[0].t=t[1].t,t[0].trace=t[1].trace,delete t[1].t,delete t[1].trace)}e.exports=function(t,e){var n=e.xaxis,o=e.yaxis,a=n._id+o._id,s=t._fullLayout._scatterStackOpts[a];if(s){var A,l,c,u,g,h,d,f,p,I,C,m,y,E,v,B=t.calcdata;for(var b in s){var Q=(I=s[b]).traceIndices;if(Q.length){for(C="interpolate"===I.stackgaps,m=I.groupnorm,"v"===I.orientation?(y="x",E="y"):(y="y",E="x"),v=new Array(Q.length),A=0;A<v.length;A++)v[A]=!1;h=B[Q[0]];var w=new Array(h.length);for(A=0;A<h.length;A++)w[A]=h[A][y];for(A=1;A<Q.length;A++){for(g=B[Q[A]],l=c=0;l<g.length;l++){for(d=g[l][y];d>w[c]&&c<w.length;c++)i(g,l,w[c],A,v,C,y),l++;if(d!==w[c]){for(u=0;u<A;u++)i(B[Q[u]],c,d,u,v,C,y);w.splice(c,0,d)}c++}for(;c<w.length;c++)i(g,l,w[c],A,v,C,y),l++}var x=w.length;for(l=0;l<h.length;l++){for(f=h[l][E]=h[l].s,A=1;A<Q.length;A++)(g=B[Q[A]])[0].trace._rawLength=g[0].trace._length,g[0].trace._length=x,f+=g[l].s,g[l][E]=f;if(m)for(p=("fraction"===m?f:f/100)||1,A=0;A<Q.length;A++){var _=B[Q[A]][l];_[E]/=p,_.sNorm=_.s/p}}for(A=0;A<Q.length;A++){var D=(g=B[Q[A]])[0].trace,M=r.calcMarkerSize(D,D._rawLength),k=Array.isArray(M);if(M&&v[A]||k){var S=M;for(M=new Array(x),l=0;l<x;l++)M[l]=g[l].gap?0:k?S[g[l].i]:S}var L=new Array(x),R=new Array(x);for(l=0;l<x;l++)L[l]=g[l].x,R[l]=g[l].y;r.calcAxisExpansion(t,D,n,o,L,R,M),g[0].t.orientation=I.orientation}}}}}},{"./calc":1113}],1118:[function(t,e,n){"use strict";e.exports=function(t){for(var e=0;e<t.length;e++){var n=t[e];if("scatter"===n.type){var r=n.fill;if("none"!==r&&"toself"!==r&&(n.opacity=void 0,"tonexty"===r||"tonextx"===r))for(var i=e-1;i>=0;i--){var o=t[i];if("scatter"===o.type&&o.xaxis===n.xaxis&&o.yaxis===n.yaxis){o.opacity=void 0;break}}}}}},{}],1119:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../registry"),o=t("./attributes"),a=t("./constants"),s=t("./subtypes"),A=t("./xy_defaults"),l=t("./stack_defaults"),c=t("./marker_defaults"),u=t("./line_defaults"),g=t("./line_shape_defaults"),h=t("./text_defaults"),d=t("./fillcolor_defaults");e.exports=function(t,e,n,f){function p(n,i){return r.coerce(t,e,o,n,i)}var I=A(t,e,f,p);if(I||(e.visible=!1),e.visible){var C=l(t,e,f,p),m=!C&&I<a.PTS_LINESONLY?"lines+markers":"lines";p("text"),p("hovertext"),p("mode",m),s.hasLines(e)&&(u(t,e,n,f,p),g(t,e,p),p("connectgaps"),p("line.simplify")),s.hasMarkers(e)&&c(t,e,n,f,p,{gradient:!0}),s.hasText(e)&&h(t,e,f,p);var y=[];(s.hasMarkers(e)||s.hasText(e))&&(p("cliponaxis"),p("marker.maxdisplayed"),y.push("points")),p("fill",C?C.fillDflt:"none"),"none"!==e.fill&&(d(t,e,n,p),s.hasLines(e)||g(t,e,p));var E=(e.line||{}).color,v=(e.marker||{}).color;"tonext"!==e.fill&&"toself"!==e.fill||y.push("fills"),p("hoveron",y.join("+")||"points"),"fills"!==e.hoveron&&p("hovertemplate");var B=i.getComponentMethod("errorbars","supplyDefaults");B(t,e,E||v||n,{axis:"y"}),B(t,e,E||v||n,{axis:"x",inherit:"y"}),r.coerceSelectionMarkerOpacity(e,p)}}},{"../../lib":719,"../../registry":847,"./attributes":1112,"./constants":1116,"./fillcolor_defaults":1120,"./line_defaults":1124,"./line_shape_defaults":1126,"./marker_defaults":1130,"./stack_defaults":1133,"./subtypes":1135,"./text_defaults":1136,"./xy_defaults":1137}],1120:[function(t,e,n){"use strict";var r=t("../../components/color"),i=t("../../lib").isArrayOrTypedArray;e.exports=function(t,e,n,o){var a=!1;if(e.marker){var s=e.marker.color,A=(e.marker.line||{}).color;s&&!i(s)?a=s:A&&!i(A)&&(a=A)}o("fillcolor",r.addOpacity((e.line||{}).color||a||n,.5))}},{"../../components/color":593,"../../lib":719}],1121:[function(t,e,n){"use strict";var r=t("../../components/color"),i=t("./subtypes");e.exports=function(t,e){var n,o;if("lines"===t.mode)return(n=t.line.color)&&r.opacity(n)?n:t.fillcolor;if("none"===t.mode)return t.fill?t.fillcolor:"";var a=e.mcc||(t.marker||{}).color,s=e.mlcc||((t.marker||{}).line||{}).color;return(o=a&&r.opacity(a)?a:s&&r.opacity(s)&&(e.mlw||((t.marker||{}).line||{}).width)?s:"")?r.opacity(o)<.3?r.addOpacity(o,.3):o:(n=(t.line||{}).color)&&r.opacity(n)&&i.hasLines(t)&&t.line.width?n:t.fillcolor}},{"../../components/color":593,"./subtypes":1135}],1122:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/fx"),o=t("../../registry"),a=t("./get_trace_color"),s=t("../../components/color"),A=r.fillText;e.exports=function(t,e,n,l){var c=t.cd,u=c[0].trace,g=t.xa,h=t.ya,d=g.c2p(e),f=h.c2p(n),p=[d,f],I=u.hoveron||"",C=-1!==u.mode.indexOf("markers")?3:.5;if(-1!==I.indexOf("points")){var m=function(t){var e=Math.max(C,t.mrc||0),n=g.c2p(t.x)-d,r=h.c2p(t.y)-f;return Math.max(Math.sqrt(n*n+r*r)-e,1-C/e)},y=i.getDistanceFunction(l,function(t){var e=Math.max(3,t.mrc||0),n=1-1/e,r=Math.abs(g.c2p(t.x)-d);return r<e?n*r/e:r-e+n},function(t){var e=Math.max(3,t.mrc||0),n=1-1/e,r=Math.abs(h.c2p(t.y)-f);return r<e?n*r/e:r-e+n},m);if(i.getClosest(c,y,t),!1!==t.index){var E=c[t.index],v=g.c2p(E.x,!0),B=h.c2p(E.y,!0),b=E.mrc||1;t.index=E.i;var Q=c[0].t.orientation,w=Q&&(E.sNorm||E.s),x="h"===Q?w:E.x,_="v"===Q?w:E.y;return r.extendFlat(t,{color:a(u,E),x0:v-b,x1:v+b,xLabelVal:x,y0:B-b,y1:B+b,yLabelVal:_,spikeDistance:m(E),hovertemplate:u.hovertemplate}),A(E,u,t),o.getComponentMethod("errorbars","hoverInfo")(E,u,t),[t]}}if(-1!==I.indexOf("fills")&&u._polygons){var D,M,k,S,L,R,N,F,T,U=u._polygons,O=[],G=!1,H=1/0,j=-1/0,Y=1/0,P=-1/0;for(D=0;D<U.length;D++)(k=U[D]).contains(p)&&(G=!G,O.push(k),Y=Math.min(Y,k.ymin),P=Math.max(P,k.ymax));if(G){var K=((Y=Math.max(Y,0))+(P=Math.min(P,h._length)))/2;for(D=0;D<O.length;D++)for(S=O[D].pts,M=1;M<S.length;M++)(F=S[M-1][1])>K!=(T=S[M][1])>=K&&(R=S[M-1][0],N=S[M][0],T-F&&(L=R+(N-R)*(K-F)/(T-F),H=Math.min(H,L),j=Math.max(j,L)));H=Math.max(H,0),j=Math.min(j,g._length);var z=s.defaultLine;return s.opacity(u.fillcolor)?z=u.fillcolor:s.opacity((u.line||{}).color)&&(z=u.line.color),r.extendFlat(t,{distance:t.maxHoverDistance,x0:H,x1:j,y0:K,y1:K,color:z,hovertemplate:!1}),delete t.index,u.text&&!Array.isArray(u.text)?t.text=String(u.text):t.text=u.name,[t]}}}},{"../../components/color":593,"../../components/fx":632,"../../lib":719,"../../registry":847,"./get_trace_color":1121}],1123:[function(t,e,n){"use strict";var r=t("./subtypes");e.exports={hasLines:r.hasLines,hasMarkers:r.hasMarkers,hasText:r.hasText,isBubble:r.isBubble,attributes:t("./attributes"),supplyDefaults:t("./defaults"),crossTraceDefaults:t("./cross_trace_defaults"),calc:t("./calc").calc,crossTraceCalc:t("./cross_trace_calc"),arraysToCalcdata:t("./arrays_to_calcdata"),plot:t("./plot"),colorbar:t("./marker_colorbar"),style:t("./style").style,styleOnSelect:t("./style").styleOnSelect,hoverPoints:t("./hover"),selectPoints:t("./select"),animatable:!0,moduleType:"trace",name:"scatter",basePlotModule:t("../../plots/cartesian"),categories:["cartesian","svg","symbols","errorBarsOK","showLegend","scatter-like","zoomScale"],meta:{}}},{"../../plots/cartesian":778,"./arrays_to_calcdata":1111,"./attributes":1112,"./calc":1113,"./cross_trace_calc":1117,"./cross_trace_defaults":1118,"./defaults":1119,"./hover":1122,"./marker_colorbar":1129,"./plot":1131,"./select":1132,"./style":1134,"./subtypes":1135}],1124:[function(t,e,n){"use strict";var r=t("../../lib").isArrayOrTypedArray,i=t("../../components/colorscale/helpers").hasColorscale,o=t("../../components/colorscale/defaults");e.exports=function(t,e,n,a,s,A){var l=(t.marker||{}).color;s("line.color",n),i(t,"line")?o(t,e,a,s,{prefix:"line.",cLetter:"c"}):s("line.color",!r(l)&&l||n),s("line.width"),(A||{}).noDash||s("line.dash")}},{"../../components/colorscale/defaults":603,"../../components/colorscale/helpers":604,"../../lib":719}],1125:[function(t,e,n){"use strict";var r=t("../../constants/numerical"),i=r.BADNUM,o=r.LOG_CLIP,a=o+.5,s=o-.5,A=t("../../lib"),l=A.segmentsIntersect,c=A.constrain,u=t("./constants");e.exports=function(t,e){var n,r,o,g,h,d,f,p,I,C,m,y,E,v,B,b,Q,w,x=e.xaxis,_=e.yaxis,D="log"===x.type,M="log"===_.type,k=x._length,S=_._length,L=e.connectGaps,R=e.baseTolerance,N=e.shape,F="linear"===N,T=e.fill&&"none"!==e.fill,U=[],O=u.minTolerance,G=t.length,H=new Array(G),j=0;function Y(e){var n=t[e];if(!n)return!1;var r=x.c2p(n.x),o=_.c2p(n.y);if(r===i){if(D&&(r=x.c2p(n.x,!0)),r===i)return!1;M&&o===i&&(r*=Math.abs(x._m*S*(x._m>0?a:s)/(_._m*k*(_._m>0?a:s)))),r*=1e3}if(o===i){if(M&&(o=_.c2p(n.y,!0)),o===i)return!1;o*=1e3}return[r,o]}function P(t,e,n,r){var i=n-t,o=r-e,a=.5-t,s=.5-e,A=i*i+o*o,l=i*a+o*s;if(l>0&&l<A){var c=a*o-s*i;if(c*c<A)return!0}}function K(t,e){var n=t[0]/k,r=t[1]/S,i=Math.max(0,-n,n-1,-r,r-1);return i&&void 0!==Q&&P(n,r,Q,w)&&(i=0),i&&e&&P(n,r,e[0]/k,e[1]/S)&&(i=0),(1+u.toleranceGrowth*i)*R}function z(t,e){var n=t[0]-e[0],r=t[1]-e[1];return Math.sqrt(n*n+r*r)}var J,q,V,W,X,Z,$,tt=u.maxScreensAway,et=-k*tt,nt=k*(1+tt),rt=-S*tt,it=S*(1+tt),ot=[[et,rt,nt,rt],[nt,rt,nt,it],[nt,it,et,it],[et,it,et,rt]];function at(t){if(t[0]<et||t[0]>nt||t[1]<rt||t[1]>it)return[c(t[0],et,nt),c(t[1],rt,it)]}function st(t,e){return t[0]===e[0]&&(t[0]===et||t[0]===nt)||t[1]===e[1]&&(t[1]===rt||t[1]===it)||void 0}function At(t,e,n){return function(r,i){var o=at(r),a=at(i),s=[];if(o&&a&&st(o,a))return s;o&&s.push(o),a&&s.push(a);var l=2*A.constrain((r[t]+i[t])/2,e,n)-((o||r)[t]+(a||i)[t]);return l&&((o&&a?l>0==o[t]>a[t]?o:a:o||a)[t]+=l),s}}function lt(t){var e=t[0],n=t[1],r=e===H[j-1][0],i=n===H[j-1][1];if(!r||!i)if(j>1){var o=e===H[j-2][0],a=n===H[j-2][1];r&&(e===et||e===nt)&&o?a?j--:H[j-1]=t:i&&(n===rt||n===it)&&a?o?j--:H[j-1]=t:H[j++]=t}else H[j++]=t}function ct(t){H[j-1][0]!==t[0]&&H[j-1][1]!==t[1]&&lt([V,W]),lt(t),X=null,V=W=0}function ut(t){if(Q=t[0]/k,w=t[1]/S,J=t[0]<et?et:t[0]>nt?nt:0,q=t[1]<rt?rt:t[1]>it?it:0,J||q){if(j)if(X){var e=$(X,t);e.length>1&&(ct(e[0]),H[j++]=e[1])}else Z=$(H[j-1],t)[0],H[j++]=Z;else H[j++]=[J||t[0],q||t[1]];var n=H[j-1];J&&q&&(n[0]!==J||n[1]!==q)?(X&&(V!==J&&W!==q?lt(V&&W?(r=X,o=(i=t)[0]-r[0],a=(i[1]-r[1])/o,(r[1]*i[0]-i[1]*r[0])/o>0?[a>0?et:nt,it]:[a>0?nt:et,rt]):[V||J,W||q]):V&&W&&lt([V,W])),lt([J,q])):V-J&&W-q&&lt([J||V,q||W]),X=t,V=J,W=q}else X&&ct($(X,t)[0]),H[j++]=t;var r,i,o,a}for("linear"===N||"spline"===N?$=function(t,e){for(var n=[],r=0,i=0;i<4;i++){var o=ot[i],a=l(t[0],t[1],e[0],e[1],o[0],o[1],o[2],o[3]);a&&(!r||Math.abs(a.x-n[0][0])>1||Math.abs(a.y-n[0][1])>1)&&(a=[a.x,a.y],r&&z(a,t)<z(n[0],t)?n.unshift(a):n.push(a),r++)}return n}:"hv"===N||"vh"===N?$=function(t,e){var n=[],r=at(t),i=at(e);return r&&i&&st(r,i)?n:(r&&n.push(r),i&&n.push(i),n)}:"hvh"===N?$=At(0,et,nt):"vhv"===N&&($=At(1,rt,it)),n=0;n<G;n++)if(r=Y(n)){for(j=0,X=null,ut(r),n++;n<G;n++){if(!(g=Y(n))){if(L)continue;break}if(F&&e.simplify){var gt=Y(n+1);if(C=z(g,r),T&&(0===j||j===G-1)||!(C<K(g,gt)*O)){for(p=[(g[0]-r[0])/C,(g[1]-r[1])/C],h=r,m=C,y=v=B=0,f=!1,o=g,n++;n<t.length;n++){if(d=gt,gt=Y(n+1),!d){if(L)continue;break}if(b=(I=[d[0]-r[0],d[1]-r[1]])[0]*p[1]-I[1]*p[0],v=Math.min(v,b),(B=Math.max(B,b))-v>K(d,gt))break;o=d,(E=I[0]*p[0]+I[1]*p[1])>m?(m=E,g=d,f=!1):E<y&&(y=E,h=d,f=!0)}if(f?(ut(g),o!==h&&ut(h)):(h!==r&&ut(h),o!==g&&ut(g)),ut(o),n>=t.length||!d)break;ut(d),r=d}}else ut(g)}X&&lt([V||X[0],W||X[1]]),U.push(H.slice(0,j))}return U}},{"../../constants/numerical":695,"../../lib":719,"./constants":1116}],1126:[function(t,e,n){"use strict";e.exports=function(t,e,n){"spline"===n("line.shape")&&n("line.smoothing")}},{}],1127:[function(t,e,n){"use strict";var r={tonextx:1,tonexty:1,tonext:1};e.exports=function(t,e,n){var i,o,a,s,A,l={},c=!1,u=-1,g=0,h=-1;for(o=0;o<n.length;o++)(a=(i=n[o][0].trace).stackgroup||"")?a in l?A=l[a]:(A=l[a]=g,g++):i.fill in r&&h>=0?A=h:(A=h=g,g++),A<u&&(c=!0),i._groupIndex=u=A;var d=n.slice();c&&d.sort(function(t,e){var n=t[0].trace,r=e[0].trace;return n._groupIndex-r._groupIndex||n.index-r.index});var f={};for(o=0;o<d.length;o++)a=(i=d[o][0].trace).stackgroup||"",!0===i.visible?(i._nexttrace=null,i.fill in r&&(s=f[a],i._prevtrace=s||null,s&&(s._nexttrace=i)),i._ownfill=i.fill&&("tozero"===i.fill.substr(0,6)||"toself"===i.fill||"to"===i.fill.substr(0,2)&&!i._prevtrace),f[a]=i):i._prevtrace=i._nexttrace=i._ownfill=null;return d}},{}],1128:[function(t,e,n){"use strict";var r=t("fast-isnumeric");e.exports=function(t){var e=t.marker,n=e.sizeref||1,i=e.sizemin||0,o="area"===e.sizemode?function(t){return Math.sqrt(t/n)}:function(t){return t/n};return function(t){var e=o(t/2);return r(e)&&e>0?Math.max(e,i):0}}},{"fast-isnumeric":225}],1129:[function(t,e,n){"use strict";e.exports={container:"marker",min:"cmin",max:"cmax"}},{}],1130:[function(t,e,n){"use strict";var r=t("../../components/color"),i=t("../../components/colorscale/helpers").hasColorscale,o=t("../../components/colorscale/defaults"),a=t("./subtypes");e.exports=function(t,e,n,s,A,l){var c=a.isBubble(t),u=(t.line||{}).color;l=l||{},u&&(n=u),A("marker.symbol"),A("marker.opacity",c?.7:1),A("marker.size"),A("marker.color",n),i(t,"marker")&&o(t,e,s,A,{prefix:"marker.",cLetter:"c"}),l.noSelect||(A("selected.marker.color"),A("unselected.marker.color"),A("selected.marker.size"),A("unselected.marker.size")),l.noLine||(A("marker.line.color",u&&!Array.isArray(u)&&e.marker.color!==u?u:c?r.background:r.defaultLine),i(t,"marker.line")&&o(t,e,s,A,{prefix:"marker.line.",cLetter:"c"}),A("marker.line.width",c?1:0)),c&&(A("marker.sizeref"),A("marker.sizemin"),A("marker.sizemode")),l.gradient&&"none"!==A("marker.gradient.type")&&A("marker.gradient.color")}},{"../../components/color":593,"../../components/colorscale/defaults":603,"../../components/colorscale/helpers":604,"./subtypes":1135}],1131:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../registry"),o=t("../../lib"),a=o.ensureSingle,s=o.identity,A=t("../../components/drawing"),l=t("./subtypes"),c=t("./line_points"),u=t("./link_traces"),g=t("../../lib/polygon").tester;function h(t,e,n,u,h,d,f){var p;!function(t,e,n,i,a){var s=n.xaxis,A=n.yaxis,c=r.extent(o.simpleMap(s.range,s.r2c)),u=r.extent(o.simpleMap(A.range,A.r2c)),g=i[0].trace;if(l.hasMarkers(g)){var h=g.marker.maxdisplayed;if(0!==h){var d=i.filter(function(t){return t.x>=c[0]&&t.x<=c[1]&&t.y>=u[0]&&t.y<=u[1]}),f=Math.ceil(d.length/h),p=0;a.forEach(function(t,n){var r=t[0].trace;l.hasMarkers(r)&&r.marker.maxdisplayed>0&&n<e&&p++});var I=Math.round(p*f/3+Math.floor(p/3)*f/7.1);i.forEach(function(t){delete t.vis}),d.forEach(function(t,e){0===Math.round((e+I)%f)&&(t.vis=!0)})}}}(0,e,n,u,h);var I=!!f&&f.duration>0;function C(t){return I?t.transition():t}var m=n.xaxis,y=n.yaxis,E=u[0].trace,v=E.line,B=r.select(d),b=a(B,"g","errorbars"),Q=a(B,"g","lines"),w=a(B,"g","points"),x=a(B,"g","text");if(i.getComponentMethod("errorbars","plot")(t,b,n,f),!0===E.visible){var _,D;C(B).style("opacity",E.opacity);var M=E.fill.charAt(E.fill.length-1);"x"!==M&&"y"!==M&&(M=""),u[0][n.isRangePlot?"nodeRangePlot3":"node3"]=B;var k,S,L="",R=[],N=E._prevtrace;N&&(L=N._prevRevpath||"",D=N._nextFill,R=N._polygons);var F,T,U,O,G,H,j,Y="",P="",K=[],z=o.noop;if(_=E._ownFill,l.hasLines(E)||"none"!==E.fill){for(D&&D.datum(u),-1!==["hv","vh","hvh","vhv"].indexOf(v.shape)?(F=A.steps(v.shape),T=A.steps(v.shape.split("").reverse().join(""))):F=T="spline"===v.shape?function(t){var e=t[t.length-1];return t.length>1&&t[0][0]===e[0]&&t[0][1]===e[1]?A.smoothclosed(t.slice(1),v.smoothing):A.smoothopen(t,v.smoothing)}:function(t){return"M"+t.join("L")},U=function(t){return T(t.reverse())},K=c(u,{xaxis:m,yaxis:y,connectGaps:E.connectgaps,baseTolerance:Math.max(v.width||1,3)/4,shape:v.shape,simplify:v.simplify,fill:E.fill}),j=E._polygons=new Array(K.length),p=0;p<K.length;p++)E._polygons[p]=g(K[p]);K.length&&(O=K[0][0],H=(G=K[K.length-1])[G.length-1]),z=function(t){return function(e){if(k=F(e),S=U(e),Y?M?(Y+="L"+k.substr(1),P=S+"L"+P.substr(1)):(Y+="Z"+k,P=S+"Z"+P):(Y=k,P=S),l.hasLines(E)&&e.length>1){var n=r.select(this);if(n.datum(u),t)C(n.style("opacity",0).attr("d",k).call(A.lineGroupStyle)).style("opacity",1);else{var i=C(n);i.attr("d",k),A.singleLineStyle(u,i)}}}}}var J=Q.selectAll(".js-line").data(K);C(J.exit()).style("opacity",0).remove(),J.each(z(!1)),J.enter().append("path").classed("js-line",!0).style("vector-effect","non-scaling-stroke").call(A.lineGroupStyle).each(z(!0)),A.setClipUrl(J,n.layerClipId,t),K.length?(_?(_.datum(u),O&&H&&(M?("y"===M?O[1]=H[1]=y.c2p(0,!0):"x"===M&&(O[0]=H[0]=m.c2p(0,!0)),C(_).attr("d","M"+H+"L"+O+"L"+Y.substr(1)).call(A.singleFillStyle)):C(_).attr("d",Y+"Z").call(A.singleFillStyle))):D&&("tonext"===E.fill.substr(0,6)&&Y&&L?("tonext"===E.fill?C(D).attr("d",Y+"Z"+L+"Z").call(A.singleFillStyle):C(D).attr("d",Y+"L"+L.substr(1)+"Z").call(A.singleFillStyle),E._polygons=E._polygons.concat(R)):(V(D),E._polygons=null)),E._prevRevpath=P,E._prevPolygons=j):(_?V(_):D&&V(D),E._polygons=E._prevRevpath=E._prevPolygons=null),w.datum(u),x.datum(u),function(e,i,o){var a,c=o[0].trace,u=l.hasMarkers(c),g=l.hasText(c),h=function(t){if(t.ids)return $}(c),d=tt,f=tt;if(u||g){var p=s,E=c.stackgroup,v=E&&"infer zero"===t._fullLayout._scatterStackOpts[m._id+y._id][E].stackgaps;c.marker.maxdisplayed||c._needsCull?p=v?X:W:E&&!v&&(p=Z),u&&(d=p),g&&(f=p)}var B,b=(a=e.selectAll("path.point").data(d,h)).enter().append("path").classed("point",!0);I&&b.call(A.pointStyle,c,t).call(A.translatePoints,m,y).style("opacity",0).transition().style("opacity",1),a.order(),u&&(B=A.makePointStyleFns(c)),a.each(function(e){var i=r.select(this),o=C(i);A.translatePoint(e,o,m,y)?(A.singlePointStyle(e,o,c,B,t),n.layerClipId&&A.hideOutsideRangePoint(e,o,m,y,c.xcalendar,c.ycalendar),c.customdata&&i.classed("plotly-customdata",null!==e.data&&void 0!==e.data)):o.remove()}),I?a.exit().transition().style("opacity",0).remove():a.exit().remove(),(a=i.selectAll("g").data(f,h)).enter().append("g").classed("textpoint",!0).append("text"),a.order(),a.each(function(t){var e=r.select(this),i=C(e.select("text"));A.translatePoint(t,i,m,y)?n.layerClipId&&A.hideOutsideRangePoint(t,e,m,y,c.xcalendar,c.ycalendar):e.remove()}),a.selectAll("text").call(A.textPointStyle,c,t).each(function(t){var e=m.c2p(t.x),n=y.c2p(t.y);r.select(this).selectAll("tspan.line").each(function(){C(r.select(this)).attr({x:e,y:n})})}),a.exit().remove()}(w,x,u);var q=!1===E.cliponaxis?null:n.layerClipId;A.setClipUrl(w,q,t),A.setClipUrl(x,q,t)}function V(t){C(t).attr("d","M0,0Z")}function W(t){return t.filter(function(t){return!t.gap&&t.vis})}function X(t){return t.filter(function(t){return t.vis})}function Z(t){return t.filter(function(t){return!t.gap})}function $(t){return t.id}function tt(){return!1}}e.exports=function(t,e,n,i,o,l){var c,g,d=!o,f=!!o&&o.duration>0,p=u(t,e,n);(c=i.selectAll("g.trace").data(p,function(t){return t[0].trace.uid})).enter().append("g").attr("class",function(t){return"trace scatter trace"+t[0].trace.uid}).style("stroke-miterlimit",2),c.order(),function(t,e,n){e.each(function(e){var i=a(r.select(this),"g","fills");A.setClipUrl(i,n.layerClipId,t);var o=e[0].trace,l=[];o._ownfill&&l.push("_ownFill"),o._nexttrace&&l.push("_nextFill");var c=i.selectAll("g").data(l,s);c.enter().append("g"),c.exit().each(function(t){o[t]=null}).remove(),c.order().each(function(t){o[t]=a(r.select(this),"path","js-fill")})})}(t,c,e),f?(l&&(g=l()),r.transition().duration(o.duration).ease(o.easing).each("end",function(){g&&g()}).each("interrupt",function(){g&&g()}).each(function(){i.selectAll("g.trace").each(function(n,r){h(t,r,e,n,p,this,o)})})):c.each(function(n,r){h(t,r,e,n,p,this,o)}),d&&c.exit().remove(),i.selectAll("path:not([d])").remove()}},{"../../components/drawing":614,"../../lib":719,"../../lib/polygon":731,"../../registry":847,"./line_points":1125,"./link_traces":1127,"./subtypes":1135,d3:163}],1132:[function(t,e,n){"use strict";var r=t("./subtypes");e.exports=function(t,e){var n,i,o,a,s=t.cd,A=t.xaxis,l=t.yaxis,c=[],u=s[0].trace;if(!r.hasMarkers(u)&&!r.hasText(u))return[];if(!1===e)for(n=0;n<s.length;n++)s[n].selected=0;else for(n=0;n<s.length;n++)i=s[n],o=A.c2p(i.x),a=l.c2p(i.y),null!==i.i&&e.contains([o,a],!1,n,t)?(c.push({pointNumber:i.i,x:A.c2d(i.x),y:l.c2d(i.y)}),i.selected=1):i.selected=0;return c}},{"./subtypes":1135}],1133:[function(t,e,n){"use strict";var r=["orientation","groupnorm","stackgaps"];e.exports=function(t,e,n,i){var o=n._scatterStackOpts,a=i("stackgroup");if(a){var s=e.xaxis+e.yaxis,A=o[s];A||(A=o[s]={});var l=A[a],c=!1;l?l.traces.push(e):(l=A[a]={traceIndices:[],traces:[e]},c=!0);for(var u={orientation:e.x&&!e.y?"h":"v"},g=0;g<r.length;g++){var h=r[g],d=h+"Found";if(!l[d]){var f=void 0!==t[h],p="orientation"===h;if((f||c)&&(l[h]=i(h,u[h]),p&&(l.fillDflt="h"===l[h]?"tonextx":"tonexty"),f&&(l[d]=!0,!c&&(delete l.traces[0][h],p))))for(var I=0;I<l.traces.length-1;I++){var C=l.traces[I];C._input.fill!==C.fill&&(C.fill=l.fillDflt)}}}return l}}},{}],1134:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/drawing"),o=t("../../registry");function a(t,e,n){i.pointStyle(t.selectAll("path.point"),e,n)}function s(t,e,n){i.textPointStyle(t.selectAll("text"),e,n)}e.exports={style:function(t){var e=r.select(t).selectAll("g.trace.scatter");e.style("opacity",function(t){return t[0].trace.opacity}),e.selectAll("g.points").each(function(e){a(r.select(this),e.trace||e[0].trace,t)}),e.selectAll("g.text").each(function(e){s(r.select(this),e.trace||e[0].trace,t)}),e.selectAll("g.trace path.js-line").call(i.lineGroupStyle),e.selectAll("g.trace path.js-fill").call(i.fillGroupStyle),o.getComponentMethod("errorbars","style")(e)},stylePoints:a,styleText:s,styleOnSelect:function(t,e,n){var r=e[0].trace;r.selectedpoints?(i.selectedPointStyle(n.selectAll("path.point"),r),i.selectedTextStyle(n.selectAll("text"),r)):(a(n,r,t),s(n,r,t))}}},{"../../components/drawing":614,"../../registry":847,d3:163}],1135:[function(t,e,n){"use strict";var r=t("../../lib");e.exports={hasLines:function(t){return t.visible&&t.mode&&-1!==t.mode.indexOf("lines")},hasMarkers:function(t){return t.visible&&(t.mode&&-1!==t.mode.indexOf("markers")||"splom"===t.type)},hasText:function(t){return t.visible&&t.mode&&-1!==t.mode.indexOf("text")},isBubble:function(t){return r.isPlainObject(t.marker)&&r.isArrayOrTypedArray(t.marker.size)}}},{"../../lib":719}],1136:[function(t,e,n){"use strict";var r=t("../../lib");e.exports=function(t,e,n,i,o){o=o||{},i("textposition"),r.coerceFont(i,"textfont",n.font),o.noSelect||(i("selected.textfont.color"),i("unselected.textfont.color"))}},{"../../lib":719}],1137:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../registry");e.exports=function(t,e,n,o){var a,s=o("x"),A=o("y");if(i.getComponentMethod("calendars","handleTraceDefaults")(t,e,["x","y"],n),s){var l=r.minRowLength(s);A?a=Math.min(l,r.minRowLength(A)):(a=l,o("y0"),o("dy"))}else{if(!A)return 0;a=r.minRowLength(A),o("x0"),o("dx")}return e._length=a,a}},{"../../lib":719,"../../registry":847}],1138:[function(t,e,n){"use strict";var r=t("../scatter/attributes"),i=t("../../components/colorscale/attributes"),o=t("../../components/fx/hovertemplate_attributes"),a=t("../../plots/attributes"),s=t("../../constants/gl3d_dashes"),A=t("../../constants/gl3d_markers"),l=t("../../lib/extend").extendFlat,c=t("../../plot_api/edit_types").overrideAll,u=r.line,g=r.marker,h=g.line,d=l({width:u.width,dash:{valType:"enumerated",values:Object.keys(s),dflt:"solid"}},i("line")),f=e.exports=c({x:r.x,y:r.y,z:{valType:"data_array"},text:l({},r.text,{}),hovertext:l({},r.hovertext,{}),hovertemplate:o(),mode:l({},r.mode,{dflt:"lines+markers"}),surfaceaxis:{valType:"enumerated",values:[-1,0,1,2],dflt:-1},surfacecolor:{valType:"color"},projection:{x:{show:{valType:"boolean",dflt:!1},opacity:{valType:"number",min:0,max:1,dflt:1},scale:{valType:"number",min:0,max:10,dflt:2/3}},y:{show:{valType:"boolean",dflt:!1},opacity:{valType:"number",min:0,max:1,dflt:1},scale:{valType:"number",min:0,max:10,dflt:2/3}},z:{show:{valType:"boolean",dflt:!1},opacity:{valType:"number",min:0,max:1,dflt:1},scale:{valType:"number",min:0,max:10,dflt:2/3}}},connectgaps:r.connectgaps,line:d,marker:l({symbol:{valType:"enumerated",values:Object.keys(A),dflt:"circle",arrayOk:!0},size:l({},g.size,{dflt:8}),sizeref:g.sizeref,sizemin:g.sizemin,sizemode:g.sizemode,opacity:l({},g.opacity,{arrayOk:!1}),colorbar:g.colorbar,line:l({width:l({},h.width,{arrayOk:!1})},i("marker.line"))},i("marker")),textposition:l({},r.textposition,{dflt:"top center"}),textfont:{color:r.textfont.color,size:r.textfont.size,family:l({},r.textfont.family,{arrayOk:!1})},hoverinfo:l({},a.hoverinfo)},"calc","nested");f.x.editType=f.y.editType=f.z.editType="calc+clearAxisTypes"},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../constants/gl3d_dashes":692,"../../constants/gl3d_markers":693,"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plots/attributes":764,"../scatter/attributes":1112}],1139:[function(t,e,n){"use strict";var r=t("../scatter/arrays_to_calcdata"),i=t("../scatter/colorscale_calc");e.exports=function(t,e){var n=[{x:!1,y:!1,trace:e,t:{}}];return r(n,e),i(t,e),n}},{"../scatter/arrays_to_calcdata":1111,"../scatter/colorscale_calc":1115}],1140:[function(t,e,n){"use strict";var r=t("../../registry");function i(t,e,n,i){if(!e||!e.visible)return null;for(var o=r.getComponentMethod("errorbars","makeComputeError")(e),a=new Array(t.length),s=0;s<t.length;s++){var A=o(+t[s],s);if("log"===i.type){var l=i.c2l(t[s]),c=t[s]-A[0],u=t[s]+A[1];if(a[s]=[(i.c2l(c,!0)-l)*n,(i.c2l(u,!0)-l)*n],c>0){var g=i.c2l(c);i._lowerLogErrorBound||(i._lowerLogErrorBound=g),i._lowerErrorBound=Math.min(i._lowerLogErrorBound,g)}}else a[s]=[-A[0]*n,A[1]*n]}return a}e.exports=function(t,e,n){var r=[i(t.x,t.error_x,e[0],n.xaxis),i(t.y,t.error_y,e[1],n.yaxis),i(t.z,t.error_z,e[2],n.zaxis)],o=function(t){for(var e=0;e<t.length;e++)if(t[e])return t[e].length;return 0}(r);if(0===o)return null;for(var a=new Array(o),s=0;s<o;s++){for(var A=[[0,0,0],[0,0,0]],l=0;l<3;l++)if(r[l])for(var c=0;c<2;c++)A[c][l]=r[l][s][c];a[s]=A}return a}},{"../../registry":847}],1141:[function(t,e,n){"use strict";var r=t("gl-line3d"),i=t("gl-scatter3d"),o=t("gl-error3d"),a=t("gl-mesh3d"),s=t("delaunay-triangulate"),A=t("../../lib"),l=t("../../lib/str2rgbarray"),c=t("../../lib/gl_format_color").formatColor,u=t("../scatter/make_bubble_size_func"),g=t("../../constants/gl3d_dashes"),h=t("../../constants/gl3d_markers"),d=t("./calc_errors");function f(t,e){this.scene=t,this.uid=e,this.linePlot=null,this.scatterPlot=null,this.errorBars=null,this.textMarkers=null,this.delaunayMesh=null,this.color=null,this.mode="",this.dataPoints=[],this.axesBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.textLabels=null,this.data=null}var p=f.prototype;function I(t){return null==t?0:t.indexOf("left")>-1?-1:t.indexOf("right")>-1?1:0}function C(t){return null==t?0:t.indexOf("top")>-1?-1:t.indexOf("bottom")>-1?1:0}function m(t,e){return e(4*t)}function y(t){return h[t]}function E(t,e,n,r,i){var o=null;if(A.isArrayOrTypedArray(t)){o=[];for(var a=0;a<e;a++)void 0===t[a]?o[a]=r:o[a]=n(t[a],i)}else o=n(t,A.identity);return o}function v(t,e){var n,r,i,o,a,s,g=[],h=t.fullSceneLayout,f=t.dataScale,p=h.xaxis,v=h.yaxis,B=h.zaxis,b=e.marker,Q=e.line,w=e.x||[],x=e.y||[],_=e.z||[],D=w.length,M=e.xcalendar,k=e.ycalendar,S=e.zcalendar;for(a=0;a<D;a++)n=p.d2l(w[a],0,M)*f[0],r=v.d2l(x[a],0,k)*f[1],i=B.d2l(_[a],0,S)*f[2],g[a]=[n,r,i];if(Array.isArray(e.text))s=e.text;else if(void 0!==e.text)for(s=new Array(D),a=0;a<D;a++)s[a]=e.text;if(o={position:g,mode:e.mode,text:s},"line"in e&&(o.lineColor=c(Q,1,D),o.lineWidth=Q.width,o.lineDashes=Q.dash),"marker"in e){var L=u(e);o.scatterColor=c(b,1,D),o.scatterSize=E(b.size,D,m,20,L),o.scatterMarker=E(b.symbol,D,y,"●"),o.scatterLineWidth=b.line.width,o.scatterLineColor=c(b.line,1,D),o.scatterAngle=0}"textposition"in e&&(o.textOffset=function(t){var e=[0,0];if(Array.isArray(t))for(var n=0;n<t.length;n++)e[n]=[0,0],t[n]&&(e[n][0]=I(t[n]),e[n][1]=C(t[n]));else e[0]=I(t),e[1]=C(t);return e}(e.textposition),o.textColor=c(e.textfont,1,D),o.textSize=E(e.textfont.size,D,A.identity,12),o.textFont=e.textfont.family,o.textAngle=0);var R=["x","y","z"];for(o.project=[!1,!1,!1],o.projectScale=[1,1,1],o.projectOpacity=[1,1,1],a=0;a<3;++a){var N=e.projection[R[a]];(o.project[a]=N.show)&&(o.projectOpacity[a]=N.opacity,o.projectScale[a]=N.scale)}o.errorBounds=d(e,f,h);var F=function(t){for(var e=[0,0,0],n=[[0,0,0],[0,0,0],[0,0,0]],r=[1,1,1],i=0;i<3;i++){var o=t[i];o&&!1!==o.copy_zstyle&&!1!==t[2].visible&&(o=t[2]),o&&o.visible&&(e[i]=o.width/2,n[i]=l(o.color),r[i]=o.thickness)}return{capSize:e,color:n,lineWidth:r}}([e.error_x,e.error_y,e.error_z]);return o.errorColor=F.color,o.errorLineWidth=F.lineWidth,o.errorCapSize=F.capSize,o.delaunayAxis=e.surfaceaxis,o.delaunayColor=l(e.surfacecolor),o}function B(t){if(Array.isArray(t)){var e=t[0];return Array.isArray(e)&&(t=e),"rgb("+t.slice(0,3).map(function(t){return Math.round(255*t)})+")"}return null}p.handlePick=function(t){if(t.object&&(t.object===this.linePlot||t.object===this.delaunayMesh||t.object===this.textMarkers||t.object===this.scatterPlot)){var e=t.index=t.data.index;return t.object.highlight&&t.object.highlight(null),this.scatterPlot&&(t.object=this.scatterPlot,this.scatterPlot.highlight(t.data)),t.textLabel="",this.textLabels&&(Array.isArray(this.textLabels)?(this.textLabels[e]||0===this.textLabels[e])&&(t.textLabel=this.textLabels[e]):t.textLabel=this.textLabels),t.traceCoordinate=[this.data.x[e],this.data.y[e],this.data.z[e]],!0}},p.update=function(t){var e,n,A,l,c=this.scene.glplot.gl,u=g.solid;this.data=t;var h=v(this.scene,t);"mode"in h&&(this.mode=h.mode),"lineDashes"in h&&h.lineDashes in g&&(u=g[h.lineDashes]),this.color=B(h.scatterColor)||B(h.lineColor),this.dataPoints=h.position,e={gl:this.scene.glplot.gl,position:h.position,color:h.lineColor,lineWidth:h.lineWidth||1,dashes:u[0],dashScale:u[1],opacity:t.opacity,connectGaps:t.connectgaps},-1!==this.mode.indexOf("lines")?this.linePlot?this.linePlot.update(e):(this.linePlot=r(e),this.linePlot._trace=this,this.scene.glplot.add(this.linePlot)):this.linePlot&&(this.scene.glplot.remove(this.linePlot),this.linePlot.dispose(),this.linePlot=null);var d=t.opacity;if(t.marker&&t.marker.opacity&&(d*=t.marker.opacity),n={gl:this.scene.glplot.gl,position:h.position,color:h.scatterColor,size:h.scatterSize,glyph:h.scatterMarker,opacity:d,orthographic:!0,lineWidth:h.scatterLineWidth,lineColor:h.scatterLineColor,project:h.project,projectScale:h.projectScale,projectOpacity:h.projectOpacity},-1!==this.mode.indexOf("markers")?this.scatterPlot?this.scatterPlot.update(n):(this.scatterPlot=i(n),this.scatterPlot._trace=this,this.scatterPlot.highlightScale=1,this.scene.glplot.add(this.scatterPlot)):this.scatterPlot&&(this.scene.glplot.remove(this.scatterPlot),this.scatterPlot.dispose(),this.scatterPlot=null),l={gl:this.scene.glplot.gl,position:h.position,glyph:h.text,color:h.textColor,size:h.textSize,angle:h.textAngle,alignment:h.textOffset,font:h.textFont,orthographic:!0,lineWidth:0,project:!1,opacity:t.opacity},this.textLabels=t.hovertext||t.text,-1!==this.mode.indexOf("text")?this.textMarkers?this.textMarkers.update(l):(this.textMarkers=i(l),this.textMarkers._trace=this,this.textMarkers.highlightScale=1,this.scene.glplot.add(this.textMarkers)):this.textMarkers&&(this.scene.glplot.remove(this.textMarkers),this.textMarkers.dispose(),this.textMarkers=null),A={gl:this.scene.glplot.gl,position:h.position,color:h.errorColor,error:h.errorBounds,lineWidth:h.errorLineWidth,capSize:h.errorCapSize,opacity:t.opacity},this.errorBars?h.errorBounds?this.errorBars.update(A):(this.scene.glplot.remove(this.errorBars),this.errorBars.dispose(),this.errorBars=null):h.errorBounds&&(this.errorBars=o(A),this.errorBars._trace=this,this.scene.glplot.add(this.errorBars)),h.delaunayAxis>=0){var f=function(t,e,n){var r,i=(n+1)%3,o=(n+2)%3,a=[],A=[];for(r=0;r<t.length;++r){var l=t[r];!isNaN(l[i])&&isFinite(l[i])&&!isNaN(l[o])&&isFinite(l[o])&&(a.push([l[i],l[o]]),A.push(r))}var c=s(a);for(r=0;r<c.length;++r)for(var u=c[r],g=0;g<u.length;++g)u[g]=A[u[g]];return{positions:t,cells:c,meshColor:e}}(h.position,h.delaunayColor,h.delaunayAxis);f.opacity=t.opacity,this.delaunayMesh?this.delaunayMesh.update(f):(f.gl=c,this.delaunayMesh=a(f),this.delaunayMesh._trace=this,this.scene.glplot.add(this.delaunayMesh))}else this.delaunayMesh&&(this.scene.glplot.remove(this.delaunayMesh),this.delaunayMesh.dispose(),this.delaunayMesh=null)},p.dispose=function(){this.linePlot&&(this.scene.glplot.remove(this.linePlot),this.linePlot.dispose()),this.scatterPlot&&(this.scene.glplot.remove(this.scatterPlot),this.scatterPlot.dispose()),this.errorBars&&(this.scene.glplot.remove(this.errorBars),this.errorBars.dispose()),this.textMarkers&&(this.scene.glplot.remove(this.textMarkers),this.textMarkers.dispose()),this.delaunayMesh&&(this.scene.glplot.remove(this.delaunayMesh),this.delaunayMesh.dispose())},e.exports=function(t,e){var n=new f(t,e.uid);return n.update(e),n}},{"../../constants/gl3d_dashes":692,"../../constants/gl3d_markers":693,"../../lib":719,"../../lib/gl_format_color":716,"../../lib/str2rgbarray":742,"../scatter/make_bubble_size_func":1128,"./calc_errors":1140,"delaunay-triangulate":165,"gl-error3d":247,"gl-line3d":255,"gl-mesh3d":280,"gl-scatter3d":297}],1142:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib"),o=t("../scatter/subtypes"),a=t("../scatter/marker_defaults"),s=t("../scatter/line_defaults"),A=t("../scatter/text_defaults"),l=t("./attributes");e.exports=function(t,e,n,c){function u(n,r){return i.coerce(t,e,l,n,r)}if(function(t,e,n,i){var o=0,a=n("x"),s=n("y"),A=n("z");return r.getComponentMethod("calendars","handleTraceDefaults")(t,e,["x","y","z"],i),a&&s&&A&&(o=Math.min(a.length,s.length,A.length),e._length=e._xlength=e._ylength=e._zlength=o),o}(t,e,u,c)){u("text"),u("hovertext"),u("hovertemplate"),u("mode"),o.hasLines(e)&&(u("connectgaps"),s(t,e,n,c,u)),o.hasMarkers(e)&&a(t,e,n,c,u,{noSelect:!0}),o.hasText(e)&&A(t,e,c,u,{noSelect:!0});var g=(e.line||{}).color,h=(e.marker||{}).color;u("surfaceaxis")>=0&&u("surfacecolor",g||h);for(var d=["x","y","z"],f=0;f<3;++f){var p="projection."+d[f];u(p+".show")&&(u(p+".opacity"),u(p+".scale"))}var I=r.getComponentMethod("errorbars","supplyDefaults");I(t,e,g||h||n,{axis:"z"}),I(t,e,g||h||n,{axis:"y",inherit:"z"}),I(t,e,g||h||n,{axis:"x",inherit:"z"})}else e.visible=!1}},{"../../lib":719,"../../registry":847,"../scatter/line_defaults":1124,"../scatter/marker_defaults":1130,"../scatter/subtypes":1135,"../scatter/text_defaults":1136,"./attributes":1138}],1143:[function(t,e,n){"use strict";e.exports={plot:t("./convert"),attributes:t("./attributes"),markerSymbols:t("../../constants/gl3d_markers"),supplyDefaults:t("./defaults"),colorbar:[{container:"marker",min:"cmin",max:"cmax"},{container:"line",min:"cmin",max:"cmax"}],calc:t("./calc"),moduleType:"trace",name:"scatter3d",basePlotModule:t("../../plots/gl3d"),categories:["gl3d","symbols","showLegend"],meta:{}}},{"../../constants/gl3d_markers":693,"../../plots/gl3d":807,"./attributes":1138,"./calc":1139,"./convert":1141,"./defaults":1142}],1144:[function(t,e,n){"use strict";var r=t("../scatter/attributes"),i=t("../../plots/attributes"),o=t("../../components/fx/hovertemplate_attributes"),a=t("../../components/colorscale/attributes"),s=t("../../lib/extend").extendFlat,A=r.marker,l=r.line,c=A.line;e.exports={carpet:{valType:"string",editType:"calc"},a:{valType:"data_array",editType:"calc"},b:{valType:"data_array",editType:"calc"},mode:s({},r.mode,{dflt:"markers"}),text:s({},r.text,{}),hovertext:s({},r.hovertext,{}),line:{color:l.color,width:l.width,dash:l.dash,shape:s({},l.shape,{values:["linear","spline"]}),smoothing:l.smoothing,editType:"calc"},connectgaps:r.connectgaps,fill:s({},r.fill,{values:["none","toself","tonext"],dflt:"none"}),fillcolor:r.fillcolor,marker:s({symbol:A.symbol,opacity:A.opacity,maxdisplayed:A.maxdisplayed,size:A.size,sizeref:A.sizeref,sizemin:A.sizemin,sizemode:A.sizemode,line:s({width:c.width,editType:"calc"},a("marker.line")),gradient:A.gradient,editType:"calc"},a("marker")),textfont:r.textfont,textposition:r.textposition,selected:r.selected,unselected:r.unselected,hoverinfo:s({},i.hoverinfo,{flags:["a","b","text","name"]}),hoveron:r.hoveron,hovertemplate:o()}},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../scatter/attributes":1112}],1145:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../scatter/colorscale_calc"),o=t("../scatter/arrays_to_calcdata"),a=t("../scatter/calc_selection"),s=t("../scatter/calc").calcMarkerSize,A=t("../carpet/lookup_carpetid");e.exports=function(t,e){var n=e._carpetTrace=A(t,e);if(n&&n.visible&&"legendonly"!==n.visible){var l;e.xaxis=n.xaxis,e.yaxis=n.yaxis;var c,u,g=e._length,h=new Array(g),d=!1;for(l=0;l<g;l++)if(c=e.a[l],u=e.b[l],r(c)&&r(u)){var f=n.ab2xy(+c,+u,!0),p=n.isVisible(+c,+u);p||(d=!0),h[l]={x:f[0],y:f[1],a:c,b:u,vis:p}}else h[l]={x:!1,y:!1};return e._needsCull=d,h[0].carpet=n,h[0].trace=e,s(e,g),i(t,e),o(h,e),a(h,e),h}}},{"../carpet/lookup_carpetid":915,"../scatter/arrays_to_calcdata":1111,"../scatter/calc":1113,"../scatter/calc_selection":1114,"../scatter/colorscale_calc":1115,"fast-isnumeric":225}],1146:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../scatter/constants"),o=t("../scatter/subtypes"),a=t("../scatter/marker_defaults"),s=t("../scatter/line_defaults"),A=t("../scatter/line_shape_defaults"),l=t("../scatter/text_defaults"),c=t("../scatter/fillcolor_defaults"),u=t("./attributes");e.exports=function(t,e,n,g){function h(n,i){return r.coerce(t,e,u,n,i)}h("carpet"),e.xaxis="x",e.yaxis="y";var d=h("a"),f=h("b"),p=Math.min(d.length,f.length);if(p){e._length=p,h("text"),h("hovertext"),h("mode",p<i.PTS_LINESONLY?"lines+markers":"lines"),o.hasLines(e)&&(s(t,e,n,g,h),A(t,e,h),h("connectgaps")),o.hasMarkers(e)&&a(t,e,n,g,h,{gradient:!0}),o.hasText(e)&&l(t,e,g,h);var I=[];(o.hasMarkers(e)||o.hasText(e))&&(h("marker.maxdisplayed"),I.push("points")),h("fill"),"none"!==e.fill&&(c(t,e,n,h),o.hasLines(e)||A(t,e,h)),"tonext"!==e.fill&&"toself"!==e.fill||I.push("fills"),"fills"!==h("hoveron",I.join("+")||"points")&&h("hovertemplate"),r.coerceSelectionMarkerOpacity(e,h)}else e.visible=!1}},{"../../lib":719,"../scatter/constants":1116,"../scatter/fillcolor_defaults":1120,"../scatter/line_defaults":1124,"../scatter/line_shape_defaults":1126,"../scatter/marker_defaults":1130,"../scatter/subtypes":1135,"../scatter/text_defaults":1136,"./attributes":1144}],1147:[function(t,e,n){"use strict";e.exports=function(t,e,n,r,i){var o=r[i];return t.a=o.a,t.b=o.b,t.y=o.y,t}},{}],1148:[function(t,e,n){"use strict";var r=t("../scatter/hover"),i=t("../../lib").fillText;e.exports=function(t,e,n,o){var a=r(t,e,n,o);if(a&&!1!==a[0].index){var s=a[0];if(void 0===s.index){var A=1-s.y0/t.ya._length,l=t.xa._length,c=l*A/2,u=l-c;return s.x0=Math.max(Math.min(s.x0,u),c),s.x1=Math.max(Math.min(s.x1,u),c),a}var g=s.cd[s.index];s.a=g.a,s.b=g.b,s.xLabelVal=void 0,s.yLabelVal=void 0;var h=s.trace,d=h._carpet,f=d.ab2ij([g.a,g.b]),p=Math.floor(f[0]),I=f[0]-p,C=Math.floor(f[1]),m=f[1]-C,y=d.evalxy([],p,C,I,m);s.yLabel=y[1].toFixed(3),delete s.text;var E=[];if(!h.hovertemplate){var v=(g.hi||h.hoverinfo).split("+");-1!==v.indexOf("all")&&(v=["a","b","text"]),-1!==v.indexOf("a")&&B(d.aaxis,g.a),-1!==v.indexOf("b")&&B(d.baxis,g.b),E.push("y: "+s.yLabel),-1!==v.indexOf("text")&&i(g,h,E),s.extraText=E.join("<br>")}return a}function B(t,e){var n;n=t.labelprefix&&t.labelprefix.length>0?t.labelprefix.replace(/ = $/,""):t._hovertitle,E.push(n+": "+e.toFixed(3)+t.labelsuffix)}}},{"../../lib":719,"../scatter/hover":1122}],1149:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:t("../scatter/marker_colorbar"),calc:t("./calc"),plot:t("./plot"),style:t("../scatter/style").style,styleOnSelect:t("../scatter/style").styleOnSelect,hoverPoints:t("./hover"),selectPoints:t("../scatter/select"),eventData:t("./event_data"),moduleType:"trace",name:"scattercarpet",basePlotModule:t("../../plots/cartesian"),categories:["svg","carpet","symbols","showLegend","carpetDependent","zoomScale"],meta:{}}},{"../../plots/cartesian":778,"../scatter/marker_colorbar":1129,"../scatter/select":1132,"../scatter/style":1134,"./attributes":1144,"./calc":1145,"./defaults":1146,"./event_data":1147,"./hover":1148,"./plot":1150}],1150:[function(t,e,n){"use strict";var r=t("../scatter/plot"),i=t("../../plots/cartesian/axes"),o=t("../../components/drawing");e.exports=function(t,e,n,a){var s,A,l,c=n[0][0].carpet,u={xaxis:i.getFromId(t,c.xaxis||"x"),yaxis:i.getFromId(t,c.yaxis||"y"),plot:e.plot};for(r(t,u,n,a),s=0;s<n.length;s++)A=n[s][0].trace,l=a.selectAll("g.trace"+A.uid+" .js-line"),o.setClipUrl(l,n[s][0].carpet._clipPathId,t)}},{"../../components/drawing":614,"../../plots/cartesian/axes":767,"../scatter/plot":1131}],1151:[function(t,e,n){"use strict";var r=t("../../components/fx/hovertemplate_attributes"),i=t("../scatter/attributes"),o=t("../../plots/attributes"),a=t("../../components/colorscale/attributes"),s=t("../../components/drawing/attributes").dash,A=t("../../lib/extend").extendFlat,l=t("../../plot_api/edit_types").overrideAll,c=i.marker,u=i.line,g=c.line;e.exports=l({lon:{valType:"data_array"},lat:{valType:"data_array"},locations:{valType:"data_array"},locationmode:{valType:"enumerated",values:["ISO-3","USA-states","country names"],dflt:"ISO-3"},mode:A({},i.mode,{dflt:"markers"}),text:A({},i.text,{}),hovertext:A({},i.hovertext,{}),textfont:i.textfont,textposition:i.textposition,line:{color:u.color,width:u.width,dash:s},connectgaps:i.connectgaps,marker:A({symbol:c.symbol,opacity:c.opacity,size:c.size,sizeref:c.sizeref,sizemin:c.sizemin,sizemode:c.sizemode,colorbar:c.colorbar,line:A({width:g.width},a("marker.line")),gradient:c.gradient},a("marker")),fill:{valType:"enumerated",values:["none","toself"],dflt:"none"},fillcolor:i.fillcolor,selected:i.selected,unselected:i.unselected,hoverinfo:A({},o.hoverinfo,{flags:["lon","lat","location","text","name"]}),hovertemplate:r()},"calc","nested")},{"../../components/colorscale/attributes":600,"../../components/drawing/attributes":613,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plots/attributes":764,"../scatter/attributes":1112}],1152:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../constants/numerical").BADNUM,o=t("../scatter/colorscale_calc"),a=t("../scatter/arrays_to_calcdata"),s=t("../scatter/calc_selection"),A=t("../../lib")._;e.exports=function(t,e){for(var n=Array.isArray(e.locations),l=n?e.locations.length:e._length,c=new Array(l),u=0;u<l;u++){var g=c[u]={};if(n){var h=e.locations[u];g.loc="string"==typeof h?h:null}else{var d=e.lon[u],f=e.lat[u];r(d)&&r(f)?g.lonlat=[+d,+f]:g.lonlat=[i,i]}}return a(c,e),o(t,e),s(c,e),l&&(c[0].t={labels:{lat:A(t,"lat:")+" ",lon:A(t,"lon:")+" "}}),c}},{"../../constants/numerical":695,"../../lib":719,"../scatter/arrays_to_calcdata":1111,"../scatter/calc_selection":1114,"../scatter/colorscale_calc":1115,"fast-isnumeric":225}],1153:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../scatter/subtypes"),o=t("../scatter/marker_defaults"),a=t("../scatter/line_defaults"),s=t("../scatter/text_defaults"),A=t("../scatter/fillcolor_defaults"),l=t("./attributes");e.exports=function(t,e,n,c){function u(n,i){return r.coerce(t,e,l,n,i)}!function(t,e,n){var r,i,o=0,a=n("locations");return a?(n("locationmode"),o=a.length):(r=n("lon")||[],i=n("lat")||[],o=Math.min(r.length,i.length),e._length=o,o)}(0,e,u)?e.visible=!1:(u("text"),u("hovertext"),u("hovertemplate"),u("mode"),i.hasLines(e)&&(a(t,e,n,c,u),u("connectgaps")),i.hasMarkers(e)&&o(t,e,n,c,u,{gradient:!0}),i.hasText(e)&&s(t,e,c,u),u("fill"),"none"!==e.fill&&A(t,e,n,u),r.coerceSelectionMarkerOpacity(e,u))}},{"../../lib":719,"../scatter/fillcolor_defaults":1120,"../scatter/line_defaults":1124,"../scatter/marker_defaults":1130,"../scatter/subtypes":1135,"../scatter/text_defaults":1136,"./attributes":1151}],1154:[function(t,e,n){"use strict";e.exports=function(t,e){return t.lon=e.lon,t.lat=e.lat,t.location=e.loc?e.loc:null,t}},{}],1155:[function(t,e,n){"use strict";var r=t("../../components/fx"),i=t("../../plots/cartesian/axes"),o=t("../../constants/numerical").BADNUM,a=t("../scatter/get_trace_color"),s=t("../../lib").fillText,A=t("./attributes");e.exports=function(t,e,n){var l=t.cd,c=l[0].trace,u=t.xa,g=t.ya,h=t.subplot,d=h.projection.isLonLatOverEdges,f=h.project;if(r.getClosest(l,function(t){var r=t.lonlat;if(r[0]===o)return 1/0;if(d(r))return 1/0;var i=f(r),a=f([e,n]),s=Math.abs(i[0]-a[0]),A=Math.abs(i[1]-a[1]),l=Math.max(3,t.mrc||0);return Math.max(Math.sqrt(s*s+A*A)-l,1-3/l)},t),!1!==t.index){var p=l[t.index],I=p.lonlat,C=[u.c2p(I),g.c2p(I)],m=p.mrc||1;t.x0=C[0]-m,t.x1=C[0]+m,t.y0=C[1]-m,t.y1=C[1]+m,t.loc=p.loc,t.lon=I[0],t.lat=I[1];var y=h.mockAxis;return t.lonLabel=i.tickText(y,y.c2l(t.lon),"hover").text,t.latLabel=i.tickText(y,y.c2l(t.lat),"hover").text,t.color=a(c,p),t.extraText=function(t,e,n,r){if(!t.hovertemplate){var i=e.hi||t.hoverinfo,o="all"===i?A.hoverinfo.flags:i.split("+"),a=-1!==o.indexOf("location")&&Array.isArray(t.locations),l=-1!==o.indexOf("lon"),c=-1!==o.indexOf("lat"),u=-1!==o.indexOf("text"),g=[];return a?g.push(e.loc):l&&c?g.push("("+h(n.lonLabel)+", "+h(n.latLabel)+")"):l?g.push(r.lon+h(n.lonLabel)):c&&g.push(r.lat+h(n.latLabel)),u&&s(e,t,g),g.join("<br>")}function h(t){return t+"°"}}(c,p,t,l[0].t.labels),t.hovertemplate=c.hovertemplate,[t]}}},{"../../components/fx":632,"../../constants/numerical":695,"../../lib":719,"../../plots/cartesian/axes":767,"../scatter/get_trace_color":1121,"./attributes":1151}],1156:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:t("../scatter/marker_colorbar"),calc:t("./calc"),plot:t("./plot"),style:t("./style"),styleOnSelect:t("../scatter/style").styleOnSelect,hoverPoints:t("./hover"),eventData:t("./event_data"),selectPoints:t("./select"),moduleType:"trace",name:"scattergeo",basePlotModule:t("../../plots/geo"),categories:["geo","symbols","showLegend","scatter-like"],meta:{}}},{"../../plots/geo":797,"../scatter/marker_colorbar":1129,"../scatter/style":1134,"./attributes":1151,"./calc":1152,"./defaults":1153,"./event_data":1154,"./hover":1155,"./plot":1157,"./select":1158,"./style":1159}],1157:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../lib"),o=t("../../constants/numerical").BADNUM,a=t("../../lib/topojson_utils").getTopojsonFeatures,s=t("../../lib/geo_location_utils").locationToFeature,A=t("../../lib/geojson_utils"),l=t("../scatter/subtypes"),c=t("./style");function u(t,e){var n=t[0].trace;if(Array.isArray(n.locations))for(var r=a(n,e),i=n.locationmode,A=0;A<t.length;A++){var l=t[A],c=s(i,l.loc,r);l.lonlat=c?c.properties.ct:[o,o]}}e.exports=function(t,e,n){for(var a=0;a<n.length;a++)u(n[a],e.topojson);function s(t,e){t.lonlat[0]===o&&r.select(e).remove()}var g=e.layers.frontplot.select(".scatterlayer"),h=i.makeTraceGroups(g,n,"trace scattergeo");h.selectAll("*").remove(),h.each(function(e){var n=r.select(this),o=e[0].trace;if(l.hasLines(o)||"none"!==o.fill){var a=A.calcTraceToLineCoords(e),u="none"!==o.fill?A.makePolygon(a):A.makeLine(a);n.selectAll("path.js-line").data([{geojson:u,trace:o}]).enter().append("path").classed("js-line",!0).style("stroke-miterlimit",2)}l.hasMarkers(o)&&n.selectAll("path.point").data(i.identity).enter().append("path").classed("point",!0).each(function(t){s(t,this)}),l.hasText(o)&&n.selectAll("g").data(i.identity).enter().append("g").append("text").each(function(t){s(t,this)}),c(t,e)})}},{"../../constants/numerical":695,"../../lib":719,"../../lib/geo_location_utils":713,"../../lib/geojson_utils":714,"../../lib/topojson_utils":746,"../scatter/subtypes":1135,"./style":1159,d3:163}],1158:[function(t,e,n){"use strict";var r=t("../scatter/subtypes"),i=t("../../constants/numerical").BADNUM;e.exports=function(t,e){var n,o,a,s,A,l=t.cd,c=t.xaxis,u=t.yaxis,g=[],h=l[0].trace;if(!r.hasMarkers(h)&&!r.hasText(h))return[];if(!1===e)for(A=0;A<l.length;A++)l[A].selected=0;else for(A=0;A<l.length;A++)(o=(n=l[A]).lonlat)[0]!==i&&(a=c.c2p(o),s=u.c2p(o),e.contains([a,s],null,A,t)?(g.push({pointNumber:A,lon:o[0],lat:o[1]}),n.selected=1):n.selected=0);return g}},{"../../constants/numerical":695,"../scatter/subtypes":1135}],1159:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/drawing"),o=t("../../components/color"),a=t("../scatter/style"),s=a.stylePoints,A=a.styleText;e.exports=function(t,e){e&&function(t,e){var n=e[0].trace,a=e[0].node3;a.style("opacity",e[0].trace.opacity),s(a,n,t),A(a,n,t),a.selectAll("path.js-line").style("fill","none").each(function(t){var e=r.select(this),n=t.trace,a=n.line||{};e.call(o.stroke,a.color).call(i.dashLine,a.dash||"",a.width||0),"none"!==n.fill&&e.call(o.fill,n.fillcolor)})}(t,e)}},{"../../components/color":593,"../../components/drawing":614,"../scatter/style":1134,d3:163}],1160:[function(t,e,n){"use strict";var r=t("../../plots/attributes"),i=t("../scatter/attributes"),o=t("../../components/colorscale/attributes"),a=t("../../lib/extend").extendFlat,s=t("../../plot_api/edit_types").overrideAll,A=t("./constants").DASHES,l=i.line,c=i.marker,u=c.line,g=e.exports=s({x:i.x,x0:i.x0,dx:i.dx,y:i.y,y0:i.y0,dy:i.dy,text:i.text,hovertext:i.hovertext,textposition:i.textposition,textfont:i.textfont,mode:{valType:"flaglist",flags:["lines","markers","text"],extras:["none"]},line:{color:l.color,width:l.width,shape:{valType:"enumerated",values:["linear","hv","vh","hvh","vhv"],dflt:"linear",editType:"plot"},dash:{valType:"enumerated",values:Object.keys(A),dflt:"solid"}},marker:a({},o("marker"),{symbol:c.symbol,size:c.size,sizeref:c.sizeref,sizemin:c.sizemin,sizemode:c.sizemode,opacity:c.opacity,colorbar:c.colorbar,line:a({},o("marker.line"),{width:u.width})}),connectgaps:i.connectgaps,fill:a({},i.fill,{dflt:"none"}),fillcolor:i.fillcolor,selected:{marker:i.selected.marker,textfont:i.selected.textfont},unselected:{marker:i.unselected.marker,textfont:i.unselected.textfont},opacity:r.opacity},"calc","nested");g.x.editType=g.y.editType=g.x0.editType=g.y0.editType="calc+clearAxisTypes",g.hovertemplate=i.hovertemplate},{"../../components/colorscale/attributes":600,"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plots/attributes":764,"../scatter/attributes":1112,"./constants":1162}],1161:[function(t,e,n){"use strict";var r=t("point-cluster"),i=t("../../lib"),o=t("../../plots/cartesian/axis_ids"),a=t("../../plots/cartesian/autorange").findExtremes,s=t("../scatter/calc"),A=s.calcMarkerSize,l=s.calcAxisExpansion,c=s.setFirstScatter,u=t("../scatter/colorscale_calc"),g=t("./convert"),h=t("./scene_update"),d=t("../../constants/numerical").BADNUM,f=t("./constants").TOO_MANY_POINTS;function p(t,e,n){var r=t._extremes[e._id],i=a(e,n._bnds,{padded:!0});r.min=r.min.concat(i.min),r.max=r.max.concat(i.max)}e.exports=function(t,e){var n,a,s,I=t._fullLayout,C=o.getFromId(t,e.xaxis),m=o.getFromId(t,e.yaxis),y=I._plots[e.xaxis+e.yaxis],E=e._length,v=E>=f,B=2*E,b={},Q=C.makeCalcdata(e,"x"),w=m.makeCalcdata(e,"y"),x=new Array(B);for(n=0;n<E;n++)a=Q[n],s=w[n],x[2*n]=a===d?NaN:a,x[2*n+1]=s===d?NaN:s;if("log"===C.type)for(n=0;n<B;n+=2)x[n]=C.c2l(x[n]);if("log"===m.type)for(n=1;n<B;n+=2)x[n]=m.c2l(x[n]);if(v&&"log"!==C.type&&"log"!==m.type)b.tree=r(x);else{var _=b.ids=new Array(E);for(n=0;n<E;n++)_[n]=n}u(t,e);var D,M=function(t,e,n,r,o,a){var s=g.style(t,n);if(s.marker&&(s.marker.positions=r),s.line&&r.length>1&&i.extendFlat(s.line,g.linePositions(t,n,r)),s.errorX||s.errorY){var A=g.errorBarPositions(t,n,r,o,a);s.errorX&&i.extendFlat(s.errorX,A.x),s.errorY&&i.extendFlat(s.errorY,A.y)}return s.text&&(i.extendFlat(s.text,{positions:r},g.textPosition(t,n,s.text,s.marker)),i.extendFlat(s.textSel,{positions:r},g.textPosition(t,n,s.text,s.markerSel)),i.extendFlat(s.textUnsel,{positions:r},g.textPosition(t,n,s.text,s.markerUnsel))),s}(t,0,e,x,Q,w),k=h(t,y);return c(I,e),v?M.marker&&(D=2*(M.marker.sizeAvg||Math.max(M.marker.size,3))):D=A(e,E),l(t,e,C,m,Q,w,D),M.errorX&&p(e,C,M.errorX),M.errorY&&p(e,m,M.errorY),M.fill&&!k.fill2d&&(k.fill2d=!0),M.marker&&!k.scatter2d&&(k.scatter2d=!0),M.line&&!k.line2d&&(k.line2d=!0),!M.errorX&&!M.errorY||k.error2d||(k.error2d=!0),M.text&&!k.glText&&(k.glText=!0),M.marker&&(M.marker.snap=b.tree||f),k.lineOptions.push(M.line),k.errorXOptions.push(M.errorX),k.errorYOptions.push(M.errorY),k.fillOptions.push(M.fill),k.markerOptions.push(M.marker),k.markerSelectedOptions.push(M.markerSel),k.markerUnselectedOptions.push(M.markerUnsel),k.textOptions.push(M.text),k.textSelectedOptions.push(M.textSel),k.textUnselectedOptions.push(M.textUnsel),k.selectBatch.push([]),k.unselectBatch.push([]),b._scene=k,b.index=k.count,b.x=Q,b.y=w,b.positions=x,k.count++,[{x:!1,y:!1,t:b,trace:e}]}},{"../../constants/numerical":695,"../../lib":719,"../../plots/cartesian/autorange":766,"../../plots/cartesian/axis_ids":770,"../scatter/calc":1113,"../scatter/colorscale_calc":1115,"./constants":1162,"./convert":1163,"./scene_update":1169,"point-cluster":469}],1162:[function(t,e,n){"use strict";e.exports={TOO_MANY_POINTS:1e5,SYMBOL_SDF_SIZE:200,SYMBOL_SIZE:20,SYMBOL_STROKE:1,DOT_RE:/-dot/,OPEN_RE:/-open/,DASHES:{solid:[1],dot:[1,1],dash:[4,1],longdash:[8,1],dashdot:[4,1,1,1],longdashdot:[8,1,1,1]}}},{}],1163:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("svg-path-sdf"),o=t("color-normalize"),a=t("../../registry"),s=t("../../lib"),A=t("../../components/drawing"),l=t("../../plots/cartesian/axis_ids"),c=t("../../lib/gl_format_color").formatColor,u=t("../scatter/subtypes"),g=t("../scatter/make_bubble_size_func"),h=t("./constants"),d=t("../../constants/interactions").DESELECTDIM,f={start:1,left:1,end:-1,right:-1,middle:0,center:0,bottom:1,top:-1};function p(t){var e,n=t._length,i=t.textfont,o=t.textposition,a=Array.isArray(o)?o:[o],s=i.color,A=i.size,l=i.family,c={};for(c.text=t.text,c.opacity=t.opacity,c.font={},c.align=[],c.baseline=[],e=0;e<a.length;e++){var u=a[e].split(/\s+/);switch(u[1]){case"left":c.align.push("right");break;case"right":c.align.push("left");break;default:c.align.push(u[1])}switch(u[0]){case"top":c.baseline.push("bottom");break;case"bottom":c.baseline.push("top");break;default:c.baseline.push(u[0])}}if(Array.isArray(s))for(c.color=new Array(n),e=0;e<n;e++)c.color[e]=s[e];else c.color=s;if(Array.isArray(A)||Array.isArray(l))for(c.font=new Array(n),e=0;e<n;e++){var g=c.font[e]={};g.size=Array.isArray(A)?r(A[e])?A[e]:0:A,g.family=Array.isArray(l)?l[e]:l}else c.font={size:A,family:l};return c}function I(t){var e,n,r=t._length,i=t.marker,a={},A=Array.isArray(i.symbol),l=s.isArrayOrTypedArray(i.color),u=s.isArrayOrTypedArray(i.line.color),d=s.isArrayOrTypedArray(i.opacity),f=s.isArrayOrTypedArray(i.size),p=s.isArrayOrTypedArray(i.line.width);if(A||(n=h.OPEN_RE.test(i.symbol)),A||l||u||d){a.colors=new Array(r),a.borderColors=new Array(r);var I=c(i,i.opacity,r),C=c(i.line,i.opacity,r);if(!Array.isArray(C[0])){var m=C;for(C=Array(r),e=0;e<r;e++)C[e]=m}if(!Array.isArray(I[0])){var y=I;for(I=Array(r),e=0;e<r;e++)I[e]=y}for(a.colors=I,a.borderColors=C,e=0;e<r;e++){if(A){var E=i.symbol[e];n=h.OPEN_RE.test(E)}n&&(C[e]=I[e].slice(),I[e]=I[e].slice(),I[e][3]=0)}a.opacity=t.opacity}else n?(a.color=o(i.color,"uint8"),a.color[3]=0,a.borderColor=o(i.color,"uint8")):(a.color=o(i.color,"uint8"),a.borderColor=o(i.line.color,"uint8")),a.opacity=t.opacity*i.opacity;if(A)for(a.markers=new Array(r),e=0;e<r;e++)a.markers[e]=w(i.symbol[e]);else a.marker=w(i.symbol);var v,B=g(t);if(f||p){var b,Q=a.sizes=new Array(r),x=a.borderSizes=new Array(r),_=0;if(f){for(e=0;e<r;e++)Q[e]=B(i.size[e]),_+=Q[e];b=_/r}else for(v=B(i.size),e=0;e<r;e++)Q[e]=v;if(p)for(e=0;e<r;e++)x[e]=i.line.width[e]/2;else for(v=i.line.width/2,e=0;e<r;e++)x[e]=v;a.sizeAvg=b}else a.size=B(i&&i.size||10),a.borderSizes=B(i.line.width);return a}function C(t,e){var n=t.marker,r={};return e?(e.marker&&e.marker.symbol?r=I(s.extendFlat({},n,e.marker)):e.marker&&(e.marker.size&&(r.size=e.marker.size/2),e.marker.color&&(r.colors=e.marker.color),void 0!==e.marker.opacity&&(r.opacity=e.marker.opacity)),r):r}function m(t,e){var n={};if(!e)return n;if(e.textfont){var r={opacity:1,text:t.text,textposition:t.textposition,textfont:s.extendFlat({},t.textfont)};e.textfont&&s.extendFlat(r.textfont,e.textfont),n=p(r)}return n}function y(t,e){var n={capSize:2*e.width,lineWidth:e.thickness,color:e.color};return e.copy_ystyle&&(n=t.error_y),n}var E=h.SYMBOL_SDF_SIZE,v=h.SYMBOL_SIZE,B=h.SYMBOL_STROKE,b={},Q=A.symbolFuncs[0](.05*v);function w(t){if("circle"===t)return null;var e,n,r=A.symbolNumber(t),o=A.symbolFuncs[r%100],a=!!A.symbolNoDot[r%100],s=!!A.symbolNoFill[r%100],l=h.DOT_RE.test(t);return b[t]?b[t]:(e=l&&!a?o(1.1*v)+Q:o(v),n=i(e,{w:E,h:E,viewBox:[-v,-v,v,v],stroke:s?B:-B}),b[t]=n,n||null)}e.exports={style:function(t,e){var n,r={marker:void 0,markerSel:void 0,markerUnsel:void 0,line:void 0,fill:void 0,errorX:void 0,errorY:void 0,text:void 0,textSel:void 0,textUnsel:void 0};if(!0!==e.visible)return r;if(u.hasText(e)&&(r.text=p(e),r.textSel=m(e,e.selected),r.textUnsel=m(e,e.unselected)),u.hasMarkers(e)&&(r.marker=I(e),r.markerSel=C(e,e.selected),r.markerUnsel=C(e,e.unselected),!e.unselected&&Array.isArray(e.marker.opacity))){var i=e.marker.opacity;for(r.markerUnsel.opacity=new Array(i.length),n=0;n<i.length;n++)r.markerUnsel.opacity[n]=d*i[n]}if(u.hasLines(e)){r.line={overlay:!0,thickness:e.line.width,color:e.line.color,opacity:e.opacity};var o=(h.DASHES[e.line.dash]||[1]).slice();for(n=0;n<o.length;++n)o[n]*=e.line.width;r.line.dashes=o}return e.error_x&&e.error_x.visible&&(r.errorX=y(e,e.error_x)),e.error_y&&e.error_y.visible&&(r.errorY=y(e,e.error_y)),e.fill&&"none"!==e.fill&&(r.fill={closed:!0,fill:e.fillcolor,thickness:0}),r},markerStyle:I,markerSelection:C,linePositions:function(t,e,n){var r,i,o=n.length,a=o/2;if(u.hasLines(e)&&a)if("hv"===e.line.shape){for(r=[],i=0;i<a-1;i++)isNaN(n[2*i])||isNaN(n[2*i+1])?r.push(NaN,NaN,NaN,NaN):(r.push(n[2*i],n[2*i+1]),isNaN(n[2*i+2])||isNaN(n[2*i+3])?r.push(NaN,NaN):r.push(n[2*i+2],n[2*i+1]));r.push(n[o-2],n[o-1])}else if("hvh"===e.line.shape){for(r=[],i=0;i<a-1;i++)if(isNaN(n[2*i])||isNaN(n[2*i+1])||isNaN(n[2*i+2])||isNaN(n[2*i+3]))isNaN(n[2*i])||isNaN(n[2*i+1])?r.push(NaN,NaN):r.push(n[2*i],n[2*i+1]),r.push(NaN,NaN);else{var s=(n[2*i]+n[2*i+2])/2;r.push(n[2*i],n[2*i+1],s,n[2*i+1],s,n[2*i+3])}r.push(n[o-2],n[o-1])}else if("vhv"===e.line.shape){for(r=[],i=0;i<a-1;i++)if(isNaN(n[2*i])||isNaN(n[2*i+1])||isNaN(n[2*i+2])||isNaN(n[2*i+3]))isNaN(n[2*i])||isNaN(n[2*i+1])?r.push(NaN,NaN):r.push(n[2*i],n[2*i+1]),r.push(NaN,NaN);else{var A=(n[2*i+1]+n[2*i+3])/2;r.push(n[2*i],n[2*i+1],n[2*i],A,n[2*i+2],A)}r.push(n[o-2],n[o-1])}else if("vh"===e.line.shape){for(r=[],i=0;i<a-1;i++)isNaN(n[2*i])||isNaN(n[2*i+1])?r.push(NaN,NaN,NaN,NaN):(r.push(n[2*i],n[2*i+1]),isNaN(n[2*i+2])||isNaN(n[2*i+3])?r.push(NaN,NaN):r.push(n[2*i],n[2*i+3]));r.push(n[o-2],n[o-1])}else r=n;var l=!1;for(i=0;i<r.length;i++)if(isNaN(r[i])){l=!0;break}var c=l||r.length>h.TOO_MANY_POINTS?"rect":u.hasMarkers(e)?"rect":"round";if(l&&e.connectgaps){var g=r[0],d=r[1];for(i=0;i<r.length;i+=2)isNaN(r[i])||isNaN(r[i+1])?(r[i]=g,r[i+1]=d):(g=r[i],d=r[i+1])}return{join:c,positions:r}},errorBarPositions:function(t,e,n,i,o){var s=a.getComponentMethod("errorbars","makeComputeError"),A=l.getFromId(t,e.xaxis),c=l.getFromId(t,e.yaxis),u=n.length/2,g={};function h(t,i){var o=i._id.charAt(0),a=e["error_"+o];if(a&&a.visible&&("linear"===i.type||"log"===i.type)){for(var A=s(a),l={x:0,y:1}[o],c={x:[0,1,2,3],y:[2,3,0,1]}[o],h=new Float64Array(4*u),d=1/0,f=-1/0,p=0,I=0;p<u;p++,I+=4){var C=t[p];if(r(C)){var m=n[2*p+l],y=A(C,p),E=y[0],v=y[1];if(r(E)&&r(v)){var B=C-E,b=C+v;h[I+c[0]]=m-i.c2l(B),h[I+c[1]]=i.c2l(b)-m,h[I+c[2]]=0,h[I+c[3]]=0,d=Math.min(d,C-E),f=Math.max(f,C+v)}}}g[o]={positions:n,errors:h,_bnds:[d,f]}}}return h(i,A),h(o,c),g},textPosition:function(t,e,n,r){var i,o=e._length,a={};if(u.hasMarkers(e)){var s=n.font,A=n.align,l=n.baseline;for(a.offset=new Array(o),i=0;i<o;i++){var c=r.sizes?r.sizes[i]:r.size,g=Array.isArray(s)?s[i].size:s.size,h=Array.isArray(A)?A.length>1?A[i]:A[0]:A,d=Array.isArray(l)?l.length>1?l[i]:l[0]:l,p=f[h],I=f[d],C=c?c/.8+1:0,m=-I*C-.5*I;a.offset[i]=[p*C/g,m/g]}}return a}}},{"../../components/drawing":614,"../../constants/interactions":694,"../../lib":719,"../../lib/gl_format_color":716,"../../plots/cartesian/axis_ids":770,"../../registry":847,"../scatter/make_bubble_size_func":1128,"../scatter/subtypes":1135,"./constants":1162,"color-normalize":120,"fast-isnumeric":225,"svg-path-sdf":535}],1164:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../registry"),o=t("./attributes"),a=t("../scatter/constants"),s=t("../scatter/subtypes"),A=t("../scatter/xy_defaults"),l=t("../scatter/marker_defaults"),c=t("../scatter/line_defaults"),u=t("../scatter/fillcolor_defaults"),g=t("../scatter/text_defaults");e.exports=function(t,e,n,h){function d(n,i){return r.coerce(t,e,o,n,i)}var f=!!t.marker&&/-open/.test(t.marker.symbol),p=s.isBubble(t),I=A(t,e,h,d);if(I){var C=I<a.PTS_LINESONLY?"lines+markers":"lines";d("text"),d("hovertext"),d("hovertemplate"),d("mode",C),s.hasLines(e)&&(d("connectgaps"),c(t,e,n,h,d),d("line.shape")),s.hasMarkers(e)&&(l(t,e,n,h,d),d("marker.line.width",f||p?1:0)),s.hasText(e)&&g(t,e,h,d);var m=(e.line||{}).color,y=(e.marker||{}).color;d("fill"),"none"!==e.fill&&u(t,e,n,d);var E=i.getComponentMethod("errorbars","supplyDefaults");E(t,e,m||y||n,{axis:"y"}),E(t,e,m||y||n,{axis:"x",inherit:"y"}),r.coerceSelectionMarkerOpacity(e,d)}else e.visible=!1}},{"../../lib":719,"../../registry":847,"../scatter/constants":1116,"../scatter/fillcolor_defaults":1120,"../scatter/line_defaults":1124,"../scatter/marker_defaults":1130,"../scatter/subtypes":1135,"../scatter/text_defaults":1136,"../scatter/xy_defaults":1137,"./attributes":1160}],1165:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/color"),o=t("../../constants/interactions").DESELECTDIM;e.exports={styleTextSelection:function(t){var e,n,a=t[0],s=a.trace,A=a.t,l=A._scene,c=A.index,u=l.selectBatch[c],g=l.unselectBatch[c],h=l.textOptions[c],d=l.textSelectedOptions[c]||{},f=l.textUnselectedOptions[c]||{},p=r.extendFlat({},h);if(u.length||g.length){var I=d.color,C=f.color,m=h.color,y=Array.isArray(m);for(p.color=new Array(s._length),e=0;e<u.length;e++)n=u[e],p.color[n]=I||(y?m[n]:m);for(e=0;e<g.length;e++){n=g[e];var E=y?m[n]:m;p.color[n]=C||(I?E:i.addOpacity(E,o))}}l.glText[c].update(p)}}},{"../../components/color":593,"../../constants/interactions":694,"../../lib":719}],1166:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib"),o=t("../scatter/get_trace_color");function a(t,e,n,a){var s=t.xa,A=t.ya,l=t.distance,c=t.dxy,u=t.index,g={pointNumber:u,x:e[u],y:n[u]};g.tx=Array.isArray(a.text)?a.text[u]:a.text,g.htx=Array.isArray(a.hovertext)?a.hovertext[u]:a.hovertext,g.data=Array.isArray(a.customdata)?a.customdata[u]:a.customdata,g.tp=Array.isArray(a.textposition)?a.textposition[u]:a.textposition;var h=a.textfont;h&&(g.ts=Array.isArray(h.size)?h.size[u]:h.size,g.tc=Array.isArray(h.color)?h.color[u]:h.color,g.tf=Array.isArray(h.family)?h.family[u]:h.family);var d=a.marker;d&&(g.ms=i.isArrayOrTypedArray(d.size)?d.size[u]:d.size,g.mo=i.isArrayOrTypedArray(d.opacity)?d.opacity[u]:d.opacity,g.mx=Array.isArray(d.symbol)?d.symbol[u]:d.symbol,g.mc=i.isArrayOrTypedArray(d.color)?d.color[u]:d.color);var f=d&&d.line;f&&(g.mlc=Array.isArray(f.color)?f.color[u]:f.color,g.mlw=i.isArrayOrTypedArray(f.width)?f.width[u]:f.width);var p=d&&d.gradient;p&&"none"!==p.type&&(g.mgt=Array.isArray(p.type)?p.type[u]:p.type,g.mgc=Array.isArray(p.color)?p.color[u]:p.color);var I=s.c2p(g.x,!0),C=A.c2p(g.y,!0),m=g.mrc||1,y=a.hoverlabel;y&&(g.hbg=Array.isArray(y.bgcolor)?y.bgcolor[u]:y.bgcolor,g.hbc=Array.isArray(y.bordercolor)?y.bordercolor[u]:y.bordercolor,g.hts=Array.isArray(y.font.size)?y.font.size[u]:y.font.size,g.htc=Array.isArray(y.font.color)?y.font.color[u]:y.font.color,g.htf=Array.isArray(y.font.family)?y.font.family[u]:y.font.family,g.hnl=Array.isArray(y.namelength)?y.namelength[u]:y.namelength);var E=a.hoverinfo;E&&(g.hi=Array.isArray(E)?E[u]:E);var v=a.hovertemplate;v&&(g.ht=Array.isArray(v)?v[u]:v);var B={};return B[t.index]=g,i.extendFlat(t,{color:o(a,g),x0:I-m,x1:I+m,xLabelVal:g.x,y0:C-m,y1:C+m,yLabelVal:g.y,cd:B,distance:l,spikeDistance:c,hovertemplate:g.ht}),g.htx?t.text=g.htx:g.tx?t.text=g.tx:a.text&&(t.text=a.text),i.fillText(g,a,t),r.getComponentMethod("errorbars","hoverInfo")(g,a,t),t}e.exports={hoverPoints:function(t,e,n,r){var i,o,s,A,l,c,u,g,h,d=t.cd,f=d[0].t,p=d[0].trace,I=t.xa,C=t.ya,m=f.x,y=f.y,E=I.c2p(e),v=C.c2p(n),B=t.distance;if(f.tree){var b=I.p2c(E-B),Q=I.p2c(E+B),w=C.p2c(v-B),x=C.p2c(v+B);i="x"===r?f.tree.range(Math.min(b,Q),Math.min(C._rl[0],C._rl[1]),Math.max(b,Q),Math.max(C._rl[0],C._rl[1])):f.tree.range(Math.min(b,Q),Math.min(w,x),Math.max(b,Q),Math.max(w,x))}else{if(!f.ids)return[t];i=f.ids}var _=B;if("x"===r)for(l=0;l<i.length;l++)s=m[i[l]],(c=Math.abs(I.c2p(s)-E))<_&&(_=c,u=C.c2p(y[i[l]])-v,h=Math.sqrt(c*c+u*u),o=i[l]);else for(l=i.length-1;l>-1;l--)s=m[i[l]],A=y[i[l]],c=I.c2p(s)-E,u=C.c2p(A)-v,(g=Math.sqrt(c*c+u*u))<_&&(_=h=g,o=i[l]);return t.index=o,t.distance=_,t.dxy=h,void 0===o?[t]:(a(t,m,y,p),[t])},calcHover:a}},{"../../lib":719,"../../registry":847,"../scatter/get_trace_color":1121}],1167:[function(t,e,n){"use strict";var r=t("./hover");e.exports={moduleType:"trace",name:"scattergl",basePlotModule:t("../../plots/cartesian"),categories:["gl","regl","cartesian","symbols","errorBarsOK","showLegend","scatter-like"],attributes:t("./attributes"),supplyDefaults:t("./defaults"),crossTraceDefaults:t("../scatter/cross_trace_defaults"),colorbar:t("../scatter/marker_colorbar"),calc:t("./calc"),plot:t("./plot"),hoverPoints:r.hoverPoints,selectPoints:t("./select"),meta:{}}},{"../../plots/cartesian":778,"../scatter/cross_trace_defaults":1118,"../scatter/marker_colorbar":1129,"./attributes":1160,"./calc":1161,"./defaults":1164,"./hover":1166,"./plot":1168,"./select":1170}],1168:[function(t,e,n){"use strict";var r=t("regl-scatter2d"),i=t("regl-line2d"),o=t("regl-error2d"),a=t("gl-text"),s=t("../../lib"),A=t("../../lib/prepare_regl"),l=t("../scatter/subtypes"),c=t("../scatter/link_traces"),u=t("./edit_style").styleTextSelection;e.exports=function(t,e,n){if(n.length){var g,h,d=t._fullLayout,f=e._scene,p=e.xaxis,I=e.yaxis;if(f)if(A(t,["ANGLE_instanced_arrays","OES_element_index_uint"])){var C=f.count,m=d._glcanvas.data()[0].regl;if(c(t,e,n),f.dirty){if(!0===f.error2d&&(f.error2d=o(m)),!0===f.line2d&&(f.line2d=i(m)),!0===f.scatter2d&&(f.scatter2d=r(m)),!0===f.fill2d&&(f.fill2d=i(m)),!0===f.glText)for(f.glText=new Array(C),g=0;g<C;g++)f.glText[g]=new a(m);if(f.glText){if(C>f.glText.length){var y=C-f.glText.length;for(g=0;g<y;g++)f.glText.push(new a(m))}else if(C<f.glText.length){var E=f.glText.length-C;f.glText.splice(C,E).forEach(function(t){t.destroy()})}for(g=0;g<C;g++)f.glText[g].update(f.textOptions[g])}if(f.line2d&&(f.line2d.update(f.lineOptions),f.lineOptions=f.lineOptions.map(function(t){if(t&&t.positions){for(var e=t.positions,n=0;n<e.length&&(isNaN(e[n])||isNaN(e[n+1]));)n+=2;for(var r=e.length-2;r>n&&(isNaN(e[r])||isNaN(e[r+1]));)r-=2;t.positions=e.slice(n,r+2)}return t}),f.line2d.update(f.lineOptions)),f.error2d){var v=(f.errorXOptions||[]).concat(f.errorYOptions||[]);f.error2d.update(v)}f.scatter2d&&f.scatter2d.update(f.markerOptions),f.fillOrder=s.repeat(null,C),f.fill2d&&(f.fillOptions=f.fillOptions.map(function(t,e){var r=n[e];if(t&&r&&r[0]&&r[0].trace){var i,o,a=r[0],s=a.trace,A=a.t,l=f.lineOptions[e],c=[];s._ownfill&&c.push(e),s._nexttrace&&c.push(e+1),c.length&&(f.fillOrder[e]=c);var u,g,h=[],d=l&&l.positions||A.positions;if("tozeroy"===s.fill){for(u=0;u<d.length&&isNaN(d[u+1]);)u+=2;for(g=d.length-2;g>u&&isNaN(d[g+1]);)g-=2;0!==d[u+1]&&(h=[d[u],0]),h=h.concat(d.slice(u,g+2)),0!==d[g+1]&&(h=h.concat([d[g],0]))}else if("tozerox"===s.fill){for(u=0;u<d.length&&isNaN(d[u]);)u+=2;for(g=d.length-2;g>u&&isNaN(d[g]);)g-=2;0!==d[u]&&(h=[0,d[u+1]]),h=h.concat(d.slice(u,g+2)),0!==d[g]&&(h=h.concat([0,d[g+1]]))}else if("toself"===s.fill||"tonext"===s.fill){for(h=[],i=0,o=0;o<d.length;o+=2)(isNaN(d[o])||isNaN(d[o+1]))&&((h=h.concat(d.slice(i,o))).push(d[i],d[i+1]),i=o+2);h=h.concat(d.slice(i)),i&&h.push(d[i],d[i+1])}else{var p=s._nexttrace;if(p){var I=f.lineOptions[e+1];if(I){var C=I.positions;if("tonexty"===s.fill){for(h=d.slice(),e=Math.floor(C.length/2);e--;){var m=C[2*e],y=C[2*e+1];isNaN(m)||isNaN(y)||h.push(m,y)}t.fill=p.fillcolor}}}}if(s._prevtrace&&"tonext"===s._prevtrace.fill){var E=f.lineOptions[e-1].positions,v=h.length/2,B=[i=v];for(o=0;o<E.length;o+=2)(isNaN(E[o])||isNaN(E[o+1]))&&(B.push(o/2+v+1),i=o+2);h=h.concat(E),t.hole=B}return t.fillmode=s.fill,t.opacity=s.opacity,t.positions=h,t}}),f.fill2d.update(f.fillOptions))}var B=d.dragmode,b="lasso"===B||"select"===B,Q=d.clickmode.indexOf("select")>-1;for(g=0;g<C;g++){var w=n[g][0],x=w.trace,_=w.t,D=_.index,M=x._length,k=_.x,S=_.y;if(x.selectedpoints||b||Q){if(b||(b=!0),x.selectedpoints){var L=f.selectBatch[D]=s.selIndices2selPoints(x),R={};for(h=0;h<L.length;h++)R[L[h]]=1;var N=[];for(h=0;h<M;h++)R[h]||N.push(h);f.unselectBatch[D]=N}var F=_.xpx=new Array(M),T=_.ypx=new Array(M);for(h=0;h<M;h++)F[h]=p.c2p(k[h]),T[h]=I.c2p(S[h])}else _.xpx=_.ypx=null}if(b){if(f.select2d||(f.select2d=r(d._glcanvas.data()[1].regl)),f.scatter2d){var U=new Array(C);for(g=0;g<C;g++)U[g]=f.selectBatch[g].length||f.unselectBatch[g].length?f.markerUnselectedOptions[g]:{};f.scatter2d.update(U)}f.select2d&&(f.select2d.update(f.markerOptions),f.select2d.update(f.markerSelectedOptions)),f.glText&&n.forEach(function(t){var e=((t||[])[0]||{}).trace||{};l.hasText(e)&&u(t)})}else f.scatter2d&&f.scatter2d.update(f.markerOptions);var O={viewport:function(t,e,n){var r=t._size,i=t.width,o=t.height;return[r.l+e.domain[0]*r.w,r.b+n.domain[0]*r.h,i-r.r-(1-e.domain[1])*r.w,o-r.t-(1-n.domain[1])*r.h]}(d,p,I),range:[(p._rl||p.range)[0],(I._rl||I.range)[0],(p._rl||p.range)[1],(I._rl||I.range)[1]]},G=s.repeat(O,f.count);f.fill2d&&f.fill2d.update(G),f.line2d&&f.line2d.update(G),f.error2d&&f.error2d.update(G.concat(G)),f.scatter2d&&f.scatter2d.update(G),f.select2d&&f.select2d.update(G),f.glText&&f.glText.forEach(function(t){t.update(O)})}else f.init()}}},{"../../lib":719,"../../lib/prepare_regl":732,"../scatter/link_traces":1127,"../scatter/subtypes":1135,"./edit_style":1165,"gl-text":317,"regl-error2d":490,"regl-line2d":491,"regl-scatter2d":497}],1169:[function(t,e,n){"use strict";var r=t("../../lib");e.exports=function(t,e){var n=e._scene,i={count:0,dirty:!0,lineOptions:[],fillOptions:[],markerOptions:[],markerSelectedOptions:[],markerUnselectedOptions:[],errorXOptions:[],errorYOptions:[],textOptions:[],textSelectedOptions:[],textUnselectedOptions:[],selectBatch:[],unselectBatch:[]},o={fill2d:!1,scatter2d:!1,error2d:!1,line2d:!1,glText:!1,select2d:!1};return e._scene||((n=e._scene={}).init=function(){r.extendFlat(n,o,i)},n.init(),n.update=function(t){var e=r.repeat(t,n.count);if(n.fill2d&&n.fill2d.update(e),n.scatter2d&&n.scatter2d.update(e),n.line2d&&n.line2d.update(e),n.error2d&&n.error2d.update(e.concat(e)),n.select2d&&n.select2d.update(e),n.glText)for(var i=0;i<n.count;i++)n.glText[i].update(t)},n.draw=function(){for(var t=n.count,e=n.fill2d,i=n.error2d,o=n.line2d,a=n.scatter2d,s=n.glText,A=n.select2d,l=n.selectBatch,c=n.unselectBatch,u=0;u<t;u++){if(e&&n.fillOrder[u]&&e.draw(n.fillOrder[u]),o&&n.lineOptions[u]&&o.draw(u),i&&(n.errorXOptions[u]&&i.draw(u),n.errorYOptions[u]&&i.draw(u+t)),a&&n.markerOptions[u])if(c[u].length){var g=r.repeat([],n.count);g[u]=c[u],a.draw(g)}else l[u].length||a.draw(u);s[u]&&n.textOptions[u]&&s[u].render()}A&&A.draw(l),n.dirty=!1},n.destroy=function(){n.fill2d&&n.fill2d.destroy&&n.fill2d.destroy(),n.scatter2d&&n.scatter2d.destroy&&n.scatter2d.destroy(),n.error2d&&n.error2d.destroy&&n.error2d.destroy(),n.line2d&&n.line2d.destroy&&n.line2d.destroy(),n.select2d&&n.select2d.destroy&&n.select2d.destroy(),n.glText&&n.glText.forEach(function(t){t.destroy&&t.destroy()}),n.lineOptions=null,n.fillOptions=null,n.markerOptions=null,n.markerSelectedOptions=null,n.markerUnselectedOptions=null,n.errorXOptions=null,n.errorYOptions=null,n.textOptions=null,n.textSelectedOptions=null,n.textUnselectedOptions=null,n.selectBatch=null,n.unselectBatch=null,e._scene=null}),n.dirty||r.extendFlat(n,i),n}},{"../../lib":719}],1170:[function(t,e,n){"use strict";var r=t("../scatter/subtypes"),i=t("./edit_style").styleTextSelection;e.exports=function(t,e){var n=t.cd,o=[],a=n[0].trace,s=n[0].t,A=a._length,l=s.x,c=s.y,u=s._scene,g=s.index;if(!u)return o;var h=r.hasText(a),d=r.hasMarkers(a),f=!d&&!h;if(!0!==a.visible||f)return o;var p=[],I=[];if(!1!==e&&!e.degenerate)for(var C=0;C<A;C++)e.contains([s.xpx[C],s.ypx[C]],!1,C,t)?(p.push(C),o.push({pointNumber:C,x:l[C],y:c[C]})):I.push(C);if(d){var m=u.scatter2d;if(p.length||I.length){if(!u.selectBatch[g].length&&!u.unselectBatch[g].length){var y=new Array(u.count);y[g]=u.markerUnselectedOptions[g],m.update.apply(m,y)}}else{var E=new Array(u.count);E[g]=u.markerOptions[g],m.update.apply(m,E)}}return u.selectBatch[g]=p,u.unselectBatch[g]=I,h&&i(n),o}},{"../scatter/subtypes":1135,"./edit_style":1165}],1171:[function(t,e,n){"use strict";var r=t("../../components/fx/hovertemplate_attributes"),i=t("../scattergeo/attributes"),o=t("../scatter/attributes"),a=t("../../plots/mapbox/layout_attributes"),s=t("../../plots/attributes"),A=t("../../components/colorscale/attributes"),l=t("../../lib/extend").extendFlat,c=t("../../plot_api/edit_types").overrideAll,u=i.line,g=i.marker;e.exports=c({lon:i.lon,lat:i.lat,mode:l({},o.mode,{dflt:"markers"}),text:l({},o.text,{}),hovertext:l({},o.hovertext,{}),line:{color:u.color,width:u.width},connectgaps:o.connectgaps,marker:l({symbol:{valType:"string",dflt:"circle",arrayOk:!0},opacity:g.opacity,size:g.size,sizeref:g.sizeref,sizemin:g.sizemin,sizemode:g.sizemode},A("marker")),fill:i.fill,fillcolor:o.fillcolor,textfont:a.layers.symbol.textfont,textposition:a.layers.symbol.textposition,below:{valType:"string"},selected:{marker:o.selected.marker},unselected:{marker:o.unselected.marker},hoverinfo:l({},s.hoverinfo,{flags:["lon","lat","text","name"]}),hovertemplate:r()},"calc","nested")},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plots/attributes":764,"../../plots/mapbox/layout_attributes":824,"../scatter/attributes":1112,"../scattergeo/attributes":1151}],1172:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../lib"),o=t("../../constants/numerical").BADNUM,a=t("../../lib/geojson_utils"),s=t("../../components/colorscale"),A=t("../../components/drawing"),l=t("../scatter/make_bubble_size_func"),c=t("../scatter/subtypes"),u=t("../../plots/mapbox/convert_text_opts");function g(){return{geojson:a.makeBlank(),layout:{visibility:"none"},paint:{}}}function h(t){return i.isArrayOrTypedArray(t)?function(t){return t}:t?function(){return t}:d}function d(){return""}function f(t){return t[0]===o}e.exports=function(t){var e,n=t[0].trace,o=!0===n.visible&&0!==n._length,p="none"!==n.fill,I=c.hasLines(n),C=c.hasMarkers(n),m=c.hasText(n),y=C&&"circle"===n.marker.symbol,E=C&&"circle"!==n.marker.symbol,v=g(),B=g(),b=g(),Q=g(),w={fill:v,line:B,circle:b,symbol:Q};if(!o)return w;if((p||I)&&(e=a.calcTraceToLineCoords(t)),p&&(v.geojson=a.makePolygon(e),v.layout.visibility="visible",i.extendFlat(v.paint,{"fill-color":n.fillcolor})),I&&(B.geojson=a.makeLine(e),B.layout.visibility="visible",i.extendFlat(B.paint,{"line-width":n.line.width,"line-color":n.line.color,"line-opacity":n.opacity})),y){var x=function(t){var e,n,o,a,c=t[0].trace,u=c.marker,g=c.selectedpoints,h=i.isArrayOrTypedArray(u.color),d=i.isArrayOrTypedArray(u.size),p=i.isArrayOrTypedArray(u.opacity);function I(t){return c.opacity*t}h&&(n=s.hasColorscale(c,"marker")?s.makeColorScaleFuncFromTrace(u):i.identity),d&&(o=l(c)),p&&(a=function(t){var e=r(t)?+i.constrain(t,0,1):0;return I(e)});var C,m,y=[];for(e=0;e<t.length;e++){var E=t[e],v=E.lonlat;if(!f(v)){var B={};n&&(B.mcc=E.mcc=n(E.mc)),o&&(B.mrc=E.mrc=o(E.ms)),a&&(B.mo=a(E.mo)),g&&(B.selected=E.selected||0),y.push({type:"Feature",geometry:{type:"Point",coordinates:v},properties:B})}}if(g)for(C=A.makeSelectedPointStyleFns(c),e=0;e<y.length;e++){var b=y[e].properties;C.selectedOpacityFn&&(b.mo=I(C.selectedOpacityFn(b))),C.selectedColorFn&&(b.mcc=C.selectedColorFn(b)),C.selectedSizeFn&&(b.mrc=C.selectedSizeFn(b))}return{geojson:{type:"FeatureCollection",features:y},mcc:h||C&&C.selectedColorFn?{type:"identity",property:"mcc"}:u.color,mrc:d||C&&C.selectedSizeFn?{type:"identity",property:"mrc"}:(m=u.size,m/2),mo:p||C&&C.selectedOpacityFn?{type:"identity",property:"mo"}:I(u.opacity)}}(t);b.geojson=x.geojson,b.layout.visibility="visible",i.extendFlat(b.paint,{"circle-color":x.mcc,"circle-radius":x.mrc,"circle-opacity":x.mo})}if((E||m)&&(Q.geojson=function(t){for(var e=t[0].trace,n=(e.marker||{}).symbol,r=e.text,i="circle"!==n?h(n):d,o=c.hasText(e)?h(r):d,a=[],s=0;s<t.length;s++){var A=t[s];f(A.lonlat)||a.push({type:"Feature",geometry:{type:"Point",coordinates:A.lonlat},properties:{symbol:i(A.mx),text:o(A.tx)}})}return{type:"FeatureCollection",features:a}}(t),i.extendFlat(Q.layout,{visibility:"visible","icon-image":"{symbol}-15","text-field":"{text}"}),E&&(i.extendFlat(Q.layout,{"icon-size":n.marker.size/10}),i.extendFlat(Q.paint,{"icon-opacity":n.opacity*n.marker.opacity,"icon-color":n.marker.color})),m)){var _=(n.marker||{}).size,D=u(n.textposition,_);i.extendFlat(Q.layout,{"text-size":n.textfont.size,"text-anchor":D.anchor,"text-offset":D.offset}),i.extendFlat(Q.paint,{"text-color":n.textfont.color,"text-opacity":n.opacity})}return w}},{"../../components/colorscale":605,"../../components/drawing":614,"../../constants/numerical":695,"../../lib":719,"../../lib/geojson_utils":714,"../../plots/mapbox/convert_text_opts":821,"../scatter/make_bubble_size_func":1128,"../scatter/subtypes":1135,"fast-isnumeric":225}],1173:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../scatter/subtypes"),o=t("../scatter/marker_defaults"),a=t("../scatter/line_defaults"),s=t("../scatter/text_defaults"),A=t("../scatter/fillcolor_defaults"),l=t("./attributes");e.exports=function(t,e,n,c){function u(n,i){return r.coerce(t,e,l,n,i)}if(function(t,e,n){var r=n("lon")||[],i=n("lat")||[],o=Math.min(r.length,i.length);return e._length=o,o}(0,e,u)){if(u("text"),u("hovertext"),u("hovertemplate"),u("mode"),u("below"),i.hasLines(e)&&(a(t,e,n,c,u,{noDash:!0}),u("connectgaps")),i.hasMarkers(e)){o(t,e,n,c,u,{noLine:!0});var g=e.marker;"circle"!==g.symbol&&(r.isArrayOrTypedArray(g.size)&&(g.size=g.size[0]),r.isArrayOrTypedArray(g.color)&&(g.color=g.color[0]))}i.hasText(e)&&s(t,e,c,u,{noSelect:!0}),u("fill"),"none"!==e.fill&&A(t,e,n,u),r.coerceSelectionMarkerOpacity(e,u)}else e.visible=!1}},{"../../lib":719,"../scatter/fillcolor_defaults":1120,"../scatter/line_defaults":1124,"../scatter/marker_defaults":1130,"../scatter/subtypes":1135,"../scatter/text_defaults":1136,"./attributes":1171}],1174:[function(t,e,n){"use strict";e.exports=function(t,e){return t.lon=e.lon,t.lat=e.lat,t}},{}],1175:[function(t,e,n){"use strict";var r=t("../../components/fx"),i=t("../../lib"),o=t("../scatter/get_trace_color"),a=i.fillText,s=t("../../constants/numerical").BADNUM;e.exports=function(t,e,n){var A=t.cd,l=A[0].trace,c=t.xa,u=t.ya,g=t.subplot,h=360*(e>=0?Math.floor((e+180)/360):Math.ceil((e-180)/360)),d=e-h;if(r.getClosest(A,function(t){var e=t.lonlat;if(e[0]===s)return 1/0;var r=i.modHalf(e[0],360),o=e[1],a=g.project([r,o]),A=a.x-c.c2p([d,o]),l=a.y-u.c2p([r,n]),h=Math.max(3,t.mrc||0);return Math.max(Math.sqrt(A*A+l*l)-h,1-3/h)},t),!1!==t.index){var f=A[t.index],p=f.lonlat,I=[i.modHalf(p[0],360)+h,p[1]],C=c.c2p(I),m=u.c2p(I),y=f.mrc||1;return t.x0=C-y,t.x1=C+y,t.y0=m-y,t.y1=m+y,t.color=o(l,f),t.extraText=function(t,e,n){if(!t.hovertemplate){var r=(e.hi||t.hoverinfo).split("+"),i=-1!==r.indexOf("all"),o=-1!==r.indexOf("lon"),s=-1!==r.indexOf("lat"),A=e.lonlat,l=[];return i||o&&s?l.push("("+c(A[0])+", "+c(A[1])+")"):o?l.push(n.lon+c(A[0])):s&&l.push(n.lat+c(A[1])),(i||-1!==r.indexOf("text"))&&a(e,t,l),l.join("<br>")}function c(t){return t+"°"}}(l,f,A[0].t.labels),t.hovertemplate=l.hovertemplate,[t]}}},{"../../components/fx":632,"../../constants/numerical":695,"../../lib":719,"../scatter/get_trace_color":1121}],1176:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:t("../scatter/marker_colorbar"),calc:t("../scattergeo/calc"),plot:t("./plot"),hoverPoints:t("./hover"),eventData:t("./event_data"),selectPoints:t("./select"),styleOnSelect:function(t,e){e&&e[0].trace._glTrace.update(e)},moduleType:"trace",name:"scattermapbox",basePlotModule:t("../../plots/mapbox"),categories:["mapbox","gl","symbols","showLegend","scatterlike"],meta:{}}},{"../../plots/mapbox":822,"../scatter/marker_colorbar":1129,"../scattergeo/calc":1152,"./attributes":1171,"./defaults":1173,"./event_data":1174,"./hover":1175,"./plot":1177,"./select":1178}],1177:[function(t,e,n){"use strict";var r=t("./convert"),i=t("../../plots/mapbox/constants").traceLayerPrefix,o=["fill","line","circle","symbol"];function a(t,e){this.subplot=t,this.uid=e,this.sourceIds={fill:"source-"+e+"-fill",line:"source-"+e+"-line",circle:"source-"+e+"-circle",symbol:"source-"+e+"-symbol"},this.layerIds={fill:i+e+"-fill",line:i+e+"-line",circle:i+e+"-circle",symbol:i+e+"-symbol"},this.below=null}var s=a.prototype;s.addSource=function(t,e){this.subplot.map.addSource(this.sourceIds[t],{type:"geojson",data:e.geojson})},s.setSourceData=function(t,e){this.subplot.map.getSource(this.sourceIds[t]).setData(e.geojson)},s.addLayer=function(t,e,n){this.subplot.addLayer({type:t,id:this.layerIds[t],source:this.sourceIds[t],layout:e.layout,paint:e.paint},n)},s.update=function(t){var e,n,i,a=this.subplot,s=a.map,A=r(t),l=a.belowLookup["trace-"+this.uid];if(l!==this.below){for(e=o.length-1;e>=0;e--)n=o[e],s.removeLayer(this.layerIds[n]);for(e=0;e<o.length;e++)i=A[n=o[e]],this.addLayer(n,i,l);this.below=l}for(e=0;e<o.length;e++)i=A[n=o[e]],a.setOptions(this.layerIds[n],"setLayoutProperty",i.layout),"visible"===i.layout.visibility&&(this.setSourceData(n,i),a.setOptions(this.layerIds[n],"setPaintProperty",i.paint));t[0].trace._glTrace=this},s.dispose=function(){for(var t=this.subplot.map,e=o.length-1;e>=0;e--){var n=o[e];t.removeLayer(this.layerIds[n]),t.removeSource(this.sourceIds[n])}},e.exports=function(t,e){for(var n=e[0].trace,i=new a(t,n.uid),s=r(e),A=i.below=t.belowLookup["trace-"+n.uid],l=0;l<o.length;l++){var c=o[l],u=s[c];i.addSource(c,u),i.addLayer(c,u,A)}return e[0].trace._glTrace=i,i}},{"../../plots/mapbox/constants":820,"./convert":1172}],1178:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../scatter/subtypes"),o=t("../../constants/numerical").BADNUM;e.exports=function(t,e){var n,a=t.cd,s=t.xaxis,A=t.yaxis,l=[],c=a[0].trace;if(!i.hasMarkers(c))return[];if(!1===e)for(n=0;n<a.length;n++)a[n].selected=0;else for(n=0;n<a.length;n++){var u=a[n],g=u.lonlat;if(g[0]!==o){var h=[r.modHalf(g[0],360),g[1]],d=[s.c2p(h),A.c2p(h)];e.contains(d,null,n,t)?(l.push({pointNumber:n,lon:g[0],lat:g[1]}),u.selected=1):u.selected=0}}return l}},{"../../constants/numerical":695,"../../lib":719,"../scatter/subtypes":1135}],1179:[function(t,e,n){"use strict";var r=t("../../components/fx/hovertemplate_attributes"),i=t("../../lib/extend").extendFlat,o=t("../scatter/attributes"),a=t("../../plots/attributes"),s=o.line;e.exports={mode:o.mode,r:{valType:"data_array",editType:"calc+clearAxisTypes"},theta:{valType:"data_array",editType:"calc+clearAxisTypes"},r0:{valType:"any",dflt:0,editType:"calc+clearAxisTypes"},dr:{valType:"number",dflt:1,editType:"calc"},theta0:{valType:"any",dflt:0,editType:"calc+clearAxisTypes"},dtheta:{valType:"number",editType:"calc"},thetaunit:{valType:"enumerated",values:["radians","degrees","gradians"],dflt:"degrees",editType:"calc+clearAxisTypes"},text:o.text,hovertext:o.hovertext,line:{color:s.color,width:s.width,dash:s.dash,shape:i({},s.shape,{values:["linear","spline"]}),smoothing:s.smoothing,editType:"calc"},connectgaps:o.connectgaps,marker:o.marker,cliponaxis:i({},o.cliponaxis,{dflt:!1}),textposition:o.textposition,textfont:o.textfont,fill:i({},o.fill,{values:["none","toself","tonext"],dflt:"none"}),fillcolor:o.fillcolor,hoverinfo:i({},a.hoverinfo,{flags:["r","theta","text","name"]}),hoveron:o.hoveron,hovertemplate:r(),selected:o.selected,unselected:o.unselected}},{"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../scatter/attributes":1112}],1180:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../../constants/numerical").BADNUM,o=t("../../plots/cartesian/axes"),a=t("../scatter/colorscale_calc"),s=t("../scatter/arrays_to_calcdata"),A=t("../scatter/calc_selection"),l=t("../scatter/calc").calcMarkerSize;e.exports=function(t,e){for(var n=t._fullLayout,c=e.subplot,u=n[c].radialaxis,g=n[c].angularaxis,h=u.makeCalcdata(e,"r"),d=g.makeCalcdata(e,"theta"),f=e._length,p=new Array(f),I=0;I<f;I++){var C=h[I],m=d[I],y=p[I]={};r(C)&&r(m)?(y.r=C,y.theta=m):y.r=i}var E=l(e,f);return e._extremes.x=o.findExtremes(u,h,{ppad:E}),a(t,e),s(p,e),A(p,e),p}},{"../../constants/numerical":695,"../../plots/cartesian/axes":767,"../scatter/arrays_to_calcdata":1111,"../scatter/calc":1113,"../scatter/calc_selection":1114,"../scatter/colorscale_calc":1115,"fast-isnumeric":225}],1181:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../scatter/subtypes"),o=t("../scatter/marker_defaults"),a=t("../scatter/line_defaults"),s=t("../scatter/line_shape_defaults"),A=t("../scatter/text_defaults"),l=t("../scatter/fillcolor_defaults"),c=t("../scatter/constants").PTS_LINESONLY,u=t("./attributes");function g(t,e,n,r){var i,o=r("r"),a=r("theta");if(o)a?i=Math.min(o.length,a.length):(i=o.length,r("theta0"),r("dtheta"));else{if(!a)return 0;i=e.theta.length,r("r0"),r("dr")}return e._length=i,i}e.exports={handleRThetaDefaults:g,supplyDefaults:function(t,e,n,h){function d(n,i){return r.coerce(t,e,u,n,i)}var f=g(0,e,0,d);if(f){d("thetaunit"),d("mode",f<c?"lines+markers":"lines"),d("text"),d("hovertext"),"fills"!==e.hoveron&&d("hovertemplate"),i.hasLines(e)&&(a(t,e,n,h,d),s(t,e,d),d("connectgaps")),i.hasMarkers(e)&&o(t,e,n,h,d,{gradient:!0}),i.hasText(e)&&A(t,e,h,d);var p=[];(i.hasMarkers(e)||i.hasText(e))&&(d("cliponaxis"),d("marker.maxdisplayed"),p.push("points")),d("fill"),"none"!==e.fill&&(l(t,e,n,d),i.hasLines(e)||s(t,e,d)),"tonext"!==e.fill&&"toself"!==e.fill||p.push("fills"),d("hoveron",p.join("+")||"points"),r.coerceSelectionMarkerOpacity(e,d)}else e.visible=!1}}},{"../../lib":719,"../scatter/constants":1116,"../scatter/fillcolor_defaults":1120,"../scatter/line_defaults":1124,"../scatter/line_shape_defaults":1126,"../scatter/marker_defaults":1130,"../scatter/subtypes":1135,"../scatter/text_defaults":1136,"./attributes":1179}],1182:[function(t,e,n){"use strict";var r=t("../scatter/hover"),i=t("../../plots/cartesian/axes"),o=t("../../lib");function a(t,e,n,r){var a=n.radialAxis,s=n.angularAxis;a._hovertitle="r",s._hovertitle="θ";var A=a.c2l(t.r);r.rLabel=i.tickText(a,A,"hover").text;var l="degrees"===s.thetaunit?o.rad2deg(t.theta):t.theta;r.thetaLabel=i.tickText(s,l,"hover").text;var c=t.hi||e.hoverinfo,u=[];function g(t,e){u.push(t._hovertitle+": "+e)}if(!e.hovertemplate){var h=c.split("+");-1!==h.indexOf("all")&&(h=["r","theta","text"]),-1!==h.indexOf("r")&&g(a,r.rLabel),-1!==h.indexOf("theta")&&g(s,r.thetaLabel),-1!==h.indexOf("text")&&r.text&&(u.push(r.text),delete r.text),r.extraText=u.join("<br>")}}e.exports={hoverPoints:function(t,e,n,i){var o=r(t,e,n,i);if(o&&!1!==o[0].index){var s=o[0];if(void 0===s.index)return o;var A=t.subplot,l=s.cd[s.index],c=s.trace;if(A.isPtInside(l))return s.xLabelVal=void 0,s.yLabelVal=void 0,a(l,c,A,s),s.hovertemplate=c.hovertemplate,o}},makeHoverPointText:a}},{"../../lib":719,"../../plots/cartesian/axes":767,"../scatter/hover":1122}],1183:[function(t,e,n){"use strict";e.exports={moduleType:"trace",name:"scatterpolar",basePlotModule:t("../../plots/polar"),categories:["polar","symbols","showLegend","scatter-like"],attributes:t("./attributes"),supplyDefaults:t("./defaults").supplyDefaults,colorbar:t("../scatter/marker_colorbar"),calc:t("./calc"),plot:t("./plot"),style:t("../scatter/style").style,styleOnSelect:t("../scatter/style").styleOnSelect,hoverPoints:t("./hover").hoverPoints,selectPoints:t("../scatter/select"),meta:{}}},{"../../plots/polar":831,"../scatter/marker_colorbar":1129,"../scatter/select":1132,"../scatter/style":1134,"./attributes":1179,"./calc":1180,"./defaults":1181,"./hover":1182,"./plot":1184}],1184:[function(t,e,n){"use strict";var r=t("../scatter/plot"),i=t("../../constants/numerical").BADNUM;e.exports=function(t,e,n){for(var o=e.layers.frontplot.select("g.scatterlayer"),a={xaxis:e.xaxis,yaxis:e.yaxis,plot:e.framework,layerClipId:e._hasClipOnAxisFalse?e.clipIds.forTraces:null},s=e.radialAxis,A=e.angularAxis,l=0;l<n.length;l++)for(var c=n[l],u=0;u<c.length;u++){var g=c[u],h=g.r;if(h===i)g.x=g.y=i;else{var d=s.c2g(h),f=A.c2g(g.theta);g.x=d*Math.cos(f),g.y=d*Math.sin(f)}}r(t,a,n,o)}},{"../../constants/numerical":695,"../scatter/plot":1131}],1185:[function(t,e,n){"use strict";var r=t("../scatterpolar/attributes"),i=t("../scattergl/attributes");e.exports={mode:r.mode,r:r.r,theta:r.theta,r0:r.r0,dr:r.dr,theta0:r.theta0,dtheta:r.dtheta,thetaunit:r.thetaunit,text:r.text,hovertext:r.hovertext,hovertemplate:r.hovertemplate,line:i.line,connectgaps:i.connectgaps,marker:i.marker,fill:i.fill,fillcolor:i.fillcolor,textposition:i.textposition,textfont:i.textfont,hoverinfo:r.hoverinfo,selected:r.selected,unselected:r.unselected}},{"../scattergl/attributes":1160,"../scatterpolar/attributes":1179}],1186:[function(t,e,n){"use strict";var r=t("../scatter/colorscale_calc"),i=t("../scatter/calc").calcMarkerSize,o=t("../scattergl/convert"),a=t("../../plots/cartesian/axes"),s=t("../scattergl/constants").TOO_MANY_POINTS;e.exports=function(t,e){var n=t._fullLayout,A=e.subplot,l=n[A].radialaxis,c=n[A].angularaxis,u=l.makeCalcdata(e,"r"),g=c.makeCalcdata(e,"theta"),h=e._length,d={};h<u.length&&(u=u.slice(0,h)),h<g.length&&(g=g.slice(0,h)),d.r=u,d.theta=g,r(t,e);var f,p=d.opts=o.style(t,e);return h<s?f=i(e,h):p.marker&&(f=2*(p.marker.sizeAvg||Math.max(p.marker.size,3))),e._extremes.x=a.findExtremes(l,u,{ppad:f}),[{x:!1,y:!1,t:d,trace:e}]}},{"../../plots/cartesian/axes":767,"../scatter/calc":1113,"../scatter/colorscale_calc":1115,"../scattergl/constants":1162,"../scattergl/convert":1163}],1187:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../scatter/subtypes"),o=t("../scatterpolar/defaults").handleRThetaDefaults,a=t("../scatter/marker_defaults"),s=t("../scatter/line_defaults"),A=t("../scatter/text_defaults"),l=t("../scatter/fillcolor_defaults"),c=t("../scatter/constants").PTS_LINESONLY,u=t("./attributes");e.exports=function(t,e,n,g){function h(n,i){return r.coerce(t,e,u,n,i)}var d=o(t,e,g,h);d?(h("thetaunit"),h("mode",d<c?"lines+markers":"lines"),h("text"),h("hovertext"),"fills"!==e.hoveron&&h("hovertemplate"),i.hasLines(e)&&(s(t,e,n,g,h),h("connectgaps")),i.hasMarkers(e)&&a(t,e,n,g,h),i.hasText(e)&&A(t,e,g,h),h("fill"),"none"!==e.fill&&l(t,e,n,h),r.coerceSelectionMarkerOpacity(e,h)):e.visible=!1}},{"../../lib":719,"../scatter/constants":1116,"../scatter/fillcolor_defaults":1120,"../scatter/line_defaults":1124,"../scatter/marker_defaults":1130,"../scatter/subtypes":1135,"../scatter/text_defaults":1136,"../scatterpolar/defaults":1181,"./attributes":1185}],1188:[function(t,e,n){"use strict";var r=t("../scattergl/hover"),i=t("../scatterpolar/hover").makeHoverPointText;e.exports={hoverPoints:function(t,e,n,o){var a=t.cd[0].t,s=a.r,A=a.theta,l=r.hoverPoints(t,e,n,o);if(l&&!1!==l[0].index){var c=l[0];if(void 0===c.index)return l;var u=t.subplot,g=c.cd[c.index],h=c.trace;if(g.r=s[c.index],g.theta=A[c.index],u.isPtInside(g))return c.xLabelVal=void 0,c.yLabelVal=void 0,i(g,h,u,c),l}}}},{"../scattergl/hover":1166,"../scatterpolar/hover":1182}],1189:[function(t,e,n){"use strict";e.exports={moduleType:"trace",name:"scatterpolargl",basePlotModule:t("../../plots/polar"),categories:["gl","regl","polar","symbols","showLegend","scatter-like"],attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:t("../scatter/marker_colorbar"),calc:t("./calc"),plot:t("./plot"),hoverPoints:t("./hover").hoverPoints,selectPoints:t("../scattergl/select"),meta:{}}},{"../../plots/polar":831,"../scatter/marker_colorbar":1129,"../scattergl/select":1170,"./attributes":1185,"./calc":1186,"./defaults":1187,"./hover":1188,"./plot":1190}],1190:[function(t,e,n){"use strict";var r=t("point-cluster"),i=t("fast-isnumeric"),o=t("../scattergl/plot"),a=t("../scattergl/scene_update"),s=t("../scattergl/convert"),A=t("../../lib"),l=t("../scattergl/constants").TOO_MANY_POINTS;e.exports=function(t,e,n){if(n.length){var c=e.radialAxis,u=e.angularAxis,g=a(t,e);return n.forEach(function(n){if(n&&n[0]&&n[0].trace){var o,a=n[0],h=a.trace,d=a.t,f=h._length,p=d.r,I=d.theta,C=d.opts,m=p.slice(),y=I.slice();for(o=0;o<p.length;o++)e.isPtInside({r:p[o],theta:I[o]})||(m[o]=NaN,y[o]=NaN);var E=new Array(2*f),v=Array(f),B=Array(f);for(o=0;o<f;o++){var b,Q,w=m[o];if(i(w)){var x=c.c2g(w),_=u.c2g(y[o],h.thetaunit);b=x*Math.cos(_),Q=x*Math.sin(_)}else b=Q=NaN;v[o]=E[2*o]=b,B[o]=E[2*o+1]=Q}d.tree=r(E),C.marker&&f>=l&&(C.marker.cluster=d.tree),C.marker&&(C.markerSel.positions=C.markerUnsel.positions=C.marker.positions=E),C.line&&E.length>1&&A.extendFlat(C.line,s.linePositions(t,h,E)),C.text&&(A.extendFlat(C.text,{positions:E},s.textPosition(t,h,C.text,C.marker)),A.extendFlat(C.textSel,{positions:E},s.textPosition(t,h,C.text,C.markerSel)),A.extendFlat(C.textUnsel,{positions:E},s.textPosition(t,h,C.text,C.markerUnsel))),C.fill&&!g.fill2d&&(g.fill2d=!0),C.marker&&!g.scatter2d&&(g.scatter2d=!0),C.line&&!g.line2d&&(g.line2d=!0),C.text&&!g.glText&&(g.glText=!0),g.lineOptions.push(C.line),g.fillOptions.push(C.fill),g.markerOptions.push(C.marker),g.markerSelectedOptions.push(C.markerSel),g.markerUnselectedOptions.push(C.markerUnsel),g.textOptions.push(C.text),g.textSelectedOptions.push(C.textSel),g.textUnselectedOptions.push(C.textUnsel),g.selectBatch.push([]),g.unselectBatch.push([]),d.x=v,d.y=B,d.rawx=v,d.rawy=B,d.r=p,d.theta=I,d.positions=E,d._scene=g,d.index=g.count,g.count++}}),o(t,e,n)}}},{"../../lib":719,"../scattergl/constants":1162,"../scattergl/convert":1163,"../scattergl/plot":1168,"../scattergl/scene_update":1169,"fast-isnumeric":225,"point-cluster":469}],1191:[function(t,e,n){"use strict";var r=t("../../components/fx/hovertemplate_attributes"),i=t("../scatter/attributes"),o=t("../../plots/attributes"),a=t("../../components/colorscale/attributes"),s=t("../../components/drawing/attributes").dash,A=t("../../lib/extend").extendFlat,l=i.marker,c=i.line,u=l.line;e.exports={a:{valType:"data_array",editType:"calc"},b:{valType:"data_array",editType:"calc"},c:{valType:"data_array",editType:"calc"},sum:{valType:"number",dflt:0,min:0,editType:"calc"},mode:A({},i.mode,{dflt:"markers"}),text:A({},i.text,{}),hovertext:A({},i.hovertext,{}),line:{color:c.color,width:c.width,dash:s,shape:A({},c.shape,{values:["linear","spline"]}),smoothing:c.smoothing,editType:"calc"},connectgaps:i.connectgaps,cliponaxis:i.cliponaxis,fill:A({},i.fill,{values:["none","toself","tonext"],dflt:"none"}),fillcolor:i.fillcolor,marker:A({symbol:l.symbol,opacity:l.opacity,maxdisplayed:l.maxdisplayed,size:l.size,sizeref:l.sizeref,sizemin:l.sizemin,sizemode:l.sizemode,line:A({width:u.width,editType:"calc"},a("marker.line")),gradient:l.gradient,editType:"calc"},a("marker")),textfont:i.textfont,textposition:i.textposition,selected:i.selected,unselected:i.unselected,hoverinfo:A({},o.hoverinfo,{flags:["a","b","c","text","name"]}),hoveron:i.hoveron,hovertemplate:r()}},{"../../components/colorscale/attributes":600,"../../components/drawing/attributes":613,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../scatter/attributes":1112}],1192:[function(t,e,n){"use strict";var r=t("fast-isnumeric"),i=t("../scatter/colorscale_calc"),o=t("../scatter/arrays_to_calcdata"),a=t("../scatter/calc_selection"),s=t("../scatter/calc").calcMarkerSize,A=["a","b","c"],l={a:["b","c"],b:["a","c"],c:["a","b"]};e.exports=function(t,e){var n,c,u,g,h,d,f=t._fullLayout[e.subplot].sum,p=e.sum||f,I={a:e.a,b:e.b,c:e.c};for(n=0;n<A.length;n++)if(!I[u=A[n]]){for(h=I[l[u][0]],d=I[l[u][1]],g=new Array(h.length),c=0;c<h.length;c++)g[c]=p-h[c]-d[c];I[u]=g}var C,m,y,E,v,B,b=e._length,Q=new Array(b);for(n=0;n<b;n++)C=I.a[n],m=I.b[n],y=I.c[n],r(C)&&r(m)&&r(y)?(1!=(E=f/((C=+C)+(m=+m)+(y=+y)))&&(C*=E,m*=E,y*=E),B=C,v=y-m,Q[n]={x:v,y:B,a:C,b:m,c:y}):Q[n]={x:!1,y:!1};return s(e,b),i(t,e),o(Q,e),a(Q,e),Q}},{"../scatter/arrays_to_calcdata":1111,"../scatter/calc":1113,"../scatter/calc_selection":1114,"../scatter/colorscale_calc":1115,"fast-isnumeric":225}],1193:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../scatter/constants"),o=t("../scatter/subtypes"),a=t("../scatter/marker_defaults"),s=t("../scatter/line_defaults"),A=t("../scatter/line_shape_defaults"),l=t("../scatter/text_defaults"),c=t("../scatter/fillcolor_defaults"),u=t("./attributes");e.exports=function(t,e,n,g){function h(n,i){return r.coerce(t,e,u,n,i)}var d,f=h("a"),p=h("b"),I=h("c");if(f?(d=f.length,p?(d=Math.min(d,p.length),I&&(d=Math.min(d,I.length))):d=I?Math.min(d,I.length):0):p&&I&&(d=Math.min(p.length,I.length)),d){e._length=d,h("sum"),h("text"),h("hovertext"),"fills"!==e.hoveron&&h("hovertemplate"),h("mode",d<i.PTS_LINESONLY?"lines+markers":"lines"),o.hasLines(e)&&(s(t,e,n,g,h),A(t,e,h),h("connectgaps")),o.hasMarkers(e)&&a(t,e,n,g,h,{gradient:!0}),o.hasText(e)&&l(t,e,g,h);var C=[];(o.hasMarkers(e)||o.hasText(e))&&(h("cliponaxis"),h("marker.maxdisplayed"),C.push("points")),h("fill"),"none"!==e.fill&&(c(t,e,n,h),o.hasLines(e)||A(t,e,h)),"tonext"!==e.fill&&"toself"!==e.fill||C.push("fills"),h("hoveron",C.join("+")||"points"),r.coerceSelectionMarkerOpacity(e,h)}else e.visible=!1}},{"../../lib":719,"../scatter/constants":1116,"../scatter/fillcolor_defaults":1120,"../scatter/line_defaults":1124,"../scatter/line_shape_defaults":1126,"../scatter/marker_defaults":1130,"../scatter/subtypes":1135,"../scatter/text_defaults":1136,"./attributes":1191}],1194:[function(t,e,n){"use strict";e.exports=function(t,e,n,r,i){if(e.xa&&(t.xaxis=e.xa),e.ya&&(t.yaxis=e.ya),r[i]){var o=r[i];t.a=o.a,t.b=o.b,t.c=o.c}else t.a=e.a,t.b=e.b,t.c=e.c;return t}},{}],1195:[function(t,e,n){"use strict";var r=t("../scatter/hover"),i=t("../../plots/cartesian/axes");e.exports=function(t,e,n,o){var a=r(t,e,n,o);if(a&&!1!==a[0].index){var s=a[0];if(void 0===s.index){var A=1-s.y0/t.ya._length,l=t.xa._length,c=l*A/2,u=l-c;return s.x0=Math.max(Math.min(s.x0,u),c),s.x1=Math.max(Math.min(s.x1,u),c),a}var g=s.cd[s.index];s.a=g.a,s.b=g.b,s.c=g.c,s.xLabelVal=void 0,s.yLabelVal=void 0;var h=s.subplot;s.aLabel=i.tickText(h.aaxis,g.a,"hover").text,s.bLabel=i.tickText(h.baxis,g.b,"hover").text,s.cLabel=i.tickText(h.caxis,g.c,"hover").text;var d=s.trace,f=g.hi||d.hoverinfo,p=[];if(!d.hovertemplate){var I=f.split("+");-1!==I.indexOf("all")&&(I=["a","b","c"]),-1!==I.indexOf("a")&&C(h.aaxis,s.aLabel),-1!==I.indexOf("b")&&C(h.baxis,s.bLabel),-1!==I.indexOf("c")&&C(h.caxis,s.cLabel)}return s.extraText=p.join("<br>"),s.hovertemplate=d.hovertemplate,a}function C(t,e){p.push(t._hovertitle+": "+e)}}},{"../../plots/cartesian/axes":767,"../scatter/hover":1122}],1196:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:t("../scatter/marker_colorbar"),calc:t("./calc"),plot:t("./plot"),style:t("../scatter/style").style,styleOnSelect:t("../scatter/style").styleOnSelect,hoverPoints:t("./hover"),selectPoints:t("../scatter/select"),eventData:t("./event_data"),moduleType:"trace",name:"scatterternary",basePlotModule:t("../../plots/ternary"),categories:["ternary","symbols","showLegend","scatter-like"],meta:{}}},{"../../plots/ternary":843,"../scatter/marker_colorbar":1129,"../scatter/select":1132,"../scatter/style":1134,"./attributes":1191,"./calc":1192,"./defaults":1193,"./event_data":1194,"./hover":1195,"./plot":1197}],1197:[function(t,e,n){"use strict";var r=t("../scatter/plot");e.exports=function(t,e,n){var i=e.plotContainer;i.select(".scatterlayer").selectAll("*").remove();var o={xaxis:e.xaxis,yaxis:e.yaxis,plot:i,layerClipId:e._hasClipOnAxisFalse?e.clipIdRelative:null},a=e.layers.frontplot.select("g.scatterlayer");r(t,o,n,a)}},{"../scatter/plot":1131}],1198:[function(t,e,n){"use strict";var r=t("../scatter/attributes"),i=t("../../components/colorscale/attributes"),o=t("../../components/fx/hovertemplate_attributes"),a=t("../scattergl/attributes"),s=t("../../plots/cartesian/constants").idRegex,A=t("../../plot_api/plot_template").templatedArray,l=t("../../lib/extend").extendFlat,c=r.marker,u=c.line,g=l(i("marker.line",{editTypeOverride:"calc"}),{width:l({},u.width,{editType:"calc"}),editType:"calc"}),h=l(i("marker"),{symbol:c.symbol,size:l({},c.size,{editType:"markerSize"}),sizeref:c.sizeref,sizemin:c.sizemin,sizemode:c.sizemode,opacity:c.opacity,colorbar:c.colorbar,line:g,editType:"calc"});function d(t){return{valType:"info_array",freeLength:!0,editType:"calc",items:{valType:"subplotid",regex:s[t],editType:"plot"}}}h.color.editType=h.cmin.editType=h.cmax.editType="style",e.exports={dimensions:A("dimension",{visible:{valType:"boolean",dflt:!0,editType:"calc"},label:{valType:"string",editType:"calc"},values:{valType:"data_array",editType:"calc+clearAxisTypes"},axis:{type:{valType:"enumerated",values:["linear","log","date","category"],editType:"calc+clearAxisTypes"},matches:{valType:"boolean",dflt:!1,editType:"calc"},editType:"calc+clearAxisTypes"},editType:"calc+clearAxisTypes"}),text:l({},a.text,{}),hovertext:l({},a.hovertext,{}),hovertemplate:o(),marker:h,xaxes:d("x"),yaxes:d("y"),diagonal:{visible:{valType:"boolean",dflt:!0,editType:"calc"},editType:"calc"},showupperhalf:{valType:"boolean",dflt:!0,editType:"calc"},showlowerhalf:{valType:"boolean",dflt:!0,editType:"calc"},selected:{marker:a.selected.marker,editType:"calc"},unselected:{marker:a.unselected.marker,editType:"calc"},opacity:a.opacity}},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plot_api/plot_template":757,"../../plots/cartesian/constants":773,"../scatter/attributes":1112,"../scattergl/attributes":1160}],1199:[function(t,e,n){"use strict";var r=t("regl-line2d"),i=t("../../registry"),o=t("../../lib/prepare_regl"),a=t("../../plots/get_data").getModuleCalcData,s=t("../../plots/cartesian"),A=t("../../plots/cartesian/axis_ids").getFromId,l=t("../../plots/cartesian/axes").shouldShowZeroLine,c="splom";function u(t,e,n){for(var r=n.matrixOptions.data.length,i=e._visibleDims,o=n.viewOpts.ranges=new Array(r),a=0;a<i.length;a++){var s=i[a],l=o[a]=new Array(4),c=A(t,e._diag[s][0]);c&&(l[0]=c.r2l(c.range[0]),l[2]=c.r2l(c.range[1]));var u=A(t,e._diag[s][1]);u&&(l[1]=u.r2l(u.range[0]),l[3]=u.r2l(u.range[1]))}n.selectBatch.length||n.unselectBatch.length?n.matrix.update({ranges:o},{ranges:o}):n.matrix.update({ranges:o})}function g(t){var e=t._fullLayout,n=e._glcanvas.data()[0].regl,i=e._splomGrid;i||(i=e._splomGrid=r(n)),i.update(function(t){var e,n=t._fullLayout,r=n._size,i=[0,0,n.width,n.height],o={};function a(t,e,n,r,a,s){var A=e[t+"color"],l=e[t+"width"],c=String(A+l);c in o?o[c].data.push(NaN,NaN,n,r,a,s):o[c]={data:[n,r,a,s],join:"rect",thickness:l,color:A,viewport:i,range:i,overlay:!1}}for(e in n._splomSubplots){var s,A,c=n._plots[e],u=c.xaxis,g=c.yaxis,h=u._vals,d=g._vals,f=r.b+g.domain[0]*r.h,p=-g._m,I=-p*g.r2l(g.range[0],g.calendar);if(u.showgrid)for(e=0;e<h.length;e++)s=u._offset+u.l2p(h[e].x),a("grid",u,s,f,s,f+g._length);if(g.showgrid)for(e=0;e<d.length;e++)A=f+I+p*d[e].x,a("grid",g,u._offset,A,u._offset+u._length,A);l(t,u,g)&&(s=u._offset+u.l2p(0),a("zeroline",u,s,f,s,f+g._length)),l(t,g,u)&&(A=f+I+0,a("zeroline",g,u._offset,A,u._offset+u._length,A))}var C=[];for(e in o)C.push(o[e]);return C}(t))}e.exports={name:c,attr:s.attr,attrRegex:s.attrRegex,layoutAttributes:s.layoutAttributes,supplyLayoutDefaults:s.supplyLayoutDefaults,drawFramework:s.drawFramework,plot:function(t){var e=t._fullLayout,n=i.getModule(c),r=a(t.calcdata,n)[0];o(t,["ANGLE_instanced_arrays","OES_element_index_uint"])&&(e._hasOnlyLargeSploms&&g(t),n.plot(t,{},r))},drag:function(t){var e=t.calcdata,n=t._fullLayout;n._hasOnlyLargeSploms&&g(t);for(var r=0;r<e.length;r++){var i=e[r][0].trace,o=n._splomScenes[i.uid];"splom"===i.type&&o&&o.matrix&&u(t,i,o)}},updateGrid:g,clean:function(t,e,n,r){var i,o={};if(r._splomScenes){for(i=0;i<t.length;i++){var a=t[i];"splom"===a.type&&(o[a.uid]=1)}for(i=0;i<n.length;i++){var A=n[i];if(!o[A.uid]){var l=r._splomScenes[A.uid];l&&l.destroy&&l.destroy(),r._splomScenes[A.uid]=null,delete r._splomScenes[A.uid]}}}0===Object.keys(r._splomScenes||{}).length&&delete r._splomScenes,r._splomGrid&&!e._hasOnlyLargeSploms&&r._hasOnlyLargeSploms&&(r._splomGrid.destroy(),r._splomGrid=null,delete r._splomGrid),s.clean(t,e,n,r)},updateFx:s.updateFx,toSVG:s.toSVG}},{"../../lib/prepare_regl":732,"../../plots/cartesian":778,"../../plots/cartesian/axes":767,"../../plots/cartesian/axis_ids":770,"../../plots/get_data":802,"../../registry":847,"regl-line2d":491}],1200:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axis_ids"),o=t("../scatter/calc").calcMarkerSize,a=t("../scatter/calc").calcAxisExpansion,s=t("../scatter/colorscale_calc"),A=t("../scattergl/convert").markerSelection,l=t("../scattergl/convert").markerStyle,c=t("./scene_update"),u=t("../../constants/numerical").BADNUM,g=t("../scattergl/constants").TOO_MANY_POINTS;e.exports=function(t,e){var n,h,d,f,p,I,C=e.dimensions,m=e._length,y={},E=y.cdata=[],v=y.data=[],B=e._visibleDims=[];function b(t,n){for(var i=t.makeCalcdata({v:n.values,vcalendar:e.calendar},"v"),o=0;o<i.length;o++)i[o]=i[o]===u?NaN:i[o];E.push(i),v.push("log"===t.type?r.simpleMap(i,t.c2l):i)}for(n=0;n<C.length;n++)if((d=C[n]).visible){if(f=i.getFromId(t,e._diag[n][0]),p=i.getFromId(t,e._diag[n][1]),f&&p&&f.type!==p.type){r.log("Skipping splom dimension "+n+" with conflicting axis types");continue}f?(b(f,d),p&&"category"===p.type&&(p._categories=f._categories.slice())):b(p,d),B.push(n)}for(s(t,e),r.extendFlat(y,l(e)),I=E.length*m>g?2*(y.sizeAvg||Math.max(y.size,3)):o(e,m),h=0;h<B.length;h++)d=C[n=B[h]],f=i.getFromId(t,e._diag[n][0])||{},p=i.getFromId(t,e._diag[n][1])||{},a(t,e,f,p,E[h],E[h],I);var Q=c(t,e);return Q.matrix||(Q.matrix=!0),Q.matrixOptions=y,Q.selectedOptions=A(e,e.selected),Q.unselectedOptions=A(e,e.unselected),[{x:!1,y:!1,t:{},trace:e}]}},{"../../constants/numerical":695,"../../lib":719,"../../plots/cartesian/axis_ids":770,"../scatter/calc":1113,"../scatter/colorscale_calc":1115,"../scattergl/constants":1162,"../scattergl/convert":1163,"./scene_update":1207}],1201:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/array_container_defaults"),o=t("./attributes"),a=t("../scatter/subtypes"),s=t("../scatter/marker_defaults"),A=t("../parcoords/merge_length"),l=/-open/;function c(t,e){function n(n,i){return r.coerce(t,e,o.dimensions,n,i)}n("label");var i=n("values");i&&i.length?n("visible"):e.visible=!1,n("axis.type"),n("axis.matches")}e.exports=function(t,e,n,u){function g(n,i){return r.coerce(t,e,o,n,i)}var h=i(t,e,{name:"dimensions",handleItemDefaults:c}),d=g("diagonal.visible"),f=g("showupperhalf"),p=g("showlowerhalf");if(A(e,h,"values")&&(d||f||p)){g("text"),g("hovertext"),g("hovertemplate"),s(t,e,n,u,g);var I=l.test(e.marker.symbol),C=a.isBubble(e);g("marker.line.width",I||C?1:0),function(t,e,n,r){var i,o,a=e.dimensions,s=a.length,A=e.showupperhalf,l=e.showlowerhalf,c=e.diagonal.visible,u=new Array(s),g=new Array(s);for(i=0;i<s;i++){var h=i?i+1:"";u[i]="x"+h,g[i]="y"+h}var d=r("xaxes",u),f=r("yaxes",g),p=e._diag=new Array(s);e._xaxes={},e._yaxes={};var I=[],C=[];function m(t,r,i,o){if(t){var a=t.charAt(0),s=n._splomAxes[a];if(e["_"+a+"axes"][t]=1,o.push(t),!(t in s)){var A=s[t]={};i&&(A.label=i.label||"",i.visible&&i.axis&&(i.axis.type&&(A.type=i.axis.type),i.axis.matches&&(A.matches=r)))}}}var y=!c&&!l,E=!c&&!A;for(e._axesDim={},i=0;i<s;i++){var v=a[i],B=0===i,b=i===s-1,Q=B&&y||b&&E?void 0:d[i],w=B&&E||b&&y?void 0:f[i];m(Q,w,v,I),m(w,Q,v,C),p[i]=[Q,w],e._axesDim[Q]=i,e._axesDim[w]=i}for(i=0;i<I.length;i++)for(o=0;o<C.length;o++){var x=I[i]+C[o];i>o&&A?n._splomSubplots[x]=1:i<o&&l?n._splomSubplots[x]=1:i!==o||!c&&l&&A||(n._splomSubplots[x]=1)}(!l||!c&&A&&l)&&(n._splomGridDflt.xside="bottom",n._splomGridDflt.yside="left")}(0,e,u,g),r.coerceSelectionMarkerOpacity(e,g)}else e.visible=!1}},{"../../lib":719,"../../plots/array_container_defaults":763,"../parcoords/merge_length":1083,"../scatter/marker_defaults":1130,"../scatter/subtypes":1135,"./attributes":1198}],1202:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../scatter/colorscale_calc"),o=t("../scattergl/convert").markerStyle;e.exports=function(t,e){var n=e.trace,a=t._fullLayout._splomScenes[n.uid];if(a){i(t,n),r.extendFlat(a.matrixOptions,o(n));var s=r.extendFlat({},a.matrixOptions,a.viewOpts);a.matrix.update(s,null)}}},{"../../lib":719,"../scatter/colorscale_calc":1115,"../scattergl/convert":1163}],1203:[function(t,e,n){"use strict";n.getDimIndex=function(t,e){for(var n=e._id,r={x:0,y:1}[n.charAt(0)],i=t._visibleDims,o=0;o<i.length;o++){var a=i[o];if(t._diag[a][r]===n)return o}return!1}},{}],1204:[function(t,e,n){"use strict";var r=t("./helpers"),i=t("../scattergl/hover").calcHover;e.exports={hoverPoints:function(t,e,n){var o=t.cd[0].trace,a=t.scene.matrixOptions.cdata,s=t.xa,A=t.ya,l=s.c2p(e),c=A.c2p(n),u=t.distance,g=r.getDimIndex(o,s),h=r.getDimIndex(o,A);if(!1===g||!1===h)return[t];for(var d,f,p=a[g],I=a[h],C=u,m=0;m<p.length;m++){var y=p[m],E=I[m],v=s.c2p(y)-l,B=A.c2p(E)-c,b=Math.sqrt(v*v+B*B);b<C&&(C=f=b,d=m)}return t.index=d,t.distance=C,t.dxy=f,void 0===d?[t]:(i(t,p,I,o),[t])}}},{"../scattergl/hover":1166,"./helpers":1203}],1205:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../components/grid");e.exports={moduleType:"trace",name:"splom",basePlotModule:t("./base_plot"),categories:["gl","regl","cartesian","symbols","showLegend","scatter-like"],attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:t("../scatter/marker_colorbar"),calc:t("./calc"),plot:t("./plot"),hoverPoints:t("./hover").hoverPoints,selectPoints:t("./select"),editStyle:t("./edit_style"),meta:{}},r.register(i)},{"../../components/grid":636,"../../registry":847,"../scatter/marker_colorbar":1129,"./attributes":1198,"./base_plot":1199,"./calc":1200,"./defaults":1201,"./edit_style":1202,"./hover":1204,"./plot":1206,"./select":1208}],1206:[function(t,e,n){"use strict";var r=t("regl-splom"),i=t("../../lib"),o=t("../../plots/cartesian/axis_ids");function a(t,e){var n,a,s,A,l,c=t._fullLayout,u=c._size,g=e.trace,h=e.t,d=c._splomScenes[g.uid],f=d.matrixOptions,p=f.cdata,I=c._glcanvas.data()[0].regl,C=c.dragmode;if(0!==p.length){f.lower=g.showupperhalf,f.upper=g.showlowerhalf,f.diagonal=g.diagonal.visible;var m=g._visibleDims,y=p.length,E=d.viewOpts={};for(E.ranges=new Array(y),E.domains=new Array(y),l=0;l<m.length;l++){s=m[l];var v=E.ranges[l]=new Array(4),B=E.domains[l]=new Array(4);(n=o.getFromId(t,g._diag[s][0]))&&(v[0]=n._rl[0],v[2]=n._rl[1],B[0]=n.domain[0],B[2]=n.domain[1]),(a=o.getFromId(t,g._diag[s][1]))&&(v[1]=a._rl[0],v[3]=a._rl[1],B[1]=a.domain[0],B[3]=a.domain[1])}E.viewport=[u.l,u.b,u.w+u.l,u.h+u.b],!0===d.matrix&&(d.matrix=r(I));var b=c.clickmode.indexOf("select")>-1,Q=!0;if("lasso"===C||"select"===C||g.selectedpoints||b){var w=g._length;if(g.selectedpoints){d.selectBatch=g.selectedpoints;var x=g.selectedpoints,_={};for(s=0;s<x.length;s++)_[x[s]]=!0;var D=[];for(s=0;s<w;s++)_[s]||D.push(s);d.unselectBatch=D}var M=h.xpx=new Array(y),k=h.ypx=new Array(y);for(l=0;l<m.length;l++){if(s=m[l],n=o.getFromId(t,g._diag[s][0]))for(M[l]=new Array(w),A=0;A<w;A++)M[l][A]=n.c2p(p[l][A]);if(a=o.getFromId(t,g._diag[s][1]))for(k[l]=new Array(w),A=0;A<w;A++)k[l][A]=a.c2p(p[l][A])}if(d.selectBatch.length||d.unselectBatch.length){var S=i.extendFlat({},f,d.unselectedOptions,E),L=i.extendFlat({},f,d.selectedOptions,E);d.matrix.update(S,L),Q=!1}}else h.xpx=h.ypx=null;if(Q){var R=i.extendFlat({},f,E);d.matrix.update(R,null)}}}e.exports=function(t,e,n){if(n.length)for(var r=0;r<n.length;r++)a(t,n[r][0])}},{"../../lib":719,"../../plots/cartesian/axis_ids":770,"regl-splom":501}],1207:[function(t,e,n){"use strict";var r=t("../../lib");e.exports=function(t,e){var n=t._fullLayout,i=e.uid,o=n._splomScenes;o||(o=n._splomScenes={});var a={dirty:!0},s=o[e.uid];return s||((s=o[i]=r.extendFlat({},a,{matrix:!1,selectBatch:[],unselectBatch:[]})).draw=function(){s.matrix&&s.matrix.draw&&(s.selectBatch.length||s.unselectBatch.length?s.matrix.draw(s.unselectBatch,s.selectBatch):s.matrix.draw()),s.dirty=!1},s.destroy=function(){s.matrix&&s.matrix.destroy&&s.matrix.destroy(),s.matrixOptions=null,s.selectBatch=null,s.unselectBatch=null,s=null}),s.dirty||r.extendFlat(s,a),s}},{"../../lib":719}],1208:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../scatter/subtypes"),o=t("./helpers");e.exports=function(t,e){var n=t.cd,a=n[0].trace,s=n[0].t,A=t.scene,l=A.matrixOptions.cdata,c=t.xaxis,u=t.yaxis,g=[];if(!A)return g;var h=!i.hasMarkers(a)&&!i.hasText(a);if(!0!==a.visible||h)return g;var d=o.getDimIndex(a,c),f=o.getDimIndex(a,u);if(!1===d||!1===f)return g;var p=s.xpx[d],I=s.ypx[f],C=l[d],m=l[f],y=[],E=[];if(!1!==e&&!e.degenerate)for(var v=0;v<C.length;v++)e.contains([p[v],I[v]],null,v,t)?(y.push(v),g.push({pointNumber:v,x:C[v],y:m[v]})):E.push(v);var B=A.matrixOptions;return y.length||E.length?A.selectBatch.length||A.unselectBatch.length||A.matrix.update(A.unselectedOptions,r.extendFlat({},B,A.selectedOptions,A.viewOpts)):A.matrix.update(B,null),A.selectBatch=y,A.unselectBatch=E,g}},{"../../lib":719,"../scatter/subtypes":1135,"./helpers":1203}],1209:[function(t,e,n){"use strict";var r=t("../../components/colorscale/attributes"),i=t("../../components/fx/hovertemplate_attributes"),o=t("../mesh3d/attributes"),a=t("../../plots/attributes"),s=t("../../lib/extend").extendFlat,A={x:{valType:"data_array",editType:"calc+clearAxisTypes"},y:{valType:"data_array",editType:"calc+clearAxisTypes"},z:{valType:"data_array",editType:"calc+clearAxisTypes"},u:{valType:"data_array",editType:"calc"},v:{valType:"data_array",editType:"calc"},w:{valType:"data_array",editType:"calc"},starts:{x:{valType:"data_array",editType:"calc"},y:{valType:"data_array",editType:"calc"},z:{valType:"data_array",editType:"calc"},editType:"calc"},maxdisplayed:{valType:"integer",min:0,dflt:1e3,editType:"calc"},sizeref:{valType:"number",editType:"calc",min:0,dflt:1},text:{valType:"string",dflt:"",editType:"calc"},hovertext:{valType:"string",dflt:"",editType:"calc"},hovertemplate:i({editType:"calc"},{keys:["tubex","tubey","tubez","tubeu","tubev","tubew","norm","divergence"]})};s(A,r("",{colorAttr:"u/v/w norm",showScaleDflt:!0,editTypeOverride:"calc"})),["opacity","lightposition","lighting"].forEach(function(t){A[t]=o[t]}),A.hoverinfo=s({},a.hoverinfo,{editType:"calc",flags:["x","y","z","u","v","w","norm","divergence","text","name"],dflt:"x+y+z+norm+text+name"}),A.transforms=void 0,e.exports=A},{"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../mesh3d/attributes":1053}],1210:[function(t,e,n){"use strict";var r=t("../../components/colorscale/calc");e.exports=function(t,e){var n,i,o,a,s=e.u,A=e.v,l=e.w,c=e.x,u=e.y,g=e.z,h=Math.min(c.length,u.length,g.length,s.length,A.length,l.length),d=0;e.starts&&(i=e.starts.x||[],o=e.starts.y||[],a=e.starts.z||[],d=Math.min(i.length,o.length,a.length));var f=0,p=1/0;for(n=0;n<h;n++){var I=s[n],C=A[n],m=l[n],y=Math.sqrt(I*I+C*C+m*m);f=Math.max(f,y),p=Math.min(p,y)}r(t,e,{vals:[p,f],containerStr:"",cLetter:"c"});var E=-1/0,v=1/0,B=-1/0,b=1/0,Q=-1/0,w=1/0;for(n=0;n<h;n++){var x=c[n];E=Math.max(E,x),v=Math.min(v,x);var _=u[n];B=Math.max(B,_),b=Math.min(b,_);var D=g[n];Q=Math.max(Q,D),w=Math.min(w,D)}for(n=0;n<d;n++){var M=i[n];E=Math.max(E,M),v=Math.min(v,M);var k=o[n];B=Math.max(B,k),b=Math.min(b,k);var S=a[n];Q=Math.max(Q,S),w=Math.min(w,S)}e._len=h,e._slen=d,e._normMax=f,e._xbnds=[v,E],e._ybnds=[b,B],e._zbnds=[w,Q]}},{"../../components/colorscale/calc":601}],1211:[function(t,e,n){"use strict";var r=t("gl-streamtube3d"),i=r.createTubeMesh,o=t("../../lib"),a=t("../../lib/gl_format_color").parseColorScale,s=t("../../components/colorscale").extractOpts,A=t("../../plots/gl3d/zip3"),l={xaxis:0,yaxis:1,zaxis:2};function c(t,e){this.scene=t,this.uid=e,this.mesh=null,this.data=null}var u=c.prototype;function g(t){return o.distinctVals(t).vals}function h(t){var e=t.length;return e>2?t.slice(1,e-1):2===e?[(t[0]+t[1])/2]:t}function d(t){var e=t.length;return 1===e?[.5,.5]:[t[1]-t[0],t[e-1]-t[e-2]]}function f(t,e){var n=t.fullSceneLayout,i=t.dataScale,c=e._len,u={};function f(t,e){var r=n[e],a=i[l[e]];return o.simpleMap(t,function(t){return r.d2l(t)*a})}u.vectors=A(f(e.u,"xaxis"),f(e.v,"yaxis"),f(e.w,"zaxis"),c);var p=g(e.x.slice(0,c)),I=g(e.y.slice(0,c)),C=g(e.z.slice(0,c));if(p.length*I.length*C.length>c)return{positions:[],cells:[]};var m=f(p,"xaxis"),y=f(I,"yaxis"),E=f(C,"zaxis");if(u.meshgrid=[m,y,E],e.starts){var v=e._slen;u.startingPositions=A(f(e.starts.x.slice(0,v),"xaxis"),f(e.starts.y.slice(0,v),"yaxis"),f(e.starts.z.slice(0,v),"zaxis"))}else{for(var B=y[0],b=h(m),Q=h(E),w=new Array(b.length*Q.length),x=0,_=0;_<b.length;_++)for(var D=0;D<Q.length;D++)w[x++]=[b[_],B,Q[D]];u.startingPositions=w}u.colormap=a(e),u.tubeSize=e.sizeref,u.maxLength=e.maxdisplayed;var M=f(e._xbnds,"xaxis"),k=f(e._ybnds,"yaxis"),S=f(e._zbnds,"zaxis"),L=d(m),R=d(y),N=d(E),F=[[M[0]-L[0],k[0]-R[0],S[0]-N[0]],[M[1]+L[1],k[1]+R[1],S[1]+N[1]]],T=r(u,F),U=s(e);T.vertexIntensityBounds=[U.min/e._normMax,U.max/e._normMax];var O=e.lightposition;return T.lightPosition=[O.x,O.y,O.z],T.ambient=e.lighting.ambient,T.diffuse=e.lighting.diffuse,T.specular=e.lighting.specular,T.roughness=e.lighting.roughness,T.fresnel=e.lighting.fresnel,T.opacity=e.opacity,e._pad=T.tubeScale*e.sizeref*2,T}u.handlePick=function(t){var e=this.scene.fullSceneLayout,n=this.scene.dataScale;function r(t,r){var i=e[r],o=n[l[r]];return i.l2c(t)/o}if(t.object===this.mesh){var i=t.data.position,o=t.data.velocity;return t.traceCoordinate=[r(i[0],"xaxis"),r(i[1],"yaxis"),r(i[2],"zaxis"),r(o[0],"xaxis"),r(o[1],"yaxis"),r(o[2],"zaxis"),t.data.intensity*this.data._normMax,t.data.divergence],t.textLabel=this.data.hovertext||this.data.text,!0}},u.update=function(t){this.data=t;var e=f(this.scene,t);this.mesh.update(e)},u.dispose=function(){this.scene.glplot.remove(this.mesh),this.mesh.dispose()},e.exports=function(t,e){var n=t.glplot.gl,r=f(t,e),o=i(n,r),a=new c(t,e.uid);return a.mesh=o,a.data=e,o._trace=a,t.glplot.add(o),a}},{"../../components/colorscale":605,"../../lib":719,"../../lib/gl_format_color":716,"../../plots/gl3d/zip3":818,"gl-streamtube3d":313}],1212:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/colorscale/defaults"),o=t("./attributes");e.exports=function(t,e,n,a){function s(n,i){return r.coerce(t,e,o,n,i)}var A=s("u"),l=s("v"),c=s("w"),u=s("x"),g=s("y"),h=s("z");A&&A.length&&l&&l.length&&c&&c.length&&u&&u.length&&g&&g.length&&h&&h.length?(s("starts.x"),s("starts.y"),s("starts.z"),s("maxdisplayed"),s("sizeref"),s("lighting.ambient"),s("lighting.diffuse"),s("lighting.specular"),s("lighting.roughness"),s("lighting.fresnel"),s("lightposition.x"),s("lightposition.y"),s("lightposition.z"),i(t,e,a,s,{prefix:"",cLetter:"c"}),s("text"),s("hovertext"),s("hovertemplate"),e._length=null):e.visible=!1}},{"../../components/colorscale/defaults":603,"../../lib":719,"./attributes":1209}],1213:[function(t,e,n){"use strict";e.exports={moduleType:"trace",name:"streamtube",basePlotModule:t("../../plots/gl3d"),categories:["gl3d"],attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:{min:"cmin",max:"cmax"},calc:t("./calc"),plot:t("./convert"),eventData:function(t,e){return t.tubex=t.x,t.tubey=t.y,t.tubez=t.z,t.tubeu=e.traceCoordinate[3],t.tubev=e.traceCoordinate[4],t.tubew=e.traceCoordinate[5],t.norm=e.traceCoordinate[6],t.divergence=e.traceCoordinate[7],delete t.x,delete t.y,delete t.z,t},meta:{}}},{"../../plots/gl3d":807,"./attributes":1209,"./calc":1210,"./convert":1211,"./defaults":1212}],1214:[function(t,e,n){"use strict";var r=t("../../plots/attributes"),i=t("../../components/fx/hovertemplate_attributes"),o=t("../../plots/domain").attributes,a=t("../pie/attributes"),s=t("../../lib/extend").extendFlat;e.exports={labels:{valType:"data_array",editType:"calc"},parents:{valType:"data_array",editType:"calc"},values:{valType:"data_array",editType:"calc"},branchvalues:{valType:"enumerated",values:["remainder","total"],dflt:"remainder",editType:"calc"},level:{valType:"any",editType:"plot",anim:!0},maxdepth:{valType:"integer",editType:"plot",dflt:-1},marker:{colors:{valType:"data_array",editType:"calc"},line:{color:s({},a.marker.line.color,{dflt:null}),width:s({},a.marker.line.width,{dflt:1}),editType:"calc"},editType:"calc"},leaf:{opacity:{valType:"number",editType:"style",min:0,max:1,dflt:.7},editType:"plot"},text:a.text,textinfo:s({},a.textinfo,{editType:"plot",flags:["label","text","value"]}),textfont:a.textfont,hovertext:a.hovertext,hoverinfo:s({},r.hoverinfo,{flags:["label","text","value","name"]}),hovertemplate:i(),insidetextfont:a.insidetextfont,outsidetextfont:a.outsidetextfont,domain:o({name:"sunburst",trace:!0,editType:"calc"})}},{"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../../plots/domain":792,"../pie/attributes":1086}],1215:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../plots/get_data").getModuleCalcData,o=n.name="sunburst";n.plot=function(t,e,n,a){var s=r.getModule(o),A=i(t.calcdata,s)[0];s.plot(t,A,n,a)},n.clean=function(t,e,n,r){var i=r._has&&r._has(o),a=e._has&&e._has(o);i&&!a&&r._sunburstlayer.selectAll("g.trace").remove()}},{"../../plots/get_data":802,"../../registry":847}],1216:[function(t,e,n){"use strict";var r=t("d3-hierarchy"),i=t("fast-isnumeric"),o=t("../../lib"),a=t("../pie/calc").makePullColorFn,s=t("../pie/calc").generateExtendedColors,A=o.isArrayOrTypedArray,l={};n.calc=function(t,e){var n,s,l,c,u,g,h=t._fullLayout,d=e.ids,f=A(d),p=e.labels,I=e.parents,C=e.values,m=A(C),y=[],E={},v={},B=function(t){return t||"number"==typeof t},b=function(t){return!m||i(C[t])&&C[t]>=0};f?(n=Math.min(d.length,I.length),s=function(t){return B(d[t])&&b(t)},l=function(t){return String(d[t])}):(n=Math.min(p.length,I.length),s=function(t){return B(p[t])&&b(t)},l=function(t){return String(p[t])}),m&&(n=Math.min(n,C.length));for(var Q=0;Q<n;Q++)if(s(Q)){var w=l(Q),x=B(I[Q])?String(I[Q]):"",_={i:Q,id:w,pid:x,label:B(p[Q])?String(p[Q]):""};m&&(_.v=+C[Q]),y.push(_),u=w,E[c=x]?E[c].push(u):E[c]=[u],v[u]=1}if(E[""]){if(E[""].length>1){for(var D=o.randstr(),M=0;M<y.length;M++)""===y[M].pid&&(y[M].pid=D);y.unshift({hasMultipleRoots:!0,id:D,pid:""})}}else{var k,S=[];for(k in E)v[k]||S.push(k);if(1!==S.length)return o.warn("Multiple implied roots, cannot build sunburst hierarchy.");k=S[0],y.unshift({id:k,pid:"",label:k})}try{g=r.stratify().id(function(t){return t.id}).parentId(function(t){return t.pid})(y)}catch(t){return o.warn("Failed to build sunburst hierarchy. Error: "+t.message)}var L=r.hierarchy(g),R=!1;if(m)switch(e.branchvalues){case"remainder":L.sum(function(t){return t.data.v});break;case"total":L.each(function(t){var e=t.data.data.v;if(t.children&&e<t.children.reduce(function(t,e){return t+e.data.data.v},0))return R=!0,o.warn(["Total value for node",t.data.data.id,"is smaller than the sum of its children."].join(" "));t.value=e})}else L.count();if(!R){L.sort(function(t,e){return e.value-t.value});var N=e.marker.colors||[],F=a(h._sunburstcolormap);return L.each(function(t){var e=t.data.data,n=e.id;e.color=F(N[e.i],n)}),y[0].hierarchy=L,y}},n.crossTraceCalc=function(t){var e=t._fullLayout,n=t.calcdata,r=e.sunburstcolorway,i=e._sunburstcolormap;e.extendsunburstcolors&&(r=s(r,l));var o=0;function a(t){var e=t.data.data,n=e.id;!1===e.color&&(i[n]?e.color=i[n]:t.parent?t.parent.parent?e.color=t.parent.data.data.color:(i[n]=e.color=r[o%r.length],o++):e.color="rgba(0,0,0,0)")}for(var A=0;A<n.length;A++){var c=n[A][0];"sunburst"===c.trace.type&&c.hierarchy&&c.hierarchy.each(a)}}},{"../../lib":719,"../pie/calc":1088,"d3-hierarchy":157,"fast-isnumeric":225}],1217:[function(t,e,n){"use strict";e.exports={CLICK_TRANSITION_TIME:750,CLICK_TRANSITION_EASING:"linear"}},{}],1218:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./attributes"),o=t("../../plots/domain").defaults,a=t("../bar/defaults").handleText;e.exports=function(t,e,n,s){function A(n,o){return r.coerce(t,e,i,n,o)}var l=A("labels"),c=A("parents");if(l&&l.length&&c&&c.length){var u=A("values");u&&u.length&&A("branchvalues"),A("level"),A("maxdepth"),A("marker.line.width")&&A("marker.line.color",s.paper_bgcolor),A("marker.colors"),A("leaf.opacity");var g=A("text");A("textinfo",Array.isArray(g)?"text+label":"label"),A("hovertext"),A("hovertemplate"),a(t,e,s,A,"auto",{moduleHasSelected:!1,moduleHasUnselected:!1,moduleHasConstrain:!1,moduleHasCliponaxis:!1,moduleHasTextangle:!1,moduleHasInsideanchor:!1}),o(e,s,A),e._length=null}else e.visible=!1}},{"../../lib":719,"../../plots/domain":792,"../bar/defaults":861,"./attributes":1214}],1219:[function(t,e,n){"use strict";e.exports={moduleType:"trace",name:"sunburst",basePlotModule:t("./base_plot"),categories:[],animatable:!0,attributes:t("./attributes"),layoutAttributes:t("./layout_attributes"),supplyDefaults:t("./defaults"),supplyLayoutDefaults:t("./layout_defaults"),calc:t("./calc").calc,crossTraceCalc:t("./calc").crossTraceCalc,plot:t("./plot"),style:t("./style").style,meta:{}}},{"./attributes":1214,"./base_plot":1215,"./calc":1216,"./defaults":1218,"./layout_attributes":1220,"./layout_defaults":1221,"./plot":1222,"./style":1223}],1220:[function(t,e,n){"use strict";e.exports={sunburstcolorway:{valType:"colorlist",editType:"calc"},extendsunburstcolors:{valType:"boolean",dflt:!0,editType:"calc"}}},{}],1221:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./layout_attributes");e.exports=function(t,e){function n(n,o){return r.coerce(t,e,i,n,o)}n("sunburstcolorway",e.colorway),n("extendsunburstcolors")}},{"../../lib":719,"./layout_attributes":1220}],1222:[function(t,e,n){"use strict";var r=t("d3"),i=t("d3-hierarchy"),o=t("../../registry"),a=t("../../components/fx"),s=t("../../components/color"),A=t("../../components/drawing"),l=t("../../lib"),c=t("../../lib/events"),u=t("../../lib/svg_text_utils"),g=t("../../lib/setcursor"),h=t("../../components/fx/helpers").appendArrayPointValue,d=t("../pie/plot").transformInsideText,f=t("../pie/helpers").formatPieValue,p=t("./style").styleOne,I=t("./constants");function C(t,e,n,o){var a=t._fullLayout,c=o&&o.duration>0,g=r.select(n).selectAll("g.slice"),h=e[0],I=h.trace,C=m(h.hierarchy,I.level),v=I.maxdepth>=0?I.maxdepth:1/0,w=a._size,x=I.domain,_=w.w*(x.x[1]-x.x[0]),D=w.h*(x.y[1]-x.y[0]),M=.5*Math.min(_,D),k=h.cx=w.l+w.w*(x.x[1]+x.x[0])/2,S=h.cy=w.t+w.h*(1-x.y[0])-D/2;if(!C)return g.remove();var L=null,R={};c&&g.each(function(t){R[B(t)]={rpx0:t.rpx0,rpx1:t.rpx1,x0:t.x0,x1:t.x1,transform:t.transform},!L&&E(t)&&(L=t)});var N=function(t){return i.partition().size([2*Math.PI,t.height+1])(t)}(C).descendants(),F=C.height+1,T=0,U=v;h.hasMultipleRoots&&y(C)&&(N=N.slice(1),F-=1,T=1,U+=1),N=N.filter(function(t){return t.y1<=U});var O=Math.min(F,v),G=function(t){return(t-T)/O*M},H=function(t,e){return[t*Math.cos(e),-t*Math.sin(e)]},j=function(t){return l.pathAnnulus(t.rpx0,t.rpx1,t.x0,t.x1,k,S)},Y=function(t){return k+t.pxmid[0]*t.transform.rCenter+(t.transform.x||0)},P=function(t){return S+t.pxmid[1]*t.transform.rCenter+(t.transform.y||0)};(g=g.data(N,function(t){return B(t)})).enter().append("g").classed("slice",!0),c?g.exit().transition().each(function(){var t=r.select(this);t.select("path.surface").transition().attrTween("d",function(t){var e=function(t){var e,n=B(t),i=R[n],o=R[B(C)];if(o){var a=t.x1>o.x1?2*Math.PI:0;e=t.rpx1<o.rpx1?{rpx0:0,rpx1:0}:{x0:a,x1:a}}else{var s,A=B(t.parent);g.each(function(t){if(B(t)===A)return s=t});var l,c=s.children;c.forEach(function(t,e){if(B(t)===n)return l=e});var u=c.length,h=r.interpolate(s.x0,s.x1);e={rpx0:M,rpx1:M,x0:h(l/u),x1:h((l+1)/u)}}return r.interpolate(i,e)}(t);return function(t){return j(e(t))}}),t.select("g.slicetext").attr("opacity",0)}).remove():g.exit().remove(),g.order();var K=null;if(c&&L){var z=B(L);g.each(function(t){null===K&&B(t)===z&&(K=t.x1)})}var J=g;function q(t){var e=t.parent,n=R[B(e)],i={};if(n){var o=e.children,a=o.indexOf(t),s=o.length,A=r.interpolate(n.x0,n.x1);i.x0=A(a/s),i.x1=A(a/s)}else i.x0=i.x1=0;return i}c&&(J=J.transition().each("end",function(){b(r.select(this),0,{isTransitioning:!1})})),J.each(function(n){var i=r.select(this),o=l.ensureSingle(i,"path","surface",function(t){t.style("pointer-events","all")});n.rpx0=G(n.y0),n.rpx1=G(n.y1),n.xmid=(n.x0+n.x1)/2,n.pxmid=H(n.rpx1,n.xmid),n.midangle=-(n.xmid-Math.PI/2),n.halfangle=.5*Math.min(l.angleDelta(n.x0,n.x1)||Math.PI,Math.PI),n.ring=1-n.rpx0/n.rpx1,n.rInscribed=function(t){return 0===t.rpx0&&l.isFullCircle([t.x0,t.x1])?1:Math.max(0,Math.min(1/(1+1/Math.sin(t.halfangle)),t.ring/2))}(n),c?o.transition().attrTween("d",function(t){var e=function(t){var e,n=R[B(t)],i={x0:t.x0,x1:t.x1,rpx0:t.rpx0,rpx1:t.rpx1};if(n)e=n;else if(L)if(t.parent)if(K){var o=t.x1>K?2*Math.PI:0;e={x0:o,x1:o}}else e={rpx0:M,rpx1:M},l.extendFlat(e,q(t));else e={rpx0:0,rpx1:0};else e={x0:0,x1:0};return r.interpolate(e,i)}(t);return function(t){return j(e(t))}}):o.attr("d",j),i.call(Q,t,e).call(b,t,{isTransitioning:t._transitioning}),o.call(p,n,I);var g=l.ensureSingle(i,"g","slicetext"),C=l.ensureSingle(g,"text","",function(t){t.attr("data-notex",1)});C.text(function(t,e,n){var r=e.textinfo;if(!r||"none"===r)return"";var i=t.data.data,o=n.separators,a=r.split("+"),s=function(t){return-1!==a.indexOf(t)},A=[];if(s("label")&&i.label&&A.push(i.label),i.hasOwnProperty("v")&&s("value")&&A.push(f(i.v,o)),s("text")){var c=l.castOption(e,i.i,"text");l.isValidTextValue(c)&&A.push(c)}return A.join("<br>")}(n,I,a)).classed("slicetext",!0).attr("text-anchor","middle").call(A.font,y(n)?function(t,e,n){var r=e.data.data.i,i=l.castOption(t,r,"outsidetextfont.color")||l.castOption(t,r,"textfont.color")||n.color,o=l.castOption(t,r,"outsidetextfont.family")||l.castOption(t,r,"textfont.family")||n.family,a=l.castOption(t,r,"outsidetextfont.size")||l.castOption(t,r,"textfont.size")||n.size;return{color:i,family:o,size:a}}(I,n,a.font):function(t,e,n){var r=e.data.data,i=r.i,o=l.castOption(t,i,"insidetextfont.color");!o&&t._input.textfont&&(o=l.castOption(t._input,i,"textfont.color"));var a=l.castOption(t,i,"insidetextfont.family")||l.castOption(t,i,"textfont.family")||n.family,A=l.castOption(t,i,"insidetextfont.size")||l.castOption(t,i,"textfont.size")||n.size;return{color:o||s.contrast(r.color),family:a,size:A}}(I,n,a.font)).call(u.convertToTspans,t);var m=A.bBox(C.node());n.transform=d(m,n,h),n.translateX=Y(n),n.translateY=P(n);var E=function(t,e){return"translate("+t.translateX+","+t.translateY+")"+(t.transform.scale<1?"scale("+t.transform.scale+")":"")+(t.transform.rotate?"rotate("+t.transform.rotate+")":"")+"translate("+-(e.left+e.right)/2+","+-(e.top+e.bottom)/2+")"};c?C.transition().attrTween("transform",function(t){var e=function(t){var e,n=R[B(t)],i=t.transform;if(n)e=n;else if(e={rpx1:t.rpx1,transform:{scale:0,rotate:i.rotate,rCenter:i.rCenter,x:i.x,y:i.y}},L)if(t.parent)if(K){var o=t.x1>K?2*Math.PI:0;e.x0=e.x1=o}else l.extendFlat(e,q(t));else e.x0=e.x1=0;else e.x0=e.x1=0;var a=r.interpolate(e.rpx1,t.rpx1),s=r.interpolate(e.x0,t.x0),A=r.interpolate(e.x1,t.x1),c=r.interpolate(e.transform.scale,i.scale),u=r.interpolate(e.transform.rotate,i.rotate),g=0===i.rCenter?3:0===e.transform.rCenter?1/3:1,h=r.interpolate(e.transform.rCenter,i.rCenter);return function(t){var e=a(t),n=s(t),r=A(t),o=function(t){return h(Math.pow(t,g))}(t),l={pxmid:H(e,(n+r)/2),transform:{rCenter:o,x:i.x,y:i.y}},d={rpx1:a(t),translateX:Y(l),translateY:P(l),transform:{scale:c(t),rotate:u(t),rCenter:o}};return d}}(t);return function(t){return E(e(t),m)}}):C.attr("transform",E(n,m))})}function m(t,e){var n;return e&&t.eachAfter(function(t){if(B(t)===e)return n=t.copy()}),n||t}function y(t){return""===t.data.data.pid}function E(t){return!t.parent}function v(t){return!t.children}function B(t){return t.data.data.id}function b(t,e,n){var r=t.datum(),i=(n||{}).isTransitioning;g(t,i||v(r)||y(r)?null:"pointer")}function Q(t,e,n){var i=n[0],s=i.trace;"_hasHoverLabel"in s||(s._hasHoverLabel=!1),"_hasHoverEvent"in s||(s._hasHoverEvent=!1),t.on("mouseover",function(t){var n=e._fullLayout;if(!e._dragging&&!1!==n.hovermode){var o=e._fullData[s.index],A=t.data.data,c=A.i,u=function(t){return l.castOption(o,c,t)},g=u("hovertemplate"),h=a.castHoverinfo(o,n,c),d=n.separators;if(g||h&&"none"!==h&&"skip"!==h){var p=t.rInscribed,I=i.cx+t.pxmid[0]*(1-p),C=i.cy+t.pxmid[1]*(1-p),m={},y=[],E=[],v=function(t){return-1!==y.indexOf(t)};if(h&&(y="all"===h?o._module.attributes.hoverinfo.flags:h.split("+")),m.label=A.label,v("label")&&m.label&&E.push(m.label),A.hasOwnProperty("v")&&(m.value=A.v,m.valueLabel=f(m.value,d),v("value")&&E.push(m.valueLabel)),m.text=u("hovertext")||u("text"),v("text")){var B=m.text;l.isValidTextValue(B)&&E.push(B)}a.loneHover({trace:o,x0:I-p*t.rpx1,x1:I+p*t.rpx1,y:C,idealAlign:t.pxmid[0]<0?"left":"right",text:E.join("<br>"),name:g||v("name")?o.name:void 0,color:u("hoverlabel.bgcolor")||A.color,borderColor:u("hoverlabel.bordercolor"),fontFamily:u("hoverlabel.font.family"),fontSize:u("hoverlabel.font.size"),fontColor:u("hoverlabel.font.color"),nameLength:u("hoverlabel.namelength"),textAlign:u("hoverlabel.align"),hovertemplate:g,hovertemplateLabels:m,eventData:[w(t,o)]},{container:n._hoverlayer.node(),outerContainer:n._paper.node(),gd:e}),s._hasHoverLabel=!0}s._hasHoverEvent=!0,e.emit("plotly_hover",{points:[w(t,o)],event:r.event})}}),t.on("mouseout",function(t){var n=e._fullLayout,i=e._fullData[s.index],o=r.select(this).datum();s._hasHoverEvent&&(t.originalEvent=r.event,e.emit("plotly_unhover",{points:[w(o,i)],event:r.event}),s._hasHoverEvent=!1),s._hasHoverLabel&&(a.loneUnhover(n._hoverlayer.node()),s._hasHoverLabel=!1)}),t.on("click",function(t){var n=e._fullLayout,A=e._fullData[s.index];if(!1===c.triggerHandler(e,"plotly_sunburstclick",{points:[w(t,A)],event:r.event})||v(t)||y(t))n.hovermode&&(e._hoverdata=[w(t,A)],a.click(e,r.event));else if(!e._dragging&&!e._transitioning){o.call("_storeDirectGUIEdit",A,n._tracePreGUI[A.uid],{level:A.level});var l=i.hierarchy,u=B(t),g=E(t)?function(t,e){var n;return t.eachAfter(function(t){for(var r=t.children||[],i=0;i<r.length;i++)if(B(r[i])===e)return n=t.copy()}),n||t}(l,u):m(l,u),h={data:[{level:B(g)}],traces:[s.index]},d={frame:{redraw:!1,duration:I.CLICK_TRANSITION_TIME},transition:{duration:I.CLICK_TRANSITION_TIME,easing:I.CLICK_TRANSITION_EASING},mode:"immediate",fromcurrent:!0};a.loneUnhover(n._hoverlayer.node()),o.call("animate",e,h,d)}})}function w(t,e){var n=t.data.data,r={curveNumber:e.index,pointNumber:n.i,data:e._input,fullData:e};return h(r,e,n.i),r}e.exports=function(t,e,n,i){var o,a,s=t._fullLayout._sunburstlayer,A=!n,l=n&&n.duration>0;(o=s.selectAll("g.trace.sunburst").data(e,function(t){return t[0].trace.uid})).enter().append("g").classed("trace",!0).classed("sunburst",!0).attr("stroke-linejoin","round"),o.order(),l?(i&&(a=i()),r.transition().duration(n.duration).ease(n.easing).each("end",function(){a&&a()}).each("interrupt",function(){a&&a()}).each(function(){s.selectAll("g.trace").each(function(e){C(t,e,this,n)})})):o.each(function(e){C(t,e,this,n)}),A&&o.exit().remove()}},{"../../components/color":593,"../../components/drawing":614,"../../components/fx":632,"../../components/fx/helpers":628,"../../lib":719,"../../lib/events":709,"../../lib/setcursor":739,"../../lib/svg_text_utils":743,"../../registry":847,"../pie/helpers":1091,"../pie/plot":1095,"./constants":1217,"./style":1223,d3:163,"d3-hierarchy":157}],1223:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/color"),o=t("../../lib");function a(t,e,n){var r=e.data.data,a=!e.children,s=r.i,A=o.castOption(n,s,"marker.line.color")||i.defaultLine,l=o.castOption(n,s,"marker.line.width")||0;t.style("stroke-width",l).call(i.fill,r.color).call(i.stroke,A).style("opacity",a?n.leaf.opacity:null)}e.exports={style:function(t){t._fullLayout._sunburstlayer.selectAll(".trace").each(function(t){var e=r.select(this),n=t[0].trace;e.style("opacity",n.opacity),e.selectAll("path.surface").each(function(t){r.select(this).call(a,t,n)})})},styleOne:a}},{"../../components/color":593,"../../lib":719,d3:163}],1224:[function(t,e,n){"use strict";var r=t("../../components/color"),i=t("../../components/colorscale/attributes"),o=t("../../components/fx/hovertemplate_attributes"),a=t("../../plots/attributes"),s=t("../../lib/extend").extendFlat,A=t("../../plot_api/edit_types").overrideAll;function l(t){return{show:{valType:"boolean",dflt:!1},start:{valType:"number",dflt:null,editType:"plot"},end:{valType:"number",dflt:null,editType:"plot"},size:{valType:"number",dflt:null,min:0,editType:"plot"},project:{x:{valType:"boolean",dflt:!1},y:{valType:"boolean",dflt:!1},z:{valType:"boolean",dflt:!1}},color:{valType:"color",dflt:r.defaultLine},usecolormap:{valType:"boolean",dflt:!1},width:{valType:"number",min:1,max:16,dflt:2},highlight:{valType:"boolean",dflt:!0},highlightcolor:{valType:"color",dflt:r.defaultLine},highlightwidth:{valType:"number",min:1,max:16,dflt:2}}}var c=e.exports=A(s({z:{valType:"data_array"},x:{valType:"data_array"},y:{valType:"data_array"},text:{valType:"string",dflt:"",arrayOk:!0},hovertext:{valType:"string",dflt:"",arrayOk:!0},hovertemplate:o(),connectgaps:{valType:"boolean",dflt:!1,editType:"calc"},surfacecolor:{valType:"data_array"}},i("",{colorAttr:"z or surfacecolor",showScaleDflt:!0,autoColorDflt:!1,editTypeOverride:"calc"}),{contours:{x:l(),y:l(),z:l()},hidesurface:{valType:"boolean",dflt:!1},lightposition:{x:{valType:"number",min:-1e5,max:1e5,dflt:10},y:{valType:"number",min:-1e5,max:1e5,dflt:1e4},z:{valType:"number",min:-1e5,max:1e5,dflt:0}},lighting:{ambient:{valType:"number",min:0,max:1,dflt:.8},diffuse:{valType:"number",min:0,max:1,dflt:.8},specular:{valType:"number",min:0,max:2,dflt:.05},roughness:{valType:"number",min:0,max:1,dflt:.5},fresnel:{valType:"number",min:0,max:5,dflt:.2}},opacity:{valType:"number",min:0,max:1,dflt:1},_deprecated:{zauto:s({},i.zauto,{}),zmin:s({},i.zmin,{}),zmax:s({},i.zmax,{})},hoverinfo:s({},a.hoverinfo)}),"calc","nested");c.x.editType=c.y.editType=c.z.editType="calc+clearAxisTypes",c.transforms=void 0},{"../../components/color":593,"../../components/colorscale/attributes":600,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plots/attributes":764}],1225:[function(t,e,n){"use strict";var r=t("../../components/colorscale/calc");e.exports=function(t,e){e.surfacecolor?r(t,e,{vals:e.surfacecolor,containerStr:"",cLetter:"c"}):r(t,e,{vals:e.z,containerStr:"",cLetter:"c"})}},{"../../components/colorscale/calc":601}],1226:[function(t,e,n){"use strict";var r=t("gl-surface3d"),i=t("ndarray"),o=t("ndarray-homography"),a=t("ndarray-fill"),s=t("../../lib").isArrayOrTypedArray,A=t("../../lib/gl_format_color").parseColorScale,l=t("../../lib/str2rgbarray"),c=t("../../components/colorscale").extractOpts,u=t("../heatmap/interp2d"),g=t("../heatmap/find_empties");function h(t,e,n){this.scene=t,this.uid=n,this.surface=e,this.data=null,this.showContour=[!1,!1,!1],this.contourStart=[null,null,null],this.contourEnd=[null,null,null],this.contourSize=[0,0,0],this.minValues=[1/0,1/0,1/0],this.maxValues=[-1/0,-1/0,-1/0],this.dataScaleX=1,this.dataScaleY=1,this.refineData=!0,this.objectOffset=[0,0,0]}var d=h.prototype;d.getXat=function(t,e,n,r){var i=s(this.data.x)?s(this.data.x[0])?this.data.x[e][t]:this.data.x[t]:t;return void 0===n?i:r.d2l(i,0,n)},d.getYat=function(t,e,n,r){var i=s(this.data.y)?s(this.data.y[0])?this.data.y[e][t]:this.data.y[e]:e;return void 0===n?i:r.d2l(i,0,n)},d.getZat=function(t,e,n,r){var i=this.data.z[e][t];return null===i&&this.data.connectgaps&&this.data._interpolatedZ&&(i=this.data._interpolatedZ[e][t]),void 0===n?i:r.d2l(i,0,n)},d.handlePick=function(t){if(t.object===this.surface){var e=(t.data.index[0]-1)/this.dataScaleX-1,n=(t.data.index[1]-1)/this.dataScaleY-1,r=Math.max(Math.min(Math.round(e),this.data.z[0].length-1),0),i=Math.max(Math.min(Math.round(n),this.data._ylength-1),0);t.index=[r,i],t.traceCoordinate=[this.getXat(r,i),this.getYat(r,i),this.getZat(r,i)],t.dataCoordinate=[this.getXat(r,i,this.data.xcalendar,this.scene.fullSceneLayout.xaxis),this.getYat(r,i,this.data.ycalendar,this.scene.fullSceneLayout.yaxis),this.getZat(r,i,this.data.zcalendar,this.scene.fullSceneLayout.zaxis)];for(var o=0;o<3;o++){var a=t.dataCoordinate[o];null!=a&&(t.dataCoordinate[o]*=this.scene.dataScale[o])}var s=this.data.hovertext||this.data.text;return Array.isArray(s)&&s[i]&&void 0!==s[i][r]?t.textLabel=s[i][r]:t.textLabel=s||"",t.data.dataCoordinate=t.dataCoordinate.slice(),this.surface.highlight(t.data),this.scene.glplot.spikes.position=t.dataCoordinate,!0}};var f=[2,3,5,7,11,13,17,19,23,29,31,37,41,43,47,53,59,61,67,71,73,79,83,89,97,101,103,107,109,113,127,131,137,139,149,151,157,163,167,173,179,181,191,193,197,199,211,223,227,229,233,239,241,251,257,263,269,271,277,281,283,293,307,311,313,317,331,337,347,349,353,359,367,373,379,383,389,397,401,409,419,421,431,433,439,443,449,457,461,463,467,479,487,491,499,503,509,521,523,541,547,557,563,569,571,577,587,593,599,601,607,613,617,619,631,641,643,647,653,659,661,673,677,683,691,701,709,719,727,733,739,743,751,757,761,769,773,787,797,809,811,821,823,827,829,839,853,857,859,863,877,881,883,887,907,911,919,929,937,941,947,953,967,971,977,983,991,997,1009,1013,1019,1021,1031,1033,1039,1049,1051,1061,1063,1069,1087,1091,1093,1097,1103,1109,1117,1123,1129,1151,1153,1163,1171,1181,1187,1193,1201,1213,1217,1223,1229,1231,1237,1249,1259,1277,1279,1283,1289,1291,1297,1301,1303,1307,1319,1321,1327,1361,1367,1373,1381,1399,1409,1423,1427,1429,1433,1439,1447,1451,1453,1459,1471,1481,1483,1487,1489,1493,1499,1511,1523,1531,1543,1549,1553,1559,1567,1571,1579,1583,1597,1601,1607,1609,1613,1619,1621,1627,1637,1657,1663,1667,1669,1693,1697,1699,1709,1721,1723,1733,1741,1747,1753,1759,1777,1783,1787,1789,1801,1811,1823,1831,1847,1861,1867,1871,1873,1877,1879,1889,1901,1907,1913,1931,1933,1949,1951,1973,1979,1987,1993,1997,1999,2003,2011,2017,2027,2029,2039,2053,2063,2069,2081,2083,2087,2089,2099,2111,2113,2129,2131,2137,2141,2143,2153,2161,2179,2203,2207,2213,2221,2237,2239,2243,2251,2267,2269,2273,2281,2287,2293,2297,2309,2311,2333,2339,2341,2347,2351,2357,2371,2377,2381,2383,2389,2393,2399,2411,2417,2423,2437,2441,2447,2459,2467,2473,2477,2503,2521,2531,2539,2543,2549,2551,2557,2579,2591,2593,2609,2617,2621,2633,2647,2657,2659,2663,2671,2677,2683,2687,2689,2693,2699,2707,2711,2713,2719,2729,2731,2741,2749,2753,2767,2777,2789,2791,2797,2801,2803,2819,2833,2837,2843,2851,2857,2861,2879,2887,2897,2903,2909,2917,2927,2939,2953,2957,2963,2969,2971,2999];function p(t,e){if(t<e)return 0;for(var n=0;0===Math.floor(t%e);)t/=e,n++;return n}function I(t){for(var e=[],n=0;n<f.length;n++){var r=f[n];e.push(p(t,r))}return e}function C(t){for(var e=I(t),n=t,r=0;r<f.length;r++)if(e[r]>0){n=f[r];break}return n}function m(t,e){if(!(t<1||e<1)){for(var n=I(t),r=I(e),i=1,o=0;o<f.length;o++)i*=Math.pow(f[o],Math.max(n[o],r[o]));return i}}d.calcXnums=function(t){var e,n=[];for(e=1;e<t;e++){var r=this.getXat(e-1,0),i=this.getXat(e,0);n[e-1]=i!==r&&null!=r&&null!=i?Math.abs(i-r):0}var o=0;for(e=1;e<t;e++)o+=n[e-1];for(e=1;e<t;e++)0===n[e-1]?n[e-1]=1:n[e-1]=Math.round(o/n[e-1]);return n},d.calcYnums=function(t){var e,n=[];for(e=1;e<t;e++){var r=this.getYat(0,e-1),i=this.getYat(0,e);n[e-1]=i!==r&&null!=r&&null!=i?Math.abs(i-r):0}var o=0;for(e=1;e<t;e++)o+=n[e-1];for(e=1;e<t;e++)0===n[e-1]?n[e-1]=1:n[e-1]=Math.round(o/n[e-1]);return n};var y=[1,2,4,6,12,24,36,48,60,120,180,240,360,720,840,1260],E=y[9],v=y[13];function B(t,e){for(var n=!1,r=0;r<t.length;r++)if(e===t[r]){n=!0;break}!1===n&&t.push(e)}d.estimateScale=function(t,e){for(var n=1+function(t){if(0!==t.length){for(var e=1,n=0;n<t.length;n++)e=m(e,t[n]);return e}}(0===e?this.calcXnums(t):this.calcYnums(t));n<E;)n*=2;for(;n>v;)n--,n/=C(n),++n<E&&(n=v);var r=Math.round(n/t);return r>1?r:1},d.refineCoords=function(t){for(var e=this.dataScaleX,n=this.dataScaleY,r=t[0].shape[0],a=t[0].shape[1],s=0|Math.floor(t[0].shape[0]*e+1),A=0|Math.floor(t[0].shape[1]*n+1),l=1+r+1,c=1+a+1,u=i(new Float32Array(l*c),[l,c]),g=0;g<t.length;++g){this.surface.padField(u,t[g]);var h=i(new Float32Array(s*A),[s,A]);o(h,u,[e,0,0,0,n,0,0,0,1]),t[g]=h}},d.setContourLevels=function(){var t,e,n,r=[[],[],[]],i=[!1,!1,!1],o=!1;for(t=0;t<3;++t)if(this.showContour[t]&&(o=!0,this.contourSize[t]>0&&null!==this.contourStart[t]&&null!==this.contourEnd[t]&&this.contourEnd[t]>this.contourStart[t]))for(i[t]=!0,e=this.contourStart[t];e<this.contourEnd[t];e+=this.contourSize[t])n=e*this.scene.dataScale[t],B(r[t],n);if(o){var a=[[],[],[]];for(t=0;t<3;++t)this.showContour[t]&&(a[t]=i[t]?r[t]:this.scene.contourLevels[t]);this.surface.update({levels:a})}},d.update=function(t){var e,n,r,o,s=this.scene,h=s.fullSceneLayout,d=this.surface,f=t.opacity,p=A(t,f),I=s.dataScale,C=t.z[0].length,m=t._ylength,y=s.contourLevels;this.data=t;var E=[];for(e=0;e<3;e++)for(E[e]=[],n=0;n<C;n++)E[e][n]=[];for(n=0;n<C;n++)for(r=0;r<m;r++)E[0][n][r]=this.getXat(n,r,t.xcalendar,h.xaxis),E[1][n][r]=this.getYat(n,r,t.ycalendar,h.yaxis),E[2][n][r]=this.getZat(n,r,t.zcalendar,h.zaxis);if(t.connectgaps)for(t._emptypoints=g(E[2]),u(E[2],t._emptypoints),t._interpolatedZ=[],n=0;n<C;n++)for(t._interpolatedZ[n]=[],r=0;r<m;r++)t._interpolatedZ[n][r]=E[2][n][r];for(e=0;e<3;e++)for(n=0;n<C;n++)for(r=0;r<m;r++)null==(o=E[e][n][r])?E[e][n][r]=NaN:o=E[e][n][r]*=I[e];for(e=0;e<3;e++)for(n=0;n<C;n++)for(r=0;r<m;r++)null!=(o=E[e][n][r])&&(this.minValues[e]>o&&(this.minValues[e]=o),this.maxValues[e]<o&&(this.maxValues[e]=o));for(e=0;e<3;e++)this.objectOffset[e]=.5*(this.minValues[e]+this.maxValues[e]);for(e=0;e<3;e++)for(n=0;n<C;n++)for(r=0;r<m;r++)null!=(o=E[e][n][r])&&(E[e][n][r]-=this.objectOffset[e]);var B=[i(new Float32Array(C*m),[C,m]),i(new Float32Array(C*m),[C,m]),i(new Float32Array(C*m),[C,m])];a(B[0],function(t,e){return E[0][t][e]}),a(B[1],function(t,e){return E[1][t][e]}),a(B[2],function(t,e){return E[2][t][e]}),E=[];var b={colormap:p,levels:[[],[],[]],showContour:[!0,!0,!0],showSurface:!t.hidesurface,contourProject:[[!1,!1,!1],[!1,!1,!1],[!1,!1,!1]],contourWidth:[1,1,1],contourColor:[[1,1,1,1],[1,1,1,1],[1,1,1,1]],contourTint:[1,1,1],dynamicColor:[[1,1,1,1],[1,1,1,1],[1,1,1,1]],dynamicWidth:[1,1,1],dynamicTint:[1,1,1],opacity:t.opacity},Q=c(t);if(b.intensityBounds=[Q.min,Q.max],t.surfacecolor){var w=i(new Float32Array(C*m),[C,m]);a(w,function(e,n){return t.surfacecolor[n][e]}),B.push(w)}else b.intensityBounds[0]*=I[2],b.intensityBounds[1]*=I[2];(v<B[0].shape[0]||v<B[0].shape[1])&&(this.refineData=!1),!0===this.refineData&&(this.dataScaleX=this.estimateScale(B[0].shape[0],0),this.dataScaleY=this.estimateScale(B[0].shape[1],1),1===this.dataScaleX&&1===this.dataScaleY||this.refineCoords(B)),t.surfacecolor&&(b.intensity=B.pop());var x=[!0,!0,!0],_=["x","y","z"];for(e=0;e<3;++e){var D=t.contours[_[e]];x[e]=D.highlight,b.showContour[e]=D.show||D.highlight,b.showContour[e]&&(b.contourProject[e]=[D.project.x,D.project.y,D.project.z],D.show?(this.showContour[e]=!0,b.levels[e]=y[e],d.highlightColor[e]=b.contourColor[e]=l(D.color),D.usecolormap?d.highlightTint[e]=b.contourTint[e]=0:d.highlightTint[e]=b.contourTint[e]=1,b.contourWidth[e]=D.width,this.contourStart[e]=D.start,this.contourEnd[e]=D.end,this.contourSize[e]=D.size):(this.showContour[e]=!1,this.contourStart[e]=null,this.contourEnd[e]=null,this.contourSize[e]=0),D.highlight&&(b.dynamicColor[e]=l(D.highlightcolor),b.dynamicWidth[e]=D.highlightwidth))}(function(t){var e=t[0].rgb,n=t[t.length-1].rgb;return e[0]===n[0]&&e[1]===n[1]&&e[2]===n[2]&&e[3]===n[3]})(p)&&(b.vertexColor=!0),b.objectOffset=this.objectOffset,b.coords=B,d.update(b),d.visible=t.visible,d.enableDynamic=x,d.enableHighlight=x,d.snapToData=!0,"lighting"in t&&(d.ambientLight=t.lighting.ambient,d.diffuseLight=t.lighting.diffuse,d.specularLight=t.lighting.specular,d.roughness=t.lighting.roughness,d.fresnel=t.lighting.fresnel),"lightposition"in t&&(d.lightPosition=[t.lightposition.x,t.lightposition.y,t.lightposition.z]),f&&f<1&&(d.supportsTransparency=!0)},d.dispose=function(){this.scene.glplot.remove(this.surface),this.surface.dispose()},e.exports=function(t,e){var n=t.glplot.gl,i=r({gl:n}),o=new h(t,i,e.uid);return i._trace=o,o.update(e),t.glplot.add(i),o}},{"../../components/colorscale":605,"../../lib":719,"../../lib/gl_format_color":716,"../../lib/str2rgbarray":742,"../heatmap/find_empties":1007,"../heatmap/interp2d":1010,"gl-surface3d":316,ndarray:450,"ndarray-fill":440,"ndarray-homography":442}],1227:[function(t,e,n){"use strict";var r=t("../../registry"),i=t("../../lib"),o=t("../../components/colorscale/defaults"),a=t("./attributes");function s(t,e,n){e in t&&!(n in t)&&(t[n]=t[e])}e.exports=function(t,e,n,A){var l,c;function u(n,r){return i.coerce(t,e,a,n,r)}var g=u("x"),h=u("y"),d=u("z");if(!d||!d.length||g&&g.length<1||h&&h.length<1)e.visible=!1;else{e._xlength=Array.isArray(g)&&i.isArrayOrTypedArray(g[0])?d.length:d[0].length,e._ylength=d.length,r.getComponentMethod("calendars","handleTraceDefaults")(t,e,["x","y","z"],A),u("text"),u("hovertext"),u("hovertemplate"),["lighting.ambient","lighting.diffuse","lighting.specular","lighting.roughness","lighting.fresnel","lightposition.x","lightposition.y","lightposition.z","hidesurface","connectgaps","opacity"].forEach(function(t){u(t)});var f=u("surfacecolor"),p=["x","y","z"];for(l=0;l<3;++l){var I="contours."+p[l],C=u(I+".show"),m=u(I+".highlight");if(C||m)for(c=0;c<3;++c)u(I+".project."+p[c]);C&&(u(I+".color"),u(I+".width"),u(I+".usecolormap")),m&&(u(I+".highlightcolor"),u(I+".highlightwidth")),u(I+".start"),u(I+".end"),u(I+".size")}f||(s(t,"zmin","cmin"),s(t,"zmax","cmax"),s(t,"zauto","cauto")),o(t,e,A,u,{prefix:"",cLetter:"c"}),e._length=null}}},{"../../components/colorscale/defaults":603,"../../lib":719,"../../registry":847,"./attributes":1224}],1228:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),colorbar:{min:"cmin",max:"cmax"},calc:t("./calc"),plot:t("./convert"),moduleType:"trace",name:"surface",basePlotModule:t("../../plots/gl3d"),categories:["gl3d","2dMap","noOpacity"],meta:{}}},{"../../plots/gl3d":807,"./attributes":1224,"./calc":1225,"./convert":1226,"./defaults":1227}],1229:[function(t,e,n){"use strict";var r=t("../../components/annotations/attributes"),i=t("../../lib/extend").extendFlat,o=t("../../plot_api/edit_types").overrideAll,a=t("../../plots/font_attributes"),s=t("../../plots/domain").attributes;t("../../constants/docs").FORMAT_LINK,(e.exports=o({domain:s({name:"table",trace:!0}),columnwidth:{valType:"number",arrayOk:!0,dflt:null},columnorder:{valType:"data_array"},header:{values:{valType:"data_array",dflt:[]},format:{valType:"data_array",dflt:[]},prefix:{valType:"string",arrayOk:!0,dflt:null},suffix:{valType:"string",arrayOk:!0,dflt:null},height:{valType:"number",dflt:28},align:i({},r.align,{arrayOk:!0}),line:{width:{valType:"number",arrayOk:!0,dflt:1},color:{valType:"color",arrayOk:!0,dflt:"grey"}},fill:{color:{valType:"color",arrayOk:!0,dflt:"white"}},font:i({},a({arrayOk:!0}))},cells:{values:{valType:"data_array",dflt:[]},format:{valType:"data_array",dflt:[]},prefix:{valType:"string",arrayOk:!0,dflt:null},suffix:{valType:"string",arrayOk:!0,dflt:null},height:{valType:"number",dflt:20},align:i({},r.align,{arrayOk:!0}),line:{width:{valType:"number",arrayOk:!0,dflt:1},color:{valType:"color",arrayOk:!0,dflt:"grey"}},fill:{color:{valType:"color",arrayOk:!0,dflt:"white"}},font:i({},a({arrayOk:!0}))}},"calc","from-root")).transforms=void 0},{"../../components/annotations/attributes":576,"../../constants/docs":690,"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plots/domain":792,"../../plots/font_attributes":793}],1230:[function(t,e,n){"use strict";var r=t("../../plots/get_data").getModuleCalcData,i=t("./plot");n.name="table",n.plot=function(t){var e=r(t.calcdata,"table")[0];e.length&&i(t,e)},n.clean=function(t,e,n,r){var i=r._has&&r._has("table"),o=e._has&&e._has("table");i&&!o&&r._paperdiv.selectAll(".table").remove()}},{"../../plots/get_data":802,"./plot":1237}],1231:[function(t,e,n){"use strict";var r=t("../../lib/gup").wrap;e.exports=function(){return r({})}},{"../../lib/gup":717}],1232:[function(t,e,n){"use strict";e.exports={cellPad:8,columnExtentOffset:10,columnTitleOffset:28,emptyHeaderHeight:16,latexCheck:/^\$.*\$$/,goldenRatio:1.618,lineBreaker:"<br>",maxDimensionCount:60,overdrag:45,releaseTransitionDuration:120,releaseTransitionEase:"cubic-out",scrollbarCaptureWidth:18,scrollbarHideDelay:1e3,scrollbarHideDuration:1e3,scrollbarOffset:5,scrollbarWidth:8,transitionDuration:100,transitionEase:"cubic-out",uplift:5,wrapSpacer:" ",wrapSplitCharacter:" ",cn:{table:"table",tableControlView:"table-control-view",scrollBackground:"scroll-background",yColumn:"y-column",columnBlock:"column-block",scrollAreaClip:"scroll-area-clip",scrollAreaClipRect:"scroll-area-clip-rect",columnBoundary:"column-boundary",columnBoundaryClippath:"column-boundary-clippath",columnBoundaryRect:"column-boundary-rect",columnCells:"column-cells",columnCell:"column-cell",cellRect:"cell-rect",cellText:"cell-text",cellTextHolder:"cell-text-holder",scrollbarKit:"scrollbar-kit",scrollbar:"scrollbar",scrollbarSlider:"scrollbar-slider",scrollbarGlyph:"scrollbar-glyph",scrollbarCaptureZone:"scrollbar-capture-zone"}}},{}],1233:[function(t,e,n){"use strict";var r=t("./constants"),i=t("../../lib/extend").extendFlat,o=t("fast-isnumeric");function a(t){if(Array.isArray(t)){for(var e=0,n=0;n<t.length;n++)e=Math.max(e,a(t[n]));return e}return t}function s(t,e){return t+e}function A(t){var e,n=t.slice(),r=1/0,i=0;for(e=0;e<n.length;e++)Array.isArray(n[e])||(n[e]=[n[e]]),r=Math.min(r,n[e].length),i=Math.max(i,n[e].length);if(r!==i)for(e=0;e<n.length;e++){var o=i-n[e].length;o&&(n[e]=n[e].concat(l(o)))}return n}function l(t){for(var e=new Array(t),n=0;n<t;n++)e[n]="";return e}function c(t){return t.calcdata.columns.reduce(function(e,n){return n.xIndex<t.xIndex?e+n.columnWidth:e},0)}function u(t,e){return Object.keys(t).map(function(n){return i({},t[n],{auxiliaryBlocks:e})})}function g(t,e){for(var n,r={},i=0,o=0,a={firstRowIndex:null,lastRowIndex:null,rows:[]},s=0,A=0,l=0;l<t.length;l++)n=t[l],a.rows.push({rowIndex:l,rowHeight:n}),((o+=n)>=e||l===t.length-1)&&(r[i]=a,a.key=A++,a.firstRowIndex=s,a.lastRowIndex=l,a={firstRowIndex:null,lastRowIndex:null,rows:[]},i+=o,s=l+1,o=0);return r}e.exports=function(t,e){var n=A(e.cells.values),h=function(t){return t.slice(e.header.values.length,t.length)},d=A(e.header.values);d.length&&!d[0].length&&(d[0]=[""],d=A(d));var f=d.concat(h(n).map(function(){return l((d[0]||[""]).length)})),p=e.domain,I=Math.floor(t._fullLayout._size.w*(p.x[1]-p.x[0])),C=Math.floor(t._fullLayout._size.h*(p.y[1]-p.y[0])),m=e.header.values.length?f[0].map(function(){return e.header.height}):[r.emptyHeaderHeight],y=n.length?n[0].map(function(){return e.cells.height}):[],E=m.reduce(s,0),v=g(y,C-E+r.uplift),B=u(g(m,E),[]),b=u(v,B),Q={},w=e._fullInput.columnorder.concat(h(n.map(function(t,e){return e}))),x=f.map(function(t,n){var r=Array.isArray(e.columnwidth)?e.columnwidth[Math.min(n,e.columnwidth.length-1)]:e.columnwidth;return o(r)?Number(r):1}),_=x.reduce(s,0);x=x.map(function(t){return t/_*I});var D=Math.max(a(e.header.line.width),a(e.cells.line.width)),M={key:e.uid+t._context.staticPlot,translateX:p.x[0]*t._fullLayout._size.w,translateY:t._fullLayout._size.h*(1-p.y[1]),size:t._fullLayout._size,width:I,maxLineWidth:D,height:C,columnOrder:w,groupHeight:C,rowBlocks:b,headerRowBlocks:B,scrollY:0,cells:i({},e.cells,{values:n}),headerCells:i({},e.header,{values:f}),gdColumns:f.map(function(t){return t[0]}),gdColumnsOriginalOrder:f.map(function(t){return t[0]}),prevPages:[0,0],scrollbarState:{scrollbarScrollInProgress:!1},columns:f.map(function(t,e){var n=Q[t];return Q[t]=(n||0)+1,{key:t+"__"+Q[t],label:t,specIndex:e,xIndex:w[e],xScale:c,x:void 0,calcdata:void 0,columnWidth:x[e]}})};return M.columns.forEach(function(t){t.calcdata=M,t.x=c(t)}),M}},{"../../lib/extend":710,"./constants":1232,"fast-isnumeric":225}],1234:[function(t,e,n){"use strict";var r=t("../../lib/extend").extendFlat;n.splitToPanels=function(t){var e=[0,0],n=r({},t,{key:"header",type:"header",page:0,prevPages:e,currentRepaint:[null,null],dragHandle:!0,values:t.calcdata.headerCells.values[t.specIndex],rowBlocks:t.calcdata.headerRowBlocks,calcdata:r({},t.calcdata,{cells:t.calcdata.headerCells})});return[r({},t,{key:"cells1",type:"cells",page:0,prevPages:e,currentRepaint:[null,null],dragHandle:!1,values:t.calcdata.cells.values[t.specIndex],rowBlocks:t.calcdata.rowBlocks}),r({},t,{key:"cells2",type:"cells",page:1,prevPages:e,currentRepaint:[null,null],dragHandle:!1,values:t.calcdata.cells.values[t.specIndex],rowBlocks:t.calcdata.rowBlocks}),n]},n.splitToCells=function(t){var e=function(t){var e=t.rowBlocks[t.page],n=e?e.rows[0].rowIndex:0,r=e?n+e.rows.length:0;return[n,r]}(t);return(t.values||[]).slice(e[0],e[1]).map(function(n,r){return{keyWithinBlock:r+("string"==typeof n&&n.match(/[<$&> ]/)?"_keybuster_"+Math.random():""),key:e[0]+r,column:t,calcdata:t.calcdata,page:t.page,rowBlocks:t.rowBlocks,value:n}})}},{"../../lib/extend":710}],1235:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./attributes"),o=t("../../plots/domain").defaults;e.exports=function(t,e,n,a){function s(n,o){return r.coerce(t,e,i,n,o)}o(e,a,s),s("columnwidth"),s("header.values"),s("header.format"),s("header.align"),s("header.prefix"),s("header.suffix"),s("header.height"),s("header.line.width"),s("header.line.color"),s("header.fill.color"),r.coerceFont(s,"header.font",r.extendFlat({},a.font)),function(t,e){for(var n=t.columnorder||[],r=t.header.values.length,i=n.slice(0,r),o=i.slice().sort(function(t,e){return t-e}),a=i.map(function(t){return o.indexOf(t)}),s=a.length;s<r;s++)a.push(s);e("columnorder",a)}(e,s),s("cells.values"),s("cells.format"),s("cells.align"),s("cells.prefix"),s("cells.suffix"),s("cells.height"),s("cells.line.width"),s("cells.line.color"),s("cells.fill.color"),r.coerceFont(s,"cells.font",r.extendFlat({},a.font)),e._length=null}},{"../../lib":719,"../../plots/domain":792,"./attributes":1229}],1236:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),calc:t("./calc"),plot:t("./plot"),moduleType:"trace",name:"table",basePlotModule:t("./base_plot"),categories:["noOpacity"],meta:{}}},{"./attributes":1229,"./base_plot":1230,"./calc":1231,"./defaults":1235,"./plot":1237}],1237:[function(t,e,n){"use strict";var r=t("./constants"),i=t("d3"),o=t("../../lib/gup"),a=t("../../components/drawing"),s=t("../../lib/svg_text_utils"),A=t("../../lib").raiseToTop,l=t("../../lib").cancelTransition,c=t("./data_preparation_helper"),u=t("./data_split_helpers"),g=t("../../components/color");function h(t){return Math.ceil(t.calcdata.maxLineWidth/2)}function d(t,e){return"clip"+t._fullLayout._uid+"_scrollAreaBottomClip_"+e.key}function f(t,e){return"clip"+t._fullLayout._uid+"_columnBoundaryClippath_"+e.calcdata.key+"_"+e.specIndex}function p(t){return[].concat.apply([],t.map(function(t){return t})).map(function(t){return t.__data__})}function I(t,e,n){var a=t.selectAll("."+r.cn.scrollbarKit).data(o.repeat,o.keyFun);a.enter().append("g").classed(r.cn.scrollbarKit,!0).style("shape-rendering","geometricPrecision"),a.each(function(t){var e=t.scrollbarState;e.totalHeight=function(t){var e=t.rowBlocks;return L(e,e.length-1)+(e.length?R(e[e.length-1],1/0):1)}(t),e.scrollableAreaHeight=t.groupHeight-Q(t),e.currentlyVisibleHeight=Math.min(e.totalHeight,e.scrollableAreaHeight),e.ratio=e.currentlyVisibleHeight/e.totalHeight,e.barLength=Math.max(e.ratio*e.currentlyVisibleHeight,r.goldenRatio*r.scrollbarWidth),e.barWiggleRoom=e.currentlyVisibleHeight-e.barLength,e.wiggleRoom=Math.max(0,e.totalHeight-e.scrollableAreaHeight),e.topY=0===e.barWiggleRoom?0:t.scrollY/e.wiggleRoom*e.barWiggleRoom,e.bottomY=e.topY+e.barLength,e.dragMultiplier=e.wiggleRoom/e.barWiggleRoom}).attr("transform",function(t){return"translate("+(t.width+r.scrollbarWidth/2+r.scrollbarOffset)+" "+Q(t)+")"});var s=a.selectAll("."+r.cn.scrollbar).data(o.repeat,o.keyFun);s.enter().append("g").classed(r.cn.scrollbar,!0);var A=s.selectAll("."+r.cn.scrollbarSlider).data(o.repeat,o.keyFun);A.enter().append("g").classed(r.cn.scrollbarSlider,!0),A.attr("transform",function(t){return"translate(0 "+(t.scrollbarState.topY||0)+")"});var l=A.selectAll("."+r.cn.scrollbarGlyph).data(o.repeat,o.keyFun);l.enter().append("line").classed(r.cn.scrollbarGlyph,!0).attr("stroke","black").attr("stroke-width",r.scrollbarWidth).attr("stroke-linecap","round").attr("y1",r.scrollbarWidth/2),l.attr("y2",function(t){return t.scrollbarState.barLength-r.scrollbarWidth/2}).attr("stroke-opacity",function(t){return t.columnDragInProgress||!t.scrollbarState.barWiggleRoom||n?0:.4}),l.transition().delay(0).duration(0),l.transition().delay(r.scrollbarHideDelay).duration(r.scrollbarHideDuration).attr("stroke-opacity",0);var c=s.selectAll("."+r.cn.scrollbarCaptureZone).data(o.repeat,o.keyFun);c.enter().append("line").classed(r.cn.scrollbarCaptureZone,!0).attr("stroke","white").attr("stroke-opacity",.01).attr("stroke-width",r.scrollbarCaptureWidth).attr("stroke-linecap","butt").attr("y1",0).on("mousedown",function(n){var r=i.event.y,o=this.getBoundingClientRect(),a=n.scrollbarState,s=r-o.top,A=i.scale.linear().domain([0,a.scrollableAreaHeight]).range([0,a.totalHeight]).clamp(!0);a.topY<=s&&s<=a.bottomY||x(e,t,null,A(s-a.barLength/2))(n)}).call(i.behavior.drag().origin(function(t){return i.event.stopPropagation(),t.scrollbarState.scrollbarScrollInProgress=!0,t}).on("drag",x(e,t)).on("dragend",function(){})),c.attr("y2",function(t){return t.scrollbarState.scrollableAreaHeight}),e._context.staticPlot&&(l.remove(),c.remove())}function C(t,e,n,s){var A=function(t){var e=t.selectAll("."+r.cn.columnCell).data(u.splitToCells,function(t){return t.keyWithinBlock});return e.enter().append("g").classed(r.cn.columnCell,!0),e.exit().remove(),e}(function(t){var e=t.selectAll("."+r.cn.columnCells).data(o.repeat,o.keyFun);return e.enter().append("g").classed(r.cn.columnCells,!0),e.exit().remove(),e}(n));!function(t){t.each(function(t,e){var n=t.calcdata.cells.font,r=t.column.specIndex,i={size:E(n.size,r,e),color:E(n.color,r,e),family:E(n.family,r,e)};t.rowNumber=t.key,t.align=E(t.calcdata.cells.align,r,e),t.cellBorderWidth=E(t.calcdata.cells.line.width,r,e),t.font=i})}(A),function(t){t.attr("width",function(t){return t.column.columnWidth}).attr("stroke-width",function(t){return t.cellBorderWidth}).each(function(t){var e=i.select(this);g.stroke(e,E(t.calcdata.cells.line.color,t.column.specIndex,t.rowNumber)),g.fill(e,E(t.calcdata.cells.fill.color,t.column.specIndex,t.rowNumber))})}(function(t){var e=t.selectAll("."+r.cn.cellRect).data(o.repeat,function(t){return t.keyWithinBlock});return e.enter().append("rect").classed(r.cn.cellRect,!0),e}(A));var l=function(t){var e=t.selectAll("."+r.cn.cellText).data(o.repeat,function(t){return t.keyWithinBlock});return e.enter().append("text").classed(r.cn.cellText,!0).style("cursor",function(){return"auto"}).on("mousedown",function(){i.event.stopPropagation()}),e}(function(t){var e=t.selectAll("."+r.cn.cellTextHolder).data(o.repeat,function(t){return t.keyWithinBlock});return e.enter().append("g").classed(r.cn.cellTextHolder,!0).style("shape-rendering","geometricPrecision"),e}(A));!function(t){t.each(function(t){a.font(i.select(this),t.font)})}(l),m(l,e,s,t),S(A)}function m(t,e,n,o){t.text(function(t){var e=t.column.specIndex,n=t.rowNumber,o=t.value,a="string"==typeof o,s=a&&o.match(/<br>/i),A=!a||s;t.mayHaveMarkup=a&&o.match(/[<&>]/);var l,c="string"==typeof(l=o)&&l.match(r.latexCheck);t.latex=c;var u,g,h=c?"":E(t.calcdata.cells.prefix,e,n)||"",d=c?"":E(t.calcdata.cells.suffix,e,n)||"",f=c?null:E(t.calcdata.cells.format,e,n)||null,p=h+(f?i.format(f)(t.value):t.value)+d;if(t.wrappingNeeded=!t.wrapped&&!A&&!c&&(u=y(p)),t.cellHeightMayIncrease=s||c||t.mayHaveMarkup||(void 0===u?y(p):u),t.needsConvertToTspans=t.mayHaveMarkup||t.wrappingNeeded||t.latex,t.wrappingNeeded){var I=(" "===r.wrapSplitCharacter?p.replace(/<a href=/gi,"<a_href="):p).split(r.wrapSplitCharacter),C=" "===r.wrapSplitCharacter?I.map(function(t){return t.replace(/<a_href=/gi,"<a href=")}):I;t.fragments=C.map(function(t){return{text:t,width:null}}),t.fragments.push({fragment:r.wrapSpacer,width:null}),g=C.join(r.lineBreaker)+r.lineBreaker+r.wrapSpacer}else delete t.fragments,g=p;return g}).attr("dy",function(t){return t.needsConvertToTspans?0:"0.75em"}).each(function(t){var a=i.select(this),A=t.wrappingNeeded?D:M;t.needsConvertToTspans?s.convertToTspans(a,o,A(n,this,e,o,t)):i.select(this.parentNode).attr("transform",function(t){return"translate("+k(t)+" "+r.cellPad+")"}).attr("text-anchor",function(t){return{left:"start",center:"middle",right:"end"}[t.align]})})}function y(t){return-1!==t.indexOf(r.wrapSplitCharacter)}function E(t,e,n){if(Array.isArray(t)){var r=t[Math.min(e,t.length-1)];return Array.isArray(r)?r[Math.min(n,r.length-1)]:r}return t}function v(t,e,n){t.transition().ease(r.releaseTransitionEase).duration(r.releaseTransitionDuration).attr("transform","translate("+e.x+" "+n+")")}function B(t){return"cells"===t.type}function b(t){return"header"===t.type}function Q(t){return(t.rowBlocks.length?t.rowBlocks[0].auxiliaryBlocks:[]).reduce(function(t,e){return t+R(e,1/0)},0)}function w(t,e,n){var r=p(e)[0];if(void 0!==r){var i=r.rowBlocks,o=r.calcdata,a=L(i,i.length),s=r.calcdata.groupHeight-Q(r),A=o.scrollY=Math.max(0,Math.min(a-s,o.scrollY)),l=function(t,e,n){for(var r=[],i=0,o=0;o<t.length;o++){for(var a=t[o],s=a.rows,A=0,l=0;l<s.length;l++)A+=s[l].rowHeight;a.allRowsHeight=A,e<i+A&&e+n>i&&r.push(o),i+=A}return r}(i,A,s);1===l.length&&(l[0]===i.length-1?l.unshift(l[0]-1):l.push(l[0]+1)),l[0]%2&&l.reverse(),e.each(function(t,e){t.page=l[e],t.scrollY=A}),e.attr("transform",function(t){return"translate(0 "+(L(t.rowBlocks,t.page)-t.scrollY)+")"}),t&&(_(t,n,e,l,r.prevPages,r,0),_(t,n,e,l,r.prevPages,r,1),I(n,t))}}function x(t,e,n,o){return function(a){var s=a.calcdata?a.calcdata:a,A=e.filter(function(t){return s.key===t.key}),l=n||s.scrollbarState.dragMultiplier,c=s.scrollY;s.scrollY=void 0===o?s.scrollY+l*i.event.dy:o;var u=A.selectAll("."+r.cn.yColumn).selectAll("."+r.cn.columnBlock).filter(B);return w(t,u,A),s.scrollY===c}}function _(t,e,n,r,i,o,a){r[a]!==i[a]&&(clearTimeout(o.currentRepaint[a]),o.currentRepaint[a]=setTimeout(function(){var o=n.filter(function(t,e){return e===a&&r[e]!==i[e]});C(t,e,o,n),i[a]=r[a]}))}function D(t,e,n,o){return function(){var a=i.select(e.parentNode);a.each(function(t){var e=t.fragments;a.selectAll("tspan.line").each(function(t,n){e[n].width=this.getComputedTextLength()});var n,i,o=e[e.length-1].width,s=e.slice(0,-1),A=[],l=0,c=t.column.columnWidth-2*r.cellPad;for(t.value="";s.length;)l+(i=(n=s.shift()).width+o)>c&&(t.value+=A.join(r.wrapSpacer)+r.lineBreaker,A=[],l=0),A.push(n.text),l+=i;l&&(t.value+=A.join(r.wrapSpacer)),t.wrapped=!0}),a.selectAll("tspan.line").remove(),m(a.select("."+r.cn.cellText),n,t,o),i.select(e.parentNode.parentNode).call(S)}}function M(t,e,n,o,a){return function(){if(!a.settledY){var s=i.select(e.parentNode),A=F(a),l=a.key-A.firstRowIndex,c=A.rows[l].rowHeight,u=a.cellHeightMayIncrease?e.parentNode.getBoundingClientRect().height+2*r.cellPad:c,g=Math.max(u,c);g-A.rows[l].rowHeight&&(A.rows[l].rowHeight=g,t.selectAll("."+r.cn.columnCell).call(S),w(null,t.filter(B),0),I(n,o,!0)),s.attr("transform",function(){var t=this.parentNode.getBoundingClientRect(),e=i.select(this.parentNode).select("."+r.cn.cellRect).node().getBoundingClientRect(),n=this.transform.baseVal.consolidate(),o=e.top-t.top+(n?n.matrix.f:r.cellPad);return"translate("+k(a,i.select(this.parentNode).select("."+r.cn.cellTextHolder).node().getBoundingClientRect().width)+" "+o+")"}),a.settledY=!0}}}function k(t,e){switch(t.align){case"left":return r.cellPad;case"right":return t.column.columnWidth-(e||0)-r.cellPad;case"center":return(t.column.columnWidth-(e||0))/2;default:return r.cellPad}}function S(t){t.attr("transform",function(t){var e=t.rowBlocks[0].auxiliaryBlocks.reduce(function(t,e){return t+R(e,1/0)},0);return"translate(0 "+(R(F(t),t.key)+e)+")"}).selectAll("."+r.cn.cellRect).attr("height",function(t){return(e=F(t),n=t.key,e.rows[n-e.firstRowIndex]).rowHeight;var e,n})}function L(t,e){for(var n=0,r=e-1;r>=0;r--)n+=N(t[r]);return n}function R(t,e){for(var n=0,r=0;r<t.rows.length&&t.rows[r].rowIndex<e;r++)n+=t.rows[r].rowHeight;return n}function N(t){var e=t.allRowsHeight;if(void 0!==e)return e;for(var n=0,r=0;r<t.rows.length;r++)n+=t.rows[r].rowHeight;return t.allRowsHeight=n,n}function F(t){return t.rowBlocks[t.page]}e.exports=function(t,e){var n=!t._context.staticPlot,s=t._fullLayout._paper.selectAll("."+r.cn.table).data(e.map(function(e){var n=o.unwrap(e).trace;return c(t,n)}),o.keyFun);s.exit().remove(),s.enter().append("g").classed(r.cn.table,!0).attr("overflow","visible").style("box-sizing","content-box").style("position","absolute").style("left",0).style("overflow","visible").style("shape-rendering","crispEdges").style("pointer-events","all"),s.attr("width",function(t){return t.width+t.size.l+t.size.r}).attr("height",function(t){return t.height+t.size.t+t.size.b}).attr("transform",function(t){return"translate("+t.translateX+","+t.translateY+")"});var g=s.selectAll("."+r.cn.tableControlView).data(o.repeat,o.keyFun),m=g.enter().append("g").classed(r.cn.tableControlView,!0).style("box-sizing","content-box");n&&m.on("mousemove",function(e){g.filter(function(t){return e===t}).call(I,t)}).on("mousewheel",function(e){if(!e.scrollbarState.wheeling){e.scrollbarState.wheeling=!0;var n=e.scrollY+i.event.deltaY;x(t,g,null,n)(e)||(i.event.stopPropagation(),i.event.preventDefault()),e.scrollbarState.wheeling=!1}}).call(I,t,!0),g.attr("transform",function(t){return"translate("+t.size.l+" "+t.size.t+")"});var y=g.selectAll("."+r.cn.scrollBackground).data(o.repeat,o.keyFun);y.enter().append("rect").classed(r.cn.scrollBackground,!0).attr("fill","none"),y.attr("width",function(t){return t.width}).attr("height",function(t){return t.height}),g.each(function(e){a.setClipUrl(i.select(this),d(t,e),t)});var E=g.selectAll("."+r.cn.yColumn).data(function(t){return t.columns},o.keyFun);E.enter().append("g").classed(r.cn.yColumn,!0),E.exit().remove(),E.attr("transform",function(t){return"translate("+t.x+" 0)"}),n&&E.call(i.behavior.drag().origin(function(e){return v(i.select(this),e,-r.uplift),A(this),e.calcdata.columnDragInProgress=!0,I(g.filter(function(t){return e.calcdata.key===t.key}),t),e}).on("drag",function(t){var e=i.select(this),n=function(e){return(t===e?i.event.x:e.x)+e.columnWidth/2};t.x=Math.max(-r.overdrag,Math.min(t.calcdata.width+r.overdrag-t.columnWidth,i.event.x)),p(E).filter(function(e){return e.calcdata.key===t.calcdata.key}).sort(function(t,e){return n(t)-n(e)}).forEach(function(e,n){e.xIndex=n,e.x=t===e?e.x:e.xScale(e)}),E.filter(function(e){return t!==e}).transition().ease(r.transitionEase).duration(r.transitionDuration).attr("transform",function(t){return"translate("+t.x+" 0)"}),e.call(l).attr("transform","translate("+t.x+" -"+r.uplift+" )")}).on("dragend",function(e){var n=i.select(this),r=e.calcdata;e.x=e.xScale(e),e.calcdata.columnDragInProgress=!1,v(n,e,0),function(t,e,n){var r=e.gdColumnsOriginalOrder;e.gdColumns.sort(function(t,e){return n[r.indexOf(t)]-n[r.indexOf(e)]}),e.columnorder=n,t.emit("plotly_restyle")}(t,r,r.columns.map(function(t){return t.xIndex}))})),E.each(function(e){a.setClipUrl(i.select(this),f(t,e),t)});var Q=E.selectAll("."+r.cn.columnBlock).data(u.splitToPanels,o.keyFun);Q.enter().append("g").classed(r.cn.columnBlock,!0).attr("id",function(t){return t.key}),Q.style("cursor",function(t){return t.dragHandle?"ew-resize":t.calcdata.scrollbarState.barWiggleRoom?"ns-resize":"default"});var _=Q.filter(b),D=Q.filter(B);n&&D.call(i.behavior.drag().origin(function(t){return i.event.stopPropagation(),t}).on("drag",x(t,g,-1)).on("dragend",function(){})),C(t,g,_,Q),C(t,g,D,Q);var M=g.selectAll("."+r.cn.scrollAreaClip).data(o.repeat,o.keyFun);M.enter().append("clipPath").classed(r.cn.scrollAreaClip,!0).attr("id",function(e){return d(t,e)});var k=M.selectAll("."+r.cn.scrollAreaClipRect).data(o.repeat,o.keyFun);k.enter().append("rect").classed(r.cn.scrollAreaClipRect,!0).attr("x",-r.overdrag).attr("y",-r.uplift).attr("fill","none"),k.attr("width",function(t){return t.width+2*r.overdrag}).attr("height",function(t){return t.height+r.uplift}),E.selectAll("."+r.cn.columnBoundary).data(o.repeat,o.keyFun).enter().append("g").classed(r.cn.columnBoundary,!0);var S=E.selectAll("."+r.cn.columnBoundaryClippath).data(o.repeat,o.keyFun);S.enter().append("clipPath").classed(r.cn.columnBoundaryClippath,!0),S.attr("id",function(e){return f(t,e)});var L=S.selectAll("."+r.cn.columnBoundaryRect).data(o.repeat,o.keyFun);L.enter().append("rect").classed(r.cn.columnBoundaryRect,!0).attr("fill","none"),L.attr("width",function(t){return t.columnWidth+2*h(t)}).attr("height",function(t){return t.calcdata.height+2*h(t)+r.uplift}).attr("x",function(t){return-h(t)}).attr("y",function(t){return-h(t)}),w(null,D,g)}},{"../../components/color":593,"../../components/drawing":614,"../../lib":719,"../../lib/gup":717,"../../lib/svg_text_utils":743,"./constants":1232,"./data_preparation_helper":1233,"./data_split_helpers":1234,d3:163}],1238:[function(t,e,n){"use strict";var r=t("../box/attributes"),i=t("../../lib/extend").extendFlat;e.exports={y:r.y,x:r.x,x0:r.x0,y0:r.y0,name:i({},r.name,{}),orientation:i({},r.orientation,{}),bandwidth:{valType:"number",min:0,editType:"calc"},scalegroup:{valType:"string",dflt:"",editType:"calc"},scalemode:{valType:"enumerated",values:["width","count"],dflt:"width",editType:"calc"},spanmode:{valType:"enumerated",values:["soft","hard","manual"],dflt:"soft",editType:"calc"},span:{valType:"info_array",items:[{valType:"any",editType:"calc"},{valType:"any",editType:"calc"}],editType:"calc"},line:{color:{valType:"color",editType:"style"},width:{valType:"number",min:0,dflt:2,editType:"style"},editType:"plot"},fillcolor:r.fillcolor,points:i({},r.boxpoints,{}),jitter:i({},r.jitter,{}),pointpos:i({},r.pointpos,{}),width:i({},r.width,{}),marker:r.marker,text:r.text,hovertext:r.hovertext,hovertemplate:r.hovertemplate,box:{visible:{valType:"boolean",dflt:!1,editType:"plot"},width:{valType:"number",min:0,max:1,dflt:.25,editType:"plot"},fillcolor:{valType:"color",editType:"style"},line:{color:{valType:"color",editType:"style"},width:{valType:"number",min:0,editType:"style"},editType:"style"},editType:"plot"},meanline:{visible:{valType:"boolean",dflt:!1,editType:"plot"},color:{valType:"color",editType:"style"},width:{valType:"number",min:0,editType:"style"},editType:"plot"},side:{valType:"enumerated",values:["both","positive","negative"],dflt:"both",editType:"calc"},offsetgroup:r.offsetgroup,alignmentgroup:r.alignmentgroup,selected:r.selected,unselected:r.unselected,hoveron:{valType:"flaglist",flags:["violins","points","kde"],dflt:"violins+points+kde",extras:["all"],editType:"style"}}},{"../../lib/extend":710,"../box/attributes":880}],1239:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axes"),o=t("../box/calc"),a=t("./helpers"),s=t("../../constants/numerical").BADNUM;function A(t,e,n){var i=e.max-e.min;if(!i)return t.bandwidth?t.bandwidth:0;if(t.bandwidth)return Math.max(t.bandwidth,i/1e4);var o=n.length,a=r.stdev(n,o-1,e.mean);return Math.max(function(t,e,n){return 1.059*Math.min(e,n/1.349)*Math.pow(t,-.2)}(o,a,e.q3-e.q1),i/100)}function l(t,e,n,r){var o,a=t.spanmode,A=t.span||[],l=[e.min,e.max],c=[e.min-2*r,e.max+2*r];function u(r){var i=A[r],o="multicategory"===n.type?n.r2c(i):n.d2c(i,0,t[e.valLetter+"calendar"]);return o===s?c[r]:o}var g={type:"linear",range:o="soft"===a?c:"hard"===a?l:[u(0),u(1)]};return i.setConvert(g),g.cleanRange(),o}e.exports=function(t,e){var n=o(t,e);if(n[0].t.empty)return n;for(var s=t._fullLayout,c=i.getFromId(t,e["h"===e.orientation?"xaxis":"yaxis"]),u=1/0,g=-1/0,h=0,d=0,f=0;f<n.length;f++){var p=n[f],I=p.pts.map(a.extractVal),C=p.bandwidth=A(e,p,I),m=p.span=l(e,p,c,C);if(p.min===p.max&&0===C)m=p.span=[p.min,p.max],p.density=[{v:1,t:m[0]}],p.bandwidth=C,h=Math.max(h,1);else{var y=m[1]-m[0],E=Math.ceil(y/(C/3)),v=y/E;if(!isFinite(v)||!isFinite(E))return r.error("Something went wrong with computing the violin span"),n[0].t.empty=!0,n;var B=a.makeKDE(p,e,I);p.density=new Array(E);for(var b=0,Q=m[0];Q<m[1]+v/2;b++,Q+=v){var w=B(Q);p.density[b]={v:w,t:Q},h=Math.max(h,w)}}d=Math.max(d,I.length),u=Math.min(u,m[0]),g=Math.max(g,m[1])}var x=i.findExtremes(c,[u,g],{padded:!0});if(e._extremes[c._id]=x,e.width)n[0].t.maxKDE=h;else{var _=s._violinScaleGroupStats,D=e.scalegroup,M=_[D];M?(M.maxKDE=Math.max(M.maxKDE,h),M.maxCount=Math.max(M.maxCount,d)):_[D]={maxKDE:h,maxCount:d}}return n[0].t.labels.kde=r._(t,"kde:"),n}},{"../../constants/numerical":695,"../../lib":719,"../../plots/cartesian/axes":767,"../box/calc":881,"./helpers":1242}],1240:[function(t,e,n){"use strict";var r=t("../box/cross_trace_calc").setPositionOffset,i=["v","h"];e.exports=function(t,e){for(var n=t.calcdata,o=e.xaxis,a=e.yaxis,s=0;s<i.length;s++){for(var A=i[s],l="h"===A?a:o,c=[],u=0;u<n.length;u++){var g=n[u],h=g[0].t,d=g[0].trace;!0!==d.visible||"violin"!==d.type||h.empty||d.orientation!==A||d.xaxis!==o._id||d.yaxis!==a._id||c.push(u)}r("violin",t,c,l)}}},{"../box/cross_trace_calc":882}],1241:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../components/color"),o=t("../box/defaults"),a=t("./attributes");e.exports=function(t,e,n,s){function A(n,i){return r.coerce(t,e,a,n,i)}function l(n,i){return r.coerce2(t,e,a,n,i)}if(o.handleSampleDefaults(t,e,A,s),!1!==e.visible){A("bandwidth"),A("side"),A("width")||(A("scalegroup",e.name),A("scalemode"));var c,u=A("span");Array.isArray(u)&&(c="manual"),A("spanmode",c);var g=A("line.color",(t.marker||{}).color||n),h=A("line.width"),d=A("fillcolor",i.addOpacity(e.line.color,.5));o.handlePointsDefaults(t,e,A,{prefix:""});var f=l("box.width"),p=l("box.fillcolor",d),I=l("box.line.color",g),C=l("box.line.width",h);A("box.visible",Boolean(f||p||I||C))||(e.box={visible:!1});var m=l("meanline.color",g),y=l("meanline.width",h);A("meanline.visible",Boolean(m||y))||(e.meanline={visible:!1})}}},{"../../components/color":593,"../../lib":719,"../box/defaults":883,"./attributes":1238}],1242:[function(t,e,n){"use strict";var r=t("../../lib"),i=function(t){return 1/Math.sqrt(2*Math.PI)*Math.exp(-.5*t*t)};n.makeKDE=function(t,e,n){var r=n.length,o=i,a=t.bandwidth,s=1/(r*a);return function(t){for(var e=0,i=0;i<r;i++)e+=o((t-n[i])/a);return s*e}},n.getPositionOnKdePath=function(t,e,n){var i,o;"h"===e.orientation?(i="y",o="x"):(i="x",o="y");var a=r.findPointOnPath(t.path,n,o,{pathLength:t.pathLength}),s=t.posCenterPx,A=a[i];return[A,"both"===e.side?2*s-A:s]},n.getKdeValue=function(t,e,r){var i=t.pts.map(n.extractVal);return n.makeKDE(t,e,i)(r)/t.posDensityScale},n.extractVal=function(t){return t.v}},{"../../lib":719}],1243:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../../plots/cartesian/axes"),o=t("../box/hover"),a=t("./helpers");e.exports=function(t,e,n,s,A){var l,c,u=t.cd,g=u[0].trace,h=g.hoveron,d=-1!==h.indexOf("violins"),f=-1!==h.indexOf("kde"),p=[];if(d||f){var I=o.hoverOnBoxes(t,e,n,s);if(f&&I.length>0){var C,m,y,E,v,B=t.xa,b=t.ya;"h"===g.orientation?(v=e,C="y",y=b,m="x",E=B):(v=n,C="x",y=B,m="y",E=b);var Q=u[t.index];if(v>=Q.span[0]&&v<=Q.span[1]){var w=r.extendFlat({},t),x=E.c2p(v,!0),_=a.getKdeValue(Q,g,v),D=a.getPositionOnKdePath(Q,g,x),M=y._offset,k=y._length;w[C+"0"]=D[0],w[C+"1"]=D[1],w[m+"0"]=w[m+"1"]=x,w[m+"Label"]=m+": "+i.hoverLabelText(E,v)+", "+u[0].t.labels.kde+" "+_.toFixed(3),w.spikeDistance=I[0].spikeDistance;var S=C+"Spike";w[S]=I[0][S],I[0].spikeDistance=void 0,I[0][S]=void 0,w.hovertemplate=!1,p.push(w),(c={stroke:t.color})[C+"1"]=r.constrain(M+D[0],M,M+k),c[C+"2"]=r.constrain(M+D[1],M,M+k),c[m+"1"]=c[m+"2"]=E._offset+x}}d&&(p=p.concat(I))}-1!==h.indexOf("points")&&(l=o.hoverOnPoints(t,e,n));var L=A.selectAll(".violinline-"+g.uid).data(c?[0]:[]);return L.enter().append("line").classed("violinline-"+g.uid,!0).attr("stroke-width",1.5),L.exit().remove(),L.attr(c),"closest"===s?l?[l]:p:l?(p.push(l),p):p}},{"../../lib":719,"../../plots/cartesian/axes":767,"../box/hover":885,"./helpers":1242}],1244:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),layoutAttributes:t("./layout_attributes"),supplyDefaults:t("./defaults"),crossTraceDefaults:t("../box/defaults").crossTraceDefaults,supplyLayoutDefaults:t("./layout_defaults"),calc:t("./calc"),crossTraceCalc:t("./cross_trace_calc"),plot:t("./plot"),style:t("./style"),styleOnSelect:t("../scatter/style").styleOnSelect,hoverPoints:t("./hover"),selectPoints:t("../box/select"),moduleType:"trace",name:"violin",basePlotModule:t("../../plots/cartesian"),categories:["cartesian","svg","symbols","oriented","box-violin","showLegend","violinLayout","zoomScale"],meta:{}}},{"../../plots/cartesian":778,"../box/defaults":883,"../box/select":890,"../scatter/style":1134,"./attributes":1238,"./calc":1239,"./cross_trace_calc":1240,"./defaults":1241,"./hover":1243,"./layout_attributes":1245,"./layout_defaults":1246,"./plot":1247,"./style":1248}],1245:[function(t,e,n){"use strict";var r=t("../box/layout_attributes"),i=t("../../lib").extendFlat;e.exports={violinmode:i({},r.boxmode,{}),violingap:i({},r.boxgap,{}),violingroupgap:i({},r.boxgroupgap,{})}},{"../../lib":719,"../box/layout_attributes":887}],1246:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./layout_attributes"),o=t("../box/layout_defaults");e.exports=function(t,e,n){o._supply(t,e,n,function(n,o){return r.coerce(t,e,i,n,o)},"violin")}},{"../../lib":719,"../box/layout_defaults":888,"./layout_attributes":1245}],1247:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../lib"),o=t("../../components/drawing"),a=t("../box/plot"),s=t("../scatter/line_points"),A=t("./helpers");e.exports=function(t,e,n,l){var c=t._fullLayout,u=e.xaxis,g=e.yaxis;function h(t){var e=s(t,{xaxis:u,yaxis:g,connectGaps:!0,baseTolerance:.75,shape:"spline",simplify:!0});return o.smoothopen(e[0],1)}i.makeTraceGroups(l,n,"trace violins").each(function(t){var n=r.select(this),o=t[0],s=o.t,l=o.trace;if(!0!==l.visible||s.empty)n.remove();else{var d=s.bPos,f=s.bdPos,p=e[s.valLetter+"axis"],I=e[s.posLetter+"axis"],C="both"===l.side,m=C||"positive"===l.side,y=C||"negative"===l.side,E=n.selectAll("path.violin").data(i.identity);E.enter().append("path").style("vector-effect","non-scaling-stroke").attr("class","violin"),E.exit().remove(),E.each(function(t){var e,n,i,o,a,A,u,g,E=r.select(this),v=t.density,B=v.length,b=t.pos+d,Q=I.c2p(b);if(l.width)e=s.maxKDE/f;else{var w=c._violinScaleGroupStats[l.scalegroup];e="count"===l.scalemode?w.maxKDE/f*(w.maxCount/t.pts.length):w.maxKDE/f}if(m){for(u=new Array(B),a=0;a<B;a++)(g=u[a]={})[s.posLetter]=b+v[a].v/e,g[s.valLetter]=v[a].t;n=h(u)}if(y){for(u=new Array(B),A=0,a=B-1;A<B;A++,a--)(g=u[A]={})[s.posLetter]=b-v[a].v/e,g[s.valLetter]=v[a].t;i=h(u)}if(C)o=n+"L"+i.substr(1)+"Z";else{var x=[Q,p.c2p(v[0].t)],_=[Q,p.c2p(v[B-1].t)];"h"===l.orientation&&(x.reverse(),_.reverse()),o=m?"M"+x+"L"+n.substr(1)+"L"+_:"M"+_+"L"+i.substr(1)+"L"+x}E.attr("d",o),t.posCenterPx=Q,t.posDensityScale=e*f,t.path=E.node(),t.pathLength=t.path.getTotalLength()/(C?2:1)});var v,B,b,Q=l.box,w=Q.width,x=(Q.line||{}).width;C?(v=f*w,B=0):m?(v=[0,f*w/2],B=-x):(v=[f*w/2,0],B=x),a.plotBoxAndWhiskers(n,{pos:I,val:p},l,{bPos:d,bdPos:v,bPosPxOffset:B}),a.plotBoxMean(n,{pos:I,val:p},l,{bPos:d,bdPos:v,bPosPxOffset:B}),!l.box.visible&&l.meanline.visible&&(b=i.identity);var _=n.selectAll("path.meanline").data(b||[]);_.enter().append("path").attr("class","meanline").style("fill","none").style("vector-effect","non-scaling-stroke"),_.exit().remove(),_.each(function(t){var e=p.c2p(t.mean,!0),n=A.getPositionOnKdePath(t,l,e);r.select(this).attr("d","h"===l.orientation?"M"+e+","+n[0]+"V"+n[1]:"M"+n[0]+","+e+"H"+n[1])}),a.plotPoints(n,{x:u,y:g},l,s)}})}},{"../../components/drawing":614,"../../lib":719,"../box/plot":889,"../scatter/line_points":1125,"./helpers":1242,d3:163}],1248:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/color"),o=t("../scatter/style").stylePoints;e.exports=function(t){var e=r.select(t).selectAll("g.trace.violins");e.style("opacity",function(t){return t[0].trace.opacity}),e.each(function(e){var n=e[0].trace,a=r.select(this),s=n.box||{},A=s.line||{},l=n.meanline||{},c=l.width;a.selectAll("path.violin").style("stroke-width",n.line.width+"px").call(i.stroke,n.line.color).call(i.fill,n.fillcolor),a.selectAll("path.box").style("stroke-width",A.width+"px").call(i.stroke,A.color).call(i.fill,s.fillcolor);var u={"stroke-width":c+"px","stroke-dasharray":2*c+"px,"+c+"px"};a.selectAll("path.mean").style(u).call(i.stroke,l.color),a.selectAll("path.meanline").style(u).call(i.stroke,l.color),o(a,n,t)})}},{"../../components/color":593,"../scatter/style":1134,d3:163}],1249:[function(t,e,n){"use strict";var r=t("../../components/colorscale/attributes"),i=t("../isosurface/attributes"),o=t("../../plots/attributes"),a=t("../../lib/extend").extendFlat,s=t("../../plot_api/edit_types").overrideAll,A=e.exports=s(a({x:i.x,y:i.y,z:i.z,value:i.value,isomin:i.isomin,isomax:i.isomax,surface:i.surface,spaceframe:{show:{valType:"boolean",dflt:!1},fill:{valType:"number",min:0,max:1,dflt:1}},slices:i.slices,caps:i.caps,text:i.text,hovertext:i.hovertext,hovertemplate:i.hovertemplate},r("",{colorAttr:"`value`",showScaleDflt:!0,editTypeOverride:"calc"}),{colorbar:i.colorbar,opacity:i.opacity,opacityscale:{valType:"any",editType:"calc"},lightposition:i.lightposition,lighting:i.lighting,flatshading:i.flatshading,contour:i.contour,hoverinfo:a({},o.hoverinfo)}),"calc","nested");A.x.editType=A.y.editType=A.z.editType=A.value.editType="calc+clearAxisTypes",A.transforms=void 0},{"../../components/colorscale/attributes":600,"../../lib/extend":710,"../../plot_api/edit_types":750,"../../plots/attributes":764,"../isosurface/attributes":1048}],1250:[function(t,e,n){"use strict";var r=t("gl-mesh3d"),i=t("../../lib/gl_format_color").parseColorScale,o=t("../../lib/str2rgbarray"),a=t("../../components/colorscale").extractOpts,s=t("../../plots/gl3d/zip3"),A=t("../isosurface/convert").findNearestOnAxis,l=t("../isosurface/convert").generateIsoMeshes;function c(t,e,n){this.scene=t,this.uid=n,this.mesh=e,this.name="",this.data=null,this.showContour=!1}var u=c.prototype;u.handlePick=function(t){if(t.object===this.mesh){var e=t.data.index,n=this.data._x[e],r=this.data._y[e],i=this.data._z[e],o=this.data._Ys.length,a=this.data._Zs.length,s=A(n,this.data._Xs).id,l=A(r,this.data._Ys).id,c=A(i,this.data._Zs).id,u=t.index=c+a*l+a*o*s;t.traceCoordinate=[this.data._x[u],this.data._y[u],this.data._z[u],this.data.value[u]];var g=this.data.hovertext||this.data.text;return Array.isArray(g)&&void 0!==g[u]?t.textLabel=g[u]:g&&(t.textLabel=g),!0}},u.update=function(t){var e=this.scene,n=e.fullSceneLayout;function r(t,e,n,r){return e.map(function(e){return t.d2l(e,0,r)*n})}this.data=l(t);var A={positions:s(r(n.xaxis,t._x,e.dataScale[0],t.xcalendar),r(n.yaxis,t._y,e.dataScale[1],t.ycalendar),r(n.zaxis,t._z,e.dataScale[2],t.zcalendar)),cells:s(t._i,t._j,t._k),lightPosition:[t.lightposition.x,t.lightposition.y,t.lightposition.z],ambient:t.lighting.ambient,diffuse:t.lighting.diffuse,specular:t.lighting.specular,roughness:t.lighting.roughness,fresnel:t.lighting.fresnel,vertexNormalsEpsilon:t.lighting.vertexnormalsepsilon,faceNormalsEpsilon:t.lighting.facenormalsepsilon,opacity:t.opacity,opacityscale:t.opacityscale,contourEnable:t.contour.show,contourColor:o(t.contour.color).slice(0,3),contourWidth:t.contour.width,useFacetNormals:t.flatshading},c=a(t);A.vertexIntensity=t._intensity,A.vertexIntensityBounds=[c.min,c.max],A.colormap=i(t),this.mesh.update(A)},u.dispose=function(){this.scene.glplot.remove(this.mesh),this.mesh.dispose()},e.exports=function(t,e){var n=t.glplot.gl,i=r({gl:n}),o=new c(t,i,e.uid);return i._trace=o,o.update(e),t.glplot.add(i),o}},{"../../components/colorscale":605,"../../lib/gl_format_color":716,"../../lib/str2rgbarray":742,"../../plots/gl3d/zip3":818,"../isosurface/convert":1050,"gl-mesh3d":280}],1251:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./attributes"),o=t("../isosurface/defaults").supplyIsoDefaults;e.exports=function(t,e,n,a){function s(n,o){return r.coerce(t,e,i,n,o)}o(t,e,n,a,s);var A=s("opacityscale");"max"===A?e.opacityscale=[[0,.1],[1,1]]:"min"===A?e.opacityscale=[[0,1],[1,.1]]:"extremes"===A?e.opacityscale=function(t,e){for(var n=[],r=0;r<32;r++){var i=r/31,o=.1+.9*(1-Math.pow(Math.sin(1*i*Math.PI),2));n.push([i,Math.max(1,Math.min(0,o))])}return n}():function(t){var e=0;if(!Array.isArray(t)||t.length<2)return!1;if(!t[0]||!t[t.length-1])return!1;if(0!=+t[0][0]||1!=+t[t.length-1][0])return!1;for(var n=0;n<t.length;n++){var r=t[n];if(2!==r.length||+r[0]<e)return!1;e=+r[0]}return!0}(A)||(e.opacityscale=void 0)}},{"../../lib":719,"../isosurface/defaults":1051,"./attributes":1249}],1252:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),supplyDefaults:t("./defaults"),calc:t("../isosurface/calc"),colorbar:{min:"cmin",max:"cmax"},plot:t("./convert"),moduleType:"trace",name:"volume",basePlotModule:t("../../plots/gl3d"),categories:["gl3d"],meta:{}}},{"../../plots/gl3d":807,"../isosurface/calc":1049,"./attributes":1249,"./convert":1250,"./defaults":1251}],1253:[function(t,e,n){"use strict";var r=t("../bar/attributes"),i=t("../scatter/attributes").line,o=t("../../plots/attributes"),a=t("../../components/fx/hovertemplate_attributes"),s=t("./constants"),A=t("../../lib/extend").extendFlat,l=t("../../components/color");function c(t){return{marker:{color:A({},r.marker.color,{arrayOk:!1,editType:"style"}),line:{color:A({},r.marker.line.color,{arrayOk:!1,editType:"style"}),width:A({},r.marker.line.width,{arrayOk:!1,editType:"style"}),editType:"style"},editType:"style"},editType:"style"}}e.exports={measure:{valType:"data_array",dflt:[],editType:"calc"},base:{valType:"number",dflt:null,arrayOk:!1,editType:"calc"},x:r.x,x0:r.x0,dx:r.dx,y:r.y,y0:r.y0,dy:r.dy,hovertext:r.hovertext,hovertemplate:a({},{keys:s.eventDataKeys}),hoverinfo:A({},o.hoverinfo,{flags:["name","x","y","text","initial","delta","final"]}),textinfo:{valType:"flaglist",flags:["label","text","initial","delta","final"],extras:["none"],editType:"plot",arrayOk:!1},text:r.text,textposition:r.textposition,insidetextanchor:r.insidetextanchor,textangle:r.textangle,textfont:r.textfont,insidetextfont:r.insidetextfont,outsidetextfont:r.outsidetextfont,constraintext:r.constraintext,cliponaxis:r.cliponaxis,orientation:r.orientation,offset:r.offset,width:r.width,increasing:c(),decreasing:c(),totals:c(),connector:{line:{color:A({},i.color,{dflt:l.defaultLine}),width:A({},i.width,{editType:"plot"}),dash:i.dash,editType:"plot"},mode:{valType:"enumerated",values:["spanning","between"],dflt:"between",editType:"plot"},visible:{valType:"boolean",dflt:!0,editType:"plot"},editType:"plot"},offsetgroup:r.offsetgroup,alignmentgroup:r.alignmentgroup}},{"../../components/color":593,"../../components/fx/hovertemplate_attributes":631,"../../lib/extend":710,"../../plots/attributes":764,"../bar/attributes":857,"../scatter/attributes":1112,"./constants":1255}],1254:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axes"),i=t("../../lib").mergeArray,o=t("../scatter/calc_selection"),a=t("../../constants/numerical").BADNUM;function s(t){return"a"===t||"absolute"===t}function A(t){return"t"===t||"total"===t}e.exports=function(t,e){var n,l,c=r.getFromId(t,e.xaxis||"x"),u=r.getFromId(t,e.yaxis||"y");"h"===e.orientation?(n=c.makeCalcdata(e,"x"),l=u.makeCalcdata(e,"y")):(n=u.makeCalcdata(e,"y"),l=c.makeCalcdata(e,"x"));for(var g,h=Math.min(l.length,n.length),d=new Array(h),f=0,p=!1,I=0;I<h;I++){var C=n[I]||0,m=!1;(n[I]!==a||A(e.measure[I])||s(e.measure[I]))&&I+1<h&&(n[I+1]!==a||A(e.measure[I+1])||s(e.measure[I+1]))&&(m=!0);var y=d[I]={i:I,p:l[I],s:C,rawS:C,cNext:m};s(e.measure[I])?(f=y.s,y.isSum=!0,y.dir="totals",y.s=f):A(e.measure[I])?(y.isSum=!0,y.dir="totals",y.s=f):(y.isSum=!1,y.dir=y.rawS<0?"decreasing":"increasing",g=y.s,y.s=f+g,f+=g),"totals"===y.dir&&(p=!0),e.ids&&(y.id=String(e.ids[I])),y.v=(e.base||0)+f}return d.length&&(d[0].hasTotals=p),i(e.text,d,"tx"),i(e.hovertext,d,"htx"),o(d,e),d}},{"../../constants/numerical":695,"../../lib":719,"../../plots/cartesian/axes":767,"../scatter/calc_selection":1114}],1255:[function(t,e,n){"use strict";e.exports={eventDataKeys:["initial","delta","final"]}},{}],1256:[function(t,e,n){"use strict";var r=t("../bar/cross_trace_calc").setGroupPositions;e.exports=function(t,e){var n,i,o=t._fullLayout,a=t._fullData,s=t.calcdata,A=e.xaxis,l=e.yaxis,c=[],u=[],g=[];for(i=0;i<a.length;i++){var h=a[i];!0===h.visible&&h.xaxis===A._id&&h.yaxis===l._id&&"waterfall"===h.type&&(n=s[i],"h"===h.orientation?g.push(n):u.push(n),c.push(n))}var d={mode:o.waterfallmode,norm:o.waterfallnorm,gap:o.waterfallgap,groupgap:o.waterfallgroupgap};for(r(t,A,l,u,d),r(t,l,A,g,d),i=0;i<c.length;i++){n=c[i];for(var f=0;f<n.length;f++){var p=n[f];!1===p.isSum&&(p.s0+=0===f?0:n[f-1].s),f+1<n.length&&(n[f].nextP0=n[f+1].p0,n[f].nextS0=n[f+1].s0)}}}},{"../bar/cross_trace_calc":860}],1257:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("../bar/defaults").handleGroupingDefaults,o=t("../bar/defaults").handleText,a=t("../scatter/xy_defaults"),s=t("./attributes"),A=t("../../components/color"),l=t("../../constants/delta.js"),c=l.INCREASING.COLOR,u=l.DECREASING.COLOR;function g(t,e,n){t(e+".marker.color",n),t(e+".marker.line.color",A.defaultLine),t(e+".marker.line.width")}e.exports={supplyDefaults:function(t,e,n,i){function A(n,i){return r.coerce(t,e,s,n,i)}if(a(t,e,i,A)){A("measure"),A("orientation",e.x&&!e.y?"h":"v"),A("base"),A("offset"),A("width"),A("text"),A("hovertext"),A("hovertemplate");var l=A("textposition");o(t,e,i,A,l,{moduleHasSelected:!1,moduleHasUnselected:!1,moduleHasConstrain:!0,moduleHasCliponaxis:!0,moduleHasTextangle:!0,moduleHasInsideanchor:!0}),"none"!==e.textposition&&A("textinfo"),g(A,"increasing",c),g(A,"decreasing",u),g(A,"totals","#4499FF"),A("connector.visible")&&(A("connector.mode"),A("connector.line.width")&&(A("connector.line.color"),A("connector.line.dash")))}else e.visible=!1},crossTraceDefaults:function(t,e){var n,o;function a(t){return r.coerce(o._input,o,s,t)}if("group"===e.waterfallmode)for(var A=0;A<t.length;A++)n=(o=t[A])._input,i(n,o,e,a)}}},{"../../components/color":593,"../../constants/delta.js":689,"../../lib":719,"../bar/defaults":861,"../scatter/xy_defaults":1137,"./attributes":1253}],1258:[function(t,e,n){"use strict";e.exports=function(t,e){return t.x="xVal"in e?e.xVal:e.x,t.y="yVal"in e?e.yVal:e.y,"initial"in e&&(t.initial=e.initial),"delta"in e&&(t.delta=e.delta),"final"in e&&(t.final=e.final),e.xa&&(t.xaxis=e.xa),e.ya&&(t.yaxis=e.ya),t}},{}],1259:[function(t,e,n){"use strict";var r=t("../../plots/cartesian/axes").hoverLabelText,i=t("../../components/color").opacity,o=t("../bar/hover").hoverOnBars,a=t("../../constants/delta.js"),s=a.INCREASING.SYMBOL,A=a.DECREASING.SYMBOL;e.exports=function(t,e,n,a){var l=o(t,e,n,a);if(l){var c=l.cd,u=c[0].trace,g="h"===u.orientation,h=g?t.xa:t.ya,d=c[l.index],f=d.isSum?d.b+d.s:d.rawS;if(!d.isSum){l.initial=d.b+d.s-f,l.delta=f,l.final=l.initial+l.delta;var p=v(Math.abs(l.delta));l.deltaLabel=f<0?"("+p+")":p,l.finalLabel=v(l.final),l.initialLabel=v(l.initial)}var I=d.hi||u.hoverinfo,C=[];if(I&&"none"!==I&&"skip"!==I){var m="all"===I,y=I.split("+"),E=function(t){return m||-1!==y.indexOf(t)};d.isSum||(!E("final")||E(g?"x":"y")||C.push(l.finalLabel),E("delta")&&(f<0?C.push(l.deltaLabel+" "+A):C.push(l.deltaLabel+" "+s)),E("initial")&&C.push("Initial: "+l.initialLabel))}return C.length&&(l.extraText=C.join("<br>")),l.color=function(t,e){var n=t[e.dir].marker,r=n.color,o=n.line.color,a=n.line.width;return i(r)?r:i(o)&&a?o:void 0}(u,d),[l]}function v(t){return r(h,t)}}},{"../../components/color":593,"../../constants/delta.js":689,"../../plots/cartesian/axes":767,"../bar/hover":863}],1260:[function(t,e,n){"use strict";e.exports={attributes:t("./attributes"),layoutAttributes:t("./layout_attributes"),supplyDefaults:t("./defaults").supplyDefaults,crossTraceDefaults:t("./defaults").crossTraceDefaults,supplyLayoutDefaults:t("./layout_defaults"),calc:t("./calc"),crossTraceCalc:t("./cross_trace_calc"),plot:t("./plot"),style:t("./style").style,hoverPoints:t("./hover"),eventData:t("./event_data"),selectPoints:t("../bar/select"),moduleType:"trace",name:"waterfall",basePlotModule:t("../../plots/cartesian"),categories:["bar-like","cartesian","svg","oriented","showLegend","zoomScale"],meta:{}}},{"../../plots/cartesian":778,"../bar/select":868,"./attributes":1253,"./calc":1254,"./cross_trace_calc":1256,"./defaults":1257,"./event_data":1258,"./hover":1259,"./layout_attributes":1261,"./layout_defaults":1262,"./plot":1263,"./style":1264}],1261:[function(t,e,n){"use strict";e.exports={waterfallmode:{valType:"enumerated",values:["group","overlay"],dflt:"group",editType:"calc"},waterfallgap:{valType:"number",min:0,max:1,editType:"calc"},waterfallgroupgap:{valType:"number",min:0,max:1,dflt:0,editType:"calc"}}},{}],1262:[function(t,e,n){"use strict";var r=t("../../lib"),i=t("./layout_attributes");e.exports=function(t,e,n){var o=!1;function a(n,o){return r.coerce(t,e,i,n,o)}for(var s=0;s<n.length;s++){var A=n[s];if(A.visible&&"waterfall"===A.type){o=!0;break}}o&&(a("waterfallmode"),a("waterfallgap",.2),a("waterfallgroupgap"))}},{"../../lib":719,"./layout_attributes":1261}],1263:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../lib"),o=t("../../components/drawing"),a=t("../bar/plot").plot;e.exports=function(t,e,n,s){var A=t._fullLayout;a(t,e,n,s,{mode:A.waterfallmode,norm:A.waterfallmode,gap:A.waterfallgap,groupgap:A.waterfallgroupgap}),function(t,e,n,a){var s=e.xaxis,A=e.yaxis;i.makeTraceGroups(a,n,"trace bars").each(function(n){var a=r.select(this),l=n[0].trace,c=i.ensureSingle(a,"g","lines");if(l.connector&&l.connector.visible){var u="h"===l.orientation,g=l.connector.mode,h=c.selectAll("g.line").data(i.identity);h.enter().append("g").classed("line",!0),h.exit().remove();var d=h.size();h.each(function(n,a){if(a===d-1||n.cNext){var l=function(t,e,n,r){var i=[],o=[],a=r?e:n,s=r?n:e;return i[0]=a.c2p(t.s0,!0),o[0]=s.c2p(t.p0,!0),i[1]=a.c2p(t.s1,!0),o[1]=s.c2p(t.p1,!0),i[2]=a.c2p(t.nextS0,!0),o[2]=s.c2p(t.nextP0,!0),r?[i,o]:[o,i]}(n,s,A,u),c=l[0],h=l[1],f="";"spanning"===g&&!n.isSum&&a>0&&(f+=u?"M"+c[0]+","+h[1]+"V"+h[0]:"M"+c[1]+","+h[0]+"H"+c[0]),"between"!==g&&(n.isSum||a<d-1)&&(f+=u?"M"+c[1]+","+h[0]+"V"+h[1]:"M"+c[0]+","+h[1]+"H"+c[1]),void 0!==c[2]&&void 0!==h[2]&&(f+=u?"M"+c[1]+","+h[1]+"V"+h[2]:"M"+c[1]+","+h[1]+"H"+c[2]),""===f&&(f="M0,0Z"),i.ensureSingle(r.select(this),"path").attr("d",f).call(o.setClipUrl,e.layerClipId,t)}})}else c.remove()})}(t,e,n,s)}},{"../../components/drawing":614,"../../lib":719,"../bar/plot":867,d3:163}],1264:[function(t,e,n){"use strict";var r=t("d3"),i=t("../../components/drawing"),o=t("../../components/color"),a=t("../../constants/interactions").DESELECTDIM,s=t("../bar/style").styleTextPoints;e.exports={style:function(t,e,n){var A=n||r.select(t).selectAll("g.waterfalllayer").selectAll("g.trace");A.style("opacity",function(t){return t[0].trace.opacity}),A.each(function(e){var n=r.select(this),A=e[0].trace;n.selectAll(".point > path").each(function(t){if(!t.isBlank){var e=A[t.dir].marker;r.select(this).call(o.fill,e.color).call(o.stroke,e.line.color).call(i.dashLine,e.line.dash,e.line.width).style("opacity",A.selectedpoints&&!t.selected?a:1)}}),s(n,A,t),n.selectAll(".lines").each(function(){var t=A.connector.line;i.lineGroupStyle(r.select(this).selectAll("path"),t.width,t.color,t.dash)})})}}},{"../../components/color":593,"../../components/drawing":614,"../../constants/interactions":694,"../bar/style":870,d3:163}],1265:[function(t,e,n){"use strict";var r=t("../plots/cartesian/axes"),i=t("../lib"),o=t("../plot_api/plot_schema"),a=t("./helpers").pointsAccessorFunction,s=t("../constants/numerical").BADNUM;n.moduleType="transform",n.name="aggregate";var A=n.attributes={enabled:{valType:"boolean",dflt:!0,editType:"calc"},groups:{valType:"string",strict:!0,noBlank:!0,arrayOk:!0,dflt:"x",editType:"calc"},aggregations:{_isLinkedToArray:"aggregation",target:{valType:"string",editType:"calc"},func:{valType:"enumerated",values:["count","sum","avg","median","mode","rms","stddev","min","max","first","last","change","range"],dflt:"first",editType:"calc"},funcmode:{valType:"enumerated",values:["sample","population"],dflt:"sample",editType:"calc"},enabled:{valType:"boolean",dflt:!0,editType:"calc"},editType:"calc"},editType:"calc"},l=A.aggregations;function c(t,e,n,o){if(o.enabled){for(var a=o.target,A=i.nestedProperty(e,a),l=A.get(),c=function(t,e){var n=t.func,r=e.d2c,i=e.c2d;switch(n){case"count":return u;case"first":return g;case"last":return h;case"sum":return function(t,e){for(var n=0,o=0;o<e.length;o++){var a=r(t[e[o]]);a!==s&&(n+=a)}return i(n)};case"avg":return function(t,e){for(var n=0,o=0,a=0;a<e.length;a++){var A=r(t[e[a]]);A!==s&&(n+=A,o++)}return o?i(n/o):s};case"min":return function(t,e){for(var n=1/0,o=0;o<e.length;o++){var a=r(t[e[o]]);a!==s&&(n=Math.min(n,a))}return n===1/0?s:i(n)};case"max":return function(t,e){for(var n=-1/0,o=0;o<e.length;o++){var a=r(t[e[o]]);a!==s&&(n=Math.max(n,a))}return n===-1/0?s:i(n)};case"range":return function(t,e){for(var n=1/0,o=-1/0,a=0;a<e.length;a++){var A=r(t[e[a]]);A!==s&&(n=Math.min(n,A),o=Math.max(o,A))}return o===-1/0||n===1/0?s:i(o-n)};case"change":return function(t,e){var n=r(t[e[0]]),o=r(t[e[e.length-1]]);return n===s||o===s?s:i(o-n)};case"median":return function(t,e){for(var n=[],o=0;o<e.length;o++){var a=r(t[e[o]]);a!==s&&n.push(a)}if(!n.length)return s;n.sort();var A=(n.length-1)/2;return i((n[Math.floor(A)]+n[Math.ceil(A)])/2)};case"mode":return function(t,e){for(var n={},o=0,a=s,A=0;A<e.length;A++){var l=r(t[e[A]]);if(l!==s){var c=n[l]=(n[l]||0)+1;c>o&&(o=c,a=l)}}return o?i(a):s};case"rms":return function(t,e){for(var n=0,o=0,a=0;a<e.length;a++){var A=r(t[e[a]]);A!==s&&(n+=A*A,o++)}return o?i(Math.sqrt(n/o)):s};case"stddev":return function(e,n){var i,o=0,a=0,A=1,l=s;for(i=0;i<n.length&&l===s;i++)l=r(e[n[i]]);if(l===s)return s;for(;i<n.length;i++){var c=r(e[n[i]]);if(c!==s){var u=c-l;o+=u,a+=u*u,A++}}var g="sample"===t.funcmode?A-1:A;return g?Math.sqrt((a-o*o/A)/g):0}}}(o,r.getDataConversions(t,e,a,l)),d=new Array(n.length),f=0;f<n.length;f++)d[f]=c(l,n[f]);A.set(d),"count"===o.func&&i.pushUnique(e._arrayAttrs,a)}}function u(t,e){return e.length}function g(t,e){return t[e[0]]}function h(t,e){return t[e[e.length-1]]}n.supplyDefaults=function(t,e){var n,r={};function a(e,n){return i.coerce(t,r,A,e,n)}if(!a("enabled"))return r;var s=o.findArrayAttributes(e),c={};for(n=0;n<s.length;n++)c[s[n]]=1;var u=a("groups");if(!Array.isArray(u)){if(!c[u])return r.enabled=!1,r;c[u]=0}var g,h=t.aggregations||[],d=r.aggregations=new Array(h.length);function f(t,e){return i.coerce(h[n],g,l,t,e)}for(n=0;n<h.length;n++){g={_index:n};var p=f("target"),I=f("func");f("enabled")&&p&&(c[p]||"count"===I&&void 0===c[p])?("stddev"===I&&f("funcmode"),c[p]=0,d[n]=g):d[n]={enabled:!1,_index:n}}for(n=0;n<s.length;n++)c[s[n]]&&d.push({target:s[n],func:l.func.dflt,enabled:!0,_index:-1});return r},n.calcTransform=function(t,e,n){if(n.enabled){var r=n.groups,o=i.getTargetArray(e,{target:r});if(o){var s,A,l,u,g={},h={},d=[],f=a(e.transforms,n),p=o.length;for(e._length&&(p=Math.min(p,e._length)),s=0;s<p;s++)void 0===(l=g[A=o[s]])?(g[A]=d.length,u=[s],d.push(u),h[g[A]]=f(s)):(d[l].push(s),h[g[A]]=(h[g[A]]||[]).concat(f(s)));n._indexToPoints=h;var I=n.aggregations;for(s=0;s<I.length;s++)c(t,e,d,I[s]);"string"==typeof r&&c(t,e,d,{target:r,func:"first",enabled:!0}),e._length=d.length}}}},{"../constants/numerical":695,"../lib":719,"../plot_api/plot_schema":756,"../plots/cartesian/axes":767,"./helpers":1268}],1266:[function(t,e,n){"use strict";var r=t("../lib"),i=t("../registry"),o=t("../plots/cartesian/axes"),a=t("./helpers").pointsAccessorFunction,s=t("../constants/filter_ops"),A=s.COMPARISON_OPS,l=s.INTERVAL_OPS,c=s.SET_OPS;n.moduleType="transform",n.name="filter",n.attributes={enabled:{valType:"boolean",dflt:!0,editType:"calc"},target:{valType:"string",strict:!0,noBlank:!0,arrayOk:!0,dflt:"x",editType:"calc"},operation:{valType:"enumerated",values:[].concat(A).concat(l).concat(c),dflt:"=",editType:"calc"},value:{valType:"any",dflt:0,editType:"calc"},preservegaps:{valType:"boolean",dflt:!1,editType:"calc"},editType:"calc"},n.supplyDefaults=function(t){var e={};function o(i,o){return r.coerce(t,e,n.attributes,i,o)}if(o("enabled")){var a=o("target");if(r.isArrayOrTypedArray(a)&&0===a.length)return e.enabled=!1,e;o("preservegaps"),o("operation"),o("value");var s=i.getComponentMethod("calendars","handleDefaults");s(t,e,"valuecalendar",null),s(t,e,"targetcalendar",null)}return e},n.calcTransform=function(t,e,n){if(n.enabled){var i=r.getTargetArray(e,n);if(i){var s=n.target,u=i.length;e._length&&(u=Math.min(u,e._length));var g=n.targetcalendar,h=e._arrayAttrs,d=n.preservegaps;if("string"==typeof s){var f=r.nestedProperty(e,s+"calendar").get();f&&(g=f)}var p,I,C=function(t,e,n){var r=t.operation,i=t.value,o=Array.isArray(i);function a(t){return-1!==t.indexOf(r)}var s,u=function(n){return e(n,0,t.valuecalendar)},g=function(t){return e(t,0,n)};switch(a(A)?s=u(o?i[0]:i):a(l)?s=o?[u(i[0]),u(i[1])]:[u(i),u(i)]:a(c)&&(s=o?i.map(u):[u(i)]),r){case"=":return function(t){return g(t)===s};case"!=":return function(t){return g(t)!==s};case"<":return function(t){return g(t)<s};case"<=":return function(t){return g(t)<=s};case">":return function(t){return g(t)>s};case">=":return function(t){return g(t)>=s};case"[]":return function(t){var e=g(t);return e>=s[0]&&e<=s[1]};case"()":return function(t){var e=g(t);return e>s[0]&&e<s[1]};case"[)":return function(t){var e=g(t);return e>=s[0]&&e<s[1]};case"(]":return function(t){var e=g(t);return e>s[0]&&e<=s[1]};case"][":return function(t){var e=g(t);return e<=s[0]||e>=s[1]};case")(":return function(t){var e=g(t);return e<s[0]||e>s[1]};case"](":return function(t){var e=g(t);return e<=s[0]||e>s[1]};case")[":return function(t){var e=g(t);return e<s[0]||e>=s[1]};case"{}":return function(t){return-1!==s.indexOf(g(t))};case"}{":return function(t){return-1===s.indexOf(g(t))}}}(n,o.getDataToCoordFunc(t,e,s,i),g),m={},y={},E=0;d?(p=function(t){m[t.astr]=r.extendDeep([],t.get()),t.set(new Array(u))},I=function(t,e){var n=m[t.astr][e];t.get()[e]=n}):(p=function(t){m[t.astr]=r.extendDeep([],t.get()),t.set([])},I=function(t,e){var n=m[t.astr][e];t.get().push(n)}),b(p);for(var v=a(e.transforms,n),B=0;B<u;B++)C(i[B])?(b(I,B),y[E++]=v(B)):d&&E++;n._indexToPoints=y,e._length=E}}function b(t,n){for(var i=0;i<h.length;i++)t(r.nestedProperty(e,h[i]),n)}}},{"../constants/filter_ops":691,"../lib":719,"../plots/cartesian/axes":767,"../registry":847,"./helpers":1268}],1267:[function(t,e,n){"use strict";var r=t("../lib"),i=t("../plot_api/plot_schema"),o=t("../plots/plots"),a=t("./helpers").pointsAccessorFunction;function s(t,e){var n,s,A,l,c,u,g,h,d,f,p=e.transform,I=e.transformIndex,C=t.transforms[I].groups,m=a(t.transforms,p);if(!Array.isArray(C)||0===C.length)return[t];var y=r.filterUnique(C),E=new Array(y.length),v=C.length,B=i.findArrayAttributes(t),b=p.styles||[],Q={};for(n=0;n<b.length;n++)Q[b[n].target]=b[n].value;p.styles&&(f=r.keyedContainer(p,"styles","target","value.name"));var w={},x={};for(n=0;n<y.length;n++){w[u=y[n]]=n,x[u]=0,(g=E[n]=r.extendDeepNoArrays({},t))._group=u,g.transforms[I]._indexToPoints={};var _=null;for(f&&(_=f.get(u)),g.name=_||""===_?_:r.templateString(p.nameformat,{trace:t.name,group:u}),h=g.transforms,g.transforms=[],s=0;s<h.length;s++)g.transforms[s]=r.extendDeepNoArrays({},h[s]);for(s=0;s<B.length;s++)r.nestedProperty(g,B[s]).set([])}for(A=0;A<B.length;A++){for(l=B[A],s=0,d=[];s<y.length;s++)d[s]=r.nestedProperty(E[s],l).get();for(c=r.nestedProperty(t,l).get(),s=0;s<v;s++)d[w[C[s]]].push(c[s])}for(s=0;s<v;s++)(g=E[w[C[s]]]).transforms[I]._indexToPoints[x[C[s]]]=m(s),x[C[s]]++;for(n=0;n<y.length;n++)u=y[n],g=E[n],o.clearExpandedTraceDefaultColors(g),g=r.extendDeepNoArrays(g,Q[u]||{});return E}n.moduleType="transform",n.name="groupby",n.attributes={enabled:{valType:"boolean",dflt:!0,editType:"calc"},groups:{valType:"data_array",dflt:[],editType:"calc"},nameformat:{valType:"string",editType:"calc"},styles:{_isLinkedToArray:"style",target:{valType:"string",editType:"calc"},value:{valType:"any",dflt:{},editType:"calc",_compareAsJSON:!0},editType:"calc"},editType:"calc"},n.supplyDefaults=function(t,e,i){var o,a={};function s(e,i){return r.coerce(t,a,n.attributes,e,i)}if(!s("enabled"))return a;s("groups"),s("nameformat",i._dataLength>1?"%{group} (%{trace})":"%{group}");var A=t.styles,l=a.styles=[];if(A)for(o=0;o<A.length;o++){var c=l[o]={};r.coerce(A[o],l[o],n.attributes.styles,"target");var u=r.coerce(A[o],l[o],n.attributes.styles,"value");r.isPlainObject(u)?c.value=r.extendDeep({},u):u&&delete c.value}return a},n.transform=function(t,e){var n,r,i,o=[];for(r=0;r<t.length;r++)for(n=s(t[r],e),i=0;i<n.length;i++)o.push(n[i]);return o}},{"../lib":719,"../plot_api/plot_schema":756,"../plots/plots":828,"./helpers":1268}],1268:[function(t,e,n){"use strict";n.pointsAccessorFunction=function(t,e){for(var n,r,i=0;i<t.length&&(n=t[i])!==e;i++)n._indexToPoints&&!1!==n.enabled&&(r=n._indexToPoints);return r?function(t){return r[t]}:function(t){return[t]}}},{}],1269:[function(t,e,n){"use strict";var r=t("../lib"),i=t("../plots/cartesian/axes"),o=t("./helpers").pointsAccessorFunction;n.moduleType="transform",n.name="sort",n.attributes={enabled:{valType:"boolean",dflt:!0,editType:"calc"},target:{valType:"string",strict:!0,noBlank:!0,arrayOk:!0,dflt:"x",editType:"calc"},order:{valType:"enumerated",values:["ascending","descending"],dflt:"ascending",editType:"calc"},editType:"calc"},n.supplyDefaults=function(t){var e={};function i(i,o){return r.coerce(t,e,n.attributes,i,o)}return i("enabled")&&(i("target"),i("order")),e},n.calcTransform=function(t,e,n){if(n.enabled){var a=r.getTargetArray(e,n);if(a){var s=n.target,A=a.length;e._length&&(A=Math.min(A,e._length));var l,c,u=e._arrayAttrs,g=function(t,e,n,r){var i,o=new Array(r),a=new Array(r);for(i=0;i<r;i++)o[i]={v:e[i],i:i};for(o.sort(function(t,e){switch(t.order){case"ascending":return function(t,n){return e(t.v)-e(n.v)};case"descending":return function(t,n){return e(n.v)-e(t.v)}}}(t,n)),i=0;i<r;i++)a[i]=o[i].i;return a}(n,a,i.getDataToCoordFunc(t,e,s,a),A),h=o(e.transforms,n),d={};for(l=0;l<u.length;l++){var f=r.nestedProperty(e,u[l]),p=f.get(),I=new Array(A);for(c=0;c<A;c++)I[c]=p[g[c]];f.set(I)}for(c=0;c<A;c++)d[c]=h(g[c]);n._indexToPoints=d,e._length=A}}}},{"../lib":719,"../plots/cartesian/axes":767,"./helpers":1268}]},{},[25])(25)}).call(this,n(37))},function(t,e,n){"use strict";var r=n(185),i=n(100),o=n(346),a=o.FP_SAFE,s=o.BADNUM,A=t.exports={};A.nestedProperty=n(347),A.keyedContainer=n(1420),A.relativeAttr=n(1421),A.isPlainObject=n(240),A.toLogRange=n(1422),A.relinkPrivateKeys=n(1423);var l=n(186);A.isTypedArray=l.isTypedArray,A.isArrayOrTypedArray=l.isArrayOrTypedArray,A.isArray1D=l.isArray1D,A.ensureArray=l.ensureArray,A.concat=l.concat,A.maxRowLength=l.maxRowLength,A.minRowLength=l.minRowLength;var c=n(187);A.mod=c.mod,A.modHalf=c.modHalf;var u=n(1424);A.valObjectMeta=u.valObjectMeta,A.coerce=u.coerce,A.coerce2=u.coerce2,A.coerceFont=u.coerceFont,A.coerceHoverinfo=u.coerceHoverinfo,A.coerceSelectionMarkerOpacity=u.coerceSelectionMarkerOpacity,A.validate=u.validate;var g=n(1430);A.dateTime2ms=g.dateTime2ms,A.isDateTime=g.isDateTime,A.ms2DateTime=g.ms2DateTime,A.ms2DateTimeLocal=g.ms2DateTimeLocal,A.cleanDate=g.cleanDate,A.isJSDate=g.isJSDate,A.formatDate=g.formatDate,A.incrementMonth=g.incrementMonth,A.dateTick0=g.dateTick0,A.dfltRange=g.dfltRange,A.findExactDates=g.findExactDates,A.MIN_MS=g.MIN_MS,A.MAX_MS=g.MAX_MS;var h=n(1436);A.findBin=h.findBin,A.sorterAsc=h.sorterAsc,A.sorterDes=h.sorterDes,A.distinctVals=h.distinctVals,A.roundUp=h.roundUp,A.sort=h.sort,A.findIndexOfMin=h.findIndexOfMin;var d=n(1437);A.aggNums=d.aggNums,A.len=d.len,A.mean=d.mean,A.median=d.median,A.midRange=d.midRange,A.variance=d.variance,A.stdev=d.stdev,A.interp=d.interp;var f=n(1438);A.init2dArray=f.init2dArray,A.transposeRagged=f.transposeRagged,A.dot=f.dot,A.translationMatrix=f.translationMatrix,A.rotationMatrix=f.rotationMatrix,A.rotationXYMatrix=f.rotationXYMatrix,A.apply2DTransform=f.apply2DTransform,A.apply2DTransform2=f.apply2DTransform2;var p=n(1439);A.deg2rad=p.deg2rad,A.rad2deg=p.rad2deg,A.angleDelta=p.angleDelta,A.angleDist=p.angleDist,A.isFullCircle=p.isFullCircle,A.isAngleInsideSector=p.isAngleInsideSector,A.isPtInsideSector=p.isPtInsideSector,A.pathArc=p.pathArc,A.pathSector=p.pathSector,A.pathAnnulus=p.pathAnnulus;var I=n(1440);A.isLeftAnchor=I.isLeftAnchor,A.isCenterAnchor=I.isCenterAnchor,A.isRightAnchor=I.isRightAnchor,A.isTopAnchor=I.isTopAnchor,A.isMiddleAnchor=I.isMiddleAnchor,A.isBottomAnchor=I.isBottomAnchor;var C=n(1441);A.segmentsIntersect=C.segmentsIntersect,A.segmentDistance=C.segmentDistance,A.getTextLocation=C.getTextLocation,A.clearLocationCache=C.clearLocationCache,A.getVisibleSegment=C.getVisibleSegment,A.findPointOnPath=C.findPointOnPath;var m=n(241);A.extendFlat=m.extendFlat,A.extendDeep=m.extendDeep,A.extendDeepAll=m.extendDeepAll,A.extendDeepNoArrays=m.extendDeepNoArrays;var y=n(188);A.log=y.log,A.warn=y.warn,A.error=y.error;var E=n(706);A.counterRegex=E.counter;var v=n(1442);A.throttle=v.throttle,A.throttleDone=v.done,A.clearThrottle=v.clear;var B=n(710);function b(t){var e={};for(var n in t)for(var r=t[n],i=0;i<r.length;i++)e[r[i]]=+n;return e}A.getGraphDiv=B.getGraphDiv,A.isPlotDiv=B.isPlotDiv,A.removeElement=B.removeElement,A.addStyleRule=B.addStyleRule,A.addRelatedStyleRule=B.addRelatedStyleRule,A.deleteRelatedStyleRule=B.deleteRelatedStyleRule,A.clearResponsive=n(1443),A.makeTraceGroups=n(1444),A._=n(1445),A.notifier=n(1446),A.filterUnique=n(1447),A.filterVisible=n(1448),A.pushUnique=n(709),A.cleanNumber=n(1449),A.ensureNumber=function(t){return i(t)?(t=Number(t))<-a||t>a?s:i(t)?Number(t):s:s},A.isIndex=function(t,e){return!(void 0!==e&&t>=e)&&(i(t)&&t>=0&&t%1==0)},A.noop=n(708),A.identity=n(711),A.repeat=function(t,e){for(var n=new Array(e),r=0;r<e;r++)n[r]=t;return n},A.swapAttrs=function(t,e,n,r){n||(n="x"),r||(r="y");for(var i=0;i<e.length;i++){var o=e[i],a=A.nestedProperty(t,o.replace("?",n)),s=A.nestedProperty(t,o.replace("?",r)),l=a.get();a.set(s.get()),s.set(l)}},A.raiseToTop=function(t){t.parentNode.appendChild(t)},A.cancelTransition=function(t){return t.transition().duration(0)},A.constrain=function(t,e,n){return e>n?Math.max(n,Math.min(e,t)):Math.max(e,Math.min(n,t))},A.bBoxIntersect=function(t,e,n){return n=n||0,t.left<=e.right+n&&e.left<=t.right+n&&t.top<=e.bottom+n&&e.top<=t.bottom+n},A.simpleMap=function(t,e,n,r){for(var i=t.length,o=new Array(i),a=0;a<i;a++)o[a]=e(t[a],n,r);return o},A.randstr=function t(e,n,r,i){if(r||(r=16),void 0===n&&(n=24),n<=0)return"0";var o,a,s=Math.log(Math.pow(2,n))/Math.log(r),l="";for(o=2;s===1/0;o*=2)s=Math.log(Math.pow(2,n/o))/Math.log(r)*o;var c=s-Math.floor(s);for(o=0;o<Math.floor(s);o++)l=Math.floor(Math.random()*r).toString(r)+l;c&&(a=Math.pow(r,c),l=Math.floor(Math.random()*a).toString(r)+l);var u=parseInt(l,r);return e&&e[l]||u!==1/0&&u>=Math.pow(2,n)?i>10?(A.warn("randstr failed uniqueness"),l):t(e,n,r,(i||0)+1):l},A.OptionControl=function(t,e){t||(t={}),e||(e="opt");var n={optionList:[],_newoption:function(r){r[e]=t,n[r.name]=r,n.optionList.push(r)}};return n["_"+e]=t,n},A.smooth=function(t,e){if((e=Math.round(e)||0)<2)return t;var n,r,i,o,a=t.length,s=2*a,A=2*e-1,l=new Array(A),c=new Array(a);for(n=0;n<A;n++)l[n]=(1-Math.cos(Math.PI*(n+1)/e))/(2*e);for(n=0;n<a;n++){for(o=0,r=0;r<A;r++)(i=n+r+1-e)<-a?i-=s*Math.round(i/s):i>=s&&(i-=s*Math.floor(i/s)),i<0?i=-1-i:i>=a&&(i=s-1-i),o+=t[i]*l[r];c[n]=o}return c},A.syncOrAsync=function(t,e,n){var r;function i(){return A.syncOrAsync(t,e,n)}for(;t.length;)if((r=(0,t.splice(0,1)[0])(e))&&r.then)return r.then(i).then(void 0,A.promiseError);return n&&n(e)},A.stripTrailingSlash=function(t){return"/"===t.substr(-1)?t.substr(0,t.length-1):t},A.noneOrAll=function(t,e,n){if(t){var r,i=!1,o=!0;for(r=0;r<n.length;r++)null!=t[n[r]]?i=!0:o=!1;if(i&&!o)for(r=0;r<n.length;r++)t[n[r]]=e[n[r]]}},A.mergeArray=function(t,e,n,r){var i="function"==typeof r;if(A.isArrayOrTypedArray(t))for(var o=Math.min(t.length,e.length),a=0;a<o;a++){var s=t[a];e[a][n]=i?r(s):s}},A.mergeArrayCastPositive=function(t,e,n){return A.mergeArray(t,e,n,function(t){var e=+t;return isFinite(e)&&e>0?e:0})},A.fillArray=function(t,e,n,r){if(r=r||A.identity,A.isArrayOrTypedArray(t))for(var i=0;i<e.length;i++)e[i][n]=r(t[i])},A.castOption=function(t,e,n,r){r=r||A.identity;var i=A.nestedProperty(t,n).get();return A.isArrayOrTypedArray(i)?Array.isArray(e)&&A.isArrayOrTypedArray(i[e[0]])?r(i[e[0]][e[1]]):r(i[e]):i},A.extractOption=function(t,e,n,r){if(n in t)return t[n];var i=A.nestedProperty(e,r).get();return Array.isArray(i)?void 0:i},A.tagSelected=function(t,e,n){var r,i,o=e.selectedpoints,a=e._indexToPoints;a&&(r=b(a));for(var s=0;s<o.length;s++){var l=o[s];if(A.isIndex(l)){var c=r?r[l]:l,u=n?n[c]:c;void 0!==(i=u)&&i<t.length&&(t[u].selected=1)}}},A.selIndices2selPoints=function(t){var e=t.selectedpoints,n=t._indexToPoints;if(n){for(var r=b(n),i=[],o=0;o<e.length;o++){var a=e[o];if(A.isIndex(a)){var s=r[a];A.isIndex(s)&&i.push(s)}}return i}return e},A.getTargetArray=function(t,e){var n=e.target;if("string"==typeof n&&n){var r=A.nestedProperty(t,n).get();return!!Array.isArray(r)&&r}return!!Array.isArray(n)&&n},A.minExtend=function(t,e){var n={};"object"!=typeof e&&(e={});var r,i,o,a=Object.keys(t);for(r=0;r<a.length;r++)o=t[i=a[r]],"_"!==i.charAt(0)&&"function"!=typeof o&&("module"===i?n[i]=o:Array.isArray(o)?n[i]="colorscale"===i?o.slice():o.slice(0,3):n[i]=o&&"object"==typeof o?A.minExtend(t[i],e[i]):o);for(a=Object.keys(e),r=0;r<a.length;r++)"object"==typeof(o=e[i=a[r]])&&i in n&&"object"==typeof n[i]||(n[i]=o);return n},A.titleCase=function(t){return t.charAt(0).toUpperCase()+t.substr(1)},A.containsAny=function(t,e){for(var n=0;n<e.length;n++)if(-1!==t.indexOf(e[n]))return!0;return!1},A.isIE=function(){return void 0!==window.navigator.msSaveBlob};var Q=/MSIE [1-9]\./;A.isIE9orBelow=function(){return A.isIE()&&Q.test(window.navigator.userAgent)};var w=/Version\/[\d\.]+.*Safari/;A.isSafari=function(){return w.test(window.navigator.userAgent)},A.isD3Selection=function(t){return t&&"function"==typeof t.classed},A.ensureSingle=function(t,e,n,r){var i=t.select(e+(n?"."+n:""));if(i.size())return i;var o=t.append(e);return n&&o.classed(n,!0),r&&o.call(r),o},A.ensureSingleById=function(t,e,n,r){var i=t.select(e+"#"+n);if(i.size())return i;var o=t.append(e).attr("id",n);return r&&o.call(r),o},A.objectFromPath=function(t,e){for(var n,r=t.split("."),i=n={},o=0;o<r.length;o++){var a=r[o],s=null,A=r[o].match(/(.*)\[([0-9]+)\]/);A?(a=A[1],s=A[2],n=n[a]=[],o===r.length-1?n[s]=e:n[s]={},n=n[s]):(o===r.length-1?n[a]=e:n[a]={},n=n[a])}return i};var x=/^([^\[\.]+)\.(.+)?/,_=/^([^\.]+)\[([0-9]+)\](\.)?(.+)?/;A.expandObjectPaths=function(t){var e,n,r,i,o,a,s;if("object"==typeof t&&!Array.isArray(t))for(n in t)t.hasOwnProperty(n)&&((e=n.match(x))?(i=t[n],r=e[1],delete t[n],t[r]=A.extendDeepNoArrays(t[r]||{},A.objectFromPath(n,A.expandObjectPaths(i))[r])):(e=n.match(_))?(i=t[n],r=e[1],o=parseInt(e[2]),delete t[n],t[r]=t[r]||[],"."===e[3]?(s=e[4],a=t[r][o]=t[r][o]||{},A.extendDeepNoArrays(a,A.objectFromPath(s,A.expandObjectPaths(i)))):t[r][o]=A.expandObjectPaths(i)):t[n]=A.expandObjectPaths(t[n]));return t},A.numSeparate=function(t,e,n){if(n||(n=!1),"string"!=typeof e||0===e.length)throw new Error("Separator string required for formatting!");"number"==typeof t&&(t=String(t));var r=/(\d+)(\d{3})/,i=e.charAt(0),o=e.charAt(1),a=t.split("."),s=a[0],A=a.length>1?i+a[1]:"";if(o&&(a.length>1||s.length>4||n))for(;r.test(s);)s=s.replace(r,"$1"+o+"$2");return s+A},A.TEMPLATE_STRING_REGEX=/%{([^\s%{}:]*)(:[^}]*)?}/g;var D=/^\w*$/;A.templateString=function(t,e){var n={};return t.replace(A.TEMPLATE_STRING_REGEX,function(t,r){return D.test(r)?e[r]||"":(n[r]=n[r]||A.nestedProperty(e,r).get,n[r]()||"")})};var M=/^:/,k=0;A.hovertemplateString=function(t,e,n){var i=arguments,o={};return t.replace(A.TEMPLATE_STRING_REGEX,function(t,a,s){var l,c,u;for(u=3;u<i.length;u++){if((l=i[u]).hasOwnProperty(a)){c=l[a];break}if(D.test(a)||(c=o[a]||A.nestedProperty(l,a).get())&&(o[a]=c),void 0!==c)break}(void 0===c&&(k<10&&(A.warn("Variable '"+a+"' in hovertemplate could not be found!"),c=t),10===k&&A.warn("Too many hovertemplate warnings - additional warnings will be suppressed"),k++),s)?c=(n?n.numberFormat:r.format)(s.replace(M,""))(c):e.hasOwnProperty(a+"Label")&&(c=e[a+"Label"]);return c})};A.subplotSort=function(t,e){for(var n=Math.min(t.length,e.length)+1,r=0,i=0,o=0;o<n;o++){var a=t.charCodeAt(o)||0,s=e.charCodeAt(o)||0,A=a>=48&&a<=57,l=s>=48&&s<=57;if(A&&(r=10*r+a-48),l&&(i=10*i+s-48),!A||!l){if(r!==i)return r-i;if(a!==s)return a-s}}return i-r};var S=2e9;A.seedPseudoRandom=function(){S=2e9},A.pseudoRandom=function(){var t=S;return S=(69069*S+1)%4294967296,Math.abs(S-t)<429496729?A.pseudoRandom():S/4294967296},A.fillText=function(t,e,n){var r=Array.isArray(n)?function(t){n.push(t)}:function(t){n.text=t},i=A.extractOption(t,e,"htx","hovertext");if(A.isValidTextValue(i))return r(i);var o=A.extractOption(t,e,"tx","text");return A.isValidTextValue(o)?r(o):void 0},A.isValidTextValue=function(t){return t||0===t},A.formatPercent=function(t,e){e=e||0;for(var n=(Math.round(100*t*Math.pow(10,e))*Math.pow(.1,e)).toFixed(e)+"%",r=0;r<e;r++)-1!==n.indexOf(".")&&(n=(n=n.replace("0%","%")).replace(".%","%"));return n},A.isHidden=function(t){var e=window.getComputedStyle(t).display;return!e||"none"===e}},function(t,e,n){"use strict";t.exports=function(t){for(var e,n=t.length,r=0;r<n;r++)if(((e=t.charCodeAt(r))<9||e>13)&&32!==e&&133!==e&&160!==e&&5760!==e&&6158!==e&&(e<8192||e>8205)&&8232!==e&&8233!==e&&8239!==e&&8287!==e&&8288!==e&&12288!==e&&65279!==e)return!1;return!0}},function(t,e,n){"use strict";var r=n(347),i=/^\w*$/;t.exports=function(t,e,n,o){var a,s,A;n=n||"name",o=o||"value";var l={};e&&e.length?(A=r(t,e),s=A.get()):s=t,e=e||"";var c={};if(s)for(a=0;a<s.length;a++)c[s[a][n]]=a;var u=i.test(o),g={set:function(t,e){var i=null===e?4:0;if(!s){if(!A||4===i)return;s=[],A.set(s)}var a=c[t];if(void 0===a){if(4===i)return;i|=3,a=s.length,c[t]=a}else e!==(u?s[a][o]:r(s[a],o).get())&&(i|=2);var h=s[a]=s[a]||{};return h[n]=t,u?h[o]=e:r(h,o).set(e),null!==e&&(i&=-5),l[a]=l[a]|i,g},get:function(t){if(s){var e=c[t];return void 0===e?void 0:u?s[e][o]:r(s[e],o).get()}},rename:function(t,e){var r=c[t];return void 0===r?g:(l[r]=1|l[r],c[e]=r,delete c[t],s[r][n]=e,g)},remove:function(t){var e=c[t];if(void 0===e)return g;var i=s[e];if(Object.keys(i).length>2)return l[e]=2|l[e],g.set(t,null);if(u){for(a=e;a<s.length;a++)l[a]=3|l[a];for(a=e;a<s.length;a++)c[s[a][n]]--;s.splice(e,1),delete c[t]}else r(i,o).set(null),l[e]=6|l[e];return g},constructUpdate:function(){for(var t,i,a={},A=Object.keys(l),c=0;c<A.length;c++)i=A[c],t=e+"["+i+"]",s[i]?(1&l[i]&&(a[t+"."+n]=s[i][n]),2&l[i]&&(a[t+"."+o]=u?4&l[i]?null:s[i][o]:4&l[i]?null:r(s[i],o).get())):a[t]=null;return a}};return g}},function(t,e,n){"use strict";var r=/^(.*)(\.[^\.\[\]]+|\[\d\])$/,i=/^[^\.\[\]]+$/;t.exports=function(t,e){for(;e;){var n=t.match(r);if(n)t=n[1];else{if(!t.match(i))throw new Error("bad relativeAttr call:"+[t,e]);t=""}if("^"!==e.charAt(0))break;e=e.slice(1)}return t&&"["!==e.charAt(0)?t+"."+e:t+e}},function(t,e,n){"use strict";var r=n(100);t.exports=function(t,e){if(t>0)return Math.log(t)/Math.LN10;var n=Math.log(Math.min(e[0],e[1]))/Math.LN10;return r(n)||(n=Math.log(Math.max(e[0],e[1]))/Math.LN10-6),n}},function(t,e,n){"use strict";var r=n(186).isArrayOrTypedArray,i=n(240);t.exports=function t(e,n){for(var o in n){var a=n[o],s=e[o];if(s!==a)if("_"===o.charAt(0)||"function"==typeof a){if(o in e)continue;e[o]=a}else if(r(a)&&r(s)&&i(a[0])){if("customdata"===o||"ids"===o)continue;for(var A=Math.min(a.length,s.length),l=0;l<A;l++)s[l]!==a[l]&&i(a[l])&&i(s[l])&&t(s[l],a[l])}else i(a)&&i(s)&&(t(s,a),Object.keys(s).length||delete e[o])}}},function(t,e,n){"use strict";var r=n(100),i=n(704),o=n(705),a=n(1428),s=n(1429).DESELECTDIM,A=n(347),l=n(706).counter,c=n(187).modHalf,u=n(186).isArrayOrTypedArray;function g(t,n){var r=e.valObjectMeta[n.valType];if(n.arrayOk&&u(t))return!0;if(r.validateFunction)return r.validateFunction(t,n);var i={},o=i,a={set:function(t){o=t}};return r.coerceFunction(t,a,i,n),o!==i}e.valObjectMeta={data_array:{description:["An {array} of data.","The value MUST be an {array}, or we ignore it.","Note that typed arrays (e.g. Float32Array) are supported."].join(" "),requiredOpts:[],otherOpts:["dflt"],coerceFunction:function(t,e,n){u(t)?e.set(t):void 0!==n&&e.set(n)}},enumerated:{description:["Enumerated value type. The available values are listed","in `values`."].join(" "),requiredOpts:["values"],otherOpts:["dflt","coerceNumber","arrayOk"],coerceFunction:function(t,e,n,r){r.coerceNumber&&(t=+t),-1===r.values.indexOf(t)?e.set(n):e.set(t)},validateFunction:function(t,e){e.coerceNumber&&(t=+t);for(var n=e.values,r=0;r<n.length;r++){var i=String(n[r]);if("/"===i.charAt(0)&&"/"===i.charAt(i.length-1)){if(new RegExp(i.substr(1,i.length-2)).test(t))return!0}else if(t===n[r])return!0}return!1}},boolean:{description:"A boolean (true/false) value.",requiredOpts:[],otherOpts:["dflt"],coerceFunction:function(t,e,n){!0===t||!1===t?e.set(t):e.set(n)}},number:{description:["A number or a numeric value","(e.g. a number inside a string).","When applicable, values greater (less) than `max` (`min`)","are coerced to the `dflt`."].join(" "),requiredOpts:[],otherOpts:["dflt","min","max","arrayOk"],coerceFunction:function(t,e,n,i){!r(t)||void 0!==i.min&&t<i.min||void 0!==i.max&&t>i.max?e.set(n):e.set(+t)}},integer:{description:["An integer or an integer inside a string.","When applicable, values greater (less) than `max` (`min`)","are coerced to the `dflt`."].join(" "),requiredOpts:[],otherOpts:["dflt","min","max","arrayOk"],coerceFunction:function(t,e,n,i){t%1||!r(t)||void 0!==i.min&&t<i.min||void 0!==i.max&&t>i.max?e.set(n):e.set(+t)}},string:{description:["A string value.","Numbers are converted to strings except for attributes with","`strict` set to true."].join(" "),requiredOpts:[],otherOpts:["dflt","noBlank","strict","arrayOk","values"],coerceFunction:function(t,e,n,r){if("string"!=typeof t){var i="number"==typeof t;!0!==r.strict&&i?e.set(String(t)):e.set(n)}else r.noBlank&&!t?e.set(n):e.set(t)}},color:{description:["A string describing color.","Supported formats:","- hex (e.g. '#d3d3d3')","- rgb (e.g. 'rgb(255, 0, 0)')","- rgba (e.g. 'rgb(255, 0, 0, 0.5)')","- hsl (e.g. 'hsl(0, 100%, 50%)')","- hsv (e.g. 'hsv(0, 100%, 100%)')","- named colors (full list: http://www.w3.org/TR/css3-color/#svg-color)"].join(" "),requiredOpts:[],otherOpts:["dflt","arrayOk"],coerceFunction:function(t,e,n){i(t).isValid()?e.set(t):e.set(n)}},colorlist:{description:["A list of colors.","Must be an {array} containing valid colors."].join(" "),requiredOpts:[],otherOpts:["dflt"],coerceFunction:function(t,e,n){Array.isArray(t)&&t.length&&t.every(function(t){return i(t).isValid()})?e.set(t):e.set(n)}},colorscale:{description:["A Plotly colorscale either picked by a name:","(any of",Object.keys(a.scales).join(", "),")","customized as an {array} of 2-element {arrays} where","the first element is the normalized color level value","(starting at *0* and ending at *1*),","and the second item is a valid color string."].join(" "),requiredOpts:[],otherOpts:["dflt"],coerceFunction:function(t,e,n){e.set(a.get(t,n))}},angle:{description:["A number (in degree) between -180 and 180."].join(" "),requiredOpts:[],otherOpts:["dflt"],coerceFunction:function(t,e,n){"auto"===t?e.set("auto"):r(t)?e.set(c(+t,360)):e.set(n)}},subplotid:{description:["An id string of a subplot type (given by dflt), optionally","followed by an integer >1. e.g. if dflt='geo', we can have","'geo', 'geo2', 'geo3', ..."].join(" "),requiredOpts:["dflt"],otherOpts:["regex"],coerceFunction:function(t,e,n,r){var i=r.regex||l(n);"string"==typeof t&&i.test(t)?e.set(t):e.set(n)},validateFunction:function(t,e){var n=e.dflt;return t===n||"string"==typeof t&&!!l(n).test(t)}},flaglist:{description:["A string representing a combination of flags","(order does not matter here).","Combine any of the available `flags` with *+*.","(e.g. ('lines+markers')).","Values in `extras` cannot be combined."].join(" "),requiredOpts:["flags"],otherOpts:["dflt","extras","arrayOk"],coerceFunction:function(t,e,n,r){if("string"==typeof t)if(-1===(r.extras||[]).indexOf(t)){for(var i=t.split("+"),o=0;o<i.length;){var a=i[o];-1===r.flags.indexOf(a)||i.indexOf(a)<o?i.splice(o,1):o++}i.length?e.set(i.join("+")):e.set(n)}else e.set(t);else e.set(n)}},any:{description:"Any type.",requiredOpts:[],otherOpts:["dflt","values","arrayOk"],coerceFunction:function(t,e,n){void 0===t?e.set(n):e.set(t)}},info_array:{description:["An {array} of plot information."].join(" "),requiredOpts:["items"],otherOpts:["dflt","freeLength","dimensions"],coerceFunction:function(t,n,r,i){function o(t,n,r){var i,o={set:function(t){i=t}};return void 0===r&&(r=n.dflt),e.valObjectMeta[n.valType].coerceFunction(t,o,r,n),i}var a=2===i.dimensions||"1-2"===i.dimensions&&Array.isArray(t)&&Array.isArray(t[0]);if(Array.isArray(t)){var s,A,l,c,u,g,h=i.items,d=[],f=Array.isArray(h),p=f&&a&&Array.isArray(h[0]),I=a&&f&&!p,C=f&&!I?h.length:t.length;if(r=Array.isArray(r)?r:[],a)for(s=0;s<C;s++)for(d[s]=[],l=Array.isArray(t[s])?t[s]:[],u=I?h.length:f?h[s].length:l.length,A=0;A<u;A++)c=I?h[A]:f?h[s][A]:h,void 0!==(g=o(l[A],c,(r[s]||[])[A]))&&(d[s][A]=g);else for(s=0;s<C;s++)void 0!==(g=o(t[s],f?h[s]:h,r[s]))&&(d[s]=g);n.set(d)}else n.set(r)},validateFunction:function(t,e){if(!Array.isArray(t))return!1;var n=e.items,r=Array.isArray(n),i=2===e.dimensions;if(!e.freeLength&&t.length!==n.length)return!1;for(var o=0;o<t.length;o++)if(i){if(!Array.isArray(t[o])||!e.freeLength&&t[o].length!==n[o].length)return!1;for(var a=0;a<t[o].length;a++)if(!g(t[o][a],r?n[o][a]:n))return!1}else if(!g(t[o],r?n[o]:n))return!1;return!0}}},e.coerce=function(t,n,r,i,o){var a=A(r,i).get(),s=A(t,i),l=A(n,i),c=s.get(),h=n._template;if(void 0===c&&h&&(c=A(h,i).get(),h=0),void 0===o&&(o=a.dflt),a.arrayOk&&u(c))return l.set(c),c;var d=e.valObjectMeta[a.valType].coerceFunction;d(c,l,o,a);var f=l.get();return h&&f===o&&!g(c,a)&&(d(c=A(h,i).get(),l,o,a),f=l.get()),f},e.coerce2=function(t,n,r,i,o){var a=A(t,i),s=e.coerce(t,n,r,i,o),l=a.get();return null!=l&&s},e.coerceFont=function(t,e,n){var r={};return n=n||{},r.family=t(e+".family",n.family),r.size=t(e+".size",n.size),r.color=t(e+".color",n.color),r},e.coerceHoverinfo=function(t,n,r){var i,a=n._module.attributes,s=a.hoverinfo?a:o,A=s.hoverinfo;if(1===r._dataLength){var l="all"===A.dflt?A.flags.slice():A.dflt.split("+");l.splice(l.indexOf("name"),1),i=l.join("+")}return e.coerce(t,n,s,"hoverinfo",i)},e.coerceSelectionMarkerOpacity=function(t,e){if(t.marker){var n,r,i=t.marker.opacity;if(void 0!==i)u(i)||t.selected||t.unselected||(n=i,r=s*i),e("selected.marker.opacity",n),e("unselected.marker.opacity",r)}},e.validate=g},function(t,e,n){"use strict";var r=n(348),i=n(1426).hoverlabel,o=n(241).extendFlat;t.exports={hoverlabel:{bgcolor:o({},i.bgcolor,{arrayOk:!0,description:"Sets the background color of the hover labels for this trace"}),bordercolor:o({},i.bordercolor,{arrayOk:!0,description:"Sets the border color of the hover labels for this trace."}),font:r({arrayOk:!0,editType:"none",description:"Sets the font used in hover labels."}),align:o({},i.align,{arrayOk:!0}),namelength:o({},i.namelength,{arrayOk:!0}),editType:"none"}}},function(t,e,n){"use strict";var r=n(1427),i=n(348)({editType:"none",description:"Sets the default hover label font used by all traces on the graph."});i.family.dflt=r.HOVERFONT,i.size.dflt=r.HOVERFONTSIZE,t.exports={clickmode:{valType:"flaglist",role:"info",flags:["event","select"],dflt:"event",editType:"plot",extras:["none"],description:["Determines the mode of single click interactions.","*event* is the default value and emits the `plotly_click`","event. In addition this mode emits the `plotly_selected` event","in drag modes *lasso* and *select*, but with no event data attached","(kept for compatibility reasons).","The *select* flag enables selecting single","data points via click. This mode also supports persistent selections,","meaning that pressing Shift while clicking, adds to / subtracts from an","existing selection. *select* with `hovermode`: *x* can be confusing, consider","explicitly setting `hovermode`: *closest* when using this feature.","Selection events are sent accordingly as long as *event* flag is set as well.","When the *event* flag is missing, `plotly_click` and `plotly_selected`","events are not fired."].join(" ")},dragmode:{valType:"enumerated",role:"info",values:["zoom","pan","select","lasso","orbit","turntable",!1],dflt:"zoom",editType:"modebar",description:["Determines the mode of drag interactions.","*select* and *lasso* apply only to scatter traces with","markers or text. *orbit* and *turntable* apply only to","3D scenes."].join(" ")},hovermode:{valType:"enumerated",role:"info",values:["x","y","closest",!1],editType:"modebar",description:["Determines the mode of hover interactions.","If `clickmode` includes the *select* flag,","`hovermode` defaults to *closest*.","If `clickmode` lacks the *select* flag,","it defaults to *x* or *y* (depending on the trace's","`orientation` value) for plots based on","cartesian coordinates. For anything else the default","value is *closest*."].join(" ")},hoverdistance:{valType:"integer",min:-1,dflt:20,role:"info",editType:"none",description:["Sets the default distance (in pixels) to look for data","to add hover labels (-1 means no cutoff, 0 means no looking for data).","This is only a real distance for hovering on point-like objects,","like scatter points. For area-like objects (bars, scatter fills, etc)","hovering is on inside the area and off outside, but these objects","will not supersede hover on point-like objects in case of conflict."].join(" ")},spikedistance:{valType:"integer",min:-1,dflt:20,role:"info",editType:"none",description:["Sets the default distance (in pixels) to look for data to draw","spikelines to (-1 means no cutoff, 0 means no looking for data).","As with hoverdistance, distance does not apply to area-like objects.","In addition, some objects can be hovered on but will not generate","spikelines, such as scatter fills."].join(" ")},hoverlabel:{bgcolor:{valType:"color",role:"style",editType:"none",description:["Sets the background color of all hover labels on graph"].join(" ")},bordercolor:{valType:"color",role:"style",editType:"none",description:["Sets the border color of all hover labels on graph."].join(" ")},font:i,align:{valType:"enumerated",values:["left","right","auto"],dflt:"auto",role:"style",editType:"none",description:["Sets the horizontal alignment of the text content within hover label box.","Has an effect only if the hover label text spans more two or more lines"].join(" ")},namelength:{valType:"integer",min:-1,dflt:15,role:"style",editType:"none",description:["Sets the default length (in number of characters) of the trace name in","the hover labels for all traces. -1 shows the whole name","regardless of length. 0-3 shows the first 0-3 characters, and","an integer >3 will show the whole name if it is less than that","many characters, but if it is longer, will truncate to","`namelength - 3` characters and add an ellipsis."].join(" ")},editType:"none"},selectdirection:{valType:"enumerated",role:"info",values:["h","v","d","any"],dflt:"any",description:['When "dragmode" is set to "select", this limits the selection of the drag to','horizontal, vertical or diagonal. "h" only allows horizontal selection,','"v" only vertical, "d" only diagonal and "any" sets no limit.'].join(" "),editType:"none"}}},function(t,e,n){"use strict";t.exports={YANGLE:60,HOVERARROWSIZE:6,HOVERTEXTPAD:3,HOVERFONTSIZE:13,HOVERFONT:"Arial, sans-serif",HOVERMINTIME:50,HOVERID:"-hover"}},function(t,e,n){"use strict";var r=n(704),i={Greys:[[0,"rgb(0,0,0)"],[1,"rgb(255,255,255)"]],YlGnBu:[[0,"rgb(8,29,88)"],[.125,"rgb(37,52,148)"],[.25,"rgb(34,94,168)"],[.375,"rgb(29,145,192)"],[.5,"rgb(65,182,196)"],[.625,"rgb(127,205,187)"],[.75,"rgb(199,233,180)"],[.875,"rgb(237,248,217)"],[1,"rgb(255,255,217)"]],Greens:[[0,"rgb(0,68,27)"],[.125,"rgb(0,109,44)"],[.25,"rgb(35,139,69)"],[.375,"rgb(65,171,93)"],[.5,"rgb(116,196,118)"],[.625,"rgb(161,217,155)"],[.75,"rgb(199,233,192)"],[.875,"rgb(229,245,224)"],[1,"rgb(247,252,245)"]],YlOrRd:[[0,"rgb(128,0,38)"],[.125,"rgb(189,0,38)"],[.25,"rgb(227,26,28)"],[.375,"rgb(252,78,42)"],[.5,"rgb(253,141,60)"],[.625,"rgb(254,178,76)"],[.75,"rgb(254,217,118)"],[.875,"rgb(255,237,160)"],[1,"rgb(255,255,204)"]],Bluered:[[0,"rgb(0,0,255)"],[1,"rgb(255,0,0)"]],RdBu:[[0,"rgb(5,10,172)"],[.35,"rgb(106,137,247)"],[.5,"rgb(190,190,190)"],[.6,"rgb(220,170,132)"],[.7,"rgb(230,145,90)"],[1,"rgb(178,10,28)"]],Reds:[[0,"rgb(220,220,220)"],[.2,"rgb(245,195,157)"],[.4,"rgb(245,160,105)"],[1,"rgb(178,10,28)"]],Blues:[[0,"rgb(5,10,172)"],[.35,"rgb(40,60,190)"],[.5,"rgb(70,100,245)"],[.6,"rgb(90,120,245)"],[.7,"rgb(106,137,247)"],[1,"rgb(220,220,220)"]],Picnic:[[0,"rgb(0,0,255)"],[.1,"rgb(51,153,255)"],[.2,"rgb(102,204,255)"],[.3,"rgb(153,204,255)"],[.4,"rgb(204,204,255)"],[.5,"rgb(255,255,255)"],[.6,"rgb(255,204,255)"],[.7,"rgb(255,153,255)"],[.8,"rgb(255,102,204)"],[.9,"rgb(255,102,102)"],[1,"rgb(255,0,0)"]],Rainbow:[[0,"rgb(150,0,90)"],[.125,"rgb(0,0,200)"],[.25,"rgb(0,25,255)"],[.375,"rgb(0,152,255)"],[.5,"rgb(44,255,150)"],[.625,"rgb(151,255,0)"],[.75,"rgb(255,234,0)"],[.875,"rgb(255,111,0)"],[1,"rgb(255,0,0)"]],Portland:[[0,"rgb(12,51,131)"],[.25,"rgb(10,136,186)"],[.5,"rgb(242,211,56)"],[.75,"rgb(242,143,56)"],[1,"rgb(217,30,30)"]],Jet:[[0,"rgb(0,0,131)"],[.125,"rgb(0,60,170)"],[.375,"rgb(5,255,255)"],[.625,"rgb(255,255,0)"],[.875,"rgb(250,0,0)"],[1,"rgb(128,0,0)"]],Hot:[[0,"rgb(0,0,0)"],[.3,"rgb(230,0,0)"],[.6,"rgb(255,210,0)"],[1,"rgb(255,255,255)"]],Blackbody:[[0,"rgb(0,0,0)"],[.2,"rgb(230,0,0)"],[.4,"rgb(230,210,0)"],[.7,"rgb(255,255,255)"],[1,"rgb(160,200,255)"]],Earth:[[0,"rgb(0,0,130)"],[.1,"rgb(0,180,180)"],[.2,"rgb(40,210,40)"],[.4,"rgb(230,230,50)"],[.6,"rgb(120,70,20)"],[1,"rgb(255,255,255)"]],Electric:[[0,"rgb(0,0,0)"],[.15,"rgb(30,0,100)"],[.4,"rgb(120,0,100)"],[.6,"rgb(160,90,0)"],[.8,"rgb(230,200,0)"],[1,"rgb(255,250,220)"]],Viridis:[[0,"#440154"],[.06274509803921569,"#48186a"],[.12549019607843137,"#472d7b"],[.18823529411764706,"#424086"],[.25098039215686274,"#3b528b"],[.3137254901960784,"#33638d"],[.3764705882352941,"#2c728e"],[.4392156862745098,"#26828e"],[.5019607843137255,"#21918c"],[.5647058823529412,"#1fa088"],[.6274509803921569,"#28ae80"],[.6901960784313725,"#3fbc73"],[.7529411764705882,"#5ec962"],[.8156862745098039,"#84d44b"],[.8784313725490196,"#addc30"],[.9411764705882353,"#d8e219"],[1,"#fde725"]],Cividis:[[0,"rgb(0,32,76)"],[.058824,"rgb(0,42,102)"],[.117647,"rgb(0,52,110)"],[.176471,"rgb(39,63,108)"],[.235294,"rgb(60,74,107)"],[.294118,"rgb(76,85,107)"],[.352941,"rgb(91,95,109)"],[.411765,"rgb(104,106,112)"],[.470588,"rgb(117,117,117)"],[.529412,"rgb(131,129,120)"],[.588235,"rgb(146,140,120)"],[.647059,"rgb(161,152,118)"],[.705882,"rgb(176,165,114)"],[.764706,"rgb(192,177,109)"],[.823529,"rgb(209,191,102)"],[.882353,"rgb(225,204,92)"],[.941176,"rgb(243,219,79)"],[1,"rgb(255,233,69)"]]},o=i.RdBu;function a(t){var e=0;if(!Array.isArray(t)||t.length<2)return!1;if(!t[0]||!t[t.length-1])return!1;if(0!=+t[0][0]||1!=+t[t.length-1][0])return!1;for(var n=0;n<t.length;n++){var i=t[n];if(2!==i.length||+i[0]<e||!r(i[1]).isValid())return!1;e=+i[0]}return!0}t.exports={scales:i,defaultScale:o,get:function(t,e){if(e||(e=o),!t)return e;function n(){try{t=i[t]||JSON.parse(t)}catch(n){t=e}}return"string"==typeof t&&(n(),"string"==typeof t&&n()),a(t)?t:e},isValid:function(t){return void 0!==i[t]||a(t)}}},function(t,e,n){"use strict";t.exports={SHOW_PLACEHOLDER:100,HIDE_PLACEHOLDER:1e3,DESELECTDIM:.2}},function(t,e,n){"use strict";var r,i,o=n(185),a=n(100),s=n(188),A=n(187).mod,l=n(346),c=l.BADNUM,u=l.ONEDAY,g=l.ONEHOUR,h=l.ONEMIN,d=l.ONESEC,f=l.EPOCHJD,p=n(707),I=o.time.format.utc,C=/^\s*(-?\d\d\d\d|\d\d)(-(\d?\d)(-(\d?\d)([ Tt]([01]?\d|2[0-3])(:([0-5]\d)(:([0-5]\d(\.\d+)?))?(Z|z|[+\-]\d\d:?\d\d)?)?)?)?)?\s*$/m,m=/^\s*(-?\d\d\d\d|\d\d)(-(\d?\di?)(-(\d?\d)([ Tt]([01]?\d|2[0-3])(:([0-5]\d)(:([0-5]\d(\.\d+)?))?(Z|z|[+\-]\d\d:?\d\d)?)?)?)?)?\s*$/m,y=(new Date).getFullYear()-70;function E(t){return t&&p.componentsRegistry.calendars&&"string"==typeof t&&"gregorian"!==t}function v(t,e){return String(t+Math.pow(10,e)).substr(1)}e.dateTick0=function(t,e){return E(t)?e?p.getComponentMethod("calendars","CANONICAL_SUNDAY")[t]:p.getComponentMethod("calendars","CANONICAL_TICK")[t]:e?"2000-01-02":"2000-01-01"},e.dfltRange=function(t){return E(t)?p.getComponentMethod("calendars","DFLTRANGE")[t]:["2000-01-01","2001-01-01"]},e.isJSDate=function(t){return"object"==typeof t&&null!==t&&"function"==typeof t.getTime},e.dateTime2ms=function(t,n){if(e.isJSDate(t)){var o=t.getTimezoneOffset()*h,a=(t.getUTCMinutes()-t.getMinutes())*h+(t.getUTCSeconds()-t.getSeconds())*d+(t.getUTCMilliseconds()-t.getMilliseconds());if(a){var s=3*h;o=o-s/2+A(a-o+s/2,s)}return(t=Number(t)-o)>=r&&t<=i?t:c}if("string"!=typeof t&&"number"!=typeof t)return c;t=String(t);var l=E(n),I=t.charAt(0);!l||"G"!==I&&"g"!==I||(t=t.substr(1),n="");var v=l&&"chinese"===n.substr(0,7),B=t.match(v?m:C);if(!B)return c;var b=B[1],Q=B[3]||"1",w=Number(B[5]||1),x=Number(B[7]||0),_=Number(B[9]||0),D=Number(B[11]||0);if(l){if(2===b.length)return c;var M;b=Number(b);try{var k=p.getComponentMethod("calendars","getCal")(n);if(v){var S="i"===Q.charAt(Q.length-1);Q=parseInt(Q,10),M=k.newDate(b,k.toMonthIndex(b,Q,S),w)}else M=k.newDate(b,Number(Q),w)}catch(t){return c}return M?(M.toJD()-f)*u+x*g+_*h+D*d:c}b=2===b.length?(Number(b)+2e3-y)%100+y:Number(b),Q-=1;var L=new Date(Date.UTC(2e3,Q,w,x,_));return L.setUTCFullYear(b),L.getUTCMonth()!==Q?c:L.getUTCDate()!==w?c:L.getTime()+D*d},r=e.MIN_MS=e.dateTime2ms("-9999"),i=e.MAX_MS=e.dateTime2ms("9999-12-31 23:59:59.9999"),e.isDateTime=function(t,n){return e.dateTime2ms(t,n)!==c};var B=90*u,b=3*g,Q=5*h;function w(t,e,n,r,i){if((e||n||r||i)&&(t+=" "+v(e,2)+":"+v(n,2),(r||i)&&(t+=":"+v(r,2),i))){for(var o=4;i%10==0;)o-=1,i/=10;t+="."+v(i,o)}return t}e.ms2DateTime=function(t,e,n){if("number"!=typeof t||!(t>=r&&t<=i))return c;e||(e=0);var o,a,s,l,C,m,y=Math.floor(10*A(t+.05,1)),v=Math.round(t-y/10);if(E(n)){var x=Math.floor(v/u)+f,_=Math.floor(A(t,u));try{o=p.getComponentMethod("calendars","getCal")(n).fromJD(x).formatDate("yyyy-mm-dd")}catch(t){o=I("G%Y-%m-%d")(new Date(v))}if("-"===o.charAt(0))for(;o.length<11;)o="-0"+o.substr(1);else for(;o.length<10;)o="0"+o;a=e<B?Math.floor(_/g):0,s=e<B?Math.floor(_%g/h):0,l=e<b?Math.floor(_%h/d):0,C=e<Q?_%d*10+y:0}else m=new Date(v),o=I("%Y-%m-%d")(m),a=e<B?m.getUTCHours():0,s=e<B?m.getUTCMinutes():0,l=e<b?m.getUTCSeconds():0,C=e<Q?10*m.getUTCMilliseconds()+y:0;return w(o,a,s,l,C)},e.ms2DateTimeLocal=function(t){if(!(t>=r+u&&t<=i-u))return c;var e=Math.floor(10*A(t+.05,1)),n=new Date(Math.round(t-e/10));return w(o.time.format("%Y-%m-%d")(n),n.getHours(),n.getMinutes(),n.getSeconds(),10*n.getUTCMilliseconds()+e)},e.cleanDate=function(t,n,r){if(t===c)return n;if(e.isJSDate(t)||"number"==typeof t&&isFinite(t)){if(E(r))return s.error("JS Dates and milliseconds are incompatible with world calendars",t),n;if(!(t=e.ms2DateTimeLocal(+t))&&void 0!==n)return n}else if(!e.isDateTime(t,r))return s.error("unrecognized date",t),n;return t};var x=/%\d?f/g;function _(t,e,n,r){t=t.replace(x,function(t){var n=Math.min(+t.charAt(1)||6,6);return(e/1e3%1+2).toFixed(n).substr(2).replace(/0+$/,"")||"0"});var i=new Date(Math.floor(e+.05));if(E(r))try{t=p.getComponentMethod("calendars","worldCalFmt")(t,e,r)}catch(t){return"Invalid"}return n(t)(i)}var D=[59,59.9,59.99,59.999,59.9999];e.formatDate=function(t,e,n,r,i,o){if(i=E(i)&&i,!e)if("y"===n)e=o.year;else if("m"===n)e=o.month;else{if("d"!==n)return function(t,e){var n=A(t+.05,u),r=v(Math.floor(n/g),2)+":"+v(A(Math.floor(n/h),60),2);if("M"!==e){a(e)||(e=0);var i=(100+Math.min(A(t/d,60),D[e])).toFixed(e).substr(1);e>0&&(i=i.replace(/0+$/,"").replace(/[\.]$/,"")),r+=":"+i}return r}(t,n)+"\n"+_(o.dayMonthYear,t,r,i);e=o.dayMonth+"\n"+o.year}return _(e,t,r,i)};var M=3*u;e.incrementMonth=function(t,e,n){n=E(n)&&n;var r=A(t,u);if(t=Math.round(t-r),n)try{var i=Math.round(t/u)+f,o=p.getComponentMethod("calendars","getCal")(n),a=o.fromJD(i);return e%12?o.add(a,e,"m"):o.add(a,e/12,"y"),(a.toJD()-f)*u+r}catch(e){s.error("invalid ms "+t+" in calendar "+n)}var l=new Date(t+M);return l.setUTCMonth(l.getUTCMonth()+e)+r-M},e.findExactDates=function(t,e){for(var n,r,i=0,o=0,s=0,A=0,l=E(e)&&p.getComponentMethod("calendars","getCal")(e),c=0;c<t.length;c++)if(r=t[c],a(r)){if(!(r%u))if(l)try{1===(n=l.fromJD(r/u+f)).day()?1===n.month()?i++:o++:s++}catch(t){}else 1===(n=new Date(r)).getUTCDate()?0===n.getUTCMonth()?i++:o++:s++}else A++;s+=o+=i;var g=t.length-A;return{exactYears:i/g,exactMonths:o/g,exactDays:s/g}}},function(t,e,n){"use strict";var r={staticPlot:{valType:"boolean",dflt:!1,description:["Determines whether the graphs are interactive or not.","If *false*, no interactivity, for export or image generation."].join(" ")},plotlyServerURL:{valType:"string",dflt:"https://plot.ly",description:["Sets base URL for the 'Edit in Chart Studio' (aka sendDataToCloud) mode bar button","and the showLink/sendData on-graph link"].join(" ")},editable:{valType:"boolean",dflt:!1,description:["Determines whether the graph is editable or not.","Sets all pieces of `edits`","unless a separate `edits` config item overrides individual parts."].join(" ")},edits:{annotationPosition:{valType:"boolean",dflt:!1,description:["Determines if the main anchor of the annotation is editable.","The main anchor corresponds to the","text (if no arrow) or the arrow (which drags the whole thing leaving","the arrow length & direction unchanged)."].join(" ")},annotationTail:{valType:"boolean",dflt:!1,description:["Has only an effect for annotations with arrows.","Enables changing the length and direction of the arrow."].join(" ")},annotationText:{valType:"boolean",dflt:!1,description:"Enables editing annotation text."},axisTitleText:{valType:"boolean",dflt:!1,description:"Enables editing axis title text."},colorbarPosition:{valType:"boolean",dflt:!1,description:"Enables moving colorbars."},colorbarTitleText:{valType:"boolean",dflt:!1,description:"Enables editing colorbar title text."},legendPosition:{valType:"boolean",dflt:!1,description:"Enables moving the legend."},legendText:{valType:"boolean",dflt:!1,description:"Enables editing the trace name fields from the legend"},shapePosition:{valType:"boolean",dflt:!1,description:"Enables moving shapes."},titleText:{valType:"boolean",dflt:!1,description:"Enables editing the global layout title."}},autosizable:{valType:"boolean",dflt:!1,description:["Determines whether the graphs are plotted with respect to","layout.autosize:true and infer its container size."].join(" ")},responsive:{valType:"boolean",dflt:!1,description:["Determines whether to change the layout size when window is resized.","In v2, this option will be removed and will always be true."].join(" ")},fillFrame:{valType:"boolean",dflt:!1,description:["When `layout.autosize` is turned on, determines whether the graph","fills the container (the default) or the screen (if set to *true*)."].join(" ")},frameMargins:{valType:"number",dflt:0,min:0,max:.5,description:["When `layout.autosize` is turned on, set the frame margins","in fraction of the graph size."].join(" ")},scrollZoom:{valType:"flaglist",flags:["cartesian","gl3d","geo","mapbox"],extras:[!0,!1],dflt:"gl3d+geo+mapbox",description:["Determines whether mouse wheel or two-finger scroll zooms is enable.","Turned on by default for gl3d, geo and mapbox subplots","(as these subplot types do not have zoombox via pan),","but turned off by default for cartesian subplots.","Set `scrollZoom` to *false* to disable scrolling for all subplots."].join(" ")},doubleClick:{valType:"enumerated",values:[!1,"reset","autosize","reset+autosize"],dflt:"reset+autosize",description:["Sets the double click interaction mode.","Has an effect only in cartesian plots.","If *false*, double click is disable.","If *reset*, double click resets the axis ranges to their initial values.","If *autosize*, double click set the axis ranges to their autorange values.","If *reset+autosize*, the odd double clicks resets the axis ranges","to their initial values and even double clicks set the axis ranges","to their autorange values."].join(" ")},doubleClickDelay:{valType:"number",dflt:300,min:0,description:["Sets the delay for registering a double-click in ms.","This is the time interval (in ms) between first mousedown and","2nd mouseup to constitute a double-click.","This setting propagates to all on-subplot double clicks","(except for geo and mapbox) and on-legend double clicks."].join(" ")},showAxisDragHandles:{valType:"boolean",dflt:!0,description:["Set to *false* to omit cartesian axis pan/zoom drag handles."].join(" ")},showAxisRangeEntryBoxes:{valType:"boolean",dflt:!0,description:["Set to *false* to omit direct range entry at the pan/zoom drag points,","note that `showAxisDragHandles` must be enabled to have an effect."].join(" ")},showTips:{valType:"boolean",dflt:!0,description:["Determines whether or not tips are shown while interacting","with the resulting graphs."].join(" ")},showLink:{valType:"boolean",dflt:!1,description:["Determines whether a link to plot.ly is displayed","at the bottom right corner of resulting graphs.","Use with `sendData` and `linkText`."].join(" ")},linkText:{valType:"string",dflt:"Edit chart",noBlank:!0,description:["Sets the text appearing in the `showLink` link."].join(" ")},sendData:{valType:"boolean",dflt:!0,description:["If *showLink* is true, does it contain data","just link to a plot.ly file?"].join(" ")},showSources:{valType:"any",dflt:!1,description:["Adds a source-displaying function to show sources on","the resulting graphs."].join(" ")},displayModeBar:{valType:"enumerated",values:["hover",!0,!1],dflt:"hover",description:["Determines the mode bar display mode.","If *true*, the mode bar is always visible.","If *false*, the mode bar is always hidden.","If *hover*, the mode bar is visible while the mouse cursor","is on the graph container."].join(" ")},showSendToCloud:{valType:"boolean",dflt:!1,description:['Should we include a ModeBar button, labeled "Edit in Chart Studio",',"that sends this chart to plot.ly or another plotly server as specified","by `plotlyServerURL` for editing, export, etc? Prior to version 1.43.0","this button was included by default, now it is opt-in using this flag.","Note that this button can (depending on `plotlyServerURL`) send your data","to an external server. However that server does not persist your data",'until you arrive at the Chart Studio and explicitly click "Save".'].join(" ")},showEditInChartStudio:{valType:"boolean",dflt:!1,description:["Same as `showSendToCloud`, but use a pencil icon instead of a floppy-disk.","Note that if both `showSendToCloud` and `showEditInChartStudio` are turned,","only `showEditInChartStudio` will be honored."].join(" ")},modeBarButtonsToRemove:{valType:"any",dflt:[],description:["Remove mode bar buttons by name.","See ./components/modebar/buttons.js for the list of names."].join(" ")},modeBarButtonsToAdd:{valType:"any",dflt:[],description:["Add mode bar button using config objects","See ./components/modebar/buttons.js for list of arguments."].join(" ")},modeBarButtons:{valType:"any",dflt:!1,description:["Define fully custom mode bar buttons as nested array,","where the outer arrays represents button groups, and","the inner arrays have buttons config objects or names of default buttons","See ./components/modebar/buttons.js for more info."].join(" ")},toImageButtonOptions:{valType:"any",dflt:{},description:["Statically override options for toImage modebar button","allowed keys are format, filename, width, height, scale","see ../components/modebar/buttons.js"].join(" ")},displaylogo:{valType:"boolean",dflt:!0,description:["Determines whether or not the plotly logo is displayed","on the end of the mode bar."].join(" ")},watermark:{valType:"boolean",dflt:!1,description:"watermark the images with the company's logo"},plotGlPixelRatio:{valType:"number",dflt:2,min:1,max:4,description:["Set the pixel ratio during WebGL image export.","This config option was formerly named `plot3dPixelRatio`","which is now deprecated."].join(" ")},setBackground:{valType:"any",dflt:"transparent",description:["Set function to add the background color (i.e. `layout.paper_color`)","to a different container.","This function take the graph div as first argument and the current background","color as second argument.","Alternatively, set to string *opaque* to ensure there is white behind it."].join(" ")},topojsonURL:{valType:"string",noBlank:!0,dflt:"https://cdn.plot.ly/",description:["Set the URL to topojson used in geo charts.","By default, the topojson files are fetched from cdn.plot.ly.","For example, set this option to:","<path-to-plotly.js>/dist/topojson/","to render geographical feature using the topojson files","that ship with the plotly.js module."].join(" ")},mapboxAccessToken:{valType:"string",dflt:null,description:["Mapbox access token (required to plot mapbox trace types)","If using an Mapbox Atlas server, set this option to ''","so that plotly.js won't attempt to authenticate to the public Mapbox server."].join(" ")},logging:{valType:"boolean",dflt:1,description:["Turn all console logging on or off (errors will be thrown)","This should ONLY be set via Plotly.setPlotConfig","Available levels:","0: no logs","1: warnings and errors, but not informational messages","2: verbose logs"].join(" ")},queueLength:{valType:"integer",min:0,dflt:0,description:"Sets the length of the undo/redo queue."},globalTransforms:{valType:"any",dflt:[],description:["Set global transform to be applied to all traces with no","specification needed"].join(" ")},locale:{valType:"string",dflt:"en-US",description:["Which localization should we use?","Should be a string like 'en' or 'en-US'."].join(" ")},locales:{valType:"any",dflt:{},description:["Localization definitions","Locales can be provided either here (specific to one chart) or globally","by registering them as modules.","Should be an object of objects {locale: {dictionary: {...}, format: {...}}}","{","  da: {","      dictionary: {'Reset axes': 'Nulstil aksler', ...},","      format: {months: [...], shortMonths: [...]}","  },","  ...","}","All parts are optional. When looking for translation or format fields, we","look first for an exact match in a config locale, then in a registered","module. If those fail, we strip off any regionalization ('en-US' -> 'en')","and try each (config, registry) again. The final fallback for translation","is untranslated (which is US English) and for formats is the base English","(the only consequence being the last fallback date format %x is DD/MM/YYYY","instead of MM/DD/YYYY). Currently `grouping` and `currency` are ignored","for our automatic number formatting, but can be used in custom formats."].join(" ")}},i={};!function t(e,n){for(var r in e){var i=e[r];i.valType?n[r]=i.dflt:(n[r]||(n[r]={}),t(i,n[r]))}}(r,i),t.exports={configAttributes:r,dfltConfig:i}},function(t,e,n){"use strict";var r=n(348),i=n(1433),o=n(1434),a=n(1435),s=n(241).extendFlat,A=r({editType:"calc",description:["Sets the global font.","Note that fonts used in traces and other","layout components inherit from the global font."].join(" ")});A.family.dflt='"Open Sans", verdana, arial, sans-serif',A.size.dflt=12,A.color.dflt=o.defaultLine,t.exports={font:A,title:{text:{valType:"string",role:"info",editType:"layoutstyle",description:["Sets the plot's title.","Note that before the existence of `title.text`, the title's","contents used to be defined as the `title` attribute itself.","This behavior has been deprecated."].join(" ")},font:r({editType:"layoutstyle",description:["Sets the title font.","Note that the title's font used to be customized","by the now deprecated `titlefont` attribute."].join(" ")}),xref:{valType:"enumerated",dflt:"container",values:["container","paper"],role:"info",editType:"layoutstyle",description:["Sets the container `x` refers to.","*container* spans the entire `width` of the plot.","*paper* refers to the width of the plotting area only."].join(" ")},yref:{valType:"enumerated",dflt:"container",values:["container","paper"],role:"info",editType:"layoutstyle",description:["Sets the container `y` refers to.","*container* spans the entire `height` of the plot.","*paper* refers to the height of the plotting area only."].join(" ")},x:{valType:"number",min:0,max:1,dflt:.5,role:"style",editType:"layoutstyle",description:["Sets the x position with respect to `xref` in normalized","coordinates from *0* (left) to *1* (right)."].join(" ")},y:{valType:"number",min:0,max:1,dflt:"auto",role:"style",editType:"layoutstyle",description:["Sets the y position with respect to `yref` in normalized","coordinates from *0* (bottom) to *1* (top).","*auto* places the baseline of the title onto the","vertical center of the top margin."].join(" ")},xanchor:{valType:"enumerated",dflt:"auto",values:["auto","left","center","right"],role:"info",editType:"layoutstyle",description:["Sets the title's horizontal alignment with respect to its x position.","*left* means that the title starts at x,","*right* means that the title ends at x","and *center* means that the title's center is at x.","*auto* divides `xref` by three and calculates the `xanchor`","value automatically based on the value of `x`."].join(" ")},yanchor:{valType:"enumerated",dflt:"auto",values:["auto","top","middle","bottom"],role:"info",editType:"layoutstyle",description:["Sets the title's vertical alignment with respect to its y position.","*top* means that the title's cap line is at y,","*bottom* means that the title's baseline is at y","and *middle* means that the title's midline is at y.","*auto* divides `yref` by three and calculates the `yanchor`","value automatically based on the value of `y`."].join(" ")},pad:s(a({editType:"layoutstyle"}),{description:["Sets the padding of the title.","Each padding value only applies when the corresponding","`xanchor`/`yanchor` value is set accordingly. E.g. for left","padding to take effect, `xanchor` must be set to *left*.","The same rule applies if `xanchor`/`yanchor` is determined automatically.","Padding is muted if the respective anchor value is *middle*/*center*."].join(" ")}),editType:"layoutstyle"},autosize:{valType:"boolean",role:"info",dflt:!1,editType:"none",description:["Determines whether or not a layout width or height","that has been left undefined by the user","is initialized on each relayout.","Note that, regardless of this attribute,","an undefined layout width or height","is always initialized on the first call to plot."].join(" ")},width:{valType:"number",role:"info",min:10,dflt:700,editType:"plot",description:["Sets the plot's width (in px)."].join(" ")},height:{valType:"number",role:"info",min:10,dflt:450,editType:"plot",description:["Sets the plot's height (in px)."].join(" ")},margin:{l:{valType:"number",role:"info",min:0,dflt:80,editType:"plot",description:"Sets the left margin (in px)."},r:{valType:"number",role:"info",min:0,dflt:80,editType:"plot",description:"Sets the right margin (in px)."},t:{valType:"number",role:"info",min:0,dflt:100,editType:"plot",description:"Sets the top margin (in px)."},b:{valType:"number",role:"info",min:0,dflt:80,editType:"plot",description:"Sets the bottom margin (in px)."},pad:{valType:"number",role:"info",min:0,dflt:0,editType:"plot",description:["Sets the amount of padding (in px)","between the plotting area and the axis lines"].join(" ")},autoexpand:{valType:"boolean",role:"info",dflt:!0,editType:"plot"},editType:"plot"},paper_bgcolor:{valType:"color",role:"style",dflt:o.background,editType:"plot",description:"Sets the color of paper where the graph is drawn."},plot_bgcolor:{valType:"color",role:"style",dflt:o.background,editType:"layoutstyle",description:["Sets the color of plotting area in-between x and y axes."].join(" ")},separators:{valType:"string",role:"style",editType:"plot",description:["Sets the decimal and thousand separators.","For example, *. * puts a '.' before decimals and a space","between thousands. In English locales, dflt is *.,* but","other locales may alter this default."].join(" ")},hidesources:{valType:"boolean",role:"info",dflt:!1,editType:"plot",description:["Determines whether or not a text link citing the data source is","placed at the bottom-right cored of the figure.","Has only an effect only on graphs that have been generated via","forked graphs from the plotly service (at https://plot.ly or on-premise)."].join(" ")},showlegend:{valType:"boolean",role:"info",editType:"legend",description:["Determines whether or not a legend is drawn.","Default is `true` if there is a trace to show and any of these:","a) Two or more traces would by default be shown in the legend.","b) One pie trace is shown in the legend.","c) One trace is explicitly given with `showlegend: true`."].join(" ")},colorway:{valType:"colorlist",dflt:o.defaults,role:"style",editType:"calc",description:"Sets the default trace colors."},datarevision:{valType:"any",role:"info",editType:"calc",description:["If provided, a changed value tells `Plotly.react` that","one or more data arrays has changed. This way you can modify","arrays in-place rather than making a complete new copy for an","incremental change.","If NOT provided, `Plotly.react` assumes that data arrays are","being treated as immutable, thus any data array with a","different identity from its predecessor contains new data."].join(" ")},uirevision:{valType:"any",role:"info",editType:"none",description:["Used to allow user interactions with the plot to persist after","`Plotly.react` calls that are unaware of these interactions.","If `uirevision` is omitted, or if it is given and it changed from","the previous `Plotly.react` call, the exact new figure is used.","If `uirevision` is truthy and did NOT change, any attribute","that has been affected by user interactions and did not receive a","different value in the new figure will keep the interaction value.","`layout.uirevision` attribute serves as the default for","`uirevision` attributes in various sub-containers. For finer","control you can set these sub-attributes directly. For example,","if your app separately controls the data on the x and y axes you","might set `xaxis.uirevision=*time*` and `yaxis.uirevision=*cost*`.","Then if only the y data is changed, you can update","`yaxis.uirevision=*quantity*` and the y axis range will reset but","the x axis range will retain any user-driven zoom."].join(" ")},editrevision:{valType:"any",role:"info",editType:"none",description:["Controls persistence of user-driven changes in `editable: true`","configuration, other than trace names and axis titles.","Defaults to `layout.uirevision`."].join(" ")},selectionrevision:{valType:"any",role:"info",editType:"none",description:["Controls persistence of user-driven changes in selected points","from all traces."].join(" ")},template:{valType:"any",role:"info",editType:"calc",description:["Default attributes to be applied to the plot. Templates can be","created from existing plots using `Plotly.makeTemplate`, or","created manually. They should be objects with format:","`{layout: layoutTemplate, data: {[type]: [traceTemplate, ...]}, ...}`","`layoutTemplate` and `traceTemplate` are objects matching the","attribute structure of `layout` and a data trace. ","Trace templates are applied cyclically to traces of each type.","Container arrays (eg `annotations`) have special handling:","An object ending in `defaults` (eg `annotationdefaults`) is applied","to each array item. But if an item has a `templateitemname` key","we look in the template array for an item with matching `name` and","apply that instead. If no matching `name` is found we mark the item","invisible. Any named template item not referenced is appended to","the end of the array, so you can use this for a watermark annotation","or a logo image, for example. To omit one of these items on the plot,","make an item with matching `templateitemname` and `visible: false`."].join(" ")},modebar:{orientation:{valType:"enumerated",values:["v","h"],dflt:"h",role:"info",editType:"modebar",description:"Sets the orientation of the modebar."},bgcolor:{valType:"color",role:"style",editType:"modebar",description:"Sets the background color of the modebar."},color:{valType:"color",role:"style",editType:"modebar",description:"Sets the color of the icons in the modebar."},activecolor:{valType:"color",role:"style",editType:"modebar",description:"Sets the color of the active or hovered on icons in the modebar."},uirevision:{valType:"any",role:"info",editType:"none",description:["Controls persistence of user-driven changes related to the modebar,","including `hovermode`, `dragmode`, and `showspikes` at both the","root level and inside subplots. Defaults to `layout.uirevision`."].join(" ")},editType:"modebar"},meta:{valType:"any",arrayOk:!0,role:"info",editType:"plot",description:["Assigns extra meta information that can be used in various `text` attributes.","Attributes such as the graph, axis and colorbar `title.text`, annotation `text`","`trace.name` in legend items, `rangeselector`, `updatemenues` and `sliders` `label` text","all support `meta`. One can access `meta` fields using template strings:","`%{meta[i]}` where `i` is the index of the `meta`","item in question.","`meta` can also be an object for example `{key: value}` which can be accessed","%{meta[key]}."].join(" ")},transition:s({},i.transition,{description:["Sets transition options used during Plotly.react updates."].join(" "),editType:"none"}),_deprecated:{title:{valType:"string",role:"info",editType:"layoutstyle",description:["Value of `title` is no longer a simple *string* but a set of sub-attributes.","To set the contents of the title, please use `title.text` now."].join(" ")},titlefont:r({editType:"layoutstyle",description:["Former `titlefont` is now the sub-attribute `font` of `title`.","To customize title font properties, please use `title.font` now."].join(" ")})}}},function(t,e,n){"use strict";t.exports={mode:{valType:"enumerated",dflt:"afterall",role:"info",values:["immediate","next","afterall"],description:["Describes how a new animate call interacts with currently-running","animations. If `immediate`, current animations are interrupted and","the new animation is started. If `next`, the current frame is allowed","to complete, after which the new animation is started. If `afterall`","all existing frames are animated to completion before the new animation","is started."].join(" ")},direction:{valType:"enumerated",role:"info",values:["forward","reverse"],dflt:"forward",description:["The direction in which to play the frames triggered by the animation call"].join(" ")},fromcurrent:{valType:"boolean",dflt:!1,role:"info",description:["Play frames starting at the current frame instead of the beginning."].join(" ")},frame:{duration:{valType:"number",role:"info",min:0,dflt:500,description:["The duration in milliseconds of each frame. If greater than the frame","duration, it will be limited to the frame duration."].join(" ")},redraw:{valType:"boolean",role:"info",dflt:!0,description:["Redraw the plot at completion of the transition. This is desirable","for transitions that include properties that cannot be transitioned,","but may significantly slow down updates that do not require a full","redraw of the plot"].join(" ")}},transition:{duration:{valType:"number",role:"info",min:0,dflt:500,editType:"none",description:["The duration of the transition, in milliseconds. If equal to zero,","updates are synchronous."].join(" ")},easing:{valType:"enumerated",dflt:"cubic-in-out",values:["linear","quad","cubic","sin","exp","circle","elastic","back","bounce","linear-in","quad-in","cubic-in","sin-in","exp-in","circle-in","elastic-in","back-in","bounce-in","linear-out","quad-out","cubic-out","sin-out","exp-out","circle-out","elastic-out","back-out","bounce-out","linear-in-out","quad-in-out","cubic-in-out","sin-in-out","exp-in-out","circle-in-out","elastic-in-out","back-in-out","bounce-in-out"],role:"info",editType:"none",description:"The easing function used for the transition"},ordering:{valType:"enumerated",values:["layout first","traces first"],dflt:"layout first",role:"info",editType:"none",description:["Determines whether the figure's layout or traces smoothly transitions","during updates that make both traces and layout change."].join(" ")}}}},function(t,e,n){"use strict";e.defaults=["#1f77b4","#ff7f0e","#2ca02c","#d62728","#9467bd","#8c564b","#e377c2","#7f7f7f","#bcbd22","#17becf"],e.defaultLine="#444",e.lightLine="#eee",e.background="#fff",e.borderLine="#BEC8D9",e.lightFraction=1e3/11},function(t,e,n){"use strict";t.exports=function(t){var e=t.editType;return{t:{valType:"number",dflt:0,role:"style",editType:e,description:"The amount of padding (in px) along the top of the component."},r:{valType:"number",dflt:0,role:"style",editType:e,description:"The amount of padding (in px) on the right side of the component."},b:{valType:"number",dflt:0,role:"style",editType:e,description:"The amount of padding (in px) along the bottom of the component."},l:{valType:"number",dflt:0,role:"style",editType:e,description:"The amount of padding (in px) on the left side of the component."},editType:e}}},function(t,e,n){"use strict";var r=n(100),i=n(188),o=n(711);function a(t,e){return t<e}function s(t,e){return t<=e}function A(t,e){return t>e}function l(t,e){return t>=e}e.findBin=function(t,e,n){if(r(e.start))return n?Math.ceil((t-e.start)/e.size-1e-9)-1:Math.floor((t-e.start)/e.size+1e-9);var o,c,u=0,g=e.length,h=0,d=g>1?(e[g-1]-e[0])/(g-1):1;for(c=d>=0?n?a:s:n?l:A,t+=1e-9*d*(n?-1:1)*(d>=0?1:-1);u<g&&h++<100;)c(e[o=Math.floor((u+g)/2)],t)?u=o+1:g=o;return h>90&&i.log("Long binary search..."),u-1},e.sorterAsc=function(t,e){return t-e},e.sorterDes=function(t,e){return e-t},e.distinctVals=function(t){var n=t.slice();n.sort(e.sorterAsc);for(var r=n.length-1,i=n[r]-n[0]||1,o=i/(r||1)/1e4,a=[n[0]],s=0;s<r;s++)n[s+1]>n[s]+o&&(i=Math.min(i,n[s+1]-n[s]),a.push(n[s+1]));return{vals:a,minDiff:i}},e.roundUp=function(t,e,n){for(var r,i=0,o=e.length-1,a=0,s=n?0:1,A=n?1:0,l=n?Math.ceil:Math.floor;i<o&&a++<100;)e[r=l((i+o)/2)]<=t?i=r+s:o=r-A;return e[i]},e.sort=function(t,e){for(var n=0,r=0,i=1;i<t.length;i++){var o=e(t[i],t[i-1]);if(o<0?n=1:o>0&&(r=1),n&&r)return t.sort(e)}return r?t:t.reverse()},e.findIndexOfMin=function(t,e){e=e||o;for(var n,r=1/0,i=0;i<t.length;i++){var a=e(t[i]);a<r&&(r=a,n=i)}return n}},function(t,e,n){"use strict";var r=n(100),i=n(186).isArrayOrTypedArray;e.aggNums=function(t,n,o,a){var s,A;if((!a||a>o.length)&&(a=o.length),r(n)||(n=!1),i(o[0])){for(A=new Array(a),s=0;s<a;s++)A[s]=e.aggNums(t,n,o[s]);o=A}for(s=0;s<a;s++)r(n)?r(o[s])&&(n=t(+n,+o[s])):n=o[s];return n},e.len=function(t){return e.aggNums(function(t){return t+1},0,t)},e.mean=function(t,n){return n||(n=e.len(t)),e.aggNums(function(t,e){return t+e},0,t)/n},e.midRange=function(t){if(void 0!==t&&0!==t.length)return(e.aggNums(Math.max,null,t)+e.aggNums(Math.min,null,t))/2},e.variance=function(t,n,i){return n||(n=e.len(t)),r(i)||(i=e.mean(t,n)),e.aggNums(function(t,e){return t+Math.pow(e-i,2)},0,t)/n},e.stdev=function(t,n,r){return Math.sqrt(e.variance(t,n,r))},e.median=function(t){var n=t.slice().sort();return e.interp(n,.5)},e.interp=function(t,e){if(!r(e))throw"n should be a finite number";if((e=e*t.length-.5)<0)return t[0];if(e>t.length-1)return t[t.length-1];var n=e%1;return n*t[Math.ceil(e)]+(1-n)*t[Math.floor(e)]}},function(t,e,n){"use strict";e.init2dArray=function(t,e){for(var n=new Array(t),r=0;r<t;r++)n[r]=new Array(e);return n},e.transposeRagged=function(t){var e,n,r=0,i=t.length;for(e=0;e<i;e++)r=Math.max(r,t[e].length);var o=new Array(r);for(e=0;e<r;e++)for(o[e]=new Array(i),n=0;n<i;n++)o[e][n]=t[n][e];return o},e.dot=function(t,n){if(!t.length||!n.length||t.length!==n.length)return null;var r,i,o=t.length;if(t[0].length)for(r=new Array(o),i=0;i<o;i++)r[i]=e.dot(t[i],n);else if(n[0].length){var a=e.transposeRagged(n);for(r=new Array(a.length),i=0;i<a.length;i++)r[i]=e.dot(t,a[i])}else for(r=0,i=0;i<o;i++)r+=t[i]*n[i];return r},e.translationMatrix=function(t,e){return[[1,0,t],[0,1,e],[0,0,1]]},e.rotationMatrix=function(t){var e=t*Math.PI/180;return[[Math.cos(e),-Math.sin(e),0],[Math.sin(e),Math.cos(e),0],[0,0,1]]},e.rotationXYMatrix=function(t,n,r){return e.dot(e.dot(e.translationMatrix(n,r),e.rotationMatrix(t)),e.translationMatrix(-n,-r))},e.apply2DTransform=function(t){return function(){var n=arguments;3===n.length&&(n=n[0]);var r=1===arguments.length?n[0]:[n[0],n[1]];return e.dot(t,[r[0],r[1],1]).slice(0,2)}},e.apply2DTransform2=function(t){var n=e.apply2DTransform(t);return function(t){return n(t.slice(0,2)).concat(n(t.slice(2,4)))}}},function(t,e,n){"use strict";var r=n(187),i=r.mod,o=r.modHalf,a=Math.PI,s=2*a;function A(t){return Math.abs(t[1]-t[0])>s-1e-14}function l(t,e){return o(e-t,s)}function c(t,e){if(A(e))return!0;var n,r;e[0]<e[1]?(n=e[0],r=e[1]):(n=e[1],r=e[0]),(n=i(n,s))>(r=i(r,s))&&(r+=s);var o=i(t,s),a=o+s;return o>=n&&o<=r||a>=n&&a<=r}function u(t,e,n,r,i,o,l){i=i||0,o=o||0;var c,u,g,h,d,f=A([n,r]);function p(t,e){return[t*Math.cos(e)+i,o-t*Math.sin(e)]}f?(c=0,u=a,g=s):n<r?(c=n,g=r):(c=r,g=n),t<e?(h=t,d=e):(h=e,d=t);var I,C=Math.abs(g-c)<=a?0:1;function m(t,e,n){return"A"+[t,t]+" "+[0,C,n]+" "+p(t,e)}return f?I=null===h?"M"+p(d,c)+m(d,u,0)+m(d,g,0)+"Z":"M"+p(h,c)+m(h,u,0)+m(h,g,0)+"ZM"+p(d,c)+m(d,u,1)+m(d,g,1)+"Z":null===h?(I="M"+p(d,c)+m(d,g,0),l&&(I+="L0,0Z")):I="M"+p(h,c)+"L"+p(d,c)+m(d,g,0)+"L"+p(h,g)+m(h,c,1)+"Z",I}t.exports={deg2rad:function(t){return t/180*a},rad2deg:function(t){return t/a*180},angleDelta:l,angleDist:function(t,e){return Math.abs(l(t,e))},isFullCircle:A,isAngleInsideSector:c,isPtInsideSector:function(t,e,n,r){return!!c(e,r)&&(n[0]<n[1]?(i=n[0],o=n[1]):(i=n[1],o=n[0]),t>=i&&t<=o);var i,o},pathArc:function(t,e,n,r,i){return u(null,t,e,n,r,i,0)},pathSector:function(t,e,n,r,i){return u(null,t,e,n,r,i,1)},pathAnnulus:function(t,e,n,r,i,o){return u(t,e,n,r,i,o,1)}}},function(t,e,n){"use strict";e.isLeftAnchor=function(t){return"left"===t.xanchor||"auto"===t.xanchor&&t.x<=1/3},e.isCenterAnchor=function(t){return"center"===t.xanchor||"auto"===t.xanchor&&t.x>1/3&&t.x<2/3},e.isRightAnchor=function(t){return"right"===t.xanchor||"auto"===t.xanchor&&t.x>=2/3},e.isTopAnchor=function(t){return"top"===t.yanchor||"auto"===t.yanchor&&t.y>=2/3},e.isMiddleAnchor=function(t){return"middle"===t.yanchor||"auto"===t.yanchor&&t.y>1/3&&t.y<2/3},e.isBottomAnchor=function(t){return"bottom"===t.yanchor||"auto"===t.yanchor&&t.y<=1/3}},function(t,e,n){"use strict";var r,i,o,a=n(187).mod;function s(t,e,n,r,i,o,a,s){var A=n-t,l=i-t,c=a-i,u=r-e,g=o-e,h=s-o,d=A*h-c*u;if(0===d)return null;var f=(l*h-c*g)/d,p=(l*u-A*g)/d;return p<0||p>1||f<0||f>1?null:{x:t+A*f,y:e+u*f}}function A(t,e,n,r,i){var o=r*t+i*e;if(o<0)return r*r+i*i;if(o>n){var a=r-t,s=i-e;return a*a+s*s}var A=r*e-i*t;return A*A/n}e.segmentsIntersect=s,e.segmentDistance=function(t,e,n,r,i,o,a,l){if(s(t,e,n,r,i,o,a,l))return 0;var c=n-t,u=r-e,g=a-i,h=l-o,d=c*c+u*u,f=g*g+h*h,p=Math.min(A(c,u,d,i-t,o-e),A(c,u,d,a-t,l-e),A(g,h,f,t-i,e-o),A(g,h,f,n-i,r-o));return Math.sqrt(p)},e.getTextLocation=function(t,e,n,s){if(t===i&&s===o||(r={},i=t,o=s),r[n])return r[n];var A=t.getPointAtLength(a(n-s/2,e)),l=t.getPointAtLength(a(n+s/2,e)),c=Math.atan((l.y-A.y)/(l.x-A.x)),u=t.getPointAtLength(a(n,e)),g={x:(4*u.x+A.x+l.x)/6,y:(4*u.y+A.y+l.y)/6,theta:c};return r[n]=g,g},e.clearLocationCache=function(){i=null},e.getVisibleSegment=function(t,e,n){var r,i,o=e.left,a=e.right,s=e.top,A=e.bottom,l=0,c=t.getTotalLength(),u=c;function g(e){var n=t.getPointAtLength(e);0===e?r=n:e===c&&(i=n);var l=n.x<o?o-n.x:n.x>a?n.x-a:0,u=n.y<s?s-n.y:n.y>A?n.y-A:0;return Math.sqrt(l*l+u*u)}for(var h=g(l);h;){if((l+=h+n)>u)return;h=g(l)}for(h=g(u);h;){if(l>(u-=h+n))return;h=g(u)}return{min:l,max:u,len:u-l,total:c,isClosed:0===l&&u===c&&Math.abs(r.x-i.x)<.1&&Math.abs(r.y-i.y)<.1}},e.findPointOnPath=function(t,e,n,r){for(var i,o,a,s=(r=r||{}).pathLength||t.getTotalLength(),A=r.tolerance||.001,l=r.iterationLimit||30,c=t.getPointAtLength(0)[n]>t.getPointAtLength(s)[n]?-1:1,u=0,g=0,h=s;u<l;){if(i=(g+h)/2,a=(o=t.getPointAtLength(i))[n]-e,Math.abs(a)<A)return o;c*a>0?h=i:g=i,u++}return o}},function(t,e,n){"use strict";var r={};function i(t){t&&null!==t.timer&&(clearTimeout(t.timer),t.timer=null)}e.throttle=function(t,e,n){var o=r[t],a=Date.now();if(!o){for(var s in r)r[s].ts<a-6e4&&delete r[s];o=r[t]={ts:0,timer:null}}function A(){n(),o.ts=Date.now(),o.onDone&&(o.onDone(),o.onDone=null)}i(o),a>o.ts+e?A():o.timer=setTimeout(function(){A(),o.timer=null},e)},e.done=function(t){var e=r[t];return e&&e.timer?new Promise(function(t){var n=e.onDone;e.onDone=function(){n&&n(),t(),e.onDone=null}}):Promise.resolve()},e.clear=function(t){if(t)i(r[t]),delete r[t];else for(var n in r)e.clear(n)}},function(t,e,n){"use strict";t.exports=function(t){t._responsiveChartHandler&&(window.removeEventListener("resize",t._responsiveChartHandler),delete t._responsiveChartHandler)}},function(t,e,n){"use strict";var r=n(185);t.exports=function(t,e,n){var i=t.selectAll("g."+n.replace(/\s/g,".")).data(e,function(t){return t[0].trace.uid});i.exit().remove(),i.enter().append("g").attr("class",n),i.order();var o=t.classed("rangeplot")?"nodeRangePlot3":"node3";return i.each(function(t){t[0][o]=r.select(this)}),i}},function(t,e,n){"use strict";var r=n(707);t.exports=function(t,e){for(var n=t._context.locale,i=0;i<2;i++){for(var o=t._context.locales,a=0;a<2;a++){var s=(o[n]||{}).dictionary;if(s){var A=s[e];if(A)return A}o=r.localeRegistry}var l=n.split("-")[0];if(l===n)break;n=l}return e}},function(t,e,n){"use strict";var r=n(185),i=n(100),o=[];t.exports=function(t,e){if(-1===o.indexOf(t)){o.push(t);var n=1e3;i(e)?n=e:"long"===e&&(n=3e3);var a=r.select("body").selectAll(".plotly-notifier").data([0]);a.enter().append("div").classed("plotly-notifier",!0),a.selectAll(".notifier-note").data(o).enter().append("div").classed("notifier-note",!0).style("opacity",0).each(function(t){var e=r.select(this);e.append("button").classed("notifier-close",!0).html("&times;").on("click",function(){e.transition().call(s)});for(var i=e.append("p"),o=t.split(/<br\s*\/?>/g),a=0;a<o.length;a++)a&&i.append("br"),i.append("span").text(o[a]);e.transition().duration(700).style("opacity",1).transition().delay(n).call(s)})}function s(t){t.duration(700).style("opacity",0).each("end",function(t){var e=o.indexOf(t);-1!==e&&o.splice(e,1),r.select(this).remove()})}}},function(t,e,n){"use strict";t.exports=function(t){for(var e={},n=[],r=0,i=0;i<t.length;i++){var o=t[i];1!==e[o]&&(e[o]=1,n[r++]=o)}return n}},function(t,e,n){"use strict";function r(t){return!0===t.visible}function i(t){var e=t[0].trace;return!0===e.visible&&0!==e._length}t.exports=function(t){for(var e,n=(e=t,Array.isArray(e)&&Array.isArray(e[0])&&e[0][0]&&e[0][0].trace?i:r),o=[],a=0;a<t.length;a++){var s=t[a];n(s)&&o.push(s)}return o}},function(t,e,n){"use strict";var r=n(100),i=n(346).BADNUM,o=/^['"%,$#\s']+|[, ]|['"%,$#\s']+$/g;t.exports=function(t){return"string"==typeof t&&(t=t.replace(o,"")),r(t)?Number(t):i}},function(t,e,n){"use strict";n.r(e);var r=n(20);function i(t,e){return function(n){var i=new r.WrappedError(t,n);return e&&console.error(i),Promise.reject(i)}}function o(t,e){void 0!==e&&(t.textContent=e),void 0!==window.MathJax&&MathJax.Hub.Queue(["Typeset",MathJax.Hub,t])}function a(t){var e=document.createElement("div");return e.textContent=t,e.innerHTML}var s,A=n(61),l=(s=function(t,e){return(s=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(t,e)},function(t,e){function n(){this.constructor=t}s(t,e),t.prototype=null===e?Object.create(e):(n.prototype=e.prototype,new n)}),c=function(){return(c=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},u=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return l(e,t),e.prototype.defaults=function(){return c({},t.prototype.defaults.call(this),{_model_name:"DescriptionStyleModel",_model_module:"@jupyter-widgets/controls",_model_module_version:A.a})},e.styleProperties={description_width:{selector:".widget-label",attribute:"width",default:null}},e}(r.StyleModel),g=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return l(e,t),e.prototype.defaults=function(){return c({},t.prototype.defaults.call(this),{_model_name:"DescriptionModel",_view_name:"DescriptionView",_view_module:"@jupyter-widgets/controls",_model_module:"@jupyter-widgets/controls",_view_module_version:A.a,_model_module_version:A.a,description:"",description_tooltip:null})},e}(r.DOMWidgetModel),h=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return l(e,t),e.prototype.render=function(){this.label=document.createElement("label"),this.el.appendChild(this.label),this.label.className="widget-label",this.label.style.display="none",this.listenTo(this.model,"change:description",this.updateDescription),this.listenTo(this.model,"change:description_tooltip",this.updateDescription),this.updateDescription()},e.prototype.typeset=function(t,e){this.displayed.then(function(){return o(t,e)})},e.prototype.updateDescription=function(){var t=this.model.get("description"),e=this.model.get("description_tooltip");null===e&&(e=t),0===t.length?this.label.style.display="none":(this.label.innerHTML=t,this.typeset(this.label),this.label.style.display=""),this.label.title=e},e}(r.DOMWidgetView),d=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return l(e,t),e}(g),f=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return l(e,t),e}(h),p=n(22),I=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),C=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return I(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"CoreWidgetModel",_view_module:"@jupyter-widgets/controls",_model_module:"@jupyter-widgets/controls",_view_module_version:A.a,_model_module_version:A.a})},e}(r.WidgetModel),m=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return I(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"CoreDOMWidgetModel",_view_module:"@jupyter-widgets/controls",_model_module:"@jupyter-widgets/controls",_view_module_version:A.a,_model_module_version:A.a})},e}(r.DOMWidgetModel),y=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return I(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"CoreDescriptionModel",_view_module:"@jupyter-widgets/controls",_model_module:"@jupyter-widgets/controls",_view_module_version:A.a,_model_module_version:A.a})},e}(g),E=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),v=function(){return(v=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},B=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return E(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{target:void 0,source:void 0,_model_name:"DirectionalLinkModel"})},e.prototype.initialize=function(e,n){t.prototype.initialize.call(this,e,n),this.on("change",this.updateBindings,this),this.updateBindings()},e.prototype.updateValue=function(t,e,n,r){if(!this._updating){this._updating=!0;try{n&&(n.set(r,t.get(e)),n.save_changes())}finally{this._updating=!1}}},e.prototype.updateBindings=function(){var t,e,n=this;this.cleanup(),t=this.get("source")||[null,null],this.sourceModel=t[0],this.sourceAttr=t[1],e=this.get("target")||[null,null],this.targetModel=e[0],this.targetAttr=e[1],this.sourceModel&&(this.listenTo(this.sourceModel,"change:"+this.sourceAttr,function(){n.updateValue(n.sourceModel,n.sourceAttr,n.targetModel,n.targetAttr)}),this.updateValue(this.sourceModel,this.sourceAttr,this.targetModel,this.targetAttr),this.listenToOnce(this.sourceModel,"destroy",this.cleanup)),this.targetModel&&this.listenToOnce(this.targetModel,"destroy",this.cleanup)},e.prototype.cleanup=function(){this.sourceModel&&(this.stopListening(this.sourceModel,"change:"+this.sourceAttr,null),this.stopListening(this.sourceModel,"destroy",null)),this.targetModel&&this.stopListening(this.targetModel,"destroy",null)},e.serializers=v({},C.serializers,{target:{deserialize:r.unpack_models},source:{deserialize:r.unpack_models}}),e}(C),b=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return E(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"LinkModel"})},e.prototype.updateBindings=function(){var e=this;t.prototype.updateBindings.call(this),this.targetModel&&this.listenTo(this.targetModel,"change:"+this.targetAttr,function(){e.updateValue(e.targetModel,e.targetAttr,e.sourceModel,e.sourceAttr)})},e.prototype.cleanup=function(){t.prototype.cleanup.call(this),this.targetModel&&this.stopListening(this.targetModel,"change:"+this.targetAttr,null)},e}(B),Q=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),w=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Q(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{value:!1,disabled:!1,_model_name:"BoolModel"})},e}(y),x=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Q(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{indent:!0,_view_name:"CheckboxView",_model_name:"CheckboxModel"})},e}(y),_=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Q(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-checkbox"),this.label.innerHTML="&#8203;",this.checkboxLabel=document.createElement("label"),this.checkboxLabel.classList.add("widget-label-basic"),this.el.appendChild(this.checkboxLabel),this.checkbox=document.createElement("input"),this.checkbox.setAttribute("type","checkbox"),this.checkboxLabel.appendChild(this.checkbox),this.descriptionSpan=document.createElement("span"),this.checkboxLabel.appendChild(this.descriptionSpan),this.listenTo(this.model,"change:indent",this.updateIndent),this.update(),this.updateDescription(),this.updateIndent()},e.prototype.updateDescription=function(){if(null!=this.checkboxLabel){var t=this.model.get("description");this.descriptionSpan.innerHTML=t,this.typeset(this.descriptionSpan),this.descriptionSpan.title=t,this.checkbox.title=t}},e.prototype.updateIndent=function(){var t=this.model.get("indent");this.label.style.display=t?"":"none"},e.prototype.events=function(){return{'click input[type="checkbox"]':"_handle_click"}},e.prototype._handle_click=function(){var t=this.model.get("value");this.model.set("value",!t,{updated_view:this}),this.touch()},e.prototype.update=function(e){return this.checkbox.checked=this.model.get("value"),void 0!==e&&e.updated_view==this||(this.checkbox.disabled=this.model.get("disabled")),t.prototype.update.call(this)},e}(h),D=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Q(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_view_name:"ToggleButtonView",_model_name:"ToggleButtonModel",tooltip:"",icon:"",button_style:""})},e}(w),M=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Q(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("jupyter-button"),this.el.classList.add("widget-toggle-button"),this.listenTo(this.model,"change:button_style",this.update_button_style),this.set_button_style(),this.update()},e.prototype.update_button_style=function(){this.update_mapped_classes(e.class_map,"button_style")},e.prototype.set_button_style=function(){this.set_mapped_classes(e.class_map,"button_style")},e.prototype.update=function(e){if(this.model.get("value")?this.el.classList.add("mod-active"):this.el.classList.remove("mod-active"),void 0===e||e.updated_view!==this){this.el.disabled=this.model.get("disabled"),this.el.setAttribute("title",this.model.get("tooltip"));var n=this.model.get("description"),r=this.model.get("icon");if(0===n.trim().length&&0===r.trim().length)this.el.innerHTML="&nbsp;";else{if(this.el.textContent="",r.trim().length){var i=document.createElement("i");this.el.appendChild(i),i.classList.add("fa"),i.classList.add("fa-"+r)}this.el.appendChild(document.createTextNode(n))}}return t.prototype.update.call(this)},e.prototype.events=function(){return{click:"_handle_click"}},e.prototype._handle_click=function(t){t.preventDefault();var e=this.model.get("value");this.model.set("value",!e,{updated_view:this}),this.touch()},Object.defineProperty(e.prototype,"tagName",{get:function(){return"button"},enumerable:!0,configurable:!0}),e.class_map={primary:["mod-primary"],success:["mod-success"],info:["mod-info"],warning:["mod-warning"],danger:["mod-danger"]},e}(r.DOMWidgetView),k=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Q(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{readout:"Invalid",_view_name:"ValidView",_model_name:"ValidModel"})},e}(w),S=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Q(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-valid"),this.el.classList.add("widget-inline-hbox");var e=document.createElement("i");this.el.appendChild(e),this.readout=document.createElement("span"),this.readout.classList.add("widget-valid-readout"),this.readout.classList.add("widget-readout"),this.el.appendChild(this.readout),this.update()},e.prototype.update=function(){this.el.classList.remove("mod-valid"),this.el.classList.remove("mod-invalid"),this.readout.textContent=this.model.get("readout"),this.model.get("value")?this.el.classList.add("mod-valid"):this.el.classList.add("mod-invalid")},e}(h),L=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),R=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return L(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"ButtonStyleModel",_model_module:"@jupyter-widgets/controls",_model_module_version:A.a})},e.styleProperties={button_color:{selector:"",attribute:"background-color",default:null},font_weight:{selector:"",attribute:"font-weight",default:""}},e}(r.StyleModel),N=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return L(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{description:"",tooltip:"",disabled:!1,icon:"",button_style:"",_view_name:"ButtonView",_model_name:"ButtonModel",style:null})},e}(m),F=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return L(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("jupyter-button"),this.el.classList.add("widget-button"),this.listenTo(this.model,"change:button_style",this.update_button_style),this.set_button_style(),this.update()},e.prototype.update=function(){this.el.disabled=this.model.get("disabled"),this.el.setAttribute("title",this.model.get("tooltip"));var e=this.model.get("description"),n=this.model.get("icon");if(e.length||n.length){if(this.el.textContent="",n.length){var r=document.createElement("i");r.classList.add("fa"),r.classList.add("fa-"+n),0===e.length&&r.classList.add("center"),this.el.appendChild(r)}this.el.appendChild(document.createTextNode(e))}return t.prototype.update.call(this)},e.prototype.update_button_style=function(){this.update_mapped_classes(e.class_map,"button_style")},e.prototype.set_button_style=function(){this.set_mapped_classes(e.class_map,"button_style")},e.prototype.events=function(){return{click:"_handle_click"}},e.prototype._handle_click=function(t){t.preventDefault(),this.send({event:"click"})},Object.defineProperty(e.prototype,"tagName",{get:function(){return"button"},enumerable:!0,configurable:!0}),e.class_map={primary:["mod-primary"],success:["mod-success"],info:["mod-info"],warning:["mod-warning"],danger:["mod-danger"]},e}(r.DOMWidgetView),T=n(5),U=n(25),O=n(6),G=n(44),H=n.n(G),j=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Y=function(){return(Y=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},P=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return j(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_view_name:"BoxView",_model_name:"BoxModel",children:[],box_style:""})},e.serializers=Y({},m.serializers,{children:{deserialize:r.unpack_models}}),e}(m),K=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return j(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_view_name:"HBoxView",_model_name:"HBoxModel"})},e}(P),z=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return j(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_view_name:"VBoxView",_model_name:"VBoxModel"})},e}(P),J=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return j(e,t),e.prototype._createElement=function(t){return this.pWidget=new r.JupyterPhosphorPanelWidget({view:this}),this.pWidget.node},e.prototype._setElement=function(t){if(this.el||t!==this.pWidget.node)throw new Error("Cannot reset the DOM element.");this.el=this.pWidget.node,this.$el=H()(this.pWidget.node)},e.prototype.initialize=function(e){t.prototype.initialize.call(this,e),this.children_views=new r.ViewList(this.add_child_model,null,this),this.listenTo(this.model,"change:children",this.update_children),this.listenTo(this.model,"change:box_style",this.update_box_style),this.pWidget.addClass("jupyter-widgets"),this.pWidget.addClass("widget-container"),this.pWidget.addClass("widget-box")},e.prototype.render=function(){t.prototype.render.call(this),this.update_children(),this.set_box_style()},e.prototype.update_children=function(){this.children_views.update(this.model.get("children")).then(function(t){t.forEach(function(t){U.MessageLoop.postMessage(t.pWidget,O.Widget.ResizeMessage.UnknownSize)})})},e.prototype.update_box_style=function(){this.update_mapped_classes(e.class_map,"box_style")},e.prototype.set_box_style=function(){this.set_mapped_classes(e.class_map,"box_style")},e.prototype.add_child_model=function(t){var e=this,n=new O.Widget;return this.pWidget.addWidget(n),this.create_child_view(t).then(function(t){var r=T.ArrayExt.firstIndexOf(e.pWidget.widgets,n);return e.pWidget.insertWidget(r,t.pWidget),n.dispose(),t}).catch(i("Could not add child view to box",!0))},e.prototype.remove=function(){this.children_views=null,t.prototype.remove.call(this)},e.class_map={success:["alert","alert-success"],info:["alert","alert-info"],warning:["alert","alert-warning"],danger:["alert","alert-danger"]},e}(r.DOMWidgetView),q=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return j(e,t),e.prototype.initialize=function(e){t.prototype.initialize.call(this,e),this.pWidget.addClass("widget-hbox")},e}(J),V=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return j(e,t),e.prototype.initialize=function(e){t.prototype.initialize.call(this,e),this.pWidget.addClass("widget-vbox")},e}(J),W=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return j(e,t),e.prototype.initialize=function(e){t.prototype.initialize.call(this,e),this.pWidget.addClass("widget-gridbox"),this.pWidget.removeClass("widget-box")},e}(J),X=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return j(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_view_name:"GridBoxView",_model_name:"GridBoxModel"})},e}(P),Z=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),$=function(){return($=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},tt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Z(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"ImageModel",_view_name:"ImageView",format:"png",width:"",height:"",value:new DataView(new ArrayBuffer(0))})},e.serializers=$({},m.serializers,{value:{serialize:function(t){return new DataView(t.buffer.slice(0))}}}),e}(m),et=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Z(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.pWidget.addClass("jupyter-widgets"),this.pWidget.addClass("widget-image"),this.update()},e.prototype.update=function(){var e,n=this.model.get("format"),r=this.model.get("value");if("url"!==n){var i=new Blob([r],{type:"image/"+this.model.get("format")});e=URL.createObjectURL(i)}else e=new TextDecoder("utf-8").decode(r.buffer);var o=this.el.src;this.el.src=e,o&&"string"!=typeof o&&URL.revokeObjectURL(o);var a=this.model.get("width");void 0!==a&&a.length>0?this.el.setAttribute("width",a):this.el.removeAttribute("width");var s=this.model.get("height");return void 0!==s&&s.length>0?this.el.setAttribute("height",s):this.el.removeAttribute("height"),t.prototype.update.call(this)},e.prototype.remove=function(){this.el.src&&URL.revokeObjectURL(this.el.src),t.prototype.remove.call(this)},Object.defineProperty(e.prototype,"tagName",{get:function(){return"img"},enumerable:!0,configurable:!0}),e}(r.DOMWidgetView),nt=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),rt=function(){return(rt=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},it=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return nt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"VideoModel",_view_name:"VideoView",format:"mp4",width:"",height:"",autoplay:!0,loop:!0,controls:!0,value:new DataView(new ArrayBuffer(0))})},e.serializers=rt({},m.serializers,{value:{serialize:function(t){return new DataView(t.buffer.slice(0))}}}),e}(m),ot=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return nt(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.pWidget.addClass("jupyter-widgets"),this.pWidget.addClass("widget-image"),this.update()},e.prototype.update=function(){var e,n=this.model.get("format"),r=this.model.get("value");if("url"!==n){var i=new Blob([r],{type:"video/"+this.model.get("format")});e=URL.createObjectURL(i)}else e=new TextDecoder("utf-8").decode(r.buffer);var o=this.el.src;this.el.src=e,o&&"string"!=typeof o&&URL.revokeObjectURL(o);var a=this.model.get("width");void 0!==a&&a.length>0?this.el.setAttribute("width",a):this.el.removeAttribute("width");var s=this.model.get("height");return void 0!==s&&s.length>0?this.el.setAttribute("height",s):this.el.removeAttribute("height"),this.el.loop=this.model.get("loop"),this.el.autoplay=this.model.get("autoplay"),this.el.controls=this.model.get("controls"),t.prototype.update.call(this)},e.prototype.remove=function(){this.el.src&&URL.revokeObjectURL(this.el.src),t.prototype.remove.call(this)},Object.defineProperty(e.prototype,"tagName",{get:function(){return"video"},enumerable:!0,configurable:!0}),e}(r.DOMWidgetView),at=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),st=function(){return(st=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},At=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return at(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"AudioModel",_view_name:"AudioView",format:"mp3",autoplay:!0,loop:!0,controls:!0,value:new DataView(new ArrayBuffer(0))})},e.serializers=st({},m.serializers,{value:{serialize:function(t){return new DataView(t.buffer.slice(0))}}}),e}(m),lt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return at(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.pWidget.addClass("jupyter-widgets"),this.update()},e.prototype.update=function(){var e,n=this.model.get("format"),r=this.model.get("value");if("url"!==n){var i=new Blob([r],{type:"audio/"+this.model.get("format")});e=URL.createObjectURL(i)}else e=new TextDecoder("utf-8").decode(r.buffer);var o=this.el.src;return this.el.src=e,o&&"string"!=typeof o&&URL.revokeObjectURL(o),this.el.loop=this.model.get("loop"),this.el.autoplay=this.model.get("autoplay"),this.el.controls=this.model.get("controls"),t.prototype.update.call(this)},e.prototype.remove=function(){this.el.src&&URL.revokeObjectURL(this.el.src),t.prototype.remove.call(this)},Object.defineProperty(e.prototype,"tagName",{get:function(){return"audio"},enumerable:!0,configurable:!0}),e}(r.DOMWidgetView),ct=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),ut=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return ct(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{value:"black",concise:!1,_model_name:"ColorPickerModel",_view_name:"ColorPickerView"})},e}(y),gt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return ct(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-colorpicker"),this._color_container=document.createElement("div"),this._color_container.className="widget-inline-hbox widget-colorpicker-input",this.el.appendChild(this._color_container),this._textbox=document.createElement("input"),this._textbox.setAttribute("type","text"),this._textbox.id=this.label.htmlFor=Object(r.uuid)(),this._color_container.appendChild(this._textbox),this._textbox.value=this.model.get("value"),this._colorpicker=document.createElement("input"),this._colorpicker.setAttribute("type","color"),this._color_container.appendChild(this._colorpicker),this.listenTo(this.model,"change:value",this._update_value),this.listenTo(this.model,"change:concise",this._update_concise),this._update_concise(),this._update_value(),this.update()},e.prototype.update=function(e){if(void 0===e||e.updated_view!=this){var n=this.model.get("disabled");this._textbox.disabled=n,this._colorpicker.disabled=n}return t.prototype.update.call(this)},e.prototype.events=function(){return this._picker_change,this._text_change,{'change [type="color"]':"_picker_change",'change [type="text"]':"_text_change"}},e.prototype._update_value=function(){var t,e,n=this.model.get("value");this._colorpicker.value=ht[(t=n).toLowerCase()]||(7===(e=t).length?e:"#"+e.charAt(1)+e.charAt(1)+e.charAt(2)+e.charAt(2)+e.charAt(3)+e.charAt(3)),this._textbox.value=n},e.prototype._update_concise=function(){this.model.get("concise")?(this.el.classList.add("concise"),this._textbox.style.display="none"):(this.el.classList.remove("concise"),this._textbox.style.display="")},e.prototype._picker_change=function(){this.model.set("value",this._colorpicker.value),this.touch()},e.prototype._text_change=function(){var t=this._validate_color(this._textbox.value,this.model.get("value"));this.model.set("value",t),this.touch()},e.prototype._validate_color=function(t,e){return t.match(/#[a-fA-F0-9]{3}(?:[a-fA-F0-9]{3})?$/)||ht[t.toLowerCase()]?t:e},e}(h),ht={aliceblue:"#f0f8ff",antiquewhite:"#faebd7",aqua:"#00ffff",aquamarine:"#7fffd4",azure:"#f0ffff",beige:"#f5f5dc",bisque:"#ffe4c4",black:"#000000",blanchedalmond:"#ffebcd",blue:"#0000ff",blueviolet:"#8a2be2",brown:"#a52a2a",burlywood:"#deb887",cadetblue:"#5f9ea0",chartreuse:"#7fff00",chocolate:"#d2691e",coral:"#ff7f50",cornflowerblue:"#6495ed",cornsilk:"#fff8dc",crimson:"#dc143c",cyan:"#00ffff",darkblue:"#00008b",darkcyan:"#008b8b",darkgoldenrod:"#b8860b",darkgray:"#a9a9a9",darkgrey:"#a9a9a9",darkgreen:"#006400",darkkhaki:"#bdb76b",darkmagenta:"#8b008b",darkolivegreen:"#556b2f",darkorange:"#ff8c00",darkorchid:"#9932cc",darkred:"#8b0000",darksalmon:"#e9967a",darkseagreen:"#8fbc8f",darkslateblue:"#483d8b",darkslategray:"#2f4f4f",darkslategrey:"#2f4f4f",darkturquoise:"#00ced1",darkviolet:"#9400d3",deeppink:"#ff1493",deepskyblue:"#00bfff",dimgray:"#696969",dimgrey:"#696969",dodgerblue:"#1e90ff",firebrick:"#b22222",floralwhite:"#fffaf0",forestgreen:"#228b22",fuchsia:"#ff00ff",gainsboro:"#dcdcdc",ghostwhite:"#f8f8ff",gold:"#ffd700",goldenrod:"#daa520",gray:"#808080",grey:"#808080",green:"#008000",greenyellow:"#adff2f",honeydew:"#f0fff0",hotpink:"#ff69b4",indianred:"#cd5c5c",indigo:"#4b0082",ivory:"#fffff0",khaki:"#f0e68c",lavender:"#e6e6fa",lavenderblush:"#fff0f5",lawngreen:"#7cfc00",lemonchiffon:"#fffacd",lightblue:"#add8e6",lightcoral:"#f08080",lightcyan:"#e0ffff",lightgoldenrodyellow:"#fafad2",lightgreen:"#90ee90",lightgray:"#d3d3d3",lightgrey:"#d3d3d3",lightpink:"#ffb6c1",lightsalmon:"#ffa07a",lightseagreen:"#20b2aa",lightskyblue:"#87cefa",lightslategray:"#778899",lightslategrey:"#778899",lightsteelblue:"#b0c4de",lightyellow:"#ffffe0",lime:"#00ff00",limegreen:"#32cd32",linen:"#faf0e6",magenta:"#ff00ff",maroon:"#800000",mediumaquamarine:"#66cdaa",mediumblue:"#0000cd",mediumorchid:"#ba55d3",mediumpurple:"#9370db",mediumseagreen:"#3cb371",mediumslateblue:"#7b68ee",mediumspringgreen:"#00fa9a",mediumturquoise:"#48d1cc",mediumvioletred:"#c71585",midnightblue:"#191970",mintcream:"#f5fffa",mistyrose:"#ffe4e1",moccasin:"#ffe4b5",navajowhite:"#ffdead",navy:"#000080",oldlace:"#fdf5e6",olive:"#808000",olivedrab:"#6b8e23",orange:"#ffa500",orangered:"#ff4500",orchid:"#da70d6",palegoldenrod:"#eee8aa",palegreen:"#98fb98",paleturquoise:"#afeeee",palevioletred:"#db7093",papayawhip:"#ffefd5",peachpuff:"#ffdab9",peru:"#cd853f",pink:"#ffc0cb",plum:"#dda0dd",powderblue:"#b0e0e6",purple:"#800080",red:"#ff0000",rosybrown:"#bc8f8f",royalblue:"#4169e1",saddlebrown:"#8b4513",salmon:"#fa8072",sandybrown:"#f4a460",seagreen:"#2e8b57",seashell:"#fff5ee",sienna:"#a0522d",silver:"#c0c0c0",skyblue:"#87ceeb",slateblue:"#6a5acd",slategray:"#708090",slategrey:"#708090",snow:"#fffafa",springgreen:"#00ff7f",steelblue:"#4682b4",tan:"#d2b48c",teal:"#008080",thistle:"#d8bfd8",tomato:"#ff6347",turquoise:"#40e0d0",violet:"#ee82ee",wheat:"#f5deb3",white:"#ffffff",whitesmoke:"#f5f5f5",yellow:"#ffff00",yellowgreen:"#9acd32"};var dt=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),ft=function(){return(ft=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)};function pt(t){return null===t?null:{year:t.getUTCFullYear(),month:t.getUTCMonth(),date:t.getUTCDate()}}function It(t){if(null===t)return null;var e=new Date;return e.setUTCFullYear(t.year,t.month,t.date),e.setUTCHours(0,0,0,0),e}var Ct=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return dt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{value:null,_model_name:"DatePickerModel",_view_name:"DatePickerView"})},e.serializers=ft({},y.serializers,{value:{serialize:pt,deserialize:It}}),e}(y),mt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return dt(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-datepicker"),this._datepicker=document.createElement("input"),this._datepicker.setAttribute("type","date"),this._datepicker.id=this.label.htmlFor=Object(r.uuid)(),this.el.appendChild(this._datepicker),this.listenTo(this.model,"change:value",this._update_value),this._update_value(),this.update()},e.prototype.update=function(e){return void 0!==e&&e.updated_view===this||(this._datepicker.disabled=this.model.get("disabled")),t.prototype.update.call(this)},e.prototype.events=function(){return this._picker_change,this._picker_focusout,{'change [type="date"]':"_picker_change",'focusout [type="date"]':"_picker_focusout"}},e.prototype._update_value=function(){var t=this.model.get("value");this._datepicker.valueAsDate=t},e.prototype._picker_change=function(){this._datepicker.validity.badInput||(this.model.set("value",this._datepicker.valueAsDate),this.touch())},e.prototype._picker_focusout=function(){this._datepicker.validity.badInput&&(this.model.set("value",null),this.touch())},e}(h),yt=function(t,e){if((n=(t=e?t.toExponential(e-1):t.toExponential()).indexOf("e"))<0)return null;var n,r=t.slice(0,n);return[r.length>1?r[0]+r.slice(2):r,+t.slice(n+1)]},Et=function(t){return(t=yt(Math.abs(t)))?t[1]:NaN},vt=/^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i;function Bt(t){return new bt(t)}function bt(t){if(!(e=vt.exec(t)))throw new Error("invalid format: "+t);var e;this.fill=e[1]||" ",this.align=e[2]||">",this.sign=e[3]||"-",this.symbol=e[4]||"",this.zero=!!e[5],this.width=e[6]&&+e[6],this.comma=!!e[7],this.precision=e[8]&&+e[8].slice(1),this.trim=!!e[9],this.type=e[10]||""}Bt.prototype=bt.prototype,bt.prototype.toString=function(){return this.fill+this.align+this.sign+this.symbol+(this.zero?"0":"")+(null==this.width?"":Math.max(1,0|this.width))+(this.comma?",":"")+(null==this.precision?"":"."+Math.max(0,0|this.precision))+(this.trim?"~":"")+this.type};var Qt,wt,xt,_t=function(t){t:for(var e,n=t.length,r=1,i=-1;r<n;++r)switch(t[r]){case".":i=e=r;break;case"0":0===i&&(i=r),e=r;break;default:if(i>0){if(!+t[r])break t;i=0}}return i>0?t.slice(0,i)+t.slice(e+1):t},Dt=function(t,e){var n=yt(t,e);if(!n)return t+"";var r=n[0],i=n[1];return i<0?"0."+new Array(-i).join("0")+r:r.length>i+1?r.slice(0,i+1)+"."+r.slice(i+1):r+new Array(i-r.length+2).join("0")},Mt={"%":function(t,e){return(100*t).toFixed(e)},b:function(t){return Math.round(t).toString(2)},c:function(t){return t+""},d:function(t){return Math.round(t).toString(10)},e:function(t,e){return t.toExponential(e)},f:function(t,e){return t.toFixed(e)},g:function(t,e){return t.toPrecision(e)},o:function(t){return Math.round(t).toString(8)},p:function(t,e){return Dt(100*t,e)},r:Dt,s:function(t,e){var n=yt(t,e);if(!n)return t+"";var r=n[0],i=n[1],o=i-(Qt=3*Math.max(-8,Math.min(8,Math.floor(i/3))))+1,a=r.length;return o===a?r:o>a?r+new Array(o-a+1).join("0"):o>0?r.slice(0,o)+"."+r.slice(o):"0."+new Array(1-o).join("0")+yt(t,Math.max(0,e+o-1))[0]},X:function(t){return Math.round(t).toString(16).toUpperCase()},x:function(t){return Math.round(t).toString(16)}},kt=function(t){return t},St=["y","z","a","f","p","n","µ","m","","k","M","G","T","P","E","Z","Y"];wt=function(t){var e,n,r=t.grouping&&t.thousands?(e=t.grouping,n=t.thousands,function(t,r){for(var i=t.length,o=[],a=0,s=e[0],A=0;i>0&&s>0&&(A+s+1>r&&(s=Math.max(1,r-A)),o.push(t.substring(i-=s,i+s)),!((A+=s+1)>r));)s=e[a=(a+1)%e.length];return o.reverse().join(n)}):kt,i=t.currency,o=t.decimal,a=t.numerals?function(t){return function(e){return e.replace(/[0-9]/g,function(e){return t[+e]})}}(t.numerals):kt,s=t.percent||"%";function A(t){var e=(t=Bt(t)).fill,n=t.align,A=t.sign,l=t.symbol,c=t.zero,u=t.width,g=t.comma,h=t.precision,d=t.trim,f=t.type;"n"===f?(g=!0,f="g"):Mt[f]||(null==h&&(h=12),d=!0,f="g"),(c||"0"===e&&"="===n)&&(c=!0,e="0",n="=");var p="$"===l?i[0]:"#"===l&&/[boxX]/.test(f)?"0"+f.toLowerCase():"",I="$"===l?i[1]:/[%p]/.test(f)?s:"",C=Mt[f],m=/[defgprs%]/.test(f);function y(t){var i,s,l,y=p,E=I;if("c"===f)E=C(t)+E,t="";else{var v=(t=+t)<0;if(t=C(Math.abs(t),h),d&&(t=_t(t)),v&&0==+t&&(v=!1),y=(v?"("===A?A:"-":"-"===A||"("===A?"":A)+y,E=("s"===f?St[8+Qt/3]:"")+E+(v&&"("===A?")":""),m)for(i=-1,s=t.length;++i<s;)if(48>(l=t.charCodeAt(i))||l>57){E=(46===l?o+t.slice(i+1):t.slice(i))+E,t=t.slice(0,i);break}}g&&!c&&(t=r(t,1/0));var B=y.length+t.length+E.length,b=B<u?new Array(u-B+1).join(e):"";switch(g&&c&&(t=r(b+t,b.length?u-E.length:1/0),b=""),n){case"<":t=y+t+E+b;break;case"=":t=y+b+t+E;break;case"^":t=b.slice(0,B=b.length>>1)+y+t+E+b.slice(B);break;default:t=b+y+t+E}return a(t)}return h=null==h?6:/[gprs]/.test(f)?Math.max(1,Math.min(21,h)):Math.max(0,Math.min(20,h)),y.toString=function(){return t+""},y}return{format:A,formatPrefix:function(t,e){var n=A(((t=Bt(t)).type="f",t)),r=3*Math.max(-8,Math.min(8,Math.floor(Et(e)/3))),i=Math.pow(10,-r),o=St[8+r/3];return function(t){return n(i*t)+o}}}}({decimal:".",thousands:",",grouping:[3],currency:["$",""]}),xt=wt.format,wt.formatPrefix;n(1410);var Lt=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Rt=function(){return(Rt=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},Nt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"IntModel",value:0})},e}(y),Ft=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"BoundedIntModel",max:100,min:0})},e}(Nt),Tt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e.prototype.defaults=function(){return Rt({},t.prototype.defaults.call(this),{_model_name:"SliderStyleModel"})},e.styleProperties=Rt({},u.styleProperties,{handle_color:{selector:".ui-slider-handle",attribute:"background-color",default:null}}),e}(u),Ut=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"IntSliderModel",_view_name:"IntSliderView",step:1,orientation:"horizontal",readout:!0,readout_format:"d",continuous_update:!0,style:null,disabled:!1})},e.prototype.initialize=function(e,n){t.prototype.initialize.call(this,e,n),this.on("change:readout_format",this.update_readout_format,this),this.update_readout_format()},e.prototype.update_readout_format=function(){this.readout_formatter=xt(this.get("readout_format"))},e}(Ft),Ot=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e}(Ut),Gt=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._parse_value=parseInt,e}return Lt(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-slider"),this.el.classList.add("widget-hslider"),(this.$slider=H()("<div />")).slider({slide:this.handleSliderChange.bind(this),stop:this.handleSliderChanged.bind(this)}).addClass("slider"),this.slider_container=document.createElement("div"),this.slider_container.classList.add("slider-container"),this.slider_container.appendChild(this.$slider[0]),this.el.appendChild(this.slider_container),this.readout=document.createElement("div"),this.el.appendChild(this.readout),this.readout.classList.add("widget-readout"),this.readout.contentEditable="true",this.readout.style.display="none",this.update()},e.prototype.update=function(e){if(void 0===e||e.updated_view!==this){var n=this;n.$slider.slider({}),["step","disabled"].forEach(function(t){var e=n.model.get(t);void 0!==e&&n.$slider.slider("option",t,e)}),this.model.get("disabled")?this.readout.contentEditable="false":this.readout.contentEditable="true";var r=this.model.get("max"),i=this.model.get("min");i<=r&&(void 0!==r&&this.$slider.slider("option","max",r),void 0!==i&&this.$slider.slider("option","min",i));var o=this.model.get("orientation");this.$slider.slider("option","orientation",o),"vertical"===o?(this.el.classList.remove("widget-hslider"),this.el.classList.add("widget-vslider"),this.el.classList.remove("widget-inline-hbox"),this.el.classList.add("widget-inline-vbox")):(this.el.classList.remove("widget-vslider"),this.el.classList.add("widget-hslider"),this.el.classList.remove("widget-inline-vbox"),this.el.classList.add("widget-inline-hbox")),this.model.get("readout")?(this.readout.style.display="",this.displayed.then(function(){n.readout_overflow()?n.readout.classList.add("overflow"):n.readout.classList.remove("overflow")})):this.readout.style.display="none"}return t.prototype.update.call(this)},e.prototype.readout_overflow=function(){return this.readout.scrollWidth>this.readout.clientWidth},e.prototype.events=function(){return{slide:"handleSliderChange",slidestop:"handleSliderChanged","blur [contentEditable=true]":"handleTextChange","keydown [contentEditable=true]":"handleKeyDown"}},e.prototype.handleKeyDown=function(t){13===t.keyCode&&(t.preventDefault(),this.handleTextChange())},e.prototype._validate_slide_value=function(t){return Math.floor(t)},e}(h),Ht=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._range_regex=/^\s*([+-]?\d+)\s*[-:–]\s*([+-]?\d+)/,e}return Lt(e,t),e.prototype.update=function(e){t.prototype.update.call(this,e),this.$slider.slider("option","range",!0);var n=this.model.get("value");this.$slider.slider("option","values",n.slice()),this.readout.textContent=this.valueToString(n),this.model.get("value")!==n&&(this.model.set("value",n,{updated_view:this}),this.touch())},e.prototype.valueToString=function(t){var e=this.model.readout_formatter;return t.map(function(t){return e(t)}).join(" – ")},e.prototype.stringToValue=function(t){var e=this._range_regex.exec(t);return e?[this._parse_value(e[1]),this._parse_value(e[2])]:null},e.prototype.handleTextChange=function(){var t=this.stringToValue(this.readout.textContent),e=this.model.get("min"),n=this.model.get("max");null===t||isNaN(t[0])||isNaN(t[1])||t[0]>t[1]?this.readout.textContent=this.valueToString(this.model.get("value")):(t=[Math.max(Math.min(t[0],n),e),Math.max(Math.min(t[1],n),e)])[0]!==this.model.get("value")[0]||t[1]!==this.model.get("value")[1]?(this.readout.textContent=this.valueToString(t),this.model.set("value",t,{updated_view:this}),this.touch()):this.readout.textContent=this.valueToString(this.model.get("value"))},e.prototype.handleSliderChange=function(t,e){var n=e.values.map(this._validate_slide_value);this.readout.textContent=this.valueToString(n),this.model.get("continuous_update")&&this.handleSliderChanged(t,e)},e.prototype.handleSliderChanged=function(t,e){var n=e.values.map(this._validate_slide_value);this.model.set("value",n,{updated_view:this}),this.touch()},e}(Gt),jt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e.prototype.update=function(e){t.prototype.update.call(this,e);var n=this.model.get("min"),r=this.model.get("max"),i=this.model.get("value");i>r?i=r:i<n&&(i=n),this.$slider.slider("option","value",i),this.readout.textContent=this.valueToString(i),this.model.get("value")!==i&&(this.model.set("value",i,{updated_view:this}),this.touch())},e.prototype.valueToString=function(t){return(0,this.model.readout_formatter)(t)},e.prototype.stringToValue=function(t){return this._parse_value(t)},e.prototype.handleTextChange=function(){var t=this.stringToValue(this.readout.textContent),e=this.model.get("min"),n=this.model.get("max");isNaN(t)?this.readout.textContent=this.valueToString(this.model.get("value")):(t=Math.max(Math.min(t,n),e))!==this.model.get("value")?(this.readout.textContent=this.valueToString(t),this.model.set("value",t,{updated_view:this}),this.touch()):this.readout.textContent=this.valueToString(this.model.get("value"))},e.prototype.handleSliderChange=function(t,e){var n=this._validate_slide_value(e.value);this.readout.textContent=this.valueToString(n),this.model.get("continuous_update")&&this.handleSliderChanged(t,e)},e.prototype.handleSliderChanged=function(t,e){var n=this._validate_slide_value(e.value);this.model.set("value",n,{updated_view:this}),this.touch()},e}(Gt),Yt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"IntTextModel",_view_name:"IntTextView",disabled:!1,continuous_update:!1})},e}(Nt),Pt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"BoundedIntTextModel",_view_name:"IntTextView",disabled:!1,continuous_update:!1,step:1})},e}(Ft),Kt=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._parse_value=parseInt,e._default_step="1",e}return Lt(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-text"),this.textbox=document.createElement("input"),this.textbox.type="number",this.textbox.required=!0,this.textbox.id=this.label.htmlFor=Object(r.uuid)(),this.el.appendChild(this.textbox),this.update()},e.prototype.update=function(e){if(void 0===e||e.updated_view!==this){var n=this.model.get("value");this._parse_value(this.textbox.value)!==n&&(this.textbox.value=n.toString()),void 0!==this.model.get("min")&&(this.textbox.min=this.model.get("min")),void 0!==this.model.get("max")&&(this.textbox.max=this.model.get("max")),void 0!==this.model.get("step")&&null!==this.model.get("step")?this.textbox.step=this.model.get("step"):this.textbox.step=this._default_step,this.textbox.disabled=this.model.get("disabled")}return t.prototype.update.call(this)},e.prototype.events=function(){return{"keydown input":"handleKeyDown","keypress input":"handleKeypress","keyup input":"handleKeyUp","input input":"handleChanging","change input":"handleChanged"}},e.prototype.handleKeyDown=function(t){t.stopPropagation()},e.prototype.handleKeypress=function(t){/[e,.\s]/.test(String.fromCharCode(t.keyCode))&&t.preventDefault()},e.prototype.handleKeyUp=function(t){if(!t.altKey&&!t.ctrlKey){var e=t.target,n=e.value;if((n=n.replace(/[e,.\s]/g,"")).length>=1){var r=n.substr(1);n=n[0]+r.replace(/[+-]/g,"")}e.value!=n&&(t.preventDefault(),e.value=n)}},e.prototype.handleChanging=function(t){var e=t.target.value.trim();""===e||["-","-.",".","+.","+"].indexOf(e)>=0||this.model.get("continuous_update")&&this.handleChanged(t)},e.prototype.handleChanged=function(t){var e=t.target,n=this._parse_value(e.value);if(isNaN(n))e.value=this.model.get("value");else{var r=n;void 0!==this.model.get("max")&&(r=Math.min(this.model.get("max"),r)),void 0!==this.model.get("min")&&(r=Math.max(this.model.get("min"),r)),r!==n&&(e.value=r,n=r),n!==this.model.get("value")&&(this.model.set("value",n,{updated_view:this}),this.touch())}},e}(h),zt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e.prototype.defaults=function(){return Rt({},t.prototype.defaults.call(this),{_model_name:"ProgressStyleModel"})},e.styleProperties=Rt({},u.styleProperties,{bar_color:{selector:".progress-bar",attribute:"background-color",default:null}}),e}(u),Jt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"IntProgressModel",_view_name:"ProgressView",orientation:"horizontal",bar_style:"",style:null})},e}(Ft),qt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e.prototype.initialize=function(e){t.prototype.initialize.call(this,e),this.listenTo(this.model,"change:bar_style",this.update_bar_style),this.pWidget.addClass("jupyter-widgets")},e.prototype.render=function(){t.prototype.render.call(this);var e="horizontal"===this.model.get("orientation")?"widget-hprogress":"widget-vprogress";this.el.classList.add(e),this.progress=document.createElement("div"),this.progress.classList.add("progress"),this.progress.style.position="relative",this.el.appendChild(this.progress),this.bar=document.createElement("div"),this.bar.classList.add("progress-bar"),this.bar.style.position="absolute",this.bar.style.bottom="0px",this.bar.style.left="0px",this.progress.appendChild(this.bar),this.update(),this.set_bar_style()},e.prototype.update=function(){var e=this.model.get("value"),n=this.model.get("max"),r=this.model.get("min"),i=100*(e-r)/(n-r);return"horizontal"===this.model.get("orientation")?(this.el.classList.remove("widget-inline-vbox"),this.el.classList.remove("widget-vprogress"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-hprogress"),this.bar.style.width=i+"%",this.bar.style.height="100%"):(this.el.classList.remove("widget-inline-hbox"),this.el.classList.remove("widget-hprogress"),this.el.classList.add("widget-inline-vbox"),this.el.classList.add("widget-vprogress"),this.bar.style.width="100%",this.bar.style.height=i+"%"),t.prototype.update.call(this)},e.prototype.update_bar_style=function(){this.update_mapped_classes(e.class_map,"bar_style",this.bar)},e.prototype.set_bar_style=function(){this.set_mapped_classes(e.class_map,"bar_style",this.bar)},e.class_map={success:["progress-bar-success"],info:["progress-bar-info"],warning:["progress-bar-warning"],danger:["progress-bar-danger"]},e}(h),Vt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"PlayModel",_view_name:"PlayView",_playing:!1,_repeat:!1,show_repeat:!0,interval:100,step:1,disabled:!1})},e.prototype.initialize=function(e,n){t.prototype.initialize.call(this,e,n)},e.prototype.loop=function(){if(this.get("_playing")){var t=this.get("value")+this.get("step");t<=this.get("max")?(this.set("value",t),this.schedule_next()):this.get("_repeat")?(this.set("value",this.get("min")),this.schedule_next()):this.set("_playing",!1),this.save_changes()}},e.prototype.schedule_next=function(){window.setTimeout(this.loop.bind(this),this.get("interval"))},e.prototype.stop=function(){this.set("_playing",!1),this.set("value",this.get("min")),this.save_changes()},e.prototype.pause=function(){this.set("_playing",!1),this.save_changes()},e.prototype.play=function(){this.set("_playing",!0),this.get("value")==this.get("max")?(this.set("value",this.get("min")),this.schedule_next(),this.save_changes()):this.loop()},e.prototype.repeat=function(){this.set("_repeat",!this.get("_repeat")),this.save_changes()},e}(Ft),Wt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Lt(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-play"),this.playButton=document.createElement("button"),this.pauseButton=document.createElement("button"),this.stopButton=document.createElement("button"),this.repeatButton=document.createElement("button"),this.playButton.className="jupyter-button",this.pauseButton.className="jupyter-button",this.stopButton.className="jupyter-button",this.repeatButton.className="jupyter-button",this.el.appendChild(this.playButton),this.el.appendChild(this.pauseButton),this.el.appendChild(this.stopButton),this.el.appendChild(this.repeatButton);var e=document.createElement("i");e.className="fa fa-play",this.playButton.appendChild(e);var n=document.createElement("i");n.className="fa fa-pause",this.pauseButton.appendChild(n);var r=document.createElement("i");r.className="fa fa-stop",this.stopButton.appendChild(r);var i=document.createElement("i");i.className="fa fa-retweet",this.repeatButton.appendChild(i),this.playButton.onclick=this.model.play.bind(this.model),this.pauseButton.onclick=this.model.pause.bind(this.model),this.stopButton.onclick=this.model.stop.bind(this.model),this.repeatButton.onclick=this.model.repeat.bind(this.model),this.listenTo(this.model,"change:_playing",this.update_playing),this.listenTo(this.model,"change:_repeat",this.update_repeat),this.listenTo(this.model,"change:show_repeat",this.update_repeat),this.update_playing(),this.update_repeat(),this.update()},e.prototype.update=function(){var t=this.model.get("disabled");this.playButton.disabled=t,this.pauseButton.disabled=t,this.stopButton.disabled=t,this.repeatButton.disabled=t,this.update_playing()},e.prototype.update_playing=function(){var t=this.model.get("_playing"),e=this.model.get("disabled");t?(e||(this.pauseButton.disabled=!1),this.playButton.classList.add("mod-active")):(e||(this.pauseButton.disabled=!0),this.playButton.classList.remove("mod-active"))},e.prototype.update_repeat=function(){var t=this.model.get("_repeat");this.repeatButton.style.display=this.model.get("show_repeat")?this.playButton.style.display:"none",t?this.repeatButton.classList.add("mod-active"):this.repeatButton.classList.remove("mod-active")},e}(r.DOMWidgetView),Xt=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Zt=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Xt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"FloatModel",value:0})},e}(y),$t=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Xt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"BoundedFloatModel",max:100,min:0})},e}(Zt),te=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Xt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"FloatSliderModel",_view_name:"FloatSliderView",step:1,orientation:"horizontal",_range:!1,readout:!0,readout_format:".2f",slider_color:null,continuous_update:!0,disabled:!1})},e.prototype.initialize=function(e,n){t.prototype.initialize.call(this,e,n),this.on("change:readout_format",this.update_readout_format,this),this.update_readout_format()},e.prototype.update_readout_format=function(){this.readout_formatter=xt(this.get("readout_format"))},e}($t),ee=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Xt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"FloatLogSliderModel",_view_name:"FloatLogSliderView",step:.1,orientation:"horizontal",_range:!1,readout:!0,readout_format:".3g",slider_color:null,continuous_update:!0,disabled:!1,base:10,value:1,min:0,max:4})},e.prototype.initialize=function(e,n){t.prototype.initialize.call(this,e,n),this.on("change:readout_format",this.update_readout_format,this),this.update_readout_format()},e.prototype.update_readout_format=function(){this.readout_formatter=xt(this.get("readout_format"))},e}($t),ne=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Xt(e,t),e}(te),re=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._parse_value=parseFloat,e}return Xt(e,t),e.prototype._validate_slide_value=function(t){return t},e}(jt),ie=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._parse_value=parseFloat,e}return Xt(e,t),e.prototype.update=function(e){t.prototype.update.call(this,e);var n=this.model.get("min"),r=this.model.get("max"),i=this.model.get("value"),o=this.model.get("base"),a=Math.log(i)/Math.log(o);a>r?a=r:a<n&&(a=n),this.$slider.slider("option","value",a),this.readout.textContent=this.valueToString(i),this.model.get("value")!==i&&(this.model.set("value",i,{updated_view:this}),this.touch())},e.prototype.valueToString=function(t){return(0,this.model.readout_formatter)(t)},e.prototype.stringToValue=function(t){return this._parse_value(t)},e.prototype.handleTextChange=function(){var t=this.stringToValue(this.readout.textContent),e=this.model.get("min"),n=this.model.get("max"),r=this.model.get("base");isNaN(t)?this.readout.textContent=this.valueToString(this.model.get("value")):(t=Math.max(Math.min(t,Math.pow(r,n)),Math.pow(r,e)))!==this.model.get("value")?(this.readout.textContent=this.valueToString(t),this.model.set("value",t,{updated_view:this}),this.touch()):this.readout.textContent=this.valueToString(this.model.get("value"))},e.prototype.handleSliderChange=function(t,e){var n=this.model.get("base"),r=Math.pow(n,this._validate_slide_value(e.value));this.readout.textContent=this.valueToString(r),this.model.get("continuous_update")&&this.handleSliderChanged(t,e)},e.prototype.handleSliderChanged=function(t,e){var n=this.model.get("base"),r=Math.pow(n,this._validate_slide_value(e.value));this.model.set("value",r,{updated_view:this}),this.touch()},e.prototype._validate_slide_value=function(t){return t},e}(Gt),oe=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._parse_value=parseFloat,e._range_regex=/^\s*([+-]?(?:\d*\.?\d+|\d+\.)(?:[eE][-:]?\d+)?)\s*[-:–]\s*([+-]?(?:\d*\.?\d+|\d+\.)(?:[eE][+-]?\d+)?)/,e}return Xt(e,t),e.prototype._validate_slide_value=function(t){return t},e}(Ht),ae=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Xt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"FloatTextModel",_view_name:"FloatTextView",disabled:!1,continuous_update:!1})},e}(Zt),se=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Xt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"BoundedFloatTextModel",_view_name:"FloatTextView",disabled:!1,continuous_update:!1,step:.1})},e}($t),Ae=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._parse_value=parseFloat,e._default_step="any",e}return Xt(e,t),e.prototype.handleKeypress=function(t){t.stopPropagation()},e.prototype.handleKeyUp=function(t){},e}(Kt),le=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Xt(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"FloatProgressModel",_view_name:"ProgressView",orientation:"horizontal",bar_style:"",style:null})},e}($t),ce=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),ue=function(){return(ue=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},ge=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return ce(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"ControllerButtonModel",_view_name:"ControllerButtonView",value:0,pressed:!1})},e}(m),he=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return ce(e,t),e.prototype.render=function(){this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-controller-button"),this.el.style.width="fit-content",this.support=document.createElement("div"),this.support.style.position="relative",this.support.style.margin="1px",this.support.style.width="16px",this.support.style.height="16px",this.support.style.border="1px solid black",this.support.style.background="lightgray",this.el.appendChild(this.support),this.bar=document.createElement("div"),this.bar.style.position="absolute",this.bar.style.width="100%",this.bar.style.bottom="0px",this.bar.style.background="gray",this.support.appendChild(this.bar),this.update(),this.label=document.createElement("div"),this.label.textContent=this.model.get("description"),this.label.style.textAlign="center",this.el.appendChild(this.label)},e.prototype.update=function(){this.bar.style.height=100*this.model.get("value")+"%"},e}(r.DOMWidgetView),de=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return ce(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"ControllerAxisModel",_view_name:"ControllerAxisView",value:0})},e}(m),fe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return ce(e,t),e.prototype.render=function(){this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-controller-axis"),this.el.style.width="16px",this.el.style.padding="4px",this.support=document.createElement("div"),this.support.style.position="relative",this.support.style.margin="1px",this.support.style.width="4px",this.support.style.height="64px",this.support.style.border="1px solid black",this.support.style.background="lightgray",this.bullet=document.createElement("div"),this.bullet.style.position="absolute",this.bullet.style.margin="-3px",this.bullet.style.boxSizing="unset",this.bullet.style.width="10px",this.bullet.style.height="10px",this.bullet.style.background="gray",this.label=document.createElement("div"),this.label.textContent=this.model.get("description"),this.label.style.textAlign="center",this.support.appendChild(this.bullet),this.el.appendChild(this.support),this.el.appendChild(this.label),this.update()},e.prototype.update=function(){this.bullet.style.top=50*(this.model.get("value")+1)+"%"},e}(r.DOMWidgetView),pe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return ce(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"ControllerModel",_view_name:"ControllerView",index:0,name:"",mapping:"",connected:!1,timestamp:0,buttons:[],axes:[]})},e.prototype.initialize=function(e,n){t.prototype.initialize.call(this,e,n),void 0===navigator.getGamepads?(this.readout="This browser does not support gamepads.",console.error(this.readout)):(this.readout="Connect gamepad and press any button.",this.get("connected")?this.update_loop():this.wait_loop())},e.prototype.wait_loop=function(){var t=this.get("index"),e=navigator.getGamepads()[t];if(e){var n=this;this.setup(e).then(function(t){n.set(t),n.save_changes(),window.requestAnimationFrame(n.update_loop.bind(n))})}else window.requestAnimationFrame(this.wait_loop.bind(this))},e.prototype.setup=function(t){this.set({name:t.id,mapping:t.mapping,connected:t.connected,timestamp:t.timestamp});var e=this;return r.resolvePromisesDict({buttons:Promise.all(t.buttons.map(function(t,n){return e._create_button_model(n)})),axes:Promise.all(t.axes.map(function(t,n){return e._create_axis_model(n)}))})},e.prototype.update_loop=function(){var t=this.get("index"),e=this.get("name"),n=navigator.getGamepads()[t];n&&t===n.index&&e===n.id?(this.set({timestamp:n.timestamp,connected:n.connected}),this.save_changes(),this.get("buttons").forEach(function(t,e){t.set({value:n.buttons[e].value,pressed:n.buttons[e].pressed}),t.save_changes()}),this.get("axes").forEach(function(t,e){t.set("value",n.axes[e]),t.save_changes()}),window.requestAnimationFrame(this.update_loop.bind(this))):this.reset_gamepad()},e.prototype.reset_gamepad=function(){this.get("buttons").forEach(function(t){t.close()}),this.get("axes").forEach(function(t){t.close()}),this.set({name:"",mapping:"",connected:!1,timestamp:0,buttons:[],axes:[]}),this.save_changes(),window.requestAnimationFrame(this.wait_loop.bind(this))},e.prototype._create_button_model=function(t){return this.widget_manager.new_widget({model_name:"ControllerButtonModel",model_module:"@jupyter-widgets/controls",model_module_version:this.get("_model_module_version"),view_name:"ControllerButtonView",view_module:"@jupyter-widgets/controls",view_module_version:this.get("_view_module_version")}).then(function(e){return e.set("description",t),e})},e.prototype._create_axis_model=function(t){return this.widget_manager.new_widget({model_name:"ControllerAxisModel",model_module:"@jupyter-widgets/controls",model_module_version:this.get("_model_module_version"),view_name:"ControllerAxisView",view_module:"@jupyter-widgets/controls",view_module_version:this.get("_view_module_version")}).then(function(e){return e.set("description",t),e})},e.serializers=ue({},m.serializers,{buttons:{deserialize:r.unpack_models},axes:{deserialize:r.unpack_models}}),e}(m),Ie=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return ce(e,t),e.prototype._createElement=function(t){return this.pWidget=new r.JupyterPhosphorPanelWidget({view:this}),this.pWidget.node},e.prototype._setElement=function(t){if(this.el||t!==this.pWidget.node)throw new Error("Cannot reset the DOM element.");this.el=this.pWidget.node,this.$el=H()(this.pWidget.node)},e.prototype.initialize=function(e){t.prototype.initialize.call(this,e),this.button_views=new r.ViewList(this.add_button,null,this),this.listenTo(this.model,"change:buttons",function(t,e){this.button_views.update(e)}),this.axis_views=new r.ViewList(this.add_axis,null,this),this.listenTo(this.model,"change:axes",function(t,e){this.axis_views.update(e)}),this.listenTo(this.model,"change:name",this.update_label)},e.prototype.render=function(){this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-controller"),this.label=document.createElement("div"),this.el.appendChild(this.label),this.axis_box=new O.Panel,this.axis_box.node.style.display="flex",this.pWidget.addWidget(this.axis_box),this.button_box=new O.Panel,this.button_box.node.style.display="flex",this.pWidget.addWidget(this.button_box),this.button_views.update(this.model.get("buttons")),this.axis_views.update(this.model.get("axes")),this.update_label()},e.prototype.update_label=function(){this.label.textContent=this.model.get("name")||this.model.readout},e.prototype.add_button=function(t){var e=this,n=new O.Widget;return this.button_box.addWidget(n),this.create_child_view(t).then(function(t){var r=T.ArrayExt.firstIndexOf(e.button_box.widgets,n);return e.button_box.insertWidget(r,t.pWidget),n.dispose(),t}).catch(i("Could not add child button view to controller",!0))},e.prototype.add_axis=function(t){var e=this,n=new O.Widget;return this.axis_box.addWidget(n),this.create_child_view(t).then(function(t){var r=T.ArrayExt.firstIndexOf(e.axis_box.widgets,n);return e.axis_box.insertWidget(r,t.pWidget),n.dispose(),t}).catch(i("Could not add child axis view to controller",!0))},e.prototype.remove=function(){t.prototype.remove.call(this),this.button_views.remove(),this.axis_views.remove()},e}(r.DOMWidgetView),Ce=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),me=function(){return(me=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},ye=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.defaults=function(){return me({},t.prototype.defaults.call(this),{_model_name:"SelectionModel",index:"",_options_labels:[],disabled:!1})},e}(y),Ee=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.defaults=function(){return me({},t.prototype.defaults.call(this),{_model_name:"DropdownModel",_view_name:"DropdownView",button_style:""})},e}(ye),ve=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.initialize=function(e){var n=this;t.prototype.initialize.call(this,e),this.listenTo(this.model,"change:_options_labels",function(){return n._updateOptions()})},e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-dropdown"),this.listbox=document.createElement("select"),this.listbox.id=this.label.htmlFor=Object(r.uuid)(),this.el.appendChild(this.listbox),this._updateOptions(),this.update()},e.prototype.update=function(){this.listbox.disabled=this.model.get("disabled");var e=this.model.get("index");return this.listbox.selectedIndex=null===e?-1:e,t.prototype.update.call(this)},e.prototype._updateOptions=function(){this.listbox.textContent="";for(var t=this.model.get("_options_labels"),e=0;e<t.length;e++){var n=t[e],r=document.createElement("option");r.textContent=n.replace(/ /g," "),r.setAttribute("data-value",encodeURIComponent(n)),r.value=n,this.listbox.appendChild(r)}},e.prototype.events=function(){return{"change select":"_handle_change"}},e.prototype._handle_change=function(){this.model.set("index",-1===this.listbox.selectedIndex?null:this.listbox.selectedIndex),this.touch()},e}(h),Be=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.defaults=function(){return me({},t.prototype.defaults.call(this),{_model_name:"SelectModel",_view_name:"SelectView",rows:5})},e}(ye),be=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.initialize=function(e){var n=this;t.prototype.initialize.call(this,e),this.listenTo(this.model,"change:_options_labels",function(){return n._updateOptions()}),this.listenTo(this.model,"change:index",function(t,e,r){return n.updateSelection(r)}),this.listbox=document.createElement("select")},e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-select"),this.listbox.id=this.label.htmlFor=Object(r.uuid)(),this.el.appendChild(this.listbox),this._updateOptions(),this.update(),this.updateSelection()},e.prototype.update=function(){t.prototype.update.call(this),this.listbox.disabled=this.model.get("disabled");var e=this.model.get("rows");null===e&&(e=""),this.listbox.setAttribute("size",e)},e.prototype.updateSelection=function(t){if(void 0===t&&(t={}),t.updated_view!==this){var e=this.model.get("index");this.listbox.selectedIndex=null===e?-1:e}},e.prototype._updateOptions=function(){this.listbox.textContent="";for(var t=this.model.get("_options_labels"),e=0;e<t.length;e++){var n=t[e],r=document.createElement("option");r.textContent=n.replace(/ /g," "),r.setAttribute("data-value",encodeURIComponent(n)),r.value=n,this.listbox.appendChild(r)}},e.prototype.events=function(){return{"change select":"_handle_change"}},e.prototype._handle_change=function(){this.model.set("index",this.listbox.selectedIndex,{updated_view:this}),this.touch()},e}(h),Qe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.defaults=function(){return me({},t.prototype.defaults.call(this),{_model_name:"RadioButtonsModel",_view_name:"RadioButtonsView",tooltips:[],icons:[],button_style:""})},e}(ye),we=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-radio"),this.container=document.createElement("div"),this.el.appendChild(this.container),this.container.classList.add("widget-radio-box"),this.update()},e.prototype.update=function(e){var n=this,r=this.model.get("_options_labels"),i=p.pluck(this.container.querySelectorAll('input[type="radio"]'),"value"),o=r.length!=i.length;if(!o)for(var a=0,s=r.length;a<s;++a)if(i[a]!==r[a]){o=!0;break}return!o||void 0!==e&&e.updated_view===this||(this.container.textContent="",r.forEach(function(t,e){var r=document.createElement("label");r.textContent=t,n.container.appendChild(r);var i=document.createElement("input");i.setAttribute("type","radio"),i.value=e.toString(),i.setAttribute("data-value",encodeURIComponent(t)),r.appendChild(i)})),r.forEach(function(t,e){var r='input[data-value="'+encodeURIComponent(t)+'"]',i=n.container.querySelectorAll(r);if(i.length>0){var o=i[0];o.checked=n.model.get("index")===e,o.disabled=n.model.get("disabled")}}),setTimeout(this.adjustPadding,0,this),t.prototype.update.call(this,e)},e.prototype.adjustPadding=function(t){var e=window.getComputedStyle(t.el),n=parseInt(e.marginTop,10)+parseInt(e.marginBottom,10),r=t.label.offsetHeight+n,i=window.getComputedStyle(t.container),o=parseInt(i.marginBottom),a=(t.el.offsetHeight+n-o)%r,s=0==a?0:r-a;t.container.style.marginBottom=s+"px"},e.prototype.events=function(){return{'click input[type="radio"]':"_handle_click"}},e.prototype._handle_click=function(t){var e=t.target;this.model.set("index",parseInt(e.value),{updated_view:this}),this.touch()},e}(h),xe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"ToggleButtonsStyleModel"})},e.styleProperties=me({},u.styleProperties,{button_width:{selector:".widget-toggle-button",attribute:"width",default:null},font_weight:{selector:".widget-toggle-button",attribute:"font-weight",default:""}}),e}(u),_e=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.defaults=function(){return me({},t.prototype.defaults.call(this),{_model_name:"ToggleButtonsModel",_view_name:"ToggleButtonsView"})},e}(ye),De=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.initialize=function(e){this._css_state={},t.prototype.initialize.call(this,e),this.listenTo(this.model,"change:button_style",this.update_button_style)},e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-toggle-buttons"),this.buttongroup=document.createElement("div"),this.el.appendChild(this.buttongroup),this.update(),this.set_button_style()},e.prototype.update=function(n){for(var r=this,i=this.model.get("_options_labels"),o=this.model.get("icons")||[],s=this.model.previous("icons")||[],A=e.classMap[this.model.previous("button_style")]||"",l=r.model.get("tooltips")||[],c=this.model.get("disabled"),u=this.buttongroup.querySelectorAll("button"),g=p.pluck(u,"value"),h=!1,d=0,f=i.length;d<f;++d)if(g[d]!==i[d]||o[d]!==s[d]){h=!0;break}return!h||void 0!==n&&n.updated_view===this||(this.buttongroup.textContent="",i.forEach(function(t,e){var n;n=0===t.trim().length&&(!o[e]||0===o[e].trim().length)?"&nbsp;":a(t);var i=document.createElement("i"),s=document.createElement("button");o[e]&&(i.className="fa fa-"+o[e]),s.setAttribute("type","button"),s.className="widget-toggle-button jupyter-button",A&&s.classList.add(A),s.innerHTML=n,s.setAttribute("data-value",encodeURIComponent(t)),s.setAttribute("value",e.toString()),s.appendChild(i),s.disabled=c,l[e]&&s.setAttribute("title",l[e]),r.update_style_traits(s),r.buttongroup.appendChild(s)})),i.forEach(function(t,e){var n='[data-value="'+encodeURIComponent(t)+'"]',i=r.buttongroup.querySelector(n);r.model.get("index")===e?i.classList.add("mod-active"):i.classList.remove("mod-active")}),this.stylePromise.then(function(t){t&&t.style()}),t.prototype.update.call(this,n)},e.prototype.update_style_traits=function(t){for(var e in this._css_state)if(this._css_state.hasOwnProperty(e))if("margin"===e)this.buttongroup.style[e]=this._css_state[e];else if("width"!==e)if(t)t.style[e]=this._css_state[e];else{var n=this.buttongroup.querySelectorAll("button");n.length&&(n[0].style[e]=this._css_state[e])}},e.prototype.update_button_style=function(){for(var t=this.buttongroup.querySelectorAll("button"),n=0;n<t.length;n++)this.update_mapped_classes(e.classMap,"button_style",t[n])},e.prototype.set_button_style=function(){for(var t=this.buttongroup.querySelectorAll("button"),n=0;n<t.length;n++)this.set_mapped_classes(e.classMap,"button_style",t[n])},e.prototype.events=function(){return{"click button":"_handle_click"}},e.prototype._handle_click=function(t){var e=t.target;this.model.set("index",parseInt(e.value,10),{updated_view:this}),this.touch(),this.send({event:"click"})},e}(h);(De||(De={})).classMap={primary:["mod-primary"],success:["mod-success"],info:["mod-info"],warning:["mod-warning"],danger:["mod-danger"]};var Me=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.defaults=function(){return me({},t.prototype.defaults.call(this),{_model_name:"SelectionSliderModel",_view_name:"SelectionSliderView",orientation:"horizontal",readout:!0,continuous_update:!0})},e}(ye),ke=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.render=function(){var e=this;t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-hslider"),this.el.classList.add("widget-slider"),(this.$slider=H()("<div />")).slider({slide:this.handleSliderChange.bind(this),stop:this.handleSliderChanged.bind(this)}).addClass("slider"),this.slider_container=document.createElement("div"),this.slider_container.classList.add("slider-container"),this.slider_container.appendChild(this.$slider[0]),this.el.appendChild(this.slider_container),this.readout=document.createElement("div"),this.el.appendChild(this.readout),this.readout.classList.add("widget-readout"),this.readout.style.display="none",this.listenTo(this.model,"change:slider_color",function(t,n){e.$slider.find("a").css("background",n)}),this.$slider.find("a").css("background",this.model.get("slider_color")),this.update()},e.prototype.update=function(e){if(void 0===e||e.updated_view!==this){var n=this.model.get("_options_labels").length-1;this.$slider.slider("option","step",1),this.$slider.slider("option","max",n),this.$slider.slider("option","min",0);var r=this.model.get("orientation");this.$slider.slider("option","value",0),this.$slider.slider("option","orientation",r);var i=this.model.get("disabled");this.$slider.slider("option","disabled",i),this.readout.contentEditable=i?"false":"true","vertical"===r?(this.el.classList.remove("widget-hslider"),this.el.classList.remove("widget-inline-hbox"),this.el.classList.add("widget-vslider"),this.el.classList.add("widget-inline-vbox")):(this.el.classList.remove("widget-vslider"),this.el.classList.remove("widget-inline-vbox"),this.el.classList.add("widget-hslider"),this.el.classList.add("widget-inline-hbox"));var o=this.model.get("readout");this.readout.style.display=o?"":"none",this.updateSelection()}return t.prototype.update.call(this,e)},e.prototype.events=function(){return{slide:"handleSliderChange",slidestop:"handleSliderChanged"}},e.prototype.updateSelection=function(){var t=this.model.get("index");this.$slider.slider("option","value",t),this.updateReadout(t)},e.prototype.updateReadout=function(t){var e=this.model.get("_options_labels")[t];this.readout.textContent=e},e.prototype.handleSliderChange=function(t,e){this.updateReadout(e.value),this.model.get("continuous_update")&&this.handleSliderChanged(t,e)},e.prototype.handleSliderChanged=function(t,e){this.updateReadout(e.value),this.model.set("index",e.value,{updated_view:this}),this.touch()},e}(h),Se=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.defaults=function(){return me({},t.prototype.defaults.call(this),{_model_name:"MultipleSelectionModel"})},e}(ye),Le=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.defaults=function(){return me({},t.prototype.defaults.call(this),{_model_name:"SelectMultipleModel",_view_name:"SelectMultipleView",rows:null})},e}(Se),Re=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.initialize=function(e){t.prototype.initialize.call(this,e),this.listbox.multiple=!0},e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("widget-select-multiple")},e.prototype.updateSelection=function(t){if(void 0===t&&(t={}),t.updated_view!==this){var e=this.model.get("index")||[],n=this.listbox.options;this.listbox.selectedIndex=-1,e.forEach(function(t){n[t].selected=!0})}},e.prototype._handle_change=function(){var t=Array.prototype.map.call(this.listbox.selectedOptions||[],function(t){return t.index});this.model.set("index",t,{updated_view:this}),this.touch()},e}(be),Ne=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.defaults=function(){return me({},t.prototype.defaults.call(this),{_model_name:"SelectionSliderModel",_view_name:"SelectionSliderView",orientation:"horizontal",readout:!0,continuous_update:!0})},e}(Se),Fe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return Ce(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.$slider.slider("option","range",!0)},e.prototype.updateSelection=function(){var t=this.model.get("index");this.$slider.slider("option","values",t.slice()),this.updateReadout(t)},e.prototype.updateReadout=function(t){var e=this.model.get("_options_labels"),n=e[t[0]],r=e[t[1]];this.readout.textContent=n+"-"+r},e.prototype.handleSliderChange=function(t,e){this.updateReadout(e.values),this.model.get("continuous_update")&&this.handleSliderChanged(t,e)},e.prototype.handleSliderChanged=function(t,e){this.updateReadout(e.values),this.model.set("index",e.values.slice(),{updated_view:this}),this.touch()},e}(ke),Te=n(10),Ue=n(46),Oe=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Ge=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e._widgetRemoved=new Te.Signal(e),e}return Oe(e,t),Object.defineProperty(e.prototype,"widgetRemoved",{get:function(){return this._widgetRemoved},enumerable:!0,configurable:!0}),e.prototype.onChildRemoved=function(t){this._widgetRemoved.emit(t.child)},e}(O.Panel),He=function(t){function e(e){void 0===e&&(e={});var n=t.call(this)||this;n._currentChanged=new Te.Signal(n),n.addClass("p-TabPanel"),n.tabBar=new O.TabBar(e),n.tabBar.addClass("p-TabPanel-tabBar"),n.tabContents=new Ge,n.tabContents.addClass("p-TabPanel-tabContents"),n.tabBar.tabMoved.connect(n._onTabMoved,n),n.tabBar.currentChanged.connect(n._onCurrentChanged,n),n.tabBar.tabCloseRequested.connect(n._onTabCloseRequested,n),n.tabBar.tabActivateRequested.connect(n._onTabActivateRequested,n),n.tabContents.widgetRemoved.connect(n._onWidgetRemoved,n);var r=new O.PanelLayout;return r.addWidget(n.tabBar),r.addWidget(n.tabContents),n.layout=r,n}return Oe(e,t),Object.defineProperty(e.prototype,"currentChanged",{get:function(){return this._currentChanged},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"currentIndex",{get:function(){var t=this.tabBar.currentIndex;return-1===t?null:t},set:function(t){this.tabBar.currentIndex=null===t?-1:t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"currentWidget",{get:function(){var t=this.tabBar.currentTitle;return t?t.owner:null},set:function(t){this.tabBar.currentTitle=t?t.title:null},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"tabsMovable",{get:function(){return this.tabBar.tabsMovable},set:function(t){this.tabBar.tabsMovable=t},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"widgets",{get:function(){return this.tabContents.widgets},enumerable:!0,configurable:!0}),e.prototype.addWidget=function(t){this.insertWidget(this.widgets.length,t)},e.prototype.insertWidget=function(t,e){e!==this.currentWidget&&e.hide(),this.tabContents.insertWidget(t,e),this.tabBar.insertTab(t,e.title)},e.prototype._onCurrentChanged=function(t,e){var n=e.previousIndex,r=e.previousTitle,i=e.currentIndex,o=e.currentTitle,a=r?r.owner:null,s=o?o.owner:null;a&&a.hide(),s&&s.show(),this._currentChanged.emit({previousIndex:n,previousWidget:a,currentIndex:i,currentWidget:s}),(Ue.Platform.IS_EDGE||Ue.Platform.IS_IE)&&U.MessageLoop.flush()},e.prototype._onTabActivateRequested=function(t,e){e.title.owner.activate()},e.prototype._onTabCloseRequested=function(t,e){e.title.owner.close()},e.prototype._onTabMoved=function(t,e){this.tabContents.insertWidget(e.toIndex,e.title.owner)},e.prototype._onWidgetRemoved=function(t,e){this.tabBar.removeTab(e.title)},e}(O.Widget),je=function(){function t(t,e){void 0===e&&(e={}),this._array=null,this._value=null,this._previousValue=null,this._selectionChanged=new Te.Signal(this),this._array=t,this._insertBehavior=e.insertBehavior||"select-item-if-needed",this._removeBehavior=e.removeBehavior||"select-item-after"}return Object.defineProperty(t.prototype,"selectionChanged",{get:function(){return this._selectionChanged},enumerable:!0,configurable:!0}),t.prototype.adjustSelectionForSet=function(t){var e=this.index,n=this.value;if(t===e){this._updateSelectedValue();var r=this.value;this._previousValue=null,n!==r&&this._selectionChanged.emit({previousIndex:e,previousValue:n,currentIndex:e,currentValue:r})}},Object.defineProperty(t.prototype,"value",{get:function(){return this._value},set:function(t){this.index=null===t?null:T.ArrayExt.firstIndexOf(this._array,t)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"index",{get:function(){return this._index},set:function(t){var e;if(null!==t?((e=Math.floor(t))<0||e>=this._array.length)&&(e=null):e=null,this._index!==e){var n=this._index,r=this._value;this._index=e,this._updateSelectedValue(),this._previousValue=r,this._selectionChanged.emit({previousIndex:n,previousValue:r,currentIndex:e,currentValue:this._value})}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"insertBehavior",{get:function(){return this._insertBehavior},set:function(t){this._insertBehavior=t},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"removeBehavior",{get:function(){return this._removeBehavior},set:function(t){this._removeBehavior=t},enumerable:!0,configurable:!0}),t.prototype.adjustSelectionForInsert=function(t,e){var n=this._value,r=this._index,i=this._insertBehavior;if("select-item"===i||"select-item-if-needed"===i&&null===r)return this._index=t,this._value=e,this._previousValue=n,void this._selectionChanged.emit({previousIndex:r,previousValue:n,currentIndex:t,currentValue:e});r>=t&&this._index++},t.prototype.adjustSelectionForMove=function(t,e){this._index===t?this._index=e:this._index<t&&this._index>=e?this._index++:this._index>t&&this._index<=e&&this._index--},t.prototype.clearSelection=function(){var t=this._index,e=this._value;this._index=null,this._value=null,this._previousValue=null,null!==t&&this._selectionChanged.emit({previousIndex:t,previousValue:e,currentIndex:this._index,currentValue:this._value})},t.prototype.adjustSelectionForRemove=function(t,e){var n=this._index,r=this._removeBehavior;if(n===t){if(0===this._array.length)return this._index=null,this._value=null,this._previousValue=null,void this._selectionChanged.emit({previousIndex:t,previousValue:e,currentIndex:this._index,currentValue:this._value});if("select-item-after"===r)return this._index=Math.min(t,this._array.length-1),this._updateSelectedValue(),this._previousValue=null,void this._selectionChanged.emit({previousIndex:t,previousValue:e,currentIndex:this._index,currentValue:this._value});if("select-item-before"===r)return this._index=Math.max(0,t-1),this._updateSelectedValue(),this._previousValue=null,void this._selectionChanged.emit({previousIndex:t,previousValue:e,currentIndex:this._index,currentValue:this._value});if("select-previous-item"===r)return this._previousValue?this.value=this._previousValue:(this._index=Math.min(t,this._array.length-1),this._updateSelectedValue()),this._previousValue=null,void this._selectionChanged.emit({previousIndex:t,previousValue:e,currentIndex:this._index,currentValue:this.value});this._index=null,this._value=null,this._previousValue=null,this._selectionChanged.emit({previousIndex:t,previousValue:e,currentIndex:this._index,currentValue:this._value})}else n>t&&this._index--},t.prototype._updateSelectedValue=function(){var t=this._index;this._value=null!==t?this._array[t]:null},t}(),Ye=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Pe="p-Collapse",Ke="p-Collapse-header",ze="p-Collapse-contents",Je=function(t){function e(e){var n=t.call(this,e)||this;n._collapseChanged=new Te.Signal(n),n.addClass(Pe),n._header=new O.Widget,n._header.addClass(Ke),n._header.node.addEventListener("click",n),n._content=new O.Panel,n._content.addClass(ze);var r=new O.PanelLayout;return n.layout=r,r.addWidget(n._header),r.addWidget(n._content),e.widget&&(n.widget=e.widget),n.collapsed=!1,n}return Ye(e,t),e.prototype.dispose=function(){this.isDisposed||(t.prototype.dispose.call(this),this._header=null,this._widget=null,this._content=null)},Object.defineProperty(e.prototype,"widget",{get:function(){return this._widget},set:function(t){var e=this._widget;e&&(e.disposed.disconnect(this._onChildDisposed,this),e.title.changed.disconnect(this._onTitleChanged,this),e.parent=null),this._widget=t,t.disposed.connect(this._onChildDisposed,this),t.title.changed.connect(this._onTitleChanged,this),this._onTitleChanged(t.title),this._content.addWidget(t)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"collapsed",{get:function(){return this._collapsed},set:function(t){t!==this._collapsed&&(t?this._collapse():this._uncollapse())},enumerable:!0,configurable:!0}),e.prototype.toggle=function(){this.collapsed=!this.collapsed},Object.defineProperty(e.prototype,"collapseChanged",{get:function(){return this._collapseChanged},enumerable:!0,configurable:!0}),e.prototype._collapse=function(){this._collapsed=!0,this._content&&this._content.hide(),this.removeClass("p-Collapse-open"),this._collapseChanged.emit(void 0)},e.prototype._uncollapse=function(){this._collapsed=!1,this._content&&this._content.show(),this.addClass("p-Collapse-open"),this._collapseChanged.emit(void 0)},e.prototype.handleEvent=function(t){switch(t.type){case"click":this._evtClick(t)}},e.prototype._evtClick=function(t){this.toggle()},e.prototype._onTitleChanged=function(t){this._header.node.textContent=this._widget.title.label},e.prototype._onChildDisposed=function(t){this.dispose()},e}(O.Widget),qe="p-Accordion",Ve=function(t){function e(e){var n=t.call(this,e)||this;return n._selection=new je(n.widgets),n._selection.selectionChanged.connect(n._onSelectionChanged,n),n.addClass(qe),n}return Ye(e,t),Object.defineProperty(e.prototype,"collapseWidgets",{get:function(){return this.layout.widgets},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"selection",{get:function(){return this._selection},enumerable:!0,configurable:!0}),e.prototype.indexOf=function(t){return T.ArrayExt.findFirstIndex(this.collapseWidgets,function(e){return e.widget===t})},e.prototype.addWidget=function(e){var n=this._wrapWidget(e);return n.collapsed=!0,t.prototype.addWidget.call(this,n),this._selection.adjustSelectionForInsert(this.widgets.length-1,n),n},e.prototype.insertWidget=function(e,n){var r=this._wrapWidget(n);r.collapsed=!0,t.prototype.insertWidget.call(this,e,r),this._selection.adjustSelectionForInsert(e,r)},e.prototype.removeWidget=function(t){var e=this.indexOf(t);if(e>=0){var n=this.collapseWidgets[e];t.parent=null,n.dispose(),this._selection.adjustSelectionForRemove(e,null)}},e.prototype._wrapWidget=function(t){var e=new Je({widget:t});return e.addClass("p-Accordion-child"),e.collapseChanged.connect(this._onCollapseChange,this),e},e.prototype._onCollapseChange=function(t){t.collapsed?this._selection.value===t&&t.collapsed&&(this._selection.value=null):this._selection.value=t},e.prototype._onSelectionChanged=function(t,e){var n=e.previousValue,r=e.currentValue;n&&(n.collapsed=!0,n.removeClass("p-Accordion-child-active")),r&&(r.collapsed=!1,r.addClass("p-Accordion-child-active"))},e}(O.Panel),We=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Xe=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return We(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"SelectionContainerModel",selected_index:0,_titles:{}})},e}(P),Ze=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return We(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"AccordionModel",_view_name:"AccordionView"})},e}(Xe),$e=function(t){function e(e){var n=this,r=e.view;return delete e.view,(n=t.call(this,e)||this)._view=r,n}return We(e,t),e.prototype.processMessage=function(e){t.prototype.processMessage.call(this,e),this._view.processPhosphorMessage(e)},e.prototype.dispose=function(){this.isDisposed||(t.prototype.dispose.call(this),this._view&&this._view.remove(),this._view=null)},e}(Ve),tn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return We(e,t),e.prototype._createElement=function(t){return this.pWidget=new $e({view:this}),this.pWidget.node},e.prototype._setElement=function(t){if(this.el||t!==this.pWidget.node)throw new Error("Cannot reset the DOM element.");this.el=this.pWidget.node,this.$el=H()(this.pWidget.node)},e.prototype.initialize=function(e){var n=this;t.prototype.initialize.call(this,e),this.children_views=new r.ViewList(this.add_child_view,this.remove_child_view,this),this.listenTo(this.model,"change:children",function(){return n.updateChildren()}),this.listenTo(this.model,"change:selected_index",function(){return n.update_selected_index()}),this.listenTo(this.model,"change:_titles",function(){return n.update_titles()})},e.prototype.render=function(){var e=this;t.prototype.render.call(this);var n=this.pWidget;n.addClass("jupyter-widgets"),n.addClass("widget-accordion"),n.addClass("widget-container"),n.selection.selectionChanged.connect(function(t){e.updatingChildren||(e.model.set("selected_index",n.selection.index),e.touch())}),this.children_views.update(this.model.get("children")),this.update_titles(),this.update_selected_index()},e.prototype.updateChildren=function(){this.updatingChildren=!0,this.pWidget.selection.index=null,this.children_views.update(this.model.get("children")),this.update_selected_index(),this.updatingChildren=!1},e.prototype.update_titles=function(){for(var t=this.pWidget.collapseWidgets,e=this.model.get("_titles"),n=0;n<t.length;n++)void 0!==e[n]&&(t[n].widget.title.label=e[n])},e.prototype.update_selected_index=function(){this.pWidget.selection.index=this.model.get("selected_index")},e.prototype.remove_child_view=function(t){this.pWidget.removeWidget(t.pWidget),t.remove()},e.prototype.add_child_view=function(t,e){var n=this.pWidget,r=new O.Widget;return r.title.label=this.model.get("_titles")[e]||"",n.addWidget(r),this.create_child_view(t).then(function(t){var e=t.pWidget;return e.title.label=r.title.label,n.collapseWidgets[n.indexOf(r)].widget=e,r.dispose(),t}).catch(i("Could not add child view to box",!0))},e.prototype.remove=function(){this.children_views=null,t.prototype.remove.call(this)},e}(r.DOMWidgetView),en=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return We(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"TabModel",_view_name:"TabView"})},e}(Xe),nn=function(t){function e(e){var n=this,r=e.view;return delete e.view,(n=t.call(this,e)||this)._view=r,U.MessageLoop.installMessageHook(n.tabContents,function(t,e){return n._view.processPhosphorMessage(e),!0}),n}return We(e,t),e.prototype.dispose=function(){this.isDisposed||(t.prototype.dispose.call(this),this._view&&this._view.remove(),this._view=null)},e}(He),rn=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.updatingTabs=!1,e}return We(e,t),e.prototype._createElement=function(t){return this.pWidget=new nn({view:this}),this.pWidget.node},e.prototype._setElement=function(t){if(this.el||t!==this.pWidget.node)throw new Error("Cannot reset the DOM element.");this.el=this.pWidget.node,this.$el=H()(this.pWidget.node)},e.prototype.initialize=function(e){var n=this;t.prototype.initialize.call(this,e),this.childrenViews=new r.ViewList(this.addChildView,function(t){t.remove()},this),this.listenTo(this.model,"change:children",function(){return n.updateTabs()}),this.listenTo(this.model,"change:_titles",function(){return n.updateTitles()})},e.prototype.render=function(){t.prototype.render.call(this);var e=this.pWidget;e.addClass("jupyter-widgets"),e.addClass("widget-container"),e.addClass("widget-tab"),e.tabsMovable=!0,e.tabBar.insertBehavior="none",e.tabBar.currentChanged.connect(this._onTabChanged,this),e.tabBar.tabMoved.connect(this._onTabMoved,this),e.tabBar.addClass("widget-tab-bar"),e.tabContents.addClass("widget-tab-contents"),e.tabBar.tabsMovable=!1,this.updateTabs(),this.update()},e.prototype.updateTabs=function(){this.updatingTabs=!0,this.pWidget.currentIndex=null,this.childrenViews.update(this.model.get("children")),this.pWidget.currentIndex=this.model.get("selected_index"),this.updatingTabs=!1},e.prototype.addChildView=function(t,e){var n=this.model.get("_titles")[e]||"",r=this.pWidget,o=new O.Widget;return o.title.label=n,r.addWidget(o),this.create_child_view(t).then(function(t){var e=t.pWidget;e.title.label=o.title.label,e.title.closable=!1;var n=T.ArrayExt.firstIndexOf(r.widgets,o);return r.insertWidget(n+1,e),o.dispose(),t}).catch(i("Could not add child view to box",!0))},e.prototype.update=function(){return this.updateSelectedIndex(),t.prototype.update.call(this)},e.prototype.updateTitles=function(){var t=this.model.get("_titles")||{};Object(T.each)(this.pWidget.widgets,function(e,n){e.title.label=t[n]||""})},e.prototype.updateSelectedIndex=function(){this.pWidget.currentIndex=this.model.get("selected_index")},e.prototype.remove=function(){this.childrenViews=null,t.prototype.remove.call(this)},e.prototype._onTabChanged=function(t,e){if(!this.updatingTabs){var n=e.currentIndex;this.model.set("selected_index",-1===n?null:n),this.touch()}},e.prototype._onTabMoved=function(t,e){var n=this.model.get("children").slice();T.ArrayExt.move(n,e.fromIndex,e.toIndex),this.model.set("children",n),this.touch()},e}(r.DOMWidgetView),on=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),an=function(){return(an=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},sn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return on(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{value:"",disabled:!1,placeholder:"​",_model_name:"StringModel"})},e}(y),An=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return on(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_view_name:"HTMLView",_model_name:"HTMLModel"})},e}(sn),ln=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return on(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-html"),this.content=document.createElement("div"),this.content.classList.add("widget-html-content"),this.el.appendChild(this.content),this.update()},e.prototype.update=function(){return this.content.innerHTML=this.model.get("value"),t.prototype.update.call(this)},e}(h),cn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return on(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_view_name:"HTMLMathView",_model_name:"HTMLMathModel"})},e}(sn),un=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return on(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-htmlmath"),this.content=document.createElement("div"),this.content.classList.add("widget-htmlmath-content"),this.el.appendChild(this.content),this.update()},e.prototype.update=function(){return this.content.innerHTML=this.model.get("value"),this.typeset(this.content),t.prototype.update.call(this)},e}(h),gn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return on(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_view_name:"LabelView",_model_name:"LabelModel"})},e}(sn),hn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return on(e,t),e.prototype.render=function(){t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-label"),this.update()},e.prototype.update=function(){return this.typeset(this.el,this.model.get("value")),t.prototype.update.call(this)},e}(h),dn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return on(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_view_name:"TextareaView",_model_name:"TextareaModel",rows:null,continuous_update:!0})},e}(sn),fn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return on(e,t),e.prototype.render=function(){var e=this;t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-textarea"),this.textbox=document.createElement("textarea"),this.textbox.setAttribute("rows","5"),this.textbox.id=this.label.htmlFor=Object(r.uuid)(),this.el.appendChild(this.textbox),this.update(),this.listenTo(this.model,"change:placeholder",function(t,n,r){e.update_placeholder(n)}),this.update_placeholder()},e.prototype.update_placeholder=function(t){t=t||this.model.get("placeholder"),this.textbox.setAttribute("placeholder",t.toString())},e.prototype.update=function(e){if(void 0===e||e.updated_view!=this){this.textbox.value=this.model.get("value");var n=this.model.get("rows");null===n&&(n=""),this.textbox.setAttribute("rows",n),this.textbox.disabled=this.model.get("disabled")}return t.prototype.update.call(this)},e.prototype.events=function(){return{"keydown input":"handleKeyDown","keypress input":"handleKeypress","input textarea":"handleChanging","change textarea":"handleChanged"}},e.prototype.handleKeyDown=function(t){t.stopPropagation()},e.prototype.handleKeypress=function(t){t.stopPropagation()},e.prototype.handleChanging=function(t){this.model.get("continuous_update")&&this.handleChanged(t)},e.prototype.handleChanged=function(t){var e=t.target;this.model.set("value",e.value,{updated_view:this}),this.touch()},e}(h),pn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return on(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_view_name:"TextView",_model_name:"TextModel",continuous_update:!0})},e}(sn),In=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.inputType="text",e}return on(e,t),e.prototype.render=function(){var e=this;t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-text"),this.textbox=document.createElement("input"),this.textbox.setAttribute("type",this.inputType),this.textbox.id=this.label.htmlFor=Object(r.uuid)(),this.el.appendChild(this.textbox),this.update(),this.listenTo(this.model,"change:placeholder",function(t,n,r){e.update_placeholder(n)}),this.listenTo(this.model,"change:description_tooltip",this.update_title),this.listenTo(this.model,"change:description",this.update_title),this.update_placeholder(),this.update_title()},e.prototype.update_placeholder=function(t){this.textbox.setAttribute("placeholder",t||this.model.get("placeholder"))},e.prototype.update_title=function(){var t=this.model.get("description_tooltip");t?0===this.model.get("description").length&&this.textbox.setAttribute("title",t):this.textbox.removeAttribute("title")},e.prototype.update=function(e){return void 0!==e&&e.updated_view===this||(this.textbox.value!==this.model.get("value")&&(this.textbox.value=this.model.get("value")),this.textbox.disabled=this.model.get("disabled")),t.prototype.update.call(this)},e.prototype.events=function(){return{"keydown input":"handleKeyDown","keypress input":"handleKeypress","input input":"handleChanging","change input":"handleChanged"}},e.prototype.handleKeyDown=function(t){t.stopPropagation()},e.prototype.handleKeypress=function(t){t.stopPropagation(),13===t.keyCode&&this.send({event:"submit"})},e.prototype.handleChanging=function(t){this.model.get("continuous_update")&&this.handleChanged(t)},e.prototype.handleChanged=function(t){var e=t.target;this.model.set("value",e.value,{updated_view:this}),this.touch()},e}(h),Cn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return on(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_view_name:"PasswordView",_model_name:"PasswordModel"})},e}(pn),mn=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.inputType="password",e}return on(e,t),e}(In),yn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return on(e,t),e.prototype.defaults=function(){return an({},t.prototype.defaults.call(this),{_model_name:"ComboboxModel",_view_name:"ComboboxView",options:[],ensure_options:!1})},e}(pn),En=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.isInitialRender=!0,e}return on(e,t),e.prototype.render=function(){this.datalist=document.createElement("datalist"),this.datalist.id=Object(r.uuid)(),t.prototype.render.call(this),this.textbox.setAttribute("list",this.datalist.id),this.el.appendChild(this.datalist)},e.prototype.update=function(e){if(t.prototype.update.call(this,e),this.datalist){var n=this.isValid(this.model.get("value"));if(this.highlightValidState(n),!(void 0!==e&&e.updated_view||!this.model.hasChanged("options")&&!this.isInitialRender)){this.isInitialRender=!1;var r=this.model.get("options").map(function(t){return'<option value="'+t+'"></option>'});this.datalist.innerHTML=r.join("\n")}}},e.prototype.isValid=function(t){if(!0===this.model.get("ensure_option")&&-1===this.model.get("options").indexOf(t))return!1;return!0},e.prototype.handleChanging=function(e){var n=e.target,r=this.isValid(n.value);this.highlightValidState(r),r&&t.prototype.handleChanging.call(this,e)},e.prototype.handleChanged=function(e){var n=e.target,r=this.isValid(n.value);this.highlightValidState(r),r&&t.prototype.handleChanged.call(this,e)},e.prototype.highlightValidState=function(t){this.textbox.classList.toggle("jpwidgets-invalidComboValue",!t)},e}(In),vn=function(){var t=function(e,n){return(t=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var n in e)e.hasOwnProperty(n)&&(t[n]=e[n])})(e,n)};return function(e,n){function r(){this.constructor=e}t(e,n),e.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Bn=function(){return(Bn=Object.assign||function(t){for(var e,n=1,r=arguments.length;n<r;n++)for(var i in e=arguments[n])Object.prototype.hasOwnProperty.call(e,i)&&(t[i]=e[i]);return t}).apply(this,arguments)},bn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return vn(e,t),e.prototype.defaults=function(){return p.extend(t.prototype.defaults.call(this),{_model_name:"FileUploadModel",_view_name:"FileUploadView",_counter:0,accept:"",description:"Upload",tooltip:"",disabled:!1,icon:"upload",button_style:"",multiple:!1,metadata:[],data:[],error:"",style:null})},e.serializers=Bn({},m.serializers,{data:{serialize:function(t){return t.slice()}}}),e}(m),Qn=function(t){function e(){return null!==t&&t.apply(this,arguments)||this}return vn(e,t),Object.defineProperty(e.prototype,"tagName",{get:function(){return"button"},enumerable:!0,configurable:!0}),e.prototype.render=function(){var e=this;t.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-upload"),this.el.classList.add("jupyter-button"),this.fileInput=document.createElement("input"),this.fileInput.type="file",this.fileInput.style.display="none",this.el.appendChild(this.fileInput),this.el.addEventListener("click",function(){e.fileInput.click()}),this.fileInput.addEventListener("click",function(){e.fileInput.value=""}),this.fileInput.addEventListener("change",function(){var t=[];Array.from(e.fileInput.files).forEach(function(n){t.push(new Promise(function(t,r){var i={name:n.name,type:n.type,size:n.size,lastModified:n.lastModified};e.fileReader=new FileReader,e.fileReader.onload=function(e){var n=e.target.result;t({buffer:n,metadata:i,error:""})},e.fileReader.onerror=function(){r()},e.fileReader.onabort=e.fileReader.onerror,e.fileReader.readAsArrayBuffer(n)}))}),Promise.all(t).then(function(t){var n=[],r=[];t.forEach(function(t){n.push(t.metadata),r.push(t.buffer)});var i=e.model.get("_counter");e.model.set({_counter:i+t.length,metadata:n,data:r,error:""}),e.touch()}).catch(function(t){console.error("error in file upload: %o",t),e.model.set({error:t}),e.touch()})}),this.listenTo(this.model,"change:button_style",this.update_button_style),this.set_button_style(),this.update()},e.prototype.update=function(){this.el.disabled=this.model.get("disabled"),this.el.setAttribute("title",this.model.get("tooltip"));var e=this.model.get("description")+" ("+this.model.get("_counter")+")",n=this.model.get("icon");if(e.length||n.length){if(this.el.textContent="",n.length){var r=document.createElement("i");r.classList.add("fa"),r.classList.add("fa-"+n),0===e.length&&r.classList.add("center"),this.el.appendChild(r)}this.el.appendChild(document.createTextNode(e))}return this.fileInput.accept=this.model.get("accept"),this.fileInput.multiple=this.model.get("multiple"),t.prototype.update.call(this)},e.prototype.update_button_style=function(){this.update_mapped_classes(e.class_map,"button_style",this.el)},e.prototype.set_button_style=function(){this.set_mapped_classes(e.class_map,"button_style",this.el)},e.class_map={primary:["mod-primary"],success:["mod-success"],info:["mod-info"],warning:["mod-warning"],danger:["mod-danger"]},e}(r.DOMWidgetView);n.d(e,"version",function(){return wn}),n.d(e,"uuid",function(){return r.uuid}),n.d(e,"WrappedError",function(){return r.WrappedError}),n.d(e,"resolvePromisesDict",function(){return r.resolvePromisesDict}),n.d(e,"reject",function(){return i}),n.d(e,"typeset",function(){return o}),n.d(e,"escape_html",function(){return a}),n.d(e,"JUPYTER_CONTROLS_VERSION",function(){return A.a}),n.d(e,"DirectionalLinkModel",function(){return B}),n.d(e,"LinkModel",function(){return b}),n.d(e,"BoolModel",function(){return w}),n.d(e,"CheckboxModel",function(){return x}),n.d(e,"CheckboxView",function(){return _}),n.d(e,"ToggleButtonModel",function(){return D}),n.d(e,"ToggleButtonView",function(){return M}),n.d(e,"ValidModel",function(){return k}),n.d(e,"ValidView",function(){return S}),n.d(e,"ButtonStyleModel",function(){return R}),n.d(e,"ButtonModel",function(){return N}),n.d(e,"ButtonView",function(){return F}),n.d(e,"BoxModel",function(){return P}),n.d(e,"HBoxModel",function(){return K}),n.d(e,"VBoxModel",function(){return z}),n.d(e,"BoxView",function(){return J}),n.d(e,"HBoxView",function(){return q}),n.d(e,"VBoxView",function(){return V}),n.d(e,"GridBoxView",function(){return W}),n.d(e,"GridBoxModel",function(){return X}),n.d(e,"ImageModel",function(){return tt}),n.d(e,"ImageView",function(){return et}),n.d(e,"VideoModel",function(){return it}),n.d(e,"VideoView",function(){return ot}),n.d(e,"AudioModel",function(){return At}),n.d(e,"AudioView",function(){return lt}),n.d(e,"ColorPickerModel",function(){return ut}),n.d(e,"ColorPickerView",function(){return gt}),n.d(e,"serialize_date",function(){return pt}),n.d(e,"deserialize_date",function(){return It}),n.d(e,"DatePickerModel",function(){return Ct}),n.d(e,"DatePickerView",function(){return mt}),n.d(e,"IntModel",function(){return Nt}),n.d(e,"BoundedIntModel",function(){return Ft}),n.d(e,"SliderStyleModel",function(){return Tt}),n.d(e,"IntSliderModel",function(){return Ut}),n.d(e,"IntRangeSliderModel",function(){return Ot}),n.d(e,"BaseIntSliderView",function(){return Gt}),n.d(e,"IntRangeSliderView",function(){return Ht}),n.d(e,"IntSliderView",function(){return jt}),n.d(e,"IntTextModel",function(){return Yt}),n.d(e,"BoundedIntTextModel",function(){return Pt}),n.d(e,"IntTextView",function(){return Kt}),n.d(e,"ProgressStyleModel",function(){return zt}),n.d(e,"IntProgressModel",function(){return Jt}),n.d(e,"ProgressView",function(){return qt}),n.d(e,"PlayModel",function(){return Vt}),n.d(e,"PlayView",function(){return Wt}),n.d(e,"FloatModel",function(){return Zt}),n.d(e,"BoundedFloatModel",function(){return $t}),n.d(e,"FloatSliderModel",function(){return te}),n.d(e,"FloatLogSliderModel",function(){return ee}),n.d(e,"FloatRangeSliderModel",function(){return ne}),n.d(e,"FloatSliderView",function(){return re}),n.d(e,"FloatLogSliderView",function(){return ie}),n.d(e,"FloatRangeSliderView",function(){return oe}),n.d(e,"FloatTextModel",function(){return ae}),n.d(e,"BoundedFloatTextModel",function(){return se}),n.d(e,"FloatTextView",function(){return Ae}),n.d(e,"FloatProgressModel",function(){return le}),n.d(e,"ControllerButtonModel",function(){return ge}),n.d(e,"ControllerButtonView",function(){return he}),n.d(e,"ControllerAxisModel",function(){return de}),n.d(e,"ControllerAxisView",function(){return fe}),n.d(e,"ControllerModel",function(){return pe}),n.d(e,"ControllerView",function(){return Ie}),n.d(e,"SelectionModel",function(){return ye}),n.d(e,"DropdownModel",function(){return Ee}),n.d(e,"DropdownView",function(){return ve}),n.d(e,"SelectModel",function(){return Be}),n.d(e,"SelectView",function(){return be}),n.d(e,"RadioButtonsModel",function(){return Qe}),n.d(e,"RadioButtonsView",function(){return we}),n.d(e,"ToggleButtonsStyleModel",function(){return xe}),n.d(e,"ToggleButtonsModel",function(){return _e}),n.d(e,"ToggleButtonsView",function(){return De}),n.d(e,"SelectionSliderModel",function(){return Me}),n.d(e,"SelectionSliderView",function(){return ke}),n.d(e,"MultipleSelectionModel",function(){return Se}),n.d(e,"SelectMultipleModel",function(){return Le}),n.d(e,"SelectMultipleView",function(){return Re}),n.d(e,"SelectionRangeSliderModel",function(){return Ne}),n.d(e,"SelectionRangeSliderView",function(){return Fe}),n.d(e,"SelectionContainerModel",function(){return Xe}),n.d(e,"AccordionModel",function(){return Ze}),n.d(e,"JupyterPhosphorAccordionWidget",function(){return $e}),n.d(e,"AccordionView",function(){return tn}),n.d(e,"TabModel",function(){return en}),n.d(e,"JupyterPhosphorTabPanelWidget",function(){return nn}),n.d(e,"TabView",function(){return rn}),n.d(e,"StringModel",function(){return sn}),n.d(e,"HTMLModel",function(){return An}),n.d(e,"HTMLView",function(){return ln}),n.d(e,"HTMLMathModel",function(){return cn}),n.d(e,"HTMLMathView",function(){return un}),n.d(e,"LabelModel",function(){return gn}),n.d(e,"LabelView",function(){return hn}),n.d(e,"TextareaModel",function(){return dn}),n.d(e,"TextareaView",function(){return fn}),n.d(e,"TextModel",function(){return pn}),n.d(e,"TextView",function(){return In}),n.d(e,"PasswordModel",function(){return Cn}),n.d(e,"PasswordView",function(){return mn}),n.d(e,"ComboboxModel",function(){return yn}),n.d(e,"ComboboxView",function(){return En}),n.d(e,"DescriptionStyleModel",function(){return u}),n.d(e,"DescriptionModel",function(){return g}),n.d(e,"DescriptionView",function(){return h}),n.d(e,"LabeledDOMWidgetModel",function(){return d}),n.d(e,"LabeledDOMWidgetView",function(){return f}),n.d(e,"FileUploadModel",function(){return bn}),n.d(e,"FileUploadView",function(){return Qn});var wn=n(1414).version},function(t,e,n){"use strict";n.r(e);var r={};n.r(r),n.d(r,"OUTPUT_WIDGET_VERSION",function(){return _}),n.d(r,"OutputModel",function(){return D}),n.d(r,"JupyterPhosphorPanelWidget",function(){return M}),n.d(r,"OutputView",function(){return k});var i=n(17),o=n(628),a=n(9);const s=new a.Token("@jupyterlab/mainmenu:IMainMenu");var A=n(611),l=n(5),c=n(50),u=n(43),g=n(6);class h extends g.Panel{constructor(t,e){super(),this._manager=new a.PromiseDelegate,this._rerenderMimeModel=null,this.mimeType=t.mimeType,e&&(this.manager=e)}set manager(t){t.restored.connect(this._rerender,this),this._manager.resolve(t)}async renderModel(t){const e=t.data[this.mimeType];this.node.textContent="Loading widget...",this.addClass("jupyter-widgets");const n=await this._manager.promise;if(""===e.model_id)return this.hide(),Promise.resolve();let r,i;try{r=await n.get_model(e.model_id)}catch(e){return n.restoredStatus?(this.node.textContent="Error displaying widget: model not found",this.addClass("jupyter-widgets"),void console.error(e)):void(this._rerenderMimeModel=t)}this._rerenderMimeModel=null;try{i=await n.display_model(void 0,r,void 0)}catch(t){return this.node.textContent="Error displaying widget",this.addClass("jupyter-widgets"),void console.error(t)}this.node.textContent="",this.addWidget(i),i.disposed.connect(()=>{this.hide(),e.model_id=""})}get isDisposed(){return null===this._manager}dispose(){this.isDisposed||(super.dispose(),this._manager=null)}_rerender(){this._rerenderMimeModel&&(this.node.textContent="",this.removeClass("jupyter-widgets"),this.renderModel(this._rerenderMimeModel))}}var d=n(20),f=n(10),p=n(265);class I{constructor(){this._cache=Object.create(null)}set(t,e,n){if(t in this._cache||(this._cache[t]=Object.create(null)),e in this._cache[t])throw`Version ${e} of key ${t} already registered.`;this._cache[t][e]=n}get(t,e){if(t in this._cache){let n=this._cache[t];return n[Object(p.maxSatisfying)(Object.keys(n),e)]}}}const C="application/vnd.jupyter.widget-view+json",m="application/vnd.jupyter.widget-state+json",y="jp-BackboneViewWrapper";class E extends g.Widget{constructor(t){super(),this._view=null,this._view=t,t.on("remove",()=>{this.dispose()}),this.addClass(y),this.node.appendChild(t.el)}onAfterAttach(t){this._view.trigger("displayed")}dispose(){this._view=null,super.dispose()}}class v extends d.ManagerBase{constructor(t,e,n){super(),this._registry=new I,this._restored=new f.Signal(this),this._restoredStatus=!1,this._initialRestoredStatus=!1,this._modelsSync=new Map,this._context=t,this._rendermime=e,this._handleCommOpen=async(t,e)=>{let n=new d.shims.services.Comm(t);await this.handle_comm_open(n,e)},t.session.kernelChanged.connect((t,e)=>{this._handleKernelChanged(e)}),t.session.statusChanged.connect((t,e)=>{this._handleKernelStatusChange(e)}),t.session.kernel&&this._handleKernelChanged({oldValue:null,newValue:t.session.kernel}),this.restoreWidgets(this._context.model),this._settings=n,t.saveState.connect((t,e)=>{"started"===e&&n.saveState&&this._saveState()})}_saveState(){const t=this.get_state_sync({drop_defaults:!0});this._context.model.metadata.set("widgets",{"application/vnd.jupyter.widget-state+json":t})}_handleKernelChanged({oldValue:t,newValue:e}){t&&t.removeCommTarget(this.comm_target_name,this._handleCommOpen),e&&e.registerCommTarget(this.comm_target_name,this._handleCommOpen)}_handleKernelStatusChange(t){switch(t){case"connected":this._initialRestoredStatus&&this.restoreWidgets(this._context.model,{loadKernel:!0,loadNotebook:!1});break;case"restarting":this.disconnect()}}async restoreWidgets(t,{loadKernel:e,loadNotebook:n}={loadKernel:!0,loadNotebook:!0}){e&&await this._loadFromKernel(),n&&await this._loadFromNotebook(t),this._restoredStatus=!0,this._initialRestoredStatus=!0,this._restored.emit()}disconnect(){super.disconnect(),this._restoredStatus=!1}async _loadFromKernel(){if(!this.context.session)return;if(await this.context.session.ready,!1===this.context.session.kernel.handleComms)return;const t=await this._get_comm_info(),e=await Promise.all(Object.keys(t).map(async t=>{try{return void await this.get_model(t)}catch(e){if("widget model not found"!==e.message)throw e;const n=await this._create_comm(this.comm_target_name,t);let r;const i=new a.PromiseDelegate;return n.on_msg(t=>{if(t.parent_header.msg_id===r&&"comm_msg"===t.header.msg_type&&"update"===t.content.data.method){let e=t.content.data,r=e.buffer_paths||[],o=(t.buffers||[]).map(t=>t instanceof DataView?t:new DataView(t instanceof ArrayBuffer?t:t.buffer));Object(d.put_buffers)(e.state,r,o),i.resolve({comm:n,msg:t})}}),r=n.send({method:"request_state"},this.callbacks(void 0)),i.promise}}));await Promise.all(e.map(async t=>{if(!t)return;const e=t.msg.content;await this.new_model({model_name:e.data.state._model_name,model_module:e.data.state._model_module,model_module_version:e.data.state._model_module_version,comm:t.comm},e.data.state)}))}async _loadFromNotebook(t){const e=t.metadata.get("widgets");if(e&&e[m]){let t=e[m];t=this.filterExistingModelState(t),await this.set_state(t)}}async display_view(t,e,n){return e.pWidget||new E(e)}async _create_comm(t,e,n,r,i){let o=this._context.session.kernel.connectToComm(t,e);return(n||r)&&o.open(n,r,i),new d.shims.services.Comm(o)}async _get_comm_info(){const t=await this._context.session.kernel.requestCommInfo({target:this.comm_target_name});return"ok"===t.content.status?t.content.comms:{}}get isDisposed(){return null===this._context}dispose(){this.isDisposed||(this._commRegistration&&this._commRegistration.dispose(),this._context=null)}async resolveUrl(t){const e=await this.context.urlResolver.resolveUrl(t);return this.context.urlResolver.getDownloadUrl(e)}async loadClass(t,e,n){"@jupyter-widgets/base"!==e&&"@jupyter-widgets/controls"!==e||!Object(p.valid)(n)||(n=`^${n}`);const r=this._registry.get(e,n);if(!r)throw new Error(`Module ${e}, semver range ${n} is not registered as a widget module`);let i;const o=(i="function"==typeof r?await r():await r)[t];if(!o)throw new Error(`Class ${t} not found in module ${e}`);return o}get context(){return this._context}get rendermime(){return this._rendermime}get restored(){return this._restored}get restoredStatus(){return this._restoredStatus}register(t){this._registry.set(t.name,t.version,t.exports)}async get_model(t){const e=super.get_model(t);if(void 0===e)throw new Error("widget model not found");return e}register_model(t,e){super.register_model(t,e),e.then(e=>{this._modelsSync.set(t,e),e.once("comm:close",()=>{this._modelsSync.delete(t)})}),this.setDirty()}async clear_state(){await super.clear_state(),this._modelsSync=new Map,this.setDirty()}get_state_sync(t={}){const e=[];for(let t of this._modelsSync.values())t.comm_live&&e.push(t);return Object(d.serialize_state)(e,t)}setDirty(){this._settings.saveState&&(this._context.model.dirty=!0)}}var B=n(203),b=n(1452),Q=n(206),w=n(44),x=n.n(w);const _=B.OUTPUT_WIDGET_VERSION;class D extends B.OutputModel{defaults(){return Object.assign({},super.defaults(),{msg_id:"",outputs:[]})}initialize(t,e){super.initialize(t,e),this._outputs=new b.a({trusted:!0}),this._msgHook=t=>(this.add(t),!1),this.widget_manager.context.session.kernelChanged.connect((t,e)=>{this._handleKernelChanged(e)}),this.listenTo(this,"change:msg_id",this.reset_msg_id),this.listenTo(this,"change:outputs",this.setOutputs),this.setOutputs()}_handleKernelChanged({oldValue:t}){const e=this.get("msg_id");e&&t&&(t.removeMessageHook(e,this._msgHook),this.set("msg_id",null))}reset_msg_id(){const t=this.widget_manager.context.session.kernel,e=this.get("msg_id"),n=this.previous("msg_id");n&&t&&t.removeMessageHook(n,this._msgHook),e&&t&&t.registerMessageHook(e,this._msgHook)}add(t){let e=t.header.msg_type;switch(e){case"execute_result":case"display_data":case"stream":case"error":let n=t.content;n.output_type=e,this._outputs.add(n);break;case"clear_output":this.clear_output(t.content.wait)}this.set("outputs",this._outputs.toJSON(),{newMessage:!0}),this.save_changes()}clear_output(t=!1){this._outputs.clear(t)}get outputs(){return this._outputs}setOutputs(t,e,n){n&&n.newMessage||(this.clear_output(),this._outputs.fromJSON(JSON.parse(JSON.stringify(this.get("outputs")))))}}class M extends g.Panel{constructor(t){let e=t.view;delete t.view,super(t),this._view=e}processMessage(t){super.processMessage(t),this._view.processPhosphorMessage(t)}dispose(){this.isDisposed||(super.dispose(),this._view&&this._view.remove(),this._view=null)}}class k extends B.OutputView{_createElement(t){return this.pWidget=new M({view:this}),this.pWidget.node}_setElement(t){if(this.el||t!==this.pWidget.node)throw new Error("Cannot reset the DOM element.");this.el=this.pWidget.node,this.$el=x()(this.pWidget.node)}render(){super.render(),this._outputView=new Q.a({rendermime:this.model.widget_manager.rendermime,contentFactory:Q.a.defaultContentFactory,model:this.model.outputs}),this.pWidget.insertWidget(0,this._outputView),this.pWidget.addClass("jupyter-widgets"),this.pWidget.addClass("widget-output"),this.update()}remove(){return this._outputView.dispose(),super.remove()}}var S=n(61);n(1405),n(1407);const L=[],R={saveState:!1};function*N(t){for(let e of t.widgets)if("code"===e.model.type)for(let t of e.outputArea.widgets)for(let e of Object(l.toArray)(t.children()))e instanceof h&&(yield e)}function*F(t,e){let n=Object(l.filter)(t.shell.widgets(),t=>t.id.startsWith("LinkedOutputView-")&&t.path===e);for(let t of Object(l.toArray)(n))for(let e of Object(l.toArray)(t.children()))for(let t of Object(l.toArray)(e.children()))t instanceof h&&(yield t)}function*T(...t){for(let e of t)yield*e}function U(t,e,n){let r=G.widgetManagerProperty.get(t);r||(r=new v(t,e,R),L.forEach(t=>r.register(t)),G.widgetManagerProperty.set(t,r));for(let t of n)t.manager=r;return e.removeMimeType(C),e.addFactory({safe:!1,mimeTypes:[C],createRenderer:t=>new h(t,r)},0),new c.DisposableDelegate(()=>{e&&e.removeMimeType(C),r.dispose()})}const O={id:"@jupyter-widgets/jupyterlab-manager:plugin",requires:[o.b,A.b,i.ISettingRegistry],optional:[s],provides:d.IJupyterWidgetRegistry,activate:function(t,e,r,i,o){const{commands:a}=t;i.load(O.id).then(t=>{t.changed.connect(j),j(t)}).catch(t=>{console.error(t.message)}),r.addFactory({safe:!1,mimeTypes:[C],createRenderer:t=>new h(t)},0),e.forEach(e=>{U(e.context,e.content.rendermime,T(N(e.content),F(t,e.context.path)))}),e.widgetAdded.connect((e,n)=>{U(n.context,n.content.rendermime,T(N(n.content),F(t,n.context.path)))}),a.addCommand("@jupyter-widgets/jupyterlab-manager:saveWidgetState",{label:"Save Widget State Automatically",execute:t=>i.set(O.id,"saveState",!R.saveState).catch(t=>{console.error(`Failed to set ${O.id}: ${t.message}`)}),isToggled:()=>R.saveState}),o&&o.settingsMenu.addGroup([{command:"@jupyter-widgets/jupyterlab-manager:saveWidgetState"}]);return L.push({name:"@jupyter-widgets/base",version:d.JUPYTER_WIDGETS_VERSION,exports:{WidgetModel:d.WidgetModel,WidgetView:d.WidgetView,DOMWidgetView:d.DOMWidgetView,DOMWidgetModel:d.DOMWidgetModel,LayoutModel:d.LayoutModel,LayoutView:d.LayoutView,StyleModel:d.StyleModel,StyleView:d.StyleView}}),L.push({name:"@jupyter-widgets/controls",version:S.a,exports:()=>new Promise((t,e)=>{Promise.resolve().then((e=>{t(n(1450))}).bind(null,n)).catch(t=>{e(t)})})}),L.push({name:"@jupyter-widgets/output",version:_,exports:{OutputModel:D,OutputView:k}}),{registerWidget(t){L.push(t)}}},autoStart:!0};var G,H=O;function j(t){R.saveState=t.get("saveState").composite}(G||(G={})).widgetManagerProperty=new u.AttachedProperty({name:"widgetManager",create:()=>void 0}),n.d(e,"registerWidgetManager",function(){return U}),n.d(e,"WidgetManager",function(){return v}),n.d(e,"WidgetRenderer",function(){return h}),n.d(e,"output",function(){return r});e.default=H},function(t,e,n){"use strict";n.d(e,"a",function(){return l});var r,i=n(5),o=n(10),a=n(17),s=n(387),A=n(383);class l{constructor(t={}){this.clearNext=!1,this.list=null,this._trusted=!1,this._isDisposed=!1,this._stateChanged=new o.Signal(this),this._changed=new o.Signal(this),this._trusted=!!t.trusted,this.contentFactory=t.contentFactory||l.defaultContentFactory,this.list=new s.a,t.values&&Object(i.each)(t.values,t=>{this._add(t)}),this.list.changed.connect(this._onListChanged,this)}get stateChanged(){return this._stateChanged}get changed(){return this._changed}get length(){return this.list?this.list.length:0}get trusted(){return this._trusted}set trusted(t){if(t===this._trusted)return;let e=this._trusted=t;for(let t=0;t<this.list.length;t++){let n=this.list.get(t),r=n.toJSON();n.dispose(),n=this._createItem({value:r,trusted:e}),this.list.set(t,n)}}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,this.list.dispose(),o.Signal.clearData(this))}get(t){return this.list.get(t)}set(t,e){r.normalize(e);let n=this._createItem({value:e,trusted:this._trusted});this.list.set(t,n)}add(t){return this.clearNext&&(this.clear(),this.clearNext=!1),this._add(t)}clear(t=!1){this._lastStream="",t?this.clearNext=!0:(Object(i.each)(this.list,t=>{t.dispose()}),this.list.clear())}fromJSON(t){this.clear(),Object(i.each)(t,t=>{this._add(t)})}toJSON(){return Object(i.toArray)(Object(i.map)(this.list,t=>t.toJSON()))}_add(t){let e=this._trusted;if(r.normalize(t),a.nbformat.isStream(t)&&this._lastStream&&t.name===this._lastName){this._lastStream+=t.text,this._lastStream=r.removeOverwrittenChars(this._lastStream),t.text=this._lastStream;let n=this._createItem({value:t,trusted:e}),i=this.length-1;return this.list.get(i).dispose(),this.list.set(i,n),i}a.nbformat.isStream(t)&&(t.text=r.removeOverwrittenChars(t.text));let n=this._createItem({value:t,trusted:e});return a.nbformat.isStream(t)?(this._lastStream=t.text,this._lastName=t.name):this._lastStream="",this.list.push(n)}_createItem(t){let e=this.contentFactory.createOutputModel(t);return e.changed.connect(this._onGenericChange,this),e}_onListChanged(t,e){this._changed.emit(e)}_onGenericChange(){this._stateChanged.emit(void 0)}}!function(t){class e{createOutputModel(t){return new A.a(t)}}t.ContentFactory=e,t.defaultContentFactory=new e}(l||(l={})),function(t){t.normalize=function(t){a.nbformat.isStream(t)&&Array.isArray(t.text)&&(t.text=t.text.join("\n"))},t.removeOverwrittenChars=function(t){return function(t){for(t=t.replace(/\r+\n/gm,"\n");t.search(/\r[^$]/g)>-1;){const e=t.match(/^(.*)\r+/m)[1];let n=t.match(/\r+(.*)$/m)[1];n+=e.slice(n.length,e.length),t=t.replace(/\r+.*$/m,"\r").replace(/^.*\r/m,n)}return t}(function(t){let e=t;do{e=(t=e).replace(/[^\n]\x08/gm,"")}while(e.length<t.length);return t}(t))}}(r||(r={}))},function(t,e,n){"use strict";var r,i=n(0),o=n.n(i),a=n(29),s=n(17),A=n(350),l=n.n(A),c=n(351),u=n.n(c),g=n(352),h=n.n(g),d=n(353),f=n.n(d),p=n(354),I=n.n(p),C=n(355),m=n.n(C),y=n(356),E=n.n(y),v=n(357),B=n.n(v),b=n(358),Q=n.n(b),w=n(359),x=n.n(w),_=n(360),D=n.n(_),M=n(361),k=n.n(M),S=n(362),L=n.n(S),R=n(363),N=n.n(R),F=n(364),T=n.n(F),U=n(365),O=n.n(U),G=n(366),H=n.n(G),j=n(367),Y=n.n(j),P=n(368),K=n.n(P),z=n(369),J=n.n(z),q=n(370),V=n.n(q),W=n(371),X=n.n(W),Z=n(372),$=n.n(Z),tt=n(373),et=n.n(tt),nt=n(374),rt=n.n(nt),it=n(375),ot=n.n(it),at=n(376),st=n.n(at),At=n(377),lt=n.n(At),ct=n(378),ut=n.n(ct),gt=n(379),ht=n.n(gt),dt=n(380),ft=n.n(dt);(r||(r={})).defaultIcons=[{name:"file",svg:l.a},{name:"folder",svg:u.a},{name:"html5",svg:h.a},{name:"image",svg:f.a},{name:"json",svg:I.a},{name:"markdown",svg:m.a},{name:"notebook",svg:E.a},{name:"python",svg:B.a},{name:"r-kernel",svg:Q.a},{name:"react",svg:x.a},{name:"spreadsheet",svg:D.a},{name:"yaml",svg:k.a},{name:"build",svg:L.a},{name:"extension",svg:N.a},{name:"palette",svg:T.a},{name:"running",svg:O.a},{name:"tab",svg:H.a},{name:"jupyter-favicon",svg:Y.a},{name:"kernel",svg:K.a},{name:"line-form",svg:J.a},{name:"not-trusted",svg:V.a},{name:"terminal",svg:X.a},{name:"trusted",svg:$.a},{name:"add",svg:et.a},{name:"copy",svg:rt.a},{name:"cut",svg:ot.a},{name:"paste",svg:st.a},{name:"refresh",svg:lt.a},{name:"run",svg:ut.a},{name:"save",svg:ht.a},{name:"stop",svg:ft.a}];var pt=function(t,e){var n={};for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&e.indexOf(r)<0&&(n[r]=t[r]);if(null!=t&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(t);i<r.length;i++)e.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(t,r[i])&&(n[r[i]]=t[r[i]])}return n};const It={alignItems:"center",display:"flex"},Ct={display:"block",margin:"0 auto",width:"100%"},mt={breadCrumb:{borderRadius:"var(--jp-border-radius)",cursor:"pointer",margin:"0px 2px",padding:"0px 2px",height:"16px",width:"16px",verticalAlign:"middle",$nest:{"&:hover":{backgroundColor:"var(--jp-layout-color2)"},"&:first-child":{marginLeft:"0px"},".jp-mod-dropTarget":{backgroundColor:"var(--jp-brand-color2)",opacity:.7}}},dockPanelBar:{height:"14px",width:"14px"},launcherCard:{height:"var(--jp-private-launcher-large-icon-size)",width:"var(--jp-private-launcher-large-icon-size)"},launcherSection:{marginRight:"12px",height:"var(--jp-private-launcher-small-icon-size)",width:"var(--jp-private-launcher-small-icon-size)"},listing:{height:"16px",width:"16px"},settingsEditor:{height:"16px",width:"16px"},sideBar:{width:"20px"},splash:{width:"100px"},statusBar:{left:"0px",top:"0px",height:"18px",width:"20px",position:"relative"},tabManager:{height:"16px",width:"16px"},toolbarButton:{height:"16px",width:"16px"},unset:{}},yt={breadCrumb:{},dockPanelBar:{marginRight:"4px"},launcherCard:{height:"var(--jp-private-launcher-card-icon-height)"},launcherSection:{},listing:{flex:"0 0 20px",marginRight:"4px",position:"relative"},settingsEditor:{display:"inline-block",flex:"0 0 20px",marginLeft:"2px",marginRight:"1px",position:"relative",height:"20px",width:"20px"},sideBar:{transform:"rotate(90deg)"},splash:{animation:"0.3s fade-in linear forwards",height:"100%",width:"100%",zIndex:1},statusBar:{},tabManager:{marginRight:"2px",position:"relative"},toolbarButton:{},unset:{}};function Et(t){const{kind:e,center:n}=t,r=pt(t,["kind","center"]);return Object.assign({},n?Ct:{},e?mt[e]:{},r)}const vt=t=>Object(a.style)(Object.assign({},function(t){const{kind:e,center:n}=t;return Object.assign({},n?It:{},e?yt[e]:{})}(t),{$nest:{svg:Et(t)}})),Bt=t=>Object(a.style)(Et(t));Object(a.cssRule)(`.p-TabBar-tab.p-mod-current .${vt({center:!0,kind:"sideBar"})}`,{transform:"rotate(90deg)\n    translate(\n      calc(-0.5 * var(--jp-border-width)),\n      calc(-0.5 * var(--jp-border-width))\n    )"});var bt=n(717),Qt=n.n(bt),wt=n(718),xt=n.n(wt);n.d(e,"a",function(){return kt});var _t=function(t,e){var n={};for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&e.indexOf(r)<0&&(n[r]=t[r]);if(null!=t&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(t);i<r.length;i++)e.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(t,r[i])&&(n[r[i]]=t[r[i]])}return n};class Dt{constructor(t={}){this._classNameToName=Object.create(null),this._debug=!1,this._nameToClassName=Object.create(null),this._svg=Object.create(null),this._debug=!!t.debug;let e=t.initialIcons||r.defaultIcons;this.addIcon(...e),this.addIcon({name:"bad",svg:Qt.a},{name:"blank",svg:xt.a})}addIcon(...t){t.forEach(t=>{let e=t.className?t.className:Dt.iconClassName(t.name);this._classNameToName[e]=t.name,this._nameToClassName[t.name]=e,this._svg[t.name]=t.svg})}contains(t){return!!this._resolveName(t)}icon(t){const{name:e,className:n,title:r,fallback:i,container:o}=t,s=_t(t,["name","className","title","fallback","container"]);let A=this.resolveName(e);if(!A)return i?o?(o.textContent=r||"",o.className=Object(a.classes)(e,n,s?Bt(s):""),o):(console.error("unimplemented"),null):null;if(o&&o.dataset.icon&&o.dataset.icon===A&&o.children[0])return o.children[0];const l=this.resolveSvg(A);if(!l)return null;if(r&&St.setTitleSvg(l,r),o){o.textContent="",o.dataset.icon=A,o.appendChild(l);let t=s?vt(s):"";n||""===n?o.className=Object(a.classes)(n,t):o.classList.contains(t)||(o.className=Object(a.classes)(o.className,t))}else l.setAttribute("class",Object(a.classes)(n,s?Bt(s):""));return l}iconReact(t){const{name:e,className:n,title:r,fallback:i,tag:s}=t,A=_t(t,["name","className","title","fallback","tag"]),l=s||"div",c=this.resolveName(e);if(!c)return i?o.a.createElement(l,{className:Object(a.classes)(e,n,A?Bt(A):"")},r||""):o.a.createElement(o.a.Fragment,null);const u=this.resolveSvg(c);return u?(r&&St.setTitleSvg(u,r),o.a.createElement(l,{className:Object(a.classes)(n,A?vt(A):""),"data-icon":c,dangerouslySetInnerHTML:{__html:u.outerHTML}})):o.a.createElement(o.a.Fragment,null)}_resolveName(t){if(!(t in this._svg)){if(t)for(let e of t.split(/\s+/))if(e in this._classNameToName)return this._classNameToName[e];return""}return t}resolveName(t){const e=this._resolveName(t);return e||(this._debug?(console.error(`Invalid icon name: ${t}`),"bad"):"")}resolveSvg(t){let e=this.svg(t);const n="data:image/svg+xml;base64,";e.startsWith(n)&&(e=atob(e.slice(n.length)));const r=(new DOMParser).parseFromString(e,"image/svg+xml").documentElement;if(r.getElementsByTagName("parsererror").length>0){const e=`SVG HTML was malformed for icon name: ${t}`;return this._debug?(console.error(e),r):(console.warn(e),null)}return r}svg(t){return this._svg[t]}static iconClassName(t){return"jp-"+s.Text.camelCase(t,!0)+"Icon"}}const Mt=new Dt,kt=t=>Mt.iconReact(t);var St;!function(t){t.setTitleSvg=function(t,e){let n=t.getElementsByTagName("title");if(n.length)n[0].textContent=e;else{let n=document.createElement("title");n.textContent=e,t.appendChild(n)}}}(St||(St={}))},function(t,e,n){"use strict";var r=n(0),i=n(720),o=(n(190),n(721),n(388),n(722));n(723);function a(...t){return t.filter(t=>!!t).join(" ")}n.d(e,"a",function(){return s}),n.d(e,"b",function(){return A});const s=t=>r.createElement(i.Button,Object.assign({},t,{className:a(t.className,t.minimal&&"minimal","jp-Button")})),A=t=>r.createElement(o.HTMLSelect,Object.assign({},t,{className:a(t.className,"jp-HTMLSelect")}))}]);